struct list_head {
	struct list_head *         next;                 /*     0     8 */
	struct list_head *         prev;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct hlist_head {
	struct hlist_node *        first;                /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct hlist_node {
	struct hlist_node *        next;                 /*     0     8 */
	struct hlist_node * *      pprev;                /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct callback_head {
	struct callback_head *     next;                 /*     0     8 */
	void                       (*func)(struct callback_head *); /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct file_system_type {
	const char  *              name;                 /*     0     8 */
	int                        fs_flags;             /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	int                        (*init_fs_context)(struct fs_context *); /*    16     8 */
	const struct fs_parameter_spec  * parameters;    /*    24     8 */
	struct dentry *            (*mount)(struct file_system_type *, int, const char  *, void *); /*    32     8 */
	void                       (*kill_sb)(struct super_block *); /*    40     8 */
	struct module *            owner;                /*    48     8 */
	struct file_system_type *  next;                 /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct hlist_head          fs_supers;            /*    64     8 */
	struct lock_class_key      s_lock_key;           /*    72     0 */
	struct lock_class_key      s_umount_key;         /*    72     0 */
	struct lock_class_key      s_vfs_rename_key;     /*    72     0 */
	struct lock_class_key      s_writers_key[3];     /*    72     0 */
	struct lock_class_key      i_lock_key;           /*    72     0 */
	struct lock_class_key      i_mutex_key;          /*    72     0 */
	struct lock_class_key      i_mutex_dir_key;      /*    72     0 */
	u64                        kabi_reserved1;       /*    72     8 */
	u64                        kabi_reserved2;       /*    80     8 */
	u64                        kabi_reserved3;       /*    88     8 */
	u64                        kabi_reserved4;       /*    96     8 */

	/* size: 104, cachelines: 2, members: 20 */
	/* sum members: 100, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct qspinlock {
	union {
		atomic_t           val;                  /*     0     4 */
		struct {
			u8         locked;               /*     0     1 */
			u8         pending;              /*     1     1 */
		};                                       /*     0     2 */
		struct {
			u16        locked_pending;       /*     0     2 */
			u16        tail;                 /*     2     2 */
		};                                       /*     0     4 */
	};                                               /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct qrwlock {
	union {
		atomic_t           cnts;                 /*     0     4 */
		struct {
			u8         wlocked;              /*     0     1 */
			u8         __lstate[3];          /*     1     3 */
		};                                       /*     0     4 */
	};                                               /*     0     4 */
	arch_spinlock_t            wait_lock;            /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct lock_class_key {

	/* size: 0, cachelines: 0, members: 0 */
};
struct raw_spinlock {
	arch_spinlock_t            raw_lock;             /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct spinlock {
	union {
		struct raw_spinlock rlock;               /*     0     4 */
	};                                               /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct ratelimit_state {
	raw_spinlock_t             lock;                 /*     0     4 */
	int                        interval;             /*     4     4 */
	int                        burst;                /*     8     4 */
	int                        printed;              /*    12     4 */
	int                        missed;               /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          begin;                /*    24     8 */
	long unsigned int          flags;                /*    32     8 */

	/* size: 40, cachelines: 1, members: 7 */
	/* sum members: 36, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct file_operations {
	struct module *            owner;                /*     0     8 */
	loff_t                     (*llseek)(struct file *, loff_t, int); /*     8     8 */
	ssize_t                    (*read)(struct file *, char *, size_t, loff_t *); /*    16     8 */
	ssize_t                    (*write)(struct file *, const char  *, size_t, loff_t *); /*    24     8 */
	ssize_t                    (*read_iter)(struct kiocb *, struct iov_iter *); /*    32     8 */
	ssize_t                    (*write_iter)(struct kiocb *, struct iov_iter *); /*    40     8 */
	int                        (*iopoll)(struct kiocb *, bool); /*    48     8 */
	int                        (*iterate)(struct file *, struct dir_context *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*iterate_shared)(struct file *, struct dir_context *); /*    64     8 */
	__poll_t                   (*poll)(struct file *, struct poll_table_struct *); /*    72     8 */
	long int                   (*unlocked_ioctl)(struct file *, unsigned int, long unsigned int); /*    80     8 */
	long int                   (*compat_ioctl)(struct file *, unsigned int, long unsigned int); /*    88     8 */
	int                        (*mmap)(struct file *, struct vm_area_struct *); /*    96     8 */
	long unsigned int          mmap_supported_flags; /*   104     8 */
	int                        (*open)(struct inode *, struct file *); /*   112     8 */
	int                        (*flush)(struct file *, fl_owner_t); /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	int                        (*release)(struct inode *, struct file *); /*   128     8 */
	int                        (*fsync)(struct file *, loff_t, loff_t, int); /*   136     8 */
	int                        (*fasync)(int, struct file *, int); /*   144     8 */
	int                        (*lock)(struct file *, int, struct file_lock *); /*   152     8 */
	ssize_t                    (*sendpage)(struct file *, struct page *, int, size_t, loff_t *, int); /*   160     8 */
	long unsigned int          (*get_unmapped_area)(struct file *, long unsigned int, long unsigned int, long unsigned int, long unsigned int); /*   168     8 */
	int                        (*check_flags)(int);  /*   176     8 */
	int                        (*flock)(struct file *, int, struct file_lock *); /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	ssize_t                    (*splice_write)(struct pipe_inode_info *, struct file *, loff_t *, size_t, unsigned int); /*   192     8 */
	ssize_t                    (*splice_read)(struct file *, loff_t *, struct pipe_inode_info *, size_t, unsigned int); /*   200     8 */
	int                        (*setlease)(struct file *, long int, struct file_lock * *, void * *); /*   208     8 */
	long int                   (*fallocate)(struct file *, int, loff_t, loff_t); /*   216     8 */
	void                       (*show_fdinfo)(struct seq_file *, struct file *); /*   224     8 */
	ssize_t                    (*copy_file_range)(struct file *, loff_t, struct file *, loff_t, size_t, unsigned int); /*   232     8 */
	loff_t                     (*remap_file_range)(struct file *, loff_t, struct file *, loff_t, loff_t, unsigned int); /*   240     8 */
	int                        (*fadvise)(struct file *, loff_t, loff_t, int); /*   248     8 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	u64                        kabi_reserved1;       /*   256     8 */
	u64                        kabi_reserved2;       /*   264     8 */
	u64                        kabi_reserved3;       /*   272     8 */
	u64                        kabi_reserved4;       /*   280     8 */

	/* size: 288, cachelines: 5, members: 36 */
	/* last cacheline: 32 bytes */
};
struct __kernel_timespec {
	__kernel_time64_t          tv_sec;               /*     0     8 */
	long long int              tv_nsec;              /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct timespec64 {
	time64_t                   tv_sec;               /*     0     8 */
	long int                   tv_nsec;              /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct old_timespec32 {
	old_time32_t               tv_sec;               /*     0     4 */
	s32                        tv_nsec;              /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct restart_block {
	long int                   (*fn)(struct restart_block *); /*     0     8 */
	union {
		struct {
			u32 *      uaddr;                /*     8     8 */
			u32        val;                  /*    16     4 */
			u32        flags;                /*    20     4 */
			u32        bitset;               /*    24     4 */

			/* XXX 4 bytes hole, try to pack */

			u64        time;                 /*    32     8 */
			u32 *      uaddr2;               /*    40     8 */
		} futex;                                 /*     8    40 */
		struct {
			clockid_t  clockid;              /*     8     4 */
			enum timespec_type type;         /*    12     4 */
			union {
				struct __kernel_timespec * rmtp; /*    16     8 */
				struct old_timespec32 * compat_rmtp; /*    16     8 */
			};                               /*    16     8 */
			u64        expires;              /*    24     8 */
		} nanosleep;                             /*     8    24 */
		struct {
			struct pollfd * ufds;            /*     8     8 */
			int        nfds;                 /*    16     4 */
			int        has_timeout;          /*    20     4 */
			long unsigned int tv_sec;        /*    24     8 */
			long unsigned int tv_nsec;       /*    32     8 */
		} poll;                                  /*     8    32 */
	};                                               /*     8    40 */

	/* size: 48, cachelines: 1, members: 2 */
	/* last cacheline: 48 bytes */
};
struct task_struct {
	struct thread_info         thread_info;          /*     0    32 */
	volatile long int          state;                /*    32     8 */
	void *                     stack;                /*    40     8 */
	refcount_t                 usage;                /*    48     4 */
	unsigned int               flags;                /*    52     4 */
	unsigned int               ptrace;               /*    56     4 */
	int                        on_cpu;               /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct __call_single_node  wake_entry;           /*    64    16 */
	unsigned int               wakee_flips;          /*    80     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          wakee_flip_decay_ts;  /*    88     8 */
	struct task_struct *       last_wakee;           /*    96     8 */
	int                        recent_used_cpu;      /*   104     4 */
	int                        wake_cpu;             /*   108     4 */
	int                        on_rq;                /*   112     4 */
	int                        prio;                 /*   116     4 */
	int                        static_prio;          /*   120     4 */
	int                        normal_prio;          /*   124     4 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	unsigned int               rt_priority;          /*   128     4 */

	/* XXX 4 bytes hole, try to pack */

	const struct sched_class  * sched_class;         /*   136     8 */

	/* XXX 48 bytes hole, try to pack */

	/* --- cacheline 3 boundary (192 bytes) --- */
	struct sched_entity        se __attribute__((__aligned__(64))); /*   192   576 */

	/* XXX last struct has 32 bytes of padding */

	/* --- cacheline 12 boundary (768 bytes) --- */
	struct sched_rt_entity     rt;                   /*   768    88 */
	/* --- cacheline 13 boundary (832 bytes) was 24 bytes ago --- */
	struct task_group *        sched_task_group;     /*   856     8 */
	struct sched_dl_entity     dl __attribute__((__aligned__(8))); /*   864   304 */
	/* --- cacheline 18 boundary (1152 bytes) was 16 bytes ago --- */
	struct hlist_head          preempt_notifiers;    /*  1168     8 */
	unsigned int               btrace_seq;           /*  1176     4 */
	unsigned int               policy;               /*  1180     4 */
	int                        nr_cpus_allowed;      /*  1184     4 */

	/* XXX 4 bytes hole, try to pack */

	const cpumask_t  *         cpus_ptr;             /*  1192     8 */
	cpumask_t                  cpus_mask;            /*  1200  1024 */
	/* --- cacheline 34 boundary (2176 bytes) was 48 bytes ago --- */
	int                        trc_reader_nesting;   /*  2224     4 */
	int                        trc_ipi_to_cpu;       /*  2228     4 */
	union rcu_special          trc_reader_special;   /*  2232     4 */
	bool                       trc_reader_checked;   /*  2236     1 */

	/* XXX 3 bytes hole, try to pack */

	/* --- cacheline 35 boundary (2240 bytes) --- */
	struct list_head           trc_holdout_list;     /*  2240    16 */
	struct sched_info          sched_info;           /*  2256    64 */
	/* --- cacheline 36 boundary (2304 bytes) was 16 bytes ago --- */
	struct list_head           tasks;                /*  2320    16 */
	struct plist_node          pushable_tasks;       /*  2336    40 */
	/* --- cacheline 37 boundary (2368 bytes) was 8 bytes ago --- */
	struct rb_node             pushable_dl_tasks __attribute__((__aligned__(8))); /*  2376    24 */
	struct mm_struct *         mm;                   /*  2400     8 */
	struct mm_struct *         active_mm;            /*  2408     8 */
	struct vmacache            vmacache;             /*  2416    40 */
	/* --- cacheline 38 boundary (2432 bytes) was 24 bytes ago --- */
	struct task_rss_stat       rss_stat;             /*  2456    20 */
	int                        exit_state;           /*  2476     4 */
	int                        exit_code;            /*  2480     4 */
	int                        exit_signal;          /*  2484     4 */
	int                        pdeath_signal;        /*  2488     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 39 boundary (2496 bytes) --- */
	long unsigned int          jobctl;               /*  2496     8 */
	unsigned int               personality;          /*  2504     4 */
	unsigned int               sched_reset_on_fork:1; /*  2508: 0  4 */
	unsigned int               sched_contributes_to_load:1; /*  2508: 1  4 */
	unsigned int               sched_migrated:1;     /*  2508: 2  4 */
	unsigned int               sched_psi_wake_requeue:1; /*  2508: 3  4 */

	/* XXX 28 bits hole, try to pack */

	/* Force alignment to the next boundary: */
	unsigned int               :0;

	unsigned int               sched_remote_wakeup:1; /*  2512: 0  4 */
	unsigned int               in_execve:1;          /*  2512: 1  4 */
	unsigned int               in_iowait:1;          /*  2512: 2  4 */
	unsigned int               restore_sigmask:1;    /*  2512: 3  4 */
	unsigned int               in_user_fault:1;      /*  2512: 4  4 */
	unsigned int               no_cgroup_migration:1; /*  2512: 5  4 */
	unsigned int               frozen:1;             /*  2512: 6  4 */
	unsigned int               use_memdelay:1;       /*  2512: 7  4 */
	unsigned int               in_memstall:1;        /*  2512: 8  4 */
	unsigned int               pasid_activated:1;    /*  2512: 9  4 */

	/* XXX 22 bits hole, try to pack */
	/* XXX 4 bytes hole, try to pack */

	long unsigned int          atomic_flags;         /*  2520     8 */
	struct restart_block       restart_block;        /*  2528    48 */
	/* --- cacheline 40 boundary (2560 bytes) was 16 bytes ago --- */
	pid_t                      pid;                  /*  2576     4 */
	pid_t                      tgid;                 /*  2580     4 */
	long unsigned int          stack_canary;         /*  2584     8 */
	struct task_struct *       real_parent;          /*  2592     8 */
	struct task_struct *       parent;               /*  2600     8 */
	struct list_head           children;             /*  2608    16 */
	/* --- cacheline 41 boundary (2624 bytes) --- */
	struct list_head           sibling;              /*  2624    16 */
	struct task_struct *       group_leader;         /*  2640     8 */
	struct list_head           ptraced;              /*  2648    16 */
	struct list_head           ptrace_entry;         /*  2664    16 */
	struct pid *               thread_pid;           /*  2680     8 */
	/* --- cacheline 42 boundary (2688 bytes) --- */
	struct hlist_node          pid_links[4];         /*  2688    64 */
	/* --- cacheline 43 boundary (2752 bytes) --- */
	struct list_head           thread_group;         /*  2752    16 */
	struct list_head           thread_node;          /*  2768    16 */
	struct completion *        vfork_done;           /*  2784     8 */
	int *                      set_child_tid;        /*  2792     8 */
	int *                      clear_child_tid;      /*  2800     8 */
	u64                        utime;                /*  2808     8 */
	/* --- cacheline 44 boundary (2816 bytes) --- */
	u64                        stime;                /*  2816     8 */
	u64                        gtime;                /*  2824     8 */
	struct prev_cputime        prev_cputime;         /*  2832    24 */

	/* XXX last struct has 4 bytes of padding */

	struct vtime               vtime;                /*  2856    48 */
	/* --- cacheline 45 boundary (2880 bytes) was 24 bytes ago --- */
	atomic_t                   tick_dep_mask;        /*  2904     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          nvcsw;                /*  2912     8 */
	long unsigned int          nivcsw;               /*  2920     8 */
	u64                        start_time;           /*  2928     8 */
	u64                        start_boottime;       /*  2936     8 */
	/* --- cacheline 46 boundary (2944 bytes) --- */
	long unsigned int          min_flt;              /*  2944     8 */
	long unsigned int          maj_flt;              /*  2952     8 */
	struct posix_cputimers     posix_cputimers;      /*  2960    80 */
	/* --- cacheline 47 boundary (3008 bytes) was 32 bytes ago --- */
	struct posix_cputimers_work posix_cputimers_work __attribute__((__aligned__(8))); /*  3040    24 */

	/* XXX last struct has 4 bytes of padding */

	const struct cred  *       ptracer_cred;         /*  3064     8 */
	/* --- cacheline 48 boundary (3072 bytes) --- */
	const struct cred  *       real_cred;            /*  3072     8 */
	const struct cred  *       cred;                 /*  3080     8 */
	struct key *               cached_requested_key; /*  3088     8 */
	char                       comm[16];             /*  3096    16 */
	struct nameidata *         nameidata;            /*  3112     8 */
	struct sysv_sem            sysvsem;              /*  3120     8 */
	struct sysv_shm            sysvshm;              /*  3128    16 */
	/* --- cacheline 49 boundary (3136 bytes) was 8 bytes ago --- */
	long unsigned int          last_switch_count;    /*  3144     8 */
	long unsigned int          last_switch_time;     /*  3152     8 */
	struct fs_struct *         fs;                   /*  3160     8 */
	struct files_struct *      files;                /*  3168     8 */
	struct io_uring_task *     io_uring;             /*  3176     8 */
	struct nsproxy *           nsproxy;              /*  3184     8 */
	struct signal_struct *     signal;               /*  3192     8 */
	/* --- cacheline 50 boundary (3200 bytes) --- */
	struct sighand_struct *    sighand;              /*  3200     8 */
	sigset_t                   blocked;              /*  3208     8 */
	sigset_t                   real_blocked;         /*  3216     8 */
	sigset_t                   saved_sigmask;        /*  3224     8 */
	struct sigpending          pending;              /*  3232    24 */
	long unsigned int          sas_ss_sp;            /*  3256     8 */
	/* --- cacheline 51 boundary (3264 bytes) --- */
	size_t                     sas_ss_size;          /*  3264     8 */
	unsigned int               sas_ss_flags;         /*  3272     4 */

	/* XXX 4 bytes hole, try to pack */

	struct callback_head *     task_works;           /*  3280     8 */
	struct audit_context *     audit_context;        /*  3288     8 */
	kuid_t                     loginuid;             /*  3296     4 */
	unsigned int               sessionid;            /*  3300     4 */
	struct seccomp             seccomp;              /*  3304    16 */
	u64                        parent_exec_id;       /*  3320     8 */
	/* --- cacheline 52 boundary (3328 bytes) --- */
	u64                        self_exec_id;         /*  3328     8 */
	spinlock_t                 alloc_lock;           /*  3336     4 */
	raw_spinlock_t             pi_lock;              /*  3340     4 */
	struct wake_q_node         wake_q;               /*  3344     8 */
	struct rb_root_cached      pi_waiters;           /*  3352    16 */
	struct task_struct *       pi_top_task;          /*  3368     8 */
	struct rt_mutex_waiter *   pi_blocked_on;        /*  3376     8 */
	void *                     journal_info;         /*  3384     8 */
	/* --- cacheline 53 boundary (3392 bytes) --- */
	struct bio_list *          bio_list;             /*  3392     8 */
	struct blk_plug *          plug;                 /*  3400     8 */
	struct reclaim_state *     reclaim_state;        /*  3408     8 */
	struct backing_dev_info *  backing_dev_info;     /*  3416     8 */
	struct io_context *        io_context;           /*  3424     8 */
	struct capture_control *   capture_control;      /*  3432     8 */
	long unsigned int          ptrace_message;       /*  3440     8 */
	kernel_siginfo_t *         last_siginfo;         /*  3448     8 */
	/* --- cacheline 54 boundary (3456 bytes) --- */
	struct task_io_accounting  ioac;                 /*  3456    56 */
	unsigned int               psi_flags;            /*  3512     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 55 boundary (3520 bytes) --- */
	u64                        acct_rss_mem1;        /*  3520     8 */
	u64                        acct_vm_mem1;         /*  3528     8 */
	u64                        acct_timexpd;         /*  3536     8 */
	nodemask_t                 mems_allowed;         /*  3544   128 */
	/* --- cacheline 57 boundary (3648 bytes) was 24 bytes ago --- */
	seqcount_spinlock_t        mems_allowed_seq;     /*  3672     4 */
	int                        cpuset_mem_spread_rotor; /*  3676     4 */
	int                        cpuset_slab_spread_rotor; /*  3680     4 */

	/* XXX 4 bytes hole, try to pack */

	struct css_set *           cgroups;              /*  3688     8 */
	struct list_head           cg_list;              /*  3696    16 */
	/* --- cacheline 58 boundary (3712 bytes) --- */
	u32                        closid;               /*  3712     4 */
	u32                        rmid;                 /*  3716     4 */
	struct robust_list_head *  robust_list;          /*  3720     8 */
	struct compat_robust_list_head * compat_robust_list; /*  3728     8 */
	struct list_head           pi_state_list;        /*  3736    16 */
	struct futex_pi_state *    pi_state_cache;       /*  3752     8 */
	struct mutex               futex_exit_mutex;     /*  3760    32 */
	/* --- cacheline 59 boundary (3776 bytes) was 16 bytes ago --- */
	unsigned int               futex_state;          /*  3792     4 */

	/* XXX 4 bytes hole, try to pack */

	struct perf_event_context * perf_event_ctxp[2];  /*  3800    16 */
	struct mutex               perf_event_mutex;     /*  3816    32 */
	/* --- cacheline 60 boundary (3840 bytes) was 8 bytes ago --- */
	struct list_head           perf_event_list;      /*  3848    16 */
	struct mempolicy *         mempolicy;            /*  3864     8 */
	short int                  il_prev;              /*  3872     2 */
	short int                  pref_node_fork;       /*  3874     2 */
	int                        numa_scan_seq;        /*  3876     4 */
	unsigned int               numa_scan_period;     /*  3880     4 */
	unsigned int               numa_scan_period_max; /*  3884     4 */
	int                        numa_preferred_nid;   /*  3888     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          numa_migrate_retry;   /*  3896     8 */
	/* --- cacheline 61 boundary (3904 bytes) --- */
	u64                        node_stamp;           /*  3904     8 */
	u64                        last_task_numa_placement; /*  3912     8 */
	u64                        last_sum_exec_runtime; /*  3920     8 */
	struct callback_head       numa_work __attribute__((__aligned__(8))); /*  3928    16 */
	struct numa_group *        numa_group;           /*  3944     8 */
	long unsigned int *        numa_faults;          /*  3952     8 */
	long unsigned int          total_numa_faults;    /*  3960     8 */
	/* --- cacheline 62 boundary (3968 bytes) --- */
	long unsigned int          numa_faults_locality[3]; /*  3968    24 */
	long unsigned int          numa_pages_migrated;  /*  3992     8 */
	struct rseq *              rseq;                 /*  4000     8 */
	u32                        rseq_sig;             /*  4008     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          rseq_event_mask;      /*  4016     8 */
	struct tlbflush_unmap_batch tlb_ubc;             /*  4024  1032 */

	/* XXX last struct has 6 bytes of padding */

	/* --- cacheline 79 boundary (5056 bytes) --- */
	union {
		refcount_t         rcu_users;            /*  5056     4 */
		struct callback_head rcu __attribute__((__aligned__(8))); /*  5056    16 */
	} __attribute__((__aligned__(8)));               /*  5056    16 */
	struct pipe_inode_info *   splice_pipe;          /*  5072     8 */
	struct page_frag           task_frag;            /*  5080    16 */
	struct task_delay_info *   delays;               /*  5096     8 */
	int                        nr_dirtied;           /*  5104     4 */
	int                        nr_dirtied_pause;     /*  5108     4 */
	long unsigned int          dirty_paused_when;    /*  5112     8 */
	/* --- cacheline 80 boundary (5120 bytes) --- */
	u64                        timer_slack_ns;       /*  5120     8 */
	u64                        default_timer_slack_ns; /*  5128     8 */
	int                        curr_ret_stack;       /*  5136     4 */
	int                        curr_ret_depth;       /*  5140     4 */
	struct ftrace_ret_stack *  ret_stack;            /*  5144     8 */
	long long unsigned int     ftrace_timestamp;     /*  5152     8 */
	atomic_t                   trace_overrun;        /*  5160     4 */
	atomic_t                   tracing_graph_pause;  /*  5164     4 */
	long unsigned int          trace;                /*  5168     8 */
	long unsigned int          trace_recursion;      /*  5176     8 */
	/* --- cacheline 81 boundary (5184 bytes) --- */
	struct mem_cgroup *        memcg_in_oom;         /*  5184     8 */
	gfp_t                      memcg_oom_gfp_mask;   /*  5192     4 */
	int                        memcg_oom_order;      /*  5196     4 */
	unsigned int               memcg_nr_pages_over_high; /*  5200     4 */

	/* XXX 4 bytes hole, try to pack */

	struct mem_cgroup *        active_memcg;         /*  5208     8 */
	struct request_queue *     throttle_queue;       /*  5216     8 */
	struct uprobe_task *       utask;                /*  5224     8 */
	int                        pagefault_disabled;   /*  5232     4 */

	/* XXX 4 bytes hole, try to pack */

	struct task_struct *       oom_reaper_list;      /*  5240     8 */
	/* --- cacheline 82 boundary (5248 bytes) --- */
	struct vm_struct *         stack_vm_area;        /*  5248     8 */
	refcount_t                 stack_refcount;       /*  5256     4 */
	int                        patch_state;          /*  5260     4 */
	void *                     security;             /*  5264     8 */
	struct bpf_run_ctx *       bpf_ctx;              /*  5272     8 */
	void *                     mce_vaddr;            /*  5280     8 */
	__u64                      mce_kflags;           /*  5288     8 */
	u64                        mce_addr;             /*  5296     8 */
	__u64                      mce_ripv:1;           /*  5304: 0  8 */
	__u64                      mce_whole_page:1;     /*  5304: 1  8 */
	__u64                      __mce_reserved:62;    /*  5304: 2  8 */
	/* --- cacheline 83 boundary (5312 bytes) --- */
	struct callback_head       mce_kill_me __attribute__((__aligned__(8))); /*  5312    16 */
	int                        mce_count;            /*  5328     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        kabi_reserved1;       /*  5336     8 */
	u64                        kabi_reserved2;       /*  5344     8 */
	u64                        kabi_reserved3;       /*  5352     8 */
	u64                        kabi_reserved4;       /*  5360     8 */
	u64                        kabi_reserved5;       /*  5368     8 */
	/* --- cacheline 84 boundary (5376 bytes) --- */
	union {
		long int           tag;                  /*  5376     8 */
		struct {
			u64        kabi_reserved6;       /*  5376     8 */
		} kabi_hidden_1424;                      /*  5376     8 */
		union {
		};                                       /*  5376     0 */
	};                                               /*  5376     8 */
	u64                        kabi_reserved7;       /*  5384     8 */
	u64                        kabi_reserved8;       /*  5392     8 */
	u64                        kabi_reserved9;       /*  5400     8 */
	u64                        kabi_reserved10;      /*  5408     8 */
	u64                        kabi_reserved11;      /*  5416     8 */
	u64                        kabi_reserved12;      /*  5424     8 */
	u64                        kabi_reserved13;      /*  5432     8 */
	/* --- cacheline 85 boundary (5440 bytes) --- */
	u64                        kabi_reserved14;      /*  5440     8 */
	u64                        kabi_reserved15;      /*  5448     8 */
	u64                        kabi_reserved16;      /*  5456     8 */
	size_t                     task_struct_size_resvd; /*  5464     8 */
	struct task_struct_resvd * _resvd;               /*  5472     8 */

	/* XXX 24 bytes hole, try to pack */

	/* --- cacheline 86 boundary (5504 bytes) --- */
	struct thread_struct       thread __attribute__((__aligned__(64))); /*  5504  8512 */

	/* size: 14016, cachelines: 219, members: 241 */
	/* sum members: 13865, holes: 18, sum holes: 135 */
	/* sum bitfield members: 78 bits, bit holes: 2, sum bit holes: 50 bits */
	/* paddings: 4, sum paddings: 46 */
	/* forced alignments: 8, forced holes: 2, sum forced holes: 72 */
} __attribute__((__aligned__(64)));
struct screen_info {
	__u8                       orig_x;               /*     0     1 */
	__u8                       orig_y;               /*     1     1 */
	__u16                      ext_mem_k;            /*     2     2 */
	__u16                      orig_video_page;      /*     4     2 */
	__u8                       orig_video_mode;      /*     6     1 */
	__u8                       orig_video_cols;      /*     7     1 */
	__u8                       flags;                /*     8     1 */
	__u8                       unused2;              /*     9     1 */
	__u16                      orig_video_ega_bx;    /*    10     2 */
	__u16                      unused3;              /*    12     2 */
	__u8                       orig_video_lines;     /*    14     1 */
	__u8                       orig_video_isVGA;     /*    15     1 */
	__u16                      orig_video_points;    /*    16     2 */
	__u16                      lfb_width;            /*    18     2 */
	__u16                      lfb_height;           /*    20     2 */
	__u16                      lfb_depth;            /*    22     2 */
	__u32                      lfb_base;             /*    24     4 */
	__u32                      lfb_size;             /*    28     4 */
	__u16                      cl_magic;             /*    32     2 */
	__u16                      cl_offset;            /*    34     2 */
	__u16                      lfb_linelength;       /*    36     2 */
	__u8                       red_size;             /*    38     1 */
	__u8                       red_pos;              /*    39     1 */
	__u8                       green_size;           /*    40     1 */
	__u8                       green_pos;            /*    41     1 */
	__u8                       blue_size;            /*    42     1 */
	__u8                       blue_pos;             /*    43     1 */
	__u8                       rsvd_size;            /*    44     1 */
	__u8                       rsvd_pos;             /*    45     1 */
	__u16                      vesapm_seg;           /*    46     2 */
	__u16                      vesapm_off;           /*    48     2 */
	__u16                      pages;                /*    50     2 */
	__u16                      vesa_attributes;      /*    52     2 */
	__u32                      capabilities;         /*    54     4 */
	__u32                      ext_lfb_base;         /*    58     4 */
	__u8                       _reserved[2];         /*    62     2 */

	/* size: 64, cachelines: 1, members: 36 */
} __attribute__((__packed__));
struct apm_bios_info {
	__u16                      version;              /*     0     2 */
	__u16                      cseg;                 /*     2     2 */
	__u32                      offset;               /*     4     4 */
	__u16                      cseg_16;              /*     8     2 */
	__u16                      dseg;                 /*    10     2 */
	__u16                      flags;                /*    12     2 */
	__u16                      cseg_len;             /*    14     2 */
	__u16                      cseg_16_len;          /*    16     2 */
	__u16                      dseg_len;             /*    18     2 */

	/* size: 20, cachelines: 1, members: 9 */
	/* last cacheline: 20 bytes */
};
struct edd_device_params {
	__u16                      length;               /*     0     2 */
	__u16                      info_flags;           /*     2     2 */
	__u32                      num_default_cylinders; /*     4     4 */
	__u32                      num_default_heads;    /*     8     4 */
	__u32                      sectors_per_track;    /*    12     4 */
	__u64                      number_of_sectors;    /*    16     8 */
	__u16                      bytes_per_sector;     /*    24     2 */
	__u32                      dpte_ptr;             /*    26     4 */
	__u16                      key;                  /*    30     2 */
	__u8                       device_path_info_length; /*    32     1 */
	__u8                       reserved2;            /*    33     1 */
	__u16                      reserved3;            /*    34     2 */
	__u8                       host_bus_type[4];     /*    36     4 */
	__u8                       interface_type[8];    /*    40     8 */
	union {
		struct {
			__u16      base_address;         /*    48     2 */
			__u16      reserved1;            /*    50     2 */
			__u32      reserved2;            /*    52     4 */
		} isa;                                   /*    48     8 */
		struct {
			__u8       bus;                  /*    48     1 */
			__u8       slot;                 /*    49     1 */
			__u8       function;             /*    50     1 */
			__u8       channel;              /*    51     1 */
			__u32      reserved;             /*    52     4 */
		} pci;                                   /*    48     8 */
		struct {
			__u64      reserved;             /*    48     8 */
		} ibnd;                                  /*    48     8 */
		struct {
			__u64      reserved;             /*    48     8 */
		} xprs;                                  /*    48     8 */
		struct {
			__u64      reserved;             /*    48     8 */
		} htpt;                                  /*    48     8 */
		struct {
			__u64      reserved;             /*    48     8 */
		} unknown;                               /*    48     8 */
	} interface_path;                                /*    48     8 */
	union {
		struct {
			__u8       device;               /*    56     1 */
			__u8       reserved1;            /*    57     1 */
			__u16      reserved2;            /*    58     2 */
			__u32      reserved3;            /*    60     4 */
			/* --- cacheline 1 boundary (64 bytes) --- */
			__u64      reserved4;            /*    64     8 */
		} ata;                                   /*    56    16 */
		struct {
			__u8       device;               /*    56     1 */
			__u8       lun;                  /*    57     1 */
			__u8       reserved1;            /*    58     1 */
			__u8       reserved2;            /*    59     1 */
			__u32      reserved3;            /*    60     4 */
			/* --- cacheline 1 boundary (64 bytes) --- */
			__u64      reserved4;            /*    64     8 */
		} atapi;                                 /*    56    16 */
		struct {
			__u16      id;                   /*    56     2 */
			__u64      lun;                  /*    58     8 */
			/* --- cacheline 1 boundary (64 bytes) was 2 bytes ago --- */
			__u16      reserved1;            /*    66     2 */
			__u32      reserved2;            /*    68     4 */
		} __attribute__((__packed__)) scsi;      /*    56    16 */
		struct {
			__u64      serial_number;        /*    56     8 */
			/* --- cacheline 1 boundary (64 bytes) --- */
			__u64      reserved;             /*    64     8 */
		} usb;                                   /*    56    16 */
		struct {
			__u64      eui;                  /*    56     8 */
			/* --- cacheline 1 boundary (64 bytes) --- */
			__u64      reserved;             /*    64     8 */
		} i1394;                                 /*    56    16 */
		struct {
			__u64      wwid;                 /*    56     8 */
			/* --- cacheline 1 boundary (64 bytes) --- */
			__u64      lun;                  /*    64     8 */
		} fibre;                                 /*    56    16 */
		struct {
			__u64      identity_tag;         /*    56     8 */
			/* --- cacheline 1 boundary (64 bytes) --- */
			__u64      reserved;             /*    64     8 */
		} i2o;                                   /*    56    16 */
		struct {
			__u32      array_number;         /*    56     4 */
			__u32      reserved1;            /*    60     4 */
			/* --- cacheline 1 boundary (64 bytes) --- */
			__u64      reserved2;            /*    64     8 */
		} raid;                                  /*    56    16 */
		struct {
			__u8       device;               /*    56     1 */
			__u8       reserved1;            /*    57     1 */
			__u16      reserved2;            /*    58     2 */
			__u32      reserved3;            /*    60     4 */
			/* --- cacheline 1 boundary (64 bytes) --- */
			__u64      reserved4;            /*    64     8 */
		} sata;                                  /*    56    16 */
		struct {
			__u64      reserved1;            /*    56     8 */
			/* --- cacheline 1 boundary (64 bytes) --- */
			__u64      reserved2;            /*    64     8 */
		} unknown;                               /*    56    16 */
	} device_path;                                   /*    56    16 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	__u8                       reserved4;            /*    72     1 */
	__u8                       checksum;             /*    73     1 */

	/* size: 74, cachelines: 2, members: 18 */
	/* last cacheline: 10 bytes */
} __attribute__((__packed__));
struct edd_info {
	__u8                       device;               /*     0     1 */
	__u8                       version;              /*     1     1 */
	__u16                      interface_support;    /*     2     2 */
	__u16                      legacy_max_cylinder;  /*     4     2 */
	__u8                       legacy_max_head;      /*     6     1 */
	__u8                       legacy_sectors_per_track; /*     7     1 */
	struct edd_device_params   params;               /*     8    74 */

	/* size: 82, cachelines: 2, members: 7 */
	/* last cacheline: 18 bytes */
} __attribute__((__packed__));
struct ist_info {
	__u32                      signature;            /*     0     4 */
	__u32                      command;              /*     4     4 */
	__u32                      event;                /*     8     4 */
	__u32                      perf_level;           /*    12     4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct edid_info {
	unsigned char              dummy[128];           /*     0   128 */

	/* size: 128, cachelines: 2, members: 1 */
};
struct setup_header {
	__u8                       setup_sects;          /*     0     1 */
	__u16                      root_flags;           /*     1     2 */
	__u32                      syssize;              /*     3     4 */
	__u16                      ram_size;             /*     7     2 */
	__u16                      vid_mode;             /*     9     2 */
	__u16                      root_dev;             /*    11     2 */
	__u16                      boot_flag;            /*    13     2 */
	__u16                      jump;                 /*    15     2 */
	__u32                      header;               /*    17     4 */
	__u16                      version;              /*    21     2 */
	__u32                      realmode_swtch;       /*    23     4 */
	__u16                      start_sys_seg;        /*    27     2 */
	__u16                      kernel_version;       /*    29     2 */
	__u8                       type_of_loader;       /*    31     1 */
	__u8                       loadflags;            /*    32     1 */
	__u16                      setup_move_size;      /*    33     2 */
	__u32                      code32_start;         /*    35     4 */
	__u32                      ramdisk_image;        /*    39     4 */
	__u32                      ramdisk_size;         /*    43     4 */
	__u32                      bootsect_kludge;      /*    47     4 */
	__u16                      heap_end_ptr;         /*    51     2 */
	__u8                       ext_loader_ver;       /*    53     1 */
	__u8                       ext_loader_type;      /*    54     1 */
	__u32                      cmd_line_ptr;         /*    55     4 */
	__u32                      initrd_addr_max;      /*    59     4 */
	__u32                      kernel_alignment;     /*    63     4 */
	/* --- cacheline 1 boundary (64 bytes) was 3 bytes ago --- */
	__u8                       relocatable_kernel;   /*    67     1 */
	__u8                       min_alignment;        /*    68     1 */
	__u16                      xloadflags;           /*    69     2 */
	__u32                      cmdline_size;         /*    71     4 */
	__u32                      hardware_subarch;     /*    75     4 */
	__u64                      hardware_subarch_data; /*    79     8 */
	__u32                      payload_offset;       /*    87     4 */
	__u32                      payload_length;       /*    91     4 */
	__u64                      setup_data;           /*    95     8 */
	__u64                      pref_address;         /*   103     8 */
	__u32                      init_size;            /*   111     4 */
	__u32                      handover_offset;      /*   115     4 */
	__u32                      kernel_info_offset;   /*   119     4 */

	/* size: 123, cachelines: 2, members: 39 */
	/* last cacheline: 59 bytes */
} __attribute__((__packed__));
struct sys_desc_table {
	__u16                      length;               /*     0     2 */
	__u8                       table[14];            /*     2    14 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct olpc_ofw_header {
	__u32                      ofw_magic;            /*     0     4 */
	__u32                      ofw_version;          /*     4     4 */
	__u32                      cif_handler;          /*     8     4 */
	__u32                      irq_desc_table;       /*    12     4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct efi_info {
	__u32                      efi_loader_signature; /*     0     4 */
	__u32                      efi_systab;           /*     4     4 */
	__u32                      efi_memdesc_size;     /*     8     4 */
	__u32                      efi_memdesc_version;  /*    12     4 */
	__u32                      efi_memmap;           /*    16     4 */
	__u32                      efi_memmap_size;      /*    20     4 */
	__u32                      efi_systab_hi;        /*    24     4 */
	__u32                      efi_memmap_hi;        /*    28     4 */

	/* size: 32, cachelines: 1, members: 8 */
	/* last cacheline: 32 bytes */
};
struct boot_e820_entry {
	__u64                      addr;                 /*     0     8 */
	__u64                      size;                 /*     8     8 */
	__u32                      type;                 /*    16     4 */

	/* size: 20, cachelines: 1, members: 3 */
	/* last cacheline: 20 bytes */
} __attribute__((__packed__));
struct boot_params {
	struct screen_info         screen_info;          /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct apm_bios_info       apm_bios_info;        /*    64    20 */
	__u8                       _pad2[4];             /*    84     4 */
	__u64                      tboot_addr;           /*    88     8 */
	struct ist_info            ist_info;             /*    96    16 */
	__u64                      acpi_rsdp_addr;       /*   112     8 */
	__u8                       _pad3[8];             /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	__u8                       hd0_info[16];         /*   128    16 */
	__u8                       hd1_info[16];         /*   144    16 */
	struct sys_desc_table      sys_desc_table;       /*   160    16 */
	struct olpc_ofw_header     olpc_ofw_header;      /*   176    16 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	__u32                      ext_ramdisk_image;    /*   192     4 */
	__u32                      ext_ramdisk_size;     /*   196     4 */
	__u32                      ext_cmd_line_ptr;     /*   200     4 */
	__u8                       _pad4[116];           /*   204   116 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	struct edid_info           edid_info;            /*   320   128 */
	/* --- cacheline 7 boundary (448 bytes) --- */
	struct efi_info            efi_info;             /*   448    32 */
	__u32                      alt_mem_k;            /*   480     4 */
	__u32                      scratch;              /*   484     4 */
	__u8                       e820_entries;         /*   488     1 */
	__u8                       eddbuf_entries;       /*   489     1 */
	__u8                       edd_mbr_sig_buf_entries; /*   490     1 */
	__u8                       kbd_status;           /*   491     1 */
	__u8                       secure_boot;          /*   492     1 */
	__u8                       _pad5[2];             /*   493     2 */
	__u8                       sentinel;             /*   495     1 */
	__u8                       _pad6[1];             /*   496     1 */
	struct setup_header        hdr;                  /*   497   123 */
	/* --- cacheline 9 boundary (576 bytes) was 44 bytes ago --- */
	__u8                       _pad7[36];            /*   620    36 */
	/* --- cacheline 10 boundary (640 bytes) was 16 bytes ago --- */
	__u32                      edd_mbr_sig_buffer[16]; /*   656    64 */
	/* --- cacheline 11 boundary (704 bytes) was 16 bytes ago --- */
	struct boot_e820_entry     e820_table[128];      /*   720  2560 */
	/* --- cacheline 51 boundary (3264 bytes) was 16 bytes ago --- */
	__u8                       _pad8[48];            /*  3280    48 */
	/* --- cacheline 52 boundary (3328 bytes) --- */
	struct edd_info            eddbuf[6];            /*  3328   492 */
	/* --- cacheline 59 boundary (3776 bytes) was 44 bytes ago --- */
	__u8                       _pad9[276];           /*  3820   276 */

	/* size: 4096, cachelines: 64, members: 34 */
} __attribute__((__packed__));
struct range {
	u64                        start;                /*     0     8 */
	u64                        end;                  /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct pt_regs {
	long unsigned int          r15;                  /*     0     8 */
	long unsigned int          r14;                  /*     8     8 */
	long unsigned int          r13;                  /*    16     8 */
	long unsigned int          r12;                  /*    24     8 */
	long unsigned int          bp;                   /*    32     8 */
	long unsigned int          bx;                   /*    40     8 */
	long unsigned int          r11;                  /*    48     8 */
	long unsigned int          r10;                  /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	long unsigned int          r9;                   /*    64     8 */
	long unsigned int          r8;                   /*    72     8 */
	long unsigned int          ax;                   /*    80     8 */
	long unsigned int          cx;                   /*    88     8 */
	long unsigned int          dx;                   /*    96     8 */
	long unsigned int          si;                   /*   104     8 */
	long unsigned int          di;                   /*   112     8 */
	long unsigned int          orig_ax;              /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	long unsigned int          ip;                   /*   128     8 */
	long unsigned int          cs;                   /*   136     8 */
	long unsigned int          flags;                /*   144     8 */
	long unsigned int          sp;                   /*   152     8 */
	long unsigned int          ss;                   /*   160     8 */

	/* size: 168, cachelines: 3, members: 21 */
	/* last cacheline: 40 bytes */
};
struct desc_struct {
	u16                        limit0;               /*     0     2 */
	u16                        base0;                /*     2     2 */
	u16                        base1:8;              /*     4: 0  2 */
	u16                        type:4;               /*     4: 8  2 */
	u16                        s:1;                  /*     4:12  2 */
	u16                        dpl:2;                /*     4:13  2 */
	u16                        p:1;                  /*     4:15  2 */
	u16                        limit1:4;             /*     6: 0  2 */
	u16                        avl:1;                /*     6: 4  2 */
	u16                        l:1;                  /*     6: 5  2 */
	u16                        d:1;                  /*     6: 6  2 */
	u16                        g:1;                  /*     6: 7  2 */
	u16                        base2:8;              /*     6: 8  2 */

	/* size: 8, cachelines: 1, members: 13 */
	/* last cacheline: 8 bytes */
};
struct idt_bits {
	u16                        ist:3;                /*     0: 0  2 */
	u16                        zero:5;               /*     0: 3  2 */
	u16                        type:5;               /*     0: 8  2 */
	u16                        dpl:2;                /*     0:13  2 */
	u16                        p:1;                  /*     0:15  2 */

	/* size: 2, cachelines: 1, members: 5 */
	/* last cacheline: 2 bytes */
};
struct idt_data {
	unsigned int               vector;               /*     0     4 */
	unsigned int               segment;              /*     4     4 */
	struct idt_bits            bits;                 /*     8     2 */

	/* XXX 6 bytes hole, try to pack */

	const void  *              addr;                 /*    16     8 */

	/* size: 24, cachelines: 1, members: 4 */
	/* sum members: 18, holes: 1, sum holes: 6 */
	/* last cacheline: 24 bytes */
};
struct gate_struct {
	u16                        offset_low;           /*     0     2 */
	u16                        segment;              /*     2     2 */
	struct idt_bits            bits;                 /*     4     2 */
	u16                        offset_middle;        /*     6     2 */
	u32                        offset_high;          /*     8     4 */
	u32                        reserved;             /*    12     4 */

	/* size: 16, cachelines: 1, members: 6 */
	/* last cacheline: 16 bytes */
};
struct desc_ptr {
	short unsigned int         size;                 /*     0     2 */
	long unsigned int          address;              /*     2     8 */

	/* size: 10, cachelines: 1, members: 2 */
	/* last cacheline: 10 bytes */
} __attribute__((__packed__));
struct pgprot {
	pgprotval_t                pgprot;               /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct page {
	long unsigned int          flags;                /*     0     8 */
	union {
		struct {
			struct list_head lru;            /*     8    16 */
			struct address_space * mapping;  /*    24     8 */
			long unsigned int index;         /*    32     8 */
			long unsigned int private;       /*    40     8 */
		};                                       /*     8    40 */
		struct {
			long unsigned int pp_magic;      /*     8     8 */
			struct page_pool * pp;           /*    16     8 */
			long unsigned int _pp_mapping_pad; /*    24     8 */
			long unsigned int dma_addr;      /*    32     8 */
			atomic_long_t pp_frag_count;     /*    40     8 */
		};                                       /*     8    40 */
		struct {
			union {
				struct list_head slab_list; /*     8    16 */
				struct {
					struct page * next; /*     8     8 */
					int    pages;    /*    16     4 */
					int    pobjects; /*    20     4 */
				};                       /*     8    16 */
			};                               /*     8    16 */
			struct kmem_cache * slab_cache;  /*    24     8 */
			void *     freelist;             /*    32     8 */
			union {
				void * s_mem;            /*    40     8 */
				long unsigned int counters; /*    40     8 */
				struct {
					unsigned int inuse:16; /*    40: 0  4 */
					unsigned int objects:15; /*    40:16  4 */
					unsigned int frozen:1; /*    40:31  4 */
				};                       /*    40     4 */
			};                               /*    40     8 */
		};                                       /*     8    40 */
		struct {
			long unsigned int compound_head; /*     8     8 */
			unsigned char compound_dtor;     /*    16     1 */
			unsigned char compound_order;    /*    17     1 */

			/* XXX 2 bytes hole, try to pack */

			atomic_t   compound_mapcount;    /*    20     4 */
			unsigned int compound_nr;        /*    24     4 */
		};                                       /*     8    24 */
		struct {
			long unsigned int _compound_pad_1; /*     8     8 */
			atomic_t   hpage_pinned_refcount; /*    16     4 */

			/* XXX 4 bytes hole, try to pack */

			struct list_head deferred_list;  /*    24    16 */
		};                                       /*     8    32 */
		struct {
			long unsigned int _pt_pad_1;     /*     8     8 */
			pgtable_t  pmd_huge_pte;         /*    16     8 */
			long unsigned int _pt_pad_2;     /*    24     8 */
			union {
				struct mm_struct * pt_mm; /*    32     8 */
				atomic_t pt_frag_refcount; /*    32     4 */
			};                               /*    32     8 */
			spinlock_t ptl;                  /*    40     4 */
		};                                       /*     8    40 */
		struct {
			struct dev_pagemap * pgmap;      /*     8     8 */
			void *     zone_device_data;     /*    16     8 */
		};                                       /*     8    16 */
		struct callback_head callback_head __attribute__((__aligned__(8))); /*     8    16 */
	} __attribute__((__aligned__(8)));               /*     8    40 */
	union {
		atomic_t           _mapcount;            /*    48     4 */
		unsigned int       page_type;            /*    48     4 */
		unsigned int       active;               /*    48     4 */
		int                units;                /*    48     4 */
	};                                               /*    48     4 */
	atomic_t                   _refcount;            /*    52     4 */
	long unsigned int          memcg_data;           /*    56     8 */

	/* size: 64, cachelines: 1, members: 5 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(16)));
struct cpumask {
	long unsigned int          bits[128];            /*     0  1024 */

	/* size: 1024, cachelines: 16, members: 1 */
};
struct mm_struct {
	struct {
		struct vm_area_struct * mmap;            /*     0     8 */
		struct rb_root     mm_rb;                /*     8     8 */
		u64                vmacache_seqnum;      /*    16     8 */
		long unsigned int  (*get_unmapped_area)(struct file *, long unsigned int, long unsigned int, long unsigned int, long unsigned int); /*    24     8 */
		long unsigned int  mmap_base;            /*    32     8 */
		long unsigned int  mmap_legacy_base;     /*    40     8 */
		long unsigned int  mmap_compat_base;     /*    48     8 */
		long unsigned int  mmap_compat_legacy_base; /*    56     8 */
		/* --- cacheline 1 boundary (64 bytes) --- */
		long unsigned int  task_size;            /*    64     8 */
		long unsigned int  highest_vm_end;       /*    72     8 */
		pgd_t *            pgd;                  /*    80     8 */
		atomic_t           membarrier_state;     /*    88     4 */
		atomic_t           mm_users;             /*    92     4 */
		atomic_t           mm_count;             /*    96     4 */
		atomic_t           has_pinned;           /*   100     4 */
		atomic_long_t      pgtables_bytes;       /*   104     8 */
		int                map_count;            /*   112     4 */
		spinlock_t         page_table_lock;      /*   116     4 */
		struct rw_semaphore mmap_lock;           /*   120    40 */
		/* --- cacheline 2 boundary (128 bytes) was 32 bytes ago --- */
		struct list_head   mmlist;               /*   160    16 */
		long unsigned int  hiwater_rss;          /*   176     8 */
		long unsigned int  hiwater_vm;           /*   184     8 */
		/* --- cacheline 3 boundary (192 bytes) --- */
		long unsigned int  total_vm;             /*   192     8 */
		long unsigned int  locked_vm;            /*   200     8 */
		atomic64_t         pinned_vm;            /*   208     8 */
		long unsigned int  data_vm;              /*   216     8 */
		long unsigned int  exec_vm;              /*   224     8 */
		long unsigned int  stack_vm;             /*   232     8 */
		long unsigned int  def_flags;            /*   240     8 */
		seqcount_t         write_protect_seq;    /*   248     4 */
		spinlock_t         arg_lock;             /*   252     4 */
		/* --- cacheline 4 boundary (256 bytes) --- */
		long unsigned int  start_code;           /*   256     8 */
		long unsigned int  end_code;             /*   264     8 */
		long unsigned int  start_data;           /*   272     8 */
		long unsigned int  end_data;             /*   280     8 */
		long unsigned int  start_brk;            /*   288     8 */
		long unsigned int  brk;                  /*   296     8 */
		long unsigned int  start_stack;          /*   304     8 */
		long unsigned int  arg_start;            /*   312     8 */
		/* --- cacheline 5 boundary (320 bytes) --- */
		long unsigned int  arg_end;              /*   320     8 */
		long unsigned int  env_start;            /*   328     8 */
		long unsigned int  env_end;              /*   336     8 */
		long unsigned int  kabi_reserved_saved_auxv[46]; /*   344   368 */
		/* --- cacheline 11 boundary (704 bytes) was 8 bytes ago --- */
		struct mm_rss_stat rss_stat;             /*   712    32 */
		struct linux_binfmt * binfmt;            /*   744     8 */
		mm_context_t       context;              /*   752   128 */
		/* --- cacheline 13 boundary (832 bytes) was 48 bytes ago --- */
		long unsigned int  flags;                /*   880     8 */
		struct core_state * core_state;          /*   888     8 */
		/* --- cacheline 14 boundary (896 bytes) --- */
		spinlock_t         ioctx_lock;           /*   896     4 */

		/* XXX 4 bytes hole, try to pack */

		struct kioctx_table * ioctx_table;       /*   904     8 */
		struct task_struct * owner;              /*   912     8 */
		struct user_namespace * user_ns;         /*   920     8 */
		struct file *      exe_file;             /*   928     8 */
		struct mmu_notifier_subscriptions * notifier_subscriptions; /*   936     8 */
		long unsigned int  numa_next_scan;       /*   944     8 */
		long unsigned int  numa_scan_offset;     /*   952     8 */
		/* --- cacheline 15 boundary (960 bytes) --- */
		int                numa_scan_seq;        /*   960     4 */
		atomic_t           tlb_flush_pending;    /*   964     4 */
		bool               tlb_flush_batched;    /*   968     1 */

		/* XXX 7 bytes hole, try to pack */

		struct uprobes_state uprobes_state;      /*   976     8 */
		atomic_long_t      hugetlb_usage;        /*   984     8 */
		struct work_struct async_put_work;       /*   992    64 */
		/* --- cacheline 16 boundary (1024 bytes) was 32 bytes ago --- */
		u32                pasid;                /*  1056     4 */
	};                                               /*     0  1064 */

	/* XXX last struct has 4 bytes of padding */

	union {
		struct mm_struct_extend * mm_extend;     /*  1064     8 */
		struct {
			u64        kabi_reserved1;       /*  1064     8 */
		} kabi_hidden_622;                       /*  1064     8 */
		union {
		};                                       /*  1064     0 */
	};                                               /*  1064     8 */
	u64                        kabi_reserved2;       /*  1072     8 */
	u64                        kabi_reserved3;       /*  1080     8 */
	/* --- cacheline 17 boundary (1088 bytes) --- */
	u64                        kabi_reserved4;       /*  1088     8 */
	u64                        kabi_reserved5;       /*  1096     8 */
	u64                        kabi_reserved6;       /*  1104     8 */
	u64                        kabi_reserved7;       /*  1112     8 */
	u64                        kabi_reserved8;       /*  1120     8 */
	struct kvm *               kvm;                  /*  1128     8 */
	long unsigned int          cpu_bitmap[];         /*  1136     0 */

	/* size: 1136, cachelines: 18, members: 11 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 48 bytes */
};
struct math_emu_info {
	long int                   ___orig_eip;          /*     0     8 */
	struct pt_regs *           regs;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct fregs_state {
	u32                        cwd;                  /*     0     4 */
	u32                        swd;                  /*     4     4 */
	u32                        twd;                  /*     8     4 */
	u32                        fip;                  /*    12     4 */
	u32                        fcs;                  /*    16     4 */
	u32                        foo;                  /*    20     4 */
	u32                        fos;                  /*    24     4 */
	u32                        st_space[20];         /*    28    80 */
	/* --- cacheline 1 boundary (64 bytes) was 44 bytes ago --- */
	u32                        status;               /*   108     4 */

	/* size: 112, cachelines: 2, members: 9 */
	/* last cacheline: 48 bytes */
};
struct fxregs_state {
	u16                        cwd;                  /*     0     2 */
	u16                        swd;                  /*     2     2 */
	u16                        twd;                  /*     4     2 */
	u16                        fop;                  /*     6     2 */
	union {
		struct {
			u64        rip;                  /*     8     8 */
			u64        rdp;                  /*    16     8 */
		};                                       /*     8    16 */
		struct {
			u32        fip;                  /*     8     4 */
			u32        fcs;                  /*    12     4 */
			u32        foo;                  /*    16     4 */
			u32        fos;                  /*    20     4 */
		};                                       /*     8    16 */
	};                                               /*     8    16 */
	u32                        mxcsr;                /*    24     4 */
	u32                        mxcsr_mask;           /*    28     4 */
	u32                        st_space[32];         /*    32   128 */
	/* --- cacheline 2 boundary (128 bytes) was 32 bytes ago --- */
	u32                        xmm_space[64];        /*   160   256 */
	/* --- cacheline 6 boundary (384 bytes) was 32 bytes ago --- */
	u32                        padding[12];          /*   416    48 */
	/* --- cacheline 7 boundary (448 bytes) was 16 bytes ago --- */
	union {
		u32                padding1[12];         /*   464    48 */
		u32                sw_reserved[12];      /*   464    48 */
	};                                               /*   464    48 */

	/* size: 512, cachelines: 8, members: 11 */
} __attribute__((__aligned__(16)));
struct swregs_state {
	u32                        cwd;                  /*     0     4 */
	u32                        swd;                  /*     4     4 */
	u32                        twd;                  /*     8     4 */
	u32                        fip;                  /*    12     4 */
	u32                        fcs;                  /*    16     4 */
	u32                        foo;                  /*    20     4 */
	u32                        fos;                  /*    24     4 */
	u32                        st_space[20];         /*    28    80 */
	/* --- cacheline 1 boundary (64 bytes) was 44 bytes ago --- */
	u8                         ftop;                 /*   108     1 */
	u8                         changed;              /*   109     1 */
	u8                         lookahead;            /*   110     1 */
	u8                         no_update;            /*   111     1 */
	u8                         rm;                   /*   112     1 */
	u8                         alimit;               /*   113     1 */

	/* XXX 6 bytes hole, try to pack */

	struct math_emu_info *     info;                 /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	u32                        entry_eip;            /*   128     4 */

	/* size: 136, cachelines: 3, members: 16 */
	/* sum members: 126, holes: 1, sum holes: 6 */
	/* padding: 4 */
	/* last cacheline: 8 bytes */
};
struct xstate_header {
	u64                        xfeatures;            /*     0     8 */
	u64                        xcomp_bv;             /*     8     8 */
	u64                        reserved[6];          /*    16    48 */

	/* size: 64, cachelines: 1, members: 3 */
};
struct xregs_state {
	struct fxregs_state        i387 __attribute__((__aligned__(1))); /*     0   512 */
	/* --- cacheline 8 boundary (512 bytes) --- */
	struct xstate_header       header;               /*   512    64 */
	/* --- cacheline 9 boundary (576 bytes) --- */
	u8                         extended_state_area[]; /*   576     0 */

	/* size: 576, cachelines: 9, members: 3 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(64)));
union fpregs_state {
	struct fregs_state         fsave;              /*     0   112 */
	struct fxregs_state        fxsave __attribute__((__aligned__(16))); /*     0   512 */
	struct swregs_state        soft;               /*     0   136 */
	struct xregs_state         xsave __attribute__((__aligned__(64))); /*     0   576 */
	u8                         __padding[4096];    /*     0  4096 */
};
struct fpstate {
	unsigned int               size;                 /*     0     4 */
	unsigned int               user_size;            /*     4     4 */
	u64                        xfeatures;            /*     8     8 */
	u64                        user_xfeatures;       /*    16     8 */
	u64                        xfd;                  /*    24     8 */
	unsigned int               is_valloc:1;          /*    32: 0  4 */
	unsigned int               is_guest:1;           /*    32: 1  4 */
	unsigned int               is_confidential:1;    /*    32: 2  4 */
	unsigned int               in_use:1;             /*    32: 3  4 */

	/* XXX 28 bits hole, try to pack */
	/* XXX 28 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	union fpregs_state         regs __attribute__((__aligned__(64))); /*    64  4096 */

	/* size: 4160, cachelines: 65, members: 10 */
	/* sum members: 4128, holes: 1, sum holes: 28 */
	/* sum bitfield members: 4 bits, bit holes: 1, sum bit holes: 28 bits */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 28 */
} __attribute__((__aligned__(64)));
struct fpu_state_perm {
	u64                        __state_perm;         /*     0     8 */
	unsigned int               __state_size;         /*     8     4 */
	unsigned int               __user_state_size;    /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct fpu {
	unsigned int               last_cpu;             /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          avx512_timestamp;     /*     8     8 */
	struct fpstate *           fpstate;              /*    16     8 */
	struct fpstate *           __task_fpstate;       /*    24     8 */
	struct fpu_state_perm      perm;                 /*    32    16 */
	struct fpu_state_perm      guest_perm;           /*    48    16 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	union fpregs_state         kabi_reserved_state __attribute__((__aligned__(64))); /*    64  4096 */
	/* --- cacheline 65 boundary (4160 bytes) --- */
	struct fpstate             __fpstate __attribute__((__aligned__(64))); /*  4160  4160 */

	/* size: 8320, cachelines: 130, members: 8 */
	/* sum members: 8316, holes: 1, sum holes: 4 */
	/* forced alignments: 2 */
} __attribute__((__aligned__(64)));
struct thread_struct {
	struct desc_struct         tls_array[3];         /*     0    24 */
	long unsigned int          sp;                   /*    24     8 */
	short unsigned int         es;                   /*    32     2 */
	short unsigned int         ds;                   /*    34     2 */
	short unsigned int         fsindex;              /*    36     2 */
	short unsigned int         gsindex;              /*    38     2 */
	long unsigned int          fsbase;               /*    40     8 */
	long unsigned int          gsbase;               /*    48     8 */
	struct perf_event *        ptrace_bps[4];        /*    56    32 */
	/* --- cacheline 1 boundary (64 bytes) was 24 bytes ago --- */
	long unsigned int          virtual_dr6;          /*    88     8 */
	long unsigned int          ptrace_dr7;           /*    96     8 */
	long unsigned int          cr2;                  /*   104     8 */
	long unsigned int          trap_nr;              /*   112     8 */
	long unsigned int          error_code;           /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct io_bitmap *         io_bitmap;            /*   128     8 */
	long unsigned int          iopl_emul;            /*   136     8 */
	unsigned int               iopl_warn:1;          /*   144: 0  4 */
	unsigned int               sig_on_uaccess_err:1; /*   144: 1  4 */

	/* XXX 30 bits hole, try to pack */

	u32                        pkru;                 /*   148     4 */

	/* XXX 40 bytes hole, try to pack */

	/* --- cacheline 3 boundary (192 bytes) --- */
	struct fpu                 fpu __attribute__((__aligned__(64))); /*   192  8320 */

	/* size: 8512, cachelines: 133, members: 20 */
	/* sum members: 8468, holes: 1, sum holes: 40 */
	/* sum bitfield members: 2 bits, bit holes: 1, sum bit holes: 30 bits */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 40 */
} __attribute__((__aligned__(64)));
struct thread_info {
	long unsigned int          flags;                /*     0     8 */
	u32                        status;               /*     8     4 */
	u32                        cpu;                  /*    12     4 */
	u64                        kabi_reserved1;       /*    16     8 */
	u64                        kabi_reserved2;       /*    24     8 */

	/* size: 32, cachelines: 1, members: 5 */
	/* last cacheline: 32 bytes */
};
struct llist_node {
	struct llist_node *        next;                 /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct __call_single_node {
	struct llist_node          llist;                /*     0     8 */
	union {
		unsigned int       u_flags;              /*     8     4 */
		atomic_t           a_flags;              /*     8     4 */
	};                                               /*     8     4 */
	u16                        src;                  /*    12     2 */
	u16                        dst;                  /*    14     2 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct rb_node {
	long unsigned int          __rb_parent_color;    /*     0     8 */
	struct rb_node *           rb_right;             /*     8     8 */
	struct rb_node *           rb_left;              /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct rb_root {
	struct rb_node *           rb_node;              /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct rb_root_cached {
	struct rb_root             rb_root;              /*     0     8 */
	struct rb_node *           rb_leftmost;          /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct vm_struct {
	struct vm_struct *         next;                 /*     0     8 */
	void *                     addr;                 /*     8     8 */
	long unsigned int          size;                 /*    16     8 */
	long unsigned int          flags;                /*    24     8 */
	struct page * *            pages;                /*    32     8 */
	unsigned int               page_order;           /*    40     4 */
	unsigned int               nr_pages;             /*    44     4 */
	phys_addr_t                phys_addr;            /*    48     8 */
	const void  *              caller;               /*    56     8 */

	/* size: 64, cachelines: 1, members: 9 */
};
struct wait_queue_head {
	spinlock_t                 lock;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           head;                 /*     8    16 */

	/* size: 24, cachelines: 1, members: 2 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct optimistic_spin_queue {
	atomic_t                   tail;                 /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct mutex {
	atomic_long_t              owner;                /*     0     8 */
	spinlock_t                 wait_lock;            /*     8     4 */
	struct optimistic_spin_queue osq;                /*    12     4 */
	struct list_head           wait_list;            /*    16    16 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct seqcount {
	unsigned int               sequence;             /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct seqcount_raw_spinlock {
	seqcount_t                 seqcount;             /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct seqcount_spinlock {
	seqcount_t                 seqcount;             /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct arch_tlbflush_unmap_batch {
	struct cpumask             cpumask;              /*     0  1024 */

	/* size: 1024, cachelines: 16, members: 1 */
};
struct vmacache {
	u64                        seqnum;               /*     0     8 */
	struct vm_area_struct *    vmas[4];              /*     8    32 */

	/* size: 40, cachelines: 1, members: 2 */
	/* last cacheline: 40 bytes */
};
struct vm_area_struct {
	long unsigned int          vm_start;             /*     0     8 */
	long unsigned int          vm_end;               /*     8     8 */
	struct vm_area_struct *    vm_next;              /*    16     8 */
	struct vm_area_struct *    vm_prev;              /*    24     8 */
	struct rb_node             vm_rb __attribute__((__aligned__(8))); /*    32    24 */
	long unsigned int          rb_subtree_gap;       /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct mm_struct *         vm_mm;                /*    64     8 */
	pgprot_t                   vm_page_prot;         /*    72     8 */
	long unsigned int          vm_flags;             /*    80     8 */
	struct {
		struct rb_node     rb __attribute__((__aligned__(8))); /*    88    24 */
		long unsigned int  rb_subtree_last;      /*   112     8 */
	} __attribute__((__aligned__(8))) shared __attribute__((__aligned__(8)));        /*    88    32 */
	struct list_head           anon_vma_chain;       /*   120    16 */
	/* --- cacheline 2 boundary (128 bytes) was 8 bytes ago --- */
	struct anon_vma *          anon_vma;             /*   136     8 */
	const struct vm_operations_struct  * vm_ops;     /*   144     8 */
	long unsigned int          vm_pgoff;             /*   152     8 */
	struct file *              vm_file;              /*   160     8 */
	void *                     vm_private_data;      /*   168     8 */
	atomic_long_t              swap_readahead_info;  /*   176     8 */
	struct mempolicy *         vm_policy;            /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	struct vm_userfaultfd_ctx  vm_userfaultfd_ctx;   /*   192     8 */
	u64                        kabi_reserved1;       /*   200     8 */
	u64                        kabi_reserved2;       /*   208     8 */
	u64                        kabi_reserved3;       /*   216     8 */
	u64                        kabi_reserved4;       /*   224     8 */

	/* size: 232, cachelines: 4, members: 23 */
	/* forced alignments: 2 */
	/* last cacheline: 40 bytes */
} __attribute__((__aligned__(8)));
struct task_rss_stat {
	int                        events;               /*     0     4 */
	int                        count[4];             /*     4    16 */

	/* size: 20, cachelines: 1, members: 2 */
	/* last cacheline: 20 bytes */
};
struct mm_rss_stat {
	atomic_long_t              count[4];             /*     0    32 */

	/* size: 32, cachelines: 1, members: 1 */
	/* last cacheline: 32 bytes */
};
struct page_frag {
	struct page *              page;                 /*     0     8 */
	__u32                      offset;               /*     8     4 */
	__u32                      size;                 /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct tlbflush_unmap_batch {
	struct arch_tlbflush_unmap_batch arch;           /*     0  1024 */
	/* --- cacheline 16 boundary (1024 bytes) --- */
	bool                       flush_required;       /*  1024     1 */
	bool                       writable;             /*  1025     1 */

	/* size: 1032, cachelines: 17, members: 3 */
	/* padding: 6 */
	/* last cacheline: 8 bytes */
};
struct rw_semaphore {
	atomic_long_t              count;                /*     0     8 */
	atomic_long_t              owner;                /*     8     8 */
	struct optimistic_spin_queue osq;                /*    16     4 */
	raw_spinlock_t             wait_lock;            /*    20     4 */
	struct list_head           wait_list;            /*    24    16 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct swait_queue_head {
	raw_spinlock_t             lock;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           task_list;            /*     8    16 */

	/* size: 24, cachelines: 1, members: 2 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct completion {
	unsigned int               done;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct swait_queue_head    wait;                 /*     8    24 */

	/* size: 32, cachelines: 1, members: 2 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct timer_list {
	struct hlist_node          entry;                /*     0    16 */
	long unsigned int          expires;              /*    16     8 */
	void                       (*function)(struct timer_list *); /*    24     8 */
	u32                        flags;                /*    32     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        kabi_reserved1;       /*    40     8 */
	u64                        kabi_reserved2;       /*    48     8 */
	u64                        kabi_reserved3;       /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u64                        kabi_reserved4;       /*    64     8 */

	/* size: 72, cachelines: 2, members: 8 */
	/* sum members: 68, holes: 1, sum holes: 4 */
	/* last cacheline: 8 bytes */
};
struct work_struct {
	atomic_long_t              data;                 /*     0     8 */
	struct list_head           entry;                /*     8    16 */
	work_func_t                func;                 /*    24     8 */
	u64                        kabi_reserved1;       /*    32     8 */
	u64                        kabi_reserved2;       /*    40     8 */
	u64                        kabi_reserved3;       /*    48     8 */
	u64                        kabi_reserved4;       /*    56     8 */

	/* size: 64, cachelines: 1, members: 7 */
};
struct arch_uprobe_task {
	long unsigned int          saved_scratch_register; /*     0     8 */
	unsigned int               saved_trap_nr;        /*     8     4 */
	unsigned int               saved_tf;             /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct uprobe_task {
	enum uprobe_task_state     state;                /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	union {
		struct {
			struct arch_uprobe_task autask;  /*     8    16 */
			long unsigned int vaddr;         /*    24     8 */
		};                                       /*     8    24 */
		struct {
			struct callback_head dup_xol_work __attribute__((__aligned__(8))); /*     8    16 */
			long unsigned int dup_xol_addr;  /*    24     8 */
		} __attribute__((__aligned__(8))) __attribute__((__aligned__(8)));       /*     8    24 */
	} __attribute__((__aligned__(8)));               /*     8    24 */
	struct uprobe *            active_uprobe;        /*    32     8 */
	long unsigned int          xol_vaddr;            /*    40     8 */
	struct return_instance *   return_instances;     /*    48     8 */
	unsigned int               depth;                /*    56     4 */

	/* size: 64, cachelines: 1, members: 6 */
	/* sum members: 56, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 4 */
} __attribute__((__aligned__(8)));
struct return_instance {
	struct uprobe *            uprobe;               /*     0     8 */
	long unsigned int          func;                 /*     8     8 */
	long unsigned int          stack;                /*    16     8 */
	long unsigned int          orig_ret_vaddr;       /*    24     8 */
	bool                       chained;              /*    32     1 */

	/* XXX 7 bytes hole, try to pack */

	struct return_instance *   next;                 /*    40     8 */

	/* size: 48, cachelines: 1, members: 6 */
	/* sum members: 41, holes: 1, sum holes: 7 */
	/* last cacheline: 48 bytes */
};
struct uprobes_state {
	struct xol_area *          xol_area;             /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct address_space {
	struct inode *             host;                 /*     0     8 */
	struct xarray              i_pages;              /*     8    16 */
	gfp_t                      gfp_mask;             /*    24     4 */
	atomic_t                   i_mmap_writable;      /*    28     4 */
	struct rb_root_cached      i_mmap;               /*    32    16 */
	struct rw_semaphore        i_mmap_rwsem;         /*    48    40 */
	/* --- cacheline 1 boundary (64 bytes) was 24 bytes ago --- */
	long unsigned int          nrpages;              /*    88     8 */
	long unsigned int          writeback_index;      /*    96     8 */
	const struct address_space_operations  * a_ops;  /*   104     8 */
	long unsigned int          flags;                /*   112     8 */
	errseq_t                   wb_err;               /*   120     4 */
	spinlock_t                 private_lock;         /*   124     4 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct list_head           private_list;         /*   128    16 */
	void *                     private_data;         /*   144     8 */
	u64                        kabi_reserved1;       /*   152     8 */
	u64                        kabi_reserved2;       /*   160     8 */
	u64                        kabi_reserved3;       /*   168     8 */
	u64                        kabi_reserved4;       /*   176     8 */

	/* size: 184, cachelines: 3, members: 18 */
	/* last cacheline: 56 bytes */
} __attribute__((__aligned__(8)));
struct dev_pagemap {
	struct vmem_altmap         altmap;               /*     0    48 */
	struct percpu_ref *        ref;                  /*    48     8 */
	struct percpu_ref          internal_ref;         /*    56    16 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	struct completion          done;                 /*    72    32 */
	enum memory_type           type;                 /*   104     4 */
	unsigned int               flags;                /*   108     4 */
	const struct dev_pagemap_ops  * ops;             /*   112     8 */
	void *                     owner;                /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	int                        nr_range;             /*   128     4 */

	/* XXX 4 bytes hole, try to pack */

	union {
		struct range       range;                /*   136    16 */
		struct range       ranges[0];            /*   136     0 */
	};                                               /*   136    16 */

	/* size: 152, cachelines: 3, members: 10 */
	/* sum members: 148, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct file {
	union {
		struct llist_node  fu_llist;             /*     0     8 */
		struct callback_head fu_rcuhead __attribute__((__aligned__(8))); /*     0    16 */
	} f_u __attribute__((__aligned__(8)));           /*     0    16 */
	struct path                f_path;               /*    16    16 */
	struct inode *             f_inode;              /*    32     8 */
	const struct file_operations  * f_op;            /*    40     8 */
	spinlock_t                 f_lock;               /*    48     4 */
	enum rw_hint               f_write_hint;         /*    52     4 */
	atomic_long_t              f_count;              /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	unsigned int               f_flags;              /*    64     4 */
	fmode_t                    f_mode;               /*    68     4 */
	struct mutex               f_pos_lock;           /*    72    32 */
	loff_t                     f_pos;                /*   104     8 */
	struct fown_struct         f_owner;              /*   112    32 */
	/* --- cacheline 2 boundary (128 bytes) was 16 bytes ago --- */
	const struct cred  *       f_cred;               /*   144     8 */
	struct file_ra_state       f_ra;                 /*   152    32 */
	u64                        f_version;            /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	void *                     f_security;           /*   192     8 */
	void *                     private_data;         /*   200     8 */
	struct list_head           f_ep_links;           /*   208    16 */
	struct list_head           f_tfile_llink;        /*   224    16 */
	struct address_space *     f_mapping;            /*   240     8 */
	errseq_t                   f_wb_err;             /*   248     4 */
	errseq_t                   f_sb_err;             /*   252     4 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	union {
		fmode_t            f_ctl_mode;           /*   256     4 */
		u64                kabi_reserved1;       /*   256     8 */
	};                                               /*   256     8 */

	/* size: 264, cachelines: 5, members: 23 */
	/* forced alignments: 1 */
	/* last cacheline: 8 bytes */
} __attribute__((__aligned__(8)));
struct vm_userfaultfd_ctx {
	struct userfaultfd_ctx *   ctx;                  /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct vm_operations_struct {
	void                       (*open)(struct vm_area_struct *); /*     0     8 */
	void                       (*close)(struct vm_area_struct *); /*     8     8 */
	int                        (*split)(struct vm_area_struct *, long unsigned int); /*    16     8 */
	int                        (*mremap)(struct vm_area_struct *); /*    24     8 */
	int                        (*mprotect)(struct vm_area_struct *, long unsigned int, long unsigned int, long unsigned int); /*    32     8 */
	vm_fault_t                 (*fault)(struct vm_fault *); /*    40     8 */
	vm_fault_t                 (*huge_fault)(struct vm_fault *, enum page_entry_size); /*    48     8 */
	void                       (*map_pages)(struct vm_fault *, long unsigned int, long unsigned int); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	long unsigned int          (*pagesize)(struct vm_area_struct *); /*    64     8 */
	vm_fault_t                 (*page_mkwrite)(struct vm_fault *); /*    72     8 */
	vm_fault_t                 (*pfn_mkwrite)(struct vm_fault *); /*    80     8 */
	int                        (*access)(struct vm_area_struct *, long unsigned int, void *, int, int); /*    88     8 */
	const char  *              (*name)(struct vm_area_struct *); /*    96     8 */
	int                        (*set_policy)(struct vm_area_struct *, struct mempolicy *); /*   104     8 */
	struct mempolicy *         (*get_policy)(struct vm_area_struct *, long unsigned int); /*   112     8 */
	struct page *              (*find_special_page)(struct vm_area_struct *, long unsigned int); /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	u64                        kabi_reserved1;       /*   128     8 */
	u64                        kabi_reserved2;       /*   136     8 */
	u64                        kabi_reserved3;       /*   144     8 */
	u64                        kabi_reserved4;       /*   152     8 */

	/* size: 160, cachelines: 3, members: 20 */
	/* last cacheline: 32 bytes */
};
struct core_thread {
	struct task_struct *       task;                 /*     0     8 */
	struct core_thread *       next;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct core_state {
	atomic_t                   nr_threads;           /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct core_thread         dumper;               /*     8    16 */
	struct completion          startup;              /*    24    32 */

	/* size: 56, cachelines: 1, members: 3 */
	/* sum members: 52, holes: 1, sum holes: 4 */
	/* last cacheline: 56 bytes */
};
struct mm_struct_extend {
	long unsigned int          saved_auxv[48];       /*     0   384 */

	/* size: 384, cachelines: 6, members: 1 */
};
struct vm_fault {
	struct vm_area_struct *    vma;                  /*     0     8 */
	unsigned int               flags;                /*     8     4 */
	gfp_t                      gfp_mask;             /*    12     4 */
	long unsigned int          pgoff;                /*    16     8 */
	long unsigned int          address;              /*    24     8 */
	pmd_t *                    pmd;                  /*    32     8 */
	pud_t *                    pud;                  /*    40     8 */
	pte_t                      orig_pte;             /*    48     8 */
	struct page *              cow_page;             /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct page *              page;                 /*    64     8 */
	pte_t *                    pte;                  /*    72     8 */
	spinlock_t *               ptl;                  /*    80     8 */
	pgtable_t                  prealloc_pte;         /*    88     8 */

	/* size: 96, cachelines: 2, members: 13 */
	/* last cacheline: 32 bytes */
};
struct vmem_altmap {
	const long unsigned int    base_pfn;             /*     0     8 */
	const long unsigned int    end_pfn;              /*     8     8 */
	const long unsigned int    reserve;              /*    16     8 */
	long unsigned int          free;                 /*    24     8 */
	long unsigned int          align;                /*    32     8 */
	long unsigned int          alloc;                /*    40     8 */

	/* size: 48, cachelines: 1, members: 6 */
	/* last cacheline: 48 bytes */
};
struct percpu_ref {
	long unsigned int          percpu_count_ptr;     /*     0     8 */
	struct percpu_ref_data *   data;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct percpu_ref_data {
	atomic_long_t              count;                /*     0     8 */
	percpu_ref_func_t *        release;              /*     8     8 */
	percpu_ref_func_t *        confirm_switch;       /*    16     8 */
	bool                       force_atomic:1;       /*    24: 0  1 */
	bool                       allow_reinit:1;       /*    24: 1  1 */

	/* XXX 6 bits hole, try to pack */
	/* XXX 7 bytes hole, try to pack */

	struct callback_head       rcu __attribute__((__aligned__(8))); /*    32    16 */
	struct percpu_ref *        ref;                  /*    48     8 */

	/* size: 56, cachelines: 1, members: 7 */
	/* sum members: 48, holes: 1, sum holes: 7 */
	/* sum bitfield members: 2 bits, bit holes: 1, sum bit holes: 6 bits */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 7 */
	/* last cacheline: 56 bytes */
} __attribute__((__aligned__(8)));
struct shrink_control {
	gfp_t                      gfp_mask;             /*     0     4 */
	int                        nid;                  /*     4     4 */
	long unsigned int          nr_to_scan;           /*     8     8 */
	long unsigned int          nr_scanned;           /*    16     8 */
	struct mem_cgroup *        memcg;                /*    24     8 */

	/* size: 32, cachelines: 1, members: 5 */
	/* last cacheline: 32 bytes */
};
struct shrinker {
	long unsigned int          (*count_objects)(struct shrinker *, struct shrink_control *); /*     0     8 */
	long unsigned int          (*scan_objects)(struct shrinker *, struct shrink_control *); /*     8     8 */
	long int                   batch;                /*    16     8 */
	int                        seeks;                /*    24     4 */
	unsigned int               flags;                /*    28     4 */
	struct list_head           list;                 /*    32    16 */
	int                        id;                   /*    48     4 */

	/* XXX 4 bytes hole, try to pack */

	atomic_long_t *            nr_deferred;          /*    56     8 */

	/* size: 64, cachelines: 1, members: 8 */
	/* sum members: 60, holes: 1, sum holes: 4 */
};
struct rlimit {
	__kernel_ulong_t           rlim_cur;             /*     0     8 */
	__kernel_ulong_t           rlim_max;             /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct dev_pagemap_ops {
	void                       (*page_free)(struct page *); /*     0     8 */
	void                       (*kill)(struct dev_pagemap *); /*     8     8 */
	void                       (*cleanup)(struct dev_pagemap *); /*    16     8 */
	vm_fault_t                 (*migrate_to_ram)(struct vm_fault *); /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct refcount_struct {
	atomic_t                   refs;                 /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct upid {
	int                        nr;                   /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct pid_namespace *     ns;                   /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct pid {
	refcount_t                 count;                /*     0     4 */
	unsigned int               level;                /*     4     4 */
	spinlock_t                 lock;                 /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct hlist_head          tasks[4];             /*    16    32 */
	struct hlist_head          inodes;               /*    48     8 */
	wait_queue_head_t          wait_pidfd;           /*    56    24 */
	/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*    80    16 */
	struct upid                numbers[1];           /*    96    16 */

	/* size: 112, cachelines: 2, members: 8 */
	/* sum members: 108, holes: 1, sum holes: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct sysv_sem {
	struct sem_undo_list *     undo_list;            /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct sysv_shm {
	struct list_head           shm_clist;            /*     0    16 */

	/* size: 16, cachelines: 1, members: 1 */
	/* last cacheline: 16 bytes */
};
struct plist_node {
	int                        prio;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           prio_list;            /*     8    16 */
	struct list_head           node_list;            /*    24    16 */

	/* size: 40, cachelines: 1, members: 3 */
	/* sum members: 36, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct timerqueue_node {
	struct rb_node             node __attribute__((__aligned__(8))); /*     0    24 */
	ktime_t                    expires;              /*    24     8 */

	/* size: 32, cachelines: 1, members: 2 */
	/* forced alignments: 1 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct timerqueue_head {
	struct rb_root_cached      rb_root;              /*     0    16 */

	/* size: 16, cachelines: 1, members: 1 */
	/* last cacheline: 16 bytes */
};
struct hrtimer_resvd {

	/* size: 0, cachelines: 0, members: 0 */
};
struct hrtimer {
	struct timerqueue_node     node __attribute__((__aligned__(8))); /*     0    32 */
	ktime_t                    _softexpires;         /*    32     8 */
	enum hrtimer_restart       (*function)(struct hrtimer *); /*    40     8 */
	struct hrtimer_clock_base * base;                /*    48     8 */
	u8                         state;                /*    56     1 */
	u8                         is_rel;               /*    57     1 */
	u8                         is_soft;              /*    58     1 */
	u8                         is_hard;              /*    59     1 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	u64                        kabi_reserved1;       /*    64     8 */
	u64                        kabi_reserved2;       /*    72     8 */
	size_t                     hrtimer_size_resvd;   /*    80     8 */
	struct hrtimer_resvd *     _resvd;               /*    88     8 */

	/* size: 96, cachelines: 2, members: 12 */
	/* sum members: 92, holes: 1, sum holes: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct hrtimer_clock_base {
	struct hrtimer_cpu_base *  cpu_base;             /*     0     8 */
	unsigned int               index;                /*     8     4 */
	clockid_t                  clockid;              /*    12     4 */
	seqcount_raw_spinlock_t    seq;                  /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	struct hrtimer *           running;              /*    24     8 */
	struct timerqueue_head     active;               /*    32    16 */
	ktime_t                    (*get_time)(void);    /*    48     8 */
	ktime_t                    offset;               /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u64                        kabi_reserved1;       /*    64     8 */
	u64                        kabi_reserved2;       /*    72     8 */

	/* size: 128, cachelines: 2, members: 10 */
	/* sum members: 76, holes: 1, sum holes: 4 */
	/* padding: 48 */
} __attribute__((__aligned__(64)));
struct hrtimer_cpu_base {
	raw_spinlock_t             lock;                 /*     0     4 */
	unsigned int               cpu;                  /*     4     4 */
	unsigned int               active_bases;         /*     8     4 */
	unsigned int               clock_was_set_seq;    /*    12     4 */
	unsigned int               hres_active:1;        /*    16: 0  4 */
	unsigned int               in_hrtirq:1;          /*    16: 1  4 */
	unsigned int               hang_detected:1;      /*    16: 2  4 */
	unsigned int               softirq_activated:1;  /*    16: 3  4 */

	/* XXX 28 bits hole, try to pack */

	unsigned int               nr_events;            /*    20     4 */
	short unsigned int         nr_retries;           /*    24     2 */
	short unsigned int         nr_hangs;             /*    26     2 */
	unsigned int               max_hang_time;        /*    28     4 */
	ktime_t                    expires_next;         /*    32     8 */
	struct hrtimer *           next_timer;           /*    40     8 */
	ktime_t                    softirq_expires_next; /*    48     8 */
	struct hrtimer *           softirq_next_timer;   /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct hrtimer_clock_base  clock_base[8] __attribute__((__aligned__(64))); /*    64  1024 */
	/* --- cacheline 17 boundary (1088 bytes) --- */
	u64                        kabi_reserved1;       /*  1088     8 */
	u64                        kabi_reserved2;       /*  1096     8 */

	/* size: 1152, cachelines: 18, members: 19 */
	/* sum members: 1100 */
	/* sum bitfield members: 4 bits, bit holes: 1, sum bit holes: 28 bits */
	/* padding: 48 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(64)));
struct seccomp {
	int                        mode;                 /*     0     4 */
	atomic_t                   filter_count;         /*     4     4 */
	struct seccomp_filter *    filter;               /*     8     8 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
union sigval {
	int                        sival_int;          /*     0     4 */
	void *                     sival_ptr;          /*     0     8 */
};
union __sifields {
	struct {
		__kernel_pid_t     _pid;               /*     0     4 */
		__kernel_uid32_t   _uid;               /*     4     4 */
	} _kill;                                       /*     0     8 */
	struct {
		__kernel_timer_t   _tid;               /*     0     4 */
		int                _overrun;           /*     4     4 */
		sigval_t           _sigval;            /*     8     8 */
		int                _sys_private;       /*    16     4 */
	} _timer;                                      /*     0    24 */
	struct {
		__kernel_pid_t     _pid;               /*     0     4 */
		__kernel_uid32_t   _uid;               /*     4     4 */
		sigval_t           _sigval;            /*     8     8 */
	} _rt;                                         /*     0    16 */
	struct {
		__kernel_pid_t     _pid;               /*     0     4 */
		__kernel_uid32_t   _uid;               /*     4     4 */
		int                _status;            /*     8     4 */

		/* XXX 4 bytes hole, try to pack */

		__kernel_clock_t   _utime;             /*    16     8 */
		__kernel_clock_t   _stime;             /*    24     8 */
	} _sigchld;                                    /*     0    32 */
	struct {
		void *             _addr;              /*     0     8 */
		union {
			short int  _addr_lsb;          /*     8     2 */
			struct {
				char _dummy_bnd[8];    /*     8     8 */
				void * _lower;         /*    16     8 */
				void * _upper;         /*    24     8 */
			} _addr_bnd;                   /*     8    24 */
			struct {
				char _dummy_pkey[8];   /*     8     8 */
				__u32 _pkey;           /*    16     4 */
			} _addr_pkey;                  /*     8    12 */
		};                                     /*     8    24 */
	} _sigfault;                                   /*     0    32 */
	struct {
		long int           _band;              /*     0     8 */
		int                _fd;                /*     8     4 */
	} _sigpoll;                                    /*     0    16 */
	struct {
		void *             _call_addr;         /*     0     8 */
		int                _syscall;           /*     8     4 */
		unsigned int       _arch;              /*    12     4 */
	} _sigsys;                                     /*     0    16 */
};
struct kernel_siginfo {
	struct {
		int                si_signo;             /*     0     4 */
		int                si_errno;             /*     4     4 */
		int                si_code;              /*     8     4 */

		/* XXX 4 bytes hole, try to pack */

		union __sifields   _sifields;            /*    16    32 */
	};                                               /*     0    48 */

	/* size: 48, cachelines: 1, members: 1 */
	/* last cacheline: 48 bytes */
};
struct user_struct {
	refcount_t                 __count;              /*     0     4 */
	atomic_t                   processes;            /*     4     4 */
	atomic_t                   sigpending;           /*     8     4 */
	atomic_t                   fanotify_listeners;   /*    12     4 */
	atomic_long_t              epoll_watches;        /*    16     8 */
	long unsigned int          mq_bytes;             /*    24     8 */
	long unsigned int          locked_shm;           /*    32     8 */
	long unsigned int          unix_inflight;        /*    40     8 */
	atomic_long_t              pipe_bufs;            /*    48     8 */
	struct hlist_node          uidhash_node;         /*    56    16 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	kuid_t                     uid;                  /*    72     4 */

	/* XXX 4 bytes hole, try to pack */

	atomic_long_t              locked_vm;            /*    80     8 */
	struct ratelimit_state     ratelimit;            /*    88    40 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	u64                        kabi_reserved1;       /*   128     8 */
	u64                        kabi_reserved2;       /*   136     8 */

	/* size: 144, cachelines: 3, members: 15 */
	/* sum members: 140, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct sigpending {
	struct list_head           list;                 /*     0    16 */
	sigset_t                   signal;               /*    16     8 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct sigaction {
	__sighandler_t             sa_handler;           /*     0     8 */
	long unsigned int          sa_flags;             /*     8     8 */
	__sigrestore_t             sa_restorer;          /*    16     8 */
	sigset_t                   sa_mask;              /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct k_sigaction {
	struct sigaction           sa;                   /*     0    32 */

	/* size: 32, cachelines: 1, members: 1 */
	/* last cacheline: 32 bytes */
};
struct task_io_accounting {
	u64                        rchar;                /*     0     8 */
	u64                        wchar;                /*     8     8 */
	u64                        syscr;                /*    16     8 */
	u64                        syscw;                /*    24     8 */
	u64                        read_bytes;           /*    32     8 */
	u64                        write_bytes;          /*    40     8 */
	u64                        cancelled_write_bytes; /*    48     8 */

	/* size: 56, cachelines: 1, members: 7 */
	/* last cacheline: 56 bytes */
};
struct posix_cputimer_base {
	u64                        nextevt;              /*     0     8 */
	struct timerqueue_head     tqhead;               /*     8    16 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct posix_cputimers {
	struct posix_cputimer_base bases[3];             /*     0    72 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	unsigned int               timers_active;        /*    72     4 */
	unsigned int               expiry_active;        /*    76     4 */

	/* size: 80, cachelines: 2, members: 3 */
	/* last cacheline: 16 bytes */
};
struct posix_cputimers_work {
	struct callback_head       work __attribute__((__aligned__(8))); /*     0    16 */
	unsigned int               scheduled;            /*    16     4 */

	/* size: 24, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct signal_struct {
	refcount_t                 sigcnt;               /*     0     4 */
	atomic_t                   live;                 /*     4     4 */
	int                        nr_threads;           /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           thread_head;          /*    16    16 */
	wait_queue_head_t          wait_chldexit;        /*    32    24 */
	struct task_struct *       curr_target;          /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct sigpending          shared_pending;       /*    64    24 */
	struct hlist_head          multiprocess;         /*    88     8 */
	int                        group_exit_code;      /*    96     4 */
	int                        notify_count;         /*   100     4 */
	struct task_struct *       group_exit_task;      /*   104     8 */
	int                        group_stop_count;     /*   112     4 */
	unsigned int               flags;                /*   116     4 */
	unsigned int               is_child_subreaper:1; /*   120: 0  4 */
	unsigned int               has_child_subreaper:1; /*   120: 1  4 */

	/* XXX 30 bits hole, try to pack */

	int                        posix_timer_id;       /*   124     4 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct list_head           posix_timers;         /*   128    16 */
	struct hrtimer             real_timer __attribute__((__aligned__(8))); /*   144    96 */
	/* --- cacheline 3 boundary (192 bytes) was 48 bytes ago --- */
	ktime_t                    it_real_incr;         /*   240     8 */
	struct cpu_itimer          it[2];                /*   248    32 */
	/* --- cacheline 4 boundary (256 bytes) was 24 bytes ago --- */
	struct thread_group_cputimer cputimer;           /*   280    24 */
	struct posix_cputimers     posix_cputimers;      /*   304    80 */
	/* --- cacheline 6 boundary (384 bytes) --- */
	struct pid *               pids[4];              /*   384    32 */
	atomic_t                   tick_dep_mask;        /*   416     4 */

	/* XXX 4 bytes hole, try to pack */

	struct pid *               tty_old_pgrp;         /*   424     8 */
	int                        leader;               /*   432     4 */

	/* XXX 4 bytes hole, try to pack */

	struct tty_struct *        tty;                  /*   440     8 */
	/* --- cacheline 7 boundary (448 bytes) --- */
	struct autogroup *         autogroup;            /*   448     8 */
	seqlock_t                  stats_lock;           /*   456     8 */
	u64                        utime;                /*   464     8 */
	u64                        stime;                /*   472     8 */
	u64                        cutime;               /*   480     8 */
	u64                        cstime;               /*   488     8 */
	u64                        gtime;                /*   496     8 */
	u64                        cgtime;               /*   504     8 */
	/* --- cacheline 8 boundary (512 bytes) --- */
	struct prev_cputime        prev_cputime;         /*   512    24 */

	/* XXX last struct has 4 bytes of padding */

	long unsigned int          nvcsw;                /*   536     8 */
	long unsigned int          nivcsw;               /*   544     8 */
	long unsigned int          cnvcsw;               /*   552     8 */
	long unsigned int          cnivcsw;              /*   560     8 */
	long unsigned int          min_flt;              /*   568     8 */
	/* --- cacheline 9 boundary (576 bytes) --- */
	long unsigned int          maj_flt;              /*   576     8 */
	long unsigned int          cmin_flt;             /*   584     8 */
	long unsigned int          cmaj_flt;             /*   592     8 */
	long unsigned int          inblock;              /*   600     8 */
	long unsigned int          oublock;              /*   608     8 */
	long unsigned int          cinblock;             /*   616     8 */
	long unsigned int          coublock;             /*   624     8 */
	long unsigned int          maxrss;               /*   632     8 */
	/* --- cacheline 10 boundary (640 bytes) --- */
	long unsigned int          cmaxrss;              /*   640     8 */
	struct task_io_accounting  ioac;                 /*   648    56 */
	/* --- cacheline 11 boundary (704 bytes) --- */
	long long unsigned int     sum_sched_runtime;    /*   704     8 */
	struct rlimit              rlim[16];             /*   712   256 */
	/* --- cacheline 15 boundary (960 bytes) was 8 bytes ago --- */
	struct pacct_struct        pacct;                /*   968    56 */
	/* --- cacheline 16 boundary (1024 bytes) --- */
	struct taskstats *         stats;                /*  1024     8 */
	unsigned int               audit_tty;            /*  1032     4 */

	/* XXX 4 bytes hole, try to pack */

	struct tty_audit_buf *     tty_audit_buf;        /*  1040     8 */
	bool                       oom_flag_origin;      /*  1048     1 */

	/* XXX 1 byte hole, try to pack */

	short int                  oom_score_adj;        /*  1050     2 */
	short int                  oom_score_adj_min;    /*  1052     2 */

	/* XXX 2 bytes hole, try to pack */

	struct mm_struct *         oom_mm;               /*  1056     8 */
	struct mutex               cred_guard_mutex;     /*  1064    32 */
	/* --- cacheline 17 boundary (1088 bytes) was 8 bytes ago --- */
	struct rw_semaphore        exec_update_lock;     /*  1096    40 */
	u64                        kabi_reserved1;       /*  1136     8 */
	u64                        kabi_reserved2;       /*  1144     8 */
	/* --- cacheline 18 boundary (1152 bytes) --- */
	u64                        kabi_reserved3;       /*  1152     8 */
	u64                        kabi_reserved4;       /*  1160     8 */

	/* size: 1168, cachelines: 19, members: 67 */
	/* sum members: 1145, holes: 6, sum holes: 19 */
	/* sum bitfield members: 2 bits, bit holes: 1, sum bit holes: 30 bits */
	/* paddings: 1, sum paddings: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct rseq {
	__u32                      cpu_id_start;         /*     0     4 */
	__u32                      cpu_id;               /*     4     4 */
	union {
		__u64              ptr64;                /*     8     8 */
		__u64              ptr;                  /*     8     8 */
	} rseq_cs;                                       /*     8     8 */
	__u32                      flags;                /*    16     4 */

	/* size: 32, cachelines: 1, members: 4 */
	/* padding: 12 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(32)));
struct prev_cputime {
	u64                        utime;                /*     0     8 */
	u64                        stime;                /*     8     8 */
	raw_spinlock_t             lock;                 /*    16     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct vtime {
	seqcount_t                 seqcount;             /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	long long unsigned int     starttime;            /*     8     8 */
	enum vtime_state           state;                /*    16     4 */
	unsigned int               cpu;                  /*    20     4 */
	u64                        utime;                /*    24     8 */
	u64                        stime;                /*    32     8 */
	u64                        gtime;                /*    40     8 */

	/* size: 48, cachelines: 1, members: 7 */
	/* sum members: 44, holes: 1, sum holes: 4 */
	/* last cacheline: 48 bytes */
};
struct sched_info {
	long unsigned int          pcount;               /*     0     8 */
	long long unsigned int     run_delay;            /*     8     8 */
	long long unsigned int     last_arrival;         /*    16     8 */
	long long unsigned int     last_queued;          /*    24     8 */
	u64                        kabi_reserved1;       /*    32     8 */
	u64                        kabi_reserved2;       /*    40     8 */
	u64                        kabi_reserved3;       /*    48     8 */
	u64                        kabi_reserved4;       /*    56     8 */

	/* size: 64, cachelines: 1, members: 8 */
};
struct load_weight {
	long unsigned int          weight;               /*     0     8 */
	u32                        inv_weight;           /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct util_est {
	unsigned int               enqueued;             /*     0     4 */
	unsigned int               ewma;                 /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
} __attribute__((__aligned__(8)));
struct sched_avg {
	u64                        last_update_time;     /*     0     8 */
	u64                        load_sum;             /*     8     8 */
	u64                        runnable_sum;         /*    16     8 */
	u32                        util_sum;             /*    24     4 */
	u32                        period_contrib;       /*    28     4 */
	long unsigned int          load_avg;             /*    32     8 */
	long unsigned int          runnable_avg;         /*    40     8 */
	long unsigned int          util_avg;             /*    48     8 */
	struct util_est            util_est __attribute__((__aligned__(8))); /*    56     8 */

	/* size: 64, cachelines: 1, members: 9 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(64)));
struct sched_statistics {
	u64                        wait_start;           /*     0     8 */
	u64                        wait_max;             /*     8     8 */
	u64                        wait_count;           /*    16     8 */
	u64                        wait_sum;             /*    24     8 */
	u64                        iowait_count;         /*    32     8 */
	u64                        iowait_sum;           /*    40     8 */
	u64                        sleep_start;          /*    48     8 */
	u64                        sleep_max;            /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	s64                        sum_sleep_runtime;    /*    64     8 */
	u64                        block_start;          /*    72     8 */
	u64                        block_max;            /*    80     8 */
	u64                        exec_max;             /*    88     8 */
	u64                        slice_max;            /*    96     8 */
	u64                        nr_migrations_cold;   /*   104     8 */
	u64                        nr_failed_migrations_affine; /*   112     8 */
	u64                        nr_failed_migrations_running; /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	u64                        nr_failed_migrations_hot; /*   128     8 */
	u64                        nr_forced_migrations; /*   136     8 */
	u64                        nr_wakeups;           /*   144     8 */
	u64                        nr_wakeups_sync;      /*   152     8 */
	u64                        nr_wakeups_migrate;   /*   160     8 */
	u64                        nr_wakeups_local;     /*   168     8 */
	u64                        nr_wakeups_remote;    /*   176     8 */
	u64                        nr_wakeups_affine;    /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	u64                        nr_wakeups_affine_attempts; /*   192     8 */
	u64                        nr_wakeups_passive;   /*   200     8 */
	u64                        nr_wakeups_idle;      /*   208     8 */
	u64                        nr_qos_smt_send_ipi;  /*   216     8 */
	u64                        nr_qos_smt_expelled;  /*   224     8 */
	u64                        kabi_reserved3;       /*   232     8 */
	u64                        kabi_reserved4;       /*   240     8 */

	/* size: 248, cachelines: 4, members: 31 */
	/* last cacheline: 56 bytes */
};
struct sched_entity {
	struct load_weight         load;                 /*     0    16 */

	/* XXX last struct has 4 bytes of padding */

	struct rb_node             run_node __attribute__((__aligned__(8))); /*    16    24 */
	struct list_head           group_node;           /*    40    16 */
	unsigned int               on_rq;                /*    56     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	u64                        exec_start;           /*    64     8 */
	u64                        sum_exec_runtime;     /*    72     8 */
	u64                        vruntime;             /*    80     8 */
	u64                        prev_sum_exec_runtime; /*    88     8 */
	u64                        nr_migrations;        /*    96     8 */
	struct sched_statistics    statistics;           /*   104   248 */
	/* --- cacheline 5 boundary (320 bytes) was 32 bytes ago --- */
	int                        depth;                /*   352     4 */

	/* XXX 4 bytes hole, try to pack */

	struct sched_entity *      parent;               /*   360     8 */
	struct cfs_rq *            cfs_rq;               /*   368     8 */
	struct cfs_rq *            my_q;                 /*   376     8 */
	/* --- cacheline 6 boundary (384 bytes) --- */
	long unsigned int          runnable_weight;      /*   384     8 */

	/* XXX 56 bytes hole, try to pack */

	/* --- cacheline 7 boundary (448 bytes) --- */
	struct sched_avg           avg __attribute__((__aligned__(64))); /*   448    64 */
	/* --- cacheline 8 boundary (512 bytes) --- */
	u64                        kabi_reserved1;       /*   512     8 */
	u64                        kabi_reserved2;       /*   520     8 */
	u64                        kabi_reserved3;       /*   528     8 */
	u64                        kabi_reserved4;       /*   536     8 */

	/* size: 576, cachelines: 9, members: 20 */
	/* sum members: 480, holes: 3, sum holes: 64 */
	/* padding: 32 */
	/* paddings: 1, sum paddings: 4 */
	/* forced alignments: 2, forced holes: 1, sum forced holes: 56 */
} __attribute__((__aligned__(64)));
struct sched_rt_entity {
	struct list_head           run_list;             /*     0    16 */
	long unsigned int          timeout;              /*    16     8 */
	long unsigned int          watchdog_stamp;       /*    24     8 */
	unsigned int               time_slice;           /*    32     4 */
	short unsigned int         on_rq;                /*    36     2 */
	short unsigned int         on_list;              /*    38     2 */
	struct sched_rt_entity *   back;                 /*    40     8 */
	struct sched_rt_entity *   parent;               /*    48     8 */
	struct rt_rq *             rt_rq;                /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct rt_rq *             my_q;                 /*    64     8 */
	u64                        kabi_reserved1;       /*    72     8 */
	u64                        kabi_reserved2;       /*    80     8 */

	/* size: 88, cachelines: 2, members: 12 */
	/* last cacheline: 24 bytes */
};
struct sched_dl_entity {
	struct rb_node             rb_node __attribute__((__aligned__(8))); /*     0    24 */
	u64                        dl_runtime;           /*    24     8 */
	u64                        dl_deadline;          /*    32     8 */
	u64                        dl_period;            /*    40     8 */
	u64                        dl_bw;                /*    48     8 */
	u64                        dl_density;           /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	s64                        runtime;              /*    64     8 */
	u64                        deadline;             /*    72     8 */
	unsigned int               flags;                /*    80     4 */
	unsigned int               dl_throttled:1;       /*    84: 0  4 */
	unsigned int               dl_yielded:1;         /*    84: 1  4 */
	unsigned int               dl_non_contending:1;  /*    84: 2  4 */
	unsigned int               dl_overrun:1;         /*    84: 3  4 */

	/* XXX 28 bits hole, try to pack */

	struct hrtimer             dl_timer __attribute__((__aligned__(8))); /*    88    96 */
	/* --- cacheline 2 boundary (128 bytes) was 56 bytes ago --- */
	struct hrtimer             inactive_timer __attribute__((__aligned__(8))); /*   184    96 */
	/* --- cacheline 4 boundary (256 bytes) was 24 bytes ago --- */
	struct sched_dl_entity *   pi_se;                /*   280     8 */
	u64                        kabi_reserved1;       /*   288     8 */
	u64                        kabi_reserved2;       /*   296     8 */

	/* size: 304, cachelines: 5, members: 18 */
	/* sum members: 300 */
	/* sum bitfield members: 4 bits, bit holes: 1, sum bit holes: 28 bits */
	/* forced alignments: 3 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
union rcu_special {
	struct {
		u8                 blocked;            /*     0     1 */
		u8                 need_qs;            /*     1     1 */
		u8                 exp_hint;           /*     2     1 */
		u8                 need_mb;            /*     3     1 */
	} b;                                           /*     0     4 */
	u32                        s;                  /*     0     4 */
};
struct wake_q_node {
	struct wake_q_node *       next;                 /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct task_struct_resvd {
	struct task_struct *       task;                 /*     0     8 */
	struct timer_list          oom_reaper_timer;     /*     8    72 */

	/* size: 80, cachelines: 2, members: 2 */
	/* last cacheline: 16 bytes */
};
struct cred {
	atomic_t                   usage;                /*     0     4 */
	kuid_t                     uid;                  /*     4     4 */
	kgid_t                     gid;                  /*     8     4 */
	kuid_t                     suid;                 /*    12     4 */
	kgid_t                     sgid;                 /*    16     4 */
	kuid_t                     euid;                 /*    20     4 */
	kgid_t                     egid;                 /*    24     4 */
	kuid_t                     fsuid;                /*    28     4 */
	kgid_t                     fsgid;                /*    32     4 */
	unsigned int               securebits;           /*    36     4 */
	kernel_cap_t               cap_inheritable;      /*    40     8 */
	kernel_cap_t               cap_permitted;        /*    48     8 */
	kernel_cap_t               cap_effective;        /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	kernel_cap_t               cap_bset;             /*    64     8 */
	kernel_cap_t               cap_ambient;          /*    72     8 */
	unsigned char              jit_keyring;          /*    80     1 */

	/* XXX 7 bytes hole, try to pack */

	struct key *               session_keyring;      /*    88     8 */
	struct key *               process_keyring;      /*    96     8 */
	struct key *               thread_keyring;       /*   104     8 */
	struct key *               request_key_auth;     /*   112     8 */
	void *                     security;             /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct user_struct *       user;                 /*   128     8 */
	struct user_namespace *    user_ns;              /*   136     8 */
	struct group_info *        group_info;           /*   144     8 */
	union {
		int                non_rcu;              /*   152     4 */
		struct callback_head rcu __attribute__((__aligned__(8))); /*   152    16 */
	} __attribute__((__aligned__(8)));               /*   152    16 */
	u64                        kabi_reserved1;       /*   168     8 */
	u64                        kabi_reserved2;       /*   176     8 */
	u64                        kabi_reserved3;       /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	u64                        kabi_reserved4;       /*   192     8 */
	u64                        kabi_reserved5;       /*   200     8 */
	u64                        kabi_reserved6;       /*   208     8 */
	u64                        kabi_reserved7;       /*   216     8 */
	u64                        kabi_reserved8;       /*   224     8 */
	u64                        kabi_reserved9;       /*   232     8 */
	u64                        kabi_reserved10;      /*   240     8 */

	/* size: 248, cachelines: 4, members: 35 */
	/* sum members: 241, holes: 1, sum holes: 7 */
	/* forced alignments: 1 */
	/* last cacheline: 56 bytes */
} __attribute__((__aligned__(8)));
struct key {
	refcount_t                 usage;                /*     0     4 */
	key_serial_t               serial;               /*     4     4 */
	union {
		struct list_head   graveyard_link;       /*     8    16 */
		struct rb_node     serial_node __attribute__((__aligned__(8))); /*     8    24 */
	} __attribute__((__aligned__(8)));               /*     8    24 */
	struct rw_semaphore        sem;                  /*    32    40 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	struct key_user *          user;                 /*    72     8 */
	void *                     security;             /*    80     8 */
	union {
		time64_t           expiry;               /*    88     8 */
		time64_t           revoked_at;           /*    88     8 */
	};                                               /*    88     8 */
	time64_t                   last_used_at;         /*    96     8 */
	kuid_t                     uid;                  /*   104     4 */
	kgid_t                     gid;                  /*   108     4 */
	key_perm_t                 perm;                 /*   112     4 */
	short unsigned int         quotalen;             /*   116     2 */
	short unsigned int         datalen;              /*   118     2 */
	short int                  state;                /*   120     2 */

	/* XXX 6 bytes hole, try to pack */

	/* --- cacheline 2 boundary (128 bytes) --- */
	long unsigned int          flags;                /*   128     8 */
	union {
		struct keyring_index_key index_key;      /*   136    40 */
		struct {
			long unsigned int hash;          /*   136     8 */
			long unsigned int len_desc;      /*   144     8 */
			struct key_type * type;          /*   152     8 */
			struct key_tag * domain_tag;     /*   160     8 */
			char *     description;          /*   168     8 */
		};                                       /*   136    40 */
	};                                               /*   136    40 */
	union {
		union key_payload  payload;              /*   176    32 */
		struct {
			struct list_head name_link;      /*   176    16 */
			/* --- cacheline 3 boundary (192 bytes) --- */
			struct assoc_array keys;         /*   192    16 */
		};                                       /*   176    32 */
	};                                               /*   176    32 */
	/* --- cacheline 3 boundary (192 bytes) was 16 bytes ago --- */
	struct key_restriction *   restrict_link;        /*   208     8 */
	u64                        kabi_reserved1;       /*   216     8 */

	/* size: 224, cachelines: 4, members: 19 */
	/* sum members: 218, holes: 1, sum holes: 6 */
	/* forced alignments: 1 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct sighand_struct {
	spinlock_t                 siglock;              /*     0     4 */
	refcount_t                 count;                /*     4     4 */
	wait_queue_head_t          signalfd_wqh;         /*     8    24 */
	struct k_sigaction         action[64];           /*    32  2048 */

	/* size: 2080, cachelines: 33, members: 4 */
	/* last cacheline: 32 bytes */
};
struct io_context {
	atomic_long_t              refcount;             /*     0     8 */
	atomic_t                   active_ref;           /*     8     4 */
	atomic_t                   nr_tasks;             /*    12     4 */
	spinlock_t                 lock;                 /*    16     4 */
	short unsigned int         ioprio;               /*    20     2 */

	/* XXX 2 bytes hole, try to pack */

	struct xarray              icq_tree;             /*    24    16 */
	struct io_cq *             icq_hint;             /*    40     8 */
	struct hlist_head          icq_list;             /*    48     8 */
	struct work_struct         release_work;         /*    56    64 */

	/* size: 120, cachelines: 2, members: 9 */
	/* sum members: 118, holes: 1, sum holes: 2 */
	/* last cacheline: 56 bytes */
};
struct hlist_bl_head {
	struct hlist_bl_node *     first;                /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct hlist_bl_node {
	struct hlist_bl_node *     next;                 /*     0     8 */
	struct hlist_bl_node * *   pprev;                /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct lockref {
	union {
		__u64              lock_count __attribute__((__aligned__(8))); /*     0     8 */
		struct {
			spinlock_t lock;                 /*     0     4 */
			int        count;                /*     4     4 */
		};                                       /*     0     8 */
	} __attribute__((__aligned__(8)));               /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* forced alignments: 1 */
	/* last cacheline: 8 bytes */
} __attribute__((__aligned__(8)));
struct qstr {
	union {
		struct {
			u32        hash;                 /*     0     4 */
			u32        len;                  /*     4     4 */
		};                                       /*     0     8 */
		u64                hash_len;             /*     0     8 */
	};                                               /*     0     8 */
	const unsigned char  *     name;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct dentry {
	unsigned int               d_flags;              /*     0     4 */
	seqcount_spinlock_t        d_seq;                /*     4     4 */
	struct hlist_bl_node       d_hash;               /*     8    16 */
	struct dentry *            d_parent;             /*    24     8 */
	struct qstr                d_name;               /*    32    16 */
	struct inode *             d_inode;              /*    48     8 */
	unsigned char              d_iname[32];          /*    56    32 */
	/* --- cacheline 1 boundary (64 bytes) was 24 bytes ago --- */
	struct lockref             d_lockref __attribute__((__aligned__(8))); /*    88     8 */
	const struct dentry_operations  * d_op;          /*    96     8 */
	struct super_block *       d_sb;                 /*   104     8 */
	long unsigned int          d_time;               /*   112     8 */
	void *                     d_fsdata;             /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	union {
		struct list_head   d_lru;                /*   128    16 */
		wait_queue_head_t * d_wait;              /*   128     8 */
	};                                               /*   128    16 */
	struct list_head           d_child;              /*   144    16 */
	struct list_head           d_subdirs;            /*   160    16 */
	union {
		struct hlist_node  d_alias;              /*   176    16 */
		struct hlist_bl_node d_in_lookup_hash;   /*   176    16 */
		struct callback_head d_rcu __attribute__((__aligned__(8))); /*   176    16 */
	} d_u __attribute__((__aligned__(8)));           /*   176    16 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	atomic_t                   d_neg_dnum;           /*   192     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        kabi_reserved1;       /*   200     8 */
	u64                        kabi_reserved2;       /*   208     8 */

	/* size: 216, cachelines: 4, members: 19 */
	/* sum members: 212, holes: 1, sum holes: 4 */
	/* forced alignments: 2 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct inode {
	umode_t                    i_mode;               /*     0     2 */
	short unsigned int         i_opflags;            /*     2     2 */
	kuid_t                     i_uid;                /*     4     4 */
	kgid_t                     i_gid;                /*     8     4 */
	unsigned int               i_flags;              /*    12     4 */
	struct posix_acl *         i_acl;                /*    16     8 */
	struct posix_acl *         i_default_acl;        /*    24     8 */
	const struct inode_operations  * i_op;           /*    32     8 */
	struct super_block *       i_sb;                 /*    40     8 */
	struct address_space *     i_mapping;            /*    48     8 */
	void *                     i_security;           /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	long unsigned int          i_ino;                /*    64     8 */
	union {
		const unsigned int i_nlink;              /*    72     4 */
		unsigned int       __i_nlink;            /*    72     4 */
	};                                               /*    72     4 */
	dev_t                      i_rdev;               /*    76     4 */
	loff_t                     i_size;               /*    80     8 */
	struct timespec64          i_atime;              /*    88    16 */
	struct timespec64          i_mtime;              /*   104    16 */
	struct timespec64          i_ctime;              /*   120    16 */
	/* --- cacheline 2 boundary (128 bytes) was 8 bytes ago --- */
	spinlock_t                 i_lock;               /*   136     4 */
	short unsigned int         i_bytes;              /*   140     2 */
	u8                         i_blkbits;            /*   142     1 */
	u8                         i_write_hint;         /*   143     1 */
	blkcnt_t                   i_blocks;             /*   144     8 */
	long unsigned int          i_state;              /*   152     8 */
	struct rw_semaphore        i_rwsem;              /*   160    40 */
	/* --- cacheline 3 boundary (192 bytes) was 8 bytes ago --- */
	long unsigned int          dirtied_when;         /*   200     8 */
	long unsigned int          dirtied_time_when;    /*   208     8 */
	struct hlist_node          i_hash;               /*   216    16 */
	struct list_head           i_io_list;            /*   232    16 */
	struct bdi_writeback *     i_wb;                 /*   248     8 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	int                        i_wb_frn_winner;      /*   256     4 */
	u16                        i_wb_frn_avg_time;    /*   260     2 */
	u16                        i_wb_frn_history;     /*   262     2 */
	struct list_head           i_lru;                /*   264    16 */
	struct list_head           i_sb_list;            /*   280    16 */
	struct list_head           i_wb_list;            /*   296    16 */
	union {
		struct hlist_head  i_dentry;             /*   312     8 */
		struct callback_head i_rcu __attribute__((__aligned__(8))); /*   312    16 */
	} __attribute__((__aligned__(8)));               /*   312    16 */
	/* --- cacheline 5 boundary (320 bytes) was 8 bytes ago --- */
	atomic64_t                 i_version;            /*   328     8 */
	atomic64_t                 i_sequence;           /*   336     8 */
	atomic_t                   i_count;              /*   344     4 */
	atomic_t                   i_dio_count;          /*   348     4 */
	atomic_t                   i_writecount;         /*   352     4 */
	atomic_t                   i_readcount;          /*   356     4 */
	union {
		const struct file_operations  * i_fop;   /*   360     8 */
		void               (*free_inode)(struct inode *); /*   360     8 */
	};                                               /*   360     8 */
	struct file_lock_context * i_flctx;              /*   368     8 */
	struct address_space       i_data __attribute__((__aligned__(8))); /*   376   184 */
	/* --- cacheline 8 boundary (512 bytes) was 48 bytes ago --- */
	struct list_head           i_devices;            /*   560    16 */
	/* --- cacheline 9 boundary (576 bytes) --- */
	union {
		struct pipe_inode_info * i_pipe;         /*   576     8 */
		struct block_device * i_bdev;            /*   576     8 */
		struct cdev *      i_cdev;               /*   576     8 */
		char *             i_link;               /*   576     8 */
		unsigned int       i_dir_seq;            /*   576     4 */
	};                                               /*   576     8 */
	__u32                      i_generation;         /*   584     4 */
	__u32                      i_fsnotify_mask;      /*   588     4 */
	struct fsnotify_mark_connector * i_fsnotify_marks; /*   592     8 */
	void *                     i_private;            /*   600     8 */
	u64                        kabi_reserved1;       /*   608     8 */
	u64                        kabi_reserved2;       /*   616     8 */

	/* size: 624, cachelines: 10, members: 54 */
	/* forced alignments: 2 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct dentry_operations {
	int                        (*d_revalidate)(struct dentry *, unsigned int); /*     0     8 */
	int                        (*d_weak_revalidate)(struct dentry *, unsigned int); /*     8     8 */
	int                        (*d_hash)(const struct dentry  *, struct qstr *); /*    16     8 */
	int                        (*d_compare)(const struct dentry  *, unsigned int, const char  *, const struct qstr  *); /*    24     8 */
	int                        (*d_delete)(const struct dentry  *); /*    32     8 */
	int                        (*d_init)(struct dentry *); /*    40     8 */
	void                       (*d_release)(struct dentry *); /*    48     8 */
	void                       (*d_prune)(struct dentry *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	void                       (*d_iput)(struct dentry *, struct inode *); /*    64     8 */
	char *                     (*d_dname)(struct dentry *, char *, int); /*    72     8 */
	struct vfsmount *          (*d_automount)(struct path *); /*    80     8 */
	int                        (*d_manage)(const struct path  *, bool); /*    88     8 */
	struct dentry *            (*d_real)(struct dentry *, const struct inode  *); /*    96     8 */
	u64                        kabi_reserved1;       /*   104     8 */
	u64                        kabi_reserved2;       /*   112     8 */
	u64                        kabi_reserved3;       /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	u64                        kabi_reserved4;       /*   128     8 */

	/* size: 192, cachelines: 3, members: 17 */
	/* padding: 56 */
} __attribute__((__aligned__(64)));
struct super_block {
	struct list_head           s_list;               /*     0    16 */
	dev_t                      s_dev;                /*    16     4 */
	unsigned char              s_blocksize_bits;     /*    20     1 */

	/* XXX 3 bytes hole, try to pack */

	long unsigned int          s_blocksize;          /*    24     8 */
	loff_t                     s_maxbytes;           /*    32     8 */
	struct file_system_type *  s_type;               /*    40     8 */
	const struct super_operations  * s_op;           /*    48     8 */
	const struct dquot_operations  * dq_op;          /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	const struct quotactl_ops  * s_qcop;             /*    64     8 */
	const struct export_operations  * s_export_op;   /*    72     8 */
	long unsigned int          s_flags;              /*    80     8 */
	long unsigned int          s_iflags;             /*    88     8 */
	long unsigned int          s_magic;              /*    96     8 */
	struct dentry *            s_root;               /*   104     8 */
	struct rw_semaphore        s_umount;             /*   112    40 */
	/* --- cacheline 2 boundary (128 bytes) was 24 bytes ago --- */
	int                        s_count;              /*   152     4 */
	atomic_t                   s_active;             /*   156     4 */
	void *                     s_security;           /*   160     8 */
	const struct xattr_handler  * * s_xattr;         /*   168     8 */
	struct hlist_bl_head       s_roots;              /*   176     8 */
	struct list_head           s_mounts;             /*   184    16 */
	/* --- cacheline 3 boundary (192 bytes) was 8 bytes ago --- */
	struct block_device *      s_bdev;               /*   200     8 */
	struct backing_dev_info *  s_bdi;                /*   208     8 */
	struct mtd_info *          s_mtd;                /*   216     8 */
	struct hlist_node          s_instances;          /*   224    16 */
	unsigned int               s_quota_types;        /*   240     4 */

	/* XXX 4 bytes hole, try to pack */

	struct quota_info          s_dquot;              /*   248   312 */
	/* --- cacheline 8 boundary (512 bytes) was 48 bytes ago --- */
	struct sb_writers          s_writers __attribute__((__aligned__(8))); /*   560   320 */
	/* --- cacheline 13 boundary (832 bytes) was 48 bytes ago --- */
	void *                     s_fs_info;            /*   880     8 */
	u32                        s_time_gran;          /*   888     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 14 boundary (896 bytes) --- */
	time64_t                   s_time_min;           /*   896     8 */
	time64_t                   s_time_max;           /*   904     8 */
	__u32                      s_fsnotify_mask;      /*   912     4 */

	/* XXX 4 bytes hole, try to pack */

	struct fsnotify_mark_connector * s_fsnotify_marks; /*   920     8 */
	char                       s_id[32];             /*   928    32 */
	/* --- cacheline 15 boundary (960 bytes) --- */
	uuid_t                     s_uuid;               /*   960    16 */
	unsigned int               s_max_links;          /*   976     4 */
	fmode_t                    s_mode;               /*   980     4 */
	struct mutex               s_vfs_rename_mutex;   /*   984    32 */
	const char  *              s_subtype;            /*  1016     8 */
	/* --- cacheline 16 boundary (1024 bytes) --- */
	const struct dentry_operations  * s_d_op;        /*  1024     8 */
	int                        cleancache_poolid;    /*  1032     4 */

	/* XXX 4 bytes hole, try to pack */

	struct shrinker            s_shrink;             /*  1040    64 */
	/* --- cacheline 17 boundary (1088 bytes) was 16 bytes ago --- */
	atomic_long_t              s_remove_count;       /*  1104     8 */
	atomic_long_t              s_fsnotify_inode_refs; /*  1112     8 */
	int                        s_readonly_remount;   /*  1120     4 */
	errseq_t                   s_wb_err;             /*  1124     4 */
	struct workqueue_struct *  s_dio_done_wq;        /*  1128     8 */
	struct hlist_head          s_pins;               /*  1136     8 */
	struct user_namespace *    s_user_ns;            /*  1144     8 */
	/* --- cacheline 18 boundary (1152 bytes) --- */
	struct list_lru            s_dentry_lru;         /*  1152    32 */

	/* XXX last struct has 3 bytes of padding */

	struct list_lru            s_inode_lru;          /*  1184    32 */

	/* XXX last struct has 3 bytes of padding */

	/* --- cacheline 19 boundary (1216 bytes) --- */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*  1216    16 */
	struct work_struct         destroy_work;         /*  1232    64 */
	/* --- cacheline 20 boundary (1280 bytes) was 16 bytes ago --- */
	struct mutex               s_sync_lock;          /*  1296    32 */
	int                        s_stack_depth;        /*  1328     4 */

	/* XXX 12 bytes hole, try to pack */

	/* --- cacheline 21 boundary (1344 bytes) --- */
	spinlock_t                 s_inode_list_lock __attribute__((__aligned__(64))); /*  1344     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           s_inodes;             /*  1352    16 */
	spinlock_t                 s_inode_wblist_lock;  /*  1368     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           s_inodes_wb;          /*  1376    16 */

	/* size: 1408, cachelines: 22, members: 60 */
	/* sum members: 1353, holes: 8, sum holes: 39 */
	/* padding: 16 */
	/* paddings: 2, sum paddings: 6 */
	/* forced alignments: 3, forced holes: 1, sum forced holes: 12 */
} __attribute__((__aligned__(64)));
struct vfsmount {
	struct dentry *            mnt_root;             /*     0     8 */
	struct super_block *       mnt_sb;               /*     8     8 */
	int                        mnt_flags;            /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        kabi_reserved1;       /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct path {
	struct vfsmount *          mnt;                  /*     0     8 */
	struct dentry *            dentry;               /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct kstat {
	u32                        result_mask;          /*     0     4 */
	umode_t                    mode;                 /*     4     2 */

	/* XXX 2 bytes hole, try to pack */

	unsigned int               nlink;                /*     8     4 */
	uint32_t                   blksize;              /*    12     4 */
	u64                        attributes;           /*    16     8 */
	u64                        attributes_mask;      /*    24     8 */
	u64                        ino;                  /*    32     8 */
	dev_t                      dev;                  /*    40     4 */
	dev_t                      rdev;                 /*    44     4 */
	kuid_t                     uid;                  /*    48     4 */
	kgid_t                     gid;                  /*    52     4 */
	loff_t                     size;                 /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct timespec64          atime;                /*    64    16 */
	struct timespec64          mtime;                /*    80    16 */
	struct timespec64          ctime;                /*    96    16 */
	struct timespec64          btime;                /*   112    16 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	u64                        blocks;               /*   128     8 */
	u64                        mnt_id;               /*   136     8 */

	/* size: 144, cachelines: 3, members: 18 */
	/* sum members: 142, holes: 1, sum holes: 2 */
	/* last cacheline: 16 bytes */
};
struct list_lru_one {
	struct list_head           list;                 /*     0    16 */
	long int                   nr_items;             /*    16     8 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct list_lru_memcg {
	struct callback_head       rcu __attribute__((__aligned__(8))); /*     0    16 */
	struct list_lru_one *      lru[];                /*    16     0 */

	/* size: 16, cachelines: 1, members: 2 */
	/* forced alignments: 1 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct list_lru_node {
	spinlock_t                 lock;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_lru_one        lru;                  /*     8    24 */
	struct list_lru_memcg *    memcg_lrus;           /*    32     8 */
	long int                   nr_items;             /*    40     8 */

	/* size: 64, cachelines: 1, members: 4 */
	/* sum members: 44, holes: 1, sum holes: 4 */
	/* padding: 16 */
} __attribute__((__aligned__(64)));
struct list_lru {
	struct list_lru_node *     node;                 /*     0     8 */
	struct list_head           list;                 /*     8    16 */
	int                        shrinker_id;          /*    24     4 */
	bool                       memcg_aware;          /*    28     1 */

	/* size: 32, cachelines: 1, members: 4 */
	/* padding: 3 */
	/* last cacheline: 32 bytes */
};
struct xarray {
	spinlock_t                 xa_lock;              /*     0     4 */
	gfp_t                      xa_flags;             /*     4     4 */
	void *                     xa_head;              /*     8     8 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct kernel_cap_struct {
	__u32                      cap[2];               /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct assoc_array {
	struct assoc_array_ptr *   root;                 /*     0     8 */
	long unsigned int          nr_leaves_on_tree;    /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct key_tag {
	struct callback_head       rcu __attribute__((__aligned__(8))); /*     0    16 */
	refcount_t                 usage;                /*    16     4 */
	bool                       removed;              /*    20     1 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 3 */
	/* forced alignments: 1 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct keyring_index_key {
	long unsigned int          hash;                 /*     0     8 */
	union {
		struct {
			u16        desc_len;             /*     8     2 */
			char       desc[6];              /*    10     6 */
		};                                       /*     8     8 */
		long unsigned int  x;                    /*     8     8 */
	};                                               /*     8     8 */
	struct key_type *          type;                 /*    16     8 */
	struct key_tag *           domain_tag;           /*    24     8 */
	const char  *              description;          /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
union key_payload {
	void *                     rcu_data0;          /*     0     8 */
	void *                     data[4];            /*     0    32 */
};
struct key_restriction {
	key_restrict_link_func_t   check;                /*     0     8 */
	struct key *               key;                  /*     8     8 */
	struct key_type *          keytype;              /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct group_info {
	atomic_t                   usage;                /*     0     4 */
	int                        ngroups;              /*     4     4 */
	kgid_t                     gid[];                /*     8     0 */

	/* size: 8, cachelines: 1, members: 3 */
	/* last cacheline: 8 bytes */
};
struct pacct_struct {
	int                        ac_flag;              /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	long int                   ac_exitcode;          /*     8     8 */
	long unsigned int          ac_mem;               /*    16     8 */
	u64                        ac_utime;             /*    24     8 */
	u64                        ac_stime;             /*    32     8 */
	long unsigned int          ac_minflt;            /*    40     8 */
	long unsigned int          ac_majflt;            /*    48     8 */

	/* size: 56, cachelines: 1, members: 7 */
	/* sum members: 52, holes: 1, sum holes: 4 */
	/* last cacheline: 56 bytes */
};
struct cpu_itimer {
	u64                        expires;              /*     0     8 */
	u64                        incr;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct task_cputime_atomic {
	atomic64_t                 utime;                /*     0     8 */
	atomic64_t                 stime;                /*     8     8 */
	atomic64_t                 sum_exec_runtime;     /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct thread_group_cputimer {
	struct task_cputime_atomic cputime_atomic;       /*     0    24 */

	/* size: 24, cachelines: 1, members: 1 */
	/* last cacheline: 24 bytes */
};
struct rcuwait {
	struct task_struct *       task;                 /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct rcu_sync {
	int                        gp_state;             /*     0     4 */
	int                        gp_count;             /*     4     4 */
	wait_queue_head_t          gp_wait;              /*     8    24 */
	struct callback_head       cb_head __attribute__((__aligned__(8))); /*    32    16 */

	/* size: 48, cachelines: 1, members: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct percpu_rw_semaphore {
	struct rcu_sync            rss __attribute__((__aligned__(8))); /*     0    48 */
	unsigned int *             read_count;           /*    48     8 */
	struct rcuwait             writer;               /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	wait_queue_head_t          waiters;              /*    64    24 */
	atomic_t                   block;                /*    88     4 */

	/* size: 96, cachelines: 2, members: 5 */
	/* padding: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct delayed_call {
	void                       (*fn)(void *);        /*     0     8 */
	void *                     arg;                  /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct io_cq {
	struct request_queue *     q;                    /*     0     8 */
	struct io_context *        ioc;                  /*     8     8 */
	union {
		struct list_head   q_node;               /*    16    16 */
		struct kmem_cache * __rcu_icq_cache;     /*    16     8 */
	};                                               /*    16    16 */
	union {
		struct hlist_node  ioc_node;             /*    32    16 */
		struct callback_head __rcu_head __attribute__((__aligned__(8))); /*    32    16 */
	} __attribute__((__aligned__(8)));               /*    32    16 */
	unsigned int               flags;                /*    48     4 */

	/* size: 56, cachelines: 1, members: 5 */
	/* padding: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 56 bytes */
} __attribute__((__aligned__(8)));
struct kiocb {
	struct file *              ki_filp;              /*     0     8 */
	loff_t                     ki_pos;               /*     8     8 */
	void                       (*ki_complete)(struct kiocb *, long int, long int); /*    16     8 */
	void *                     private;              /*    24     8 */
	int                        ki_flags;             /*    32     4 */
	u16                        ki_hint;              /*    36     2 */
	u16                        ki_ioprio;            /*    38     2 */
	union {
		unsigned int       ki_cookie;            /*    40     4 */
		struct wait_page_queue * ki_waitq;       /*    40     8 */
	};                                               /*    40     8 */

	/* size: 48, cachelines: 1, members: 8 */
	/* last cacheline: 48 bytes */
};
struct iattr {
	unsigned int               ia_valid;             /*     0     4 */
	umode_t                    ia_mode;              /*     4     2 */

	/* XXX 2 bytes hole, try to pack */

	kuid_t                     ia_uid;               /*     8     4 */
	kgid_t                     ia_gid;               /*    12     4 */
	loff_t                     ia_size;              /*    16     8 */
	struct timespec64          ia_atime;             /*    24    16 */
	struct timespec64          ia_mtime;             /*    40    16 */
	struct timespec64          ia_ctime;             /*    56    16 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	struct file *              ia_file;              /*    72     8 */

	/* size: 80, cachelines: 2, members: 9 */
	/* sum members: 78, holes: 1, sum holes: 2 */
	/* last cacheline: 16 bytes */
};
struct dquot {
	struct hlist_node          dq_hash;              /*     0    16 */
	struct list_head           dq_inuse;             /*    16    16 */
	struct list_head           dq_free;              /*    32    16 */
	struct list_head           dq_dirty;             /*    48    16 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct mutex               dq_lock;              /*    64    32 */
	spinlock_t                 dq_dqb_lock;          /*    96     4 */
	atomic_t                   dq_count;             /*   100     4 */
	struct super_block *       dq_sb;                /*   104     8 */
	struct kqid                dq_id;                /*   112     8 */
	loff_t                     dq_off;               /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	long unsigned int          dq_flags;             /*   128     8 */
	struct mem_dqblk           dq_dqb;               /*   136    72 */

	/* size: 208, cachelines: 4, members: 12 */
	/* last cacheline: 16 bytes */
};
struct kqid {
	union {
		kuid_t             uid;                  /*     0     4 */
		kgid_t             gid;                  /*     0     4 */
		kprojid_t          projid;               /*     0     4 */
	};                                               /*     0     4 */
	enum quota_type            type;                 /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct mem_dqblk {
	qsize_t                    dqb_bhardlimit;       /*     0     8 */
	qsize_t                    dqb_bsoftlimit;       /*     8     8 */
	qsize_t                    dqb_curspace;         /*    16     8 */
	qsize_t                    dqb_rsvspace;         /*    24     8 */
	qsize_t                    dqb_ihardlimit;       /*    32     8 */
	qsize_t                    dqb_isoftlimit;       /*    40     8 */
	qsize_t                    dqb_curinodes;        /*    48     8 */
	time64_t                   dqb_btime;            /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	time64_t                   dqb_itime;            /*    64     8 */

	/* size: 72, cachelines: 2, members: 9 */
	/* last cacheline: 8 bytes */
};
struct mem_dqinfo {
	struct quota_format_type * dqi_format;           /*     0     8 */
	int                        dqi_fmt_id;           /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           dqi_dirty_list;       /*    16    16 */
	long unsigned int          dqi_flags;            /*    32     8 */
	unsigned int               dqi_bgrace;           /*    40     4 */
	unsigned int               dqi_igrace;           /*    44     4 */
	qsize_t                    dqi_max_spc_limit;    /*    48     8 */
	qsize_t                    dqi_max_ino_limit;    /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	void *                     dqi_priv;             /*    64     8 */

	/* size: 72, cachelines: 2, members: 9 */
	/* sum members: 68, holes: 1, sum holes: 4 */
	/* last cacheline: 8 bytes */
};
struct quota_format_type {
	int                        qf_fmt_id;            /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	const struct quota_format_ops  * qf_ops;         /*     8     8 */
	struct module *            qf_owner;             /*    16     8 */
	struct quota_format_type * qf_next;              /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct quota_format_ops {
	int                        (*check_quota_file)(struct super_block *, int); /*     0     8 */
	int                        (*read_file_info)(struct super_block *, int); /*     8     8 */
	int                        (*write_file_info)(struct super_block *, int); /*    16     8 */
	int                        (*free_file_info)(struct super_block *, int); /*    24     8 */
	int                        (*read_dqblk)(struct dquot *); /*    32     8 */
	int                        (*commit_dqblk)(struct dquot *); /*    40     8 */
	int                        (*release_dqblk)(struct dquot *); /*    48     8 */
	int                        (*get_next_id)(struct super_block *, struct kqid *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u64                        kabi_reserved1;       /*    64     8 */
	u64                        kabi_reserved2;       /*    72     8 */

	/* size: 80, cachelines: 2, members: 10 */
	/* last cacheline: 16 bytes */
};
struct dquot_operations {
	int                        (*write_dquot)(struct dquot *); /*     0     8 */
	struct dquot *             (*alloc_dquot)(struct super_block *, int); /*     8     8 */
	void                       (*destroy_dquot)(struct dquot *); /*    16     8 */
	int                        (*acquire_dquot)(struct dquot *); /*    24     8 */
	int                        (*release_dquot)(struct dquot *); /*    32     8 */
	int                        (*mark_dirty)(struct dquot *); /*    40     8 */
	int                        (*write_info)(struct super_block *, int); /*    48     8 */
	qsize_t *                  (*get_reserved_space)(struct inode *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*get_projid)(struct inode *, kprojid_t *); /*    64     8 */
	int                        (*get_inode_usage)(struct inode *, qsize_t *); /*    72     8 */
	int                        (*get_next_id)(struct super_block *, struct kqid *); /*    80     8 */
	u64                        kabi_reserved1;       /*    88     8 */
	u64                        kabi_reserved2;       /*    96     8 */

	/* size: 104, cachelines: 2, members: 13 */
	/* last cacheline: 40 bytes */
};
struct qc_dqblk {
	int                        d_fieldmask;          /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        d_spc_hardlimit;      /*     8     8 */
	u64                        d_spc_softlimit;      /*    16     8 */
	u64                        d_ino_hardlimit;      /*    24     8 */
	u64                        d_ino_softlimit;      /*    32     8 */
	u64                        d_space;              /*    40     8 */
	u64                        d_ino_count;          /*    48     8 */
	s64                        d_ino_timer;          /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	s64                        d_spc_timer;          /*    64     8 */
	int                        d_ino_warns;          /*    72     4 */
	int                        d_spc_warns;          /*    76     4 */
	u64                        d_rt_spc_hardlimit;   /*    80     8 */
	u64                        d_rt_spc_softlimit;   /*    88     8 */
	u64                        d_rt_space;           /*    96     8 */
	s64                        d_rt_spc_timer;       /*   104     8 */
	int                        d_rt_spc_warns;       /*   112     4 */

	/* size: 120, cachelines: 2, members: 16 */
	/* sum members: 112, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 56 bytes */
};
struct qc_type_state {
	unsigned int               flags;                /*     0     4 */
	unsigned int               spc_timelimit;        /*     4     4 */
	unsigned int               ino_timelimit;        /*     8     4 */
	unsigned int               rt_spc_timelimit;     /*    12     4 */
	unsigned int               spc_warnlimit;        /*    16     4 */
	unsigned int               ino_warnlimit;        /*    20     4 */
	unsigned int               rt_spc_warnlimit;     /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	long long unsigned int     ino;                  /*    32     8 */
	blkcnt_t                   blocks;               /*    40     8 */
	blkcnt_t                   nextents;             /*    48     8 */

	/* size: 56, cachelines: 1, members: 10 */
	/* sum members: 52, holes: 1, sum holes: 4 */
	/* last cacheline: 56 bytes */
};
struct qc_state {
	unsigned int               s_incoredqs;          /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct qc_type_state       s_state[3];           /*     8   168 */

	/* size: 176, cachelines: 3, members: 2 */
	/* sum members: 172, holes: 1, sum holes: 4 */
	/* last cacheline: 48 bytes */
};
struct qc_info {
	int                        i_fieldmask;          /*     0     4 */
	unsigned int               i_flags;              /*     4     4 */
	unsigned int               i_spc_timelimit;      /*     8     4 */
	unsigned int               i_ino_timelimit;      /*    12     4 */
	unsigned int               i_rt_spc_timelimit;   /*    16     4 */
	unsigned int               i_spc_warnlimit;      /*    20     4 */
	unsigned int               i_ino_warnlimit;      /*    24     4 */
	unsigned int               i_rt_spc_warnlimit;   /*    28     4 */

	/* size: 32, cachelines: 1, members: 8 */
	/* last cacheline: 32 bytes */
};
struct quotactl_ops {
	int                        (*quota_on)(struct super_block *, int, int, const struct path  *); /*     0     8 */
	int                        (*quota_off)(struct super_block *, int); /*     8     8 */
	int                        (*quota_enable)(struct super_block *, unsigned int); /*    16     8 */
	int                        (*quota_disable)(struct super_block *, unsigned int); /*    24     8 */
	int                        (*quota_sync)(struct super_block *, int); /*    32     8 */
	int                        (*set_info)(struct super_block *, int, struct qc_info *); /*    40     8 */
	int                        (*get_dqblk)(struct super_block *, struct kqid, struct qc_dqblk *); /*    48     8 */
	int                        (*get_nextdqblk)(struct super_block *, struct kqid *, struct qc_dqblk *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*set_dqblk)(struct super_block *, struct kqid, struct qc_dqblk *); /*    64     8 */
	int                        (*get_state)(struct super_block *, struct qc_state *); /*    72     8 */
	int                        (*rm_xquota)(struct super_block *, unsigned int); /*    80     8 */

	/* size: 88, cachelines: 2, members: 11 */
	/* last cacheline: 24 bytes */
};
struct quota_info {
	unsigned int               flags;                /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct rw_semaphore        dqio_sem;             /*     8    40 */
	struct inode *             files[3];             /*    48    24 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	struct mem_dqinfo          info[3];              /*    72   216 */
	/* --- cacheline 4 boundary (256 bytes) was 32 bytes ago --- */
	const struct quota_format_ops  * ops[3];         /*   288    24 */

	/* size: 312, cachelines: 5, members: 5 */
	/* sum members: 308, holes: 1, sum holes: 4 */
	/* last cacheline: 56 bytes */
};
struct address_space_operations {
	int                        (*writepage)(struct page *, struct writeback_control *); /*     0     8 */
	int                        (*readpage)(struct file *, struct page *); /*     8     8 */
	int                        (*writepages)(struct address_space *, struct writeback_control *); /*    16     8 */
	int                        (*set_page_dirty)(struct page *); /*    24     8 */
	int                        (*readpages)(struct file *, struct address_space *, struct list_head *, unsigned int); /*    32     8 */
	void                       (*readahead)(struct readahead_control *); /*    40     8 */
	int                        (*write_begin)(struct file *, struct address_space *, loff_t, unsigned int, unsigned int, struct page * *, void * *); /*    48     8 */
	int                        (*write_end)(struct file *, struct address_space *, loff_t, unsigned int, unsigned int, struct page *, void *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	sector_t                   (*bmap)(struct address_space *, sector_t); /*    64     8 */
	void                       (*invalidatepage)(struct page *, unsigned int, unsigned int); /*    72     8 */
	int                        (*releasepage)(struct page *, gfp_t); /*    80     8 */
	void                       (*freepage)(struct page *); /*    88     8 */
	ssize_t                    (*direct_IO)(struct kiocb *, struct iov_iter *); /*    96     8 */
	int                        (*migratepage)(struct address_space *, struct page *, struct page *, enum migrate_mode); /*   104     8 */
	bool                       (*isolate_page)(struct page *, isolate_mode_t); /*   112     8 */
	void                       (*putback_page)(struct page *); /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	int                        (*launder_page)(struct page *); /*   128     8 */
	int                        (*is_partially_uptodate)(struct page *, long unsigned int, long unsigned int); /*   136     8 */
	void                       (*is_dirty_writeback)(struct page *, bool *, bool *); /*   144     8 */
	int                        (*error_remove_page)(struct address_space *, struct page *); /*   152     8 */
	int                        (*swap_activate)(struct swap_info_struct *, struct file *, sector_t *); /*   160     8 */
	void                       (*swap_deactivate)(struct file *); /*   168     8 */
	u64                        kabi_reserved1;       /*   176     8 */
	u64                        kabi_reserved2;       /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	u64                        kabi_reserved3;       /*   192     8 */
	u64                        kabi_reserved4;       /*   200     8 */

	/* size: 208, cachelines: 4, members: 26 */
	/* last cacheline: 16 bytes */
};
struct inode_operations {
	struct dentry *            (*lookup)(struct inode *, struct dentry *, unsigned int); /*     0     8 */
	const char  *              (*get_link)(struct dentry *, struct inode *, struct delayed_call *); /*     8     8 */
	int                        (*permission)(struct inode *, int); /*    16     8 */
	struct posix_acl *         (*get_acl)(struct inode *, int); /*    24     8 */
	int                        (*readlink)(struct dentry *, char *, int); /*    32     8 */
	int                        (*create)(struct inode *, struct dentry *, umode_t, bool); /*    40     8 */
	int                        (*link)(struct dentry *, struct inode *, struct dentry *); /*    48     8 */
	int                        (*unlink)(struct inode *, struct dentry *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*symlink)(struct inode *, struct dentry *, const char  *); /*    64     8 */
	int                        (*mkdir)(struct inode *, struct dentry *, umode_t); /*    72     8 */
	int                        (*rmdir)(struct inode *, struct dentry *); /*    80     8 */
	int                        (*mknod)(struct inode *, struct dentry *, umode_t, dev_t); /*    88     8 */
	int                        (*rename)(struct inode *, struct dentry *, struct inode *, struct dentry *, unsigned int); /*    96     8 */
	int                        (*setattr)(struct dentry *, struct iattr *); /*   104     8 */
	int                        (*getattr)(const struct path  *, struct kstat *, u32, unsigned int); /*   112     8 */
	ssize_t                    (*listxattr)(struct dentry *, char *, size_t); /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	int                        (*fiemap)(struct inode *, struct fiemap_extent_info *, u64, u64); /*   128     8 */
	int                        (*update_time)(struct inode *, struct timespec64 *, int); /*   136     8 */
	int                        (*atomic_open)(struct inode *, struct dentry *, struct file *, unsigned int, umode_t); /*   144     8 */
	int                        (*tmpfile)(struct inode *, struct dentry *, umode_t); /*   152     8 */
	int                        (*set_acl)(struct inode *, struct posix_acl *, int); /*   160     8 */
	u64                        kabi_reserved1;       /*   168     8 */
	u64                        kabi_reserved2;       /*   176     8 */
	u64                        kabi_reserved3;       /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	u64                        kabi_reserved4;       /*   192     8 */
	u64                        kabi_reserved5;       /*   200     8 */
	u64                        kabi_reserved6;       /*   208     8 */
	u64                        kabi_reserved7;       /*   216     8 */
	u64                        kabi_reserved8;       /*   224     8 */
	u64                        kabi_reserved9;       /*   232     8 */
	u64                        kabi_reserved10;      /*   240     8 */
	u64                        kabi_reserved11;      /*   248     8 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	u64                        kabi_reserved12;      /*   256     8 */

	/* size: 320, cachelines: 5, members: 33 */
	/* padding: 56 */
} __attribute__((__aligned__(64)));
struct file_lock_context {
	spinlock_t                 flc_lock;             /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           flc_flock;            /*     8    16 */
	struct list_head           flc_posix;            /*    24    16 */
	struct list_head           flc_lease;            /*    40    16 */

	/* size: 56, cachelines: 1, members: 4 */
	/* sum members: 52, holes: 1, sum holes: 4 */
	/* last cacheline: 56 bytes */
};
struct fown_struct {
	rwlock_t                   lock;                 /*     0     8 */
	struct pid *               pid;                  /*     8     8 */
	enum pid_type              pid_type;             /*    16     4 */
	kuid_t                     uid;                  /*    20     4 */
	kuid_t                     euid;                 /*    24     4 */
	int                        signum;               /*    28     4 */

	/* size: 32, cachelines: 1, members: 6 */
	/* last cacheline: 32 bytes */
};
struct file_ra_state {
	long unsigned int          start;                /*     0     8 */
	unsigned int               size;                 /*     8     4 */
	unsigned int               async_size;           /*    12     4 */
	unsigned int               ra_pages;             /*    16     4 */
	unsigned int               mmap_miss;            /*    20     4 */
	loff_t                     prev_pos;             /*    24     8 */

	/* size: 32, cachelines: 1, members: 6 */
	/* last cacheline: 32 bytes */
};
struct file_lock_operations {
	void                       (*fl_copy_lock)(struct file_lock *, struct file_lock *); /*     0     8 */
	void                       (*fl_release_private)(struct file_lock *); /*     8     8 */
	u64                        kabi_reserved1;       /*    16     8 */
	u64                        kabi_reserved2;       /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct file_lock {
	struct file_lock *         fl_blocker;           /*     0     8 */
	struct list_head           fl_list;              /*     8    16 */
	struct hlist_node          fl_link;              /*    24    16 */
	struct list_head           fl_blocked_requests;  /*    40    16 */
	struct list_head           fl_blocked_member;    /*    56    16 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	fl_owner_t                 fl_owner;             /*    72     8 */
	unsigned int               fl_flags;             /*    80     4 */
	unsigned char              fl_type;              /*    84     1 */

	/* XXX 3 bytes hole, try to pack */

	unsigned int               fl_pid;               /*    88     4 */
	int                        fl_link_cpu;          /*    92     4 */
	wait_queue_head_t          fl_wait;              /*    96    24 */
	struct file *              fl_file;              /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	loff_t                     fl_start;             /*   128     8 */
	loff_t                     fl_end;               /*   136     8 */
	struct fasync_struct *     fl_fasync;            /*   144     8 */
	long unsigned int          fl_break_time;        /*   152     8 */
	long unsigned int          fl_downgrade_time;    /*   160     8 */
	const struct file_lock_operations  * fl_ops;     /*   168     8 */
	const struct lock_manager_operations  * fl_lmops; /*   176     8 */
	union {
		struct nfs_lock_info nfs_fl;             /*   184    32 */
		struct nfs4_lock_info nfs4_fl;           /*   184     8 */
		struct {
			struct list_head link;           /*   184    16 */
			/* --- cacheline 3 boundary (192 bytes) was 8 bytes ago --- */
			int        state;                /*   200     4 */
			unsigned int debug_id;           /*   204     4 */
		} afs;                                   /*   184    24 */
	} fl_u;                                          /*   184    32 */

	/* size: 216, cachelines: 4, members: 20 */
	/* sum members: 213, holes: 1, sum holes: 3 */
	/* last cacheline: 24 bytes */
};
struct lock_manager_operations {
	fl_owner_t                 (*lm_get_owner)(fl_owner_t); /*     0     8 */
	void                       (*lm_put_owner)(fl_owner_t); /*     8     8 */
	void                       (*lm_notify)(struct file_lock *); /*    16     8 */
	int                        (*lm_grant)(struct file_lock *, int); /*    24     8 */
	bool                       (*lm_break)(struct file_lock *); /*    32     8 */
	int                        (*lm_change)(struct file_lock *, int, struct list_head *); /*    40     8 */
	void                       (*lm_setup)(struct file_lock *, void * *); /*    48     8 */
	bool                       (*lm_breaker_owns_lease)(struct file_lock *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u64                        kabi_reserved1;       /*    64     8 */
	u64                        kabi_reserved2;       /*    72     8 */

	/* size: 80, cachelines: 2, members: 10 */
	/* last cacheline: 16 bytes */
};
struct nfs_lock_info {
	u32                        state;                /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct nlm_lockowner *     owner;                /*     8     8 */
	struct list_head           list;                 /*    16    16 */

	/* size: 32, cachelines: 1, members: 3 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct nfs4_lock_info {
	struct nfs4_lock_state *   owner;                /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct fasync_struct {
	rwlock_t                   fa_lock;              /*     0     8 */
	int                        magic;                /*     8     4 */
	int                        fa_fd;                /*    12     4 */
	struct fasync_struct *     fa_next;              /*    16     8 */
	struct file *              fa_file;              /*    24     8 */
	struct callback_head       fa_rcu __attribute__((__aligned__(8))); /*    32    16 */

	/* size: 48, cachelines: 1, members: 6 */
	/* forced alignments: 1 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct sb_writers {
	short unsigned int         frozen;               /*     0     2 */
	short unsigned int         frozen_ro;            /*     2     2 */

	/* XXX 4 bytes hole, try to pack */

	wait_queue_head_t          wait_unfrozen;        /*     8    24 */
	struct percpu_rw_semaphore rw_sem[3] __attribute__((__aligned__(8))); /*    32   288 */

	/* size: 320, cachelines: 5, members: 4 */
	/* sum members: 316, holes: 1, sum holes: 4 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(8)));
struct super_operations {
	struct inode *             (*alloc_inode)(struct super_block *); /*     0     8 */
	void                       (*destroy_inode)(struct inode *); /*     8     8 */
	void                       (*free_inode)(struct inode *); /*    16     8 */
	void                       (*dirty_inode)(struct inode *, int); /*    24     8 */
	int                        (*write_inode)(struct inode *, struct writeback_control *); /*    32     8 */
	int                        (*drop_inode)(struct inode *); /*    40     8 */
	void                       (*evict_inode)(struct inode *); /*    48     8 */
	void                       (*put_super)(struct super_block *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*sync_fs)(struct super_block *, int); /*    64     8 */
	int                        (*freeze_super)(struct super_block *); /*    72     8 */
	int                        (*freeze_fs)(struct super_block *); /*    80     8 */
	int                        (*thaw_super)(struct super_block *); /*    88     8 */
	int                        (*unfreeze_fs)(struct super_block *); /*    96     8 */
	int                        (*statfs)(struct dentry *, struct kstatfs *); /*   104     8 */
	int                        (*remount_fs)(struct super_block *, int *, char *); /*   112     8 */
	void                       (*umount_begin)(struct super_block *); /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	int                        (*show_options)(struct seq_file *, struct dentry *); /*   128     8 */
	int                        (*show_devname)(struct seq_file *, struct dentry *); /*   136     8 */
	int                        (*show_path)(struct seq_file *, struct dentry *); /*   144     8 */
	int                        (*show_stats)(struct seq_file *, struct dentry *); /*   152     8 */
	ssize_t                    (*quota_read)(struct super_block *, int, char *, size_t, loff_t); /*   160     8 */
	ssize_t                    (*quota_write)(struct super_block *, int, const char  *, size_t, loff_t); /*   168     8 */
	struct dquot * *           (*get_dquots)(struct inode *); /*   176     8 */
	long int                   (*nr_cached_objects)(struct super_block *, struct shrink_control *); /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	long int                   (*free_cached_objects)(struct super_block *, struct shrink_control *); /*   192     8 */
	u64                        kabi_reserved1;       /*   200     8 */
	u64                        kabi_reserved2;       /*   208     8 */
	u64                        kabi_reserved3;       /*   216     8 */
	u64                        kabi_reserved4;       /*   224     8 */

	/* size: 232, cachelines: 4, members: 29 */
	/* last cacheline: 40 bytes */
};
struct dir_context {
	filldir_t                  actor;                /*     0     8 */
	loff_t                     pos;                  /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct tlb_context {
	u64                        ctx_id;               /*     0     8 */
	u64                        tlb_gen;              /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct tlb_state {
	struct mm_struct *         loaded_mm;            /*     0     8 */
	union {
		struct mm_struct * last_user_mm;         /*     8     8 */
		long unsigned int  last_user_mm_ibpb;    /*     8     8 */
	};                                               /*     8     8 */
	u16                        loaded_mm_asid;       /*    16     2 */
	u16                        next_asid;            /*    18     2 */
	bool                       invalidate_other;     /*    20     1 */

	/* XXX 1 byte hole, try to pack */

	short unsigned int         user_pcid_flush_mask; /*    22     2 */
	long unsigned int          cr4;                  /*    24     8 */
	struct tlb_context         ctxs[6];              /*    32    96 */

	/* size: 128, cachelines: 2, members: 8 */
	/* sum members: 127, holes: 1, sum holes: 1 */
};
struct boot_params_to_save {
	unsigned int               start;                /*     0     4 */
	unsigned int               len;                  /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct x86_legacy_devices {
	int                        pnpbios;              /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct x86_legacy_features {
	enum x86_legacy_i8042_state i8042;               /*     0     4 */
	int                        rtc;                  /*     4     4 */
	int                        warm_reset;           /*     8     4 */
	int                        no_vga;               /*    12     4 */
	int                        reserve_bios_regions; /*    16     4 */
	struct x86_legacy_devices  devices;              /*    20     4 */

	/* size: 24, cachelines: 1, members: 6 */
	/* last cacheline: 24 bytes */
};
struct x86_hyper_runtime {
	void                       (*pin_vcpu)(int);     /*     0     8 */
	void                       (*sev_es_hcall_prepare)(struct ghcb *, struct pt_regs *); /*     8     8 */
	bool                       (*sev_es_hcall_finish)(struct ghcb *, struct pt_regs *); /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct x86_platform_ops {
	long unsigned int          (*calibrate_cpu)(void); /*     0     8 */
	long unsigned int          (*calibrate_tsc)(void); /*     8     8 */
	void                       (*get_wallclock)(struct timespec64 *); /*    16     8 */
	int                        (*set_wallclock)(const struct timespec64  *); /*    24     8 */
	void                       (*iommu_shutdown)(void); /*    32     8 */
	bool                       (*is_untracked_pat_range)(u64, u64); /*    40     8 */
	void                       (*nmi_init)(void);    /*    48     8 */
	unsigned char              (*get_nmi_reason)(void); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	void                       (*save_sched_clock_state)(void); /*    64     8 */
	void                       (*restore_sched_clock_state)(void); /*    72     8 */
	void                       (*apic_post_init)(void); /*    80     8 */
	struct x86_legacy_features legacy;               /*    88    24 */
	void                       (*set_legacy_features)(void); /*   112     8 */
	struct x86_hyper_runtime   hyper;                /*   120    24 */

	/* size: 144, cachelines: 3, members: 14 */
	/* last cacheline: 16 bytes */
};
struct kernel_symbol {
	int                        value_offset;         /*     0     4 */
	int                        name_offset;          /*     4     4 */
	int                        namespace_offset;     /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct obs_kernel_param {
	const char  *              str;                  /*     0     8 */
	int                        (*setup_func)(char *); /*     8     8 */
	int                        early;                /*    16     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct lockdep_map {

	/* size: 0, cachelines: 0, members: 0 */
};
struct jump_entry {
	s32                        code;                 /*     0     4 */
	s32                        target;               /*     4     4 */
	long int                   key;                  /*     8     8 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct static_key {
	atomic_t                   enabled;              /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	union {
		long unsigned int  type;                 /*     8     8 */
		struct jump_entry * entries;             /*     8     8 */
		struct static_key_mod * next;            /*     8     8 */
	};                                               /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct static_key_true {
	struct static_key          key;                  /*     0    16 */

	/* size: 16, cachelines: 1, members: 1 */
	/* last cacheline: 16 bytes */
};
struct static_key_false {
	struct static_key          key;                  /*     0    16 */

	/* size: 16, cachelines: 1, members: 1 */
	/* last cacheline: 16 bytes */
};
struct _ddebug {
	const char  *              modname;              /*     0     8 */
	const char  *              function;             /*     8     8 */
	const char  *              filename;             /*    16     8 */
	const char  *              format;               /*    24     8 */
	unsigned int               lineno:18;            /*    32: 0  4 */
	unsigned int               flags:8;              /*    32:18  4 */

	/* XXX 6 bits hole, try to pack */
	/* XXX 4 bytes hole, try to pack */

	union {
		struct static_key_true dd_key_true;      /*    40    16 */
		struct static_key_false dd_key_false;    /*    40    16 */
	} key;                                           /*    40    16 */

	/* size: 56, cachelines: 1, members: 7 */
	/* sum members: 48, holes: 1, sum holes: 4 */
	/* sum bitfield members: 26 bits, bit holes: 1, sum bit holes: 6 bits */
	/* last cacheline: 56 bytes */
} __attribute__((__aligned__(8)));
struct static_call_site {
	s32                        addr;                 /*     0     4 */
	s32                        key;                  /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct orc_entry {
	s16                        sp_offset;            /*     0     2 */
	s16                        bp_offset;            /*     2     2 */
	unsigned int               sp_reg:4;             /*     4: 0  4 */
	unsigned int               bp_reg:4;             /*     4: 4  4 */
	unsigned int               type:2;               /*     4: 8  4 */
	unsigned int               end:1;                /*     4:10  4 */

	/* size: 6, cachelines: 1, members: 6 */
	/* bit_padding: 5 bits */
	/* last cacheline: 6 bytes */
} __attribute__((__packed__));
struct paravirt_callee_save {
	void *                     func;                 /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct pv_init_ops {
	unsigned int               (*patch)(u8, void *, long unsigned int, unsigned int); /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct pv_time_ops {
	long long unsigned int     (*sched_clock)(void); /*     0     8 */
	long long unsigned int     (*steal_clock)(int);  /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct pv_cpu_ops {
	void                       (*io_delay)(void);    /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct pv_irq_ops {

	/* size: 0, cachelines: 0, members: 0 */
};
struct pv_mmu_ops {
	void                       (*flush_tlb_user)(void); /*     0     8 */
	void                       (*flush_tlb_kernel)(void); /*     8     8 */
	void                       (*flush_tlb_one_user)(long unsigned int); /*    16     8 */
	void                       (*flush_tlb_multi)(const struct cpumask  *, const struct flush_tlb_info  *); /*    24     8 */
	void                       (*tlb_remove_table)(struct mmu_gather *, void *); /*    32     8 */
	void                       (*exit_mmap)(struct mm_struct *); /*    40     8 */

	/* size: 48, cachelines: 1, members: 6 */
	/* last cacheline: 48 bytes */
};
struct flush_tlb_info {
	struct mm_struct *         mm;                   /*     0     8 */
	long unsigned int          start;                /*     8     8 */
	long unsigned int          end;                  /*    16     8 */
	u64                        new_tlb_gen;          /*    24     8 */
	unsigned int               initiating_cpu;       /*    32     4 */
	u8                         stride_shift;         /*    36     1 */
	u8                         freed_tables;         /*    37     1 */

	/* size: 40, cachelines: 1, members: 7 */
	/* padding: 2 */
	/* last cacheline: 40 bytes */
};
struct pv_lock_ops {
	void                       (*queued_spin_lock_slowpath)(struct qspinlock *, u32); /*     0     8 */
	struct paravirt_callee_save queued_spin_unlock;  /*     8     8 */
	void                       (*wait)(u8 *, u8);    /*    16     8 */
	void                       (*kick)(int);         /*    24     8 */
	struct paravirt_callee_save vcpu_is_preempted;   /*    32     8 */
	u64                        kabi_reserved1;       /*    40     8 */

	/* size: 48, cachelines: 1, members: 6 */
	/* last cacheline: 48 bytes */
};
struct paravirt_patch_template {
	struct pv_init_ops         init;                 /*     0     8 */
	struct pv_time_ops         time;                 /*     8    16 */
	struct pv_cpu_ops          cpu;                  /*    24     8 */
	struct pv_irq_ops          irq;                  /*    32     0 */
	struct pv_mmu_ops          mmu;                  /*    32    48 */
	/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
	struct pv_lock_ops         lock;                 /*    80    48 */

	/* size: 128, cachelines: 2, members: 6 */
};
struct bug_entry {
	int                        bug_addr_disp;        /*     0     4 */
	int                        file_disp;            /*     4     4 */
	short unsigned int         line;                 /*     8     2 */
	short unsigned int         flags;                /*    10     2 */

	/* size: 12, cachelines: 1, members: 4 */
	/* last cacheline: 12 bytes */
};
struct tracepoint_func {
	void *                     func;                 /*     0     8 */
	void *                     data;                 /*     8     8 */
	int                        prio;                 /*    16     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct tracepoint {
	const char  *              name;                 /*     0     8 */
	struct static_key          key;                  /*     8    16 */
	struct static_call_key *   static_call_key;      /*    24     8 */
	void *                     static_call_tramp;    /*    32     8 */
	void *                     iterator;             /*    40     8 */
	int                        (*regfunc)(void);     /*    48     8 */
	void                       (*unregfunc)(void);   /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct tracepoint_func *   funcs;                /*    64     8 */

	/* size: 72, cachelines: 2, members: 8 */
	/* last cacheline: 8 bytes */
};
struct static_call_key {
	void *                     func;                 /*     0     8 */
	union {
		long unsigned int  type;                 /*     8     8 */
		struct static_call_mod * mods;           /*     8     8 */
		struct static_call_site * sites;         /*     8     8 */
	};                                               /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct bpf_raw_event_map {
	struct tracepoint *        tp;                   /*     0     8 */
	void *                     bpf_func;             /*     8     8 */
	u32                        num_args;             /*    16     4 */
	u32                        writable_size;        /*    20     4 */

	/* size: 32, cachelines: 1, members: 4 */
	/* padding: 8 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(32)));
struct seq_operations {
	void *                     (*start)(struct seq_file *, loff_t *); /*     0     8 */
	void                       (*stop)(struct seq_file *, void *); /*     8     8 */
	void *                     (*next)(struct seq_file *, void *, loff_t *); /*    16     8 */
	int                        (*show)(struct seq_file *, void *); /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct fixed_percpu_data {
	char                       gs_base[40];          /*     0    40 */
	long unsigned int          stack_canary;         /*    40     8 */

	/* size: 48, cachelines: 1, members: 2 */
	/* last cacheline: 48 bytes */
};
struct perf_event {
	struct list_head           event_entry;          /*     0    16 */
	struct list_head           sibling_list;         /*    16    16 */
	struct list_head           active_list;          /*    32    16 */
	struct rb_node             group_node __attribute__((__aligned__(8))); /*    48    24 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	u64                        group_index;          /*    72     8 */
	struct list_head           migrate_entry;        /*    80    16 */
	struct hlist_node          hlist_entry;          /*    96    16 */
	struct list_head           active_entry;         /*   112    16 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	int                        nr_siblings;          /*   128     4 */
	int                        event_caps;           /*   132     4 */
	int                        group_caps;           /*   136     4 */

	/* XXX 4 bytes hole, try to pack */

	struct perf_event *        group_leader;         /*   144     8 */
	struct pmu *               pmu;                  /*   152     8 */
	void *                     pmu_private;          /*   160     8 */
	enum perf_event_state      state;                /*   168     4 */
	unsigned int               attach_state;         /*   172     4 */
	local64_t                  count;                /*   176     8 */
	atomic64_t                 child_count;          /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	u64                        total_time_enabled;   /*   192     8 */
	u64                        total_time_running;   /*   200     8 */
	u64                        tstamp;               /*   208     8 */
	u64                        shadow_ctx_time;      /*   216     8 */
	struct perf_event_attr     attr;                 /*   224   120 */
	/* --- cacheline 5 boundary (320 bytes) was 24 bytes ago --- */
	u16                        header_size;          /*   344     2 */
	u16                        id_header_size;       /*   346     2 */
	u16                        read_size;            /*   348     2 */

	/* XXX 2 bytes hole, try to pack */

	struct hw_perf_event       hw __attribute__((__aligned__(8))); /*   352   192 */
	/* --- cacheline 8 boundary (512 bytes) was 32 bytes ago --- */
	struct perf_event_context * ctx;                 /*   544     8 */
	atomic_long_t              refcount;             /*   552     8 */
	atomic64_t                 child_total_time_enabled; /*   560     8 */
	atomic64_t                 child_total_time_running; /*   568     8 */
	/* --- cacheline 9 boundary (576 bytes) --- */
	struct mutex               child_mutex;          /*   576    32 */
	struct list_head           child_list;           /*   608    16 */
	struct perf_event *        parent;               /*   624     8 */
	int                        oncpu;                /*   632     4 */
	int                        cpu;                  /*   636     4 */
	/* --- cacheline 10 boundary (640 bytes) --- */
	struct list_head           owner_entry;          /*   640    16 */
	struct task_struct *       owner;                /*   656     8 */
	struct mutex               mmap_mutex;           /*   664    32 */
	atomic_t                   mmap_count;           /*   696     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 11 boundary (704 bytes) --- */
	struct perf_buffer *       rb;                   /*   704     8 */
	struct list_head           rb_entry;             /*   712    16 */
	long unsigned int          rcu_batches;          /*   728     8 */
	int                        rcu_pending;          /*   736     4 */

	/* XXX 4 bytes hole, try to pack */

	wait_queue_head_t          waitq;                /*   744    24 */
	/* --- cacheline 12 boundary (768 bytes) --- */
	struct fasync_struct *     fasync;               /*   768     8 */
	int                        pending_wakeup;       /*   776     4 */
	int                        pending_kill;         /*   780     4 */
	int                        pending_disable;      /*   784     4 */

	/* XXX 4 bytes hole, try to pack */

	struct irq_work            pending;              /*   792    24 */
	atomic_t                   event_limit;          /*   816     4 */

	/* XXX 4 bytes hole, try to pack */

	struct perf_addr_filters_head addr_filters;      /*   824    24 */
	/* --- cacheline 13 boundary (832 bytes) was 16 bytes ago --- */
	struct perf_addr_filter_range * addr_filter_ranges; /*   848     8 */
	long unsigned int          addr_filters_gen;     /*   856     8 */
	struct perf_event *        aux_event;            /*   864     8 */
	void                       (*destroy)(struct perf_event *); /*   872     8 */
	struct callback_head       callback_head __attribute__((__aligned__(8))); /*   880    16 */
	/* --- cacheline 14 boundary (896 bytes) --- */
	struct pid_namespace *     ns;                   /*   896     8 */
	u64                        id;                   /*   904     8 */
	u64                        (*clock)(void);       /*   912     8 */
	perf_overflow_handler_t    overflow_handler;     /*   920     8 */
	void *                     overflow_handler_context; /*   928     8 */
	perf_overflow_handler_t    orig_overflow_handler; /*   936     8 */
	struct bpf_prog *          prog;                 /*   944     8 */
	struct trace_event_call *  tp_event;             /*   952     8 */
	/* --- cacheline 15 boundary (960 bytes) --- */
	struct event_filter *      filter;               /*   960     8 */
	struct ftrace_ops          ftrace_ops;           /*   968   176 */
	/* --- cacheline 17 boundary (1088 bytes) was 56 bytes ago --- */
	struct perf_cgroup *       cgrp;                 /*  1144     8 */
	/* --- cacheline 18 boundary (1152 bytes) --- */
	void *                     security;             /*  1152     8 */
	struct list_head           sb_list;              /*  1160    16 */

	/* size: 1176, cachelines: 19, members: 70 */
	/* sum members: 1154, holes: 6, sum holes: 22 */
	/* forced alignments: 3, forced holes: 1, sum forced holes: 2 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct user_namespace {
	struct uid_gid_map         uid_map;              /*     0    72 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	struct uid_gid_map         gid_map;              /*    72    72 */
	/* --- cacheline 2 boundary (128 bytes) was 16 bytes ago --- */
	struct uid_gid_map         projid_map;           /*   144    72 */
	/* --- cacheline 3 boundary (192 bytes) was 24 bytes ago --- */
	atomic_t                   count;                /*   216     4 */

	/* XXX 4 bytes hole, try to pack */

	struct user_namespace *    parent;               /*   224     8 */
	int                        level;                /*   232     4 */
	kuid_t                     owner;                /*   236     4 */
	kgid_t                     group;                /*   240     4 */

	/* XXX 4 bytes hole, try to pack */

	struct ns_common           ns;                   /*   248    32 */
	/* --- cacheline 4 boundary (256 bytes) was 24 bytes ago --- */
	long unsigned int          flags;                /*   280     8 */
	bool                       parent_could_setfcap; /*   288     1 */

	/* XXX 7 bytes hole, try to pack */

	struct list_head           keyring_name_list;    /*   296    16 */
	struct key *               user_keyring_register; /*   312     8 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	struct rw_semaphore        keyring_sem;          /*   320    40 */
	struct key *               persistent_keyring_register; /*   360     8 */
	struct work_struct         work;                 /*   368    64 */
	/* --- cacheline 6 boundary (384 bytes) was 48 bytes ago --- */
	struct ctl_table_set       set __attribute__((__aligned__(8))); /*   432    96 */
	/* --- cacheline 8 boundary (512 bytes) was 16 bytes ago --- */
	struct ctl_table_header *  sysctls;              /*   528     8 */
	struct ucounts *           ucounts;              /*   536     8 */
	long int                   ucount_max[25];       /*   544   200 */
	/* --- cacheline 11 boundary (704 bytes) was 40 bytes ago --- */
	u64                        kabi_reserved1;       /*   744     8 */
	u64                        kabi_reserved2;       /*   752     8 */
	u64                        kabi_reserved3;       /*   760     8 */
	/* --- cacheline 12 boundary (768 bytes) --- */
	u64                        kabi_reserved4;       /*   768     8 */
	u64                        kabi_reserved5;       /*   776     8 */
	u64                        kabi_reserved6;       /*   784     8 */
	u64                        kabi_reserved7;       /*   792     8 */
	u64                        kabi_reserved8;       /*   800     8 */
	u64                        kabi_reserved9;       /*   808     8 */
	u64                        kabi_reserved10;      /*   816     8 */

	/* size: 824, cachelines: 13, members: 30 */
	/* sum members: 809, holes: 3, sum holes: 15 */
	/* forced alignments: 1 */
	/* last cacheline: 56 bytes */
} __attribute__((__aligned__(8)));
struct pollfd {
	int                        fd;                   /*     0     4 */
	short int                  events;               /*     4     2 */
	short int                  revents;              /*     6     2 */

	/* size: 8, cachelines: 1, members: 3 */
	/* last cacheline: 8 bytes */
};
struct __call_single_data {
	struct __call_single_node  node;                 /*     0    16 */
	smp_call_func_t            func;                 /*    16     8 */
	void *                     info;                 /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* last cacheline: 32 bytes */
};
struct smp_ops {
	void                       (*smp_prepare_boot_cpu)(void); /*     0     8 */
	void                       (*smp_prepare_cpus)(unsigned int); /*     8     8 */
	void                       (*smp_cpus_done)(unsigned int); /*    16     8 */
	void                       (*stop_other_cpus)(int); /*    24     8 */
	void                       (*crash_stop_other_cpus)(void); /*    32     8 */
	void                       (*smp_send_reschedule)(int); /*    40     8 */
	int                        (*cpu_up)(unsigned int, struct task_struct *); /*    48     8 */
	int                        (*cpu_disable)(void); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	void                       (*cpu_die)(unsigned int); /*    64     8 */
	void                       (*play_dead)(void);   /*    72     8 */
	void                       (*send_call_func_ipi)(const struct cpumask  *); /*    80     8 */
	void                       (*send_call_func_single_ipi)(int); /*    88     8 */

	/* size: 96, cachelines: 2, members: 12 */
	/* last cacheline: 32 bytes */
};
struct wait_queue_entry {
	unsigned int               flags;                /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     private;              /*     8     8 */
	wait_queue_func_t          func;                 /*    16     8 */
	struct list_head           entry;                /*    24    16 */

	/* size: 40, cachelines: 1, members: 4 */
	/* sum members: 36, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct attribute_group {
	const char  *              name;                 /*     0     8 */
	umode_t                    (*is_visible)(struct kobject *, struct attribute *, int); /*     8     8 */
	umode_t                    (*is_bin_visible)(struct kobject *, struct bin_attribute *, int); /*    16     8 */
	struct attribute * *       attrs;                /*    24     8 */
	struct bin_attribute * *   bin_attrs;            /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct delayed_work {
	struct work_struct         work;                 /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct timer_list          timer;                /*    64    72 */
	/* --- cacheline 2 boundary (128 bytes) was 8 bytes ago --- */
	struct workqueue_struct *  wq;                   /*   136     8 */
	int                        cpu;                  /*   144     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          data;                 /*   152     8 */
	u64                        kabi_reserved1;       /*   160     8 */
	u64                        kabi_reserved2;       /*   168     8 */
	u64                        kabi_reserved3;       /*   176     8 */
	u64                        kabi_reserved4;       /*   184     8 */

	/* size: 192, cachelines: 3, members: 9 */
	/* sum members: 188, holes: 1, sum holes: 4 */
};
struct rcu_work {
	struct work_struct         work;                 /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*    64    16 */
	struct workqueue_struct *  wq;                   /*    80     8 */

	/* size: 88, cachelines: 2, members: 3 */
	/* forced alignments: 1 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct rcu_segcblist {
	struct callback_head *     head;                 /*     0     8 */
	struct callback_head * *   tails[4];             /*     8    32 */
	long unsigned int          gp_seq[4];            /*    40    32 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	atomic_long_t              len;                  /*    72     8 */
	u8                         enabled;              /*    80     1 */
	u8                         offloaded;            /*    81     1 */

	/* size: 88, cachelines: 2, members: 6 */
	/* padding: 6 */
	/* last cacheline: 24 bytes */
};
struct srcu_data {
	long unsigned int          srcu_lock_count[2];   /*     0    16 */
	long unsigned int          srcu_unlock_count[2]; /*    16    16 */

	/* XXX 32 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	spinlock_t                 lock __attribute__((__aligned__(64))); /*    64     4 */

	/* XXX 4 bytes hole, try to pack */

	struct rcu_segcblist       srcu_cblist;          /*    72    88 */

	/* XXX last struct has 6 bytes of padding */

	/* --- cacheline 2 boundary (128 bytes) was 32 bytes ago --- */
	long unsigned int          srcu_gp_seq_needed;   /*   160     8 */
	long unsigned int          srcu_gp_seq_needed_exp; /*   168     8 */
	bool                       srcu_cblist_invoking; /*   176     1 */

	/* XXX 7 bytes hole, try to pack */

	struct timer_list          delay_work;           /*   184    72 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	struct work_struct         work;                 /*   256    64 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	struct callback_head       srcu_barrier_head __attribute__((__aligned__(8))); /*   320    16 */
	struct srcu_node *         mynode;               /*   336     8 */
	long unsigned int          grpmask;              /*   344     8 */
	int                        cpu;                  /*   352     4 */

	/* XXX 4 bytes hole, try to pack */

	struct srcu_struct *       ssp;                  /*   360     8 */

	/* size: 384, cachelines: 6, members: 14 */
	/* sum members: 321, holes: 4, sum holes: 47 */
	/* padding: 16 */
	/* paddings: 1, sum paddings: 6 */
	/* forced alignments: 2, forced holes: 1, sum forced holes: 32 */
} __attribute__((__aligned__(64)));
struct srcu_node {
	spinlock_t                 lock;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          srcu_have_cbs[4];     /*     8    32 */
	long unsigned int          srcu_data_have_cbs[4]; /*    40    32 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	long unsigned int          srcu_gp_seq_needed_exp; /*    72     8 */
	struct srcu_node *         srcu_parent;          /*    80     8 */
	int                        grplo;                /*    88     4 */
	int                        grphi;                /*    92     4 */

	/* size: 96, cachelines: 2, members: 7 */
	/* sum members: 92, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct srcu_struct {
	struct srcu_node           node[521];            /*     0 50016 */
	/* --- cacheline 781 boundary (49984 bytes) was 32 bytes ago --- */
	struct srcu_node *         level[4];             /* 50016    32 */
	/* --- cacheline 782 boundary (50048 bytes) --- */
	struct mutex               srcu_cb_mutex;        /* 50048    32 */
	spinlock_t                 lock;                 /* 50080     4 */

	/* XXX 4 bytes hole, try to pack */

	struct mutex               srcu_gp_mutex;        /* 50088    32 */
	/* --- cacheline 783 boundary (50112 bytes) was 8 bytes ago --- */
	unsigned int               srcu_idx;             /* 50120     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          srcu_gp_seq;          /* 50128     8 */
	long unsigned int          srcu_gp_seq_needed;   /* 50136     8 */
	long unsigned int          srcu_gp_seq_needed_exp; /* 50144     8 */
	long unsigned int          srcu_last_gp_end;     /* 50152     8 */
	struct srcu_data *         sda;                  /* 50160     8 */
	long unsigned int          srcu_barrier_seq;     /* 50168     8 */
	/* --- cacheline 784 boundary (50176 bytes) --- */
	struct mutex               srcu_barrier_mutex;   /* 50176    32 */
	struct completion          srcu_barrier_completion; /* 50208    32 */
	/* --- cacheline 785 boundary (50240 bytes) --- */
	atomic_t                   srcu_barrier_cpu_cnt; /* 50240     4 */

	/* XXX 4 bytes hole, try to pack */

	struct delayed_work        work;                 /* 50248   192 */

	/* size: 50440, cachelines: 789, members: 16 */
	/* sum members: 50428, holes: 3, sum holes: 12 */
	/* last cacheline: 8 bytes */
};
struct vdso_image {
	void *                     data;                 /*     0     8 */
	long unsigned int          size;                 /*     8     8 */
	long unsigned int          alt;                  /*    16     8 */
	long unsigned int          alt_len;              /*    24     8 */
	long unsigned int          extable_base;         /*    32     8 */
	long unsigned int          extable_len;          /*    40     8 */
	const void  *              extable;              /*    48     8 */
	long int                   sym_vvar_start;       /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	long int                   sym_vvar_page;        /*    64     8 */
	long int                   sym_pvclock_page;     /*    72     8 */
	long int                   sym_hvclock_page;     /*    80     8 */
	long int                   sym_timens_page;      /*    88     8 */
	long int                   sym_VDSO32_NOTE_MASK; /*    96     8 */
	long int                   sym___kernel_sigreturn; /*   104     8 */
	long int                   sym___kernel_rt_sigreturn; /*   112     8 */
	long int                   sym___kernel_vsyscall; /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	long int                   sym_int80_landing_pad; /*   128     8 */

	/* size: 136, cachelines: 3, members: 17 */
	/* last cacheline: 8 bytes */
};
struct anon_vma {
	struct anon_vma *          root;                 /*     0     8 */
	struct rw_semaphore        rwsem;                /*     8    40 */
	atomic_t                   refcount;             /*    48     4 */
	unsigned int               kabi_reserved_degree; /*    52     4 */
	struct anon_vma *          parent;               /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct rb_root_cached      rb_root;              /*    64    16 */
	long unsigned int          num_children;         /*    80     8 */
	long unsigned int          num_active_vmas;      /*    88     8 */

	/* size: 96, cachelines: 2, members: 8 */
	/* last cacheline: 32 bytes */
};
struct mempolicy {
	atomic_t                   refcnt;               /*     0     4 */
	short unsigned int         mode;                 /*     4     2 */
	short unsigned int         flags;                /*     6     2 */
	union {
		short int          preferred_node;       /*     8     2 */
		nodemask_t         nodes;                /*     8   128 */
	} v;                                             /*     8   128 */
	/* --- cacheline 2 boundary (128 bytes) was 8 bytes ago --- */
	union {
		nodemask_t         cpuset_mems_allowed;  /*   136   128 */
		nodemask_t         user_nodemask;        /*   136   128 */
	} w;                                             /*   136   128 */

	/* size: 264, cachelines: 5, members: 5 */
	/* last cacheline: 8 bytes */
};
struct linux_binfmt {
	struct list_head           lh;                   /*     0    16 */
	struct module *            module;               /*    16     8 */
	int                        (*load_binary)(struct linux_binprm *); /*    24     8 */
	int                        (*load_shlib)(struct file *); /*    32     8 */
	int                        (*core_dump)(struct coredump_params *); /*    40     8 */
	long unsigned int          min_coredump;         /*    48     8 */

	/* size: 56, cachelines: 1, members: 6 */
	/* last cacheline: 56 bytes */
};
struct free_area {
	struct list_head           free_list[5];         /*     0    80 */
	/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
	long unsigned int          nr_free;              /*    80     8 */

	/* size: 88, cachelines: 2, members: 2 */
	/* last cacheline: 24 bytes */
};
struct zone_padding {
	char                       x[0];                 /*     0     0 */

	/* size: 0, cachelines: 0, members: 1 */
} __attribute__((__aligned__(64)));
struct lruvec {
	struct list_head           lists[5];             /*     0    80 */
	/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
	spinlock_t                 lru_lock;             /*    80     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          anon_cost;            /*    88     8 */
	long unsigned int          file_cost;            /*    96     8 */
	atomic_long_t              nonresident_age;      /*   104     8 */
	long unsigned int          refaults[2];          /*   112    16 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	long unsigned int          flags;                /*   128     8 */
	struct pglist_data *       pgdat;                /*   136     8 */

	/* size: 144, cachelines: 3, members: 8 */
	/* sum members: 140, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct pglist_data {
	struct zone                node_zones[5] __attribute__((__aligned__(64))); /*     0  7360 */
	/* --- cacheline 115 boundary (7360 bytes) --- */
	struct zonelist            node_zonelists[2];    /*  7360 163872 */
	/* --- cacheline 2675 boundary (171200 bytes) was 32 bytes ago --- */
	int                        nr_zones;             /* 171232     4 */
	spinlock_t                 node_size_lock;       /* 171236     4 */
	long unsigned int          node_start_pfn;       /* 171240     8 */
	long unsigned int          node_present_pages;   /* 171248     8 */
	long unsigned int          node_spanned_pages;   /* 171256     8 */
	/* --- cacheline 2676 boundary (171264 bytes) --- */
	int                        node_id;              /* 171264     4 */

	/* XXX 4 bytes hole, try to pack */

	wait_queue_head_t          kswapd_wait;          /* 171272    24 */
	wait_queue_head_t          pfmemalloc_wait;      /* 171296    24 */
	struct task_struct *       kswapd;               /* 171320     8 */
	/* --- cacheline 2677 boundary (171328 bytes) --- */
	int                        kswapd_order;         /* 171328     4 */
	enum zone_type             kswapd_highest_zoneidx; /* 171332     4 */
	int                        kswapd_failures;      /* 171336     4 */
	int                        kcompactd_max_order;  /* 171340     4 */
	enum zone_type             kcompactd_highest_zoneidx; /* 171344     4 */

	/* XXX 4 bytes hole, try to pack */

	wait_queue_head_t          kcompactd_wait;       /* 171352    24 */
	struct task_struct *       kcompactd;            /* 171376     8 */
	long unsigned int          totalreserve_pages;   /* 171384     8 */
	/* --- cacheline 2678 boundary (171392 bytes) --- */
	long unsigned int          min_unmapped_pages;   /* 171392     8 */
	long unsigned int          min_slab_pages;       /* 171400     8 */

	/* XXX 48 bytes hole, try to pack */

	/* --- cacheline 2679 boundary (171456 bytes) --- */
	struct zone_padding        _pad1_ __attribute__((__aligned__(64))); /* 171456     0 */
	long unsigned int          first_deferred_pfn;   /* 171456     8 */
	struct deferred_split      deferred_split_queue; /* 171464    32 */
	struct lruvec              __lruvec;             /* 171496   144 */
	/* --- cacheline 2681 boundary (171584 bytes) was 56 bytes ago --- */
	long unsigned int          flags;                /* 171640     8 */
	/* --- cacheline 2682 boundary (171648 bytes) --- */
	struct zone_padding        _pad2_ __attribute__((__aligned__(64))); /* 171648     0 */
	u64                        kabi_reserved1;       /* 171648     8 */
	u64                        kabi_reserved2;       /* 171656     8 */
	u64                        kabi_reserved3;       /* 171664     8 */
	struct per_cpu_nodestat *  per_cpu_nodestats;    /* 171672     8 */
	atomic_long_t              vm_stat[37];          /* 171680   296 */

	/* size: 172032, cachelines: 2688, members: 32 */
	/* sum members: 171920, holes: 3, sum holes: 56 */
	/* padding: 56 */
	/* forced alignments: 3, forced holes: 1, sum forced holes: 48 */
} __attribute__((__aligned__(64)));
struct per_cpu_pages {
	int                        count;                /*     0     4 */
	int                        high;                 /*     4     4 */
	int                        batch;                /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           lists[3];             /*    16    48 */

	/* size: 64, cachelines: 1, members: 4 */
	/* sum members: 60, holes: 1, sum holes: 4 */
};
struct per_cpu_pageset {
	struct per_cpu_pages       pcp;                  /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	s8                         expire;               /*    64     1 */

	/* XXX 1 byte hole, try to pack */

	u16                        vm_numa_stat_diff[6]; /*    66    12 */
	s8                         stat_threshold;       /*    78     1 */
	s8                         vm_stat_diff[12];     /*    79    12 */

	/* size: 96, cachelines: 2, members: 5 */
	/* sum members: 90, holes: 1, sum holes: 1 */
	/* padding: 5 */
	/* last cacheline: 32 bytes */
};
struct per_cpu_nodestat {
	s8                         stat_threshold;       /*     0     1 */
	s8                         vm_node_stat_diff[37]; /*     1    37 */

	/* size: 38, cachelines: 1, members: 2 */
	/* last cacheline: 38 bytes */
};
struct zone {
	long unsigned int          _watermark[3];        /*     0    24 */
	long unsigned int          watermark_boost;      /*    24     8 */
	long unsigned int          nr_reserved_highatomic; /*    32     8 */
	long int                   lowmem_reserve[5];    /*    40    40 */
	/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
	int                        node;                 /*    80     4 */

	/* XXX 4 bytes hole, try to pack */

	struct pglist_data *       zone_pgdat;           /*    88     8 */
	struct per_cpu_pageset *   pageset;              /*    96     8 */
	int                        pageset_high;         /*   104     4 */
	int                        pageset_batch;        /*   108     4 */
	long unsigned int          zone_start_pfn;       /*   112     8 */
	atomic_long_t              managed_pages;        /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	long unsigned int          spanned_pages;        /*   128     8 */
	long unsigned int          present_pages;        /*   136     8 */
	const char  *              name;                 /*   144     8 */
	long unsigned int          nr_isolate_pageblock; /*   152     8 */
	seqlock_t                  span_seqlock;         /*   160     8 */
	int                        initialized;          /*   168     4 */

	/* XXX 20 bytes hole, try to pack */

	/* --- cacheline 3 boundary (192 bytes) --- */
	struct zone_padding        _pad1_ __attribute__((__aligned__(64))); /*   192     0 */
	struct free_area           free_area[11];        /*   192   968 */
	/* --- cacheline 18 boundary (1152 bytes) was 8 bytes ago --- */
	long unsigned int          flags;                /*  1160     8 */
	spinlock_t                 lock;                 /*  1168     4 */

	/* XXX 44 bytes hole, try to pack */

	/* --- cacheline 19 boundary (1216 bytes) --- */
	struct zone_padding        _pad2_ __attribute__((__aligned__(64))); /*  1216     0 */
	long unsigned int          percpu_drift_mark;    /*  1216     8 */
	long unsigned int          compact_cached_free_pfn; /*  1224     8 */
	long unsigned int          compact_cached_migrate_pfn[2]; /*  1232    16 */
	long unsigned int          compact_init_migrate_pfn; /*  1248     8 */
	long unsigned int          compact_init_free_pfn; /*  1256     8 */
	unsigned int               compact_considered;   /*  1264     4 */
	unsigned int               compact_defer_shift;  /*  1268     4 */
	int                        compact_order_failed; /*  1272     4 */
	bool                       compact_blockskip_flush; /*  1276     1 */
	bool                       contiguous;           /*  1277     1 */

	/* XXX 2 bytes hole, try to pack */

	/* --- cacheline 20 boundary (1280 bytes) --- */
	struct zone_padding        _pad3_ __attribute__((__aligned__(64))); /*  1280     0 */
	u64                        kabi_reserved1;       /*  1280     8 */
	u64                        kabi_reserved2;       /*  1288     8 */
	u64                        kabi_reserved3;       /*  1296     8 */
	u64                        kabi_reserved4;       /*  1304     8 */
	atomic_long_t              vm_stat[12];          /*  1312    96 */
	/* --- cacheline 22 boundary (1408 bytes) --- */
	atomic_long_t              vm_numa_stat[6];      /*  1408    48 */

	/* size: 1472, cachelines: 23, members: 39 */
	/* sum members: 1386, holes: 4, sum holes: 70 */
	/* padding: 16 */
	/* forced alignments: 3, forced holes: 3, sum forced holes: 66 */
} __attribute__((__aligned__(64)));
struct zoneref {
	struct zone *              zone;                 /*     0     8 */
	int                        zone_idx;             /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct zonelist {
	struct zoneref             _zonerefs[5121];      /*     0 81936 */

	/* size: 81936, cachelines: 1281, members: 1 */
	/* last cacheline: 16 bytes */
};
struct deferred_split {
	spinlock_t                 split_queue_lock;     /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           split_queue;          /*     8    16 */
	long unsigned int          split_queue_len;      /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct irq_domain {
	struct list_head           link;                 /*     0    16 */
	const char  *              name;                 /*    16     8 */
	const struct irq_domain_ops  * ops;              /*    24     8 */
	void *                     host_data;            /*    32     8 */
	unsigned int               flags;                /*    40     4 */
	unsigned int               mapcount;             /*    44     4 */
	struct fwnode_handle *     fwnode;               /*    48     8 */
	enum irq_domain_bus_token  bus_token;            /*    56     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	struct irq_domain_chip_generic * gc;             /*    64     8 */
	struct irq_domain *        parent;               /*    72     8 */
	irq_hw_number_t            hwirq_max;            /*    80     8 */
	unsigned int               revmap_direct_max_irq; /*    88     4 */
	unsigned int               revmap_size;          /*    92     4 */
	struct xarray              revmap_tree;          /*    96    16 */
	struct mutex               revmap_tree_mutex;    /*   112    32 */
	/* --- cacheline 2 boundary (128 bytes) was 16 bytes ago --- */
	u64                        kabi_reserved1;       /*   144     8 */
	unsigned int               linear_revmap[];      /*   152     0 */

	/* size: 152, cachelines: 3, members: 17 */
	/* sum members: 148, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct ctl_table {
	const char  *              procname;             /*     0     8 */
	void *                     data;                 /*     8     8 */
	int                        maxlen;               /*    16     4 */
	umode_t                    mode;                 /*    20     2 */

	/* XXX 2 bytes hole, try to pack */

	struct ctl_table *         child;                /*    24     8 */
	proc_handler *             proc_handler;         /*    32     8 */
	struct ctl_table_poll *    poll;                 /*    40     8 */
	void *                     extra1;               /*    48     8 */
	void *                     extra2;               /*    56     8 */

	/* size: 64, cachelines: 1, members: 9 */
	/* sum members: 62, holes: 1, sum holes: 2 */
};
struct ctl_table_poll {
	atomic_t                   event;                /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	wait_queue_head_t          wait;                 /*     8    24 */

	/* size: 32, cachelines: 1, members: 2 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct ctl_node {
	struct rb_node             node __attribute__((__aligned__(8))); /*     0    24 */
	struct ctl_table_header *  header;               /*    24     8 */

	/* size: 32, cachelines: 1, members: 2 */
	/* forced alignments: 1 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct ctl_table_header {
	union {
		struct {
			struct ctl_table * ctl_table;    /*     0     8 */
			int        used;                 /*     8     4 */
			int        count;                /*    12     4 */
			int        nreg;                 /*    16     4 */
		};                                       /*     0    24 */
		struct callback_head rcu __attribute__((__aligned__(8))); /*     0    16 */
	} __attribute__((__aligned__(8)));               /*     0    24 */
	struct completion *        unregistering;        /*    24     8 */
	struct ctl_table *         ctl_table_arg;        /*    32     8 */
	struct ctl_table_root *    root;                 /*    40     8 */
	struct ctl_table_set *     set;                  /*    48     8 */
	struct ctl_dir *           parent;               /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct ctl_node *          node;                 /*    64     8 */
	struct hlist_head          inodes;               /*    72     8 */

	/* size: 80, cachelines: 2, members: 8 */
	/* forced alignments: 1 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct ctl_table_root {
	struct ctl_table_set       default_set __attribute__((__aligned__(8))); /*     0    96 */
	/* --- cacheline 1 boundary (64 bytes) was 32 bytes ago --- */
	struct ctl_table_set *     (*lookup)(struct ctl_table_root *); /*    96     8 */
	void                       (*set_ownership)(struct ctl_table_header *, struct ctl_table *, kuid_t *, kgid_t *); /*   104     8 */
	int                        (*permissions)(struct ctl_table_header *, struct ctl_table *); /*   112     8 */

	/* size: 120, cachelines: 2, members: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 56 bytes */
} __attribute__((__aligned__(8)));
struct ctl_table_set {
	int                        (*is_seen)(struct ctl_table_set *); /*     0     8 */
	struct ctl_dir             dir __attribute__((__aligned__(8))); /*     8    88 */

	/* size: 96, cachelines: 2, members: 2 */
	/* forced alignments: 1 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct ctl_dir {
	struct ctl_table_header    header __attribute__((__aligned__(8))); /*     0    80 */
	/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
	struct rb_root             root;                 /*    80     8 */

	/* size: 88, cachelines: 2, members: 2 */
	/* forced alignments: 1 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct elf64_sym {
	Elf64_Word                 st_name;              /*     0     4 */
	unsigned char              st_info;              /*     4     1 */
	unsigned char              st_other;             /*     5     1 */
	Elf64_Half                 st_shndx;             /*     6     2 */
	Elf64_Addr                 st_value;             /*     8     8 */
	Elf64_Xword                st_size;              /*    16     8 */

	/* size: 24, cachelines: 1, members: 6 */
	/* last cacheline: 24 bytes */
};
struct elf64_hdr {
	unsigned char              e_ident[16];          /*     0    16 */
	Elf64_Half                 e_type;               /*    16     2 */
	Elf64_Half                 e_machine;            /*    18     2 */
	Elf64_Word                 e_version;            /*    20     4 */
	Elf64_Addr                 e_entry;              /*    24     8 */
	Elf64_Off                  e_phoff;              /*    32     8 */
	Elf64_Off                  e_shoff;              /*    40     8 */
	Elf64_Word                 e_flags;              /*    48     4 */
	Elf64_Half                 e_ehsize;             /*    52     2 */
	Elf64_Half                 e_phentsize;          /*    54     2 */
	Elf64_Half                 e_phnum;              /*    56     2 */
	Elf64_Half                 e_shentsize;          /*    58     2 */
	Elf64_Half                 e_shnum;              /*    60     2 */
	Elf64_Half                 e_shstrndx;           /*    62     2 */

	/* size: 64, cachelines: 1, members: 14 */
};
struct elf64_shdr {
	Elf64_Word                 sh_name;              /*     0     4 */
	Elf64_Word                 sh_type;              /*     4     4 */
	Elf64_Xword                sh_flags;             /*     8     8 */
	Elf64_Addr                 sh_addr;              /*    16     8 */
	Elf64_Off                  sh_offset;            /*    24     8 */
	Elf64_Xword                sh_size;              /*    32     8 */
	Elf64_Word                 sh_link;              /*    40     4 */
	Elf64_Word                 sh_info;              /*    44     4 */
	Elf64_Xword                sh_addralign;         /*    48     8 */
	Elf64_Xword                sh_entsize;           /*    56     8 */

	/* size: 64, cachelines: 1, members: 10 */
};
struct idr {
	struct xarray              idr_rt;               /*     0    16 */
	unsigned int               idr_base;             /*    16     4 */
	unsigned int               idr_next;             /*    20     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct kernfs_elem_dir {
	long unsigned int          subdirs;              /*     0     8 */
	struct rb_root             children;             /*     8     8 */
	struct kernfs_root *       root;                 /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct kernfs_root {
	struct kernfs_node *       kn;                   /*     0     8 */
	unsigned int               flags;                /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct idr                 ino_idr;              /*    16    24 */
	u32                        last_id_lowbits;      /*    40     4 */
	u32                        id_highbits;          /*    44     4 */
	struct kernfs_syscall_ops * syscall_ops;         /*    48     8 */
	struct list_head           supers;               /*    56    16 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	wait_queue_head_t          deactivate_waitq;     /*    72    24 */

	/* size: 96, cachelines: 2, members: 8 */
	/* sum members: 92, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct kernfs_elem_symlink {
	struct kernfs_node *       target_kn;            /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct kernfs_node {
	atomic_t                   count;                /*     0     4 */
	atomic_t                   active;               /*     4     4 */
	struct kernfs_node *       parent;               /*     8     8 */
	const char  *              name;                 /*    16     8 */
	struct rb_node             rb __attribute__((__aligned__(8))); /*    24    24 */
	const void  *              ns;                   /*    48     8 */
	unsigned int               hash;                 /*    56     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	union {
		struct kernfs_elem_dir dir;              /*    64    24 */
		struct kernfs_elem_symlink symlink;      /*    64     8 */
		struct kernfs_elem_attr attr;            /*    64    32 */
	};                                               /*    64    32 */
	void *                     priv;                 /*    96     8 */
	u64                        id;                   /*   104     8 */
	short unsigned int         flags;                /*   112     2 */
	umode_t                    mode;                 /*   114     2 */

	/* XXX 4 bytes hole, try to pack */

	struct kernfs_iattrs *     iattr;                /*   120     8 */

	/* size: 128, cachelines: 2, members: 13 */
	/* sum members: 120, holes: 2, sum holes: 8 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(8)));
struct kernfs_elem_attr {
	const struct kernfs_ops  * ops;                  /*     0     8 */
	struct kernfs_open_node *  open;                 /*     8     8 */
	loff_t                     size;                 /*    16     8 */
	struct kernfs_node *       notify_next;          /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct kernfs_ops {
	int                        (*open)(struct kernfs_open_file *); /*     0     8 */
	void                       (*release)(struct kernfs_open_file *); /*     8     8 */
	int                        (*seq_show)(struct seq_file *, void *); /*    16     8 */
	void *                     (*seq_start)(struct seq_file *, loff_t *); /*    24     8 */
	void *                     (*seq_next)(struct seq_file *, void *, loff_t *); /*    32     8 */
	void                       (*seq_stop)(struct seq_file *, void *); /*    40     8 */
	ssize_t                    (*read)(struct kernfs_open_file *, char *, size_t, loff_t); /*    48     8 */
	size_t                     atomic_write_len;     /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	bool                       prealloc;             /*    64     1 */

	/* XXX 7 bytes hole, try to pack */

	ssize_t                    (*write)(struct kernfs_open_file *, char *, size_t, loff_t); /*    72     8 */
	__poll_t                   (*poll)(struct kernfs_open_file *, struct poll_table_struct *); /*    80     8 */
	int                        (*mmap)(struct kernfs_open_file *, struct vm_area_struct *); /*    88     8 */
	u64                        kabi_reserved1;       /*    96     8 */
	u64                        kabi_reserved2;       /*   104     8 */

	/* size: 112, cachelines: 2, members: 14 */
	/* sum members: 105, holes: 1, sum holes: 7 */
	/* last cacheline: 48 bytes */
};
struct kernfs_syscall_ops {
	int                        (*show_options)(struct seq_file *, struct kernfs_root *); /*     0     8 */
	int                        (*mkdir)(struct kernfs_node *, const char  *, umode_t); /*     8     8 */
	int                        (*rmdir)(struct kernfs_node *); /*    16     8 */
	int                        (*rename)(struct kernfs_node *, struct kernfs_node *, const char  *); /*    24     8 */
	int                        (*show_path)(struct seq_file *, struct kernfs_node *, struct kernfs_root *); /*    32     8 */
	u64                        kabi_reserved1;       /*    40     8 */
	u64                        kabi_reserved2;       /*    48     8 */
	u64                        kabi_reserved3;       /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u64                        kabi_reserved4;       /*    64     8 */

	/* size: 72, cachelines: 2, members: 9 */
	/* last cacheline: 8 bytes */
};
struct seq_file {
	char *                     buf;                  /*     0     8 */
	size_t                     size;                 /*     8     8 */
	size_t                     from;                 /*    16     8 */
	size_t                     count;                /*    24     8 */
	size_t                     pad_until;            /*    32     8 */
	loff_t                     index;                /*    40     8 */
	loff_t                     read_pos;             /*    48     8 */
	struct mutex               lock;                 /*    56    32 */
	/* --- cacheline 1 boundary (64 bytes) was 24 bytes ago --- */
	const struct seq_operations  * op;               /*    88     8 */
	int                        poll_event;           /*    96     4 */

	/* XXX 4 bytes hole, try to pack */

	const struct file  *       file;                 /*   104     8 */
	void *                     private;              /*   112     8 */

	/* size: 120, cachelines: 2, members: 12 */
	/* sum members: 116, holes: 1, sum holes: 4 */
	/* last cacheline: 56 bytes */
};
struct kernfs_open_file {
	struct kernfs_node *       kn;                   /*     0     8 */
	struct file *              file;                 /*     8     8 */
	struct seq_file *          seq_file;             /*    16     8 */
	void *                     priv;                 /*    24     8 */
	struct mutex               mutex;                /*    32    32 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct mutex               prealloc_mutex;       /*    64    32 */
	int                        event;                /*    96     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           list;                 /*   104    16 */
	char *                     prealloc_buf;         /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	size_t                     atomic_write_len;     /*   128     8 */
	bool                       mmapped:1;            /*   136: 0  1 */
	bool                       released:1;           /*   136: 1  1 */

	/* XXX 6 bits hole, try to pack */
	/* XXX 7 bytes hole, try to pack */

	const struct vm_operations_struct  * vm_ops;     /*   144     8 */

	/* size: 152, cachelines: 3, members: 13 */
	/* sum members: 140, holes: 2, sum holes: 11 */
	/* sum bitfield members: 2 bits, bit holes: 1, sum bit holes: 6 bits */
	/* last cacheline: 24 bytes */
};
struct poll_table_struct {
	poll_queue_proc            _qproc;               /*     0     8 */
	__poll_t                   _key;                 /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct kobj_ns_type_operations {
	enum kobj_ns_type          type;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	bool                       (*current_may_mount)(void); /*     8     8 */
	void *                     (*grab_current_ns)(void); /*    16     8 */
	const void  *              (*netlink_ns)(struct sock *); /*    24     8 */
	const void  *              (*initial_ns)(void);  /*    32     8 */
	void                       (*drop_ns)(void *);   /*    40     8 */

	/* size: 48, cachelines: 1, members: 6 */
	/* sum members: 44, holes: 1, sum holes: 4 */
	/* last cacheline: 48 bytes */
};
struct attribute {
	const char  *              name;                 /*     0     8 */
	umode_t                    mode;                 /*     8     2 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 6 */
	/* last cacheline: 16 bytes */
};
struct kobject {
	const char  *              name;                 /*     0     8 */
	struct list_head           entry;                /*     8    16 */
	struct kobject *           parent;               /*    24     8 */
	struct kset *              kset;                 /*    32     8 */
	struct kobj_type *         ktype;                /*    40     8 */
	struct kernfs_node *       sd;                   /*    48     8 */
	struct kref                kref;                 /*    56     4 */
	unsigned int               state_initialized:1;  /*    60: 0  4 */
	unsigned int               state_in_sysfs:1;     /*    60: 1  4 */
	unsigned int               state_add_uevent_sent:1; /*    60: 2  4 */
	unsigned int               state_remove_uevent_sent:1; /*    60: 3  4 */
	unsigned int               uevent_suppress:1;    /*    60: 4  4 */

	/* XXX 27 bits hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	u64                        kabi_reserved1;       /*    64     8 */
	u64                        kabi_reserved2;       /*    72     8 */
	u64                        kabi_reserved3;       /*    80     8 */
	u64                        kabi_reserved4;       /*    88     8 */

	/* size: 96, cachelines: 2, members: 16 */
	/* sum members: 92 */
	/* sum bitfield members: 5 bits, bit holes: 1, sum bit holes: 27 bits */
	/* last cacheline: 32 bytes */
};
struct bin_attribute {
	struct attribute           attr;                 /*     0    16 */

	/* XXX last struct has 6 bytes of padding */

	size_t                     size;                 /*    16     8 */
	void *                     private;              /*    24     8 */
	ssize_t                    (*read)(struct file *, struct kobject *, struct bin_attribute *, char *, loff_t, size_t); /*    32     8 */
	ssize_t                    (*write)(struct file *, struct kobject *, struct bin_attribute *, char *, loff_t, size_t); /*    40     8 */
	int                        (*mmap)(struct file *, struct kobject *, struct bin_attribute *, struct vm_area_struct *); /*    48     8 */
	u64                        kabi_reserved1;       /*    56     8 */

	/* size: 64, cachelines: 1, members: 7 */
	/* paddings: 1, sum paddings: 6 */
};
struct sysfs_ops {
	ssize_t                    (*show)(struct kobject *, struct attribute *, char *); /*     0     8 */
	ssize_t                    (*store)(struct kobject *, struct attribute *, const char  *, size_t); /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct kref {
	refcount_t                 refcount;             /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct kset {
	struct list_head           list;                 /*     0    16 */
	spinlock_t                 list_lock;            /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	struct kobject             kobj;                 /*    24    96 */
	/* --- cacheline 1 boundary (64 bytes) was 56 bytes ago --- */
	const struct kset_uevent_ops  * uevent_ops;      /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	u64                        kabi_reserved1;       /*   128     8 */
	u64                        kabi_reserved2;       /*   136     8 */
	u64                        kabi_reserved3;       /*   144     8 */
	u64                        kabi_reserved4;       /*   152     8 */

	/* size: 160, cachelines: 3, members: 8 */
	/* sum members: 156, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct kobj_type {
	void                       (*release)(struct kobject *); /*     0     8 */
	const struct sysfs_ops  *  sysfs_ops;            /*     8     8 */
	struct attribute * *       default_attrs;        /*    16     8 */
	const struct attribute_group  * * default_groups; /*    24     8 */
	const struct kobj_ns_type_operations  * (*child_ns_type)(struct kobject *); /*    32     8 */
	const void  *              (*namespace)(struct kobject *); /*    40     8 */
	void                       (*get_ownership)(struct kobject *, kuid_t *, kgid_t *); /*    48     8 */
	u64                        kabi_reserved1;       /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u64                        kabi_reserved2;       /*    64     8 */
	u64                        kabi_reserved3;       /*    72     8 */
	u64                        kabi_reserved4;       /*    80     8 */

	/* size: 88, cachelines: 2, members: 11 */
	/* last cacheline: 24 bytes */
};
struct kobj_uevent_env {
	char *                     argv[3];              /*     0    24 */
	char *                     envp[64];             /*    24   512 */
	/* --- cacheline 8 boundary (512 bytes) was 24 bytes ago --- */
	int                        envp_idx;             /*   536     4 */
	char                       buf[2048];            /*   540  2048 */
	/* --- cacheline 40 boundary (2560 bytes) was 28 bytes ago --- */
	int                        buflen;               /*  2588     4 */

	/* size: 2592, cachelines: 41, members: 5 */
	/* last cacheline: 32 bytes */
};
struct kset_uevent_ops {
	const int                  (*filter)(struct kset *, struct kobject *); /*     0     8 */
	const const char  *        (*name)(struct kset *, struct kobject *); /*     8     8 */
	const int                  (*uevent)(struct kset *, struct kobject *, struct kobj_uevent_env *); /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct kernel_param_ops {
	unsigned int               flags;                /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	int                        (*set)(const char  *, const struct kernel_param  *); /*     8     8 */
	int                        (*get)(char *, const struct kernel_param  *); /*    16     8 */
	void                       (*free)(void *);      /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct kernel_param {
	const char  *              name;                 /*     0     8 */
	struct module *            mod;                  /*     8     8 */
	const struct kernel_param_ops  * ops;            /*    16     8 */
	const u16                  perm;                 /*    24     2 */
	s8                         level;                /*    26     1 */
	u8                         flags;                /*    27     1 */

	/* XXX 4 bytes hole, try to pack */

	union {
		void *             arg;                  /*    32     8 */
		const struct kparam_string  * str;       /*    32     8 */
		const struct kparam_array  * arr;        /*    32     8 */
	};                                               /*    32     8 */

	/* size: 40, cachelines: 1, members: 7 */
	/* sum members: 36, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct kparam_string {
	unsigned int               maxlen;               /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	char *                     string;               /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct kparam_array {
	unsigned int               max;                  /*     0     4 */
	unsigned int               elemsize;             /*     4     4 */
	unsigned int *             num;                  /*     8     8 */
	const struct kernel_param_ops  * ops;            /*    16     8 */
	void *                     elem;                 /*    24     8 */

	/* size: 32, cachelines: 1, members: 5 */
	/* last cacheline: 32 bytes */
};
struct module {
	enum module_state          state;                /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           list;                 /*     8    16 */
	char                       name[56];             /*    24    56 */
	/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
	struct module_kobject      mkobj;                /*    80   128 */
	/* --- cacheline 3 boundary (192 bytes) was 16 bytes ago --- */
	struct module_attribute *  modinfo_attrs;        /*   208     8 */
	const char  *              version;              /*   216     8 */
	const char  *              srcversion;           /*   224     8 */
	struct kobject *           holders_dir;          /*   232     8 */
	const struct kernel_symbol  * syms;              /*   240     8 */
	const s32  *               crcs;                 /*   248     8 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	unsigned int               num_syms;             /*   256     4 */

	/* XXX 4 bytes hole, try to pack */

	struct mutex               param_lock;           /*   264    32 */
	struct kernel_param *      kp;                   /*   296     8 */
	unsigned int               num_kp;               /*   304     4 */
	unsigned int               num_gpl_syms;         /*   308     4 */
	const struct kernel_symbol  * gpl_syms;          /*   312     8 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	const s32  *               gpl_crcs;             /*   320     8 */
	bool                       using_gplonly_symbols; /*   328     1 */
	bool                       sig_ok;               /*   329     1 */
	bool                       async_probe_requested; /*   330     1 */

	/* XXX 5 bytes hole, try to pack */

	const struct kernel_symbol  * gpl_future_syms;   /*   336     8 */
	const s32  *               gpl_future_crcs;      /*   344     8 */
	unsigned int               num_gpl_future_syms;  /*   352     4 */
	unsigned int               num_exentries;        /*   356     4 */
	struct exception_table_entry * extable;          /*   360     8 */
	int                        (*init)(void);        /*   368     8 */

	/* XXX 8 bytes hole, try to pack */

	/* --- cacheline 6 boundary (384 bytes) --- */
	struct module_layout       core_layout __attribute__((__aligned__(64))); /*   384    80 */
	/* --- cacheline 7 boundary (448 bytes) was 16 bytes ago --- */
	struct module_layout       init_layout __attribute__((__aligned__(8))); /*   464    80 */
	/* --- cacheline 8 boundary (512 bytes) was 32 bytes ago --- */
	struct mod_arch_specific   arch;                 /*   544    24 */
	long unsigned int          taints;               /*   568     8 */
	/* --- cacheline 9 boundary (576 bytes) --- */
	unsigned int               num_bugs;             /*   576     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           bug_list;             /*   584    16 */
	struct bug_entry *         bug_table;            /*   600     8 */
	struct mod_kallsyms *      kallsyms;             /*   608     8 */
	struct mod_kallsyms        core_kallsyms;        /*   616    32 */
	/* --- cacheline 10 boundary (640 bytes) was 8 bytes ago --- */
	struct module_sect_attrs * sect_attrs;           /*   648     8 */
	struct module_notes_attrs * notes_attrs;         /*   656     8 */
	char *                     args;                 /*   664     8 */
	void *                     percpu;               /*   672     8 */
	unsigned int               percpu_size;          /*   680     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     noinstr_text_start;   /*   688     8 */
	unsigned int               noinstr_text_size;    /*   696     4 */
	unsigned int               num_tracepoints;      /*   700     4 */
	/* --- cacheline 11 boundary (704 bytes) --- */
	tracepoint_ptr_t *         tracepoints_ptrs;     /*   704     8 */
	unsigned int               num_srcu_structs;     /*   712     4 */

	/* XXX 4 bytes hole, try to pack */

	struct srcu_struct * *     srcu_struct_ptrs;     /*   720     8 */
	unsigned int               num_bpf_raw_events;   /*   728     4 */

	/* XXX 4 bytes hole, try to pack */

	struct bpf_raw_event_map * bpf_raw_events;       /*   736     8 */
	struct jump_entry *        jump_entries;         /*   744     8 */
	unsigned int               num_jump_entries;     /*   752     4 */
	unsigned int               num_trace_bprintk_fmt; /*   756     4 */
	const char  * *            trace_bprintk_fmt_start; /*   760     8 */
	/* --- cacheline 12 boundary (768 bytes) --- */
	struct trace_event_call * * trace_events;        /*   768     8 */
	unsigned int               num_trace_events;     /*   776     4 */

	/* XXX 4 bytes hole, try to pack */

	struct trace_eval_map * *  trace_evals;          /*   784     8 */
	unsigned int               num_trace_evals;      /*   792     4 */
	unsigned int               num_ftrace_callsites; /*   796     4 */
	long unsigned int *        ftrace_callsites;     /*   800     8 */
	void *                     kprobes_text_start;   /*   808     8 */
	unsigned int               kprobes_text_size;    /*   816     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int *        kprobe_blacklist;     /*   824     8 */
	/* --- cacheline 13 boundary (832 bytes) --- */
	unsigned int               num_kprobe_blacklist; /*   832     4 */
	int                        num_static_call_sites; /*   836     4 */
	struct static_call_site *  static_call_sites;    /*   840     8 */
	bool                       klp;                  /*   848     1 */
	bool                       klp_alive;            /*   849     1 */

	/* XXX 6 bytes hole, try to pack */

	struct klp_modinfo *       klp_info;             /*   856     8 */
	enum MODULE_KLP_REL_STATE  klp_rel_state;        /*   864     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           source_list;          /*   872    16 */
	struct list_head           target_list;          /*   888    16 */
	/* --- cacheline 14 boundary (896 bytes) was 8 bytes ago --- */
	void                       (*exit)(void);        /*   904     8 */
	atomic_t                   refcnt;               /*   912     4 */

	/* XXX 4 bytes hole, try to pack */

	struct error_injection_entry * ei_funcs;         /*   920     8 */
	unsigned int               num_ei_funcs;         /*   928     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        kabi_reserved1;       /*   936     8 */
	u64                        kabi_reserved2;       /*   944     8 */
	u64                        kabi_reserved3;       /*   952     8 */
	/* --- cacheline 15 boundary (960 bytes) --- */
	u64                        kabi_reserved4;       /*   960     8 */

	/* size: 1024, cachelines: 16, members: 78 */
	/* sum members: 905, holes: 14, sum holes: 63 */
	/* padding: 56 */
	/* forced alignments: 2, forced holes: 1, sum forced holes: 8 */
} __attribute__((__aligned__(64)));
struct latch_tree_node {
	struct rb_node             node[2] __attribute__((__aligned__(8))); /*     0    48 */

	/* size: 48, cachelines: 1, members: 1 */
	/* forced alignments: 1 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct error_injection_entry {
	long unsigned int          addr;                 /*     0     8 */
	int                        etype;                /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct mod_arch_specific {
	unsigned int               num_orcs;             /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	int *                      orc_unwind_ip;        /*     8     8 */
	struct orc_entry *         orc_unwind;           /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct module_kobject {
	struct kobject             kobj;                 /*     0    96 */
	/* --- cacheline 1 boundary (64 bytes) was 32 bytes ago --- */
	struct module *            mod;                  /*    96     8 */
	struct kobject *           drivers_dir;          /*   104     8 */
	struct module_param_attrs * mp;                  /*   112     8 */
	struct completion *        kobj_completion;      /*   120     8 */

	/* size: 128, cachelines: 2, members: 5 */
};
struct module_attribute {
	struct attribute           attr;                 /*     0    16 */

	/* XXX last struct has 6 bytes of padding */

	ssize_t                    (*show)(struct module_attribute *, struct module_kobject *, char *); /*    16     8 */
	ssize_t                    (*store)(struct module_attribute *, struct module_kobject *, const char  *, size_t); /*    24     8 */
	void                       (*setup)(struct module *, const char  *); /*    32     8 */
	int                        (*test)(struct module *); /*    40     8 */
	void                       (*free)(struct module *); /*    48     8 */

	/* size: 56, cachelines: 1, members: 6 */
	/* paddings: 1, sum paddings: 6 */
	/* last cacheline: 56 bytes */
};
struct mod_tree_node {
	struct module *            mod;                  /*     0     8 */
	struct latch_tree_node     node __attribute__((__aligned__(8))); /*     8    48 */

	/* size: 56, cachelines: 1, members: 2 */
	/* forced alignments: 1 */
	/* last cacheline: 56 bytes */
} __attribute__((__aligned__(8)));
struct module_layout {
	void *                     base;                 /*     0     8 */
	unsigned int               size;                 /*     8     4 */
	unsigned int               text_size;            /*    12     4 */
	unsigned int               ro_size;              /*    16     4 */
	unsigned int               ro_after_init_size;   /*    20     4 */
	struct mod_tree_node       mtn __attribute__((__aligned__(8))); /*    24    56 */

	/* size: 80, cachelines: 2, members: 6 */
	/* forced alignments: 1 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct mod_kallsyms {
	Elf64_Sym *                symtab;               /*     0     8 */
	unsigned int               num_symtab;           /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	char *                     strtab;               /*    16     8 */
	char *                     typetab;              /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct klp_modinfo {
	Elf64_Ehdr                 hdr;                  /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	Elf64_Shdr *               sechdrs;              /*    64     8 */
	char *                     secstrings;           /*    72     8 */
	unsigned int               symndx;               /*    80     4 */

	/* size: 88, cachelines: 2, members: 4 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct exception_table_entry {
	int                        insn;                 /*     0     4 */
	int                        fixup;                /*     4     4 */
	int                        handler;              /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct trace_event_call {
	struct list_head           list;                 /*     0    16 */
	struct trace_event_class * class;                /*    16     8 */
	union {
		char *             name;                 /*    24     8 */
		struct tracepoint * tp;                  /*    24     8 */
	};                                               /*    24     8 */
	struct trace_event         event;                /*    32    48 */
	/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
	char *                     print_fmt;            /*    80     8 */
	struct event_filter *      filter;               /*    88     8 */
	void *                     mod;                  /*    96     8 */
	void *                     data;                 /*   104     8 */
	int                        flags;                /*   112     4 */
	int                        perf_refcount;        /*   116     4 */
	struct hlist_head *        perf_events;          /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct bpf_prog_array *    prog_array;           /*   128     8 */
	int                        (*perf_perm)(struct trace_event_call *, struct perf_event *); /*   136     8 */

	/* size: 144, cachelines: 3, members: 13 */
	/* last cacheline: 16 bytes */
};
struct trace_eval_map {
	const char  *              system;               /*     0     8 */
	const char  *              eval_string;          /*     8     8 */
	long unsigned int          eval_value;           /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct mem_cgroup {
	struct cgroup_subsys_state css __attribute__((__aligned__(8))); /*     0   296 */
	/* --- cacheline 4 boundary (256 bytes) was 40 bytes ago --- */
	struct mem_cgroup_id       id;                   /*   296    16 */
	struct page_counter        memory;               /*   312   112 */
	/* --- cacheline 6 boundary (384 bytes) was 40 bytes ago --- */
	union {
		struct page_counter swap;                /*   424   112 */
		struct page_counter memsw;               /*   424   112 */
	};                                               /*   424   112 */
	/* --- cacheline 8 boundary (512 bytes) was 24 bytes ago --- */
	struct page_counter        kmem;                 /*   536   112 */
	/* --- cacheline 10 boundary (640 bytes) was 8 bytes ago --- */
	struct page_counter        tcpmem;               /*   648   112 */
	/* --- cacheline 11 boundary (704 bytes) was 56 bytes ago --- */
	struct work_struct         high_work;            /*   760    64 */
	/* --- cacheline 12 boundary (768 bytes) was 56 bytes ago --- */
	long unsigned int          soft_limit;           /*   824     8 */
	/* --- cacheline 13 boundary (832 bytes) --- */
	struct vmpressure          vmpressure;           /*   832   152 */
	/* --- cacheline 15 boundary (960 bytes) was 24 bytes ago --- */
	bool                       use_hierarchy;        /*   984     1 */
	bool                       oom_group;            /*   985     1 */
	bool                       oom_lock;             /*   986     1 */

	/* XXX 1 byte hole, try to pack */

	int                        under_oom;            /*   988     4 */
	int                        swappiness;           /*   992     4 */
	int                        oom_kill_disable;     /*   996     4 */
	struct cgroup_file         events_file;          /*  1000   104 */
	/* --- cacheline 17 boundary (1088 bytes) was 16 bytes ago --- */
	struct cgroup_file         events_local_file;    /*  1104   104 */
	/* --- cacheline 18 boundary (1152 bytes) was 56 bytes ago --- */
	struct cgroup_file         swap_events_file;     /*  1208   104 */
	/* --- cacheline 20 boundary (1280 bytes) was 32 bytes ago --- */
	struct mutex               thresholds_lock;      /*  1312    32 */
	/* --- cacheline 21 boundary (1344 bytes) --- */
	struct mem_cgroup_thresholds thresholds;         /*  1344    16 */
	struct mem_cgroup_thresholds memsw_thresholds;   /*  1360    16 */
	struct list_head           oom_notify;           /*  1376    16 */
	long unsigned int          move_charge_at_immigrate; /*  1392     8 */
	spinlock_t                 move_lock;            /*  1400     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 22 boundary (1408 bytes) --- */
	long unsigned int          move_lock_flags;      /*  1408     8 */

	/* XXX 56 bytes hole, try to pack */

	/* --- cacheline 23 boundary (1472 bytes) --- */
	struct memcg_padding       _pad1_ __attribute__((__aligned__(64))); /*  1472     0 */
	struct memcg_vmstats       vmstats;              /*  1472  2176 */
	/* --- cacheline 57 boundary (3648 bytes) --- */
	atomic_long_t              memory_events[8];     /*  3648    64 */
	/* --- cacheline 58 boundary (3712 bytes) --- */
	atomic_long_t              memory_events_local[8]; /*  3712    64 */
	/* --- cacheline 59 boundary (3776 bytes) --- */
	long unsigned int          socket_pressure;      /*  3776     8 */
	bool                       tcpmem_active;        /*  3784     1 */

	/* XXX 3 bytes hole, try to pack */

	int                        tcpmem_pressure;      /*  3788     4 */
	int                        memcg_priority;       /*  3792     4 */
	int                        kmemcg_id;            /*  3796     4 */
	enum memcg_kmem_state      kmem_state;           /*  3800     4 */

	/* XXX 4 bytes hole, try to pack */

	struct obj_cgroup *        objcg;                /*  3808     8 */
	struct list_head           objcg_list;           /*  3816    16 */

	/* XXX 8 bytes hole, try to pack */

	/* --- cacheline 60 boundary (3840 bytes) --- */
	struct memcg_padding       _pad2_ __attribute__((__aligned__(64))); /*  3840     0 */
	atomic_t                   moving_account;       /*  3840     4 */

	/* XXX 4 bytes hole, try to pack */

	struct task_struct *       move_lock_task;       /*  3848     8 */
	struct memcg_vmstats_percpu * vmstats_percpu;    /*  3856     8 */
	struct list_head           cgwb_list;            /*  3864    16 */
	struct wb_domain           cgwb_domain;          /*  3880   152 */
	/* --- cacheline 63 boundary (4032 bytes) --- */
	struct memcg_cgwb_frn      cgwb_frn[4];          /*  4032   160 */
	/* --- cacheline 65 boundary (4160 bytes) was 32 bytes ago --- */
	struct list_head           event_list;           /*  4192    16 */
	spinlock_t                 event_list_lock;      /*  4208     4 */

	/* XXX 4 bytes hole, try to pack */

	struct deferred_split      deferred_split_queue; /*  4216    32 */
	/* --- cacheline 66 boundary (4224 bytes) was 24 bytes ago --- */
	struct dhugetlb_pool *     hpool;                /*  4248     8 */
	int                        high_async_ratio;     /*  4256     4 */
	bool                       high_async_reclaim;   /*  4260     1 */

	/* XXX 3 bytes hole, try to pack */

	union {
		struct cgroup_subsys_state * wb_blk_css; /*  4264     8 */
		struct {
			u64        kabi_reserved2;       /*  4264     8 */
		} kabi_hidden_385;                       /*  4264     8 */
		union {
		};                                       /*  4264     0 */
	};                                               /*  4264     8 */
	struct list_head           memcg_node;           /*  4272    16 */
	/* --- cacheline 67 boundary (4288 bytes) --- */
	u64                        kabi_reserved5;       /*  4288     8 */
	u64                        kabi_reserved6;       /*  4296     8 */
	u64                        kabi_reserved7;       /*  4304     8 */
	u64                        kabi_reserved8;       /*  4312     8 */
	struct mem_cgroup_per_node * nodeinfo[];         /*  4320     0 */

	/* size: 4352, cachelines: 68, members: 57 */
	/* sum members: 4233, holes: 9, sum holes: 87 */
	/* padding: 32 */
	/* forced alignments: 3, forced holes: 2, sum forced holes: 64 */
} __attribute__((__aligned__(64)));
struct pid_namespace {
	struct kref                kref;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct idr                 idr;                  /*     8    24 */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*    32    16 */
	unsigned int               pid_allocated;        /*    48     4 */

	/* XXX 4 bytes hole, try to pack */

	struct task_struct *       child_reaper;         /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct kmem_cache *        pid_cachep;           /*    64     8 */
	unsigned int               level;                /*    72     4 */

	/* XXX 4 bytes hole, try to pack */

	struct pid_namespace *     parent;               /*    80     8 */
	struct fs_pin *            bacct;                /*    88     8 */
	struct user_namespace *    user_ns;              /*    96     8 */
	struct ucounts *           ucounts;              /*   104     8 */
	int                        pid_max;              /*   112     4 */
	int                        reboot;               /*   116     4 */
	struct ns_common           ns;                   /*   120    32 */

	/* size: 152, cachelines: 3, members: 14 */
	/* sum members: 140, holes: 3, sum holes: 12 */
	/* forced alignments: 1 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct task_cputime {
	u64                        stime;                /*     0     8 */
	u64                        utime;                /*     8     8 */
	long long unsigned int     sum_exec_runtime;     /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct nsproxy {
	atomic_t                   count;                /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct uts_namespace *     uts_ns;               /*     8     8 */
	struct ipc_namespace *     ipc_ns;               /*    16     8 */
	struct mnt_namespace *     mnt_ns;               /*    24     8 */
	struct pid_namespace *     pid_ns_for_children;  /*    32     8 */
	struct net *               net_ns;               /*    40     8 */
	struct time_namespace *    time_ns;              /*    48     8 */
	struct time_namespace *    time_ns_for_children; /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct cgroup_namespace *  cgroup_ns;            /*    64     8 */
	u64                        kabi_reserved1;       /*    72     8 */
	u64                        kabi_reserved2;       /*    80     8 */
	u64                        kabi_reserved3;       /*    88     8 */
	u64                        kabi_reserved4;       /*    96     8 */
	u64                        kabi_reserved5;       /*   104     8 */
	u64                        kabi_reserved6;       /*   112     8 */
	u64                        kabi_reserved7;       /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	u64                        kabi_reserved8;       /*   128     8 */

	/* size: 136, cachelines: 3, members: 17 */
	/* sum members: 132, holes: 1, sum holes: 4 */
	/* last cacheline: 8 bytes */
};
struct bio_list {
	struct bio *               head;                 /*     0     8 */
	struct bio *               tail;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct blk_plug {
	struct list_head           mq_list;              /*     0    16 */
	struct list_head           cb_list;              /*    16    16 */
	short unsigned int         rq_count;             /*    32     2 */
	bool                       multiple_queues;      /*    34     1 */
	bool                       nowait;               /*    35     1 */

	/* size: 40, cachelines: 1, members: 5 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct reclaim_state {
	long unsigned int          reclaimed_slab;       /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct backing_dev_info {
	u64                        id;                   /*     0     8 */
	struct rb_node             rb_node __attribute__((__aligned__(8))); /*     8    24 */
	struct list_head           bdi_list;             /*    32    16 */
	long unsigned int          ra_pages;             /*    48     8 */
	long unsigned int          io_pages;             /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct kref                refcnt;               /*    64     4 */
	unsigned int               capabilities;         /*    68     4 */
	unsigned int               min_ratio;            /*    72     4 */
	unsigned int               max_ratio;            /*    76     4 */
	unsigned int               max_prop_frac;        /*    80     4 */

	/* XXX 4 bytes hole, try to pack */

	atomic_long_t              tot_write_bandwidth;  /*    88     8 */
	struct bdi_writeback       wb __attribute__((__aligned__(8))); /*    96   824 */
	/* --- cacheline 14 boundary (896 bytes) was 24 bytes ago --- */
	struct list_head           wb_list;              /*   920    16 */
	struct xarray              cgwb_tree;            /*   936    16 */
	struct mutex               cgwb_release_mutex;   /*   952    32 */
	/* --- cacheline 15 boundary (960 bytes) was 24 bytes ago --- */
	struct rw_semaphore        wb_switch_rwsem;      /*   984    40 */
	/* --- cacheline 16 boundary (1024 bytes) --- */
	wait_queue_head_t          wb_waitq;             /*  1024    24 */
	struct device *            dev;                  /*  1048     8 */
	char                       dev_name[64];         /*  1056    64 */
	/* --- cacheline 17 boundary (1088 bytes) was 32 bytes ago --- */
	struct device *            owner;                /*  1120     8 */
	struct timer_list          laptop_mode_wb_timer; /*  1128    72 */
	/* --- cacheline 18 boundary (1152 bytes) was 48 bytes ago --- */
	struct dentry *            debug_dir;            /*  1200     8 */
	u64                        kabi_reserved1;       /*  1208     8 */
	/* --- cacheline 19 boundary (1216 bytes) --- */
	u64                        kabi_reserved2;       /*  1216     8 */
	u64                        kabi_reserved3;       /*  1224     8 */
	u64                        kabi_reserved4;       /*  1232     8 */

	/* size: 1240, cachelines: 20, members: 26 */
	/* sum members: 1236, holes: 1, sum holes: 4 */
	/* forced alignments: 2 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct css_set {
	struct cgroup_subsys_state * subsys[14];         /*     0   112 */
	/* --- cacheline 1 boundary (64 bytes) was 48 bytes ago --- */
	refcount_t                 refcount;             /*   112     4 */

	/* XXX 4 bytes hole, try to pack */

	struct css_set *           dom_cset;             /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct cgroup *            dfl_cgrp;             /*   128     8 */
	int                        nr_tasks;             /*   136     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           tasks;                /*   144    16 */
	struct list_head           mg_tasks;             /*   160    16 */
	struct list_head           dying_tasks;          /*   176    16 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	struct list_head           task_iters;           /*   192    16 */
	struct list_head           e_cset_node[14];      /*   208   224 */
	/* --- cacheline 6 boundary (384 bytes) was 48 bytes ago --- */
	struct list_head           threaded_csets;       /*   432    16 */
	/* --- cacheline 7 boundary (448 bytes) --- */
	struct list_head           threaded_csets_node;  /*   448    16 */
	struct hlist_node          hlist;                /*   464    16 */
	struct list_head           cgrp_links;           /*   480    16 */
	struct list_head           mg_preload_node;      /*   496    16 */
	/* --- cacheline 8 boundary (512 bytes) --- */
	struct list_head           mg_node;              /*   512    16 */
	struct cgroup *            mg_src_cgrp;          /*   528     8 */
	struct cgroup *            mg_dst_cgrp;          /*   536     8 */
	struct css_set *           mg_dst_cset;          /*   544     8 */
	bool                       dead;                 /*   552     1 */

	/* XXX 7 bytes hole, try to pack */

	struct callback_head       callback_head __attribute__((__aligned__(8))); /*   560    16 */
	/* --- cacheline 9 boundary (576 bytes) --- */
	u64                        kabi_reserved1;       /*   576     8 */
	u64                        kabi_reserved2;       /*   584     8 */
	u64                        kabi_reserved3;       /*   592     8 */
	u64                        kabi_reserved4;       /*   600     8 */

	/* size: 608, cachelines: 10, members: 25 */
	/* sum members: 593, holes: 3, sum holes: 15 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 7 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct compat_robust_list_head {
	struct compat_robust_list  list;                 /*     0     4 */
	compat_long_t              futex_offset;         /*     4     4 */
	compat_uptr_t              list_op_pending;      /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct perf_event_context {
	struct pmu *               pmu;                  /*     0     8 */
	raw_spinlock_t             lock;                 /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct mutex               mutex;                /*    16    32 */
	struct list_head           active_ctx_list;      /*    48    16 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct perf_event_groups   pinned_groups;        /*    64    16 */
	struct perf_event_groups   flexible_groups;      /*    80    16 */
	struct list_head           event_list;           /*    96    16 */
	struct list_head           pinned_active;        /*   112    16 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct list_head           flexible_active;      /*   128    16 */
	int                        nr_events;            /*   144     4 */
	int                        nr_active;            /*   148     4 */
	int                        is_active;            /*   152     4 */
	int                        nr_stat;              /*   156     4 */
	int                        nr_freq;              /*   160     4 */
	int                        rotate_disable;       /*   164     4 */
	int                        rotate_necessary;     /*   168     4 */
	refcount_t                 refcount;             /*   172     4 */
	struct task_struct *       task;                 /*   176     8 */
	u64                        time;                 /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	u64                        timestamp;            /*   192     8 */
	struct perf_event_context * parent_ctx;          /*   200     8 */
	u64                        parent_gen;           /*   208     8 */
	u64                        generation;           /*   216     8 */
	int                        pin_count;            /*   224     4 */
	int                        nr_cgroups;           /*   228     4 */
	void *                     task_ctx_data;        /*   232     8 */
	struct callback_head       callback_head __attribute__((__aligned__(8))); /*   240    16 */

	/* size: 256, cachelines: 4, members: 27 */
	/* sum members: 252, holes: 1, sum holes: 4 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(8)));
struct task_delay_info {
	raw_spinlock_t             lock;                 /*     0     4 */
	unsigned int               flags;                /*     4     4 */
	u64                        blkio_start;          /*     8     8 */
	u64                        blkio_delay;          /*    16     8 */
	u64                        swapin_delay;         /*    24     8 */
	u32                        blkio_count;          /*    32     4 */
	u32                        swapin_count;         /*    36     4 */
	u64                        freepages_start;      /*    40     8 */
	u64                        freepages_delay;      /*    48     8 */
	u64                        thrashing_start;      /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u64                        thrashing_delay;      /*    64     8 */
	u32                        freepages_count;      /*    72     4 */
	u32                        thrashing_count;      /*    76     4 */
	u64                        kabi_reserved1;       /*    80     8 */
	u64                        kabi_reserved2;       /*    88     8 */

	/* size: 96, cachelines: 2, members: 15 */
	/* last cacheline: 32 bytes */
};
struct ftrace_ret_stack {
	long unsigned int          ret;                  /*     0     8 */
	long unsigned int          func;                 /*     8     8 */
	long long unsigned int     calltime;             /*    16     8 */
	long long unsigned int     subtime;              /*    24     8 */
	long unsigned int *        retp;                 /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct request_queue {
	struct request *           last_merge;           /*     0     8 */
	struct elevator_queue *    elevator;             /*     8     8 */
	struct percpu_ref          q_usage_counter;      /*    16    16 */
	struct blk_queue_stats *   stats;                /*    32     8 */
	struct rq_qos *            rq_qos;               /*    40     8 */
	const struct blk_mq_ops  * mq_ops;               /*    48     8 */
	struct blk_mq_ctx *        queue_ctx;            /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	unsigned int               queue_depth;          /*    64     4 */

	/* XXX 4 bytes hole, try to pack */

	struct blk_mq_hw_ctx * *   queue_hw_ctx;         /*    72     8 */
	unsigned int               nr_hw_queues;         /*    80     4 */

	/* XXX 4 bytes hole, try to pack */

	struct backing_dev_info *  backing_dev_info;     /*    88     8 */
	void *                     queuedata;            /*    96     8 */
	long unsigned int          queue_flags;          /*   104     8 */
	atomic_t                   pm_only;              /*   112     4 */
	int                        id;                   /*   116     4 */
	gfp_t                      bounce_gfp;           /*   120     4 */
	spinlock_t                 queue_lock;           /*   124     4 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct kobject             kobj;                 /*   128    96 */
	/* --- cacheline 3 boundary (192 bytes) was 32 bytes ago --- */
	struct kobject *           mq_kobj;              /*   224     8 */
	struct blk_integrity       integrity;            /*   232    32 */
	/* --- cacheline 4 boundary (256 bytes) was 8 bytes ago --- */
	struct device *            dev;                  /*   264     8 */
	enum rpm_status            rpm_status;           /*   272     4 */
	unsigned int               nr_pending;           /*   276     4 */
	long unsigned int          nr_requests;          /*   280     8 */
	unsigned int               dma_pad_mask;         /*   288     4 */
	unsigned int               dma_alignment;        /*   292     4 */
	unsigned int               rq_timeout;           /*   296     4 */
	int                        poll_nsec;            /*   300     4 */
	struct blk_stat_callback * poll_cb;              /*   304     8 */
	struct blk_rq_stat         poll_stat[16];        /*   312   640 */
	/* --- cacheline 14 boundary (896 bytes) was 56 bytes ago --- */
	struct timer_list          timeout;              /*   952    72 */
	/* --- cacheline 16 boundary (1024 bytes) --- */
	struct work_struct         timeout_work;         /*  1024    64 */
	/* --- cacheline 17 boundary (1088 bytes) --- */
	atomic_t                   nr_active_requests_shared_sbitmap; /*  1088     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           icq_list;             /*  1096    16 */
	long unsigned int          blkcg_pols[1];        /*  1112     8 */
	struct blkcg_gq *          root_blkg;            /*  1120     8 */
	struct list_head           blkg_list;            /*  1128    16 */
	struct queue_limits        limits;               /*  1144   120 */
	/* --- cacheline 19 boundary (1216 bytes) was 48 bytes ago --- */
	unsigned int               required_elevator_features; /*  1264     4 */
	unsigned int               nr_zones;             /*  1268     4 */
	long unsigned int *        conv_zones_bitmap;    /*  1272     8 */
	/* --- cacheline 20 boundary (1280 bytes) --- */
	long unsigned int *        seq_zones_wlock;      /*  1280     8 */
	unsigned int               max_open_zones;       /*  1288     4 */
	unsigned int               max_active_zones;     /*  1292     4 */
	unsigned int               sg_timeout;           /*  1296     4 */
	unsigned int               sg_reserved_size;     /*  1300     4 */
	int                        node;                 /*  1304     4 */

	/* XXX 4 bytes hole, try to pack */

	struct mutex               debugfs_mutex;        /*  1312    32 */
	/* --- cacheline 21 boundary (1344 bytes) --- */
	struct blk_trace *         blk_trace;            /*  1344     8 */
	struct blk_flush_queue *   fq;                   /*  1352     8 */
	struct list_head           requeue_list;         /*  1360    16 */
	spinlock_t                 requeue_lock;         /*  1376     4 */

	/* XXX 4 bytes hole, try to pack */

	struct delayed_work        requeue_work;         /*  1384   192 */
	/* --- cacheline 24 boundary (1536 bytes) was 40 bytes ago --- */
	struct mutex               sysfs_lock;           /*  1576    32 */
	/* --- cacheline 25 boundary (1600 bytes) was 8 bytes ago --- */
	struct mutex               sysfs_dir_lock;       /*  1608    32 */
	struct list_head           unused_hctx_list;     /*  1640    16 */
	spinlock_t                 unused_hctx_lock;     /*  1656     4 */
	int                        mq_freeze_depth;      /*  1660     4 */
	/* --- cacheline 26 boundary (1664 bytes) --- */
	struct bsg_class_device    bsg_dev;              /*  1664    32 */
	struct throtl_data *       td;                   /*  1696     8 */
	struct callback_head       callback_head __attribute__((__aligned__(8))); /*  1704    16 */
	wait_queue_head_t          mq_freeze_wq;         /*  1720    24 */
	/* --- cacheline 27 boundary (1728 bytes) was 16 bytes ago --- */
	struct mutex               mq_freeze_lock;       /*  1744    32 */
	int                        quiesce_depth;        /*  1776     4 */

	/* XXX 4 bytes hole, try to pack */

	struct blk_mq_tag_set *    tag_set;              /*  1784     8 */
	/* --- cacheline 28 boundary (1792 bytes) --- */
	struct list_head           tag_set_list;         /*  1792    16 */
	struct bio_set             bio_split;            /*  1808   432 */
	/* --- cacheline 35 boundary (2240 bytes) --- */
	struct dentry *            debugfs_dir;          /*  2240     8 */
	struct dentry *            sched_debugfs_dir;    /*  2248     8 */
	struct dentry *            rqos_debugfs_dir;     /*  2256     8 */
	bool                       mq_sysfs_init_done;   /*  2264     1 */

	/* XXX 7 bytes hole, try to pack */

	size_t                     cmd_size;             /*  2272     8 */
	u64                        write_hints[5];       /*  2280    40 */
	/* --- cacheline 36 boundary (2304 bytes) was 16 bytes ago --- */
	union {
		struct blk_mq_tags * shared_sbitmap_tags; /*  2320     8 */
		struct {
			long unsigned int dtag_wait_time; /*  2320     8 */
		} kabi_hidden_606;                       /*  2320     8 */
		union {
		};                                       /*  2320     0 */
	};                                               /*  2320     8 */
	u64                        kabi_reserved1;       /*  2328     8 */
	u64                        kabi_reserved2;       /*  2336     8 */
	u64                        kabi_reserved3;       /*  2344     8 */
	u64                        kabi_reserved4;       /*  2352     8 */

	/* size: 2360, cachelines: 37, members: 78 */
	/* sum members: 2329, holes: 7, sum holes: 31 */
	/* forced alignments: 1 */
	/* last cacheline: 56 bytes */
} __attribute__((__aligned__(8)));
struct bpf_run_ctx {

	/* size: 0, cachelines: 0, members: 0 */
};
struct cgroup {
	struct cgroup_subsys_state self __attribute__((__aligned__(8))); /*     0   296 */
	/* --- cacheline 4 boundary (256 bytes) was 40 bytes ago --- */
	long unsigned int          flags;                /*   296     8 */
	int                        level;                /*   304     4 */
	int                        max_depth;            /*   308     4 */
	int                        nr_descendants;       /*   312     4 */
	int                        nr_dying_descendants; /*   316     4 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	int                        max_descendants;      /*   320     4 */
	int                        nr_populated_csets;   /*   324     4 */
	int                        nr_populated_domain_children; /*   328     4 */
	int                        nr_populated_threaded_children; /*   332     4 */
	int                        nr_threaded_children; /*   336     4 */

	/* XXX 4 bytes hole, try to pack */

	struct kernfs_node *       kn;                   /*   344     8 */
	struct cgroup_file         procs_file;           /*   352   104 */
	/* --- cacheline 7 boundary (448 bytes) was 8 bytes ago --- */
	struct cgroup_file         events_file;          /*   456   104 */
	/* --- cacheline 8 boundary (512 bytes) was 48 bytes ago --- */
	u16                        subtree_control;      /*   560     2 */
	u16                        subtree_ss_mask;      /*   562     2 */
	u16                        old_subtree_control;  /*   564     2 */
	u16                        old_subtree_ss_mask;  /*   566     2 */
	struct cgroup_subsys_state * subsys[14];         /*   568   112 */
	/* --- cacheline 10 boundary (640 bytes) was 40 bytes ago --- */
	struct cgroup_root *       root;                 /*   680     8 */
	struct list_head           cset_links;           /*   688    16 */
	/* --- cacheline 11 boundary (704 bytes) --- */
	struct list_head           e_csets[14];          /*   704   224 */
	/* --- cacheline 14 boundary (896 bytes) was 32 bytes ago --- */
	struct cgroup *            dom_cgrp;             /*   928     8 */
	struct cgroup *            old_dom_cgrp;         /*   936     8 */
	struct cgroup_rstat_cpu *  rstat_cpu;            /*   944     8 */
	struct list_head           rstat_css_list;       /*   952    16 */
	/* --- cacheline 15 boundary (960 bytes) was 8 bytes ago --- */
	struct cgroup_base_stat    last_bstat;           /*   968    24 */
	struct cgroup_base_stat    bstat;                /*   992    24 */
	struct prev_cputime        prev_cputime;         /*  1016    24 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 16 boundary (1024 bytes) was 16 bytes ago --- */
	struct list_head           pidlists;             /*  1040    16 */
	struct mutex               pidlist_mutex;        /*  1056    32 */
	/* --- cacheline 17 boundary (1088 bytes) --- */
	wait_queue_head_t          offline_waitq;        /*  1088    24 */
	struct work_struct         release_agent_work;   /*  1112    64 */
	/* --- cacheline 18 boundary (1152 bytes) was 24 bytes ago --- */
	struct psi_group           psi;                  /*  1176   800 */
	/* --- cacheline 30 boundary (1920 bytes) was 56 bytes ago --- */
	struct cgroup_bpf          bpf;                  /*  1976  1104 */
	/* --- cacheline 48 boundary (3072 bytes) was 8 bytes ago --- */
	atomic_t                   congestion_count;     /*  3080     4 */
	struct cgroup_freezer_state freezer;             /*  3084    16 */

	/* XXX 4 bytes hole, try to pack */

	u64                        kabi_reserved1;       /*  3104     8 */
	u64                        kabi_reserved2;       /*  3112     8 */
	u64                        kabi_reserved3;       /*  3120     8 */
	u64                        ancestor_ids[];       /*  3128     0 */

	/* size: 3128, cachelines: 49, members: 41 */
	/* sum members: 3120, holes: 2, sum holes: 8 */
	/* paddings: 1, sum paddings: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 56 bytes */
} __attribute__((__aligned__(8)));
struct taskstats {
	__u16                      version;              /*     0     2 */

	/* XXX 2 bytes hole, try to pack */

	__u32                      ac_exitcode;          /*     4     4 */
	__u8                       ac_flag;              /*     8     1 */
	__u8                       ac_nice;              /*     9     1 */

	/* XXX 6 bytes hole, try to pack */

	__u64                      cpu_count __attribute__((__aligned__(8))); /*    16     8 */
	__u64                      cpu_delay_total;      /*    24     8 */
	__u64                      blkio_count;          /*    32     8 */
	__u64                      blkio_delay_total;    /*    40     8 */
	__u64                      swapin_count;         /*    48     8 */
	__u64                      swapin_delay_total;   /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	__u64                      cpu_run_real_total;   /*    64     8 */
	__u64                      cpu_run_virtual_total; /*    72     8 */
	char                       ac_comm[32];          /*    80    32 */
	__u8                       ac_sched __attribute__((__aligned__(8))); /*   112     1 */
	__u8                       ac_pad[3];            /*   113     3 */

	/* XXX 4 bytes hole, try to pack */

	__u32                      ac_uid __attribute__((__aligned__(8))); /*   120     4 */
	__u32                      ac_gid;               /*   124     4 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	__u32                      ac_pid;               /*   128     4 */
	__u32                      ac_ppid;              /*   132     4 */
	__u32                      ac_btime;             /*   136     4 */

	/* XXX 4 bytes hole, try to pack */

	__u64                      ac_etime __attribute__((__aligned__(8))); /*   144     8 */
	__u64                      ac_utime;             /*   152     8 */
	__u64                      ac_stime;             /*   160     8 */
	__u64                      ac_minflt;            /*   168     8 */
	__u64                      ac_majflt;            /*   176     8 */
	__u64                      coremem;              /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	__u64                      virtmem;              /*   192     8 */
	__u64                      hiwater_rss;          /*   200     8 */
	__u64                      hiwater_vm;           /*   208     8 */
	__u64                      read_char;            /*   216     8 */
	__u64                      write_char;           /*   224     8 */
	__u64                      read_syscalls;        /*   232     8 */
	__u64                      write_syscalls;       /*   240     8 */
	__u64                      read_bytes;           /*   248     8 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	__u64                      write_bytes;          /*   256     8 */
	__u64                      cancelled_write_bytes; /*   264     8 */
	__u64                      nvcsw;                /*   272     8 */
	__u64                      nivcsw;               /*   280     8 */
	__u64                      ac_utimescaled;       /*   288     8 */
	__u64                      ac_stimescaled;       /*   296     8 */
	__u64                      cpu_scaled_run_real_total; /*   304     8 */
	__u64                      freepages_count;      /*   312     8 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	__u64                      freepages_delay_total; /*   320     8 */
	__u64                      thrashing_count;      /*   328     8 */
	__u64                      thrashing_delay_total; /*   336     8 */
	__u64                      ac_btime64;           /*   344     8 */

	/* size: 352, cachelines: 6, members: 46 */
	/* sum members: 336, holes: 4, sum holes: 16 */
	/* forced alignments: 4, forced holes: 3, sum forced holes: 14 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct percpu_counter {
	raw_spinlock_t             lock;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	s64                        count;                /*     8     8 */
	struct list_head           list;                 /*    16    16 */
	s32 *                      counters;             /*    32     8 */

	/* size: 40, cachelines: 1, members: 4 */
	/* sum members: 36, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct wait_page_queue {
	struct page *              page;                 /*     0     8 */
	int                        bit_nr;               /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	wait_queue_entry_t         wait;                 /*    16    40 */

	/* size: 56, cachelines: 1, members: 3 */
	/* sum members: 52, holes: 1, sum holes: 4 */
	/* last cacheline: 56 bytes */
};
struct writeback_control {
	long int                   nr_to_write;          /*     0     8 */
	long int                   pages_skipped;        /*     8     8 */
	loff_t                     range_start;          /*    16     8 */
	loff_t                     range_end;            /*    24     8 */
	enum writeback_sync_modes  sync_mode;            /*    32     4 */
	unsigned int               for_kupdate:1;        /*    36: 0  4 */
	unsigned int               for_background:1;     /*    36: 1  4 */
	unsigned int               tagged_writepages:1;  /*    36: 2  4 */
	unsigned int               for_reclaim:1;        /*    36: 3  4 */
	unsigned int               range_cyclic:1;       /*    36: 4  4 */
	unsigned int               for_sync:1;           /*    36: 5  4 */
	unsigned int               no_cgroup_owner:1;    /*    36: 6  4 */
	unsigned int               punt_to_cgroup:1;     /*    36: 7  4 */

	/* XXX 24 bits hole, try to pack */

	struct bdi_writeback *     wb;                   /*    40     8 */
	struct inode *             inode;                /*    48     8 */
	int                        wb_id;                /*    56     4 */
	int                        wb_lcand_id;          /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        wb_tcand_id;          /*    64     4 */

	/* XXX 4 bytes hole, try to pack */

	size_t                     wb_bytes;             /*    72     8 */
	size_t                     wb_lcand_bytes;       /*    80     8 */
	size_t                     wb_tcand_bytes;       /*    88     8 */
	u64                        kabi_reserved1;       /*    96     8 */
	u64                        kabi_reserved2;       /*   104     8 */

	/* size: 112, cachelines: 2, members: 23 */
	/* sum members: 104, holes: 1, sum holes: 4 */
	/* sum bitfield members: 8 bits, bit holes: 1, sum bit holes: 24 bits */
	/* last cacheline: 48 bytes */
};
struct readahead_control {
	struct file *              file;                 /*     0     8 */
	struct address_space *     mapping;              /*     8     8 */
	long unsigned int          _index;               /*    16     8 */
	unsigned int               _nr_pages;            /*    24     4 */
	unsigned int               _batch_count;         /*    28     4 */

	/* size: 32, cachelines: 1, members: 5 */
	/* last cacheline: 32 bytes */
};
struct iov_iter {
	unsigned int               type;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	size_t                     iov_offset;           /*     8     8 */
	size_t                     count;                /*    16     8 */
	union {
		const struct iovec  * iov;               /*    24     8 */
		const struct kvec  * kvec;               /*    24     8 */
		const struct bio_vec  * bvec;            /*    24     8 */
		struct pipe_inode_info * pipe;           /*    24     8 */
	};                                               /*    24     8 */
	union {
		long unsigned int  nr_segs;              /*    32     8 */
		struct {
			unsigned int head;               /*    32     4 */
			unsigned int start_head;         /*    36     4 */
		};                                       /*    32     8 */
	};                                               /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* sum members: 36, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct swap_info_struct {
	long unsigned int          flags;                /*     0     8 */
	short int                  prio;                 /*     8     2 */

	/* XXX 6 bytes hole, try to pack */

	struct plist_node          list;                 /*    16    40 */
	signed char                type;                 /*    56     1 */

	/* XXX 3 bytes hole, try to pack */

	unsigned int               max;                  /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	unsigned char *            swap_map;             /*    64     8 */
	struct swap_cluster_info * cluster_info;         /*    72     8 */
	struct swap_cluster_list   free_clusters;        /*    80    16 */
	unsigned int               lowest_bit;           /*    96     4 */
	unsigned int               highest_bit;          /*   100     4 */
	unsigned int               pages;                /*   104     4 */
	unsigned int               inuse_pages;          /*   108     4 */
	unsigned int               cluster_next;         /*   112     4 */
	unsigned int               cluster_nr;           /*   116     4 */
	unsigned int *             cluster_next_cpu;     /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct percpu_cluster *    percpu_cluster;       /*   128     8 */
	struct rb_root             swap_extent_root;     /*   136     8 */
	struct block_device *      bdev;                 /*   144     8 */
	struct file *              swap_file;            /*   152     8 */
	unsigned int               old_block_size;       /*   160     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int *        frontswap_map;        /*   168     8 */
	atomic_t                   frontswap_pages;      /*   176     4 */
	spinlock_t                 lock;                 /*   180     4 */
	spinlock_t                 cont_lock;            /*   184     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 3 boundary (192 bytes) --- */
	struct work_struct         discard_work;         /*   192    64 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	struct swap_cluster_list   discard_clusters;     /*   256    16 */
	u64                        kabi_reserved1;       /*   272     8 */
	u64                        kabi_reserved2;       /*   280     8 */
	struct plist_node          avail_lists[];        /*   288     0 */

	/* size: 288, cachelines: 5, members: 29 */
	/* sum members: 271, holes: 4, sum holes: 17 */
	/* last cacheline: 32 bytes */
};
struct block_device {
	dev_t                      bd_dev;               /*     0     4 */
	int                        bd_openers;           /*     4     4 */
	struct inode *             bd_inode;             /*     8     8 */
	struct super_block *       bd_super;             /*    16     8 */
	struct mutex               bd_mutex;             /*    24    32 */
	void *                     bd_claiming;          /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	void *                     bd_holder;            /*    64     8 */
	int                        bd_holders;           /*    72     4 */
	bool                       bd_write_holder;      /*    76     1 */

	/* XXX 3 bytes hole, try to pack */

	struct list_head           bd_holder_disks;      /*    80    16 */
	struct block_device *      bd_contains;          /*    96     8 */
	u8                         bd_partno;            /*   104     1 */

	/* XXX 7 bytes hole, try to pack */

	struct hd_struct *         bd_part;              /*   112     8 */
	unsigned int               bd_part_count;        /*   120     4 */
	spinlock_t                 bd_size_lock;         /*   124     4 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct gendisk *           bd_disk;              /*   128     8 */
	struct backing_dev_info *  bd_bdi;               /*   136     8 */
	int                        bd_fsfreeze_count;    /*   144     4 */

	/* XXX 4 bytes hole, try to pack */

	struct mutex               bd_fsfreeze_mutex;    /*   152    32 */
	union {
		struct {
			int        bd_write_openers;     /*   184     4 */
			int        bd_part_write_openers; /*   188     4 */
		};                                       /*   184     8 */
		struct {
			u64        kabi_reserved1;       /*   184     8 */
		} kabi_hidden_52;                        /*   184     8 */
		union {
		};                                       /*   184     0 */
	};                                               /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	u64                        kabi_reserved2;       /*   192     8 */
	u64                        kabi_reserved3;       /*   200     8 */
	u64                        kabi_reserved4;       /*   208     8 */
	u64                        kabi_reserved5;       /*   216     8 */

	/* size: 224, cachelines: 4, members: 24 */
	/* sum members: 210, holes: 3, sum holes: 14 */
	/* last cacheline: 32 bytes */
};
struct cdev {
	struct kobject             kobj;                 /*     0    96 */
	/* --- cacheline 1 boundary (64 bytes) was 32 bytes ago --- */
	struct module *            owner;                /*    96     8 */
	const struct file_operations  * ops;             /*   104     8 */
	struct list_head           list;                 /*   112    16 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	dev_t                      dev;                  /*   128     4 */
	unsigned int               count;                /*   132     4 */

	/* size: 136, cachelines: 3, members: 6 */
	/* last cacheline: 8 bytes */
};
struct bdi_writeback {
	struct backing_dev_info *  bdi;                  /*     0     8 */
	long unsigned int          state;                /*     8     8 */
	long unsigned int          last_old_flush;       /*    16     8 */
	struct list_head           b_dirty;              /*    24    16 */
	struct list_head           b_io;                 /*    40    16 */
	struct list_head           b_more_io;            /*    56    16 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	struct list_head           b_dirty_time;         /*    72    16 */
	spinlock_t                 list_lock;            /*    88     4 */

	/* XXX 4 bytes hole, try to pack */

	struct percpu_counter      stat[4];              /*    96   160 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	long unsigned int          congested;            /*   256     8 */
	long unsigned int          bw_time_stamp;        /*   264     8 */
	long unsigned int          dirtied_stamp;        /*   272     8 */
	long unsigned int          written_stamp;        /*   280     8 */
	long unsigned int          write_bandwidth;      /*   288     8 */
	long unsigned int          avg_write_bandwidth;  /*   296     8 */
	long unsigned int          dirty_ratelimit;      /*   304     8 */
	long unsigned int          balanced_dirty_ratelimit; /*   312     8 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	struct fprop_local_percpu  completions;          /*   320    48 */
	int                        dirty_exceeded;       /*   368     4 */
	enum wb_reason             start_all_reason;     /*   372     4 */
	spinlock_t                 work_lock;            /*   376     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 6 boundary (384 bytes) --- */
	struct list_head           work_list;            /*   384    16 */
	struct delayed_work        dwork;                /*   400   192 */
	/* --- cacheline 9 boundary (576 bytes) was 16 bytes ago --- */
	long unsigned int          dirty_sleep;          /*   592     8 */
	struct list_head           bdi_node;             /*   600    16 */
	struct percpu_ref          refcnt;               /*   616    16 */
	struct fprop_local_percpu  memcg_completions;    /*   632    48 */
	/* --- cacheline 10 boundary (640 bytes) was 40 bytes ago --- */
	struct cgroup_subsys_state * memcg_css;          /*   680     8 */
	struct cgroup_subsys_state * blkcg_css;          /*   688     8 */
	struct list_head           memcg_node;           /*   696    16 */
	/* --- cacheline 11 boundary (704 bytes) was 8 bytes ago --- */
	struct list_head           blkcg_node;           /*   712    16 */
	union {
		struct work_struct release_work;         /*   728    64 */
		struct callback_head rcu __attribute__((__aligned__(8))); /*   728    16 */
	} __attribute__((__aligned__(8)));               /*   728    64 */
	/* --- cacheline 12 boundary (768 bytes) was 24 bytes ago --- */
	u64                        kabi_reserved1;       /*   792     8 */
	u64                        kabi_reserved2;       /*   800     8 */
	u64                        kabi_reserved3;       /*   808     8 */
	u64                        kabi_reserved4;       /*   816     8 */

	/* size: 824, cachelines: 13, members: 36 */
	/* sum members: 816, holes: 2, sum holes: 8 */
	/* forced alignments: 1 */
	/* last cacheline: 56 bytes */
} __attribute__((__aligned__(8)));
struct xattr_handler {
	const char  *              name;                 /*     0     8 */
	const char  *              prefix;               /*     8     8 */
	int                        flags;                /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	bool                       (*list)(struct dentry *); /*    24     8 */
	int                        (*get)(const struct xattr_handler  *, struct dentry *, struct inode *, const char  *, void *, size_t); /*    32     8 */
	int                        (*set)(const struct xattr_handler  *, struct dentry *, struct inode *, const char  *, const void  *, size_t, int); /*    40     8 */
	u64                        kabi_reserved1;       /*    48     8 */

	/* size: 56, cachelines: 1, members: 7 */
	/* sum members: 52, holes: 1, sum holes: 4 */
	/* last cacheline: 56 bytes */
};
struct fs_context {
	const struct fs_context_operations  * ops;       /*     0     8 */
	struct mutex               uapi_mutex;           /*     8    32 */
	struct file_system_type *  fs_type;              /*    40     8 */
	void *                     fs_private;           /*    48     8 */
	void *                     sget_key;             /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct dentry *            root;                 /*    64     8 */
	struct user_namespace *    user_ns;              /*    72     8 */
	struct net *               net_ns;               /*    80     8 */
	const struct cred  *       cred;                 /*    88     8 */
	struct p_log               log;                  /*    96    16 */
	const char  *              source;               /*   112     8 */
	void *                     security;             /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	void *                     s_fs_info;            /*   128     8 */
	unsigned int               sb_flags;             /*   136     4 */
	unsigned int               sb_flags_mask;        /*   140     4 */
	unsigned int               s_iflags;             /*   144     4 */
	unsigned int               lsm_flags;            /*   148     4 */
	enum fs_context_purpose    purpose:8;            /*   152: 0  4 */
	enum fs_context_phase      phase:8;              /*   152: 8  4 */

	/* Bitfield combined with next fields */

	bool                       need_free:1;          /*   154: 0  1 */
	bool                       global:1;             /*   154: 1  1 */
	bool                       oldapi:1;             /*   154: 2  1 */

	/* size: 160, cachelines: 3, members: 22 */
	/* padding: 4 */
	/* bit_padding: 13 bits */
	/* last cacheline: 32 bytes */
};
struct fs_parameter_spec {
	const char  *              name;                 /*     0     8 */
	fs_param_type *            type;                 /*     8     8 */
	u8                         opt;                  /*    16     1 */

	/* XXX 1 byte hole, try to pack */

	short unsigned int         flags;                /*    18     2 */

	/* XXX 4 bytes hole, try to pack */

	const void  *              data;                 /*    24     8 */

	/* size: 32, cachelines: 1, members: 5 */
	/* sum members: 27, holes: 2, sum holes: 5 */
	/* last cacheline: 32 bytes */
};
struct filename {
	const char  *              name;                 /*     0     8 */
	const char  *              uptr;                 /*     8     8 */
	int                        refcnt;               /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	struct audit_names *       aname;                /*    24     8 */
	const char                 iname[];              /*    32     0 */

	/* size: 32, cachelines: 1, members: 5 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct bio {
	struct bio *               bi_next;              /*     0     8 */
	struct gendisk *           bi_disk;              /*     8     8 */
	unsigned int               bi_opf;               /*    16     4 */
	short unsigned int         bi_flags;             /*    20     2 */
	short unsigned int         bi_ioprio;            /*    22     2 */
	short unsigned int         bi_write_hint;        /*    24     2 */
	blk_status_t               bi_status;            /*    26     1 */
	u8                         bi_partno;            /*    27     1 */
	atomic_t                   __bi_remaining;       /*    28     4 */
	struct bvec_iter           bi_iter;              /*    32    24 */

	/* XXX last struct has 4 bytes of padding */

	bio_end_io_t *             bi_end_io;            /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	void *                     bi_private;           /*    64     8 */
	struct blkcg_gq *          bi_blkg;              /*    72     8 */
	struct bio_issue           bi_issue;             /*    80     8 */
	union {
		struct bio_integrity_payload * bi_integrity; /*    88     8 */
	};                                               /*    88     8 */
	short unsigned int         bi_vcnt;              /*    96     2 */
	short unsigned int         bi_max_vecs;          /*    98     2 */
	atomic_t                   __bi_cnt;             /*   100     4 */
	struct bio_vec *           bi_io_vec;            /*   104     8 */
	struct bio_set *           bi_pool;              /*   112     8 */
	u64                        kabi_reserved1;       /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	u64                        kabi_reserved2;       /*   128     8 */
	u64                        kabi_reserved3;       /*   136     8 */
	u64                        kabi_reserved4;       /*   144     8 */
	struct bio_vec             bi_inline_vecs[];     /*   152     0 */

	/* size: 152, cachelines: 3, members: 25 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 24 bytes */
};
struct linux_binprm {
	struct vm_area_struct *    vma;                  /*     0     8 */
	long unsigned int          vma_pages;            /*     8     8 */
	struct mm_struct *         mm;                   /*    16     8 */
	long unsigned int          p;                    /*    24     8 */
	long unsigned int          argmin;               /*    32     8 */
	unsigned int               have_execfd:1;        /*    40: 0  4 */
	unsigned int               execfd_creds:1;       /*    40: 1  4 */
	unsigned int               secureexec:1;         /*    40: 2  4 */
	unsigned int               point_of_no_return:1; /*    40: 3  4 */

	/* XXX 28 bits hole, try to pack */
	/* XXX 4 bytes hole, try to pack */

	struct file *              executable;           /*    48     8 */
	struct file *              interpreter;          /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct file *              file;                 /*    64     8 */
	struct cred *              cred;                 /*    72     8 */
	int                        unsafe;               /*    80     4 */
	unsigned int               per_clear;            /*    84     4 */
	int                        argc;                 /*    88     4 */
	int                        envc;                 /*    92     4 */
	const char  *              filename;             /*    96     8 */
	const char  *              interp;               /*   104     8 */
	const char  *              fdpath;               /*   112     8 */
	unsigned int               interp_flags;         /*   120     4 */
	int                        execfd;               /*   124     4 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	long unsigned int          loader;               /*   128     8 */
	long unsigned int          exec;                 /*   136     8 */
	struct rlimit              rlim_stack;           /*   144    16 */
	char                       buf[256];             /*   160   256 */

	/* size: 416, cachelines: 7, members: 26 */
	/* sum members: 408, holes: 1, sum holes: 4 */
	/* sum bitfield members: 4 bits, bit holes: 1, sum bit holes: 28 bits */
	/* last cacheline: 32 bytes */
};
struct coredump_params {
	const kernel_siginfo_t  *  siginfo;              /*     0     8 */
	struct pt_regs *           regs;                 /*     8     8 */
	struct file *              file;                 /*    16     8 */
	long unsigned int          limit;                /*    24     8 */
	long unsigned int          mm_flags;             /*    32     8 */
	loff_t                     written;              /*    40     8 */
	loff_t                     pos;                  /*    48     8 */

	/* size: 56, cachelines: 1, members: 7 */
	/* last cacheline: 56 bytes */
};
struct pm_message {
	int                        event;                /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct dev_pm_ops {
	int                        (*prepare)(struct device *); /*     0     8 */
	void                       (*complete)(struct device *); /*     8     8 */
	int                        (*suspend)(struct device *); /*    16     8 */
	int                        (*resume)(struct device *); /*    24     8 */
	int                        (*freeze)(struct device *); /*    32     8 */
	int                        (*thaw)(struct device *); /*    40     8 */
	int                        (*poweroff)(struct device *); /*    48     8 */
	int                        (*restore)(struct device *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*suspend_late)(struct device *); /*    64     8 */
	int                        (*resume_early)(struct device *); /*    72     8 */
	int                        (*freeze_late)(struct device *); /*    80     8 */
	int                        (*thaw_early)(struct device *); /*    88     8 */
	int                        (*poweroff_late)(struct device *); /*    96     8 */
	int                        (*restore_early)(struct device *); /*   104     8 */
	int                        (*suspend_noirq)(struct device *); /*   112     8 */
	int                        (*resume_noirq)(struct device *); /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	int                        (*freeze_noirq)(struct device *); /*   128     8 */
	int                        (*thaw_noirq)(struct device *); /*   136     8 */
	int                        (*poweroff_noirq)(struct device *); /*   144     8 */
	int                        (*restore_noirq)(struct device *); /*   152     8 */
	int                        (*runtime_suspend)(struct device *); /*   160     8 */
	int                        (*runtime_resume)(struct device *); /*   168     8 */
	int                        (*runtime_idle)(struct device *); /*   176     8 */

	/* size: 184, cachelines: 3, members: 23 */
	/* last cacheline: 56 bytes */
};
struct device {
	struct kobject             kobj;                 /*     0    96 */
	/* --- cacheline 1 boundary (64 bytes) was 32 bytes ago --- */
	struct device *            parent;               /*    96     8 */
	struct device_private *    p;                    /*   104     8 */
	const char  *              init_name;            /*   112     8 */
	const struct device_type  * type;                /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct bus_type *          bus;                  /*   128     8 */
	struct device_driver *     driver;               /*   136     8 */
	void *                     platform_data;        /*   144     8 */
	void *                     driver_data;          /*   152     8 */
	struct mutex               mutex;                /*   160    32 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	struct dev_links_info      links;                /*   192    72 */
	/* --- cacheline 4 boundary (256 bytes) was 8 bytes ago --- */
	struct dev_pm_info         power __attribute__((__aligned__(8))); /*   264   384 */
	/* --- cacheline 10 boundary (640 bytes) was 8 bytes ago --- */
	struct dev_pm_domain *     pm_domain;            /*   648     8 */
	struct irq_domain *        msi_domain;           /*   656     8 */
	struct dev_pin_info *      pins;                 /*   664     8 */
	raw_spinlock_t             msi_lock;             /*   672     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           msi_list;             /*   680    16 */
	const struct dma_map_ops  * dma_ops;             /*   696     8 */
	/* --- cacheline 11 boundary (704 bytes) --- */
	u64 *                      dma_mask;             /*   704     8 */
	u64                        coherent_dma_mask;    /*   712     8 */
	u64                        bus_dma_limit;        /*   720     8 */
	const struct bus_dma_region  * dma_range_map;    /*   728     8 */
	struct device_dma_parameters * dma_parms;        /*   736     8 */
	struct list_head           dma_pools;            /*   744    16 */
	struct dev_archdata        archdata;             /*   760     0 */
	struct device_node *       of_node;              /*   760     8 */
	/* --- cacheline 12 boundary (768 bytes) --- */
	struct fwnode_handle *     fwnode;               /*   768     8 */
	int                        numa_node;            /*   776     4 */
	dev_t                      devt;                 /*   780     4 */
	u32                        id;                   /*   784     4 */
	spinlock_t                 devres_lock;          /*   788     4 */
	struct list_head           devres_head;          /*   792    16 */
	struct class *             class;                /*   808     8 */
	const struct attribute_group  * * groups;        /*   816     8 */
	void                       (*release)(struct device *); /*   824     8 */
	/* --- cacheline 13 boundary (832 bytes) --- */
	struct iommu_group *       iommu_group;          /*   832     8 */
	struct dev_iommu *         iommu;                /*   840     8 */
	bool                       offline_disabled:1;   /*   848: 0  1 */
	bool                       offline:1;            /*   848: 1  1 */
	bool                       of_node_reused:1;     /*   848: 2  1 */
	bool                       state_synced:1;       /*   848: 3  1 */

	/* XXX 4 bits hole, try to pack */
	/* XXX 7 bytes hole, try to pack */

	u64                        kabi_reserved1;       /*   856     8 */
	u64                        kabi_reserved2;       /*   864     8 */
	u64                        kabi_reserved3;       /*   872     8 */
	u64                        kabi_reserved4;       /*   880     8 */
	u64                        kabi_reserved5;       /*   888     8 */
	/* --- cacheline 14 boundary (896 bytes) --- */
	u64                        kabi_reserved6;       /*   896     8 */
	u64                        kabi_reserved7;       /*   904     8 */
	u64                        kabi_reserved8;       /*   912     8 */
	u64                        kabi_reserved9;       /*   920     8 */
	u64                        kabi_reserved10;      /*   928     8 */
	u64                        kabi_reserved11;      /*   936     8 */
	u64                        kabi_reserved12;      /*   944     8 */
	u64                        kabi_reserved13;      /*   952     8 */
	/* --- cacheline 15 boundary (960 bytes) --- */
	u64                        kabi_reserved14;      /*   960     8 */
	u64                        kabi_reserved15;      /*   968     8 */
	u64                        kabi_reserved16;      /*   976     8 */
	size_t                     device_extended_size_resvd; /*   984     8 */
	struct device_extended_resvd * _resvd;           /*   992     8 */

	/* size: 1000, cachelines: 16, members: 59 */
	/* sum members: 988, holes: 2, sum holes: 11 */
	/* sum bitfield members: 4 bits, bit holes: 1, sum bit holes: 4 bits */
	/* forced alignments: 1 */
	/* last cacheline: 40 bytes */
} __attribute__((__aligned__(8)));
struct pm_subsys_data {
	spinlock_t                 lock;                 /*     0     4 */
	unsigned int               refcount;             /*     4     4 */
	struct list_head           clock_list;           /*     8    16 */
	struct pm_domain_data *    domain_data;          /*    24     8 */
	u64                        kabi_reserved1;       /*    32     8 */
	u64                        kabi_reserved2;       /*    40     8 */
	u64                        kabi_reserved3;       /*    48     8 */
	u64                        kabi_reserved4;       /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u64                        kabi_reserved5;       /*    64     8 */
	u64                        kabi_reserved6;       /*    72     8 */

	/* size: 80, cachelines: 2, members: 10 */
	/* last cacheline: 16 bytes */
};
struct dev_pm_info {
	pm_message_t               power_state;          /*     0     4 */
	unsigned int               can_wakeup:1;         /*     4: 0  4 */
	unsigned int               async_suspend:1;      /*     4: 1  4 */
	bool                       in_dpm_list:1;        /*     4: 2  1 */
	bool                       is_prepared:1;        /*     4: 3  1 */
	bool                       is_suspended:1;       /*     4: 4  1 */
	bool                       is_noirq_suspended:1; /*     4: 5  1 */
	bool                       is_late_suspended:1;  /*     4: 6  1 */
	bool                       no_pm:1;              /*     4: 7  1 */

	/* Bitfield combined with next fields */

	bool                       early_init:1;         /*     5: 0  1 */
	bool                       direct_complete:1;    /*     5: 1  1 */

	/* XXX 22 bits hole, try to pack */

	u32                        driver_flags;         /*     8     4 */
	spinlock_t                 lock;                 /*    12     4 */
	struct list_head           entry;                /*    16    16 */
	struct completion          completion;           /*    32    32 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct wakeup_source *     wakeup;               /*    64     8 */
	bool                       wakeup_path:1;        /*    72: 0  1 */
	bool                       syscore:1;            /*    72: 1  1 */
	bool                       no_pm_callbacks:1;    /*    72: 2  1 */
	unsigned int               must_resume:1;        /*    72: 3  4 */
	unsigned int               may_skip_resume:1;    /*    72: 4  4 */

	/* XXX 27 bits hole, try to pack */
	/* XXX 4 bytes hole, try to pack */

	struct hrtimer             suspend_timer __attribute__((__aligned__(8))); /*    80    96 */
	/* --- cacheline 2 boundary (128 bytes) was 48 bytes ago --- */
	u64                        timer_expires;        /*   176     8 */
	struct work_struct         work;                 /*   184    64 */
	/* --- cacheline 3 boundary (192 bytes) was 56 bytes ago --- */
	wait_queue_head_t          wait_queue;           /*   248    24 */
	/* --- cacheline 4 boundary (256 bytes) was 16 bytes ago --- */
	struct wake_irq *          wakeirq;              /*   272     8 */
	atomic_t                   usage_count;          /*   280     4 */
	atomic_t                   child_count;          /*   284     4 */
	unsigned int               disable_depth:3;      /*   288: 0  4 */
	unsigned int               idle_notification:1;  /*   288: 3  4 */
	unsigned int               request_pending:1;    /*   288: 4  4 */
	unsigned int               deferred_resume:1;    /*   288: 5  4 */
	unsigned int               needs_force_resume:1; /*   288: 6  4 */
	unsigned int               runtime_auto:1;       /*   288: 7  4 */

	/* Bitfield combined with next fields */

	bool                       ignore_children:1;    /*   289: 0  1 */

	/* Bitfield combined with previous fields */

	unsigned int               no_callbacks:1;       /*   288: 9  4 */
	unsigned int               irq_safe:1;           /*   288:10  4 */
	unsigned int               use_autosuspend:1;    /*   288:11  4 */
	unsigned int               timer_autosuspends:1; /*   288:12  4 */
	unsigned int               memalloc_noio:1;      /*   288:13  4 */

	/* XXX 18 bits hole, try to pack */

	unsigned int               links_count;          /*   292     4 */
	enum rpm_request           request;              /*   296     4 */
	enum rpm_status            runtime_status;       /*   300     4 */
	int                        runtime_error;        /*   304     4 */
	int                        autosuspend_delay;    /*   308     4 */
	u64                        last_busy;            /*   312     8 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	u64                        active_time;          /*   320     8 */
	u64                        suspended_time;       /*   328     8 */
	u64                        accounting_timestamp; /*   336     8 */
	struct pm_subsys_data *    subsys_data;          /*   344     8 */
	void                       (*set_latency_tolerance)(struct device *, s32); /*   352     8 */
	struct dev_pm_qos *        qos;                  /*   360     8 */
	u64                        kabi_reserved1;       /*   368     8 */
	u64                        kabi_reserved2;       /*   376     8 */

	/* size: 384, cachelines: 6, members: 54 */
	/* sum members: 368, holes: 1, sum holes: 4 */
	/* sum bitfield members: 29 bits, bit holes: 3, sum bit holes: 67 bits */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 4 */
} __attribute__((__aligned__(8)));
struct wakeup_source {
	const char  *              name;                 /*     0     8 */
	int                        id;                   /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           entry;                /*    16    16 */
	spinlock_t                 lock;                 /*    32     4 */

	/* XXX 4 bytes hole, try to pack */

	struct wake_irq *          wakeirq;              /*    40     8 */
	struct timer_list          timer;                /*    48    72 */
	/* --- cacheline 1 boundary (64 bytes) was 56 bytes ago --- */
	long unsigned int          timer_expires;        /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	ktime_t                    total_time;           /*   128     8 */
	ktime_t                    max_time;             /*   136     8 */
	ktime_t                    last_time;            /*   144     8 */
	ktime_t                    start_prevent_time;   /*   152     8 */
	ktime_t                    prevent_sleep_time;   /*   160     8 */
	long unsigned int          event_count;          /*   168     8 */
	long unsigned int          active_count;         /*   176     8 */
	long unsigned int          relax_count;          /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	long unsigned int          expire_count;         /*   192     8 */
	long unsigned int          wakeup_count;         /*   200     8 */
	struct device *            dev;                  /*   208     8 */
	bool                       active:1;             /*   216: 0  1 */
	bool                       autosleep_enabled:1;  /*   216: 1  1 */

	/* XXX 6 bits hole, try to pack */
	/* XXX 7 bytes hole, try to pack */

	u64                        kabi_reserved1;       /*   224     8 */
	u64                        kabi_reserved2;       /*   232     8 */

	/* size: 240, cachelines: 4, members: 22 */
	/* sum members: 224, holes: 3, sum holes: 15 */
	/* sum bitfield members: 2 bits, bit holes: 1, sum bit holes: 6 bits */
	/* last cacheline: 48 bytes */
};
struct dev_pm_domain {
	struct dev_pm_ops          ops;                  /*     0   184 */
	/* --- cacheline 2 boundary (128 bytes) was 56 bytes ago --- */
	int                        (*start)(struct device *); /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	void                       (*detach)(struct device *, bool); /*   192     8 */
	int                        (*activate)(struct device *); /*   200     8 */
	void                       (*sync)(struct device *); /*   208     8 */
	void                       (*dismiss)(struct device *); /*   216     8 */
	u64                        kabi_reserved1;       /*   224     8 */
	u64                        kabi_reserved2;       /*   232     8 */

	/* size: 240, cachelines: 4, members: 8 */
	/* last cacheline: 48 bytes */
};
struct bus_type {
	const char  *              name;                 /*     0     8 */
	const char  *              dev_name;             /*     8     8 */
	struct device *            dev_root;             /*    16     8 */
	const struct attribute_group  * * bus_groups;    /*    24     8 */
	const struct attribute_group  * * dev_groups;    /*    32     8 */
	const struct attribute_group  * * drv_groups;    /*    40     8 */
	int                        (*match)(struct device *, struct device_driver *); /*    48     8 */
	int                        (*uevent)(struct device *, struct kobj_uevent_env *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*probe)(struct device *); /*    64     8 */
	void                       (*sync_state)(struct device *); /*    72     8 */
	int                        (*remove)(struct device *); /*    80     8 */
	void                       (*shutdown)(struct device *); /*    88     8 */
	int                        (*online)(struct device *); /*    96     8 */
	int                        (*offline)(struct device *); /*   104     8 */
	int                        (*suspend)(struct device *, pm_message_t); /*   112     8 */
	int                        (*resume)(struct device *); /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	int                        (*num_vf)(struct device *); /*   128     8 */
	int                        (*dma_configure)(struct device *); /*   136     8 */
	const struct dev_pm_ops  * pm;                   /*   144     8 */
	const struct iommu_ops  *  iommu_ops;            /*   152     8 */
	struct subsys_private *    p;                    /*   160     8 */
	struct lock_class_key      lock_key;             /*   168     0 */
	bool                       need_parent_lock;     /*   168     1 */

	/* size: 176, cachelines: 3, members: 23 */
	/* padding: 7 */
	/* last cacheline: 48 bytes */
};
struct device_driver {
	const char  *              name;                 /*     0     8 */
	struct bus_type *          bus;                  /*     8     8 */
	struct module *            owner;                /*    16     8 */
	const char  *              mod_name;             /*    24     8 */
	bool                       suppress_bind_attrs;  /*    32     1 */

	/* XXX 3 bytes hole, try to pack */

	enum probe_type            probe_type;           /*    36     4 */
	const struct of_device_id  * of_match_table;     /*    40     8 */
	const struct acpi_device_id  * acpi_match_table; /*    48     8 */
	int                        (*probe)(struct device *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	void                       (*sync_state)(struct device *); /*    64     8 */
	int                        (*remove)(struct device *); /*    72     8 */
	void                       (*shutdown)(struct device *); /*    80     8 */
	int                        (*suspend)(struct device *, pm_message_t); /*    88     8 */
	int                        (*resume)(struct device *); /*    96     8 */
	const struct attribute_group  * * groups;        /*   104     8 */
	const struct attribute_group  * * dev_groups;    /*   112     8 */
	const struct dev_pm_ops  * pm;                   /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	void                       (*coredump)(struct device *); /*   128     8 */
	struct driver_private *    p;                    /*   136     8 */
	u64                        kabi_reserved1;       /*   144     8 */
	u64                        kabi_reserved2;       /*   152     8 */
	u64                        kabi_reserved3;       /*   160     8 */
	u64                        kabi_reserved4;       /*   168     8 */
	size_t                     device_driver_size_resvd; /*   176     8 */
	struct device_driver_resvd * _resvd;             /*   184     8 */

	/* size: 192, cachelines: 3, members: 25 */
	/* sum members: 189, holes: 1, sum holes: 3 */
};
struct device_type {
	const char  *              name;                 /*     0     8 */
	const struct attribute_group  * * groups;        /*     8     8 */
	int                        (*uevent)(struct device *, struct kobj_uevent_env *); /*    16     8 */
	char *                     (*devnode)(struct device *, umode_t *, kuid_t *, kgid_t *); /*    24     8 */
	void                       (*release)(struct device *); /*    32     8 */
	const struct dev_pm_ops  * pm;                   /*    40     8 */

	/* size: 48, cachelines: 1, members: 6 */
	/* last cacheline: 48 bytes */
};
struct class_resvd {

	/* size: 0, cachelines: 0, members: 0 */
};
struct class {
	const char  *              name;                 /*     0     8 */
	struct module *            owner;                /*     8     8 */
	const struct attribute_group  * * class_groups;  /*    16     8 */
	const struct attribute_group  * * dev_groups;    /*    24     8 */
	struct kobject *           dev_kobj;             /*    32     8 */
	int                        (*dev_uevent)(struct device *, struct kobj_uevent_env *); /*    40     8 */
	char *                     (*devnode)(struct device *, umode_t *); /*    48     8 */
	void                       (*class_release)(struct class *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	void                       (*dev_release)(struct device *); /*    64     8 */
	int                        (*shutdown_pre)(struct device *); /*    72     8 */
	const struct kobj_ns_type_operations  * ns_type; /*    80     8 */
	const void  *              (*namespace)(struct device *); /*    88     8 */
	void                       (*get_ownership)(struct device *, kuid_t *, kgid_t *); /*    96     8 */
	const struct dev_pm_ops  * pm;                   /*   104     8 */
	struct subsys_private *    p;                    /*   112     8 */
	u64                        kabi_reserved1;       /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	u64                        kabi_reserved2;       /*   128     8 */
	u64                        kabi_reserved3;       /*   136     8 */
	u64                        kabi_reserved4;       /*   144     8 */
	size_t                     class_size_resvd;     /*   152     8 */
	struct class_resvd *       _resvd;               /*   160     8 */

	/* size: 168, cachelines: 3, members: 21 */
	/* last cacheline: 40 bytes */
};
struct device_driver_resvd {

	/* size: 0, cachelines: 0, members: 0 */
};
struct of_device_id {
	char                       name[32];             /*     0    32 */
	char                       type[32];             /*    32    32 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	char                       compatible[128];      /*    64   128 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	const void  *              data;                 /*   192     8 */

	/* size: 200, cachelines: 4, members: 4 */
	/* last cacheline: 8 bytes */
};
struct acpi_device_id {
	__u8                       id[9];                /*     0     9 */

	/* XXX 7 bytes hole, try to pack */

	kernel_ulong_t             driver_data;          /*    16     8 */
	__u32                      cls;                  /*    24     4 */
	__u32                      cls_msk;              /*    28     4 */

	/* size: 32, cachelines: 1, members: 4 */
	/* sum members: 25, holes: 1, sum holes: 7 */
	/* last cacheline: 32 bytes */
};
struct dev_archdata {

	/* size: 0, cachelines: 0, members: 0 */
};
struct device_dma_parameters {
	unsigned int               max_segment_size;     /*     0     4 */
	unsigned int               min_align_mask;       /*     4     4 */
	long unsigned int          segment_boundary_mask; /*     8     8 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct dev_links_info {
	struct list_head           suppliers;            /*     0    16 */
	struct list_head           consumers;            /*    16    16 */
	struct list_head           needs_suppliers;      /*    32    16 */
	struct list_head           defer_hook;           /*    48    16 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	bool                       need_for_probe;       /*    64     1 */

	/* XXX 3 bytes hole, try to pack */

	enum dl_dev_state          status;               /*    68     4 */

	/* size: 72, cachelines: 2, members: 6 */
	/* sum members: 69, holes: 1, sum holes: 3 */
	/* last cacheline: 8 bytes */
};
struct device_extended_resvd {

	/* size: 0, cachelines: 0, members: 0 */
};
struct device_node {
	const char  *              name;                 /*     0     8 */
	phandle                    phandle;              /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	const char  *              full_name;            /*    16     8 */
	struct fwnode_handle       fwnode;               /*    24    40 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct property *          properties;           /*    64     8 */
	struct property *          deadprops;            /*    72     8 */
	struct device_node *       parent;               /*    80     8 */
	struct device_node *       child;                /*    88     8 */
	struct device_node *       sibling;              /*    96     8 */
	long unsigned int          _flags;               /*   104     8 */
	void *                     data;                 /*   112     8 */

	/* size: 120, cachelines: 2, members: 11 */
	/* sum members: 116, holes: 1, sum holes: 4 */
	/* last cacheline: 56 bytes */
};
struct fwnode_handle {
	struct fwnode_handle *     secondary;            /*     0     8 */
	const struct fwnode_operations  * ops;           /*     8     8 */
	struct device *            dev;                  /*    16     8 */
	u64                        kabi_reserved1;       /*    24     8 */
	u64                        kabi_reserved2;       /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct static_call_mod {
	struct static_call_mod *   next;                 /*     0     8 */
	struct module *            mod;                  /*     8     8 */
	struct static_call_site *  sites;                /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct ring_buffer_event {
	u32                        type_len:5;           /*     0: 0  4 */
	u32                        time_delta:27;        /*     0: 5  4 */
	u32                        array[];              /*     4     0 */

	/* size: 4, cachelines: 1, members: 3 */
	/* last cacheline: 4 bytes */
};
struct seq_buf {
	char *                     buffer;               /*     0     8 */
	size_t                     size;                 /*     8     8 */
	size_t                     len;                  /*    16     8 */
	loff_t                     readpos;              /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct trace_seq {
	char                       buffer[4096];         /*     0  4096 */
	/* --- cacheline 64 boundary (4096 bytes) --- */
	struct seq_buf             seq;                  /*  4096    32 */
	int                        full;                 /*  4128     4 */

	/* size: 4136, cachelines: 65, members: 3 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct perf_event_attr {
	__u32                      type;                 /*     0     4 */
	__u32                      size;                 /*     4     4 */
	__u64                      config;               /*     8     8 */
	union {
		__u64              sample_period;        /*    16     8 */
		__u64              sample_freq;          /*    16     8 */
	};                                               /*    16     8 */
	__u64                      sample_type;          /*    24     8 */
	__u64                      read_format;          /*    32     8 */
	__u64                      disabled:1;           /*    40: 0  8 */
	__u64                      inherit:1;            /*    40: 1  8 */
	__u64                      pinned:1;             /*    40: 2  8 */
	__u64                      exclusive:1;          /*    40: 3  8 */
	__u64                      exclude_user:1;       /*    40: 4  8 */
	__u64                      exclude_kernel:1;     /*    40: 5  8 */
	__u64                      exclude_hv:1;         /*    40: 6  8 */
	__u64                      exclude_idle:1;       /*    40: 7  8 */
	__u64                      mmap:1;               /*    40: 8  8 */
	__u64                      comm:1;               /*    40: 9  8 */
	__u64                      freq:1;               /*    40:10  8 */
	__u64                      inherit_stat:1;       /*    40:11  8 */
	__u64                      enable_on_exec:1;     /*    40:12  8 */
	__u64                      task:1;               /*    40:13  8 */
	__u64                      watermark:1;          /*    40:14  8 */
	__u64                      precise_ip:2;         /*    40:15  8 */
	__u64                      mmap_data:1;          /*    40:17  8 */
	__u64                      sample_id_all:1;      /*    40:18  8 */
	__u64                      exclude_host:1;       /*    40:19  8 */
	__u64                      exclude_guest:1;      /*    40:20  8 */
	__u64                      exclude_callchain_kernel:1; /*    40:21  8 */
	__u64                      exclude_callchain_user:1; /*    40:22  8 */
	__u64                      mmap2:1;              /*    40:23  8 */
	__u64                      comm_exec:1;          /*    40:24  8 */
	__u64                      use_clockid:1;        /*    40:25  8 */
	__u64                      context_switch:1;     /*    40:26  8 */
	__u64                      write_backward:1;     /*    40:27  8 */
	__u64                      namespaces:1;         /*    40:28  8 */
	__u64                      ksymbol:1;            /*    40:29  8 */
	__u64                      bpf_event:1;          /*    40:30  8 */
	__u64                      aux_output:1;         /*    40:31  8 */
	__u64                      cgroup:1;             /*    40:32  8 */
	__u64                      text_poke:1;          /*    40:33  8 */
	__u64                      __reserved_1:30;      /*    40:34  8 */
	union {
		__u32              wakeup_events;        /*    48     4 */
		__u32              wakeup_watermark;     /*    48     4 */
	};                                               /*    48     4 */
	__u32                      bp_type;              /*    52     4 */
	union {
		__u64              bp_addr;              /*    56     8 */
		__u64              kprobe_func;          /*    56     8 */
		__u64              uprobe_path;          /*    56     8 */
		__u64              config1;              /*    56     8 */
	};                                               /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	union {
		__u64              bp_len;               /*    64     8 */
		__u64              kprobe_addr;          /*    64     8 */
		__u64              probe_offset;         /*    64     8 */
		__u64              config2;              /*    64     8 */
	};                                               /*    64     8 */
	__u64                      branch_sample_type;   /*    72     8 */
	__u64                      sample_regs_user;     /*    80     8 */
	__u32                      sample_stack_user;    /*    88     4 */
	__s32                      clockid;              /*    92     4 */
	__u64                      sample_regs_intr;     /*    96     8 */
	__u32                      aux_watermark;        /*   104     4 */
	__u16                      sample_max_stack;     /*   108     2 */
	__u16                      __reserved_2;         /*   110     2 */
	__u32                      aux_sample_size;      /*   112     4 */
	__u32                      __reserved_3;         /*   116     4 */

	/* size: 120, cachelines: 2, members: 54 */
	/* last cacheline: 56 bytes */
};
union perf_mem_data_src {
	__u64                      val;                /*     0     8 */
	struct {
		__u64              mem_op:5;           /*     0: 0  8 */
		__u64              mem_lvl:14;         /*     0: 5  8 */
		__u64              mem_snoop:5;        /*     0:19  8 */
		__u64              mem_lock:2;         /*     0:24  8 */
		__u64              mem_dtlb:7;         /*     0:26  8 */
		__u64              mem_lvl_num:4;      /*     0:33  8 */
		__u64              mem_remote:1;       /*     0:37  8 */
		__u64              mem_snoopx:2;       /*     0:38  8 */
		__u64              mem_blk:3;          /*     0:40  8 */
		__u64              mem_hops:3;         /*     0:43  8 */
		__u64              mem_rsvd:18;        /*     0:46  8 */
	};                                             /*     0     8 */
};
struct perf_branch_entry {
	__u64                      from;                 /*     0     8 */
	__u64                      to;                   /*     8     8 */
	__u64                      mispred:1;            /*    16: 0  8 */
	__u64                      predicted:1;          /*    16: 1  8 */
	__u64                      in_tx:1;              /*    16: 2  8 */
	__u64                      abort:1;              /*    16: 3  8 */
	__u64                      cycles:16;            /*    16: 4  8 */
	__u64                      type:4;               /*    16:20  8 */
	__u64                      reserved:40;          /*    16:24  8 */

	/* size: 24, cachelines: 1, members: 9 */
	/* last cacheline: 24 bytes */
};
union perf_sample_weight {
	__u64                      full;               /*     0     8 */
	struct {
		__u32              var1_dw;            /*     0     4 */
		__u16              var2_w;             /*     4     2 */
		__u16              var3_w;             /*     6     2 */
	};                                             /*     0     8 */
};
struct uts_namespace {
	struct kref                kref;                 /*     0     4 */
	struct new_utsname         name;                 /*     4   390 */

	/* XXX 6 bytes hole, try to pack */

	/* --- cacheline 6 boundary (384 bytes) was 16 bytes ago --- */
	struct user_namespace *    user_ns;              /*   400     8 */
	struct ucounts *           ucounts;              /*   408     8 */
	struct ns_common           ns;                   /*   416    32 */

	/* size: 448, cachelines: 7, members: 5 */
	/* sum members: 442, holes: 1, sum holes: 6 */
};
struct cgroup_namespace {
	refcount_t                 count;                /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct ns_common           ns;                   /*     8    32 */
	struct user_namespace *    user_ns;              /*    40     8 */
	struct ucounts *           ucounts;              /*    48     8 */
	struct css_set *           root_cset;            /*    56     8 */

	/* size: 64, cachelines: 1, members: 5 */
	/* sum members: 60, holes: 1, sum holes: 4 */
};
struct nsset {
	unsigned int               flags;                /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct nsproxy *           nsproxy;              /*     8     8 */
	struct fs_struct *         fs;                   /*    16     8 */
	const struct cred  *       cred;                 /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct ns_common {
	atomic_long_t              stashed;              /*     0     8 */
	const struct proc_ns_operations  * ops;          /*     8     8 */
	unsigned int               inum;                 /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        kabi_reserved1;       /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct proc_ns_operations {
	const char  *              name;                 /*     0     8 */
	const char  *              real_ns_name;         /*     8     8 */
	u64                        type;                 /*    16     8 */
	struct ns_common *         (*get)(struct task_struct *); /*    24     8 */
	void                       (*put)(struct ns_common *); /*    32     8 */
	int                        (*install)(struct nsset *, struct ns_common *); /*    40     8 */
	struct user_namespace *    (*owner)(struct ns_common *); /*    48     8 */
	struct ns_common *         (*get_parent)(struct ns_common *); /*    56     8 */

	/* size: 64, cachelines: 1, members: 8 */
};
struct ucounts {
	struct hlist_node          node;                 /*     0    16 */
	struct user_namespace *    ns;                   /*    16     8 */
	kuid_t                     uid;                  /*    24     4 */
	int                        count;                /*    28     4 */
	atomic_long_t              ucount[25];           /*    32   200 */

	/* size: 232, cachelines: 4, members: 5 */
	/* last cacheline: 40 bytes */
};
struct arch_hw_breakpoint {
	long unsigned int          address;              /*     0     8 */
	long unsigned int          mask;                 /*     8     8 */
	u8                         len;                  /*    16     1 */
	u8                         type;                 /*    17     1 */

	/* size: 24, cachelines: 1, members: 4 */
	/* padding: 6 */
	/* last cacheline: 24 bytes */
};
struct pmu {
	struct list_head           entry;                /*     0    16 */
	struct module *            module;               /*    16     8 */
	struct device *            dev;                  /*    24     8 */
	const struct attribute_group  * * attr_groups;   /*    32     8 */
	const struct attribute_group  * * attr_update;   /*    40     8 */
	const char  *              name;                 /*    48     8 */
	int                        type;                 /*    56     4 */
	int                        capabilities;         /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int *                      pmu_disable_count;    /*    64     8 */
	struct perf_cpu_context *  pmu_cpu_context;      /*    72     8 */
	atomic_t                   exclusive_cnt;        /*    80     4 */
	int                        task_ctx_nr;          /*    84     4 */
	int                        hrtimer_interval_ms;  /*    88     4 */
	unsigned int               nr_addr_filters;      /*    92     4 */
	void                       (*pmu_enable)(struct pmu *); /*    96     8 */
	void                       (*pmu_disable)(struct pmu *); /*   104     8 */
	int                        (*event_init)(struct perf_event *); /*   112     8 */
	void                       (*event_mapped)(struct perf_event *, struct mm_struct *); /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	void                       (*event_unmapped)(struct perf_event *, struct mm_struct *); /*   128     8 */
	int                        (*add)(struct perf_event *, int); /*   136     8 */
	void                       (*del)(struct perf_event *, int); /*   144     8 */
	void                       (*start)(struct perf_event *, int); /*   152     8 */
	void                       (*stop)(struct perf_event *, int); /*   160     8 */
	void                       (*read)(struct perf_event *); /*   168     8 */
	void                       (*start_txn)(struct pmu *, unsigned int); /*   176     8 */
	int                        (*commit_txn)(struct pmu *); /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	void                       (*cancel_txn)(struct pmu *); /*   192     8 */
	int                        (*event_idx)(struct perf_event *); /*   200     8 */
	void                       (*sched_task)(struct perf_event_context *, bool); /*   208     8 */
	struct kmem_cache *        task_ctx_cache;       /*   216     8 */
	void                       (*swap_task_ctx)(struct perf_event_context *, struct perf_event_context *); /*   224     8 */
	void *                     (*setup_aux)(struct perf_event *, void * *, int, bool); /*   232     8 */
	void                       (*free_aux)(void *);  /*   240     8 */
	long int                   (*snapshot_aux)(struct perf_event *, struct perf_output_handle *, long unsigned int); /*   248     8 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	int                        (*addr_filters_validate)(struct list_head *); /*   256     8 */
	void                       (*addr_filters_sync)(struct perf_event *); /*   264     8 */
	int                        (*aux_output_match)(struct perf_event *); /*   272     8 */
	int                        (*filter_match)(struct perf_event *); /*   280     8 */
	int                        (*check_period)(struct perf_event *, u64); /*   288     8 */
	u64                        kabi_reserved1;       /*   296     8 */
	u64                        kabi_reserved2;       /*   304     8 */
	u64                        kabi_reserved3;       /*   312     8 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	u64                        kabi_reserved4;       /*   320     8 */
	u64                        kabi_reserved5;       /*   328     8 */
	u64                        kabi_reserved6;       /*   336     8 */

	/* size: 344, cachelines: 6, members: 45 */
	/* last cacheline: 24 bytes */
};
struct iovec {
	void *                     iov_base;             /*     0     8 */
	__kernel_size_t            iov_len;              /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct kvec {
	void *                     iov_base;             /*     0     8 */
	size_t                     iov_len;              /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct bio_vec {
	struct page *              bv_page;              /*     0     8 */
	unsigned int               bv_len;               /*     8     4 */
	unsigned int               bv_offset;            /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct compat_robust_list {
	compat_uptr_t              next;                 /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct ftrace_ops {
	ftrace_func_t              func;                 /*     0     8 */
	struct ftrace_ops *        next;                 /*     8     8 */
	long unsigned int          flags;                /*    16     8 */
	void *                     private;              /*    24     8 */
	ftrace_func_t              saved_func;           /*    32     8 */
	struct ftrace_ops_hash     local_hash;           /*    40    48 */
	/* --- cacheline 1 boundary (64 bytes) was 24 bytes ago --- */
	struct ftrace_ops_hash *   func_hash;            /*    88     8 */
	struct ftrace_ops_hash     old_hash;             /*    96    48 */
	/* --- cacheline 2 boundary (128 bytes) was 16 bytes ago --- */
	long unsigned int          trampoline;           /*   144     8 */
	long unsigned int          trampoline_size;      /*   152     8 */
	struct list_head           list;                 /*   160    16 */

	/* size: 176, cachelines: 3, members: 11 */
	/* last cacheline: 48 bytes */
};
struct ftrace_ops_hash {
	struct ftrace_hash *       notrace_hash;         /*     0     8 */
	struct ftrace_hash *       filter_hash;          /*     8     8 */
	struct mutex               regex_lock;           /*    16    32 */

	/* size: 48, cachelines: 1, members: 3 */
	/* last cacheline: 48 bytes */
};
struct irq_work {
	struct __call_single_node  node;                 /*     0    16 */
	void                       (*func)(struct irq_work *); /*    16     8 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct perf_regs {
	__u64                      abi;                  /*     0     8 */
	struct pt_regs *           regs;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct uid_gid_extent {
	u32                        first;                /*     0     4 */
	u32                        lower_first;          /*     4     4 */
	u32                        count;                /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct uid_gid_map {
	u32                        nr_extents;           /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	union {
		struct uid_gid_extent extent[5];         /*     8    60 */
		struct {
			struct uid_gid_extent * forward; /*     8     8 */
			struct uid_gid_extent * reverse; /*    16     8 */
		};                                       /*     8    16 */
	};                                               /*     8    64 */

	/* size: 72, cachelines: 2, members: 2 */
	/* sum members: 68, holes: 1, sum holes: 4 */
	/* last cacheline: 8 bytes */
};
struct u64_stats_sync {

	/* size: 0, cachelines: 0, members: 0 */
};
struct bpf_cgroup_storage_key {
	__u64                      cgroup_inode_id;      /*     0     8 */
	__u32                      attach_type;          /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct bpf_prog_array_item {
	struct bpf_prog *          prog;                 /*     0     8 */
	struct bpf_cgroup_storage * cgroup_storage[10];  /*     8    80 */

	/* size: 88, cachelines: 2, members: 2 */
	/* last cacheline: 24 bytes */
};
struct bpf_cgroup_storage {
	union {
		struct bpf_storage_buffer * buf;         /*     0     8 */
		void *             percpu_buf;           /*     0     8 */
	};                                               /*     0     8 */
	struct bpf_cgroup_storage_map * map;             /*     8     8 */
	struct bpf_cgroup_storage_key key;               /*    16    16 */

	/* XXX last struct has 4 bytes of padding */

	struct list_head           list_map;             /*    32    16 */
	struct list_head           list_cg;              /*    48    16 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct rb_node             node __attribute__((__aligned__(8))); /*    64    24 */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*    88    16 */

	/* size: 104, cachelines: 2, members: 7 */
	/* paddings: 1, sum paddings: 4 */
	/* forced alignments: 2 */
	/* last cacheline: 40 bytes */
} __attribute__((__aligned__(8)));
struct bpf_prog_array {
	struct callback_head       rcu __attribute__((__aligned__(8))); /*     0    16 */
	struct bpf_prog_array_item items[];              /*    16     0 */

	/* size: 16, cachelines: 1, members: 2 */
	/* forced alignments: 1 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct bpf_storage_buffer {
	struct callback_head       rcu __attribute__((__aligned__(8))); /*     0    16 */
	char                       data[];               /*    16     0 */

	/* size: 16, cachelines: 1, members: 2 */
	/* forced alignments: 1 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct cgroup_bpf {
	struct bpf_prog_array *    effective[31];        /*     0   248 */
	/* --- cacheline 3 boundary (192 bytes) was 56 bytes ago --- */
	struct list_head           progs[31];            /*   248   496 */
	/* --- cacheline 11 boundary (704 bytes) was 40 bytes ago --- */
	u32                        flags[31];            /*   744   124 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 13 boundary (832 bytes) was 40 bytes ago --- */
	struct list_head           storages;             /*   872    16 */
	struct bpf_prog_array *    inactive;             /*   888     8 */
	/* --- cacheline 14 boundary (896 bytes) --- */
	struct percpu_ref          refcnt;               /*   896    16 */
	struct work_struct         release_work;         /*   912    64 */
	/* --- cacheline 15 boundary (960 bytes) was 16 bytes ago --- */
	u64                        kabi_reserved1;       /*   976     8 */
	u64                        kabi_reserved2;       /*   984     8 */
	u64                        kabi_reserved3;       /*   992     8 */
	u64                        kabi_reserved4;       /*  1000     8 */
	u64                        kabi_reserved5;       /*  1008     8 */
	u64                        kabi_reserved6;       /*  1016     8 */
	/* --- cacheline 16 boundary (1024 bytes) --- */
	u64                        kabi_reserved7;       /*  1024     8 */
	u64                        kabi_reserved8;       /*  1032     8 */
	u64                        kabi_reserved9;       /*  1040     8 */
	u64                        kabi_reserved10;      /*  1048     8 */
	u64                        kabi_reserved11;      /*  1056     8 */
	u64                        kabi_reserved12;      /*  1064     8 */
	u64                        kabi_reserved13;      /*  1072     8 */
	u64                        kabi_reserved14;      /*  1080     8 */
	/* --- cacheline 17 boundary (1088 bytes) --- */
	u64                        kabi_reserved15;      /*  1088     8 */
	u64                        kabi_reserved16;      /*  1096     8 */

	/* size: 1104, cachelines: 18, members: 23 */
	/* sum members: 1100, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct psi_group_cpu {
	seqcount_t                 seq __attribute__((__aligned__(64))); /*     0     4 */
	unsigned int               tasks[5];             /*     4    20 */
	u32                        state_mask;           /*    24     4 */
	u32                        times[7];             /*    28    28 */
	u64                        state_start;          /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u32                        times_prev[2][7] __attribute__((__aligned__(64))); /*    64    56 */

	/* size: 128, cachelines: 2, members: 6 */
	/* padding: 8 */
	/* forced alignments: 2 */
} __attribute__((__aligned__(64)));
struct psi_group {
	struct mutex               avgs_lock;            /*     0    32 */
	struct psi_group_cpu *     pcpu;                 /*    32     8 */
	u64                        avg_total[6];         /*    40    48 */
	/* --- cacheline 1 boundary (64 bytes) was 24 bytes ago --- */
	u64                        avg_last_update;      /*    88     8 */
	u64                        avg_next_update;      /*    96     8 */
	struct delayed_work        avgs_work;            /*   104   192 */
	/* --- cacheline 4 boundary (256 bytes) was 40 bytes ago --- */
	u64                        total[2][6];          /*   296    96 */
	/* --- cacheline 6 boundary (384 bytes) was 8 bytes ago --- */
	long unsigned int          avg[6][3];            /*   392   144 */
	/* --- cacheline 8 boundary (512 bytes) was 24 bytes ago --- */
	struct task_struct *       poll_task;            /*   536     8 */
	struct timer_list          poll_timer;           /*   544    72 */
	/* --- cacheline 9 boundary (576 bytes) was 40 bytes ago --- */
	wait_queue_head_t          poll_wait;            /*   616    24 */
	/* --- cacheline 10 boundary (640 bytes) --- */
	atomic_t                   poll_wakeup;          /*   640     4 */

	/* XXX 4 bytes hole, try to pack */

	struct mutex               trigger_lock;         /*   648    32 */
	struct list_head           triggers;             /*   680    16 */
	u32                        nr_triggers[6];       /*   696    24 */
	/* --- cacheline 11 boundary (704 bytes) was 16 bytes ago --- */
	u32                        poll_states;          /*   720     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        poll_min_period;      /*   728     8 */
	u64                        polling_total[6];     /*   736    48 */
	/* --- cacheline 12 boundary (768 bytes) was 16 bytes ago --- */
	u64                        polling_next_update;  /*   784     8 */
	u64                        polling_until;        /*   792     8 */

	/* size: 800, cachelines: 13, members: 20 */
	/* sum members: 792, holes: 2, sum holes: 8 */
	/* last cacheline: 32 bytes */
};
struct cgroup_file {
	struct kernfs_node *       kn;                   /*     0     8 */
	long unsigned int          notified_at;          /*     8     8 */
	struct timer_list          notify_timer;         /*    16    72 */
	/* --- cacheline 1 boundary (64 bytes) was 24 bytes ago --- */
	u64                        kabi_reserved1;       /*    88     8 */
	u64                        kabi_reserved2;       /*    96     8 */

	/* size: 104, cachelines: 2, members: 5 */
	/* last cacheline: 40 bytes */
};
struct cgroup_subsys_state {
	struct cgroup *            cgroup;               /*     0     8 */
	struct cgroup_subsys *     ss;                   /*     8     8 */
	struct percpu_ref          refcnt;               /*    16    16 */
	struct list_head           sibling;              /*    32    16 */
	struct list_head           children;             /*    48    16 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct list_head           rstat_css_node;       /*    64    16 */
	int                        id;                   /*    80     4 */
	unsigned int               flags;                /*    84     4 */
	u64                        serial_nr;            /*    88     8 */
	atomic_t                   online_cnt;           /*    96     4 */

	/* XXX 4 bytes hole, try to pack */

	struct work_struct         destroy_work;         /*   104    64 */
	/* --- cacheline 2 boundary (128 bytes) was 40 bytes ago --- */
	struct rcu_work            destroy_rwork __attribute__((__aligned__(8))); /*   168    88 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	struct cgroup_subsys_state * parent;             /*   256     8 */
	u64                        kabi_reserved1;       /*   264     8 */
	u64                        kabi_reserved2;       /*   272     8 */
	u64                        kabi_reserved3;       /*   280     8 */
	u64                        kabi_reserved4;       /*   288     8 */

	/* size: 296, cachelines: 5, members: 17 */
	/* sum members: 292, holes: 1, sum holes: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 40 bytes */
} __attribute__((__aligned__(8)));
struct cgroup_subsys {
	struct cgroup_subsys_state * (*css_alloc)(struct cgroup_subsys_state *); /*     0     8 */
	int                        (*css_online)(struct cgroup_subsys_state *); /*     8     8 */
	void                       (*css_offline)(struct cgroup_subsys_state *); /*    16     8 */
	void                       (*css_released)(struct cgroup_subsys_state *); /*    24     8 */
	void                       (*css_free)(struct cgroup_subsys_state *); /*    32     8 */
	void                       (*css_reset)(struct cgroup_subsys_state *); /*    40     8 */
	void                       (*css_rstat_flush)(struct cgroup_subsys_state *, int); /*    48     8 */
	int                        (*css_extra_stat_show)(struct seq_file *, struct cgroup_subsys_state *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*can_attach)(struct cgroup_taskset *); /*    64     8 */
	void                       (*cancel_attach)(struct cgroup_taskset *); /*    72     8 */
	void                       (*attach)(struct cgroup_taskset *); /*    80     8 */
	void                       (*post_attach)(void); /*    88     8 */
	int                        (*can_fork)(struct task_struct *, struct css_set *); /*    96     8 */
	void                       (*cancel_fork)(struct task_struct *, struct css_set *); /*   104     8 */
	void                       (*fork)(struct task_struct *); /*   112     8 */
	void                       (*exit)(struct task_struct *); /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	void                       (*release)(struct task_struct *); /*   128     8 */
	void                       (*bind)(struct cgroup_subsys_state *); /*   136     8 */
	u64                        kabi_reserved1;       /*   144     8 */
	u64                        kabi_reserved2;       /*   152     8 */
	u64                        kabi_reserved3;       /*   160     8 */
	u64                        kabi_reserved4;       /*   168     8 */
	bool                       early_init:1;         /*   176: 0  1 */
	bool                       implicit_on_dfl:1;    /*   176: 1  1 */
	bool                       threaded:1;           /*   176: 2  1 */
	bool                       broken_hierarchy:1;   /*   176: 3  1 */
	bool                       warned_broken_hierarchy:1; /*   176: 4  1 */

	/* XXX 3 bits hole, try to pack */
	/* XXX 3 bytes hole, try to pack */

	int                        id;                   /*   180     4 */
	const char  *              name;                 /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	const char  *              legacy_name;          /*   192     8 */
	struct cgroup_root *       root;                 /*   200     8 */
	struct idr                 css_idr;              /*   208    24 */
	struct list_head           cfts;                 /*   232    16 */
	struct cftype *            dfl_cftypes;          /*   248     8 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	struct cftype *            legacy_cftypes;       /*   256     8 */
	struct cftype *            base_cftypes;         /*   264     8 */
	unsigned int               depends_on;           /*   272     4 */

	/* size: 280, cachelines: 5, members: 37 */
	/* sum members: 272, holes: 1, sum holes: 3 */
	/* sum bitfield members: 5 bits, bit holes: 1, sum bit holes: 3 bits */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct cgroup_base_stat {
	struct task_cputime        cputime;              /*     0    24 */

	/* size: 24, cachelines: 1, members: 1 */
	/* last cacheline: 24 bytes */
};
struct cgroup_rstat_cpu {
	struct u64_stats_sync      bsync;                /*     0     0 */
	struct cgroup_base_stat    bstat;                /*     0    24 */
	struct cgroup_base_stat    last_bstat;           /*    24    24 */
	struct cgroup *            updated_children;     /*    48     8 */
	struct cgroup *            updated_next;         /*    56     8 */

	/* size: 64, cachelines: 1, members: 5 */
};
struct cgroup_freezer_state {
	bool                       freeze;               /*     0     1 */

	/* XXX 3 bytes hole, try to pack */

	int                        e_freeze;             /*     4     4 */
	int                        nr_frozen_descendants; /*     8     4 */
	int                        nr_frozen_tasks;      /*    12     4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* sum members: 13, holes: 1, sum holes: 3 */
	/* last cacheline: 16 bytes */
};
struct cgroup_root {
	struct kernfs_root *       kf_root;              /*     0     8 */
	unsigned int               subsys_mask;          /*     8     4 */
	int                        hierarchy_id;         /*    12     4 */
	struct cgroup              cgrp __attribute__((__aligned__(8))); /*    16  3128 */
	/* --- cacheline 49 boundary (3136 bytes) was 8 bytes ago --- */
	u64                        cgrp_ancestor_id_storage; /*  3144     8 */
	atomic_t                   nr_cgrps;             /*  3152     4 */

	/* XXX 4 bytes hole, try to pack */

	wait_queue_head_t          wait;                 /*  3160    24 */
	struct list_head           root_list;            /*  3184    16 */
	/* --- cacheline 50 boundary (3200 bytes) --- */
	unsigned int               flags;                /*  3200     4 */
	char                       release_agent_path[4096]; /*  3204  4096 */
	/* --- cacheline 114 boundary (7296 bytes) was 4 bytes ago --- */
	char                       name[64];             /*  7300    64 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 115 boundary (7360 bytes) was 8 bytes ago --- */
	u64                        kabi_reserved1;       /*  7368     8 */
	u64                        kabi_reserved2;       /*  7376     8 */
	u64                        kabi_reserved3;       /*  7384     8 */
	u64                        kabi_reserved4;       /*  7392     8 */

	/* size: 7400, cachelines: 116, members: 15 */
	/* sum members: 7392, holes: 2, sum holes: 8 */
	/* forced alignments: 1 */
	/* last cacheline: 40 bytes */
} __attribute__((__aligned__(8)));
struct cftype {
	char                       name[64];             /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	long unsigned int          private;              /*    64     8 */
	size_t                     max_write_len;        /*    72     8 */
	unsigned int               flags;                /*    80     4 */
	unsigned int               file_offset;          /*    84     4 */
	struct cgroup_subsys *     ss;                   /*    88     8 */
	struct list_head           node;                 /*    96    16 */
	struct kernfs_ops *        kf_ops;               /*   112     8 */
	int                        (*open)(struct kernfs_open_file *); /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	void                       (*release)(struct kernfs_open_file *); /*   128     8 */
	u64                        (*read_u64)(struct cgroup_subsys_state *, struct cftype *); /*   136     8 */
	s64                        (*read_s64)(struct cgroup_subsys_state *, struct cftype *); /*   144     8 */
	int                        (*seq_show)(struct seq_file *, void *); /*   152     8 */
	void *                     (*seq_start)(struct seq_file *, loff_t *); /*   160     8 */
	void *                     (*seq_next)(struct seq_file *, void *, loff_t *); /*   168     8 */
	void                       (*seq_stop)(struct seq_file *, void *); /*   176     8 */
	int                        (*write_u64)(struct cgroup_subsys_state *, struct cftype *, u64); /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	int                        (*write_s64)(struct cgroup_subsys_state *, struct cftype *, s64); /*   192     8 */
	ssize_t                    (*write)(struct kernfs_open_file *, char *, size_t, loff_t); /*   200     8 */
	int                        (*read_seq_string)(struct cgroup *, struct cftype *, struct seq_file *); /*   208     8 */
	int                        (*write_string)(struct cgroup *, struct cftype *, const char  *); /*   216     8 */
	__poll_t                   (*poll)(struct kernfs_open_file *, struct poll_table_struct *); /*   224     8 */
	u64                        kabi_reserved1;       /*   232     8 */

	/* size: 240, cachelines: 4, members: 23 */
	/* last cacheline: 48 bytes */
};
struct perf_callchain_entry {
	__u64                      nr;                   /*     0     8 */
	__u64                      ip[];                 /*     8     0 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct perf_raw_frag {
	union {
		struct perf_raw_frag * next;             /*     0     8 */
		long unsigned int  pad;                  /*     0     8 */
	};                                               /*     0     8 */
	perf_copy_f                copy;                 /*     8     8 */
	void *                     data;                 /*    16     8 */
	u32                        size;                 /*    24     4 */

	/* size: 28, cachelines: 1, members: 4 */
	/* last cacheline: 28 bytes */
} __attribute__((__packed__));
struct perf_raw_record {
	struct perf_raw_frag       frag;                 /*     0    28 */
	u32                        size;                 /*    28     4 */

	/* size: 32, cachelines: 1, members: 2 */
	/* last cacheline: 32 bytes */
};
struct perf_branch_stack {
	__u64                      nr;                   /*     0     8 */
	__u64                      hw_idx;               /*     8     8 */
	struct perf_branch_entry   entries[];            /*    16     0 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct hw_perf_event_extra {
	u64                        config;               /*     0     8 */
	unsigned int               reg;                  /*     8     4 */
	int                        alloc;                /*    12     4 */
	int                        idx;                  /*    16     4 */

	/* size: 24, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct hw_perf_event {
	union {
		struct {
			u64        config;               /*     0     8 */
			u64        last_tag;             /*     8     8 */
			long unsigned int config_base;   /*    16     8 */
			long unsigned int event_base;    /*    24     8 */
			int        event_base_rdpmc;     /*    32     4 */
			int        idx;                  /*    36     4 */
			int        last_cpu;             /*    40     4 */
			int        flags;                /*    44     4 */
			struct hw_perf_event_extra extra_reg; /*    48    24 */

			/* XXX last struct has 4 bytes of padding */

			/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
			struct hw_perf_event_extra branch_reg; /*    72    24 */

			/* XXX last struct has 4 bytes of padding */
		};                                       /*     0    96 */
		struct {
			struct hrtimer hrtimer __attribute__((__aligned__(8))); /*     0    96 */
		} __attribute__((__aligned__(8))) __attribute__((__aligned__(8)));       /*     0    96 */
		struct {
			struct list_head tp_list;        /*     0    16 */
		};                                       /*     0    16 */
		struct {
			u64        pwr_acc;              /*     0     8 */
			u64        ptsc;                 /*     8     8 */
		};                                       /*     0    16 */
		struct {
			struct arch_hw_breakpoint info;  /*     0    24 */

			/* XXX last struct has 6 bytes of padding */

			struct list_head bp_list;        /*    24    16 */
		};                                       /*     0    40 */
		struct {
			u8         iommu_bank;           /*     0     1 */
			u8         iommu_cntr;           /*     1     1 */
			u16        padding;              /*     2     2 */

			/* XXX 4 bytes hole, try to pack */

			u64        conf;                 /*     8     8 */
			u64        conf1;                /*    16     8 */
		};                                       /*     0    24 */
	} __attribute__((__aligned__(8)));               /*     0    96 */
	/* --- cacheline 1 boundary (64 bytes) was 32 bytes ago --- */
	struct task_struct *       target;               /*    96     8 */
	void *                     addr_filters;         /*   104     8 */
	long unsigned int          addr_filters_gen;     /*   112     8 */
	int                        state;                /*   120     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 2 boundary (128 bytes) --- */
	local64_t                  prev_count;           /*   128     8 */
	u64                        sample_period;        /*   136     8 */
	union {
		struct {
			u64        last_period;          /*   144     8 */
			local64_t  period_left;          /*   152     8 */
		};                                       /*   144    16 */
		struct {
			u64        saved_metric;         /*   144     8 */
			u64        saved_slots;          /*   152     8 */
		};                                       /*   144    16 */
	};                                               /*   144    16 */
	u64                        interrupts_seq;       /*   160     8 */
	u64                        interrupts;           /*   168     8 */
	u64                        freq_time_stamp;      /*   176     8 */
	u64                        freq_count_stamp;     /*   184     8 */

	/* size: 192, cachelines: 3, members: 12 */
	/* sum members: 188, holes: 1, sum holes: 4 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(8)));
struct perf_cpu_context {
	struct perf_event_context  ctx __attribute__((__aligned__(8))); /*     0   256 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	struct perf_event_context * task_ctx;            /*   256     8 */
	int                        active_oncpu;         /*   264     4 */
	int                        exclusive;            /*   268     4 */
	raw_spinlock_t             hrtimer_lock;         /*   272     4 */

	/* XXX 4 bytes hole, try to pack */

	struct hrtimer             hrtimer __attribute__((__aligned__(8))); /*   280    96 */
	/* --- cacheline 5 boundary (320 bytes) was 56 bytes ago --- */
	ktime_t                    hrtimer_interval;     /*   376     8 */
	/* --- cacheline 6 boundary (384 bytes) --- */
	unsigned int               hrtimer_active;       /*   384     4 */

	/* XXX 4 bytes hole, try to pack */

	struct perf_cgroup *       cgrp;                 /*   392     8 */
	struct list_head           cgrp_cpuctx_entry;    /*   400    16 */
	struct list_head           sched_cb_entry;       /*   416    16 */
	int                        sched_cb_usage;       /*   432     4 */
	int                        online;               /*   436     4 */
	int                        heap_size;            /*   440     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 7 boundary (448 bytes) --- */
	struct perf_event * *      heap;                 /*   448     8 */
	struct perf_event *        heap_default[2];      /*   456    16 */

	/* size: 472, cachelines: 8, members: 16 */
	/* sum members: 460, holes: 3, sum holes: 12 */
	/* forced alignments: 2, forced holes: 1, sum forced holes: 4 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct perf_output_handle {
	struct perf_event *        event;                /*     0     8 */
	struct perf_buffer *       rb;                   /*     8     8 */
	long unsigned int          wakeup;               /*    16     8 */
	long unsigned int          size;                 /*    24     8 */
	u64                        aux_flags;            /*    32     8 */
	union {
		void *             addr;                 /*    40     8 */
		long unsigned int  head;                 /*    40     8 */
	};                                               /*    40     8 */
	int                        page;                 /*    48     4 */

	/* size: 56, cachelines: 1, members: 7 */
	/* padding: 4 */
	/* last cacheline: 56 bytes */
};
struct perf_addr_filters_head {
	struct list_head           list;                 /*     0    16 */
	raw_spinlock_t             lock;                 /*    16     4 */
	unsigned int               nr_file_filters;      /*    20     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct perf_addr_filter_range {
	long unsigned int          start;                /*     0     8 */
	long unsigned int          size;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct perf_sample_data {
	u64                        addr;                 /*     0     8 */
	struct perf_raw_record *   raw;                  /*     8     8 */
	struct perf_branch_stack * br_stack;             /*    16     8 */
	u64                        period;               /*    24     8 */
	union {
		union perf_sample_weight weight;         /*    32     8 */
		struct {
			u64        weight;               /*    32     8 */
		} kabi_hidden_1013;                      /*    32     8 */
		union {
		};                                       /*    32     0 */
	};                                               /*    32     8 */
	u64                        txn;                  /*    40     8 */
	union perf_mem_data_src    data_src;             /*    48     8 */
	u64                        type;                 /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u64                        ip;                   /*    64     8 */
	struct {
		u32                pid;                  /*    72     4 */
		u32                tid;                  /*    76     4 */
	} tid_entry;                                     /*    72     8 */
	u64                        time;                 /*    80     8 */
	u64                        id;                   /*    88     8 */
	u64                        stream_id;            /*    96     8 */
	struct {
		u32                cpu;                  /*   104     4 */
		u32                reserved;             /*   108     4 */
	} cpu_entry;                                     /*   104     8 */
	struct perf_callchain_entry * callchain;         /*   112     8 */
	u64                        aux_size;             /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct perf_regs           regs_user;            /*   128    16 */
	struct perf_regs           regs_intr;            /*   144    16 */
	u64                        stack_user_size;      /*   160     8 */
	u64                        phys_addr;            /*   168     8 */
	u64                        cgroup;               /*   176     8 */
	u64                        sample_flags;         /*   184     8 */

	/* size: 192, cachelines: 3, members: 22 */
} __attribute__((__aligned__(64)));
struct perf_cgroup {
	struct cgroup_subsys_state css __attribute__((__aligned__(8))); /*     0   296 */
	/* --- cacheline 4 boundary (256 bytes) was 40 bytes ago --- */
	struct perf_cgroup_info *  info;                 /*   296     8 */

	/* size: 304, cachelines: 5, members: 2 */
	/* forced alignments: 1 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct perf_event_groups {
	struct rb_root             tree;                 /*     0     8 */
	u64                        index;                /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct perf_cgroup_info {
	u64                        time;                 /*     0     8 */
	u64                        timestamp;            /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct trace_entry {
	short unsigned int         type;                 /*     0     2 */
	unsigned char              flags;                /*     2     1 */
	unsigned char              preempt_count;        /*     3     1 */
	int                        pid;                  /*     4     4 */

	/* size: 8, cachelines: 1, members: 4 */
	/* last cacheline: 8 bytes */
};
struct trace_iterator {
	struct trace_array *       tr;                   /*     0     8 */
	struct tracer *            trace;                /*     8     8 */
	struct array_buffer *      array_buffer;         /*    16     8 */
	void *                     private;              /*    24     8 */
	int                        cpu_file;             /*    32     4 */

	/* XXX 4 bytes hole, try to pack */

	struct mutex               mutex;                /*    40    32 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	struct ring_buffer_iter * * buffer_iter;         /*    72     8 */
	long unsigned int          iter_flags;           /*    80     8 */
	void *                     temp;                 /*    88     8 */
	unsigned int               temp_size;            /*    96     4 */

	/* XXX 4 bytes hole, try to pack */

	struct trace_seq           tmp_seq;              /*   104  4136 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 66 boundary (4224 bytes) was 16 bytes ago --- */
	cpumask_var_t              started;              /*  4240     8 */
	bool                       snapshot;             /*  4248     1 */

	/* XXX 7 bytes hole, try to pack */

	struct trace_seq           seq;                  /*  4256  4136 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 131 boundary (8384 bytes) was 8 bytes ago --- */
	struct trace_entry *       ent;                  /*  8392     8 */
	long unsigned int          lost_events;          /*  8400     8 */
	int                        leftover;             /*  8408     4 */
	int                        ent_size;             /*  8412     4 */
	int                        cpu;                  /*  8416     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        ts;                   /*  8424     8 */
	loff_t                     pos;                  /*  8432     8 */
	long int                   idx;                  /*  8440     8 */

	/* size: 8448, cachelines: 132, members: 22 */
	/* sum members: 8429, holes: 4, sum holes: 19 */
	/* paddings: 2, sum paddings: 8 */
};
struct trace_event {
	struct hlist_node          node;                 /*     0    16 */
	struct list_head           list;                 /*    16    16 */
	int                        type;                 /*    32     4 */

	/* XXX 4 bytes hole, try to pack */

	struct trace_event_functions * funcs;            /*    40     8 */

	/* size: 48, cachelines: 1, members: 4 */
	/* sum members: 44, holes: 1, sum holes: 4 */
	/* last cacheline: 48 bytes */
};
struct trace_event_functions {
	trace_print_func           trace;                /*     0     8 */
	trace_print_func           raw;                  /*     8     8 */
	trace_print_func           hex;                  /*    16     8 */
	trace_print_func           binary;               /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct trace_event_fields {
	const char  *              type;                 /*     0     8 */
	union {
		struct {
			const char  * name;              /*     8     8 */
			const int  size;                 /*    16     4 */
			const int  align;                /*    20     4 */
			const int  is_signed;            /*    24     4 */
			const int  filter_type;          /*    28     4 */
		};                                       /*     8    24 */
		int                (*define_fields)(struct trace_event_call *); /*     8     8 */
	};                                               /*     8    24 */

	/* size: 32, cachelines: 1, members: 2 */
	/* last cacheline: 32 bytes */
};
struct trace_event_class {
	const char  *              system;               /*     0     8 */
	void *                     probe;                /*     8     8 */
	void *                     perf_probe;           /*    16     8 */
	int                        (*reg)(struct trace_event_call *, enum trace_reg, void *); /*    24     8 */
	struct trace_event_fields * fields_array;        /*    32     8 */
	struct list_head *         (*get_fields)(struct trace_event_call *); /*    40     8 */
	struct list_head           fields;               /*    48    16 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*raw_init)(struct trace_event_call *); /*    64     8 */

	/* size: 72, cachelines: 2, members: 8 */
	/* last cacheline: 8 bytes */
};
struct trace_event_buffer {
	struct trace_buffer *      buffer;               /*     0     8 */
	struct ring_buffer_event * event;                /*     8     8 */
	struct trace_event_file *  trace_file;           /*    16     8 */
	void *                     entry;                /*    24     8 */
	unsigned int               trace_ctx;            /*    32     4 */

	/* XXX 4 bytes hole, try to pack */

	struct pt_regs *           regs;                 /*    40     8 */

	/* size: 48, cachelines: 1, members: 6 */
	/* sum members: 44, holes: 1, sum holes: 4 */
	/* last cacheline: 48 bytes */
};
struct trace_event_file {
	struct list_head           list;                 /*     0    16 */
	struct trace_event_call *  event_call;           /*    16     8 */
	struct event_filter *      filter;               /*    24     8 */
	struct dentry *            dir;                  /*    32     8 */
	struct trace_array *       tr;                   /*    40     8 */
	struct trace_subsystem_dir * system;             /*    48     8 */
	struct list_head           triggers;             /*    56    16 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	long unsigned int          flags;                /*    72     8 */
	atomic_t                   sm_ref;               /*    80     4 */
	atomic_t                   tm_ref;               /*    84     4 */

	/* size: 88, cachelines: 2, members: 10 */
	/* last cacheline: 24 bytes */
};
struct fwnode_operations {
	struct fwnode_handle *     (*get)(struct fwnode_handle *); /*     0     8 */
	void                       (*put)(struct fwnode_handle *); /*     8     8 */
	bool                       (*device_is_available)(const struct fwnode_handle  *); /*    16     8 */
	const void  *              (*device_get_match_data)(const struct fwnode_handle  *, const struct device  *); /*    24     8 */
	bool                       (*property_present)(const struct fwnode_handle  *, const char  *); /*    32     8 */
	int                        (*property_read_int_array)(const struct fwnode_handle  *, const char  *, unsigned int, void *, size_t); /*    40     8 */
	int                        (*property_read_string_array)(const struct fwnode_handle  *, const char  *, const char  * *, size_t); /*    48     8 */
	const char  *              (*get_name)(const struct fwnode_handle  *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	const char  *              (*get_name_prefix)(const struct fwnode_handle  *); /*    64     8 */
	struct fwnode_handle *     (*get_parent)(const struct fwnode_handle  *); /*    72     8 */
	struct fwnode_handle *     (*get_next_child_node)(const struct fwnode_handle  *, struct fwnode_handle *); /*    80     8 */
	struct fwnode_handle *     (*get_named_child_node)(const struct fwnode_handle  *, const char  *); /*    88     8 */
	int                        (*get_reference_args)(const struct fwnode_handle  *, const char  *, const char  *, unsigned int, unsigned int, struct fwnode_reference_args *); /*    96     8 */
	struct fwnode_handle *     (*graph_get_next_endpoint)(const struct fwnode_handle  *, struct fwnode_handle *); /*   104     8 */
	struct fwnode_handle *     (*graph_get_remote_endpoint)(const struct fwnode_handle  *); /*   112     8 */
	struct fwnode_handle *     (*graph_get_port_parent)(struct fwnode_handle *); /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	int                        (*graph_parse_endpoint)(const struct fwnode_handle  *, struct fwnode_endpoint *); /*   128     8 */
	int                        (*add_links)(const struct fwnode_handle  *, struct device *); /*   136     8 */
	u64                        kabi_reserved1;       /*   144     8 */
	u64                        kabi_reserved2;       /*   152     8 */
	u64                        kabi_reserved3;       /*   160     8 */
	u64                        kabi_reserved4;       /*   168     8 */

	/* size: 176, cachelines: 3, members: 22 */
	/* last cacheline: 48 bytes */
};
struct fwnode_endpoint {
	unsigned int               port;                 /*     0     4 */
	unsigned int               id;                   /*     4     4 */
	const struct fwnode_handle  * local_fwnode;      /*     8     8 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct fwnode_reference_args {
	struct fwnode_handle *     fwnode;               /*     0     8 */
	unsigned int               nargs;                /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        args[8];              /*    16    64 */

	/* size: 80, cachelines: 2, members: 3 */
	/* sum members: 76, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct property {
	char *                     name;                 /*     0     8 */
	int                        length;               /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     value;                /*    16     8 */
	struct property *          next;                 /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct irq_fwspec {
	struct fwnode_handle *     fwnode;               /*     0     8 */
	int                        param_count;          /*     8     4 */
	u32                        param[16];            /*    12    64 */

	/* size: 80, cachelines: 2, members: 3 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct irq_domain_ops {
	int                        (*match)(struct irq_domain *, struct device_node *, enum irq_domain_bus_token); /*     0     8 */
	int                        (*select)(struct irq_domain *, struct irq_fwspec *, enum irq_domain_bus_token); /*     8     8 */
	int                        (*map)(struct irq_domain *, unsigned int, irq_hw_number_t); /*    16     8 */
	void                       (*unmap)(struct irq_domain *, unsigned int); /*    24     8 */
	int                        (*xlate)(struct irq_domain *, struct device_node *, const u32  *, unsigned int, long unsigned int *, unsigned int *); /*    32     8 */
	int                        (*alloc)(struct irq_domain *, unsigned int, unsigned int, void *); /*    40     8 */
	void                       (*free)(struct irq_domain *, unsigned int, unsigned int); /*    48     8 */
	int                        (*activate)(struct irq_domain *, struct irq_data *, bool); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	void                       (*deactivate)(struct irq_domain *, struct irq_data *); /*    64     8 */
	int                        (*translate)(struct irq_domain *, struct irq_fwspec *, long unsigned int *, unsigned int *); /*    72     8 */

	/* size: 80, cachelines: 2, members: 10 */
	/* last cacheline: 16 bytes */
};
struct xbc_node {
	u16                        next;                 /*     0     2 */
	u16                        child;                /*     2     2 */
	u16                        parent;               /*     4     2 */
	u16                        data;                 /*     6     2 */

	/* size: 8, cachelines: 1, members: 4 */
	/* last cacheline: 8 bytes */
};
struct fprop_global {
	struct percpu_counter      events;               /*     0    40 */
	unsigned int               period;               /*    40     4 */
	seqcount_t                 sequence;             /*    44     4 */

	/* size: 48, cachelines: 1, members: 3 */
	/* last cacheline: 48 bytes */
};
struct fprop_local_percpu {
	struct percpu_counter      events;               /*     0    40 */
	unsigned int               period;               /*    40     4 */
	raw_spinlock_t             lock;                 /*    44     4 */

	/* size: 48, cachelines: 1, members: 3 */
	/* last cacheline: 48 bytes */
};
struct wb_completion {
	atomic_t                   cnt;                  /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	wait_queue_head_t *        waitq;                /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct bvec_iter {
	sector_t                   bi_sector;            /*     0     8 */
	unsigned int               bi_size;              /*     8     4 */
	unsigned int               bi_idx;               /*    12     4 */
	unsigned int               bi_bvec_done;         /*    16     4 */

	/* size: 24, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct hd_struct {
	sector_t                   start_sect;           /*     0     8 */
	sector_t                   nr_sects;             /*     8     8 */
	long unsigned int          stamp;                /*    16     8 */
	struct disk_stats *        dkstats;              /*    24     8 */
	struct percpu_ref          ref;                  /*    32    16 */
	struct device              __dev __attribute__((__aligned__(8))); /*    48  1000 */
	/* --- cacheline 16 boundary (1024 bytes) was 24 bytes ago --- */
	struct kobject *           holder_dir;           /*  1048     8 */
	bool                       read_only;            /*  1056     1 */

	/* XXX 3 bytes hole, try to pack */

	int                        partno;               /*  1060     4 */
	struct partition_meta_info * info;               /*  1064     8 */
	struct rcu_work            rcu_work __attribute__((__aligned__(8))); /*  1072    88 */
	/* --- cacheline 18 boundary (1152 bytes) was 8 bytes ago --- */
	union {
		u64                stat_time;            /*  1160     8 */
		struct {
			u64        kabi_reserved1;       /*  1160     8 */
		} kabi_hidden_79;                        /*  1160     8 */
		union {
		};                                       /*  1160     0 */
	};                                               /*  1160     8 */
	u64                        kabi_reserved2;       /*  1168     8 */
	u64                        kabi_reserved3;       /*  1176     8 */
	u64                        kabi_reserved4;       /*  1184     8 */

	/* size: 1192, cachelines: 19, members: 15 */
	/* sum members: 1189, holes: 1, sum holes: 3 */
	/* forced alignments: 2 */
	/* last cacheline: 40 bytes */
} __attribute__((__aligned__(8)));
struct gendisk {
	int                        major;                /*     0     4 */
	int                        first_minor;          /*     4     4 */
	int                        minors;               /*     8     4 */
	char                       disk_name[32];        /*    12    32 */
	short unsigned int         events;               /*    44     2 */
	short unsigned int         event_flags;          /*    46     2 */
	struct disk_part_tbl *     part_tbl;             /*    48     8 */
	struct hd_struct           part0 __attribute__((__aligned__(8))); /*    56  1192 */
	/* --- cacheline 19 boundary (1216 bytes) was 32 bytes ago --- */
	long unsigned int          user_ro_bitmap[4];    /*  1248    32 */
	/* --- cacheline 20 boundary (1280 bytes) --- */
	const struct block_device_operations  * fops;    /*  1280     8 */
	struct request_queue *     queue;                /*  1288     8 */
	void *                     private_data;         /*  1296     8 */
	int                        flags;                /*  1304     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          state;                /*  1312     8 */
	struct rw_semaphore        lookup_sem;           /*  1320    40 */
	/* --- cacheline 21 boundary (1344 bytes) was 16 bytes ago --- */
	struct kobject *           slave_dir;            /*  1360     8 */
	struct timer_rand_state *  random;               /*  1368     8 */
	atomic64_t                 sync_io_sectors;      /*  1376     8 */
	struct disk_events *       ev;                   /*  1384     8 */
	struct kobject             integrity_kobj;       /*  1392    96 */
	/* --- cacheline 23 boundary (1472 bytes) was 16 bytes ago --- */
	struct cdrom_device_info * cdi;                  /*  1488     8 */
	int                        node_id;              /*  1496     4 */

	/* XXX 4 bytes hole, try to pack */

	struct badblocks *         bb;                   /*  1504     8 */
	struct lockdep_map         lockdep_map;          /*  1512     0 */
	u64                        kabi_reserved1;       /*  1512     8 */
	u64                        kabi_reserved2;       /*  1520     8 */
	u64                        kabi_reserved3;       /*  1528     8 */
	/* --- cacheline 24 boundary (1536 bytes) --- */
	u64                        kabi_reserved4;       /*  1536     8 */

	/* size: 1544, cachelines: 25, members: 28 */
	/* sum members: 1536, holes: 2, sum holes: 8 */
	/* forced alignments: 1 */
	/* last cacheline: 8 bytes */
} __attribute__((__aligned__(8)));
struct bio_issue {
	u64                        value;                /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct bio_integrity_payload {
	struct bio *               bip_bio;              /*     0     8 */
	struct bvec_iter           bip_iter;             /*     8    24 */

	/* XXX last struct has 4 bytes of padding */

	short unsigned int         bip_slab;             /*    32     2 */
	short unsigned int         bip_vcnt;             /*    34     2 */
	short unsigned int         bip_max_vcnt;         /*    36     2 */
	short unsigned int         bip_flags;            /*    38     2 */
	struct bvec_iter           bio_iter;             /*    40    24 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 1 boundary (64 bytes) --- */
	struct work_struct         bip_work;             /*    64    64 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct bio_vec *           bip_vec;              /*   128     8 */
	u64                        kabi_reserved1;       /*   136     8 */
	u64                        kabi_reserved2;       /*   144     8 */
	struct bio_vec             bip_inline_vecs[];    /*   152     0 */

	/* size: 152, cachelines: 3, members: 12 */
	/* paddings: 2, sum paddings: 8 */
	/* last cacheline: 24 bytes */
};
struct blkcg_gq {
	struct request_queue *     q;                    /*     0     8 */
	struct list_head           q_node;               /*     8    16 */
	struct hlist_node          blkcg_node;           /*    24    16 */
	struct blkcg *             blkcg;                /*    40     8 */
	struct blkcg_gq *          parent;               /*    48     8 */
	struct percpu_ref          refcnt;               /*    56    16 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	bool                       online;               /*    72     1 */

	/* XXX 7 bytes hole, try to pack */

	struct blkg_iostat_set *   iostat_cpu;           /*    80     8 */
	struct blkg_iostat_set     iostat;               /*    88    96 */
	/* --- cacheline 2 boundary (128 bytes) was 56 bytes ago --- */
	struct blkg_policy_data *  pd[5];                /*   184    40 */
	/* --- cacheline 3 boundary (192 bytes) was 32 bytes ago --- */
	spinlock_t                 async_bio_lock;       /*   224     4 */

	/* XXX 4 bytes hole, try to pack */

	struct bio_list            async_bios;           /*   232    16 */
	struct work_struct         async_bio_work;       /*   248    64 */
	/* --- cacheline 4 boundary (256 bytes) was 56 bytes ago --- */
	atomic_t                   use_delay;            /*   312     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 5 boundary (320 bytes) --- */
	atomic64_t                 delay_nsec;           /*   320     8 */
	atomic64_t                 delay_start;          /*   328     8 */
	u64                        last_delay;           /*   336     8 */
	int                        last_use;             /*   344     4 */

	/* XXX 4 bytes hole, try to pack */

	struct callback_head       callback_head __attribute__((__aligned__(8))); /*   352    16 */

	/* size: 368, cachelines: 6, members: 19 */
	/* sum members: 349, holes: 4, sum holes: 19 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 4 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct bio_set {
	struct kmem_cache *        bio_slab;             /*     0     8 */
	unsigned int               front_pad;            /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	mempool_t                  bio_pool;             /*    16    72 */
	/* --- cacheline 1 boundary (64 bytes) was 24 bytes ago --- */
	mempool_t                  bvec_pool;            /*    88    72 */
	/* --- cacheline 2 boundary (128 bytes) was 32 bytes ago --- */
	mempool_t                  bio_integrity_pool;   /*   160    72 */
	/* --- cacheline 3 boundary (192 bytes) was 40 bytes ago --- */
	mempool_t                  bvec_integrity_pool;  /*   232    72 */
	/* --- cacheline 4 boundary (256 bytes) was 48 bytes ago --- */
	spinlock_t                 rescue_lock;          /*   304     4 */

	/* XXX 4 bytes hole, try to pack */

	struct bio_list            rescue_list;          /*   312    16 */
	/* --- cacheline 5 boundary (320 bytes) was 8 bytes ago --- */
	struct work_struct         rescue_work;          /*   328    64 */
	/* --- cacheline 6 boundary (384 bytes) was 8 bytes ago --- */
	struct workqueue_struct *  rescue_workqueue;     /*   392     8 */
	u64                        kabi_reserved1;       /*   400     8 */
	u64                        kabi_reserved2;       /*   408     8 */
	u64                        kabi_reserved3;       /*   416     8 */
	u64                        kabi_reserved4;       /*   424     8 */

	/* size: 432, cachelines: 7, members: 14 */
	/* sum members: 424, holes: 2, sum holes: 8 */
	/* last cacheline: 48 bytes */
};
struct blk_rq_stat {
	u64                        mean;                 /*     0     8 */
	u64                        min;                  /*     8     8 */
	u64                        max;                  /*    16     8 */
	u32                        nr_samples;           /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        batch;                /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* sum members: 36, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct partition_meta_info {
	char                       uuid[37];             /*     0    37 */
	u8                         volname[64];          /*    37    64 */

	/* size: 101, cachelines: 2, members: 2 */
	/* last cacheline: 37 bytes */
};
struct disk_part_tbl {
	struct callback_head       callback_head __attribute__((__aligned__(8))); /*     0    16 */
	int                        len;                  /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	struct hd_struct *         last_lookup;          /*    24     8 */
	struct hd_struct *         part[];               /*    32     0 */

	/* size: 32, cachelines: 1, members: 4 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct blk_integrity {
	const struct blk_integrity_profile  * profile;   /*     0     8 */
	unsigned char              flags;                /*     8     1 */
	unsigned char              tuple_size;           /*     9     1 */
	unsigned char              interval_exp;         /*    10     1 */
	unsigned char              tag_size;             /*    11     1 */

	/* XXX 4 bytes hole, try to pack */

	u64                        kabi_reserved1;       /*    16     8 */
	u64                        kabi_reserved2;       /*    24     8 */

	/* size: 32, cachelines: 1, members: 7 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct blk_integrity_profile {
	integrity_processing_fn *  generate_fn;          /*     0     8 */
	integrity_processing_fn *  verify_fn;            /*     8     8 */
	integrity_prepare_fn *     prepare_fn;           /*    16     8 */
	integrity_complete_fn *    complete_fn;          /*    24     8 */
	const char  *              name;                 /*    32     8 */
	u64                        kabi_reserved1;       /*    40     8 */
	u64                        kabi_reserved2;       /*    48     8 */

	/* size: 56, cachelines: 1, members: 7 */
	/* last cacheline: 56 bytes */
};
struct block_device_operations {
	blk_qc_t                   (*submit_bio)(struct bio *); /*     0     8 */
	int                        (*open)(struct block_device *, fmode_t); /*     8     8 */
	void                       (*release)(struct gendisk *, fmode_t); /*    16     8 */
	int                        (*rw_page)(struct block_device *, sector_t, struct page *, unsigned int); /*    24     8 */
	int                        (*ioctl)(struct block_device *, fmode_t, unsigned int, long unsigned int); /*    32     8 */
	int                        (*compat_ioctl)(struct block_device *, fmode_t, unsigned int, long unsigned int); /*    40     8 */
	unsigned int               (*check_events)(struct gendisk *, unsigned int); /*    48     8 */
	void                       (*unlock_native_capacity)(struct gendisk *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*revalidate_disk)(struct gendisk *); /*    64     8 */
	int                        (*getgeo)(struct block_device *, struct hd_geometry *); /*    72     8 */
	void                       (*swap_slot_free_notify)(struct block_device *, long unsigned int); /*    80     8 */
	int                        (*report_zones)(struct gendisk *, sector_t, unsigned int, report_zones_cb, void *); /*    88     8 */
	char *                     (*devnode)(struct gendisk *, umode_t *); /*    96     8 */
	struct module *            owner;                /*   104     8 */
	const struct pr_ops  *     pr_ops;               /*   112     8 */
	u64                        kabi_reserved1;       /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	u64                        kabi_reserved2;       /*   128     8 */
	u64                        kabi_reserved3;       /*   136     8 */
	u64                        kabi_reserved4;       /*   144     8 */

	/* size: 152, cachelines: 3, members: 19 */
	/* last cacheline: 24 bytes */
};
struct mempool_s {
	spinlock_t                 lock;                 /*     0     4 */
	int                        min_nr;               /*     4     4 */
	int                        curr_nr;              /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	void * *                   elements;             /*    16     8 */
	void *                     pool_data;            /*    24     8 */
	mempool_alloc_t *          alloc;                /*    32     8 */
	mempool_free_t *           free;                 /*    40     8 */
	wait_queue_head_t          wait;                 /*    48    24 */

	/* size: 72, cachelines: 2, members: 8 */
	/* sum members: 68, holes: 1, sum holes: 4 */
	/* last cacheline: 8 bytes */
};
struct sg_io_v4 {
	__s32                      guard;                /*     0     4 */
	__u32                      protocol;             /*     4     4 */
	__u32                      subprotocol;          /*     8     4 */
	__u32                      request_len;          /*    12     4 */
	__u64                      request;              /*    16     8 */
	__u64                      request_tag;          /*    24     8 */
	__u32                      request_attr;         /*    32     4 */
	__u32                      request_priority;     /*    36     4 */
	__u32                      request_extra;        /*    40     4 */
	__u32                      max_response_len;     /*    44     4 */
	__u64                      response;             /*    48     8 */
	__u32                      dout_iovec_count;     /*    56     4 */
	__u32                      dout_xfer_len;        /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	__u32                      din_iovec_count;      /*    64     4 */
	__u32                      din_xfer_len;         /*    68     4 */
	__u64                      dout_xferp;           /*    72     8 */
	__u64                      din_xferp;            /*    80     8 */
	__u32                      timeout;              /*    88     4 */
	__u32                      flags;                /*    92     4 */
	__u64                      usr_ptr;              /*    96     8 */
	__u32                      spare_in;             /*   104     4 */
	__u32                      driver_status;        /*   108     4 */
	__u32                      transport_status;     /*   112     4 */
	__u32                      device_status;        /*   116     4 */
	__u32                      retry_delay;          /*   120     4 */
	__u32                      info;                 /*   124     4 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	__u32                      duration;             /*   128     4 */
	__u32                      response_len;         /*   132     4 */
	__s32                      din_resid;            /*   136     4 */
	__s32                      dout_resid;           /*   140     4 */
	__u64                      generated_tag;        /*   144     8 */
	__u32                      spare_out;            /*   152     4 */
	__u32                      padding;              /*   156     4 */

	/* size: 160, cachelines: 3, members: 33 */
	/* last cacheline: 32 bytes */
};
struct bsg_ops {
	int                        (*check_proto)(struct sg_io_v4 *); /*     0     8 */
	int                        (*fill_hdr)(struct request *, struct sg_io_v4 *, fmode_t); /*     8     8 */
	int                        (*complete_rq)(struct request *, struct sg_io_v4 *); /*    16     8 */
	void                       (*free_rq)(struct request *); /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct request {
	struct request_queue *     q;                    /*     0     8 */
	struct blk_mq_ctx *        mq_ctx;               /*     8     8 */
	struct blk_mq_hw_ctx *     mq_hctx;              /*    16     8 */
	unsigned int               cmd_flags;            /*    24     4 */
	req_flags_t                rq_flags;             /*    28     4 */
	int                        tag;                  /*    32     4 */
	int                        internal_tag;         /*    36     4 */
	unsigned int               __data_len;           /*    40     4 */

	/* XXX 4 bytes hole, try to pack */

	sector_t                   __sector;             /*    48     8 */
	struct bio *               bio;                  /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct bio *               biotail;              /*    64     8 */
	struct list_head           queuelist;            /*    72    16 */
	union {
		struct hlist_node  hash;                 /*    88    16 */
		struct list_head   ipi_list;             /*    88    16 */
	};                                               /*    88    16 */
	union {
		struct rb_node     rb_node __attribute__((__aligned__(8))); /*   104    24 */
		struct bio_vec     special_vec;          /*   104    16 */
		void *             completion_data;      /*   104     8 */
		int                error_count;          /*   104     4 */
	} __attribute__((__aligned__(8)));               /*   104    24 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	union {
		struct {
			struct io_cq * icq;              /*   128     8 */
			void *     priv[2];              /*   136    16 */
		} elv;                                   /*   128    24 */
		struct {
			unsigned int seq;                /*   128     4 */

			/* XXX 4 bytes hole, try to pack */

			struct list_head list;           /*   136    16 */
			rq_end_io_fn * saved_end_io;     /*   152     8 */
		} flush;                                 /*   128    32 */
	};                                               /*   128    32 */
	struct gendisk *           rq_disk;              /*   160     8 */
	struct hd_struct *         part;                 /*   168     8 */
	u64                        start_time_ns;        /*   176     8 */
	u64                        io_start_time_ns;     /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	short unsigned int         wbt_flags;            /*   192     2 */
	short unsigned int         stats_sectors;        /*   194     2 */
	short unsigned int         nr_phys_segments;     /*   196     2 */
	short unsigned int         nr_integrity_segments; /*   198     2 */
	short unsigned int         write_hint;           /*   200     2 */
	short unsigned int         ioprio;               /*   202     2 */
	enum mq_rq_state           state;                /*   204     4 */
	refcount_t                 ref;                  /*   208     4 */
	unsigned int               timeout;              /*   212     4 */
	long unsigned int          deadline;             /*   216     8 */
	union {
		struct __call_single_data csd;           /*   224    32 */
		u64                fifo_time;            /*   224     8 */
	};                                               /*   224    32 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	rq_end_io_fn *             end_io;               /*   256     8 */
	void *                     end_io_data;          /*   264     8 */

	/* size: 272, cachelines: 5, members: 32 */
	/* sum members: 268, holes: 1, sum holes: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct bsg_class_device {
	struct device *            class_dev;            /*     0     8 */
	int                        minor;                /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct request_queue *     queue;                /*    16     8 */
	const struct bsg_ops  *    ops;                  /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct blk_zone {
	__u64                      start;                /*     0     8 */
	__u64                      len;                  /*     8     8 */
	__u64                      wp;                   /*    16     8 */
	__u8                       type;                 /*    24     1 */
	__u8                       cond;                 /*    25     1 */
	__u8                       non_seq;              /*    26     1 */
	__u8                       reset;                /*    27     1 */
	__u8                       resv[4];              /*    28     4 */
	__u64                      capacity;             /*    32     8 */
	__u8                       reserved[24];         /*    40    24 */

	/* size: 64, cachelines: 1, members: 10 */
};
struct elevator_mq_ops {
	int                        (*init_sched)(struct request_queue *, struct elevator_type *); /*     0     8 */
	void                       (*exit_sched)(struct elevator_queue *); /*     8     8 */
	int                        (*init_hctx)(struct blk_mq_hw_ctx *, unsigned int); /*    16     8 */
	void                       (*exit_hctx)(struct blk_mq_hw_ctx *, unsigned int); /*    24     8 */
	void                       (*depth_updated)(struct blk_mq_hw_ctx *); /*    32     8 */
	bool                       (*allow_merge)(struct request_queue *, struct request *, struct bio *); /*    40     8 */
	bool                       (*bio_merge)(struct request_queue *, struct bio *, unsigned int); /*    48     8 */
	int                        (*request_merge)(struct request_queue *, struct request * *, struct bio *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	void                       (*request_merged)(struct request_queue *, struct request *, enum elv_merge); /*    64     8 */
	void                       (*requests_merged)(struct request_queue *, struct request *, struct request *); /*    72     8 */
	void                       (*limit_depth)(unsigned int, struct blk_mq_alloc_data *); /*    80     8 */
	void                       (*prepare_request)(struct request *); /*    88     8 */
	void                       (*finish_request)(struct request *); /*    96     8 */
	void                       (*insert_requests)(struct blk_mq_hw_ctx *, struct list_head *, bool); /*   104     8 */
	struct request *           (*dispatch_request)(struct blk_mq_hw_ctx *); /*   112     8 */
	bool                       (*has_work)(struct blk_mq_hw_ctx *); /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	void                       (*completed_request)(struct request *, u64); /*   128     8 */
	void                       (*requeue_request)(struct request *); /*   136     8 */
	struct request *           (*former_request)(struct request_queue *, struct request *); /*   144     8 */
	struct request *           (*next_request)(struct request_queue *, struct request *); /*   152     8 */
	void                       (*init_icq)(struct io_cq *); /*   160     8 */
	void                       (*exit_icq)(struct io_cq *); /*   168     8 */
	u64                        kabi_reserved1;       /*   176     8 */
	u64                        kabi_reserved2;       /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	u64                        kabi_reserved3;       /*   192     8 */
	u64                        kabi_reserved4;       /*   200     8 */
	u64                        kabi_reserved5;       /*   208     8 */
	u64                        kabi_reserved6;       /*   216     8 */
	u64                        kabi_reserved7;       /*   224     8 */
	u64                        kabi_reserved8;       /*   232     8 */

	/* size: 240, cachelines: 4, members: 30 */
	/* last cacheline: 48 bytes */
};
struct elevator_type {
	struct kmem_cache *        icq_cache;            /*     0     8 */
	struct elevator_mq_ops     ops;                  /*     8   240 */
	/* --- cacheline 3 boundary (192 bytes) was 56 bytes ago --- */
	size_t                     icq_size;             /*   248     8 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	size_t                     icq_align;            /*   256     8 */
	struct elv_fs_entry *      elevator_attrs;       /*   264     8 */
	const char  *              elevator_name;        /*   272     8 */
	const char  *              elevator_alias;       /*   280     8 */
	const unsigned int         elevator_features;    /*   288     4 */

	/* XXX 4 bytes hole, try to pack */

	struct module *            elevator_owner;       /*   296     8 */
	const struct blk_mq_debugfs_attr  * queue_debugfs_attrs; /*   304     8 */
	const struct blk_mq_debugfs_attr  * hctx_debugfs_attrs; /*   312     8 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	char                       icq_cache_name[22];   /*   320    22 */

	/* XXX 2 bytes hole, try to pack */

	struct list_head           list;                 /*   344    16 */
	u64                        kabi_reserved1;       /*   360     8 */
	u64                        kabi_reserved2;       /*   368     8 */
	u64                        kabi_reserved3;       /*   376     8 */
	/* --- cacheline 6 boundary (384 bytes) --- */
	u64                        kabi_reserved4;       /*   384     8 */

	/* size: 392, cachelines: 7, members: 17 */
	/* sum members: 386, holes: 2, sum holes: 6 */
	/* last cacheline: 8 bytes */
};
struct elevator_queue {
	struct elevator_type *     type;                 /*     0     8 */
	void *                     elevator_data;        /*     8     8 */
	struct kobject             kobj;                 /*    16    96 */
	/* --- cacheline 1 boundary (64 bytes) was 48 bytes ago --- */
	struct mutex               sysfs_lock;           /*   112    32 */
	/* --- cacheline 2 boundary (128 bytes) was 16 bytes ago --- */
	unsigned int               registered:1;         /*   144: 0  4 */

	/* XXX 31 bits hole, try to pack */
	/* XXX 4 bytes hole, try to pack */

	struct hlist_head          hash[64];             /*   152   512 */

	/* size: 664, cachelines: 11, members: 6 */
	/* sum members: 656, holes: 1, sum holes: 4 */
	/* sum bitfield members: 1 bits, bit holes: 1, sum bit holes: 31 bits */
	/* last cacheline: 24 bytes */
};
struct elv_fs_entry {
	struct attribute           attr;                 /*     0    16 */

	/* XXX last struct has 6 bytes of padding */

	ssize_t                    (*show)(struct elevator_queue *, char *); /*    16     8 */
	ssize_t                    (*store)(struct elevator_queue *, const char  *, size_t); /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* paddings: 1, sum paddings: 6 */
	/* last cacheline: 32 bytes */
};
struct queue_limits {
	long unsigned int          bounce_pfn;           /*     0     8 */
	long unsigned int          seg_boundary_mask;    /*     8     8 */
	long unsigned int          virt_boundary_mask;   /*    16     8 */
	unsigned int               max_hw_sectors;       /*    24     4 */
	unsigned int               max_dev_sectors;      /*    28     4 */
	unsigned int               chunk_sectors;        /*    32     4 */
	unsigned int               max_sectors;          /*    36     4 */
	unsigned int               max_segment_size;     /*    40     4 */
	unsigned int               physical_block_size;  /*    44     4 */
	unsigned int               logical_block_size;   /*    48     4 */
	unsigned int               alignment_offset;     /*    52     4 */
	unsigned int               io_min;               /*    56     4 */
	unsigned int               io_opt;               /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	unsigned int               max_discard_sectors;  /*    64     4 */
	unsigned int               max_hw_discard_sectors; /*    68     4 */
	unsigned int               max_write_same_sectors; /*    72     4 */
	unsigned int               max_write_zeroes_sectors; /*    76     4 */
	unsigned int               max_zone_append_sectors; /*    80     4 */
	unsigned int               discard_granularity;  /*    84     4 */
	unsigned int               discard_alignment;    /*    88     4 */
	short unsigned int         max_segments;         /*    92     2 */
	short unsigned int         max_integrity_segments; /*    94     2 */
	short unsigned int         max_discard_segments; /*    96     2 */
	unsigned char              misaligned;           /*    98     1 */
	unsigned char              discard_misaligned;   /*    99     1 */
	unsigned char              raid_partial_stripes_expensive; /*   100     1 */

	/* XXX 3 bytes hole, try to pack */

	enum blk_zoned_model       zoned;                /*   104     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        kabi_reserved1;       /*   112     8 */

	/* size: 120, cachelines: 2, members: 28 */
	/* sum members: 113, holes: 2, sum holes: 7 */
	/* last cacheline: 56 bytes */
};
struct blk_integrity_iter {
	void *                     prot_buf;             /*     0     8 */
	void *                     data_buf;             /*     8     8 */
	sector_t                   seed;                 /*    16     8 */
	unsigned int               data_size;            /*    24     4 */
	short unsigned int         interval;             /*    28     2 */

	/* XXX 2 bytes hole, try to pack */

	const char  *              disk_name;            /*    32     8 */

	/* size: 40, cachelines: 1, members: 6 */
	/* sum members: 38, holes: 1, sum holes: 2 */
	/* last cacheline: 40 bytes */
};
struct blkcg {
	struct cgroup_subsys_state css __attribute__((__aligned__(8))); /*     0   296 */
	/* --- cacheline 4 boundary (256 bytes) was 40 bytes ago --- */
	spinlock_t                 lock;                 /*   296     4 */
	refcount_t                 online_pin;           /*   300     4 */
	struct xarray              blkg_tree;            /*   304    16 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	struct blkcg_gq *          blkg_hint;            /*   320     8 */
	struct hlist_head          blkg_list;            /*   328     8 */
	struct blkcg_policy_data * cpd[5];               /*   336    40 */
	struct list_head           all_blkcgs_node;      /*   376    16 */
	/* --- cacheline 6 boundary (384 bytes) was 8 bytes ago --- */
	struct list_head           cgwb_list;            /*   392    16 */
	struct list_head           memcg_list;           /*   408    16 */
	u64                        kabi_reserved3;       /*   424     8 */
	u64                        kabi_reserved4;       /*   432     8 */

	/* size: 440, cachelines: 7, members: 12 */
	/* forced alignments: 1 */
	/* last cacheline: 56 bytes */
} __attribute__((__aligned__(8)));
struct blkcg_policy_data {
	struct blkcg *             blkcg;                /*     0     8 */
	int                        plid;                 /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct blkg_iostat {
	u64                        bytes[3];             /*     0    24 */
	u64                        ios[3];               /*    24    24 */

	/* size: 48, cachelines: 1, members: 2 */
	/* last cacheline: 48 bytes */
};
struct blkg_iostat_set {
	struct u64_stats_sync      sync;                 /*     0     0 */
	struct blkg_iostat         cur;                  /*     0    48 */
	struct blkg_iostat         last;                 /*    48    48 */

	/* size: 96, cachelines: 2, members: 3 */
	/* last cacheline: 32 bytes */
};
struct blkg_policy_data {
	struct blkcg_gq *          blkg;                 /*     0     8 */
	int                        plid;                 /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct wb_domain {
	spinlock_t                 lock;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct fprop_global        completions;          /*     8    48 */
	struct timer_list          period_timer;         /*    56    72 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	long unsigned int          period_time;          /*   128     8 */
	long unsigned int          dirty_limit_tstamp;   /*   136     8 */
	long unsigned int          dirty_limit;          /*   144     8 */

	/* size: 152, cachelines: 3, members: 6 */
	/* sum members: 148, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct efi_memory_map {
	phys_addr_t                phys_map;             /*     0     8 */
	void *                     map;                  /*     8     8 */
	void *                     map_end;              /*    16     8 */
	int                        nr_map;               /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          desc_version;         /*    32     8 */
	long unsigned int          desc_size;            /*    40     8 */
	long unsigned int          flags;                /*    48     8 */

	/* size: 56, cachelines: 1, members: 7 */
	/* sum members: 52, holes: 1, sum holes: 4 */
	/* last cacheline: 56 bytes */
};
struct efi {
	const efi_runtime_services_t  * runtime;         /*     0     8 */
	unsigned int               runtime_version;      /*     8     4 */
	unsigned int               runtime_supported_mask; /*    12     4 */
	long unsigned int          acpi;                 /*    16     8 */
	long unsigned int          acpi20;               /*    24     8 */
	long unsigned int          smbios;               /*    32     8 */
	long unsigned int          smbios3;              /*    40     8 */
	long unsigned int          esrt;                 /*    48     8 */
	long unsigned int          tpm_log;              /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	long unsigned int          tpm_final_log;        /*    64     8 */
	long unsigned int          mokvar_table;         /*    72     8 */
	efi_get_time_t *           get_time;             /*    80     8 */
	efi_set_time_t *           set_time;             /*    88     8 */
	efi_get_wakeup_time_t *    get_wakeup_time;      /*    96     8 */
	efi_set_wakeup_time_t *    set_wakeup_time;      /*   104     8 */
	efi_get_variable_t *       get_variable;         /*   112     8 */
	efi_get_next_variable_t *  get_next_variable;    /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	efi_set_variable_t *       set_variable;         /*   128     8 */
	efi_set_variable_t *       set_variable_nonblocking; /*   136     8 */
	efi_query_variable_info_t * query_variable_info; /*   144     8 */
	efi_query_variable_info_t * query_variable_info_nonblocking; /*   152     8 */
	efi_update_capsule_t *     update_capsule;       /*   160     8 */
	efi_query_capsule_caps_t * query_capsule_caps;   /*   168     8 */
	efi_get_next_high_mono_count_t * get_next_high_mono_count; /*   176     8 */
	efi_reset_system_t *       reset_system;         /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	struct efi_memory_map      memmap;               /*   192    56 */
	long unsigned int          flags;                /*   248     8 */

	/* size: 256, cachelines: 4, members: 27 */
};
struct new_utsname {
	char                       sysname[65];          /*     0    65 */
	/* --- cacheline 1 boundary (64 bytes) was 1 byte ago --- */
	char                       nodename[65];         /*    65    65 */
	/* --- cacheline 2 boundary (128 bytes) was 2 bytes ago --- */
	char                       release[65];          /*   130    65 */
	/* --- cacheline 3 boundary (192 bytes) was 3 bytes ago --- */
	char                       version[65];          /*   195    65 */
	/* --- cacheline 4 boundary (256 bytes) was 4 bytes ago --- */
	char                       machine[65];          /*   260    65 */
	/* --- cacheline 5 boundary (320 bytes) was 5 bytes ago --- */
	char                       domainname[65];       /*   325    65 */

	/* size: 390, cachelines: 7, members: 6 */
	/* last cacheline: 6 bytes */
};
struct page_counter {
	atomic_long_t              usage;                /*     0     8 */
	long unsigned int          min;                  /*     8     8 */
	long unsigned int          low;                  /*    16     8 */
	long unsigned int          high;                 /*    24     8 */
	long unsigned int          max;                  /*    32     8 */
	struct page_counter *      parent;               /*    40     8 */
	long unsigned int          emin;                 /*    48     8 */
	atomic_long_t              min_usage;            /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	atomic_long_t              children_min_usage;   /*    64     8 */
	long unsigned int          elow;                 /*    72     8 */
	atomic_long_t              low_usage;            /*    80     8 */
	atomic_long_t              children_low_usage;   /*    88     8 */
	long unsigned int          watermark;            /*    96     8 */
	long unsigned int          failcnt;              /*   104     8 */

	/* size: 112, cachelines: 2, members: 14 */
	/* last cacheline: 48 bytes */
};
struct vmpressure {
	long unsigned int          scanned;              /*     0     8 */
	long unsigned int          reclaimed;            /*     8     8 */
	long unsigned int          tree_scanned;         /*    16     8 */
	long unsigned int          tree_reclaimed;       /*    24     8 */
	spinlock_t                 sr_lock;              /*    32     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           events;               /*    40    16 */
	struct mutex               events_lock;          /*    56    32 */
	/* --- cacheline 1 boundary (64 bytes) was 24 bytes ago --- */
	struct work_struct         work;                 /*    88    64 */

	/* size: 152, cachelines: 3, members: 8 */
	/* sum members: 148, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct dhugetlb_pool {
	int                        nid;                  /*     0     4 */
	spinlock_t                 lock;                 /*     4     4 */
	spinlock_t                 reserved_lock;        /*     8     4 */
	atomic_t                   refcnt;               /*    12     4 */
	long unsigned int          normal_pages_disabled; /*    16     8 */
	struct mem_cgroup *        attach_memcg;         /*    24     8 */
	long unsigned int          total_huge_pages;     /*    32     8 */
	struct huge_pages_pool     hpages_pool[3];       /*    40   240 */
	/* --- cacheline 4 boundary (256 bytes) was 24 bytes ago --- */
	struct percpu_pages_pool   percpu_pool[];        /*   280     0 */

	/* size: 280, cachelines: 5, members: 9 */
	/* last cacheline: 24 bytes */
};
struct percpu_pages_pool {
	spinlock_t                 lock;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          free_pages;           /*     8     8 */
	long int                   used_pages;           /*    16     8 */
	struct list_head           head_page;            /*    24    16 */

	/* size: 40, cachelines: 1, members: 4 */
	/* sum members: 36, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct huge_pages_pool {
	long unsigned int          nr_huge_pages;        /*     0     8 */
	long unsigned int          free_huge_pages;      /*     8     8 */
	long unsigned int          resv_huge_pages;      /*    16     8 */
	long unsigned int          used_huge_pages;      /*    24     8 */
	long unsigned int          free_normal_pages;    /*    32     8 */
	long unsigned int          split_normal_pages;   /*    40     8 */
	struct list_head           hugepage_freelists;   /*    48    16 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct list_head           hugepage_splitlists;  /*    64    16 */

	/* size: 80, cachelines: 2, members: 8 */
	/* last cacheline: 16 bytes */
};
struct mem_cgroup_id {
	int                        id;                   /*     0     4 */
	refcount_t                 ref;                  /*     4     4 */
	u64                        kabi_reserved1;       /*     8     8 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct memcg_vmstats_percpu {
	long int                   state[40];            /*     0   320 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	long unsigned int          events[96];           /*   320   768 */
	/* --- cacheline 17 boundary (1088 bytes) --- */
	long int                   state_prev[40];       /*  1088   320 */
	/* --- cacheline 22 boundary (1408 bytes) --- */
	long unsigned int          events_prev[96];      /*  1408   768 */
	/* --- cacheline 34 boundary (2176 bytes) --- */
	long unsigned int          nr_page_events;       /*  2176     8 */
	long unsigned int          targets[2];           /*  2184    16 */
	u64                        kabi_reserved1;       /*  2200     8 */
	u64                        kabi_reserved2;       /*  2208     8 */

	/* size: 2216, cachelines: 35, members: 8 */
	/* last cacheline: 40 bytes */
};
struct memcg_vmstats {
	long int                   state[40];            /*     0   320 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	long unsigned int          events[96];           /*   320   768 */
	/* --- cacheline 17 boundary (1088 bytes) --- */
	long int                   state_pending[40];    /*  1088   320 */
	/* --- cacheline 22 boundary (1408 bytes) --- */
	long unsigned int          events_pending[96];   /*  1408   768 */

	/* size: 2176, cachelines: 34, members: 4 */
};
struct mem_cgroup_reclaim_iter {
	struct mem_cgroup *        position;             /*     0     8 */
	unsigned int               generation;           /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct shrinker_info {
	struct callback_head       rcu __attribute__((__aligned__(8))); /*     0    16 */
	atomic_long_t *            nr_deferred;          /*    16     8 */
	long unsigned int *        map;                  /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* forced alignments: 1 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct lruvec_stats_percpu {
	long int                   state[37];            /*     0   296 */
	/* --- cacheline 4 boundary (256 bytes) was 40 bytes ago --- */
	long int                   state_prev[37];       /*   296   296 */

	/* size: 592, cachelines: 10, members: 2 */
	/* last cacheline: 16 bytes */
};
struct lruvec_stats {
	long int                   state[37];            /*     0   296 */
	/* --- cacheline 4 boundary (256 bytes) was 40 bytes ago --- */
	long int                   state_pending[37];    /*   296   296 */

	/* size: 592, cachelines: 10, members: 2 */
	/* last cacheline: 16 bytes */
};
struct mem_cgroup_per_node {
	struct lruvec              lruvec;               /*     0   144 */
	/* --- cacheline 2 boundary (128 bytes) was 16 bytes ago --- */
	struct lruvec_stats_percpu * lruvec_stats_percpu; /*   144     8 */
	struct lruvec_stats        lruvec_stats;         /*   152   592 */
	/* --- cacheline 11 boundary (704 bytes) was 40 bytes ago --- */
	long unsigned int          lru_zone_size[5][5];  /*   744   200 */
	/* --- cacheline 14 boundary (896 bytes) was 48 bytes ago --- */
	struct mem_cgroup_reclaim_iter iter;             /*   944    16 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 15 boundary (960 bytes) --- */
	struct shrinker_info *     shrinker_info;        /*   960     8 */
	struct rb_node             tree_node __attribute__((__aligned__(8))); /*   968    24 */
	long unsigned int          usage_in_excess;      /*   992     8 */
	bool                       on_tree;              /*  1000     1 */

	/* XXX 7 bytes hole, try to pack */

	struct mem_cgroup *        memcg;                /*  1008     8 */
	u64                        kabi_reserved1;       /*  1016     8 */
	/* --- cacheline 16 boundary (1024 bytes) --- */
	u64                        kabi_reserved2;       /*  1024     8 */
	u64                        kabi_reserved3;       /*  1032     8 */
	u64                        kabi_reserved4;       /*  1040     8 */
	u64                        kabi_reserved5;       /*  1048     8 */

	/* size: 1056, cachelines: 17, members: 15 */
	/* sum members: 1049, holes: 1, sum holes: 7 */
	/* paddings: 1, sum paddings: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct mem_cgroup_threshold {
	struct eventfd_ctx *       eventfd;              /*     0     8 */
	long unsigned int          threshold;            /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct mem_cgroup_threshold_ary {
	int                        current_threshold;    /*     0     4 */
	unsigned int               size;                 /*     4     4 */
	struct mem_cgroup_threshold entries[];           /*     8     0 */

	/* size: 8, cachelines: 1, members: 3 */
	/* last cacheline: 8 bytes */
};
struct mem_cgroup_thresholds {
	struct mem_cgroup_threshold_ary * primary;       /*     0     8 */
	struct mem_cgroup_threshold_ary * spare;         /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct memcg_padding {
	char                       x[0];                 /*     0     0 */

	/* size: 0, cachelines: 0, members: 1 */
} __attribute__((__aligned__(64)));
struct memcg_cgwb_frn {
	u64                        bdi_id;               /*     0     8 */
	int                        memcg_id;             /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        at;                   /*    16     8 */
	struct wb_completion       done;                 /*    24    16 */

	/* size: 40, cachelines: 1, members: 4 */
	/* sum members: 36, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct obj_cgroup {
	struct percpu_ref          refcnt;               /*     0    16 */
	struct mem_cgroup *        memcg;                /*    16     8 */
	atomic_t                   nr_charged_bytes;     /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	union {
		struct list_head   list;                 /*    32    16 */
		struct callback_head rcu __attribute__((__aligned__(8))); /*    32    16 */
	} __attribute__((__aligned__(8)));               /*    32    16 */

	/* size: 48, cachelines: 1, members: 4 */
	/* sum members: 44, holes: 1, sum holes: 4 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 4 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct swap_cluster_info {
	spinlock_t                 lock;                 /*     0     4 */
	unsigned int               data:24;              /*     4: 0  4 */
	unsigned int               flags:8;              /*     4:24  4 */

	/* size: 8, cachelines: 1, members: 3 */
	/* last cacheline: 8 bytes */
};
struct percpu_cluster {
	struct swap_cluster_info   index;                /*     0     8 */
	unsigned int               next;                 /*     8     4 */

	/* size: 12, cachelines: 1, members: 2 */
	/* last cacheline: 12 bytes */
};
struct swap_cluster_list {
	struct swap_cluster_info   head;                 /*     0     8 */
	struct swap_cluster_info   tail;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct fs_parameter {
	const char  *              key;                  /*     0     8 */
	enum fs_value_type         type:8;               /*     8: 0  4 */

	/* XXX 24 bits hole, try to pack */
	/* XXX 4 bytes hole, try to pack */

	union {
		char *             string;               /*    16     8 */
		void *             blob;                 /*    16     8 */
		struct filename *  name;                 /*    16     8 */
		struct file *      file;                 /*    16     8 */
	};                                               /*    16     8 */
	size_t                     size;                 /*    24     8 */
	int                        dirfd;                /*    32     4 */

	/* size: 40, cachelines: 1, members: 5 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* sum bitfield members: 8 bits, bit holes: 1, sum bit holes: 24 bits */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct p_log {
	const char  *              prefix;               /*     0     8 */
	struct fc_log *            log;                  /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct fc_log {
	refcount_t                 usage;                /*     0     4 */
	u8                         head;                 /*     4     1 */
	u8                         tail;                 /*     5     1 */
	u8                         need_free;            /*     6     1 */

	/* XXX 1 byte hole, try to pack */

	struct module *            owner;                /*     8     8 */
	char *                     buffer[8];            /*    16    64 */

	/* size: 80, cachelines: 2, members: 6 */
	/* sum members: 79, holes: 1, sum holes: 1 */
	/* last cacheline: 16 bytes */
};
struct fs_context_operations {
	void                       (*free)(struct fs_context *); /*     0     8 */
	int                        (*dup)(struct fs_context *, struct fs_context *); /*     8     8 */
	int                        (*parse_param)(struct fs_context *, struct fs_parameter *); /*    16     8 */
	int                        (*parse_monolithic)(struct fs_context *, void *); /*    24     8 */
	int                        (*get_tree)(struct fs_context *); /*    32     8 */
	int                        (*reconfigure)(struct fs_context *); /*    40     8 */

	/* size: 48, cachelines: 1, members: 6 */
	/* last cacheline: 48 bytes */
};
struct fs_parse_result {
	bool                       negated;              /*     0     1 */

	/* XXX 7 bytes hole, try to pack */

	union {
		bool               boolean;              /*     8     1 */
		int                int_32;               /*     8     4 */
		unsigned int       uint_32;              /*     8     4 */
		u64                uint_64;              /*     8     8 */
	};                                               /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 9, holes: 1, sum holes: 7 */
	/* last cacheline: 16 bytes */
};
struct trace_event_raw_initcall_level {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        __data_loc_level;     /*     8     4 */
	char                       __data[];             /*    12     0 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct trace_event_raw_initcall_start {
	struct trace_entry         ent;                  /*     0     8 */
	initcall_t                 func;                 /*     8     8 */
	char                       __data[];             /*    16     0 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct trace_event_raw_initcall_finish {
	struct trace_entry         ent;                  /*     0     8 */
	initcall_t                 func;                 /*     8     8 */
	int                        ret;                  /*    16     4 */
	char                       __data[];             /*    20     0 */

	/* size: 24, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct trace_event_data_offsets_initcall_level {
	u32                        level;                /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct trace_event_data_offsets_initcall_start {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_initcall_finish {

	/* size: 0, cachelines: 0, members: 0 */
};
struct blacklist_entry {
	struct list_head           next;                 /*     0    16 */
	char *                     buf;                  /*    16     8 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct elf32_note {
	Elf32_Word                 n_namesz;             /*     0     4 */
	Elf32_Word                 n_descsz;             /*     4     4 */
	Elf32_Word                 n_type;               /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct llist_head {
	struct llist_node *        first;                /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct notifier_block {
	notifier_fn_t              notifier_call;        /*     0     8 */
	struct notifier_block *    next;                 /*     8     8 */
	int                        priority;             /*    16     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct blocking_notifier_head {
	struct rw_semaphore        rwsem;                /*     0    40 */
	struct notifier_block *    head;                 /*    40     8 */

	/* size: 48, cachelines: 1, members: 2 */
	/* last cacheline: 48 bytes */
};
struct raw_notifier_head {
	struct notifier_block *    head;                 /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct page_pool {
	struct page_pool_params    p;                    /*     0    40 */

	/* XXX last struct has 4 bytes of padding */

	struct delayed_work        release_dw;           /*    40   192 */
	/* --- cacheline 3 boundary (192 bytes) was 40 bytes ago --- */
	void                       (*disconnect)(void *); /*   232     8 */
	long unsigned int          defer_start;          /*   240     8 */
	long unsigned int          defer_warn;           /*   248     8 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	u32                        pages_state_hold_cnt; /*   256     4 */
	unsigned int               frag_offset;          /*   260     4 */
	struct page *              frag_page;            /*   264     8 */
	long int                   frag_users;           /*   272     8 */

	/* XXX 40 bytes hole, try to pack */

	/* --- cacheline 5 boundary (320 bytes) --- */
	struct pp_alloc_cache      alloc __attribute__((__aligned__(64))); /*   320  1032 */

	/* XXX 56 bytes hole, try to pack */

	/* --- cacheline 22 boundary (1408 bytes) --- */
	struct ptr_ring            ring __attribute__((__aligned__(64))); /*  1408   192 */

	/* XXX last struct has 48 bytes of padding */

	/* --- cacheline 25 boundary (1600 bytes) --- */
	atomic_t                   pages_state_release_cnt; /*  1600     4 */
	refcount_t                 user_cnt;             /*  1604     4 */
	u64                        destroy_cnt;          /*  1608     8 */
	u64                        kabi_reserved1;       /*  1616     8 */

	/* size: 1664, cachelines: 26, members: 15 */
	/* sum members: 1528, holes: 2, sum holes: 96 */
	/* padding: 40 */
	/* paddings: 2, sum paddings: 52 */
	/* forced alignments: 2, forced holes: 2, sum forced holes: 96 */
} __attribute__((__aligned__(64)));
struct sock {
	struct sock_common         __sk_common;          /*     0   136 */
	/* --- cacheline 2 boundary (128 bytes) was 8 bytes ago --- */
	socket_lock_t              sk_lock;              /*   136    32 */
	atomic_t                   sk_drops;             /*   168     4 */
	int                        sk_rcvlowat;          /*   172     4 */
	struct sk_buff_head        sk_error_queue;       /*   176    24 */
	/* --- cacheline 3 boundary (192 bytes) was 8 bytes ago --- */
	struct sk_buff *           sk_rx_skb_cache;      /*   200     8 */
	struct sk_buff_head        sk_receive_queue;     /*   208    24 */
	struct {
		atomic_t           rmem_alloc;           /*   232     4 */
		int                len;                  /*   236     4 */
		struct sk_buff *   head;                 /*   240     8 */
		struct sk_buff *   tail;                 /*   248     8 */
	} sk_backlog;                                    /*   232    24 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	struct llist_head          defer_list;           /*   256     8 */
	int                        sk_forward_alloc;     /*   264     4 */
	unsigned int               sk_ll_usec;           /*   268     4 */
	unsigned int               sk_napi_id;           /*   272     4 */
	int                        sk_rcvbuf;            /*   276     4 */
	struct sk_filter *         sk_filter;            /*   280     8 */
	union {
		struct socket_wq * sk_wq;                /*   288     8 */
		struct socket_wq * sk_wq_raw;            /*   288     8 */
	};                                               /*   288     8 */
	struct xfrm_policy *       sk_policy[2];         /*   296    16 */
	struct dst_entry *         sk_rx_dst;            /*   312     8 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	struct dst_entry *         sk_dst_cache;         /*   320     8 */
	atomic_t                   sk_omem_alloc;        /*   328     4 */
	int                        sk_sndbuf;            /*   332     4 */
	int                        sk_wmem_queued;       /*   336     4 */
	refcount_t                 sk_wmem_alloc;        /*   340     4 */
	long unsigned int          sk_tsq_flags;         /*   344     8 */
	union {
		struct sk_buff *   sk_send_head;         /*   352     8 */
		struct rb_root     tcp_rtx_queue;        /*   352     8 */
	};                                               /*   352     8 */
	struct sk_buff *           sk_tx_skb_cache;      /*   360     8 */
	struct sk_buff_head        sk_write_queue;       /*   368    24 */
	/* --- cacheline 6 boundary (384 bytes) was 8 bytes ago --- */
	__s32                      sk_peek_off;          /*   392     4 */
	int                        sk_write_pending;     /*   396     4 */
	__u32                      sk_dst_pending_confirm; /*   400     4 */
	u32                        sk_pacing_status;     /*   404     4 */
	long int                   sk_sndtimeo;          /*   408     8 */
	struct timer_list          sk_timer;             /*   416    72 */
	/* --- cacheline 7 boundary (448 bytes) was 40 bytes ago --- */
	__u32                      sk_priority;          /*   488     4 */
	__u32                      sk_mark;              /*   492     4 */
	long unsigned int          sk_pacing_rate;       /*   496     8 */
	long unsigned int          sk_max_pacing_rate;   /*   504     8 */
	/* --- cacheline 8 boundary (512 bytes) --- */
	struct page_frag           sk_frag;              /*   512    16 */
	netdev_features_t          sk_route_caps;        /*   528     8 */
	netdev_features_t          sk_route_nocaps;      /*   536     8 */
	netdev_features_t          sk_route_forced_caps; /*   544     8 */
	int                        sk_gso_type;          /*   552     4 */
	unsigned int               sk_gso_max_size;      /*   556     4 */
	gfp_t                      sk_allocation;        /*   560     4 */
	__u32                      sk_txhash;            /*   564     4 */
	u8                         sk_padding:1;         /*   568: 0  1 */
	u8                         sk_kern_sock:1;       /*   568: 1  1 */
	u8                         sk_no_check_tx:1;     /*   568: 2  1 */
	u8                         sk_no_check_rx:1;     /*   568: 3  1 */
	u8                         sk_userlocks:4;       /*   568: 4  1 */
	u8                         sk_pacing_shift;      /*   569     1 */
	u16                        sk_type;              /*   570     2 */
	u16                        sk_protocol;          /*   572     2 */
	u16                        sk_gso_max_segs;      /*   574     2 */
	/* --- cacheline 9 boundary (576 bytes) --- */
	long unsigned int          sk_lingertime;        /*   576     8 */
	struct proto *             sk_prot_creator;      /*   584     8 */
	rwlock_t                   sk_callback_lock;     /*   592     8 */
	int                        sk_err;               /*   600     4 */
	int                        sk_err_soft;          /*   604     4 */
	u32                        sk_ack_backlog;       /*   608     4 */
	u32                        sk_max_ack_backlog;   /*   612     4 */
	kuid_t                     sk_uid;               /*   616     4 */
	spinlock_t                 sk_peer_lock;         /*   620     4 */
	struct pid *               sk_peer_pid;          /*   624     8 */
	const struct cred  *       sk_peer_cred;         /*   632     8 */
	/* --- cacheline 10 boundary (640 bytes) --- */
	long int                   sk_rcvtimeo;          /*   640     8 */
	ktime_t                    sk_stamp;             /*   648     8 */
	u16                        sk_tsflags;           /*   656     2 */
	u8                         sk_shutdown;          /*   658     1 */

	/* XXX 1 byte hole, try to pack */

	u32                        sk_tskey;             /*   660     4 */
	atomic_t                   sk_zckey;             /*   664     4 */
	u8                         sk_clockid;           /*   668     1 */
	u8                         sk_txtime_deadline_mode:1; /*   669: 0  1 */
	u8                         sk_txtime_report_errors:1; /*   669: 1  1 */
	u8                         sk_txtime_unused:6;   /*   669: 2  1 */

	/* XXX 2 bytes hole, try to pack */

	struct socket *            sk_socket;            /*   672     8 */
	void *                     sk_user_data;         /*   680     8 */
	void *                     sk_security;          /*   688     8 */
	struct sock_cgroup_data    sk_cgrp_data;         /*   696    16 */

	/* XXX last struct has 2 bytes of padding */

	/* --- cacheline 11 boundary (704 bytes) was 8 bytes ago --- */
	struct mem_cgroup *        sk_memcg;             /*   712     8 */
	void                       (*sk_state_change)(struct sock *); /*   720     8 */
	void                       (*sk_data_ready)(struct sock *); /*   728     8 */
	void                       (*sk_write_space)(struct sock *); /*   736     8 */
	void                       (*sk_error_report)(struct sock *); /*   744     8 */
	int                        (*sk_backlog_rcv)(struct sock *, struct sk_buff *); /*   752     8 */
	struct sk_buff *           (*sk_validate_xmit_skb)(struct sock *, struct net_device *, struct sk_buff *); /*   760     8 */
	/* --- cacheline 12 boundary (768 bytes) --- */
	void                       (*sk_destruct)(struct sock *); /*   768     8 */
	struct sock_reuseport *    sk_reuseport_cb;      /*   776     8 */
	struct bpf_local_storage * sk_bpf_storage;       /*   784     8 */
	struct callback_head       sk_rcu __attribute__((__aligned__(8))); /*   792    16 */
	union {
		kgid_t             sk_gid;               /*   808     4 */
		u64                sk_gid_padding;       /*   808     8 */
	};                                               /*   808     8 */
	u64                        kabi_reserved2;       /*   816     8 */
	u64                        kabi_reserved3;       /*   824     8 */
	/* --- cacheline 13 boundary (832 bytes) --- */
	u64                        kabi_reserved4;       /*   832     8 */
	u64                        kabi_reserved5;       /*   840     8 */
	u64                        kabi_reserved6;       /*   848     8 */
	u64                        kabi_reserved7;       /*   856     8 */
	u64                        kabi_reserved8;       /*   864     8 */

	/* size: 872, cachelines: 14, members: 97 */
	/* sum members: 867, holes: 2, sum holes: 3 */
	/* sum bitfield members: 16 bits (2 bytes) */
	/* paddings: 1, sum paddings: 2 */
	/* forced alignments: 1 */
	/* last cacheline: 40 bytes */
} __attribute__((__aligned__(8)));
struct rhash_head {
	struct rhash_head *        next;                 /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct rhashtable_compare_arg {
	struct rhashtable *        ht;                   /*     0     8 */
	const void  *              key;                  /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct rhashtable {
	struct bucket_table *      tbl;                  /*     0     8 */
	unsigned int               key_len;              /*     8     4 */
	unsigned int               max_elems;            /*    12     4 */
	struct rhashtable_params   p;                    /*    16    40 */
	bool                       rhlist;               /*    56     1 */

	/* XXX 7 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	struct work_struct         run_work;             /*    64    64 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct mutex               mutex;                /*   128    32 */
	spinlock_t                 lock;                 /*   160     4 */
	atomic_t                   nelems;               /*   164     4 */

	/* size: 168, cachelines: 3, members: 9 */
	/* sum members: 161, holes: 1, sum holes: 7 */
	/* last cacheline: 40 bytes */
};
struct rhashtable_params {
	u16                        nelem_hint;           /*     0     2 */
	u16                        key_len;              /*     2     2 */
	u16                        key_offset;           /*     4     2 */
	u16                        head_offset;          /*     6     2 */
	unsigned int               max_size;             /*     8     4 */
	u16                        min_size;             /*    12     2 */
	bool                       automatic_shrinking;  /*    14     1 */

	/* XXX 1 byte hole, try to pack */

	rht_hashfn_t               hashfn;               /*    16     8 */
	rht_obj_hashfn_t           obj_hashfn;           /*    24     8 */
	rht_obj_cmpfn_t            obj_cmpfn;            /*    32     8 */

	/* size: 40, cachelines: 1, members: 10 */
	/* sum members: 39, holes: 1, sum holes: 1 */
	/* last cacheline: 40 bytes */
};
struct fs_struct {
	int                        users;                /*     0     4 */
	spinlock_t                 lock;                 /*     4     4 */
	seqcount_spinlock_t        seq;                  /*     8     4 */
	int                        umask;                /*    12     4 */
	int                        in_exec;              /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	struct path                root;                 /*    24    16 */
	struct path                pwd;                  /*    40    16 */

	/* size: 56, cachelines: 1, members: 7 */
	/* sum members: 52, holes: 1, sum holes: 4 */
	/* last cacheline: 56 bytes */
};
struct pipe_inode_info {
	struct mutex               mutex;                /*     0    32 */
	wait_queue_head_t          rd_wait;              /*    32    24 */
	wait_queue_head_t          wr_wait;              /*    56    24 */
	/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
	unsigned int               head;                 /*    80     4 */
	unsigned int               tail;                 /*    84     4 */
	unsigned int               max_usage;            /*    88     4 */
	unsigned int               ring_size;            /*    92     4 */
	unsigned int               nr_accounted;         /*    96     4 */
	unsigned int               readers;              /*   100     4 */
	unsigned int               writers;              /*   104     4 */
	unsigned int               files;                /*   108     4 */
	unsigned int               r_counter;            /*   112     4 */
	unsigned int               w_counter;            /*   116     4 */
	union {
		bool               poll_usage;           /*   120     1 */
		struct {
			unsigned int poll_usage;         /*   120     4 */
		} kabi_hidden_74;                        /*   120     4 */
		union {
		};                                       /*   120     0 */
	};                                               /*   120     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 2 boundary (128 bytes) --- */
	struct page *              tmp_page;             /*   128     8 */
	struct fasync_struct *     fasync_readers;       /*   136     8 */
	struct fasync_struct *     fasync_writers;       /*   144     8 */
	struct pipe_buffer *       bufs;                 /*   152     8 */
	struct user_struct *       user;                 /*   160     8 */

	/* size: 168, cachelines: 3, members: 19 */
	/* sum members: 164, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct sockaddr {
	sa_family_t                sa_family;            /*     0     2 */
	char                       sa_data[14];          /*     2    14 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct msghdr {
	void *                     msg_name;             /*     0     8 */
	int                        msg_namelen;          /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct iov_iter            msg_iter;             /*    16    40 */
	union {
		void *             msg_control;          /*    56     8 */
		void *             msg_control_user;     /*    56     8 */
	};                                               /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	bool                       msg_control_is_user:1; /*    64: 0  1 */

	/* XXX 7 bits hole, try to pack */
	/* XXX 7 bytes hole, try to pack */

	__kernel_size_t            msg_controllen;       /*    72     8 */
	unsigned int               msg_flags;            /*    80     4 */

	/* XXX 4 bytes hole, try to pack */

	struct kiocb *             msg_iocb;             /*    88     8 */

	/* size: 96, cachelines: 2, members: 8 */
	/* sum members: 80, holes: 3, sum holes: 15 */
	/* sum bitfield members: 1 bits, bit holes: 1, sum bit holes: 7 bits */
	/* last cacheline: 32 bytes */
};
struct ifmap {
	long unsigned int          mem_start;            /*     0     8 */
	long unsigned int          mem_end;              /*     8     8 */
	short unsigned int         base_addr;            /*    16     2 */
	unsigned char              irq;                  /*    18     1 */
	unsigned char              dma;                  /*    19     1 */
	unsigned char              port;                 /*    20     1 */

	/* size: 24, cachelines: 1, members: 6 */
	/* padding: 3 */
	/* last cacheline: 24 bytes */
};
struct if_settings {
	unsigned int               type;                 /*     0     4 */
	unsigned int               size;                 /*     4     4 */
	union {
		raw_hdlc_proto *   raw_hdlc;             /*     8     8 */
		cisco_proto *      cisco;                /*     8     8 */
		fr_proto *         fr;                   /*     8     8 */
		fr_proto_pvc *     fr_pvc;               /*     8     8 */
		fr_proto_pvc_info * fr_pvc_info;         /*     8     8 */
		x25_hdlc_proto *   x25;                  /*     8     8 */
		sync_serial_settings * sync;             /*     8     8 */
		te1_settings *     te1;                  /*     8     8 */
	} ifs_ifsu;                                      /*     8     8 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct ifreq {
	union {
		char               ifrn_name[16];        /*     0    16 */
	} ifr_ifrn;                                      /*     0    16 */
	union {
		struct sockaddr    ifru_addr;            /*    16    16 */
		struct sockaddr    ifru_dstaddr;         /*    16    16 */
		struct sockaddr    ifru_broadaddr;       /*    16    16 */
		struct sockaddr    ifru_netmask;         /*    16    16 */
		struct sockaddr    ifru_hwaddr;          /*    16    16 */
		short int          ifru_flags;           /*    16     2 */
		int                ifru_ivalue;          /*    16     4 */
		int                ifru_mtu;             /*    16     4 */
		struct ifmap       ifru_map;             /*    16    24 */
		char               ifru_slave[16];       /*    16    16 */
		char               ifru_newname[16];     /*    16    16 */
		void *             ifru_data;            /*    16     8 */
		struct if_settings ifru_settings;        /*    16    16 */
	} ifr_ifru;                                      /*    16    24 */

	/* size: 40, cachelines: 1, members: 2 */
	/* last cacheline: 40 bytes */
};
struct tty_struct {
	int                        magic;                /*     0     4 */
	struct kref                kref;                 /*     4     4 */
	struct device *            dev;                  /*     8     8 */
	struct tty_driver *        driver;               /*    16     8 */
	const struct tty_operations  * ops;              /*    24     8 */
	int                        index;                /*    32     4 */

	/* XXX 4 bytes hole, try to pack */

	struct ld_semaphore        ldisc_sem;            /*    40    48 */
	/* --- cacheline 1 boundary (64 bytes) was 24 bytes ago --- */
	struct tty_ldisc *         ldisc;                /*    88     8 */
	struct mutex               atomic_write_lock;    /*    96    32 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct mutex               legacy_mutex;         /*   128    32 */
	struct mutex               throttle_mutex;       /*   160    32 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	struct rw_semaphore        termios_rwsem;        /*   192    40 */
	struct mutex               winsize_mutex;        /*   232    32 */
	/* --- cacheline 4 boundary (256 bytes) was 8 bytes ago --- */
	spinlock_t                 ctrl_lock;            /*   264     4 */
	spinlock_t                 flow_lock;            /*   268     4 */
	struct ktermios            termios;              /*   272    44 */
	struct ktermios            termios_locked;       /*   316    44 */
	/* --- cacheline 5 boundary (320 bytes) was 40 bytes ago --- */
	char                       name[64];             /*   360    64 */
	/* --- cacheline 6 boundary (384 bytes) was 40 bytes ago --- */
	struct pid *               pgrp;                 /*   424     8 */
	struct pid *               session;              /*   432     8 */
	long unsigned int          flags;                /*   440     8 */
	/* --- cacheline 7 boundary (448 bytes) --- */
	int                        count;                /*   448     4 */
	struct winsize             winsize;              /*   452     8 */

	/* Bitfield combined with next fields */

	long unsigned int          stopped:1;            /*   456:32  8 */
	long unsigned int          flow_stopped:1;       /*   456:33  8 */

	/* XXX 30 bits hole, try to pack */

	/* Force alignment to the next boundary: */
	long unsigned int          :0;

	long unsigned int          unused:62;            /*   464: 0  8 */

	/* XXX 2 bits hole, try to pack */

	int                        hw_stopped;           /*   472     4 */

	/* Bitfield combined with previous fields */

	long unsigned int          ctrl_status:8;        /*   472:32  8 */
	long unsigned int          packet:1;             /*   472:40  8 */

	/* XXX 23 bits hole, try to pack */

	/* Force alignment to the next boundary: */
	long unsigned int          :0;

	long unsigned int          unused_ctrl:55;       /*   480: 0  8 */

	/* XXX 9 bits hole, try to pack */

	unsigned int               receive_room;         /*   488     4 */
	int                        flow_change;          /*   492     4 */
	struct tty_struct *        link;                 /*   496     8 */
	struct fasync_struct *     fasync;               /*   504     8 */
	/* --- cacheline 8 boundary (512 bytes) --- */
	wait_queue_head_t          write_wait;           /*   512    24 */
	wait_queue_head_t          read_wait;            /*   536    24 */
	struct work_struct         hangup_work;          /*   560    64 */
	/* --- cacheline 9 boundary (576 bytes) was 48 bytes ago --- */
	void *                     disc_data;            /*   624     8 */
	void *                     driver_data;          /*   632     8 */
	/* --- cacheline 10 boundary (640 bytes) --- */
	spinlock_t                 files_lock;           /*   640     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           tty_files;            /*   648    16 */
	int                        closing;              /*   664     4 */

	/* XXX 4 bytes hole, try to pack */

	unsigned char *            write_buf;            /*   672     8 */
	int                        write_cnt;            /*   680     4 */

	/* XXX 4 bytes hole, try to pack */

	struct work_struct         SAK_work;             /*   688    64 */
	/* --- cacheline 11 boundary (704 bytes) was 48 bytes ago --- */
	struct tty_port *          port;                 /*   752     8 */

	/* size: 760, cachelines: 12, members: 46 */
	/* sum members: 720, holes: 4, sum holes: 16 */
	/* sum bitfield members: 128 bits, bit holes: 4, sum bit holes: 64 bits */
	/* last cacheline: 56 bytes */
};
struct posix_acl {
	refcount_t                 a_refcount;           /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct callback_head       a_rcu __attribute__((__aligned__(8))); /*     8    16 */
	unsigned int               a_count;              /*    24     4 */
	struct posix_acl_entry     a_entries[];          /*    28     0 */

	/* size: 32, cachelines: 1, members: 4 */
	/* sum members: 24, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 4 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct ktermios {
	tcflag_t                   c_iflag;              /*     0     4 */
	tcflag_t                   c_oflag;              /*     4     4 */
	tcflag_t                   c_cflag;              /*     8     4 */
	tcflag_t                   c_lflag;              /*    12     4 */
	cc_t                       c_line;               /*    16     1 */
	cc_t                       c_cc[19];             /*    17    19 */
	speed_t                    c_ispeed;             /*    36     4 */
	speed_t                    c_ospeed;             /*    40     4 */

	/* size: 44, cachelines: 1, members: 8 */
	/* last cacheline: 44 bytes */
};
struct winsize {
	short unsigned int         ws_row;               /*     0     2 */
	short unsigned int         ws_col;               /*     2     2 */
	short unsigned int         ws_xpixel;            /*     4     2 */
	short unsigned int         ws_ypixel;            /*     6     2 */

	/* size: 8, cachelines: 1, members: 4 */
	/* last cacheline: 8 bytes */
};
struct tty_operations {
	struct tty_struct *        (*lookup)(struct tty_driver *, struct file *, int); /*     0     8 */
	int                        (*install)(struct tty_driver *, struct tty_struct *); /*     8     8 */
	void                       (*remove)(struct tty_driver *, struct tty_struct *); /*    16     8 */
	int                        (*open)(struct tty_struct *, struct file *); /*    24     8 */
	void                       (*close)(struct tty_struct *, struct file *); /*    32     8 */
	void                       (*shutdown)(struct tty_struct *); /*    40     8 */
	void                       (*cleanup)(struct tty_struct *); /*    48     8 */
	int                        (*write)(struct tty_struct *, const unsigned char  *, int); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*put_char)(struct tty_struct *, unsigned char); /*    64     8 */
	void                       (*flush_chars)(struct tty_struct *); /*    72     8 */
	int                        (*write_room)(struct tty_struct *); /*    80     8 */
	int                        (*chars_in_buffer)(struct tty_struct *); /*    88     8 */
	int                        (*ioctl)(struct tty_struct *, unsigned int, long unsigned int); /*    96     8 */
	long int                   (*compat_ioctl)(struct tty_struct *, unsigned int, long unsigned int); /*   104     8 */
	void                       (*set_termios)(struct tty_struct *, struct ktermios *); /*   112     8 */
	void                       (*throttle)(struct tty_struct *); /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	void                       (*unthrottle)(struct tty_struct *); /*   128     8 */
	void                       (*stop)(struct tty_struct *); /*   136     8 */
	void                       (*start)(struct tty_struct *); /*   144     8 */
	void                       (*hangup)(struct tty_struct *); /*   152     8 */
	int                        (*break_ctl)(struct tty_struct *, int); /*   160     8 */
	void                       (*flush_buffer)(struct tty_struct *); /*   168     8 */
	void                       (*set_ldisc)(struct tty_struct *); /*   176     8 */
	void                       (*wait_until_sent)(struct tty_struct *, int); /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	void                       (*send_xchar)(struct tty_struct *, char); /*   192     8 */
	int                        (*tiocmget)(struct tty_struct *); /*   200     8 */
	int                        (*tiocmset)(struct tty_struct *, unsigned int, unsigned int); /*   208     8 */
	int                        (*resize)(struct tty_struct *, struct winsize *); /*   216     8 */
	int                        (*get_icount)(struct tty_struct *, struct serial_icounter_struct *); /*   224     8 */
	int                        (*get_serial)(struct tty_struct *, struct serial_struct *); /*   232     8 */
	int                        (*set_serial)(struct tty_struct *, struct serial_struct *); /*   240     8 */
	void                       (*show_fdinfo)(struct tty_struct *, struct seq_file *); /*   248     8 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	int                        (*poll_init)(struct tty_driver *, int, char *); /*   256     8 */
	int                        (*poll_get_char)(struct tty_driver *, int); /*   264     8 */
	void                       (*poll_put_char)(struct tty_driver *, int, char); /*   272     8 */
	int                        (*proc_show)(struct seq_file *, void *); /*   280     8 */

	/* size: 288, cachelines: 5, members: 36 */
	/* last cacheline: 32 bytes */
};
struct tty_driver {
	int                        magic;                /*     0     4 */
	struct kref                kref;                 /*     4     4 */
	struct cdev * *            cdevs;                /*     8     8 */
	struct module *            owner;                /*    16     8 */
	const char  *              driver_name;          /*    24     8 */
	const char  *              name;                 /*    32     8 */
	int                        name_base;            /*    40     4 */
	int                        major;                /*    44     4 */
	int                        minor_start;          /*    48     4 */
	unsigned int               num;                  /*    52     4 */
	short int                  type;                 /*    56     2 */
	short int                  subtype;              /*    58     2 */
	struct ktermios            init_termios;         /*    60    44 */
	/* --- cacheline 1 boundary (64 bytes) was 40 bytes ago --- */
	long unsigned int          flags;                /*   104     8 */
	struct proc_dir_entry *    proc_entry;           /*   112     8 */
	struct tty_driver *        other;                /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct tty_struct * *      ttys;                 /*   128     8 */
	struct tty_port * *        ports;                /*   136     8 */
	struct ktermios * *        termios;              /*   144     8 */
	void *                     driver_state;         /*   152     8 */
	const struct tty_operations  * ops;              /*   160     8 */
	struct list_head           tty_drivers;          /*   168    16 */

	/* size: 184, cachelines: 3, members: 22 */
	/* last cacheline: 56 bytes */
};
struct tty_port {
	struct tty_bufhead         buf;                  /*     0   168 */
	/* --- cacheline 2 boundary (128 bytes) was 40 bytes ago --- */
	struct tty_struct *        tty;                  /*   168     8 */
	struct tty_struct *        itty;                 /*   176     8 */
	const struct tty_port_operations  * ops;         /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	const struct tty_port_client_operations  * client_ops; /*   192     8 */
	spinlock_t                 lock;                 /*   200     4 */
	int                        blocked_open;         /*   204     4 */
	int                        count;                /*   208     4 */

	/* XXX 4 bytes hole, try to pack */

	wait_queue_head_t          open_wait;            /*   216    24 */
	wait_queue_head_t          delta_msr_wait;       /*   240    24 */
	/* --- cacheline 4 boundary (256 bytes) was 8 bytes ago --- */
	long unsigned int          flags;                /*   264     8 */
	long unsigned int          iflags;               /*   272     8 */
	unsigned char              console:1;            /*   280: 0  1 */
	unsigned char              low_latency:1;        /*   280: 1  1 */

	/* XXX 6 bits hole, try to pack */
	/* XXX 7 bytes hole, try to pack */

	struct mutex               mutex;                /*   288    32 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	struct mutex               buf_mutex;            /*   320    32 */
	unsigned char *            xmit_buf;             /*   352     8 */
	unsigned int               close_delay;          /*   360     4 */
	unsigned int               closing_wait;         /*   364     4 */
	int                        drain_delay;          /*   368     4 */
	struct kref                kref;                 /*   372     4 */
	void *                     client_data;          /*   376     8 */

	/* size: 384, cachelines: 6, members: 22 */
	/* sum members: 372, holes: 2, sum holes: 11 */
	/* sum bitfield members: 2 bits, bit holes: 1, sum bit holes: 6 bits */
};
struct ld_semaphore {
	atomic_long_t              count;                /*     0     8 */
	raw_spinlock_t             wait_lock;            /*     8     4 */
	unsigned int               wait_readers;         /*    12     4 */
	struct list_head           read_wait;            /*    16    16 */
	struct list_head           write_wait;           /*    32    16 */

	/* size: 48, cachelines: 1, members: 5 */
	/* last cacheline: 48 bytes */
};
struct tty_ldisc_ops {
	int                        magic;                /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	char *                     name;                 /*     8     8 */
	int                        num;                  /*    16     4 */
	int                        flags;                /*    20     4 */
	int                        (*open)(struct tty_struct *); /*    24     8 */
	void                       (*close)(struct tty_struct *); /*    32     8 */
	void                       (*flush_buffer)(struct tty_struct *); /*    40     8 */
	ssize_t                    (*read)(struct tty_struct *, struct file *, unsigned char *, size_t, void * *, long unsigned int); /*    48     8 */
	ssize_t                    (*write)(struct tty_struct *, struct file *, const unsigned char  *, size_t); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*ioctl)(struct tty_struct *, struct file *, unsigned int, long unsigned int); /*    64     8 */
	int                        (*compat_ioctl)(struct tty_struct *, struct file *, unsigned int, long unsigned int); /*    72     8 */
	void                       (*set_termios)(struct tty_struct *, struct ktermios *); /*    80     8 */
	__poll_t                   (*poll)(struct tty_struct *, struct file *, struct poll_table_struct *); /*    88     8 */
	int                        (*hangup)(struct tty_struct *); /*    96     8 */
	void                       (*receive_buf)(struct tty_struct *, const unsigned char  *, char *, int); /*   104     8 */
	void                       (*write_wakeup)(struct tty_struct *); /*   112     8 */
	void                       (*dcd_change)(struct tty_struct *, unsigned int); /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	int                        (*receive_buf2)(struct tty_struct *, const unsigned char  *, char *, int); /*   128     8 */
	struct module *            owner;                /*   136     8 */
	int                        refcount;             /*   144     4 */

	/* size: 152, cachelines: 3, members: 20 */
	/* sum members: 144, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct tty_ldisc {
	struct tty_ldisc_ops *     ops;                  /*     0     8 */
	struct tty_struct *        tty;                  /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct tty_buffer {
	union {
		struct tty_buffer * next;                /*     0     8 */
		struct llist_node  free;                 /*     0     8 */
	};                                               /*     0     8 */
	int                        used;                 /*     8     4 */
	int                        size;                 /*    12     4 */
	int                        commit;               /*    16     4 */
	int                        read;                 /*    20     4 */
	int                        flags;                /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          data[];               /*    32     0 */

	/* size: 32, cachelines: 1, members: 7 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct tty_bufhead {
	struct tty_buffer *        head;                 /*     0     8 */
	struct work_struct         work;                 /*     8    64 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	struct mutex               lock;                 /*    72    32 */
	atomic_t                   priority;             /*   104     4 */

	/* XXX 4 bytes hole, try to pack */

	struct tty_buffer          sentinel;             /*   112    32 */
	/* --- cacheline 2 boundary (128 bytes) was 16 bytes ago --- */
	struct llist_head          free;                 /*   144     8 */
	atomic_t                   mem_used;             /*   152     4 */
	int                        mem_limit;            /*   156     4 */
	struct tty_buffer *        tail;                 /*   160     8 */

	/* size: 168, cachelines: 3, members: 9 */
	/* sum members: 164, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct tty_port_operations {
	int                        (*carrier_raised)(struct tty_port *); /*     0     8 */
	void                       (*dtr_rts)(struct tty_port *, int); /*     8     8 */
	void                       (*shutdown)(struct tty_port *); /*    16     8 */
	int                        (*activate)(struct tty_port *, struct tty_struct *); /*    24     8 */
	void                       (*destruct)(struct tty_port *); /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct tty_port_client_operations {
	int                        (*receive_buf)(struct tty_port *, const unsigned char  *, const unsigned char  *, size_t); /*     0     8 */
	void                       (*write_wakeup)(struct tty_port *); /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct net {
	refcount_t                 passive;              /*     0     4 */
	refcount_t                 count;                /*     4     4 */
	spinlock_t                 rules_mod_lock;       /*     8     4 */
	unsigned int               dev_unreg_count;      /*    12     4 */
	unsigned int               dev_base_seq;         /*    16     4 */
	int                        ifindex;              /*    20     4 */
	spinlock_t                 nsid_lock;            /*    24     4 */
	atomic_t                   fnhe_genid;           /*    28     4 */
	struct list_head           list;                 /*    32    16 */
	struct list_head           exit_list;            /*    48    16 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct llist_node          cleanup_list;         /*    64     8 */
	struct key_tag *           key_domain;           /*    72     8 */
	struct user_namespace *    user_ns;              /*    80     8 */
	struct ucounts *           ucounts;              /*    88     8 */
	struct idr                 netns_ids;            /*    96    24 */
	struct ns_common           ns;                   /*   120    32 */
	/* --- cacheline 2 boundary (128 bytes) was 24 bytes ago --- */
	struct list_head           dev_base_head;        /*   152    16 */
	struct proc_dir_entry *    proc_net;             /*   168     8 */
	struct proc_dir_entry *    proc_net_stat;        /*   176     8 */
	struct ctl_table_set       sysctls __attribute__((__aligned__(8))); /*   184    96 */
	/* --- cacheline 4 boundary (256 bytes) was 24 bytes ago --- */
	struct sock *              rtnl;                 /*   280     8 */
	struct sock *              genl_sock;            /*   288     8 */
	struct uevent_sock *       uevent_sock;          /*   296     8 */
	struct hlist_head *        dev_name_head;        /*   304     8 */
	struct hlist_head *        dev_index_head;       /*   312     8 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	struct raw_notifier_head   netdev_chain;         /*   320     8 */
	u32                        hash_mix;             /*   328     4 */

	/* XXX 4 bytes hole, try to pack */

	struct net_device *        loopback_dev;         /*   336     8 */
	struct list_head           rules_ops;            /*   344    16 */
	struct netns_core          core;                 /*   360    32 */
	/* --- cacheline 6 boundary (384 bytes) was 8 bytes ago --- */
	struct netns_mib           mib;                  /*   392   120 */
	/* --- cacheline 8 boundary (512 bytes) --- */
	struct netns_packet        packet;               /*   512    40 */
	struct netns_unix          unx;                  /*   552    16 */
	struct netns_nexthop       nexthop;              /*   568    72 */
	/* --- cacheline 10 boundary (640 bytes) --- */
	struct netns_ipv4          ipv4 __attribute__((__aligned__(64))); /*   640  1024 */

	/* XXX last struct has 8 bytes of padding */

	/* --- cacheline 26 boundary (1664 bytes) --- */
	struct netns_ipv6          ipv6 __attribute__((__aligned__(64))); /*  1664   896 */
	/* --- cacheline 40 boundary (2560 bytes) --- */
	struct netns_ieee802154_lowpan ieee802154_lowpan; /*  2560    16 */
	struct netns_sctp          sctp;                 /*  2576   296 */
	/* --- cacheline 44 boundary (2816 bytes) was 56 bytes ago --- */
	struct netns_nf            nf;                   /*  2872   288 */
	/* --- cacheline 49 boundary (3136 bytes) was 24 bytes ago --- */
	struct netns_xt            xt;                   /*  3160   240 */
	/* --- cacheline 53 boundary (3392 bytes) was 8 bytes ago --- */
	struct netns_ct            ct;                   /*  3400   480 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 60 boundary (3840 bytes) was 40 bytes ago --- */
	struct netns_nftables      nft;                  /*  3880   112 */
	/* --- cacheline 62 boundary (3968 bytes) was 24 bytes ago --- */
	struct netns_nf_frag       nf_frag;              /*  3992     8 */
	struct ctl_table_header *  nf_frag_frags_hdr;    /*  4000     8 */
	struct sock *              nfnl;                 /*  4008     8 */
	struct sock *              nfnl_stash;           /*  4016     8 */
	struct list_head           nfct_timeout_list;    /*  4024    16 */
	/* --- cacheline 63 boundary (4032 bytes) was 8 bytes ago --- */
	struct net_generic *       gen;                  /*  4040     8 */
	struct netns_bpf           bpf;                  /*  4048    64 */

	/* XXX 48 bytes hole, try to pack */

	/* --- cacheline 65 boundary (4160 bytes) --- */
	struct netns_xfrm          xfrm __attribute__((__aligned__(64))); /*  4160  1024 */

	/* XXX last struct has 8 bytes of padding */

	/* --- cacheline 81 boundary (5184 bytes) --- */
	atomic64_t                 net_cookie;           /*  5184     8 */
	struct netns_ipvs *        ipvs;                 /*  5192     8 */
	struct netns_mpls          mpls;                 /*  5200    32 */
	struct netns_can           can;                  /*  5232   200 */
	/* --- cacheline 84 boundary (5376 bytes) was 56 bytes ago --- */
	struct netns_xdp           xdp;                  /*  5432    40 */
	/* --- cacheline 85 boundary (5440 bytes) was 32 bytes ago --- */
	struct sock *              crypto_nlsk;          /*  5472     8 */
	struct sock *              diag_nlsk;            /*  5480     8 */
	u64                        kabi_reserved1;       /*  5488     8 */
	u64                        kabi_reserved2;       /*  5496     8 */
	/* --- cacheline 86 boundary (5504 bytes) --- */
	u64                        kabi_reserved3;       /*  5504     8 */
	u64                        kabi_reserved4;       /*  5512     8 */

	/* size: 5568, cachelines: 87, members: 61 */
	/* sum members: 5468, holes: 2, sum holes: 52 */
	/* padding: 48 */
	/* paddings: 3, sum paddings: 20 */
	/* forced alignments: 4, forced holes: 1, sum forced holes: 48 */
} __attribute__((__aligned__(64)));
struct bpf_insn {
	__u8                       code;                 /*     0     1 */
	__u8                       dst_reg:4;            /*     1: 0  1 */
	__u8                       src_reg:4;            /*     1: 4  1 */
	__s16                      off;                  /*     2     2 */
	__s32                      imm;                  /*     4     4 */

	/* size: 8, cachelines: 1, members: 5 */
	/* last cacheline: 8 bytes */
};
union bpf_attr {
	struct {
		__u32              map_type;           /*     0     4 */
		__u32              key_size;           /*     4     4 */
		__u32              value_size;         /*     8     4 */
		__u32              max_entries;        /*    12     4 */
		__u32              map_flags;          /*    16     4 */
		__u32              inner_map_fd;       /*    20     4 */
		__u32              numa_node;          /*    24     4 */
		char               map_name[16];       /*    28    16 */
		__u32              map_ifindex;        /*    44     4 */
		__u32              btf_fd;             /*    48     4 */
		__u32              btf_key_type_id;    /*    52     4 */
		__u32              btf_value_type_id;  /*    56     4 */
		__u32              btf_vmlinux_value_type_id; /*    60     4 */
	};                                             /*     0    64 */
	struct {
		__u32              map_fd;             /*     0     4 */

		/* XXX 4 bytes hole, try to pack */

		__u64              key __attribute__((__aligned__(8))); /*     8     8 */
		union {
			__u64      value __attribute__((__aligned__(8))); /*    16     8 */
			__u64      next_key __attribute__((__aligned__(8))); /*    16     8 */
		} __attribute__((__aligned__(8)));     /*    16     8 */
		__u64              flags;              /*    24     8 */
	} __attribute__((__aligned__(8))) __attribute__((__aligned__(8)));             /*     0    32 */
	struct {
		__u64              in_batch __attribute__((__aligned__(8))); /*     0     8 */
		__u64              out_batch __attribute__((__aligned__(8))); /*     8     8 */
		__u64              keys __attribute__((__aligned__(8))); /*    16     8 */
		__u64              values __attribute__((__aligned__(8))); /*    24     8 */
		__u32              count;              /*    32     4 */
		__u32              map_fd;             /*    36     4 */
		__u64              elem_flags;         /*    40     8 */
		__u64              flags;              /*    48     8 */
	} __attribute__((__aligned__(8))) batch __attribute__((__aligned__(8)));       /*     0    56 */
	struct {
		__u32              prog_type;          /*     0     4 */
		__u32              insn_cnt;           /*     4     4 */
		__u64              insns __attribute__((__aligned__(8))); /*     8     8 */
		__u64              license __attribute__((__aligned__(8))); /*    16     8 */
		__u32              log_level;          /*    24     4 */
		__u32              log_size;           /*    28     4 */
		__u64              log_buf __attribute__((__aligned__(8))); /*    32     8 */
		__u32              kern_version;       /*    40     4 */
		__u32              prog_flags;         /*    44     4 */
		char               prog_name[16];      /*    48    16 */
		/* --- cacheline 1 boundary (64 bytes) --- */
		__u32              prog_ifindex;       /*    64     4 */
		__u32              expected_attach_type; /*    68     4 */
		__u32              prog_btf_fd;        /*    72     4 */
		__u32              func_info_rec_size; /*    76     4 */
		__u64              func_info __attribute__((__aligned__(8))); /*    80     8 */
		__u32              func_info_cnt;      /*    88     4 */
		__u32              line_info_rec_size; /*    92     4 */
		__u64              line_info __attribute__((__aligned__(8))); /*    96     8 */
		__u32              line_info_cnt;      /*   104     4 */
		__u32              attach_btf_id;      /*   108     4 */
		__u32              attach_prog_fd;     /*   112     4 */
	} __attribute__((__aligned__(8))) __attribute__((__aligned__(8)));             /*     0   120 */
	struct {
		__u64              pathname __attribute__((__aligned__(8))); /*     0     8 */
		__u32              bpf_fd;             /*     8     4 */
		__u32              file_flags;         /*    12     4 */
	} __attribute__((__aligned__(8))) __attribute__((__aligned__(8)));             /*     0    16 */
	struct {
		__u32              target_fd;          /*     0     4 */
		__u32              attach_bpf_fd;      /*     4     4 */
		__u32              attach_type;        /*     8     4 */
		__u32              attach_flags;       /*    12     4 */
		__u32              replace_bpf_fd;     /*    16     4 */
	};                                             /*     0    20 */
	struct {
		__u32              prog_fd;            /*     0     4 */
		__u32              retval;             /*     4     4 */
		__u32              data_size_in;       /*     8     4 */
		__u32              data_size_out;      /*    12     4 */
		__u64              data_in __attribute__((__aligned__(8))); /*    16     8 */
		__u64              data_out __attribute__((__aligned__(8))); /*    24     8 */
		__u32              repeat;             /*    32     4 */
		__u32              duration;           /*    36     4 */
		__u32              ctx_size_in;        /*    40     4 */
		__u32              ctx_size_out;       /*    44     4 */
		__u64              ctx_in __attribute__((__aligned__(8))); /*    48     8 */
		__u64              ctx_out __attribute__((__aligned__(8))); /*    56     8 */
		/* --- cacheline 1 boundary (64 bytes) --- */
		__u32              flags;              /*    64     4 */
		__u32              cpu;                /*    68     4 */
	} __attribute__((__aligned__(8))) test __attribute__((__aligned__(8)));        /*     0    72 */
	struct {
		union {
			__u32      start_id;           /*     0     4 */
			__u32      prog_id;            /*     0     4 */
			__u32      map_id;             /*     0     4 */
			__u32      btf_id;             /*     0     4 */
			__u32      link_id;            /*     0     4 */
		};                                     /*     0     4 */
		__u32              next_id;            /*     4     4 */
		__u32              open_flags;         /*     8     4 */
	};                                             /*     0    12 */
	struct {
		__u32              bpf_fd;             /*     0     4 */
		__u32              info_len;           /*     4     4 */
		__u64              info __attribute__((__aligned__(8))); /*     8     8 */
	} __attribute__((__aligned__(8))) info __attribute__((__aligned__(8)));        /*     0    16 */
	struct {
		__u32              target_fd;          /*     0     4 */
		__u32              attach_type;        /*     4     4 */
		__u32              query_flags;        /*     8     4 */
		__u32              attach_flags;       /*    12     4 */
		__u64              prog_ids __attribute__((__aligned__(8))); /*    16     8 */
		__u32              prog_cnt;           /*    24     4 */
	} __attribute__((__aligned__(8))) query __attribute__((__aligned__(8)));       /*     0    32 */
	struct {
		__u64              name;               /*     0     8 */
		__u32              prog_fd;            /*     8     4 */
	} raw_tracepoint;                              /*     0    16 */
	struct {
		__u64              btf __attribute__((__aligned__(8))); /*     0     8 */
		__u64              btf_log_buf __attribute__((__aligned__(8))); /*     8     8 */
		__u32              btf_size;           /*    16     4 */
		__u32              btf_log_size;       /*    20     4 */
		__u32              btf_log_level;      /*    24     4 */
	} __attribute__((__aligned__(8))) __attribute__((__aligned__(8)));             /*     0    32 */
	struct {
		__u32              pid;                /*     0     4 */
		__u32              fd;                 /*     4     4 */
		__u32              flags;              /*     8     4 */
		__u32              buf_len;            /*    12     4 */
		__u64              buf __attribute__((__aligned__(8))); /*    16     8 */
		__u32              prog_id;            /*    24     4 */
		__u32              fd_type;            /*    28     4 */
		__u64              probe_offset;       /*    32     8 */
		__u64              probe_addr;         /*    40     8 */
	} __attribute__((__aligned__(8))) task_fd_query __attribute__((__aligned__(8))); /*     0    48 */
	struct {
		__u32              prog_fd;            /*     0     4 */
		union {
			__u32      target_fd;          /*     4     4 */
			__u32      target_ifindex;     /*     4     4 */
		};                                     /*     4     4 */
		__u32              attach_type;        /*     8     4 */
		__u32              flags;              /*    12     4 */
		union {
			__u32      target_btf_id;      /*    16     4 */
			struct {
				__u64 iter_info __attribute__((__aligned__(8))); /*    16     8 */
				__u32 iter_info_len;   /*    24     4 */
			} __attribute__((__aligned__(8))) __attribute__((__aligned__(8)));   /*    16    16 */
		} __attribute__((__aligned__(8)));     /*    16    16 */
	} __attribute__((__aligned__(8))) link_create __attribute__((__aligned__(8))); /*     0    32 */
	struct {
		__u32              link_fd;            /*     0     4 */
		__u32              new_prog_fd;        /*     4     4 */
		__u32              flags;              /*     8     4 */
		__u32              old_prog_fd;        /*    12     4 */
	} link_update;                                 /*     0    16 */
	struct {
		__u32              link_fd;            /*     0     4 */
	} link_detach;                                 /*     0     4 */
	struct {
		__u32              type;               /*     0     4 */
	} enable_stats;                                /*     0     4 */
	struct {
		__u32              link_fd;            /*     0     4 */
		__u32              flags;              /*     4     4 */
	} iter_create;                                 /*     0     8 */
	struct {
		__u32              prog_fd;            /*     0     4 */
		__u32              map_fd;             /*     4     4 */
		__u32              flags;              /*     8     4 */
	} prog_bind_map;                               /*     0    12 */
};
struct bpf_func_info {
	__u32                      insn_off;             /*     0     4 */
	__u32                      type_id;              /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct bpf_line_info {
	__u32                      insn_off;             /*     0     4 */
	__u32                      file_name_off;        /*     4     4 */
	__u32                      line_off;             /*     8     4 */
	__u32                      line_col;             /*    12     4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct bpf_iter_aux_info {
	struct bpf_map *           map;                  /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct bpf_iter_seq_info {
	const struct seq_operations  * seq_ops;          /*     0     8 */
	bpf_iter_init_seq_priv_t   init_seq_private;     /*     8     8 */
	bpf_iter_fini_seq_priv_t   fini_seq_private;     /*    16     8 */
	u32                        seq_priv_size;        /*    24     4 */

	/* size: 32, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct bpf_map_ops {
	int                        (*map_alloc_check)(union bpf_attr *); /*     0     8 */
	struct bpf_map *           (*map_alloc)(union bpf_attr *); /*     8     8 */
	void                       (*map_release)(struct bpf_map *, struct file *); /*    16     8 */
	void                       (*map_free)(struct bpf_map *); /*    24     8 */
	int                        (*map_get_next_key)(struct bpf_map *, void *, void *); /*    32     8 */
	void                       (*map_release_uref)(struct bpf_map *); /*    40     8 */
	void *                     (*map_lookup_elem_sys_only)(struct bpf_map *, void *); /*    48     8 */
	int                        (*map_lookup_batch)(struct bpf_map *, const union bpf_attr  *, union bpf_attr *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*map_lookup_and_delete_batch)(struct bpf_map *, const union bpf_attr  *, union bpf_attr *); /*    64     8 */
	int                        (*map_update_batch)(struct bpf_map *, const union bpf_attr  *, union bpf_attr *); /*    72     8 */
	int                        (*map_delete_batch)(struct bpf_map *, const union bpf_attr  *, union bpf_attr *); /*    80     8 */
	void *                     (*map_lookup_elem)(struct bpf_map *, void *); /*    88     8 */
	int                        (*map_update_elem)(struct bpf_map *, void *, void *, u64); /*    96     8 */
	int                        (*map_delete_elem)(struct bpf_map *, void *); /*   104     8 */
	int                        (*map_push_elem)(struct bpf_map *, void *, u64); /*   112     8 */
	int                        (*map_pop_elem)(struct bpf_map *, void *); /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	int                        (*map_peek_elem)(struct bpf_map *, void *); /*   128     8 */
	void *                     (*map_fd_get_ptr)(struct bpf_map *, struct file *, int); /*   136     8 */
	void                       (*map_fd_put_ptr)(void *); /*   144     8 */
	int                        (*map_gen_lookup)(struct bpf_map *, struct bpf_insn *); /*   152     8 */
	u32                        (*map_fd_sys_lookup_elem)(void *); /*   160     8 */
	void                       (*map_seq_show_elem)(struct bpf_map *, void *, struct seq_file *); /*   168     8 */
	int                        (*map_check_btf)(const struct bpf_map  *, const struct btf  *, const struct btf_type  *, const struct btf_type  *); /*   176     8 */
	int                        (*map_poke_track)(struct bpf_map *, struct bpf_prog_aux *); /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	void                       (*map_poke_untrack)(struct bpf_map *, struct bpf_prog_aux *); /*   192     8 */
	void                       (*map_poke_run)(struct bpf_map *, u32, struct bpf_prog *, struct bpf_prog *); /*   200     8 */
	int                        (*map_direct_value_addr)(const struct bpf_map  *, u64 *, u32); /*   208     8 */
	int                        (*map_direct_value_meta)(const struct bpf_map  *, u64, u32 *); /*   216     8 */
	int                        (*map_mmap)(struct bpf_map *, struct vm_area_struct *); /*   224     8 */
	__poll_t                   (*map_poll)(struct bpf_map *, struct file *, struct poll_table_struct *); /*   232     8 */
	int                        (*map_local_storage_charge)(struct bpf_local_storage_map *, void *, u32); /*   240     8 */
	void                       (*map_local_storage_uncharge)(struct bpf_local_storage_map *, void *, u32); /*   248     8 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	struct bpf_local_storage * * (*map_owner_storage_ptr)(void *); /*   256     8 */
	bool                       (*map_meta_equal)(const struct bpf_map  *, const struct bpf_map  *); /*   264     8 */
	const const char  *        map_btf_name;         /*   272     8 */
	int *                      map_btf_id;           /*   280     8 */
	const struct bpf_iter_seq_info  * iter_seq_info; /*   288     8 */

	/* size: 296, cachelines: 5, members: 37 */
	/* last cacheline: 40 bytes */
};
struct bpf_map {
	const struct bpf_map_ops  * ops __attribute__((__aligned__(64))); /*     0     8 */
	struct bpf_map *           inner_map_meta;       /*     8     8 */
	void *                     security;             /*    16     8 */
	enum bpf_map_type          map_type;             /*    24     4 */
	u32                        key_size;             /*    28     4 */
	u32                        value_size;           /*    32     4 */
	u32                        max_entries;          /*    36     4 */
	u32                        map_flags;            /*    40     4 */
	int                        spin_lock_off;        /*    44     4 */
	u32                        id;                   /*    48     4 */
	int                        numa_node;            /*    52     4 */
	u32                        btf_key_type_id;      /*    56     4 */
	u32                        btf_value_type_id;    /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct btf *               btf;                  /*    64     8 */
	struct bpf_map_memory      memory;               /*    72    16 */
	char                       name[16];             /*    88    16 */
	u32                        btf_vmlinux_value_type_id; /*   104     4 */
	bool                       bypass_spec_v1;       /*   108     1 */
	bool                       frozen;               /*   109     1 */

	/* XXX 18 bytes hole, try to pack */

	/* --- cacheline 2 boundary (128 bytes) --- */
	atomic64_t                 refcnt __attribute__((__aligned__(64))); /*   128     8 */
	atomic64_t                 usercnt;              /*   136     8 */
	struct work_struct         work;                 /*   144    64 */
	/* --- cacheline 3 boundary (192 bytes) was 16 bytes ago --- */
	struct mutex               freeze_mutex;         /*   208    32 */
	atomic64_t                 writecnt;             /*   240     8 */

	/* size: 256, cachelines: 4, members: 24 */
	/* sum members: 230, holes: 1, sum holes: 18 */
	/* padding: 8 */
	/* forced alignments: 2, forced holes: 1, sum forced holes: 18 */
} __attribute__((__aligned__(64)));
struct bpf_prog_aux {
	atomic64_t                 refcnt;               /*     0     8 */
	u32                        used_map_cnt;         /*     8     4 */
	u32                        max_ctx_offset;       /*    12     4 */
	u32                        max_pkt_offset;       /*    16     4 */
	u32                        max_tp_access;        /*    20     4 */
	u32                        stack_depth;          /*    24     4 */
	u32                        id;                   /*    28     4 */
	u32                        func_cnt;             /*    32     4 */
	u32                        func_idx;             /*    36     4 */
	u32                        attach_btf_id;        /*    40     4 */
	u32                        ctx_arg_info_size;    /*    44     4 */
	u32                        max_rdonly_access;    /*    48     4 */
	u32                        max_rdwr_access;      /*    52     4 */
	const struct bpf_ctx_arg_aux  * ctx_arg_info;    /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct mutex               dst_mutex;            /*    64    32 */
	struct bpf_prog *          dst_prog;             /*    96     8 */
	struct bpf_trampoline *    dst_trampoline;       /*   104     8 */
	enum bpf_prog_type         saved_dst_prog_type;  /*   112     4 */
	enum bpf_attach_type       saved_dst_attach_type; /*   116     4 */
	bool                       verifier_zext;        /*   120     1 */
	bool                       offload_requested;    /*   121     1 */
	bool                       attach_btf_trace;     /*   122     1 */
	bool                       func_proto_unreliable; /*   123     1 */
	bool                       sleepable;            /*   124     1 */
	bool                       tail_call_reachable;  /*   125     1 */

	/* XXX 2 bytes hole, try to pack */

	/* --- cacheline 2 boundary (128 bytes) --- */
	struct hlist_node          tramp_hlist;          /*   128    16 */
	const struct btf_type  *   attach_func_proto;    /*   144     8 */
	const char  *              attach_func_name;     /*   152     8 */
	struct bpf_prog * *        func;                 /*   160     8 */
	void *                     jit_data;             /*   168     8 */
	struct bpf_jit_poke_descriptor * poke_tab;       /*   176     8 */
	u32                        size_poke_tab;        /*   184     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 3 boundary (192 bytes) --- */
	struct bpf_ksym            ksym __attribute__((__aligned__(8))); /*   192   216 */

	/* XXX last struct has 7 bytes of padding */

	/* --- cacheline 6 boundary (384 bytes) was 24 bytes ago --- */
	const struct bpf_prog_ops  * ops;                /*   408     8 */
	struct bpf_map * *         used_maps;            /*   416     8 */
	struct mutex               used_maps_mutex;      /*   424    32 */
	/* --- cacheline 7 boundary (448 bytes) was 8 bytes ago --- */
	struct bpf_prog *          prog;                 /*   456     8 */
	struct user_struct *       user;                 /*   464     8 */
	u64                        load_time;            /*   472     8 */
	struct bpf_map *           cgroup_storage[10];   /*   480    80 */
	/* --- cacheline 8 boundary (512 bytes) was 48 bytes ago --- */
	char                       name[16];             /*   560    16 */
	/* --- cacheline 9 boundary (576 bytes) --- */
	void *                     security;             /*   576     8 */
	struct bpf_prog_offload *  offload;              /*   584     8 */
	struct btf *               btf;                  /*   592     8 */
	struct bpf_func_info *     func_info;            /*   600     8 */
	struct bpf_func_info_aux * func_info_aux;        /*   608     8 */
	struct bpf_line_info *     linfo;                /*   616     8 */
	void * *                   jited_linfo;          /*   624     8 */
	u32                        func_info_cnt;        /*   632     4 */
	u32                        nr_linfo;             /*   636     4 */
	/* --- cacheline 10 boundary (640 bytes) --- */
	u32                        linfo_idx;            /*   640     4 */
	u32                        num_exentries;        /*   644     4 */
	struct exception_table_entry * extable;          /*   648     8 */
	struct bpf_prog_stats *    stats;                /*   656     8 */
	union {
		struct work_struct work;                 /*   664    64 */
		struct callback_head rcu __attribute__((__aligned__(8))); /*   664    16 */
	} __attribute__((__aligned__(8)));               /*   664    64 */

	/* size: 728, cachelines: 12, members: 55 */
	/* sum members: 722, holes: 2, sum holes: 6 */
	/* paddings: 1, sum paddings: 7 */
	/* forced alignments: 2, forced holes: 1, sum forced holes: 4 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct bpf_prog {
	u16                        pages;                /*     0     2 */
	u16                        jited:1;              /*     2: 0  2 */
	u16                        jit_requested:1;      /*     2: 1  2 */
	u16                        gpl_compatible:1;     /*     2: 2  2 */
	u16                        cb_access:1;          /*     2: 3  2 */
	u16                        dst_needed:1;         /*     2: 4  2 */
	u16                        blinded:1;            /*     2: 5  2 */
	u16                        is_func:1;            /*     2: 6  2 */
	u16                        kprobe_override:1;    /*     2: 7  2 */
	u16                        has_callchain_buf:1;  /*     2: 8  2 */
	u16                        enforce_expected_attach_type:1; /*     2: 9  2 */
	u16                        call_get_stack:1;     /*     2:10  2 */

	/* XXX 5 bits hole, try to pack */

	enum bpf_prog_type         type;                 /*     4     4 */
	enum bpf_attach_type       expected_attach_type; /*     8     4 */
	u32                        len;                  /*    12     4 */
	u32                        jited_len;            /*    16     4 */
	u8                         tag[8];               /*    20     8 */

	/* XXX 4 bytes hole, try to pack */

	struct bpf_prog_aux *      aux;                  /*    32     8 */
	struct sock_fprog_kern *   orig_prog;            /*    40     8 */
	unsigned int               (*bpf_func)(const void  *, const struct bpf_insn  *); /*    48     8 */
	struct sock_filter         insns[0];             /*    56     0 */
	struct bpf_insn            insnsi[];             /*    56     0 */

	/* size: 56, cachelines: 1, members: 22 */
	/* sum members: 50, holes: 1, sum holes: 4 */
	/* sum bitfield members: 11 bits, bit holes: 1, sum bit holes: 5 bits */
	/* last cacheline: 56 bytes */
};
struct bpf_map_memory {
	u32                        pages;                /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct user_struct *       user;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct bpf_map_dev_ops {
	int                        (*map_get_next_key)(struct bpf_offloaded_map *, void *, void *); /*     0     8 */
	int                        (*map_lookup_elem)(struct bpf_offloaded_map *, void *, void *); /*     8     8 */
	int                        (*map_update_elem)(struct bpf_offloaded_map *, void *, void *, u64); /*    16     8 */
	int                        (*map_delete_elem)(struct bpf_offloaded_map *, void *); /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct bpf_offloaded_map {
	struct bpf_map             map __attribute__((__aligned__(64))); /*     0   256 */

	/* XXX last struct has 8 bytes of padding */

	/* --- cacheline 4 boundary (256 bytes) --- */
	struct net_device *        netdev;               /*   256     8 */
	const struct bpf_map_dev_ops  * dev_ops;         /*   264     8 */
	void *                     dev_priv;             /*   272     8 */
	struct list_head           offloads;             /*   280    16 */

	/* size: 320, cachelines: 5, members: 5 */
	/* padding: 24 */
	/* paddings: 1, sum paddings: 8 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(64)));
struct net_device {
	char                       name[16];             /*     0    16 */
	struct netdev_name_node *  name_node;            /*    16     8 */
	struct dev_ifalias *       ifalias;              /*    24     8 */
	long unsigned int          mem_end;              /*    32     8 */
	long unsigned int          mem_start;            /*    40     8 */
	long unsigned int          base_addr;            /*    48     8 */
	int                        irq;                  /*    56     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	long unsigned int          state;                /*    64     8 */
	struct list_head           dev_list;             /*    72    16 */
	struct list_head           napi_list;            /*    88    16 */
	struct list_head           unreg_list;           /*   104    16 */
	struct list_head           close_list;           /*   120    16 */
	/* --- cacheline 2 boundary (128 bytes) was 8 bytes ago --- */
	struct list_head           ptype_all;            /*   136    16 */
	struct list_head           ptype_specific;       /*   152    16 */
	struct {
		struct list_head   upper;                /*   168    16 */
		struct list_head   lower;                /*   184    16 */
	} adj_list;                                      /*   168    32 */
	/* --- cacheline 3 boundary (192 bytes) was 8 bytes ago --- */
	netdev_features_t          features;             /*   200     8 */
	netdev_features_t          hw_features;          /*   208     8 */
	netdev_features_t          wanted_features;      /*   216     8 */
	netdev_features_t          vlan_features;        /*   224     8 */
	netdev_features_t          hw_enc_features;      /*   232     8 */
	netdev_features_t          mpls_features;        /*   240     8 */
	netdev_features_t          gso_partial_features; /*   248     8 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	int                        ifindex;              /*   256     4 */
	int                        group;                /*   260     4 */
	struct net_device_stats    stats;                /*   264   184 */
	/* --- cacheline 7 boundary (448 bytes) --- */
	atomic_long_t              rx_dropped;           /*   448     8 */
	atomic_long_t              tx_dropped;           /*   456     8 */
	atomic_long_t              rx_nohandler;         /*   464     8 */
	atomic_t                   carrier_up_count;     /*   472     4 */
	atomic_t                   carrier_down_count;   /*   476     4 */
	const struct net_device_ops  * netdev_ops;       /*   480     8 */
	const struct ethtool_ops  * ethtool_ops;         /*   488     8 */
	const struct l3mdev_ops  * l3mdev_ops;           /*   496     8 */
	const struct ndisc_ops  *  ndisc_ops;            /*   504     8 */
	/* --- cacheline 8 boundary (512 bytes) --- */
	const struct xfrmdev_ops  * xfrmdev_ops;         /*   512     8 */
	const struct tlsdev_ops  * tlsdev_ops;           /*   520     8 */
	const struct header_ops  * header_ops;           /*   528     8 */
	unsigned int               flags;                /*   536     4 */
	unsigned int               priv_flags;           /*   540     4 */
	short unsigned int         gflags;               /*   544     2 */
	short unsigned int         padded;               /*   546     2 */
	unsigned char              operstate;            /*   548     1 */
	unsigned char              link_mode;            /*   549     1 */
	unsigned char              if_port;              /*   550     1 */
	unsigned char              dma;                  /*   551     1 */
	unsigned int               mtu;                  /*   552     4 */
	unsigned int               min_mtu;              /*   556     4 */
	unsigned int               max_mtu;              /*   560     4 */
	short unsigned int         type;                 /*   564     2 */
	short unsigned int         hard_header_len;      /*   566     2 */
	unsigned char              min_header_len;       /*   568     1 */
	unsigned char              name_assign_type;     /*   569     1 */
	short unsigned int         needed_headroom;      /*   570     2 */
	short unsigned int         needed_tailroom;      /*   572     2 */
	unsigned char              perm_addr[32];        /*   574    32 */
	/* --- cacheline 9 boundary (576 bytes) was 30 bytes ago --- */
	unsigned char              addr_assign_type;     /*   606     1 */
	unsigned char              addr_len;             /*   607     1 */
	unsigned char              upper_level;          /*   608     1 */
	unsigned char              lower_level;          /*   609     1 */
	short unsigned int         neigh_priv_len;       /*   610     2 */
	short unsigned int         dev_id;               /*   612     2 */
	short unsigned int         dev_port;             /*   614     2 */
	spinlock_t                 addr_list_lock;       /*   616     4 */

	/* XXX 4 bytes hole, try to pack */

	struct netdev_hw_addr_list uc;                   /*   624    24 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 10 boundary (640 bytes) was 8 bytes ago --- */
	struct netdev_hw_addr_list mc;                   /*   648    24 */

	/* XXX last struct has 4 bytes of padding */

	struct netdev_hw_addr_list dev_addrs;            /*   672    24 */

	/* XXX last struct has 4 bytes of padding */

	struct kset *              queues_kset;          /*   696     8 */
	/* --- cacheline 11 boundary (704 bytes) --- */
	unsigned int               promiscuity;          /*   704     4 */
	unsigned int               allmulti;             /*   708     4 */
	bool                       uc_promisc;           /*   712     1 */

	/* XXX 7 bytes hole, try to pack */

	struct vlan_info *         vlan_info;            /*   720     8 */
	struct tipc_bearer *       tipc_ptr;             /*   728     8 */
	struct in_device *         ip_ptr;               /*   736     8 */
	struct inet6_dev *         ip6_ptr;              /*   744     8 */
	struct wireless_dev *      ieee80211_ptr;        /*   752     8 */
	struct wpan_dev *          ieee802154_ptr;       /*   760     8 */
	/* --- cacheline 12 boundary (768 bytes) --- */
	struct mpls_dev *          mpls_ptr;             /*   768     8 */
	unsigned char *            dev_addr;             /*   776     8 */
	struct netdev_rx_queue *   _rx;                  /*   784     8 */
	unsigned int               num_rx_queues;        /*   792     4 */
	unsigned int               real_num_rx_queues;   /*   796     4 */
	struct bpf_prog *          xdp_prog;             /*   800     8 */
	long unsigned int          gro_flush_timeout;    /*   808     8 */
	int                        napi_defer_hard_irqs; /*   816     4 */

	/* XXX 4 bytes hole, try to pack */

	rx_handler_func_t *        rx_handler;           /*   824     8 */
	/* --- cacheline 13 boundary (832 bytes) --- */
	void *                     rx_handler_data;      /*   832     8 */
	struct mini_Qdisc *        miniq_ingress;        /*   840     8 */
	struct netdev_queue *      ingress_queue;        /*   848     8 */
	struct nf_hook_entries *   nf_hooks_ingress;     /*   856     8 */
	unsigned char              broadcast[32];        /*   864    32 */
	/* --- cacheline 14 boundary (896 bytes) --- */
	struct cpu_rmap *          rx_cpu_rmap;          /*   896     8 */
	struct hlist_node          index_hlist;          /*   904    16 */

	/* XXX 40 bytes hole, try to pack */

	/* --- cacheline 15 boundary (960 bytes) --- */
	struct netdev_queue *      _tx __attribute__((__aligned__(64))); /*   960     8 */
	unsigned int               num_tx_queues;        /*   968     4 */
	unsigned int               real_num_tx_queues;   /*   972     4 */
	struct Qdisc *             qdisc;                /*   976     8 */
	unsigned int               tx_queue_len;         /*   984     4 */
	spinlock_t                 tx_global_lock;       /*   988     4 */
	struct xdp_dev_bulk_queue * xdp_bulkq;           /*   992     8 */
	struct xps_dev_maps *      xps_cpus_map;         /*  1000     8 */
	struct xps_dev_maps *      xps_rxqs_map;         /*  1008     8 */
	struct mini_Qdisc *        miniq_egress;         /*  1016     8 */
	/* --- cacheline 16 boundary (1024 bytes) --- */
	struct hlist_head          qdisc_hash[16];       /*  1024   128 */
	/* --- cacheline 18 boundary (1152 bytes) --- */
	struct timer_list          watchdog_timer;       /*  1152    72 */
	/* --- cacheline 19 boundary (1216 bytes) was 8 bytes ago --- */
	int                        watchdog_timeo;       /*  1224     4 */
	u32                        proto_down_reason;    /*  1228     4 */
	struct list_head           todo_list;            /*  1232    16 */
	int *                      pcpu_refcnt;          /*  1248     8 */
	struct list_head           link_watch_list;      /*  1256    16 */
	enum {
		NETREG_UNINITIALIZED = 0,
		NETREG_REGISTERED    = 1,
		NETREG_UNREGISTERING = 2,
		NETREG_UNREGISTERED  = 3,
		NETREG_RELEASED      = 4,
		NETREG_DUMMY         = 5,
	} reg_state:8;                                     /*  1272: 0  4 */

	/* Bitfield combined with next fields */

	bool                       dismantle;            /*  1273     1 */

	/* Bitfield combined with previous fields */

	enum {
		RTNL_LINK_INITIALIZED  = 0,
		RTNL_LINK_INITIALIZING = 1,
	} rtnl_link_state:16;                               /*  1272:16  4 */
	bool                       needs_free_netdev;    /*  1276     1 */

	/* XXX 3 bytes hole, try to pack */

	/* --- cacheline 20 boundary (1280 bytes) --- */
	void                       (*priv_destructor)(struct net_device *); /*  1280     8 */
	struct netpoll_info *      npinfo;               /*  1288     8 */
	possible_net_t             nd_net;               /*  1296     8 */
	void *                     ml_priv;              /*  1304     8 */
	enum netdev_ml_priv_type   ml_priv_type;         /*  1312     4 */

	/* XXX 4 bytes hole, try to pack */

	union {
		struct pcpu_lstats * lstats;             /*  1320     8 */
		struct pcpu_sw_netstats * tstats;        /*  1320     8 */
		struct pcpu_dstats * dstats;             /*  1320     8 */
	};                                               /*  1320     8 */
	struct garp_port *         garp_port;            /*  1328     8 */
	struct mrp_port *          mrp_port;             /*  1336     8 */
	/* --- cacheline 21 boundary (1344 bytes) --- */
	struct device              dev __attribute__((__aligned__(8))); /*  1344  1000 */
	/* --- cacheline 36 boundary (2304 bytes) was 40 bytes ago --- */
	const struct attribute_group  * sysfs_groups[4]; /*  2344    32 */
	/* --- cacheline 37 boundary (2368 bytes) was 8 bytes ago --- */
	const struct attribute_group  * sysfs_rx_queue_group; /*  2376     8 */
	const struct rtnl_link_ops  * rtnl_link_ops;     /*  2384     8 */
	unsigned int               gso_max_size;         /*  2392     4 */
	u16                        gso_max_segs;         /*  2396     2 */

	/* XXX 2 bytes hole, try to pack */

	const struct dcbnl_rtnl_ops  * dcbnl_ops;        /*  2400     8 */
	s16                        num_tc;               /*  2408     2 */
	struct netdev_tc_txq       tc_to_txq[16];        /*  2410    64 */
	/* --- cacheline 38 boundary (2432 bytes) was 42 bytes ago --- */
	u8                         prio_tc_map[16];      /*  2474    16 */

	/* XXX 2 bytes hole, try to pack */

	unsigned int               fcoe_ddp_xid;         /*  2492     4 */
	/* --- cacheline 39 boundary (2496 bytes) --- */
	struct netprio_map *       priomap;              /*  2496     8 */
	struct phy_device *        phydev;               /*  2504     8 */
	struct sfp_bus *           sfp_bus;              /*  2512     8 */
	struct lock_class_key *    qdisc_tx_busylock;    /*  2520     8 */
	struct lock_class_key *    qdisc_running_key;    /*  2528     8 */
	bool                       proto_down;           /*  2536     1 */

	/* Bitfield combined with previous fields */

	unsigned int               wol_enabled:1;        /*  2536: 8  4 */

	/* XXX 23 bits hole, try to pack */
	/* XXX 4 bytes hole, try to pack */

	struct list_head           net_notifier_list;    /*  2544    16 */
	/* --- cacheline 40 boundary (2560 bytes) --- */
	const struct macsec_ops  * macsec_ops;           /*  2560     8 */
	const struct udp_tunnel_nic_info  * udp_tunnel_nic_info; /*  2568     8 */
	struct udp_tunnel_nic *    udp_tunnel_nic;       /*  2576     8 */
	struct bpf_xdp_entity      xdp_state[3];         /*  2584    48 */
	/* --- cacheline 41 boundary (2624 bytes) was 8 bytes ago --- */
	u64                        kabi_reserved1;       /*  2632     8 */
	u64                        kabi_reserved2;       /*  2640     8 */
	u64                        kabi_reserved3;       /*  2648     8 */
	u64                        kabi_reserved4;       /*  2656     8 */
	u64                        kabi_reserved5;       /*  2664     8 */
	u64                        kabi_reserved6;       /*  2672     8 */
	u64                        kabi_reserved7;       /*  2680     8 */
	/* --- cacheline 42 boundary (2688 bytes) --- */
	u64                        kabi_reserved8;       /*  2688     8 */
	size_t                     net_device_extended_size_resvd; /*  2696     8 */
	struct net_device_extended_resvd * _resvd;       /*  2704     8 */

	/* size: 2752, cachelines: 43, members: 154 */
	/* sum members: 2632, holes: 10, sum holes: 74 */
	/* sum bitfield members: 25 bits, bit holes: 1, sum bit holes: 23 bits */
	/* padding: 40 */
	/* paddings: 3, sum paddings: 12 */
	/* forced alignments: 2, forced holes: 1, sum forced holes: 40 */
} __attribute__((__aligned__(64)));
struct bpf_prog_ops {
	int                        (*test_run)(struct bpf_prog *, const union bpf_attr  *, union bpf_attr *); /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct bpf_prog_offload {
	struct bpf_prog *          prog;                 /*     0     8 */
	struct net_device *        netdev;               /*     8     8 */
	struct bpf_offload_dev *   offdev;               /*    16     8 */
	void *                     dev_priv;             /*    24     8 */
	struct list_head           offloads;             /*    32    16 */
	bool                       dev_state;            /*    48     1 */
	bool                       opt_failed;           /*    49     1 */

	/* XXX 6 bytes hole, try to pack */

	void *                     jited_image;          /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u32                        jited_len;            /*    64     4 */

	/* size: 72, cachelines: 2, members: 9 */
	/* sum members: 62, holes: 1, sum holes: 6 */
	/* padding: 4 */
	/* last cacheline: 8 bytes */
};
struct bpf_prog_stats {
	u64                        cnt;                  /*     0     8 */
	u64                        nsecs;                /*     8     8 */
	struct u64_stats_sync      syncp;                /*    16     0 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(16)));
struct btf_func_model {
	u8                         ret_size;             /*     0     1 */
	u8                         nr_args;              /*     1     1 */
	u8                         arg_size[12];         /*     2    12 */

	/* size: 14, cachelines: 1, members: 3 */
	/* last cacheline: 14 bytes */
};
struct bpf_ksym {
	long unsigned int          start;                /*     0     8 */
	long unsigned int          end;                  /*     8     8 */
	char                       name[128];            /*    16   128 */
	/* --- cacheline 2 boundary (128 bytes) was 16 bytes ago --- */
	struct list_head           lnode;                /*   144    16 */
	struct latch_tree_node     tnode __attribute__((__aligned__(8))); /*   160    48 */
	/* --- cacheline 3 boundary (192 bytes) was 16 bytes ago --- */
	bool                       prog;                 /*   208     1 */

	/* size: 216, cachelines: 4, members: 6 */
	/* padding: 7 */
	/* forced alignments: 1 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct bpf_tramp_image {
	void *                     image;                /*     0     8 */
	struct bpf_ksym            ksym __attribute__((__aligned__(8))); /*     8   216 */

	/* XXX last struct has 7 bytes of padding */

	/* --- cacheline 3 boundary (192 bytes) was 32 bytes ago --- */
	struct percpu_ref          pcref;                /*   224    16 */
	void *                     ip_after_call;        /*   240     8 */
	void *                     ip_epilogue;          /*   248     8 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	union {
		struct callback_head rcu __attribute__((__aligned__(8))); /*   256    16 */
		struct work_struct work;                 /*   256    64 */
	} __attribute__((__aligned__(8)));               /*   256    64 */

	/* size: 320, cachelines: 5, members: 6 */
	/* paddings: 1, sum paddings: 7 */
	/* forced alignments: 2 */
} __attribute__((__aligned__(8)));
struct bpf_trampoline {
	struct hlist_node          hlist;                /*     0    16 */
	struct mutex               mutex;                /*    16    32 */
	refcount_t                 refcnt;               /*    48     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        key;                  /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct {
		struct btf_func_model model;             /*    64    14 */

		/* XXX 2 bytes hole, try to pack */

		void *             addr;                 /*    80     8 */
		bool               ftrace_managed;       /*    88     1 */
	} func;                                          /*    64    32 */

	/* XXX last struct has 7 bytes of padding */

	struct bpf_prog *          extension_prog;       /*    96     8 */
	struct hlist_head          progs_hlist[3];       /*   104    24 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	int                        progs_cnt[3];         /*   128    12 */

	/* XXX 4 bytes hole, try to pack */

	struct bpf_tramp_image *   cur_image;            /*   144     8 */
	u64                        selector;             /*   152     8 */

	/* size: 160, cachelines: 3, members: 10 */
	/* sum members: 152, holes: 2, sum holes: 8 */
	/* paddings: 1, sum paddings: 7 */
	/* last cacheline: 32 bytes */
};
struct bpf_func_info_aux {
	u16                        linkage;              /*     0     2 */
	bool                       unreliable;           /*     2     1 */

	/* size: 4, cachelines: 1, members: 2 */
	/* padding: 1 */
	/* last cacheline: 4 bytes */
};
struct bpf_jit_poke_descriptor {
	void *                     tailcall_target;      /*     0     8 */
	void *                     tailcall_bypass;      /*     8     8 */
	void *                     bypass_addr;          /*    16     8 */
	void *                     aux;                  /*    24     8 */
	union {
		struct {
			struct bpf_map * map;            /*    32     8 */
			u32        key;                  /*    40     4 */
		} tail_call;                             /*    32    16 */
	};                                               /*    32    16 */
	bool                       tailcall_target_stable; /*    48     1 */
	u8                         adj_off;              /*    49     1 */
	u16                        reason;               /*    50     2 */
	u32                        insn_idx;             /*    52     4 */

	/* size: 56, cachelines: 1, members: 9 */
	/* last cacheline: 56 bytes */
};
struct bpf_ctx_arg_aux {
	u32                        offset;               /*     0     4 */
	enum bpf_reg_type          reg_type;             /*     4     4 */
	u32                        btf_id;               /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct sk_buff {
	union {
		struct {
			struct sk_buff * next;           /*     0     8 */
			struct sk_buff * prev;           /*     8     8 */
			union {
				struct net_device * dev; /*    16     8 */
				long unsigned int dev_scratch; /*    16     8 */
			};                               /*    16     8 */
		};                                       /*     0    24 */
		struct rb_node     rbnode __attribute__((__aligned__(8))); /*     0    24 */
		struct list_head   list;                 /*     0    16 */
		struct llist_node  ll_node;              /*     0     8 */
	} __attribute__((__aligned__(8)));               /*     0    24 */
	union {
		struct sock *      sk;                   /*    24     8 */
		int                ip_defrag_offset;     /*    24     4 */
	};                                               /*    24     8 */
	union {
		ktime_t            tstamp;               /*    32     8 */
		u64                skb_mstamp_ns;        /*    32     8 */
	};                                               /*    32     8 */
	char                       cb[48] __attribute__((__aligned__(8))); /*    40    48 */
	/* --- cacheline 1 boundary (64 bytes) was 24 bytes ago --- */
	union {
		struct {
			long unsigned int _skb_refdst;   /*    88     8 */
			void       (*destructor)(struct sk_buff *); /*    96     8 */
		};                                       /*    88    16 */
		struct list_head   tcp_tsorted_anchor;   /*    88    16 */
	};                                               /*    88    16 */
	long unsigned int          _nfct;                /*   104     8 */
	unsigned int               len;                  /*   112     4 */
	unsigned int               data_len;             /*   116     4 */
	__u16                      mac_len;              /*   120     2 */
	__u16                      hdr_len;              /*   122     2 */
	__u16                      queue_mapping;        /*   124     2 */
	__u8                       __cloned_offset[0];   /*   126     0 */
	__u8                       cloned:1;             /*   126: 0  1 */
	__u8                       nohdr:1;              /*   126: 1  1 */
	__u8                       fclone:2;             /*   126: 2  1 */
	__u8                       peeked:1;             /*   126: 4  1 */
	__u8                       head_frag:1;          /*   126: 5  1 */
	__u8                       pfmemalloc:1;         /*   126: 6  1 */
	__u8                       pp_recycle:1;         /*   126: 7  1 */
	__u8                       active_extensions;    /*   127     1 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	__u32                      headers_start[0];     /*   128     0 */
	__u8                       __pkt_type_offset[0]; /*   128     0 */
	__u8                       pkt_type:3;           /*   128: 0  1 */
	__u8                       ignore_df:1;          /*   128: 3  1 */
	__u8                       nf_trace:1;           /*   128: 4  1 */
	__u8                       ip_summed:2;          /*   128: 5  1 */
	__u8                       ooo_okay:1;           /*   128: 7  1 */
	__u8                       l4_hash:1;            /*   129: 0  1 */
	__u8                       sw_hash:1;            /*   129: 1  1 */
	__u8                       wifi_acked_valid:1;   /*   129: 2  1 */
	__u8                       wifi_acked:1;         /*   129: 3  1 */
	__u8                       no_fcs:1;             /*   129: 4  1 */
	__u8                       encapsulation:1;      /*   129: 5  1 */
	__u8                       encap_hdr_csum:1;     /*   129: 6  1 */
	__u8                       csum_valid:1;         /*   129: 7  1 */
	__u8                       __pkt_vlan_present_offset[0]; /*   130     0 */
	__u8                       vlan_present:1;       /*   130: 0  1 */
	__u8                       csum_complete_sw:1;   /*   130: 1  1 */
	__u8                       csum_level:2;         /*   130: 2  1 */
	__u8                       csum_not_inet:1;      /*   130: 4  1 */
	__u8                       dst_pending_confirm:1; /*   130: 5  1 */
	__u8                       ndisc_nodetype:2;     /*   130: 6  1 */
	__u8                       ipvs_property:1;      /*   131: 0  1 */
	__u8                       inner_protocol_type:1; /*   131: 1  1 */
	__u8                       remcsum_offload:1;    /*   131: 2  1 */
	__u8                       offload_fwd_mark:1;   /*   131: 3  1 */
	__u8                       offload_l3_fwd_mark:1; /*   131: 4  1 */
	__u8                       tc_skip_classify:1;   /*   131: 5  1 */
	__u8                       tc_at_ingress:1;      /*   131: 6  1 */
	__u8                       redirected:1;         /*   131: 7  1 */
	__u8                       from_ingress:1;       /*   132: 0  1 */
	__u8                       decrypted:1;          /*   132: 1  1 */

	/* XXX 6 bits hole, try to pack */
	/* XXX 1 byte hole, try to pack */

	__u16                      tc_index;             /*   134     2 */
	union {
		__wsum             csum;                 /*   136     4 */
		struct {
			__u16      csum_start;           /*   136     2 */
			__u16      csum_offset;          /*   138     2 */
		};                                       /*   136     4 */
	};                                               /*   136     4 */
	__u32                      priority;             /*   140     4 */
	int                        skb_iif;              /*   144     4 */
	__u32                      hash;                 /*   148     4 */
	__be16                     vlan_proto;           /*   152     2 */
	__u16                      vlan_tci;             /*   154     2 */
	union {
		unsigned int       napi_id;              /*   156     4 */
		unsigned int       sender_cpu;           /*   156     4 */
	};                                               /*   156     4 */
	__u32                      secmark;              /*   160     4 */
	union {
		__u32              mark;                 /*   164     4 */
		__u32              reserved_tailroom;    /*   164     4 */
	};                                               /*   164     4 */
	union {
		__be16             inner_protocol;       /*   168     2 */
		__u8               inner_ipproto;        /*   168     1 */
	};                                               /*   168     2 */
	__u16                      inner_transport_header; /*   170     2 */
	__u16                      inner_network_header; /*   172     2 */
	__u16                      inner_mac_header;     /*   174     2 */
	__be16                     protocol;             /*   176     2 */
	__u16                      transport_header;     /*   178     2 */
	__u16                      network_header;       /*   180     2 */
	__u16                      mac_header;           /*   182     2 */
	__u32                      headers_end[0];       /*   184     0 */
	union {
		__u8               scm_io_uring:1;       /*   184: 0  1 */
		struct {
			u64        kabi_reserved1;       /*   184     8 */
		} kabi_hidden_920;                       /*   184     8 */
		union {
		};                                       /*   184     0 */
	};                                               /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	u64                        kabi_reserved2;       /*   192     8 */
	u64                        kabi_reserved3;       /*   200     8 */
	u64                        kabi_reserved4;       /*   208     8 */
	sk_buff_data_t             tail;                 /*   216     4 */
	sk_buff_data_t             end;                  /*   220     4 */
	unsigned char *            head;                 /*   224     8 */
	unsigned char *            data;                 /*   232     8 */
	unsigned int               truesize;             /*   240     4 */
	refcount_t                 users;                /*   244     4 */
	struct skb_ext *           extensions;           /*   248     8 */

	/* size: 256, cachelines: 4, members: 82 */
	/* sum members: 249, holes: 1, sum holes: 1 */
	/* sum bitfield members: 42 bits, bit holes: 1, sum bit holes: 6 bits */
	/* forced alignments: 2 */
} __attribute__((__aligned__(8)));
struct sock_cgroup_data {
	struct cgroup *            cgroup;               /*     0     8 */
	u32                        classid;              /*     8     4 */
	u16                        prioidx;              /*    12     2 */

	/* size: 16, cachelines: 1, members: 3 */
	/* padding: 2 */
	/* last cacheline: 16 bytes */
};
struct scatterlist {
	long unsigned int          page_link;            /*     0     8 */
	unsigned int               offset;               /*     8     4 */
	unsigned int               length;               /*    12     4 */
	dma_addr_t                 dma_address;          /*    16     8 */
	unsigned int               dma_length;           /*    24     4 */

	/* size: 32, cachelines: 1, members: 5 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct in6_addr {
	union {
		__u8               u6_addr8[16];         /*     0    16 */
		__be16             u6_addr16[8];         /*     0    16 */
		__be32             u6_addr32[4];         /*     0    16 */
	} in6_u;                                         /*     0    16 */

	/* size: 16, cachelines: 1, members: 1 */
	/* last cacheline: 16 bytes */
};
struct ethhdr {
	unsigned char              h_dest[6];            /*     0     6 */
	unsigned char              h_source[6];          /*     6     6 */
	__be16                     h_proto;              /*    12     2 */

	/* size: 14, cachelines: 1, members: 3 */
	/* last cacheline: 14 bytes */
};
struct flowi_tunnel {
	__be64                     tun_id;               /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct flowi_common {
	int                        flowic_oif;           /*     0     4 */
	int                        flowic_iif;           /*     4     4 */
	__u32                      flowic_mark;          /*     8     4 */
	__u8                       flowic_tos;           /*    12     1 */
	__u8                       flowic_scope;         /*    13     1 */
	__u8                       flowic_proto;         /*    14     1 */
	__u8                       flowic_flags;         /*    15     1 */
	__u32                      flowic_secid;         /*    16     4 */
	kuid_t                     flowic_uid;           /*    20     4 */
	struct flowi_tunnel        flowic_tun_key;       /*    24     8 */
	__u32                      flowic_multipath_hash; /*    32     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        kabi_reserved1;       /*    40     8 */
	u64                        kabi_reserved2;       /*    48     8 */

	/* size: 56, cachelines: 1, members: 13 */
	/* sum members: 52, holes: 1, sum holes: 4 */
	/* last cacheline: 56 bytes */
};
union flowi_uli {
	struct {
		__be16             dport;              /*     0     2 */
		__be16             sport;              /*     2     2 */
	} ports;                                       /*     0     4 */
	struct {
		__u8               type;               /*     0     1 */
		__u8               code;               /*     1     1 */
	} icmpt;                                       /*     0     2 */
	struct {
		__le16             dport;              /*     0     2 */
		__le16             sport;              /*     2     2 */
	} dnports;                                     /*     0     4 */
	__be32                     spi;                /*     0     4 */
	__be32                     gre_key;            /*     0     4 */
	struct {
		__u8               type;               /*     0     1 */
	} mht;                                         /*     0     1 */
};
struct flowi4 {
	struct flowi_common        __fl_common;          /*     0    56 */
	__be32                     saddr;                /*    56     4 */
	__be32                     daddr;                /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	union flowi_uli            uli;                  /*    64     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        kabi_reserved1;       /*    72     8 */
	u64                        kabi_reserved2;       /*    80     8 */

	/* size: 88, cachelines: 2, members: 6 */
	/* sum members: 84, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct flowi6 {
	struct flowi_common        __fl_common;          /*     0    56 */
	struct in6_addr            daddr;                /*    56    16 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	struct in6_addr            saddr;                /*    72    16 */
	__be32                     flowlabel;            /*    88     4 */
	union flowi_uli            uli;                  /*    92     4 */
	__u32                      mp_hash;              /*    96     4 */

	/* size: 104, cachelines: 2, members: 6 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
} __attribute__((__aligned__(8)));
struct flowidn {
	struct flowi_common        __fl_common;          /*     0    56 */
	__le16                     daddr;                /*    56     2 */
	__le16                     saddr;                /*    58     2 */
	union flowi_uli            uli;                  /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u64                        kabi_reserved1;       /*    64     8 */
	u64                        kabi_reserved2;       /*    72     8 */

	/* size: 80, cachelines: 2, members: 6 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct flowi {
	union {
		struct flowi_common __fl_common;         /*     0    56 */
		struct flowi4      ip4 __attribute__((__aligned__(8))); /*     0    88 */
		struct flowi6      ip6 __attribute__((__aligned__(8))); /*     0   104 */
		struct flowidn     dn __attribute__((__aligned__(8))); /*     0    80 */
	} u __attribute__((__aligned__(8)));             /*     0   104 */

	/* size: 104, cachelines: 2, members: 1 */
	/* forced alignments: 1 */
	/* last cacheline: 40 bytes */
} __attribute__((__aligned__(8)));
struct netns_core {
	struct ctl_table_header *  sysctl_hdr;           /*     0     8 */
	int                        sysctl_somaxconn;     /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	int *                      sock_inuse;           /*    16     8 */
	struct prot_inuse *        prot_inuse;           /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct ipstats_mib {
	u64                        mibs[37];             /*     0   296 */
	/* --- cacheline 4 boundary (256 bytes) was 40 bytes ago --- */
	struct u64_stats_sync      syncp;                /*   296     0 */

	/* size: 296, cachelines: 5, members: 2 */
	/* last cacheline: 40 bytes */
};
struct icmp_mib {
	long unsigned int          mibs[28];             /*     0   224 */

	/* size: 224, cachelines: 4, members: 1 */
	/* last cacheline: 32 bytes */
};
struct icmpmsg_mib {
	atomic_long_t              mibs[512];            /*     0  4096 */

	/* size: 4096, cachelines: 64, members: 1 */
};
struct icmpv6_mib {
	long unsigned int          mibs[6];              /*     0    48 */

	/* size: 48, cachelines: 1, members: 1 */
	/* last cacheline: 48 bytes */
};
struct icmpv6_mib_device {
	atomic_long_t              mibs[6];              /*     0    48 */

	/* size: 48, cachelines: 1, members: 1 */
	/* last cacheline: 48 bytes */
};
struct icmpv6msg_mib {
	atomic_long_t              mibs[512];            /*     0  4096 */

	/* size: 4096, cachelines: 64, members: 1 */
};
struct icmpv6msg_mib_device {
	atomic_long_t              mibs[512];            /*     0  4096 */

	/* size: 4096, cachelines: 64, members: 1 */
};
struct tcp_mib {
	long unsigned int          mibs[16];             /*     0   128 */

	/* size: 128, cachelines: 2, members: 1 */
};
struct udp_mib {
	long unsigned int          mibs[9];              /*     0    72 */

	/* size: 72, cachelines: 2, members: 1 */
	/* last cacheline: 8 bytes */
};
struct linux_mib {
	long unsigned int          mibs[124];            /*     0   992 */

	/* size: 992, cachelines: 16, members: 1 */
	/* last cacheline: 32 bytes */
};
struct linux_xfrm_mib {
	long unsigned int          mibs[29];             /*     0   232 */

	/* size: 232, cachelines: 4, members: 1 */
	/* last cacheline: 40 bytes */
};
struct linux_tls_mib {
	long unsigned int          mibs[11];             /*     0    88 */

	/* size: 88, cachelines: 2, members: 1 */
	/* last cacheline: 24 bytes */
};
struct netns_mib {
	struct tcp_mib *           tcp_statistics;       /*     0     8 */
	struct ipstats_mib *       ip_statistics;        /*     8     8 */
	struct linux_mib *         net_statistics;       /*    16     8 */
	struct udp_mib *           udp_statistics;       /*    24     8 */
	struct udp_mib *           udplite_statistics;   /*    32     8 */
	struct icmp_mib *          icmp_statistics;      /*    40     8 */
	struct icmpmsg_mib *       icmpmsg_statistics;   /*    48     8 */
	struct proc_dir_entry *    proc_net_devsnmp6;    /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct udp_mib *           udp_stats_in6;        /*    64     8 */
	struct udp_mib *           udplite_stats_in6;    /*    72     8 */
	struct ipstats_mib *       ipv6_statistics;      /*    80     8 */
	struct icmpv6_mib *        icmpv6_statistics;    /*    88     8 */
	struct icmpv6msg_mib *     icmpv6msg_statistics; /*    96     8 */
	struct linux_xfrm_mib *    xfrm_statistics;      /*   104     8 */
	struct linux_tls_mib *     tls_statistics;       /*   112     8 */

	/* size: 120, cachelines: 2, members: 15 */
	/* last cacheline: 56 bytes */
};
struct netns_unix {
	int                        sysctl_max_dgram_qlen; /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct ctl_table_header *  ctl;                  /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct netns_packet {
	struct mutex               sklist_lock;          /*     0    32 */
	struct hlist_head          sklist;               /*    32     8 */

	/* size: 40, cachelines: 1, members: 2 */
	/* last cacheline: 40 bytes */
};
struct fqdir {
	long int                   high_thresh;          /*     0     8 */
	long int                   low_thresh;           /*     8     8 */
	int                        timeout;              /*    16     4 */
	int                        max_dist;             /*    20     4 */
	struct inet_frags *        f;                    /*    24     8 */
	struct net *               net;                  /*    32     8 */
	bool                       dead;                 /*    40     1 */

	/* XXX 23 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	struct rhashtable          rhashtable __attribute__((__aligned__(64))); /*    64   168 */

	/* XXX 24 bytes hole, try to pack */

	/* --- cacheline 4 boundary (256 bytes) --- */
	atomic_long_t              mem __attribute__((__aligned__(64))); /*   256     8 */
	struct work_struct         destroy_work;         /*   264    64 */

	/* size: 384, cachelines: 6, members: 10 */
	/* sum members: 281, holes: 2, sum holes: 47 */
	/* padding: 56 */
	/* forced alignments: 2, forced holes: 2, sum forced holes: 47 */
} __attribute__((__aligned__(64)));
struct inet_frags {
	unsigned int               qsize;                /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	void                       (*constructor)(struct inet_frag_queue *, const void  *); /*     8     8 */
	void                       (*destructor)(struct inet_frag_queue *); /*    16     8 */
	void                       (*frag_expire)(struct timer_list *); /*    24     8 */
	struct kmem_cache *        frags_cachep;         /*    32     8 */
	const char  *              frags_cache_name;     /*    40     8 */
	struct rhashtable_params   rhash_params;         /*    48    40 */
	/* --- cacheline 1 boundary (64 bytes) was 24 bytes ago --- */
	refcount_t                 refcnt;               /*    88     4 */

	/* XXX 4 bytes hole, try to pack */

	struct completion          completion;           /*    96    32 */

	/* size: 128, cachelines: 2, members: 9 */
	/* sum members: 120, holes: 2, sum holes: 8 */
};
struct frag_v4_compare_key {
	__be32                     saddr;                /*     0     4 */
	__be32                     daddr;                /*     4     4 */
	u32                        user;                 /*     8     4 */
	u32                        vif;                  /*    12     4 */
	__be16                     id;                   /*    16     2 */
	u16                        protocol;             /*    18     2 */

	/* size: 20, cachelines: 1, members: 6 */
	/* last cacheline: 20 bytes */
};
struct frag_v6_compare_key {
	struct in6_addr            saddr;                /*     0    16 */
	struct in6_addr            daddr;                /*    16    16 */
	u32                        user;                 /*    32     4 */
	__be32                     id;                   /*    36     4 */
	u32                        iif;                  /*    40     4 */

	/* size: 44, cachelines: 1, members: 5 */
	/* last cacheline: 44 bytes */
};
struct inet_frag_queue {
	struct rhash_head          node;                 /*     0     8 */
	union {
		struct frag_v4_compare_key v4;           /*     8    20 */
		struct frag_v6_compare_key v6;           /*     8    44 */
	} key;                                           /*     8    44 */

	/* XXX 4 bytes hole, try to pack */

	struct timer_list          timer;                /*    56    72 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	spinlock_t                 lock;                 /*   128     4 */
	refcount_t                 refcnt;               /*   132     4 */
	struct rb_root             rb_fragments;         /*   136     8 */
	struct sk_buff *           fragments_tail;       /*   144     8 */
	struct sk_buff *           last_run_head;        /*   152     8 */
	ktime_t                    stamp;                /*   160     8 */
	int                        len;                  /*   168     4 */
	int                        meat;                 /*   172     4 */
	__u8                       flags;                /*   176     1 */

	/* XXX 1 byte hole, try to pack */

	u16                        max_size;             /*   178     2 */

	/* XXX 4 bytes hole, try to pack */

	struct fqdir *             fqdir;                /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*   192    16 */

	/* size: 208, cachelines: 4, members: 15 */
	/* sum members: 199, holes: 3, sum holes: 9 */
	/* forced alignments: 1 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct local_ports {
	seqlock_t                  lock;                 /*     0     8 */
	int                        range[2];             /*     8     8 */
	bool                       warned;               /*    16     1 */

	/* size: 20, cachelines: 1, members: 3 */
	/* padding: 3 */
	/* last cacheline: 20 bytes */
};
struct ping_group_range {
	seqlock_t                  lock;                 /*     0     8 */
	kgid_t                     range[2];             /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct inet_timewait_death_row {
	atomic_t                   tw_count;             /*     0     4 */

	/* XXX 60 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	struct inet_hashinfo *     hashinfo __attribute__((__aligned__(64))); /*    64     8 */
	int                        sysctl_max_tw_buckets; /*    72     4 */

	/* size: 128, cachelines: 2, members: 3 */
	/* sum members: 16, holes: 1, sum holes: 60 */
	/* padding: 52 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 60 */
} __attribute__((__aligned__(64)));
struct netns_ipv4 {
	struct ctl_table_header *  forw_hdr;             /*     0     8 */
	struct ctl_table_header *  frags_hdr;            /*     8     8 */
	struct ctl_table_header *  ipv4_hdr;             /*    16     8 */
	struct ctl_table_header *  route_hdr;            /*    24     8 */
	struct ctl_table_header *  xfrm4_hdr;            /*    32     8 */
	struct ipv4_devconf *      devconf_all;          /*    40     8 */
	struct ipv4_devconf *      devconf_dflt;         /*    48     8 */
	struct ip_ra_chain *       ra_chain;             /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct mutex               ra_mutex;             /*    64    32 */
	struct fib_rules_ops *     rules_ops;            /*    96     8 */
	bool                       fib_has_custom_rules; /*   104     1 */

	/* XXX 3 bytes hole, try to pack */

	unsigned int               fib_rules_require_fldissect; /*   108     4 */
	struct fib_table *         fib_main;             /*   112     8 */
	struct fib_table *         fib_default;          /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	bool                       fib_has_custom_local_routes; /*   128     1 */

	/* XXX 3 bytes hole, try to pack */

	atomic_t                   fib_num_tclassid_users; /*   132     4 */
	struct hlist_head *        fib_table_hash;       /*   136     8 */
	bool                       fib_offload_disabled; /*   144     1 */

	/* XXX 7 bytes hole, try to pack */

	struct sock *              fibnl;                /*   152     8 */
	struct sock * *            icmp_sk;              /*   160     8 */
	struct sock *              mc_autojoin_sk;       /*   168     8 */
	struct inet_peer_base *    peers;                /*   176     8 */
	struct sock * *            tcp_sk;               /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	struct fqdir *             fqdir;                /*   192     8 */
	struct xt_table *          iptable_filter;       /*   200     8 */
	struct xt_table *          iptable_mangle;       /*   208     8 */
	struct xt_table *          iptable_raw;          /*   216     8 */
	struct xt_table *          arptable_filter;      /*   224     8 */
	struct xt_table *          iptable_security;     /*   232     8 */
	struct xt_table *          nat_table;            /*   240     8 */
	int                        sysctl_icmp_echo_ignore_all; /*   248     4 */
	int                        sysctl_icmp_echo_ignore_broadcasts; /*   252     4 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	int                        sysctl_icmp_ignore_bogus_error_responses; /*   256     4 */
	int                        sysctl_icmp_ratelimit; /*   260     4 */
	int                        sysctl_icmp_ratemask; /*   264     4 */
	int                        sysctl_icmp_errors_use_inbound_ifaddr; /*   268     4 */
	struct local_ports         ip_local_ports;       /*   272    20 */

	/* XXX last struct has 3 bytes of padding */

	int                        sysctl_tcp_ecn;       /*   292     4 */
	int                        sysctl_tcp_ecn_fallback; /*   296     4 */
	int                        sysctl_ip_default_ttl; /*   300     4 */
	int                        sysctl_ip_no_pmtu_disc; /*   304     4 */
	int                        sysctl_ip_fwd_use_pmtu; /*   308     4 */
	int                        sysctl_ip_fwd_update_priority; /*   312     4 */
	int                        sysctl_ip_nonlocal_bind; /*   316     4 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	int                        sysctl_ip_autobind_reuse; /*   320     4 */
	int                        sysctl_ip_dynaddr;    /*   324     4 */
	int                        sysctl_ip_early_demux; /*   328     4 */
	int                        sysctl_raw_l3mdev_accept; /*   332     4 */
	int                        sysctl_tcp_early_demux; /*   336     4 */
	int                        sysctl_udp_early_demux; /*   340     4 */
	int                        sysctl_nexthop_compat_mode; /*   344     4 */
	int                        sysctl_fwmark_reflect; /*   348     4 */
	int                        sysctl_tcp_fwmark_accept; /*   352     4 */
	int                        sysctl_tcp_l3mdev_accept; /*   356     4 */
	int                        sysctl_tcp_mtu_probing; /*   360     4 */
	int                        sysctl_tcp_mtu_probe_floor; /*   364     4 */
	int                        sysctl_tcp_base_mss;  /*   368     4 */
	int                        sysctl_tcp_min_snd_mss; /*   372     4 */
	int                        sysctl_tcp_probe_threshold; /*   376     4 */
	u32                        sysctl_tcp_probe_interval; /*   380     4 */
	/* --- cacheline 6 boundary (384 bytes) --- */
	int                        sysctl_tcp_keepalive_time; /*   384     4 */
	int                        sysctl_tcp_keepalive_probes; /*   388     4 */
	int                        sysctl_tcp_keepalive_intvl; /*   392     4 */
	int                        sysctl_tcp_syn_retries; /*   396     4 */
	int                        sysctl_tcp_synack_retries; /*   400     4 */
	int                        sysctl_tcp_syncookies; /*   404     4 */
	int                        sysctl_tcp_reordering; /*   408     4 */
	int                        sysctl_tcp_retries1;  /*   412     4 */
	int                        sysctl_tcp_retries2;  /*   416     4 */
	int                        sysctl_tcp_orphan_retries; /*   420     4 */
	int                        sysctl_tcp_fin_timeout; /*   424     4 */
	unsigned int               sysctl_tcp_notsent_lowat; /*   428     4 */
	int                        sysctl_tcp_tw_reuse;  /*   432     4 */
	int                        sysctl_tcp_sack;      /*   436     4 */
	int                        sysctl_tcp_window_scaling; /*   440     4 */
	int                        sysctl_tcp_timestamps; /*   444     4 */
	/* --- cacheline 7 boundary (448 bytes) --- */
	int                        sysctl_tcp_early_retrans; /*   448     4 */
	int                        sysctl_tcp_recovery;  /*   452     4 */
	int                        sysctl_tcp_thin_linear_timeouts; /*   456     4 */
	int                        sysctl_tcp_slow_start_after_idle; /*   460     4 */
	int                        sysctl_tcp_retrans_collapse; /*   464     4 */
	int                        sysctl_tcp_stdurg;    /*   468     4 */
	int                        sysctl_tcp_rfc1337;   /*   472     4 */
	int                        sysctl_tcp_abort_on_overflow; /*   476     4 */
	int                        sysctl_tcp_fack;      /*   480     4 */
	int                        sysctl_tcp_max_reordering; /*   484     4 */
	int                        sysctl_tcp_dsack;     /*   488     4 */
	int                        sysctl_tcp_app_win;   /*   492     4 */
	int                        sysctl_tcp_adv_win_scale; /*   496     4 */
	int                        sysctl_tcp_frto;      /*   500     4 */
	int                        sysctl_tcp_nometrics_save; /*   504     4 */
	int                        sysctl_tcp_no_ssthresh_metrics_save; /*   508     4 */
	/* --- cacheline 8 boundary (512 bytes) --- */
	int                        sysctl_tcp_moderate_rcvbuf; /*   512     4 */
	int                        sysctl_tcp_tso_win_divisor; /*   516     4 */
	int                        sysctl_tcp_workaround_signed_windows; /*   520     4 */
	int                        sysctl_tcp_limit_output_bytes; /*   524     4 */
	int                        sysctl_tcp_challenge_ack_limit; /*   528     4 */
	int                        sysctl_tcp_min_tso_segs; /*   532     4 */
	int                        sysctl_tcp_min_rtt_wlen; /*   536     4 */
	int                        sysctl_tcp_autocorking; /*   540     4 */
	int                        sysctl_tcp_invalid_ratelimit; /*   544     4 */
	int                        sysctl_tcp_pacing_ss_ratio; /*   548     4 */
	int                        sysctl_tcp_pacing_ca_ratio; /*   552     4 */
	int                        sysctl_tcp_wmem[3];   /*   556    12 */
	int                        sysctl_tcp_rmem[3];   /*   568    12 */
	/* --- cacheline 9 boundary (576 bytes) was 4 bytes ago --- */
	int                        sysctl_tcp_comp_sack_nr; /*   580     4 */
	long unsigned int          sysctl_tcp_comp_sack_delay_ns; /*   584     8 */
	long unsigned int          sysctl_tcp_comp_sack_slack_ns; /*   592     8 */

	/* XXX 40 bytes hole, try to pack */

	/* --- cacheline 10 boundary (640 bytes) --- */
	struct inet_timewait_death_row tcp_death_row __attribute__((__aligned__(64))); /*   640   128 */

	/* XXX last struct has 52 bytes of padding */

	/* --- cacheline 12 boundary (768 bytes) --- */
	int                        sysctl_max_syn_backlog; /*   768     4 */
	int                        sysctl_tcp_fastopen;  /*   772     4 */
	const struct tcp_congestion_ops  * tcp_congestion_control; /*   776     8 */
	struct tcp_fastopen_context * tcp_fastopen_ctx;  /*   784     8 */
	spinlock_t                 tcp_fastopen_ctx_lock; /*   792     4 */
	unsigned int               sysctl_tcp_fastopen_blackhole_timeout; /*   796     4 */
	atomic_t                   tfo_active_disable_times; /*   800     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          tfo_active_disable_stamp; /*   808     8 */
	int                        sysctl_tcp_reflect_tos; /*   816     4 */
	int                        sysctl_udp_wmem_min;  /*   820     4 */
	int                        sysctl_udp_rmem_min;  /*   824     4 */
	int                        sysctl_udp_l3mdev_accept; /*   828     4 */
	/* --- cacheline 13 boundary (832 bytes) --- */
	int                        sysctl_igmp_max_memberships; /*   832     4 */
	int                        sysctl_igmp_max_msf;  /*   836     4 */
	int                        sysctl_igmp_llm_reports; /*   840     4 */
	int                        sysctl_igmp_qrv;      /*   844     4 */
	struct ping_group_range    ping_group_range;     /*   848    16 */
	atomic_t                   dev_addr_genid;       /*   864     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int *        sysctl_local_reserved_ports; /*   872     8 */
	int                        sysctl_ip_prot_sock;  /*   880     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           mr_tables;            /*   888    16 */
	/* --- cacheline 14 boundary (896 bytes) was 8 bytes ago --- */
	struct fib_rules_ops *     mr_rules_ops;         /*   904     8 */
	int                        sysctl_fib_multipath_use_neigh; /*   912     4 */
	int                        sysctl_fib_multipath_hash_policy; /*   916     4 */
	struct fib_notifier_ops *  notifier_ops;         /*   920     8 */
	unsigned int               fib_seq;              /*   928     4 */

	/* XXX 4 bytes hole, try to pack */

	struct fib_notifier_ops *  ipmr_notifier_ops;    /*   936     8 */
	unsigned int               ipmr_seq;             /*   944     4 */
	atomic_t                   rt_genid;             /*   948     4 */
	siphash_key_t              ip_id_key;            /*   952    16 */
	/* --- cacheline 15 boundary (960 bytes) was 8 bytes ago --- */
	u64                        kabi_reserved1;       /*   968     8 */
	u64                        kabi_reserved2;       /*   976     8 */
	u64                        kabi_reserved3;       /*   984     8 */
	u64                        kabi_reserved4;       /*   992     8 */
	u64                        kabi_reserved5;       /*  1000     8 */
	u64                        kabi_reserved6;       /*  1008     8 */

	/* size: 1024, cachelines: 16, members: 145 */
	/* sum members: 947, holes: 8, sum holes: 69 */
	/* padding: 8 */
	/* paddings: 2, sum paddings: 55 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 40 */
} __attribute__((__aligned__(64)));
struct fib_rules_ops {
	int                        family;               /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           list;                 /*     8    16 */
	int                        rule_size;            /*    24     4 */
	int                        addr_size;            /*    28     4 */
	int                        unresolved_rules;     /*    32     4 */
	int                        nr_goto_rules;        /*    36     4 */
	unsigned int               fib_rules_seq;        /*    40     4 */

	/* XXX 4 bytes hole, try to pack */

	int                        (*action)(struct fib_rule *, struct flowi *, int, struct fib_lookup_arg *); /*    48     8 */
	bool                       (*suppress)(struct fib_rule *, int, struct fib_lookup_arg *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*match)(struct fib_rule *, struct flowi *, int); /*    64     8 */
	int                        (*configure)(struct fib_rule *, struct sk_buff *, struct fib_rule_hdr *, struct nlattr * *, struct netlink_ext_ack *); /*    72     8 */
	int                        (*delete)(struct fib_rule *); /*    80     8 */
	int                        (*compare)(struct fib_rule *, struct fib_rule_hdr *, struct nlattr * *); /*    88     8 */
	int                        (*fill)(struct fib_rule *, struct sk_buff *, struct fib_rule_hdr *); /*    96     8 */
	size_t                     (*nlmsg_payload)(struct fib_rule *); /*   104     8 */
	void                       (*flush_cache)(struct fib_rules_ops *); /*   112     8 */
	int                        nlgroup;              /*   120     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 2 boundary (128 bytes) --- */
	const struct nla_policy  * policy;               /*   128     8 */
	struct list_head           rules_list;           /*   136    16 */
	struct module *            owner;                /*   152     8 */
	struct net *               fro_net;              /*   160     8 */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*   168    16 */

	/* size: 184, cachelines: 3, members: 22 */
	/* sum members: 172, holes: 3, sum holes: 12 */
	/* forced alignments: 1 */
	/* last cacheline: 56 bytes */
} __attribute__((__aligned__(8)));
struct fib_notifier_ops {
	int                        family;               /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           list;                 /*     8    16 */
	unsigned int               (*fib_seq_read)(struct net *); /*    24     8 */
	int                        (*fib_dump)(struct net *, struct notifier_block *, struct netlink_ext_ack *); /*    32     8 */
	struct module *            owner;                /*    40     8 */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*    48    16 */

	/* size: 64, cachelines: 1, members: 6 */
	/* sum members: 60, holes: 1, sum holes: 4 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(8)));
struct dst_ops {
	short unsigned int         family;               /*     0     2 */

	/* XXX 2 bytes hole, try to pack */

	unsigned int               gc_thresh;            /*     4     4 */
	int                        (*gc)(struct dst_ops *); /*     8     8 */
	struct dst_entry *         (*check)(struct dst_entry *, __u32); /*    16     8 */
	unsigned int               (*default_advmss)(const struct dst_entry  *); /*    24     8 */
	unsigned int               (*mtu)(const struct dst_entry  *); /*    32     8 */
	u32 *                      (*cow_metrics)(struct dst_entry *, long unsigned int); /*    40     8 */
	void                       (*destroy)(struct dst_entry *); /*    48     8 */
	void                       (*ifdown)(struct dst_entry *, struct net_device *, int); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct dst_entry *         (*negative_advice)(struct dst_entry *); /*    64     8 */
	void                       (*link_failure)(struct sk_buff *); /*    72     8 */
	void                       (*update_pmtu)(struct dst_entry *, struct sock *, struct sk_buff *, u32, bool); /*    80     8 */
	void                       (*redirect)(struct dst_entry *, struct sock *, struct sk_buff *); /*    88     8 */
	int                        (*local_out)(struct net *, struct sock *, struct sk_buff *); /*    96     8 */
	struct neighbour *         (*neigh_lookup)(const struct dst_entry  *, struct sk_buff *, const void  *); /*   104     8 */
	void                       (*confirm_neigh)(const struct dst_entry  *, const void  *); /*   112     8 */
	struct kmem_cache *        kmem_cachep;          /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct percpu_counter      pcpuc_entries __attribute__((__aligned__(64))); /*   128    40 */
	u64                        kabi_reserved1;       /*   168     8 */
	u64                        kabi_reserved2;       /*   176     8 */
	u64                        kabi_reserved3;       /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	u64                        kabi_reserved4;       /*   192     8 */
	u64                        kabi_reserved5;       /*   200     8 */
	u64                        kabi_reserved6;       /*   208     8 */
	u64                        kabi_reserved7;       /*   216     8 */
	u64                        kabi_reserved8;       /*   224     8 */

	/* size: 256, cachelines: 4, members: 26 */
	/* sum members: 230, holes: 1, sum holes: 2 */
	/* padding: 24 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(64)));
struct dst_entry {
	struct net_device *        dev;                  /*     0     8 */
	struct dst_ops *           ops;                  /*     8     8 */
	long unsigned int          _metrics;             /*    16     8 */
	long unsigned int          expires;              /*    24     8 */
	struct xfrm_state *        xfrm;                 /*    32     8 */
	int                        (*input)(struct sk_buff *); /*    40     8 */
	int                        (*output)(struct net *, struct sock *, struct sk_buff *); /*    48     8 */
	short unsigned int         flags;                /*    56     2 */
	short int                  obsolete;             /*    58     2 */
	short unsigned int         header_len;           /*    60     2 */
	short unsigned int         trailer_len;          /*    62     2 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	atomic_t                   __refcnt;             /*    64     4 */
	int                        __use;                /*    68     4 */
	long unsigned int          lastuse;              /*    72     8 */
	struct lwtunnel_state *    lwtstate;             /*    80     8 */
	struct callback_head       callback_head __attribute__((__aligned__(8))); /*    88    16 */
	short int                  error;                /*   104     2 */
	short int                  __pad;                /*   106     2 */
	__u32                      tclassid;             /*   108     4 */
	u64                        kabi_reserved1;       /*   112     8 */
	u64                        kabi_reserved2;       /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	u64                        kabi_reserved3;       /*   128     8 */
	u64                        kabi_reserved4;       /*   136     8 */
	u64                        kabi_reserved5;       /*   144     8 */
	u64                        kabi_reserved6;       /*   152     8 */
	u64                        kabi_reserved7;       /*   160     8 */
	u64                        kabi_reserved8;       /*   168     8 */

	/* size: 176, cachelines: 3, members: 27 */
	/* forced alignments: 1 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct neighbour {
	struct neighbour *         next;                 /*     0     8 */
	struct neigh_table *       tbl;                  /*     8     8 */
	struct neigh_parms *       parms;                /*    16     8 */
	long unsigned int          confirmed;            /*    24     8 */
	long unsigned int          updated;              /*    32     8 */
	rwlock_t                   lock;                 /*    40     8 */
	refcount_t                 refcnt;               /*    48     4 */
	unsigned int               arp_queue_len_bytes;  /*    52     4 */
	struct sk_buff_head        arp_queue;            /*    56    24 */
	/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
	struct timer_list          timer;                /*    80    72 */
	/* --- cacheline 2 boundary (128 bytes) was 24 bytes ago --- */
	long unsigned int          used;                 /*   152     8 */
	atomic_t                   probes;               /*   160     4 */
	__u8                       flags;                /*   164     1 */
	__u8                       nud_state;            /*   165     1 */
	__u8                       type;                 /*   166     1 */
	__u8                       dead;                 /*   167     1 */
	u8                         protocol;             /*   168     1 */

	/* XXX 3 bytes hole, try to pack */

	seqlock_t                  ha_lock;              /*   172     8 */

	/* XXX 4 bytes hole, try to pack */

	unsigned char              ha[32] __attribute__((__aligned__(8))); /*   184    32 */
	/* --- cacheline 3 boundary (192 bytes) was 24 bytes ago --- */
	struct hh_cache            hh;                   /*   216   112 */
	/* --- cacheline 5 boundary (320 bytes) was 8 bytes ago --- */
	int                        (*output)(struct neighbour *, struct sk_buff *); /*   328     8 */
	const struct neigh_ops  *  ops;                  /*   336     8 */
	struct list_head           gc_list;              /*   344    16 */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*   360    16 */
	struct net_device *        dev;                  /*   376     8 */
	/* --- cacheline 6 boundary (384 bytes) --- */
	u64                        kabi_reserved1;       /*   384     8 */
	u8                         primary_key[];        /*   392     0 */

	/* size: 392, cachelines: 7, members: 27 */
	/* sum members: 385, holes: 2, sum holes: 7 */
	/* forced alignments: 2, forced holes: 1, sum forced holes: 4 */
	/* last cacheline: 8 bytes */
} __attribute__((__aligned__(8)));
struct netns_sysctl_ipv6 {
	struct ctl_table_header *  hdr;                  /*     0     8 */
	struct ctl_table_header *  route_hdr;            /*     8     8 */
	struct ctl_table_header *  icmp_hdr;             /*    16     8 */
	struct ctl_table_header *  frags_hdr;            /*    24     8 */
	struct ctl_table_header *  xfrm6_hdr;            /*    32     8 */
	int                        bindv6only;           /*    40     4 */
	int                        flush_delay;          /*    44     4 */
	int                        ip6_rt_max_size;      /*    48     4 */
	int                        ip6_rt_gc_min_interval; /*    52     4 */
	int                        ip6_rt_gc_timeout;    /*    56     4 */
	int                        ip6_rt_gc_interval;   /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        ip6_rt_gc_elasticity; /*    64     4 */
	int                        ip6_rt_mtu_expires;   /*    68     4 */
	int                        ip6_rt_min_advmss;    /*    72     4 */
	int                        multipath_hash_policy; /*    76     4 */
	int                        flowlabel_consistency; /*    80     4 */
	int                        auto_flowlabels;      /*    84     4 */
	int                        icmpv6_time;          /*    88     4 */
	int                        icmpv6_echo_ignore_all; /*    92     4 */
	int                        icmpv6_echo_ignore_multicast; /*    96     4 */
	int                        icmpv6_echo_ignore_anycast; /*   100     4 */
	long unsigned int          icmpv6_ratemask[4];   /*   104    32 */
	/* --- cacheline 2 boundary (128 bytes) was 8 bytes ago --- */
	long unsigned int *        icmpv6_ratemask_ptr;  /*   136     8 */
	int                        anycast_src_echo_reply; /*   144     4 */
	int                        ip_nonlocal_bind;     /*   148     4 */
	int                        fwmark_reflect;       /*   152     4 */
	int                        idgen_retries;        /*   156     4 */
	int                        idgen_delay;          /*   160     4 */
	int                        flowlabel_state_ranges; /*   164     4 */
	int                        flowlabel_reflect;    /*   168     4 */
	int                        max_dst_opts_cnt;     /*   172     4 */
	int                        max_hbh_opts_cnt;     /*   176     4 */
	int                        max_dst_opts_len;     /*   180     4 */
	int                        max_hbh_opts_len;     /*   184     4 */
	int                        seg6_flowlabel;       /*   188     4 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	bool                       skip_notify_on_dev_down; /*   192     1 */

	/* size: 200, cachelines: 4, members: 36 */
	/* padding: 7 */
	/* last cacheline: 8 bytes */
};
struct netns_ipv6 {
	struct netns_sysctl_ipv6   sysctl;               /*     0   200 */

	/* XXX last struct has 7 bytes of padding */

	/* --- cacheline 3 boundary (192 bytes) was 8 bytes ago --- */
	struct ipv6_devconf *      devconf_all;          /*   200     8 */
	struct ipv6_devconf *      devconf_dflt;         /*   208     8 */
	struct inet_peer_base *    peers;                /*   216     8 */
	struct fqdir *             fqdir;                /*   224     8 */
	struct xt_table *          ip6table_filter;      /*   232     8 */
	struct xt_table *          ip6table_mangle;      /*   240     8 */
	struct xt_table *          ip6table_raw;         /*   248     8 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	struct xt_table *          ip6table_security;    /*   256     8 */
	struct xt_table *          ip6table_nat;         /*   264     8 */
	struct fib6_info *         fib6_null_entry;      /*   272     8 */
	struct rt6_info *          ip6_null_entry;       /*   280     8 */
	struct rt6_statistics *    rt6_stats;            /*   288     8 */
	struct timer_list          ip6_fib_timer;        /*   296    72 */
	/* --- cacheline 5 boundary (320 bytes) was 48 bytes ago --- */
	struct hlist_head *        fib_table_hash;       /*   368     8 */
	struct fib6_table *        fib6_main_tbl;        /*   376     8 */
	/* --- cacheline 6 boundary (384 bytes) --- */
	struct list_head           fib6_walkers;         /*   384    16 */

	/* XXX 48 bytes hole, try to pack */

	/* --- cacheline 7 boundary (448 bytes) --- */
	struct dst_ops             ip6_dst_ops __attribute__((__aligned__(64))); /*   448   256 */

	/* XXX last struct has 24 bytes of padding */

	/* --- cacheline 11 boundary (704 bytes) --- */
	rwlock_t                   fib6_walker_lock;     /*   704     8 */
	spinlock_t                 fib6_gc_lock;         /*   712     4 */
	union {
		atomic_t           ip6_rt_gc_expire;     /*   716     4 */
		struct {
			unsigned int ip6_rt_gc_expire;   /*   716     4 */
		} kabi_hidden_82;                        /*   716     4 */
		union {
		};                                       /*   716     0 */
	};                                               /*   716     4 */
	long unsigned int          ip6_rt_last_gc;       /*   720     8 */
	unsigned int               fib6_rules_require_fldissect; /*   728     4 */
	unsigned char              flowlabel_has_excl;   /*   732     1 */
	bool                       fib6_has_custom_rules; /*   733     1 */

	/* XXX 2 bytes hole, try to pack */

	struct rt6_info *          ip6_prohibit_entry;   /*   736     8 */
	struct rt6_info *          ip6_blk_hole_entry;   /*   744     8 */
	struct fib6_table *        fib6_local_tbl;       /*   752     8 */
	struct fib_rules_ops *     fib6_rules_ops;       /*   760     8 */
	/* --- cacheline 12 boundary (768 bytes) --- */
	struct sock * *            icmp_sk;              /*   768     8 */
	struct sock *              ndisc_sk;             /*   776     8 */
	struct sock *              tcp_sk;               /*   784     8 */
	struct sock *              igmp_sk;              /*   792     8 */
	struct sock *              mc_autojoin_sk;       /*   800     8 */
	struct list_head           mr6_tables;           /*   808    16 */
	struct fib_rules_ops *     mr6_rules_ops;        /*   824     8 */
	/* --- cacheline 13 boundary (832 bytes) --- */
	atomic_t                   dev_addr_genid;       /*   832     4 */
	atomic_t                   fib6_sernum;          /*   836     4 */
	struct seg6_pernet_data *  seg6_data;            /*   840     8 */
	struct fib_notifier_ops *  notifier_ops;         /*   848     8 */
	struct fib_notifier_ops *  ip6mr_notifier_ops;   /*   856     8 */
	unsigned int               ipmr_seq;             /*   864     4 */

	/* XXX 4 bytes hole, try to pack */

	struct {
		struct hlist_head  head;                 /*   872     8 */
		spinlock_t         lock;                 /*   880     4 */
		u32                seq;                  /*   884     4 */
	} ip6addrlbl_table;                              /*   872    16 */
	u64                        kabi_reserved1;       /*   888     8 */

	/* size: 896, cachelines: 14, members: 44 */
	/* sum members: 842, holes: 3, sum holes: 54 */
	/* paddings: 2, sum paddings: 31 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 48 */
} __attribute__((__aligned__(64)));
struct ipv6_devconf {
	__s32                      forwarding;           /*     0     4 */
	__s32                      hop_limit;            /*     4     4 */
	__s32                      mtu6;                 /*     8     4 */
	__s32                      accept_ra;            /*    12     4 */
	__s32                      accept_redirects;     /*    16     4 */
	__s32                      autoconf;             /*    20     4 */
	__s32                      dad_transmits;        /*    24     4 */
	__s32                      rtr_solicits;         /*    28     4 */
	__s32                      rtr_solicit_interval; /*    32     4 */
	__s32                      rtr_solicit_max_interval; /*    36     4 */
	__s32                      rtr_solicit_delay;    /*    40     4 */
	__s32                      force_mld_version;    /*    44     4 */
	__s32                      mldv1_unsolicited_report_interval; /*    48     4 */
	__s32                      mldv2_unsolicited_report_interval; /*    52     4 */
	__s32                      use_tempaddr;         /*    56     4 */
	__s32                      temp_valid_lft;       /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	__s32                      temp_prefered_lft;    /*    64     4 */
	__s32                      regen_max_retry;      /*    68     4 */
	__s32                      max_desync_factor;    /*    72     4 */
	__s32                      max_addresses;        /*    76     4 */
	__s32                      accept_ra_defrtr;     /*    80     4 */
	__s32                      accept_ra_min_hop_limit; /*    84     4 */
	__s32                      accept_ra_pinfo;      /*    88     4 */
	__s32                      ignore_routes_with_linkdown; /*    92     4 */
	__s32                      accept_ra_rtr_pref;   /*    96     4 */
	__s32                      rtr_probe_interval;   /*   100     4 */
	__s32                      accept_ra_rt_info_min_plen; /*   104     4 */
	__s32                      accept_ra_rt_info_max_plen; /*   108     4 */
	__s32                      proxy_ndp;            /*   112     4 */
	__s32                      accept_source_route;  /*   116     4 */
	__s32                      accept_ra_from_local; /*   120     4 */
	__s32                      optimistic_dad;       /*   124     4 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	__s32                      use_optimistic;       /*   128     4 */
	union {
		atomic_t           mc_forwarding;        /*   132     4 */
		struct {
			__s32      mc_forwarding;        /*   132     4 */
		} kabi_hidden_54;                        /*   132     4 */
		union {
		};                                       /*   132     0 */
	};                                               /*   132     4 */
	__s32                      disable_ipv6;         /*   136     4 */
	__s32                      drop_unicast_in_l2_multicast; /*   140     4 */
	__s32                      accept_dad;           /*   144     4 */
	__s32                      force_tllao;          /*   148     4 */
	__s32                      ndisc_notify;         /*   152     4 */
	__s32                      suppress_frag_ndisc;  /*   156     4 */
	__s32                      accept_ra_mtu;        /*   160     4 */
	__s32                      drop_unsolicited_na;  /*   164     4 */
	struct ipv6_stable_secret  stable_secret;        /*   168    20 */
	__s32                      use_oif_addrs_only;   /*   188     4 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	__s32                      keep_addr_on_down;    /*   192     4 */
	__s32                      seg6_enabled;         /*   196     4 */
	__u32                      enhanced_dad;         /*   200     4 */
	__u32                      addr_gen_mode;        /*   204     4 */
	__s32                      disable_policy;       /*   208     4 */
	__s32                      ndisc_tclass;         /*   212     4 */
	__s32                      rpl_seg_enabled;      /*   216     4 */

	/* XXX 4 bytes hole, try to pack */

	struct ctl_table_header *  sysctl_header;        /*   224     8 */
	u64                        kabi_reserved1;       /*   232     8 */
	u64                        kabi_reserved2;       /*   240     8 */
	u64                        kabi_reserved3;       /*   248     8 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	u64                        kabi_reserved4;       /*   256     8 */

	/* size: 264, cachelines: 5, members: 56 */
	/* sum members: 260, holes: 1, sum holes: 4 */
	/* last cacheline: 8 bytes */
};
struct netns_nf_frag {
	struct fqdir *             fqdir;                /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct netns_nexthop {
	struct rb_root             rb_root;              /*     0     8 */
	struct hlist_head *        devhash;              /*     8     8 */
	unsigned int               seq;                  /*    16     4 */
	u32                        last_id_allocated;    /*    20     4 */
	struct blocking_notifier_head notifier_chain;    /*    24    48 */

	/* size: 72, cachelines: 2, members: 5 */
	/* last cacheline: 8 bytes */
};
struct netns_sysctl_lowpan {
	struct ctl_table_header *  frags_hdr;            /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct netns_ieee802154_lowpan {
	struct netns_sysctl_lowpan sysctl;               /*     0     8 */
	struct fqdir *             fqdir;                /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct netns_sctp {
	struct sctp_mib *          sctp_statistics;      /*     0     8 */
	struct proc_dir_entry *    proc_net_sctp;        /*     8     8 */
	struct ctl_table_header *  sysctl_header;        /*    16     8 */
	struct sock *              ctl_sock;             /*    24     8 */
	struct list_head           local_addr_list;      /*    32    16 */
	struct list_head           addr_waitq;           /*    48    16 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct timer_list          addr_wq_timer;        /*    64    72 */
	/* --- cacheline 2 boundary (128 bytes) was 8 bytes ago --- */
	struct list_head           auto_asconf_splist;   /*   136    16 */
	spinlock_t                 addr_wq_lock;         /*   152     4 */
	spinlock_t                 local_addr_lock;      /*   156     4 */
	unsigned int               rto_initial;          /*   160     4 */
	unsigned int               rto_min;              /*   164     4 */
	unsigned int               rto_max;              /*   168     4 */
	int                        rto_alpha;            /*   172     4 */
	int                        rto_beta;             /*   176     4 */
	int                        max_burst;            /*   180     4 */
	int                        cookie_preserve_enable; /*   184     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 3 boundary (192 bytes) --- */
	char *                     sctp_hmac_alg;        /*   192     8 */
	unsigned int               valid_cookie_life;    /*   200     4 */
	unsigned int               sack_timeout;         /*   204     4 */
	unsigned int               hb_interval;          /*   208     4 */
	int                        max_retrans_association; /*   212     4 */
	int                        max_retrans_path;     /*   216     4 */
	int                        max_retrans_init;     /*   220     4 */
	int                        pf_retrans;           /*   224     4 */
	int                        ps_retrans;           /*   228     4 */
	int                        pf_enable;            /*   232     4 */
	int                        pf_expose;            /*   236     4 */
	int                        sndbuf_policy;        /*   240     4 */
	int                        rcvbuf_policy;        /*   244     4 */
	int                        default_auto_asconf;  /*   248     4 */
	int                        addip_enable;         /*   252     4 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	int                        addip_noauth;         /*   256     4 */
	int                        prsctp_enable;        /*   260     4 */
	int                        reconf_enable;        /*   264     4 */
	int                        auth_enable;          /*   268     4 */
	int                        intl_enable;          /*   272     4 */
	int                        ecn_enable;           /*   276     4 */
	int                        scope_policy;         /*   280     4 */
	int                        rwnd_upd_shift;       /*   284     4 */
	long unsigned int          max_autoclose;        /*   288     8 */

	/* size: 296, cachelines: 5, members: 41 */
	/* sum members: 292, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct netns_nf {
	struct proc_dir_entry *    proc_netfilter;       /*     0     8 */
	const struct nf_queue_handler  * queue_handler;  /*     8     8 */
	const struct nf_logger  *  nf_loggers[13];       /*    16   104 */
	/* --- cacheline 1 boundary (64 bytes) was 56 bytes ago --- */
	struct ctl_table_header *  nf_log_dir_header;    /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct nf_hook_entries *   hooks_ipv4[5];        /*   128    40 */
	struct nf_hook_entries *   hooks_ipv6[5];        /*   168    40 */
	/* --- cacheline 3 boundary (192 bytes) was 16 bytes ago --- */
	struct nf_hook_entries *   hooks_arp[3];         /*   208    24 */
	struct nf_hook_entries *   hooks_bridge[5];      /*   232    40 */
	/* --- cacheline 4 boundary (256 bytes) was 16 bytes ago --- */
	bool                       defrag_ipv4;          /*   272     1 */
	bool                       defrag_ipv6;          /*   273     1 */

	/* XXX 6 bytes hole, try to pack */

	u64                        kabi_reserved1;       /*   280     8 */

	/* size: 288, cachelines: 5, members: 11 */
	/* sum members: 282, holes: 1, sum holes: 6 */
	/* last cacheline: 32 bytes */
};
struct netns_xt {
	struct list_head           tables[13];           /*     0   208 */
	/* --- cacheline 3 boundary (192 bytes) was 16 bytes ago --- */
	bool                       notrack_deprecated_warning; /*   208     1 */
	bool                       clusterip_deprecated_warning; /*   209     1 */

	/* XXX 6 bytes hole, try to pack */

	struct ebt_table *         broute_table;         /*   216     8 */
	struct ebt_table *         frame_filter;         /*   224     8 */
	struct ebt_table *         frame_nat;            /*   232     8 */

	/* size: 240, cachelines: 4, members: 6 */
	/* sum members: 234, holes: 1, sum holes: 6 */
	/* last cacheline: 48 bytes */
};
struct hlist_nulls_head {
	struct hlist_nulls_node *  first;                /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct hlist_nulls_node {
	struct hlist_nulls_node *  next;                 /*     0     8 */
	struct hlist_nulls_node * * pprev;               /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct ip_conntrack_stat {
	unsigned int               found;                /*     0     4 */
	unsigned int               invalid;              /*     4     4 */
	unsigned int               insert;               /*     8     4 */
	unsigned int               insert_failed;        /*    12     4 */
	unsigned int               clash_resolve;        /*    16     4 */
	unsigned int               drop;                 /*    20     4 */
	unsigned int               early_drop;           /*    24     4 */
	unsigned int               error;                /*    28     4 */
	unsigned int               expect_new;           /*    32     4 */
	unsigned int               expect_create;        /*    36     4 */
	unsigned int               expect_delete;        /*    40     4 */
	unsigned int               search_restart;       /*    44     4 */

	/* size: 48, cachelines: 1, members: 12 */
	/* last cacheline: 48 bytes */
};
struct nf_generic_net {
	unsigned int               timeout;              /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct nf_tcp_net {
	unsigned int               timeouts[14];         /*     0    56 */
	int                        tcp_loose;            /*    56     4 */
	int                        tcp_be_liberal;       /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        tcp_max_retrans;      /*    64     4 */

	/* size: 68, cachelines: 2, members: 4 */
	/* last cacheline: 4 bytes */
};
struct nf_udp_net {
	unsigned int               timeouts[2];          /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct nf_icmp_net {
	unsigned int               timeout;              /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct nf_dccp_net {
	int                        dccp_loose;           /*     0     4 */
	unsigned int               dccp_timeout[10];     /*     4    40 */

	/* size: 44, cachelines: 1, members: 2 */
	/* last cacheline: 44 bytes */
};
struct nf_sctp_net {
	unsigned int               timeouts[10];         /*     0    40 */

	/* size: 40, cachelines: 1, members: 1 */
	/* last cacheline: 40 bytes */
};
struct nf_gre_net {
	struct list_head           keymap_list;          /*     0    16 */
	unsigned int               timeouts[2];          /*    16     8 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct nf_ip_net {
	struct nf_generic_net      generic;              /*     0     4 */
	struct nf_tcp_net          tcp;                  /*     4    68 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	struct nf_udp_net          udp;                  /*    72     8 */
	struct nf_icmp_net         icmp;                 /*    80     4 */
	struct nf_icmp_net         icmpv6;               /*    84     4 */
	struct nf_dccp_net         dccp;                 /*    88    44 */
	/* --- cacheline 2 boundary (128 bytes) was 4 bytes ago --- */
	struct nf_sctp_net         sctp;                 /*   132    40 */

	/* XXX 4 bytes hole, try to pack */

	struct nf_gre_net          gre;                  /*   176    24 */

	/* size: 200, cachelines: 4, members: 8 */
	/* sum members: 196, holes: 1, sum holes: 4 */
	/* last cacheline: 8 bytes */
};
struct ct_pcpu {
	spinlock_t                 lock;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct hlist_nulls_head    unconfirmed;          /*     8     8 */
	struct hlist_nulls_head    dying;                /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct netns_ct {
	atomic_t                   count;                /*     0     4 */
	unsigned int               expect_count;         /*     4     4 */
	struct delayed_work        ecache_dwork;         /*     8   192 */
	/* --- cacheline 3 boundary (192 bytes) was 8 bytes ago --- */
	bool                       ecache_dwork_pending; /*   200     1 */
	bool                       auto_assign_helper_warned; /*   201     1 */

	/* XXX 6 bytes hole, try to pack */

	struct ctl_table_header *  sysctl_header;        /*   208     8 */
	unsigned int               sysctl_log_invalid;   /*   216     4 */
	int                        sysctl_events;        /*   220     4 */
	int                        sysctl_acct;          /*   224     4 */
	int                        sysctl_auto_assign_helper; /*   228     4 */
	int                        sysctl_tstamp;        /*   232     4 */
	int                        sysctl_checksum;      /*   236     4 */
	struct ct_pcpu *           pcpu_lists;           /*   240     8 */
	struct ip_conntrack_stat * stat;                 /*   248     8 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	struct nf_ct_event_notifier * nf_conntrack_event_cb; /*   256     8 */
	struct nf_exp_event_notifier * nf_expect_event_cb; /*   264     8 */
	struct nf_ip_net           nf_ct_proto;          /*   272   200 */
	/* --- cacheline 7 boundary (448 bytes) was 24 bytes ago --- */
	unsigned int               labels_used;          /*   472     4 */

	/* size: 480, cachelines: 8, members: 18 */
	/* sum members: 470, holes: 1, sum holes: 6 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct netns_nftables {
	struct list_head           tables;               /*     0    16 */
	struct list_head           commit_list;          /*    16    16 */
	struct list_head           module_list;          /*    32    16 */
	struct list_head           notify_list;          /*    48    16 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct mutex               commit_mutex;         /*    64    32 */
	unsigned int               base_seq;             /*    96     4 */
	u8                         gencursor;            /*   100     1 */
	u8                         validate_state;       /*   101     1 */

	/* XXX 2 bytes hole, try to pack */

	u64                        kabi_reserved1;       /*   104     8 */

	/* size: 112, cachelines: 2, members: 9 */
	/* sum members: 110, holes: 1, sum holes: 2 */
	/* last cacheline: 48 bytes */
};
struct xfrm_policy_hash {
	struct hlist_head *        table;                /*     0     8 */
	unsigned int               hmask;                /*     8     4 */
	u8                         dbits4;               /*    12     1 */
	u8                         sbits4;               /*    13     1 */
	u8                         dbits6;               /*    14     1 */
	u8                         sbits6;               /*    15     1 */

	/* size: 16, cachelines: 1, members: 6 */
	/* last cacheline: 16 bytes */
};
struct xfrm_policy_hthresh {
	struct work_struct         work;                 /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	seqlock_t                  lock;                 /*    64     8 */
	u8                         lbits4;               /*    72     1 */
	u8                         rbits4;               /*    73     1 */
	u8                         lbits6;               /*    74     1 */
	u8                         rbits6;               /*    75     1 */

	/* size: 80, cachelines: 2, members: 6 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct netns_xfrm {
	struct list_head           state_all;            /*     0    16 */
	struct hlist_head *        state_bydst;          /*    16     8 */
	struct hlist_head *        state_bysrc;          /*    24     8 */
	struct hlist_head *        state_byspi;          /*    32     8 */
	unsigned int               state_hmask;          /*    40     4 */
	unsigned int               state_num;            /*    44     4 */
	struct work_struct         state_hash_work;      /*    48    64 */
	/* --- cacheline 1 boundary (64 bytes) was 48 bytes ago --- */
	struct list_head           policy_all;           /*   112    16 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct hlist_head *        policy_byidx;         /*   128     8 */
	unsigned int               policy_idx_hmask;     /*   136     4 */

	/* XXX 4 bytes hole, try to pack */

	struct hlist_head          policy_inexact[3];    /*   144    24 */
	struct xfrm_policy_hash    policy_bydst[3];      /*   168    48 */
	/* --- cacheline 3 boundary (192 bytes) was 24 bytes ago --- */
	unsigned int               policy_count[6];      /*   216    24 */
	struct work_struct         policy_hash_work;     /*   240    64 */
	/* --- cacheline 4 boundary (256 bytes) was 48 bytes ago --- */
	struct xfrm_policy_hthresh policy_hthresh;       /*   304    80 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 6 boundary (384 bytes) --- */
	struct list_head           inexact_bins;         /*   384    16 */
	struct sock *              nlsk;                 /*   400     8 */
	struct sock *              nlsk_stash;           /*   408     8 */
	u32                        sysctl_aevent_etime;  /*   416     4 */
	u32                        sysctl_aevent_rseqth; /*   420     4 */
	int                        sysctl_larval_drop;   /*   424     4 */
	u32                        sysctl_acq_expires;   /*   428     4 */
	struct ctl_table_header *  sysctl_hdr;           /*   432     8 */

	/* XXX 8 bytes hole, try to pack */

	/* --- cacheline 7 boundary (448 bytes) --- */
	struct dst_ops             xfrm4_dst_ops __attribute__((__aligned__(64))); /*   448   256 */

	/* XXX last struct has 24 bytes of padding */

	/* --- cacheline 11 boundary (704 bytes) --- */
	struct dst_ops             xfrm6_dst_ops __attribute__((__aligned__(64))); /*   704   256 */

	/* XXX last struct has 24 bytes of padding */

	/* --- cacheline 15 boundary (960 bytes) --- */
	spinlock_t                 xfrm_state_lock;      /*   960     4 */
	seqcount_t                 xfrm_state_hash_generation; /*   964     4 */
	seqcount_spinlock_t        xfrm_policy_hash_generation; /*   968     4 */
	spinlock_t                 xfrm_policy_lock;     /*   972     4 */
	struct mutex               xfrm_cfg_mutex;       /*   976    32 */
	u64                        kabi_reserved1;       /*  1008     8 */

	/* size: 1024, cachelines: 16, members: 31 */
	/* sum members: 1004, holes: 2, sum holes: 12 */
	/* padding: 8 */
	/* paddings: 3, sum paddings: 52 */
	/* forced alignments: 2, forced holes: 1, sum forced holes: 8 */
} __attribute__((__aligned__(64)));
struct netns_mpls {
	int                        ip_ttl_propagate;     /*     0     4 */
	int                        default_ttl;          /*     4     4 */
	size_t                     platform_labels;      /*     8     8 */
	struct mpls_route * *      platform_label;       /*    16     8 */
	struct ctl_table_header *  ctl;                  /*    24     8 */

	/* size: 32, cachelines: 1, members: 5 */
	/* last cacheline: 32 bytes */
};
struct netns_can {
	struct proc_dir_entry *    proc_dir;             /*     0     8 */
	struct proc_dir_entry *    pde_stats;            /*     8     8 */
	struct proc_dir_entry *    pde_reset_stats;      /*    16     8 */
	struct proc_dir_entry *    pde_rcvlist_all;      /*    24     8 */
	struct proc_dir_entry *    pde_rcvlist_fil;      /*    32     8 */
	struct proc_dir_entry *    pde_rcvlist_inv;      /*    40     8 */
	struct proc_dir_entry *    pde_rcvlist_sff;      /*    48     8 */
	struct proc_dir_entry *    pde_rcvlist_eff;      /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct proc_dir_entry *    pde_rcvlist_err;      /*    64     8 */
	struct proc_dir_entry *    bcmproc_dir;          /*    72     8 */
	struct can_dev_rcv_lists * rx_alldev_list;       /*    80     8 */
	spinlock_t                 rcvlists_lock;        /*    88     4 */

	/* XXX 4 bytes hole, try to pack */

	struct timer_list          stattimer;            /*    96    72 */
	/* --- cacheline 2 boundary (128 bytes) was 40 bytes ago --- */
	struct can_pkg_stats *     pkg_stats;            /*   168     8 */
	struct can_rcv_lists_stats * rcv_lists_stats;    /*   176     8 */
	struct hlist_head          cgw_list;             /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	u64                        kabi_reserved1;       /*   192     8 */

	/* size: 200, cachelines: 4, members: 17 */
	/* sum members: 196, holes: 1, sum holes: 4 */
	/* last cacheline: 8 bytes */
};
struct netns_xdp {
	struct mutex               lock;                 /*     0    32 */
	struct hlist_head          list;                 /*    32     8 */

	/* size: 40, cachelines: 1, members: 2 */
	/* last cacheline: 40 bytes */
};
struct netns_bpf {
	struct bpf_prog_array *    run_array[2];         /*     0    16 */
	struct bpf_prog *          progs[2];             /*    16    16 */
	struct list_head           links[2];             /*    32    32 */

	/* size: 64, cachelines: 1, members: 3 */
};
struct socket_wq {
	wait_queue_head_t          wait;                 /*     0    24 */
	struct fasync_struct *     fasync_list;          /*    24     8 */
	long unsigned int          flags;                /*    32     8 */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*    40    16 */

	/* size: 64, cachelines: 1, members: 4 */
	/* padding: 8 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(64)));
struct socket {
	socket_state               state;                /*     0     4 */
	short int                  type;                 /*     4     2 */

	/* XXX 2 bytes hole, try to pack */

	long unsigned int          flags;                /*     8     8 */
	struct file *              file;                 /*    16     8 */
	struct sock *              sk;                   /*    24     8 */
	const struct proto_ops  *  ops;                  /*    32     8 */

	/* XXX 24 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	struct socket_wq           wq __attribute__((__aligned__(64))); /*    64    64 */

	/* XXX last struct has 8 bytes of padding */

	/* size: 128, cachelines: 2, members: 7 */
	/* sum members: 102, holes: 2, sum holes: 26 */
	/* paddings: 1, sum paddings: 8 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 24 */
} __attribute__((__aligned__(64)));
struct proto_ops {
	int                        family;               /*     0     4 */
	unsigned int               flags;                /*     4     4 */
	struct module *            owner;                /*     8     8 */
	int                        (*release)(struct socket *); /*    16     8 */
	int                        (*bind)(struct socket *, struct sockaddr *, int); /*    24     8 */
	int                        (*connect)(struct socket *, struct sockaddr *, int, int); /*    32     8 */
	int                        (*socketpair)(struct socket *, struct socket *); /*    40     8 */
	int                        (*accept)(struct socket *, struct socket *, int, bool); /*    48     8 */
	int                        (*getname)(struct socket *, struct sockaddr *, int); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	__poll_t                   (*poll)(struct file *, struct socket *, struct poll_table_struct *); /*    64     8 */
	int                        (*ioctl)(struct socket *, unsigned int, long unsigned int); /*    72     8 */
	int                        (*compat_ioctl)(struct socket *, unsigned int, long unsigned int); /*    80     8 */
	int                        (*gettstamp)(struct socket *, void *, bool, bool); /*    88     8 */
	int                        (*listen)(struct socket *, int); /*    96     8 */
	int                        (*shutdown)(struct socket *, int); /*   104     8 */
	int                        (*setsockopt)(struct socket *, int, int, sockptr_t, unsigned int); /*   112     8 */
	int                        (*getsockopt)(struct socket *, int, int, char *, int *); /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	void                       (*show_fdinfo)(struct seq_file *, struct socket *); /*   128     8 */
	int                        (*sendmsg)(struct socket *, struct msghdr *, size_t); /*   136     8 */
	int                        (*recvmsg)(struct socket *, struct msghdr *, size_t, int); /*   144     8 */
	int                        (*mmap)(struct file *, struct socket *, struct vm_area_struct *); /*   152     8 */
	ssize_t                    (*sendpage)(struct socket *, struct page *, int, size_t, int); /*   160     8 */
	ssize_t                    (*splice_read)(struct socket *, loff_t *, struct pipe_inode_info *, size_t, unsigned int); /*   168     8 */
	int                        (*set_peek_off)(struct sock *, int); /*   176     8 */
	int                        (*peek_len)(struct socket *); /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	int                        (*read_sock)(struct sock *, read_descriptor_t *, sk_read_actor_t); /*   192     8 */
	int                        (*sendpage_locked)(struct sock *, struct page *, int, size_t, int); /*   200     8 */
	int                        (*sendmsg_locked)(struct sock *, struct msghdr *, size_t); /*   208     8 */
	int                        (*set_rcvlowat)(struct sock *, int); /*   216     8 */
	u64                        kabi_reserved1;       /*   224     8 */
	u64                        kabi_reserved2;       /*   232     8 */
	u64                        kabi_reserved3;       /*   240     8 */
	u64                        kabi_reserved4;       /*   248     8 */

	/* size: 256, cachelines: 4, members: 33 */
};
struct pipe_buffer {
	struct page *              page;                 /*     0     8 */
	unsigned int               offset;               /*     8     4 */
	unsigned int               len;                  /*    12     4 */
	const struct pipe_buf_operations  * ops;         /*    16     8 */
	unsigned int               flags;                /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          private;              /*    32     8 */

	/* size: 40, cachelines: 1, members: 6 */
	/* sum members: 36, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct pipe_buf_operations {
	int                        (*confirm)(struct pipe_inode_info *, struct pipe_buffer *); /*     0     8 */
	void                       (*release)(struct pipe_inode_info *, struct pipe_buffer *); /*     8     8 */
	bool                       (*try_steal)(struct pipe_inode_info *, struct pipe_buffer *); /*    16     8 */
	bool                       (*get)(struct pipe_inode_info *, struct pipe_buffer *); /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct ptr_ring {
	int                        producer __attribute__((__aligned__(64))); /*     0     4 */
	spinlock_t                 producer_lock;        /*     4     4 */

	/* XXX 56 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        consumer_head __attribute__((__aligned__(64))); /*    64     4 */
	int                        consumer_tail;        /*    68     4 */
	spinlock_t                 consumer_lock;        /*    72     4 */

	/* XXX 52 bytes hole, try to pack */

	/* --- cacheline 2 boundary (128 bytes) --- */
	int                        size __attribute__((__aligned__(64))); /*   128     4 */
	int                        batch;                /*   132     4 */
	void * *                   queue;                /*   136     8 */

	/* size: 192, cachelines: 3, members: 8 */
	/* sum members: 36, holes: 2, sum holes: 108 */
	/* padding: 48 */
	/* forced alignments: 3, forced holes: 2, sum forced holes: 108 */
} __attribute__((__aligned__(64)));
struct pp_alloc_cache {
	u32                        count;                /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct page *              cache[128];           /*     8  1024 */

	/* size: 1032, cachelines: 17, members: 2 */
	/* sum members: 1028, holes: 1, sum holes: 4 */
	/* last cacheline: 8 bytes */
};
struct page_pool_params {
	unsigned int               flags;                /*     0     4 */
	unsigned int               order;                /*     4     4 */
	unsigned int               pool_size;            /*     8     4 */
	int                        nid;                  /*    12     4 */
	struct device *            dev;                  /*    16     8 */
	enum dma_data_direction    dma_dir;              /*    24     4 */
	unsigned int               max_len;              /*    28     4 */
	unsigned int               offset;               /*    32     4 */

	/* size: 40, cachelines: 1, members: 8 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct sk_buff_head {
	struct sk_buff *           next;                 /*     0     8 */
	struct sk_buff *           prev;                 /*     8     8 */
	__u32                      qlen;                 /*    16     4 */
	spinlock_t                 lock;                 /*    20     4 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct skb_ext {
	refcount_t                 refcnt;               /*     0     4 */
	u8                         offset[2];            /*     4     2 */
	u8                         chunks;               /*     6     1 */

	/* XXX 1 byte hole, try to pack */

	char                       data[] __attribute__((__aligned__(8))); /*     8     0 */

	/* size: 8, cachelines: 1, members: 4 */
	/* sum members: 7, holes: 1, sum holes: 1 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 1 */
	/* last cacheline: 8 bytes */
} __attribute__((__aligned__(8)));
struct ipv6_stable_secret {
	bool                       initialized;          /*     0     1 */

	/* XXX 3 bytes hole, try to pack */

	struct in6_addr            secret;               /*     4    16 */

	/* size: 20, cachelines: 1, members: 2 */
	/* sum members: 17, holes: 1, sum holes: 3 */
	/* last cacheline: 20 bytes */
};
struct dql {
	unsigned int               num_queued;           /*     0     4 */
	unsigned int               adj_limit;            /*     4     4 */
	unsigned int               last_obj_cnt;         /*     8     4 */

	/* XXX 52 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	unsigned int               limit __attribute__((__aligned__(64))); /*    64     4 */
	unsigned int               num_completed;        /*    68     4 */
	unsigned int               prev_ovlimit;         /*    72     4 */
	unsigned int               prev_num_queued;      /*    76     4 */
	unsigned int               prev_last_obj_cnt;    /*    80     4 */
	unsigned int               lowest_slack;         /*    84     4 */
	long unsigned int          slack_start_time;     /*    88     8 */
	unsigned int               max_limit;            /*    96     4 */
	unsigned int               min_limit;            /*   100     4 */
	unsigned int               slack_hold_time;      /*   104     4 */

	/* size: 128, cachelines: 2, members: 13 */
	/* sum members: 56, holes: 1, sum holes: 52 */
	/* padding: 20 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 52 */
} __attribute__((__aligned__(64)));
struct nlmsghdr {
	__u32                      nlmsg_len;            /*     0     4 */
	__u16                      nlmsg_type;           /*     4     2 */
	__u16                      nlmsg_flags;          /*     6     2 */
	__u32                      nlmsg_seq;            /*     8     4 */
	__u32                      nlmsg_pid;            /*    12     4 */

	/* size: 16, cachelines: 1, members: 5 */
	/* last cacheline: 16 bytes */
};
struct nlattr {
	__u16                      nla_len;              /*     0     2 */
	__u16                      nla_type;             /*     2     2 */

	/* size: 4, cachelines: 1, members: 2 */
	/* last cacheline: 4 bytes */
};
struct netlink_ext_ack {
	const char  *              _msg;                 /*     0     8 */
	const struct nlattr  *     bad_attr;             /*     8     8 */
	const struct nla_policy  * policy;               /*    16     8 */
	u8                         cookie[20];           /*    24    20 */
	u8                         cookie_len;           /*    44     1 */

	/* size: 48, cachelines: 1, members: 5 */
	/* padding: 3 */
	/* last cacheline: 48 bytes */
};
struct nla_policy {
	u8                         type;                 /*     0     1 */
	u8                         validation_type;      /*     1     1 */
	u16                        len;                  /*     2     2 */

	/* XXX 4 bytes hole, try to pack */

	union {
		const u32          bitfield32_valid;     /*     8     4 */
		const u32          mask;                 /*     8     4 */
		const char  *      reject_message;       /*     8     8 */
		const struct nla_policy  * nested_policy; /*     8     8 */
		struct netlink_range_validation * range; /*     8     8 */
		struct netlink_range_validation_signed * range_signed; /*     8     8 */
		struct {
			s16        min;                  /*     8     2 */
			s16        max;                  /*    10     2 */
		};                                       /*     8     4 */
		int                (*validate)(const struct nlattr  *, struct netlink_ext_ack *); /*     8     8 */
		u16                strict_start_type;    /*     8     2 */
	};                                               /*     8     8 */
	u64                        kabi_reserved1;       /*    16     8 */
	u64                        kabi_reserved2;       /*    24     8 */
	u64                        kabi_reserved3;       /*    32     8 */

	/* size: 40, cachelines: 1, members: 7 */
	/* sum members: 36, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct netlink_callback {
	struct sk_buff *           skb;                  /*     0     8 */
	const struct nlmsghdr  *   nlh;                  /*     8     8 */
	int                        (*dump)(struct sk_buff *, struct netlink_callback *); /*    16     8 */
	int                        (*done)(struct netlink_callback *); /*    24     8 */
	void *                     data;                 /*    32     8 */
	struct module *            module;               /*    40     8 */
	struct netlink_ext_ack *   extack;               /*    48     8 */
	u16                        family;               /*    56     2 */
	u16                        answer_flags;         /*    58     2 */
	u32                        min_dump_alloc;       /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	unsigned int               prev_seq;             /*    64     4 */
	unsigned int               seq;                  /*    68     4 */
	bool                       strict_check;         /*    72     1 */

	/* XXX 7 bytes hole, try to pack */

	union {
		u8                 ctx[48];              /*    80    48 */
		long int           args[6];              /*    80    48 */
	};                                               /*    80    48 */

	/* size: 128, cachelines: 2, members: 14 */
	/* sum members: 121, holes: 1, sum holes: 7 */
};
struct ethtool_drvinfo {
	__u32                      cmd;                  /*     0     4 */
	char                       driver[32];           /*     4    32 */
	char                       version[32];          /*    36    32 */
	/* --- cacheline 1 boundary (64 bytes) was 4 bytes ago --- */
	char                       fw_version[32];       /*    68    32 */
	char                       bus_info[32];         /*   100    32 */
	/* --- cacheline 2 boundary (128 bytes) was 4 bytes ago --- */
	char                       erom_version[32];     /*   132    32 */
	char                       reserved2[12];        /*   164    12 */
	__u32                      n_priv_flags;         /*   176     4 */
	__u32                      n_stats;              /*   180     4 */
	__u32                      testinfo_len;         /*   184     4 */
	__u32                      eedump_len;           /*   188     4 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	__u32                      regdump_len;          /*   192     4 */

	/* size: 196, cachelines: 4, members: 12 */
	/* last cacheline: 4 bytes */
};
struct ethtool_wolinfo {
	__u32                      cmd;                  /*     0     4 */
	__u32                      supported;            /*     4     4 */
	__u32                      wolopts;              /*     8     4 */
	__u8                       sopass[6];            /*    12     6 */

	/* size: 20, cachelines: 1, members: 4 */
	/* padding: 2 */
	/* last cacheline: 20 bytes */
};
struct ethtool_tunable {
	__u32                      cmd;                  /*     0     4 */
	__u32                      id;                   /*     4     4 */
	__u32                      type_id;              /*     8     4 */
	__u32                      len;                  /*    12     4 */
	void *                     data[];               /*    16     0 */

	/* size: 16, cachelines: 1, members: 5 */
	/* last cacheline: 16 bytes */
};
struct ethtool_regs {
	__u32                      cmd;                  /*     0     4 */
	__u32                      version;              /*     4     4 */
	__u32                      len;                  /*     8     4 */
	__u8                       data[];               /*    12     0 */

	/* size: 12, cachelines: 1, members: 4 */
	/* last cacheline: 12 bytes */
};
struct ethtool_eeprom {
	__u32                      cmd;                  /*     0     4 */
	__u32                      magic;                /*     4     4 */
	__u32                      offset;               /*     8     4 */
	__u32                      len;                  /*    12     4 */
	__u8                       data[];               /*    16     0 */

	/* size: 16, cachelines: 1, members: 5 */
	/* last cacheline: 16 bytes */
};
struct ethtool_eee {
	__u32                      cmd;                  /*     0     4 */
	__u32                      supported;            /*     4     4 */
	__u32                      advertised;           /*     8     4 */
	__u32                      lp_advertised;        /*    12     4 */
	__u32                      eee_active;           /*    16     4 */
	__u32                      eee_enabled;          /*    20     4 */
	__u32                      tx_lpi_enabled;       /*    24     4 */
	__u32                      tx_lpi_timer;         /*    28     4 */
	__u32                      reserved[2];          /*    32     8 */

	/* size: 40, cachelines: 1, members: 9 */
	/* last cacheline: 40 bytes */
};
struct ethtool_modinfo {
	__u32                      cmd;                  /*     0     4 */
	__u32                      type;                 /*     4     4 */
	__u32                      eeprom_len;           /*     8     4 */
	__u32                      reserved[8];          /*    12    32 */

	/* size: 44, cachelines: 1, members: 4 */
	/* last cacheline: 44 bytes */
};
struct ethtool_coalesce {
	__u32                      cmd;                  /*     0     4 */
	__u32                      rx_coalesce_usecs;    /*     4     4 */
	__u32                      rx_max_coalesced_frames; /*     8     4 */
	__u32                      rx_coalesce_usecs_irq; /*    12     4 */
	__u32                      rx_max_coalesced_frames_irq; /*    16     4 */
	__u32                      tx_coalesce_usecs;    /*    20     4 */
	__u32                      tx_max_coalesced_frames; /*    24     4 */
	__u32                      tx_coalesce_usecs_irq; /*    28     4 */
	__u32                      tx_max_coalesced_frames_irq; /*    32     4 */
	__u32                      stats_block_coalesce_usecs; /*    36     4 */
	__u32                      use_adaptive_rx_coalesce; /*    40     4 */
	__u32                      use_adaptive_tx_coalesce; /*    44     4 */
	__u32                      pkt_rate_low;         /*    48     4 */
	__u32                      rx_coalesce_usecs_low; /*    52     4 */
	__u32                      rx_max_coalesced_frames_low; /*    56     4 */
	__u32                      tx_coalesce_usecs_low; /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	__u32                      tx_max_coalesced_frames_low; /*    64     4 */
	__u32                      pkt_rate_high;        /*    68     4 */
	__u32                      rx_coalesce_usecs_high; /*    72     4 */
	__u32                      rx_max_coalesced_frames_high; /*    76     4 */
	__u32                      tx_coalesce_usecs_high; /*    80     4 */
	__u32                      tx_max_coalesced_frames_high; /*    84     4 */
	__u32                      rate_sample_interval; /*    88     4 */

	/* size: 92, cachelines: 2, members: 23 */
	/* last cacheline: 28 bytes */
};
struct ethtool_ringparam {
	__u32                      cmd;                  /*     0     4 */
	__u32                      rx_max_pending;       /*     4     4 */
	__u32                      rx_mini_max_pending;  /*     8     4 */
	__u32                      rx_jumbo_max_pending; /*    12     4 */
	__u32                      tx_max_pending;       /*    16     4 */
	__u32                      rx_pending;           /*    20     4 */
	__u32                      rx_mini_pending;      /*    24     4 */
	__u32                      rx_jumbo_pending;     /*    28     4 */
	__u32                      tx_pending;           /*    32     4 */

	/* size: 36, cachelines: 1, members: 9 */
	/* last cacheline: 36 bytes */
};
struct ethtool_channels {
	__u32                      cmd;                  /*     0     4 */
	__u32                      max_rx;               /*     4     4 */
	__u32                      max_tx;               /*     8     4 */
	__u32                      max_other;            /*    12     4 */
	__u32                      max_combined;         /*    16     4 */
	__u32                      rx_count;             /*    20     4 */
	__u32                      tx_count;             /*    24     4 */
	__u32                      other_count;          /*    28     4 */
	__u32                      combined_count;       /*    32     4 */

	/* size: 36, cachelines: 1, members: 9 */
	/* last cacheline: 36 bytes */
};
struct ethtool_pauseparam {
	__u32                      cmd;                  /*     0     4 */
	__u32                      autoneg;              /*     4     4 */
	__u32                      rx_pause;             /*     8     4 */
	__u32                      tx_pause;             /*    12     4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct ethtool_test {
	__u32                      cmd;                  /*     0     4 */
	__u32                      flags;                /*     4     4 */
	__u32                      reserved;             /*     8     4 */
	__u32                      len;                  /*    12     4 */
	__u64                      data[];               /*    16     0 */

	/* size: 16, cachelines: 1, members: 5 */
	/* last cacheline: 16 bytes */
};
struct ethtool_stats {
	__u32                      cmd;                  /*     0     4 */
	__u32                      n_stats;              /*     4     4 */
	__u64                      data[];               /*     8     0 */

	/* size: 8, cachelines: 1, members: 3 */
	/* last cacheline: 8 bytes */
};
struct ethtool_tcpip4_spec {
	__be32                     ip4src;               /*     0     4 */
	__be32                     ip4dst;               /*     4     4 */
	__be16                     psrc;                 /*     8     2 */
	__be16                     pdst;                 /*    10     2 */
	__u8                       tos;                  /*    12     1 */

	/* size: 16, cachelines: 1, members: 5 */
	/* padding: 3 */
	/* last cacheline: 16 bytes */
};
struct ethtool_ah_espip4_spec {
	__be32                     ip4src;               /*     0     4 */
	__be32                     ip4dst;               /*     4     4 */
	__be32                     spi;                  /*     8     4 */
	__u8                       tos;                  /*    12     1 */

	/* size: 16, cachelines: 1, members: 4 */
	/* padding: 3 */
	/* last cacheline: 16 bytes */
};
struct ethtool_usrip4_spec {
	__be32                     ip4src;               /*     0     4 */
	__be32                     ip4dst;               /*     4     4 */
	__be32                     l4_4_bytes;           /*     8     4 */
	__u8                       tos;                  /*    12     1 */
	__u8                       ip_ver;               /*    13     1 */
	__u8                       proto;                /*    14     1 */

	/* size: 16, cachelines: 1, members: 6 */
	/* padding: 1 */
	/* last cacheline: 16 bytes */
};
struct ethtool_vxlan4_spec {
	__be32                     vni;                  /*     0     4 */
	__u8                       dst[6];               /*     4     6 */
	__u8                       src[6];               /*    10     6 */
	__be16                     eth_type;             /*    16     2 */
	__u8                       tos;                  /*    18     1 */
	__u8                       l4_proto;             /*    19     1 */
	__be32                     ip4src;               /*    20     4 */
	__be32                     ip4dst;               /*    24     4 */

	/* size: 28, cachelines: 1, members: 8 */
	/* last cacheline: 28 bytes */
};
struct ethtool_tcpip6_spec {
	__be32                     ip6src[4];            /*     0    16 */
	__be32                     ip6dst[4];            /*    16    16 */
	__be16                     psrc;                 /*    32     2 */
	__be16                     pdst;                 /*    34     2 */
	__u8                       tclass;               /*    36     1 */

	/* size: 40, cachelines: 1, members: 5 */
	/* padding: 3 */
	/* last cacheline: 40 bytes */
};
struct ethtool_ah_espip6_spec {
	__be32                     ip6src[4];            /*     0    16 */
	__be32                     ip6dst[4];            /*    16    16 */
	__be32                     spi;                  /*    32     4 */
	__u8                       tclass;               /*    36     1 */

	/* size: 40, cachelines: 1, members: 4 */
	/* padding: 3 */
	/* last cacheline: 40 bytes */
};
struct ethtool_usrip6_spec {
	__be32                     ip6src[4];            /*     0    16 */
	__be32                     ip6dst[4];            /*    16    16 */
	__be32                     l4_4_bytes;           /*    32     4 */
	__u8                       tclass;               /*    36     1 */
	__u8                       l4_proto;             /*    37     1 */

	/* size: 40, cachelines: 1, members: 5 */
	/* padding: 2 */
	/* last cacheline: 40 bytes */
};
struct ethtool_vxlan6_spec {
	__be32                     vni;                  /*     0     4 */
	__u8                       dst[6];               /*     4     6 */
	__u8                       src[6];               /*    10     6 */
	__be16                     eth_type;             /*    16     2 */
	__u8                       tclass;               /*    18     1 */
	__u8                       l4_proto;             /*    19     1 */
	__be32                     ip6src[4];            /*    20    16 */
	__be32                     ip6dst[4];            /*    36    16 */

	/* size: 52, cachelines: 1, members: 8 */
	/* last cacheline: 52 bytes */
};
union ethtool_flow_union {
	struct ethtool_tcpip4_spec tcp_ip4_spec;       /*     0    16 */
	struct ethtool_tcpip4_spec udp_ip4_spec;       /*     0    16 */
	struct ethtool_tcpip4_spec sctp_ip4_spec;      /*     0    16 */
	struct ethtool_ah_espip4_spec ah_ip4_spec;     /*     0    16 */
	struct ethtool_ah_espip4_spec esp_ip4_spec;    /*     0    16 */
	struct ethtool_usrip4_spec usr_ip4_spec;       /*     0    16 */
	struct ethtool_tcpip6_spec tcp_ip6_spec;       /*     0    40 */
	struct ethtool_tcpip6_spec udp_ip6_spec;       /*     0    40 */
	struct ethtool_tcpip6_spec sctp_ip6_spec;      /*     0    40 */
	struct ethtool_ah_espip6_spec ah_ip6_spec;     /*     0    40 */
	struct ethtool_ah_espip6_spec esp_ip6_spec;    /*     0    40 */
	struct ethtool_usrip6_spec usr_ip6_spec;       /*     0    40 */
	struct ethhdr              ether_spec;         /*     0    14 */
	__u8                       hdata[52];          /*     0    52 */
	struct ethtool_vxlan4_spec vxlan_ip4_spec;     /*     0    28 */
	struct ethtool_vxlan6_spec vxlan_ip6_spec;     /*     0    52 */
};
struct ethtool_flow_ext {
	__u8                       padding[2];           /*     0     2 */
	unsigned char              h_dest[6];            /*     2     6 */
	__be16                     vlan_etype;           /*     8     2 */
	__be16                     vlan_tci;             /*    10     2 */
	__be32                     data[2];              /*    12     8 */

	/* size: 20, cachelines: 1, members: 5 */
	/* last cacheline: 20 bytes */
};
struct ethtool_rx_flow_spec {
	__u32                      flow_type;            /*     0     4 */
	union ethtool_flow_union   h_u;                  /*     4    52 */
	struct ethtool_flow_ext    h_ext;                /*    56    20 */
	/* --- cacheline 1 boundary (64 bytes) was 12 bytes ago --- */
	union ethtool_flow_union   m_u;                  /*    76    52 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct ethtool_flow_ext    m_ext;                /*   128    20 */

	/* XXX 4 bytes hole, try to pack */

	__u64                      ring_cookie;          /*   152     8 */
	__u32                      location;             /*   160     4 */

	/* size: 168, cachelines: 3, members: 7 */
	/* sum members: 160, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct ethtool_rxnfc {
	__u32                      cmd;                  /*     0     4 */
	__u32                      flow_type;            /*     4     4 */
	__u64                      data;                 /*     8     8 */
	struct ethtool_rx_flow_spec fs;                  /*    16   168 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 2 boundary (128 bytes) was 56 bytes ago --- */
	union {
		__u32              rule_cnt;             /*   184     4 */
		__u32              rss_context;          /*   184     4 */
	};                                               /*   184     4 */
	__u32                      rule_locs[];          /*   188     0 */

	/* size: 192, cachelines: 3, members: 6 */
	/* padding: 4 */
	/* paddings: 1, sum paddings: 4 */
};
struct ethtool_flash {
	__u32                      cmd;                  /*     0     4 */
	__u32                      region;               /*     4     4 */
	char                       data[128];            /*     8   128 */

	/* size: 136, cachelines: 3, members: 3 */
	/* last cacheline: 8 bytes */
};
struct ethtool_dump {
	__u32                      cmd;                  /*     0     4 */
	__u32                      version;              /*     4     4 */
	__u32                      flag;                 /*     8     4 */
	__u32                      len;                  /*    12     4 */
	__u8                       data[];               /*    16     0 */

	/* size: 16, cachelines: 1, members: 5 */
	/* last cacheline: 16 bytes */
};
struct ethtool_ts_info {
	__u32                      cmd;                  /*     0     4 */
	__u32                      so_timestamping;      /*     4     4 */
	__s32                      phc_index;            /*     8     4 */
	__u32                      tx_types;             /*    12     4 */
	__u32                      tx_reserved[3];       /*    16    12 */
	__u32                      rx_filters;           /*    28     4 */
	__u32                      rx_reserved[3];       /*    32    12 */

	/* size: 44, cachelines: 1, members: 7 */
	/* last cacheline: 44 bytes */
};
struct ethtool_fecparam {
	__u32                      cmd;                  /*     0     4 */
	__u32                      active_fec;           /*     4     4 */
	__u32                      fec;                  /*     8     4 */
	__u32                      reserved;             /*    12     4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct ethtool_link_settings {
	__u32                      cmd;                  /*     0     4 */
	__u32                      speed;                /*     4     4 */
	__u8                       duplex;               /*     8     1 */
	__u8                       port;                 /*     9     1 */
	__u8                       phy_address;          /*    10     1 */
	__u8                       autoneg;              /*    11     1 */
	__u8                       mdio_support;         /*    12     1 */
	__u8                       eth_tp_mdix;          /*    13     1 */
	__u8                       eth_tp_mdix_ctrl;     /*    14     1 */
	__s8                       link_mode_masks_nwords; /*    15     1 */
	__u8                       transceiver;          /*    16     1 */
	__u8                       master_slave_cfg;     /*    17     1 */
	__u8                       master_slave_state;   /*    18     1 */
	__u8                       reserved1[1];         /*    19     1 */
	__u32                      reserved[7];          /*    20    28 */
	__u32                      link_mode_masks[];    /*    48     0 */

	/* size: 48, cachelines: 1, members: 16 */
	/* last cacheline: 48 bytes */
};
struct kernel_ethtool_ringparam {
	u32                        rx_buf_len;           /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct ethtool_link_ext_state_info {
	enum ethtool_link_ext_state link_ext_state;      /*     0     4 */
	union {
		enum ethtool_link_ext_substate_autoneg autoneg; /*     4     4 */
		enum ethtool_link_ext_substate_link_training link_training; /*     4     4 */
		enum ethtool_link_ext_substate_link_logical_mismatch link_logical_mismatch; /*     4     4 */
		enum ethtool_link_ext_substate_bad_signal_integrity bad_signal_integrity; /*     4     4 */
		enum ethtool_link_ext_substate_cable_issue cable_issue; /*     4     4 */
		u8                 __link_ext_substate;  /*     4     1 */
	};                                               /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct ethtool_link_ksettings {
	struct ethtool_link_settings base;               /*     0    48 */
	struct {
		long unsigned int  supported[2];         /*    48    16 */
		/* --- cacheline 1 boundary (64 bytes) --- */
		long unsigned int  advertising[2];       /*    64    16 */
		long unsigned int  lp_advertising[2];    /*    80    16 */
	} link_modes;                                    /*    48    48 */

	/* size: 96, cachelines: 2, members: 2 */
	/* last cacheline: 32 bytes */
};
struct kernel_ethtool_coalesce {
	u8                         use_cqe_mode_tx;      /*     0     1 */
	u8                         use_cqe_mode_rx;      /*     1     1 */

	/* size: 2, cachelines: 1, members: 2 */
	/* last cacheline: 2 bytes */
};
struct ethtool_pause_stats {
	u64                        tx_pause_frames;      /*     0     8 */
	u64                        rx_pause_frames;      /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct ethtool_ops {
	u32                        supported_coalesce_params; /*     0     4 */
	u32                        supported_ring_params; /*     4     4 */
	void                       (*get_drvinfo)(struct net_device *, struct ethtool_drvinfo *); /*     8     8 */
	int                        (*get_regs_len)(struct net_device *); /*    16     8 */
	void                       (*get_regs)(struct net_device *, struct ethtool_regs *, void *); /*    24     8 */
	void                       (*get_wol)(struct net_device *, struct ethtool_wolinfo *); /*    32     8 */
	int                        (*set_wol)(struct net_device *, struct ethtool_wolinfo *); /*    40     8 */
	u32                        (*get_msglevel)(struct net_device *); /*    48     8 */
	void                       (*set_msglevel)(struct net_device *, u32); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*nway_reset)(struct net_device *); /*    64     8 */
	u32                        (*get_link)(struct net_device *); /*    72     8 */
	int                        (*get_link_ext_state)(struct net_device *, struct ethtool_link_ext_state_info *); /*    80     8 */
	int                        (*get_eeprom_len)(struct net_device *); /*    88     8 */
	int                        (*get_eeprom)(struct net_device *, struct ethtool_eeprom *, u8 *); /*    96     8 */
	int                        (*set_eeprom)(struct net_device *, struct ethtool_eeprom *, u8 *); /*   104     8 */
	int                        (*get_coalesce)(struct net_device *, struct ethtool_coalesce *, struct kernel_ethtool_coalesce *, struct netlink_ext_ack *); /*   112     8 */
	int                        (*set_coalesce)(struct net_device *, struct ethtool_coalesce *, struct kernel_ethtool_coalesce *, struct netlink_ext_ack *); /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	void                       (*get_ringparam)(struct net_device *, struct ethtool_ringparam *, struct kernel_ethtool_ringparam *, struct netlink_ext_ack *); /*   128     8 */
	int                        (*set_ringparam)(struct net_device *, struct ethtool_ringparam *, struct kernel_ethtool_ringparam *, struct netlink_ext_ack *); /*   136     8 */
	void                       (*get_pause_stats)(struct net_device *, struct ethtool_pause_stats *); /*   144     8 */
	void                       (*get_pauseparam)(struct net_device *, struct ethtool_pauseparam *); /*   152     8 */
	int                        (*set_pauseparam)(struct net_device *, struct ethtool_pauseparam *); /*   160     8 */
	void                       (*self_test)(struct net_device *, struct ethtool_test *, u64 *); /*   168     8 */
	void                       (*get_strings)(struct net_device *, u32, u8 *); /*   176     8 */
	int                        (*set_phys_id)(struct net_device *, enum ethtool_phys_id_state); /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	void                       (*get_ethtool_stats)(struct net_device *, struct ethtool_stats *, u64 *); /*   192     8 */
	int                        (*begin)(struct net_device *); /*   200     8 */
	void                       (*complete)(struct net_device *); /*   208     8 */
	u32                        (*get_priv_flags)(struct net_device *); /*   216     8 */
	int                        (*set_priv_flags)(struct net_device *, u32); /*   224     8 */
	int                        (*get_sset_count)(struct net_device *, int); /*   232     8 */
	int                        (*get_rxnfc)(struct net_device *, struct ethtool_rxnfc *, u32 *); /*   240     8 */
	int                        (*set_rxnfc)(struct net_device *, struct ethtool_rxnfc *); /*   248     8 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	int                        (*flash_device)(struct net_device *, struct ethtool_flash *); /*   256     8 */
	int                        (*reset)(struct net_device *, u32 *); /*   264     8 */
	u32                        (*get_rxfh_key_size)(struct net_device *); /*   272     8 */
	u32                        (*get_rxfh_indir_size)(struct net_device *); /*   280     8 */
	int                        (*get_rxfh)(struct net_device *, u32 *, u8 *, u8 *); /*   288     8 */
	int                        (*set_rxfh)(struct net_device *, const u32  *, const u8  *, const u8 ); /*   296     8 */
	int                        (*get_rxfh_context)(struct net_device *, u32 *, u8 *, u8 *, u32); /*   304     8 */
	int                        (*set_rxfh_context)(struct net_device *, const u32  *, const u8  *, const u8 , u32 *, bool); /*   312     8 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	void                       (*get_channels)(struct net_device *, struct ethtool_channels *); /*   320     8 */
	int                        (*set_channels)(struct net_device *, struct ethtool_channels *); /*   328     8 */
	int                        (*get_dump_flag)(struct net_device *, struct ethtool_dump *); /*   336     8 */
	int                        (*get_dump_data)(struct net_device *, struct ethtool_dump *, void *); /*   344     8 */
	int                        (*set_dump)(struct net_device *, struct ethtool_dump *); /*   352     8 */
	int                        (*get_ts_info)(struct net_device *, struct ethtool_ts_info *); /*   360     8 */
	int                        (*get_module_info)(struct net_device *, struct ethtool_modinfo *); /*   368     8 */
	int                        (*get_module_eeprom)(struct net_device *, struct ethtool_eeprom *, u8 *); /*   376     8 */
	/* --- cacheline 6 boundary (384 bytes) --- */
	int                        (*get_eee)(struct net_device *, struct ethtool_eee *); /*   384     8 */
	int                        (*set_eee)(struct net_device *, struct ethtool_eee *); /*   392     8 */
	int                        (*get_tunable)(struct net_device *, const struct ethtool_tunable  *, void *); /*   400     8 */
	int                        (*set_tunable)(struct net_device *, const struct ethtool_tunable  *, const void  *); /*   408     8 */
	int                        (*get_per_queue_coalesce)(struct net_device *, u32, struct ethtool_coalesce *); /*   416     8 */
	int                        (*set_per_queue_coalesce)(struct net_device *, u32, struct ethtool_coalesce *); /*   424     8 */
	int                        (*get_link_ksettings)(struct net_device *, struct ethtool_link_ksettings *); /*   432     8 */
	int                        (*set_link_ksettings)(struct net_device *, const struct ethtool_link_ksettings  *); /*   440     8 */
	/* --- cacheline 7 boundary (448 bytes) --- */
	int                        (*get_fecparam)(struct net_device *, struct ethtool_fecparam *); /*   448     8 */
	int                        (*set_fecparam)(struct net_device *, struct ethtool_fecparam *); /*   456     8 */
	void                       (*get_ethtool_phy_stats)(struct net_device *, struct ethtool_stats *, u64 *); /*   464     8 */
	int                        (*get_phy_tunable)(struct net_device *, const struct ethtool_tunable  *, void *); /*   472     8 */
	int                        (*set_phy_tunable)(struct net_device *, const struct ethtool_tunable  *, const void  *); /*   480     8 */
	u64                        kabi_reserved1;       /*   488     8 */
	u64                        kabi_reserved2;       /*   496     8 */
	u64                        kabi_reserved3;       /*   504     8 */
	/* --- cacheline 8 boundary (512 bytes) --- */
	u64                        kabi_reserved4;       /*   512     8 */

	/* size: 520, cachelines: 9, members: 66 */
	/* last cacheline: 8 bytes */
};
struct ieee_ets {
	__u8                       willing;              /*     0     1 */
	__u8                       ets_cap;              /*     1     1 */
	__u8                       cbs;                  /*     2     1 */
	__u8                       tc_tx_bw[8];          /*     3     8 */
	__u8                       tc_rx_bw[8];          /*    11     8 */
	__u8                       tc_tsa[8];            /*    19     8 */
	__u8                       prio_tc[8];           /*    27     8 */
	__u8                       tc_reco_bw[8];        /*    35     8 */
	__u8                       tc_reco_tsa[8];       /*    43     8 */
	__u8                       reco_prio_tc[8];      /*    51     8 */

	/* size: 59, cachelines: 1, members: 10 */
	/* last cacheline: 59 bytes */
};
struct ieee_maxrate {
	__u64                      tc_maxrate[8];        /*     0    64 */

	/* size: 64, cachelines: 1, members: 1 */
};
struct ieee_qcn {
	__u8                       rpg_enable[8];        /*     0     8 */
	__u32                      rppp_max_rps[8];      /*     8    32 */
	__u32                      rpg_time_reset[8];    /*    40    32 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	__u32                      rpg_byte_reset[8];    /*    72    32 */
	__u32                      rpg_threshold[8];     /*   104    32 */
	/* --- cacheline 2 boundary (128 bytes) was 8 bytes ago --- */
	__u32                      rpg_max_rate[8];      /*   136    32 */
	__u32                      rpg_ai_rate[8];       /*   168    32 */
	/* --- cacheline 3 boundary (192 bytes) was 8 bytes ago --- */
	__u32                      rpg_hai_rate[8];      /*   200    32 */
	__u32                      rpg_gd[8];            /*   232    32 */
	/* --- cacheline 4 boundary (256 bytes) was 8 bytes ago --- */
	__u32                      rpg_min_dec_fac[8];   /*   264    32 */
	__u32                      rpg_min_rate[8];      /*   296    32 */
	/* --- cacheline 5 boundary (320 bytes) was 8 bytes ago --- */
	__u32                      cndd_state_machine[8]; /*   328    32 */

	/* size: 360, cachelines: 6, members: 12 */
	/* last cacheline: 40 bytes */
};
struct ieee_qcn_stats {
	__u64                      rppp_rp_centiseconds[8]; /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	__u32                      rppp_created_rps[8];  /*    64    32 */

	/* size: 96, cachelines: 2, members: 2 */
	/* last cacheline: 32 bytes */
};
struct ieee_pfc {
	__u8                       pfc_cap;              /*     0     1 */
	__u8                       pfc_en;               /*     1     1 */
	__u8                       mbc;                  /*     2     1 */

	/* XXX 1 byte hole, try to pack */

	__u16                      delay;                /*     4     2 */

	/* XXX 2 bytes hole, try to pack */

	__u64                      requests[8];          /*     8    64 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	__u64                      indications[8];       /*    72    64 */

	/* size: 136, cachelines: 3, members: 6 */
	/* sum members: 133, holes: 2, sum holes: 3 */
	/* last cacheline: 8 bytes */
};
struct dcbnl_buffer {
	__u8                       prio2buffer[8];       /*     0     8 */
	__u32                      buffer_size[8];       /*     8    32 */
	__u32                      total_size;           /*    40     4 */

	/* size: 44, cachelines: 1, members: 3 */
	/* last cacheline: 44 bytes */
};
struct cee_pg {
	__u8                       willing;              /*     0     1 */
	__u8                       error;                /*     1     1 */
	__u8                       pg_en;                /*     2     1 */
	__u8                       tcs_supported;        /*     3     1 */
	__u8                       pg_bw[8];             /*     4     8 */
	__u8                       prio_pg[8];           /*    12     8 */

	/* size: 20, cachelines: 1, members: 6 */
	/* last cacheline: 20 bytes */
};
struct cee_pfc {
	__u8                       willing;              /*     0     1 */
	__u8                       error;                /*     1     1 */
	__u8                       pfc_en;               /*     2     1 */
	__u8                       tcs_supported;        /*     3     1 */

	/* size: 4, cachelines: 1, members: 4 */
	/* last cacheline: 4 bytes */
};
struct dcb_app {
	__u8                       selector;             /*     0     1 */
	__u8                       priority;             /*     1     1 */
	__u16                      protocol;             /*     2     2 */

	/* size: 4, cachelines: 1, members: 3 */
	/* last cacheline: 4 bytes */
};
struct dcb_peer_app_info {
	__u8                       willing;              /*     0     1 */
	__u8                       error;                /*     1     1 */

	/* size: 2, cachelines: 1, members: 2 */
	/* last cacheline: 2 bytes */
};
struct dcbnl_rtnl_ops {
	int                        (*ieee_getets)(struct net_device *, struct ieee_ets *); /*     0     8 */
	int                        (*ieee_setets)(struct net_device *, struct ieee_ets *); /*     8     8 */
	int                        (*ieee_getmaxrate)(struct net_device *, struct ieee_maxrate *); /*    16     8 */
	int                        (*ieee_setmaxrate)(struct net_device *, struct ieee_maxrate *); /*    24     8 */
	int                        (*ieee_getqcn)(struct net_device *, struct ieee_qcn *); /*    32     8 */
	int                        (*ieee_setqcn)(struct net_device *, struct ieee_qcn *); /*    40     8 */
	int                        (*ieee_getqcnstats)(struct net_device *, struct ieee_qcn_stats *); /*    48     8 */
	int                        (*ieee_getpfc)(struct net_device *, struct ieee_pfc *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*ieee_setpfc)(struct net_device *, struct ieee_pfc *); /*    64     8 */
	int                        (*ieee_getapp)(struct net_device *, struct dcb_app *); /*    72     8 */
	int                        (*ieee_setapp)(struct net_device *, struct dcb_app *); /*    80     8 */
	int                        (*ieee_delapp)(struct net_device *, struct dcb_app *); /*    88     8 */
	int                        (*ieee_peer_getets)(struct net_device *, struct ieee_ets *); /*    96     8 */
	int                        (*ieee_peer_getpfc)(struct net_device *, struct ieee_pfc *); /*   104     8 */
	u8                         (*getstate)(struct net_device *); /*   112     8 */
	u8                         (*setstate)(struct net_device *, u8); /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	void                       (*getpermhwaddr)(struct net_device *, u8 *); /*   128     8 */
	void                       (*setpgtccfgtx)(struct net_device *, int, u8, u8, u8, u8); /*   136     8 */
	void                       (*setpgbwgcfgtx)(struct net_device *, int, u8); /*   144     8 */
	void                       (*setpgtccfgrx)(struct net_device *, int, u8, u8, u8, u8); /*   152     8 */
	void                       (*setpgbwgcfgrx)(struct net_device *, int, u8); /*   160     8 */
	void                       (*getpgtccfgtx)(struct net_device *, int, u8 *, u8 *, u8 *, u8 *); /*   168     8 */
	void                       (*getpgbwgcfgtx)(struct net_device *, int, u8 *); /*   176     8 */
	void                       (*getpgtccfgrx)(struct net_device *, int, u8 *, u8 *, u8 *, u8 *); /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	void                       (*getpgbwgcfgrx)(struct net_device *, int, u8 *); /*   192     8 */
	void                       (*setpfccfg)(struct net_device *, int, u8); /*   200     8 */
	void                       (*getpfccfg)(struct net_device *, int, u8 *); /*   208     8 */
	u8                         (*setall)(struct net_device *); /*   216     8 */
	u8                         (*getcap)(struct net_device *, int, u8 *); /*   224     8 */
	int                        (*getnumtcs)(struct net_device *, int, u8 *); /*   232     8 */
	int                        (*setnumtcs)(struct net_device *, int, u8); /*   240     8 */
	u8                         (*getpfcstate)(struct net_device *); /*   248     8 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	void                       (*setpfcstate)(struct net_device *, u8); /*   256     8 */
	void                       (*getbcncfg)(struct net_device *, int, u32 *); /*   264     8 */
	void                       (*setbcncfg)(struct net_device *, int, u32); /*   272     8 */
	void                       (*getbcnrp)(struct net_device *, int, u8 *); /*   280     8 */
	void                       (*setbcnrp)(struct net_device *, int, u8); /*   288     8 */
	int                        (*setapp)(struct net_device *, u8, u16, u8); /*   296     8 */
	int                        (*getapp)(struct net_device *, u8, u16); /*   304     8 */
	u8                         (*getfeatcfg)(struct net_device *, int, u8 *); /*   312     8 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	u8                         (*setfeatcfg)(struct net_device *, int, u8); /*   320     8 */
	u8                         (*getdcbx)(struct net_device *); /*   328     8 */
	u8                         (*setdcbx)(struct net_device *, u8); /*   336     8 */
	int                        (*peer_getappinfo)(struct net_device *, struct dcb_peer_app_info *, u16 *); /*   344     8 */
	int                        (*peer_getapptable)(struct net_device *, struct dcb_app *); /*   352     8 */
	int                        (*cee_peer_getpg)(struct net_device *, struct cee_pg *); /*   360     8 */
	int                        (*cee_peer_getpfc)(struct net_device *, struct cee_pfc *); /*   368     8 */
	int                        (*dcbnl_getbuffer)(struct net_device *, struct dcbnl_buffer *); /*   376     8 */
	/* --- cacheline 6 boundary (384 bytes) --- */
	int                        (*dcbnl_setbuffer)(struct net_device *, struct dcbnl_buffer *); /*   384     8 */
	u64                        kabi_reserved1;       /*   392     8 */
	u64                        kabi_reserved2;       /*   400     8 */
	u64                        kabi_reserved3;       /*   408     8 */
	u64                        kabi_reserved4;       /*   416     8 */
	u64                        kabi_reserved5;       /*   424     8 */
	u64                        kabi_reserved6;       /*   432     8 */
	u64                        kabi_reserved7;       /*   440     8 */
	/* --- cacheline 7 boundary (448 bytes) --- */
	u64                        kabi_reserved8;       /*   448     8 */

	/* size: 456, cachelines: 8, members: 57 */
	/* last cacheline: 8 bytes */
};
struct netprio_map {
	struct callback_head       rcu __attribute__((__aligned__(8))); /*     0    16 */
	u32                        priomap_len;          /*    16     4 */
	u32                        priomap[];            /*    20     0 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct xdp_mem_info {
	u32                        type;                 /*     0     4 */
	u32                        id;                   /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct xdp_rxq_info {
	struct net_device *        dev;                  /*     0     8 */
	u32                        queue_index;          /*     8     4 */
	u32                        reg_state;            /*    12     4 */
	struct xdp_mem_info        mem;                  /*    16     8 */
	u64                        kabi_reserved1;       /*    24     8 */
	u64                        kabi_reserved2;       /*    32     8 */
	u64                        kabi_reserved3;       /*    40     8 */
	u64                        kabi_reserved4;       /*    48     8 */

	/* size: 64, cachelines: 1, members: 8 */
	/* padding: 8 */
} __attribute__((__aligned__(64)));
struct xdp_frame {
	void *                     data;                 /*     0     8 */
	u16                        len;                  /*     8     2 */
	u16                        headroom;             /*    10     2 */
	u32                        metasize:8;           /*    12: 0  4 */
	u32                        frame_sz:24;          /*    12: 8  4 */
	struct xdp_mem_info        mem;                  /*    16     8 */
	struct net_device *        dev_rx;               /*    24     8 */

	/* size: 32, cachelines: 1, members: 7 */
	/* last cacheline: 32 bytes */
};
struct ndmsg {
	__u8                       ndm_family;           /*     0     1 */
	__u8                       ndm_pad1;             /*     1     1 */
	__u16                      ndm_pad2;             /*     2     2 */
	__s32                      ndm_ifindex;          /*     4     4 */
	__u16                      ndm_state;            /*     8     2 */
	__u8                       ndm_flags;            /*    10     1 */
	__u8                       ndm_type;             /*    11     1 */

	/* size: 12, cachelines: 1, members: 7 */
	/* last cacheline: 12 bytes */
};
struct rtnl_link_stats64 {
	__u64                      rx_packets;           /*     0     8 */
	__u64                      tx_packets;           /*     8     8 */
	__u64                      rx_bytes;             /*    16     8 */
	__u64                      tx_bytes;             /*    24     8 */
	__u64                      rx_errors;            /*    32     8 */
	__u64                      tx_errors;            /*    40     8 */
	__u64                      rx_dropped;           /*    48     8 */
	__u64                      tx_dropped;           /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	__u64                      multicast;            /*    64     8 */
	__u64                      collisions;           /*    72     8 */
	__u64                      rx_length_errors;     /*    80     8 */
	__u64                      rx_over_errors;       /*    88     8 */
	__u64                      rx_crc_errors;        /*    96     8 */
	__u64                      rx_frame_errors;      /*   104     8 */
	__u64                      rx_fifo_errors;       /*   112     8 */
	__u64                      rx_missed_errors;     /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	__u64                      tx_aborted_errors;    /*   128     8 */
	__u64                      tx_carrier_errors;    /*   136     8 */
	__u64                      tx_fifo_errors;       /*   144     8 */
	__u64                      tx_heartbeat_errors;  /*   152     8 */
	__u64                      tx_window_errors;     /*   160     8 */
	__u64                      rx_compressed;        /*   168     8 */
	__u64                      tx_compressed;        /*   176     8 */
	__u64                      rx_nohandler;         /*   184     8 */

	/* size: 192, cachelines: 3, members: 24 */
};
struct ifla_vf_guid {
	__u32                      vf;                   /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	__u64                      guid;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct ifla_vf_stats {
	__u64                      rx_packets;           /*     0     8 */
	__u64                      tx_packets;           /*     8     8 */
	__u64                      rx_bytes;             /*    16     8 */
	__u64                      tx_bytes;             /*    24     8 */
	__u64                      broadcast;            /*    32     8 */
	__u64                      multicast;            /*    40     8 */
	__u64                      rx_dropped;           /*    48     8 */
	__u64                      tx_dropped;           /*    56     8 */

	/* size: 64, cachelines: 1, members: 8 */
};
struct ifla_vf_info {
	__u32                      vf;                   /*     0     4 */
	__u8                       mac[32];              /*     4    32 */
	__u32                      vlan;                 /*    36     4 */
	__u32                      qos;                  /*    40     4 */
	__u32                      spoofchk;             /*    44     4 */
	__u32                      linkstate;            /*    48     4 */
	__u32                      min_tx_rate;          /*    52     4 */
	__u32                      max_tx_rate;          /*    56     4 */
	__u32                      rss_query_en;         /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	__u32                      trusted;              /*    64     4 */
	__be16                     vlan_proto;           /*    68     2 */

	/* size: 72, cachelines: 2, members: 11 */
	/* padding: 2 */
	/* last cacheline: 8 bytes */
};
struct tc_stats {
	__u64                      bytes;                /*     0     8 */
	__u32                      packets;              /*     8     4 */
	__u32                      drops;                /*    12     4 */
	__u32                      overlimits;           /*    16     4 */
	__u32                      bps;                  /*    20     4 */
	__u32                      pps;                  /*    24     4 */
	__u32                      qlen;                 /*    28     4 */
	__u32                      backlog;              /*    32     4 */

	/* size: 40, cachelines: 1, members: 8 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct tc_sizespec {
	unsigned char              cell_log;             /*     0     1 */
	unsigned char              size_log;             /*     1     1 */
	short int                  cell_align;           /*     2     2 */
	int                        overhead;             /*     4     4 */
	unsigned int               linklayer;            /*     8     4 */
	unsigned int               mpu;                  /*    12     4 */
	unsigned int               mtu;                  /*    16     4 */
	unsigned int               tsize;                /*    20     4 */

	/* size: 24, cachelines: 1, members: 8 */
	/* last cacheline: 24 bytes */
};
struct net_device_stats {
	long unsigned int          rx_packets;           /*     0     8 */
	long unsigned int          tx_packets;           /*     8     8 */
	long unsigned int          rx_bytes;             /*    16     8 */
	long unsigned int          tx_bytes;             /*    24     8 */
	long unsigned int          rx_errors;            /*    32     8 */
	long unsigned int          tx_errors;            /*    40     8 */
	long unsigned int          rx_dropped;           /*    48     8 */
	long unsigned int          tx_dropped;           /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	long unsigned int          multicast;            /*    64     8 */
	long unsigned int          collisions;           /*    72     8 */
	long unsigned int          rx_length_errors;     /*    80     8 */
	long unsigned int          rx_over_errors;       /*    88     8 */
	long unsigned int          rx_crc_errors;        /*    96     8 */
	long unsigned int          rx_frame_errors;      /*   104     8 */
	long unsigned int          rx_fifo_errors;       /*   112     8 */
	long unsigned int          rx_missed_errors;     /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	long unsigned int          tx_aborted_errors;    /*   128     8 */
	long unsigned int          tx_carrier_errors;    /*   136     8 */
	long unsigned int          tx_fifo_errors;       /*   144     8 */
	long unsigned int          tx_heartbeat_errors;  /*   152     8 */
	long unsigned int          tx_window_errors;     /*   160     8 */
	long unsigned int          rx_compressed;        /*   168     8 */
	long unsigned int          tx_compressed;        /*   176     8 */

	/* size: 184, cachelines: 3, members: 23 */
	/* last cacheline: 56 bytes */
};
struct netdev_hw_addr_list {
	struct list_head           list;                 /*     0    16 */
	int                        count;                /*    16     4 */

	/* size: 24, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct hh_cache {
	unsigned int               hh_len;               /*     0     4 */
	seqlock_t                  hh_lock;              /*     4     8 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          hh_data[12];          /*    16    96 */

	/* size: 112, cachelines: 2, members: 3 */
	/* sum members: 108, holes: 1, sum holes: 4 */
	/* last cacheline: 48 bytes */
};
struct header_ops {
	int                        (*create)(struct sk_buff *, struct net_device *, short unsigned int, const void  *, const void  *, unsigned int); /*     0     8 */
	int                        (*parse)(const struct sk_buff  *, unsigned char *); /*     8     8 */
	int                        (*cache)(const struct neighbour  *, struct hh_cache *, __be16); /*    16     8 */
	void                       (*cache_update)(struct hh_cache *, const struct net_device  *, const unsigned char  *); /*    24     8 */
	bool                       (*validate)(const char  *, unsigned int); /*    32     8 */
	__be16                     (*parse_protocol)(const struct sk_buff  *); /*    40     8 */
	u64                        kabi_reserved1;       /*    48     8 */
	u64                        kabi_reserved2;       /*    56     8 */

	/* size: 64, cachelines: 1, members: 8 */
};
struct netdev_queue {
	struct net_device *        dev;                  /*     0     8 */
	struct Qdisc *             qdisc;                /*     8     8 */
	struct Qdisc *             qdisc_sleeping;       /*    16     8 */
	struct kobject             kobj;                 /*    24    96 */
	/* --- cacheline 1 boundary (64 bytes) was 56 bytes ago --- */
	int                        numa_node;            /*   120     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 2 boundary (128 bytes) --- */
	long unsigned int          tx_maxrate;           /*   128     8 */
	long unsigned int          trans_timeout;        /*   136     8 */
	struct net_device *        sb_dev;               /*   144     8 */
	struct xsk_buff_pool *     pool;                 /*   152     8 */

	/* XXX 32 bytes hole, try to pack */

	/* --- cacheline 3 boundary (192 bytes) --- */
	spinlock_t                 _xmit_lock __attribute__((__aligned__(64))); /*   192     4 */
	int                        xmit_lock_owner;      /*   196     4 */
	long unsigned int          trans_start;          /*   200     8 */
	long unsigned int          state;                /*   208     8 */

	/* XXX 40 bytes hole, try to pack */

	/* --- cacheline 4 boundary (256 bytes) --- */
	struct dql                 dql __attribute__((__aligned__(64))); /*   256   128 */

	/* XXX last struct has 20 bytes of padding */

	/* --- cacheline 6 boundary (384 bytes) --- */
	u64                        kabi_reserved1;       /*   384     8 */
	u64                        kabi_reserved2;       /*   392     8 */
	u64                        kabi_reserved3;       /*   400     8 */
	u64                        kabi_reserved4;       /*   408     8 */

	/* size: 448, cachelines: 7, members: 18 */
	/* sum members: 340, holes: 3, sum holes: 76 */
	/* padding: 32 */
	/* paddings: 1, sum paddings: 20 */
	/* forced alignments: 2, forced holes: 2, sum forced holes: 72 */
} __attribute__((__aligned__(64)));
struct Qdisc {
	int                        (*enqueue)(struct sk_buff *, struct Qdisc *, struct sk_buff * *); /*     0     8 */
	struct sk_buff *           (*dequeue)(struct Qdisc *); /*     8     8 */
	unsigned int               flags;                /*    16     4 */
	u32                        limit;                /*    20     4 */
	const struct Qdisc_ops  *  ops;                  /*    24     8 */
	struct qdisc_size_table *  stab;                 /*    32     8 */
	struct hlist_node          hash;                 /*    40    16 */
	u32                        handle;               /*    56     4 */
	u32                        parent;               /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct netdev_queue *      dev_queue;            /*    64     8 */
	struct net_rate_estimator * rate_est;            /*    72     8 */
	struct gnet_stats_basic_cpu * cpu_bstats;        /*    80     8 */
	struct gnet_stats_queue *  cpu_qstats;           /*    88     8 */
	int                        pad;                  /*    96     4 */
	refcount_t                 refcnt;               /*   100     4 */

	/* XXX 24 bytes hole, try to pack */

	/* --- cacheline 2 boundary (128 bytes) --- */
	struct sk_buff_head        gso_skb __attribute__((__aligned__(64))); /*   128    24 */
	struct qdisc_skb_head      q;                    /*   152    24 */
	struct gnet_stats_basic_packed bstats;           /*   176    16 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	seqcount_t                 running;              /*   192     4 */
	struct gnet_stats_queue    qstats;               /*   196    20 */
	long unsigned int          state;                /*   216     8 */
	struct Qdisc *             next_sched;           /*   224     8 */
	struct sk_buff_head        skb_bad_txq;          /*   232    24 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	spinlock_t                 busylock __attribute__((__aligned__(64))); /*   256     4 */
	spinlock_t                 seqlock;              /*   260     4 */
	bool                       empty;                /*   264     1 */

	/* XXX 7 bytes hole, try to pack */

	struct callback_head       rcu __attribute__((__aligned__(8))); /*   272    16 */
	u64                        kabi_reserved1;       /*   288     8 */
	u64                        kabi_reserved2;       /*   296     8 */

	/* XXX 16 bytes hole, try to pack */

	/* --- cacheline 5 boundary (320 bytes) --- */
	long int                   privdata[] __attribute__((__aligned__(64))); /*   320     0 */

	/* size: 320, cachelines: 5, members: 30 */
	/* sum members: 273, holes: 3, sum holes: 47 */
	/* forced alignments: 4, forced holes: 3, sum forced holes: 47 */
} __attribute__((__aligned__(64)));
struct rps_map {
	unsigned int               len;                  /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct callback_head       rcu __attribute__((__aligned__(8))); /*     8    16 */
	u16                        cpus[];               /*    24     0 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 4 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct rps_dev_flow {
	u16                        cpu;                  /*     0     2 */
	u16                        filter;               /*     2     2 */
	unsigned int               last_qtail;           /*     4     4 */

	/* size: 8, cachelines: 1, members: 3 */
	/* last cacheline: 8 bytes */
};
struct rps_dev_flow_table {
	unsigned int               mask;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct callback_head       rcu __attribute__((__aligned__(8))); /*     8    16 */
	struct rps_dev_flow        flows[];              /*    24     0 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 4 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct netdev_rx_queue {
	struct rps_map *           rps_map;              /*     0     8 */
	struct rps_dev_flow_table * rps_flow_table;      /*     8     8 */
	struct kobject             kobj;                 /*    16    96 */
	/* --- cacheline 1 boundary (64 bytes) was 48 bytes ago --- */
	struct net_device *        dev;                  /*   112     8 */

	/* XXX 8 bytes hole, try to pack */

	/* --- cacheline 2 boundary (128 bytes) --- */
	struct xdp_rxq_info        xdp_rxq __attribute__((__aligned__(64))); /*   128    64 */

	/* XXX last struct has 8 bytes of padding */

	/* --- cacheline 3 boundary (192 bytes) --- */
	struct xsk_buff_pool *     pool;                 /*   192     8 */
	u64                        kabi_reserved1;       /*   200     8 */
	u64                        kabi_reserved2;       /*   208     8 */
	u64                        kabi_reserved3;       /*   216     8 */
	u64                        kabi_reserved4;       /*   224     8 */

	/* size: 256, cachelines: 4, members: 10 */
	/* sum members: 224, holes: 1, sum holes: 8 */
	/* padding: 24 */
	/* paddings: 1, sum paddings: 8 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 8 */
} __attribute__((__aligned__(64)));
struct xps_map {
	unsigned int               len;                  /*     0     4 */
	unsigned int               alloc_len;            /*     4     4 */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*     8    16 */
	u16                        queues[];             /*    24     0 */

	/* size: 24, cachelines: 1, members: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct xps_dev_maps {
	struct callback_head       rcu __attribute__((__aligned__(8))); /*     0    16 */
	struct xps_map *           attr_map[];           /*    16     0 */

	/* size: 16, cachelines: 1, members: 2 */
	/* forced alignments: 1 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct netdev_tc_txq {
	u16                        count;                /*     0     2 */
	u16                        offset;               /*     2     2 */

	/* size: 4, cachelines: 1, members: 2 */
	/* last cacheline: 4 bytes */
};
struct netdev_fcoe_hbainfo {
	char                       manufacturer[64];     /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	char                       serial_number[64];    /*    64    64 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	char                       hardware_version[64]; /*   128    64 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	char                       driver_version[64];   /*   192    64 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	char                       optionrom_version[64]; /*   256    64 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	char                       firmware_version[64]; /*   320    64 */
	/* --- cacheline 6 boundary (384 bytes) --- */
	char                       model[256];           /*   384   256 */
	/* --- cacheline 10 boundary (640 bytes) --- */
	char                       model_description[256]; /*   640   256 */

	/* size: 896, cachelines: 14, members: 8 */
};
struct netdev_phys_item_id {
	unsigned char              id[32];               /*     0    32 */
	unsigned char              id_len;               /*    32     1 */

	/* size: 33, cachelines: 1, members: 2 */
	/* last cacheline: 33 bytes */
};
struct bpf_xdp_entity {
	struct bpf_prog *          prog;                 /*     0     8 */
	struct bpf_xdp_link *      link;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct netdev_bpf {
	enum bpf_netdev_command    command;              /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	union {
		struct {
			u32        flags;                /*     8     4 */

			/* XXX 4 bytes hole, try to pack */

			struct bpf_prog * prog;          /*    16     8 */
			struct netlink_ext_ack * extack; /*    24     8 */
		};                                       /*     8    24 */
		struct {
			struct bpf_offloaded_map * offmap; /*     8     8 */
		};                                       /*     8     8 */
		struct {
			struct xsk_buff_pool * pool;     /*     8     8 */
			u16        queue_id;             /*    16     2 */
		} xsk;                                   /*     8    16 */
	};                                               /*     8    24 */

	/* size: 32, cachelines: 1, members: 2 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct xfrmdev_ops {
	int                        (*xdo_dev_state_add)(struct xfrm_state *); /*     0     8 */
	void                       (*xdo_dev_state_delete)(struct xfrm_state *); /*     8     8 */
	void                       (*xdo_dev_state_free)(struct xfrm_state *); /*    16     8 */
	bool                       (*xdo_dev_offload_ok)(struct sk_buff *, struct xfrm_state *); /*    24     8 */
	void                       (*xdo_dev_state_advance_esn)(struct xfrm_state *); /*    32     8 */
	u64                        kabi_reserved1;       /*    40     8 */
	u64                        kabi_reserved2;       /*    48     8 */
	u64                        kabi_reserved3;       /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u64                        kabi_reserved4;       /*    64     8 */

	/* size: 72, cachelines: 2, members: 9 */
	/* last cacheline: 8 bytes */
};
struct dev_ifalias {
	struct callback_head       rcuhead __attribute__((__aligned__(8))); /*     0    16 */
	char                       ifalias[];            /*    16     0 */

	/* size: 16, cachelines: 1, members: 2 */
	/* forced alignments: 1 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct netdev_name_node {
	struct hlist_node          hlist;                /*     0    16 */
	struct list_head           list;                 /*    16    16 */
	struct net_device *        dev;                  /*    32     8 */
	const char  *              name;                 /*    40     8 */

	/* size: 48, cachelines: 1, members: 4 */
	/* last cacheline: 48 bytes */
};
struct net_device_ops {
	int                        (*ndo_init)(struct net_device *); /*     0     8 */
	void                       (*ndo_uninit)(struct net_device *); /*     8     8 */
	int                        (*ndo_open)(struct net_device *); /*    16     8 */
	int                        (*ndo_stop)(struct net_device *); /*    24     8 */
	netdev_tx_t                (*ndo_start_xmit)(struct sk_buff *, struct net_device *); /*    32     8 */
	netdev_features_t          (*ndo_features_check)(struct sk_buff *, struct net_device *, netdev_features_t); /*    40     8 */
	u16                        (*ndo_select_queue)(struct net_device *, struct sk_buff *, struct net_device *); /*    48     8 */
	void                       (*ndo_change_rx_flags)(struct net_device *, int); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	void                       (*ndo_set_rx_mode)(struct net_device *); /*    64     8 */
	int                        (*ndo_set_mac_address)(struct net_device *, void *); /*    72     8 */
	int                        (*ndo_validate_addr)(struct net_device *); /*    80     8 */
	int                        (*ndo_do_ioctl)(struct net_device *, struct ifreq *, int); /*    88     8 */
	int                        (*ndo_set_config)(struct net_device *, struct ifmap *); /*    96     8 */
	int                        (*ndo_change_mtu)(struct net_device *, int); /*   104     8 */
	int                        (*ndo_neigh_setup)(struct net_device *, struct neigh_parms *); /*   112     8 */
	void                       (*ndo_tx_timeout)(struct net_device *, unsigned int); /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	void                       (*ndo_get_stats64)(struct net_device *, struct rtnl_link_stats64 *); /*   128     8 */
	bool                       (*ndo_has_offload_stats)(const struct net_device  *, int); /*   136     8 */
	int                        (*ndo_get_offload_stats)(int, const struct net_device  *, void *); /*   144     8 */
	struct net_device_stats *  (*ndo_get_stats)(struct net_device *); /*   152     8 */
	int                        (*ndo_vlan_rx_add_vid)(struct net_device *, __be16, u16); /*   160     8 */
	int                        (*ndo_vlan_rx_kill_vid)(struct net_device *, __be16, u16); /*   168     8 */
	void                       (*ndo_poll_controller)(struct net_device *); /*   176     8 */
	int                        (*ndo_netpoll_setup)(struct net_device *, struct netpoll_info *); /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	void                       (*ndo_netpoll_cleanup)(struct net_device *); /*   192     8 */
	int                        (*ndo_set_vf_mac)(struct net_device *, int, u8 *); /*   200     8 */
	int                        (*ndo_set_vf_vlan)(struct net_device *, int, u16, u8, __be16); /*   208     8 */
	int                        (*ndo_set_vf_rate)(struct net_device *, int, int, int); /*   216     8 */
	int                        (*ndo_set_vf_spoofchk)(struct net_device *, int, bool); /*   224     8 */
	int                        (*ndo_set_vf_trust)(struct net_device *, int, bool); /*   232     8 */
	int                        (*ndo_get_vf_config)(struct net_device *, int, struct ifla_vf_info *); /*   240     8 */
	int                        (*ndo_set_vf_link_state)(struct net_device *, int, int); /*   248     8 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	int                        (*ndo_get_vf_stats)(struct net_device *, int, struct ifla_vf_stats *); /*   256     8 */
	int                        (*ndo_set_vf_port)(struct net_device *, int, struct nlattr * *); /*   264     8 */
	int                        (*ndo_get_vf_port)(struct net_device *, int, struct sk_buff *); /*   272     8 */
	int                        (*ndo_get_vf_guid)(struct net_device *, int, struct ifla_vf_guid *, struct ifla_vf_guid *); /*   280     8 */
	int                        (*ndo_set_vf_guid)(struct net_device *, int, u64, int); /*   288     8 */
	int                        (*ndo_set_vf_rss_query_en)(struct net_device *, int, bool); /*   296     8 */
	int                        (*ndo_setup_tc)(struct net_device *, enum tc_setup_type, void *); /*   304     8 */
	int                        (*ndo_fcoe_enable)(struct net_device *); /*   312     8 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	int                        (*ndo_fcoe_disable)(struct net_device *); /*   320     8 */
	int                        (*ndo_fcoe_ddp_setup)(struct net_device *, u16, struct scatterlist *, unsigned int); /*   328     8 */
	int                        (*ndo_fcoe_ddp_done)(struct net_device *, u16); /*   336     8 */
	int                        (*ndo_fcoe_ddp_target)(struct net_device *, u16, struct scatterlist *, unsigned int); /*   344     8 */
	int                        (*ndo_fcoe_get_hbainfo)(struct net_device *, struct netdev_fcoe_hbainfo *); /*   352     8 */
	int                        (*ndo_fcoe_get_wwn)(struct net_device *, u64 *, int); /*   360     8 */
	int                        (*ndo_rx_flow_steer)(struct net_device *, const struct sk_buff  *, u16, u32); /*   368     8 */
	int                        (*ndo_add_slave)(struct net_device *, struct net_device *, struct netlink_ext_ack *); /*   376     8 */
	/* --- cacheline 6 boundary (384 bytes) --- */
	int                        (*ndo_del_slave)(struct net_device *, struct net_device *); /*   384     8 */
	struct net_device *        (*ndo_get_xmit_slave)(struct net_device *, struct sk_buff *, bool); /*   392     8 */
	netdev_features_t          (*ndo_fix_features)(struct net_device *, netdev_features_t); /*   400     8 */
	int                        (*ndo_set_features)(struct net_device *, netdev_features_t); /*   408     8 */
	int                        (*ndo_neigh_construct)(struct net_device *, struct neighbour *); /*   416     8 */
	void                       (*ndo_neigh_destroy)(struct net_device *, struct neighbour *); /*   424     8 */
	int                        (*ndo_fdb_add)(struct ndmsg *, struct nlattr * *, struct net_device *, const unsigned char  *, u16, u16, struct netlink_ext_ack *); /*   432     8 */
	int                        (*ndo_fdb_del)(struct ndmsg *, struct nlattr * *, struct net_device *, const unsigned char  *, u16); /*   440     8 */
	/* --- cacheline 7 boundary (448 bytes) --- */
	int                        (*ndo_fdb_dump)(struct sk_buff *, struct netlink_callback *, struct net_device *, struct net_device *, int *); /*   448     8 */
	int                        (*ndo_fdb_get)(struct sk_buff *, struct nlattr * *, struct net_device *, const unsigned char  *, u16, u32, u32, struct netlink_ext_ack *); /*   456     8 */
	int                        (*ndo_bridge_setlink)(struct net_device *, struct nlmsghdr *, u16, struct netlink_ext_ack *); /*   464     8 */
	int                        (*ndo_bridge_getlink)(struct sk_buff *, u32, u32, struct net_device *, u32, int); /*   472     8 */
	int                        (*ndo_bridge_dellink)(struct net_device *, struct nlmsghdr *, u16); /*   480     8 */
	int                        (*ndo_change_carrier)(struct net_device *, bool); /*   488     8 */
	int                        (*ndo_get_phys_port_id)(struct net_device *, struct netdev_phys_item_id *); /*   496     8 */
	int                        (*ndo_get_port_parent_id)(struct net_device *, struct netdev_phys_item_id *); /*   504     8 */
	/* --- cacheline 8 boundary (512 bytes) --- */
	int                        (*ndo_get_phys_port_name)(struct net_device *, char *, size_t); /*   512     8 */
	void                       (*ndo_udp_tunnel_add)(struct net_device *, struct udp_tunnel_info *); /*   520     8 */
	void                       (*ndo_udp_tunnel_del)(struct net_device *, struct udp_tunnel_info *); /*   528     8 */
	void *                     (*ndo_dfwd_add_station)(struct net_device *, struct net_device *); /*   536     8 */
	void                       (*ndo_dfwd_del_station)(struct net_device *, void *); /*   544     8 */
	int                        (*ndo_set_tx_maxrate)(struct net_device *, int, u32); /*   552     8 */
	int                        (*ndo_get_iflink)(const struct net_device  *); /*   560     8 */
	int                        (*ndo_change_proto_down)(struct net_device *, bool); /*   568     8 */
	/* --- cacheline 9 boundary (576 bytes) --- */
	int                        (*ndo_fill_metadata_dst)(struct net_device *, struct sk_buff *); /*   576     8 */
	void                       (*ndo_set_rx_headroom)(struct net_device *, int); /*   584     8 */
	int                        (*ndo_bpf)(struct net_device *, struct netdev_bpf *); /*   592     8 */
	int                        (*ndo_xdp_xmit)(struct net_device *, int, struct xdp_frame * *, u32); /*   600     8 */
	int                        (*ndo_xsk_wakeup)(struct net_device *, u32, u32); /*   608     8 */
	struct devlink_port *      (*ndo_get_devlink_port)(struct net_device *); /*   616     8 */
	int                        (*ndo_tunnel_ctl)(struct net_device *, struct ip_tunnel_parm *, int); /*   624     8 */
	struct net_device *        (*ndo_get_peer_dev)(struct net_device *); /*   632     8 */
	/* --- cacheline 10 boundary (640 bytes) --- */
	u64                        kabi_reserved1;       /*   640     8 */
	u64                        kabi_reserved2;       /*   648     8 */
	u64                        kabi_reserved3;       /*   656     8 */
	u64                        kabi_reserved4;       /*   664     8 */
	u64                        kabi_reserved5;       /*   672     8 */
	u64                        kabi_reserved6;       /*   680     8 */
	u64                        kabi_reserved7;       /*   688     8 */
	u64                        kabi_reserved8;       /*   696     8 */

	/* size: 704, cachelines: 11, members: 88 */
};
struct neigh_parms {
	possible_net_t             net;                  /*     0     8 */
	struct net_device *        dev;                  /*     8     8 */
	struct list_head           list;                 /*    16    16 */
	int                        (*neigh_setup)(struct neighbour *); /*    32     8 */
	struct neigh_table *       tbl;                  /*    40     8 */
	void *                     sysctl_table;         /*    48     8 */
	int                        dead;                 /*    56     4 */
	refcount_t                 refcnt;               /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct callback_head       callback_head __attribute__((__aligned__(8))); /*    64    16 */
	int                        reachable_time;       /*    80     4 */
	int                        data[13];             /*    84    52 */
	/* --- cacheline 2 boundary (128 bytes) was 8 bytes ago --- */
	long unsigned int          data_state[1];        /*   136     8 */
	u64                        kabi_reserved1;       /*   144     8 */

	/* size: 152, cachelines: 3, members: 13 */
	/* forced alignments: 1 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct net_device_extended_resvd {

	/* size: 0, cachelines: 0, members: 0 */
};
struct pcpu_lstats {
	u64_stats_t                packets;              /*     0     8 */
	u64_stats_t                bytes;                /*     8     8 */
	struct u64_stats_sync      syncp;                /*    16     0 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(16)));
struct pcpu_sw_netstats {
	u64                        rx_packets;           /*     0     8 */
	u64                        rx_bytes;             /*     8     8 */
	u64                        tx_packets;           /*    16     8 */
	u64                        tx_bytes;             /*    24     8 */
	struct u64_stats_sync      syncp;                /*    32     0 */

	/* size: 32, cachelines: 1, members: 5 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(32)));
struct l3mdev_ops {
	u32                        (*l3mdev_fib_table)(const struct net_device  *); /*     0     8 */
	struct sk_buff *           (*l3mdev_l3_rcv)(struct net_device *, struct sk_buff *, u16); /*     8     8 */
	struct sk_buff *           (*l3mdev_l3_out)(struct net_device *, struct sock *, struct sk_buff *, u16); /*    16     8 */
	struct dst_entry *         (*l3mdev_link_scope_lookup)(const struct net_device  *, struct flowi6 *); /*    24     8 */
	u64                        kabi_reserved1;       /*    32     8 */
	u64                        kabi_reserved2;       /*    40     8 */
	u64                        kabi_reserved3;       /*    48     8 */
	u64                        kabi_reserved4;       /*    56     8 */

	/* size: 64, cachelines: 1, members: 8 */
};
struct ndisc_ops {
	int                        (*is_useropt)(u8);    /*     0     8 */
	int                        (*parse_options)(const struct net_device  *, struct nd_opt_hdr *, struct ndisc_options *); /*     8     8 */
	void                       (*update)(const struct net_device  *, struct neighbour *, u32, u8, const struct ndisc_options  *); /*    16     8 */
	int                        (*opt_addr_space)(const struct net_device  *, u8, struct neighbour *, u8 *, u8 * *); /*    24     8 */
	void                       (*fill_addr_option)(const struct net_device  *, struct sk_buff *, u8, const u8  *); /*    32     8 */
	void                       (*prefix_rcv_add_addr)(struct net *, struct net_device *, const struct prefix_info  *, struct inet6_dev *, struct in6_addr *, int, u32, bool, bool, __u32, u32, bool); /*    40     8 */

	/* size: 48, cachelines: 1, members: 6 */
	/* last cacheline: 48 bytes */
};
struct inet6_dev {
	struct net_device *        dev;                  /*     0     8 */
	struct list_head           addr_list;            /*     8    16 */
	struct ifmcaddr6 *         mc_list;              /*    24     8 */
	struct ifmcaddr6 *         mc_tomb;              /*    32     8 */
	spinlock_t                 mc_lock;              /*    40     4 */
	unsigned char              mc_qrv;               /*    44     1 */
	unsigned char              mc_gq_running;        /*    45     1 */
	unsigned char              mc_ifc_count;         /*    46     1 */
	unsigned char              mc_dad_count;         /*    47     1 */
	long unsigned int          mc_v1_seen;           /*    48     8 */
	long unsigned int          mc_qi;                /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	long unsigned int          mc_qri;               /*    64     8 */
	long unsigned int          mc_maxdelay;          /*    72     8 */
	struct timer_list          mc_gq_timer;          /*    80    72 */
	/* --- cacheline 2 boundary (128 bytes) was 24 bytes ago --- */
	struct timer_list          mc_ifc_timer;         /*   152    72 */
	/* --- cacheline 3 boundary (192 bytes) was 32 bytes ago --- */
	struct timer_list          mc_dad_timer;         /*   224    72 */
	/* --- cacheline 4 boundary (256 bytes) was 40 bytes ago --- */
	struct ifacaddr6 *         ac_list;              /*   296     8 */
	rwlock_t                   lock;                 /*   304     8 */
	refcount_t                 refcnt;               /*   312     4 */
	__u32                      if_flags;             /*   316     4 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	int                        dead;                 /*   320     4 */
	u32                        desync_factor;        /*   324     4 */
	struct list_head           tempaddr_list;        /*   328    16 */
	struct in6_addr            token;                /*   344    16 */
	struct neigh_parms *       nd_parms;             /*   360     8 */
	struct ipv6_devconf        cnf;                  /*   368   264 */
	/* --- cacheline 9 boundary (576 bytes) was 56 bytes ago --- */
	struct ipv6_devstat        stats;                /*   632    32 */
	/* --- cacheline 10 boundary (640 bytes) was 24 bytes ago --- */
	struct timer_list          rs_timer;             /*   664    72 */
	/* --- cacheline 11 boundary (704 bytes) was 32 bytes ago --- */
	__s32                      rs_interval;          /*   736     4 */
	__u8                       rs_probes;            /*   740     1 */

	/* XXX 3 bytes hole, try to pack */

	long unsigned int          tstamp;               /*   744     8 */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*   752    16 */

	/* size: 768, cachelines: 12, members: 32 */
	/* sum members: 765, holes: 1, sum holes: 3 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(8)));
struct mini_Qdisc {
	struct tcf_proto *         filter_list;          /*     0     8 */
	struct tcf_block *         block;                /*     8     8 */
	struct gnet_stats_basic_cpu * cpu_bstats;        /*    16     8 */
	struct gnet_stats_queue *  cpu_qstats;           /*    24     8 */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*    32    16 */

	/* size: 48, cachelines: 1, members: 5 */
	/* forced alignments: 1 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct rtnl_link_ops {
	struct list_head           list;                 /*     0    16 */
	const char  *              kind;                 /*    16     8 */
	size_t                     priv_size;            /*    24     8 */
	void                       (*setup)(struct net_device *); /*    32     8 */
	bool                       netns_refund;         /*    40     1 */

	/* XXX 3 bytes hole, try to pack */

	unsigned int               maxtype;              /*    44     4 */
	const struct nla_policy  * policy;               /*    48     8 */
	int                        (*validate)(struct nlattr * *, struct nlattr * *, struct netlink_ext_ack *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*newlink)(struct net *, struct net_device *, struct nlattr * *, struct nlattr * *, struct netlink_ext_ack *); /*    64     8 */
	int                        (*changelink)(struct net_device *, struct nlattr * *, struct nlattr * *, struct netlink_ext_ack *); /*    72     8 */
	void                       (*dellink)(struct net_device *, struct list_head *); /*    80     8 */
	size_t                     (*get_size)(const struct net_device  *); /*    88     8 */
	int                        (*fill_info)(struct sk_buff *, const struct net_device  *); /*    96     8 */
	size_t                     (*get_xstats_size)(const struct net_device  *); /*   104     8 */
	int                        (*fill_xstats)(struct sk_buff *, const struct net_device  *); /*   112     8 */
	unsigned int               (*get_num_tx_queues)(void); /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	unsigned int               (*get_num_rx_queues)(void); /*   128     8 */
	unsigned int               slave_maxtype;        /*   136     4 */

	/* XXX 4 bytes hole, try to pack */

	const struct nla_policy  * slave_policy;         /*   144     8 */
	int                        (*slave_changelink)(struct net_device *, struct net_device *, struct nlattr * *, struct nlattr * *, struct netlink_ext_ack *); /*   152     8 */
	size_t                     (*get_slave_size)(const struct net_device  *, const struct net_device  *); /*   160     8 */
	int                        (*fill_slave_info)(struct sk_buff *, const struct net_device  *, const struct net_device  *); /*   168     8 */
	struct net *               (*get_link_net)(const struct net_device  *); /*   176     8 */
	size_t                     (*get_linkxstats_size)(const struct net_device  *, int); /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	int                        (*fill_linkxstats)(struct sk_buff *, const struct net_device  *, int *, int); /*   192     8 */
	u64                        kabi_reserved1;       /*   200     8 */
	u64                        kabi_reserved2;       /*   208     8 */
	u64                        kabi_reserved3;       /*   216     8 */
	u64                        kabi_reserved4;       /*   224     8 */
	u64                        kabi_reserved5;       /*   232     8 */
	u64                        kabi_reserved6;       /*   240     8 */
	u64                        kabi_reserved7;       /*   248     8 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	u64                        kabi_reserved8;       /*   256     8 */

	/* size: 264, cachelines: 5, members: 33 */
	/* sum members: 257, holes: 2, sum holes: 7 */
	/* last cacheline: 8 bytes */
};
struct tcmsg {
	unsigned char              tcm_family;           /*     0     1 */
	unsigned char              tcm__pad1;            /*     1     1 */
	short unsigned int         tcm__pad2;            /*     2     2 */
	int                        tcm_ifindex;          /*     4     4 */
	__u32                      tcm_handle;           /*     8     4 */
	__u32                      tcm_parent;           /*    12     4 */
	__u32                      tcm_info;             /*    16     4 */

	/* size: 20, cachelines: 1, members: 7 */
	/* last cacheline: 20 bytes */
};
struct gnet_stats_queue {
	__u32                      qlen;                 /*     0     4 */
	__u32                      backlog;              /*     4     4 */
	__u32                      drops;                /*     8     4 */
	__u32                      requeues;             /*    12     4 */
	__u32                      overlimits;           /*    16     4 */

	/* size: 20, cachelines: 1, members: 5 */
	/* last cacheline: 20 bytes */
};
struct gnet_stats_basic_packed {
	__u64                      bytes;                /*     0     8 */
	__u64                      packets;              /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct gnet_stats_basic_cpu {
	struct gnet_stats_basic_packed bstats;           /*     0    16 */
	struct u64_stats_sync      syncp;                /*    16     0 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(16)));
struct gnet_dump {
	spinlock_t *               lock;                 /*     0     8 */
	struct sk_buff *           skb;                  /*     8     8 */
	struct nlattr *            tail;                 /*    16     8 */
	int                        compat_tc_stats;      /*    24     4 */
	int                        compat_xstats;        /*    28     4 */
	int                        padattr;              /*    32     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     xstats;               /*    40     8 */
	int                        xstats_len;           /*    48     4 */

	/* XXX 4 bytes hole, try to pack */

	struct tc_stats            tc_stats;             /*    56    40 */

	/* XXX last struct has 4 bytes of padding */

	/* size: 96, cachelines: 2, members: 9 */
	/* sum members: 88, holes: 2, sum holes: 8 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 32 bytes */
};
struct netlink_range_validation {
	u64                        min;                  /*     0     8 */
	u64                        max;                  /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct netlink_range_validation_signed {
	s64                        min;                  /*     0     8 */
	s64                        max;                  /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct flow_block {
	struct list_head           cb_list;              /*     0    16 */

	/* size: 16, cachelines: 1, members: 1 */
	/* last cacheline: 16 bytes */
};
struct qdisc_size_table {
	struct callback_head       rcu __attribute__((__aligned__(8))); /*     0    16 */
	struct list_head           list;                 /*    16    16 */
	struct tc_sizespec         szopts;               /*    32    24 */
	int                        refcnt;               /*    56     4 */
	u16                        data[];               /*    60     0 */

	/* size: 64, cachelines: 1, members: 5 */
	/* padding: 4 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(8)));
struct qdisc_skb_head {
	struct sk_buff *           head;                 /*     0     8 */
	struct sk_buff *           tail;                 /*     8     8 */
	__u32                      qlen;                 /*    16     4 */
	spinlock_t                 lock;                 /*    20     4 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct Qdisc_ops {
	struct Qdisc_ops *         next;                 /*     0     8 */
	const struct Qdisc_class_ops  * cl_ops;          /*     8     8 */
	char                       id[16];               /*    16    16 */
	int                        priv_size;            /*    32     4 */
	unsigned int               static_flags;         /*    36     4 */
	int                        (*enqueue)(struct sk_buff *, struct Qdisc *, struct sk_buff * *); /*    40     8 */
	struct sk_buff *           (*dequeue)(struct Qdisc *); /*    48     8 */
	struct sk_buff *           (*peek)(struct Qdisc *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*init)(struct Qdisc *, struct nlattr *, struct netlink_ext_ack *); /*    64     8 */
	void                       (*reset)(struct Qdisc *); /*    72     8 */
	void                       (*destroy)(struct Qdisc *); /*    80     8 */
	int                        (*change)(struct Qdisc *, struct nlattr *, struct netlink_ext_ack *); /*    88     8 */
	void                       (*attach)(struct Qdisc *); /*    96     8 */
	int                        (*change_tx_queue_len)(struct Qdisc *, unsigned int); /*   104     8 */
	void                       (*change_real_num_tx)(struct Qdisc *, unsigned int); /*   112     8 */
	int                        (*dump)(struct Qdisc *, struct sk_buff *); /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	int                        (*dump_stats)(struct Qdisc *, struct gnet_dump *); /*   128     8 */
	void                       (*ingress_block_set)(struct Qdisc *, u32); /*   136     8 */
	void                       (*egress_block_set)(struct Qdisc *, u32); /*   144     8 */
	u32                        (*ingress_block_get)(struct Qdisc *); /*   152     8 */
	u32                        (*egress_block_get)(struct Qdisc *); /*   160     8 */
	struct module *            owner;                /*   168     8 */
	u64                        kabi_reserved1;       /*   176     8 */

	/* size: 184, cachelines: 3, members: 23 */
	/* last cacheline: 56 bytes */
};
struct Qdisc_class_ops {
	unsigned int               flags;                /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct netdev_queue *      (*select_queue)(struct Qdisc *, struct tcmsg *); /*     8     8 */
	int                        (*graft)(struct Qdisc *, long unsigned int, struct Qdisc *, struct Qdisc * *, struct netlink_ext_ack *); /*    16     8 */
	struct Qdisc *             (*leaf)(struct Qdisc *, long unsigned int); /*    24     8 */
	void                       (*qlen_notify)(struct Qdisc *, long unsigned int); /*    32     8 */
	long unsigned int          (*find)(struct Qdisc *, u32); /*    40     8 */
	int                        (*change)(struct Qdisc *, u32, u32, struct nlattr * *, long unsigned int *, struct netlink_ext_ack *); /*    48     8 */
	int                        (*delete)(struct Qdisc *, long unsigned int); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	void                       (*walk)(struct Qdisc *, struct qdisc_walker *); /*    64     8 */
	struct tcf_block *         (*tcf_block)(struct Qdisc *, long unsigned int, struct netlink_ext_ack *); /*    72     8 */
	long unsigned int          (*bind_tcf)(struct Qdisc *, long unsigned int, u32); /*    80     8 */
	void                       (*unbind_tcf)(struct Qdisc *, long unsigned int); /*    88     8 */
	int                        (*dump)(struct Qdisc *, long unsigned int, struct sk_buff *, struct tcmsg *); /*    96     8 */
	int                        (*dump_stats)(struct Qdisc *, long unsigned int, struct gnet_dump *); /*   104     8 */
	u64                        kabi_reserved1;       /*   112     8 */

	/* size: 120, cachelines: 2, members: 15 */
	/* sum members: 116, holes: 1, sum holes: 4 */
	/* last cacheline: 56 bytes */
};
struct tcf_block {
	struct mutex               lock;                 /*     0    32 */
	struct list_head           chain_list;           /*    32    16 */
	u32                        index;                /*    48     4 */
	u32                        classid;              /*    52     4 */
	refcount_t                 refcnt;               /*    56     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	struct net *               net;                  /*    64     8 */
	struct Qdisc *             q;                    /*    72     8 */
	struct rw_semaphore        cb_lock;              /*    80    40 */
	struct flow_block          flow_block;           /*   120    16 */
	/* --- cacheline 2 boundary (128 bytes) was 8 bytes ago --- */
	struct list_head           owner_list;           /*   136    16 */
	bool                       keep_dst;             /*   152     1 */

	/* XXX 3 bytes hole, try to pack */

	atomic_t                   offloadcnt;           /*   156     4 */
	unsigned int               nooffloaddevcnt;      /*   160     4 */
	unsigned int               lockeddevcnt;         /*   164     4 */
	struct {
		struct tcf_chain * chain;                /*   168     8 */
		struct list_head   filter_chain_list;    /*   176    16 */
	} chain0;                                        /*   168    24 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*   192    16 */
	struct hlist_head          proto_destroy_ht[128]; /*   208  1024 */
	/* --- cacheline 19 boundary (1216 bytes) was 16 bytes ago --- */
	struct mutex               proto_destroy_lock;   /*  1232    32 */

	/* size: 1264, cachelines: 20, members: 18 */
	/* sum members: 1257, holes: 2, sum holes: 7 */
	/* forced alignments: 1 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct tcf_proto {
	struct tcf_proto *         next;                 /*     0     8 */
	void *                     root;                 /*     8     8 */
	int                        (*classify)(struct sk_buff *, const struct tcf_proto  *, struct tcf_result *); /*    16     8 */
	__be16                     protocol;             /*    24     2 */

	/* XXX 2 bytes hole, try to pack */

	u32                        prio;                 /*    28     4 */
	void *                     data;                 /*    32     8 */
	const struct tcf_proto_ops  * ops;               /*    40     8 */
	struct tcf_chain *         chain;                /*    48     8 */
	spinlock_t                 lock;                 /*    56     4 */
	bool                       deleting;             /*    60     1 */

	/* XXX 3 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	refcount_t                 refcnt;               /*    64     4 */

	/* XXX 4 bytes hole, try to pack */

	struct callback_head       rcu __attribute__((__aligned__(8))); /*    72    16 */
	struct hlist_node          destroy_ht_node;      /*    88    16 */

	/* size: 104, cachelines: 2, members: 13 */
	/* sum members: 95, holes: 3, sum holes: 9 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 4 */
	/* last cacheline: 40 bytes */
} __attribute__((__aligned__(8)));
struct tcf_result {
	union {
		struct {
			long unsigned int class;         /*     0     8 */
			u32        classid;              /*     8     4 */
		};                                       /*     0    16 */
		const struct tcf_proto  * goto_tp;       /*     0     8 */
		struct {
			bool       ingress;              /*     0     1 */

			/* XXX 7 bytes hole, try to pack */

			struct gnet_stats_queue * qstats; /*     8     8 */
		};                                       /*     0    16 */
	};                                               /*     0    16 */

	/* size: 16, cachelines: 1, members: 1 */
	/* last cacheline: 16 bytes */
};
struct tcf_proto_ops {
	struct list_head           head;                 /*     0    16 */
	char                       kind[16];             /*    16    16 */
	int                        (*classify)(struct sk_buff *, const struct tcf_proto  *, struct tcf_result *); /*    32     8 */
	int                        (*init)(struct tcf_proto *); /*    40     8 */
	void                       (*destroy)(struct tcf_proto *, bool, struct netlink_ext_ack *); /*    48     8 */
	void *                     (*get)(struct tcf_proto *, u32); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	void                       (*put)(struct tcf_proto *, void *); /*    64     8 */
	int                        (*change)(struct net *, struct sk_buff *, struct tcf_proto *, long unsigned int, u32, struct nlattr * *, void * *, bool, bool, struct netlink_ext_ack *); /*    72     8 */
	int                        (*delete)(struct tcf_proto *, void *, bool *, bool, struct netlink_ext_ack *); /*    80     8 */
	bool                       (*delete_empty)(struct tcf_proto *); /*    88     8 */
	void                       (*walk)(struct tcf_proto *, struct tcf_walker *, bool); /*    96     8 */
	int                        (*reoffload)(struct tcf_proto *, bool, flow_setup_cb_t *, void *, struct netlink_ext_ack *); /*   104     8 */
	void                       (*hw_add)(struct tcf_proto *, void *); /*   112     8 */
	void                       (*hw_del)(struct tcf_proto *, void *); /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	void                       (*bind_class)(void *, u32, long unsigned int, void *, long unsigned int); /*   128     8 */
	void *                     (*tmplt_create)(struct net *, struct tcf_chain *, struct nlattr * *, struct netlink_ext_ack *); /*   136     8 */
	void                       (*tmplt_destroy)(void *); /*   144     8 */
	int                        (*dump)(struct net *, struct tcf_proto *, void *, struct sk_buff *, struct tcmsg *, bool); /*   152     8 */
	int                        (*terse_dump)(struct net *, struct tcf_proto *, void *, struct sk_buff *, struct tcmsg *, bool); /*   160     8 */
	int                        (*tmplt_dump)(struct sk_buff *, struct net *, void *); /*   168     8 */
	struct module *            owner;                /*   176     8 */
	int                        flags;                /*   184     4 */

	/* size: 192, cachelines: 3, members: 22 */
	/* padding: 4 */
};
struct tcf_chain {
	struct mutex               filter_chain_lock;    /*     0    32 */
	struct tcf_proto *         filter_chain;         /*    32     8 */
	struct list_head           list;                 /*    40    16 */
	struct tcf_block *         block;                /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u32                        index;                /*    64     4 */
	unsigned int               refcnt;               /*    68     4 */
	unsigned int               action_refcnt;        /*    72     4 */
	bool                       explicitly_created;   /*    76     1 */
	bool                       flushing;             /*    77     1 */

	/* XXX 2 bytes hole, try to pack */

	const struct tcf_proto_ops  * tmplt_ops;         /*    80     8 */
	void *                     tmplt_priv;           /*    88     8 */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*    96    16 */

	/* size: 112, cachelines: 2, members: 12 */
	/* sum members: 110, holes: 1, sum holes: 2 */
	/* forced alignments: 1 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct sock_filter {
	__u16                      code;                 /*     0     2 */
	__u8                       jt;                   /*     2     1 */
	__u8                       jf;                   /*     3     1 */
	__u32                      k;                    /*     4     4 */

	/* size: 8, cachelines: 1, members: 4 */
	/* last cacheline: 8 bytes */
};
struct sock_fprog_kern {
	u16                        len;                  /*     0     2 */

	/* XXX 6 bytes hole, try to pack */

	struct sock_filter *       filter;               /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 10, holes: 1, sum holes: 6 */
	/* last cacheline: 16 bytes */
};
struct sk_filter {
	refcount_t                 refcnt;               /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct callback_head       rcu __attribute__((__aligned__(8))); /*     8    16 */
	struct bpf_prog *          prog;                 /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 4 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct neigh_table {
	int                        family;               /*     0     4 */
	unsigned int               entry_size;           /*     4     4 */
	unsigned int               key_len;              /*     8     4 */
	__be16                     protocol;             /*    12     2 */

	/* XXX 2 bytes hole, try to pack */

	__u32                      (*hash)(const void  *, const struct net_device  *, __u32 *); /*    16     8 */
	bool                       (*key_eq)(const struct neighbour  *, const void  *); /*    24     8 */
	int                        (*constructor)(struct neighbour *); /*    32     8 */
	int                        (*pconstructor)(struct pneigh_entry *); /*    40     8 */
	void                       (*pdestructor)(struct pneigh_entry *); /*    48     8 */
	void                       (*proxy_redo)(struct sk_buff *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*is_multicast)(const void  *); /*    64     8 */
	bool                       (*allow_add)(const struct net_device  *, struct netlink_ext_ack *); /*    72     8 */
	char *                     id;                   /*    80     8 */
	struct neigh_parms         parms __attribute__((__aligned__(8))); /*    88   152 */
	/* --- cacheline 3 boundary (192 bytes) was 48 bytes ago --- */
	struct list_head           parms_list;           /*   240    16 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	int                        gc_interval;          /*   256     4 */
	int                        gc_thresh1;           /*   260     4 */
	int                        gc_thresh2;           /*   264     4 */
	int                        gc_thresh3;           /*   268     4 */
	long unsigned int          last_flush;           /*   272     8 */
	struct delayed_work        gc_work;              /*   280   192 */
	/* --- cacheline 7 boundary (448 bytes) was 24 bytes ago --- */
	struct timer_list          proxy_timer;          /*   472    72 */
	/* --- cacheline 8 boundary (512 bytes) was 32 bytes ago --- */
	struct sk_buff_head        proxy_queue;          /*   544    24 */
	atomic_t                   entries;              /*   568     4 */
	atomic_t                   gc_entries;           /*   572     4 */
	/* --- cacheline 9 boundary (576 bytes) --- */
	struct list_head           gc_list;              /*   576    16 */
	rwlock_t                   lock;                 /*   592     8 */
	long unsigned int          last_rand;            /*   600     8 */
	struct neigh_statistics *  stats;                /*   608     8 */
	struct neigh_hash_table *  nht;                  /*   616     8 */
	struct pneigh_entry * *    phash_buckets;        /*   624     8 */
	u64                        kabi_reserved1;       /*   632     8 */

	/* size: 640, cachelines: 10, members: 32 */
	/* sum members: 638, holes: 1, sum holes: 2 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(8)));
struct neigh_statistics {
	long unsigned int          allocs;               /*     0     8 */
	long unsigned int          destroys;             /*     8     8 */
	long unsigned int          hash_grows;           /*    16     8 */
	long unsigned int          res_failed;           /*    24     8 */
	long unsigned int          lookups;              /*    32     8 */
	long unsigned int          hits;                 /*    40     8 */
	long unsigned int          rcv_probes_mcast;     /*    48     8 */
	long unsigned int          rcv_probes_ucast;     /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	long unsigned int          periodic_gc_runs;     /*    64     8 */
	long unsigned int          forced_gc_runs;       /*    72     8 */
	long unsigned int          unres_discards;       /*    80     8 */
	long unsigned int          table_fulls;          /*    88     8 */

	/* size: 96, cachelines: 2, members: 12 */
	/* last cacheline: 32 bytes */
};
struct neigh_ops {
	int                        family;               /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	void                       (*solicit)(struct neighbour *, struct sk_buff *); /*     8     8 */
	void                       (*error_report)(struct neighbour *, struct sk_buff *); /*    16     8 */
	int                        (*output)(struct neighbour *, struct sk_buff *); /*    24     8 */
	int                        (*connected_output)(struct neighbour *, struct sk_buff *); /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* sum members: 36, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct pneigh_entry {
	struct pneigh_entry *      next;                 /*     0     8 */
	possible_net_t             net;                  /*     8     8 */
	struct net_device *        dev;                  /*    16     8 */
	u8                         flags;                /*    24     1 */
	u8                         protocol;             /*    25     1 */
	u8                         key[];                /*    26     0 */

	/* size: 32, cachelines: 1, members: 6 */
	/* padding: 6 */
	/* last cacheline: 32 bytes */
};
struct neigh_hash_table {
	struct neighbour * *       hash_buckets;         /*     0     8 */
	unsigned int               hash_shift;           /*     8     4 */
	__u32                      hash_rnd[4];          /*    12    16 */

	/* XXX 4 bytes hole, try to pack */

	struct callback_head       rcu __attribute__((__aligned__(8))); /*    32    16 */

	/* size: 48, cachelines: 1, members: 4 */
	/* sum members: 44, holes: 1, sum holes: 4 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 4 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct fib_rule_hdr {
	__u8                       family;               /*     0     1 */
	__u8                       dst_len;              /*     1     1 */
	__u8                       src_len;              /*     2     1 */
	__u8                       tos;                  /*     3     1 */
	__u8                       table;                /*     4     1 */
	__u8                       res1;                 /*     5     1 */
	__u8                       res2;                 /*     6     1 */
	__u8                       action;               /*     7     1 */
	__u32                      flags;                /*     8     4 */

	/* size: 12, cachelines: 1, members: 9 */
	/* last cacheline: 12 bytes */
};
struct fib_rule_port_range {
	__u16                      start;                /*     0     2 */
	__u16                      end;                  /*     2     2 */

	/* size: 4, cachelines: 1, members: 2 */
	/* last cacheline: 4 bytes */
};
struct fib_kuid_range {
	kuid_t                     start;                /*     0     4 */
	kuid_t                     end;                  /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct fib_rule {
	struct list_head           list;                 /*     0    16 */
	int                        iifindex;             /*    16     4 */
	int                        oifindex;             /*    20     4 */
	u32                        mark;                 /*    24     4 */
	u32                        mark_mask;            /*    28     4 */
	u32                        flags;                /*    32     4 */
	u32                        table;                /*    36     4 */
	u8                         action;               /*    40     1 */
	u8                         l3mdev;               /*    41     1 */
	u8                         proto;                /*    42     1 */
	u8                         ip_proto;             /*    43     1 */
	u32                        target;               /*    44     4 */
	__be64                     tun_id;               /*    48     8 */
	struct fib_rule *          ctarget;              /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct net *               fr_net;               /*    64     8 */
	refcount_t                 refcnt;               /*    72     4 */
	u32                        pref;                 /*    76     4 */
	int                        suppress_ifgroup;     /*    80     4 */
	int                        suppress_prefixlen;   /*    84     4 */
	char                       iifname[16];          /*    88    16 */
	char                       oifname[16];          /*   104    16 */
	struct fib_kuid_range      uid_range;            /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct fib_rule_port_range sport_range;          /*   128     4 */
	struct fib_rule_port_range dport_range;          /*   132     4 */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*   136    16 */
	u64                        kabi_reserved1;       /*   152     8 */
	u64                        kabi_reserved2;       /*   160     8 */
	u64                        kabi_reserved3;       /*   168     8 */
	u64                        kabi_reserved4;       /*   176     8 */
	u64                        kabi_reserved5;       /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	u64                        kabi_reserved6;       /*   192     8 */
	u64                        kabi_reserved7;       /*   200     8 */
	u64                        kabi_reserved8;       /*   208     8 */

	/* size: 216, cachelines: 4, members: 33 */
	/* forced alignments: 1 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct fib_lookup_arg {
	void *                     lookup_ptr;           /*     0     8 */
	const void  *              lookup_data;          /*     8     8 */
	void *                     result;               /*    16     8 */
	struct fib_rule *          rule;                 /*    24     8 */
	u32                        table;                /*    32     4 */
	int                        flags;                /*    36     4 */

	/* size: 40, cachelines: 1, members: 6 */
	/* last cacheline: 40 bytes */
};
struct sock_common {
	union {
		__addrpair         skc_addrpair;         /*     0     8 */
		struct {
			__be32     skc_daddr;            /*     0     4 */
			__be32     skc_rcv_saddr;        /*     4     4 */
		};                                       /*     0     8 */
	};                                               /*     0     8 */
	union {
		unsigned int       skc_hash;             /*     8     4 */
		__u16              skc_u16hashes[2];     /*     8     4 */
	};                                               /*     8     4 */
	union {
		__portpair         skc_portpair;         /*    12     4 */
		struct {
			__be16     skc_dport;            /*    12     2 */
			__u16      skc_num;              /*    14     2 */
		};                                       /*    12     4 */
	};                                               /*    12     4 */
	short unsigned int         skc_family;           /*    16     2 */
	volatile unsigned char     skc_state;            /*    18     1 */
	unsigned char              skc_reuse:4;          /*    19: 0  1 */
	unsigned char              skc_reuseport:1;      /*    19: 4  1 */
	unsigned char              skc_ipv6only:1;       /*    19: 5  1 */
	unsigned char              skc_net_refcnt:1;     /*    19: 6  1 */

	/* XXX 1 bit hole, try to pack */

	int                        skc_bound_dev_if;     /*    20     4 */
	union {
		struct hlist_node  skc_bind_node;        /*    24    16 */
		struct hlist_node  skc_portaddr_node;    /*    24    16 */
	};                                               /*    24    16 */
	struct proto *             skc_prot;             /*    40     8 */
	possible_net_t             skc_net;              /*    48     8 */
	struct in6_addr            skc_v6_daddr;         /*    56    16 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	struct in6_addr            skc_v6_rcv_saddr;     /*    72    16 */
	atomic64_t                 skc_cookie;           /*    88     8 */
	union {
		long unsigned int  skc_flags;            /*    96     8 */
		struct sock *      skc_listener;         /*    96     8 */
		struct inet_timewait_death_row * skc_tw_dr; /*    96     8 */
	};                                               /*    96     8 */
	int                        skc_dontcopy_begin[0]; /*   104     0 */
	union {
		struct hlist_node  skc_node;             /*   104    16 */
		struct hlist_nulls_node skc_nulls_node;  /*   104    16 */
	};                                               /*   104    16 */
	short unsigned int         skc_tx_queue_mapping; /*   120     2 */
	short unsigned int         skc_rx_queue_mapping; /*   122     2 */
	union {
		int                skc_incoming_cpu;     /*   124     4 */
		u32                skc_rcv_wnd;          /*   124     4 */
		u32                skc_tw_rcv_nxt;       /*   124     4 */
	};                                               /*   124     4 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	refcount_t                 skc_refcnt;           /*   128     4 */
	int                        skc_dontcopy_end[0];  /*   132     0 */
	union {
		u32                skc_rxhash;           /*   132     4 */
		u32                skc_window_clamp;     /*   132     4 */
		u32                skc_tw_snd_nxt;       /*   132     4 */
	};                                               /*   132     4 */

	/* size: 136, cachelines: 3, members: 25 */
	/* sum members: 135 */
	/* sum bitfield members: 7 bits, bit holes: 1, sum bit holes: 1 bits */
	/* last cacheline: 8 bytes */
};
struct proto {
	void                       (*close)(struct sock *, long int); /*     0     8 */
	int                        (*pre_connect)(struct sock *, struct sockaddr *, int); /*     8     8 */
	int                        (*connect)(struct sock *, struct sockaddr *, int); /*    16     8 */
	int                        (*disconnect)(struct sock *, int); /*    24     8 */
	struct sock *              (*accept)(struct sock *, int, int *, bool); /*    32     8 */
	int                        (*ioctl)(struct sock *, int, long unsigned int); /*    40     8 */
	int                        (*init)(struct sock *); /*    48     8 */
	void                       (*destroy)(struct sock *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	void                       (*shutdown)(struct sock *, int); /*    64     8 */
	int                        (*setsockopt)(struct sock *, int, int, sockptr_t, unsigned int); /*    72     8 */
	int                        (*getsockopt)(struct sock *, int, int, char *, int *); /*    80     8 */
	void                       (*keepalive)(struct sock *, int); /*    88     8 */
	int                        (*compat_ioctl)(struct sock *, unsigned int, long unsigned int); /*    96     8 */
	int                        (*sendmsg)(struct sock *, struct msghdr *, size_t); /*   104     8 */
	int                        (*recvmsg)(struct sock *, struct msghdr *, size_t, int, int, int *); /*   112     8 */
	int                        (*sendpage)(struct sock *, struct page *, int, size_t, int); /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	int                        (*bind)(struct sock *, struct sockaddr *, int); /*   128     8 */
	int                        (*bind_add)(struct sock *, struct sockaddr *, int); /*   136     8 */
	int                        (*backlog_rcv)(struct sock *, struct sk_buff *); /*   144     8 */
	void                       (*release_cb)(struct sock *); /*   152     8 */
	int                        (*hash)(struct sock *); /*   160     8 */
	void                       (*unhash)(struct sock *); /*   168     8 */
	void                       (*rehash)(struct sock *); /*   176     8 */
	int                        (*get_port)(struct sock *, short unsigned int); /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	unsigned int               inuse_idx;            /*   192     4 */

	/* XXX 4 bytes hole, try to pack */

	bool                       (*stream_memory_free)(const struct sock  *, int); /*   200     8 */
	bool                       (*stream_memory_read)(const struct sock  *); /*   208     8 */
	void                       (*enter_memory_pressure)(struct sock *); /*   216     8 */
	void                       (*leave_memory_pressure)(struct sock *); /*   224     8 */
	atomic_long_t *            memory_allocated;     /*   232     8 */
	struct percpu_counter *    sockets_allocated;    /*   240     8 */
	long unsigned int *        memory_pressure;      /*   248     8 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	long int *                 sysctl_mem;           /*   256     8 */
	int *                      sysctl_wmem;          /*   264     8 */
	int *                      sysctl_rmem;          /*   272     8 */
	u32                        sysctl_wmem_offset;   /*   280     4 */
	u32                        sysctl_rmem_offset;   /*   284     4 */
	int                        max_header;           /*   288     4 */
	bool                       no_autobind;          /*   292     1 */

	/* XXX 3 bytes hole, try to pack */

	struct kmem_cache *        slab;                 /*   296     8 */
	unsigned int               obj_size;             /*   304     4 */
	slab_flags_t               slab_flags;           /*   308     4 */
	unsigned int               useroffset;           /*   312     4 */
	unsigned int               usersize;             /*   316     4 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	unsigned int *             orphan_count;         /*   320     8 */
	struct request_sock_ops *  rsk_prot;             /*   328     8 */
	struct timewait_sock_ops * twsk_prot;            /*   336     8 */
	union {
		struct inet_hashinfo * hashinfo;         /*   344     8 */
		struct udp_table * udp_table;            /*   344     8 */
		struct raw_hashinfo * raw_hash;          /*   344     8 */
		struct smc_hashinfo * smc_hash;          /*   344     8 */
	} h;                                             /*   344     8 */
	struct module *            owner;                /*   352     8 */
	char                       name[32];             /*   360    32 */
	/* --- cacheline 6 boundary (384 bytes) was 8 bytes ago --- */
	struct list_head           node;                 /*   392    16 */
	int                        (*diag_destroy)(struct sock *, int); /*   408     8 */
	u64                        kabi_reserved1;       /*   416     8 */
	u64                        kabi_reserved2;       /*   424     8 */
	u64                        kabi_reserved3;       /*   432     8 */
	u64                        kabi_reserved4;       /*   440     8 */
	/* --- cacheline 7 boundary (448 bytes) --- */
	u64                        kabi_reserved5;       /*   448     8 */
	u64                        kabi_reserved6;       /*   456     8 */
	u64                        kabi_reserved7;       /*   464     8 */
	u64                        kabi_reserved8;       /*   472     8 */

	/* size: 480, cachelines: 8, members: 60 */
	/* sum members: 473, holes: 2, sum holes: 7 */
	/* last cacheline: 32 bytes */
};
struct request_sock_ops {
	int                        family;               /*     0     4 */
	unsigned int               obj_size;             /*     4     4 */
	struct kmem_cache *        slab;                 /*     8     8 */
	char *                     slab_name;            /*    16     8 */
	int                        (*rtx_syn_ack)(const struct sock  *, struct request_sock *); /*    24     8 */
	void                       (*send_ack)(const struct sock  *, struct sk_buff *, struct request_sock *); /*    32     8 */
	void                       (*send_reset)(const struct sock  *, struct sk_buff *); /*    40     8 */
	void                       (*destructor)(struct request_sock *); /*    48     8 */
	void                       (*syn_ack_timeout)(const struct request_sock  *); /*    56     8 */

	/* size: 64, cachelines: 1, members: 9 */
};
struct timewait_sock_ops {
	struct kmem_cache *        twsk_slab;            /*     0     8 */
	char *                     twsk_slab_name;       /*     8     8 */
	unsigned int               twsk_obj_size;        /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	int                        (*twsk_unique)(struct sock *, struct sock *, void *); /*    24     8 */
	void                       (*twsk_destructor)(struct sock *); /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* sum members: 36, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct request_sock {
	struct sock_common         __req_common;         /*     0   136 */
	/* --- cacheline 2 boundary (128 bytes) was 8 bytes ago --- */
	struct request_sock *      dl_next;              /*   136     8 */
	u16                        mss;                  /*   144     2 */
	u8                         num_retrans;          /*   146     1 */
	u8                         syncookie:1;          /*   147: 0  1 */
	u8                         num_timeout:7;        /*   147: 1  1 */
	u32                        ts_recent;            /*   148     4 */
	struct timer_list          rsk_timer;            /*   152    72 */
	/* --- cacheline 3 boundary (192 bytes) was 32 bytes ago --- */
	const struct request_sock_ops  * rsk_ops;        /*   224     8 */
	struct sock *              sk;                   /*   232     8 */
	struct saved_syn *         saved_syn;            /*   240     8 */
	u32                        secid;                /*   248     4 */
	u32                        peer_secid;           /*   252     4 */

	/* size: 256, cachelines: 4, members: 13 */
};
struct saved_syn {
	u32                        mac_hdrlen;           /*     0     4 */
	u32                        network_hdrlen;       /*     4     4 */
	u32                        tcp_hdrlen;           /*     8     4 */
	u8                         data[];               /*    12     0 */

	/* size: 12, cachelines: 1, members: 4 */
	/* last cacheline: 12 bytes */
};
struct ip6_sf_list {
	struct ip6_sf_list *       sf_next;              /*     0     8 */
	struct in6_addr            sf_addr;              /*     8    16 */
	long unsigned int          sf_count[2];          /*    24    16 */
	unsigned char              sf_gsresp;            /*    40     1 */
	unsigned char              sf_oldin;             /*    41     1 */
	unsigned char              sf_crcount;           /*    42     1 */

	/* size: 48, cachelines: 1, members: 6 */
	/* padding: 5 */
	/* last cacheline: 48 bytes */
};
struct ifmcaddr6 {
	struct in6_addr            mca_addr;             /*     0    16 */
	struct inet6_dev *         idev;                 /*    16     8 */
	struct ifmcaddr6 *         next;                 /*    24     8 */
	struct ip6_sf_list *       mca_sources;          /*    32     8 */
	struct ip6_sf_list *       mca_tomb;             /*    40     8 */
	unsigned int               mca_sfmode;           /*    48     4 */
	unsigned char              mca_crcount;          /*    52     1 */

	/* XXX 3 bytes hole, try to pack */

	long unsigned int          mca_sfcount[2];       /*    56    16 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	struct timer_list          mca_timer;            /*    72    72 */
	/* --- cacheline 2 boundary (128 bytes) was 16 bytes ago --- */
	unsigned int               mca_flags;            /*   144     4 */
	int                        mca_users;            /*   148     4 */
	refcount_t                 mca_refcnt;           /*   152     4 */
	spinlock_t                 mca_lock;             /*   156     4 */
	long unsigned int          mca_cstamp;           /*   160     8 */
	long unsigned int          mca_tstamp;           /*   168     8 */

	/* size: 176, cachelines: 3, members: 15 */
	/* sum members: 173, holes: 1, sum holes: 3 */
	/* last cacheline: 48 bytes */
};
struct ifacaddr6 {
	struct in6_addr            aca_addr;             /*     0    16 */
	struct fib6_info *         aca_rt;               /*    16     8 */
	struct ifacaddr6 *         aca_next;             /*    24     8 */
	struct hlist_node          aca_addr_lst;         /*    32    16 */
	int                        aca_users;            /*    48     4 */
	refcount_t                 aca_refcnt;           /*    52     4 */
	long unsigned int          aca_cstamp;           /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	long unsigned int          aca_tstamp;           /*    64     8 */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*    72    16 */

	/* size: 88, cachelines: 2, members: 9 */
	/* forced alignments: 1 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct ipv6_devstat {
	struct proc_dir_entry *    proc_dir_entry;       /*     0     8 */
	struct ipstats_mib *       ipv6;                 /*     8     8 */
	struct icmpv6_mib_device * icmpv6dev;            /*    16     8 */
	struct icmpv6msg_mib_device * icmpv6msgdev;      /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct nd_opt_hdr {
	__u8                       nd_opt_type;          /*     0     1 */
	__u8                       nd_opt_len;           /*     1     1 */

	/* size: 2, cachelines: 1, members: 2 */
	/* last cacheline: 2 bytes */
};
struct ndisc_options {
	struct nd_opt_hdr *        nd_opt_array[15];     /*     0   120 */
	/* --- cacheline 1 boundary (64 bytes) was 56 bytes ago --- */
	struct nd_opt_hdr *        nd_opts_ri;           /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct nd_opt_hdr *        nd_opts_ri_end;       /*   128     8 */
	struct nd_opt_hdr *        nd_useropts;          /*   136     8 */
	struct nd_opt_hdr *        nd_useropts_end;      /*   144     8 */
	struct nd_opt_hdr *        nd_802154_opt_array[3]; /*   152    24 */

	/* size: 176, cachelines: 3, members: 6 */
	/* last cacheline: 48 bytes */
};
struct posix_acl_entry {
	short int                  e_tag;                /*     0     2 */
	short unsigned int         e_perm;               /*     2     2 */
	union {
		kuid_t             e_uid;                /*     4     4 */
		kgid_t             e_gid;                /*     4     4 */
	};                                               /*     4     4 */

	/* size: 8, cachelines: 1, members: 3 */
	/* last cacheline: 8 bytes */
};
struct uuidcmp {
	const char  *              uuid;                 /*     0     8 */
	int                        len;                  /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct subprocess_info {
	struct work_struct         work;                 /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct completion *        complete;             /*    64     8 */
	const char  *              path;                 /*    72     8 */
	char * *                   argv;                 /*    80     8 */
	char * *                   envp;                 /*    88     8 */
	int                        wait;                 /*    96     4 */
	int                        retval;               /*   100     4 */
	int                        (*init)(struct subprocess_info *, struct cred *); /*   104     8 */
	void                       (*cleanup)(struct subprocess_info *); /*   112     8 */
	void *                     data;                 /*   120     8 */

	/* size: 128, cachelines: 2, members: 10 */
};
struct resource {
	resource_size_t            start;                /*     0     8 */
	resource_size_t            end;                  /*     8     8 */
	const char  *              name;                 /*    16     8 */
	long unsigned int          flags;                /*    24     8 */
	long unsigned int          desc;                 /*    32     8 */
	struct resource *          parent;               /*    40     8 */
	struct resource *          sibling;              /*    48     8 */
	struct resource *          child;                /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u64                        kabi_reserved1;       /*    64     8 */
	u64                        kabi_reserved2;       /*    72     8 */
	u64                        kabi_reserved3;       /*    80     8 */
	u64                        kabi_reserved4;       /*    88     8 */

	/* size: 96, cachelines: 2, members: 12 */
	/* last cacheline: 32 bytes */
};
struct metadata_hdr {
	char                       c_size[8];            /*     0     8 */
	char                       c_version;            /*     8     1 */
	char                       c_type;               /*     9     1 */
	char                       c_metadata[];         /*    10     0 */

	/* size: 10, cachelines: 1, members: 4 */
	/* last cacheline: 10 bytes */
};
struct hash {
	int                        ino;                  /*     0     4 */
	int                        minor;                /*     4     4 */
	int                        major;                /*     8     4 */
	umode_t                    mode;                 /*    12     2 */

	/* XXX 2 bytes hole, try to pack */

	struct hash *              next;                 /*    16     8 */
	char                       name[4098];           /*    24  4098 */

	/* size: 4128, cachelines: 65, members: 6 */
	/* sum members: 4120, holes: 1, sum holes: 2 */
	/* padding: 6 */
	/* last cacheline: 32 bytes */
};
struct dir_entry {
	struct list_head           list;                 /*     0    16 */
	char *                     name;                 /*    16     8 */
	time64_t                   mtime;                /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* last cacheline: 32 bytes */
};
struct io_bitmap {
	u64                        sequence;             /*     0     8 */
	refcount_t                 refcnt;               /*     8     4 */
	unsigned int               max;                  /*    12     4 */
	long unsigned int          bitmap[1024];         /*    16  8192 */

	/* size: 8208, cachelines: 129, members: 4 */
	/* last cacheline: 16 bytes */
};
struct syscall_metadata {
	const char  *              name;                 /*     0     8 */
	int                        syscall_nr;           /*     8     4 */
	int                        nb_args;              /*    12     4 */
	const char  * *            types;                /*    16     8 */
	const char  * *            args;                 /*    24     8 */
	struct list_head           enter_fields;         /*    32    16 */
	struct trace_event_call *  enter_event;          /*    48     8 */
	struct trace_event_call *  exit_event;           /*    56     8 */

	/* size: 64, cachelines: 1, members: 8 */
};
struct alt_instr {
	s32                        instr_offset;         /*     0     4 */
	s32                        repl_offset;          /*     4     4 */
	u16                        cpuid;                /*     8     2 */
	u8                         instrlen;             /*    10     1 */
	u8                         replacementlen;       /*    11     1 */
	u8                         padlen;               /*    12     1 */

	/* size: 13, cachelines: 1, members: 6 */
	/* last cacheline: 13 bytes */
} __attribute__((__packed__));
struct cpuinfo_x86 {
	__u8                       x86;                  /*     0     1 */
	__u8                       x86_vendor;           /*     1     1 */
	__u8                       x86_model;            /*     2     1 */
	__u8                       x86_stepping;         /*     3     1 */
	int                        x86_tlbsize;          /*     4     4 */
	__u32                      vmx_capability[3];    /*     8    12 */
	__u8                       x86_virt_bits;        /*    20     1 */
	__u8                       x86_phys_bits;        /*    21     1 */
	__u8                       x86_coreid_bits;      /*    22     1 */
	__u8                       cu_id;                /*    23     1 */
	__u32                      extended_cpuid_level; /*    24     4 */
	int                        cpuid_level;          /*    28     4 */
	union {
		__u32              x86_capability[20];   /*    32    80 */
		long unsigned int  x86_capability_alignment; /*    32     8 */
	};                                               /*    32    80 */
	/* --- cacheline 1 boundary (64 bytes) was 48 bytes ago --- */
	char                       x86_vendor_id[16];    /*   112    16 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	char                       x86_model_id[64];     /*   128    64 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	unsigned int               x86_cache_size;       /*   192     4 */
	int                        x86_cache_alignment;  /*   196     4 */
	int                        x86_cache_max_rmid;   /*   200     4 */
	int                        x86_cache_occ_scale;  /*   204     4 */
	int                        x86_cache_mbm_width_offset; /*   208     4 */
	int                        x86_power;            /*   212     4 */
	long unsigned int          loops_per_jiffy;      /*   216     8 */
	u16                        x86_max_cores;        /*   224     2 */
	u16                        apicid;               /*   226     2 */
	u16                        initial_apicid;       /*   228     2 */
	u16                        x86_clflush_size;     /*   230     2 */
	u16                        booted_cores;         /*   232     2 */
	u16                        phys_proc_id;         /*   234     2 */
	u16                        logical_proc_id;      /*   236     2 */
	u16                        cpu_core_id;          /*   238     2 */
	u16                        cpu_die_id;           /*   240     2 */
	u16                        logical_die_id;       /*   242     2 */
	u16                        cpu_index;            /*   244     2 */

	/* XXX 2 bytes hole, try to pack */

	u32                        microcode;            /*   248     4 */
	u8                         x86_cache_bits;       /*   252     1 */

	/* Bitfield combined with previous fields */

	unsigned int               initialized:1;        /*   252: 8  4 */

	/* size: 256, cachelines: 4, members: 36 */
	/* sum members: 251, holes: 1, sum holes: 2 */
	/* sum bitfield members: 1 bits (0 bytes) */
	/* bit_padding: 23 bits */
};
struct timens_offset {
	s64                        sec;                  /*     0     8 */
	u64                        nsec;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct vm_special_mapping {
	const char  *              name;                 /*     0     8 */
	struct page * *            pages;                /*     8     8 */
	vm_fault_t                 (*fault)(const struct vm_special_mapping  *, struct vm_area_struct *, struct vm_fault *); /*    16     8 */
	int                        (*mremap)(const struct vm_special_mapping  *, struct vm_area_struct *); /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct time_namespace {
	struct kref                kref;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct user_namespace *    user_ns;              /*     8     8 */
	struct ucounts *           ucounts;              /*    16     8 */
	struct ns_common           ns;                   /*    24    32 */
	struct timens_offsets      offsets;              /*    56    32 */
	/* --- cacheline 1 boundary (64 bytes) was 24 bytes ago --- */
	struct page *              vvar_page;            /*    88     8 */
	bool                       frozen_offsets;       /*    96     1 */

	/* size: 104, cachelines: 2, members: 7 */
	/* sum members: 93, holes: 1, sum holes: 4 */
	/* padding: 7 */
	/* last cacheline: 40 bytes */
};
struct timens_offsets {
	struct timespec64          monotonic;            /*     0    16 */
	struct timespec64          boottime;             /*    16    16 */

	/* size: 32, cachelines: 1, members: 2 */
	/* last cacheline: 32 bytes */
};
struct pvclock_vcpu_time_info {
	u32                        version;              /*     0     4 */
	u32                        pad0;                 /*     4     4 */
	u64                        tsc_timestamp;        /*     8     8 */
	u64                        system_time;          /*    16     8 */
	u32                        tsc_to_system_mul;    /*    24     4 */
	s8                         tsc_shift;            /*    28     1 */
	u8                         flags;                /*    29     1 */
	u8                         pad[2];               /*    30     2 */

	/* size: 32, cachelines: 1, members: 8 */
	/* last cacheline: 32 bytes */
};
struct pvclock_vsyscall_time_info {
	struct pvclock_vcpu_time_info pvti;              /*     0    32 */

	/* size: 64, cachelines: 1, members: 1 */
	/* padding: 32 */
} __attribute__((__aligned__(64)));
struct arch_vdso_data {

	/* size: 0, cachelines: 0, members: 0 */
};
struct vdso_timestamp {
	u64                        sec;                  /*     0     8 */
	u64                        nsec;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct vdso_data {
	u32                        seq;                  /*     0     4 */
	s32                        clock_mode;           /*     4     4 */
	u64                        cycle_last;           /*     8     8 */
	u64                        mask;                 /*    16     8 */
	u32                        mult;                 /*    24     4 */
	u32                        shift;                /*    28     4 */
	union {
		struct vdso_timestamp basetime[12];      /*    32   192 */
		struct timens_offset offset[12];         /*    32   192 */
	};                                               /*    32   192 */
	/* --- cacheline 3 boundary (192 bytes) was 32 bytes ago --- */
	s32                        tz_minuteswest;       /*   224     4 */
	s32                        tz_dsttime;           /*   228     4 */
	u32                        hrtimer_res;          /*   232     4 */
	u16                        vdso_fix;             /*   236     2 */
	u16                        vdso_shift;           /*   238     2 */
	struct arch_vdso_data      arch_data;            /*   240     0 */

	/* size: 240, cachelines: 4, members: 13 */
	/* last cacheline: 48 bytes */
};
struct ms_hyperv_tsc_page {
	volatile u32               tsc_sequence;         /*     0     4 */
	u32                        reserved1;            /*     4     4 */
	volatile u64               tsc_scale;            /*     8     8 */
	volatile s64               tsc_offset;           /*    16     8 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct vdso_exception_table_entry {
	int                        insn;                 /*     0     4 */
	int                        fixup;                /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct trace_event_raw_emulate_vsyscall {
	struct trace_entry         ent;                  /*     0     8 */
	int                        nr;                   /*     8     4 */
	char                       __data[];             /*    12     0 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct trace_event_data_offsets_emulate_vsyscall {

	/* size: 0, cachelines: 0, members: 0 */
};
struct perf_event_mmap_page {
	__u32                      version;              /*     0     4 */
	__u32                      compat_version;       /*     4     4 */
	__u32                      lock;                 /*     8     4 */
	__u32                      index;                /*    12     4 */
	__s64                      offset;               /*    16     8 */
	__u64                      time_enabled;         /*    24     8 */
	__u64                      time_running;         /*    32     8 */
	union {
		__u64              capabilities;         /*    40     8 */
		struct {
			__u64      cap_bit0:1;           /*    40: 0  8 */
			__u64      cap_bit0_is_deprecated:1; /*    40: 1  8 */
			__u64      cap_user_rdpmc:1;     /*    40: 2  8 */
			__u64      cap_user_time:1;      /*    40: 3  8 */
			__u64      cap_user_time_zero:1; /*    40: 4  8 */
			__u64      cap_user_time_short:1; /*    40: 5  8 */
			__u64      cap_____res:58;       /*    40: 6  8 */
		};                                       /*    40     8 */
	};                                               /*    40     8 */
	__u16                      pmc_width;            /*    48     2 */
	__u16                      time_shift;           /*    50     2 */
	__u32                      time_mult;            /*    52     4 */
	__u64                      time_offset;          /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	__u64                      time_zero;            /*    64     8 */
	__u32                      size;                 /*    72     4 */
	__u32                      __reserved_1;         /*    76     4 */
	__u64                      time_cycles;          /*    80     8 */
	__u64                      time_mask;            /*    88     8 */
	__u8                       __reserved[928];      /*    96   928 */
	/* --- cacheline 16 boundary (1024 bytes) --- */
	__u64                      data_head;            /*  1024     8 */
	__u64                      data_tail;            /*  1032     8 */
	__u64                      data_offset;          /*  1040     8 */
	__u64                      data_size;            /*  1048     8 */
	__u64                      aux_head;             /*  1056     8 */
	__u64                      aux_tail;             /*  1064     8 */
	__u64                      aux_offset;           /*  1072     8 */
	__u64                      aux_size;             /*  1080     8 */

	/* size: 1088, cachelines: 17, members: 26 */
};
struct ldt_struct {
	struct desc_struct *       entries;              /*     0     8 */
	unsigned int               nr_entries;           /*     8     4 */
	int                        slot;                 /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct physid_mask {
	long unsigned int          mask[512];            /*     0  4096 */

	/* size: 4096, cachelines: 64, members: 1 */
};
struct device_attribute {
	struct attribute           attr;                 /*     0    16 */

	/* XXX last struct has 6 bytes of padding */

	ssize_t                    (*show)(struct device *, struct device_attribute *, char *); /*    16     8 */
	ssize_t                    (*store)(struct device *, struct device_attribute *, const char  *, size_t); /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* paddings: 1, sum paddings: 6 */
	/* last cacheline: 32 bytes */
};
struct x86_pmu_capability {
	int                        version;              /*     0     4 */
	int                        num_counters_gp;      /*     4     4 */
	int                        num_counters_fixed;   /*     8     4 */
	int                        bit_width_gp;         /*    12     4 */
	int                        bit_width_fixed;      /*    16     4 */
	unsigned int               events_mask;          /*    20     4 */
	int                        events_mask_len;      /*    24     4 */

	/* size: 28, cachelines: 1, members: 7 */
	/* last cacheline: 28 bytes */
};
struct debug_store {
	u64                        bts_buffer_base;      /*     0     8 */
	u64                        bts_index;            /*     8     8 */
	u64                        bts_absolute_maximum; /*    16     8 */
	u64                        bts_interrupt_threshold; /*    24     8 */
	u64                        pebs_buffer_base;     /*    32     8 */
	u64                        pebs_index;           /*    40     8 */
	u64                        pebs_absolute_maximum; /*    48     8 */
	u64                        pebs_interrupt_threshold; /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u64                        pebs_event_reset[12]; /*    64    96 */

	/* size: 4096, cachelines: 64, members: 9 */
	/* padding: 3936 */
} __attribute__((__aligned__(4096)));
struct stack_info {
	enum stack_type            type;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int *        begin;                /*     8     8 */
	long unsigned int *        end;                  /*    16     8 */
	long unsigned int *        next_sp;              /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct stack_frame {
	struct stack_frame *       next_frame;           /*     0     8 */
	long unsigned int          return_address;       /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct stack_frame_ia32 {
	u32                        next_frame;           /*     0     4 */
	u32                        return_address;       /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct perf_guest_switch_msr {
	unsigned int               msr;                  /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        host;                 /*     8     8 */
	u64                        guest;                /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct perf_guest_info_callbacks {
	int                        (*is_in_guest)(void); /*     0     8 */
	int                        (*is_user_mode)(void); /*     8     8 */
	long unsigned int          (*get_guest_ip)(void); /*    16     8 */
	void                       (*handle_intel_pt_intr)(void); /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct perf_callchain_entry_ctx {
	struct perf_callchain_entry * entry;             /*     0     8 */
	u32                        max_stack;            /*     8     4 */
	u32                        nr;                   /*    12     4 */
	short int                  contexts;             /*    16     2 */
	bool                       contexts_maxed;       /*    18     1 */

	/* size: 24, cachelines: 1, members: 5 */
	/* padding: 5 */
	/* last cacheline: 24 bytes */
};
struct perf_pmu_events_attr {
	struct device_attribute    attr;                 /*     0    32 */
	u64                        id;                   /*    32     8 */
	const char  *              event_str;            /*    40     8 */

	/* size: 48, cachelines: 1, members: 3 */
	/* last cacheline: 48 bytes */
};
struct perf_pmu_events_ht_attr {
	struct device_attribute    attr;                 /*     0    32 */
	u64                        id;                   /*    32     8 */
	const char  *              event_str_ht;         /*    40     8 */
	const char  *              event_str_noht;       /*    48     8 */

	/* size: 56, cachelines: 1, members: 4 */
	/* last cacheline: 56 bytes */
};
struct apic {
	void                       (*eoi_write)(u32, u32); /*     0     8 */
	void                       (*native_eoi_write)(u32, u32); /*     8     8 */
	void                       (*write)(u32, u32);   /*    16     8 */
	u32                        (*read)(u32);         /*    24     8 */
	void                       (*wait_icr_idle)(void); /*    32     8 */
	u32                        (*safe_wait_icr_idle)(void); /*    40     8 */
	void                       (*send_IPI)(int, int); /*    48     8 */
	void                       (*send_IPI_mask)(const struct cpumask  *, int); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	void                       (*send_IPI_mask_allbutself)(const struct cpumask  *, int); /*    64     8 */
	void                       (*send_IPI_allbutself)(int); /*    72     8 */
	void                       (*send_IPI_all)(int); /*    80     8 */
	void                       (*send_IPI_self)(int); /*    88     8 */
	u32                        dest_logical;         /*    96     4 */
	u32                        disable_esr;          /*   100     4 */
	u32                        irq_delivery_mode;    /*   104     4 */
	u32                        irq_dest_mode;        /*   108     4 */
	u32                        (*calc_dest_apicid)(unsigned int); /*   112     8 */
	u64                        (*icr_read)(void);    /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	void                       (*icr_write)(u32, u32); /*   128     8 */
	int                        (*probe)(void);       /*   136     8 */
	int                        (*acpi_madt_oem_check)(char *, char *); /*   144     8 */
	int                        (*apic_id_valid)(u32); /*   152     8 */
	int                        (*apic_id_registered)(void); /*   160     8 */
	bool                       (*check_apicid_used)(physid_mask_t *, int); /*   168     8 */
	void                       (*init_apic_ldr)(void); /*   176     8 */
	void                       (*ioapic_phys_id_map)(physid_mask_t *, physid_mask_t *); /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	void                       (*setup_apic_routing)(void); /*   192     8 */
	int                        (*cpu_present_to_apicid)(int); /*   200     8 */
	void                       (*apicid_to_cpu_present)(int, physid_mask_t *); /*   208     8 */
	int                        (*check_phys_apicid_present)(int); /*   216     8 */
	int                        (*phys_pkg_id)(int, int); /*   224     8 */
	u32                        (*get_apic_id)(long unsigned int); /*   232     8 */
	u32                        (*set_apic_id)(unsigned int); /*   240     8 */
	int                        (*wakeup_secondary_cpu)(int, long unsigned int); /*   248     8 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	void                       (*inquire_remote_apic)(int); /*   256     8 */
	char *                     name;                 /*   264     8 */

	/* size: 272, cachelines: 5, members: 36 */
	/* last cacheline: 16 bytes */
};
struct nmiaction {
	struct list_head           list;                 /*     0    16 */
	nmi_handler_t              handler;              /*    16     8 */
	u64                        max_duration;         /*    24     8 */
	long unsigned int          flags;                /*    32     8 */
	const char  *              name;                 /*    40     8 */

	/* size: 48, cachelines: 1, members: 5 */
	/* last cacheline: 48 bytes */
};
struct gdt_page {
	struct desc_struct         gdt[16];              /*     0   128 */

	/* size: 4096, cachelines: 64, members: 1 */
	/* padding: 3968 */
} __attribute__((__aligned__(4096)));
struct cyc2ns_data {
	u32                        cyc2ns_mul;           /*     0     4 */
	u32                        cyc2ns_shift;         /*     4     4 */
	u64                        cyc2ns_offset;        /*     8     8 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct unwind_state {
	struct stack_info          stack_info;           /*     0    32 */
	long unsigned int          stack_mask;           /*    32     8 */
	struct task_struct *       task;                 /*    40     8 */
	int                        graph_idx;            /*    48     4 */
	bool                       error;                /*    52     1 */
	bool                       signal;               /*    53     1 */
	bool                       full_regs;            /*    54     1 */

	/* XXX 1 byte hole, try to pack */

	long unsigned int          sp;                   /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	long unsigned int          bp;                   /*    64     8 */
	long unsigned int          ip;                   /*    72     8 */
	struct pt_regs *           regs;                 /*    80     8 */
	struct pt_regs *           prev_regs;            /*    88     8 */

	/* size: 96, cachelines: 2, members: 12 */
	/* sum members: 95, holes: 1, sum holes: 1 */
	/* last cacheline: 32 bytes */
};
struct event_constraint {
	union {
		long unsigned int  idxmsk[1];            /*     0     8 */
		u64                idxmsk64;             /*     0     8 */
	};                                               /*     0     8 */
	u64                        code;                 /*     8     8 */
	u64                        cmask;                /*    16     8 */
	int                        weight;               /*    24     4 */
	int                        overlap;              /*    28     4 */
	int                        flags;                /*    32     4 */
	unsigned int               size;                 /*    36     4 */

	/* size: 40, cachelines: 1, members: 7 */
	/* last cacheline: 40 bytes */
};
struct amd_nb {
	int                        nb_id;                /*     0     4 */
	int                        refcnt;               /*     4     4 */
	struct perf_event *        owners[64];           /*     8   512 */
	/* --- cacheline 8 boundary (512 bytes) was 8 bytes ago --- */
	struct event_constraint    event_constraints[64]; /*   520  2560 */

	/* size: 3080, cachelines: 49, members: 4 */
	/* last cacheline: 8 bytes */
};
struct er_account {
	raw_spinlock_t             lock;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        config;               /*     8     8 */
	u64                        reg;                  /*    16     8 */
	atomic_t                   ref;                  /*    24     4 */

	/* size: 32, cachelines: 1, members: 4 */
	/* sum members: 24, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct intel_shared_regs {
	struct er_account          regs[5];              /*     0   160 */
	/* --- cacheline 2 boundary (128 bytes) was 32 bytes ago --- */
	int                        refcnt;               /*   160     4 */
	unsigned int               core_id;              /*   164     4 */

	/* size: 168, cachelines: 3, members: 3 */
	/* last cacheline: 40 bytes */
};
struct intel_excl_states {
	enum intel_excl_state_type state[64];            /*     0   256 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	bool                       sched_started;        /*   256     1 */

	/* size: 260, cachelines: 5, members: 2 */
	/* padding: 3 */
	/* last cacheline: 4 bytes */
};
struct intel_excl_cntrs {
	raw_spinlock_t             lock;                 /*     0     4 */
	struct intel_excl_states   states[2];            /*     4   520 */
	/* --- cacheline 8 boundary (512 bytes) was 12 bytes ago --- */
	union {
		u16                has_exclusive[2];     /*   524     4 */
		u32                exclusive_present;    /*   524     4 */
	};                                               /*   524     4 */
	int                        refcnt;               /*   528     4 */
	unsigned int               core_id;              /*   532     4 */

	/* size: 536, cachelines: 9, members: 5 */
	/* last cacheline: 24 bytes */
};
struct cpu_hw_events {
	struct perf_event *        events[64];           /*     0   512 */
	/* --- cacheline 8 boundary (512 bytes) --- */
	long unsigned int          active_mask[1];       /*   512     8 */
	long unsigned int          running[1];           /*   520     8 */
	int                        enabled;              /*   528     4 */
	int                        n_events;             /*   532     4 */
	int                        n_added;              /*   536     4 */
	int                        n_txn;                /*   540     4 */
	int                        n_txn_pair;           /*   544     4 */
	int                        n_txn_metric;         /*   548     4 */
	int                        assign[64];           /*   552   256 */
	/* --- cacheline 12 boundary (768 bytes) was 40 bytes ago --- */
	u64                        tags[64];             /*   808   512 */
	/* --- cacheline 20 boundary (1280 bytes) was 40 bytes ago --- */
	struct perf_event *        event_list[64];       /*  1320   512 */
	/* --- cacheline 28 boundary (1792 bytes) was 40 bytes ago --- */
	struct event_constraint *  event_constraint[64]; /*  1832   512 */
	/* --- cacheline 36 boundary (2304 bytes) was 40 bytes ago --- */
	int                        n_excl;               /*  2344     4 */
	unsigned int               txn_flags;            /*  2348     4 */
	int                        is_fake;              /*  2352     4 */

	/* XXX 4 bytes hole, try to pack */

	struct debug_store *       ds;                   /*  2360     8 */
	/* --- cacheline 37 boundary (2368 bytes) --- */
	void *                     ds_pebs_vaddr;        /*  2368     8 */
	void *                     ds_bts_vaddr;         /*  2376     8 */
	u64                        pebs_enabled;         /*  2384     8 */
	int                        n_pebs;               /*  2392     4 */
	int                        n_large_pebs;         /*  2396     4 */
	int                        n_pebs_via_pt;        /*  2400     4 */
	int                        pebs_output;          /*  2404     4 */
	u64                        pebs_data_cfg;        /*  2408     8 */
	u64                        active_pebs_data_cfg; /*  2416     8 */
	int                        pebs_record_size;     /*  2424     4 */
	int                        lbr_users;            /*  2428     4 */
	/* --- cacheline 38 boundary (2432 bytes) --- */
	int                        lbr_pebs_users;       /*  2432     4 */

	/* XXX 4 bytes hole, try to pack */

	struct perf_branch_stack   lbr_stack;            /*  2440    16 */
	struct perf_branch_entry   lbr_entries[32];      /*  2456   768 */
	/* --- cacheline 50 boundary (3200 bytes) was 24 bytes ago --- */
	union {
		struct er_account * lbr_sel;             /*  3224     8 */
		struct er_account * lbr_ctl;             /*  3224     8 */
	};                                               /*  3224     8 */
	u64                        br_sel;               /*  3232     8 */
	void *                     last_task_ctx;        /*  3240     8 */
	int                        last_log_id;          /*  3248     4 */
	int                        lbr_select;           /*  3252     4 */
	void *                     lbr_xsave;            /*  3256     8 */
	/* --- cacheline 51 boundary (3264 bytes) --- */
	u64                        intel_ctrl_guest_mask; /*  3264     8 */
	u64                        intel_ctrl_host_mask; /*  3272     8 */
	struct perf_guest_switch_msr guest_switch_msrs[64]; /*  3280  1536 */
	/* --- cacheline 75 boundary (4800 bytes) was 16 bytes ago --- */
	u64                        intel_cp_status;      /*  4816     8 */
	struct intel_shared_regs * shared_regs;          /*  4824     8 */
	struct event_constraint *  constraint_list;      /*  4832     8 */
	struct intel_excl_cntrs *  excl_cntrs;           /*  4840     8 */
	int                        excl_thread_id;       /*  4848     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        tfa_shadow;           /*  4856     8 */
	/* --- cacheline 76 boundary (4864 bytes) --- */
	int                        n_metric;             /*  4864     4 */

	/* XXX 4 bytes hole, try to pack */

	struct amd_nb *            amd_nb;               /*  4872     8 */
	int                        brs_active;           /*  4880     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        perf_ctr_virt_mask;   /*  4888     8 */
	int                        n_pair;               /*  4896     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     kfree_on_online[2];   /*  4904    16 */
	struct pmu *               pmu;                  /*  4920     8 */

	/* size: 4928, cachelines: 77, members: 53 */
	/* sum members: 4904, holes: 6, sum holes: 24 */
};
struct extra_reg {
	unsigned int               event;                /*     0     4 */
	unsigned int               msr;                  /*     4     4 */
	u64                        config_mask;          /*     8     8 */
	u64                        valid_mask;           /*    16     8 */
	int                        idx;                  /*    24     4 */
	bool                       extra_msr_access;     /*    28     1 */

	/* size: 32, cachelines: 1, members: 6 */
	/* padding: 3 */
	/* last cacheline: 32 bytes */
};
union perf_capabilities {
	struct {
		u64                lbr_format:6;       /*     0: 0  8 */
		u64                pebs_trap:1;        /*     0: 6  8 */
		u64                pebs_arch_reg:1;    /*     0: 7  8 */
		u64                pebs_format:4;      /*     0: 8  8 */
		u64                smm_freeze:1;       /*     0:12  8 */
		u64                full_width_write:1; /*     0:13  8 */
		u64                pebs_baseline:1;    /*     0:14  8 */
		u64                perf_metrics:1;     /*     0:15  8 */
		u64                pebs_output_pt_available:1; /*     0:16  8 */
		u64                anythread_deprecated:1; /*     0:17  8 */
	};                                             /*     0     8 */
	u64                        capabilities;       /*     0     8 */
};
struct x86_pmu_quirk {
	struct x86_pmu_quirk *     next;                 /*     0     8 */
	void                       (*func)(void);        /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct x86_pmu {
	const char  *              name;                 /*     0     8 */
	int                        version;              /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	int                        (*handle_irq)(struct pt_regs *); /*    16     8 */
	void                       (*disable_all)(void); /*    24     8 */
	void                       (*enable_all)(int);   /*    32     8 */
	void                       (*enable)(struct perf_event *); /*    40     8 */
	void                       (*disable)(struct perf_event *); /*    48     8 */
	void                       (*add)(struct perf_event *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	void                       (*del)(struct perf_event *); /*    64     8 */
	void                       (*read)(struct perf_event *); /*    72     8 */
	int                        (*hw_config)(struct perf_event *); /*    80     8 */
	int                        (*schedule_events)(struct cpu_hw_events *, int, int *); /*    88     8 */
	unsigned int               eventsel;             /*    96     4 */
	unsigned int               perfctr;              /*   100     4 */
	int                        (*addr_offset)(int, bool); /*   104     8 */
	int                        (*rdpmc_index)(int);  /*   112     8 */
	u64                        (*event_map)(int);    /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	int                        max_events;           /*   128     4 */
	int                        num_counters;         /*   132     4 */
	int                        num_counters_fixed;   /*   136     4 */
	int                        cntval_bits;          /*   140     4 */
	u64                        cntval_mask;          /*   144     8 */
	union {
		long unsigned int  events_maskl;         /*   152     8 */
		long unsigned int  events_mask[1];       /*   152     8 */
	};                                               /*   152     8 */
	int                        events_mask_len;      /*   160     4 */
	int                        apic;                 /*   164     4 */
	u64                        max_period;           /*   168     8 */
	struct event_constraint *  (*get_event_constraints)(struct cpu_hw_events *, int, struct perf_event *); /*   176     8 */
	void                       (*put_event_constraints)(struct cpu_hw_events *, struct perf_event *); /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	void                       (*start_scheduling)(struct cpu_hw_events *); /*   192     8 */
	void                       (*commit_scheduling)(struct cpu_hw_events *, int, int); /*   200     8 */
	void                       (*stop_scheduling)(struct cpu_hw_events *); /*   208     8 */
	struct event_constraint *  event_constraints;    /*   216     8 */
	struct x86_pmu_quirk *     quirks;               /*   224     8 */
	int                        perfctr_second_write; /*   232     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        (*limit_period)(struct perf_event *, u64); /*   240     8 */
	unsigned int               late_ack:1;           /*   248: 0  4 */
	unsigned int               enabled_ack:1;        /*   248: 1  4 */
	unsigned int               counter_freezing:1;   /*   248: 2  4 */

	/* XXX 29 bits hole, try to pack */

	int                        attr_rdpmc_broken;    /*   252     4 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	int                        attr_rdpmc;           /*   256     4 */

	/* XXX 4 bytes hole, try to pack */

	struct attribute * *       format_attrs;         /*   264     8 */
	ssize_t                    (*events_sysfs_show)(char *, u64); /*   272     8 */
	const struct attribute_group  * * attr_update;   /*   280     8 */
	long unsigned int          attr_freeze_on_smi;   /*   288     8 */
	int                        (*cpu_prepare)(int);  /*   296     8 */
	void                       (*cpu_starting)(int); /*   304     8 */
	void                       (*cpu_dying)(int);    /*   312     8 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	void                       (*cpu_dead)(int);     /*   320     8 */
	void                       (*check_microcode)(void); /*   328     8 */
	void                       (*sched_task)(struct perf_event_context *, bool); /*   336     8 */
	u64                        intel_ctrl;           /*   344     8 */
	union perf_capabilities    intel_cap;            /*   352     8 */
	unsigned int               bts:1;                /*   360: 0  4 */
	unsigned int               bts_active:1;         /*   360: 1  4 */
	unsigned int               pebs:1;               /*   360: 2  4 */
	unsigned int               pebs_active:1;        /*   360: 3  4 */
	unsigned int               pebs_broken:1;        /*   360: 4  4 */
	unsigned int               pebs_prec_dist:1;     /*   360: 5  4 */
	unsigned int               pebs_no_tlb:1;        /*   360: 6  4 */
	unsigned int               pebs_no_isolation:1;  /*   360: 7  4 */
	unsigned int               pebs_block:1;         /*   360: 8  4 */

	/* XXX 23 bits hole, try to pack */

	int                        pebs_record_size;     /*   364     4 */
	int                        pebs_buffer_size;     /*   368     4 */
	int                        max_pebs_events;      /*   372     4 */
	void                       (*drain_pebs)(struct pt_regs *, struct perf_sample_data *); /*   376     8 */
	/* --- cacheline 6 boundary (384 bytes) --- */
	struct event_constraint *  pebs_constraints;     /*   384     8 */
	void                       (*pebs_aliases)(struct perf_event *); /*   392     8 */
	long unsigned int          large_pebs_flags;     /*   400     8 */
	u64                        rtm_abort_event;      /*   408     8 */
	unsigned int               lbr_tos;              /*   416     4 */
	unsigned int               lbr_from;             /*   420     4 */
	unsigned int               lbr_to;               /*   424     4 */
	unsigned int               lbr_info;             /*   428     4 */
	unsigned int               lbr_nr;               /*   432     4 */

	/* XXX 4 bytes hole, try to pack */

	union {
		u64                lbr_sel_mask;         /*   440     8 */
		u64                lbr_ctl_mask;         /*   440     8 */
	};                                               /*   440     8 */
	/* --- cacheline 7 boundary (448 bytes) --- */
	union {
		const int  *       lbr_sel_map;          /*   448     8 */
		int *              lbr_ctl_map;          /*   448     8 */
	};                                               /*   448     8 */
	bool                       lbr_double_abort;     /*   456     1 */
	bool                       lbr_pt_coexist;       /*   457     1 */

	/* Bitfield combined with previous fields */

	unsigned int               lbr_depth_mask:8;     /*   456:16  4 */
	unsigned int               lbr_deep_c_reset:1;   /*   456:24  4 */
	unsigned int               lbr_lip:1;            /*   456:25  4 */
	unsigned int               lbr_cpl:1;            /*   456:26  4 */
	unsigned int               lbr_filter:1;         /*   456:27  4 */
	unsigned int               lbr_call_stack:1;     /*   456:28  4 */
	unsigned int               lbr_mispred:1;        /*   456:29  4 */
	unsigned int               lbr_timed_lbr:1;      /*   456:30  4 */
	unsigned int               lbr_br_type:1;        /*   456:31  4 */

	/* XXX 4 bytes hole, try to pack */

	void                       (*lbr_reset)(void);   /*   464     8 */
	void                       (*lbr_read)(struct cpu_hw_events *); /*   472     8 */
	void                       (*lbr_save)(void *);  /*   480     8 */
	void                       (*lbr_restore)(void *); /*   488     8 */
	atomic_t                   lbr_exclusive[3];     /*   496    12 */
	int                        num_topdown_events;   /*   508     4 */
	/* --- cacheline 8 boundary (512 bytes) --- */
	u64                        (*update_topdown_event)(struct perf_event *); /*   512     8 */
	int                        (*set_topdown_event_period)(struct perf_event *); /*   520     8 */
	void                       (*swap_task_ctx)(struct perf_event_context *, struct perf_event_context *); /*   528     8 */
	unsigned int               amd_nb_constraints:1; /*   536: 0  4 */

	/* XXX 31 bits hole, try to pack */
	/* XXX 4 bytes hole, try to pack */

	u64                        perf_ctr_pair_en;     /*   544     8 */
	struct extra_reg *         extra_regs;           /*   552     8 */
	unsigned int               flags;                /*   560     4 */

	/* XXX 4 bytes hole, try to pack */

	struct perf_guest_switch_msr * (*guest_get_msrs)(int *); /*   568     8 */
	/* --- cacheline 9 boundary (576 bytes) --- */
	int                        (*check_period)(struct perf_event *, u64); /*   576     8 */
	int                        (*aux_output_match)(struct perf_event *); /*   584     8 */

	/* size: 592, cachelines: 10, members: 103 */
	/* sum members: 550, holes: 7, sum holes: 28 */
	/* sum bitfield members: 29 bits, bit holes: 3, sum bit holes: 83 bits */
	/* last cacheline: 16 bytes */
};
struct sched_state {
	int                        weight;               /*     0     4 */
	int                        event;                /*     4     4 */
	int                        counter;              /*     8     4 */
	int                        unassigned;           /*    12     4 */
	int                        nr_gp;                /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        used;                 /*    24     8 */

	/* size: 32, cachelines: 1, members: 6 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct perf_sched {
	int                        max_weight;           /*     0     4 */
	int                        max_events;           /*     4     4 */
	int                        max_gp;               /*     8     4 */
	int                        saved_states;         /*    12     4 */
	struct event_constraint * * constraints;         /*    16     8 */
	struct sched_state         state;                /*    24    32 */
	struct sched_state         saved[2];             /*    56    64 */

	/* size: 120, cachelines: 2, members: 7 */
	/* last cacheline: 56 bytes */
};
struct perf_msr {
	u64                        msr;                  /*     0     8 */
	struct attribute_group *   grp;                  /*     8     8 */
	bool                       (*test)(int, void *); /*    16     8 */
	bool                       no_check;             /*    24     1 */

	/* size: 32, cachelines: 1, members: 4 */
	/* padding: 7 */
	/* last cacheline: 32 bytes */
};
union cpuid_0x80000022_ebx {
	struct {
		unsigned int       num_core_pmc:4;     /*     0: 0  4 */
		unsigned int       reserved:6;         /*     0: 4  4 */
		unsigned int       num_df_pmc:6;       /*     0:10  4 */
	} split;                                       /*     0     4 */
	unsigned int               full;               /*     0     4 */
};
union amd_debug_extn_cfg {
	__u64                      val;                /*     0     8 */
	struct {
		__u64              rsvd0:2;            /*     0: 0  8 */
		__u64              brsmen:1;           /*     0: 2  8 */
		__u64              rsvd4_3:2;          /*     0: 3  8 */
		__u64              vb:1;               /*     0: 5  8 */
		__u64              rsvd2:10;           /*     0: 6  8 */
		__u64              msroff:4;           /*     0:16  8 */
		__u64              rsvd3:4;            /*     0:20  8 */
		__u64              pmc:3;              /*     0:24  8 */
		__u64              rsvd4:37;           /*     0:27  8 */
	};                                             /*     0     8 */
};
struct pci_dev {
	struct list_head           bus_list;             /*     0    16 */
	struct pci_bus *           bus;                  /*    16     8 */
	struct pci_bus *           subordinate;          /*    24     8 */
	void *                     sysdata;              /*    32     8 */
	struct proc_dir_entry *    procent;              /*    40     8 */
	struct pci_slot *          slot;                 /*    48     8 */
	unsigned int               devfn;                /*    56     4 */
	short unsigned int         vendor;               /*    60     2 */
	short unsigned int         device;               /*    62     2 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	short unsigned int         subsystem_vendor;     /*    64     2 */
	short unsigned int         subsystem_device;     /*    66     2 */
	unsigned int               class;                /*    68     4 */
	u8                         revision;             /*    72     1 */
	u8                         hdr_type;             /*    73     1 */
	u16                        aer_cap;              /*    74     2 */

	/* XXX 4 bytes hole, try to pack */

	struct aer_stats *         aer_stats;            /*    80     8 */
	u8                         pcie_cap;             /*    88     1 */
	u8                         msi_cap;              /*    89     1 */
	u8                         msix_cap;             /*    90     1 */
	u8                         pcie_mpss:3;          /*    91: 0  1 */

	/* XXX 5 bits hole, try to pack */

	u8                         rom_base_reg;         /*    92     1 */
	u8                         pin;                  /*    93     1 */
	u16                        pcie_flags_reg;       /*    94     2 */
	long unsigned int *        dma_alias_mask;       /*    96     8 */
	struct pci_driver *        driver;               /*   104     8 */
	u64                        dma_mask;             /*   112     8 */
	struct device_dma_parameters dma_parms;          /*   120    16 */
	/* --- cacheline 2 boundary (128 bytes) was 8 bytes ago --- */
	pci_power_t                current_state;        /*   136     4 */
	unsigned int               imm_ready:1;          /*   140: 0  4 */

	/* XXX 7 bits hole, try to pack */
	/* Bitfield combined with next fields */

	u8                         pm_cap;               /*   141     1 */

	/* Bitfield combined with previous fields */

	unsigned int               pme_support:5;        /*   140:16  4 */
	unsigned int               pme_poll:1;           /*   140:21  4 */
	unsigned int               d1_support:1;         /*   140:22  4 */
	unsigned int               d2_support:1;         /*   140:23  4 */
	unsigned int               no_d1d2:1;            /*   140:24  4 */
	unsigned int               no_d3cold:1;          /*   140:25  4 */
	unsigned int               bridge_d3:1;          /*   140:26  4 */
	unsigned int               d3cold_allowed:1;     /*   140:27  4 */
	unsigned int               mmio_always_on:1;     /*   140:28  4 */
	unsigned int               wakeup_prepared:1;    /*   140:29  4 */
	unsigned int               runtime_d3cold:1;     /*   140:30  4 */
	unsigned int               skip_bus_pm:1;        /*   140:31  4 */
	unsigned int               ignore_hotplug:1;     /*   144: 0  4 */
	unsigned int               hotplug_user_indicators:1; /*   144: 1  4 */
	unsigned int               clear_retrain_link:1; /*   144: 2  4 */

	/* XXX 29 bits hole, try to pack */

	unsigned int               d3hot_delay;          /*   148     4 */
	unsigned int               d3cold_delay;         /*   152     4 */

	/* XXX 4 bytes hole, try to pack */

	struct pcie_link_state *   link_state;           /*   160     8 */
	unsigned int               ltr_path:1;           /*   168: 0  4 */

	/* XXX 31 bits hole, try to pack */

	int                        l1ss;                 /*   172     4 */
	unsigned int               pasid_no_tlp:1;       /*   176: 0  4 */
	unsigned int               eetlp_prefix_path:1;  /*   176: 1  4 */

	/* XXX 30 bits hole, try to pack */

	pci_channel_state_t        error_state;          /*   180     4 */
	struct device              dev __attribute__((__aligned__(8))); /*   184  1000 */
	/* --- cacheline 18 boundary (1152 bytes) was 32 bytes ago --- */
	int                        cfg_size;             /*  1184     4 */
	unsigned int               irq;                  /*  1188     4 */
	struct resource            resource[17];         /*  1192  1632 */
	/* --- cacheline 44 boundary (2816 bytes) was 8 bytes ago --- */
	bool                       match_driver;         /*  2824     1 */

	/* Bitfield combined with previous fields */

	unsigned int               transparent:1;        /*  2824: 8  4 */
	unsigned int               io_window:1;          /*  2824: 9  4 */
	unsigned int               pref_window:1;        /*  2824:10  4 */
	unsigned int               pref_64_window:1;     /*  2824:11  4 */
	unsigned int               multifunction:1;      /*  2824:12  4 */
	unsigned int               is_busmaster:1;       /*  2824:13  4 */
	unsigned int               no_msi:1;             /*  2824:14  4 */
	unsigned int               no_64bit_msi:1;       /*  2824:15  4 */
	unsigned int               block_cfg_access:1;   /*  2824:16  4 */
	unsigned int               broken_parity_status:1; /*  2824:17  4 */
	unsigned int               irq_reroute_variant:2; /*  2824:18  4 */
	unsigned int               msi_enabled:1;        /*  2824:20  4 */
	unsigned int               msix_enabled:1;       /*  2824:21  4 */
	unsigned int               ari_enabled:1;        /*  2824:22  4 */
	unsigned int               ats_enabled:1;        /*  2824:23  4 */
	unsigned int               pasid_enabled:1;      /*  2824:24  4 */
	unsigned int               pri_enabled:1;        /*  2824:25  4 */
	unsigned int               is_managed:1;         /*  2824:26  4 */
	unsigned int               needs_freset:1;       /*  2824:27  4 */
	unsigned int               state_saved:1;        /*  2824:28  4 */
	unsigned int               is_physfn:1;          /*  2824:29  4 */
	unsigned int               is_virtfn:1;          /*  2824:30  4 */
	unsigned int               reset_fn:1;           /*  2824:31  4 */
	unsigned int               is_hotplug_bridge:1;  /*  2828: 0  4 */
	unsigned int               shpc_managed:1;       /*  2828: 1  4 */
	unsigned int               is_thunderbolt:1;     /*  2828: 2  4 */
	unsigned int               untrusted:1;          /*  2828: 3  4 */
	unsigned int               external_facing:1;    /*  2828: 4  4 */
	unsigned int               broken_intx_masking:1; /*  2828: 5  4 */
	unsigned int               io_window_1k:1;       /*  2828: 6  4 */
	unsigned int               irq_managed:1;        /*  2828: 7  4 */
	unsigned int               non_compliant_bars:1; /*  2828: 8  4 */
	unsigned int               is_probed:1;          /*  2828: 9  4 */
	unsigned int               link_active_reporting:1; /*  2828:10  4 */
	unsigned int               no_vf_scan:1;         /*  2828:11  4 */
	unsigned int               no_command_memory:1;  /*  2828:12  4 */

	/* XXX 3 bits hole, try to pack */
	/* Bitfield combined with next fields */

	pci_dev_flags_t            dev_flags;            /*  2830     2 */
	atomic_t                   enable_cnt;           /*  2832     4 */
	u32                        saved_config_space[16]; /*  2836    64 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 45 boundary (2880 bytes) was 24 bytes ago --- */
	struct hlist_head          saved_cap_space;      /*  2904     8 */
	struct bin_attribute *     rom_attr;             /*  2912     8 */
	int                        rom_attr_enabled;     /*  2920     4 */

	/* XXX 4 bytes hole, try to pack */

	struct bin_attribute *     res_attr[17];         /*  2928   136 */
	/* --- cacheline 47 boundary (3008 bytes) was 56 bytes ago --- */
	struct bin_attribute *     res_attr_wc[17];      /*  3064   136 */
	/* --- cacheline 50 boundary (3200 bytes) --- */
	unsigned int               broken_cmd_compl:1;   /*  3200: 0  4 */

	/* XXX 31 bits hole, try to pack */
	/* XXX 4 bytes hole, try to pack */

	const struct attribute_group  * * msi_irq_groups; /*  3208     8 */
	struct pci_vpd *           vpd;                  /*  3216     8 */
	u16                        dpc_cap;              /*  3224     2 */

	/* Bitfield combined with previous fields */

	unsigned int               dpc_rp_extensions:1;  /*  3224:16  4 */

	/* XXX 7 bits hole, try to pack */
	/* Bitfield combined with next fields */

	u8                         dpc_rp_log_size;      /*  3227     1 */

	/* XXX 4 bytes hole, try to pack */

	union {
		struct pci_sriov * sriov;                /*  3232     8 */
		struct pci_dev *   physfn;               /*  3232     8 */
	};                                               /*  3232     8 */
	u16                        ats_cap;              /*  3240     2 */
	u8                         ats_stu;              /*  3242     1 */

	/* XXX 1 byte hole, try to pack */

	u16                        pri_cap;              /*  3244     2 */

	/* XXX 2 bytes hole, try to pack */

	u32                        pri_reqs_alloc;       /*  3248     4 */
	unsigned int               pasid_required:1;     /*  3252: 0  4 */

	/* XXX 15 bits hole, try to pack */
	/* Bitfield combined with next fields */

	u16                        pasid_cap;            /*  3254     2 */
	u16                        pasid_features;       /*  3256     2 */
	u16                        acs_cap;              /*  3258     2 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 51 boundary (3264 bytes) --- */
	phys_addr_t                rom;                  /*  3264     8 */
	size_t                     romlen;               /*  3272     8 */
	char *                     driver_override;      /*  3280     8 */
	long unsigned int          priv_flags;           /*  3288     8 */
	long unsigned int          slot_being_removed_rescanned; /*  3296     8 */
	struct pci_dev *           rpdev;                /*  3304     8 */
	struct rcec_ea *           rcec_ea;              /*  3312     8 */
	struct pci_dev *           rcec;                 /*  3320     8 */
	/* --- cacheline 52 boundary (3328 bytes) --- */
	u64                        kabi_reserved3;       /*  3328     8 */
	u64                        kabi_reserved4;       /*  3336     8 */
	u64                        kabi_reserved5;       /*  3344     8 */
	u64                        kabi_reserved6;       /*  3352     8 */
	u64                        kabi_reserved7;       /*  3360     8 */
	u64                        kabi_reserved8;       /*  3368     8 */
	u64                        kabi_reserved9;       /*  3376     8 */
	u64                        kabi_reserved10;      /*  3384     8 */
	/* --- cacheline 53 boundary (3392 bytes) --- */
	u64                        kabi_reserved11;      /*  3392     8 */
	u64                        kabi_reserved12;      /*  3400     8 */
	u64                        kabi_reserved13;      /*  3408     8 */
	u64                        kabi_reserved14;      /*  3416     8 */
	u64                        kabi_reserved15;      /*  3424     8 */

	/* size: 3432, cachelines: 54, members: 138 */
	/* sum members: 3373, holes: 9, sum holes: 31 */
	/* sum bitfield members: 66 bits, bit holes: 9, sum bit holes: 158 bits */
	/* forced alignments: 1 */
	/* last cacheline: 40 bytes */
} __attribute__((__aligned__(8)));
struct pci_device_id {
	__u32                      vendor;               /*     0     4 */
	__u32                      device;               /*     4     4 */
	__u32                      subvendor;            /*     8     4 */
	__u32                      subdevice;            /*    12     4 */
	__u32                      class;                /*    16     4 */
	__u32                      class_mask;           /*    20     4 */
	kernel_ulong_t             driver_data;          /*    24     8 */

	/* size: 32, cachelines: 1, members: 7 */
	/* last cacheline: 32 bytes */
};
struct pci_slot {
	struct pci_bus *           bus;                  /*     0     8 */
	struct list_head           list;                 /*     8    16 */
	struct hotplug_slot *      hotplug;              /*    24     8 */
	unsigned char              number;               /*    32     1 */

	/* XXX 7 bytes hole, try to pack */

	struct kobject             kobj;                 /*    40    96 */

	/* size: 136, cachelines: 3, members: 5 */
	/* sum members: 129, holes: 1, sum holes: 7 */
	/* last cacheline: 8 bytes */
};
struct pci_bus {
	struct list_head           node;                 /*     0    16 */
	struct pci_bus *           parent;               /*    16     8 */
	struct list_head           children;             /*    24    16 */
	struct list_head           devices;              /*    40    16 */
	struct pci_dev *           self;                 /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct list_head           slots;                /*    64    16 */
	struct resource *          resource[4];          /*    80    32 */
	struct list_head           resources;            /*   112    16 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct resource            busn_res;             /*   128    96 */
	/* --- cacheline 3 boundary (192 bytes) was 32 bytes ago --- */
	struct pci_ops *           ops;                  /*   224     8 */
	struct pci_ops *           backup_ops;           /*   232     8 */
	struct msi_controller *    msi;                  /*   240     8 */
	void *                     sysdata;              /*   248     8 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	struct proc_dir_entry *    procdir;              /*   256     8 */
	unsigned char              number;               /*   264     1 */
	unsigned char              primary;              /*   265     1 */
	unsigned char              max_bus_speed;        /*   266     1 */
	unsigned char              cur_bus_speed;        /*   267     1 */
	char                       name[48];             /*   268    48 */
	short unsigned int         bridge_ctl;           /*   316     2 */
	pci_bus_flags_t            bus_flags;            /*   318     2 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	struct device *            bridge;               /*   320     8 */
	struct device              dev __attribute__((__aligned__(8))); /*   328  1000 */
	/* --- cacheline 20 boundary (1280 bytes) was 48 bytes ago --- */
	struct bin_attribute *     legacy_io;            /*  1328     8 */
	struct bin_attribute *     legacy_mem;           /*  1336     8 */
	/* --- cacheline 21 boundary (1344 bytes) --- */
	unsigned int               is_added:1;           /*  1344: 0  4 */

	/* XXX 31 bits hole, try to pack */
	/* XXX 4 bytes hole, try to pack */

	u64                        kabi_reserved1;       /*  1352     8 */
	u64                        kabi_reserved2;       /*  1360     8 */
	u64                        kabi_reserved3;       /*  1368     8 */
	u64                        kabi_reserved4;       /*  1376     8 */
	u64                        kabi_reserved5;       /*  1384     8 */
	u64                        kabi_reserved6;       /*  1392     8 */
	u64                        kabi_reserved7;       /*  1400     8 */

	/* size: 1408, cachelines: 22, members: 33 */
	/* sum members: 1400, holes: 1, sum holes: 4 */
	/* sum bitfield members: 1 bits, bit holes: 1, sum bit holes: 31 bits */
	/* forced alignments: 1 */
} __attribute__((__aligned__(8)));
struct pci_driver {
	struct list_head           node;                 /*     0    16 */
	const char  *              name;                 /*    16     8 */
	const struct pci_device_id  * id_table;          /*    24     8 */
	int                        (*probe)(struct pci_dev *, const struct pci_device_id  *); /*    32     8 */
	void                       (*remove)(struct pci_dev *); /*    40     8 */
	int                        (*suspend)(struct pci_dev *, pm_message_t); /*    48     8 */
	int                        (*resume)(struct pci_dev *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	void                       (*shutdown)(struct pci_dev *); /*    64     8 */
	int                        (*sriov_configure)(struct pci_dev *, int); /*    72     8 */
	const struct pci_error_handlers  * err_handler;  /*    80     8 */
	const struct attribute_group  * * groups;        /*    88     8 */
	struct device_driver       driver;               /*    96   192 */
	/* --- cacheline 4 boundary (256 bytes) was 32 bytes ago --- */
	struct pci_dynids          dynids;               /*   288    24 */
	u64                        kabi_reserved1;       /*   312     8 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	u64                        kabi_reserved2;       /*   320     8 */
	u64                        kabi_reserved3;       /*   328     8 */
	u64                        kabi_reserved4;       /*   336     8 */
	u64                        kabi_reserved5;       /*   344     8 */
	u64                        kabi_reserved6;       /*   352     8 */
	u64                        kabi_reserved7;       /*   360     8 */
	u64                        kabi_reserved8;       /*   368     8 */

	/* size: 376, cachelines: 6, members: 21 */
	/* last cacheline: 56 bytes */
};
struct pci_ops {
	int                        (*add_bus)(struct pci_bus *); /*     0     8 */
	void                       (*remove_bus)(struct pci_bus *); /*     8     8 */
	void *                     (*map_bus)(struct pci_bus *, unsigned int, int); /*    16     8 */
	int                        (*read)(struct pci_bus *, unsigned int, int, int, u32 *); /*    24     8 */
	int                        (*write)(struct pci_bus *, unsigned int, int, int, u32); /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct pci_dynids {
	spinlock_t                 lock;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           list;                 /*     8    16 */

	/* size: 24, cachelines: 1, members: 2 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct pci_error_handlers {
	pci_ers_result_t           (*error_detected)(struct pci_dev *, pci_channel_state_t); /*     0     8 */
	pci_ers_result_t           (*mmio_enabled)(struct pci_dev *); /*     8     8 */
	pci_ers_result_t           (*slot_reset)(struct pci_dev *); /*    16     8 */
	void                       (*reset_prepare)(struct pci_dev *); /*    24     8 */
	void                       (*reset_done)(struct pci_dev *); /*    32     8 */
	void                       (*resume)(struct pci_dev *); /*    40     8 */

	/* size: 48, cachelines: 1, members: 6 */
	/* last cacheline: 48 bytes */
};
struct syscore_ops {
	struct list_head           node;                 /*     0    16 */
	int                        (*suspend)(void);     /*    16     8 */
	void                       (*resume)(void);      /*    24     8 */
	void                       (*shutdown)(void);    /*    32     8 */

	/* size: 40, cachelines: 1, members: 4 */
	/* last cacheline: 40 bytes */
};
union ibs_fetch_ctl {
	__u64                      val;                /*     0     8 */
	struct {
		__u64              fetch_maxcnt:16;    /*     0: 0  8 */
		__u64              fetch_cnt:16;       /*     0:16  8 */
		__u64              fetch_lat:16;       /*     0:32  8 */
		__u64              fetch_en:1;         /*     0:48  8 */
		__u64              fetch_val:1;        /*     0:49  8 */
		__u64              fetch_comp:1;       /*     0:50  8 */
		__u64              ic_miss:1;          /*     0:51  8 */
		__u64              phy_addr_valid:1;   /*     0:52  8 */
		__u64              l1tlb_pgsz:2;       /*     0:53  8 */
		__u64              l1tlb_miss:1;       /*     0:55  8 */
		__u64              l2tlb_miss:1;       /*     0:56  8 */
		__u64              rand_en:1;          /*     0:57  8 */
		__u64              fetch_l2_miss:1;    /*     0:58  8 */
		__u64              l3_miss_only:1;     /*     0:59  8 */
		__u64              fetch_oc_miss:1;    /*     0:60  8 */
		__u64              fetch_l3_miss:1;    /*     0:61  8 */
		__u64              reserved:2;         /*     0:62  8 */
	};                                             /*     0     8 */
};
union ibs_op_ctl {
	__u64                      val;                /*     0     8 */
	struct {
		__u64              opmaxcnt:16;        /*     0: 0  8 */
		__u64              l3_miss_only:1;     /*     0:16  8 */
		__u64              op_en:1;            /*     0:17  8 */
		__u64              op_val:1;           /*     0:18  8 */
		__u64              cnt_ctl:1;          /*     0:19  8 */
		__u64              opmaxcnt_ext:7;     /*     0:20  8 */
		__u64              reserved0:5;        /*     0:27  8 */
		__u64              opcurcnt:27;        /*     0:32  8 */
		__u64              reserved1:5;        /*     0:59  8 */
	};                                             /*     0     8 */
};
union ibs_op_data {
	__u64                      val;                /*     0     8 */
	struct {
		__u64              comp_to_ret_ctr:16; /*     0: 0  8 */
		__u64              tag_to_ret_ctr:16;  /*     0:16  8 */
		__u64              reserved1:2;        /*     0:32  8 */
		__u64              op_return:1;        /*     0:34  8 */
		__u64              op_brn_taken:1;     /*     0:35  8 */
		__u64              op_brn_misp:1;      /*     0:36  8 */
		__u64              op_brn_ret:1;       /*     0:37  8 */
		__u64              op_rip_invalid:1;   /*     0:38  8 */
		__u64              op_brn_fuse:1;      /*     0:39  8 */
		__u64              op_microcode:1;     /*     0:40  8 */
		__u64              reserved2:23;       /*     0:41  8 */
	};                                             /*     0     8 */
};
union ibs_op_data2 {
	__u64                      val;                /*     0     8 */
	struct {
		__u64              data_src_lo:3;      /*     0: 0  8 */
		__u64              reserved0:1;        /*     0: 3  8 */
		__u64              rmt_node:1;         /*     0: 4  8 */
		__u64              cache_hit_st:1;     /*     0: 5  8 */
		__u64              data_src_hi:2;      /*     0: 6  8 */
		__u64              reserved1:56;       /*     0: 8  8 */
	};                                             /*     0     8 */
};
union ibs_op_data3 {
	__u64                      val;                /*     0     8 */
	struct {
		__u64              ld_op:1;            /*     0: 0  8 */
		__u64              st_op:1;            /*     0: 1  8 */
		__u64              dc_l1tlb_miss:1;    /*     0: 2  8 */
		__u64              dc_l2tlb_miss:1;    /*     0: 3  8 */
		__u64              dc_l1tlb_hit_2m:1;  /*     0: 4  8 */
		__u64              dc_l1tlb_hit_1g:1;  /*     0: 5  8 */
		__u64              dc_l2tlb_hit_2m:1;  /*     0: 6  8 */
		__u64              dc_miss:1;          /*     0: 7  8 */
		__u64              dc_mis_acc:1;       /*     0: 8  8 */
		__u64              reserved:4;         /*     0: 9  8 */
		__u64              dc_wc_mem_acc:1;    /*     0:13  8 */
		__u64              dc_uc_mem_acc:1;    /*     0:14  8 */
		__u64              dc_locked_op:1;     /*     0:15  8 */
		__u64              dc_miss_no_mab_alloc:1; /*     0:16  8 */
		__u64              dc_lin_addr_valid:1; /*     0:17  8 */
		__u64              dc_phy_addr_valid:1; /*     0:18  8 */
		__u64              dc_l2_tlb_hit_1g:1; /*     0:19  8 */
		__u64              l2_miss:1;          /*     0:20  8 */
		__u64              sw_pf:1;            /*     0:21  8 */
		__u64              op_mem_width:4;     /*     0:22  8 */
		__u64              op_dc_miss_open_mem_reqs:6; /*     0:26  8 */
		__u64              dc_miss_lat:16;     /*     0:32  8 */
		__u64              tlb_refill_lat:16;  /*     0:48  8 */
	};                                             /*     0     8 */
};
struct perf_ibs_data {
	u32                        size;                 /*     0     4 */
	union {
		u32                data[0];              /*     4     0 */
		u32                caps;                 /*     4     4 */
	};                                               /*     4     4 */
	u64                        regs[8];              /*     8    64 */

	/* size: 72, cachelines: 2, members: 3 */
	/* last cacheline: 8 bytes */
};
struct cpu_perf_ibs {
	struct perf_event *        event;                /*     0     8 */
	long unsigned int          state[1];             /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct perf_ibs {
	struct pmu                 pmu;                  /*     0   344 */
	/* --- cacheline 5 boundary (320 bytes) was 24 bytes ago --- */
	unsigned int               msr;                  /*   344     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        config_mask;          /*   352     8 */
	u64                        cnt_mask;             /*   360     8 */
	u64                        enable_mask;          /*   368     8 */
	u64                        valid_mask;           /*   376     8 */
	/* --- cacheline 6 boundary (384 bytes) --- */
	u64                        max_period;           /*   384     8 */
	long unsigned int          offset_mask[1];       /*   392     8 */
	int                        offset_max;           /*   400     4 */
	unsigned int               fetch_count_reset_broken:1; /*   404: 0  4 */
	unsigned int               fetch_ignore_if_zero_rip:1; /*   404: 1  4 */

	/* XXX 30 bits hole, try to pack */

	struct cpu_perf_ibs *      pcpu;                 /*   408     8 */
	u64                        (*get_count)(u64);    /*   416     8 */

	/* size: 424, cachelines: 7, members: 13 */
	/* sum members: 416, holes: 1, sum holes: 4 */
	/* sum bitfield members: 2 bits, bit holes: 1, sum bit holes: 30 bits */
	/* last cacheline: 40 bytes */
};
struct amd_uncore {
	int                        id;                   /*     0     4 */
	int                        refcnt;               /*     4     4 */
	int                        cpu;                  /*     8     4 */
	int                        num_counters;         /*    12     4 */
	int                        rdpmc_base;           /*    16     4 */
	u32                        msr_base;             /*    20     4 */
	cpumask_t *                active_mask;          /*    24     8 */
	struct pmu *               pmu;                  /*    32     8 */
	struct perf_event * *      events;               /*    40     8 */
	struct hlist_node          node;                 /*    48    16 */

	/* size: 64, cachelines: 1, members: 10 */
};
struct perf_amd_iommu {
	struct list_head           list;                 /*     0    16 */
	struct pmu                 pmu;                  /*    16   344 */
	/* --- cacheline 5 boundary (320 bytes) was 40 bytes ago --- */
	struct amd_iommu *         iommu;                /*   360     8 */
	char                       name[16];             /*   368    16 */
	/* --- cacheline 6 boundary (384 bytes) --- */
	u8                         max_banks;            /*   384     1 */
	u8                         max_counters;         /*   385     1 */

	/* XXX 6 bytes hole, try to pack */

	u64                        cntr_assign_mask;     /*   392     8 */
	raw_spinlock_t             lock;                 /*   400     4 */

	/* size: 408, cachelines: 7, members: 8 */
	/* sum members: 398, holes: 1, sum holes: 6 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct amd_iommu_event_desc {
	struct device_attribute    attr;                 /*     0    32 */
	const char  *              event;                /*    32     8 */

	/* size: 40, cachelines: 1, members: 2 */
	/* last cacheline: 40 bytes */
};
struct x86_cpu_desc {
	u8                         x86_family;           /*     0     1 */
	u8                         x86_vendor;           /*     1     1 */
	u8                         x86_model;            /*     2     1 */
	u8                         x86_stepping;         /*     3     1 */
	u32                        x86_microcode_rev;    /*     4     4 */

	/* size: 8, cachelines: 1, members: 5 */
	/* last cacheline: 8 bytes */
};
union cpuid10_eax {
	struct {
		unsigned int       version_id:8;       /*     0: 0  4 */
		unsigned int       num_counters:8;     /*     0: 8  4 */
		unsigned int       bit_width:8;        /*     0:16  4 */
		unsigned int       mask_length:8;      /*     0:24  4 */
	} split;                                       /*     0     4 */
	unsigned int               full;               /*     0     4 */
};
union cpuid10_ebx {
	struct {
		unsigned int       no_unhalted_core_cycles:1; /*     0: 0  4 */
		unsigned int       no_instructions_retired:1; /*     0: 1  4 */
		unsigned int       no_unhalted_reference_cycles:1; /*     0: 2  4 */
		unsigned int       no_llc_reference:1; /*     0: 3  4 */
		unsigned int       no_llc_misses:1;    /*     0: 4  4 */
		unsigned int       no_branch_instruction_retired:1; /*     0: 5  4 */
		unsigned int       no_branch_misses_retired:1; /*     0: 6  4 */
	} split;                                       /*     0     4 */
	unsigned int               full;               /*     0     4 */
};
union cpuid10_edx {
	struct {
		unsigned int       num_counters_fixed:5; /*     0: 0  4 */
		unsigned int       bit_width_fixed:8;  /*     0: 5  4 */
		unsigned int       reserved1:2;        /*     0:13  4 */
		unsigned int       anythread_deprecated:1; /*     0:15  4 */
		unsigned int       reserved2:16;       /*     0:16  4 */
	} split;                                       /*     0     4 */
	unsigned int               full;               /*     0     4 */
};
union x86_pmu_config {
	struct {
		u64                event:8;            /*     0: 0  8 */
		u64                umask:8;            /*     0: 8  8 */
		u64                usr:1;              /*     0:16  8 */
		u64                os:1;               /*     0:17  8 */
		u64                edge:1;             /*     0:18  8 */
		u64                pc:1;               /*     0:19  8 */
		u64                interrupt:1;        /*     0:20  8 */
		u64                __reserved1:1;      /*     0:21  8 */
		u64                en:1;               /*     0:22  8 */
		u64                inv:1;              /*     0:23  8 */
		u64                cmask:8;            /*     0:24  8 */
		u64                event2:4;           /*     0:32  8 */
		u64                __reserved2:4;      /*     0:36  8 */
		u64                go:1;               /*     0:40  8 */
		u64                ho:1;               /*     0:41  8 */
	} bits;                                        /*     0     8 */
	u64                        value;              /*     0     8 */
};
struct bts_ctx {
	struct perf_output_handle  handle;               /*     0    56 */

	/* XXX last struct has 4 bytes of padding */
	/* XXX 4040 bytes hole, try to pack */

	/* --- cacheline 64 boundary (4096 bytes) --- */
	struct debug_store         ds_back __attribute__((__aligned__(4096))); /*  4096  4096 */

	/* XXX last struct has 3936 bytes of padding */

	/* --- cacheline 128 boundary (8192 bytes) --- */
	int                        state;                /*  8192     4 */

	/* size: 12288, cachelines: 192, members: 3 */
	/* sum members: 4156, holes: 1, sum holes: 4040 */
	/* padding: 4092 */
	/* paddings: 2, sum paddings: 3940 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 4040 */
} __attribute__((__aligned__(4096)));
struct bts_phys {
	struct page *              page;                 /*     0     8 */
	long unsigned int          size;                 /*     8     8 */
	long unsigned int          offset;               /*    16     8 */
	long unsigned int          displacement;         /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct bts_buffer {
	size_t                     real_size;            /*     0     8 */
	unsigned int               nr_pages;             /*     8     4 */
	unsigned int               nr_bufs;              /*    12     4 */
	unsigned int               cur_buf;              /*    16     4 */
	bool                       snapshot;             /*    20     1 */

	/* XXX 3 bytes hole, try to pack */

	local_t                    data_size;            /*    24     8 */
	local_t                    head;                 /*    32     8 */
	long unsigned int          end;                  /*    40     8 */
	void * *                   data_pages;           /*    48     8 */
	struct bts_phys            buf[];                /*    56     0 */

	/* size: 56, cachelines: 1, members: 10 */
	/* sum members: 53, holes: 1, sum holes: 3 */
	/* last cacheline: 56 bytes */
};
struct lbr_entry {
	u64                        from;                 /*     0     8 */
	u64                        to;                   /*     8     8 */
	u64                        info;                 /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct pebs_basic {
	u64                        format_size;          /*     0     8 */
	u64                        ip;                   /*     8     8 */
	u64                        applicable_counters;  /*    16     8 */
	u64                        tsc;                  /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct pebs_meminfo {
	u64                        address;              /*     0     8 */
	u64                        aux;                  /*     8     8 */
	u64                        latency;              /*    16     8 */
	u64                        tsx_tuning;           /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct pebs_gprs {
	u64                        flags;                /*     0     8 */
	u64                        ip;                   /*     8     8 */
	u64                        ax;                   /*    16     8 */
	u64                        cx;                   /*    24     8 */
	u64                        dx;                   /*    32     8 */
	u64                        bx;                   /*    40     8 */
	u64                        sp;                   /*    48     8 */
	u64                        bp;                   /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u64                        si;                   /*    64     8 */
	u64                        di;                   /*    72     8 */
	u64                        r8;                   /*    80     8 */
	u64                        r9;                   /*    88     8 */
	u64                        r10;                  /*    96     8 */
	u64                        r11;                  /*   104     8 */
	u64                        r12;                  /*   112     8 */
	u64                        r13;                  /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	u64                        r14;                  /*   128     8 */
	u64                        r15;                  /*   136     8 */

	/* size: 144, cachelines: 3, members: 18 */
	/* last cacheline: 16 bytes */
};
struct pebs_xmm {
	u64                        xmm[32];              /*     0   256 */

	/* size: 256, cachelines: 4, members: 1 */
};
struct x86_perf_regs {
	struct pt_regs             regs;                 /*     0   168 */
	/* --- cacheline 2 boundary (128 bytes) was 40 bytes ago --- */
	u64 *                      xmm_regs;             /*   168     8 */

	/* size: 176, cachelines: 3, members: 2 */
	/* last cacheline: 48 bytes */
};
struct insn_field {
	union {
		insn_value_t       value;                /*     0     4 */
		insn_byte_t        bytes[4];             /*     0     4 */
	};                                               /*     0     4 */
	unsigned char              got;                  /*     4     1 */
	unsigned char              nbytes;               /*     5     1 */

	/* size: 8, cachelines: 1, members: 3 */
	/* padding: 2 */
	/* last cacheline: 8 bytes */
};
struct insn {
	struct insn_field          prefixes;             /*     0     8 */

	/* XXX last struct has 2 bytes of padding */

	struct insn_field          rex_prefix;           /*     8     8 */

	/* XXX last struct has 2 bytes of padding */

	struct insn_field          vex_prefix;           /*    16     8 */

	/* XXX last struct has 2 bytes of padding */

	struct insn_field          opcode;               /*    24     8 */

	/* XXX last struct has 2 bytes of padding */

	struct insn_field          modrm;                /*    32     8 */

	/* XXX last struct has 2 bytes of padding */

	struct insn_field          sib;                  /*    40     8 */

	/* XXX last struct has 2 bytes of padding */

	struct insn_field          displacement;         /*    48     8 */

	/* XXX last struct has 2 bytes of padding */

	union {
		struct insn_field  immediate;            /*    56     8 */
		struct insn_field  moffset1;             /*    56     8 */
		struct insn_field  immediate1;           /*    56     8 */
	};                                               /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	union {
		struct insn_field  moffset2;             /*    64     8 */
		struct insn_field  immediate2;           /*    64     8 */
	};                                               /*    64     8 */
	int                        emulate_prefix_size;  /*    72     4 */
	insn_attr_t                attr;                 /*    76     4 */
	unsigned char              opnd_bytes;           /*    80     1 */
	unsigned char              addr_bytes;           /*    81     1 */
	unsigned char              length;               /*    82     1 */
	unsigned char              x86_64;               /*    83     1 */

	/* XXX 4 bytes hole, try to pack */

	const insn_byte_t  *       kaddr;                /*    88     8 */
	const insn_byte_t  *       end_kaddr;            /*    96     8 */
	const insn_byte_t  *       next_byte;            /*   104     8 */

	/* size: 112, cachelines: 2, members: 18 */
	/* sum members: 108, holes: 1, sum holes: 4 */
	/* paddings: 7, sum paddings: 14 */
	/* last cacheline: 48 bytes */
};
struct perf_event_header {
	__u32                      type;                 /*     0     4 */
	__u16                      misc;                 /*     4     2 */
	__u16                      size;                 /*     6     2 */

	/* size: 8, cachelines: 1, members: 3 */
	/* last cacheline: 8 bytes */
};
union intel_x86_pebs_dse {
	u64                        val;                /*     0     8 */
	struct {
		unsigned int       ld_dse:4;           /*     0: 0  4 */
		unsigned int       ld_stlb_miss:1;     /*     0: 4  4 */
		unsigned int       ld_locked:1;        /*     0: 5  4 */
		unsigned int       ld_data_blk:1;      /*     0: 6  4 */
		unsigned int       ld_addr_blk:1;      /*     0: 7  4 */
		unsigned int       ld_reserved:24;     /*     0: 8  4 */
	};                                             /*     0     4 */
	struct {
		unsigned int       st_l1d_hit:1;       /*     0: 0  4 */
		unsigned int       st_reserved1:3;     /*     0: 1  4 */
		unsigned int       st_stlb_miss:1;     /*     0: 4  4 */
		unsigned int       st_locked:1;        /*     0: 5  4 */
		unsigned int       st_reserved2:26;    /*     0: 6  4 */
	};                                             /*     0     4 */
	struct {
		unsigned int       st_lat_dse:4;       /*     0: 0  4 */
		unsigned int       st_lat_stlb_miss:1; /*     0: 4  4 */
		unsigned int       st_lat_locked:1;    /*     0: 5  4 */
		unsigned int       ld_reserved3:26;    /*     0: 6  4 */
	};                                             /*     0     4 */
};
struct pebs_record_core {
	u64                        flags;                /*     0     8 */
	u64                        ip;                   /*     8     8 */
	u64                        ax;                   /*    16     8 */
	u64                        bx;                   /*    24     8 */
	u64                        cx;                   /*    32     8 */
	u64                        dx;                   /*    40     8 */
	u64                        si;                   /*    48     8 */
	u64                        di;                   /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u64                        bp;                   /*    64     8 */
	u64                        sp;                   /*    72     8 */
	u64                        r8;                   /*    80     8 */
	u64                        r9;                   /*    88     8 */
	u64                        r10;                  /*    96     8 */
	u64                        r11;                  /*   104     8 */
	u64                        r12;                  /*   112     8 */
	u64                        r13;                  /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	u64                        r14;                  /*   128     8 */
	u64                        r15;                  /*   136     8 */

	/* size: 144, cachelines: 3, members: 18 */
	/* last cacheline: 16 bytes */
};
struct pebs_record_nhm {
	u64                        flags;                /*     0     8 */
	u64                        ip;                   /*     8     8 */
	u64                        ax;                   /*    16     8 */
	u64                        bx;                   /*    24     8 */
	u64                        cx;                   /*    32     8 */
	u64                        dx;                   /*    40     8 */
	u64                        si;                   /*    48     8 */
	u64                        di;                   /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u64                        bp;                   /*    64     8 */
	u64                        sp;                   /*    72     8 */
	u64                        r8;                   /*    80     8 */
	u64                        r9;                   /*    88     8 */
	u64                        r10;                  /*    96     8 */
	u64                        r11;                  /*   104     8 */
	u64                        r12;                  /*   112     8 */
	u64                        r13;                  /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	u64                        r14;                  /*   128     8 */
	u64                        r15;                  /*   136     8 */
	u64                        status;               /*   144     8 */
	u64                        dla;                  /*   152     8 */
	u64                        dse;                  /*   160     8 */
	u64                        lat;                  /*   168     8 */

	/* size: 176, cachelines: 3, members: 22 */
	/* last cacheline: 48 bytes */
};
union hsw_tsx_tuning {
	struct {
		u32                cycles_last_block:32; /*     0: 0  4 */
		u32                hle_abort:1;        /*     4: 0  4 */
		u32                rtm_abort:1;        /*     4: 1  4 */
		u32                instruction_abort:1; /*     4: 2  4 */
		u32                non_instruction_abort:1; /*     4: 3  4 */
		u32                retry:1;            /*     4: 4  4 */
		u32                data_conflict:1;    /*     4: 5  4 */
		u32                capacity_writes:1;  /*     4: 6  4 */
		u32                capacity_reads:1;   /*     4: 7  4 */
	};                                             /*     0     8 */
	u64                        value;              /*     0     8 */
};
struct pebs_record_skl {
	u64                        flags;                /*     0     8 */
	u64                        ip;                   /*     8     8 */
	u64                        ax;                   /*    16     8 */
	u64                        bx;                   /*    24     8 */
	u64                        cx;                   /*    32     8 */
	u64                        dx;                   /*    40     8 */
	u64                        si;                   /*    48     8 */
	u64                        di;                   /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u64                        bp;                   /*    64     8 */
	u64                        sp;                   /*    72     8 */
	u64                        r8;                   /*    80     8 */
	u64                        r9;                   /*    88     8 */
	u64                        r10;                  /*    96     8 */
	u64                        r11;                  /*   104     8 */
	u64                        r12;                  /*   112     8 */
	u64                        r13;                  /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	u64                        r14;                  /*   128     8 */
	u64                        r15;                  /*   136     8 */
	u64                        status;               /*   144     8 */
	u64                        dla;                  /*   152     8 */
	u64                        dse;                  /*   160     8 */
	u64                        lat;                  /*   168     8 */
	u64                        real_ip;              /*   176     8 */
	u64                        tsx_tuning;           /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	u64                        tsc;                  /*   192     8 */

	/* size: 200, cachelines: 4, members: 25 */
	/* last cacheline: 8 bytes */
};
struct arch_lbr_state {
	u64                        lbr_ctl;              /*     0     8 */
	u64                        lbr_depth;            /*     8     8 */
	u64                        ler_from;             /*    16     8 */
	u64                        ler_to;               /*    24     8 */
	u64                        ler_info;             /*    32     8 */
	struct lbr_entry           entries[];            /*    40     0 */

	/* size: 40, cachelines: 1, members: 6 */
	/* last cacheline: 40 bytes */
};
union cpuid28_eax {
	struct {
		unsigned int       lbr_depth_mask:8;   /*     0: 0  4 */
		unsigned int       reserved:22;        /*     0: 8  4 */
		unsigned int       lbr_deep_c_reset:1; /*     0:30  4 */
		unsigned int       lbr_lip:1;          /*     0:31  4 */
	} split;                                       /*     0     4 */
	unsigned int               full;               /*     0     4 */
};
union cpuid28_ebx {
	struct {
		unsigned int       lbr_cpl:1;          /*     0: 0  4 */
		unsigned int       lbr_filter:1;       /*     0: 1  4 */
		unsigned int       lbr_call_stack:1;   /*     0: 2  4 */
	} split;                                       /*     0     4 */
	unsigned int               full;               /*     0     4 */
};
union cpuid28_ecx {
	struct {
		unsigned int       lbr_mispred:1;      /*     0: 0  4 */
		unsigned int       lbr_timed_lbr:1;    /*     0: 1  4 */
		unsigned int       lbr_br_type:1;      /*     0: 2  4 */
	} split;                                       /*     0     4 */
	unsigned int               full;               /*     0     4 */
};
struct x86_pmu_lbr {
	unsigned int               nr;                   /*     0     4 */
	unsigned int               from;                 /*     4     4 */
	unsigned int               to;                   /*     8     4 */
	unsigned int               info;                 /*    12     4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct x86_perf_task_context_opt {
	int                        lbr_callstack_users;  /*     0     4 */
	int                        lbr_stack_state;      /*     4     4 */
	int                        log_id;               /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct x86_perf_task_context {
	u64                        lbr_sel;              /*     0     8 */
	int                        tos;                  /*     8     4 */
	int                        valid_lbrs;           /*    12     4 */
	struct x86_perf_task_context_opt opt;            /*    16    12 */

	/* XXX 4 bytes hole, try to pack */

	struct lbr_entry           lbr[32];              /*    32   768 */

	/* size: 800, cachelines: 13, members: 5 */
	/* sum members: 796, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct x86_perf_task_context_arch_lbr {
	struct x86_perf_task_context_opt opt;            /*     0    12 */

	/* XXX 4 bytes hole, try to pack */

	struct lbr_entry           entries[];            /*    16     0 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct x86_perf_task_context_arch_lbr_xsave {
	struct x86_perf_task_context_opt opt;            /*     0    12 */

	/* XXX 52 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	union {
		struct xregs_state xsave __attribute__((__aligned__(64))); /*    64   576 */
		struct {
			struct fxregs_state i387 __attribute__((__aligned__(1))); /*    64   512 */
			/* --- cacheline 9 boundary (576 bytes) --- */
			struct xstate_header header;     /*   576    64 */
			/* --- cacheline 10 boundary (640 bytes) --- */
			struct arch_lbr_state lbr;       /*   640    40 */
		} __attribute__((__aligned__(64))) __attribute__((__aligned__(64)));      /*    64   640 */
	} __attribute__((__aligned__(64)));              /*    64   640 */

	/* size: 704, cachelines: 11, members: 2 */
	/* sum members: 652, holes: 1, sum holes: 52 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 52 */
} __attribute__((__aligned__(64)));
struct p4_event_bind {
	unsigned int               opcode;               /*     0     4 */
	unsigned int               escr_msr[2];          /*     4     8 */
	unsigned int               escr_emask;           /*    12     4 */
	unsigned int               shared;               /*    16     4 */
	char                       cntr[2][3];           /*    20     6 */

	/* size: 28, cachelines: 1, members: 5 */
	/* padding: 2 */
	/* last cacheline: 28 bytes */
};
struct p4_pebs_bind {
	unsigned int               metric_pebs;          /*     0     4 */
	unsigned int               metric_vert;          /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct p4_event_alias {
	u64                        original;             /*     0     8 */
	u64                        alternative;          /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct dev_ext_attribute {
	struct device_attribute    attr;                 /*     0    32 */
	void *                     var;                  /*    32     8 */

	/* size: 40, cachelines: 1, members: 2 */
	/* last cacheline: 40 bytes */
};
struct perf_addr_filter {
	struct list_head           entry;                /*     0    16 */
	struct path                path;                 /*    16    16 */
	long unsigned int          offset;               /*    32     8 */
	long unsigned int          size;                 /*    40     8 */
	enum perf_addr_filter_action_t action;           /*    48     4 */

	/* size: 56, cachelines: 1, members: 5 */
	/* padding: 4 */
	/* last cacheline: 56 bytes */
};
struct topa_entry {
	u64                        end:1;                /*     0: 0  8 */
	u64                        rsvd0:1;              /*     0: 1  8 */
	u64                        intr:1;               /*     0: 2  8 */
	u64                        rsvd1:1;              /*     0: 3  8 */
	u64                        stop:1;               /*     0: 4  8 */
	u64                        rsvd2:1;              /*     0: 5  8 */
	u64                        size:4;               /*     0: 6  8 */
	u64                        rsvd3:2;              /*     0:10  8 */
	u64                        base:36;              /*     0:12  8 */
	u64                        rsvd4:16;             /*     0:48  8 */

	/* size: 8, cachelines: 1, members: 10 */
	/* last cacheline: 8 bytes */
};
struct pt_pmu {
	struct pmu                 pmu;                  /*     0   344 */
	/* --- cacheline 5 boundary (320 bytes) was 24 bytes ago --- */
	u32                        caps[8];              /*   344    32 */
	bool                       vmx;                  /*   376     1 */
	bool                       branch_en_always_on;  /*   377     1 */

	/* XXX 6 bytes hole, try to pack */

	/* --- cacheline 6 boundary (384 bytes) --- */
	long unsigned int          max_nonturbo_ratio;   /*   384     8 */
	unsigned int               tsc_art_num;          /*   392     4 */
	unsigned int               tsc_art_den;          /*   396     4 */

	/* size: 400, cachelines: 7, members: 7 */
	/* sum members: 394, holes: 1, sum holes: 6 */
	/* last cacheline: 16 bytes */
};
struct pt_buffer {
	struct list_head           tables;               /*     0    16 */
	struct topa *              first;                /*    16     8 */
	struct topa *              last;                 /*    24     8 */
	struct topa *              cur;                  /*    32     8 */
	unsigned int               cur_idx;              /*    40     4 */

	/* XXX 4 bytes hole, try to pack */

	size_t                     output_off;           /*    48     8 */
	long unsigned int          nr_pages;             /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	local_t                    data_size;            /*    64     8 */
	local64_t                  head;                 /*    72     8 */
	bool                       snapshot;             /*    80     1 */
	bool                       single;               /*    81     1 */

	/* XXX 6 bytes hole, try to pack */

	long int                   stop_pos;             /*    88     8 */
	long int                   intr_pos;             /*    96     8 */
	struct topa_entry *        stop_te;              /*   104     8 */
	struct topa_entry *        intr_te;              /*   112     8 */
	void * *                   data_pages;           /*   120     8 */

	/* size: 128, cachelines: 2, members: 16 */
	/* sum members: 118, holes: 2, sum holes: 10 */
};
struct topa {
	struct list_head           list;                 /*     0    16 */
	u64                        offset;               /*    16     8 */
	size_t                     size;                 /*    24     8 */
	int                        last;                 /*    32     4 */
	unsigned int               z_count;              /*    36     4 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct pt_filter {
	long unsigned int          msr_a;                /*     0     8 */
	long unsigned int          msr_b;                /*     8     8 */
	long unsigned int          config;               /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct pt_filters {
	struct pt_filter           filter[4];            /*     0    96 */
	/* --- cacheline 1 boundary (64 bytes) was 32 bytes ago --- */
	unsigned int               nr_filters;           /*    96     4 */

	/* size: 104, cachelines: 2, members: 2 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct pt {
	struct perf_output_handle  handle;               /*     0    56 */

	/* XXX last struct has 4 bytes of padding */

	struct pt_filters          filters;              /*    56   104 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 2 boundary (128 bytes) was 32 bytes ago --- */
	int                        handle_nmi;           /*   160     4 */
	int                        vmx_on;               /*   164     4 */
	u64                        output_base;          /*   168     8 */
	u64                        output_mask;          /*   176     8 */

	/* size: 184, cachelines: 3, members: 6 */
	/* paddings: 2, sum paddings: 8 */
	/* last cacheline: 56 bytes */
};
struct pt_cap_desc {
	const char  *              name;                 /*     0     8 */
	u32                        leaf;                 /*     8     4 */
	u8                         reg;                  /*    12     1 */

	/* XXX 3 bytes hole, try to pack */

	u32                        mask;                 /*    16     4 */

	/* size: 24, cachelines: 1, members: 4 */
	/* sum members: 17, holes: 1, sum holes: 3 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct pt_address_range {
	long unsigned int          msr_a;                /*     0     8 */
	long unsigned int          msr_b;                /*     8     8 */
	unsigned int               reg_off;              /*    16     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct topa_page {
	struct topa_entry          table[507];           /*     0  4056 */
	/* --- cacheline 63 boundary (4032 bytes) was 24 bytes ago --- */
	struct topa                topa;                 /*  4056    40 */

	/* size: 4096, cachelines: 64, members: 2 */
};
struct x86_cpu_id {
	__u16                      vendor;               /*     0     2 */
	__u16                      family;               /*     2     2 */
	__u16                      model;                /*     4     2 */
	__u16                      steppings;            /*     6     2 */
	__u16                      feature;              /*     8     2 */

	/* XXX 6 bytes hole, try to pack */

	kernel_ulong_t             driver_data;          /*    16     8 */

	/* size: 24, cachelines: 1, members: 6 */
	/* sum members: 18, holes: 1, sum holes: 6 */
	/* last cacheline: 24 bytes */
};
struct zhaoxin_uncore_type {
	const char  *              name;                 /*     0     8 */
	int                        num_counters;         /*     8     4 */
	int                        num_boxes;            /*    12     4 */
	int                        perf_ctr_bits;        /*    16     4 */
	int                        fixed_ctr_bits;       /*    20     4 */
	unsigned int               perf_ctr;             /*    24     4 */
	unsigned int               event_ctl;            /*    28     4 */
	unsigned int               event_mask;           /*    32     4 */
	unsigned int               event_mask_ext;       /*    36     4 */
	unsigned int               fixed_ctr;            /*    40     4 */
	unsigned int               fixed_ctl;            /*    44     4 */
	unsigned int               box_ctl;              /*    48     4 */
	unsigned int               msr_offset;           /*    52     4 */
	unsigned int               num_shared_regs:8;    /*    56: 0  4 */
	unsigned int               single_fixed:1;       /*    56: 8  4 */
	unsigned int               pair_ctr_ctl:1;       /*    56: 9  4 */

	/* XXX 22 bits hole, try to pack */
	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	unsigned int *             msr_offsets;          /*    64     8 */
	struct event_constraint    unconstrainted;       /*    72    40 */
	struct event_constraint *  constraints;          /*   112     8 */
	struct zhaoxin_uncore_pmu * pmus;                /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct zhaoxin_uncore_ops * ops;                 /*   128     8 */
	struct uncore_event_desc * event_descs;          /*   136     8 */
	const struct attribute_group  * attr_groups[4];  /*   144    32 */
	struct pmu *               pmu;                  /*   176     8 */

	/* size: 184, cachelines: 3, members: 24 */
	/* sum members: 176, holes: 1, sum holes: 4 */
	/* sum bitfield members: 10 bits, bit holes: 1, sum bit holes: 22 bits */
	/* last cacheline: 56 bytes */
};
struct zhaoxin_uncore_pmu {
	struct pmu                 pmu;                  /*     0   344 */
	/* --- cacheline 5 boundary (320 bytes) was 24 bytes ago --- */
	char                       name[32];             /*   344    32 */
	int                        pmu_idx;              /*   376     4 */
	int                        func_id;              /*   380     4 */
	/* --- cacheline 6 boundary (384 bytes) --- */
	bool                       registered;           /*   384     1 */

	/* XXX 3 bytes hole, try to pack */

	atomic_t                   activeboxes;          /*   388     4 */
	struct zhaoxin_uncore_type * type;               /*   392     8 */
	struct zhaoxin_uncore_box * * boxes;             /*   400     8 */

	/* size: 408, cachelines: 7, members: 8 */
	/* sum members: 405, holes: 1, sum holes: 3 */
	/* last cacheline: 24 bytes */
};
struct zhaoxin_uncore_ops {
	void                       (*init_box)(struct zhaoxin_uncore_box *); /*     0     8 */
	void                       (*exit_box)(struct zhaoxin_uncore_box *); /*     8     8 */
	void                       (*disable_box)(struct zhaoxin_uncore_box *); /*    16     8 */
	void                       (*enable_box)(struct zhaoxin_uncore_box *); /*    24     8 */
	void                       (*disable_event)(struct zhaoxin_uncore_box *, struct perf_event *); /*    32     8 */
	void                       (*enable_event)(struct zhaoxin_uncore_box *, struct perf_event *); /*    40     8 */
	u64                        (*read_counter)(struct zhaoxin_uncore_box *, struct perf_event *); /*    48     8 */
	int                        (*hw_config)(struct zhaoxin_uncore_box *, struct perf_event *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct event_constraint *  (*get_constraint)(struct zhaoxin_uncore_box *, struct perf_event *); /*    64     8 */
	void                       (*put_constraint)(struct zhaoxin_uncore_box *, struct perf_event *); /*    72     8 */

	/* size: 80, cachelines: 2, members: 10 */
	/* last cacheline: 16 bytes */
};
struct uncore_event_desc {
	struct device_attribute    attr;                 /*     0    32 */
	const char  *              config;               /*    32     8 */

	/* size: 40, cachelines: 1, members: 2 */
	/* last cacheline: 40 bytes */
};
struct zhaoxin_uncore_box {
	int                        pci_phys_id;          /*     0     4 */
	int                        package_id;           /*     4     4 */
	int                        cluster_id;           /*     8     4 */
	int                        subnode_id;           /*    12     4 */
	int                        n_active;             /*    16     4 */
	int                        n_events;             /*    20     4 */
	int                        cpu;                  /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          flags;                /*    32     8 */
	atomic_t                   refcnt;               /*    40     4 */

	/* XXX 4 bytes hole, try to pack */

	struct perf_event *        events[5];            /*    48    40 */
	/* --- cacheline 1 boundary (64 bytes) was 24 bytes ago --- */
	struct perf_event *        event_list[5];        /*    88    40 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct event_constraint *  event_constraint[5];  /*   128    40 */
	long unsigned int          active_mask[1];       /*   168     8 */
	u64                        tags[5];              /*   176    40 */
	/* --- cacheline 3 boundary (192 bytes) was 24 bytes ago --- */
	struct pci_dev *           pci_dev;              /*   216     8 */
	struct zhaoxin_uncore_pmu * pmu;                 /*   224     8 */
	u64                        hrtimer_duration;     /*   232     8 */
	struct hrtimer             hrtimer __attribute__((__aligned__(8))); /*   240    96 */
	/* --- cacheline 5 boundary (320 bytes) was 16 bytes ago --- */
	struct list_head           list;                 /*   336    16 */
	struct list_head           active_list;          /*   352    16 */
	void *                     io_addr;              /*   368     8 */
	struct zhaoxin_uncore_extra_reg shared_regs[];   /*   376     0 */

	/* size: 376, cachelines: 6, members: 22 */
	/* sum members: 368, holes: 2, sum holes: 8 */
	/* forced alignments: 1 */
	/* last cacheline: 56 bytes */
} __attribute__((__aligned__(8)));
struct zhaoxin_uncore_extra_reg {
	raw_spinlock_t             lock;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        config;               /*     8     8 */
	u64                        config1;              /*    16     8 */
	u64                        config2;              /*    24     8 */
	atomic_t                   ref;                  /*    32     4 */

	/* size: 40, cachelines: 1, members: 5 */
	/* sum members: 32, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct hw_info {
	u64                        config_info;          /*     0     8 */
	u64                        active_state;         /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct zhaoxin_uncore_init_fun {
	void                       (*cpu_init)(void);    /*     0     8 */
	int                        (*pci_init)(void);    /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct atomic_notifier_head {
	spinlock_t                 lock;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct notifier_block *    head;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct arch_shared_info {
	long unsigned int          max_pfn;              /*     0     8 */
	xen_pfn_t                  pfn_to_mfn_frame_list_list; /*     8     8 */
	long unsigned int          nmi_reason;           /*    16     8 */
	long unsigned int          p2m_cr3;              /*    24     8 */
	long unsigned int          p2m_vaddr;            /*    32     8 */
	long unsigned int          p2m_generation;       /*    40     8 */

	/* size: 48, cachelines: 1, members: 6 */
	/* last cacheline: 48 bytes */
};
struct arch_vcpu_info {
	long unsigned int          cr2;                  /*     0     8 */
	long unsigned int          pad;                  /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct pvclock_wall_clock {
	u32                        version;              /*     0     4 */
	u32                        sec;                  /*     4     4 */
	u32                        nsec;                 /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct vcpu_info {
	uint8_t                    evtchn_upcall_pending; /*     0     1 */
	uint8_t                    evtchn_upcall_mask;   /*     1     1 */

	/* XXX 6 bytes hole, try to pack */

	xen_ulong_t                evtchn_pending_sel;   /*     8     8 */
	struct arch_vcpu_info      arch;                 /*    16    16 */
	struct pvclock_vcpu_time_info time;              /*    32    32 */

	/* size: 64, cachelines: 1, members: 5 */
	/* sum members: 58, holes: 1, sum holes: 6 */
};
struct shared_info {
	struct vcpu_info           vcpu_info[32];        /*     0  2048 */
	/* --- cacheline 32 boundary (2048 bytes) --- */
	xen_ulong_t                evtchn_pending[64];   /*  2048   512 */
	/* --- cacheline 40 boundary (2560 bytes) --- */
	xen_ulong_t                evtchn_mask[64];      /*  2560   512 */
	/* --- cacheline 48 boundary (3072 bytes) --- */
	struct pvclock_wall_clock  wc;                   /*  3072    12 */

	/* XXX 4 bytes hole, try to pack */

	struct arch_shared_info    arch;                 /*  3088    48 */

	/* size: 3136, cachelines: 49, members: 5 */
	/* sum members: 3132, holes: 1, sum holes: 4 */
};
struct start_info {
	char                       magic[32];            /*     0    32 */
	long unsigned int          nr_pages;             /*    32     8 */
	long unsigned int          shared_info;          /*    40     8 */
	uint32_t                   flags;                /*    48     4 */

	/* XXX 4 bytes hole, try to pack */

	xen_pfn_t                  store_mfn;            /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	uint32_t                   store_evtchn;         /*    64     4 */

	/* XXX 4 bytes hole, try to pack */

	union {
		struct {
			xen_pfn_t  mfn;                  /*    72     8 */
			uint32_t   evtchn;               /*    80     4 */
		} domU;                                  /*    72    16 */
		struct {
			uint32_t   info_off;             /*    72     4 */
			uint32_t   info_size;            /*    76     4 */
		} dom0;                                  /*    72     8 */
	} console;                                       /*    72    16 */
	long unsigned int          pt_base;              /*    88     8 */
	long unsigned int          nr_pt_frames;         /*    96     8 */
	long unsigned int          mfn_list;             /*   104     8 */
	long unsigned int          mod_start;            /*   112     8 */
	long unsigned int          mod_len;              /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	int8_t                     cmd_line[1024];       /*   128  1024 */
	/* --- cacheline 18 boundary (1152 bytes) --- */
	long unsigned int          first_p2m_pfn;        /*  1152     8 */
	long unsigned int          nr_p2m_frames;        /*  1160     8 */

	/* size: 1168, cachelines: 19, members: 15 */
	/* sum members: 1160, holes: 2, sum holes: 8 */
	/* last cacheline: 16 bytes */
};
struct sched_shutdown {
	unsigned int               reason;               /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct sched_pin_override {
	int32_t                    pcpu;                 /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct vcpu_register_vcpu_info {
	uint64_t                   mfn;                  /*     0     8 */
	uint32_t                   offset;               /*     8     4 */
	uint32_t                   rsvd;                 /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct xmaddr {
	phys_addr_t                maddr;                /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct xpaddr {
	phys_addr_t                paddr;                /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct clocksource {
	u64                        (*read)(struct clocksource *); /*     0     8 */
	u64                        mask;                 /*     8     8 */
	u32                        mult;                 /*    16     4 */
	u32                        shift;                /*    20     4 */
	u64                        max_idle_ns;          /*    24     8 */
	u32                        maxadj;               /*    32     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        max_cycles;           /*    40     8 */
	const char  *              name;                 /*    48     8 */
	struct list_head           list;                 /*    56    16 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	int                        rating;               /*    72     4 */
	enum vdso_clock_mode       vdso_clock_mode;      /*    76     4 */
	u16                        vdso_fix;             /*    80     2 */
	u16                        vdso_shift;           /*    82     2 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          flags;                /*    88     8 */
	int                        (*enable)(struct clocksource *); /*    96     8 */
	void                       (*disable)(struct clocksource *); /*   104     8 */
	void                       (*suspend)(struct clocksource *); /*   112     8 */
	void                       (*resume)(struct clocksource *); /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	void                       (*mark_unstable)(struct clocksource *); /*   128     8 */
	void                       (*tick_stable)(struct clocksource *); /*   136     8 */
	struct list_head           wd_list;              /*   144    16 */
	u64                        cs_last;              /*   160     8 */
	u64                        wd_last;              /*   168     8 */
	struct module *            owner;                /*   176     8 */

	/* size: 184, cachelines: 3, members: 24 */
	/* sum members: 176, holes: 2, sum holes: 8 */
	/* last cacheline: 56 bytes */
};
struct irqaction {
	irq_handler_t              handler;              /*     0     8 */
	void *                     dev_id;               /*     8     8 */
	void *                     percpu_dev_id;        /*    16     8 */
	struct irqaction *         next;                 /*    24     8 */
	irq_handler_t              thread_fn;            /*    32     8 */
	struct task_struct *       thread;               /*    40     8 */
	struct irqaction *         secondary;            /*    48     8 */
	unsigned int               irq;                  /*    56     4 */
	unsigned int               flags;                /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	long unsigned int          thread_flags;         /*    64     8 */
	long unsigned int          thread_mask;          /*    72     8 */
	const char  *              name;                 /*    80     8 */
	struct proc_dir_entry *    dir;                  /*    88     8 */

	/* size: 128, cachelines: 2, members: 13 */
	/* padding: 32 */
} __attribute__((__aligned__(64)));
struct irq_affinity_notify {
	unsigned int               irq;                  /*     0     4 */
	struct kref                kref;                 /*     4     4 */
	struct work_struct         work;                 /*     8    64 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	void                       (*notify)(struct irq_affinity_notify *, const cpumask_t  *); /*    72     8 */
	void                       (*release)(struct kref *); /*    80     8 */

	/* size: 88, cachelines: 2, members: 5 */
	/* last cacheline: 24 bytes */
};
struct irq_affinity_desc {
	struct cpumask             mask;                 /*     0  1024 */
	/* --- cacheline 16 boundary (1024 bytes) --- */
	unsigned int               is_managed:1;         /*  1024: 0  4 */

	/* size: 1032, cachelines: 17, members: 2 */
	/* padding: 4 */
	/* bit_padding: 31 bits */
	/* last cacheline: 8 bytes */
};
struct x86_init_mpparse {
	void                       (*setup_ioapic_ids)(void); /*     0     8 */
	void                       (*find_smp_config)(void); /*     8     8 */
	void                       (*get_smp_config)(unsigned int); /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct x86_init_resources {
	void                       (*probe_roms)(void);  /*     0     8 */
	void                       (*reserve_resources)(void); /*     8     8 */
	char *                     (*memory_setup)(void); /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct x86_init_irqs {
	void                       (*pre_vector_init)(void); /*     0     8 */
	void                       (*intr_init)(void);   /*     8     8 */
	void                       (*intr_mode_select)(void); /*    16     8 */
	void                       (*intr_mode_init)(void); /*    24     8 */
	struct irq_domain *        (*create_pci_msi_domain)(void); /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct x86_init_oem {
	void                       (*arch_setup)(void);  /*     0     8 */
	void                       (*banner)(void);      /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct x86_init_paging {
	void                       (*pagetable_init)(void); /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct x86_init_timers {
	void                       (*setup_percpu_clockev)(void); /*     0     8 */
	void                       (*timer_init)(void);  /*     8     8 */
	void                       (*wallclock_init)(void); /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct x86_init_iommu {
	int                        (*iommu_init)(void);  /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct x86_init_pci {
	int                        (*arch_init)(void);   /*     0     8 */
	int                        (*init)(void);        /*     8     8 */
	void                       (*init_irq)(void);    /*    16     8 */
	void                       (*fixup_irqs)(void);  /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct x86_hyper_init {
	void                       (*init_platform)(void); /*     0     8 */
	void                       (*guest_late_init)(void); /*     8     8 */
	bool                       (*x2apic_available)(void); /*    16     8 */
	void                       (*init_mem_mapping)(void); /*    24     8 */
	void                       (*init_after_bootmem)(void); /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct x86_init_acpi {
	void                       (*set_root_pointer)(u64); /*     0     8 */
	u64                        (*get_root_pointer)(void); /*     8     8 */
	void                       (*reduced_hw_early_init)(void); /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct x86_init_ops {
	struct x86_init_resources  resources;            /*     0    24 */
	struct x86_init_mpparse    mpparse;              /*    24    24 */
	struct x86_init_irqs       irqs;                 /*    48    40 */
	/* --- cacheline 1 boundary (64 bytes) was 24 bytes ago --- */
	struct x86_init_oem        oem;                  /*    88    16 */
	struct x86_init_paging     paging;               /*   104     8 */
	struct x86_init_timers     timers;               /*   112    24 */
	/* --- cacheline 2 boundary (128 bytes) was 8 bytes ago --- */
	struct x86_init_iommu      iommu;                /*   136     8 */
	struct x86_init_pci        pci;                  /*   144    32 */
	struct x86_hyper_init      hyper;                /*   176    40 */
	/* --- cacheline 3 boundary (192 bytes) was 24 bytes ago --- */
	struct x86_init_acpi       acpi;                 /*   216    24 */

	/* size: 240, cachelines: 4, members: 10 */
	/* last cacheline: 48 bytes */
};
struct x86_cpuinit_ops {
	void                       (*setup_percpu_clockev)(void); /*     0     8 */
	void                       (*early_percpu_clock_init)(void); /*     8     8 */
	void                       (*fixup_cpu_id)(struct cpuinfo_x86 *, int); /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct clock_event_device {
	void                       (*event_handler)(struct clock_event_device *); /*     0     8 */
	int                        (*set_next_event)(long unsigned int, struct clock_event_device *); /*     8     8 */
	int                        (*set_next_ktime)(ktime_t, struct clock_event_device *); /*    16     8 */
	ktime_t                    next_event;           /*    24     8 */
	u64                        max_delta_ns;         /*    32     8 */
	u64                        min_delta_ns;         /*    40     8 */
	u32                        mult;                 /*    48     4 */
	u32                        shift;                /*    52     4 */
	enum clock_event_state     state_use_accessors;  /*    56     4 */
	unsigned int               features;             /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	long unsigned int          retries;              /*    64     8 */
	int                        (*set_state_periodic)(struct clock_event_device *); /*    72     8 */
	int                        (*set_state_oneshot)(struct clock_event_device *); /*    80     8 */
	int                        (*set_state_oneshot_stopped)(struct clock_event_device *); /*    88     8 */
	int                        (*set_state_shutdown)(struct clock_event_device *); /*    96     8 */
	int                        (*tick_resume)(struct clock_event_device *); /*   104     8 */
	void                       (*broadcast)(const struct cpumask  *); /*   112     8 */
	void                       (*suspend)(struct clock_event_device *); /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	void                       (*resume)(struct clock_event_device *); /*   128     8 */
	long unsigned int          min_delta_ticks;      /*   136     8 */
	long unsigned int          max_delta_ticks;      /*   144     8 */
	const char  *              name;                 /*   152     8 */
	int                        rating;               /*   160     4 */
	int                        irq;                  /*   164     4 */
	int                        bound_on;             /*   168     4 */

	/* XXX 4 bytes hole, try to pack */

	const struct cpumask  *    cpumask;              /*   176     8 */
	struct list_head           list;                 /*   184    16 */
	/* --- cacheline 3 boundary (192 bytes) was 8 bytes ago --- */
	struct module *            owner;                /*   200     8 */

	/* size: 256, cachelines: 4, members: 28 */
	/* sum members: 204, holes: 1, sum holes: 4 */
	/* padding: 48 */
} __attribute__((__aligned__(64)));
struct vcpu_time_info {
	uint32_t                   version;              /*     0     4 */
	uint32_t                   pad0;                 /*     4     4 */
	uint64_t                   tsc_timestamp;        /*     8     8 */
	uint64_t                   system_time;          /*    16     8 */
	uint32_t                   tsc_to_system_mul;    /*    24     4 */
	int8_t                     tsc_shift;            /*    28     1 */
	int8_t                     pad1[3];              /*    29     3 */

	/* size: 32, cachelines: 1, members: 7 */
	/* last cacheline: 32 bytes */
};
struct irq_desc {
	struct irq_common_data     irq_common_data;      /*     0    56 */
	struct irq_data            irq_data;             /*    56    56 */
	/* --- cacheline 1 boundary (64 bytes) was 48 bytes ago --- */
	unsigned int *             kstat_irqs;           /*   112     8 */
	irq_flow_handler_t         handle_irq;           /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct irqaction *         action;               /*   128     8 */
	unsigned int               status_use_accessors; /*   136     4 */
	unsigned int               core_internal_state__do_not_mess_with_it; /*   140     4 */
	unsigned int               depth;                /*   144     4 */
	unsigned int               wake_depth;           /*   148     4 */
	unsigned int               tot_count;            /*   152     4 */
	unsigned int               irq_count;            /*   156     4 */
	long unsigned int          last_unhandled;       /*   160     8 */
	unsigned int               irqs_unhandled;       /*   168     4 */
	atomic_t                   threads_handled;      /*   172     4 */
	int                        threads_handled_last; /*   176     4 */
	raw_spinlock_t             lock;                 /*   180     4 */
	struct cpumask *           percpu_enabled;       /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	const struct cpumask  *    percpu_affinity;      /*   192     8 */
	const struct cpumask  *    affinity_hint;        /*   200     8 */
	struct irq_affinity_notify * affinity_notify;    /*   208     8 */
	cpumask_var_t              pending_mask;         /*   216     8 */
	long unsigned int          threads_oneshot;      /*   224     8 */
	atomic_t                   threads_active;       /*   232     4 */

	/* XXX 4 bytes hole, try to pack */

	wait_queue_head_t          wait_for_threads;     /*   240    24 */
	/* --- cacheline 4 boundary (256 bytes) was 8 bytes ago --- */
	unsigned int               nr_actions;           /*   264     4 */
	unsigned int               no_suspend_depth;     /*   268     4 */
	unsigned int               cond_suspend_depth;   /*   272     4 */
	unsigned int               force_resume_depth;   /*   276     4 */
	struct proc_dir_entry *    dir;                  /*   280     8 */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*   288    16 */
	struct kobject             kobj;                 /*   304    96 */
	/* --- cacheline 6 boundary (384 bytes) was 16 bytes ago --- */
	struct mutex               request_mutex;        /*   400    32 */
	int                        parent_irq;           /*   432     4 */

	/* XXX 4 bytes hole, try to pack */

	struct module *            owner;                /*   440     8 */
	/* --- cacheline 7 boundary (448 bytes) --- */
	const char  *              name;                 /*   448     8 */
	u64                        kabi_reserved1;       /*   456     8 */

	/* size: 512, cachelines: 8, members: 36 */
	/* sum members: 456, holes: 2, sum holes: 8 */
	/* padding: 48 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(64)));
struct irq_common_data {
	unsigned int               state_use_accessors;  /*     0     4 */
	unsigned int               node;                 /*     4     4 */
	void *                     handler_data;         /*     8     8 */
	struct msi_desc *          msi_desc;             /*    16     8 */
	cpumask_var_t              affinity;             /*    24     8 */
	cpumask_var_t              effective_affinity;   /*    32     8 */
	u64                        kabi_reserved1;       /*    40     8 */
	u64                        kabi_reserved2;       /*    48     8 */

	/* size: 56, cachelines: 1, members: 8 */
	/* last cacheline: 56 bytes */
};
struct msi_desc {
	struct list_head           list;                 /*     0    16 */
	unsigned int               irq;                  /*    16     4 */
	unsigned int               nvec_used;            /*    20     4 */
	struct device *            dev;                  /*    24     8 */
	struct msi_msg             msg;                  /*    32    12 */

	/* XXX 4 bytes hole, try to pack */

	struct irq_affinity_desc * affinity;             /*    48     8 */
	const void  *              iommu_cookie;         /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	void                       (*write_msi_msg)(struct msi_desc *, void *); /*    64     8 */
	void *                     write_msi_msg_data;   /*    72     8 */
	union {
		struct {
			u32        masked;               /*    80     4 */
			struct {
				u8 is_msix:1;            /*    84: 0  1 */
				u8 multiple:3;           /*    84: 1  1 */
				u8 multi_cap:3;          /*    84: 4  1 */
				u8 maskbit:1;            /*    84: 7  1 */
				u8 is_64:1;              /*    85: 0  1 */
				u8 is_virtual:1;         /*    85: 1  1 */

				/* XXX 6 bits hole, try to pack */

				u16 entry_nr;            /*    86     2 */
				unsigned int default_irq; /*    88     4 */
			} msi_attrib;                    /*    84     8 */

			/* XXX 4 bytes hole, try to pack */

			union {
				u8 mask_pos;             /*    96     1 */
				void * mask_base;        /*    96     8 */
			};                               /*    96     8 */
		};                                       /*    80    24 */
		struct platform_msi_desc platform;       /*    80    16 */
		struct fsl_mc_msi_desc fsl_mc;           /*    80     2 */
		struct ti_sci_inta_msi_desc inta;        /*    80     2 */
	};                                               /*    80    24 */
	u64                        kabi_reserved1;       /*   104     8 */
	u64                        kabi_reserved2;       /*   112     8 */

	/* size: 120, cachelines: 2, members: 12 */
	/* sum members: 116, holes: 1, sum holes: 4 */
	/* last cacheline: 56 bytes */
};
struct irq_data {
	u32                        mask;                 /*     0     4 */
	unsigned int               irq;                  /*     4     4 */
	long unsigned int          hwirq;                /*     8     8 */
	struct irq_common_data *   common;               /*    16     8 */
	struct irq_chip *          chip;                 /*    24     8 */
	struct irq_domain *        domain;               /*    32     8 */
	struct irq_data *          parent_data;          /*    40     8 */
	void *                     chip_data;            /*    48     8 */

	/* size: 56, cachelines: 1, members: 8 */
	/* last cacheline: 56 bytes */
};
struct irq_chip {
	struct device *            parent_device;        /*     0     8 */
	const char  *              name;                 /*     8     8 */
	unsigned int               (*irq_startup)(struct irq_data *); /*    16     8 */
	void                       (*irq_shutdown)(struct irq_data *); /*    24     8 */
	void                       (*irq_enable)(struct irq_data *); /*    32     8 */
	void                       (*irq_disable)(struct irq_data *); /*    40     8 */
	void                       (*irq_ack)(struct irq_data *); /*    48     8 */
	void                       (*irq_mask)(struct irq_data *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	void                       (*irq_mask_ack)(struct irq_data *); /*    64     8 */
	void                       (*irq_unmask)(struct irq_data *); /*    72     8 */
	void                       (*irq_eoi)(struct irq_data *); /*    80     8 */
	int                        (*irq_set_affinity)(struct irq_data *, const struct cpumask  *, bool); /*    88     8 */
	int                        (*irq_retrigger)(struct irq_data *); /*    96     8 */
	int                        (*irq_set_type)(struct irq_data *, unsigned int); /*   104     8 */
	int                        (*irq_set_wake)(struct irq_data *, unsigned int); /*   112     8 */
	void                       (*irq_bus_lock)(struct irq_data *); /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	void                       (*irq_bus_sync_unlock)(struct irq_data *); /*   128     8 */
	void                       (*irq_cpu_online)(struct irq_data *); /*   136     8 */
	void                       (*irq_cpu_offline)(struct irq_data *); /*   144     8 */
	void                       (*irq_suspend)(struct irq_data *); /*   152     8 */
	void                       (*irq_resume)(struct irq_data *); /*   160     8 */
	void                       (*irq_pm_shutdown)(struct irq_data *); /*   168     8 */
	void                       (*irq_calc_mask)(struct irq_data *); /*   176     8 */
	void                       (*irq_print_chip)(struct irq_data *, struct seq_file *); /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	int                        (*irq_request_resources)(struct irq_data *); /*   192     8 */
	void                       (*irq_release_resources)(struct irq_data *); /*   200     8 */
	void                       (*irq_compose_msi_msg)(struct irq_data *, struct msi_msg *); /*   208     8 */
	void                       (*irq_write_msi_msg)(struct irq_data *, struct msi_msg *); /*   216     8 */
	int                        (*irq_get_irqchip_state)(struct irq_data *, enum irqchip_irq_state, bool *); /*   224     8 */
	int                        (*irq_set_irqchip_state)(struct irq_data *, enum irqchip_irq_state, bool); /*   232     8 */
	int                        (*irq_set_vcpu_affinity)(struct irq_data *, void *); /*   240     8 */
	void                       (*ipi_send_single)(struct irq_data *, unsigned int); /*   248     8 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	void                       (*ipi_send_mask)(struct irq_data *, const struct cpumask  *); /*   256     8 */
	int                        (*irq_nmi_setup)(struct irq_data *); /*   264     8 */
	void                       (*irq_nmi_teardown)(struct irq_data *); /*   272     8 */
	long unsigned int          flags;                /*   280     8 */
	u64                        kabi_reserved1;       /*   288     8 */

	/* size: 296, cachelines: 5, members: 37 */
	/* last cacheline: 40 bytes */
};
struct msi_msg {
	u32                        address_lo;           /*     0     4 */
	u32                        address_hi;           /*     4     4 */
	u32                        data;                 /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct irq_chip_regs {
	long unsigned int          enable;               /*     0     8 */
	long unsigned int          disable;              /*     8     8 */
	long unsigned int          mask;                 /*    16     8 */
	long unsigned int          ack;                  /*    24     8 */
	long unsigned int          eoi;                  /*    32     8 */
	long unsigned int          type;                 /*    40     8 */
	long unsigned int          polarity;             /*    48     8 */

	/* size: 56, cachelines: 1, members: 7 */
	/* last cacheline: 56 bytes */
};
struct irq_chip_type {
	struct irq_chip            chip;                 /*     0   296 */
	/* --- cacheline 4 boundary (256 bytes) was 40 bytes ago --- */
	struct irq_chip_regs       regs;                 /*   296    56 */
	/* --- cacheline 5 boundary (320 bytes) was 32 bytes ago --- */
	irq_flow_handler_t         handler;              /*   352     8 */
	u32                        type;                 /*   360     4 */
	u32                        mask_cache_priv;      /*   364     4 */
	u32 *                      mask_cache;           /*   368     8 */
	u64                        kabi_reserved1;       /*   376     8 */

	/* size: 384, cachelines: 6, members: 7 */
};
struct irq_chip_generic {
	raw_spinlock_t             lock;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     reg_base;             /*     8     8 */
	u32                        (*reg_readl)(void *); /*    16     8 */
	void                       (*reg_writel)(u32, void *); /*    24     8 */
	void                       (*suspend)(struct irq_chip_generic *); /*    32     8 */
	void                       (*resume)(struct irq_chip_generic *); /*    40     8 */
	unsigned int               irq_base;             /*    48     4 */
	unsigned int               irq_cnt;              /*    52     4 */
	u32                        mask_cache;           /*    56     4 */
	u32                        type_cache;           /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u32                        polarity_cache;       /*    64     4 */
	u32                        wake_enabled;         /*    68     4 */
	u32                        wake_active;          /*    72     4 */
	unsigned int               num_ct;               /*    76     4 */
	void *                     private;              /*    80     8 */
	long unsigned int          installed;            /*    88     8 */
	long unsigned int          unused;               /*    96     8 */
	struct irq_domain *        domain;               /*   104     8 */
	struct list_head           list;                 /*   112    16 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct irq_chip_type       chip_types[];         /*   128     0 */

	/* size: 128, cachelines: 2, members: 20 */
	/* sum members: 124, holes: 1, sum holes: 4 */
};
struct irq_domain_chip_generic {
	unsigned int               irqs_per_chip;        /*     0     4 */
	unsigned int               num_chips;            /*     4     4 */
	unsigned int               irq_flags_to_clear;   /*     8     4 */
	unsigned int               irq_flags_to_set;     /*    12     4 */
	enum irq_gc_flags          gc_flags;             /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	struct irq_chip_generic *  gc[];                 /*    24     0 */

	/* size: 24, cachelines: 1, members: 6 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct platform_msi_desc {
	struct platform_msi_priv_data * msi_priv_data;   /*     0     8 */
	u16                        msi_index;            /*     8     2 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 6 */
	/* last cacheline: 16 bytes */
};
struct fsl_mc_msi_desc {
	u16                        msi_index;            /*     0     2 */

	/* size: 2, cachelines: 1, members: 1 */
	/* last cacheline: 2 bytes */
};
struct ti_sci_inta_msi_desc {
	u16                        dev_index;            /*     0     2 */

	/* size: 2, cachelines: 1, members: 1 */
	/* last cacheline: 2 bytes */
};
struct vcpu_set_singleshot_timer {
	uint64_t                   timeout_abs_ns;       /*     0     8 */
	uint32_t                   flags;                /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct vcpu_register_time_memory_area {
	union {
		__guest_handle_vcpu_time_info h;         /*     0     8 */
		struct pvclock_vcpu_time_info * v;       /*     0     8 */
		uint64_t           p;                    /*     0     8 */
	} addr;                                          /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct xen_clock_event_device {
	struct clock_event_device  evt __attribute__((__aligned__(64))); /*     0   256 */

	/* XXX last struct has 48 bytes of padding */

	/* --- cacheline 4 boundary (256 bytes) --- */
	char                       name[16];             /*   256    16 */

	/* size: 320, cachelines: 5, members: 2 */
	/* padding: 48 */
	/* paddings: 1, sum paddings: 48 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(64)));
struct gnttab_vm_area {
	struct vm_struct *         area;                 /*     0     8 */
	pte_t * *                  ptes;                 /*     8     8 */
	int                        idx;                  /*    16     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct pv_info {
	const char  *              name;                 /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct irq_stack {
	char                       stack[16384];         /*     0 16384 */

	/* size: 16384, cachelines: 256, members: 1 */
} __attribute__((__aligned__(16384)));
struct xen_add_to_physmap {
	domid_t                    domid;                /*     0     2 */
	uint16_t                   size;                 /*     2     2 */
	unsigned int               space;                /*     4     4 */
	xen_ulong_t                idx;                  /*     8     8 */
	xen_pfn_t                  gpfn;                 /*    16     8 */

	/* size: 24, cachelines: 1, members: 5 */
	/* last cacheline: 24 bytes */
};
struct machine_ops {
	void                       (*restart)(char *);   /*     0     8 */
	void                       (*halt)(void);        /*     8     8 */
	void                       (*power_off)(void);   /*    16     8 */
	void                       (*shutdown)(void);    /*    24     8 */
	void                       (*crash_shutdown)(struct pt_regs *); /*    32     8 */
	void                       (*emergency_restart)(void); /*    40     8 */

	/* size: 48, cachelines: 1, members: 6 */
	/* last cacheline: 48 bytes */
};
struct irqentry_state {
	union {
		bool               exit_rcu;             /*     0     1 */
		bool               lockdep;              /*     0     1 */
	};                                               /*     0     1 */

	/* size: 1, cachelines: 1, members: 1 */
	/* last cacheline: 1 bytes */
};
struct hypervisor_x86 {
	const char  *              name;                 /*     0     8 */
	uint32_t                   (*detect)(void);      /*     8     8 */
	enum x86_hypervisor_type   type;                 /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	struct x86_hyper_init      init;                 /*    24    40 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct x86_hyper_runtime   runtime;              /*    64    24 */
	bool                       ignore_nopv;          /*    88     1 */

	/* size: 96, cachelines: 2, members: 6 */
	/* sum members: 85, holes: 1, sum holes: 4 */
	/* padding: 7 */
	/* last cacheline: 32 bytes */
};
struct xen_hvm_pagetable_dying {
	domid_t                    domid;                /*     0     2 */

	/* XXX 6 bytes hole, try to pack */

	__u64                      gpa __attribute__((__aligned__(8))); /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 10, holes: 1, sum holes: 6 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 6 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct xen_hvm_get_mem_type {
	domid_t                    domid;                /*     0     2 */
	uint16_t                   mem_type;             /*     2     2 */
	uint16_t                   pad[2];               /*     4     4 */
	uint64_t                   pfn;                  /*     8     8 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct multicall_entry {
	xen_ulong_t                op;                   /*     0     8 */
	xen_long_t                 result;               /*     8     8 */
	xen_ulong_t                args[6];              /*    16    48 */

	/* size: 64, cachelines: 1, members: 3 */
};
struct trace_event_raw_xen_mc__batch {
	struct trace_entry         ent;                  /*     0     8 */
	enum paravirt_lazy_mode    mode;                 /*     8     4 */
	char                       __data[];             /*    12     0 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct trace_event_raw_xen_mc_entry {
	struct trace_entry         ent;                  /*     0     8 */
	unsigned int               op;                   /*     8     4 */
	unsigned int               nargs;                /*    12     4 */
	long unsigned int          args[6];              /*    16    48 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	char                       __data[];             /*    64     0 */

	/* size: 64, cachelines: 1, members: 5 */
};
struct trace_event_raw_xen_mc_entry_alloc {
	struct trace_entry         ent;                  /*     0     8 */
	size_t                     args;                 /*     8     8 */
	char                       __data[];             /*    16     0 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct trace_event_raw_xen_mc_callback {
	struct trace_entry         ent;                  /*     0     8 */
	xen_mc_callback_fn_t       fn;                   /*     8     8 */
	void *                     data;                 /*    16     8 */
	char                       __data[];             /*    24     0 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_xen_mc_flush_reason {
	struct trace_entry         ent;                  /*     0     8 */
	enum xen_mc_flush_reason   reason;               /*     8     4 */
	char                       __data[];             /*    12     0 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct trace_event_raw_xen_mc_flush {
	struct trace_entry         ent;                  /*     0     8 */
	unsigned int               mcidx;                /*     8     4 */
	unsigned int               argidx;               /*    12     4 */
	unsigned int               cbidx;                /*    16     4 */
	char                       __data[];             /*    20     0 */

	/* size: 20, cachelines: 1, members: 5 */
	/* last cacheline: 20 bytes */
};
struct trace_event_raw_xen_mc_extend_args {
	struct trace_entry         ent;                  /*     0     8 */
	unsigned int               op;                   /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	size_t                     args;                 /*    16     8 */
	enum xen_mc_extend_args    res;                  /*    24     4 */
	char                       __data[];             /*    28     0 */

	/* size: 32, cachelines: 1, members: 5 */
	/* sum members: 24, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct trace_event_raw_xen_mmu__set_pte {
	struct trace_entry         ent;                  /*     0     8 */
	pte_t *                    ptep;                 /*     8     8 */
	pteval_t                   pteval;               /*    16     8 */
	char                       __data[];             /*    24     0 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_xen_mmu_set_pmd {
	struct trace_entry         ent;                  /*     0     8 */
	pmd_t *                    pmdp;                 /*     8     8 */
	pmdval_t                   pmdval;               /*    16     8 */
	char                       __data[];             /*    24     0 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_xen_mmu_set_pud {
	struct trace_entry         ent;                  /*     0     8 */
	pud_t *                    pudp;                 /*     8     8 */
	pudval_t                   pudval;               /*    16     8 */
	char                       __data[];             /*    24     0 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_xen_mmu_set_p4d {
	struct trace_entry         ent;                  /*     0     8 */
	p4d_t *                    p4dp;                 /*     8     8 */
	p4d_t *                    user_p4dp;            /*    16     8 */
	p4dval_t                   p4dval;               /*    24     8 */
	char                       __data[];             /*    32     0 */

	/* size: 32, cachelines: 1, members: 5 */
	/* last cacheline: 32 bytes */
};
struct trace_event_raw_xen_mmu_ptep_modify_prot {
	struct trace_entry         ent;                  /*     0     8 */
	struct mm_struct *         mm;                   /*     8     8 */
	long unsigned int          addr;                 /*    16     8 */
	pte_t *                    ptep;                 /*    24     8 */
	pteval_t                   pteval;               /*    32     8 */
	char                       __data[];             /*    40     0 */

	/* size: 40, cachelines: 1, members: 6 */
	/* last cacheline: 40 bytes */
};
struct trace_event_raw_xen_mmu_alloc_ptpage {
	struct trace_entry         ent;                  /*     0     8 */
	struct mm_struct *         mm;                   /*     8     8 */
	long unsigned int          pfn;                  /*    16     8 */
	unsigned int               level;                /*    24     4 */
	bool                       pinned;               /*    28     1 */
	char                       __data[];             /*    29     0 */

	/* size: 32, cachelines: 1, members: 6 */
	/* padding: 3 */
	/* last cacheline: 32 bytes */
};
struct trace_event_raw_xen_mmu_release_ptpage {
	struct trace_entry         ent;                  /*     0     8 */
	long unsigned int          pfn;                  /*     8     8 */
	unsigned int               level;                /*    16     4 */
	bool                       pinned;               /*    20     1 */
	char                       __data[];             /*    21     0 */

	/* size: 24, cachelines: 1, members: 5 */
	/* padding: 3 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_xen_mmu_pgd {
	struct trace_entry         ent;                  /*     0     8 */
	struct mm_struct *         mm;                   /*     8     8 */
	pgd_t *                    pgd;                  /*    16     8 */
	char                       __data[];             /*    24     0 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_xen_mmu_flush_tlb_one_user {
	struct trace_entry         ent;                  /*     0     8 */
	long unsigned int          addr;                 /*     8     8 */
	char                       __data[];             /*    16     0 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct trace_event_raw_xen_mmu_flush_tlb_multi {
	struct trace_entry         ent;                  /*     0     8 */
	unsigned int               ncpus;                /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct mm_struct *         mm;                   /*    16     8 */
	long unsigned int          addr;                 /*    24     8 */
	long unsigned int          end;                  /*    32     8 */
	char                       __data[];             /*    40     0 */

	/* size: 40, cachelines: 1, members: 6 */
	/* sum members: 36, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct trace_event_raw_xen_mmu_write_cr3 {
	struct trace_entry         ent;                  /*     0     8 */
	bool                       kernel;               /*     8     1 */

	/* XXX 7 bytes hole, try to pack */

	long unsigned int          cr3;                  /*    16     8 */
	char                       __data[];             /*    24     0 */

	/* size: 24, cachelines: 1, members: 4 */
	/* sum members: 17, holes: 1, sum holes: 7 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_xen_cpu_write_ldt_entry {
	struct trace_entry         ent;                  /*     0     8 */
	struct desc_struct *       dt;                   /*     8     8 */
	int                        entrynum;             /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        desc;                 /*    24     8 */
	char                       __data[];             /*    32     0 */

	/* size: 32, cachelines: 1, members: 5 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct trace_event_raw_xen_cpu_write_idt_entry {
	struct trace_entry         ent;                  /*     0     8 */
	gate_desc *                dt;                   /*     8     8 */
	int                        entrynum;             /*    16     4 */
	char                       __data[];             /*    20     0 */

	/* size: 24, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_xen_cpu_load_idt {
	struct trace_entry         ent;                  /*     0     8 */
	long unsigned int          addr;                 /*     8     8 */
	char                       __data[];             /*    16     0 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct trace_event_raw_xen_cpu_write_gdt_entry {
	struct trace_entry         ent;                  /*     0     8 */
	u64                        desc;                 /*     8     8 */
	struct desc_struct *       dt;                   /*    16     8 */
	int                        entrynum;             /*    24     4 */
	int                        type;                 /*    28     4 */
	char                       __data[];             /*    32     0 */

	/* size: 32, cachelines: 1, members: 6 */
	/* last cacheline: 32 bytes */
};
struct trace_event_raw_xen_cpu_set_ldt {
	struct trace_entry         ent;                  /*     0     8 */
	const void  *              addr;                 /*     8     8 */
	unsigned int               entries;              /*    16     4 */
	char                       __data[];             /*    20     0 */

	/* size: 24, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct trace_event_data_offsets_xen_mc__batch {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_xen_mc_entry {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_xen_mc_entry_alloc {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_xen_mc_callback {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_xen_mc_flush_reason {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_xen_mc_flush {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_xen_mc_extend_args {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_xen_mmu__set_pte {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_xen_mmu_set_pmd {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_xen_mmu_set_pud {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_xen_mmu_set_p4d {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_xen_mmu_ptep_modify_prot {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_xen_mmu_alloc_ptpage {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_xen_mmu_release_ptpage {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_xen_mmu_pgd {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_xen_mmu_flush_tlb_one_user {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_xen_mmu_flush_tlb_multi {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_xen_mmu_write_cr3 {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_xen_cpu_write_ldt_entry {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_xen_cpu_write_idt_entry {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_xen_cpu_load_idt {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_xen_cpu_write_gdt_entry {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_xen_cpu_set_ldt {

	/* size: 0, cachelines: 0, members: 0 */
};
struct xen_common_irq {
	int                        irq;                  /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	char *                     name;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct dma_map_ops {
	void *                     (*alloc)(struct device *, size_t, dma_addr_t *, gfp_t, long unsigned int); /*     0     8 */
	void                       (*free)(struct device *, size_t, void *, dma_addr_t, long unsigned int); /*     8     8 */
	struct page *              (*alloc_pages)(struct device *, size_t, dma_addr_t *, enum dma_data_direction, gfp_t); /*    16     8 */
	void                       (*free_pages)(struct device *, size_t, struct page *, dma_addr_t, enum dma_data_direction); /*    24     8 */
	void *                     (*alloc_noncoherent)(struct device *, size_t, dma_addr_t *, enum dma_data_direction, gfp_t); /*    32     8 */
	void                       (*free_noncoherent)(struct device *, size_t, void *, dma_addr_t, enum dma_data_direction); /*    40     8 */
	int                        (*mmap)(struct device *, struct vm_area_struct *, void *, dma_addr_t, size_t, long unsigned int); /*    48     8 */
	int                        (*get_sgtable)(struct device *, struct sg_table *, void *, dma_addr_t, size_t, long unsigned int); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	dma_addr_t                 (*map_page)(struct device *, struct page *, long unsigned int, size_t, enum dma_data_direction, long unsigned int); /*    64     8 */
	void                       (*unmap_page)(struct device *, dma_addr_t, size_t, enum dma_data_direction, long unsigned int); /*    72     8 */
	int                        (*map_sg)(struct device *, struct scatterlist *, int, enum dma_data_direction, long unsigned int); /*    80     8 */
	void                       (*unmap_sg)(struct device *, struct scatterlist *, int, enum dma_data_direction, long unsigned int); /*    88     8 */
	dma_addr_t                 (*map_resource)(struct device *, phys_addr_t, size_t, enum dma_data_direction, long unsigned int); /*    96     8 */
	void                       (*unmap_resource)(struct device *, dma_addr_t, size_t, enum dma_data_direction, long unsigned int); /*   104     8 */
	void                       (*sync_single_for_cpu)(struct device *, dma_addr_t, size_t, enum dma_data_direction); /*   112     8 */
	void                       (*sync_single_for_device)(struct device *, dma_addr_t, size_t, enum dma_data_direction); /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	void                       (*sync_sg_for_cpu)(struct device *, struct scatterlist *, int, enum dma_data_direction); /*   128     8 */
	void                       (*sync_sg_for_device)(struct device *, struct scatterlist *, int, enum dma_data_direction); /*   136     8 */
	void                       (*cache_sync)(struct device *, void *, size_t, enum dma_data_direction); /*   144     8 */
	int                        (*dma_supported)(struct device *, u64); /*   152     8 */
	u64                        (*get_required_mask)(struct device *); /*   160     8 */
	size_t                     (*max_mapping_size)(struct device *); /*   168     8 */
	long unsigned int          (*get_merge_boundary)(struct device *); /*   176     8 */
	u64                        kabi_reserved1;       /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	u64                        kabi_reserved2;       /*   192     8 */
	u64                        kabi_reserved3;       /*   200     8 */
	u64                        kabi_reserved4;       /*   208     8 */
	u64                        kabi_reserved5;       /*   216     8 */
	u64                        kabi_reserved6;       /*   224     8 */

	/* size: 232, cachelines: 4, members: 29 */
	/* last cacheline: 40 bytes */
};
struct sg_table {
	struct scatterlist *       sgl;                  /*     0     8 */
	unsigned int               nents;                /*     8     4 */
	unsigned int               orig_nents;           /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct iommu_table_entry {
	initcall_t                 detect;               /*     0     8 */
	initcall_t                 depend;               /*     8     8 */
	void                       (*early_init)(void);  /*    16     8 */
	void                       (*late_init)(void);   /*    24     8 */
	int                        flags;                /*    32     4 */

	/* size: 40, cachelines: 1, members: 5 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct xenpf_settime32 {
	uint32_t                   secs;                 /*     0     4 */
	uint32_t                   nsecs;                /*     4     4 */
	uint64_t                   system_time;          /*     8     8 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct xenpf_settime64 {
	uint64_t                   secs;                 /*     0     8 */
	uint32_t                   nsecs;                /*     8     4 */
	uint32_t                   mbz;                  /*    12     4 */
	uint64_t                   system_time;          /*    16     8 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct xenpf_add_memtype {
	xen_pfn_t                  mfn;                  /*     0     8 */
	uint64_t                   nr_mfns;              /*     8     8 */
	uint32_t                   type;                 /*    16     4 */
	uint32_t                   handle;               /*    20     4 */
	uint32_t                   reg;                  /*    24     4 */

	/* size: 32, cachelines: 1, members: 5 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct xenpf_del_memtype {
	uint32_t                   handle;               /*     0     4 */
	uint32_t                   reg;                  /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct xenpf_read_memtype {
	uint32_t                   reg;                  /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	xen_pfn_t                  mfn;                  /*     8     8 */
	uint64_t                   nr_mfns;              /*    16     8 */
	uint32_t                   type;                 /*    24     4 */

	/* size: 32, cachelines: 1, members: 4 */
	/* sum members: 24, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct xenpf_microcode_update {
	__guest_handle_void        data;                 /*     0     8 */
	uint32_t                   length;               /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct xenpf_platform_quirk {
	uint32_t                   quirk_id;             /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct xenpf_efi_time {
	uint16_t                   year;                 /*     0     2 */
	uint8_t                    month;                /*     2     1 */
	uint8_t                    day;                  /*     3     1 */
	uint8_t                    hour;                 /*     4     1 */
	uint8_t                    min;                  /*     5     1 */
	uint8_t                    sec;                  /*     6     1 */

	/* XXX 1 byte hole, try to pack */

	uint32_t                   ns;                   /*     8     4 */
	int16_t                    tz;                   /*    12     2 */
	uint8_t                    daylight;             /*    14     1 */

	/* size: 16, cachelines: 1, members: 9 */
	/* sum members: 14, holes: 1, sum holes: 1 */
	/* padding: 1 */
	/* last cacheline: 16 bytes */
};
struct xenpf_efi_guid {
	uint32_t                   data1;                /*     0     4 */
	uint16_t                   data2;                /*     4     2 */
	uint16_t                   data3;                /*     6     2 */
	uint8_t                    data4[8];             /*     8     8 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct xenpf_efi_runtime_call {
	uint32_t                   function;             /*     0     4 */
	uint32_t                   misc;                 /*     4     4 */
	xen_ulong_t                status;               /*     8     8 */
	union {
		struct {
			struct xenpf_efi_time time;      /*    16    16 */

			/* XXX last struct has 1 byte of padding */

			uint32_t   resolution;           /*    32     4 */
			uint32_t   accuracy;             /*    36     4 */
		} get_time;                              /*    16    24 */
		struct xenpf_efi_time set_time;          /*    16    16 */
		struct xenpf_efi_time get_wakeup_time;   /*    16    16 */
		struct xenpf_efi_time set_wakeup_time;   /*    16    16 */
		struct {
			__guest_handle_void name;        /*    16     8 */
			xen_ulong_t size;                /*    24     8 */
			__guest_handle_void data;        /*    32     8 */
			struct xenpf_efi_guid vendor_guid; /*    40    16 */
		} get_variable;                          /*    16    40 */
		struct {
			__guest_handle_void name;        /*    16     8 */
			xen_ulong_t size;                /*    24     8 */
			__guest_handle_void data;        /*    32     8 */
			struct xenpf_efi_guid vendor_guid; /*    40    16 */
		} set_variable;                          /*    16    40 */
		struct {
			xen_ulong_t size;                /*    16     8 */
			__guest_handle_void name;        /*    24     8 */
			struct xenpf_efi_guid vendor_guid; /*    32    16 */
		} get_next_variable_name;                /*    16    32 */
		struct {
			uint32_t   attr;                 /*    16     4 */

			/* XXX 4 bytes hole, try to pack */

			uint64_t   max_store_size;       /*    24     8 */
			uint64_t   remain_store_size;    /*    32     8 */
			uint64_t   max_size;             /*    40     8 */
		} query_variable_info;                   /*    16    32 */
		struct {
			__guest_handle_void capsule_header_array; /*    16     8 */
			xen_ulong_t capsule_count;       /*    24     8 */
			uint64_t   max_capsule_size;     /*    32     8 */
			uint32_t   reset_type;           /*    40     4 */
		} query_capsule_capabilities;            /*    16    32 */
		struct {
			__guest_handle_void capsule_header_array; /*    16     8 */
			xen_ulong_t capsule_count;       /*    24     8 */
			uint64_t   sg_list;              /*    32     8 */
		} update_capsule;                        /*    16    24 */
	} u;                                             /*    16    40 */

	/* size: 56, cachelines: 1, members: 4 */
	/* last cacheline: 56 bytes */
};
union xenpf_efi_info {
	uint32_t                   version;            /*     0     4 */
	struct {
		uint64_t           addr;               /*     0     8 */
		uint32_t           nent;               /*     8     4 */
	} cfg;                                         /*     0    16 */
	struct {
		uint32_t           revision;           /*     0     4 */
		uint32_t           bufsz;              /*     4     4 */
		__guest_handle_void name;              /*     8     8 */
	} vendor;                                      /*     0    16 */
	struct {
		uint64_t           addr;               /*     0     8 */
		uint64_t           size;               /*     8     8 */
		uint64_t           attr;               /*    16     8 */
		uint32_t           type;               /*    24     4 */
	} mem;                                         /*     0    32 */
};
struct xenpf_firmware_info {
	uint32_t                   type;                 /*     0     4 */
	uint32_t                   index;                /*     4     4 */
	union {
		struct {
			uint8_t    device;               /*     8     1 */
			uint8_t    version;              /*     9     1 */
			uint16_t   interface_support;    /*    10     2 */
			uint16_t   legacy_max_cylinder;  /*    12     2 */
			uint8_t    legacy_max_head;      /*    14     1 */
			uint8_t    legacy_sectors_per_track; /*    15     1 */
			__guest_handle_void edd_params;  /*    16     8 */
		} disk_info;                             /*     8    16 */
		struct {
			uint8_t    device;               /*     8     1 */

			/* XXX 3 bytes hole, try to pack */

			uint32_t   mbr_signature;        /*    12     4 */
		} disk_mbr_signature;                    /*     8     8 */
		struct {
			uint8_t    capabilities;         /*     8     1 */
			uint8_t    edid_transfer_time;   /*     9     1 */

			/* XXX 6 bytes hole, try to pack */

			__guest_handle_uchar edid;       /*    16     8 */
		} vbeddc_info;                           /*     8    16 */
		union xenpf_efi_info efi_info;           /*     8    32 */
		uint8_t            kbd_shift_flags;      /*     8     1 */
	} u;                                             /*     8    32 */

	/* size: 40, cachelines: 1, members: 3 */
	/* last cacheline: 40 bytes */
};
struct xenpf_enter_acpi_sleep {
	uint16_t                   val_a;                /*     0     2 */
	uint16_t                   val_b;                /*     2     2 */
	uint32_t                   sleep_state;          /*     4     4 */
	uint32_t                   flags;                /*     8     4 */

	/* size: 12, cachelines: 1, members: 4 */
	/* last cacheline: 12 bytes */
};
struct xenpf_change_freq {
	uint32_t                   flags;                /*     0     4 */
	uint32_t                   cpu;                  /*     4     4 */
	uint64_t                   freq;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct xenpf_getidletime {
	__guest_handle_uchar       cpumap_bitmap;        /*     0     8 */
	uint32_t                   cpumap_nr_cpus;       /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	__guest_handle_uint64_t    idletime;             /*    16     8 */
	uint64_t                   now;                  /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct xen_power_register {
	uint32_t                   space_id;             /*     0     4 */
	uint32_t                   bit_width;            /*     4     4 */
	uint32_t                   bit_offset;           /*     8     4 */
	uint32_t                   access_size;          /*    12     4 */
	uint64_t                   address;              /*    16     8 */

	/* size: 24, cachelines: 1, members: 5 */
	/* last cacheline: 24 bytes */
};
struct xen_processor_csd {
	uint32_t                   domain;               /*     0     4 */
	uint32_t                   coord_type;           /*     4     4 */
	uint32_t                   num;                  /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct xen_processor_cx {
	struct xen_power_register  reg;                  /*     0    24 */
	uint8_t                    type;                 /*    24     1 */

	/* XXX 3 bytes hole, try to pack */

	uint32_t                   latency;              /*    28     4 */
	uint32_t                   power;                /*    32     4 */
	uint32_t                   dpcnt;                /*    36     4 */
	__guest_handle_xen_processor_csd dp;             /*    40     8 */

	/* size: 48, cachelines: 1, members: 6 */
	/* sum members: 45, holes: 1, sum holes: 3 */
	/* last cacheline: 48 bytes */
};
struct xen_processor_flags {
	uint32_t                   bm_control:1;         /*     0: 0  4 */
	uint32_t                   bm_check:1;           /*     0: 1  4 */
	uint32_t                   has_cst:1;            /*     0: 2  4 */
	uint32_t                   power_setup_done:1;   /*     0: 3  4 */
	uint32_t                   bm_rld_set:1;         /*     0: 4  4 */

	/* size: 4, cachelines: 1, members: 5 */
	/* bit_padding: 27 bits */
	/* last cacheline: 4 bytes */
};
struct xen_processor_power {
	uint32_t                   count;                /*     0     4 */
	struct xen_processor_flags flags;                /*     4     4 */
	__guest_handle_xen_processor_cx states;          /*     8     8 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct xen_pct_register {
	uint8_t                    descriptor;           /*     0     1 */

	/* XXX 1 byte hole, try to pack */

	uint16_t                   length;               /*     2     2 */
	uint8_t                    space_id;             /*     4     1 */
	uint8_t                    bit_width;            /*     5     1 */
	uint8_t                    bit_offset;           /*     6     1 */
	uint8_t                    reserved;             /*     7     1 */
	uint64_t                   address;              /*     8     8 */

	/* size: 16, cachelines: 1, members: 7 */
	/* sum members: 15, holes: 1, sum holes: 1 */
	/* last cacheline: 16 bytes */
};
struct xen_processor_px {
	uint64_t                   core_frequency;       /*     0     8 */
	uint64_t                   power;                /*     8     8 */
	uint64_t                   transition_latency;   /*    16     8 */
	uint64_t                   bus_master_latency;   /*    24     8 */
	uint64_t                   control;              /*    32     8 */
	uint64_t                   status;               /*    40     8 */

	/* size: 48, cachelines: 1, members: 6 */
	/* last cacheline: 48 bytes */
};
struct xen_psd_package {
	uint64_t                   num_entries;          /*     0     8 */
	uint64_t                   revision;             /*     8     8 */
	uint64_t                   domain;               /*    16     8 */
	uint64_t                   coord_type;           /*    24     8 */
	uint64_t                   num_processors;       /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct xen_processor_performance {
	uint32_t                   flags;                /*     0     4 */
	uint32_t                   platform_limit;       /*     4     4 */
	struct xen_pct_register    control_register;     /*     8    16 */
	struct xen_pct_register    status_register;      /*    24    16 */
	uint32_t                   state_count;          /*    40     4 */

	/* XXX 4 bytes hole, try to pack */

	__guest_handle_xen_processor_px states;          /*    48     8 */
	struct xen_psd_package     domain_info;          /*    56    40 */
	/* --- cacheline 1 boundary (64 bytes) was 32 bytes ago --- */
	uint32_t                   shared_type;          /*    96     4 */

	/* size: 104, cachelines: 2, members: 8 */
	/* sum members: 96, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct xenpf_set_processor_pminfo {
	uint32_t                   id;                   /*     0     4 */
	uint32_t                   type;                 /*     4     4 */
	union {
		struct xen_processor_power power;        /*     8    16 */
		struct xen_processor_performance perf;   /*     8   104 */
		__guest_handle_uint32_t pdc;             /*     8     8 */
	};                                               /*     8   104 */

	/* size: 112, cachelines: 2, members: 3 */
	/* last cacheline: 48 bytes */
};
struct xenpf_pcpuinfo {
	uint32_t                   xen_cpuid;            /*     0     4 */
	uint32_t                   max_present;          /*     4     4 */
	uint32_t                   flags;                /*     8     4 */
	uint32_t                   apic_id;              /*    12     4 */
	uint32_t                   acpi_id;              /*    16     4 */

	/* size: 20, cachelines: 1, members: 5 */
	/* last cacheline: 20 bytes */
};
struct xenpf_cpu_ol {
	uint32_t                   cpuid;                /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct xenpf_cpu_hotadd {
	uint32_t                   apic_id;              /*     0     4 */
	uint32_t                   acpi_id;              /*     4     4 */
	uint32_t                   pxm;                  /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct xenpf_mem_hotadd {
	uint64_t                   spfn;                 /*     0     8 */
	uint64_t                   epfn;                 /*     8     8 */
	uint32_t                   pxm;                  /*    16     4 */
	uint32_t                   flags;                /*    20     4 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct xenpf_core_parking {
	uint32_t                   type;                 /*     0     4 */
	uint32_t                   idle_nums;            /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct xenpf_symdata {
	uint32_t                   namelen;              /*     0     4 */
	uint32_t                   symnum;               /*     4     4 */
	__guest_handle_char        name;                 /*     8     8 */
	uint64_t                   address;              /*    16     8 */
	char                       type;                 /*    24     1 */

	/* size: 32, cachelines: 1, members: 5 */
	/* padding: 7 */
	/* last cacheline: 32 bytes */
};
struct xen_platform_op {
	uint32_t                   cmd;                  /*     0     4 */
	uint32_t                   interface_version;    /*     4     4 */
	union {
		struct xenpf_settime32 settime32;        /*     8    16 */
		struct xenpf_settime64 settime64;        /*     8    24 */
		struct xenpf_add_memtype add_memtype;    /*     8    32 */
		struct xenpf_del_memtype del_memtype;    /*     8     8 */
		struct xenpf_read_memtype read_memtype;  /*     8    32 */
		struct xenpf_microcode_update microcode; /*     8    16 */
		struct xenpf_platform_quirk platform_quirk; /*     8     4 */
		struct xenpf_efi_runtime_call efi_runtime_call; /*     8    56 */
		struct xenpf_firmware_info firmware_info; /*     8    40 */
		struct xenpf_enter_acpi_sleep enter_acpi_sleep; /*     8    12 */
		struct xenpf_change_freq change_freq;    /*     8    16 */
		struct xenpf_getidletime getidletime;    /*     8    32 */
		struct xenpf_set_processor_pminfo set_pminfo; /*     8   112 */
		struct xenpf_pcpuinfo pcpu_info;         /*     8    20 */
		struct xenpf_cpu_ol cpu_ol;              /*     8     4 */
		struct xenpf_cpu_hotadd cpu_add;         /*     8    12 */
		struct xenpf_mem_hotadd mem_add;         /*     8    24 */
		struct xenpf_core_parking core_parking;  /*     8     8 */
		struct xenpf_symdata symdata;            /*     8    32 */
		uint8_t            pad[128];             /*     8   128 */
	} u;                                             /*     8   128 */

	/* size: 136, cachelines: 3, members: 3 */
	/* last cacheline: 8 bytes */
};
union hv_x64_msr_hypercall_contents {
	u64                        as_uint64;          /*     0     8 */
	struct {
		u64                enable:1;           /*     0: 0  8 */
		u64                reserved:11;        /*     0: 1  8 */
		u64                guest_physical_address:52; /*     0:12  8 */
	};                                             /*     0     8 */
};
struct hv_reenlightenment_control {
	__u64                      vector:8;             /*     0: 0  8 */
	__u64                      reserved1:8;          /*     0: 8  8 */
	__u64                      enabled:1;            /*     0:16  8 */
	__u64                      reserved2:15;         /*     0:17  8 */
	__u64                      target_vp:32;         /*     0:32  8 */

	/* size: 8, cachelines: 1, members: 5 */
	/* last cacheline: 8 bytes */
};
struct hv_tsc_emulation_control {
	__u64                      enabled:1;            /*     0: 0  8 */
	__u64                      reserved:63;          /*     0: 1  8 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct hv_tsc_emulation_status {
	__u64                      inprogress:1;         /*     0: 0  8 */
	__u64                      reserved:63;          /*     0: 1  8 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct hv_nested_enlightenments_control {
	struct {
		__u32              directhypercall:1;    /*     0: 0  4 */
		__u32              reserved:31;          /*     0: 1  4 */
	} features;                                      /*     0     4 */
	struct {
		__u32              reserved;             /*     4     4 */
	} hypercallControls;                             /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct hv_vp_assist_page {
	__u32                      apic_assist;          /*     0     4 */
	__u32                      reserved1;            /*     4     4 */
	__u64                      vtl_control[3];       /*     8    24 */
	struct hv_nested_enlightenments_control nested_control; /*    32     8 */
	__u8                       enlighten_vmentry;    /*    40     1 */
	__u8                       reserved2[7];         /*    41     7 */
	__u64                      current_nested_vmcs;  /*    48     8 */

	/* size: 56, cachelines: 1, members: 7 */
	/* last cacheline: 56 bytes */
};
struct ms_hyperv_info {
	u32                        features;             /*     0     4 */
	u32                        misc_features;        /*     4     4 */
	u32                        hints;                /*     8     4 */
	u32                        nested_features;      /*    12     4 */
	u32                        max_vp_index;         /*    16     4 */
	u32                        max_lp_index;         /*    20     4 */

	/* size: 24, cachelines: 1, members: 6 */
	/* last cacheline: 24 bytes */
};
struct mmu_gather {
	struct mm_struct *         mm;                   /*     0     8 */
	struct mmu_table_batch *   batch;                /*     8     8 */
	long unsigned int          start;                /*    16     8 */
	long unsigned int          end;                  /*    24     8 */
	unsigned int               fullmm:1;             /*    32: 0  4 */
	unsigned int               need_flush_all:1;     /*    32: 1  4 */
	unsigned int               freed_tables:1;       /*    32: 2  4 */
	unsigned int               cleared_ptes:1;       /*    32: 3  4 */
	unsigned int               cleared_pmds:1;       /*    32: 4  4 */
	unsigned int               cleared_puds:1;       /*    32: 5  4 */
	unsigned int               cleared_p4ds:1;       /*    32: 6  4 */
	unsigned int               vma_exec:1;           /*    32: 7  4 */
	unsigned int               vma_huge:1;           /*    32: 8  4 */

	/* XXX 23 bits hole, try to pack */

	unsigned int               batch_count;          /*    36     4 */
	struct mmu_gather_batch *  active;               /*    40     8 */
	struct mmu_gather_batch    local;                /*    48    16 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct page *              __pages[8];           /*    64    64 */

	/* size: 128, cachelines: 2, members: 17 */
	/* sum members: 124 */
	/* sum bitfield members: 9 bits, bit holes: 1, sum bit holes: 23 bits */
};
struct hv_vpset {
	u64                        format;               /*     0     8 */
	u64                        valid_bank_mask;      /*     8     8 */
	u64                        bank_contents[];      /*    16     0 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct hv_tlb_flush {
	u64                        address_space;        /*     0     8 */
	u64                        flags;                /*     8     8 */
	u64                        processor_mask;       /*    16     8 */
	u64                        gva_list[];           /*    24     0 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct hv_tlb_flush_ex {
	u64                        address_space;        /*     0     8 */
	u64                        flags;                /*     8     8 */
	struct hv_vpset            hv_vp_set;            /*    16    16 */
	u64                        gva_list[];           /*    32     0 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct mmu_table_batch {
	struct callback_head       rcu __attribute__((__aligned__(8))); /*     0    16 */
	unsigned int               nr;                   /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     tables[];             /*    24     0 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct mmu_gather_batch {
	struct mmu_gather_batch *  next;                 /*     0     8 */
	unsigned int               nr;                   /*     8     4 */
	unsigned int               max;                  /*    12     4 */
	struct page *              pages[];              /*    16     0 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct trace_event_raw_hyperv_mmu_flush_tlb_multi {
	struct trace_entry         ent;                  /*     0     8 */
	unsigned int               ncpus;                /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct mm_struct *         mm;                   /*    16     8 */
	long unsigned int          addr;                 /*    24     8 */
	long unsigned int          end;                  /*    32     8 */
	char                       __data[];             /*    40     0 */

	/* size: 40, cachelines: 1, members: 6 */
	/* sum members: 36, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct trace_event_raw_hyperv_nested_flush_guest_mapping {
	struct trace_entry         ent;                  /*     0     8 */
	u64                        as;                   /*     8     8 */
	int                        ret;                  /*    16     4 */
	char                       __data[];             /*    20     0 */

	/* size: 24, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_hyperv_nested_flush_guest_mapping_range {
	struct trace_entry         ent;                  /*     0     8 */
	u64                        as;                   /*     8     8 */
	int                        ret;                  /*    16     4 */
	char                       __data[];             /*    20     0 */

	/* size: 24, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_hyperv_send_ipi_mask {
	struct trace_entry         ent;                  /*     0     8 */
	unsigned int               ncpus;                /*     8     4 */
	int                        vector;               /*    12     4 */
	char                       __data[];             /*    16     0 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct trace_event_raw_hyperv_send_ipi_one {
	struct trace_entry         ent;                  /*     0     8 */
	int                        cpu;                  /*     8     4 */
	int                        vector;               /*    12     4 */
	char                       __data[];             /*    16     0 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct trace_event_data_offsets_hyperv_mmu_flush_tlb_multi {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_hyperv_nested_flush_guest_mapping {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_hyperv_nested_flush_guest_mapping_range {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_hyperv_send_ipi_mask {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_hyperv_send_ipi_one {

	/* size: 0, cachelines: 0, members: 0 */
};
struct hv_guest_mapping_flush {
	u64                        address_space;        /*     0     8 */
	u64                        flags;                /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
union hv_gpa_page_range {
	u64                        address_space;      /*     0     8 */
	struct {
		u64                additional_pages:11; /*     0: 0  8 */
		u64                largepage:1;        /*     0:11  8 */
		u64                basepfn:52;         /*     0:12  8 */
	} page;                                        /*     0     8 */
};
struct hv_guest_mapping_flush_list {
	u64                        address_space;        /*     0     8 */
	u64                        flags;                /*     8     8 */
	union hv_gpa_page_range    gpa_list[510];        /*    16  4080 */

	/* size: 4096, cachelines: 64, members: 3 */
};
struct hv_send_ipi {
	u32                        vector;               /*     0     4 */
	u32                        reserved;             /*     4     4 */
	u64                        cpu_mask;             /*     8     8 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct hv_send_ipi_ex {
	u32                        vector;               /*     0     4 */
	u32                        reserved;             /*     4     4 */
	struct hv_vpset            vp_set;               /*     8    16 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct real_mode_header {
	u32                        text_start;           /*     0     4 */
	u32                        ro_end;               /*     4     4 */
	u32                        trampoline_start;     /*     8     4 */
	u32                        trampoline_header;    /*    12     4 */
	u32                        sev_es_trampoline_start; /*    16     4 */
	u32                        trampoline_pgd;       /*    20     4 */
	u32                        wakeup_start;         /*    24     4 */
	u32                        wakeup_header;        /*    28     4 */
	u32                        machine_real_restart_asm; /*    32     4 */
	u32                        machine_real_restart_seg; /*    36     4 */

	/* size: 40, cachelines: 1, members: 10 */
	/* last cacheline: 40 bytes */
};
struct trampoline_header {
	u64                        start;                /*     0     8 */
	u64                        efer;                 /*     8     8 */
	u32                        cr4;                  /*    16     4 */
	u32                        flags;                /*    20     4 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct x86_hw_tss {
	u32                        reserved1;            /*     0     4 */
	u64                        sp0;                  /*     4     8 */
	u64                        sp1;                  /*    12     8 */
	u64                        sp2;                  /*    20     8 */
	u64                        reserved2;            /*    28     8 */
	u64                        ist[7];               /*    36    56 */
	/* --- cacheline 1 boundary (64 bytes) was 28 bytes ago --- */
	u32                        reserved3;            /*    92     4 */
	u32                        reserved4;            /*    96     4 */
	u16                        reserved5;            /*   100     2 */
	u16                        io_bitmap_base;       /*   102     2 */

	/* size: 104, cachelines: 2, members: 10 */
	/* last cacheline: 40 bytes */
} __attribute__((__packed__));
struct x86_io_bitmap {
	u64                        prev_sequence;        /*     0     8 */
	unsigned int               prev_max;             /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          bitmap[1025];         /*    16  8200 */
	/* --- cacheline 128 boundary (8192 bytes) was 24 bytes ago --- */
	long unsigned int          mapall[1025];         /*  8216  8200 */

	/* size: 16416, cachelines: 257, members: 4 */
	/* sum members: 16412, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct tss_struct {
	struct x86_hw_tss          x86_tss;              /*     0   104 */
	/* --- cacheline 1 boundary (64 bytes) was 40 bytes ago --- */
	struct x86_io_bitmap       io_bitmap;            /*   104 16416 */

	/* size: 20480, cachelines: 320, members: 2 */
	/* padding: 3960 */
} __attribute__((__aligned__(4096)));
struct resctrl_pqr_state {
	u32                        cur_rmid;             /*     0     4 */
	u32                        cur_closid;           /*     4     4 */
	u32                        default_rmid;         /*     8     4 */
	u32                        default_closid;       /*    12     4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct sigcontext_64 {
	__u64                      r8;                   /*     0     8 */
	__u64                      r9;                   /*     8     8 */
	__u64                      r10;                  /*    16     8 */
	__u64                      r11;                  /*    24     8 */
	__u64                      r12;                  /*    32     8 */
	__u64                      r13;                  /*    40     8 */
	__u64                      r14;                  /*    48     8 */
	__u64                      r15;                  /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	__u64                      di;                   /*    64     8 */
	__u64                      si;                   /*    72     8 */
	__u64                      bp;                   /*    80     8 */
	__u64                      bx;                   /*    88     8 */
	__u64                      dx;                   /*    96     8 */
	__u64                      ax;                   /*   104     8 */
	__u64                      cx;                   /*   112     8 */
	__u64                      sp;                   /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	__u64                      ip;                   /*   128     8 */
	__u64                      flags;                /*   136     8 */
	__u16                      cs;                   /*   144     2 */
	__u16                      gs;                   /*   146     2 */
	__u16                      fs;                   /*   148     2 */
	__u16                      ss;                   /*   150     2 */
	__u64                      err;                  /*   152     8 */
	__u64                      trapno;               /*   160     8 */
	__u64                      oldmask;              /*   168     8 */
	__u64                      cr2;                  /*   176     8 */
	__u64                      fpstate;              /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	__u64                      reserved1[8];         /*   192    64 */

	/* size: 256, cachelines: 4, members: 28 */
};
struct sigaltstack {
	void *                     ss_sp;                /*     0     8 */
	int                        ss_flags;             /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	size_t                     ss_size;              /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct siginfo {
	union {
		struct {
			int        si_signo;             /*     0     4 */
			int        si_errno;             /*     4     4 */
			int        si_code;              /*     8     4 */

			/* XXX 4 bytes hole, try to pack */

			union __sifields _sifields;      /*    16    32 */
		};                                       /*     0    48 */
		int                _si_pad[32];          /*     0   128 */
	};                                               /*     0   128 */

	/* size: 128, cachelines: 2, members: 1 */
};
struct ksignal {
	struct k_sigaction         ka;                   /*     0    32 */
	kernel_siginfo_t           info;                 /*    32    48 */
	/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
	int                        sig;                  /*    80     4 */

	/* size: 88, cachelines: 2, members: 3 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct __large_struct {
	long unsigned int          buf[100];             /*     0   800 */

	/* size: 800, cachelines: 13, members: 1 */
	/* last cacheline: 32 bytes */
};
struct ucontext {
	long unsigned int          uc_flags;             /*     0     8 */
	struct ucontext *          uc_link;              /*     8     8 */
	stack_t                    uc_stack;             /*    16    24 */
	struct sigcontext_64       uc_mcontext;          /*    40   256 */
	/* --- cacheline 4 boundary (256 bytes) was 40 bytes ago --- */
	sigset_t                   uc_sigmask;           /*   296     8 */

	/* size: 304, cachelines: 5, members: 5 */
	/* last cacheline: 48 bytes */
};
struct kernel_vm86_regs {
	struct pt_regs             pt;                   /*     0   168 */
	/* --- cacheline 2 boundary (128 bytes) was 40 bytes ago --- */
	short unsigned int         es;                   /*   168     2 */
	short unsigned int         __esh;                /*   170     2 */
	short unsigned int         ds;                   /*   172     2 */
	short unsigned int         __dsh;                /*   174     2 */
	short unsigned int         fs;                   /*   176     2 */
	short unsigned int         __fsh;                /*   178     2 */
	short unsigned int         gs;                   /*   180     2 */
	short unsigned int         __gsh;                /*   182     2 */

	/* size: 184, cachelines: 3, members: 9 */
	/* last cacheline: 56 bytes */
};
struct rt_sigframe {
	char *                     pretcode;             /*     0     8 */
	struct ucontext            uc;                   /*     8   304 */
	/* --- cacheline 4 boundary (256 bytes) was 56 bytes ago --- */
	struct siginfo             info;                 /*   312   128 */

	/* size: 440, cachelines: 7, members: 3 */
	/* last cacheline: 56 bytes */
};
union compat_sigval {
	compat_int_t               sival_int;          /*     0     4 */
	compat_uptr_t              sival_ptr;          /*     0     4 */
};
struct compat_siginfo {
	int                        si_signo;             /*     0     4 */
	int                        si_errno;             /*     4     4 */
	int                        si_code;              /*     8     4 */
	union {
		int                _pad[29];             /*    12   116 */
		struct {
			compat_pid_t _pid;               /*    12     4 */
			__compat_uid32_t _uid;           /*    16     4 */
		} _kill;                                 /*    12     8 */
		struct {
			compat_timer_t _tid;             /*    12     4 */
			int        _overrun;             /*    16     4 */
			compat_sigval_t _sigval;         /*    20     4 */
		} _timer;                                /*    12    12 */
		struct {
			compat_pid_t _pid;               /*    12     4 */
			__compat_uid32_t _uid;           /*    16     4 */
			compat_sigval_t _sigval;         /*    20     4 */
		} _rt;                                   /*    12    12 */
		struct {
			compat_pid_t _pid;               /*    12     4 */
			__compat_uid32_t _uid;           /*    16     4 */
			int        _status;              /*    20     4 */
			compat_clock_t _utime;           /*    24     4 */
			compat_clock_t _stime;           /*    28     4 */
		} _sigchld;                              /*    12    20 */
		struct {
			compat_uptr_t _addr;             /*    12     4 */
			union {
				short int _addr_lsb;     /*    16     2 */
				struct {
					char   _dummy_bnd[4]; /*    16     4 */
					compat_uptr_t _lower; /*    20     4 */
					compat_uptr_t _upper; /*    24     4 */
				} _addr_bnd;             /*    16    12 */
				struct {
					char   _dummy_pkey[4]; /*    16     4 */
					u32    _pkey;    /*    20     4 */
				} _addr_pkey;            /*    16     8 */
			};                               /*    16    12 */
		} _sigfault;                             /*    12    16 */
		struct {
			compat_long_t _band;             /*    12     4 */
			int        _fd;                  /*    16     4 */
		} _sigpoll;                              /*    12     8 */
		struct {
			compat_uptr_t _call_addr;        /*    12     4 */
			int        _syscall;             /*    16     4 */
			unsigned int _arch;              /*    20     4 */
		} _sigsys;                               /*    12    12 */
	} _sifields;                                     /*    12   116 */

	/* size: 128, cachelines: 2, members: 4 */
};
struct arch_specific_insn {
	kprobe_opcode_t *          insn;                 /*     0     8 */
	bool                       boostable;            /*     8     1 */
	bool                       if_modifier;          /*     9     1 */

	/* XXX 2 bytes hole, try to pack */

	int                        tp_len;               /*    12     4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* sum members: 14, holes: 1, sum holes: 2 */
	/* last cacheline: 16 bytes */
};
struct kprobe {
	struct hlist_node          hlist;                /*     0    16 */
	struct list_head           list;                 /*    16    16 */
	long unsigned int          nmissed;              /*    32     8 */
	kprobe_opcode_t *          addr;                 /*    40     8 */
	const char  *              symbol_name;          /*    48     8 */
	unsigned int               offset;               /*    56     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	kprobe_pre_handler_t       pre_handler;          /*    64     8 */
	kprobe_post_handler_t      post_handler;         /*    72     8 */
	kprobe_fault_handler_t     fault_handler;        /*    80     8 */
	kprobe_opcode_t            opcode;               /*    88     1 */

	/* XXX 7 bytes hole, try to pack */

	struct arch_specific_insn  ainsn;                /*    96    16 */
	u32                        flags;                /*   112     4 */

	/* size: 120, cachelines: 2, members: 12 */
	/* sum members: 105, holes: 2, sum holes: 11 */
	/* padding: 4 */
	/* last cacheline: 56 bytes */
};
struct cea_exception_stacks {
	char                       DF_stack_guard[4096]; /*     0  4096 */
	/* --- cacheline 64 boundary (4096 bytes) --- */
	char                       DF_stack[8192];       /*  4096  8192 */
	/* --- cacheline 192 boundary (12288 bytes) --- */
	char                       NMI_stack_guard[4096]; /* 12288  4096 */
	/* --- cacheline 256 boundary (16384 bytes) --- */
	char                       NMI_stack[8192];      /* 16384  8192 */
	/* --- cacheline 384 boundary (24576 bytes) --- */
	char                       DB_stack_guard[4096]; /* 24576  4096 */
	/* --- cacheline 448 boundary (28672 bytes) --- */
	char                       DB_stack[8192];       /* 28672  8192 */
	/* --- cacheline 576 boundary (36864 bytes) --- */
	char                       MCE_stack_guard[4096]; /* 36864  4096 */
	/* --- cacheline 640 boundary (40960 bytes) --- */
	char                       MCE_stack[8192];      /* 40960  8192 */
	/* --- cacheline 768 boundary (49152 bytes) --- */
	char                       VC_stack_guard[4096]; /* 49152  4096 */
	/* --- cacheline 832 boundary (53248 bytes) --- */
	char                       VC_stack[8192];       /* 53248  8192 */
	/* --- cacheline 960 boundary (61440 bytes) --- */
	char                       VC2_stack_guard[4096]; /* 61440  4096 */
	/* --- cacheline 1024 boundary (65536 bytes) --- */
	char                       VC2_stack[8192];      /* 65536  8192 */
	/* --- cacheline 1152 boundary (73728 bytes) --- */
	char                       IST_top_guard[4096];  /* 73728  4096 */

	/* size: 77824, cachelines: 1216, members: 13 */
};
struct bad_iret_stack {
	void *                     error_entry_ret;      /*     0     8 */
	struct pt_regs             regs;                 /*     8   168 */

	/* size: 176, cachelines: 3, members: 2 */
	/* last cacheline: 48 bytes */
};
struct trace_event_raw_x86_irq_vector {
	struct trace_entry         ent;                  /*     0     8 */
	int                        vector;               /*     8     4 */
	char                       __data[];             /*    12     0 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct trace_event_raw_vector_config {
	struct trace_entry         ent;                  /*     0     8 */
	unsigned int               irq;                  /*     8     4 */
	unsigned int               vector;               /*    12     4 */
	unsigned int               cpu;                  /*    16     4 */
	unsigned int               apicdest;             /*    20     4 */
	char                       __data[];             /*    24     0 */

	/* size: 24, cachelines: 1, members: 6 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_vector_mod {
	struct trace_entry         ent;                  /*     0     8 */
	unsigned int               irq;                  /*     8     4 */
	unsigned int               vector;               /*    12     4 */
	unsigned int               cpu;                  /*    16     4 */
	unsigned int               prev_vector;          /*    20     4 */
	unsigned int               prev_cpu;             /*    24     4 */
	char                       __data[];             /*    28     0 */

	/* size: 28, cachelines: 1, members: 7 */
	/* last cacheline: 28 bytes */
};
struct trace_event_raw_vector_reserve {
	struct trace_entry         ent;                  /*     0     8 */
	unsigned int               irq;                  /*     8     4 */
	int                        ret;                  /*    12     4 */
	char                       __data[];             /*    16     0 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct trace_event_raw_vector_alloc {
	struct trace_entry         ent;                  /*     0     8 */
	unsigned int               irq;                  /*     8     4 */
	unsigned int               vector;               /*    12     4 */
	bool                       reserved;             /*    16     1 */

	/* XXX 3 bytes hole, try to pack */

	int                        ret;                  /*    20     4 */
	char                       __data[];             /*    24     0 */

	/* size: 24, cachelines: 1, members: 6 */
	/* sum members: 21, holes: 1, sum holes: 3 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_vector_alloc_managed {
	struct trace_entry         ent;                  /*     0     8 */
	unsigned int               irq;                  /*     8     4 */
	unsigned int               vector;               /*    12     4 */
	int                        ret;                  /*    16     4 */
	char                       __data[];             /*    20     0 */

	/* size: 20, cachelines: 1, members: 5 */
	/* last cacheline: 20 bytes */
};
struct trace_event_raw_vector_activate {
	struct trace_entry         ent;                  /*     0     8 */
	unsigned int               irq;                  /*     8     4 */
	bool                       is_managed;           /*    12     1 */
	bool                       can_reserve;          /*    13     1 */
	bool                       reserve;              /*    14     1 */
	char                       __data[];             /*    15     0 */

	/* size: 16, cachelines: 1, members: 6 */
	/* padding: 1 */
	/* last cacheline: 16 bytes */
};
struct trace_event_raw_vector_teardown {
	struct trace_entry         ent;                  /*     0     8 */
	unsigned int               irq;                  /*     8     4 */
	bool                       is_managed;           /*    12     1 */
	bool                       has_reserved;         /*    13     1 */
	char                       __data[];             /*    14     0 */

	/* size: 16, cachelines: 1, members: 5 */
	/* padding: 2 */
	/* last cacheline: 16 bytes */
};
struct trace_event_raw_vector_setup {
	struct trace_entry         ent;                  /*     0     8 */
	unsigned int               irq;                  /*     8     4 */
	bool                       is_legacy;            /*    12     1 */

	/* XXX 3 bytes hole, try to pack */

	int                        ret;                  /*    16     4 */
	char                       __data[];             /*    20     0 */

	/* size: 20, cachelines: 1, members: 5 */
	/* sum members: 17, holes: 1, sum holes: 3 */
	/* last cacheline: 20 bytes */
};
struct trace_event_raw_vector_free_moved {
	struct trace_entry         ent;                  /*     0     8 */
	unsigned int               irq;                  /*     8     4 */
	unsigned int               cpu;                  /*    12     4 */
	unsigned int               vector;               /*    16     4 */
	bool                       is_managed;           /*    20     1 */
	char                       __data[];             /*    21     0 */

	/* size: 24, cachelines: 1, members: 6 */
	/* padding: 3 */
	/* last cacheline: 24 bytes */
};
struct trace_event_data_offsets_x86_irq_vector {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_vector_config {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_vector_mod {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_vector_reserve {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_vector_alloc {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_vector_alloc_managed {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_vector_activate {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_vector_teardown {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_vector_setup {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_vector_free_moved {

	/* size: 0, cachelines: 0, members: 0 */
};
struct estack_pages {
	u32                        offs;                 /*     0     4 */
	u16                        size;                 /*     4     2 */
	u16                        type;                 /*     6     2 */

	/* size: 8, cachelines: 1, members: 3 */
	/* last cacheline: 8 bytes */
};
struct entry_stack {
	char                       stack[4096];          /*     0  4096 */

	/* size: 4096, cachelines: 64, members: 1 */
};
struct trace_event_raw_nmi_handler {
	struct trace_entry         ent;                  /*     0     8 */
	void *                     handler;              /*     8     8 */
	s64                        delta_ns;             /*    16     8 */
	int                        handled;              /*    24     4 */
	char                       __data[];             /*    28     0 */

	/* size: 32, cachelines: 1, members: 5 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct trace_event_data_offsets_nmi_handler {

	/* size: 0, cachelines: 0, members: 0 */
};
struct nmi_desc {
	raw_spinlock_t             lock;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           head;                 /*     8    16 */

	/* size: 24, cachelines: 1, members: 2 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct nmi_stats {
	unsigned int               normal;               /*     0     4 */
	unsigned int               unknown;              /*     4     4 */
	unsigned int               external;             /*     8     4 */
	unsigned int               swallow;              /*    12     4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct ldttss_desc {
	u16                        limit0;               /*     0     2 */
	u16                        base0;                /*     2     2 */
	u16                        base1:8;              /*     4: 0  2 */
	u16                        type:5;               /*     4: 8  2 */
	u16                        dpl:2;                /*     4:13  2 */
	u16                        p:1;                  /*     4:15  2 */
	u16                        limit1:4;             /*     6: 0  2 */
	u16                        zero0:3;              /*     6: 4  2 */
	u16                        g:1;                  /*     6: 7  2 */
	u16                        base2:8;              /*     6: 8  2 */
	u32                        base3;                /*     8     4 */
	u32                        zero1;                /*    12     4 */

	/* size: 16, cachelines: 1, members: 12 */
	/* last cacheline: 16 bytes */
};
struct user_desc {
	unsigned int               entry_number;         /*     0     4 */
	unsigned int               base_addr;            /*     4     4 */
	unsigned int               limit;                /*     8     4 */
	unsigned int               seg_32bit:1;          /*    12: 0  4 */
	unsigned int               contents:2;           /*    12: 1  4 */
	unsigned int               read_exec_only:1;     /*    12: 3  4 */
	unsigned int               limit_in_pages:1;     /*    12: 4  4 */
	unsigned int               seg_not_present:1;    /*    12: 5  4 */
	unsigned int               useable:1;            /*    12: 6  4 */
	unsigned int               lm:1;                 /*    12: 7  4 */

	/* size: 16, cachelines: 1, members: 10 */
	/* bit_padding: 24 bits */
	/* last cacheline: 16 bytes */
};
struct edd {
	unsigned int               mbr_signature[16];    /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct edd_info            edd_info[6];          /*    64   492 */
	/* --- cacheline 8 boundary (512 bytes) was 44 bytes ago --- */
	unsigned char              mbr_signature_nr;     /*   556     1 */
	unsigned char              edd_info_nr;          /*   557     1 */

	/* size: 560, cachelines: 9, members: 4 */
	/* padding: 2 */
	/* last cacheline: 48 bytes */
};
struct setup_data {
	__u64                      next;                 /*     0     8 */
	__u32                      type;                 /*     8     4 */
	__u32                      len;                  /*    12     4 */
	__u8                       data[];               /*    16     0 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct setup_indirect {
	__u32                      type;                 /*     0     4 */
	__u32                      reserved;             /*     4     4 */
	__u64                      len;                  /*     8     8 */
	__u64                      addr;                 /*    16     8 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct extra_cpuinfo_x86 {
	__u32                      vmx_tertiary_capability[2]; /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct consw {
	struct module *            owner;                /*     0     8 */
	const char  *              (*con_startup)(void); /*     8     8 */
	void                       (*con_init)(struct vc_data *, int); /*    16     8 */
	void                       (*con_deinit)(struct vc_data *); /*    24     8 */
	void                       (*con_clear)(struct vc_data *, int, int, int, int); /*    32     8 */
	void                       (*con_putc)(struct vc_data *, int, int, int); /*    40     8 */
	void                       (*con_putcs)(struct vc_data *, const short unsigned int  *, int, int, int); /*    48     8 */
	void                       (*con_cursor)(struct vc_data *, int); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	bool                       (*con_scroll)(struct vc_data *, unsigned int, unsigned int, enum con_scroll, unsigned int); /*    64     8 */
	int                        (*con_switch)(struct vc_data *); /*    72     8 */
	int                        (*con_blank)(struct vc_data *, int, int); /*    80     8 */
	int                        (*con_font_set)(struct vc_data *, struct console_font *, unsigned int); /*    88     8 */
	int                        (*con_font_get)(struct vc_data *, struct console_font *); /*    96     8 */
	int                        (*con_font_default)(struct vc_data *, struct console_font *, char *); /*   104     8 */
	int                        (*con_font_copy)(struct vc_data *, int); /*   112     8 */
	int                        (*con_resize)(struct vc_data *, unsigned int, unsigned int, unsigned int); /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	void                       (*con_set_palette)(struct vc_data *, const unsigned char  *); /*   128     8 */
	void                       (*con_scrolldelta)(struct vc_data *, int); /*   136     8 */
	int                        (*con_set_origin)(struct vc_data *); /*   144     8 */
	void                       (*con_save_screen)(struct vc_data *); /*   152     8 */
	u8                         (*con_build_attr)(struct vc_data *, u8, enum vc_intensity, bool, bool, bool, bool); /*   160     8 */
	void                       (*con_invert_region)(struct vc_data *, u16 *, int); /*   168     8 */
	u16 *                      (*con_screen_pos)(const struct vc_data  *, int); /*   176     8 */
	long unsigned int          (*con_getxy)(struct vc_data *, long unsigned int, int *, int *); /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	void                       (*con_flush_scrollback)(struct vc_data *); /*   192     8 */
	int                        (*con_debug_enter)(struct vc_data *); /*   200     8 */
	int                        (*con_debug_leave)(struct vc_data *); /*   208     8 */

	/* size: 216, cachelines: 4, members: 27 */
	/* last cacheline: 24 bytes */
};
struct memblock_region {
	phys_addr_t                base;                 /*     0     8 */
	phys_addr_t                size;                 /*     8     8 */
	enum memblock_flags        flags;                /*    16     4 */
	int                        nid;                  /*    20     4 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct memblock_type {
	long unsigned int          cnt;                  /*     0     8 */
	long unsigned int          max;                  /*     8     8 */
	phys_addr_t                total_size;           /*    16     8 */
	struct memblock_region *   regions;              /*    24     8 */
	char *                     name;                 /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct memblock {
	bool                       bottom_up;            /*     0     1 */

	/* XXX 7 bytes hole, try to pack */

	phys_addr_t                current_limit;        /*     8     8 */
	struct memblock_type       memory;               /*    16    40 */
	struct memblock_type       reserved;             /*    56    40 */

	/* size: 96, cachelines: 2, members: 4 */
	/* sum members: 89, holes: 1, sum holes: 7 */
	/* last cacheline: 32 bytes */
};
struct e820_entry {
	u64                        addr;                 /*     0     8 */
	u64                        size;                 /*     8     8 */
	enum e820_type             type;                 /*    16     4 */

	/* size: 20, cachelines: 1, members: 3 */
	/* last cacheline: 20 bytes */
} __attribute__((__packed__));
struct e820_table {
	__u32                      nr_entries;           /*     0     4 */
	struct e820_entry          entries[3200];        /*     4 64000 */

	/* size: 64004, cachelines: 1001, members: 2 */
	/* last cacheline: 4 bytes */
} __attribute__((__packed__));
struct x86_msi_ops {
	void                       (*restore_msi_irqs)(struct pci_dev *); /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct x86_apic_ops {
	unsigned int               (*io_apic_read)(unsigned int, unsigned int); /*     0     8 */
	void                       (*restore)(void);     /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct msi_controller {
	struct module *            owner;                /*     0     8 */
	struct device *            dev;                  /*     8     8 */
	struct device_node *       of_node;              /*    16     8 */
	struct list_head           list;                 /*    24    16 */
	int                        (*setup_irq)(struct msi_controller *, struct pci_dev *, struct msi_desc *); /*    40     8 */
	int                        (*setup_irqs)(struct msi_controller *, struct pci_dev *, int, int); /*    48     8 */
	void                       (*teardown_irq)(struct msi_controller *, unsigned int); /*    56     8 */

	/* size: 64, cachelines: 1, members: 7 */
};
struct legacy_pic {
	int                        nr_legacy_irqs;       /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct irq_chip *          chip;                 /*     8     8 */
	void                       (*mask)(unsigned int); /*    16     8 */
	void                       (*unmask)(unsigned int); /*    24     8 */
	void                       (*mask_all)(void);    /*    32     8 */
	void                       (*restore_mask)(void); /*    40     8 */
	void                       (*init)(int);         /*    48     8 */
	int                        (*probe)(void);       /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*irq_pending)(unsigned int); /*    64     8 */
	void                       (*make_irq)(unsigned int); /*    72     8 */

	/* size: 80, cachelines: 2, members: 10 */
	/* sum members: 76, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
union text_poke_insn {
	u8                         text[5];            /*     0     5 */
	struct {
		u8                 opcode;             /*     0     1 */
		s32                disp;               /*     1     4 */
	} __attribute__((__packed__));                 /*     0     5 */
};
struct kernel_clone_args {
	u64                        flags;                /*     0     8 */
	int *                      pidfd;                /*     8     8 */
	int *                      child_tid;            /*    16     8 */
	int *                      parent_tid;           /*    24     8 */
	int                        exit_signal;          /*    32     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          stack;                /*    40     8 */
	long unsigned int          stack_size;           /*    48     8 */
	long unsigned int          tls;                  /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	pid_t *                    set_tid;              /*    64     8 */
	size_t                     set_tid_size;         /*    72     8 */
	int                        cgroup;               /*    80     4 */

	/* XXX 4 bytes hole, try to pack */

	struct cgroup *            cgrp;                 /*    88     8 */
	struct css_set *           cset;                 /*    96     8 */

	/* size: 104, cachelines: 2, members: 13 */
	/* sum members: 96, holes: 2, sum holes: 8 */
	/* last cacheline: 40 bytes */
};
struct kstatfs {
	long int                   f_type;               /*     0     8 */
	long int                   f_bsize;              /*     8     8 */
	u64                        f_blocks;             /*    16     8 */
	u64                        f_bfree;              /*    24     8 */
	u64                        f_bavail;             /*    32     8 */
	u64                        f_files;              /*    40     8 */
	u64                        f_ffree;              /*    48     8 */
	__kernel_fsid_t            f_fsid;               /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	long int                   f_namelen;            /*    64     8 */
	long int                   f_frsize;             /*    72     8 */
	long int                   f_flags;              /*    80     8 */
	long int                   f_spare[4];           /*    88    32 */

	/* size: 120, cachelines: 2, members: 12 */
	/* last cacheline: 56 bytes */
};
struct stat64 {
	long long unsigned int     st_dev;               /*     0     8 */
	unsigned char              __pad0[4];            /*     8     4 */
	unsigned int               __st_ino;             /*    12     4 */
	unsigned int               st_mode;              /*    16     4 */
	unsigned int               st_nlink;             /*    20     4 */
	unsigned int               st_uid;               /*    24     4 */
	unsigned int               st_gid;               /*    28     4 */
	long long unsigned int     st_rdev;              /*    32     8 */
	unsigned char              __pad3[4];            /*    40     4 */
	long long int              st_size;              /*    44     8 */
	unsigned int               st_blksize;           /*    52     4 */
	long long int              st_blocks;            /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	unsigned int               st_atime;             /*    64     4 */
	unsigned int               st_atime_nsec;        /*    68     4 */
	unsigned int               st_mtime;             /*    72     4 */
	unsigned int               st_mtime_nsec;        /*    76     4 */
	unsigned int               st_ctime;             /*    80     4 */
	unsigned int               st_ctime_nsec;        /*    84     4 */
	long long unsigned int     st_ino;               /*    88     8 */

	/* size: 96, cachelines: 2, members: 19 */
	/* last cacheline: 32 bytes */
} __attribute__((__packed__));
struct mmap_arg_struct32 {
	unsigned int               addr;                 /*     0     4 */
	unsigned int               len;                  /*     4     4 */
	unsigned int               prot;                 /*     8     4 */
	unsigned int               flags;                /*    12     4 */
	unsigned int               fd;                   /*    16     4 */
	unsigned int               offset;               /*    20     4 */

	/* size: 24, cachelines: 1, members: 6 */
	/* last cacheline: 24 bytes */
};
struct vm_unmapped_area_info {
	long unsigned int          flags;                /*     0     8 */
	long unsigned int          length;               /*     8     8 */
	long unsigned int          low_limit;            /*    16     8 */
	long unsigned int          high_limit;           /*    24     8 */
	long unsigned int          align_mask;           /*    32     8 */
	long unsigned int          align_offset;         /*    40     8 */

	/* size: 48, cachelines: 1, members: 6 */
	/* last cacheline: 48 bytes */
};
struct va_alignment {
	int                        flags;                /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          mask;                 /*     8     8 */
	long unsigned int          bits;                 /*    16     8 */

	/* size: 64, cachelines: 1, members: 3 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* padding: 40 */
} __attribute__((__aligned__(64)));
struct kobj_attribute {
	struct attribute           attr;                 /*     0    16 */

	/* XXX last struct has 6 bytes of padding */

	ssize_t                    (*show)(struct kobject *, struct kobj_attribute *, char *); /*    16     8 */
	ssize_t                    (*store)(struct kobject *, struct kobj_attribute *, const char  *, size_t); /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* paddings: 1, sum paddings: 6 */
	/* last cacheline: 32 bytes */
};
struct change_member {
	struct e820_entry *        entry;                /*     0     8 */
	long long unsigned int     addr;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct iommu_ops {
	bool                       (*capable)(enum iommu_cap); /*     0     8 */
	struct iommu_domain *      (*domain_alloc)(unsigned int); /*     8     8 */
	void                       (*domain_free)(struct iommu_domain *); /*    16     8 */
	int                        (*attach_dev)(struct iommu_domain *, struct device *); /*    24     8 */
	void                       (*detach_dev)(struct iommu_domain *, struct device *); /*    32     8 */
	int                        (*map)(struct iommu_domain *, long unsigned int, phys_addr_t, size_t, int, gfp_t); /*    40     8 */
	size_t                     (*unmap)(struct iommu_domain *, long unsigned int, size_t, struct iommu_iotlb_gather *); /*    48     8 */
	void                       (*flush_iotlb_all)(struct iommu_domain *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	void                       (*iotlb_sync_map)(struct iommu_domain *, long unsigned int, size_t); /*    64     8 */
	void                       (*iotlb_sync)(struct iommu_domain *, struct iommu_iotlb_gather *); /*    72     8 */
	phys_addr_t                (*iova_to_phys)(struct iommu_domain *, dma_addr_t); /*    80     8 */
	struct iommu_device *      (*probe_device)(struct device *); /*    88     8 */
	void                       (*release_device)(struct device *); /*    96     8 */
	void                       (*probe_finalize)(struct device *); /*   104     8 */
	struct iommu_group *       (*device_group)(struct device *); /*   112     8 */
	int                        (*domain_get_attr)(struct iommu_domain *, enum iommu_attr, void *); /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	int                        (*domain_set_attr)(struct iommu_domain *, enum iommu_attr, void *); /*   128     8 */
	bool                       (*support_dirty_log)(struct iommu_domain *); /*   136     8 */
	int                        (*switch_dirty_log)(struct iommu_domain *, bool, long unsigned int, size_t, int); /*   144     8 */
	int                        (*sync_dirty_log)(struct iommu_domain *, long unsigned int, size_t, long unsigned int *, long unsigned int, long unsigned int); /*   152     8 */
	int                        (*clear_dirty_log)(struct iommu_domain *, long unsigned int, size_t, long unsigned int *, long unsigned int, long unsigned int); /*   160     8 */
	void                       (*get_resv_regions)(struct device *, struct list_head *); /*   168     8 */
	void                       (*put_resv_regions)(struct device *, struct list_head *); /*   176     8 */
	void                       (*apply_resv_region)(struct device *, struct iommu_domain *, struct iommu_resv_region *); /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	int                        (*domain_window_enable)(struct iommu_domain *, u32, phys_addr_t, u64, int); /*   192     8 */
	void                       (*domain_window_disable)(struct iommu_domain *, u32); /*   200     8 */
	int                        (*of_xlate)(struct device *, struct of_phandle_args *); /*   208     8 */
	bool                       (*is_attach_deferred)(struct iommu_domain *, struct device *); /*   216     8 */
	bool                       (*dev_has_feat)(struct device *, enum iommu_dev_features); /*   224     8 */
	bool                       (*dev_feat_enabled)(struct device *, enum iommu_dev_features); /*   232     8 */
	int                        (*dev_enable_feat)(struct device *, enum iommu_dev_features); /*   240     8 */
	int                        (*dev_disable_feat)(struct device *, enum iommu_dev_features); /*   248     8 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	int                        (*aux_attach_dev)(struct iommu_domain *, struct device *); /*   256     8 */
	void                       (*aux_detach_dev)(struct iommu_domain *, struct device *); /*   264     8 */
	int                        (*aux_get_pasid)(struct iommu_domain *, struct device *); /*   272     8 */
	struct iommu_sva *         (*sva_bind)(struct device *, struct mm_struct *, void *); /*   280     8 */
	void                       (*sva_unbind)(struct iommu_sva *); /*   288     8 */
	u32                        (*sva_get_pasid)(struct iommu_sva *); /*   296     8 */
	int                        (*page_response)(struct device *, struct iommu_fault_event *, struct iommu_page_response *); /*   304     8 */
	int                        (*cache_invalidate)(struct iommu_domain *, struct device *, struct iommu_cache_invalidate_info *); /*   312     8 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	int                        (*sva_bind_gpasid)(struct iommu_domain *, struct device *, struct iommu_gpasid_bind_data *); /*   320     8 */
	int                        (*sva_unbind_gpasid)(struct device *, u32); /*   328     8 */
	int                        (*attach_pasid_table)(struct iommu_domain *, struct iommu_pasid_table_config *); /*   336     8 */
	void                       (*detach_pasid_table)(struct iommu_domain *); /*   344     8 */
	int                        (*def_domain_type)(struct device *); /*   352     8 */
	int                        (*kabi_reserved_bind_guest_msi)(struct iommu_domain *, dma_addr_t, phys_addr_t, size_t); /*   360     8 */
	void                       (*kabi_reserved_unbind_guest_msi)(struct iommu_domain *, dma_addr_t); /*   368     8 */
	int                        (*dev_get_config)(struct device *, int, void *); /*   376     8 */
	/* --- cacheline 6 boundary (384 bytes) --- */
	int                        (*dev_set_config)(struct device *, int, void *); /*   384     8 */
	long unsigned int          pgsize_bitmap;        /*   392     8 */
	struct module *            owner;                /*   400     8 */
	u64                        kabi_reserved1;       /*   408     8 */
	u64                        kabi_reserved2;       /*   416     8 */
	u64                        kabi_reserved3;       /*   424     8 */
	u64                        kabi_reserved4;       /*   432     8 */
	u64                        kabi_reserved5;       /*   440     8 */
	/* --- cacheline 7 boundary (448 bytes) --- */
	u64                        kabi_reserved6;       /*   448     8 */
	u64                        kabi_reserved7;       /*   456     8 */
	u64                        kabi_reserved8;       /*   464     8 */

	/* size: 472, cachelines: 8, members: 59 */
	/* last cacheline: 24 bytes */
};
struct bus_dma_region {
	phys_addr_t                cpu_start;            /*     0     8 */
	dma_addr_t                 dma_start;            /*     8     8 */
	u64                        size;                 /*    16     8 */
	u64                        offset;               /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct dev_iommu {
	struct mutex               lock;                 /*     0    32 */
	struct iommu_fault_param * fault_param;          /*    32     8 */
	struct iopf_device_param * iopf_param;           /*    40     8 */
	struct iommu_fwspec *      fwspec;               /*    48     8 */
	struct iommu_device *      iommu_dev;            /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	void *                     priv;                 /*    64     8 */

	/* size: 72, cachelines: 2, members: 6 */
	/* last cacheline: 8 bytes */
};
struct of_phandle_args {
	struct device_node *       np;                   /*     0     8 */
	int                        args_count;           /*     8     4 */
	uint32_t                   args[16];             /*    12    64 */

	/* size: 80, cachelines: 2, members: 3 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct iommu_fault_unrecoverable {
	__u32                      reason;               /*     0     4 */
	__u32                      flags;                /*     4     4 */
	__u32                      pasid;                /*     8     4 */
	__u32                      perm;                 /*    12     4 */
	__u64                      addr;                 /*    16     8 */
	__u64                      fetch_addr;           /*    24     8 */

	/* size: 32, cachelines: 1, members: 6 */
	/* last cacheline: 32 bytes */
};
struct iommu_fault_page_request {
	__u32                      flags;                /*     0     4 */
	__u32                      pasid;                /*     4     4 */
	__u32                      grpid;                /*     8     4 */
	__u32                      perm;                 /*    12     4 */
	__u64                      addr;                 /*    16     8 */
	__u64                      private_data[2];      /*    24    16 */

	/* size: 40, cachelines: 1, members: 6 */
	/* last cacheline: 40 bytes */
};
struct iommu_fault {
	__u32                      type;                 /*     0     4 */
	__u32                      padding;              /*     4     4 */
	union {
		struct iommu_fault_unrecoverable event;  /*     8    32 */
		struct iommu_fault_page_request prm;     /*     8    40 */
		__u8               padding2[56];         /*     8    56 */
	};                                               /*     8    56 */

	/* size: 64, cachelines: 1, members: 3 */
};
struct iommu_page_response {
	__u32                      argsz;                /*     0     4 */
	__u32                      version;              /*     4     4 */
	__u32                      flags;                /*     8     4 */
	__u32                      pasid;                /*    12     4 */
	__u32                      grpid;                /*    16     4 */
	__u32                      code;                 /*    20     4 */

	/* size: 24, cachelines: 1, members: 6 */
	/* last cacheline: 24 bytes */
};
struct iommu_inv_addr_info {
	__u32                      flags;                /*     0     4 */
	__u32                      archid;               /*     4     4 */
	__u64                      pasid;                /*     8     8 */
	__u64                      addr;                 /*    16     8 */
	__u64                      granule_size;         /*    24     8 */
	__u64                      nb_granules;          /*    32     8 */

	/* size: 40, cachelines: 1, members: 6 */
	/* last cacheline: 40 bytes */
};
struct iommu_inv_pasid_info {
	__u32                      flags;                /*     0     4 */
	__u32                      archid;               /*     4     4 */
	__u64                      pasid;                /*     8     8 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct iommu_cache_invalidate_info {
	__u32                      argsz;                /*     0     4 */
	__u32                      version;              /*     4     4 */
	__u8                       cache;                /*     8     1 */
	__u8                       granularity;          /*     9     1 */
	__u8                       padding[6];           /*    10     6 */
	union {
		struct iommu_inv_pasid_info pasid_info;  /*    16    16 */
		struct iommu_inv_addr_info addr_info;    /*    16    40 */
	} granu;                                         /*    16    40 */

	/* size: 56, cachelines: 1, members: 6 */
	/* last cacheline: 56 bytes */
};
struct iommu_gpasid_bind_data_vtd {
	__u64                      flags;                /*     0     8 */
	__u32                      pat;                  /*     8     4 */
	__u32                      emt;                  /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct iommu_gpasid_bind_data {
	__u32                      argsz;                /*     0     4 */
	__u32                      version;              /*     4     4 */
	__u32                      format;               /*     8     4 */
	__u32                      addr_width;           /*    12     4 */
	__u64                      flags;                /*    16     8 */
	__u64                      gpgd;                 /*    24     8 */
	__u64                      hpasid;               /*    32     8 */
	__u64                      gpasid;               /*    40     8 */
	__u8                       padding[8];           /*    48     8 */
	union {
		struct iommu_gpasid_bind_data_vtd vtd;   /*    56    16 */
	} vendor;                                        /*    56    16 */

	/* size: 72, cachelines: 2, members: 10 */
	/* last cacheline: 8 bytes */
};
struct iommu_pasid_smmuv3 {
	__u32                      version;              /*     0     4 */
	__u8                       s1fmt;                /*     4     1 */
	__u8                       s1dss;                /*     5     1 */
	__u8                       padding[2];           /*     6     2 */

	/* size: 8, cachelines: 1, members: 4 */
	/* last cacheline: 8 bytes */
};
struct iommu_pasid_table_config {
	__u32                      argsz;                /*     0     4 */
	__u32                      version;              /*     4     4 */
	__u64                      base_ptr;             /*     8     8 */
	__u32                      format;               /*    16     4 */
	__u8                       pasid_bits;           /*    20     1 */
	__u8                       config;               /*    21     1 */
	__u8                       padding[2];           /*    22     2 */
	union {
		struct iommu_pasid_smmuv3 smmuv3;        /*    24     8 */
	} vendor_data;                                   /*    24     8 */

	/* size: 32, cachelines: 1, members: 8 */
	/* last cacheline: 32 bytes */
};
struct iommu_domain {
	unsigned int               type;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	const struct iommu_ops  *  ops;                  /*     8     8 */
	long unsigned int          pgsize_bitmap;        /*    16     8 */
	iommu_fault_handler_t      handler;              /*    24     8 */
	void *                     handler_token;        /*    32     8 */
	struct iommu_domain_geometry geometry;           /*    40    24 */

	/* XXX last struct has 7 bytes of padding */

	/* --- cacheline 1 boundary (64 bytes) --- */
	void *                     iova_cookie;          /*    64     8 */
	struct mutex               switch_log_lock;      /*    72    32 */
	u64                        kabi_reserved1;       /*   104     8 */
	u64                        kabi_reserved2;       /*   112     8 */
	u64                        kabi_reserved3;       /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	u64                        kabi_reserved4;       /*   128     8 */

	/* size: 136, cachelines: 3, members: 12 */
	/* sum members: 132, holes: 1, sum holes: 4 */
	/* paddings: 1, sum paddings: 7 */
	/* last cacheline: 8 bytes */
};
struct iommu_domain_geometry {
	dma_addr_t                 aperture_start;       /*     0     8 */
	dma_addr_t                 aperture_end;         /*     8     8 */
	bool                       force_aperture;       /*    16     1 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 7 */
	/* last cacheline: 24 bytes */
};
struct iommu_resv_region {
	struct list_head           list;                 /*     0    16 */
	phys_addr_t                start;                /*    16     8 */
	size_t                     length;               /*    24     8 */
	int                        prot;                 /*    32     4 */
	enum iommu_resv_type       type;                 /*    36     4 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct iommu_iotlb_gather {
	long unsigned int          start;                /*     0     8 */
	long unsigned int          end;                  /*     8     8 */
	size_t                     pgsize;               /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct iommu_device {
	struct list_head           list;                 /*     0    16 */
	const struct iommu_ops  *  ops;                  /*    16     8 */
	struct fwnode_handle *     fwnode;               /*    24     8 */
	struct device *            dev;                  /*    32     8 */

	/* size: 40, cachelines: 1, members: 4 */
	/* last cacheline: 40 bytes */
};
struct iommu_sva {
	struct device *            dev;                  /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct iommu_fault_event {
	struct iommu_fault         fault;                /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct list_head           list;                 /*    64    16 */
	u64                        kabi_reserved_expire; /*    80     8 */

	/* size: 88, cachelines: 2, members: 3 */
	/* last cacheline: 24 bytes */
};
struct iommu_fault_param {
	iommu_dev_fault_handler_t  handler;              /*     0     8 */
	void *                     data;                 /*     8     8 */
	struct list_head           faults;               /*    16    16 */
	struct timer_list          kabi_reserved_timer;  /*    32    72 */
	/* --- cacheline 1 boundary (64 bytes) was 40 bytes ago --- */
	struct mutex               lock;                 /*   104    32 */

	/* size: 136, cachelines: 3, members: 5 */
	/* last cacheline: 8 bytes */
};
struct iommu_fwspec {
	const struct iommu_ops  *  ops;                  /*     0     8 */
	struct fwnode_handle *     iommu_fwnode;         /*     8     8 */
	u32                        flags;                /*    16     4 */
	unsigned int               num_ids;              /*    20     4 */
	u32                        ids[];                /*    24     0 */

	/* size: 24, cachelines: 1, members: 5 */
	/* last cacheline: 24 bytes */
};
struct pci_sysdata {
	int                        domain;               /*     0     4 */
	int                        node;                 /*     4     4 */
	struct acpi_device *       companion;            /*     8     8 */
	void *                     iommu;                /*    16     8 */
	void *                     fwnode;               /*    24     8 */
	struct pci_dev *           vmd_dev;              /*    32     8 */

	/* size: 40, cachelines: 1, members: 6 */
	/* last cacheline: 40 bytes */
};
struct cpu {
	int                        node_id;              /*     0     4 */
	int                        hotpluggable;         /*     4     4 */
	struct device              dev __attribute__((__aligned__(8))); /*     8  1000 */

	/* size: 1008, cachelines: 16, members: 3 */
	/* forced alignments: 1 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct x86_cpu {
	struct cpu                 cpu __attribute__((__aligned__(8))); /*     0  1008 */

	/* size: 1008, cachelines: 16, members: 1 */
	/* forced alignments: 1 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct debugfs_blob_wrapper {
	void *                     data;                 /*     0     8 */
	long unsigned int          size;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct setup_data_node {
	u64                        paddr;                /*     0     8 */
	u32                        type;                 /*     8     4 */
	u32                        len;                  /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct paravirt_patch_site {
	u8 *                       instr;                /*     0     8 */
	u8                         type;                 /*     8     1 */
	u8                         len;                  /*     9     1 */

	/* size: 16, cachelines: 1, members: 3 */
	/* padding: 6 */
	/* last cacheline: 16 bytes */
};
struct die_args {
	struct pt_regs *           regs;                 /*     0     8 */
	const char  *              str;                  /*     8     8 */
	long int                   err;                  /*    16     8 */
	int                        trapnr;               /*    24     4 */
	int                        signr;                /*    28     4 */

	/* size: 32, cachelines: 1, members: 5 */
	/* last cacheline: 32 bytes */
};
struct tlb_state_shared {
	bool                       is_lazy;              /*     0     1 */

	/* size: 1, cachelines: 1, members: 1 */
	/* last cacheline: 1 bytes */
};
struct smp_alt_module {
	struct module *            mod;                  /*     0     8 */
	char *                     name;                 /*     8     8 */
	const s32  *               locks;                /*    16     8 */
	const s32  *               locks_end;            /*    24     8 */
	u8 *                       text;                 /*    32     8 */
	u8 *                       text_end;             /*    40     8 */
	struct list_head           next;                 /*    48    16 */

	/* size: 64, cachelines: 1, members: 7 */
};
struct text_poke_loc {
	s32                        rel_addr;             /*     0     4 */
	s32                        disp;                 /*     4     4 */
	u8                         len;                  /*     8     1 */
	u8                         opcode;               /*     9     1 */
	const const u8             text[5];              /*    10     5 */
	u8                         old;                  /*    15     1 */

	/* size: 16, cachelines: 1, members: 6 */
	/* last cacheline: 16 bytes */
};
struct bp_patching_desc {
	struct text_poke_loc *     vec;                  /*     0     8 */
	int                        nr_entries;           /*     8     4 */
	atomic_t                   refs;                 /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct system_counterval_t {
	u64                        cycles;               /*     0     8 */
	struct clocksource *       cs;                   /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct plist_head {
	struct list_head           node_list;            /*     0    16 */

	/* size: 16, cachelines: 1, members: 1 */
	/* last cacheline: 16 bytes */
};
struct pm_qos_constraints {
	struct plist_head          list;                 /*     0    16 */
	s32                        target_value;         /*    16     4 */
	s32                        default_value;        /*    20     4 */
	s32                        no_constraint_value;  /*    24     4 */
	enum pm_qos_type           type;                 /*    28     4 */
	struct blocking_notifier_head * notifiers;       /*    32     8 */

	/* size: 40, cachelines: 1, members: 6 */
	/* last cacheline: 40 bytes */
};
struct freq_constraints {
	struct pm_qos_constraints  min_freq;             /*     0    40 */
	struct blocking_notifier_head min_freq_notifiers; /*    40    48 */
	/* --- cacheline 1 boundary (64 bytes) was 24 bytes ago --- */
	struct pm_qos_constraints  max_freq;             /*    88    40 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct blocking_notifier_head max_freq_notifiers; /*   128    48 */

	/* size: 176, cachelines: 3, members: 4 */
	/* last cacheline: 48 bytes */
};
struct freq_qos_request {
	enum freq_qos_req_type     type;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct plist_node          pnode;                /*     8    40 */
	struct freq_constraints *  qos;                  /*    48     8 */

	/* size: 56, cachelines: 1, members: 3 */
	/* sum members: 52, holes: 1, sum holes: 4 */
	/* last cacheline: 56 bytes */
};
struct cpufreq_cpuinfo {
	unsigned int               max_freq;             /*     0     4 */
	unsigned int               min_freq;             /*     4     4 */
	unsigned int               transition_latency;   /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct cpufreq_policy {
	cpumask_var_t              cpus;                 /*     0     8 */
	cpumask_var_t              related_cpus;         /*     8     8 */
	cpumask_var_t              real_cpus;            /*    16     8 */
	unsigned int               shared_type;          /*    24     4 */
	unsigned int               cpu;                  /*    28     4 */
	struct clk *               clk;                  /*    32     8 */
	struct cpufreq_cpuinfo     cpuinfo;              /*    40    12 */
	unsigned int               min;                  /*    52     4 */
	unsigned int               max;                  /*    56     4 */
	unsigned int               cur;                  /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	unsigned int               restore_freq;         /*    64     4 */
	unsigned int               suspend_freq;         /*    68     4 */
	unsigned int               policy;               /*    72     4 */
	unsigned int               last_policy;          /*    76     4 */
	struct cpufreq_governor *  governor;             /*    80     8 */
	void *                     governor_data;        /*    88     8 */
	char                       last_governor[16];    /*    96    16 */
	struct work_struct         update;               /*   112    64 */
	/* --- cacheline 2 boundary (128 bytes) was 48 bytes ago --- */
	struct freq_constraints    constraints;          /*   176   176 */
	/* --- cacheline 5 boundary (320 bytes) was 32 bytes ago --- */
	struct freq_qos_request *  min_freq_req;         /*   352     8 */
	struct freq_qos_request *  max_freq_req;         /*   360     8 */
	struct cpufreq_frequency_table * freq_table;     /*   368     8 */
	enum cpufreq_table_sorting freq_table_sorted;    /*   376     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 6 boundary (384 bytes) --- */
	struct list_head           policy_list;          /*   384    16 */
	struct kobject             kobj;                 /*   400    96 */
	/* --- cacheline 7 boundary (448 bytes) was 48 bytes ago --- */
	struct completion          kobj_unregister;      /*   496    32 */
	/* --- cacheline 8 boundary (512 bytes) was 16 bytes ago --- */
	struct rw_semaphore        rwsem;                /*   528    40 */
	bool                       fast_switch_possible; /*   568     1 */
	bool                       fast_switch_enabled;  /*   569     1 */
	bool                       strict_target;        /*   570     1 */

	/* XXX 1 byte hole, try to pack */

	unsigned int               transition_delay_us;  /*   572     4 */
	/* --- cacheline 9 boundary (576 bytes) --- */
	bool                       dvfs_possible_from_any_cpu; /*   576     1 */

	/* XXX 3 bytes hole, try to pack */

	unsigned int               cached_target_freq;   /*   580     4 */
	unsigned int               cached_resolved_idx;  /*   584     4 */
	bool                       transition_ongoing;   /*   588     1 */

	/* XXX 3 bytes hole, try to pack */

	spinlock_t                 transition_lock;      /*   592     4 */

	/* XXX 4 bytes hole, try to pack */

	wait_queue_head_t          transition_wait;      /*   600    24 */
	struct task_struct *       transition_task;      /*   624     8 */
	struct cpufreq_stats *     stats;                /*   632     8 */
	/* --- cacheline 10 boundary (640 bytes) --- */
	void *                     driver_data;          /*   640     8 */
	struct thermal_cooling_device * cdev;            /*   648     8 */
	struct notifier_block      nb_min;               /*   656    24 */

	/* XXX last struct has 4 bytes of padding */

	struct notifier_block      nb_max;               /*   680    24 */

	/* XXX last struct has 4 bytes of padding */

	/* size: 704, cachelines: 11, members: 43 */
	/* sum members: 689, holes: 5, sum holes: 15 */
	/* paddings: 2, sum paddings: 8 */
};
struct cpufreq_governor {
	char                       name[16];             /*     0    16 */
	int                        (*init)(struct cpufreq_policy *); /*    16     8 */
	void                       (*exit)(struct cpufreq_policy *); /*    24     8 */
	int                        (*start)(struct cpufreq_policy *); /*    32     8 */
	void                       (*stop)(struct cpufreq_policy *); /*    40     8 */
	void                       (*limits)(struct cpufreq_policy *); /*    48     8 */
	ssize_t                    (*show_setspeed)(struct cpufreq_policy *, char *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*store_setspeed)(struct cpufreq_policy *, unsigned int); /*    64     8 */
	struct list_head           governor_list;        /*    72    16 */
	struct module *            owner;                /*    88     8 */
	u8                         flags;                /*    96     1 */

	/* size: 104, cachelines: 2, members: 11 */
	/* padding: 7 */
	/* last cacheline: 40 bytes */
};
struct cpufreq_frequency_table {
	unsigned int               flags;                /*     0     4 */
	unsigned int               driver_data;          /*     4     4 */
	unsigned int               frequency;            /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct cpufreq_freqs {
	struct cpufreq_policy *    policy;               /*     0     8 */
	unsigned int               old;                  /*     8     4 */
	unsigned int               new;                  /*    12     4 */
	u8                         flags;                /*    16     1 */

	/* size: 24, cachelines: 1, members: 4 */
	/* padding: 7 */
	/* last cacheline: 24 bytes */
};
struct cyc2ns {
	struct cyc2ns_data         data[2];              /*     0    32 */
	seqcount_latch_t           seq;                  /*    32     4 */

	/* size: 40, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct muldiv {
	u32                        multiplier;           /*     0     4 */
	u32                        divider;              /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct freq_desc {
	bool                       use_msr_plat;         /*     0     1 */

	/* XXX 3 bytes hole, try to pack */

	struct muldiv              muldiv[16];           /*     4   128 */
	/* --- cacheline 2 boundary (128 bytes) was 4 bytes ago --- */
	u32                        freqs[16];            /*   132    64 */
	/* --- cacheline 3 boundary (192 bytes) was 4 bytes ago --- */
	u32                        mask;                 /*   196     4 */

	/* size: 200, cachelines: 4, members: 4 */
	/* sum members: 197, holes: 1, sum holes: 3 */
	/* last cacheline: 8 bytes */
};
struct dmi_strmatch {
	unsigned char              slot:7;               /*     0: 0  1 */
	unsigned char              exact_match:1;        /*     0: 7  1 */
	char                       substr[79];           /*     1    79 */

	/* size: 80, cachelines: 2, members: 3 */
	/* last cacheline: 16 bytes */
};
struct dmi_system_id {
	int                        (*callback)(const struct dmi_system_id  *); /*     0     8 */
	const char  *              ident;                /*     8     8 */
	struct dmi_strmatch        matches[4];           /*    16   320 */
	/* --- cacheline 5 boundary (320 bytes) was 16 bytes ago --- */
	void *                     driver_data;          /*   336     8 */

	/* size: 344, cachelines: 6, members: 4 */
	/* last cacheline: 24 bytes */
};
struct pdev_archdata {

	/* size: 0, cachelines: 0, members: 0 */
};
struct platform_device {
	const char  *              name;                 /*     0     8 */
	int                        id;                   /*     8     4 */
	bool                       id_auto;              /*    12     1 */

	/* XXX 3 bytes hole, try to pack */

	struct device              dev __attribute__((__aligned__(8))); /*    16  1000 */
	/* --- cacheline 15 boundary (960 bytes) was 56 bytes ago --- */
	u64                        platform_dma_mask;    /*  1016     8 */
	/* --- cacheline 16 boundary (1024 bytes) --- */
	struct device_dma_parameters dma_parms;          /*  1024    16 */
	u32                        num_resources;        /*  1040     4 */

	/* XXX 4 bytes hole, try to pack */

	struct resource *          resource;             /*  1048     8 */
	const struct platform_device_id  * id_entry;     /*  1056     8 */
	char *                     driver_override;      /*  1064     8 */
	struct mfd_cell *          mfd_cell;             /*  1072     8 */
	struct pdev_archdata       archdata;             /*  1080     0 */

	/* size: 1080, cachelines: 17, members: 12 */
	/* sum members: 1073, holes: 2, sum holes: 7 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 3 */
	/* last cacheline: 56 bytes */
} __attribute__((__aligned__(8)));
struct platform_device_id {
	char                       name[20];             /*     0    20 */

	/* XXX 4 bytes hole, try to pack */

	kernel_ulong_t             driver_data;          /*    24     8 */

	/* size: 32, cachelines: 1, members: 2 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct rtc_time {
	int                        tm_sec;               /*     0     4 */
	int                        tm_min;               /*     4     4 */
	int                        tm_hour;              /*     8     4 */
	int                        tm_mday;              /*    12     4 */
	int                        tm_mon;               /*    16     4 */
	int                        tm_year;              /*    20     4 */
	int                        tm_wday;              /*    24     4 */
	int                        tm_yday;              /*    28     4 */
	int                        tm_isdst;             /*    32     4 */

	/* size: 36, cachelines: 1, members: 9 */
	/* last cacheline: 36 bytes */
};
struct pnp_device_id {
	__u8                       id[8];                /*     0     8 */
	kernel_ulong_t             driver_data;          /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct pnp_card_device_id {
	__u8                       id[8];                /*     0     8 */
	kernel_ulong_t             driver_data;          /*     8     8 */
	struct {
		__u8               id[8];                /*    16     8 */
	} devs[8]; /*    16    64 */

	/* size: 80, cachelines: 2, members: 3 */
	/* last cacheline: 16 bytes */
};
struct acpi_table_header {
	char                       signature[4];         /*     0     4 */
	u32                        length;               /*     4     4 */
	u8                         revision;             /*     8     1 */
	u8                         checksum;             /*     9     1 */
	char                       oem_id[6];            /*    10     6 */
	char                       oem_table_id[8];      /*    16     8 */
	u32                        oem_revision;         /*    24     4 */
	char                       asl_compiler_id[4];   /*    28     4 */
	u32                        asl_compiler_revision; /*    32     4 */

	/* size: 36, cachelines: 1, members: 9 */
	/* last cacheline: 36 bytes */
};
struct acpi_generic_address {
	u8                         space_id;             /*     0     1 */
	u8                         bit_width;            /*     1     1 */
	u8                         bit_offset;           /*     2     1 */
	u8                         access_width;         /*     3     1 */
	u64                        address;              /*     4     8 */

	/* size: 12, cachelines: 1, members: 5 */
	/* last cacheline: 12 bytes */
} __attribute__((__packed__));
struct acpi_table_fadt {
	struct acpi_table_header   header;               /*     0    36 */
	u32                        facs;                 /*    36     4 */
	u32                        dsdt;                 /*    40     4 */
	u8                         model;                /*    44     1 */
	u8                         preferred_profile;    /*    45     1 */
	u16                        sci_interrupt;        /*    46     2 */
	u32                        smi_command;          /*    48     4 */
	u8                         acpi_enable;          /*    52     1 */
	u8                         acpi_disable;         /*    53     1 */
	u8                         s4_bios_request;      /*    54     1 */
	u8                         pstate_control;       /*    55     1 */
	u32                        pm1a_event_block;     /*    56     4 */
	u32                        pm1b_event_block;     /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u32                        pm1a_control_block;   /*    64     4 */
	u32                        pm1b_control_block;   /*    68     4 */
	u32                        pm2_control_block;    /*    72     4 */
	u32                        pm_timer_block;       /*    76     4 */
	u32                        gpe0_block;           /*    80     4 */
	u32                        gpe1_block;           /*    84     4 */
	u8                         pm1_event_length;     /*    88     1 */
	u8                         pm1_control_length;   /*    89     1 */
	u8                         pm2_control_length;   /*    90     1 */
	u8                         pm_timer_length;      /*    91     1 */
	u8                         gpe0_block_length;    /*    92     1 */
	u8                         gpe1_block_length;    /*    93     1 */
	u8                         gpe1_base;            /*    94     1 */
	u8                         cst_control;          /*    95     1 */
	u16                        c2_latency;           /*    96     2 */
	u16                        c3_latency;           /*    98     2 */
	u16                        flush_size;           /*   100     2 */
	u16                        flush_stride;         /*   102     2 */
	u8                         duty_offset;          /*   104     1 */
	u8                         duty_width;           /*   105     1 */
	u8                         day_alarm;            /*   106     1 */
	u8                         month_alarm;          /*   107     1 */
	u8                         century;              /*   108     1 */
	u16                        boot_flags;           /*   109     2 */
	u8                         reserved;             /*   111     1 */
	u32                        flags;                /*   112     4 */
	struct acpi_generic_address reset_register;      /*   116    12 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	u8                         reset_value;          /*   128     1 */
	u16                        arm_boot_flags;       /*   129     2 */
	u8                         minor_revision;       /*   131     1 */
	u64                        Xfacs;                /*   132     8 */
	u64                        Xdsdt;                /*   140     8 */
	struct acpi_generic_address xpm1a_event_block;   /*   148    12 */
	struct acpi_generic_address xpm1b_event_block;   /*   160    12 */
	struct acpi_generic_address xpm1a_control_block; /*   172    12 */
	struct acpi_generic_address xpm1b_control_block; /*   184    12 */
	/* --- cacheline 3 boundary (192 bytes) was 4 bytes ago --- */
	struct acpi_generic_address xpm2_control_block;  /*   196    12 */
	struct acpi_generic_address xpm_timer_block;     /*   208    12 */
	struct acpi_generic_address xgpe0_block;         /*   220    12 */
	struct acpi_generic_address xgpe1_block;         /*   232    12 */
	struct acpi_generic_address sleep_control;       /*   244    12 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	struct acpi_generic_address sleep_status;        /*   256    12 */
	u64                        hypervisor_id;        /*   268     8 */

	/* size: 276, cachelines: 5, members: 56 */
	/* last cacheline: 20 bytes */
} __attribute__((__packed__));
struct pnp_card {
	struct device              dev __attribute__((__aligned__(8))); /*     0  1000 */
	/* --- cacheline 15 boundary (960 bytes) was 40 bytes ago --- */
	unsigned char              number;               /*  1000     1 */

	/* XXX 7 bytes hole, try to pack */

	struct list_head           global_list;          /*  1008    16 */
	/* --- cacheline 16 boundary (1024 bytes) --- */
	struct list_head           protocol_list;        /*  1024    16 */
	struct list_head           devices;              /*  1040    16 */
	struct pnp_protocol *      protocol;             /*  1056     8 */
	struct pnp_id *            id;                   /*  1064     8 */
	char                       name[50];             /*  1072    50 */
	/* --- cacheline 17 boundary (1088 bytes) was 34 bytes ago --- */
	unsigned char              pnpver;               /*  1122     1 */
	unsigned char              productver;           /*  1123     1 */
	unsigned int               serial;               /*  1124     4 */
	unsigned char              checksum;             /*  1128     1 */

	/* XXX 7 bytes hole, try to pack */

	struct proc_dir_entry *    procdir;              /*  1136     8 */

	/* size: 1144, cachelines: 18, members: 13 */
	/* sum members: 1130, holes: 2, sum holes: 14 */
	/* forced alignments: 1 */
	/* last cacheline: 56 bytes */
} __attribute__((__aligned__(8)));
struct pnp_protocol {
	struct list_head           protocol_list;        /*     0    16 */
	char *                     name;                 /*    16     8 */
	int                        (*get)(struct pnp_dev *); /*    24     8 */
	int                        (*set)(struct pnp_dev *); /*    32     8 */
	int                        (*disable)(struct pnp_dev *); /*    40     8 */
	bool                       (*can_wakeup)(struct pnp_dev *); /*    48     8 */
	int                        (*suspend)(struct pnp_dev *, pm_message_t); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*resume)(struct pnp_dev *); /*    64     8 */
	unsigned char              number;               /*    72     1 */

	/* XXX 7 bytes hole, try to pack */

	struct device              dev __attribute__((__aligned__(8))); /*    80  1000 */
	/* --- cacheline 16 boundary (1024 bytes) was 56 bytes ago --- */
	struct list_head           cards;                /*  1080    16 */
	/* --- cacheline 17 boundary (1088 bytes) was 8 bytes ago --- */
	struct list_head           devices;              /*  1096    16 */

	/* size: 1112, cachelines: 18, members: 12 */
	/* sum members: 1105, holes: 1, sum holes: 7 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 7 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct pnp_id {
	char                       id[8];                /*     0     8 */
	struct pnp_id *            next;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct pnp_card_link {
	struct pnp_card *          card;                 /*     0     8 */
	struct pnp_card_driver *   driver;               /*     8     8 */
	void *                     driver_data;          /*    16     8 */
	pm_message_t               pm_state;             /*    24     4 */

	/* size: 32, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct pnp_card_driver {
	struct list_head           global_list;          /*     0    16 */
	char *                     name;                 /*    16     8 */
	const struct pnp_card_device_id  * id_table;     /*    24     8 */
	unsigned int               flags;                /*    32     4 */

	/* XXX 4 bytes hole, try to pack */

	int                        (*probe)(struct pnp_card_link *, const struct pnp_card_device_id  *); /*    40     8 */
	void                       (*remove)(struct pnp_card_link *); /*    48     8 */
	int                        (*suspend)(struct pnp_card_link *, pm_message_t); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*resume)(struct pnp_card_link *); /*    64     8 */
	struct pnp_driver          link;                 /*    72   256 */

	/* size: 328, cachelines: 6, members: 9 */
	/* sum members: 324, holes: 1, sum holes: 4 */
	/* last cacheline: 8 bytes */
};
struct pnp_dev {
	struct device              dev __attribute__((__aligned__(8))); /*     0  1000 */
	/* --- cacheline 15 boundary (960 bytes) was 40 bytes ago --- */
	u64                        dma_mask;             /*  1000     8 */
	unsigned int               number;               /*  1008     4 */
	int                        status;               /*  1012     4 */
	struct list_head           global_list;          /*  1016    16 */
	/* --- cacheline 16 boundary (1024 bytes) was 8 bytes ago --- */
	struct list_head           protocol_list;        /*  1032    16 */
	struct list_head           card_list;            /*  1048    16 */
	struct list_head           rdev_list;            /*  1064    16 */
	struct pnp_protocol *      protocol;             /*  1080     8 */
	/* --- cacheline 17 boundary (1088 bytes) --- */
	struct pnp_card *          card;                 /*  1088     8 */
	struct pnp_driver *        driver;               /*  1096     8 */
	struct pnp_card_link *     card_link;            /*  1104     8 */
	struct pnp_id *            id;                   /*  1112     8 */
	int                        active;               /*  1120     4 */
	int                        capabilities;         /*  1124     4 */
	unsigned int               num_dependent_sets;   /*  1128     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           resources;            /*  1136    16 */
	/* --- cacheline 18 boundary (1152 bytes) --- */
	struct list_head           options;              /*  1152    16 */
	char                       name[50];             /*  1168    50 */

	/* XXX 2 bytes hole, try to pack */

	/* --- cacheline 19 boundary (1216 bytes) was 4 bytes ago --- */
	int                        flags;                /*  1220     4 */
	struct proc_dir_entry *    procent;              /*  1224     8 */
	void *                     data;                 /*  1232     8 */

	/* size: 1240, cachelines: 20, members: 22 */
	/* sum members: 1234, holes: 2, sum holes: 6 */
	/* forced alignments: 1 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct pnp_driver {
	const char  *              name;                 /*     0     8 */
	const struct pnp_device_id  * id_table;          /*     8     8 */
	unsigned int               flags;                /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	int                        (*probe)(struct pnp_dev *, const struct pnp_device_id  *); /*    24     8 */
	void                       (*remove)(struct pnp_dev *); /*    32     8 */
	void                       (*shutdown)(struct pnp_dev *); /*    40     8 */
	int                        (*suspend)(struct pnp_dev *, pm_message_t); /*    48     8 */
	int                        (*resume)(struct pnp_dev *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct device_driver       driver;               /*    64   192 */

	/* size: 256, cachelines: 4, members: 9 */
	/* sum members: 252, holes: 1, sum holes: 4 */
};
struct dev_pm_qos {
	struct pm_qos_constraints  resume_latency;       /*     0    40 */
	struct pm_qos_constraints  latency_tolerance;    /*    40    40 */
	/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
	struct freq_constraints    freq;                 /*    80   176 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	struct pm_qos_flags        flags;                /*   256    24 */

	/* XXX last struct has 4 bytes of padding */

	struct dev_pm_qos_request * resume_latency_req;  /*   280     8 */
	struct dev_pm_qos_request * latency_tolerance_req; /*   288     8 */
	struct dev_pm_qos_request * flags_req;           /*   296     8 */
	u64                        kabi_reserved1;       /*   304     8 */
	u64                        kabi_reserved2;       /*   312     8 */

	/* size: 320, cachelines: 5, members: 9 */
	/* paddings: 1, sum paddings: 4 */
};
struct pm_qos_flags_request {
	struct list_head           node;                 /*     0    16 */
	s32                        flags;                /*    16     4 */

	/* size: 24, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct pm_qos_flags {
	struct list_head           list;                 /*     0    16 */
	s32                        effective_flags;      /*    16     4 */

	/* size: 24, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct dev_pm_qos_request {
	enum dev_pm_qos_req_type   type;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	union {
		struct plist_node  pnode;                /*     8    40 */
		struct pm_qos_flags_request flr;         /*     8    24 */
		struct freq_qos_request freq;            /*     8    56 */
	} data;                                          /*     8    56 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct device *            dev;                  /*    64     8 */
	u64                        kabi_reserved1;       /*    72     8 */
	u64                        kabi_reserved2;       /*    80     8 */

	/* size: 88, cachelines: 2, members: 5 */
	/* sum members: 84, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct inactive_task_frame {
	long unsigned int          r15;                  /*     0     8 */
	long unsigned int          r14;                  /*     8     8 */
	long unsigned int          r13;                  /*    16     8 */
	long unsigned int          r12;                  /*    24     8 */
	long unsigned int          bx;                   /*    32     8 */
	long unsigned int          bp;                   /*    40     8 */
	long unsigned int          ret_addr;             /*    48     8 */

	/* size: 56, cachelines: 1, members: 7 */
	/* last cacheline: 56 bytes */
};
struct fork_frame {
	struct inactive_task_frame frame;                /*     0    56 */
	struct pt_regs             regs;                 /*    56   168 */

	/* size: 224, cachelines: 4, members: 2 */
	/* last cacheline: 32 bytes */
};
struct ssb_state {
	struct ssb_state *         shared_state;         /*     0     8 */
	raw_spinlock_t             lock;                 /*     8     4 */
	unsigned int               disable_state;        /*    12     4 */
	long unsigned int          local_state;          /*    16     8 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct fpu_state_config {
	unsigned int               max_size;             /*     0     4 */
	unsigned int               default_size;         /*     4     4 */
	u64                        max_features;         /*     8     8 */
	u64                        default_features;     /*    16     8 */
	u64                        legacy_features;      /*    24     8 */

	/* size: 32, cachelines: 1, members: 5 */
	/* last cacheline: 32 bytes */
};
struct pkru_state {
	u32                        pkru;                 /*     0     4 */
	u32                        pad;                  /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct fpu_guest {
	u64                        xfeatures;            /*     0     8 */
	u64                        perm;                 /*     8     8 */
	u64                        xfd_err;              /*    16     8 */
	unsigned int               uabi_size;            /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	struct fpstate *           fpstate;              /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* sum members: 36, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct membuf {
	void *                     p;                    /*     0     8 */
	size_t                     left;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct trace_event_raw_x86_fpu {
	struct trace_entry         ent;                  /*     0     8 */
	struct fpu *               fpu;                  /*     8     8 */
	bool                       load_fpu;             /*    16     1 */

	/* XXX 7 bytes hole, try to pack */

	u64                        xfeatures;            /*    24     8 */
	u64                        xcomp_bv;             /*    32     8 */
	char                       __data[];             /*    40     0 */

	/* size: 40, cachelines: 1, members: 6 */
	/* sum members: 33, holes: 1, sum holes: 7 */
	/* last cacheline: 40 bytes */
};
struct trace_event_data_offsets_x86_fpu {

	/* size: 0, cachelines: 0, members: 0 */
};
struct _fpreg {
	__u16                      significand[4];       /*     0     8 */
	__u16                      exponent;             /*     8     2 */

	/* size: 10, cachelines: 1, members: 2 */
	/* last cacheline: 10 bytes */
};
struct _fpxreg {
	__u16                      significand[4];       /*     0     8 */
	__u16                      exponent;             /*     8     2 */
	__u16                      padding[3];           /*    10     6 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct user_i387_ia32_struct {
	u32                        cwd;                  /*     0     4 */
	u32                        swd;                  /*     4     4 */
	u32                        twd;                  /*     8     4 */
	u32                        fip;                  /*    12     4 */
	u32                        fcs;                  /*    16     4 */
	u32                        foo;                  /*    20     4 */
	u32                        fos;                  /*    24     4 */
	u32                        st_space[20];         /*    28    80 */

	/* size: 108, cachelines: 2, members: 8 */
	/* last cacheline: 44 bytes */
};
struct user_regset {
	user_regset_get2_fn *      regset_get;           /*     0     8 */
	user_regset_set_fn *       set;                  /*     8     8 */
	user_regset_active_fn *    active;               /*    16     8 */
	user_regset_writeback_fn * writeback;            /*    24     8 */
	unsigned int               n;                    /*    32     4 */
	unsigned int               size;                 /*    36     4 */
	unsigned int               align;                /*    40     4 */
	unsigned int               bias;                 /*    44     4 */
	unsigned int               core_note_type;       /*    48     4 */

	/* size: 56, cachelines: 1, members: 9 */
	/* padding: 4 */
	/* last cacheline: 56 bytes */
};
struct _fpx_sw_bytes {
	__u32                      magic1;               /*     0     4 */
	__u32                      extended_size;        /*     4     4 */
	__u64                      xfeatures;            /*     8     8 */
	__u32                      xstate_size;          /*    16     4 */
	__u32                      padding[7];           /*    20    28 */

	/* size: 48, cachelines: 1, members: 5 */
	/* last cacheline: 48 bytes */
};
struct _xmmreg {
	__u32                      element[4];           /*     0    16 */

	/* size: 16, cachelines: 1, members: 1 */
	/* last cacheline: 16 bytes */
};
struct _fpstate_32 {
	__u32                      cw;                   /*     0     4 */
	__u32                      sw;                   /*     4     4 */
	__u32                      tag;                  /*     8     4 */
	__u32                      ipoff;                /*    12     4 */
	__u32                      cssel;                /*    16     4 */
	__u32                      dataoff;              /*    20     4 */
	__u32                      datasel;              /*    24     4 */
	struct _fpreg              _st[8];               /*    28    80 */
	/* --- cacheline 1 boundary (64 bytes) was 44 bytes ago --- */
	__u16                      status;               /*   108     2 */
	__u16                      magic;                /*   110     2 */
	__u32                      _fxsr_env[6];         /*   112    24 */
	/* --- cacheline 2 boundary (128 bytes) was 8 bytes ago --- */
	__u32                      mxcsr;                /*   136     4 */
	__u32                      reserved;             /*   140     4 */
	struct _fpxreg             _fxsr_st[8];          /*   144   128 */
	/* --- cacheline 4 boundary (256 bytes) was 16 bytes ago --- */
	struct _xmmreg             _xmm[8];              /*   272   128 */
	/* --- cacheline 6 boundary (384 bytes) was 16 bytes ago --- */
	union {
		__u32              padding1[44];         /*   400   176 */
		__u32              padding[44];          /*   400   176 */
	};                                               /*   400   176 */
	/* --- cacheline 9 boundary (576 bytes) --- */
	union {
		__u32              padding2[12];         /*   576    48 */
		struct _fpx_sw_bytes sw_reserved;        /*   576    48 */
	};                                               /*   576    48 */

	/* size: 624, cachelines: 10, members: 17 */
	/* last cacheline: 48 bytes */
};
struct user_regset_view {
	const char  *              name;                 /*     0     8 */
	const struct user_regset  * regsets;             /*     8     8 */
	unsigned int               n;                    /*    16     4 */
	u32                        e_flags;              /*    20     4 */
	u16                        e_machine;            /*    24     2 */
	u8                         ei_osabi;             /*    26     1 */

	/* size: 32, cachelines: 1, members: 6 */
	/* padding: 5 */
	/* last cacheline: 32 bytes */
};
struct pt_regs_offset {
	const char  *              name;                 /*     0     8 */
	int                        offset;               /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct tboot_mac_region {
	u64                        start;                /*     0     8 */
	u32                        size;                 /*     8     4 */

	/* size: 12, cachelines: 1, members: 2 */
	/* last cacheline: 12 bytes */
} __attribute__((__packed__));
struct tboot_acpi_generic_address {
	u8                         space_id;             /*     0     1 */
	u8                         bit_width;            /*     1     1 */
	u8                         bit_offset;           /*     2     1 */
	u8                         access_width;         /*     3     1 */
	u64                        address;              /*     4     8 */

	/* size: 12, cachelines: 1, members: 5 */
	/* last cacheline: 12 bytes */
} __attribute__((__packed__));
struct tboot_acpi_sleep_info {
	struct tboot_acpi_generic_address pm1a_cnt_blk;  /*     0    12 */
	struct tboot_acpi_generic_address pm1b_cnt_blk;  /*    12    12 */
	struct tboot_acpi_generic_address pm1a_evt_blk;  /*    24    12 */
	struct tboot_acpi_generic_address pm1b_evt_blk;  /*    36    12 */
	u16                        pm1a_cnt_val;         /*    48     2 */
	u16                        pm1b_cnt_val;         /*    50     2 */
	u64                        wakeup_vector;        /*    52     8 */
	u32                        vector_width;         /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u64                        kernel_s3_resume_vector; /*    64     8 */

	/* size: 72, cachelines: 2, members: 9 */
	/* last cacheline: 8 bytes */
} __attribute__((__packed__));
struct tboot {
	u8                         uuid[16];             /*     0    16 */
	u32                        version;              /*    16     4 */
	u32                        log_addr;             /*    20     4 */
	u32                        shutdown_entry;       /*    24     4 */
	u32                        shutdown_type;        /*    28     4 */
	struct tboot_acpi_sleep_info acpi_sinfo;         /*    32    72 */
	/* --- cacheline 1 boundary (64 bytes) was 40 bytes ago --- */
	u32                        tboot_base;           /*   104     4 */
	u32                        tboot_size;           /*   108     4 */
	u8                         num_mac_regions;      /*   112     1 */
	struct tboot_mac_region    mac_regions[32];      /*   113   384 */
	/* --- cacheline 7 boundary (448 bytes) was 49 bytes ago --- */
	u8                         s3_key[64];           /*   497    64 */
	/* --- cacheline 8 boundary (512 bytes) was 49 bytes ago --- */
	u8                         reserved_align[3];    /*   561     3 */
	u32                        num_in_wfs;           /*   564     4 */

	/* size: 568, cachelines: 9, members: 13 */
	/* last cacheline: 56 bytes */
} __attribute__((__packed__));
struct sha1_hash {
	u8                         hash[20];             /*     0    20 */

	/* size: 20, cachelines: 1, members: 1 */
	/* last cacheline: 20 bytes */
};
struct sinit_mle_data {
	u32                        version;              /*     0     4 */
	struct sha1_hash           bios_acm_id;          /*     4    20 */
	u32                        edx_senter_flags;     /*    24     4 */
	u64                        mseg_valid;           /*    28     8 */
	struct sha1_hash           sinit_hash;           /*    36    20 */
	struct sha1_hash           mle_hash;             /*    56    20 */
	/* --- cacheline 1 boundary (64 bytes) was 12 bytes ago --- */
	struct sha1_hash           stm_hash;             /*    76    20 */
	struct sha1_hash           lcp_policy_hash;      /*    96    20 */
	u32                        lcp_policy_control;   /*   116     4 */
	u32                        rlp_wakeup_addr;      /*   120     4 */
	u32                        reserved;             /*   124     4 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	u32                        num_mdrs;             /*   128     4 */
	u32                        mdrs_off;             /*   132     4 */
	u32                        num_vtd_dmars;        /*   136     4 */
	u32                        vtd_dmars_off;        /*   140     4 */

	/* size: 144, cachelines: 3, members: 15 */
	/* last cacheline: 16 bytes */
} __attribute__((__packed__));
struct stack_frame_user {
	const void  *              next_fp;              /*     0     8 */
	long unsigned int          ret_addr;             /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct cacheinfo {
	unsigned int               id;                   /*     0     4 */
	enum cache_type            type;                 /*     4     4 */
	unsigned int               level;                /*     8     4 */
	unsigned int               coherency_line_size;  /*    12     4 */
	unsigned int               number_of_sets;       /*    16     4 */
	unsigned int               ways_of_associativity; /*    20     4 */
	unsigned int               physical_line_partition; /*    24     4 */
	unsigned int               size;                 /*    28     4 */
	cpumask_t                  shared_cpu_map;       /*    32  1024 */
	/* --- cacheline 16 boundary (1024 bytes) was 32 bytes ago --- */
	unsigned int               attributes;           /*  1056     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     fw_token;             /*  1064     8 */
	bool                       disable_sysfs;        /*  1072     1 */

	/* XXX 7 bytes hole, try to pack */

	void *                     priv;                 /*  1080     8 */

	/* size: 1088, cachelines: 17, members: 13 */
	/* sum members: 1077, holes: 2, sum holes: 11 */
};
struct cpu_cacheinfo {
	struct cacheinfo *         info_list;            /*     0     8 */
	unsigned int               num_levels;           /*     8     4 */
	unsigned int               num_leaves;           /*    12     4 */
	bool                       cpu_map_populated;    /*    16     1 */

	/* size: 24, cachelines: 1, members: 4 */
	/* padding: 7 */
	/* last cacheline: 24 bytes */
};
struct amd_l3_cache {
	unsigned int               indices;              /*     0     4 */
	u8                         subcaches[4];         /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct threshold_block {
	unsigned int               block;                /*     0     4 */
	unsigned int               bank;                 /*     4     4 */
	unsigned int               cpu;                  /*     8     4 */
	u32                        address;              /*    12     4 */
	u16                        interrupt_enable;     /*    16     2 */
	bool                       interrupt_capable;    /*    18     1 */

	/* XXX 1 byte hole, try to pack */

	u16                        threshold_limit;      /*    20     2 */

	/* XXX 2 bytes hole, try to pack */

	struct kobject             kobj;                 /*    24    96 */
	/* --- cacheline 1 boundary (64 bytes) was 56 bytes ago --- */
	struct list_head           miscj;                /*   120    16 */

	/* size: 136, cachelines: 3, members: 9 */
	/* sum members: 133, holes: 2, sum holes: 3 */
	/* last cacheline: 8 bytes */
};
struct threshold_bank {
	struct kobject *           kobj;                 /*     0     8 */
	struct threshold_block *   blocks;               /*     8     8 */
	refcount_t                 cpus;                 /*    16     4 */
	unsigned int               shared;               /*    20     4 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct amd_northbridge {
	struct pci_dev *           root;                 /*     0     8 */
	struct pci_dev *           misc;                 /*     8     8 */
	struct pci_dev *           link;                 /*    16     8 */
	struct amd_l3_cache        l3_cache;             /*    24     8 */
	struct threshold_bank *    bank4;                /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct _cache_table {
	unsigned char              descriptor;           /*     0     1 */
	char                       cache_type;           /*     1     1 */
	short int                  size;                 /*     2     2 */

	/* size: 4, cachelines: 1, members: 3 */
	/* last cacheline: 4 bytes */
};
union _cpuid4_leaf_eax {
	struct {
		enum _cache_type   type:5;             /*     0: 0  4 */
		unsigned int       level:3;            /*     0: 5  4 */
		unsigned int       is_self_initializing:1; /*     0: 8  4 */
		unsigned int       is_fully_associative:1; /*     0: 9  4 */
		unsigned int       reserved:4;         /*     0:10  4 */
		unsigned int       num_threads_sharing:12; /*     0:14  4 */
		unsigned int       num_cores_on_die:6; /*     0:26  4 */
	} split;                                       /*     0     4 */
	u32                        full;               /*     0     4 */
};
union _cpuid4_leaf_ebx {
	struct {
		unsigned int       coherency_line_size:12; /*     0: 0  4 */
		unsigned int       physical_line_partition:10; /*     0:12  4 */
		unsigned int       ways_of_associativity:10; /*     0:22  4 */
	} split;                                       /*     0     4 */
	u32                        full;               /*     0     4 */
};
union _cpuid4_leaf_ecx {
	struct {
		unsigned int       number_of_sets:32;  /*     0: 0  4 */
	} split;                                       /*     0     4 */
	u32                        full;               /*     0     4 */
};
struct _cpuid4_info_regs {
	union _cpuid4_leaf_eax     eax;                  /*     0     4 */
	union _cpuid4_leaf_ebx     ebx;                  /*     4     4 */
	union _cpuid4_leaf_ecx     ecx;                  /*     8     4 */
	unsigned int               id;                   /*    12     4 */
	long unsigned int          size;                 /*    16     8 */
	struct amd_northbridge *   nb;                   /*    24     8 */

	/* size: 32, cachelines: 1, members: 6 */
	/* last cacheline: 32 bytes */
};
union l1_cache {
	struct {
		unsigned int       line_size:8;        /*     0: 0  4 */
		unsigned int       lines_per_tag:8;    /*     0: 8  4 */
		unsigned int       assoc:8;            /*     0:16  4 */
		unsigned int       size_in_kb:8;       /*     0:24  4 */
	};                                             /*     0     4 */
	unsigned int               val;                /*     0     4 */
};
union l2_cache {
	struct {
		unsigned int       line_size:8;        /*     0: 0  4 */
		unsigned int       lines_per_tag:4;    /*     0: 8  4 */
		unsigned int       assoc:4;            /*     0:12  4 */
		unsigned int       size_in_kb:16;      /*     0:16  4 */
	};                                             /*     0     4 */
	unsigned int               val;                /*     0     4 */
};
union l3_cache {
	struct {
		unsigned int       line_size:8;        /*     0: 0  4 */
		unsigned int       lines_per_tag:4;    /*     0: 8  4 */
		unsigned int       assoc:4;            /*     0:12  4 */
		unsigned int       res:2;              /*     0:16  4 */
		unsigned int       size_encoded:14;    /*     0:18  4 */
	};                                             /*     0     4 */
	unsigned int               val;                /*     0     4 */
};
struct cpuid_bit {
	u16                        feature;              /*     0     2 */
	u8                         reg;                  /*     2     1 */
	u8                         bit;                  /*     3     1 */
	u32                        level;                /*     4     4 */
	u32                        sub_leaf;             /*     8     4 */

	/* size: 12, cachelines: 1, members: 5 */
	/* last cacheline: 12 bytes */
};
struct kgdb_arch {
	unsigned char              gdb_bpt_instr[1];     /*     0     1 */

	/* XXX 7 bytes hole, try to pack */

	long unsigned int          flags;                /*     8     8 */
	int                        (*set_breakpoint)(long unsigned int, char *); /*    16     8 */
	int                        (*remove_breakpoint)(long unsigned int, char *); /*    24     8 */
	int                        (*set_hw_breakpoint)(long unsigned int, int, enum kgdb_bptype); /*    32     8 */
	int                        (*remove_hw_breakpoint)(long unsigned int, int, enum kgdb_bptype); /*    40     8 */
	void                       (*disable_hw_break)(struct pt_regs *); /*    48     8 */
	void                       (*remove_all_hw_break)(void); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	void                       (*correct_hw_break)(void); /*    64     8 */
	void                       (*enable_nmi)(bool);  /*    72     8 */

	/* size: 80, cachelines: 2, members: 10 */
	/* sum members: 73, holes: 1, sum holes: 7 */
	/* last cacheline: 16 bytes */
};
struct cpu_dev {
	const char  *              c_vendor;             /*     0     8 */
	const char  *              c_ident[2];           /*     8    16 */
	void                       (*c_early_init)(struct cpuinfo_x86 *); /*    24     8 */
	void                       (*c_bsp_init)(struct cpuinfo_x86 *); /*    32     8 */
	void                       (*c_init)(struct cpuinfo_x86 *); /*    40     8 */
	void                       (*c_identify)(struct cpuinfo_x86 *); /*    48     8 */
	void                       (*c_detect_tlb)(struct cpuinfo_x86 *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        c_x86_vendor;         /*    64     4 */

	/* size: 72, cachelines: 2, members: 8 */
	/* padding: 4 */
	/* last cacheline: 8 bytes */
};
struct cpuid_dependent_feature {
	u32                        feature;              /*     0     4 */
	u32                        level;                /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct aperfmperf_sample {
	unsigned int               khz;                  /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	ktime_t                    time;                 /*     8     8 */
	u64                        aperf;                /*    16     8 */
	u64                        mperf;                /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct cpuid_dep {
	unsigned int               feature;              /*     0     4 */
	unsigned int               depends;              /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct _tlb_table {
	unsigned char              descriptor;           /*     0     1 */
	char                       tlb_type;             /*     1     1 */

	/* XXX 2 bytes hole, try to pack */

	unsigned int               entries;              /*     4     4 */
	char                       info[128];            /*     8   128 */

	/* size: 136, cachelines: 3, members: 4 */
	/* sum members: 134, holes: 1, sum holes: 2 */
	/* last cacheline: 8 bytes */
};
struct sku_microcode {
	u8                         model;                /*     0     1 */
	u8                         stepping;             /*     1     1 */

	/* XXX 2 bytes hole, try to pack */

	u32                        microcode;            /*     4     4 */

	/* size: 8, cachelines: 1, members: 3 */
	/* sum members: 6, holes: 1, sum holes: 2 */
	/* last cacheline: 8 bytes */
};
struct cpuid_regs {
	u32                        eax;                  /*     0     4 */
	u32                        ebx;                  /*     4     4 */
	u32                        ecx;                  /*     8     4 */
	u32                        edx;                  /*    12     4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct mce {
	__u64                      status;               /*     0     8 */
	__u64                      misc;                 /*     8     8 */
	__u64                      addr;                 /*    16     8 */
	__u64                      mcgstatus;            /*    24     8 */
	__u64                      ip;                   /*    32     8 */
	__u64                      tsc;                  /*    40     8 */
	__u64                      time;                 /*    48     8 */
	__u8                       cpuvendor;            /*    56     1 */
	__u8                       inject_flags;         /*    57     1 */
	__u8                       severity;             /*    58     1 */
	__u8                       pad;                  /*    59     1 */
	__u32                      cpuid;                /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	__u8                       cs;                   /*    64     1 */
	__u8                       bank;                 /*    65     1 */
	__u8                       cpu;                  /*    66     1 */
	__u8                       finished;             /*    67     1 */
	__u32                      extcpu;               /*    68     4 */
	__u32                      socketid;             /*    72     4 */
	__u32                      apicid;               /*    76     4 */
	__u64                      mcgcap;               /*    80     8 */
	__u64                      synd;                 /*    88     8 */
	__u64                      ipid;                 /*    96     8 */
	__u64                      ppin;                 /*   104     8 */
	__u32                      microcode;            /*   112     4 */

	/* XXX 4 bytes hole, try to pack */

	__u64                      kflags;               /*   120     8 */

	/* size: 128, cachelines: 2, members: 25 */
	/* sum members: 124, holes: 1, sum holes: 4 */
};
struct mce_evt_llist {
	struct llist_node          llnode;               /*     0     8 */
	struct mce                 mce;                  /*     8   128 */

	/* size: 136, cachelines: 3, members: 2 */
	/* last cacheline: 8 bytes */
};
struct mca_config {
	bool                       dont_log_ce;          /*     0     1 */
	bool                       cmci_disabled;        /*     1     1 */
	bool                       ignore_ce;            /*     2     1 */
	bool                       print_all;            /*     3     1 */

	/* Bitfield combined with previous fields */

	__u64                      lmce_disabled:1;      /*     0:32  8 */
	__u64                      disabled:1;           /*     0:33  8 */
	__u64                      ser:1;                /*     0:34  8 */
	__u64                      recovery:1;           /*     0:35  8 */
	__u64                      bios_cmci_threshold:1; /*     0:36  8 */

	/* XXX 27 bits hole, try to pack */

	/* Force alignment to the next boundary: */
	__u64                      :0;

	__u64                      __reserved:59;        /*     8: 0  8 */

	/* XXX 5 bits hole, try to pack */

	s8                         bootlog;              /*    16     1 */

	/* XXX 3 bytes hole, try to pack */

	int                        tolerant;             /*    20     4 */
	int                        monarch_timeout;      /*    24     4 */
	int                        panic_timeout;        /*    28     4 */
	u32                        rip_msr;              /*    32     4 */

	/* size: 40, cachelines: 1, members: 15 */
	/* sum members: 21, holes: 1, sum holes: 3 */
	/* sum bitfield members: 64 bits, bit holes: 2, sum bit holes: 32 bits */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct mce_vendor_flags {
	__u64                      overflow_recov:1;     /*     0: 0  8 */
	__u64                      succor:1;             /*     0: 1  8 */
	__u64                      smca:1;               /*     0: 2  8 */
	__u64                      amd_threshold:1;      /*     0: 3  8 */
	__u64                      __reserved_0:60;      /*     0: 4  8 */

	/* size: 8, cachelines: 1, members: 5 */
	/* last cacheline: 8 bytes */
};
struct mca_msr_regs {
	u32                        (*ctl)(int);          /*     0     8 */
	u32                        (*status)(int);       /*     8     8 */
	u32                        (*addr)(int);         /*    16     8 */
	u32                        (*misc)(int);         /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct trace_event_raw_mce_record {
	struct trace_entry         ent;                  /*     0     8 */
	u64                        mcgcap;               /*     8     8 */
	u64                        mcgstatus;            /*    16     8 */
	u64                        status;               /*    24     8 */
	u64                        addr;                 /*    32     8 */
	u64                        misc;                 /*    40     8 */
	u64                        synd;                 /*    48     8 */
	u64                        ipid;                 /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u64                        ip;                   /*    64     8 */
	u64                        tsc;                  /*    72     8 */
	u64                        walltime;             /*    80     8 */
	u32                        cpu;                  /*    88     4 */
	u32                        cpuid;                /*    92     4 */
	u32                        apicid;               /*    96     4 */
	u32                        socketid;             /*   100     4 */
	u8                         cs;                   /*   104     1 */
	u8                         bank;                 /*   105     1 */
	u8                         cpuvendor;            /*   106     1 */
	char                       __data[];             /*   107     0 */

	/* size: 112, cachelines: 2, members: 19 */
	/* padding: 5 */
	/* last cacheline: 48 bytes */
};
struct trace_event_data_offsets_mce_record {

	/* size: 0, cachelines: 0, members: 0 */
};
struct mce_bank {
	u64                        ctl;                  /*     0     8 */
	bool                       init;                 /*     8     1 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 7 */
	/* last cacheline: 16 bytes */
};
struct mce_bank_dev {
	struct device_attribute    attr;                 /*     0    32 */
	char                       attrname[16];         /*    32    16 */
	u8                         bank;                 /*    48     1 */

	/* size: 56, cachelines: 1, members: 3 */
	/* padding: 7 */
	/* last cacheline: 56 bytes */
};
struct severity {
	u64                        mask;                 /*     0     8 */
	u64                        result;               /*     8     8 */
	unsigned char              sev;                  /*    16     1 */
	unsigned char              mcgmask;              /*    17     1 */
	unsigned char              mcgres;               /*    18     1 */
	unsigned char              ser;                  /*    19     1 */
	unsigned char              context;              /*    20     1 */
	unsigned char              excp;                 /*    21     1 */
	unsigned char              covered;              /*    22     1 */
	unsigned char              cpu_model;            /*    23     1 */
	unsigned char              cpu_minstepping;      /*    24     1 */
	unsigned char              bank_lo;              /*    25     1 */
	unsigned char              bank_hi;              /*    26     1 */

	/* XXX 5 bytes hole, try to pack */

	char *                     msg;                  /*    32     8 */

	/* size: 40, cachelines: 1, members: 14 */
	/* sum members: 35, holes: 1, sum holes: 5 */
	/* last cacheline: 40 bytes */
};
struct gen_pool {
	spinlock_t                 lock;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           chunks;               /*     8    16 */
	int                        min_alloc_order;      /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	genpool_algo_t             algo;                 /*    32     8 */
	void *                     data;                 /*    40     8 */
	const char  *              name;                 /*    48     8 */

	/* size: 56, cachelines: 1, members: 6 */
	/* sum members: 48, holes: 2, sum holes: 8 */
	/* last cacheline: 56 bytes */
};
struct smca_hwid {
	unsigned int               bank_type;            /*     0     4 */
	u32                        hwid_mcatype;         /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct smca_bank {
	const struct smca_hwid  *  hwid;                 /*     0     8 */
	u32                        id;                   /*     8     4 */
	u8                         sysfs_id;             /*    12     1 */

	/* size: 16, cachelines: 1, members: 3 */
	/* padding: 3 */
	/* last cacheline: 16 bytes */
};
struct smca_bank_name {
	const char  *              name;                 /*     0     8 */
	const char  *              long_name;            /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct thresh_restart {
	struct threshold_block *   b;                    /*     0     8 */
	int                        reset;                /*     8     4 */
	int                        set_lvt_off;          /*    12     4 */
	int                        lvt_off;              /*    16     4 */
	u16                        old_limit;            /*    20     2 */

	/* size: 24, cachelines: 1, members: 5 */
	/* padding: 2 */
	/* last cacheline: 24 bytes */
};
struct threshold_attr {
	struct attribute           attr;                 /*     0    16 */

	/* XXX last struct has 6 bytes of padding */

	ssize_t                    (*show)(struct threshold_block *, char *); /*    16     8 */
	ssize_t                    (*store)(struct threshold_block *, const char  *, size_t); /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* paddings: 1, sum paddings: 6 */
	/* last cacheline: 32 bytes */
};
struct cper_record_header {
	char                       signature[4];         /*     0     4 */
	u16                        revision;             /*     4     2 */
	u32                        signature_end;        /*     6     4 */
	u16                        section_count;        /*    10     2 */
	u32                        error_severity;       /*    12     4 */
	u32                        validation_bits;      /*    16     4 */
	u32                        record_length;        /*    20     4 */
	u64                        timestamp;            /*    24     8 */
	guid_t                     platform_id;          /*    32    16 */
	guid_t                     partition_id;         /*    48    16 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	guid_t                     creator_id;           /*    64    16 */
	guid_t                     notification_type;    /*    80    16 */
	u64                        record_id;            /*    96     8 */
	u32                        flags;                /*   104     4 */
	u64                        persistence_information; /*   108     8 */
	u8                         reserved[12];         /*   116    12 */

	/* size: 128, cachelines: 2, members: 16 */
} __attribute__((__packed__));
struct cper_section_descriptor {
	u32                        section_offset;       /*     0     4 */
	u32                        section_length;       /*     4     4 */
	u16                        revision;             /*     8     2 */
	u8                         validation_bits;      /*    10     1 */
	u8                         reserved;             /*    11     1 */
	u32                        flags;                /*    12     4 */
	guid_t                     section_type;         /*    16    16 */
	guid_t                     fru_id;               /*    32    16 */
	u32                        section_severity;     /*    48     4 */
	u8                         fru_text[20];         /*    52    20 */

	/* size: 72, cachelines: 2, members: 10 */
	/* last cacheline: 8 bytes */
};
struct cper_sec_mem_err {
	u64                        validation_bits;      /*     0     8 */
	u64                        error_status;         /*     8     8 */
	u64                        physical_addr;        /*    16     8 */
	u64                        physical_addr_mask;   /*    24     8 */
	u16                        node;                 /*    32     2 */
	u16                        card;                 /*    34     2 */
	u16                        module;               /*    36     2 */
	u16                        bank;                 /*    38     2 */
	u16                        device;               /*    40     2 */
	u16                        row;                  /*    42     2 */
	u16                        column;               /*    44     2 */
	u16                        bit_pos;              /*    46     2 */
	u64                        requestor_id;         /*    48     8 */
	u64                        responder_id;         /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u64                        target_id;            /*    64     8 */
	u8                         error_type;           /*    72     1 */
	u8                         extended;             /*    73     1 */
	u16                        rank;                 /*    74     2 */
	u16                        mem_array_handle;     /*    76     2 */
	u16                        mem_dev_handle;       /*    78     2 */

	/* size: 80, cachelines: 2, members: 20 */
	/* last cacheline: 16 bytes */
};
struct cper_mce_record {
	struct cper_record_header  hdr;                  /*     0   128 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct cper_section_descriptor sec_hdr;          /*   128    72 */
	/* --- cacheline 3 boundary (192 bytes) was 8 bytes ago --- */
	struct mce                 mce;                  /*   200   128 */

	/* size: 328, cachelines: 6, members: 3 */
	/* last cacheline: 8 bytes */
};
struct miscdevice {
	int                        minor;                /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	const char  *              name;                 /*     8     8 */
	const struct file_operations  * fops;            /*    16     8 */
	struct list_head           list;                 /*    24    16 */
	struct device *            parent;               /*    40     8 */
	struct device *            this_device;          /*    48     8 */
	const struct attribute_group  * * groups;        /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	const char  *              nodename;             /*    64     8 */
	umode_t                    mode;                 /*    72     2 */

	/* size: 80, cachelines: 2, members: 9 */
	/* sum members: 70, holes: 1, sum holes: 4 */
	/* padding: 6 */
	/* last cacheline: 16 bytes */
};
struct mce_log_buffer {
	char                       signature[12];        /*     0    12 */
	unsigned int               len;                  /*    12     4 */
	unsigned int               next;                 /*    16     4 */
	unsigned int               flags;                /*    20     4 */
	unsigned int               recordlen;            /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	struct mce                 entry[];              /*    32     0 */

	/* size: 32, cachelines: 1, members: 6 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct mtrr_ops {
	u32                        vendor;               /*     0     4 */
	u32                        use_intel_if;         /*     4     4 */
	void                       (*set)(unsigned int, long unsigned int, long unsigned int, mtrr_type); /*     8     8 */
	void                       (*set_all)(void);     /*    16     8 */
	void                       (*get)(unsigned int, long unsigned int *, long unsigned int *, mtrr_type *); /*    24     8 */
	int                        (*get_free_region)(long unsigned int, long unsigned int, int); /*    32     8 */
	int                        (*validate_add_page)(long unsigned int, long unsigned int, unsigned int); /*    40     8 */
	int                        (*have_wrcomb)(void); /*    48     8 */

	/* size: 56, cachelines: 1, members: 8 */
	/* last cacheline: 56 bytes */
};
struct set_mtrr_data {
	long unsigned int          smp_base;             /*     0     8 */
	long unsigned int          smp_size;             /*     8     8 */
	unsigned int               smp_reg;              /*    16     4 */
	mtrr_type                  smp_type;             /*    20     1 */

	/* size: 24, cachelines: 1, members: 4 */
	/* padding: 3 */
	/* last cacheline: 24 bytes */
};
struct mtrr_value {
	mtrr_type                  ltype;                /*     0     1 */

	/* XXX 7 bytes hole, try to pack */

	long unsigned int          lbase;                /*     8     8 */
	long unsigned int          lsize;                /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 17, holes: 1, sum holes: 7 */
	/* last cacheline: 24 bytes */
};
struct proc_ops {
	unsigned int               proc_flags;           /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	int                        (*proc_open)(struct inode *, struct file *); /*     8     8 */
	ssize_t                    (*proc_read)(struct file *, char *, size_t, loff_t *); /*    16     8 */
	ssize_t                    (*proc_read_iter)(struct kiocb *, struct iov_iter *); /*    24     8 */
	ssize_t                    (*proc_write)(struct file *, const char  *, size_t, loff_t *); /*    32     8 */
	loff_t                     (*proc_lseek)(struct file *, loff_t, int); /*    40     8 */
	int                        (*proc_release)(struct inode *, struct file *); /*    48     8 */
	__poll_t                   (*proc_poll)(struct file *, struct poll_table_struct *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	long int                   (*proc_ioctl)(struct file *, unsigned int, long unsigned int); /*    64     8 */
	long int                   (*proc_compat_ioctl)(struct file *, unsigned int, long unsigned int); /*    72     8 */
	int                        (*proc_mmap)(struct file *, struct vm_area_struct *); /*    80     8 */
	long unsigned int          (*proc_get_unmapped_area)(struct file *, long unsigned int, long unsigned int, long unsigned int, long unsigned int); /*    88     8 */

	/* size: 96, cachelines: 2, members: 12 */
	/* sum members: 92, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct mtrr_sentry {
	__u64                      base;                 /*     0     8 */
	__u32                      size;                 /*     8     4 */
	__u32                      type;                 /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct mtrr_gentry {
	__u64                      base;                 /*     0     8 */
	__u32                      size;                 /*     8     4 */
	__u32                      regnum;               /*    12     4 */
	__u32                      type;                 /*    16     4 */
	__u32                      _pad;                 /*    20     4 */

	/* size: 24, cachelines: 1, members: 5 */
	/* last cacheline: 24 bytes */
};
struct mtrr_sentry32 {
	compat_ulong_t             base;                 /*     0     4 */
	compat_uint_t              size;                 /*     4     4 */
	compat_uint_t              type;                 /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct mtrr_gentry32 {
	compat_ulong_t             regnum;               /*     0     4 */
	compat_uint_t              base;                 /*     4     4 */
	compat_uint_t              size;                 /*     8     4 */
	compat_uint_t              type;                 /*    12     4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct mtrr_var_range {
	__u32                      base_lo;              /*     0     4 */
	__u32                      base_hi;              /*     4     4 */
	__u32                      mask_lo;              /*     8     4 */
	__u32                      mask_hi;              /*    12     4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct mtrr_state_type {
	struct mtrr_var_range      var_ranges[256];      /*     0  4096 */
	/* --- cacheline 64 boundary (4096 bytes) --- */
	mtrr_type                  fixed_ranges[88];     /*  4096    88 */
	/* --- cacheline 65 boundary (4160 bytes) was 24 bytes ago --- */
	unsigned char              enabled;              /*  4184     1 */
	unsigned char              have_fixed;           /*  4185     1 */
	mtrr_type                  def_type;             /*  4186     1 */

	/* size: 4188, cachelines: 66, members: 5 */
	/* padding: 1 */
	/* last cacheline: 28 bytes */
};
struct fixed_range_block {
	int                        base_msr;             /*     0     4 */
	int                        ranges;               /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct var_mtrr_range_state {
	long unsigned int          base_pfn;             /*     0     8 */
	long unsigned int          size_pfn;             /*     8     8 */
	mtrr_type                  type;                 /*    16     1 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 7 */
	/* last cacheline: 24 bytes */
};
struct var_mtrr_state {
	long unsigned int          range_startk;         /*     0     8 */
	long unsigned int          range_sizek;          /*     8     8 */
	long unsigned int          chunk_sizek;          /*    16     8 */
	long unsigned int          gran_sizek;           /*    24     8 */
	unsigned int               reg;                  /*    32     4 */

	/* size: 40, cachelines: 1, members: 5 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct mtrr_cleanup_result {
	long unsigned int          gran_sizek;           /*     0     8 */
	long unsigned int          chunk_sizek;          /*     8     8 */
	long unsigned int          lose_cover_sizek;     /*    16     8 */
	unsigned int               num_reg;              /*    24     4 */
	int                        bad;                  /*    28     4 */

	/* size: 32, cachelines: 1, members: 5 */
	/* last cacheline: 32 bytes */
};
struct subsys_interface {
	const char  *              name;                 /*     0     8 */
	struct bus_type *          subsys;               /*     8     8 */
	struct list_head           node;                 /*    16    16 */
	int                        (*add_dev)(struct device *, struct subsys_interface *); /*    32     8 */
	void                       (*remove_dev)(struct device *, struct subsys_interface *); /*    40     8 */

	/* size: 48, cachelines: 1, members: 5 */
	/* last cacheline: 48 bytes */
};
struct platform_device_info {
	struct device *            parent;               /*     0     8 */
	struct fwnode_handle *     fwnode;               /*     8     8 */
	bool                       of_node_reused;       /*    16     1 */

	/* XXX 7 bytes hole, try to pack */

	const char  *              name;                 /*    24     8 */
	int                        id;                   /*    32     4 */

	/* XXX 4 bytes hole, try to pack */

	const struct resource  *   res;                  /*    40     8 */
	unsigned int               num_res;              /*    48     4 */

	/* XXX 4 bytes hole, try to pack */

	const void  *              data;                 /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	size_t                     size_data;            /*    64     8 */
	u64                        dma_mask;             /*    72     8 */
	const struct property_entry  * properties;       /*    80     8 */

	/* size: 88, cachelines: 2, members: 11 */
	/* sum members: 73, holes: 3, sum holes: 15 */
	/* last cacheline: 24 bytes */
};
struct builtin_fw {
	char *                     name;                 /*     0     8 */
	void *                     data;                 /*     8     8 */
	long unsigned int          size;                 /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct cpio_data {
	void *                     data;                 /*     0     8 */
	size_t                     size;                 /*     8     8 */
	char                       name[18];             /*    16    18 */

	/* size: 40, cachelines: 1, members: 3 */
	/* padding: 6 */
	/* last cacheline: 40 bytes */
};
struct cpu_signature {
	unsigned int               sig;                  /*     0     4 */
	unsigned int               pf;                   /*     4     4 */
	unsigned int               rev;                  /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct microcode_ops {
	enum ucode_state           (*request_microcode_user)(int, const void  *, size_t); /*     0     8 */
	enum ucode_state           (*request_microcode_fw)(int, struct device *, bool); /*     8     8 */
	void                       (*microcode_fini_cpu)(int); /*    16     8 */
	enum ucode_state           (*apply_microcode)(int); /*    24     8 */
	int                        (*collect_cpu_info)(int, struct cpu_signature *); /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct ucode_cpu_info {
	struct cpu_signature       cpu_sig;              /*     0    12 */
	int                        valid;                /*    12     4 */
	void *                     mc;                   /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct cpu_info_ctx {
	struct cpu_signature *     cpu_sig;              /*     0     8 */
	int                        err;                  /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct firmware {
	size_t                     size;                 /*     0     8 */
	const u8  *                data;                 /*     8     8 */
	void *                     priv;                 /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct ucode_patch {
	struct list_head           plist;                /*     0    16 */
	void *                     data;                 /*    16     8 */
	u32                        patch_id;             /*    24     4 */
	u16                        equiv_cpu;            /*    28     2 */

	/* size: 32, cachelines: 1, members: 4 */
	/* padding: 2 */
	/* last cacheline: 32 bytes */
};
struct microcode_header_intel {
	unsigned int               hdrver;               /*     0     4 */
	unsigned int               rev;                  /*     4     4 */
	unsigned int               date;                 /*     8     4 */
	unsigned int               sig;                  /*    12     4 */
	unsigned int               cksum;                /*    16     4 */
	unsigned int               ldrver;               /*    20     4 */
	unsigned int               pf;                   /*    24     4 */
	unsigned int               datasize;             /*    28     4 */
	unsigned int               totalsize;            /*    32     4 */
	unsigned int               reserved[3];          /*    36    12 */

	/* size: 48, cachelines: 1, members: 10 */
	/* last cacheline: 48 bytes */
};
struct microcode_intel {
	struct microcode_header_intel hdr;               /*     0    48 */
	unsigned int               bits[];               /*    48     0 */

	/* size: 48, cachelines: 1, members: 2 */
	/* last cacheline: 48 bytes */
};
struct extended_signature {
	unsigned int               sig;                  /*     0     4 */
	unsigned int               pf;                   /*     4     4 */
	unsigned int               cksum;                /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct extended_sigtable {
	unsigned int               count;                /*     0     4 */
	unsigned int               cksum;                /*     4     4 */
	unsigned int               reserved[3];          /*     8    12 */
	struct extended_signature  sigs[];               /*    20     0 */

	/* size: 20, cachelines: 1, members: 4 */
	/* last cacheline: 20 bytes */
};
struct equiv_cpu_entry {
	u32                        installed_cpu;        /*     0     4 */
	u32                        fixed_errata_mask;    /*     4     4 */
	u32                        fixed_errata_compare; /*     8     4 */
	u16                        equiv_cpu;            /*    12     2 */
	u16                        res;                  /*    14     2 */

	/* size: 16, cachelines: 1, members: 5 */
	/* last cacheline: 16 bytes */
};
struct microcode_header_amd {
	u32                        data_code;            /*     0     4 */
	u32                        patch_id;             /*     4     4 */
	u16                        mc_patch_data_id;     /*     8     2 */
	u8                         mc_patch_data_len;    /*    10     1 */
	u8                         init_flag;            /*    11     1 */
	u32                        mc_patch_data_checksum; /*    12     4 */
	u32                        nb_dev_id;            /*    16     4 */
	u32                        sb_dev_id;            /*    20     4 */
	u16                        processor_rev_id;     /*    24     2 */
	u8                         nb_rev_id;            /*    26     1 */
	u8                         sb_rev_id;            /*    27     1 */
	u8                         bios_api_rev;         /*    28     1 */
	u8                         reserved1[3];         /*    29     3 */
	u32                        match_reg[8];         /*    32    32 */

	/* size: 64, cachelines: 1, members: 14 */
};
struct microcode_amd {
	struct microcode_header_amd hdr;                 /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	unsigned int               mpb[];                /*    64     0 */

	/* size: 64, cachelines: 1, members: 2 */
};
struct equiv_cpu_table {
	unsigned int               num_entries;          /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct equiv_cpu_entry *   entry;                /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct cont_desc {
	struct microcode_amd *     mc;                   /*     0     8 */
	u32                        cpuid_1_eax;          /*     8     4 */
	u32                        psize;                /*    12     4 */
	u8 *                       data;                 /*    16     8 */
	size_t                     size;                 /*    24     8 */

	/* size: 32, cachelines: 1, members: 5 */
	/* last cacheline: 32 bytes */
};
struct rdtgroup {
	struct kernfs_node *       kn;                   /*     0     8 */
	struct list_head           rdtgroup_list;        /*     8    16 */
	u32                        closid;               /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	struct cpumask             cpu_mask;             /*    32  1024 */
	/* --- cacheline 16 boundary (1024 bytes) was 32 bytes ago --- */
	int                        flags;                /*  1056     4 */
	atomic_t                   waitcount;            /*  1060     4 */
	enum rdt_group_type        type;                 /*  1064     4 */

	/* XXX 4 bytes hole, try to pack */

	struct mongroup            mon;                  /*  1072    40 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 17 boundary (1088 bytes) was 24 bytes ago --- */
	enum rdtgrp_mode           mode;                 /*  1112     4 */

	/* XXX 4 bytes hole, try to pack */

	struct pseudo_lock_region * plr;                 /*  1120     8 */

	/* size: 1128, cachelines: 18, members: 10 */
	/* sum members: 1116, holes: 3, sum holes: 12 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 40 bytes */
};
struct rdt_resource {
	int                        rid;                  /*     0     4 */
	bool                       alloc_enabled;        /*     4     1 */
	bool                       mon_enabled;          /*     5     1 */
	bool                       alloc_capable;        /*     6     1 */
	bool                       mon_capable;          /*     7     1 */
	char *                     name;                 /*     8     8 */
	int                        num_closid;           /*    16     4 */
	int                        cache_level;          /*    20     4 */
	u32                        default_ctrl;         /*    24     4 */
	unsigned int               msr_base;             /*    28     4 */
	void                       (*msr_update)(struct rdt_domain *, struct msr_param *, struct rdt_resource *); /*    32     8 */
	int                        data_width;           /*    40     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           domains;              /*    48    16 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct rdt_cache           cache;                /*    64    24 */

	/* XXX last struct has 1 byte of padding */

	struct rdt_membw           membw;                /*    88    32 */
	const char  *              format_str;           /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	int                        (*parse_ctrlval)(struct rdt_parse_data *, struct rdt_resource *, struct rdt_domain *); /*   128     8 */
	struct list_head           evt_list;             /*   136    16 */
	int                        num_rmid;             /*   152     4 */
	unsigned int               mon_scale;            /*   156     4 */
	unsigned int               mbm_width;            /*   160     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          fflags;               /*   168     8 */

	/* size: 176, cachelines: 3, members: 22 */
	/* sum members: 168, holes: 2, sum holes: 8 */
	/* paddings: 1, sum paddings: 1 */
	/* last cacheline: 48 bytes */
};
struct rdt_domain {
	struct list_head           list;                 /*     0    16 */
	int                        id;                   /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	struct cpumask             cpu_mask;             /*    24  1024 */
	/* --- cacheline 16 boundary (1024 bytes) was 24 bytes ago --- */
	long unsigned int *        rmid_busy_llc;        /*  1048     8 */
	struct mbm_state *         mbm_total;            /*  1056     8 */
	struct mbm_state *         mbm_local;            /*  1064     8 */
	struct delayed_work        mbm_over;             /*  1072   192 */
	/* --- cacheline 19 boundary (1216 bytes) was 48 bytes ago --- */
	struct delayed_work        cqm_limbo;            /*  1264   192 */
	/* --- cacheline 22 boundary (1408 bytes) was 48 bytes ago --- */
	int                        mbm_work_cpu;         /*  1456     4 */
	int                        cqm_work_cpu;         /*  1460     4 */
	u32 *                      ctrl_val;             /*  1464     8 */
	/* --- cacheline 23 boundary (1472 bytes) --- */
	u32 *                      mbps_val;             /*  1472     8 */
	u32                        new_ctrl;             /*  1480     4 */
	bool                       have_new_ctrl;        /*  1484     1 */

	/* XXX 3 bytes hole, try to pack */

	struct pseudo_lock_region * plr;                 /*  1488     8 */

	/* size: 1496, cachelines: 24, members: 15 */
	/* sum members: 1489, holes: 2, sum holes: 7 */
	/* last cacheline: 24 bytes */
};
struct mongroup {
	struct kernfs_node *       mon_data_kn;          /*     0     8 */
	struct rdtgroup *          parent;               /*     8     8 */
	struct list_head           crdtgrp_list;         /*    16    16 */
	u32                        rmid;                 /*    32     4 */

	/* size: 40, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct pseudo_lock_region {
	struct rdt_resource *      r;                    /*     0     8 */
	struct rdt_domain *        d;                    /*     8     8 */
	u32                        cbm;                  /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	wait_queue_head_t          lock_thread_wq;       /*    24    24 */
	int                        thread_done;          /*    48     4 */
	int                        cpu;                  /*    52     4 */
	unsigned int               line_size;            /*    56     4 */
	unsigned int               size;                 /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	void *                     kmem;                 /*    64     8 */
	unsigned int               minor;                /*    72     4 */

	/* XXX 4 bytes hole, try to pack */

	struct dentry *            debugfs_dir;          /*    80     8 */
	struct list_head           pm_reqs;              /*    88    16 */

	/* size: 104, cachelines: 2, members: 12 */
	/* sum members: 96, holes: 2, sum holes: 8 */
	/* last cacheline: 40 bytes */
};
struct mbm_state {
	u64                        chunks;               /*     0     8 */
	u64                        prev_msr;             /*     8     8 */
	u64                        prev_bw_msr;          /*    16     8 */
	u32                        prev_bw;              /*    24     4 */
	u32                        delta_bw;             /*    28     4 */
	bool                       delta_comp;           /*    32     1 */

	/* size: 40, cachelines: 1, members: 6 */
	/* padding: 7 */
	/* last cacheline: 40 bytes */
};
struct msr_param {
	struct rdt_resource *      res;                  /*     0     8 */
	int                        low;                  /*     8     4 */
	int                        high;                 /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct rdt_cache {
	unsigned int               cbm_len;              /*     0     4 */
	unsigned int               min_cbm_bits;         /*     4     4 */
	unsigned int               cbm_idx_mult;         /*     8     4 */
	unsigned int               cbm_idx_offset;       /*    12     4 */
	unsigned int               shareable_bits;       /*    16     4 */
	bool                       arch_has_sparse_bitmaps; /*    20     1 */
	bool                       arch_has_empty_bitmaps; /*    21     1 */
	bool                       arch_has_per_cpu_cfg; /*    22     1 */

	/* size: 24, cachelines: 1, members: 8 */
	/* padding: 1 */
	/* last cacheline: 24 bytes */
};
struct rdt_membw {
	u32                        min_bw;               /*     0     4 */
	u32                        bw_gran;              /*     4     4 */
	u32                        delay_linear;         /*     8     4 */
	bool                       arch_needs_linear;    /*    12     1 */

	/* XXX 3 bytes hole, try to pack */

	enum membw_throttle_mode   throttle_mode;        /*    16     4 */
	bool                       mba_sc;               /*    20     1 */

	/* XXX 3 bytes hole, try to pack */

	u32 *                      mb_map;               /*    24     8 */

	/* size: 32, cachelines: 1, members: 7 */
	/* sum members: 26, holes: 2, sum holes: 6 */
	/* last cacheline: 32 bytes */
};
struct rdt_parse_data {
	struct rdtgroup *          rdtgrp;               /*     0     8 */
	char *                     buf;                  /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
union cpuid_0x10_1_eax {
	struct {
		unsigned int       cbm_len:5;          /*     0: 0  4 */
	} split;                                       /*     0     4 */
	unsigned int               full;               /*     0     4 */
};
union cpuid_0x10_3_eax {
	struct {
		unsigned int       max_delay:12;       /*     0: 0  4 */
	} split;                                       /*     0     4 */
	unsigned int               full;               /*     0     4 */
};
union cpuid_0x10_x_edx {
	struct {
		unsigned int       cos_max:16;         /*     0: 0  4 */
	} split;                                       /*     0     4 */
	unsigned int               full;               /*     0     4 */
};
struct rdt_options {
	char *                     name;                 /*     0     8 */
	int                        flag;                 /*     8     4 */
	bool                       force_off;            /*    12     1 */
	bool                       force_on;             /*    13     1 */

	/* size: 16, cachelines: 1, members: 4 */
	/* padding: 2 */
	/* last cacheline: 16 bytes */
};
struct __va_list_tag {
	unsigned int               gp_offset;            /*     0     4 */
	unsigned int               fp_offset;            /*     4     4 */
	void *                     overflow_arg_area;    /*     8     8 */
	void *                     reg_save_area;        /*    16     8 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct kernfs_fs_context {
	struct kernfs_root *       root;                 /*     0     8 */
	void *                     ns_tag;               /*     8     8 */
	long unsigned int          magic;                /*    16     8 */
	bool                       new_sb_created;       /*    24     1 */

	/* size: 32, cachelines: 1, members: 4 */
	/* padding: 7 */
	/* last cacheline: 32 bytes */
};
struct rdt_fs_context {
	struct kernfs_fs_context   kfc;                  /*     0    32 */

	/* XXX last struct has 7 bytes of padding */

	bool                       enable_cdpl2;         /*    32     1 */
	bool                       enable_cdpl3;         /*    33     1 */
	bool                       enable_mba_mbps;      /*    34     1 */

	/* size: 40, cachelines: 1, members: 4 */
	/* padding: 5 */
	/* paddings: 1, sum paddings: 7 */
	/* last cacheline: 40 bytes */
};
struct mon_evt {
	u32                        evtid;                /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	char *                     name;                 /*     8     8 */
	struct list_head           list;                 /*    16    16 */

	/* size: 32, cachelines: 1, members: 3 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
union mon_data_bits {
	void *                     priv;               /*     0     8 */
	struct {
		unsigned int       rid:10;             /*     0: 0  4 */
		unsigned int       evtid:8;            /*     0:10  4 */
		unsigned int       domid:14;           /*     0:18  4 */
	} u;                                           /*     0     4 */
};
struct rmid_read {
	struct rdtgroup *          rgrp;                 /*     0     8 */
	struct rdt_resource *      r;                    /*     8     8 */
	struct rdt_domain *        d;                    /*    16     8 */
	int                        evtid;                /*    24     4 */
	bool                       first;                /*    28     1 */

	/* XXX 3 bytes hole, try to pack */

	u64                        val;                  /*    32     8 */

	/* size: 40, cachelines: 1, members: 6 */
	/* sum members: 37, holes: 1, sum holes: 3 */
	/* last cacheline: 40 bytes */
};
struct rftype {
	char *                     name;                 /*     0     8 */
	umode_t                    mode;                 /*     8     2 */

	/* XXX 6 bytes hole, try to pack */

	struct kernfs_ops *        kf_ops;               /*    16     8 */
	long unsigned int          flags;                /*    24     8 */
	long unsigned int          fflags;               /*    32     8 */
	int                        (*seq_show)(struct kernfs_open_file *, struct seq_file *, void *); /*    40     8 */
	ssize_t                    (*write)(struct kernfs_open_file *, char *, size_t, loff_t); /*    48     8 */

	/* size: 56, cachelines: 1, members: 7 */
	/* sum members: 50, holes: 1, sum holes: 6 */
	/* last cacheline: 56 bytes */
};
struct rmid_entry {
	u32                        rmid;                 /*     0     4 */
	int                        busy;                 /*     4     4 */
	struct list_head           list;                 /*     8    16 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_pseudo_lock_mem_latency {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        latency;              /*     8     4 */
	char                       __data[];             /*    12     0 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct trace_event_raw_pseudo_lock_l2 {
	struct trace_entry         ent;                  /*     0     8 */
	u64                        l2_hits;              /*     8     8 */
	u64                        l2_miss;              /*    16     8 */
	char                       __data[];             /*    24     0 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_pseudo_lock_l3 {
	struct trace_entry         ent;                  /*     0     8 */
	u64                        l3_hits;              /*     8     8 */
	u64                        l3_miss;              /*    16     8 */
	char                       __data[];             /*    24     0 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct trace_event_data_offsets_pseudo_lock_mem_latency {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_pseudo_lock_l2 {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_pseudo_lock_l3 {

	/* size: 0, cachelines: 0, members: 0 */
};
struct pseudo_lock_pm_req {
	struct list_head           list;                 /*     0    16 */
	struct dev_pm_qos_request  req;                  /*    16    88 */

	/* size: 104, cachelines: 2, members: 2 */
	/* last cacheline: 40 bytes */
};
struct residency_counts {
	u64                        miss_before;          /*     0     8 */
	u64                        hits_before;          /*     8     8 */
	u64                        miss_after;           /*    16     8 */
	u64                        hits_after;           /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct mmu_notifier_ops {
	void                       (*release)(struct mmu_notifier *, struct mm_struct *); /*     0     8 */
	int                        (*clear_flush_young)(struct mmu_notifier *, struct mm_struct *, long unsigned int, long unsigned int); /*     8     8 */
	int                        (*clear_young)(struct mmu_notifier *, struct mm_struct *, long unsigned int, long unsigned int); /*    16     8 */
	int                        (*test_young)(struct mmu_notifier *, struct mm_struct *, long unsigned int); /*    24     8 */
	void                       (*change_pte)(struct mmu_notifier *, struct mm_struct *, long unsigned int, pte_t); /*    32     8 */
	int                        (*invalidate_range_start)(struct mmu_notifier *, const struct mmu_notifier_range  *); /*    40     8 */
	void                       (*invalidate_range_end)(struct mmu_notifier *, const struct mmu_notifier_range  *); /*    48     8 */
	void                       (*invalidate_range)(struct mmu_notifier *, struct mm_struct *, long unsigned int, long unsigned int); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct mmu_notifier *      (*alloc_notifier)(struct mm_struct *); /*    64     8 */
	void                       (*free_notifier)(struct mmu_notifier *); /*    72     8 */
	u64                        kabi_reserved1;       /*    80     8 */
	u64                        kabi_reserved2;       /*    88     8 */
	u64                        kabi_reserved3;       /*    96     8 */
	u64                        kabi_reserved4;       /*   104     8 */

	/* size: 112, cachelines: 2, members: 14 */
	/* last cacheline: 48 bytes */
};
struct mmu_notifier {
	struct hlist_node          hlist;                /*     0    16 */
	const struct mmu_notifier_ops  * ops;            /*    16     8 */
	struct mm_struct *         mm;                   /*    24     8 */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*    32    16 */
	unsigned int               users;                /*    48     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        kabi_reserved1;       /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u64                        kabi_reserved2;       /*    64     8 */

	/* size: 72, cachelines: 2, members: 7 */
	/* sum members: 68, holes: 1, sum holes: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 8 bytes */
} __attribute__((__aligned__(8)));
struct mmu_notifier_range {
	struct vm_area_struct *    vma;                  /*     0     8 */
	struct mm_struct *         mm;                   /*     8     8 */
	long unsigned int          start;                /*    16     8 */
	long unsigned int          end;                  /*    24     8 */
	unsigned int               flags;                /*    32     4 */
	enum mmu_notifier_event    event;                /*    36     4 */
	void *                     migrate_pgmap_owner;  /*    40     8 */

	/* size: 48, cachelines: 1, members: 7 */
	/* last cacheline: 48 bytes */
};
struct sgx_epc_page {
	unsigned int               section;              /*     0     4 */
	u16                        flags;                /*     4     2 */
	u16                        poison;               /*     6     2 */
	struct sgx_encl_page *     owner;                /*     8     8 */
	struct list_head           list;                 /*    16    16 */

	/* size: 32, cachelines: 1, members: 5 */
	/* last cacheline: 32 bytes */
};
struct sgx_encl_page {
	long unsigned int          desc;                 /*     0     8 */
	long unsigned int          vm_max_prot_bits:8;   /*     8: 0  8 */
	enum sgx_page_type         type:16;              /*     8: 8  4 */

	/* XXX 40 bits hole, try to pack */

	struct sgx_epc_page *      epc_page;             /*    16     8 */
	struct sgx_encl *          encl;                 /*    24     8 */
	struct sgx_va_page *       va_page;              /*    32     8 */

	/* size: 40, cachelines: 1, members: 6 */
	/* sum members: 32 */
	/* sum bitfield members: 24 bits, bit holes: 1, sum bit holes: 40 bits */
	/* last cacheline: 40 bytes */
};
struct sgx_encl {
	long unsigned int          base;                 /*     0     8 */
	long unsigned int          size;                 /*     8     8 */
	long unsigned int          flags;                /*    16     8 */
	unsigned int               page_cnt;             /*    24     4 */
	unsigned int               secs_child_cnt;       /*    28     4 */
	struct mutex               lock;                 /*    32    32 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct xarray              page_array;           /*    64    16 */
	struct sgx_encl_page       secs;                 /*    80    40 */
	long unsigned int          attributes;           /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	long unsigned int          attributes_mask;      /*   128     8 */
	cpumask_t                  cpumask;              /*   136  1024 */
	/* --- cacheline 18 boundary (1152 bytes) was 8 bytes ago --- */
	struct file *              backing;              /*  1160     8 */
	struct kref                refcount;             /*  1168     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           va_pages;             /*  1176    16 */
	long unsigned int          mm_list_version;      /*  1192     8 */
	struct list_head           mm_list;              /*  1200    16 */
	/* --- cacheline 19 boundary (1216 bytes) --- */
	spinlock_t                 mm_lock;              /*  1216     4 */

	/* XXX 4 bytes hole, try to pack */

	struct srcu_struct         srcu;                 /*  1224 50440 */

	/* size: 51664, cachelines: 808, members: 18 */
	/* sum members: 51656, holes: 2, sum holes: 8 */
	/* last cacheline: 16 bytes */
};
struct sgx_va_page {
	struct sgx_epc_page *      epc_page;             /*     0     8 */
	long unsigned int          slots[8];             /*     8    64 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	struct list_head           list;                 /*    72    16 */

	/* size: 88, cachelines: 2, members: 3 */
	/* last cacheline: 24 bytes */
};
struct sgx_encl_mm {
	struct sgx_encl *          encl;                 /*     0     8 */
	struct mm_struct *         mm;                   /*     8     8 */
	struct list_head           list;                 /*    16    16 */
	struct mmu_notifier        mmu_notifier __attribute__((__aligned__(8))); /*    32    72 */

	/* size: 104, cachelines: 2, members: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 40 bytes */
} __attribute__((__aligned__(8)));
struct xa_node {
	unsigned char              shift;                /*     0     1 */
	unsigned char              offset;               /*     1     1 */
	unsigned char              count;                /*     2     1 */
	unsigned char              nr_values;            /*     3     1 */

	/* XXX 4 bytes hole, try to pack */

	struct xa_node *           parent;               /*     8     8 */
	struct xarray *            array;                /*    16     8 */
	union {
		struct list_head   private_list;         /*    24    16 */
		struct callback_head callback_head __attribute__((__aligned__(8))); /*    24    16 */
	} __attribute__((__aligned__(8)));               /*    24    16 */
	void *                     slots[64];            /*    40   512 */
	/* --- cacheline 8 boundary (512 bytes) was 40 bytes ago --- */
	union {
		long unsigned int  tags[3][1];           /*   552    24 */
		long unsigned int  marks[3][1];          /*   552    24 */
	};                                               /*   552    24 */

	/* size: 576, cachelines: 9, members: 9 */
	/* sum members: 572, holes: 1, sum holes: 4 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(8)));
struct xa_state {
	struct xarray *            xa;                   /*     0     8 */
	long unsigned int          xa_index;             /*     8     8 */
	unsigned char              xa_shift;             /*    16     1 */
	unsigned char              xa_sibs;              /*    17     1 */
	unsigned char              xa_offset;            /*    18     1 */
	unsigned char              xa_pad;               /*    19     1 */

	/* XXX 4 bytes hole, try to pack */

	struct xa_node *           xa_node;              /*    24     8 */
	struct xa_node *           xa_alloc;             /*    32     8 */
	xa_update_node_t           xa_update;            /*    40     8 */

	/* size: 48, cachelines: 1, members: 9 */
	/* sum members: 44, holes: 1, sum holes: 4 */
	/* last cacheline: 48 bytes */
};
struct sgx_pageinfo {
	u64                        addr;                 /*     0     8 */
	u64                        contents;             /*     8     8 */
	u64                        metadata;             /*    16     8 */
	u64                        secs;                 /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(32)));
struct sgx_numa_node {
	struct list_head           free_page_list;       /*     0    16 */
	struct list_head           sgx_poison_page_list; /*    16    16 */
	long unsigned int          size;                 /*    32     8 */
	spinlock_t                 lock;                 /*    40     4 */

	/* size: 48, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 48 bytes */
};
struct sgx_epc_section {
	long unsigned int          phys_addr;            /*     0     8 */
	void *                     virt_addr;            /*     8     8 */
	struct sgx_epc_page *      pages;                /*    16     8 */
	struct sgx_numa_node *     node;                 /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct sgx_backing {
	struct page *              contents;             /*     0     8 */
	struct page *              pcmd;                 /*     8     8 */
	long unsigned int          pcmd_offset;          /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct sgx_secs {
	u64                        size;                 /*     0     8 */
	u64                        base;                 /*     8     8 */
	u32                        ssa_frame_size;       /*    16     4 */
	u32                        miscselect;           /*    20     4 */
	u8                         reserved1[24];        /*    24    24 */
	u64                        attributes;           /*    48     8 */
	u64                        xfrm;                 /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u32                        mrenclave[8];         /*    64    32 */
	u8                         reserved2[32];        /*    96    32 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	u32                        mrsigner[8];          /*   128    32 */
	u8                         reserved3[32];        /*   160    32 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	u32                        config_id[16];        /*   192    64 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	u16                        isv_prod_id;          /*   256     2 */
	u16                        isv_svn;              /*   258     2 */
	u16                        config_svn;           /*   260     2 */
	u8                         reserved4[3834];      /*   262  3834 */

	/* size: 4096, cachelines: 64, members: 16 */
};
struct sgx_secinfo {
	u64                        flags;                /*     0     8 */
	u8                         reserved[56];         /*     8    56 */

	/* size: 64, cachelines: 1, members: 2 */
} __attribute__((__aligned__(64)));
struct sgx_sigstruct_header {
	u64                        header1[2];           /*     0    16 */
	u32                        vendor;               /*    16     4 */
	u32                        date;                 /*    20     4 */
	u64                        header2[2];           /*    24    16 */
	u32                        swdefined;            /*    40     4 */
	u8                         reserved1[84];        /*    44    84 */

	/* size: 128, cachelines: 2, members: 6 */
};
struct sgx_sigstruct_body {
	u32                        miscselect;           /*     0     4 */
	u32                        misc_mask;            /*     4     4 */
	u8                         reserved2[20];        /*     8    20 */
	u64                        attributes;           /*    28     8 */
	u64                        xfrm;                 /*    36     8 */
	u64                        attributes_mask;      /*    44     8 */
	u64                        xfrm_mask;            /*    52     8 */
	u8                         mrenclave[32];        /*    60    32 */
	/* --- cacheline 1 boundary (64 bytes) was 28 bytes ago --- */
	u8                         reserved3[32];        /*    92    32 */
	u16                        isvprodid;            /*   124     2 */
	u16                        isvsvn;               /*   126     2 */

	/* size: 128, cachelines: 2, members: 11 */
} __attribute__((__packed__));
struct sgx_sigstruct {
	struct sgx_sigstruct_header header;              /*     0   128 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	u8                         modulus[384];         /*   128   384 */
	/* --- cacheline 8 boundary (512 bytes) --- */
	u32                        exponent;             /*   512     4 */
	u8                         signature[384];       /*   516   384 */
	/* --- cacheline 14 boundary (896 bytes) was 4 bytes ago --- */
	struct sgx_sigstruct_body  body;                 /*   900   128 */
	/* --- cacheline 16 boundary (1024 bytes) was 4 bytes ago --- */
	u8                         reserved4[12];        /*  1028    12 */
	u8                         q1[384];              /*  1040   384 */
	/* --- cacheline 22 boundary (1408 bytes) was 16 bytes ago --- */
	u8                         q2[384];              /*  1424   384 */

	/* size: 1808, cachelines: 29, members: 8 */
	/* last cacheline: 16 bytes */
} __attribute__((__packed__));
struct crypto_tfm {
	u32                        crt_flags;            /*     0     4 */
	int                        node;                 /*     4     4 */
	void                       (*exit)(struct crypto_tfm *); /*     8     8 */
	struct crypto_alg *        __crt_alg;            /*    16     8 */
	void *                     __crt_ctx[] __attribute__((__aligned__(8))); /*    24     0 */

	/* size: 24, cachelines: 1, members: 5 */
	/* forced alignments: 1 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct cipher_alg {
	unsigned int               cia_min_keysize;      /*     0     4 */
	unsigned int               cia_max_keysize;      /*     4     4 */
	int                        (*cia_setkey)(struct crypto_tfm *, const u8  *, unsigned int); /*     8     8 */
	void                       (*cia_encrypt)(struct crypto_tfm *, u8 *, const u8  *); /*    16     8 */
	void                       (*cia_decrypt)(struct crypto_tfm *, u8 *, const u8  *); /*    24     8 */

	/* size: 32, cachelines: 1, members: 5 */
	/* last cacheline: 32 bytes */
};
struct compress_alg {
	int                        (*coa_compress)(struct crypto_tfm *, const u8  *, unsigned int, u8 *, unsigned int *); /*     0     8 */
	int                        (*coa_decompress)(struct crypto_tfm *, const u8  *, unsigned int, u8 *, unsigned int *); /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct crypto_alg {
	struct list_head           cra_list;             /*     0    16 */
	struct list_head           cra_users;            /*    16    16 */
	u32                        cra_flags;            /*    32     4 */
	unsigned int               cra_blocksize;        /*    36     4 */
	unsigned int               cra_ctxsize;          /*    40     4 */
	unsigned int               cra_alignmask;        /*    44     4 */
	int                        cra_priority;         /*    48     4 */
	refcount_t                 cra_refcnt;           /*    52     4 */
	char                       cra_name[128];        /*    56   128 */
	/* --- cacheline 2 boundary (128 bytes) was 56 bytes ago --- */
	char                       cra_driver_name[128]; /*   184   128 */
	/* --- cacheline 4 boundary (256 bytes) was 56 bytes ago --- */
	const struct crypto_type  * cra_type;            /*   312     8 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	union {
		struct cipher_alg  cipher;               /*   320    32 */
		struct compress_alg compress;            /*   320    16 */
	} cra_u;                                         /*   320    32 */
	int                        (*cra_init)(struct crypto_tfm *); /*   352     8 */
	void                       (*cra_exit)(struct crypto_tfm *); /*   360     8 */
	void                       (*cra_destroy)(struct crypto_alg *); /*   368     8 */
	struct module *            cra_module;           /*   376     8 */

	/* size: 384, cachelines: 6, members: 16 */
} __attribute__((__aligned__(8)));
struct shash_desc {
	struct crypto_shash *      tfm;                  /*     0     8 */
	void *                     __ctx[] __attribute__((__aligned__(8))); /*     8     0 */

	/* size: 8, cachelines: 1, members: 2 */
	/* forced alignments: 1 */
	/* last cacheline: 8 bytes */
} __attribute__((__aligned__(8)));
struct crypto_shash {
	unsigned int               descsize;             /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct crypto_tfm          base __attribute__((__aligned__(8))); /*     8    24 */

	/* size: 32, cachelines: 1, members: 2 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 4 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct sgx_enclave_create {
	__u64                      src;                  /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct sgx_enclave_add_pages {
	__u64                      src;                  /*     0     8 */
	__u64                      offset;               /*     8     8 */
	__u64                      length;               /*    16     8 */
	__u64                      secinfo;              /*    24     8 */
	__u64                      flags;                /*    32     8 */
	__u64                      count;                /*    40     8 */

	/* size: 48, cachelines: 1, members: 6 */
	/* last cacheline: 48 bytes */
};
struct sgx_enclave_init {
	__u64                      sigstruct;            /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct sgx_enclave_provision {
	__u64                      fd;                   /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct sgx_enclave_restrict_permissions {
	__u64                      offset;               /*     0     8 */
	__u64                      length;               /*     8     8 */
	__u64                      permissions;          /*    16     8 */
	__u64                      result;               /*    24     8 */
	__u64                      count;                /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct sgx_enclave_modify_types {
	__u64                      offset;               /*     0     8 */
	__u64                      length;               /*     8     8 */
	__u64                      page_type;            /*    16     8 */
	__u64                      result;               /*    24     8 */
	__u64                      count;                /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct sgx_enclave_remove_pages {
	__u64                      offset;               /*     0     8 */
	__u64                      length;               /*     8     8 */
	__u64                      count;                /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct node {
	struct device              dev __attribute__((__aligned__(8))); /*     0  1000 */
	/* --- cacheline 15 boundary (960 bytes) was 40 bytes ago --- */
	struct list_head           access_list;          /*  1000    16 */
	struct work_struct         node_work;            /*  1016    64 */
	/* --- cacheline 16 boundary (1024 bytes) was 56 bytes ago --- */
	struct list_head           cache_attrs;          /*  1080    16 */
	/* --- cacheline 17 boundary (1088 bytes) was 8 bytes ago --- */
	struct device *            cache_dev;            /*  1096     8 */

	/* size: 1104, cachelines: 18, members: 5 */
	/* forced alignments: 1 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct sgx_vepc {
	struct xarray              page_array;           /*     0    16 */
	struct mutex               lock;                 /*    16    32 */

	/* size: 48, cachelines: 1, members: 2 */
	/* last cacheline: 48 bytes */
};
struct ghcb {
	struct vmcb_save_area      save;                 /*     0  1032 */
	/* --- cacheline 16 boundary (1024 bytes) was 8 bytes ago --- */
	u8                         reserved_save[1016];  /*  1032  1016 */
	/* --- cacheline 32 boundary (2048 bytes) --- */
	u8                         shared_buffer[2032];  /*  2048  2032 */
	/* --- cacheline 63 boundary (4032 bytes) was 48 bytes ago --- */
	u8                         reserved_1[10];       /*  4080    10 */
	u16                        protocol_version;     /*  4090     2 */
	u32                        ghcb_usage;           /*  4092     4 */

	/* size: 4096, cachelines: 64, members: 6 */
};
struct vmcb_seg {
	u16                        selector;             /*     0     2 */
	u16                        attrib;               /*     2     2 */
	u32                        limit;                /*     4     4 */
	u64                        base;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct vmcb_save_area {
	struct vmcb_seg            es;                   /*     0    16 */
	struct vmcb_seg            cs;                   /*    16    16 */
	struct vmcb_seg            ss;                   /*    32    16 */
	struct vmcb_seg            ds;                   /*    48    16 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct vmcb_seg            fs;                   /*    64    16 */
	struct vmcb_seg            gs;                   /*    80    16 */
	struct vmcb_seg            gdtr;                 /*    96    16 */
	struct vmcb_seg            ldtr;                 /*   112    16 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct vmcb_seg            idtr;                 /*   128    16 */
	struct vmcb_seg            tr;                   /*   144    16 */
	u8                         reserved_1[43];       /*   160    43 */
	/* --- cacheline 3 boundary (192 bytes) was 11 bytes ago --- */
	u8                         cpl;                  /*   203     1 */
	u8                         reserved_2[4];        /*   204     4 */
	u64                        efer;                 /*   208     8 */
	u8                         reserved_3[112];      /*   216   112 */
	/* --- cacheline 5 boundary (320 bytes) was 8 bytes ago --- */
	u64                        cr4;                  /*   328     8 */
	u64                        cr3;                  /*   336     8 */
	u64                        cr0;                  /*   344     8 */
	u64                        dr7;                  /*   352     8 */
	u64                        dr6;                  /*   360     8 */
	u64                        rflags;               /*   368     8 */
	u64                        rip;                  /*   376     8 */
	/* --- cacheline 6 boundary (384 bytes) --- */
	u8                         reserved_4[88];       /*   384    88 */
	/* --- cacheline 7 boundary (448 bytes) was 24 bytes ago --- */
	u64                        rsp;                  /*   472     8 */
	u8                         reserved_5[24];       /*   480    24 */
	u64                        rax;                  /*   504     8 */
	/* --- cacheline 8 boundary (512 bytes) --- */
	u64                        star;                 /*   512     8 */
	u64                        lstar;                /*   520     8 */
	u64                        cstar;                /*   528     8 */
	u64                        sfmask;               /*   536     8 */
	u64                        kernel_gs_base;       /*   544     8 */
	u64                        sysenter_cs;          /*   552     8 */
	u64                        sysenter_esp;         /*   560     8 */
	u64                        sysenter_eip;         /*   568     8 */
	/* --- cacheline 9 boundary (576 bytes) --- */
	u64                        cr2;                  /*   576     8 */
	u8                         reserved_6[32];       /*   584    32 */
	u64                        g_pat;                /*   616     8 */
	u64                        dbgctl;               /*   624     8 */
	u64                        br_from;              /*   632     8 */
	/* --- cacheline 10 boundary (640 bytes) --- */
	u64                        br_to;                /*   640     8 */
	u64                        last_excp_from;       /*   648     8 */
	u64                        last_excp_to;         /*   656     8 */
	u8                         reserved_7[104];      /*   664   104 */
	/* --- cacheline 12 boundary (768 bytes) --- */
	u64                        reserved_8;           /*   768     8 */
	u64                        rcx;                  /*   776     8 */
	u64                        rdx;                  /*   784     8 */
	u64                        rbx;                  /*   792     8 */
	u64                        reserved_9;           /*   800     8 */
	u64                        rbp;                  /*   808     8 */
	u64                        rsi;                  /*   816     8 */
	u64                        rdi;                  /*   824     8 */
	/* --- cacheline 13 boundary (832 bytes) --- */
	u64                        r8;                   /*   832     8 */
	u64                        r9;                   /*   840     8 */
	u64                        r10;                  /*   848     8 */
	u64                        r11;                  /*   856     8 */
	u64                        r12;                  /*   864     8 */
	u64                        r13;                  /*   872     8 */
	u64                        r14;                  /*   880     8 */
	u64                        r15;                  /*   888     8 */
	/* --- cacheline 14 boundary (896 bytes) --- */
	u8                         reserved_10[16];      /*   896    16 */
	u64                        sw_exit_code;         /*   912     8 */
	u64                        sw_exit_info_1;       /*   920     8 */
	u64                        sw_exit_info_2;       /*   928     8 */
	u64                        sw_scratch;           /*   936     8 */
	u8                         reserved_11[56];      /*   944    56 */
	/* --- cacheline 15 boundary (960 bytes) was 40 bytes ago --- */
	u64                        xcr0;                 /*  1000     8 */
	u8                         valid_bitmap[16];     /*  1008    16 */
	/* --- cacheline 16 boundary (1024 bytes) --- */
	u64                        x87_state_gpa;        /*  1024     8 */

	/* size: 1032, cachelines: 17, members: 68 */
	/* last cacheline: 8 bytes */
};
struct vmware_steal_time {
	union {
		uint64_t           clock;                /*     0     8 */
		struct {
			uint32_t   clock_low;            /*     0     4 */
			uint32_t   clock_high;           /*     4     4 */
		};                                       /*     0     8 */
	};                                               /*     0     8 */
	uint64_t                   reserved[7];          /*     8    56 */

	/* size: 64, cachelines: 1, members: 2 */
};
struct mpc_intsrc {
	unsigned char              type;                 /*     0     1 */
	unsigned char              irqtype;              /*     1     1 */
	short unsigned int         irqflag;              /*     2     2 */
	unsigned char              srcbus;               /*     4     1 */
	unsigned char              srcbusirq;            /*     5     1 */
	unsigned char              dstapic;              /*     6     1 */
	unsigned char              dstirq;               /*     7     1 */

	/* size: 8, cachelines: 1, members: 7 */
	/* last cacheline: 8 bytes */
};
struct acpi_subtable_header {
	u8                         type;                 /*     0     1 */
	u8                         length;               /*     1     1 */

	/* size: 2, cachelines: 1, members: 2 */
	/* last cacheline: 2 bytes */
};
struct acpi_table_boot {
	struct acpi_table_header   header;               /*     0    36 */
	u8                         cmos_index;           /*    36     1 */
	u8                         reserved[3];          /*    37     3 */

	/* size: 40, cachelines: 1, members: 3 */
	/* last cacheline: 40 bytes */
};
struct acpi_hmat_structure {
	u16                        type;                 /*     0     2 */
	u16                        reserved;             /*     2     2 */
	u32                        length;               /*     4     4 */

	/* size: 8, cachelines: 1, members: 3 */
	/* last cacheline: 8 bytes */
};
struct acpi_table_hpet {
	struct acpi_table_header   header;               /*     0    36 */
	u32                        id;                   /*    36     4 */
	struct acpi_generic_address address;             /*    40    12 */
	u8                         sequence;             /*    52     1 */
	u16                        minimum_tick;         /*    53     2 */
	u8                         flags;                /*    55     1 */

	/* size: 56, cachelines: 1, members: 6 */
	/* last cacheline: 56 bytes */
} __attribute__((__packed__));
struct acpi_table_madt {
	struct acpi_table_header   header;               /*     0    36 */
	u32                        address;              /*    36     4 */
	u32                        flags;                /*    40     4 */

	/* size: 44, cachelines: 1, members: 3 */
	/* last cacheline: 44 bytes */
};
struct acpi_madt_local_apic {
	struct acpi_subtable_header header;              /*     0     2 */
	u8                         processor_id;         /*     2     1 */
	u8                         id;                   /*     3     1 */
	u32                        lapic_flags;          /*     4     4 */

	/* size: 8, cachelines: 1, members: 4 */
	/* last cacheline: 8 bytes */
};
struct acpi_madt_io_apic {
	struct acpi_subtable_header header;              /*     0     2 */
	u8                         id;                   /*     2     1 */
	u8                         reserved;             /*     3     1 */
	u32                        address;              /*     4     4 */
	u32                        global_irq_base;      /*     8     4 */

	/* size: 12, cachelines: 1, members: 5 */
	/* last cacheline: 12 bytes */
};
struct acpi_madt_interrupt_override {
	struct acpi_subtable_header header;              /*     0     2 */
	u8                         bus;                  /*     2     1 */
	u8                         source_irq;           /*     3     1 */
	u32                        global_irq;           /*     4     4 */
	u16                        inti_flags;           /*     8     2 */

	/* size: 10, cachelines: 1, members: 5 */
	/* last cacheline: 10 bytes */
} __attribute__((__packed__));
struct acpi_madt_nmi_source {
	struct acpi_subtable_header header;              /*     0     2 */
	u16                        inti_flags;           /*     2     2 */
	u32                        global_irq;           /*     4     4 */

	/* size: 8, cachelines: 1, members: 3 */
	/* last cacheline: 8 bytes */
};
struct acpi_madt_local_apic_nmi {
	struct acpi_subtable_header header;              /*     0     2 */
	u8                         processor_id;         /*     2     1 */
	u16                        inti_flags;           /*     3     2 */
	u8                         lint;                 /*     5     1 */

	/* size: 6, cachelines: 1, members: 4 */
	/* last cacheline: 6 bytes */
} __attribute__((__packed__));
struct acpi_madt_local_apic_override {
	struct acpi_subtable_header header;              /*     0     2 */
	u16                        reserved;             /*     2     2 */
	u64                        address;              /*     4     8 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
} __attribute__((__packed__));
struct acpi_madt_local_sapic {
	struct acpi_subtable_header header;              /*     0     2 */
	u8                         processor_id;         /*     2     1 */
	u8                         id;                   /*     3     1 */
	u8                         eid;                  /*     4     1 */
	u8                         reserved[3];          /*     5     3 */
	u32                        lapic_flags;          /*     8     4 */
	u32                        uid;                  /*    12     4 */
	char                       uid_string[1];        /*    16     1 */

	/* size: 17, cachelines: 1, members: 8 */
	/* last cacheline: 17 bytes */
} __attribute__((__packed__));
struct acpi_madt_local_x2apic {
	struct acpi_subtable_header header;              /*     0     2 */
	u16                        reserved;             /*     2     2 */
	u32                        local_apic_id;        /*     4     4 */
	u32                        lapic_flags;          /*     8     4 */
	u32                        uid;                  /*    12     4 */

	/* size: 16, cachelines: 1, members: 5 */
	/* last cacheline: 16 bytes */
};
struct acpi_madt_local_x2apic_nmi {
	struct acpi_subtable_header header;              /*     0     2 */
	u16                        inti_flags;           /*     2     2 */
	u32                        uid;                  /*     4     4 */
	u8                         lint;                 /*     8     1 */
	u8                         reserved[3];          /*     9     3 */

	/* size: 12, cachelines: 1, members: 5 */
	/* last cacheline: 12 bytes */
};
union acpi_subtable_headers {
	struct acpi_subtable_header common;            /*     0     2 */
	struct acpi_hmat_structure hmat;               /*     0     8 */
};
struct acpi_subtable_proc {
	int                        id;                   /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	acpi_tbl_entry_handler     handler;              /*     8     8 */
	int                        count;                /*    16     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 16, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct ioapic_alloc_info {
	int                        pin;                  /*     0     4 */
	int                        node;                 /*     4     4 */
	u32                        trigger:1;            /*     8: 0  4 */
	u32                        polarity:1;           /*     8: 1  4 */
	u32                        valid:1;              /*     8: 2  4 */

	/* XXX 29 bits hole, try to pack */
	/* XXX 4 bytes hole, try to pack */

	struct IO_APIC_route_entry * entry;              /*    16     8 */

	/* size: 24, cachelines: 1, members: 6 */
	/* sum members: 16, holes: 1, sum holes: 4 */
	/* sum bitfield members: 3 bits, bit holes: 1, sum bit holes: 29 bits */
	/* last cacheline: 24 bytes */
};
struct IO_APIC_route_entry {
	__u32                      vector:8;             /*     0: 0  4 */
	__u32                      delivery_mode:3;      /*     0: 8  4 */
	__u32                      dest_mode:1;          /*     0:11  4 */
	__u32                      delivery_status:1;    /*     0:12  4 */
	__u32                      polarity:1;           /*     0:13  4 */
	__u32                      irr:1;                /*     0:14  4 */
	__u32                      trigger:1;            /*     0:15  4 */
	__u32                      mask:1;               /*     0:16  4 */
	__u32                      __reserved_2:15;      /*     0:17  4 */
	__u32                      __reserved_3:24;      /*     4: 0  4 */
	__u32                      dest:8;               /*     4:24  4 */

	/* size: 8, cachelines: 1, members: 11 */
	/* last cacheline: 8 bytes */
};
struct uv_alloc_info {
	int                        limit;                /*     0     4 */
	int                        blade;                /*     4     4 */
	long unsigned int          offset;               /*     8     8 */
	char *                     name;                 /*    16     8 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct irq_alloc_info {
	enum irq_alloc_type        type;                 /*     0     4 */
	u32                        flags;                /*     4     4 */
	u32                        devid;                /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	irq_hw_number_t            hwirq;                /*    16     8 */
	const struct cpumask  *    mask;                 /*    24     8 */
	struct msi_desc *          desc;                 /*    32     8 */
	void *                     data;                 /*    40     8 */
	union {
		struct ioapic_alloc_info ioapic;         /*    48    24 */
		struct uv_alloc_info uv;                 /*    48    24 */
	};                                               /*    48    24 */

	/* size: 72, cachelines: 2, members: 8 */
	/* sum members: 68, holes: 1, sum holes: 4 */
	/* last cacheline: 8 bytes */
};
struct serial_icounter_struct {
	int                        cts;                  /*     0     4 */
	int                        dsr;                  /*     4     4 */
	int                        rng;                  /*     8     4 */
	int                        dcd;                  /*    12     4 */
	int                        rx;                   /*    16     4 */
	int                        tx;                   /*    20     4 */
	int                        frame;                /*    24     4 */
	int                        overrun;              /*    28     4 */
	int                        parity;               /*    32     4 */
	int                        brk;                  /*    36     4 */
	int                        buf_overrun;          /*    40     4 */
	int                        reserved[9];          /*    44    36 */

	/* size: 80, cachelines: 2, members: 12 */
	/* last cacheline: 16 bytes */
};
struct serial_struct {
	int                        type;                 /*     0     4 */
	int                        line;                 /*     4     4 */
	unsigned int               port;                 /*     8     4 */
	int                        irq;                  /*    12     4 */
	int                        flags;                /*    16     4 */
	int                        xmit_fifo_size;       /*    20     4 */
	int                        custom_divisor;       /*    24     4 */
	int                        baud_base;            /*    28     4 */
	short unsigned int         close_delay;          /*    32     2 */
	char                       io_type;              /*    34     1 */
	char                       reserved_char[1];     /*    35     1 */
	int                        hub6;                 /*    36     4 */
	short unsigned int         closing_wait;         /*    40     2 */
	short unsigned int         closing_wait2;        /*    42     2 */

	/* XXX 4 bytes hole, try to pack */

	unsigned char *            iomem_base;           /*    48     8 */
	short unsigned int         iomem_reg_shift;      /*    56     2 */

	/* XXX 2 bytes hole, try to pack */

	unsigned int               port_high;            /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	long unsigned int          iomap_base;           /*    64     8 */

	/* size: 72, cachelines: 2, members: 18 */
	/* sum members: 66, holes: 2, sum holes: 6 */
	/* last cacheline: 8 bytes */
};
struct ioapic_domain_cfg {
	enum ioapic_domain_type    type;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	const struct irq_domain_ops  * ops;              /*     8     8 */
	struct device_node *       dev;                  /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct wakeup_header {
	u16                        video_mode;           /*     0     2 */
	u32                        pmode_entry;          /*     2     4 */
	u16                        pmode_cs;             /*     6     2 */
	u32                        pmode_cr0;            /*     8     4 */
	u32                        pmode_cr3;            /*    12     4 */
	u32                        pmode_cr4;            /*    16     4 */
	u32                        pmode_efer_low;       /*    20     4 */
	u32                        pmode_efer_high;      /*    24     4 */
	u64                        pmode_gdt;            /*    28     8 */
	u32                        pmode_misc_en_low;    /*    36     4 */
	u32                        pmode_misc_en_high;   /*    40     4 */
	u32                        pmode_behavior;       /*    44     4 */
	u32                        realmode_flags;       /*    48     4 */
	u32                        real_magic;           /*    52     4 */
	u32                        signature;            /*    56     4 */

	/* size: 60, cachelines: 1, members: 15 */
	/* last cacheline: 60 bytes */
} __attribute__((__packed__));
struct acpi_hest_header {
	u16                        type;                 /*     0     2 */
	u16                        source_id;            /*     2     2 */

	/* size: 4, cachelines: 1, members: 2 */
	/* last cacheline: 4 bytes */
};
struct acpi_hest_ia_error_bank {
	u8                         bank_number;          /*     0     1 */
	u8                         clear_status_on_init; /*     1     1 */
	u8                         status_format;        /*     2     1 */
	u8                         reserved;             /*     3     1 */
	u32                        control_register;     /*     4     4 */
	u64                        control_data;         /*     8     8 */
	u32                        status_register;      /*    16     4 */
	u32                        address_register;     /*    20     4 */
	u32                        misc_register;        /*    24     4 */

	/* size: 28, cachelines: 1, members: 9 */
	/* last cacheline: 28 bytes */
} __attribute__((__packed__));
struct acpi_hest_notify {
	u8                         type;                 /*     0     1 */
	u8                         length;               /*     1     1 */
	u16                        config_write_enable;  /*     2     2 */
	u32                        poll_interval;        /*     4     4 */
	u32                        vector;               /*     8     4 */
	u32                        polling_threshold_value; /*    12     4 */
	u32                        polling_threshold_window; /*    16     4 */
	u32                        error_threshold_value; /*    20     4 */
	u32                        error_threshold_window; /*    24     4 */

	/* size: 28, cachelines: 1, members: 9 */
	/* last cacheline: 28 bytes */
};
struct acpi_hest_ia_corrected {
	struct acpi_hest_header    header;               /*     0     4 */
	u16                        reserved1;            /*     4     2 */
	u8                         flags;                /*     6     1 */
	u8                         enabled;              /*     7     1 */
	u32                        records_to_preallocate; /*     8     4 */
	u32                        max_sections_per_record; /*    12     4 */
	struct acpi_hest_notify    notify;               /*    16    28 */
	u8                         num_hardware_banks;   /*    44     1 */
	u8                         reserved2[3];         /*    45     3 */

	/* size: 48, cachelines: 1, members: 9 */
	/* last cacheline: 48 bytes */
};
struct cpc_reg {
	u8                         descriptor;           /*     0     1 */
	u16                        length;               /*     1     2 */
	u8                         space_id;             /*     3     1 */
	u8                         bit_width;            /*     4     1 */
	u8                         bit_offset;           /*     5     1 */
	u8                         access_width;         /*     6     1 */
	u64                        address;              /*     7     8 */

	/* size: 15, cachelines: 1, members: 7 */
	/* last cacheline: 15 bytes */
} __attribute__((__packed__));
struct acpi_power_register {
	u8                         descriptor;           /*     0     1 */
	u16                        length;               /*     1     2 */
	u8                         space_id;             /*     3     1 */
	u8                         bit_width;            /*     4     1 */
	u8                         bit_offset;           /*     5     1 */
	u8                         access_size;          /*     6     1 */
	u64                        address;              /*     7     8 */

	/* size: 15, cachelines: 1, members: 7 */
	/* last cacheline: 15 bytes */
} __attribute__((__packed__));
struct acpi_processor_cx {
	u8                         valid;                /*     0     1 */
	u8                         type;                 /*     1     1 */

	/* XXX 2 bytes hole, try to pack */

	u32                        address;              /*     4     4 */
	u8                         entry_method;         /*     8     1 */
	u8                         index;                /*     9     1 */

	/* XXX 2 bytes hole, try to pack */

	u32                        latency;              /*    12     4 */
	u8                         bm_sts_skip;          /*    16     1 */
	char                       desc[32];             /*    17    32 */

	/* size: 52, cachelines: 1, members: 8 */
	/* sum members: 45, holes: 2, sum holes: 4 */
	/* padding: 3 */
	/* last cacheline: 52 bytes */
};
struct acpi_processor_flags {
	u8                         power:1;              /*     0: 0  1 */
	u8                         performance:1;        /*     0: 1  1 */
	u8                         throttling:1;         /*     0: 2  1 */
	u8                         limit:1;              /*     0: 3  1 */
	u8                         bm_control:1;         /*     0: 4  1 */
	u8                         bm_check:1;           /*     0: 5  1 */
	u8                         has_cst:1;            /*     0: 6  1 */
	u8                         has_lpi:1;            /*     0: 7  1 */
	u8                         power_setup_done:1;   /*     1: 0  1 */
	u8                         bm_rld_set:1;         /*     1: 1  1 */
	u8                         need_hotplug_init:1;  /*     1: 2  1 */

	/* size: 2, cachelines: 1, members: 11 */
	/* bit_padding: 5 bits */
	/* last cacheline: 2 bytes */
};
struct cstate_entry {
	struct {
		unsigned int       eax;                  /*     0     4 */
		unsigned int       ecx;                  /*     4     4 */
	} states[8]; /*     0    64 */

	/* size: 64, cachelines: 1, members: 1 */
};
struct cpuid_regs_done {
	struct cpuid_regs          regs;                 /*     0    16 */
	struct completion          done;                 /*    16    32 */

	/* size: 48, cachelines: 1, members: 2 */
	/* last cacheline: 48 bytes */
};
struct intel_early_ops {
	resource_size_t            (*stolen_size)(int, int, int); /*     0     8 */
	resource_size_t            (*stolen_base)(int, int, int, resource_size_t); /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct chipset {
	u32                        vendor;               /*     0     4 */
	u32                        device;               /*     4     4 */
	u32                        class;                /*     8     4 */
	u32                        class_mask;           /*    12     4 */
	u32                        flags;                /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	void                       (*f)(int, int, int);  /*    24     8 */

	/* size: 32, cachelines: 1, members: 6 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct sched_domain_shared {
	atomic_t                   ref;                  /*     0     4 */
	atomic_t                   nr_busy_cpus;         /*     4     4 */
	int                        has_idle_cores;       /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct sparsemask *        cfs_overload_cpus;    /*    16     8 */
	int                        nr_idle_scan;         /*    24     4 */

	/* size: 32, cachelines: 1, members: 5 */
	/* sum members: 24, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct sched_domain {
	struct sched_domain *      parent;               /*     0     8 */
	struct sched_domain *      child;                /*     8     8 */
	struct sched_group *       groups;               /*    16     8 */
	long unsigned int          min_interval;         /*    24     8 */
	long unsigned int          max_interval;         /*    32     8 */
	unsigned int               busy_factor;          /*    40     4 */
	unsigned int               imbalance_pct;        /*    44     4 */
	unsigned int               cache_nice_tries;     /*    48     4 */
	int                        nohz_idle;            /*    52     4 */
	int                        flags;                /*    56     4 */
	int                        level;                /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	long unsigned int          last_balance;         /*    64     8 */
	unsigned int               balance_interval;     /*    72     4 */
	unsigned int               nr_balance_failed;    /*    76     4 */
	u64                        max_newidle_lb_cost;  /*    80     8 */
	long unsigned int          next_decay_max_lb_cost; /*    88     8 */
	u64                        avg_scan_cost;        /*    96     8 */
	unsigned int               lb_count[3];          /*   104    12 */
	unsigned int               lb_failed[3];         /*   116    12 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	unsigned int               lb_balanced[3];       /*   128    12 */
	unsigned int               lb_imbalance[3];      /*   140    12 */
	unsigned int               lb_gained[3];         /*   152    12 */
	unsigned int               lb_hot_gained[3];     /*   164    12 */
	unsigned int               lb_nobusyg[3];        /*   176    12 */
	unsigned int               lb_nobusyq[3];        /*   188    12 */
	/* --- cacheline 3 boundary (192 bytes) was 8 bytes ago --- */
	unsigned int               alb_count;            /*   200     4 */
	unsigned int               alb_failed;           /*   204     4 */
	unsigned int               alb_pushed;           /*   208     4 */
	unsigned int               sbe_count;            /*   212     4 */
	unsigned int               sbe_balanced;         /*   216     4 */
	unsigned int               sbe_pushed;           /*   220     4 */
	unsigned int               sbf_count;            /*   224     4 */
	unsigned int               sbf_balanced;         /*   228     4 */
	unsigned int               sbf_pushed;           /*   232     4 */
	unsigned int               ttwu_wake_remote;     /*   236     4 */
	unsigned int               ttwu_move_affine;     /*   240     4 */
	unsigned int               ttwu_move_balance;    /*   244     4 */
	char *                     name;                 /*   248     8 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	union {
		void *             private;              /*   256     8 */
		struct callback_head rcu __attribute__((__aligned__(8))); /*   256    16 */
	} __attribute__((__aligned__(8)));               /*   256    16 */
	struct sched_domain_shared * shared;             /*   272     8 */
	u64                        kabi_reserved1;       /*   280     8 */
	u64                        kabi_reserved2;       /*   288     8 */
	unsigned int               span_weight;          /*   296     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          span[];               /*   304     0 */

	/* size: 304, cachelines: 5, members: 44 */
	/* sum members: 300, holes: 1, sum holes: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct sd_data {
	struct sched_domain * *    sd;                   /*     0     8 */
	struct sched_domain_shared * * sds;              /*     8     8 */
	struct sched_group * *     sg;                   /*    16     8 */
	struct sched_group_capacity * * sgc;             /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct sched_domain_topology_level {
	sched_domain_mask_f        mask;                 /*     0     8 */
	sched_domain_flags_f       sd_flags;             /*     8     8 */
	int                        flags;                /*    16     4 */
	int                        numa_level;           /*    20     4 */
	struct sd_data             data;                 /*    24    32 */
	char *                     name;                 /*    56     8 */

	/* size: 64, cachelines: 1, members: 6 */
};
struct tsc_adjust {
	s64                        bootval;              /*     0     8 */
	s64                        adjusted;             /*     8     8 */
	long unsigned int          nextcheck;            /*    16     8 */
	bool                       warned;               /*    24     1 */

	/* size: 32, cachelines: 1, members: 4 */
	/* padding: 7 */
	/* last cacheline: 32 bytes */
};
struct mpf_intel {
	char                       signature[4];         /*     0     4 */
	unsigned int               physptr;              /*     4     4 */
	unsigned char              length;               /*     8     1 */
	unsigned char              specification;        /*     9     1 */
	unsigned char              checksum;             /*    10     1 */
	unsigned char              feature1;             /*    11     1 */
	unsigned char              feature2;             /*    12     1 */
	unsigned char              feature3;             /*    13     1 */
	unsigned char              feature4;             /*    14     1 */
	unsigned char              feature5;             /*    15     1 */

	/* size: 16, cachelines: 1, members: 10 */
	/* last cacheline: 16 bytes */
};
struct mpc_table {
	char                       signature[4];         /*     0     4 */
	short unsigned int         length;               /*     4     2 */
	char                       spec;                 /*     6     1 */
	char                       checksum;             /*     7     1 */
	char                       oem[8];               /*     8     8 */
	char                       productid[12];        /*    16    12 */
	unsigned int               oemptr;               /*    28     4 */
	short unsigned int         oemsize;              /*    32     2 */
	short unsigned int         oemcount;             /*    34     2 */
	unsigned int               lapic;                /*    36     4 */
	unsigned int               reserved;             /*    40     4 */

	/* size: 44, cachelines: 1, members: 11 */
	/* last cacheline: 44 bytes */
};
struct mpc_cpu {
	unsigned char              type;                 /*     0     1 */
	unsigned char              apicid;               /*     1     1 */
	unsigned char              apicver;              /*     2     1 */
	unsigned char              cpuflag;              /*     3     1 */
	unsigned int               cpufeature;           /*     4     4 */
	unsigned int               featureflag;          /*     8     4 */
	unsigned int               reserved[2];          /*    12     8 */

	/* size: 20, cachelines: 1, members: 7 */
	/* last cacheline: 20 bytes */
};
struct mpc_bus {
	unsigned char              type;                 /*     0     1 */
	unsigned char              busid;                /*     1     1 */
	unsigned char              bustype[6];           /*     2     6 */

	/* size: 8, cachelines: 1, members: 3 */
	/* last cacheline: 8 bytes */
};
struct mpc_ioapic {
	unsigned char              type;                 /*     0     1 */
	unsigned char              apicid;               /*     1     1 */
	unsigned char              apicver;              /*     2     1 */
	unsigned char              flags;                /*     3     1 */
	unsigned int               apicaddr;             /*     4     4 */

	/* size: 8, cachelines: 1, members: 5 */
	/* last cacheline: 8 bytes */
};
struct mpc_lintsrc {
	unsigned char              type;                 /*     0     1 */
	unsigned char              irqtype;              /*     1     1 */
	short unsigned int         irqflag;              /*     2     2 */
	unsigned char              srcbusid;             /*     4     1 */
	unsigned char              srcbusirq;            /*     5     1 */
	unsigned char              destapic;             /*     6     1 */
	unsigned char              destapiclint;         /*     7     1 */

	/* size: 8, cachelines: 1, members: 7 */
	/* last cacheline: 8 bytes */
};
union apic_ir {
	long unsigned int          map[4];             /*     0    32 */
	u32                        regs[8];            /*     0    32 */
};
struct irq_cfg {
	unsigned int               dest_apicid;          /*     0     4 */
	unsigned int               vector;               /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct apic_chip_data {
	struct irq_cfg             hw_irq_cfg;           /*     0     8 */
	unsigned int               vector;               /*     8     4 */
	unsigned int               prev_vector;          /*    12     4 */
	unsigned int               cpu;                  /*    16     4 */
	unsigned int               prev_cpu;             /*    20     4 */
	unsigned int               irq;                  /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	struct hlist_node          clist;                /*    32    16 */
	unsigned int               move_in_progress:1;   /*    48: 0  4 */
	unsigned int               is_managed:1;         /*    48: 1  4 */
	unsigned int               can_reserve:1;        /*    48: 2  4 */
	unsigned int               has_reserved:1;       /*    48: 3  4 */

	/* size: 56, cachelines: 1, members: 11 */
	/* sum members: 44, holes: 1, sum holes: 4 */
	/* sum bitfield members: 4 bits (0 bytes) */
	/* padding: 4 */
	/* bit_padding: 28 bits */
	/* last cacheline: 56 bytes */
};
union IO_APIC_reg_00 {
	u32                        raw;                /*     0     4 */
	struct {
		u32                __reserved_2:14;    /*     0: 0  4 */
		u32                LTS:1;              /*     0:14  4 */
		u32                delivery_type:1;    /*     0:15  4 */
		u32                __reserved_1:8;     /*     0:16  4 */
		u32                ID:8;               /*     0:24  4 */
	} bits;                                        /*     0     4 */
};
union IO_APIC_reg_01 {
	u32                        raw;                /*     0     4 */
	struct {
		u32                version:8;          /*     0: 0  4 */
		u32                __reserved_2:7;     /*     0: 8  4 */
		u32                PRQ:1;              /*     0:15  4 */
		u32                entries:8;          /*     0:16  4 */
		u32                __reserved_1:8;     /*     0:24  4 */
	} bits;                                        /*     0     4 */
};
union IO_APIC_reg_02 {
	u32                        raw;                /*     0     4 */
	struct {
		u32                __reserved_2:24;    /*     0: 0  4 */
		u32                arbitration:4;      /*     0:24  4 */
		u32                __reserved_1:4;     /*     0:28  4 */
	} bits;                                        /*     0     4 */
};
union IO_APIC_reg_03 {
	u32                        raw;                /*     0     4 */
	struct {
		u32                boot_DT:1;          /*     0: 0  4 */
		u32                __reserved_1:31;    /*     0: 1  4 */
	} bits;                                        /*     0     4 */
};
struct IR_IO_APIC_route_entry {
	__u64                      vector:8;             /*     0: 0  8 */
	__u64                      zero:3;               /*     0: 8  8 */
	__u64                      index2:1;             /*     0:11  8 */
	__u64                      delivery_status:1;    /*     0:12  8 */
	__u64                      polarity:1;           /*     0:13  8 */
	__u64                      irr:1;                /*     0:14  8 */
	__u64                      trigger:1;            /*     0:15  8 */
	__u64                      mask:1;               /*     0:16  8 */
	__u64                      reserved:31;          /*     0:17  8 */
	__u64                      format:1;             /*     0:48  8 */
	__u64                      index:15;             /*     0:49  8 */

	/* size: 8, cachelines: 1, members: 11 */
	/* last cacheline: 8 bytes */
};
struct irq_pin_list {
	struct list_head           list;                 /*     0    16 */
	int                        apic;                 /*    16     4 */
	int                        pin;                  /*    20     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct mp_chip_data {
	struct list_head           irq_2_pin;            /*     0    16 */
	struct IO_APIC_route_entry entry;                /*    16     8 */
	int                        trigger;              /*    24     4 */
	int                        polarity;             /*    28     4 */
	u32                        count;                /*    32     4 */
	bool                       isa_irq;              /*    36     1 */

	/* size: 40, cachelines: 1, members: 6 */
	/* padding: 3 */
	/* last cacheline: 40 bytes */
};
struct mp_ioapic_gsi {
	u32                        gsi_base;             /*     0     4 */
	u32                        gsi_end;              /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct ioapic {
	int                        nr_registers;         /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct IO_APIC_route_entry * saved_registers;    /*     8     8 */
	struct mpc_ioapic          mp_config;            /*    16     8 */
	struct mp_ioapic_gsi       gsi_config;           /*    24     8 */
	struct ioapic_domain_cfg   irqdomain_cfg;        /*    32    24 */
	struct irq_domain *        irqdomain;            /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct resource *          iomem_res;            /*    64     8 */

	/* size: 72, cachelines: 2, members: 7 */
	/* sum members: 68, holes: 1, sum holes: 4 */
	/* last cacheline: 8 bytes */
};
struct io_apic {
	unsigned int               index;                /*     0     4 */
	unsigned int               unused[3];            /*     4    12 */
	unsigned int               data;                 /*    16     4 */
	unsigned int               unused2[11];          /*    20    44 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	unsigned int               eoi;                  /*    64     4 */

	/* size: 68, cachelines: 2, members: 5 */
	/* last cacheline: 4 bytes */
};
union entry_union {
	struct {
		u32                w1;                 /*     0     4 */
		u32                w2;                 /*     4     4 */
	};                                             /*     0     8 */
	struct IO_APIC_route_entry entry;              /*     0     8 */
};
struct msi_domain_ops {
	irq_hw_number_t            (*get_hwirq)(struct msi_domain_info *, msi_alloc_info_t *); /*     0     8 */
	int                        (*msi_init)(struct irq_domain *, struct msi_domain_info *, unsigned int, irq_hw_number_t, msi_alloc_info_t *); /*     8     8 */
	void                       (*msi_free)(struct irq_domain *, struct msi_domain_info *, unsigned int); /*    16     8 */
	int                        (*msi_check)(struct irq_domain *, struct msi_domain_info *, struct device *); /*    24     8 */
	int                        (*msi_prepare)(struct irq_domain *, struct device *, int, msi_alloc_info_t *); /*    32     8 */
	void                       (*msi_finish)(msi_alloc_info_t *, int); /*    40     8 */
	void                       (*set_desc)(msi_alloc_info_t *, struct msi_desc *); /*    48     8 */
	int                        (*handle_error)(struct irq_domain *, struct msi_desc *, int); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*domain_alloc_irqs)(struct irq_domain *, struct device *, int); /*    64     8 */
	void                       (*domain_free_irqs)(struct irq_domain *, struct device *); /*    72     8 */

	/* size: 80, cachelines: 2, members: 10 */
	/* last cacheline: 16 bytes */
};
struct msi_domain_info {
	u32                        flags;                /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct msi_domain_ops *    ops;                  /*     8     8 */
	struct irq_chip *          chip;                 /*    16     8 */
	void *                     chip_data;            /*    24     8 */
	irq_flow_handler_t         handler;              /*    32     8 */
	void *                     handler_data;         /*    40     8 */
	const char  *              handler_name;         /*    48     8 */
	void *                     data;                 /*    56     8 */

	/* size: 64, cachelines: 1, members: 8 */
	/* sum members: 60, holes: 1, sum holes: 4 */
};
struct acpi_device {
	int                        device_type;          /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	acpi_handle                handle;               /*     8     8 */
	struct fwnode_handle       fwnode;               /*    16    40 */
	struct acpi_device *       parent;               /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct list_head           children;             /*    64    16 */
	struct list_head           node;                 /*    80    16 */
	struct list_head           wakeup_list;          /*    96    16 */
	struct list_head           del_list;             /*   112    16 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct acpi_device_status  status;               /*   128     4 */
	struct acpi_device_flags   flags;                /*   132     4 */
	struct acpi_device_pnp     pnp;                  /*   136   120 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	struct acpi_device_power   power;                /*   256   176 */
	/* --- cacheline 6 boundary (384 bytes) was 48 bytes ago --- */
	struct acpi_device_wakeup  wakeup;               /*   432    80 */
	/* --- cacheline 8 boundary (512 bytes) --- */
	struct acpi_device_perf    performance;          /*   512    24 */
	struct acpi_device_dir     dir;                  /*   536     8 */
	struct acpi_device_data    data;                 /*   544    48 */
	/* --- cacheline 9 boundary (576 bytes) was 16 bytes ago --- */
	struct acpi_scan_handler * handler;              /*   592     8 */
	struct acpi_hotplug_context * hp;                /*   600     8 */
	struct acpi_driver *       driver;               /*   608     8 */
	const struct acpi_gpio_mapping  * driver_gpios;  /*   616     8 */
	void *                     driver_data;          /*   624     8 */
	struct device              dev __attribute__((__aligned__(8))); /*   632  1000 */
	/* --- cacheline 25 boundary (1600 bytes) was 32 bytes ago --- */
	unsigned int               physical_node_count;  /*  1632     4 */
	unsigned int               dep_unmet;            /*  1636     4 */
	struct list_head           physical_node_list;   /*  1640    16 */
	struct mutex               physical_node_lock;   /*  1656    32 */
	/* --- cacheline 26 boundary (1664 bytes) was 24 bytes ago --- */
	void                       (*remove)(struct acpi_device *); /*  1688     8 */

	/* size: 1696, cachelines: 27, members: 27 */
	/* sum members: 1692, holes: 1, sum holes: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
union acpi_object {
	acpi_object_type           type;               /*     0     4 */
	struct {
		acpi_object_type   type;               /*     0     4 */

		/* XXX 4 bytes hole, try to pack */

		u64                value;              /*     8     8 */
	} integer;                                     /*     0    16 */
	struct {
		acpi_object_type   type;               /*     0     4 */
		u32                length;             /*     4     4 */
		char *             pointer;            /*     8     8 */
	} string;                                      /*     0    16 */
	struct {
		acpi_object_type   type;               /*     0     4 */
		u32                length;             /*     4     4 */
		u8 *               pointer;            /*     8     8 */
	} buffer;                                      /*     0    16 */
	struct {
		acpi_object_type   type;               /*     0     4 */
		u32                count;              /*     4     4 */
		union acpi_object * elements;          /*     8     8 */
	} package;                                     /*     0    16 */
	struct {
		acpi_object_type   type;               /*     0     4 */
		acpi_object_type   actual_type;        /*     4     4 */
		acpi_handle        handle;             /*     8     8 */
	} reference;                                   /*     0    16 */
	struct {
		acpi_object_type   type;               /*     0     4 */
		u32                proc_id;            /*     4     4 */
		acpi_io_address    pblk_address;       /*     8     8 */
		u32                pblk_length;        /*    16     4 */
	} processor;                                   /*     0    24 */
	struct {
		acpi_object_type   type;               /*     0     4 */
		u32                system_level;       /*     4     4 */
		u32                resource_order;     /*     8     4 */
	} power_resource;                              /*     0    12 */
};
struct acpi_hotplug_profile {
	struct kobject             kobj;                 /*     0    96 */
	/* --- cacheline 1 boundary (64 bytes) was 32 bytes ago --- */
	int                        (*scan_dependent)(struct acpi_device *); /*    96     8 */
	void                       (*notify_online)(struct acpi_device *); /*   104     8 */
	bool                       enabled:1;            /*   112: 0  1 */
	bool                       demand_offline:1;     /*   112: 1  1 */

	/* size: 120, cachelines: 2, members: 5 */
	/* padding: 7 */
	/* bit_padding: 6 bits */
	/* last cacheline: 56 bytes */
};
struct acpi_scan_handler {
	const struct acpi_device_id  * ids;              /*     0     8 */
	struct list_head           list_node;            /*     8    16 */
	bool                       (*match)(const char  *, const struct acpi_device_id  * *); /*    24     8 */
	int                        (*attach)(struct acpi_device *, const struct acpi_device_id  *); /*    32     8 */
	void                       (*detach)(struct acpi_device *); /*    40     8 */
	void                       (*bind)(struct device *); /*    48     8 */
	void                       (*unbind)(struct device *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct acpi_hotplug_profile hotplug;             /*    64   120 */

	/* XXX last struct has 7 bytes of padding */

	/* size: 184, cachelines: 3, members: 8 */
	/* paddings: 1, sum paddings: 7 */
	/* last cacheline: 56 bytes */
};
struct acpi_hotplug_context {
	struct acpi_device *       self;                 /*     0     8 */
	int                        (*notify)(struct acpi_device *, u32); /*     8     8 */
	void                       (*uevent)(struct acpi_device *, u32); /*    16     8 */
	void                       (*fixup)(struct acpi_device *); /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct acpi_device_ops {
	acpi_op_add                add;                  /*     0     8 */
	acpi_op_remove             remove;               /*     8     8 */
	acpi_op_notify             notify;               /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct acpi_driver {
	char                       name[80];             /*     0    80 */
	/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
	char                       class[80];            /*    80    80 */
	/* --- cacheline 2 boundary (128 bytes) was 32 bytes ago --- */
	const struct acpi_device_id  * ids;              /*   160     8 */
	unsigned int               flags;                /*   168     4 */

	/* XXX 4 bytes hole, try to pack */

	struct acpi_device_ops     ops;                  /*   176    24 */
	/* --- cacheline 3 boundary (192 bytes) was 8 bytes ago --- */
	struct device_driver       drv;                  /*   200   192 */
	/* --- cacheline 6 boundary (384 bytes) was 8 bytes ago --- */
	struct module *            owner;                /*   392     8 */

	/* size: 400, cachelines: 7, members: 7 */
	/* sum members: 396, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct acpi_device_status {
	u32                        present:1;            /*     0: 0  4 */
	u32                        enabled:1;            /*     0: 1  4 */
	u32                        show_in_ui:1;         /*     0: 2  4 */
	u32                        functional:1;         /*     0: 3  4 */
	u32                        battery_present:1;    /*     0: 4  4 */
	u32                        reserved:27;          /*     0: 5  4 */

	/* size: 4, cachelines: 1, members: 6 */
	/* last cacheline: 4 bytes */
};
struct acpi_device_flags {
	u32                        dynamic_status:1;     /*     0: 0  4 */
	u32                        removable:1;          /*     0: 1  4 */
	u32                        ejectable:1;          /*     0: 2  4 */
	u32                        power_manageable:1;   /*     0: 3  4 */
	u32                        match_driver:1;       /*     0: 4  4 */
	u32                        initialized:1;        /*     0: 5  4 */
	u32                        visited:1;            /*     0: 6  4 */
	u32                        hotplug_notify:1;     /*     0: 7  4 */
	u32                        is_dock_station:1;    /*     0: 8  4 */
	u32                        of_compatible_ok:1;   /*     0: 9  4 */
	u32                        coherent_dma:1;       /*     0:10  4 */
	u32                        cca_seen:1;           /*     0:11  4 */
	u32                        enumeration_by_parent:1; /*     0:12  4 */
	u32                        reserved:19;          /*     0:13  4 */

	/* size: 4, cachelines: 1, members: 14 */
	/* last cacheline: 4 bytes */
};
struct acpi_device_dir {
	struct proc_dir_entry *    entry;                /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct acpi_pnp_type {
	u32                        hardware_id:1;        /*     0: 0  4 */
	u32                        bus_address:1;        /*     0: 1  4 */
	u32                        platform_id:1;        /*     0: 2  4 */
	u32                        reserved:29;          /*     0: 3  4 */

	/* size: 4, cachelines: 1, members: 4 */
	/* last cacheline: 4 bytes */
};
struct acpi_device_pnp {
	acpi_bus_id                bus_id;               /*     0     8 */
	int                        instance_no;          /*     8     4 */
	struct acpi_pnp_type       type;                 /*    12     4 */
	acpi_bus_address           bus_address;          /*    16     8 */
	char *                     unique_id;            /*    24     8 */
	struct list_head           ids;                  /*    32    16 */
	acpi_device_name           device_name;          /*    48    40 */
	/* --- cacheline 1 boundary (64 bytes) was 24 bytes ago --- */
	acpi_device_class          device_class;         /*    88    20 */

	/* XXX 4 bytes hole, try to pack */

	union acpi_object *        str_obj;              /*   112     8 */

	/* size: 120, cachelines: 2, members: 9 */
	/* sum members: 116, holes: 1, sum holes: 4 */
	/* last cacheline: 56 bytes */
};
struct acpi_device_power_flags {
	u32                        explicit_get:1;       /*     0: 0  4 */
	u32                        power_resources:1;    /*     0: 1  4 */
	u32                        inrush_current:1;     /*     0: 2  4 */
	u32                        power_removed:1;      /*     0: 3  4 */
	u32                        ignore_parent:1;      /*     0: 4  4 */
	u32                        dsw_present:1;        /*     0: 5  4 */
	u32                        reserved:26;          /*     0: 6  4 */

	/* size: 4, cachelines: 1, members: 7 */
	/* last cacheline: 4 bytes */
};
struct acpi_device_power_state {
	struct {
		u8                 valid:1;              /*     0: 0  1 */
		u8                 explicit_set:1;       /*     0: 1  1 */
		u8                 reserved:6;           /*     0: 2  1 */
	} flags;                                         /*     0     1 */

	/* XXX 3 bytes hole, try to pack */

	int                        power;                /*     4     4 */
	int                        latency;              /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           resources;            /*    16    16 */

	/* size: 32, cachelines: 1, members: 4 */
	/* sum members: 25, holes: 2, sum holes: 7 */
	/* last cacheline: 32 bytes */
};
struct acpi_device_power {
	int                        state;                /*     0     4 */
	struct acpi_device_power_flags flags;            /*     4     4 */
	struct acpi_device_power_state states[5];        /*     8   160 */
	/* --- cacheline 2 boundary (128 bytes) was 40 bytes ago --- */
	u64                        kabi_reserved1;       /*   168     8 */

	/* size: 176, cachelines: 3, members: 4 */
	/* last cacheline: 48 bytes */
};
struct acpi_device_perf_flags {
	u8                         reserved:8;           /*     0: 0  1 */

	/* size: 1, cachelines: 1, members: 1 */
	/* last cacheline: 1 bytes */
};
struct acpi_device_perf_state {
	struct {
		u8                 valid:1;              /*     0: 0  1 */
		u8                 reserved:7;           /*     0: 1  1 */
	} flags;                                         /*     0     1 */
	u8                         power;                /*     1     1 */
	u8                         performance;          /*     2     1 */

	/* XXX 1 byte hole, try to pack */

	int                        latency;              /*     4     4 */

	/* size: 8, cachelines: 1, members: 4 */
	/* sum members: 7, holes: 1, sum holes: 1 */
	/* last cacheline: 8 bytes */
};
struct acpi_device_perf {
	int                        state;                /*     0     4 */
	struct acpi_device_perf_flags flags;             /*     4     1 */

	/* XXX 3 bytes hole, try to pack */

	int                        state_count;          /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct acpi_device_perf_state * states;          /*    16     8 */

	/* size: 24, cachelines: 1, members: 4 */
	/* sum members: 17, holes: 2, sum holes: 7 */
	/* last cacheline: 24 bytes */
};
struct acpi_device_wakeup_flags {
	u8                         valid:1;              /*     0: 0  1 */
	u8                         notifier_present:1;   /*     0: 1  1 */

	/* size: 1, cachelines: 1, members: 2 */
	/* bit_padding: 6 bits */
	/* last cacheline: 1 bytes */
};
struct acpi_device_wakeup_context {
	void                       (*func)(struct acpi_device_wakeup_context *); /*     0     8 */
	struct device *            dev;                  /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct acpi_device_wakeup {
	acpi_handle                gpe_device;           /*     0     8 */
	u64                        gpe_number;           /*     8     8 */
	u64                        sleep_state;          /*    16     8 */
	struct list_head           resources;            /*    24    16 */
	struct acpi_device_wakeup_flags flags;           /*    40     1 */

	/* XXX 7 bytes hole, try to pack */

	struct acpi_device_wakeup_context context;       /*    48    16 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct wakeup_source *     ws;                   /*    64     8 */
	int                        prepare_count;        /*    72     4 */
	int                        enable_count;         /*    76     4 */

	/* size: 80, cachelines: 2, members: 9 */
	/* sum members: 73, holes: 1, sum holes: 7 */
	/* last cacheline: 16 bytes */
};
struct acpi_device_data {
	const union acpi_object  * pointer;              /*     0     8 */
	struct list_head           properties;           /*     8    16 */
	const union acpi_object  * of_compatible;        /*    24     8 */
	struct list_head           subnodes;             /*    32    16 */

	/* size: 48, cachelines: 1, members: 4 */
	/* last cacheline: 48 bytes */
};
struct acpi_gpio_mapping {
	const char  *              name;                 /*     0     8 */
	const struct acpi_gpio_params  * data;           /*     8     8 */
	unsigned int               size;                 /*    16     4 */
	unsigned int               quirks;               /*    20     4 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct acpi_gpio_params {
	unsigned int               crs_entry_index;      /*     0     4 */
	unsigned int               line_index;           /*     4     4 */
	bool                       active_low;           /*     8     1 */

	/* size: 12, cachelines: 1, members: 3 */
	/* padding: 3 */
	/* last cacheline: 12 bytes */
};
struct uvyh_gr0_gam_gr_config_s {
	long unsigned int          rsvd_0_9:10;          /*     0: 0  8 */
	long unsigned int          subspace:1;           /*     0:10  8 */
	long unsigned int          rsvd_11_63:53;        /*     0:11  8 */

	/* size: 8, cachelines: 1, members: 3 */
	/* last cacheline: 8 bytes */
};
struct uv5h_gr0_gam_gr_config_s {
	long unsigned int          rsvd_0_9:10;          /*     0: 0  8 */
	long unsigned int          subspace:1;           /*     0:10  8 */
	long unsigned int          rsvd_11_63:53;        /*     0:11  8 */

	/* size: 8, cachelines: 1, members: 3 */
	/* last cacheline: 8 bytes */
};
struct uv4h_gr0_gam_gr_config_s {
	long unsigned int          rsvd_0_9:10;          /*     0: 0  8 */
	long unsigned int          subspace:1;           /*     0:10  8 */
	long unsigned int          rsvd_11_63:53;        /*     0:11  8 */

	/* size: 8, cachelines: 1, members: 3 */
	/* last cacheline: 8 bytes */
};
struct uv3h_gr0_gam_gr_config_s {
	long unsigned int          m_skt:6;              /*     0: 0  8 */
	long unsigned int          undef_6_9:4;          /*     0: 6  8 */
	long unsigned int          subspace:1;           /*     0:10  8 */
	long unsigned int          reserved:53;          /*     0:11  8 */

	/* size: 8, cachelines: 1, members: 4 */
	/* last cacheline: 8 bytes */
};
struct uv2h_gr0_gam_gr_config_s {
	long unsigned int          n_gr:4;               /*     0: 0  8 */
	long unsigned int          reserved:60;          /*     0: 4  8 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
union uvyh_gr0_gam_gr_config_u {
	long unsigned int          v;                  /*     0     8 */
	struct uvyh_gr0_gam_gr_config_s sy;            /*     0     8 */
	struct uv5h_gr0_gam_gr_config_s s5;            /*     0     8 */
	struct uv4h_gr0_gam_gr_config_s s4;            /*     0     8 */
	struct uv3h_gr0_gam_gr_config_s s3;            /*     0     8 */
	struct uv2h_gr0_gam_gr_config_s s2;            /*     0     8 */
};
struct uvh_node_id_s {
	long unsigned int          force1:1;             /*     0: 0  8 */
	long unsigned int          manufacturer:11;      /*     0: 1  8 */
	long unsigned int          part_number:16;       /*     0:12  8 */
	long unsigned int          revision:4;           /*     0:28  8 */
	long unsigned int          rsvd_32_63:32;        /*     0:32  8 */

	/* size: 8, cachelines: 1, members: 5 */
	/* last cacheline: 8 bytes */
};
struct uvxh_node_id_s {
	long unsigned int          force1:1;             /*     0: 0  8 */
	long unsigned int          manufacturer:11;      /*     0: 1  8 */
	long unsigned int          part_number:16;       /*     0:12  8 */
	long unsigned int          revision:4;           /*     0:28  8 */
	long unsigned int          node_id:15;           /*     0:32  8 */
	long unsigned int          rsvd_47_49:3;         /*     0:47  8 */
	long unsigned int          nodes_per_bit:7;      /*     0:50  8 */
	long unsigned int          ni_port:5;            /*     0:57  8 */
	long unsigned int          rsvd_62_63:2;         /*     0:62  8 */

	/* size: 8, cachelines: 1, members: 9 */
	/* last cacheline: 8 bytes */
};
struct uvyh_node_id_s {
	long unsigned int          force1:1;             /*     0: 0  8 */
	long unsigned int          manufacturer:11;      /*     0: 1  8 */
	long unsigned int          part_number:16;       /*     0:12  8 */
	long unsigned int          revision:4;           /*     0:28  8 */
	long unsigned int          node_id:7;            /*     0:32  8 */
	long unsigned int          rsvd_39_56:18;        /*     0:39  8 */
	long unsigned int          ni_port:6;            /*     0:57  8 */
	long unsigned int          rsvd_63:1;            /*     0:63  8 */

	/* size: 8, cachelines: 1, members: 8 */
	/* last cacheline: 8 bytes */
};
struct uv5h_node_id_s {
	long unsigned int          force1:1;             /*     0: 0  8 */
	long unsigned int          manufacturer:11;      /*     0: 1  8 */
	long unsigned int          part_number:16;       /*     0:12  8 */
	long unsigned int          revision:4;           /*     0:28  8 */
	long unsigned int          node_id:7;            /*     0:32  8 */
	long unsigned int          rsvd_39_56:18;        /*     0:39  8 */
	long unsigned int          ni_port:6;            /*     0:57  8 */
	long unsigned int          rsvd_63:1;            /*     0:63  8 */

	/* size: 8, cachelines: 1, members: 8 */
	/* last cacheline: 8 bytes */
};
struct uv4h_node_id_s {
	long unsigned int          force1:1;             /*     0: 0  8 */
	long unsigned int          manufacturer:11;      /*     0: 1  8 */
	long unsigned int          part_number:16;       /*     0:12  8 */
	long unsigned int          revision:4;           /*     0:28  8 */
	long unsigned int          node_id:15;           /*     0:32  8 */
	long unsigned int          rsvd_47:1;            /*     0:47  8 */
	long unsigned int          router_select:1;      /*     0:48  8 */
	long unsigned int          rsvd_49:1;            /*     0:49  8 */
	long unsigned int          nodes_per_bit:7;      /*     0:50  8 */
	long unsigned int          ni_port:5;            /*     0:57  8 */
	long unsigned int          rsvd_62_63:2;         /*     0:62  8 */

	/* size: 8, cachelines: 1, members: 11 */
	/* last cacheline: 8 bytes */
};
struct uv3h_node_id_s {
	long unsigned int          force1:1;             /*     0: 0  8 */
	long unsigned int          manufacturer:11;      /*     0: 1  8 */
	long unsigned int          part_number:16;       /*     0:12  8 */
	long unsigned int          revision:4;           /*     0:28  8 */
	long unsigned int          node_id:15;           /*     0:32  8 */
	long unsigned int          rsvd_47:1;            /*     0:47  8 */
	long unsigned int          router_select:1;      /*     0:48  8 */
	long unsigned int          rsvd_49:1;            /*     0:49  8 */
	long unsigned int          nodes_per_bit:7;      /*     0:50  8 */
	long unsigned int          ni_port:5;            /*     0:57  8 */
	long unsigned int          rsvd_62_63:2;         /*     0:62  8 */

	/* size: 8, cachelines: 1, members: 11 */
	/* last cacheline: 8 bytes */
};
struct uv2h_node_id_s {
	long unsigned int          force1:1;             /*     0: 0  8 */
	long unsigned int          manufacturer:11;      /*     0: 1  8 */
	long unsigned int          part_number:16;       /*     0:12  8 */
	long unsigned int          revision:4;           /*     0:28  8 */
	long unsigned int          node_id:15;           /*     0:32  8 */
	long unsigned int          rsvd_47_49:3;         /*     0:47  8 */
	long unsigned int          nodes_per_bit:7;      /*     0:50  8 */
	long unsigned int          ni_port:5;            /*     0:57  8 */
	long unsigned int          rsvd_62_63:2;         /*     0:62  8 */

	/* size: 8, cachelines: 1, members: 9 */
	/* last cacheline: 8 bytes */
};
union uvh_node_id_u {
	long unsigned int          v;                  /*     0     8 */
	struct uvh_node_id_s       s;                  /*     0     8 */
	struct uvxh_node_id_s      sx;                 /*     0     8 */
	struct uvyh_node_id_s      sy;                 /*     0     8 */
	struct uv5h_node_id_s      s5;                 /*     0     8 */
	struct uv4h_node_id_s      s4;                 /*     0     8 */
	struct uv3h_node_id_s      s3;                 /*     0     8 */
	struct uv2h_node_id_s      s2;                 /*     0     8 */
};
struct uvh_rh10_gam_addr_map_config_s {
	long unsigned int          undef_0_5:6;          /*     0: 0  8 */
	long unsigned int          n_skt:3;              /*     0: 6  8 */
	long unsigned int          undef_9_11:3;         /*     0: 9  8 */
	long unsigned int          ls_enable:1;          /*     0:12  8 */
	long unsigned int          undef_13_15:3;        /*     0:13  8 */
	long unsigned int          mk_tme_keyid_bits:4;  /*     0:16  8 */
	long unsigned int          rsvd_20_63:44;        /*     0:20  8 */

	/* size: 8, cachelines: 1, members: 7 */
	/* last cacheline: 8 bytes */
};
struct uvyh_rh10_gam_addr_map_config_s {
	long unsigned int          undef_0_5:6;          /*     0: 0  8 */
	long unsigned int          n_skt:3;              /*     0: 6  8 */
	long unsigned int          undef_9_11:3;         /*     0: 9  8 */
	long unsigned int          ls_enable:1;          /*     0:12  8 */
	long unsigned int          undef_13_15:3;        /*     0:13  8 */
	long unsigned int          mk_tme_keyid_bits:4;  /*     0:16  8 */
	long unsigned int          rsvd_20_63:44;        /*     0:20  8 */

	/* size: 8, cachelines: 1, members: 7 */
	/* last cacheline: 8 bytes */
};
struct uv5h_rh10_gam_addr_map_config_s {
	long unsigned int          undef_0_5:6;          /*     0: 0  8 */
	long unsigned int          n_skt:3;              /*     0: 6  8 */
	long unsigned int          undef_9_11:3;         /*     0: 9  8 */
	long unsigned int          ls_enable:1;          /*     0:12  8 */
	long unsigned int          undef_13_15:3;        /*     0:13  8 */
	long unsigned int          mk_tme_keyid_bits:4;  /*     0:16  8 */

	/* size: 8, cachelines: 1, members: 6 */
	/* bit_padding: 44 bits */
	/* last cacheline: 8 bytes */
};
union uvh_rh10_gam_addr_map_config_u {
	long unsigned int          v;                  /*     0     8 */
	struct uvh_rh10_gam_addr_map_config_s s;       /*     0     8 */
	struct uvyh_rh10_gam_addr_map_config_s sy;     /*     0     8 */
	struct uv5h_rh10_gam_addr_map_config_s s5;     /*     0     8 */
};
struct uvh_rh10_gam_mmioh_overlay_config0_s {
	long unsigned int          rsvd_0_25:26;         /*     0: 0  8 */
	long unsigned int          base:26;              /*     0:26  8 */
	long unsigned int          m_io:6;               /*     0:52  8 */
	long unsigned int          n_io:4;               /*     0:58  8 */
	long unsigned int          undef_62:1;           /*     0:62  8 */
	long unsigned int          enable:1;             /*     0:63  8 */

	/* size: 8, cachelines: 1, members: 6 */
	/* last cacheline: 8 bytes */
};
struct uvyh_rh10_gam_mmioh_overlay_config0_s {
	long unsigned int          rsvd_0_25:26;         /*     0: 0  8 */
	long unsigned int          base:26;              /*     0:26  8 */
	long unsigned int          m_io:6;               /*     0:52  8 */
	long unsigned int          n_io:4;               /*     0:58  8 */
	long unsigned int          undef_62:1;           /*     0:62  8 */
	long unsigned int          enable:1;             /*     0:63  8 */

	/* size: 8, cachelines: 1, members: 6 */
	/* last cacheline: 8 bytes */
};
struct uv5h_rh10_gam_mmioh_overlay_config0_s {
	long unsigned int          rsvd_0_25:26;         /*     0: 0  8 */
	long unsigned int          base:26;              /*     0:26  8 */
	long unsigned int          m_io:6;               /*     0:52  8 */
	long unsigned int          n_io:4;               /*     0:58  8 */
	long unsigned int          undef_62:1;           /*     0:62  8 */
	long unsigned int          enable:1;             /*     0:63  8 */

	/* size: 8, cachelines: 1, members: 6 */
	/* last cacheline: 8 bytes */
};
union uvh_rh10_gam_mmioh_overlay_config0_u {
	long unsigned int          v;                  /*     0     8 */
	struct uvh_rh10_gam_mmioh_overlay_config0_s s; /*     0     8 */
	struct uvyh_rh10_gam_mmioh_overlay_config0_s sy; /*     0     8 */
	struct uv5h_rh10_gam_mmioh_overlay_config0_s s5; /*     0     8 */
};
struct uvh_rh10_gam_mmioh_overlay_config1_s {
	long unsigned int          rsvd_0_25:26;         /*     0: 0  8 */
	long unsigned int          base:26;              /*     0:26  8 */
	long unsigned int          m_io:6;               /*     0:52  8 */
	long unsigned int          n_io:4;               /*     0:58  8 */
	long unsigned int          undef_62:1;           /*     0:62  8 */
	long unsigned int          enable:1;             /*     0:63  8 */

	/* size: 8, cachelines: 1, members: 6 */
	/* last cacheline: 8 bytes */
};
struct uvyh_rh10_gam_mmioh_overlay_config1_s {
	long unsigned int          rsvd_0_25:26;         /*     0: 0  8 */
	long unsigned int          base:26;              /*     0:26  8 */
	long unsigned int          m_io:6;               /*     0:52  8 */
	long unsigned int          n_io:4;               /*     0:58  8 */
	long unsigned int          undef_62:1;           /*     0:62  8 */
	long unsigned int          enable:1;             /*     0:63  8 */

	/* size: 8, cachelines: 1, members: 6 */
	/* last cacheline: 8 bytes */
};
struct uv5h_rh10_gam_mmioh_overlay_config1_s {
	long unsigned int          rsvd_0_25:26;         /*     0: 0  8 */
	long unsigned int          base:26;              /*     0:26  8 */
	long unsigned int          m_io:6;               /*     0:52  8 */
	long unsigned int          n_io:4;               /*     0:58  8 */
	long unsigned int          undef_62:1;           /*     0:62  8 */
	long unsigned int          enable:1;             /*     0:63  8 */

	/* size: 8, cachelines: 1, members: 6 */
	/* last cacheline: 8 bytes */
};
union uvh_rh10_gam_mmioh_overlay_config1_u {
	long unsigned int          v;                  /*     0     8 */
	struct uvh_rh10_gam_mmioh_overlay_config1_s s; /*     0     8 */
	struct uvyh_rh10_gam_mmioh_overlay_config1_s sy; /*     0     8 */
	struct uv5h_rh10_gam_mmioh_overlay_config1_s s5; /*     0     8 */
};
struct uvh_rh10_gam_mmr_overlay_config_s {
	long unsigned int          undef_0_24:25;        /*     0: 0  8 */
	long unsigned int          base:27;              /*     0:25  8 */
	long unsigned int          undef_52_62:11;       /*     0:52  8 */
	long unsigned int          enable:1;             /*     0:63  8 */

	/* size: 8, cachelines: 1, members: 4 */
	/* last cacheline: 8 bytes */
};
struct uvyh_rh10_gam_mmr_overlay_config_s {
	long unsigned int          undef_0_24:25;        /*     0: 0  8 */
	long unsigned int          base:27;              /*     0:25  8 */
	long unsigned int          undef_52_62:11;       /*     0:52  8 */
	long unsigned int          enable:1;             /*     0:63  8 */

	/* size: 8, cachelines: 1, members: 4 */
	/* last cacheline: 8 bytes */
};
struct uv5h_rh10_gam_mmr_overlay_config_s {
	long unsigned int          undef_0_24:25;        /*     0: 0  8 */
	long unsigned int          base:27;              /*     0:25  8 */
	long unsigned int          undef_52_62:11;       /*     0:52  8 */
	long unsigned int          enable:1;             /*     0:63  8 */

	/* size: 8, cachelines: 1, members: 4 */
	/* last cacheline: 8 bytes */
};
union uvh_rh10_gam_mmr_overlay_config_u {
	long unsigned int          v;                  /*     0     8 */
	struct uvh_rh10_gam_mmr_overlay_config_s s;    /*     0     8 */
	struct uvyh_rh10_gam_mmr_overlay_config_s sy;  /*     0     8 */
	struct uv5h_rh10_gam_mmr_overlay_config_s s5;  /*     0     8 */
};
struct uvh_rh_gam_addr_map_config_s {
	long unsigned int          rsvd_0_5:6;           /*     0: 0  8 */
	long unsigned int          n_skt:4;              /*     0: 6  8 */
	long unsigned int          rsvd_10_63:54;        /*     0:10  8 */

	/* size: 8, cachelines: 1, members: 3 */
	/* last cacheline: 8 bytes */
};
struct uvxh_rh_gam_addr_map_config_s {
	long unsigned int          rsvd_0_5:6;           /*     0: 0  8 */
	long unsigned int          n_skt:4;              /*     0: 6  8 */
	long unsigned int          rsvd_10_63:54;        /*     0:10  8 */

	/* size: 8, cachelines: 1, members: 3 */
	/* last cacheline: 8 bytes */
};
struct uv4h_rh_gam_addr_map_config_s {
	long unsigned int          rsvd_0_5:6;           /*     0: 0  8 */
	long unsigned int          n_skt:4;              /*     0: 6  8 */
	long unsigned int          rsvd_10_63:54;        /*     0:10  8 */

	/* size: 8, cachelines: 1, members: 3 */
	/* last cacheline: 8 bytes */
};
struct uv3h_rh_gam_addr_map_config_s {
	long unsigned int          m_skt:6;              /*     0: 0  8 */
	long unsigned int          n_skt:4;              /*     0: 6  8 */
	long unsigned int          rsvd_10_63:54;        /*     0:10  8 */

	/* size: 8, cachelines: 1, members: 3 */
	/* last cacheline: 8 bytes */
};
struct uv2h_rh_gam_addr_map_config_s {
	long unsigned int          m_skt:6;              /*     0: 0  8 */
	long unsigned int          n_skt:4;              /*     0: 6  8 */
	long unsigned int          rsvd_10_63:54;        /*     0:10  8 */

	/* size: 8, cachelines: 1, members: 3 */
	/* last cacheline: 8 bytes */
};
union uvh_rh_gam_addr_map_config_u {
	long unsigned int          v;                  /*     0     8 */
	struct uvh_rh_gam_addr_map_config_s s;         /*     0     8 */
	struct uvxh_rh_gam_addr_map_config_s sx;       /*     0     8 */
	struct uv4h_rh_gam_addr_map_config_s s4;       /*     0     8 */
	struct uv3h_rh_gam_addr_map_config_s s3;       /*     0     8 */
	struct uv2h_rh_gam_addr_map_config_s s2;       /*     0     8 */
};
struct uvh_rh_gam_alias_2_overlay_config_s {
	long unsigned int          rsvd_0_23:24;         /*     0: 0  8 */
	long unsigned int          base:8;               /*     0:24  8 */
	long unsigned int          rsvd_32_47:16;        /*     0:32  8 */
	long unsigned int          m_alias:5;            /*     0:48  8 */
	long unsigned int          rsvd_53_62:10;        /*     0:53  8 */
	long unsigned int          enable:1;             /*     0:63  8 */

	/* size: 8, cachelines: 1, members: 6 */
	/* last cacheline: 8 bytes */
};
struct uvxh_rh_gam_alias_2_overlay_config_s {
	long unsigned int          rsvd_0_23:24;         /*     0: 0  8 */
	long unsigned int          base:8;               /*     0:24  8 */
	long unsigned int          rsvd_32_47:16;        /*     0:32  8 */
	long unsigned int          m_alias:5;            /*     0:48  8 */
	long unsigned int          rsvd_53_62:10;        /*     0:53  8 */
	long unsigned int          enable:1;             /*     0:63  8 */

	/* size: 8, cachelines: 1, members: 6 */
	/* last cacheline: 8 bytes */
};
struct uv4h_rh_gam_alias_2_overlay_config_s {
	long unsigned int          rsvd_0_23:24;         /*     0: 0  8 */
	long unsigned int          base:8;               /*     0:24  8 */
	long unsigned int          rsvd_32_47:16;        /*     0:32  8 */
	long unsigned int          m_alias:5;            /*     0:48  8 */
	long unsigned int          rsvd_53_62:10;        /*     0:53  8 */
	long unsigned int          enable:1;             /*     0:63  8 */

	/* size: 8, cachelines: 1, members: 6 */
	/* last cacheline: 8 bytes */
};
struct uv3h_rh_gam_alias_2_overlay_config_s {
	long unsigned int          rsvd_0_23:24;         /*     0: 0  8 */
	long unsigned int          base:8;               /*     0:24  8 */
	long unsigned int          rsvd_32_47:16;        /*     0:32  8 */
	long unsigned int          m_alias:5;            /*     0:48  8 */
	long unsigned int          rsvd_53_62:10;        /*     0:53  8 */
	long unsigned int          enable:1;             /*     0:63  8 */

	/* size: 8, cachelines: 1, members: 6 */
	/* last cacheline: 8 bytes */
};
struct uv2h_rh_gam_alias_2_overlay_config_s {
	long unsigned int          rsvd_0_23:24;         /*     0: 0  8 */
	long unsigned int          base:8;               /*     0:24  8 */
	long unsigned int          rsvd_32_47:16;        /*     0:32  8 */
	long unsigned int          m_alias:5;            /*     0:48  8 */
	long unsigned int          rsvd_53_62:10;        /*     0:53  8 */
	long unsigned int          enable:1;             /*     0:63  8 */

	/* size: 8, cachelines: 1, members: 6 */
	/* last cacheline: 8 bytes */
};
union uvh_rh_gam_alias_2_overlay_config_u {
	long unsigned int          v;                  /*     0     8 */
	struct uvh_rh_gam_alias_2_overlay_config_s s;  /*     0     8 */
	struct uvxh_rh_gam_alias_2_overlay_config_s sx; /*     0     8 */
	struct uv4h_rh_gam_alias_2_overlay_config_s s4; /*     0     8 */
	struct uv3h_rh_gam_alias_2_overlay_config_s s3; /*     0     8 */
	struct uv2h_rh_gam_alias_2_overlay_config_s s2; /*     0     8 */
};
struct uvh_rh_gam_alias_2_redirect_config_s {
	long unsigned int          rsvd_0_23:24;         /*     0: 0  8 */
	long unsigned int          dest_base:22;         /*     0:24  8 */
	long unsigned int          rsvd_46_63:18;        /*     0:46  8 */

	/* size: 8, cachelines: 1, members: 3 */
	/* last cacheline: 8 bytes */
};
struct uvxh_rh_gam_alias_2_redirect_config_s {
	long unsigned int          rsvd_0_23:24;         /*     0: 0  8 */
	long unsigned int          dest_base:22;         /*     0:24  8 */
	long unsigned int          rsvd_46_63:18;        /*     0:46  8 */

	/* size: 8, cachelines: 1, members: 3 */
	/* last cacheline: 8 bytes */
};
struct uv4h_rh_gam_alias_2_redirect_config_s {
	long unsigned int          rsvd_0_23:24;         /*     0: 0  8 */
	long unsigned int          dest_base:22;         /*     0:24  8 */
	long unsigned int          rsvd_46_63:18;        /*     0:46  8 */

	/* size: 8, cachelines: 1, members: 3 */
	/* last cacheline: 8 bytes */
};
struct uv3h_rh_gam_alias_2_redirect_config_s {
	long unsigned int          rsvd_0_23:24;         /*     0: 0  8 */
	long unsigned int          dest_base:22;         /*     0:24  8 */
	long unsigned int          rsvd_46_63:18;        /*     0:46  8 */

	/* size: 8, cachelines: 1, members: 3 */
	/* last cacheline: 8 bytes */
};
struct uv2h_rh_gam_alias_2_redirect_config_s {
	long unsigned int          rsvd_0_23:24;         /*     0: 0  8 */
	long unsigned int          dest_base:22;         /*     0:24  8 */
	long unsigned int          rsvd_46_63:18;        /*     0:46  8 */

	/* size: 8, cachelines: 1, members: 3 */
	/* last cacheline: 8 bytes */
};
union uvh_rh_gam_alias_2_redirect_config_u {
	long unsigned int          v;                  /*     0     8 */
	struct uvh_rh_gam_alias_2_redirect_config_s s; /*     0     8 */
	struct uvxh_rh_gam_alias_2_redirect_config_s sx; /*     0     8 */
	struct uv4h_rh_gam_alias_2_redirect_config_s s4; /*     0     8 */
	struct uv3h_rh_gam_alias_2_redirect_config_s s3; /*     0     8 */
	struct uv2h_rh_gam_alias_2_redirect_config_s s2; /*     0     8 */
};
struct uvh_rh_gam_gru_overlay_config_s {
	long unsigned int          rsvd_0_45:46;         /*     0: 0  8 */
	long unsigned int          rsvd_46_51:6;         /*     0:46  8 */
	long unsigned int          n_gru:4;              /*     0:52  8 */
	long unsigned int          rsvd_56_62:7;         /*     0:56  8 */
	long unsigned int          enable:1;             /*     0:63  8 */

	/* size: 8, cachelines: 1, members: 5 */
	/* last cacheline: 8 bytes */
};
struct uvxh_rh_gam_gru_overlay_config_s {
	long unsigned int          rsvd_0_45:46;         /*     0: 0  8 */
	long unsigned int          rsvd_46_51:6;         /*     0:46  8 */
	long unsigned int          n_gru:4;              /*     0:52  8 */
	long unsigned int          rsvd_56_62:7;         /*     0:56  8 */
	long unsigned int          enable:1;             /*     0:63  8 */

	/* size: 8, cachelines: 1, members: 5 */
	/* last cacheline: 8 bytes */
};
struct uv4ah_rh_gam_gru_overlay_config_s {
	long unsigned int          rsvd_0_24:25;         /*     0: 0  8 */
	long unsigned int          undef_25:1;           /*     0:25  8 */
	long unsigned int          base:26;              /*     0:26  8 */
	long unsigned int          n_gru:4;              /*     0:52  8 */
	long unsigned int          rsvd_56_62:7;         /*     0:56  8 */
	long unsigned int          enable:1;             /*     0:63  8 */

	/* size: 8, cachelines: 1, members: 6 */
	/* last cacheline: 8 bytes */
};
struct uv4h_rh_gam_gru_overlay_config_s {
	long unsigned int          rsvd_0_24:25;         /*     0: 0  8 */
	long unsigned int          undef_25:1;           /*     0:25  8 */
	long unsigned int          base:20;              /*     0:26  8 */
	long unsigned int          rsvd_46_51:6;         /*     0:46  8 */
	long unsigned int          n_gru:4;              /*     0:52  8 */
	long unsigned int          rsvd_56_62:7;         /*     0:56  8 */
	long unsigned int          enable:1;             /*     0:63  8 */

	/* size: 8, cachelines: 1, members: 7 */
	/* last cacheline: 8 bytes */
};
struct uv3h_rh_gam_gru_overlay_config_s {
	long unsigned int          rsvd_0_27:28;         /*     0: 0  8 */
	long unsigned int          base:18;              /*     0:28  8 */
	long unsigned int          rsvd_46_51:6;         /*     0:46  8 */
	long unsigned int          n_gru:4;              /*     0:52  8 */
	long unsigned int          rsvd_56_61:6;         /*     0:56  8 */
	long unsigned int          mode:1;               /*     0:62  8 */
	long unsigned int          enable:1;             /*     0:63  8 */

	/* size: 8, cachelines: 1, members: 7 */
	/* last cacheline: 8 bytes */
};
struct uv2h_rh_gam_gru_overlay_config_s {
	long unsigned int          rsvd_0_27:28;         /*     0: 0  8 */
	long unsigned int          base:18;              /*     0:28  8 */
	long unsigned int          rsvd_46_51:6;         /*     0:46  8 */
	long unsigned int          n_gru:4;              /*     0:52  8 */
	long unsigned int          rsvd_56_62:7;         /*     0:56  8 */
	long unsigned int          enable:1;             /*     0:63  8 */

	/* size: 8, cachelines: 1, members: 6 */
	/* last cacheline: 8 bytes */
};
union uvh_rh_gam_gru_overlay_config_u {
	long unsigned int          v;                  /*     0     8 */
	struct uvh_rh_gam_gru_overlay_config_s s;      /*     0     8 */
	struct uvxh_rh_gam_gru_overlay_config_s sx;    /*     0     8 */
	struct uv4ah_rh_gam_gru_overlay_config_s s4a;  /*     0     8 */
	struct uv4h_rh_gam_gru_overlay_config_s s4;    /*     0     8 */
	struct uv3h_rh_gam_gru_overlay_config_s s3;    /*     0     8 */
	struct uv2h_rh_gam_gru_overlay_config_s s2;    /*     0     8 */
};
struct uvh_rh_gam_mmioh_overlay_config_s {
	long unsigned int          rsvd_0_26:27;         /*     0: 0  8 */
	long unsigned int          base:19;              /*     0:27  8 */
	long unsigned int          m_io:6;               /*     0:46  8 */
	long unsigned int          n_io:4;               /*     0:52  8 */
	long unsigned int          rsvd_56_62:7;         /*     0:56  8 */
	long unsigned int          enable:1;             /*     0:63  8 */

	/* size: 8, cachelines: 1, members: 6 */
	/* last cacheline: 8 bytes */
};
struct uvxh_rh_gam_mmioh_overlay_config_s {
	long unsigned int          rsvd_0_26:27;         /*     0: 0  8 */
	long unsigned int          base:19;              /*     0:27  8 */
	long unsigned int          m_io:6;               /*     0:46  8 */
	long unsigned int          n_io:4;               /*     0:52  8 */
	long unsigned int          rsvd_56_62:7;         /*     0:56  8 */
	long unsigned int          enable:1;             /*     0:63  8 */

	/* size: 8, cachelines: 1, members: 6 */
	/* last cacheline: 8 bytes */
};
struct uv2h_rh_gam_mmioh_overlay_config_s {
	long unsigned int          rsvd_0_26:27;         /*     0: 0  8 */
	long unsigned int          base:19;              /*     0:27  8 */
	long unsigned int          m_io:6;               /*     0:46  8 */
	long unsigned int          n_io:4;               /*     0:52  8 */
	long unsigned int          rsvd_56_62:7;         /*     0:56  8 */
	long unsigned int          enable:1;             /*     0:63  8 */

	/* size: 8, cachelines: 1, members: 6 */
	/* last cacheline: 8 bytes */
};
union uvh_rh_gam_mmioh_overlay_config_u {
	long unsigned int          v;                  /*     0     8 */
	struct uvh_rh_gam_mmioh_overlay_config_s s;    /*     0     8 */
	struct uvxh_rh_gam_mmioh_overlay_config_s sx;  /*     0     8 */
	struct uv2h_rh_gam_mmioh_overlay_config_s s2;  /*     0     8 */
};
struct uvh_rh_gam_mmioh_overlay_config0_s {
	long unsigned int          rsvd_0_25:26;         /*     0: 0  8 */
	long unsigned int          base:20;              /*     0:26  8 */
	long unsigned int          m_io:6;               /*     0:46  8 */
	long unsigned int          n_io:4;               /*     0:52  8 */
	long unsigned int          rsvd_56_62:7;         /*     0:56  8 */
	long unsigned int          enable:1;             /*     0:63  8 */

	/* size: 8, cachelines: 1, members: 6 */
	/* last cacheline: 8 bytes */
};
struct uvxh_rh_gam_mmioh_overlay_config0_s {
	long unsigned int          rsvd_0_25:26;         /*     0: 0  8 */
	long unsigned int          base:20;              /*     0:26  8 */
	long unsigned int          m_io:6;               /*     0:46  8 */
	long unsigned int          n_io:4;               /*     0:52  8 */
	long unsigned int          rsvd_56_62:7;         /*     0:56  8 */
	long unsigned int          enable:1;             /*     0:63  8 */

	/* size: 8, cachelines: 1, members: 6 */
	/* last cacheline: 8 bytes */
};
struct uv4ah_rh_gam_mmioh_overlay_config0_mmr_s {
	long unsigned int          rsvd_0_25:26;         /*     0: 0  8 */
	long unsigned int          base:26;              /*     0:26  8 */
	long unsigned int          m_io:6;               /*     0:52  8 */
	long unsigned int          n_io:4;               /*     0:58  8 */
	long unsigned int          undef_62:1;           /*     0:62  8 */
	long unsigned int          enable:1;             /*     0:63  8 */

	/* size: 8, cachelines: 1, members: 6 */
	/* last cacheline: 8 bytes */
};
struct uv4h_rh_gam_mmioh_overlay_config0_s {
	long unsigned int          rsvd_0_25:26;         /*     0: 0  8 */
	long unsigned int          base:20;              /*     0:26  8 */
	long unsigned int          m_io:6;               /*     0:46  8 */
	long unsigned int          n_io:4;               /*     0:52  8 */
	long unsigned int          rsvd_56_62:7;         /*     0:56  8 */
	long unsigned int          enable:1;             /*     0:63  8 */

	/* size: 8, cachelines: 1, members: 6 */
	/* last cacheline: 8 bytes */
};
struct uv3h_rh_gam_mmioh_overlay_config0_s {
	long unsigned int          rsvd_0_25:26;         /*     0: 0  8 */
	long unsigned int          base:20;              /*     0:26  8 */
	long unsigned int          m_io:6;               /*     0:46  8 */
	long unsigned int          n_io:4;               /*     0:52  8 */
	long unsigned int          rsvd_56_62:7;         /*     0:56  8 */
	long unsigned int          enable:1;             /*     0:63  8 */

	/* size: 8, cachelines: 1, members: 6 */
	/* last cacheline: 8 bytes */
};
union uvh_rh_gam_mmioh_overlay_config0_u {
	long unsigned int          v;                  /*     0     8 */
	struct uvh_rh_gam_mmioh_overlay_config0_s s;   /*     0     8 */
	struct uvxh_rh_gam_mmioh_overlay_config0_s sx; /*     0     8 */
	struct uv4ah_rh_gam_mmioh_overlay_config0_mmr_s s4a; /*     0     8 */
	struct uv4h_rh_gam_mmioh_overlay_config0_s s4; /*     0     8 */
	struct uv3h_rh_gam_mmioh_overlay_config0_s s3; /*     0     8 */
};
struct uvh_rh_gam_mmioh_overlay_config1_s {
	long unsigned int          rsvd_0_25:26;         /*     0: 0  8 */
	long unsigned int          base:20;              /*     0:26  8 */
	long unsigned int          m_io:6;               /*     0:46  8 */
	long unsigned int          n_io:4;               /*     0:52  8 */
	long unsigned int          rsvd_56_62:7;         /*     0:56  8 */
	long unsigned int          enable:1;             /*     0:63  8 */

	/* size: 8, cachelines: 1, members: 6 */
	/* last cacheline: 8 bytes */
};
struct uvxh_rh_gam_mmioh_overlay_config1_s {
	long unsigned int          rsvd_0_25:26;         /*     0: 0  8 */
	long unsigned int          base:20;              /*     0:26  8 */
	long unsigned int          m_io:6;               /*     0:46  8 */
	long unsigned int          n_io:4;               /*     0:52  8 */
	long unsigned int          rsvd_56_62:7;         /*     0:56  8 */
	long unsigned int          enable:1;             /*     0:63  8 */

	/* size: 8, cachelines: 1, members: 6 */
	/* last cacheline: 8 bytes */
};
struct uv4ah_rh_gam_mmioh_overlay_config1_mmr_s {
	long unsigned int          rsvd_0_25:26;         /*     0: 0  8 */
	long unsigned int          base:26;              /*     0:26  8 */
	long unsigned int          m_io:6;               /*     0:52  8 */
	long unsigned int          n_io:4;               /*     0:58  8 */
	long unsigned int          undef_62:1;           /*     0:62  8 */
	long unsigned int          enable:1;             /*     0:63  8 */

	/* size: 8, cachelines: 1, members: 6 */
	/* last cacheline: 8 bytes */
};
struct uv4h_rh_gam_mmioh_overlay_config1_s {
	long unsigned int          rsvd_0_25:26;         /*     0: 0  8 */
	long unsigned int          base:20;              /*     0:26  8 */
	long unsigned int          m_io:6;               /*     0:46  8 */
	long unsigned int          n_io:4;               /*     0:52  8 */
	long unsigned int          rsvd_56_62:7;         /*     0:56  8 */
	long unsigned int          enable:1;             /*     0:63  8 */

	/* size: 8, cachelines: 1, members: 6 */
	/* last cacheline: 8 bytes */
};
struct uv3h_rh_gam_mmioh_overlay_config1_s {
	long unsigned int          rsvd_0_25:26;         /*     0: 0  8 */
	long unsigned int          base:20;              /*     0:26  8 */
	long unsigned int          m_io:6;               /*     0:46  8 */
	long unsigned int          n_io:4;               /*     0:52  8 */
	long unsigned int          rsvd_56_62:7;         /*     0:56  8 */
	long unsigned int          enable:1;             /*     0:63  8 */

	/* size: 8, cachelines: 1, members: 6 */
	/* last cacheline: 8 bytes */
};
union uvh_rh_gam_mmioh_overlay_config1_u {
	long unsigned int          v;                  /*     0     8 */
	struct uvh_rh_gam_mmioh_overlay_config1_s s;   /*     0     8 */
	struct uvxh_rh_gam_mmioh_overlay_config1_s sx; /*     0     8 */
	struct uv4ah_rh_gam_mmioh_overlay_config1_mmr_s s4a; /*     0     8 */
	struct uv4h_rh_gam_mmioh_overlay_config1_s s4; /*     0     8 */
	struct uv3h_rh_gam_mmioh_overlay_config1_s s3; /*     0     8 */
};
struct uvh_rh_gam_mmr_overlay_config_s {
	long unsigned int          rsvd_0_25:26;         /*     0: 0  8 */
	long unsigned int          base:20;              /*     0:26  8 */
	long unsigned int          rsvd_46_62:17;        /*     0:46  8 */
	long unsigned int          enable:1;             /*     0:63  8 */

	/* size: 8, cachelines: 1, members: 4 */
	/* last cacheline: 8 bytes */
};
struct uvxh_rh_gam_mmr_overlay_config_s {
	long unsigned int          rsvd_0_25:26;         /*     0: 0  8 */
	long unsigned int          base:20;              /*     0:26  8 */
	long unsigned int          rsvd_46_62:17;        /*     0:46  8 */
	long unsigned int          enable:1;             /*     0:63  8 */

	/* size: 8, cachelines: 1, members: 4 */
	/* last cacheline: 8 bytes */
};
struct uv4h_rh_gam_mmr_overlay_config_s {
	long unsigned int          rsvd_0_25:26;         /*     0: 0  8 */
	long unsigned int          base:20;              /*     0:26  8 */
	long unsigned int          rsvd_46_62:17;        /*     0:46  8 */
	long unsigned int          enable:1;             /*     0:63  8 */

	/* size: 8, cachelines: 1, members: 4 */
	/* last cacheline: 8 bytes */
};
struct uv3h_rh_gam_mmr_overlay_config_s {
	long unsigned int          rsvd_0_25:26;         /*     0: 0  8 */
	long unsigned int          base:20;              /*     0:26  8 */
	long unsigned int          rsvd_46_62:17;        /*     0:46  8 */
	long unsigned int          enable:1;             /*     0:63  8 */

	/* size: 8, cachelines: 1, members: 4 */
	/* last cacheline: 8 bytes */
};
struct uv2h_rh_gam_mmr_overlay_config_s {
	long unsigned int          rsvd_0_25:26;         /*     0: 0  8 */
	long unsigned int          base:20;              /*     0:26  8 */
	long unsigned int          rsvd_46_62:17;        /*     0:46  8 */
	long unsigned int          enable:1;             /*     0:63  8 */

	/* size: 8, cachelines: 1, members: 4 */
	/* last cacheline: 8 bytes */
};
union uvh_rh_gam_mmr_overlay_config_u {
	long unsigned int          v;                  /*     0     8 */
	struct uvh_rh_gam_mmr_overlay_config_s s;      /*     0     8 */
	struct uvxh_rh_gam_mmr_overlay_config_s sx;    /*     0     8 */
	struct uv4h_rh_gam_mmr_overlay_config_s s4;    /*     0     8 */
	struct uv3h_rh_gam_mmr_overlay_config_s s3;    /*     0     8 */
	struct uv2h_rh_gam_mmr_overlay_config_s s2;    /*     0     8 */
};
struct uv_gam_parameters {
	u64                        mmr_base;             /*     0     8 */
	u64                        gru_base;             /*     8     8 */
	u8                         mmr_shift;            /*    16     1 */
	u8                         gru_shift;            /*    17     1 */
	u8                         gpa_shift;            /*    18     1 */
	u8                         unused1;              /*    19     1 */

	/* size: 24, cachelines: 1, members: 6 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct uv_gam_range_entry {
	char                       type;                 /*     0     1 */
	char                       unused1;              /*     1     1 */
	u16                        nasid;                /*     2     2 */
	u16                        sockid;               /*     4     2 */
	u16                        pnode;                /*     6     2 */
	u32                        unused2;              /*     8     4 */
	u32                        limit;                /*    12     4 */

	/* size: 16, cachelines: 1, members: 7 */
	/* last cacheline: 16 bytes */
};
struct uv_arch_type_entry {
	char                       archtype[8];          /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct uv_systab {
	char                       signature[4];         /*     0     4 */
	u32                        revision;             /*     4     4 */
	u64                        function;             /*     8     8 */
	u32                        size;                 /*    16     4 */
	struct {
		u32                type:8;               /*    20: 0  4 */
		u32                offset:24;            /*    20: 8  4 */
	} entry[1]; /*    20     4 */

	/* size: 24, cachelines: 1, members: 5 */
	/* last cacheline: 24 bytes */
};
struct uv_gam_range_s {
	u32                        limit;                /*     0     4 */
	u16                        nasid;                /*     4     2 */
	s8                         base;                 /*     6     1 */
	u8                         reserved;             /*     7     1 */

	/* size: 8, cachelines: 1, members: 4 */
	/* last cacheline: 8 bytes */
};
struct uv_hub_info_s {
	unsigned int               hub_type;             /*     0     4 */
	unsigned char              hub_revision;         /*     4     1 */

	/* XXX 3 bytes hole, try to pack */

	long unsigned int          global_mmr_base;      /*     8     8 */
	long unsigned int          global_mmr_shift;     /*    16     8 */
	long unsigned int          gpa_mask;             /*    24     8 */
	short unsigned int *       socket_to_node;       /*    32     8 */
	short unsigned int *       socket_to_pnode;      /*    40     8 */
	short unsigned int *       pnode_to_socket;      /*    48     8 */
	struct uv_gam_range_s *    gr_table;             /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	short unsigned int         min_socket;           /*    64     2 */
	short unsigned int         min_pnode;            /*    66     2 */
	unsigned char              m_val;                /*    68     1 */
	unsigned char              n_val;                /*    69     1 */
	unsigned char              gr_table_len;         /*    70     1 */
	unsigned char              apic_pnode_shift;     /*    71     1 */
	unsigned char              gpa_shift;            /*    72     1 */
	unsigned char              nasid_shift;          /*    73     1 */
	unsigned char              m_shift;              /*    74     1 */
	unsigned char              n_lshift;             /*    75     1 */
	unsigned int               gnode_extra;          /*    76     4 */
	long unsigned int          gnode_upper;          /*    80     8 */
	long unsigned int          lowmem_remap_top;     /*    88     8 */
	long unsigned int          lowmem_remap_base;    /*    96     8 */
	long unsigned int          global_gru_base;      /*   104     8 */
	long unsigned int          global_gru_shift;     /*   112     8 */
	short unsigned int         pnode;                /*   120     2 */
	short unsigned int         pnode_mask;           /*   122     2 */
	short unsigned int         coherency_domain_number; /*   124     2 */
	short unsigned int         numa_blade_id;        /*   126     2 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	short unsigned int         nr_possible_cpus;     /*   128     2 */
	short unsigned int         nr_online_cpus;       /*   130     2 */
	short int                  memory_nid;           /*   132     2 */

	/* size: 136, cachelines: 3, members: 32 */
	/* sum members: 131, holes: 1, sum holes: 3 */
	/* padding: 2 */
	/* last cacheline: 8 bytes */
};
struct uv_cpu_info_s {
	void *                     p_uv_hub_info;        /*     0     8 */
	unsigned char              blade_cpu_id;         /*     8     1 */

	/* XXX 7 bytes hole, try to pack */

	void *                     reserved;             /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 17, holes: 1, sum holes: 7 */
	/* last cacheline: 24 bytes */
};
struct uvh_apicid_s {
	long unsigned int          local_apic_mask:24;   /*     0: 0  8 */
	long unsigned int          local_apic_shift:5;   /*     0:24  8 */
	long unsigned int          unused1:3;            /*     0:29  8 */
	long unsigned int          pnode_mask:24;        /*     0:32  8 */
	long unsigned int          pnode_shift:5;        /*     0:56  8 */
	long unsigned int          unused2:3;            /*     0:61  8 */

	/* size: 8, cachelines: 1, members: 6 */
	/* last cacheline: 8 bytes */
};
union uvh_apicid {
	long unsigned int          v;                  /*     0     8 */
	struct uvh_apicid_s        s;                  /*     0     8 */
};
struct mn {
	unsigned char              m_val;                /*     0     1 */
	unsigned char              n_val;                /*     1     1 */
	unsigned char              m_shift;              /*     2     1 */
	unsigned char              n_lshift;             /*     3     1 */

	/* size: 4, cachelines: 1, members: 4 */
	/* last cacheline: 4 bytes */
};
struct cluster_mask {
	unsigned int               clusterid;            /*     0     4 */
	int                        node;                 /*     4     4 */
	struct cpumask             mask;                 /*     8  1024 */

	/* size: 1032, cachelines: 17, members: 3 */
	/* last cacheline: 8 bytes */
};
struct dyn_arch_ftrace {

	/* size: 0, cachelines: 0, members: 0 */
};
struct dyn_ftrace {
	long unsigned int          ip;                   /*     0     8 */
	long unsigned int          flags;                /*     8     8 */
	struct dyn_arch_ftrace     arch;                 /*    16     0 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
union ftrace_op_code_union {
	char                       code[7];            /*     0     7 */
	struct {
		char               op[3];              /*     0     3 */
		int                offset;             /*     3     4 */
	} __attribute__((__packed__));                 /*     0     7 */
};
struct arch_klp_data {
	unsigned char              old_insns[5];         /*     0     5 */
	unsigned char              saved_opcode;         /*     5     1 */

	/* size: 6, cachelines: 1, members: 2 */
	/* last cacheline: 6 bytes */
};
struct klp_func {
	const char  *              old_name;             /*     0     8 */
	void *                     new_func;             /*     8     8 */
	long unsigned int          old_sympos;           /*    16     8 */
	int                        force;                /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     old_func;             /*    32     8 */
	struct kobject             kobj;                 /*    40    96 */
	/* --- cacheline 2 boundary (128 bytes) was 8 bytes ago --- */
	struct list_head           node;                 /*   136    16 */
	struct list_head           stack_node;           /*   152    16 */
	long unsigned int          old_size;             /*   168     8 */
	long unsigned int          new_size;             /*   176     8 */
	bool                       nop;                  /*   184     1 */
	bool                       patched;              /*   185     1 */

	/* XXX 6 bytes hole, try to pack */

	/* --- cacheline 3 boundary (192 bytes) --- */
	void *                     func_node;            /*   192     8 */

	/* size: 200, cachelines: 4, members: 13 */
	/* sum members: 190, holes: 2, sum holes: 10 */
	/* last cacheline: 8 bytes */
};
struct klp_hook {
	void                       (*hook)(void);        /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct klp_callbacks {
	int                        (*pre_patch)(struct klp_object *); /*     0     8 */
	void                       (*post_patch)(struct klp_object *); /*     8     8 */
	void                       (*pre_unpatch)(struct klp_object *); /*    16     8 */
	void                       (*post_unpatch)(struct klp_object *); /*    24     8 */
	bool                       post_unpatch_enabled; /*    32     1 */

	/* size: 40, cachelines: 1, members: 5 */
	/* padding: 7 */
	/* last cacheline: 40 bytes */
};
struct klp_object {
	const char  *              name;                 /*     0     8 */
	struct klp_func *          funcs;                /*     8     8 */
	struct klp_hook *          hooks_load;           /*    16     8 */
	struct klp_hook *          hooks_unload;         /*    24     8 */
	struct klp_callbacks       callbacks;            /*    32    40 */

	/* XXX last struct has 7 bytes of padding */

	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	struct kobject             kobj;                 /*    72    96 */
	/* --- cacheline 2 boundary (128 bytes) was 40 bytes ago --- */
	struct list_head           func_list;            /*   168    16 */
	struct list_head           node;                 /*   184    16 */
	/* --- cacheline 3 boundary (192 bytes) was 8 bytes ago --- */
	struct module *            mod;                  /*   200     8 */
	bool                       dynamic;              /*   208     1 */
	bool                       patched;              /*   209     1 */

	/* size: 216, cachelines: 4, members: 11 */
	/* padding: 6 */
	/* paddings: 1, sum paddings: 7 */
	/* last cacheline: 24 bytes */
};
struct klp_state {
	long unsigned int          id;                   /*     0     8 */
	unsigned int               version;              /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     data;                 /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct klp_patch {
	struct module *            mod;                  /*     0     8 */
	struct klp_object *        objs;                 /*     8     8 */
	struct klp_state *         states;               /*    16     8 */
	bool                       replace;              /*    24     1 */

	/* XXX 7 bytes hole, try to pack */

	struct list_head           list;                 /*    32    16 */
	struct kobject             kobj;                 /*    48    96 */
	/* --- cacheline 2 boundary (128 bytes) was 16 bytes ago --- */
	struct list_head           obj_list;             /*   144    16 */
	bool                       enabled;              /*   160     1 */
	bool                       forced;               /*   161     1 */

	/* XXX 6 bytes hole, try to pack */

	struct work_struct         free_work;            /*   168    64 */
	/* --- cacheline 3 boundary (192 bytes) was 40 bytes ago --- */
	struct completion          finish;               /*   232    32 */

	/* size: 264, cachelines: 5, members: 11 */
	/* sum members: 251, holes: 2, sum holes: 13 */
	/* last cacheline: 8 bytes */
};
struct klp_func_node {
	struct list_head           node;                 /*     0    16 */
	struct list_head           func_stack;           /*    16    16 */
	void *                     old_func;             /*    32     8 */
	struct arch_klp_data       arch_data;            /*    40     6 */

	/* XXX 2 bytes hole, try to pack */

	void *                     brk_func;             /*    48     8 */

	/* size: 56, cachelines: 1, members: 5 */
	/* sum members: 54, holes: 1, sum holes: 2 */
	/* last cacheline: 56 bytes */
};
struct klp_func_list {
	struct klp_func_list *     next;                 /*     0     8 */
	long unsigned int          func_addr;            /*     8     8 */
	long unsigned int          func_size;            /*    16     8 */
	const char  *              func_name;            /*    24     8 */
	int                        force;                /*    32     4 */

	/* size: 40, cachelines: 1, members: 5 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct elf64_rela {
	Elf64_Addr                 r_offset;             /*     0     8 */
	Elf64_Xword                r_info;               /*     8     8 */
	Elf64_Sxword               r_addend;             /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct kimage_arch {
	p4d_t *                    p4d;                  /*     0     8 */
	pud_t *                    pud;                  /*     8     8 */
	pmd_t *                    pmd;                  /*    16     8 */
	pte_t *                    pte;                  /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct kexec_segment {
	union {
		void *             buf;                  /*     0     8 */
		void *             kbuf;                 /*     0     8 */
	};                                               /*     0     8 */
	size_t                     bufsz;                /*     8     8 */
	long unsigned int          mem;                  /*    16     8 */
	size_t                     memsz;                /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct purgatory_info {
	const Elf64_Ehdr  *        ehdr;                 /*     0     8 */
	Elf64_Shdr *               sechdrs;              /*     8     8 */
	void *                     purgatory_buf;        /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct kimage {
	kimage_entry_t             head;                 /*     0     8 */
	kimage_entry_t *           entry;                /*     8     8 */
	kimage_entry_t *           last_entry;           /*    16     8 */
	long unsigned int          start;                /*    24     8 */
	struct page *              control_code_page;    /*    32     8 */
	struct page *              swap_page;            /*    40     8 */
	void *                     vmcoreinfo_data_copy; /*    48     8 */
	long unsigned int          nr_segments;          /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct kexec_segment       segment[16];          /*    64   512 */
	/* --- cacheline 9 boundary (576 bytes) --- */
	struct list_head           control_pages;        /*   576    16 */
	struct list_head           dest_pages;           /*   592    16 */
	struct list_head           unusable_pages;       /*   608    16 */
	long unsigned int          control_page;         /*   624     8 */
	unsigned int               type:2;               /*   632: 0  4 */
	unsigned int               preserve_context:1;   /*   632: 2  4 */
	unsigned int               file_mode:1;          /*   632: 3  4 */

	/* XXX 28 bits hole, try to pack */
	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 10 boundary (640 bytes) --- */
	struct kimage_arch         arch;                 /*   640    32 */
	void *                     kernel_buf;           /*   672     8 */
	long unsigned int          kernel_buf_len;       /*   680     8 */
	void *                     initrd_buf;           /*   688     8 */
	long unsigned int          initrd_buf_len;       /*   696     8 */
	/* --- cacheline 11 boundary (704 bytes) --- */
	char *                     cmdline_buf;          /*   704     8 */
	long unsigned int          cmdline_buf_len;      /*   712     8 */
	const struct kexec_file_ops  * fops;             /*   720     8 */
	void *                     image_loader_data;    /*   728     8 */
	struct purgatory_info      purgatory_info;       /*   736    24 */
	void *                     elf_headers;          /*   760     8 */
	/* --- cacheline 12 boundary (768 bytes) --- */
	long unsigned int          elf_headers_sz;       /*   768     8 */
	long unsigned int          elf_load_addr;        /*   776     8 */

	/* size: 784, cachelines: 13, members: 29 */
	/* sum members: 776, holes: 1, sum holes: 4 */
	/* sum bitfield members: 4 bits, bit holes: 1, sum bit holes: 28 bits */
	/* last cacheline: 16 bytes */
};
struct kexec_file_ops {
	kexec_probe_t *            probe;                /*     0     8 */
	kexec_load_t *             load;                 /*     8     8 */
	kexec_cleanup_t *          cleanup;              /*    16     8 */
	kexec_verify_sig_t *       verify_sig;           /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct x86_mapping_info {
	void *                     (*alloc_pgt_page)(void *); /*     0     8 */
	void *                     context;              /*     8     8 */
	long unsigned int          page_flag;            /*    16     8 */
	long unsigned int          offset;               /*    24     8 */
	bool                       direct_gbpages;       /*    32     1 */

	/* XXX 7 bytes hole, try to pack */

	long unsigned int          kernpg_flag;          /*    40     8 */

	/* size: 48, cachelines: 1, members: 6 */
	/* sum members: 41, holes: 1, sum holes: 7 */
	/* last cacheline: 48 bytes */
};
struct init_pgtable_data {
	struct x86_mapping_info *  info;                 /*     0     8 */
	pgd_t *                    level4p;              /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct kexec_buf {
	struct kimage *            image;                /*     0     8 */
	void *                     buffer;               /*     8     8 */
	long unsigned int          bufsz;                /*    16     8 */
	long unsigned int          mem;                  /*    24     8 */
	long unsigned int          memsz;                /*    32     8 */
	long unsigned int          buf_align;            /*    40     8 */
	long unsigned int          buf_min;              /*    48     8 */
	long unsigned int          buf_max;              /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	bool                       top_down;             /*    64     1 */

	/* size: 72, cachelines: 2, members: 9 */
	/* padding: 7 */
	/* last cacheline: 8 bytes */
};
struct crash_mem_range {
	u64                        start;                /*     0     8 */
	u64                        end;                  /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct crash_mem {
	unsigned int               max_nr_ranges;        /*     0     4 */
	unsigned int               nr_ranges;            /*     4     4 */
	struct crash_mem_range     ranges[];             /*     8     0 */

	/* size: 8, cachelines: 1, members: 3 */
	/* last cacheline: 8 bytes */
};
struct crash_memmap_data {
	struct boot_params *       params;               /*     0     8 */
	unsigned int               type;                 /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct kexec_entry64_regs {
	uint64_t                   rax;                  /*     0     8 */
	uint64_t                   rcx;                  /*     8     8 */
	uint64_t                   rdx;                  /*    16     8 */
	uint64_t                   rbx;                  /*    24     8 */
	uint64_t                   rsp;                  /*    32     8 */
	uint64_t                   rbp;                  /*    40     8 */
	uint64_t                   rsi;                  /*    48     8 */
	uint64_t                   rdi;                  /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	uint64_t                   r8;                   /*    64     8 */
	uint64_t                   r9;                   /*    72     8 */
	uint64_t                   r10;                  /*    80     8 */
	uint64_t                   r11;                  /*    88     8 */
	uint64_t                   r12;                  /*    96     8 */
	uint64_t                   r13;                  /*   104     8 */
	uint64_t                   r14;                  /*   112     8 */
	uint64_t                   r15;                  /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	uint64_t                   rip;                  /*   128     8 */

	/* size: 136, cachelines: 3, members: 17 */
	/* last cacheline: 8 bytes */
};
struct efi_setup_data {
	u64                        fw_vendor;            /*     0     8 */
	u64                        __unused;             /*     8     8 */
	u64                        tables;               /*    16     8 */
	u64                        smbios;               /*    24     8 */
	u64                        reserved[8];          /*    32    64 */

	/* size: 96, cachelines: 2, members: 5 */
	/* last cacheline: 32 bytes */
};
struct bzimage64_data {
	void *                     bootparams_buf;       /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct prev_kprobe {
	struct kprobe *            kp;                   /*     0     8 */
	long unsigned int          status;               /*     8     8 */
	long unsigned int          old_flags;            /*    16     8 */
	long unsigned int          saved_flags;          /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct kprobe_ctlblk {
	long unsigned int          kprobe_status;        /*     0     8 */
	long unsigned int          kprobe_old_flags;     /*     8     8 */
	long unsigned int          kprobe_saved_flags;   /*    16     8 */
	struct prev_kprobe         prev_kprobe;          /*    24    32 */

	/* size: 56, cachelines: 1, members: 4 */
	/* last cacheline: 56 bytes */
};
struct kretprobe_instance {
	union {
		struct hlist_node  hlist;                /*     0    16 */
		struct callback_head rcu __attribute__((__aligned__(8))); /*     0    16 */
	} __attribute__((__aligned__(8)));               /*     0    16 */
	struct kretprobe *         rp;                   /*    16     8 */
	kprobe_opcode_t *          ret_addr;             /*    24     8 */
	struct task_struct *       task;                 /*    32     8 */
	void *                     fp;                   /*    40     8 */
	char                       data[];               /*    48     0 */

	/* size: 48, cachelines: 1, members: 6 */
	/* forced alignments: 1 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct kretprobe {
	struct kprobe              kp;                   /*     0   120 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 1 boundary (64 bytes) was 56 bytes ago --- */
	kretprobe_handler_t        handler;              /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	kretprobe_handler_t        entry_handler;        /*   128     8 */
	int                        maxactive;            /*   136     4 */
	int                        nmissed;              /*   140     4 */
	size_t                     data_size;            /*   144     8 */
	struct hlist_head          free_instances;       /*   152     8 */
	raw_spinlock_t             lock;                 /*   160     4 */

	/* size: 168, cachelines: 3, members: 8 */
	/* padding: 4 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 40 bytes */
};
struct kretprobe_blackpoint {
	const char  *              name;                 /*     0     8 */
	void *                     addr;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct kprobe_insn_cache {
	struct mutex               mutex;                /*     0    32 */
	void *                     (*alloc)(void);       /*    32     8 */
	void                       (*free)(void *);      /*    40     8 */
	const char  *              sym;                  /*    48     8 */
	struct list_head           pages;                /*    56    16 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	size_t                     insn_size;            /*    72     8 */
	int                        nr_garbage;           /*    80     4 */

	/* size: 88, cachelines: 2, members: 7 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct arch_optimized_insn {
	kprobe_opcode_t            copied_insn[4];       /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	kprobe_opcode_t *          insn;                 /*     8     8 */
	size_t                     size;                 /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct optimized_kprobe {
	struct kprobe              kp;                   /*     0   120 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 1 boundary (64 bytes) was 56 bytes ago --- */
	struct list_head           list;                 /*   120    16 */
	/* --- cacheline 2 boundary (128 bytes) was 8 bytes ago --- */
	struct arch_optimized_insn optinsn;              /*   136    24 */

	/* size: 160, cachelines: 3, members: 3 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 32 bytes */
};
struct kgdb_bkpt {
	long unsigned int          bpt_addr;             /*     0     8 */
	unsigned char              saved_instr[1];       /*     8     1 */

	/* XXX 3 bytes hole, try to pack */

	enum kgdb_bptype           type;                 /*    12     4 */
	enum kgdb_bpstate          state;                /*    16     4 */

	/* size: 24, cachelines: 1, members: 4 */
	/* sum members: 17, holes: 1, sum holes: 3 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct dbg_reg_def_t {
	char *                     name;                 /*     0     8 */
	int                        size;                 /*     8     4 */
	int                        offset;               /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct hw_breakpoint {
	unsigned int               enabled;              /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          addr;                 /*     8     8 */
	int                        len;                  /*    16     4 */
	int                        type;                 /*    20     4 */
	struct perf_event * *      pev;                  /*    24     8 */

	/* size: 32, cachelines: 1, members: 5 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct console {
	char                       name[16];             /*     0    16 */
	void                       (*write)(struct console *, const char  *, unsigned int); /*    16     8 */
	int                        (*read)(struct console *, char *, unsigned int); /*    24     8 */
	struct tty_driver *        (*device)(struct console *, int *); /*    32     8 */
	void                       (*unblank)(void);     /*    40     8 */
	int                        (*setup)(struct console *, char *); /*    48     8 */
	int                        (*exit)(struct console *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*match)(struct console *, char *, int, char *); /*    64     8 */
	short int                  flags;                /*    72     2 */
	short int                  index;                /*    74     2 */
	int                        cflag;                /*    76     4 */
	uint                       ispeed;               /*    80     4 */
	uint                       ospeed;               /*    84     4 */
	void *                     data;                 /*    88     8 */
	struct console *           next;                 /*    96     8 */

	/* size: 104, cachelines: 2, members: 15 */
	/* last cacheline: 40 bytes */
};
struct hpet_data {
	long unsigned int          hd_phys_address;      /*     0     8 */
	void *                     hd_address;           /*     8     8 */
	short unsigned int         hd_nirqs;             /*    16     2 */

	/* XXX 2 bytes hole, try to pack */

	unsigned int               hd_state;             /*    20     4 */
	unsigned int               hd_irq[32];           /*    24   128 */

	/* size: 152, cachelines: 3, members: 5 */
	/* sum members: 150, holes: 1, sum holes: 2 */
	/* last cacheline: 24 bytes */
};
struct hpet_channel {
	struct clock_event_device  evt __attribute__((__aligned__(64))); /*     0   256 */

	/* XXX last struct has 48 bytes of padding */

	/* --- cacheline 4 boundary (256 bytes) --- */
	unsigned int               num;                  /*   256     4 */
	unsigned int               cpu;                  /*   260     4 */
	unsigned int               irq;                  /*   264     4 */
	unsigned int               in_use;               /*   268     4 */
	enum hpet_mode             mode;                 /*   272     4 */
	unsigned int               boot_cfg;             /*   276     4 */
	char                       name[10];             /*   280    10 */

	/* size: 320, cachelines: 5, members: 8 */
	/* padding: 30 */
	/* paddings: 1, sum paddings: 48 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(64)));
struct hpet_base {
	unsigned int               nr_channels;          /*     0     4 */
	unsigned int               nr_clockevents;       /*     4     4 */
	unsigned int               boot_cfg;             /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct hpet_channel *      channels;             /*    16     8 */

	/* size: 24, cachelines: 1, members: 4 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
union hpet_lock {
	struct {
		arch_spinlock_t    lock;               /*     0     4 */
		u32                value;              /*     4     4 */
	};                                             /*     0     8 */
	u64                        lockval;            /*     0     8 */
};
struct amd_nb_bus_dev_range {
	u8                         bus;                  /*     0     1 */
	u8                         dev_base;             /*     1     1 */
	u8                         dev_limit;            /*     2     1 */

	/* size: 3, cachelines: 1, members: 3 */
	/* last cacheline: 3 bytes */
};
struct amd_northbridge_info {
	u16                        num;                  /*     0     2 */

	/* XXX 6 bytes hole, try to pack */

	u64                        flags;                /*     8     8 */
	struct amd_northbridge *   nb;                   /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 18, holes: 1, sum holes: 6 */
	/* last cacheline: 24 bytes */
};
struct swait_queue {
	struct task_struct *       task;                 /*     0     8 */
	struct list_head           task_list;            /*     8    16 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct kvm_steal_time {
	__u64                      steal;                /*     0     8 */
	__u32                      version;              /*     8     4 */
	__u32                      flags;                /*    12     4 */
	__u8                       preempted;            /*    16     1 */
	__u8                       u8_pad[3];            /*    17     3 */
	__u32                      pad[11];              /*    20    44 */

	/* size: 64, cachelines: 1, members: 6 */
};
struct kvm_vcpu_pv_apf_data {
	__u32                      flags;                /*     0     4 */
	__u32                      token;                /*     4     4 */
	__u8                       pad[56];              /*     8    56 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	__u32                      enabled;              /*    64     4 */

	/* size: 68, cachelines: 2, members: 4 */
	/* last cacheline: 4 bytes */
};
struct kvm_task_sleep_node {
	struct hlist_node          link;                 /*     0    16 */
	struct swait_queue_head    wq;                   /*    16    24 */
	u32                        token;                /*    40     4 */
	int                        cpu;                  /*    44     4 */

	/* size: 48, cachelines: 1, members: 4 */
	/* last cacheline: 48 bytes */
};
struct kvm_task_sleep_head {
	raw_spinlock_t             lock;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct hlist_head          list;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct branch {
	unsigned char              opcode;               /*     0     1 */
	u32                        delta;                /*     1     4 */

	/* size: 5, cachelines: 1, members: 2 */
	/* last cacheline: 5 bytes */
} __attribute__((__packed__));
struct patch_lock {
	unsigned char              queued_spin_unlock[3]; /*     0     3 */
	unsigned char              vcpu_is_preempted[2]; /*     3     2 */

	/* size: 5, cachelines: 1, members: 2 */
	/* last cacheline: 5 bytes */
};
struct scan_area {
	u64                        addr;                 /*     0     8 */
	u64                        size;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct arch_uprobe {
	union {
		u8                 insn[16];             /*     0    16 */
		u8                 ixol[16];             /*     0    16 */
	};                                               /*     0    16 */
	const struct uprobe_xol_ops  * ops;              /*    16     8 */
	union {
		struct {
			s32        offs;                 /*    24     4 */
			u8         ilen;                 /*    28     1 */
			u8         opc1;                 /*    29     1 */
		} branch;                                /*    24     8 */
		struct {
			u8         fixups;               /*    24     1 */
			u8         ilen;                 /*    25     1 */
		} defparam;                              /*    24     2 */
		struct {
			u8         reg_offset;           /*    24     1 */
			u8         ilen;                 /*    25     1 */
		} push;                                  /*    24     2 */
	};                                               /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* last cacheline: 32 bytes */
};
struct uprobe_xol_ops {
	bool                       (*emulate)(struct arch_uprobe *, struct pt_regs *); /*     0     8 */
	int                        (*pre_xol)(struct arch_uprobe *, struct pt_regs *); /*     8     8 */
	int                        (*post_xol)(struct arch_uprobe *, struct pt_regs *); /*    16     8 */
	void                       (*abort)(struct arch_uprobe *, struct pt_regs *); /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct property_entry {
	const char  *              name;                 /*     0     8 */
	size_t                     length;               /*     8     8 */
	bool                       is_inline;            /*    16     1 */

	/* XXX 3 bytes hole, try to pack */

	enum dev_prop_type         type;                 /*    20     4 */
	union {
		const void  *      pointer;              /*    24     8 */
		union {
			u8         u8_data[8];           /*    24     8 */
			u16        u16_data[4];          /*    24     8 */
			u32        u32_data[2];          /*    24     8 */
			u64        u64_data[1];          /*    24     8 */
			const char  * str[1];            /*    24     8 */
		} value;                                 /*    24     8 */
	};                                               /*    24     8 */

	/* size: 32, cachelines: 1, members: 5 */
	/* sum members: 29, holes: 1, sum holes: 3 */
	/* last cacheline: 32 bytes */
};
struct simplefb_platform_data {
	u32                        width;                /*     0     4 */
	u32                        height;               /*     4     4 */
	u32                        stride;               /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	const char  *              format;               /*    16     8 */

	/* size: 24, cachelines: 1, members: 4 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct efifb_dmi_info {
	char *                     optname;              /*     0     8 */
	long unsigned int          base;                 /*     8     8 */
	int                        stride;               /*    16     4 */
	int                        width;                /*    20     4 */
	int                        height;               /*    24     4 */
	int                        flags;                /*    28     4 */

	/* size: 32, cachelines: 1, members: 6 */
	/* last cacheline: 32 bytes */
};
struct va_format {
	const char  *              fmt;                  /*     0     8 */
	va_list *                  va;                   /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct es_fault_info {
	long unsigned int          vector;               /*     0     8 */
	long unsigned int          error_code;           /*     8     8 */
	long unsigned int          cr2;                  /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct es_em_ctxt {
	struct pt_regs *           regs;                 /*     0     8 */
	struct insn                insn;                 /*     8   112 */
	/* --- cacheline 1 boundary (64 bytes) was 56 bytes ago --- */
	struct es_fault_info       fi;                   /*   120    24 */

	/* size: 144, cachelines: 3, members: 3 */
	/* last cacheline: 16 bytes */
};
struct sev_es_runtime_data {
	struct ghcb                ghcb_page;            /*     0  4096 */
	/* --- cacheline 64 boundary (4096 bytes) --- */
	struct ghcb                backup_ghcb;          /*  4096  4096 */
	/* --- cacheline 128 boundary (8192 bytes) --- */
	bool                       ghcb_active;          /*  8192     1 */
	bool                       backup_ghcb_active;   /*  8193     1 */

	/* XXX 6 bytes hole, try to pack */

	long unsigned int          dr7;                  /*  8200     8 */

	/* size: 8208, cachelines: 129, members: 5 */
	/* sum members: 8202, holes: 1, sum holes: 6 */
	/* last cacheline: 16 bytes */
};
struct ghcb_state {
	struct ghcb *              ghcb;                 /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct pci_hostbridge_probe {
	u32                        bus;                  /*     0     4 */
	u32                        slot;                 /*     4     4 */
	u32                        vendor;               /*     8     4 */
	u32                        device;               /*    12     4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct trace_print_flags {
	long unsigned int          mask;                 /*     0     8 */
	const char  *              name;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct trace_event_raw_tlb_flush {
	struct trace_entry         ent;                  /*     0     8 */
	int                        reason;               /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          pages;                /*    16     8 */
	char                       __data[];             /*    24     0 */

	/* size: 24, cachelines: 1, members: 4 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct trace_event_data_offsets_tlb_flush {

	/* size: 0, cachelines: 0, members: 0 */
};
struct map_range {
	long unsigned int          start;                /*     0     8 */
	long unsigned int          end;                  /*     8     8 */
	unsigned int               page_size_mask;       /*    16     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct mhp_params {
	struct vmem_altmap *       altmap;               /*     0     8 */
	pgprot_t                   pgprot;               /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct mem_section_usage {
	long unsigned int          subsection_map[1];    /*     0     8 */
	long unsigned int          pageblock_flags[];    /*     8     0 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct mem_section {
	long unsigned int          section_mem_map;      /*     0     8 */
	struct mem_section_usage * usage;                /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct kcore_list {
	struct list_head           list;                 /*     0    16 */
	long unsigned int          addr;                 /*    16     8 */
	long unsigned int          vaddr;                /*    24     8 */
	size_t                     size;                 /*    32     8 */
	int                        type;                 /*    40     4 */

	/* size: 48, cachelines: 1, members: 5 */
	/* padding: 4 */
	/* last cacheline: 48 bytes */
};
struct hstate {
	struct mutex               resize_lock;          /*     0    32 */
	int                        next_nid_to_alloc;    /*    32     4 */
	int                        next_nid_to_free;     /*    36     4 */
	unsigned int               order;                /*    40     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          mask;                 /*    48     8 */
	long unsigned int          max_huge_pages;       /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	long unsigned int          nr_huge_pages;        /*    64     8 */
	long unsigned int          free_huge_pages;      /*    72     8 */
	long unsigned int          resv_huge_pages;      /*    80     8 */
	long unsigned int          surplus_huge_pages;   /*    88     8 */
	long unsigned int          nr_overcommit_huge_pages; /*    96     8 */
	struct list_head           hugepage_activelist;  /*   104    16 */
	struct list_head           hugepage_freelists[1024]; /*   120 16384 */
	/* --- cacheline 257 boundary (16448 bytes) was 56 bytes ago --- */
	unsigned int               max_huge_pages_node[1024]; /* 16504  4096 */
	/* --- cacheline 321 boundary (20544 bytes) was 56 bytes ago --- */
	unsigned int               nr_huge_pages_node[1024]; /* 20600  4096 */
	/* --- cacheline 385 boundary (24640 bytes) was 56 bytes ago --- */
	unsigned int               free_huge_pages_node[1024]; /* 24696  4096 */
	/* --- cacheline 449 boundary (28736 bytes) was 56 bytes ago --- */
	unsigned int               surplus_huge_pages_node[1024]; /* 28792  4096 */
	/* --- cacheline 513 boundary (32832 bytes) was 56 bytes ago --- */
	unsigned int               resv_huge_pages_node[1024]; /* 32888  4096 */
	/* --- cacheline 577 boundary (36928 bytes) was 56 bytes ago --- */
	unsigned int               optimize_vmemmap_pages; /* 36984     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 578 boundary (36992 bytes) --- */
	struct cftype              cgroup_files_dfl[7];  /* 36992  1680 */
	/* --- cacheline 604 boundary (38656 bytes) was 16 bytes ago --- */
	struct cftype              cgroup_files_legacy[9]; /* 38672  2160 */
	/* --- cacheline 638 boundary (40832 bytes) --- */
	char                       name[32];             /* 40832    32 */

	/* size: 40864, cachelines: 639, members: 22 */
	/* sum members: 40856, holes: 2, sum holes: 8 */
	/* last cacheline: 32 bytes */
};
struct trace_event_raw_x86_exceptions {
	struct trace_entry         ent;                  /*     0     8 */
	long unsigned int          address;              /*     8     8 */
	long unsigned int          ip;                   /*    16     8 */
	long unsigned int          error_code;           /*    24     8 */
	char                       __data[];             /*    32     0 */

	/* size: 32, cachelines: 1, members: 5 */
	/* last cacheline: 32 bytes */
};
struct trace_event_data_offsets_x86_exceptions {

	/* size: 0, cachelines: 0, members: 0 */
};
struct ioremap_desc {
	unsigned int               flags;                /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct hugepage_subpool {
	spinlock_t                 lock;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	long int                   count;                /*     8     8 */
	long int                   max_hpages;           /*    16     8 */
	long int                   used_hpages;          /*    24     8 */
	struct hstate *            hstate;               /*    32     8 */
	long int                   min_hpages;           /*    40     8 */
	long int                   rsv_hpages;           /*    48     8 */

	/* size: 56, cachelines: 1, members: 7 */
	/* sum members: 52, holes: 1, sum holes: 4 */
	/* last cacheline: 56 bytes */
};
struct hugetlbfs_sb_info {
	long int                   max_inodes;           /*     0     8 */
	long int                   free_inodes;          /*     8     8 */
	spinlock_t                 stat_lock;            /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	struct hstate *            hstate;               /*    24     8 */
	struct hugepage_subpool *  spool;                /*    32     8 */
	kuid_t                     uid;                  /*    40     4 */
	kgid_t                     gid;                  /*    44     4 */
	umode_t                    mode;                 /*    48     2 */

	/* size: 56, cachelines: 1, members: 8 */
	/* sum members: 46, holes: 1, sum holes: 4 */
	/* padding: 6 */
	/* last cacheline: 56 bytes */
};
struct entry_stack_page {
	struct entry_stack         stack;                /*     0  4096 */

	/* size: 4096, cachelines: 64, members: 1 */
} __attribute__((__aligned__(4096)));
struct debug_store_buffers {
	char                       bts_buffer[65536];    /*     0 65536 */
	/* --- cacheline 1024 boundary (65536 bytes) --- */
	char                       pebs_buffer[65536];   /* 65536 65536 */

	/* size: 131072, cachelines: 2048, members: 2 */
};
struct exception_stacks {
	char                       DF_stack_guard[0];    /*     0     0 */
	char                       DF_stack[8192];       /*     0  8192 */
	/* --- cacheline 128 boundary (8192 bytes) --- */
	char                       NMI_stack_guard[0];   /*  8192     0 */
	char                       NMI_stack[8192];      /*  8192  8192 */
	/* --- cacheline 256 boundary (16384 bytes) --- */
	char                       DB_stack_guard[0];    /* 16384     0 */
	char                       DB_stack[8192];       /* 16384  8192 */
	/* --- cacheline 384 boundary (24576 bytes) --- */
	char                       MCE_stack_guard[0];   /* 24576     0 */
	char                       MCE_stack[8192];      /* 24576  8192 */
	/* --- cacheline 512 boundary (32768 bytes) --- */
	char                       VC_stack_guard[0];    /* 32768     0 */
	char                       VC_stack[8192];       /* 32768  8192 */
	/* --- cacheline 640 boundary (40960 bytes) --- */
	char                       VC2_stack_guard[0];   /* 40960     0 */
	char                       VC2_stack[8192];      /* 40960  8192 */
	/* --- cacheline 768 boundary (49152 bytes) --- */
	char                       IST_top_guard[];      /* 49152     0 */

	/* size: 49152, cachelines: 768, members: 13 */
};
struct cpu_entry_area {
	char                       gdt[4096];            /*     0  4096 */
	/* --- cacheline 64 boundary (4096 bytes) --- */
	struct entry_stack_page    entry_stack_page __attribute__((__aligned__(4096))); /*  4096  4096 */
	/* --- cacheline 128 boundary (8192 bytes) --- */
	struct tss_struct          tss __attribute__((__aligned__(4096))); /*  8192 20480 */

	/* XXX last struct has 3960 bytes of padding */

	/* --- cacheline 448 boundary (28672 bytes) --- */
	struct cea_exception_stacks estacks;             /* 28672 77824 */
	/* --- cacheline 1664 boundary (106496 bytes) --- */
	struct debug_store         cpu_debug_store __attribute__((__aligned__(4096))); /* 106496  4096 */

	/* XXX last struct has 3936 bytes of padding */

	/* --- cacheline 1728 boundary (110592 bytes) --- */
	struct debug_store_buffers cpu_debug_buffers;    /* 110592 131072 */

	/* size: 241664, cachelines: 3776, members: 6 */
	/* paddings: 2, sum paddings: 7896 */
	/* forced alignments: 3 */
} __attribute__((__aligned__(4096)));
struct cpa_data {
	long unsigned int *        vaddr;                /*     0     8 */
	pgd_t *                    pgd;                  /*     8     8 */
	pgprot_t                   mask_set;             /*    16     8 */
	pgprot_t                   mask_clr;             /*    24     8 */
	long unsigned int          numpages;             /*    32     8 */
	long unsigned int          curpage;              /*    40     8 */
	long unsigned int          pfn;                  /*    48     8 */
	unsigned int               flags;                /*    56     4 */
	unsigned int               force_split:1;        /*    60: 0  4 */
	unsigned int               force_static_prot:1;  /*    60: 1  4 */
	unsigned int               force_flush_all:1;    /*    60: 2  4 */

	/* XXX 29 bits hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	struct page * *            pages;                /*    64     8 */

	/* size: 72, cachelines: 2, members: 12 */
	/* sum members: 68 */
	/* sum bitfield members: 3 bits, bit holes: 1, sum bit holes: 29 bits */
	/* last cacheline: 8 bytes */
};
struct memtype {
	u64                        start;                /*     0     8 */
	u64                        end;                  /*     8     8 */
	u64                        subtree_max_end;      /*    16     8 */
	enum page_cache_mode       type;                 /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	struct rb_node             rb __attribute__((__aligned__(8))); /*    32    24 */

	/* size: 56, cachelines: 1, members: 5 */
	/* sum members: 52, holes: 1, sum holes: 4 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 4 */
	/* last cacheline: 56 bytes */
} __attribute__((__aligned__(8)));
struct pagerange_state {
	long unsigned int          cur_pfn;              /*     0     8 */
	int                        ram;                  /*     8     4 */
	int                        not_ram;              /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct rb_augment_callbacks {
	void                       (*propagate)(struct rb_node *, struct rb_node *); /*     0     8 */
	void                       (*copy)(struct rb_node *, struct rb_node *); /*     8     8 */
	void                       (*rotate)(struct rb_node *, struct rb_node *); /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct numa_memblk {
	u64                        start;                /*     0     8 */
	u64                        end;                  /*     8     8 */
	int                        nid;                  /*    16     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct numa_meminfo {
	int                        nr_blks;              /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct numa_memblk         blk[2048];            /*     8 49152 */

	/* size: 49160, cachelines: 769, members: 2 */
	/* sum members: 49156, holes: 1, sum holes: 4 */
	/* last cacheline: 8 bytes */
};
struct acpi_srat_cpu_affinity {
	struct acpi_subtable_header header;              /*     0     2 */
	u8                         proximity_domain_lo;  /*     2     1 */
	u8                         apic_id;              /*     3     1 */
	u32                        flags;                /*     4     4 */
	u8                         local_sapic_eid;      /*     8     1 */
	u8                         proximity_domain_hi[3]; /*     9     3 */
	u32                        clock_domain;         /*    12     4 */

	/* size: 16, cachelines: 1, members: 7 */
	/* last cacheline: 16 bytes */
};
struct acpi_srat_x2apic_cpu_affinity {
	struct acpi_subtable_header header;              /*     0     2 */
	u16                        reserved;             /*     2     2 */
	u32                        proximity_domain;     /*     4     4 */
	u32                        apic_id;              /*     8     4 */
	u32                        flags;                /*    12     4 */
	u32                        clock_domain;         /*    16     4 */
	u32                        reserved2;            /*    20     4 */

	/* size: 24, cachelines: 1, members: 7 */
	/* last cacheline: 24 bytes */
};
struct rnd_state {
	__u32                      s1;                   /*     0     4 */
	__u32                      s2;                   /*     4     4 */
	__u32                      s3;                   /*     8     4 */
	__u32                      s4;                   /*    12     4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct kaslr_memory_region {
	long unsigned int *        base;                 /*     0     8 */
	long unsigned int          size_tb;              /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct sme_populate_pgd_data {
	void *                     pgtable_area;         /*     0     8 */
	pgd_t *                    pgd;                  /*     8     8 */
	pmdval_t                   pmd_flags;            /*    16     8 */
	pteval_t                   pte_flags;            /*    24     8 */
	long unsigned int          paddr;                /*    32     8 */
	long unsigned int          vaddr;                /*    40     8 */
	long unsigned int          vaddr_end;            /*    48     8 */

	/* size: 56, cachelines: 1, members: 7 */
	/* last cacheline: 56 bytes */
};
struct crypto_async_request {
	struct list_head           list;                 /*     0    16 */
	crypto_completion_t        complete;             /*    16     8 */
	void *                     data;                 /*    24     8 */
	struct crypto_tfm *        tfm;                  /*    32     8 */
	u32                        flags;                /*    40     4 */

	/* size: 48, cachelines: 1, members: 5 */
	/* padding: 4 */
	/* last cacheline: 48 bytes */
};
struct crypto_type {
	unsigned int               (*ctxsize)(struct crypto_alg *, u32, u32); /*     0     8 */
	unsigned int               (*extsize)(struct crypto_alg *); /*     8     8 */
	int                        (*init)(struct crypto_tfm *, u32, u32); /*    16     8 */
	int                        (*init_tfm)(struct crypto_tfm *); /*    24     8 */
	void                       (*show)(struct seq_file *, struct crypto_alg *); /*    32     8 */
	int                        (*report)(struct sk_buff *, struct crypto_alg *); /*    40     8 */
	void                       (*free)(struct crypto_instance *); /*    48     8 */
	unsigned int               type;                 /*    56     4 */
	unsigned int               maskclear;            /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	unsigned int               maskset;              /*    64     4 */
	unsigned int               tfmsize;              /*    68     4 */

	/* size: 72, cachelines: 2, members: 11 */
	/* last cacheline: 8 bytes */
};
struct crypto_instance {
	struct crypto_alg          alg __attribute__((__aligned__(8))); /*     0   384 */
	/* --- cacheline 6 boundary (384 bytes) --- */
	struct crypto_template *   tmpl;                 /*   384     8 */
	union {
		struct hlist_node  list;                 /*   392    16 */
		struct crypto_spawn * spawns;            /*   392     8 */
	};                                               /*   392    16 */
	void *                     __ctx[] __attribute__((__aligned__(8))); /*   408     0 */

	/* size: 408, cachelines: 7, members: 4 */
	/* forced alignments: 2 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct crypto_spawn {
	struct list_head           list;                 /*     0    16 */
	struct crypto_alg *        alg;                  /*    16     8 */
	union {
		struct crypto_instance * inst;           /*    24     8 */
		struct crypto_spawn * next;              /*    24     8 */
	};                                               /*    24     8 */
	const struct crypto_type  * frontend;            /*    32     8 */
	u32                        mask;                 /*    40     4 */
	bool                       dead;                 /*    44     1 */
	bool                       registered;           /*    45     1 */

	/* size: 48, cachelines: 1, members: 7 */
	/* padding: 2 */
	/* last cacheline: 48 bytes */
};
struct crypto_template {
	struct list_head           list;                 /*     0    16 */
	struct hlist_head          instances;            /*    16     8 */
	struct module *            module;               /*    24     8 */
	int                        (*create)(struct crypto_template *, struct rtattr * *); /*    32     8 */
	char                       name[128];            /*    40   128 */

	/* size: 168, cachelines: 3, members: 5 */
	/* last cacheline: 40 bytes */
};
struct scatter_walk {
	struct scatterlist *       sg;                   /*     0     8 */
	unsigned int               offset;               /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct skcipher_request {
	unsigned int               cryptlen;             /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	u8 *                       iv;                   /*     8     8 */
	struct scatterlist *       src;                  /*    16     8 */
	struct scatterlist *       dst;                  /*    24     8 */
	struct crypto_async_request base;                /*    32    48 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
	void *                     __ctx[] __attribute__((__aligned__(8))); /*    80     0 */

	/* size: 80, cachelines: 2, members: 6 */
	/* sum members: 76, holes: 1, sum holes: 4 */
	/* paddings: 1, sum paddings: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct crypto_skcipher {
	unsigned int               reqsize;              /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct crypto_tfm          base __attribute__((__aligned__(8))); /*     8    24 */

	/* size: 32, cachelines: 1, members: 2 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 4 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct skcipher_walk {
	union {
		struct {
			struct page * page;              /*     0     8 */
			long unsigned int offset;        /*     8     8 */
		} phys;                                  /*     0    16 */
		struct {
			u8 *       page;                 /*     0     8 */
			void *     addr;                 /*     8     8 */
		} virt;                                  /*     0    16 */
	} src;                                           /*     0    16 */
	union {
		struct {
			struct page * page;              /*    16     8 */
			long unsigned int offset;        /*    24     8 */
		} phys;                                  /*    16    16 */
		struct {
			u8 *       page;                 /*    16     8 */
			void *     addr;                 /*    24     8 */
		} virt;                                  /*    16    16 */
	} dst;                                           /*    16    16 */
	struct scatter_walk        in;                   /*    32    16 */

	/* XXX last struct has 4 bytes of padding */

	unsigned int               nbytes;               /*    48     4 */

	/* XXX 4 bytes hole, try to pack */

	struct scatter_walk        out;                  /*    56    16 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	unsigned int               total;                /*    72     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           buffers;              /*    80    16 */
	u8 *                       page;                 /*    96     8 */
	u8 *                       buffer;               /*   104     8 */
	u8 *                       oiv;                  /*   112     8 */
	void *                     iv;                   /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	unsigned int               ivsize;               /*   128     4 */
	int                        flags;                /*   132     4 */
	unsigned int               blocksize;            /*   136     4 */
	unsigned int               stride;               /*   140     4 */
	unsigned int               alignmask;            /*   144     4 */

	/* size: 152, cachelines: 3, members: 16 */
	/* sum members: 140, holes: 2, sum holes: 8 */
	/* padding: 4 */
	/* paddings: 2, sum paddings: 8 */
	/* last cacheline: 24 bytes */
};
struct common_glue_func_entry {
	unsigned int               num_blocks;           /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	union {
		common_glue_func_t ecb;                  /*     8     8 */
		common_glue_cbc_func_t cbc;              /*     8     8 */
		common_glue_ctr_func_t ctr;              /*     8     8 */
		common_glue_xts_func_t xts;              /*     8     8 */
	} fn_u;                                          /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct common_glue_ctx {
	unsigned int               num_funcs;            /*     0     4 */
	int                        fpu_blocks_limit;     /*     4     4 */
	struct common_glue_func_entry funcs[];           /*     8     0 */

	/* size: 8, cachelines: 1, members: 3 */
	/* last cacheline: 8 bytes */
};
struct crypto_aes_ctx {
	u32                        key_enc[60];          /*     0   240 */
	/* --- cacheline 3 boundary (192 bytes) was 48 bytes ago --- */
	u32                        key_dec[60];          /*   240   240 */
	/* --- cacheline 7 boundary (448 bytes) was 32 bytes ago --- */
	u32                        key_length;           /*   480     4 */

	/* size: 484, cachelines: 8, members: 3 */
	/* last cacheline: 36 bytes */
};
struct skcipher_alg {
	int                        (*setkey)(struct crypto_skcipher *, const u8  *, unsigned int); /*     0     8 */
	int                        (*encrypt)(struct skcipher_request *); /*     8     8 */
	int                        (*decrypt)(struct skcipher_request *); /*    16     8 */
	int                        (*init)(struct crypto_skcipher *); /*    24     8 */
	void                       (*exit)(struct crypto_skcipher *); /*    32     8 */
	unsigned int               min_keysize;          /*    40     4 */
	unsigned int               max_keysize;          /*    44     4 */
	unsigned int               ivsize;               /*    48     4 */
	unsigned int               chunksize;            /*    52     4 */
	unsigned int               walksize;             /*    56     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	struct crypto_alg          base __attribute__((__aligned__(8))); /*    64   384 */

	/* size: 448, cachelines: 7, members: 11 */
	/* sum members: 444, holes: 1, sum holes: 4 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 4 */
} __attribute__((__aligned__(8)));
struct aead_request {
	struct crypto_async_request base;                /*     0    48 */

	/* XXX last struct has 4 bytes of padding */

	unsigned int               assoclen;             /*    48     4 */
	unsigned int               cryptlen;             /*    52     4 */
	u8 *                       iv;                   /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct scatterlist *       src;                  /*    64     8 */
	struct scatterlist *       dst;                  /*    72     8 */
	void *                     __ctx[] __attribute__((__aligned__(8))); /*    80     0 */

	/* size: 80, cachelines: 2, members: 7 */
	/* paddings: 1, sum paddings: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct aead_alg {
	int                        (*setkey)(struct crypto_aead *, const u8  *, unsigned int); /*     0     8 */
	int                        (*setauthsize)(struct crypto_aead *, unsigned int); /*     8     8 */
	int                        (*encrypt)(struct aead_request *); /*    16     8 */
	int                        (*decrypt)(struct aead_request *); /*    24     8 */
	int                        (*init)(struct crypto_aead *); /*    32     8 */
	void                       (*exit)(struct crypto_aead *); /*    40     8 */
	unsigned int               ivsize;               /*    48     4 */
	unsigned int               maxauthsize;          /*    52     4 */
	unsigned int               chunksize;            /*    56     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	struct crypto_alg          base __attribute__((__aligned__(8))); /*    64   384 */

	/* size: 448, cachelines: 7, members: 10 */
	/* sum members: 444, holes: 1, sum holes: 4 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 4 */
} __attribute__((__aligned__(8)));
struct crypto_aead {
	unsigned int               authsize;             /*     0     4 */
	unsigned int               reqsize;              /*     4     4 */
	struct crypto_tfm          base __attribute__((__aligned__(8))); /*     8    24 */

	/* size: 32, cachelines: 1, members: 3 */
	/* forced alignments: 1 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct aesni_rfc4106_gcm_ctx {
	u8                         hash_subkey[16] __attribute__((__aligned__(16))); /*     0    16 */
	struct crypto_aes_ctx      aes_key_expanded __attribute__((__aligned__(16))); /*    16   484 */
	/* --- cacheline 7 boundary (448 bytes) was 52 bytes ago --- */
	u8                         nonce[4];             /*   500     4 */

	/* size: 512, cachelines: 8, members: 3 */
	/* padding: 8 */
	/* forced alignments: 2 */
} __attribute__((__aligned__(16)));
struct generic_gcmaes_ctx {
	u8                         hash_subkey[16] __attribute__((__aligned__(16))); /*     0    16 */
	struct crypto_aes_ctx      aes_key_expanded __attribute__((__aligned__(16))); /*    16   484 */

	/* size: 512, cachelines: 8, members: 2 */
	/* padding: 12 */
	/* forced alignments: 2 */
} __attribute__((__aligned__(16)));
struct aesni_xts_ctx {
	u8                         raw_tweak_ctx[484] __attribute__((__aligned__(16))); /*     0   484 */

	/* XXX 12 bytes hole, try to pack */

	/* --- cacheline 7 boundary (448 bytes) was 48 bytes ago --- */
	u8                         raw_crypt_ctx[484] __attribute__((__aligned__(16))); /*   496   484 */

	/* size: 992, cachelines: 16, members: 2 */
	/* sum members: 968, holes: 1, sum holes: 12 */
	/* padding: 12 */
	/* forced alignments: 2, forced holes: 1, sum forced holes: 12 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(16)));
struct gcm_context_data {
	u8                         aad_hash[16];         /*     0    16 */
	u64                        aad_length;           /*    16     8 */
	u64                        in_length;            /*    24     8 */
	u8                         partial_block_enc_key[16]; /*    32    16 */
	u8                         orig_IV[16];          /*    48    16 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u8                         current_counter[16];  /*    64    16 */
	u64                        partial_block_len;    /*    80     8 */
	u64                        unused;               /*    88     8 */
	u8                         hash_keys[256];       /*    96   256 */

	/* size: 352, cachelines: 6, members: 9 */
	/* last cacheline: 32 bytes */
};
struct aesni_gcm_tfm_s {
	void                       (*init)(void *, struct gcm_context_data *, u8 *, u8 *, const u8  *, long unsigned int); /*     0     8 */
	void                       (*enc_update)(void *, struct gcm_context_data *, u8 *, const u8  *, long unsigned int); /*     8     8 */
	void                       (*dec_update)(void *, struct gcm_context_data *, u8 *, const u8  *, long unsigned int); /*    16     8 */
	void                       (*finalize)(void *, struct gcm_context_data *, u8 *, long unsigned int); /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct shash_alg {
	int                        (*init)(struct shash_desc *); /*     0     8 */
	int                        (*update)(struct shash_desc *, const u8  *, unsigned int); /*     8     8 */
	int                        (*final)(struct shash_desc *, u8 *); /*    16     8 */
	int                        (*finup)(struct shash_desc *, const u8  *, unsigned int, u8 *); /*    24     8 */
	int                        (*digest)(struct shash_desc *, const u8  *, unsigned int, u8 *); /*    32     8 */
	int                        (*export)(struct shash_desc *, void *); /*    40     8 */
	int                        (*import)(struct shash_desc *, const void  *); /*    48     8 */
	int                        (*setkey)(struct crypto_shash *, const u8  *, unsigned int); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*init_tfm)(struct crypto_shash *); /*    64     8 */
	void                       (*exit_tfm)(struct crypto_shash *); /*    72     8 */
	unsigned int               descsize;             /*    80     4 */

	/* XXX 4 bytes hole, try to pack */

	unsigned int               digestsize __attribute__((__aligned__(8))); /*    88     4 */
	unsigned int               statesize;            /*    92     4 */
	struct crypto_alg          base __attribute__((__aligned__(8))); /*    96   384 */

	/* size: 480, cachelines: 8, members: 14 */
	/* sum members: 476, holes: 1, sum holes: 4 */
	/* forced alignments: 2, forced holes: 1, sum forced holes: 4 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct sha1_state {
	u32                        state[5];             /*     0    20 */

	/* XXX 4 bytes hole, try to pack */

	u64                        count;                /*    24     8 */
	u8                         buffer[64];           /*    32    64 */

	/* size: 96, cachelines: 2, members: 3 */
	/* sum members: 92, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct sha256_state {
	u32                        state[8];             /*     0    32 */
	u64                        count;                /*    32     8 */
	u8                         buf[64];              /*    40    64 */

	/* size: 104, cachelines: 2, members: 3 */
	/* last cacheline: 40 bytes */
};
struct sigcontext_32 {
	__u16                      gs;                   /*     0     2 */
	__u16                      __gsh;                /*     2     2 */
	__u16                      fs;                   /*     4     2 */
	__u16                      __fsh;                /*     6     2 */
	__u16                      es;                   /*     8     2 */
	__u16                      __esh;                /*    10     2 */
	__u16                      ds;                   /*    12     2 */
	__u16                      __dsh;                /*    14     2 */
	__u32                      di;                   /*    16     4 */
	__u32                      si;                   /*    20     4 */
	__u32                      bp;                   /*    24     4 */
	__u32                      sp;                   /*    28     4 */
	__u32                      bx;                   /*    32     4 */
	__u32                      dx;                   /*    36     4 */
	__u32                      cx;                   /*    40     4 */
	__u32                      ax;                   /*    44     4 */
	__u32                      trapno;               /*    48     4 */
	__u32                      err;                  /*    52     4 */
	__u32                      ip;                   /*    56     4 */
	__u16                      cs;                   /*    60     2 */
	__u16                      __csh;                /*    62     2 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	__u32                      flags;                /*    64     4 */
	__u32                      sp_at_signal;         /*    68     4 */
	__u16                      ss;                   /*    72     2 */
	__u16                      __ssh;                /*    74     2 */
	__u32                      fpstate;              /*    76     4 */
	__u32                      oldmask;              /*    80     4 */
	__u32                      cr2;                  /*    84     4 */

	/* size: 88, cachelines: 2, members: 28 */
	/* last cacheline: 24 bytes */
};
struct compat_sigaltstack {
	compat_uptr_t              ss_sp;                /*     0     4 */
	int                        ss_flags;             /*     4     4 */
	compat_size_t              ss_size;              /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct ucontext_ia32 {
	unsigned int               uc_flags;             /*     0     4 */
	unsigned int               uc_link;              /*     4     4 */
	compat_stack_t             uc_stack;             /*     8    12 */
	struct sigcontext_32       uc_mcontext;          /*    20    88 */
	/* --- cacheline 1 boundary (64 bytes) was 44 bytes ago --- */
	compat_sigset_t            uc_sigmask;           /*   108     8 */

	/* size: 116, cachelines: 2, members: 5 */
	/* last cacheline: 52 bytes */
};
struct sigframe_ia32 {
	u32                        pretcode;             /*     0     4 */
	int                        sig;                  /*     4     4 */
	struct sigcontext_32       sc;                   /*     8    88 */
	/* --- cacheline 1 boundary (64 bytes) was 32 bytes ago --- */
	struct _fpstate_32         fpstate_unused;       /*    96   624 */
	/* --- cacheline 11 boundary (704 bytes) was 16 bytes ago --- */
	unsigned int               extramask[1];         /*   720     4 */
	char                       retcode[8];           /*   724     8 */

	/* size: 736, cachelines: 12, members: 6 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct rt_sigframe_ia32 {
	u32                        pretcode;             /*     0     4 */
	int                        sig;                  /*     4     4 */
	u32                        pinfo;                /*     8     4 */
	u32                        puc;                  /*    12     4 */
	compat_siginfo_t           info;                 /*    16   128 */
	/* --- cacheline 2 boundary (128 bytes) was 16 bytes ago --- */
	struct ucontext_ia32       uc;                   /*   144   116 */
	/* --- cacheline 4 boundary (256 bytes) was 4 bytes ago --- */
	char                       retcode[8];           /*   260     8 */

	/* size: 268, cachelines: 5, members: 7 */
	/* last cacheline: 12 bytes */
};
struct efi_memory_map_data {
	phys_addr_t                phys_map;             /*     0     8 */
	long unsigned int          size;                 /*     8     8 */
	long unsigned int          desc_version;         /*    16     8 */
	long unsigned int          desc_size;            /*    24     8 */
	long unsigned int          flags;                /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct efi_mem_range {
	struct range               range;                /*     0    16 */
	u64                        attribute;            /*    16     8 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct efi_runtime_work {
	void *                     arg1;                 /*     0     8 */
	void *                     arg2;                 /*     8     8 */
	void *                     arg3;                 /*    16     8 */
	void *                     arg4;                 /*    24     8 */
	void *                     arg5;                 /*    32     8 */
	efi_status_t               status;               /*    40     8 */
	struct work_struct         work;                 /*    48    64 */
	/* --- cacheline 1 boundary (64 bytes) was 48 bytes ago --- */
	enum efi_rts_ids           efi_rts_id;           /*   112     4 */

	/* XXX 4 bytes hole, try to pack */

	struct completion          efi_rts_comp;         /*   120    32 */

	/* size: 152, cachelines: 3, members: 9 */
	/* sum members: 148, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct efi_scratch {
	u64                        phys_stack;           /*     0     8 */
	struct mm_struct *         prev_mm;              /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct pm_qos_request {
	struct plist_node          node;                 /*     0    40 */
	struct pm_qos_constraints * qos;                 /*    40     8 */

	/* size: 48, cachelines: 1, members: 2 */
	/* last cacheline: 48 bytes */
};
struct sfi_table_header {
	char                       sig[4];               /*     0     4 */
	u32                        len;                  /*     4     4 */
	u8                         rev;                  /*     8     1 */
	u8                         csum;                 /*     9     1 */
	char                       oem_id[6];            /*    10     6 */
	char                       oem_table_id[8];      /*    16     8 */

	/* size: 24, cachelines: 1, members: 6 */
	/* last cacheline: 24 bytes */
};
struct sfi_table_simple {
	struct sfi_table_header    header;               /*     0    24 */
	u64                        pentry[1];            /*    24     8 */

	/* size: 32, cachelines: 1, members: 2 */
	/* last cacheline: 32 bytes */
};
struct sfi_cpu_table_entry {
	u32                        apic_id;              /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct sfi_apic_table_entry {
	u64                        phys_addr;            /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct semaphore {
	raw_spinlock_t             lock;                 /*     0     4 */
	unsigned int               count;                /*     4     4 */
	struct list_head           wait_list;            /*     8    16 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
union partition_info_u {
	u64                        val;                /*     0     8 */
	struct {
		u64                hub_version:8;      /*     0: 0  8 */
		u64                partition_id:16;    /*     0: 8  8 */
		u64                coherence_id:16;    /*     0:24  8 */
		u64                region_size:24;     /*     0:40  8 */
	};                                             /*     0     8 */
};
struct uv_IO_APIC_route_entry {
	__u64                      vector:8;             /*     0: 0  8 */
	__u64                      delivery_mode:3;      /*     0: 8  8 */
	__u64                      dest_mode:1;          /*     0:11  8 */
	__u64                      delivery_status:1;    /*     0:12  8 */
	__u64                      polarity:1;           /*     0:13  8 */
	__u64                      __reserved_1:1;       /*     0:14  8 */
	__u64                      trigger:1;            /*     0:15  8 */
	__u64                      mask:1;               /*     0:16  8 */
	__u64                      __reserved_2:15;      /*     0:17  8 */
	__u64                      dest:32;              /*     0:32  8 */

	/* size: 8, cachelines: 1, members: 10 */
	/* last cacheline: 8 bytes */
};
struct uv_irq_2_mmr_pnode {
	long unsigned int          offset;               /*     0     8 */
	int                        pnode;                /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct uv_rtc_timer_head {
	spinlock_t                 lock;                 /*     0     4 */
	int                        next_cpu;             /*     4     4 */
	int                        ncpus;                /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct {
		int                lcpu;                 /*    16     4 */

		/* XXX 4 bytes hole, try to pack */

		u64                expires;              /*    24     8 */
	} cpu[]; /*    16     0 */

	/* size: 16, cachelines: 1, members: 4 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct uv_hub_nmi_s {
	raw_spinlock_t             nmi_lock;             /*     0     4 */
	atomic_t                   in_nmi;               /*     4     4 */
	atomic_t                   cpu_owner;            /*     8     4 */
	atomic_t                   read_mmr_count;       /*    12     4 */
	atomic_t                   nmi_count;            /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          nmi_value;            /*    24     8 */
	bool                       hub_present;          /*    32     1 */
	bool                       pch_owner;            /*    33     1 */

	/* size: 40, cachelines: 1, members: 8 */
	/* sum members: 30, holes: 1, sum holes: 4 */
	/* padding: 6 */
	/* last cacheline: 40 bytes */
};
struct uv_cpu_nmi_s {
	struct uv_hub_nmi_s *      hub;                  /*     0     8 */
	int                        state;                /*     8     4 */
	int                        pinging;              /*    12     4 */
	int                        queries;              /*    16     4 */
	int                        pings;                /*    20     4 */

	/* size: 24, cachelines: 1, members: 5 */
	/* last cacheline: 24 bytes */
};
struct nmi_action {
	char *                     action;               /*     0     8 */
	char *                     desc;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct init_nmi {
	unsigned int               offset;               /*     0     4 */
	unsigned int               mask;                 /*     4     4 */
	unsigned int               data;                 /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct bpf_tramp_progs {
	struct bpf_prog *          progs[40];            /*     0   320 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	int                        nr_progs;             /*   320     4 */

	/* size: 328, cachelines: 6, members: 2 */
	/* padding: 4 */
	/* last cacheline: 8 bytes */
};
struct bpf_array_aux {
	struct {
		spinlock_t         lock;                 /*     0     4 */
		enum bpf_prog_type type;                 /*     4     4 */
		bool               jited;                /*     8     1 */
	} owner;                                         /*     0    12 */

	/* XXX last struct has 3 bytes of padding */
	/* XXX 4 bytes hole, try to pack */

	struct list_head           poke_progs;           /*    16    16 */
	struct bpf_map *           map;                  /*    32     8 */
	struct mutex               poke_mutex;           /*    40    32 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	struct work_struct         work;                 /*    72    64 */

	/* size: 136, cachelines: 3, members: 5 */
	/* sum members: 132, holes: 1, sum holes: 4 */
	/* paddings: 1, sum paddings: 3 */
	/* last cacheline: 8 bytes */
};
struct bpf_array {
	struct bpf_map             map __attribute__((__aligned__(64))); /*     0   256 */

	/* XXX last struct has 8 bytes of padding */

	/* --- cacheline 4 boundary (256 bytes) --- */
	u32                        elem_size;            /*   256     4 */
	u32                        index_mask;           /*   260     4 */
	struct bpf_array_aux *     aux;                  /*   264     8 */
	union {
		char               value[0] __attribute__((__aligned__(8))); /*   272     0 */
		void *             ptrs[0] __attribute__((__aligned__(8))); /*   272     0 */
		void *             pptrs[0] __attribute__((__aligned__(8))); /*   272     0 */
	} __attribute__((__aligned__(8)));               /*   272     0 */

	/* size: 320, cachelines: 5, members: 5 */
	/* padding: 48 */
	/* paddings: 1, sum paddings: 8 */
	/* forced alignments: 2 */
} __attribute__((__aligned__(64)));
struct bpf_binary_header {
	u32                        pages;                /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	u8                         image[] __attribute__((__aligned__(8))); /*     8     0 */

	/* size: 8, cachelines: 1, members: 2 */
	/* sum members: 4, holes: 1, sum holes: 4 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 4 */
	/* last cacheline: 8 bytes */
} __attribute__((__aligned__(8)));
struct jit_context {
	int                        cleanup_addr;         /*     0     4 */
	int                        tail_call_direct_label; /*     4     4 */
	int                        tail_call_indirect_label; /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct x64_jit_data {
	struct bpf_binary_header * header;               /*     0     8 */
	int *                      addrs;                /*     8     8 */
	u8 *                       image;                /*    16     8 */
	int                        proglen;              /*    24     4 */
	struct jit_context         ctx;                  /*    28    12 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct clone_args {
	__u64                      flags __attribute__((__aligned__(8))); /*     0     8 */
	__u64                      pidfd __attribute__((__aligned__(8))); /*     8     8 */
	__u64                      child_tid __attribute__((__aligned__(8))); /*    16     8 */
	__u64                      parent_tid __attribute__((__aligned__(8))); /*    24     8 */
	__u64                      exit_signal __attribute__((__aligned__(8))); /*    32     8 */
	__u64                      stack __attribute__((__aligned__(8))); /*    40     8 */
	__u64                      stack_size __attribute__((__aligned__(8))); /*    48     8 */
	__u64                      tls __attribute__((__aligned__(8))); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	__u64                      set_tid __attribute__((__aligned__(8))); /*    64     8 */
	__u64                      set_tid_size __attribute__((__aligned__(8))); /*    72     8 */
	__u64                      cgroup __attribute__((__aligned__(8))); /*    80     8 */

	/* size: 88, cachelines: 2, members: 11 */
	/* forced alignments: 11 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct files_struct {
	atomic_t                   count;                /*     0     4 */
	bool                       resize_in_progress;   /*     4     1 */

	/* XXX 3 bytes hole, try to pack */

	wait_queue_head_t          resize_wait;          /*     8    24 */
	struct fdtable *           fdt;                  /*    32     8 */
	struct fdtable             fdtab __attribute__((__aligned__(8))); /*    40    56 */

	/* XXX 32 bytes hole, try to pack */

	/* --- cacheline 2 boundary (128 bytes) --- */
	spinlock_t                 file_lock __attribute__((__aligned__(64))); /*   128     4 */
	unsigned int               next_fd;              /*   132     4 */
	long unsigned int          close_on_exec_init[1]; /*   136     8 */
	long unsigned int          open_fds_init[1];     /*   144     8 */
	long unsigned int          full_fds_bits_init[1]; /*   152     8 */
	struct file *              fd_array[64];         /*   160   512 */
	/* --- cacheline 10 boundary (640 bytes) was 32 bytes ago --- */
	struct files_cgroup *      files_cgroup;         /*   672     8 */

	/* size: 704, cachelines: 11, members: 12 */
	/* sum members: 645, holes: 2, sum holes: 35 */
	/* padding: 24 */
	/* forced alignments: 2, forced holes: 1, sum forced holes: 32 */
} __attribute__((__aligned__(64)));
struct io_uring_task {
	struct xarray              xa;                   /*     0    16 */
	struct wait_queue_head     wait;                 /*    16    24 */
	struct file *              last;                 /*    40     8 */
	struct percpu_counter      inflight;             /*    48    40 */
	/* --- cacheline 1 boundary (64 bytes) was 24 bytes ago --- */
	struct io_identity         __identity;           /*    88    72 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 2 boundary (128 bytes) was 32 bytes ago --- */
	struct io_identity *       identity;             /*   160     8 */
	atomic_t                   in_idle;              /*   168     4 */
	bool                       sqpoll;               /*   172     1 */

	/* size: 176, cachelines: 3, members: 8 */
	/* padding: 3 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 48 bytes */
};
struct robust_list_head {
	struct robust_list         list;                 /*     0     8 */
	long int                   futex_offset;         /*     8     8 */
	struct robust_list *       list_op_pending;      /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct multiprocess_signals {
	sigset_t                   signal;               /*     0     8 */
	struct hlist_node          node;                 /*     8    16 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct fdtable {
	unsigned int               max_fds;              /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct file * *            fd;                   /*     8     8 */
	long unsigned int *        close_on_exec;        /*    16     8 */
	long unsigned int *        open_fds;             /*    24     8 */
	long unsigned int *        full_fds_bits;        /*    32     8 */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*    40    16 */

	/* size: 56, cachelines: 1, members: 6 */
	/* sum members: 52, holes: 1, sum holes: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 56 bytes */
} __attribute__((__aligned__(8)));
struct robust_list {
	struct robust_list *       next;                 /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct proc_fs_info {
	struct pid_namespace *     pid_ns;               /*     0     8 */
	struct dentry *            proc_self;            /*     8     8 */
	struct dentry *            proc_thread_self;     /*    16     8 */
	kgid_t                     pid_gid;              /*    24     4 */
	enum proc_hidepid          hide_pid;             /*    28     4 */
	enum proc_pidonly          pidonly;              /*    32     4 */

	/* size: 40, cachelines: 1, members: 6 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct io_identity {
	struct files_struct *      files;                /*     0     8 */
	struct mm_struct *         mm;                   /*     8     8 */
	struct cgroup_subsys_state * blkcg_css;          /*    16     8 */
	const struct cred  *       creds;                /*    24     8 */
	struct nsproxy *           nsproxy;              /*    32     8 */
	struct fs_struct *         fs;                   /*    40     8 */
	long unsigned int          fsize;                /*    48     8 */
	kuid_t                     loginuid;             /*    56     4 */
	unsigned int               sessionid;            /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	refcount_t                 count;                /*    64     4 */

	/* size: 72, cachelines: 2, members: 10 */
	/* padding: 4 */
	/* last cacheline: 8 bytes */
};
struct trace_event_raw_task_newtask {
	struct trace_entry         ent;                  /*     0     8 */
	pid_t                      pid;                  /*     8     4 */
	char                       comm[16];             /*    12    16 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          clone_flags;          /*    32     8 */
	short int                  oom_score_adj;        /*    40     2 */
	char                       __data[];             /*    42     0 */

	/* size: 48, cachelines: 1, members: 6 */
	/* sum members: 38, holes: 1, sum holes: 4 */
	/* padding: 6 */
	/* last cacheline: 48 bytes */
};
struct trace_event_raw_task_rename {
	struct trace_entry         ent;                  /*     0     8 */
	pid_t                      pid;                  /*     8     4 */
	char                       oldcomm[16];          /*    12    16 */
	char                       newcomm[16];          /*    28    16 */
	short int                  oom_score_adj;        /*    44     2 */
	char                       __data[];             /*    46     0 */

	/* size: 48, cachelines: 1, members: 6 */
	/* padding: 2 */
	/* last cacheline: 48 bytes */
};
struct trace_event_data_offsets_task_newtask {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_task_rename {

	/* size: 0, cachelines: 0, members: 0 */
};
struct taint_flag {
	char                       c_true;               /*     0     1 */
	char                       c_false;              /*     1     1 */
	bool                       module;               /*     2     1 */

	/* size: 3, cachelines: 1, members: 3 */
	/* last cacheline: 3 bytes */
};
struct warn_args {
	const char  *              fmt;                  /*     0     8 */
	va_list                    args;                 /*     8    24 */

	/* size: 32, cachelines: 1, members: 2 */
	/* last cacheline: 32 bytes */
};
struct smp_hotplug_thread {
	struct task_struct * *     store;                /*     0     8 */
	struct list_head           list;                 /*     8    16 */
	int                        (*thread_should_run)(unsigned int); /*    24     8 */
	void                       (*thread_fn)(unsigned int); /*    32     8 */
	void                       (*create)(unsigned int); /*    40     8 */
	void                       (*setup)(unsigned int); /*    48     8 */
	void                       (*cleanup)(unsigned int, bool); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	void                       (*park)(unsigned int); /*    64     8 */
	void                       (*unpark)(unsigned int); /*    72     8 */
	bool                       selfparking;          /*    80     1 */

	/* XXX 7 bytes hole, try to pack */

	const char  *              thread_comm;          /*    88     8 */

	/* size: 96, cachelines: 2, members: 11 */
	/* sum members: 89, holes: 1, sum holes: 7 */
	/* last cacheline: 32 bytes */
};
struct trace_event_raw_cpuhp_enter {
	struct trace_entry         ent;                  /*     0     8 */
	unsigned int               cpu;                  /*     8     4 */
	int                        target;               /*    12     4 */
	int                        idx;                  /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     fun;                  /*    24     8 */
	char                       __data[];             /*    32     0 */

	/* size: 32, cachelines: 1, members: 6 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct trace_event_raw_cpuhp_multi_enter {
	struct trace_entry         ent;                  /*     0     8 */
	unsigned int               cpu;                  /*     8     4 */
	int                        target;               /*    12     4 */
	int                        idx;                  /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     fun;                  /*    24     8 */
	char                       __data[];             /*    32     0 */

	/* size: 32, cachelines: 1, members: 6 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct trace_event_raw_cpuhp_exit {
	struct trace_entry         ent;                  /*     0     8 */
	unsigned int               cpu;                  /*     8     4 */
	int                        state;                /*    12     4 */
	int                        idx;                  /*    16     4 */
	int                        ret;                  /*    20     4 */
	char                       __data[];             /*    24     0 */

	/* size: 24, cachelines: 1, members: 6 */
	/* last cacheline: 24 bytes */
};
struct trace_event_data_offsets_cpuhp_enter {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_cpuhp_multi_enter {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_cpuhp_exit {

	/* size: 0, cachelines: 0, members: 0 */
};
struct cpuhp_cpu_state {
	enum cpuhp_state           state;                /*     0     4 */
	enum cpuhp_state           target;               /*     4     4 */
	enum cpuhp_state           fail;                 /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct task_struct *       thread;               /*    16     8 */
	bool                       should_run;           /*    24     1 */
	bool                       rollback;             /*    25     1 */
	bool                       single;               /*    26     1 */
	bool                       bringup;              /*    27     1 */

	/* XXX 4 bytes hole, try to pack */

	struct hlist_node *        node;                 /*    32     8 */
	struct hlist_node *        last;                 /*    40     8 */
	enum cpuhp_state           cb_state;             /*    48     4 */
	int                        result;               /*    52     4 */
	struct completion          done_up;              /*    56    32 */
	/* --- cacheline 1 boundary (64 bytes) was 24 bytes ago --- */
	struct completion          done_down;            /*    88    32 */

	/* size: 120, cachelines: 2, members: 14 */
	/* sum members: 112, holes: 2, sum holes: 8 */
	/* last cacheline: 56 bytes */
};
struct cpuhp_step {
	const char  *              name;                 /*     0     8 */
	union {
		int                (*single)(unsigned int); /*     8     8 */
		int                (*multi)(unsigned int, struct hlist_node *); /*     8     8 */
	} startup;                                       /*     8     8 */
	union {
		int                (*single)(unsigned int); /*    16     8 */
		int                (*multi)(unsigned int, struct hlist_node *); /*    16     8 */
	} teardown;                                      /*    16     8 */
	struct hlist_head          list;                 /*    24     8 */
	bool                       cant_stop;            /*    32     1 */
	bool                       multi_instance;       /*    33     1 */

	/* size: 40, cachelines: 1, members: 6 */
	/* padding: 6 */
	/* last cacheline: 40 bytes */
};
struct __kernel_old_timeval {
	__kernel_long_t            tv_sec;               /*     0     8 */
	__kernel_long_t            tv_usec;              /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct old_timeval32 {
	old_time32_t               tv_sec;               /*     0     4 */
	s32                        tv_usec;              /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct rusage {
	struct __kernel_old_timeval ru_utime;            /*     0    16 */
	struct __kernel_old_timeval ru_stime;            /*    16    16 */
	__kernel_long_t            ru_maxrss;            /*    32     8 */
	__kernel_long_t            ru_ixrss;             /*    40     8 */
	__kernel_long_t            ru_idrss;             /*    48     8 */
	__kernel_long_t            ru_isrss;             /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	__kernel_long_t            ru_minflt;            /*    64     8 */
	__kernel_long_t            ru_majflt;            /*    72     8 */
	__kernel_long_t            ru_nswap;             /*    80     8 */
	__kernel_long_t            ru_inblock;           /*    88     8 */
	__kernel_long_t            ru_oublock;           /*    96     8 */
	__kernel_long_t            ru_msgsnd;            /*   104     8 */
	__kernel_long_t            ru_msgrcv;            /*   112     8 */
	__kernel_long_t            ru_nsignals;          /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	__kernel_long_t            ru_nvcsw;             /*   128     8 */
	__kernel_long_t            ru_nivcsw;            /*   136     8 */

	/* size: 144, cachelines: 3, members: 16 */
	/* last cacheline: 16 bytes */
};
struct compat_rusage {
	struct old_timeval32       ru_utime;             /*     0     8 */
	struct old_timeval32       ru_stime;             /*     8     8 */
	compat_long_t              ru_maxrss;            /*    16     4 */
	compat_long_t              ru_ixrss;             /*    20     4 */
	compat_long_t              ru_idrss;             /*    24     4 */
	compat_long_t              ru_isrss;             /*    28     4 */
	compat_long_t              ru_minflt;            /*    32     4 */
	compat_long_t              ru_majflt;            /*    36     4 */
	compat_long_t              ru_nswap;             /*    40     4 */
	compat_long_t              ru_inblock;           /*    44     4 */
	compat_long_t              ru_oublock;           /*    48     4 */
	compat_long_t              ru_msgsnd;            /*    52     4 */
	compat_long_t              ru_msgrcv;            /*    56     4 */
	compat_long_t              ru_nsignals;          /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	compat_long_t              ru_nvcsw;             /*    64     4 */
	compat_long_t              ru_nivcsw;            /*    68     4 */

	/* size: 72, cachelines: 2, members: 16 */
	/* last cacheline: 8 bytes */
};
struct waitid_info {
	pid_t                      pid;                  /*     0     4 */
	uid_t                      uid;                  /*     4     4 */
	int                        status;               /*     8     4 */
	int                        cause;                /*    12     4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct wait_opts {
	enum pid_type              wo_type;              /*     0     4 */
	int                        wo_flags;             /*     4     4 */
	struct pid *               wo_pid;               /*     8     8 */
	struct waitid_info *       wo_info;              /*    16     8 */
	int                        wo_stat;              /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	struct rusage *            wo_rusage;            /*    32     8 */
	wait_queue_entry_t         child_wait;           /*    40    40 */
	/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
	int                        notask_error;         /*    80     4 */

	/* size: 88, cachelines: 2, members: 8 */
	/* sum members: 80, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct softirq_action {
	void                       (*action)(struct softirq_action *); /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct tasklet_struct {
	struct tasklet_struct *    next;                 /*     0     8 */
	long unsigned int          state;                /*     8     8 */
	atomic_t                   count;                /*    16     4 */
	bool                       use_callback;         /*    20     1 */

	/* XXX 3 bytes hole, try to pack */

	union {
		void               (*func)(long unsigned int); /*    24     8 */
		void               (*callback)(struct tasklet_struct *); /*    24     8 */
	};                                               /*    24     8 */
	long unsigned int          data;                 /*    32     8 */
	u64                        kabi_reserved1;       /*    40     8 */

	/* size: 48, cachelines: 1, members: 7 */
	/* sum members: 45, holes: 1, sum holes: 3 */
	/* last cacheline: 48 bytes */
};
struct kernel_stat {
	long unsigned int          irqs_sum;             /*     0     8 */
	unsigned int               softirqs[10];         /*     8    40 */

	/* size: 48, cachelines: 1, members: 2 */
	/* last cacheline: 48 bytes */
};
struct trace_event_raw_irq_handler_entry {
	struct trace_entry         ent;                  /*     0     8 */
	int                        irq;                  /*     8     4 */
	u32                        __data_loc_name;      /*    12     4 */
	char                       __data[];             /*    16     0 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct trace_event_raw_irq_handler_exit {
	struct trace_entry         ent;                  /*     0     8 */
	int                        irq;                  /*     8     4 */
	int                        ret;                  /*    12     4 */
	char                       __data[];             /*    16     0 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct trace_event_raw_softirq {
	struct trace_entry         ent;                  /*     0     8 */
	unsigned int               vec;                  /*     8     4 */
	char                       __data[];             /*    12     0 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct trace_event_data_offsets_irq_handler_entry {
	u32                        name;                 /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct trace_event_data_offsets_irq_handler_exit {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_softirq {

	/* size: 0, cachelines: 0, members: 0 */
};
struct tasklet_head {
	struct tasklet_struct *    head;                 /*     0     8 */
	struct tasklet_struct * *  tail;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct resource_entry {
	struct list_head           node;                 /*     0    16 */
	struct resource *          res;                  /*    16     8 */
	resource_size_t            offset;               /*    24     8 */
	struct resource            __res;                /*    32    96 */

	/* size: 128, cachelines: 2, members: 4 */
};
struct resource_constraint {
	resource_size_t            min;                  /*     0     8 */
	resource_size_t            max;                  /*     8     8 */
	resource_size_t            align;                /*    16     8 */
	resource_size_t            (*alignf)(void *, const struct resource  *, resource_size_t, resource_size_t); /*    24     8 */
	void *                     alignf_data;          /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct region_devres {
	struct resource *          parent;               /*     0     8 */
	resource_size_t            start;                /*     8     8 */
	resource_size_t            n;                    /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct dentry_stat_t {
	long int                   nr_dentry;            /*     0     8 */
	long int                   nr_unused;            /*     8     8 */
	long int                   age_limit;            /*    16     8 */
	long int                   want_pages;           /*    24     8 */
	long int                   nr_negative;          /*    32     8 */
	long int                   dummy;                /*    40     8 */

	/* size: 48, cachelines: 1, members: 6 */
	/* last cacheline: 48 bytes */
};
struct files_stat_struct {
	long unsigned int          nr_files;             /*     0     8 */
	long unsigned int          nr_free_files;        /*     8     8 */
	long unsigned int          max_files;            /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct inodes_stat_t {
	long int                   nr_inodes;            /*     0     8 */
	long int                   nr_unused;            /*     8     8 */
	long int                   dummy[5];             /*    16    40 */

	/* size: 56, cachelines: 1, members: 3 */
	/* last cacheline: 56 bytes */
};
struct do_proc_dointvec_minmax_conv_param {
	int *                      min;                  /*     0     8 */
	int *                      max;                  /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct do_proc_douintvec_minmax_conv_param {
	unsigned int *             min;                  /*     0     8 */
	unsigned int *             max;                  /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct __user_cap_header_struct {
	__u32                      version;              /*     0     4 */
	int                        pid;                  /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct __user_cap_data_struct {
	__u32                      effective;            /*     0     4 */
	__u32                      permitted;            /*     4     4 */
	__u32                      inheritable;          /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct sigqueue {
	struct list_head           list;                 /*     0    16 */
	int                        flags;                /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	kernel_siginfo_t           info;                 /*    24    48 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	struct user_struct *       user;                 /*    72     8 */

	/* size: 80, cachelines: 2, members: 4 */
	/* sum members: 76, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct ptrace_peeksiginfo_args {
	__u64                      off;                  /*     0     8 */
	__u32                      flags;                /*     8     4 */
	__s32                      nr;                   /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct ptrace_syscall_info {
	__u8                       op;                   /*     0     1 */
	__u8                       pad[3];               /*     1     3 */
	__u32                      arch;                 /*     4     4 */
	__u64                      instruction_pointer;  /*     8     8 */
	__u64                      stack_pointer;        /*    16     8 */
	union {
		struct {
			__u64      nr;                   /*    24     8 */
			__u64      args[6];              /*    32    48 */
		} entry;                                 /*    24    56 */
		struct {
			__s64      rval;                 /*    24     8 */
			__u8       is_error;             /*    32     1 */
		} exit;                                  /*    24    16 */
		struct {
			__u64      nr;                   /*    24     8 */
			__u64      args[6];              /*    32    48 */
			/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
			__u32      ret_data;             /*    80     4 */
		} seccomp;                               /*    24    64 */
	};                                               /*    24    64 */

	/* size: 88, cachelines: 2, members: 6 */
	/* last cacheline: 24 bytes */
};
struct ptrace_rseq_configuration {
	__u64                      rseq_abi_pointer;     /*     0     8 */
	__u32                      rseq_abi_size;        /*     8     4 */
	__u32                      signature;            /*    12     4 */
	__u32                      flags;                /*    16     4 */
	__u32                      pad;                  /*    20     4 */

	/* size: 24, cachelines: 1, members: 5 */
	/* last cacheline: 24 bytes */
};
struct compat_iovec {
	compat_uptr_t              iov_base;             /*     0     4 */
	compat_size_t              iov_len;              /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct fd {
	struct file *              file;                 /*     0     8 */
	unsigned int               flags;                /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct compat_sigaction {
	compat_uptr_t              sa_handler;           /*     0     4 */
	compat_ulong_t             sa_flags;             /*     4     4 */
	compat_uptr_t              sa_restorer;          /*     8     4 */
	compat_sigset_t            sa_mask;              /*    12     8 */

	/* size: 20, cachelines: 1, members: 4 */
	/* last cacheline: 20 bytes */
};
struct compat_old_sigaction {
	compat_uptr_t              sa_handler;           /*     0     4 */
	compat_old_sigset_t        sa_mask;              /*     4     4 */
	compat_ulong_t             sa_flags;             /*     8     4 */
	compat_uptr_t              sa_restorer;          /*    12     4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct trace_event_raw_signal_generate {
	struct trace_entry         ent;                  /*     0     8 */
	int                        sig;                  /*     8     4 */
	int                        errno;                /*    12     4 */
	int                        code;                 /*    16     4 */
	char                       comm[16];             /*    20    16 */
	pid_t                      pid;                  /*    36     4 */
	int                        group;                /*    40     4 */
	int                        result;               /*    44     4 */
	char                       __data[];             /*    48     0 */

	/* size: 48, cachelines: 1, members: 9 */
	/* last cacheline: 48 bytes */
};
struct trace_event_raw_signal_deliver {
	struct trace_entry         ent;                  /*     0     8 */
	int                        sig;                  /*     8     4 */
	int                        errno;                /*    12     4 */
	int                        code;                 /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          sa_handler;           /*    24     8 */
	long unsigned int          sa_flags;             /*    32     8 */
	char                       __data[];             /*    40     0 */

	/* size: 40, cachelines: 1, members: 7 */
	/* sum members: 36, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct trace_event_data_offsets_signal_generate {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_signal_deliver {

	/* size: 0, cachelines: 0, members: 0 */
};
struct sysinfo {
	__kernel_long_t            uptime;               /*     0     8 */
	__kernel_ulong_t           loads[3];             /*     8    24 */
	__kernel_ulong_t           totalram;             /*    32     8 */
	__kernel_ulong_t           freeram;              /*    40     8 */
	__kernel_ulong_t           sharedram;            /*    48     8 */
	__kernel_ulong_t           bufferram;            /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	__kernel_ulong_t           totalswap;            /*    64     8 */
	__kernel_ulong_t           freeswap;             /*    72     8 */
	__u16                      procs;                /*    80     2 */
	__u16                      pad;                  /*    82     2 */

	/* XXX 4 bytes hole, try to pack */

	__kernel_ulong_t           totalhigh;            /*    88     8 */
	__kernel_ulong_t           freehigh;             /*    96     8 */
	__u32                      mem_unit;             /*   104     4 */
	char                       _f[];                 /*   108     0 */

	/* size: 112, cachelines: 2, members: 14 */
	/* sum members: 104, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 48 bytes */
};
struct rlimit64 {
	__u64                      rlim_cur;             /*     0     8 */
	__u64                      rlim_max;             /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct oldold_utsname {
	char                       sysname[9];           /*     0     9 */
	char                       nodename[9];          /*     9     9 */
	char                       release[9];           /*    18     9 */
	char                       version[9];           /*    27     9 */
	char                       machine[9];           /*    36     9 */

	/* size: 45, cachelines: 1, members: 5 */
	/* last cacheline: 45 bytes */
};
struct old_utsname {
	char                       sysname[65];          /*     0    65 */
	/* --- cacheline 1 boundary (64 bytes) was 1 byte ago --- */
	char                       nodename[65];         /*    65    65 */
	/* --- cacheline 2 boundary (128 bytes) was 2 bytes ago --- */
	char                       release[65];          /*   130    65 */
	/* --- cacheline 3 boundary (192 bytes) was 3 bytes ago --- */
	char                       version[65];          /*   195    65 */
	/* --- cacheline 4 boundary (256 bytes) was 4 bytes ago --- */
	char                       machine[65];          /*   260    65 */

	/* size: 325, cachelines: 6, members: 5 */
	/* last cacheline: 5 bytes */
};
struct prctl_mm_map {
	__u64                      start_code;           /*     0     8 */
	__u64                      end_code;             /*     8     8 */
	__u64                      start_data;           /*    16     8 */
	__u64                      end_data;             /*    24     8 */
	__u64                      start_brk;            /*    32     8 */
	__u64                      brk;                  /*    40     8 */
	__u64                      start_stack;          /*    48     8 */
	__u64                      arg_start;            /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	__u64                      arg_end;              /*    64     8 */
	__u64                      env_start;            /*    72     8 */
	__u64                      env_end;              /*    80     8 */
	__u64 *                    auxv;                 /*    88     8 */
	__u32                      auxv_size;            /*    96     4 */
	__u32                      exe_fd;               /*   100     4 */

	/* size: 104, cachelines: 2, members: 14 */
	/* last cacheline: 40 bytes */
};
struct compat_tms {
	compat_clock_t             tms_utime;            /*     0     4 */
	compat_clock_t             tms_stime;            /*     4     4 */
	compat_clock_t             tms_cutime;           /*     8     4 */
	compat_clock_t             tms_cstime;           /*    12     4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct compat_rlimit {
	compat_ulong_t             rlim_cur;             /*     0     4 */
	compat_ulong_t             rlim_max;             /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct tms {
	__kernel_clock_t           tms_utime;            /*     0     8 */
	__kernel_clock_t           tms_stime;            /*     8     8 */
	__kernel_clock_t           tms_cutime;           /*    16     8 */
	__kernel_clock_t           tms_cstime;           /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct getcpu_cache {
	long unsigned int          blob[16];             /*     0   128 */

	/* size: 128, cachelines: 2, members: 1 */
};
struct compat_sysinfo {
	s32                        uptime;               /*     0     4 */
	u32                        loads[3];             /*     4    12 */
	u32                        totalram;             /*    16     4 */
	u32                        freeram;              /*    20     4 */
	u32                        sharedram;            /*    24     4 */
	u32                        bufferram;            /*    28     4 */
	u32                        totalswap;            /*    32     4 */
	u32                        freeswap;             /*    36     4 */
	u16                        procs;                /*    40     2 */
	u16                        pad;                  /*    42     2 */
	u32                        totalhigh;            /*    44     4 */
	u32                        freehigh;             /*    48     4 */
	u32                        mem_unit;             /*    52     4 */
	char                       _f[8];                /*    56     8 */

	/* size: 64, cachelines: 1, members: 14 */
};
struct workqueue_struct {
	struct list_head           pwqs;                 /*     0    16 */
	struct list_head           list;                 /*    16    16 */
	struct mutex               mutex;                /*    32    32 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        work_color;           /*    64     4 */
	int                        flush_color;          /*    68     4 */
	atomic_t                   nr_pwqs_to_flush;     /*    72     4 */

	/* XXX 4 bytes hole, try to pack */

	struct wq_flusher *        first_flusher;        /*    80     8 */
	struct list_head           flusher_queue;        /*    88    16 */
	struct list_head           flusher_overflow;     /*   104    16 */
	struct list_head           maydays;              /*   120    16 */
	/* --- cacheline 2 boundary (128 bytes) was 8 bytes ago --- */
	struct worker *            rescuer;              /*   136     8 */
	int                        nr_drainers;          /*   144     4 */
	int                        saved_max_active;     /*   148     4 */
	struct workqueue_attrs *   unbound_attrs;        /*   152     8 */
	struct pool_workqueue *    dfl_pwq;              /*   160     8 */
	struct wq_device *         wq_dev;               /*   168     8 */
	char                       name[24];             /*   176    24 */
	/* --- cacheline 3 boundary (192 bytes) was 8 bytes ago --- */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*   200    16 */

	/* XXX 40 bytes hole, try to pack */

	/* --- cacheline 4 boundary (256 bytes) --- */
	unsigned int               flags __attribute__((__aligned__(64))); /*   256     4 */

	/* XXX 4 bytes hole, try to pack */

	struct pool_workqueue *    cpu_pwqs;             /*   264     8 */
	struct pool_workqueue *    numa_pwq_tbl[];       /*   272     0 */

	/* size: 320, cachelines: 5, members: 21 */
	/* sum members: 224, holes: 3, sum holes: 48 */
	/* padding: 48 */
	/* forced alignments: 2, forced holes: 1, sum forced holes: 40 */
} __attribute__((__aligned__(64)));
struct workqueue_attrs {
	int                        nice;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	cpumask_var_t              cpumask;              /*     8     8 */
	bool                       no_numa;              /*    16     1 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 13, holes: 1, sum holes: 4 */
	/* padding: 7 */
	/* last cacheline: 24 bytes */
};
struct execute_work {
	struct work_struct         work;                 /*     0    64 */

	/* size: 64, cachelines: 1, members: 1 */
};
struct ida {
	struct xarray              xa;                   /*     0    16 */

	/* size: 16, cachelines: 1, members: 1 */
	/* last cacheline: 16 bytes */
};
struct __una_u32 {
	u32                        x;                    /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct worker {
	union {
		struct list_head   entry;                /*     0    16 */
		struct hlist_node  hentry;               /*     0    16 */
	};                                               /*     0    16 */
	struct work_struct *       current_work;         /*    16     8 */
	work_func_t                current_func;         /*    24     8 */
	struct pool_workqueue *    current_pwq;          /*    32     8 */
	struct list_head           scheduled;            /*    40    16 */
	struct task_struct *       task;                 /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct worker_pool *       pool;                 /*    64     8 */
	struct list_head           node;                 /*    72    16 */
	long unsigned int          last_active;          /*    88     8 */
	unsigned int               flags;                /*    96     4 */
	int                        id;                   /*   100     4 */
	int                        sleeping;             /*   104     4 */
	char                       desc[24];             /*   108    24 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 2 boundary (128 bytes) was 8 bytes ago --- */
	struct workqueue_struct *  rescue_wq;            /*   136     8 */
	work_func_t                last_func;            /*   144     8 */
	u64                        kabi_reserved1;       /*   152     8 */
	u64                        kabi_reserved2;       /*   160     8 */
	u64                        kabi_reserved3;       /*   168     8 */
	u64                        kabi_reserved4;       /*   176     8 */

	/* size: 184, cachelines: 3, members: 19 */
	/* sum members: 180, holes: 1, sum holes: 4 */
	/* last cacheline: 56 bytes */
};
struct pool_workqueue {
	struct worker_pool *       pool;                 /*     0     8 */
	struct workqueue_struct *  wq;                   /*     8     8 */
	int                        work_color;           /*    16     4 */
	int                        flush_color;          /*    20     4 */
	int                        refcnt;               /*    24     4 */
	int                        nr_in_flight[15];     /*    28    60 */
	/* --- cacheline 1 boundary (64 bytes) was 24 bytes ago --- */
	int                        nr_active;            /*    88     4 */
	int                        max_active;           /*    92     4 */
	struct list_head           delayed_works;        /*    96    16 */
	struct list_head           pwqs_node;            /*   112    16 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct list_head           mayday_node;          /*   128    16 */
	struct work_struct         unbound_release_work; /*   144    64 */
	/* --- cacheline 3 boundary (192 bytes) was 16 bytes ago --- */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*   208    16 */

	/* size: 256, cachelines: 4, members: 13 */
	/* padding: 32 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(256)));
struct worker_pool {
	raw_spinlock_t             lock;                 /*     0     4 */
	int                        cpu;                  /*     4     4 */
	int                        node;                 /*     8     4 */
	int                        id;                   /*    12     4 */
	unsigned int               flags;                /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          watchdog_ts;          /*    24     8 */
	struct list_head           worklist;             /*    32    16 */
	int                        nr_workers;           /*    48     4 */
	int                        nr_idle;              /*    52     4 */
	struct list_head           idle_list;            /*    56    16 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	struct timer_list          idle_timer;           /*    72    72 */
	/* --- cacheline 2 boundary (128 bytes) was 16 bytes ago --- */
	struct timer_list          mayday_timer;         /*   144    72 */
	/* --- cacheline 3 boundary (192 bytes) was 24 bytes ago --- */
	struct hlist_head          busy_hash[64];        /*   216   512 */
	/* --- cacheline 11 boundary (704 bytes) was 24 bytes ago --- */
	struct worker *            manager;              /*   728     8 */
	struct list_head           workers;              /*   736    16 */
	struct completion *        detach_completion;    /*   752     8 */
	struct ida                 worker_ida;           /*   760    16 */
	/* --- cacheline 12 boundary (768 bytes) was 8 bytes ago --- */
	struct workqueue_attrs *   attrs;                /*   776     8 */
	struct hlist_node          hash_node;            /*   784    16 */
	int                        refcnt;               /*   800     4 */

	/* XXX 28 bytes hole, try to pack */

	/* --- cacheline 13 boundary (832 bytes) --- */
	atomic_t                   nr_running __attribute__((__aligned__(64))); /*   832     4 */

	/* XXX 4 bytes hole, try to pack */

	struct callback_head       rcu __attribute__((__aligned__(8))); /*   840    16 */

	/* size: 896, cachelines: 14, members: 22 */
	/* sum members: 820, holes: 3, sum holes: 36 */
	/* padding: 40 */
	/* forced alignments: 2, forced holes: 2, sum forced holes: 32 */
} __attribute__((__aligned__(64)));
struct wq_flusher {
	struct list_head           list;                 /*     0    16 */
	int                        flush_color;          /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	struct completion          done;                 /*    24    32 */

	/* size: 56, cachelines: 1, members: 3 */
	/* sum members: 52, holes: 1, sum holes: 4 */
	/* last cacheline: 56 bytes */
};
struct wq_device {
	struct workqueue_struct *  wq;                   /*     0     8 */
	struct device              dev __attribute__((__aligned__(8))); /*     8  1000 */

	/* size: 1008, cachelines: 16, members: 2 */
	/* forced alignments: 1 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct trace_event_raw_workqueue_queue_work {
	struct trace_entry         ent;                  /*     0     8 */
	void *                     work;                 /*     8     8 */
	void *                     function;             /*    16     8 */
	void *                     workqueue;            /*    24     8 */
	unsigned int               req_cpu;              /*    32     4 */
	unsigned int               cpu;                  /*    36     4 */
	char                       __data[];             /*    40     0 */

	/* size: 40, cachelines: 1, members: 7 */
	/* last cacheline: 40 bytes */
};
struct trace_event_raw_workqueue_activate_work {
	struct trace_entry         ent;                  /*     0     8 */
	void *                     work;                 /*     8     8 */
	char                       __data[];             /*    16     0 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct trace_event_raw_workqueue_execute_start {
	struct trace_entry         ent;                  /*     0     8 */
	void *                     work;                 /*     8     8 */
	void *                     function;             /*    16     8 */
	char                       __data[];             /*    24     0 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_workqueue_execute_end {
	struct trace_entry         ent;                  /*     0     8 */
	void *                     work;                 /*     8     8 */
	void *                     function;             /*    16     8 */
	char                       __data[];             /*    24     0 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct trace_event_data_offsets_workqueue_queue_work {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_workqueue_activate_work {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_workqueue_execute_start {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_workqueue_execute_end {

	/* size: 0, cachelines: 0, members: 0 */
};
struct wq_barrier {
	struct work_struct         work;                 /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct completion          done;                 /*    64    32 */
	struct task_struct *       task;                 /*    96     8 */

	/* size: 104, cachelines: 2, members: 3 */
	/* last cacheline: 40 bytes */
};
struct cwt_wait {
	wait_queue_entry_t         wait;                 /*     0    40 */
	struct work_struct *       work;                 /*    40     8 */

	/* size: 48, cachelines: 1, members: 2 */
	/* last cacheline: 48 bytes */
};
struct apply_wqattrs_ctx {
	struct workqueue_struct *  wq;                   /*     0     8 */
	struct workqueue_attrs *   attrs;                /*     8     8 */
	struct list_head           list;                 /*    16    16 */
	struct pool_workqueue *    dfl_pwq;              /*    32     8 */
	struct pool_workqueue *    pwq_tbl[];            /*    40     0 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct work_for_cpu {
	struct work_struct         work;                 /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	long int                   (*fn)(void *);        /*    64     8 */
	void *                     arg;                  /*    72     8 */
	long int                   ret;                  /*    80     8 */

	/* size: 88, cachelines: 2, members: 4 */
	/* last cacheline: 24 bytes */
};
struct radix_tree_preload {
	local_lock_t               lock;                 /*     0     0 */
	unsigned int               nr;                   /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct xa_node *           nodes;                /*     8     8 */

	/* size: 16, cachelines: 1, members: 3 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct ctl_path {
	const char  *              procname;             /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct module_param_attrs {
	unsigned int               num;                  /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct attribute_group     grp;                  /*     8    40 */
	struct param_attribute     attrs[];              /*    48     0 */

	/* size: 48, cachelines: 1, members: 3 */
	/* sum members: 44, holes: 1, sum holes: 4 */
	/* last cacheline: 48 bytes */
};
struct module_version_attribute {
	struct module_attribute    mattr;                /*     0    56 */
	const char  *              module_name;          /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	const char  *              version;              /*    64     8 */

	/* size: 72, cachelines: 2, members: 3 */
	/* last cacheline: 8 bytes */
} __attribute__((__aligned__(8)));
struct kmalloced_param {
	struct list_head           list;                 /*     0    16 */
	char                       val[];                /*    16     0 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct param_attribute {
	struct module_attribute    mattr;                /*     0    56 */
	const struct kernel_param  * param;              /*    56     8 */

	/* size: 64, cachelines: 1, members: 2 */
};
struct sched_param {
	int                        sched_priority;       /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct kthread_work {
	struct list_head           node;                 /*     0    16 */
	kthread_work_func_t        func;                 /*    16     8 */
	struct kthread_worker *    worker;               /*    24     8 */
	int                        canceling;            /*    32     4 */

	/* size: 40, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct kthread_worker {
	unsigned int               flags;                /*     0     4 */
	raw_spinlock_t             lock;                 /*     4     4 */
	struct list_head           work_list;            /*     8    16 */
	struct list_head           delayed_work_list;    /*    24    16 */
	struct task_struct *       task;                 /*    40     8 */
	struct kthread_work *      current_work;         /*    48     8 */

	/* size: 56, cachelines: 1, members: 6 */
	/* last cacheline: 56 bytes */
};
struct kthread_delayed_work {
	struct kthread_work        work;                 /*     0    40 */

	/* XXX last struct has 4 bytes of padding */

	struct timer_list          timer;                /*    40    72 */

	/* size: 112, cachelines: 2, members: 2 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 48 bytes */
};
struct kthread_create_info {
	int                        (*threadfn)(void *);  /*     0     8 */
	void *                     data;                 /*     8     8 */
	int                        node;                 /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	struct task_struct *       result;               /*    24     8 */
	struct completion *        done;                 /*    32     8 */
	struct list_head           list;                 /*    40    16 */

	/* size: 56, cachelines: 1, members: 6 */
	/* sum members: 52, holes: 1, sum holes: 4 */
	/* last cacheline: 56 bytes */
};
struct kthread {
	long unsigned int          flags;                /*     0     8 */
	unsigned int               cpu;                  /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	int                        (*threadfn)(void *);  /*    16     8 */
	void *                     data;                 /*    24     8 */
	mm_segment_t               oldfs;                /*    32     0 */
	struct completion          parked;               /*    32    32 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct completion          exited;               /*    64    32 */
	struct cgroup_subsys_state * blkcg_css;          /*    96     8 */

	/* size: 104, cachelines: 2, members: 8 */
	/* sum members: 100, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct kthread_flush_work {
	struct kthread_work        work;                 /*     0    40 */

	/* XXX last struct has 4 bytes of padding */

	struct completion          done;                 /*    40    32 */

	/* size: 72, cachelines: 2, members: 2 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 8 bytes */
};
struct ipc_namespace {
	refcount_t                 count;                /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct ipc_ids             ids[3];               /*     8   768 */
	/* --- cacheline 12 boundary (768 bytes) was 8 bytes ago --- */
	int                        sem_ctls[4];          /*   776    16 */
	int                        used_sems;            /*   792     4 */
	unsigned int               msg_ctlmax;           /*   796     4 */
	unsigned int               msg_ctlmnb;           /*   800     4 */
	unsigned int               msg_ctlmni;           /*   804     4 */
	atomic_t                   msg_bytes;            /*   808     4 */
	atomic_t                   msg_hdrs;             /*   812     4 */
	size_t                     shm_ctlmax;           /*   816     8 */
	size_t                     shm_ctlall;           /*   824     8 */
	/* --- cacheline 13 boundary (832 bytes) --- */
	long unsigned int          shm_tot;              /*   832     8 */
	int                        shm_ctlmni;           /*   840     4 */
	int                        shm_rmid_forced;      /*   844     4 */
	struct notifier_block      ipcns_nb;             /*   848    24 */

	/* XXX last struct has 4 bytes of padding */

	struct vfsmount *          mq_mnt;               /*   872     8 */
	unsigned int               mq_queues_count;      /*   880     4 */
	unsigned int               mq_queues_max;        /*   884     4 */
	unsigned int               mq_msg_max;           /*   888     4 */
	unsigned int               mq_msgsize_max;       /*   892     4 */
	/* --- cacheline 14 boundary (896 bytes) --- */
	unsigned int               mq_msg_default;       /*   896     4 */
	unsigned int               mq_msgsize_default;   /*   900     4 */
	struct user_namespace *    user_ns;              /*   904     8 */
	struct ucounts *           ucounts;              /*   912     8 */
	struct llist_node          mnt_llist;            /*   920     8 */
	struct ns_common           ns;                   /*   928    32 */

	/* size: 960, cachelines: 15, members: 26 */
	/* sum members: 956, holes: 1, sum holes: 4 */
	/* paddings: 1, sum paddings: 4 */
};
struct ipc_ids {
	int                        in_use;               /*     0     4 */
	short unsigned int         seq;                  /*     4     2 */

	/* XXX 2 bytes hole, try to pack */

	struct rw_semaphore        rwsem;                /*     8    40 */
	struct idr                 ipcs_idr;             /*    48    24 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	int                        max_idx;              /*    72     4 */
	int                        last_idx;             /*    76     4 */
	int                        next_id;              /*    80     4 */

	/* XXX 4 bytes hole, try to pack */

	struct rhashtable          key_ht;               /*    88   168 */

	/* size: 256, cachelines: 4, members: 8 */
	/* sum members: 250, holes: 2, sum holes: 6 */
};
struct srcu_notifier_head {
	struct mutex               mutex;                /*     0    32 */
	struct srcu_struct         srcu;                 /*    32 50440 */
	/* --- cacheline 788 boundary (50432 bytes) was 40 bytes ago --- */
	struct notifier_block *    head;                 /* 50472     8 */

	/* size: 50480, cachelines: 789, members: 3 */
	/* last cacheline: 48 bytes */
};
struct async_domain {
	struct list_head           pending;              /*     0    16 */
	unsigned int               registered:1;         /*    16: 0  4 */

	/* size: 24, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* bit_padding: 31 bits */
	/* last cacheline: 24 bytes */
};
struct async_entry {
	struct list_head           domain_list;          /*     0    16 */
	struct list_head           global_list;          /*    16    16 */
	struct work_struct         work;                 /*    32    64 */
	/* --- cacheline 1 boundary (64 bytes) was 32 bytes ago --- */
	async_cookie_t             cookie;               /*    96     8 */
	async_func_t               func;                 /*   104     8 */
	void *                     data;                 /*   112     8 */
	struct async_domain *      domain;               /*   120     8 */

	/* size: 128, cachelines: 2, members: 7 */
};
struct smpboot_thread_data {
	unsigned int               cpu;                  /*     0     4 */
	unsigned int               status;               /*     4     4 */
	struct smp_hotplug_thread * ht;                  /*     8     8 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct pin_cookie {

	/* size: 0, cachelines: 0, members: 0 */
};
struct preempt_ops {
	void                       (*sched_in)(struct preempt_notifier *, int); /*     0     8 */
	void                       (*sched_out)(struct preempt_notifier *, struct task_struct *); /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct preempt_notifier {
	struct hlist_node          link;                 /*     0    16 */
	struct preempt_ops *       ops;                  /*    16     8 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct root_domain {
	atomic_t                   refcount;             /*     0     4 */
	atomic_t                   rto_count;            /*     4     4 */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*     8    16 */
	cpumask_var_t              span;                 /*    24     8 */
	cpumask_var_t              online;               /*    32     8 */
	int                        overload;             /*    40     4 */
	int                        overutilized;         /*    44     4 */
	cpumask_var_t              dlo_mask;             /*    48     8 */
	atomic_t                   dlo_count;            /*    56     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	struct dl_bw               dl_bw;                /*    64    24 */
	struct cpudl               cpudl;                /*    88    24 */
	struct irq_work            rto_push_work;        /*   112    24 */
	/* --- cacheline 2 boundary (128 bytes) was 8 bytes ago --- */
	raw_spinlock_t             rto_lock;             /*   136     4 */
	int                        rto_loop;             /*   140     4 */
	int                        rto_cpu;              /*   144     4 */
	atomic_t                   rto_loop_next;        /*   148     4 */
	atomic_t                   rto_loop_start;       /*   152     4 */

	/* XXX 4 bytes hole, try to pack */

	cpumask_var_t              rto_mask;             /*   160     8 */
	struct cpupri              cpupri;               /*   168  1640 */
	/* --- cacheline 28 boundary (1792 bytes) was 16 bytes ago --- */
	long unsigned int          max_cpu_capacity;     /*  1808     8 */
	struct perf_domain *       pd;                   /*  1816     8 */
	u64                        kabi_reserved1;       /*  1824     8 */
	u64                        kabi_reserved2;       /*  1832     8 */
	u64                        kabi_reserved3;       /*  1840     8 */
	u64                        kabi_reserved4;       /*  1848     8 */

	/* size: 1856, cachelines: 29, members: 25 */
	/* sum members: 1848, holes: 2, sum holes: 8 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(8)));
struct cfs_rq {
	struct load_weight         load;                 /*     0    16 */

	/* XXX last struct has 4 bytes of padding */

	unsigned int               nr_running;           /*    16     4 */
	unsigned int               h_nr_running;         /*    20     4 */
	unsigned int               idle_h_nr_running;    /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        exec_clock;           /*    32     8 */
	u64                        min_vruntime;         /*    40     8 */
	struct rb_root_cached      tasks_timeline;       /*    48    16 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct sched_entity *      curr;                 /*    64     8 */
	struct sched_entity *      next;                 /*    72     8 */
	struct sched_entity *      last;                 /*    80     8 */
	struct sched_entity *      skip;                 /*    88     8 */
	unsigned int               nr_spread_over;       /*    96     4 */

	/* XXX 28 bytes hole, try to pack */

	/* --- cacheline 2 boundary (128 bytes) --- */
	struct sched_avg           avg __attribute__((__aligned__(64))); /*   128    64 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	struct {
		raw_spinlock_t     lock __attribute__((__aligned__(64))); /*   192     4 */
		int                nr;                   /*   196     4 */
		long unsigned int  load_avg;             /*   200     8 */
		long unsigned int  util_avg;             /*   208     8 */
		long unsigned int  runnable_avg;         /*   216     8 */
	} __attribute__((__aligned__(64))) removed __attribute__((__aligned__(64)));      /*   192    64 */

	/* XXX last struct has 32 bytes of padding */

	/* --- cacheline 4 boundary (256 bytes) --- */
	long unsigned int          tg_load_avg_contrib;  /*   256     8 */
	long int                   propagate;            /*   264     8 */
	long int                   prop_runnable_sum;    /*   272     8 */
	long unsigned int          h_load;               /*   280     8 */
	u64                        last_h_load_update;   /*   288     8 */
	struct sched_entity *      h_load_next;          /*   296     8 */
	struct rq *                rq;                   /*   304     8 */
	int                        on_list;              /*   312     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 5 boundary (320 bytes) --- */
	struct list_head           leaf_cfs_rq_list;     /*   320    16 */
	struct task_group *        tg;                   /*   336     8 */
	int                        runtime_enabled;      /*   344     4 */

	/* XXX 4 bytes hole, try to pack */

	s64                        runtime_remaining;    /*   352     8 */
	u64                        throttled_clock;      /*   360     8 */
	union {
		u64                throttled_clock_pelt; /*   368     8 */
		struct {
			u64        throttled_clock_task; /*   368     8 */
		} kabi_hidden_635;                       /*   368     8 */
		union {
		};                                       /*   368     0 */
	};                                               /*   368     8 */
	union {
		u64                throttled_clock_pelt_time; /*   376     8 */
		struct {
			u64        throttled_clock_task_time; /*   376     8 */
		} kabi_hidden_636;                       /*   376     8 */
		union {
		};                                       /*   376     0 */
	};                                               /*   376     8 */
	/* --- cacheline 6 boundary (384 bytes) --- */
	int                        throttled;            /*   384     4 */
	int                        throttle_count;       /*   388     4 */
	struct list_head           throttled_list;       /*   392    16 */
	struct list_head           qos_throttled_list;   /*   408    16 */
	u64                        kabi_reserved3;       /*   424     8 */
	u64                        kabi_reserved4;       /*   432     8 */

	/* size: 448, cachelines: 7, members: 35 */
	/* sum members: 400, holes: 4, sum holes: 40 */
	/* padding: 8 */
	/* paddings: 2, sum paddings: 36 */
	/* forced alignments: 2, forced holes: 1, sum forced holes: 28 */
} __attribute__((__aligned__(64)));
struct rt_rq {
	struct rt_prio_array       active;               /*     0  1616 */
	/* --- cacheline 25 boundary (1600 bytes) was 16 bytes ago --- */
	unsigned int               rt_nr_running;        /*  1616     4 */
	unsigned int               rr_nr_running;        /*  1620     4 */
	struct {
		int                curr;                 /*  1624     4 */
		int                next;                 /*  1628     4 */
	} highest_prio;                                  /*  1624     8 */
	long unsigned int          rt_nr_migratory;      /*  1632     8 */
	long unsigned int          rt_nr_total;          /*  1640     8 */
	int                        overloaded;           /*  1648     4 */

	/* XXX 4 bytes hole, try to pack */

	struct plist_head          pushable_tasks;       /*  1656    16 */
	/* --- cacheline 26 boundary (1664 bytes) was 8 bytes ago --- */
	int                        rt_queued;            /*  1672     4 */
	int                        rt_throttled;         /*  1676     4 */
	u64                        rt_time;              /*  1680     8 */
	u64                        rt_runtime;           /*  1688     8 */
	raw_spinlock_t             rt_runtime_lock;      /*  1696     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          rt_nr_boosted;        /*  1704     8 */
	struct rq *                rq;                   /*  1712     8 */
	struct task_group *        tg;                   /*  1720     8 */

	/* size: 1728, cachelines: 27, members: 16 */
	/* sum members: 1720, holes: 2, sum holes: 8 */
};
struct sched_class {
	void                       (*enqueue_task)(struct rq *, struct task_struct *, int); /*     0     8 */
	void                       (*dequeue_task)(struct rq *, struct task_struct *, int); /*     8     8 */
	void                       (*yield_task)(struct rq *); /*    16     8 */
	bool                       (*yield_to_task)(struct rq *, struct task_struct *); /*    24     8 */
	void                       (*check_preempt_curr)(struct rq *, struct task_struct *, int); /*    32     8 */
	struct task_struct *       (*pick_next_task)(struct rq *); /*    40     8 */
	void                       (*put_prev_task)(struct rq *, struct task_struct *); /*    48     8 */
	void                       (*set_next_task)(struct rq *, struct task_struct *, bool); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*balance)(struct rq *, struct task_struct *, struct rq_flags *); /*    64     8 */
	int                        (*select_task_rq)(struct task_struct *, int, int, int); /*    72     8 */
	void                       (*migrate_task_rq)(struct task_struct *, int); /*    80     8 */
	void                       (*task_woken)(struct rq *, struct task_struct *); /*    88     8 */
	void                       (*set_cpus_allowed)(struct task_struct *, const struct cpumask  *); /*    96     8 */
	void                       (*rq_online)(struct rq *); /*   104     8 */
	void                       (*rq_offline)(struct rq *); /*   112     8 */
	void                       (*task_tick)(struct rq *, struct task_struct *, int); /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	void                       (*task_fork)(struct task_struct *); /*   128     8 */
	void                       (*task_dead)(struct task_struct *); /*   136     8 */
	void                       (*switched_from)(struct rq *, struct task_struct *); /*   144     8 */
	void                       (*switched_to)(struct rq *, struct task_struct *); /*   152     8 */
	void                       (*prio_changed)(struct rq *, struct task_struct *, int); /*   160     8 */
	unsigned int               (*get_rr_interval)(struct rq *, struct task_struct *); /*   168     8 */
	void                       (*update_curr)(struct rq *); /*   176     8 */
	void                       (*task_change_group)(struct task_struct *, int); /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	struct task_struct *       (*pick_task)(struct rq *); /*   192     8 */
	u64                        kabi_reserved2;       /*   200     8 */

	/* size: 224, cachelines: 4, members: 26 */
	/* padding: 16 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(32)));
struct task_group {
	struct cgroup_subsys_state css __attribute__((__aligned__(8))); /*     0   296 */
	/* --- cacheline 4 boundary (256 bytes) was 40 bytes ago --- */
	struct sched_entity * *    se;                   /*   296     8 */
	struct cfs_rq * *          cfs_rq;               /*   304     8 */
	long unsigned int          shares;               /*   312     8 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	atomic_long_t              load_avg __attribute__((__aligned__(64))); /*   320     8 */
	struct sched_rt_entity * * rt_se;                /*   328     8 */
	struct rt_rq * *           rt_rq;                /*   336     8 */
	struct rt_bandwidth        rt_bandwidth __attribute__((__aligned__(8))); /*   344   128 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 7 boundary (448 bytes) was 24 bytes ago --- */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*   472    16 */
	struct list_head           list;                 /*   488    16 */
	struct task_group *        parent;               /*   504     8 */
	/* --- cacheline 8 boundary (512 bytes) --- */
	struct list_head           siblings;             /*   512    16 */
	struct list_head           children;             /*   528    16 */
	struct autogroup *         autogroup;            /*   544     8 */
	struct cfs_bandwidth       cfs_bandwidth __attribute__((__aligned__(8))); /*   552   320 */
	/* --- cacheline 13 boundary (832 bytes) was 40 bytes ago --- */
	long int                   qos_level;            /*   872     8 */
	union {
		long int           tag;                  /*   880     8 */
		struct {
			u64        kabi_reserved1;       /*   880     8 */
		} kabi_hidden_459;                       /*   880     8 */
		union {
		};                                       /*   880     0 */
	};                                               /*   880     8 */
	u64                        kabi_reserved2;       /*   888     8 */
	/* --- cacheline 14 boundary (896 bytes) --- */
	u64                        kabi_reserved3;       /*   896     8 */
	u64                        kabi_reserved4;       /*   904     8 */

	/* size: 960, cachelines: 15, members: 20 */
	/* padding: 48 */
	/* paddings: 1, sum paddings: 4 */
	/* forced alignments: 5 */
} __attribute__((__aligned__(64)));
struct sched_group {
	struct sched_group *       next;                 /*     0     8 */
	atomic_t                   ref;                  /*     8     4 */
	unsigned int               group_weight;         /*    12     4 */
	struct sched_group_capacity * sgc;               /*    16     8 */
	int                        asym_prefer_cpu;      /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        kabi_reserved1;       /*    32     8 */
	u64                        kabi_reserved2;       /*    40     8 */
	long unsigned int          cpumask[];            /*    48     0 */

	/* size: 48, cachelines: 1, members: 8 */
	/* sum members: 44, holes: 1, sum holes: 4 */
	/* last cacheline: 48 bytes */
};
struct sched_group_capacity {
	atomic_t                   ref;                  /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          capacity;             /*     8     8 */
	long unsigned int          min_capacity;         /*    16     8 */
	long unsigned int          max_capacity;         /*    24     8 */
	long unsigned int          next_update;          /*    32     8 */
	int                        imbalance;            /*    40     4 */
	int                        id;                   /*    44     4 */
	long unsigned int          cpumask[];            /*    48     0 */

	/* size: 48, cachelines: 1, members: 8 */
	/* sum members: 44, holes: 1, sum holes: 4 */
	/* last cacheline: 48 bytes */
};
struct em_perf_state {
	long unsigned int          frequency;            /*     0     8 */
	long unsigned int          power;                /*     8     8 */
	long unsigned int          cost;                 /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct em_perf_domain {
	struct em_perf_state *     table;                /*     0     8 */
	int                        nr_perf_states;       /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          cpus[];               /*    16     0 */

	/* size: 16, cachelines: 1, members: 3 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct autogroup {
	struct kref                kref;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct task_group *        tg;                   /*     8     8 */
	struct rw_semaphore        lock;                 /*    16    40 */
	long unsigned int          id;                   /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        nice;                 /*    64     4 */

	/* size: 72, cachelines: 2, members: 5 */
	/* sum members: 64, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 8 bytes */
};
struct context_tracking {
	bool                       active;               /*     0     1 */

	/* XXX 3 bytes hole, try to pack */

	int                        recursion;            /*     4     4 */
	enum ctx_state             state;                /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* sum members: 9, holes: 1, sum holes: 3 */
	/* last cacheline: 12 bytes */
};
struct kernel_cpustat {
	u64                        cpustat[10];          /*     0    80 */

	/* size: 80, cachelines: 2, members: 1 */
	/* last cacheline: 16 bytes */
};
struct css_task_iter {
	struct cgroup_subsys *     ss;                   /*     0     8 */
	unsigned int               flags;                /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head *         cset_pos;             /*    16     8 */
	struct list_head *         cset_head;            /*    24     8 */
	struct list_head *         tcset_pos;            /*    32     8 */
	struct list_head *         tcset_head;           /*    40     8 */
	struct list_head *         task_pos;             /*    48     8 */
	struct list_head *         cur_tasks_head;       /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct css_set *           cur_cset;             /*    64     8 */
	struct css_set *           cur_dcset;            /*    72     8 */
	struct task_struct *       cur_task;             /*    80     8 */
	struct list_head           iters_node;           /*    88    16 */

	/* size: 104, cachelines: 2, members: 12 */
	/* sum members: 100, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct trace_event_raw_sched_kthread_stop {
	struct trace_entry         ent;                  /*     0     8 */
	char                       comm[16];             /*     8    16 */
	pid_t                      pid;                  /*    24     4 */
	char                       __data[];             /*    28     0 */

	/* size: 28, cachelines: 1, members: 4 */
	/* last cacheline: 28 bytes */
};
struct trace_event_raw_sched_kthread_stop_ret {
	struct trace_entry         ent;                  /*     0     8 */
	int                        ret;                  /*     8     4 */
	char                       __data[];             /*    12     0 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct trace_event_raw_sched_wakeup_template {
	struct trace_entry         ent;                  /*     0     8 */
	char                       comm[16];             /*     8    16 */
	pid_t                      pid;                  /*    24     4 */
	int                        prio;                 /*    28     4 */
	int                        success;              /*    32     4 */
	int                        target_cpu;           /*    36     4 */
	char                       __data[];             /*    40     0 */

	/* size: 40, cachelines: 1, members: 7 */
	/* last cacheline: 40 bytes */
};
struct trace_event_raw_sched_switch {
	struct trace_entry         ent;                  /*     0     8 */
	char                       prev_comm[16];        /*     8    16 */
	pid_t                      prev_pid;             /*    24     4 */
	int                        prev_prio;            /*    28     4 */
	long int                   prev_state;           /*    32     8 */
	char                       next_comm[16];        /*    40    16 */
	pid_t                      next_pid;             /*    56     4 */
	int                        next_prio;            /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	char                       __data[];             /*    64     0 */

	/* size: 64, cachelines: 1, members: 9 */
};
struct trace_event_raw_sched_qos_smt_expel {
	struct trace_entry         ent;                  /*     0     8 */
	char                       sibling_comm[16];     /*     8    16 */
	pid_t                      sibling_pid;          /*    24     4 */
	int                        sibling_qos_status;   /*    28     4 */
	int                        sibling_cpu;          /*    32     4 */
	char                       __data[];             /*    36     0 */

	/* size: 36, cachelines: 1, members: 6 */
	/* last cacheline: 36 bytes */
};
struct trace_event_raw_sched_qos_smt_expelled {
	struct trace_entry         ent;                  /*     0     8 */
	char                       comm[16];             /*     8    16 */
	pid_t                      pid;                  /*    24     4 */
	int                        qos_status;           /*    28     4 */
	char                       __data[];             /*    32     0 */

	/* size: 32, cachelines: 1, members: 5 */
	/* last cacheline: 32 bytes */
};
struct trace_event_raw_sched_migrate_task {
	struct trace_entry         ent;                  /*     0     8 */
	char                       comm[16];             /*     8    16 */
	pid_t                      pid;                  /*    24     4 */
	int                        prio;                 /*    28     4 */
	int                        orig_cpu;             /*    32     4 */
	int                        dest_cpu;             /*    36     4 */
	char                       __data[];             /*    40     0 */

	/* size: 40, cachelines: 1, members: 7 */
	/* last cacheline: 40 bytes */
};
struct trace_event_raw_sched_process_template {
	struct trace_entry         ent;                  /*     0     8 */
	char                       comm[16];             /*     8    16 */
	pid_t                      pid;                  /*    24     4 */
	int                        prio;                 /*    28     4 */
	char                       __data[];             /*    32     0 */

	/* size: 32, cachelines: 1, members: 5 */
	/* last cacheline: 32 bytes */
};
struct trace_event_raw_sched_process_wait {
	struct trace_entry         ent;                  /*     0     8 */
	char                       comm[16];             /*     8    16 */
	pid_t                      pid;                  /*    24     4 */
	int                        prio;                 /*    28     4 */
	char                       __data[];             /*    32     0 */

	/* size: 32, cachelines: 1, members: 5 */
	/* last cacheline: 32 bytes */
};
struct trace_event_raw_sched_process_fork {
	struct trace_entry         ent;                  /*     0     8 */
	char                       parent_comm[16];      /*     8    16 */
	pid_t                      parent_pid;           /*    24     4 */
	char                       child_comm[16];       /*    28    16 */
	pid_t                      child_pid;            /*    44     4 */
	char                       __data[];             /*    48     0 */

	/* size: 48, cachelines: 1, members: 6 */
	/* last cacheline: 48 bytes */
};
struct trace_event_raw_sched_process_exec {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        __data_loc_filename;  /*     8     4 */
	pid_t                      pid;                  /*    12     4 */
	pid_t                      old_pid;              /*    16     4 */
	char                       __data[];             /*    20     0 */

	/* size: 20, cachelines: 1, members: 5 */
	/* last cacheline: 20 bytes */
};
struct trace_event_raw_sched_stat_template {
	struct trace_entry         ent;                  /*     0     8 */
	char                       comm[16];             /*     8    16 */
	pid_t                      pid;                  /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        delay;                /*    32     8 */
	char                       __data[];             /*    40     0 */

	/* size: 40, cachelines: 1, members: 5 */
	/* sum members: 36, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct trace_event_raw_sched_stat_runtime {
	struct trace_entry         ent;                  /*     0     8 */
	char                       comm[16];             /*     8    16 */
	pid_t                      pid;                  /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        runtime;              /*    32     8 */
	u64                        vruntime;             /*    40     8 */
	char                       __data[];             /*    48     0 */

	/* size: 48, cachelines: 1, members: 6 */
	/* sum members: 44, holes: 1, sum holes: 4 */
	/* last cacheline: 48 bytes */
};
struct trace_event_raw_sched_pi_setprio {
	struct trace_entry         ent;                  /*     0     8 */
	char                       comm[16];             /*     8    16 */
	pid_t                      pid;                  /*    24     4 */
	int                        oldprio;              /*    28     4 */
	int                        newprio;              /*    32     4 */
	char                       __data[];             /*    36     0 */

	/* size: 36, cachelines: 1, members: 6 */
	/* last cacheline: 36 bytes */
};
struct trace_event_raw_sched_process_hang {
	struct trace_entry         ent;                  /*     0     8 */
	char                       comm[16];             /*     8    16 */
	pid_t                      pid;                  /*    24     4 */
	char                       __data[];             /*    28     0 */

	/* size: 28, cachelines: 1, members: 4 */
	/* last cacheline: 28 bytes */
};
struct trace_event_raw_sched_move_numa {
	struct trace_entry         ent;                  /*     0     8 */
	pid_t                      pid;                  /*     8     4 */
	pid_t                      tgid;                 /*    12     4 */
	pid_t                      ngid;                 /*    16     4 */
	int                        src_cpu;              /*    20     4 */
	int                        src_nid;              /*    24     4 */
	int                        dst_cpu;              /*    28     4 */
	int                        dst_nid;              /*    32     4 */
	char                       __data[];             /*    36     0 */

	/* size: 36, cachelines: 1, members: 9 */
	/* last cacheline: 36 bytes */
};
struct trace_event_raw_sched_numa_pair_template {
	struct trace_entry         ent;                  /*     0     8 */
	pid_t                      src_pid;              /*     8     4 */
	pid_t                      src_tgid;             /*    12     4 */
	pid_t                      src_ngid;             /*    16     4 */
	int                        src_cpu;              /*    20     4 */
	int                        src_nid;              /*    24     4 */
	pid_t                      dst_pid;              /*    28     4 */
	pid_t                      dst_tgid;             /*    32     4 */
	pid_t                      dst_ngid;             /*    36     4 */
	int                        dst_cpu;              /*    40     4 */
	int                        dst_nid;              /*    44     4 */
	char                       __data[];             /*    48     0 */

	/* size: 48, cachelines: 1, members: 12 */
	/* last cacheline: 48 bytes */
};
struct trace_event_raw_sched_wake_idle_without_ipi {
	struct trace_entry         ent;                  /*     0     8 */
	int                        cpu;                  /*     8     4 */
	char                       __data[];             /*    12     0 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct trace_event_raw_psi_memstall_template {
	struct trace_entry         ent;                  /*     0     8 */
	long unsigned int          function;             /*     8     8 */
	char                       __data[];             /*    16     0 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct trace_event_data_offsets_sched_kthread_stop {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_sched_kthread_stop_ret {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_sched_wakeup_template {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_sched_switch {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_sched_qos_smt_expel {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_sched_qos_smt_expelled {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_sched_migrate_task {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_sched_process_template {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_sched_process_wait {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_sched_process_fork {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_sched_process_exec {
	u32                        filename;             /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct trace_event_data_offsets_sched_stat_template {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_sched_stat_runtime {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_sched_pi_setprio {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_sched_process_hang {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_sched_move_numa {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_sched_numa_pair_template {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_sched_wake_idle_without_ipi {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_psi_memstall_template {

	/* size: 0, cachelines: 0, members: 0 */
};
struct rq {
	raw_spinlock_t             __lock;               /*     0     4 */
	unsigned int               nr_running;           /*     4     4 */
	unsigned int               nr_numa_running;      /*     8     4 */
	unsigned int               nr_preferred_running; /*    12     4 */
	unsigned int               numa_migrate_on;      /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          last_blocked_load_update_tick; /*    24     8 */
	unsigned int               has_blocked_load;     /*    32     4 */

	/* XXX 28 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	call_single_data_t         nohz_csd __attribute__((__aligned__(32))); /*    64    32 */
	unsigned int               nohz_tick_stopped;    /*    96     4 */
	atomic_t                   nohz_flags;           /*   100     4 */
	unsigned int               ttwu_pending;         /*   104     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        nr_switches;          /*   112     8 */

	/* XXX 8 bytes hole, try to pack */

	/* --- cacheline 2 boundary (128 bytes) --- */
	struct cfs_rq              cfs __attribute__((__aligned__(64))); /*   128   448 */

	/* XXX last struct has 8 bytes of padding */

	/* --- cacheline 9 boundary (576 bytes) --- */
	struct rt_rq               rt;                   /*   576  1728 */
	/* --- cacheline 36 boundary (2304 bytes) --- */
	struct dl_rq               dl;                   /*  2304   104 */
	/* --- cacheline 37 boundary (2368 bytes) was 40 bytes ago --- */
	struct sparsemask *        cfs_overload_cpus;    /*  2408     8 */
	struct list_head           leaf_cfs_rq_list;     /*  2416    16 */
	/* --- cacheline 38 boundary (2432 bytes) --- */
	struct list_head *         tmp_alone_branch;     /*  2432     8 */
	long unsigned int          nr_uninterruptible;   /*  2440     8 */
	struct task_struct *       curr;                 /*  2448     8 */
	struct task_struct *       idle;                 /*  2456     8 */
	struct task_struct *       stop;                 /*  2464     8 */
	long unsigned int          next_balance;         /*  2472     8 */
	struct mm_struct *         prev_mm;              /*  2480     8 */
	unsigned int               clock_update_flags;   /*  2488     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 39 boundary (2496 bytes) --- */
	u64                        clock;                /*  2496     8 */

	/* XXX 56 bytes hole, try to pack */

	/* --- cacheline 40 boundary (2560 bytes) --- */
	u64                        clock_task __attribute__((__aligned__(64))); /*  2560     8 */
	u64                        clock_pelt;           /*  2568     8 */
	long unsigned int          lost_idle_time;       /*  2576     8 */
	atomic_t                   nr_iowait;            /*  2584     4 */
	int                        membarrier_state;     /*  2588     4 */
	struct root_domain *       rd;                   /*  2592     8 */
	struct sched_domain *      sd;                   /*  2600     8 */
	long unsigned int          cpu_capacity;         /*  2608     8 */
	long unsigned int          cpu_capacity_orig;    /*  2616     8 */
	/* --- cacheline 41 boundary (2624 bytes) --- */
	struct callback_head *     balance_callback;     /*  2624     8 */
	unsigned char              nohz_idle_balance;    /*  2632     1 */
	unsigned char              idle_balance;         /*  2633     1 */

	/* XXX 6 bytes hole, try to pack */

	long unsigned int          misfit_task_load;     /*  2640     8 */
	int                        active_balance;       /*  2648     4 */
	int                        push_cpu;             /*  2652     4 */
	struct cpu_stop_work       active_balance_work;  /*  2656    48 */
	/* --- cacheline 42 boundary (2688 bytes) was 16 bytes ago --- */
	int                        cpu;                  /*  2704     4 */
	int                        online;               /*  2708     4 */
	struct list_head           cfs_tasks;            /*  2712    16 */

	/* XXX 24 bytes hole, try to pack */

	/* --- cacheline 43 boundary (2752 bytes) --- */
	struct sched_avg           avg_rt __attribute__((__aligned__(64))); /*  2752    64 */
	/* --- cacheline 44 boundary (2816 bytes) --- */
	struct sched_avg           avg_dl __attribute__((__aligned__(64))); /*  2816    64 */
	/* --- cacheline 45 boundary (2880 bytes) --- */
	struct sched_avg           avg_irq __attribute__((__aligned__(64))); /*  2880    64 */
	/* --- cacheline 46 boundary (2944 bytes) --- */
	u64                        idle_stamp;           /*  2944     8 */
	u64                        avg_idle;             /*  2952     8 */
	u64                        max_idle_balance_cost; /*  2960     8 */
	u64                        prev_irq_time;        /*  2968     8 */
	u64                        prev_steal_time;      /*  2976     8 */
	u64                        prev_steal_time_rq;   /*  2984     8 */
	long unsigned int          calc_load_update;     /*  2992     8 */
	long int                   calc_load_active;     /*  3000     8 */
	/* --- cacheline 47 boundary (3008 bytes) --- */
	call_single_data_t         hrtick_csd __attribute__((__aligned__(32))); /*  3008    32 */
	struct hrtimer             hrtick_timer __attribute__((__aligned__(8))); /*  3040    96 */
	/* --- cacheline 49 boundary (3136 bytes) --- */
	ktime_t                    hrtick_time;          /*  3136     8 */
	struct sched_info          rq_sched_info;        /*  3144    64 */
	/* --- cacheline 50 boundary (3200 bytes) was 8 bytes ago --- */
	long long unsigned int     rq_cpu_time;          /*  3208     8 */
	unsigned int               yld_count;            /*  3216     4 */
	unsigned int               sched_count;          /*  3220     4 */
	unsigned int               sched_goidle;         /*  3224     4 */
	unsigned int               ttwu_count;           /*  3228     4 */
	unsigned int               ttwu_local;           /*  3232     4 */
	unsigned int               found_idle_cpu_capacity; /*  3236     4 */
	unsigned int               found_idle_cpu;       /*  3240     4 */
	unsigned int               found_idle_cpu_easy;  /*  3244     4 */
	unsigned int               nofound_idle_cpu;     /*  3248     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          find_time;            /*  3256     8 */
	/* --- cacheline 51 boundary (3264 bytes) --- */
	unsigned int               steal;                /*  3264     4 */
	unsigned int               steal_fail;           /*  3268     4 */
	struct cpuidle_state *     idle_state;           /*  3272     8 */
	u64                        kabi_reserved1;       /*  3280     8 */
	u64                        kabi_reserved2;       /*  3288     8 */
	u64                        kabi_reserved3;       /*  3296     8 */
	u64                        kabi_reserved4;       /*  3304     8 */
	u64                        kabi_reserved5;       /*  3312     8 */
	u64                        kabi_reserved6;       /*  3320     8 */
	/* --- cacheline 52 boundary (3328 bytes) --- */
	u64                        kabi_reserved7;       /*  3328     8 */
	u64                        kabi_reserved8;       /*  3336     8 */

	/* size: 3392, cachelines: 53, members: 82 */
	/* sum members: 3206, holes: 9, sum holes: 138 */
	/* padding: 48 */
	/* paddings: 1, sum paddings: 8 */
	/* forced alignments: 8, forced holes: 4, sum forced holes: 116 */
} __attribute__((__aligned__(64)));
struct wake_q_head {
	struct wake_q_node *       first;                /*     0     8 */
	struct wake_q_node * *     lastp;                /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct sched_attr {
	__u32                      size;                 /*     0     4 */
	__u32                      sched_policy;         /*     4     4 */
	__u64                      sched_flags;          /*     8     8 */
	__s32                      sched_nice;           /*    16     4 */
	__u32                      sched_priority;       /*    20     4 */
	__u64                      sched_runtime;        /*    24     8 */
	__u64                      sched_deadline;       /*    32     8 */
	__u64                      sched_period;         /*    40     8 */
	__u32                      sched_util_min;       /*    48     4 */
	__u32                      sched_util_max;       /*    52     4 */

	/* size: 56, cachelines: 1, members: 10 */
	/* last cacheline: 56 bytes */
};
struct cpuidle_state_usage {
	long long unsigned int     disable;              /*     0     8 */
	long long unsigned int     usage;                /*     8     8 */
	u64                        time_ns;              /*    16     8 */
	long long unsigned int     above;                /*    24     8 */
	long long unsigned int     below;                /*    32     8 */
	long long unsigned int     rejected;             /*    40     8 */
	long long unsigned int     s2idle_usage;         /*    48     8 */
	long long unsigned int     s2idle_time;          /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u64                        kabi_reserved1;       /*    64     8 */
	u64                        kabi_reserved2;       /*    72     8 */
	u64                        kabi_reserved3;       /*    80     8 */
	u64                        kabi_reserved4;       /*    88     8 */

	/* size: 96, cachelines: 2, members: 12 */
	/* last cacheline: 32 bytes */
};
struct cpuidle_state {
	char                       name[16];             /*     0    16 */
	char                       desc[32];             /*    16    32 */
	u64                        exit_latency_ns;      /*    48     8 */
	u64                        target_residency_ns;  /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	unsigned int               flags;                /*    64     4 */
	unsigned int               exit_latency;         /*    68     4 */
	int                        power_usage;          /*    72     4 */
	unsigned int               target_residency;     /*    76     4 */
	int                        (*enter)(struct cpuidle_device *, struct cpuidle_driver *, int); /*    80     8 */
	int                        (*enter_dead)(struct cpuidle_device *, int); /*    88     8 */
	int                        (*enter_s2idle)(struct cpuidle_device *, struct cpuidle_driver *, int); /*    96     8 */
	u64                        kabi_reserved1;       /*   104     8 */
	u64                        kabi_reserved2;       /*   112     8 */
	u64                        kabi_reserved3;       /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	u64                        kabi_reserved4;       /*   128     8 */

	/* size: 136, cachelines: 3, members: 15 */
	/* last cacheline: 8 bytes */
};
struct cpuidle_device {
	unsigned int               registered:1;         /*     0: 0  4 */
	unsigned int               enabled:1;            /*     0: 1  4 */
	unsigned int               poll_time_limit:1;    /*     0: 2  4 */

	/* XXX 29 bits hole, try to pack */

	unsigned int               cpu;                  /*     4     4 */
	ktime_t                    next_hrtimer;         /*     8     8 */
	int                        last_state_idx;       /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        last_residency_ns;    /*    24     8 */
	u64                        poll_limit_ns;        /*    32     8 */
	u64                        forced_idle_latency_limit_ns; /*    40     8 */
	struct cpuidle_state_usage states_usage[10];     /*    48   960 */
	/* --- cacheline 15 boundary (960 bytes) was 48 bytes ago --- */
	struct cpuidle_state_kobj * kobjs[10];           /*  1008    80 */
	/* --- cacheline 17 boundary (1088 bytes) --- */
	struct cpuidle_driver_kobj * kobj_driver;        /*  1088     8 */
	struct cpuidle_device_kobj * kobj_dev;           /*  1096     8 */
	struct list_head           device_list;          /*  1104    16 */
	u64                        kabi_reserved1;       /*  1120     8 */
	u64                        kabi_reserved2;       /*  1128     8 */
	u64                        kabi_reserved3;       /*  1136     8 */
	u64                        kabi_reserved4;       /*  1144     8 */

	/* size: 1152, cachelines: 18, members: 18 */
	/* sum members: 1144, holes: 1, sum holes: 4 */
	/* sum bitfield members: 3 bits, bit holes: 1, sum bit holes: 29 bits */
};
struct cpuidle_driver {
	const char  *              name;                 /*     0     8 */
	struct module *            owner;                /*     8     8 */
	unsigned int               bctimer:1;            /*    16: 0  4 */

	/* XXX 31 bits hole, try to pack */
	/* XXX 4 bytes hole, try to pack */

	struct cpuidle_state       states[10];           /*    24  1360 */
	/* --- cacheline 21 boundary (1344 bytes) was 40 bytes ago --- */
	int                        state_count;          /*  1384     4 */
	int                        safe_state_index;     /*  1388     4 */
	struct cpumask *           cpumask;              /*  1392     8 */
	const char  *              governor;             /*  1400     8 */
	/* --- cacheline 22 boundary (1408 bytes) --- */
	u64                        kabi_reserved1;       /*  1408     8 */
	u64                        kabi_reserved2;       /*  1416     8 */

	/* size: 1424, cachelines: 23, members: 10 */
	/* sum members: 1416, holes: 1, sum holes: 4 */
	/* sum bitfield members: 1 bits, bit holes: 1, sum bit holes: 31 bits */
	/* last cacheline: 16 bytes */
};
struct cpu_stop_work {
	struct list_head           list;                 /*     0    16 */
	cpu_stop_fn_t              fn;                   /*    16     8 */
	void *                     arg;                  /*    24     8 */
	struct cpu_stop_done *     done;                 /*    32     8 */
	u64                        kabi_reserved1;       /*    40     8 */

	/* size: 48, cachelines: 1, members: 5 */
	/* last cacheline: 48 bytes */
};
struct cpupri_vec {
	atomic_t                   count;                /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	cpumask_var_t              mask;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct cpupri {
	struct cpupri_vec          pri_to_cpu[102];      /*     0  1632 */
	/* --- cacheline 25 boundary (1600 bytes) was 32 bytes ago --- */
	int *                      cpu_to_pri;           /*  1632     8 */

	/* size: 1640, cachelines: 26, members: 2 */
	/* last cacheline: 40 bytes */
};
struct cpudl_item {
	u64                        dl;                   /*     0     8 */
	int                        cpu;                  /*     8     4 */
	int                        idx;                  /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct cpudl {
	raw_spinlock_t             lock;                 /*     0     4 */
	int                        size;                 /*     4     4 */
	cpumask_var_t              free_cpus;            /*     8     8 */
	struct cpudl_item *        elements;             /*    16     8 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct rt_prio_array {
	long unsigned int          bitmap[2];            /*     0    16 */
	struct list_head           queue[100];           /*    16  1600 */

	/* size: 1616, cachelines: 26, members: 2 */
	/* last cacheline: 16 bytes */
};
struct rt_bandwidth {
	raw_spinlock_t             rt_runtime_lock;      /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	ktime_t                    rt_period;            /*     8     8 */
	u64                        rt_runtime;           /*    16     8 */
	struct hrtimer             rt_period_timer __attribute__((__aligned__(8))); /*    24    96 */
	/* --- cacheline 1 boundary (64 bytes) was 56 bytes ago --- */
	unsigned int               rt_period_active;     /*   120     4 */

	/* size: 128, cachelines: 2, members: 5 */
	/* sum members: 120, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(8)));
struct dl_bandwidth {
	raw_spinlock_t             dl_runtime_lock;      /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        dl_runtime;           /*     8     8 */
	u64                        dl_period;            /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct dl_bw {
	raw_spinlock_t             lock;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        bw;                   /*     8     8 */
	u64                        total_bw;             /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct cfs_bandwidth {
	raw_spinlock_t             lock;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	ktime_t                    period;               /*     8     8 */
	u64                        quota;                /*    16     8 */
	u64                        runtime;              /*    24     8 */
	s64                        hierarchical_quota;   /*    32     8 */
	u8                         idle;                 /*    40     1 */
	u8                         period_active;        /*    41     1 */
	u8                         slack_started;        /*    42     1 */

	/* XXX 5 bytes hole, try to pack */

	struct hrtimer             period_timer __attribute__((__aligned__(8))); /*    48    96 */
	/* --- cacheline 2 boundary (128 bytes) was 16 bytes ago --- */
	struct hrtimer             slack_timer __attribute__((__aligned__(8))); /*   144    96 */
	/* --- cacheline 3 boundary (192 bytes) was 48 bytes ago --- */
	struct list_head           throttled_cfs_rq;     /*   240    16 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	int                        nr_periods;           /*   256     4 */
	int                        nr_throttled;         /*   260     4 */
	u64                        throttled_time;       /*   264     8 */
	u64                        burst;                /*   272     8 */
	u64                        runtime_snap;         /*   280     8 */
	int                        nr_burst;             /*   288     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        burst_time;           /*   296     8 */
	u64                        kabi_reserved5;       /*   304     8 */
	u64                        kabi_reserved6;       /*   312     8 */

	/* size: 320, cachelines: 5, members: 20 */
	/* sum members: 307, holes: 3, sum holes: 13 */
	/* forced alignments: 2, forced holes: 1, sum forced holes: 5 */
} __attribute__((__aligned__(8)));
struct dl_rq {
	struct rb_root_cached      root;                 /*     0    16 */
	long unsigned int          dl_nr_running;        /*    16     8 */
	struct {
		u64                curr;                 /*    24     8 */
		u64                next;                 /*    32     8 */
	} earliest_dl;                                   /*    24    16 */
	long unsigned int          dl_nr_migratory;      /*    40     8 */
	int                        overloaded;           /*    48     4 */

	/* XXX 4 bytes hole, try to pack */

	struct rb_root_cached      pushable_dl_tasks_root; /*    56    16 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	u64                        running_bw;           /*    72     8 */
	u64                        this_bw;              /*    80     8 */
	u64                        extra_bw;             /*    88     8 */
	u64                        bw_ratio;             /*    96     8 */

	/* size: 104, cachelines: 2, members: 10 */
	/* sum members: 100, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct perf_domain {
	struct em_perf_domain *    em_pd;                /*     0     8 */
	struct perf_domain *       next;                 /*     8     8 */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*    16    16 */

	/* size: 32, cachelines: 1, members: 3 */
	/* forced alignments: 1 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct rq_flags {
	long unsigned int          flags;                /*     0     8 */
	struct pin_cookie          cookie;               /*     8     0 */
	unsigned int               clock_update_flags;   /*     8     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct irqtime {
	u64                        total;                /*     0     8 */
	u64                        tick_delta;           /*     8     8 */
	u64                        irq_start_time;       /*    16     8 */
	struct u64_stats_sync      sync;                 /*    24     0 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct migration_arg {
	struct task_struct *       task;                 /*     0     8 */
	int                        dest_cpu;             /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct migration_swap_arg {
	struct task_struct *       src_task;             /*     0     8 */
	struct task_struct *       dst_task;             /*     8     8 */
	int                        src_cpu;              /*    16     4 */
	int                        dst_cpu;              /*    20     4 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct tick_work {
	int                        cpu;                  /*     0     4 */
	atomic_t                   state;                /*     4     4 */
	struct delayed_work        work;                 /*     8   192 */

	/* size: 200, cachelines: 4, members: 3 */
	/* last cacheline: 8 bytes */
};
struct offline_args {
	struct work_struct         work;                 /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct task_struct *       p;                    /*    64     8 */

	/* size: 72, cachelines: 2, members: 2 */
	/* last cacheline: 8 bytes */
};
struct cfs_schedulable_data {
	struct task_group *        tg;                   /*     0     8 */
	u64                        period;               /*     8     8 */
	u64                        quota;                /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct sched_clock_data {
	u64                        tick_raw;             /*     0     8 */
	u64                        tick_gtod;            /*     8     8 */
	u64                        clock;                /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct idle_timer {
	struct hrtimer             timer __attribute__((__aligned__(8))); /*     0    96 */
	/* --- cacheline 1 boundary (64 bytes) was 32 bytes ago --- */
	int                        done;                 /*    96     4 */

	/* size: 104, cachelines: 2, members: 2 */
	/* padding: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 40 bytes */
} __attribute__((__aligned__(8)));
struct numa_group {
	refcount_t                 refcount;             /*     0     4 */
	spinlock_t                 lock;                 /*     4     4 */
	int                        nr_tasks;             /*     8     4 */
	pid_t                      gid;                  /*    12     4 */
	int                        active_nodes;         /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	struct callback_head       rcu __attribute__((__aligned__(8))); /*    24    16 */
	long unsigned int          total_faults;         /*    40     8 */
	long unsigned int          max_faults_cpu;       /*    48     8 */
	long unsigned int *        faults_cpu;           /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	long unsigned int          faults[];             /*    64     0 */

	/* size: 64, cachelines: 1, members: 10 */
	/* sum members: 60, holes: 1, sum holes: 4 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 4 */
} __attribute__((__aligned__(8)));
struct sched_migrate_ctx {
	struct task_struct *       task;                 /*     0     8 */
	struct cpumask *           select_idle_mask;     /*     8     8 */
	int                        prev_cpu;             /*    16     4 */
	int                        curr_cpu;             /*    20     4 */
	int                        is_sync;              /*    24     4 */
	int                        want_affine;          /*    28     4 */
	int                        wake_flags;           /*    32     4 */
	int                        sd_flag;              /*    36     4 */
	int                        new_cpu;              /*    40     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        kabi_reserved1;       /*    48     8 */
	u64                        kabi_reserved2;       /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u64                        kabi_reserved3;       /*    64     8 */
	u64                        kabi_reserved4;       /*    72     8 */

	/* size: 80, cachelines: 2, members: 13 */
	/* sum members: 76, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct sched_affine_ctx {
	struct task_struct *       task;                 /*     0     8 */
	int                        prev_cpu;             /*     8     4 */
	int                        curr_cpu;             /*    12     4 */
	int                        is_sync;              /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        kabi_reserved1;       /*    24     8 */
	u64                        kabi_reserved2;       /*    32     8 */
	u64                        kabi_reserved3;       /*    40     8 */
	u64                        kabi_reserved4;       /*    48     8 */

	/* size: 56, cachelines: 1, members: 8 */
	/* sum members: 52, holes: 1, sum holes: 4 */
	/* last cacheline: 56 bytes */
};
struct update_util_data {
	void                       (*func)(struct update_util_data *, u64, unsigned int); /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct sparsemask {
	short int                  nelems;               /*     0     2 */
	short int                  density;              /*     2     2 */

	/* XXX 60 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	struct sparsemask_chunk    chunks[] __attribute__((__aligned__(64))); /*    64     0 */

	/* size: 64, cachelines: 1, members: 3 */
	/* sum members: 4, holes: 1, sum holes: 60 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 60 */
} __attribute__((__aligned__(64)));
struct sparsemask_chunk {
	long unsigned int          word;                 /*     0     8 */

	/* size: 64, cachelines: 1, members: 1 */
	/* padding: 56 */
} __attribute__((__aligned__(64)));
struct numa_stats {
	long unsigned int          load;                 /*     0     8 */
	long unsigned int          runnable;             /*     8     8 */
	long unsigned int          util;                 /*    16     8 */
	long unsigned int          compute_capacity;     /*    24     8 */
	unsigned int               nr_running;           /*    32     4 */
	unsigned int               weight;               /*    36     4 */
	enum numa_type             node_type;            /*    40     4 */
	int                        idle_cpu;             /*    44     4 */

	/* size: 48, cachelines: 1, members: 8 */
	/* last cacheline: 48 bytes */
};
struct task_numa_env {
	struct task_struct *       p;                    /*     0     8 */
	int                        src_cpu;              /*     8     4 */
	int                        src_nid;              /*    12     4 */
	int                        dst_cpu;              /*    16     4 */
	int                        dst_nid;              /*    20     4 */
	struct numa_stats          src_stats;            /*    24    48 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	struct numa_stats          dst_stats;            /*    72    48 */
	int                        imbalance_pct;        /*   120     4 */
	int                        dist;                 /*   124     4 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct task_struct *       best_task;            /*   128     8 */
	long int                   best_imp;             /*   136     8 */
	int                        best_cpu;             /*   144     4 */

	/* size: 152, cachelines: 3, members: 12 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct lb_env {
	struct sched_domain *      sd;                   /*     0     8 */
	struct rq *                src_rq;               /*     8     8 */
	int                        src_cpu;              /*    16     4 */
	int                        dst_cpu;              /*    20     4 */
	struct rq *                dst_rq;               /*    24     8 */
	struct cpumask *           dst_grpmask;          /*    32     8 */
	int                        new_dst_cpu;          /*    40     4 */
	enum cpu_idle_type         idle;                 /*    44     4 */
	long int                   imbalance;            /*    48     8 */
	struct cpumask *           cpus;                 /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	unsigned int               flags;                /*    64     4 */
	unsigned int               loop;                 /*    68     4 */
	unsigned int               loop_break;           /*    72     4 */
	unsigned int               loop_max;             /*    76     4 */
	enum fbq_type              fbq_type;             /*    80     4 */
	enum migration_type        migration_type;       /*    84     4 */
	struct list_head           tasks;                /*    88    16 */

	/* size: 104, cachelines: 2, members: 17 */
	/* last cacheline: 40 bytes */
};
struct sg_lb_stats {
	long unsigned int          avg_load;             /*     0     8 */
	long unsigned int          group_load;           /*     8     8 */
	long unsigned int          group_capacity;       /*    16     8 */
	long unsigned int          group_util;           /*    24     8 */
	long unsigned int          group_runnable;       /*    32     8 */
	unsigned int               sum_nr_running;       /*    40     4 */
	unsigned int               sum_h_nr_running;     /*    44     4 */
	unsigned int               idle_cpus;            /*    48     4 */
	unsigned int               group_weight;         /*    52     4 */
	enum group_type            group_type;           /*    56     4 */
	unsigned int               group_asym_packing;   /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	long unsigned int          group_misfit_task_load; /*    64     8 */
	unsigned int               nr_numa_running;      /*    72     4 */
	unsigned int               nr_preferred_running; /*    76     4 */

	/* size: 80, cachelines: 2, members: 14 */
	/* last cacheline: 16 bytes */
};
struct sd_lb_stats {
	struct sched_group *       busiest;              /*     0     8 */
	struct sched_group *       local;                /*     8     8 */
	long unsigned int          total_load;           /*    16     8 */
	long unsigned int          total_capacity;       /*    24     8 */
	long unsigned int          avg_load;             /*    32     8 */
	unsigned int               prefer_sibling;       /*    40     4 */

	/* XXX 4 bytes hole, try to pack */

	struct sg_lb_stats         busiest_stat;         /*    48    80 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct sg_lb_stats         local_stat;           /*   128    80 */

	/* size: 208, cachelines: 4, members: 8 */
	/* sum members: 204, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct rt_schedulable_data {
	struct task_group *        tg;                   /*     0     8 */
	u64                        rt_period;            /*     8     8 */
	u64                        rt_runtime;           /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct wait_bit_key {
	void *                     flags;                /*     0     8 */
	int                        bit_nr;               /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          timeout;              /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct wait_bit_queue_entry {
	struct wait_bit_key        key;                  /*     0    24 */
	struct wait_queue_entry    wq_entry;             /*    24    40 */

	/* size: 64, cachelines: 1, members: 2 */
};
struct sd_flag_debug {
	unsigned int               meta_flags;           /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	char *                     name;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct sched_domain_attr {
	int                        relax_domain_level;   /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct s_data {
	struct sched_domain * *    sd;                   /*     0     8 */
	struct root_domain *       rd;                   /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct cpuacct_usage {
	u64                        usages[2];            /*     0    16 */

	/* size: 16, cachelines: 1, members: 1 */
	/* last cacheline: 16 bytes */
};
struct cpuacct {
	struct cgroup_subsys_state css __attribute__((__aligned__(8))); /*     0   296 */
	/* --- cacheline 4 boundary (256 bytes) was 40 bytes ago --- */
	struct cpuacct_usage *     cpuusage;             /*   296     8 */
	struct kernel_cpustat *    cpustat;              /*   304     8 */

	/* size: 312, cachelines: 5, members: 3 */
	/* forced alignments: 1 */
	/* last cacheline: 56 bytes */
} __attribute__((__aligned__(8)));
struct gov_attr_set {
	struct kobject             kobj;                 /*     0    96 */
	/* --- cacheline 1 boundary (64 bytes) was 32 bytes ago --- */
	struct list_head           policy_list;          /*    96    16 */
	struct mutex               update_lock;          /*   112    32 */
	/* --- cacheline 2 boundary (128 bytes) was 16 bytes ago --- */
	int                        usage_count;          /*   144     4 */

	/* size: 152, cachelines: 3, members: 4 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct governor_attr {
	struct attribute           attr;                 /*     0    16 */

	/* XXX last struct has 6 bytes of padding */

	ssize_t                    (*show)(struct gov_attr_set *, char *); /*    16     8 */
	ssize_t                    (*store)(struct gov_attr_set *, const char  *, size_t); /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* paddings: 1, sum paddings: 6 */
	/* last cacheline: 32 bytes */
};
struct sugov_tunables {
	struct gov_attr_set        attr_set;             /*     0   152 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 2 boundary (128 bytes) was 24 bytes ago --- */
	unsigned int               rate_limit_us;        /*   152     4 */

	/* size: 160, cachelines: 3, members: 2 */
	/* padding: 4 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 32 bytes */
};
struct sugov_policy {
	struct cpufreq_policy *    policy;               /*     0     8 */
	struct sugov_tunables *    tunables;             /*     8     8 */
	struct list_head           tunables_hook;        /*    16    16 */
	raw_spinlock_t             update_lock;          /*    32     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        last_freq_update_time; /*    40     8 */
	s64                        freq_update_delay_ns; /*    48     8 */
	unsigned int               next_freq;            /*    56     4 */
	unsigned int               cached_raw_freq;      /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct irq_work            irq_work;             /*    64    24 */
	struct kthread_work        work;                 /*    88    40 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 2 boundary (128 bytes) --- */
	struct mutex               work_lock;            /*   128    32 */
	struct kthread_worker      worker;               /*   160    56 */
	/* --- cacheline 3 boundary (192 bytes) was 24 bytes ago --- */
	struct task_struct *       thread;               /*   216     8 */
	bool                       work_in_progress;     /*   224     1 */
	bool                       limits_changed;       /*   225     1 */
	bool                       need_freq_update;     /*   226     1 */

	/* size: 232, cachelines: 4, members: 16 */
	/* sum members: 223, holes: 1, sum holes: 4 */
	/* padding: 5 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 40 bytes */
};
struct sugov_cpu {
	struct update_util_data    update_util;          /*     0     8 */
	struct sugov_policy *      sg_policy;            /*     8     8 */
	unsigned int               cpu;                  /*    16     4 */
	bool                       iowait_boost_pending; /*    20     1 */

	/* XXX 3 bytes hole, try to pack */

	unsigned int               iowait_boost;         /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        last_update;          /*    32     8 */
	long unsigned int          bw_dl;                /*    40     8 */
	long unsigned int          max;                  /*    48     8 */
	long unsigned int          saved_idle_calls;     /*    56     8 */

	/* size: 64, cachelines: 1, members: 9 */
	/* sum members: 57, holes: 2, sum holes: 7 */
};
struct psi_window {
	u64                        size;                 /*     0     8 */
	u64                        start_time;           /*     8     8 */
	u64                        start_value;          /*    16     8 */
	u64                        prev_growth;          /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct psi_trigger {
	enum psi_states            state;                /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        threshold;            /*     8     8 */
	struct list_head           node;                 /*    16    16 */
	struct psi_group *         group;                /*    32     8 */
	wait_queue_head_t          event_wait;           /*    40    24 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        event;                /*    64     4 */

	/* XXX 4 bytes hole, try to pack */

	struct psi_window          win;                  /*    72    32 */
	u64                        last_event_time;      /*   104     8 */

	/* size: 112, cachelines: 2, members: 8 */
	/* sum members: 104, holes: 2, sum holes: 8 */
	/* last cacheline: 48 bytes */
};
struct bpf_sched_cpu_stats {
	long unsigned int          cfs_load_avg;         /*     0     8 */
	long unsigned int          cfs_runnable_avg;     /*     8     8 */
	long unsigned int          cfs_util_avg;         /*    16     8 */
	long unsigned int          rt_load_avg;          /*    24     8 */
	long unsigned int          rt_runnable_avg;      /*    32     8 */
	long unsigned int          rt_util_avg;          /*    40     8 */
	long unsigned int          irq_load_avg;         /*    48     8 */
	long unsigned int          irq_runnable_avg;     /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	long unsigned int          irq_util_avg;         /*    64     8 */
	unsigned int               nr_running;           /*    72     4 */
	unsigned int               cfs_nr_running;       /*    76     4 */
	unsigned int               cfs_h_nr_running;     /*    80     4 */
	unsigned int               cfs_idle_h_nr_running; /*    84     4 */
	unsigned int               rt_nr_running;        /*    88     4 */
	unsigned int               rr_nr_running;        /*    92     4 */
	int                        available_idle;       /*    96     4 */
	unsigned int               exit_latency;         /*   100     4 */
	long unsigned int          idle_stamp;           /*   104     8 */
	long unsigned int          avg_idle;             /*   112     8 */
	long unsigned int          capacity;             /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	long unsigned int          capacity_orig;        /*   128     8 */
	u64                        kabi_reserved1;       /*   136     8 */
	u64                        kabi_reserved2;       /*   144     8 */
	u64                        kabi_reserved3;       /*   152     8 */
	u64                        kabi_reserved4;       /*   160     8 */

	/* size: 168, cachelines: 3, members: 25 */
	/* last cacheline: 40 bytes */
};
struct cpumask_op_args {
	unsigned int               op_type;              /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     arg1;                 /*     8     8 */
	void *                     arg2;                 /*    16     8 */
	void *                     arg3;                 /*    24     8 */
	void *                     arg4;                 /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* sum members: 36, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct bpf_func_proto {
	u64                        (*func)(u64, u64, u64, u64, u64); /*     0     8 */
	bool                       gpl_only;             /*     8     1 */
	bool                       pkt_access;           /*     9     1 */

	/* XXX 2 bytes hole, try to pack */

	enum bpf_return_type       ret_type;             /*    12     4 */
	union {
		struct {
			enum bpf_arg_type arg1_type;     /*    16     4 */
			enum bpf_arg_type arg2_type;     /*    20     4 */
			enum bpf_arg_type arg3_type;     /*    24     4 */
			enum bpf_arg_type arg4_type;     /*    28     4 */
			enum bpf_arg_type arg5_type;     /*    32     4 */
		};                                       /*    16    20 */
		enum bpf_arg_type  arg_type[5];          /*    16    20 */
	};                                               /*    16    20 */

	/* XXX 4 bytes hole, try to pack */

	union {
		struct {
			u32 *      arg1_btf_id;          /*    40     8 */
			u32 *      arg2_btf_id;          /*    48     8 */
			u32 *      arg3_btf_id;          /*    56     8 */
			/* --- cacheline 1 boundary (64 bytes) --- */
			u32 *      arg4_btf_id;          /*    64     8 */
			u32 *      arg5_btf_id;          /*    72     8 */
		};                                       /*    40    40 */
		u32 *              arg_btf_id[5];        /*    40    40 */
	};                                               /*    40    40 */
	/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
	int *                      ret_btf_id;           /*    80     8 */
	bool                       (*allowed)(const struct bpf_prog  *); /*    88     8 */

	/* size: 96, cachelines: 2, members: 8 */
	/* sum members: 90, holes: 2, sum holes: 6 */
	/* last cacheline: 32 bytes */
};
struct bpf_insn_access_aux {
	enum bpf_reg_type          reg_type;             /*     0     4 */
	union {
		int                ctx_field_size;       /*     4     4 */
		u32                btf_id;               /*     4     4 */
	};                                               /*     4     4 */
	struct bpf_verifier_log *  log;                  /*     8     8 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct bpf_verifier_log {
	u32                        level;                /*     0     4 */
	char                       kbuf[1024];           /*     4  1024 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 16 boundary (1024 bytes) was 8 bytes ago --- */
	char *                     ubuf;                 /*  1032     8 */
	u32                        len_used;             /*  1040     4 */
	u32                        len_total;            /*  1044     4 */

	/* size: 1048, cachelines: 17, members: 5 */
	/* sum members: 1044, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct bpf_verifier_ops {
	const struct bpf_func_proto  * (*get_func_proto)(enum bpf_func_id, const struct bpf_prog  *); /*     0     8 */
	bool                       (*is_valid_access)(int, int, enum bpf_access_type, const struct bpf_prog  *, struct bpf_insn_access_aux *); /*     8     8 */
	int                        (*gen_prologue)(struct bpf_insn *, bool, const struct bpf_prog  *); /*    16     8 */
	int                        (*gen_ld_abs)(const struct bpf_insn  *, struct bpf_insn *); /*    24     8 */
	u32                        (*convert_ctx_access)(enum bpf_access_type, const struct bpf_insn  *, struct bpf_insn *, struct bpf_prog *, u32 *); /*    32     8 */
	int                        (*btf_struct_access)(struct bpf_verifier_log *, const struct btf_type  *, int, int, enum bpf_access_type, u32 *); /*    40     8 */

	/* size: 48, cachelines: 1, members: 6 */
	/* last cacheline: 48 bytes */
};
struct btf_id_set {
	u32                        cnt;                  /*     0     4 */
	u32                        ids[];                /*     4     0 */

	/* size: 4, cachelines: 1, members: 2 */
	/* last cacheline: 4 bytes */
};
struct bpf_cpu_topology {
	int                        cpu;                  /*     0     4 */
	int                        core_id;              /*     4     4 */
	int                        cluster_id;           /*     8     4 */
	int                        die_id;               /*    12     4 */
	int                        physical_package_id;  /*    16     4 */
	int                        numa_node;            /*    20     4 */
	struct cpumask             thread_siblings;      /*    24  1024 */
	/* --- cacheline 16 boundary (1024 bytes) was 24 bytes ago --- */
	struct cpumask             core_siblings;        /*  1048  1024 */
	/* --- cacheline 32 boundary (2048 bytes) was 24 bytes ago --- */
	struct cpumask             cluster_cpus;         /*  2072  1024 */
	/* --- cacheline 48 boundary (3072 bytes) was 24 bytes ago --- */
	struct cpumask             die_cpus;             /*  3096  1024 */
	/* --- cacheline 64 boundary (4096 bytes) was 24 bytes ago --- */
	struct cpumask             package_cpus;         /*  4120  1024 */
	/* --- cacheline 80 boundary (5120 bytes) was 24 bytes ago --- */
	struct cpumask             node_cpu_lists;       /*  5144  1024 */

	/* size: 6168, cachelines: 97, members: 12 */
	/* last cacheline: 24 bytes */
};
struct bpf_cpumask_info {
	unsigned int               nums_possible_cpus;   /*     0     4 */
	unsigned int               nums_active_cpus;     /*     4     4 */
	unsigned int               nums_isolate_cpus;    /*     8     4 */
	unsigned int               bpf_nr_cpu_ids;       /*    12     4 */
	unsigned int               bpf_nr_cpumask_bits;  /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	struct cpumask             cpu_possible_cpumask; /*    24  1024 */
	/* --- cacheline 16 boundary (1024 bytes) was 24 bytes ago --- */
	struct cpumask             cpu_active_cpumask;   /*  1048  1024 */
	/* --- cacheline 32 boundary (2048 bytes) was 24 bytes ago --- */
	struct cpumask             cpu_isolate_cpumask;  /*  2072  1024 */

	/* size: 3096, cachelines: 49, members: 8 */
	/* sum members: 3092, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct ww_mutex {
	struct mutex               base;                 /*     0    32 */
	struct ww_acquire_ctx *    ctx;                  /*    32     8 */

	/* size: 40, cachelines: 1, members: 2 */
	/* last cacheline: 40 bytes */
};
struct ww_acquire_ctx {
	struct task_struct *       task;                 /*     0     8 */
	long unsigned int          stamp;                /*     8     8 */
	unsigned int               acquired;             /*    16     4 */
	short unsigned int         wounded;              /*    20     2 */
	short unsigned int         is_wait_die;          /*    22     2 */

	/* size: 24, cachelines: 1, members: 5 */
	/* last cacheline: 24 bytes */
};
struct mutex_waiter {
	struct list_head           list;                 /*     0    16 */
	struct task_struct *       task;                 /*    16     8 */
	struct ww_acquire_ctx *    ww_ctx;               /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* last cacheline: 32 bytes */
};
struct semaphore_waiter {
	struct list_head           list;                 /*     0    16 */
	struct task_struct *       task;                 /*    16     8 */
	bool                       up;                   /*    24     1 */

	/* size: 32, cachelines: 1, members: 3 */
	/* padding: 7 */
	/* last cacheline: 32 bytes */
};
struct rwsem_waiter {
	struct list_head           list;                 /*     0    16 */
	struct task_struct *       task;                 /*    16     8 */
	enum rwsem_waiter_type     type;                 /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          timeout;              /*    32     8 */
	long unsigned int          last_rowner;          /*    40     8 */

	/* size: 48, cachelines: 1, members: 5 */
	/* sum members: 44, holes: 1, sum holes: 4 */
	/* last cacheline: 48 bytes */
};
struct optimistic_spin_node {
	struct optimistic_spin_node * next;              /*     0     8 */
	struct optimistic_spin_node * prev;              /*     8     8 */
	int                        locked;               /*    16     4 */
	int                        cpu;                  /*    20     4 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct mcs_spinlock {
	struct mcs_spinlock *      next;                 /*     0     8 */
	unsigned int               locked;               /*     8     4 */
	int                        count;                /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct qnode {
	struct mcs_spinlock        mcs;                  /*     0    16 */
	long int                   reserved[2];          /*    16    16 */

	/* size: 32, cachelines: 1, members: 2 */
	/* last cacheline: 32 bytes */
};
struct cna_node {
	struct mcs_spinlock        mcs;                  /*     0    16 */
	u16                        numa_node;            /*    16     2 */
	u16                        real_numa_node;       /*    18     2 */
	u32                        encoded_tail;         /*    20     4 */
	u64                        start_time;           /*    24     8 */

	/* size: 32, cachelines: 1, members: 5 */
	/* last cacheline: 32 bytes */
};
struct pv_node {
	struct mcs_spinlock        mcs;                  /*     0    16 */
	int                        cpu;                  /*    16     4 */
	u8                         state;                /*    20     1 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 3 */
	/* last cacheline: 24 bytes */
};
struct pv_hash_entry {
	struct qspinlock *         lock;                 /*     0     8 */
	struct pv_node *           node;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct hrtimer_sleeper {
	struct hrtimer             timer __attribute__((__aligned__(8))); /*     0    96 */
	/* --- cacheline 1 boundary (64 bytes) was 32 bytes ago --- */
	struct task_struct *       task;                 /*    96     8 */

	/* size: 104, cachelines: 2, members: 2 */
	/* forced alignments: 1 */
	/* last cacheline: 40 bytes */
} __attribute__((__aligned__(8)));
struct rt_mutex_waiter {
	struct rb_node             tree_entry __attribute__((__aligned__(8))); /*     0    24 */
	struct rb_node             pi_tree_entry __attribute__((__aligned__(8))); /*    24    24 */
	struct task_struct *       task;                 /*    48     8 */
	struct rt_mutex *          lock;                 /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        prio;                 /*    64     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        deadline;             /*    72     8 */

	/* size: 80, cachelines: 2, members: 6 */
	/* sum members: 76, holes: 1, sum holes: 4 */
	/* forced alignments: 2 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct rt_mutex {
	raw_spinlock_t             wait_lock;            /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct rb_root_cached      waiters;              /*     8    16 */
	struct task_struct *       owner;                /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct suspend_stats {
	int                        success;              /*     0     4 */
	int                        fail;                 /*     4     4 */
	int                        failed_freeze;        /*     8     4 */
	int                        failed_prepare;       /*    12     4 */
	int                        failed_suspend;       /*    16     4 */
	int                        failed_suspend_late;  /*    20     4 */
	int                        failed_suspend_noirq; /*    24     4 */
	int                        failed_resume;        /*    28     4 */
	int                        failed_resume_early;  /*    32     4 */
	int                        failed_resume_noirq;  /*    36     4 */
	int                        last_failed_dev;      /*    40     4 */
	char                       failed_devs[2][40];   /*    44    80 */
	/* --- cacheline 1 boundary (64 bytes) was 60 bytes ago --- */
	int                        last_failed_errno;    /*   124     4 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	int                        errno[2];             /*   128     8 */
	int                        last_failed_step;     /*   136     4 */
	enum suspend_stat_step     failed_steps[2];      /*   140     8 */

	/* size: 148, cachelines: 3, members: 16 */
	/* last cacheline: 20 bytes */
};
struct pm_vt_switch {
	struct list_head           head;                 /*     0    16 */
	struct device *            dev;                  /*    16     8 */
	bool                       required;             /*    24     1 */

	/* size: 32, cachelines: 1, members: 3 */
	/* padding: 7 */
	/* last cacheline: 32 bytes */
};
struct platform_suspend_ops {
	int                        (*valid)(suspend_state_t); /*     0     8 */
	int                        (*begin)(suspend_state_t); /*     8     8 */
	int                        (*prepare)(void);     /*    16     8 */
	int                        (*prepare_late)(void); /*    24     8 */
	int                        (*enter)(suspend_state_t); /*    32     8 */
	void                       (*wake)(void);        /*    40     8 */
	void                       (*finish)(void);      /*    48     8 */
	bool                       (*suspend_again)(void); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	void                       (*end)(void);         /*    64     8 */
	void                       (*recover)(void);     /*    72     8 */

	/* size: 80, cachelines: 2, members: 10 */
	/* last cacheline: 16 bytes */
};
struct platform_s2idle_ops {
	int                        (*begin)(void);       /*     0     8 */
	int                        (*prepare)(void);     /*     8     8 */
	int                        (*prepare_late)(void); /*    16     8 */
	bool                       (*wake)(void);        /*    24     8 */
	void                       (*restore_early)(void); /*    32     8 */
	void                       (*restore)(void);     /*    40     8 */
	void                       (*end)(void);         /*    48     8 */

	/* size: 56, cachelines: 1, members: 7 */
	/* last cacheline: 56 bytes */
};
struct platform_hibernation_ops {
	int                        (*begin)(pm_message_t); /*     0     8 */
	void                       (*end)(void);         /*     8     8 */
	int                        (*pre_snapshot)(void); /*    16     8 */
	void                       (*finish)(void);      /*    24     8 */
	int                        (*prepare)(void);     /*    32     8 */
	int                        (*enter)(void);       /*    40     8 */
	void                       (*leave)(void);       /*    48     8 */
	int                        (*pre_restore)(void); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	void                       (*restore_cleanup)(void); /*    64     8 */
	void                       (*recover)(void);     /*    72     8 */

	/* size: 80, cachelines: 2, members: 10 */
	/* last cacheline: 16 bytes */
};
struct pbe {
	void *                     address;              /*     0     8 */
	void *                     orig_address;         /*     8     8 */
	struct pbe *               next;                 /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct swsusp_info {
	struct new_utsname         uts;                  /*     0   390 */

	/* XXX 2 bytes hole, try to pack */

	/* --- cacheline 6 boundary (384 bytes) was 8 bytes ago --- */
	u32                        version_code;         /*   392     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          num_physpages;        /*   400     8 */
	int                        cpus;                 /*   408     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          image_pages;          /*   416     8 */
	long unsigned int          pages;                /*   424     8 */
	long unsigned int          size;                 /*   432     8 */

	/* size: 4096, cachelines: 64, members: 7 */
	/* sum members: 430, holes: 3, sum holes: 10 */
	/* padding: 3656 */
} __attribute__((__aligned__(4096)));
struct snapshot_handle {
	unsigned int               cur;                  /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     buffer;               /*     8     8 */
	int                        sync_read;            /*    16     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 16, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct linked_page {
	struct linked_page *       next;                 /*     0     8 */
	char                       data[4088];           /*     8  4088 */

	/* size: 4096, cachelines: 64, members: 2 */
};
struct chain_allocator {
	struct linked_page *       chain;                /*     0     8 */
	unsigned int               used_space;           /*     8     4 */
	gfp_t                      gfp_mask;             /*    12     4 */
	int                        safe_needed;          /*    16     4 */

	/* size: 24, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct rtree_node {
	struct list_head           list;                 /*     0    16 */
	long unsigned int *        data;                 /*    16     8 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct mem_zone_bm_rtree {
	struct list_head           list;                 /*     0    16 */
	struct list_head           nodes;                /*    16    16 */
	struct list_head           leaves;               /*    32    16 */
	long unsigned int          start_pfn;            /*    48     8 */
	long unsigned int          end_pfn;              /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct rtree_node *        rtree;                /*    64     8 */
	int                        levels;               /*    72     4 */
	unsigned int               blocks;               /*    76     4 */

	/* size: 80, cachelines: 2, members: 8 */
	/* last cacheline: 16 bytes */
};
struct bm_position {
	struct mem_zone_bm_rtree * zone;                 /*     0     8 */
	struct rtree_node *        node;                 /*     8     8 */
	long unsigned int          node_pfn;             /*    16     8 */
	int                        node_bit;             /*    24     4 */

	/* size: 32, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct memory_bitmap {
	struct list_head           zones;                /*     0    16 */
	struct linked_page *       p_list;               /*    16     8 */
	struct bm_position         cur;                  /*    24    32 */

	/* XXX last struct has 4 bytes of padding */

	/* size: 56, cachelines: 1, members: 3 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 56 bytes */
};
struct mem_extent {
	struct list_head           hook;                 /*     0    16 */
	long unsigned int          start;                /*    16     8 */
	long unsigned int          end;                  /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* last cacheline: 32 bytes */
};
struct nosave_region {
	struct list_head           list;                 /*     0    16 */
	long unsigned int          start_pfn;            /*    16     8 */
	long unsigned int          end_pfn;              /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* last cacheline: 32 bytes */
};
struct swap_map_page {
	sector_t                   entries[511];         /*     0  4088 */
	/* --- cacheline 63 boundary (4032 bytes) was 56 bytes ago --- */
	sector_t                   next_swap;            /*  4088     8 */

	/* size: 4096, cachelines: 64, members: 2 */
};
struct swap_map_page_list {
	struct swap_map_page *     map;                  /*     0     8 */
	struct swap_map_page_list * next;                /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct swap_map_handle {
	struct swap_map_page *     cur;                  /*     0     8 */
	struct swap_map_page_list * maps;                /*     8     8 */
	sector_t                   cur_swap;             /*    16     8 */
	sector_t                   first_sector;         /*    24     8 */
	unsigned int               k;                    /*    32     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          reqd_free_pages;      /*    40     8 */
	u32                        crc32;                /*    48     4 */

	/* size: 56, cachelines: 1, members: 7 */
	/* sum members: 48, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 56 bytes */
};
struct swsusp_header {
	char                       reserved[4060];       /*     0  4060 */
	/* --- cacheline 63 boundary (4032 bytes) was 28 bytes ago --- */
	u32                        crc32;                /*  4060     4 */
	sector_t                   image;                /*  4064     8 */
	unsigned int               flags;                /*  4072     4 */
	char                       orig_sig[10];         /*  4076    10 */
	char                       sig[10];              /*  4086    10 */

	/* size: 4096, cachelines: 64, members: 6 */
};
struct swsusp_extent {
	struct rb_node             node __attribute__((__aligned__(8))); /*     0    24 */
	long unsigned int          start;                /*    24     8 */
	long unsigned int          end;                  /*    32     8 */

	/* size: 40, cachelines: 1, members: 3 */
	/* forced alignments: 1 */
	/* last cacheline: 40 bytes */
} __attribute__((__aligned__(8)));
struct hib_bio_batch {
	atomic_t                   count;                /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	wait_queue_head_t          wait;                 /*     8    24 */
	blk_status_t               error;                /*    32     1 */

	/* XXX 7 bytes hole, try to pack */

	struct blk_plug            plug;                 /*    40    40 */

	/* XXX last struct has 4 bytes of padding */

	/* size: 80, cachelines: 2, members: 4 */
	/* sum members: 69, holes: 2, sum holes: 11 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 16 bytes */
};
struct crc_data {
	struct task_struct *       thr;                  /*     0     8 */
	atomic_t                   ready;                /*     8     4 */
	atomic_t                   stop;                 /*    12     4 */
	unsigned int               run_threads;          /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	wait_queue_head_t          go;                   /*    24    24 */
	wait_queue_head_t          done;                 /*    48    24 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	u32 *                      crc32;                /*    72     8 */
	size_t *                   unc_len[3];           /*    80    24 */
	unsigned char *            unc[3];               /*   104    24 */

	/* size: 128, cachelines: 2, members: 9 */
	/* sum members: 124, holes: 1, sum holes: 4 */
};
struct cmp_data {
	struct task_struct *       thr;                  /*     0     8 */
	atomic_t                   ready;                /*     8     4 */
	atomic_t                   stop;                 /*    12     4 */
	int                        ret;                  /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	wait_queue_head_t          go;                   /*    24    24 */
	wait_queue_head_t          done;                 /*    48    24 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	size_t                     unc_len;              /*    72     8 */
	size_t                     cmp_len;              /*    80     8 */
	unsigned char              unc[131072];          /*    88 131072 */
	/* --- cacheline 2049 boundary (131136 bytes) was 24 bytes ago --- */
	unsigned char              cmp[143360];          /* 131160 143360 */
	/* --- cacheline 4289 boundary (274496 bytes) was 24 bytes ago --- */
	unsigned char              wrk[16384];           /* 274520 16384 */

	/* size: 290904, cachelines: 4546, members: 11 */
	/* sum members: 290900, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct dec_data {
	struct task_struct *       thr;                  /*     0     8 */
	atomic_t                   ready;                /*     8     4 */
	atomic_t                   stop;                 /*    12     4 */
	int                        ret;                  /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	wait_queue_head_t          go;                   /*    24    24 */
	wait_queue_head_t          done;                 /*    48    24 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	size_t                     unc_len;              /*    72     8 */
	size_t                     cmp_len;              /*    80     8 */
	unsigned char              unc[131072];          /*    88 131072 */
	/* --- cacheline 2049 boundary (131136 bytes) was 24 bytes ago --- */
	unsigned char              cmp[143360];          /* 131160 143360 */

	/* size: 274520, cachelines: 4290, members: 10 */
	/* sum members: 274516, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct resume_swap_area {
	__kernel_loff_t            offset;               /*     0     8 */
	__u32                      dev;                  /*     8     4 */

	/* size: 12, cachelines: 1, members: 2 */
	/* last cacheline: 12 bytes */
} __attribute__((__packed__));
struct snapshot_data {
	struct snapshot_handle     handle;               /*     0    24 */

	/* XXX last struct has 4 bytes of padding */

	int                        swap;                 /*    24     4 */
	int                        mode;                 /*    28     4 */
	bool                       frozen;               /*    32     1 */
	bool                       ready;                /*    33     1 */
	bool                       platform_support;     /*    34     1 */
	bool                       free_bitmaps;         /*    35     1 */
	dev_t                      dev;                  /*    36     4 */

	/* size: 40, cachelines: 1, members: 8 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 40 bytes */
};
struct compat_resume_swap_area {
	compat_loff_t              offset;               /*     0     8 */
	u32                        dev;                  /*     8     4 */

	/* size: 12, cachelines: 1, members: 2 */
	/* last cacheline: 12 bytes */
} __attribute__((__packed__));
struct sysrq_key_op {
	const void                 (*handler)(int);      /*     0     8 */
	const const char  *        help_msg;             /*     8     8 */
	const const char  *        action_msg;           /*    16     8 */
	const int                  enable_mask;          /*    24     4 */

	/* size: 32, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct dev_printk_info {
	char                       subsystem[16];        /*     0    16 */
	char                       device[48];           /*    16    48 */

	/* size: 64, cachelines: 1, members: 2 */
};
struct kmsg_dumper {
	struct list_head           list;                 /*     0    16 */
	void                       (*dump)(struct kmsg_dumper *, enum kmsg_dump_reason); /*    16     8 */
	enum kmsg_dump_reason      max_reason;           /*    24     4 */
	bool                       active;               /*    28     1 */
	bool                       registered;           /*    29     1 */

	/* XXX 2 bytes hole, try to pack */

	u32                        cur_idx;              /*    32     4 */
	u32                        next_idx;             /*    36     4 */
	u64                        cur_seq;              /*    40     8 */
	u64                        next_seq;             /*    48     8 */

	/* size: 56, cachelines: 1, members: 9 */
	/* sum members: 54, holes: 1, sum holes: 2 */
	/* last cacheline: 56 bytes */
};
struct trace_event_raw_console {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        __data_loc_msg;       /*     8     4 */
	char                       __data[];             /*    12     0 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct trace_event_data_offsets_console {
	u32                        msg;                  /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct printk_info {
	u64                        seq;                  /*     0     8 */
	u64                        ts_nsec;              /*     8     8 */
	u16                        text_len;             /*    16     2 */
	u8                         facility;             /*    18     1 */
	u8                         flags:5;              /*    19: 0  1 */
	u8                         level:3;              /*    19: 5  1 */
	u32                        caller_id;            /*    20     4 */
	struct dev_printk_info     dev_info;             /*    24    64 */

	/* size: 88, cachelines: 2, members: 8 */
	/* last cacheline: 24 bytes */
};
struct printk_record {
	struct printk_info *       info;                 /*     0     8 */
	char *                     text_buf;             /*     8     8 */
	unsigned int               text_buf_size;        /*    16     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct prb_data_blk_lpos {
	long unsigned int          begin;                /*     0     8 */
	long unsigned int          next;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct prb_desc {
	atomic_long_t              state_var;            /*     0     8 */
	struct prb_data_blk_lpos   text_blk_lpos;        /*     8    16 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct prb_data_ring {
	unsigned int               size_bits;            /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	char *                     data;                 /*     8     8 */
	atomic_long_t              head_lpos;            /*    16     8 */
	atomic_long_t              tail_lpos;            /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct prb_desc_ring {
	unsigned int               count_bits;           /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct prb_desc *          descs;                /*     8     8 */
	struct printk_info *       infos;                /*    16     8 */
	atomic_long_t              head_id;              /*    24     8 */
	atomic_long_t              tail_id;              /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* sum members: 36, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct printk_ringbuffer {
	struct prb_desc_ring       desc_ring;            /*     0    40 */
	struct prb_data_ring       text_data_ring;       /*    40    32 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	atomic_long_t              fail;                 /*    72     8 */

	/* size: 80, cachelines: 2, members: 3 */
	/* last cacheline: 16 bytes */
};
struct prb_reserved_entry {
	struct printk_ringbuffer * rb;                   /*     0     8 */
	long unsigned int          irqflags;             /*     8     8 */
	long unsigned int          id;                   /*    16     8 */
	unsigned int               text_space;           /*    24     4 */

	/* size: 32, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct console_cmdline {
	char                       name[16];             /*     0    16 */
	int                        index;                /*    16     4 */
	bool                       user_specified;       /*    20     1 */

	/* XXX 3 bytes hole, try to pack */

	char *                     options;              /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* sum members: 29, holes: 1, sum holes: 3 */
	/* last cacheline: 32 bytes */
};
struct devkmsg_user {
	u64                        seq;                  /*     0     8 */
	struct ratelimit_state     rs;                   /*     8    40 */
	struct mutex               lock;                 /*    48    32 */
	/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
	char                       buf[8192];            /*    80  8192 */
	/* --- cacheline 129 boundary (8256 bytes) was 16 bytes ago --- */
	struct printk_info         info;                 /*  8272    88 */
	/* --- cacheline 130 boundary (8320 bytes) was 40 bytes ago --- */
	char                       text_buf[8192];       /*  8360  8192 */
	/* --- cacheline 258 boundary (16512 bytes) was 40 bytes ago --- */
	struct printk_record       record;               /* 16552    24 */

	/* XXX last struct has 4 bytes of padding */

	/* size: 16576, cachelines: 259, members: 7 */
	/* paddings: 1, sum paddings: 4 */
};
struct printk_safe_seq_buf {
	atomic_t                   len;                  /*     0     4 */
	atomic_t                   message_lost;         /*     4     4 */
	struct irq_work            work;                 /*     8    24 */
	unsigned char              buffer[8160];         /*    32  8160 */

	/* size: 8192, cachelines: 128, members: 4 */
};
struct prb_data_block {
	long unsigned int          id;                   /*     0     8 */
	char                       data[];               /*     8     0 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct irq_devres {
	unsigned int               irq;                  /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     dev_id;               /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct irq_desc_devres {
	unsigned int               from;                 /*     0     4 */
	unsigned int               cnt;                  /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct irqchip_fwid {
	struct fwnode_handle       fwnode;               /*     0    40 */
	unsigned int               type;                 /*    40     4 */

	/* XXX 4 bytes hole, try to pack */

	char *                     name;                 /*    48     8 */
	phys_addr_t *              pa;                   /*    56     8 */

	/* size: 64, cachelines: 1, members: 4 */
	/* sum members: 60, holes: 1, sum holes: 4 */
};
struct irq_affinity {
	unsigned int               pre_vectors;          /*     0     4 */
	unsigned int               post_vectors;         /*     4     4 */
	unsigned int               nr_sets;              /*     8     4 */
	unsigned int               set_size[4];          /*    12    16 */

	/* XXX 4 bytes hole, try to pack */

	void                       (*calc_sets)(struct irq_affinity *, unsigned int); /*    32     8 */
	void *                     priv;                 /*    40     8 */
	u64                        kabi_reserved1;       /*    48     8 */

	/* size: 56, cachelines: 1, members: 7 */
	/* sum members: 52, holes: 1, sum holes: 4 */
	/* last cacheline: 56 bytes */
};
struct node_vectors {
	unsigned int               id;                   /*     0     4 */
	union {
		unsigned int       nvectors;             /*     4     4 */
		unsigned int       ncpus;                /*     4     4 */
	};                                               /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct cpumap {
	unsigned int               available;            /*     0     4 */
	unsigned int               allocated;            /*     4     4 */
	unsigned int               managed;              /*     8     4 */
	unsigned int               managed_allocated;    /*    12     4 */
	bool                       initialized;          /*    16     1 */
	bool                       online;               /*    17     1 */

	/* XXX 6 bytes hole, try to pack */

	long unsigned int          alloc_map[4];         /*    24    32 */
	long unsigned int          managed_map[4];       /*    56    32 */

	/* size: 88, cachelines: 2, members: 8 */
	/* sum members: 82, holes: 1, sum holes: 6 */
	/* last cacheline: 24 bytes */
};
struct irq_matrix {
	unsigned int               matrix_bits;          /*     0     4 */
	unsigned int               alloc_start;          /*     4     4 */
	unsigned int               alloc_end;            /*     8     4 */
	unsigned int               alloc_size;           /*    12     4 */
	unsigned int               global_available;     /*    16     4 */
	unsigned int               global_reserved;      /*    20     4 */
	unsigned int               systembits_inalloc;   /*    24     4 */
	unsigned int               total_allocated;      /*    28     4 */
	unsigned int               online_maps;          /*    32     4 */

	/* XXX 4 bytes hole, try to pack */

	struct cpumap *            maps;                 /*    40     8 */
	long unsigned int          scratch_map[4];       /*    48    32 */
	/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
	long unsigned int          system_map[4];        /*    80    32 */

	/* size: 112, cachelines: 2, members: 12 */
	/* sum members: 108, holes: 1, sum holes: 4 */
	/* last cacheline: 48 bytes */
};
struct trace_event_raw_irq_matrix_global {
	struct trace_entry         ent;                  /*     0     8 */
	unsigned int               online_maps;          /*     8     4 */
	unsigned int               global_available;     /*    12     4 */
	unsigned int               global_reserved;      /*    16     4 */
	unsigned int               total_allocated;      /*    20     4 */
	char                       __data[];             /*    24     0 */

	/* size: 24, cachelines: 1, members: 6 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_irq_matrix_global_update {
	struct trace_entry         ent;                  /*     0     8 */
	int                        bit;                  /*     8     4 */
	unsigned int               online_maps;          /*    12     4 */
	unsigned int               global_available;     /*    16     4 */
	unsigned int               global_reserved;      /*    20     4 */
	unsigned int               total_allocated;      /*    24     4 */
	char                       __data[];             /*    28     0 */

	/* size: 28, cachelines: 1, members: 7 */
	/* last cacheline: 28 bytes */
};
struct trace_event_raw_irq_matrix_cpu {
	struct trace_entry         ent;                  /*     0     8 */
	int                        bit;                  /*     8     4 */
	unsigned int               cpu;                  /*    12     4 */
	bool                       online;               /*    16     1 */

	/* XXX 3 bytes hole, try to pack */

	unsigned int               available;            /*    20     4 */
	unsigned int               allocated;            /*    24     4 */
	unsigned int               managed;              /*    28     4 */
	unsigned int               online_maps;          /*    32     4 */
	unsigned int               global_available;     /*    36     4 */
	unsigned int               global_reserved;      /*    40     4 */
	unsigned int               total_allocated;      /*    44     4 */
	char                       __data[];             /*    48     0 */

	/* size: 48, cachelines: 1, members: 12 */
	/* sum members: 45, holes: 1, sum holes: 3 */
	/* last cacheline: 48 bytes */
};
struct trace_event_data_offsets_irq_matrix_global {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_irq_matrix_global_update {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_irq_matrix_cpu {

	/* size: 0, cachelines: 0, members: 0 */
};
struct rcu_synchronize {
	struct callback_head       head __attribute__((__aligned__(8))); /*     0    16 */
	struct completion          completion;           /*    16    32 */

	/* size: 48, cachelines: 1, members: 2 */
	/* forced alignments: 1 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct trace_event_raw_rcu_utilization {
	struct trace_entry         ent;                  /*     0     8 */
	const char  *              s;                    /*     8     8 */
	char                       __data[];             /*    16     0 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct trace_event_data_offsets_rcu_utilization {

	/* size: 0, cachelines: 0, members: 0 */
};
struct rcu_tasks {
	struct callback_head *     cbs_head;             /*     0     8 */
	struct callback_head * *   cbs_tail;             /*     8     8 */
	struct wait_queue_head     cbs_wq;               /*    16    24 */
	raw_spinlock_t             cbs_lock;             /*    40     4 */
	int                        gp_state;             /*    44     4 */
	int                        gp_sleep;             /*    48     4 */
	int                        init_fract;           /*    52     4 */
	long unsigned int          gp_jiffies;           /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	long unsigned int          gp_start;             /*    64     8 */
	long unsigned int          n_gps;                /*    72     8 */
	long unsigned int          n_ipis;               /*    80     8 */
	long unsigned int          n_ipis_fails;         /*    88     8 */
	struct task_struct *       kthread_ptr;          /*    96     8 */
	rcu_tasks_gp_func_t        gp_func;              /*   104     8 */
	pregp_func_t               pregp_func;           /*   112     8 */
	pertask_func_t             pertask_func;         /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	postscan_func_t            postscan_func;        /*   128     8 */
	holdouts_func_t            holdouts_func;        /*   136     8 */
	postgp_func_t              postgp_func;          /*   144     8 */
	call_rcu_func_t            call_func;            /*   152     8 */
	char *                     name;                 /*   160     8 */
	char *                     kname;                /*   168     8 */

	/* size: 176, cachelines: 3, members: 22 */
	/* last cacheline: 48 bytes */
};
struct rcu_cblist {
	struct callback_head *     head;                 /*     0     8 */
	struct callback_head * *   tail;                 /*     8     8 */
	long int                   len;                  /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct rcu_exp_work {
	long unsigned int          rew_s;                /*     0     8 */
	struct work_struct         rew_work;             /*     8    64 */

	/* size: 72, cachelines: 2, members: 2 */
	/* last cacheline: 8 bytes */
};
struct rcu_node {
	raw_spinlock_t             lock;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          gp_seq;               /*     8     8 */
	long unsigned int          gp_seq_needed;        /*    16     8 */
	long unsigned int          completedqs;          /*    24     8 */
	long unsigned int          qsmask;               /*    32     8 */
	long unsigned int          rcu_gp_init_mask;     /*    40     8 */
	long unsigned int          qsmaskinit;           /*    48     8 */
	long unsigned int          qsmaskinitnext;       /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	long unsigned int          expmask;              /*    64     8 */
	long unsigned int          expmaskinit;          /*    72     8 */
	long unsigned int          expmaskinitnext;      /*    80     8 */
	long unsigned int          cbovldmask;           /*    88     8 */
	long unsigned int          ffmask;               /*    96     8 */
	long unsigned int          grpmask;              /*   104     8 */
	int                        grplo;                /*   112     4 */
	int                        grphi;                /*   116     4 */
	u8                         grpnum;               /*   120     1 */
	u8                         level;                /*   121     1 */
	bool                       wait_blkd_tasks;      /*   122     1 */

	/* XXX 5 bytes hole, try to pack */

	/* --- cacheline 2 boundary (128 bytes) --- */
	struct rcu_node *          parent;               /*   128     8 */
	struct list_head           blkd_tasks;           /*   136    16 */
	struct list_head *         gp_tasks;             /*   152     8 */
	struct list_head *         exp_tasks;            /*   160     8 */
	struct list_head *         boost_tasks;          /*   168     8 */
	struct rt_mutex            boost_mtx;            /*   176    32 */
	/* --- cacheline 3 boundary (192 bytes) was 16 bytes ago --- */
	long unsigned int          boost_time;           /*   208     8 */
	struct task_struct *       boost_kthread_task;   /*   216     8 */
	unsigned int               boost_kthread_status; /*   224     4 */

	/* XXX 4 bytes hole, try to pack */

	struct swait_queue_head    nocb_gp_wq[2];        /*   232    48 */

	/* XXX 40 bytes hole, try to pack */

	/* --- cacheline 5 boundary (320 bytes) --- */
	raw_spinlock_t             fqslock __attribute__((__aligned__(64))); /*   320     4 */

	/* XXX 60 bytes hole, try to pack */

	/* --- cacheline 6 boundary (384 bytes) --- */
	spinlock_t                 exp_lock __attribute__((__aligned__(64))); /*   384     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          exp_seq_rq;           /*   392     8 */
	wait_queue_head_t          exp_wq[4];            /*   400    96 */
	/* --- cacheline 7 boundary (448 bytes) was 48 bytes ago --- */
	struct rcu_exp_work        rew;                  /*   496    72 */
	/* --- cacheline 8 boundary (512 bytes) was 56 bytes ago --- */
	bool                       exp_need_flush;       /*   568     1 */

	/* size: 576, cachelines: 9, members: 35 */
	/* sum members: 452, holes: 6, sum holes: 117 */
	/* padding: 7 */
	/* forced alignments: 2, forced holes: 2, sum forced holes: 100 */
} __attribute__((__aligned__(64)));
union rcu_noqs {
	struct {
		u8                 norm;               /*     0     1 */
		u8                 exp;                /*     1     1 */
	} b;                                           /*     0     2 */
	u16                        s;                  /*     0     2 */
};
struct rcu_data {
	long unsigned int          gp_seq;               /*     0     8 */
	long unsigned int          gp_seq_needed;        /*     8     8 */
	union rcu_noqs             cpu_no_qs;            /*    16     2 */
	bool                       core_needs_qs;        /*    18     1 */
	bool                       beenonline;           /*    19     1 */
	bool                       gpwrap;               /*    20     1 */
	bool                       exp_deferred_qs;      /*    21     1 */
	bool                       cpu_started;          /*    22     1 */

	/* XXX 1 byte hole, try to pack */

	struct rcu_node *          mynode;               /*    24     8 */
	long unsigned int          grpmask;              /*    32     8 */
	long unsigned int          ticks_this_gp;        /*    40     8 */
	struct irq_work            defer_qs_iw;          /*    48    24 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	bool                       defer_qs_iw_pending;  /*    72     1 */

	/* XXX 7 bytes hole, try to pack */

	struct work_struct         strict_work;          /*    80    64 */
	/* --- cacheline 2 boundary (128 bytes) was 16 bytes ago --- */
	struct rcu_segcblist       cblist;               /*   144    88 */

	/* XXX last struct has 6 bytes of padding */

	/* --- cacheline 3 boundary (192 bytes) was 40 bytes ago --- */
	long int                   qlen_last_fqs_check;  /*   232     8 */
	long unsigned int          n_cbs_invoked;        /*   240     8 */
	long unsigned int          n_force_qs_snap;      /*   248     8 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	long int                   blimit;               /*   256     8 */
	int                        dynticks_snap;        /*   264     4 */

	/* XXX 4 bytes hole, try to pack */

	long int                   dynticks_nesting;     /*   272     8 */
	long int                   dynticks_nmi_nesting; /*   280     8 */
	atomic_t                   dynticks;             /*   288     4 */
	bool                       rcu_need_heavy_qs;    /*   292     1 */
	bool                       rcu_urgent_qs;        /*   293     1 */
	bool                       rcu_forced_tick;      /*   294     1 */
	bool                       rcu_forced_tick_exp;  /*   295     1 */
	struct callback_head       barrier_head __attribute__((__aligned__(8))); /*   296    16 */
	int                        exp_dynticks_snap;    /*   312     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 5 boundary (320 bytes) --- */
	struct swait_queue_head    nocb_cb_wq;           /*   320    24 */
	struct task_struct *       nocb_gp_kthread;      /*   344     8 */
	raw_spinlock_t             nocb_lock;            /*   352     4 */
	atomic_t                   nocb_lock_contended;  /*   356     4 */
	int                        nocb_defer_wakeup;    /*   360     4 */

	/* XXX 4 bytes hole, try to pack */

	struct timer_list          nocb_timer;           /*   368    72 */
	/* --- cacheline 6 boundary (384 bytes) was 56 bytes ago --- */
	long unsigned int          nocb_gp_adv_time;     /*   440     8 */
	/* --- cacheline 7 boundary (448 bytes) --- */
	raw_spinlock_t             nocb_bypass_lock __attribute__((__aligned__(64))); /*   448     4 */

	/* XXX 4 bytes hole, try to pack */

	struct rcu_cblist          nocb_bypass;          /*   456    24 */
	long unsigned int          nocb_bypass_first;    /*   480     8 */
	long unsigned int          nocb_nobypass_last;   /*   488     8 */
	int                        nocb_nobypass_count;  /*   496     4 */

	/* XXX 12 bytes hole, try to pack */

	/* --- cacheline 8 boundary (512 bytes) --- */
	raw_spinlock_t             nocb_gp_lock __attribute__((__aligned__(64))); /*   512     4 */

	/* XXX 4 bytes hole, try to pack */

	struct timer_list          nocb_bypass_timer;    /*   520    72 */
	/* --- cacheline 9 boundary (576 bytes) was 16 bytes ago --- */
	u8                         nocb_gp_sleep;        /*   592     1 */
	u8                         nocb_gp_bypass;       /*   593     1 */
	u8                         nocb_gp_gp;           /*   594     1 */

	/* XXX 5 bytes hole, try to pack */

	long unsigned int          nocb_gp_seq;          /*   600     8 */
	long unsigned int          nocb_gp_loops;        /*   608     8 */
	struct swait_queue_head    nocb_gp_wq;           /*   616    24 */
	/* --- cacheline 10 boundary (640 bytes) --- */
	bool                       nocb_cb_sleep;        /*   640     1 */

	/* XXX 7 bytes hole, try to pack */

	struct task_struct *       nocb_cb_kthread;      /*   648     8 */
	struct rcu_data *          nocb_next_cb_rdp;     /*   656     8 */

	/* XXX 40 bytes hole, try to pack */

	/* --- cacheline 11 boundary (704 bytes) --- */
	struct rcu_data *          nocb_gp_rdp __attribute__((__aligned__(64))); /*   704     8 */
	struct task_struct *       rcu_cpu_kthread_task; /*   712     8 */
	unsigned int               rcu_cpu_kthread_status; /*   720     4 */
	char                       rcu_cpu_has_work;     /*   724     1 */

	/* XXX 3 bytes hole, try to pack */

	unsigned int               softirq_snap;         /*   728     4 */

	/* XXX 4 bytes hole, try to pack */

	struct irq_work            rcu_iw;               /*   736    24 */
	bool                       rcu_iw_pending;       /*   760     1 */

	/* XXX 7 bytes hole, try to pack */

	/* --- cacheline 12 boundary (768 bytes) --- */
	long unsigned int          rcu_iw_gp_seq;        /*   768     8 */
	long unsigned int          rcu_ofl_gp_seq;       /*   776     8 */
	short int                  rcu_ofl_gp_flags;     /*   784     2 */

	/* XXX 6 bytes hole, try to pack */

	long unsigned int          rcu_onl_gp_seq;       /*   792     8 */
	short int                  rcu_onl_gp_flags;     /*   800     2 */

	/* XXX 6 bytes hole, try to pack */

	long unsigned int          last_fqs_resched;     /*   808     8 */
	int                        cpu;                  /*   816     4 */

	/* size: 832, cachelines: 13, members: 66 */
	/* sum members: 702, holes: 16, sum holes: 118 */
	/* padding: 12 */
	/* paddings: 1, sum paddings: 6 */
	/* forced alignments: 4, forced holes: 2, sum forced holes: 52 */
} __attribute__((__aligned__(64)));
struct rcu_state {
	struct rcu_node            node[521] __attribute__((__aligned__(64))); /*     0 300096 */
	/* --- cacheline 4689 boundary (300096 bytes) --- */
	struct rcu_node *          level[4];             /* 300096    32 */
	int                        ncpus;                /* 300128     4 */

	/* XXX 28 bytes hole, try to pack */

	/* --- cacheline 4690 boundary (300160 bytes) --- */
	u8                         boost __attribute__((__aligned__(64))); /* 300160     1 */

	/* XXX 7 bytes hole, try to pack */

	long unsigned int          gp_seq;               /* 300168     8 */
	long unsigned int          gp_max;               /* 300176     8 */
	struct task_struct *       gp_kthread;           /* 300184     8 */
	struct swait_queue_head    gp_wq;                /* 300192    24 */
	short int                  gp_flags;             /* 300216     2 */
	short int                  gp_state;             /* 300218     2 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 4691 boundary (300224 bytes) --- */
	long unsigned int          gp_wake_time;         /* 300224     8 */
	long unsigned int          gp_wake_seq;          /* 300232     8 */
	struct mutex               barrier_mutex;        /* 300240    32 */
	atomic_t                   barrier_cpu_count;    /* 300272     4 */

	/* XXX 4 bytes hole, try to pack */

	struct completion          barrier_completion;   /* 300280    32 */
	/* --- cacheline 4692 boundary (300288 bytes) was 24 bytes ago --- */
	long unsigned int          barrier_sequence;     /* 300312     8 */
	struct mutex               exp_mutex;            /* 300320    32 */
	/* --- cacheline 4693 boundary (300352 bytes) --- */
	struct mutex               exp_wake_mutex;       /* 300352    32 */
	long unsigned int          expedited_sequence;   /* 300384     8 */
	atomic_t                   expedited_need_qs;    /* 300392     4 */

	/* XXX 4 bytes hole, try to pack */

	struct swait_queue_head    expedited_wq;         /* 300400    24 */
	/* --- cacheline 4694 boundary (300416 bytes) was 8 bytes ago --- */
	int                        ncpus_snap;           /* 300424     4 */
	u8                         cbovld;               /* 300428     1 */
	u8                         cbovldnext;           /* 300429     1 */

	/* XXX 2 bytes hole, try to pack */

	long unsigned int          jiffies_force_qs;     /* 300432     8 */
	long unsigned int          jiffies_kick_kthreads; /* 300440     8 */
	long unsigned int          n_force_qs;           /* 300448     8 */
	long unsigned int          gp_start;             /* 300456     8 */
	long unsigned int          gp_end;               /* 300464     8 */
	long unsigned int          gp_activity;          /* 300472     8 */
	/* --- cacheline 4695 boundary (300480 bytes) --- */
	long unsigned int          gp_req_activity;      /* 300480     8 */
	long unsigned int          jiffies_stall;        /* 300488     8 */
	long unsigned int          jiffies_resched;      /* 300496     8 */
	long unsigned int          n_force_qs_gpstart;   /* 300504     8 */
	const char  *              name;                 /* 300512     8 */
	char                       abbr;                 /* 300520     1 */

	/* XXX 23 bytes hole, try to pack */

	/* --- cacheline 4696 boundary (300544 bytes) --- */
	raw_spinlock_t             ofl_lock __attribute__((__aligned__(64))); /* 300544     4 */

	/* size: 300608, cachelines: 4697, members: 37 */
	/* sum members: 300476, holes: 7, sum holes: 72 */
	/* padding: 60 */
	/* forced alignments: 3, forced holes: 2, sum forced holes: 51 */
} __attribute__((__aligned__(64)));
struct kvfree_rcu_bulk_data {
	long unsigned int          nr_records;           /*     0     8 */
	struct kvfree_rcu_bulk_data * next;              /*     8     8 */
	void *                     records[];            /*    16     0 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct kfree_rcu_cpu_work {
	struct rcu_work            rcu_work __attribute__((__aligned__(8))); /*     0    88 */
	/* --- cacheline 1 boundary (64 bytes) was 24 bytes ago --- */
	struct callback_head *     head_free;            /*    88     8 */
	struct kvfree_rcu_bulk_data * bkvhead_free[2];   /*    96    16 */
	struct kfree_rcu_cpu *     krcp;                 /*   112     8 */

	/* size: 120, cachelines: 2, members: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 56 bytes */
} __attribute__((__aligned__(8)));
struct kfree_rcu_cpu {
	struct callback_head *     head;                 /*     0     8 */
	struct kvfree_rcu_bulk_data * bkvhead[2];        /*     8    16 */
	struct kfree_rcu_cpu_work  krw_arr[2] __attribute__((__aligned__(8))); /*    24   240 */
	/* --- cacheline 4 boundary (256 bytes) was 8 bytes ago --- */
	raw_spinlock_t             lock;                 /*   264     4 */

	/* XXX 4 bytes hole, try to pack */

	struct delayed_work        monitor_work;         /*   272   192 */
	/* --- cacheline 7 boundary (448 bytes) was 16 bytes ago --- */
	bool                       monitor_todo;         /*   464     1 */
	bool                       initialized;          /*   465     1 */

	/* XXX 2 bytes hole, try to pack */

	int                        count;                /*   468     4 */
	struct work_struct         page_cache_work;      /*   472    64 */
	/* --- cacheline 8 boundary (512 bytes) was 24 bytes ago --- */
	atomic_t                   work_in_progress;     /*   536     4 */

	/* XXX 4 bytes hole, try to pack */

	struct hrtimer             hrtimer __attribute__((__aligned__(8))); /*   544    96 */
	/* --- cacheline 10 boundary (640 bytes) --- */
	struct llist_head          bkvcache;             /*   640     8 */
	int                        nr_bkv_objs;          /*   648     4 */

	/* size: 656, cachelines: 11, members: 13 */
	/* sum members: 642, holes: 3, sum holes: 10 */
	/* padding: 4 */
	/* forced alignments: 2, forced holes: 1, sum forced holes: 4 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct patch_data {
	struct klp_patch *         patch;                /*     0     8 */
	atomic_t                   cpu_count;            /*     8     4 */
	bool                       rollback;             /*    12     1 */

	/* size: 16, cachelines: 1, members: 3 */
	/* padding: 3 */
	/* last cacheline: 16 bytes */
};
struct klp_find_arg {
	const char  *              objname;              /*     0     8 */
	const char  *              name;                 /*     8     8 */
	long unsigned int          addr;                 /*    16     8 */
	long unsigned int          count;                /*    24     8 */
	long unsigned int          pos;                  /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct dma_devres {
	size_t                     size;                 /*     0     8 */
	void *                     vaddr;                /*     8     8 */
	dma_addr_t                 dma_handle;           /*    16     8 */
	long unsigned int          attrs;                /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct trace_event_raw_swiotlb_bounced {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        __data_loc_dev_name;  /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        dma_mask;             /*    16     8 */
	dma_addr_t                 dev_addr;             /*    24     8 */
	size_t                     size;                 /*    32     8 */
	enum swiotlb_force         swiotlb_force;        /*    40     4 */
	char                       __data[];             /*    44     0 */

	/* size: 48, cachelines: 1, members: 7 */
	/* sum members: 40, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 48 bytes */
};
struct trace_event_data_offsets_swiotlb_bounced {
	u32                        dev_name;             /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct trace_event_raw_sys_enter {
	struct trace_entry         ent;                  /*     0     8 */
	long int                   id;                   /*     8     8 */
	long unsigned int          args[6];              /*    16    48 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	char                       __data[];             /*    64     0 */

	/* size: 64, cachelines: 1, members: 4 */
};
struct trace_event_raw_sys_exit {
	struct trace_entry         ent;                  /*     0     8 */
	long int                   id;                   /*     8     8 */
	long int                   ret;                  /*    16     8 */
	char                       __data[];             /*    24     0 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct trace_event_data_offsets_sys_enter {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_sys_exit {

	/* size: 0, cachelines: 0, members: 0 */
};
struct kvm {
	spinlock_t                 mmu_lock;             /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct mutex               slots_lock;           /*     8    32 */
	struct mm_struct *         mm;                   /*    40     8 */
	struct kvm_memslots *      memslots[2];          /*    48    16 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct kvm_vcpu *          vcpus[288];           /*    64  2304 */
	/* --- cacheline 37 boundary (2368 bytes) --- */
	atomic_t                   online_vcpus;         /*  2368     4 */
	int                        created_vcpus;        /*  2372     4 */
	int                        last_boosted_vcpu;    /*  2376     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           vm_list;              /*  2384    16 */
	struct mutex               lock;                 /*  2400    32 */
	/* --- cacheline 38 boundary (2432 bytes) --- */
	struct kvm_io_bus *        buses[4];             /*  2432    32 */
	struct {
		spinlock_t         lock;                 /*  2464     4 */

		/* XXX 4 bytes hole, try to pack */

		struct list_head   items;                /*  2472    16 */
		struct list_head   resampler_list;       /*  2488    16 */
		/* --- cacheline 39 boundary (2496 bytes) was 8 bytes ago --- */
		struct mutex       resampler_lock;       /*  2504    32 */
	} irqfds;                                        /*  2464    72 */
	struct list_head           ioeventfds;           /*  2536    16 */
	struct kvm_vm_stat         stat;                 /*  2552    88 */
	/* --- cacheline 41 boundary (2624 bytes) was 16 bytes ago --- */
	struct kvm_arch            arch;                 /*  2640 84400 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 1360 boundary (87040 bytes) --- */
	refcount_t                 users_count;          /* 87040     4 */

	/* XXX 4 bytes hole, try to pack */

	struct kvm_coalesced_mmio_ring * coalesced_mmio_ring; /* 87048     8 */
	spinlock_t                 ring_lock;            /* 87056     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           coalesced_zones;      /* 87064    16 */
	struct mutex               irq_lock;             /* 87080    32 */
	/* --- cacheline 1361 boundary (87104 bytes) was 8 bytes ago --- */
	struct kvm_irq_routing_table * irq_routing;      /* 87112     8 */
	struct hlist_head          irq_ack_notifier_list; /* 87120     8 */
	struct mmu_notifier        mmu_notifier __attribute__((__aligned__(8))); /* 87128    72 */
	/* --- cacheline 1362 boundary (87168 bytes) was 32 bytes ago --- */
	long unsigned int          mmu_notifier_seq;     /* 87200     8 */
	long int                   mmu_notifier_count;   /* 87208     8 */
	long int                   tlbs_dirty;           /* 87216     8 */
	struct list_head           devices;              /* 87224    16 */
	/* --- cacheline 1363 boundary (87232 bytes) was 8 bytes ago --- */
	u64                        manual_dirty_log_protect; /* 87240     8 */
	struct dentry *            debugfs_dentry;       /* 87248     8 */
	struct kvm_stat_data * *   debugfs_stat_data;    /* 87256     8 */
	struct srcu_struct         srcu;                 /* 87264 50440 */
	/* --- cacheline 2151 boundary (137664 bytes) was 40 bytes ago --- */
	struct srcu_struct         irq_srcu;             /* 137704 50440 */
	/* --- cacheline 2939 boundary (188096 bytes) was 48 bytes ago --- */
	pid_t                      userspace_pid;        /* 188144     4 */
	unsigned int               max_halt_poll_ns;     /* 188148     4 */
	bool                       vm_bugged;            /* 188152     1 */

	/* size: 188160, cachelines: 2940, members: 35 */
	/* sum members: 188137, holes: 4, sum holes: 16 */
	/* padding: 7 */
	/* paddings: 1, sum paddings: 4 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(8)));
struct kvm_regs {
	__u64                      rax;                  /*     0     8 */
	__u64                      rbx;                  /*     8     8 */
	__u64                      rcx;                  /*    16     8 */
	__u64                      rdx;                  /*    24     8 */
	__u64                      rsi;                  /*    32     8 */
	__u64                      rdi;                  /*    40     8 */
	__u64                      rsp;                  /*    48     8 */
	__u64                      rbp;                  /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	__u64                      r8;                   /*    64     8 */
	__u64                      r9;                   /*    72     8 */
	__u64                      r10;                  /*    80     8 */
	__u64                      r11;                  /*    88     8 */
	__u64                      r12;                  /*    96     8 */
	__u64                      r13;                  /*   104     8 */
	__u64                      r14;                  /*   112     8 */
	__u64                      r15;                  /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	__u64                      rip;                  /*   128     8 */
	__u64                      rflags;               /*   136     8 */

	/* size: 144, cachelines: 3, members: 18 */
	/* last cacheline: 16 bytes */
};
struct kvm_segment {
	__u64                      base;                 /*     0     8 */
	__u32                      limit;                /*     8     4 */
	__u16                      selector;             /*    12     2 */
	__u8                       type;                 /*    14     1 */
	__u8                       present;              /*    15     1 */
	__u8                       dpl;                  /*    16     1 */
	__u8                       db;                   /*    17     1 */
	__u8                       s;                    /*    18     1 */
	__u8                       l;                    /*    19     1 */
	__u8                       g;                    /*    20     1 */
	__u8                       avl;                  /*    21     1 */
	__u8                       unusable;             /*    22     1 */
	__u8                       padding;              /*    23     1 */

	/* size: 24, cachelines: 1, members: 13 */
	/* last cacheline: 24 bytes */
};
struct kvm_dtable {
	__u64                      base;                 /*     0     8 */
	__u16                      limit;                /*     8     2 */
	__u16                      padding[3];           /*    10     6 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct kvm_sregs {
	struct kvm_segment         cs;                   /*     0    24 */
	struct kvm_segment         ds;                   /*    24    24 */
	struct kvm_segment         es;                   /*    48    24 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	struct kvm_segment         fs;                   /*    72    24 */
	struct kvm_segment         gs;                   /*    96    24 */
	struct kvm_segment         ss;                   /*   120    24 */
	/* --- cacheline 2 boundary (128 bytes) was 16 bytes ago --- */
	struct kvm_segment         tr;                   /*   144    24 */
	struct kvm_segment         ldt;                  /*   168    24 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	struct kvm_dtable          gdt;                  /*   192    16 */
	struct kvm_dtable          idt;                  /*   208    16 */
	__u64                      cr0;                  /*   224     8 */
	__u64                      cr2;                  /*   232     8 */
	__u64                      cr3;                  /*   240     8 */
	__u64                      cr4;                  /*   248     8 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	__u64                      cr8;                  /*   256     8 */
	__u64                      efer;                 /*   264     8 */
	__u64                      apic_base;            /*   272     8 */
	__u64                      interrupt_bitmap[4];  /*   280    32 */

	/* size: 312, cachelines: 5, members: 18 */
	/* last cacheline: 56 bytes */
};
struct kvm_cpuid_entry2 {
	__u32                      function;             /*     0     4 */
	__u32                      index;                /*     4     4 */
	__u32                      flags;                /*     8     4 */
	__u32                      eax;                  /*    12     4 */
	__u32                      ebx;                  /*    16     4 */
	__u32                      ecx;                  /*    20     4 */
	__u32                      edx;                  /*    24     4 */
	__u32                      padding[3];           /*    28    12 */

	/* size: 40, cachelines: 1, members: 8 */
	/* last cacheline: 40 bytes */
};
struct kvm_debug_exit_arch {
	__u32                      exception;            /*     0     4 */
	__u32                      pad;                  /*     4     4 */
	__u64                      pc;                   /*     8     8 */
	__u64                      dr6;                  /*    16     8 */
	__u64                      dr7;                  /*    24     8 */

	/* size: 32, cachelines: 1, members: 5 */
	/* last cacheline: 32 bytes */
};
struct kvm_vcpu_events {
	struct {
		__u8               injected;             /*     0     1 */
		__u8               nr;                   /*     1     1 */
		__u8               has_error_code;       /*     2     1 */
		__u8               pending;              /*     3     1 */
		__u32              error_code;           /*     4     4 */
	} exception;                                     /*     0     8 */
	struct {
		__u8               injected;             /*     8     1 */
		__u8               nr;                   /*     9     1 */
		__u8               soft;                 /*    10     1 */
		__u8               shadow;               /*    11     1 */
	} interrupt;                                     /*     8     4 */
	struct {
		__u8               injected;             /*    12     1 */
		__u8               pending;              /*    13     1 */
		__u8               masked;               /*    14     1 */
		__u8               pad;                  /*    15     1 */
	} nmi;                                           /*    12     4 */
	__u32                      sipi_vector;          /*    16     4 */
	__u32                      flags;                /*    20     4 */
	struct {
		__u8               smm;                  /*    24     1 */
		__u8               pending;              /*    25     1 */
		__u8               smm_inside_nmi;       /*    26     1 */
		__u8               latched_init;         /*    27     1 */
	} smi;                                           /*    24     4 */
	struct {
		__u8               pending;              /*    28     1 */
	} triple_fault;                                  /*    28     1 */
	__u8                       reserved[26];         /*    29    26 */
	__u8                       exception_has_payload; /*    55     1 */
	__u64                      exception_payload;    /*    56     8 */

	/* size: 64, cachelines: 1, members: 10 */
};
struct kvm_sync_regs {
	struct kvm_regs            regs;                 /*     0   144 */
	/* --- cacheline 2 boundary (128 bytes) was 16 bytes ago --- */
	struct kvm_sregs           sregs;                /*   144   312 */
	/* --- cacheline 7 boundary (448 bytes) was 8 bytes ago --- */
	struct kvm_vcpu_events     events;               /*   456    64 */

	/* size: 520, cachelines: 9, members: 3 */
	/* last cacheline: 8 bytes */
};
struct kvm_pmu_event_filter {
	__u32                      action;               /*     0     4 */
	__u32                      nevents;              /*     4     4 */
	__u32                      fixed_counter_bitmap; /*     8     4 */
	__u32                      flags;                /*    12     4 */
	__u32                      pad[4];               /*    16    16 */
	__u64                      events[];             /*    32     0 */

	/* size: 32, cachelines: 1, members: 6 */
	/* last cacheline: 32 bytes */
};
struct kvm_hyperv_exit {
	__u32                      type;                 /*     0     4 */
	__u32                      pad1;                 /*     4     4 */
	union {
		struct {
			__u32      msr;                  /*     8     4 */
			__u32      pad2;                 /*    12     4 */
			__u64      control;              /*    16     8 */
			__u64      evt_page;             /*    24     8 */
			__u64      msg_page;             /*    32     8 */
		} synic;                                 /*     8    32 */
		struct {
			__u64      input;                /*     8     8 */
			__u64      result;               /*    16     8 */
			__u64      params[2];            /*    24    16 */
		} hcall;                                 /*     8    32 */
		struct {
			__u32      msr;                  /*     8     4 */
			__u32      pad2;                 /*    12     4 */
			__u64      control;              /*    16     8 */
			__u64      status;               /*    24     8 */
			__u64      send_page;            /*    32     8 */
			__u64      recv_page;            /*    40     8 */
			__u64      pending_page;         /*    48     8 */
		} syndbg;                                /*     8    48 */
	} u;                                             /*     8    48 */

	/* size: 56, cachelines: 1, members: 3 */
	/* last cacheline: 56 bytes */
};
struct kvm_run {
	__u8                       request_interrupt_window; /*     0     1 */
	__u8                       immediate_exit;       /*     1     1 */
	__u8                       padding1[6];          /*     2     6 */
	__u32                      exit_reason;          /*     8     4 */
	__u8                       ready_for_interrupt_injection; /*    12     1 */
	__u8                       if_flag;              /*    13     1 */
	__u16                      flags;                /*    14     2 */
	__u64                      cr8;                  /*    16     8 */
	__u64                      apic_base;            /*    24     8 */
	union {
		struct {
			__u64      hardware_exit_reason; /*    32     8 */
		} hw;                                    /*    32     8 */
		struct {
			__u64      hardware_entry_failure_reason; /*    32     8 */
			__u32      cpu;                  /*    40     4 */
		} fail_entry;                            /*    32    16 */
		struct {
			__u32      exception;            /*    32     4 */
			__u32      error_code;           /*    36     4 */
		} ex;                                    /*    32     8 */
		struct {
			__u8       direction;            /*    32     1 */
			__u8       size;                 /*    33     1 */
			__u16      port;                 /*    34     2 */
			__u32      count;                /*    36     4 */
			__u64      data_offset;          /*    40     8 */
		} io;                                    /*    32    16 */
		struct {
			struct kvm_debug_exit_arch arch; /*    32    32 */
		} debug;                                 /*    32    32 */
		struct {
			__u64      phys_addr;            /*    32     8 */
			__u8       data[8];              /*    40     8 */
			__u32      len;                  /*    48     4 */
			__u8       is_write;             /*    52     1 */
		} mmio;                                  /*    32    24 */
		struct {
			__u64      nr;                   /*    32     8 */
			__u64      args[6];              /*    40    48 */
			/* --- cacheline 1 boundary (64 bytes) was 24 bytes ago --- */
			__u64      ret;                  /*    88     8 */
			__u32      longmode;             /*    96     4 */
			__u32      pad;                  /*   100     4 */
		} hypercall;                             /*    32    72 */
		struct {
			__u64      rip;                  /*    32     8 */
			__u32      is_write;             /*    40     4 */
			__u32      pad;                  /*    44     4 */
		} tpr_access;                            /*    32    16 */
		struct {
			__u8       icptcode;             /*    32     1 */

			/* XXX 1 byte hole, try to pack */

			__u16      ipa;                  /*    34     2 */
			__u32      ipb;                  /*    36     4 */
		} s390_sieic;                            /*    32     8 */
		__u64              s390_reset_flags;     /*    32     8 */
		struct {
			__u64      trans_exc_code;       /*    32     8 */
			__u32      pgm_code;             /*    40     4 */
		} s390_ucontrol;                         /*    32    16 */
		struct {
			__u32      dcrn;                 /*    32     4 */
			__u32      data;                 /*    36     4 */
			__u8       is_write;             /*    40     1 */
		} dcr;                                   /*    32    12 */
		struct {
			__u32      suberror;             /*    32     4 */
			__u32      ndata;                /*    36     4 */
			__u64      data[16];             /*    40   128 */
		} internal;                              /*    32   136 */
		struct {
			__u64      gprs[32];             /*    32   256 */
		} osi;                                   /*    32   256 */
		struct {
			__u64      nr;                   /*    32     8 */
			__u64      ret;                  /*    40     8 */
			__u64      args[9];              /*    48    72 */
		} papr_hcall;                            /*    32    88 */
		struct {
			__u16      subchannel_id;        /*    32     2 */
			__u16      subchannel_nr;        /*    34     2 */
			__u32      io_int_parm;          /*    36     4 */
			__u32      io_int_word;          /*    40     4 */
			__u32      ipb;                  /*    44     4 */
			__u8       dequeued;             /*    48     1 */
		} s390_tsch;                             /*    32    20 */
		struct {
			__u32      epr;                  /*    32     4 */
		} epr;                                   /*    32     4 */
		struct {
			__u32      type;                 /*    32     4 */

			/* XXX 4 bytes hole, try to pack */

			__u64      flags;                /*    40     8 */
		} system_event;                          /*    32    16 */
		struct {
			__u64      addr;                 /*    32     8 */
			__u8       ar;                   /*    40     1 */
			__u8       reserved;             /*    41     1 */
			__u8       fc;                   /*    42     1 */
			__u8       sel1;                 /*    43     1 */
			__u16      sel2;                 /*    44     2 */
		} s390_stsi;                             /*    32    16 */
		struct {
			__u8       vector;               /*    32     1 */
		} eoi;                                   /*    32     1 */
		struct kvm_hyperv_exit hyperv;           /*    32    56 */
		struct {
			__u64      esr_iss;              /*    32     8 */
			__u64      fault_ipa;            /*    40     8 */
		} arm_nisv;                              /*    32    16 */
		struct {
			__u8       error;                /*    32     1 */
			__u8       pad[7];               /*    33     7 */
			__u32      reason;               /*    40     4 */
			__u32      index;                /*    44     4 */
			__u64      data;                 /*    48     8 */
		} msr;                                   /*    32    24 */
		struct {
			long unsigned int extension_id;  /*    32     8 */
			long unsigned int function_id;   /*    40     8 */
			long unsigned int args[6];       /*    48    48 */
			/* --- cacheline 1 boundary (64 bytes) was 32 bytes ago --- */
			long unsigned int ret[2];        /*    96    16 */
		} riscv_sbi;                             /*    32    80 */
		struct {
			__u32      flags;                /*    32     4 */
		} notify;                                /*    32     4 */
		char               padding[256];         /*    32   256 */
	};                                               /*    32   256 */
	/* --- cacheline 4 boundary (256 bytes) was 32 bytes ago --- */
	__u64                      kvm_valid_regs;       /*   288     8 */
	__u64                      kvm_dirty_regs;       /*   296     8 */
	union {
		struct kvm_sync_regs regs;               /*   304   520 */
		char               padding[2048];        /*   304  2048 */
	} s;                                             /*   304  2048 */

	/* size: 2352, cachelines: 37, members: 13 */
	/* last cacheline: 48 bytes */
};
struct kvm_coalesced_mmio {
	__u64                      phys_addr;            /*     0     8 */
	__u32                      len;                  /*     8     4 */
	union {
		__u32              pad;                  /*    12     4 */
		__u32              pio;                  /*    12     4 */
	};                                               /*    12     4 */
	__u8                       data[8];              /*    16     8 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct kvm_coalesced_mmio_ring {
	__u32                      first;                /*     0     4 */
	__u32                      last;                 /*     4     4 */
	struct kvm_coalesced_mmio  coalesced_mmio[];     /*     8     0 */

	/* size: 8, cachelines: 1, members: 3 */
	/* last cacheline: 8 bytes */
};
struct kvm_xen_hvm_config {
	__u32                      flags;                /*     0     4 */
	__u32                      msr;                  /*     4     4 */
	__u64                      blob_addr_32;         /*     8     8 */
	__u64                      blob_addr_64;         /*    16     8 */
	__u8                       blob_size_32;         /*    24     1 */
	__u8                       blob_size_64;         /*    25     1 */
	__u8                       pad2[30];             /*    26    30 */

	/* size: 56, cachelines: 1, members: 7 */
	/* last cacheline: 56 bytes */
};
struct gfn_to_hva_cache {
	u64                        generation;           /*     0     8 */
	gpa_t                      gpa;                  /*     8     8 */
	long unsigned int          hva;                  /*    16     8 */
	long unsigned int          len;                  /*    24     8 */
	struct kvm_memory_slot *   memslot;              /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct kvm_memory_slot {
	gfn_t                      base_gfn;             /*     0     8 */
	long unsigned int          npages;               /*     8     8 */
	long unsigned int *        dirty_bitmap;         /*    16     8 */
	struct kvm_arch_memory_slot arch;                /*    24    48 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	long unsigned int          userspace_addr;       /*    72     8 */
	u32                        flags;                /*    80     4 */
	short int                  id;                   /*    84     2 */
	u16                        as_id;                /*    86     2 */

	/* size: 88, cachelines: 2, members: 8 */
	/* last cacheline: 24 bytes */
};
struct gfn_to_pfn_cache {
	u64                        generation;           /*     0     8 */
	gfn_t                      gfn;                  /*     8     8 */
	kvm_pfn_t                  pfn;                  /*    16     8 */
	bool                       dirty;                /*    24     1 */

	/* size: 32, cachelines: 1, members: 4 */
	/* padding: 7 */
	/* last cacheline: 32 bytes */
};
struct kvm_mmu_memory_cache {
	int                        nobjs;                /*     0     4 */
	gfp_t                      gfp_zero;             /*     4     4 */
	struct kmem_cache *        kmem_cache;           /*     8     8 */
	void *                     objects[40];          /*    16   320 */

	/* size: 336, cachelines: 6, members: 4 */
	/* last cacheline: 16 bytes */
};
struct hv_partition_assist_pg {
	u32                        tlb_lock_count;       /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
union hv_message_flags {
	__u8                       asu8;               /*     0     1 */
	struct {
		__u8               msg_pending:1;      /*     0: 0  1 */
		__u8               reserved:7;         /*     0: 1  1 */
	};                                             /*     0     1 */
};
union hv_port_id {
	__u32                      asu32;              /*     0     4 */
	struct {
		__u32              id:24;              /*     0: 0  4 */
		__u32              reserved:8;         /*     0:24  4 */
	} u;                                           /*     0     4 */
};
struct hv_message_header {
	__u32                      message_type;         /*     0     4 */
	__u8                       payload_size;         /*     4     1 */
	union hv_message_flags     message_flags;        /*     5     1 */
	__u8                       reserved[2];          /*     6     2 */
	union {
		__u64              sender;               /*     8     8 */
		union hv_port_id   port;                 /*     8     4 */
	};                                               /*     8     8 */

	/* size: 16, cachelines: 1, members: 5 */
	/* last cacheline: 16 bytes */
};
struct hv_message {
	struct hv_message_header   header;               /*     0    16 */
	union {
		__u64              payload[30];          /*    16   240 */
	} u;                                             /*    16   240 */

	/* size: 256, cachelines: 4, members: 2 */
};
union hv_stimer_config {
	u64                        as_uint64;          /*     0     8 */
	struct {
		u64                enable:1;           /*     0: 0  8 */
		u64                periodic:1;         /*     0: 1  8 */
		u64                lazy:1;             /*     0: 2  8 */
		u64                auto_enable:1;      /*     0: 3  8 */
		u64                apic_vector:8;      /*     0: 4  8 */
		u64                direct_mode:1;      /*     0:12  8 */
		u64                reserved_z0:3;      /*     0:13  8 */
		u64                sintx:4;            /*     0:16  8 */
		u64                reserved_z1:44;     /*     0:20  8 */
	};                                             /*     0     8 */
};
struct kvm_page_track_notifier_head {
	struct srcu_struct         track_srcu;           /*     0 50440 */
	/* --- cacheline 788 boundary (50432 bytes) was 8 bytes ago --- */
	struct hlist_head          track_notifier_list;  /* 50440     8 */

	/* size: 50448, cachelines: 789, members: 2 */
	/* last cacheline: 16 bytes */
};
struct kvm_page_track_notifier_node {
	struct hlist_node          node;                 /*     0    16 */
	void                       (*track_write)(struct kvm_vcpu *, gpa_t, const u8  *, int, struct kvm_page_track_notifier_node *); /*    16     8 */
	void                       (*track_flush_slot)(struct kvm *, struct kvm_memory_slot *, struct kvm_page_track_notifier_node *); /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* last cacheline: 32 bytes */
};
struct kvm_vcpu {
	struct kvm *               kvm;                  /*     0     8 */
	struct preempt_notifier    preempt_notifier;     /*     8    24 */
	int                        cpu;                  /*    32     4 */
	int                        vcpu_id;              /*    36     4 */
	int                        vcpu_idx;             /*    40     4 */
	int                        srcu_idx;             /*    44     4 */
	int                        mode;                 /*    48     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        requests;             /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	long unsigned int          guest_debug;          /*    64     8 */
	int                        pre_pcpu;             /*    72     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           blocked_vcpu_list;    /*    80    16 */
	struct mutex               mutex;                /*    96    32 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct kvm_run *           run;                  /*   128     8 */
	struct rcuwait             wait;                 /*   136     8 */
	struct pid *               pid;                  /*   144     8 */
	int                        sigset_active;        /*   152     4 */

	/* XXX 4 bytes hole, try to pack */

	sigset_t                   sigset;               /*   160     8 */
	struct kvm_vcpu_stat       stat;                 /*   168   360 */
	/* --- cacheline 8 boundary (512 bytes) was 16 bytes ago --- */
	unsigned int               halt_poll_ns;         /*   528     4 */
	bool                       valid_wakeup;         /*   532     1 */

	/* XXX 3 bytes hole, try to pack */

	int                        mmio_needed;          /*   536     4 */
	int                        mmio_read_completed;  /*   540     4 */
	int                        mmio_is_write;        /*   544     4 */
	int                        mmio_cur_fragment;    /*   548     4 */
	int                        mmio_nr_fragments;    /*   552     4 */

	/* XXX 4 bytes hole, try to pack */

	struct kvm_mmio_fragment   mmio_fragments[2];    /*   560    48 */
	/* --- cacheline 9 boundary (576 bytes) was 32 bytes ago --- */
	struct {
		u32                queued;               /*   608     4 */

		/* XXX 4 bytes hole, try to pack */

		struct list_head   queue;                /*   616    16 */
		struct list_head   done;                 /*   632    16 */
		/* --- cacheline 10 boundary (640 bytes) was 8 bytes ago --- */
		spinlock_t         lock;                 /*   648     4 */
	} async_pf;                                      /*   608    48 */

	/* XXX last struct has 4 bytes of padding */

	struct {
		bool               in_spin_loop;         /*   656     1 */
		bool               dy_eligible;          /*   657     1 */
	} spin_loop;                                     /*   656     2 */
	bool                       preempted;            /*   658     1 */
	bool                       ready;                /*   659     1 */

	/* XXX 4 bytes hole, try to pack */

	struct kvm_vcpu_arch       arch __attribute__((__aligned__(8))); /*   664  9904 */

	/* size: 10568, cachelines: 166, members: 31 */
	/* sum members: 10545, holes: 6, sum holes: 23 */
	/* paddings: 1, sum paddings: 4 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 4 */
	/* last cacheline: 8 bytes */
} __attribute__((__aligned__(8)));
union kvm_mmu_page_role {
	u32                        word;               /*     0     4 */
	struct {
		unsigned int       level:4;            /*     0: 0  4 */
		unsigned int       gpte_is_8_bytes:1;  /*     0: 4  4 */
		unsigned int       quadrant:2;         /*     0: 5  4 */
		unsigned int       direct:1;           /*     0: 7  4 */
		unsigned int       access:3;           /*     0: 8  4 */
		unsigned int       invalid:1;          /*     0:11  4 */
		unsigned int       nxe:1;              /*     0:12  4 */
		unsigned int       cr0_wp:1;           /*     0:13  4 */
		unsigned int       smep_andnot_wp:1;   /*     0:14  4 */
		unsigned int       smap_andnot_wp:1;   /*     0:15  4 */
		unsigned int       ad_disabled:1;      /*     0:16  4 */
		unsigned int       guest_mode:1;       /*     0:17  4 */

		/* XXX 6 bits hole, try to pack */
		unsigned int       :6;

		unsigned int       smm:8;              /*     0:24  4 */
	};                                             /*     0     4 */
};
union kvm_mmu_extended_role {
	u32                        word;               /*     0     4 */
	struct {
		unsigned int       valid:1;            /*     0: 0  4 */
		unsigned int       execonly:1;         /*     0: 1  4 */
		unsigned int       cr0_pg:1;           /*     0: 2  4 */
		unsigned int       cr4_pae:1;          /*     0: 3  4 */
		unsigned int       cr4_pse:1;          /*     0: 4  4 */
		unsigned int       cr4_pke:1;          /*     0: 5  4 */
		unsigned int       cr4_smap:1;         /*     0: 6  4 */
		unsigned int       cr4_smep:1;         /*     0: 7  4 */
		unsigned int       cr4_la57:1;         /*     0: 8  4 */
		unsigned int       maxphyaddr:6;       /*     0: 9  4 */
	};                                             /*     0     4 */
};
union kvm_mmu_role {
	u64                        as_u64;             /*     0     8 */
	struct {
		union kvm_mmu_page_role base;          /*     0     4 */
		union kvm_mmu_extended_role ext;       /*     4     4 */
	};                                             /*     0     8 */
};
struct kvm_rmap_head {
	long unsigned int          val;                  /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct kvm_pio_request {
	long unsigned int          linear_rip;           /*     0     8 */
	long unsigned int          count;                /*     8     8 */
	int                        in;                   /*    16     4 */
	int                        port;                 /*    20     4 */
	int                        size;                 /*    24     4 */

	/* size: 32, cachelines: 1, members: 5 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct rsvd_bits_validate {
	u64                        rsvd_bits_mask[2][5]; /*     0    80 */
	/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
	u64                        bad_mt_xwr;           /*    80     8 */

	/* size: 88, cachelines: 2, members: 2 */
	/* last cacheline: 24 bytes */
};
struct kvm_mmu_root_info {
	gpa_t                      pgd;                  /*     0     8 */
	hpa_t                      hpa;                  /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct kvm_mmu {
	long unsigned int          (*get_guest_pgd)(struct kvm_vcpu *); /*     0     8 */
	u64                        (*get_pdptr)(struct kvm_vcpu *, int); /*     8     8 */
	int                        (*page_fault)(struct kvm_vcpu *, gpa_t, u32, bool); /*    16     8 */
	void                       (*inject_page_fault)(struct kvm_vcpu *, struct x86_exception *); /*    24     8 */
	gpa_t                      (*gva_to_gpa)(struct kvm_vcpu *, gpa_t, u32, struct x86_exception *); /*    32     8 */
	gpa_t                      (*translate_gpa)(struct kvm_vcpu *, gpa_t, u32, struct x86_exception *); /*    40     8 */
	int                        (*sync_page)(struct kvm_vcpu *, struct kvm_mmu_page *); /*    48     8 */
	void                       (*invlpg)(struct kvm_vcpu *, gva_t, hpa_t); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	hpa_t                      root_hpa;             /*    64     8 */
	gpa_t                      root_pgd;             /*    72     8 */
	union kvm_mmu_role         mmu_role;             /*    80     8 */
	u8                         root_level;           /*    88     1 */
	u8                         shadow_root_level;    /*    89     1 */
	u8                         ept_ad;               /*    90     1 */
	bool                       direct_map;           /*    91     1 */

	/* XXX 4 bytes hole, try to pack */

	struct kvm_mmu_root_info   prev_roots[3];        /*    96    48 */
	/* --- cacheline 2 boundary (128 bytes) was 16 bytes ago --- */
	u8                         permissions[16];      /*   144    16 */
	u32                        pkru_mask;            /*   160     4 */

	/* XXX 4 bytes hole, try to pack */

	u64 *                      pae_root;             /*   168     8 */
	u64 *                      pml4_root;            /*   176     8 */
	u64 *                      pml5_root;            /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	struct rsvd_bits_validate  shadow_zero_check;    /*   192    88 */
	/* --- cacheline 4 boundary (256 bytes) was 24 bytes ago --- */
	struct rsvd_bits_validate  guest_rsvd_check;     /*   280    88 */
	/* --- cacheline 5 boundary (320 bytes) was 48 bytes ago --- */
	u8                         last_nonleaf_level;   /*   368     1 */
	bool                       nx;                   /*   369     1 */

	/* XXX 6 bytes hole, try to pack */

	u64                        pdptrs[4];            /*   376    32 */

	/* size: 408, cachelines: 7, members: 26 */
	/* sum members: 394, holes: 3, sum holes: 14 */
	/* last cacheline: 24 bytes */
};
struct kvm_pmc {
	enum pmc_type              type;                 /*     0     4 */
	u8                         idx;                  /*     4     1 */

	/* XXX 3 bytes hole, try to pack */

	u64                        counter;              /*     8     8 */
	u64                        eventsel;             /*    16     8 */
	struct perf_event *        perf_event;           /*    24     8 */
	struct kvm_vcpu *          vcpu;                 /*    32     8 */
	u64                        current_config;       /*    40     8 */
	bool                       is_paused;            /*    48     1 */

	/* size: 56, cachelines: 1, members: 8 */
	/* sum members: 46, holes: 1, sum holes: 3 */
	/* padding: 7 */
	/* last cacheline: 56 bytes */
};
struct kvm_pmu {
	unsigned int               nr_arch_gp_counters;  /*     0     4 */
	unsigned int               nr_arch_fixed_counters; /*     4     4 */
	unsigned int               available_event_types; /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        fixed_ctr_ctrl;       /*    16     8 */
	u64                        fixed_ctr_ctrl_mask;  /*    24     8 */
	u64                        global_ctrl;          /*    32     8 */
	u64                        global_status;        /*    40     8 */
	u64                        global_ovf_ctrl;      /*    48     8 */
	u64                        counter_bitmask[2];   /*    56    16 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	u64                        global_ctrl_mask;     /*    72     8 */
	u64                        global_ovf_ctrl_mask; /*    80     8 */
	u64                        reserved_bits;        /*    88     8 */
	u64                        raw_event_mask;       /*    96     8 */
	u8                         version;              /*   104     1 */

	/* XXX 7 bytes hole, try to pack */

	struct kvm_pmc             gp_counters[32];      /*   112  1792 */
	/* --- cacheline 29 boundary (1856 bytes) was 48 bytes ago --- */
	struct kvm_pmc             fixed_counters[4];    /*  1904   224 */
	/* --- cacheline 33 boundary (2112 bytes) was 16 bytes ago --- */
	struct irq_work            irq_work;             /*  2128    24 */
	long unsigned int          reprogram_pmi[1];     /*  2152     8 */
	long unsigned int          all_valid_pmc_idx[1]; /*  2160     8 */
	long unsigned int          pmc_in_use[1];        /*  2168     8 */
	/* --- cacheline 34 boundary (2176 bytes) --- */
	bool                       need_cleanup;         /*  2176     1 */
	u8                         event_count;          /*  2177     1 */

	/* size: 2184, cachelines: 35, members: 22 */
	/* sum members: 2167, holes: 2, sum holes: 11 */
	/* padding: 6 */
	/* last cacheline: 8 bytes */
};
struct kvm_mtrr_range {
	u64                        base;                 /*     0     8 */
	u64                        mask;                 /*     8     8 */
	struct list_head           node;                 /*    16    16 */

	/* size: 32, cachelines: 1, members: 3 */
	/* last cacheline: 32 bytes */
};
struct kvm_mtrr {
	struct kvm_mtrr_range      var_ranges[8];        /*     0   256 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	mtrr_type                  fixed_ranges[88];     /*   256    88 */
	/* --- cacheline 5 boundary (320 bytes) was 24 bytes ago --- */
	u64                        deftype;              /*   344     8 */
	struct list_head           head;                 /*   352    16 */

	/* size: 368, cachelines: 6, members: 4 */
	/* last cacheline: 48 bytes */
};
struct kvm_vcpu_hv_stimer {
	struct hrtimer             timer __attribute__((__aligned__(8))); /*     0    96 */
	/* --- cacheline 1 boundary (64 bytes) was 32 bytes ago --- */
	int                        index;                /*    96     4 */

	/* XXX 4 bytes hole, try to pack */

	union hv_stimer_config     config;               /*   104     8 */
	u64                        count;                /*   112     8 */
	u64                        exp_time;             /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct hv_message          msg;                  /*   128   256 */
	/* --- cacheline 6 boundary (384 bytes) --- */
	bool                       msg_pending;          /*   384     1 */

	/* size: 392, cachelines: 7, members: 7 */
	/* sum members: 381, holes: 1, sum holes: 4 */
	/* padding: 7 */
	/* forced alignments: 1 */
	/* last cacheline: 8 bytes */
} __attribute__((__aligned__(8)));
struct kvm_vcpu_hv_synic {
	u64                        version;              /*     0     8 */
	u64                        control;              /*     8     8 */
	u64                        msg_page;             /*    16     8 */
	u64                        evt_page;             /*    24     8 */
	atomic64_t                 sint[16];             /*    32   128 */
	/* --- cacheline 2 boundary (128 bytes) was 32 bytes ago --- */
	atomic_t                   sint_to_gsi[16];      /*   160    64 */
	/* --- cacheline 3 boundary (192 bytes) was 32 bytes ago --- */
	long unsigned int          auto_eoi_bitmap[4];   /*   224    32 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	long unsigned int          vec_bitmap[4];        /*   256    32 */
	bool                       active;               /*   288     1 */
	bool                       dont_zero_synic_pages; /*   289     1 */

	/* size: 296, cachelines: 5, members: 10 */
	/* padding: 6 */
	/* last cacheline: 40 bytes */
};
struct kvm_vcpu_hv {
	u32                        vp_index;             /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        hv_vapic;             /*     8     8 */
	s64                        runtime_offset;       /*    16     8 */
	struct kvm_vcpu_hv_synic   synic;                /*    24   296 */

	/* XXX last struct has 6 bytes of padding */

	/* --- cacheline 5 boundary (320 bytes) --- */
	struct kvm_hyperv_exit     exit;                 /*   320    56 */
	struct kvm_vcpu_hv_stimer  stimer[4] __attribute__((__aligned__(8))); /*   376  1568 */
	/* --- cacheline 30 boundary (1920 bytes) was 24 bytes ago --- */
	long unsigned int          stimer_pending_bitmap[1]; /*  1944     8 */
	cpumask_t                  tlb_flush;            /*  1952  1024 */

	/* size: 2976, cachelines: 47, members: 8 */
	/* sum members: 2972, holes: 1, sum holes: 4 */
	/* paddings: 1, sum paddings: 6 */
	/* forced alignments: 1 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct kvm_queued_exception {
	bool                       pending;              /*     0     1 */
	bool                       injected;             /*     1     1 */
	bool                       has_error_code;       /*     2     1 */
	u8                         nr;                   /*     3     1 */
	u32                        error_code;           /*     4     4 */
	long unsigned int          payload;              /*     8     8 */
	bool                       has_payload;          /*    16     1 */
	u8                         nested_apf;           /*    17     1 */

	/* size: 24, cachelines: 1, members: 8 */
	/* padding: 6 */
	/* last cacheline: 24 bytes */
};
struct kvm_queued_interrupt {
	bool                       injected;             /*     0     1 */
	bool                       soft;                 /*     1     1 */
	u8                         nr;                   /*     2     1 */

	/* size: 3, cachelines: 1, members: 3 */
	/* last cacheline: 3 bytes */
};
struct kvm_vcpu_arch {
	long unsigned int          regs[17];             /*     0   136 */
	/* --- cacheline 2 boundary (128 bytes) was 8 bytes ago --- */
	u32                        regs_avail;           /*   136     4 */
	u32                        regs_dirty;           /*   140     4 */
	long unsigned int          cr0;                  /*   144     8 */
	long unsigned int          cr0_guest_owned_bits; /*   152     8 */
	long unsigned int          cr2;                  /*   160     8 */
	long unsigned int          cr3;                  /*   168     8 */
	long unsigned int          cr4;                  /*   176     8 */
	long unsigned int          cr4_guest_owned_bits; /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	long unsigned int          cr4_guest_rsvd_bits;  /*   192     8 */
	long unsigned int          cr8;                  /*   200     8 */
	u32                        host_pkru;            /*   208     4 */
	u32                        pkru;                 /*   212     4 */
	u32                        hflags;               /*   216     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        efer;                 /*   224     8 */
	u64                        apic_base;            /*   232     8 */
	struct kvm_lapic *         apic;                 /*   240     8 */
	bool                       apicv_active;         /*   248     1 */
	bool                       load_eoi_exitmap_pending; /*   249     1 */

	/* XXX 6 bytes hole, try to pack */

	/* --- cacheline 4 boundary (256 bytes) --- */
	long unsigned int          ioapic_handled_vectors[4]; /*   256    32 */
	long unsigned int          apic_attention;       /*   288     8 */
	int32_t                    apic_arb_prio;        /*   296     4 */
	int                        mp_state;             /*   300     4 */
	u64                        ia32_misc_enable_msr; /*   304     8 */
	u64                        smbase;               /*   312     8 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	u64                        smi_count;            /*   320     8 */
	bool                       at_instruction_boundary; /*   328     1 */
	bool                       tpr_access_reporting; /*   329     1 */
	bool                       xsaves_enabled;       /*   330     1 */
	bool                       xfd_no_write_intercept; /*   331     1 */

	/* XXX 4 bytes hole, try to pack */

	u64                        ia32_xss;             /*   336     8 */
	u64                        microcode_version;    /*   344     8 */
	u64                        arch_capabilities;    /*   352     8 */
	u64                        perf_capabilities;    /*   360     8 */
	struct kvm_mmu *           mmu;                  /*   368     8 */
	struct kvm_mmu             root_mmu;             /*   376   408 */
	/* --- cacheline 12 boundary (768 bytes) was 16 bytes ago --- */
	struct kvm_mmu             guest_mmu;            /*   784   408 */
	/* --- cacheline 18 boundary (1152 bytes) was 40 bytes ago --- */
	struct kvm_mmu             nested_mmu;           /*  1192   408 */
	/* --- cacheline 25 boundary (1600 bytes) --- */
	struct kvm_mmu *           walk_mmu;             /*  1600     8 */
	struct kvm_mmu_memory_cache mmu_pte_list_desc_cache; /*  1608   336 */
	/* --- cacheline 30 boundary (1920 bytes) was 24 bytes ago --- */
	struct kvm_mmu_memory_cache mmu_shadow_page_cache; /*  1944   336 */
	/* --- cacheline 35 boundary (2240 bytes) was 40 bytes ago --- */
	struct kvm_mmu_memory_cache mmu_gfn_array_cache; /*  2280   336 */
	/* --- cacheline 40 boundary (2560 bytes) was 56 bytes ago --- */
	struct kvm_mmu_memory_cache mmu_page_header_cache; /*  2616   336 */
	/* --- cacheline 46 boundary (2944 bytes) was 8 bytes ago --- */
	struct fpu_guest           guest_fpu;            /*  2952    40 */
	u64                        xcr0;                 /*  2992     8 */
	u64                        guest_supported_xcr0; /*  3000     8 */
	/* --- cacheline 47 boundary (3008 bytes) --- */
	struct kvm_pio_request     pio;                  /*  3008    32 */

	/* XXX last struct has 4 bytes of padding */

	void *                     pio_data;             /*  3040     8 */
	u8                         event_exit_inst_len;  /*  3048     1 */

	/* XXX 7 bytes hole, try to pack */

	struct kvm_queued_exception exception;           /*  3056    24 */

	/* XXX last struct has 6 bytes of padding */

	/* --- cacheline 48 boundary (3072 bytes) was 8 bytes ago --- */
	struct kvm_queued_interrupt interrupt;           /*  3080     3 */

	/* XXX 1 byte hole, try to pack */

	int                        halt_request;         /*  3084     4 */
	int                        cpuid_nent;           /*  3088     4 */

	/* XXX 4 bytes hole, try to pack */

	struct kvm_cpuid_entry2 *  cpuid_entries;        /*  3096     8 */
	long unsigned int          cr3_lm_rsvd_bits;     /*  3104     8 */
	int                        maxphyaddr;           /*  3112     4 */
	int                        max_tdp_level;        /*  3116     4 */
	struct x86_emulate_ctxt *  emulate_ctxt;         /*  3120     8 */
	bool                       emulate_regs_need_sync_to_vcpu; /*  3128     1 */
	bool                       emulate_regs_need_sync_from_vcpu; /*  3129     1 */

	/* XXX 6 bytes hole, try to pack */

	/* --- cacheline 49 boundary (3136 bytes) --- */
	int                        (*complete_userspace_io)(struct kvm_vcpu *); /*  3136     8 */
	gpa_t                      time;                 /*  3144     8 */
	struct pvclock_vcpu_time_info hv_clock;          /*  3152    32 */
	unsigned int               hw_tsc_khz;           /*  3184     4 */

	/* XXX 4 bytes hole, try to pack */

	struct gfn_to_hva_cache    pv_time;              /*  3192    40 */
	/* --- cacheline 50 boundary (3200 bytes) was 32 bytes ago --- */
	bool                       pv_time_enabled;      /*  3232     1 */
	bool                       pvclock_set_guest_stopped_request; /*  3233     1 */

	/* XXX 6 bytes hole, try to pack */

	struct {
		u8                 preempted;            /*  3240     1 */

		/* XXX 7 bytes hole, try to pack */

		u64                msr_val;              /*  3248     8 */
		u64                last_steal;           /*  3256     8 */
		/* --- cacheline 51 boundary (3264 bytes) --- */
		struct gfn_to_pfn_cache cache;           /*  3264    32 */

		/* XXX last struct has 7 bytes of padding */
	} st;                                            /*  3240    56 */
	u64                        l1_tsc_offset;        /*  3296     8 */
	u64                        tsc_offset;           /*  3304     8 */
	u64                        last_guest_tsc;       /*  3312     8 */
	u64                        last_host_tsc;        /*  3320     8 */
	/* --- cacheline 52 boundary (3328 bytes) --- */
	u64                        tsc_offset_adjustment; /*  3328     8 */
	u64                        this_tsc_nsec;        /*  3336     8 */
	u64                        this_tsc_write;       /*  3344     8 */
	u64                        this_tsc_generation;  /*  3352     8 */
	bool                       tsc_catchup;          /*  3360     1 */
	bool                       tsc_always_catchup;   /*  3361     1 */
	s8                         virtual_tsc_shift;    /*  3362     1 */

	/* XXX 1 byte hole, try to pack */

	u32                        virtual_tsc_mult;     /*  3364     4 */
	u32                        virtual_tsc_khz;      /*  3368     4 */

	/* XXX 4 bytes hole, try to pack */

	s64                        ia32_tsc_adjust_msr;  /*  3376     8 */
	u64                        msr_ia32_power_ctl;   /*  3384     8 */
	/* --- cacheline 53 boundary (3392 bytes) --- */
	u64                        tsc_scaling_ratio;    /*  3392     8 */
	atomic_t                   nmi_queued;           /*  3400     4 */
	unsigned int               nmi_pending;          /*  3404     4 */
	bool                       nmi_injected;         /*  3408     1 */
	bool                       smi_pending;          /*  3409     1 */

	/* XXX 6 bytes hole, try to pack */

	struct kvm_mtrr            mtrr_state;           /*  3416   368 */
	/* --- cacheline 59 boundary (3776 bytes) was 8 bytes ago --- */
	u64                        pat;                  /*  3784     8 */
	unsigned int               switch_db_regs;       /*  3792     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          db[4];                /*  3800    32 */
	long unsigned int          dr6;                  /*  3832     8 */
	/* --- cacheline 60 boundary (3840 bytes) --- */
	long unsigned int          dr7;                  /*  3840     8 */
	long unsigned int          eff_db[4];            /*  3848    32 */
	long unsigned int          guest_debug_dr7;      /*  3880     8 */
	u64                        msr_platform_info;    /*  3888     8 */
	u64                        msr_misc_features_enables; /*  3896     8 */
	/* --- cacheline 61 boundary (3904 bytes) --- */
	u64                        mcg_cap;              /*  3904     8 */
	u64                        mcg_status;           /*  3912     8 */
	u64                        mcg_ctl;              /*  3920     8 */
	u64                        mcg_ext_ctl;          /*  3928     8 */
	u64 *                      mce_banks;            /*  3936     8 */
	u64                        mmio_gva;             /*  3944     8 */
	unsigned int               mmio_access;          /*  3952     4 */

	/* XXX 4 bytes hole, try to pack */

	gfn_t                      mmio_gfn;             /*  3960     8 */
	/* --- cacheline 62 boundary (3968 bytes) --- */
	u64                        mmio_gen;             /*  3968     8 */
	struct kvm_pmu             pmu;                  /*  3976  2184 */

	/* XXX last struct has 6 bytes of padding */

	/* --- cacheline 96 boundary (6144 bytes) was 16 bytes ago --- */
	long unsigned int          singlestep_rip;       /*  6160     8 */
	struct kvm_vcpu_hv         hyperv __attribute__((__aligned__(8))); /*  6168  2976 */
	/* --- cacheline 142 boundary (9088 bytes) was 56 bytes ago --- */
	cpumask_var_t              wbinvd_dirty_mask;    /*  9144     8 */
	/* --- cacheline 143 boundary (9152 bytes) --- */
	long unsigned int          last_retry_eip;       /*  9152     8 */
	long unsigned int          last_retry_addr;      /*  9160     8 */
	struct {
		bool               halted;               /*  9168     1 */

		/* XXX 7 bytes hole, try to pack */

		gfn_t              gfns[64];             /*  9176   512 */
		/* --- cacheline 151 boundary (9664 bytes) was 24 bytes ago --- */
		struct gfn_to_hva_cache data;            /*  9688    40 */
		/* --- cacheline 152 boundary (9728 bytes) --- */
		u64                msr_en_val;           /*  9728     8 */
		u64                msr_int_val;          /*  9736     8 */
		u16                vec;                  /*  9744     2 */

		/* XXX 2 bytes hole, try to pack */

		u32                id;                   /*  9748     4 */
		bool               send_user_only;       /*  9752     1 */

		/* XXX 3 bytes hole, try to pack */

		u32                host_apf_flags;       /*  9756     4 */
		long unsigned int  nested_apf_token;     /*  9760     8 */
		bool               delivery_as_pf_vmexit; /*  9768     1 */
		bool               pageready_pending;    /*  9769     1 */
	} apf;                                           /*  9168   608 */

	/* XXX last struct has 6 bytes of padding */

	struct {
		u64                length;               /*  9776     8 */
		u64                status;               /*  9784     8 */
	} osvw;                                          /*  9776    16 */
	/* --- cacheline 153 boundary (9792 bytes) --- */
	struct {
		u64                msr_val;              /*  9792     8 */
		struct gfn_to_hva_cache data;            /*  9800    40 */
	} pv_eoi;                                        /*  9792    48 */
	u64                        msr_kvm_poll_control; /*  9840     8 */
	bool                       write_fault_to_shadow_pgtable; /*  9848     1 */

	/* XXX 7 bytes hole, try to pack */

	/* --- cacheline 154 boundary (9856 bytes) --- */
	long unsigned int          exit_qualification;   /*  9856     8 */
	struct {
		bool               pv_unhalted;          /*  9864     1 */
	} pv;                                            /*  9864     1 */

	/* XXX 3 bytes hole, try to pack */

	int                        pending_ioapic_eoi;   /*  9868     4 */
	int                        pending_external_vector; /*  9872     4 */
	bool                       preempted_in_kernel;  /*  9876     1 */
	bool                       l1tf_flush_l1d;       /*  9877     1 */

	/* XXX 2 bytes hole, try to pack */

	unsigned int               last_vmentry_cpu;     /*  9880     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        msr_hwcr;             /*  9888     8 */
	struct {
		u32                features;             /*  9896     4 */
		bool               enforce;              /*  9900     1 */
	} pv_cpuid;                                      /*  9896     8 */

	/* XXX last struct has 3 bytes of padding */

	/* size: 9904, cachelines: 155, members: 127 */
	/* sum members: 9827, holes: 18, sum holes: 77 */
	/* paddings: 5, sum paddings: 25 */
	/* forced alignments: 1 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct kvm_lpage_info {
	int                        disallow_lpage;       /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct kvm_arch_memory_slot {
	struct kvm_rmap_head *     rmap[3];              /*     0    24 */
	struct kvm_lpage_info *    lpage_info[2];        /*    24    16 */
	short unsigned int *       gfn_track[1];         /*    40     8 */

	/* size: 48, cachelines: 1, members: 3 */
	/* last cacheline: 48 bytes */
};
struct kvm_apic_map {
	struct callback_head       rcu __attribute__((__aligned__(8))); /*     0    16 */
	u8                         mode;                 /*    16     1 */

	/* XXX 3 bytes hole, try to pack */

	u32                        max_apic_id;          /*    20     4 */
	union {
		struct kvm_lapic * xapic_flat_map[8];    /*    24    64 */
		struct kvm_lapic * xapic_cluster_map[16][4]; /*    24   512 */
	};                                               /*    24   512 */
	/* --- cacheline 8 boundary (512 bytes) was 24 bytes ago --- */
	struct kvm_lapic *         phys_map[];           /*   536     0 */

	/* size: 536, cachelines: 9, members: 5 */
	/* sum members: 533, holes: 1, sum holes: 3 */
	/* forced alignments: 1 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct kvm_hv_syndbg {
	struct {
		u64                control;              /*     0     8 */
		u64                status;               /*     8     8 */
		u64                send_page;            /*    16     8 */
		u64                recv_page;            /*    24     8 */
		u64                pending_page;         /*    32     8 */
	} control;                                       /*     0    40 */
	u64                        options;              /*    40     8 */

	/* size: 48, cachelines: 1, members: 2 */
	/* last cacheline: 48 bytes */
};
struct kvm_hv {
	struct mutex               hv_lock;              /*     0    32 */
	u64                        hv_guest_os_id;       /*    32     8 */
	u64                        hv_hypercall;         /*    40     8 */
	u64                        hv_tsc_page;          /*    48     8 */
	u64                        hv_crash_param[5];    /*    56    40 */
	/* --- cacheline 1 boundary (64 bytes) was 32 bytes ago --- */
	u64                        hv_crash_ctl;         /*    96     8 */
	struct ms_hyperv_tsc_page  tsc_ref;              /*   104    24 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct idr                 conn_to_evt;          /*   128    24 */
	u64                        hv_reenlightenment_control; /*   152     8 */
	u64                        hv_tsc_emulation_control; /*   160     8 */
	u64                        hv_tsc_emulation_status; /*   168     8 */
	atomic_t                   num_mismatched_vp_indexes; /*   176     4 */

	/* XXX 4 bytes hole, try to pack */

	struct hv_partition_assist_pg * hv_pa_pg;        /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	struct kvm_hv_syndbg       hv_syndbg;            /*   192    48 */

	/* size: 240, cachelines: 4, members: 14 */
	/* sum members: 236, holes: 1, sum holes: 4 */
	/* last cacheline: 48 bytes */
};
struct msr_bitmap_range {
	u32                        flags;                /*     0     4 */
	u32                        nmsrs;                /*     4     4 */
	u32                        base;                 /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int *        bitmap;               /*    16     8 */

	/* size: 24, cachelines: 1, members: 4 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct kvm_x86_msr_filter {
	u8                         count;                /*     0     1 */
	bool                       default_allow:1;      /*     1: 0  1 */

	/* XXX 7 bits hole, try to pack */
	/* XXX 6 bytes hole, try to pack */

	struct msr_bitmap_range    ranges[16];           /*     8   384 */

	/* size: 392, cachelines: 7, members: 3 */
	/* sum members: 385, holes: 1, sum holes: 6 */
	/* sum bitfield members: 1 bits, bit holes: 1, sum bit holes: 7 bits */
	/* last cacheline: 8 bytes */
};
struct kvm_arch {
	long unsigned int          n_used_mmu_pages;     /*     0     8 */
	long unsigned int          n_requested_mmu_pages; /*     8     8 */
	long unsigned int          n_max_mmu_pages;      /*    16     8 */
	unsigned int               indirect_shadow_pages; /*    24     4 */
	u8                         mmu_valid_gen;        /*    28     1 */

	/* XXX 3 bytes hole, try to pack */

	struct hlist_head          mmu_page_hash[4096];  /*    32 32768 */
	/* --- cacheline 512 boundary (32768 bytes) was 32 bytes ago --- */
	struct list_head           active_mmu_pages;     /* 32800    16 */
	struct list_head           zapped_obsolete_pages; /* 32816    16 */
	/* --- cacheline 513 boundary (32832 bytes) --- */
	struct list_head           lpage_disallowed_mmu_pages; /* 32832    16 */
	struct kvm_page_track_notifier_node mmu_sp_tracker; /* 32848    32 */
	struct kvm_page_track_notifier_head track_notifier_head; /* 32880 50448 */
	/* --- cacheline 1302 boundary (83328 bytes) --- */
	struct list_head           assigned_dev_head;    /* 83328    16 */
	struct iommu_domain *      iommu_domain;         /* 83344     8 */
	bool                       iommu_noncoherent;    /* 83352     1 */

	/* XXX 3 bytes hole, try to pack */

	atomic_t                   noncoherent_dma_count; /* 83356     4 */
	atomic_t                   assigned_device_count; /* 83360     4 */

	/* XXX 4 bytes hole, try to pack */

	struct kvm_pic *           vpic;                 /* 83368     8 */
	struct kvm_ioapic *        vioapic;              /* 83376     8 */
	struct kvm_pit *           vpit;                 /* 83384     8 */
	/* --- cacheline 1303 boundary (83392 bytes) --- */
	atomic_t                   vapics_in_nmi_mode;   /* 83392     4 */

	/* XXX 4 bytes hole, try to pack */

	struct mutex               apic_map_lock;        /* 83400    32 */
	struct kvm_apic_map *      apic_map;             /* 83432     8 */
	atomic_t                   apic_map_dirty;       /* 83440     4 */
	bool                       apic_access_page_done; /* 83444     1 */

	/* XXX 3 bytes hole, try to pack */

	long unsigned int          apicv_inhibit_reasons; /* 83448     8 */
	/* --- cacheline 1304 boundary (83456 bytes) --- */
	gpa_t                      wall_clock;           /* 83456     8 */
	bool                       mwait_in_guest;       /* 83464     1 */
	bool                       hlt_in_guest;         /* 83465     1 */
	bool                       pause_in_guest;       /* 83466     1 */
	bool                       cstate_in_guest;      /* 83467     1 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          irq_sources_bitmap;   /* 83472     8 */
	s64                        kvmclock_offset;      /* 83480     8 */
	raw_spinlock_t             tsc_write_lock;       /* 83488     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        last_tsc_nsec;        /* 83496     8 */
	u64                        last_tsc_write;       /* 83504     8 */
	u32                        last_tsc_khz;         /* 83512     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 1305 boundary (83520 bytes) --- */
	u64                        cur_tsc_nsec;         /* 83520     8 */
	u64                        cur_tsc_write;        /* 83528     8 */
	u64                        cur_tsc_offset;       /* 83536     8 */
	u64                        cur_tsc_generation;   /* 83544     8 */
	int                        nr_vcpus_matched_tsc; /* 83552     4 */
	spinlock_t                 pvclock_gtod_sync_lock; /* 83556     4 */
	bool                       use_master_clock;     /* 83560     1 */

	/* XXX 7 bytes hole, try to pack */

	u64                        master_kernel_ns;     /* 83568     8 */
	u64                        master_cycle_now;     /* 83576     8 */
	/* --- cacheline 1306 boundary (83584 bytes) --- */
	struct delayed_work        kvmclock_update_work; /* 83584   192 */
	/* --- cacheline 1309 boundary (83776 bytes) --- */
	struct delayed_work        kvmclock_sync_work;   /* 83776   192 */
	/* --- cacheline 1312 boundary (83968 bytes) --- */
	struct kvm_xen_hvm_config  xen_hvm_config;       /* 83968    56 */
	struct hlist_head          mask_notifier_list;   /* 84024     8 */
	/* --- cacheline 1313 boundary (84032 bytes) --- */
	struct kvm_hv              hyperv;               /* 84032   240 */
	/* --- cacheline 1316 boundary (84224 bytes) was 48 bytes ago --- */
	int                        audit_point;          /* 84272     4 */
	bool                       backwards_tsc_observed; /* 84276     1 */
	bool                       boot_vcpu_runs_old_kvmclock; /* 84277     1 */

	/* XXX 2 bytes hole, try to pack */

	u32                        bsp_vcpu_id;          /* 84280     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 1317 boundary (84288 bytes) --- */
	u64                        disabled_quirks;      /* 84288     8 */
	enum kvm_irqchip_mode      irqchip_mode;         /* 84296     4 */
	u8                         nr_reserved_ioapic_pins; /* 84300     1 */
	bool                       disabled_lapic_found; /* 84301     1 */
	bool                       x2apic_format;        /* 84302     1 */
	bool                       x2apic_broadcast_quirk_disabled; /* 84303     1 */
	bool                       guest_can_read_msr_platform_info; /* 84304     1 */
	bool                       exception_payload_enabled; /* 84305     1 */
	bool                       triple_fault_event;   /* 84306     1 */
	bool                       bus_lock_detection_enabled; /* 84307     1 */
	u32                        notify_window;        /* 84308     4 */
	u32                        notify_vmexit_flags;  /* 84312     4 */
	bool                       sgx_provisioning_allowed; /* 84316     1 */

	/* XXX 3 bytes hole, try to pack */

	u32                        user_space_msr_mask;  /* 84320     4 */

	/* XXX 4 bytes hole, try to pack */

	struct kvm_x86_msr_filter * msr_filter;          /* 84328     8 */
	struct kvm_pmu_event_filter * pmu_event_filter;  /* 84336     8 */
	struct task_struct *       nx_lpage_recovery_thread; /* 84344     8 */
	/* --- cacheline 1318 boundary (84352 bytes) --- */
	bool                       tdp_mmu_enabled;      /* 84352     1 */

	/* XXX 7 bytes hole, try to pack */

	struct list_head           tdp_mmu_roots;        /* 84360    16 */
	struct list_head           tdp_mmu_pages;        /* 84376    16 */
	u32                        max_vcpu_ids;         /* 84392     4 */

	/* size: 84400, cachelines: 1319, members: 75 */
	/* sum members: 84340, holes: 14, sum holes: 56 */
	/* padding: 4 */
	/* last cacheline: 48 bytes */
};
struct kvm_vm_stat {
	ulong                      mmu_shadow_zapped;    /*     0     8 */
	ulong                      mmu_pte_write;        /*     8     8 */
	ulong                      mmu_pde_zapped;       /*    16     8 */
	ulong                      mmu_flooded;          /*    24     8 */
	ulong                      mmu_recycled;         /*    32     8 */
	ulong                      mmu_cache_miss;       /*    40     8 */
	ulong                      mmu_unsync;           /*    48     8 */
	ulong                      remote_tlb_flush;     /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	ulong                      lpages;               /*    64     8 */
	ulong                      nx_lpage_splits;      /*    72     8 */
	ulong                      max_mmu_page_hash_collisions; /*    80     8 */

	/* size: 88, cachelines: 2, members: 11 */
	/* last cacheline: 24 bytes */
};
struct kvm_vcpu_stat {
	u64                        pid;                  /*     0     8 */
	u64                        pf_fixed;             /*     8     8 */
	u64                        pf_guest;             /*    16     8 */
	u64                        tlb_flush;            /*    24     8 */
	u64                        invlpg;               /*    32     8 */
	u64                        exits;                /*    40     8 */
	u64                        io_exits;             /*    48     8 */
	u64                        mmio_exits;           /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u64                        signal_exits;         /*    64     8 */
	u64                        irq_window_exits;     /*    72     8 */
	u64                        nmi_window_exits;     /*    80     8 */
	u64                        l1d_flush;            /*    88     8 */
	u64                        halt_exits;           /*    96     8 */
	u64                        halt_successful_poll; /*   104     8 */
	u64                        halt_attempted_poll;  /*   112     8 */
	u64                        halt_poll_invalid;    /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	u64                        halt_wakeup;          /*   128     8 */
	u64                        request_irq_exits;    /*   136     8 */
	u64                        irq_exits;            /*   144     8 */
	u64                        host_state_reload;    /*   152     8 */
	u64                        fpu_reload;           /*   160     8 */
	u64                        insn_emulation;       /*   168     8 */
	u64                        insn_emulation_fail;  /*   176     8 */
	u64                        hypercalls;           /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	u64                        irq_injections;       /*   192     8 */
	u64                        nmi_injections;       /*   200     8 */
	u64                        req_event;            /*   208     8 */
	u64                        halt_poll_success_ns; /*   216     8 */
	u64                        halt_poll_fail_ns;    /*   224     8 */
	u64                        cr_exits;             /*   232     8 */
	u64                        msr_rd_exits;         /*   240     8 */
	u64                        msr_wr_exits;         /*   248     8 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	u64                        apic_wr_exits;        /*   256     8 */
	u64                        ept_vio_exits;        /*   264     8 */
	u64                        ept_mis_exits;        /*   272     8 */
	u64                        pause_exits;          /*   280     8 */
	u64                        steal;                /*   288     8 */
	u64                        st_max;               /*   296     8 */
	u64                        utime;                /*   304     8 */
	u64                        stime;                /*   312     8 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	u64                        gtime;                /*   320     8 */
	u64                        preemption_reported;  /*   328     8 */
	u64                        preemption_other;     /*   336     8 */
	u64                        preemption_timer_exits; /*   344     8 */
	u64                        notify_window_exits;  /*   352     8 */

	/* size: 360, cachelines: 6, members: 45 */
	/* last cacheline: 40 bytes */
};
struct kvm_io_range {
	gpa_t                      addr;                 /*     0     8 */
	int                        len;                  /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct kvm_io_device *     dev;                  /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct kvm_io_bus {
	int                        dev_count;            /*     0     4 */
	int                        ioeventfd_count;      /*     4     4 */
	struct kvm_io_range        range[];              /*     8     0 */

	/* size: 8, cachelines: 1, members: 3 */
	/* last cacheline: 8 bytes */
};
struct kvm_mmio_fragment {
	gpa_t                      gpa;                  /*     0     8 */
	void *                     data;                 /*     8     8 */
	unsigned int               len;                  /*    16     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct kvm_irq_routing_table {
	int                        chip[3][24];          /*     0   288 */
	/* --- cacheline 4 boundary (256 bytes) was 32 bytes ago --- */
	u32                        nr_rt_entries;        /*   288     4 */

	/* XXX 4 bytes hole, try to pack */

	struct hlist_head          map[];                /*   296     0 */

	/* size: 296, cachelines: 5, members: 3 */
	/* sum members: 292, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct kvm_memslots {
	u64                        generation;           /*     0     8 */
	short int                  id_to_index[512];     /*     8  1024 */
	/* --- cacheline 16 boundary (1024 bytes) was 8 bytes ago --- */
	atomic_t                   lru_slot;             /*  1032     4 */
	int                        used_slots;           /*  1036     4 */
	struct kvm_memory_slot     memslots[];           /*  1040     0 */

	/* size: 1040, cachelines: 17, members: 5 */
	/* last cacheline: 16 bytes */
};
struct kvm_stat_data {
	struct kvm *               kvm;                  /*     0     8 */
	struct kvm_stats_debugfs_item * dbgfs_item;      /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct kvm_stats_debugfs_item {
	const char  *              name;                 /*     0     8 */
	int                        offset;               /*     8     4 */
	enum kvm_stat_kind         kind;                 /*    12     4 */
	int                        mode;                 /*    16     4 */

	/* size: 24, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct kcmp_epoll_slot {
	__u32                      efd;                  /*     0     4 */
	__u32                      tfd;                  /*     4     4 */
	__u32                      toff;                 /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct profile_hit {
	u32                        pc;                   /*     0     4 */
	u32                        hits;                 /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct stacktrace_cookie {
	long unsigned int *        store;                /*     0     8 */
	unsigned int               size;                 /*     8     4 */
	unsigned int               skip;                 /*    12     4 */
	unsigned int               len;                  /*    16     4 */

	/* size: 24, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct __kernel_itimerspec {
	struct __kernel_timespec   it_interval;          /*     0    16 */
	struct __kernel_timespec   it_value;             /*    16    16 */

	/* size: 32, cachelines: 1, members: 2 */
	/* last cacheline: 32 bytes */
};
struct timezone {
	int                        tz_minuteswest;       /*     0     4 */
	int                        tz_dsttime;           /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct itimerspec64 {
	struct timespec64          it_interval;          /*     0    16 */
	struct timespec64          it_value;             /*    16    16 */

	/* size: 32, cachelines: 1, members: 2 */
	/* last cacheline: 32 bytes */
};
struct old_itimerspec32 {
	struct old_timespec32      it_interval;          /*     0     8 */
	struct old_timespec32      it_value;             /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct old_timex32 {
	u32                        modes;                /*     0     4 */
	s32                        offset;               /*     4     4 */
	s32                        freq;                 /*     8     4 */
	s32                        maxerror;             /*    12     4 */
	s32                        esterror;             /*    16     4 */
	s32                        status;               /*    20     4 */
	s32                        constant;             /*    24     4 */
	s32                        precision;            /*    28     4 */
	s32                        tolerance;            /*    32     4 */
	struct old_timeval32       time;                 /*    36     8 */
	s32                        tick;                 /*    44     4 */
	s32                        ppsfreq;              /*    48     4 */
	s32                        jitter;               /*    52     4 */
	s32                        shift;                /*    56     4 */
	s32                        stabil;               /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	s32                        jitcnt;               /*    64     4 */
	s32                        calcnt;               /*    68     4 */
	s32                        errcnt;               /*    72     4 */
	s32                        stbcnt;               /*    76     4 */
	s32                        tai;                  /*    80     4 */

	/* Force padding: */
	s32                        :32;
	s32                        :32;
	s32                        :32;
	s32                        :32;
	s32                        :32;
	s32                        :32;
	s32                        :32;
	s32                        :32;
	s32                        :32;
	s32                        :32;
	s32                        :32;

	/* size: 128, cachelines: 2, members: 20 */
	/* padding: 44 */
};
struct __kernel_timex_timeval {
	__kernel_time64_t          tv_sec;               /*     0     8 */
	long long int              tv_usec;              /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct __kernel_timex {
	unsigned int               modes;                /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	long long int              offset;               /*     8     8 */
	long long int              freq;                 /*    16     8 */
	long long int              maxerror;             /*    24     8 */
	long long int              esterror;             /*    32     8 */
	int                        status;               /*    40     4 */

	/* XXX 4 bytes hole, try to pack */

	long long int              constant;             /*    48     8 */
	long long int              precision;            /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	long long int              tolerance;            /*    64     8 */
	struct __kernel_timex_timeval time;              /*    72    16 */
	long long int              tick;                 /*    88     8 */
	long long int              ppsfreq;              /*    96     8 */
	long long int              jitter;               /*   104     8 */
	int                        shift;                /*   112     4 */

	/* XXX 4 bytes hole, try to pack */

	long long int              stabil;               /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	long long int              jitcnt;               /*   128     8 */
	long long int              calcnt;               /*   136     8 */
	long long int              errcnt;               /*   144     8 */
	long long int              stbcnt;               /*   152     8 */
	int                        tai;                  /*   160     4 */

	/* Force padding: */
	int                        :32;
	int                        :32;
	int                        :32;
	int                        :32;
	int                        :32;
	int                        :32;
	int                        :32;
	int                        :32;
	int                        :32;
	int                        :32;
	int                        :32;

	/* size: 208, cachelines: 4, members: 20 */
	/* sum members: 152, holes: 3, sum holes: 12 */
	/* padding: 44 */
	/* last cacheline: 16 bytes */
};
struct trace_event_raw_timer_class {
	struct trace_entry         ent;                  /*     0     8 */
	void *                     timer;                /*     8     8 */
	char                       __data[];             /*    16     0 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct trace_event_raw_timer_start {
	struct trace_entry         ent;                  /*     0     8 */
	void *                     timer;                /*     8     8 */
	void *                     function;             /*    16     8 */
	long unsigned int          expires;              /*    24     8 */
	long unsigned int          now;                  /*    32     8 */
	unsigned int               flags;                /*    40     4 */
	char                       __data[];             /*    44     0 */

	/* size: 48, cachelines: 1, members: 7 */
	/* padding: 4 */
	/* last cacheline: 48 bytes */
};
struct trace_event_raw_timer_expire_entry {
	struct trace_entry         ent;                  /*     0     8 */
	void *                     timer;                /*     8     8 */
	long unsigned int          now;                  /*    16     8 */
	void *                     function;             /*    24     8 */
	long unsigned int          baseclk;              /*    32     8 */
	char                       __data[];             /*    40     0 */

	/* size: 40, cachelines: 1, members: 6 */
	/* last cacheline: 40 bytes */
};
struct trace_event_raw_hrtimer_init {
	struct trace_entry         ent;                  /*     0     8 */
	void *                     hrtimer;              /*     8     8 */
	clockid_t                  clockid;              /*    16     4 */
	enum hrtimer_mode          mode;                 /*    20     4 */
	char                       __data[];             /*    24     0 */

	/* size: 24, cachelines: 1, members: 5 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_hrtimer_start {
	struct trace_entry         ent;                  /*     0     8 */
	void *                     hrtimer;              /*     8     8 */
	void *                     function;             /*    16     8 */
	s64                        expires;              /*    24     8 */
	s64                        softexpires;          /*    32     8 */
	enum hrtimer_mode          mode;                 /*    40     4 */
	char                       __data[];             /*    44     0 */

	/* size: 48, cachelines: 1, members: 7 */
	/* padding: 4 */
	/* last cacheline: 48 bytes */
};
struct trace_event_raw_hrtimer_expire_entry {
	struct trace_entry         ent;                  /*     0     8 */
	void *                     hrtimer;              /*     8     8 */
	s64                        now;                  /*    16     8 */
	void *                     function;             /*    24     8 */
	char                       __data[];             /*    32     0 */

	/* size: 32, cachelines: 1, members: 5 */
	/* last cacheline: 32 bytes */
};
struct trace_event_raw_hrtimer_class {
	struct trace_entry         ent;                  /*     0     8 */
	void *                     hrtimer;              /*     8     8 */
	char                       __data[];             /*    16     0 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct trace_event_raw_itimer_state {
	struct trace_entry         ent;                  /*     0     8 */
	int                        which;                /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	long long unsigned int     expires;              /*    16     8 */
	long int                   value_sec;            /*    24     8 */
	long int                   value_nsec;           /*    32     8 */
	long int                   interval_sec;         /*    40     8 */
	long int                   interval_nsec;        /*    48     8 */
	char                       __data[];             /*    56     0 */

	/* size: 56, cachelines: 1, members: 8 */
	/* sum members: 52, holes: 1, sum holes: 4 */
	/* last cacheline: 56 bytes */
};
struct trace_event_raw_itimer_expire {
	struct trace_entry         ent;                  /*     0     8 */
	int                        which;                /*     8     4 */
	pid_t                      pid;                  /*    12     4 */
	long long unsigned int     now;                  /*    16     8 */
	char                       __data[];             /*    24     0 */

	/* size: 24, cachelines: 1, members: 5 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_tick_stop {
	struct trace_entry         ent;                  /*     0     8 */
	int                        success;              /*     8     4 */
	int                        dependency;           /*    12     4 */
	char                       __data[];             /*    16     0 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct trace_event_data_offsets_timer_class {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_timer_start {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_timer_expire_entry {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_hrtimer_init {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_hrtimer_start {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_hrtimer_expire_entry {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_hrtimer_class {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_itimer_state {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_itimer_expire {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_tick_stop {

	/* size: 0, cachelines: 0, members: 0 */
};
struct timer_base {
	raw_spinlock_t             lock;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct timer_list *        running_timer;        /*     8     8 */
	long unsigned int          clk;                  /*    16     8 */
	long unsigned int          next_expiry;          /*    24     8 */
	unsigned int               cpu;                  /*    32     4 */
	bool                       next_expiry_recalc;   /*    36     1 */
	bool                       is_idle;              /*    37     1 */
	bool                       timers_pending;       /*    38     1 */

	/* XXX 1 byte hole, try to pack */

	long unsigned int          pending_map[9];       /*    40    72 */
	/* --- cacheline 1 boundary (64 bytes) was 48 bytes ago --- */
	struct hlist_head          vectors[576];         /*   112  4608 */

	/* size: 4736, cachelines: 74, members: 10 */
	/* sum members: 4715, holes: 2, sum holes: 5 */
	/* padding: 16 */
} __attribute__((__aligned__(64)));
struct process_timer {
	struct timer_list          timer;                /*     0    72 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	struct task_struct *       task;                 /*    72     8 */

	/* size: 80, cachelines: 2, members: 2 */
	/* last cacheline: 16 bytes */
};
struct tick_device {
	struct clock_event_device * evtdev;              /*     0     8 */
	enum tick_device_mode      mode;                 /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct ktime_timestamps {
	u64                        mono;                 /*     0     8 */
	u64                        boot;                 /*     8     8 */
	u64                        real;                 /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct system_time_snapshot {
	u64                        cycles;               /*     0     8 */
	ktime_t                    real;                 /*     8     8 */
	ktime_t                    raw;                  /*    16     8 */
	unsigned int               clock_was_set_seq;    /*    24     4 */
	u8                         cs_was_changed_seq;   /*    28     1 */

	/* size: 32, cachelines: 1, members: 5 */
	/* padding: 3 */
	/* last cacheline: 32 bytes */
};
struct system_device_crosststamp {
	ktime_t                    device;               /*     0     8 */
	ktime_t                    sys_realtime;         /*     8     8 */
	ktime_t                    sys_monoraw;          /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct tk_read_base {
	struct clocksource *       clock;                /*     0     8 */
	u64                        mask;                 /*     8     8 */
	u64                        cycle_last;           /*    16     8 */
	u32                        mult;                 /*    24     4 */
	u32                        shift;                /*    28     4 */
	u64                        xtime_nsec;           /*    32     8 */
	ktime_t                    base;                 /*    40     8 */
	u64                        base_real;            /*    48     8 */

	/* size: 56, cachelines: 1, members: 8 */
	/* last cacheline: 56 bytes */
};
struct timekeeper {
	struct tk_read_base        tkr_mono;             /*     0    56 */
	struct tk_read_base        tkr_raw;              /*    56    56 */
	/* --- cacheline 1 boundary (64 bytes) was 48 bytes ago --- */
	u64                        xtime_sec;            /*   112     8 */
	long unsigned int          ktime_sec;            /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct timespec64          wall_to_monotonic;    /*   128    16 */
	ktime_t                    offs_real;            /*   144     8 */
	ktime_t                    offs_boot;            /*   152     8 */
	ktime_t                    offs_tai;             /*   160     8 */
	s32                        tai_offset;           /*   168     4 */
	unsigned int               clock_was_set_seq;    /*   172     4 */
	u8                         cs_was_changed_seq;   /*   176     1 */

	/* XXX 7 bytes hole, try to pack */

	ktime_t                    next_leap_ktime;      /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	u64                        raw_sec;              /*   192     8 */
	struct timespec64          monotonic_to_boot;    /*   200    16 */
	u64                        cycle_interval;       /*   216     8 */
	u64                        xtime_interval;       /*   224     8 */
	s64                        xtime_remainder;      /*   232     8 */
	u64                        raw_interval;         /*   240     8 */
	u64                        ntp_tick;             /*   248     8 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	s64                        ntp_error;            /*   256     8 */
	u32                        ntp_error_shift;      /*   264     4 */
	u32                        ntp_err_mult;         /*   268     4 */
	u32                        skip_second_overflow; /*   272     4 */

	/* size: 280, cachelines: 5, members: 23 */
	/* sum members: 269, holes: 1, sum holes: 7 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct audit_ntp_val {
	long long int              oldval;               /*     0     8 */
	long long int              newval;               /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct audit_ntp_data {
	struct audit_ntp_val       vals[6];              /*     0    96 */

	/* size: 96, cachelines: 2, members: 1 */
	/* last cacheline: 32 bytes */
};
struct tk_fast {
	seqcount_latch_t           seq;                  /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct tk_read_base        base[2];              /*     8   112 */

	/* size: 120, cachelines: 2, members: 2 */
	/* sum members: 116, holes: 1, sum holes: 4 */
	/* last cacheline: 56 bytes */
};
struct tick_sched {
	struct hrtimer             sched_timer __attribute__((__aligned__(8))); /*     0    96 */
	/* --- cacheline 1 boundary (64 bytes) was 32 bytes ago --- */
	long unsigned int          check_clocks;         /*    96     8 */
	enum tick_nohz_mode        nohz_mode;            /*   104     4 */
	unsigned int               inidle:1;             /*   108: 0  4 */
	unsigned int               tick_stopped:1;       /*   108: 1  4 */
	unsigned int               idle_active:1;        /*   108: 2  4 */
	unsigned int               do_timer_last:1;      /*   108: 3  4 */
	unsigned int               got_idle_tick:1;      /*   108: 4  4 */

	/* XXX 27 bits hole, try to pack */

	ktime_t                    last_tick;            /*   112     8 */
	ktime_t                    next_tick;            /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	long unsigned int          idle_jiffies;         /*   128     8 */
	long unsigned int          idle_calls;           /*   136     8 */
	long unsigned int          idle_sleeps;          /*   144     8 */
	ktime_t                    idle_entrytime;       /*   152     8 */
	ktime_t                    idle_waketime;        /*   160     8 */
	ktime_t                    idle_exittime;        /*   168     8 */
	ktime_t                    idle_sleeptime;       /*   176     8 */
	ktime_t                    iowait_sleeptime;     /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	long unsigned int          last_jiffies;         /*   192     8 */
	u64                        timer_expires;        /*   200     8 */
	u64                        timer_expires_base;   /*   208     8 */
	u64                        next_timer;           /*   216     8 */
	ktime_t                    idle_expires;         /*   224     8 */
	atomic_t                   tick_dep_mask;        /*   232     4 */

	/* size: 240, cachelines: 4, members: 24 */
	/* sum members: 232 */
	/* sum bitfield members: 5 bits, bit holes: 1, sum bit holes: 27 bits */
	/* padding: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct timer_list_iter {
	int                        cpu;                  /*     0     4 */
	bool                       second_pass;          /*     4     1 */

	/* XXX 3 bytes hole, try to pack */

	u64                        now;                  /*     8     8 */

	/* size: 16, cachelines: 1, members: 3 */
	/* sum members: 13, holes: 1, sum holes: 3 */
	/* last cacheline: 16 bytes */
};
struct tm {
	int                        tm_sec;               /*     0     4 */
	int                        tm_min;               /*     4     4 */
	int                        tm_hour;              /*     8     4 */
	int                        tm_mday;              /*    12     4 */
	int                        tm_mon;               /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	long int                   tm_year;              /*    24     8 */
	int                        tm_wday;              /*    32     4 */
	int                        tm_yday;              /*    36     4 */

	/* size: 40, cachelines: 1, members: 8 */
	/* sum members: 36, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct cyclecounter {
	u64                        (*read)(const struct cyclecounter  *); /*     0     8 */
	u64                        mask;                 /*     8     8 */
	u32                        mult;                 /*    16     4 */
	u32                        shift;                /*    20     4 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct timecounter {
	const struct cyclecounter  * cc;                 /*     0     8 */
	u64                        cycle_last;           /*     8     8 */
	u64                        nsec;                 /*    16     8 */
	u64                        mask;                 /*    24     8 */
	u64                        frac;                 /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct rtc_wkalrm {
	unsigned char              enabled;              /*     0     1 */
	unsigned char              pending;              /*     1     1 */

	/* XXX 2 bytes hole, try to pack */

	struct rtc_time            time;                 /*     4    36 */

	/* size: 40, cachelines: 1, members: 3 */
	/* sum members: 38, holes: 1, sum holes: 2 */
	/* last cacheline: 40 bytes */
};
struct alarm {
	struct timerqueue_node     node __attribute__((__aligned__(8))); /*     0    32 */
	struct hrtimer             timer __attribute__((__aligned__(8))); /*    32    96 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	enum alarmtimer_restart    (*function)(struct alarm *, ktime_t); /*   128     8 */
	enum alarmtimer_type       type;                 /*   136     4 */
	int                        state;                /*   140     4 */
	void *                     data;                 /*   144     8 */

	/* size: 152, cachelines: 3, members: 6 */
	/* forced alignments: 2 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct cpu_timer {
	struct timerqueue_node     node __attribute__((__aligned__(8))); /*     0    32 */
	struct timerqueue_head *   head;                 /*    32     8 */
	struct pid *               pid;                  /*    40     8 */
	struct list_head           elist;                /*    48    16 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        firing;               /*    64     4 */

	/* size: 72, cachelines: 2, members: 5 */
	/* padding: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 8 bytes */
} __attribute__((__aligned__(8)));
struct k_itimer {
	struct list_head           list;                 /*     0    16 */
	struct hlist_node          t_hash;               /*    16    16 */
	spinlock_t                 it_lock;              /*    32     4 */

	/* XXX 4 bytes hole, try to pack */

	const struct k_clock  *    kclock;               /*    40     8 */
	clockid_t                  it_clock;             /*    48     4 */
	timer_t                    it_id;                /*    52     4 */
	int                        it_active;            /*    56     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	s64                        it_overrun;           /*    64     8 */
	s64                        it_overrun_last;      /*    72     8 */
	int                        it_requeue_pending;   /*    80     4 */
	int                        it_sigev_notify;      /*    84     4 */
	ktime_t                    it_interval;          /*    88     8 */
	struct signal_struct *     it_signal;            /*    96     8 */
	union {
		struct pid *       it_pid;               /*   104     8 */
		struct task_struct * it_process;         /*   104     8 */
	};                                               /*   104     8 */
	struct sigqueue *          sigq;                 /*   112     8 */
	union {
		struct {
			struct hrtimer timer __attribute__((__aligned__(8))); /*   120    96 */
		} __attribute__((__aligned__(8))) real __attribute__((__aligned__(8)));  /*   120    96 */
		struct cpu_timer   cpu __attribute__((__aligned__(8))); /*   120    72 */
		struct {
			struct alarm alarmtimer __attribute__((__aligned__(8))); /*   120   152 */
		} __attribute__((__aligned__(8))) alarm __attribute__((__aligned__(8))); /*   120   152 */
	} it __attribute__((__aligned__(8)));            /*   120   152 */
	/* --- cacheline 4 boundary (256 bytes) was 16 bytes ago --- */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*   272    16 */

	/* size: 288, cachelines: 5, members: 17 */
	/* sum members: 280, holes: 2, sum holes: 8 */
	/* forced alignments: 2 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct k_clock {
	int                        (*clock_getres)(const clockid_t , struct timespec64 *); /*     0     8 */
	int                        (*clock_set)(const clockid_t , const struct timespec64  *); /*     8     8 */
	int                        (*clock_get_timespec)(const clockid_t , struct timespec64 *); /*    16     8 */
	ktime_t                    (*clock_get_ktime)(const clockid_t ); /*    24     8 */
	int                        (*clock_adj)(const clockid_t , struct __kernel_timex *); /*    32     8 */
	int                        (*timer_create)(struct k_itimer *); /*    40     8 */
	int                        (*nsleep)(const clockid_t , int, const struct timespec64  *); /*    48     8 */
	int                        (*timer_set)(struct k_itimer *, int, struct itimerspec64 *, struct itimerspec64 *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*timer_del)(struct k_itimer *); /*    64     8 */
	void                       (*timer_get)(struct k_itimer *, struct itimerspec64 *); /*    72     8 */
	void                       (*timer_rearm)(struct k_itimer *); /*    80     8 */
	s64                        (*timer_forward)(struct k_itimer *, ktime_t); /*    88     8 */
	ktime_t                    (*timer_remaining)(struct k_itimer *, ktime_t); /*    96     8 */
	int                        (*timer_try_to_cancel)(struct k_itimer *); /*   104     8 */
	void                       (*timer_arm)(struct k_itimer *, ktime_t, bool, bool); /*   112     8 */
	void                       (*timer_wait_running)(struct k_itimer *); /*   120     8 */

	/* size: 128, cachelines: 2, members: 16 */
};
struct class_interface {
	struct list_head           node;                 /*     0    16 */
	struct class *             class;                /*    16     8 */
	int                        (*add_dev)(struct device *, struct class_interface *); /*    24     8 */
	void                       (*remove_dev)(struct device *, struct class_interface *); /*    32     8 */

	/* size: 40, cachelines: 1, members: 4 */
	/* last cacheline: 40 bytes */
};
struct rtc_class_ops {
	int                        (*ioctl)(struct device *, unsigned int, long unsigned int); /*     0     8 */
	int                        (*read_time)(struct device *, struct rtc_time *); /*     8     8 */
	int                        (*set_time)(struct device *, struct rtc_time *); /*    16     8 */
	int                        (*read_alarm)(struct device *, struct rtc_wkalrm *); /*    24     8 */
	int                        (*set_alarm)(struct device *, struct rtc_wkalrm *); /*    32     8 */
	int                        (*proc)(struct device *, struct seq_file *); /*    40     8 */
	int                        (*alarm_irq_enable)(struct device *, unsigned int); /*    48     8 */
	int                        (*read_offset)(struct device *, long int *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*set_offset)(struct device *, long int); /*    64     8 */

	/* size: 72, cachelines: 2, members: 9 */
	/* last cacheline: 8 bytes */
};
struct rtc_timer {
	struct timerqueue_node     node __attribute__((__aligned__(8))); /*     0    32 */
	ktime_t                    period;               /*    32     8 */
	void                       (*func)(struct rtc_device *); /*    40     8 */
	struct rtc_device *        rtc;                  /*    48     8 */
	int                        enabled;              /*    56     4 */

	/* size: 64, cachelines: 1, members: 5 */
	/* padding: 4 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(8)));
struct rtc_device {
	struct device              dev __attribute__((__aligned__(8))); /*     0  1000 */
	/* --- cacheline 15 boundary (960 bytes) was 40 bytes ago --- */
	struct module *            owner;                /*  1000     8 */
	int                        id;                   /*  1008     4 */

	/* XXX 4 bytes hole, try to pack */

	const struct rtc_class_ops  * ops;               /*  1016     8 */
	/* --- cacheline 16 boundary (1024 bytes) --- */
	struct mutex               ops_lock;             /*  1024    32 */
	struct cdev                char_dev;             /*  1056   136 */
	/* --- cacheline 18 boundary (1152 bytes) was 40 bytes ago --- */
	long unsigned int          flags;                /*  1192     8 */
	long unsigned int          irq_data;             /*  1200     8 */
	spinlock_t                 irq_lock;             /*  1208     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 19 boundary (1216 bytes) --- */
	wait_queue_head_t          irq_queue;            /*  1216    24 */
	struct fasync_struct *     async_queue;          /*  1240     8 */
	int                        irq_freq;             /*  1248     4 */
	int                        max_user_freq;        /*  1252     4 */
	struct timerqueue_head     timerqueue;           /*  1256    16 */
	struct rtc_timer           aie_timer __attribute__((__aligned__(8))); /*  1272    64 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 20 boundary (1280 bytes) was 56 bytes ago --- */
	struct rtc_timer           uie_rtctimer __attribute__((__aligned__(8))); /*  1336    64 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 21 boundary (1344 bytes) was 56 bytes ago --- */
	struct hrtimer             pie_timer __attribute__((__aligned__(8))); /*  1400    96 */
	/* --- cacheline 23 boundary (1472 bytes) was 24 bytes ago --- */
	int                        pie_enabled;          /*  1496     4 */

	/* XXX 4 bytes hole, try to pack */

	struct work_struct         irqwork;              /*  1504    64 */
	/* --- cacheline 24 boundary (1536 bytes) was 32 bytes ago --- */
	int                        uie_unsupported;      /*  1568     4 */

	/* XXX 4 bytes hole, try to pack */

	long int                   set_offset_nsec;      /*  1576     8 */
	bool                       registered;           /*  1584     1 */
	bool                       nvram_old_abi;        /*  1585     1 */

	/* XXX 6 bytes hole, try to pack */

	struct bin_attribute *     nvram;                /*  1592     8 */
	/* --- cacheline 25 boundary (1600 bytes) --- */
	time64_t                   range_min;            /*  1600     8 */
	timeu64_t                  range_max;            /*  1608     8 */
	time64_t                   start_secs;           /*  1616     8 */
	time64_t                   offset_secs;          /*  1624     8 */
	bool                       set_start_time;       /*  1632     1 */

	/* size: 1640, cachelines: 26, members: 29 */
	/* sum members: 1611, holes: 5, sum holes: 22 */
	/* padding: 7 */
	/* paddings: 2, sum paddings: 8 */
	/* forced alignments: 4 */
	/* last cacheline: 40 bytes */
} __attribute__((__aligned__(8)));
struct platform_driver {
	int                        (*probe)(struct platform_device *); /*     0     8 */
	int                        (*remove)(struct platform_device *); /*     8     8 */
	void                       (*shutdown)(struct platform_device *); /*    16     8 */
	int                        (*suspend)(struct platform_device *, pm_message_t); /*    24     8 */
	int                        (*resume)(struct platform_device *); /*    32     8 */
	struct device_driver       driver;               /*    40   192 */
	/* --- cacheline 3 boundary (192 bytes) was 40 bytes ago --- */
	const struct platform_device_id  * id_table;     /*   232     8 */
	bool                       prevent_deferred_probe; /*   240     1 */

	/* size: 248, cachelines: 4, members: 8 */
	/* padding: 7 */
	/* last cacheline: 56 bytes */
};
struct trace_event_raw_alarmtimer_suspend {
	struct trace_entry         ent;                  /*     0     8 */
	s64                        expires;              /*     8     8 */
	unsigned char              alarm_type;           /*    16     1 */
	char                       __data[];             /*    17     0 */

	/* size: 24, cachelines: 1, members: 4 */
	/* padding: 7 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_alarm_class {
	struct trace_entry         ent;                  /*     0     8 */
	void *                     alarm;                /*     8     8 */
	unsigned char              alarm_type;           /*    16     1 */

	/* XXX 7 bytes hole, try to pack */

	s64                        expires;              /*    24     8 */
	s64                        now;                  /*    32     8 */
	char                       __data[];             /*    40     0 */

	/* size: 40, cachelines: 1, members: 6 */
	/* sum members: 33, holes: 1, sum holes: 7 */
	/* last cacheline: 40 bytes */
};
struct trace_event_data_offsets_alarmtimer_suspend {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_alarm_class {

	/* size: 0, cachelines: 0, members: 0 */
};
struct alarm_base {
	spinlock_t                 lock;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct timerqueue_head     timerqueue;           /*     8    16 */
	ktime_t                    (*get_ktime)(void);   /*    24     8 */
	void                       (*get_timespec)(struct timespec64 *); /*    32     8 */
	clockid_t                  base_clockid;         /*    40     4 */

	/* size: 48, cachelines: 1, members: 5 */
	/* sum members: 40, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 48 bytes */
};
struct sigevent {
	sigval_t                   sigev_value;          /*     0     8 */
	int                        sigev_signo;          /*     8     4 */
	int                        sigev_notify;         /*    12     4 */
	union {
		int                _pad[12];             /*    16    48 */
		int                _tid;                 /*    16     4 */
		struct {
			void       (*_function)(sigval_t); /*    16     8 */
			void *     _attribute;           /*    24     8 */
		} _sigev_thread;                         /*    16    16 */
	} _sigev_un;                                     /*    16    48 */

	/* size: 64, cachelines: 1, members: 4 */
};
struct compat_sigevent {
	compat_sigval_t            sigev_value;          /*     0     4 */
	compat_int_t               sigev_signo;          /*     4     4 */
	compat_int_t               sigev_notify;         /*     8     4 */
	union {
		compat_int_t       _pad[13];             /*    12    52 */
		compat_int_t       _tid;                 /*    12     4 */
		struct {
			compat_uptr_t _function;         /*    12     4 */
			compat_uptr_t _attribute;        /*    16     4 */
		} _sigev_thread;                         /*    12     8 */
	} _sigev_un;                                     /*    12    52 */

	/* size: 64, cachelines: 1, members: 4 */
};
struct posix_clock_operations {
	struct module *            owner;                /*     0     8 */
	int                        (*clock_adjtime)(struct posix_clock *, struct __kernel_timex *); /*     8     8 */
	int                        (*clock_gettime)(struct posix_clock *, struct timespec64 *); /*    16     8 */
	int                        (*clock_getres)(struct posix_clock *, struct timespec64 *); /*    24     8 */
	int                        (*clock_settime)(struct posix_clock *, const struct timespec64  *); /*    32     8 */
	long int                   (*ioctl)(struct posix_clock *, unsigned int, long unsigned int); /*    40     8 */
	int                        (*open)(struct posix_clock *, fmode_t); /*    48     8 */
	__poll_t                   (*poll)(struct posix_clock *, struct file *, poll_table *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*release)(struct posix_clock *); /*    64     8 */
	ssize_t                    (*read)(struct posix_clock *, uint, char *, size_t); /*    72     8 */
	u64                        kabi_reserved1;       /*    80     8 */
	u64                        kabi_reserved2;       /*    88     8 */
	u64                        kabi_reserved3;       /*    96     8 */
	u64                        kabi_reserved4;       /*   104     8 */
	u64                        kabi_reserved5;       /*   112     8 */
	u64                        kabi_reserved6;       /*   120     8 */

	/* size: 128, cachelines: 2, members: 16 */
};
struct posix_clock {
	struct posix_clock_operations ops;               /*     0   128 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct cdev                cdev;                 /*   128   136 */
	/* --- cacheline 4 boundary (256 bytes) was 8 bytes ago --- */
	struct device *            dev;                  /*   264     8 */
	struct rw_semaphore        rwsem;                /*   272    40 */
	bool                       zombie;               /*   312     1 */

	/* XXX 7 bytes hole, try to pack */

	/* --- cacheline 5 boundary (320 bytes) --- */
	u64                        kabi_reserved1;       /*   320     8 */
	u64                        kabi_reserved2;       /*   328     8 */
	u64                        kabi_reserved3;       /*   336     8 */
	u64                        kabi_reserved4;       /*   344     8 */
	u64                        kabi_reserved5;       /*   352     8 */
	u64                        kabi_reserved6;       /*   360     8 */
	u64                        kabi_reserved7;       /*   368     8 */
	u64                        kabi_reserved8;       /*   376     8 */

	/* size: 384, cachelines: 6, members: 13 */
	/* sum members: 377, holes: 1, sum holes: 7 */
};
struct posix_clock_desc {
	struct file *              fp;                   /*     0     8 */
	struct posix_clock *       clk;                  /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct __kernel_old_itimerval {
	struct __kernel_old_timeval it_interval;         /*     0    16 */
	struct __kernel_old_timeval it_value;            /*    16    16 */

	/* size: 32, cachelines: 1, members: 2 */
	/* last cacheline: 32 bytes */
};
struct old_itimerval32 {
	struct old_timeval32       it_interval;          /*     0     8 */
	struct old_timeval32       it_value;             /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct ce_unbind {
	struct clock_event_device * ce;                  /*     0     8 */
	int                        res;                  /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct proc_timens_offset {
	int                        clockid;              /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct timespec64          val;                  /*     8    16 */

	/* size: 24, cachelines: 1, members: 2 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct futex_pi_state {
	struct list_head           list;                 /*     0    16 */
	struct rt_mutex            pi_mutex;             /*    16    32 */
	struct task_struct *       owner;                /*    48     8 */
	refcount_t                 refcount;             /*    56     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	union futex_key            key;                  /*    64    24 */

	/* size: 88, cachelines: 2, members: 5 */
	/* sum members: 84, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
union futex_key {
	struct {
		u64                i_seq;              /*     0     8 */
		long unsigned int  pgoff;              /*     8     8 */
		unsigned int       offset;             /*    16     4 */
	} shared;                                      /*     0    24 */
	struct {
		union {
			struct mm_struct * mm;         /*     0     8 */
			u64        __tmp;              /*     0     8 */
		};                                     /*     0     8 */
		long unsigned int  address;            /*     8     8 */
		unsigned int       offset;             /*    16     4 */
	} private;                                     /*     0    24 */
	struct {
		u64                ptr;                /*     0     8 */
		long unsigned int  word;               /*     8     8 */
		unsigned int       offset;             /*    16     4 */
	} both;                                        /*     0    24 */
};
struct futex_q {
	struct plist_node          list;                 /*     0    40 */
	struct task_struct *       task;                 /*    40     8 */
	spinlock_t *               lock_ptr;             /*    48     8 */
	union futex_key            key;                  /*    56    24 */
	/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
	struct futex_pi_state *    pi_state;             /*    80     8 */
	struct rt_mutex_waiter *   rt_waiter;            /*    88     8 */
	union futex_key *          requeue_pi_key;       /*    96     8 */
	u32                        bitset;               /*   104     4 */

	/* size: 112, cachelines: 2, members: 8 */
	/* padding: 4 */
	/* last cacheline: 48 bytes */
};
struct futex_hash_bucket {
	atomic_t                   waiters;              /*     0     4 */
	spinlock_t                 lock;                 /*     4     4 */
	struct plist_head          chain;                /*     8    16 */

	/* size: 64, cachelines: 1, members: 3 */
	/* padding: 40 */
} __attribute__((__aligned__(64)));
struct dma_chan {
	int                        lock;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	const char  *              device_id;            /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct call_function_data {
	call_single_data_t *       csd;                  /*     0     8 */
	cpumask_var_t              cpumask;              /*     8     8 */
	cpumask_var_t              cpumask_ipi;          /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct smp_call_on_cpu_struct {
	struct work_struct         work;                 /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct completion          done;                 /*    64    32 */
	int                        (*func)(void *);      /*    96     8 */
	void *                     data;                 /*   104     8 */
	int                        ret;                  /*   112     4 */
	int                        cpu;                  /*   116     4 */

	/* size: 120, cachelines: 2, members: 6 */
	/* last cacheline: 56 bytes */
};
struct latch_tree_root {
	seqcount_latch_t           seq;                  /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct rb_root             tree[2];              /*     8    16 */

	/* size: 24, cachelines: 1, members: 2 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct latch_tree_ops {
	bool                       (*less)(struct latch_tree_node *, struct latch_tree_node *); /*     0     8 */
	int                        (*comp)(void *, struct latch_tree_node *); /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct modversion_info {
	long unsigned int          crc;                  /*     0     8 */
	char                       name[56];             /*     8    56 */

	/* size: 64, cachelines: 1, members: 2 */
};
struct module_use {
	struct list_head           source_list;          /*     0    16 */
	struct list_head           target_list;          /*    16    16 */
	struct module *            source;               /*    32     8 */
	struct module *            target;               /*    40     8 */

	/* size: 48, cachelines: 1, members: 4 */
	/* last cacheline: 48 bytes */
};
struct module_sect_attrs {
	struct attribute_group     grp;                  /*     0    40 */
	unsigned int               nsections;            /*    40     4 */

	/* XXX 4 bytes hole, try to pack */

	struct module_sect_attr    attrs[];              /*    48     0 */

	/* size: 48, cachelines: 1, members: 3 */
	/* sum members: 44, holes: 1, sum holes: 4 */
	/* last cacheline: 48 bytes */
};
struct module_notes_attrs {
	struct kobject *           dir;                  /*     0     8 */
	unsigned int               notes;                /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct bin_attribute       attrs[];              /*    16     0 */

	/* size: 16, cachelines: 1, members: 3 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct symsearch {
	const struct kernel_symbol  * start;             /*     0     8 */
	const struct kernel_symbol  * stop;              /*     8     8 */
	const s32  *               crcs;                 /*    16     8 */
	enum mod_license           license;              /*    24     4 */
	bool                       unused;               /*    28     1 */

	/* size: 32, cachelines: 1, members: 5 */
	/* padding: 3 */
	/* last cacheline: 32 bytes */
};
struct load_info {
	const char  *              name;                 /*     0     8 */
	struct module *            mod;                  /*     8     8 */
	Elf64_Ehdr *               hdr;                  /*    16     8 */
	long unsigned int          len;                  /*    24     8 */
	Elf64_Shdr *               sechdrs;              /*    32     8 */
	char *                     secstrings;           /*    40     8 */
	char *                     strtab;               /*    48     8 */
	long unsigned int          symoffs;              /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	long unsigned int          stroffs;              /*    64     8 */
	long unsigned int          init_typeoffs;        /*    72     8 */
	long unsigned int          core_typeoffs;        /*    80     8 */
	struct _ddebug *           debug;                /*    88     8 */
	unsigned int               num_debug;            /*    96     4 */
	bool                       sig_ok;               /*   100     1 */

	/* XXX 3 bytes hole, try to pack */

	long unsigned int          mod_kallsyms_init_off; /*   104     8 */
	struct {
		unsigned int       sym;                  /*   112     4 */
		unsigned int       str;                  /*   116     4 */
		unsigned int       mod;                  /*   120     4 */
		unsigned int       vers;                 /*   124     4 */
		/* --- cacheline 2 boundary (128 bytes) --- */
		unsigned int       info;                 /*   128     4 */
		unsigned int       pcpu;                 /*   132     4 */
	} index;                                         /*   112    24 */

	/* size: 136, cachelines: 3, members: 16 */
	/* sum members: 133, holes: 1, sum holes: 3 */
	/* last cacheline: 8 bytes */
};
struct trace_event_raw_module_load {
	struct trace_entry         ent;                  /*     0     8 */
	unsigned int               taints;               /*     8     4 */
	u32                        __data_loc_name;      /*    12     4 */
	char                       __data[];             /*    16     0 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct trace_event_raw_module_free {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        __data_loc_name;      /*     8     4 */
	char                       __data[];             /*    12     0 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct trace_event_raw_module_refcnt {
	struct trace_entry         ent;                  /*     0     8 */
	long unsigned int          ip;                   /*     8     8 */
	int                        refcnt;               /*    16     4 */
	u32                        __data_loc_name;      /*    20     4 */
	char                       __data[];             /*    24     0 */

	/* size: 24, cachelines: 1, members: 5 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_module_request {
	struct trace_entry         ent;                  /*     0     8 */
	long unsigned int          ip;                   /*     8     8 */
	bool                       wait;                 /*    16     1 */

	/* XXX 3 bytes hole, try to pack */

	u32                        __data_loc_name;      /*    20     4 */
	char                       __data[];             /*    24     0 */

	/* size: 24, cachelines: 1, members: 5 */
	/* sum members: 21, holes: 1, sum holes: 3 */
	/* last cacheline: 24 bytes */
};
struct trace_event_data_offsets_module_load {
	u32                        name;                 /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct trace_event_data_offsets_module_free {
	u32                        name;                 /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct trace_event_data_offsets_module_refcnt {
	u32                        name;                 /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct trace_event_data_offsets_module_request {
	u32                        name;                 /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct mod_tree_root {
	struct latch_tree_root     root;                 /*     0    24 */
	long unsigned int          addr_min;             /*    24     8 */
	long unsigned int          addr_max;             /*    32     8 */

	/* size: 40, cachelines: 1, members: 3 */
	/* last cacheline: 40 bytes */
};
struct find_symbol_arg {
	const char  *              name;                 /*     0     8 */
	bool                       gplok;                /*     8     1 */
	bool                       warn;                 /*     9     1 */

	/* XXX 6 bytes hole, try to pack */

	struct module *            owner;                /*    16     8 */
	const s32  *               crc;                  /*    24     8 */
	const struct kernel_symbol  * sym;               /*    32     8 */
	enum mod_license           license;              /*    40     4 */

	/* size: 48, cachelines: 1, members: 7 */
	/* sum members: 38, holes: 1, sum holes: 6 */
	/* padding: 4 */
	/* last cacheline: 48 bytes */
};
struct module_sect_attr {
	struct bin_attribute       battr;                /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	long unsigned int          address;              /*    64     8 */

	/* size: 72, cachelines: 2, members: 2 */
	/* last cacheline: 8 bytes */
};
struct mod_initfree {
	struct llist_node          node;                 /*     0     8 */
	void *                     module_init;          /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct module_signature {
	u8                         algo;                 /*     0     1 */
	u8                         hash;                 /*     1     1 */
	u8                         id_type;              /*     2     1 */
	u8                         signer_len;           /*     3     1 */
	u8                         key_id_len;           /*     4     1 */
	u8                         __pad[3];             /*     5     3 */
	__be32                     sig_len;              /*     8     4 */

	/* size: 12, cachelines: 1, members: 7 */
	/* last cacheline: 12 bytes */
};
struct kallsym_iter {
	loff_t                     pos;                  /*     0     8 */
	loff_t                     pos_arch_end;         /*     8     8 */
	loff_t                     pos_mod_end;          /*    16     8 */
	loff_t                     pos_ftrace_mod_end;   /*    24     8 */
	loff_t                     pos_bpf_end;          /*    32     8 */
	long unsigned int          value;                /*    40     8 */
	unsigned int               nameoff;              /*    48     4 */
	char                       type;                 /*    52     1 */
	char                       name[128];            /*    53   128 */
	/* --- cacheline 2 boundary (128 bytes) was 53 bytes ago --- */
	char                       module_name[56];      /*   181    56 */

	/* XXX 3 bytes hole, try to pack */

	/* --- cacheline 3 boundary (192 bytes) was 48 bytes ago --- */
	int                        exported;             /*   240     4 */
	int                        show_value;           /*   244     4 */

	/* size: 248, cachelines: 4, members: 12 */
	/* sum members: 245, holes: 1, sum holes: 3 */
	/* last cacheline: 56 bytes */
};
struct acct_v3 {
	char                       ac_flag;              /*     0     1 */
	char                       ac_version;           /*     1     1 */
	__u16                      ac_tty;               /*     2     2 */
	__u32                      ac_exitcode;          /*     4     4 */
	__u32                      ac_uid;               /*     8     4 */
	__u32                      ac_gid;               /*    12     4 */
	__u32                      ac_pid;               /*    16     4 */
	__u32                      ac_ppid;              /*    20     4 */
	__u32                      ac_btime;             /*    24     4 */
	__u32                      ac_etime;             /*    28     4 */
	comp_t                     ac_utime;             /*    32     2 */
	comp_t                     ac_stime;             /*    34     2 */
	comp_t                     ac_mem;               /*    36     2 */
	comp_t                     ac_io;                /*    38     2 */
	comp_t                     ac_rw;                /*    40     2 */
	comp_t                     ac_minflt;            /*    42     2 */
	comp_t                     ac_majflt;            /*    44     2 */
	comp_t                     ac_swaps;             /*    46     2 */
	char                       ac_comm[16];          /*    48    16 */

	/* size: 64, cachelines: 1, members: 19 */
};
struct fs_pin {
	wait_queue_head_t          wait;                 /*     0    24 */
	int                        done;                 /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	struct hlist_node          s_list;               /*    32    16 */
	struct hlist_node          m_list;               /*    48    16 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	void                       (*kill)(struct fs_pin *); /*    64     8 */

	/* size: 72, cachelines: 2, members: 5 */
	/* sum members: 68, holes: 1, sum holes: 4 */
	/* last cacheline: 8 bytes */
};
struct bsd_acct_struct {
	struct fs_pin              pin;                  /*     0    72 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	atomic_long_t              count;                /*    72     8 */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*    80    16 */
	struct mutex               lock;                 /*    96    32 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	int                        active;               /*   128     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          needcheck;            /*   136     8 */
	struct file *              file;                 /*   144     8 */
	struct pid_namespace *     ns;                   /*   152     8 */
	struct work_struct         work;                 /*   160    64 */
	/* --- cacheline 3 boundary (192 bytes) was 32 bytes ago --- */
	struct completion          done;                 /*   224    32 */

	/* size: 256, cachelines: 4, members: 10 */
	/* sum members: 252, holes: 1, sum holes: 4 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(8)));
struct elf64_note {
	Elf64_Word                 n_namesz;             /*     0     4 */
	Elf64_Word                 n_descsz;             /*     4     4 */
	Elf64_Word                 n_type;               /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct elf_note_section {
	struct elf64_note          n_hdr;                /*     0    12 */
	u8                         n_data[];             /*    12     0 */

	/* size: 12, cachelines: 1, members: 2 */
	/* last cacheline: 12 bytes */
};
struct elf_siginfo {
	int                        si_signo;             /*     0     4 */
	int                        si_code;              /*     4     4 */
	int                        si_errno;             /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct elf_prstatus {
	struct elf_siginfo         pr_info;              /*     0    12 */
	short int                  pr_cursig;            /*    12     2 */

	/* XXX 2 bytes hole, try to pack */

	long unsigned int          pr_sigpend;           /*    16     8 */
	long unsigned int          pr_sighold;           /*    24     8 */
	pid_t                      pr_pid;               /*    32     4 */
	pid_t                      pr_ppid;              /*    36     4 */
	pid_t                      pr_pgrp;              /*    40     4 */
	pid_t                      pr_sid;               /*    44     4 */
	struct __kernel_old_timeval pr_utime;            /*    48    16 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct __kernel_old_timeval pr_stime;            /*    64    16 */
	struct __kernel_old_timeval pr_cutime;           /*    80    16 */
	struct __kernel_old_timeval pr_cstime;           /*    96    16 */
	elf_gregset_t              pr_reg;               /*   112   216 */
	/* --- cacheline 5 boundary (320 bytes) was 8 bytes ago --- */
	int                        pr_fpvalid;           /*   328     4 */

	/* size: 336, cachelines: 6, members: 14 */
	/* sum members: 330, holes: 1, sum holes: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct compat_kexec_segment {
	compat_uptr_t              buf;                  /*     0     4 */
	compat_size_t              bufsz;                /*     4     4 */
	compat_ulong_t             mem;                  /*     8     4 */
	compat_size_t              memsz;                /*    12     4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct elf64_phdr {
	Elf64_Word                 p_type;               /*     0     4 */
	Elf64_Word                 p_flags;              /*     4     4 */
	Elf64_Off                  p_offset;             /*     8     8 */
	Elf64_Addr                 p_vaddr;              /*    16     8 */
	Elf64_Addr                 p_paddr;              /*    24     8 */
	Elf64_Xword                p_filesz;             /*    32     8 */
	Elf64_Xword                p_memsz;              /*    40     8 */
	Elf64_Xword                p_align;              /*    48     8 */

	/* size: 56, cachelines: 1, members: 8 */
	/* last cacheline: 56 bytes */
};
struct kexec_sha_region {
	long unsigned int          start;                /*     0     8 */
	long unsigned int          len;                  /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct bpf_link_info {
	__u32                      type;                 /*     0     4 */
	__u32                      id;                   /*     4     4 */
	__u32                      prog_id;              /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	union {
		struct {
			__u64      tp_name __attribute__((__aligned__(8))); /*    16     8 */
			__u32      tp_name_len;          /*    24     4 */
		} __attribute__((__aligned__(8))) raw_tracepoint __attribute__((__aligned__(8)));        /*    16    16 */
		struct {
			__u32      attach_type;          /*    16     4 */
		} tracing;                               /*    16     4 */
		struct {
			__u64      cgroup_id;            /*    16     8 */
			__u32      attach_type;          /*    24     4 */
		} cgroup;                                /*    16    16 */
		struct {
			__u64      target_name __attribute__((__aligned__(8))); /*    16     8 */
			__u32      target_name_len;      /*    24     4 */
			union {
				struct {
					__u32  map_id;   /*    28     4 */
				} map;                   /*    28     4 */
			};                               /*    28     4 */
		} __attribute__((__aligned__(8))) iter __attribute__((__aligned__(8)));  /*    16    16 */
		struct {
			__u32      netns_ino;            /*    16     4 */
			__u32      attach_type;          /*    20     4 */
		} netns;                                 /*    16     8 */
		struct {
			__u32      ifindex;              /*    16     4 */
		} xdp;                                   /*    16     4 */
	} __attribute__((__aligned__(8)));               /*    16    16 */

	/* size: 32, cachelines: 1, members: 4 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 4 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct bpf_link {
	atomic64_t                 refcnt;               /*     0     8 */
	u32                        id;                   /*     8     4 */
	enum bpf_link_type         type;                 /*    12     4 */
	const struct bpf_link_ops  * ops;                /*    16     8 */
	struct bpf_prog *          prog;                 /*    24     8 */
	struct work_struct         work;                 /*    32    64 */

	/* size: 96, cachelines: 2, members: 6 */
	/* last cacheline: 32 bytes */
};
struct bpf_link_ops {
	void                       (*release)(struct bpf_link *); /*     0     8 */
	void                       (*dealloc)(struct bpf_link *); /*     8     8 */
	int                        (*detach)(struct bpf_link *); /*    16     8 */
	int                        (*update_prog)(struct bpf_link *, struct bpf_prog *, struct bpf_prog *); /*    24     8 */
	void                       (*show_fdinfo)(const struct bpf_link  *, struct seq_file *); /*    32     8 */
	int                        (*fill_link_info)(const struct bpf_link  *, struct bpf_link_info *); /*    40     8 */

	/* size: 48, cachelines: 1, members: 6 */
	/* last cacheline: 48 bytes */
};
struct bpf_cgroup_link {
	struct bpf_link            link;                 /*     0    96 */
	/* --- cacheline 1 boundary (64 bytes) was 32 bytes ago --- */
	struct cgroup *            cgroup;               /*    96     8 */
	enum bpf_attach_type       type;                 /*   104     4 */

	/* size: 112, cachelines: 2, members: 3 */
	/* padding: 4 */
	/* last cacheline: 48 bytes */
};
struct cgroup_taskset {
	struct list_head           src_csets;            /*     0    16 */
	struct list_head           dst_csets;            /*    16    16 */
	int                        nr_tasks;             /*    32     4 */
	int                        ssid;                 /*    36     4 */
	struct list_head *         csets;                /*    40     8 */
	struct css_set *           cur_cset;             /*    48     8 */
	struct task_struct *       cur_task;             /*    56     8 */

	/* size: 64, cachelines: 1, members: 7 */
};
struct cgroup_fs_context {
	struct kernfs_fs_context   kfc;                  /*     0    32 */

	/* XXX last struct has 7 bytes of padding */

	struct cgroup_root *       root;                 /*    32     8 */
	struct cgroup_namespace *  ns;                   /*    40     8 */
	unsigned int               flags;                /*    48     4 */
	bool                       cpuset_clone_children; /*    52     1 */
	bool                       none;                 /*    53     1 */
	bool                       all_ss;               /*    54     1 */

	/* XXX 1 byte hole, try to pack */

	u16                        subsys_mask;          /*    56     2 */

	/* XXX 6 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	char *                     name;                 /*    64     8 */
	char *                     release_agent;        /*    72     8 */

	/* size: 80, cachelines: 2, members: 10 */
	/* sum members: 73, holes: 2, sum holes: 7 */
	/* paddings: 1, sum paddings: 7 */
	/* last cacheline: 16 bytes */
};
struct cgroup_file_ctx {
	struct cgroup_namespace *  ns;                   /*     0     8 */
	struct {
		void *             trigger;              /*     8     8 */
	} psi;                                           /*     8     8 */
	struct {
		bool               started;              /*    16     1 */

		/* XXX 7 bytes hole, try to pack */

		struct css_task_iter iter;               /*    24   104 */
	} procs;                                         /*    16   112 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct {
		struct cgroup_pidlist * pidlist;         /*   128     8 */
	} procs1;                                        /*   128     8 */

	/* size: 136, cachelines: 3, members: 4 */
	/* last cacheline: 8 bytes */
};
struct cgrp_cset_link {
	struct cgroup *            cgrp;                 /*     0     8 */
	struct css_set *           cset;                 /*     8     8 */
	struct list_head           cset_link;            /*    16    16 */
	struct list_head           cgrp_link;            /*    32    16 */

	/* size: 48, cachelines: 1, members: 4 */
	/* last cacheline: 48 bytes */
};
struct cgroup_mgctx {
	struct list_head           preloaded_src_csets;  /*     0    16 */
	struct list_head           preloaded_dst_csets;  /*    16    16 */
	struct cgroup_taskset      tset;                 /*    32    64 */
	/* --- cacheline 1 boundary (64 bytes) was 32 bytes ago --- */
	u16                        ss_mask;              /*    96     2 */

	/* size: 104, cachelines: 2, members: 4 */
	/* padding: 6 */
	/* last cacheline: 40 bytes */
};
struct trace_event_raw_cgroup_root {
	struct trace_entry         ent;                  /*     0     8 */
	int                        root;                 /*     8     4 */
	u16                        ss_mask;              /*    12     2 */

	/* XXX 2 bytes hole, try to pack */

	u32                        __data_loc_name;      /*    16     4 */
	char                       __data[];             /*    20     0 */

	/* size: 20, cachelines: 1, members: 5 */
	/* sum members: 18, holes: 1, sum holes: 2 */
	/* last cacheline: 20 bytes */
};
struct trace_event_raw_cgroup {
	struct trace_entry         ent;                  /*     0     8 */
	int                        root;                 /*     8     4 */
	int                        level;                /*    12     4 */
	u64                        id;                   /*    16     8 */
	u32                        __data_loc_path;      /*    24     4 */
	char                       __data[];             /*    28     0 */

	/* size: 32, cachelines: 1, members: 6 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct trace_event_raw_cgroup_migrate {
	struct trace_entry         ent;                  /*     0     8 */
	int                        dst_root;             /*     8     4 */
	int                        dst_level;            /*    12     4 */
	u64                        dst_id;               /*    16     8 */
	int                        pid;                  /*    24     4 */
	u32                        __data_loc_dst_path;  /*    28     4 */
	u32                        __data_loc_comm;      /*    32     4 */
	char                       __data[];             /*    36     0 */

	/* size: 40, cachelines: 1, members: 8 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct trace_event_raw_cgroup_event {
	struct trace_entry         ent;                  /*     0     8 */
	int                        root;                 /*     8     4 */
	int                        level;                /*    12     4 */
	u64                        id;                   /*    16     8 */
	u32                        __data_loc_path;      /*    24     4 */
	int                        val;                  /*    28     4 */
	char                       __data[];             /*    32     0 */

	/* size: 32, cachelines: 1, members: 7 */
	/* last cacheline: 32 bytes */
};
struct trace_event_data_offsets_cgroup_root {
	u32                        name;                 /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct trace_event_data_offsets_cgroup {
	u32                        path;                 /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct trace_event_data_offsets_cgroup_migrate {
	u32                        dst_path;             /*     0     4 */
	u32                        comm;                 /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct trace_event_data_offsets_cgroup_event {
	u32                        path;                 /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct cgroupstats {
	__u64                      nr_sleeping;          /*     0     8 */
	__u64                      nr_running;           /*     8     8 */
	__u64                      nr_stopped;           /*    16     8 */
	__u64                      nr_uninterruptible;   /*    24     8 */
	__u64                      nr_io_wait;           /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct cgroup_pidlist {
	struct {
		enum cgroup_filetype type;               /*     0     4 */

		/* XXX 4 bytes hole, try to pack */

		struct pid_namespace * ns;               /*     8     8 */
	} key;                                           /*     0    16 */
	pid_t *                    list;                 /*    16     8 */
	int                        length;               /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           links;                /*    32    16 */
	struct cgroup *            owner;                /*    48     8 */
	struct delayed_work        destroy_dwork;        /*    56   192 */

	/* size: 248, cachelines: 4, members: 6 */
	/* sum members: 244, holes: 1, sum holes: 4 */
	/* last cacheline: 56 bytes */
};
struct freezer {
	struct cgroup_subsys_state css __attribute__((__aligned__(8))); /*     0   296 */
	/* --- cacheline 4 boundary (256 bytes) was 40 bytes ago --- */
	unsigned int               state;                /*   296     4 */

	/* size: 304, cachelines: 5, members: 2 */
	/* padding: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct pids_cgroup {
	struct cgroup_subsys_state css __attribute__((__aligned__(8))); /*     0   296 */
	/* --- cacheline 4 boundary (256 bytes) was 40 bytes ago --- */
	atomic64_t                 counter;              /*   296     8 */
	atomic64_t                 limit;                /*   304     8 */
	struct cgroup_file         events_file;          /*   312   104 */
	/* --- cacheline 6 boundary (384 bytes) was 32 bytes ago --- */
	atomic64_t                 events_limit;         /*   416     8 */

	/* size: 424, cachelines: 7, members: 5 */
	/* forced alignments: 1 */
	/* last cacheline: 40 bytes */
} __attribute__((__aligned__(8)));
struct rdma_cgroup {
	struct cgroup_subsys_state css __attribute__((__aligned__(8))); /*     0   296 */
	/* --- cacheline 4 boundary (256 bytes) was 40 bytes ago --- */
	struct list_head           rpools;               /*   296    16 */

	/* size: 312, cachelines: 5, members: 2 */
	/* forced alignments: 1 */
	/* last cacheline: 56 bytes */
} __attribute__((__aligned__(8)));
struct rdmacg_device {
	struct list_head           dev_node;             /*     0    16 */
	struct list_head           rpools;               /*    16    16 */
	char *                     name;                 /*    32     8 */

	/* size: 40, cachelines: 1, members: 3 */
	/* last cacheline: 40 bytes */
};
struct rdmacg_resource {
	int                        max;                  /*     0     4 */
	int                        usage;                /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct rdmacg_resource_pool {
	struct rdmacg_device *     device;               /*     0     8 */
	struct rdmacg_resource     resources[2];         /*     8    16 */
	struct list_head           cg_node;              /*    24    16 */
	struct list_head           dev_node;             /*    40    16 */
	u64                        usage_sum;            /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        num_max_cnt;          /*    64     4 */

	/* size: 72, cachelines: 2, members: 6 */
	/* padding: 4 */
	/* last cacheline: 8 bytes */
};
struct fmeter {
	int                        cnt;                  /*     0     4 */
	int                        val;                  /*     4     4 */
	time64_t                   time;                 /*     8     8 */
	spinlock_t                 lock;                 /*    16     4 */

	/* size: 24, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct cpuset {
	struct cgroup_subsys_state css __attribute__((__aligned__(8))); /*     0   296 */
	/* --- cacheline 4 boundary (256 bytes) was 40 bytes ago --- */
	long unsigned int          flags;                /*   296     8 */
	cpumask_var_t              cpus_allowed;         /*   304     8 */
	nodemask_t                 mems_allowed;         /*   312   128 */
	/* --- cacheline 6 boundary (384 bytes) was 56 bytes ago --- */
	cpumask_var_t              effective_cpus;       /*   440     8 */
	/* --- cacheline 7 boundary (448 bytes) --- */
	nodemask_t                 effective_mems;       /*   448   128 */
	/* --- cacheline 9 boundary (576 bytes) --- */
	cpumask_var_t              subparts_cpus;        /*   576     8 */
	nodemask_t                 old_mems_allowed;     /*   584   128 */
	/* --- cacheline 11 boundary (704 bytes) was 8 bytes ago --- */
	struct fmeter              fmeter;               /*   712    24 */

	/* XXX last struct has 4 bytes of padding */

	int                        attach_in_progress;   /*   736     4 */
	int                        pn;                   /*   740     4 */
	int                        relax_domain_level;   /*   744     4 */
	int                        nr_subparts_cpus;     /*   748     4 */
	int                        partition_root_state; /*   752     4 */
	int                        use_parent_ecpus;     /*   756     4 */
	int                        child_ecpus_count;    /*   760     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 12 boundary (768 bytes) --- */
	u64                        kabi_reserved1;       /*   768     8 */
	u64                        kabi_reserved2;       /*   776     8 */
	u64                        kabi_reserved3;       /*   784     8 */
	u64                        kabi_reserved4;       /*   792     8 */

	/* size: 800, cachelines: 13, members: 20 */
	/* sum members: 796, holes: 1, sum holes: 4 */
	/* paddings: 1, sum paddings: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct tmpmasks {
	cpumask_var_t              addmask;              /*     0     8 */
	cpumask_var_t              delmask;              /*     8     8 */
	cpumask_var_t              new_cpus;             /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct cpuset_migrate_mm_work {
	struct work_struct         work;                 /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct mm_struct *         mm;                   /*    64     8 */
	nodemask_t                 from;                 /*    72   128 */
	/* --- cacheline 3 boundary (192 bytes) was 8 bytes ago --- */
	nodemask_t                 to;                   /*   200   128 */

	/* size: 328, cachelines: 6, members: 4 */
	/* last cacheline: 8 bytes */
};
struct key_type {
	const char  *              name;                 /*     0     8 */
	size_t                     def_datalen;          /*     8     8 */
	unsigned int               flags;                /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	int                        (*vet_description)(const char  *); /*    24     8 */
	int                        (*preparse)(struct key_preparsed_payload *); /*    32     8 */
	void                       (*free_preparse)(struct key_preparsed_payload *); /*    40     8 */
	int                        (*instantiate)(struct key *, struct key_preparsed_payload *); /*    48     8 */
	int                        (*update)(struct key *, struct key_preparsed_payload *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*match_preparse)(struct key_match_data *); /*    64     8 */
	void                       (*match_free)(struct key_match_data *); /*    72     8 */
	void                       (*revoke)(struct key *); /*    80     8 */
	void                       (*destroy)(struct key *); /*    88     8 */
	void                       (*describe)(const struct key  *, struct seq_file *); /*    96     8 */
	long int                   (*read)(const struct key  *, char *, size_t); /*   104     8 */
	request_key_actor_t        request_key;          /*   112     8 */
	struct key_restriction *   (*lookup_restriction)(const char  *); /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	int                        (*asym_query)(const struct kernel_pkey_params  *, struct kernel_pkey_query *); /*   128     8 */
	int                        (*asym_eds_op)(struct kernel_pkey_params *, const void  *, void *); /*   136     8 */
	int                        (*asym_verify_signature)(struct kernel_pkey_params *, const void  *, const void  *); /*   144     8 */
	struct list_head           link;                 /*   152    16 */
	struct lock_class_key      lock_class;           /*   168     0 */

	/* size: 168, cachelines: 3, members: 21 */
	/* sum members: 164, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct kernel_pkey_query {
	__u32                      supported_ops;        /*     0     4 */
	__u32                      key_size;             /*     4     4 */
	__u16                      max_data_size;        /*     8     2 */
	__u16                      max_sig_size;         /*    10     2 */
	__u16                      max_enc_size;         /*    12     2 */
	__u16                      max_dec_size;         /*    14     2 */

	/* size: 16, cachelines: 1, members: 6 */
	/* last cacheline: 16 bytes */
};
struct kernel_pkey_params {
	struct key *               key;                  /*     0     8 */
	const char  *              encoding;             /*     8     8 */
	const char  *              hash_algo;            /*    16     8 */
	char *                     info;                 /*    24     8 */
	__u32                      in_len;               /*    32     4 */
	union {
		__u32              out_len;              /*    36     4 */
		__u32              in2_len;              /*    36     4 */
	};                                               /*    36     4 */
	enum kernel_pkey_operation op:8;                 /*    40: 0  4 */

	/* XXX 24 bits hole, try to pack */
	/* XXX 4 bytes hole, try to pack */

	u64                        kabi_reserved1;       /*    48     8 */
	u64                        kabi_reserved2;       /*    56     8 */

	/* size: 64, cachelines: 1, members: 9 */
	/* sum members: 56, holes: 1, sum holes: 4 */
	/* sum bitfield members: 8 bits, bit holes: 1, sum bit holes: 24 bits */
};
struct key_preparsed_payload {
	char *                     description;          /*     0     8 */
	union key_payload          payload;              /*     8    32 */
	const void  *              data;                 /*    40     8 */
	size_t                     datalen;              /*    48     8 */
	size_t                     quotalen;             /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	time64_t                   expiry;               /*    64     8 */

	/* size: 72, cachelines: 2, members: 6 */
	/* last cacheline: 8 bytes */
};
struct key_match_data {
	bool                       (*cmp)(const struct key  *, const struct key_match_data  *); /*     0     8 */
	const void  *              raw_data;             /*     8     8 */
	void *                     preparsed;            /*    16     8 */
	unsigned int               lookup_type;          /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        kabi_reserved1;       /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* sum members: 36, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct idmap_key {
	bool                       map_up;               /*     0     1 */

	/* XXX 3 bytes hole, try to pack */

	u32                        id;                   /*     4     4 */
	u32                        count;                /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* sum members: 9, holes: 1, sum holes: 3 */
	/* last cacheline: 12 bytes */
};
struct cpu_stop_done {
	atomic_t                   nr_todo;              /*     0     4 */
	int                        ret;                  /*     4     4 */
	struct completion          completion;           /*     8    32 */

	/* size: 40, cachelines: 1, members: 3 */
	/* last cacheline: 40 bytes */
};
struct cpu_stopper {
	struct task_struct *       thread;               /*     0     8 */
	raw_spinlock_t             lock;                 /*     8     4 */
	bool                       enabled;              /*    12     1 */

	/* XXX 3 bytes hole, try to pack */

	struct list_head           works;                /*    16    16 */
	struct cpu_stop_work       stop_work;            /*    32    48 */

	/* size: 80, cachelines: 2, members: 5 */
	/* sum members: 77, holes: 1, sum holes: 3 */
	/* last cacheline: 16 bytes */
};
struct multi_stop_data {
	cpu_stop_fn_t              fn;                   /*     0     8 */
	void *                     data;                 /*     8     8 */
	unsigned int               num_threads;          /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	const struct cpumask  *    active_cpus;          /*    24     8 */
	enum multi_stop_state      state;                /*    32     4 */
	atomic_t                   thread_ack;           /*    36     4 */

	/* size: 40, cachelines: 1, members: 6 */
	/* sum members: 36, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct audit_context {
	int                        dummy;                /*     0     4 */
	int                        in_syscall;           /*     4     4 */
	enum audit_state           state;                /*     8     4 */
	enum audit_state           current_state;        /*    12     4 */
	unsigned int               serial;               /*    16     4 */
	int                        major;                /*    20     4 */
	struct timespec64          ctime;                /*    24    16 */
	long unsigned int          argv[4];              /*    40    32 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	long int                   return_code;          /*    72     8 */
	u64                        prio;                 /*    80     8 */
	int                        return_valid;         /*    88     4 */

	/* XXX 4 bytes hole, try to pack */

	struct audit_names         preallocated_names[5]; /*    96   560 */
	/* --- cacheline 10 boundary (640 bytes) was 16 bytes ago --- */
	int                        name_count;           /*   656     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           names_list;           /*   664    16 */
	char *                     filterkey;            /*   680     8 */
	struct path                pwd;                  /*   688    16 */
	/* --- cacheline 11 boundary (704 bytes) --- */
	struct audit_aux_data *    aux;                  /*   704     8 */
	struct audit_aux_data *    aux_pids;             /*   712     8 */
	struct __kernel_sockaddr_storage * sockaddr;     /*   720     8 */
	size_t                     sockaddr_len;         /*   728     8 */
	pid_t                      pid;                  /*   736     4 */
	pid_t                      ppid;                 /*   740     4 */
	kuid_t                     uid;                  /*   744     4 */
	kuid_t                     euid;                 /*   748     4 */
	kuid_t                     suid;                 /*   752     4 */
	kuid_t                     fsuid;                /*   756     4 */
	kgid_t                     gid;                  /*   760     4 */
	kgid_t                     egid;                 /*   764     4 */
	/* --- cacheline 12 boundary (768 bytes) --- */
	kgid_t                     sgid;                 /*   768     4 */
	kgid_t                     fsgid;                /*   772     4 */
	long unsigned int          personality;          /*   776     8 */
	int                        arch;                 /*   784     4 */
	pid_t                      target_pid;           /*   788     4 */
	kuid_t                     target_auid;          /*   792     4 */
	kuid_t                     target_uid;           /*   796     4 */
	unsigned int               target_sessionid;     /*   800     4 */
	u32                        target_sid;           /*   804     4 */
	char                       target_comm[16];      /*   808    16 */
	struct audit_tree_refs *   trees;                /*   824     8 */
	/* --- cacheline 13 boundary (832 bytes) --- */
	struct audit_tree_refs *   first_trees;          /*   832     8 */
	struct list_head           killed_trees;         /*   840    16 */
	int                        tree_count;           /*   856     4 */
	int                        type;                 /*   860     4 */
	union {
		struct {
			int        nargs;                /*   864     4 */

			/* XXX 4 bytes hole, try to pack */

			long int   args[6];              /*   872    48 */
		} socketcall;                            /*   864    56 */
		struct {
			kuid_t     uid;                  /*   864     4 */
			kgid_t     gid;                  /*   868     4 */
			umode_t    mode;                 /*   872     2 */

			/* XXX 2 bytes hole, try to pack */

			u32        osid;                 /*   876     4 */
			int        has_perm;             /*   880     4 */
			uid_t      perm_uid;             /*   884     4 */
			gid_t      perm_gid;             /*   888     4 */
			umode_t    perm_mode;            /*   892     2 */

			/* XXX 2 bytes hole, try to pack */

			/* --- cacheline 14 boundary (896 bytes) --- */
			long unsigned int qbytes;        /*   896     8 */
		} ipc;                                   /*   864    40 */
		struct {
			mqd_t      mqdes;                /*   864     4 */

			/* XXX 4 bytes hole, try to pack */

			struct mq_attr mqstat;           /*   872    64 */
		} mq_getsetattr;                         /*   864    72 */
		struct {
			mqd_t      mqdes;                /*   864     4 */
			int        sigev_signo;          /*   868     4 */
		} mq_notify;                             /*   864     8 */
		struct {
			mqd_t      mqdes;                /*   864     4 */

			/* XXX 4 bytes hole, try to pack */

			size_t     msg_len;              /*   872     8 */
			unsigned int msg_prio;           /*   880     4 */

			/* XXX 4 bytes hole, try to pack */

			struct timespec64 abs_timeout;   /*   888    16 */
		} mq_sendrecv;                           /*   864    40 */
		struct {
			int        oflag;                /*   864     4 */
			umode_t    mode;                 /*   868     2 */

			/* XXX 2 bytes hole, try to pack */

			struct mq_attr attr;             /*   872    64 */
		} mq_open;                               /*   864    72 */
		struct {
			pid_t      pid;                  /*   864     4 */
			struct audit_cap_data cap;       /*   868    36 */
		} capset;                                /*   864    40 */
		struct {
			int        fd;                   /*   864     4 */
			int        flags;                /*   868     4 */
		} mmap;                                  /*   864     8 */
		struct {
			int        argc;                 /*   864     4 */
		} execve;                                /*   864     4 */
		struct {
			char *     name;                 /*   864     8 */
		} module;                                /*   864     8 */
		struct {
			struct audit_ntp_data ntp_data;  /*   864    96 */
			/* --- cacheline 15 boundary (960 bytes) --- */
			struct timespec64 tk_injoffset;  /*   960    16 */
		} time;                                  /*   864   112 */
	};                                               /*   864   112 */
	/* --- cacheline 15 boundary (960 bytes) was 16 bytes ago --- */
	int                        fds[2];               /*   976     8 */
	struct audit_proctitle     proctitle;            /*   984    16 */

	/* size: 1000, cachelines: 16, members: 46 */
	/* sum members: 992, holes: 2, sum holes: 8 */
	/* last cacheline: 40 bytes */
};
struct audit_names {
	struct list_head           list;                 /*     0    16 */
	struct filename *          name;                 /*    16     8 */
	int                        name_len;             /*    24     4 */
	bool                       hidden;               /*    28     1 */

	/* XXX 3 bytes hole, try to pack */

	long unsigned int          ino;                  /*    32     8 */
	dev_t                      dev;                  /*    40     4 */
	umode_t                    mode;                 /*    44     2 */

	/* XXX 2 bytes hole, try to pack */

	kuid_t                     uid;                  /*    48     4 */
	kgid_t                     gid;                  /*    52     4 */
	dev_t                      rdev;                 /*    56     4 */
	u32                        osid;                 /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct audit_cap_data      fcap;                 /*    64    36 */
	unsigned int               fcap_ver;             /*   100     4 */
	unsigned char              type;                 /*   104     1 */
	bool                       should_free;          /*   105     1 */

	/* size: 112, cachelines: 2, members: 15 */
	/* sum members: 101, holes: 2, sum holes: 5 */
	/* padding: 6 */
	/* last cacheline: 48 bytes */
};
struct __kernel_sockaddr_storage {
	union {
		struct {
			__kernel_sa_family_t ss_family;  /*     0     2 */
			char       __data[126];          /*     2   126 */
		};                                       /*     0   128 */
		void *             __align;              /*     0     8 */
	};                                               /*     0   128 */

	/* size: 128, cachelines: 2, members: 1 */
};
struct audit_status {
	__u32                      mask;                 /*     0     4 */
	__u32                      enabled;              /*     4     4 */
	__u32                      failure;              /*     8     4 */
	__u32                      pid;                  /*    12     4 */
	__u32                      rate_limit;           /*    16     4 */
	__u32                      backlog_limit;        /*    20     4 */
	__u32                      lost;                 /*    24     4 */
	__u32                      backlog;              /*    28     4 */
	union {
		__u32              version;              /*    32     4 */
		__u32              feature_bitmap;       /*    32     4 */
	};                                               /*    32     4 */
	__u32                      backlog_wait_time;    /*    36     4 */
	__u32                      backlog_wait_time_actual; /*    40     4 */

	/* size: 44, cachelines: 1, members: 11 */
	/* last cacheline: 44 bytes */
};
struct audit_features {
	__u32                      vers;                 /*     0     4 */
	__u32                      mask;                 /*     4     4 */
	__u32                      features;             /*     8     4 */
	__u32                      lock;                 /*    12     4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct audit_tty_status {
	__u32                      enabled;              /*     0     4 */
	__u32                      log_passwd;           /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct audit_sig_info {
	uid_t                      uid;                  /*     0     4 */
	pid_t                      pid;                  /*     4     4 */
	char                       ctx[];                /*     8     0 */

	/* size: 8, cachelines: 1, members: 3 */
	/* last cacheline: 8 bytes */
};
struct scm_creds {
	u32                        pid;                  /*     0     4 */
	kuid_t                     uid;                  /*     4     4 */
	kgid_t                     gid;                  /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct netlink_skb_parms {
	struct scm_creds           creds;                /*     0    12 */
	__u32                      portid;               /*    12     4 */
	__u32                      dst_group;            /*    16     4 */
	__u32                      flags;                /*    20     4 */
	struct sock *              sk;                   /*    24     8 */
	bool                       nsid_is_set;          /*    32     1 */

	/* XXX 3 bytes hole, try to pack */

	int                        nsid;                 /*    36     4 */

	/* size: 40, cachelines: 1, members: 7 */
	/* sum members: 37, holes: 1, sum holes: 3 */
	/* last cacheline: 40 bytes */
};
struct netlink_kernel_cfg {
	unsigned int               groups;               /*     0     4 */
	unsigned int               flags;                /*     4     4 */
	void                       (*input)(struct sk_buff *); /*     8     8 */
	struct mutex *             cb_mutex;             /*    16     8 */
	int                        (*bind)(struct net *, int); /*    24     8 */
	void                       (*unbind)(struct net *, int); /*    32     8 */
	bool                       (*compare)(struct net *, struct sock *); /*    40     8 */

	/* size: 48, cachelines: 1, members: 7 */
	/* last cacheline: 48 bytes */
};
struct net_generic {
	union {
		struct {
			unsigned int len;                /*     0     4 */

			/* XXX 4 bytes hole, try to pack */

			struct callback_head rcu __attribute__((__aligned__(8))); /*     8    16 */
		} __attribute__((__aligned__(8))) s __attribute__((__aligned__(8)));     /*     0    24 */
		void *             ptr[0];               /*     0     0 */
	} __attribute__((__aligned__(8)));               /*     0    24 */

	/* size: 24, cachelines: 1, members: 1 */
	/* forced alignments: 1 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct pernet_operations {
	struct list_head           list;                 /*     0    16 */
	int                        (*init)(struct net *); /*    16     8 */
	void                       (*pre_exit)(struct net *); /*    24     8 */
	void                       (*exit)(struct net *); /*    32     8 */
	void                       (*exit_batch)(struct list_head *); /*    40     8 */
	unsigned int *             id;                   /*    48     8 */
	size_t                     size;                 /*    56     8 */

	/* size: 64, cachelines: 1, members: 7 */
};
struct mq_attr {
	__kernel_long_t            mq_flags;             /*     0     8 */
	__kernel_long_t            mq_maxmsg;            /*     8     8 */
	__kernel_long_t            mq_msgsize;           /*    16     8 */
	__kernel_long_t            mq_curmsgs;           /*    24     8 */
	__kernel_long_t            __reserved[4];        /*    32    32 */

	/* size: 64, cachelines: 1, members: 5 */
};
struct audit_cap_data {
	kernel_cap_t               permitted;            /*     0     8 */
	kernel_cap_t               inheritable;          /*     8     8 */
	union {
		unsigned int       fE;                   /*    16     4 */
		kernel_cap_t       effective;            /*    16     8 */
	};                                               /*    16     8 */
	kernel_cap_t               ambient;              /*    24     8 */
	kuid_t                     rootid;               /*    32     4 */

	/* size: 36, cachelines: 1, members: 5 */
	/* last cacheline: 36 bytes */
};
struct audit_proctitle {
	int                        len;                  /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	char *                     value;                /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct audit_netlink_list {
	__u32                      portid;               /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct net *               net;                  /*     8     8 */
	struct sk_buff_head        q;                    /*    16    24 */

	/* size: 40, cachelines: 1, members: 3 */
	/* sum members: 36, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct audit_net {
	struct sock *              sk;                   /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct auditd_connection {
	struct pid *               pid;                  /*     0     8 */
	u32                        portid;               /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct net *               net;                  /*    16     8 */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*    24    16 */

	/* size: 40, cachelines: 1, members: 4 */
	/* sum members: 36, holes: 1, sum holes: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 40 bytes */
} __attribute__((__aligned__(8)));
struct audit_ctl_mutex {
	struct mutex               lock;                 /*     0    32 */
	void *                     owner;                /*    32     8 */

	/* size: 40, cachelines: 1, members: 2 */
	/* last cacheline: 40 bytes */
};
struct audit_buffer {
	struct sk_buff *           skb;                  /*     0     8 */
	struct audit_context *     ctx;                  /*     8     8 */
	gfp_t                      gfp_mask;             /*    16     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct audit_reply {
	__u32                      portid;               /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct net *               net;                  /*     8     8 */
	struct sk_buff *           skb;                  /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct audit_rule_data {
	__u32                      flags;                /*     0     4 */
	__u32                      action;               /*     4     4 */
	__u32                      field_count;          /*     8     4 */
	__u32                      mask[64];             /*    12   256 */
	/* --- cacheline 4 boundary (256 bytes) was 12 bytes ago --- */
	__u32                      fields[64];           /*   268   256 */
	/* --- cacheline 8 boundary (512 bytes) was 12 bytes ago --- */
	__u32                      values[64];           /*   524   256 */
	/* --- cacheline 12 boundary (768 bytes) was 12 bytes ago --- */
	__u32                      fieldflags[64];       /*   780   256 */
	/* --- cacheline 16 boundary (1024 bytes) was 12 bytes ago --- */
	__u32                      buflen;               /*  1036     4 */
	char                       buf[];                /*  1040     0 */

	/* size: 1040, cachelines: 17, members: 9 */
	/* last cacheline: 16 bytes */
};
struct audit_krule {
	u32                        pflags;               /*     0     4 */
	u32                        flags;                /*     4     4 */
	u32                        listnr;               /*     8     4 */
	u32                        action;               /*    12     4 */
	u32                        mask[64];             /*    16   256 */
	/* --- cacheline 4 boundary (256 bytes) was 16 bytes ago --- */
	u32                        buflen;               /*   272     4 */
	u32                        field_count;          /*   276     4 */
	char *                     filterkey;            /*   280     8 */
	struct audit_field *       fields;               /*   288     8 */
	struct audit_field *       arch_f;               /*   296     8 */
	struct audit_field *       inode_f;              /*   304     8 */
	struct audit_watch *       watch;                /*   312     8 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	struct audit_tree *        tree;                 /*   320     8 */
	struct audit_fsnotify_mark * exe;                /*   328     8 */
	struct list_head           rlist;                /*   336    16 */
	struct list_head           list;                 /*   352    16 */
	u64                        prio;                 /*   368     8 */

	/* size: 376, cachelines: 6, members: 17 */
	/* last cacheline: 56 bytes */
};
struct audit_field {
	u32                        type;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	union {
		u32                val;                  /*     8     4 */
		kuid_t             uid;                  /*     8     4 */
		kgid_t             gid;                  /*     8     4 */
		struct {
			char *     lsm_str;              /*     8     8 */
			void *     lsm_rule;             /*    16     8 */
		};                                       /*     8    16 */
	};                                               /*     8    16 */
	u32                        op;                   /*    24     4 */

	/* size: 32, cachelines: 1, members: 3 */
	/* sum members: 24, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct audit_entry {
	struct list_head           list;                 /*     0    16 */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*    16    16 */
	struct audit_krule         rule;                 /*    32   376 */

	/* size: 408, cachelines: 7, members: 3 */
	/* forced alignments: 1 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct cpu_vfs_cap_data {
	__u32                      magic_etc;            /*     0     4 */
	kernel_cap_t               permitted;            /*     4     8 */
	kernel_cap_t               inheritable;          /*    12     8 */
	kuid_t                     rootid;               /*    20     4 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct kern_ipc_perm {
	spinlock_t                 lock;                 /*     0     4 */
	bool                       deleted;              /*     4     1 */

	/* XXX 3 bytes hole, try to pack */

	int                        id;                   /*     8     4 */
	key_t                      key;                  /*    12     4 */
	kuid_t                     uid;                  /*    16     4 */
	kgid_t                     gid;                  /*    20     4 */
	kuid_t                     cuid;                 /*    24     4 */
	kgid_t                     cgid;                 /*    28     4 */
	umode_t                    mode;                 /*    32     2 */

	/* XXX 6 bytes hole, try to pack */

	long unsigned int          seq;                  /*    40     8 */
	void *                     security;             /*    48     8 */
	struct rhash_head          khtnode;              /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*    64    16 */
	refcount_t                 refcount;             /*    80     4 */

	/* size: 128, cachelines: 2, members: 14 */
	/* sum members: 75, holes: 2, sum holes: 9 */
	/* padding: 44 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(64)));
struct fsnotify_mark_connector {
	spinlock_t                 lock;                 /*     0     4 */
	short unsigned int         type;                 /*     4     2 */
	short unsigned int         flags;                /*     6     2 */
	__kernel_fsid_t            fsid;                 /*     8     8 */
	union {
		fsnotify_connp_t * obj;                  /*    16     8 */
		struct fsnotify_mark_connector * destroy_next; /*    16     8 */
	};                                               /*    16     8 */
	struct hlist_head          list;                 /*    24     8 */

	/* size: 32, cachelines: 1, members: 6 */
	/* last cacheline: 32 bytes */
};
struct audit_aux_data {
	struct audit_aux_data *    next;                 /*     0     8 */
	int                        type;                 /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct audit_tree_refs {
	struct audit_tree_refs *   next;                 /*     0     8 */
	struct audit_chunk *       c[31];                /*     8   248 */

	/* size: 256, cachelines: 4, members: 2 */
};
struct audit_aux_data_pids {
	struct audit_aux_data      d;                    /*     0    16 */

	/* XXX last struct has 4 bytes of padding */

	pid_t                      target_pid[16];       /*    16    64 */
	/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
	kuid_t                     target_auid[16];      /*    80    64 */
	/* --- cacheline 2 boundary (128 bytes) was 16 bytes ago --- */
	kuid_t                     target_uid[16];       /*   144    64 */
	/* --- cacheline 3 boundary (192 bytes) was 16 bytes ago --- */
	unsigned int               target_sessionid[16]; /*   208    64 */
	/* --- cacheline 4 boundary (256 bytes) was 16 bytes ago --- */
	u32                        target_sid[16];       /*   272    64 */
	/* --- cacheline 5 boundary (320 bytes) was 16 bytes ago --- */
	char                       target_comm[16][16];  /*   336   256 */
	/* --- cacheline 9 boundary (576 bytes) was 16 bytes ago --- */
	int                        pid_count;            /*   592     4 */

	/* size: 600, cachelines: 10, members: 8 */
	/* padding: 4 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 24 bytes */
};
struct audit_aux_data_bprm_fcaps {
	struct audit_aux_data      d;                    /*     0    16 */

	/* XXX last struct has 4 bytes of padding */

	struct audit_cap_data      fcap;                 /*    16    36 */
	unsigned int               fcap_ver;             /*    52     4 */
	struct audit_cap_data      old_pcap;             /*    56    36 */
	/* --- cacheline 1 boundary (64 bytes) was 28 bytes ago --- */
	struct audit_cap_data      new_pcap;             /*    92    36 */

	/* size: 128, cachelines: 2, members: 5 */
	/* paddings: 1, sum paddings: 4 */
};
struct audit_nfcfgop_tab {
	enum audit_nfcfgop         op;                   /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	const char  *              s;                    /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct audit_watch {
	refcount_t                 count;                /*     0     4 */
	dev_t                      dev;                  /*     4     4 */
	char *                     path;                 /*     8     8 */
	long unsigned int          ino;                  /*    16     8 */
	struct audit_parent *      parent;               /*    24     8 */
	struct list_head           wlist;                /*    32    16 */
	struct list_head           rules;                /*    48    16 */

	/* size: 64, cachelines: 1, members: 7 */
};
struct fsnotify_ops {
	int                        (*handle_event)(struct fsnotify_group *, u32, const void  *, int, struct inode *, const struct qstr  *, u32, struct fsnotify_iter_info *); /*     0     8 */
	int                        (*handle_inode_event)(struct fsnotify_mark *, u32, struct inode *, struct inode *, const struct qstr  *, u32); /*     8     8 */
	void                       (*free_group_priv)(struct fsnotify_group *); /*    16     8 */
	void                       (*freeing_mark)(struct fsnotify_mark *, struct fsnotify_group *); /*    24     8 */
	void                       (*free_event)(struct fsnotify_event *); /*    32     8 */
	void                       (*free_mark)(struct fsnotify_mark *); /*    40     8 */
	u64                        kabi_reserved1;       /*    48     8 */

	/* size: 56, cachelines: 1, members: 7 */
	/* last cacheline: 56 bytes */
};
struct fsnotify_group {
	const struct fsnotify_ops  * ops;                /*     0     8 */
	refcount_t                 refcnt;               /*     8     4 */
	spinlock_t                 notification_lock;    /*    12     4 */
	struct list_head           notification_list;    /*    16    16 */
	wait_queue_head_t          notification_waitq;   /*    32    24 */
	unsigned int               q_len;                /*    56     4 */
	unsigned int               max_events;           /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	unsigned int               priority;             /*    64     4 */
	bool                       shutdown;             /*    68     1 */

	/* XXX 3 bytes hole, try to pack */

	struct mutex               mark_mutex;           /*    72    32 */
	atomic_t                   num_marks;            /*   104     4 */
	atomic_t                   user_waits;           /*   108     4 */
	struct list_head           marks_list;           /*   112    16 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct fasync_struct *     fsn_fa;               /*   128     8 */
	struct fsnotify_event *    overflow_event;       /*   136     8 */
	struct mem_cgroup *        memcg;                /*   144     8 */
	union {
		void *             private;              /*   152     8 */
		struct inotify_group_private_data inotify_data; /*   152    40 */
		struct fanotify_group_private_data fanotify_data; /*   152    64 */
	};                                               /*   152    64 */

	/* size: 216, cachelines: 4, members: 17 */
	/* sum members: 213, holes: 1, sum holes: 3 */
	/* last cacheline: 24 bytes */
};
struct fsnotify_iter_info {
	struct fsnotify_mark *     marks[4];             /*     0    32 */
	unsigned int               report_mask;          /*    32     4 */
	int                        srcu_idx;             /*    36     4 */

	/* size: 40, cachelines: 1, members: 3 */
	/* last cacheline: 40 bytes */
};
struct fsnotify_mark {
	__u32                      mask;                 /*     0     4 */
	refcount_t                 refcnt;               /*     4     4 */
	struct fsnotify_group *    group;                /*     8     8 */
	struct list_head           g_list;               /*    16    16 */
	spinlock_t                 lock;                 /*    32     4 */

	/* XXX 4 bytes hole, try to pack */

	struct hlist_node          obj_list;             /*    40    16 */
	struct fsnotify_mark_connector * connector;      /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	__u32                      ignored_mask;         /*    64     4 */
	unsigned int               flags;                /*    68     4 */

	/* size: 72, cachelines: 2, members: 9 */
	/* sum members: 68, holes: 1, sum holes: 4 */
	/* last cacheline: 8 bytes */
};
struct fsnotify_event {
	struct list_head           list;                 /*     0    16 */
	long unsigned int          objectid;             /*    16     8 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct inotify_group_private_data {
	spinlock_t                 idr_lock;             /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct idr                 idr;                  /*     8    24 */
	struct ucounts *           ucounts;              /*    32     8 */

	/* size: 40, cachelines: 1, members: 3 */
	/* sum members: 36, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct fanotify_group_private_data {
	struct list_head           access_list;          /*     0    16 */
	wait_queue_head_t          access_waitq;         /*    16    24 */
	int                        flags;                /*    40     4 */
	int                        f_flags;              /*    44     4 */
	unsigned int               max_marks;            /*    48     4 */

	/* XXX 4 bytes hole, try to pack */

	struct user_struct *       user;                 /*    56     8 */

	/* size: 64, cachelines: 1, members: 6 */
	/* sum members: 60, holes: 1, sum holes: 4 */
};
struct audit_parent {
	struct list_head           watches;              /*     0    16 */
	struct fsnotify_mark       mark;                 /*    16    72 */

	/* size: 88, cachelines: 2, members: 2 */
	/* last cacheline: 24 bytes */
};
struct audit_fsnotify_mark {
	dev_t                      dev;                  /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          ino;                  /*     8     8 */
	char *                     path;                 /*    16     8 */
	struct fsnotify_mark       mark;                 /*    24    72 */
	/* --- cacheline 1 boundary (64 bytes) was 32 bytes ago --- */
	struct audit_krule *       rule;                 /*    96     8 */

	/* size: 104, cachelines: 2, members: 5 */
	/* sum members: 100, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct audit_tree {
	refcount_t                 count;                /*     0     4 */
	int                        goner;                /*     4     4 */
	struct audit_chunk *       root;                 /*     8     8 */
	struct list_head           chunks;               /*    16    16 */
	struct list_head           rules;                /*    32    16 */
	struct list_head           list;                 /*    48    16 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct list_head           same_root;            /*    64    16 */
	struct callback_head       head __attribute__((__aligned__(8))); /*    80    16 */
	char                       pathname[];           /*    96     0 */

	/* size: 96, cachelines: 2, members: 9 */
	/* forced alignments: 1 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct audit_chunk {
	struct list_head           hash;                 /*     0    16 */
	long unsigned int          key;                  /*    16     8 */
	struct fsnotify_mark *     mark;                 /*    24     8 */
	struct list_head           trees;                /*    32    16 */
	int                        count;                /*    48     4 */

	/* XXX 4 bytes hole, try to pack */

	atomic_long_t              refs;                 /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct callback_head       head __attribute__((__aligned__(8))); /*    64    16 */
	struct node                owners[];             /*    80     0 */

	/* size: 80, cachelines: 2, members: 8 */
	/* sum members: 76, holes: 1, sum holes: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct node {
	struct list_head           list;                 /*     0    16 */
	struct audit_tree *        owner;                /*    16     8 */
	unsigned int               index;                /*    24     4 */

	/* size: 32, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct audit_tree_mark {
	struct fsnotify_mark       mark;                 /*     0    72 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	struct audit_chunk *       chunk;                /*    72     8 */

	/* size: 80, cachelines: 2, members: 2 */
	/* last cacheline: 16 bytes */
};
struct kprobe_blacklist_entry {
	struct list_head           list;                 /*     0    16 */
	long unsigned int          start_addr;           /*    16     8 */
	long unsigned int          end_addr;             /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* last cacheline: 32 bytes */
};
struct kprobe_insn_page {
	struct list_head           list;                 /*     0    16 */
	kprobe_opcode_t *          insns;                /*    16     8 */
	struct kprobe_insn_cache * cache;                /*    24     8 */
	int                        nused;                /*    32     4 */
	int                        ngarbage;             /*    36     4 */
	char                       slot_used[];          /*    40     0 */

	/* size: 40, cachelines: 1, members: 6 */
	/* last cacheline: 40 bytes */
};
struct kgdb_io {
	const char  *              name;                 /*     0     8 */
	int                        (*read_char)(void);   /*     8     8 */
	void                       (*write_char)(u8);    /*    16     8 */
	void                       (*flush)(void);       /*    24     8 */
	int                        (*init)(void);        /*    32     8 */
	void                       (*deinit)(void);      /*    40     8 */
	void                       (*pre_exception)(void); /*    48     8 */
	void                       (*post_exception)(void); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct console *           cons;                 /*    64     8 */

	/* size: 72, cachelines: 2, members: 9 */
	/* last cacheline: 8 bytes */
};
struct kgdb_state {
	int                        ex_vector;            /*     0     4 */
	int                        signo;                /*     4     4 */
	int                        err_code;             /*     8     4 */
	int                        cpu;                  /*    12     4 */
	int                        pass_exception;       /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          thr_query;            /*    24     8 */
	long unsigned int          threadid;             /*    32     8 */
	long int                   kgdb_usethreadid;     /*    40     8 */
	struct pt_regs *           linux_regs;           /*    48     8 */
	atomic_t *                 send_ready;           /*    56     8 */

	/* size: 64, cachelines: 1, members: 10 */
	/* sum members: 60, holes: 1, sum holes: 4 */
};
struct debuggerinfo_struct {
	void *                     debuggerinfo;         /*     0     8 */
	struct task_struct *       task;                 /*     8     8 */
	int                        exception_state;      /*    16     4 */
	int                        ret_state;            /*    20     4 */
	int                        irq_depth;            /*    24     4 */
	int                        enter_kgdb;           /*    28     4 */
	bool                       rounding_up;          /*    32     1 */

	/* size: 40, cachelines: 1, members: 7 */
	/* padding: 7 */
	/* last cacheline: 40 bytes */
};
struct __ksymtab {
	long unsigned int          value;                /*     0     8 */
	const char  *              mod_name;             /*     8     8 */
	long unsigned int          mod_start;            /*    16     8 */
	long unsigned int          mod_end;              /*    24     8 */
	const char  *              sec_name;             /*    32     8 */
	long unsigned int          sec_start;            /*    40     8 */
	long unsigned int          sec_end;              /*    48     8 */
	const char  *              sym_name;             /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	long unsigned int          sym_start;            /*    64     8 */
	long unsigned int          sym_end;              /*    72     8 */

	/* size: 80, cachelines: 2, members: 10 */
	/* last cacheline: 16 bytes */
};
struct _kdbtab {
	char *                     cmd_name;             /*     0     8 */
	kdb_func_t                 cmd_func;             /*     8     8 */
	char *                     cmd_usage;            /*    16     8 */
	char *                     cmd_help;             /*    24     8 */
	short int                  cmd_minlen;           /*    32     2 */

	/* XXX 2 bytes hole, try to pack */

	kdb_cmdflags_t             cmd_flags;            /*    36     4 */

	/* size: 40, cachelines: 1, members: 6 */
	/* sum members: 38, holes: 1, sum holes: 2 */
	/* last cacheline: 40 bytes */
};
struct _kdbmsg {
	int                        km_diag;              /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	char *                     km_msg;               /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct defcmd_set {
	int                        count;                /*     0     4 */
	bool                       usable;               /*     4     1 */

	/* XXX 3 bytes hole, try to pack */

	char *                     name;                 /*     8     8 */
	char *                     usage;                /*    16     8 */
	char *                     help;                 /*    24     8 */
	char * *                   command;              /*    32     8 */

	/* size: 40, cachelines: 1, members: 6 */
	/* sum members: 37, holes: 1, sum holes: 3 */
	/* last cacheline: 40 bytes */
};
struct debug_alloc_header {
	u32                        next;                 /*     0     4 */
	u32                        size;                 /*     4     4 */
	void *                     caller;               /*     8     8 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct _kdb_bp {
	long unsigned int          bp_addr;              /*     0     8 */
	unsigned int               bp_free:1;            /*     8: 0  4 */
	unsigned int               bp_enabled:1;         /*     8: 1  4 */
	unsigned int               bp_type:4;            /*     8: 2  4 */
	unsigned int               bp_installed:1;       /*     8: 6  4 */
	unsigned int               bp_delay:1;           /*     8: 7  4 */
	unsigned int               bp_delayed:1;         /*     8: 8  4 */

	/* XXX 23 bits hole, try to pack */

	unsigned int               bph_length;           /*    12     4 */

	/* size: 16, cachelines: 1, members: 8 */
	/* sum members: 12 */
	/* sum bitfield members: 9 bits, bit holes: 1, sum bit holes: 23 bits */
	/* last cacheline: 16 bytes */
};
struct watchdog_operations {
	void                       (*watchdog_nmi_stop)(void); /*     0     8 */
	void                       (*watchdog_nmi_start)(void); /*     8     8 */
	int                        (*watchdog_nmi_probe)(void); /*    16     8 */
	int                        (*watchdog_nmi_enable)(unsigned int); /*    24     8 */
	void                       (*watchdog_nmi_disable)(unsigned int); /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct seccomp_data {
	int                        nr;                   /*     0     4 */
	__u32                      arch;                 /*     4     4 */
	__u64                      instruction_pointer;  /*     8     8 */
	__u64                      args[6];              /*    16    48 */

	/* size: 64, cachelines: 1, members: 4 */
};
struct seccomp_notif_sizes {
	__u16                      seccomp_notif;        /*     0     2 */
	__u16                      seccomp_notif_resp;   /*     2     2 */
	__u16                      seccomp_data;         /*     4     2 */

	/* size: 6, cachelines: 1, members: 3 */
	/* last cacheline: 6 bytes */
};
struct seccomp_notif {
	__u64                      id;                   /*     0     8 */
	__u32                      pid;                  /*     8     4 */
	__u32                      flags;                /*    12     4 */
	struct seccomp_data        data;                 /*    16    64 */

	/* size: 80, cachelines: 2, members: 4 */
	/* last cacheline: 16 bytes */
};
struct seccomp_notif_resp {
	__u64                      id;                   /*     0     8 */
	__s64                      val;                  /*     8     8 */
	__s32                      error;                /*    16     4 */
	__u32                      flags;                /*    20     4 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct seccomp_notif_addfd {
	__u64                      id;                   /*     0     8 */
	__u32                      flags;                /*     8     4 */
	__u32                      srcfd;                /*    12     4 */
	__u32                      newfd;                /*    16     4 */
	__u32                      newfd_flags;          /*    20     4 */

	/* size: 24, cachelines: 1, members: 5 */
	/* last cacheline: 24 bytes */
};
struct seccomp_filter {
	refcount_t                 refs;                 /*     0     4 */
	refcount_t                 users;                /*     4     4 */
	bool                       log;                  /*     8     1 */

	/* XXX 7 bytes hole, try to pack */

	struct action_cache        cache;                /*    16   128 */
	/* --- cacheline 2 boundary (128 bytes) was 16 bytes ago --- */
	struct seccomp_filter *    prev;                 /*   144     8 */
	struct bpf_prog *          prog;                 /*   152     8 */
	struct notification *      notif;                /*   160     8 */
	struct mutex               notify_lock;          /*   168    32 */
	/* --- cacheline 3 boundary (192 bytes) was 8 bytes ago --- */
	wait_queue_head_t          wqh;                  /*   200    24 */

	/* size: 224, cachelines: 4, members: 9 */
	/* sum members: 217, holes: 1, sum holes: 7 */
	/* last cacheline: 32 bytes */
};
struct seccomp_metadata {
	__u64                      filter_off;           /*     0     8 */
	__u64                      flags;                /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct sock_fprog {
	short unsigned int         len;                  /*     0     2 */

	/* XXX 6 bytes hole, try to pack */

	struct sock_filter *       filter;               /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 10, holes: 1, sum holes: 6 */
	/* last cacheline: 16 bytes */
};
struct compat_sock_fprog {
	u16                        len;                  /*     0     2 */

	/* XXX 2 bytes hole, try to pack */

	compat_uptr_t              filter;               /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* sum members: 6, holes: 1, sum holes: 2 */
	/* last cacheline: 8 bytes */
};
struct seccomp_knotif {
	struct task_struct *       task;                 /*     0     8 */
	u64                        id;                   /*     8     8 */
	const struct seccomp_data  * data;               /*    16     8 */
	enum notify_state          state;                /*    24     4 */
	int                        error;                /*    28     4 */
	long int                   val;                  /*    32     8 */
	u32                        flags;                /*    40     4 */

	/* XXX 4 bytes hole, try to pack */

	struct completion          ready;                /*    48    32 */
	/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
	struct list_head           list;                 /*    80    16 */
	struct list_head           addfd;                /*    96    16 */

	/* size: 112, cachelines: 2, members: 10 */
	/* sum members: 108, holes: 1, sum holes: 4 */
	/* last cacheline: 48 bytes */
};
struct seccomp_kaddfd {
	struct file *              file;                 /*     0     8 */
	int                        fd;                   /*     8     4 */
	unsigned int               flags;                /*    12     4 */
	int                        ret;                  /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	struct completion          completion;           /*    24    32 */
	struct list_head           list;                 /*    56    16 */

	/* size: 72, cachelines: 2, members: 6 */
	/* sum members: 68, holes: 1, sum holes: 4 */
	/* last cacheline: 8 bytes */
};
struct notification {
	struct semaphore           request;              /*     0    24 */
	u64                        next_id;              /*    24     8 */
	struct list_head           notifications;        /*    32    16 */

	/* size: 48, cachelines: 1, members: 3 */
	/* last cacheline: 48 bytes */
};
struct action_cache {
	long unsigned int          allow_native[8];      /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	long unsigned int          allow_compat[8];      /*    64    64 */

	/* size: 128, cachelines: 2, members: 2 */
};
struct seccomp_log_name {
	u32                        log;                  /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	const char  *              name;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct rchan_buf {
	void *                     start;                /*     0     8 */
	void *                     data;                 /*     8     8 */
	size_t                     offset;               /*    16     8 */
	size_t                     subbufs_produced;     /*    24     8 */
	size_t                     subbufs_consumed;     /*    32     8 */
	struct rchan *             chan;                 /*    40     8 */
	wait_queue_head_t          read_wait;            /*    48    24 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	struct irq_work            wakeup_work;          /*    72    24 */
	struct dentry *            dentry;               /*    96     8 */
	struct kref                kref;                 /*   104     4 */

	/* XXX 4 bytes hole, try to pack */

	struct page * *            page_array;           /*   112     8 */
	unsigned int               page_count;           /*   120     4 */
	unsigned int               finalized;            /*   124     4 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	size_t *                   padding;              /*   128     8 */
	size_t                     prev_padding;         /*   136     8 */
	size_t                     bytes_consumed;       /*   144     8 */
	size_t                     early_bytes;          /*   152     8 */
	unsigned int               cpu;                  /*   160     4 */

	/* size: 192, cachelines: 3, members: 18 */
	/* sum members: 160, holes: 1, sum holes: 4 */
	/* padding: 28 */
} __attribute__((__aligned__(64)));
struct rchan {
	u32                        version;              /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	size_t                     subbuf_size;          /*     8     8 */
	size_t                     n_subbufs;            /*    16     8 */
	size_t                     alloc_size;           /*    24     8 */
	struct rchan_callbacks *   cb;                   /*    32     8 */
	struct kref                kref;                 /*    40     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     private_data;         /*    48     8 */
	size_t                     last_toobig;          /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct rchan_buf * *       buf;                  /*    64     8 */
	int                        is_global;            /*    72     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           list;                 /*    80    16 */
	struct dentry *            parent;               /*    96     8 */
	int                        has_base_filename;    /*   104     4 */
	char                       base_filename[255];   /*   108   255 */

	/* size: 368, cachelines: 6, members: 14 */
	/* sum members: 351, holes: 3, sum holes: 12 */
	/* padding: 5 */
	/* last cacheline: 48 bytes */
};
struct rchan_callbacks {
	int                        (*subbuf_start)(struct rchan_buf *, void *, void *, size_t); /*     0     8 */
	void                       (*buf_mapped)(struct rchan_buf *, struct file *); /*     8     8 */
	void                       (*buf_unmapped)(struct rchan_buf *, struct file *); /*    16     8 */
	struct dentry *            (*create_buf_file)(const char  *, struct dentry *, umode_t, struct rchan_buf *, int *); /*    24     8 */
	int                        (*remove_buf_file)(struct dentry *); /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct partial_page {
	unsigned int               offset;               /*     0     4 */
	unsigned int               len;                  /*     4     4 */
	long unsigned int          private;              /*     8     8 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct splice_pipe_desc {
	struct page * *            pages;                /*     0     8 */
	struct partial_page *      partial;              /*     8     8 */
	int                        nr_pages;             /*    16     4 */
	unsigned int               nr_pages_max;         /*    20     4 */
	const struct pipe_buf_operations  * ops;         /*    24     8 */
	void                       (*spd_release)(struct splice_pipe_desc *, unsigned int); /*    32     8 */

	/* size: 40, cachelines: 1, members: 6 */
	/* last cacheline: 40 bytes */
};
struct rchan_percpu_buf_dispatcher {
	struct rchan_buf *         buf;                  /*     0     8 */
	struct dentry *            dentry;               /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct genlmsghdr {
	__u8                       cmd;                  /*     0     1 */
	__u8                       version;              /*     1     1 */
	__u16                      reserved;             /*     2     2 */

	/* size: 4, cachelines: 1, members: 3 */
	/* last cacheline: 4 bytes */
};
struct genl_multicast_group {
	char                       name[16];             /*     0    16 */

	/* size: 16, cachelines: 1, members: 1 */
	/* last cacheline: 16 bytes */
};
struct genl_family {
	int                        id;                   /*     0     4 */
	unsigned int               hdrsize;              /*     4     4 */
	char                       name[16];             /*     8    16 */
	unsigned int               version;              /*    24     4 */
	unsigned int               maxattr;              /*    28     4 */
	unsigned int               mcgrp_offset;         /*    32     4 */
	u8                         netnsok:1;            /*    36: 0  1 */
	u8                         parallel_ops:1;       /*    36: 1  1 */

	/* XXX 6 bits hole, try to pack */

	u8                         n_ops;                /*    37     1 */
	u8                         n_small_ops;          /*    38     1 */
	u8                         n_mcgrps;             /*    39     1 */
	const struct nla_policy  * policy;               /*    40     8 */
	int                        (*pre_doit)(const struct genl_ops  *, struct sk_buff *, struct genl_info *); /*    48     8 */
	void                       (*post_doit)(const struct genl_ops  *, struct sk_buff *, struct genl_info *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	const struct genl_ops  *   ops;                  /*    64     8 */
	const struct genl_small_ops  * small_ops;        /*    72     8 */
	const struct genl_multicast_group  * mcgrps;     /*    80     8 */
	struct module *            module;               /*    88     8 */
	u64                        kabi_reserved1;       /*    96     8 */

	/* size: 104, cachelines: 2, members: 19 */
	/* sum members: 103 */
	/* sum bitfield members: 2 bits, bit holes: 1, sum bit holes: 6 bits */
	/* last cacheline: 40 bytes */
};
struct genl_ops {
	int                        (*doit)(struct sk_buff *, struct genl_info *); /*     0     8 */
	int                        (*start)(struct netlink_callback *); /*     8     8 */
	int                        (*dumpit)(struct sk_buff *, struct netlink_callback *); /*    16     8 */
	int                        (*done)(struct netlink_callback *); /*    24     8 */
	const struct nla_policy  * policy;               /*    32     8 */
	unsigned int               maxattr;              /*    40     4 */
	u8                         cmd;                  /*    44     1 */
	u8                         internal_flags;       /*    45     1 */
	u8                         flags;                /*    46     1 */
	u8                         validate;             /*    47     1 */
	u64                        kabi_reserved1;       /*    48     8 */
	u64                        kabi_reserved2;       /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u64                        kabi_reserved3;       /*    64     8 */
	u64                        kabi_reserved4;       /*    72     8 */

	/* size: 80, cachelines: 2, members: 14 */
	/* last cacheline: 16 bytes */
};
struct genl_info {
	u32                        snd_seq;              /*     0     4 */
	u32                        snd_portid;           /*     4     4 */
	struct nlmsghdr *          nlhdr;                /*     8     8 */
	struct genlmsghdr *        genlhdr;              /*    16     8 */
	void *                     userhdr;              /*    24     8 */
	struct nlattr * *          attrs;                /*    32     8 */
	possible_net_t             _net;                 /*    40     8 */
	void *                     user_ptr[2];          /*    48    16 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct netlink_ext_ack *   extack;               /*    64     8 */
	u64                        kabi_reserved1;       /*    72     8 */

	/* size: 80, cachelines: 2, members: 10 */
	/* last cacheline: 16 bytes */
};
struct genl_small_ops {
	int                        (*doit)(struct sk_buff *, struct genl_info *); /*     0     8 */
	int                        (*dumpit)(struct sk_buff *, struct netlink_callback *); /*     8     8 */
	u8                         cmd;                  /*    16     1 */
	u8                         internal_flags;       /*    17     1 */
	u8                         flags;                /*    18     1 */
	u8                         validate;             /*    19     1 */

	/* size: 24, cachelines: 1, members: 6 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct listener {
	struct list_head           list;                 /*     0    16 */
	pid_t                      pid;                  /*    16     4 */
	char                       valid;                /*    20     1 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 3 */
	/* last cacheline: 24 bytes */
};
struct listener_list {
	struct rw_semaphore        sem;                  /*     0    40 */
	struct list_head           list;                 /*    40    16 */

	/* size: 56, cachelines: 1, members: 2 */
	/* last cacheline: 56 bytes */
};
struct tp_module {
	struct list_head           list;                 /*     0    16 */
	struct module *            mod;                  /*    16     8 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct tp_transition_snapshot {
	long unsigned int          rcu;                  /*     0     8 */
	long unsigned int          srcu;                 /*     8     8 */
	bool                       ongoing;              /*    16     1 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 7 */
	/* last cacheline: 24 bytes */
};
struct tp_probes {
	struct callback_head       rcu __attribute__((__aligned__(8))); /*     0    16 */
	struct tracepoint_func     probes[];             /*    16     0 */

	/* size: 16, cachelines: 1, members: 2 */
	/* forced alignments: 1 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct ftrace_hash {
	long unsigned int          size_bits;            /*     0     8 */
	struct hlist_head *        buckets;              /*     8     8 */
	long unsigned int          count;                /*    16     8 */
	long unsigned int          flags;                /*    24     8 */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*    32    16 */

	/* size: 48, cachelines: 1, members: 5 */
	/* forced alignments: 1 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct ftrace_func_entry {
	struct hlist_node          hlist;                /*     0    16 */
	long unsigned int          ip;                   /*    16     8 */
	long unsigned int          direct;               /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* last cacheline: 32 bytes */
};
struct ftrace_graph_ent {
	long unsigned int          func;                 /*     0     8 */
	int                        depth;                /*     8     4 */

	/* size: 12, cachelines: 1, members: 2 */
	/* last cacheline: 12 bytes */
} __attribute__((__packed__));
struct ftrace_graph_ret {
	long unsigned int          func;                 /*     0     8 */
	long unsigned int          overrun;              /*     8     8 */
	long long unsigned int     calltime;             /*    16     8 */
	long long unsigned int     rettime;              /*    24     8 */
	int                        depth;                /*    32     4 */

	/* size: 36, cachelines: 1, members: 5 */
	/* last cacheline: 36 bytes */
} __attribute__((__packed__));
struct fgraph_ops {
	trace_func_graph_ent_t     entryfunc;            /*     0     8 */
	trace_func_graph_ret_t     retfunc;              /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct event_filter {
	struct prog_entry *        prog;                 /*     0     8 */
	char *                     filter_string;        /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct trace_array {
	struct list_head           list;                 /*     0    16 */
	char *                     name;                 /*    16     8 */
	struct array_buffer        array_buffer;         /*    24    40 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 1 boundary (64 bytes) --- */
	struct array_buffer        max_buffer;           /*    64    40 */

	/* XXX last struct has 4 bytes of padding */

	bool                       allocated_snapshot;   /*   104     1 */

	/* XXX 7 bytes hole, try to pack */

	long unsigned int          max_latency;          /*   112     8 */
	struct dentry *            d_max_latency;        /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct work_struct         fsnotify_work;        /*   128    64 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	struct irq_work            fsnotify_irqwork;     /*   192    24 */
	struct trace_pid_list *    filtered_pids;        /*   216     8 */
	struct trace_pid_list *    filtered_no_pids;     /*   224     8 */
	arch_spinlock_t            max_lock;             /*   232     4 */
	int                        buffer_disabled;      /*   236     4 */
	int                        sys_refcount_enter;   /*   240     4 */
	int                        sys_refcount_exit;    /*   244     4 */
	struct trace_event_file *  enter_syscall_files[457]; /*   248  3656 */
	/* --- cacheline 61 boundary (3904 bytes) --- */
	struct trace_event_file *  exit_syscall_files[457]; /*  3904  3656 */
	/* --- cacheline 118 boundary (7552 bytes) was 8 bytes ago --- */
	int                        stop_count;           /*  7560     4 */
	int                        clock_id;             /*  7564     4 */
	int                        nr_topts;             /*  7568     4 */
	bool                       clear_trace;          /*  7572     1 */

	/* XXX 3 bytes hole, try to pack */

	int                        buffer_percent;       /*  7576     4 */
	unsigned int               n_err_log_entries;    /*  7580     4 */
	struct tracer *            current_trace;        /*  7584     8 */
	unsigned int               trace_flags;          /*  7592     4 */
	unsigned char              trace_flags_index[32]; /*  7596    32 */
	/* --- cacheline 119 boundary (7616 bytes) was 12 bytes ago --- */
	unsigned int               flags;                /*  7628     4 */
	raw_spinlock_t             start_lock;           /*  7632     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           err_log;              /*  7640    16 */
	struct dentry *            dir;                  /*  7656     8 */
	struct dentry *            options;              /*  7664     8 */
	struct dentry *            percpu_dir;           /*  7672     8 */
	/* --- cacheline 120 boundary (7680 bytes) --- */
	struct dentry *            event_dir;            /*  7680     8 */
	struct trace_options *     topts;                /*  7688     8 */
	struct list_head           systems;              /*  7696    16 */
	struct list_head           events;               /*  7712    16 */
	struct trace_event_file *  trace_marker_file;    /*  7728     8 */
	cpumask_var_t              tracing_cpumask;      /*  7736     8 */
	/* --- cacheline 121 boundary (7744 bytes) --- */
	int                        ref;                  /*  7744     4 */
	int                        trace_ref;            /*  7748     4 */
	struct ftrace_ops *        ops;                  /*  7752     8 */
	struct trace_pid_list *    function_pids;        /*  7760     8 */
	struct trace_pid_list *    function_no_pids;     /*  7768     8 */
	struct list_head           func_probes;          /*  7776    16 */
	struct list_head           mod_trace;            /*  7792    16 */
	/* --- cacheline 122 boundary (7808 bytes) --- */
	struct list_head           mod_notrace;          /*  7808    16 */
	int                        function_enabled;     /*  7824     4 */
	int                        time_stamp_abs_ref;   /*  7828     4 */
	struct list_head           hist_vars;            /*  7832    16 */
	struct cond_snapshot *     cond_snapshot;        /*  7848     8 */

	/* size: 7856, cachelines: 123, members: 50 */
	/* sum members: 7842, holes: 3, sum holes: 14 */
	/* paddings: 2, sum paddings: 8 */
	/* last cacheline: 48 bytes */
};
struct tracer {
	const char  *              name;                 /*     0     8 */
	int                        (*init)(struct trace_array *); /*     8     8 */
	void                       (*reset)(struct trace_array *); /*    16     8 */
	void                       (*start)(struct trace_array *); /*    24     8 */
	void                       (*stop)(struct trace_array *); /*    32     8 */
	int                        (*update_thresh)(struct trace_array *); /*    40     8 */
	void                       (*open)(struct trace_iterator *); /*    48     8 */
	void                       (*pipe_open)(struct trace_iterator *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	void                       (*close)(struct trace_iterator *); /*    64     8 */
	void                       (*pipe_close)(struct trace_iterator *); /*    72     8 */
	ssize_t                    (*read)(struct trace_iterator *, struct file *, char *, size_t, loff_t *); /*    80     8 */
	ssize_t                    (*splice_read)(struct trace_iterator *, struct file *, loff_t *, struct pipe_inode_info *, size_t, unsigned int); /*    88     8 */
	void                       (*print_header)(struct seq_file *); /*    96     8 */
	enum print_line_t          (*print_line)(struct trace_iterator *); /*   104     8 */
	int                        (*set_flag)(struct trace_array *, u32, u32, int); /*   112     8 */
	int                        (*flag_changed)(struct trace_array *, u32, int); /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct tracer *            next;                 /*   128     8 */
	struct tracer_flags *      flags;                /*   136     8 */
	int                        enabled;              /*   144     4 */
	bool                       print_max;            /*   148     1 */
	bool                       allow_instances;      /*   149     1 */
	bool                       use_max_tr;           /*   150     1 */
	bool                       noboot;               /*   151     1 */

	/* size: 152, cachelines: 3, members: 23 */
	/* last cacheline: 24 bytes */
};
struct array_buffer {
	struct trace_array *       tr;                   /*     0     8 */
	struct trace_buffer *      buffer;               /*     8     8 */
	struct trace_array_cpu *   data;                 /*    16     8 */
	u64                        time_start;           /*    24     8 */
	int                        cpu;                  /*    32     4 */

	/* size: 40, cachelines: 1, members: 5 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct trace_subsystem_dir {
	struct list_head           list;                 /*     0    16 */
	struct event_subsystem *   subsystem;            /*    16     8 */
	struct trace_array *       tr;                   /*    24     8 */
	struct dentry *            entry;                /*    32     8 */
	int                        ref_count;            /*    40     4 */
	int                        nr_events;            /*    44     4 */

	/* size: 48, cachelines: 1, members: 6 */
	/* last cacheline: 48 bytes */
};
struct trace_array_cpu {
	atomic_t                   disabled;             /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     buffer_page;          /*     8     8 */
	long unsigned int          entries;              /*    16     8 */
	long unsigned int          saved_latency;        /*    24     8 */
	long unsigned int          critical_start;       /*    32     8 */
	long unsigned int          critical_end;         /*    40     8 */
	long unsigned int          critical_sequence;    /*    48     8 */
	long unsigned int          nice;                 /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	long unsigned int          policy;               /*    64     8 */
	long unsigned int          rt_priority;          /*    72     8 */
	long unsigned int          skipped_entries;      /*    80     8 */
	u64                        preempt_timestamp;    /*    88     8 */
	pid_t                      pid;                  /*    96     4 */
	kuid_t                     uid;                  /*   100     4 */
	char                       comm[16];             /*   104    16 */
	int                        ftrace_ignore_pid;    /*   120     4 */
	bool                       ignore_pid;           /*   124     1 */

	/* size: 128, cachelines: 2, members: 17 */
	/* sum members: 121, holes: 1, sum holes: 4 */
	/* padding: 3 */
};
struct trace_options {
	struct tracer *            tracer;               /*     0     8 */
	struct trace_option_dentry * topts;              /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct trace_option_dentry {
	struct tracer_opt *        opt;                  /*     0     8 */
	struct tracer_flags *      flags;                /*     8     8 */
	struct trace_array *       tr;                   /*    16     8 */
	struct dentry *            entry;                /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct trace_pid_list {
	int                        pid_max;              /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int *        pids;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct cond_snapshot {
	void *                     cond_data;            /*     0     8 */
	cond_update_fn_t           update;               /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct tracer_opt {
	const char  *              name;                 /*     0     8 */
	u32                        bit;                  /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct tracer_flags {
	u32                        val;                  /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct tracer_opt *        opts;                 /*     8     8 */
	struct tracer *            trace;                /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct ftrace_mod_load {
	struct list_head           list;                 /*     0    16 */
	char *                     func;                 /*    16     8 */
	char *                     module;               /*    24     8 */
	int                        enable;               /*    32     4 */

	/* size: 40, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct ftrace_func_command {
	struct list_head           list;                 /*     0    16 */
	char *                     name;                 /*    16     8 */
	int                        (*func)(struct trace_array *, struct ftrace_hash *, char *, char *, char *, int); /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* last cacheline: 32 bytes */
};
struct ftrace_probe_ops {
	void                       (*func)(long unsigned int, long unsigned int, struct trace_array *, struct ftrace_probe_ops *, void *); /*     0     8 */
	int                        (*init)(struct ftrace_probe_ops *, struct trace_array *, long unsigned int, void *, void * *); /*     8     8 */
	void                       (*free)(struct ftrace_probe_ops *, struct trace_array *, long unsigned int, void *); /*    16     8 */
	int                        (*print)(struct seq_file *, long unsigned int, struct ftrace_probe_ops *, void *); /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct trace_parser {
	bool                       cont;                 /*     0     1 */

	/* XXX 7 bytes hole, try to pack */

	char *                     buffer;               /*     8     8 */
	unsigned int               idx;                  /*    16     4 */
	unsigned int               size;                 /*    20     4 */

	/* size: 24, cachelines: 1, members: 4 */
	/* sum members: 17, holes: 1, sum holes: 7 */
	/* last cacheline: 24 bytes */
};
struct event_subsystem {
	struct list_head           list;                 /*     0    16 */
	const char  *              name;                 /*    16     8 */
	struct event_filter *      filter;               /*    24     8 */
	int                        ref_count;            /*    32     4 */

	/* size: 40, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct tracer_stat {
	const char  *              name;                 /*     0     8 */
	void *                     (*stat_start)(struct tracer_stat *); /*     8     8 */
	void *                     (*stat_next)(void *, int); /*    16     8 */
	cmp_func_t                 stat_cmp;             /*    24     8 */
	int                        (*stat_show)(struct seq_file *, void *); /*    32     8 */
	void                       (*stat_release)(void *); /*    40     8 */
	int                        (*stat_headers)(struct seq_file *); /*    48     8 */

	/* size: 56, cachelines: 1, members: 7 */
	/* last cacheline: 56 bytes */
};
struct ftrace_profile {
	struct hlist_node          node;                 /*     0    16 */
	long unsigned int          ip;                   /*    16     8 */
	long unsigned int          counter;              /*    24     8 */
	long long unsigned int     time;                 /*    32     8 */
	long long unsigned int     time_squared;         /*    40     8 */

	/* size: 48, cachelines: 1, members: 5 */
	/* last cacheline: 48 bytes */
};
struct ftrace_profile_page {
	struct ftrace_profile_page * next;               /*     0     8 */
	long unsigned int          index;                /*     8     8 */
	struct ftrace_profile      records[];            /*    16     0 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct ftrace_profile_stat {
	atomic_t                   disabled;             /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct hlist_head *        hash;                 /*     8     8 */
	struct ftrace_profile_page * pages;              /*    16     8 */
	struct ftrace_profile_page * start;              /*    24     8 */
	struct tracer_stat         stat;                 /*    32    56 */

	/* size: 88, cachelines: 2, members: 5 */
	/* sum members: 84, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct ftrace_func_probe {
	struct ftrace_probe_ops *  probe_ops;            /*     0     8 */
	struct ftrace_ops          ops;                  /*     8   176 */
	/* --- cacheline 2 boundary (128 bytes) was 56 bytes ago --- */
	struct trace_array *       tr;                   /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	struct list_head           list;                 /*   192    16 */
	void *                     data;                 /*   208     8 */
	int                        ref;                  /*   216     4 */

	/* size: 224, cachelines: 4, members: 6 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct ftrace_page {
	struct ftrace_page *       next;                 /*     0     8 */
	struct dyn_ftrace *        records;              /*     8     8 */
	int                        index;                /*    16     4 */
	int                        size;                 /*    20     4 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct ftrace_rec_iter {
	struct ftrace_page *       pg;                   /*     0     8 */
	int                        index;                /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct ftrace_iterator {
	loff_t                     pos;                  /*     0     8 */
	loff_t                     func_pos;             /*     8     8 */
	loff_t                     mod_pos;              /*    16     8 */
	struct ftrace_page *       pg;                   /*    24     8 */
	struct dyn_ftrace *        func;                 /*    32     8 */
	struct ftrace_func_probe * probe;                /*    40     8 */
	struct ftrace_func_entry * probe_entry;          /*    48     8 */
	struct trace_parser        parser;               /*    56    24 */
	/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
	struct ftrace_hash *       hash;                 /*    80     8 */
	struct ftrace_ops *        ops;                  /*    88     8 */
	struct trace_array *       tr;                   /*    96     8 */
	struct list_head *         mod_list;             /*   104     8 */
	int                        pidx;                 /*   112     4 */
	int                        idx;                  /*   116     4 */
	unsigned int               flags;                /*   120     4 */

	/* size: 128, cachelines: 2, members: 15 */
	/* padding: 4 */
};
struct ftrace_glob {
	char *                     search;               /*     0     8 */
	unsigned int               len;                  /*     8     4 */
	int                        type;                 /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct ftrace_func_map {
	struct ftrace_func_entry   entry;                /*     0    32 */
	void *                     data;                 /*    32     8 */

	/* size: 40, cachelines: 1, members: 2 */
	/* last cacheline: 40 bytes */
};
struct ftrace_func_mapper {
	struct ftrace_hash         hash __attribute__((__aligned__(8))); /*     0    48 */

	/* size: 48, cachelines: 1, members: 1 */
	/* forced alignments: 1 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct ftrace_direct_func {
	struct list_head           next;                 /*     0    16 */
	long unsigned int          addr;                 /*    16     8 */
	int                        count;                /*    24     4 */

	/* size: 32, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct ftrace_graph_data {
	struct ftrace_hash *       hash;                 /*     0     8 */
	struct ftrace_func_entry * entry;                /*     8     8 */
	int                        idx;                  /*    16     4 */
	enum graph_filter_type     type;                 /*    20     4 */
	struct ftrace_hash *       new_hash;             /*    24     8 */
	const struct seq_operations  * seq_ops;          /*    32     8 */
	struct trace_parser        parser;               /*    40    24 */

	/* size: 64, cachelines: 1, members: 7 */
};
struct ftrace_mod_func {
	struct list_head           list;                 /*     0    16 */
	char *                     name;                 /*    16     8 */
	long unsigned int          ip;                   /*    24     8 */
	unsigned int               size;                 /*    32     4 */

	/* size: 40, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct ftrace_mod_map {
	struct callback_head       rcu __attribute__((__aligned__(8))); /*     0    16 */
	struct list_head           list;                 /*    16    16 */
	struct module *            mod;                  /*    32     8 */
	long unsigned int          start_addr;           /*    40     8 */
	long unsigned int          end_addr;             /*    48     8 */
	struct list_head           funcs;                /*    56    16 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	unsigned int               num_funcs;            /*    72     4 */

	/* size: 80, cachelines: 2, members: 7 */
	/* padding: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct ftrace_init_func {
	struct list_head           list;                 /*     0    16 */
	long unsigned int          ip;                   /*    16     8 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct ring_buffer_iter {
	struct ring_buffer_per_cpu * cpu_buffer;         /*     0     8 */
	long unsigned int          head;                 /*     8     8 */
	long unsigned int          next_event;           /*    16     8 */
	struct buffer_page *       head_page;            /*    24     8 */
	struct buffer_page *       cache_reader_page;    /*    32     8 */
	long unsigned int          cache_read;           /*    40     8 */
	u64                        read_stamp;           /*    48     8 */
	u64                        page_stamp;           /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct ring_buffer_event * event;                /*    64     8 */
	int                        missed_events;        /*    72     4 */

	/* size: 80, cachelines: 2, members: 10 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct trace_buffer {
	unsigned int               flags;                /*     0     4 */
	int                        cpus;                 /*     4     4 */
	atomic_t                   record_disabled;      /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	cpumask_var_t              cpumask;              /*    16     8 */
	struct lock_class_key *    reader_lock_key;      /*    24     8 */
	struct mutex               mutex;                /*    32    32 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct ring_buffer_per_cpu * * buffers;          /*    64     8 */
	struct hlist_node          node;                 /*    72    16 */
	u64                        (*clock)(void);       /*    88     8 */
	struct rb_irq_work         irq_work;             /*    96    80 */

	/* XXX last struct has 5 bytes of padding */

	/* --- cacheline 2 boundary (128 bytes) was 48 bytes ago --- */
	bool                       time_stamp_abs;       /*   176     1 */

	/* size: 184, cachelines: 3, members: 11 */
	/* sum members: 173, holes: 1, sum holes: 4 */
	/* padding: 7 */
	/* paddings: 1, sum paddings: 5 */
	/* last cacheline: 56 bytes */
};
struct buffer_data_page {
	u64                        time_stamp;           /*     0     8 */
	local_t                    commit;               /*     8     8 */
	unsigned char              data[] __attribute__((__aligned__(4))); /*    16     0 */

	/* size: 16, cachelines: 1, members: 3 */
	/* forced alignments: 1 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct buffer_page {
	struct list_head           list;                 /*     0    16 */
	local_t                    write;                /*    16     8 */
	unsigned int               read;                 /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	local_t                    entries;              /*    32     8 */
	long unsigned int          real_end;             /*    40     8 */
	struct buffer_data_page *  page;                 /*    48     8 */

	/* size: 56, cachelines: 1, members: 6 */
	/* sum members: 52, holes: 1, sum holes: 4 */
	/* last cacheline: 56 bytes */
};
struct rb_irq_work {
	struct irq_work            work;                 /*     0    24 */
	wait_queue_head_t          waiters;              /*    24    24 */
	wait_queue_head_t          full_waiters;         /*    48    24 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	bool                       waiters_pending;      /*    72     1 */
	bool                       full_waiters_pending; /*    73     1 */
	bool                       wakeup_full;          /*    74     1 */

	/* size: 80, cachelines: 2, members: 6 */
	/* padding: 5 */
	/* last cacheline: 16 bytes */
};
struct rb_event_info {
	u64                        ts;                   /*     0     8 */
	u64                        delta;                /*     8     8 */
	u64                        before;               /*    16     8 */
	u64                        after;                /*    24     8 */
	long unsigned int          length;               /*    32     8 */
	struct buffer_page *       tail_page;            /*    40     8 */
	int                        add_timestamp;        /*    48     4 */

	/* size: 56, cachelines: 1, members: 7 */
	/* padding: 4 */
	/* last cacheline: 56 bytes */
};
struct rb_time_struct {
	local64_t                  time;                 /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct ring_buffer_per_cpu {
	int                        cpu;                  /*     0     4 */
	atomic_t                   record_disabled;      /*     4     4 */
	atomic_t                   resize_disabled;      /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct trace_buffer *      buffer;               /*    16     8 */
	raw_spinlock_t             reader_lock;          /*    24     4 */
	arch_spinlock_t            lock;                 /*    28     4 */
	struct lock_class_key      lock_key;             /*    32     0 */
	struct buffer_data_page *  free_page;            /*    32     8 */
	long unsigned int          nr_pages;             /*    40     8 */
	unsigned int               current_context;      /*    48     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head *         pages;                /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct buffer_page *       head_page;            /*    64     8 */
	struct buffer_page *       tail_page;            /*    72     8 */
	struct buffer_page *       commit_page;          /*    80     8 */
	struct buffer_page *       reader_page;          /*    88     8 */
	long unsigned int          lost_events;          /*    96     8 */
	long unsigned int          last_overrun;         /*   104     8 */
	long unsigned int          nest;                 /*   112     8 */
	local_t                    entries_bytes;        /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	local_t                    entries;              /*   128     8 */
	local_t                    overrun;              /*   136     8 */
	local_t                    commit_overrun;       /*   144     8 */
	local_t                    dropped_events;       /*   152     8 */
	local_t                    committing;           /*   160     8 */
	local_t                    commits;              /*   168     8 */
	local_t                    pages_touched;        /*   176     8 */
	local_t                    pages_read;           /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	long int                   last_pages_touch;     /*   192     8 */
	size_t                     shortest_full;        /*   200     8 */
	long unsigned int          read;                 /*   208     8 */
	long unsigned int          read_bytes;           /*   216     8 */
	rb_time_t                  write_stamp;          /*   224     8 */
	rb_time_t                  before_stamp;         /*   232     8 */
	u64                        read_stamp;           /*   240     8 */
	long int                   nr_pages_to_update;   /*   248     8 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	struct list_head           new_pages;            /*   256    16 */
	struct work_struct         update_pages_work;    /*   272    64 */
	/* --- cacheline 5 boundary (320 bytes) was 16 bytes ago --- */
	struct completion          update_done;          /*   336    32 */
	struct rb_irq_work         irq_work;             /*   368    80 */

	/* XXX last struct has 5 bytes of padding */

	/* size: 448, cachelines: 7, members: 39 */
	/* sum members: 440, holes: 2, sum holes: 8 */
	/* paddings: 1, sum paddings: 5 */
};
struct trace_export {
	struct trace_export *      next;                 /*     0     8 */
	void                       (*write)(struct trace_export *, const void  *, unsigned int); /*     8     8 */
	int                        flags;                /*    16     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct ftrace_entry {
	struct trace_entry         ent;                  /*     0     8 */
	long unsigned int          ip;                   /*     8     8 */
	long unsigned int          parent_ip;            /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct stack_entry {
	struct trace_entry         ent;                  /*     0     8 */
	int                        size;                 /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          caller[8];            /*    16    64 */

	/* size: 80, cachelines: 2, members: 3 */
	/* sum members: 76, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct userstack_entry {
	struct trace_entry         ent;                  /*     0     8 */
	unsigned int               tgid;                 /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          caller[8];            /*    16    64 */

	/* size: 80, cachelines: 2, members: 3 */
	/* sum members: 76, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct bprint_entry {
	struct trace_entry         ent;                  /*     0     8 */
	long unsigned int          ip;                   /*     8     8 */
	const char  *              fmt;                  /*    16     8 */
	u32                        buf[];                /*    24     0 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct print_entry {
	struct trace_entry         ent;                  /*     0     8 */
	long unsigned int          ip;                   /*     8     8 */
	char                       buf[];                /*    16     0 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct raw_data_entry {
	struct trace_entry         ent;                  /*     0     8 */
	unsigned int               id;                   /*     8     4 */
	char                       buf[];                /*    12     0 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct bputs_entry {
	struct trace_entry         ent;                  /*     0     8 */
	long unsigned int          ip;                   /*     8     8 */
	const char  *              str;                  /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct trace_min_max_param {
	struct mutex *             lock;                 /*     0     8 */
	u64 *                      val;                  /*     8     8 */
	u64 *                      min;                  /*    16     8 */
	u64 *                      max;                  /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct saved_cmdlines_buffer {
	unsigned int               map_pid_to_cmdline[32769]; /*     0 131076 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 2048 boundary (131072 bytes) was 8 bytes ago --- */
	unsigned int *             map_cmdline_to_pid;   /* 131080     8 */
	unsigned int               cmdline_num;          /* 131088     4 */
	int                        cmdline_idx;          /* 131092     4 */
	char *                     saved_cmdlines;       /* 131096     8 */

	/* size: 131104, cachelines: 2049, members: 5 */
	/* sum members: 131100, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct ftrace_stack {
	long unsigned int          calls[1024];          /*     0  8192 */

	/* size: 8192, cachelines: 128, members: 1 */
};
struct ftrace_stacks {
	struct ftrace_stack        stacks[4];            /*     0 32768 */

	/* size: 32768, cachelines: 512, members: 1 */
};
struct trace_buffer_struct {
	int                        nesting;              /*     0     4 */
	char                       buffer[4][1024];      /*     4  4096 */

	/* size: 4100, cachelines: 65, members: 2 */
	/* last cacheline: 4 bytes */
};
struct ftrace_buffer_info {
	struct trace_iterator      iter;                 /*     0  8448 */
	/* --- cacheline 132 boundary (8448 bytes) --- */
	void *                     spare;                /*  8448     8 */
	unsigned int               spare_cpu;            /*  8456     4 */
	unsigned int               read;                 /*  8460     4 */

	/* size: 8464, cachelines: 133, members: 4 */
	/* last cacheline: 16 bytes */
};
struct err_info {
	const char  * *            errs;                 /*     0     8 */
	u8                         type;                 /*     8     1 */
	u8                         pos;                  /*     9     1 */

	/* XXX 6 bytes hole, try to pack */

	u64                        ts;                   /*    16     8 */

	/* size: 24, cachelines: 1, members: 4 */
	/* sum members: 18, holes: 1, sum holes: 6 */
	/* last cacheline: 24 bytes */
};
struct tracing_log_err {
	struct list_head           list;                 /*     0    16 */
	struct err_info            info;                 /*    16    24 */
	char                       loc[128];             /*    40   128 */
	/* --- cacheline 2 boundary (128 bytes) was 40 bytes ago --- */
	char                       cmd[256];             /*   168   256 */

	/* size: 424, cachelines: 7, members: 4 */
	/* last cacheline: 40 bytes */
};
struct buffer_ref {
	struct trace_buffer *      buffer;               /*     0     8 */
	void *                     page;                 /*     8     8 */
	int                        cpu;                  /*    16     4 */
	refcount_t                 refcount;             /*    20     4 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct ctx_switch_entry {
	struct trace_entry         ent;                  /*     0     8 */
	unsigned int               prev_pid;             /*     8     4 */
	unsigned int               next_pid;             /*    12     4 */
	unsigned int               next_cpu;             /*    16     4 */
	unsigned char              prev_prio;            /*    20     1 */
	unsigned char              prev_state;           /*    21     1 */
	unsigned char              next_prio;            /*    22     1 */
	unsigned char              next_state;           /*    23     1 */

	/* size: 24, cachelines: 1, members: 8 */
	/* last cacheline: 24 bytes */
};
struct hwlat_entry {
	struct trace_entry         ent;                  /*     0     8 */
	u64                        duration;             /*     8     8 */
	u64                        outer_duration;       /*    16     8 */
	u64                        nmi_total_ts;         /*    24     8 */
	struct timespec64          timestamp;            /*    32    16 */
	unsigned int               nmi_count;            /*    48     4 */
	unsigned int               seqnum;               /*    52     4 */
	unsigned int               count;                /*    56     4 */

	/* size: 64, cachelines: 1, members: 8 */
	/* padding: 4 */
};
struct osnoise_entry {
	struct trace_entry         ent;                  /*     0     8 */
	u64                        noise;                /*     8     8 */
	u64                        runtime;              /*    16     8 */
	u64                        max_sample;           /*    24     8 */
	unsigned int               hw_count;             /*    32     4 */
	unsigned int               nmi_count;            /*    36     4 */
	unsigned int               irq_count;            /*    40     4 */
	unsigned int               softirq_count;        /*    44     4 */
	unsigned int               thread_count;         /*    48     4 */

	/* size: 56, cachelines: 1, members: 9 */
	/* padding: 4 */
	/* last cacheline: 56 bytes */
};
struct timerlat_entry {
	struct trace_entry         ent;                  /*     0     8 */
	unsigned int               seqnum;               /*     8     4 */
	int                        context;              /*    12     4 */
	u64                        timer_latency;        /*    16     8 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct trace_mark {
	long long unsigned int     val;                  /*     0     8 */
	char                       sym;                  /*     8     1 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 7 */
	/* last cacheline: 16 bytes */
};
struct stat_node {
	struct rb_node             node __attribute__((__aligned__(8))); /*     0    24 */
	void *                     stat;                 /*    24     8 */

	/* size: 32, cachelines: 1, members: 2 */
	/* forced alignments: 1 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct stat_session {
	struct list_head           session_list;         /*     0    16 */
	struct tracer_stat *       ts;                   /*    16     8 */
	struct rb_root             stat_root;            /*    24     8 */
	struct mutex               stat_mutex;           /*    32    32 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct dentry *            file;                 /*    64     8 */

	/* size: 72, cachelines: 2, members: 5 */
	/* last cacheline: 8 bytes */
};
struct trace_bprintk_fmt {
	struct list_head           list;                 /*     0    16 */
	const char  *              fmt;                  /*    16     8 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct tracing_map_field {
	tracing_map_cmp_fn_t       cmp_fn;               /*     0     8 */
	union {
		atomic64_t         sum;                  /*     8     8 */
		unsigned int       offset;               /*     8     4 */
	};                                               /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct tracing_map_elt {
	struct tracing_map *       map;                  /*     0     8 */
	struct tracing_map_field * fields;               /*     8     8 */
	atomic64_t *               vars;                 /*    16     8 */
	bool *                     var_set;              /*    24     8 */
	void *                     key;                  /*    32     8 */
	void *                     private_data;         /*    40     8 */

	/* size: 48, cachelines: 1, members: 6 */
	/* last cacheline: 48 bytes */
};
struct tracing_map {
	unsigned int               key_size;             /*     0     4 */
	unsigned int               map_bits;             /*     4     4 */
	unsigned int               map_size;             /*     8     4 */
	unsigned int               max_elts;             /*    12     4 */
	atomic_t                   next_elt;             /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	struct tracing_map_array * elts;                 /*    24     8 */
	struct tracing_map_array * map;                  /*    32     8 */
	const struct tracing_map_ops  * ops;             /*    40     8 */
	void *                     private_data;         /*    48     8 */
	struct tracing_map_field   fields[6];            /*    56    96 */
	/* --- cacheline 2 boundary (128 bytes) was 24 bytes ago --- */
	unsigned int               n_fields;             /*   152     4 */
	int                        key_idx[3];           /*   156    12 */
	unsigned int               n_keys;               /*   168     4 */
	struct tracing_map_sort_key sort_key;            /*   172     8 */

	/* XXX last struct has 3 bytes of padding */

	unsigned int               n_vars;               /*   180     4 */
	atomic64_t                 hits;                 /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	atomic64_t                 drops;                /*   192     8 */

	/* size: 200, cachelines: 4, members: 17 */
	/* sum members: 196, holes: 1, sum holes: 4 */
	/* paddings: 1, sum paddings: 3 */
	/* last cacheline: 8 bytes */
};
struct tracing_map_entry {
	u32                        key;                  /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct tracing_map_elt *   val;                  /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct tracing_map_sort_key {
	unsigned int               field_idx;            /*     0     4 */
	bool                       descending;           /*     4     1 */

	/* size: 8, cachelines: 1, members: 2 */
	/* padding: 3 */
	/* last cacheline: 8 bytes */
};
struct tracing_map_sort_entry {
	void *                     key;                  /*     0     8 */
	struct tracing_map_elt *   elt;                  /*     8     8 */
	bool                       elt_copied;           /*    16     1 */
	bool                       dup;                  /*    17     1 */

	/* size: 24, cachelines: 1, members: 4 */
	/* padding: 6 */
	/* last cacheline: 24 bytes */
};
struct tracing_map_array {
	unsigned int               entries_per_page;     /*     0     4 */
	unsigned int               entry_size_shift;     /*     4     4 */
	unsigned int               entry_shift;          /*     8     4 */
	unsigned int               entry_mask;           /*    12     4 */
	unsigned int               n_pages;              /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	void * *                   pages;                /*    24     8 */

	/* size: 32, cachelines: 1, members: 6 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct tracing_map_ops {
	int                        (*elt_alloc)(struct tracing_map_elt *); /*     0     8 */
	void                       (*elt_free)(struct tracing_map_elt *); /*     8     8 */
	void                       (*elt_clear)(struct tracing_map_elt *); /*    16     8 */
	void                       (*elt_init)(struct tracing_map_elt *); /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct hwlat_sample {
	u64                        seqnum;               /*     0     8 */
	u64                        duration;             /*     8     8 */
	u64                        outer_duration;       /*    16     8 */
	u64                        nmi_total_ts;         /*    24     8 */
	struct timespec64          timestamp;            /*    32    16 */
	int                        nmi_count;            /*    48     4 */
	int                        count;                /*    52     4 */

	/* size: 56, cachelines: 1, members: 7 */
	/* last cacheline: 56 bytes */
};
struct hwlat_data {
	struct mutex               lock;                 /*     0    32 */
	u64                        count;                /*    32     8 */
	u64                        sample_window;        /*    40     8 */
	u64                        sample_width;         /*    48     8 */

	/* size: 56, cachelines: 1, members: 4 */
	/* last cacheline: 56 bytes */
};
struct trace_event_raw_thread_noise {
	struct trace_entry         ent;                  /*     0     8 */
	char                       comm[16];             /*     8    16 */
	u64                        start;                /*    24     8 */
	u64                        duration;             /*    32     8 */
	pid_t                      pid;                  /*    40     4 */
	char                       __data[];             /*    44     0 */

	/* size: 48, cachelines: 1, members: 6 */
	/* padding: 4 */
	/* last cacheline: 48 bytes */
};
struct trace_event_raw_softirq_noise {
	struct trace_entry         ent;                  /*     0     8 */
	u64                        start;                /*     8     8 */
	u64                        duration;             /*    16     8 */
	int                        vector;               /*    24     4 */
	char                       __data[];             /*    28     0 */

	/* size: 32, cachelines: 1, members: 5 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct trace_event_raw_irq_noise {
	struct trace_entry         ent;                  /*     0     8 */
	u64                        start;                /*     8     8 */
	u64                        duration;             /*    16     8 */
	u32                        __data_loc_desc;      /*    24     4 */
	int                        vector;               /*    28     4 */
	char                       __data[];             /*    32     0 */

	/* size: 32, cachelines: 1, members: 6 */
	/* last cacheline: 32 bytes */
};
struct trace_event_raw_nmi_noise {
	struct trace_entry         ent;                  /*     0     8 */
	u64                        start;                /*     8     8 */
	u64                        duration;             /*    16     8 */
	char                       __data[];             /*    24     0 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_sample_threshold {
	struct trace_entry         ent;                  /*     0     8 */
	u64                        start;                /*     8     8 */
	u64                        duration;             /*    16     8 */
	u64                        interference;         /*    24     8 */
	char                       __data[];             /*    32     0 */

	/* size: 32, cachelines: 1, members: 5 */
	/* last cacheline: 32 bytes */
};
struct trace_event_data_offsets_thread_noise {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_softirq_noise {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_irq_noise {
	u32                        desc;                 /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct trace_event_data_offsets_nmi_noise {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_sample_threshold {

	/* size: 0, cachelines: 0, members: 0 */
};
struct osnoise_instance {
	struct list_head           list;                 /*     0    16 */
	struct trace_array *       tr;                   /*    16     8 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct osn_nmi {
	u64                        count;                /*     0     8 */
	u64                        delta_start;          /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct osn_irq {
	u64                        count;                /*     0     8 */
	u64                        arrival_time;         /*     8     8 */
	u64                        delta_start;          /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct osn_softirq {
	u64                        count;                /*     0     8 */
	u64                        arrival_time;         /*     8     8 */
	u64                        delta_start;          /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct osn_thread {
	u64                        count;                /*     0     8 */
	u64                        arrival_time;         /*     8     8 */
	u64                        delta_start;          /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct osnoise_variables {
	struct task_struct *       kthread;              /*     0     8 */
	bool                       sampling;             /*     8     1 */

	/* XXX 3 bytes hole, try to pack */

	pid_t                      pid;                  /*    12     4 */
	struct osn_nmi             nmi;                  /*    16    16 */
	struct osn_irq             irq;                  /*    32    24 */
	struct osn_softirq         softirq;              /*    56    24 */
	/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
	struct osn_thread          thread;               /*    80    24 */
	local_t                    int_counter;          /*   104     8 */

	/* size: 112, cachelines: 2, members: 8 */
	/* sum members: 109, holes: 1, sum holes: 3 */
	/* last cacheline: 48 bytes */
};
struct timerlat_variables {
	struct task_struct *       kthread;              /*     0     8 */
	struct hrtimer             timer __attribute__((__aligned__(8))); /*     8    96 */
	/* --- cacheline 1 boundary (64 bytes) was 40 bytes ago --- */
	u64                        rel_period;           /*   104     8 */
	u64                        abs_period;           /*   112     8 */
	bool                       tracing_thread;       /*   120     1 */

	/* XXX 7 bytes hole, try to pack */

	/* --- cacheline 2 boundary (128 bytes) --- */
	u64                        count;                /*   128     8 */

	/* size: 136, cachelines: 3, members: 6 */
	/* sum members: 129, holes: 1, sum holes: 7 */
	/* forced alignments: 1 */
	/* last cacheline: 8 bytes */
} __attribute__((__aligned__(8)));
struct osnoise_sample {
	u64                        runtime;              /*     0     8 */
	u64                        noise;                /*     8     8 */
	u64                        max_sample;           /*    16     8 */
	int                        hw_count;             /*    24     4 */
	int                        nmi_count;            /*    28     4 */
	int                        irq_count;            /*    32     4 */
	int                        softirq_count;        /*    36     4 */
	int                        thread_count;         /*    40     4 */

	/* size: 48, cachelines: 1, members: 8 */
	/* padding: 4 */
	/* last cacheline: 48 bytes */
};
struct timerlat_sample {
	u64                        timer_latency;        /*     0     8 */
	unsigned int               seqnum;               /*     8     4 */
	int                        context;              /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct osnoise_data {
	u64                        sample_period;        /*     0     8 */
	u64                        sample_runtime;       /*     8     8 */
	u64                        stop_tracing;         /*    16     8 */
	u64                        stop_tracing_total;   /*    24     8 */
	u64                        timerlat_period;      /*    32     8 */
	u64                        print_stack;          /*    40     8 */
	int                        timerlat_tracer;      /*    48     4 */
	bool                       tainted;              /*    52     1 */

	/* size: 56, cachelines: 1, members: 8 */
	/* padding: 3 */
	/* last cacheline: 56 bytes */
};
struct trace_stack {
	int                        stack_size;           /*     0     4 */
	int                        nr_entries;           /*     4     4 */
	long unsigned int          calls[256];           /*     8  2048 */

	/* size: 2056, cachelines: 33, members: 3 */
	/* last cacheline: 8 bytes */
};
struct ftrace_graph_ent_entry {
	struct trace_entry         ent;                  /*     0     8 */
	struct ftrace_graph_ent    graph_ent;            /*     8    12 */

	/* size: 20, cachelines: 1, members: 2 */
	/* last cacheline: 20 bytes */
} __attribute__((__packed__));
struct ftrace_graph_ret_entry {
	struct trace_entry         ent;                  /*     0     8 */
	struct ftrace_graph_ret    ret;                  /*     8    36 */

	/* size: 44, cachelines: 1, members: 2 */
	/* last cacheline: 44 bytes */
} __attribute__((__packed__));
struct fgraph_cpu_data {
	pid_t                      last_pid;             /*     0     4 */
	int                        depth;                /*     4     4 */
	int                        depth_irq;            /*     8     4 */
	int                        ignore;               /*    12     4 */
	long unsigned int          enter_funcs[50];      /*    16   400 */

	/* size: 416, cachelines: 7, members: 5 */
	/* last cacheline: 32 bytes */
};
struct fgraph_data {
	struct fgraph_cpu_data *   cpu_data;             /*     0     8 */
	struct ftrace_graph_ent_entry ent;               /*     8    20 */
	struct ftrace_graph_ret_entry ret;               /*    28    44 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	int                        failed;               /*    72     4 */
	int                        cpu;                  /*    76     4 */

	/* size: 80, cachelines: 2, members: 5 */
	/* last cacheline: 16 bytes */
} __attribute__((__packed__));
struct disk_stats {
	u64                        nsecs[4];             /*     0    32 */
	long unsigned int          sectors[4];           /*    32    32 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	long unsigned int          ios[4];               /*    64    32 */
	long unsigned int          merges[4];            /*    96    32 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	long unsigned int          io_ticks;             /*   128     8 */
	local_t                    in_flight[2];         /*   136    16 */

	/* size: 152, cachelines: 3, members: 6 */
	/* last cacheline: 24 bytes */
};
struct sbitmap_word {
	long unsigned int          depth;                /*     0     8 */

	/* XXX 56 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	long unsigned int          word __attribute__((__aligned__(64))); /*    64     8 */

	/* XXX 56 bytes hole, try to pack */

	/* --- cacheline 2 boundary (128 bytes) --- */
	long unsigned int          cleared __attribute__((__aligned__(64))); /*   128     8 */
	spinlock_t                 swap_lock;            /*   136     4 */

	/* size: 192, cachelines: 3, members: 4 */
	/* sum members: 28, holes: 2, sum holes: 112 */
	/* padding: 52 */
	/* forced alignments: 2, forced holes: 2, sum forced holes: 112 */
} __attribute__((__aligned__(64)));
struct sbitmap {
	unsigned int               depth;                /*     0     4 */
	unsigned int               shift;                /*     4     4 */
	unsigned int               map_nr;               /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct sbitmap_word *      map;                  /*    16     8 */
	u64                        kabi_reserved1;       /*    24     8 */

	/* size: 32, cachelines: 1, members: 5 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct sbq_wait_state {
	atomic_t                   wait_cnt;             /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	wait_queue_head_t          wait;                 /*     8    24 */

	/* size: 64, cachelines: 1, members: 2 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* padding: 32 */
} __attribute__((__aligned__(64)));
struct sbitmap_queue {
	struct sbitmap             sb;                   /*     0    32 */
	unsigned int *             alloc_hint;           /*    32     8 */
	unsigned int               wake_batch;           /*    40     4 */
	atomic_t                   wake_index;           /*    44     4 */
	struct sbq_wait_state *    ws;                   /*    48     8 */
	atomic_t                   ws_active;            /*    56     4 */
	bool                       round_robin;          /*    60     1 */

	/* XXX 3 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	unsigned int               min_shallow_depth;    /*    64     4 */

	/* size: 72, cachelines: 2, members: 8 */
	/* sum members: 65, holes: 1, sum holes: 3 */
	/* padding: 4 */
	/* last cacheline: 8 bytes */
};
struct blk_mq_ctx {
	struct {
		spinlock_t         lock;                 /*     0     4 */

		/* XXX 4 bytes hole, try to pack */

		struct list_head   rq_lists[3];          /*     8    48 */
	} __attribute__((__aligned__(64))) __attribute__((__aligned__(64)));              /*     0    64 */

	/* XXX last struct has 8 bytes of padding */

	/* --- cacheline 1 boundary (64 bytes) --- */
	unsigned int               cpu;                  /*    64     4 */
	short unsigned int         index_hw[3];          /*    68     6 */

	/* XXX 6 bytes hole, try to pack */

	struct blk_mq_hw_ctx *     hctxs[3];             /*    80    24 */
	long unsigned int          rq_dispatched[2];     /*   104    16 */
	long unsigned int          rq_merged;            /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	long unsigned int          rq_completed[2] __attribute__((__aligned__(64))); /*   128    16 */
	struct request_queue *     queue;                /*   144     8 */
	struct blk_mq_ctxs *       ctxs;                 /*   152     8 */
	struct kobject             kobj;                 /*   160    96 */

	/* size: 256, cachelines: 4, members: 10 */
	/* sum members: 250, holes: 1, sum holes: 6 */
	/* paddings: 1, sum paddings: 8 */
	/* forced alignments: 2 */
} __attribute__((__aligned__(64)));
struct blk_mq_hw_ctx {
	struct {
		spinlock_t         lock;                 /*     0     4 */

		/* XXX 4 bytes hole, try to pack */

		struct list_head   dispatch;             /*     8    16 */
		long unsigned int  state;                /*    24     8 */
	} __attribute__((__aligned__(64))) __attribute__((__aligned__(64)));              /*     0    64 */

	/* XXX last struct has 32 bytes of padding */

	/* --- cacheline 1 boundary (64 bytes) --- */
	struct delayed_work        run_work;             /*    64   192 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	cpumask_var_t              cpumask;              /*   256     8 */
	int                        next_cpu;             /*   264     4 */
	int                        next_cpu_batch;       /*   268     4 */
	long unsigned int          flags;                /*   272     8 */
	void *                     sched_data;           /*   280     8 */
	struct request_queue *     queue;                /*   288     8 */
	struct blk_flush_queue *   fq;                   /*   296     8 */
	void *                     driver_data;          /*   304     8 */
	struct sbitmap             ctx_map;              /*   312    32 */
	/* --- cacheline 5 boundary (320 bytes) was 24 bytes ago --- */
	struct blk_mq_ctx *        dispatch_from;        /*   344     8 */
	unsigned int               dispatch_busy;        /*   352     4 */
	short unsigned int         type;                 /*   356     2 */
	short unsigned int         nr_ctx;               /*   358     2 */
	struct blk_mq_ctx * *      ctxs;                 /*   360     8 */
	spinlock_t                 dispatch_wait_lock;   /*   368     4 */

	/* XXX 4 bytes hole, try to pack */

	wait_queue_entry_t         dispatch_wait;        /*   376    40 */
	/* --- cacheline 6 boundary (384 bytes) was 32 bytes ago --- */
	atomic_t                   wait_index;           /*   416     4 */

	/* XXX 4 bytes hole, try to pack */

	struct blk_mq_tags *       tags;                 /*   424     8 */
	struct blk_mq_tags *       sched_tags;           /*   432     8 */
	long unsigned int          queued;               /*   440     8 */
	/* --- cacheline 7 boundary (448 bytes) --- */
	long unsigned int          run;                  /*   448     8 */
	long unsigned int          dispatched[7];        /*   456    56 */
	/* --- cacheline 8 boundary (512 bytes) --- */
	unsigned int               numa_node;            /*   512     4 */
	unsigned int               queue_num;            /*   516     4 */
	atomic_t                   nr_active;            /*   520     4 */

	/* XXX 4 bytes hole, try to pack */

	struct hlist_node          cpuhp_online;         /*   528    16 */
	struct hlist_node          cpuhp_dead;           /*   544    16 */
	struct kobject             kobj;                 /*   560    96 */
	/* --- cacheline 10 boundary (640 bytes) was 16 bytes ago --- */
	long unsigned int          poll_considered;      /*   656     8 */
	long unsigned int          poll_invoked;         /*   664     8 */
	long unsigned int          poll_success;         /*   672     8 */
	struct dentry *            debugfs_dir;          /*   680     8 */
	struct dentry *            sched_debugfs_dir;    /*   688     8 */
	struct list_head           hctx_list;            /*   696    16 */
	/* --- cacheline 11 boundary (704 bytes) was 8 bytes ago --- */
	long unsigned int          dtag_wait_time;       /*   712     8 */
	u64                        kabi_reserved1;       /*   720     8 */
	u64                        kabi_reserved2;       /*   728     8 */
	u64                        kabi_reserved3;       /*   736     8 */
	u64                        kabi_reserved4;       /*   744     8 */
	u64                        kabi_reserved5;       /*   752     8 */
	u64                        kabi_reserved6;       /*   760     8 */
	/* --- cacheline 12 boundary (768 bytes) --- */
	u64                        kabi_reserved7;       /*   768     8 */
	u64                        kabi_reserved8;       /*   776     8 */
	struct srcu_struct         srcu[];               /*   784     0 */

	/* size: 832, cachelines: 13, members: 46 */
	/* sum members: 772, holes: 3, sum holes: 12 */
	/* padding: 48 */
	/* paddings: 1, sum paddings: 32 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(64)));
struct blk_mq_alloc_data {
	struct request_queue *     q;                    /*     0     8 */
	blk_mq_req_flags_t         flags;                /*     8     4 */
	unsigned int               shallow_depth;        /*    12     4 */
	unsigned int               cmd_flags;            /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	struct blk_mq_ctx *        ctx;                  /*    24     8 */
	struct blk_mq_hw_ctx *     hctx;                 /*    32     8 */

	/* size: 40, cachelines: 1, members: 6 */
	/* sum members: 36, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct blk_mq_tags {
	unsigned int               nr_tags;              /*     0     4 */
	unsigned int               nr_reserved_tags;     /*     4     4 */
	atomic_t                   active_queues;        /*     8     4 */
	atomic_t                   pending_queues;       /*    12     4 */
	struct sbitmap_queue *     kabi_reserved_bitmap_tags; /*    16     8 */
	struct sbitmap_queue *     kabi_reserved_breserved_tags; /*    24     8 */
	union {
		struct sbitmap_queue bitmap_tags;        /*    32    72 */
		struct {
			struct sbitmap_queue __bitmap_tags; /*    32    72 */

			/* XXX last struct has 4 bytes of padding */
		} kabi_hidden_24;                        /*    32    72 */
		union {
		};                                       /*    32     0 */
	};                                               /*    32    72 */
	/* --- cacheline 1 boundary (64 bytes) was 40 bytes ago --- */
	union {
		struct sbitmap_queue breserved_tags;     /*   104    72 */
		struct {
			struct sbitmap_queue __breserved_tags; /*   104    72 */

			/* XXX last struct has 4 bytes of padding */
		} kabi_hidden_26;                        /*   104    72 */
		union {
		};                                       /*   104     0 */
	};                                               /*   104    72 */
	/* --- cacheline 2 boundary (128 bytes) was 48 bytes ago --- */
	struct request * *         rqs;                  /*   176     8 */
	struct request * *         static_rqs;           /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	struct list_head           page_list;            /*   192    16 */
	spinlock_t                 lock;                 /*   208     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        kabi_reserved1;       /*   216     8 */
	u64                        kabi_reserved2;       /*   224     8 */
	u64                        kabi_reserved3;       /*   232     8 */
	u64                        kabi_reserved4;       /*   240     8 */

	/* size: 248, cachelines: 4, members: 16 */
	/* sum members: 244, holes: 1, sum holes: 4 */
	/* last cacheline: 56 bytes */
};
struct blk_mq_ops {
	blk_status_t               (*queue_rq)(struct blk_mq_hw_ctx *, const struct blk_mq_queue_data  *); /*     0     8 */
	void                       (*commit_rqs)(struct blk_mq_hw_ctx *); /*     8     8 */
	bool                       (*get_budget)(struct request_queue *); /*    16     8 */
	void                       (*put_budget)(struct request_queue *); /*    24     8 */
	enum blk_eh_timer_return   (*timeout)(struct request *, bool); /*    32     8 */
	int                        (*poll)(struct blk_mq_hw_ctx *); /*    40     8 */
	void                       (*complete)(struct request *); /*    48     8 */
	int                        (*init_hctx)(struct blk_mq_hw_ctx *, void *, unsigned int); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	void                       (*exit_hctx)(struct blk_mq_hw_ctx *, unsigned int); /*    64     8 */
	int                        (*init_request)(struct blk_mq_tag_set *, struct request *, unsigned int, unsigned int); /*    72     8 */
	void                       (*exit_request)(struct blk_mq_tag_set *, struct request *, unsigned int); /*    80     8 */
	void                       (*initialize_rq_fn)(struct request *); /*    88     8 */
	void                       (*cleanup_rq)(struct request *); /*    96     8 */
	bool                       (*busy)(struct request_queue *); /*   104     8 */
	int                        (*map_queues)(struct blk_mq_tag_set *); /*   112     8 */
	void                       (*show_rq)(struct seq_file *, struct request *); /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	u64                        kabi_reserved1;       /*   128     8 */
	u64                        kabi_reserved2;       /*   136     8 */
	u64                        kabi_reserved3;       /*   144     8 */
	u64                        kabi_reserved4;       /*   152     8 */
	u64                        kabi_reserved5;       /*   160     8 */
	u64                        kabi_reserved6;       /*   168     8 */
	u64                        kabi_reserved7;       /*   176     8 */
	u64                        kabi_reserved8;       /*   184     8 */

	/* size: 192, cachelines: 3, members: 24 */
};
struct blk_stat_callback {
	struct list_head           list;                 /*     0    16 */
	struct timer_list          timer;                /*    16    72 */
	/* --- cacheline 1 boundary (64 bytes) was 24 bytes ago --- */
	struct blk_rq_stat *       cpu_stat;             /*    88     8 */
	int                        (*bucket_fn)(const struct request  *); /*    96     8 */
	unsigned int               buckets;              /*   104     4 */

	/* XXX 4 bytes hole, try to pack */

	struct blk_rq_stat *       stat;                 /*   112     8 */
	void                       (*timer_fn)(struct blk_stat_callback *); /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	void *                     data;                 /*   128     8 */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*   136    16 */

	/* size: 152, cachelines: 3, members: 9 */
	/* sum members: 148, holes: 1, sum holes: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct blk_trace {
	int                        trace_state;          /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct rchan *             rchan;                /*     8     8 */
	long unsigned int *        sequence;             /*    16     8 */
	unsigned char *            msg_data;             /*    24     8 */
	u16                        act_mask;             /*    32     2 */

	/* XXX 6 bytes hole, try to pack */

	u64                        start_lba;            /*    40     8 */
	u64                        end_lba;              /*    48     8 */
	u32                        pid;                  /*    56     4 */
	u32                        dev;                  /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct dentry *            dir;                  /*    64     8 */
	struct dentry *            dropped_file;         /*    72     8 */
	struct dentry *            msg_file;             /*    80     8 */
	struct list_head           running_list;         /*    88    16 */
	atomic_t                   dropped;              /*   104     4 */

	/* size: 112, cachelines: 2, members: 14 */
	/* sum members: 98, holes: 2, sum holes: 10 */
	/* padding: 4 */
	/* last cacheline: 48 bytes */
};
struct blk_flush_queue {
	unsigned int               flush_pending_idx:1;  /*     0: 0  4 */
	unsigned int               flush_running_idx:1;  /*     0: 1  4 */

	/* XXX 6 bits hole, try to pack */
	/* Bitfield combined with next fields */

	blk_status_t               rq_status;            /*     1     1 */

	/* XXX 6 bytes hole, try to pack */

	long unsigned int          flush_pending_since;  /*     8     8 */
	struct list_head           flush_queue[2];       /*    16    32 */
	struct list_head           flush_data_in_flight; /*    48    16 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct request *           flush_rq;             /*    64     8 */
	spinlock_t                 mq_flush_lock;        /*    72     4 */

	/* size: 80, cachelines: 2, members: 8 */
	/* sum members: 69, holes: 1, sum holes: 6 */
	/* sum bitfield members: 2 bits, bit holes: 1, sum bit holes: 6 bits */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct blk_mq_tag_set {
	struct blk_mq_queue_map    map[3];               /*     0    48 */
	unsigned int               nr_maps;              /*    48     4 */

	/* XXX 4 bytes hole, try to pack */

	const struct blk_mq_ops  * ops;                  /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	unsigned int               nr_hw_queues;         /*    64     4 */
	unsigned int               queue_depth;          /*    68     4 */
	unsigned int               reserved_tags;        /*    72     4 */
	unsigned int               cmd_size;             /*    76     4 */
	int                        numa_node;            /*    80     4 */
	unsigned int               timeout;              /*    84     4 */
	unsigned int               flags;                /*    88     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     driver_data;          /*    96     8 */
	atomic_t                   kabi_reserved_active_queues_shared_sbitmap; /*   104     4 */
	atomic_t                   kabi_reserved_pending_queues_shared_sbitmap; /*   108     4 */
	struct sbitmap_queue       kabi_reserved___bitmap_tags; /*   112    72 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 2 boundary (128 bytes) was 56 bytes ago --- */
	struct sbitmap_queue       kabi_reserved___breserved_tags; /*   184    72 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 4 boundary (256 bytes) --- */
	struct blk_mq_tags * *     tags;                 /*   256     8 */
	struct mutex               tag_list_lock;        /*   264    32 */
	struct list_head           tag_list;             /*   296    16 */
	union {
		struct blk_mq_tags * shared_sbitmap_tags; /*   312     8 */
		struct {
			u64        kabi_reserved1;       /*   312     8 */
		} kabi_hidden_283;                       /*   312     8 */
		union {
		};                                       /*   312     0 */
	};                                               /*   312     8 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	u64                        kabi_reserved2;       /*   320     8 */
	u64                        kabi_reserved3;       /*   328     8 */
	u64                        kabi_reserved4;       /*   336     8 */
	u64                        kabi_reserved5;       /*   344     8 */
	u64                        kabi_reserved6;       /*   352     8 */
	u64                        kabi_reserved7;       /*   360     8 */
	u64                        kabi_reserved8;       /*   368     8 */

	/* size: 376, cachelines: 6, members: 26 */
	/* sum members: 368, holes: 2, sum holes: 8 */
	/* paddings: 2, sum paddings: 8 */
	/* last cacheline: 56 bytes */
};
struct blk_io_trace {
	__u32                      magic;                /*     0     4 */
	__u32                      sequence;             /*     4     4 */
	__u64                      time;                 /*     8     8 */
	__u64                      sector;               /*    16     8 */
	__u32                      bytes;                /*    24     4 */
	__u32                      action;               /*    28     4 */
	__u32                      pid;                  /*    32     4 */
	__u32                      device;               /*    36     4 */
	__u32                      cpu;                  /*    40     4 */
	__u16                      error;                /*    44     2 */
	__u16                      pdu_len;              /*    46     2 */

	/* size: 48, cachelines: 1, members: 11 */
	/* last cacheline: 48 bytes */
};
struct blk_io_trace_remap {
	__be32                     device_from;          /*     0     4 */
	__be32                     device_to;            /*     4     4 */
	__be64                     sector_from;          /*     8     8 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct blk_user_trace_setup {
	char                       name[32];             /*     0    32 */
	__u16                      act_mask;             /*    32     2 */

	/* XXX 2 bytes hole, try to pack */

	__u32                      buf_size;             /*    36     4 */
	__u32                      buf_nr;               /*    40     4 */

	/* XXX 4 bytes hole, try to pack */

	__u64                      start_lba;            /*    48     8 */
	__u64                      end_lba;              /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	__u32                      pid;                  /*    64     4 */

	/* size: 72, cachelines: 2, members: 7 */
	/* sum members: 62, holes: 2, sum holes: 6 */
	/* padding: 4 */
	/* last cacheline: 8 bytes */
};
struct compat_blk_user_trace_setup {
	char                       name[32];             /*     0    32 */
	u16                        act_mask;             /*    32     2 */

	/* XXX 2 bytes hole, try to pack */

	u32                        buf_size;             /*    36     4 */
	u32                        buf_nr;               /*    40     4 */
	compat_u64                 start_lba __attribute__((__aligned__(4))); /*    44     8 */
	compat_u64                 end_lba __attribute__((__aligned__(4))); /*    52     8 */
	u32                        pid;                  /*    60     4 */

	/* size: 64, cachelines: 1, members: 7 */
	/* sum members: 62, holes: 1, sum holes: 2 */
	/* forced alignments: 2 */
} __attribute__((__aligned__(4)));
struct blk_mq_queue_map {
	unsigned int *             mq_map;               /*     0     8 */
	unsigned int               nr_queues;            /*     8     4 */
	unsigned int               queue_offset;         /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct blk_mq_queue_data {
	struct request *           rq;                   /*     0     8 */
	bool                       last;                 /*     8     1 */

	/* XXX 7 bytes hole, try to pack */

	u64                        kabi_reserved1;       /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 17, holes: 1, sum holes: 7 */
	/* last cacheline: 24 bytes */
};
struct blk_mq_ctxs {
	struct kobject             kobj;                 /*     0    96 */
	/* --- cacheline 1 boundary (64 bytes) was 32 bytes ago --- */
	struct blk_mq_ctx *        queue_ctx;            /*    96     8 */

	/* size: 104, cachelines: 2, members: 2 */
	/* last cacheline: 40 bytes */
};
struct ftrace_event_field {
	struct list_head           link;                 /*     0    16 */
	const char  *              name;                 /*    16     8 */
	const char  *              type;                 /*    24     8 */
	int                        filter_type;          /*    32     4 */
	int                        offset;               /*    36     4 */
	int                        size;                 /*    40     4 */
	int                        is_signed;            /*    44     4 */

	/* size: 48, cachelines: 1, members: 7 */
	/* last cacheline: 48 bytes */
};
struct event_probe_data {
	struct trace_event_file *  file;                 /*     0     8 */
	long unsigned int          count;                /*     8     8 */
	int                        ref;                  /*    16     4 */
	bool                       enable;               /*    20     1 */

	/* size: 24, cachelines: 1, members: 4 */
	/* padding: 3 */
	/* last cacheline: 24 bytes */
};
struct syscall_trace_enter {
	struct trace_entry         ent;                  /*     0     8 */
	int                        nr;                   /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          args[];               /*    16     0 */

	/* size: 16, cachelines: 1, members: 3 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct syscall_trace_exit {
	struct trace_entry         ent;                  /*     0     8 */
	int                        nr;                   /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	long int                   ret;                  /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct prog_entry {
	int                        target;               /*     0     4 */
	int                        when_to_branch;       /*     4     4 */
	struct filter_pred *       pred;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct filter_pred {
	filter_pred_fn_t           fn;                   /*     0     8 */
	u64                        val;                  /*     8     8 */
	struct regex               regex;                /*    16   272 */
	/* --- cacheline 4 boundary (256 bytes) was 32 bytes ago --- */
	short unsigned int *       ops;                  /*   288     8 */
	struct ftrace_event_field * field;               /*   296     8 */
	int                        offset;               /*   304     4 */
	int                        not;                  /*   308     4 */
	int                        op;                   /*   312     4 */

	/* size: 320, cachelines: 5, members: 8 */
	/* padding: 4 */
};
struct regex {
	char                       pattern[256];         /*     0   256 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	int                        len;                  /*   256     4 */
	int                        field_len;            /*   260     4 */
	regex_match_func           match;                /*   264     8 */

	/* size: 272, cachelines: 5, members: 4 */
	/* last cacheline: 16 bytes */
};
struct filter_parse_error {
	int                        lasterr;              /*     0     4 */
	int                        lasterr_pos;          /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct ustring_buffer {
	char                       buffer[1024];         /*     0  1024 */

	/* size: 1024, cachelines: 16, members: 1 */
};
struct filter_list {
	struct list_head           list;                 /*     0    16 */
	struct event_filter *      filter;               /*    16     8 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct function_filter_data {
	struct ftrace_ops *        ops;                  /*     0     8 */
	int                        first_filter;         /*     8     4 */
	int                        first_notrace;        /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct event_trigger_data {
	long unsigned int          count;                /*     0     8 */
	int                        ref;                  /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct event_trigger_ops * ops;                  /*    16     8 */
	struct event_command *     cmd_ops;              /*    24     8 */
	struct event_filter *      filter;               /*    32     8 */
	char *                     filter_str;           /*    40     8 */
	void *                     private_data;         /*    48     8 */
	bool                       paused;               /*    56     1 */
	bool                       paused_tmp;           /*    57     1 */

	/* XXX 6 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	struct list_head           list;                 /*    64    16 */
	char *                     name;                 /*    80     8 */
	struct list_head           named_list;           /*    88    16 */
	struct event_trigger_data * named_data;          /*   104     8 */

	/* size: 112, cachelines: 2, members: 13 */
	/* sum members: 102, holes: 2, sum holes: 10 */
	/* last cacheline: 48 bytes */
};
struct event_trigger_ops {
	void                       (*func)(struct event_trigger_data *, void *, struct ring_buffer_event *); /*     0     8 */
	int                        (*init)(struct event_trigger_ops *, struct event_trigger_data *); /*     8     8 */
	void                       (*free)(struct event_trigger_ops *, struct event_trigger_data *); /*    16     8 */
	int                        (*print)(struct seq_file *, struct event_trigger_ops *, struct event_trigger_data *); /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct event_command {
	struct list_head           list;                 /*     0    16 */
	char *                     name;                 /*    16     8 */
	enum event_trigger_type    trigger_type;         /*    24     4 */
	int                        flags;                /*    28     4 */
	int                        (*func)(struct event_command *, struct trace_event_file *, char *, char *, char *); /*    32     8 */
	int                        (*reg)(char *, struct event_trigger_ops *, struct event_trigger_data *, struct trace_event_file *); /*    40     8 */
	void                       (*unreg)(char *, struct event_trigger_ops *, struct event_trigger_data *, struct trace_event_file *); /*    48     8 */
	void                       (*unreg_all)(struct trace_event_file *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*set_filter)(char *, struct event_trigger_data *, struct trace_event_file *); /*    64     8 */
	struct event_trigger_ops * (*get_trigger_ops)(char *, char *); /*    72     8 */

	/* size: 80, cachelines: 2, members: 10 */
	/* last cacheline: 16 bytes */
};
struct enable_trigger_data {
	struct trace_event_file *  file;                 /*     0     8 */
	bool                       enable;               /*     8     1 */
	bool                       hist;                 /*     9     1 */

	/* size: 16, cachelines: 1, members: 3 */
	/* padding: 6 */
	/* last cacheline: 16 bytes */
};
struct dynevent_cmd {
	struct seq_buf             seq;                  /*     0    32 */
	const char  *              event_name;           /*    32     8 */
	unsigned int               n_fields;             /*    40     4 */
	enum dynevent_type         type;                 /*    44     4 */
	dynevent_create_fn_t       run_command;          /*    48     8 */
	void *                     private_data;         /*    56     8 */

	/* size: 64, cachelines: 1, members: 6 */
};
struct synth_field_desc {
	const char  *              type;                 /*     0     8 */
	const char  *              name;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct synth_event_trace_state {
	struct trace_event_buffer  fbuffer;              /*     0    48 */
	struct synth_trace_event * entry;                /*    48     8 */
	struct trace_buffer *      buffer;               /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct synth_event *       event;                /*    64     8 */
	unsigned int               cur_field;            /*    72     4 */
	unsigned int               n_u64;                /*    76     4 */
	bool                       disabled;             /*    80     1 */
	bool                       add_next;             /*    81     1 */
	bool                       add_name;             /*    82     1 */

	/* size: 88, cachelines: 2, members: 9 */
	/* padding: 5 */
	/* last cacheline: 24 bytes */
};
struct synth_trace_event {
	struct trace_entry         ent;                  /*     0     8 */
	u64                        fields[];             /*     8     0 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct synth_event {
	struct dyn_event           devent;               /*     0    24 */
	int                        ref;                  /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	char *                     name;                 /*    32     8 */
	struct synth_field * *     fields;               /*    40     8 */
	unsigned int               n_fields;             /*    48     4 */

	/* XXX 4 bytes hole, try to pack */

	struct synth_field * *     dynamic_fields;       /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	unsigned int               n_dynamic_fields;     /*    64     4 */
	unsigned int               n_u64;                /*    68     4 */
	struct trace_event_class   class;                /*    72    72 */
	/* --- cacheline 2 boundary (128 bytes) was 16 bytes ago --- */
	struct trace_event_call    call;                 /*   144   144 */
	/* --- cacheline 4 boundary (256 bytes) was 32 bytes ago --- */
	struct tracepoint *        tp;                   /*   288     8 */
	struct module *            mod;                  /*   296     8 */

	/* size: 304, cachelines: 5, members: 12 */
	/* sum members: 296, holes: 2, sum holes: 8 */
	/* last cacheline: 48 bytes */
};
struct dyn_event_operations {
	struct list_head           list;                 /*     0    16 */
	int                        (*create)(int, const char  * *); /*    16     8 */
	int                        (*show)(struct seq_file *, struct dyn_event *); /*    24     8 */
	bool                       (*is_busy)(struct dyn_event *); /*    32     8 */
	int                        (*free)(struct dyn_event *); /*    40     8 */
	bool                       (*match)(const char  *, const char  *, int, const char  * *, struct dyn_event *); /*    48     8 */

	/* size: 56, cachelines: 1, members: 6 */
	/* last cacheline: 56 bytes */
};
struct dyn_event {
	struct list_head           list;                 /*     0    16 */
	struct dyn_event_operations * ops;               /*    16     8 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct dynevent_arg {
	const char  *              str;                  /*     0     8 */
	char                       separator;            /*     8     1 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 7 */
	/* last cacheline: 16 bytes */
};
struct dynevent_arg_pair {
	const char  *              lhs;                  /*     0     8 */
	const char  *              rhs;                  /*     8     8 */
	char                       operator;             /*    16     1 */
	char                       separator;            /*    17     1 */

	/* size: 24, cachelines: 1, members: 4 */
	/* padding: 6 */
	/* last cacheline: 24 bytes */
};
struct synth_field {
	char *                     type;                 /*     0     8 */
	char *                     name;                 /*     8     8 */
	size_t                     size;                 /*    16     8 */
	unsigned int               offset;               /*    24     4 */
	unsigned int               field_pos;            /*    28     4 */
	bool                       is_signed;            /*    32     1 */
	bool                       is_string;            /*    33     1 */
	bool                       is_dynamic;           /*    34     1 */

	/* size: 40, cachelines: 1, members: 8 */
	/* padding: 5 */
	/* last cacheline: 40 bytes */
};
struct hist_field {
	struct ftrace_event_field * field;               /*     0     8 */
	long unsigned int          flags;                /*     8     8 */
	hist_field_fn_t            fn;                   /*    16     8 */
	unsigned int               ref;                  /*    24     4 */
	unsigned int               size;                 /*    28     4 */
	unsigned int               offset;               /*    32     4 */
	unsigned int               is_signed;            /*    36     4 */
	const char  *              type;                 /*    40     8 */
	struct hist_field *        operands[2];          /*    48    16 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct hist_trigger_data * hist_data;            /*    64     8 */
	struct hist_var            var;                  /*    72    24 */

	/* XXX last struct has 4 bytes of padding */

	enum field_op_id           operator;             /*    96     4 */

	/* XXX 4 bytes hole, try to pack */

	char *                     system;               /*   104     8 */
	char *                     event_name;           /*   112     8 */
	char *                     name;                 /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	unsigned int               var_ref_idx;          /*   128     4 */
	bool                       read_once;            /*   132     1 */

	/* XXX 3 bytes hole, try to pack */

	unsigned int               var_str_idx;          /*   136     4 */

	/* size: 144, cachelines: 3, members: 18 */
	/* sum members: 133, holes: 2, sum holes: 7 */
	/* padding: 4 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 16 bytes */
};
struct hist_var {
	char *                     name;                 /*     0     8 */
	struct hist_trigger_data * hist_data;            /*     8     8 */
	unsigned int               idx;                  /*    16     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct hist_trigger_data {
	struct hist_field *        fields[22];           /*     0   176 */
	/* --- cacheline 2 boundary (128 bytes) was 48 bytes ago --- */
	unsigned int               n_vals;               /*   176     4 */
	unsigned int               n_keys;               /*   180     4 */
	unsigned int               n_fields;             /*   184     4 */
	unsigned int               n_vars;               /*   188     4 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	unsigned int               n_var_str;            /*   192     4 */
	unsigned int               key_size;             /*   196     4 */
	struct tracing_map_sort_key sort_keys[2];        /*   200    16 */
	unsigned int               n_sort_keys;          /*   216     4 */

	/* XXX 4 bytes hole, try to pack */

	struct trace_event_file *  event_file;           /*   224     8 */
	struct hist_trigger_attrs * attrs;               /*   232     8 */
	struct tracing_map *       map;                  /*   240     8 */
	bool                       enable_timestamps;    /*   248     1 */
	bool                       remove;               /*   249     1 */

	/* XXX 6 bytes hole, try to pack */

	/* --- cacheline 4 boundary (256 bytes) --- */
	struct hist_field *        var_refs[16];         /*   256   128 */
	/* --- cacheline 6 boundary (384 bytes) --- */
	unsigned int               n_var_refs;           /*   384     4 */

	/* XXX 4 bytes hole, try to pack */

	struct action_data *       actions[8];           /*   392    64 */
	/* --- cacheline 7 boundary (448 bytes) was 8 bytes ago --- */
	unsigned int               n_actions;            /*   456     4 */

	/* XXX 4 bytes hole, try to pack */

	struct field_var *         field_vars[32];       /*   464   256 */
	/* --- cacheline 11 boundary (704 bytes) was 16 bytes ago --- */
	unsigned int               n_field_vars;         /*   720     4 */
	unsigned int               n_field_var_str;      /*   724     4 */
	struct field_var_hist *    field_var_hists[32];  /*   728   256 */
	/* --- cacheline 15 boundary (960 bytes) was 24 bytes ago --- */
	unsigned int               n_field_var_hists;    /*   984     4 */

	/* XXX 4 bytes hole, try to pack */

	struct field_var *         save_vars[32];        /*   992   256 */
	/* --- cacheline 19 boundary (1216 bytes) was 32 bytes ago --- */
	unsigned int               n_save_vars;          /*  1248     4 */
	unsigned int               n_save_var_str;       /*  1252     4 */

	/* size: 1256, cachelines: 20, members: 26 */
	/* sum members: 1234, holes: 5, sum holes: 22 */
	/* last cacheline: 40 bytes */
};
struct var_defs {
	unsigned int               n_vars;               /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	char *                     name[16];             /*     8   128 */
	/* --- cacheline 2 boundary (128 bytes) was 8 bytes ago --- */
	char *                     expr[16];             /*   136   128 */

	/* size: 264, cachelines: 5, members: 3 */
	/* sum members: 260, holes: 1, sum holes: 4 */
	/* last cacheline: 8 bytes */
};
struct hist_trigger_attrs {
	char *                     keys_str;             /*     0     8 */
	char *                     vals_str;             /*     8     8 */
	char *                     sort_key_str;         /*    16     8 */
	char *                     name;                 /*    24     8 */
	char *                     clock;                /*    32     8 */
	bool                       pause;                /*    40     1 */
	bool                       cont;                 /*    41     1 */
	bool                       clear;                /*    42     1 */
	bool                       ts_in_usecs;          /*    43     1 */
	unsigned int               map_bits;             /*    44     4 */
	char *                     assignment_str[16];   /*    48   128 */
	/* --- cacheline 2 boundary (128 bytes) was 48 bytes ago --- */
	unsigned int               n_assignments;        /*   176     4 */

	/* XXX 4 bytes hole, try to pack */

	char *                     action_str[8];        /*   184    64 */
	/* --- cacheline 3 boundary (192 bytes) was 56 bytes ago --- */
	unsigned int               n_actions;            /*   248     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 4 boundary (256 bytes) --- */
	struct var_defs            var_defs;             /*   256   264 */

	/* size: 520, cachelines: 9, members: 15 */
	/* sum members: 512, holes: 2, sum holes: 8 */
	/* last cacheline: 8 bytes */
};
struct field_var {
	struct hist_field *        var;                  /*     0     8 */
	struct hist_field *        val;                  /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct field_var_hist {
	struct hist_trigger_data * hist_data;            /*     0     8 */
	char *                     cmd;                  /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct action_data {
	enum handler_id            handler;              /*     0     4 */
	enum action_id             action;               /*     4     4 */
	char *                     action_name;          /*     8     8 */
	action_fn_t                fn;                   /*    16     8 */
	unsigned int               n_params;             /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	char *                     params[32];           /*    32   256 */
	/* --- cacheline 4 boundary (256 bytes) was 32 bytes ago --- */
	unsigned int               var_ref_idx[16];      /*   288    64 */
	/* --- cacheline 5 boundary (320 bytes) was 32 bytes ago --- */
	struct synth_event *       synth_event;          /*   352     8 */
	bool                       use_trace_keyword;    /*   360     1 */

	/* XXX 7 bytes hole, try to pack */

	char *                     synth_event_name;     /*   368     8 */
	union {
		struct {
			char *     event;                /*   376     8 */
			/* --- cacheline 6 boundary (384 bytes) --- */
			char *     event_system;         /*   384     8 */
		} match_data;                            /*   376    16 */
		struct {
			char *     var_str;              /*   376     8 */
			/* --- cacheline 6 boundary (384 bytes) --- */
			struct hist_field * var_ref;     /*   384     8 */
			struct hist_field * track_var;   /*   392     8 */
			check_track_val_fn_t check_val;  /*   400     8 */
			action_fn_t save_data;           /*   408     8 */
		} track_data;                            /*   376    40 */
	};                                               /*   376    40 */

	/* size: 416, cachelines: 7, members: 11 */
	/* sum members: 405, holes: 2, sum holes: 11 */
	/* last cacheline: 32 bytes */
};
struct track_data {
	u64                        track_val;            /*     0     8 */
	bool                       updated;              /*     8     1 */

	/* XXX 3 bytes hole, try to pack */

	unsigned int               key_len;              /*    12     4 */
	void *                     key;                  /*    16     8 */
	struct tracing_map_elt     elt;                  /*    24    48 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	struct action_data *       action_data;          /*    72     8 */
	struct hist_trigger_data * hist_data;            /*    80     8 */

	/* size: 88, cachelines: 2, members: 7 */
	/* sum members: 85, holes: 1, sum holes: 3 */
	/* last cacheline: 24 bytes */
};
struct hist_elt_data {
	char *                     comm;                 /*     0     8 */
	u64 *                      var_ref_vals;         /*     8     8 */
	char *                     field_var_str[32];    /*    16   256 */

	/* size: 272, cachelines: 5, members: 3 */
	/* last cacheline: 16 bytes */
};
struct snapshot_context {
	struct tracing_map_elt *   elt;                  /*     0     8 */
	void *                     key;                  /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct hist_var_data {
	struct list_head           list;                 /*     0    16 */
	struct hist_trigger_data * hist_data;            /*    16     8 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct bpf_perf_event_value {
	__u64                      counter;              /*     0     8 */
	__u64                      enabled;              /*     8     8 */
	__u64                      running;              /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct bpf_raw_tracepoint_args {
	__u64                      args[0];              /*     0     0 */

	/* size: 0, cachelines: 0, members: 1 */
};
struct btf_ptr {
	void *                     ptr;                  /*     0     8 */
	__u32                      type_id;              /*     8     4 */
	__u32                      flags;                /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct btf_type {
	__u32                      name_off;             /*     0     4 */
	__u32                      info;                 /*     4     4 */
	union {
		__u32              size;                 /*     8     4 */
		__u32              type;                 /*     8     4 */
	};                                               /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct bpf_event_entry {
	struct perf_event *        event;                /*     0     8 */
	struct file *              perf_file;            /*     8     8 */
	struct file *              map_file;             /*    16     8 */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*    24    16 */

	/* size: 40, cachelines: 1, members: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 40 bytes */
} __attribute__((__aligned__(8)));
struct bpf_cg_run_ctx {
	struct bpf_run_ctx         run_ctx;              /*     0     0 */
	struct bpf_prog_array_item * prog_item;          /*     0     8 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct bpf_perf_event_data {
	bpf_user_pt_regs_t         regs;                 /*     0   168 */
	/* --- cacheline 2 boundary (128 bytes) was 40 bytes ago --- */
	__u64                      sample_period;        /*   168     8 */
	__u64                      addr;                 /*   176     8 */

	/* size: 184, cachelines: 3, members: 3 */
	/* last cacheline: 56 bytes */
};
struct perf_event_query_bpf {
	__u32                      ids_len;              /*     0     4 */
	__u32                      prog_cnt;             /*     4     4 */
	__u32                      ids[];                /*     8     0 */

	/* size: 8, cachelines: 1, members: 3 */
	/* last cacheline: 8 bytes */
};
struct bpf_perf_event_data_kern {
	bpf_user_pt_regs_t *       regs;                 /*     0     8 */
	struct perf_sample_data *  data;                 /*     8     8 */
	struct perf_event *        event;                /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_bpf_trace_printk {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        __data_loc_bpf_string; /*     8     4 */
	char                       __data[];             /*    12     0 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct trace_event_data_offsets_bpf_trace_printk {
	u32                        bpf_string;           /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct bpf_trace_module {
	struct module *            module;               /*     0     8 */
	struct list_head           list;                 /*     8    16 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct bpf_seq_printf_buf {
	char                       buf[6][128];          /*     0   768 */

	/* size: 768, cachelines: 12, members: 1 */
};
struct bpf_trace_sample_data {
	struct perf_sample_data    sds[3] __attribute__((__aligned__(64))); /*     0   576 */

	/* size: 576, cachelines: 9, members: 1 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(64)));
struct bpf_nested_pt_regs {
	struct pt_regs             regs[3];              /*     0   504 */

	/* size: 504, cachelines: 8, members: 1 */
	/* last cacheline: 56 bytes */
};
struct send_signal_irq_work {
	struct irq_work            irq_work;             /*     0    24 */
	struct task_struct *       task;                 /*    24     8 */
	u32                        sig;                  /*    32     4 */
	enum pid_type              type;                 /*    36     4 */

	/* size: 40, cachelines: 1, members: 4 */
	/* last cacheline: 40 bytes */
};
struct bpf_raw_tp_regs {
	struct pt_regs             regs[3];              /*     0   504 */

	/* size: 504, cachelines: 8, members: 1 */
	/* last cacheline: 56 bytes */
};
struct kprobe_trace_entry_head {
	struct trace_entry         ent;                  /*     0     8 */
	long unsigned int          ip;                   /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct kretprobe_trace_entry_head {
	struct trace_entry         ent;                  /*     0     8 */
	long unsigned int          func;                 /*     8     8 */
	long unsigned int          ret_ip;               /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct fetch_insn {
	enum fetch_op              op;                   /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	union {
		unsigned int       param;                /*     8     4 */
		struct {
			unsigned int size;               /*     8     4 */
			int        offset;               /*    12     4 */
		};                                       /*     8     8 */
		struct {
			unsigned char basesize;          /*     8     1 */
			unsigned char lshift;            /*     9     1 */
			unsigned char rshift;            /*    10     1 */
		};                                       /*     8     3 */
		long unsigned int  immediate;            /*     8     8 */
		void *             data;                 /*     8     8 */
	};                                               /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct fetch_type {
	const char  *              name;                 /*     0     8 */
	size_t                     size;                 /*     8     8 */
	int                        is_signed;            /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	print_type_func_t          print;                /*    24     8 */
	const char  *              fmt;                  /*    32     8 */
	const char  *              fmttype;              /*    40     8 */

	/* size: 48, cachelines: 1, members: 6 */
	/* sum members: 44, holes: 1, sum holes: 4 */
	/* last cacheline: 48 bytes */
};
struct probe_arg {
	struct fetch_insn *        code;                 /*     0     8 */
	bool                       dynamic;              /*     8     1 */

	/* XXX 3 bytes hole, try to pack */

	unsigned int               offset;               /*    12     4 */
	unsigned int               count;                /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	const char  *              name;                 /*    24     8 */
	const char  *              comm;                 /*    32     8 */
	char *                     fmt;                  /*    40     8 */
	const struct fetch_type  * type;                 /*    48     8 */

	/* size: 56, cachelines: 1, members: 8 */
	/* sum members: 49, holes: 2, sum holes: 7 */
	/* last cacheline: 56 bytes */
};
struct trace_uprobe_filter {
	rwlock_t                   rwlock;               /*     0     8 */
	int                        nr_systemwide;        /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           perf_events;          /*    16    16 */

	/* size: 32, cachelines: 1, members: 3 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct trace_probe_event {
	unsigned int               flags;                /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct trace_event_class   class;                /*     8    72 */
	/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
	struct trace_event_call    call;                 /*    80   144 */
	/* --- cacheline 3 boundary (192 bytes) was 32 bytes ago --- */
	struct list_head           files;                /*   224    16 */
	struct list_head           probes;               /*   240    16 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	struct trace_uprobe_filter filter[];             /*   256     0 */

	/* size: 256, cachelines: 4, members: 6 */
	/* sum members: 252, holes: 1, sum holes: 4 */
};
struct trace_probe {
	struct list_head           list;                 /*     0    16 */
	struct trace_probe_event * event;                /*    16     8 */
	ssize_t                    size;                 /*    24     8 */
	unsigned int               nr_args;              /*    32     4 */

	/* XXX 4 bytes hole, try to pack */

	struct probe_arg           args[];               /*    40     0 */

	/* size: 40, cachelines: 1, members: 5 */
	/* sum members: 36, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct event_file_link {
	struct trace_event_file *  file;                 /*     0     8 */
	struct list_head           list;                 /*     8    16 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct trace_kprobe {
	struct dyn_event           devent;               /*     0    24 */
	struct kretprobe           rp;                   /*    24   168 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 3 boundary (192 bytes) --- */
	long unsigned int *        nhit;                 /*   192     8 */
	const char  *              symbol;               /*   200     8 */
	struct trace_probe         tp;                   /*   208    40 */

	/* size: 248, cachelines: 4, members: 5 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 56 bytes */
};
struct trace_event_raw_error_report_template {
	struct trace_entry         ent;                  /*     0     8 */
	enum error_detector        error_detector;       /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          id;                   /*    16     8 */
	char                       __data[];             /*    24     0 */

	/* size: 24, cachelines: 1, members: 4 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct trace_event_data_offsets_error_report_template {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_raw_cpu {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        state;                /*     8     4 */
	u32                        cpu_id;               /*    12     4 */
	char                       __data[];             /*    16     0 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct trace_event_raw_powernv_throttle {
	struct trace_entry         ent;                  /*     0     8 */
	int                        chip_id;              /*     8     4 */
	u32                        __data_loc_reason;    /*    12     4 */
	int                        pmax;                 /*    16     4 */
	char                       __data[];             /*    20     0 */

	/* size: 20, cachelines: 1, members: 5 */
	/* last cacheline: 20 bytes */
};
struct trace_event_raw_pstate_sample {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        core_busy;            /*     8     4 */
	u32                        scaled_busy;          /*    12     4 */
	u32                        from;                 /*    16     4 */
	u32                        to;                   /*    20     4 */
	u64                        mperf;                /*    24     8 */
	u64                        aperf;                /*    32     8 */
	u64                        tsc;                  /*    40     8 */
	u32                        freq;                 /*    48     4 */
	u32                        io_boost;             /*    52     4 */
	char                       __data[];             /*    56     0 */

	/* size: 56, cachelines: 1, members: 11 */
	/* last cacheline: 56 bytes */
};
struct trace_event_raw_cpu_frequency_limits {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        min_freq;             /*     8     4 */
	u32                        max_freq;             /*    12     4 */
	u32                        cpu_id;               /*    16     4 */
	char                       __data[];             /*    20     0 */

	/* size: 20, cachelines: 1, members: 5 */
	/* last cacheline: 20 bytes */
};
struct trace_event_raw_device_pm_callback_start {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        __data_loc_device;    /*     8     4 */
	u32                        __data_loc_driver;    /*    12     4 */
	u32                        __data_loc_parent;    /*    16     4 */
	u32                        __data_loc_pm_ops;    /*    20     4 */
	int                        event;                /*    24     4 */
	char                       __data[];             /*    28     0 */

	/* size: 28, cachelines: 1, members: 7 */
	/* last cacheline: 28 bytes */
};
struct trace_event_raw_device_pm_callback_end {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        __data_loc_device;    /*     8     4 */
	u32                        __data_loc_driver;    /*    12     4 */
	int                        error;                /*    16     4 */
	char                       __data[];             /*    20     0 */

	/* size: 20, cachelines: 1, members: 5 */
	/* last cacheline: 20 bytes */
};
struct trace_event_raw_suspend_resume {
	struct trace_entry         ent;                  /*     0     8 */
	const char  *              action;               /*     8     8 */
	int                        val;                  /*    16     4 */
	bool                       start;                /*    20     1 */
	char                       __data[];             /*    21     0 */

	/* size: 24, cachelines: 1, members: 5 */
	/* padding: 3 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_wakeup_source {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        __data_loc_name;      /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        state;                /*    16     8 */
	char                       __data[];             /*    24     0 */

	/* size: 24, cachelines: 1, members: 4 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_clock {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        __data_loc_name;      /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        state;                /*    16     8 */
	u64                        cpu_id;               /*    24     8 */
	char                       __data[];             /*    32     0 */

	/* size: 32, cachelines: 1, members: 5 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct trace_event_raw_power_domain {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        __data_loc_name;      /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        state;                /*    16     8 */
	u64                        cpu_id;               /*    24     8 */
	char                       __data[];             /*    32     0 */

	/* size: 32, cachelines: 1, members: 5 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct trace_event_raw_cpu_latency_qos_request {
	struct trace_entry         ent;                  /*     0     8 */
	s32                        value;                /*     8     4 */
	char                       __data[];             /*    12     0 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct trace_event_raw_pm_qos_update {
	struct trace_entry         ent;                  /*     0     8 */
	enum pm_qos_req_action     action;               /*     8     4 */
	int                        prev_value;           /*    12     4 */
	int                        curr_value;           /*    16     4 */
	char                       __data[];             /*    20     0 */

	/* size: 20, cachelines: 1, members: 5 */
	/* last cacheline: 20 bytes */
};
struct trace_event_raw_dev_pm_qos_request {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        __data_loc_name;      /*     8     4 */
	enum dev_pm_qos_req_type   type;                 /*    12     4 */
	s32                        new_value;            /*    16     4 */
	char                       __data[];             /*    20     0 */

	/* size: 20, cachelines: 1, members: 5 */
	/* last cacheline: 20 bytes */
};
struct trace_event_data_offsets_cpu {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_powernv_throttle {
	u32                        reason;               /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct trace_event_data_offsets_pstate_sample {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_cpu_frequency_limits {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_device_pm_callback_start {
	u32                        device;               /*     0     4 */
	u32                        driver;               /*     4     4 */
	u32                        parent;               /*     8     4 */
	u32                        pm_ops;               /*    12     4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct trace_event_data_offsets_device_pm_callback_end {
	u32                        device;               /*     0     4 */
	u32                        driver;               /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct trace_event_data_offsets_suspend_resume {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_wakeup_source {
	u32                        name;                 /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct trace_event_data_offsets_clock {
	u32                        name;                 /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct trace_event_data_offsets_power_domain {
	u32                        name;                 /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct trace_event_data_offsets_cpu_latency_qos_request {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_pm_qos_update {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_dev_pm_qos_request {
	u32                        name;                 /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct trace_event_raw_rpm_internal {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        __data_loc_name;      /*     8     4 */
	int                        flags;                /*    12     4 */
	int                        usage_count;          /*    16     4 */
	int                        disable_depth;        /*    20     4 */
	int                        runtime_auto;         /*    24     4 */
	int                        request_pending;      /*    28     4 */
	int                        irq_safe;             /*    32     4 */
	int                        child_count;          /*    36     4 */
	char                       __data[];             /*    40     0 */

	/* size: 40, cachelines: 1, members: 10 */
	/* last cacheline: 40 bytes */
};
struct trace_event_raw_rpm_return_int {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        __data_loc_name;      /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          ip;                   /*    16     8 */
	int                        ret;                  /*    24     4 */
	char                       __data[];             /*    28     0 */

	/* size: 32, cachelines: 1, members: 5 */
	/* sum members: 24, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct trace_event_data_offsets_rpm_internal {
	u32                        name;                 /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct trace_event_data_offsets_rpm_return_int {
	u32                        name;                 /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct trace_probe_log {
	const char  *              subsystem;            /*     0     8 */
	const char  * *            argv;                 /*     8     8 */
	int                        argc;                 /*    16     4 */
	int                        index;                /*    20     4 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct uprobe_consumer {
	int                        (*handler)(struct uprobe_consumer *, struct pt_regs *); /*     0     8 */
	int                        (*ret_handler)(struct uprobe_consumer *, long unsigned int, struct pt_regs *); /*     8     8 */
	bool                       (*filter)(struct uprobe_consumer *, enum uprobe_filter_ctx, struct mm_struct *); /*    16     8 */
	struct uprobe_consumer *   next;                 /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct uprobe_trace_entry_head {
	struct trace_entry         ent;                  /*     0     8 */
	long unsigned int          vaddr[];              /*     8     0 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct trace_uprobe {
	struct dyn_event           devent;               /*     0    24 */
	struct uprobe_consumer     consumer;             /*    24    32 */
	struct path                path;                 /*    56    16 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	struct inode *             inode;                /*    72     8 */
	char *                     filename;             /*    80     8 */
	long unsigned int          offset;               /*    88     8 */
	long unsigned int          ref_ctr_offset;       /*    96     8 */
	long unsigned int          nhit;                 /*   104     8 */
	struct trace_probe         tp;                   /*   112    40 */

	/* size: 152, cachelines: 3, members: 9 */
	/* last cacheline: 24 bytes */
};
struct uprobe_dispatch_data {
	struct trace_uprobe *      tu;                   /*     0     8 */
	long unsigned int          bp_addr;              /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct uprobe_cpu_buffer {
	struct mutex               mutex;                /*     0    32 */
	void *                     buf;                  /*    32     8 */

	/* size: 40, cachelines: 1, members: 2 */
	/* last cacheline: 40 bytes */
};
struct bucket_table {
	unsigned int               size;                 /*     0     4 */
	unsigned int               nest;                 /*     4     4 */
	u32                        hash_rnd;             /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           walkers;              /*    16    16 */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*    32    16 */
	struct bucket_table *      future_tbl;           /*    48     8 */
	struct lockdep_map         dep_map;              /*    56     0 */

	/* XXX 8 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	struct rhash_lock_head *   buckets[] __attribute__((__aligned__(64))); /*    64     0 */

	/* size: 64, cachelines: 1, members: 8 */
	/* sum members: 52, holes: 2, sum holes: 12 */
	/* forced alignments: 2, forced holes: 1, sum forced holes: 8 */
} __attribute__((__aligned__(64)));
struct xdp_cpumap_stats {
	unsigned int               redirect;             /*     0     4 */
	unsigned int               pass;                 /*     4     4 */
	unsigned int               drop;                 /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct bpf_prog_dummy {
	struct bpf_prog            prog;                 /*     0    56 */

	/* size: 56, cachelines: 1, members: 1 */
	/* last cacheline: 56 bytes */
};
struct _bpf_dtab_netdev {
	struct net_device *        dev;                  /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct rhash_lock_head {

	/* size: 0, cachelines: 0, members: 0 */
};
struct xdp_mem_allocator {
	struct xdp_mem_info        mem;                  /*     0     8 */
	union {
		void *             allocator;            /*     8     8 */
		struct page_pool * page_pool;            /*     8     8 */
		struct zero_copy_allocator * zc_alloc;   /*     8     8 */
	};                                               /*     8     8 */
	struct rhash_head          node;                 /*    16     8 */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*    24    16 */

	/* size: 40, cachelines: 1, members: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 40 bytes */
} __attribute__((__aligned__(8)));
struct trace_event_raw_xdp_exception {
	struct trace_entry         ent;                  /*     0     8 */
	int                        prog_id;              /*     8     4 */
	u32                        act;                  /*    12     4 */
	int                        ifindex;              /*    16     4 */
	char                       __data[];             /*    20     0 */

	/* size: 20, cachelines: 1, members: 5 */
	/* last cacheline: 20 bytes */
};
struct trace_event_raw_xdp_bulk_tx {
	struct trace_entry         ent;                  /*     0     8 */
	int                        ifindex;              /*     8     4 */
	u32                        act;                  /*    12     4 */
	int                        drops;                /*    16     4 */
	int                        sent;                 /*    20     4 */
	int                        err;                  /*    24     4 */
	char                       __data[];             /*    28     0 */

	/* size: 28, cachelines: 1, members: 7 */
	/* last cacheline: 28 bytes */
};
struct trace_event_raw_xdp_redirect_template {
	struct trace_entry         ent;                  /*     0     8 */
	int                        prog_id;              /*     8     4 */
	u32                        act;                  /*    12     4 */
	int                        ifindex;              /*    16     4 */
	int                        err;                  /*    20     4 */
	int                        to_ifindex;           /*    24     4 */
	u32                        map_id;               /*    28     4 */
	int                        map_index;            /*    32     4 */
	char                       __data[];             /*    36     0 */

	/* size: 36, cachelines: 1, members: 9 */
	/* last cacheline: 36 bytes */
};
struct trace_event_raw_xdp_cpumap_kthread {
	struct trace_entry         ent;                  /*     0     8 */
	int                        map_id;               /*     8     4 */
	u32                        act;                  /*    12     4 */
	int                        cpu;                  /*    16     4 */
	unsigned int               drops;                /*    20     4 */
	unsigned int               processed;            /*    24     4 */
	int                        sched;                /*    28     4 */
	unsigned int               xdp_pass;             /*    32     4 */
	unsigned int               xdp_drop;             /*    36     4 */
	unsigned int               xdp_redirect;         /*    40     4 */
	char                       __data[];             /*    44     0 */

	/* size: 44, cachelines: 1, members: 11 */
	/* last cacheline: 44 bytes */
};
struct trace_event_raw_xdp_cpumap_enqueue {
	struct trace_entry         ent;                  /*     0     8 */
	int                        map_id;               /*     8     4 */
	u32                        act;                  /*    12     4 */
	int                        cpu;                  /*    16     4 */
	unsigned int               drops;                /*    20     4 */
	unsigned int               processed;            /*    24     4 */
	int                        to_cpu;               /*    28     4 */
	char                       __data[];             /*    32     0 */

	/* size: 32, cachelines: 1, members: 8 */
	/* last cacheline: 32 bytes */
};
struct trace_event_raw_xdp_devmap_xmit {
	struct trace_entry         ent;                  /*     0     8 */
	int                        from_ifindex;         /*     8     4 */
	u32                        act;                  /*    12     4 */
	int                        to_ifindex;           /*    16     4 */
	int                        drops;                /*    20     4 */
	int                        sent;                 /*    24     4 */
	int                        err;                  /*    28     4 */
	char                       __data[];             /*    32     0 */

	/* size: 32, cachelines: 1, members: 8 */
	/* last cacheline: 32 bytes */
};
struct trace_event_raw_mem_disconnect {
	struct trace_entry         ent;                  /*     0     8 */
	const struct xdp_mem_allocator  * xa;            /*     8     8 */
	u32                        mem_id;               /*    16     4 */
	u32                        mem_type;             /*    20     4 */
	const void  *              allocator;            /*    24     8 */
	char                       __data[];             /*    32     0 */

	/* size: 32, cachelines: 1, members: 6 */
	/* last cacheline: 32 bytes */
};
struct trace_event_raw_mem_connect {
	struct trace_entry         ent;                  /*     0     8 */
	const struct xdp_mem_allocator  * xa;            /*     8     8 */
	u32                        mem_id;               /*    16     4 */
	u32                        mem_type;             /*    20     4 */
	const void  *              allocator;            /*    24     8 */
	const struct xdp_rxq_info  * rxq;                /*    32     8 */
	int                        ifindex;              /*    40     4 */
	char                       __data[];             /*    44     0 */

	/* size: 48, cachelines: 1, members: 8 */
	/* padding: 4 */
	/* last cacheline: 48 bytes */
};
struct trace_event_raw_mem_return_failed {
	struct trace_entry         ent;                  /*     0     8 */
	const struct page  *       page;                 /*     8     8 */
	u32                        mem_id;               /*    16     4 */
	u32                        mem_type;             /*    20     4 */
	char                       __data[];             /*    24     0 */

	/* size: 24, cachelines: 1, members: 5 */
	/* last cacheline: 24 bytes */
};
struct trace_event_data_offsets_xdp_exception {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_xdp_bulk_tx {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_xdp_redirect_template {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_xdp_cpumap_kthread {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_xdp_cpumap_enqueue {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_xdp_devmap_xmit {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_mem_disconnect {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_mem_connect {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_mem_return_failed {

	/* size: 0, cachelines: 0, members: 0 */
};
struct bpf_prog_info {
	__u32                      type;                 /*     0     4 */
	__u32                      id;                   /*     4     4 */
	__u8                       tag[8];               /*     8     8 */
	__u32                      jited_prog_len;       /*    16     4 */
	__u32                      xlated_prog_len;      /*    20     4 */
	__u64                      jited_prog_insns __attribute__((__aligned__(8))); /*    24     8 */
	__u64                      xlated_prog_insns __attribute__((__aligned__(8))); /*    32     8 */
	__u64                      load_time;            /*    40     8 */
	__u32                      created_by_uid;       /*    48     4 */
	__u32                      nr_map_ids;           /*    52     4 */
	__u64                      map_ids __attribute__((__aligned__(8))); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	char                       name[16];             /*    64    16 */
	__u32                      ifindex;              /*    80     4 */
	__u32                      gpl_compatible:1;     /*    84: 0  4 */

	/* XXX 31 bits hole, try to pack */

	__u64                      netns_dev;            /*    88     8 */
	__u64                      netns_ino;            /*    96     8 */
	__u32                      nr_jited_ksyms;       /*   104     4 */
	__u32                      nr_jited_func_lens;   /*   108     4 */
	__u64                      jited_ksyms __attribute__((__aligned__(8))); /*   112     8 */
	__u64                      jited_func_lens __attribute__((__aligned__(8))); /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	__u32                      btf_id;               /*   128     4 */
	__u32                      func_info_rec_size;   /*   132     4 */
	__u64                      func_info __attribute__((__aligned__(8))); /*   136     8 */
	__u32                      nr_func_info;         /*   144     4 */
	__u32                      nr_line_info;         /*   148     4 */
	__u64                      line_info __attribute__((__aligned__(8))); /*   152     8 */
	__u64                      jited_line_info __attribute__((__aligned__(8))); /*   160     8 */
	__u32                      nr_jited_line_info;   /*   168     4 */
	__u32                      line_info_rec_size;   /*   172     4 */
	__u32                      jited_line_info_rec_size; /*   176     4 */
	__u32                      nr_prog_tags;         /*   180     4 */
	__u64                      prog_tags __attribute__((__aligned__(8))); /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	__u64                      run_time_ns;          /*   192     8 */
	__u64                      run_cnt;              /*   200     8 */

	/* size: 208, cachelines: 4, members: 34 */
	/* sum members: 204 */
	/* sum bitfield members: 1 bits, bit holes: 1, sum bit holes: 31 bits */
	/* forced alignments: 9 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct bpf_map_info {
	__u32                      type;                 /*     0     4 */
	__u32                      id;                   /*     4     4 */
	__u32                      key_size;             /*     8     4 */
	__u32                      value_size;           /*    12     4 */
	__u32                      max_entries;          /*    16     4 */
	__u32                      map_flags;            /*    20     4 */
	char                       name[16];             /*    24    16 */
	__u32                      ifindex;              /*    40     4 */
	__u32                      btf_vmlinux_value_type_id; /*    44     4 */
	__u64                      netns_dev;            /*    48     8 */
	__u64                      netns_ino;            /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	__u32                      btf_id;               /*    64     4 */
	__u32                      btf_key_type_id;      /*    68     4 */
	__u32                      btf_value_type_id;    /*    72     4 */

	/* size: 80, cachelines: 2, members: 14 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct bpf_btf_info {
	__u64                      btf __attribute__((__aligned__(8))); /*     0     8 */
	__u32                      btf_size;             /*     8     4 */
	__u32                      id;                   /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* forced alignments: 1 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct bpf_spin_lock {
	__u32                      val;                  /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct bpf_attach_target_info {
	struct btf_func_model      fmodel;               /*     0    14 */

	/* XXX 2 bytes hole, try to pack */

	long int                   tgt_addr;             /*    16     8 */
	const char  *              tgt_name;             /*    24     8 */
	const struct btf_type  *   tgt_type;             /*    32     8 */

	/* size: 40, cachelines: 1, members: 4 */
	/* sum members: 38, holes: 1, sum holes: 2 */
	/* last cacheline: 40 bytes */
};
struct bpf_link_primer {
	struct bpf_link *          link;                 /*     0     8 */
	struct file *              file;                 /*     8     8 */
	int                        fd;                   /*    16     4 */
	u32                        id;                   /*    20     4 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct bpf_tracing_link {
	struct bpf_link            link;                 /*     0    96 */
	/* --- cacheline 1 boundary (64 bytes) was 32 bytes ago --- */
	enum bpf_attach_type       attach_type;          /*    96     4 */

	/* XXX 4 bytes hole, try to pack */

	struct bpf_trampoline *    trampoline;           /*   104     8 */
	struct bpf_prog *          tgt_prog;             /*   112     8 */

	/* size: 120, cachelines: 2, members: 4 */
	/* sum members: 116, holes: 1, sum holes: 4 */
	/* last cacheline: 56 bytes */
};
struct bpf_raw_tp_link {
	struct bpf_link            link;                 /*     0    96 */
	/* --- cacheline 1 boundary (64 bytes) was 32 bytes ago --- */
	struct bpf_raw_event_map * btp;                  /*    96     8 */

	/* size: 104, cachelines: 2, members: 2 */
	/* last cacheline: 40 bytes */
};
struct btf_member {
	__u32                      name_off;             /*     0     4 */
	__u32                      type;                 /*     4     4 */
	__u32                      offset;               /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct btf_var_secinfo {
	__u32                      type;                 /*     0     4 */
	__u32                      offset;               /*     4     4 */
	__u32                      size;                 /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct bpf_verifier_env {
	u32                        insn_idx;             /*     0     4 */
	u32                        prev_insn_idx;        /*     4     4 */
	struct bpf_prog *          prog;                 /*     8     8 */
	const struct bpf_verifier_ops  * ops;            /*    16     8 */
	struct bpf_verifier_stack_elem * head;           /*    24     8 */
	int                        stack_size;           /*    32     4 */
	bool                       strict_alignment;     /*    36     1 */
	bool                       test_state_freq;      /*    37     1 */

	/* XXX 2 bytes hole, try to pack */

	struct bpf_verifier_state * cur_state;           /*    40     8 */
	struct bpf_verifier_state_list * * explored_states; /*    48     8 */
	struct bpf_verifier_state_list * free_list;      /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct bpf_map *           used_maps[64];        /*    64   512 */
	/* --- cacheline 9 boundary (576 bytes) --- */
	u32                        used_map_cnt;         /*   576     4 */
	u32                        id_gen;               /*   580     4 */
	bool                       explore_alu_limits;   /*   584     1 */
	bool                       allow_ptr_leaks;      /*   585     1 */
	bool                       allow_uninit_stack;   /*   586     1 */
	bool                       allow_ptr_to_map_access; /*   587     1 */
	bool                       bpf_capable;          /*   588     1 */
	bool                       bypass_spec_v1;       /*   589     1 */
	bool                       bypass_spec_v4;       /*   590     1 */
	bool                       seen_direct_write;    /*   591     1 */
	struct bpf_insn_aux_data * insn_aux_data;        /*   592     8 */
	const struct bpf_line_info  * prev_linfo;        /*   600     8 */
	struct bpf_verifier_log    log;                  /*   608  1048 */
	/* --- cacheline 25 boundary (1600 bytes) was 56 bytes ago --- */
	struct bpf_subprog_info    subprog_info[257];    /*  1656  4112 */
	/* --- cacheline 90 boundary (5760 bytes) was 8 bytes ago --- */
	struct bpf_id_pair         idmap_scratch[75];    /*  5768   600 */
	/* --- cacheline 99 boundary (6336 bytes) was 32 bytes ago --- */
	struct {
		int *              insn_state;           /*  6368     8 */
		int *              insn_stack;           /*  6376     8 */
		int                cur_stack;            /*  6384     4 */
	} cfg;                                           /*  6368    24 */

	/* XXX last struct has 4 bytes of padding */

	u32                        pass_cnt;             /*  6392     4 */
	u32                        subprog_cnt;          /*  6396     4 */
	/* --- cacheline 100 boundary (6400 bytes) --- */
	u32                        prev_insn_processed;  /*  6400     4 */
	u32                        insn_processed;       /*  6404     4 */
	u32                        prev_jmps_processed;  /*  6408     4 */
	u32                        jmps_processed;       /*  6412     4 */
	u64                        verification_time;    /*  6416     8 */
	u32                        max_states_per_insn;  /*  6424     4 */
	u32                        total_states;         /*  6428     4 */
	u32                        peak_states;          /*  6432     4 */
	u32                        longest_mark_read_walk; /*  6436     4 */
	char                       type_str_buf[64];     /*  6440    64 */

	/* size: 6504, cachelines: 102, members: 40 */
	/* sum members: 6502, holes: 1, sum holes: 2 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 40 bytes */
};
struct bpf_struct_ops {
	const struct bpf_verifier_ops  * verifier_ops;   /*     0     8 */
	int                        (*init)(struct btf *); /*     8     8 */
	int                        (*check_member)(const struct btf_type  *, const struct btf_member  *); /*    16     8 */
	int                        (*init_member)(const struct btf_type  *, const struct btf_member  *, void *, const void  *); /*    24     8 */
	int                        (*reg)(void *);       /*    32     8 */
	void                       (*unreg)(void *);     /*    40     8 */
	const struct btf_type  *   type;                 /*    48     8 */
	const struct btf_type  *   value_type;           /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	const char  *              name;                 /*    64     8 */
	struct btf_func_model      func_models[64];      /*    72   896 */
	/* --- cacheline 15 boundary (960 bytes) was 8 bytes ago --- */
	u32                        type_id;              /*   968     4 */
	u32                        value_id;             /*   972     4 */

	/* size: 976, cachelines: 16, members: 12 */
	/* last cacheline: 16 bytes */
};
struct tnum {
	u64                        value;                /*     0     8 */
	u64                        mask;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct bpf_reg_state {
	enum bpf_reg_type          type;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	union {
		u16                range;                /*     8     2 */
		struct bpf_map *   map_ptr;              /*     8     8 */
		u32                btf_id;               /*     8     4 */
		u32                mem_size;             /*     8     4 */
		long unsigned int  raw;                  /*     8     8 */
	};                                               /*     8     8 */
	s32                        off;                  /*    16     4 */
	u32                        id;                   /*    20     4 */
	u32                        ref_obj_id;           /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	struct tnum                var_off;              /*    32    16 */
	s64                        smin_value;           /*    48     8 */
	s64                        smax_value;           /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u64                        umin_value;           /*    64     8 */
	u64                        umax_value;           /*    72     8 */
	s32                        s32_min_value;        /*    80     4 */
	s32                        s32_max_value;        /*    84     4 */
	u32                        u32_min_value;        /*    88     4 */
	u32                        u32_max_value;        /*    92     4 */
	struct bpf_reg_state *     parent;               /*    96     8 */
	u32                        frameno;              /*   104     4 */
	s32                        subreg_def;           /*   108     4 */
	enum bpf_reg_liveness      live;                 /*   112     4 */
	bool                       precise;              /*   116     1 */

	/* size: 120, cachelines: 2, members: 19 */
	/* sum members: 109, holes: 2, sum holes: 8 */
	/* padding: 3 */
	/* last cacheline: 56 bytes */
};
struct bpf_stack_state {
	struct bpf_reg_state       spilled_ptr;          /*     0   120 */

	/* XXX last struct has 3 bytes of padding */

	/* --- cacheline 1 boundary (64 bytes) was 56 bytes ago --- */
	u8                         slot_type[8];         /*   120     8 */

	/* size: 128, cachelines: 2, members: 2 */
	/* paddings: 1, sum paddings: 3 */
};
struct bpf_reference_state {
	int                        id;                   /*     0     4 */
	int                        insn_idx;             /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct bpf_func_state {
	struct bpf_reg_state       regs[11];             /*     0  1320 */
	/* --- cacheline 20 boundary (1280 bytes) was 40 bytes ago --- */
	int                        callsite;             /*  1320     4 */
	u32                        frameno;              /*  1324     4 */
	u32                        subprogno;            /*  1328     4 */
	int                        acquired_refs;        /*  1332     4 */
	struct bpf_reference_state * refs;               /*  1336     8 */
	/* --- cacheline 21 boundary (1344 bytes) --- */
	int                        allocated_stack;      /*  1344     4 */

	/* XXX 4 bytes hole, try to pack */

	struct bpf_stack_state *   stack;                /*  1352     8 */

	/* size: 1360, cachelines: 22, members: 8 */
	/* sum members: 1356, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct bpf_idx_pair {
	u32                        prev_idx;             /*     0     4 */
	u32                        idx;                  /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct bpf_id_pair {
	u32                        old;                  /*     0     4 */
	u32                        cur;                  /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct bpf_verifier_state {
	struct bpf_func_state *    frame[8];             /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct bpf_verifier_state * parent;              /*    64     8 */
	u32                        branches;             /*    72     4 */
	u32                        insn_idx;             /*    76     4 */
	u32                        curframe;             /*    80     4 */
	u32                        active_spin_lock;     /*    84     4 */
	bool                       speculative;          /*    88     1 */

	/* XXX 3 bytes hole, try to pack */

	u32                        first_insn_idx;       /*    92     4 */
	u32                        last_insn_idx;        /*    96     4 */

	/* XXX 4 bytes hole, try to pack */

	struct bpf_idx_pair *      jmp_history;          /*   104     8 */
	u32                        jmp_history_cnt;      /*   112     4 */

	/* size: 120, cachelines: 2, members: 11 */
	/* sum members: 109, holes: 2, sum holes: 7 */
	/* padding: 4 */
	/* last cacheline: 56 bytes */
};
struct bpf_verifier_state_list {
	struct bpf_verifier_state  state;                /*     0   120 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 1 boundary (64 bytes) was 56 bytes ago --- */
	struct bpf_verifier_state_list * next;           /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	int                        miss_cnt;             /*   128     4 */
	int                        hit_cnt;              /*   132     4 */

	/* size: 136, cachelines: 3, members: 4 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 8 bytes */
};
struct bpf_insn_aux_data {
	union {
		enum bpf_reg_type  ptr_type;             /*     0     4 */
		long unsigned int  map_ptr_state;        /*     0     8 */
		s32                call_imm;             /*     0     4 */
		u32                alu_limit;            /*     0     4 */
		struct {
			u32        map_index;            /*     0     4 */
			u32        map_off;              /*     4     4 */
		};                                       /*     0     8 */
		struct {
			enum bpf_reg_type reg_type;      /*     0     4 */
			union {
				u32 btf_id;              /*     4     4 */
				u32 mem_size;            /*     4     4 */
			};                               /*     4     4 */
		} btf_var;                               /*     0     8 */
	};                                               /*     0     8 */
	u64                        map_key_state;        /*     8     8 */
	int                        ctx_field_size;       /*    16     4 */
	u32                        seen;                 /*    20     4 */
	bool                       sanitize_stack_spill; /*    24     1 */
	bool                       zext_dst;             /*    25     1 */
	u8                         alu_state;            /*    26     1 */

	/* XXX 1 byte hole, try to pack */

	unsigned int               orig_idx;             /*    28     4 */
	bool                       prune_point;          /*    32     1 */

	/* size: 40, cachelines: 1, members: 9 */
	/* sum members: 32, holes: 1, sum holes: 1 */
	/* padding: 7 */
	/* last cacheline: 40 bytes */
};
struct bpf_subprog_info {
	u32                        start;                /*     0     4 */
	u32                        linfo_idx;            /*     4     4 */
	u16                        stack_depth;          /*     8     2 */
	bool                       has_tail_call;        /*    10     1 */
	bool                       tail_call_reachable;  /*    11     1 */
	bool                       has_ld_abs;           /*    12     1 */

	/* size: 16, cachelines: 1, members: 6 */
	/* padding: 3 */
	/* last cacheline: 16 bytes */
};
struct bpf_verifier_stack_elem {
	struct bpf_verifier_state  st;                   /*     0   120 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 1 boundary (64 bytes) was 56 bytes ago --- */
	int                        insn_idx;             /*   120     4 */
	int                        prev_insn_idx;        /*   124     4 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct bpf_verifier_stack_elem * next;           /*   128     8 */
	u32                        log_pos;              /*   136     4 */

	/* size: 144, cachelines: 3, members: 5 */
	/* padding: 4 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 16 bytes */
};
struct bpf_insn_cbs {
	bpf_insn_print_t           cb_print;             /*     0     8 */
	bpf_insn_revmap_call_t     cb_call;              /*     8     8 */
	bpf_insn_print_imm_t       cb_imm;               /*    16     8 */
	void *                     private_data;         /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct bpf_call_arg_meta {
	struct bpf_map *           map_ptr;              /*     0     8 */
	bool                       raw_mode;             /*     8     1 */
	bool                       pkt_access;           /*     9     1 */

	/* XXX 2 bytes hole, try to pack */

	int                        regno;                /*    12     4 */
	int                        access_size;          /*    16     4 */
	int                        mem_size;             /*    20     4 */
	u64                        msize_max_value;      /*    24     8 */
	int                        ref_obj_id;           /*    32     4 */
	int                        func_id;              /*    36     4 */
	u32                        btf_id;               /*    40     4 */
	u32                        ret_btf_id;           /*    44     4 */

	/* size: 48, cachelines: 1, members: 11 */
	/* sum members: 46, holes: 1, sum holes: 2 */
	/* last cacheline: 48 bytes */
};
struct bpf_reg_types {
	const const enum bpf_reg_type types[10];         /*     0    40 */
	u32 *                      btf_id;               /*    40     8 */

	/* size: 48, cachelines: 1, members: 2 */
	/* last cacheline: 48 bytes */
};
struct bpf_sanitize_info {
	struct bpf_insn_aux_data   aux;                  /*     0    40 */

	/* XXX last struct has 7 bytes of padding */

	bool                       mask_to_left;         /*    40     1 */

	/* size: 48, cachelines: 1, members: 2 */
	/* padding: 7 */
	/* paddings: 1, sum paddings: 7 */
	/* last cacheline: 48 bytes */
};
struct tree_descr {
	const char  *              name;                 /*     0     8 */
	const struct file_operations  * ops;             /*     8     8 */
	int                        mode;                 /*    16     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct umd_info {
	const char  *              driver_name;          /*     0     8 */
	struct file *              pipe_to_umh;          /*     8     8 */
	struct file *              pipe_from_umh;        /*    16     8 */
	struct path                wd;                   /*    24    16 */
	struct pid *               tgid;                 /*    40     8 */

	/* size: 48, cachelines: 1, members: 5 */
	/* last cacheline: 48 bytes */
};
struct bpf_preload_info {
	char                       link_name[16];        /*     0    16 */
	int                        link_id;              /*    16     4 */

	/* size: 20, cachelines: 1, members: 2 */
	/* last cacheline: 20 bytes */
};
struct bpf_preload_ops {
	struct umd_info            info;                 /*     0    48 */
	int                        (*preload)(struct bpf_preload_info *); /*    48     8 */
	int                        (*finish)(void);      /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct module *            owner;                /*    64     8 */

	/* size: 72, cachelines: 2, members: 4 */
	/* last cacheline: 8 bytes */
};
struct map_iter {
	void *                     key;                  /*     0     8 */
	bool                       done;                 /*     8     1 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 7 */
	/* last cacheline: 16 bytes */
};
struct bpf_mount_opts {
	umode_t                    mode;                 /*     0     2 */

	/* size: 2, cachelines: 1, members: 1 */
	/* last cacheline: 2 bytes */
};
struct bpf_pidns_info {
	__u32                      pid;                  /*     0     4 */
	__u32                      tgid;                 /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
union bpf_iter_link_info {
	struct {
		__u32              map_fd;             /*     0     4 */
	} map;                                         /*     0     4 */
};
struct bpf_iter_reg {
	const char  *              target;               /*     0     8 */
	bpf_iter_attach_target_t   attach_target;        /*     8     8 */
	bpf_iter_detach_target_t   detach_target;        /*    16     8 */
	bpf_iter_show_fdinfo_t     show_fdinfo;          /*    24     8 */
	bpf_iter_fill_link_info_t  fill_link_info;       /*    32     8 */
	u32                        ctx_arg_info_size;    /*    40     4 */
	struct bpf_ctx_arg_aux     ctx_arg_info[2];      /*    44    24 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	const struct bpf_iter_seq_info  * seq_info;      /*    72     8 */

	/* size: 80, cachelines: 2, members: 8 */
	/* sum members: 76, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct bpf_iter_meta {
	union {
		struct seq_file *  seq;                  /*     0     8 */
	} __attribute__((__aligned__(8)));               /*     0     8 */
	u64                        session_id;           /*     8     8 */
	u64                        seq_num;              /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* forced alignments: 1 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct bpf_iter_target_info {
	struct list_head           list;                 /*     0    16 */
	const struct bpf_iter_reg  * reg_info;           /*    16     8 */
	u32                        btf_id;               /*    24     4 */

	/* size: 32, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct bpf_iter_link {
	struct bpf_link            link;                 /*     0    96 */
	/* --- cacheline 1 boundary (64 bytes) was 32 bytes ago --- */
	struct bpf_iter_aux_info   aux;                  /*    96     8 */
	struct bpf_iter_target_info * tinfo;             /*   104     8 */

	/* size: 112, cachelines: 2, members: 3 */
	/* last cacheline: 48 bytes */
};
struct bpf_iter_priv_data {
	struct bpf_iter_target_info * tinfo;             /*     0     8 */
	const struct bpf_iter_seq_info  * seq_info;      /*     8     8 */
	struct bpf_prog *          prog;                 /*    16     8 */
	u64                        session_id;           /*    24     8 */
	u64                        seq_num;              /*    32     8 */
	bool                       done_stop;            /*    40     1 */

	/* XXX 7 bytes hole, try to pack */

	u8                         target_private[] __attribute__((__aligned__(8))); /*    48     0 */

	/* size: 48, cachelines: 1, members: 7 */
	/* sum members: 41, holes: 1, sum holes: 7 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 7 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct bpf_iter_seq_map_info {
	u32                        map_id;               /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct bpf_iter__bpf_map {
	union {
		struct bpf_iter_meta * meta;             /*     0     8 */
	} __attribute__((__aligned__(8)));               /*     0     8 */
	union {
		struct bpf_map *   map;                  /*     8     8 */
	} __attribute__((__aligned__(8)));               /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* forced alignments: 2 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct bpf_iter_seq_task_common {
	struct pid_namespace *     ns;                   /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct bpf_iter_seq_task_info {
	struct bpf_iter_seq_task_common common;          /*     0     8 */
	u32                        tid;                  /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct bpf_iter__task {
	union {
		struct bpf_iter_meta * meta;             /*     0     8 */
	} __attribute__((__aligned__(8)));               /*     0     8 */
	union {
		struct task_struct * task;               /*     8     8 */
	} __attribute__((__aligned__(8)));               /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* forced alignments: 2 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct bpf_iter_seq_task_file_info {
	struct bpf_iter_seq_task_common common;          /*     0     8 */
	struct task_struct *       task;                 /*     8     8 */
	struct files_struct *      files;                /*    16     8 */
	u32                        tid;                  /*    24     4 */
	u32                        fd;                   /*    28     4 */

	/* size: 32, cachelines: 1, members: 5 */
	/* last cacheline: 32 bytes */
};
struct bpf_iter__task_file {
	union {
		struct bpf_iter_meta * meta;             /*     0     8 */
	} __attribute__((__aligned__(8)));               /*     0     8 */
	union {
		struct task_struct * task;               /*     8     8 */
	} __attribute__((__aligned__(8)));               /*     8     8 */
	u32                        fd __attribute__((__aligned__(8))); /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	union {
		struct file *      file;                 /*    24     8 */
	} __attribute__((__aligned__(8)));               /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* forced alignments: 4, forced holes: 1, sum forced holes: 4 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct bpf_iter_seq_prog_info {
	u32                        prog_id;              /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct bpf_iter__bpf_prog {
	union {
		struct bpf_iter_meta * meta;             /*     0     8 */
	} __attribute__((__aligned__(8)));               /*     0     8 */
	union {
		struct bpf_prog *  prog;                 /*     8     8 */
	} __attribute__((__aligned__(8)));               /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* forced alignments: 2 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct bpf_iter__bpf_map_elem {
	union {
		struct bpf_iter_meta * meta;             /*     0     8 */
	} __attribute__((__aligned__(8)));               /*     0     8 */
	union {
		struct bpf_map *   map;                  /*     8     8 */
	} __attribute__((__aligned__(8)));               /*     8     8 */
	union {
		void *             key;                  /*    16     8 */
	} __attribute__((__aligned__(8)));               /*    16     8 */
	union {
		void *             value;                /*    24     8 */
	} __attribute__((__aligned__(8)));               /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* forced alignments: 4 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct pcpu_freelist_head {
	struct pcpu_freelist_node * first;               /*     0     8 */
	raw_spinlock_t             lock;                 /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct pcpu_freelist_node {
	struct pcpu_freelist_node * next;                /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct pcpu_freelist {
	struct pcpu_freelist_head * freelist;            /*     0     8 */
	struct pcpu_freelist_head  extralist;            /*     8    16 */

	/* XXX last struct has 4 bytes of padding */

	/* size: 24, cachelines: 1, members: 2 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 24 bytes */
};
struct bpf_lru_node {
	struct list_head           list;                 /*     0    16 */
	u16                        cpu;                  /*    16     2 */
	u8                         type;                 /*    18     1 */
	u8                         ref;                  /*    19     1 */

	/* size: 24, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct bpf_lru_list {
	struct list_head           lists[3];             /*     0    48 */
	unsigned int               counts[2];            /*    48     8 */
	struct list_head *         next_inactive_rotation; /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	raw_spinlock_t             lock __attribute__((__aligned__(64))); /*    64     4 */

	/* size: 128, cachelines: 2, members: 4 */
	/* padding: 60 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(64)));
struct bpf_lru_locallist {
	struct list_head           lists[2];             /*     0    32 */
	u16                        next_steal;           /*    32     2 */

	/* XXX 2 bytes hole, try to pack */

	raw_spinlock_t             lock;                 /*    36     4 */

	/* size: 40, cachelines: 1, members: 3 */
	/* sum members: 38, holes: 1, sum holes: 2 */
	/* last cacheline: 40 bytes */
};
struct bpf_common_lru {
	struct bpf_lru_list        lru_list __attribute__((__aligned__(64))); /*     0   128 */

	/* XXX last struct has 60 bytes of padding */

	/* --- cacheline 2 boundary (128 bytes) --- */
	struct bpf_lru_locallist * local_list;           /*   128     8 */

	/* size: 192, cachelines: 3, members: 2 */
	/* padding: 56 */
	/* paddings: 1, sum paddings: 60 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(64)));
struct bpf_lru {
	union {
		struct bpf_common_lru common_lru __attribute__((__aligned__(64))); /*     0   192 */
		struct bpf_lru_list * percpu_lru;        /*     0     8 */
	} __attribute__((__aligned__(64)));              /*     0   192 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	del_from_htab_func         del_from_htab;        /*   192     8 */
	void *                     del_arg;              /*   200     8 */
	unsigned int               hash_offset;          /*   208     4 */
	unsigned int               nr_scans;             /*   212     4 */
	bool                       percpu;               /*   216     1 */

	/* size: 256, cachelines: 4, members: 6 */
	/* padding: 39 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(64)));
struct bucket {
	struct hlist_nulls_head    head;                 /*     0     8 */
	union {
		raw_spinlock_t     raw_lock;             /*     8     4 */
		spinlock_t         lock;                 /*     8     4 */
	};                                               /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct bpf_htab {
	struct bpf_map             map __attribute__((__aligned__(64))); /*     0   256 */

	/* XXX last struct has 8 bytes of padding */

	/* --- cacheline 4 boundary (256 bytes) --- */
	struct bucket *            buckets;              /*   256     8 */
	void *                     elems;                /*   264     8 */

	/* XXX 48 bytes hole, try to pack */

	/* --- cacheline 5 boundary (320 bytes) --- */
	union {
		struct pcpu_freelist freelist;           /*   320    24 */
		struct bpf_lru     lru __attribute__((__aligned__(64))); /*   320   256 */
	} __attribute__((__aligned__(64)));              /*   320   256 */
	/* --- cacheline 9 boundary (576 bytes) --- */
	struct htab_elem * *       extra_elems;          /*   576     8 */
	atomic_t                   count;                /*   584     4 */
	u32                        n_buckets;            /*   588     4 */
	u32                        elem_size;            /*   592     4 */
	u32                        hashrnd;              /*   596     4 */

	/* size: 640, cachelines: 10, members: 9 */
	/* sum members: 552, holes: 1, sum holes: 48 */
	/* padding: 40 */
	/* paddings: 1, sum paddings: 8 */
	/* forced alignments: 2, forced holes: 1, sum forced holes: 48 */
} __attribute__((__aligned__(64)));
struct htab_elem {
	union {
		struct hlist_nulls_node hash_node;       /*     0    16 */
		struct {
			void *     padding;              /*     0     8 */
			union {
				struct bpf_htab * htab;  /*     8     8 */
				struct pcpu_freelist_node fnode; /*     8     8 */
				struct htab_elem * batch_flink; /*     8     8 */
			};                               /*     8     8 */
		};                                       /*     0    16 */
	};                                               /*     0    16 */
	union {
		struct callback_head rcu __attribute__((__aligned__(8))); /*    16    16 */
		struct bpf_lru_node lru_node;            /*    16    24 */
	} __attribute__((__aligned__(8)));               /*    16    24 */
	u32                        hash;                 /*    40     4 */

	/* XXX 4 bytes hole, try to pack */

	char                       key[] __attribute__((__aligned__(8))); /*    48     0 */

	/* size: 48, cachelines: 1, members: 4 */
	/* sum members: 44, holes: 1, sum holes: 4 */
	/* forced alignments: 2, forced holes: 1, sum forced holes: 4 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct bpf_iter_seq_hash_map_info {
	struct bpf_map *           map;                  /*     0     8 */
	struct bpf_htab *          htab;                 /*     8     8 */
	void *                     percpu_value_buf;     /*    16     8 */
	u32                        bucket_id;            /*    24     4 */
	u32                        skip_elems;           /*    28     4 */

	/* size: 32, cachelines: 1, members: 5 */
	/* last cacheline: 32 bytes */
};
struct bpf_iter_seq_array_map_info {
	struct bpf_map *           map;                  /*     0     8 */
	void *                     percpu_value_buf;     /*     8     8 */
	u32                        index;                /*    16     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct prog_poke_elem {
	struct list_head           list;                 /*     0    16 */
	struct bpf_prog_aux *      aux;                  /*    16     8 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct bpf_lpm_trie_key {
	__u32                      prefixlen;            /*     0     4 */
	__u8                       data[];               /*     4     0 */

	/* size: 4, cachelines: 1, members: 2 */
	/* last cacheline: 4 bytes */
};
struct lpm_trie_node {
	struct callback_head       rcu __attribute__((__aligned__(8))); /*     0    16 */
	struct lpm_trie_node *     child[2];             /*    16    16 */
	u32                        prefixlen;            /*    32     4 */
	u32                        flags;                /*    36     4 */
	u8                         data[];               /*    40     0 */

	/* size: 40, cachelines: 1, members: 5 */
	/* forced alignments: 1 */
	/* last cacheline: 40 bytes */
} __attribute__((__aligned__(8)));
struct lpm_trie {
	struct bpf_map             map __attribute__((__aligned__(64))); /*     0   256 */

	/* XXX last struct has 8 bytes of padding */

	/* --- cacheline 4 boundary (256 bytes) --- */
	struct lpm_trie_node *     root;                 /*   256     8 */
	size_t                     n_entries;            /*   264     8 */
	size_t                     max_prefixlen;        /*   272     8 */
	size_t                     data_size;            /*   280     8 */
	spinlock_t                 lock;                 /*   288     4 */

	/* size: 320, cachelines: 5, members: 6 */
	/* padding: 28 */
	/* paddings: 1, sum paddings: 8 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(64)));
struct bpf_cgroup_storage_map {
	struct bpf_map             map __attribute__((__aligned__(64))); /*     0   256 */

	/* XXX last struct has 8 bytes of padding */

	/* --- cacheline 4 boundary (256 bytes) --- */
	spinlock_t                 lock;                 /*   256     4 */

	/* XXX 4 bytes hole, try to pack */

	struct rb_root             root;                 /*   264     8 */
	struct list_head           list;                 /*   272    16 */

	/* size: 320, cachelines: 5, members: 4 */
	/* sum members: 284, holes: 1, sum holes: 4 */
	/* padding: 32 */
	/* paddings: 1, sum paddings: 8 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(64)));
struct bpf_queue_stack {
	struct bpf_map             map __attribute__((__aligned__(64))); /*     0   256 */

	/* XXX last struct has 8 bytes of padding */

	/* --- cacheline 4 boundary (256 bytes) --- */
	raw_spinlock_t             lock;                 /*   256     4 */
	u32                        head;                 /*   260     4 */
	u32                        tail;                 /*   264     4 */
	u32                        size;                 /*   268     4 */
	char                       elements[] __attribute__((__aligned__(8))); /*   272     0 */

	/* size: 320, cachelines: 5, members: 6 */
	/* padding: 48 */
	/* paddings: 1, sum paddings: 8 */
	/* forced alignments: 2 */
} __attribute__((__aligned__(64)));
struct bpf_ringbuf {
	wait_queue_head_t          waitq;                /*     0    24 */
	struct irq_work            work;                 /*    24    24 */
	u64                        mask;                 /*    48     8 */
	struct page * *            pages;                /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        nr_pages;             /*    64     4 */

	/* XXX 60 bytes hole, try to pack */

	/* --- cacheline 2 boundary (128 bytes) --- */
	spinlock_t                 spinlock __attribute__((__aligned__(64))); /*   128     4 */

	/* XXX 3964 bytes hole, try to pack */

	/* --- cacheline 64 boundary (4096 bytes) --- */
	long unsigned int          consumer_pos __attribute__((__aligned__(4096))); /*  4096     8 */

	/* XXX 4088 bytes hole, try to pack */

	/* --- cacheline 128 boundary (8192 bytes) --- */
	long unsigned int          producer_pos __attribute__((__aligned__(4096))); /*  8192     8 */

	/* XXX 4088 bytes hole, try to pack */

	/* --- cacheline 192 boundary (12288 bytes) --- */
	char                       data[] __attribute__((__aligned__(4096))); /* 12288     0 */

	/* size: 12288, cachelines: 192, members: 9 */
	/* sum members: 88, holes: 4, sum holes: 12200 */
	/* forced alignments: 4, forced holes: 4, sum forced holes: 12200 */
} __attribute__((__aligned__(4096)));
struct bpf_ringbuf_map {
	struct bpf_map             map __attribute__((__aligned__(64))); /*     0   256 */

	/* XXX last struct has 8 bytes of padding */

	/* --- cacheline 4 boundary (256 bytes) --- */
	struct bpf_map_memory      memory;               /*   256    16 */
	struct bpf_ringbuf *       rb;                   /*   272     8 */

	/* size: 320, cachelines: 5, members: 3 */
	/* padding: 40 */
	/* paddings: 1, sum paddings: 8 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(64)));
struct bpf_ringbuf_hdr {
	u32                        len;                  /*     0     4 */
	u32                        pg_off;               /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct btf_header {
	__u16                      magic;                /*     0     2 */
	__u8                       version;              /*     2     1 */
	__u8                       flags;                /*     3     1 */
	__u32                      hdr_len;              /*     4     4 */
	__u32                      type_off;             /*     8     4 */
	__u32                      type_len;             /*    12     4 */
	__u32                      str_off;              /*    16     4 */
	__u32                      str_len;              /*    20     4 */

	/* size: 24, cachelines: 1, members: 8 */
	/* last cacheline: 24 bytes */
};
struct btf_enum {
	__u32                      name_off;             /*     0     4 */
	__s32                      val;                  /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct btf_array {
	__u32                      type;                 /*     0     4 */
	__u32                      index_type;           /*     4     4 */
	__u32                      nelems;               /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct btf_param {
	__u32                      name_off;             /*     0     4 */
	__u32                      type;                 /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct btf_var {
	__u32                      linkage;              /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct bpf_flow_keys {
	__u16                      nhoff;                /*     0     2 */
	__u16                      thoff;                /*     2     2 */
	__u16                      addr_proto;           /*     4     2 */
	__u8                       is_frag;              /*     6     1 */
	__u8                       is_first_frag;        /*     7     1 */
	__u8                       is_encap;             /*     8     1 */
	__u8                       ip_proto;             /*     9     1 */
	__be16                     n_proto;              /*    10     2 */
	__be16                     sport;                /*    12     2 */
	__be16                     dport;                /*    14     2 */
	union {
		struct {
			__be32     ipv4_src;             /*    16     4 */
			__be32     ipv4_dst;             /*    20     4 */
		};                                       /*    16     8 */
		struct {
			__u32      ipv6_src[4];          /*    16    16 */
			__u32      ipv6_dst[4];          /*    32    16 */
		};                                       /*    16    32 */
	};                                               /*    16    32 */
	__u32                      flags;                /*    48     4 */
	__be32                     flow_label;           /*    52     4 */

	/* size: 56, cachelines: 1, members: 13 */
	/* last cacheline: 56 bytes */
};
struct bpf_sock {
	__u32                      bound_dev_if;         /*     0     4 */
	__u32                      family;               /*     4     4 */
	__u32                      type;                 /*     8     4 */
	__u32                      protocol;             /*    12     4 */
	__u32                      mark;                 /*    16     4 */
	__u32                      priority;             /*    20     4 */
	__u32                      src_ip4;              /*    24     4 */
	__u32                      src_ip6[4];           /*    28    16 */
	__u32                      src_port;             /*    44     4 */
	__be16                     dst_port;             /*    48     2 */

	/* XXX 2 bytes hole, try to pack */

	__u32                      dst_ip4;              /*    52     4 */
	__u32                      dst_ip6[4];           /*    56    16 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	__u32                      state;                /*    72     4 */
	__s32                      rx_queue_mapping;     /*    76     4 */

	/* size: 80, cachelines: 2, members: 14 */
	/* sum members: 78, holes: 1, sum holes: 2 */
	/* last cacheline: 16 bytes */
};
struct __sk_buff {
	__u32                      len;                  /*     0     4 */
	__u32                      pkt_type;             /*     4     4 */
	__u32                      mark;                 /*     8     4 */
	__u32                      queue_mapping;        /*    12     4 */
	__u32                      protocol;             /*    16     4 */
	__u32                      vlan_present;         /*    20     4 */
	__u32                      vlan_tci;             /*    24     4 */
	__u32                      vlan_proto;           /*    28     4 */
	__u32                      priority;             /*    32     4 */
	__u32                      ingress_ifindex;      /*    36     4 */
	__u32                      ifindex;              /*    40     4 */
	__u32                      tc_index;             /*    44     4 */
	__u32                      cb[5];                /*    48    20 */
	/* --- cacheline 1 boundary (64 bytes) was 4 bytes ago --- */
	__u32                      hash;                 /*    68     4 */
	__u32                      tc_classid;           /*    72     4 */
	__u32                      data;                 /*    76     4 */
	__u32                      data_end;             /*    80     4 */
	__u32                      napi_id;              /*    84     4 */
	__u32                      family;               /*    88     4 */
	__u32                      remote_ip4;           /*    92     4 */
	__u32                      local_ip4;            /*    96     4 */
	__u32                      remote_ip6[4];        /*   100    16 */
	__u32                      local_ip6[4];         /*   116    16 */
	/* --- cacheline 2 boundary (128 bytes) was 4 bytes ago --- */
	__u32                      remote_port;          /*   132     4 */
	__u32                      local_port;           /*   136     4 */
	__u32                      data_meta;            /*   140     4 */
	union {
		struct bpf_flow_keys * flow_keys;        /*   144     8 */
	} __attribute__((__aligned__(8)));               /*   144     8 */
	__u64                      tstamp;               /*   152     8 */
	__u32                      wire_len;             /*   160     4 */
	__u32                      gso_segs;             /*   164     4 */
	union {
		struct bpf_sock *  sk;                   /*   168     8 */
	} __attribute__((__aligned__(8)));               /*   168     8 */
	__u32                      gso_size;             /*   176     4 */

	/* size: 184, cachelines: 3, members: 32 */
	/* padding: 4 */
	/* forced alignments: 2 */
	/* last cacheline: 56 bytes */
} __attribute__((__aligned__(8)));
struct xdp_md {
	__u32                      data;                 /*     0     4 */
	__u32                      data_end;             /*     4     4 */
	__u32                      data_meta;            /*     8     4 */
	__u32                      ingress_ifindex;      /*    12     4 */
	__u32                      rx_queue_index;       /*    16     4 */
	__u32                      egress_ifindex;       /*    20     4 */

	/* size: 24, cachelines: 1, members: 6 */
	/* last cacheline: 24 bytes */
};
struct sk_msg_md {
	union {
		void *             data;                 /*     0     8 */
	} __attribute__((__aligned__(8)));               /*     0     8 */
	union {
		void *             data_end;             /*     8     8 */
	} __attribute__((__aligned__(8)));               /*     8     8 */
	__u32                      family;               /*    16     4 */
	__u32                      remote_ip4;           /*    20     4 */
	__u32                      local_ip4;            /*    24     4 */
	__u32                      remote_ip6[4];        /*    28    16 */
	__u32                      local_ip6[4];         /*    44    16 */
	__u32                      remote_port;          /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	__u32                      local_port;           /*    64     4 */
	__u32                      size;                 /*    68     4 */
	union {
		struct bpf_sock *  sk;                   /*    72     8 */
	} __attribute__((__aligned__(8)));               /*    72     8 */

	/* size: 80, cachelines: 2, members: 11 */
	/* forced alignments: 3 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct sk_reuseport_md {
	union {
		void *             data;                 /*     0     8 */
	} __attribute__((__aligned__(8)));               /*     0     8 */
	union {
		void *             data_end;             /*     8     8 */
	} __attribute__((__aligned__(8)));               /*     8     8 */
	__u32                      len;                  /*    16     4 */
	__u32                      eth_protocol;         /*    20     4 */
	__u32                      ip_protocol;          /*    24     4 */
	__u32                      bind_inany;           /*    28     4 */
	__u32                      hash;                 /*    32     4 */

	/* size: 40, cachelines: 1, members: 7 */
	/* padding: 4 */
	/* forced alignments: 2 */
	/* last cacheline: 40 bytes */
} __attribute__((__aligned__(8)));
struct bpf_sock_addr {
	__u32                      user_family;          /*     0     4 */
	__u32                      user_ip4;             /*     4     4 */
	__u32                      user_ip6[4];          /*     8    16 */
	__u32                      user_port;            /*    24     4 */
	__u32                      family;               /*    28     4 */
	__u32                      type;                 /*    32     4 */
	__u32                      protocol;             /*    36     4 */
	__u32                      msg_src_ip4;          /*    40     4 */
	__u32                      msg_src_ip6[4];       /*    44    16 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	union {
		struct bpf_sock *  sk;                   /*    64     8 */
	} __attribute__((__aligned__(8)));               /*    64     8 */

	/* size: 72, cachelines: 2, members: 10 */
	/* sum members: 68, holes: 1, sum holes: 4 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 4 */
	/* last cacheline: 8 bytes */
} __attribute__((__aligned__(8)));
struct bpf_sock_ops {
	__u32                      op;                   /*     0     4 */
	union {
		__u32              args[4];              /*     4    16 */
		__u32              reply;                /*     4     4 */
		__u32              replylong[4];         /*     4    16 */
	};                                               /*     4    16 */
	__u32                      family;               /*    20     4 */
	__u32                      remote_ip4;           /*    24     4 */
	__u32                      local_ip4;            /*    28     4 */
	__u32                      remote_ip6[4];        /*    32    16 */
	__u32                      local_ip6[4];         /*    48    16 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	__u32                      remote_port;          /*    64     4 */
	__u32                      local_port;           /*    68     4 */
	__u32                      is_fullsock;          /*    72     4 */
	__u32                      snd_cwnd;             /*    76     4 */
	__u32                      srtt_us;              /*    80     4 */
	__u32                      bpf_sock_ops_cb_flags; /*    84     4 */
	__u32                      state;                /*    88     4 */
	__u32                      rtt_min;              /*    92     4 */
	__u32                      snd_ssthresh;         /*    96     4 */
	__u32                      rcv_nxt;              /*   100     4 */
	__u32                      snd_nxt;              /*   104     4 */
	__u32                      snd_una;              /*   108     4 */
	__u32                      mss_cache;            /*   112     4 */
	__u32                      ecn_flags;            /*   116     4 */
	__u32                      rate_delivered;       /*   120     4 */
	__u32                      rate_interval_us;     /*   124     4 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	__u32                      packets_out;          /*   128     4 */
	__u32                      retrans_out;          /*   132     4 */
	__u32                      total_retrans;        /*   136     4 */
	__u32                      segs_in;              /*   140     4 */
	__u32                      data_segs_in;         /*   144     4 */
	__u32                      segs_out;             /*   148     4 */
	__u32                      data_segs_out;        /*   152     4 */
	__u32                      lost_out;             /*   156     4 */
	__u32                      sacked_out;           /*   160     4 */
	__u32                      sk_txhash;            /*   164     4 */
	__u64                      bytes_received;       /*   168     8 */
	__u64                      bytes_acked;          /*   176     8 */
	union {
		struct bpf_sock *  sk;                   /*   184     8 */
	} __attribute__((__aligned__(8)));               /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	union {
		void *             skb_data;             /*   192     8 */
	} __attribute__((__aligned__(8)));               /*   192     8 */
	union {
		void *             skb_data_end;         /*   200     8 */
	} __attribute__((__aligned__(8)));               /*   200     8 */
	__u32                      skb_len;              /*   208     4 */
	__u32                      skb_tcp_flags;        /*   212     4 */

	/* size: 216, cachelines: 4, members: 40 */
	/* forced alignments: 3 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct bpf_cgroup_dev_ctx {
	__u32                      access_type;          /*     0     4 */
	__u32                      major;                /*     4     4 */
	__u32                      minor;                /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct bpf_sysctl {
	__u32                      write;                /*     0     4 */
	__u32                      file_pos;             /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct bpf_sockopt {
	union {
		struct bpf_sock *  sk;                   /*     0     8 */
	} __attribute__((__aligned__(8)));               /*     0     8 */
	union {
		void *             optval;               /*     8     8 */
	} __attribute__((__aligned__(8)));               /*     8     8 */
	union {
		void *             optval_end;           /*    16     8 */
	} __attribute__((__aligned__(8)));               /*    16     8 */
	__s32                      level;                /*    24     4 */
	__s32                      optname;              /*    28     4 */
	__s32                      optlen;               /*    32     4 */
	__s32                      retval;               /*    36     4 */

	/* size: 40, cachelines: 1, members: 7 */
	/* forced alignments: 3 */
	/* last cacheline: 40 bytes */
} __attribute__((__aligned__(8)));
struct bpf_sk_lookup {
	union {
		union {
			struct bpf_sock * sk;            /*     0     8 */
		} __attribute__((__aligned__(8)));       /*     0     8 */
		__u64              cookie;               /*     0     8 */
	} __attribute__((__aligned__(8)));               /*     0     8 */
	__u32                      family;               /*     8     4 */
	__u32                      protocol;             /*    12     4 */
	__u32                      remote_ip4;           /*    16     4 */
	__u32                      remote_ip6[4];        /*    20    16 */
	__u32                      remote_port;          /*    36     4 */
	__u32                      local_ip4;            /*    40     4 */
	__u32                      local_ip6[4];         /*    44    16 */
	__u32                      local_port;           /*    60     4 */

	/* size: 64, cachelines: 1, members: 9 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(8)));
struct btf {
	void *                     data;                 /*     0     8 */
	struct btf_type * *        types;                /*     8     8 */
	u32 *                      resolved_ids;         /*    16     8 */
	u32 *                      resolved_sizes;       /*    24     8 */
	const char  *              strings;              /*    32     8 */
	void *                     nohdr_data;           /*    40     8 */
	struct btf_header          hdr;                  /*    48    24 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	u32                        nr_types;             /*    72     4 */
	u32                        types_size;           /*    76     4 */
	u32                        data_size;            /*    80     4 */
	refcount_t                 refcnt;               /*    84     4 */
	u32                        id;                   /*    88     4 */

	/* XXX 4 bytes hole, try to pack */

	struct callback_head       rcu __attribute__((__aligned__(8))); /*    96    16 */

	/* size: 112, cachelines: 2, members: 13 */
	/* sum members: 108, holes: 1, sum holes: 4 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 4 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct sk_reuseport_kern {
	struct sk_buff *           skb;                  /*     0     8 */
	struct sock *              sk;                   /*     8     8 */
	struct sock *              selected_sk;          /*    16     8 */
	void *                     data_end;             /*    24     8 */
	u32                        hash;                 /*    32     4 */
	u32                        reuseport_id;         /*    36     4 */
	bool                       bind_inany;           /*    40     1 */

	/* size: 48, cachelines: 1, members: 7 */
	/* padding: 7 */
	/* last cacheline: 48 bytes */
};
struct bpf_flow_dissector {
	struct bpf_flow_keys *     flow_keys;            /*     0     8 */
	const struct sk_buff  *    skb;                  /*     8     8 */
	void *                     data;                 /*    16     8 */
	void *                     data_end;             /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct inet_hashinfo {
	struct inet_ehash_bucket * ehash;                /*     0     8 */
	spinlock_t *               ehash_locks;          /*     8     8 */
	unsigned int               ehash_mask;           /*    16     4 */
	unsigned int               ehash_locks_mask;     /*    20     4 */
	struct kmem_cache *        bind_bucket_cachep;   /*    24     8 */
	struct inet_bind_hashbucket * bhash;             /*    32     8 */
	unsigned int               bhash_size;           /*    40     4 */
	unsigned int               lhash2_mask;          /*    44     4 */
	struct inet_listen_hashbucket * lhash2;          /*    48     8 */

	/* XXX 8 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	struct inet_listen_hashbucket listening_hash[32] __attribute__((__aligned__(64))); /*    64   512 */

	/* size: 576, cachelines: 9, members: 10 */
	/* sum members: 568, holes: 1, sum holes: 8 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 8 */
} __attribute__((__aligned__(64)));
struct ip_ra_chain {
	struct ip_ra_chain *       next;                 /*     0     8 */
	struct sock *              sk;                   /*     8     8 */
	union {
		void               (*destructor)(struct sock *); /*    16     8 */
		struct sock *      saved_sk;             /*    16     8 */
	};                                               /*    16     8 */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*    24    16 */

	/* size: 40, cachelines: 1, members: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 40 bytes */
} __attribute__((__aligned__(8)));
struct fib_table {
	struct hlist_node          tb_hlist;             /*     0    16 */
	u32                        tb_id;                /*    16     4 */
	int                        tb_num_default;       /*    20     4 */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*    24    16 */
	long unsigned int *        tb_data;              /*    40     8 */
	long unsigned int          __data[];             /*    48     0 */

	/* size: 48, cachelines: 1, members: 6 */
	/* forced alignments: 1 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct inet_peer_base {
	struct rb_root             rb_root;              /*     0     8 */
	seqlock_t                  lock;                 /*     8     8 */
	int                        total;                /*    16     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct tcp_congestion_ops {
	struct list_head           list;                 /*     0    16 */
	u32                        key;                  /*    16     4 */
	u32                        flags;                /*    20     4 */
	void                       (*init)(struct sock *); /*    24     8 */
	void                       (*release)(struct sock *); /*    32     8 */
	u32                        (*ssthresh)(struct sock *); /*    40     8 */
	void                       (*cong_avoid)(struct sock *, u32, u32); /*    48     8 */
	void                       (*set_state)(struct sock *, u8); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	void                       (*cwnd_event)(struct sock *, enum tcp_ca_event); /*    64     8 */
	void                       (*in_ack_event)(struct sock *, u32); /*    72     8 */
	u32                        (*undo_cwnd)(struct sock *); /*    80     8 */
	void                       (*pkts_acked)(struct sock *, const struct ack_sample  *); /*    88     8 */
	u32                        (*min_tso_segs)(struct sock *); /*    96     8 */
	u32                        (*sndbuf_expand)(struct sock *); /*   104     8 */
	void                       (*cong_control)(struct sock *, const struct rate_sample  *); /*   112     8 */
	size_t                     (*get_info)(struct sock *, u32, int *, union tcp_cc_info *); /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	char                       name[16];             /*   128    16 */
	struct module *            owner;                /*   144     8 */
	u64                        kabi_reserved1;       /*   152     8 */
	u64                        kabi_reserved2;       /*   160     8 */
	u64                        kabi_reserved3;       /*   168     8 */
	u64                        kabi_reserved4;       /*   176     8 */

	/* size: 184, cachelines: 3, members: 22 */
	/* last cacheline: 56 bytes */
};
struct tcp_fastopen_context {
	siphash_key_t              key[2];               /*     0    32 */
	int                        num;                  /*    32     4 */

	/* XXX 4 bytes hole, try to pack */

	struct callback_head       rcu __attribute__((__aligned__(8))); /*    40    16 */

	/* size: 56, cachelines: 1, members: 3 */
	/* sum members: 52, holes: 1, sum holes: 4 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 4 */
	/* last cacheline: 56 bytes */
} __attribute__((__aligned__(8)));
struct xdp_txq_info {
	struct net_device *        dev;                  /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct xdp_buff {
	void *                     data;                 /*     0     8 */
	void *                     data_end;             /*     8     8 */
	void *                     data_meta;            /*    16     8 */
	void *                     data_hard_start;      /*    24     8 */
	struct xdp_rxq_info *      rxq;                  /*    32     8 */
	struct xdp_txq_info *      txq;                  /*    40     8 */
	u32                        frame_sz;             /*    48     4 */

	/* size: 56, cachelines: 1, members: 7 */
	/* padding: 4 */
	/* last cacheline: 56 bytes */
};
struct bpf_sock_addr_kern {
	struct sock *              sk;                   /*     0     8 */
	struct sockaddr *          uaddr;                /*     8     8 */
	u64                        tmp_reg;              /*    16     8 */
	void *                     t_ctx;                /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct bpf_sock_ops_kern {
	struct sock *              sk;                   /*     0     8 */
	union {
		u32                args[4];              /*     8    16 */
		u32                reply;                /*     8     4 */
		u32                replylong[4];         /*     8    16 */
	};                                               /*     8    16 */
	struct sk_buff *           syn_skb;              /*    24     8 */
	struct sk_buff *           skb;                  /*    32     8 */
	void *                     skb_data_end;         /*    40     8 */
	u8                         op;                   /*    48     1 */
	u8                         is_fullsock;          /*    49     1 */
	u8                         remaining_opt_len;    /*    50     1 */

	/* XXX 5 bytes hole, try to pack */

	u64                        temp;                 /*    56     8 */

	/* size: 64, cachelines: 1, members: 9 */
	/* sum members: 59, holes: 1, sum holes: 5 */
};
struct bpf_sysctl_kern {
	struct ctl_table_header *  head;                 /*     0     8 */
	struct ctl_table *         table;                /*     8     8 */
	void *                     cur_val;              /*    16     8 */
	size_t                     cur_len;              /*    24     8 */
	void *                     new_val;              /*    32     8 */
	size_t                     new_len;              /*    40     8 */
	int                        new_updated;          /*    48     4 */
	int                        write;                /*    52     4 */
	loff_t *                   ppos;                 /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u64                        tmp_reg;              /*    64     8 */

	/* size: 72, cachelines: 2, members: 10 */
	/* last cacheline: 8 bytes */
};
struct bpf_sockopt_kern {
	struct sock *              sk;                   /*     0     8 */
	u8 *                       optval;               /*     8     8 */
	u8 *                       optval_end;           /*    16     8 */
	s32                        level;                /*    24     4 */
	s32                        optname;              /*    28     4 */
	s32                        optlen;               /*    32     4 */
	s32                        retval;               /*    36     4 */

	/* size: 40, cachelines: 1, members: 7 */
	/* last cacheline: 40 bytes */
};
struct bpf_sk_lookup_kern {
	u16                        family;               /*     0     2 */
	u16                        protocol;             /*     2     2 */
	__be16                     sport;                /*     4     2 */
	u16                        dport;                /*     6     2 */
	struct {
		__be32             saddr;                /*     8     4 */
		__be32             daddr;                /*    12     4 */
	} v4;                                            /*     8     8 */
	struct {
		const struct in6_addr  * saddr;          /*    16     8 */
		const struct in6_addr  * daddr;          /*    24     8 */
	} v6;                                            /*    16    16 */
	struct sock *              selected_sk;          /*    32     8 */
	bool                       no_reuseport;         /*    40     1 */

	/* size: 48, cachelines: 1, members: 8 */
	/* padding: 7 */
	/* last cacheline: 48 bytes */
};
struct lwtunnel_state {
	__u16                      type;                 /*     0     2 */
	__u16                      flags;                /*     2     2 */
	__u16                      headroom;             /*     4     2 */

	/* XXX 2 bytes hole, try to pack */

	atomic_t                   refcnt;               /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	int                        (*orig_output)(struct net *, struct sock *, struct sk_buff *); /*    16     8 */
	int                        (*orig_input)(struct sk_buff *); /*    24     8 */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*    32    16 */
	u64                        kabi_reserved1;       /*    48     8 */
	u64                        kabi_reserved2;       /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u64                        kabi_reserved3;       /*    64     8 */
	u64                        kabi_reserved4;       /*    72     8 */
	__u8                       data[];               /*    80     0 */

	/* size: 80, cachelines: 2, members: 12 */
	/* sum members: 74, holes: 2, sum holes: 6 */
	/* forced alignments: 1 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct sock_reuseport {
	struct callback_head       rcu __attribute__((__aligned__(8))); /*     0    16 */
	u16                        max_socks;            /*    16     2 */
	u16                        num_socks;            /*    18     2 */
	unsigned int               synq_overflow_ts;     /*    20     4 */
	unsigned int               reuseport_id;         /*    24     4 */
	unsigned int               bind_inany:1;         /*    28: 0  4 */
	unsigned int               has_conns:1;          /*    28: 1  4 */

	/* XXX 30 bits hole, try to pack */

	struct bpf_prog *          prog;                 /*    32     8 */
	struct sock *              socks[];              /*    40     0 */

	/* size: 40, cachelines: 1, members: 9 */
	/* sum members: 36 */
	/* sum bitfield members: 2 bits, bit holes: 1, sum bit holes: 30 bits */
	/* forced alignments: 1 */
	/* last cacheline: 40 bytes */
} __attribute__((__aligned__(8)));
struct inet_ehash_bucket {
	struct hlist_nulls_head    chain;                /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct inet_bind_hashbucket {
	spinlock_t                 lock;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct hlist_head          chain;                /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct inet_listen_hashbucket {
	spinlock_t                 lock;                 /*     0     4 */
	unsigned int               count;                /*     4     4 */
	union {
		struct hlist_head  head;                 /*     8     8 */
		struct hlist_nulls_head nulls_head;      /*     8     8 */
	};                                               /*     8     8 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct ack_sample {
	u32                        pkts_acked;           /*     0     4 */
	s32                        rtt_us;               /*     4     4 */
	u32                        in_flight;            /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct rate_sample {
	u64                        prior_mstamp;         /*     0     8 */
	u32                        prior_delivered;      /*     8     4 */
	s32                        delivered;            /*    12     4 */
	long int                   interval_us;          /*    16     8 */
	u32                        snd_interval_us;      /*    24     4 */
	u32                        rcv_interval_us;      /*    28     4 */
	long int                   rtt_us;               /*    32     8 */
	int                        losses;               /*    40     4 */
	u32                        acked_sacked;         /*    44     4 */
	u32                        prior_in_flight;      /*    48     4 */
	bool                       is_app_limited;       /*    52     1 */
	bool                       is_retrans;           /*    53     1 */
	bool                       is_ack_delayed;       /*    54     1 */

	/* size: 56, cachelines: 1, members: 13 */
	/* padding: 1 */
	/* last cacheline: 56 bytes */
};
struct sk_msg_sg {
	u32                        start;                /*     0     4 */
	u32                        curr;                 /*     4     4 */
	u32                        end;                  /*     8     4 */
	u32                        size;                 /*    12     4 */
	u32                        copybreak;            /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          copy;                 /*    24     8 */
	struct scatterlist         data[19];             /*    32   608 */

	/* size: 640, cachelines: 10, members: 7 */
	/* sum members: 636, holes: 1, sum holes: 4 */
};
struct sk_msg {
	struct sk_msg_sg           sg;                   /*     0   640 */
	/* --- cacheline 10 boundary (640 bytes) --- */
	void *                     data;                 /*   640     8 */
	void *                     data_end;             /*   648     8 */
	u32                        apply_bytes;          /*   656     4 */
	u32                        cork_bytes;           /*   660     4 */
	u32                        flags;                /*   664     4 */

	/* XXX 4 bytes hole, try to pack */

	struct sk_buff *           skb;                  /*   672     8 */
	struct sock *              sk_redir;             /*   680     8 */
	struct sock *              sk;                   /*   688     8 */
	struct list_head           list;                 /*   696    16 */

	/* size: 712, cachelines: 12, members: 10 */
	/* sum members: 708, holes: 1, sum holes: 4 */
	/* last cacheline: 8 bytes */
};
struct resolve_vertex {
	const struct btf_type  *   t;                    /*     0     8 */
	u32                        type_id;              /*     8     4 */
	u16                        next_member;          /*    12     2 */

	/* size: 16, cachelines: 1, members: 3 */
	/* padding: 2 */
	/* last cacheline: 16 bytes */
};
struct btf_sec_info {
	u32                        off;                  /*     0     4 */
	u32                        len;                  /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct btf_verifier_env {
	struct btf *               btf;                  /*     0     8 */
	u8 *                       visit_states;         /*     8     8 */
	struct resolve_vertex      stack[32];            /*    16   512 */
	/* --- cacheline 8 boundary (512 bytes) was 16 bytes ago --- */
	struct bpf_verifier_log    log;                  /*   528  1048 */
	/* --- cacheline 24 boundary (1536 bytes) was 40 bytes ago --- */
	u32                        log_type_id;          /*  1576     4 */
	u32                        top_stack;            /*  1580     4 */
	enum verifier_phase        phase;                /*  1584     4 */
	enum resolve_mode          resolve_mode;         /*  1588     4 */

	/* size: 1592, cachelines: 25, members: 8 */
	/* last cacheline: 56 bytes */
};
struct btf_show {
	u64                        flags;                /*     0     8 */
	void *                     target;               /*     8     8 */
	void                       (*showfn)(struct btf_show *, const char  *, struct __va_list_tag *); /*    16     8 */
	const struct btf  *        btf;                  /*    24     8 */
	struct {
		u8                 depth;                /*    32     1 */
		u8                 depth_to_show;        /*    33     1 */
		u8                 depth_check;          /*    34     1 */
		u8                 array_member:1;       /*    35: 0  1 */
		u8                 array_terminated:1;   /*    35: 1  1 */

		/* XXX 6 bits hole, try to pack */

		u16                array_encoding;       /*    36     2 */

		/* XXX 2 bytes hole, try to pack */

		u32                type_id;              /*    40     4 */
		int                status;               /*    44     4 */
		const struct btf_type  * type;           /*    48     8 */
		const struct btf_member  * member;       /*    56     8 */
		/* --- cacheline 1 boundary (64 bytes) --- */
		char               name[80];             /*    64    80 */
	} state;                                         /*    32   112 */
	/* --- cacheline 2 boundary (128 bytes) was 16 bytes ago --- */
	struct {
		u32                size;                 /*   144     4 */

		/* XXX 4 bytes hole, try to pack */

		void *             head;                 /*   152     8 */
		void *             data;                 /*   160     8 */
		u8                 safe[32];             /*   168    32 */
	} obj;                                           /*   144    56 */

	/* size: 200, cachelines: 4, members: 6 */
	/* last cacheline: 8 bytes */
};
struct btf_kind_operations {
	s32                        (*check_meta)(struct btf_verifier_env *, const struct btf_type  *, u32); /*     0     8 */
	int                        (*resolve)(struct btf_verifier_env *, const struct resolve_vertex  *); /*     8     8 */
	int                        (*check_member)(struct btf_verifier_env *, const struct btf_type  *, const struct btf_member  *, const struct btf_type  *); /*    16     8 */
	int                        (*check_kflag_member)(struct btf_verifier_env *, const struct btf_type  *, const struct btf_member  *, const struct btf_type  *); /*    24     8 */
	void                       (*log_details)(struct btf_verifier_env *, const struct btf_type  *); /*    32     8 */
	void                       (*show)(const struct btf  *, const struct btf_type  *, u32, void *, u8, struct btf_show *); /*    40     8 */

	/* size: 48, cachelines: 1, members: 6 */
	/* last cacheline: 48 bytes */
};
struct bpf_ctx_convert {
	struct __sk_buff           BPF_PROG_TYPE_SOCKET_FILTER_prog __attribute__((__aligned__(8))); /*     0   184 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 2 boundary (128 bytes) was 56 bytes ago --- */
	struct sk_buff             BPF_PROG_TYPE_SOCKET_FILTER_kern __attribute__((__aligned__(8))); /*   184   256 */
	/* --- cacheline 6 boundary (384 bytes) was 56 bytes ago --- */
	struct __sk_buff           BPF_PROG_TYPE_SCHED_CLS_prog __attribute__((__aligned__(8))); /*   440   184 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 9 boundary (576 bytes) was 48 bytes ago --- */
	struct sk_buff             BPF_PROG_TYPE_SCHED_CLS_kern __attribute__((__aligned__(8))); /*   624   256 */
	/* --- cacheline 13 boundary (832 bytes) was 48 bytes ago --- */
	struct __sk_buff           BPF_PROG_TYPE_SCHED_ACT_prog __attribute__((__aligned__(8))); /*   880   184 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 16 boundary (1024 bytes) was 40 bytes ago --- */
	struct sk_buff             BPF_PROG_TYPE_SCHED_ACT_kern __attribute__((__aligned__(8))); /*  1064   256 */
	/* --- cacheline 20 boundary (1280 bytes) was 40 bytes ago --- */
	struct xdp_md              BPF_PROG_TYPE_XDP_prog; /*  1320    24 */
	/* --- cacheline 21 boundary (1344 bytes) --- */
	struct xdp_buff            BPF_PROG_TYPE_XDP_kern; /*  1344    56 */

	/* XXX last struct has 4 bytes of padding */

	struct __sk_buff           BPF_PROG_TYPE_CGROUP_SKB_prog __attribute__((__aligned__(8))); /*  1400   184 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 24 boundary (1536 bytes) was 48 bytes ago --- */
	struct sk_buff             BPF_PROG_TYPE_CGROUP_SKB_kern __attribute__((__aligned__(8))); /*  1584   256 */
	/* --- cacheline 28 boundary (1792 bytes) was 48 bytes ago --- */
	struct bpf_sock            BPF_PROG_TYPE_CGROUP_SOCK_prog; /*  1840    80 */
	/* --- cacheline 30 boundary (1920 bytes) --- */
	struct sock                BPF_PROG_TYPE_CGROUP_SOCK_kern __attribute__((__aligned__(8))); /*  1920   872 */
	/* --- cacheline 43 boundary (2752 bytes) was 40 bytes ago --- */
	struct bpf_sock_addr       BPF_PROG_TYPE_CGROUP_SOCK_ADDR_prog __attribute__((__aligned__(8))); /*  2792    72 */
	/* --- cacheline 44 boundary (2816 bytes) was 48 bytes ago --- */
	struct bpf_sock_addr_kern  BPF_PROG_TYPE_CGROUP_SOCK_ADDR_kern; /*  2864    32 */
	/* --- cacheline 45 boundary (2880 bytes) was 16 bytes ago --- */
	struct __sk_buff           BPF_PROG_TYPE_LWT_IN_prog __attribute__((__aligned__(8))); /*  2896   184 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 48 boundary (3072 bytes) was 8 bytes ago --- */
	struct sk_buff             BPF_PROG_TYPE_LWT_IN_kern __attribute__((__aligned__(8))); /*  3080   256 */
	/* --- cacheline 52 boundary (3328 bytes) was 8 bytes ago --- */
	struct __sk_buff           BPF_PROG_TYPE_LWT_OUT_prog __attribute__((__aligned__(8))); /*  3336   184 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 55 boundary (3520 bytes) --- */
	struct sk_buff             BPF_PROG_TYPE_LWT_OUT_kern __attribute__((__aligned__(8))); /*  3520   256 */
	/* --- cacheline 59 boundary (3776 bytes) --- */
	struct __sk_buff           BPF_PROG_TYPE_LWT_XMIT_prog __attribute__((__aligned__(8))); /*  3776   184 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 61 boundary (3904 bytes) was 56 bytes ago --- */
	struct sk_buff             BPF_PROG_TYPE_LWT_XMIT_kern __attribute__((__aligned__(8))); /*  3960   256 */
	/* --- cacheline 65 boundary (4160 bytes) was 56 bytes ago --- */
	struct __sk_buff           BPF_PROG_TYPE_LWT_SEG6LOCAL_prog __attribute__((__aligned__(8))); /*  4216   184 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 68 boundary (4352 bytes) was 48 bytes ago --- */
	struct sk_buff             BPF_PROG_TYPE_LWT_SEG6LOCAL_kern __attribute__((__aligned__(8))); /*  4400   256 */
	/* --- cacheline 72 boundary (4608 bytes) was 48 bytes ago --- */
	struct bpf_sock_ops        BPF_PROG_TYPE_SOCK_OPS_prog __attribute__((__aligned__(8))); /*  4656   216 */
	/* --- cacheline 76 boundary (4864 bytes) was 8 bytes ago --- */
	struct bpf_sock_ops_kern   BPF_PROG_TYPE_SOCK_OPS_kern; /*  4872    64 */
	/* --- cacheline 77 boundary (4928 bytes) was 8 bytes ago --- */
	struct __sk_buff           BPF_PROG_TYPE_SK_SKB_prog __attribute__((__aligned__(8))); /*  4936   184 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 80 boundary (5120 bytes) --- */
	struct sk_buff             BPF_PROG_TYPE_SK_SKB_kern __attribute__((__aligned__(8))); /*  5120   256 */
	/* --- cacheline 84 boundary (5376 bytes) --- */
	struct sk_msg_md           BPF_PROG_TYPE_SK_MSG_prog __attribute__((__aligned__(8))); /*  5376    80 */
	/* --- cacheline 85 boundary (5440 bytes) was 16 bytes ago --- */
	struct sk_msg              BPF_PROG_TYPE_SK_MSG_kern; /*  5456   712 */
	/* --- cacheline 96 boundary (6144 bytes) was 24 bytes ago --- */
	struct __sk_buff           BPF_PROG_TYPE_FLOW_DISSECTOR_prog __attribute__((__aligned__(8))); /*  6168   184 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 99 boundary (6336 bytes) was 16 bytes ago --- */
	struct bpf_flow_dissector  BPF_PROG_TYPE_FLOW_DISSECTOR_kern; /*  6352    32 */
	bpf_user_pt_regs_t         BPF_PROG_TYPE_KPROBE_prog; /*  6384   168 */
	/* --- cacheline 102 boundary (6528 bytes) was 24 bytes ago --- */
	struct pt_regs             BPF_PROG_TYPE_KPROBE_kern; /*  6552   168 */
	/* --- cacheline 105 boundary (6720 bytes) --- */
	__u64                      BPF_PROG_TYPE_TRACEPOINT_prog; /*  6720     8 */
	u64                        BPF_PROG_TYPE_TRACEPOINT_kern; /*  6728     8 */
	struct bpf_perf_event_data BPF_PROG_TYPE_PERF_EVENT_prog; /*  6736   184 */
	/* --- cacheline 108 boundary (6912 bytes) was 8 bytes ago --- */
	struct bpf_perf_event_data_kern BPF_PROG_TYPE_PERF_EVENT_kern; /*  6920    24 */
	struct bpf_raw_tracepoint_args BPF_PROG_TYPE_RAW_TRACEPOINT_prog; /*  6944     0 */
	u64                        BPF_PROG_TYPE_RAW_TRACEPOINT_kern; /*  6944     8 */
	struct bpf_raw_tracepoint_args BPF_PROG_TYPE_RAW_TRACEPOINT_WRITABLE_prog; /*  6952     0 */
	u64                        BPF_PROG_TYPE_RAW_TRACEPOINT_WRITABLE_kern; /*  6952     8 */
	void *                     BPF_PROG_TYPE_TRACING_prog; /*  6960     8 */
	void *                     BPF_PROG_TYPE_TRACING_kern; /*  6968     8 */
	/* --- cacheline 109 boundary (6976 bytes) --- */
	struct bpf_cgroup_dev_ctx  BPF_PROG_TYPE_CGROUP_DEVICE_prog; /*  6976    12 */
	struct bpf_cgroup_dev_ctx  BPF_PROG_TYPE_CGROUP_DEVICE_kern; /*  6988    12 */
	struct bpf_sysctl          BPF_PROG_TYPE_CGROUP_SYSCTL_prog; /*  7000     8 */
	struct bpf_sysctl_kern     BPF_PROG_TYPE_CGROUP_SYSCTL_kern; /*  7008    72 */
	/* --- cacheline 110 boundary (7040 bytes) was 40 bytes ago --- */
	struct bpf_sockopt         BPF_PROG_TYPE_CGROUP_SOCKOPT_prog __attribute__((__aligned__(8))); /*  7080    40 */
	/* --- cacheline 111 boundary (7104 bytes) was 16 bytes ago --- */
	struct bpf_sockopt_kern    BPF_PROG_TYPE_CGROUP_SOCKOPT_kern; /*  7120    40 */
	struct sk_reuseport_md     BPF_PROG_TYPE_SK_REUSEPORT_prog __attribute__((__aligned__(8))); /*  7160    40 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 112 boundary (7168 bytes) was 32 bytes ago --- */
	struct sk_reuseport_kern   BPF_PROG_TYPE_SK_REUSEPORT_kern; /*  7200    48 */

	/* XXX last struct has 7 bytes of padding */

	/* --- cacheline 113 boundary (7232 bytes) was 16 bytes ago --- */
	struct bpf_sk_lookup       BPF_PROG_TYPE_SK_LOOKUP_prog __attribute__((__aligned__(8))); /*  7248    64 */
	/* --- cacheline 114 boundary (7296 bytes) was 16 bytes ago --- */
	struct bpf_sk_lookup_kern  BPF_PROG_TYPE_SK_LOOKUP_kern; /*  7312    48 */

	/* XXX last struct has 7 bytes of padding */

	/* --- cacheline 115 boundary (7360 bytes) --- */
	void *                     BPF_PROG_TYPE_STRUCT_OPS_prog; /*  7360     8 */
	void *                     BPF_PROG_TYPE_STRUCT_OPS_kern; /*  7368     8 */
	void *                     BPF_PROG_TYPE_EXT_prog; /*  7376     8 */
	void *                     BPF_PROG_TYPE_EXT_kern; /*  7384     8 */
	void *                     BPF_PROG_TYPE_SCHED_prog; /*  7392     8 */
	void *                     BPF_PROG_TYPE_SCHED_kern; /*  7400     8 */

	/* size: 7408, cachelines: 116, members: 58 */
	/* paddings: 14, sum paddings: 62 */
	/* forced alignments: 26 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct btf_show_snprintf {
	struct btf_show            show;                 /*     0   200 */
	/* --- cacheline 3 boundary (192 bytes) was 8 bytes ago --- */
	int                        len_left;             /*   200     4 */
	int                        len;                  /*   204     4 */

	/* size: 208, cachelines: 4, members: 3 */
	/* last cacheline: 16 bytes */
};
struct bpf_dispatcher_prog {
	struct bpf_prog *          prog;                 /*     0     8 */
	refcount_t                 users;                /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct bpf_dispatcher {
	struct mutex               mutex;                /*     0    32 */
	void *                     func;                 /*    32     8 */
	struct bpf_dispatcher_prog progs[48];            /*    40   768 */
	/* --- cacheline 12 boundary (768 bytes) was 40 bytes ago --- */
	int                        num_progs;            /*   808     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     image;                /*   816     8 */
	u32                        image_off;            /*   824     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 13 boundary (832 bytes) --- */
	struct bpf_ksym            ksym __attribute__((__aligned__(8))); /*   832   216 */

	/* XXX last struct has 7 bytes of padding */

	/* size: 1048, cachelines: 17, members: 7 */
	/* sum members: 1040, holes: 2, sum holes: 8 */
	/* paddings: 1, sum paddings: 7 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 4 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct bpf_devmap_val {
	__u32                      ifindex;              /*     0     4 */
	union {
		int                fd;                   /*     4     4 */
		__u32              id;                   /*     4     4 */
	} bpf_prog;                                      /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct xdp_dev_bulk_queue {
	struct xdp_frame *         q[16];                /*     0   128 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct list_head           flush_node;           /*   128    16 */
	struct net_device *        dev;                  /*   144     8 */
	struct net_device *        dev_rx;               /*   152     8 */
	unsigned int               count;                /*   160     4 */

	/* size: 168, cachelines: 3, members: 5 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct netdev_notifier_info {
	struct net_device *        dev;                  /*     0     8 */
	struct netlink_ext_ack *   extack;               /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct bpf_dtab_netdev {
	struct net_device *        dev;                  /*     0     8 */
	struct hlist_node          index_hlist;          /*     8    16 */
	struct bpf_dtab *          dtab;                 /*    24     8 */
	struct bpf_prog *          xdp_prog;             /*    32     8 */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*    40    16 */
	unsigned int               idx;                  /*    56     4 */
	struct bpf_devmap_val      val;                  /*    60     8 */

	/* size: 72, cachelines: 2, members: 7 */
	/* padding: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 8 bytes */
} __attribute__((__aligned__(8)));
struct bpf_dtab {
	struct bpf_map             map __attribute__((__aligned__(64))); /*     0   256 */

	/* XXX last struct has 8 bytes of padding */

	/* --- cacheline 4 boundary (256 bytes) --- */
	struct bpf_dtab_netdev * * netdev_map;           /*   256     8 */
	struct list_head           list;                 /*   264    16 */
	struct hlist_head *        dev_index_head;       /*   280     8 */
	spinlock_t                 index_lock;           /*   288     4 */
	unsigned int               items;                /*   292     4 */
	u32                        n_buckets;            /*   296     4 */

	/* size: 320, cachelines: 5, members: 7 */
	/* padding: 20 */
	/* paddings: 1, sum paddings: 8 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(64)));
struct bpf_cpumap_val {
	__u32                      qsize;                /*     0     4 */
	union {
		int                fd;                   /*     4     4 */
		__u32              id;                   /*     4     4 */
	} bpf_prog;                                      /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct skb_shared_hwtstamps {
	ktime_t                    hwtstamp;             /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct skb_shared_info {
	__u8                       __unused;             /*     0     1 */
	__u8                       meta_len;             /*     1     1 */
	__u8                       nr_frags;             /*     2     1 */
	__u8                       tx_flags;             /*     3     1 */
	short unsigned int         gso_size;             /*     4     2 */
	short unsigned int         gso_segs;             /*     6     2 */
	struct sk_buff *           frag_list;            /*     8     8 */
	struct skb_shared_hwtstamps hwtstamps;           /*    16     8 */
	unsigned int               gso_type;             /*    24     4 */
	u32                        tskey;                /*    28     4 */
	atomic_t                   dataref;              /*    32     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     destructor_arg;       /*    40     8 */
	skb_frag_t                 frags[17];            /*    48   272 */

	/* size: 320, cachelines: 5, members: 13 */
	/* sum members: 316, holes: 1, sum holes: 4 */
};
struct bpf_nh_params {
	u32                        nh_family;            /*     0     4 */
	union {
		u32                ipv4_nh;              /*     4     4 */
		struct in6_addr    ipv6_nh;              /*     4    16 */
	};                                               /*     4    16 */

	/* size: 20, cachelines: 1, members: 2 */
	/* last cacheline: 20 bytes */
};
struct bpf_redirect_info {
	u32                        flags;                /*     0     4 */
	u32                        tgt_index;            /*     4     4 */
	void *                     tgt_value;            /*     8     8 */
	struct bpf_map *           map;                  /*    16     8 */
	u32                        kern_flags;           /*    24     4 */
	struct bpf_nh_params       nh;                   /*    28    20 */

	/* size: 48, cachelines: 1, members: 6 */
	/* last cacheline: 48 bytes */
};
struct xdp_bulk_queue {
	void *                     q[8];                 /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct list_head           flush_node;           /*    64    16 */
	struct bpf_cpu_map_entry * obj;                  /*    80     8 */
	unsigned int               count;                /*    88     4 */

	/* size: 96, cachelines: 2, members: 4 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct bpf_cpu_map_entry {
	u32                        cpu;                  /*     0     4 */
	int                        map_id;               /*     4     4 */
	struct xdp_bulk_queue *    bulkq;                /*     8     8 */
	struct bpf_cpu_map *       cmap;                 /*    16     8 */
	struct ptr_ring *          queue;                /*    24     8 */
	struct task_struct *       kthread;              /*    32     8 */
	struct bpf_cpumap_val      value;                /*    40     8 */
	struct bpf_prog *          prog;                 /*    48     8 */
	atomic_t                   refcnt;               /*    56     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*    64    16 */
	struct work_struct         kthread_stop_wq;      /*    80    64 */

	/* size: 144, cachelines: 3, members: 11 */
	/* sum members: 140, holes: 1, sum holes: 4 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 4 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct bpf_cpu_map {
	struct bpf_map             map __attribute__((__aligned__(64))); /*     0   256 */

	/* XXX last struct has 8 bytes of padding */

	/* --- cacheline 4 boundary (256 bytes) --- */
	struct bpf_cpu_map_entry * * cpu_map;            /*   256     8 */

	/* size: 320, cachelines: 5, members: 2 */
	/* padding: 56 */
	/* paddings: 1, sum paddings: 8 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(64)));
struct bpf_local_storage_map {
	struct bpf_map             map __attribute__((__aligned__(64))); /*     0   256 */

	/* XXX last struct has 8 bytes of padding */

	/* --- cacheline 4 boundary (256 bytes) --- */
	struct bpf_local_storage_map_bucket * buckets;   /*   256     8 */
	u32                        bucket_log;           /*   264     4 */
	u16                        elem_size;            /*   268     2 */
	u16                        cache_idx;            /*   270     2 */

	/* size: 320, cachelines: 5, members: 5 */
	/* padding: 48 */
	/* paddings: 1, sum paddings: 8 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(64)));
struct bpf_local_storage {
	struct bpf_local_storage_data * cache[16];       /*     0   128 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct hlist_head          list;                 /*   128     8 */
	void *                     owner;                /*   136     8 */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*   144    16 */
	raw_spinlock_t             lock;                 /*   160     4 */

	/* size: 168, cachelines: 3, members: 5 */
	/* padding: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 40 bytes */
} __attribute__((__aligned__(8)));
struct bpf_local_storage_map_bucket {
	struct hlist_head          list;                 /*     0     8 */
	raw_spinlock_t             lock;                 /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct bpf_local_storage_data {
	struct bpf_local_storage_map * smap;             /*     0     8 */
	u8                         data[] __attribute__((__aligned__(8))); /*     8     0 */

	/* size: 8, cachelines: 1, members: 2 */
	/* forced alignments: 1 */
	/* last cacheline: 8 bytes */
} __attribute__((__aligned__(8)));
struct bpf_local_storage_elem {
	struct hlist_node          map_node;             /*     0    16 */
	struct hlist_node          snode;                /*    16    16 */
	struct bpf_local_storage * local_storage;        /*    32     8 */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*    40    16 */

	/* XXX 8 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	struct bpf_local_storage_data sdata __attribute__((__aligned__(64))); /*    64     8 */

	/* size: 128, cachelines: 2, members: 5 */
	/* sum members: 64, holes: 1, sum holes: 8 */
	/* padding: 56 */
	/* forced alignments: 2, forced holes: 1, sum forced holes: 8 */
} __attribute__((__aligned__(64)));
struct bpf_local_storage_cache {
	spinlock_t                 idx_lock;             /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        idx_usage_counts[16]; /*     8   128 */

	/* size: 136, cachelines: 3, members: 2 */
	/* sum members: 132, holes: 1, sum holes: 4 */
	/* last cacheline: 8 bytes */
};
struct rhlist_head {
	struct rhash_head          rhead;                /*     0     8 */
	struct rhlist_head *       next;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct bpf_prog_offload_ops {
	int                        (*insn_hook)(struct bpf_verifier_env *, int, int); /*     0     8 */
	int                        (*finalize)(struct bpf_verifier_env *); /*     8     8 */
	int                        (*replace_insn)(struct bpf_verifier_env *, u32, struct bpf_insn *); /*    16     8 */
	int                        (*remove_insns)(struct bpf_verifier_env *, u32, u32); /*    24     8 */
	int                        (*prepare)(struct bpf_prog *); /*    32     8 */
	int                        (*translate)(struct bpf_prog *); /*    40     8 */
	void                       (*destroy)(struct bpf_prog *); /*    48     8 */

	/* size: 56, cachelines: 1, members: 7 */
	/* last cacheline: 56 bytes */
};
struct bpf_offload_dev {
	const struct bpf_prog_offload_ops  * ops;        /*     0     8 */
	struct list_head           netdevs;              /*     8    16 */
	void *                     priv;                 /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* last cacheline: 32 bytes */
};
struct bpf_offload_netdev {
	struct rhash_head          l;                    /*     0     8 */
	struct net_device *        netdev;               /*     8     8 */
	struct bpf_offload_dev *   offdev;               /*    16     8 */
	struct list_head           progs;                /*    24    16 */
	struct list_head           maps;                 /*    40    16 */
	struct list_head           offdev_netdevs;       /*    56    16 */

	/* size: 72, cachelines: 2, members: 6 */
	/* last cacheline: 8 bytes */
};
struct ns_get_path_bpf_prog_args {
	struct bpf_prog *          prog;                 /*     0     8 */
	struct bpf_prog_info *     info;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct ns_get_path_bpf_map_args {
	struct bpf_offloaded_map * offmap;               /*     0     8 */
	struct bpf_map_info *      info;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct bpf_netns_link {
	struct bpf_link            link;                 /*     0    96 */
	/* --- cacheline 1 boundary (64 bytes) was 32 bytes ago --- */
	enum bpf_attach_type       type;                 /*    96     4 */
	enum netns_bpf_attach_type netns_type;           /*   100     4 */
	struct net *               net;                  /*   104     8 */
	struct list_head           node;                 /*   112    16 */

	/* size: 128, cachelines: 2, members: 5 */
};
struct bpf_stack_build_id {
	__s32                      status;               /*     0     4 */
	unsigned char              build_id[20];         /*     4    20 */
	union {
		__u64              offset;               /*    24     8 */
		__u64              ip;                   /*    24     8 */
	};                                               /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* last cacheline: 32 bytes */
};
struct elf32_hdr {
	unsigned char              e_ident[16];          /*     0    16 */
	Elf32_Half                 e_type;               /*    16     2 */
	Elf32_Half                 e_machine;            /*    18     2 */
	Elf32_Word                 e_version;            /*    20     4 */
	Elf32_Addr                 e_entry;              /*    24     4 */
	Elf32_Off                  e_phoff;              /*    28     4 */
	Elf32_Off                  e_shoff;              /*    32     4 */
	Elf32_Word                 e_flags;              /*    36     4 */
	Elf32_Half                 e_ehsize;             /*    40     2 */
	Elf32_Half                 e_phentsize;          /*    42     2 */
	Elf32_Half                 e_phnum;              /*    44     2 */
	Elf32_Half                 e_shentsize;          /*    46     2 */
	Elf32_Half                 e_shnum;              /*    48     2 */
	Elf32_Half                 e_shstrndx;           /*    50     2 */

	/* size: 52, cachelines: 1, members: 14 */
	/* last cacheline: 52 bytes */
};
struct elf32_phdr {
	Elf32_Word                 p_type;               /*     0     4 */
	Elf32_Off                  p_offset;             /*     4     4 */
	Elf32_Addr                 p_vaddr;              /*     8     4 */
	Elf32_Addr                 p_paddr;              /*    12     4 */
	Elf32_Word                 p_filesz;             /*    16     4 */
	Elf32_Word                 p_memsz;              /*    20     4 */
	Elf32_Word                 p_flags;              /*    24     4 */
	Elf32_Word                 p_align;              /*    28     4 */

	/* size: 32, cachelines: 1, members: 8 */
	/* last cacheline: 32 bytes */
};
struct stack_map_bucket {
	struct pcpu_freelist_node  fnode;                /*     0     8 */
	u32                        hash;                 /*     8     4 */
	u32                        nr;                   /*    12     4 */
	u64                        data[];               /*    16     0 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct bpf_stack_map {
	struct bpf_map             map __attribute__((__aligned__(64))); /*     0   256 */

	/* XXX last struct has 8 bytes of padding */

	/* --- cacheline 4 boundary (256 bytes) --- */
	void *                     elems;                /*   256     8 */
	struct pcpu_freelist       freelist;             /*   264    24 */
	u32                        n_buckets;            /*   288     4 */

	/* XXX 4 bytes hole, try to pack */

	struct stack_map_bucket *  buckets[];            /*   296     0 */

	/* size: 320, cachelines: 5, members: 5 */
	/* sum members: 292, holes: 1, sum holes: 4 */
	/* padding: 24 */
	/* paddings: 1, sum paddings: 8 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(64)));
struct stack_map_irq_work {
	struct irq_work            irq_work;             /*     0    24 */
	struct mm_struct *         mm;                   /*    24     8 */

	/* size: 32, cachelines: 1, members: 2 */
	/* last cacheline: 32 bytes */
};
struct bpf_prog_list {
	struct list_head           node;                 /*     0    16 */
	struct bpf_prog *          prog;                 /*    16     8 */
	struct bpf_cgroup_link *   link;                 /*    24     8 */
	struct bpf_cgroup_storage * storage[10];         /*    32    80 */

	/* size: 112, cachelines: 2, members: 4 */
	/* last cacheline: 48 bytes */
};
struct qdisc_skb_cb {
	struct {
		unsigned int       pkt_len;              /*     0     4 */
		u16                slave_dev_queue_mapping; /*     4     2 */
		u16                tc_classid;           /*     6     2 */
	};                                               /*     0     8 */
	unsigned char              data[20];             /*     8    20 */
	u16                        mru;                  /*    28     2 */

	/* size: 32, cachelines: 1, members: 3 */
	/* padding: 2 */
	/* last cacheline: 32 bytes */
};
struct bpf_skb_data_end {
	struct qdisc_skb_cb        qdisc_cb;             /*     0    32 */

	/* XXX last struct has 2 bytes of padding */

	void *                     data_meta;            /*    32     8 */
	void *                     data_end;             /*    40     8 */

	/* size: 48, cachelines: 1, members: 3 */
	/* paddings: 1, sum paddings: 2 */
	/* last cacheline: 48 bytes */
};
struct bpf_sockopt_buf {
	u8                         data[32];             /*     0    32 */

	/* size: 32, cachelines: 1, members: 1 */
	/* last cacheline: 32 bytes */
};
struct reuseport_array {
	struct bpf_map             map __attribute__((__aligned__(64))); /*     0   256 */

	/* XXX last struct has 8 bytes of padding */

	/* --- cacheline 4 boundary (256 bytes) --- */
	struct sock *              ptrs[];               /*   256     0 */

	/* size: 256, cachelines: 4, members: 2 */
	/* paddings: 1, sum paddings: 8 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(64)));
struct bpf_struct_ops_value {
	refcount_t                 refcnt;               /*     0     4 */
	enum bpf_struct_ops_state  state;                /*     4     4 */

	/* XXX 56 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	char                       data[] __attribute__((__aligned__(64))); /*    64     0 */

	/* size: 64, cachelines: 1, members: 3 */
	/* sum members: 8, holes: 1, sum holes: 56 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 56 */
} __attribute__((__aligned__(64)));
struct bpf_struct_ops_map {
	struct bpf_map             map __attribute__((__aligned__(64))); /*     0   256 */

	/* XXX last struct has 8 bytes of padding */

	/* --- cacheline 4 boundary (256 bytes) --- */
	const struct bpf_struct_ops  * st_ops;           /*   256     8 */
	struct mutex               lock;                 /*   264    32 */
	struct bpf_prog * *        progs;                /*   296     8 */
	void *                     image;                /*   304     8 */
	struct bpf_struct_ops_value * uvalue;            /*   312     8 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	struct bpf_struct_ops_value kvalue __attribute__((__aligned__(64))); /*   320    64 */

	/* size: 384, cachelines: 6, members: 7 */
	/* paddings: 1, sum paddings: 8 */
	/* forced alignments: 2 */
} __attribute__((__aligned__(64)));
struct bpf_struct_ops_tcp_congestion_ops {
	refcount_t                 refcnt;               /*     0     4 */
	enum bpf_struct_ops_state  state;                /*     4     4 */

	/* XXX 56 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	struct tcp_congestion_ops  data __attribute__((__aligned__(64))); /*    64   184 */

	/* size: 256, cachelines: 4, members: 3 */
	/* sum members: 192, holes: 1, sum holes: 56 */
	/* padding: 8 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 56 */
} __attribute__((__aligned__(64)));
struct static_call_tramp_key {
	s32                        tramp;                /*     0     4 */
	s32                        key;                  /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct perf_ns_link_info {
	__u64                      dev;                  /*     0     8 */
	__u64                      ino;                  /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct swevent_hlist {
	struct hlist_head          heads[256];           /*     0  2048 */
	/* --- cacheline 32 boundary (2048 bytes) --- */
	struct callback_head       callback_head __attribute__((__aligned__(8))); /*  2048    16 */

	/* size: 2064, cachelines: 33, members: 2 */
	/* forced alignments: 1 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct pmu_event_list {
	raw_spinlock_t             lock;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           list;                 /*     8    16 */

	/* size: 24, cachelines: 1, members: 2 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct perf_buffer {
	refcount_t                 refcount;             /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct callback_head       callback_head __attribute__((__aligned__(8))); /*     8    16 */
	int                        nr_pages;             /*    24     4 */
	int                        overwrite;            /*    28     4 */
	int                        paused;               /*    32     4 */
	atomic_t                   poll;                 /*    36     4 */
	local_t                    head;                 /*    40     8 */
	unsigned int               nest;                 /*    48     4 */

	/* XXX 4 bytes hole, try to pack */

	local_t                    events;               /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	local_t                    wakeup;               /*    64     8 */
	local_t                    lost;                 /*    72     8 */
	long int                   watermark;            /*    80     8 */
	long int                   aux_watermark;        /*    88     8 */
	spinlock_t                 event_lock;           /*    96     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           event_list;           /*   104    16 */
	atomic_t                   mmap_count;           /*   120     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 2 boundary (128 bytes) --- */
	long unsigned int          mmap_locked;          /*   128     8 */
	struct user_struct *       mmap_user;            /*   136     8 */
	long int                   aux_head;             /*   144     8 */
	unsigned int               aux_nest;             /*   152     4 */

	/* XXX 4 bytes hole, try to pack */

	long int                   aux_wakeup;           /*   160     8 */
	long unsigned int          aux_pgoff;            /*   168     8 */
	int                        aux_nr_pages;         /*   176     4 */
	int                        aux_overwrite;        /*   180     4 */
	atomic_t                   aux_mmap_count;       /*   184     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 3 boundary (192 bytes) --- */
	long unsigned int          aux_mmap_locked;      /*   192     8 */
	void                       (*free_aux)(void *);  /*   200     8 */
	refcount_t                 aux_refcount;         /*   208     4 */
	int                        aux_in_sampling;      /*   212     4 */
	void * *                   aux_pages;            /*   216     8 */
	void *                     aux_priv;             /*   224     8 */
	struct perf_event_mmap_page * user_page;         /*   232     8 */
	void *                     data_pages[];         /*   240     0 */

	/* size: 240, cachelines: 4, members: 33 */
	/* sum members: 216, holes: 6, sum holes: 24 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 4 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct match_token {
	int                        token;                /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	const char  *              pattern;              /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct min_heap {
	void *                     data;                 /*     0     8 */
	int                        nr;                   /*     8     4 */
	int                        size;                 /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct min_heap_callbacks {
	int                        elem_size;            /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	bool                       (*less)(const void  *, const void  *); /*     8     8 */
	void                       (*swp)(void *, void *); /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct remote_function_call {
	struct task_struct *       p;                    /*     0     8 */
	remote_function_f          func;                 /*     8     8 */
	void *                     info;                 /*    16     8 */
	int                        ret;                  /*    24     4 */

	/* size: 32, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct event_function_struct {
	struct perf_event *        event;                /*     0     8 */
	event_f                    func;                 /*     8     8 */
	void *                     data;                 /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct __group_key {
	int                        cpu;                  /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct cgroup *            cgroup;               /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct stop_event_data {
	struct perf_event *        event;                /*     0     8 */
	unsigned int               restart;              /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct perf_read_data {
	struct perf_event *        event;                /*     0     8 */
	bool                       group;                /*     8     1 */

	/* XXX 3 bytes hole, try to pack */

	int                        ret;                  /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* sum members: 13, holes: 1, sum holes: 3 */
	/* last cacheline: 16 bytes */
};
struct perf_read_event {
	struct perf_event_header   header;               /*     0     8 */
	u32                        pid;                  /*     8     4 */
	u32                        tid;                  /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct remote_output {
	struct perf_buffer *       rb;                   /*     0     8 */
	int                        err;                  /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct perf_task_event {
	struct task_struct *       task;                 /*     0     8 */
	struct perf_event_context * task_ctx;            /*     8     8 */
	struct {
		struct perf_event_header header;         /*    16     8 */
		u32                pid;                  /*    24     4 */
		u32                ppid;                 /*    28     4 */
		u32                tid;                  /*    32     4 */
		u32                ptid;                 /*    36     4 */
		u64                time;                 /*    40     8 */
	} event_id;                                      /*    16    32 */

	/* size: 48, cachelines: 1, members: 3 */
	/* last cacheline: 48 bytes */
};
struct perf_comm_event {
	struct task_struct *       task;                 /*     0     8 */
	char *                     comm;                 /*     8     8 */
	int                        comm_size;            /*    16     4 */
	struct {
		struct perf_event_header header;         /*    20     8 */
		u32                pid;                  /*    28     4 */
		u32                tid;                  /*    32     4 */
	} event_id;                                      /*    20    16 */

	/* size: 40, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct perf_namespaces_event {
	struct task_struct *       task;                 /*     0     8 */
	struct {
		struct perf_event_header header;         /*     8     8 */
		u32                pid;                  /*    16     4 */
		u32                tid;                  /*    20     4 */
		u64                nr_namespaces;        /*    24     8 */
		struct perf_ns_link_info link_info[7];   /*    32   112 */
	} event_id;                                      /*     8   136 */

	/* size: 144, cachelines: 3, members: 2 */
	/* last cacheline: 16 bytes */
};
struct perf_cgroup_event {
	char *                     path;                 /*     0     8 */
	int                        path_size;            /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct {
		struct perf_event_header header;         /*    16     8 */
		u64                id;                   /*    24     8 */
		char               path[];               /*    32     0 */
	} event_id;                                      /*    16    16 */

	/* size: 32, cachelines: 1, members: 3 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct perf_mmap_event {
	struct vm_area_struct *    vma;                  /*     0     8 */
	const char  *              file_name;            /*     8     8 */
	int                        file_size;            /*    16     4 */
	int                        maj;                  /*    20     4 */
	int                        min;                  /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        ino;                  /*    32     8 */
	u64                        ino_generation;       /*    40     8 */
	u32                        prot;                 /*    48     4 */
	u32                        flags;                /*    52     4 */
	struct {
		struct perf_event_header header;         /*    56     8 */
		/* --- cacheline 1 boundary (64 bytes) --- */
		u32                pid;                  /*    64     4 */
		u32                tid;                  /*    68     4 */
		u64                start;                /*    72     8 */
		u64                len;                  /*    80     8 */
		u64                pgoff;                /*    88     8 */
	} event_id;                                      /*    56    40 */

	/* size: 96, cachelines: 2, members: 10 */
	/* sum members: 92, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct perf_switch_event {
	struct task_struct *       task;                 /*     0     8 */
	struct task_struct *       next_prev;            /*     8     8 */
	struct {
		struct perf_event_header header;         /*    16     8 */
		u32                next_prev_pid;        /*    24     4 */
		u32                next_prev_tid;        /*    28     4 */
	} event_id;                                      /*    16    16 */

	/* size: 32, cachelines: 1, members: 3 */
	/* last cacheline: 32 bytes */
};
struct perf_ksymbol_event {
	const char  *              name;                 /*     0     8 */
	int                        name_len;             /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct {
		struct perf_event_header header;         /*    16     8 */
		u64                addr;                 /*    24     8 */
		u32                len;                  /*    32     4 */
		u16                ksym_type;            /*    36     2 */
		u16                flags;                /*    38     2 */
	} event_id;                                      /*    16    24 */

	/* size: 40, cachelines: 1, members: 3 */
	/* sum members: 36, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct perf_bpf_event {
	struct bpf_prog *          prog;                 /*     0     8 */
	struct {
		struct perf_event_header header;         /*     8     8 */
		u16                type;                 /*    16     2 */
		u16                flags;                /*    18     2 */
		u32                id;                   /*    20     4 */
		u8                 tag[8];               /*    24     8 */
	} event_id;                                      /*     8    24 */

	/* size: 32, cachelines: 1, members: 2 */
	/* last cacheline: 32 bytes */
};
struct perf_text_poke_event {
	const void  *              old_bytes;            /*     0     8 */
	const void  *              new_bytes;            /*     8     8 */
	size_t                     pad;                  /*    16     8 */
	u16                        old_len;              /*    24     2 */
	u16                        new_len;              /*    26     2 */

	/* XXX 4 bytes hole, try to pack */

	struct {
		struct perf_event_header header;         /*    32     8 */
		u64                addr;                 /*    40     8 */
	} event_id;                                      /*    32    16 */

	/* size: 48, cachelines: 1, members: 6 */
	/* sum members: 44, holes: 1, sum holes: 4 */
	/* last cacheline: 48 bytes */
};
struct swevent_htable {
	struct swevent_hlist *     swevent_hlist;        /*     0     8 */
	struct mutex               hlist_mutex;          /*     8    32 */
	int                        hlist_refcount;       /*    40     4 */
	int                        recursion[4];         /*    44    16 */

	/* size: 64, cachelines: 1, members: 4 */
	/* padding: 4 */
};
struct callchain_cpus_entries {
	struct callback_head       callback_head __attribute__((__aligned__(8))); /*     0    16 */
	struct perf_callchain_entry * cpu_entries[];     /*    16     0 */

	/* size: 16, cachelines: 1, members: 2 */
	/* forced alignments: 1 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct bp_cpuinfo {
	unsigned int               cpu_pinned;           /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	unsigned int *             tsk_pinned;           /*     8     8 */
	unsigned int               flexible;             /*    16     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 16, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct bp_busy_slots {
	unsigned int               pinned;               /*     0     4 */
	unsigned int               flexible;             /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct uprobe {
	struct rb_node             rb_node __attribute__((__aligned__(8))); /*     0    24 */
	refcount_t                 ref;                  /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	struct rw_semaphore        register_rwsem;       /*    32    40 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	struct rw_semaphore        consumer_rwsem;       /*    72    40 */
	struct list_head           pending_list;         /*   112    16 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct uprobe_consumer *   consumers;            /*   128     8 */
	struct inode *             inode;                /*   136     8 */
	loff_t                     offset;               /*   144     8 */
	loff_t                     ref_ctr_offset;       /*   152     8 */
	long unsigned int          flags;                /*   160     8 */
	struct arch_uprobe         arch;                 /*   168    32 */

	/* size: 200, cachelines: 4, members: 11 */
	/* sum members: 196, holes: 1, sum holes: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 8 bytes */
} __attribute__((__aligned__(8)));
struct xol_area {
	wait_queue_head_t          wq;                   /*     0    24 */
	atomic_t                   slot_count;           /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int *        bitmap;               /*    32     8 */
	struct vm_special_mapping  xol_mapping;          /*    40    32 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	struct page *              pages[2];             /*    72    16 */
	long unsigned int          vaddr;                /*    88     8 */

	/* size: 96, cachelines: 2, members: 6 */
	/* sum members: 92, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct capture_control {
	struct compact_control *   cc;                   /*     0     8 */
	struct page *              page;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct page_vma_mapped_walk {
	struct page *              page;                 /*     0     8 */
	struct vm_area_struct *    vma;                  /*     8     8 */
	long unsigned int          address;              /*    16     8 */
	pmd_t *                    pmd;                  /*    24     8 */
	pte_t *                    pte;                  /*    32     8 */
	spinlock_t *               ptl;                  /*    40     8 */
	unsigned int               flags;                /*    48     4 */

	/* size: 56, cachelines: 1, members: 7 */
	/* padding: 4 */
	/* last cacheline: 56 bytes */
};
struct compact_control {
	struct list_head           freepages;            /*     0    16 */
	struct list_head           migratepages;         /*    16    16 */
	unsigned int               nr_freepages;         /*    32     4 */
	unsigned int               nr_migratepages;      /*    36     4 */
	long unsigned int          free_pfn;             /*    40     8 */
	long unsigned int          migrate_pfn;          /*    48     8 */
	long unsigned int          fast_start_pfn;       /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct zone *              zone;                 /*    64     8 */
	long unsigned int          total_migrate_scanned; /*    72     8 */
	long unsigned int          total_free_scanned;   /*    80     8 */
	short unsigned int         fast_search_fail;     /*    88     2 */
	short int                  search_order;         /*    90     2 */
	const gfp_t                gfp_mask;             /*    92     4 */
	int                        order;                /*    96     4 */
	int                        migratetype;          /*   100     4 */
	const unsigned int         alloc_flags;          /*   104     4 */
	const int                  highest_zoneidx;      /*   108     4 */
	enum migrate_mode          mode;                 /*   112     4 */
	bool                       ignore_skip_hint;     /*   116     1 */
	bool                       no_set_skip_hint;     /*   117     1 */
	bool                       ignore_block_suitable; /*   118     1 */
	bool                       direct_compaction;    /*   119     1 */
	bool                       proactive_compaction; /*   120     1 */
	bool                       whole_zone;           /*   121     1 */
	bool                       contended;            /*   122     1 */
	bool                       rescan;               /*   123     1 */
	bool                       alloc_contig;         /*   124     1 */

	/* size: 128, cachelines: 2, members: 27 */
	/* padding: 3 */
};
struct delayed_uprobe {
	struct list_head           list;                 /*     0    16 */
	struct uprobe *            uprobe;               /*    16     8 */
	struct mm_struct *         mm;                   /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* last cacheline: 32 bytes */
};
struct __uprobe_key {
	struct inode *             inode;                /*     0     8 */
	loff_t                     offset;               /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct map_info {
	struct map_info *          next;                 /*     0     8 */
	struct mm_struct *         mm;                   /*     8     8 */
	long unsigned int          vaddr;                /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct user_return_notifier {
	void                       (*on_user_return)(struct user_return_notifier *); /*     0     8 */
	struct hlist_node          link;                 /*     8    16 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct padata_priv {
	struct list_head           list;                 /*     0    16 */
	struct parallel_data *     pd;                   /*    16     8 */
	int                        cb_cpu;               /*    24     4 */
	unsigned int               seq_nr;               /*    28     4 */
	int                        info;                 /*    32     4 */

	/* XXX 4 bytes hole, try to pack */

	void                       (*parallel)(struct padata_priv *); /*    40     8 */
	void                       (*serial)(struct padata_priv *); /*    48     8 */

	/* size: 56, cachelines: 1, members: 7 */
	/* sum members: 52, holes: 1, sum holes: 4 */
	/* last cacheline: 56 bytes */
};
struct parallel_data {
	struct padata_shell *      ps;                   /*     0     8 */
	struct padata_list *       reorder_list;         /*     8     8 */
	struct padata_serial_queue * squeue;             /*    16     8 */
	atomic_t                   refcnt;               /*    24     4 */
	unsigned int               seq_nr;               /*    28     4 */
	unsigned int               processed;            /*    32     4 */
	int                        cpu;                  /*    36     4 */
	struct padata_cpumask      cpumask;              /*    40    16 */
	struct work_struct         reorder_work;         /*    56    64 */

	/* XXX 8 bytes hole, try to pack */

	/* --- cacheline 2 boundary (128 bytes) --- */
	spinlock_t                 lock __attribute__((__aligned__(64))); /*   128     4 */

	/* size: 192, cachelines: 3, members: 10 */
	/* sum members: 124, holes: 1, sum holes: 8 */
	/* padding: 60 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 8 */
} __attribute__((__aligned__(64)));
struct padata_list {
	struct list_head           list;                 /*     0    16 */
	spinlock_t                 lock;                 /*    16     4 */

	/* size: 24, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct padata_serial_queue {
	struct padata_list         serial;               /*     0    24 */

	/* XXX last struct has 4 bytes of padding */

	struct work_struct         work;                 /*    24    64 */
	/* --- cacheline 1 boundary (64 bytes) was 24 bytes ago --- */
	struct parallel_data *     pd;                   /*    88     8 */

	/* size: 96, cachelines: 2, members: 3 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 32 bytes */
};
struct padata_cpumask {
	cpumask_var_t              pcpu;                 /*     0     8 */
	cpumask_var_t              cbcpu;                /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct padata_shell {
	struct padata_instance *   pinst;                /*     0     8 */
	struct parallel_data *     pd;                   /*     8     8 */
	struct parallel_data *     opd;                  /*    16     8 */
	struct list_head           list;                 /*    24    16 */

	/* size: 40, cachelines: 1, members: 4 */
	/* last cacheline: 40 bytes */
};
struct padata_instance {
	struct hlist_node          cpu_online_node;      /*     0    16 */
	struct hlist_node          cpu_dead_node;        /*    16    16 */
	struct workqueue_struct *  parallel_wq;          /*    32     8 */
	struct workqueue_struct *  serial_wq;            /*    40     8 */
	struct list_head           pslist;               /*    48    16 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct padata_cpumask      cpumask;              /*    64    16 */
	struct kobject             kobj;                 /*    80    96 */
	/* --- cacheline 2 boundary (128 bytes) was 48 bytes ago --- */
	struct mutex               lock;                 /*   176    32 */
	/* --- cacheline 3 boundary (192 bytes) was 16 bytes ago --- */
	u8                         flags;                /*   208     1 */

	/* size: 216, cachelines: 4, members: 9 */
	/* padding: 7 */
	/* last cacheline: 24 bytes */
};
struct padata_mt_job {
	void                       (*thread_fn)(long unsigned int, long unsigned int, void *); /*     0     8 */
	void *                     fn_arg;               /*     8     8 */
	long unsigned int          start;                /*    16     8 */
	long unsigned int          size;                 /*    24     8 */
	long unsigned int          align;                /*    32     8 */
	long unsigned int          min_chunk;            /*    40     8 */
	int                        max_threads;          /*    48     4 */

	/* size: 56, cachelines: 1, members: 7 */
	/* padding: 4 */
	/* last cacheline: 56 bytes */
};
struct padata_work {
	struct work_struct         pw_work;              /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct list_head           pw_list;              /*    64    16 */
	void *                     pw_data;              /*    80     8 */

	/* size: 88, cachelines: 2, members: 3 */
	/* last cacheline: 24 bytes */
};
struct padata_mt_job_state {
	spinlock_t                 lock;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct completion          completion;           /*     8    32 */
	struct padata_mt_job *     job;                  /*    40     8 */
	int                        nworks;               /*    48     4 */
	int                        nworks_fini;          /*    52     4 */
	long unsigned int          chunk_size;           /*    56     8 */

	/* size: 64, cachelines: 1, members: 6 */
	/* sum members: 60, holes: 1, sum holes: 4 */
};
struct padata_sysfs_entry {
	struct attribute           attr;                 /*     0    16 */

	/* XXX last struct has 6 bytes of padding */

	ssize_t                    (*show)(struct padata_instance *, struct attribute *, char *); /*    16     8 */
	ssize_t                    (*store)(struct padata_instance *, struct attribute *, const char  *, size_t); /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* paddings: 1, sum paddings: 6 */
	/* last cacheline: 32 bytes */
};
struct static_key_mod {
	struct static_key_mod *    next;                 /*     0     8 */
	struct jump_entry *        entries;              /*     8     8 */
	struct module *            mod;                  /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct static_key_deferred {
	struct static_key          key;                  /*     0    16 */
	long unsigned int          timeout;              /*    16     8 */
	struct delayed_work        work;                 /*    24   192 */

	/* size: 216, cachelines: 4, members: 3 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_context_tracking_user {
	struct trace_entry         ent;                  /*     0     8 */
	int                        dummy;                /*     8     4 */
	char                       __data[];             /*    12     0 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct trace_event_data_offsets_context_tracking_user {

	/* size: 0, cachelines: 0, members: 0 */
};
struct rseq_cs {
	__u32                      version;              /*     0     4 */
	__u32                      flags;                /*     4     4 */
	__u64                      start_ip;             /*     8     8 */
	__u64                      post_commit_offset;   /*    16     8 */
	__u64                      abort_ip;             /*    24     8 */

	/* size: 32, cachelines: 1, members: 5 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(32)));
struct trace_event_raw_rseq_update {
	struct trace_entry         ent;                  /*     0     8 */
	s32                        cpu_id;               /*     8     4 */
	char                       __data[];             /*    12     0 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct trace_event_raw_rseq_ip_fixup {
	struct trace_entry         ent;                  /*     0     8 */
	long unsigned int          regs_ip;              /*     8     8 */
	long unsigned int          start_ip;             /*    16     8 */
	long unsigned int          post_commit_offset;   /*    24     8 */
	long unsigned int          abort_ip;             /*    32     8 */
	char                       __data[];             /*    40     0 */

	/* size: 40, cachelines: 1, members: 6 */
	/* last cacheline: 40 bytes */
};
struct trace_event_data_offsets_rseq_update {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_rseq_ip_fixup {

	/* size: 0, cachelines: 0, members: 0 */
};
struct export_operations {
	int                        (*encode_fh)(struct inode *, __u32 *, int *, struct inode *); /*     0     8 */
	struct dentry *            (*fh_to_dentry)(struct super_block *, struct fid *, int, int); /*     8     8 */
	struct dentry *            (*fh_to_parent)(struct super_block *, struct fid *, int, int); /*    16     8 */
	int                        (*get_name)(struct dentry *, char *, struct dentry *); /*    24     8 */
	struct dentry *            (*get_parent)(struct dentry *); /*    32     8 */
	int                        (*commit_metadata)(struct inode *); /*    40     8 */
	int                        (*get_uuid)(struct super_block *, u8 *, u32 *, u64 *); /*    48     8 */
	int                        (*map_blocks)(struct inode *, loff_t, u64, struct iomap *, bool, u32 *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*commit_blocks)(struct inode *, struct iomap *, int, struct iattr *); /*    64     8 */
	u64                        kabi_reserved1;       /*    72     8 */
	u64                        kabi_reserved2;       /*    80     8 */

	/* size: 88, cachelines: 2, members: 11 */
	/* last cacheline: 24 bytes */
};
struct vm_event_state {
	long unsigned int          event[96];            /*     0   768 */

	/* size: 768, cachelines: 12, members: 1 */
};
struct wait_page_key {
	struct page *              page;                 /*     0     8 */
	int                        bit_nr;               /*     8     4 */
	int                        page_match;           /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct pagevec {
	unsigned char              nr;                   /*     0     1 */
	bool                       percpu_pvec_drained;  /*     1     1 */

	/* XXX 6 bytes hole, try to pack */

	struct page *              pages[15];            /*     8   120 */

	/* size: 128, cachelines: 2, members: 3 */
	/* sum members: 122, holes: 1, sum holes: 6 */
};
struct fid {
	union {
		struct {
			u32        ino;                  /*     0     4 */
			u32        gen;                  /*     4     4 */
			u32        parent_ino;           /*     8     4 */
			u32        parent_gen;           /*    12     4 */
		} i32;                                   /*     0    16 */
		struct {
			u32        block;                /*     0     4 */
			u16        partref;              /*     4     2 */
			u16        parent_partref;       /*     6     2 */
			u32        generation;           /*     8     4 */
			u32        parent_block;         /*    12     4 */
			u32        parent_generation;    /*    16     4 */
		} udf;                                   /*     0    20 */
		__u32              raw[0];               /*     0     0 */
	};                                               /*     0    20 */

	/* size: 20, cachelines: 1, members: 1 */
	/* last cacheline: 20 bytes */
};
struct trace_event_raw_mm_filemap_op_page_cache {
	struct trace_entry         ent;                  /*     0     8 */
	long unsigned int          pfn;                  /*     8     8 */
	long unsigned int          i_ino;                /*    16     8 */
	long unsigned int          index;                /*    24     8 */
	dev_t                      s_dev;                /*    32     4 */
	char                       __data[];             /*    36     0 */

	/* size: 40, cachelines: 1, members: 6 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct trace_event_raw_filemap_set_wb_err {
	struct trace_entry         ent;                  /*     0     8 */
	long unsigned int          i_ino;                /*     8     8 */
	dev_t                      s_dev;                /*    16     4 */
	errseq_t                   errseq;               /*    20     4 */
	char                       __data[];             /*    24     0 */

	/* size: 24, cachelines: 1, members: 5 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_file_check_and_advance_wb_err {
	struct trace_entry         ent;                  /*     0     8 */
	struct file *              file;                 /*     8     8 */
	long unsigned int          i_ino;                /*    16     8 */
	dev_t                      s_dev;                /*    24     4 */
	errseq_t                   old;                  /*    28     4 */
	errseq_t                   new;                  /*    32     4 */
	char                       __data[];             /*    36     0 */

	/* size: 40, cachelines: 1, members: 7 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct trace_event_data_offsets_mm_filemap_op_page_cache {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_filemap_set_wb_err {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_file_check_and_advance_wb_err {

	/* size: 0, cachelines: 0, members: 0 */
};
struct kmem_cache {
	struct kmem_cache_cpu *    cpu_slab;             /*     0     8 */
	slab_flags_t               flags;                /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          min_partial;          /*    16     8 */
	unsigned int               size;                 /*    24     4 */
	unsigned int               object_size;          /*    28     4 */
	struct reciprocal_value    reciprocal_size;      /*    32     8 */

	/* XXX last struct has 2 bytes of padding */

	unsigned int               offset;               /*    40     4 */
	unsigned int               cpu_partial;          /*    44     4 */
	struct kmem_cache_order_objects oo;              /*    48     4 */
	struct kmem_cache_order_objects max;             /*    52     4 */
	struct kmem_cache_order_objects min;             /*    56     4 */
	gfp_t                      allocflags;           /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        refcount;             /*    64     4 */

	/* XXX 4 bytes hole, try to pack */

	void                       (*ctor)(void *);      /*    72     8 */
	unsigned int               inuse;                /*    80     4 */
	unsigned int               align;                /*    84     4 */
	unsigned int               red_left_pad;         /*    88     4 */

	/* XXX 4 bytes hole, try to pack */

	const char  *              name;                 /*    96     8 */
	struct list_head           list;                 /*   104    16 */
	struct kobject             kobj;                 /*   120    96 */
	/* --- cacheline 3 boundary (192 bytes) was 24 bytes ago --- */
	unsigned int               remote_node_defrag_ratio; /*   216     4 */

	/* XXX 4 bytes hole, try to pack */

	unsigned int *             random_seq;           /*   224     8 */
	unsigned int               useroffset;           /*   232     4 */
	unsigned int               usersize;             /*   236     4 */
	struct kmem_cache_node *   node[1024];           /*   240  8192 */

	/* size: 8432, cachelines: 132, members: 25 */
	/* sum members: 8416, holes: 4, sum holes: 16 */
	/* paddings: 1, sum paddings: 2 */
	/* last cacheline: 48 bytes */
};
struct reciprocal_value {
	u32                        m;                    /*     0     4 */
	u8                         sh1;                  /*     4     1 */
	u8                         sh2;                  /*     5     1 */

	/* size: 8, cachelines: 1, members: 3 */
	/* padding: 2 */
	/* last cacheline: 8 bytes */
};
struct kmem_cache_cpu {
	void * *                   freelist;             /*     0     8 */
	long unsigned int          tid;                  /*     8     8 */
	struct page *              page;                 /*    16     8 */
	struct page *              partial;              /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct kmem_cache_order_objects {
	unsigned int               x;                    /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct kmem_cache_node {
	spinlock_t                 list_lock;            /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          nr_partial;           /*     8     8 */
	struct list_head           partial;              /*    16    16 */
	atomic_long_t              nr_slabs;             /*    32     8 */
	atomic_long_t              total_objects;        /*    40     8 */
	struct list_head           full;                 /*    48    16 */

	/* size: 64, cachelines: 1, members: 6 */
	/* sum members: 60, holes: 1, sum holes: 4 */
};
struct oom_control {
	struct zonelist *          zonelist;             /*     0     8 */
	nodemask_t *               nodemask;             /*     8     8 */
	struct mem_cgroup *        memcg;                /*    16     8 */
	const gfp_t                gfp_mask;             /*    24     4 */
	const int                  order;                /*    28     4 */
	long unsigned int          totalpages;           /*    32     8 */
	struct task_struct *       chosen;               /*    40     8 */
	long int                   chosen_points;        /*    48     8 */
	enum oom_constraint        constraint;           /*    56     4 */

	/* size: 64, cachelines: 1, members: 9 */
	/* padding: 4 */
};
struct trace_event_raw_oom_score_adj_update {
	struct trace_entry         ent;                  /*     0     8 */
	pid_t                      pid;                  /*     8     4 */
	char                       comm[16];             /*    12    16 */
	short int                  oom_score_adj;        /*    28     2 */
	char                       __data[];             /*    30     0 */

	/* size: 32, cachelines: 1, members: 5 */
	/* padding: 2 */
	/* last cacheline: 32 bytes */
};
struct trace_event_raw_reclaim_retry_zone {
	struct trace_entry         ent;                  /*     0     8 */
	int                        node;                 /*     8     4 */
	int                        zone_idx;             /*    12     4 */
	int                        order;                /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          reclaimable;          /*    24     8 */
	long unsigned int          available;            /*    32     8 */
	long unsigned int          min_wmark;            /*    40     8 */
	int                        no_progress_loops;    /*    48     4 */
	bool                       wmark_check;          /*    52     1 */
	char                       __data[];             /*    53     0 */

	/* size: 56, cachelines: 1, members: 10 */
	/* sum members: 49, holes: 1, sum holes: 4 */
	/* padding: 3 */
	/* last cacheline: 56 bytes */
};
struct trace_event_raw_mark_victim {
	struct trace_entry         ent;                  /*     0     8 */
	int                        pid;                  /*     8     4 */
	char                       __data[];             /*    12     0 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct trace_event_raw_wake_reaper {
	struct trace_entry         ent;                  /*     0     8 */
	int                        pid;                  /*     8     4 */
	char                       __data[];             /*    12     0 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct trace_event_raw_start_task_reaping {
	struct trace_entry         ent;                  /*     0     8 */
	int                        pid;                  /*     8     4 */
	char                       __data[];             /*    12     0 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct trace_event_raw_finish_task_reaping {
	struct trace_entry         ent;                  /*     0     8 */
	int                        pid;                  /*     8     4 */
	char                       __data[];             /*    12     0 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct trace_event_raw_skip_task_reaping {
	struct trace_entry         ent;                  /*     0     8 */
	int                        pid;                  /*     8     4 */
	char                       __data[];             /*    12     0 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct trace_event_raw_compact_retry {
	struct trace_entry         ent;                  /*     0     8 */
	int                        order;                /*     8     4 */
	int                        priority;             /*    12     4 */
	int                        result;               /*    16     4 */
	int                        retries;              /*    20     4 */
	int                        max_retries;          /*    24     4 */
	bool                       ret;                  /*    28     1 */
	char                       __data[];             /*    29     0 */

	/* size: 32, cachelines: 1, members: 8 */
	/* padding: 3 */
	/* last cacheline: 32 bytes */
};
struct trace_event_data_offsets_oom_score_adj_update {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_reclaim_retry_zone {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_mark_victim {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_wake_reaper {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_start_task_reaping {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_finish_task_reaping {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_skip_task_reaping {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_compact_retry {

	/* size: 0, cachelines: 0, members: 0 */
};
struct wb_lock_cookie {
	bool                       locked;               /*     0     1 */

	/* XXX 7 bytes hole, try to pack */

	long unsigned int          flags;                /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 9, holes: 1, sum holes: 7 */
	/* last cacheline: 16 bytes */
};
struct dirty_throttle_control {
	struct wb_domain *         dom;                  /*     0     8 */
	struct dirty_throttle_control * gdtc;            /*     8     8 */
	struct bdi_writeback *     wb;                   /*    16     8 */
	struct fprop_local_percpu * wb_completions;      /*    24     8 */
	long unsigned int          avail;                /*    32     8 */
	long unsigned int          dirty;                /*    40     8 */
	long unsigned int          thresh;               /*    48     8 */
	long unsigned int          bg_thresh;            /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	long unsigned int          wb_dirty;             /*    64     8 */
	long unsigned int          wb_thresh;            /*    72     8 */
	long unsigned int          wb_bg_thresh;         /*    80     8 */
	long unsigned int          pos_ratio;            /*    88     8 */

	/* size: 96, cachelines: 2, members: 12 */
	/* last cacheline: 32 bytes */
};
struct trace_event_raw_mm_lru_insertion {
	struct trace_entry         ent;                  /*     0     8 */
	struct page *              page;                 /*     8     8 */
	long unsigned int          pfn;                  /*    16     8 */
	int                        lru;                  /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          flags;                /*    32     8 */
	char                       __data[];             /*    40     0 */

	/* size: 40, cachelines: 1, members: 6 */
	/* sum members: 36, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct trace_event_raw_mm_lru_activate {
	struct trace_entry         ent;                  /*     0     8 */
	struct page *              page;                 /*     8     8 */
	long unsigned int          pfn;                  /*    16     8 */
	char                       __data[];             /*    24     0 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct trace_event_data_offsets_mm_lru_insertion {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_mm_lru_activate {

	/* size: 0, cachelines: 0, members: 0 */
};
struct lru_rotate {
	local_lock_t               lock;                 /*     0     0 */
	struct pagevec             pvec;                 /*     0   128 */

	/* size: 128, cachelines: 2, members: 2 */
};
struct lru_pvecs {
	local_lock_t               lock;                 /*     0     0 */
	struct pagevec             lru_add;              /*     0   128 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct pagevec             lru_deactivate_file;  /*   128   128 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	struct pagevec             lru_deactivate;       /*   256   128 */
	/* --- cacheline 6 boundary (384 bytes) --- */
	struct pagevec             lru_lazyfree;         /*   384   128 */
	/* --- cacheline 8 boundary (512 bytes) --- */
	struct pagevec             activate_page;        /*   512   128 */

	/* size: 640, cachelines: 10, members: 6 */
};
struct reclaim_stat {
	unsigned int               nr_dirty;             /*     0     4 */
	unsigned int               nr_unqueued_dirty;    /*     4     4 */
	unsigned int               nr_congested;         /*     8     4 */
	unsigned int               nr_writeback;         /*    12     4 */
	unsigned int               nr_immediate;         /*    16     4 */
	unsigned int               nr_pageout;           /*    20     4 */
	unsigned int               nr_activate[2];       /*    24     8 */
	unsigned int               nr_ref_keep;          /*    32     4 */
	unsigned int               nr_unmap_fail;        /*    36     4 */
	unsigned int               nr_lazyfree_fail;     /*    40     4 */

	/* size: 44, cachelines: 1, members: 10 */
	/* last cacheline: 44 bytes */
};
struct trace_event_raw_mm_vmscan_kswapd_sleep {
	struct trace_entry         ent;                  /*     0     8 */
	int                        nid;                  /*     8     4 */
	char                       __data[];             /*    12     0 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct trace_event_raw_mm_vmscan_kswapd_wake {
	struct trace_entry         ent;                  /*     0     8 */
	int                        nid;                  /*     8     4 */
	int                        zid;                  /*    12     4 */
	int                        order;                /*    16     4 */
	char                       __data[];             /*    20     0 */

	/* size: 20, cachelines: 1, members: 5 */
	/* last cacheline: 20 bytes */
};
struct trace_event_raw_mm_vmscan_wakeup_kswapd {
	struct trace_entry         ent;                  /*     0     8 */
	int                        nid;                  /*     8     4 */
	int                        zid;                  /*    12     4 */
	int                        order;                /*    16     4 */
	gfp_t                      gfp_flags;            /*    20     4 */
	char                       __data[];             /*    24     0 */

	/* size: 24, cachelines: 1, members: 6 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_mm_vmscan_direct_reclaim_begin_template {
	struct trace_entry         ent;                  /*     0     8 */
	int                        order;                /*     8     4 */
	gfp_t                      gfp_flags;            /*    12     4 */
	char                       __data[];             /*    16     0 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct trace_event_raw_mm_vmscan_direct_reclaim_end_template {
	struct trace_entry         ent;                  /*     0     8 */
	long unsigned int          nr_reclaimed;         /*     8     8 */
	char                       __data[];             /*    16     0 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct trace_event_raw_mm_shrink_slab_start {
	struct trace_entry         ent;                  /*     0     8 */
	struct shrinker *          shr;                  /*     8     8 */
	void *                     shrink;               /*    16     8 */
	int                        nid;                  /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	long int                   nr_objects_to_shrink; /*    32     8 */
	gfp_t                      gfp_flags;            /*    40     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          cache_items;          /*    48     8 */
	long long unsigned int     delta;                /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	long unsigned int          total_scan;           /*    64     8 */
	int                        priority;             /*    72     4 */
	char                       __data[];             /*    76     0 */

	/* size: 80, cachelines: 2, members: 11 */
	/* sum members: 68, holes: 2, sum holes: 8 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct trace_event_raw_mm_shrink_slab_end {
	struct trace_entry         ent;                  /*     0     8 */
	struct shrinker *          shr;                  /*     8     8 */
	int                        nid;                  /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     shrink;               /*    24     8 */
	long int                   unused_scan;          /*    32     8 */
	long int                   new_scan;             /*    40     8 */
	int                        retval;               /*    48     4 */

	/* XXX 4 bytes hole, try to pack */

	long int                   total_scan;           /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	char                       __data[];             /*    64     0 */

	/* size: 64, cachelines: 1, members: 9 */
	/* sum members: 56, holes: 2, sum holes: 8 */
};
struct trace_event_raw_mm_vmscan_lru_isolate {
	struct trace_entry         ent;                  /*     0     8 */
	int                        highest_zoneidx;      /*     8     4 */
	int                        order;                /*    12     4 */
	long unsigned int          nr_requested;         /*    16     8 */
	long unsigned int          nr_scanned;           /*    24     8 */
	long unsigned int          nr_skipped;           /*    32     8 */
	long unsigned int          nr_taken;             /*    40     8 */
	unsigned int               isolate_mode;         /*    48     4 */
	int                        lru;                  /*    52     4 */
	char                       __data[];             /*    56     0 */

	/* size: 56, cachelines: 1, members: 10 */
	/* last cacheline: 56 bytes */
};
struct trace_event_raw_mm_vmscan_writepage {
	struct trace_entry         ent;                  /*     0     8 */
	long unsigned int          pfn;                  /*     8     8 */
	int                        reclaim_flags;        /*    16     4 */
	char                       __data[];             /*    20     0 */

	/* size: 24, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_mm_vmscan_lru_shrink_inactive {
	struct trace_entry         ent;                  /*     0     8 */
	int                        nid;                  /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          nr_scanned;           /*    16     8 */
	long unsigned int          nr_reclaimed;         /*    24     8 */
	long unsigned int          nr_dirty;             /*    32     8 */
	long unsigned int          nr_writeback;         /*    40     8 */
	long unsigned int          nr_congested;         /*    48     8 */
	long unsigned int          nr_immediate;         /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	unsigned int               nr_activate0;         /*    64     4 */
	unsigned int               nr_activate1;         /*    68     4 */
	long unsigned int          nr_ref_keep;          /*    72     8 */
	long unsigned int          nr_unmap_fail;        /*    80     8 */
	int                        priority;             /*    88     4 */
	int                        reclaim_flags;        /*    92     4 */
	char                       __data[];             /*    96     0 */

	/* size: 96, cachelines: 2, members: 15 */
	/* sum members: 92, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct trace_event_raw_mm_vmscan_lru_shrink_active {
	struct trace_entry         ent;                  /*     0     8 */
	int                        nid;                  /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          nr_taken;             /*    16     8 */
	long unsigned int          nr_active;            /*    24     8 */
	long unsigned int          nr_deactivated;       /*    32     8 */
	long unsigned int          nr_referenced;        /*    40     8 */
	int                        priority;             /*    48     4 */
	int                        reclaim_flags;        /*    52     4 */
	char                       __data[];             /*    56     0 */

	/* size: 56, cachelines: 1, members: 9 */
	/* sum members: 52, holes: 1, sum holes: 4 */
	/* last cacheline: 56 bytes */
};
struct trace_event_raw_mm_vmscan_inactive_list_is_low {
	struct trace_entry         ent;                  /*     0     8 */
	int                        nid;                  /*     8     4 */
	int                        reclaim_idx;          /*    12     4 */
	long unsigned int          total_inactive;       /*    16     8 */
	long unsigned int          inactive;             /*    24     8 */
	long unsigned int          total_active;         /*    32     8 */
	long unsigned int          active;               /*    40     8 */
	long unsigned int          ratio;                /*    48     8 */
	int                        reclaim_flags;        /*    56     4 */
	char                       __data[];             /*    60     0 */

	/* size: 64, cachelines: 1, members: 10 */
	/* padding: 4 */
};
struct trace_event_raw_mm_vmscan_node_reclaim_begin {
	struct trace_entry         ent;                  /*     0     8 */
	int                        nid;                  /*     8     4 */
	int                        order;                /*    12     4 */
	gfp_t                      gfp_flags;            /*    16     4 */
	char                       __data[];             /*    20     0 */

	/* size: 20, cachelines: 1, members: 5 */
	/* last cacheline: 20 bytes */
};
struct trace_event_data_offsets_mm_vmscan_kswapd_sleep {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_mm_vmscan_kswapd_wake {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_mm_vmscan_wakeup_kswapd {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_mm_vmscan_direct_reclaim_begin_template {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_mm_vmscan_direct_reclaim_end_template {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_mm_shrink_slab_start {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_mm_shrink_slab_end {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_mm_vmscan_lru_isolate {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_mm_vmscan_writepage {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_mm_vmscan_lru_shrink_inactive {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_mm_vmscan_lru_shrink_active {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_mm_vmscan_inactive_list_is_low {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_mm_vmscan_node_reclaim_begin {

	/* size: 0, cachelines: 0, members: 0 */
};
struct scan_control {
	long unsigned int          nr_to_reclaim;        /*     0     8 */
	nodemask_t *               nodemask;             /*     8     8 */
	struct mem_cgroup *        target_mem_cgroup;    /*    16     8 */
	long unsigned int          anon_cost;            /*    24     8 */
	long unsigned int          file_cost;            /*    32     8 */
	unsigned int               may_deactivate:2;     /*    40: 0  4 */
	unsigned int               force_deactivate:1;   /*    40: 2  4 */
	unsigned int               skipped_deactivate:1; /*    40: 3  4 */
	unsigned int               may_writepage:1;      /*    40: 4  4 */
	unsigned int               may_unmap:1;          /*    40: 5  4 */
	unsigned int               may_swap:1;           /*    40: 6  4 */
	unsigned int               memcg_low_reclaim:1;  /*    40: 7  4 */
	unsigned int               memcg_low_skipped:1;  /*    40: 8  4 */
	unsigned int               hibernation_mode:1;   /*    40: 9  4 */
	unsigned int               compaction_ready:1;   /*    40:10  4 */
	unsigned int               cache_trim_mode:1;    /*    40:11  4 */
	unsigned int               file_is_tiny:1;       /*    40:12  4 */

	/* XXX 3 bits hole, try to pack */
	/* Bitfield combined with next fields */

	s8                         order;                /*    42     1 */
	s8                         priority;             /*    43     1 */
	s8                         reclaim_idx;          /*    44     1 */

	/* XXX 3 bytes hole, try to pack */

	gfp_t                      gfp_mask;             /*    48     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          nr_scanned;           /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	long unsigned int          nr_reclaimed;         /*    64     8 */
	struct {
		unsigned int       dirty;                /*    72     4 */
		unsigned int       unqueued_dirty;       /*    76     4 */
		unsigned int       congested;            /*    80     4 */
		unsigned int       writeback;            /*    84     4 */
		unsigned int       immediate;            /*    88     4 */
		unsigned int       file_taken;           /*    92     4 */
		unsigned int       taken;                /*    96     4 */
	} nr;                                            /*    72    28 */

	/* XXX 4 bytes hole, try to pack */

	struct reclaim_state       reclaim_state;        /*   104     8 */

	/* size: 112, cachelines: 2, members: 25 */
	/* sum members: 99, holes: 3, sum holes: 11 */
	/* sum bitfield members: 13 bits, bit holes: 1, sum bit holes: 3 bits */
	/* last cacheline: 48 bytes */
};
struct xattr {
	const char  *              name;                 /*     0     8 */
	void *                     value;                /*     8     8 */
	size_t                     value_len;            /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct constant_table {
	const char  *              name;                 /*     0     8 */
	int                        value;                /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct shared_policy {
	struct rb_root             root;                 /*     0     8 */
	rwlock_t                   lock;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct simple_xattrs {
	struct list_head           head;                 /*     0    16 */
	spinlock_t                 lock;                 /*    16     4 */

	/* size: 24, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct simple_xattr {
	struct list_head           list;                 /*     0    16 */
	char *                     name;                 /*    16     8 */
	size_t                     size;                 /*    24     8 */
	char                       value[];              /*    32     0 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct shmem_inode_info {
	spinlock_t                 lock;                 /*     0     4 */
	unsigned int               seals;                /*     4     4 */
	long unsigned int          flags;                /*     8     8 */
	long unsigned int          alloced;              /*    16     8 */
	long unsigned int          swapped;              /*    24     8 */
	long unsigned int          fallocend;            /*    32     8 */
	struct list_head           shrinklist;           /*    40    16 */
	struct list_head           swaplist;             /*    56    16 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	struct shared_policy       policy;               /*    72    16 */
	struct simple_xattrs       xattrs;               /*    88    24 */

	/* XXX last struct has 4 bytes of padding */

	atomic_t                   stop_eviction;        /*   112     4 */

	/* XXX 4 bytes hole, try to pack */

	struct inode               vfs_inode __attribute__((__aligned__(8))); /*   120   624 */

	/* size: 744, cachelines: 12, members: 12 */
	/* sum members: 740, holes: 1, sum holes: 4 */
	/* paddings: 1, sum paddings: 4 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 4 */
	/* last cacheline: 40 bytes */
} __attribute__((__aligned__(8)));
struct shmem_sb_info {
	long unsigned int          max_blocks;           /*     0     8 */
	struct percpu_counter      used_blocks;          /*     8    40 */
	long unsigned int          max_inodes;           /*    48     8 */
	long unsigned int          free_inodes;          /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	spinlock_t                 stat_lock;            /*    64     4 */
	umode_t                    mode;                 /*    68     2 */
	unsigned char              huge;                 /*    70     1 */

	/* XXX 1 byte hole, try to pack */

	kuid_t                     uid;                  /*    72     4 */
	kgid_t                     gid;                  /*    76     4 */
	bool                       full_inums;           /*    80     1 */

	/* XXX 7 bytes hole, try to pack */

	ino_t                      next_ino;             /*    88     8 */
	ino_t *                    ino_batch;            /*    96     8 */
	struct mempolicy *         mpol;                 /*   104     8 */
	spinlock_t                 shrinklist_lock;      /*   112     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           shrinklist;           /*   120    16 */
	/* --- cacheline 2 boundary (128 bytes) was 8 bytes ago --- */
	long unsigned int          shrinklist_len;       /*   136     8 */

	/* size: 144, cachelines: 3, members: 16 */
	/* sum members: 132, holes: 3, sum holes: 12 */
	/* last cacheline: 16 bytes */
};
struct shmem_falloc {
	wait_queue_head_t *        waitq;                /*     0     8 */
	long unsigned int          start;                /*     8     8 */
	long unsigned int          next;                 /*    16     8 */
	long unsigned int          nr_falloced;          /*    24     8 */
	long unsigned int          nr_unswapped;         /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct shmem_options {
	long long unsigned int     blocks;               /*     0     8 */
	long long unsigned int     inodes;               /*     8     8 */
	struct mempolicy *         mpol;                 /*    16     8 */
	kuid_t                     uid;                  /*    24     4 */
	kgid_t                     gid;                  /*    28     4 */
	umode_t                    mode;                 /*    32     2 */
	bool                       full_inums;           /*    34     1 */

	/* XXX 1 byte hole, try to pack */

	int                        huge;                 /*    36     4 */
	int                        seen;                 /*    40     4 */

	/* size: 48, cachelines: 1, members: 9 */
	/* sum members: 43, holes: 1, sum holes: 1 */
	/* padding: 4 */
	/* last cacheline: 48 bytes */
};
struct contig_page_info {
	long unsigned int          free_pages;           /*     0     8 */
	long unsigned int          free_blocks_total;    /*     8     8 */
	long unsigned int          free_blocks_suitable; /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct radix_tree_iter {
	long unsigned int          index;                /*     0     8 */
	long unsigned int          next_index;           /*     8     8 */
	long unsigned int          tags;                 /*    16     8 */
	struct xa_node *           node;                 /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct pcpu_group_info {
	int                        nr_units;             /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          base_offset;          /*     8     8 */
	unsigned int *             cpu_map;              /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct pcpu_alloc_info {
	size_t                     static_size;          /*     0     8 */
	size_t                     reserved_size;        /*     8     8 */
	size_t                     dyn_size;             /*    16     8 */
	size_t                     unit_size;            /*    24     8 */
	size_t                     atom_size;            /*    32     8 */
	size_t                     alloc_size;           /*    40     8 */
	size_t                     __ai_size;            /*    48     8 */
	int                        nr_groups;            /*    56     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	struct pcpu_group_info     groups[];             /*    64     0 */

	/* size: 64, cachelines: 1, members: 9 */
	/* sum members: 60, holes: 1, sum holes: 4 */
};
struct trace_event_raw_percpu_alloc_percpu {
	struct trace_entry         ent;                  /*     0     8 */
	bool                       reserved;             /*     8     1 */
	bool                       is_atomic;            /*     9     1 */

	/* XXX 6 bytes hole, try to pack */

	size_t                     size;                 /*    16     8 */
	size_t                     align;                /*    24     8 */
	void *                     base_addr;            /*    32     8 */
	int                        off;                  /*    40     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     ptr;                  /*    48     8 */
	char                       __data[];             /*    56     0 */

	/* size: 56, cachelines: 1, members: 9 */
	/* sum members: 46, holes: 2, sum holes: 10 */
	/* last cacheline: 56 bytes */
};
struct trace_event_raw_percpu_free_percpu {
	struct trace_entry         ent;                  /*     0     8 */
	void *                     base_addr;            /*     8     8 */
	int                        off;                  /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     ptr;                  /*    24     8 */
	char                       __data[];             /*    32     0 */

	/* size: 32, cachelines: 1, members: 5 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct trace_event_raw_percpu_alloc_percpu_fail {
	struct trace_entry         ent;                  /*     0     8 */
	bool                       reserved;             /*     8     1 */
	bool                       is_atomic;            /*     9     1 */

	/* XXX 6 bytes hole, try to pack */

	size_t                     size;                 /*    16     8 */
	size_t                     align;                /*    24     8 */
	char                       __data[];             /*    32     0 */

	/* size: 32, cachelines: 1, members: 6 */
	/* sum members: 26, holes: 1, sum holes: 6 */
	/* last cacheline: 32 bytes */
};
struct trace_event_raw_percpu_create_chunk {
	struct trace_entry         ent;                  /*     0     8 */
	void *                     base_addr;            /*     8     8 */
	char                       __data[];             /*    16     0 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct trace_event_raw_percpu_destroy_chunk {
	struct trace_entry         ent;                  /*     0     8 */
	void *                     base_addr;            /*     8     8 */
	char                       __data[];             /*    16     0 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct trace_event_data_offsets_percpu_alloc_percpu {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_percpu_free_percpu {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_percpu_alloc_percpu_fail {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_percpu_create_chunk {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_percpu_destroy_chunk {

	/* size: 0, cachelines: 0, members: 0 */
};
struct pcpu_block_md {
	int                        scan_hint;            /*     0     4 */
	int                        scan_hint_start;      /*     4     4 */
	int                        contig_hint;          /*     8     4 */
	int                        contig_hint_start;    /*    12     4 */
	int                        left_free;            /*    16     4 */
	int                        right_free;           /*    20     4 */
	int                        first_free;           /*    24     4 */
	int                        nr_bits;              /*    28     4 */

	/* size: 32, cachelines: 1, members: 8 */
	/* last cacheline: 32 bytes */
};
struct pcpu_chunk {
	struct list_head           list;                 /*     0    16 */
	int                        free_bytes;           /*    16     4 */
	struct pcpu_block_md       chunk_md;             /*    20    32 */

	/* XXX 4 bytes hole, try to pack */

	void *                     base_addr;            /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	long unsigned int *        alloc_map;            /*    64     8 */
	long unsigned int *        bound_map;            /*    72     8 */
	struct pcpu_block_md *     md_blocks;            /*    80     8 */
	void *                     data;                 /*    88     8 */
	bool                       immutable;            /*    96     1 */
	bool                       isolated;             /*    97     1 */

	/* XXX 2 bytes hole, try to pack */

	int                        start_offset;         /*   100     4 */
	int                        end_offset;           /*   104     4 */

	/* XXX 4 bytes hole, try to pack */

	struct obj_cgroup * *      obj_cgroups;          /*   112     8 */
	int                        nr_pages;             /*   120     4 */
	int                        nr_populated;         /*   124     4 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	int                        nr_empty_pop_pages;   /*   128     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          populated[];          /*   136     0 */

	/* size: 136, cachelines: 3, members: 17 */
	/* sum members: 122, holes: 4, sum holes: 14 */
	/* last cacheline: 8 bytes */
};
struct trace_event_raw_kmem_alloc {
	struct trace_entry         ent;                  /*     0     8 */
	long unsigned int          call_site;            /*     8     8 */
	const void  *              ptr;                  /*    16     8 */
	size_t                     bytes_req;            /*    24     8 */
	size_t                     bytes_alloc;          /*    32     8 */
	gfp_t                      gfp_flags;            /*    40     4 */
	char                       __data[];             /*    44     0 */

	/* size: 48, cachelines: 1, members: 7 */
	/* padding: 4 */
	/* last cacheline: 48 bytes */
};
struct trace_event_raw_kmem_alloc_node {
	struct trace_entry         ent;                  /*     0     8 */
	long unsigned int          call_site;            /*     8     8 */
	const void  *              ptr;                  /*    16     8 */
	size_t                     bytes_req;            /*    24     8 */
	size_t                     bytes_alloc;          /*    32     8 */
	gfp_t                      gfp_flags;            /*    40     4 */
	int                        node;                 /*    44     4 */
	char                       __data[];             /*    48     0 */

	/* size: 48, cachelines: 1, members: 8 */
	/* last cacheline: 48 bytes */
};
struct trace_event_raw_kmem_free {
	struct trace_entry         ent;                  /*     0     8 */
	long unsigned int          call_site;            /*     8     8 */
	const void  *              ptr;                  /*    16     8 */
	char                       __data[];             /*    24     0 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_mm_page_free {
	struct trace_entry         ent;                  /*     0     8 */
	long unsigned int          pfn;                  /*     8     8 */
	unsigned int               order;                /*    16     4 */
	char                       __data[];             /*    20     0 */

	/* size: 24, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_mm_page_free_batched {
	struct trace_entry         ent;                  /*     0     8 */
	long unsigned int          pfn;                  /*     8     8 */
	char                       __data[];             /*    16     0 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct trace_event_raw_mm_page_alloc {
	struct trace_entry         ent;                  /*     0     8 */
	long unsigned int          pfn;                  /*     8     8 */
	unsigned int               order;                /*    16     4 */
	gfp_t                      gfp_flags;            /*    20     4 */
	int                        migratetype;          /*    24     4 */
	char                       __data[];             /*    28     0 */

	/* size: 32, cachelines: 1, members: 6 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct trace_event_raw_mm_page {
	struct trace_entry         ent;                  /*     0     8 */
	long unsigned int          pfn;                  /*     8     8 */
	unsigned int               order;                /*    16     4 */
	int                        migratetype;          /*    20     4 */
	char                       __data[];             /*    24     0 */

	/* size: 24, cachelines: 1, members: 5 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_mm_page_pcpu_drain {
	struct trace_entry         ent;                  /*     0     8 */
	long unsigned int          pfn;                  /*     8     8 */
	unsigned int               order;                /*    16     4 */
	int                        migratetype;          /*    20     4 */
	char                       __data[];             /*    24     0 */

	/* size: 24, cachelines: 1, members: 5 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_mm_page_alloc_extfrag {
	struct trace_entry         ent;                  /*     0     8 */
	long unsigned int          pfn;                  /*     8     8 */
	int                        alloc_order;          /*    16     4 */
	int                        fallback_order;       /*    20     4 */
	int                        alloc_migratetype;    /*    24     4 */
	int                        fallback_migratetype; /*    28     4 */
	int                        change_ownership;     /*    32     4 */
	char                       __data[];             /*    36     0 */

	/* size: 40, cachelines: 1, members: 8 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct trace_event_raw_rss_stat {
	struct trace_entry         ent;                  /*     0     8 */
	unsigned int               mm_id;                /*     8     4 */
	unsigned int               curr;                 /*    12     4 */
	int                        member;               /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	long int                   size;                 /*    24     8 */
	char                       __data[];             /*    32     0 */

	/* size: 32, cachelines: 1, members: 6 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct trace_event_data_offsets_kmem_alloc {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_kmem_alloc_node {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_kmem_free {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_mm_page_free {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_mm_page_free_batched {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_mm_page_alloc {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_mm_page {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_mm_page_pcpu_drain {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_mm_page_alloc_extfrag {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_rss_stat {

	/* size: 0, cachelines: 0, members: 0 */
};
struct kmalloc_info_struct {
	const char  *              name[3];              /*     0    24 */
	unsigned int               size;                 /*    24     4 */

	/* size: 32, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct slabinfo {
	long unsigned int          active_objs;          /*     0     8 */
	long unsigned int          num_objs;             /*     8     8 */
	long unsigned int          active_slabs;         /*    16     8 */
	long unsigned int          num_slabs;            /*    24     8 */
	long unsigned int          shared_avail;         /*    32     8 */
	unsigned int               limit;                /*    40     4 */
	unsigned int               batchcount;           /*    44     4 */
	unsigned int               shared;               /*    48     4 */
	unsigned int               objects_per_slab;     /*    52     4 */
	unsigned int               cache_order;          /*    56     4 */

	/* size: 64, cachelines: 1, members: 10 */
	/* padding: 4 */
};
struct alloc_context {
	struct zonelist *          zonelist;             /*     0     8 */
	nodemask_t *               nodemask;             /*     8     8 */
	struct zoneref *           preferred_zoneref;    /*    16     8 */
	int                        migratetype;          /*    24     4 */
	enum zone_type             highest_zoneidx;      /*    28     4 */
	bool                       spread_dirty_pages;   /*    32     1 */

	/* size: 40, cachelines: 1, members: 6 */
	/* padding: 7 */
	/* last cacheline: 40 bytes */
};
struct trace_event_raw_mm_compaction_isolate_template {
	struct trace_entry         ent;                  /*     0     8 */
	long unsigned int          start_pfn;            /*     8     8 */
	long unsigned int          end_pfn;              /*    16     8 */
	long unsigned int          nr_scanned;           /*    24     8 */
	long unsigned int          nr_taken;             /*    32     8 */
	char                       __data[];             /*    40     0 */

	/* size: 40, cachelines: 1, members: 6 */
	/* last cacheline: 40 bytes */
};
struct trace_event_raw_mm_compaction_migratepages {
	struct trace_entry         ent;                  /*     0     8 */
	long unsigned int          nr_migrated;          /*     8     8 */
	long unsigned int          nr_failed;            /*    16     8 */
	char                       __data[];             /*    24     0 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_mm_compaction_begin {
	struct trace_entry         ent;                  /*     0     8 */
	long unsigned int          zone_start;           /*     8     8 */
	long unsigned int          migrate_pfn;          /*    16     8 */
	long unsigned int          free_pfn;             /*    24     8 */
	long unsigned int          zone_end;             /*    32     8 */
	bool                       sync;                 /*    40     1 */
	char                       __data[];             /*    41     0 */

	/* size: 48, cachelines: 1, members: 7 */
	/* padding: 7 */
	/* last cacheline: 48 bytes */
};
struct trace_event_raw_mm_compaction_end {
	struct trace_entry         ent;                  /*     0     8 */
	long unsigned int          zone_start;           /*     8     8 */
	long unsigned int          migrate_pfn;          /*    16     8 */
	long unsigned int          free_pfn;             /*    24     8 */
	long unsigned int          zone_end;             /*    32     8 */
	bool                       sync;                 /*    40     1 */

	/* XXX 3 bytes hole, try to pack */

	int                        status;               /*    44     4 */
	char                       __data[];             /*    48     0 */

	/* size: 48, cachelines: 1, members: 8 */
	/* sum members: 45, holes: 1, sum holes: 3 */
	/* last cacheline: 48 bytes */
};
struct trace_event_raw_mm_compaction_try_to_compact_pages {
	struct trace_entry         ent;                  /*     0     8 */
	int                        order;                /*     8     4 */
	gfp_t                      gfp_mask;             /*    12     4 */
	int                        prio;                 /*    16     4 */
	char                       __data[];             /*    20     0 */

	/* size: 20, cachelines: 1, members: 5 */
	/* last cacheline: 20 bytes */
};
struct trace_event_raw_mm_compaction_suitable_template {
	struct trace_entry         ent;                  /*     0     8 */
	int                        nid;                  /*     8     4 */
	enum zone_type             idx;                  /*    12     4 */
	int                        order;                /*    16     4 */
	int                        ret;                  /*    20     4 */
	char                       __data[];             /*    24     0 */

	/* size: 24, cachelines: 1, members: 6 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_mm_compaction_defer_template {
	struct trace_entry         ent;                  /*     0     8 */
	int                        nid;                  /*     8     4 */
	enum zone_type             idx;                  /*    12     4 */
	int                        order;                /*    16     4 */
	unsigned int               considered;           /*    20     4 */
	unsigned int               defer_shift;          /*    24     4 */
	int                        order_failed;         /*    28     4 */
	char                       __data[];             /*    32     0 */

	/* size: 32, cachelines: 1, members: 8 */
	/* last cacheline: 32 bytes */
};
struct trace_event_raw_mm_compaction_kcompactd_sleep {
	struct trace_entry         ent;                  /*     0     8 */
	int                        nid;                  /*     8     4 */
	char                       __data[];             /*    12     0 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct trace_event_raw_kcompactd_wake_template {
	struct trace_entry         ent;                  /*     0     8 */
	int                        nid;                  /*     8     4 */
	int                        order;                /*    12     4 */
	enum zone_type             highest_zoneidx;      /*    16     4 */
	char                       __data[];             /*    20     0 */

	/* size: 20, cachelines: 1, members: 5 */
	/* last cacheline: 20 bytes */
};
struct trace_event_data_offsets_mm_compaction_isolate_template {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_mm_compaction_migratepages {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_mm_compaction_begin {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_mm_compaction_end {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_mm_compaction_try_to_compact_pages {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_mm_compaction_suitable_template {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_mm_compaction_defer_template {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_mm_compaction_kcompactd_sleep {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_kcompactd_wake_template {

	/* size: 0, cachelines: 0, members: 0 */
};
struct anon_vma_chain {
	struct vm_area_struct *    vma;                  /*     0     8 */
	struct anon_vma *          anon_vma;             /*     8     8 */
	struct list_head           same_vma;             /*    16    16 */
	struct rb_node             rb __attribute__((__aligned__(8))); /*    32    24 */
	long unsigned int          rb_subtree_last;      /*    56     8 */

	/* size: 64, cachelines: 1, members: 5 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(8)));
struct follow_page_context {
	struct dev_pagemap *       pgmap;                /*     0     8 */
	unsigned int               page_mask;            /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct trace_event_raw_mmap_lock_start_locking {
	struct trace_entry         ent;                  /*     0     8 */
	struct mm_struct *         mm;                   /*     8     8 */
	u32                        __data_loc_memcg_path; /*    16     4 */
	bool                       write;                /*    20     1 */
	char                       __data[];             /*    21     0 */

	/* size: 24, cachelines: 1, members: 5 */
	/* padding: 3 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_mmap_lock_acquire_returned {
	struct trace_entry         ent;                  /*     0     8 */
	struct mm_struct *         mm;                   /*     8     8 */
	u32                        __data_loc_memcg_path; /*    16     4 */
	bool                       write;                /*    20     1 */
	bool                       success;              /*    21     1 */
	char                       __data[];             /*    22     0 */

	/* size: 24, cachelines: 1, members: 6 */
	/* padding: 2 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_mmap_lock_released {
	struct trace_entry         ent;                  /*     0     8 */
	struct mm_struct *         mm;                   /*     8     8 */
	u32                        __data_loc_memcg_path; /*    16     4 */
	bool                       write;                /*    20     1 */
	char                       __data[];             /*    21     0 */

	/* size: 24, cachelines: 1, members: 5 */
	/* padding: 3 */
	/* last cacheline: 24 bytes */
};
struct trace_event_data_offsets_mmap_lock_start_locking {
	u32                        memcg_path;           /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct trace_event_data_offsets_mmap_lock_acquire_returned {
	u32                        memcg_path;           /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct trace_event_data_offsets_mmap_lock_released {
	u32                        memcg_path;           /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct memcg_path {
	local_lock_t               lock;                 /*     0     0 */
	char *                     buf;                  /*     0     8 */
	local_t                    buf_idx;              /*     8     8 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct zap_details {
	struct address_space *     check_mapping;        /*     0     8 */
	long unsigned int          first_index;          /*     8     8 */
	long unsigned int          last_index;           /*    16     8 */
	struct page *              single_page;          /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct copy_subpage_arg {
	struct page *              dst;                  /*     0     8 */
	struct page *              src;                  /*     8     8 */
	struct vm_area_struct *    vma;                  /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct mm_walk_ops {
	int                        (*pgd_entry)(pgd_t *, long unsigned int, long unsigned int, struct mm_walk *); /*     0     8 */
	int                        (*p4d_entry)(p4d_t *, long unsigned int, long unsigned int, struct mm_walk *); /*     8     8 */
	int                        (*pud_entry)(pud_t *, long unsigned int, long unsigned int, struct mm_walk *); /*    16     8 */
	int                        (*pmd_entry)(pmd_t *, long unsigned int, long unsigned int, struct mm_walk *); /*    24     8 */
	int                        (*pte_entry)(pte_t *, long unsigned int, long unsigned int, struct mm_walk *); /*    32     8 */
	int                        (*pte_hole)(long unsigned int, long unsigned int, int, struct mm_walk *); /*    40     8 */
	int                        (*hugetlb_entry)(pte_t *, long unsigned int, long unsigned int, long unsigned int, struct mm_walk *); /*    48     8 */
	int                        (*test_walk)(long unsigned int, long unsigned int, struct mm_walk *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*pre_vma)(long unsigned int, long unsigned int, struct mm_walk *); /*    64     8 */
	void                       (*post_vma)(struct mm_walk *); /*    72     8 */

	/* size: 80, cachelines: 2, members: 10 */
	/* last cacheline: 16 bytes */
};
struct mm_walk {
	const struct mm_walk_ops  * ops;                 /*     0     8 */
	struct mm_struct *         mm;                   /*     8     8 */
	pgd_t *                    pgd;                  /*    16     8 */
	struct vm_area_struct *    vma;                  /*    24     8 */
	enum page_walk_action      action;               /*    32     4 */
	bool                       no_vma;               /*    36     1 */

	/* XXX 3 bytes hole, try to pack */

	void *                     private;              /*    40     8 */

	/* size: 48, cachelines: 1, members: 7 */
	/* sum members: 45, holes: 1, sum holes: 3 */
	/* last cacheline: 48 bytes */
};
struct trace_event_raw_vm_unmapped_area {
	struct trace_entry         ent;                  /*     0     8 */
	long unsigned int          addr;                 /*     8     8 */
	long unsigned int          total_vm;             /*    16     8 */
	long unsigned int          flags;                /*    24     8 */
	long unsigned int          length;               /*    32     8 */
	long unsigned int          low_limit;            /*    40     8 */
	long unsigned int          high_limit;           /*    48     8 */
	long unsigned int          align_mask;           /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	long unsigned int          align_offset;         /*    64     8 */
	char                       __data[];             /*    72     0 */

	/* size: 72, cachelines: 2, members: 10 */
	/* last cacheline: 8 bytes */
};
struct trace_event_data_offsets_vm_unmapped_area {

	/* size: 0, cachelines: 0, members: 0 */
};
struct rmap_walk_control {
	void *                     arg;                  /*     0     8 */
	bool                       (*rmap_one)(struct page *, struct vm_area_struct *, long unsigned int, void *); /*     8     8 */
	int                        (*done)(struct page *); /*    16     8 */
	struct anon_vma *          (*anon_lock)(struct page *); /*    24     8 */
	bool                       (*invalid_vma)(struct vm_area_struct *, void *); /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct page_referenced_arg {
	int                        mapcount;             /*     0     4 */
	int                        referenced;           /*     4     4 */
	long unsigned int          vm_flags;             /*     8     8 */
	struct mem_cgroup *        memcg;                /*    16     8 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct vmap_area {
	long unsigned int          va_start;             /*     0     8 */
	long unsigned int          va_end;               /*     8     8 */
	struct rb_node             rb_node __attribute__((__aligned__(8))); /*    16    24 */
	struct list_head           list;                 /*    40    16 */
	union {
		long unsigned int  subtree_max_size;     /*    56     8 */
		struct vm_struct * vm;                   /*    56     8 */
	};                                               /*    56     8 */

	/* size: 64, cachelines: 1, members: 5 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(8)));
struct vfree_deferred {
	struct llist_head          list;                 /*     0     8 */
	struct work_struct         wq;                   /*     8    64 */

	/* size: 72, cachelines: 2, members: 2 */
	/* last cacheline: 8 bytes */
};
struct vmap_block_queue {
	spinlock_t                 lock;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           free;                 /*     8    16 */

	/* size: 24, cachelines: 1, members: 2 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct vmap_block {
	spinlock_t                 lock;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct vmap_area *         va;                   /*     8     8 */
	long unsigned int          free;                 /*    16     8 */
	long unsigned int          dirty;                /*    24     8 */
	long unsigned int          dirty_min;            /*    32     8 */
	long unsigned int          dirty_max;            /*    40     8 */
	struct list_head           free_list;            /*    48    16 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct callback_head       callback_head __attribute__((__aligned__(8))); /*    64    16 */
	struct list_head           purge;                /*    80    16 */

	/* size: 96, cachelines: 2, members: 9 */
	/* sum members: 92, holes: 1, sum holes: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct vmap_pfn_data {
	long unsigned int *        pfns;                 /*     0     8 */
	pgprot_t                   prot;                 /*     8     8 */
	unsigned int               idx;                  /*    16     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct page_frag_cache {
	void *                     va;                   /*     0     8 */
	__u16                      offset;               /*     8     2 */
	__u16                      size;                 /*    10     2 */
	unsigned int               pagecnt_bias;         /*    12     4 */
	bool                       pfmemalloc;           /*    16     1 */

	/* size: 24, cachelines: 1, members: 5 */
	/* padding: 7 */
	/* last cacheline: 24 bytes */
};
struct mminit_pfnnid_cache {
	long unsigned int          last_start;           /*     0     8 */
	long unsigned int          last_end;             /*     8     8 */
	int                        last_nid;             /*    16     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct migration_target_control {
	int                        nid;                  /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	nodemask_t *               nmask;                /*     8     8 */
	gfp_t                      gfp_mask;             /*    16     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 16, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct pcpu_drain {
	struct zone *              zone;                 /*     0     8 */
	struct work_struct         work;                 /*     8    64 */

	/* size: 72, cachelines: 2, members: 2 */
	/* last cacheline: 8 bytes */
};
struct madvise_walk_private {
	struct mmu_gather *        tlb;                  /*     0     8 */
	bool                       pageout;              /*     8     1 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 7 */
	/* last cacheline: 16 bytes */
};
struct vma_swap_readahead {
	short unsigned int         win;                  /*     0     2 */
	short unsigned int         offset;               /*     2     2 */
	short unsigned int         nr_pte;               /*     4     2 */

	/* XXX 2 bytes hole, try to pack */

	pte_t *                    ptes;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 4 */
	/* sum members: 14, holes: 1, sum holes: 2 */
	/* last cacheline: 16 bytes */
};
union swap_header {
	struct {
		char               reserved[4086];     /*     0  4086 */
		/* --- cacheline 63 boundary (4032 bytes) was 54 bytes ago --- */
		char               magic[10];          /*  4086    10 */
	} magic;                                       /*     0  4096 */
	struct {
		char               bootbits[1024];     /*     0  1024 */
		/* --- cacheline 16 boundary (1024 bytes) --- */
		__u32              version;            /*  1024     4 */
		__u32              last_page;          /*  1028     4 */
		__u32              nr_badpages;        /*  1032     4 */
		unsigned char      sws_uuid[16];       /*  1036    16 */
		unsigned char      sws_volume[16];     /*  1052    16 */
		__u32              padding[117];       /*  1068   468 */
		/* --- cacheline 24 boundary (1536 bytes) --- */
		__u32              badpages[1];        /*  1536     4 */
	} info;                                        /*     0  1540 */
};
struct swap_extent {
	struct rb_node             rb_node __attribute__((__aligned__(8))); /*     0    24 */
	long unsigned int          start_page;           /*    24     8 */
	long unsigned int          nr_pages;             /*    32     8 */
	sector_t                   start_block;          /*    40     8 */

	/* size: 48, cachelines: 1, members: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct swap_slots_cache {
	bool                       lock_initialized;     /*     0     1 */

	/* XXX 7 bytes hole, try to pack */

	struct mutex               alloc_lock;           /*     8    32 */
	swp_entry_t *              slots;                /*    40     8 */
	int                        nr;                   /*    48     4 */
	int                        cur;                  /*    52     4 */
	spinlock_t                 free_lock;            /*    56     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	swp_entry_t *              slots_ret;            /*    64     8 */
	int                        n_ret;                /*    72     4 */

	/* size: 80, cachelines: 2, members: 8 */
	/* sum members: 65, holes: 2, sum holes: 11 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct frontswap_ops {
	void                       (*init)(unsigned int); /*     0     8 */
	int                        (*store)(unsigned int, long unsigned int, struct page *); /*     8     8 */
	int                        (*load)(unsigned int, long unsigned int, struct page *); /*    16     8 */
	void                       (*invalidate_page)(unsigned int, long unsigned int); /*    24     8 */
	void                       (*invalidate_area)(unsigned int); /*    32     8 */
	struct frontswap_ops *     next;                 /*    40     8 */

	/* size: 48, cachelines: 1, members: 6 */
	/* last cacheline: 48 bytes */
};
struct crypto_wait {
	struct completion          completion;           /*     0    32 */
	int                        err;                  /*    32     4 */

	/* size: 40, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct zpool_ops {
	int                        (*evict)(struct zpool *, long unsigned int); /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct acomp_req {
	struct crypto_async_request base;                /*     0    48 */

	/* XXX last struct has 4 bytes of padding */

	struct scatterlist *       src;                  /*    48     8 */
	struct scatterlist *       dst;                  /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	unsigned int               slen;                 /*    64     4 */
	unsigned int               dlen;                 /*    68     4 */
	u32                        flags;                /*    72     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     __ctx[] __attribute__((__aligned__(8))); /*    80     0 */

	/* size: 80, cachelines: 2, members: 7 */
	/* sum members: 76, holes: 1, sum holes: 4 */
	/* paddings: 1, sum paddings: 4 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 4 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct crypto_acomp {
	int                        (*compress)(struct acomp_req *); /*     0     8 */
	int                        (*decompress)(struct acomp_req *); /*     8     8 */
	void                       (*dst_free)(struct scatterlist *); /*    16     8 */
	unsigned int               reqsize;              /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	struct crypto_tfm          base __attribute__((__aligned__(8))); /*    32    24 */

	/* size: 56, cachelines: 1, members: 5 */
	/* sum members: 52, holes: 1, sum holes: 4 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 4 */
	/* last cacheline: 56 bytes */
} __attribute__((__aligned__(8)));
struct crypto_acomp_ctx {
	struct crypto_acomp *      acomp;                /*     0     8 */
	struct acomp_req *         req;                  /*     8     8 */
	struct crypto_wait         wait;                 /*    16    40 */

	/* XXX last struct has 4 bytes of padding */

	u8 *                       dstmem;               /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct mutex *             mutex;                /*    64     8 */

	/* size: 72, cachelines: 2, members: 5 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 8 bytes */
};
struct zswap_pool {
	struct zpool *             zpool;                /*     0     8 */
	struct crypto_acomp_ctx *  acomp_ctx;            /*     8     8 */
	struct kref                kref;                 /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           list;                 /*    24    16 */
	struct work_struct         release_work;         /*    40    64 */
	/* --- cacheline 1 boundary (64 bytes) was 40 bytes ago --- */
	struct work_struct         shrink_work;          /*   104    64 */
	/* --- cacheline 2 boundary (128 bytes) was 40 bytes ago --- */
	struct hlist_node          node;                 /*   168    16 */
	char                       tfm_name[128];        /*   184   128 */

	/* size: 312, cachelines: 5, members: 8 */
	/* sum members: 308, holes: 1, sum holes: 4 */
	/* last cacheline: 56 bytes */
};
struct zswap_entry {
	struct rb_node             rbnode __attribute__((__aligned__(8))); /*     0    24 */
	long unsigned int          offset;               /*    24     8 */
	int                        refcount;             /*    32     4 */
	unsigned int               length;               /*    36     4 */
	struct zswap_pool *        pool;                 /*    40     8 */
	union {
		long unsigned int  handle;               /*    48     8 */
		long unsigned int  value;                /*    48     8 */
	};                                               /*    48     8 */

	/* size: 56, cachelines: 1, members: 6 */
	/* forced alignments: 1 */
	/* last cacheline: 56 bytes */
} __attribute__((__aligned__(8)));
struct zswap_header {
	swp_entry_t                swpentry;             /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct zswap_tree {
	struct rb_root             rbroot;               /*     0     8 */
	spinlock_t                 lock;                 /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct dma_pool {
	struct list_head           page_list;            /*     0    16 */
	spinlock_t                 lock;                 /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	size_t                     size;                 /*    24     8 */
	struct device *            dev;                  /*    32     8 */
	size_t                     allocation;           /*    40     8 */
	size_t                     boundary;             /*    48     8 */
	char                       name[32];             /*    56    32 */
	/* --- cacheline 1 boundary (64 bytes) was 24 bytes ago --- */
	struct list_head           pools;                /*    88    16 */

	/* size: 104, cachelines: 2, members: 8 */
	/* sum members: 100, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct dma_page {
	struct list_head           page_list;            /*     0    16 */
	void *                     vaddr;                /*    16     8 */
	dma_addr_t                 dma;                  /*    24     8 */
	unsigned int               in_use;               /*    32     4 */
	unsigned int               offset;               /*    36     4 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct resv_map {
	struct kref                refs;                 /*     0     4 */
	spinlock_t                 lock;                 /*     4     4 */
	struct list_head           regions;              /*     8    16 */
	long int                   adds_in_progress;     /*    24     8 */
	struct list_head           region_cache;         /*    32    16 */
	long int                   region_cache_count;   /*    48     8 */
	struct page_counter *      reservation_counter;  /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	long unsigned int          pages_per_hpage;      /*    64     8 */
	struct cgroup_subsys_state * css;                /*    72     8 */

	/* size: 80, cachelines: 2, members: 9 */
	/* last cacheline: 16 bytes */
};
struct file_region {
	struct list_head           link;                 /*     0    16 */
	long int                   from;                 /*    16     8 */
	long int                   to;                   /*    24     8 */
	struct page_counter *      reservation_counter;  /*    32     8 */
	struct cgroup_subsys_state * css;                /*    40     8 */

	/* size: 48, cachelines: 1, members: 5 */
	/* last cacheline: 48 bytes */
};
struct hugetlbfs_inode_info {
	struct shared_policy       policy;               /*     0    16 */
	struct inode               vfs_inode __attribute__((__aligned__(8))); /*    16   624 */
	/* --- cacheline 10 boundary (640 bytes) --- */
	unsigned int               seals;                /*   640     4 */

	/* XXX 4 bytes hole, try to pack */

	struct dhugetlb_pool *     hpool;                /*   648     8 */

	/* size: 656, cachelines: 11, members: 4 */
	/* sum members: 652, holes: 1, sum holes: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct huge_bootmem_page {
	struct list_head           list;                 /*     0    16 */
	struct hstate *            hstate;               /*    16     8 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct hugetlb_cgroup {
	struct cgroup_subsys_state css __attribute__((__aligned__(8))); /*     0   296 */
	/* --- cacheline 4 boundary (256 bytes) was 40 bytes ago --- */
	struct page_counter        hugepage[2];          /*   296   224 */
	/* --- cacheline 8 boundary (512 bytes) was 8 bytes ago --- */
	struct page_counter        rsvd_hugepage[2];     /*   520   224 */
	/* --- cacheline 11 boundary (704 bytes) was 40 bytes ago --- */
	atomic_long_t              events[2][1];         /*   744    16 */
	atomic_long_t              events_local[2][1];   /*   760    16 */
	/* --- cacheline 12 boundary (768 bytes) was 8 bytes ago --- */
	struct cgroup_file         events_file[2];       /*   776   208 */
	/* --- cacheline 15 boundary (960 bytes) was 24 bytes ago --- */
	struct cgroup_file         events_local_file[2]; /*   984   208 */

	/* size: 1192, cachelines: 19, members: 7 */
	/* forced alignments: 1 */
	/* last cacheline: 40 bytes */
} __attribute__((__aligned__(8)));
struct node_hstate {
	struct kobject *           hugepages_kobj;       /*     0     8 */
	struct kobject *           hstate_kobjs[2];      /*     8    16 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct split_hugepage {
	struct list_head           head_pages;           /*     0    16 */
	long unsigned int          start_pfn;            /*    16     8 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_dynamic_hugetlb_split_merge {
	struct trace_entry         ent;                  /*     0     8 */
	const void  *              hpool;                /*     8     8 */
	long unsigned int          pfn;                  /*    16     8 */
	long unsigned int          action;               /*    24     8 */
	long unsigned int          size;                 /*    32     8 */
	char                       __data[];             /*    40     0 */

	/* size: 40, cachelines: 1, members: 6 */
	/* last cacheline: 40 bytes */
};
struct trace_event_raw_dynamic_hugetlb_acct_memory {
	struct trace_entry         ent;                  /*     0     8 */
	const void  *              hpool;                /*     8     8 */
	long unsigned int          count;                /*    16     8 */
	long unsigned int          action;               /*    24     8 */
	long unsigned int          size;                 /*    32     8 */
	char                       __data[];             /*    40     0 */

	/* size: 40, cachelines: 1, members: 6 */
	/* last cacheline: 40 bytes */
};
struct trace_event_raw_dynamic_hugetlb_alloc_free {
	struct trace_entry         ent;                  /*     0     8 */
	const void  *              hpool;                /*     8     8 */
	long unsigned int          pfn;                  /*    16     8 */
	long unsigned int          count;                /*    24     8 */
	long unsigned int          action;               /*    32     8 */
	long unsigned int          size;                 /*    40     8 */
	char                       __data[];             /*    48     0 */

	/* size: 48, cachelines: 1, members: 7 */
	/* last cacheline: 48 bytes */
};
struct trace_event_data_offsets_dynamic_hugetlb_split_merge {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_dynamic_hugetlb_acct_memory {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_dynamic_hugetlb_alloc_free {

	/* size: 0, cachelines: 0, members: 0 */
};
struct dhugetlb_pagelist {
	long unsigned int          count;                /*     0     8 */
	struct dhugetlb_pool *     hpool[];              /*     8     0 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct nodemask_scratch {
	nodemask_t                 mask1;                /*     0   128 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	nodemask_t                 mask2;                /*   128   128 */

	/* size: 256, cachelines: 4, members: 2 */
};
struct sp_node {
	struct rb_node             nd __attribute__((__aligned__(8))); /*     0    24 */
	long unsigned int          start;                /*    24     8 */
	long unsigned int          end;                  /*    32     8 */
	struct mempolicy *         policy;               /*    40     8 */

	/* size: 48, cachelines: 1, members: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct mempolicy_operations {
	int                        (*create)(struct mempolicy *, const nodemask_t  *); /*     0     8 */
	void                       (*rebind)(struct mempolicy *, const nodemask_t  *); /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct queue_pages {
	struct list_head *         pagelist;             /*     0     8 */
	long unsigned int          flags;                /*     8     8 */
	nodemask_t *               nmask;                /*    16     8 */
	long unsigned int          start;                /*    24     8 */
	long unsigned int          end;                  /*    32     8 */
	struct vm_area_struct *    first;                /*    40     8 */

	/* size: 48, cachelines: 1, members: 6 */
	/* last cacheline: 48 bytes */
};
struct vmemmap_remap_walk {
	void                       (*remap_pte)(pte_t *, long unsigned int, struct vmemmap_remap_walk *); /*     0     8 */
	long unsigned int          nr_walked;            /*     8     8 */
	struct page *              reuse_page;           /*    16     8 */
	long unsigned int          reuse_addr;           /*    24     8 */
	struct list_head *         vmemmap_pages;        /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct mmu_notifier_subscriptions {
	struct hlist_head          list;                 /*     0     8 */
	bool                       has_itree;            /*     8     1 */

	/* XXX 3 bytes hole, try to pack */

	spinlock_t                 lock;                 /*    12     4 */
	long unsigned int          invalidate_seq;       /*    16     8 */
	long unsigned int          active_invalidate_ranges; /*    24     8 */
	struct rb_root_cached      itree;                /*    32    16 */
	wait_queue_head_t          wq;                   /*    48    24 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	struct hlist_head          deferred_list;        /*    72     8 */

	/* size: 80, cachelines: 2, members: 8 */
	/* sum members: 77, holes: 1, sum holes: 3 */
	/* last cacheline: 16 bytes */
};
struct interval_tree_node {
	struct rb_node             rb __attribute__((__aligned__(8))); /*     0    24 */
	long unsigned int          start;                /*    24     8 */
	long unsigned int          last;                 /*    32     8 */
	long unsigned int          __subtree_last;       /*    40     8 */

	/* size: 48, cachelines: 1, members: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct mmu_interval_notifier_ops {
	bool                       (*invalidate)(struct mmu_interval_notifier *, const struct mmu_notifier_range  *, long unsigned int); /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct mmu_interval_notifier {
	struct interval_tree_node  interval_tree __attribute__((__aligned__(8))); /*     0    48 */
	const struct mmu_interval_notifier_ops  * ops;   /*    48     8 */
	struct mm_struct *         mm;                   /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct hlist_node          deferred_item;        /*    64    16 */
	long unsigned int          invalidate_seq;       /*    80     8 */

	/* size: 88, cachelines: 2, members: 5 */
	/* forced alignments: 1 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct memory_notify {
	long unsigned int          start_pfn;            /*     0     8 */
	long unsigned int          nr_pages;             /*     8     8 */
	int                        status_change_nid_normal; /*    16     4 */
	int                        status_change_nid_high; /*    20     4 */
	int                        status_change_nid;    /*    24     4 */

	/* size: 32, cachelines: 1, members: 5 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct mm_slot {
	struct hlist_node          link;                 /*     0    16 */
	struct list_head           mm_list;              /*    16    16 */
	struct rmap_item *         rmap_list;            /*    32     8 */
	struct mm_struct *         mm;                   /*    40     8 */

	/* size: 48, cachelines: 1, members: 4 */
	/* last cacheline: 48 bytes */
};
struct rmap_item {
	struct rmap_item *         rmap_list;            /*     0     8 */
	union {
		struct anon_vma *  anon_vma;             /*     8     8 */
		int                nid;                  /*     8     4 */
	};                                               /*     8     8 */
	struct mm_struct *         mm;                   /*    16     8 */
	long unsigned int          address;              /*    24     8 */
	unsigned int               oldchecksum;          /*    32     4 */

	/* XXX 4 bytes hole, try to pack */

	union {
		struct rb_node     node __attribute__((__aligned__(8))); /*    40    24 */
		struct {
			struct stable_node * head;       /*    40     8 */
			struct hlist_node hlist;         /*    48    16 */
		};                                       /*    40    24 */
	} __attribute__((__aligned__(8)));               /*    40    24 */

	/* size: 64, cachelines: 1, members: 6 */
	/* sum members: 60, holes: 1, sum holes: 4 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 4 */
} __attribute__((__aligned__(8)));
struct ksm_scan {
	struct mm_slot *           mm_slot;              /*     0     8 */
	long unsigned int          address;              /*     8     8 */
	struct rmap_item * *       rmap_list;            /*    16     8 */
	long unsigned int          seqnr;                /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct stable_node {
	union {
		struct rb_node     node __attribute__((__aligned__(8))); /*     0    24 */
		struct {
			struct list_head * head;         /*     0     8 */
			struct {
				struct hlist_node hlist_dup; /*     8    16 */
				struct list_head list;   /*    24    16 */
			};                               /*     8    32 */
		};                                       /*     0    40 */
	} __attribute__((__aligned__(8)));               /*     0    40 */
	struct hlist_head          hlist;                /*    40     8 */
	union {
		long unsigned int  kpfn;                 /*    48     8 */
		long unsigned int  chain_prune_time;     /*    48     8 */
	};                                               /*    48     8 */
	int                        rmap_hlist_len;       /*    56     4 */
	int                        nid;                  /*    60     4 */

	/* size: 64, cachelines: 1, members: 5 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(8)));
struct track {
	long unsigned int          addr;                 /*     0     8 */
	long unsigned int          addrs[16];            /*     8   128 */
	/* --- cacheline 2 boundary (128 bytes) was 8 bytes ago --- */
	int                        cpu;                  /*   136     4 */
	int                        pid;                  /*   140     4 */
	long unsigned int          when;                 /*   144     8 */

	/* size: 152, cachelines: 3, members: 5 */
	/* last cacheline: 24 bytes */
};
struct detached_freelist {
	struct page *              page;                 /*     0     8 */
	void *                     tail;                 /*     8     8 */
	void *                     freelist;             /*    16     8 */
	int                        cnt;                  /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	struct kmem_cache *        s;                    /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* sum members: 36, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct location {
	long unsigned int          count;                /*     0     8 */
	long unsigned int          addr;                 /*     8     8 */
	long long int              sum_time;             /*    16     8 */
	long int                   min_time;             /*    24     8 */
	long int                   max_time;             /*    32     8 */
	long int                   min_pid;              /*    40     8 */
	long int                   max_pid;              /*    48     8 */
	long unsigned int          cpus[128];            /*    56  1024 */
	/* --- cacheline 16 boundary (1024 bytes) was 56 bytes ago --- */
	nodemask_t                 nodes;                /*  1080   128 */

	/* size: 1208, cachelines: 19, members: 9 */
	/* last cacheline: 56 bytes */
};
struct loc_track {
	long unsigned int          max;                  /*     0     8 */
	long unsigned int          count;                /*     8     8 */
	struct location *          loc;                  /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct slab_attribute {
	struct attribute           attr;                 /*     0    16 */

	/* XXX last struct has 6 bytes of padding */

	ssize_t                    (*show)(struct kmem_cache *, char *); /*    16     8 */
	ssize_t                    (*store)(struct kmem_cache *, const char  *, size_t); /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* paddings: 1, sum paddings: 6 */
	/* last cacheline: 32 bytes */
};
struct saved_alias {
	struct kmem_cache *        s;                    /*     0     8 */
	const char  *              name;                 /*     8     8 */
	struct saved_alias *       next;                 /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct memory_block {
	long unsigned int          start_section_nr;     /*     0     8 */
	long unsigned int          state;                /*     8     8 */
	int                        online_type;          /*    16     4 */
	int                        nid;                  /*    20     4 */
	struct device              dev __attribute__((__aligned__(8))); /*    24  1000 */

	/* size: 1024, cachelines: 16, members: 5 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(8)));
struct buffer_head {
	long unsigned int          b_state;              /*     0     8 */
	struct buffer_head *       b_this_page;          /*     8     8 */
	struct page *              b_page;               /*    16     8 */
	sector_t                   b_blocknr;            /*    24     8 */
	size_t                     b_size;               /*    32     8 */
	char *                     b_data;               /*    40     8 */
	struct block_device *      b_bdev;               /*    48     8 */
	bh_end_io_t *              b_end_io;             /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	void *                     b_private;            /*    64     8 */
	struct list_head           b_assoc_buffers;      /*    72    16 */
	struct address_space *     b_assoc_map;          /*    88     8 */
	atomic_t                   b_count;              /*    96     4 */
	spinlock_t                 b_uptodate_lock;      /*   100     4 */

	/* size: 104, cachelines: 2, members: 13 */
	/* last cacheline: 40 bytes */
};
struct migrate_vma {
	struct vm_area_struct *    vma;                  /*     0     8 */
	long unsigned int *        dst;                  /*     8     8 */
	long unsigned int *        src;                  /*    16     8 */
	long unsigned int          cpages;               /*    24     8 */
	long unsigned int          npages;               /*    32     8 */
	long unsigned int          start;                /*    40     8 */
	long unsigned int          end;                  /*    48     8 */
	void *                     pgmap_owner;          /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	long unsigned int          flags;                /*    64     8 */
	struct page *              fault_page;           /*    72     8 */

	/* size: 80, cachelines: 2, members: 10 */
	/* last cacheline: 16 bytes */
};
struct trace_event_raw_mm_migrate_pages {
	struct trace_entry         ent;                  /*     0     8 */
	long unsigned int          succeeded;            /*     8     8 */
	long unsigned int          failed;               /*    16     8 */
	long unsigned int          thp_succeeded;        /*    24     8 */
	long unsigned int          thp_failed;           /*    32     8 */
	long unsigned int          thp_split;            /*    40     8 */
	enum migrate_mode          mode;                 /*    48     4 */
	int                        reason;               /*    52     4 */
	char                       __data[];             /*    56     0 */

	/* size: 56, cachelines: 1, members: 9 */
	/* last cacheline: 56 bytes */
};
struct trace_event_data_offsets_mm_migrate_pages {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_raw_mm_khugepaged_scan_pmd {
	struct trace_entry         ent;                  /*     0     8 */
	struct mm_struct *         mm;                   /*     8     8 */
	long unsigned int          pfn;                  /*    16     8 */
	bool                       writable;             /*    24     1 */

	/* XXX 3 bytes hole, try to pack */

	int                        referenced;           /*    28     4 */
	int                        none_or_zero;         /*    32     4 */
	int                        status;               /*    36     4 */
	int                        unmapped;             /*    40     4 */
	char                       __data[];             /*    44     0 */

	/* size: 48, cachelines: 1, members: 9 */
	/* sum members: 41, holes: 1, sum holes: 3 */
	/* padding: 4 */
	/* last cacheline: 48 bytes */
};
struct trace_event_raw_mm_collapse_huge_page {
	struct trace_entry         ent;                  /*     0     8 */
	struct mm_struct *         mm;                   /*     8     8 */
	int                        isolated;             /*    16     4 */
	int                        status;               /*    20     4 */
	char                       __data[];             /*    24     0 */

	/* size: 24, cachelines: 1, members: 5 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_mm_collapse_huge_page_isolate {
	struct trace_entry         ent;                  /*     0     8 */
	long unsigned int          pfn;                  /*     8     8 */
	int                        none_or_zero;         /*    16     4 */
	int                        referenced;           /*    20     4 */
	bool                       writable;             /*    24     1 */

	/* XXX 3 bytes hole, try to pack */

	int                        status;               /*    28     4 */
	char                       __data[];             /*    32     0 */

	/* size: 32, cachelines: 1, members: 7 */
	/* sum members: 29, holes: 1, sum holes: 3 */
	/* last cacheline: 32 bytes */
};
struct trace_event_raw_mm_collapse_huge_page_swapin {
	struct trace_entry         ent;                  /*     0     8 */
	struct mm_struct *         mm;                   /*     8     8 */
	int                        swapped_in;           /*    16     4 */
	int                        referenced;           /*    20     4 */
	int                        ret;                  /*    24     4 */
	char                       __data[];             /*    28     0 */

	/* size: 32, cachelines: 1, members: 6 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct trace_event_data_offsets_mm_khugepaged_scan_pmd {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_mm_collapse_huge_page {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_mm_collapse_huge_page_isolate {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_mm_collapse_huge_page_swapin {

	/* size: 0, cachelines: 0, members: 0 */
};
struct mm_slot {
	struct hlist_node          hash;                 /*     0    16 */
	struct list_head           mm_node;              /*    16    16 */
	struct mm_struct *         mm;                   /*    32     8 */
	int                        nr_pte_mapped_thp;    /*    40     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          pte_mapped_thp[8];    /*    48    64 */

	/* size: 112, cachelines: 2, members: 5 */
	/* sum members: 108, holes: 1, sum holes: 4 */
	/* last cacheline: 48 bytes */
};
struct khugepaged_scan {
	struct list_head           mm_head;              /*     0    16 */
	struct mm_slot *           mm_slot;              /*    16     8 */
	long unsigned int          address;              /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* last cacheline: 32 bytes */
};
struct mem_cgroup_reclaim_cookie {
	pg_data_t *                pgdat;                /*     0     8 */
	unsigned int               generation;           /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct mem_cgroup_tree_per_node {
	struct rb_root             rb_root;              /*     0     8 */
	struct rb_node *           rb_rightmost;         /*     8     8 */
	spinlock_t                 lock;                 /*    16     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct mem_cgroup_tree {
	struct mem_cgroup_tree_per_node * rb_tree_per_node[1024]; /*     0  8192 */

	/* size: 8192, cachelines: 128, members: 1 */
};
struct mem_cgroup_eventfd_list {
	struct list_head           list;                 /*     0    16 */
	struct eventfd_ctx *       eventfd;              /*    16     8 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct mem_cgroup_event {
	struct mem_cgroup *        memcg;                /*     0     8 */
	struct eventfd_ctx *       eventfd;              /*     8     8 */
	struct list_head           list;                 /*    16    16 */
	int                        (*register_event)(struct mem_cgroup *, struct eventfd_ctx *, const char  *); /*    32     8 */
	void                       (*unregister_event)(struct mem_cgroup *, struct eventfd_ctx *); /*    40     8 */
	poll_table                 pt;                   /*    48    16 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	wait_queue_head_t *        wqh;                  /*    64     8 */
	wait_queue_entry_t         wait;                 /*    72    40 */
	struct work_struct         remove;               /*   112    64 */

	/* size: 176, cachelines: 3, members: 9 */
	/* last cacheline: 48 bytes */
};
struct move_charge_struct {
	spinlock_t                 lock;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct mm_struct *         mm;                   /*     8     8 */
	struct mem_cgroup *        from;                 /*    16     8 */
	struct mem_cgroup *        to;                   /*    24     8 */
	long unsigned int          flags;                /*    32     8 */
	long unsigned int          precharge;            /*    40     8 */
	long unsigned int          moved_charge;         /*    48     8 */
	long unsigned int          moved_swap;           /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct task_struct *       moving_task;          /*    64     8 */
	wait_queue_head_t          waitq;                /*    72    24 */

	/* size: 96, cachelines: 2, members: 10 */
	/* sum members: 92, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct memory_stat {
	const char  *              name;                 /*     0     8 */
	unsigned int               ratio;                /*     8     4 */
	unsigned int               idx;                  /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct oom_wait_info {
	struct mem_cgroup *        memcg;                /*     0     8 */
	wait_queue_entry_t         wait;                 /*     8    40 */

	/* size: 48, cachelines: 1, members: 2 */
	/* last cacheline: 48 bytes */
};
struct memcg_stock_pcp {
	struct mem_cgroup *        cached;               /*     0     8 */
	unsigned int               nr_pages;             /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct obj_cgroup *        cached_objcg;         /*    16     8 */
	unsigned int               nr_bytes;             /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	struct work_struct         work;                 /*    32    64 */
	/* --- cacheline 1 boundary (64 bytes) was 32 bytes ago --- */
	long unsigned int          flags;                /*    96     8 */

	/* size: 104, cachelines: 2, members: 6 */
	/* sum members: 96, holes: 2, sum holes: 8 */
	/* last cacheline: 40 bytes */
};
union mc_target {
	struct page *              page;               /*     0     8 */
	swp_entry_t                ent;                /*     0     8 */
};
struct uncharge_gather {
	struct mem_cgroup *        memcg;                /*     0     8 */
	long unsigned int          nr_memory;            /*     8     8 */
	long unsigned int          pgpgout;              /*    16     8 */
	long unsigned int          nr_kmem;              /*    24     8 */
	struct page *              dummy_page;           /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct vmpressure_event {
	struct eventfd_ctx *       efd;                  /*     0     8 */
	enum vmpressure_levels     level;                /*     8     4 */
	enum vmpressure_modes      mode;                 /*    12     4 */
	struct list_head           node;                 /*    16    16 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct swap_cgroup_ctrl {
	struct page * *            map;                  /*     0     8 */
	long unsigned int          length;               /*     8     8 */
	spinlock_t                 lock;                 /*    16     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct swap_cgroup {
	short unsigned int         id;                   /*     0     2 */

	/* size: 2, cachelines: 1, members: 1 */
	/* last cacheline: 2 bytes */
};
struct __kfifo {
	unsigned int               in;                   /*     0     4 */
	unsigned int               out;                  /*     4     4 */
	unsigned int               mask;                 /*     8     4 */
	unsigned int               esize;                /*    12     4 */
	void *                     data;                 /*    16     8 */

	/* size: 24, cachelines: 1, members: 5 */
	/* last cacheline: 24 bytes */
};
struct to_kill {
	struct list_head           nd;                   /*     0    16 */
	struct task_struct *       tsk;                  /*    16     8 */
	long unsigned int          addr;                 /*    24     8 */
	short int                  size_shift;           /*    32     2 */

	/* size: 40, cachelines: 1, members: 4 */
	/* padding: 6 */
	/* last cacheline: 40 bytes */
};
struct hwp_walk {
	struct to_kill             tk;                   /*     0    40 */

	/* XXX last struct has 6 bytes of padding */

	long unsigned int          pfn;                  /*    40     8 */
	int                        flags;                /*    48     4 */

	/* size: 56, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* paddings: 1, sum paddings: 6 */
	/* last cacheline: 56 bytes */
};
struct page_state {
	long unsigned int          mask;                 /*     0     8 */
	long unsigned int          res;                  /*     8     8 */
	enum mf_action_page_type   type;                 /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	int                        (*action)(struct page *, long unsigned int); /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct memory_failure_entry {
	long unsigned int          pfn;                  /*     0     8 */
	int                        flags;                /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct memory_failure_cpu {
	struct {
		union {
			struct __kfifo kfifo;            /*     0    24 */
			struct memory_failure_entry * type; /*     0     8 */
			const struct memory_failure_entry  * const_type; /*     0     8 */
			char *     rectype;              /*     0     8 */
			struct memory_failure_entry * ptr; /*     0     8 */
			const struct memory_failure_entry  * ptr_const; /*     0     8 */
		};                                       /*     0    24 */
		struct memory_failure_entry buf[16];     /*    24   256 */
	} fifo;                                          /*     0   280 */
	/* --- cacheline 4 boundary (256 bytes) was 24 bytes ago --- */
	spinlock_t                 lock;                 /*   280     4 */

	/* XXX 4 bytes hole, try to pack */

	struct work_struct         work;                 /*   288    64 */

	/* size: 352, cachelines: 6, members: 3 */
	/* sum members: 348, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct cleancache_filekey {
	union {
		ino_t              ino;                  /*     0     8 */
		__u32              fh[6];                /*     0    24 */
		u32                key[6];               /*     0    24 */
	} u;                                             /*     0    24 */

	/* size: 24, cachelines: 1, members: 1 */
	/* last cacheline: 24 bytes */
};
struct cleancache_ops {
	int                        (*init_fs)(size_t);   /*     0     8 */
	int                        (*init_shared_fs)(uuid_t *, size_t); /*     8     8 */
	int                        (*get_page)(int, struct cleancache_filekey, long unsigned int, struct page *); /*    16     8 */
	void                       (*put_page)(int, struct cleancache_filekey, long unsigned int, struct page *); /*    24     8 */
	void                       (*invalidate_page)(int, struct cleancache_filekey, long unsigned int); /*    32     8 */
	void                       (*invalidate_inode)(int, struct cleancache_filekey); /*    40     8 */
	void                       (*invalidate_fs)(int); /*    48     8 */

	/* size: 56, cachelines: 1, members: 7 */
	/* last cacheline: 56 bytes */
};
struct trace_event_raw_test_pages_isolated {
	struct trace_entry         ent;                  /*     0     8 */
	long unsigned int          start_pfn;            /*     8     8 */
	long unsigned int          end_pfn;              /*    16     8 */
	long unsigned int          fin_pfn;              /*    24     8 */
	char                       __data[];             /*    32     0 */

	/* size: 32, cachelines: 1, members: 5 */
	/* last cacheline: 32 bytes */
};
struct trace_event_data_offsets_test_pages_isolated {

	/* size: 0, cachelines: 0, members: 0 */
};
struct zpool {
	struct zpool_driver *      driver;               /*     0     8 */
	void *                     pool;                 /*     8     8 */
	const struct zpool_ops  *  ops;                  /*    16     8 */
	bool                       evictable;            /*    24     1 */
	bool                       can_sleep_mapped;     /*    25     1 */

	/* XXX 6 bytes hole, try to pack */

	struct list_head           list;                 /*    32    16 */

	/* size: 48, cachelines: 1, members: 6 */
	/* sum members: 42, holes: 1, sum holes: 6 */
	/* last cacheline: 48 bytes */
};
struct zpool_driver {
	char *                     type;                 /*     0     8 */
	struct module *            owner;                /*     8     8 */
	atomic_t                   refcount;             /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           list;                 /*    24    16 */
	void *                     (*create)(const char  *, gfp_t, const struct zpool_ops  *, struct zpool *); /*    40     8 */
	void                       (*destroy)(void *);   /*    48     8 */
	bool                       malloc_support_movable; /*    56     1 */

	/* XXX 7 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*malloc)(void *, size_t, gfp_t, long unsigned int *); /*    64     8 */
	void                       (*free)(void *, long unsigned int); /*    72     8 */
	int                        (*shrink)(void *, unsigned int, unsigned int *); /*    80     8 */
	bool                       sleep_mapped;         /*    88     1 */

	/* XXX 7 bytes hole, try to pack */

	void *                     (*map)(void *, long unsigned int, enum zpool_mapmode); /*    96     8 */
	void                       (*unmap)(void *, long unsigned int); /*   104     8 */
	u64                        (*total_size)(void *); /*   112     8 */

	/* size: 120, cachelines: 2, members: 14 */
	/* sum members: 102, holes: 3, sum holes: 18 */
	/* last cacheline: 56 bytes */
};
struct zbud_ops {
	int                        (*evict)(struct zbud_pool *, long unsigned int); /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct zbud_pool {
	spinlock_t                 lock;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           unbuddied[63];        /*     8  1008 */
	/* --- cacheline 15 boundary (960 bytes) was 56 bytes ago --- */
	struct list_head           buddied;              /*  1016    16 */
	/* --- cacheline 16 boundary (1024 bytes) was 8 bytes ago --- */
	struct list_head           lru;                  /*  1032    16 */
	u64                        pages_nr;             /*  1048     8 */
	const struct zbud_ops  *   ops;                  /*  1056     8 */
	struct zpool *             zpool;                /*  1064     8 */
	const struct zpool_ops  *  zpool_ops;            /*  1072     8 */

	/* size: 1080, cachelines: 17, members: 8 */
	/* sum members: 1076, holes: 1, sum holes: 4 */
	/* last cacheline: 56 bytes */
};
struct zbud_header {
	struct list_head           buddy;                /*     0    16 */
	struct list_head           lru;                  /*    16    16 */
	unsigned int               first_chunks;         /*    32     4 */
	unsigned int               last_chunks;          /*    36     4 */
	bool                       under_reclaim;        /*    40     1 */

	/* size: 48, cachelines: 1, members: 5 */
	/* padding: 7 */
	/* last cacheline: 48 bytes */
};
struct zs_pool_stats {
	atomic_long_t              pages_compacted;      /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct zs_size_stat {
	long unsigned int          objs[6];              /*     0    48 */

	/* size: 48, cachelines: 1, members: 1 */
	/* last cacheline: 48 bytes */
};
struct size_class {
	spinlock_t                 lock;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           fullness_list[4];     /*     8    64 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	int                        size;                 /*    72     4 */
	int                        objs_per_zspage;      /*    76     4 */
	int                        pages_per_zspage;     /*    80     4 */
	unsigned int               index;                /*    84     4 */
	struct zs_size_stat        stats;                /*    88    48 */

	/* size: 136, cachelines: 3, members: 7 */
	/* sum members: 132, holes: 1, sum holes: 4 */
	/* last cacheline: 8 bytes */
};
struct link_free {
	union {
		long unsigned int  next;                 /*     0     8 */
		long unsigned int  handle;               /*     0     8 */
	};                                               /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct zs_pool {
	const char  *              name;                 /*     0     8 */
	struct size_class *        size_class[255];      /*     8  2040 */
	/* --- cacheline 32 boundary (2048 bytes) --- */
	struct kmem_cache *        handle_cachep;        /*  2048     8 */
	struct kmem_cache *        zspage_cachep;        /*  2056     8 */
	atomic_long_t              pages_allocated;      /*  2064     8 */
	struct zs_pool_stats       stats;                /*  2072     8 */
	struct shrinker            shrinker;             /*  2080    64 */
	/* --- cacheline 33 boundary (2112 bytes) was 32 bytes ago --- */
	struct dentry *            stat_dentry;          /*  2144     8 */
	struct inode *             inode;                /*  2152     8 */
	struct work_struct         free_work;            /*  2160    64 */
	/* --- cacheline 34 boundary (2176 bytes) was 48 bytes ago --- */
	struct wait_queue_head     migration_wait;       /*  2224    24 */
	/* --- cacheline 35 boundary (2240 bytes) was 8 bytes ago --- */
	atomic_long_t              isolated_pages;       /*  2248     8 */
	bool                       destroying;           /*  2256     1 */

	/* size: 2264, cachelines: 36, members: 13 */
	/* padding: 7 */
	/* last cacheline: 24 bytes */
};
struct zspage {
	struct {
		unsigned int       fullness:2;           /*     0: 0  4 */
		unsigned int       class:9;              /*     0: 2  4 */
		unsigned int       isolated:3;           /*     0:11  4 */
		unsigned int       magic:8;              /*     0:14  4 */
	};                                               /*     0     4 */
	unsigned int               inuse;                /*     4     4 */
	unsigned int               freeobj;              /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct page *              first_page;           /*    16     8 */
	struct list_head           list;                 /*    24    16 */
	rwlock_t                   lock;                 /*    40     8 */

	/* size: 48, cachelines: 1, members: 6 */
	/* sum members: 44, holes: 1, sum holes: 4 */
	/* last cacheline: 48 bytes */
};
struct mapping_area {
	char *                     vm_buf;               /*     0     8 */
	char *                     vm_addr;              /*     8     8 */
	enum zs_mapmode            vm_mm;                /*    16     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct zs_compact_control {
	struct page *              s_page;               /*     0     8 */
	struct page *              d_page;               /*     8     8 */
	int                        obj_idx;              /*    16     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct balloon_dev_info {
	long unsigned int          isolated_pages;       /*     0     8 */
	spinlock_t                 pages_lock;           /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           pages;                /*    16    16 */
	int                        (*migratepage)(struct balloon_dev_info *, struct page *, struct page *, enum migrate_mode); /*    32     8 */
	struct inode *             inode;                /*    40     8 */

	/* size: 48, cachelines: 1, members: 5 */
	/* sum members: 44, holes: 1, sum holes: 4 */
	/* last cacheline: 48 bytes */
};
struct frame_vector {
	unsigned int               nr_allocated;         /*     0     4 */
	unsigned int               nr_frames;            /*     4     4 */
	bool                       got_ref;              /*     8     1 */
	bool                       is_pfns;              /*     9     1 */

	/* XXX 6 bytes hole, try to pack */

	void *                     ptrs[];               /*    16     0 */

	/* size: 16, cachelines: 1, members: 5 */
	/* sum members: 10, holes: 1, sum holes: 6 */
	/* last cacheline: 16 bytes */
};
struct hmm_range {
	struct mmu_interval_notifier * notifier;         /*     0     8 */
	long unsigned int          notifier_seq;         /*     8     8 */
	long unsigned int          start;                /*    16     8 */
	long unsigned int          end;                  /*    24     8 */
	long unsigned int *        hmm_pfns;             /*    32     8 */
	long unsigned int          default_flags;        /*    40     8 */
	long unsigned int          pfn_flags_mask;       /*    48     8 */
	void *                     dev_private_owner;    /*    56     8 */

	/* size: 64, cachelines: 1, members: 8 */
};
struct hmm_vma_walk {
	struct hmm_range *         range;                /*     0     8 */
	long unsigned int          last;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct wp_walk {
	struct mmu_notifier_range  range;                /*     0    48 */
	long unsigned int          tlbflush_start;       /*    48     8 */
	long unsigned int          tlbflush_end;         /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	long unsigned int          total;                /*    64     8 */

	/* size: 72, cachelines: 2, members: 4 */
	/* last cacheline: 8 bytes */
};
struct clean_walk {
	struct wp_walk             base;                 /*     0    72 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	long unsigned int          bitmap_pgoff;         /*    72     8 */
	long unsigned int *        bitmap;               /*    80     8 */
	long unsigned int          start;                /*    88     8 */
	long unsigned int          end;                  /*    96     8 */

	/* size: 104, cachelines: 2, members: 5 */
	/* last cacheline: 40 bytes */
};
struct page_reporting_dev_info {
	int                        (*report)(struct page_reporting_dev_info *, struct scatterlist *, unsigned int); /*     0     8 */
	struct delayed_work        work;                 /*     8   192 */
	/* --- cacheline 3 boundary (192 bytes) was 8 bytes ago --- */
	atomic_t                   state;                /*   200     4 */

	/* size: 208, cachelines: 4, members: 3 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct mnt_namespace {
	atomic_t                   count;                /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct ns_common           ns;                   /*     8    32 */
	struct mount *             root;                 /*    40     8 */
	struct list_head           list;                 /*    48    16 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	spinlock_t                 ns_lock;              /*    64     4 */

	/* XXX 4 bytes hole, try to pack */

	struct user_namespace *    user_ns;              /*    72     8 */
	struct ucounts *           ucounts;              /*    80     8 */
	u64                        seq;                  /*    88     8 */
	wait_queue_head_t          poll;                 /*    96    24 */
	u64                        event;                /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	unsigned int               mounts;               /*   128     4 */
	unsigned int               pending_mounts;       /*   132     4 */

	/* size: 136, cachelines: 3, members: 12 */
	/* sum members: 128, holes: 2, sum holes: 8 */
	/* last cacheline: 8 bytes */
};
struct mount {
	struct hlist_node          mnt_hash;             /*     0    16 */
	struct mount *             mnt_parent;           /*    16     8 */
	struct dentry *            mnt_mountpoint;       /*    24     8 */
	struct vfsmount            mnt;                  /*    32    32 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	union {
		struct callback_head mnt_rcu __attribute__((__aligned__(8))); /*    64    16 */
		struct llist_node  mnt_llist;            /*    64     8 */
	} __attribute__((__aligned__(8)));               /*    64    16 */
	struct mnt_pcp *           mnt_pcp;              /*    80     8 */
	struct list_head           mnt_mounts;           /*    88    16 */
	struct list_head           mnt_child;            /*   104    16 */
	struct list_head           mnt_instance;         /*   120    16 */
	/* --- cacheline 2 boundary (128 bytes) was 8 bytes ago --- */
	const char  *              mnt_devname;          /*   136     8 */
	struct list_head           mnt_list;             /*   144    16 */
	struct list_head           mnt_expire;           /*   160    16 */
	struct list_head           mnt_share;            /*   176    16 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	struct list_head           mnt_slave_list;       /*   192    16 */
	struct list_head           mnt_slave;            /*   208    16 */
	struct mount *             mnt_master;           /*   224     8 */
	struct mnt_namespace *     mnt_ns;               /*   232     8 */
	struct mountpoint *        mnt_mp;               /*   240     8 */
	union {
		struct hlist_node  mnt_mp_list;          /*   248    16 */
		struct hlist_node  mnt_umount;           /*   248    16 */
	};                                               /*   248    16 */
	/* --- cacheline 4 boundary (256 bytes) was 8 bytes ago --- */
	struct list_head           mnt_umounting;        /*   264    16 */
	struct fsnotify_mark_connector * mnt_fsnotify_marks; /*   280     8 */
	__u32                      mnt_fsnotify_mask;    /*   288     4 */
	int                        mnt_id;               /*   292     4 */
	int                        mnt_group_id;         /*   296     4 */
	int                        mnt_expiry_mark;      /*   300     4 */
	struct hlist_head          mnt_pins;             /*   304     8 */
	struct hlist_head          mnt_stuck_children;   /*   312     8 */

	/* size: 320, cachelines: 5, members: 27 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(8)));
struct mnt_pcp {
	int                        mnt_count;            /*     0     4 */
	int                        mnt_writers;          /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct mountpoint {
	struct hlist_node          m_hash;               /*     0    16 */
	struct dentry *            m_dentry;             /*    16     8 */
	struct hlist_head          m_list;               /*    24     8 */
	int                        m_count;              /*    32     4 */

	/* size: 40, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct print_files_control {
	struct mem_cgroup *        memcg;                /*     0     8 */
	struct seq_file *          m;                    /*     8     8 */
	long unsigned int          size_threshold;       /*    16     8 */
	long unsigned int          max_print_files;      /*    24     8 */
	char *                     pathbuf;              /*    32     8 */
	long unsigned int          pathbuf_size;         /*    40     8 */
	const char  *              fs_type_name;         /*    48     8 */
	struct vfsmount *          vfsmnt;               /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	long unsigned int          total_print_files;    /*    64     8 */
	long unsigned int          total_files_size;     /*    72     8 */

	/* size: 80, cachelines: 2, members: 10 */
	/* last cacheline: 16 bytes */
};
struct open_how {
	__u64                      flags;                /*     0     8 */
	__u64                      mode;                 /*     8     8 */
	__u64                      resolve;              /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct open_flags {
	int                        open_flag;            /*     0     4 */
	umode_t                    mode;                 /*     4     2 */

	/* XXX 2 bytes hole, try to pack */

	int                        acc_mode;             /*     8     4 */
	int                        intent;               /*    12     4 */
	int                        lookup_flags;         /*    16     4 */

	/* size: 20, cachelines: 1, members: 5 */
	/* sum members: 18, holes: 1, sum holes: 2 */
	/* last cacheline: 20 bytes */
};
struct fs_file_read_ctx {
	const unsigned char  *     name;                 /*     0     8 */
	unsigned int               f_mode;               /*     8     4 */
	unsigned int               rsvd;                 /*    12     4 */
	unsigned int               clr_f_mode;           /*    16     4 */
	unsigned int               set_f_mode;           /*    20     4 */
	long unsigned int          key;                  /*    24     8 */
	long long int              i_size;               /*    32     8 */
	long long int              prev_index;           /*    40     8 */
	long long int              index;                /*    48     8 */

	/* size: 56, cachelines: 1, members: 9 */
	/* last cacheline: 56 bytes */
};
struct char_device_struct {
	struct char_device_struct * next;                /*     0     8 */
	unsigned int               major;                /*     8     4 */
	unsigned int               baseminor;            /*    12     4 */
	int                        minorct;              /*    16     4 */
	char                       name[64];             /*    20    64 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) was 24 bytes ago --- */
	struct cdev *              cdev;                 /*    88     8 */

	/* size: 96, cachelines: 2, members: 6 */
	/* sum members: 92, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct stat {
	__kernel_ulong_t           st_dev;               /*     0     8 */
	__kernel_ulong_t           st_ino;               /*     8     8 */
	__kernel_ulong_t           st_nlink;             /*    16     8 */
	unsigned int               st_mode;              /*    24     4 */
	unsigned int               st_uid;               /*    28     4 */
	unsigned int               st_gid;               /*    32     4 */
	unsigned int               __pad0;               /*    36     4 */
	__kernel_ulong_t           st_rdev;              /*    40     8 */
	__kernel_long_t            st_size;              /*    48     8 */
	__kernel_long_t            st_blksize;           /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	__kernel_long_t            st_blocks;            /*    64     8 */
	__kernel_ulong_t           st_atime;             /*    72     8 */
	__kernel_ulong_t           st_atime_nsec;        /*    80     8 */
	__kernel_ulong_t           st_mtime;             /*    88     8 */
	__kernel_ulong_t           st_mtime_nsec;        /*    96     8 */
	__kernel_ulong_t           st_ctime;             /*   104     8 */
	__kernel_ulong_t           st_ctime_nsec;        /*   112     8 */
	__kernel_long_t            __unused[3];          /*   120    24 */

	/* size: 144, cachelines: 3, members: 18 */
	/* last cacheline: 16 bytes */
};
struct __old_kernel_stat {
	short unsigned int         st_dev;               /*     0     2 */
	short unsigned int         st_ino;               /*     2     2 */
	short unsigned int         st_mode;              /*     4     2 */
	short unsigned int         st_nlink;             /*     6     2 */
	short unsigned int         st_uid;               /*     8     2 */
	short unsigned int         st_gid;               /*    10     2 */
	short unsigned int         st_rdev;              /*    12     2 */

	/* XXX 2 bytes hole, try to pack */

	unsigned int               st_size;              /*    16     4 */
	unsigned int               st_atime;             /*    20     4 */
	unsigned int               st_mtime;             /*    24     4 */
	unsigned int               st_ctime;             /*    28     4 */

	/* size: 32, cachelines: 1, members: 11 */
	/* sum members: 30, holes: 1, sum holes: 2 */
	/* last cacheline: 32 bytes */
};
struct statx_timestamp {
	__s64                      tv_sec;               /*     0     8 */
	__u32                      tv_nsec;              /*     8     4 */
	__s32                      __reserved;           /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct statx {
	__u32                      stx_mask;             /*     0     4 */
	__u32                      stx_blksize;          /*     4     4 */
	__u64                      stx_attributes;       /*     8     8 */
	__u32                      stx_nlink;            /*    16     4 */
	__u32                      stx_uid;              /*    20     4 */
	__u32                      stx_gid;              /*    24     4 */
	__u16                      stx_mode;             /*    28     2 */
	__u16                      __spare0[1];          /*    30     2 */
	__u64                      stx_ino;              /*    32     8 */
	__u64                      stx_size;             /*    40     8 */
	__u64                      stx_blocks;           /*    48     8 */
	__u64                      stx_attributes_mask;  /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct statx_timestamp     stx_atime;            /*    64    16 */
	struct statx_timestamp     stx_btime;            /*    80    16 */
	struct statx_timestamp     stx_ctime;            /*    96    16 */
	struct statx_timestamp     stx_mtime;            /*   112    16 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	__u32                      stx_rdev_major;       /*   128     4 */
	__u32                      stx_rdev_minor;       /*   132     4 */
	__u32                      stx_dev_major;        /*   136     4 */
	__u32                      stx_dev_minor;        /*   140     4 */
	__u64                      stx_mnt_id;           /*   144     8 */
	__u64                      __spare2;             /*   152     8 */
	__u64                      __spare3[12];         /*   160    96 */

	/* size: 256, cachelines: 4, members: 23 */
};
struct compat_stat {
	u32                        st_dev;               /*     0     4 */
	compat_ino_t               st_ino;               /*     4     4 */
	compat_mode_t              st_mode;              /*     8     2 */
	compat_nlink_t             st_nlink;             /*    10     2 */
	__compat_uid_t             st_uid;               /*    12     2 */
	__compat_gid_t             st_gid;               /*    14     2 */
	u32                        st_rdev;              /*    16     4 */
	u32                        st_size;              /*    20     4 */
	u32                        st_blksize;           /*    24     4 */
	u32                        st_blocks;            /*    28     4 */
	u32                        st_atime;             /*    32     4 */
	u32                        st_atime_nsec;        /*    36     4 */
	u32                        st_mtime;             /*    40     4 */
	u32                        st_mtime_nsec;        /*    44     4 */
	u32                        st_ctime;             /*    48     4 */
	u32                        st_ctime_nsec;        /*    52     4 */
	u32                        __unused4;            /*    56     4 */
	u32                        __unused5;            /*    60     4 */

	/* size: 64, cachelines: 1, members: 18 */
};
struct user_arg_ptr {
	bool                       is_compat;            /*     0     1 */

	/* XXX 7 bytes hole, try to pack */

	union {
		const char  * const * native;            /*     8     8 */
		const compat_uptr_t  * compat;           /*     8     8 */
	} ptr;                                           /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 9, holes: 1, sum holes: 7 */
	/* last cacheline: 16 bytes */
};
struct pseudo_fs_context {
	const struct super_operations  * ops;            /*     0     8 */
	const struct xattr_handler  * * xattr;           /*     8     8 */
	const struct dentry_operations  * dops;          /*    16     8 */
	long unsigned int          magic;                /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct name_snapshot {
	struct qstr                name;                 /*     0    16 */
	unsigned char              inline_name[32];      /*    16    32 */

	/* size: 48, cachelines: 1, members: 2 */
	/* last cacheline: 48 bytes */
};
struct nameidata {
	struct path                path;                 /*     0    16 */
	struct qstr                last;                 /*    16    16 */
	struct path                root;                 /*    32    16 */
	struct inode *             inode;                /*    48     8 */
	unsigned int               flags;                /*    56     4 */
	unsigned int               state;                /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	unsigned int               seq;                  /*    64     4 */
	unsigned int               m_seq;                /*    68     4 */
	unsigned int               r_seq;                /*    72     4 */
	int                        last_type;            /*    76     4 */
	unsigned int               depth;                /*    80     4 */
	int                        total_link_count;     /*    84     4 */
	struct saved *             stack;                /*    88     8 */
	struct saved               internal[2];          /*    96    96 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	struct filename *          name;                 /*   192     8 */
	struct nameidata *         saved;                /*   200     8 */
	unsigned int               root_seq;             /*   208     4 */
	int                        dfd;                  /*   212     4 */
	kuid_t                     dir_uid;              /*   216     4 */
	umode_t                    dir_mode;             /*   220     2 */

	/* size: 224, cachelines: 4, members: 20 */
	/* padding: 2 */
	/* last cacheline: 32 bytes */
};
struct saved {
	struct path                link;                 /*     0    16 */
	struct delayed_call        done;                 /*    16    16 */
	const char  *              name;                 /*    32     8 */
	unsigned int               seq;                  /*    40     4 */

	/* size: 48, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 48 bytes */
};
struct word_at_a_time {
	const long unsigned int    one_bits;             /*     0     8 */
	const long unsigned int    high_bits;            /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct f_owner_ex {
	int                        type;                 /*     0     4 */
	__kernel_pid_t             pid;                  /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct flock {
	short int                  l_type;               /*     0     2 */
	short int                  l_whence;             /*     2     2 */

	/* XXX 4 bytes hole, try to pack */

	__kernel_off_t             l_start;              /*     8     8 */
	__kernel_off_t             l_len;                /*    16     8 */
	__kernel_pid_t             l_pid;                /*    24     4 */

	/* size: 32, cachelines: 1, members: 5 */
	/* sum members: 24, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct compat_flock {
	short int                  l_type;               /*     0     2 */
	short int                  l_whence;             /*     2     2 */
	compat_off_t               l_start;              /*     4     4 */
	compat_off_t               l_len;                /*     8     4 */
	compat_pid_t               l_pid;                /*    12     4 */

	/* size: 16, cachelines: 1, members: 5 */
	/* last cacheline: 16 bytes */
};
struct compat_flock64 {
	short int                  l_type;               /*     0     2 */
	short int                  l_whence;             /*     2     2 */
	compat_loff_t              l_start;              /*     4     8 */
	compat_loff_t              l_len;                /*    12     8 */
	compat_pid_t               l_pid;                /*    20     4 */

	/* size: 24, cachelines: 1, members: 5 */
	/* last cacheline: 24 bytes */
} __attribute__((__packed__));
struct file_clone_range {
	__s64                      src_fd;               /*     0     8 */
	__u64                      src_offset;           /*     8     8 */
	__u64                      src_length;           /*    16     8 */
	__u64                      dest_offset;          /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct file_dedupe_range_info {
	__s64                      dest_fd;              /*     0     8 */
	__u64                      dest_offset;          /*     8     8 */
	__u64                      bytes_deduped;        /*    16     8 */
	__s32                      status;               /*    24     4 */
	__u32                      reserved;             /*    28     4 */

	/* size: 32, cachelines: 1, members: 5 */
	/* last cacheline: 32 bytes */
};
struct file_dedupe_range {
	__u64                      src_offset;           /*     0     8 */
	__u64                      src_length;           /*     8     8 */
	__u16                      dest_count;           /*    16     2 */
	__u16                      reserved1;            /*    18     2 */
	__u32                      reserved2;            /*    20     4 */
	struct file_dedupe_range_info info[];            /*    24     0 */

	/* size: 24, cachelines: 1, members: 6 */
	/* last cacheline: 24 bytes */
};
struct fiemap_extent_info {
	unsigned int               fi_flags;             /*     0     4 */
	unsigned int               fi_extents_mapped;    /*     4     4 */
	unsigned int               fi_extents_max;       /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct fiemap_extent *     fi_extents_start;     /*    16     8 */

	/* size: 24, cachelines: 1, members: 4 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct space_resv {
	__s16                      l_type;               /*     0     2 */
	__s16                      l_whence;             /*     2     2 */

	/* XXX 4 bytes hole, try to pack */

	__s64                      l_start;              /*     8     8 */
	__s64                      l_len;                /*    16     8 */
	__s32                      l_sysid;              /*    24     4 */
	__u32                      l_pid;                /*    28     4 */
	__s32                      l_pad[4];             /*    32    16 */

	/* size: 48, cachelines: 1, members: 7 */
	/* sum members: 44, holes: 1, sum holes: 4 */
	/* last cacheline: 48 bytes */
};
struct space_resv_32 {
	__s16                      l_type;               /*     0     2 */
	__s16                      l_whence;             /*     2     2 */
	__s64                      l_start;              /*     4     8 */
	__s64                      l_len;                /*    12     8 */
	__s32                      l_sysid;              /*    20     4 */
	__u32                      l_pid;                /*    24     4 */
	__s32                      l_pad[4];             /*    28    16 */

	/* size: 44, cachelines: 1, members: 7 */
	/* last cacheline: 44 bytes */
} __attribute__((__packed__));
struct fiemap_extent {
	__u64                      fe_logical;           /*     0     8 */
	__u64                      fe_physical;          /*     8     8 */
	__u64                      fe_length;            /*    16     8 */
	__u64                      fe_reserved64[2];     /*    24    16 */
	__u32                      fe_flags;             /*    40     4 */
	__u32                      fe_reserved[3];       /*    44    12 */

	/* size: 56, cachelines: 1, members: 6 */
	/* last cacheline: 56 bytes */
};
struct fiemap {
	__u64                      fm_start;             /*     0     8 */
	__u64                      fm_length;            /*     8     8 */
	__u32                      fm_flags;             /*    16     4 */
	__u32                      fm_mapped_extents;    /*    20     4 */
	__u32                      fm_extent_count;      /*    24     4 */
	__u32                      fm_reserved;          /*    28     4 */
	struct fiemap_extent       fm_extents[];         /*    32     0 */

	/* size: 32, cachelines: 1, members: 7 */
	/* last cacheline: 32 bytes */
};
struct linux_dirent64 {
	u64                        d_ino;                /*     0     8 */
	s64                        d_off;                /*     8     8 */
	short unsigned int         d_reclen;             /*    16     2 */
	unsigned char              d_type;               /*    18     1 */
	char                       d_name[];             /*    19     0 */

	/* size: 24, cachelines: 1, members: 5 */
	/* padding: 5 */
	/* last cacheline: 24 bytes */
};
struct old_linux_dirent {
	long unsigned int          d_ino;                /*     0     8 */
	long unsigned int          d_offset;             /*     8     8 */
	short unsigned int         d_namlen;             /*    16     2 */
	char                       d_name[1];            /*    18     1 */

	/* size: 24, cachelines: 1, members: 4 */
	/* padding: 5 */
	/* last cacheline: 24 bytes */
};
struct readdir_callback {
	struct dir_context         ctx;                  /*     0    16 */
	struct old_linux_dirent *  dirent;               /*    16     8 */
	int                        result;               /*    24     4 */

	/* size: 32, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct linux_dirent {
	long unsigned int          d_ino;                /*     0     8 */
	long unsigned int          d_off;                /*     8     8 */
	short unsigned int         d_reclen;             /*    16     2 */
	char                       d_name[1];            /*    18     1 */

	/* size: 24, cachelines: 1, members: 4 */
	/* padding: 5 */
	/* last cacheline: 24 bytes */
};
struct getdents_callback {
	struct dir_context         ctx;                  /*     0    16 */
	struct linux_dirent *      current_dir;          /*    16     8 */
	int                        prev_reclen;          /*    24     4 */
	int                        count;                /*    28     4 */
	int                        error;                /*    32     4 */

	/* size: 40, cachelines: 1, members: 5 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct getdents_callback64 {
	struct dir_context         ctx;                  /*     0    16 */
	struct linux_dirent64 *    current_dir;          /*    16     8 */
	int                        prev_reclen;          /*    24     4 */
	int                        count;                /*    28     4 */
	int                        error;                /*    32     4 */

	/* size: 40, cachelines: 1, members: 5 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct compat_old_linux_dirent {
	compat_ulong_t             d_ino;                /*     0     4 */
	compat_ulong_t             d_offset;             /*     4     4 */
	short unsigned int         d_namlen;             /*     8     2 */
	char                       d_name[1];            /*    10     1 */

	/* size: 12, cachelines: 1, members: 4 */
	/* padding: 1 */
	/* last cacheline: 12 bytes */
};
struct compat_readdir_callback {
	struct dir_context         ctx;                  /*     0    16 */
	struct compat_old_linux_dirent * dirent;         /*    16     8 */
	int                        result;               /*    24     4 */

	/* size: 32, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct compat_linux_dirent {
	compat_ulong_t             d_ino;                /*     0     4 */
	compat_ulong_t             d_off;                /*     4     4 */
	short unsigned int         d_reclen;             /*     8     2 */
	char                       d_name[1];            /*    10     1 */

	/* size: 12, cachelines: 1, members: 4 */
	/* padding: 1 */
	/* last cacheline: 12 bytes */
};
struct compat_getdents_callback {
	struct dir_context         ctx;                  /*     0    16 */
	struct compat_linux_dirent * current_dir;        /*    16     8 */
	int                        prev_reclen;          /*    24     4 */
	int                        count;                /*    28     4 */
	int                        error;                /*    32     4 */

	/* size: 40, cachelines: 1, members: 5 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct poll_table_entry {
	struct file *              filp;                 /*     0     8 */
	__poll_t                   key;                  /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	wait_queue_entry_t         wait;                 /*    16    40 */
	wait_queue_head_t *        wait_address;         /*    56     8 */

	/* size: 64, cachelines: 1, members: 4 */
	/* sum members: 60, holes: 1, sum holes: 4 */
};
struct poll_wqueues {
	poll_table                 pt;                   /*     0    16 */
	struct poll_table_page *   table;                /*    16     8 */
	struct task_struct *       polling_task;         /*    24     8 */
	int                        triggered;            /*    32     4 */
	int                        error;                /*    36     4 */
	int                        inline_index;         /*    40     4 */

	/* XXX 4 bytes hole, try to pack */

	struct poll_table_entry    inline_entries[9];    /*    48   576 */

	/* size: 624, cachelines: 10, members: 7 */
	/* sum members: 620, holes: 1, sum holes: 4 */
	/* last cacheline: 48 bytes */
};
struct poll_table_page {
	struct poll_table_page *   next;                 /*     0     8 */
	struct poll_table_entry *  entry;                /*     8     8 */
	struct poll_table_entry    entries[];            /*    16     0 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct sigset_argpack {
	sigset_t *                 p;                    /*     0     8 */
	size_t                     size;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct poll_list {
	struct poll_list *         next;                 /*     0     8 */
	int                        len;                  /*     8     4 */
	struct pollfd              entries[];            /*    12     0 */

	/* size: 16, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct compat_sel_arg_struct {
	compat_ulong_t             n;                    /*     0     4 */
	compat_uptr_t              inp;                  /*     4     4 */
	compat_uptr_t              outp;                 /*     8     4 */
	compat_uptr_t              exp;                  /*    12     4 */
	compat_uptr_t              tvp;                  /*    16     4 */

	/* size: 20, cachelines: 1, members: 5 */
	/* last cacheline: 20 bytes */
};
struct compat_sigset_argpack {
	compat_uptr_t              p;                    /*     0     4 */
	compat_size_t              size;                 /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct external_name {
	union {
		atomic_t           count;                /*     0     4 */
		struct callback_head head __attribute__((__aligned__(8))); /*     0    16 */
	} u __attribute__((__aligned__(8)));             /*     0    16 */
	unsigned char              name[];               /*    16     0 */

	/* size: 16, cachelines: 1, members: 2 */
	/* forced alignments: 1 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct check_mount {
	struct vfsmount *          mnt;                  /*     0     8 */
	unsigned int               mounted;              /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct select_data {
	struct dentry *            start;                /*     0     8 */
	union {
		long int           found;                /*     8     8 */
		struct dentry *    victim;               /*     8     8 */
	};                                               /*     8     8 */
	struct list_head           dispose;              /*    16    16 */

	/* size: 32, cachelines: 1, members: 3 */
	/* last cacheline: 32 bytes */
};
struct fsxattr {
	__u32                      fsx_xflags;           /*     0     4 */
	__u32                      fsx_extsize;          /*     4     4 */
	__u32                      fsx_nextents;         /*     8     4 */
	__u32                      fsx_projid;           /*    12     4 */
	__u32                      fsx_cowextsize;       /*    16     4 */
	unsigned char              fsx_pad[8];           /*    20     8 */

	/* size: 28, cachelines: 1, members: 6 */
	/* last cacheline: 28 bytes */
};
struct proc_mounts {
	struct mnt_namespace *     ns;                   /*     0     8 */
	struct path                root;                 /*     8    16 */
	int                        (*show)(struct seq_file *, struct vfsmount *); /*    24     8 */
	struct mount               cursor __attribute__((__aligned__(8))); /*    32   320 */

	/* size: 352, cachelines: 6, members: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct simple_transaction_argresp {
	ssize_t                    size;                 /*     0     8 */
	char                       data[];               /*     8     0 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct simple_attr {
	int                        (*get)(void *, u64 *); /*     0     8 */
	int                        (*set)(void *, u64);  /*     8     8 */
	char                       get_buf[24];          /*    16    24 */
	char                       set_buf[24];          /*    40    24 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	void *                     data;                 /*    64     8 */
	const char  *              fmt;                  /*    72     8 */
	struct mutex               mutex;                /*    80    32 */

	/* size: 112, cachelines: 2, members: 7 */
	/* last cacheline: 48 bytes */
};
struct wb_writeback_work {
	long int                   nr_pages;             /*     0     8 */
	struct super_block *       sb;                   /*     8     8 */
	enum writeback_sync_modes  sync_mode;            /*    16     4 */
	unsigned int               tagged_writepages:1;  /*    20: 0  4 */
	unsigned int               for_kupdate:1;        /*    20: 1  4 */
	unsigned int               range_cyclic:1;       /*    20: 2  4 */
	unsigned int               for_background:1;     /*    20: 3  4 */
	unsigned int               for_sync:1;           /*    20: 4  4 */
	unsigned int               auto_free:1;          /*    20: 5  4 */

	/* XXX 26 bits hole, try to pack */

	enum wb_reason             reason;               /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           list;                 /*    32    16 */
	struct wb_completion *     done;                 /*    48     8 */

	/* size: 56, cachelines: 1, members: 12 */
	/* sum members: 48, holes: 1, sum holes: 4 */
	/* sum bitfield members: 6 bits, bit holes: 1, sum bit holes: 26 bits */
	/* last cacheline: 56 bytes */
};
struct trace_event_raw_writeback_page_template {
	struct trace_entry         ent;                  /*     0     8 */
	char                       name[32];             /*     8    32 */
	ino_t                      ino;                  /*    40     8 */
	long unsigned int          index;                /*    48     8 */
	char                       __data[];             /*    56     0 */

	/* size: 56, cachelines: 1, members: 5 */
	/* last cacheline: 56 bytes */
};
struct trace_event_raw_writeback_dirty_inode_template {
	struct trace_entry         ent;                  /*     0     8 */
	char                       name[32];             /*     8    32 */
	ino_t                      ino;                  /*    40     8 */
	long unsigned int          state;                /*    48     8 */
	long unsigned int          flags;                /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	char                       __data[];             /*    64     0 */

	/* size: 64, cachelines: 1, members: 6 */
};
struct trace_event_raw_inode_foreign_history {
	struct trace_entry         ent;                  /*     0     8 */
	char                       name[32];             /*     8    32 */
	ino_t                      ino;                  /*    40     8 */
	ino_t                      cgroup_ino;           /*    48     8 */
	unsigned int               history;              /*    56     4 */
	char                       __data[];             /*    60     0 */

	/* size: 64, cachelines: 1, members: 6 */
	/* padding: 4 */
};
struct trace_event_raw_inode_switch_wbs {
	struct trace_entry         ent;                  /*     0     8 */
	char                       name[32];             /*     8    32 */
	ino_t                      ino;                  /*    40     8 */
	ino_t                      old_cgroup_ino;       /*    48     8 */
	ino_t                      new_cgroup_ino;       /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	char                       __data[];             /*    64     0 */

	/* size: 64, cachelines: 1, members: 6 */
};
struct trace_event_raw_track_foreign_dirty {
	struct trace_entry         ent;                  /*     0     8 */
	char                       name[32];             /*     8    32 */
	u64                        bdi_id;               /*    40     8 */
	ino_t                      ino;                  /*    48     8 */
	unsigned int               memcg_id;             /*    56     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	ino_t                      cgroup_ino;           /*    64     8 */
	ino_t                      page_cgroup_ino;      /*    72     8 */
	char                       __data[];             /*    80     0 */

	/* size: 80, cachelines: 2, members: 8 */
	/* sum members: 76, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct trace_event_raw_flush_foreign {
	struct trace_entry         ent;                  /*     0     8 */
	char                       name[32];             /*     8    32 */
	ino_t                      cgroup_ino;           /*    40     8 */
	unsigned int               frn_bdi_id;           /*    48     4 */
	unsigned int               frn_memcg_id;         /*    52     4 */
	char                       __data[];             /*    56     0 */

	/* size: 56, cachelines: 1, members: 6 */
	/* last cacheline: 56 bytes */
};
struct trace_event_raw_writeback_write_inode_template {
	struct trace_entry         ent;                  /*     0     8 */
	char                       name[32];             /*     8    32 */
	ino_t                      ino;                  /*    40     8 */
	int                        sync_mode;            /*    48     4 */

	/* XXX 4 bytes hole, try to pack */

	ino_t                      cgroup_ino;           /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	char                       __data[];             /*    64     0 */

	/* size: 64, cachelines: 1, members: 6 */
	/* sum members: 60, holes: 1, sum holes: 4 */
};
struct trace_event_raw_writeback_work_class {
	struct trace_entry         ent;                  /*     0     8 */
	char                       name[32];             /*     8    32 */
	long int                   nr_pages;             /*    40     8 */
	dev_t                      sb_dev;               /*    48     4 */
	int                        sync_mode;            /*    52     4 */
	int                        for_kupdate;          /*    56     4 */
	int                        range_cyclic;         /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        for_background;       /*    64     4 */
	int                        reason;               /*    68     4 */
	ino_t                      cgroup_ino;           /*    72     8 */
	char                       __data[];             /*    80     0 */

	/* size: 80, cachelines: 2, members: 11 */
	/* last cacheline: 16 bytes */
};
struct trace_event_raw_writeback_pages_written {
	struct trace_entry         ent;                  /*     0     8 */
	long int                   pages;                /*     8     8 */
	char                       __data[];             /*    16     0 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct trace_event_raw_writeback_class {
	struct trace_entry         ent;                  /*     0     8 */
	char                       name[32];             /*     8    32 */
	ino_t                      cgroup_ino;           /*    40     8 */
	char                       __data[];             /*    48     0 */

	/* size: 48, cachelines: 1, members: 4 */
	/* last cacheline: 48 bytes */
};
struct trace_event_raw_writeback_bdi_register {
	struct trace_entry         ent;                  /*     0     8 */
	char                       name[32];             /*     8    32 */
	char                       __data[];             /*    40     0 */

	/* size: 40, cachelines: 1, members: 3 */
	/* last cacheline: 40 bytes */
};
struct trace_event_raw_wbc_class {
	struct trace_entry         ent;                  /*     0     8 */
	char                       name[32];             /*     8    32 */
	long int                   nr_to_write;          /*    40     8 */
	long int                   pages_skipped;        /*    48     8 */
	int                        sync_mode;            /*    56     4 */
	int                        for_kupdate;          /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        for_background;       /*    64     4 */
	int                        for_reclaim;          /*    68     4 */
	int                        range_cyclic;         /*    72     4 */

	/* XXX 4 bytes hole, try to pack */

	long int                   range_start;          /*    80     8 */
	long int                   range_end;            /*    88     8 */
	ino_t                      cgroup_ino;           /*    96     8 */
	char                       __data[];             /*   104     0 */

	/* size: 104, cachelines: 2, members: 13 */
	/* sum members: 100, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct trace_event_raw_writeback_queue_io {
	struct trace_entry         ent;                  /*     0     8 */
	char                       name[32];             /*     8    32 */
	long unsigned int          older;                /*    40     8 */
	long int                   age;                  /*    48     8 */
	int                        moved;                /*    56     4 */
	int                        reason;               /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	ino_t                      cgroup_ino;           /*    64     8 */
	char                       __data[];             /*    72     0 */

	/* size: 72, cachelines: 2, members: 8 */
	/* last cacheline: 8 bytes */
};
struct trace_event_raw_global_dirty_state {
	struct trace_entry         ent;                  /*     0     8 */
	long unsigned int          nr_dirty;             /*     8     8 */
	long unsigned int          nr_writeback;         /*    16     8 */
	long unsigned int          background_thresh;    /*    24     8 */
	long unsigned int          dirty_thresh;         /*    32     8 */
	long unsigned int          dirty_limit;          /*    40     8 */
	long unsigned int          nr_dirtied;           /*    48     8 */
	long unsigned int          nr_written;           /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	char                       __data[];             /*    64     0 */

	/* size: 64, cachelines: 1, members: 9 */
};
struct trace_event_raw_bdi_dirty_ratelimit {
	struct trace_entry         ent;                  /*     0     8 */
	char                       bdi[32];              /*     8    32 */
	long unsigned int          write_bw;             /*    40     8 */
	long unsigned int          avg_write_bw;         /*    48     8 */
	long unsigned int          dirty_rate;           /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	long unsigned int          dirty_ratelimit;      /*    64     8 */
	long unsigned int          task_ratelimit;       /*    72     8 */
	long unsigned int          balanced_dirty_ratelimit; /*    80     8 */
	ino_t                      cgroup_ino;           /*    88     8 */
	char                       __data[];             /*    96     0 */

	/* size: 96, cachelines: 2, members: 10 */
	/* last cacheline: 32 bytes */
};
struct trace_event_raw_balance_dirty_pages {
	struct trace_entry         ent;                  /*     0     8 */
	char                       bdi[32];              /*     8    32 */
	long unsigned int          limit;                /*    40     8 */
	long unsigned int          setpoint;             /*    48     8 */
	long unsigned int          dirty;                /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	long unsigned int          bdi_setpoint;         /*    64     8 */
	long unsigned int          bdi_dirty;            /*    72     8 */
	long unsigned int          dirty_ratelimit;      /*    80     8 */
	long unsigned int          task_ratelimit;       /*    88     8 */
	unsigned int               dirtied;              /*    96     4 */
	unsigned int               dirtied_pause;        /*   100     4 */
	long unsigned int          paused;               /*   104     8 */
	long int                   pause;                /*   112     8 */
	long unsigned int          period;               /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	long int                   think;                /*   128     8 */
	ino_t                      cgroup_ino;           /*   136     8 */
	char                       __data[];             /*   144     0 */

	/* size: 144, cachelines: 3, members: 17 */
	/* last cacheline: 16 bytes */
};
struct trace_event_raw_writeback_sb_inodes_requeue {
	struct trace_entry         ent;                  /*     0     8 */
	char                       name[32];             /*     8    32 */
	ino_t                      ino;                  /*    40     8 */
	long unsigned int          state;                /*    48     8 */
	long unsigned int          dirtied_when;         /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	ino_t                      cgroup_ino;           /*    64     8 */
	char                       __data[];             /*    72     0 */

	/* size: 72, cachelines: 2, members: 7 */
	/* last cacheline: 8 bytes */
};
struct trace_event_raw_writeback_congest_waited_template {
	struct trace_entry         ent;                  /*     0     8 */
	unsigned int               usec_timeout;         /*     8     4 */
	unsigned int               usec_delayed;         /*    12     4 */
	char                       __data[];             /*    16     0 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct trace_event_raw_writeback_single_inode_template {
	struct trace_entry         ent;                  /*     0     8 */
	char                       name[32];             /*     8    32 */
	ino_t                      ino;                  /*    40     8 */
	long unsigned int          state;                /*    48     8 */
	long unsigned int          dirtied_when;         /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	long unsigned int          writeback_index;      /*    64     8 */
	long int                   nr_to_write;          /*    72     8 */
	long unsigned int          wrote;                /*    80     8 */
	ino_t                      cgroup_ino;           /*    88     8 */
	char                       __data[];             /*    96     0 */

	/* size: 96, cachelines: 2, members: 10 */
	/* last cacheline: 32 bytes */
};
struct trace_event_raw_writeback_inode_template {
	struct trace_entry         ent;                  /*     0     8 */
	dev_t                      dev;                  /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	ino_t                      ino;                  /*    16     8 */
	long unsigned int          state;                /*    24     8 */
	__u16                      mode;                 /*    32     2 */

	/* XXX 6 bytes hole, try to pack */

	long unsigned int          dirtied_when;         /*    40     8 */
	char                       __data[];             /*    48     0 */

	/* size: 48, cachelines: 1, members: 7 */
	/* sum members: 38, holes: 2, sum holes: 10 */
	/* last cacheline: 48 bytes */
};
struct trace_event_data_offsets_writeback_page_template {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_writeback_dirty_inode_template {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_inode_foreign_history {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_inode_switch_wbs {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_track_foreign_dirty {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_flush_foreign {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_writeback_write_inode_template {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_writeback_work_class {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_writeback_pages_written {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_writeback_class {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_writeback_bdi_register {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_wbc_class {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_writeback_queue_io {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_global_dirty_state {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_bdi_dirty_ratelimit {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_balance_dirty_pages {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_writeback_sb_inodes_requeue {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_writeback_congest_waited_template {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_writeback_single_inode_template {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_writeback_inode_template {

	/* size: 0, cachelines: 0, members: 0 */
};
struct inode_switch_wbs_context {
	struct inode *             inode;                /*     0     8 */
	struct bdi_writeback *     new_wb;               /*     8     8 */
	struct callback_head       callback_head __attribute__((__aligned__(8))); /*    16    16 */
	struct work_struct         work;                 /*    32    64 */

	/* size: 96, cachelines: 2, members: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct splice_desc {
	size_t                     total_len;            /*     0     8 */
	unsigned int               len;                  /*     8     4 */
	unsigned int               flags;                /*    12     4 */
	union {
		void *             userptr;              /*    16     8 */
		struct file *      file;                 /*    16     8 */
		void *             data;                 /*    16     8 */
	} u;                                             /*    16     8 */
	loff_t                     pos;                  /*    24     8 */
	loff_t *                   opos;                 /*    32     8 */
	size_t                     num_spliced;          /*    40     8 */
	bool                       need_wakeup;          /*    48     1 */

	/* size: 56, cachelines: 1, members: 8 */
	/* padding: 7 */
	/* last cacheline: 56 bytes */
};
struct old_utimbuf32 {
	old_time32_t               actime;               /*     0     4 */
	old_time32_t               modtime;              /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct utimbuf {
	__kernel_old_time_t        actime;               /*     0     8 */
	__kernel_old_time_t        modtime;              /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct ustat {
	__kernel_daddr_t           f_tfree;              /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	__kernel_ino_t             f_tinode;             /*     8     8 */
	char                       f_fname[6];           /*    16     6 */
	char                       f_fpack[6];           /*    22     6 */

	/* size: 32, cachelines: 1, members: 4 */
	/* sum members: 24, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct compat_statfs {
	int                        f_type;               /*     0     4 */
	int                        f_bsize;              /*     4     4 */
	int                        f_blocks;             /*     8     4 */
	int                        f_bfree;              /*    12     4 */
	int                        f_bavail;             /*    16     4 */
	int                        f_files;              /*    20     4 */
	int                        f_ffree;              /*    24     4 */
	compat_fsid_t              f_fsid;               /*    28     8 */
	int                        f_namelen;            /*    36     4 */
	int                        f_frsize;             /*    40     4 */
	int                        f_flags;              /*    44     4 */
	int                        f_spare[4];           /*    48    16 */

	/* size: 64, cachelines: 1, members: 12 */
};
struct compat_ustat {
	compat_daddr_t             f_tfree;              /*     0     4 */
	compat_ino_t               f_tinode;             /*     4     4 */
	char                       f_fname[6];           /*     8     6 */
	char                       f_fpack[6];           /*    14     6 */

	/* size: 20, cachelines: 1, members: 4 */
	/* last cacheline: 20 bytes */
};
struct statfs {
	__kernel_long_t            f_type;               /*     0     8 */
	__kernel_long_t            f_bsize;              /*     8     8 */
	__kernel_long_t            f_blocks;             /*    16     8 */
	__kernel_long_t            f_bfree;              /*    24     8 */
	__kernel_long_t            f_bavail;             /*    32     8 */
	__kernel_long_t            f_files;              /*    40     8 */
	__kernel_long_t            f_ffree;              /*    48     8 */
	__kernel_fsid_t            f_fsid;               /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	__kernel_long_t            f_namelen;            /*    64     8 */
	__kernel_long_t            f_frsize;             /*    72     8 */
	__kernel_long_t            f_flags;              /*    80     8 */
	__kernel_long_t            f_spare[4];           /*    88    32 */

	/* size: 120, cachelines: 2, members: 12 */
	/* last cacheline: 56 bytes */
};
struct statfs64 {
	__kernel_long_t            f_type;               /*     0     8 */
	__kernel_long_t            f_bsize;              /*     8     8 */
	__u64                      f_blocks;             /*    16     8 */
	__u64                      f_bfree;              /*    24     8 */
	__u64                      f_bavail;             /*    32     8 */
	__u64                      f_files;              /*    40     8 */
	__u64                      f_ffree;              /*    48     8 */
	__kernel_fsid_t            f_fsid;               /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	__kernel_long_t            f_namelen;            /*    64     8 */
	__kernel_long_t            f_frsize;             /*    72     8 */
	__kernel_long_t            f_flags;              /*    80     8 */
	__kernel_long_t            f_spare[4];           /*    88    32 */

	/* size: 120, cachelines: 2, members: 12 */
	/* last cacheline: 56 bytes */
};
struct compat_statfs64 {
	__u32                      f_type;               /*     0     4 */
	__u32                      f_bsize;              /*     4     4 */
	__u64                      f_blocks;             /*     8     8 */
	__u64                      f_bfree;              /*    16     8 */
	__u64                      f_bavail;             /*    24     8 */
	__u64                      f_files;              /*    32     8 */
	__u64                      f_ffree;              /*    40     8 */
	__kernel_fsid_t            f_fsid;               /*    48     8 */
	__u32                      f_namelen;            /*    56     4 */
	__u32                      f_frsize;             /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	__u32                      f_flags;              /*    64     4 */
	__u32                      f_spare[4];           /*    68    16 */

	/* size: 84, cachelines: 2, members: 12 */
	/* last cacheline: 20 bytes */
} __attribute__((__packed__));
struct ns_get_path_task_args {
	const struct proc_ns_operations  * ns_ops;       /*     0     8 */
	struct task_struct *       task;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct legacy_fs_context {
	char *                     legacy_data;          /*     0     8 */
	size_t                     data_size;            /*     8     8 */
	enum legacy_fs_param       param_type;           /*    16     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct iomap {
	u64                        addr;                 /*     0     8 */
	loff_t                     offset;               /*     8     8 */
	u64                        length;               /*    16     8 */
	u16                        type;                 /*    24     2 */
	u16                        flags;                /*    26     2 */

	/* XXX 4 bytes hole, try to pack */

	struct block_device *      bdev;                 /*    32     8 */
	struct dax_device *        dax_dev;              /*    40     8 */
	void *                     inline_data;          /*    48     8 */
	void *                     private;              /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	const struct iomap_page_ops  * page_ops;         /*    64     8 */

	/* size: 72, cachelines: 2, members: 10 */
	/* sum members: 68, holes: 1, sum holes: 4 */
	/* last cacheline: 8 bytes */
};
struct iomap_page_ops {
	int                        (*page_prepare)(struct inode *, loff_t, unsigned int, struct iomap *); /*     0     8 */
	void                       (*page_done)(struct inode *, loff_t, unsigned int, struct page *, struct iomap *); /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct decrypt_bh_ctx {
	struct work_struct         work;                 /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct buffer_head *       bh;                   /*    64     8 */

	/* size: 72, cachelines: 2, members: 2 */
	/* last cacheline: 8 bytes */
};
struct bh_lru {
	struct buffer_head *       bhs[16];              /*     0   128 */

	/* size: 128, cachelines: 2, members: 1 */
};
struct bh_accounting {
	int                        nr;                   /*     0     4 */
	int                        ratelimit;            /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct bdev_inode {
	struct block_device        bdev;                 /*     0   224 */
	/* --- cacheline 3 boundary (192 bytes) was 32 bytes ago --- */
	struct inode               vfs_inode __attribute__((__aligned__(8))); /*   224   624 */

	/* size: 848, cachelines: 14, members: 2 */
	/* forced alignments: 1 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct blkdev_dio {
	union {
		struct kiocb *     iocb;                 /*     0     8 */
		struct task_struct * waiter;             /*     0     8 */
	};                                               /*     0     8 */
	size_t                     size;                 /*     8     8 */
	atomic_t                   ref;                  /*    16     4 */
	bool                       multi_bio:1;          /*    20: 0  1 */
	bool                       should_dirty:1;       /*    20: 1  1 */
	bool                       is_sync:1;            /*    20: 2  1 */

	/* XXX 5 bits hole, try to pack */
	/* XXX 3 bytes hole, try to pack */

	struct bio                 bio;                  /*    24   152 */

	/* size: 176, cachelines: 3, members: 7 */
	/* sum members: 172, holes: 1, sum holes: 3 */
	/* sum bitfield members: 3 bits, bit holes: 1, sum bit holes: 5 bits */
	/* last cacheline: 48 bytes */
};
struct bd_holder_disk {
	struct list_head           list;                 /*     0    16 */
	struct gendisk *           disk;                 /*    16     8 */
	int                        refcnt;               /*    24     4 */

	/* size: 32, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct dio_submit {
	struct bio *               bio;                  /*     0     8 */
	unsigned int               blkbits;              /*     8     4 */
	unsigned int               blkfactor;            /*    12     4 */
	unsigned int               start_zero_done;      /*    16     4 */
	int                        pages_in_io;          /*    20     4 */
	sector_t                   block_in_file;        /*    24     8 */
	unsigned int               blocks_available;     /*    32     4 */
	int                        reap_counter;         /*    36     4 */
	sector_t                   final_block_in_request; /*    40     8 */
	int                        boundary;             /*    48     4 */

	/* XXX 4 bytes hole, try to pack */

	get_block_t *              get_block;            /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	dio_submit_t *             submit_io;            /*    64     8 */
	loff_t                     logical_offset_in_bio; /*    72     8 */
	sector_t                   final_block_in_bio;   /*    80     8 */
	sector_t                   next_block_for_io;    /*    88     8 */
	struct page *              cur_page;             /*    96     8 */
	unsigned int               cur_page_offset;      /*   104     4 */
	unsigned int               cur_page_len;         /*   108     4 */
	sector_t                   cur_page_block;       /*   112     8 */
	loff_t                     cur_page_fs_offset;   /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct iov_iter *          iter;                 /*   128     8 */
	unsigned int               head;                 /*   136     4 */
	unsigned int               tail;                 /*   140     4 */
	size_t                     from;                 /*   144     8 */
	size_t                     to;                   /*   152     8 */

	/* size: 160, cachelines: 3, members: 25 */
	/* sum members: 156, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct dio {
	int                        flags;                /*     0     4 */
	int                        op;                   /*     4     4 */
	int                        op_flags;             /*     8     4 */
	blk_qc_t                   bio_cookie;           /*    12     4 */
	struct gendisk *           bio_disk;             /*    16     8 */
	struct inode *             inode;                /*    24     8 */
	loff_t                     i_size;               /*    32     8 */
	dio_iodone_t *             end_io;               /*    40     8 */
	void *                     private;              /*    48     8 */
	spinlock_t                 bio_lock;             /*    56     4 */
	int                        page_errors;          /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        is_async;             /*    64     4 */
	bool                       defer_completion;     /*    68     1 */
	bool                       should_dirty;         /*    69     1 */

	/* XXX 2 bytes hole, try to pack */

	int                        io_error;             /*    72     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          refcount;             /*    80     8 */
	struct bio *               bio_list;             /*    88     8 */
	struct task_struct *       waiter;               /*    96     8 */
	struct kiocb *             iocb;                 /*   104     8 */
	ssize_t                    result;               /*   112     8 */
	union {
		struct page *      pages[64];            /*   120   512 */
		struct work_struct complete_work;        /*   120    64 */
	};                                               /*   120   512 */

	/* size: 640, cachelines: 10, members: 21 */
	/* sum members: 626, holes: 2, sum holes: 6 */
	/* padding: 8 */
} __attribute__((__aligned__(64)));
struct bvec_iter_all {
	struct bio_vec             bv;                   /*     0    16 */
	int                        idx;                  /*    16     4 */
	unsigned int               done;                 /*    20     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct mpage_readpage_args {
	struct bio *               bio;                  /*     0     8 */
	struct page *              page;                 /*     8     8 */
	unsigned int               nr_pages;             /*    16     4 */
	bool                       is_readahead;         /*    20     1 */

	/* XXX 3 bytes hole, try to pack */

	sector_t                   last_block_in_bio;    /*    24     8 */
	struct buffer_head         map_bh;               /*    32   104 */
	/* --- cacheline 2 boundary (128 bytes) was 8 bytes ago --- */
	long unsigned int          first_logical_block;  /*   136     8 */
	get_block_t *              get_block;            /*   144     8 */

	/* size: 152, cachelines: 3, members: 8 */
	/* sum members: 149, holes: 1, sum holes: 3 */
	/* last cacheline: 24 bytes */
};
struct mpage_data {
	struct bio *               bio;                  /*     0     8 */
	sector_t                   last_block_in_bio;    /*     8     8 */
	get_block_t *              get_block;            /*    16     8 */
	unsigned int               use_writepage;        /*    24     4 */

	/* size: 32, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct proc_dir_entry {
	atomic_t                   in_use;               /*     0     4 */
	refcount_t                 refcnt;               /*     4     4 */
	struct list_head           pde_openers;          /*     8    16 */
	spinlock_t                 pde_unload_lock;      /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	struct completion *        pde_unload_completion; /*    32     8 */
	const struct inode_operations  * proc_iops;      /*    40     8 */
	union {
		const struct proc_ops  * proc_ops;       /*    48     8 */
		const struct file_operations  * proc_dir_ops; /*    48     8 */
	};                                               /*    48     8 */
	const struct dentry_operations  * proc_dops;     /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	union {
		const struct seq_operations  * seq_ops;  /*    64     8 */
		int                (*single_show)(struct seq_file *, void *); /*    64     8 */
	};                                               /*    64     8 */
	proc_write_t               write;                /*    72     8 */
	void *                     data;                 /*    80     8 */
	unsigned int               state_size;           /*    88     4 */
	unsigned int               low_ino;              /*    92     4 */
	nlink_t                    nlink;                /*    96     4 */
	kuid_t                     uid;                  /*   100     4 */
	kgid_t                     gid;                  /*   104     4 */

	/* XXX 4 bytes hole, try to pack */

	loff_t                     size;                 /*   112     8 */
	struct proc_dir_entry *    parent;               /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct rb_root             subdir;               /*   128     8 */
	struct rb_node             subdir_node __attribute__((__aligned__(8))); /*   136    24 */
	char *                     name;                 /*   160     8 */
	umode_t                    mode;                 /*   168     2 */
	u8                         flags;                /*   170     1 */
	u8                         namelen;              /*   171     1 */
	char                       inline_name[];        /*   172     0 */

	/* size: 176, cachelines: 3, members: 25 */
	/* sum members: 164, holes: 2, sum holes: 8 */
	/* padding: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
union proc_op {
	int                        (*proc_get_link)(struct dentry *, struct path *); /*     0     8 */
	int                        (*proc_show)(struct seq_file *, struct pid_namespace *, struct pid *, struct task_struct *); /*     0     8 */
	const char  *              lsm;                /*     0     8 */
};
struct proc_inode {
	struct pid *               pid;                  /*     0     8 */
	unsigned int               fd;                   /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	union proc_op              op;                   /*    16     8 */
	struct proc_dir_entry *    pde;                  /*    24     8 */
	struct ctl_table_header *  sysctl;               /*    32     8 */
	struct ctl_table *         sysctl_entry;         /*    40     8 */
	struct hlist_node          sibling_inodes;       /*    48    16 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	const struct proc_ns_operations  * ns_ops;       /*    64     8 */
	struct inode               vfs_inode __attribute__((__aligned__(8))); /*    72   624 */

	/* size: 696, cachelines: 11, members: 9 */
	/* sum members: 692, holes: 1, sum holes: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 56 bytes */
} __attribute__((__aligned__(8)));
struct proc_fs_opts {
	int                        flag;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	const char  *              str;                  /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct file_handle {
	__u32                      handle_bytes;         /*     0     4 */
	int                        handle_type;          /*     4     4 */
	unsigned char              f_handle[];           /*     8     0 */

	/* size: 8, cachelines: 1, members: 3 */
	/* last cacheline: 8 bytes */
};
struct inotify_inode_mark {
	struct fsnotify_mark       fsn_mark;             /*     0    72 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	int                        wd;                   /*    72     4 */

	/* size: 80, cachelines: 2, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct dnotify_struct {
	struct dnotify_struct *    dn_next;              /*     0     8 */
	__u32                      dn_mask;              /*     8     4 */
	int                        dn_fd;                /*    12     4 */
	struct file *              dn_filp;              /*    16     8 */
	fl_owner_t                 dn_owner;             /*    24     8 */

	/* size: 32, cachelines: 1, members: 5 */
	/* last cacheline: 32 bytes */
};
struct dnotify_mark {
	struct fsnotify_mark       fsn_mark;             /*     0    72 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	struct dnotify_struct *    dn;                   /*    72     8 */

	/* size: 80, cachelines: 2, members: 2 */
	/* last cacheline: 16 bytes */
};
struct inotify_event_info {
	struct fsnotify_event      fse;                  /*     0    24 */
	u32                        mask;                 /*    24     4 */
	int                        wd;                   /*    28     4 */
	u32                        sync_cookie;          /*    32     4 */
	int                        name_len;             /*    36     4 */
	char                       name[];               /*    40     0 */

	/* size: 40, cachelines: 1, members: 6 */
	/* last cacheline: 40 bytes */
};
struct inotify_event {
	__s32                      wd;                   /*     0     4 */
	__u32                      mask;                 /*     4     4 */
	__u32                      cookie;               /*     8     4 */
	__u32                      len;                  /*    12     4 */
	char                       name[];               /*    16     0 */

	/* size: 16, cachelines: 1, members: 5 */
	/* last cacheline: 16 bytes */
};
struct fanotify_fh {
	u8                         type;                 /*     0     1 */
	u8                         len;                  /*     1     1 */
	u8                         flags;                /*     2     1 */
	u8                         pad;                  /*     3     1 */
	unsigned char              buf[];                /*     4     0 */

	/* size: 4, cachelines: 1, members: 5 */
	/* last cacheline: 4 bytes */
} __attribute__((__aligned__(4)));
struct fanotify_info {
	u8                         dir_fh_totlen;        /*     0     1 */
	u8                         file_fh_totlen;       /*     1     1 */
	u8                         name_len;             /*     2     1 */
	u8                         pad;                  /*     3     1 */
	unsigned char              buf[];                /*     4     0 */

	/* size: 4, cachelines: 1, members: 5 */
	/* last cacheline: 4 bytes */
} __attribute__((__aligned__(4)));
struct fanotify_event {
	struct fsnotify_event      fse;                  /*     0    24 */
	u32                        mask;                 /*    24     4 */
	enum fanotify_event_type   type;                 /*    28     4 */
	struct pid *               pid;                  /*    32     8 */

	/* size: 40, cachelines: 1, members: 4 */
	/* last cacheline: 40 bytes */
};
struct fanotify_fid_event {
	struct fanotify_event      fae;                  /*     0    40 */
	__kernel_fsid_t            fsid;                 /*    40     8 */
	struct fanotify_fh         object_fh __attribute__((__aligned__(4))); /*    48     4 */
	unsigned char              _inline_fh_buf[12];   /*    52    12 */

	/* size: 64, cachelines: 1, members: 4 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(8)));
struct fanotify_name_event {
	struct fanotify_event      fae;                  /*     0    40 */
	__kernel_fsid_t            fsid;                 /*    40     8 */
	struct fanotify_info       info __attribute__((__aligned__(4))); /*    48     4 */

	/* size: 56, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 56 bytes */
} __attribute__((__aligned__(8)));
struct fanotify_path_event {
	struct fanotify_event      fae;                  /*     0    40 */
	struct path                path;                 /*    40    16 */

	/* size: 56, cachelines: 1, members: 2 */
	/* last cacheline: 56 bytes */
};
struct fanotify_perm_event {
	struct fanotify_event      fae;                  /*     0    40 */
	struct path                path;                 /*    40    16 */
	short unsigned int         response;             /*    56     2 */
	short unsigned int         state;                /*    58     2 */
	int                        fd;                   /*    60     4 */

	/* size: 64, cachelines: 1, members: 5 */
};
struct fanotify_event_metadata {
	__u32                      event_len;            /*     0     4 */
	__u8                       vers;                 /*     4     1 */
	__u8                       reserved;             /*     5     1 */
	__u16                      metadata_len;         /*     6     2 */
	__u64                      mask __attribute__((__aligned__(8))); /*     8     8 */
	__s32                      fd;                   /*    16     4 */
	__s32                      pid;                  /*    20     4 */

	/* size: 24, cachelines: 1, members: 7 */
	/* forced alignments: 1 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct fanotify_event_info_header {
	__u8                       info_type;            /*     0     1 */
	__u8                       pad;                  /*     1     1 */
	__u16                      len;                  /*     2     2 */

	/* size: 4, cachelines: 1, members: 3 */
	/* last cacheline: 4 bytes */
};
struct fanotify_event_info_fid {
	struct fanotify_event_info_header hdr;           /*     0     4 */
	__kernel_fsid_t            fsid;                 /*     4     8 */
	unsigned char              handle[];             /*    12     0 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct fanotify_response {
	__s32                      fd;                   /*     0     4 */
	__u32                      response;             /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct epoll_event {
	__poll_t                   events;               /*     0     4 */
	__u64                      data;                 /*     4     8 */

	/* size: 12, cachelines: 1, members: 2 */
	/* last cacheline: 12 bytes */
} __attribute__((__packed__));
struct epoll_filefd {
	struct file *              file;                 /*     0     8 */
	int                        fd;                   /*     8     4 */

	/* size: 12, cachelines: 1, members: 2 */
	/* last cacheline: 12 bytes */
} __attribute__((__packed__));
struct nested_call_node {
	struct list_head           llink;                /*     0    16 */
	void *                     cookie;               /*    16     8 */
	void *                     ctx;                  /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* last cacheline: 32 bytes */
};
struct nested_calls {
	struct list_head           tasks_call_list;      /*     0    16 */
	spinlock_t                 lock;                 /*    16     4 */

	/* size: 24, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct epitem {
	union {
		struct rb_node     rbn __attribute__((__aligned__(8))); /*     0    24 */
		struct callback_head rcu __attribute__((__aligned__(8))); /*     0    16 */
	} __attribute__((__aligned__(8)));               /*     0    24 */
	struct list_head           rdllink;              /*    24    16 */
	struct epitem *            next;                 /*    40     8 */
	struct epoll_filefd        ffd;                  /*    48    12 */
	int                        nwait;                /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct list_head           pwqlist;              /*    64    16 */
	struct eventpoll *         ep;                   /*    80     8 */
	struct list_head           fllink;               /*    88    16 */
	struct wakeup_source *     ws;                   /*   104     8 */
	struct epoll_event         event;                /*   112    12 */

	/* size: 128, cachelines: 2, members: 10 */
	/* padding: 4 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(8)));
struct eventpoll {
	struct mutex               mtx;                  /*     0    32 */
	wait_queue_head_t          wq;                   /*    32    24 */
	wait_queue_head_t          poll_wait;            /*    56    24 */
	/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
	struct list_head           rdllist;              /*    80    16 */
	rwlock_t                   lock;                 /*    96     8 */
	struct rb_root_cached      rbr;                  /*   104    16 */
	struct epitem *            ovflist;              /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct wakeup_source *     ws;                   /*   128     8 */
	struct user_struct *       user;                 /*   136     8 */
	struct file *              file;                 /*   144     8 */
	u64                        gen;                  /*   152     8 */
	unsigned int               napi_id;              /*   160     4 */

	/* size: 168, cachelines: 3, members: 12 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct eppoll_entry {
	struct list_head           llink;                /*     0    16 */
	struct epitem *            base;                 /*    16     8 */
	wait_queue_entry_t         wait;                 /*    24    40 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	wait_queue_head_t *        whead;                /*    64     8 */

	/* size: 72, cachelines: 2, members: 4 */
	/* last cacheline: 8 bytes */
};
struct ep_pqueue {
	poll_table                 pt;                   /*     0    16 */
	struct epitem *            epi;                  /*    16     8 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct ep_send_events_data {
	int                        maxevents;            /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct epoll_event *       events;               /*     8     8 */
	int                        res;                  /*    16     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 16, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct signalfd_siginfo {
	__u32                      ssi_signo;            /*     0     4 */
	__s32                      ssi_errno;            /*     4     4 */
	__s32                      ssi_code;             /*     8     4 */
	__u32                      ssi_pid;              /*    12     4 */
	__u32                      ssi_uid;              /*    16     4 */
	__s32                      ssi_fd;               /*    20     4 */
	__u32                      ssi_tid;              /*    24     4 */
	__u32                      ssi_band;             /*    28     4 */
	__u32                      ssi_overrun;          /*    32     4 */
	__u32                      ssi_trapno;           /*    36     4 */
	__s32                      ssi_status;           /*    40     4 */
	__s32                      ssi_int;              /*    44     4 */
	__u64                      ssi_ptr;              /*    48     8 */
	__u64                      ssi_utime;            /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	__u64                      ssi_stime;            /*    64     8 */
	__u64                      ssi_addr;             /*    72     8 */
	__u16                      ssi_addr_lsb;         /*    80     2 */
	__u16                      __pad2;               /*    82     2 */
	__s32                      ssi_syscall;          /*    84     4 */
	__u64                      ssi_call_addr;        /*    88     8 */
	__u32                      ssi_arch;             /*    96     4 */
	__u8                       __pad[28];            /*   100    28 */

	/* size: 128, cachelines: 2, members: 22 */
};
struct signalfd_ctx {
	sigset_t                   sigmask;              /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct timerfd_ctx {
	union {
		struct hrtimer     tmr __attribute__((__aligned__(8))); /*     0    96 */
		struct alarm       alarm __attribute__((__aligned__(8))); /*     0   152 */
	} t __attribute__((__aligned__(8)));             /*     0   152 */
	/* --- cacheline 2 boundary (128 bytes) was 24 bytes ago --- */
	ktime_t                    tintv;                /*   152     8 */
	ktime_t                    moffs;                /*   160     8 */
	wait_queue_head_t          wqh;                  /*   168    24 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	u64                        ticks;                /*   192     8 */
	int                        clockid;              /*   200     4 */
	short unsigned int         expired;              /*   204     2 */
	short unsigned int         settime_flags;        /*   206     2 */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*   208    16 */
	struct list_head           clist;                /*   224    16 */
	spinlock_t                 cancel_lock;          /*   240     4 */
	bool                       might_cancel;         /*   244     1 */

	/* size: 248, cachelines: 4, members: 12 */
	/* padding: 3 */
	/* forced alignments: 2 */
	/* last cacheline: 56 bytes */
} __attribute__((__aligned__(8)));
struct eventfd_ctx {
	struct kref                kref;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	wait_queue_head_t          wqh;                  /*     8    24 */
	__u64                      count;                /*    32     8 */
	unsigned int               flags;                /*    40     4 */
	int                        id;                   /*    44     4 */

	/* size: 48, cachelines: 1, members: 5 */
	/* sum members: 44, holes: 1, sum holes: 4 */
	/* last cacheline: 48 bytes */
};
struct userfaultfd_ctx {
	wait_queue_head_t          fault_pending_wqh;    /*     0    24 */
	wait_queue_head_t          fault_wqh;            /*    24    24 */
	wait_queue_head_t          fd_wqh;               /*    48    24 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	wait_queue_head_t          event_wqh;            /*    72    24 */
	seqcount_spinlock_t        refile_seq;           /*    96     4 */
	refcount_t                 refcount;             /*   100     4 */
	unsigned int               flags;                /*   104     4 */
	unsigned int               features;             /*   108     4 */
	bool                       released;             /*   112     1 */
	bool                       mmap_changing;        /*   113     1 */

	/* XXX 6 bytes hole, try to pack */

	struct mm_struct *         mm;                   /*   120     8 */

	/* size: 128, cachelines: 2, members: 11 */
	/* sum members: 122, holes: 1, sum holes: 6 */
};
struct uffd_msg {
	__u8                       event;                /*     0     1 */
	__u8                       reserved1;            /*     1     1 */
	__u16                      reserved2;            /*     2     2 */
	__u32                      reserved3;            /*     4     4 */
	union {
		struct {
			__u64      flags;                /*     8     8 */
			__u64      address;              /*    16     8 */
			union {
				__u32 ptid;              /*    24     4 */
			} feat;                          /*    24     4 */
		} pagefault;                             /*     8    24 */
		struct {
			__u32      ufd;                  /*     8     4 */
		} fork;                                  /*     8     4 */
		struct {
			__u64      from;                 /*     8     8 */
			__u64      to;                   /*    16     8 */
			__u64      len;                  /*    24     8 */
		} remap;                                 /*     8    24 */
		struct {
			__u64      start;                /*     8     8 */
			__u64      end;                  /*    16     8 */
		} remove;                                /*     8    16 */
		struct {
			__u64      reserved1;            /*     8     8 */
			__u64      reserved2;            /*    16     8 */
			__u64      reserved3;            /*    24     8 */
		} reserved;                              /*     8    24 */
	} arg;                                           /*     8    24 */

	/* size: 32, cachelines: 1, members: 5 */
	/* last cacheline: 32 bytes */
};
struct uffdio_api {
	__u64                      api;                  /*     0     8 */
	__u64                      features;             /*     8     8 */
	__u64                      ioctls;               /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct uffdio_range {
	__u64                      start;                /*     0     8 */
	__u64                      len;                  /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct uffdio_register {
	struct uffdio_range        range;                /*     0    16 */
	__u64                      mode;                 /*    16     8 */
	__u64                      ioctls;               /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* last cacheline: 32 bytes */
};
struct uffdio_copy {
	__u64                      dst;                  /*     0     8 */
	__u64                      src;                  /*     8     8 */
	__u64                      len;                  /*    16     8 */
	__u64                      mode;                 /*    24     8 */
	__s64                      copy;                 /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct uffdio_zeropage {
	struct uffdio_range        range;                /*     0    16 */
	__u64                      mode;                 /*    16     8 */
	__s64                      zeropage;             /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* last cacheline: 32 bytes */
};
struct uffdio_writeprotect {
	struct uffdio_range        range;                /*     0    16 */
	__u64                      mode;                 /*    16     8 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct userfaultfd_fork_ctx {
	struct userfaultfd_ctx *   orig;                 /*     0     8 */
	struct userfaultfd_ctx *   new;                  /*     8     8 */
	struct list_head           list;                 /*    16    16 */

	/* size: 32, cachelines: 1, members: 3 */
	/* last cacheline: 32 bytes */
};
struct userfaultfd_unmap_ctx {
	struct userfaultfd_ctx *   ctx;                  /*     0     8 */
	long unsigned int          start;                /*     8     8 */
	long unsigned int          end;                  /*    16     8 */
	struct list_head           list;                 /*    24    16 */

	/* size: 40, cachelines: 1, members: 4 */
	/* last cacheline: 40 bytes */
};
struct userfaultfd_wait_queue {
	struct uffd_msg            msg;                  /*     0    32 */
	wait_queue_entry_t         wq;                   /*    32    40 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	struct userfaultfd_ctx *   ctx;                  /*    72     8 */
	bool                       waken;                /*    80     1 */

	/* size: 88, cachelines: 2, members: 4 */
	/* padding: 7 */
	/* last cacheline: 24 bytes */
};
struct userfaultfd_wake_range {
	long unsigned int          start;                /*     0     8 */
	long unsigned int          len;                  /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct kioctx_table {
	struct callback_head       rcu __attribute__((__aligned__(8))); /*     0    16 */
	unsigned int               nr;                   /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	struct kioctx *            table[];              /*    24     0 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct io_event {
	__u64                      data;                 /*     0     8 */
	__u64                      obj;                  /*     8     8 */
	__s64                      res;                  /*    16     8 */
	__s64                      res2;                 /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct iocb {
	__u64                      aio_data;             /*     0     8 */
	__u32                      aio_key;              /*     8     4 */
	__kernel_rwf_t             aio_rw_flags;         /*    12     4 */
	__u16                      aio_lio_opcode;       /*    16     2 */
	__s16                      aio_reqprio;          /*    18     2 */
	__u32                      aio_fildes;           /*    20     4 */
	__u64                      aio_buf;              /*    24     8 */
	__u64                      aio_nbytes;           /*    32     8 */
	__s64                      aio_offset;           /*    40     8 */
	__u64                      aio_reserved2;        /*    48     8 */
	__u32                      aio_flags;            /*    56     4 */
	__u32                      aio_resfd;            /*    60     4 */

	/* size: 64, cachelines: 1, members: 12 */
};
struct aio_ring {
	unsigned int               id;                   /*     0     4 */
	unsigned int               nr;                   /*     4     4 */
	unsigned int               head;                 /*     8     4 */
	unsigned int               tail;                 /*    12     4 */
	unsigned int               magic;                /*    16     4 */
	unsigned int               compat_features;      /*    20     4 */
	unsigned int               incompat_features;    /*    24     4 */
	unsigned int               header_length;        /*    28     4 */
	struct io_event            io_events[];          /*    32     0 */

	/* size: 32, cachelines: 1, members: 9 */
	/* last cacheline: 32 bytes */
};
struct kioctx {
	struct percpu_ref          users;                /*     0    16 */
	atomic_t                   dead;                 /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	struct percpu_ref          reqs;                 /*    24    16 */
	long unsigned int          user_id;              /*    40     8 */
	struct kioctx_cpu *        cpu;                  /*    48     8 */
	unsigned int               req_batch;            /*    56     4 */
	unsigned int               max_reqs;             /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	unsigned int               nr_events;            /*    64     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          mmap_base;            /*    72     8 */
	long unsigned int          mmap_size;            /*    80     8 */
	struct page * *            ring_pages;           /*    88     8 */
	long int                   nr_pages;             /*    96     8 */
	struct rcu_work            free_rwork __attribute__((__aligned__(8))); /*   104    88 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	struct ctx_rq_wait *       rq_wait;              /*   192     8 */

	/* XXX 56 bytes hole, try to pack */

	/* --- cacheline 4 boundary (256 bytes) --- */
	struct {
		atomic_t           reqs_available;       /*   256     4 */
	} __attribute__((__aligned__(64))) __attribute__((__aligned__(64)));              /*   256    64 */

	/* XXX last struct has 60 bytes of padding */

	/* --- cacheline 5 boundary (320 bytes) --- */
	struct {
		spinlock_t         ctx_lock;             /*   320     4 */

		/* XXX 4 bytes hole, try to pack */

		struct list_head   active_reqs;          /*   328    16 */
	} __attribute__((__aligned__(64))) __attribute__((__aligned__(64)));              /*   320    64 */

	/* XXX last struct has 40 bytes of padding */

	/* --- cacheline 6 boundary (384 bytes) --- */
	struct {
		struct mutex       ring_lock;            /*   384    32 */
		wait_queue_head_t  wait;                 /*   416    24 */
	} __attribute__((__aligned__(64))) __attribute__((__aligned__(64)));              /*   384    64 */

	/* XXX last struct has 8 bytes of padding */

	/* --- cacheline 7 boundary (448 bytes) --- */
	struct {
		unsigned int       tail;                 /*   448     4 */
		unsigned int       completed_events;     /*   452     4 */
		spinlock_t         completion_lock;      /*   456     4 */
	} __attribute__((__aligned__(64))) __attribute__((__aligned__(64)));              /*   448    64 */

	/* XXX last struct has 52 bytes of padding */

	/* --- cacheline 8 boundary (512 bytes) --- */
	struct page *              internal_pages[8];    /*   512    64 */
	/* --- cacheline 9 boundary (576 bytes) --- */
	struct file *              aio_ring_file;        /*   576     8 */
	unsigned int               id;                   /*   584     4 */

	/* size: 640, cachelines: 10, members: 21 */
	/* sum members: 524, holes: 3, sum holes: 64 */
	/* padding: 52 */
	/* paddings: 4, sum paddings: 160 */
	/* forced alignments: 5, forced holes: 1, sum forced holes: 56 */
} __attribute__((__aligned__(64)));
struct kioctx_cpu {
	unsigned int               reqs_available;       /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct ctx_rq_wait {
	struct completion          comp;                 /*     0    32 */
	atomic_t                   count;                /*    32     4 */

	/* size: 40, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct fsync_iocb {
	struct file *              file;                 /*     0     8 */
	struct work_struct         work;                 /*     8    64 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	bool                       datasync;             /*    72     1 */

	/* XXX 7 bytes hole, try to pack */

	struct cred *              creds;                /*    80     8 */

	/* size: 88, cachelines: 2, members: 4 */
	/* sum members: 81, holes: 1, sum holes: 7 */
	/* last cacheline: 24 bytes */
};
struct poll_iocb {
	struct file *              file;                 /*     0     8 */
	struct wait_queue_head *   head;                 /*     8     8 */
	__poll_t                   events;               /*    16     4 */
	bool                       cancelled;            /*    20     1 */
	bool                       work_scheduled;       /*    21     1 */
	bool                       work_need_resched;    /*    22     1 */

	/* XXX 1 byte hole, try to pack */

	struct wait_queue_entry    wait;                 /*    24    40 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct work_struct         work;                 /*    64    64 */

	/* size: 128, cachelines: 2, members: 8 */
	/* sum members: 127, holes: 1, sum holes: 1 */
};
struct aio_kiocb {
	union {
		struct file *      ki_filp;              /*     0     8 */
		struct kiocb       rw;                   /*     0    48 */
		struct fsync_iocb  fsync;                /*     0    88 */
		struct poll_iocb   poll;                 /*     0   128 */
	};                                               /*     0   128 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct kioctx *            ki_ctx;               /*   128     8 */
	kiocb_cancel_fn *          ki_cancel;            /*   136     8 */
	struct io_event            ki_res;               /*   144    32 */
	struct list_head           ki_list;              /*   176    16 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	refcount_t                 ki_refcnt;            /*   192     4 */

	/* XXX 4 bytes hole, try to pack */

	struct eventfd_ctx *       ki_eventfd;           /*   200     8 */

	/* size: 208, cachelines: 4, members: 7 */
	/* sum members: 204, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct aio_poll_table {
	struct poll_table_struct   pt;                   /*     0    16 */

	/* XXX last struct has 4 bytes of padding */

	struct aio_kiocb *         iocb;                 /*    16     8 */
	bool                       queued;               /*    24     1 */

	/* XXX 3 bytes hole, try to pack */

	int                        error;                /*    28     4 */

	/* size: 32, cachelines: 1, members: 4 */
	/* sum members: 29, holes: 1, sum holes: 3 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 32 bytes */
};
struct __aio_sigset {
	const sigset_t  *          sigmask;              /*     0     8 */
	size_t                     sigsetsize;           /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct __compat_aio_sigset {
	compat_uptr_t              sigmask;              /*     0     4 */
	compat_size_t              sigsetsize;           /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct xa_limit {
	u32                        max;                  /*     0     4 */
	u32                        min;                  /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct user_msghdr {
	void *                     msg_name;             /*     0     8 */
	int                        msg_namelen;          /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct iovec *             msg_iov;              /*    16     8 */
	__kernel_size_t            msg_iovlen;           /*    24     8 */
	void *                     msg_control;          /*    32     8 */
	__kernel_size_t            msg_controllen;       /*    40     8 */
	unsigned int               msg_flags;            /*    48     4 */

	/* size: 56, cachelines: 1, members: 7 */
	/* sum members: 48, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 56 bytes */
};
struct compat_msghdr {
	compat_uptr_t              msg_name;             /*     0     4 */
	compat_int_t               msg_namelen;          /*     4     4 */
	compat_uptr_t              msg_iov;              /*     8     4 */
	compat_size_t              msg_iovlen;           /*    12     4 */
	compat_uptr_t              msg_control;          /*    16     4 */
	compat_size_t              msg_controllen;       /*    20     4 */
	compat_uint_t              msg_flags;            /*    24     4 */

	/* size: 28, cachelines: 1, members: 7 */
	/* last cacheline: 28 bytes */
};
struct scm_fp_list {
	short int                  count;                /*     0     2 */
	short int                  max;                  /*     2     2 */

	/* XXX 4 bytes hole, try to pack */

	struct user_struct *       user;                 /*     8     8 */
	struct file *              fp[253];              /*    16  2024 */

	/* size: 2040, cachelines: 32, members: 4 */
	/* sum members: 2036, holes: 1, sum holes: 4 */
	/* last cacheline: 56 bytes */
};
struct unix_skb_parms {
	struct pid *               pid;                  /*     0     8 */
	kuid_t                     uid;                  /*     8     4 */
	kgid_t                     gid;                  /*    12     4 */
	struct scm_fp_list *       fp;                   /*    16     8 */
	u32                        secid;                /*    24     4 */
	u32                        consumed;             /*    28     4 */

	/* size: 32, cachelines: 1, members: 6 */
	/* last cacheline: 32 bytes */
};
struct trace_event_raw_io_uring_create {
	struct trace_entry         ent;                  /*     0     8 */
	int                        fd;                   /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     ctx;                  /*    16     8 */
	u32                        sq_entries;           /*    24     4 */
	u32                        cq_entries;           /*    28     4 */
	u32                        flags;                /*    32     4 */
	char                       __data[];             /*    36     0 */

	/* size: 40, cachelines: 1, members: 7 */
	/* sum members: 32, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct trace_event_raw_io_uring_register {
	struct trace_entry         ent;                  /*     0     8 */
	void *                     ctx;                  /*     8     8 */
	unsigned int               opcode;               /*    16     4 */
	unsigned int               nr_files;             /*    20     4 */
	unsigned int               nr_bufs;              /*    24     4 */
	bool                       eventfd;              /*    28     1 */

	/* XXX 3 bytes hole, try to pack */

	long int                   ret;                  /*    32     8 */
	char                       __data[];             /*    40     0 */

	/* size: 40, cachelines: 1, members: 8 */
	/* sum members: 37, holes: 1, sum holes: 3 */
	/* last cacheline: 40 bytes */
};
struct trace_event_raw_io_uring_file_get {
	struct trace_entry         ent;                  /*     0     8 */
	void *                     ctx;                  /*     8     8 */
	int                        fd;                   /*    16     4 */
	char                       __data[];             /*    20     0 */

	/* size: 24, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_io_uring_queue_async_work {
	struct trace_entry         ent;                  /*     0     8 */
	void *                     ctx;                  /*     8     8 */
	int                        rw;                   /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     req;                  /*    24     8 */
	struct io_wq_work *        work;                 /*    32     8 */
	unsigned int               flags;                /*    40     4 */
	char                       __data[];             /*    44     0 */

	/* size: 48, cachelines: 1, members: 7 */
	/* sum members: 40, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 48 bytes */
};
struct io_wq_work {
	struct io_wq_work_node     list;                 /*     0     8 */
	struct io_identity *       identity;             /*     8     8 */
	unsigned int               flags;                /*    16     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_io_uring_defer {
	struct trace_entry         ent;                  /*     0     8 */
	void *                     ctx;                  /*     8     8 */
	void *                     req;                  /*    16     8 */
	long long unsigned int     data;                 /*    24     8 */
	char                       __data[];             /*    32     0 */

	/* size: 32, cachelines: 1, members: 5 */
	/* last cacheline: 32 bytes */
};
struct trace_event_raw_io_uring_link {
	struct trace_entry         ent;                  /*     0     8 */
	void *                     ctx;                  /*     8     8 */
	void *                     req;                  /*    16     8 */
	void *                     target_req;           /*    24     8 */
	char                       __data[];             /*    32     0 */

	/* size: 32, cachelines: 1, members: 5 */
	/* last cacheline: 32 bytes */
};
struct trace_event_raw_io_uring_cqring_wait {
	struct trace_entry         ent;                  /*     0     8 */
	void *                     ctx;                  /*     8     8 */
	int                        min_events;           /*    16     4 */
	char                       __data[];             /*    20     0 */

	/* size: 24, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_io_uring_fail_link {
	struct trace_entry         ent;                  /*     0     8 */
	void *                     req;                  /*     8     8 */
	void *                     link;                 /*    16     8 */
	char                       __data[];             /*    24     0 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_io_uring_complete {
	struct trace_entry         ent;                  /*     0     8 */
	void *                     ctx;                  /*     8     8 */
	u64                        user_data;            /*    16     8 */
	long int                   res;                  /*    24     8 */
	char                       __data[];             /*    32     0 */

	/* size: 32, cachelines: 1, members: 5 */
	/* last cacheline: 32 bytes */
};
struct trace_event_raw_io_uring_submit_sqe {
	struct trace_entry         ent;                  /*     0     8 */
	void *                     ctx;                  /*     8     8 */
	u8                         opcode;               /*    16     1 */

	/* XXX 7 bytes hole, try to pack */

	u64                        user_data;            /*    24     8 */
	bool                       force_nonblock;       /*    32     1 */
	bool                       sq_thread;            /*    33     1 */
	char                       __data[];             /*    34     0 */

	/* size: 40, cachelines: 1, members: 7 */
	/* sum members: 27, holes: 1, sum holes: 7 */
	/* padding: 6 */
	/* last cacheline: 40 bytes */
};
struct trace_event_raw_io_uring_poll_arm {
	struct trace_entry         ent;                  /*     0     8 */
	void *                     ctx;                  /*     8     8 */
	u8                         opcode;               /*    16     1 */

	/* XXX 7 bytes hole, try to pack */

	u64                        user_data;            /*    24     8 */
	int                        mask;                 /*    32     4 */
	int                        events;               /*    36     4 */
	char                       __data[];             /*    40     0 */

	/* size: 40, cachelines: 1, members: 7 */
	/* sum members: 33, holes: 1, sum holes: 7 */
	/* last cacheline: 40 bytes */
};
struct trace_event_raw_io_uring_poll_wake {
	struct trace_entry         ent;                  /*     0     8 */
	void *                     ctx;                  /*     8     8 */
	u8                         opcode;               /*    16     1 */

	/* XXX 7 bytes hole, try to pack */

	u64                        user_data;            /*    24     8 */
	int                        mask;                 /*    32     4 */
	char                       __data[];             /*    36     0 */

	/* size: 40, cachelines: 1, members: 6 */
	/* sum members: 29, holes: 1, sum holes: 7 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct trace_event_raw_io_uring_task_add {
	struct trace_entry         ent;                  /*     0     8 */
	void *                     ctx;                  /*     8     8 */
	u8                         opcode;               /*    16     1 */

	/* XXX 7 bytes hole, try to pack */

	u64                        user_data;            /*    24     8 */
	int                        mask;                 /*    32     4 */
	char                       __data[];             /*    36     0 */

	/* size: 40, cachelines: 1, members: 6 */
	/* sum members: 29, holes: 1, sum holes: 7 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct trace_event_raw_io_uring_task_run {
	struct trace_entry         ent;                  /*     0     8 */
	void *                     ctx;                  /*     8     8 */
	u8                         opcode;               /*    16     1 */

	/* XXX 7 bytes hole, try to pack */

	u64                        user_data;            /*    24     8 */
	char                       __data[];             /*    32     0 */

	/* size: 32, cachelines: 1, members: 5 */
	/* sum members: 25, holes: 1, sum holes: 7 */
	/* last cacheline: 32 bytes */
};
struct trace_event_data_offsets_io_uring_create {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_io_uring_register {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_io_uring_file_get {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_io_uring_queue_async_work {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_io_uring_defer {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_io_uring_link {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_io_uring_cqring_wait {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_io_uring_fail_link {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_io_uring_complete {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_io_uring_submit_sqe {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_io_uring_poll_arm {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_io_uring_poll_wake {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_io_uring_task_add {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_io_uring_task_run {

	/* size: 0, cachelines: 0, members: 0 */
};
struct io_uring_sqe {
	__u8                       opcode;               /*     0     1 */
	__u8                       flags;                /*     1     1 */
	__u16                      ioprio;               /*     2     2 */
	__s32                      fd;                   /*     4     4 */
	union {
		__u64              off;                  /*     8     8 */
		__u64              addr2;                /*     8     8 */
	};                                               /*     8     8 */
	union {
		__u64              addr;                 /*    16     8 */
		__u64              splice_off_in;        /*    16     8 */
	};                                               /*    16     8 */
	__u32                      len;                  /*    24     4 */
	union {
		__kernel_rwf_t     rw_flags;             /*    28     4 */
		__u32              fsync_flags;          /*    28     4 */
		__u16              poll_events;          /*    28     2 */
		__u32              poll32_events;        /*    28     4 */
		__u32              sync_range_flags;     /*    28     4 */
		__u32              msg_flags;            /*    28     4 */
		__u32              timeout_flags;        /*    28     4 */
		__u32              accept_flags;         /*    28     4 */
		__u32              cancel_flags;         /*    28     4 */
		__u32              open_flags;           /*    28     4 */
		__u32              statx_flags;          /*    28     4 */
		__u32              fadvise_advice;       /*    28     4 */
		__u32              splice_flags;         /*    28     4 */
	};                                               /*    28     4 */
	__u64                      user_data;            /*    32     8 */
	union {
		struct {
			union {
				__u16 buf_index;         /*    40     2 */
				__u16 buf_group;         /*    40     2 */
			};                               /*    40     2 */
			__u16      personality;          /*    42     2 */
			__s32      splice_fd_in;         /*    44     4 */
		};                                       /*    40     8 */
		__u64              __pad2[3];            /*    40    24 */
	};                                               /*    40    24 */

	/* size: 64, cachelines: 1, members: 10 */
};
struct io_uring_cqe {
	__u64                      user_data;            /*     0     8 */
	__s32                      res;                  /*     8     4 */
	__u32                      flags;                /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct io_sqring_offsets {
	__u32                      head;                 /*     0     4 */
	__u32                      tail;                 /*     4     4 */
	__u32                      ring_mask;            /*     8     4 */
	__u32                      ring_entries;         /*    12     4 */
	__u32                      flags;                /*    16     4 */
	__u32                      dropped;              /*    20     4 */
	__u32                      array;                /*    24     4 */
	__u32                      resv1;                /*    28     4 */
	__u64                      resv2;                /*    32     8 */

	/* size: 40, cachelines: 1, members: 9 */
	/* last cacheline: 40 bytes */
};
struct io_cqring_offsets {
	__u32                      head;                 /*     0     4 */
	__u32                      tail;                 /*     4     4 */
	__u32                      ring_mask;            /*     8     4 */
	__u32                      ring_entries;         /*    12     4 */
	__u32                      overflow;             /*    16     4 */
	__u32                      cqes;                 /*    20     4 */
	__u32                      flags;                /*    24     4 */
	__u32                      resv1;                /*    28     4 */
	__u64                      resv2;                /*    32     8 */

	/* size: 40, cachelines: 1, members: 9 */
	/* last cacheline: 40 bytes */
};
struct io_uring_params {
	__u32                      sq_entries;           /*     0     4 */
	__u32                      cq_entries;           /*     4     4 */
	__u32                      flags;                /*     8     4 */
	__u32                      sq_thread_cpu;        /*    12     4 */
	__u32                      sq_thread_idle;       /*    16     4 */
	__u32                      features;             /*    20     4 */
	__u32                      wq_fd;                /*    24     4 */
	__u32                      resv[3];              /*    28    12 */
	struct io_sqring_offsets   sq_off;               /*    40    40 */
	/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
	struct io_cqring_offsets   cq_off;               /*    80    40 */

	/* size: 120, cachelines: 2, members: 10 */
	/* last cacheline: 56 bytes */
};
struct io_uring_files_update {
	__u32                      offset;               /*     0     4 */
	__u32                      resv;                 /*     4     4 */
	__u64                      fds __attribute__((__aligned__(8))); /*     8     8 */

	/* size: 16, cachelines: 1, members: 3 */
	/* forced alignments: 1 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct io_uring_probe_op {
	__u8                       op;                   /*     0     1 */
	__u8                       resv;                 /*     1     1 */
	__u16                      flags;                /*     2     2 */
	__u32                      resv2;                /*     4     4 */

	/* size: 8, cachelines: 1, members: 4 */
	/* last cacheline: 8 bytes */
};
struct io_uring_probe {
	__u8                       last_op;              /*     0     1 */
	__u8                       ops_len;              /*     1     1 */
	__u16                      resv;                 /*     2     2 */
	__u32                      resv2[3];             /*     4    12 */
	struct io_uring_probe_op   ops[];                /*    16     0 */

	/* size: 16, cachelines: 1, members: 5 */
	/* last cacheline: 16 bytes */
};
struct io_uring_restriction {
	__u16                      opcode;               /*     0     2 */
	union {
		__u8               register_op;          /*     2     1 */
		__u8               sqe_op;               /*     2     1 */
		__u8               sqe_flags;            /*     2     1 */
	};                                               /*     2     1 */
	__u8                       resv;                 /*     3     1 */
	__u32                      resv2[3];             /*     4    12 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct io_wq_work_node {
	struct io_wq_work_node *   next;                 /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct io_wq_data {
	struct user_struct *       user;                 /*     0     8 */
	io_wq_work_fn *            do_work;              /*     8     8 */
	free_work_fn *             free_work;            /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct io_uring {
	u32                        head __attribute__((__aligned__(64))); /*     0     4 */

	/* XXX 60 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	u32                        tail __attribute__((__aligned__(64))); /*    64     4 */

	/* size: 128, cachelines: 2, members: 2 */
	/* sum members: 8, holes: 1, sum holes: 60 */
	/* padding: 60 */
	/* forced alignments: 2, forced holes: 1, sum forced holes: 60 */
} __attribute__((__aligned__(64)));
struct io_rings {
	struct io_uring            sq __attribute__((__aligned__(64))); /*     0   128 */

	/* XXX last struct has 60 bytes of padding */

	/* --- cacheline 2 boundary (128 bytes) --- */
	struct io_uring            cq __attribute__((__aligned__(64))); /*   128   128 */

	/* XXX last struct has 60 bytes of padding */

	/* --- cacheline 4 boundary (256 bytes) --- */
	u32                        sq_ring_mask;         /*   256     4 */
	u32                        cq_ring_mask;         /*   260     4 */
	u32                        sq_ring_entries;      /*   264     4 */
	u32                        cq_ring_entries;      /*   268     4 */
	u32                        sq_dropped;           /*   272     4 */
	u32                        sq_flags;             /*   276     4 */
	u32                        cq_flags;             /*   280     4 */
	u32                        cq_overflow;          /*   284     4 */

	/* XXX 32 bytes hole, try to pack */

	/* --- cacheline 5 boundary (320 bytes) --- */
	struct io_uring_cqe        cqes[] __attribute__((__aligned__(64))); /*   320     0 */

	/* size: 320, cachelines: 5, members: 11 */
	/* sum members: 288, holes: 1, sum holes: 32 */
	/* paddings: 2, sum paddings: 120 */
	/* forced alignments: 3, forced holes: 1, sum forced holes: 32 */
} __attribute__((__aligned__(64)));
struct io_mapped_ubuf {
	u64                        ubuf;                 /*     0     8 */
	size_t                     len;                  /*     8     8 */
	struct bio_vec *           bvec;                 /*    16     8 */
	unsigned int               nr_bvecs;             /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          acct_pages;           /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* sum members: 36, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct fixed_file_table {
	struct file * *            files;                /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct fixed_file_ref_node {
	struct percpu_ref          refs;                 /*     0    16 */
	struct list_head           node;                 /*    16    16 */
	struct list_head           file_list;            /*    32    16 */
	struct fixed_file_data *   file_data;            /*    48     8 */
	struct llist_node          llist;                /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	bool                       done;                 /*    64     1 */

	/* size: 72, cachelines: 2, members: 6 */
	/* padding: 7 */
	/* last cacheline: 8 bytes */
};
struct fixed_file_data {
	struct fixed_file_table *  table;                /*     0     8 */
	struct io_ring_ctx *       ctx;                  /*     8     8 */
	struct fixed_file_ref_node * node;               /*    16     8 */
	struct percpu_ref          refs;                 /*    24    16 */
	struct completion          done;                 /*    40    32 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	struct list_head           ref_list;             /*    72    16 */
	spinlock_t                 lock;                 /*    88     4 */

	/* size: 96, cachelines: 2, members: 7 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct io_ring_ctx {
	struct {
		struct percpu_ref  refs;                 /*     0    16 */
	} __attribute__((__aligned__(64))) __attribute__((__aligned__(64)));              /*     0    64 */

	/* XXX last struct has 48 bytes of padding */

	/* --- cacheline 1 boundary (64 bytes) --- */
	struct {
		unsigned int       flags;                /*    64     4 */
		unsigned int       compat:1;             /*    68: 0  4 */
		unsigned int       limit_mem:1;          /*    68: 1  4 */
		unsigned int       cq_overflow_flushed:1; /*    68: 2  4 */
		unsigned int       drain_next:1;         /*    68: 3  4 */
		unsigned int       eventfd_async:1;      /*    68: 4  4 */
		unsigned int       restricted:1;         /*    68: 5  4 */
		unsigned int       sqo_dead:1;           /*    68: 6  4 */

		/* XXX 25 bits hole, try to pack */

		u32 *              sq_array;             /*    72     8 */
		unsigned int       cached_sq_head;       /*    80     4 */
		unsigned int       sq_entries;           /*    84     4 */
		unsigned int       sq_mask;              /*    88     4 */
		unsigned int       sq_thread_idle;       /*    92     4 */
		unsigned int       cached_sq_dropped;    /*    96     4 */
		unsigned int       cached_cq_overflow;   /*   100     4 */
		long unsigned int  sq_check_overflow;    /*   104     8 */
		struct list_head   defer_list;           /*   112    16 */
		/* --- cacheline 2 boundary (128 bytes) --- */
		struct list_head   timeout_list;         /*   128    16 */
		struct list_head   cq_overflow_list;     /*   144    16 */
		struct io_uring_sqe * sq_sqes;           /*   160     8 */
	} __attribute__((__aligned__(64))) __attribute__((__aligned__(64)));              /*    64   128 */

	/* XXX last struct has 24 bytes of padding */

	/* --- cacheline 3 boundary (192 bytes) --- */
	struct io_rings *          rings;                /*   192     8 */
	struct io_wq *             io_wq;                /*   200     8 */
	struct task_struct *       sqo_task;             /*   208     8 */
	struct mm_struct *         mm_account;           /*   216     8 */
	struct cgroup_subsys_state * sqo_blkcg_css;      /*   224     8 */
	struct io_sq_data *        sq_data;              /*   232     8 */
	struct wait_queue_head     sqo_sq_wait;          /*   240    24 */
	/* --- cacheline 4 boundary (256 bytes) was 8 bytes ago --- */
	struct wait_queue_entry    sqo_wait_entry;       /*   264    40 */
	struct list_head           sqd_list;             /*   304    16 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	struct fixed_file_data *   file_data;            /*   320     8 */
	unsigned int               nr_user_files;        /*   328     4 */
	unsigned int               nr_user_bufs;         /*   332     4 */
	struct io_mapped_ubuf *    user_bufs;            /*   336     8 */
	struct user_struct *       user;                 /*   344     8 */
	const struct cred  *       creds;                /*   352     8 */
	kuid_t                     loginuid;             /*   360     4 */
	unsigned int               sessionid;            /*   364     4 */
	struct completion          ref_comp;             /*   368    32 */
	/* --- cacheline 6 boundary (384 bytes) was 16 bytes ago --- */
	struct completion          sq_thread_comp;       /*   400    32 */
	struct io_kiocb *          fallback_req;         /*   432     8 */
	struct socket *            ring_sock;            /*   440     8 */
	/* --- cacheline 7 boundary (448 bytes) --- */
	struct xarray              io_buffers;           /*   448    16 */
	struct xarray              personalities;        /*   464    16 */
	u32                        pers_next;            /*   480     4 */

	/* XXX 28 bytes hole, try to pack */

	/* --- cacheline 8 boundary (512 bytes) --- */
	struct {
		unsigned int       cached_cq_tail;       /*   512     4 */
		unsigned int       cq_entries;           /*   516     4 */
		unsigned int       cq_mask;              /*   520     4 */
		atomic_t           cq_timeouts;          /*   524     4 */
		unsigned int       cq_last_tm_flush;     /*   528     4 */

		/* XXX 4 bytes hole, try to pack */

		long unsigned int  cq_check_overflow;    /*   536     8 */
		struct wait_queue_head cq_wait;          /*   544    24 */
		struct fasync_struct * cq_fasync;        /*   568     8 */
		/* --- cacheline 9 boundary (576 bytes) --- */
		struct eventfd_ctx * cq_ev_fd;           /*   576     8 */
	} __attribute__((__aligned__(64))) __attribute__((__aligned__(64)));              /*   512   128 */

	/* XXX last struct has 56 bytes of padding */

	/* --- cacheline 10 boundary (640 bytes) --- */
	struct {
		struct mutex       uring_lock;           /*   640    32 */
		wait_queue_head_t  wait;                 /*   672    24 */
	} __attribute__((__aligned__(64))) __attribute__((__aligned__(64)));              /*   640    64 */

	/* XXX last struct has 8 bytes of padding */

	/* --- cacheline 11 boundary (704 bytes) --- */
	struct {
		spinlock_t         completion_lock;      /*   704     4 */

		/* XXX 4 bytes hole, try to pack */

		struct list_head   iopoll_list;          /*   712    16 */
		struct hlist_head * cancel_hash;         /*   728     8 */
		unsigned int       cancel_hash_bits;     /*   736     4 */
		bool               poll_multi_file;      /*   740     1 */

		/* XXX 3 bytes hole, try to pack */

		spinlock_t         inflight_lock;        /*   744     4 */

		/* XXX 4 bytes hole, try to pack */

		struct list_head   inflight_list;        /*   752    16 */
	} __attribute__((__aligned__(64))) __attribute__((__aligned__(64)));              /*   704    64 */
	/* --- cacheline 12 boundary (768 bytes) --- */
	struct delayed_work        file_put_work;        /*   768   192 */
	/* --- cacheline 15 boundary (960 bytes) --- */
	struct llist_head          file_put_llist;       /*   960     8 */
	struct work_struct         exit_work;            /*   968    64 */
	/* --- cacheline 16 boundary (1024 bytes) was 8 bytes ago --- */
	struct io_restriction      restrictions;         /*  1032    24 */

	/* XXX last struct has 5 bytes of padding */

	/* size: 1088, cachelines: 17, members: 33 */
	/* sum members: 1028, holes: 1, sum holes: 28 */
	/* padding: 32 */
	/* paddings: 5, sum paddings: 141 */
	/* forced alignments: 5, forced holes: 1, sum forced holes: 28 */
} __attribute__((__aligned__(64)));
struct io_buffer {
	struct list_head           list;                 /*     0    16 */
	__u64                      addr;                 /*    16     8 */
	__u32                      len;                  /*    24     4 */
	__u16                      bid;                  /*    28     2 */

	/* size: 32, cachelines: 1, members: 4 */
	/* padding: 2 */
	/* last cacheline: 32 bytes */
};
struct io_restriction {
	long unsigned int          register_op[1];       /*     0     8 */
	long unsigned int          sqe_op[1];            /*     8     8 */
	u8                         sqe_flags_allowed;    /*    16     1 */
	u8                         sqe_flags_required;   /*    17     1 */
	bool                       registered;           /*    18     1 */

	/* size: 24, cachelines: 1, members: 5 */
	/* padding: 5 */
	/* last cacheline: 24 bytes */
};
struct io_sq_data {
	refcount_t                 refs;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct mutex               lock;                 /*     8    32 */
	struct list_head           ctx_list;             /*    40    16 */
	struct list_head           ctx_new_list;         /*    56    16 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	struct mutex               ctx_lock;             /*    72    32 */
	struct task_struct *       thread;               /*   104     8 */
	struct wait_queue_head     wait;                 /*   112    24 */

	/* size: 136, cachelines: 3, members: 7 */
	/* sum members: 132, holes: 1, sum holes: 4 */
	/* last cacheline: 8 bytes */
};
struct io_kiocb {
	union {
		struct file *      file;                 /*     0     8 */
		struct io_rw       rw;                   /*     0    64 */
		struct io_poll_iocb poll;                /*     0    64 */
		struct io_accept   accept;               /*     0    40 */
		struct io_sync     sync;                 /*     0    32 */
		struct io_cancel   cancel;               /*     0    16 */
		struct io_timeout  timeout;              /*     0    32 */
		struct io_timeout_rem timeout_rem;       /*     0    16 */
		struct io_connect  connect;              /*     0    24 */
		struct io_sr_msg   sr_msg;               /*     0    40 */
		struct io_open     open;                 /*     0    56 */
		struct io_close    close;                /*     0    24 */
		struct io_files_update files_update;     /*     0    24 */
		struct io_fadvise  fadvise;              /*     0    24 */
		struct io_madvise  madvise;              /*     0    24 */
		struct io_epoll    epoll;                /*     0    32 */
		struct io_splice   splice;               /*     0    48 */
		struct io_provide_buf pbuf;              /*     0    32 */
		struct io_statx    statx;                /*     0    40 */
		struct io_completion compl;              /*     0    32 */
	};                                               /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	void *                     async_data;           /*    64     8 */
	u8                         opcode;               /*    72     1 */
	u8                         iopoll_completed;     /*    73     1 */
	u16                        buf_index;            /*    74     2 */
	u32                        result;               /*    76     4 */
	struct io_ring_ctx *       ctx;                  /*    80     8 */
	unsigned int               flags;                /*    88     4 */
	refcount_t                 refs;                 /*    92     4 */
	struct task_struct *       task;                 /*    96     8 */
	u64                        user_data;            /*   104     8 */
	struct list_head           link_list;            /*   112    16 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct list_head           inflight_entry;       /*   128    16 */
	struct list_head           iopoll_entry;         /*   144    16 */
	struct percpu_ref *        fixed_file_refs;      /*   160     8 */
	struct callback_head       task_work __attribute__((__aligned__(8))); /*   168    16 */
	struct hlist_node          hash_node;            /*   184    16 */
	/* --- cacheline 3 boundary (192 bytes) was 8 bytes ago --- */
	struct async_poll *        apoll;                /*   200     8 */
	struct io_wq_work          work;                 /*   208    24 */

	/* XXX last struct has 4 bytes of padding */

	/* size: 232, cachelines: 4, members: 19 */
	/* paddings: 1, sum paddings: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 40 bytes */
} __attribute__((__aligned__(8)));
struct io_poll_iocb {
	struct file *              file;                 /*     0     8 */
	union {
		struct wait_queue_head * head;           /*     8     8 */
		u64                addr;                 /*     8     8 */
	};                                               /*     8     8 */
	__poll_t                   events;               /*    16     4 */
	bool                       done;                 /*    20     1 */
	bool                       canceled;             /*    21     1 */

	/* XXX 2 bytes hole, try to pack */

	struct wait_queue_entry    wait;                 /*    24    40 */

	/* size: 64, cachelines: 1, members: 6 */
	/* sum members: 62, holes: 1, sum holes: 2 */
};
struct io_close {
	struct file *              file;                 /*     0     8 */
	struct file *              put_file;             /*     8     8 */
	int                        fd;                   /*    16     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct io_timeout_data {
	struct io_kiocb *          req;                  /*     0     8 */
	struct hrtimer             timer __attribute__((__aligned__(8))); /*     8    96 */
	/* --- cacheline 1 boundary (64 bytes) was 40 bytes ago --- */
	struct timespec64          ts;                   /*   104    16 */
	enum hrtimer_mode          mode;                 /*   120     4 */

	/* size: 128, cachelines: 2, members: 4 */
	/* padding: 4 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(8)));
struct io_accept {
	struct file *              file;                 /*     0     8 */
	struct sockaddr *          addr;                 /*     8     8 */
	int *                      addr_len;             /*    16     8 */
	int                        flags;                /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          nofile;               /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* sum members: 36, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct io_sync {
	struct file *              file;                 /*     0     8 */
	loff_t                     len;                  /*     8     8 */
	loff_t                     off;                  /*    16     8 */
	int                        flags;                /*    24     4 */
	int                        mode;                 /*    28     4 */

	/* size: 32, cachelines: 1, members: 5 */
	/* last cacheline: 32 bytes */
};
struct io_cancel {
	struct file *              file;                 /*     0     8 */
	u64                        addr;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct io_timeout {
	struct file *              file;                 /*     0     8 */
	u32                        off;                  /*     8     4 */
	u32                        target_seq;           /*    12     4 */
	struct list_head           list;                 /*    16    16 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct io_timeout_rem {
	struct file *              file;                 /*     0     8 */
	u64                        addr;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct io_rw {
	struct kiocb               kiocb;                /*     0    48 */
	u64                        addr;                 /*    48     8 */
	u64                        len;                  /*    56     8 */

	/* size: 64, cachelines: 1, members: 3 */
};
struct io_connect {
	struct file *              file;                 /*     0     8 */
	struct sockaddr *          addr;                 /*     8     8 */
	int                        addr_len;             /*    16     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct io_sr_msg {
	struct file *              file;                 /*     0     8 */
	union {
		struct user_msghdr * umsg;               /*     8     8 */
		void *             buf;                  /*     8     8 */
	};                                               /*     8     8 */
	int                        msg_flags;            /*    16     4 */
	int                        bgid;                 /*    20     4 */
	size_t                     len;                  /*    24     8 */
	struct io_buffer *         kbuf;                 /*    32     8 */

	/* size: 40, cachelines: 1, members: 6 */
	/* last cacheline: 40 bytes */
};
struct io_open {
	struct file *              file;                 /*     0     8 */
	int                        dfd;                  /*     8     4 */
	bool                       ignore_nonblock;      /*    12     1 */

	/* XXX 3 bytes hole, try to pack */

	struct filename *          filename;             /*    16     8 */
	struct open_how            how;                  /*    24    24 */
	long unsigned int          nofile;               /*    48     8 */

	/* size: 56, cachelines: 1, members: 6 */
	/* sum members: 53, holes: 1, sum holes: 3 */
	/* last cacheline: 56 bytes */
};
struct io_files_update {
	struct file *              file;                 /*     0     8 */
	u64                        arg;                  /*     8     8 */
	u32                        nr_args;              /*    16     4 */
	u32                        offset;               /*    20     4 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct io_fadvise {
	struct file *              file;                 /*     0     8 */
	u64                        offset;               /*     8     8 */
	u32                        len;                  /*    16     4 */
	u32                        advice;               /*    20     4 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct io_madvise {
	struct file *              file;                 /*     0     8 */
	u64                        addr;                 /*     8     8 */
	u32                        len;                  /*    16     4 */
	u32                        advice;               /*    20     4 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct io_epoll {
	struct file *              file;                 /*     0     8 */
	int                        epfd;                 /*     8     4 */
	int                        op;                   /*    12     4 */
	int                        fd;                   /*    16     4 */
	struct epoll_event         event;                /*    20    12 */

	/* size: 32, cachelines: 1, members: 5 */
	/* last cacheline: 32 bytes */
} __attribute__((__packed__));
struct io_splice {
	struct file *              file_out;             /*     0     8 */
	struct file *              file_in;              /*     8     8 */
	loff_t                     off_out;              /*    16     8 */
	loff_t                     off_in;               /*    24     8 */
	u64                        len;                  /*    32     8 */
	unsigned int               flags;                /*    40     4 */

	/* size: 48, cachelines: 1, members: 6 */
	/* padding: 4 */
	/* last cacheline: 48 bytes */
};
struct io_provide_buf {
	struct file *              file;                 /*     0     8 */
	__u64                      addr;                 /*     8     8 */
	__u32                      len;                  /*    16     4 */
	__u32                      bgid;                 /*    20     4 */
	__u16                      nbufs;                /*    24     2 */
	__u16                      bid;                  /*    26     2 */

	/* size: 32, cachelines: 1, members: 6 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct io_statx {
	struct file *              file;                 /*     0     8 */
	int                        dfd;                  /*     8     4 */
	unsigned int               mask;                 /*    12     4 */
	unsigned int               flags;                /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	const char  *              filename;             /*    24     8 */
	struct statx *             buffer;               /*    32     8 */

	/* size: 40, cachelines: 1, members: 6 */
	/* sum members: 36, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct io_completion {
	struct file *              file;                 /*     0     8 */
	struct list_head           list;                 /*     8    16 */
	u32                        cflags;               /*    24     4 */

	/* size: 32, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct io_async_connect {
	struct __kernel_sockaddr_storage address;        /*     0   128 */

	/* size: 128, cachelines: 2, members: 1 */
};
struct io_async_msghdr {
	struct iovec               fast_iov[8];          /*     0   128 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct iovec *             iov;                  /*   128     8 */
	struct sockaddr *          uaddr;                /*   136     8 */
	struct msghdr              msg;                  /*   144    96 */
	/* --- cacheline 3 boundary (192 bytes) was 48 bytes ago --- */
	struct __kernel_sockaddr_storage addr;           /*   240   128 */

	/* size: 368, cachelines: 6, members: 5 */
	/* last cacheline: 48 bytes */
};
struct io_async_rw {
	struct iovec               fast_iov[8];          /*     0   128 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	const struct iovec  *      free_iovec;           /*   128     8 */
	struct iov_iter            iter;                 /*   136    40 */
	size_t                     bytes_done;           /*   176     8 */
	struct wait_page_queue     wpq;                  /*   184    56 */

	/* size: 240, cachelines: 4, members: 5 */
	/* last cacheline: 48 bytes */
};
struct async_poll {
	struct io_poll_iocb        poll;                 /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct io_poll_iocb *      double_poll;          /*    64     8 */

	/* size: 72, cachelines: 2, members: 2 */
	/* last cacheline: 8 bytes */
};
struct io_defer_entry {
	struct list_head           list;                 /*     0    16 */
	struct io_kiocb *          req;                  /*    16     8 */
	u32                        seq;                  /*    24     4 */

	/* size: 32, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct io_comp_state {
	unsigned int               nr;                   /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           list;                 /*     8    16 */
	struct io_ring_ctx *       ctx;                  /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct io_submit_state {
	struct blk_plug            plug;                 /*     0    40 */

	/* XXX last struct has 4 bytes of padding */

	void *                     reqs[8];              /*    40    64 */
	/* --- cacheline 1 boundary (64 bytes) was 40 bytes ago --- */
	unsigned int               free_reqs;            /*   104     4 */

	/* XXX 4 bytes hole, try to pack */

	struct io_comp_state       comp;                 /*   112    32 */
	/* --- cacheline 2 boundary (128 bytes) was 16 bytes ago --- */
	struct file *              file;                 /*   144     8 */
	unsigned int               fd;                   /*   152     4 */
	unsigned int               has_refs;             /*   156     4 */
	unsigned int               ios_left;             /*   160     4 */

	/* size: 168, cachelines: 3, members: 8 */
	/* sum members: 160, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 40 bytes */
};
struct io_op_def {
	unsigned int               needs_file:1;         /*     0: 0  4 */
	unsigned int               needs_file_no_error:1; /*     0: 1  4 */
	unsigned int               hash_reg_file:1;      /*     0: 2  4 */
	unsigned int               unbound_nonreg_file:1; /*     0: 3  4 */
	unsigned int               not_supported:1;      /*     0: 4  4 */
	unsigned int               pollin:1;             /*     0: 5  4 */
	unsigned int               pollout:1;            /*     0: 6  4 */
	unsigned int               buffer_select:1;      /*     0: 7  4 */
	unsigned int               needs_async_data:1;   /*     0: 8  4 */

	/* XXX 7 bits hole, try to pack */
	/* Bitfield combined with next fields */

	short unsigned int         async_size;           /*     2     2 */
	unsigned int               work_flags;           /*     4     4 */

	/* size: 8, cachelines: 1, members: 11 */
	/* sum members: 6 */
	/* sum bitfield members: 9 bits, bit holes: 1, sum bit holes: 7 bits */
	/* last cacheline: 8 bytes */
};
struct req_batch {
	void *                     reqs[8];              /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        to_free;              /*    64     4 */

	/* XXX 4 bytes hole, try to pack */

	struct task_struct *       task;                 /*    72     8 */
	int                        task_refs;            /*    80     4 */

	/* size: 88, cachelines: 2, members: 4 */
	/* sum members: 80, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct io_poll_table {
	struct poll_table_struct   pt;                   /*     0    16 */

	/* XXX last struct has 4 bytes of padding */

	struct io_kiocb *          req;                  /*    16     8 */
	int                        nr_entries;           /*    24     4 */
	int                        error;                /*    28     4 */

	/* size: 32, cachelines: 1, members: 4 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 32 bytes */
};
struct io_wait_queue {
	struct wait_queue_entry    wq;                   /*     0    40 */
	struct io_ring_ctx *       ctx;                  /*    40     8 */
	unsigned int               to_wait;              /*    48     4 */
	unsigned int               nr_timeouts;          /*    52     4 */

	/* size: 56, cachelines: 1, members: 4 */
	/* last cacheline: 56 bytes */
};
struct io_file_put {
	struct list_head           list;                 /*     0    16 */
	struct file *              file;                 /*    16     8 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct io_task_cancel {
	struct task_struct *       task;                 /*     0     8 */
	struct files_struct *      files;                /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct io_wq_work_list {
	struct io_wq_work_node *   first;                /*     0     8 */
	struct io_wq_work_node *   last;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct io_worker {
	refcount_t                 ref;                  /*     0     4 */
	unsigned int               flags;                /*     4     4 */
	struct hlist_nulls_node    nulls_node;           /*     8    16 */
	struct list_head           all_list;             /*    24    16 */
	struct task_struct *       task;                 /*    40     8 */
	struct io_wqe *            wqe;                  /*    48     8 */
	struct io_wq_work *        cur_work;             /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	spinlock_t                 lock;                 /*    64     4 */

	/* XXX 4 bytes hole, try to pack */

	struct callback_head       rcu __attribute__((__aligned__(8))); /*    72    16 */
	struct mm_struct *         mm;                   /*    88     8 */
	struct cgroup_subsys_state * blkcg_css;          /*    96     8 */
	const struct cred  *       cur_creds;            /*   104     8 */
	const struct cred  *       saved_creds;          /*   112     8 */
	struct files_struct *      restore_files;        /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct nsproxy *           restore_nsproxy;      /*   128     8 */
	struct fs_struct *         restore_fs;           /*   136     8 */

	/* size: 144, cachelines: 3, members: 16 */
	/* sum members: 140, holes: 1, sum holes: 4 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 4 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct io_wqe {
	struct {
		raw_spinlock_t     lock;                 /*     0     4 */

		/* XXX 4 bytes hole, try to pack */

		struct io_wq_work_list work_list;        /*     8    16 */
		long unsigned int  hash_map;             /*    24     8 */
		unsigned int       flags;                /*    32     4 */
	} __attribute__((__aligned__(64))) __attribute__((__aligned__(64)));              /*     0    64 */

	/* XXX last struct has 28 bytes of padding */

	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        node;                 /*    64     4 */
	struct io_wqe_acct         acct[2];              /*    68    24 */

	/* XXX 4 bytes hole, try to pack */

	struct hlist_nulls_head    free_list;            /*    96     8 */
	struct list_head           all_list;             /*   104    16 */
	struct io_wq *             wq;                   /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct io_wq_work *        hash_tail[64];        /*   128   512 */

	/* size: 640, cachelines: 10, members: 7 */
	/* sum members: 636, holes: 1, sum holes: 4 */
	/* paddings: 1, sum paddings: 28 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(64)));
struct io_wqe_acct {
	unsigned int               nr_workers;           /*     0     4 */
	unsigned int               max_workers;          /*     4     4 */
	atomic_t                   nr_running;           /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct io_wq {
	struct io_wqe * *          wqes;                 /*     0     8 */
	long unsigned int          state;                /*     8     8 */
	free_work_fn *             free_work;            /*    16     8 */
	io_wq_work_fn *            do_work;              /*    24     8 */
	struct task_struct *       manager;              /*    32     8 */
	struct user_struct *       user;                 /*    40     8 */
	refcount_t                 refs;                 /*    48     4 */

	/* XXX 4 bytes hole, try to pack */

	struct completion          done;                 /*    56    32 */
	/* --- cacheline 1 boundary (64 bytes) was 24 bytes ago --- */
	struct hlist_node          cpuhp_node;           /*    88    16 */
	refcount_t                 use_refs;             /*   104     4 */

	/* size: 112, cachelines: 2, members: 10 */
	/* sum members: 104, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 48 bytes */
};
struct io_cb_cancel_data {
	work_cancel_fn *           fn;                   /*     0     8 */
	void *                     data;                 /*     8     8 */
	int                        nr_running;           /*    16     4 */
	int                        nr_pending;           /*    20     4 */
	bool                       cancel_all;           /*    24     1 */

	/* size: 32, cachelines: 1, members: 5 */
	/* padding: 7 */
	/* last cacheline: 32 bytes */
};
struct iomap_ops {
	int                        (*iomap_begin)(struct inode *, loff_t, loff_t, unsigned int, struct iomap *, struct iomap *); /*     0     8 */
	int                        (*iomap_end)(struct inode *, loff_t, loff_t, ssize_t, unsigned int, struct iomap *); /*     8     8 */
	u64                        kabi_reserved1;       /*    16     8 */
	u64                        kabi_reserved2;       /*    24     8 */
	u64                        kabi_reserved3;       /*    32     8 */
	u64                        kabi_reserved4;       /*    40     8 */

	/* size: 48, cachelines: 1, members: 6 */
	/* last cacheline: 48 bytes */
};
struct trace_event_raw_dax_pmd_fault_class {
	struct trace_entry         ent;                  /*     0     8 */
	long unsigned int          ino;                  /*     8     8 */
	long unsigned int          vm_start;             /*    16     8 */
	long unsigned int          vm_end;               /*    24     8 */
	long unsigned int          vm_flags;             /*    32     8 */
	long unsigned int          address;              /*    40     8 */
	long unsigned int          pgoff;                /*    48     8 */
	long unsigned int          max_pgoff;            /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	dev_t                      dev;                  /*    64     4 */
	unsigned int               flags;                /*    68     4 */
	int                        result;               /*    72     4 */
	char                       __data[];             /*    76     0 */

	/* size: 80, cachelines: 2, members: 12 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct trace_event_raw_dax_pmd_load_hole_class {
	struct trace_entry         ent;                  /*     0     8 */
	long unsigned int          ino;                  /*     8     8 */
	long unsigned int          vm_flags;             /*    16     8 */
	long unsigned int          address;              /*    24     8 */
	struct page *              zero_page;            /*    32     8 */
	void *                     radix_entry;          /*    40     8 */
	dev_t                      dev;                  /*    48     4 */
	char                       __data[];             /*    52     0 */

	/* size: 56, cachelines: 1, members: 8 */
	/* padding: 4 */
	/* last cacheline: 56 bytes */
};
struct trace_event_raw_dax_pmd_insert_mapping_class {
	struct trace_entry         ent;                  /*     0     8 */
	long unsigned int          ino;                  /*     8     8 */
	long unsigned int          vm_flags;             /*    16     8 */
	long unsigned int          address;              /*    24     8 */
	long int                   length;               /*    32     8 */
	u64                        pfn_val;              /*    40     8 */
	void *                     radix_entry;          /*    48     8 */
	dev_t                      dev;                  /*    56     4 */
	int                        write;                /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	char                       __data[];             /*    64     0 */

	/* size: 64, cachelines: 1, members: 10 */
};
struct trace_event_raw_dax_pte_fault_class {
	struct trace_entry         ent;                  /*     0     8 */
	long unsigned int          ino;                  /*     8     8 */
	long unsigned int          vm_flags;             /*    16     8 */
	long unsigned int          address;              /*    24     8 */
	long unsigned int          pgoff;                /*    32     8 */
	dev_t                      dev;                  /*    40     4 */
	unsigned int               flags;                /*    44     4 */
	int                        result;               /*    48     4 */
	char                       __data[];             /*    52     0 */

	/* size: 56, cachelines: 1, members: 9 */
	/* padding: 4 */
	/* last cacheline: 56 bytes */
};
struct trace_event_raw_dax_insert_mapping {
	struct trace_entry         ent;                  /*     0     8 */
	long unsigned int          ino;                  /*     8     8 */
	long unsigned int          vm_flags;             /*    16     8 */
	long unsigned int          address;              /*    24     8 */
	void *                     radix_entry;          /*    32     8 */
	dev_t                      dev;                  /*    40     4 */
	int                        write;                /*    44     4 */
	char                       __data[];             /*    48     0 */

	/* size: 48, cachelines: 1, members: 8 */
	/* last cacheline: 48 bytes */
};
struct trace_event_raw_dax_writeback_range_class {
	struct trace_entry         ent;                  /*     0     8 */
	long unsigned int          ino;                  /*     8     8 */
	long unsigned int          start_index;          /*    16     8 */
	long unsigned int          end_index;            /*    24     8 */
	dev_t                      dev;                  /*    32     4 */
	char                       __data[];             /*    36     0 */

	/* size: 40, cachelines: 1, members: 6 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct trace_event_raw_dax_writeback_one {
	struct trace_entry         ent;                  /*     0     8 */
	long unsigned int          ino;                  /*     8     8 */
	long unsigned int          pgoff;                /*    16     8 */
	long unsigned int          pglen;                /*    24     8 */
	dev_t                      dev;                  /*    32     4 */
	char                       __data[];             /*    36     0 */

	/* size: 40, cachelines: 1, members: 6 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct trace_event_data_offsets_dax_pmd_fault_class {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_dax_pmd_load_hole_class {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_dax_pmd_insert_mapping_class {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_dax_pte_fault_class {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_dax_insert_mapping {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_dax_writeback_range_class {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_dax_writeback_one {

	/* size: 0, cachelines: 0, members: 0 */
};
struct exceptional_entry_key {
	struct xarray *            xa;                   /*     0     8 */
	long unsigned int          entry_start;          /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct wait_exceptional_entry_queue {
	wait_queue_entry_t         wait;                 /*     0    40 */
	struct exceptional_entry_key key;                /*    40    16 */

	/* size: 56, cachelines: 1, members: 2 */
	/* last cacheline: 56 bytes */
};
struct flock64 {
	short int                  l_type;               /*     0     2 */
	short int                  l_whence;             /*     2     2 */

	/* XXX 4 bytes hole, try to pack */

	__kernel_loff_t            l_start;              /*     8     8 */
	__kernel_loff_t            l_len;                /*    16     8 */
	__kernel_pid_t             l_pid;                /*    24     4 */

	/* size: 32, cachelines: 1, members: 5 */
	/* sum members: 24, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct trace_event_raw_locks_get_lock_context {
	struct trace_entry         ent;                  /*     0     8 */
	long unsigned int          i_ino;                /*     8     8 */
	dev_t                      s_dev;                /*    16     4 */
	unsigned char              type;                 /*    20     1 */

	/* XXX 3 bytes hole, try to pack */

	struct file_lock_context * ctx;                  /*    24     8 */
	char                       __data[];             /*    32     0 */

	/* size: 32, cachelines: 1, members: 6 */
	/* sum members: 29, holes: 1, sum holes: 3 */
	/* last cacheline: 32 bytes */
};
struct trace_event_raw_filelock_lock {
	struct trace_entry         ent;                  /*     0     8 */
	struct file_lock *         fl;                   /*     8     8 */
	long unsigned int          i_ino;                /*    16     8 */
	dev_t                      s_dev;                /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	struct file_lock *         fl_blocker;           /*    32     8 */
	fl_owner_t                 fl_owner;             /*    40     8 */
	unsigned int               fl_pid;               /*    48     4 */
	unsigned int               fl_flags;             /*    52     4 */
	unsigned char              fl_type;              /*    56     1 */

	/* XXX 7 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	loff_t                     fl_start;             /*    64     8 */
	loff_t                     fl_end;               /*    72     8 */
	int                        ret;                  /*    80     4 */
	char                       __data[];             /*    84     0 */

	/* size: 88, cachelines: 2, members: 13 */
	/* sum members: 73, holes: 2, sum holes: 11 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_filelock_lease {
	struct trace_entry         ent;                  /*     0     8 */
	struct file_lock *         fl;                   /*     8     8 */
	long unsigned int          i_ino;                /*    16     8 */
	dev_t                      s_dev;                /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	struct file_lock *         fl_blocker;           /*    32     8 */
	fl_owner_t                 fl_owner;             /*    40     8 */
	unsigned int               fl_flags;             /*    48     4 */
	unsigned char              fl_type;              /*    52     1 */

	/* XXX 3 bytes hole, try to pack */

	long unsigned int          fl_break_time;        /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	long unsigned int          fl_downgrade_time;    /*    64     8 */
	char                       __data[];             /*    72     0 */

	/* size: 72, cachelines: 2, members: 11 */
	/* sum members: 65, holes: 2, sum holes: 7 */
	/* last cacheline: 8 bytes */
};
struct trace_event_raw_generic_add_lease {
	struct trace_entry         ent;                  /*     0     8 */
	long unsigned int          i_ino;                /*     8     8 */
	int                        wcount;               /*    16     4 */
	int                        rcount;               /*    20     4 */
	int                        icount;               /*    24     4 */
	dev_t                      s_dev;                /*    28     4 */
	fl_owner_t                 fl_owner;             /*    32     8 */
	unsigned int               fl_flags;             /*    40     4 */
	unsigned char              fl_type;              /*    44     1 */
	char                       __data[];             /*    45     0 */

	/* size: 48, cachelines: 1, members: 10 */
	/* padding: 3 */
	/* last cacheline: 48 bytes */
};
struct trace_event_raw_leases_conflict {
	struct trace_entry         ent;                  /*     0     8 */
	void *                     lease;                /*     8     8 */
	void *                     breaker;              /*    16     8 */
	unsigned int               l_fl_flags;           /*    24     4 */
	unsigned int               b_fl_flags;           /*    28     4 */
	unsigned char              l_fl_type;            /*    32     1 */
	unsigned char              b_fl_type;            /*    33     1 */
	bool                       conflict;             /*    34     1 */
	char                       __data[];             /*    35     0 */

	/* size: 40, cachelines: 1, members: 9 */
	/* padding: 5 */
	/* last cacheline: 40 bytes */
};
struct trace_event_data_offsets_locks_get_lock_context {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_filelock_lock {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_filelock_lease {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_generic_add_lease {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_leases_conflict {

	/* size: 0, cachelines: 0, members: 0 */
};
struct file_lock_list_struct {
	spinlock_t                 lock;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct hlist_head          hlist;                /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct locks_iterator {
	int                        li_cpu;               /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	loff_t                     li_pos;               /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct elf_prpsinfo {
	char                       pr_state;             /*     0     1 */
	char                       pr_sname;             /*     1     1 */
	char                       pr_zomb;              /*     2     1 */
	char                       pr_nice;              /*     3     1 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          pr_flag;              /*     8     8 */
	__kernel_uid_t             pr_uid;               /*    16     4 */
	__kernel_gid_t             pr_gid;               /*    20     4 */
	pid_t                      pr_pid;               /*    24     4 */
	pid_t                      pr_ppid;              /*    28     4 */
	pid_t                      pr_pgrp;              /*    32     4 */
	pid_t                      pr_sid;               /*    36     4 */
	char                       pr_fname[16];         /*    40    16 */
	char                       pr_psargs[80];        /*    56    80 */

	/* size: 136, cachelines: 3, members: 13 */
	/* sum members: 132, holes: 1, sum holes: 4 */
	/* last cacheline: 8 bytes */
};
struct core_vma_metadata {
	long unsigned int          start;                /*     0     8 */
	long unsigned int          end;                  /*     8     8 */
	long unsigned int          flags;                /*    16     8 */
	long unsigned int          dump_size;            /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct arch_elf_state {

	/* size: 0, cachelines: 0, members: 0 */
};
struct memelfnote {
	const char  *              name;                 /*     0     8 */
	int                        type;                 /*     8     4 */
	unsigned int               datasz;               /*    12     4 */
	void *                     data;                 /*    16     8 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct elf_thread_core_info {
	struct elf_thread_core_info * next;              /*     0     8 */
	struct task_struct *       task;                 /*     8     8 */
	struct elf_prstatus        prstatus;             /*    16   336 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 5 boundary (320 bytes) was 32 bytes ago --- */
	struct memelfnote          notes[];              /*   352     0 */

	/* size: 352, cachelines: 6, members: 4 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 32 bytes */
};
struct elf_note_info {
	struct elf_thread_core_info * thread;            /*     0     8 */
	struct memelfnote          psinfo;               /*     8    24 */
	struct memelfnote          signote;              /*    32    24 */
	struct memelfnote          auxv;                 /*    56    24 */
	/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
	struct memelfnote          files;                /*    80    24 */
	siginfo_t                  csigdata;             /*   104   128 */
	/* --- cacheline 3 boundary (192 bytes) was 40 bytes ago --- */
	size_t                     size;                 /*   232     8 */
	int                        thread_notes;         /*   240     4 */

	/* size: 248, cachelines: 4, members: 8 */
	/* padding: 4 */
	/* last cacheline: 56 bytes */
};
struct user_regs_struct {
	long unsigned int          r15;                  /*     0     8 */
	long unsigned int          r14;                  /*     8     8 */
	long unsigned int          r13;                  /*    16     8 */
	long unsigned int          r12;                  /*    24     8 */
	long unsigned int          bp;                   /*    32     8 */
	long unsigned int          bx;                   /*    40     8 */
	long unsigned int          r11;                  /*    48     8 */
	long unsigned int          r10;                  /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	long unsigned int          r9;                   /*    64     8 */
	long unsigned int          r8;                   /*    72     8 */
	long unsigned int          ax;                   /*    80     8 */
	long unsigned int          cx;                   /*    88     8 */
	long unsigned int          dx;                   /*    96     8 */
	long unsigned int          si;                   /*   104     8 */
	long unsigned int          di;                   /*   112     8 */
	long unsigned int          orig_ax;              /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	long unsigned int          ip;                   /*   128     8 */
	long unsigned int          cs;                   /*   136     8 */
	long unsigned int          flags;                /*   144     8 */
	long unsigned int          sp;                   /*   152     8 */
	long unsigned int          ss;                   /*   160     8 */
	long unsigned int          fs_base;              /*   168     8 */
	long unsigned int          gs_base;              /*   176     8 */
	long unsigned int          ds;                   /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	long unsigned int          es;                   /*   192     8 */
	long unsigned int          fs;                   /*   200     8 */
	long unsigned int          gs;                   /*   208     8 */

	/* size: 216, cachelines: 4, members: 27 */
	/* last cacheline: 24 bytes */
};
struct elf32_shdr {
	Elf32_Word                 sh_name;              /*     0     4 */
	Elf32_Word                 sh_type;              /*     4     4 */
	Elf32_Word                 sh_flags;             /*     8     4 */
	Elf32_Addr                 sh_addr;              /*    12     4 */
	Elf32_Off                  sh_offset;            /*    16     4 */
	Elf32_Word                 sh_size;              /*    20     4 */
	Elf32_Word                 sh_link;              /*    24     4 */
	Elf32_Word                 sh_info;              /*    28     4 */
	Elf32_Word                 sh_addralign;         /*    32     4 */
	Elf32_Word                 sh_entsize;           /*    36     4 */

	/* size: 40, cachelines: 1, members: 10 */
	/* last cacheline: 40 bytes */
};
struct compat_elf_siginfo {
	compat_int_t               si_signo;             /*     0     4 */
	compat_int_t               si_code;              /*     4     4 */
	compat_int_t               si_errno;             /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct compat_elf_prstatus {
	struct compat_elf_siginfo  pr_info;              /*     0    12 */
	short int                  pr_cursig;            /*    12     2 */

	/* XXX 2 bytes hole, try to pack */

	compat_ulong_t             pr_sigpend;           /*    16     4 */
	compat_ulong_t             pr_sighold;           /*    20     4 */
	compat_pid_t               pr_pid;               /*    24     4 */
	compat_pid_t               pr_ppid;              /*    28     4 */
	compat_pid_t               pr_pgrp;              /*    32     4 */
	compat_pid_t               pr_sid;               /*    36     4 */
	struct old_timeval32       pr_utime;             /*    40     8 */
	struct old_timeval32       pr_stime;             /*    48     8 */
	struct old_timeval32       pr_cutime;            /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct old_timeval32       pr_cstime;            /*    64     8 */
	compat_elf_gregset_t       pr_reg;               /*    72   216 */
	/* --- cacheline 4 boundary (256 bytes) was 32 bytes ago --- */
	compat_int_t               pr_fpvalid;           /*   288     4 */

	/* size: 296, cachelines: 5, members: 14 */
	/* sum members: 290, holes: 1, sum holes: 2 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct compat_elf_prpsinfo {
	char                       pr_state;             /*     0     1 */
	char                       pr_sname;             /*     1     1 */
	char                       pr_zomb;              /*     2     1 */
	char                       pr_nice;              /*     3     1 */
	compat_ulong_t             pr_flag;              /*     4     4 */
	__compat_uid_t             pr_uid;               /*     8     2 */
	__compat_gid_t             pr_gid;               /*    10     2 */
	compat_pid_t               pr_pid;               /*    12     4 */
	compat_pid_t               pr_ppid;              /*    16     4 */
	compat_pid_t               pr_pgrp;              /*    20     4 */
	compat_pid_t               pr_sid;               /*    24     4 */
	char                       pr_fname[16];         /*    28    16 */
	char                       pr_psargs[80];        /*    44    80 */

	/* size: 124, cachelines: 2, members: 13 */
	/* last cacheline: 60 bytes */
};
struct elf_thread_core_info {
	struct elf_thread_core_info * next;              /*     0     8 */
	struct task_struct *       task;                 /*     8     8 */
	struct compat_elf_prstatus prstatus;             /*    16   296 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 4 boundary (256 bytes) was 56 bytes ago --- */
	struct memelfnote          notes[];              /*   312     0 */

	/* size: 312, cachelines: 5, members: 4 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 56 bytes */
};
struct posix_acl_xattr_entry {
	__le16                     e_tag;                /*     0     2 */
	__le16                     e_perm;               /*     2     2 */
	__le32                     e_id;                 /*     4     4 */

	/* size: 8, cachelines: 1, members: 3 */
	/* last cacheline: 8 bytes */
};
struct posix_acl_xattr_header {
	__le32                     a_version;            /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct core_name {
	char *                     corename;             /*     0     8 */
	int                        used;                 /*     8     4 */
	int                        size;                 /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct files_cgroup {
	struct cgroup_subsys_state css __attribute__((__aligned__(8))); /*     0   296 */
	/* --- cacheline 4 boundary (256 bytes) was 40 bytes ago --- */
	struct page_counter        open_handles;         /*   296   112 */

	/* size: 408, cachelines: 7, members: 2 */
	/* forced alignments: 1 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct trace_event_raw_iomap_readpage_class {
	struct trace_entry         ent;                  /*     0     8 */
	dev_t                      dev;                  /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        ino;                  /*    16     8 */
	int                        nr_pages;             /*    24     4 */
	char                       __data[];             /*    28     0 */

	/* size: 32, cachelines: 1, members: 5 */
	/* sum members: 24, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct trace_event_raw_iomap_range_class {
	struct trace_entry         ent;                  /*     0     8 */
	dev_t                      dev;                  /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        ino;                  /*    16     8 */
	loff_t                     size;                 /*    24     8 */
	long unsigned int          offset;               /*    32     8 */
	unsigned int               length;               /*    40     4 */
	char                       __data[];             /*    44     0 */

	/* size: 48, cachelines: 1, members: 7 */
	/* sum members: 40, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 48 bytes */
};
struct trace_event_raw_iomap_class {
	struct trace_entry         ent;                  /*     0     8 */
	dev_t                      dev;                  /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        ino;                  /*    16     8 */
	u64                        addr;                 /*    24     8 */
	loff_t                     offset;               /*    32     8 */
	u64                        length;               /*    40     8 */
	u16                        type;                 /*    48     2 */
	u16                        flags;                /*    50     2 */
	dev_t                      bdev;                 /*    52     4 */
	char                       __data[];             /*    56     0 */

	/* size: 56, cachelines: 1, members: 10 */
	/* sum members: 52, holes: 1, sum holes: 4 */
	/* last cacheline: 56 bytes */
};
struct trace_event_raw_iomap_apply {
	struct trace_entry         ent;                  /*     0     8 */
	dev_t                      dev;                  /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        ino;                  /*    16     8 */
	loff_t                     pos;                  /*    24     8 */
	loff_t                     length;               /*    32     8 */
	unsigned int               flags;                /*    40     4 */

	/* XXX 4 bytes hole, try to pack */

	const void  *              ops;                  /*    48     8 */
	void *                     actor;                /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	long unsigned int          caller;               /*    64     8 */
	char                       __data[];             /*    72     0 */

	/* size: 72, cachelines: 2, members: 10 */
	/* sum members: 64, holes: 2, sum holes: 8 */
	/* last cacheline: 8 bytes */
};
struct trace_event_data_offsets_iomap_readpage_class {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_iomap_range_class {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_iomap_class {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_iomap_apply {

	/* size: 0, cachelines: 0, members: 0 */
};
struct iomap_ioend {
	struct list_head           io_list;              /*     0    16 */
	u16                        io_type;              /*    16     2 */
	u16                        io_flags;             /*    18     2 */

	/* XXX 4 bytes hole, try to pack */

	struct inode *             io_inode;             /*    24     8 */
	size_t                     io_size;              /*    32     8 */
	loff_t                     io_offset;            /*    40     8 */
	void *                     io_private;           /*    48     8 */
	struct bio *               io_bio;               /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct bio                 io_inline_bio;        /*    64   152 */

	/* size: 216, cachelines: 4, members: 9 */
	/* sum members: 212, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct iomap_writeback_ops {
	int                        (*map_blocks)(struct iomap_writepage_ctx *, struct inode *, loff_t); /*     0     8 */
	int                        (*prepare_ioend)(struct iomap_ioend *, int); /*     8     8 */
	void                       (*discard_page)(struct page *, loff_t); /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct iomap_writepage_ctx {
	struct iomap               iomap;                /*     0    72 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	struct iomap_ioend *       ioend;                /*    72     8 */
	const struct iomap_writeback_ops  * ops;         /*    80     8 */

	/* size: 88, cachelines: 2, members: 3 */
	/* last cacheline: 24 bytes */
};
struct iomap_page {
	atomic_t                   read_bytes_pending;   /*     0     4 */
	atomic_t                   write_bytes_pending;  /*     4     4 */
	spinlock_t                 uptodate_lock;        /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          uptodate[];           /*    16     0 */

	/* size: 16, cachelines: 1, members: 4 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct iomap_readpage_ctx {
	struct page *              cur_page;             /*     0     8 */
	bool                       cur_page_in_bio;      /*     8     1 */

	/* XXX 7 bytes hole, try to pack */

	struct bio *               bio;                  /*    16     8 */
	struct readahead_control * rac;                  /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* sum members: 25, holes: 1, sum holes: 7 */
	/* last cacheline: 32 bytes */
};
struct iomap_dio_ops {
	int                        (*end_io)(struct kiocb *, ssize_t, int, unsigned int); /*     0     8 */
	blk_qc_t                   (*submit_io)(struct inode *, struct iomap *, struct bio *, loff_t); /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct iomap_dio {
	struct kiocb *             iocb;                 /*     0     8 */
	const struct iomap_dio_ops  * dops;              /*     8     8 */
	loff_t                     i_size;               /*    16     8 */
	loff_t                     size;                 /*    24     8 */
	atomic_t                   ref;                  /*    32     4 */
	unsigned int               flags;                /*    36     4 */
	int                        error;                /*    40     4 */
	bool                       wait_for_completion;  /*    44     1 */

	/* XXX 3 bytes hole, try to pack */

	union {
		struct {
			struct iov_iter * iter;          /*    48     8 */
			struct task_struct * waiter;     /*    56     8 */
			/* --- cacheline 1 boundary (64 bytes) --- */
			struct request_queue * last_queue; /*    64     8 */
			blk_qc_t   cookie;               /*    72     4 */
		} submit;                                /*    48    32 */
		struct {
			struct work_struct work;         /*    48    64 */
		} aio;                                   /*    48    64 */
	};                                               /*    48    64 */

	/* size: 112, cachelines: 2, members: 9 */
	/* sum members: 109, holes: 1, sum holes: 3 */
	/* last cacheline: 48 bytes */
};
struct fiemap_ctx {
	struct fiemap_extent_info * fi;                  /*     0     8 */
	struct iomap               prev;                 /*     8    72 */

	/* size: 80, cachelines: 2, members: 2 */
	/* last cacheline: 16 bytes */
};
struct iomap_swapfile_info {
	struct iomap               iomap;                /*     0    72 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	struct swap_info_struct *  sis;                  /*    72     8 */
	uint64_t                   lowest_ppage;         /*    80     8 */
	uint64_t                   highest_ppage;        /*    88     8 */
	long unsigned int          nr_pages;             /*    96     8 */
	int                        nr_extents;           /*   104     4 */

	/* size: 112, cachelines: 2, members: 6 */
	/* padding: 4 */
	/* last cacheline: 48 bytes */
};
struct dqstats {
	long unsigned int          stat[8];              /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct percpu_counter      counter[8];           /*    64   320 */

	/* size: 384, cachelines: 6, members: 2 */
};
struct quota_module_name {
	int                        qm_fmt_id;            /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	char *                     qm_mod_name;          /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct dquot_warn {
	struct super_block *       w_sb;                 /*     0     8 */
	struct kqid                w_dq_id;              /*     8     8 */
	short int                  w_type;               /*    16     2 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 6 */
	/* last cacheline: 24 bytes */
};
struct qtree_fmt_operations {
	void                       (*mem2disk_dqblk)(void *, struct dquot *); /*     0     8 */
	void                       (*disk2mem_dqblk)(struct dquot *, void *); /*     8     8 */
	int                        (*is_id)(void *, struct dquot *); /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct qtree_mem_dqinfo {
	struct super_block *       dqi_sb;               /*     0     8 */
	int                        dqi_type;             /*     8     4 */
	unsigned int               dqi_blocks;           /*    12     4 */
	unsigned int               dqi_free_blk;         /*    16     4 */
	unsigned int               dqi_free_entry;       /*    20     4 */
	unsigned int               dqi_blocksize_bits;   /*    24     4 */
	unsigned int               dqi_entry_size;       /*    28     4 */
	unsigned int               dqi_usable_bs;        /*    32     4 */
	unsigned int               dqi_qtree_depth;      /*    36     4 */
	const struct qtree_fmt_operations  * dqi_ops;    /*    40     8 */

	/* size: 48, cachelines: 1, members: 10 */
	/* last cacheline: 48 bytes */
};
struct v2_disk_dqheader {
	__le32                     dqh_magic;            /*     0     4 */
	__le32                     dqh_version;          /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct v2r0_disk_dqblk {
	__le32                     dqb_id;               /*     0     4 */
	__le32                     dqb_ihardlimit;       /*     4     4 */
	__le32                     dqb_isoftlimit;       /*     8     4 */
	__le32                     dqb_curinodes;        /*    12     4 */
	__le32                     dqb_bhardlimit;       /*    16     4 */
	__le32                     dqb_bsoftlimit;       /*    20     4 */
	__le64                     dqb_curspace;         /*    24     8 */
	__le64                     dqb_btime;            /*    32     8 */
	__le64                     dqb_itime;            /*    40     8 */

	/* size: 48, cachelines: 1, members: 9 */
	/* last cacheline: 48 bytes */
};
struct v2r1_disk_dqblk {
	__le32                     dqb_id;               /*     0     4 */
	__le32                     dqb_pad;              /*     4     4 */
	__le64                     dqb_ihardlimit;       /*     8     8 */
	__le64                     dqb_isoftlimit;       /*    16     8 */
	__le64                     dqb_curinodes;        /*    24     8 */
	__le64                     dqb_bhardlimit;       /*    32     8 */
	__le64                     dqb_bsoftlimit;       /*    40     8 */
	__le64                     dqb_curspace;         /*    48     8 */
	__le64                     dqb_btime;            /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	__le64                     dqb_itime;            /*    64     8 */

	/* size: 72, cachelines: 2, members: 10 */
	/* last cacheline: 8 bytes */
};
struct v2_disk_dqinfo {
	__le32                     dqi_bgrace;           /*     0     4 */
	__le32                     dqi_igrace;           /*     4     4 */
	__le32                     dqi_flags;            /*     8     4 */
	__le32                     dqi_blocks;           /*    12     4 */
	__le32                     dqi_free_blk;         /*    16     4 */
	__le32                     dqi_free_entry;       /*    20     4 */

	/* size: 24, cachelines: 1, members: 6 */
	/* last cacheline: 24 bytes */
};
struct qt_disk_dqdbheader {
	__le32                     dqdh_next_free;       /*     0     4 */
	__le32                     dqdh_prev_free;       /*     4     4 */
	__le16                     dqdh_entries;         /*     8     2 */
	__le16                     dqdh_pad1;            /*    10     2 */
	__le32                     dqdh_pad2;            /*    12     4 */

	/* size: 16, cachelines: 1, members: 5 */
	/* last cacheline: 16 bytes */
};
struct fs_disk_quota {
	__s8                       d_version;            /*     0     1 */
	__s8                       d_flags;              /*     1     1 */
	__u16                      d_fieldmask;          /*     2     2 */
	__u32                      d_id;                 /*     4     4 */
	__u64                      d_blk_hardlimit;      /*     8     8 */
	__u64                      d_blk_softlimit;      /*    16     8 */
	__u64                      d_ino_hardlimit;      /*    24     8 */
	__u64                      d_ino_softlimit;      /*    32     8 */
	__u64                      d_bcount;             /*    40     8 */
	__u64                      d_icount;             /*    48     8 */
	__s32                      d_itimer;             /*    56     4 */
	__s32                      d_btimer;             /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	__u16                      d_iwarns;             /*    64     2 */
	__u16                      d_bwarns;             /*    66     2 */
	__s8                       d_itimer_hi;          /*    68     1 */
	__s8                       d_btimer_hi;          /*    69     1 */
	__s8                       d_rtbtimer_hi;        /*    70     1 */
	__s8                       d_padding2;           /*    71     1 */
	__u64                      d_rtb_hardlimit;      /*    72     8 */
	__u64                      d_rtb_softlimit;      /*    80     8 */
	__u64                      d_rtbcount;           /*    88     8 */
	__s32                      d_rtbtimer;           /*    96     4 */
	__u16                      d_rtbwarns;           /*   100     2 */
	__s16                      d_padding3;           /*   102     2 */
	char                       d_padding4[8];        /*   104     8 */

	/* size: 112, cachelines: 2, members: 25 */
	/* last cacheline: 48 bytes */
};
struct fs_qfilestat {
	__u64                      qfs_ino;              /*     0     8 */
	__u64                      qfs_nblks;            /*     8     8 */
	__u32                      qfs_nextents;         /*    16     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct fs_quota_stat {
	__s8                       qs_version;           /*     0     1 */

	/* XXX 1 byte hole, try to pack */

	__u16                      qs_flags;             /*     2     2 */
	__s8                       qs_pad;               /*     4     1 */

	/* XXX 3 bytes hole, try to pack */

	fs_qfilestat_t             qs_uquota;            /*     8    24 */
	fs_qfilestat_t             qs_gquota;            /*    32    24 */
	__u32                      qs_incoredqs;         /*    56     4 */
	__s32                      qs_btimelimit;        /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	__s32                      qs_itimelimit;        /*    64     4 */
	__s32                      qs_rtbtimelimit;      /*    68     4 */
	__u16                      qs_bwarnlimit;        /*    72     2 */
	__u16                      qs_iwarnlimit;        /*    74     2 */

	/* size: 80, cachelines: 2, members: 11 */
	/* sum members: 72, holes: 2, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct fs_qfilestatv {
	__u64                      qfs_ino;              /*     0     8 */
	__u64                      qfs_nblks;            /*     8     8 */
	__u32                      qfs_nextents;         /*    16     4 */
	__u32                      qfs_pad;              /*    20     4 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct fs_quota_statv {
	__s8                       qs_version;           /*     0     1 */
	__u8                       qs_pad1;              /*     1     1 */
	__u16                      qs_flags;             /*     2     2 */
	__u32                      qs_incoredqs;         /*     4     4 */
	struct fs_qfilestatv       qs_uquota;            /*     8    24 */
	struct fs_qfilestatv       qs_gquota;            /*    32    24 */
	struct fs_qfilestatv       qs_pquota;            /*    56    24 */
	/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
	__s32                      qs_btimelimit;        /*    80     4 */
	__s32                      qs_itimelimit;        /*    84     4 */
	__s32                      qs_rtbtimelimit;      /*    88     4 */
	__u16                      qs_bwarnlimit;        /*    92     2 */
	__u16                      qs_iwarnlimit;        /*    94     2 */
	__u64                      qs_pad2[8];           /*    96    64 */

	/* size: 160, cachelines: 3, members: 13 */
	/* last cacheline: 32 bytes */
};
struct if_dqblk {
	__u64                      dqb_bhardlimit;       /*     0     8 */
	__u64                      dqb_bsoftlimit;       /*     8     8 */
	__u64                      dqb_curspace;         /*    16     8 */
	__u64                      dqb_ihardlimit;       /*    24     8 */
	__u64                      dqb_isoftlimit;       /*    32     8 */
	__u64                      dqb_curinodes;        /*    40     8 */
	__u64                      dqb_btime;            /*    48     8 */
	__u64                      dqb_itime;            /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	__u32                      dqb_valid;            /*    64     4 */

	/* size: 72, cachelines: 2, members: 9 */
	/* padding: 4 */
	/* last cacheline: 8 bytes */
};
struct if_nextdqblk {
	__u64                      dqb_bhardlimit;       /*     0     8 */
	__u64                      dqb_bsoftlimit;       /*     8     8 */
	__u64                      dqb_curspace;         /*    16     8 */
	__u64                      dqb_ihardlimit;       /*    24     8 */
	__u64                      dqb_isoftlimit;       /*    32     8 */
	__u64                      dqb_curinodes;        /*    40     8 */
	__u64                      dqb_btime;            /*    48     8 */
	__u64                      dqb_itime;            /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	__u32                      dqb_valid;            /*    64     4 */
	__u32                      dqb_id;               /*    68     4 */

	/* size: 72, cachelines: 2, members: 10 */
	/* last cacheline: 8 bytes */
};
struct if_dqinfo {
	__u64                      dqi_bgrace;           /*     0     8 */
	__u64                      dqi_igrace;           /*     8     8 */
	__u32                      dqi_flags;            /*    16     4 */
	__u32                      dqi_valid;            /*    20     4 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct compat_if_dqblk {
	compat_u64                 dqb_bhardlimit __attribute__((__aligned__(4))); /*     0     8 */
	compat_u64                 dqb_bsoftlimit __attribute__((__aligned__(4))); /*     8     8 */
	compat_u64                 dqb_curspace __attribute__((__aligned__(4))); /*    16     8 */
	compat_u64                 dqb_ihardlimit __attribute__((__aligned__(4))); /*    24     8 */
	compat_u64                 dqb_isoftlimit __attribute__((__aligned__(4))); /*    32     8 */
	compat_u64                 dqb_curinodes __attribute__((__aligned__(4))); /*    40     8 */
	compat_u64                 dqb_btime __attribute__((__aligned__(4))); /*    48     8 */
	compat_u64                 dqb_itime __attribute__((__aligned__(4))); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	compat_uint_t              dqb_valid;            /*    64     4 */

	/* size: 68, cachelines: 2, members: 9 */
	/* forced alignments: 8 */
	/* last cacheline: 4 bytes */
} __attribute__((__packed__));
struct compat_fs_qfilestat {
	compat_u64                 dqb_bhardlimit __attribute__((__aligned__(4))); /*     0     8 */
	compat_u64                 qfs_nblks __attribute__((__aligned__(4))); /*     8     8 */
	compat_uint_t              qfs_nextents;         /*    16     4 */

	/* size: 20, cachelines: 1, members: 3 */
	/* forced alignments: 2 */
	/* last cacheline: 20 bytes */
} __attribute__((__packed__));
struct compat_fs_quota_stat {
	__s8                       qs_version;           /*     0     1 */

	/* XXX 1 byte hole, try to pack */

	__u16                      qs_flags;             /*     2     2 */
	__s8                       qs_pad;               /*     4     1 */

	/* XXX 3 bytes hole, try to pack */

	struct compat_fs_qfilestat qs_uquota __attribute__((__aligned__(4))); /*     8    20 */
	struct compat_fs_qfilestat qs_gquota __attribute__((__aligned__(4))); /*    28    20 */
	compat_uint_t              qs_incoredqs;         /*    48     4 */
	compat_int_t               qs_btimelimit;        /*    52     4 */
	compat_int_t               qs_itimelimit;        /*    56     4 */
	compat_int_t               qs_rtbtimelimit;      /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	__u16                      qs_bwarnlimit;        /*    64     2 */
	__u16                      qs_iwarnlimit;        /*    66     2 */

	/* size: 68, cachelines: 2, members: 11 */
	/* sum members: 64, holes: 2, sum holes: 4 */
	/* forced alignments: 2, forced holes: 1, sum forced holes: 3 */
	/* last cacheline: 4 bytes */
} __attribute__((__aligned__(4)));
struct proc_maps_private {
	struct inode *             inode;                /*     0     8 */
	struct task_struct *       task;                 /*     8     8 */
	struct mm_struct *         mm;                   /*    16     8 */
	struct vm_area_struct *    tail_vma;             /*    24     8 */
	struct mempolicy *         task_mempolicy;       /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct mem_size_stats {
	long unsigned int          resident;             /*     0     8 */
	long unsigned int          shared_clean;         /*     8     8 */
	long unsigned int          shared_dirty;         /*    16     8 */
	long unsigned int          private_clean;        /*    24     8 */
	long unsigned int          private_dirty;        /*    32     8 */
	long unsigned int          referenced;           /*    40     8 */
	long unsigned int          anonymous;            /*    48     8 */
	long unsigned int          lazyfree;             /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	long unsigned int          anonymous_thp;        /*    64     8 */
	long unsigned int          shmem_thp;            /*    72     8 */
	long unsigned int          file_thp;             /*    80     8 */
	long unsigned int          swap;                 /*    88     8 */
	long unsigned int          shared_hugetlb;       /*    96     8 */
	long unsigned int          private_hugetlb;      /*   104     8 */
	u64                        pss;                  /*   112     8 */
	u64                        pss_anon;             /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	u64                        pss_file;             /*   128     8 */
	u64                        pss_shmem;            /*   136     8 */
	u64                        pss_locked;           /*   144     8 */
	u64                        swap_pss;             /*   152     8 */
	bool                       check_shmem_swap;     /*   160     1 */

	/* size: 168, cachelines: 3, members: 21 */
	/* padding: 7 */
	/* last cacheline: 40 bytes */
};
struct clear_refs_private {
	enum clear_refs_types      type;                 /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct pagemapread {
	int                        pos;                  /*     0     4 */
	int                        len;                  /*     4     4 */
	pagemap_entry_t *          buffer;               /*     8     8 */
	bool                       show_pfn;             /*    16     1 */

	/* size: 24, cachelines: 1, members: 4 */
	/* padding: 7 */
	/* last cacheline: 24 bytes */
};
struct numa_maps {
	long unsigned int          pages;                /*     0     8 */
	long unsigned int          anon;                 /*     8     8 */
	long unsigned int          active;               /*    16     8 */
	long unsigned int          writeback;            /*    24     8 */
	long unsigned int          mapcount_max;         /*    32     8 */
	long unsigned int          dirty;                /*    40     8 */
	long unsigned int          swapcache;            /*    48     8 */
	long unsigned int          node[1024];           /*    56  8192 */

	/* size: 8248, cachelines: 129, members: 8 */
	/* last cacheline: 56 bytes */
};
struct numa_maps_private {
	struct proc_maps_private   proc_maps;            /*     0    40 */
	struct numa_maps           md;                   /*    40  8248 */

	/* size: 8288, cachelines: 130, members: 2 */
	/* last cacheline: 32 bytes */
};
struct pde_opener {
	struct list_head           lh;                   /*     0    16 */
	struct file *              file;                 /*    16     8 */
	bool                       closing;              /*    24     1 */

	/* XXX 7 bytes hole, try to pack */

	struct completion *        c;                    /*    32     8 */

	/* size: 40, cachelines: 1, members: 4 */
	/* sum members: 33, holes: 1, sum holes: 7 */
	/* last cacheline: 40 bytes */
};
struct proc_fs_context {
	struct pid_namespace *     pid_ns;               /*     0     8 */
	unsigned int               mask;                 /*     8     4 */
	enum proc_hidepid          hidepid;              /*    12     4 */
	int                        gid;                  /*    16     4 */
	enum proc_pidonly          pidonly;              /*    20     4 */

	/* size: 24, cachelines: 1, members: 5 */
	/* last cacheline: 24 bytes */
};
struct __genradix {
	struct genradix_root *     root;                 /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct syscall_info {
	__u64                      sp;                   /*     0     8 */
	struct seccomp_data        data;                 /*     8    64 */

	/* size: 72, cachelines: 2, members: 2 */
	/* last cacheline: 8 bytes */
};
struct pid_entry {
	const char  *              name;                 /*     0     8 */
	unsigned int               len;                  /*     8     4 */
	umode_t                    mode;                 /*    12     2 */

	/* XXX 2 bytes hole, try to pack */

	const struct inode_operations  * iop;            /*    16     8 */
	const struct file_operations  * fop;             /*    24     8 */
	union proc_op              op;                   /*    32     8 */

	/* size: 40, cachelines: 1, members: 6 */
	/* sum members: 38, holes: 1, sum holes: 2 */
	/* last cacheline: 40 bytes */
};
struct limit_names {
	const char  *              name;                 /*     0     8 */
	const char  *              unit;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct map_files_info {
	long unsigned int          start;                /*     0     8 */
	long unsigned int          end;                  /*     8     8 */
	fmode_t                    mode;                 /*    16     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct timers_private {
	struct pid *               pid;                  /*     0     8 */
	struct task_struct *       task;                 /*     8     8 */
	struct sighand_struct *    sighand;              /*    16     8 */
	struct pid_namespace *     ns;                   /*    24     8 */
	long unsigned int          flags;                /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct tgid_iter {
	unsigned int               tgid;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct task_struct *       task;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct fd_data {
	fmode_t                    mode;                 /*     0     4 */
	unsigned int               fd;                   /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct sysctl_alias {
	const char  *              kernel_param;         /*     0     8 */
	const char  *              sysctl_param;         /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct seq_net_private {
	struct net *               net;                  /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct vmcore {
	struct list_head           list;                 /*     0    16 */
	long long unsigned int     paddr;                /*    16     8 */
	long long unsigned int     size;                 /*    24     8 */
	loff_t                     offset;               /*    32     8 */

	/* size: 40, cachelines: 1, members: 4 */
	/* last cacheline: 40 bytes */
};
struct vmcoredd_node {
	struct list_head           list;                 /*     0    16 */
	void *                     buf;                  /*    16     8 */
	unsigned int               size;                 /*    24     4 */

	/* size: 32, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct vmcoredd_header {
	__u32                      n_namesz;             /*     0     4 */
	__u32                      n_descsz;             /*     4     4 */
	__u32                      n_type;               /*     8     4 */
	__u8                       name[8];              /*    12     8 */
	__u8                       dump_name[44];        /*    20    44 */

	/* size: 64, cachelines: 1, members: 5 */
};
struct vmcoredd_data {
	char                       dump_name[44];        /*     0    44 */
	unsigned int               size;                 /*    44     4 */
	int                        (*vmcoredd_callback)(struct vmcoredd_data *, void *); /*    48     8 */

	/* size: 56, cachelines: 1, members: 3 */
	/* last cacheline: 56 bytes */
};
struct kernfs_iattrs {
	kuid_t                     ia_uid;               /*     0     4 */
	kgid_t                     ia_gid;               /*     4     4 */
	struct timespec64          ia_atime;             /*     8    16 */
	struct timespec64          ia_mtime;             /*    24    16 */
	struct timespec64          ia_ctime;             /*    40    16 */
	struct simple_xattrs       xattrs;               /*    56    24 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
	atomic_t                   nr_user_xattrs;       /*    80     4 */
	atomic_t                   user_xattr_size;      /*    84     4 */

	/* size: 88, cachelines: 2, members: 8 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 24 bytes */
};
struct kernfs_super_info {
	struct super_block *       sb;                   /*     0     8 */
	struct kernfs_root *       root;                 /*     8     8 */
	const void  *              ns;                   /*    16     8 */
	struct list_head           node;                 /*    24    16 */

	/* size: 40, cachelines: 1, members: 4 */
	/* last cacheline: 40 bytes */
};
struct kernfs_open_node {
	atomic_t                   refcnt;               /*     0     4 */
	atomic_t                   event;                /*     4     4 */
	wait_queue_head_t          poll;                 /*     8    24 */
	struct list_head           files;                /*    32    16 */

	/* size: 48, cachelines: 1, members: 4 */
	/* last cacheline: 48 bytes */
};
struct config_item {
	char *                     ci_name;              /*     0     8 */
	char                       ci_namebuf[20];       /*     8    20 */
	struct kref                ci_kref;              /*    28     4 */
	struct list_head           ci_entry;             /*    32    16 */
	struct config_item *       ci_parent;            /*    48     8 */
	struct config_group *      ci_group;             /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	const struct config_item_type  * ci_type;        /*    64     8 */
	struct dentry *            ci_dentry;            /*    72     8 */

	/* size: 80, cachelines: 2, members: 8 */
	/* last cacheline: 16 bytes */
};
struct config_group {
	struct config_item         cg_item;              /*     0    80 */
	/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
	struct list_head           cg_children;          /*    80    16 */
	struct configfs_subsystem * cg_subsys;           /*    96     8 */
	struct list_head           default_groups;       /*   104    16 */
	struct list_head           group_entry;          /*   120    16 */

	/* size: 136, cachelines: 3, members: 5 */
	/* last cacheline: 8 bytes */
};
struct config_item_type {
	struct module *            ct_owner;             /*     0     8 */
	struct configfs_item_operations * ct_item_ops;   /*     8     8 */
	struct configfs_group_operations * ct_group_ops; /*    16     8 */
	struct configfs_attribute * * ct_attrs;          /*    24     8 */
	struct configfs_bin_attribute * * ct_bin_attrs;  /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct configfs_item_operations {
	void                       (*release)(struct config_item *); /*     0     8 */
	int                        (*allow_link)(struct config_item *, struct config_item *); /*     8     8 */
	void                       (*drop_link)(struct config_item *, struct config_item *); /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct configfs_group_operations {
	struct config_item *       (*make_item)(struct config_group *, const char  *); /*     0     8 */
	struct config_group *      (*make_group)(struct config_group *, const char  *); /*     8     8 */
	int                        (*commit_item)(struct config_item *); /*    16     8 */
	void                       (*disconnect_notify)(struct config_group *, struct config_item *); /*    24     8 */
	void                       (*drop_item)(struct config_group *, struct config_item *); /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct configfs_attribute {
	const char  *              ca_name;              /*     0     8 */
	struct module *            ca_owner;             /*     8     8 */
	umode_t                    ca_mode;              /*    16     2 */

	/* XXX 6 bytes hole, try to pack */

	ssize_t                    (*show)(struct config_item *, char *); /*    24     8 */
	ssize_t                    (*store)(struct config_item *, const char  *, size_t); /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* sum members: 34, holes: 1, sum holes: 6 */
	/* last cacheline: 40 bytes */
};
struct configfs_bin_attribute {
	struct configfs_attribute  cb_attr;              /*     0    40 */
	void *                     cb_private;           /*    40     8 */
	size_t                     cb_max_size;          /*    48     8 */
	ssize_t                    (*read)(struct config_item *, void *, size_t); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	ssize_t                    (*write)(struct config_item *, const void  *, size_t); /*    64     8 */

	/* size: 72, cachelines: 2, members: 5 */
	/* last cacheline: 8 bytes */
};
struct configfs_subsystem {
	struct config_group        su_group;             /*     0   136 */
	/* --- cacheline 2 boundary (128 bytes) was 8 bytes ago --- */
	struct mutex               su_mutex;             /*   136    32 */

	/* size: 168, cachelines: 3, members: 2 */
	/* last cacheline: 40 bytes */
};
struct configfs_fragment {
	atomic_t                   frag_count;           /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct rw_semaphore        frag_sem;             /*     8    40 */
	bool                       frag_dead;            /*    48     1 */

	/* size: 56, cachelines: 1, members: 3 */
	/* sum members: 45, holes: 1, sum holes: 4 */
	/* padding: 7 */
	/* last cacheline: 56 bytes */
};
struct configfs_dirent {
	atomic_t                   s_count;              /*     0     4 */
	int                        s_dependent_count;    /*     4     4 */
	struct list_head           s_sibling;            /*     8    16 */
	struct list_head           s_children;           /*    24    16 */
	int                        s_links;              /*    40     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     s_element;            /*    48     8 */
	int                        s_type;               /*    56     4 */
	umode_t                    s_mode;               /*    60     2 */

	/* XXX 2 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	struct dentry *            s_dentry;             /*    64     8 */
	struct iattr *             s_iattr;              /*    72     8 */
	struct configfs_fragment * s_frag;               /*    80     8 */

	/* size: 88, cachelines: 2, members: 11 */
	/* sum members: 82, holes: 2, sum holes: 6 */
	/* last cacheline: 24 bytes */
};
struct configfs_buffer {
	size_t                     count;                /*     0     8 */
	loff_t                     pos;                  /*     8     8 */
	char *                     page;                 /*    16     8 */
	struct configfs_item_operations * ops;           /*    24     8 */
	struct mutex               mutex;                /*    32    32 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        needs_read_fill;      /*    64     4 */
	bool                       read_in_progress;     /*    68     1 */
	bool                       write_in_progress;    /*    69     1 */

	/* XXX 2 bytes hole, try to pack */

	char *                     bin_buffer;           /*    72     8 */
	int                        bin_buffer_size;      /*    80     4 */
	int                        cb_max_size;          /*    84     4 */
	struct config_item *       item;                 /*    88     8 */
	struct module *            owner;                /*    96     8 */
	union {
		struct configfs_attribute * attr;        /*   104     8 */
		struct configfs_bin_attribute * bin_attr; /*   104     8 */
	};                                               /*   104     8 */

	/* size: 112, cachelines: 2, members: 14 */
	/* sum members: 110, holes: 1, sum holes: 2 */
	/* last cacheline: 48 bytes */
};
struct pts_mount_opts {
	int                        setuid;               /*     0     4 */
	int                        setgid;               /*     4     4 */
	kuid_t                     uid;                  /*     8     4 */
	kgid_t                     gid;                  /*    12     4 */
	umode_t                    mode;                 /*    16     2 */
	umode_t                    ptmxmode;             /*    18     2 */
	int                        reserve;              /*    20     4 */
	int                        max;                  /*    24     4 */

	/* size: 28, cachelines: 1, members: 8 */
	/* last cacheline: 28 bytes */
};
struct pts_fs_info {
	struct ida                 allocated_ptys;       /*     0    16 */
	struct pts_mount_opts      mount_opts;           /*    16    28 */

	/* XXX 4 bytes hole, try to pack */

	struct super_block *       sb;                   /*    48     8 */
	struct dentry *            ptmx_dentry;          /*    56     8 */

	/* size: 64, cachelines: 1, members: 4 */
	/* sum members: 60, holes: 1, sum holes: 4 */
};
struct dcookie_struct {
	struct path                path;                 /*     0    16 */
	struct list_head           hash_list;            /*    16    16 */

	/* size: 32, cachelines: 1, members: 2 */
	/* last cacheline: 32 bytes */
};
struct dcookie_user {
	struct list_head           next;                 /*     0    16 */

	/* size: 16, cachelines: 1, members: 1 */
	/* last cacheline: 16 bytes */
};
struct ramfs_mount_opts {
	umode_t                    mode;                 /*     0     2 */

	/* size: 2, cachelines: 1, members: 1 */
	/* last cacheline: 2 bytes */
};
struct ramfs_fs_info {
	struct ramfs_mount_opts    mount_opts;           /*     0     2 */

	/* size: 2, cachelines: 1, members: 1 */
	/* last cacheline: 2 bytes */
};
struct hugetlbfs_fs_context {
	struct hstate *            hstate;               /*     0     8 */
	long long unsigned int     max_size_opt;         /*     8     8 */
	long long unsigned int     min_size_opt;         /*    16     8 */
	long int                   max_hpages;           /*    24     8 */
	long int                   nr_inodes;            /*    32     8 */
	long int                   min_hpages;           /*    40     8 */
	enum hugetlbfs_size_type   max_val_type;         /*    48     4 */
	enum hugetlbfs_size_type   min_val_type;         /*    52     4 */
	kuid_t                     uid;                  /*    56     4 */
	kgid_t                     gid;                  /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	umode_t                    mode;                 /*    64     2 */

	/* size: 72, cachelines: 2, members: 11 */
	/* padding: 6 */
	/* last cacheline: 8 bytes */
};
struct getdents_callback {
	struct dir_context         ctx;                  /*     0    16 */
	char *                     name;                 /*    16     8 */
	u64                        ino;                  /*    24     8 */
	int                        found;                /*    32     4 */
	int                        sequence;             /*    36     4 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct nls_table {
	const char  *              charset;              /*     0     8 */
	const char  *              alias;                /*     8     8 */
	int                        (*uni2char)(wchar_t, unsigned char *, int); /*    16     8 */
	int                        (*char2uni)(const unsigned char  *, int, wchar_t *); /*    24     8 */
	const unsigned char  *     charset2lower;        /*    32     8 */
	const unsigned char  *     charset2upper;        /*    40     8 */
	struct module *            owner;                /*    48     8 */
	struct nls_table *         next;                 /*    56     8 */

	/* size: 64, cachelines: 1, members: 8 */
};
struct utf8_table {
	int                        cmask;                /*     0     4 */
	int                        cval;                 /*     4     4 */
	int                        shift;                /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	long int                   lmask;                /*    16     8 */
	long int                   lval;                 /*    24     8 */

	/* size: 32, cachelines: 1, members: 5 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct autofs_info {
	struct dentry *            dentry;               /*     0     8 */
	struct inode *             inode;                /*     8     8 */
	int                        flags;                /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	struct completion          expire_complete;      /*    24    32 */
	struct list_head           active;               /*    56    16 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	struct list_head           expiring;             /*    72    16 */
	struct autofs_sb_info *    sbi;                  /*    88     8 */
	long unsigned int          last_used;            /*    96     8 */
	int                        count;                /*   104     4 */
	kuid_t                     uid;                  /*   108     4 */
	kgid_t                     gid;                  /*   112     4 */

	/* XXX 4 bytes hole, try to pack */

	struct callback_head       rcu __attribute__((__aligned__(8))); /*   120    16 */

	/* size: 136, cachelines: 3, members: 12 */
	/* sum members: 128, holes: 2, sum holes: 8 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 4 */
	/* last cacheline: 8 bytes */
} __attribute__((__aligned__(8)));
struct autofs_sb_info {
	u32                        magic;                /*     0     4 */
	int                        pipefd;               /*     4     4 */
	struct file *              pipe;                 /*     8     8 */
	struct pid *               oz_pgrp;              /*    16     8 */
	int                        version;              /*    24     4 */
	int                        sub_version;          /*    28     4 */
	int                        min_proto;            /*    32     4 */
	int                        max_proto;            /*    36     4 */
	unsigned int               flags;                /*    40     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          exp_timeout;          /*    48     8 */
	unsigned int               type;                 /*    56     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	struct super_block *       sb;                   /*    64     8 */
	struct mutex               wq_mutex;             /*    72    32 */
	struct mutex               pipe_mutex;           /*   104    32 */
	/* --- cacheline 2 boundary (128 bytes) was 8 bytes ago --- */
	spinlock_t                 fs_lock;              /*   136     4 */

	/* XXX 4 bytes hole, try to pack */

	struct autofs_wait_queue * queues;               /*   144     8 */
	spinlock_t                 lookup_lock;          /*   152     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           active_list;          /*   160    16 */
	struct list_head           expiring_list;        /*   176    16 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*   192    16 */

	/* size: 208, cachelines: 4, members: 20 */
	/* sum members: 192, holes: 4, sum holes: 16 */
	/* forced alignments: 1 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct autofs_wait_queue {
	wait_queue_head_t          queue;                /*     0    24 */
	struct autofs_wait_queue * next;                 /*    24     8 */
	autofs_wqt_t               wait_queue_token;     /*    32     4 */

	/* XXX 4 bytes hole, try to pack */

	struct qstr                name;                 /*    40    16 */
	u32                        dev;                  /*    56     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	u64                        ino;                  /*    64     8 */
	kuid_t                     uid;                  /*    72     4 */
	kgid_t                     gid;                  /*    76     4 */
	pid_t                      pid;                  /*    80     4 */
	pid_t                      tgid;                 /*    84     4 */
	int                        status;               /*    88     4 */
	unsigned int               wait_ctr;             /*    92     4 */

	/* size: 96, cachelines: 2, members: 12 */
	/* sum members: 88, holes: 2, sum holes: 8 */
	/* last cacheline: 32 bytes */
};
struct autofs_packet_hdr {
	int                        proto_version;        /*     0     4 */
	int                        type;                 /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct autofs_packet_missing {
	struct autofs_packet_hdr   hdr;                  /*     0     8 */
	autofs_wqt_t               wait_queue_token;     /*     8     4 */
	int                        len;                  /*    12     4 */
	char                       name[256];            /*    16   256 */

	/* size: 272, cachelines: 5, members: 4 */
	/* last cacheline: 16 bytes */
};
struct autofs_packet_expire {
	struct autofs_packet_hdr   hdr;                  /*     0     8 */
	int                        len;                  /*     8     4 */
	char                       name[256];            /*    12   256 */

	/* size: 268, cachelines: 5, members: 3 */
	/* last cacheline: 12 bytes */
};
struct autofs_packet_expire_multi {
	struct autofs_packet_hdr   hdr;                  /*     0     8 */
	autofs_wqt_t               wait_queue_token;     /*     8     4 */
	int                        len;                  /*    12     4 */
	char                       name[256];            /*    16   256 */

	/* size: 272, cachelines: 5, members: 4 */
	/* last cacheline: 16 bytes */
};
union autofs_packet_union {
	struct autofs_packet_hdr   hdr;                /*     0     8 */
	struct autofs_packet_missing missing;          /*     0   272 */
	struct autofs_packet_expire expire;            /*     0   268 */
	struct autofs_packet_expire_multi expire_multi; /*     0   272 */
};
struct autofs_v5_packet {
	struct autofs_packet_hdr   hdr;                  /*     0     8 */
	autofs_wqt_t               wait_queue_token;     /*     8     4 */
	__u32                      dev;                  /*    12     4 */
	__u64                      ino;                  /*    16     8 */
	__u32                      uid;                  /*    24     4 */
	__u32                      gid;                  /*    28     4 */
	__u32                      pid;                  /*    32     4 */
	__u32                      tgid;                 /*    36     4 */
	__u32                      len;                  /*    40     4 */
	char                       name[256];            /*    44   256 */

	/* size: 304, cachelines: 5, members: 10 */
	/* padding: 4 */
	/* last cacheline: 48 bytes */
};
union autofs_v5_packet_union {
	struct autofs_packet_hdr   hdr;                /*     0     8 */
	struct autofs_v5_packet    v5_packet;          /*     0   304 */
	autofs_packet_missing_indirect_t missing_indirect; /*     0   304 */
	autofs_packet_expire_indirect_t expire_indirect; /*     0   304 */
	autofs_packet_missing_direct_t missing_direct; /*     0   304 */
	autofs_packet_expire_direct_t expire_direct;   /*     0   304 */
};
struct args_protover {
	__u32                      version;              /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct args_protosubver {
	__u32                      sub_version;          /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct args_openmount {
	__u32                      devid;                /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct args_ready {
	__u32                      token;                /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct args_fail {
	__u32                      token;                /*     0     4 */
	__s32                      status;               /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct args_setpipefd {
	__s32                      pipefd;               /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct args_timeout {
	__u64                      timeout;              /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct args_requester {
	__u32                      uid;                  /*     0     4 */
	__u32                      gid;                  /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct args_expire {
	__u32                      how;                  /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct args_askumount {
	__u32                      may_umount;           /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct args_in {
	__u32                      type;                 /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct args_out {
	__u32                      devid;                /*     0     4 */
	__u32                      magic;                /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct args_ismountpoint {
	union {
		struct args_in     in;                   /*     0     4 */
		struct args_out    out;                  /*     0     8 */
	};                                               /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct autofs_dev_ioctl {
	__u32                      ver_major;            /*     0     4 */
	__u32                      ver_minor;            /*     4     4 */
	__u32                      size;                 /*     8     4 */
	__s32                      ioctlfd;              /*    12     4 */
	union {
		struct args_protover protover;           /*    16     4 */
		struct args_protosubver protosubver;     /*    16     4 */
		struct args_openmount openmount;         /*    16     4 */
		struct args_ready  ready;                /*    16     4 */
		struct args_fail   fail;                 /*    16     8 */
		struct args_setpipefd setpipefd;         /*    16     4 */
		struct args_timeout timeout;             /*    16     8 */
		struct args_requester requester;         /*    16     8 */
		struct args_expire expire;               /*    16     4 */
		struct args_askumount askumount;         /*    16     4 */
		struct args_ismountpoint ismountpoint;   /*    16     8 */
	};                                               /*    16     8 */
	char                       path[];               /*    24     0 */

	/* size: 24, cachelines: 1, members: 6 */
	/* last cacheline: 24 bytes */
};
struct debugfs_fsdata {
	const struct file_operations  * real_fops;       /*     0     8 */
	refcount_t                 active_users;         /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct completion          active_users_drained; /*    16    32 */

	/* size: 48, cachelines: 1, members: 3 */
	/* sum members: 44, holes: 1, sum holes: 4 */
	/* last cacheline: 48 bytes */
};
struct debugfs_mount_opts {
	kuid_t                     uid;                  /*     0     4 */
	kgid_t                     gid;                  /*     4     4 */
	umode_t                    mode;                 /*     8     2 */

	/* size: 12, cachelines: 1, members: 3 */
	/* padding: 2 */
	/* last cacheline: 12 bytes */
};
struct debugfs_fs_info {
	struct debugfs_mount_opts  mount_opts;           /*     0    12 */

	/* XXX last struct has 2 bytes of padding */

	/* size: 12, cachelines: 1, members: 1 */
	/* paddings: 1, sum paddings: 2 */
	/* last cacheline: 12 bytes */
};
struct debugfs_reg32 {
	char *                     name;                 /*     0     8 */
	long unsigned int          offset;               /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct debugfs_regset32 {
	const struct debugfs_reg32  * regs;              /*     0     8 */
	int                        nregs;                /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     base;                 /*    16     8 */
	struct device *            dev;                  /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct debugfs_u32_array {
	u32 *                      array;                /*     0     8 */
	u32                        n_elements;           /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct debugfs_devm_entry {
	int                        (*read)(struct seq_file *, void *); /*     0     8 */
	struct device *            dev;                  /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct tracefs_dir_ops {
	int                        (*mkdir)(const char  *); /*     0     8 */
	int                        (*rmdir)(const char  *); /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct tracefs_mount_opts {
	kuid_t                     uid;                  /*     0     4 */
	kgid_t                     gid;                  /*     4     4 */
	umode_t                    mode;                 /*     8     2 */

	/* size: 12, cachelines: 1, members: 3 */
	/* padding: 2 */
	/* last cacheline: 12 bytes */
};
struct tracefs_fs_info {
	struct tracefs_mount_opts  mount_opts;           /*     0    12 */

	/* XXX last struct has 2 bytes of padding */

	/* size: 12, cachelines: 1, members: 1 */
	/* paddings: 1, sum paddings: 2 */
	/* last cacheline: 12 bytes */
};
struct pstore_record {
	struct pstore_info *       psi;                  /*     0     8 */
	enum pstore_type_id        type;                 /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        id;                   /*    16     8 */
	struct timespec64          time;                 /*    24    16 */
	char *                     buf;                  /*    40     8 */
	ssize_t                    size;                 /*    48     8 */
	ssize_t                    ecc_notice_size;      /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        count;                /*    64     4 */
	enum kmsg_dump_reason      reason;               /*    68     4 */
	unsigned int               part;                 /*    72     4 */
	bool                       compressed;           /*    76     1 */

	/* size: 80, cachelines: 2, members: 11 */
	/* sum members: 73, holes: 1, sum holes: 4 */
	/* padding: 3 */
	/* last cacheline: 16 bytes */
};
struct pstore_info {
	struct module *            owner;                /*     0     8 */
	const char  *              name;                 /*     8     8 */
	spinlock_t                 buf_lock;             /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	char *                     buf;                  /*    24     8 */
	size_t                     bufsize;              /*    32     8 */
	struct mutex               read_mutex;           /*    40    32 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	int                        flags;                /*    72     4 */
	int                        max_reason;           /*    76     4 */
	void *                     data;                 /*    80     8 */
	int                        (*open)(struct pstore_info *); /*    88     8 */
	int                        (*close)(struct pstore_info *); /*    96     8 */
	ssize_t                    (*read)(struct pstore_record *); /*   104     8 */
	int                        (*write)(struct pstore_record *); /*   112     8 */
	int                        (*write_user)(struct pstore_record *, const char  *); /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	int                        (*erase)(struct pstore_record *); /*   128     8 */

	/* size: 136, cachelines: 3, members: 15 */
	/* sum members: 132, holes: 1, sum holes: 4 */
	/* last cacheline: 8 bytes */
};
struct pstore_ftrace_record {
	long unsigned int          ip;                   /*     0     8 */
	long unsigned int          parent_ip;            /*     8     8 */
	u64                        ts;                   /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct pstore_private {
	struct list_head           list;                 /*     0    16 */
	struct dentry *            dentry;               /*    16     8 */
	struct pstore_record *     record;               /*    24     8 */
	size_t                     total_size;           /*    32     8 */

	/* size: 40, cachelines: 1, members: 4 */
	/* last cacheline: 40 bytes */
};
struct pstore_ftrace_seq_data {
	const void  *              ptr;                  /*     0     8 */
	size_t                     off;                  /*     8     8 */
	size_t                     size;                 /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct crypto_comp {
	struct crypto_tfm          base __attribute__((__aligned__(8))); /*     0    24 */

	/* size: 24, cachelines: 1, members: 1 */
	/* forced alignments: 1 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct pstore_zbackend {
	int                        (*zbufsize)(size_t);  /*     0     8 */
	const char  *              name;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct efi_variable {
	efi_char16_t               VariableName[512];    /*     0  1024 */
	/* --- cacheline 16 boundary (1024 bytes) --- */
	efi_guid_t                 VendorGuid __attribute__((__aligned__(1))); /*  1024    16 */
	long unsigned int          DataSize;             /*  1040     8 */
	__u8                       Data[1024];           /*  1048  1024 */
	/* --- cacheline 32 boundary (2048 bytes) was 24 bytes ago --- */
	efi_status_t               Status;               /*  2072     8 */
	__u32                      Attributes;           /*  2080     4 */

	/* size: 2084, cachelines: 33, members: 6 */
	/* forced alignments: 1 */
	/* last cacheline: 36 bytes */
} __attribute__((__packed__));
struct efivar_entry {
	struct efi_variable        var __attribute__((__aligned__(1))); /*     0  2084 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 32 boundary (2048 bytes) was 40 bytes ago --- */
	struct list_head           list;                 /*  2088    16 */
	struct kobject             kobj;                 /*  2104    96 */
	/* --- cacheline 34 boundary (2176 bytes) was 24 bytes ago --- */
	bool                       scanning;             /*  2200     1 */
	bool                       deleting;             /*  2201     1 */

	/* size: 2208, cachelines: 35, members: 5 */
	/* sum members: 2198, holes: 1, sum holes: 4 */
	/* padding: 6 */
	/* forced alignments: 1 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct ipc64_perm {
	__kernel_key_t             key;                  /*     0     4 */
	__kernel_uid32_t           uid;                  /*     4     4 */
	__kernel_gid32_t           gid;                  /*     8     4 */
	__kernel_uid32_t           cuid;                 /*    12     4 */
	__kernel_gid32_t           cgid;                 /*    16     4 */
	__kernel_mode_t            mode;                 /*    20     4 */
	unsigned char              __pad1[0];            /*    24     0 */
	short unsigned int         seq;                  /*    24     2 */
	short unsigned int         __pad2;               /*    26     2 */

	/* XXX 4 bytes hole, try to pack */

	__kernel_ulong_t           __unused1;            /*    32     8 */
	__kernel_ulong_t           __unused2;            /*    40     8 */

	/* size: 48, cachelines: 1, members: 11 */
	/* sum members: 44, holes: 1, sum holes: 4 */
	/* last cacheline: 48 bytes */
};
struct compat_ipc64_perm {
	compat_key_t               key;                  /*     0     4 */
	__compat_uid32_t           uid;                  /*     4     4 */
	__compat_gid32_t           gid;                  /*     8     4 */
	__compat_uid32_t           cuid;                 /*    12     4 */
	__compat_gid32_t           cgid;                 /*    16     4 */
	short unsigned int         mode;                 /*    20     2 */
	short unsigned int         __pad1;               /*    22     2 */
	short unsigned int         seq;                  /*    24     2 */
	short unsigned int         __pad2;               /*    26     2 */
	compat_ulong_t             unused1;              /*    28     4 */
	compat_ulong_t             unused2;              /*    32     4 */

	/* size: 36, cachelines: 1, members: 11 */
	/* last cacheline: 36 bytes */
};
struct compat_ipc_perm {
	key_t                      key;                  /*     0     4 */
	__compat_uid_t             uid;                  /*     4     2 */
	__compat_gid_t             gid;                  /*     6     2 */
	__compat_uid_t             cuid;                 /*     8     2 */
	__compat_gid_t             cgid;                 /*    10     2 */
	compat_mode_t              mode;                 /*    12     2 */
	short unsigned int         seq;                  /*    14     2 */

	/* size: 16, cachelines: 1, members: 7 */
	/* last cacheline: 16 bytes */
};
struct ipc_perm {
	__kernel_key_t             key;                  /*     0     4 */
	__kernel_uid_t             uid;                  /*     4     4 */
	__kernel_gid_t             gid;                  /*     8     4 */
	__kernel_uid_t             cuid;                 /*    12     4 */
	__kernel_gid_t             cgid;                 /*    16     4 */
	__kernel_mode_t            mode;                 /*    20     4 */
	short unsigned int         seq;                  /*    24     2 */

	/* size: 28, cachelines: 1, members: 7 */
	/* padding: 2 */
	/* last cacheline: 28 bytes */
};
struct ipc_params {
	key_t                      key;                  /*     0     4 */
	int                        flg;                  /*     4     4 */
	union {
		size_t             size;                 /*     8     8 */
		int                nsems;                /*     8     4 */
	} u;                                             /*     8     8 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct ipc_ops {
	int                        (*getnew)(struct ipc_namespace *, struct ipc_params *); /*     0     8 */
	int                        (*associate)(struct kern_ipc_perm *, int); /*     8     8 */
	int                        (*more_checks)(struct kern_ipc_perm *, struct ipc_params *); /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct ipc_proc_iface {
	const char  *              path;                 /*     0     8 */
	const char  *              header;               /*     8     8 */
	int                        ids;                  /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	int                        (*show)(struct seq_file *, void *); /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct ipc_proc_iter {
	struct ipc_namespace *     ns;                   /*     0     8 */
	struct pid_namespace *     pid_ns;               /*     8     8 */
	struct ipc_proc_iface *    iface;                /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct msg_msg {
	struct list_head           m_list;               /*     0    16 */
	long int                   m_type;               /*    16     8 */
	size_t                     m_ts;                 /*    24     8 */
	struct msg_msgseg *        next;                 /*    32     8 */
	void *                     security;             /*    40     8 */

	/* size: 48, cachelines: 1, members: 5 */
	/* last cacheline: 48 bytes */
};
struct msg_msgseg {
	struct msg_msgseg *        next;                 /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct msgbuf {
	__kernel_long_t            mtype;                /*     0     8 */
	char                       mtext[1];             /*     8     1 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 7 */
	/* last cacheline: 16 bytes */
};
struct msqid_ds {
	struct ipc_perm            msg_perm;             /*     0    28 */

	/* XXX last struct has 2 bytes of padding */
	/* XXX 4 bytes hole, try to pack */

	struct msg *               msg_first;            /*    32     8 */
	struct msg *               msg_last;             /*    40     8 */
	__kernel_old_time_t        msg_stime;            /*    48     8 */
	__kernel_old_time_t        msg_rtime;            /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	__kernel_old_time_t        msg_ctime;            /*    64     8 */
	long unsigned int          msg_lcbytes;          /*    72     8 */
	long unsigned int          msg_lqbytes;          /*    80     8 */
	short unsigned int         msg_cbytes;           /*    88     2 */
	short unsigned int         msg_qnum;             /*    90     2 */
	short unsigned int         msg_qbytes;           /*    92     2 */

	/* XXX 2 bytes hole, try to pack */

	__kernel_ipc_pid_t         msg_lspid;            /*    96     4 */
	__kernel_ipc_pid_t         msg_lrpid;            /*   100     4 */

	/* size: 104, cachelines: 2, members: 13 */
	/* sum members: 98, holes: 2, sum holes: 6 */
	/* paddings: 1, sum paddings: 2 */
	/* last cacheline: 40 bytes */
};
struct msqid64_ds {
	struct ipc64_perm          msg_perm;             /*     0    48 */
	long int                   msg_stime;            /*    48     8 */
	long int                   msg_rtime;            /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	long int                   msg_ctime;            /*    64     8 */
	long unsigned int          msg_cbytes;           /*    72     8 */
	long unsigned int          msg_qnum;             /*    80     8 */
	long unsigned int          msg_qbytes;           /*    88     8 */
	__kernel_pid_t             msg_lspid;            /*    96     4 */
	__kernel_pid_t             msg_lrpid;            /*   100     4 */
	long unsigned int          __unused4;            /*   104     8 */
	long unsigned int          __unused5;            /*   112     8 */

	/* size: 120, cachelines: 2, members: 11 */
	/* last cacheline: 56 bytes */
};
struct msginfo {
	int                        msgpool;              /*     0     4 */
	int                        msgmap;               /*     4     4 */
	int                        msgmax;               /*     8     4 */
	int                        msgmnb;               /*    12     4 */
	int                        msgmni;               /*    16     4 */
	int                        msgssz;               /*    20     4 */
	int                        msgtql;               /*    24     4 */
	short unsigned int         msgseg;               /*    28     2 */

	/* size: 32, cachelines: 1, members: 8 */
	/* padding: 2 */
	/* last cacheline: 32 bytes */
};
struct compat_msqid64_ds {
	struct compat_ipc64_perm   msg_perm;             /*     0    36 */
	compat_ulong_t             msg_stime;            /*    36     4 */
	compat_ulong_t             msg_stime_high;       /*    40     4 */
	compat_ulong_t             msg_rtime;            /*    44     4 */
	compat_ulong_t             msg_rtime_high;       /*    48     4 */
	compat_ulong_t             msg_ctime;            /*    52     4 */
	compat_ulong_t             msg_ctime_high;       /*    56     4 */
	compat_ulong_t             msg_cbytes;           /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	compat_ulong_t             msg_qnum;             /*    64     4 */
	compat_ulong_t             msg_qbytes;           /*    68     4 */
	compat_pid_t               msg_lspid;            /*    72     4 */
	compat_pid_t               msg_lrpid;            /*    76     4 */
	compat_ulong_t             __unused4;            /*    80     4 */
	compat_ulong_t             __unused5;            /*    84     4 */

	/* size: 88, cachelines: 2, members: 14 */
	/* last cacheline: 24 bytes */
};
struct msg_queue {
	struct kern_ipc_perm       q_perm __attribute__((__aligned__(64))); /*     0   128 */

	/* XXX last struct has 44 bytes of padding */

	/* --- cacheline 2 boundary (128 bytes) --- */
	time64_t                   q_stime;              /*   128     8 */
	time64_t                   q_rtime;              /*   136     8 */
	time64_t                   q_ctime;              /*   144     8 */
	long unsigned int          q_cbytes;             /*   152     8 */
	long unsigned int          q_qnum;               /*   160     8 */
	long unsigned int          q_qbytes;             /*   168     8 */
	struct pid *               q_lspid;              /*   176     8 */
	struct pid *               q_lrpid;              /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	struct list_head           q_messages;           /*   192    16 */
	struct list_head           q_receivers;          /*   208    16 */
	struct list_head           q_senders;            /*   224    16 */

	/* size: 256, cachelines: 4, members: 12 */
	/* padding: 16 */
	/* paddings: 1, sum paddings: 44 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(64)));
struct msg_receiver {
	struct list_head           r_list;               /*     0    16 */
	struct task_struct *       r_tsk;                /*    16     8 */
	int                        r_mode;               /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	long int                   r_msgtype;            /*    32     8 */
	long int                   r_maxsize;            /*    40     8 */
	struct msg_msg *           r_msg;                /*    48     8 */

	/* size: 56, cachelines: 1, members: 6 */
	/* sum members: 52, holes: 1, sum holes: 4 */
	/* last cacheline: 56 bytes */
};
struct msg_sender {
	struct list_head           list;                 /*     0    16 */
	struct task_struct *       tsk;                  /*    16     8 */
	size_t                     msgsz;                /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* last cacheline: 32 bytes */
};
struct compat_msqid_ds {
	struct compat_ipc_perm     msg_perm;             /*     0    16 */
	compat_uptr_t              msg_first;            /*    16     4 */
	compat_uptr_t              msg_last;             /*    20     4 */
	old_time32_t               msg_stime;            /*    24     4 */
	old_time32_t               msg_rtime;            /*    28     4 */
	old_time32_t               msg_ctime;            /*    32     4 */
	compat_ulong_t             msg_lcbytes;          /*    36     4 */
	compat_ulong_t             msg_lqbytes;          /*    40     4 */
	short unsigned int         msg_cbytes;           /*    44     2 */
	short unsigned int         msg_qnum;             /*    46     2 */
	short unsigned int         msg_qbytes;           /*    48     2 */
	compat_ipc_pid_t           msg_lspid;            /*    50     2 */
	compat_ipc_pid_t           msg_lrpid;            /*    52     2 */

	/* size: 56, cachelines: 1, members: 13 */
	/* padding: 2 */
	/* last cacheline: 56 bytes */
};
struct compat_msgbuf {
	compat_long_t              mtype;                /*     0     4 */
	char                       mtext[1];             /*     4     1 */

	/* size: 8, cachelines: 1, members: 2 */
	/* padding: 3 */
	/* last cacheline: 8 bytes */
};
struct semid_ds {
	struct ipc_perm            sem_perm;             /*     0    28 */

	/* XXX last struct has 2 bytes of padding */
	/* XXX 4 bytes hole, try to pack */

	__kernel_old_time_t        sem_otime;            /*    32     8 */
	__kernel_old_time_t        sem_ctime;            /*    40     8 */
	struct sem *               sem_base;             /*    48     8 */
	struct sem_queue *         sem_pending;          /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct sem_queue * *       sem_pending_last;     /*    64     8 */
	struct sem_undo *          undo;                 /*    72     8 */
	short unsigned int         sem_nsems;            /*    80     2 */

	/* size: 88, cachelines: 2, members: 8 */
	/* sum members: 78, holes: 1, sum holes: 4 */
	/* padding: 6 */
	/* paddings: 1, sum paddings: 2 */
	/* last cacheline: 24 bytes */
};
struct sem {
	int                        semval;               /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct pid *               sempid;               /*     8     8 */
	spinlock_t                 lock;                 /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           pending_alter;        /*    24    16 */
	struct list_head           pending_const;        /*    40    16 */
	time64_t                   sem_otime;            /*    56     8 */

	/* size: 64, cachelines: 1, members: 6 */
	/* sum members: 56, holes: 2, sum holes: 8 */
} __attribute__((__aligned__(64)));
struct sem_queue {
	struct list_head           list;                 /*     0    16 */
	struct task_struct *       sleeper;              /*    16     8 */
	struct sem_undo *          undo;                 /*    24     8 */
	struct pid *               pid;                  /*    32     8 */
	int                        status;               /*    40     4 */

	/* XXX 4 bytes hole, try to pack */

	struct sembuf *            sops;                 /*    48     8 */
	struct sembuf *            blocking;             /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        nsops;                /*    64     4 */
	bool                       alter;                /*    68     1 */
	bool                       dupsop;               /*    69     1 */

	/* size: 72, cachelines: 2, members: 10 */
	/* sum members: 66, holes: 1, sum holes: 4 */
	/* padding: 2 */
	/* last cacheline: 8 bytes */
};
struct sem_undo {
	struct list_head           list_proc;            /*     0    16 */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*    16    16 */
	struct sem_undo_list *     ulp;                  /*    32     8 */
	struct list_head           list_id;              /*    40    16 */
	int                        semid;                /*    56     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	short int *                semadj;               /*    64     8 */

	/* size: 72, cachelines: 2, members: 6 */
	/* sum members: 68, holes: 1, sum holes: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 8 bytes */
} __attribute__((__aligned__(8)));
struct semid64_ds {
	struct ipc64_perm          sem_perm;             /*     0    48 */
	__kernel_long_t            sem_otime;            /*    48     8 */
	__kernel_ulong_t           __unused1;            /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	__kernel_long_t            sem_ctime;            /*    64     8 */
	__kernel_ulong_t           __unused2;            /*    72     8 */
	__kernel_ulong_t           sem_nsems;            /*    80     8 */
	__kernel_ulong_t           __unused3;            /*    88     8 */
	__kernel_ulong_t           __unused4;            /*    96     8 */

	/* size: 104, cachelines: 2, members: 8 */
	/* last cacheline: 40 bytes */
};
struct sembuf {
	short unsigned int         sem_num;              /*     0     2 */
	short int                  sem_op;               /*     2     2 */
	short int                  sem_flg;              /*     4     2 */

	/* size: 6, cachelines: 1, members: 3 */
	/* last cacheline: 6 bytes */
};
struct seminfo {
	int                        semmap;               /*     0     4 */
	int                        semmni;               /*     4     4 */
	int                        semmns;               /*     8     4 */
	int                        semmnu;               /*    12     4 */
	int                        semmsl;               /*    16     4 */
	int                        semopm;               /*    20     4 */
	int                        semume;               /*    24     4 */
	int                        semusz;               /*    28     4 */
	int                        semvmx;               /*    32     4 */
	int                        semaem;               /*    36     4 */

	/* size: 40, cachelines: 1, members: 10 */
	/* last cacheline: 40 bytes */
};
struct sem_undo_list {
	refcount_t                 refcnt;               /*     0     4 */
	spinlock_t                 lock;                 /*     4     4 */
	struct list_head           list_proc;            /*     8    16 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct compat_semid64_ds {
	struct compat_ipc64_perm   sem_perm;             /*     0    36 */
	compat_ulong_t             sem_otime;            /*    36     4 */
	compat_ulong_t             sem_otime_high;       /*    40     4 */
	compat_ulong_t             sem_ctime;            /*    44     4 */
	compat_ulong_t             sem_ctime_high;       /*    48     4 */
	compat_ulong_t             sem_nsems;            /*    52     4 */
	compat_ulong_t             __unused3;            /*    56     4 */
	compat_ulong_t             __unused4;            /*    60     4 */

	/* size: 64, cachelines: 1, members: 8 */
};
struct sem_array {
	struct kern_ipc_perm       sem_perm __attribute__((__aligned__(64))); /*     0   128 */

	/* XXX last struct has 44 bytes of padding */

	/* --- cacheline 2 boundary (128 bytes) --- */
	time64_t                   sem_ctime;            /*   128     8 */
	struct list_head           pending_alter;        /*   136    16 */
	struct list_head           pending_const;        /*   152    16 */
	struct list_head           list_id;              /*   168    16 */
	int                        sem_nsems;            /*   184     4 */
	int                        complex_count;        /*   188     4 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	unsigned int               use_global_lock;      /*   192     4 */

	/* XXX 60 bytes hole, try to pack */

	/* --- cacheline 4 boundary (256 bytes) --- */
	struct sem                 sems[] __attribute__((__aligned__(64))); /*   256     0 */

	/* size: 256, cachelines: 4, members: 9 */
	/* sum members: 196, holes: 1, sum holes: 60 */
	/* paddings: 1, sum paddings: 44 */
	/* forced alignments: 2, forced holes: 1, sum forced holes: 60 */
} __attribute__((__aligned__(64)));
struct compat_semid_ds {
	struct compat_ipc_perm     sem_perm;             /*     0    16 */
	old_time32_t               sem_otime;            /*    16     4 */
	old_time32_t               sem_ctime;            /*    20     4 */
	compat_uptr_t              sem_base;             /*    24     4 */
	compat_uptr_t              sem_pending;          /*    28     4 */
	compat_uptr_t              sem_pending_last;     /*    32     4 */
	compat_uptr_t              undo;                 /*    36     4 */
	short unsigned int         sem_nsems;            /*    40     2 */

	/* size: 44, cachelines: 1, members: 8 */
	/* padding: 2 */
	/* last cacheline: 44 bytes */
};
struct shmid_ds {
	struct ipc_perm            shm_perm;             /*     0    28 */

	/* XXX last struct has 2 bytes of padding */

	int                        shm_segsz;            /*    28     4 */
	__kernel_old_time_t        shm_atime;            /*    32     8 */
	__kernel_old_time_t        shm_dtime;            /*    40     8 */
	__kernel_old_time_t        shm_ctime;            /*    48     8 */
	__kernel_ipc_pid_t         shm_cpid;             /*    56     4 */
	__kernel_ipc_pid_t         shm_lpid;             /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	short unsigned int         shm_nattch;           /*    64     2 */
	short unsigned int         shm_unused;           /*    66     2 */

	/* XXX 4 bytes hole, try to pack */

	void *                     shm_unused2;          /*    72     8 */
	void *                     shm_unused3;          /*    80     8 */

	/* size: 88, cachelines: 2, members: 11 */
	/* sum members: 84, holes: 1, sum holes: 4 */
	/* paddings: 1, sum paddings: 2 */
	/* last cacheline: 24 bytes */
};
struct shmid64_ds {
	struct ipc64_perm          shm_perm;             /*     0    48 */
	size_t                     shm_segsz;            /*    48     8 */
	long int                   shm_atime;            /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	long int                   shm_dtime;            /*    64     8 */
	long int                   shm_ctime;            /*    72     8 */
	__kernel_pid_t             shm_cpid;             /*    80     4 */
	__kernel_pid_t             shm_lpid;             /*    84     4 */
	long unsigned int          shm_nattch;           /*    88     8 */
	long unsigned int          __unused4;            /*    96     8 */
	long unsigned int          __unused5;            /*   104     8 */

	/* size: 112, cachelines: 2, members: 10 */
	/* last cacheline: 48 bytes */
};
struct shminfo64 {
	long unsigned int          shmmax;               /*     0     8 */
	long unsigned int          shmmin;               /*     8     8 */
	long unsigned int          shmmni;               /*    16     8 */
	long unsigned int          shmseg;               /*    24     8 */
	long unsigned int          shmall;               /*    32     8 */
	long unsigned int          __unused1;            /*    40     8 */
	long unsigned int          __unused2;            /*    48     8 */
	long unsigned int          __unused3;            /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	long unsigned int          __unused4;            /*    64     8 */

	/* size: 72, cachelines: 2, members: 9 */
	/* last cacheline: 8 bytes */
};
struct shminfo {
	int                        shmmax;               /*     0     4 */
	int                        shmmin;               /*     4     4 */
	int                        shmmni;               /*     8     4 */
	int                        shmseg;               /*    12     4 */
	int                        shmall;               /*    16     4 */

	/* size: 20, cachelines: 1, members: 5 */
	/* last cacheline: 20 bytes */
};
struct shm_info {
	int                        used_ids;             /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	__kernel_ulong_t           shm_tot;              /*     8     8 */
	__kernel_ulong_t           shm_rss;              /*    16     8 */
	__kernel_ulong_t           shm_swp;              /*    24     8 */
	__kernel_ulong_t           swap_attempts;        /*    32     8 */
	__kernel_ulong_t           swap_successes;       /*    40     8 */

	/* size: 48, cachelines: 1, members: 6 */
	/* sum members: 44, holes: 1, sum holes: 4 */
	/* last cacheline: 48 bytes */
};
struct compat_shmid64_ds {
	struct compat_ipc64_perm   shm_perm;             /*     0    36 */
	compat_size_t              shm_segsz;            /*    36     4 */
	compat_ulong_t             shm_atime;            /*    40     4 */
	compat_ulong_t             shm_atime_high;       /*    44     4 */
	compat_ulong_t             shm_dtime;            /*    48     4 */
	compat_ulong_t             shm_dtime_high;       /*    52     4 */
	compat_ulong_t             shm_ctime;            /*    56     4 */
	compat_ulong_t             shm_ctime_high;       /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	compat_pid_t               shm_cpid;             /*    64     4 */
	compat_pid_t               shm_lpid;             /*    68     4 */
	compat_ulong_t             shm_nattch;           /*    72     4 */
	compat_ulong_t             __unused4;            /*    76     4 */
	compat_ulong_t             __unused5;            /*    80     4 */

	/* size: 84, cachelines: 2, members: 13 */
	/* last cacheline: 20 bytes */
};
struct shmid_kernel {
	struct kern_ipc_perm       shm_perm __attribute__((__aligned__(64))); /*     0   128 */

	/* XXX last struct has 44 bytes of padding */

	/* --- cacheline 2 boundary (128 bytes) --- */
	struct file *              shm_file;             /*   128     8 */
	long unsigned int          shm_nattch;           /*   136     8 */
	long unsigned int          shm_segsz;            /*   144     8 */
	time64_t                   shm_atim;             /*   152     8 */
	time64_t                   shm_dtim;             /*   160     8 */
	time64_t                   shm_ctim;             /*   168     8 */
	struct pid *               shm_cprid;            /*   176     8 */
	struct pid *               shm_lprid;            /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	struct user_struct *       mlock_user;           /*   192     8 */
	struct task_struct *       shm_creator;          /*   200     8 */
	struct list_head           shm_clist;            /*   208    16 */
	struct ipc_namespace *     ns;                   /*   224     8 */

	/* size: 256, cachelines: 4, members: 13 */
	/* padding: 24 */
	/* paddings: 1, sum paddings: 44 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(64)));
struct shm_file_data {
	int                        id;                   /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct ipc_namespace *     ns;                   /*     8     8 */
	struct file *              file;                 /*    16     8 */
	const struct vm_operations_struct  * vm_ops;     /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct compat_shmid_ds {
	struct compat_ipc_perm     shm_perm;             /*     0    16 */
	int                        shm_segsz;            /*    16     4 */
	old_time32_t               shm_atime;            /*    20     4 */
	old_time32_t               shm_dtime;            /*    24     4 */
	old_time32_t               shm_ctime;            /*    28     4 */
	compat_ipc_pid_t           shm_cpid;             /*    32     2 */
	compat_ipc_pid_t           shm_lpid;             /*    34     2 */
	short unsigned int         shm_nattch;           /*    36     2 */
	short unsigned int         shm_unused;           /*    38     2 */
	compat_uptr_t              shm_unused2;          /*    40     4 */
	compat_uptr_t              shm_unused3;          /*    44     4 */

	/* size: 48, cachelines: 1, members: 11 */
	/* last cacheline: 48 bytes */
};
struct compat_shminfo64 {
	compat_ulong_t             shmmax;               /*     0     4 */
	compat_ulong_t             shmmin;               /*     4     4 */
	compat_ulong_t             shmmni;               /*     8     4 */
	compat_ulong_t             shmseg;               /*    12     4 */
	compat_ulong_t             shmall;               /*    16     4 */
	compat_ulong_t             __unused1;            /*    20     4 */
	compat_ulong_t             __unused2;            /*    24     4 */
	compat_ulong_t             __unused3;            /*    28     4 */
	compat_ulong_t             __unused4;            /*    32     4 */

	/* size: 36, cachelines: 1, members: 9 */
	/* last cacheline: 36 bytes */
};
struct compat_shm_info {
	compat_int_t               used_ids;             /*     0     4 */
	compat_ulong_t             shm_tot;              /*     4     4 */
	compat_ulong_t             shm_rss;              /*     8     4 */
	compat_ulong_t             shm_swp;              /*    12     4 */
	compat_ulong_t             swap_attempts;        /*    16     4 */
	compat_ulong_t             swap_successes;       /*    20     4 */

	/* size: 24, cachelines: 1, members: 6 */
	/* last cacheline: 24 bytes */
};
struct compat_ipc_kludge {
	compat_uptr_t              msgp;                 /*     0     4 */
	compat_long_t              msgtyp;               /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct mqueue_fs_context {
	struct ipc_namespace *     ipc_ns;               /*     0     8 */
	bool                       newns;                /*     8     1 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 7 */
	/* last cacheline: 16 bytes */
};
struct posix_msg_tree_node {
	struct rb_node             rb_node __attribute__((__aligned__(8))); /*     0    24 */
	struct list_head           msg_list;             /*    24    16 */
	int                        priority;             /*    40     4 */

	/* size: 48, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct ext_wait_queue {
	struct task_struct *       task;                 /*     0     8 */
	struct list_head           list;                 /*     8    16 */
	struct msg_msg *           msg;                  /*    24     8 */
	int                        state;                /*    32     4 */

	/* size: 40, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct mqueue_inode_info {
	spinlock_t                 lock;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct inode               vfs_inode __attribute__((__aligned__(8))); /*     8   624 */
	/* --- cacheline 9 boundary (576 bytes) was 56 bytes ago --- */
	wait_queue_head_t          wait_q;               /*   632    24 */
	/* --- cacheline 10 boundary (640 bytes) was 16 bytes ago --- */
	struct rb_root             msg_tree;             /*   656     8 */
	struct rb_node *           msg_tree_rightmost;   /*   664     8 */
	struct posix_msg_tree_node * node_cache;         /*   672     8 */
	struct mq_attr             attr;                 /*   680    64 */
	/* --- cacheline 11 boundary (704 bytes) was 40 bytes ago --- */
	struct sigevent            notify;               /*   744    64 */
	/* --- cacheline 12 boundary (768 bytes) was 40 bytes ago --- */
	struct pid *               notify_owner;         /*   808     8 */
	u32                        notify_self_exec_id;  /*   816     4 */

	/* XXX 4 bytes hole, try to pack */

	struct user_namespace *    notify_user_ns;       /*   824     8 */
	/* --- cacheline 13 boundary (832 bytes) --- */
	struct user_struct *       user;                 /*   832     8 */
	struct sock *              notify_sock;          /*   840     8 */
	struct sk_buff *           notify_cookie;        /*   848     8 */
	struct ext_wait_queue      e_wait_q[2];          /*   856    80 */
	/* --- cacheline 14 boundary (896 bytes) was 40 bytes ago --- */
	long unsigned int          qsize;                /*   936     8 */

	/* size: 944, cachelines: 15, members: 16 */
	/* sum members: 936, holes: 2, sum holes: 8 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 4 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct compat_mq_attr {
	compat_long_t              mq_flags;             /*     0     4 */
	compat_long_t              mq_maxmsg;            /*     4     4 */
	compat_long_t              mq_msgsize;           /*     8     4 */
	compat_long_t              mq_curmsgs;           /*    12     4 */
	compat_long_t              __reserved[4];        /*    16    16 */

	/* size: 32, cachelines: 1, members: 5 */
	/* last cacheline: 32 bytes */
};
struct key_user {
	struct rb_node             node __attribute__((__aligned__(8))); /*     0    24 */
	struct mutex               cons_lock;            /*    24    32 */
	spinlock_t                 lock;                 /*    56     4 */
	refcount_t                 usage;                /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	atomic_t                   nkeys;                /*    64     4 */
	atomic_t                   nikeys;               /*    68     4 */
	kuid_t                     uid;                  /*    72     4 */
	int                        qnkeys;               /*    76     4 */
	int                        qnbytes;              /*    80     4 */

	/* size: 88, cachelines: 2, members: 9 */
	/* padding: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct assoc_array_ops {
	long unsigned int          (*get_key_chunk)(const void  *, int); /*     0     8 */
	long unsigned int          (*get_object_key_chunk)(const void  *, int); /*     8     8 */
	bool                       (*compare_object)(const void  *, const void  *); /*    16     8 */
	int                        (*diff_objects)(const void  *, const void  *); /*    24     8 */
	void                       (*free_object)(void *); /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct assoc_array_node {
	struct assoc_array_ptr *   back_pointer;         /*     0     8 */
	u8                         parent_slot;          /*     8     1 */

	/* XXX 7 bytes hole, try to pack */

	struct assoc_array_ptr *   slots[16];            /*    16   128 */
	/* --- cacheline 2 boundary (128 bytes) was 16 bytes ago --- */
	long unsigned int          nr_leaves_on_branch;  /*   144     8 */

	/* size: 152, cachelines: 3, members: 4 */
	/* sum members: 145, holes: 1, sum holes: 7 */
	/* last cacheline: 24 bytes */
};
struct assoc_array_shortcut {
	struct assoc_array_ptr *   back_pointer;         /*     0     8 */
	int                        parent_slot;          /*     8     4 */
	int                        skip_to_level;        /*    12     4 */
	struct assoc_array_ptr *   next_node;            /*    16     8 */
	long unsigned int          index_key[];          /*    24     0 */

	/* size: 24, cachelines: 1, members: 5 */
	/* last cacheline: 24 bytes */
};
struct assoc_array_edit {
	struct callback_head       rcu __attribute__((__aligned__(8))); /*     0    16 */
	struct assoc_array *       array;                /*    16     8 */
	const struct assoc_array_ops  * ops;             /*    24     8 */
	const struct assoc_array_ops  * ops_for_excised_subtree; /*    32     8 */
	struct assoc_array_ptr *   leaf;                 /*    40     8 */
	struct assoc_array_ptr * * leaf_p;               /*    48     8 */
	struct assoc_array_ptr *   dead_leaf;            /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct assoc_array_ptr *   new_meta[3];          /*    64    24 */
	struct assoc_array_ptr *   excised_meta[1];      /*    88     8 */
	struct assoc_array_ptr *   excised_subtree;      /*    96     8 */
	struct assoc_array_ptr * * set_backpointers[16]; /*   104   128 */
	/* --- cacheline 3 boundary (192 bytes) was 40 bytes ago --- */
	struct assoc_array_ptr *   set_backpointers_to;  /*   232     8 */
	struct assoc_array_node *  adjust_count_on;      /*   240     8 */
	long int                   adjust_count_by;      /*   248     8 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	struct {
		struct assoc_array_ptr * * ptr;          /*   256     8 */
		struct assoc_array_ptr * to;             /*   264     8 */
	} set[2]; /*   256    32 */
	struct {
		u8 *               p;                    /*   288     8 */
		u8                 to;                   /*   296     1 */
	} set_parent_slot[1]; /*   288    16 */
	u8                         segment_cache[17];    /*   304    17 */

	/* size: 328, cachelines: 6, members: 17 */
	/* padding: 7 */
	/* forced alignments: 1 */
	/* last cacheline: 8 bytes */
} __attribute__((__aligned__(8)));
struct keyring_search_context {
	struct keyring_index_key   index_key;            /*     0    40 */
	const struct cred  *       cred;                 /*    40     8 */
	struct key_match_data      match_data;           /*    48    40 */
	/* --- cacheline 1 boundary (64 bytes) was 24 bytes ago --- */
	unsigned int               flags;                /*    88     4 */

	/* XXX 4 bytes hole, try to pack */

	int                        (*iterator)(const void  *, void *); /*    96     8 */
	int                        skipped_ret;          /*   104     4 */
	bool                       possessed;            /*   108     1 */

	/* XXX 3 bytes hole, try to pack */

	key_ref_t                  result;               /*   112     8 */
	time64_t                   now;                  /*   120     8 */

	/* size: 128, cachelines: 2, members: 9 */
	/* sum members: 121, holes: 2, sum holes: 7 */
};
struct keyring_read_iterator_context {
	size_t                     buflen;               /*     0     8 */
	size_t                     count;                /*     8     8 */
	key_serial_t *             buffer;               /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct keyctl_dh_params {
	union {
		__s32              private;              /*     0     4 */
		__s32              priv;                 /*     0     4 */
	};                                               /*     0     4 */
	__s32                      prime;                /*     4     4 */
	__s32                      base;                 /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct keyctl_kdf_params {
	char *                     hashname;             /*     0     8 */
	char *                     otherinfo;            /*     8     8 */
	__u32                      otherinfolen;         /*    16     4 */
	__u32                      __spare[8];           /*    20    32 */

	/* size: 56, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 56 bytes */
};
struct keyctl_pkey_query {
	__u32                      supported_ops;        /*     0     4 */
	__u32                      key_size;             /*     4     4 */
	__u16                      max_data_size;        /*     8     2 */
	__u16                      max_sig_size;         /*    10     2 */
	__u16                      max_enc_size;         /*    12     2 */
	__u16                      max_dec_size;         /*    14     2 */
	__u32                      __spare[10];          /*    16    40 */

	/* size: 56, cachelines: 1, members: 7 */
	/* last cacheline: 56 bytes */
};
struct keyctl_pkey_params {
	__s32                      key_id;               /*     0     4 */
	__u32                      in_len;               /*     4     4 */
	union {
		__u32              out_len;              /*     8     4 */
		__u32              in2_len;              /*     8     4 */
	};                                               /*     8     4 */
	__u32                      __spare[7];           /*    12    28 */

	/* size: 40, cachelines: 1, members: 4 */
	/* last cacheline: 40 bytes */
};
struct request_key_auth {
	struct callback_head       rcu __attribute__((__aligned__(8))); /*     0    16 */
	struct key *               target_key;           /*    16     8 */
	struct key *               dest_keyring;         /*    24     8 */
	const struct cred  *       cred;                 /*    32     8 */
	void *                     callout_info;         /*    40     8 */
	size_t                     callout_len;          /*    48     8 */
	pid_t                      pid;                  /*    56     4 */
	char                       op[8];                /*    60     8 */

	/* size: 72, cachelines: 2, members: 8 */
	/* padding: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 8 bytes */
} __attribute__((__aligned__(8)));
struct user_key_payload {
	struct callback_head       rcu __attribute__((__aligned__(8))); /*     0    16 */
	short unsigned int         datalen;              /*    16     2 */

	/* XXX 6 bytes hole, try to pack */

	char                       data[] __attribute__((__aligned__(8))); /*    24     0 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 18, holes: 1, sum holes: 6 */
	/* forced alignments: 2, forced holes: 1, sum forced holes: 6 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct hwrng {
	const char  *              name;                 /*     0     8 */
	int                        (*init)(struct hwrng *); /*     8     8 */
	void                       (*cleanup)(struct hwrng *); /*    16     8 */
	int                        (*data_present)(struct hwrng *, int); /*    24     8 */
	int                        (*data_read)(struct hwrng *, u32 *); /*    32     8 */
	int                        (*read)(struct hwrng *, void *, size_t, bool); /*    40     8 */
	long unsigned int          priv;                 /*    48     8 */
	short unsigned int         quality;              /*    56     2 */

	/* XXX 6 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	struct list_head           list;                 /*    64    16 */
	struct kref                ref;                  /*    80     4 */

	/* XXX 4 bytes hole, try to pack */

	struct completion          cleanup_done;         /*    88    32 */

	/* size: 120, cachelines: 2, members: 11 */
	/* sum members: 110, holes: 2, sum holes: 10 */
	/* last cacheline: 56 bytes */
};
struct tpm_digest {
	u16                        alg_id;               /*     0     2 */
	u8                         digest[64];           /*     2    64 */

	/* size: 66, cachelines: 2, members: 2 */
	/* last cacheline: 2 bytes */
};
struct tpm_bank_info {
	u16                        alg_id;               /*     0     2 */
	u16                        digest_size;          /*     2     2 */
	u16                        crypto_id;            /*     4     2 */

	/* size: 6, cachelines: 1, members: 3 */
	/* last cacheline: 6 bytes */
};
struct tpm_class_ops {
	unsigned int               flags;                /*     0     4 */
	const u8                   req_complete_mask;    /*     4     1 */
	const u8                   req_complete_val;     /*     5     1 */

	/* XXX 2 bytes hole, try to pack */

	bool                       (*req_canceled)(struct tpm_chip *, u8); /*     8     8 */
	int                        (*recv)(struct tpm_chip *, u8 *, size_t); /*    16     8 */
	int                        (*send)(struct tpm_chip *, u8 *, size_t); /*    24     8 */
	void                       (*cancel)(struct tpm_chip *); /*    32     8 */
	u8                         (*status)(struct tpm_chip *); /*    40     8 */
	void                       (*update_timeouts)(struct tpm_chip *, long unsigned int *); /*    48     8 */
	void                       (*update_durations)(struct tpm_chip *, long unsigned int *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*go_idle)(struct tpm_chip *); /*    64     8 */
	int                        (*cmd_ready)(struct tpm_chip *); /*    72     8 */
	int                        (*request_locality)(struct tpm_chip *, int); /*    80     8 */
	int                        (*relinquish_locality)(struct tpm_chip *, int); /*    88     8 */
	void                       (*clk_enable)(struct tpm_chip *, bool); /*    96     8 */

	/* size: 104, cachelines: 2, members: 15 */
	/* sum members: 102, holes: 1, sum holes: 2 */
	/* last cacheline: 40 bytes */
};
struct tpm_chip {
	struct device              dev __attribute__((__aligned__(8))); /*     0  1000 */
	/* --- cacheline 15 boundary (960 bytes) was 40 bytes ago --- */
	struct device              devs __attribute__((__aligned__(8))); /*  1000  1000 */
	/* --- cacheline 31 boundary (1984 bytes) was 16 bytes ago --- */
	struct cdev                cdev;                 /*  2000   136 */
	/* --- cacheline 33 boundary (2112 bytes) was 24 bytes ago --- */
	struct cdev                cdevs;                /*  2136   136 */
	/* --- cacheline 35 boundary (2240 bytes) was 32 bytes ago --- */
	struct rw_semaphore        ops_sem;              /*  2272    40 */
	/* --- cacheline 36 boundary (2304 bytes) was 8 bytes ago --- */
	const struct tpm_class_ops  * ops;               /*  2312     8 */
	struct tpm_bios_log        log;                  /*  2320    16 */
	struct tpm_chip_seqops     bin_log_seqops;       /*  2336    16 */
	struct tpm_chip_seqops     ascii_log_seqops;     /*  2352    16 */
	/* --- cacheline 37 boundary (2368 bytes) --- */
	unsigned int               flags;                /*  2368     4 */
	int                        dev_num;              /*  2372     4 */
	long unsigned int          is_open;              /*  2376     8 */
	char                       hwrng_name[64];       /*  2384    64 */
	/* --- cacheline 38 boundary (2432 bytes) was 16 bytes ago --- */
	struct hwrng               hwrng;                /*  2448   120 */
	/* --- cacheline 40 boundary (2560 bytes) was 8 bytes ago --- */
	struct mutex               tpm_mutex;            /*  2568    32 */
	long unsigned int          timeout_a;            /*  2600     8 */
	long unsigned int          timeout_b;            /*  2608     8 */
	long unsigned int          timeout_c;            /*  2616     8 */
	/* --- cacheline 41 boundary (2624 bytes) --- */
	long unsigned int          timeout_d;            /*  2624     8 */
	bool                       timeout_adjusted;     /*  2632     1 */

	/* XXX 7 bytes hole, try to pack */

	long unsigned int          duration[4];          /*  2640    32 */
	bool                       duration_adjusted;    /*  2672     1 */

	/* XXX 7 bytes hole, try to pack */

	struct dentry *            bios_dir[3];          /*  2680    24 */
	/* --- cacheline 42 boundary (2688 bytes) was 16 bytes ago --- */
	const struct attribute_group  * groups[3];       /*  2704    24 */
	unsigned int               groups_cnt;           /*  2728     4 */
	u32                        nr_allocated_banks;   /*  2732     4 */
	struct tpm_bank_info *     allocated_banks;      /*  2736     8 */
	acpi_handle                acpi_dev_handle;      /*  2744     8 */
	/* --- cacheline 43 boundary (2752 bytes) --- */
	char                       ppi_version[4];       /*  2752     4 */

	/* XXX 4 bytes hole, try to pack */

	struct tpm_space           work_space;           /*  2760    56 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 44 boundary (2816 bytes) --- */
	u32                        last_cc;              /*  2816     4 */
	u32                        nr_commands;          /*  2820     4 */
	u32 *                      cc_attrs_tbl;         /*  2824     8 */
	int                        locality;             /*  2832     4 */

	/* size: 2840, cachelines: 45, members: 34 */
	/* sum members: 2818, holes: 3, sum holes: 18 */
	/* padding: 4 */
	/* paddings: 1, sum paddings: 4 */
	/* forced alignments: 2 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct tpm_space {
	u32                        context_tbl[3];       /*     0    12 */

	/* XXX 4 bytes hole, try to pack */

	u8 *                       context_buf;          /*    16     8 */
	u32                        session_tbl[3];       /*    24    12 */

	/* XXX 4 bytes hole, try to pack */

	u8 *                       session_buf;          /*    40     8 */
	u32                        buf_size;             /*    48     4 */

	/* size: 56, cachelines: 1, members: 5 */
	/* sum members: 44, holes: 2, sum holes: 8 */
	/* padding: 4 */
	/* last cacheline: 56 bytes */
};
struct tpm_bios_log {
	void *                     bios_event_log;       /*     0     8 */
	void *                     bios_event_log_end;   /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct tpm_chip_seqops {
	struct tpm_chip *          chip;                 /*     0     8 */
	const struct seq_operations  * seqops;           /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct tpm_header {
	__be16                     tag;                  /*     0     2 */
	__be32                     length;               /*     2     4 */
	union {
		__be32             ordinal;              /*     6     4 */
		__be32             return_code;          /*     6     4 */
	};                                               /*     6     4 */

	/* size: 10, cachelines: 1, members: 3 */
	/* last cacheline: 10 bytes */
} __attribute__((__packed__));
struct tpm_buf {
	unsigned int               flags;                /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	u8 *                       data;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct trusted_key_payload {
	struct callback_head       rcu __attribute__((__aligned__(8))); /*     0    16 */
	unsigned int               key_len;              /*    16     4 */
	unsigned int               blob_len;             /*    20     4 */
	unsigned char              migratable;           /*    24     1 */
	unsigned char              key[129];             /*    25   129 */
	/* --- cacheline 2 boundary (128 bytes) was 26 bytes ago --- */
	unsigned char              blob[512];            /*   154   512 */

	/* size: 672, cachelines: 11, members: 6 */
	/* padding: 6 */
	/* forced alignments: 1 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct trusted_key_options {
	uint16_t                   keytype;              /*     0     2 */

	/* XXX 2 bytes hole, try to pack */

	uint32_t                   keyhandle;            /*     4     4 */
	unsigned char              keyauth[20];          /*     8    20 */
	uint32_t                   blobauth_len;         /*    28     4 */
	unsigned char              blobauth[20];         /*    32    20 */
	uint32_t                   pcrinfo_len;          /*    52     4 */
	unsigned char              pcrinfo[64];          /*    56    64 */
	/* --- cacheline 1 boundary (64 bytes) was 56 bytes ago --- */
	int                        pcrlock;              /*   120     4 */
	uint32_t                   hash;                 /*   124     4 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	uint32_t                   policydigest_len;     /*   128     4 */
	unsigned char              policydigest[64];     /*   132    64 */
	/* --- cacheline 3 boundary (192 bytes) was 4 bytes ago --- */
	uint32_t                   policyhandle;         /*   196     4 */

	/* size: 200, cachelines: 4, members: 12 */
	/* sum members: 198, holes: 1, sum holes: 2 */
	/* last cacheline: 8 bytes */
};
struct osapsess {
	uint32_t                   handle;               /*     0     4 */
	unsigned char              secret[20];           /*     4    20 */
	unsigned char              enonce[20];           /*    24    20 */

	/* size: 44, cachelines: 1, members: 3 */
	/* last cacheline: 44 bytes */
};
struct sdesc {
	struct shash_desc          shash __attribute__((__aligned__(8))); /*     0     8 */
	char                       ctx[];                /*     8     0 */

	/* size: 8, cachelines: 1, members: 2 */
	/* forced alignments: 1 */
	/* last cacheline: 8 bytes */
} __attribute__((__aligned__(8)));
struct tpm_digests {
	unsigned char              encauth[20];          /*     0    20 */
	unsigned char              pubauth[20];          /*    20    20 */
	unsigned char              xorwork[40];          /*    40    40 */
	/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
	unsigned char              xorhash[20];          /*    80    20 */
	unsigned char              nonceodd[20];         /*   100    20 */

	/* size: 120, cachelines: 2, members: 5 */
	/* last cacheline: 56 bytes */
};
struct tpm2_hash {
	unsigned int               crypto_id;            /*     0     4 */
	unsigned int               tpm_id;               /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct encrypted_key_payload {
	struct callback_head       rcu __attribute__((__aligned__(8))); /*     0    16 */
	char *                     format;               /*    16     8 */
	char *                     master_desc;          /*    24     8 */
	char *                     datalen;              /*    32     8 */
	u8 *                       iv;                   /*    40     8 */
	u8 *                       encrypted_data;       /*    48     8 */
	short unsigned int         datablob_len;         /*    56     2 */
	short unsigned int         decrypted_datalen;    /*    58     2 */
	short unsigned int         payload_datalen;      /*    60     2 */
	short unsigned int         encrypted_key_format; /*    62     2 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u8 *                       decrypted_data;       /*    64     8 */
	u8                         payload_data[];       /*    72     0 */

	/* size: 72, cachelines: 2, members: 12 */
	/* forced alignments: 1 */
	/* last cacheline: 8 bytes */
} __attribute__((__aligned__(8)));
struct ecryptfs_session_key {
	u32                        flags;                /*     0     4 */
	u32                        encrypted_key_size;   /*     4     4 */
	u32                        decrypted_key_size;   /*     8     4 */
	u8                         encrypted_key[512];   /*    12   512 */
	/* --- cacheline 8 boundary (512 bytes) was 12 bytes ago --- */
	u8                         decrypted_key[64];    /*   524    64 */

	/* size: 588, cachelines: 10, members: 5 */
	/* last cacheline: 12 bytes */
};
struct ecryptfs_password {
	u32                        password_bytes;       /*     0     4 */
	s32                        hash_algo;            /*     4     4 */
	u32                        hash_iterations;      /*     8     4 */
	u32                        session_key_encryption_key_bytes; /*    12     4 */
	u32                        flags;                /*    16     4 */
	u8                         session_key_encryption_key[64]; /*    20    64 */
	/* --- cacheline 1 boundary (64 bytes) was 20 bytes ago --- */
	u8                         signature[17];        /*    84    17 */
	u8                         salt[8];              /*   101     8 */

	/* size: 112, cachelines: 2, members: 8 */
	/* padding: 3 */
	/* last cacheline: 48 bytes */
};
struct ecryptfs_private_key {
	u32                        key_size;             /*     0     4 */
	u32                        data_len;             /*     4     4 */
	u8                         signature[17];        /*     8    17 */
	char                       pki_type[17];         /*    25    17 */
	u8                         data[];               /*    42     0 */

	/* size: 44, cachelines: 1, members: 5 */
	/* padding: 2 */
	/* last cacheline: 44 bytes */
};
struct ecryptfs_auth_tok {
	u16                        version;              /*     0     2 */
	u16                        token_type;           /*     2     2 */
	u32                        flags;                /*     4     4 */
	struct ecryptfs_session_key session_key;         /*     8   588 */
	/* --- cacheline 9 boundary (576 bytes) was 20 bytes ago --- */
	u8                         reserved[32];         /*   596    32 */
	union {
		struct ecryptfs_password password;       /*   628   112 */
		struct ecryptfs_private_key private_key; /*   628    44 */
	} token;                                         /*   628   112 */

	/* size: 740, cachelines: 12, members: 6 */
	/* last cacheline: 36 bytes */
};
struct vfs_cap_data {
	__le32                     magic_etc;            /*     0     4 */
	struct {
		__le32             permitted;            /*     4     4 */
		__le32             inheritable;          /*     8     4 */
	} data[2]; /*     4    16 */

	/* size: 20, cachelines: 1, members: 2 */
	/* last cacheline: 20 bytes */
};
struct vfs_ns_cap_data {
	__le32                     magic_etc;            /*     0     4 */
	struct {
		__le32             permitted;            /*     4     4 */
		__le32             inheritable;          /*     8     4 */
	} data[2]; /*     4    16 */
	__le32                     rootid;               /*    20     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
union security_list_options {
	int                        (*binder_set_context_mgr)(const struct cred  *); /*     0     8 */
	int                        (*binder_transaction)(const struct cred  *, const struct cred  *); /*     0     8 */
	int                        (*binder_transfer_binder)(const struct cred  *, const struct cred  *); /*     0     8 */
	int                        (*binder_transfer_file)(const struct cred  *, const struct cred  *, struct file *); /*     0     8 */
	int                        (*ptrace_access_check)(struct task_struct *, unsigned int); /*     0     8 */
	int                        (*ptrace_traceme)(struct task_struct *); /*     0     8 */
	int                        (*capget)(struct task_struct *, kernel_cap_t *, kernel_cap_t *, kernel_cap_t *); /*     0     8 */
	int                        (*capset)(struct cred *, const struct cred  *, const kernel_cap_t  *, const kernel_cap_t  *, const kernel_cap_t  *); /*     0     8 */
	int                        (*capable)(const struct cred  *, struct user_namespace *, int, unsigned int); /*     0     8 */
	int                        (*quotactl)(int, int, int, struct super_block *); /*     0     8 */
	int                        (*quota_on)(struct dentry *); /*     0     8 */
	int                        (*syslog)(int);     /*     0     8 */
	int                        (*settime)(const struct timespec64  *, const struct timezone  *); /*     0     8 */
	int                        (*vm_enough_memory)(struct mm_struct *, long int); /*     0     8 */
	int                        (*bprm_creds_for_exec)(struct linux_binprm *); /*     0     8 */
	int                        (*bprm_creds_from_file)(struct linux_binprm *, struct file *); /*     0     8 */
	int                        (*bprm_check_security)(struct linux_binprm *); /*     0     8 */
	void                       (*bprm_committing_creds)(struct linux_binprm *); /*     0     8 */
	void                       (*bprm_committed_creds)(struct linux_binprm *); /*     0     8 */
	int                        (*fs_context_dup)(struct fs_context *, struct fs_context *); /*     0     8 */
	int                        (*fs_context_parse_param)(struct fs_context *, struct fs_parameter *); /*     0     8 */
	int                        (*sb_alloc_security)(struct super_block *); /*     0     8 */
	void                       (*sb_free_security)(struct super_block *); /*     0     8 */
	void                       (*sb_free_mnt_opts)(void *); /*     0     8 */
	int                        (*sb_eat_lsm_opts)(char *, void * *); /*     0     8 */
	int                        (*sb_remount)(struct super_block *, void *); /*     0     8 */
	int                        (*sb_kern_mount)(struct super_block *); /*     0     8 */
	int                        (*sb_show_options)(struct seq_file *, struct super_block *); /*     0     8 */
	int                        (*sb_statfs)(struct dentry *); /*     0     8 */
	int                        (*sb_mount)(const char  *, const struct path  *, const char  *, long unsigned int, void *); /*     0     8 */
	int                        (*sb_umount)(struct vfsmount *, int); /*     0     8 */
	int                        (*sb_pivotroot)(const struct path  *, const struct path  *); /*     0     8 */
	int                        (*sb_set_mnt_opts)(struct super_block *, void *, long unsigned int, long unsigned int *); /*     0     8 */
	int                        (*sb_clone_mnt_opts)(const struct super_block  *, struct super_block *, long unsigned int, long unsigned int *); /*     0     8 */
	int                        (*sb_add_mnt_opt)(const char  *, const char  *, int, void * *); /*     0     8 */
	int                        (*move_mount)(const struct path  *, const struct path  *); /*     0     8 */
	int                        (*dentry_init_security)(struct dentry *, int, const struct qstr  *, void * *, u32 *); /*     0     8 */
	int                        (*dentry_create_files_as)(struct dentry *, int, struct qstr *, const struct cred  *, struct cred *); /*     0     8 */
	int                        (*path_unlink)(const struct path  *, struct dentry *); /*     0     8 */
	int                        (*path_mkdir)(const struct path  *, struct dentry *, umode_t); /*     0     8 */
	int                        (*path_rmdir)(const struct path  *, struct dentry *); /*     0     8 */
	int                        (*path_mknod)(const struct path  *, struct dentry *, umode_t, unsigned int); /*     0     8 */
	int                        (*path_truncate)(const struct path  *); /*     0     8 */
	int                        (*path_symlink)(const struct path  *, struct dentry *, const char  *); /*     0     8 */
	int                        (*path_link)(struct dentry *, const struct path  *, struct dentry *); /*     0     8 */
	int                        (*path_rename)(const struct path  *, struct dentry *, const struct path  *, struct dentry *); /*     0     8 */
	int                        (*path_chmod)(const struct path  *, umode_t); /*     0     8 */
	int                        (*path_chown)(const struct path  *, kuid_t, kgid_t); /*     0     8 */
	int                        (*path_chroot)(const struct path  *); /*     0     8 */
	int                        (*path_notify)(const struct path  *, u64, unsigned int); /*     0     8 */
	int                        (*inode_alloc_security)(struct inode *); /*     0     8 */
	void                       (*inode_free_security)(struct inode *); /*     0     8 */
	int                        (*inode_init_security)(struct inode *, struct inode *, const struct qstr  *, const char  * *, void * *, size_t *); /*     0     8 */
	int                        (*inode_create)(struct inode *, struct dentry *, umode_t); /*     0     8 */
	int                        (*inode_link)(struct dentry *, struct inode *, struct dentry *); /*     0     8 */
	int                        (*inode_unlink)(struct inode *, struct dentry *); /*     0     8 */
	int                        (*inode_symlink)(struct inode *, struct dentry *, const char  *); /*     0     8 */
	int                        (*inode_mkdir)(struct inode *, struct dentry *, umode_t); /*     0     8 */
	int                        (*inode_rmdir)(struct inode *, struct dentry *); /*     0     8 */
	int                        (*inode_mknod)(struct inode *, struct dentry *, umode_t, dev_t); /*     0     8 */
	int                        (*inode_rename)(struct inode *, struct dentry *, struct inode *, struct dentry *); /*     0     8 */
	int                        (*inode_readlink)(struct dentry *); /*     0     8 */
	int                        (*inode_follow_link)(struct dentry *, struct inode *, bool); /*     0     8 */
	int                        (*inode_permission)(struct inode *, int); /*     0     8 */
	int                        (*inode_setattr)(struct dentry *, struct iattr *); /*     0     8 */
	int                        (*inode_getattr)(const struct path  *); /*     0     8 */
	int                        (*inode_setxattr)(struct dentry *, const char  *, const void  *, size_t, int); /*     0     8 */
	void                       (*inode_post_setxattr)(struct dentry *, const char  *, const void  *, size_t, int); /*     0     8 */
	int                        (*inode_getxattr)(struct dentry *, const char  *); /*     0     8 */
	int                        (*inode_listxattr)(struct dentry *); /*     0     8 */
	int                        (*inode_removexattr)(struct dentry *, const char  *); /*     0     8 */
	int                        (*inode_need_killpriv)(struct dentry *); /*     0     8 */
	int                        (*inode_killpriv)(struct dentry *); /*     0     8 */
	int                        (*inode_getsecurity)(struct inode *, const char  *, void * *, bool); /*     0     8 */
	int                        (*inode_setsecurity)(struct inode *, const char  *, const void  *, size_t, int); /*     0     8 */
	int                        (*inode_listsecurity)(struct inode *, char *, size_t); /*     0     8 */
	void                       (*inode_getsecid)(struct inode *, u32 *); /*     0     8 */
	int                        (*inode_copy_up)(struct dentry *, struct cred * *); /*     0     8 */
	int                        (*inode_copy_up_xattr)(const char  *); /*     0     8 */
	int                        (*kernfs_init_security)(struct kernfs_node *, struct kernfs_node *); /*     0     8 */
	int                        (*file_permission)(struct file *, int); /*     0     8 */
	int                        (*file_alloc_security)(struct file *); /*     0     8 */
	void                       (*file_free_security)(struct file *); /*     0     8 */
	int                        (*file_ioctl)(struct file *, unsigned int, long unsigned int); /*     0     8 */
	int                        (*mmap_addr)(long unsigned int); /*     0     8 */
	int                        (*mmap_file)(struct file *, long unsigned int, long unsigned int, long unsigned int); /*     0     8 */
	int                        (*file_mprotect)(struct vm_area_struct *, long unsigned int, long unsigned int); /*     0     8 */
	int                        (*file_lock)(struct file *, unsigned int); /*     0     8 */
	int                        (*file_fcntl)(struct file *, unsigned int, long unsigned int); /*     0     8 */
	void                       (*file_set_fowner)(struct file *); /*     0     8 */
	int                        (*file_send_sigiotask)(struct task_struct *, struct fown_struct *, int); /*     0     8 */
	int                        (*file_receive)(struct file *); /*     0     8 */
	int                        (*file_open)(struct file *); /*     0     8 */
	int                        (*task_alloc)(struct task_struct *, long unsigned int); /*     0     8 */
	void                       (*task_free)(struct task_struct *); /*     0     8 */
	int                        (*cred_alloc_blank)(struct cred *, gfp_t); /*     0     8 */
	void                       (*cred_free)(struct cred *); /*     0     8 */
	int                        (*cred_prepare)(struct cred *, const struct cred  *, gfp_t); /*     0     8 */
	void                       (*cred_transfer)(struct cred *, const struct cred  *); /*     0     8 */
	void                       (*cred_getsecid)(const struct cred  *, u32 *); /*     0     8 */
	int                        (*kernel_act_as)(struct cred *, u32); /*     0     8 */
	int                        (*kernel_create_files_as)(struct cred *, struct inode *); /*     0     8 */
	int                        (*kernel_module_request)(char *); /*     0     8 */
	int                        (*kernel_load_data)(enum kernel_load_data_id, bool); /*     0     8 */
	int                        (*kernel_post_load_data)(char *, loff_t, enum kernel_load_data_id, char *); /*     0     8 */
	int                        (*kernel_read_file)(struct file *, enum kernel_read_file_id, bool); /*     0     8 */
	int                        (*kernel_post_read_file)(struct file *, char *, loff_t, enum kernel_read_file_id); /*     0     8 */
	int                        (*task_fix_setuid)(struct cred *, const struct cred  *, int); /*     0     8 */
	int                        (*task_fix_setgid)(struct cred *, const struct cred  *, int); /*     0     8 */
	int                        (*task_setpgid)(struct task_struct *, pid_t); /*     0     8 */
	int                        (*task_getpgid)(struct task_struct *); /*     0     8 */
	int                        (*task_getsid)(struct task_struct *); /*     0     8 */
	void                       (*task_getsecid)(struct task_struct *, u32 *); /*     0     8 */
	int                        (*task_setnice)(struct task_struct *, int); /*     0     8 */
	int                        (*task_setioprio)(struct task_struct *, int); /*     0     8 */
	int                        (*task_getioprio)(struct task_struct *); /*     0     8 */
	int                        (*task_prlimit)(const struct cred  *, const struct cred  *, unsigned int); /*     0     8 */
	int                        (*task_setrlimit)(struct task_struct *, unsigned int, struct rlimit *); /*     0     8 */
	int                        (*task_setscheduler)(struct task_struct *); /*     0     8 */
	int                        (*task_getscheduler)(struct task_struct *); /*     0     8 */
	int                        (*task_movememory)(struct task_struct *); /*     0     8 */
	int                        (*task_kill)(struct task_struct *, struct kernel_siginfo *, int, const struct cred  *); /*     0     8 */
	int                        (*task_prctl)(int, long unsigned int, long unsigned int, long unsigned int, long unsigned int); /*     0     8 */
	void                       (*task_to_inode)(struct task_struct *, struct inode *); /*     0     8 */
	int                        (*ipc_permission)(struct kern_ipc_perm *, short int); /*     0     8 */
	void                       (*ipc_getsecid)(struct kern_ipc_perm *, u32 *); /*     0     8 */
	int                        (*msg_msg_alloc_security)(struct msg_msg *); /*     0     8 */
	void                       (*msg_msg_free_security)(struct msg_msg *); /*     0     8 */
	int                        (*msg_queue_alloc_security)(struct kern_ipc_perm *); /*     0     8 */
	void                       (*msg_queue_free_security)(struct kern_ipc_perm *); /*     0     8 */
	int                        (*msg_queue_associate)(struct kern_ipc_perm *, int); /*     0     8 */
	int                        (*msg_queue_msgctl)(struct kern_ipc_perm *, int); /*     0     8 */
	int                        (*msg_queue_msgsnd)(struct kern_ipc_perm *, struct msg_msg *, int); /*     0     8 */
	int                        (*msg_queue_msgrcv)(struct kern_ipc_perm *, struct msg_msg *, struct task_struct *, long int, int); /*     0     8 */
	int                        (*shm_alloc_security)(struct kern_ipc_perm *); /*     0     8 */
	void                       (*shm_free_security)(struct kern_ipc_perm *); /*     0     8 */
	int                        (*shm_associate)(struct kern_ipc_perm *, int); /*     0     8 */
	int                        (*shm_shmctl)(struct kern_ipc_perm *, int); /*     0     8 */
	int                        (*shm_shmat)(struct kern_ipc_perm *, char *, int); /*     0     8 */
	int                        (*sem_alloc_security)(struct kern_ipc_perm *); /*     0     8 */
	void                       (*sem_free_security)(struct kern_ipc_perm *); /*     0     8 */
	int                        (*sem_associate)(struct kern_ipc_perm *, int); /*     0     8 */
	int                        (*sem_semctl)(struct kern_ipc_perm *, int); /*     0     8 */
	int                        (*sem_semop)(struct kern_ipc_perm *, struct sembuf *, unsigned int, int); /*     0     8 */
	int                        (*netlink_send)(struct sock *, struct sk_buff *); /*     0     8 */
	void                       (*d_instantiate)(struct dentry *, struct inode *); /*     0     8 */
	int                        (*getprocattr)(struct task_struct *, char *, char * *); /*     0     8 */
	int                        (*setprocattr)(const char  *, void *, size_t); /*     0     8 */
	int                        (*ismaclabel)(const char  *); /*     0     8 */
	int                        (*secid_to_secctx)(u32, char * *, u32 *); /*     0     8 */
	int                        (*secctx_to_secid)(const char  *, u32, u32 *); /*     0     8 */
	void                       (*release_secctx)(char *, u32); /*     0     8 */
	void                       (*inode_invalidate_secctx)(struct inode *); /*     0     8 */
	int                        (*inode_notifysecctx)(struct inode *, void *, u32); /*     0     8 */
	int                        (*inode_setsecctx)(struct dentry *, void *, u32); /*     0     8 */
	int                        (*inode_getsecctx)(struct inode *, void * *, u32 *); /*     0     8 */
	int                        (*unix_stream_connect)(struct sock *, struct sock *, struct sock *); /*     0     8 */
	int                        (*unix_may_send)(struct socket *, struct socket *); /*     0     8 */
	int                        (*socket_create)(int, int, int, int); /*     0     8 */
	int                        (*socket_post_create)(struct socket *, int, int, int, int); /*     0     8 */
	int                        (*socket_socketpair)(struct socket *, struct socket *); /*     0     8 */
	int                        (*socket_bind)(struct socket *, struct sockaddr *, int); /*     0     8 */
	int                        (*socket_connect)(struct socket *, struct sockaddr *, int); /*     0     8 */
	int                        (*socket_listen)(struct socket *, int); /*     0     8 */
	int                        (*socket_accept)(struct socket *, struct socket *); /*     0     8 */
	int                        (*socket_sendmsg)(struct socket *, struct msghdr *, int); /*     0     8 */
	int                        (*socket_recvmsg)(struct socket *, struct msghdr *, int, int); /*     0     8 */
	int                        (*socket_getsockname)(struct socket *); /*     0     8 */
	int                        (*socket_getpeername)(struct socket *); /*     0     8 */
	int                        (*socket_getsockopt)(struct socket *, int, int); /*     0     8 */
	int                        (*socket_setsockopt)(struct socket *, int, int); /*     0     8 */
	int                        (*socket_shutdown)(struct socket *, int); /*     0     8 */
	int                        (*socket_sock_rcv_skb)(struct sock *, struct sk_buff *); /*     0     8 */
	int                        (*socket_getpeersec_stream)(struct socket *, char *, int *, unsigned int); /*     0     8 */
	int                        (*socket_getpeersec_dgram)(struct socket *, struct sk_buff *, u32 *); /*     0     8 */
	int                        (*sk_alloc_security)(struct sock *, int, gfp_t); /*     0     8 */
	void                       (*sk_free_security)(struct sock *); /*     0     8 */
	void                       (*sk_clone_security)(const struct sock  *, struct sock *); /*     0     8 */
	void                       (*sk_getsecid)(struct sock *, u32 *); /*     0     8 */
	void                       (*sock_graft)(struct sock *, struct socket *); /*     0     8 */
	int                        (*inet_conn_request)(struct sock *, struct sk_buff *, struct request_sock *); /*     0     8 */
	void                       (*inet_csk_clone)(struct sock *, const struct request_sock  *); /*     0     8 */
	void                       (*inet_conn_established)(struct sock *, struct sk_buff *); /*     0     8 */
	int                        (*secmark_relabel_packet)(u32); /*     0     8 */
	void                       (*secmark_refcount_inc)(void); /*     0     8 */
	void                       (*secmark_refcount_dec)(void); /*     0     8 */
	void                       (*req_classify_flow)(const struct request_sock  *, struct flowi_common *); /*     0     8 */
	int                        (*tun_dev_alloc_security)(void * *); /*     0     8 */
	void                       (*tun_dev_free_security)(void *); /*     0     8 */
	int                        (*tun_dev_create)(void); /*     0     8 */
	int                        (*tun_dev_attach_queue)(void *); /*     0     8 */
	int                        (*tun_dev_attach)(struct sock *, void *); /*     0     8 */
	int                        (*tun_dev_open)(void *); /*     0     8 */
	int                        (*sctp_assoc_request)(struct sctp_endpoint *, struct sk_buff *); /*     0     8 */
	int                        (*sctp_bind_connect)(struct sock *, int, struct sockaddr *, int); /*     0     8 */
	void                       (*sctp_sk_clone)(struct sctp_endpoint *, struct sock *, struct sock *); /*     0     8 */
	int                        (*ib_pkey_access)(void *, u64, u16); /*     0     8 */
	int                        (*ib_endport_manage_subnet)(void *, const char  *, u8); /*     0     8 */
	int                        (*ib_alloc_security)(void * *); /*     0     8 */
	void                       (*ib_free_security)(void *); /*     0     8 */
	int                        (*xfrm_policy_alloc_security)(struct xfrm_sec_ctx * *, struct xfrm_user_sec_ctx *, gfp_t); /*     0     8 */
	int                        (*xfrm_policy_clone_security)(struct xfrm_sec_ctx *, struct xfrm_sec_ctx * *); /*     0     8 */
	void                       (*xfrm_policy_free_security)(struct xfrm_sec_ctx *); /*     0     8 */
	int                        (*xfrm_policy_delete_security)(struct xfrm_sec_ctx *); /*     0     8 */
	int                        (*xfrm_state_alloc)(struct xfrm_state *, struct xfrm_user_sec_ctx *); /*     0     8 */
	int                        (*xfrm_state_alloc_acquire)(struct xfrm_state *, struct xfrm_sec_ctx *, u32); /*     0     8 */
	void                       (*xfrm_state_free_security)(struct xfrm_state *); /*     0     8 */
	int                        (*xfrm_state_delete_security)(struct xfrm_state *); /*     0     8 */
	int                        (*xfrm_policy_lookup)(struct xfrm_sec_ctx *, u32, u8); /*     0     8 */
	int                        (*xfrm_state_pol_flow_match)(struct xfrm_state *, struct xfrm_policy *, const struct flowi_common  *); /*     0     8 */
	int                        (*xfrm_decode_session)(struct sk_buff *, u32 *, int); /*     0     8 */
	int                        (*key_alloc)(struct key *, const struct cred  *, long unsigned int); /*     0     8 */
	void                       (*key_free)(struct key *); /*     0     8 */
	int                        (*key_permission)(key_ref_t, const struct cred  *, enum key_need_perm); /*     0     8 */
	int                        (*key_getsecurity)(struct key *, char * *); /*     0     8 */
	int                        (*audit_rule_init)(u32, u32, char *, void * *); /*     0     8 */
	int                        (*audit_rule_known)(struct audit_krule *); /*     0     8 */
	int                        (*audit_rule_match)(u32, u32, u32, void *); /*     0     8 */
	void                       (*audit_rule_free)(void *); /*     0     8 */
	int                        (*bpf)(int, union bpf_attr *, unsigned int); /*     0     8 */
	int                        (*bpf_map)(struct bpf_map *, fmode_t); /*     0     8 */
	int                        (*bpf_prog)(struct bpf_prog *); /*     0     8 */
	int                        (*bpf_map_alloc_security)(struct bpf_map *); /*     0     8 */
	void                       (*bpf_map_free_security)(struct bpf_map *); /*     0     8 */
	int                        (*bpf_prog_alloc_security)(struct bpf_prog_aux *); /*     0     8 */
	void                       (*bpf_prog_free_security)(struct bpf_prog_aux *); /*     0     8 */
	int                        (*locked_down)(enum lockdown_reason); /*     0     8 */
	int                        (*perf_event_open)(struct perf_event_attr *, int); /*     0     8 */
	int                        (*perf_event_alloc)(struct perf_event *); /*     0     8 */
	void                       (*perf_event_free)(struct perf_event *); /*     0     8 */
	int                        (*perf_event_read)(struct perf_event *); /*     0     8 */
	int                        (*perf_event_write)(struct perf_event *); /*     0     8 */
};
struct security_hook_heads {
	struct hlist_head          binder_set_context_mgr; /*     0     8 */
	struct hlist_head          binder_transaction;   /*     8     8 */
	struct hlist_head          binder_transfer_binder; /*    16     8 */
	struct hlist_head          binder_transfer_file; /*    24     8 */
	struct hlist_head          ptrace_access_check;  /*    32     8 */
	struct hlist_head          ptrace_traceme;       /*    40     8 */
	struct hlist_head          capget;               /*    48     8 */
	struct hlist_head          capset;               /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct hlist_head          capable;              /*    64     8 */
	struct hlist_head          quotactl;             /*    72     8 */
	struct hlist_head          quota_on;             /*    80     8 */
	struct hlist_head          syslog;               /*    88     8 */
	struct hlist_head          settime;              /*    96     8 */
	struct hlist_head          vm_enough_memory;     /*   104     8 */
	struct hlist_head          bprm_creds_for_exec;  /*   112     8 */
	struct hlist_head          bprm_creds_from_file; /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct hlist_head          bprm_check_security;  /*   128     8 */
	struct hlist_head          bprm_committing_creds; /*   136     8 */
	struct hlist_head          bprm_committed_creds; /*   144     8 */
	struct hlist_head          fs_context_dup;       /*   152     8 */
	struct hlist_head          fs_context_parse_param; /*   160     8 */
	struct hlist_head          sb_alloc_security;    /*   168     8 */
	struct hlist_head          sb_free_security;     /*   176     8 */
	struct hlist_head          sb_free_mnt_opts;     /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	struct hlist_head          sb_eat_lsm_opts;      /*   192     8 */
	struct hlist_head          sb_remount;           /*   200     8 */
	struct hlist_head          sb_kern_mount;        /*   208     8 */
	struct hlist_head          sb_show_options;      /*   216     8 */
	struct hlist_head          sb_statfs;            /*   224     8 */
	struct hlist_head          sb_mount;             /*   232     8 */
	struct hlist_head          sb_umount;            /*   240     8 */
	struct hlist_head          sb_pivotroot;         /*   248     8 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	struct hlist_head          sb_set_mnt_opts;      /*   256     8 */
	struct hlist_head          sb_clone_mnt_opts;    /*   264     8 */
	struct hlist_head          sb_add_mnt_opt;       /*   272     8 */
	struct hlist_head          move_mount;           /*   280     8 */
	struct hlist_head          dentry_init_security; /*   288     8 */
	struct hlist_head          dentry_create_files_as; /*   296     8 */
	struct hlist_head          path_unlink;          /*   304     8 */
	struct hlist_head          path_mkdir;           /*   312     8 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	struct hlist_head          path_rmdir;           /*   320     8 */
	struct hlist_head          path_mknod;           /*   328     8 */
	struct hlist_head          path_truncate;        /*   336     8 */
	struct hlist_head          path_symlink;         /*   344     8 */
	struct hlist_head          path_link;            /*   352     8 */
	struct hlist_head          path_rename;          /*   360     8 */
	struct hlist_head          path_chmod;           /*   368     8 */
	struct hlist_head          path_chown;           /*   376     8 */
	/* --- cacheline 6 boundary (384 bytes) --- */
	struct hlist_head          path_chroot;          /*   384     8 */
	struct hlist_head          path_notify;          /*   392     8 */
	struct hlist_head          inode_alloc_security; /*   400     8 */
	struct hlist_head          inode_free_security;  /*   408     8 */
	struct hlist_head          inode_init_security;  /*   416     8 */
	struct hlist_head          inode_create;         /*   424     8 */
	struct hlist_head          inode_link;           /*   432     8 */
	struct hlist_head          inode_unlink;         /*   440     8 */
	/* --- cacheline 7 boundary (448 bytes) --- */
	struct hlist_head          inode_symlink;        /*   448     8 */
	struct hlist_head          inode_mkdir;          /*   456     8 */
	struct hlist_head          inode_rmdir;          /*   464     8 */
	struct hlist_head          inode_mknod;          /*   472     8 */
	struct hlist_head          inode_rename;         /*   480     8 */
	struct hlist_head          inode_readlink;       /*   488     8 */
	struct hlist_head          inode_follow_link;    /*   496     8 */
	struct hlist_head          inode_permission;     /*   504     8 */
	/* --- cacheline 8 boundary (512 bytes) --- */
	struct hlist_head          inode_setattr;        /*   512     8 */
	struct hlist_head          inode_getattr;        /*   520     8 */
	struct hlist_head          inode_setxattr;       /*   528     8 */
	struct hlist_head          inode_post_setxattr;  /*   536     8 */
	struct hlist_head          inode_getxattr;       /*   544     8 */
	struct hlist_head          inode_listxattr;      /*   552     8 */
	struct hlist_head          inode_removexattr;    /*   560     8 */
	struct hlist_head          inode_need_killpriv;  /*   568     8 */
	/* --- cacheline 9 boundary (576 bytes) --- */
	struct hlist_head          inode_killpriv;       /*   576     8 */
	struct hlist_head          inode_getsecurity;    /*   584     8 */
	struct hlist_head          inode_setsecurity;    /*   592     8 */
	struct hlist_head          inode_listsecurity;   /*   600     8 */
	struct hlist_head          inode_getsecid;       /*   608     8 */
	struct hlist_head          inode_copy_up;        /*   616     8 */
	struct hlist_head          inode_copy_up_xattr;  /*   624     8 */
	struct hlist_head          kernfs_init_security; /*   632     8 */
	/* --- cacheline 10 boundary (640 bytes) --- */
	struct hlist_head          file_permission;      /*   640     8 */
	struct hlist_head          file_alloc_security;  /*   648     8 */
	struct hlist_head          file_free_security;   /*   656     8 */
	struct hlist_head          file_ioctl;           /*   664     8 */
	struct hlist_head          mmap_addr;            /*   672     8 */
	struct hlist_head          mmap_file;            /*   680     8 */
	struct hlist_head          file_mprotect;        /*   688     8 */
	struct hlist_head          file_lock;            /*   696     8 */
	/* --- cacheline 11 boundary (704 bytes) --- */
	struct hlist_head          file_fcntl;           /*   704     8 */
	struct hlist_head          file_set_fowner;      /*   712     8 */
	struct hlist_head          file_send_sigiotask;  /*   720     8 */
	struct hlist_head          file_receive;         /*   728     8 */
	struct hlist_head          file_open;            /*   736     8 */
	struct hlist_head          task_alloc;           /*   744     8 */
	struct hlist_head          task_free;            /*   752     8 */
	struct hlist_head          cred_alloc_blank;     /*   760     8 */
	/* --- cacheline 12 boundary (768 bytes) --- */
	struct hlist_head          cred_free;            /*   768     8 */
	struct hlist_head          cred_prepare;         /*   776     8 */
	struct hlist_head          cred_transfer;        /*   784     8 */
	struct hlist_head          cred_getsecid;        /*   792     8 */
	struct hlist_head          kernel_act_as;        /*   800     8 */
	struct hlist_head          kernel_create_files_as; /*   808     8 */
	struct hlist_head          kernel_module_request; /*   816     8 */
	struct hlist_head          kernel_load_data;     /*   824     8 */
	/* --- cacheline 13 boundary (832 bytes) --- */
	struct hlist_head          kernel_post_load_data; /*   832     8 */
	struct hlist_head          kernel_read_file;     /*   840     8 */
	struct hlist_head          kernel_post_read_file; /*   848     8 */
	struct hlist_head          task_fix_setuid;      /*   856     8 */
	struct hlist_head          task_fix_setgid;      /*   864     8 */
	struct hlist_head          task_setpgid;         /*   872     8 */
	struct hlist_head          task_getpgid;         /*   880     8 */
	struct hlist_head          task_getsid;          /*   888     8 */
	/* --- cacheline 14 boundary (896 bytes) --- */
	struct hlist_head          task_getsecid;        /*   896     8 */
	struct hlist_head          task_setnice;         /*   904     8 */
	struct hlist_head          task_setioprio;       /*   912     8 */
	struct hlist_head          task_getioprio;       /*   920     8 */
	struct hlist_head          task_prlimit;         /*   928     8 */
	struct hlist_head          task_setrlimit;       /*   936     8 */
	struct hlist_head          task_setscheduler;    /*   944     8 */
	struct hlist_head          task_getscheduler;    /*   952     8 */
	/* --- cacheline 15 boundary (960 bytes) --- */
	struct hlist_head          task_movememory;      /*   960     8 */
	struct hlist_head          task_kill;            /*   968     8 */
	struct hlist_head          task_prctl;           /*   976     8 */
	struct hlist_head          task_to_inode;        /*   984     8 */
	struct hlist_head          ipc_permission;       /*   992     8 */
	struct hlist_head          ipc_getsecid;         /*  1000     8 */
	struct hlist_head          msg_msg_alloc_security; /*  1008     8 */
	struct hlist_head          msg_msg_free_security; /*  1016     8 */
	/* --- cacheline 16 boundary (1024 bytes) --- */
	struct hlist_head          msg_queue_alloc_security; /*  1024     8 */
	struct hlist_head          msg_queue_free_security; /*  1032     8 */
	struct hlist_head          msg_queue_associate;  /*  1040     8 */
	struct hlist_head          msg_queue_msgctl;     /*  1048     8 */
	struct hlist_head          msg_queue_msgsnd;     /*  1056     8 */
	struct hlist_head          msg_queue_msgrcv;     /*  1064     8 */
	struct hlist_head          shm_alloc_security;   /*  1072     8 */
	struct hlist_head          shm_free_security;    /*  1080     8 */
	/* --- cacheline 17 boundary (1088 bytes) --- */
	struct hlist_head          shm_associate;        /*  1088     8 */
	struct hlist_head          shm_shmctl;           /*  1096     8 */
	struct hlist_head          shm_shmat;            /*  1104     8 */
	struct hlist_head          sem_alloc_security;   /*  1112     8 */
	struct hlist_head          sem_free_security;    /*  1120     8 */
	struct hlist_head          sem_associate;        /*  1128     8 */
	struct hlist_head          sem_semctl;           /*  1136     8 */
	struct hlist_head          sem_semop;            /*  1144     8 */
	/* --- cacheline 18 boundary (1152 bytes) --- */
	struct hlist_head          netlink_send;         /*  1152     8 */
	struct hlist_head          d_instantiate;        /*  1160     8 */
	struct hlist_head          getprocattr;          /*  1168     8 */
	struct hlist_head          setprocattr;          /*  1176     8 */
	struct hlist_head          ismaclabel;           /*  1184     8 */
	struct hlist_head          secid_to_secctx;      /*  1192     8 */
	struct hlist_head          secctx_to_secid;      /*  1200     8 */
	struct hlist_head          release_secctx;       /*  1208     8 */
	/* --- cacheline 19 boundary (1216 bytes) --- */
	struct hlist_head          inode_invalidate_secctx; /*  1216     8 */
	struct hlist_head          inode_notifysecctx;   /*  1224     8 */
	struct hlist_head          inode_setsecctx;      /*  1232     8 */
	struct hlist_head          inode_getsecctx;      /*  1240     8 */
	struct hlist_head          unix_stream_connect;  /*  1248     8 */
	struct hlist_head          unix_may_send;        /*  1256     8 */
	struct hlist_head          socket_create;        /*  1264     8 */
	struct hlist_head          socket_post_create;   /*  1272     8 */
	/* --- cacheline 20 boundary (1280 bytes) --- */
	struct hlist_head          socket_socketpair;    /*  1280     8 */
	struct hlist_head          socket_bind;          /*  1288     8 */
	struct hlist_head          socket_connect;       /*  1296     8 */
	struct hlist_head          socket_listen;        /*  1304     8 */
	struct hlist_head          socket_accept;        /*  1312     8 */
	struct hlist_head          socket_sendmsg;       /*  1320     8 */
	struct hlist_head          socket_recvmsg;       /*  1328     8 */
	struct hlist_head          socket_getsockname;   /*  1336     8 */
	/* --- cacheline 21 boundary (1344 bytes) --- */
	struct hlist_head          socket_getpeername;   /*  1344     8 */
	struct hlist_head          socket_getsockopt;    /*  1352     8 */
	struct hlist_head          socket_setsockopt;    /*  1360     8 */
	struct hlist_head          socket_shutdown;      /*  1368     8 */
	struct hlist_head          socket_sock_rcv_skb;  /*  1376     8 */
	struct hlist_head          socket_getpeersec_stream; /*  1384     8 */
	struct hlist_head          socket_getpeersec_dgram; /*  1392     8 */
	struct hlist_head          sk_alloc_security;    /*  1400     8 */
	/* --- cacheline 22 boundary (1408 bytes) --- */
	struct hlist_head          sk_free_security;     /*  1408     8 */
	struct hlist_head          sk_clone_security;    /*  1416     8 */
	struct hlist_head          sk_getsecid;          /*  1424     8 */
	struct hlist_head          sock_graft;           /*  1432     8 */
	struct hlist_head          inet_conn_request;    /*  1440     8 */
	struct hlist_head          inet_csk_clone;       /*  1448     8 */
	struct hlist_head          inet_conn_established; /*  1456     8 */
	struct hlist_head          secmark_relabel_packet; /*  1464     8 */
	/* --- cacheline 23 boundary (1472 bytes) --- */
	struct hlist_head          secmark_refcount_inc; /*  1472     8 */
	struct hlist_head          secmark_refcount_dec; /*  1480     8 */
	struct hlist_head          req_classify_flow;    /*  1488     8 */
	struct hlist_head          tun_dev_alloc_security; /*  1496     8 */
	struct hlist_head          tun_dev_free_security; /*  1504     8 */
	struct hlist_head          tun_dev_create;       /*  1512     8 */
	struct hlist_head          tun_dev_attach_queue; /*  1520     8 */
	struct hlist_head          tun_dev_attach;       /*  1528     8 */
	/* --- cacheline 24 boundary (1536 bytes) --- */
	struct hlist_head          tun_dev_open;         /*  1536     8 */
	struct hlist_head          sctp_assoc_request;   /*  1544     8 */
	struct hlist_head          sctp_bind_connect;    /*  1552     8 */
	struct hlist_head          sctp_sk_clone;        /*  1560     8 */
	struct hlist_head          ib_pkey_access;       /*  1568     8 */
	struct hlist_head          ib_endport_manage_subnet; /*  1576     8 */
	struct hlist_head          ib_alloc_security;    /*  1584     8 */
	struct hlist_head          ib_free_security;     /*  1592     8 */
	/* --- cacheline 25 boundary (1600 bytes) --- */
	struct hlist_head          xfrm_policy_alloc_security; /*  1600     8 */
	struct hlist_head          xfrm_policy_clone_security; /*  1608     8 */
	struct hlist_head          xfrm_policy_free_security; /*  1616     8 */
	struct hlist_head          xfrm_policy_delete_security; /*  1624     8 */
	struct hlist_head          xfrm_state_alloc;     /*  1632     8 */
	struct hlist_head          xfrm_state_alloc_acquire; /*  1640     8 */
	struct hlist_head          xfrm_state_free_security; /*  1648     8 */
	struct hlist_head          xfrm_state_delete_security; /*  1656     8 */
	/* --- cacheline 26 boundary (1664 bytes) --- */
	struct hlist_head          xfrm_policy_lookup;   /*  1664     8 */
	struct hlist_head          xfrm_state_pol_flow_match; /*  1672     8 */
	struct hlist_head          xfrm_decode_session;  /*  1680     8 */
	struct hlist_head          key_alloc;            /*  1688     8 */
	struct hlist_head          key_free;             /*  1696     8 */
	struct hlist_head          key_permission;       /*  1704     8 */
	struct hlist_head          key_getsecurity;      /*  1712     8 */
	struct hlist_head          audit_rule_init;      /*  1720     8 */
	/* --- cacheline 27 boundary (1728 bytes) --- */
	struct hlist_head          audit_rule_known;     /*  1728     8 */
	struct hlist_head          audit_rule_match;     /*  1736     8 */
	struct hlist_head          audit_rule_free;      /*  1744     8 */
	struct hlist_head          bpf;                  /*  1752     8 */
	struct hlist_head          bpf_map;              /*  1760     8 */
	struct hlist_head          bpf_prog;             /*  1768     8 */
	struct hlist_head          bpf_map_alloc_security; /*  1776     8 */
	struct hlist_head          bpf_map_free_security; /*  1784     8 */
	/* --- cacheline 28 boundary (1792 bytes) --- */
	struct hlist_head          bpf_prog_alloc_security; /*  1792     8 */
	struct hlist_head          bpf_prog_free_security; /*  1800     8 */
	struct hlist_head          locked_down;          /*  1808     8 */
	struct hlist_head          perf_event_open;      /*  1816     8 */
	struct hlist_head          perf_event_alloc;     /*  1824     8 */
	struct hlist_head          perf_event_free;      /*  1832     8 */
	struct hlist_head          perf_event_read;      /*  1840     8 */
	struct hlist_head          perf_event_write;     /*  1848     8 */

	/* size: 1856, cachelines: 29, members: 232 */
};
struct security_hook_list {
	struct hlist_node          list;                 /*     0    16 */
	struct hlist_head *        head;                 /*    16     8 */
	union security_list_options hook;                /*    24     8 */
	char *                     lsm;                  /*    32     8 */

	/* size: 40, cachelines: 1, members: 4 */
	/* last cacheline: 40 bytes */
};
struct lsm_blob_sizes {
	int                        lbs_cred;             /*     0     4 */
	int                        lbs_file;             /*     4     4 */
	int                        lbs_inode;            /*     8     4 */
	int                        lbs_ipc;              /*    12     4 */
	int                        lbs_msg_msg;          /*    16     4 */
	int                        lbs_task;             /*    20     4 */
	u64                        kabi_reserved1;       /*    24     8 */

	/* size: 32, cachelines: 1, members: 7 */
	/* last cacheline: 32 bytes */
};
struct lsm_info {
	const char  *              name;                 /*     0     8 */
	enum lsm_order             order;                /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          flags;                /*    16     8 */
	int *                      enabled;              /*    24     8 */
	int                        (*init)(void);        /*    32     8 */
	struct lsm_blob_sizes *    blobs;                /*    40     8 */

	/* size: 48, cachelines: 1, members: 6 */
	/* sum members: 44, holes: 1, sum holes: 4 */
	/* last cacheline: 48 bytes */
};
struct lsm_network_audit {
	int                        netif;                /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct sock *              sk;                   /*     8     8 */
	u16                        family;               /*    16     2 */
	__be16                     dport;                /*    18     2 */
	__be16                     sport;                /*    20     2 */

	/* XXX 2 bytes hole, try to pack */

	union {
		struct {
			__be32     daddr;                /*    24     4 */
			__be32     saddr;                /*    28     4 */
		} v4;                                    /*    24     8 */
		struct {
			struct in6_addr daddr;           /*    24    16 */
			struct in6_addr saddr;           /*    40    16 */
		} v6;                                    /*    24    32 */
	} fam;                                           /*    24    32 */

	/* size: 56, cachelines: 1, members: 6 */
	/* sum members: 50, holes: 2, sum holes: 6 */
	/* last cacheline: 56 bytes */
};
struct lsm_ioctlop_audit {
	struct path                path;                 /*     0    16 */
	u16                        cmd;                  /*    16     2 */

	/* size: 24, cachelines: 1, members: 2 */
	/* padding: 6 */
	/* last cacheline: 24 bytes */
};
struct lsm_ibpkey_audit {
	u64                        subnet_prefix;        /*     0     8 */
	u16                        pkey;                 /*     8     2 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 6 */
	/* last cacheline: 16 bytes */
};
struct lsm_ibendport_audit {
	char                       dev_name[64];         /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u8                         port;                 /*    64     1 */

	/* size: 65, cachelines: 2, members: 2 */
	/* last cacheline: 1 bytes */
};
struct selinux_audit_data {
	u32                        ssid;                 /*     0     4 */
	u32                        tsid;                 /*     4     4 */
	u16                        tclass;               /*     8     2 */

	/* XXX 2 bytes hole, try to pack */

	u32                        requested;            /*    12     4 */
	u32                        audited;              /*    16     4 */
	u32                        denied;               /*    20     4 */
	int                        result;               /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	struct selinux_state *     state;                /*    32     8 */

	/* size: 40, cachelines: 1, members: 8 */
	/* sum members: 34, holes: 2, sum holes: 6 */
	/* last cacheline: 40 bytes */
};
struct common_audit_data {
	char                       type;                 /*     0     1 */

	/* XXX 7 bytes hole, try to pack */

	union {
		struct path        path;                 /*     8    16 */
		struct dentry *    dentry;               /*     8     8 */
		struct inode *     inode;                /*     8     8 */
		struct lsm_network_audit * net;          /*     8     8 */
		int                cap;                  /*     8     4 */
		int                ipc_id;               /*     8     4 */
		struct task_struct * tsk;                /*     8     8 */
		struct {
			key_serial_t key;                /*     8     4 */

			/* XXX 4 bytes hole, try to pack */

			char *     key_desc;             /*    16     8 */
		} key_struct;                            /*     8    16 */
		char *             kmod_name;            /*     8     8 */
		struct lsm_ioctlop_audit * op;           /*     8     8 */
		struct file *      file;                 /*     8     8 */
		struct lsm_ibpkey_audit * ibpkey;        /*     8     8 */
		struct lsm_ibendport_audit * ibendport;  /*     8     8 */
		int                reason;               /*     8     4 */
	} u;                                             /*     8    16 */
	union {
		struct selinux_audit_data * selinux_audit_data; /*    24     8 */
		struct apparmor_audit_data * apparmor_audit_data; /*    24     8 */
	};                                               /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* sum members: 25, holes: 1, sum holes: 7 */
	/* last cacheline: 32 bytes */
};
struct selinux_state {
	bool                       disabled;             /*     0     1 */
	bool                       enforcing;            /*     1     1 */
	bool                       checkreqprot;         /*     2     1 */
	bool                       initialized;          /*     3     1 */
	bool                       policycap[8];         /*     4     8 */

	/* XXX 4 bytes hole, try to pack */

	struct page *              status_page;          /*    16     8 */
	struct mutex               status_lock;          /*    24    32 */
	struct selinux_avc *       avc;                  /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct selinux_policy *    policy;               /*    64     8 */
	struct mutex               policy_mutex;         /*    72    32 */

	/* size: 104, cachelines: 2, members: 10 */
	/* sum members: 100, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct selinux_avc {
	unsigned int               avc_cache_threshold;  /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct avc_cache           avc_cache;            /*     8  6160 */

	/* XXX last struct has 4 bytes of padding */

	/* size: 6168, cachelines: 97, members: 2 */
	/* sum members: 6164, holes: 1, sum holes: 4 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 24 bytes */
};
struct av_decision {
	u32                        allowed;              /*     0     4 */
	u32                        auditallow;           /*     4     4 */
	u32                        auditdeny;            /*     8     4 */
	u32                        seqno;                /*    12     4 */
	u32                        flags;                /*    16     4 */

	/* size: 20, cachelines: 1, members: 5 */
	/* last cacheline: 20 bytes */
};
struct extended_perms_data {
	u32                        p[8];                 /*     0    32 */

	/* size: 32, cachelines: 1, members: 1 */
	/* last cacheline: 32 bytes */
};
struct extended_perms_decision {
	u8                         used;                 /*     0     1 */
	u8                         driver;               /*     1     1 */

	/* XXX 6 bytes hole, try to pack */

	struct extended_perms_data * allowed;            /*     8     8 */
	struct extended_perms_data * auditallow;         /*    16     8 */
	struct extended_perms_data * dontaudit;          /*    24     8 */

	/* size: 32, cachelines: 1, members: 5 */
	/* sum members: 26, holes: 1, sum holes: 6 */
	/* last cacheline: 32 bytes */
};
struct extended_perms {
	u16                        len;                  /*     0     2 */

	/* XXX 2 bytes hole, try to pack */

	struct extended_perms_data drivers;              /*     4    32 */

	/* size: 36, cachelines: 1, members: 2 */
	/* sum members: 34, holes: 1, sum holes: 2 */
	/* last cacheline: 36 bytes */
};
struct avc_cache_stats {
	unsigned int               lookups;              /*     0     4 */
	unsigned int               misses;               /*     4     4 */
	unsigned int               allocations;          /*     8     4 */
	unsigned int               reclaims;             /*    12     4 */
	unsigned int               frees;                /*    16     4 */

	/* size: 20, cachelines: 1, members: 5 */
	/* last cacheline: 20 bytes */
};
struct security_class_mapping {
	const char  *              name;                 /*     0     8 */
	const char  *              perms[33];            /*     8   264 */

	/* size: 272, cachelines: 5, members: 2 */
	/* last cacheline: 16 bytes */
};
struct trace_event_raw_selinux_audited {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        requested;            /*     8     4 */
	u32                        denied;               /*    12     4 */
	u32                        audited;              /*    16     4 */
	int                        result;               /*    20     4 */
	u32                        __data_loc_scontext;  /*    24     4 */
	u32                        __data_loc_tcontext;  /*    28     4 */
	u32                        __data_loc_tclass;    /*    32     4 */
	char                       __data[];             /*    36     0 */

	/* size: 36, cachelines: 1, members: 9 */
	/* last cacheline: 36 bytes */
};
struct trace_event_data_offsets_selinux_audited {
	u32                        scontext;             /*     0     4 */
	u32                        tcontext;             /*     4     4 */
	u32                        tclass;               /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct avc_entry {
	u32                        ssid;                 /*     0     4 */
	u32                        tsid;                 /*     4     4 */
	u16                        tclass;               /*     8     2 */

	/* XXX 2 bytes hole, try to pack */

	struct av_decision         avd;                  /*    12    20 */
	struct avc_xperms_node *   xp_node;              /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* sum members: 38, holes: 1, sum holes: 2 */
	/* last cacheline: 40 bytes */
};
struct avc_xperms_node {
	struct extended_perms      xp;                   /*     0    36 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           xpd_head;             /*    40    16 */

	/* size: 56, cachelines: 1, members: 2 */
	/* sum members: 52, holes: 1, sum holes: 4 */
	/* last cacheline: 56 bytes */
};
struct avc_node {
	struct avc_entry           ae;                   /*     0    40 */
	struct hlist_node          list;                 /*    40    16 */
	struct callback_head       rhead __attribute__((__aligned__(8))); /*    56    16 */

	/* size: 72, cachelines: 2, members: 3 */
	/* forced alignments: 1 */
	/* last cacheline: 8 bytes */
} __attribute__((__aligned__(8)));
struct avc_xperms_decision_node {
	struct extended_perms_decision xpd;              /*     0    32 */
	struct list_head           xpd_list;             /*    32    16 */

	/* size: 48, cachelines: 1, members: 2 */
	/* last cacheline: 48 bytes */
};
struct avc_cache {
	struct hlist_head          slots[512];           /*     0  4096 */
	/* --- cacheline 64 boundary (4096 bytes) --- */
	spinlock_t                 slots_lock[512];      /*  4096  2048 */
	/* --- cacheline 96 boundary (6144 bytes) --- */
	atomic_t                   lru_hint;             /*  6144     4 */
	atomic_t                   active_nodes;         /*  6148     4 */
	u32                        latest_notif;         /*  6152     4 */

	/* size: 6160, cachelines: 97, members: 5 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct avc_callback_node {
	int                        (*callback)(u32);     /*     0     8 */
	u32                        events;               /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct avc_callback_node * next;                 /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct sctp_endpoint {
	struct sctp_ep_common      base;                 /*     0   160 */
	/* --- cacheline 2 boundary (128 bytes) was 32 bytes ago --- */
	struct list_head           asocs;                /*   160    16 */
	__u8                       secret_key[32];       /*   176    32 */
	/* --- cacheline 3 boundary (192 bytes) was 16 bytes ago --- */
	__u8 *                     digest;               /*   208     8 */
	__u32                      sndbuf_policy;        /*   216     4 */
	__u32                      rcvbuf_policy;        /*   220     4 */
	struct crypto_shash * *    auth_hmacs;           /*   224     8 */
	struct sctp_hmac_algo_param * auth_hmacs_list;   /*   232     8 */
	struct sctp_chunks_param * auth_chunk_list;      /*   240     8 */
	struct list_head           endpoint_shared_keys; /*   248    16 */
	/* --- cacheline 4 boundary (256 bytes) was 8 bytes ago --- */
	__u16                      active_key_id;        /*   264     2 */
	__u8                       ecn_enable:1;         /*   266: 0  1 */
	__u8                       auth_enable:1;        /*   266: 1  1 */
	__u8                       intl_enable:1;        /*   266: 2  1 */
	__u8                       prsctp_enable:1;      /*   266: 3  1 */
	__u8                       asconf_enable:1;      /*   266: 4  1 */
	__u8                       reconf_enable:1;      /*   266: 5  1 */

	/* XXX 2 bits hole, try to pack */

	__u8                       strreset_enable;      /*   267     1 */
	u32                        secid;                /*   268     4 */
	u32                        peer_secid;           /*   272     4 */

	/* XXX 4 bytes hole, try to pack */

	struct callback_head       rcu __attribute__((__aligned__(8))); /*   280    16 */

	/* size: 296, cachelines: 5, members: 21 */
	/* sum members: 291, holes: 1, sum holes: 4 */
	/* sum bitfield members: 6 bits, bit holes: 1, sum bit holes: 2 bits */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 4 */
	/* last cacheline: 40 bytes */
} __attribute__((__aligned__(8)));
struct xfrm_sec_ctx {
	__u8                       ctx_doi;              /*     0     1 */
	__u8                       ctx_alg;              /*     1     1 */
	__u16                      ctx_len;              /*     2     2 */
	__u32                      ctx_sid;              /*     4     4 */
	char                       ctx_str[];            /*     8     0 */

	/* size: 8, cachelines: 1, members: 5 */
	/* last cacheline: 8 bytes */
};
struct xfrm_user_sec_ctx {
	__u16                      len;                  /*     0     2 */
	__u16                      exttype;              /*     2     2 */
	__u8                       ctx_alg;              /*     4     1 */
	__u8                       ctx_doi;              /*     5     1 */
	__u16                      ctx_len;              /*     6     2 */

	/* size: 8, cachelines: 1, members: 5 */
	/* last cacheline: 8 bytes */
};
struct sockaddr_in6 {
	short unsigned int         sin6_family;          /*     0     2 */
	__be16                     sin6_port;            /*     2     2 */
	__be32                     sin6_flowinfo;        /*     4     4 */
	struct in6_addr            sin6_addr;            /*     8    16 */
	__u32                      sin6_scope_id;        /*    24     4 */

	/* size: 28, cachelines: 1, members: 5 */
	/* last cacheline: 28 bytes */
};
struct in_addr {
	__be32                     s_addr;               /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct sockaddr_in {
	__kernel_sa_family_t       sin_family;           /*     0     2 */
	__be16                     sin_port;             /*     2     2 */
	struct in_addr             sin_addr;             /*     4     4 */
	unsigned char              __pad[8];             /*     8     8 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct nf_hook_entries {
	u16                        num_hook_entries;     /*     0     2 */

	/* XXX 6 bytes hole, try to pack */

	struct nf_hook_entry       hooks[];              /*     8     0 */

	/* size: 8, cachelines: 1, members: 2 */
	/* sum members: 2, holes: 1, sum holes: 6 */
	/* last cacheline: 8 bytes */
};
struct nf_hook_state {
	unsigned int               hook;                 /*     0     4 */
	u_int8_t                   pf;                   /*     4     1 */

	/* XXX 3 bytes hole, try to pack */

	struct net_device *        in;                   /*     8     8 */
	struct net_device *        out;                  /*    16     8 */
	struct sock *              sk;                   /*    24     8 */
	struct net *               net;                  /*    32     8 */
	int                        (*okfn)(struct net *, struct sock *, struct sk_buff *); /*    40     8 */

	/* size: 48, cachelines: 1, members: 7 */
	/* sum members: 45, holes: 1, sum holes: 3 */
	/* last cacheline: 48 bytes */
};
struct nf_hook_ops {
	nf_hookfn *                hook;                 /*     0     8 */
	struct net_device *        dev;                  /*     8     8 */
	void *                     priv;                 /*    16     8 */
	u_int8_t                   pf;                   /*    24     1 */

	/* XXX 3 bytes hole, try to pack */

	unsigned int               hooknum;              /*    28     4 */
	int                        priority;             /*    32     4 */

	/* size: 40, cachelines: 1, members: 6 */
	/* sum members: 33, holes: 1, sum holes: 3 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct nf_hook_entry {
	nf_hookfn *                hook;                 /*     0     8 */
	void *                     priv;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct socket_alloc {
	struct socket              socket __attribute__((__aligned__(64))); /*     0   128 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct inode               vfs_inode __attribute__((__aligned__(8))); /*   128   624 */

	/* size: 768, cachelines: 12, members: 2 */
	/* padding: 16 */
	/* forced alignments: 2 */
} __attribute__((__aligned__(64)));
struct ip_options {
	__be32                     faddr;                /*     0     4 */
	__be32                     nexthop;              /*     4     4 */
	unsigned char              optlen;               /*     8     1 */
	unsigned char              srr;                  /*     9     1 */
	unsigned char              rr;                   /*    10     1 */
	unsigned char              ts;                   /*    11     1 */
	unsigned char              is_strictroute:1;     /*    12: 0  1 */
	unsigned char              srr_is_hit:1;         /*    12: 1  1 */
	unsigned char              is_changed:1;         /*    12: 2  1 */
	unsigned char              rr_needaddr:1;        /*    12: 3  1 */
	unsigned char              ts_needtime:1;        /*    12: 4  1 */
	unsigned char              ts_needaddr:1;        /*    12: 5  1 */

	/* XXX 2 bits hole, try to pack */

	unsigned char              router_alert;         /*    13     1 */
	unsigned char              cipso;                /*    14     1 */
	unsigned char              __pad2;               /*    15     1 */
	unsigned char              __data[];             /*    16     0 */

	/* size: 16, cachelines: 1, members: 16 */
	/* sum members: 15 */
	/* sum bitfield members: 6 bits, bit holes: 1, sum bit holes: 2 bits */
	/* last cacheline: 16 bytes */
};
struct ip_options_rcu {
	struct callback_head       rcu __attribute__((__aligned__(8))); /*     0    16 */
	struct ip_options          opt;                  /*    16    16 */

	/* size: 32, cachelines: 1, members: 2 */
	/* forced alignments: 1 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct ipv6_txoptions {
	refcount_t                 refcnt;               /*     0     4 */
	int                        tot_len;              /*     4     4 */
	__u16                      opt_flen;             /*     8     2 */
	__u16                      opt_nflen;            /*    10     2 */

	/* XXX 4 bytes hole, try to pack */

	struct ipv6_opt_hdr *      hopopt;               /*    16     8 */
	struct ipv6_opt_hdr *      dst0opt;              /*    24     8 */
	struct ipv6_rt_hdr *       srcrt;                /*    32     8 */
	struct ipv6_opt_hdr *      dst1opt;              /*    40     8 */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*    48    16 */

	/* size: 64, cachelines: 1, members: 9 */
	/* sum members: 60, holes: 1, sum holes: 4 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(8)));
struct inet_cork {
	unsigned int               flags;                /*     0     4 */
	__be32                     addr;                 /*     4     4 */
	struct ip_options *        opt;                  /*     8     8 */
	unsigned int               fragsize;             /*    16     4 */
	int                        length;               /*    20     4 */
	struct dst_entry *         dst;                  /*    24     8 */
	u8                         tx_flags;             /*    32     1 */
	__u8                       ttl;                  /*    33     1 */
	__s16                      tos;                  /*    34     2 */
	char                       priority;             /*    36     1 */

	/* XXX 1 byte hole, try to pack */

	__u16                      gso_size;             /*    38     2 */
	u64                        transmit_time;        /*    40     8 */
	u32                        mark;                 /*    48     4 */

	/* size: 56, cachelines: 1, members: 13 */
	/* sum members: 51, holes: 1, sum holes: 1 */
	/* padding: 4 */
	/* last cacheline: 56 bytes */
};
struct inet_cork_full {
	struct inet_cork           base;                 /*     0    56 */

	/* XXX last struct has 4 bytes of padding */

	struct flowi               fl __attribute__((__aligned__(8))); /*    56   104 */

	/* size: 160, cachelines: 3, members: 2 */
	/* paddings: 1, sum paddings: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct inet_sock {
	struct sock                sk __attribute__((__aligned__(8))); /*     0   872 */
	/* --- cacheline 13 boundary (832 bytes) was 40 bytes ago --- */
	struct ipv6_pinfo *        pinet6;               /*   872     8 */
	__be32                     inet_saddr;           /*   880     4 */
	__s16                      uc_ttl;               /*   884     2 */
	__u16                      cmsg_flags;           /*   886     2 */
	__be16                     inet_sport;           /*   888     2 */
	__u16                      inet_id;              /*   890     2 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 14 boundary (896 bytes) --- */
	struct ip_options_rcu *    inet_opt;             /*   896     8 */
	int                        rx_dst_ifindex;       /*   904     4 */
	__u8                       tos;                  /*   908     1 */
	__u8                       min_ttl;              /*   909     1 */
	__u8                       mc_ttl;               /*   910     1 */
	__u8                       pmtudisc;             /*   911     1 */
	__u8                       recverr:1;            /*   912: 0  1 */
	__u8                       is_icsk:1;            /*   912: 1  1 */
	__u8                       freebind:1;           /*   912: 2  1 */
	__u8                       hdrincl:1;            /*   912: 3  1 */
	__u8                       mc_loop:1;            /*   912: 4  1 */
	__u8                       transparent:1;        /*   912: 5  1 */
	__u8                       mc_all:1;             /*   912: 6  1 */
	__u8                       nodefrag:1;           /*   912: 7  1 */
	__u8                       bind_address_no_port:1; /*   913: 0  1 */
	__u8                       recverr_rfc4884:1;    /*   913: 1  1 */
	__u8                       defer_connect:1;      /*   913: 2  1 */

	/* XXX 5 bits hole, try to pack */

	__u8                       rcv_tos;              /*   914     1 */
	__u8                       convert_csum;         /*   915     1 */
	int                        uc_index;             /*   916     4 */
	int                        mc_index;             /*   920     4 */
	__be32                     mc_addr;              /*   924     4 */
	struct ip_mc_socklist *    mc_list;              /*   928     8 */
	struct inet_cork_full      cork __attribute__((__aligned__(8))); /*   936   160 */

	/* size: 1096, cachelines: 18, members: 31 */
	/* sum members: 1090, holes: 1, sum holes: 4 */
	/* sum bitfield members: 11 bits, bit holes: 1, sum bit holes: 5 bits */
	/* forced alignments: 2 */
	/* last cacheline: 8 bytes */
} __attribute__((__aligned__(8)));
struct ipv6_pinfo {
	struct in6_addr            saddr;                /*     0    16 */
	struct in6_pktinfo         sticky_pktinfo;       /*    16    20 */

	/* XXX 4 bytes hole, try to pack */

	const struct in6_addr  *   daddr_cache;          /*    40     8 */
	__be32                     flow_label;           /*    48     4 */
	__u32                      frag_size;            /*    52     4 */
	__u16                      __unused_1:7;         /*    56: 0  2 */
	__s16                      hop_limit:9;          /*    56: 7  2 */
	__u16                      mc_loop:1;            /*    58: 0  2 */
	__u16                      __unused_2:6;         /*    58: 1  2 */
	__s16                      mcast_hops:9;         /*    58: 7  2 */
	int                        ucast_oif;            /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        mcast_oif;            /*    64     4 */
	union {
		struct {
			__u16      srcrt:1;              /*    68: 0  2 */
			__u16      osrcrt:1;             /*    68: 1  2 */
			__u16      rxinfo:1;             /*    68: 2  2 */
			__u16      rxoinfo:1;            /*    68: 3  2 */
			__u16      rxhlim:1;             /*    68: 4  2 */
			__u16      rxohlim:1;            /*    68: 5  2 */
			__u16      hopopts:1;            /*    68: 6  2 */
			__u16      ohopopts:1;           /*    68: 7  2 */
			__u16      dstopts:1;            /*    68: 8  2 */
			__u16      odstopts:1;           /*    68: 9  2 */
			__u16      rxflow:1;             /*    68:10  2 */
			__u16      rxtclass:1;           /*    68:11  2 */
			__u16      rxpmtu:1;             /*    68:12  2 */
			__u16      rxorigdstaddr:1;      /*    68:13  2 */
			__u16      recvfragsize:1;       /*    68:14  2 */
		} bits;                                  /*    68     2 */
		__u16              all;                  /*    68     2 */
	} rxopt;                                         /*    68     2 */
	__u16                      recverr:1;            /*    70: 0  2 */
	__u16                      sndflow:1;            /*    70: 1  2 */
	__u16                      repflow:1;            /*    70: 2  2 */
	__u16                      pmtudisc:3;           /*    70: 3  2 */
	__u16                      padding:1;            /*    70: 6  2 */
	__u16                      srcprefs:3;           /*    70: 7  2 */
	__u16                      dontfrag:1;           /*    70:10  2 */
	__u16                      autoflowlabel:1;      /*    70:11  2 */
	__u16                      autoflowlabel_set:1;  /*    70:12  2 */
	__u16                      mc_all:1;             /*    70:13  2 */
	__u16                      recverr_rfc4884:1;    /*    70:14  2 */
	__u16                      rtalert_isolate:1;    /*    70:15  2 */
	__u8                       min_hopcount;         /*    72     1 */
	__u8                       tclass;               /*    73     1 */

	/* XXX 2 bytes hole, try to pack */

	__be32                     rcv_flowinfo;         /*    76     4 */
	__u32                      dst_cookie;           /*    80     4 */
	__u32                      rx_dst_cookie;        /*    84     4 */
	struct ipv6_mc_socklist *  ipv6_mc_list;         /*    88     8 */
	struct ipv6_ac_socklist *  ipv6_ac_list;         /*    96     8 */
	struct ipv6_fl_socklist *  ipv6_fl_list;         /*   104     8 */
	struct ipv6_txoptions *    opt;                  /*   112     8 */
	struct sk_buff *           pktoptions;           /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct sk_buff *           rxpmtu;               /*   128     8 */
	struct inet6_cork          cork;                 /*   136    16 */

	/* XXX last struct has 6 bytes of padding */

	/* size: 152, cachelines: 3, members: 37 */
	/* sum members: 140, holes: 2, sum holes: 6 */
	/* sum bitfield members: 48 bits (6 bytes) */
	/* paddings: 1, sum paddings: 6 */
	/* last cacheline: 24 bytes */
};
struct tcphdr {
	__be16                     source;               /*     0     2 */
	__be16                     dest;                 /*     2     2 */
	__be32                     seq;                  /*     4     4 */
	__be32                     ack_seq;              /*     8     4 */
	__u16                      res1:4;               /*    12: 0  2 */
	__u16                      doff:4;               /*    12: 4  2 */
	__u16                      fin:1;                /*    12: 8  2 */
	__u16                      syn:1;                /*    12: 9  2 */
	__u16                      rst:1;                /*    12:10  2 */
	__u16                      psh:1;                /*    12:11  2 */
	__u16                      ack:1;                /*    12:12  2 */
	__u16                      urg:1;                /*    12:13  2 */
	__u16                      ece:1;                /*    12:14  2 */
	__u16                      cwr:1;                /*    12:15  2 */
	__be16                     window;               /*    14     2 */
	__sum16                    check;                /*    16     2 */
	__be16                     urg_ptr;              /*    18     2 */

	/* size: 20, cachelines: 1, members: 17 */
	/* last cacheline: 20 bytes */
};
struct iphdr {
	__u8                       ihl:4;                /*     0: 0  1 */
	__u8                       version:4;            /*     0: 4  1 */
	__u8                       tos;                  /*     1     1 */
	__be16                     tot_len;              /*     2     2 */
	__be16                     id;                   /*     4     2 */
	__be16                     frag_off;             /*     6     2 */
	__u8                       ttl;                  /*     8     1 */
	__u8                       protocol;             /*     9     1 */
	__sum16                    check;                /*    10     2 */
	__be32                     saddr;                /*    12     4 */
	__be32                     daddr;                /*    16     4 */

	/* size: 20, cachelines: 1, members: 11 */
	/* last cacheline: 20 bytes */
};
struct in6_pktinfo {
	struct in6_addr            ipi6_addr;            /*     0    16 */
	int                        ipi6_ifindex;         /*    16     4 */

	/* size: 20, cachelines: 1, members: 2 */
	/* last cacheline: 20 bytes */
};
struct ipv6_rt_hdr {
	__u8                       nexthdr;              /*     0     1 */
	__u8                       hdrlen;               /*     1     1 */
	__u8                       type;                 /*     2     1 */
	__u8                       segments_left;        /*     3     1 */

	/* size: 4, cachelines: 1, members: 4 */
	/* last cacheline: 4 bytes */
};
struct ipv6_opt_hdr {
	__u8                       nexthdr;              /*     0     1 */
	__u8                       hdrlen;               /*     1     1 */

	/* size: 2, cachelines: 1, members: 2 */
	/* last cacheline: 2 bytes */
};
struct ipv6hdr {
	__u8                       priority:4;           /*     0: 0  1 */
	__u8                       version:4;            /*     0: 4  1 */
	__u8                       flow_lbl[3];          /*     1     3 */
	__be16                     payload_len;          /*     4     2 */
	__u8                       nexthdr;              /*     6     1 */
	__u8                       hop_limit;            /*     7     1 */
	struct in6_addr            saddr;                /*     8    16 */
	struct in6_addr            daddr;                /*    24    16 */

	/* size: 40, cachelines: 1, members: 8 */
	/* last cacheline: 40 bytes */
};
struct udphdr {
	__be16                     source;               /*     0     2 */
	__be16                     dest;                 /*     2     2 */
	__be16                     len;                  /*     4     2 */
	__sum16                    check;                /*     6     2 */

	/* size: 8, cachelines: 1, members: 4 */
	/* last cacheline: 8 bytes */
};
struct inet6_skb_parm {
	int                        iif;                  /*     0     4 */
	__be16                     ra;                   /*     4     2 */
	__u16                      dst0;                 /*     6     2 */
	__u16                      srcrt;                /*     8     2 */
	__u16                      dst1;                 /*    10     2 */
	__u16                      lastopt;              /*    12     2 */
	__u16                      nhoff;                /*    14     2 */
	__u16                      flags;                /*    16     2 */
	__u16                      dsthao;               /*    18     2 */
	__u16                      frag_max_size;        /*    20     2 */

	/* size: 24, cachelines: 1, members: 10 */
	/* padding: 2 */
	/* last cacheline: 24 bytes */
};
struct inet6_cork {
	struct ipv6_txoptions *    opt;                  /*     0     8 */
	u8                         hop_limit;            /*     8     1 */
	u8                         tclass;               /*     9     1 */

	/* size: 16, cachelines: 1, members: 3 */
	/* padding: 6 */
	/* last cacheline: 16 bytes */
};
struct ipv6_mc_socklist {
	struct in6_addr            addr;                 /*     0    16 */
	int                        ifindex;              /*    16     4 */
	unsigned int               sfmode;               /*    20     4 */
	struct ipv6_mc_socklist *  next;                 /*    24     8 */
	rwlock_t                   sflock;               /*    32     8 */
	struct ip6_sf_socklist *   sflist;               /*    40     8 */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*    48    16 */

	/* size: 64, cachelines: 1, members: 7 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(8)));
struct ipv6_ac_socklist {
	struct in6_addr            acl_addr;             /*     0    16 */
	int                        acl_ifindex;          /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	struct ipv6_ac_socklist *  acl_next;             /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct ipv6_fl_socklist {
	struct ipv6_fl_socklist *  next;                 /*     0     8 */
	struct ip6_flowlabel *     fl;                   /*     8     8 */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*    16    16 */

	/* size: 32, cachelines: 1, members: 3 */
	/* forced alignments: 1 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct ip6_sf_socklist {
	unsigned int               sl_max;               /*     0     4 */
	unsigned int               sl_count;             /*     4     4 */
	struct in6_addr            sl_addr[];            /*     8     0 */

	/* size: 8, cachelines: 1, members: 3 */
	/* last cacheline: 8 bytes */
};
struct prefix_info {
	__u8                       type;                 /*     0     1 */
	__u8                       length;               /*     1     1 */
	__u8                       prefix_len;           /*     2     1 */
	__u8                       reserved:6;           /*     3: 0  1 */
	__u8                       autoconf:1;           /*     3: 6  1 */
	__u8                       onlink:1;             /*     3: 7  1 */
	__be32                     valid;                /*     4     4 */
	__be32                     prefered;             /*     8     4 */
	__be32                     reserved2;            /*    12     4 */
	struct in6_addr            prefix;               /*    16    16 */

	/* size: 32, cachelines: 1, members: 10 */
	/* last cacheline: 32 bytes */
};
struct ip6_flowlabel {
	struct ip6_flowlabel *     next;                 /*     0     8 */
	__be32                     label;                /*     8     4 */
	atomic_t                   users;                /*    12     4 */
	struct in6_addr            dst;                  /*    16    16 */
	struct ipv6_txoptions *    opt;                  /*    32     8 */
	long unsigned int          linger;               /*    40     8 */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*    48    16 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u8                         share;                /*    64     1 */

	/* XXX 7 bytes hole, try to pack */

	union {
		struct pid *       pid;                  /*    72     8 */
		kuid_t             uid;                  /*    72     4 */
	} owner;                                         /*    72     8 */
	long unsigned int          lastuse;              /*    80     8 */
	long unsigned int          expires;              /*    88     8 */
	struct net *               fl_net;               /*    96     8 */

	/* size: 104, cachelines: 2, members: 12 */
	/* sum members: 97, holes: 1, sum holes: 7 */
	/* forced alignments: 1 */
	/* last cacheline: 40 bytes */
} __attribute__((__aligned__(8)));
struct inet_skb_parm {
	int                        iif;                  /*     0     4 */
	struct ip_options          opt;                  /*     4    16 */
	u16                        flags;                /*    20     2 */
	u16                        frag_max_size;        /*    22     2 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct tty_file_private {
	struct tty_struct *        tty;                  /*     0     8 */
	struct file *              file;                 /*     8     8 */
	struct list_head           list;                 /*    16    16 */

	/* size: 32, cachelines: 1, members: 3 */
	/* last cacheline: 32 bytes */
};
struct netlbl_lsm_cache {
	refcount_t                 refcount;             /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	void                       (*free)(const void  *); /*     8     8 */
	void *                     data;                 /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct netlbl_lsm_catmap {
	u32                        startbit;             /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        bitmap[4];            /*     8    32 */
	struct netlbl_lsm_catmap * next;                 /*    40     8 */

	/* size: 48, cachelines: 1, members: 3 */
	/* sum members: 44, holes: 1, sum holes: 4 */
	/* last cacheline: 48 bytes */
};
struct netlbl_lsm_secattr {
	u32                        flags;                /*     0     4 */
	u32                        type;                 /*     4     4 */
	char *                     domain;               /*     8     8 */
	struct netlbl_lsm_cache *  cache;                /*    16     8 */
	struct {
		struct {
			struct netlbl_lsm_catmap * cat;  /*    24     8 */
			u32        lvl;                  /*    32     4 */
		} mls;                                   /*    24    16 */

		/* XXX last struct has 4 bytes of padding */

		u32                secid;                /*    40     4 */
	} attr;                                          /*    24    24 */

	/* XXX last struct has 4 bytes of padding */

	/* size: 48, cachelines: 1, members: 5 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 48 bytes */
};
struct dccp_hdr {
	__be16                     dccph_sport;          /*     0     2 */
	__be16                     dccph_dport;          /*     2     2 */
	__u8                       dccph_doff;           /*     4     1 */
	__u8                       dccph_cscov:4;        /*     5: 0  1 */
	__u8                       dccph_ccval:4;        /*     5: 4  1 */
	__sum16                    dccph_checksum;       /*     6     2 */
	__u8                       dccph_x:1;            /*     8: 0  1 */
	__u8                       dccph_type:4;         /*     8: 1  1 */
	__u8                       dccph_reserved:3;     /*     8: 5  1 */
	__u8                       dccph_seq2;           /*     9     1 */
	__be16                     dccph_seq;            /*    10     2 */

	/* size: 12, cachelines: 1, members: 11 */
	/* last cacheline: 12 bytes */
};
struct sctp_initmsg {
	__u16                      sinit_num_ostreams;   /*     0     2 */
	__u16                      sinit_max_instreams;  /*     2     2 */
	__u16                      sinit_max_attempts;   /*     4     2 */
	__u16                      sinit_max_init_timeo; /*     6     2 */

	/* size: 8, cachelines: 1, members: 4 */
	/* last cacheline: 8 bytes */
};
struct sctp_sndrcvinfo {
	__u16                      sinfo_stream;         /*     0     2 */
	__u16                      sinfo_ssn;            /*     2     2 */
	__u16                      sinfo_flags;          /*     4     2 */

	/* XXX 2 bytes hole, try to pack */

	__u32                      sinfo_ppid;           /*     8     4 */
	__u32                      sinfo_context;        /*    12     4 */
	__u32                      sinfo_timetolive;     /*    16     4 */
	__u32                      sinfo_tsn;            /*    20     4 */
	__u32                      sinfo_cumtsn;         /*    24     4 */
	sctp_assoc_t               sinfo_assoc_id;       /*    28     4 */

	/* size: 32, cachelines: 1, members: 9 */
	/* sum members: 30, holes: 1, sum holes: 2 */
	/* last cacheline: 32 bytes */
};
struct sctp_rtoinfo {
	sctp_assoc_t               srto_assoc_id;        /*     0     4 */
	__u32                      srto_initial;         /*     4     4 */
	__u32                      srto_max;             /*     8     4 */
	__u32                      srto_min;             /*    12     4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct sctp_assocparams {
	sctp_assoc_t               sasoc_assoc_id;       /*     0     4 */
	__u16                      sasoc_asocmaxrxt;     /*     4     2 */
	__u16                      sasoc_number_peer_destinations; /*     6     2 */
	__u32                      sasoc_peer_rwnd;      /*     8     4 */
	__u32                      sasoc_local_rwnd;     /*    12     4 */
	__u32                      sasoc_cookie_life;    /*    16     4 */

	/* size: 20, cachelines: 1, members: 6 */
	/* last cacheline: 20 bytes */
};
struct sctp_paddrparams {
	sctp_assoc_t               spp_assoc_id;         /*     0     4 */
	struct __kernel_sockaddr_storage spp_address;    /*     4   128 */
	/* --- cacheline 2 boundary (128 bytes) was 4 bytes ago --- */
	__u32                      spp_hbinterval;       /*   132     4 */
	__u16                      spp_pathmaxrxt;       /*   136     2 */
	__u32                      spp_pathmtu;          /*   138     4 */
	__u32                      spp_sackdelay;        /*   142     4 */
	__u32                      spp_flags;            /*   146     4 */
	__u32                      spp_ipv6_flowlabel;   /*   150     4 */
	__u8                       spp_dscp;             /*   154     1 */

	/* size: 156, cachelines: 3, members: 9 */
	/* padding: 1 */
	/* last cacheline: 28 bytes */
} __attribute__((__aligned__(4)));
struct sctphdr {
	__be16                     source;               /*     0     2 */
	__be16                     dest;                 /*     2     2 */
	__be32                     vtag;                 /*     4     4 */
	__le32                     checksum;             /*     8     4 */

	/* size: 12, cachelines: 1, members: 4 */
	/* last cacheline: 12 bytes */
};
struct sctp_chunkhdr {
	__u8                       type;                 /*     0     1 */
	__u8                       flags;                /*     1     1 */
	__be16                     length;               /*     2     2 */

	/* size: 4, cachelines: 1, members: 3 */
	/* last cacheline: 4 bytes */
};
struct sctp_paramhdr {
	__be16                     type;                 /*     0     2 */
	__be16                     length;               /*     2     2 */

	/* size: 4, cachelines: 1, members: 2 */
	/* last cacheline: 4 bytes */
};
struct sctp_datahdr {
	__be32                     tsn;                  /*     0     4 */
	__be16                     stream;               /*     4     2 */
	__be16                     ssn;                  /*     6     2 */
	__u32                      ppid;                 /*     8     4 */
	__u8                       payload[];            /*    12     0 */

	/* size: 12, cachelines: 1, members: 5 */
	/* last cacheline: 12 bytes */
};
struct sctp_idatahdr {
	__be32                     tsn;                  /*     0     4 */
	__be16                     stream;               /*     4     2 */
	__be16                     reserved;             /*     6     2 */
	__be32                     mid;                  /*     8     4 */
	union {
		__u32              ppid;                 /*    12     4 */
		__be32             fsn;                  /*    12     4 */
	};                                               /*    12     4 */
	__u8                       payload[];            /*    16     0 */

	/* size: 16, cachelines: 1, members: 6 */
	/* last cacheline: 16 bytes */
};
struct sctp_inithdr {
	__be32                     init_tag;             /*     0     4 */
	__be32                     a_rwnd;               /*     4     4 */
	__be16                     num_outbound_streams; /*     8     2 */
	__be16                     num_inbound_streams;  /*    10     2 */
	__be32                     initial_tsn;          /*    12     4 */
	__u8                       params[];             /*    16     0 */

	/* size: 16, cachelines: 1, members: 6 */
	/* last cacheline: 16 bytes */
};
struct sctp_init_chunk {
	struct sctp_chunkhdr       chunk_hdr;            /*     0     4 */
	struct sctp_inithdr        init_hdr;             /*     4    16 */

	/* size: 20, cachelines: 1, members: 2 */
	/* last cacheline: 20 bytes */
};
struct sctp_ipv4addr_param {
	struct sctp_paramhdr       param_hdr;            /*     0     4 */
	struct in_addr             addr;                 /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct sctp_ipv6addr_param {
	struct sctp_paramhdr       param_hdr;            /*     0     4 */
	struct in6_addr            addr;                 /*     4    16 */

	/* size: 20, cachelines: 1, members: 2 */
	/* last cacheline: 20 bytes */
};
struct sctp_cookie_preserve_param {
	struct sctp_paramhdr       param_hdr;            /*     0     4 */
	__be32                     lifespan_increment;   /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct sctp_hostname_param {
	struct sctp_paramhdr       param_hdr;            /*     0     4 */
	uint8_t                    hostname[];           /*     4     0 */

	/* size: 4, cachelines: 1, members: 2 */
	/* last cacheline: 4 bytes */
};
struct sctp_supported_addrs_param {
	struct sctp_paramhdr       param_hdr;            /*     0     4 */
	__be16                     types[];              /*     4     0 */

	/* size: 4, cachelines: 1, members: 2 */
	/* last cacheline: 4 bytes */
};
struct sctp_adaptation_ind_param {
	struct sctp_paramhdr       param_hdr;            /*     0     4 */
	__be32                     adaptation_ind;       /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct sctp_supported_ext_param {
	struct sctp_paramhdr       param_hdr;            /*     0     4 */
	__u8                       chunks[];             /*     4     0 */

	/* size: 4, cachelines: 1, members: 2 */
	/* last cacheline: 4 bytes */
};
struct sctp_random_param {
	struct sctp_paramhdr       param_hdr;            /*     0     4 */
	__u8                       random_val[];         /*     4     0 */

	/* size: 4, cachelines: 1, members: 2 */
	/* last cacheline: 4 bytes */
};
struct sctp_chunks_param {
	struct sctp_paramhdr       param_hdr;            /*     0     4 */
	__u8                       chunks[];             /*     4     0 */

	/* size: 4, cachelines: 1, members: 2 */
	/* last cacheline: 4 bytes */
};
struct sctp_hmac_algo_param {
	struct sctp_paramhdr       param_hdr;            /*     0     4 */
	__be16                     hmac_ids[];           /*     4     0 */

	/* size: 4, cachelines: 1, members: 2 */
	/* last cacheline: 4 bytes */
};
struct sctp_cookie_param {
	struct sctp_paramhdr       p;                    /*     0     4 */
	__u8                       body[];               /*     4     0 */

	/* size: 4, cachelines: 1, members: 2 */
	/* last cacheline: 4 bytes */
};
struct sctp_gap_ack_block {
	__be16                     start;                /*     0     2 */
	__be16                     end;                  /*     2     2 */

	/* size: 4, cachelines: 1, members: 2 */
	/* last cacheline: 4 bytes */
};
union sctp_sack_variable {
	struct sctp_gap_ack_block  gab;                /*     0     4 */
	__be32                     dup;                /*     0     4 */
};
struct sctp_sackhdr {
	__be32                     cum_tsn_ack;          /*     0     4 */
	__be32                     a_rwnd;               /*     4     4 */
	__be16                     num_gap_ack_blocks;   /*     8     2 */
	__be16                     num_dup_tsns;         /*    10     2 */
	union sctp_sack_variable   variable[];           /*    12     0 */

	/* size: 12, cachelines: 1, members: 5 */
	/* last cacheline: 12 bytes */
};
struct sctp_heartbeathdr {
	struct sctp_paramhdr       info;                 /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct sctp_shutdownhdr {
	__be32                     cum_tsn_ack;          /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct sctp_errhdr {
	__be16                     cause;                /*     0     2 */
	__be16                     length;               /*     2     2 */
	__u8                       variable[];           /*     4     0 */

	/* size: 4, cachelines: 1, members: 3 */
	/* last cacheline: 4 bytes */
};
struct sctp_ecnehdr {
	__be32                     lowest_tsn;           /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct sctp_cwrhdr {
	__be32                     lowest_tsn;           /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct sctp_fwdtsn_skip {
	__be16                     stream;               /*     0     2 */
	__be16                     ssn;                  /*     2     2 */

	/* size: 4, cachelines: 1, members: 2 */
	/* last cacheline: 4 bytes */
};
struct sctp_fwdtsn_hdr {
	__be32                     new_cum_tsn;          /*     0     4 */
	struct sctp_fwdtsn_skip    skip[];               /*     4     0 */

	/* size: 4, cachelines: 1, members: 2 */
	/* last cacheline: 4 bytes */
};
struct sctp_ifwdtsn_skip {
	__be16                     stream;               /*     0     2 */
	__u8                       reserved;             /*     2     1 */
	__u8                       flags;                /*     3     1 */
	__be32                     mid;                  /*     4     4 */

	/* size: 8, cachelines: 1, members: 4 */
	/* last cacheline: 8 bytes */
};
struct sctp_ifwdtsn_hdr {
	__be32                     new_cum_tsn;          /*     0     4 */
	struct sctp_ifwdtsn_skip   skip[];               /*     4     0 */

	/* size: 4, cachelines: 1, members: 2 */
	/* last cacheline: 4 bytes */
};
struct sctp_addip_param {
	struct sctp_paramhdr       param_hdr;            /*     0     4 */
	__be32                     crr_id;               /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct sctp_addiphdr {
	__be32                     serial;               /*     0     4 */
	__u8                       params[];             /*     4     0 */

	/* size: 4, cachelines: 1, members: 2 */
	/* last cacheline: 4 bytes */
};
struct sctp_authhdr {
	__be16                     shkey_id;             /*     0     2 */
	__be16                     hmac_id;              /*     2     2 */
	__u8                       hmac[];               /*     4     0 */

	/* size: 4, cachelines: 1, members: 3 */
	/* last cacheline: 4 bytes */
};
struct sctp_association {
	struct sctp_ep_common      base;                 /*     0   160 */
	/* --- cacheline 2 boundary (128 bytes) was 32 bytes ago --- */
	struct list_head           asocs;                /*   160    16 */
	sctp_assoc_t               assoc_id;             /*   176     4 */

	/* XXX 4 bytes hole, try to pack */

	struct sctp_endpoint *     ep;                   /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	struct sctp_cookie         c;                    /*   192   144 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 5 boundary (320 bytes) was 16 bytes ago --- */
	struct {
		struct list_head   transport_addr_list;  /*   336    16 */
		__u32              rwnd;                 /*   352     4 */
		__u16              transport_count;      /*   356     2 */
		__u16              port;                 /*   358     2 */
		struct sctp_transport * primary_path;    /*   360     8 */
		union sctp_addr    primary_addr;         /*   368    28 */

		/* XXX 4 bytes hole, try to pack */

		/* --- cacheline 6 boundary (384 bytes) was 16 bytes ago --- */
		struct sctp_transport * active_path;     /*   400     8 */
		struct sctp_transport * retran_path;     /*   408     8 */
		struct sctp_transport * last_sent_to;    /*   416     8 */
		struct sctp_transport * last_data_from;  /*   424     8 */
		struct sctp_tsnmap tsn_map;              /*   432    96 */

		/* XXX last struct has 4 bytes of padding */

		/* --- cacheline 8 boundary (512 bytes) was 16 bytes ago --- */
		__be16             addip_disabled_mask;  /*   528     2 */
		__u16              ecn_capable:1;        /*   530: 0  2 */
		__u16              ipv4_address:1;       /*   530: 1  2 */
		__u16              ipv6_address:1;       /*   530: 2  2 */
		__u16              hostname_address:1;   /*   530: 3  2 */
		__u16              asconf_capable:1;     /*   530: 4  2 */
		__u16              prsctp_capable:1;     /*   530: 5  2 */
		__u16              reconf_capable:1;     /*   530: 6  2 */
		__u16              intl_capable:1;       /*   530: 7  2 */
		__u16              auth_capable:1;       /*   530: 8  2 */
		__u16              sack_needed:1;        /*   530: 9  2 */
		__u16              sack_generation:1;    /*   530:10  2 */
		__u16              zero_window_announced:1; /*   530:11  2 */

		/* XXX 4 bits hole, try to pack */

		__u32              sack_cnt;             /*   532     4 */
		__u32              adaptation_ind;       /*   536     4 */
		struct sctp_inithdr_host i;              /*   540    16 */

		/* XXX 4 bytes hole, try to pack */

		void *             cookie;               /*   560     8 */
		int                cookie_len;           /*   568     4 */
		__u32              addip_serial;         /*   572     4 */
		/* --- cacheline 9 boundary (576 bytes) --- */
		struct sctp_random_param * peer_random;  /*   576     8 */
		struct sctp_chunks_param * peer_chunks;  /*   584     8 */
		struct sctp_hmac_algo_param * peer_hmacs; /*   592     8 */
	} peer;                                          /*   336   264 */
	enum sctp_state            state;                /*   600     4 */
	int                        overall_error_count;  /*   604     4 */
	ktime_t                    cookie_life;          /*   608     8 */
	long unsigned int          rto_initial;          /*   616     8 */
	long unsigned int          rto_max;              /*   624     8 */
	long unsigned int          rto_min;              /*   632     8 */
	/* --- cacheline 10 boundary (640 bytes) --- */
	int                        max_burst;            /*   640     4 */
	int                        max_retrans;          /*   644     4 */
	__u16                      pf_retrans;           /*   648     2 */
	__u16                      ps_retrans;           /*   650     2 */
	__u16                      max_init_attempts;    /*   652     2 */
	__u16                      init_retries;         /*   654     2 */
	long unsigned int          max_init_timeo;       /*   656     8 */
	long unsigned int          hbinterval;           /*   664     8 */
	__u16                      pathmaxrxt;           /*   672     2 */

	/* XXX 2 bytes hole, try to pack */

	__u32                      flowlabel;            /*   676     4 */
	__u8                       dscp;                 /*   680     1 */
	__u8                       pmtu_pending;         /*   681     1 */

	/* XXX 2 bytes hole, try to pack */

	__u32                      pathmtu;              /*   684     4 */
	__u32                      param_flags;          /*   688     4 */
	__u32                      sackfreq;             /*   692     4 */
	long unsigned int          sackdelay;            /*   696     8 */
	/* --- cacheline 11 boundary (704 bytes) --- */
	long unsigned int          timeouts[11];         /*   704    88 */
	/* --- cacheline 12 boundary (768 bytes) was 24 bytes ago --- */
	struct timer_list          timers[11];           /*   792   792 */
	/* --- cacheline 24 boundary (1536 bytes) was 48 bytes ago --- */
	struct sctp_transport *    shutdown_last_sent_to; /*  1584     8 */
	struct sctp_transport *    init_last_sent_to;    /*  1592     8 */
	/* --- cacheline 25 boundary (1600 bytes) --- */
	int                        shutdown_retries;     /*  1600     4 */
	__u32                      next_tsn;             /*  1604     4 */
	__u32                      ctsn_ack_point;       /*  1608     4 */
	__u32                      adv_peer_ack_point;   /*  1612     4 */
	__u32                      highest_sacked;       /*  1616     4 */
	__u32                      fast_recovery_exit;   /*  1620     4 */
	__u8                       fast_recovery;        /*  1624     1 */

	/* XXX 1 byte hole, try to pack */

	__u16                      unack_data;           /*  1626     2 */
	__u32                      rtx_data_chunks;      /*  1628     4 */
	__u32                      rwnd;                 /*  1632     4 */
	__u32                      a_rwnd;               /*  1636     4 */
	__u32                      rwnd_over;            /*  1640     4 */
	__u32                      rwnd_press;           /*  1644     4 */
	int                        sndbuf_used;          /*  1648     4 */
	atomic_t                   rmem_alloc;           /*  1652     4 */
	wait_queue_head_t          wait;                 /*  1656    24 */
	/* --- cacheline 26 boundary (1664 bytes) was 16 bytes ago --- */
	__u32                      frag_point;           /*  1680     4 */
	__u32                      user_frag;            /*  1684     4 */
	int                        init_err_counter;     /*  1688     4 */
	int                        init_cycle;           /*  1692     4 */
	__u16                      default_stream;       /*  1696     2 */
	__u16                      default_flags;        /*  1698     2 */
	__u32                      default_ppid;         /*  1700     4 */
	__u32                      default_context;      /*  1704     4 */
	__u32                      default_timetolive;   /*  1708     4 */
	__u32                      default_rcv_context;  /*  1712     4 */

	/* XXX 4 bytes hole, try to pack */

	struct sctp_stream         stream;               /*  1720    64 */
	/* --- cacheline 27 boundary (1728 bytes) was 56 bytes ago --- */
	struct sctp_outq           outqueue;             /*  1784   112 */

	/* XXX last struct has 2 bytes of padding */

	/* --- cacheline 29 boundary (1856 bytes) was 40 bytes ago --- */
	struct sctp_ulpq           ulpq;                 /*  1896    88 */
	/* --- cacheline 31 boundary (1984 bytes) --- */
	__u32                      last_ecne_tsn;        /*  1984     4 */
	__u32                      last_cwr_tsn;         /*  1988     4 */
	int                        numduptsns;           /*  1992     4 */

	/* XXX 4 bytes hole, try to pack */

	struct sctp_chunk *        addip_last_asconf;    /*  2000     8 */
	struct list_head           asconf_ack_list;      /*  2008    16 */
	struct list_head           addip_chunk_list;     /*  2024    16 */
	__u32                      addip_serial;         /*  2040     4 */
	int                        src_out_of_asoc_ok;   /*  2044     4 */
	/* --- cacheline 32 boundary (2048 bytes) --- */
	union sctp_addr *          asconf_addr_del_pending; /*  2048     8 */
	struct sctp_transport *    new_transport;        /*  2056     8 */
	struct list_head           endpoint_shared_keys; /*  2064    16 */
	struct sctp_auth_bytes *   asoc_shared_key;      /*  2080     8 */
	struct sctp_shared_key *   shkey;                /*  2088     8 */
	__u16                      default_hmac_id;      /*  2096     2 */
	__u16                      active_key_id;        /*  2098     2 */
	__u8                       need_ecne:1;          /*  2100: 0  1 */
	__u8                       temp:1;               /*  2100: 1  1 */
	__u8                       pf_expose:2;          /*  2100: 2  1 */
	__u8                       force_delay:1;        /*  2100: 4  1 */

	/* XXX 3 bits hole, try to pack */

	__u8                       strreset_enable;      /*  2101     1 */
	__u8                       strreset_outstanding; /*  2102     1 */

	/* XXX 1 byte hole, try to pack */

	__u32                      strreset_outseq;      /*  2104     4 */
	__u32                      strreset_inseq;       /*  2108     4 */
	/* --- cacheline 33 boundary (2112 bytes) --- */
	__u32                      strreset_result[2];   /*  2112     8 */
	struct sctp_chunk *        strreset_chunk;       /*  2120     8 */
	struct sctp_priv_assoc_stats stats;              /*  2128   248 */
	/* --- cacheline 37 boundary (2368 bytes) was 8 bytes ago --- */
	int                        sent_cnt_removable;   /*  2376     4 */
	__u16                      subscribe;            /*  2380     2 */

	/* XXX 2 bytes hole, try to pack */

	__u64                      abandoned_unsent[3];  /*  2384    24 */
	__u64                      abandoned_sent[3];    /*  2408    24 */
	/* --- cacheline 38 boundary (2432 bytes) --- */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*  2432    16 */

	/* size: 2448, cachelines: 39, members: 92 */
	/* sum members: 2427, holes: 8, sum holes: 20 */
	/* sum bitfield members: 5 bits, bit holes: 1, sum bit holes: 3 bits */
	/* paddings: 2, sum paddings: 6 */
	/* forced alignments: 1 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct sctp_auth_bytes {
	refcount_t                 refcnt;               /*     0     4 */
	__u32                      len;                  /*     4     4 */
	__u8                       data[];               /*     8     0 */

	/* size: 8, cachelines: 1, members: 3 */
	/* last cacheline: 8 bytes */
};
struct sctp_shared_key {
	struct list_head           key_list;             /*     0    16 */
	struct sctp_auth_bytes *   key;                  /*    16     8 */
	refcount_t                 refcnt;               /*    24     4 */
	__u16                      key_id;               /*    28     2 */
	__u8                       deactivated;          /*    30     1 */

	/* size: 32, cachelines: 1, members: 5 */
	/* padding: 1 */
	/* last cacheline: 32 bytes */
};
union sctp_addr {
	struct sockaddr_in         v4;                 /*     0    16 */
	struct sockaddr_in6        v6;                 /*     0    28 */
	struct sockaddr            sa;                 /*     0    16 */
};
struct sctp_tsnmap {
	long unsigned int *        tsn_map;              /*     0     8 */
	__u32                      base_tsn;             /*     8     4 */
	__u32                      cumulative_tsn_ack_point; /*    12     4 */
	__u32                      max_tsn_seen;         /*    16     4 */
	__u16                      len;                  /*    20     2 */
	__u16                      pending_data;         /*    22     2 */
	__u16                      num_dup_tsns;         /*    24     2 */

	/* XXX 2 bytes hole, try to pack */

	__be32                     dup_tsns[16];         /*    28    64 */

	/* size: 96, cachelines: 2, members: 8 */
	/* sum members: 90, holes: 1, sum holes: 2 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct sctp_ulpevent {
	struct sctp_association *  asoc;                 /*     0     8 */
	struct sctp_chunk *        chunk;                /*     8     8 */
	unsigned int               rmem_len;             /*    16     4 */
	union {
		__u32              mid;                  /*    20     4 */
		__u16              ssn;                  /*    20     2 */
	};                                               /*    20     4 */
	union {
		__u32              ppid;                 /*    24     4 */
		__u32              fsn;                  /*    24     4 */
	};                                               /*    24     4 */
	__u32                      tsn;                  /*    28     4 */
	__u32                      cumtsn;               /*    32     4 */
	__u16                      stream;               /*    36     2 */
	__u16                      flags;                /*    38     2 */
	__u16                      msg_flags;            /*    40     2 */

	/* size: 42, cachelines: 1, members: 10 */
	/* last cacheline: 42 bytes */
} __attribute__((__packed__));
struct sctp_chunk {
	struct list_head           list;                 /*     0    16 */
	refcount_t                 refcnt;               /*    16     4 */
	int                        sent_count;           /*    20     4 */
	union {
		struct list_head   transmitted_list;     /*    24    16 */
		struct list_head   stream_list;          /*    24    16 */
	};                                               /*    24    16 */
	struct list_head           frag_list;            /*    40    16 */
	struct sk_buff *           skb;                  /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	union {
		struct sk_buff *   head_skb;             /*    64     8 */
		struct sctp_shared_key * shkey;          /*    64     8 */
	};                                               /*    64     8 */
	union sctp_params          param_hdr;            /*    72     8 */
	union {
		__u8 *             v;                    /*    80     8 */
		struct sctp_datahdr * data_hdr;          /*    80     8 */
		struct sctp_inithdr * init_hdr;          /*    80     8 */
		struct sctp_sackhdr * sack_hdr;          /*    80     8 */
		struct sctp_heartbeathdr * hb_hdr;       /*    80     8 */
		struct sctp_sender_hb_info * hbs_hdr;    /*    80     8 */
		struct sctp_shutdownhdr * shutdown_hdr;  /*    80     8 */
		struct sctp_signed_cookie * cookie_hdr;  /*    80     8 */
		struct sctp_ecnehdr * ecne_hdr;          /*    80     8 */
		struct sctp_cwrhdr * ecn_cwr_hdr;        /*    80     8 */
		struct sctp_errhdr * err_hdr;            /*    80     8 */
		struct sctp_addiphdr * addip_hdr;        /*    80     8 */
		struct sctp_fwdtsn_hdr * fwdtsn_hdr;     /*    80     8 */
		struct sctp_authhdr * auth_hdr;          /*    80     8 */
		struct sctp_idatahdr * idata_hdr;        /*    80     8 */
		struct sctp_ifwdtsn_hdr * ifwdtsn_hdr;   /*    80     8 */
	} subh;                                          /*    80     8 */
	__u8 *                     chunk_end;            /*    88     8 */
	struct sctp_chunkhdr *     chunk_hdr;            /*    96     8 */
	struct sctphdr *           sctp_hdr;             /*   104     8 */
	struct sctp_sndrcvinfo     sinfo;                /*   112    32 */
	/* --- cacheline 2 boundary (128 bytes) was 16 bytes ago --- */
	struct sctp_association *  asoc;                 /*   144     8 */
	struct sctp_ep_common *    rcvr;                 /*   152     8 */
	long unsigned int          sent_at;              /*   160     8 */
	union sctp_addr            source;               /*   168    28 */
	/* --- cacheline 3 boundary (192 bytes) was 4 bytes ago --- */
	union sctp_addr            dest;                 /*   196    28 */
	struct sctp_datamsg *      msg;                  /*   224     8 */
	struct sctp_transport *    transport;            /*   232     8 */
	struct sk_buff *           auth_chunk;           /*   240     8 */
	__u16                      rtt_in_progress:1;    /*   248: 0  2 */
	__u16                      has_tsn:1;            /*   248: 1  2 */
	__u16                      has_ssn:1;            /*   248: 2  2 */
	__u16                      singleton:1;          /*   248: 3  2 */
	__u16                      end_of_packet:1;      /*   248: 4  2 */
	__u16                      ecn_ce_done:1;        /*   248: 5  2 */
	__u16                      pdiscard:1;           /*   248: 6  2 */
	__u16                      tsn_gap_acked:1;      /*   248: 7  2 */
	__u16                      data_accepted:1;      /*   248: 8  2 */
	__u16                      auth:1;               /*   248: 9  2 */
	__u16                      has_asconf:1;         /*   248:10  2 */
	__u16                      tsn_missing_report:2; /*   248:11  2 */
	__u16                      fast_retransmit:2;    /*   248:13  2 */

	/* size: 256, cachelines: 4, members: 34 */
	/* padding: 6 */
	/* bit_padding: 1 bits */
};
struct sctp_ulpq {
	char                       pd_mode;              /*     0     1 */

	/* XXX 7 bytes hole, try to pack */

	struct sctp_association *  asoc;                 /*     8     8 */
	struct sk_buff_head        reasm;                /*    16    24 */
	struct sk_buff_head        reasm_uo;             /*    40    24 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct sk_buff_head        lobby;                /*    64    24 */

	/* size: 88, cachelines: 2, members: 5 */
	/* sum members: 81, holes: 1, sum holes: 7 */
	/* last cacheline: 24 bytes */
};
struct sctp_stream_interleave {
	__u16                      data_chunk_len;       /*     0     2 */
	__u16                      ftsn_chunk_len;       /*     2     2 */

	/* XXX 4 bytes hole, try to pack */

	struct sctp_chunk *        (*make_datafrag)(const struct sctp_association  *, const struct sctp_sndrcvinfo  *, int, __u8, gfp_t); /*     8     8 */
	void                       (*assign_number)(struct sctp_chunk *); /*    16     8 */
	bool                       (*validate_data)(struct sctp_chunk *); /*    24     8 */
	int                        (*ulpevent_data)(struct sctp_ulpq *, struct sctp_chunk *, gfp_t); /*    32     8 */
	int                        (*enqueue_event)(struct sctp_ulpq *, struct sctp_ulpevent *); /*    40     8 */
	void                       (*renege_events)(struct sctp_ulpq *, struct sctp_chunk *, gfp_t); /*    48     8 */
	void                       (*start_pd)(struct sctp_ulpq *, gfp_t); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	void                       (*abort_pd)(struct sctp_ulpq *, gfp_t); /*    64     8 */
	void                       (*generate_ftsn)(struct sctp_outq *, __u32); /*    72     8 */
	bool                       (*validate_ftsn)(struct sctp_chunk *); /*    80     8 */
	void                       (*report_ftsn)(struct sctp_ulpq *, __u32); /*    88     8 */
	void                       (*handle_ftsn)(struct sctp_ulpq *, struct sctp_chunk *); /*    96     8 */

	/* size: 104, cachelines: 2, members: 14 */
	/* sum members: 100, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct sctp_outq {
	struct sctp_association *  asoc;                 /*     0     8 */
	struct list_head           out_chunk_list;       /*     8    16 */
	struct sctp_sched_ops *    sched;                /*    24     8 */
	unsigned int               out_qlen;             /*    32     4 */
	unsigned int               error;                /*    36     4 */
	struct list_head           control_chunk_list;   /*    40    16 */
	struct list_head           sacked;               /*    56    16 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	struct list_head           retransmit;           /*    72    16 */
	struct list_head           abandoned;            /*    88    16 */
	__u32                      outstanding_bytes;    /*   104     4 */
	char                       fast_rtx;             /*   108     1 */
	char                       cork;                 /*   109     1 */

	/* size: 112, cachelines: 2, members: 12 */
	/* padding: 2 */
	/* last cacheline: 48 bytes */
};
struct sctp_bind_bucket {
	short unsigned int         port;                 /*     0     2 */
	signed char                fastreuse;            /*     2     1 */
	signed char                fastreuseport;        /*     3     1 */
	kuid_t                     fastuid;              /*     4     4 */
	struct hlist_node          node;                 /*     8    16 */
	struct hlist_head          owner;                /*    24     8 */
	struct net *               net;                  /*    32     8 */

	/* size: 40, cachelines: 1, members: 7 */
	/* last cacheline: 40 bytes */
};
struct sctp_sock {
	struct inet_sock           inet __attribute__((__aligned__(8))); /*     0  1096 */
	/* --- cacheline 17 boundary (1088 bytes) was 8 bytes ago --- */
	enum sctp_socket_type      type;                 /*  1096     4 */

	/* XXX 4 bytes hole, try to pack */

	struct sctp_pf *           pf;                   /*  1104     8 */
	struct crypto_shash *      hmac;                 /*  1112     8 */
	char *                     sctp_hmac_alg;        /*  1120     8 */
	struct sctp_endpoint *     ep;                   /*  1128     8 */
	struct sctp_bind_bucket *  bind_hash;            /*  1136     8 */
	__u16                      default_stream;       /*  1144     2 */

	/* XXX 2 bytes hole, try to pack */

	__u32                      default_ppid;         /*  1148     4 */
	/* --- cacheline 18 boundary (1152 bytes) --- */
	__u16                      default_flags;        /*  1152     2 */

	/* XXX 2 bytes hole, try to pack */

	__u32                      default_context;      /*  1156     4 */
	__u32                      default_timetolive;   /*  1160     4 */
	__u32                      default_rcv_context;  /*  1164     4 */
	int                        max_burst;            /*  1168     4 */
	__u32                      hbinterval;           /*  1172     4 */
	__u16                      pathmaxrxt;           /*  1176     2 */

	/* XXX 2 bytes hole, try to pack */

	__u32                      flowlabel;            /*  1180     4 */
	__u8                       dscp;                 /*  1184     1 */

	/* XXX 1 byte hole, try to pack */

	__u16                      pf_retrans;           /*  1186     2 */
	__u16                      ps_retrans;           /*  1188     2 */

	/* XXX 2 bytes hole, try to pack */

	__u32                      pathmtu;              /*  1192     4 */
	__u32                      sackdelay;            /*  1196     4 */
	__u32                      sackfreq;             /*  1200     4 */
	__u32                      param_flags;          /*  1204     4 */
	__u32                      default_ss;           /*  1208     4 */
	struct sctp_rtoinfo        rtoinfo;              /*  1212    16 */
	/* --- cacheline 19 boundary (1216 bytes) was 12 bytes ago --- */
	struct sctp_paddrparams    paddrparam __attribute__((__aligned__(4))); /*  1228   156 */

	/* XXX last struct has 1 byte of padding */

	/* --- cacheline 21 boundary (1344 bytes) was 40 bytes ago --- */
	struct sctp_assocparams    assocparams;          /*  1384    20 */
	__u16                      subscribe;            /*  1404     2 */
	struct sctp_initmsg        initmsg;              /*  1406     8 */

	/* XXX 2 bytes hole, try to pack */

	/* --- cacheline 22 boundary (1408 bytes) was 8 bytes ago --- */
	int                        user_frag;            /*  1416     4 */
	__u32                      autoclose;            /*  1420     4 */
	__u32                      adaptation_ind;       /*  1424     4 */
	__u32                      pd_point;             /*  1428     4 */
	__u16                      nodelay:1;            /*  1432: 0  2 */
	__u16                      pf_expose:2;          /*  1432: 1  2 */
	__u16                      reuse:1;              /*  1432: 3  2 */
	__u16                      disable_fragments:1;  /*  1432: 4  2 */
	__u16                      v4mapped:1;           /*  1432: 5  2 */
	__u16                      frag_interleave:1;    /*  1432: 6  2 */
	__u16                      recvrcvinfo:1;        /*  1432: 7  2 */
	__u16                      recvnxtinfo:1;        /*  1432: 8  2 */
	__u16                      data_ready_signalled:1; /*  1432: 9  2 */

	/* XXX 6 bits hole, try to pack */
	/* XXX 2 bytes hole, try to pack */

	atomic_t                   pd_mode;              /*  1436     4 */
	struct sk_buff_head        pd_lobby;             /*  1440    24 */
	struct list_head           auto_asconf_list;     /*  1464    16 */
	/* --- cacheline 23 boundary (1472 bytes) was 8 bytes ago --- */
	int                        do_auto_asconf;       /*  1480     4 */

	/* size: 1488, cachelines: 24, members: 47 */
	/* sum members: 1465, holes: 8, sum holes: 17 */
	/* sum bitfield members: 10 bits, bit holes: 1, sum bit holes: 6 bits */
	/* padding: 4 */
	/* paddings: 1, sum paddings: 1 */
	/* forced alignments: 2 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct sctp_pf {
	void                       (*event_msgname)(struct sctp_ulpevent *, char *, int *); /*     0     8 */
	void                       (*skb_msgname)(struct sk_buff *, char *, int *); /*     8     8 */
	int                        (*af_supported)(sa_family_t, struct sctp_sock *); /*    16     8 */
	int                        (*cmp_addr)(const union sctp_addr  *, const union sctp_addr  *, struct sctp_sock *); /*    24     8 */
	int                        (*bind_verify)(struct sctp_sock *, union sctp_addr *); /*    32     8 */
	int                        (*send_verify)(struct sctp_sock *, union sctp_addr *); /*    40     8 */
	int                        (*supported_addrs)(const struct sctp_sock  *, __be16 *); /*    48     8 */
	struct sock *              (*create_accept_sk)(struct sock *, struct sctp_association *, bool); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*addr_to_user)(struct sctp_sock *, union sctp_addr *); /*    64     8 */
	void                       (*to_sk_saddr)(union sctp_addr *, struct sock *); /*    72     8 */
	void                       (*to_sk_daddr)(union sctp_addr *, struct sock *); /*    80     8 */
	void                       (*copy_ip_options)(struct sock *, struct sock *); /*    88     8 */
	struct sctp_af *           af;                   /*    96     8 */

	/* size: 104, cachelines: 2, members: 13 */
	/* last cacheline: 40 bytes */
};
struct sctp_cookie {
	__u32                      my_vtag;              /*     0     4 */
	__u32                      peer_vtag;            /*     4     4 */
	__u32                      my_ttag;              /*     8     4 */
	__u32                      peer_ttag;            /*    12     4 */
	ktime_t                    expiration;           /*    16     8 */
	__u16                      sinit_num_ostreams;   /*    24     2 */
	__u16                      sinit_max_instreams;  /*    26     2 */
	__u32                      initial_tsn;          /*    28     4 */
	union sctp_addr            peer_addr;            /*    32    28 */
	__u16                      my_port;              /*    60     2 */
	__u8                       prsctp_capable;       /*    62     1 */
	__u8                       padding;              /*    63     1 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	__u32                      adaptation_ind;       /*    64     4 */
	__u8                       auth_random[36];      /*    68    36 */
	__u8                       auth_hmacs[10];       /*   104    10 */
	__u8                       auth_chunks[20];      /*   114    20 */

	/* XXX 2 bytes hole, try to pack */

	/* --- cacheline 2 boundary (128 bytes) was 8 bytes ago --- */
	__u32                      raw_addr_list_len;    /*   136     4 */
	struct sctp_init_chunk     peer_init[];          /*   140     0 */

	/* size: 144, cachelines: 3, members: 18 */
	/* sum members: 138, holes: 1, sum holes: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct sctp_signed_cookie {
	__u8                       signature[32];        /*     0    32 */
	__u32                      __pad;                /*    32     4 */
	struct sctp_cookie         c;                    /*    36   144 */

	/* XXX last struct has 4 bytes of padding */

	/* size: 180, cachelines: 3, members: 3 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 52 bytes */
} __attribute__((__packed__));
union sctp_addr_param {
	struct sctp_paramhdr       p;                  /*     0     4 */
	struct sctp_ipv4addr_param v4;                 /*     0     8 */
	struct sctp_ipv6addr_param v6;                 /*     0    20 */
};
union sctp_params {
	void *                     v;                  /*     0     8 */
	struct sctp_paramhdr *     p;                  /*     0     8 */
	struct sctp_cookie_preserve_param * life;      /*     0     8 */
	struct sctp_hostname_param * dns;              /*     0     8 */
	struct sctp_cookie_param * cookie;             /*     0     8 */
	struct sctp_supported_addrs_param * sat;       /*     0     8 */
	struct sctp_ipv4addr_param * v4;               /*     0     8 */
	struct sctp_ipv6addr_param * v6;               /*     0     8 */
	union sctp_addr_param *    addr;               /*     0     8 */
	struct sctp_adaptation_ind_param * aind;       /*     0     8 */
	struct sctp_supported_ext_param * ext;         /*     0     8 */
	struct sctp_random_param * random;             /*     0     8 */
	struct sctp_chunks_param * chunks;             /*     0     8 */
	struct sctp_hmac_algo_param * hmac_algo;       /*     0     8 */
	struct sctp_addip_param *  addip;              /*     0     8 */
};
struct sctp_sender_hb_info {
	struct sctp_paramhdr       param_hdr;            /*     0     4 */
	union sctp_addr            daddr;                /*     4    28 */
	long unsigned int          sent_at;              /*    32     8 */
	__u64                      hb_nonce;             /*    40     8 */

	/* size: 48, cachelines: 1, members: 4 */
	/* last cacheline: 48 bytes */
};
struct sctp_af {
	int                        (*sctp_xmit)(struct sk_buff *, struct sctp_transport *); /*     0     8 */
	int                        (*setsockopt)(struct sock *, int, int, sockptr_t, unsigned int); /*     8     8 */
	int                        (*getsockopt)(struct sock *, int, int, char *, int *); /*    16     8 */
	void                       (*get_dst)(struct sctp_transport *, union sctp_addr *, struct flowi *, struct sock *); /*    24     8 */
	void                       (*get_saddr)(struct sctp_sock *, struct sctp_transport *, struct flowi *); /*    32     8 */
	void                       (*copy_addrlist)(struct list_head *, struct net_device *); /*    40     8 */
	int                        (*cmp_addr)(const union sctp_addr  *, const union sctp_addr  *); /*    48     8 */
	void                       (*addr_copy)(union sctp_addr *, union sctp_addr *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	void                       (*from_skb)(union sctp_addr *, struct sk_buff *, int); /*    64     8 */
	void                       (*from_sk)(union sctp_addr *, struct sock *); /*    72     8 */
	bool                       (*from_addr_param)(union sctp_addr *, union sctp_addr_param *, __be16, int); /*    80     8 */
	int                        (*to_addr_param)(const union sctp_addr  *, union sctp_addr_param *); /*    88     8 */
	int                        (*addr_valid)(union sctp_addr *, struct sctp_sock *, const struct sk_buff  *); /*    96     8 */
	enum sctp_scope            (*scope)(union sctp_addr *); /*   104     8 */
	void                       (*inaddr_any)(union sctp_addr *, __be16); /*   112     8 */
	int                        (*is_any)(const union sctp_addr  *); /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	int                        (*available)(union sctp_addr *, struct sctp_sock *); /*   128     8 */
	int                        (*skb_iif)(const struct sk_buff  *); /*   136     8 */
	int                        (*is_ce)(const struct sk_buff  *); /*   144     8 */
	void                       (*seq_dump_addr)(struct seq_file *, union sctp_addr *); /*   152     8 */
	void                       (*ecn_capable)(struct sock *); /*   160     8 */
	__u16                      net_header_len;       /*   168     2 */

	/* XXX 2 bytes hole, try to pack */

	int                        sockaddr_len;         /*   172     4 */
	int                        (*ip_options_len)(struct sock *); /*   176     8 */
	sa_family_t                sa_family;            /*   184     2 */

	/* XXX 6 bytes hole, try to pack */

	/* --- cacheline 3 boundary (192 bytes) --- */
	struct list_head           list;                 /*   192    16 */

	/* size: 208, cachelines: 4, members: 26 */
	/* sum members: 200, holes: 2, sum holes: 8 */
	/* last cacheline: 16 bytes */
};
struct sctp_transport {
	struct list_head           transports;           /*     0    16 */
	struct rhlist_head         node;                 /*    16    16 */
	refcount_t                 refcnt;               /*    32     4 */
	__u32                      rto_pending:1;        /*    36: 0  4 */
	__u32                      hb_sent:1;            /*    36: 1  4 */
	__u32                      pmtu_pending:1;       /*    36: 2  4 */
	__u32                      dst_pending_confirm:1; /*    36: 3  4 */
	__u32                      sack_generation:1;    /*    36: 4  4 */

	/* XXX 27 bits hole, try to pack */

	u32                        dst_cookie;           /*    40     4 */

	/* XXX 4 bytes hole, try to pack */

	struct flowi               fl __attribute__((__aligned__(8))); /*    48   104 */
	/* --- cacheline 2 boundary (128 bytes) was 24 bytes ago --- */
	union sctp_addr            ipaddr;               /*   152    28 */

	/* XXX 4 bytes hole, try to pack */

	struct sctp_af *           af_specific;          /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	struct sctp_association *  asoc;                 /*   192     8 */
	long unsigned int          rto;                  /*   200     8 */
	__u32                      rtt;                  /*   208     4 */
	__u32                      rttvar;               /*   212     4 */
	__u32                      srtt;                 /*   216     4 */
	__u32                      cwnd;                 /*   220     4 */
	__u32                      ssthresh;             /*   224     4 */
	__u32                      partial_bytes_acked;  /*   228     4 */
	__u32                      flight_size;          /*   232     4 */
	__u32                      burst_limited;        /*   236     4 */
	struct dst_entry *         dst;                  /*   240     8 */
	union sctp_addr            saddr;                /*   248    28 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 4 boundary (256 bytes) was 24 bytes ago --- */
	long unsigned int          hbinterval;           /*   280     8 */
	long unsigned int          sackdelay;            /*   288     8 */
	__u32                      sackfreq;             /*   296     4 */
	atomic_t                   mtu_info;             /*   300     4 */
	ktime_t                    last_time_heard;      /*   304     8 */
	long unsigned int          last_time_sent;       /*   312     8 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	long unsigned int          last_time_ecne_reduced; /*   320     8 */
	__u16                      pathmaxrxt;           /*   328     2 */

	/* XXX 2 bytes hole, try to pack */

	__u32                      flowlabel;            /*   332     4 */
	__u8                       dscp;                 /*   336     1 */

	/* XXX 1 byte hole, try to pack */

	__u16                      pf_retrans;           /*   338     2 */
	__u16                      ps_retrans;           /*   340     2 */

	/* XXX 2 bytes hole, try to pack */

	__u32                      pathmtu;              /*   344     4 */
	__u32                      param_flags;          /*   348     4 */
	int                        init_sent_count;      /*   352     4 */
	int                        state;                /*   356     4 */
	short unsigned int         error_count;          /*   360     2 */

	/* XXX 6 bytes hole, try to pack */

	struct timer_list          T3_rtx_timer;         /*   368    72 */
	/* --- cacheline 6 boundary (384 bytes) was 56 bytes ago --- */
	struct timer_list          hb_timer;             /*   440    72 */
	/* --- cacheline 8 boundary (512 bytes) --- */
	struct timer_list          proto_unreach_timer;  /*   512    72 */
	/* --- cacheline 9 boundary (576 bytes) was 8 bytes ago --- */
	struct timer_list          reconf_timer;         /*   584    72 */
	/* --- cacheline 10 boundary (640 bytes) was 16 bytes ago --- */
	struct list_head           transmitted;          /*   656    16 */
	struct sctp_packet         packet;               /*   672    72 */

	/* XXX last struct has 7 bytes of padding */

	/* --- cacheline 11 boundary (704 bytes) was 40 bytes ago --- */
	struct list_head           send_ready;           /*   744    16 */
	struct {
		__u32              next_tsn_at_change;   /*   760     4 */
		char               changeover_active;    /*   764     1 */
		char               cycling_changeover;   /*   765     1 */
		char               cacc_saw_newack;      /*   766     1 */
	} cacc;                                          /*   760     8 */

	/* XXX last struct has 1 byte of padding */

	/* --- cacheline 12 boundary (768 bytes) --- */
	__u64                      hb_nonce;             /*   768     8 */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*   776    16 */

	/* size: 792, cachelines: 13, members: 51 */
	/* sum members: 765, holes: 7, sum holes: 23 */
	/* sum bitfield members: 5 bits, bit holes: 1, sum bit holes: 27 bits */
	/* paddings: 2, sum paddings: 8 */
	/* forced alignments: 2, forced holes: 1, sum forced holes: 4 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct sctp_datamsg {
	struct list_head           chunks;               /*     0    16 */
	refcount_t                 refcnt;               /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          expires_at;           /*    24     8 */
	int                        send_error;           /*    32     4 */
	u8                         send_failed:1;        /*    36: 0  1 */
	u8                         can_delay:1;          /*    36: 1  1 */
	u8                         abandoned:1;          /*    36: 2  1 */

	/* size: 40, cachelines: 1, members: 7 */
	/* sum members: 32, holes: 1, sum holes: 4 */
	/* sum bitfield members: 3 bits (0 bytes) */
	/* padding: 3 */
	/* bit_padding: 5 bits */
	/* last cacheline: 40 bytes */
};
struct sctp_ep_common {
	struct hlist_node          node;                 /*     0    16 */
	int                        hashent;              /*    16     4 */
	enum sctp_endpoint_type    type;                 /*    20     4 */
	refcount_t                 refcnt;               /*    24     4 */
	bool                       dead;                 /*    28     1 */

	/* XXX 3 bytes hole, try to pack */

	struct sock *              sk;                   /*    32     8 */
	struct net *               net;                  /*    40     8 */
	struct sctp_inq            inqueue;              /*    48    88 */
	/* --- cacheline 2 boundary (128 bytes) was 8 bytes ago --- */
	struct sctp_bind_addr      bind_addr;            /*   136    24 */

	/* size: 160, cachelines: 3, members: 9 */
	/* sum members: 157, holes: 1, sum holes: 3 */
	/* last cacheline: 32 bytes */
};
struct sctp_packet {
	__u16                      source_port;          /*     0     2 */
	__u16                      destination_port;     /*     2     2 */
	__u32                      vtag;                 /*     4     4 */
	struct list_head           chunk_list;           /*     8    16 */
	size_t                     overhead;             /*    24     8 */
	size_t                     size;                 /*    32     8 */
	size_t                     max_size;             /*    40     8 */
	struct sctp_transport *    transport;            /*    48     8 */
	struct sctp_chunk *        auth;                 /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u8                         has_cookie_echo:1;    /*    64: 0  1 */
	u8                         has_sack:1;           /*    64: 1  1 */
	u8                         has_auth:1;           /*    64: 2  1 */
	u8                         has_data:1;           /*    64: 3  1 */
	u8                         ipfragok:1;           /*    64: 4  1 */

	/* size: 72, cachelines: 2, members: 14 */
	/* padding: 7 */
	/* bit_padding: 3 bits */
	/* last cacheline: 8 bytes */
};
struct sctp_inq {
	struct list_head           in_chunk_list;        /*     0    16 */
	struct sctp_chunk *        in_progress;          /*    16     8 */
	struct work_struct         immediate;            /*    24    64 */

	/* size: 88, cachelines: 2, members: 3 */
	/* last cacheline: 24 bytes */
};
struct sctp_bind_addr {
	__u16                      port;                 /*     0     2 */

	/* XXX 6 bytes hole, try to pack */

	struct list_head           address_list;         /*     8    16 */

	/* size: 24, cachelines: 1, members: 2 */
	/* sum members: 18, holes: 1, sum holes: 6 */
	/* last cacheline: 24 bytes */
};
struct sctp_inithdr_host {
	__u32                      init_tag;             /*     0     4 */
	__u32                      a_rwnd;               /*     4     4 */
	__u16                      num_outbound_streams; /*     8     2 */
	__u16                      num_inbound_streams;  /*    10     2 */
	__u32                      initial_tsn;          /*    12     4 */

	/* size: 16, cachelines: 1, members: 5 */
	/* last cacheline: 16 bytes */
};
struct sctp_stream_priorities {
	struct list_head           prio_sched;           /*     0    16 */
	struct list_head           active;               /*    16    16 */
	struct sctp_stream_out_ext * next;               /*    32     8 */
	__u16                      prio;                 /*    40     2 */

	/* size: 48, cachelines: 1, members: 4 */
	/* padding: 6 */
	/* last cacheline: 48 bytes */
};
struct sctp_stream_out_ext {
	__u64                      abandoned_unsent[3];  /*     0    24 */
	__u64                      abandoned_sent[3];    /*    24    24 */
	struct list_head           outq;                 /*    48    16 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	union {
		struct {
			struct list_head prio_list;      /*    64    16 */
			struct sctp_stream_priorities * prio_head; /*    80     8 */
		};                                       /*    64    24 */
		struct {
			struct list_head rr_list;        /*    64    16 */
		};                                       /*    64    16 */
	};                                               /*    64    24 */

	/* size: 88, cachelines: 2, members: 4 */
	/* last cacheline: 24 bytes */
};
struct sctp_stream_out {
	union {
		__u32              mid;                  /*     0     4 */
		__u16              ssn;                  /*     0     2 */
	};                                               /*     0     4 */
	__u32                      mid_uo;               /*     4     4 */
	struct sctp_stream_out_ext * ext;                /*     8     8 */
	__u8                       state;                /*    16     1 */

	/* size: 24, cachelines: 1, members: 4 */
	/* padding: 7 */
	/* last cacheline: 24 bytes */
};
struct sctp_stream_in {
	union {
		__u32              mid;                  /*     0     4 */
		__u16              ssn;                  /*     0     2 */
	};                                               /*     0     4 */
	__u32                      mid_uo;               /*     4     4 */
	__u32                      fsn;                  /*     8     4 */
	__u32                      fsn_uo;               /*    12     4 */
	char                       pd_mode;              /*    16     1 */
	char                       pd_mode_uo;           /*    17     1 */

	/* size: 20, cachelines: 1, members: 6 */
	/* padding: 2 */
	/* last cacheline: 20 bytes */
};
struct sctp_stream {
	struct {
		struct __genradix  tree;                 /*     0     8 */
		struct sctp_stream_out type[];           /*     8     0 */
	} out;                                           /*     0     8 */
	struct {
		struct __genradix  tree;                 /*     8     8 */
		struct sctp_stream_in type[];            /*    16     0 */
	} in;                                            /*     8     8 */
	__u16                      outcnt;               /*    16     2 */
	__u16                      incnt;                /*    18     2 */

	/* XXX 4 bytes hole, try to pack */

	struct sctp_stream_out *   out_curr;             /*    24     8 */
	union {
		struct {
			struct list_head prio_list;      /*    32    16 */
		};                                       /*    32    16 */
		struct {
			struct list_head rr_list;        /*    32    16 */
			struct sctp_stream_out_ext * rr_next; /*    48     8 */
		};                                       /*    32    24 */
	};                                               /*    32    24 */
	struct sctp_stream_interleave * si;              /*    56     8 */

	/* size: 64, cachelines: 1, members: 7 */
	/* sum members: 60, holes: 1, sum holes: 4 */
};
struct sctp_priv_assoc_stats {
	struct __kernel_sockaddr_storage obs_rto_ipaddr; /*     0   128 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	__u64                      max_obs_rto;          /*   128     8 */
	__u64                      isacks;               /*   136     8 */
	__u64                      osacks;               /*   144     8 */
	__u64                      opackets;             /*   152     8 */
	__u64                      ipackets;             /*   160     8 */
	__u64                      rtxchunks;            /*   168     8 */
	__u64                      outofseqtsns;         /*   176     8 */
	__u64                      idupchunks;           /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	__u64                      gapcnt;               /*   192     8 */
	__u64                      ouodchunks;           /*   200     8 */
	__u64                      iuodchunks;           /*   208     8 */
	__u64                      oodchunks;            /*   216     8 */
	__u64                      iodchunks;            /*   224     8 */
	__u64                      octrlchunks;          /*   232     8 */
	__u64                      ictrlchunks;          /*   240     8 */

	/* size: 248, cachelines: 4, members: 16 */
	/* last cacheline: 56 bytes */
};
struct task_security_struct {
	u32                        osid;                 /*     0     4 */
	u32                        sid;                  /*     4     4 */
	u32                        exec_sid;             /*     8     4 */
	u32                        create_sid;           /*    12     4 */
	u32                        keycreate_sid;        /*    16     4 */
	u32                        sockcreate_sid;       /*    20     4 */

	/* size: 24, cachelines: 1, members: 6 */
	/* last cacheline: 24 bytes */
};
struct inode_security_struct {
	struct inode *             inode;                /*     0     8 */
	struct list_head           list;                 /*     8    16 */
	u32                        task_sid;             /*    24     4 */
	u32                        sid;                  /*    28     4 */
	u16                        sclass;               /*    32     2 */
	unsigned char              initialized;          /*    34     1 */

	/* XXX 1 byte hole, try to pack */

	spinlock_t                 lock;                 /*    36     4 */

	/* size: 40, cachelines: 1, members: 7 */
	/* sum members: 39, holes: 1, sum holes: 1 */
	/* last cacheline: 40 bytes */
};
struct file_security_struct {
	u32                        sid;                  /*     0     4 */
	u32                        fown_sid;             /*     4     4 */
	u32                        isid;                 /*     8     4 */
	u32                        pseqno;               /*    12     4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct superblock_security_struct {
	struct super_block *       sb;                   /*     0     8 */
	u32                        sid;                  /*     8     4 */
	u32                        def_sid;              /*    12     4 */
	u32                        mntpoint_sid;         /*    16     4 */
	short unsigned int         behavior;             /*    20     2 */
	short unsigned int         flags;                /*    22     2 */
	struct mutex               lock;                 /*    24    32 */
	struct list_head           isec_head;            /*    56    16 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	spinlock_t                 isec_lock;            /*    72     4 */

	/* size: 80, cachelines: 2, members: 9 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct msg_security_struct {
	u32                        sid;                  /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct ipc_security_struct {
	u16                        sclass;               /*     0     2 */

	/* XXX 2 bytes hole, try to pack */

	u32                        sid;                  /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* sum members: 6, holes: 1, sum holes: 2 */
	/* last cacheline: 8 bytes */
};
struct sk_security_struct {
	enum {
		NLBL_UNSET       = 0,
		NLBL_REQUIRE     = 1,
		NLBL_LABELED     = 2,
		NLBL_REQSKB      = 3,
		NLBL_CONNLABELED = 4,
	} nlbl_state;                                    /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct netlbl_lsm_secattr * nlbl_secattr;        /*     8     8 */
	u32                        sid;                  /*    16     4 */
	u32                        peer_sid;             /*    20     4 */
	u16                        sclass;               /*    24     2 */

	/* XXX 2 bytes hole, try to pack */

	enum {
		SCTP_ASSOC_UNSET = 0,
		SCTP_ASSOC_SET   = 1,
	} sctp_assoc_state;                              /*    28     4 */

	/* size: 32, cachelines: 1, members: 6 */
	/* sum members: 26, holes: 2, sum holes: 6 */
	/* last cacheline: 32 bytes */
};
struct tun_security_struct {
	u32                        sid;                  /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct key_security_struct {
	u32                        sid;                  /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct ib_security_struct {
	u32                        sid;                  /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct bpf_security_struct {
	u32                        sid;                  /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct perf_event_security_struct {
	u32                        sid;                  /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct selinux_mnt_opts {
	const char  *              fscontext;            /*     0     8 */
	const char  *              context;              /*     8     8 */
	const char  *              rootcontext;          /*    16     8 */
	const char  *              defcontext;           /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct selinux_load_state {
	struct selinux_policy *    policy;               /*     0     8 */
	struct selinux_policy_convert_data * convert_data; /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct selinux_fs_info {
	struct dentry *            bool_dir;             /*     0     8 */
	unsigned int               bool_num;             /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	char * *                   bool_pending_names;   /*    16     8 */
	unsigned int *             bool_pending_values;  /*    24     8 */
	struct dentry *            class_dir;            /*    32     8 */
	long unsigned int          last_class_ino;       /*    40     8 */
	bool                       policy_opened;        /*    48     1 */

	/* XXX 7 bytes hole, try to pack */

	struct dentry *            policycap_dir;        /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	long unsigned int          last_ino;             /*    64     8 */
	struct selinux_state *     state;                /*    72     8 */
	struct super_block *       sb;                   /*    80     8 */

	/* size: 88, cachelines: 2, members: 11 */
	/* sum members: 77, holes: 2, sum holes: 11 */
	/* last cacheline: 24 bytes */
};
struct policy_load_memory {
	size_t                     len;                  /*     0     8 */
	void *                     data;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct selnl_msg_setenforce {
	__s32                      val;                  /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct selnl_msg_policyload {
	__u32                      seqno;                /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct nlmsg_perm {
	u16                        nlmsg_type;           /*     0     2 */

	/* XXX 2 bytes hole, try to pack */

	u32                        perm;                 /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* sum members: 6, holes: 1, sum holes: 2 */
	/* last cacheline: 8 bytes */
};
struct netif_security_struct {
	struct net *               ns;                   /*     0     8 */
	int                        ifindex;              /*     8     4 */
	u32                        sid;                  /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct sel_netif {
	struct list_head           list;                 /*     0    16 */
	struct netif_security_struct nsec;               /*    16    16 */
	struct callback_head       callback_head __attribute__((__aligned__(8))); /*    32    16 */

	/* size: 48, cachelines: 1, members: 3 */
	/* forced alignments: 1 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct netnode_security_struct {
	union {
		__be32             ipv4;                 /*     0     4 */
		struct in6_addr    ipv6;                 /*     0    16 */
	} addr;                                          /*     0    16 */
	u32                        sid;                  /*    16     4 */
	u16                        family;               /*    20     2 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 2 */
	/* last cacheline: 24 bytes */
};
struct sel_netnode_bkt {
	unsigned int               size;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           list;                 /*     8    16 */

	/* size: 24, cachelines: 1, members: 2 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct sel_netnode {
	struct netnode_security_struct nsec;             /*     0    24 */

	/* XXX last struct has 2 bytes of padding */

	struct list_head           list;                 /*    24    16 */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*    40    16 */

	/* size: 56, cachelines: 1, members: 3 */
	/* paddings: 1, sum paddings: 2 */
	/* forced alignments: 1 */
	/* last cacheline: 56 bytes */
} __attribute__((__aligned__(8)));
struct netport_security_struct {
	u32                        sid;                  /*     0     4 */
	u16                        port;                 /*     4     2 */
	u8                         protocol;             /*     6     1 */

	/* size: 8, cachelines: 1, members: 3 */
	/* padding: 1 */
	/* last cacheline: 8 bytes */
};
struct sel_netport_bkt {
	int                        size;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           list;                 /*     8    16 */

	/* size: 24, cachelines: 1, members: 2 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct sel_netport {
	struct netport_security_struct psec;             /*     0     8 */

	/* XXX last struct has 1 byte of padding */

	struct list_head           list;                 /*     8    16 */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*    24    16 */

	/* size: 40, cachelines: 1, members: 3 */
	/* paddings: 1, sum paddings: 1 */
	/* forced alignments: 1 */
	/* last cacheline: 40 bytes */
} __attribute__((__aligned__(8)));
struct selinux_kernel_status {
	u32                        version;              /*     0     4 */
	u32                        sequence;             /*     4     4 */
	u32                        enforcing;            /*     8     4 */
	u32                        policyload;           /*    12     4 */
	u32                        deny_unknown;         /*    16     4 */

	/* size: 20, cachelines: 1, members: 5 */
	/* last cacheline: 20 bytes */
};
struct ebitmap_node {
	struct ebitmap_node *      next;                 /*     0     8 */
	long unsigned int          maps[6];              /*     8    48 */
	u32                        startbit;             /*    56     4 */

	/* size: 64, cachelines: 1, members: 3 */
	/* padding: 4 */
};
struct ebitmap {
	struct ebitmap_node *      node;                 /*     0     8 */
	u32                        highbit;              /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct policy_file {
	char *                     data;                 /*     0     8 */
	size_t                     len;                  /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct hashtab_node {
	void *                     key;                  /*     0     8 */
	void *                     datum;                /*     8     8 */
	struct hashtab_node *      next;                 /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct hashtab {
	struct hashtab_node * *    htable;               /*     0     8 */
	u32                        size;                 /*     8     4 */
	u32                        nel;                  /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct hashtab_info {
	u32                        slots_used;           /*     0     4 */
	u32                        max_chain_len;        /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct hashtab_key_params {
	u32                        (*hash)(const void  *); /*     0     8 */
	int                        (*cmp)(const void  *, const void  *); /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct symtab {
	struct hashtab             table;                /*     0    16 */
	u32                        nprim;                /*    16     4 */

	/* size: 24, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct mls_level {
	u32                        sens;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct ebitmap             cat;                  /*     8    16 */

	/* XXX last struct has 4 bytes of padding */

	/* size: 24, cachelines: 1, members: 2 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 24 bytes */
};
struct mls_range {
	struct mls_level           level[2];             /*     0    48 */

	/* size: 48, cachelines: 1, members: 1 */
	/* last cacheline: 48 bytes */
};
struct context {
	u32                        user;                 /*     0     4 */
	u32                        role;                 /*     4     4 */
	u32                        type;                 /*     8     4 */
	u32                        len;                  /*    12     4 */
	struct mls_range           range;                /*    16    48 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	char *                     str;                  /*    64     8 */

	/* size: 72, cachelines: 2, members: 6 */
	/* last cacheline: 8 bytes */
};
struct sidtab_entry {
	u32                        sid;                  /*     0     4 */
	u32                        hash;                 /*     4     4 */
	struct context             context;              /*     8    72 */
	/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
	struct sidtab_str_cache *  cache;                /*    80     8 */
	struct hlist_node          list;                 /*    88    16 */

	/* size: 104, cachelines: 2, members: 5 */
	/* last cacheline: 40 bytes */
};
struct sidtab_str_cache {
	struct callback_head       rcu_member __attribute__((__aligned__(8))); /*     0    16 */
	struct list_head           lru_member;           /*    16    16 */
	struct sidtab_entry *      parent;               /*    32     8 */
	u32                        len;                  /*    40     4 */
	char                       str[];                /*    44     0 */

	/* size: 48, cachelines: 1, members: 5 */
	/* padding: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
union sidtab_entry_inner {
	struct sidtab_node_inner * ptr_inner;          /*     0     8 */
	struct sidtab_node_leaf *  ptr_leaf;           /*     0     8 */
};
struct sidtab_node_inner {
	union sidtab_entry_inner   entries[512];         /*     0  4096 */

	/* size: 4096, cachelines: 64, members: 1 */
};
struct sidtab_node_leaf {
	struct sidtab_entry        entries[39];          /*     0  4056 */

	/* size: 4056, cachelines: 64, members: 1 */
	/* last cacheline: 24 bytes */
};
struct sidtab_isid_entry {
	int                        set;                  /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct sidtab_entry        entry;                /*     8   104 */

	/* size: 112, cachelines: 2, members: 2 */
	/* sum members: 108, holes: 1, sum holes: 4 */
	/* last cacheline: 48 bytes */
};
struct sidtab_convert_params {
	int                        (*func)(struct context *, struct context *, void *); /*     0     8 */
	void *                     args;                 /*     8     8 */
	struct sidtab *            target;               /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct sidtab {
	union sidtab_entry_inner   roots[4];             /*     0    32 */
	u32                        count;                /*    32     4 */

	/* XXX 4 bytes hole, try to pack */

	struct sidtab_convert_params * convert;          /*    40     8 */
	bool                       frozen;               /*    48     1 */

	/* XXX 3 bytes hole, try to pack */

	spinlock_t                 lock;                 /*    52     4 */
	u32                        cache_free_slots;     /*    56     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	struct list_head           cache_lru_list;       /*    64    16 */
	spinlock_t                 cache_lock;           /*    80     4 */

	/* XXX 4 bytes hole, try to pack */

	struct sidtab_isid_entry   isids[27];            /*    88  3024 */
	/* --- cacheline 48 boundary (3072 bytes) was 40 bytes ago --- */
	struct hlist_head          context_to_sid[512];  /*  3112  4096 */

	/* size: 7208, cachelines: 113, members: 10 */
	/* sum members: 7193, holes: 4, sum holes: 15 */
	/* last cacheline: 40 bytes */
};
struct avtab_key {
	u16                        source_type;          /*     0     2 */
	u16                        target_type;          /*     2     2 */
	u16                        target_class;         /*     4     2 */
	u16                        specified;            /*     6     2 */

	/* size: 8, cachelines: 1, members: 4 */
	/* last cacheline: 8 bytes */
};
struct avtab_extended_perms {
	u8                         specified;            /*     0     1 */
	u8                         driver;               /*     1     1 */

	/* XXX 2 bytes hole, try to pack */

	struct extended_perms_data perms;                /*     4    32 */

	/* size: 36, cachelines: 1, members: 3 */
	/* sum members: 34, holes: 1, sum holes: 2 */
	/* last cacheline: 36 bytes */
};
struct avtab_datum {
	union {
		u32                data;                 /*     0     4 */
		struct avtab_extended_perms * xperms;    /*     0     8 */
	} u;                                             /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct avtab_node {
	struct avtab_key           key;                  /*     0     8 */
	struct avtab_datum         datum;                /*     8     8 */
	struct avtab_node *        next;                 /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct avtab {
	struct avtab_node * *      htable;               /*     0     8 */
	u32                        nel;                  /*     8     4 */
	u32                        nslot;                /*    12     4 */
	u32                        mask;                 /*    16     4 */

	/* size: 24, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct constraint_expr {
	u32                        expr_type;            /*     0     4 */
	u32                        attr;                 /*     4     4 */
	u32                        op;                   /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct ebitmap             names;                /*    16    16 */

	/* XXX last struct has 4 bytes of padding */

	struct type_set *          type_names;           /*    32     8 */
	struct constraint_expr *   next;                 /*    40     8 */

	/* size: 48, cachelines: 1, members: 6 */
	/* sum members: 44, holes: 1, sum holes: 4 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 48 bytes */
};
struct type_set {
	struct ebitmap             types;                /*     0    16 */

	/* XXX last struct has 4 bytes of padding */

	struct ebitmap             negset;               /*    16    16 */

	/* XXX last struct has 4 bytes of padding */

	u32                        flags;                /*    32     4 */

	/* size: 40, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* paddings: 2, sum paddings: 8 */
	/* last cacheline: 40 bytes */
};
struct constraint_node {
	u32                        permissions;          /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct constraint_expr *   expr;                 /*     8     8 */
	struct constraint_node *   next;                 /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct common_datum {
	u32                        value;                /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct symtab              permissions;          /*     8    24 */

	/* XXX last struct has 4 bytes of padding */

	/* size: 32, cachelines: 1, members: 2 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 32 bytes */
};
struct class_datum {
	u32                        value;                /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	char *                     comkey;               /*     8     8 */
	struct common_datum *      comdatum;             /*    16     8 */
	struct symtab              permissions;          /*    24    24 */

	/* XXX last struct has 4 bytes of padding */

	struct constraint_node *   constraints;          /*    48     8 */
	struct constraint_node *   validatetrans;        /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	char                       default_user;         /*    64     1 */
	char                       default_role;         /*    65     1 */
	char                       default_type;         /*    66     1 */
	char                       default_range;        /*    67     1 */

	/* size: 72, cachelines: 2, members: 10 */
	/* sum members: 64, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 8 bytes */
};
struct role_datum {
	u32                        value;                /*     0     4 */
	u32                        bounds;               /*     4     4 */
	struct ebitmap             dominates;            /*     8    16 */

	/* XXX last struct has 4 bytes of padding */

	struct ebitmap             types;                /*    24    16 */

	/* XXX last struct has 4 bytes of padding */

	/* size: 40, cachelines: 1, members: 4 */
	/* paddings: 2, sum paddings: 8 */
	/* last cacheline: 40 bytes */
};
struct role_allow {
	u32                        role;                 /*     0     4 */
	u32                        new_role;             /*     4     4 */
	struct role_allow *        next;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct type_datum {
	u32                        value;                /*     0     4 */
	u32                        bounds;               /*     4     4 */
	unsigned char              primary;              /*     8     1 */
	unsigned char              attribute;            /*     9     1 */

	/* size: 12, cachelines: 1, members: 4 */
	/* padding: 2 */
	/* last cacheline: 12 bytes */
};
struct user_datum {
	u32                        value;                /*     0     4 */
	u32                        bounds;               /*     4     4 */
	struct ebitmap             roles;                /*     8    16 */

	/* XXX last struct has 4 bytes of padding */

	struct mls_range           range;                /*    24    48 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	struct mls_level           dfltlevel;            /*    72    24 */

	/* size: 96, cachelines: 2, members: 5 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 32 bytes */
};
struct cond_bool_datum {
	__u32                      value;                /*     0     4 */
	int                        state;                /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct ocontext {
	union {
		char *             name;                 /*     0     8 */
		struct {
			u8         protocol;             /*     0     1 */

			/* XXX 1 byte hole, try to pack */

			u16        low_port;             /*     2     2 */
			u16        high_port;            /*     4     2 */
		} port;                                  /*     0     6 */
		struct {
			u32        addr;                 /*     0     4 */
			u32        mask;                 /*     4     4 */
		} node;                                  /*     0     8 */
		struct {
			u32        addr[4];              /*     0    16 */
			u32        mask[4];              /*    16    16 */
		} node6;                                 /*     0    32 */
		struct {
			u64        subnet_prefix;        /*     0     8 */
			u16        low_pkey;             /*     8     2 */
			u16        high_pkey;            /*    10     2 */
		} ibpkey;                                /*     0    16 */
		struct {
			char *     dev_name;             /*     0     8 */
			u8         port;                 /*     8     1 */
		} ibendport;                             /*     0    16 */
	} u;                                             /*     0    32 */
	union {
		u32                sclass;               /*    32     4 */
		u32                behavior;             /*    32     4 */
	} v;                                             /*    32     4 */

	/* XXX 4 bytes hole, try to pack */

	struct context             context[2];           /*    40   144 */
	/* --- cacheline 2 boundary (128 bytes) was 56 bytes ago --- */
	u32                        sid[2];               /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	struct ocontext *          next;                 /*   192     8 */

	/* size: 200, cachelines: 4, members: 5 */
	/* sum members: 196, holes: 1, sum holes: 4 */
	/* last cacheline: 8 bytes */
};
struct genfs {
	char *                     fstype;               /*     0     8 */
	struct ocontext *          head;                 /*     8     8 */
	struct genfs *             next;                 /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct policydb {
	int                        mls_enabled;          /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct symtab              symtab[8];            /*     8   192 */
	/* --- cacheline 3 boundary (192 bytes) was 8 bytes ago --- */
	char * *                   sym_val_to_name[8];   /*   200    64 */
	/* --- cacheline 4 boundary (256 bytes) was 8 bytes ago --- */
	struct class_datum * *     class_val_to_struct;  /*   264     8 */
	struct role_datum * *      role_val_to_struct;   /*   272     8 */
	struct user_datum * *      user_val_to_struct;   /*   280     8 */
	struct type_datum * *      type_val_to_struct;   /*   288     8 */
	struct avtab               te_avtab;             /*   296    24 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 5 boundary (320 bytes) --- */
	struct hashtab             role_tr;              /*   320    16 */
	struct ebitmap             filename_trans_ttypes; /*   336    16 */

	/* XXX last struct has 4 bytes of padding */

	struct hashtab             filename_trans;       /*   352    16 */
	u32                        compat_filename_trans_count; /*   368     4 */

	/* XXX 4 bytes hole, try to pack */

	struct cond_bool_datum * * bool_val_to_struct;   /*   376     8 */
	/* --- cacheline 6 boundary (384 bytes) --- */
	struct avtab               te_cond_avtab;        /*   384    24 */

	/* XXX last struct has 4 bytes of padding */

	struct cond_node *         cond_list;            /*   408     8 */
	u32                        cond_list_len;        /*   416     4 */

	/* XXX 4 bytes hole, try to pack */

	struct role_allow *        role_allow;           /*   424     8 */
	struct ocontext *          ocontexts[9];         /*   432    72 */
	/* --- cacheline 7 boundary (448 bytes) was 56 bytes ago --- */
	struct genfs *             genfs;                /*   504     8 */
	/* --- cacheline 8 boundary (512 bytes) --- */
	struct hashtab             range_tr;             /*   512    16 */
	struct ebitmap *           type_attr_map_array;  /*   528     8 */
	struct ebitmap             policycaps;           /*   536    16 */

	/* XXX last struct has 4 bytes of padding */

	struct ebitmap             permissive_map;       /*   552    16 */

	/* XXX last struct has 4 bytes of padding */

	size_t                     len;                  /*   568     8 */
	/* --- cacheline 9 boundary (576 bytes) --- */
	unsigned int               policyvers;           /*   576     4 */
	unsigned int               reject_unknown:1;     /*   580: 0  4 */
	unsigned int               allow_unknown:1;      /*   580: 1  4 */

	/* XXX 14 bits hole, try to pack */
	/* Bitfield combined with next fields */

	u16                        process_class;        /*   582     2 */
	u32                        process_trans_perms;  /*   584     4 */

	/* size: 592, cachelines: 10, members: 29 */
	/* sum members: 574, holes: 3, sum holes: 12 */
	/* sum bitfield members: 2 bits, bit holes: 1, sum bit holes: 14 bits */
	/* padding: 4 */
	/* paddings: 5, sum paddings: 20 */
	/* last cacheline: 16 bytes */
};
struct perm_datum {
	u32                        value;                /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct role_trans_key {
	u32                        role;                 /*     0     4 */
	u32                        type;                 /*     4     4 */
	u32                        tclass;               /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct role_trans_datum {
	u32                        new_role;             /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct filename_trans_key {
	u32                        ttype;                /*     0     4 */
	u16                        tclass;               /*     4     2 */

	/* XXX 2 bytes hole, try to pack */

	const char  *              name;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 3 */
	/* sum members: 14, holes: 1, sum holes: 2 */
	/* last cacheline: 16 bytes */
};
struct filename_trans_datum {
	struct ebitmap             stypes;               /*     0    16 */

	/* XXX last struct has 4 bytes of padding */

	u32                        otype;                /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	struct filename_trans_datum * next;              /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 32 bytes */
};
struct level_datum {
	struct mls_level *         level;                /*     0     8 */
	unsigned char              isalias;              /*     8     1 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 7 */
	/* last cacheline: 16 bytes */
};
struct cat_datum {
	u32                        value;                /*     0     4 */
	unsigned char              isalias;              /*     4     1 */

	/* size: 8, cachelines: 1, members: 2 */
	/* padding: 3 */
	/* last cacheline: 8 bytes */
};
struct range_trans {
	u32                        source_type;          /*     0     4 */
	u32                        target_type;          /*     4     4 */
	u32                        target_class;         /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct cond_node {
	int                        cur_state;            /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct cond_expr           expr;                 /*     8    16 */

	/* XXX last struct has 4 bytes of padding */

	struct cond_av_list        true_list;            /*    24    16 */

	/* XXX last struct has 4 bytes of padding */

	struct cond_av_list        false_list;           /*    40    16 */

	/* XXX last struct has 4 bytes of padding */

	/* size: 56, cachelines: 1, members: 4 */
	/* sum members: 52, holes: 1, sum holes: 4 */
	/* paddings: 3, sum paddings: 12 */
	/* last cacheline: 56 bytes */
};
struct policy_data {
	struct policydb *          p;                    /*     0     8 */
	void *                     fp;                   /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct cond_expr_node {
	u32                        expr_type;            /*     0     4 */
	u32                        bool;                 /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct cond_expr {
	struct cond_expr_node *    nodes;                /*     0     8 */
	u32                        len;                  /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct cond_av_list {
	struct avtab_node * *      nodes;                /*     0     8 */
	u32                        len;                  /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct policydb_compat_info {
	int                        version;              /*     0     4 */
	int                        sym_num;              /*     4     4 */
	int                        ocon_num;             /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct selinux_policy {
	struct sidtab *            sidtab;               /*     0     8 */
	struct policydb            policydb;             /*     8   592 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 9 boundary (576 bytes) was 24 bytes ago --- */
	struct selinux_map         map;                  /*   600    16 */

	/* XXX last struct has 6 bytes of padding */

	u32                        latest_granting;      /*   616     4 */

	/* size: 624, cachelines: 10, members: 4 */
	/* padding: 4 */
	/* paddings: 2, sum paddings: 10 */
	/* last cacheline: 48 bytes */
};
struct selinux_policy_convert_data {
	struct convert_context_args args;                /*     0    24 */
	struct sidtab_convert_params sidtab_params;      /*    24    24 */

	/* size: 48, cachelines: 1, members: 2 */
	/* last cacheline: 48 bytes */
};
struct selinux_mapping {
	u16                        value;                /*     0     2 */

	/* XXX 2 bytes hole, try to pack */

	unsigned int               num_perms;            /*     4     4 */
	u32                        perms[32];            /*     8   128 */

	/* size: 136, cachelines: 3, members: 3 */
	/* sum members: 134, holes: 1, sum holes: 2 */
	/* last cacheline: 8 bytes */
};
struct selinux_map {
	struct selinux_mapping *   mapping;              /*     0     8 */
	u16                        size;                 /*     8     2 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 6 */
	/* last cacheline: 16 bytes */
};
struct convert_context_args {
	struct selinux_state *     state;                /*     0     8 */
	struct policydb *          oldp;                 /*     8     8 */
	struct policydb *          newp;                 /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct selinux_audit_rule {
	u32                        au_seqno;             /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct context             au_ctxt;              /*     8    72 */

	/* size: 80, cachelines: 2, members: 2 */
	/* sum members: 76, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct cond_insertf_data {
	struct policydb *          p;                    /*     0     8 */
	struct avtab_node * *      dst;                  /*     8     8 */
	struct cond_av_list *      other;                /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct fib6_info {
	struct fib6_table *        fib6_table;           /*     0     8 */
	struct fib6_info *         fib6_next;            /*     8     8 */
	struct fib6_node *         fib6_node;            /*    16     8 */
	union {
		struct list_head   fib6_siblings;        /*    24    16 */
		struct list_head   nh_list;              /*    24    16 */
	};                                               /*    24    16 */
	unsigned int               fib6_nsiblings;       /*    40     4 */
	refcount_t                 fib6_ref;             /*    44     4 */
	long unsigned int          expires;              /*    48     8 */
	struct dst_metrics *       fib6_metrics;         /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct rt6key              fib6_dst;             /*    64    20 */
	u32                        fib6_flags;           /*    84     4 */
	struct rt6key              fib6_src;             /*    88    20 */
	struct rt6key              fib6_prefsrc;         /*   108    20 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	u32                        fib6_metric;          /*   128     4 */
	u8                         fib6_protocol;        /*   132     1 */
	u8                         fib6_type;            /*   133     1 */
	u8                         should_flush:1;       /*   134: 0  1 */
	u8                         dst_nocount:1;        /*   134: 1  1 */
	u8                         dst_nopolicy:1;       /*   134: 2  1 */
	u8                         fib6_destroying:1;    /*   134: 3  1 */
	u8                         offload:1;            /*   134: 4  1 */
	u8                         trap:1;               /*   134: 5  1 */
	u8                         unused:2;             /*   134: 6  1 */

	/* XXX 1 byte hole, try to pack */

	struct callback_head       rcu __attribute__((__aligned__(8))); /*   136    16 */
	struct nexthop *           nh;                   /*   152     8 */
	u64                        kabi_reserved1;       /*   160     8 */
	struct fib6_nh             fib6_nh[];            /*   168     0 */

	/* size: 168, cachelines: 3, members: 26 */
	/* sum members: 166, holes: 1, sum holes: 1 */
	/* sum bitfield members: 8 bits (1 bytes) */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 1 */
	/* last cacheline: 40 bytes */
} __attribute__((__aligned__(8)));
struct rt6_info {
	struct dst_entry           dst __attribute__((__aligned__(8))); /*     0   176 */
	/* --- cacheline 2 boundary (128 bytes) was 48 bytes ago --- */
	struct fib6_info *         from;                 /*   176     8 */
	int                        sernum;               /*   184     4 */
	struct rt6key              rt6i_dst;             /*   188    20 */
	/* --- cacheline 3 boundary (192 bytes) was 16 bytes ago --- */
	struct rt6key              rt6i_src;             /*   208    20 */
	struct in6_addr            rt6i_gateway;         /*   228    16 */

	/* XXX 4 bytes hole, try to pack */

	struct inet6_dev *         rt6i_idev;            /*   248     8 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	u32                        rt6i_flags;           /*   256     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           rt6i_uncached;        /*   264    16 */
	struct uncached_list *     rt6i_uncached_list;   /*   280     8 */
	short unsigned int         rt6i_nfheader_len;    /*   288     2 */

	/* XXX 6 bytes hole, try to pack */

	u64                        kabi_reserved1;       /*   296     8 */

	/* size: 304, cachelines: 5, members: 12 */
	/* sum members: 290, holes: 3, sum holes: 14 */
	/* forced alignments: 1 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct rt6_statistics {
	__u32                      fib_nodes;            /*     0     4 */
	__u32                      fib_route_nodes;      /*     4     4 */
	__u32                      fib_rt_entries;       /*     8     4 */
	__u32                      fib_rt_cache;         /*    12     4 */
	__u32                      fib_discarded_routes; /*    16     4 */
	atomic_t                   fib_rt_alloc;         /*    20     4 */
	atomic_t                   fib_rt_uncache;       /*    24     4 */

	/* size: 28, cachelines: 1, members: 7 */
	/* last cacheline: 28 bytes */
};
struct fib6_table {
	struct hlist_node          tb6_hlist;            /*     0    16 */
	u32                        tb6_id;               /*    16     4 */
	spinlock_t                 tb6_lock;             /*    20     4 */
	struct fib6_node           tb6_root __attribute__((__aligned__(8))); /*    24    72 */
	/* --- cacheline 1 boundary (64 bytes) was 32 bytes ago --- */
	struct inet_peer_base      tb6_peers;            /*    96    24 */

	/* XXX last struct has 4 bytes of padding */

	unsigned int               flags;                /*   120     4 */
	unsigned int               fib_seq;              /*   124     4 */

	/* size: 128, cachelines: 2, members: 7 */
	/* paddings: 1, sum paddings: 4 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(8)));
struct xfrm_id {
	xfrm_address_t             daddr;                /*     0    16 */
	__be32                     spi;                  /*    16     4 */
	__u8                       proto;                /*    20     1 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 3 */
	/* last cacheline: 24 bytes */
};
struct xfrm_selector {
	xfrm_address_t             daddr;                /*     0    16 */
	xfrm_address_t             saddr;                /*    16    16 */
	__be16                     dport;                /*    32     2 */
	__be16                     dport_mask;           /*    34     2 */
	__be16                     sport;                /*    36     2 */
	__be16                     sport_mask;           /*    38     2 */
	__u16                      family;               /*    40     2 */
	__u8                       prefixlen_d;          /*    42     1 */
	__u8                       prefixlen_s;          /*    43     1 */
	__u8                       proto;                /*    44     1 */

	/* XXX 3 bytes hole, try to pack */

	int                        ifindex;              /*    48     4 */
	__kernel_uid32_t           user;                 /*    52     4 */

	/* size: 56, cachelines: 1, members: 12 */
	/* sum members: 53, holes: 1, sum holes: 3 */
	/* last cacheline: 56 bytes */
};
struct xfrm_lifetime_cfg {
	__u64                      soft_byte_limit;      /*     0     8 */
	__u64                      hard_byte_limit;      /*     8     8 */
	__u64                      soft_packet_limit;    /*    16     8 */
	__u64                      hard_packet_limit;    /*    24     8 */
	__u64                      soft_add_expires_seconds; /*    32     8 */
	__u64                      hard_add_expires_seconds; /*    40     8 */
	__u64                      soft_use_expires_seconds; /*    48     8 */
	__u64                      hard_use_expires_seconds; /*    56     8 */

	/* size: 64, cachelines: 1, members: 8 */
};
struct xfrm_lifetime_cur {
	__u64                      bytes;                /*     0     8 */
	__u64                      packets;              /*     8     8 */
	__u64                      add_time;             /*    16     8 */
	__u64                      use_time;             /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct xfrm_replay_state {
	__u32                      oseq;                 /*     0     4 */
	__u32                      seq;                  /*     4     4 */
	__u32                      bitmap;               /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct xfrm_replay_state_esn {
	unsigned int               bmp_len;              /*     0     4 */
	__u32                      oseq;                 /*     4     4 */
	__u32                      seq;                  /*     8     4 */
	__u32                      oseq_hi;              /*    12     4 */
	__u32                      seq_hi;               /*    16     4 */
	__u32                      replay_window;        /*    20     4 */
	__u32                      bmp[];                /*    24     0 */

	/* size: 24, cachelines: 1, members: 7 */
	/* last cacheline: 24 bytes */
};
struct xfrm_algo {
	char                       alg_name[64];         /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	unsigned int               alg_key_len;          /*    64     4 */
	char                       alg_key[];            /*    68     0 */

	/* size: 68, cachelines: 2, members: 3 */
	/* last cacheline: 4 bytes */
};
struct xfrm_algo_auth {
	char                       alg_name[64];         /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	unsigned int               alg_key_len;          /*    64     4 */
	unsigned int               alg_trunc_len;        /*    68     4 */
	char                       alg_key[];            /*    72     0 */

	/* size: 72, cachelines: 2, members: 4 */
	/* last cacheline: 8 bytes */
};
struct xfrm_algo_aead {
	char                       alg_name[64];         /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	unsigned int               alg_key_len;          /*    64     4 */
	unsigned int               alg_icv_len;          /*    68     4 */
	char                       alg_key[];            /*    72     0 */

	/* size: 72, cachelines: 2, members: 4 */
	/* last cacheline: 8 bytes */
};
struct xfrm_stats {
	__u32                      replay_window;        /*     0     4 */
	__u32                      replay;               /*     4     4 */
	__u32                      integrity_failed;     /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct xfrm_encap_tmpl {
	__u16                      encap_type;           /*     0     2 */
	__be16                     encap_sport;          /*     2     2 */
	__be16                     encap_dport;          /*     4     2 */

	/* XXX 2 bytes hole, try to pack */

	xfrm_address_t             encap_oa;             /*     8    16 */

	/* size: 24, cachelines: 1, members: 4 */
	/* sum members: 22, holes: 1, sum holes: 2 */
	/* last cacheline: 24 bytes */
};
struct xfrm_mark {
	__u32                      v;                    /*     0     4 */
	__u32                      m;                    /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct xfrm_address_filter {
	xfrm_address_t             saddr;                /*     0    16 */
	xfrm_address_t             daddr;                /*    16    16 */
	__u16                      family;               /*    32     2 */
	__u8                       splen;                /*    34     1 */
	__u8                       dplen;                /*    35     1 */

	/* size: 36, cachelines: 1, members: 5 */
	/* last cacheline: 36 bytes */
};
struct xfrm_state {
	possible_net_t             xs_net;               /*     0     8 */
	union {
		struct hlist_node  gclist;               /*     8    16 */
		struct hlist_node  bydst;                /*     8    16 */
	};                                               /*     8    16 */
	struct hlist_node          bysrc;                /*    24    16 */
	struct hlist_node          byspi;                /*    40    16 */
	refcount_t                 refcnt;               /*    56     4 */
	spinlock_t                 lock;                 /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct xfrm_id             id;                   /*    64    24 */

	/* XXX last struct has 3 bytes of padding */

	struct xfrm_selector       sel;                  /*    88    56 */
	/* --- cacheline 2 boundary (128 bytes) was 16 bytes ago --- */
	struct xfrm_mark           mark;                 /*   144     8 */
	u32                        if_id;                /*   152     4 */
	u32                        tfcpad;               /*   156     4 */
	u32                        genid;                /*   160     4 */

	/* XXX 4 bytes hole, try to pack */

	struct xfrm_state_walk     km;                   /*   168    32 */
	/* --- cacheline 3 boundary (192 bytes) was 8 bytes ago --- */
	struct {
		u32                reqid;                /*   200     4 */
		u8                 mode;                 /*   204     1 */
		u8                 replay_window;        /*   205     1 */
		u8                 aalgo;                /*   206     1 */
		u8                 ealgo;                /*   207     1 */
		u8                 calgo;                /*   208     1 */
		u8                 flags;                /*   209     1 */
		u16                family;               /*   210     2 */
		xfrm_address_t     saddr;                /*   212    16 */
		int                header_len;           /*   228     4 */
		int                trailer_len;          /*   232     4 */
		u32                extra_flags;          /*   236     4 */
		struct xfrm_mark   smark;                /*   240     8 */
	} props;                                         /*   200    48 */
	struct xfrm_lifetime_cfg   lft;                  /*   248    64 */
	/* --- cacheline 4 boundary (256 bytes) was 56 bytes ago --- */
	struct xfrm_algo_auth *    aalg;                 /*   312     8 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	struct xfrm_algo *         ealg;                 /*   320     8 */
	struct xfrm_algo *         calg;                 /*   328     8 */
	struct xfrm_algo_aead *    aead;                 /*   336     8 */
	const char  *              geniv;                /*   344     8 */
	struct xfrm_encap_tmpl *   encap;                /*   352     8 */
	struct sock *              encap_sk;             /*   360     8 */
	xfrm_address_t *           coaddr;               /*   368     8 */
	struct xfrm_state *        tunnel;               /*   376     8 */
	/* --- cacheline 6 boundary (384 bytes) --- */
	atomic_t                   tunnel_users;         /*   384     4 */
	struct xfrm_replay_state   replay;               /*   388    12 */
	struct xfrm_replay_state_esn * replay_esn;       /*   400     8 */
	struct xfrm_replay_state   preplay;              /*   408    12 */

	/* XXX 4 bytes hole, try to pack */

	struct xfrm_replay_state_esn * preplay_esn;      /*   424     8 */
	const struct xfrm_replay  * repl;                /*   432     8 */
	u32                        xflags;               /*   440     4 */
	u32                        replay_maxage;        /*   444     4 */
	/* --- cacheline 7 boundary (448 bytes) --- */
	u32                        replay_maxdiff;       /*   448     4 */

	/* XXX 4 bytes hole, try to pack */

	struct timer_list          rtimer;               /*   456    72 */
	/* --- cacheline 8 boundary (512 bytes) was 16 bytes ago --- */
	struct xfrm_stats          stats;                /*   528    12 */

	/* XXX 4 bytes hole, try to pack */

	struct xfrm_lifetime_cur   curlft;               /*   544    32 */
	/* --- cacheline 9 boundary (576 bytes) --- */
	struct hrtimer             mtimer __attribute__((__aligned__(8))); /*   576    96 */
	/* --- cacheline 10 boundary (640 bytes) was 32 bytes ago --- */
	struct xfrm_state_offload  xso;                  /*   672    64 */
	/* --- cacheline 11 boundary (704 bytes) was 32 bytes ago --- */
	long int                   saved_tmo;            /*   736     8 */
	time64_t                   lastused;             /*   744     8 */
	struct page_frag           xfrag;                /*   752    16 */
	/* --- cacheline 12 boundary (768 bytes) --- */
	const struct xfrm_type  *  type;                 /*   768     8 */
	struct xfrm_mode           inner_mode;           /*   776     3 */
	struct xfrm_mode           inner_mode_iaf;       /*   779     3 */
	struct xfrm_mode           outer_mode;           /*   782     3 */

	/* XXX 7 bytes hole, try to pack */

	const struct xfrm_type_offload  * type_offload;  /*   792     8 */
	struct xfrm_sec_ctx *      security;             /*   800     8 */
	void *                     data;                 /*   808     8 */

	/* size: 816, cachelines: 13, members: 48 */
	/* sum members: 793, holes: 5, sum holes: 23 */
	/* paddings: 1, sum paddings: 3 */
	/* forced alignments: 1 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct dst_metrics {
	u32                        metrics[17];          /*     0    68 */
	/* --- cacheline 1 boundary (64 bytes) was 4 bytes ago --- */
	refcount_t                 refcnt;               /*    68     4 */

	/* size: 72, cachelines: 2, members: 2 */
	/* last cacheline: 8 bytes */
} __attribute__((__aligned__(4)));
struct xfrm_policy {
	possible_net_t             xp_net;               /*     0     8 */
	struct hlist_node          bydst;                /*     8    16 */
	struct hlist_node          byidx;                /*    24    16 */
	rwlock_t                   lock;                 /*    40     8 */
	refcount_t                 refcnt;               /*    48     4 */
	u32                        pos;                  /*    52     4 */
	struct timer_list          timer;                /*    56    72 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	atomic_t                   genid;                /*   128     4 */
	u32                        priority;             /*   132     4 */
	u32                        index;                /*   136     4 */
	u32                        if_id;                /*   140     4 */
	struct xfrm_mark           mark;                 /*   144     8 */
	struct xfrm_selector       selector;             /*   152    56 */
	/* --- cacheline 3 boundary (192 bytes) was 16 bytes ago --- */
	struct xfrm_lifetime_cfg   lft;                  /*   208    64 */
	/* --- cacheline 4 boundary (256 bytes) was 16 bytes ago --- */
	struct xfrm_lifetime_cur   curlft;               /*   272    32 */
	struct xfrm_policy_walk_entry walk;              /*   304    24 */

	/* XXX last struct has 7 bytes of padding */

	/* --- cacheline 5 boundary (320 bytes) was 8 bytes ago --- */
	struct xfrm_policy_queue   polq;                 /*   328   104 */
	/* --- cacheline 6 boundary (384 bytes) was 48 bytes ago --- */
	bool                       bydst_reinsert;       /*   432     1 */
	u8                         type;                 /*   433     1 */
	u8                         action;               /*   434     1 */
	u8                         flags;                /*   435     1 */
	u8                         xfrm_nr;              /*   436     1 */

	/* XXX 1 byte hole, try to pack */

	u16                        family;               /*   438     2 */
	struct xfrm_sec_ctx *      security;             /*   440     8 */
	/* --- cacheline 7 boundary (448 bytes) --- */
	struct xfrm_tmpl           xfrm_vec[6];          /*   448   384 */
	/* --- cacheline 13 boundary (832 bytes) --- */
	struct hlist_node          bydst_inexact_list;   /*   832    16 */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*   848    16 */

	/* size: 864, cachelines: 14, members: 27 */
	/* sum members: 863, holes: 1, sum holes: 1 */
	/* paddings: 1, sum paddings: 7 */
	/* forced alignments: 1 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct udp_table {
	struct udp_hslot *         hash;                 /*     0     8 */
	struct udp_hslot *         hash2;                /*     8     8 */
	unsigned int               mask;                 /*    16     4 */
	unsigned int               log;                  /*    20     4 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct fib6_nh {
	struct fib_nh_common       nh_common;            /*     0    72 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	long unsigned int          last_probe;           /*    72     8 */
	struct rt6_info * *        rt6i_pcpu;            /*    80     8 */
	struct rt6_exception_bucket * rt6i_exception_bucket; /*    88     8 */

	/* size: 96, cachelines: 2, members: 4 */
	/* last cacheline: 32 bytes */
};
struct fib_nh_exception {
	struct fib_nh_exception *  fnhe_next;            /*     0     8 */
	int                        fnhe_genid;           /*     8     4 */
	__be32                     fnhe_daddr;           /*    12     4 */
	u32                        fnhe_pmtu;            /*    16     4 */
	bool                       fnhe_mtu_locked;      /*    20     1 */

	/* XXX 3 bytes hole, try to pack */

	__be32                     fnhe_gw;              /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          fnhe_expires;         /*    32     8 */
	struct rtable *            fnhe_rth_input;       /*    40     8 */
	struct rtable *            fnhe_rth_output;      /*    48     8 */
	long unsigned int          fnhe_stamp;           /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*    64    16 */

	/* size: 80, cachelines: 2, members: 11 */
	/* sum members: 73, holes: 2, sum holes: 7 */
	/* forced alignments: 1 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct rtable {
	struct dst_entry           dst __attribute__((__aligned__(8))); /*     0   176 */
	/* --- cacheline 2 boundary (128 bytes) was 48 bytes ago --- */
	int                        rt_genid;             /*   176     4 */
	unsigned int               rt_flags;             /*   180     4 */
	__u16                      rt_type;              /*   184     2 */
	__u8                       rt_is_input;          /*   186     1 */
	__u8                       rt_uses_gateway;      /*   187     1 */
	int                        rt_iif;               /*   188     4 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	u8                         rt_gw_family;         /*   192     1 */

	/* XXX 3 bytes hole, try to pack */

	union {
		__be32             rt_gw4;               /*   196     4 */
		struct in6_addr    rt_gw6;               /*   196    16 */
	};                                               /*   196    16 */
	u32                        rt_mtu_locked:1;      /*   212: 0  4 */
	u32                        rt_pmtu:31;           /*   212: 1  4 */
	struct list_head           rt_uncached;          /*   216    16 */
	struct uncached_list *     rt_uncached_list;     /*   232     8 */

	/* size: 240, cachelines: 4, members: 13 */
	/* sum members: 233, holes: 1, sum holes: 3 */
	/* sum bitfield members: 32 bits (4 bytes) */
	/* forced alignments: 1 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct fnhe_hash_bucket {
	struct fib_nh_exception *  chain;                /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct fib_nh_common {
	struct net_device *        nhc_dev;              /*     0     8 */
	int                        nhc_oif;              /*     8     4 */
	unsigned char              nhc_scope;            /*    12     1 */
	u8                         nhc_family;           /*    13     1 */
	u8                         nhc_gw_family;        /*    14     1 */
	unsigned char              nhc_flags;            /*    15     1 */
	struct lwtunnel_state *    nhc_lwtstate;         /*    16     8 */
	union {
		__be32             ipv4;                 /*    24     4 */
		struct in6_addr    ipv6;                 /*    24    16 */
	} nhc_gw;                                        /*    24    16 */
	int                        nhc_weight;           /*    40     4 */
	atomic_t                   nhc_upper_bound;      /*    44     4 */
	struct rtable * *          nhc_pcpu_rth_output;  /*    48     8 */
	struct rtable *            nhc_rth_input;        /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct fnhe_hash_bucket *  nhc_exceptions;       /*    64     8 */

	/* size: 72, cachelines: 2, members: 13 */
	/* last cacheline: 8 bytes */
};
struct fib6_node {
	struct fib6_node *         parent;               /*     0     8 */
	struct fib6_node *         left;                 /*     8     8 */
	struct fib6_node *         right;                /*    16     8 */
	struct fib6_info *         leaf;                 /*    24     8 */
	__u16                      fn_bit;               /*    32     2 */
	__u16                      fn_flags;             /*    34     2 */
	int                        fn_sernum;            /*    36     4 */
	struct fib6_info *         rr_ptr;               /*    40     8 */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*    48    16 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u64                        kabi_reserved1;       /*    64     8 */

	/* size: 72, cachelines: 2, members: 10 */
	/* forced alignments: 1 */
	/* last cacheline: 8 bytes */
} __attribute__((__aligned__(8)));
struct rt6key {
	struct in6_addr            addr;                 /*     0    16 */
	int                        plen;                 /*    16     4 */

	/* size: 20, cachelines: 1, members: 2 */
	/* last cacheline: 20 bytes */
};
struct rt6_exception_bucket {
	struct hlist_head          chain;                /*     0     8 */
	int                        depth;                /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct xfrm_state_walk {
	struct list_head           all;                  /*     0    16 */
	u8                         state;                /*    16     1 */
	u8                         dying;                /*    17     1 */
	u8                         proto;                /*    18     1 */

	/* XXX 1 byte hole, try to pack */

	u32                        seq;                  /*    20     4 */
	struct xfrm_address_filter * filter;             /*    24     8 */

	/* size: 32, cachelines: 1, members: 6 */
	/* sum members: 31, holes: 1, sum holes: 1 */
	/* last cacheline: 32 bytes */
};
struct xfrm_state_offload {
	struct net_device *        dev;                  /*     0     8 */
	struct net_device *        real_dev;             /*     8     8 */
	long unsigned int          offload_handle;       /*    16     8 */
	unsigned int               num_exthdrs;          /*    24     4 */
	u8                         flags;                /*    28     1 */

	/* XXX 3 bytes hole, try to pack */

	u64                        kabi_reserved1;       /*    32     8 */
	u64                        kabi_reserved2;       /*    40     8 */
	u64                        kabi_reserved3;       /*    48     8 */
	u64                        kabi_reserved4;       /*    56     8 */

	/* size: 64, cachelines: 1, members: 9 */
	/* sum members: 61, holes: 1, sum holes: 3 */
};
struct xfrm_mode {
	u8                         encap;                /*     0     1 */
	u8                         family;               /*     1     1 */
	u8                         flags;                /*     2     1 */

	/* size: 3, cachelines: 1, members: 3 */
	/* last cacheline: 3 bytes */
};
struct xfrm_replay {
	void                       (*advance)(struct xfrm_state *, __be32); /*     0     8 */
	int                        (*check)(struct xfrm_state *, struct sk_buff *, __be32); /*     8     8 */
	int                        (*recheck)(struct xfrm_state *, struct sk_buff *, __be32); /*    16     8 */
	void                       (*notify)(struct xfrm_state *, int); /*    24     8 */
	int                        (*overflow)(struct xfrm_state *, struct sk_buff *); /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct xfrm_type {
	char *                     description;          /*     0     8 */
	struct module *            owner;                /*     8     8 */
	u8                         proto;                /*    16     1 */
	u8                         flags;                /*    17     1 */

	/* XXX 6 bytes hole, try to pack */

	int                        (*init_state)(struct xfrm_state *); /*    24     8 */
	void                       (*destructor)(struct xfrm_state *); /*    32     8 */
	int                        (*input)(struct xfrm_state *, struct sk_buff *); /*    40     8 */
	int                        (*output)(struct xfrm_state *, struct sk_buff *); /*    48     8 */
	int                        (*reject)(struct xfrm_state *, struct sk_buff *, const struct flowi  *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*hdr_offset)(struct xfrm_state *, struct sk_buff *, u8 * *); /*    64     8 */

	/* size: 72, cachelines: 2, members: 10 */
	/* sum members: 66, holes: 1, sum holes: 6 */
	/* last cacheline: 8 bytes */
};
struct xfrm_type_offload {
	char *                     description;          /*     0     8 */
	struct module *            owner;                /*     8     8 */
	u8                         proto;                /*    16     1 */

	/* XXX 7 bytes hole, try to pack */

	void                       (*encap)(struct xfrm_state *, struct sk_buff *); /*    24     8 */
	int                        (*input_tail)(struct xfrm_state *, struct sk_buff *); /*    32     8 */
	int                        (*xmit)(struct xfrm_state *, struct sk_buff *, netdev_features_t); /*    40     8 */

	/* size: 48, cachelines: 1, members: 6 */
	/* sum members: 41, holes: 1, sum holes: 7 */
	/* last cacheline: 48 bytes */
};
struct xfrm_dst {
	union {
		struct dst_entry   dst __attribute__((__aligned__(8))); /*     0   176 */
		struct rtable      rt __attribute__((__aligned__(8))); /*     0   240 */
		struct rt6_info    rt6 __attribute__((__aligned__(8))); /*     0   304 */
	} u __attribute__((__aligned__(8)));             /*     0   304 */
	/* --- cacheline 4 boundary (256 bytes) was 48 bytes ago --- */
	struct dst_entry *         route;                /*   304     8 */
	struct dst_entry *         child;                /*   312     8 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	struct dst_entry *         path;                 /*   320     8 */
	struct xfrm_policy *       pols[2];              /*   328    16 */
	int                        num_pols;             /*   344     4 */
	int                        num_xfrms;            /*   348     4 */
	u32                        xfrm_genid;           /*   352     4 */
	u32                        policy_genid;         /*   356     4 */
	u32                        route_mtu_cached;     /*   360     4 */
	u32                        child_mtu_cached;     /*   364     4 */
	u32                        route_cookie;         /*   368     4 */
	u32                        path_cookie;          /*   372     4 */

	/* size: 376, cachelines: 6, members: 13 */
	/* forced alignments: 1 */
	/* last cacheline: 56 bytes */
} __attribute__((__aligned__(8)));
struct xfrm_tmpl {
	struct xfrm_id             id;                   /*     0    24 */

	/* XXX last struct has 3 bytes of padding */

	xfrm_address_t             saddr;                /*    24    16 */
	short unsigned int         encap_family;         /*    40     2 */

	/* XXX 2 bytes hole, try to pack */

	u32                        reqid;                /*    44     4 */
	u8                         mode;                 /*    48     1 */
	u8                         share;                /*    49     1 */
	u8                         optional;             /*    50     1 */
	u8                         allalgs;              /*    51     1 */
	u32                        aalgos;               /*    52     4 */
	u32                        ealgos;               /*    56     4 */
	u32                        calgos;               /*    60     4 */

	/* size: 64, cachelines: 1, members: 11 */
	/* sum members: 62, holes: 1, sum holes: 2 */
	/* paddings: 1, sum paddings: 3 */
};
struct xfrm_policy_walk_entry {
	struct list_head           all;                  /*     0    16 */
	u8                         dead;                 /*    16     1 */

	/* size: 24, cachelines: 1, members: 2 */
	/* padding: 7 */
	/* last cacheline: 24 bytes */
};
struct xfrm_policy_queue {
	struct sk_buff_head        hold_queue;           /*     0    24 */
	struct timer_list          hold_timer;           /*    24    72 */
	/* --- cacheline 1 boundary (64 bytes) was 32 bytes ago --- */
	long unsigned int          timeout;              /*    96     8 */

	/* size: 104, cachelines: 2, members: 3 */
	/* last cacheline: 40 bytes */
};
struct xfrm_offload {
	struct {
		__u32              low;                  /*     0     4 */
		__u32              hi;                   /*     4     4 */
	} seq;                                           /*     0     8 */
	__u32                      flags;                /*     8     4 */
	__u32                      status;               /*    12     4 */
	__u8                       proto;                /*    16     1 */

	/* size: 20, cachelines: 1, members: 4 */
	/* padding: 3 */
	/* last cacheline: 20 bytes */
};
struct sec_path {
	int                        len;                  /*     0     4 */
	int                        olen;                 /*     4     4 */
	struct xfrm_state *        xvec[6];              /*     8    48 */
	struct xfrm_offload        ovec[1];              /*    56    20 */

	/* size: 80, cachelines: 2, members: 4 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct udp_hslot {
	struct hlist_head          head;                 /*     0     8 */
	int                        count;                /*     8     4 */
	spinlock_t                 lock;                 /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(16)));
struct pkey_security_struct {
	u64                        subnet_prefix;        /*     0     8 */
	u16                        pkey;                 /*     8     2 */

	/* XXX 2 bytes hole, try to pack */

	u32                        sid;                  /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* sum members: 14, holes: 1, sum holes: 2 */
	/* last cacheline: 16 bytes */
};
struct sel_ib_pkey_bkt {
	int                        size;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           list;                 /*     8    16 */

	/* size: 24, cachelines: 1, members: 2 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct sel_ib_pkey {
	struct pkey_security_struct psec;                /*     0    16 */
	struct list_head           list;                 /*    16    16 */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*    32    16 */

	/* size: 48, cachelines: 1, members: 3 */
	/* forced alignments: 1 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct sockaddr_un {
	__kernel_sa_family_t       sun_family;           /*     0     2 */
	char                       sun_path[108];        /*     2   108 */

	/* size: 110, cachelines: 2, members: 2 */
	/* last cacheline: 46 bytes */
};
struct unix_address {
	refcount_t                 refcnt;               /*     0     4 */
	int                        len;                  /*     4     4 */
	unsigned int               hash;                 /*     8     4 */
	struct sockaddr_un         name[];               /*    12     0 */

	/* size: 12, cachelines: 1, members: 4 */
	/* last cacheline: 12 bytes */
};
struct scm_stat {
	atomic_t                   nr_fds;               /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct unix_sock {
	struct sock                sk __attribute__((__aligned__(8))); /*     0   872 */
	/* --- cacheline 13 boundary (832 bytes) was 40 bytes ago --- */
	struct unix_address *      addr;                 /*   872     8 */
	struct path                path;                 /*   880    16 */
	/* --- cacheline 14 boundary (896 bytes) --- */
	struct mutex               iolock;               /*   896    32 */
	struct mutex               bindlock;             /*   928    32 */
	/* --- cacheline 15 boundary (960 bytes) --- */
	struct sock *              peer;                 /*   960     8 */
	struct list_head           link;                 /*   968    16 */
	atomic_long_t              inflight;             /*   984     8 */
	spinlock_t                 lock;                 /*   992     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          gc_flags;             /*  1000     8 */

	/* XXX 16 bytes hole, try to pack */

	/* --- cacheline 16 boundary (1024 bytes) --- */
	struct socket_wq           peer_wq __attribute__((__aligned__(64))); /*  1024    64 */

	/* XXX last struct has 8 bytes of padding */

	/* --- cacheline 17 boundary (1088 bytes) --- */
	wait_queue_entry_t         peer_wake;            /*  1088    40 */
	struct scm_stat            scm_stat;             /*  1128     4 */

	/* size: 1152, cachelines: 18, members: 13 */
	/* sum members: 1112, holes: 2, sum holes: 20 */
	/* padding: 20 */
	/* paddings: 1, sum paddings: 8 */
	/* forced alignments: 2, forced holes: 1, sum forced holes: 16 */
} __attribute__((__aligned__(64)));
struct z_stream_s {
	const Byte  *              next_in;              /*     0     8 */
	uLong                      avail_in;             /*     8     8 */
	uLong                      total_in;             /*    16     8 */
	Byte *                     next_out;             /*    24     8 */
	uLong                      avail_out;            /*    32     8 */
	uLong                      total_out;            /*    40     8 */
	char *                     msg;                  /*    48     8 */
	struct internal_state *    state;                /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	void *                     workspace;            /*    64     8 */
	int                        data_type;            /*    72     4 */

	/* XXX 4 bytes hole, try to pack */

	uLong                      adler;                /*    80     8 */
	uLong                      reserved;             /*    88     8 */

	/* size: 96, cachelines: 2, members: 12 */
	/* sum members: 92, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct internal_state {
	int                        dummy;                /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct aa_sfs_entry {
	const char  *              name;                 /*     0     8 */
	struct dentry *            dentry;               /*     8     8 */
	umode_t                    mode;                 /*    16     2 */

	/* XXX 2 bytes hole, try to pack */

	enum aa_sfs_type           v_type;               /*    20     4 */
	union {
		bool               boolean;              /*    24     1 */
		char *             string;               /*    24     8 */
		long unsigned int  u64;                  /*    24     8 */
		struct aa_sfs_entry * files;             /*    24     8 */
	} v;                                             /*    24     8 */
	const struct file_operations  * file_ops;        /*    32     8 */

	/* size: 40, cachelines: 1, members: 6 */
	/* sum members: 38, holes: 1, sum holes: 2 */
	/* last cacheline: 40 bytes */
};
struct table_header {
	u16                        td_id;                /*     0     2 */
	u16                        td_flags;             /*     2     2 */
	u32                        td_hilen;             /*     4     4 */
	u32                        td_lolen;             /*     8     4 */
	char                       td_data[];            /*    12     0 */

	/* size: 12, cachelines: 1, members: 5 */
	/* last cacheline: 12 bytes */
};
struct aa_dfa {
	struct kref                count;                /*     0     4 */
	u16                        flags;                /*     4     2 */

	/* XXX 2 bytes hole, try to pack */

	u32                        max_oob;              /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct table_header *      tables[8];            /*    16    64 */

	/* size: 80, cachelines: 2, members: 4 */
	/* sum members: 74, holes: 2, sum holes: 6 */
	/* last cacheline: 16 bytes */
};
struct aa_policy {
	const char  *              name;                 /*     0     8 */
	char *                     hname;                /*     8     8 */
	struct list_head           list;                 /*    16    16 */
	struct list_head           profiles;             /*    32    16 */

	/* size: 48, cachelines: 1, members: 4 */
	/* last cacheline: 48 bytes */
};
struct aa_labelset {
	rwlock_t                   lock;                 /*     0     8 */
	struct rb_root             root;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct aa_proxy {
	struct kref                count;                /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct aa_label *          label;                /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct aa_label {
	struct kref                count;                /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct rb_node             node __attribute__((__aligned__(8))); /*     8    24 */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*    32    16 */
	struct aa_proxy *          proxy;                /*    48     8 */
	char *                     hname;                /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	long int                   flags;                /*    64     8 */
	u32                        secid;                /*    72     4 */
	int                        size;                 /*    76     4 */
	struct aa_profile *        vec[];                /*    80     0 */

	/* size: 80, cachelines: 2, members: 9 */
	/* sum members: 76, holes: 1, sum holes: 4 */
	/* forced alignments: 2, forced holes: 1, sum forced holes: 4 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct label_it {
	int                        i;                    /*     0     4 */
	int                        j;                    /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct aa_profile {
	struct aa_policy           base;                 /*     0    48 */
	struct aa_profile *        parent;               /*    48     8 */
	struct aa_ns *             ns;                   /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	const char  *              rename;               /*    64     8 */
	const char  *              attach;               /*    72     8 */
	struct aa_dfa *            xmatch;               /*    80     8 */
	unsigned int               xmatch_len;           /*    88     4 */
	enum audit_mode            audit;                /*    92     4 */
	long int                   mode;                 /*    96     8 */
	u32                        path_flags;           /*   104     4 */

	/* XXX 4 bytes hole, try to pack */

	const char  *              disconnected;         /*   112     8 */
	int                        size;                 /*   120     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 2 boundary (128 bytes) --- */
	struct aa_policydb         policy;               /*   128    80 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 3 boundary (192 bytes) was 16 bytes ago --- */
	struct aa_file_rules       file;                 /*   208    32 */
	struct aa_caps             caps;                 /*   240    48 */
	/* --- cacheline 4 boundary (256 bytes) was 32 bytes ago --- */
	int                        xattr_count;          /*   288     4 */

	/* XXX 4 bytes hole, try to pack */

	char * *                   xattrs;               /*   296     8 */
	struct aa_rlimit           rlimits;              /*   304   264 */
	/* --- cacheline 8 boundary (512 bytes) was 56 bytes ago --- */
	int                        secmark_count;        /*   568     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 9 boundary (576 bytes) --- */
	struct aa_secmark *        secmark;              /*   576     8 */
	struct aa_loaddata *       rawdata;              /*   584     8 */
	unsigned char *            hash;                 /*   592     8 */
	char *                     dirname;              /*   600     8 */
	struct dentry *            dents[9];             /*   608    72 */
	/* --- cacheline 10 boundary (640 bytes) was 40 bytes ago --- */
	struct rhashtable *        data;                 /*   680     8 */
	struct aa_label            label __attribute__((__aligned__(8))); /*   688    80 */

	/* size: 768, cachelines: 12, members: 26 */
	/* sum members: 752, holes: 4, sum holes: 16 */
	/* paddings: 1, sum paddings: 4 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(8)));
struct aa_domain {
	int                        size;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	char * *                   table;                /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct aa_perms {
	u32                        allow;                /*     0     4 */
	u32                        audit;                /*     4     4 */
	u32                        deny;                 /*     8     4 */
	u32                        quiet;                /*    12     4 */
	u32                        kill;                 /*    16     4 */
	u32                        stop;                 /*    20     4 */
	u32                        complain;             /*    24     4 */
	u32                        cond;                 /*    28     4 */
	u32                        hide;                 /*    32     4 */
	u32                        prompt;               /*    36     4 */
	u16                        xindex;               /*    40     2 */

	/* size: 44, cachelines: 1, members: 11 */
	/* padding: 2 */
	/* last cacheline: 44 bytes */
};
struct path_cond {
	kuid_t                     uid;                  /*     0     4 */
	umode_t                    mode;                 /*     4     2 */

	/* size: 8, cachelines: 1, members: 2 */
	/* padding: 2 */
	/* last cacheline: 8 bytes */
};
struct aa_file_rules {
	unsigned int               start;                /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct aa_dfa *            dfa;                  /*     8     8 */
	struct aa_domain           trans;                /*    16    16 */

	/* size: 32, cachelines: 1, members: 3 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct aa_caps {
	kernel_cap_t               allow;                /*     0     8 */
	kernel_cap_t               audit;                /*     8     8 */
	kernel_cap_t               denied;               /*    16     8 */
	kernel_cap_t               quiet;                /*    24     8 */
	kernel_cap_t               kill;                 /*    32     8 */
	kernel_cap_t               extended;             /*    40     8 */

	/* size: 48, cachelines: 1, members: 6 */
	/* last cacheline: 48 bytes */
};
struct aa_secmark {
	u8                         audit;                /*     0     1 */
	u8                         deny;                 /*     1     1 */

	/* XXX 2 bytes hole, try to pack */

	u32                        secid;                /*     4     4 */
	char *                     label;                /*     8     8 */

	/* size: 16, cachelines: 1, members: 4 */
	/* sum members: 14, holes: 1, sum holes: 2 */
	/* last cacheline: 16 bytes */
};
struct aa_rlimit {
	unsigned int               mask;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct rlimit              limits[16];           /*     8   256 */

	/* size: 264, cachelines: 5, members: 2 */
	/* sum members: 260, holes: 1, sum holes: 4 */
	/* last cacheline: 8 bytes */
};
struct aa_policydb {
	struct aa_dfa *            dfa;                  /*     0     8 */
	unsigned int               start[17];            /*     8    68 */

	/* size: 80, cachelines: 2, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct aa_data {
	char *                     key;                  /*     0     8 */
	u32                        size;                 /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	char *                     data;                 /*    16     8 */
	struct rhash_head          head;                 /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct aa_ns {
	struct aa_policy           base;                 /*     0    48 */
	struct aa_ns *             parent;               /*    48     8 */
	struct mutex               lock;                 /*    56    32 */
	/* --- cacheline 1 boundary (64 bytes) was 24 bytes ago --- */
	struct aa_ns_acct          acct;                 /*    88    16 */
	struct aa_profile *        unconfined;           /*   104     8 */
	struct list_head           sub_ns;               /*   112    16 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	atomic_t                   uniq_null;            /*   128     4 */

	/* XXX 4 bytes hole, try to pack */

	long int                   uniq_id;              /*   136     8 */
	int                        level;                /*   144     4 */

	/* XXX 4 bytes hole, try to pack */

	long int                   revision;             /*   152     8 */
	wait_queue_head_t          wait;                 /*   160    24 */
	struct aa_labelset         labels;               /*   184    16 */
	/* --- cacheline 3 boundary (192 bytes) was 8 bytes ago --- */
	struct list_head           rawdata_list;         /*   200    16 */
	struct dentry *            dents[13];            /*   216   104 */

	/* size: 320, cachelines: 5, members: 14 */
	/* sum members: 312, holes: 2, sum holes: 8 */
};
struct aa_loaddata {
	struct kref                count;                /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           list;                 /*     8    16 */
	struct work_struct         work;                 /*    24    64 */
	/* --- cacheline 1 boundary (64 bytes) was 24 bytes ago --- */
	struct dentry *            dents[6];             /*    88    48 */
	/* --- cacheline 2 boundary (128 bytes) was 8 bytes ago --- */
	struct aa_ns *             ns;                   /*   136     8 */
	char *                     name;                 /*   144     8 */
	size_t                     size;                 /*   152     8 */
	size_t                     compressed_size;      /*   160     8 */
	long int                   revision;             /*   168     8 */
	int                        abi;                  /*   176     4 */

	/* XXX 4 bytes hole, try to pack */

	unsigned char *            hash;                 /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	char *                     data;                 /*   192     8 */

	/* size: 200, cachelines: 4, members: 12 */
	/* sum members: 192, holes: 2, sum holes: 8 */
	/* last cacheline: 8 bytes */
};
struct aa_ns_acct {
	int                        max_size;             /*     0     4 */
	int                        max_count;            /*     4     4 */
	int                        size;                 /*     8     4 */
	int                        count;                /*    12     4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct rawdata_f_data {
	struct aa_loaddata *       loaddata;             /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct aa_revision {
	struct aa_ns *             ns;                   /*     0     8 */
	long int                   last_read;            /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct multi_transaction {
	struct kref                count;                /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	ssize_t                    size;                 /*     8     8 */
	char                       data[];               /*    16     0 */

	/* size: 16, cachelines: 1, members: 3 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct apparmor_audit_data {
	int                        error;                /*     0     4 */
	int                        type;                 /*     4     4 */
	const char  *              op;                   /*     8     8 */
	struct aa_label *          label;                /*    16     8 */
	const char  *              name;                 /*    24     8 */
	const char  *              info;                 /*    32     8 */
	u32                        request;              /*    40     4 */
	u32                        denied;               /*    44     4 */
	union {
		struct {
			struct aa_label * peer;          /*    48     8 */
			union {
				struct {
					const char  * target; /*    56     8 */
					/* --- cacheline 1 boundary (64 bytes) --- */
					kuid_t ouid;     /*    64     4 */
				} fs;                    /*    56    16 */
				struct {
					int    rlim;     /*    56     4 */

					/* XXX 4 bytes hole, try to pack */

					/* --- cacheline 1 boundary (64 bytes) --- */
					long unsigned int max; /*    64     8 */
				} rlim;                  /*    56    16 */
				struct {
					int    signal;   /*    56     4 */
					int    unmappedsig; /*    60     4 */
				};                       /*    56     8 */
				struct {
					int    type;     /*    56     4 */
					int    protocol; /*    60     4 */
					/* --- cacheline 1 boundary (64 bytes) --- */
					struct sock * peer_sk; /*    64     8 */
					void * addr;     /*    72     8 */
					int    addrlen;  /*    80     4 */
				} net;                   /*    56    32 */
			};                               /*    56    32 */
		};                                       /*    48    40 */
		struct {
			struct aa_profile * profile;     /*    48     8 */
			const char  * ns;                /*    56     8 */
			/* --- cacheline 1 boundary (64 bytes) --- */
			long int   pos;                  /*    64     8 */
		} iface;                                 /*    48    24 */
		struct {
			const char  * src_name;          /*    48     8 */
			const char  * type;              /*    56     8 */
			/* --- cacheline 1 boundary (64 bytes) --- */
			const char  * trans;             /*    64     8 */
			const char  * data;              /*    72     8 */
			long unsigned int flags;         /*    80     8 */
		} mnt;                                   /*    48    40 */
	};                                               /*    48    40 */

	/* size: 88, cachelines: 2, members: 9 */
	/* last cacheline: 24 bytes */
};
struct aa_audit_rule {
	struct aa_label *          label;                /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct audit_cache {
	struct aa_profile *        profile;              /*     0     8 */
	kernel_cap_t               caps;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct aa_task_ctx {
	struct aa_label *          nnp;                  /*     0     8 */
	struct aa_label *          onexec;               /*     8     8 */
	struct aa_label *          previous;             /*    16     8 */
	u64                        token;                /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct counted_str {
	struct kref                count;                /*     0     4 */
	char                       name[];               /*     4     0 */

	/* size: 4, cachelines: 1, members: 2 */
	/* last cacheline: 4 bytes */
};
struct match_workbuf {
	unsigned int               count;                /*     0     4 */
	unsigned int               pos;                  /*     4     4 */
	unsigned int               len;                  /*     8     4 */
	unsigned int               size;                 /*    12     4 */
	unsigned int               history[24];          /*    16    96 */

	/* size: 112, cachelines: 2, members: 5 */
	/* last cacheline: 48 bytes */
};
struct aa_load_ent {
	struct list_head           list;                 /*     0    16 */
	struct aa_profile *        new;                  /*    16     8 */
	struct aa_profile *        old;                  /*    24     8 */
	struct aa_profile *        rename;               /*    32     8 */
	const char  *              ns_name;              /*    40     8 */

	/* size: 48, cachelines: 1, members: 5 */
	/* last cacheline: 48 bytes */
};
struct aa_ext {
	void *                     start;                /*     0     8 */
	void *                     end;                  /*     8     8 */
	void *                     pos;                  /*    16     8 */
	u32                        version;              /*    24     4 */

	/* size: 32, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct aa_file_ctx {
	spinlock_t                 lock;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct aa_label *          label;                /*     8     8 */
	u32                        allow;                /*    16     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 16, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct aa_sk_ctx {
	struct aa_label *          label;                /*     0     8 */
	struct aa_label *          peer;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
union aa_buffer {
	struct list_head           list;               /*     0    16 */
	char                       buffer[1];          /*     0     1 */
};
struct ptrace_relation {
	struct task_struct *       tracer;               /*     0     8 */
	struct task_struct *       tracee;               /*     8     8 */
	bool                       invalid;              /*    16     1 */

	/* XXX 7 bytes hole, try to pack */

	struct list_head           node;                 /*    24    16 */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*    40    16 */

	/* size: 56, cachelines: 1, members: 5 */
	/* sum members: 49, holes: 1, sum holes: 7 */
	/* forced alignments: 1 */
	/* last cacheline: 56 bytes */
} __attribute__((__aligned__(8)));
struct access_report_info {
	struct callback_head       work __attribute__((__aligned__(8))); /*     0    16 */
	const char  *              access;               /*    16     8 */
	struct task_struct *       target;               /*    24     8 */
	struct task_struct *       agent;                /*    32     8 */

	/* size: 40, cachelines: 1, members: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 40 bytes */
} __attribute__((__aligned__(8)));
struct dev_exception_item {
	u32                        major;                /*     0     4 */
	u32                        minor;                /*     4     4 */
	short int                  type;                 /*     8     2 */
	short int                  access;               /*    10     2 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           list;                 /*    16    16 */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*    32    16 */

	/* size: 48, cachelines: 1, members: 6 */
	/* sum members: 44, holes: 1, sum holes: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct dev_cgroup {
	struct cgroup_subsys_state css __attribute__((__aligned__(8))); /*     0   296 */
	/* --- cacheline 4 boundary (256 bytes) was 40 bytes ago --- */
	struct list_head           exceptions;           /*   296    16 */
	enum devcg_behavior        behavior;             /*   312     4 */

	/* size: 320, cachelines: 5, members: 3 */
	/* padding: 4 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(8)));
struct ima_digest_data {
	u8                         algo;                 /*     0     1 */
	u8                         length;               /*     1     1 */
	union {
		struct {
			u8         unused;               /*     2     1 */
			u8         type;                 /*     3     1 */
		} sha1;                                  /*     2     2 */
		struct {
			u8         type;                 /*     2     1 */
			u8         algo;                 /*     3     1 */
		} ng;                                    /*     2     2 */
		u8                 data[2];              /*     2     2 */
	} xattr;                                         /*     2     2 */
	u8                         digest[];             /*     4     0 */

	/* size: 4, cachelines: 1, members: 4 */
	/* last cacheline: 4 bytes */
};
struct integrity_iint_cache {
	struct rb_node             rb_node __attribute__((__aligned__(8))); /*     0    24 */
	struct mutex               mutex;                /*    24    32 */
	struct inode *             inode;                /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u64                        version;              /*    64     8 */
	long unsigned int          flags;                /*    72     8 */
	long unsigned int          measured_pcrs;        /*    80     8 */
	long unsigned int          atomic_flags;         /*    88     8 */
	enum integrity_status      ima_file_status:4;    /*    96: 0  4 */
	enum integrity_status      ima_mmap_status:4;    /*    96: 4  4 */
	enum integrity_status      ima_bprm_status:4;    /*    96: 8  4 */
	enum integrity_status      ima_read_status:4;    /*    96:12  4 */
	enum integrity_status      ima_creds_status:4;   /*    96:16  4 */
	enum integrity_status      evm_status:4;         /*    96:20  4 */

	/* XXX 8 bits hole, try to pack */
	/* XXX 4 bytes hole, try to pack */

	struct ima_digest_data *   ima_hash;             /*   104     8 */

	/* size: 112, cachelines: 2, members: 14 */
	/* sum members: 104, holes: 1, sum holes: 4 */
	/* sum bitfield members: 24 bits, bit holes: 1, sum bit holes: 8 bits */
	/* forced alignments: 1 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct public_key_signature {
	struct asymmetric_key_id * auth_ids[2];          /*     0    16 */
	u8 *                       s;                    /*    16     8 */
	u8 *                       digest;               /*    24     8 */
	u32                        s_size;               /*    32     4 */
	u32                        digest_size;          /*    36     4 */
	const char  *              pkey_algo;            /*    40     8 */
	const char  *              hash_algo;            /*    48     8 */
	const char  *              encoding;             /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	const void  *              data;                 /*    64     8 */
	unsigned int               data_size;            /*    72     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        kabi_reserved1;       /*    80     8 */
	u64                        kabi_reserved2;       /*    88     8 */
	u64                        kabi_reserved3;       /*    96     8 */

	/* size: 104, cachelines: 2, members: 13 */
	/* sum members: 100, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct asymmetric_key_id {
	short unsigned int         len;                  /*     0     2 */
	unsigned char              data[];               /*     2     0 */

	/* size: 2, cachelines: 1, members: 2 */
	/* last cacheline: 2 bytes */
};
struct signature_v2_hdr {
	uint8_t                    type;                 /*     0     1 */
	uint8_t                    version;              /*     1     1 */
	uint8_t                    hash_algo;            /*     2     1 */
	__be32                     keyid;                /*     3     4 */
	__be16                     sig_size;             /*     7     2 */
	uint8_t                    sig[];                /*     9     0 */

	/* size: 9, cachelines: 1, members: 6 */
	/* last cacheline: 9 bytes */
} __attribute__((__packed__));
struct efi_mokvar_table_entry {
	char                       name[256];            /*     0   256 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	u64                        data_size;            /*   256     8 */
	u8                         data[];               /*   264     0 */

	/* size: 264, cachelines: 5, members: 3 */
	/* last cacheline: 8 bytes */
};
struct evm_ima_xattr_data {
	u8                         type;                 /*     0     1 */
	u8                         data[];               /*     1     0 */

	/* size: 1, cachelines: 1, members: 2 */
	/* last cacheline: 1 bytes */
};
struct ima_event_data {
	struct integrity_iint_cache * iint;              /*     0     8 */
	struct file *              file;                 /*     8     8 */
	const unsigned char  *     filename;             /*    16     8 */
	struct evm_ima_xattr_data * xattr_value;         /*    24     8 */
	int                        xattr_len;            /*    32     4 */

	/* XXX 4 bytes hole, try to pack */

	const struct modsig  *     modsig;               /*    40     8 */
	const char  *              violation;            /*    48     8 */
	const void  *              buf;                  /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        buf_len;              /*    64     4 */

	/* size: 72, cachelines: 2, members: 9 */
	/* sum members: 64, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 8 bytes */
};
struct ima_field_data {
	u8 *                       data;                 /*     0     8 */
	u32                        len;                  /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct ima_template_field {
	const const char           field_id[16];         /*     0    16 */
	int                        (*field_init)(struct ima_event_data *, struct ima_field_data *); /*    16     8 */
	void                       (*field_show)(struct seq_file *, enum ima_show_type, struct ima_field_data *); /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* last cacheline: 32 bytes */
};
struct ima_template_desc {
	struct list_head           list;                 /*     0    16 */
	char *                     name;                 /*    16     8 */
	char *                     fmt;                  /*    24     8 */
	int                        num_fields;           /*    32     4 */

	/* XXX 4 bytes hole, try to pack */

	const struct ima_template_field  * * fields;     /*    40     8 */

	/* size: 48, cachelines: 1, members: 5 */
	/* sum members: 44, holes: 1, sum holes: 4 */
	/* last cacheline: 48 bytes */
};
struct ima_template_entry {
	int                        pcr;                  /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct tpm_digest *        digests;              /*     8     8 */
	struct ima_template_desc * template_desc;        /*    16     8 */
	u32                        template_data_len;    /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	struct ima_field_data      template_data[];      /*    32     0 */

	/* size: 32, cachelines: 1, members: 5 */
	/* sum members: 24, holes: 2, sum holes: 8 */
	/* last cacheline: 32 bytes */
};
struct ima_queue_entry {
	struct hlist_node          hnext;                /*     0    16 */
	struct list_head           later;                /*    16    16 */
	struct ima_template_entry * entry;               /*    32     8 */

	/* size: 40, cachelines: 1, members: 3 */
	/* last cacheline: 40 bytes */
};
struct ima_h_table {
	atomic_long_t              len;                  /*     0     8 */
	atomic_long_t              violations;           /*     8     8 */
	struct hlist_head          queue[1024];          /*    16  8192 */

	/* size: 8208, cachelines: 129, members: 3 */
	/* last cacheline: 16 bytes */
};
struct ima_digest {
	struct hlist_node          hnext;                /*     0    16 */
	enum hash_algo             algo;                 /*    16     4 */
	enum compact_types         type;                 /*    20     4 */
	u16                        modifiers;            /*    24     2 */
	u16                        count;                /*    26     2 */
	u8                         digest[];             /*    28     0 */

	/* size: 32, cachelines: 1, members: 6 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct hash_alg_common {
	unsigned int               digestsize;           /*     0     4 */
	unsigned int               statesize;            /*     4     4 */
	struct crypto_alg          base __attribute__((__aligned__(8))); /*     8   384 */

	/* size: 392, cachelines: 7, members: 3 */
	/* forced alignments: 1 */
	/* last cacheline: 8 bytes */
} __attribute__((__aligned__(8)));
struct ahash_request {
	struct crypto_async_request base;                /*     0    48 */

	/* XXX last struct has 4 bytes of padding */

	unsigned int               nbytes;               /*    48     4 */

	/* XXX 4 bytes hole, try to pack */

	struct scatterlist *       src;                  /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u8 *                       result;               /*    64     8 */
	void *                     priv;                 /*    72     8 */
	void *                     __ctx[] __attribute__((__aligned__(8))); /*    80     0 */

	/* size: 80, cachelines: 2, members: 6 */
	/* sum members: 76, holes: 1, sum holes: 4 */
	/* paddings: 1, sum paddings: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct crypto_ahash {
	int                        (*init)(struct ahash_request *); /*     0     8 */
	int                        (*update)(struct ahash_request *); /*     8     8 */
	int                        (*final)(struct ahash_request *); /*    16     8 */
	int                        (*finup)(struct ahash_request *); /*    24     8 */
	int                        (*digest)(struct ahash_request *); /*    32     8 */
	int                        (*export)(struct ahash_request *, void *); /*    40     8 */
	int                        (*import)(struct ahash_request *, const void  *); /*    48     8 */
	int                        (*setkey)(struct crypto_ahash *, const u8  *, unsigned int); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	unsigned int               reqsize;              /*    64     4 */

	/* XXX 4 bytes hole, try to pack */

	struct crypto_tfm          base __attribute__((__aligned__(8))); /*    72    24 */

	/* size: 96, cachelines: 2, members: 10 */
	/* sum members: 92, holes: 1, sum holes: 4 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 4 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct ima_algo_desc {
	struct crypto_shash *      tfm;                  /*     0     8 */
	enum hash_algo             algo;                 /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct ima_rule_opt_list {
	size_t                     count;                /*     0     8 */
	char *                     items[];              /*     8     0 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct ima_rule_entry {
	struct list_head           list;                 /*     0    16 */
	int                        action;               /*    16     4 */
	unsigned int               flags;                /*    20     4 */
	enum ima_hooks             func;                 /*    24     4 */
	int                        mask;                 /*    28     4 */
	long unsigned int          fsmagic;              /*    32     8 */
	uuid_t                     fsuuid;               /*    40    16 */
	kuid_t                     uid;                  /*    56     4 */
	kuid_t                     fowner;               /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	bool                       (*uid_op)(kuid_t, kuid_t); /*    64     8 */
	bool                       (*fowner_op)(kuid_t, kuid_t); /*    72     8 */
	int                        pcr;                  /*    80     4 */

	/* XXX 4 bytes hole, try to pack */

	struct {
		void *             rule;                 /*    88     8 */
		char *             args_p;               /*    96     8 */
		int                type;                 /*   104     4 */
	} lsm[6]; /*    88   144 */
	/* --- cacheline 3 boundary (192 bytes) was 40 bytes ago --- */
	char *                     fsname;               /*   232     8 */
	struct ima_rule_opt_list * keyrings;             /*   240     8 */
	struct ima_template_desc * template;             /*   248     8 */

	/* size: 256, cachelines: 4, members: 16 */
	/* sum members: 252, holes: 1, sum holes: 4 */
};
struct ima_kexec_hdr {
	u16                        version;              /*     0     2 */
	u16                        _reserved0;           /*     2     2 */
	u32                        _reserved1;           /*     4     4 */
	u64                        buffer_size;          /*     8     8 */
	u64                        count;                /*    16     8 */

	/* size: 24, cachelines: 1, members: 5 */
	/* last cacheline: 24 bytes */
};
struct ima_key_entry {
	struct list_head           list;                 /*     0    16 */
	void *                     payload;              /*    16     8 */
	size_t                     payload_len;          /*    24     8 */
	char *                     keyring_name;         /*    32     8 */

	/* size: 40, cachelines: 1, members: 4 */
	/* last cacheline: 40 bytes */
};
struct compact_list_hdr {
	u8                         version;              /*     0     1 */
	u8                         _reserved;            /*     1     1 */
	u16                        type;                 /*     2     2 */
	u16                        modifiers;            /*     4     2 */
	u16                        algo;                 /*     6     2 */
	u32                        count;                /*     8     4 */
	u32                        datalen;              /*    12     4 */

	/* size: 16, cachelines: 1, members: 7 */
	/* last cacheline: 16 bytes */
};
struct readdir_callback {
	struct dir_context         ctx;                  /*     0    16 */
	struct path *              path;                 /*    16     8 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct evm_xattr {
	struct evm_ima_xattr_data  data;                 /*     0     1 */
	u8                         digest[64];           /*     1    64 */

	/* size: 65, cachelines: 2, members: 2 */
	/* last cacheline: 1 bytes */
};
struct xattr_list {
	struct list_head           list;                 /*     0    16 */
	char *                     name;                 /*    16     8 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct evm_digest {
	struct ima_digest_data     hdr;                  /*     0     4 */
	char                       digest[64];           /*     4    64 */

	/* size: 68, cachelines: 2, members: 2 */
	/* last cacheline: 4 bytes */
};
struct crypto_larval {
	struct crypto_alg          alg __attribute__((__aligned__(8))); /*     0   384 */
	/* --- cacheline 6 boundary (384 bytes) --- */
	struct crypto_alg *        adult;                /*   384     8 */
	struct completion          completion;           /*   392    32 */
	u32                        mask;                 /*   424     4 */

	/* size: 432, cachelines: 7, members: 4 */
	/* padding: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct crypto_cipher {
	struct crypto_tfm          base __attribute__((__aligned__(8))); /*     0    24 */

	/* size: 24, cachelines: 1, members: 1 */
	/* forced alignments: 1 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct crypto_attr_alg {
	char                       name[128];            /*     0   128 */

	/* size: 128, cachelines: 2, members: 1 */
};
struct crypto_attr_type {
	u32                        type;                 /*     0     4 */
	u32                        mask;                 /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct crypto_attr_u32 {
	u32                        num;                  /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct rtattr {
	short unsigned int         rta_len;              /*     0     2 */
	short unsigned int         rta_type;             /*     2     2 */

	/* size: 4, cachelines: 1, members: 2 */
	/* last cacheline: 4 bytes */
};
struct crypto_queue {
	struct list_head           list;                 /*     0    16 */
	struct list_head *         backlog;              /*    16     8 */
	unsigned int               qlen;                 /*    24     4 */
	unsigned int               max_qlen;             /*    28     4 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct aead_instance {
	void                       (*free)(struct aead_instance *); /*     0     8 */
	union {
		struct {
			char       head[64];             /*     8    64 */
			/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
			struct crypto_instance base __attribute__((__aligned__(8))); /*    72   408 */
		} __attribute__((__aligned__(8))) s __attribute__((__aligned__(8)));     /*     8   472 */
		struct aead_alg    alg __attribute__((__aligned__(8))); /*     8   448 */
	} __attribute__((__aligned__(8)));               /*     8   472 */

	/* size: 480, cachelines: 8, members: 2 */
	/* forced alignments: 1 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct crypto_aead_spawn {
	struct crypto_spawn        base;                 /*     0    48 */

	/* XXX last struct has 2 bytes of padding */

	/* size: 48, cachelines: 1, members: 1 */
	/* paddings: 1, sum paddings: 2 */
	/* last cacheline: 48 bytes */
};
struct crypto_report_aead {
	char                       type[64];             /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	char                       geniv[64];            /*    64    64 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	unsigned int               blocksize;            /*   128     4 */
	unsigned int               maxauthsize;          /*   132     4 */
	unsigned int               ivsize;               /*   136     4 */

	/* size: 140, cachelines: 3, members: 5 */
	/* last cacheline: 12 bytes */
};
struct aead_geniv_ctx {
	spinlock_t                 lock;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct crypto_aead *       child;                /*     8     8 */
	struct crypto_sync_skcipher * sknull;            /*    16     8 */
	u8                         salt[] __attribute__((__aligned__(4))); /*    24     0 */

	/* size: 24, cachelines: 1, members: 4 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct rng_alg {
	int                        (*generate)(struct crypto_rng *, const u8  *, unsigned int, u8 *, unsigned int); /*     0     8 */
	int                        (*seed)(struct crypto_rng *, const u8  *, unsigned int); /*     8     8 */
	void                       (*set_ent)(struct crypto_rng *, const u8  *, unsigned int); /*    16     8 */
	unsigned int               seedsize;             /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	struct crypto_alg          base __attribute__((__aligned__(8))); /*    32   384 */

	/* size: 416, cachelines: 7, members: 5 */
	/* sum members: 412, holes: 1, sum holes: 4 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 4 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct crypto_rng {
	struct crypto_tfm          base __attribute__((__aligned__(8))); /*     0    24 */

	/* size: 24, cachelines: 1, members: 1 */
	/* forced alignments: 1 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct crypto_cipher_spawn {
	struct crypto_spawn        base;                 /*     0    48 */

	/* XXX last struct has 2 bytes of padding */

	/* size: 48, cachelines: 1, members: 1 */
	/* paddings: 1, sum paddings: 2 */
	/* last cacheline: 48 bytes */
};
struct crypto_sync_skcipher {
	struct crypto_skcipher     base __attribute__((__aligned__(8))); /*     0    32 */

	/* size: 32, cachelines: 1, members: 1 */
	/* forced alignments: 1 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct skcipher_instance {
	void                       (*free)(struct skcipher_instance *); /*     0     8 */
	union {
		struct {
			char       head[64];             /*     8    64 */
			/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
			struct crypto_instance base __attribute__((__aligned__(8))); /*    72   408 */
		} __attribute__((__aligned__(8))) s __attribute__((__aligned__(8)));     /*     8   472 */
		struct skcipher_alg alg __attribute__((__aligned__(8))); /*     8   448 */
	} __attribute__((__aligned__(8)));               /*     8   472 */

	/* size: 480, cachelines: 8, members: 2 */
	/* forced alignments: 1 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct crypto_skcipher_spawn {
	struct crypto_spawn        base;                 /*     0    48 */

	/* XXX last struct has 2 bytes of padding */

	/* size: 48, cachelines: 1, members: 1 */
	/* paddings: 1, sum paddings: 2 */
	/* last cacheline: 48 bytes */
};
struct skcipher_ctx_simple {
	struct crypto_cipher *     cipher;               /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct crypto_report_blkcipher {
	char                       type[64];             /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	char                       geniv[64];            /*    64    64 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	unsigned int               blocksize;            /*   128     4 */
	unsigned int               min_keysize;          /*   132     4 */
	unsigned int               max_keysize;          /*   136     4 */
	unsigned int               ivsize;               /*   140     4 */

	/* size: 144, cachelines: 3, members: 6 */
	/* last cacheline: 16 bytes */
};
struct skcipher_walk_buffer {
	struct list_head           entry;                /*     0    16 */
	struct scatter_walk        dst;                  /*    16    16 */

	/* XXX last struct has 4 bytes of padding */

	unsigned int               len;                  /*    32     4 */

	/* XXX 4 bytes hole, try to pack */

	u8 *                       data;                 /*    40     8 */
	u8                         buffer[];             /*    48     0 */

	/* size: 48, cachelines: 1, members: 5 */
	/* sum members: 44, holes: 1, sum holes: 4 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 48 bytes */
};
struct ahash_alg {
	int                        (*init)(struct ahash_request *); /*     0     8 */
	int                        (*update)(struct ahash_request *); /*     8     8 */
	int                        (*final)(struct ahash_request *); /*    16     8 */
	int                        (*finup)(struct ahash_request *); /*    24     8 */
	int                        (*digest)(struct ahash_request *); /*    32     8 */
	int                        (*export)(struct ahash_request *, void *); /*    40     8 */
	int                        (*import)(struct ahash_request *, const void  *); /*    48     8 */
	int                        (*setkey)(struct crypto_ahash *, const u8  *, unsigned int); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*init_tfm)(struct crypto_ahash *); /*    64     8 */
	void                       (*exit_tfm)(struct crypto_ahash *); /*    72     8 */
	struct hash_alg_common     halg __attribute__((__aligned__(8))); /*    80   392 */

	/* size: 472, cachelines: 8, members: 11 */
	/* forced alignments: 1 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct crypto_hash_walk {
	char *                     data;                 /*     0     8 */
	unsigned int               offset;               /*     8     4 */
	unsigned int               alignmask;            /*    12     4 */
	struct page *              pg;                   /*    16     8 */
	unsigned int               entrylen;             /*    24     4 */
	unsigned int               total;                /*    28     4 */
	struct scatterlist *       sg;                   /*    32     8 */
	unsigned int               flags;                /*    40     4 */

	/* size: 48, cachelines: 1, members: 8 */
	/* padding: 4 */
	/* last cacheline: 48 bytes */
};
struct ahash_instance {
	void                       (*free)(struct ahash_instance *); /*     0     8 */
	union {
		struct {
			char       head[88];             /*     8    88 */
			/* --- cacheline 1 boundary (64 bytes) was 32 bytes ago --- */
			struct crypto_instance base __attribute__((__aligned__(8))); /*    96   408 */
		} __attribute__((__aligned__(8))) s __attribute__((__aligned__(8)));     /*     8   496 */
		struct ahash_alg   alg __attribute__((__aligned__(8))); /*     8   472 */
	} __attribute__((__aligned__(8)));               /*     8   496 */

	/* size: 504, cachelines: 8, members: 2 */
	/* forced alignments: 1 */
	/* last cacheline: 56 bytes */
} __attribute__((__aligned__(8)));
struct crypto_ahash_spawn {
	struct crypto_spawn        base;                 /*     0    48 */

	/* XXX last struct has 2 bytes of padding */

	/* size: 48, cachelines: 1, members: 1 */
	/* paddings: 1, sum paddings: 2 */
	/* last cacheline: 48 bytes */
};
struct crypto_report_hash {
	char                       type[64];             /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	unsigned int               blocksize;            /*    64     4 */
	unsigned int               digestsize;           /*    68     4 */

	/* size: 72, cachelines: 2, members: 3 */
	/* last cacheline: 8 bytes */
};
struct ahash_request_priv {
	crypto_completion_t        complete;             /*     0     8 */
	void *                     data;                 /*     8     8 */
	u8 *                       result;               /*    16     8 */
	u32                        flags;                /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     ubuf[] __attribute__((__aligned__(8))); /*    32     0 */

	/* size: 32, cachelines: 1, members: 5 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 4 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct shash_instance {
	void                       (*free)(struct shash_instance *); /*     0     8 */
	union {
		struct {
			char       head[96];             /*     8    96 */
			/* --- cacheline 1 boundary (64 bytes) was 40 bytes ago --- */
			struct crypto_instance base __attribute__((__aligned__(8))); /*   104   408 */
		} __attribute__((__aligned__(8))) s __attribute__((__aligned__(8)));     /*     8   504 */
		struct shash_alg   alg __attribute__((__aligned__(8))); /*     8   480 */
	} __attribute__((__aligned__(8)));               /*     8   504 */

	/* size: 512, cachelines: 8, members: 2 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(8)));
struct crypto_shash_spawn {
	struct crypto_spawn        base;                 /*     0    48 */

	/* XXX last struct has 2 bytes of padding */

	/* size: 48, cachelines: 1, members: 1 */
	/* paddings: 1, sum paddings: 2 */
	/* last cacheline: 48 bytes */
};
struct crypto_report_akcipher {
	char                       type[64];             /*     0    64 */

	/* size: 64, cachelines: 1, members: 1 */
};
struct akcipher_request {
	struct crypto_async_request base;                /*     0    48 */

	/* XXX last struct has 4 bytes of padding */

	struct scatterlist *       src;                  /*    48     8 */
	struct scatterlist *       dst;                  /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	unsigned int               src_len;              /*    64     4 */
	unsigned int               dst_len;              /*    68     4 */
	void *                     __ctx[] __attribute__((__aligned__(8))); /*    72     0 */

	/* size: 72, cachelines: 2, members: 6 */
	/* paddings: 1, sum paddings: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 8 bytes */
} __attribute__((__aligned__(8)));
struct crypto_akcipher {
	struct crypto_tfm          base __attribute__((__aligned__(8))); /*     0    24 */

	/* size: 24, cachelines: 1, members: 1 */
	/* forced alignments: 1 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct akcipher_alg {
	int                        (*sign)(struct akcipher_request *); /*     0     8 */
	int                        (*verify)(struct akcipher_request *); /*     8     8 */
	int                        (*encrypt)(struct akcipher_request *); /*    16     8 */
	int                        (*decrypt)(struct akcipher_request *); /*    24     8 */
	int                        (*set_pub_key)(struct crypto_akcipher *, const void  *, unsigned int); /*    32     8 */
	int                        (*set_priv_key)(struct crypto_akcipher *, const void  *, unsigned int); /*    40     8 */
	unsigned int               (*max_size)(struct crypto_akcipher *); /*    48     8 */
	int                        (*init)(struct crypto_akcipher *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	void                       (*exit)(struct crypto_akcipher *); /*    64     8 */
	u64                        kabi_reserved1;       /*    72     8 */
	unsigned int               reqsize;              /*    80     4 */

	/* XXX 4 bytes hole, try to pack */

	struct crypto_alg          base __attribute__((__aligned__(8))); /*    88   384 */

	/* size: 472, cachelines: 8, members: 12 */
	/* sum members: 468, holes: 1, sum holes: 4 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 4 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct akcipher_instance {
	void                       (*free)(struct akcipher_instance *); /*     0     8 */
	union {
		struct {
			char       head[88];             /*     8    88 */
			/* --- cacheline 1 boundary (64 bytes) was 32 bytes ago --- */
			struct crypto_instance base __attribute__((__aligned__(8))); /*    96   408 */
		} __attribute__((__aligned__(8))) s __attribute__((__aligned__(8)));     /*     8   496 */
		struct akcipher_alg alg __attribute__((__aligned__(8))); /*     8   472 */
	} __attribute__((__aligned__(8)));               /*     8   496 */

	/* size: 504, cachelines: 8, members: 2 */
	/* forced alignments: 1 */
	/* last cacheline: 56 bytes */
} __attribute__((__aligned__(8)));
struct crypto_akcipher_spawn {
	struct crypto_spawn        base;                 /*     0    48 */

	/* XXX last struct has 2 bytes of padding */

	/* size: 48, cachelines: 1, members: 1 */
	/* paddings: 1, sum paddings: 2 */
	/* last cacheline: 48 bytes */
};
struct crypto_report_kpp {
	char                       type[64];             /*     0    64 */

	/* size: 64, cachelines: 1, members: 1 */
};
struct kpp_request {
	struct crypto_async_request base;                /*     0    48 */

	/* XXX last struct has 4 bytes of padding */

	struct scatterlist *       src;                  /*    48     8 */
	struct scatterlist *       dst;                  /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	unsigned int               src_len;              /*    64     4 */
	unsigned int               dst_len;              /*    68     4 */
	void *                     __ctx[] __attribute__((__aligned__(8))); /*    72     0 */

	/* size: 72, cachelines: 2, members: 6 */
	/* paddings: 1, sum paddings: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 8 bytes */
} __attribute__((__aligned__(8)));
struct crypto_kpp {
	struct crypto_tfm          base __attribute__((__aligned__(8))); /*     0    24 */

	/* size: 24, cachelines: 1, members: 1 */
	/* forced alignments: 1 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct kpp_alg {
	int                        (*set_secret)(struct crypto_kpp *, const void  *, unsigned int); /*     0     8 */
	int                        (*generate_public_key)(struct kpp_request *); /*     8     8 */
	int                        (*compute_shared_secret)(struct kpp_request *); /*    16     8 */
	unsigned int               (*max_size)(struct crypto_kpp *); /*    24     8 */
	int                        (*init)(struct crypto_kpp *); /*    32     8 */
	void                       (*exit)(struct crypto_kpp *); /*    40     8 */
	unsigned int               reqsize;              /*    48     4 */

	/* XXX 4 bytes hole, try to pack */

	struct crypto_alg          base __attribute__((__aligned__(8))); /*    56   384 */

	/* size: 440, cachelines: 7, members: 8 */
	/* sum members: 436, holes: 1, sum holes: 4 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 4 */
	/* last cacheline: 56 bytes */
} __attribute__((__aligned__(8)));
struct asn1_decoder {
	const unsigned char  *     machine;              /*     0     8 */
	size_t                     machlen;              /*     8     8 */
	const asn1_action_t  *     actions;              /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct gcry_mpi {
	int                        alloced;              /*     0     4 */
	int                        nlimbs;               /*     4     4 */
	int                        nbits;                /*     8     4 */
	int                        sign;                 /*    12     4 */
	unsigned int               flags;                /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	mpi_limb_t *               d;                    /*    24     8 */

	/* size: 32, cachelines: 1, members: 6 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct rsa_key {
	const u8  *                n;                    /*     0     8 */
	const u8  *                e;                    /*     8     8 */
	const u8  *                d;                    /*    16     8 */
	const u8  *                p;                    /*    24     8 */
	const u8  *                q;                    /*    32     8 */
	const u8  *                dp;                   /*    40     8 */
	const u8  *                dq;                   /*    48     8 */
	const u8  *                qinv;                 /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	size_t                     n_sz;                 /*    64     8 */
	size_t                     e_sz;                 /*    72     8 */
	size_t                     d_sz;                 /*    80     8 */
	size_t                     p_sz;                 /*    88     8 */
	size_t                     q_sz;                 /*    96     8 */
	size_t                     dp_sz;                /*   104     8 */
	size_t                     dq_sz;                /*   112     8 */
	size_t                     qinv_sz;              /*   120     8 */

	/* size: 128, cachelines: 2, members: 16 */
};
struct rsa_mpi_key {
	MPI                        n;                    /*     0     8 */
	MPI                        e;                    /*     8     8 */
	MPI                        d;                    /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct rsa_asn1_template {
	const char  *              name;                 /*     0     8 */
	const u8  *                data;                 /*     8     8 */
	size_t                     size;                 /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct pkcs1pad_ctx {
	struct crypto_akcipher *   child;                /*     0     8 */
	unsigned int               key_size;             /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct pkcs1pad_inst_ctx {
	struct crypto_akcipher_spawn spawn;              /*     0    48 */
	const struct rsa_asn1_template  * digest_info;   /*    48     8 */

	/* size: 56, cachelines: 1, members: 2 */
	/* last cacheline: 56 bytes */
};
struct pkcs1pad_request {
	struct scatterlist         in_sg[2];             /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct scatterlist         out_sg[1];            /*    64    32 */
	uint8_t *                  in_buf;               /*    96     8 */
	uint8_t *                  out_buf;              /*   104     8 */
	struct akcipher_request    child_req __attribute__((__aligned__(8))); /*   112    72 */

	/* size: 184, cachelines: 3, members: 5 */
	/* forced alignments: 1 */
	/* last cacheline: 56 bytes */
} __attribute__((__aligned__(8)));
struct gcry_mpi_point {
	MPI                        x;                    /*     0     8 */
	MPI                        y;                    /*     8     8 */
	MPI                        z;                    /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct mpi_ec_ctx {
	enum gcry_mpi_ec_models    model;                /*     0     4 */
	enum ecc_dialects          dialect;              /*     4     4 */
	int                        flags;                /*     8     4 */
	unsigned int               nbits;                /*    12     4 */
	MPI                        p;                    /*    16     8 */
	MPI                        a;                    /*    24     8 */
	MPI                        b;                    /*    32     8 */
	MPI_POINT                  G;                    /*    40     8 */
	MPI                        n;                    /*    48     8 */
	unsigned int               h;                    /*    56     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	MPI_POINT                  Q;                    /*    64     8 */
	MPI                        d;                    /*    72     8 */
	const char  *              name;                 /*    80     8 */
	struct {
		struct {
			unsigned int a_is_pminus3:1;     /*    88: 0  4 */
			unsigned int two_inv_p:1;        /*    88: 1  4 */
		} valid;                                 /*    88     4 */
		int                a_is_pminus3;         /*    92     4 */
		MPI                two_inv_p;            /*    96     8 */
		mpi_barrett_t      p_barrett;            /*   104     8 */
		MPI                scratch[11];          /*   112    88 */
	} t;                                             /*    88   112 */
	/* --- cacheline 3 boundary (192 bytes) was 8 bytes ago --- */
	void                       (*addm)(MPI, MPI, MPI, struct mpi_ec_ctx *); /*   200     8 */
	void                       (*subm)(MPI, MPI, MPI, struct mpi_ec_ctx *); /*   208     8 */
	void                       (*mulm)(MPI, MPI, MPI, struct mpi_ec_ctx *); /*   216     8 */
	void                       (*pow2)(MPI, const MPI , struct mpi_ec_ctx *); /*   224     8 */
	void                       (*mul2)(MPI, MPI, struct mpi_ec_ctx *); /*   232     8 */

	/* size: 240, cachelines: 4, members: 19 */
	/* sum members: 236, holes: 1, sum holes: 4 */
	/* last cacheline: 48 bytes */
};
struct sm3_state {
	u32                        state[8];             /*     0    32 */
	u64                        count;                /*    32     8 */
	u8                         buffer[64];           /*    40    64 */

	/* size: 104, cachelines: 2, members: 3 */
	/* last cacheline: 40 bytes */
};
struct ecc_domain_parms {
	const char  *              desc;                 /*     0     8 */
	unsigned int               nbits;                /*     8     4 */
	unsigned int               fips:1;               /*    12: 0  4 */

	/* XXX 31 bits hole, try to pack */

	enum gcry_mpi_ec_models    model;                /*    16     4 */
	enum ecc_dialects          dialect;              /*    20     4 */
	const char  *              p;                    /*    24     8 */
	const char  *              a;                    /*    32     8 */
	const char  *              b;                    /*    40     8 */
	const char  *              n;                    /*    48     8 */
	const char  *              g_x;                  /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	const char  *              g_y;                  /*    64     8 */
	unsigned int               h;                    /*    72     4 */

	/* size: 80, cachelines: 2, members: 12 */
	/* sum members: 72 */
	/* sum bitfield members: 1 bits, bit holes: 1, sum bit holes: 31 bits */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct sm2_signature_ctx {
	MPI                        sig_r;                /*     0     8 */
	MPI                        sig_s;                /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct crypto_report_acomp {
	char                       type[64];             /*     0    64 */

	/* size: 64, cachelines: 1, members: 1 */
};
struct acomp_alg {
	int                        (*compress)(struct acomp_req *); /*     0     8 */
	int                        (*decompress)(struct acomp_req *); /*     8     8 */
	void                       (*dst_free)(struct scatterlist *); /*    16     8 */
	int                        (*init)(struct crypto_acomp *); /*    24     8 */
	void                       (*exit)(struct crypto_acomp *); /*    32     8 */
	unsigned int               reqsize;              /*    40     4 */

	/* XXX 4 bytes hole, try to pack */

	struct crypto_alg          base __attribute__((__aligned__(8))); /*    48   384 */

	/* size: 432, cachelines: 7, members: 7 */
	/* sum members: 428, holes: 1, sum holes: 4 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 4 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct crypto_report_comp {
	char                       type[64];             /*     0    64 */

	/* size: 64, cachelines: 1, members: 1 */
};
struct crypto_scomp {
	struct crypto_tfm          base __attribute__((__aligned__(8))); /*     0    24 */

	/* size: 24, cachelines: 1, members: 1 */
	/* forced alignments: 1 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct scomp_alg {
	void *                     (*alloc_ctx)(struct crypto_scomp *); /*     0     8 */
	void                       (*free_ctx)(struct crypto_scomp *, void *); /*     8     8 */
	int                        (*compress)(struct crypto_scomp *, const u8  *, unsigned int, u8 *, unsigned int *, void *); /*    16     8 */
	int                        (*decompress)(struct crypto_scomp *, const u8  *, unsigned int, u8 *, unsigned int *, void *); /*    24     8 */
	struct crypto_alg          base __attribute__((__aligned__(8))); /*    32   384 */

	/* size: 416, cachelines: 7, members: 5 */
	/* forced alignments: 1 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct scomp_scratch {
	spinlock_t                 lock;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     src;                  /*     8     8 */
	void *                     dst;                  /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct cryptomgr_param {
	struct rtattr *            tb[34];               /*     0   272 */
	/* --- cacheline 4 boundary (256 bytes) was 16 bytes ago --- */
	struct {
		struct rtattr      attr;                 /*   272     4 */
		struct crypto_attr_type data;            /*   276     8 */
	} type;                                          /*   272    12 */
	union {
		struct rtattr      attr;                 /*   284     4 */
		struct {
			struct rtattr attr;              /*   284     4 */
			struct crypto_attr_alg data;     /*   288   128 */
		} alg;                                   /*   284   132 */
		struct {
			struct rtattr attr;              /*   284     4 */
			struct crypto_attr_u32 data;     /*   288     4 */
		} nu32;                                  /*   284     8 */
	} attrs[32]; /*   284  4224 */
	/* --- cacheline 70 boundary (4480 bytes) was 28 bytes ago --- */
	char                       template[128];        /*  4508   128 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 72 boundary (4608 bytes) was 32 bytes ago --- */
	struct crypto_larval *     larval;               /*  4640     8 */
	u32                        otype;                /*  4648     4 */
	u32                        omask;                /*  4652     4 */

	/* size: 4656, cachelines: 73, members: 7 */
	/* sum members: 4652, holes: 1, sum holes: 4 */
	/* last cacheline: 48 bytes */
};
struct crypto_test_param {
	char                       driver[128];          /*     0   128 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	char                       alg[128];             /*   128   128 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	u32                        type;                 /*   256     4 */

	/* size: 260, cachelines: 5, members: 3 */
	/* last cacheline: 4 bytes */
};
struct drbg_string {
	const unsigned char  *     buf;                  /*     0     8 */
	size_t                     len;                  /*     8     8 */
	struct list_head           list;                 /*    16    16 */

	/* size: 32, cachelines: 1, members: 3 */
	/* last cacheline: 32 bytes */
};
struct drbg_test_data {
	struct drbg_string *       testentropy;          /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct hash_testvec {
	const char  *              key;                  /*     0     8 */
	const char  *              plaintext;            /*     8     8 */
	const char  *              digest;               /*    16     8 */
	unsigned int               psize;                /*    24     4 */
	short unsigned int         ksize;                /*    28     2 */

	/* XXX 2 bytes hole, try to pack */

	int                        setkey_error;         /*    32     4 */
	int                        digest_error;         /*    36     4 */

	/* size: 40, cachelines: 1, members: 7 */
	/* sum members: 38, holes: 1, sum holes: 2 */
	/* last cacheline: 40 bytes */
};
struct cipher_testvec {
	const char  *              key;                  /*     0     8 */
	const char  *              iv;                   /*     8     8 */
	const char  *              iv_out;               /*    16     8 */
	const char  *              ptext;                /*    24     8 */
	const char  *              ctext;                /*    32     8 */
	unsigned char              wk;                   /*    40     1 */

	/* XXX 1 byte hole, try to pack */

	short unsigned int         klen;                 /*    42     2 */
	unsigned int               len;                  /*    44     4 */
	bool                       fips_skip;            /*    48     1 */
	bool                       generates_iv;         /*    49     1 */

	/* XXX 2 bytes hole, try to pack */

	int                        setkey_error;         /*    52     4 */
	int                        crypt_error;          /*    56     4 */

	/* size: 64, cachelines: 1, members: 12 */
	/* sum members: 57, holes: 2, sum holes: 3 */
	/* padding: 4 */
};
struct aead_testvec {
	const char  *              key;                  /*     0     8 */
	const char  *              iv;                   /*     8     8 */
	const char  *              ptext;                /*    16     8 */
	const char  *              assoc;                /*    24     8 */
	const char  *              ctext;                /*    32     8 */
	unsigned char              novrfy;               /*    40     1 */
	unsigned char              wk;                   /*    41     1 */
	unsigned char              klen;                 /*    42     1 */

	/* XXX 1 byte hole, try to pack */

	unsigned int               plen;                 /*    44     4 */
	unsigned int               clen;                 /*    48     4 */
	unsigned int               alen;                 /*    52     4 */
	int                        setkey_error;         /*    56     4 */
	int                        setauthsize_error;    /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        crypt_error;          /*    64     4 */

	/* size: 72, cachelines: 2, members: 14 */
	/* sum members: 67, holes: 1, sum holes: 1 */
	/* padding: 4 */
	/* last cacheline: 8 bytes */
};
struct cprng_testvec {
	const char  *              key;                  /*     0     8 */
	const char  *              dt;                   /*     8     8 */
	const char  *              v;                    /*    16     8 */
	const char  *              result;               /*    24     8 */
	unsigned char              klen;                 /*    32     1 */

	/* XXX 1 byte hole, try to pack */

	short unsigned int         dtlen;                /*    34     2 */
	short unsigned int         vlen;                 /*    36     2 */
	short unsigned int         rlen;                 /*    38     2 */
	short unsigned int         loops;                /*    40     2 */

	/* size: 48, cachelines: 1, members: 9 */
	/* sum members: 41, holes: 1, sum holes: 1 */
	/* padding: 6 */
	/* last cacheline: 48 bytes */
};
struct drbg_testvec {
	const unsigned char  *     entropy;              /*     0     8 */
	size_t                     entropylen;           /*     8     8 */
	const unsigned char  *     entpra;               /*    16     8 */
	const unsigned char  *     entprb;               /*    24     8 */
	size_t                     entprlen;             /*    32     8 */
	const unsigned char  *     addtla;               /*    40     8 */
	const unsigned char  *     addtlb;               /*    48     8 */
	size_t                     addtllen;             /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	const unsigned char  *     pers;                 /*    64     8 */
	size_t                     perslen;              /*    72     8 */
	const unsigned char  *     expected;             /*    80     8 */
	size_t                     expectedlen;          /*    88     8 */

	/* size: 96, cachelines: 2, members: 12 */
	/* last cacheline: 32 bytes */
};
struct akcipher_testvec {
	const unsigned char  *     key;                  /*     0     8 */
	const unsigned char  *     params;               /*     8     8 */
	const unsigned char  *     m;                    /*    16     8 */
	const unsigned char  *     c;                    /*    24     8 */
	unsigned int               key_len;              /*    32     4 */
	unsigned int               param_len;            /*    36     4 */
	unsigned int               m_size;               /*    40     4 */
	unsigned int               c_size;               /*    44     4 */
	bool                       public_key_vec;       /*    48     1 */
	bool                       siggen_sigver_test;   /*    49     1 */

	/* XXX 2 bytes hole, try to pack */

	enum OID                   algo;                 /*    52     4 */

	/* size: 56, cachelines: 1, members: 11 */
	/* sum members: 54, holes: 1, sum holes: 2 */
	/* last cacheline: 56 bytes */
};
struct kpp_testvec {
	const unsigned char  *     secret;               /*     0     8 */
	const unsigned char  *     b_secret;             /*     8     8 */
	const unsigned char  *     b_public;             /*    16     8 */
	const unsigned char  *     expected_a_public;    /*    24     8 */
	const unsigned char  *     expected_ss;          /*    32     8 */
	short unsigned int         secret_size;          /*    40     2 */
	short unsigned int         b_secret_size;        /*    42     2 */
	short unsigned int         b_public_size;        /*    44     2 */
	short unsigned int         expected_a_public_size; /*    46     2 */
	short unsigned int         expected_ss_size;     /*    48     2 */
	bool                       genkey;               /*    50     1 */

	/* size: 56, cachelines: 1, members: 11 */
	/* padding: 5 */
	/* last cacheline: 56 bytes */
};
struct comp_testvec {
	int                        inlen;                /*     0     4 */
	int                        outlen;               /*     4     4 */
	char                       input[512];           /*     8   512 */
	/* --- cacheline 8 boundary (512 bytes) was 8 bytes ago --- */
	char                       output[512];          /*   520   512 */

	/* size: 1032, cachelines: 17, members: 4 */
	/* last cacheline: 8 bytes */
};
struct aead_test_suite {
	const struct aead_testvec  * vecs;               /*     0     8 */
	unsigned int               count;                /*     8     4 */
	unsigned int               einval_allowed:1;     /*    12: 0  4 */
	unsigned int               aad_iv:1;             /*    12: 1  4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* bit_padding: 30 bits */
	/* last cacheline: 16 bytes */
};
struct cipher_test_suite {
	const struct cipher_testvec  * vecs;             /*     0     8 */
	unsigned int               count;                /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct comp_test_suite {
	struct {
		const struct comp_testvec  * vecs;       /*     0     8 */
		unsigned int       count;                /*     8     4 */
	} comp;                                          /*     0    16 */

	/* XXX last struct has 4 bytes of padding */

	struct {
		const struct comp_testvec  * vecs;       /*    16     8 */
		unsigned int       count;                /*    24     4 */
	} decomp;                                        /*    16    16 */

	/* XXX last struct has 4 bytes of padding */

	/* size: 32, cachelines: 1, members: 2 */
	/* paddings: 2, sum paddings: 8 */
	/* last cacheline: 32 bytes */
};
struct hash_test_suite {
	const struct hash_testvec  * vecs;               /*     0     8 */
	unsigned int               count;                /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct cprng_test_suite {
	const struct cprng_testvec  * vecs;              /*     0     8 */
	unsigned int               count;                /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct drbg_test_suite {
	const struct drbg_testvec  * vecs;               /*     0     8 */
	unsigned int               count;                /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct akcipher_test_suite {
	const struct akcipher_testvec  * vecs;           /*     0     8 */
	unsigned int               count;                /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct kpp_test_suite {
	const struct kpp_testvec  * vecs;                /*     0     8 */
	unsigned int               count;                /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct alg_test_desc {
	const char  *              alg;                  /*     0     8 */
	const char  *              generic_driver;       /*     8     8 */
	int                        (*test)(const struct alg_test_desc  *, const char  *, u32, u32); /*    16     8 */
	int                        fips_allowed;         /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	union {
		struct aead_test_suite aead;             /*    32    16 */
		struct cipher_test_suite cipher;         /*    32    16 */
		struct comp_test_suite comp;             /*    32    32 */
		struct hash_test_suite hash;             /*    32    16 */
		struct cprng_test_suite cprng;           /*    32    16 */
		struct drbg_test_suite drbg;             /*    32    16 */
		struct akcipher_test_suite akcipher;     /*    32    16 */
		struct kpp_test_suite kpp;               /*    32    16 */
	} suite;                                         /*    32    32 */

	/* size: 64, cachelines: 1, members: 5 */
	/* sum members: 60, holes: 1, sum holes: 4 */
};
struct test_sg_division {
	unsigned int               proportion_of_total;  /*     0     4 */
	unsigned int               offset;               /*     4     4 */
	bool                       offset_relative_to_alignmask; /*     8     1 */

	/* XXX 3 bytes hole, try to pack */

	enum flush_type            flush_type;           /*    12     4 */
	bool                       nosimd;               /*    16     1 */

	/* size: 20, cachelines: 1, members: 5 */
	/* sum members: 14, holes: 1, sum holes: 3 */
	/* padding: 3 */
	/* last cacheline: 20 bytes */
};
struct testvec_config {
	const char  *              name;                 /*     0     8 */
	bool                       inplace;              /*     8     1 */

	/* XXX 3 bytes hole, try to pack */

	u32                        req_flags;            /*    12     4 */
	struct test_sg_division    src_divs[8];          /*    16   160 */
	/* --- cacheline 2 boundary (128 bytes) was 48 bytes ago --- */
	struct test_sg_division    dst_divs[8];          /*   176   160 */
	/* --- cacheline 5 boundary (320 bytes) was 16 bytes ago --- */
	unsigned int               iv_offset;            /*   336     4 */
	unsigned int               key_offset;           /*   340     4 */
	bool                       iv_offset_relative_to_alignmask; /*   344     1 */
	bool                       key_offset_relative_to_alignmask; /*   345     1 */

	/* XXX 2 bytes hole, try to pack */

	enum finalization_type     finalization_type;    /*   348     4 */
	bool                       nosimd;               /*   352     1 */

	/* size: 360, cachelines: 6, members: 11 */
	/* sum members: 348, holes: 2, sum holes: 5 */
	/* padding: 7 */
	/* last cacheline: 40 bytes */
};
struct test_sglist {
	char *                     bufs[8];              /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct scatterlist         sgl[8];               /*    64   256 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	struct scatterlist         sgl_saved[8];         /*   320   256 */
	/* --- cacheline 9 boundary (576 bytes) --- */
	struct scatterlist *       sgl_ptr;              /*   576     8 */
	unsigned int               nents;                /*   584     4 */

	/* size: 592, cachelines: 10, members: 5 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct cipher_test_sglists {
	struct test_sglist         src;                  /*     0   592 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 9 boundary (576 bytes) was 16 bytes ago --- */
	struct test_sglist         dst;                  /*   592   592 */

	/* XXX last struct has 4 bytes of padding */

	/* size: 1184, cachelines: 19, members: 2 */
	/* paddings: 2, sum paddings: 8 */
	/* last cacheline: 32 bytes */
};
struct hmac_ctx {
	struct crypto_shash *      hash;                 /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct md5_state {
	u32                        hash[4];              /*     0    16 */
	u32                        block[16];            /*    16    64 */
	/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
	u64                        byte_count;           /*    80     8 */

	/* size: 88, cachelines: 2, members: 3 */
	/* last cacheline: 24 bytes */
};
struct blake2b_state {
	u64                        h[8];                 /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u64                        t[2];                 /*    64    16 */
	u64                        f[2];                 /*    80    16 */
	u8                         buf[128];             /*    96   128 */
	/* --- cacheline 3 boundary (192 bytes) was 32 bytes ago --- */
	size_t                     buflen;               /*   224     8 */

	/* size: 232, cachelines: 4, members: 5 */
	/* last cacheline: 40 bytes */
};
struct blake2b_tfm_ctx {
	u8                         key[64];              /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	unsigned int               keylen;               /*    64     4 */

	/* size: 68, cachelines: 2, members: 2 */
	/* last cacheline: 4 bytes */
};
struct gf128mul_4k {
	be128                      t[256];               /*     0  4096 */

	/* size: 4096, cachelines: 64, members: 1 */
};
struct gf128mul_64k {
	struct gf128mul_4k *       t[16];                /*     0   128 */

	/* size: 128, cachelines: 2, members: 1 */
};
struct crypto_rfc3686_ctx {
	struct crypto_skcipher *   child;                /*     0     8 */
	u8                         nonce[4];             /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct crypto_rfc3686_req_ctx {
	u8                         iv[16];               /*     0    16 */
	struct skcipher_request    subreq __attribute__((__aligned__(8))); /*    16    80 */

	/* size: 96, cachelines: 2, members: 2 */
	/* forced alignments: 1 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct gcm_instance_ctx {
	struct crypto_skcipher_spawn ctr;                /*     0    48 */
	struct crypto_ahash_spawn  ghash;                /*    48    48 */

	/* size: 96, cachelines: 2, members: 2 */
	/* last cacheline: 32 bytes */
};
struct crypto_gcm_ctx {
	struct crypto_skcipher *   ctr;                  /*     0     8 */
	struct crypto_ahash *      ghash;                /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct crypto_rfc4106_ctx {
	struct crypto_aead *       child;                /*     0     8 */
	u8                         nonce[4];             /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct crypto_rfc4106_req_ctx {
	struct scatterlist         src[3];               /*     0    96 */
	/* --- cacheline 1 boundary (64 bytes) was 32 bytes ago --- */
	struct scatterlist         dst[3];               /*    96    96 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	struct aead_request        subreq __attribute__((__aligned__(8))); /*   192    80 */

	/* size: 272, cachelines: 5, members: 3 */
	/* forced alignments: 1 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct crypto_rfc4543_instance_ctx {
	struct crypto_aead_spawn   aead;                 /*     0    48 */

	/* size: 48, cachelines: 1, members: 1 */
	/* last cacheline: 48 bytes */
};
struct crypto_rfc4543_ctx {
	struct crypto_aead *       child;                /*     0     8 */
	struct crypto_sync_skcipher * null;              /*     8     8 */
	u8                         nonce[4];             /*    16     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct crypto_rfc4543_req_ctx {
	struct aead_request        subreq __attribute__((__aligned__(8))); /*     0    80 */

	/* size: 80, cachelines: 2, members: 1 */
	/* forced alignments: 1 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct crypto_gcm_ghash_ctx {
	unsigned int               cryptlen;             /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct scatterlist *       src;                  /*     8     8 */
	int                        (*complete)(struct aead_request *, u32); /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct crypto_gcm_req_priv_ctx {
	u8                         iv[16];               /*     0    16 */
	u8                         auth_tag[16];         /*    16    16 */
	u8                         iauth_tag[16];        /*    32    16 */
	struct scatterlist         src[3];               /*    48    96 */
	/* --- cacheline 2 boundary (128 bytes) was 16 bytes ago --- */
	struct scatterlist         dst[3];               /*   144    96 */
	/* --- cacheline 3 boundary (192 bytes) was 48 bytes ago --- */
	struct scatterlist         sg;                   /*   240    32 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 4 boundary (256 bytes) was 16 bytes ago --- */
	struct crypto_gcm_ghash_ctx ghash_ctx;           /*   272    24 */
	union {
		struct ahash_request ahreq __attribute__((__aligned__(8))); /*   296    80 */
		struct skcipher_request skreq __attribute__((__aligned__(8))); /*   296    80 */
	} u __attribute__((__aligned__(8)));             /*   296    80 */

	/* size: 376, cachelines: 6, members: 8 */
	/* paddings: 1, sum paddings: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 56 bytes */
} __attribute__((__aligned__(8)));
struct cryptd_skcipher {
	struct crypto_skcipher     base __attribute__((__aligned__(8))); /*     0    32 */

	/* size: 32, cachelines: 1, members: 1 */
	/* forced alignments: 1 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct cryptd_ahash {
	struct crypto_ahash        base __attribute__((__aligned__(8))); /*     0    96 */

	/* size: 96, cachelines: 2, members: 1 */
	/* forced alignments: 1 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct cryptd_aead {
	struct crypto_aead         base __attribute__((__aligned__(8))); /*     0    32 */

	/* size: 32, cachelines: 1, members: 1 */
	/* forced alignments: 1 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct cryptd_cpu_queue {
	struct crypto_queue        queue;                /*     0    32 */
	struct work_struct         work;                 /*    32    64 */

	/* size: 96, cachelines: 2, members: 2 */
	/* last cacheline: 32 bytes */
};
struct cryptd_queue {
	struct cryptd_cpu_queue *  cpu_queue;            /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct cryptd_instance_ctx {
	struct crypto_spawn        spawn;                /*     0    48 */

	/* XXX last struct has 2 bytes of padding */

	struct cryptd_queue *      queue;                /*    48     8 */

	/* size: 56, cachelines: 1, members: 2 */
	/* paddings: 1, sum paddings: 2 */
	/* last cacheline: 56 bytes */
};
struct skcipherd_instance_ctx {
	struct crypto_skcipher_spawn spawn;              /*     0    48 */
	struct cryptd_queue *      queue;                /*    48     8 */

	/* size: 56, cachelines: 1, members: 2 */
	/* last cacheline: 56 bytes */
};
struct hashd_instance_ctx {
	struct crypto_shash_spawn  spawn;                /*     0    48 */
	struct cryptd_queue *      queue;                /*    48     8 */

	/* size: 56, cachelines: 1, members: 2 */
	/* last cacheline: 56 bytes */
};
struct aead_instance_ctx {
	struct crypto_aead_spawn   aead_spawn;           /*     0    48 */
	struct cryptd_queue *      queue;                /*    48     8 */

	/* size: 56, cachelines: 1, members: 2 */
	/* last cacheline: 56 bytes */
};
struct cryptd_skcipher_ctx {
	refcount_t                 refcnt;               /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct crypto_sync_skcipher * child;             /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct cryptd_skcipher_request_ctx {
	crypto_completion_t        complete;             /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct cryptd_hash_ctx {
	refcount_t                 refcnt;               /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct crypto_shash *      child;                /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct cryptd_hash_request_ctx {
	crypto_completion_t        complete;             /*     0     8 */
	struct shash_desc          desc __attribute__((__aligned__(8))); /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* forced alignments: 1 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct cryptd_aead_ctx {
	refcount_t                 refcnt;               /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct crypto_aead *       child;                /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct cryptd_aead_request_ctx {
	crypto_completion_t        complete;             /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct deflate_ctx {
	struct z_stream_s          comp_stream;          /*     0    96 */
	/* --- cacheline 1 boundary (64 bytes) was 32 bytes ago --- */
	struct z_stream_s          decomp_stream;        /*    96    96 */

	/* size: 192, cachelines: 3, members: 2 */
};
struct chksum_ctx {
	u32                        key;                  /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct chksum_desc_ctx {
	u32                        crc;                  /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct chksum_desc_ctx {
	__u16                      crc;                  /*     0     2 */

	/* size: 2, cachelines: 1, members: 1 */
	/* last cacheline: 2 bytes */
};
struct lzo_ctx {
	void *                     lzo_comp_mem;         /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct lzorle_ctx {
	void *                     lzorle_comp_mem;      /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct xxh64_state {
	uint64_t                   total_len;            /*     0     8 */
	uint64_t                   v1;                   /*     8     8 */
	uint64_t                   v2;                   /*    16     8 */
	uint64_t                   v3;                   /*    24     8 */
	uint64_t                   v4;                   /*    32     8 */
	uint64_t                   mem64[4];             /*    40    32 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	uint32_t                   memsize;              /*    72     4 */

	/* size: 80, cachelines: 2, members: 7 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct xxhash64_tfm_ctx {
	u64                        seed;                 /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct xxhash64_desc_ctx {
	struct xxh64_state         xxhstate;             /*     0    80 */

	/* XXX last struct has 4 bytes of padding */

	/* size: 80, cachelines: 2, members: 1 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 16 bytes */
};
struct crypto_report_rng {
	char                       type[64];             /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	unsigned int               seedsize;             /*    64     4 */

	/* size: 68, cachelines: 2, members: 2 */
	/* last cacheline: 4 bytes */
};
struct drbg_core {
	drbg_flag_t                flags;                /*     0     4 */
	__u8                       statelen;             /*     4     1 */
	__u8                       blocklen_bytes;       /*     5     1 */
	char                       cra_name[128];        /*     6   128 */
	/* --- cacheline 2 boundary (128 bytes) was 6 bytes ago --- */
	char                       backend_cra_name[128]; /*   134   128 */

	/* size: 264, cachelines: 5, members: 5 */
	/* padding: 2 */
	/* last cacheline: 8 bytes */
};
struct drbg_state_ops {
	int                        (*update)(struct drbg_state *, struct list_head *, int); /*     0     8 */
	int                        (*generate)(struct drbg_state *, unsigned char *, unsigned int, struct list_head *); /*     8     8 */
	int                        (*crypto_init)(struct drbg_state *); /*    16     8 */
	int                        (*crypto_fini)(struct drbg_state *); /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct drbg_state {
	struct mutex               drbg_mutex;           /*     0    32 */
	unsigned char *            V;                    /*    32     8 */
	unsigned char *            Vbuf;                 /*    40     8 */
	unsigned char *            C;                    /*    48     8 */
	unsigned char *            Cbuf;                 /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	size_t                     reseed_ctr;           /*    64     8 */
	size_t                     reseed_threshold;     /*    72     8 */
	unsigned char *            scratchpad;           /*    80     8 */
	unsigned char *            scratchpadbuf;        /*    88     8 */
	void *                     priv_data;            /*    96     8 */
	struct crypto_skcipher *   ctr_handle;           /*   104     8 */
	struct skcipher_request *  ctr_req;              /*   112     8 */
	__u8 *                     outscratchpadbuf;     /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	__u8 *                     outscratchpad;        /*   128     8 */
	struct crypto_wait         ctr_wait;             /*   136    40 */

	/* XXX last struct has 4 bytes of padding */

	struct scatterlist         sg_in;                /*   176    32 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 3 boundary (192 bytes) was 16 bytes ago --- */
	struct scatterlist         sg_out;               /*   208    32 */

	/* XXX last struct has 4 bytes of padding */

	enum drbg_seed_state       seeded;               /*   240     4 */
	bool                       pr;                   /*   244     1 */
	bool                       fips_primed;          /*   245     1 */

	/* XXX 2 bytes hole, try to pack */

	unsigned char *            prev;                 /*   248     8 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	struct crypto_rng *        jent;                 /*   256     8 */
	const struct drbg_state_ops  * d_ops;            /*   264     8 */
	const struct drbg_core  *  core;                 /*   272     8 */
	struct drbg_string         test_data;            /*   280    32 */

	/* size: 312, cachelines: 5, members: 25 */
	/* sum members: 310, holes: 1, sum holes: 2 */
	/* paddings: 3, sum paddings: 12 */
	/* last cacheline: 56 bytes */
};
struct rand_data {
	__u64                      data;                 /*     0     8 */
	__u64                      old_data;             /*     8     8 */
	__u64                      prev_time;            /*    16     8 */
	__u64                      last_delta;           /*    24     8 */
	__s64                      last_delta2;          /*    32     8 */
	unsigned int               osr;                  /*    40     4 */

	/* XXX 4 bytes hole, try to pack */

	unsigned char *            mem;                  /*    48     8 */
	unsigned int               memlocation;          /*    56     4 */
	unsigned int               memblocks;            /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	unsigned int               memblocksize;         /*    64     4 */
	unsigned int               memaccessloops;       /*    68     4 */
	int                        rct_count;            /*    72     4 */
	unsigned int               apt_observations;     /*    76     4 */
	unsigned int               apt_count;            /*    80     4 */
	unsigned int               apt_base;             /*    84     4 */
	unsigned int               apt_base_set:1;       /*    88: 0  4 */
	unsigned int               health_failure:1;     /*    88: 1  4 */

	/* size: 96, cachelines: 2, members: 17 */
	/* sum members: 84, holes: 1, sum holes: 4 */
	/* sum bitfield members: 2 bits (0 bytes) */
	/* padding: 4 */
	/* bit_padding: 30 bits */
	/* last cacheline: 32 bytes */
};
struct jitterentropy {
	spinlock_t                 jent_lock;            /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct rand_data *         entropy_collector;    /*     8     8 */
	unsigned int               reset_cnt;            /*    16     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 16, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct ghash_ctx {
	struct gf128mul_4k *       gf128;                /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct ghash_desc_ctx {
	u8                         buffer[16];           /*     0    16 */
	u32                        bytes;                /*    16     4 */

	/* size: 20, cachelines: 1, members: 2 */
	/* last cacheline: 20 bytes */
};
struct sockaddr_alg_new {
	__u16                      salg_family;          /*     0     2 */
	__u8                       salg_type[14];        /*     2    14 */
	__u32                      salg_feat;            /*    16     4 */
	__u32                      salg_mask;            /*    20     4 */
	__u8                       salg_name[];          /*    24     0 */

	/* size: 24, cachelines: 1, members: 5 */
	/* last cacheline: 24 bytes */
};
struct af_alg_iv {
	__u32                      ivlen;                /*     0     4 */
	__u8                       iv[];                 /*     4     0 */

	/* size: 4, cachelines: 1, members: 2 */
	/* last cacheline: 4 bytes */
};
struct cmsghdr {
	__kernel_size_t            cmsg_len;             /*     0     8 */
	int                        cmsg_level;           /*     8     4 */
	int                        cmsg_type;            /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct net_proto_family {
	int                        family;               /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	int                        (*create)(struct net *, struct socket *, int, int); /*     8     8 */
	struct module *            owner;                /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct alg_sock {
	struct sock                sk __attribute__((__aligned__(8))); /*     0   872 */
	/* --- cacheline 13 boundary (832 bytes) was 40 bytes ago --- */
	struct sock *              parent;               /*   872     8 */
	atomic_t                   refcnt;               /*   880     4 */
	atomic_t                   nokey_refcnt;         /*   884     4 */
	const struct af_alg_type  * type;                /*   888     8 */
	/* --- cacheline 14 boundary (896 bytes) --- */
	void *                     private;              /*   896     8 */

	/* size: 904, cachelines: 15, members: 6 */
	/* forced alignments: 1 */
	/* last cacheline: 8 bytes */
} __attribute__((__aligned__(8)));
struct af_alg_type {
	void *                     (*bind)(const char  *, u32, u32); /*     0     8 */
	void                       (*release)(void *);   /*     8     8 */
	int                        (*setkey)(void *, const u8  *, unsigned int); /*    16     8 */
	int                        (*setentropy)(void *, sockptr_t, unsigned int); /*    24     8 */
	int                        (*accept)(void *, struct sock *); /*    32     8 */
	int                        (*accept_nokey)(void *, struct sock *); /*    40     8 */
	int                        (*setauthsize)(void *, unsigned int); /*    48     8 */
	struct proto_ops *         ops;                  /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct proto_ops *         ops_nokey;            /*    64     8 */
	struct module *            owner;                /*    72     8 */
	char                       name[14];             /*    80    14 */

	/* size: 96, cachelines: 2, members: 11 */
	/* padding: 2 */
	/* last cacheline: 32 bytes */
};
struct af_alg_control {
	struct af_alg_iv *         iv;                   /*     0     8 */
	int                        op;                   /*     8     4 */
	unsigned int               aead_assoclen;        /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct af_alg_sgl {
	struct scatterlist         sg[17];               /*     0   544 */
	/* --- cacheline 8 boundary (512 bytes) was 32 bytes ago --- */
	struct page *              pages[16];            /*   544   128 */
	/* --- cacheline 10 boundary (640 bytes) was 32 bytes ago --- */
	unsigned int               npages;               /*   672     4 */

	/* size: 680, cachelines: 11, members: 3 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct af_alg_tsgl {
	struct list_head           list;                 /*     0    16 */
	unsigned int               cur;                  /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	struct scatterlist         sg[];                 /*    24     0 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct af_alg_rsgl {
	struct af_alg_sgl          sgl;                  /*     0   680 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 10 boundary (640 bytes) was 40 bytes ago --- */
	struct list_head           list;                 /*   680    16 */
	size_t                     sg_num_bytes;         /*   696     8 */

	/* size: 704, cachelines: 11, members: 3 */
	/* paddings: 1, sum paddings: 4 */
};
struct af_alg_async_req {
	struct kiocb *             iocb;                 /*     0     8 */
	struct sock *              sk;                   /*     8     8 */
	struct af_alg_rsgl         first_rsgl;           /*    16   704 */
	/* --- cacheline 11 boundary (704 bytes) was 16 bytes ago --- */
	struct af_alg_rsgl *       last_rsgl;            /*   720     8 */
	struct list_head           rsgl_list;            /*   728    16 */
	struct scatterlist *       tsgl;                 /*   744     8 */
	unsigned int               tsgl_entries;         /*   752     4 */
	unsigned int               outlen;               /*   756     4 */
	unsigned int               areqlen;              /*   760     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 12 boundary (768 bytes) --- */
	union {
		struct aead_request aead_req __attribute__((__aligned__(8))); /*   768    80 */
		struct skcipher_request skcipher_req __attribute__((__aligned__(8))); /*   768    80 */
	} cra_u __attribute__((__aligned__(8)));         /*   768    80 */

	/* size: 848, cachelines: 14, members: 10 */
	/* sum members: 844, holes: 1, sum holes: 4 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 4 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct af_alg_ctx {
	struct list_head           tsgl_list;            /*     0    16 */
	void *                     iv;                   /*    16     8 */
	size_t                     aead_assoclen;        /*    24     8 */
	struct crypto_wait         wait;                 /*    32    40 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	size_t                     used;                 /*    72     8 */
	atomic_t                   rcvused;              /*    80     4 */
	bool                       more;                 /*    84     1 */
	bool                       merge;                /*    85     1 */
	bool                       enc;                  /*    86     1 */
	bool                       init;                 /*    87     1 */
	unsigned int               len;                  /*    88     4 */

	/* size: 96, cachelines: 2, members: 11 */
	/* padding: 4 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 32 bytes */
};
struct alg_type_list {
	const struct af_alg_type  * type;                /*     0     8 */
	struct list_head           list;                 /*     8    16 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct hash_ctx {
	struct af_alg_sgl          sgl;                  /*     0   680 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 10 boundary (640 bytes) was 40 bytes ago --- */
	u8 *                       result;               /*   680     8 */
	struct crypto_wait         wait;                 /*   688    40 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 11 boundary (704 bytes) was 24 bytes ago --- */
	unsigned int               len;                  /*   728     4 */
	bool                       more;                 /*   732     1 */

	/* XXX 3 bytes hole, try to pack */

	struct ahash_request       req __attribute__((__aligned__(8))); /*   736    80 */

	/* size: 816, cachelines: 13, members: 6 */
	/* sum members: 813, holes: 1, sum holes: 3 */
	/* paddings: 2, sum paddings: 8 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 3 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct rng_ctx {
	unsigned int               len;                  /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct crypto_rng *        drng;                 /*     8     8 */
	u8 *                       addtl;                /*    16     8 */
	size_t                     addtl_len;            /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct rng_parent_ctx {
	struct crypto_rng *        drng;                 /*     0     8 */
	u8 *                       entropy;              /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct aead_tfm {
	struct crypto_aead *       aead;                 /*     0     8 */
	struct crypto_sync_skcipher * null_tfm;          /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct zstd_ctx {
	ZSTD_CCtx *                cctx;                 /*     0     8 */
	ZSTD_DCtx *                dctx;                 /*     8     8 */
	void *                     cwksp;                /*    16     8 */
	void *                     dwksp;                /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct asymmetric_key_ids {
	void *                     id[2];                /*     0    16 */

	/* size: 16, cachelines: 1, members: 1 */
	/* last cacheline: 16 bytes */
};
struct asymmetric_key_subtype {
	struct module *            owner;                /*     0     8 */
	const char  *              name;                 /*     8     8 */
	short unsigned int         name_len;             /*    16     2 */

	/* XXX 6 bytes hole, try to pack */

	void                       (*describe)(const struct key  *, struct seq_file *); /*    24     8 */
	void                       (*destroy)(void *, void *); /*    32     8 */
	int                        (*query)(const struct kernel_pkey_params  *, struct kernel_pkey_query *); /*    40     8 */
	int                        (*eds_op)(struct kernel_pkey_params *, const void  *, void *); /*    48     8 */
	int                        (*verify_signature)(const struct key  *, const struct public_key_signature  *); /*    56     8 */

	/* size: 64, cachelines: 1, members: 8 */
	/* sum members: 58, holes: 1, sum holes: 6 */
};
struct asymmetric_key_parser {
	struct list_head           link;                 /*     0    16 */
	struct module *            owner;                /*    16     8 */
	const char  *              name;                 /*    24     8 */
	int                        (*parse)(struct key_preparsed_payload *); /*    32     8 */

	/* size: 40, cachelines: 1, members: 4 */
	/* last cacheline: 40 bytes */
};
struct public_key {
	void *                     key;                  /*     0     8 */
	u32                        keylen;               /*     8     4 */
	enum OID                   algo;                 /*    12     4 */
	void *                     params;               /*    16     8 */
	u32                        paramlen;             /*    24     4 */
	bool                       key_is_private;       /*    28     1 */

	/* XXX 3 bytes hole, try to pack */

	const char  *              id_type;              /*    32     8 */
	const char  *              pkey_algo;            /*    40     8 */
	u64                        kabi_reserved1;       /*    48     8 */
	u64                        kabi_reserved2;       /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u64                        kabi_reserved3;       /*    64     8 */
	u64                        kabi_reserved4;       /*    72     8 */
	u64                        kabi_reserved5;       /*    80     8 */

	/* size: 88, cachelines: 2, members: 13 */
	/* sum members: 85, holes: 1, sum holes: 3 */
	/* last cacheline: 24 bytes */
};
struct x509_certificate {
	struct x509_certificate *  next;                 /*     0     8 */
	struct x509_certificate *  signer;               /*     8     8 */
	struct public_key *        pub;                  /*    16     8 */
	struct public_key_signature * sig;               /*    24     8 */
	char *                     issuer;               /*    32     8 */
	char *                     subject;              /*    40     8 */
	struct asymmetric_key_id * id;                   /*    48     8 */
	struct asymmetric_key_id * skid;                 /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	time64_t                   valid_from;           /*    64     8 */
	time64_t                   valid_to;             /*    72     8 */
	const void  *              tbs;                  /*    80     8 */
	unsigned int               tbs_size;             /*    88     4 */
	unsigned int               raw_sig_size;         /*    92     4 */
	const void  *              raw_sig;              /*    96     8 */
	const void  *              raw_serial;           /*   104     8 */
	unsigned int               raw_serial_size;      /*   112     4 */
	unsigned int               raw_issuer_size;      /*   116     4 */
	const void  *              raw_issuer;           /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	const void  *              raw_subject;          /*   128     8 */
	unsigned int               raw_subject_size;     /*   136     4 */
	unsigned int               raw_skid_size;        /*   140     4 */
	const void  *              raw_skid;             /*   144     8 */
	unsigned int               index;                /*   152     4 */
	bool                       seen;                 /*   156     1 */
	bool                       verified;             /*   157     1 */
	bool                       self_signed;          /*   158     1 */
	bool                       unsupported_key;      /*   159     1 */
	bool                       unsupported_sig;      /*   160     1 */
	bool                       blacklisted;          /*   161     1 */

	/* size: 168, cachelines: 3, members: 29 */
	/* padding: 6 */
	/* last cacheline: 40 bytes */
};
struct x509_parse_context {
	struct x509_certificate *  cert;                 /*     0     8 */
	long unsigned int          data;                 /*     8     8 */
	const void  *              cert_start;           /*    16     8 */
	const void  *              key;                  /*    24     8 */
	size_t                     key_size;             /*    32     8 */
	const void  *              params;               /*    40     8 */
	size_t                     params_size;          /*    48     8 */
	enum OID                   key_algo;             /*    56     4 */
	enum OID                   last_oid;             /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	enum OID                   algo_oid;             /*    64     4 */
	unsigned char              nr_mpi;               /*    68     1 */
	u8                         o_size;               /*    69     1 */
	u8                         cn_size;              /*    70     1 */
	u8                         email_size;           /*    71     1 */
	u16                        o_offset;             /*    72     2 */
	u16                        cn_offset;            /*    74     2 */
	u16                        email_offset;         /*    76     2 */

	/* XXX 2 bytes hole, try to pack */

	unsigned int               raw_akid_size;        /*    80     4 */

	/* XXX 4 bytes hole, try to pack */

	const void  *              raw_akid;             /*    88     8 */
	const void  *              akid_raw_issuer;      /*    96     8 */
	unsigned int               akid_raw_issuer_size; /*   104     4 */

	/* size: 112, cachelines: 2, members: 21 */
	/* sum members: 102, holes: 2, sum holes: 6 */
	/* padding: 4 */
	/* last cacheline: 48 bytes */
};
struct pkcs7_signed_info {
	struct pkcs7_signed_info * next;                 /*     0     8 */
	struct x509_certificate *  signer;               /*     8     8 */
	unsigned int               index;                /*    16     4 */
	bool                       unsupported_crypto;   /*    20     1 */
	bool                       blacklisted;          /*    21     1 */

	/* XXX 2 bytes hole, try to pack */

	const void  *              msgdigest;            /*    24     8 */
	unsigned int               msgdigest_len;        /*    32     4 */
	unsigned int               authattrs_len;        /*    36     4 */
	const void  *              authattrs;            /*    40     8 */
	long unsigned int          aa_set;               /*    48     8 */
	time64_t                   signing_time;         /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct public_key_signature * sig;               /*    64     8 */

	/* size: 72, cachelines: 2, members: 12 */
	/* sum members: 70, holes: 1, sum holes: 2 */
	/* last cacheline: 8 bytes */
};
struct pkcs7_message {
	struct x509_certificate *  certs;                /*     0     8 */
	struct x509_certificate *  crl;                  /*     8     8 */
	struct pkcs7_signed_info * signed_infos;         /*    16     8 */
	u8                         version;              /*    24     1 */
	bool                       have_authattrs;       /*    25     1 */

	/* XXX 2 bytes hole, try to pack */

	enum OID                   data_type;            /*    28     4 */
	size_t                     data_len;             /*    32     8 */
	size_t                     data_hdrlen;          /*    40     8 */
	const void  *              data;                 /*    48     8 */

	/* size: 56, cachelines: 1, members: 9 */
	/* sum members: 54, holes: 1, sum holes: 2 */
	/* last cacheline: 56 bytes */
};
struct pkcs7_parse_context {
	struct pkcs7_message *     msg;                  /*     0     8 */
	struct pkcs7_signed_info * sinfo;                /*     8     8 */
	struct pkcs7_signed_info * * ppsinfo;            /*    16     8 */
	struct x509_certificate *  certs;                /*    24     8 */
	struct x509_certificate * * ppcerts;             /*    32     8 */
	long unsigned int          data;                 /*    40     8 */
	enum OID                   last_oid;             /*    48     4 */
	unsigned int               x509_index;           /*    52     4 */
	unsigned int               sinfo_index;          /*    56     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	const void  *              raw_serial;           /*    64     8 */
	unsigned int               raw_serial_size;      /*    72     4 */
	unsigned int               raw_issuer_size;      /*    76     4 */
	const void  *              raw_issuer;           /*    80     8 */
	const void  *              raw_skid;             /*    88     8 */
	unsigned int               raw_skid_size;        /*    96     4 */
	bool                       expect_skid;          /*   100     1 */

	/* size: 104, cachelines: 2, members: 16 */
	/* sum members: 97, holes: 1, sum holes: 4 */
	/* padding: 3 */
	/* last cacheline: 40 bytes */
};
struct mz_hdr {
	uint16_t                   magic;                /*     0     2 */
	uint16_t                   lbsize;               /*     2     2 */
	uint16_t                   blocks;               /*     4     2 */
	uint16_t                   relocs;               /*     6     2 */
	uint16_t                   hdrsize;              /*     8     2 */
	uint16_t                   min_extra_pps;        /*    10     2 */
	uint16_t                   max_extra_pps;        /*    12     2 */
	uint16_t                   ss;                   /*    14     2 */
	uint16_t                   sp;                   /*    16     2 */
	uint16_t                   checksum;             /*    18     2 */
	uint16_t                   ip;                   /*    20     2 */
	uint16_t                   cs;                   /*    22     2 */
	uint16_t                   reloc_table_offset;   /*    24     2 */
	uint16_t                   overlay_num;          /*    26     2 */
	uint16_t                   reserved0[4];         /*    28     8 */
	uint16_t                   oem_id;               /*    36     2 */
	uint16_t                   oem_info;             /*    38     2 */
	uint16_t                   reserved1[10];        /*    40    20 */
	uint32_t                   peaddr;               /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	char                       message[];            /*    64     0 */

	/* size: 64, cachelines: 1, members: 20 */
};
struct pe_hdr {
	uint32_t                   magic;                /*     0     4 */
	uint16_t                   machine;              /*     4     2 */
	uint16_t                   sections;             /*     6     2 */
	uint32_t                   timestamp;            /*     8     4 */
	uint32_t                   symbol_table;         /*    12     4 */
	uint32_t                   symbols;              /*    16     4 */
	uint16_t                   opt_hdr_size;         /*    20     2 */
	uint16_t                   flags;                /*    22     2 */

	/* size: 24, cachelines: 1, members: 8 */
	/* last cacheline: 24 bytes */
};
struct pe32_opt_hdr {
	uint16_t                   magic;                /*     0     2 */
	uint8_t                    ld_major;             /*     2     1 */
	uint8_t                    ld_minor;             /*     3     1 */
	uint32_t                   text_size;            /*     4     4 */
	uint32_t                   data_size;            /*     8     4 */
	uint32_t                   bss_size;             /*    12     4 */
	uint32_t                   entry_point;          /*    16     4 */
	uint32_t                   code_base;            /*    20     4 */
	uint32_t                   data_base;            /*    24     4 */
	uint32_t                   image_base;           /*    28     4 */
	uint32_t                   section_align;        /*    32     4 */
	uint32_t                   file_align;           /*    36     4 */
	uint16_t                   os_major;             /*    40     2 */
	uint16_t                   os_minor;             /*    42     2 */
	uint16_t                   image_major;          /*    44     2 */
	uint16_t                   image_minor;          /*    46     2 */
	uint16_t                   subsys_major;         /*    48     2 */
	uint16_t                   subsys_minor;         /*    50     2 */
	uint32_t                   win32_version;        /*    52     4 */
	uint32_t                   image_size;           /*    56     4 */
	uint32_t                   header_size;          /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	uint32_t                   csum;                 /*    64     4 */
	uint16_t                   subsys;               /*    68     2 */
	uint16_t                   dll_flags;            /*    70     2 */
	uint32_t                   stack_size_req;       /*    72     4 */
	uint32_t                   stack_size;           /*    76     4 */
	uint32_t                   heap_size_req;        /*    80     4 */
	uint32_t                   heap_size;            /*    84     4 */
	uint32_t                   loader_flags;         /*    88     4 */
	uint32_t                   data_dirs;            /*    92     4 */

	/* size: 96, cachelines: 2, members: 30 */
	/* last cacheline: 32 bytes */
};
struct pe32plus_opt_hdr {
	uint16_t                   magic;                /*     0     2 */
	uint8_t                    ld_major;             /*     2     1 */
	uint8_t                    ld_minor;             /*     3     1 */
	uint32_t                   text_size;            /*     4     4 */
	uint32_t                   data_size;            /*     8     4 */
	uint32_t                   bss_size;             /*    12     4 */
	uint32_t                   entry_point;          /*    16     4 */
	uint32_t                   code_base;            /*    20     4 */
	uint64_t                   image_base;           /*    24     8 */
	uint32_t                   section_align;        /*    32     4 */
	uint32_t                   file_align;           /*    36     4 */
	uint16_t                   os_major;             /*    40     2 */
	uint16_t                   os_minor;             /*    42     2 */
	uint16_t                   image_major;          /*    44     2 */
	uint16_t                   image_minor;          /*    46     2 */
	uint16_t                   subsys_major;         /*    48     2 */
	uint16_t                   subsys_minor;         /*    50     2 */
	uint32_t                   win32_version;        /*    52     4 */
	uint32_t                   image_size;           /*    56     4 */
	uint32_t                   header_size;          /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	uint32_t                   csum;                 /*    64     4 */
	uint16_t                   subsys;               /*    68     2 */
	uint16_t                   dll_flags;            /*    70     2 */
	uint64_t                   stack_size_req;       /*    72     8 */
	uint64_t                   stack_size;           /*    80     8 */
	uint64_t                   heap_size_req;        /*    88     8 */
	uint64_t                   heap_size;            /*    96     8 */
	uint32_t                   loader_flags;         /*   104     4 */
	uint32_t                   data_dirs;            /*   108     4 */

	/* size: 112, cachelines: 2, members: 29 */
	/* last cacheline: 48 bytes */
};
struct data_dirent {
	uint32_t                   virtual_address;      /*     0     4 */
	uint32_t                   size;                 /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct data_directory {
	struct data_dirent         exports;              /*     0     8 */
	struct data_dirent         imports;              /*     8     8 */
	struct data_dirent         resources;            /*    16     8 */
	struct data_dirent         exceptions;           /*    24     8 */
	struct data_dirent         certs;                /*    32     8 */
	struct data_dirent         base_relocations;     /*    40     8 */
	struct data_dirent         debug;                /*    48     8 */
	struct data_dirent         arch;                 /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct data_dirent         global_ptr;           /*    64     8 */
	struct data_dirent         tls;                  /*    72     8 */
	struct data_dirent         load_config;          /*    80     8 */
	struct data_dirent         bound_imports;        /*    88     8 */
	struct data_dirent         import_addrs;         /*    96     8 */
	struct data_dirent         delay_imports;        /*   104     8 */
	struct data_dirent         clr_runtime_hdr;      /*   112     8 */
	struct data_dirent         reserved;             /*   120     8 */

	/* size: 128, cachelines: 2, members: 16 */
};
struct section_header {
	char                       name[8];              /*     0     8 */
	uint32_t                   virtual_size;         /*     8     4 */
	uint32_t                   virtual_address;      /*    12     4 */
	uint32_t                   raw_data_size;        /*    16     4 */
	uint32_t                   data_addr;            /*    20     4 */
	uint32_t                   relocs;               /*    24     4 */
	uint32_t                   line_numbers;         /*    28     4 */
	uint16_t                   num_relocs;           /*    32     2 */
	uint16_t                   num_lin_numbers;      /*    34     2 */
	uint32_t                   flags;                /*    36     4 */

	/* size: 40, cachelines: 1, members: 10 */
	/* last cacheline: 40 bytes */
};
struct win_certificate {
	uint32_t                   length;               /*     0     4 */
	uint16_t                   revision;             /*     4     2 */
	uint16_t                   cert_type;            /*     6     2 */

	/* size: 8, cachelines: 1, members: 3 */
	/* last cacheline: 8 bytes */
};
struct pefile_context {
	unsigned int               header_size;          /*     0     4 */
	unsigned int               image_checksum_offset; /*     4     4 */
	unsigned int               cert_dirent_offset;   /*     8     4 */
	unsigned int               n_data_dirents;       /*    12     4 */
	unsigned int               n_sections;           /*    16     4 */
	unsigned int               certs_size;           /*    20     4 */
	unsigned int               sig_offset;           /*    24     4 */
	unsigned int               sig_len;              /*    28     4 */
	const struct section_header  * secs;             /*    32     8 */
	const void  *              digest;               /*    40     8 */
	unsigned int               digest_len;           /*    48     4 */

	/* XXX 4 bytes hole, try to pack */

	const char  *              digest_algo;          /*    56     8 */

	/* size: 64, cachelines: 1, members: 12 */
	/* sum members: 60, holes: 1, sum holes: 4 */
};
struct pgp_parse_context {
	u64                        types_of_interest;    /*     0     8 */
	int                        (*process_packet)(struct pgp_parse_context *, enum pgp_packet_tag, u8, const u8  *, size_t); /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct pgp_parse_pubkey {
	enum pgp_key_version       version:8;            /*     0: 0  4 */
	enum pgp_pubkey_algo       pubkey_algo:8;        /*     0: 8  4 */

	/* XXX 16 bits hole, try to pack */
	/* XXX 4 bytes hole, try to pack */

	__kernel_old_time_t        creation_time;        /*     8     8 */
	__kernel_old_time_t        expires_at;           /*    16     8 */

	/* size: 24, cachelines: 1, members: 4 */
	/* sum members: 16, holes: 1, sum holes: 4 */
	/* sum bitfield members: 16 bits, bit holes: 1, sum bit holes: 16 bits */
	/* last cacheline: 24 bytes */
};
struct preload_pgp_keys_context {
	struct pgp_parse_context   pgp;                  /*     0    16 */
	key_ref_t                  keyring;              /*    16     8 */
	const u8  *                key_start;            /*    24     8 */
	const u8  *                key_end;              /*    32     8 */
	bool                       found_key;            /*    40     1 */

	/* size: 48, cachelines: 1, members: 5 */
	/* padding: 7 */
	/* last cacheline: 48 bytes */
};
struct pgp_key_data_parse_context {
	struct pgp_parse_context   pgp;                  /*     0    16 */
	struct public_key *        pub;                  /*    16     8 */
	unsigned char *            raw_fingerprint;      /*    24     8 */
	char *                     fingerprint;          /*    32     8 */
	const char  *              user_id;              /*    40     8 */
	size_t                     user_id_len;          /*    48     8 */
	size_t                     fingerprint_len;      /*    56     8 */

	/* size: 64, cachelines: 1, members: 7 */
};
struct simd_skcipher_alg {
	const char  *              ialg_name;            /*     0     8 */
	struct skcipher_alg        alg __attribute__((__aligned__(8))); /*     8   448 */

	/* size: 456, cachelines: 8, members: 2 */
	/* forced alignments: 1 */
	/* last cacheline: 8 bytes */
} __attribute__((__aligned__(8)));
struct simd_skcipher_ctx {
	struct cryptd_skcipher *   cryptd_tfm;           /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct simd_aead_alg {
	const char  *              ialg_name;            /*     0     8 */
	struct aead_alg            alg __attribute__((__aligned__(8))); /*     8   448 */

	/* size: 456, cachelines: 8, members: 2 */
	/* forced alignments: 1 */
	/* last cacheline: 8 bytes */
} __attribute__((__aligned__(8)));
struct simd_aead_ctx {
	struct cryptd_aead *       cryptd_tfm;           /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct biovec_slab {
	int                        nr_vecs;              /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	char *                     name;                 /*     8     8 */
	struct kmem_cache *        slab;                 /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct blk_mq_debugfs_attr {
	const char  *              name;                 /*     0     8 */
	umode_t                    mode;                 /*     8     2 */

	/* XXX 6 bytes hole, try to pack */

	int                        (*show)(void *, struct seq_file *); /*    16     8 */
	ssize_t                    (*write)(void *, const char  *, size_t, loff_t *); /*    24     8 */
	const struct seq_operations  * seq_ops;          /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* sum members: 34, holes: 1, sum holes: 6 */
	/* last cacheline: 40 bytes */
};
struct rq_qos {
	struct rq_qos_ops *        ops;                  /*     0     8 */
	struct request_queue *     q;                    /*     8     8 */
	enum rq_qos_id             id;                   /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	struct rq_qos *            next;                 /*    24     8 */
	struct dentry *            debugfs_dir;          /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* sum members: 36, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct rq_qos_ops {
	void                       (*throttle)(struct rq_qos *, struct bio *); /*     0     8 */
	void                       (*track)(struct rq_qos *, struct request *, struct bio *); /*     8     8 */
	void                       (*merge)(struct rq_qos *, struct request *, struct bio *); /*    16     8 */
	void                       (*issue)(struct rq_qos *, struct request *); /*    24     8 */
	void                       (*requeue)(struct rq_qos *, struct request *); /*    32     8 */
	void                       (*done)(struct rq_qos *, struct request *); /*    40     8 */
	void                       (*done_bio)(struct rq_qos *, struct bio *); /*    48     8 */
	void                       (*cleanup)(struct rq_qos *, struct bio *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	void                       (*queue_depth_changed)(struct rq_qos *); /*    64     8 */
	void                       (*exit)(struct rq_qos *); /*    72     8 */
	const struct blk_mq_debugfs_attr  * debugfs_attrs; /*    80     8 */

	/* size: 88, cachelines: 2, members: 11 */
	/* last cacheline: 24 bytes */
};
struct bio_slab {
	struct kmem_cache *        slab;                 /*     0     8 */
	unsigned int               slab_ref;             /*     8     4 */
	unsigned int               slab_size;            /*    12     4 */
	char                       name[8];              /*    16     8 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct blk_plug_cb {
	struct list_head           list;                 /*     0    16 */
	blk_plug_cb_fn             callback;             /*    16     8 */
	void *                     data;                 /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* last cacheline: 32 bytes */
};
struct request_wrapper {
	struct request             rq __attribute__((__aligned__(8))); /*     0   272 */
	/* --- cacheline 4 boundary (256 bytes) was 16 bytes ago --- */
	u64                        stat_time_ns;         /*   272     8 */

	/* size: 280, cachelines: 5, members: 2 */
	/* forced alignments: 1 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct trace_event_raw_block_buffer {
	struct trace_entry         ent;                  /*     0     8 */
	dev_t                      dev;                  /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	sector_t                   sector;               /*    16     8 */
	size_t                     size;                 /*    24     8 */
	char                       __data[];             /*    32     0 */

	/* size: 32, cachelines: 1, members: 5 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct trace_event_raw_block_rq_requeue {
	struct trace_entry         ent;                  /*     0     8 */
	dev_t                      dev;                  /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	sector_t                   sector;               /*    16     8 */
	unsigned int               nr_sector;            /*    24     4 */
	char                       rwbs[8];              /*    28     8 */
	u32                        __data_loc_cmd;       /*    36     4 */
	char                       __data[];             /*    40     0 */

	/* size: 40, cachelines: 1, members: 7 */
	/* sum members: 36, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct trace_event_raw_block_rq_complete {
	struct trace_entry         ent;                  /*     0     8 */
	dev_t                      dev;                  /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	sector_t                   sector;               /*    16     8 */
	unsigned int               nr_sector;            /*    24     4 */
	int                        error;                /*    28     4 */
	char                       rwbs[8];              /*    32     8 */
	u32                        __data_loc_cmd;       /*    40     4 */
	char                       __data[];             /*    44     0 */

	/* size: 48, cachelines: 1, members: 8 */
	/* sum members: 40, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 48 bytes */
};
struct trace_event_raw_block_rq {
	struct trace_entry         ent;                  /*     0     8 */
	dev_t                      dev;                  /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	sector_t                   sector;               /*    16     8 */
	unsigned int               nr_sector;            /*    24     4 */
	unsigned int               bytes;                /*    28     4 */
	char                       rwbs[8];              /*    32     8 */
	char                       comm[16];             /*    40    16 */
	u32                        __data_loc_cmd;       /*    56     4 */
	char                       __data[];             /*    60     0 */

	/* size: 64, cachelines: 1, members: 9 */
	/* sum members: 56, holes: 1, sum holes: 4 */
	/* padding: 4 */
};
struct trace_event_raw_block_bio_bounce {
	struct trace_entry         ent;                  /*     0     8 */
	dev_t                      dev;                  /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	sector_t                   sector;               /*    16     8 */
	unsigned int               nr_sector;            /*    24     4 */
	char                       rwbs[8];              /*    28     8 */
	char                       comm[16];             /*    36    16 */
	char                       __data[];             /*    52     0 */

	/* size: 56, cachelines: 1, members: 7 */
	/* sum members: 48, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 56 bytes */
};
struct trace_event_raw_block_bio_complete {
	struct trace_entry         ent;                  /*     0     8 */
	dev_t                      dev;                  /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	sector_t                   sector;               /*    16     8 */
	unsigned int               nr_sector;            /*    24     4 */
	int                        error;                /*    28     4 */
	char                       rwbs[8];              /*    32     8 */
	char                       __data[];             /*    40     0 */

	/* size: 40, cachelines: 1, members: 7 */
	/* sum members: 36, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct trace_event_raw_block_bio_merge {
	struct trace_entry         ent;                  /*     0     8 */
	dev_t                      dev;                  /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	sector_t                   sector;               /*    16     8 */
	unsigned int               nr_sector;            /*    24     4 */
	char                       rwbs[8];              /*    28     8 */
	char                       comm[16];             /*    36    16 */
	char                       __data[];             /*    52     0 */

	/* size: 56, cachelines: 1, members: 7 */
	/* sum members: 48, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 56 bytes */
};
struct trace_event_raw_block_bio_queue {
	struct trace_entry         ent;                  /*     0     8 */
	dev_t                      dev;                  /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	sector_t                   sector;               /*    16     8 */
	unsigned int               nr_sector;            /*    24     4 */
	char                       rwbs[8];              /*    28     8 */
	char                       comm[16];             /*    36    16 */
	char                       __data[];             /*    52     0 */

	/* size: 56, cachelines: 1, members: 7 */
	/* sum members: 48, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 56 bytes */
};
struct trace_event_raw_block_get_rq {
	struct trace_entry         ent;                  /*     0     8 */
	dev_t                      dev;                  /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	sector_t                   sector;               /*    16     8 */
	unsigned int               nr_sector;            /*    24     4 */
	char                       rwbs[8];              /*    28     8 */
	char                       comm[16];             /*    36    16 */
	char                       __data[];             /*    52     0 */

	/* size: 56, cachelines: 1, members: 7 */
	/* sum members: 48, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 56 bytes */
};
struct trace_event_raw_block_plug {
	struct trace_entry         ent;                  /*     0     8 */
	char                       comm[16];             /*     8    16 */
	char                       __data[];             /*    24     0 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_block_unplug {
	struct trace_entry         ent;                  /*     0     8 */
	int                        nr_rq;                /*     8     4 */
	char                       comm[16];             /*    12    16 */
	char                       __data[];             /*    28     0 */

	/* size: 28, cachelines: 1, members: 4 */
	/* last cacheline: 28 bytes */
};
struct trace_event_raw_block_split {
	struct trace_entry         ent;                  /*     0     8 */
	dev_t                      dev;                  /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	sector_t                   sector;               /*    16     8 */
	sector_t                   new_sector;           /*    24     8 */
	char                       rwbs[8];              /*    32     8 */
	char                       comm[16];             /*    40    16 */
	char                       __data[];             /*    56     0 */

	/* size: 56, cachelines: 1, members: 7 */
	/* sum members: 52, holes: 1, sum holes: 4 */
	/* last cacheline: 56 bytes */
};
struct trace_event_raw_block_bio_remap {
	struct trace_entry         ent;                  /*     0     8 */
	dev_t                      dev;                  /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	sector_t                   sector;               /*    16     8 */
	unsigned int               nr_sector;            /*    24     4 */
	dev_t                      old_dev;              /*    28     4 */
	sector_t                   old_sector;           /*    32     8 */
	char                       rwbs[8];              /*    40     8 */
	char                       __data[];             /*    48     0 */

	/* size: 48, cachelines: 1, members: 8 */
	/* sum members: 44, holes: 1, sum holes: 4 */
	/* last cacheline: 48 bytes */
};
struct trace_event_raw_block_rq_remap {
	struct trace_entry         ent;                  /*     0     8 */
	dev_t                      dev;                  /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	sector_t                   sector;               /*    16     8 */
	unsigned int               nr_sector;            /*    24     4 */
	dev_t                      old_dev;              /*    28     4 */
	sector_t                   old_sector;           /*    32     8 */
	unsigned int               nr_bios;              /*    40     4 */
	char                       rwbs[8];              /*    44     8 */
	char                       __data[];             /*    52     0 */

	/* size: 56, cachelines: 1, members: 9 */
	/* sum members: 48, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 56 bytes */
};
struct trace_event_data_offsets_block_buffer {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_block_rq_requeue {
	u32                        cmd;                  /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct trace_event_data_offsets_block_rq_complete {
	u32                        cmd;                  /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct trace_event_data_offsets_block_rq {
	u32                        cmd;                  /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct trace_event_data_offsets_block_bio_bounce {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_block_bio_complete {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_block_bio_merge {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_block_bio_queue {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_block_get_rq {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_block_plug {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_block_unplug {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_block_split {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_block_bio_remap {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_block_rq_remap {

	/* size: 0, cachelines: 0, members: 0 */
};
struct queue_sysfs_entry {
	struct attribute           attr;                 /*     0    16 */

	/* XXX last struct has 6 bytes of padding */

	ssize_t                    (*show)(struct request_queue *, char *); /*    16     8 */
	ssize_t                    (*store)(struct request_queue *, const char  *, size_t); /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* paddings: 1, sum paddings: 6 */
	/* last cacheline: 32 bytes */
};
struct rq_map_data {
	struct page * *            pages;                /*     0     8 */
	int                        page_order;           /*     8     4 */
	int                        nr_entries;           /*    12     4 */
	long unsigned int          offset;               /*    16     8 */
	int                        null_mapped;          /*    24     4 */
	int                        from_user;            /*    28     4 */

	/* size: 32, cachelines: 1, members: 6 */
	/* last cacheline: 32 bytes */
};
struct bio_map_data {
	bool                       is_our_pages:1;       /*     0: 0  1 */
	bool                       is_null_mapped:1;     /*     0: 1  1 */

	/* XXX 6 bits hole, try to pack */
	/* XXX 7 bytes hole, try to pack */

	struct iov_iter            iter;                 /*     8    40 */
	struct iovec               iov[];                /*    48     0 */

	/* size: 48, cachelines: 1, members: 4 */
	/* sum members: 40, holes: 1, sum holes: 7 */
	/* sum bitfield members: 2 bits, bit holes: 1, sum bit holes: 6 bits */
	/* last cacheline: 48 bytes */
};
struct req_iterator {
	struct bvec_iter           iter;                 /*     0    24 */

	/* XXX last struct has 4 bytes of padding */

	struct bio *               bio;                  /*    24     8 */

	/* size: 32, cachelines: 1, members: 2 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 32 bytes */
};
struct mq_inflight {
	struct hd_struct *         part;                 /*     0     8 */
	unsigned int               inflight[2];          /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct flush_busy_ctx_data {
	struct blk_mq_hw_ctx *     hctx;                 /*     0     8 */
	struct list_head *         list;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct dispatch_rq_data {
	struct blk_mq_hw_ctx *     hctx;                 /*     0     8 */
	struct request *           rq;                   /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct rq_iter_data {
	struct blk_mq_hw_ctx *     hctx;                 /*     0     8 */
	bool                       has_rq;               /*     8     1 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 7 */
	/* last cacheline: 16 bytes */
};
struct blk_mq_qe_pair {
	struct list_head           node;                 /*     0    16 */
	struct request_queue *     q;                    /*    16     8 */
	struct elevator_type *     type;                 /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* last cacheline: 32 bytes */
};
struct sbq_wait {
	struct sbitmap_queue *     sbq;                  /*     0     8 */
	struct wait_queue_entry    wait;                 /*     8    40 */

	/* size: 48, cachelines: 1, members: 2 */
	/* last cacheline: 48 bytes */
};
struct bt_iter_data {
	struct blk_mq_hw_ctx *     hctx;                 /*     0     8 */
	struct request_queue *     q;                    /*     8     8 */
	busy_iter_fn *             fn;                   /*    16     8 */
	void *                     data;                 /*    24     8 */
	bool                       reserved;             /*    32     1 */

	/* size: 40, cachelines: 1, members: 5 */
	/* padding: 7 */
	/* last cacheline: 40 bytes */
};
struct bt_tags_iter_data {
	struct blk_mq_tags *       tags;                 /*     0     8 */
	busy_tag_iter_fn *         fn;                   /*     8     8 */
	void *                     data;                 /*    16     8 */
	unsigned int               flags;                /*    24     4 */

	/* size: 32, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct blk_queue_stats {
	struct list_head           callbacks;            /*     0    16 */
	spinlock_t                 lock;                 /*    16     4 */
	bool                       enable_accounting;    /*    20     1 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 3 */
	/* last cacheline: 24 bytes */
};
struct blk_mq_ctx_sysfs_entry {
	struct attribute           attr;                 /*     0    16 */

	/* XXX last struct has 6 bytes of padding */

	ssize_t                    (*show)(struct blk_mq_ctx *, char *); /*    16     8 */
	ssize_t                    (*store)(struct blk_mq_ctx *, const char  *, size_t); /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* paddings: 1, sum paddings: 6 */
	/* last cacheline: 32 bytes */
};
struct blk_mq_hw_ctx_sysfs_entry {
	struct attribute           attr;                 /*     0    16 */

	/* XXX last struct has 6 bytes of padding */

	ssize_t                    (*show)(struct blk_mq_hw_ctx *, char *); /*    16     8 */
	ssize_t                    (*store)(struct blk_mq_hw_ctx *, const char  *, size_t); /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* paddings: 1, sum paddings: 6 */
	/* last cacheline: 32 bytes */
};
struct hd_geometry {
	unsigned char              heads;                /*     0     1 */
	unsigned char              sectors;              /*     1     1 */
	short unsigned int         cylinders;            /*     2     2 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          start;                /*     8     8 */

	/* size: 16, cachelines: 1, members: 4 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct pr_ops {
	int                        (*pr_register)(struct block_device *, u64, u64, u32); /*     0     8 */
	int                        (*pr_reserve)(struct block_device *, u64, enum pr_type, u32); /*     8     8 */
	int                        (*pr_release)(struct block_device *, u64, enum pr_type); /*    16     8 */
	int                        (*pr_preempt)(struct block_device *, u64, u64, enum pr_type, bool); /*    24     8 */
	int                        (*pr_clear)(struct block_device *, u64); /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct blkpg_ioctl_arg {
	int                        op;                   /*     0     4 */
	int                        flags;                /*     4     4 */
	int                        datalen;              /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     data;                 /*    16     8 */

	/* size: 24, cachelines: 1, members: 4 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct blkpg_partition {
	long long int              start;                /*     0     8 */
	long long int              length;               /*     8     8 */
	int                        pno;                  /*    16     4 */
	char                       devname[64];          /*    20    64 */
	/* --- cacheline 1 boundary (64 bytes) was 20 bytes ago --- */
	char                       volname[64];          /*    84    64 */

	/* size: 152, cachelines: 3, members: 5 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct pr_reservation {
	__u64                      key;                  /*     0     8 */
	__u32                      type;                 /*     8     4 */
	__u32                      flags;                /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct pr_registration {
	__u64                      old_key;              /*     0     8 */
	__u64                      new_key;              /*     8     8 */
	__u32                      flags;                /*    16     4 */
	__u32                      __pad;                /*    20     4 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct pr_preempt {
	__u64                      old_key;              /*     0     8 */
	__u64                      new_key;              /*     8     8 */
	__u32                      type;                 /*    16     4 */
	__u32                      flags;                /*    20     4 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct pr_clear {
	__u64                      key;                  /*     0     8 */
	__u32                      flags;                /*     8     4 */
	__u32                      __pad;                /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct compat_blkpg_ioctl_arg {
	compat_int_t               op;                   /*     0     4 */
	compat_int_t               flags;                /*     4     4 */
	compat_int_t               datalen;              /*     8     4 */
	compat_caddr_t             data;                 /*    12     4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct compat_hd_geometry {
	unsigned char              heads;                /*     0     1 */
	unsigned char              sectors;              /*     1     1 */
	short unsigned int         cylinders;            /*     2     2 */
	u32                        start;                /*     4     4 */

	/* size: 8, cachelines: 1, members: 4 */
	/* last cacheline: 8 bytes */
};
struct klist {
	spinlock_t                 k_lock;               /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           k_list;               /*     8    16 */
	void                       (*get)(struct klist_node *); /*    24     8 */
	void                       (*put)(struct klist_node *); /*    32     8 */

	/* size: 40, cachelines: 1, members: 4 */
	/* sum members: 36, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
} __attribute__((__aligned__(8)));
struct klist_node {
	void *                     n_klist;              /*     0     8 */
	struct list_head           n_node;               /*     8    16 */
	struct kref                n_ref;                /*    24     4 */

	/* size: 32, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct klist_iter {
	struct klist *             i_klist;              /*     0     8 */
	struct klist_node *        i_cur;                /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct class_dev_iter {
	struct klist_iter          ki;                   /*     0    16 */
	const struct device_type  * type;                /*    16     8 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct disk_events {
	struct list_head           node;                 /*     0    16 */
	struct gendisk *           disk;                 /*    16     8 */
	spinlock_t                 lock;                 /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	struct mutex               block_mutex;          /*    32    32 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        block;                /*    64     4 */
	unsigned int               pending;              /*    68     4 */
	unsigned int               clearing;             /*    72     4 */

	/* XXX 4 bytes hole, try to pack */

	long int                   poll_msecs;           /*    80     8 */
	struct delayed_work        dwork;                /*    88   192 */

	/* size: 280, cachelines: 5, members: 9 */
	/* sum members: 272, holes: 2, sum holes: 8 */
	/* last cacheline: 24 bytes */
};
struct badblocks {
	struct device *            dev;                  /*     0     8 */
	int                        count;                /*     8     4 */
	int                        unacked_exist;        /*    12     4 */
	int                        shift;                /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	u64 *                      page;                 /*    24     8 */
	int                        changed;              /*    32     4 */
	seqlock_t                  lock;                 /*    36     8 */

	/* XXX 4 bytes hole, try to pack */

	sector_t                   sector;               /*    48     8 */
	sector_t                   size;                 /*    56     8 */

	/* size: 64, cachelines: 1, members: 9 */
	/* sum members: 56, holes: 2, sum holes: 8 */
};
struct disk_part_iter {
	struct gendisk *           disk;                 /*     0     8 */
	struct hd_struct *         part;                 /*     8     8 */
	int                        idx;                  /*    16     4 */
	unsigned int               flags;                /*    20     4 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct blk_major_name {
	struct blk_major_name *    next;                 /*     0     8 */
	int                        major;                /*     8     4 */
	char                       name[16];             /*    12    16 */

	/* size: 32, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct parsed_partitions {
	struct block_device *      bdev;                 /*     0     8 */
	char                       name[32];             /*     8    32 */
	struct {
		sector_t   from;                 /*     0     8 */
		sector_t   size;                 /*     8     8 */
		int        flags;                /*    16     4 */
		bool       has_info;             /*    20     1 */
		struct partition_meta_info info; /*    21   101 */
	} * parts; /*    40     8 */
	int                        next;                 /*    48     4 */
	int                        limit;                /*    52     4 */
	bool                       access_beyond_eod;    /*    56     1 */

	/* XXX 7 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	char *                     pp_buf;               /*    64     8 */

	/* size: 72, cachelines: 2, members: 7 */
	/* sum members: 65, holes: 1, sum holes: 7 */
	/* last cacheline: 8 bytes */
};
struct RigidDiskBlock {
	__u32                      rdb_ID;               /*     0     4 */
	__be32                     rdb_SummedLongs;      /*     4     4 */
	__s32                      rdb_ChkSum;           /*     8     4 */
	__u32                      rdb_HostID;           /*    12     4 */
	__be32                     rdb_BlockBytes;       /*    16     4 */
	__u32                      rdb_Flags;            /*    20     4 */
	__u32                      rdb_BadBlockList;     /*    24     4 */
	__be32                     rdb_PartitionList;    /*    28     4 */
	__u32                      rdb_FileSysHeaderList; /*    32     4 */
	__u32                      rdb_DriveInit;        /*    36     4 */
	__u32                      rdb_Reserved1[6];     /*    40    24 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	__u32                      rdb_Cylinders;        /*    64     4 */
	__u32                      rdb_Sectors;          /*    68     4 */
	__u32                      rdb_Heads;            /*    72     4 */
	__u32                      rdb_Interleave;       /*    76     4 */
	__u32                      rdb_Park;             /*    80     4 */
	__u32                      rdb_Reserved2[3];     /*    84    12 */
	__u32                      rdb_WritePreComp;     /*    96     4 */
	__u32                      rdb_ReducedWrite;     /*   100     4 */
	__u32                      rdb_StepRate;         /*   104     4 */
	__u32                      rdb_Reserved3[5];     /*   108    20 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	__u32                      rdb_RDBBlocksLo;      /*   128     4 */
	__u32                      rdb_RDBBlocksHi;      /*   132     4 */
	__u32                      rdb_LoCylinder;       /*   136     4 */
	__u32                      rdb_HiCylinder;       /*   140     4 */
	__u32                      rdb_CylBlocks;        /*   144     4 */
	__u32                      rdb_AutoParkSeconds;  /*   148     4 */
	__u32                      rdb_HighRDSKBlock;    /*   152     4 */
	__u32                      rdb_Reserved4;        /*   156     4 */
	char                       rdb_DiskVendor[8];    /*   160     8 */
	char                       rdb_DiskProduct[16];  /*   168    16 */
	char                       rdb_DiskRevision[4];  /*   184     4 */
	char                       rdb_ControllerVendor[8]; /*   188     8 */
	/* --- cacheline 3 boundary (192 bytes) was 4 bytes ago --- */
	char                       rdb_ControllerProduct[16]; /*   196    16 */
	char                       rdb_ControllerRevision[4]; /*   212     4 */
	__u32                      rdb_Reserved5[10];    /*   216    40 */

	/* size: 256, cachelines: 4, members: 36 */
};
struct PartitionBlock {
	__be32                     pb_ID;                /*     0     4 */
	__be32                     pb_SummedLongs;       /*     4     4 */
	__s32                      pb_ChkSum;            /*     8     4 */
	__u32                      pb_HostID;            /*    12     4 */
	__be32                     pb_Next;              /*    16     4 */
	__u32                      pb_Flags;             /*    20     4 */
	__u32                      pb_Reserved1[2];      /*    24     8 */
	__u32                      pb_DevFlags;          /*    32     4 */
	__u8                       pb_DriveName[32];     /*    36    32 */
	/* --- cacheline 1 boundary (64 bytes) was 4 bytes ago --- */
	__u32                      pb_Reserved2[15];     /*    68    60 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	__be32                     pb_Environment[17];   /*   128    68 */
	/* --- cacheline 3 boundary (192 bytes) was 4 bytes ago --- */
	__u32                      pb_EReserved[15];     /*   196    60 */

	/* size: 256, cachelines: 4, members: 12 */
};
struct mac_partition {
	__be16                     signature;            /*     0     2 */
	__be16                     res1;                 /*     2     2 */
	__be32                     map_count;            /*     4     4 */
	__be32                     start_block;          /*     8     4 */
	__be32                     block_count;          /*    12     4 */
	char                       name[32];             /*    16    32 */
	char                       type[32];             /*    48    32 */
	/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
	__be32                     data_start;           /*    80     4 */
	__be32                     data_count;           /*    84     4 */
	__be32                     status;               /*    88     4 */
	__be32                     boot_start;           /*    92     4 */
	__be32                     boot_size;            /*    96     4 */
	__be32                     boot_load;            /*   100     4 */
	__be32                     boot_load2;           /*   104     4 */
	__be32                     boot_entry;           /*   108     4 */
	__be32                     boot_entry2;          /*   112     4 */
	__be32                     boot_cksum;           /*   116     4 */
	char                       processor[16];        /*   120    16 */

	/* size: 136, cachelines: 3, members: 18 */
	/* last cacheline: 8 bytes */
};
struct mac_driver_desc {
	__be16                     signature;            /*     0     2 */
	__be16                     block_size;           /*     2     2 */
	__be32                     block_count;          /*     4     4 */

	/* size: 8, cachelines: 1, members: 3 */
	/* last cacheline: 8 bytes */
};
struct fat_boot_sector {
	__u8                       ignored[3];           /*     0     3 */
	__u8                       system_id[8];         /*     3     8 */
	__u8                       sector_size[2];       /*    11     2 */
	__u8                       sec_per_clus;         /*    13     1 */
	__le16                     reserved;             /*    14     2 */
	__u8                       fats;                 /*    16     1 */
	__u8                       dir_entries[2];       /*    17     2 */
	__u8                       sectors[2];           /*    19     2 */
	__u8                       media;                /*    21     1 */
	__le16                     fat_length;           /*    22     2 */
	__le16                     secs_track;           /*    24     2 */
	__le16                     heads;                /*    26     2 */
	__le32                     hidden;               /*    28     4 */
	__le32                     total_sect;           /*    32     4 */
	union {
		struct {
			__u8       drive_number;         /*    36     1 */
			__u8       state;                /*    37     1 */
			__u8       signature;            /*    38     1 */
			__u8       vol_id[4];            /*    39     4 */
			__u8       vol_label[11];        /*    43    11 */
			__u8       fs_type[8];           /*    54     8 */
		} fat16;                                 /*    36    26 */
		struct {
			__le32     length;               /*    36     4 */
			__le16     flags;                /*    40     2 */
			__u8       version[2];           /*    42     2 */
			__le32     root_cluster;         /*    44     4 */
			__le16     info_sector;          /*    48     2 */
			__le16     backup_boot;          /*    50     2 */
			__le16     reserved2[6];         /*    52    12 */
			/* --- cacheline 1 boundary (64 bytes) --- */
			__u8       drive_number;         /*    64     1 */
			__u8       state;                /*    65     1 */
			__u8       signature;            /*    66     1 */
			__u8       vol_id[4];            /*    67     4 */
			__u8       vol_label[11];        /*    71    11 */
			__u8       fs_type[8];           /*    82     8 */
		} fat32;                                 /*    36    56 */
	};                                               /*    36    56 */

	/* size: 92, cachelines: 2, members: 15 */
	/* last cacheline: 28 bytes */
};
struct msdos_partition {
	u8                         boot_ind;             /*     0     1 */
	u8                         head;                 /*     1     1 */
	u8                         sector;               /*     2     1 */
	u8                         cyl;                  /*     3     1 */
	u8                         sys_ind;              /*     4     1 */
	u8                         end_head;             /*     5     1 */
	u8                         end_sector;           /*     6     1 */
	u8                         end_cyl;              /*     7     1 */
	__le32                     start_sect;           /*     8     4 */
	__le32                     nr_sects;             /*    12     4 */

	/* size: 16, cachelines: 1, members: 10 */
	/* last cacheline: 16 bytes */
};
struct solaris_x86_slice {
	__le16                     s_tag;                /*     0     2 */
	__le16                     s_flag;               /*     2     2 */
	__le32                     s_start;              /*     4     4 */
	__le32                     s_size;               /*     8     4 */

	/* size: 12, cachelines: 1, members: 4 */
	/* last cacheline: 12 bytes */
};
struct solaris_x86_vtoc {
	unsigned int               v_bootinfo[3];        /*     0    12 */
	__le32                     v_sanity;             /*    12     4 */
	__le32                     v_version;            /*    16     4 */
	char                       v_volume[8];          /*    20     8 */
	__le16                     v_sectorsz;           /*    28     2 */
	__le16                     v_nparts;             /*    30     2 */
	unsigned int               v_reserved[10];       /*    32    40 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	struct solaris_x86_slice   v_slice[16];          /*    72   192 */
	/* --- cacheline 4 boundary (256 bytes) was 8 bytes ago --- */
	unsigned int               timestamp[16];        /*   264    64 */
	/* --- cacheline 5 boundary (320 bytes) was 8 bytes ago --- */
	char                       v_asciilabel[128];    /*   328   128 */

	/* size: 456, cachelines: 8, members: 10 */
	/* last cacheline: 8 bytes */
};
struct bsd_partition {
	__le32                     p_size;               /*     0     4 */
	__le32                     p_offset;             /*     4     4 */
	__le32                     p_fsize;              /*     8     4 */
	__u8                       p_fstype;             /*    12     1 */
	__u8                       p_frag;               /*    13     1 */
	__le16                     p_cpg;                /*    14     2 */

	/* size: 16, cachelines: 1, members: 6 */
	/* last cacheline: 16 bytes */
};
struct bsd_disklabel {
	__le32                     d_magic;              /*     0     4 */
	__s16                      d_type;               /*     4     2 */
	__s16                      d_subtype;            /*     6     2 */
	char                       d_typename[16];       /*     8    16 */
	char                       d_packname[16];       /*    24    16 */
	__u32                      d_secsize;            /*    40     4 */
	__u32                      d_nsectors;           /*    44     4 */
	__u32                      d_ntracks;            /*    48     4 */
	__u32                      d_ncylinders;         /*    52     4 */
	__u32                      d_secpercyl;          /*    56     4 */
	__u32                      d_secperunit;         /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	__u16                      d_sparespertrack;     /*    64     2 */
	__u16                      d_sparespercyl;       /*    66     2 */
	__u32                      d_acylinders;         /*    68     4 */
	__u16                      d_rpm;                /*    72     2 */
	__u16                      d_interleave;         /*    74     2 */
	__u16                      d_trackskew;          /*    76     2 */
	__u16                      d_cylskew;            /*    78     2 */
	__u32                      d_headswitch;         /*    80     4 */
	__u32                      d_trkseek;            /*    84     4 */
	__u32                      d_flags;              /*    88     4 */
	__u32                      d_drivedata[5];       /*    92    20 */
	__u32                      d_spare[5];           /*   112    20 */
	/* --- cacheline 2 boundary (128 bytes) was 4 bytes ago --- */
	__le32                     d_magic2;             /*   132     4 */
	__le16                     d_checksum;           /*   136     2 */
	__le16                     d_npartitions;        /*   138     2 */
	__le32                     d_bbsize;             /*   140     4 */
	__le32                     d_sbsize;             /*   144     4 */
	struct bsd_partition       d_partitions[16];     /*   148   256 */

	/* size: 404, cachelines: 7, members: 29 */
	/* last cacheline: 20 bytes */
};
struct unixware_slice {
	__le16                     s_label;              /*     0     2 */
	__le16                     s_flags;              /*     2     2 */
	__le32                     start_sect;           /*     4     4 */
	__le32                     nr_sects;             /*     8     4 */

	/* size: 12, cachelines: 1, members: 4 */
	/* last cacheline: 12 bytes */
};
struct unixware_vtoc {
	__le32                     v_magic;              /*     0     4 */
	__le32                     v_version;            /*     4     4 */
	char                       v_name[8];            /*     8     8 */
	__le16                     v_nslices;            /*    16     2 */
	__le16                     v_unknown1;           /*    18     2 */
	__le32                     v_reserved[10];       /*    20    40 */
	struct unixware_slice      v_slice[16];          /*    60   192 */

	/* size: 252, cachelines: 4, members: 7 */
	/* last cacheline: 60 bytes */
};
struct unixware_disklabel {
	__le32                     d_type;               /*     0     4 */
	__le32                     d_magic;              /*     4     4 */
	__le32                     d_version;            /*     8     4 */
	char                       d_serial[12];         /*    12    12 */
	__le32                     d_ncylinders;         /*    24     4 */
	__le32                     d_ntracks;            /*    28     4 */
	__le32                     d_nsectors;           /*    32     4 */
	__le32                     d_secsize;            /*    36     4 */
	__le32                     d_part_start;         /*    40     4 */
	__le32                     d_unknown1[12];       /*    44    48 */
	/* --- cacheline 1 boundary (64 bytes) was 28 bytes ago --- */
	__le32                     d_alt_tbl;            /*    92     4 */
	__le32                     d_alt_len;            /*    96     4 */
	__le32                     d_phys_cyl;           /*   100     4 */
	__le32                     d_phys_trk;           /*   104     4 */
	__le32                     d_phys_sec;           /*   108     4 */
	__le32                     d_phys_bytes;         /*   112     4 */
	__le32                     d_unknown2;           /*   116     4 */
	__le32                     d_unknown3;           /*   120     4 */
	__le32                     d_pad[8];             /*   124    32 */
	/* --- cacheline 2 boundary (128 bytes) was 28 bytes ago --- */
	struct unixware_vtoc       vtoc;                 /*   156   252 */

	/* size: 408, cachelines: 7, members: 20 */
	/* last cacheline: 24 bytes */
};
struct sgi_volume {
	s8                         name[8];              /*     0     8 */
	__be32                     block_num;            /*     8     4 */
	__be32                     num_bytes;            /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct sgi_partition {
	__be32                     num_blocks;           /*     0     4 */
	__be32                     first_block;          /*     4     4 */
	__be32                     type;                 /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct sgi_disklabel {
	__be32                     magic_mushroom;       /*     0     4 */
	__be16                     root_part_num;        /*     4     2 */
	__be16                     swap_part_num;        /*     6     2 */
	s8                         boot_file[16];        /*     8    16 */
	u8                         _unused0[48];         /*    24    48 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	struct sgi_volume          volume[15];           /*    72   240 */
	/* --- cacheline 4 boundary (256 bytes) was 56 bytes ago --- */
	struct sgi_partition       partitions[16];       /*   312   192 */
	/* --- cacheline 7 boundary (448 bytes) was 56 bytes ago --- */
	__be32                     csum;                 /*   504     4 */
	__be32                     _unused1;             /*   508     4 */

	/* size: 512, cachelines: 8, members: 9 */
};
struct _gpt_header {
	__le64                     signature;            /*     0     8 */
	__le32                     revision;             /*     8     4 */
	__le32                     header_size;          /*    12     4 */
	__le32                     header_crc32;         /*    16     4 */
	__le32                     reserved1;            /*    20     4 */
	__le64                     my_lba;               /*    24     8 */
	__le64                     alternate_lba;        /*    32     8 */
	__le64                     first_usable_lba;     /*    40     8 */
	__le64                     last_usable_lba;      /*    48     8 */
	efi_guid_t                 disk_guid __attribute__((__aligned__(1))); /*    56    16 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	__le64                     partition_entry_lba;  /*    72     8 */
	__le32                     num_partition_entries; /*    80     4 */
	__le32                     sizeof_partition_entry; /*    84     4 */
	__le32                     partition_entry_array_crc32; /*    88     4 */

	/* size: 92, cachelines: 2, members: 14 */
	/* forced alignments: 1 */
	/* last cacheline: 28 bytes */
} __attribute__((__packed__));
struct _gpt_entry_attributes {
	u64                        required_to_function:1; /*     0: 0  8 */
	u64                        reserved:47;          /*     0: 1  8 */
	u64                        type_guid_specific:16; /*     0:48  8 */

	/* size: 8, cachelines: 1, members: 3 */
	/* last cacheline: 8 bytes */
};
struct _gpt_entry {
	efi_guid_t                 partition_type_guid __attribute__((__aligned__(1))); /*     0    16 */
	efi_guid_t                 unique_partition_guid __attribute__((__aligned__(1))); /*    16    16 */
	__le64                     starting_lba;         /*    32     8 */
	__le64                     ending_lba;           /*    40     8 */
	gpt_entry_attributes       attributes;           /*    48     8 */
	__le16                     partition_name[36];   /*    56    72 */

	/* size: 128, cachelines: 2, members: 6 */
	/* forced alignments: 2 */
} __attribute__((__packed__));
struct _gpt_mbr_record {
	u8                         boot_indicator;       /*     0     1 */
	u8                         start_head;           /*     1     1 */
	u8                         start_sector;         /*     2     1 */
	u8                         start_track;          /*     3     1 */
	u8                         os_type;              /*     4     1 */
	u8                         end_head;             /*     5     1 */
	u8                         end_sector;           /*     6     1 */
	u8                         end_track;            /*     7     1 */
	__le32                     starting_lba;         /*     8     4 */
	__le32                     size_in_lba;          /*    12     4 */

	/* size: 16, cachelines: 1, members: 10 */
	/* last cacheline: 16 bytes */
};
struct _legacy_mbr {
	u8                         boot_code[440];       /*     0   440 */
	/* --- cacheline 6 boundary (384 bytes) was 56 bytes ago --- */
	__le32                     unique_mbr_signature; /*   440     4 */
	__le16                     unknown;              /*   444     2 */
	gpt_mbr_record             partition_record[4];  /*   446    64 */
	/* --- cacheline 7 boundary (448 bytes) was 62 bytes ago --- */
	__le16                     signature;            /*   510     2 */

	/* size: 512, cachelines: 8, members: 5 */
} __attribute__((__packed__));
struct rq_wait {
	wait_queue_head_t          wait;                 /*     0    24 */
	atomic_t                   inflight;             /*    24     4 */

	/* size: 32, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct rq_depth {
	unsigned int               max_depth;            /*     0     4 */
	int                        scale_step;           /*     4     4 */
	bool                       scaled_max;           /*     8     1 */

	/* XXX 3 bytes hole, try to pack */

	unsigned int               queue_depth;          /*    12     4 */
	unsigned int               default_depth;        /*    16     4 */

	/* size: 20, cachelines: 1, members: 5 */
	/* sum members: 17, holes: 1, sum holes: 3 */
	/* last cacheline: 20 bytes */
};
struct rq_qos_wait_data {
	struct wait_queue_entry    wq;                   /*     0    40 */
	struct task_struct *       task;                 /*    40     8 */
	struct rq_wait *           rqw;                  /*    48     8 */
	acquire_inflight_cb_t *    cb;                   /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	void *                     private_data;         /*    64     8 */
	bool                       got_token;            /*    72     1 */

	/* size: 80, cachelines: 2, members: 6 */
	/* padding: 7 */
	/* last cacheline: 16 bytes */
};
struct cdrom_device_info {
	const struct cdrom_device_ops  * ops;            /*     0     8 */
	struct list_head           list;                 /*     8    16 */
	struct gendisk *           disk;                 /*    24     8 */
	void *                     handle;               /*    32     8 */
	int                        mask;                 /*    40     4 */
	int                        speed;                /*    44     4 */
	int                        capacity;             /*    48     4 */
	unsigned int               options:30;           /*    52: 0  4 */
	unsigned int               mc_flags:2;           /*    52:30  4 */
	unsigned int               vfs_events;           /*    56     4 */
	unsigned int               ioctl_events;         /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        use_count;            /*    64     4 */
	char                       name[20];             /*    68    20 */
	__u8                       sanyo_slot:2;         /*    88: 0  1 */
	__u8                       keeplocked:1;         /*    88: 2  1 */
	__u8                       reserved:5;           /*    88: 3  1 */

	/* XXX 3 bytes hole, try to pack */

	int                        cdda_method;          /*    92     4 */
	__u8                       last_sense;           /*    96     1 */
	__u8                       media_written;        /*    97     1 */
	short unsigned int         mmc3_profile;         /*    98     2 */
	int                        for_data;             /*   100     4 */
	int                        (*exit)(struct cdrom_device_info *); /*   104     8 */
	int                        mrw_mode_page;        /*   112     4 */

	/* size: 120, cachelines: 2, members: 23 */
	/* sum members: 108, holes: 1, sum holes: 3 */
	/* sum bitfield members: 40 bits (5 bytes) */
	/* padding: 4 */
	/* last cacheline: 56 bytes */
};
struct scsi_sense_hdr {
	u8                         response_code;        /*     0     1 */
	u8                         sense_key;            /*     1     1 */
	u8                         asc;                  /*     2     1 */
	u8                         ascq;                 /*     3     1 */
	u8                         byte4;                /*     4     1 */
	u8                         byte5;                /*     5     1 */
	u8                         byte6;                /*     6     1 */
	u8                         additional_length;    /*     7     1 */

	/* size: 8, cachelines: 1, members: 8 */
	/* last cacheline: 8 bytes */
};
struct cdrom_msf0 {
	__u8                       minute;               /*     0     1 */
	__u8                       second;               /*     1     1 */
	__u8                       frame;                /*     2     1 */

	/* size: 3, cachelines: 1, members: 3 */
	/* last cacheline: 3 bytes */
};
union cdrom_addr {
	struct cdrom_msf0          msf;                /*     0     3 */
	int                        lba;                /*     0     4 */
};
struct cdrom_multisession {
	union cdrom_addr           addr;                 /*     0     4 */
	__u8                       xa_flag;              /*     4     1 */
	__u8                       addr_format;          /*     5     1 */

	/* size: 8, cachelines: 1, members: 3 */
	/* padding: 2 */
	/* last cacheline: 8 bytes */
};
struct cdrom_mcn {
	__u8                       medium_catalog_number[14]; /*     0    14 */

	/* size: 14, cachelines: 1, members: 1 */
	/* last cacheline: 14 bytes */
};
struct cdrom_generic_command {
	unsigned char              cmd[12];              /*     0    12 */

	/* XXX 4 bytes hole, try to pack */

	unsigned char *            buffer;               /*    16     8 */
	unsigned int               buflen;               /*    24     4 */
	int                        stat;                 /*    28     4 */
	struct request_sense *     sense;                /*    32     8 */
	unsigned char              data_direction;       /*    40     1 */

	/* XXX 3 bytes hole, try to pack */

	int                        quiet;                /*    44     4 */
	int                        timeout;              /*    48     4 */

	/* XXX 4 bytes hole, try to pack */

	union {
		void *             reserved[1];          /*    56     8 */
		void *             unused;               /*    56     8 */
	};                                               /*    56     8 */

	/* size: 64, cachelines: 1, members: 9 */
	/* sum members: 53, holes: 3, sum holes: 11 */
};
struct request_sense {
	__u8                       error_code:7;         /*     0: 0  1 */
	__u8                       valid:1;              /*     0: 7  1 */
	__u8                       segment_number;       /*     1     1 */
	__u8                       sense_key:4;          /*     2: 0  1 */
	__u8                       reserved2:1;          /*     2: 4  1 */
	__u8                       ili:1;                /*     2: 5  1 */
	__u8                       reserved1:2;          /*     2: 6  1 */
	__u8                       information[4];       /*     3     4 */
	__u8                       add_sense_len;        /*     7     1 */
	__u8                       command_info[4];      /*     8     4 */
	__u8                       asc;                  /*    12     1 */
	__u8                       ascq;                 /*    13     1 */
	__u8                       fruc;                 /*    14     1 */
	__u8                       sks[3];               /*    15     3 */
	__u8                       asb[46];              /*    18    46 */

	/* size: 64, cachelines: 1, members: 15 */
};
struct packet_command {
	unsigned char              cmd[12];              /*     0    12 */

	/* XXX 4 bytes hole, try to pack */

	unsigned char *            buffer;               /*    16     8 */
	unsigned int               buflen;               /*    24     4 */
	int                        stat;                 /*    28     4 */
	struct scsi_sense_hdr *    sshdr;                /*    32     8 */
	unsigned char              data_direction;       /*    40     1 */

	/* XXX 3 bytes hole, try to pack */

	int                        quiet;                /*    44     4 */
	int                        timeout;              /*    48     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     reserved[1];          /*    56     8 */

	/* size: 64, cachelines: 1, members: 9 */
	/* sum members: 53, holes: 3, sum holes: 11 */
};
struct cdrom_device_ops {
	int                        (*open)(struct cdrom_device_info *, int); /*     0     8 */
	void                       (*release)(struct cdrom_device_info *); /*     8     8 */
	int                        (*drive_status)(struct cdrom_device_info *, int); /*    16     8 */
	unsigned int               (*check_events)(struct cdrom_device_info *, unsigned int, int); /*    24     8 */
	int                        (*tray_move)(struct cdrom_device_info *, int); /*    32     8 */
	int                        (*lock_door)(struct cdrom_device_info *, int); /*    40     8 */
	int                        (*select_speed)(struct cdrom_device_info *, int); /*    48     8 */
	int                        (*select_disc)(struct cdrom_device_info *, int); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*get_last_session)(struct cdrom_device_info *, struct cdrom_multisession *); /*    64     8 */
	int                        (*get_mcn)(struct cdrom_device_info *, struct cdrom_mcn *); /*    72     8 */
	int                        (*reset)(struct cdrom_device_info *); /*    80     8 */
	int                        (*audio_ioctl)(struct cdrom_device_info *, unsigned int, void *); /*    88     8 */
	const int                  capability;           /*    96     4 */

	/* XXX 4 bytes hole, try to pack */

	int                        (*generic_packet)(struct cdrom_device_info *, struct packet_command *); /*   104     8 */

	/* size: 112, cachelines: 2, members: 14 */
	/* sum members: 108, holes: 1, sum holes: 4 */
	/* last cacheline: 48 bytes */
};
struct scsi_ioctl_command {
	unsigned int               inlen;                /*     0     4 */
	unsigned int               outlen;               /*     4     4 */
	unsigned char              data[];               /*     8     0 */

	/* size: 8, cachelines: 1, members: 3 */
	/* last cacheline: 8 bytes */
};
struct scsi_request {
	unsigned char              __cmd[16];            /*     0    16 */
	unsigned char *            cmd;                  /*    16     8 */
	short unsigned int         cmd_len;              /*    24     2 */

	/* XXX 2 bytes hole, try to pack */

	int                        result;               /*    28     4 */
	unsigned int               sense_len;            /*    32     4 */
	unsigned int               resid_len;            /*    36     4 */
	int                        retries;              /*    40     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     sense;                /*    48     8 */

	/* size: 56, cachelines: 1, members: 8 */
	/* sum members: 50, holes: 2, sum holes: 6 */
	/* last cacheline: 56 bytes */
};
struct sg_io_hdr {
	int                        interface_id;         /*     0     4 */
	int                        dxfer_direction;      /*     4     4 */
	unsigned char              cmd_len;              /*     8     1 */
	unsigned char              mx_sb_len;            /*     9     1 */
	short unsigned int         iovec_count;          /*    10     2 */
	unsigned int               dxfer_len;            /*    12     4 */
	void *                     dxferp;               /*    16     8 */
	unsigned char *            cmdp;                 /*    24     8 */
	void *                     sbp;                  /*    32     8 */
	unsigned int               timeout;              /*    40     4 */
	unsigned int               flags;                /*    44     4 */
	int                        pack_id;              /*    48     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     usr_ptr;              /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	unsigned char              status;               /*    64     1 */
	unsigned char              masked_status;        /*    65     1 */
	unsigned char              msg_status;           /*    66     1 */
	unsigned char              sb_len_wr;            /*    67     1 */
	short unsigned int         host_status;          /*    68     2 */
	short unsigned int         driver_status;        /*    70     2 */
	int                        resid;                /*    72     4 */
	unsigned int               duration;             /*    76     4 */
	unsigned int               info;                 /*    80     4 */

	/* size: 88, cachelines: 2, members: 22 */
	/* sum members: 80, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct compat_sg_io_hdr {
	compat_int_t               interface_id;         /*     0     4 */
	compat_int_t               dxfer_direction;      /*     4     4 */
	unsigned char              cmd_len;              /*     8     1 */
	unsigned char              mx_sb_len;            /*     9     1 */
	short unsigned int         iovec_count;          /*    10     2 */
	compat_uint_t              dxfer_len;            /*    12     4 */
	compat_uint_t              dxferp;               /*    16     4 */
	compat_uptr_t              cmdp;                 /*    20     4 */
	compat_uptr_t              sbp;                  /*    24     4 */
	compat_uint_t              timeout;              /*    28     4 */
	compat_uint_t              flags;                /*    32     4 */
	compat_int_t               pack_id;              /*    36     4 */
	compat_uptr_t              usr_ptr;              /*    40     4 */
	unsigned char              status;               /*    44     1 */
	unsigned char              masked_status;        /*    45     1 */
	unsigned char              msg_status;           /*    46     1 */
	unsigned char              sb_len_wr;            /*    47     1 */
	short unsigned int         host_status;          /*    48     2 */
	short unsigned int         driver_status;        /*    50     2 */
	compat_int_t               resid;                /*    52     4 */
	compat_uint_t              duration;             /*    56     4 */
	compat_uint_t              info;                 /*    60     4 */

	/* size: 64, cachelines: 1, members: 22 */
};
struct blk_cmd_filter {
	long unsigned int          read_ok[4];           /*     0    32 */
	long unsigned int          write_ok[4];          /*    32    32 */

	/* size: 64, cachelines: 1, members: 2 */
};
struct compat_cdrom_generic_command {
	unsigned char              cmd[12];              /*     0    12 */
	compat_caddr_t             buffer;               /*    12     4 */
	compat_uint_t              buflen;               /*    16     4 */
	compat_int_t               stat;                 /*    20     4 */
	compat_caddr_t             sense;                /*    24     4 */
	unsigned char              data_direction;       /*    28     1 */
	unsigned char              pad[3];               /*    29     3 */
	compat_int_t               quiet;                /*    32     4 */
	compat_int_t               timeout;              /*    36     4 */
	compat_caddr_t             unused;               /*    40     4 */

	/* size: 44, cachelines: 1, members: 10 */
	/* last cacheline: 44 bytes */
};
struct bsg_device {
	struct request_queue *     queue;                /*     0     8 */
	spinlock_t                 lock;                 /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct hlist_node          dev_list;             /*    16    16 */
	refcount_t                 ref_count;            /*    32     4 */
	char                       name[20];             /*    36    20 */
	int                        max_queue;            /*    56     4 */

	/* size: 64, cachelines: 1, members: 6 */
	/* sum members: 56, holes: 1, sum holes: 4 */
	/* padding: 4 */
};
struct bsg_job {
	struct device *            dev;                  /*     0     8 */
	struct kref                kref;                 /*     8     4 */
	unsigned int               timeout;              /*    12     4 */
	void *                     request;              /*    16     8 */
	void *                     reply;                /*    24     8 */
	unsigned int               request_len;          /*    32     4 */
	unsigned int               reply_len;            /*    36     4 */
	struct bsg_buffer          request_payload;      /*    40    16 */
	struct bsg_buffer          reply_payload;        /*    56    16 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	int                        result;               /*    72     4 */
	unsigned int               reply_payload_rcv_len; /*    76     4 */
	struct request *           bidi_rq;              /*    80     8 */
	struct bio *               bidi_bio;             /*    88     8 */
	void *                     dd_data;              /*    96     8 */

	/* size: 104, cachelines: 2, members: 14 */
	/* last cacheline: 40 bytes */
};
struct bsg_buffer {
	unsigned int               payload_len;          /*     0     4 */
	int                        sg_cnt;               /*     4     4 */
	struct scatterlist *       sg_list;              /*     8     8 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct bsg_set {
	struct blk_mq_tag_set      tag_set;              /*     0   376 */
	/* --- cacheline 5 boundary (320 bytes) was 56 bytes ago --- */
	bsg_job_fn *               job_fn;               /*   376     8 */
	/* --- cacheline 6 boundary (384 bytes) --- */
	bsg_timeout_fn *           timeout_fn;           /*   384     8 */

	/* size: 392, cachelines: 7, members: 3 */
	/* last cacheline: 8 bytes */
};
struct blkcg_policy {
	int                        plid;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct cftype *            dfl_cftypes;          /*     8     8 */
	struct cftype *            legacy_cftypes;       /*    16     8 */
	blkcg_pol_alloc_cpd_fn *   cpd_alloc_fn;         /*    24     8 */
	blkcg_pol_init_cpd_fn *    cpd_init_fn;          /*    32     8 */
	blkcg_pol_free_cpd_fn *    cpd_free_fn;          /*    40     8 */
	blkcg_pol_bind_cpd_fn *    cpd_bind_fn;          /*    48     8 */
	blkcg_pol_alloc_pd_fn *    pd_alloc_fn;          /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	blkcg_pol_init_pd_fn *     pd_init_fn;           /*    64     8 */
	blkcg_pol_online_pd_fn *   pd_online_fn;         /*    72     8 */
	blkcg_pol_offline_pd_fn *  pd_offline_fn;        /*    80     8 */
	blkcg_pol_free_pd_fn *     pd_free_fn;           /*    88     8 */
	blkcg_pol_reset_pd_stats_fn * pd_reset_stats_fn; /*    96     8 */
	blkcg_pol_stat_pd_fn *     pd_stat_fn;           /*   104     8 */
	u64                        kabi_reserved1;       /*   112     8 */
	u64                        kabi_reserved2;       /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	u64                        kabi_reserved3;       /*   128     8 */
	u64                        kabi_reserved4;       /*   136     8 */

	/* size: 144, cachelines: 3, members: 18 */
	/* sum members: 140, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct blkg_conf_ctx {
	struct gendisk *           disk;                 /*     0     8 */
	struct blkcg_gq *          blkg;                 /*     8     8 */
	char *                     body;                 /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct blkg_rwstat {
	struct percpu_counter      cpu_cnt[5];           /*     0   200 */
	/* --- cacheline 3 boundary (192 bytes) was 8 bytes ago --- */
	atomic64_t                 aux_cnt[5];           /*   200    40 */

	/* size: 240, cachelines: 4, members: 2 */
	/* last cacheline: 48 bytes */
};
struct blkg_rwstat_sample {
	u64                        cnt[5];               /*     0    40 */

	/* size: 40, cachelines: 1, members: 1 */
	/* last cacheline: 40 bytes */
};
struct throtl_data {
	struct throtl_service_queue service_queue;       /*     0   152 */
	/* --- cacheline 2 boundary (128 bytes) was 24 bytes ago --- */
	struct request_queue *     queue;                /*   152     8 */
	unsigned int               nr_queued[2];         /*   160     8 */
	unsigned int               throtl_slice;         /*   168     4 */

	/* XXX 4 bytes hole, try to pack */

	struct work_struct         dispatch_work;        /*   176    64 */
	/* --- cacheline 3 boundary (192 bytes) was 48 bytes ago --- */
	unsigned int               limit_index;          /*   240     4 */
	bool                       limit_valid[2];       /*   244     2 */

	/* XXX 2 bytes hole, try to pack */

	long unsigned int          low_upgrade_time;     /*   248     8 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	long unsigned int          low_downgrade_time;   /*   256     8 */
	unsigned int               scale;                /*   264     4 */

	/* XXX 4 bytes hole, try to pack */

	struct latency_bucket      tmp_buckets[2][9];    /*   272   288 */
	/* --- cacheline 8 boundary (512 bytes) was 48 bytes ago --- */
	struct avg_latency_bucket  avg_buckets[2][9];    /*   560   288 */
	/* --- cacheline 13 boundary (832 bytes) was 16 bytes ago --- */
	struct latency_bucket *    latency_buckets[2];   /*   848    16 */
	long unsigned int          last_calculate_time;  /*   864     8 */
	long unsigned int          filtered_latency;     /*   872     8 */
	bool                       track_bio_latency;    /*   880     1 */

	/* size: 888, cachelines: 14, members: 16 */
	/* sum members: 871, holes: 3, sum holes: 10 */
	/* padding: 7 */
	/* last cacheline: 56 bytes */
};
struct throtl_qnode {
	struct list_head           node;                 /*     0    16 */
	struct bio_list            bios;                 /*    16    16 */
	struct throtl_grp *        tg;                   /*    32     8 */

	/* size: 40, cachelines: 1, members: 3 */
	/* last cacheline: 40 bytes */
};
struct throtl_grp {
	struct blkg_policy_data    pd;                   /*     0    16 */

	/* XXX last struct has 4 bytes of padding */

	struct rb_node             rb_node __attribute__((__aligned__(8))); /*    16    24 */
	struct throtl_data *       td;                   /*    40     8 */
	struct throtl_service_queue service_queue;       /*    48   152 */
	/* --- cacheline 3 boundary (192 bytes) was 8 bytes ago --- */
	struct throtl_qnode        qnode_on_self[2];     /*   200    80 */
	/* --- cacheline 4 boundary (256 bytes) was 24 bytes ago --- */
	struct throtl_qnode        qnode_on_parent[2];   /*   280    80 */
	/* --- cacheline 5 boundary (320 bytes) was 40 bytes ago --- */
	long unsigned int          disptime;             /*   360     8 */
	unsigned int               flags;                /*   368     4 */
	bool                       has_rules[2];         /*   372     2 */

	/* XXX 2 bytes hole, try to pack */

	uint64_t                   bps[2][2];            /*   376    32 */
	/* --- cacheline 6 boundary (384 bytes) was 24 bytes ago --- */
	uint64_t                   bps_conf[2][2];       /*   408    32 */
	unsigned int               iops[2][2];           /*   440    16 */
	/* --- cacheline 7 boundary (448 bytes) was 8 bytes ago --- */
	unsigned int               iops_conf[2][2];      /*   456    16 */
	uint64_t                   bytes_disp[2];        /*   472    16 */
	unsigned int               io_disp[2];           /*   488     8 */
	long unsigned int          last_low_overflow_time[2]; /*   496    16 */
	/* --- cacheline 8 boundary (512 bytes) --- */
	uint64_t                   last_bytes_disp[2];   /*   512    16 */
	unsigned int               last_io_disp[2];      /*   528     8 */
	long unsigned int          last_check_time;      /*   536     8 */
	long unsigned int          latency_target;       /*   544     8 */
	long unsigned int          latency_target_conf;  /*   552     8 */
	long unsigned int          slice_start[2];       /*   560    16 */
	/* --- cacheline 9 boundary (576 bytes) --- */
	long unsigned int          slice_end[2];         /*   576    16 */
	long unsigned int          last_finish_time;     /*   592     8 */
	long unsigned int          checked_last_finish_time; /*   600     8 */
	long unsigned int          avg_idletime;         /*   608     8 */
	long unsigned int          idletime_threshold;   /*   616     8 */
	long unsigned int          idletime_threshold_conf; /*   624     8 */
	unsigned int               bio_cnt;              /*   632     4 */
	unsigned int               bad_bio_cnt;          /*   636     4 */
	/* --- cacheline 10 boundary (640 bytes) --- */
	long unsigned int          bio_cnt_reset_time;   /*   640     8 */
	atomic_t                   io_split_cnt[2];      /*   648     8 */
	atomic_t                   last_io_split_cnt[2]; /*   656     8 */
	struct blkg_rwstat         stat_bytes;           /*   664   240 */
	/* --- cacheline 14 boundary (896 bytes) was 8 bytes ago --- */
	struct blkg_rwstat         stat_ios;             /*   904   240 */

	/* size: 1144, cachelines: 18, members: 35 */
	/* sum members: 1142, holes: 1, sum holes: 2 */
	/* paddings: 1, sum paddings: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 56 bytes */
} __attribute__((__aligned__(8)));
struct throtl_service_queue {
	struct throtl_service_queue * parent_sq;         /*     0     8 */
	struct list_head           queued[2];            /*     8    32 */
	unsigned int               nr_queued[2];         /*    40     8 */
	struct rb_root_cached      pending_tree;         /*    48    16 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	unsigned int               nr_pending;           /*    64     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          first_pending_disptime; /*    72     8 */
	struct timer_list          pending_timer;        /*    80    72 */

	/* size: 152, cachelines: 3, members: 7 */
	/* sum members: 148, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct latency_bucket {
	long unsigned int          total_latency;        /*     0     8 */
	int                        samples;              /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct avg_latency_bucket {
	long unsigned int          latency;              /*     0     8 */
	bool                       valid;                /*     8     1 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 7 */
	/* last cacheline: 16 bytes */
};
struct deadline_data {
	struct rb_root             sort_list[2];         /*     0    16 */
	struct list_head           fifo_list[2];         /*    16    32 */
	struct request *           next_rq[2];           /*    48    16 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	unsigned int               batching;             /*    64     4 */
	unsigned int               starved;              /*    68     4 */
	int                        fifo_expire[2];       /*    72     8 */
	int                        fifo_batch;           /*    80     4 */
	int                        writes_starved;       /*    84     4 */
	int                        front_merges;         /*    88     4 */
	spinlock_t                 lock;                 /*    92     4 */
	spinlock_t                 zone_lock;            /*    96     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           dispatch;             /*   104    16 */

	/* size: 120, cachelines: 2, members: 12 */
	/* sum members: 116, holes: 1, sum holes: 4 */
	/* last cacheline: 56 bytes */
};
struct trace_event_raw_kyber_latency {
	struct trace_entry         ent;                  /*     0     8 */
	dev_t                      dev;                  /*     8     4 */
	char                       domain[16];           /*    12    16 */
	char                       type[8];              /*    28     8 */
	u8                         percentile;           /*    36     1 */
	u8                         numerator;            /*    37     1 */
	u8                         denominator;          /*    38     1 */

	/* XXX 1 byte hole, try to pack */

	unsigned int               samples;              /*    40     4 */
	char                       __data[];             /*    44     0 */

	/* size: 44, cachelines: 1, members: 9 */
	/* sum members: 43, holes: 1, sum holes: 1 */
	/* last cacheline: 44 bytes */
};
struct trace_event_raw_kyber_adjust {
	struct trace_entry         ent;                  /*     0     8 */
	dev_t                      dev;                  /*     8     4 */
	char                       domain[16];           /*    12    16 */
	unsigned int               depth;                /*    28     4 */
	char                       __data[];             /*    32     0 */

	/* size: 32, cachelines: 1, members: 5 */
	/* last cacheline: 32 bytes */
};
struct trace_event_raw_kyber_throttled {
	struct trace_entry         ent;                  /*     0     8 */
	dev_t                      dev;                  /*     8     4 */
	char                       domain[16];           /*    12    16 */
	char                       __data[];             /*    28     0 */

	/* size: 28, cachelines: 1, members: 4 */
	/* last cacheline: 28 bytes */
};
struct trace_event_data_offsets_kyber_latency {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_kyber_adjust {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_kyber_throttled {

	/* size: 0, cachelines: 0, members: 0 */
};
struct kyber_cpu_latency {
	atomic_t                   buckets[3][2][8];     /*     0   192 */

	/* size: 192, cachelines: 3, members: 1 */
};
struct kyber_ctx_queue {
	spinlock_t                 lock;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           rq_list[4];           /*     8    64 */

	/* size: 128, cachelines: 2, members: 2 */
	/* sum members: 68, holes: 1, sum holes: 4 */
	/* padding: 56 */
} __attribute__((__aligned__(64)));
struct kyber_queue_data {
	struct request_queue *     q;                    /*     0     8 */
	struct sbitmap_queue       domain_tokens[4];     /*     8   288 */
	/* --- cacheline 4 boundary (256 bytes) was 40 bytes ago --- */
	unsigned int               async_depth;          /*   296     4 */

	/* XXX 4 bytes hole, try to pack */

	struct kyber_cpu_latency * cpu_latency;          /*   304     8 */
	struct timer_list          timer;                /*   312    72 */
	/* --- cacheline 6 boundary (384 bytes) --- */
	unsigned int               latency_buckets[3][2][8]; /*   384   192 */
	/* --- cacheline 9 boundary (576 bytes) --- */
	long unsigned int          latency_timeout[3];   /*   576    24 */
	int                        domain_p99[3];        /*   600    12 */

	/* XXX 4 bytes hole, try to pack */

	u64                        latency_targets[3];   /*   616    24 */

	/* size: 640, cachelines: 10, members: 9 */
	/* sum members: 632, holes: 2, sum holes: 8 */
};
struct kyber_hctx_data {
	spinlock_t                 lock;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           rqs[4];               /*     8    64 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	unsigned int               cur_domain;           /*    72     4 */
	unsigned int               batching;             /*    76     4 */
	struct kyber_ctx_queue *   kcqs;                 /*    80     8 */
	struct sbitmap             kcq_map[4];           /*    88   128 */
	/* --- cacheline 3 boundary (192 bytes) was 24 bytes ago --- */
	struct sbq_wait            domain_wait[4];       /*   216   192 */
	/* --- cacheline 6 boundary (384 bytes) was 24 bytes ago --- */
	struct sbq_wait_state *    domain_ws[4];         /*   408    32 */
	atomic_t                   wait_index[4];        /*   440    16 */

	/* size: 456, cachelines: 8, members: 9 */
	/* sum members: 452, holes: 1, sum holes: 4 */
	/* last cacheline: 8 bytes */
};
struct flush_kcq_data {
	struct kyber_hctx_data *   khd;                  /*     0     8 */
	unsigned int               sched_domain;         /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head *         list;                 /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct bfq_service_tree {
	struct rb_root             active;               /*     0     8 */
	struct rb_root             idle;                 /*     8     8 */
	struct bfq_entity *        first_idle;           /*    16     8 */
	struct bfq_entity *        last_idle;            /*    24     8 */
	u64                        vtime;                /*    32     8 */
	long unsigned int          wsum;                 /*    40     8 */

	/* size: 48, cachelines: 1, members: 6 */
	/* last cacheline: 48 bytes */
};
struct bfq_entity {
	struct rb_node             rb_node __attribute__((__aligned__(8))); /*     0    24 */
	bool                       on_st_or_in_serv;     /*    24     1 */

	/* XXX 7 bytes hole, try to pack */

	u64                        start;                /*    32     8 */
	u64                        finish;               /*    40     8 */
	struct rb_root *           tree;                 /*    48     8 */
	u64                        min_start;            /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        service;              /*    64     4 */
	int                        budget;               /*    68     4 */
	int                        dev_weight;           /*    72     4 */
	int                        weight;               /*    76     4 */
	int                        new_weight;           /*    80     4 */
	int                        orig_weight;          /*    84     4 */
	struct bfq_entity *        parent;               /*    88     8 */
	struct bfq_sched_data *    my_sched_data;        /*    96     8 */
	struct bfq_sched_data *    sched_data;           /*   104     8 */
	int                        prio_changed;         /*   112     4 */
	bool                       in_groups_with_pending_reqs; /*   116     1 */

	/* size: 120, cachelines: 2, members: 17 */
	/* sum members: 110, holes: 1, sum holes: 7 */
	/* padding: 3 */
	/* forced alignments: 1 */
	/* last cacheline: 56 bytes */
} __attribute__((__aligned__(8)));
struct bfq_sched_data {
	struct bfq_entity *        in_service_entity;    /*     0     8 */
	struct bfq_entity *        next_in_service;      /*     8     8 */
	struct bfq_service_tree    service_tree[3];      /*    16   144 */
	/* --- cacheline 2 boundary (128 bytes) was 32 bytes ago --- */
	long unsigned int          bfq_class_idle_last_service; /*   160     8 */

	/* size: 168, cachelines: 3, members: 4 */
	/* last cacheline: 40 bytes */
};
struct bfq_weight_counter {
	unsigned int               weight;               /*     0     4 */
	unsigned int               num_active;           /*     4     4 */
	struct rb_node             weights_node __attribute__((__aligned__(8))); /*     8    24 */

	/* size: 32, cachelines: 1, members: 3 */
	/* forced alignments: 1 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct bfq_ttime {
	u64                        last_end_request;     /*     0     8 */
	u64                        ttime_total;          /*     8     8 */
	long unsigned int          ttime_samples;        /*    16     8 */
	u64                        ttime_mean;           /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct bfq_queue {
	int                        ref;                  /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct bfq_data *          bfqd;                 /*     8     8 */
	short unsigned int         ioprio;               /*    16     2 */
	short unsigned int         ioprio_class;         /*    18     2 */
	short unsigned int         new_ioprio;           /*    20     2 */
	short unsigned int         new_ioprio_class;     /*    22     2 */
	u64                        last_serv_time_ns;    /*    24     8 */
	unsigned int               inject_limit;         /*    32     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          decrease_time_jif;    /*    40     8 */
	struct bfq_queue *         new_bfqq;             /*    48     8 */
	struct rb_node             pos_node __attribute__((__aligned__(8))); /*    56    24 */
	/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
	struct rb_root *           pos_root;             /*    80     8 */
	struct rb_root             sort_list;            /*    88     8 */
	struct request *           next_rq;              /*    96     8 */
	int                        queued[2];            /*   104     8 */
	int                        allocated;            /*   112     4 */
	int                        meta_pending;         /*   116     4 */
	struct list_head           fifo;                 /*   120    16 */
	/* --- cacheline 2 boundary (128 bytes) was 8 bytes ago --- */
	struct bfq_entity          entity __attribute__((__aligned__(8))); /*   136   120 */

	/* XXX last struct has 3 bytes of padding */

	/* --- cacheline 4 boundary (256 bytes) --- */
	struct bfq_weight_counter * weight_counter;      /*   256     8 */
	int                        max_budget;           /*   264     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          budget_timeout;       /*   272     8 */
	int                        dispatched;           /*   280     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          flags;                /*   288     8 */
	struct list_head           bfqq_list;            /*   296    16 */
	struct bfq_ttime           ttime;                /*   312    32 */
	/* --- cacheline 5 boundary (320 bytes) was 24 bytes ago --- */
	u32                        seek_history;         /*   344     4 */

	/* XXX 4 bytes hole, try to pack */

	struct hlist_node          burst_list_node;      /*   352    16 */
	sector_t                   last_request_pos;     /*   368     8 */
	unsigned int               requests_within_timer; /*   376     4 */
	pid_t                      pid;                  /*   380     4 */
	/* --- cacheline 6 boundary (384 bytes) --- */
	struct bfq_io_cq *         bic;                  /*   384     8 */
	long unsigned int          wr_cur_max_time;      /*   392     8 */
	long unsigned int          soft_rt_next_start;   /*   400     8 */
	long unsigned int          last_wr_start_finish; /*   408     8 */
	unsigned int               wr_coeff;             /*   416     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          last_idle_bklogged;   /*   424     8 */
	long unsigned int          service_from_backlogged; /*   432     8 */
	long unsigned int          service_from_wr;      /*   440     8 */
	/* --- cacheline 7 boundary (448 bytes) --- */
	long unsigned int          wr_start_at_switch_to_srt; /*   448     8 */
	long unsigned int          split_time;           /*   456     8 */
	long unsigned int          first_IO_time;        /*   464     8 */
	u32                        max_service_rate;     /*   472     4 */

	/* XXX 4 bytes hole, try to pack */

	struct bfq_queue *         waker_bfqq;           /*   480     8 */
	struct hlist_node          woken_list_node;      /*   488    16 */
	struct hlist_head          woken_list;           /*   504     8 */

	/* size: 512, cachelines: 8, members: 46 */
	/* sum members: 484, holes: 7, sum holes: 28 */
	/* paddings: 1, sum paddings: 3 */
	/* forced alignments: 2 */
} __attribute__((__aligned__(8)));
struct bfq_data {
	struct request_queue *     queue;                /*     0     8 */
	struct list_head           dispatch;             /*     8    16 */
	struct bfq_group *         root_group;           /*    24     8 */
	struct rb_root_cached      queue_weights_tree;   /*    32    16 */
	unsigned int               num_groups_with_pending_reqs; /*    48     4 */
	unsigned int               busy_queues[3];       /*    52    12 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        wr_busy_queues;       /*    64     4 */
	int                        queued;               /*    68     4 */
	int                        rq_in_driver;         /*    72     4 */
	bool                       nonrot_with_queueing; /*    76     1 */

	/* XXX 3 bytes hole, try to pack */

	int                        max_rq_in_driver;     /*    80     4 */
	int                        hw_tag_samples;       /*    84     4 */
	int                        hw_tag;               /*    88     4 */
	int                        budgets_assigned;     /*    92     4 */
	struct hrtimer             idle_slice_timer __attribute__((__aligned__(8))); /*    96    96 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	struct bfq_queue *         in_service_queue;     /*   192     8 */
	sector_t                   last_position;        /*   200     8 */
	sector_t                   in_serv_last_pos;     /*   208     8 */
	u64                        last_completion;      /*   216     8 */
	struct bfq_queue *         last_completed_rq_bfqq; /*   224     8 */
	u64                        last_empty_occupied_ns; /*   232     8 */
	bool                       wait_dispatch;        /*   240     1 */

	/* XXX 7 bytes hole, try to pack */

	struct request *           waited_rq;            /*   248     8 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	bool                       rqs_injected;         /*   256     1 */

	/* XXX 7 bytes hole, try to pack */

	u64                        first_dispatch;       /*   264     8 */
	u64                        last_dispatch;        /*   272     8 */
	ktime_t                    last_budget_start;    /*   280     8 */
	ktime_t                    last_idling_start;    /*   288     8 */
	long unsigned int          last_idling_start_jiffies; /*   296     8 */
	int                        peak_rate_samples;    /*   304     4 */
	u32                        sequential_samples;   /*   308     4 */
	u64                        tot_sectors_dispatched; /*   312     8 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	u32                        last_rq_max_size;     /*   320     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        delta_from_first;     /*   328     8 */
	u32                        peak_rate;            /*   336     4 */
	int                        bfq_max_budget;       /*   340     4 */
	struct list_head           active_list;          /*   344    16 */
	struct list_head           idle_list;            /*   360    16 */
	u64                        bfq_fifo_expire[2];   /*   376    16 */
	/* --- cacheline 6 boundary (384 bytes) was 8 bytes ago --- */
	unsigned int               bfq_back_penalty;     /*   392     4 */
	unsigned int               bfq_back_max;         /*   396     4 */
	u32                        bfq_slice_idle;       /*   400     4 */
	int                        bfq_user_max_budget;  /*   404     4 */
	unsigned int               bfq_timeout;          /*   408     4 */
	unsigned int               bfq_requests_within_timer; /*   412     4 */
	bool                       strict_guarantees;    /*   416     1 */

	/* XXX 7 bytes hole, try to pack */

	long unsigned int          last_ins_in_burst;    /*   424     8 */
	long unsigned int          bfq_burst_interval;   /*   432     8 */
	int                        burst_size;           /*   440     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 7 boundary (448 bytes) --- */
	struct bfq_entity *        burst_parent_entity;  /*   448     8 */
	long unsigned int          bfq_large_burst_thresh; /*   456     8 */
	bool                       large_burst;          /*   464     1 */

	/* XXX 7 bytes hole, try to pack */

	struct hlist_head          burst_list;           /*   472     8 */
	bool                       low_latency;          /*   480     1 */

	/* XXX 3 bytes hole, try to pack */

	unsigned int               bfq_wr_coeff;         /*   484     4 */
	unsigned int               bfq_wr_max_time;      /*   488     4 */
	unsigned int               bfq_wr_rt_max_time;   /*   492     4 */
	unsigned int               bfq_wr_min_idle_time; /*   496     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          bfq_wr_min_inter_arr_async; /*   504     8 */
	/* --- cacheline 8 boundary (512 bytes) --- */
	unsigned int               bfq_wr_max_softrt_rate; /*   512     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        rate_dur_prod;        /*   520     8 */
	struct bfq_queue           oom_bfqq __attribute__((__aligned__(8))); /*   528   512 */
	/* --- cacheline 16 boundary (1024 bytes) was 16 bytes ago --- */
	spinlock_t                 lock;                 /*  1040     4 */

	/* XXX 4 bytes hole, try to pack */

	struct bfq_io_cq *         bio_bic;              /*  1048     8 */
	struct bfq_queue *         bio_bfqq;             /*  1056     8 */
	unsigned int               word_depths[2][2];    /*  1064    16 */

	/* size: 1080, cachelines: 17, members: 66 */
	/* sum members: 1026, holes: 11, sum holes: 54 */
	/* forced alignments: 2 */
	/* last cacheline: 56 bytes */
} __attribute__((__aligned__(8)));
struct bfq_io_cq {
	struct io_cq               icq __attribute__((__aligned__(8))); /*     0    56 */

	/* XXX last struct has 4 bytes of padding */

	struct bfq_queue *         bfqq[2];              /*    56    16 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	int                        ioprio;               /*    72     4 */

	/* XXX 4 bytes hole, try to pack */

	uint64_t                   blkcg_serial_nr;      /*    80     8 */
	bool                       saved_has_short_ttime; /*    88     1 */
	bool                       saved_IO_bound;       /*    89     1 */
	bool                       saved_in_large_burst; /*    90     1 */
	bool                       was_in_burst_list;    /*    91     1 */
	unsigned int               saved_weight;         /*    92     4 */
	long unsigned int          saved_wr_coeff;       /*    96     8 */
	long unsigned int          saved_last_wr_start_finish; /*   104     8 */
	long unsigned int          saved_wr_start_at_switch_to_srt; /*   112     8 */
	unsigned int               saved_wr_cur_max_time; /*   120     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 2 boundary (128 bytes) --- */
	struct bfq_ttime           saved_ttime;          /*   128    32 */

	/* size: 160, cachelines: 3, members: 14 */
	/* sum members: 152, holes: 2, sum holes: 8 */
	/* paddings: 1, sum paddings: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct bfq_group {
	struct blkg_policy_data    pd;                   /*     0    16 */

	/* XXX last struct has 4 bytes of padding */

	char                       blkg_path[128];       /*    16   128 */
	/* --- cacheline 2 boundary (128 bytes) was 16 bytes ago --- */
	int                        ref;                  /*   144     4 */
	bool                       online;               /*   148     1 */

	/* XXX 3 bytes hole, try to pack */

	struct bfq_entity          entity __attribute__((__aligned__(8))); /*   152   120 */

	/* XXX last struct has 3 bytes of padding */

	/* --- cacheline 4 boundary (256 bytes) was 16 bytes ago --- */
	struct bfq_sched_data      sched_data;           /*   272   168 */
	/* --- cacheline 6 boundary (384 bytes) was 56 bytes ago --- */
	void *                     bfqd;                 /*   440     8 */
	/* --- cacheline 7 boundary (448 bytes) --- */
	struct bfq_queue *         async_bfqq[2][8];     /*   448   128 */
	/* --- cacheline 9 boundary (576 bytes) --- */
	struct bfq_queue *         async_idle_bfqq;      /*   576     8 */
	struct bfq_entity *        my_entity;            /*   584     8 */
	int                        active_entities;      /*   592     4 */

	/* XXX 4 bytes hole, try to pack */

	struct rb_root             rq_pos_tree;          /*   600     8 */
	struct bfqg_stats          stats;                /*   608   480 */

	/* size: 1088, cachelines: 17, members: 13 */
	/* sum members: 1081, holes: 2, sum holes: 7 */
	/* paddings: 2, sum paddings: 7 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 3 */
} __attribute__((__aligned__(8)));
struct bfqg_stats {
	struct blkg_rwstat         bytes;                /*     0   240 */
	/* --- cacheline 3 boundary (192 bytes) was 48 bytes ago --- */
	struct blkg_rwstat         ios;                  /*   240   240 */

	/* size: 480, cachelines: 8, members: 2 */
	/* last cacheline: 32 bytes */
};
struct bfq_group_data {
	struct blkcg_policy_data   pd;                   /*     0    16 */

	/* XXX last struct has 4 bytes of padding */

	unsigned int               weight;               /*    16     4 */

	/* size: 24, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 24 bytes */
};
struct integrity_sysfs_entry {
	struct attribute           attr;                 /*     0    16 */

	/* XXX last struct has 6 bytes of padding */

	ssize_t                    (*show)(struct blk_integrity *, char *); /*    16     8 */
	ssize_t                    (*store)(struct blk_integrity *, const char  *, size_t); /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* paddings: 1, sum paddings: 6 */
	/* last cacheline: 32 bytes */
};
struct virtio_device_id {
	__u32                      device;               /*     0     4 */
	__u32                      vendor;               /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct virtqueue {
	struct list_head           list;                 /*     0    16 */
	void                       (*callback)(struct virtqueue *); /*    16     8 */
	const char  *              name;                 /*    24     8 */
	struct virtio_device *     vdev;                 /*    32     8 */
	unsigned int               index;                /*    40     4 */
	unsigned int               num_free;             /*    44     4 */
	void *                     priv;                 /*    48     8 */

	/* size: 56, cachelines: 1, members: 7 */
	/* last cacheline: 56 bytes */
};
struct virtio_device {
	int                        index;                /*     0     4 */
	bool                       failed;               /*     4     1 */
	bool                       config_enabled;       /*     5     1 */
	bool                       config_change_pending; /*     6     1 */

	/* XXX 1 byte hole, try to pack */

	spinlock_t                 config_lock;          /*     8     4 */
	spinlock_t                 vqs_list_lock;        /*    12     4 */
	struct device              dev __attribute__((__aligned__(8))); /*    16  1000 */
	/* --- cacheline 15 boundary (960 bytes) was 56 bytes ago --- */
	struct virtio_device_id    id;                   /*  1016     8 */
	/* --- cacheline 16 boundary (1024 bytes) --- */
	const struct virtio_config_ops  * config;        /*  1024     8 */
	const struct vringh_config_ops  * vringh_config; /*  1032     8 */
	struct list_head           vqs;                  /*  1040    16 */
	u64                        features;             /*  1056     8 */
	void *                     priv;                 /*  1064     8 */

	/* size: 1072, cachelines: 17, members: 13 */
	/* sum members: 1071, holes: 1, sum holes: 1 */
	/* forced alignments: 1 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct virtio_config_ops {
	void                       (*get)(struct virtio_device *, unsigned int, void *, unsigned int); /*     0     8 */
	void                       (*set)(struct virtio_device *, unsigned int, const void  *, unsigned int); /*     8     8 */
	u32                        (*generation)(struct virtio_device *); /*    16     8 */
	u8                         (*get_status)(struct virtio_device *); /*    24     8 */
	void                       (*set_status)(struct virtio_device *, u8); /*    32     8 */
	void                       (*reset)(struct virtio_device *); /*    40     8 */
	int                        (*find_vqs)(struct virtio_device *, unsigned int, struct virtqueue * *, vq_callback_t * *, const char  * const *, const bool  *, struct irq_affinity *); /*    48     8 */
	void                       (*del_vqs)(struct virtio_device *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u64                        (*get_features)(struct virtio_device *); /*    64     8 */
	int                        (*finalize_features)(struct virtio_device *); /*    72     8 */
	const char  *              (*bus_name)(struct virtio_device *); /*    80     8 */
	int                        (*set_vq_affinity)(struct virtqueue *, const struct cpumask  *); /*    88     8 */
	const struct cpumask  *    (*get_vq_affinity)(struct virtio_device *, int); /*    96     8 */
	bool                       (*get_shm_region)(struct virtio_device *, struct virtio_shm_region *, u8); /*   104     8 */

	/* size: 112, cachelines: 2, members: 14 */
	/* last cacheline: 48 bytes */
};
struct virtio_shm_region {
	u64                        addr;                 /*     0     8 */
	u64                        len;                  /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct irq_poll {
	struct list_head           list;                 /*     0    16 */
	long unsigned int          state;                /*    16     8 */
	int                        weight;               /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	irq_poll_fn *              poll;                 /*    32     8 */

	/* size: 40, cachelines: 1, members: 4 */
	/* sum members: 36, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct dim_sample {
	ktime_t                    time;                 /*     0     8 */
	u32                        pkt_ctr;              /*     8     4 */
	u32                        byte_ctr;             /*    12     4 */
	u16                        event_ctr;            /*    16     2 */

	/* XXX 2 bytes hole, try to pack */

	u32                        comp_ctr;             /*    20     4 */

	/* size: 24, cachelines: 1, members: 5 */
	/* sum members: 22, holes: 1, sum holes: 2 */
	/* last cacheline: 24 bytes */
};
struct dim_stats {
	int                        ppms;                 /*     0     4 */
	int                        bpms;                 /*     4     4 */
	int                        epms;                 /*     8     4 */
	int                        cpms;                 /*    12     4 */
	int                        cpe_ratio;            /*    16     4 */

	/* size: 20, cachelines: 1, members: 5 */
	/* last cacheline: 20 bytes */
};
struct dim {
	u8                         state;                /*     0     1 */

	/* XXX 3 bytes hole, try to pack */

	struct dim_stats           prev_stats;           /*     4    20 */
	struct dim_sample          start_sample;         /*    24    24 */
	struct dim_sample          measuring_sample;     /*    48    24 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	struct work_struct         work;                 /*    72    64 */
	/* --- cacheline 2 boundary (128 bytes) was 8 bytes ago --- */
	void *                     priv;                 /*   136     8 */
	u8                         profile_ix;           /*   144     1 */
	u8                         mode;                 /*   145     1 */
	u8                         tune_state;           /*   146     1 */
	u8                         steps_right;          /*   147     1 */
	u8                         steps_left;           /*   148     1 */
	u8                         tired;                /*   149     1 */

	/* size: 152, cachelines: 3, members: 12 */
	/* sum members: 147, holes: 1, sum holes: 3 */
	/* padding: 2 */
	/* last cacheline: 24 bytes */
};
struct rdma_restrack_entry {
	bool                       valid;                /*     0     1 */

	/* XXX 3 bytes hole, try to pack */

	struct kref                kref;                 /*     4     4 */
	struct completion          comp;                 /*     8    32 */
	struct task_struct *       task;                 /*    40     8 */
	const char  *              kern_name;            /*    48     8 */
	enum rdma_restrack_type    type;                 /*    56     4 */
	bool                       user;                 /*    60     1 */

	/* XXX 3 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	u32                        id;                   /*    64     4 */

	/* size: 72, cachelines: 2, members: 8 */
	/* sum members: 62, holes: 2, sum holes: 6 */
	/* padding: 4 */
	/* last cacheline: 8 bytes */
};
struct rdma_link_ops {
	struct list_head           list;                 /*     0    16 */
	const char  *              type;                 /*    16     8 */
	int                        (*newlink)(const char  *, struct net_device *); /*    24     8 */
	u64                        kabi_reserved1;       /*    32     8 */
	u64                        kabi_reserved2;       /*    40     8 */

	/* size: 48, cachelines: 1, members: 5 */
	/* last cacheline: 48 bytes */
};
struct auto_mode_param {
	int                        qp_type;              /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct rdma_counter_mode {
	enum rdma_nl_counter_mode  mode;                 /*     0     4 */
	enum rdma_nl_counter_mask  mask;                 /*     4     4 */
	struct auto_mode_param     param;                /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct rdma_port_counter {
	struct rdma_counter_mode   mode;                 /*     0    12 */

	/* XXX 4 bytes hole, try to pack */

	struct rdma_hw_stats *     hstats;               /*    16     8 */
	unsigned int               num_counters;         /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	struct mutex               lock;                 /*    32    32 */

	/* size: 64, cachelines: 1, members: 4 */
	/* sum members: 56, holes: 2, sum holes: 8 */
};
struct rdma_hw_stats {
	struct mutex               lock;                 /*     0    32 */
	long unsigned int          timestamp;            /*    32     8 */
	long unsigned int          lifespan;             /*    40     8 */
	const char  * const *      names;                /*    48     8 */
	int                        num_counters;         /*    56     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	u64                        value[];              /*    64     0 */

	/* size: 64, cachelines: 1, members: 6 */
	/* sum members: 60, holes: 1, sum holes: 4 */
};
struct rdma_counter {
	struct rdma_restrack_entry res;                  /*     0    72 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	struct ib_device *         device;               /*    72     8 */
	uint32_t                   id;                   /*    80     4 */
	struct kref                kref;                 /*    84     4 */
	struct rdma_counter_mode   mode;                 /*    88    12 */

	/* XXX 4 bytes hole, try to pack */

	struct mutex               lock;                 /*   104    32 */
	/* --- cacheline 2 boundary (128 bytes) was 8 bytes ago --- */
	struct rdma_hw_stats *     stats;                /*   136     8 */
	u8                         port;                 /*   144     1 */

	/* XXX 7 bytes hole, try to pack */

	u64                        kabi_reserved1;       /*   152     8 */

	/* size: 160, cachelines: 3, members: 9 */
	/* sum members: 149, holes: 2, sum holes: 11 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 32 bytes */
};
struct ib_device {
	struct device *            dma_device;           /*     0     8 */
	struct ib_device_ops       ops;                  /*     8   992 */
	/* --- cacheline 15 boundary (960 bytes) was 40 bytes ago --- */
	char                       name[64];             /*  1000    64 */
	/* --- cacheline 16 boundary (1024 bytes) was 40 bytes ago --- */
	struct callback_head       callback_head __attribute__((__aligned__(8))); /*  1064    16 */
	struct list_head           event_handler_list;   /*  1080    16 */
	/* --- cacheline 17 boundary (1088 bytes) was 8 bytes ago --- */
	struct rw_semaphore        event_handler_rwsem;  /*  1096    40 */
	spinlock_t                 qp_open_list_lock;    /*  1136     4 */

	/* XXX 4 bytes hole, try to pack */

	struct rw_semaphore        client_data_rwsem;    /*  1144    40 */
	/* --- cacheline 18 boundary (1152 bytes) was 32 bytes ago --- */
	struct xarray              client_data;          /*  1184    16 */
	struct mutex               unregistration_lock;  /*  1200    32 */
	/* --- cacheline 19 boundary (1216 bytes) was 16 bytes ago --- */
	rwlock_t                   cache_lock;           /*  1232     8 */
	struct ib_port_data *      port_data;            /*  1240     8 */
	int                        num_comp_vectors;     /*  1248     4 */

	/* XXX 4 bytes hole, try to pack */

	union {
		struct device      dev __attribute__((__aligned__(8))); /*  1256  1000 */
		struct ib_core_device coredev __attribute__((__aligned__(8))); /*  1256  1040 */
	} __attribute__((__aligned__(8)));               /*  1256  1040 */
	/* --- cacheline 35 boundary (2240 bytes) was 56 bytes ago --- */
	const struct attribute_group  * groups[3];       /*  2296    24 */
	/* --- cacheline 36 boundary (2304 bytes) was 16 bytes ago --- */
	u64                        uverbs_cmd_mask;      /*  2320     8 */
	u64                        uverbs_ex_cmd_mask;   /*  2328     8 */
	char                       node_desc[64];        /*  2336    64 */
	/* --- cacheline 37 boundary (2368 bytes) was 32 bytes ago --- */
	__be64                     node_guid;            /*  2400     8 */
	u32                        local_dma_lkey;       /*  2408     4 */
	u16                        is_switch:1;          /*  2412: 0  2 */
	u16                        kverbs_provider:1;    /*  2412: 1  2 */
	u16                        use_cq_dim:1;         /*  2412: 2  2 */

	/* XXX 5 bits hole, try to pack */
	/* Bitfield combined with next fields */

	u8                         node_type;            /*  2413     1 */
	u8                         phys_port_cnt;        /*  2414     1 */

	/* XXX 1 byte hole, try to pack */

	struct ib_device_attr      attrs;                /*  2416   296 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 42 boundary (2688 bytes) was 24 bytes ago --- */
	struct attribute_group *   hw_stats_ag;          /*  2712     8 */
	struct rdma_hw_stats *     hw_stats;             /*  2720     8 */
	struct rdmacg_device       cg_device;            /*  2728    40 */
	/* --- cacheline 43 boundary (2752 bytes) was 16 bytes ago --- */
	u32                        index;                /*  2768     4 */
	spinlock_t                 cq_pools_lock;        /*  2772     4 */
	struct list_head           cq_pools[3];          /*  2776    48 */
	/* --- cacheline 44 boundary (2816 bytes) was 8 bytes ago --- */
	struct rdma_restrack_root * res;                 /*  2824     8 */
	const struct uapi_definition  * driver_def;      /*  2832     8 */
	refcount_t                 refcount;             /*  2840     4 */

	/* XXX 4 bytes hole, try to pack */

	struct completion          unreg_completion;     /*  2848    32 */
	/* --- cacheline 45 boundary (2880 bytes) --- */
	struct work_struct         unregistration_work;  /*  2880    64 */
	/* --- cacheline 46 boundary (2944 bytes) --- */
	const struct rdma_link_ops  * link_ops;          /*  2944     8 */
	struct mutex               compat_devs_mutex;    /*  2952    32 */
	struct xarray              compat_devs;          /*  2984    16 */
	char                       iw_ifname[16];        /*  3000    16 */
	/* --- cacheline 47 boundary (3008 bytes) was 8 bytes ago --- */
	u32                        iw_driver_flags;      /*  3016     4 */
	u32                        lag_flags;            /*  3020     4 */

	/* size: 3024, cachelines: 48, members: 43 */
	/* sum members: 3010, holes: 4, sum holes: 13 */
	/* sum bitfield members: 3 bits, bit holes: 1, sum bit holes: 5 bits */
	/* paddings: 1, sum paddings: 4 */
	/* forced alignments: 2, forced holes: 1, sum forced holes: 4 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct ib_t10_dif_domain {
	enum ib_t10_dif_bg_type    bg_type;              /*     0     4 */
	u16                        pi_interval;          /*     4     2 */
	u16                        bg;                   /*     6     2 */
	u16                        app_tag;              /*     8     2 */

	/* XXX 2 bytes hole, try to pack */

	u32                        ref_tag;              /*    12     4 */
	bool                       ref_remap;            /*    16     1 */
	bool                       app_escape;           /*    17     1 */
	bool                       ref_escape;           /*    18     1 */

	/* XXX 1 byte hole, try to pack */

	u16                        apptag_check_mask;    /*    20     2 */

	/* size: 24, cachelines: 1, members: 9 */
	/* sum members: 19, holes: 2, sum holes: 3 */
	/* padding: 2 */
	/* last cacheline: 24 bytes */
};
struct ib_sig_domain {
	enum ib_signature_type     sig_type;             /*     0     4 */
	union {
		struct ib_t10_dif_domain dif;            /*     4    24 */
	} sig;                                           /*     4    24 */

	/* size: 28, cachelines: 1, members: 2 */
	/* last cacheline: 28 bytes */
};
struct ib_sig_attrs {
	u8                         check_mask;           /*     0     1 */

	/* XXX 3 bytes hole, try to pack */

	struct ib_sig_domain       mem;                  /*     4    28 */
	struct ib_sig_domain       wire;                 /*    32    28 */
	int                        meta_length;          /*    60     4 */

	/* size: 64, cachelines: 1, members: 4 */
	/* sum members: 61, holes: 1, sum holes: 3 */
};
struct ib_sig_err {
	enum ib_sig_err_type       err_type;             /*     0     4 */
	u32                        expected;             /*     4     4 */
	u32                        actual;               /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        sig_err_offset;       /*    16     8 */
	u32                        key;                  /*    24     4 */

	/* size: 32, cachelines: 1, members: 5 */
	/* sum members: 24, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct ib_uverbs_flow_action_esp_keymat_aes_gcm {
	__u64                      iv __attribute__((__aligned__(8))); /*     0     8 */
	__u32                      iv_algo;              /*     8     4 */
	__u32                      salt;                 /*    12     4 */
	__u32                      icv_len;              /*    16     4 */
	__u32                      key_len;              /*    20     4 */
	__u32                      aes_key[8];           /*    24    32 */

	/* size: 56, cachelines: 1, members: 6 */
	/* forced alignments: 1 */
	/* last cacheline: 56 bytes */
} __attribute__((__aligned__(8)));
struct ib_uverbs_flow_action_esp_replay_bmp {
	__u32                      size;                 /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
union ib_gid {
	u8                         raw[16];            /*     0    16 */
	struct {
		__be64             subnet_prefix;      /*     0     8 */
		__be64             interface_id;       /*     8     8 */
	} global;                                      /*     0    16 */
};
struct ib_gid_attr {
	struct net_device *        ndev;                 /*     0     8 */
	struct ib_device *         device;               /*     8     8 */
	union ib_gid               gid;                  /*    16    16 */
	enum ib_gid_type           gid_type;             /*    32     4 */
	u16                        index;                /*    36     2 */
	u8                         port_num;             /*    38     1 */

	/* size: 40, cachelines: 1, members: 6 */
	/* padding: 1 */
	/* last cacheline: 40 bytes */
};
struct ib_odp_caps {
	uint64_t                   general_caps;         /*     0     8 */
	struct {
		uint32_t           rc_odp_caps;          /*     8     4 */
		uint32_t           uc_odp_caps;          /*    12     4 */
		uint32_t           ud_odp_caps;          /*    16     4 */
		uint32_t           xrc_odp_caps;         /*    20     4 */
	} per_transport_caps;                            /*     8    16 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct ib_rss_caps {
	u32                        supported_qpts;       /*     0     4 */
	u32                        max_rwq_indirection_tables; /*     4     4 */
	u32                        max_rwq_indirection_table_size; /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct ib_tm_caps {
	u32                        max_rndv_hdr_size;    /*     0     4 */
	u32                        max_num_tags;         /*     4     4 */
	u32                        flags;                /*     8     4 */
	u32                        max_ops;              /*    12     4 */
	u32                        max_sge;              /*    16     4 */

	/* size: 20, cachelines: 1, members: 5 */
	/* last cacheline: 20 bytes */
};
struct ib_cq_init_attr {
	unsigned int               cqe;                  /*     0     4 */
	u32                        comp_vector;          /*     4     4 */
	u32                        flags;                /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct ib_cq_caps {
	u16                        max_cq_moderation_count; /*     0     2 */
	u16                        max_cq_moderation_period; /*     2     2 */

	/* size: 4, cachelines: 1, members: 2 */
	/* last cacheline: 4 bytes */
};
struct ib_dm_mr_attr {
	u64                        length;               /*     0     8 */
	u64                        offset;               /*     8     8 */
	u32                        access_flags;         /*    16     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct ib_dm_alloc_attr {
	u64                        length;               /*     0     8 */
	u32                        alignment;            /*     8     4 */
	u32                        flags;                /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct ib_device_attr {
	u64                        fw_ver;               /*     0     8 */
	__be64                     sys_image_guid;       /*     8     8 */
	u64                        max_mr_size;          /*    16     8 */
	u64                        page_size_cap;        /*    24     8 */
	u32                        vendor_id;            /*    32     4 */
	u32                        vendor_part_id;       /*    36     4 */
	u32                        hw_ver;               /*    40     4 */
	int                        max_qp;               /*    44     4 */
	int                        max_qp_wr;            /*    48     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        device_cap_flags;     /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        max_send_sge;         /*    64     4 */
	int                        max_recv_sge;         /*    68     4 */
	int                        max_sge_rd;           /*    72     4 */
	int                        max_cq;               /*    76     4 */
	int                        max_cqe;              /*    80     4 */
	int                        max_mr;               /*    84     4 */
	int                        max_pd;               /*    88     4 */
	int                        max_qp_rd_atom;       /*    92     4 */
	int                        max_ee_rd_atom;       /*    96     4 */
	int                        max_res_rd_atom;      /*   100     4 */
	int                        max_qp_init_rd_atom;  /*   104     4 */
	int                        max_ee_init_rd_atom;  /*   108     4 */
	enum ib_atomic_cap         atomic_cap;           /*   112     4 */
	enum ib_atomic_cap         masked_atomic_cap;    /*   116     4 */
	int                        max_ee;               /*   120     4 */
	int                        max_rdd;              /*   124     4 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	int                        max_mw;               /*   128     4 */
	int                        max_raw_ipv6_qp;      /*   132     4 */
	int                        max_raw_ethy_qp;      /*   136     4 */
	int                        max_mcast_grp;        /*   140     4 */
	int                        max_mcast_qp_attach;  /*   144     4 */
	int                        max_total_mcast_qp_attach; /*   148     4 */
	int                        max_ah;               /*   152     4 */
	int                        max_srq;              /*   156     4 */
	int                        max_srq_wr;           /*   160     4 */
	int                        max_srq_sge;          /*   164     4 */
	unsigned int               max_fast_reg_page_list_len; /*   168     4 */
	unsigned int               max_pi_fast_reg_page_list_len; /*   172     4 */
	u16                        max_pkeys;            /*   176     2 */
	u8                         local_ca_ack_delay;   /*   178     1 */

	/* XXX 1 byte hole, try to pack */

	int                        sig_prot_cap;         /*   180     4 */
	int                        sig_guard_cap;        /*   184     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 3 boundary (192 bytes) --- */
	struct ib_odp_caps         odp_caps;             /*   192    24 */
	uint64_t                   timestamp_mask;       /*   216     8 */
	uint64_t                   hca_core_clock;       /*   224     8 */
	struct ib_rss_caps         rss_caps;             /*   232    12 */
	u32                        max_wq_type_rq;       /*   244     4 */
	u32                        raw_packet_caps;      /*   248     4 */
	struct ib_tm_caps          tm_caps;              /*   252    20 */
	/* --- cacheline 4 boundary (256 bytes) was 16 bytes ago --- */
	struct ib_cq_caps          cq_caps;              /*   272     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        max_dm_size;          /*   280     8 */
	u32                        max_sgl_rd;           /*   288     4 */

	/* size: 296, cachelines: 5, members: 52 */
	/* sum members: 279, holes: 4, sum holes: 13 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct ib_port_attr {
	u64                        subnet_prefix;        /*     0     8 */
	enum ib_port_state         state;                /*     8     4 */
	enum ib_mtu                max_mtu;              /*    12     4 */
	enum ib_mtu                active_mtu;           /*    16     4 */
	u32                        phys_mtu;             /*    20     4 */
	int                        gid_tbl_len;          /*    24     4 */
	unsigned int               ip_gids:1;            /*    28: 0  4 */

	/* XXX 31 bits hole, try to pack */

	u32                        port_cap_flags;       /*    32     4 */
	u32                        max_msg_sz;           /*    36     4 */
	u32                        bad_pkey_cntr;        /*    40     4 */
	u32                        qkey_viol_cntr;       /*    44     4 */
	u16                        pkey_tbl_len;         /*    48     2 */

	/* XXX 2 bytes hole, try to pack */

	u32                        sm_lid;               /*    52     4 */
	u32                        lid;                  /*    56     4 */
	u8                         lmc;                  /*    60     1 */
	u8                         max_vl_num;           /*    61     1 */
	u8                         sm_sl;                /*    62     1 */
	u8                         subnet_timeout;       /*    63     1 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u8                         init_type_reply;      /*    64     1 */
	u8                         active_width;         /*    65     1 */
	u16                        active_speed;         /*    66     2 */
	u8                         phys_state;           /*    68     1 */

	/* XXX 1 byte hole, try to pack */

	u16                        port_cap_flags2;      /*    70     2 */

	/* size: 72, cachelines: 2, members: 23 */
	/* sum members: 65, holes: 2, sum holes: 3 */
	/* sum bitfield members: 1 bits, bit holes: 1, sum bit holes: 31 bits */
	/* last cacheline: 8 bytes */
};
struct ib_device_modify {
	u64                        sys_image_guid;       /*     0     8 */
	char                       node_desc[64];        /*     8    64 */

	/* size: 72, cachelines: 2, members: 2 */
	/* last cacheline: 8 bytes */
};
struct ib_port_modify {
	u32                        set_port_cap_mask;    /*     0     4 */
	u32                        clr_port_cap_mask;    /*     4     4 */
	u8                         init_type;            /*     8     1 */

	/* size: 12, cachelines: 1, members: 3 */
	/* padding: 3 */
	/* last cacheline: 12 bytes */
};
struct ib_cq {
	struct ib_device *         device;               /*     0     8 */
	struct ib_ucq_object *     uobject;              /*     8     8 */
	ib_comp_handler            comp_handler;         /*    16     8 */
	void                       (*event_handler)(struct ib_event *, void *); /*    24     8 */
	void *                     cq_context;           /*    32     8 */
	int                        cqe;                  /*    40     4 */
	unsigned int               cqe_used;             /*    44     4 */
	atomic_t                   usecnt;               /*    48     4 */
	enum ib_poll_context       poll_ctx;             /*    52     4 */
	struct ib_wc *             wc;                   /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct list_head           pool_entry;           /*    64    16 */
	union {
		struct irq_poll    iop;                  /*    80    40 */
		struct work_struct work;                 /*    80    64 */
	};                                               /*    80    64 */
	/* --- cacheline 2 boundary (128 bytes) was 16 bytes ago --- */
	struct workqueue_struct *  comp_wq;              /*   144     8 */
	struct dim *               dim;                  /*   152     8 */
	ktime_t                    timestamp;            /*   160     8 */
	u8                         interrupt:1;          /*   168: 0  1 */
	u8                         shared:1;             /*   168: 1  1 */

	/* XXX 6 bits hole, try to pack */
	/* XXX 3 bytes hole, try to pack */

	unsigned int               comp_vector;          /*   172     4 */
	struct rdma_restrack_entry res;                  /*   176    72 */

	/* XXX last struct has 4 bytes of padding */

	/* size: 248, cachelines: 4, members: 19 */
	/* sum members: 244, holes: 1, sum holes: 3 */
	/* sum bitfield members: 2 bits, bit holes: 1, sum bit holes: 6 bits */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 56 bytes */
};
struct ib_qp {
	struct ib_device *         device;               /*     0     8 */
	struct ib_pd *             pd;                   /*     8     8 */
	struct ib_cq *             send_cq;              /*    16     8 */
	struct ib_cq *             recv_cq;              /*    24     8 */
	spinlock_t                 mr_lock;              /*    32     4 */
	int                        mrs_used;             /*    36     4 */
	struct list_head           rdma_mrs;             /*    40    16 */
	struct list_head           sig_mrs;              /*    56    16 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	struct ib_srq *            srq;                  /*    72     8 */
	struct ib_xrcd *           xrcd;                 /*    80     8 */
	struct list_head           xrcd_list;            /*    88    16 */
	atomic_t                   usecnt;               /*   104     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           open_list;            /*   112    16 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct ib_qp *             real_qp;              /*   128     8 */
	struct ib_uqp_object *     uobject;              /*   136     8 */
	void                       (*event_handler)(struct ib_event *, void *); /*   144     8 */
	void *                     qp_context;           /*   152     8 */
	const struct ib_gid_attr  * av_sgid_attr;        /*   160     8 */
	const struct ib_gid_attr  * alt_path_sgid_attr;  /*   168     8 */
	u32                        qp_num;               /*   176     4 */
	u32                        max_write_sge;        /*   180     4 */
	u32                        max_read_sge;         /*   184     4 */
	enum ib_qp_type            qp_type;              /*   188     4 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	struct ib_rwq_ind_table *  rwq_ind_tbl;          /*   192     8 */
	struct ib_qp_security *    qp_sec;               /*   200     8 */
	u8                         port;                 /*   208     1 */
	bool                       integrity_en;         /*   209     1 */

	/* XXX 6 bytes hole, try to pack */

	struct rdma_restrack_entry res;                  /*   216    72 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 4 boundary (256 bytes) was 32 bytes ago --- */
	struct rdma_counter *      counter;              /*   288     8 */

	/* size: 296, cachelines: 5, members: 29 */
	/* sum members: 286, holes: 2, sum holes: 10 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 40 bytes */
};
struct ib_srq {
	struct ib_device *         device;               /*     0     8 */
	struct ib_pd *             pd;                   /*     8     8 */
	struct ib_usrq_object *    uobject;              /*    16     8 */
	void                       (*event_handler)(struct ib_event *, void *); /*    24     8 */
	void *                     srq_context;          /*    32     8 */
	enum ib_srq_type           srq_type;             /*    40     4 */
	atomic_t                   usecnt;               /*    44     4 */
	struct {
		struct ib_cq *     cq;                   /*    48     8 */
		union {
			struct {
				struct ib_xrcd * xrcd;   /*    56     8 */
				/* --- cacheline 1 boundary (64 bytes) --- */
				u32 srq_num;             /*    64     4 */
			} xrc;                           /*    56    16 */
		};                                       /*    56    16 */
	} ext;                                           /*    48    24 */

	/* size: 72, cachelines: 2, members: 8 */
	/* last cacheline: 8 bytes */
};
struct ib_wq {
	struct ib_device *         device;               /*     0     8 */
	struct ib_uwq_object *     uobject;              /*     8     8 */
	void *                     wq_context;           /*    16     8 */
	void                       (*event_handler)(struct ib_event *, void *); /*    24     8 */
	struct ib_pd *             pd;                   /*    32     8 */
	struct ib_cq *             cq;                   /*    40     8 */
	u32                        wq_num;               /*    48     4 */
	enum ib_wq_state           state;                /*    52     4 */
	enum ib_wq_type            wq_type;              /*    56     4 */
	atomic_t                   usecnt;               /*    60     4 */

	/* size: 64, cachelines: 1, members: 10 */
};
struct ib_event {
	struct ib_device *         device;               /*     0     8 */
	union {
		struct ib_cq *     cq;                   /*     8     8 */
		struct ib_qp *     qp;                   /*     8     8 */
		struct ib_srq *    srq;                  /*     8     8 */
		struct ib_wq *     wq;                   /*     8     8 */
		u8                 port_num;             /*     8     1 */
	} element;                                       /*     8     8 */
	enum ib_event_type         event;                /*    16     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct ib_global_route {
	const struct ib_gid_attr  * sgid_attr;           /*     0     8 */
	union ib_gid               dgid;                 /*     8    16 */
	u32                        flow_label;           /*    24     4 */
	u8                         sgid_index;           /*    28     1 */
	u8                         hop_limit;            /*    29     1 */
	u8                         traffic_class;        /*    30     1 */

	/* size: 32, cachelines: 1, members: 6 */
	/* padding: 1 */
	/* last cacheline: 32 bytes */
};
struct ib_grh {
	__be32                     version_tclass_flow;  /*     0     4 */
	__be16                     paylen;               /*     4     2 */
	u8                         next_hdr;             /*     6     1 */
	u8                         hop_limit;            /*     7     1 */
	union ib_gid               sgid;                 /*     8    16 */
	union ib_gid               dgid;                 /*    24    16 */

	/* size: 40, cachelines: 1, members: 6 */
	/* last cacheline: 40 bytes */
};
struct ib_mr_status {
	u32                        fail_status;          /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct ib_sig_err          sig_err;              /*     8    32 */

	/* XXX last struct has 4 bytes of padding */

	/* size: 40, cachelines: 1, members: 2 */
	/* sum members: 36, holes: 1, sum holes: 4 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 40 bytes */
};
struct rdma_ah_init_attr {
	struct rdma_ah_attr *      ah_attr;              /*     0     8 */
	u32                        flags;                /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct net_device *        xmit_slave;           /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct rdma_ah_attr {
	struct ib_global_route     grh;                  /*     0    32 */

	/* XXX last struct has 1 byte of padding */

	u8                         sl;                   /*    32     1 */
	u8                         static_rate;          /*    33     1 */
	u8                         port_num;             /*    34     1 */
	u8                         ah_flags;             /*    35     1 */
	enum rdma_ah_attr_type     type;                 /*    36     4 */
	union {
		struct ib_ah_attr  ib;                   /*    40     4 */
		struct roce_ah_attr roce;                /*    40     6 */
		struct opa_ah_attr opa;                  /*    40     8 */
	};                                               /*    40     8 */

	/* size: 48, cachelines: 1, members: 7 */
	/* paddings: 1, sum paddings: 1 */
	/* last cacheline: 48 bytes */
};
struct ib_ah_attr {
	u16                        dlid;                 /*     0     2 */
	u8                         src_path_bits;        /*     2     1 */

	/* size: 4, cachelines: 1, members: 2 */
	/* padding: 1 */
	/* last cacheline: 4 bytes */
};
struct roce_ah_attr {
	u8                         dmac[6];              /*     0     6 */

	/* size: 6, cachelines: 1, members: 1 */
	/* last cacheline: 6 bytes */
};
struct opa_ah_attr {
	u32                        dlid;                 /*     0     4 */
	u8                         src_path_bits;        /*     4     1 */
	bool                       make_grd;             /*     5     1 */

	/* size: 8, cachelines: 1, members: 3 */
	/* padding: 2 */
	/* last cacheline: 8 bytes */
};
struct ib_cqe {
	void                       (*done)(struct ib_cq *, struct ib_wc *); /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct ib_wc {
	union {
		u64                wr_id;                /*     0     8 */
		struct ib_cqe *    wr_cqe;               /*     0     8 */
	};                                               /*     0     8 */
	enum ib_wc_status          status;               /*     8     4 */
	enum ib_wc_opcode          opcode;               /*    12     4 */
	u32                        vendor_err;           /*    16     4 */
	u32                        byte_len;             /*    20     4 */
	struct ib_qp *             qp;                   /*    24     8 */
	union {
		__be32             imm_data;             /*    32     4 */
		u32                invalidate_rkey;      /*    32     4 */
	} ex;                                            /*    32     4 */
	u32                        src_qp;               /*    36     4 */
	u32                        slid;                 /*    40     4 */
	int                        wc_flags;             /*    44     4 */
	u16                        pkey_index;           /*    48     2 */
	u8                         sl;                   /*    50     1 */
	u8                         dlid_path_bits;       /*    51     1 */
	u8                         port_num;             /*    52     1 */
	u8                         smac[6];              /*    53     6 */

	/* XXX 1 byte hole, try to pack */

	u16                        vlan_id;              /*    60     2 */
	u8                         network_hdr_type;     /*    62     1 */

	/* size: 64, cachelines: 1, members: 17 */
	/* sum members: 62, holes: 1, sum holes: 1 */
	/* padding: 1 */
};
struct ib_srq_attr {
	u32                        max_wr;               /*     0     4 */
	u32                        max_sge;              /*     4     4 */
	u32                        srq_limit;            /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct ib_xrcd {
	struct ib_device *         device;               /*     0     8 */
	atomic_t                   usecnt;               /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct inode *             inode;                /*    16     8 */
	struct rw_semaphore        tgt_qps_rwsem;        /*    24    40 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct xarray              tgt_qps;              /*    64    16 */

	/* size: 80, cachelines: 2, members: 5 */
	/* sum members: 76, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct ib_srq_init_attr {
	void                       (*event_handler)(struct ib_event *, void *); /*     0     8 */
	void *                     srq_context;          /*     8     8 */
	struct ib_srq_attr         attr;                 /*    16    12 */
	enum ib_srq_type           srq_type;             /*    28     4 */
	struct {
		struct ib_cq *     cq;                   /*    32     8 */
		union {
			struct {
				struct ib_xrcd * xrcd;   /*    40     8 */
			} xrc;                           /*    40     8 */
			struct {
				u32 max_num_tags;        /*    40     4 */
			} tag_matching;                  /*    40     4 */
		};                                       /*    40     8 */
	} ext;                                           /*    32    16 */

	/* size: 48, cachelines: 1, members: 5 */
	/* last cacheline: 48 bytes */
};
struct ib_qp_cap {
	u32                        max_send_wr;          /*     0     4 */
	u32                        max_recv_wr;          /*     4     4 */
	u32                        max_send_sge;         /*     8     4 */
	u32                        max_recv_sge;         /*    12     4 */
	u32                        max_inline_data;      /*    16     4 */
	u32                        max_rdma_ctxs;        /*    20     4 */

	/* size: 24, cachelines: 1, members: 6 */
	/* last cacheline: 24 bytes */
};
struct ib_qp_init_attr {
	void                       (*event_handler)(struct ib_event *, void *); /*     0     8 */
	void *                     qp_context;           /*     8     8 */
	struct ib_cq *             send_cq;              /*    16     8 */
	struct ib_cq *             recv_cq;              /*    24     8 */
	struct ib_srq *            srq;                  /*    32     8 */
	struct ib_xrcd *           xrcd;                 /*    40     8 */
	struct ib_qp_cap           cap;                  /*    48    24 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	enum ib_sig_type           sq_sig_type;          /*    72     4 */
	enum ib_qp_type            qp_type;              /*    76     4 */
	u32                        create_flags;         /*    80     4 */
	u8                         port_num;             /*    84     1 */

	/* XXX 3 bytes hole, try to pack */

	struct ib_rwq_ind_table *  rwq_ind_tbl;          /*    88     8 */
	u32                        source_qpn;           /*    96     4 */

	/* size: 104, cachelines: 2, members: 13 */
	/* sum members: 97, holes: 1, sum holes: 3 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct ib_rwq_ind_table {
	struct ib_device *         device;               /*     0     8 */
	struct ib_uobject *        uobject;              /*     8     8 */
	atomic_t                   usecnt;               /*    16     4 */
	u32                        ind_tbl_num;          /*    20     4 */
	u32                        log_ind_tbl_size;     /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	struct ib_wq * *           ind_tbl;              /*    32     8 */

	/* size: 40, cachelines: 1, members: 6 */
	/* sum members: 36, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct ib_qp_attr {
	enum ib_qp_state           qp_state;             /*     0     4 */
	enum ib_qp_state           cur_qp_state;         /*     4     4 */
	enum ib_mtu                path_mtu;             /*     8     4 */
	enum ib_mig_state          path_mig_state;       /*    12     4 */
	u32                        qkey;                 /*    16     4 */
	u32                        rq_psn;               /*    20     4 */
	u32                        sq_psn;               /*    24     4 */
	u32                        dest_qp_num;          /*    28     4 */
	int                        qp_access_flags;      /*    32     4 */
	struct ib_qp_cap           cap;                  /*    36    24 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	struct rdma_ah_attr        ah_attr;              /*    64    48 */
	struct rdma_ah_attr        alt_ah_attr;          /*   112    48 */
	/* --- cacheline 2 boundary (128 bytes) was 32 bytes ago --- */
	u16                        pkey_index;           /*   160     2 */
	u16                        alt_pkey_index;       /*   162     2 */
	u8                         en_sqd_async_notify;  /*   164     1 */
	u8                         sq_draining;          /*   165     1 */
	u8                         max_rd_atomic;        /*   166     1 */
	u8                         max_dest_rd_atomic;   /*   167     1 */
	u8                         min_rnr_timer;        /*   168     1 */
	u8                         port_num;             /*   169     1 */
	u8                         timeout;              /*   170     1 */
	u8                         retry_cnt;            /*   171     1 */
	u8                         rnr_retry;            /*   172     1 */
	u8                         alt_port_num;         /*   173     1 */
	u8                         alt_timeout;          /*   174     1 */

	/* XXX 1 byte hole, try to pack */

	u32                        rate_limit;           /*   176     4 */

	/* XXX 4 bytes hole, try to pack */

	struct net_device *        xmit_slave;           /*   184     8 */

	/* size: 192, cachelines: 3, members: 27 */
	/* sum members: 183, holes: 3, sum holes: 9 */
};
struct ib_sge {
	u64                        addr;                 /*     0     8 */
	u32                        length;               /*     8     4 */
	u32                        lkey;                 /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct ib_send_wr {
	struct ib_send_wr *        next;                 /*     0     8 */
	union {
		u64                wr_id;                /*     8     8 */
		struct ib_cqe *    wr_cqe;               /*     8     8 */
	};                                               /*     8     8 */
	struct ib_sge *            sg_list;              /*    16     8 */
	int                        num_sge;              /*    24     4 */
	enum ib_wr_opcode          opcode;               /*    28     4 */
	int                        send_flags;           /*    32     4 */
	union {
		__be32             imm_data;             /*    36     4 */
		u32                invalidate_rkey;      /*    36     4 */
	} ex;                                            /*    36     4 */

	/* size: 40, cachelines: 1, members: 7 */
	/* last cacheline: 40 bytes */
};
struct ib_ah {
	struct ib_device *         device;               /*     0     8 */
	struct ib_pd *             pd;                   /*     8     8 */
	struct ib_uobject *        uobject;              /*    16     8 */
	const struct ib_gid_attr  * sgid_attr;           /*    24     8 */
	enum rdma_ah_attr_type     type;                 /*    32     4 */

	/* size: 40, cachelines: 1, members: 5 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct ib_mr {
	struct ib_device *         device;               /*     0     8 */
	struct ib_pd *             pd;                   /*     8     8 */
	u32                        lkey;                 /*    16     4 */
	u32                        rkey;                 /*    20     4 */
	u64                        iova;                 /*    24     8 */
	u64                        length;               /*    32     8 */
	unsigned int               page_size;            /*    40     4 */
	enum ib_mr_type            type;                 /*    44     4 */
	bool                       need_inval;           /*    48     1 */

	/* XXX 7 bytes hole, try to pack */

	union {
		struct ib_uobject * uobject;             /*    56     8 */
		struct list_head   qp_entry;             /*    56    16 */
	};                                               /*    56    16 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	struct ib_dm *             dm;                   /*    72     8 */
	struct ib_sig_attrs *      sig_attrs;            /*    80     8 */
	struct rdma_restrack_entry res;                  /*    88    72 */

	/* XXX last struct has 4 bytes of padding */

	/* size: 160, cachelines: 3, members: 13 */
	/* sum members: 153, holes: 1, sum holes: 7 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 32 bytes */
};
struct ib_recv_wr {
	struct ib_recv_wr *        next;                 /*     0     8 */
	union {
		u64                wr_id;                /*     8     8 */
		struct ib_cqe *    wr_cqe;               /*     8     8 */
	};                                               /*     8     8 */
	struct ib_sge *            sg_list;              /*    16     8 */
	int                        num_sge;              /*    24     4 */

	/* size: 32, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct ib_rdmacg_object {
	struct rdma_cgroup *       cg;                   /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct ib_ucontext {
	struct ib_device *         device;               /*     0     8 */
	struct ib_uverbs_file *    ufile;                /*     8     8 */
	bool                       cleanup_retryable;    /*    16     1 */

	/* XXX 7 bytes hole, try to pack */

	struct ib_rdmacg_object    cg_obj;               /*    24     8 */
	struct rdma_restrack_entry res;                  /*    32    72 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 1 boundary (64 bytes) was 40 bytes ago --- */
	struct xarray              mmap_xa;              /*   104    16 */

	/* size: 120, cachelines: 2, members: 6 */
	/* sum members: 113, holes: 1, sum holes: 7 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 56 bytes */
};
struct ib_uobject {
	u64                        user_handle;          /*     0     8 */
	struct ib_uverbs_file *    ufile;                /*     8     8 */
	struct ib_ucontext *       context;              /*    16     8 */
	void *                     object;               /*    24     8 */
	struct list_head           list;                 /*    32    16 */
	struct ib_rdmacg_object    cg_obj;               /*    48     8 */
	int                        id;                   /*    56     4 */
	struct kref                ref;                  /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	atomic_t                   usecnt;               /*    64     4 */

	/* XXX 4 bytes hole, try to pack */

	struct callback_head       rcu __attribute__((__aligned__(8))); /*    72    16 */
	const struct uverbs_api_object  * uapi_object;   /*    88     8 */

	/* size: 96, cachelines: 2, members: 11 */
	/* sum members: 92, holes: 1, sum holes: 4 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 4 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct ib_udata {
	const void  *              inbuf;                /*     0     8 */
	void *                     outbuf;               /*     8     8 */
	size_t                     inlen;                /*    16     8 */
	size_t                     outlen;               /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct ib_pd {
	u32                        local_dma_lkey;       /*     0     4 */
	u32                        flags;                /*     4     4 */
	struct ib_device *         device;               /*     8     8 */
	struct ib_uobject *        uobject;              /*    16     8 */
	atomic_t                   usecnt;               /*    24     4 */
	u32                        unsafe_global_rkey;   /*    28     4 */
	struct ib_mr *             __internal_mr;        /*    32     8 */
	struct rdma_restrack_entry res;                  /*    40    72 */

	/* XXX last struct has 4 bytes of padding */

	/* size: 112, cachelines: 2, members: 8 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 48 bytes */
};
struct ib_wq_init_attr {
	void *                     wq_context;           /*     0     8 */
	enum ib_wq_type            wq_type;              /*     8     4 */
	u32                        max_wr;               /*    12     4 */
	u32                        max_sge;              /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	struct ib_cq *             cq;                   /*    24     8 */
	void                       (*event_handler)(struct ib_event *, void *); /*    32     8 */
	u32                        create_flags;         /*    40     4 */

	/* size: 48, cachelines: 1, members: 7 */
	/* sum members: 40, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 48 bytes */
};
struct ib_wq_attr {
	enum ib_wq_state           wq_state;             /*     0     4 */
	enum ib_wq_state           curr_wq_state;        /*     4     4 */
	u32                        flags;                /*     8     4 */
	u32                        flags_mask;           /*    12     4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct ib_rwq_ind_table_init_attr {
	u32                        log_ind_tbl_size;     /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct ib_wq * *           ind_tbl;              /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct ib_port_pkey {
	enum port_pkey_state       state;                /*     0     4 */
	u16                        pkey_index;           /*     4     2 */
	u8                         port_num;             /*     6     1 */

	/* XXX 1 byte hole, try to pack */

	struct list_head           qp_list;              /*     8    16 */
	struct list_head           to_error_list;        /*    24    16 */
	struct ib_qp_security *    sec;                  /*    40     8 */

	/* size: 48, cachelines: 1, members: 6 */
	/* sum members: 47, holes: 1, sum holes: 1 */
	/* last cacheline: 48 bytes */
};
struct ib_qp_security {
	struct ib_qp *             qp;                   /*     0     8 */
	struct ib_device *         dev;                  /*     8     8 */
	struct mutex               mutex;                /*    16    32 */
	struct ib_ports_pkeys *    ports_pkeys;          /*    48     8 */
	struct list_head           shared_qp_list;       /*    56    16 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	void *                     security;             /*    72     8 */
	bool                       destroying;           /*    80     1 */

	/* XXX 3 bytes hole, try to pack */

	atomic_t                   error_list_count;     /*    84     4 */
	struct completion          error_complete;       /*    88    32 */
	int                        error_comps_pending;  /*   120     4 */

	/* size: 128, cachelines: 2, members: 10 */
	/* sum members: 121, holes: 1, sum holes: 3 */
	/* padding: 4 */
};
struct ib_ports_pkeys {
	struct ib_port_pkey        main;                 /*     0    48 */
	struct ib_port_pkey        alt;                  /*    48    48 */

	/* size: 96, cachelines: 2, members: 2 */
	/* last cacheline: 32 bytes */
};
struct ib_dm {
	struct ib_device *         device;               /*     0     8 */
	u32                        length;               /*     8     4 */
	u32                        flags;                /*    12     4 */
	struct ib_uobject *        uobject;              /*    16     8 */
	atomic_t                   usecnt;               /*    24     4 */

	/* size: 32, cachelines: 1, members: 5 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct ib_mw {
	struct ib_device *         device;               /*     0     8 */
	struct ib_pd *             pd;                   /*     8     8 */
	struct ib_uobject *        uobject;              /*    16     8 */
	u32                        rkey;                 /*    24     4 */
	enum ib_mw_type            type;                 /*    28     4 */

	/* size: 32, cachelines: 1, members: 5 */
	/* last cacheline: 32 bytes */
};
struct ib_flow_eth_filter {
	u8                         dst_mac[6];           /*     0     6 */
	u8                         src_mac[6];           /*     6     6 */
	__be16                     ether_type;           /*    12     2 */
	__be16                     vlan_tag;             /*    14     2 */
	u8                         real_sz[];            /*    16     0 */

	/* size: 16, cachelines: 1, members: 5 */
	/* last cacheline: 16 bytes */
};
struct ib_flow_spec_eth {
	u32                        type;                 /*     0     4 */
	u16                        size;                 /*     4     2 */
	struct ib_flow_eth_filter  val;                  /*     6    16 */
	struct ib_flow_eth_filter  mask;                 /*    22    16 */

	/* size: 40, cachelines: 1, members: 4 */
	/* padding: 2 */
	/* last cacheline: 40 bytes */
};
struct ib_flow_ib_filter {
	__be16                     dlid;                 /*     0     2 */
	__u8                       sl;                   /*     2     1 */
	u8                         real_sz[];            /*     3     0 */

	/* size: 4, cachelines: 1, members: 3 */
	/* padding: 1 */
	/* last cacheline: 4 bytes */
};
struct ib_flow_spec_ib {
	u32                        type;                 /*     0     4 */
	u16                        size;                 /*     4     2 */
	struct ib_flow_ib_filter   val;                  /*     6     4 */

	/* XXX last struct has 1 byte of padding */

	struct ib_flow_ib_filter   mask;                 /*    10     4 */

	/* XXX last struct has 1 byte of padding */

	/* size: 16, cachelines: 1, members: 4 */
	/* padding: 2 */
	/* paddings: 2, sum paddings: 2 */
	/* last cacheline: 16 bytes */
};
struct ib_flow_ipv4_filter {
	__be32                     src_ip;               /*     0     4 */
	__be32                     dst_ip;               /*     4     4 */
	u8                         proto;                /*     8     1 */
	u8                         tos;                  /*     9     1 */
	u8                         ttl;                  /*    10     1 */
	u8                         flags;                /*    11     1 */
	u8                         real_sz[];            /*    12     0 */

	/* size: 12, cachelines: 1, members: 7 */
	/* last cacheline: 12 bytes */
};
struct ib_flow_spec_ipv4 {
	u32                        type;                 /*     0     4 */
	u16                        size;                 /*     4     2 */

	/* XXX 2 bytes hole, try to pack */

	struct ib_flow_ipv4_filter val;                  /*     8    12 */
	struct ib_flow_ipv4_filter mask;                 /*    20    12 */

	/* size: 32, cachelines: 1, members: 4 */
	/* sum members: 30, holes: 1, sum holes: 2 */
	/* last cacheline: 32 bytes */
};
struct ib_flow_ipv6_filter {
	u8                         src_ip[16];           /*     0    16 */
	u8                         dst_ip[16];           /*    16    16 */
	__be32                     flow_label;           /*    32     4 */
	u8                         next_hdr;             /*    36     1 */
	u8                         traffic_class;        /*    37     1 */
	u8                         hop_limit;            /*    38     1 */
	u8                         real_sz[];            /*    39     0 */

	/* size: 40, cachelines: 1, members: 7 */
	/* padding: 1 */
	/* last cacheline: 40 bytes */
};
struct ib_flow_spec_ipv6 {
	u32                        type;                 /*     0     4 */
	u16                        size;                 /*     4     2 */

	/* XXX 2 bytes hole, try to pack */

	struct ib_flow_ipv6_filter val;                  /*     8    40 */

	/* XXX last struct has 1 byte of padding */

	struct ib_flow_ipv6_filter mask;                 /*    48    40 */

	/* XXX last struct has 1 byte of padding */

	/* size: 88, cachelines: 2, members: 4 */
	/* sum members: 86, holes: 1, sum holes: 2 */
	/* paddings: 2, sum paddings: 2 */
	/* last cacheline: 24 bytes */
};
struct ib_flow_tcp_udp_filter {
	__be16                     dst_port;             /*     0     2 */
	__be16                     src_port;             /*     2     2 */
	u8                         real_sz[];            /*     4     0 */

	/* size: 4, cachelines: 1, members: 3 */
	/* last cacheline: 4 bytes */
};
struct ib_flow_spec_tcp_udp {
	u32                        type;                 /*     0     4 */
	u16                        size;                 /*     4     2 */
	struct ib_flow_tcp_udp_filter val;               /*     6     4 */
	struct ib_flow_tcp_udp_filter mask;              /*    10     4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* padding: 2 */
	/* last cacheline: 16 bytes */
};
struct ib_flow_tunnel_filter {
	__be32                     tunnel_id;            /*     0     4 */
	u8                         real_sz[];            /*     4     0 */

	/* size: 4, cachelines: 1, members: 2 */
	/* last cacheline: 4 bytes */
};
struct ib_flow_spec_tunnel {
	u32                        type;                 /*     0     4 */
	u16                        size;                 /*     4     2 */

	/* XXX 2 bytes hole, try to pack */

	struct ib_flow_tunnel_filter val;                /*     8     4 */
	struct ib_flow_tunnel_filter mask;               /*    12     4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* sum members: 14, holes: 1, sum holes: 2 */
	/* last cacheline: 16 bytes */
};
struct ib_flow_esp_filter {
	__be32                     spi;                  /*     0     4 */
	__be32                     seq;                  /*     4     4 */
	u8                         real_sz[];            /*     8     0 */

	/* size: 8, cachelines: 1, members: 3 */
	/* last cacheline: 8 bytes */
};
struct ib_flow_spec_esp {
	u32                        type;                 /*     0     4 */
	u16                        size;                 /*     4     2 */

	/* XXX 2 bytes hole, try to pack */

	struct ib_flow_esp_filter  val;                  /*     8     8 */
	struct ib_flow_esp_filter  mask;                 /*    16     8 */

	/* size: 24, cachelines: 1, members: 4 */
	/* sum members: 22, holes: 1, sum holes: 2 */
	/* last cacheline: 24 bytes */
};
struct ib_flow_gre_filter {
	__be16                     c_ks_res0_ver;        /*     0     2 */
	__be16                     protocol;             /*     2     2 */
	__be32                     key;                  /*     4     4 */
	u8                         real_sz[];            /*     8     0 */

	/* size: 8, cachelines: 1, members: 4 */
	/* last cacheline: 8 bytes */
};
struct ib_flow_spec_gre {
	u32                        type;                 /*     0     4 */
	u16                        size;                 /*     4     2 */

	/* XXX 2 bytes hole, try to pack */

	struct ib_flow_gre_filter  val;                  /*     8     8 */
	struct ib_flow_gre_filter  mask;                 /*    16     8 */

	/* size: 24, cachelines: 1, members: 4 */
	/* sum members: 22, holes: 1, sum holes: 2 */
	/* last cacheline: 24 bytes */
};
struct ib_flow_mpls_filter {
	__be32                     tag;                  /*     0     4 */
	u8                         real_sz[];            /*     4     0 */

	/* size: 4, cachelines: 1, members: 2 */
	/* last cacheline: 4 bytes */
};
struct ib_flow_spec_mpls {
	u32                        type;                 /*     0     4 */
	u16                        size;                 /*     4     2 */

	/* XXX 2 bytes hole, try to pack */

	struct ib_flow_mpls_filter val;                  /*     8     4 */
	struct ib_flow_mpls_filter mask;                 /*    12     4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* sum members: 14, holes: 1, sum holes: 2 */
	/* last cacheline: 16 bytes */
};
struct ib_flow_spec_action_tag {
	enum ib_flow_spec_type     type;                 /*     0     4 */
	u16                        size;                 /*     4     2 */

	/* XXX 2 bytes hole, try to pack */

	u32                        tag_id;               /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* sum members: 10, holes: 1, sum holes: 2 */
	/* last cacheline: 12 bytes */
};
struct ib_flow_spec_action_drop {
	enum ib_flow_spec_type     type;                 /*     0     4 */
	u16                        size;                 /*     4     2 */

	/* size: 8, cachelines: 1, members: 2 */
	/* padding: 2 */
	/* last cacheline: 8 bytes */
};
struct ib_flow_spec_action_handle {
	enum ib_flow_spec_type     type;                 /*     0     4 */
	u16                        size;                 /*     4     2 */

	/* XXX 2 bytes hole, try to pack */

	struct ib_flow_action *    act;                  /*     8     8 */

	/* size: 16, cachelines: 1, members: 3 */
	/* sum members: 14, holes: 1, sum holes: 2 */
	/* last cacheline: 16 bytes */
};
struct ib_flow_action {
	struct ib_device *         device;               /*     0     8 */
	struct ib_uobject *        uobject;              /*     8     8 */
	enum ib_flow_action_type   type;                 /*    16     4 */
	atomic_t                   usecnt;               /*    20     4 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct ib_flow_spec_action_count {
	enum ib_flow_spec_type     type;                 /*     0     4 */
	u16                        size;                 /*     4     2 */

	/* XXX 2 bytes hole, try to pack */

	struct ib_counters *       counters;             /*     8     8 */

	/* size: 16, cachelines: 1, members: 3 */
	/* sum members: 14, holes: 1, sum holes: 2 */
	/* last cacheline: 16 bytes */
};
struct ib_counters {
	struct ib_device *         device;               /*     0     8 */
	struct ib_uobject *        uobject;              /*     8     8 */
	atomic_t                   usecnt;               /*    16     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
union ib_flow_spec {
	struct {
		u32                type;               /*     0     4 */
		u16                size;               /*     4     2 */
	};                                             /*     0     8 */
	struct ib_flow_spec_eth    eth;                /*     0    40 */
	struct ib_flow_spec_ib     ib;                 /*     0    16 */
	struct ib_flow_spec_ipv4   ipv4;               /*     0    32 */
	struct ib_flow_spec_tcp_udp tcp_udp;           /*     0    16 */
	struct ib_flow_spec_ipv6   ipv6;               /*     0    88 */
	struct ib_flow_spec_tunnel tunnel;             /*     0    16 */
	struct ib_flow_spec_esp    esp;                /*     0    24 */
	struct ib_flow_spec_gre    gre;                /*     0    24 */
	struct ib_flow_spec_mpls   mpls;               /*     0    16 */
	struct ib_flow_spec_action_tag flow_tag;       /*     0    12 */
	struct ib_flow_spec_action_drop drop;          /*     0     8 */
	struct ib_flow_spec_action_handle action;      /*     0    16 */
	struct ib_flow_spec_action_count flow_count;   /*     0    16 */
};
struct ib_flow_attr {
	enum ib_flow_attr_type     type;                 /*     0     4 */
	u16                        size;                 /*     4     2 */
	u16                        priority;             /*     6     2 */
	u32                        flags;                /*     8     4 */
	u8                         num_of_specs;         /*    12     1 */
	u8                         port;                 /*    13     1 */

	/* XXX 2 bytes hole, try to pack */

	union ib_flow_spec         flows[];              /*    16     0 */

	/* size: 16, cachelines: 1, members: 7 */
	/* sum members: 14, holes: 1, sum holes: 2 */
	/* last cacheline: 16 bytes */
};
struct ib_flow {
	struct ib_qp *             qp;                   /*     0     8 */
	struct ib_device *         device;               /*     8     8 */
	struct ib_uobject *        uobject;              /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct ib_flow_action_attrs_esp_keymats {
	enum ib_uverbs_flow_action_esp_keymat protocol;  /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	union {
		struct ib_uverbs_flow_action_esp_keymat_aes_gcm aes_gcm __attribute__((__aligned__(8))); /*     8    56 */
	} keymat __attribute__((__aligned__(8)));        /*     8    56 */

	/* size: 64, cachelines: 1, members: 2 */
	/* sum members: 60, holes: 1, sum holes: 4 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 4 */
} __attribute__((__aligned__(8)));
struct ib_flow_action_attrs_esp_replays {
	enum ib_uverbs_flow_action_esp_replay protocol;  /*     0     4 */
	union {
		struct ib_uverbs_flow_action_esp_replay_bmp bmp; /*     4     4 */
	} replay;                                        /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct ib_flow_spec_list {
	struct ib_flow_spec_list * next;                 /*     0     8 */
	union ib_flow_spec         spec;                 /*     8    88 */

	/* size: 96, cachelines: 2, members: 2 */
	/* last cacheline: 32 bytes */
};
struct ib_flow_action_attrs_esp {
	struct ib_flow_action_attrs_esp_keymats * keymat; /*     0     8 */
	struct ib_flow_action_attrs_esp_replays * replay; /*     8     8 */
	struct ib_flow_spec_list * encap;                /*    16     8 */
	u32                        esn;                  /*    24     4 */
	u32                        spi;                  /*    28     4 */
	u32                        seq;                  /*    32     4 */
	u32                        tfc_pad;              /*    36     4 */
	u64                        flags;                /*    40     8 */
	u64                        hard_limit_pkts;      /*    48     8 */

	/* size: 56, cachelines: 1, members: 9 */
	/* last cacheline: 56 bytes */
};
struct ib_port_cache {
	u64                        subnet_prefix;        /*     0     8 */
	struct ib_pkey_cache *     pkey;                 /*     8     8 */
	struct ib_gid_table *      gid;                  /*    16     8 */
	u8                         lmc;                  /*    24     1 */

	/* XXX 3 bytes hole, try to pack */

	enum ib_port_state         port_state;           /*    28     4 */

	/* size: 32, cachelines: 1, members: 5 */
	/* sum members: 29, holes: 1, sum holes: 3 */
	/* last cacheline: 32 bytes */
};
struct ib_port_immutable {
	int                        pkey_tbl_len;         /*     0     4 */
	int                        gid_tbl_len;          /*     4     4 */
	u32                        core_cap_flags;       /*     8     4 */
	u32                        max_mad_size;         /*    12     4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct ib_port_data {
	struct ib_device *         ib_dev;               /*     0     8 */
	struct ib_port_immutable   immutable;            /*     8    16 */
	spinlock_t                 pkey_list_lock;       /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           pkey_list;            /*    32    16 */
	struct ib_port_cache       cache;                /*    48    32 */
	/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
	spinlock_t                 netdev_lock;          /*    80     4 */

	/* XXX 4 bytes hole, try to pack */

	struct net_device *        netdev;               /*    88     8 */
	struct hlist_node          ndev_hash_link;       /*    96    16 */
	struct rdma_port_counter   port_counter;         /*   112    64 */
	/* --- cacheline 2 boundary (128 bytes) was 48 bytes ago --- */
	struct rdma_hw_stats *     hw_stats;             /*   176     8 */

	/* size: 184, cachelines: 3, members: 10 */
	/* sum members: 176, holes: 2, sum holes: 8 */
	/* last cacheline: 56 bytes */
};
struct rdma_netdev_alloc_params {
	size_t                     sizeof_priv;          /*     0     8 */
	unsigned int               txqs;                 /*     8     4 */
	unsigned int               rxqs;                 /*    12     4 */
	void *                     param;                /*    16     8 */
	int                        (*initialize_rdma_netdev)(struct ib_device *, u8, struct net_device *, void *); /*    24     8 */

	/* size: 32, cachelines: 1, members: 5 */
	/* last cacheline: 32 bytes */
};
struct ib_counters_read_attr {
	u64 *                      counters_buff;        /*     0     8 */
	u32                        ncounters;            /*     8     4 */
	u32                        flags;                /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct rdma_user_mmap_entry {
	struct kref                ref;                  /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct ib_ucontext *       ucontext;             /*     8     8 */
	long unsigned int          start_pgoff;          /*    16     8 */
	size_t                     npages;               /*    24     8 */
	bool                       driver_removed;       /*    32     1 */

	/* size: 40, cachelines: 1, members: 5 */
	/* sum members: 29, holes: 1, sum holes: 4 */
	/* padding: 7 */
	/* last cacheline: 40 bytes */
};
struct ib_device_ops {
	struct module *            owner;                /*     0     8 */
	enum rdma_driver_id        driver_id;            /*     8     4 */
	u32                        uverbs_abi_ver;       /*    12     4 */
	unsigned int               uverbs_no_driver_id_binding:1; /*    16: 0  4 */

	/* XXX 31 bits hole, try to pack */
	/* XXX 4 bytes hole, try to pack */

	int                        (*post_send)(struct ib_qp *, const struct ib_send_wr  *, const struct ib_send_wr  * *); /*    24     8 */
	int                        (*post_recv)(struct ib_qp *, const struct ib_recv_wr  *, const struct ib_recv_wr  * *); /*    32     8 */
	void                       (*drain_rq)(struct ib_qp *); /*    40     8 */
	void                       (*drain_sq)(struct ib_qp *); /*    48     8 */
	int                        (*poll_cq)(struct ib_cq *, int, struct ib_wc *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*peek_cq)(struct ib_cq *, int); /*    64     8 */
	int                        (*req_notify_cq)(struct ib_cq *, enum ib_cq_notify_flags); /*    72     8 */
	int                        (*req_ncomp_notif)(struct ib_cq *, int); /*    80     8 */
	int                        (*post_srq_recv)(struct ib_srq *, const struct ib_recv_wr  *, const struct ib_recv_wr  * *); /*    88     8 */
	int                        (*process_mad)(struct ib_device *, int, u8, const struct ib_wc  *, const struct ib_grh  *, const struct ib_mad  *, struct ib_mad *, size_t *, u16 *); /*    96     8 */
	int                        (*query_device)(struct ib_device *, struct ib_device_attr *, struct ib_udata *); /*   104     8 */
	int                        (*modify_device)(struct ib_device *, int, struct ib_device_modify *); /*   112     8 */
	void                       (*get_dev_fw_str)(struct ib_device *, char *); /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	const struct cpumask  *    (*get_vector_affinity)(struct ib_device *, int); /*   128     8 */
	int                        (*query_port)(struct ib_device *, u8, struct ib_port_attr *); /*   136     8 */
	int                        (*modify_port)(struct ib_device *, u8, int, struct ib_port_modify *); /*   144     8 */
	int                        (*get_port_immutable)(struct ib_device *, u8, struct ib_port_immutable *); /*   152     8 */
	enum rdma_link_layer       (*get_link_layer)(struct ib_device *, u8); /*   160     8 */
	struct net_device *        (*get_netdev)(struct ib_device *, u8); /*   168     8 */
	struct net_device *        (*alloc_rdma_netdev)(struct ib_device *, u8, enum rdma_netdev_t, const char  *, unsigned char, void (*)(struct net_device *)); /*   176     8 */
	int                        (*rdma_netdev_get_params)(struct ib_device *, u8, enum rdma_netdev_t, struct rdma_netdev_alloc_params *); /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	int                        (*query_gid)(struct ib_device *, u8, int, union ib_gid *); /*   192     8 */
	int                        (*add_gid)(const struct ib_gid_attr  *, void * *); /*   200     8 */
	int                        (*del_gid)(const struct ib_gid_attr  *, void * *); /*   208     8 */
	int                        (*query_pkey)(struct ib_device *, u8, u16, u16 *); /*   216     8 */
	int                        (*alloc_ucontext)(struct ib_ucontext *, struct ib_udata *); /*   224     8 */
	void                       (*dealloc_ucontext)(struct ib_ucontext *); /*   232     8 */
	int                        (*mmap)(struct ib_ucontext *, struct vm_area_struct *); /*   240     8 */
	void                       (*mmap_free)(struct rdma_user_mmap_entry *); /*   248     8 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	void                       (*disassociate_ucontext)(struct ib_ucontext *); /*   256     8 */
	int                        (*alloc_pd)(struct ib_pd *, struct ib_udata *); /*   264     8 */
	int                        (*dealloc_pd)(struct ib_pd *, struct ib_udata *); /*   272     8 */
	int                        (*create_ah)(struct ib_ah *, struct rdma_ah_init_attr *, struct ib_udata *); /*   280     8 */
	int                        (*modify_ah)(struct ib_ah *, struct rdma_ah_attr *); /*   288     8 */
	int                        (*query_ah)(struct ib_ah *, struct rdma_ah_attr *); /*   296     8 */
	int                        (*destroy_ah)(struct ib_ah *, u32); /*   304     8 */
	int                        (*create_srq)(struct ib_srq *, struct ib_srq_init_attr *, struct ib_udata *); /*   312     8 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	int                        (*modify_srq)(struct ib_srq *, struct ib_srq_attr *, enum ib_srq_attr_mask, struct ib_udata *); /*   320     8 */
	int                        (*query_srq)(struct ib_srq *, struct ib_srq_attr *); /*   328     8 */
	int                        (*destroy_srq)(struct ib_srq *, struct ib_udata *); /*   336     8 */
	struct ib_qp *             (*create_qp)(struct ib_pd *, struct ib_qp_init_attr *, struct ib_udata *); /*   344     8 */
	int                        (*modify_qp)(struct ib_qp *, struct ib_qp_attr *, int, struct ib_udata *); /*   352     8 */
	int                        (*query_qp)(struct ib_qp *, struct ib_qp_attr *, int, struct ib_qp_init_attr *); /*   360     8 */
	int                        (*destroy_qp)(struct ib_qp *, struct ib_udata *); /*   368     8 */
	int                        (*create_cq)(struct ib_cq *, const struct ib_cq_init_attr  *, struct ib_udata *); /*   376     8 */
	/* --- cacheline 6 boundary (384 bytes) --- */
	int                        (*modify_cq)(struct ib_cq *, u16, u16); /*   384     8 */
	int                        (*destroy_cq)(struct ib_cq *, struct ib_udata *); /*   392     8 */
	int                        (*resize_cq)(struct ib_cq *, int, struct ib_udata *); /*   400     8 */
	struct ib_mr *             (*get_dma_mr)(struct ib_pd *, int); /*   408     8 */
	struct ib_mr *             (*reg_user_mr)(struct ib_pd *, u64, u64, u64, int, struct ib_udata *); /*   416     8 */
	int                        (*rereg_user_mr)(struct ib_mr *, int, u64, u64, u64, int, struct ib_pd *, struct ib_udata *); /*   424     8 */
	int                        (*dereg_mr)(struct ib_mr *, struct ib_udata *); /*   432     8 */
	struct ib_mr *             (*alloc_mr)(struct ib_pd *, enum ib_mr_type, u32); /*   440     8 */
	/* --- cacheline 7 boundary (448 bytes) --- */
	struct ib_mr *             (*alloc_mr_integrity)(struct ib_pd *, u32, u32); /*   448     8 */
	int                        (*advise_mr)(struct ib_pd *, enum ib_uverbs_advise_mr_advice, u32, struct ib_sge *, u32, struct uverbs_attr_bundle *); /*   456     8 */
	int                        (*map_mr_sg)(struct ib_mr *, struct scatterlist *, int, unsigned int *); /*   464     8 */
	int                        (*check_mr_status)(struct ib_mr *, u32, struct ib_mr_status *); /*   472     8 */
	int                        (*alloc_mw)(struct ib_mw *, struct ib_udata *); /*   480     8 */
	int                        (*dealloc_mw)(struct ib_mw *); /*   488     8 */
	int                        (*attach_mcast)(struct ib_qp *, union ib_gid *, u16); /*   496     8 */
	int                        (*detach_mcast)(struct ib_qp *, union ib_gid *, u16); /*   504     8 */
	/* --- cacheline 8 boundary (512 bytes) --- */
	int                        (*alloc_xrcd)(struct ib_xrcd *, struct ib_udata *); /*   512     8 */
	int                        (*dealloc_xrcd)(struct ib_xrcd *, struct ib_udata *); /*   520     8 */
	struct ib_flow *           (*create_flow)(struct ib_qp *, struct ib_flow_attr *, struct ib_udata *); /*   528     8 */
	int                        (*destroy_flow)(struct ib_flow *); /*   536     8 */
	struct ib_flow_action *    (*create_flow_action_esp)(struct ib_device *, const struct ib_flow_action_attrs_esp  *, struct uverbs_attr_bundle *); /*   544     8 */
	int                        (*destroy_flow_action)(struct ib_flow_action *); /*   552     8 */
	int                        (*modify_flow_action_esp)(struct ib_flow_action *, const struct ib_flow_action_attrs_esp  *, struct uverbs_attr_bundle *); /*   560     8 */
	int                        (*set_vf_link_state)(struct ib_device *, int, u8, int); /*   568     8 */
	/* --- cacheline 9 boundary (576 bytes) --- */
	int                        (*get_vf_config)(struct ib_device *, int, u8, struct ifla_vf_info *); /*   576     8 */
	int                        (*get_vf_stats)(struct ib_device *, int, u8, struct ifla_vf_stats *); /*   584     8 */
	int                        (*get_vf_guid)(struct ib_device *, int, u8, struct ifla_vf_guid *, struct ifla_vf_guid *); /*   592     8 */
	int                        (*set_vf_guid)(struct ib_device *, int, u8, u64, int); /*   600     8 */
	struct ib_wq *             (*create_wq)(struct ib_pd *, struct ib_wq_init_attr *, struct ib_udata *); /*   608     8 */
	int                        (*destroy_wq)(struct ib_wq *, struct ib_udata *); /*   616     8 */
	int                        (*modify_wq)(struct ib_wq *, struct ib_wq_attr *, u32, struct ib_udata *); /*   624     8 */
	int                        (*create_rwq_ind_table)(struct ib_rwq_ind_table *, struct ib_rwq_ind_table_init_attr *, struct ib_udata *); /*   632     8 */
	/* --- cacheline 10 boundary (640 bytes) --- */
	int                        (*destroy_rwq_ind_table)(struct ib_rwq_ind_table *); /*   640     8 */
	struct ib_dm *             (*alloc_dm)(struct ib_device *, struct ib_ucontext *, struct ib_dm_alloc_attr *, struct uverbs_attr_bundle *); /*   648     8 */
	int                        (*dealloc_dm)(struct ib_dm *, struct uverbs_attr_bundle *); /*   656     8 */
	struct ib_mr *             (*reg_dm_mr)(struct ib_pd *, struct ib_dm *, struct ib_dm_mr_attr *, struct uverbs_attr_bundle *); /*   664     8 */
	int                        (*create_counters)(struct ib_counters *, struct uverbs_attr_bundle *); /*   672     8 */
	int                        (*destroy_counters)(struct ib_counters *); /*   680     8 */
	int                        (*read_counters)(struct ib_counters *, struct ib_counters_read_attr *, struct uverbs_attr_bundle *); /*   688     8 */
	int                        (*map_mr_sg_pi)(struct ib_mr *, struct scatterlist *, int, unsigned int *, struct scatterlist *, int, unsigned int *); /*   696     8 */
	/* --- cacheline 11 boundary (704 bytes) --- */
	struct rdma_hw_stats *     (*alloc_hw_stats)(struct ib_device *, u8); /*   704     8 */
	int                        (*get_hw_stats)(struct ib_device *, struct rdma_hw_stats *, u8, int); /*   712     8 */
	int                        (*init_port)(struct ib_device *, u8, struct kobject *); /*   720     8 */
	int                        (*fill_res_mr_entry)(struct sk_buff *, struct ib_mr *); /*   728     8 */
	int                        (*fill_res_mr_entry_raw)(struct sk_buff *, struct ib_mr *); /*   736     8 */
	int                        (*fill_res_cq_entry)(struct sk_buff *, struct ib_cq *); /*   744     8 */
	int                        (*fill_res_cq_entry_raw)(struct sk_buff *, struct ib_cq *); /*   752     8 */
	int                        (*fill_res_qp_entry)(struct sk_buff *, struct ib_qp *); /*   760     8 */
	/* --- cacheline 12 boundary (768 bytes) --- */
	int                        (*fill_res_qp_entry_raw)(struct sk_buff *, struct ib_qp *); /*   768     8 */
	int                        (*fill_res_cm_id_entry)(struct sk_buff *, struct rdma_cm_id *); /*   776     8 */
	int                        (*enable_driver)(struct ib_device *); /*   784     8 */
	void                       (*dealloc_driver)(struct ib_device *); /*   792     8 */
	void                       (*iw_add_ref)(struct ib_qp *); /*   800     8 */
	void                       (*iw_rem_ref)(struct ib_qp *); /*   808     8 */
	struct ib_qp *             (*iw_get_qp)(struct ib_device *, int); /*   816     8 */
	int                        (*iw_connect)(struct iw_cm_id *, struct iw_cm_conn_param *); /*   824     8 */
	/* --- cacheline 13 boundary (832 bytes) --- */
	int                        (*iw_accept)(struct iw_cm_id *, struct iw_cm_conn_param *); /*   832     8 */
	int                        (*iw_reject)(struct iw_cm_id *, const void  *, u8); /*   840     8 */
	int                        (*iw_create_listen)(struct iw_cm_id *, int); /*   848     8 */
	int                        (*iw_destroy_listen)(struct iw_cm_id *); /*   856     8 */
	int                        (*counter_bind_qp)(struct rdma_counter *, struct ib_qp *); /*   864     8 */
	int                        (*counter_unbind_qp)(struct ib_qp *); /*   872     8 */
	int                        (*counter_dealloc)(struct rdma_counter *); /*   880     8 */
	struct rdma_hw_stats *     (*counter_alloc_stats)(struct rdma_counter *); /*   888     8 */
	/* --- cacheline 14 boundary (896 bytes) --- */
	int                        (*counter_update_stats)(struct rdma_counter *); /*   896     8 */
	int                        (*fill_stat_mr_entry)(struct sk_buff *, struct ib_mr *); /*   904     8 */
	int                        (*query_ucontext)(struct ib_ucontext *, struct uverbs_attr_bundle *); /*   912     8 */
	size_t                     size_ib_ah;           /*   920     8 */
	size_t                     size_ib_counters;     /*   928     8 */
	size_t                     size_ib_cq;           /*   936     8 */
	size_t                     size_ib_mw;           /*   944     8 */
	size_t                     size_ib_pd;           /*   952     8 */
	/* --- cacheline 15 boundary (960 bytes) --- */
	size_t                     size_ib_rwq_ind_table; /*   960     8 */
	size_t                     size_ib_srq;          /*   968     8 */
	size_t                     size_ib_ucontext;     /*   976     8 */
	size_t                     size_ib_xrcd;         /*   984     8 */

	/* size: 992, cachelines: 16, members: 125 */
	/* sum members: 984, holes: 1, sum holes: 4 */
	/* sum bitfield members: 1 bits, bit holes: 1, sum bit holes: 31 bits */
	/* last cacheline: 32 bytes */
};
struct ib_core_device {
	struct device              dev __attribute__((__aligned__(8))); /*     0  1000 */
	/* --- cacheline 15 boundary (960 bytes) was 40 bytes ago --- */
	possible_net_t             rdma_net;             /*  1000     8 */
	struct kobject *           ports_kobj;           /*  1008     8 */
	struct list_head           port_list;            /*  1016    16 */
	/* --- cacheline 16 boundary (1024 bytes) was 8 bytes ago --- */
	struct ib_device *         owner;                /*  1032     8 */

	/* size: 1040, cachelines: 17, members: 5 */
	/* forced alignments: 1 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct blk_zone_report {
	__u64                      sector;               /*     0     8 */
	__u32                      nr_zones;             /*     8     4 */
	__u32                      flags;                /*    12     4 */
	struct blk_zone            zones[];              /*    16     0 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct blk_zone_range {
	__u64                      sector;               /*     0     8 */
	__u64                      nr_sectors;           /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct zone_report_args {
	struct blk_zone *          zones;                /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct blk_revalidate_zone_args {
	struct gendisk *           disk;                 /*     0     8 */
	long unsigned int *        conv_zones_bitmap;    /*     8     8 */
	long unsigned int *        seq_zones_wlock;      /*    16     8 */
	unsigned int               nr_zones;             /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	sector_t                   zone_sectors;         /*    32     8 */
	sector_t                   sector;               /*    40     8 */

	/* size: 48, cachelines: 1, members: 6 */
	/* sum members: 44, holes: 1, sum holes: 4 */
	/* last cacheline: 48 bytes */
};
struct rq_wb {
	unsigned int               wb_background;        /*     0     4 */
	unsigned int               wb_normal;            /*     4     4 */
	short int                  enable_state;         /*     8     2 */

	/* XXX 2 bytes hole, try to pack */

	unsigned int               unknown_cnt;          /*    12     4 */
	u64                        win_nsec;             /*    16     8 */
	u64                        cur_win_nsec;         /*    24     8 */
	struct blk_stat_callback * cb;                   /*    32     8 */
	u64                        sync_issue;           /*    40     8 */
	void *                     sync_cookie;          /*    48     8 */
	unsigned int               wc;                   /*    56     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	long unsigned int          last_issue;           /*    64     8 */
	long unsigned int          last_comp;            /*    72     8 */
	long unsigned int          min_lat_nsec;         /*    80     8 */
	struct rq_qos              rqos;                 /*    88    40 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct rq_wait             rq_wait[3];           /*   128    96 */
	/* --- cacheline 3 boundary (192 bytes) was 32 bytes ago --- */
	struct rq_depth            rq_depth;             /*   224    20 */

	/* size: 248, cachelines: 4, members: 16 */
	/* sum members: 238, holes: 2, sum holes: 6 */
	/* padding: 4 */
	/* last cacheline: 56 bytes */
};
struct trace_event_raw_wbt_stat {
	struct trace_entry         ent;                  /*     0     8 */
	char                       name[32];             /*     8    32 */
	s64                        rmean;                /*    40     8 */
	u64                        rmin;                 /*    48     8 */
	u64                        rmax;                 /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	s64                        rnr_samples;          /*    64     8 */
	s64                        rtime;                /*    72     8 */
	s64                        wmean;                /*    80     8 */
	u64                        wmin;                 /*    88     8 */
	u64                        wmax;                 /*    96     8 */
	s64                        wnr_samples;          /*   104     8 */
	s64                        wtime;                /*   112     8 */
	char                       __data[];             /*   120     0 */

	/* size: 120, cachelines: 2, members: 13 */
	/* last cacheline: 56 bytes */
};
struct trace_event_raw_wbt_lat {
	struct trace_entry         ent;                  /*     0     8 */
	char                       name[32];             /*     8    32 */
	long unsigned int          lat;                  /*    40     8 */
	char                       __data[];             /*    48     0 */

	/* size: 48, cachelines: 1, members: 4 */
	/* last cacheline: 48 bytes */
};
struct trace_event_raw_wbt_step {
	struct trace_entry         ent;                  /*     0     8 */
	char                       name[32];             /*     8    32 */
	const char  *              msg;                  /*    40     8 */
	int                        step;                 /*    48     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          window;               /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	unsigned int               bg;                   /*    64     4 */
	unsigned int               normal;               /*    68     4 */
	unsigned int               max;                  /*    72     4 */
	char                       __data[];             /*    76     0 */

	/* size: 80, cachelines: 2, members: 9 */
	/* sum members: 72, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct trace_event_raw_wbt_timer {
	struct trace_entry         ent;                  /*     0     8 */
	char                       name[32];             /*     8    32 */
	unsigned int               status;               /*    40     4 */
	int                        step;                 /*    44     4 */
	unsigned int               inflight;             /*    48     4 */
	char                       __data[];             /*    52     0 */

	/* size: 52, cachelines: 1, members: 6 */
	/* last cacheline: 52 bytes */
};
struct trace_event_data_offsets_wbt_stat {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_wbt_lat {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_wbt_step {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_wbt_timer {

	/* size: 0, cachelines: 0, members: 0 */
};
struct wbt_wait_data {
	struct rq_wb *             rwb;                  /*     0     8 */
	enum wbt_flags             wb_acct;              /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          rw;                   /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct show_busy_params {
	struct seq_file *          m;                    /*     0     8 */
	struct blk_mq_hw_ctx *     hctx;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct siprand_state {
	long unsigned int          v0;                   /*     0     8 */
	long unsigned int          v1;                   /*     8     8 */
	long unsigned int          v2;                   /*    16     8 */
	long unsigned int          v3;                   /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct region {
	unsigned int               start;                /*     0     4 */
	unsigned int               off;                  /*     4     4 */
	unsigned int               group_len;            /*     8     4 */
	unsigned int               end;                  /*    12     4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct sg_page_iter {
	struct scatterlist *       sg;                   /*     0     8 */
	unsigned int               sg_pgoffset;          /*     8     4 */
	unsigned int               __nents;              /*    12     4 */
	int                        __pg_advance;         /*    16     4 */

	/* size: 24, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct sg_dma_page_iter {
	struct sg_page_iter        base;                 /*     0    24 */

	/* XXX last struct has 4 bytes of padding */

	/* size: 24, cachelines: 1, members: 1 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 24 bytes */
};
struct sg_mapping_iter {
	struct page *              page;                 /*     0     8 */
	void *                     addr;                 /*     8     8 */
	size_t                     length;               /*    16     8 */
	size_t                     consumed;             /*    24     8 */
	struct sg_page_iter        piter;                /*    32    24 */

	/* XXX last struct has 4 bytes of padding */

	unsigned int               __offset;             /*    56     4 */
	unsigned int               __remaining;          /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	unsigned int               __flags;              /*    64     4 */

	/* size: 72, cachelines: 2, members: 8 */
	/* padding: 4 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 8 bytes */
};
struct csum_state {
	__wsum                     csum;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	size_t                     off;                  /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct rhltable {
	struct rhashtable          ht;                   /*     0   168 */

	/* size: 168, cachelines: 3, members: 1 */
	/* last cacheline: 40 bytes */
};
struct rhashtable_walker {
	struct list_head           list;                 /*     0    16 */
	struct bucket_table *      tbl;                  /*    16     8 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct rhashtable_iter {
	struct rhashtable *        ht;                   /*     0     8 */
	struct rhash_head *        p;                    /*     8     8 */
	struct rhlist_head *       list;                 /*    16     8 */
	struct rhashtable_walker   walker;               /*    24    24 */
	unsigned int               slot;                 /*    48     4 */
	unsigned int               skip;                 /*    52     4 */
	bool                       end_of_table;         /*    56     1 */

	/* size: 64, cachelines: 1, members: 7 */
	/* padding: 7 */
};
union nested_table {
	union nested_table *       table;              /*     0     8 */
	struct rhash_lock_head *   bucket;             /*     0     8 */
};
struct once_work {
	struct work_struct         work;                 /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct static_key_true *   key;                  /*    64     8 */
	struct module *            module;               /*    72     8 */

	/* size: 80, cachelines: 2, members: 3 */
	/* last cacheline: 16 bytes */
};
struct genradix_iter {
	size_t                     offset;               /*     0     8 */
	size_t                     pos;                  /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct genradix_node {
	union {
		struct genradix_node * children[512];    /*     0  4096 */
		u8                 data[4096];           /*     0  4096 */
	};                                               /*     0  4096 */

	/* size: 4096, cachelines: 64, members: 1 */
};
struct reciprocal_value_adv {
	u32                        m;                    /*     0     4 */
	u8                         sh;                   /*     4     1 */
	u8                         exp;                  /*     5     1 */
	bool                       is_wide_m;            /*     6     1 */

	/* size: 8, cachelines: 1, members: 4 */
	/* padding: 1 */
	/* last cacheline: 8 bytes */
};
struct blake2s_state {
	u32                        h[8];                 /*     0    32 */
	u32                        t[2];                 /*    32     8 */
	u32                        f[2];                 /*    40     8 */
	u8                         buf[64];              /*    48    64 */
	/* --- cacheline 1 boundary (64 bytes) was 48 bytes ago --- */
	unsigned int               buflen;               /*   112     4 */
	unsigned int               outlen;               /*   116     4 */

	/* size: 120, cachelines: 2, members: 6 */
	/* last cacheline: 56 bytes */
};
struct pcim_iomap_devres {
	void *                     table[6];             /*     0    48 */

	/* size: 48, cachelines: 1, members: 1 */
	/* last cacheline: 48 bytes */
};
struct btree_head {
	long unsigned int *        node;                 /*     0     8 */
	mempool_t *                mempool;              /*     8     8 */
	int                        height;               /*    16     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct btree_geo {
	int                        keylen;               /*     0     4 */
	int                        no_pairs;             /*     4     4 */
	int                        no_longs;             /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct assoc_array_walk_result {
	struct {
		struct assoc_array_node * node;          /*     0     8 */
		int                level;                /*     8     4 */
		int                slot;                 /*    12     4 */
	} terminal_node;                                 /*     0    16 */
	struct {
		struct assoc_array_shortcut * shortcut;  /*    16     8 */
		int                level;                /*    24     4 */
		int                sc_level;             /*    28     4 */
		long unsigned int  sc_segments;          /*    32     8 */
		long unsigned int  dissimilarity;        /*    40     8 */
	} wrong_shortcut;                                /*    16    32 */

	/* size: 48, cachelines: 1, members: 2 */
	/* last cacheline: 48 bytes */
};
struct assoc_array_delete_collapse_context {
	struct assoc_array_node *  node;                 /*     0     8 */
	const void  *              skip_leaf;            /*     8     8 */
	int                        slot;                 /*    16     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct xxh32_state {
	uint32_t                   total_len_32;         /*     0     4 */
	uint32_t                   large_len;            /*     4     4 */
	uint32_t                   v1;                   /*     8     4 */
	uint32_t                   v2;                   /*    12     4 */
	uint32_t                   v3;                   /*    16     4 */
	uint32_t                   v4;                   /*    20     4 */
	uint32_t                   mem32[4];             /*    24    16 */
	uint32_t                   memsize;              /*    40     4 */

	/* size: 44, cachelines: 1, members: 8 */
	/* last cacheline: 44 bytes */
};
struct gen_pool_chunk {
	struct list_head           next_chunk;           /*     0    16 */
	atomic_long_t              avail;                /*    16     8 */
	phys_addr_t                phys_addr;            /*    24     8 */
	void *                     owner;                /*    32     8 */
	long unsigned int          start_addr;           /*    40     8 */
	long unsigned int          end_addr;             /*    48     8 */
	long unsigned int          bits[];               /*    56     0 */

	/* size: 56, cachelines: 1, members: 7 */
	/* last cacheline: 56 bytes */
};
struct genpool_data_align {
	int                        align;                /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct genpool_data_fixed {
	long unsigned int          offset;               /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct inflate_state {
	inflate_mode               mode;                 /*     0     4 */
	int                        last;                 /*     4     4 */
	int                        wrap;                 /*     8     4 */
	int                        havedict;             /*    12     4 */
	int                        flags;                /*    16     4 */
	unsigned int               dmax;                 /*    20     4 */
	long unsigned int          check;                /*    24     8 */
	long unsigned int          total;                /*    32     8 */
	unsigned int               wbits;                /*    40     4 */
	unsigned int               wsize;                /*    44     4 */
	unsigned int               whave;                /*    48     4 */
	unsigned int               write;                /*    52     4 */
	unsigned char *            window;               /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	long unsigned int          hold;                 /*    64     8 */
	unsigned int               bits;                 /*    72     4 */
	unsigned int               length;               /*    76     4 */
	unsigned int               offset;               /*    80     4 */
	unsigned int               extra;                /*    84     4 */
	const code  *              lencode;              /*    88     8 */
	const code  *              distcode;             /*    96     8 */
	unsigned int               lenbits;              /*   104     4 */
	unsigned int               distbits;             /*   108     4 */
	unsigned int               ncode;                /*   112     4 */
	unsigned int               nlen;                 /*   116     4 */
	unsigned int               ndist;                /*   120     4 */
	unsigned int               have;                 /*   124     4 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	code *                     next;                 /*   128     8 */
	short unsigned int         lens[320];            /*   136   640 */
	/* --- cacheline 12 boundary (768 bytes) was 8 bytes ago --- */
	short unsigned int         work[288];            /*   776   576 */
	/* --- cacheline 21 boundary (1344 bytes) was 8 bytes ago --- */
	code                       codes[2048];          /*  1352  8192 */

	/* size: 9544, cachelines: 150, members: 30 */
	/* last cacheline: 8 bytes */
};
union uu {
	short unsigned int         us;                 /*     0     2 */
	unsigned char              b[2];               /*     0     2 */
};
struct inflate_workspace {
	struct inflate_state       inflate_state;        /*     0  9544 */
	/* --- cacheline 149 boundary (9536 bytes) was 8 bytes ago --- */
	unsigned char              working_window[32768]; /*  9544 32768 */

	/* size: 42312, cachelines: 662, members: 2 */
	/* last cacheline: 8 bytes */
};
struct ct_data_s {
	union {
		ush                freq;                 /*     0     2 */
		ush                code;                 /*     0     2 */
	} fc;                                            /*     0     2 */
	union {
		ush                dad;                  /*     2     2 */
		ush                len;                  /*     2     2 */
	} dl;                                            /*     2     2 */

	/* size: 4, cachelines: 1, members: 2 */
	/* last cacheline: 4 bytes */
};
struct tree_desc_s {
	ct_data *                  dyn_tree;             /*     0     8 */
	int                        max_code;             /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	static_tree_desc *         stat_desc;            /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct deflate_state {
	z_streamp                  strm;                 /*     0     8 */
	int                        status;               /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	Byte *                     pending_buf;          /*    16     8 */
	ulg                        pending_buf_size;     /*    24     8 */
	Byte *                     pending_out;          /*    32     8 */
	int                        pending;              /*    40     4 */
	int                        noheader;             /*    44     4 */
	Byte                       data_type;            /*    48     1 */
	Byte                       method;               /*    49     1 */

	/* XXX 2 bytes hole, try to pack */

	int                        last_flush;           /*    52     4 */
	uInt                       w_size;               /*    56     4 */
	uInt                       w_bits;               /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	uInt                       w_mask;               /*    64     4 */

	/* XXX 4 bytes hole, try to pack */

	Byte *                     window;               /*    72     8 */
	ulg                        window_size;          /*    80     8 */
	Pos *                      prev;                 /*    88     8 */
	Pos *                      head;                 /*    96     8 */
	uInt                       ins_h;                /*   104     4 */
	uInt                       hash_size;            /*   108     4 */
	uInt                       hash_bits;            /*   112     4 */
	uInt                       hash_mask;            /*   116     4 */
	uInt                       hash_shift;           /*   120     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 2 boundary (128 bytes) --- */
	long int                   block_start;          /*   128     8 */
	uInt                       match_length;         /*   136     4 */
	IPos                       prev_match;           /*   140     4 */
	int                        match_available;      /*   144     4 */
	uInt                       strstart;             /*   148     4 */
	uInt                       match_start;          /*   152     4 */
	uInt                       lookahead;            /*   156     4 */
	uInt                       prev_length;          /*   160     4 */
	uInt                       max_chain_length;     /*   164     4 */
	uInt                       max_lazy_match;       /*   168     4 */
	int                        level;                /*   172     4 */
	int                        strategy;             /*   176     4 */
	uInt                       good_match;           /*   180     4 */
	int                        nice_match;           /*   184     4 */
	struct ct_data_s           dyn_ltree[573];       /*   188  2292 */
	/* --- cacheline 38 boundary (2432 bytes) was 48 bytes ago --- */
	struct ct_data_s           dyn_dtree[61];        /*  2480   244 */
	/* --- cacheline 42 boundary (2688 bytes) was 36 bytes ago --- */
	struct ct_data_s           bl_tree[39];          /*  2724   156 */
	/* --- cacheline 45 boundary (2880 bytes) --- */
	struct tree_desc_s         l_desc;               /*  2880    24 */
	struct tree_desc_s         d_desc;               /*  2904    24 */
	struct tree_desc_s         bl_desc;              /*  2928    24 */
	/* --- cacheline 46 boundary (2944 bytes) was 8 bytes ago --- */
	ush                        bl_count[16];         /*  2952    32 */
	int                        heap[573];            /*  2984  2292 */
	/* --- cacheline 82 boundary (5248 bytes) was 28 bytes ago --- */
	int                        heap_len;             /*  5276     4 */
	int                        heap_max;             /*  5280     4 */
	uch                        depth[573];           /*  5284   573 */

	/* XXX 7 bytes hole, try to pack */

	/* --- cacheline 91 boundary (5824 bytes) was 40 bytes ago --- */
	uch *                      l_buf;                /*  5864     8 */
	uInt                       lit_bufsize;          /*  5872     4 */
	uInt                       last_lit;             /*  5876     4 */
	ush *                      d_buf;                /*  5880     8 */
	/* --- cacheline 92 boundary (5888 bytes) --- */
	ulg                        opt_len;              /*  5888     8 */
	ulg                        static_len;           /*  5896     8 */
	ulg                        compressed_len;       /*  5904     8 */
	uInt                       matches;              /*  5912     4 */
	int                        last_eob_len;         /*  5916     4 */
	ush                        bi_buf;               /*  5920     2 */

	/* XXX 2 bytes hole, try to pack */

	int                        bi_valid;             /*  5924     4 */

	/* size: 5928, cachelines: 93, members: 58 */
	/* sum members: 5905, holes: 6, sum holes: 23 */
	/* last cacheline: 40 bytes */
};
struct deflate_workspace {
	deflate_state              deflate_memory;       /*     0  5928 */
	/* --- cacheline 92 boundary (5888 bytes) was 40 bytes ago --- */
	Byte *                     window_memory;        /*  5928     8 */
	Pos *                      prev_memory;          /*  5936     8 */
	Pos *                      head_memory;          /*  5944     8 */
	/* --- cacheline 93 boundary (5952 bytes) --- */
	char *                     overlay_memory;       /*  5952     8 */

	/* size: 5960, cachelines: 94, members: 5 */
	/* last cacheline: 8 bytes */
};
struct config_s {
	ush                        good_length;          /*     0     2 */
	ush                        max_lazy;             /*     2     2 */
	ush                        nice_length;          /*     4     2 */
	ush                        max_chain;            /*     6     2 */
	compress_func              func;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 5 */
	/* last cacheline: 16 bytes */
};
struct static_tree_desc_s {
	const ct_data  *           static_tree;          /*     0     8 */
	const int  *               extra_bits;           /*     8     8 */
	int                        extra_base;           /*    16     4 */
	int                        elems;                /*    20     4 */
	int                        max_length;           /*    24     4 */

	/* size: 32, cachelines: 1, members: 5 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct HUF_CElt_s {
	U16                        val;                  /*     0     2 */
	BYTE                       nbBits;               /*     2     1 */

	/* size: 4, cachelines: 1, members: 2 */
	/* padding: 1 */
	/* last cacheline: 4 bytes */
};
struct nodeElt_s {
	U32                        count;                /*     0     4 */
	U16                        parent;               /*     4     2 */
	BYTE                       byte;                 /*     6     1 */
	BYTE                       nbBits;               /*     7     1 */

	/* size: 8, cachelines: 1, members: 4 */
	/* last cacheline: 8 bytes */
};
struct ZSTD_CCtx_s {
	const BYTE  *              nextSrc;              /*     0     8 */
	const BYTE  *              base;                 /*     8     8 */
	const BYTE  *              dictBase;             /*    16     8 */
	U32                        dictLimit;            /*    24     4 */
	U32                        lowLimit;             /*    28     4 */
	U32                        nextToUpdate;         /*    32     4 */
	U32                        nextToUpdate3;        /*    36     4 */
	U32                        hashLog3;             /*    40     4 */
	U32                        loadedDictEnd;        /*    44     4 */
	U32                        forceWindow;          /*    48     4 */
	U32                        forceRawDict;         /*    52     4 */
	ZSTD_compressionStage_e    stage;                /*    56     4 */
	U32                        rep[3];               /*    60    12 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	U32                        repToConfirm[3];      /*    72    12 */
	U32                        dictID;               /*    84     4 */
	ZSTD_parameters            params;               /*    88    40 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	void *                     workSpace;            /*   128     8 */
	size_t                     workSpaceSize;        /*   136     8 */
	size_t                     blockSize;            /*   144     8 */
	U64                        frameContentSize;     /*   152     8 */
	struct xxh64_state         xxhState;             /*   160    80 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 3 boundary (192 bytes) was 48 bytes ago --- */
	ZSTD_customMem             customMem;            /*   240    24 */
	/* --- cacheline 4 boundary (256 bytes) was 8 bytes ago --- */
	seqStore_t                 seqStore;             /*   264   176 */
	/* --- cacheline 6 boundary (384 bytes) was 56 bytes ago --- */
	U32 *                      hashTable;            /*   440     8 */
	/* --- cacheline 7 boundary (448 bytes) --- */
	U32 *                      hashTable3;           /*   448     8 */
	U32 *                      chainTable;           /*   456     8 */
	HUF_CElt *                 hufTable;             /*   464     8 */
	U32                        flagStaticTables;     /*   472     4 */
	HUF_repeat                 flagStaticHufTable;   /*   476     4 */
	FSE_CTable                 offcodeCTable[187];   /*   480   748 */
	/* --- cacheline 19 boundary (1216 bytes) was 12 bytes ago --- */
	FSE_CTable                 matchlengthCTable[363]; /*  1228  1452 */
	/* --- cacheline 41 boundary (2624 bytes) was 56 bytes ago --- */
	FSE_CTable                 litlengthCTable[329]; /*  2680  1316 */
	/* --- cacheline 62 boundary (3968 bytes) was 28 bytes ago --- */
	unsigned int               tmpCounters[1536];    /*  3996  6144 */

	/* size: 10144, cachelines: 159, members: 33 */
	/* padding: 4 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 32 bytes */
};
struct ZSTD_CDict_s {
	void *                     dictBuffer;           /*     0     8 */
	const void  *              dictContent;          /*     8     8 */
	size_t                     dictContentSize;      /*    16     8 */
	ZSTD_CCtx *                refContext;           /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct ZSTD_inBuffer_s {
	const void  *              src;                  /*     0     8 */
	size_t                     size;                 /*     8     8 */
	size_t                     pos;                  /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct ZSTD_outBuffer_s {
	void *                     dst;                  /*     0     8 */
	size_t                     size;                 /*     8     8 */
	size_t                     pos;                  /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct ZSTD_CStream_s {
	ZSTD_CCtx *                cctx;                 /*     0     8 */
	ZSTD_CDict *               cdictLocal;           /*     8     8 */
	const ZSTD_CDict  *        cdict;                /*    16     8 */
	char *                     inBuff;               /*    24     8 */
	size_t                     inBuffSize;           /*    32     8 */
	size_t                     inToCompress;         /*    40     8 */
	size_t                     inBuffPos;            /*    48     8 */
	size_t                     inBuffTarget;         /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	size_t                     blockSize;            /*    64     8 */
	char *                     outBuff;              /*    72     8 */
	size_t                     outBuffSize;          /*    80     8 */
	size_t                     outBuffContentSize;   /*    88     8 */
	size_t                     outBuffFlushedSize;   /*    96     8 */
	ZSTD_cStreamStage          stage;                /*   104     4 */
	U32                        checksum;             /*   108     4 */
	U32                        frameEnded;           /*   112     4 */

	/* XXX 4 bytes hole, try to pack */

	U64                        pledgedSrcSize;       /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	U64                        inputProcessed;       /*   128     8 */
	ZSTD_parameters            params;               /*   136    40 */
	ZSTD_customMem             customMem;            /*   176    24 */

	/* size: 200, cachelines: 4, members: 20 */
	/* sum members: 196, holes: 1, sum holes: 4 */
	/* last cacheline: 8 bytes */
};
struct seqDef_s {
	U32                        offset;               /*     0     4 */
	U16                        litLength;            /*     4     2 */
	U16                        matchLength;          /*     6     2 */

	/* size: 8, cachelines: 1, members: 3 */
	/* last cacheline: 8 bytes */
};
struct ZSTD_DCtx_s {
	const FSE_DTable  *        LLTptr;               /*     0     8 */
	const FSE_DTable  *        MLTptr;               /*     8     8 */
	const FSE_DTable  *        OFTptr;               /*    16     8 */
	const HUF_DTable  *        HUFptr;               /*    24     8 */
	ZSTD_entropyTables_t       entropy;              /*    32 24608 */
	/* --- cacheline 385 boundary (24640 bytes) --- */
	const void  *              previousDstEnd;       /* 24640     8 */
	const void  *              base;                 /* 24648     8 */
	const void  *              vBase;                /* 24656     8 */
	const void  *              dictEnd;              /* 24664     8 */
	size_t                     expected;             /* 24672     8 */
	ZSTD_frameParams           fParams;              /* 24680    24 */
	/* --- cacheline 386 boundary (24704 bytes) --- */
	blockType_e                bType;                /* 24704     4 */
	ZSTD_dStage                stage;                /* 24708     4 */
	U32                        litEntropy;           /* 24712     4 */
	U32                        fseEntropy;           /* 24716     4 */
	struct xxh64_state         xxhState;             /* 24720    80 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 387 boundary (24768 bytes) was 32 bytes ago --- */
	size_t                     headerSize;           /* 24800     8 */
	U32                        dictID;               /* 24808     4 */

	/* XXX 4 bytes hole, try to pack */

	const BYTE  *              litPtr;               /* 24816     8 */
	ZSTD_customMem             customMem;            /* 24824    24 */
	/* --- cacheline 388 boundary (24832 bytes) was 16 bytes ago --- */
	size_t                     litSize;              /* 24848     8 */
	size_t                     rleSize;              /* 24856     8 */
	BYTE                       litBuffer[131080];    /* 24864 131080 */
	/* --- cacheline 2436 boundary (155904 bytes) was 40 bytes ago --- */
	BYTE                       headerBuffer[18];     /* 155944    18 */

	/* size: 155968, cachelines: 2437, members: 24 */
	/* sum members: 155958, holes: 1, sum holes: 4 */
	/* padding: 6 */
	/* paddings: 1, sum paddings: 4 */
};
struct ZSTD_DDict_s {
	void *                     dictBuffer;           /*     0     8 */
	const void  *              dictContent;          /*     8     8 */
	size_t                     dictSize;             /*    16     8 */
	ZSTD_entropyTables_t       entropy;              /*    24 24608 */
	/* --- cacheline 384 boundary (24576 bytes) was 56 bytes ago --- */
	U32                        dictID;               /* 24632     4 */
	U32                        entropyPresent;       /* 24636     4 */
	/* --- cacheline 385 boundary (24640 bytes) --- */
	ZSTD_customMem             cMem;                 /* 24640    24 */

	/* size: 24664, cachelines: 386, members: 7 */
	/* last cacheline: 24 bytes */
};
struct ZSTD_DStream_s {
	ZSTD_DCtx *                dctx;                 /*     0     8 */
	ZSTD_DDict *               ddictLocal;           /*     8     8 */
	const ZSTD_DDict  *        ddict;                /*    16     8 */
	ZSTD_frameParams           fParams;              /*    24    24 */
	ZSTD_dStreamStage          stage;                /*    48     4 */

	/* XXX 4 bytes hole, try to pack */

	char *                     inBuff;               /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	size_t                     inBuffSize;           /*    64     8 */
	size_t                     inPos;                /*    72     8 */
	size_t                     maxWindowSize;        /*    80     8 */
	char *                     outBuff;              /*    88     8 */
	size_t                     outBuffSize;          /*    96     8 */
	size_t                     outStart;             /*   104     8 */
	size_t                     outEnd;               /*   112     8 */
	size_t                     blockSize;            /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	BYTE                       headerBuffer[18];     /*   128    18 */

	/* XXX 6 bytes hole, try to pack */

	size_t                     lhSize;               /*   152     8 */
	ZSTD_customMem             customMem;            /*   160    24 */
	void *                     legacyContext;        /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	U32                        previousLegacyVersion; /*   192     4 */
	U32                        legacyVersion;        /*   196     4 */
	U32                        hostageByte;          /*   200     4 */

	/* size: 208, cachelines: 4, members: 21 */
	/* sum members: 194, holes: 2, sum holes: 10 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct xz_buf {
	const uint8_t  *           in;                   /*     0     8 */
	size_t                     in_pos;               /*     8     8 */
	size_t                     in_size;              /*    16     8 */
	uint8_t *                  out;                  /*    24     8 */
	size_t                     out_pos;              /*    32     8 */
	size_t                     out_size;             /*    40     8 */

	/* size: 48, cachelines: 1, members: 6 */
	/* last cacheline: 48 bytes */
};
struct xz_dec_hash {
	vli_type                   unpadded;             /*     0     8 */
	vli_type                   uncompressed;         /*     8     8 */
	uint32_t                   crc32;                /*    16     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct xz_dec {
	enum {
		SEQ_STREAM_HEADER    = 0,
		SEQ_BLOCK_START      = 1,
		SEQ_BLOCK_HEADER     = 2,
		SEQ_BLOCK_UNCOMPRESS = 3,
		SEQ_BLOCK_PADDING    = 4,
		SEQ_BLOCK_CHECK      = 5,
		SEQ_INDEX            = 6,
		SEQ_INDEX_PADDING    = 7,
		SEQ_INDEX_CRC32      = 8,
		SEQ_STREAM_FOOTER    = 9,
	} sequence;                                      /*     0     4 */
	uint32_t                   pos;                  /*     4     4 */
	vli_type                   vli;                  /*     8     8 */
	size_t                     in_start;             /*    16     8 */
	size_t                     out_start;            /*    24     8 */
	uint32_t                   crc32;                /*    32     4 */
	enum xz_check              check_type;           /*    36     4 */
	enum xz_mode               mode;                 /*    40     4 */
	bool                       allow_buf_error;      /*    44     1 */

	/* XXX 3 bytes hole, try to pack */

	struct {
		vli_type           compressed;           /*    48     8 */
		vli_type           uncompressed;         /*    56     8 */
		/* --- cacheline 1 boundary (64 bytes) --- */
		uint32_t           size;                 /*    64     4 */
	} block_header;                                  /*    48    24 */

	/* XXX last struct has 4 bytes of padding */

	struct {
		vli_type           compressed;           /*    72     8 */
		vli_type           uncompressed;         /*    80     8 */
		vli_type           count;                /*    88     8 */
		struct xz_dec_hash hash;                 /*    96    24 */

		/* XXX last struct has 4 bytes of padding */
	} block;                                         /*    72    48 */
	struct {
		enum {
			SEQ_INDEX_COUNT        = 0,
			SEQ_INDEX_UNPADDED     = 1,
			SEQ_INDEX_UNCOMPRESSED = 2,
		} sequence;                              /*   120     4 */

		/* XXX 4 bytes hole, try to pack */

		/* --- cacheline 2 boundary (128 bytes) --- */
		vli_type           size;                 /*   128     8 */
		vli_type           count;                /*   136     8 */
		struct xz_dec_hash hash;                 /*   144    24 */

		/* XXX last struct has 4 bytes of padding */
	} index;                                         /*   120    48 */
	struct {
		size_t             pos;                  /*   168     8 */
		size_t             size;                 /*   176     8 */
		uint8_t            buf[1024];            /*   184  1024 */
	} temp;                                          /*   168  1040 */
	/* --- cacheline 18 boundary (1152 bytes) was 56 bytes ago --- */
	struct xz_dec_lzma2 *      lzma2;                /*  1208     8 */
	/* --- cacheline 19 boundary (1216 bytes) --- */
	struct xz_dec_bcj *        bcj;                  /*  1216     8 */
	bool                       bcj_active;           /*  1224     1 */

	/* size: 1232, cachelines: 20, members: 16 */
	/* sum members: 1222, holes: 1, sum holes: 3 */
	/* padding: 7 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 16 bytes */
};
struct dictionary {
	uint8_t *                  buf;                  /*     0     8 */
	size_t                     start;                /*     8     8 */
	size_t                     pos;                  /*    16     8 */
	size_t                     full;                 /*    24     8 */
	size_t                     limit;                /*    32     8 */
	size_t                     end;                  /*    40     8 */
	uint32_t                   size;                 /*    48     4 */
	uint32_t                   size_max;             /*    52     4 */
	uint32_t                   allocated;            /*    56     4 */
	enum xz_mode               mode;                 /*    60     4 */

	/* size: 64, cachelines: 1, members: 10 */
};
struct rc_dec {
	uint32_t                   range;                /*     0     4 */
	uint32_t                   code;                 /*     4     4 */
	uint32_t                   init_bytes_left;      /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	const uint8_t  *           in;                   /*    16     8 */
	size_t                     in_pos;               /*    24     8 */
	size_t                     in_limit;             /*    32     8 */

	/* size: 40, cachelines: 1, members: 6 */
	/* sum members: 36, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct lzma_len_dec {
	uint16_t                   choice;               /*     0     2 */
	uint16_t                   choice2;              /*     2     2 */
	uint16_t                   low[16][8];           /*     4   256 */
	/* --- cacheline 4 boundary (256 bytes) was 4 bytes ago --- */
	uint16_t                   mid[16][8];           /*   260   256 */
	/* --- cacheline 8 boundary (512 bytes) was 4 bytes ago --- */
	uint16_t                   high[256];            /*   516   512 */

	/* size: 1028, cachelines: 17, members: 5 */
	/* last cacheline: 4 bytes */
};
struct lzma_dec {
	uint32_t                   rep0;                 /*     0     4 */
	uint32_t                   rep1;                 /*     4     4 */
	uint32_t                   rep2;                 /*     8     4 */
	uint32_t                   rep3;                 /*    12     4 */
	enum lzma_state            state;                /*    16     4 */
	uint32_t                   len;                  /*    20     4 */
	uint32_t                   lc;                   /*    24     4 */
	uint32_t                   literal_pos_mask;     /*    28     4 */
	uint32_t                   pos_mask;             /*    32     4 */
	uint16_t                   is_match[12][16];     /*    36   384 */
	/* --- cacheline 6 boundary (384 bytes) was 36 bytes ago --- */
	uint16_t                   is_rep[12];           /*   420    24 */
	uint16_t                   is_rep0[12];          /*   444    24 */
	/* --- cacheline 7 boundary (448 bytes) was 20 bytes ago --- */
	uint16_t                   is_rep1[12];          /*   468    24 */
	uint16_t                   is_rep2[12];          /*   492    24 */
	/* --- cacheline 8 boundary (512 bytes) was 4 bytes ago --- */
	uint16_t                   is_rep0_long[12][16]; /*   516   384 */
	/* --- cacheline 14 boundary (896 bytes) was 4 bytes ago --- */
	uint16_t                   dist_slot[4][64];     /*   900   512 */
	/* --- cacheline 22 boundary (1408 bytes) was 4 bytes ago --- */
	uint16_t                   dist_special[114];    /*  1412   228 */
	/* --- cacheline 25 boundary (1600 bytes) was 40 bytes ago --- */
	uint16_t                   dist_align[16];       /*  1640    32 */
	/* --- cacheline 26 boundary (1664 bytes) was 8 bytes ago --- */
	struct lzma_len_dec        match_len_dec;        /*  1672  1028 */
	/* --- cacheline 42 boundary (2688 bytes) was 12 bytes ago --- */
	struct lzma_len_dec        rep_len_dec;          /*  2700  1028 */
	/* --- cacheline 58 boundary (3712 bytes) was 16 bytes ago --- */
	uint16_t                   literal[16][768];     /*  3728 24576 */

	/* size: 28304, cachelines: 443, members: 21 */
	/* last cacheline: 16 bytes */
};
struct lzma2_dec {
	enum lzma2_seq             sequence;             /*     0     4 */
	enum lzma2_seq             next_sequence;        /*     4     4 */
	uint32_t                   uncompressed;         /*     8     4 */
	uint32_t                   compressed;           /*    12     4 */
	bool                       need_dict_reset;      /*    16     1 */
	bool                       need_props;           /*    17     1 */

	/* size: 20, cachelines: 1, members: 6 */
	/* padding: 2 */
	/* last cacheline: 20 bytes */
};
struct xz_dec_lzma2 {
	struct rc_dec              rc;                   /*     0    40 */
	struct dictionary          dict;                 /*    40    64 */
	/* --- cacheline 1 boundary (64 bytes) was 40 bytes ago --- */
	struct lzma2_dec           lzma2;                /*   104    20 */

	/* XXX last struct has 2 bytes of padding */

	struct lzma_dec            lzma;                 /*   124 28304 */
	/* --- cacheline 444 boundary (28416 bytes) was 12 bytes ago --- */
	struct {
		uint32_t           size;                 /* 28428     4 */
		uint8_t            buf[63];              /* 28432    63 */
	} temp;                                          /* 28428    68 */

	/* XXX last struct has 1 byte of padding */

	/* size: 28496, cachelines: 446, members: 5 */
	/* paddings: 2, sum paddings: 3 */
	/* last cacheline: 16 bytes */
};
struct xz_dec_bcj {
	enum {
		BCJ_X86      = 4,
		BCJ_POWERPC  = 5,
		BCJ_IA64     = 6,
		BCJ_ARM      = 7,
		BCJ_ARMTHUMB = 8,
		BCJ_SPARC    = 9,
	} type;                                          /*     0     4 */
	enum xz_ret                ret;                  /*     4     4 */
	bool                       single_call;          /*     8     1 */

	/* XXX 3 bytes hole, try to pack */

	uint32_t                   pos;                  /*    12     4 */
	uint32_t                   x86_prev_mask;        /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	uint8_t *                  out;                  /*    24     8 */
	size_t                     out_pos;              /*    32     8 */
	size_t                     out_size;             /*    40     8 */
	struct {
		size_t             filtered;             /*    48     8 */
		size_t             size;                 /*    56     8 */
		/* --- cacheline 1 boundary (64 bytes) --- */
		uint8_t            buf[16];              /*    64    16 */
	} temp;                                          /*    48    32 */

	/* size: 80, cachelines: 2, members: 9 */
	/* sum members: 73, holes: 2, sum holes: 7 */
	/* last cacheline: 16 bytes */
};
struct ts_state {
	unsigned int               offset;               /*     0     4 */
	char                       cb[40];               /*     4    40 */

	/* size: 44, cachelines: 1, members: 2 */
	/* last cacheline: 44 bytes */
};
struct ts_ops {
	const char  *              name;                 /*     0     8 */
	struct ts_config *         (*init)(const void  *, unsigned int, gfp_t, int); /*     8     8 */
	unsigned int               (*find)(struct ts_config *, struct ts_state *); /*    16     8 */
	void                       (*destroy)(struct ts_config *); /*    24     8 */
	void *                     (*get_pattern)(struct ts_config *); /*    32     8 */
	unsigned int               (*get_pattern_len)(struct ts_config *); /*    40     8 */
	struct module *            owner;                /*    48     8 */
	struct list_head           list;                 /*    56    16 */

	/* size: 72, cachelines: 2, members: 8 */
	/* last cacheline: 8 bytes */
};
struct ts_config {
	struct ts_ops *            ops;                  /*     0     8 */
	int                        flags;                /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	unsigned int               (*get_next_block)(unsigned int, const u8  * *, struct ts_config *, struct ts_state *); /*    16     8 */
	void                       (*finish)(struct ts_config *, struct ts_state *); /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct ts_linear_state {
	unsigned int               len;                  /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	const void  *              data;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct ei_entry {
	struct list_head           list;                 /*     0    16 */
	long unsigned int          start_addr;           /*    16     8 */
	long unsigned int          end_addr;             /*    24     8 */
	int                        etype;                /*    32     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     priv;                 /*    40     8 */

	/* size: 48, cachelines: 1, members: 5 */
	/* sum members: 44, holes: 1, sum holes: 4 */
	/* last cacheline: 48 bytes */
};
struct ddebug_table {
	struct list_head           link;                 /*     0    16 */
	const char  *              mod_name;             /*    16     8 */
	unsigned int               num_ddebugs;          /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	struct _ddebug *           ddebugs;              /*    32     8 */

	/* size: 40, cachelines: 1, members: 4 */
	/* sum members: 36, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct ddebug_query {
	const char  *              filename;             /*     0     8 */
	const char  *              module;               /*     8     8 */
	const char  *              function;             /*    16     8 */
	const char  *              format;               /*    24     8 */
	unsigned int               first_lineno;         /*    32     4 */
	unsigned int               last_lineno;          /*    36     4 */

	/* size: 40, cachelines: 1, members: 6 */
	/* last cacheline: 40 bytes */
};
struct ddebug_iter {
	struct ddebug_table *      table;                /*     0     8 */
	unsigned int               idx;                  /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct flag_settings {
	unsigned int               flags;                /*     0     4 */
	unsigned int               mask;                 /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct flagsbuf {
	char                       buf[7];               /*     0     7 */

	/* size: 7, cachelines: 1, members: 1 */
	/* last cacheline: 7 bytes */
};
struct nla_bitfield32 {
	__u32                      value;                /*     0     4 */
	__u32                      selector;             /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct cpu_rmap {
	struct kref                refcount;             /*     0     4 */
	u16                        size;                 /*     4     2 */
	u16                        used;                 /*     6     2 */
	void * *                   obj;                  /*     8     8 */
	struct {
		u16                index;                /*    16     2 */
		u16                dist;                 /*    18     2 */
	} near[]; /*    16     0 */

	/* size: 16, cachelines: 1, members: 5 */
	/* last cacheline: 16 bytes */
};
struct irq_glue {
	struct irq_affinity_notify notify;               /*     0    88 */
	/* --- cacheline 1 boundary (64 bytes) was 24 bytes ago --- */
	struct cpu_rmap *          rmap;                 /*    88     8 */
	u16                        index;                /*    96     2 */

	/* size: 104, cachelines: 2, members: 3 */
	/* padding: 6 */
	/* last cacheline: 40 bytes */
};
struct field_table {
	const char  *              p;                    /*     0     8 */
	void                       (*addm)(MPI, MPI, MPI, struct mpi_ec_ctx *); /*     8     8 */
	void                       (*subm)(MPI, MPI, MPI, struct mpi_ec_ctx *); /*    16     8 */
	void                       (*mulm)(MPI, MPI, MPI, struct mpi_ec_ctx *); /*    24     8 */
	void                       (*mul2)(MPI, MPI, struct mpi_ec_ctx *); /*    32     8 */
	void                       (*pow2)(MPI, const MPI , struct mpi_ec_ctx *); /*    40     8 */

	/* size: 48, cachelines: 1, members: 6 */
	/* last cacheline: 48 bytes */
};
struct barrett_ctx_s {
	MPI                        m;                    /*     0     8 */
	int                        m_copied;             /*     8     4 */
	int                        k;                    /*    12     4 */
	MPI                        y;                    /*    16     8 */
	MPI                        r1;                   /*    24     8 */
	MPI                        r2;                   /*    32     8 */
	MPI                        r3;                   /*    40     8 */

	/* size: 48, cachelines: 1, members: 7 */
	/* last cacheline: 48 bytes */
};
struct karatsuba_ctx {
	struct karatsuba_ctx *     next;                 /*     0     8 */
	mpi_ptr_t                  tspace;               /*     8     8 */
	mpi_size_t                 tspace_size;          /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	mpi_ptr_t                  tp;                   /*    24     8 */
	mpi_size_t                 tp_size;              /*    32     4 */

	/* size: 40, cachelines: 1, members: 5 */
	/* sum members: 32, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct dim_cq_moder {
	u16                        usec;                 /*     0     2 */
	u16                        pkts;                 /*     2     2 */
	u16                        comps;                /*     4     2 */
	u8                         cq_period_mode;       /*     6     1 */

	/* size: 8, cachelines: 1, members: 4 */
	/* padding: 1 */
	/* last cacheline: 8 bytes */
};
struct pubkey_hdr {
	uint8_t                    version;              /*     0     1 */
	uint32_t                   timestamp;            /*     1     4 */
	uint8_t                    algo;                 /*     5     1 */
	uint8_t                    nmpi;                 /*     6     1 */
	char                       mpi[];                /*     7     0 */

	/* size: 7, cachelines: 1, members: 5 */
	/* last cacheline: 7 bytes */
} __attribute__((__packed__));
struct signature_hdr {
	uint8_t                    version;              /*     0     1 */
	uint32_t                   timestamp;            /*     1     4 */
	uint8_t                    algo;                 /*     5     1 */
	uint8_t                    hash;                 /*     6     1 */
	uint8_t                    keyid[8];             /*     7     8 */
	uint8_t                    nmpi;                 /*    15     1 */
	char                       mpi[];                /*    16     0 */

	/* size: 16, cachelines: 1, members: 7 */
	/* last cacheline: 16 bytes */
} __attribute__((__packed__));
struct sg_pool {
	size_t                     size;                 /*     0     8 */
	char *                     name;                 /*     8     8 */
	struct kmem_cache *        slab;                 /*    16     8 */
	mempool_t *                pool;                 /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct font_desc {
	int                        idx;                  /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	const char  *              name;                 /*     8     8 */
	unsigned int               width;                /*    16     4 */
	unsigned int               height;               /*    20     4 */
	const void  *              data;                 /*    24     8 */
	int                        pref;                 /*    32     4 */

	/* size: 40, cachelines: 1, members: 6 */
	/* sum members: 32, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct font_data {
	unsigned int               extra[4];             /*     0    16 */
	const unsigned char        data[];               /*    16     0 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct pldmfw_record {
	struct list_head           entry;                /*     0    16 */
	struct list_head           descs;                /*    16    16 */
	const u8  *                version_string;       /*    32     8 */
	u8                         version_type;         /*    40     1 */
	u8                         version_len;          /*    41     1 */
	u16                        package_data_len;     /*    42     2 */
	u32                        device_update_flags;  /*    44     4 */
	const u8  *                package_data;         /*    48     8 */
	long unsigned int *        component_bitmap;     /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u16                        component_bitmap_len; /*    64     2 */

	/* size: 72, cachelines: 2, members: 10 */
	/* padding: 6 */
	/* last cacheline: 8 bytes */
};
struct pldmfw_desc_tlv {
	struct list_head           entry;                /*     0    16 */
	const u8  *                data;                 /*    16     8 */
	u16                        type;                 /*    24     2 */
	u16                        size;                 /*    26     2 */

	/* size: 32, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct pldmfw_component {
	struct list_head           entry;                /*     0    16 */
	u16                        classification;       /*    16     2 */
	u16                        identifier;           /*    18     2 */
	u16                        options;              /*    20     2 */
	u16                        activation_method;    /*    22     2 */
	u32                        comparison_stamp;     /*    24     4 */
	u32                        component_size;       /*    28     4 */
	const u8  *                component_data;       /*    32     8 */
	const u8  *                version_string;       /*    40     8 */
	u8                         version_type;         /*    48     1 */
	u8                         version_len;          /*    49     1 */
	u8                         index;                /*    50     1 */

	/* size: 56, cachelines: 1, members: 12 */
	/* padding: 5 */
	/* last cacheline: 56 bytes */
};
struct pldmfw {
	const struct pldmfw_ops  * ops;                  /*     0     8 */
	struct device *            dev;                  /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct pldmfw_ops {
	bool                       (*match_record)(struct pldmfw *, struct pldmfw_record *); /*     0     8 */
	int                        (*send_package_data)(struct pldmfw *, const u8  *, u16); /*     8     8 */
	int                        (*send_component_table)(struct pldmfw *, struct pldmfw_component *, u8); /*    16     8 */
	int                        (*flash_component)(struct pldmfw *, struct pldmfw_component *); /*    24     8 */
	int                        (*finalize_update)(struct pldmfw *); /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct __pldm_timestamp {
	u8                         b[13];                /*     0    13 */

	/* size: 13, cachelines: 1, members: 1 */
	/* last cacheline: 13 bytes */
} __attribute__((__aligned__(1)));
struct __pldm_header {
	uuid_t                     id;                   /*     0    16 */
	u8                         revision;             /*    16     1 */
	__le16                     size;                 /*    17     2 */
	struct __pldm_timestamp    release_date __attribute__((__aligned__(1))); /*    19    13 */
	__le16                     component_bitmap_len; /*    32     2 */
	u8                         version_type;         /*    34     1 */
	u8                         version_len;          /*    35     1 */
	u8                         version_string[];     /*    36     0 */

	/* size: 36, cachelines: 1, members: 8 */
	/* forced alignments: 1 */
	/* last cacheline: 36 bytes */
} __attribute__((__packed__));
struct __pldmfw_record_info {
	__le16                     record_len;           /*     0     2 */
	u8                         descriptor_count;     /*     2     1 */
	__le32                     device_update_flags;  /*     3     4 */
	u8                         version_type;         /*     7     1 */
	u8                         version_len;          /*     8     1 */
	__le16                     package_data_len;     /*     9     2 */
	u8                         variable_record_data[]; /*    11     0 */

	/* size: 11, cachelines: 1, members: 7 */
	/* last cacheline: 11 bytes */
} __attribute__((__packed__));
struct __pldmfw_desc_tlv {
	__le16                     type;                 /*     0     2 */
	__le16                     size;                 /*     2     2 */
	u8                         data[];               /*     4     0 */

	/* size: 4, cachelines: 1, members: 3 */
	/* last cacheline: 4 bytes */
} __attribute__((__aligned__(2)));
struct __pldmfw_record_area {
	u8                         record_count;         /*     0     1 */
	u8                         records[];            /*     1     0 */

	/* size: 1, cachelines: 1, members: 2 */
	/* last cacheline: 1 bytes */
} __attribute__((__aligned__(1)));
struct __pldmfw_component_info {
	__le16                     classification;       /*     0     2 */
	__le16                     identifier;           /*     2     2 */
	__le32                     comparison_stamp;     /*     4     4 */
	__le16                     options;              /*     8     2 */
	__le16                     activation_method;    /*    10     2 */
	__le32                     location_offset;      /*    12     4 */
	__le32                     size;                 /*    16     4 */
	u8                         version_type;         /*    20     1 */
	u8                         version_len;          /*    21     1 */
	u8                         version_string[];     /*    22     0 */

	/* size: 22, cachelines: 1, members: 10 */
	/* last cacheline: 22 bytes */
} __attribute__((__packed__));
struct __pldmfw_component_area {
	__le16                     component_image_count; /*     0     2 */
	u8                         components[];         /*     2     0 */

	/* size: 2, cachelines: 1, members: 2 */
	/* last cacheline: 2 bytes */
} __attribute__((__aligned__(2)));
struct pldmfw_priv {
	struct pldmfw *            context;              /*     0     8 */
	const struct firmware  *   fw;                   /*     8     8 */
	size_t                     offset;               /*    16     8 */
	struct list_head           records;              /*    24    16 */
	struct list_head           components;           /*    40    16 */
	const struct __pldm_header  * header;            /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u16                        total_header_size;    /*    64     2 */
	u16                        component_bitmap_len; /*    66     2 */
	u16                        bitmap_size;          /*    68     2 */
	u16                        component_count;      /*    70     2 */
	const u8  *                component_start;      /*    72     8 */
	const u8  *                record_start;         /*    80     8 */
	u8                         record_count;         /*    88     1 */

	/* XXX 3 bytes hole, try to pack */

	u32                        header_crc;           /*    92     4 */
	struct pldmfw_record *     matching_record;      /*    96     8 */

	/* size: 104, cachelines: 2, members: 15 */
	/* sum members: 101, holes: 1, sum holes: 3 */
	/* last cacheline: 40 bytes */
};
struct pldm_pci_record_id {
	int                        vendor;               /*     0     4 */
	int                        device;               /*     4     4 */
	int                        subsystem_vendor;     /*     8     4 */
	int                        subsystem_device;     /*    12     4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct msr {
	union {
		struct {
			u32        l;                    /*     0     4 */
			u32        h;                    /*     4     4 */
		};                                       /*     0     8 */
		u64                q;                    /*     0     8 */
	};                                               /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct msr_info {
	u32                        msr_no;               /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct msr                 reg;                  /*     8     8 */
	struct msr *               msrs;                 /*    16     8 */
	int                        err;                  /*    24     4 */

	/* size: 32, cachelines: 1, members: 4 */
	/* sum members: 24, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct msr_regs_info {
	u32 *                      regs;                 /*     0     8 */
	int                        err;                  /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct msr_info_completion {
	struct msr_info            msr;                  /*     0    32 */

	/* XXX last struct has 4 bytes of padding */

	struct completion          done;                 /*    32    32 */

	/* size: 64, cachelines: 1, members: 2 */
	/* paddings: 1, sum paddings: 4 */
};
struct trace_event_raw_msr_trace_class {
	struct trace_entry         ent;                  /*     0     8 */
	unsigned int               msr;                  /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        val;                  /*    16     8 */
	int                        failed;               /*    24     4 */
	char                       __data[];             /*    28     0 */

	/* size: 32, cachelines: 1, members: 5 */
	/* sum members: 24, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct trace_event_data_offsets_msr_trace_class {

	/* size: 0, cachelines: 0, members: 0 */
};
struct compress_format {
	unsigned char              magic[2];             /*     0     2 */

	/* XXX 6 bytes hole, try to pack */

	const char  *              name;                 /*     8     8 */
	decompress_fn              decompressor;         /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 18, holes: 1, sum holes: 6 */
	/* last cacheline: 24 bytes */
};
struct group_data {
	int                        limit[21];            /*     0    84 */
	/* --- cacheline 1 boundary (64 bytes) was 20 bytes ago --- */
	int                        base[20];             /*    84    80 */
	/* --- cacheline 2 boundary (128 bytes) was 36 bytes ago --- */
	int                        permute[258];         /*   164  1032 */
	/* --- cacheline 18 boundary (1152 bytes) was 44 bytes ago --- */
	int                        minLen;               /*  1196     4 */
	int                        maxLen;               /*  1200     4 */

	/* size: 1204, cachelines: 19, members: 5 */
	/* last cacheline: 52 bytes */
};
struct bunzip_data {
	int                        writeCopies;          /*     0     4 */
	int                        writePos;             /*     4     4 */
	int                        writeRunCountdown;    /*     8     4 */
	int                        writeCount;           /*    12     4 */
	int                        writeCurrent;         /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	long int                   (*fill)(void *, long unsigned int); /*    24     8 */
	long int                   inbufCount;           /*    32     8 */
	long int                   inbufPos;             /*    40     8 */
	unsigned char *            inbuf;                /*    48     8 */
	unsigned int               inbufBitCount;        /*    56     4 */
	unsigned int               inbufBits;            /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	unsigned int               crc32Table[256];      /*    64  1024 */
	/* --- cacheline 17 boundary (1088 bytes) --- */
	unsigned int               headerCRC;            /*  1088     4 */
	unsigned int               totalCRC;             /*  1092     4 */
	unsigned int               writeCRC;             /*  1096     4 */

	/* XXX 4 bytes hole, try to pack */

	unsigned int *             dbuf;                 /*  1104     8 */
	unsigned int               dbufSize;             /*  1112     4 */
	unsigned char              selectors[32768];     /*  1116 32768 */
	/* --- cacheline 529 boundary (33856 bytes) was 28 bytes ago --- */
	struct group_data          groups[6];            /* 33884  7224 */
	/* --- cacheline 642 boundary (41088 bytes) was 20 bytes ago --- */
	int                        io_error;             /* 41108     4 */
	int                        byteCount[256];       /* 41112  1024 */
	/* --- cacheline 658 boundary (42112 bytes) was 24 bytes ago --- */
	unsigned char              symToByte[256];       /* 42136   256 */
	/* --- cacheline 662 boundary (42368 bytes) was 24 bytes ago --- */
	unsigned char              mtfSymbol[256];       /* 42392   256 */

	/* size: 42648, cachelines: 667, members: 23 */
	/* sum members: 42640, holes: 2, sum holes: 8 */
	/* last cacheline: 24 bytes */
};
struct rc {
	long int                   (*fill)(void *, long unsigned int); /*     0     8 */
	uint8_t *                  ptr;                  /*     8     8 */
	uint8_t *                  buffer;               /*    16     8 */
	uint8_t *                  buffer_end;           /*    24     8 */
	long int                   buffer_size;          /*    32     8 */
	uint32_t                   code;                 /*    40     4 */
	uint32_t                   range;                /*    44     4 */
	uint32_t                   bound;                /*    48     4 */

	/* XXX 4 bytes hole, try to pack */

	void                       (*error)(char *);     /*    56     8 */

	/* size: 64, cachelines: 1, members: 9 */
	/* sum members: 60, holes: 1, sum holes: 4 */
};
struct lzma_header {
	uint8_t                    pos;                  /*     0     1 */
	uint32_t                   dict_size;            /*     1     4 */
	uint64_t                   dst_size;             /*     5     8 */

	/* size: 13, cachelines: 1, members: 3 */
	/* last cacheline: 13 bytes */
} __attribute__((__packed__));
struct writer {
	uint8_t *                  buffer;               /*     0     8 */
	uint8_t                    previous_byte;        /*     8     1 */

	/* XXX 7 bytes hole, try to pack */

	size_t                     buffer_pos;           /*    16     8 */
	int                        bufsize;              /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	size_t                     global_pos;           /*    32     8 */
	long int                   (*flush)(void *, long unsigned int); /*    40     8 */
	struct lzma_header *       header;               /*    48     8 */

	/* size: 56, cachelines: 1, members: 7 */
	/* sum members: 45, holes: 2, sum holes: 11 */
	/* last cacheline: 56 bytes */
};
struct cstate {
	int                        state;                /*     0     4 */
	uint32_t                   rep0;                 /*     4     4 */
	uint32_t                   rep1;                 /*     8     4 */
	uint32_t                   rep2;                 /*    12     4 */
	uint32_t                   rep3;                 /*    16     4 */

	/* size: 20, cachelines: 1, members: 5 */
	/* last cacheline: 20 bytes */
};
struct fprop_local_single {
	long unsigned int          events;               /*     0     8 */
	unsigned int               period;               /*     8     4 */
	raw_spinlock_t             lock;                 /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct ida_bitmap {
	long unsigned int          bitmap[16];           /*     0   128 */

	/* size: 128, cachelines: 2, members: 1 */
};
struct klist_waiter {
	struct list_head           list;                 /*     0    16 */
	struct klist_node *        node;                 /*    16     8 */
	struct task_struct *       process;              /*    24     8 */
	int                        woken;                /*    32     4 */

	/* size: 40, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct uevent_sock {
	struct list_head           list;                 /*     0    16 */
	struct sock *              sk;                   /*    16     8 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct logic_pio_hwaddr {
	struct list_head           list;                 /*     0    16 */
	struct fwnode_handle *     fwnode;               /*    16     8 */
	resource_size_t            hw_start;             /*    24     8 */
	resource_size_t            io_start;             /*    32     8 */
	resource_size_t            size;                 /*    40     8 */
	long unsigned int          flags;                /*    48     8 */
	void *                     hostdata;             /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	const struct logic_pio_host_ops  * ops;          /*    64     8 */

	/* size: 72, cachelines: 2, members: 8 */
	/* last cacheline: 8 bytes */
};
struct logic_pio_host_ops {
	u32                        (*in)(void *, long unsigned int, size_t); /*     0     8 */
	void                       (*out)(void *, long unsigned int, u32, size_t); /*     8     8 */
	u32                        (*ins)(void *, long unsigned int, void *, size_t, unsigned int); /*    16     8 */
	void                       (*outs)(void *, long unsigned int, const void  *, size_t, unsigned int); /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct printf_spec {
	unsigned int               type:8;               /*     0: 0  4 */
	int                        field_width:24;       /*     0: 8  4 */
	unsigned int               flags:8;              /*     4: 0  4 */
	unsigned int               base:8;               /*     4: 8  4 */
	int                        precision:16;         /*     4:16  4 */

	/* size: 8, cachelines: 1, members: 5 */
	/* last cacheline: 8 bytes */
};
struct page_flags_fields {
	int                        width;                /*     0     4 */
	int                        shift;                /*     4     4 */
	int                        mask;                 /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	const struct printf_spec  * spec;                /*    16     8 */
	const char  *              name;                 /*    24     8 */

	/* size: 32, cachelines: 1, members: 5 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct minmax_sample {
	u32                        t;                    /*     0     4 */
	u32                        v;                    /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct minmax {
	struct minmax_sample       s[3];                 /*     0    24 */

	/* size: 24, cachelines: 1, members: 1 */
	/* last cacheline: 24 bytes */
};
struct dev_pin_info {
	struct pinctrl *           p;                    /*     0     8 */
	struct pinctrl_state *     default_state;        /*     8     8 */
	struct pinctrl_state *     init_state;           /*    16     8 */
	struct pinctrl_state *     sleep_state;          /*    24     8 */
	struct pinctrl_state *     idle_state;           /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct pinctrl {
	struct list_head           node;                 /*     0    16 */
	struct device *            dev;                  /*    16     8 */
	struct list_head           states;               /*    24    16 */
	struct pinctrl_state *     state;                /*    40     8 */
	struct list_head           dt_maps;              /*    48    16 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct kref                users;                /*    64     4 */

	/* size: 72, cachelines: 2, members: 6 */
	/* padding: 4 */
	/* last cacheline: 8 bytes */
};
struct pinctrl_state {
	struct list_head           node;                 /*     0    16 */
	const char  *              name;                 /*    16     8 */
	struct list_head           settings;             /*    24    16 */

	/* size: 40, cachelines: 1, members: 3 */
	/* last cacheline: 40 bytes */
};
struct pinctrl_pin_desc {
	unsigned int               number;               /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	const char  *              name;                 /*     8     8 */
	void *                     drv_data;             /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct pinctrl_gpio_range {
	struct list_head           node;                 /*     0    16 */
	const char  *              name;                 /*    16     8 */
	unsigned int               id;                   /*    24     4 */
	unsigned int               base;                 /*    28     4 */
	unsigned int               pin_base;             /*    32     4 */

	/* XXX 4 bytes hole, try to pack */

	const unsigned int  *      pins;                 /*    40     8 */
	unsigned int               npins;                /*    48     4 */

	/* XXX 4 bytes hole, try to pack */

	struct gpio_chip *         gc;                   /*    56     8 */

	/* size: 64, cachelines: 1, members: 8 */
	/* sum members: 56, holes: 2, sum holes: 8 */
};
struct gpio_chip {
	const char  *              label;                /*     0     8 */
	struct gpio_device *       gpiodev;              /*     8     8 */
	struct device *            parent;               /*    16     8 */
	struct module *            owner;                /*    24     8 */
	int                        (*request)(struct gpio_chip *, unsigned int); /*    32     8 */
	void                       (*free)(struct gpio_chip *, unsigned int); /*    40     8 */
	int                        (*get_direction)(struct gpio_chip *, unsigned int); /*    48     8 */
	int                        (*direction_input)(struct gpio_chip *, unsigned int); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*direction_output)(struct gpio_chip *, unsigned int, int); /*    64     8 */
	int                        (*get)(struct gpio_chip *, unsigned int); /*    72     8 */
	int                        (*get_multiple)(struct gpio_chip *, long unsigned int *, long unsigned int *); /*    80     8 */
	void                       (*set)(struct gpio_chip *, unsigned int, int); /*    88     8 */
	void                       (*set_multiple)(struct gpio_chip *, long unsigned int *, long unsigned int *); /*    96     8 */
	int                        (*set_config)(struct gpio_chip *, unsigned int, long unsigned int); /*   104     8 */
	int                        (*to_irq)(struct gpio_chip *, unsigned int); /*   112     8 */
	void                       (*dbg_show)(struct seq_file *, struct gpio_chip *); /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	int                        (*init_valid_mask)(struct gpio_chip *, long unsigned int *, unsigned int); /*   128     8 */
	int                        (*add_pin_ranges)(struct gpio_chip *); /*   136     8 */
	int                        base;                 /*   144     4 */
	u16                        ngpio;                /*   148     2 */

	/* XXX 2 bytes hole, try to pack */

	const char  * const *      names;                /*   152     8 */
	bool                       can_sleep;            /*   160     1 */

	/* XXX 7 bytes hole, try to pack */

	long unsigned int          (*read_reg)(void *);  /*   168     8 */
	void                       (*write_reg)(void *, long unsigned int); /*   176     8 */
	bool                       be_bits;              /*   184     1 */

	/* XXX 7 bytes hole, try to pack */

	/* --- cacheline 3 boundary (192 bytes) --- */
	void *                     reg_dat;              /*   192     8 */
	void *                     reg_set;              /*   200     8 */
	void *                     reg_clr;              /*   208     8 */
	void *                     reg_dir_out;          /*   216     8 */
	void *                     reg_dir_in;           /*   224     8 */
	bool                       bgpio_dir_unreadable; /*   232     1 */

	/* XXX 3 bytes hole, try to pack */

	int                        bgpio_bits;           /*   236     4 */
	spinlock_t                 bgpio_lock;           /*   240     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          bgpio_data;           /*   248     8 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	long unsigned int          bgpio_dir;            /*   256     8 */
	struct gpio_irq_chip       irq;                  /*   264   296 */
	/* --- cacheline 8 boundary (512 bytes) was 48 bytes ago --- */
	long unsigned int *        valid_mask;           /*   560     8 */

	/* size: 568, cachelines: 9, members: 37 */
	/* sum members: 545, holes: 5, sum holes: 23 */
	/* last cacheline: 56 bytes */
};
struct pinctrl_ops {
	int                        (*get_groups_count)(struct pinctrl_dev *); /*     0     8 */
	const char  *              (*get_group_name)(struct pinctrl_dev *, unsigned int); /*     8     8 */
	int                        (*get_group_pins)(struct pinctrl_dev *, unsigned int, const unsigned int  * *, unsigned int *); /*    16     8 */
	void                       (*pin_dbg_show)(struct pinctrl_dev *, struct seq_file *, unsigned int); /*    24     8 */
	int                        (*dt_node_to_map)(struct pinctrl_dev *, struct device_node *, struct pinctrl_map * *, unsigned int *); /*    32     8 */
	void                       (*dt_free_map)(struct pinctrl_dev *, struct pinctrl_map *, unsigned int); /*    40     8 */

	/* size: 48, cachelines: 1, members: 6 */
	/* last cacheline: 48 bytes */
};
struct pinctrl_dev {
	struct list_head           node;                 /*     0    16 */
	struct pinctrl_desc *      desc;                 /*    16     8 */
	struct xarray              pin_desc_tree;        /*    24    16 */
	struct list_head           gpio_ranges;          /*    40    16 */
	struct device *            dev;                  /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct module *            owner;                /*    64     8 */
	void *                     driver_data;          /*    72     8 */
	struct pinctrl *           p;                    /*    80     8 */
	struct pinctrl_state *     hog_default;          /*    88     8 */
	struct pinctrl_state *     hog_sleep;            /*    96     8 */
	struct mutex               mutex;                /*   104    32 */
	/* --- cacheline 2 boundary (128 bytes) was 8 bytes ago --- */
	struct dentry *            device_root;          /*   136     8 */

	/* size: 144, cachelines: 3, members: 12 */
	/* last cacheline: 16 bytes */
};
struct pinctrl_map {
	const char  *              dev_name;             /*     0     8 */
	const char  *              name;                 /*     8     8 */
	enum pinctrl_map_type      type;                 /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	const char  *              ctrl_dev_name;        /*    24     8 */
	union {
		struct pinctrl_map_mux mux;              /*    32    16 */
		struct pinctrl_map_configs configs;      /*    32    24 */
	} data;                                          /*    32    24 */

	/* size: 56, cachelines: 1, members: 5 */
	/* sum members: 52, holes: 1, sum holes: 4 */
	/* last cacheline: 56 bytes */
};
struct pinctrl_desc {
	const char  *              name;                 /*     0     8 */
	const struct pinctrl_pin_desc  * pins;           /*     8     8 */
	unsigned int               npins;                /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	const struct pinctrl_ops  * pctlops;             /*    24     8 */
	const struct pinmux_ops  * pmxops;               /*    32     8 */
	const struct pinconf_ops  * confops;             /*    40     8 */
	struct module *            owner;                /*    48     8 */
	unsigned int               num_custom_params;    /*    56     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	const struct pinconf_generic_params  * custom_params; /*    64     8 */
	const struct pin_config_item  * custom_conf_items; /*    72     8 */
	bool                       link_consumers;       /*    80     1 */

	/* size: 88, cachelines: 2, members: 11 */
	/* sum members: 73, holes: 2, sum holes: 8 */
	/* padding: 7 */
	/* last cacheline: 24 bytes */
};
struct pinconf_ops {
	bool                       is_generic;           /*     0     1 */

	/* XXX 7 bytes hole, try to pack */

	int                        (*pin_config_get)(struct pinctrl_dev *, unsigned int, long unsigned int *); /*     8     8 */
	int                        (*pin_config_set)(struct pinctrl_dev *, unsigned int, long unsigned int *, unsigned int); /*    16     8 */
	int                        (*pin_config_group_get)(struct pinctrl_dev *, unsigned int, long unsigned int *); /*    24     8 */
	int                        (*pin_config_group_set)(struct pinctrl_dev *, unsigned int, long unsigned int *, unsigned int); /*    32     8 */
	void                       (*pin_config_dbg_show)(struct pinctrl_dev *, struct seq_file *, unsigned int); /*    40     8 */
	void                       (*pin_config_group_dbg_show)(struct pinctrl_dev *, struct seq_file *, unsigned int); /*    48     8 */
	void                       (*pin_config_config_dbg_show)(struct pinctrl_dev *, struct seq_file *, long unsigned int); /*    56     8 */

	/* size: 64, cachelines: 1, members: 8 */
	/* sum members: 57, holes: 1, sum holes: 7 */
};
struct pinconf_generic_params {
	const const char  *        property;             /*     0     8 */
	enum pin_config_param      param;                /*     8     4 */
	u32                        default_value;        /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct pin_config_item {
	const enum pin_config_param param;               /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	const const char  *        display;              /*     8     8 */
	const const char  *        format;               /*    16     8 */
	bool                       has_arg;              /*    24     1 */

	/* size: 32, cachelines: 1, members: 4 */
	/* sum members: 21, holes: 1, sum holes: 4 */
	/* padding: 7 */
	/* last cacheline: 32 bytes */
};
struct pinctrl_map_mux {
	const char  *              group;                /*     0     8 */
	const char  *              function;             /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct pinctrl_map_configs {
	const char  *              group_or_pin;         /*     0     8 */
	long unsigned int *        configs;              /*     8     8 */
	unsigned int               num_configs;          /*    16     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct gpio_irq_chip {
	struct irq_chip *          chip;                 /*     0     8 */
	struct irq_domain *        domain;               /*     8     8 */
	const struct irq_domain_ops  * domain_ops;       /*    16     8 */
	struct fwnode_handle *     fwnode;               /*    24     8 */
	struct irq_domain *        parent_domain;        /*    32     8 */
	int                        (*child_to_parent_hwirq)(struct gpio_chip *, unsigned int, unsigned int, unsigned int *, unsigned int *); /*    40     8 */
	void *                     (*populate_parent_alloc_arg)(struct gpio_chip *, unsigned int, unsigned int); /*    48     8 */
	unsigned int               (*child_offset_to_irq)(struct gpio_chip *, unsigned int); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct irq_domain_ops      child_irq_domain_ops; /*    64    80 */
	/* --- cacheline 2 boundary (128 bytes) was 16 bytes ago --- */
	irq_flow_handler_t         handler;              /*   144     8 */
	unsigned int               default_type;         /*   152     4 */

	/* XXX 4 bytes hole, try to pack */

	struct lock_class_key *    lock_key;             /*   160     8 */
	struct lock_class_key *    request_key;          /*   168     8 */
	irq_flow_handler_t         parent_handler;       /*   176     8 */
	void *                     parent_handler_data;  /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	unsigned int               num_parents;          /*   192     4 */

	/* XXX 4 bytes hole, try to pack */

	unsigned int *             parents;              /*   200     8 */
	unsigned int *             map;                  /*   208     8 */
	bool                       threaded;             /*   216     1 */

	/* XXX 7 bytes hole, try to pack */

	int                        (*init_hw)(struct gpio_chip *); /*   224     8 */
	void                       (*init_valid_mask)(struct gpio_chip *, long unsigned int *, unsigned int); /*   232     8 */
	bool                       initialized;          /*   240     1 */

	/* XXX 7 bytes hole, try to pack */

	long unsigned int *        valid_mask;           /*   248     8 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	unsigned int               first;                /*   256     4 */

	/* XXX 4 bytes hole, try to pack */

	void                       (*irq_enable)(struct irq_data *); /*   264     8 */
	void                       (*irq_disable)(struct irq_data *); /*   272     8 */
	void                       (*irq_unmask)(struct irq_data *); /*   280     8 */
	void                       (*irq_mask)(struct irq_data *); /*   288     8 */

	/* size: 296, cachelines: 5, members: 28 */
	/* sum members: 270, holes: 5, sum holes: 26 */
	/* last cacheline: 40 bytes */
};
struct gpio_device {
	int                        id;                   /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct device              dev __attribute__((__aligned__(8))); /*     8  1000 */
	/* --- cacheline 15 boundary (960 bytes) was 48 bytes ago --- */
	struct cdev                chrdev;               /*  1008   136 */
	/* --- cacheline 17 boundary (1088 bytes) was 56 bytes ago --- */
	struct device *            mockdev;              /*  1144     8 */
	/* --- cacheline 18 boundary (1152 bytes) --- */
	struct module *            owner;                /*  1152     8 */
	struct gpio_chip *         chip;                 /*  1160     8 */
	struct gpio_desc *         descs;                /*  1168     8 */
	int                        base;                 /*  1176     4 */
	u16                        ngpio;                /*  1180     2 */

	/* XXX 2 bytes hole, try to pack */

	const char  *              label;                /*  1184     8 */
	void *                     data;                 /*  1192     8 */
	struct list_head           list;                 /*  1200    16 */
	/* --- cacheline 19 boundary (1216 bytes) --- */
	struct blocking_notifier_head notifier;          /*  1216    48 */
	struct list_head           pin_ranges;           /*  1264    16 */

	/* size: 1280, cachelines: 20, members: 14 */
	/* sum members: 1274, holes: 2, sum holes: 6 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 4 */
} __attribute__((__aligned__(8)));
struct gpio_desc {
	struct gpio_device *       gdev;                 /*     0     8 */
	long unsigned int          flags;                /*     8     8 */
	const char  *              label;                /*    16     8 */
	const char  *              name;                 /*    24     8 */
	unsigned int               debounce_period_us;   /*    32     4 */

	/* size: 40, cachelines: 1, members: 5 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct pinctrl_setting_mux {
	unsigned int               group;                /*     0     4 */
	unsigned int               func;                 /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct pinctrl_setting_configs {
	unsigned int               group_or_pin;         /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int *        configs;              /*     8     8 */
	unsigned int               num_configs;          /*    16     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 16, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct pinctrl_setting {
	struct list_head           node;                 /*     0    16 */
	enum pinctrl_map_type      type;                 /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	struct pinctrl_dev *       pctldev;              /*    24     8 */
	const char  *              dev_name;             /*    32     8 */
	union {
		struct pinctrl_setting_mux mux;          /*    40     8 */
		struct pinctrl_setting_configs configs;  /*    40    24 */
	} data;                                          /*    40    24 */

	/* size: 64, cachelines: 1, members: 5 */
	/* sum members: 60, holes: 1, sum holes: 4 */
};
struct pin_desc {
	struct pinctrl_dev *       pctldev;              /*     0     8 */
	const char  *              name;                 /*     8     8 */
	bool                       dynamic_name;         /*    16     1 */

	/* XXX 7 bytes hole, try to pack */

	void *                     drv_data;             /*    24     8 */
	unsigned int               mux_usecount;         /*    32     4 */

	/* XXX 4 bytes hole, try to pack */

	const char  *              mux_owner;            /*    40     8 */
	const struct pinctrl_setting_mux  * mux_setting; /*    48     8 */
	const char  *              gpio_owner;           /*    56     8 */

	/* size: 64, cachelines: 1, members: 8 */
	/* sum members: 53, holes: 2, sum holes: 11 */
};
struct pinctrl_maps {
	struct list_head           node;                 /*     0    16 */
	const struct pinctrl_map  * maps;                /*    16     8 */
	unsigned int               num_maps;             /*    24     4 */

	/* size: 32, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct pinmux_ops {
	int                        (*request)(struct pinctrl_dev *, unsigned int); /*     0     8 */
	int                        (*free)(struct pinctrl_dev *, unsigned int); /*     8     8 */
	int                        (*get_functions_count)(struct pinctrl_dev *); /*    16     8 */
	const char  *              (*get_function_name)(struct pinctrl_dev *, unsigned int); /*    24     8 */
	int                        (*get_function_groups)(struct pinctrl_dev *, unsigned int, const char  * const * *, unsigned int *); /*    32     8 */
	int                        (*set_mux)(struct pinctrl_dev *, unsigned int, unsigned int); /*    40     8 */
	int                        (*gpio_request_enable)(struct pinctrl_dev *, struct pinctrl_gpio_range *, unsigned int); /*    48     8 */
	void                       (*gpio_disable_free)(struct pinctrl_dev *, struct pinctrl_gpio_range *, unsigned int); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*gpio_set_direction)(struct pinctrl_dev *, struct pinctrl_gpio_range *, unsigned int, bool); /*    64     8 */
	bool                       strict;               /*    72     1 */

	/* size: 80, cachelines: 2, members: 10 */
	/* padding: 7 */
	/* last cacheline: 16 bytes */
};
struct intel_pingroup {
	const char  *              name;                 /*     0     8 */
	const unsigned int  *      pins;                 /*     8     8 */
	size_t                     npins;                /*    16     8 */
	short unsigned int         mode;                 /*    24     2 */

	/* XXX 6 bytes hole, try to pack */

	const unsigned int  *      modes;                /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* sum members: 34, holes: 1, sum holes: 6 */
	/* last cacheline: 40 bytes */
};
struct intel_function {
	const char  *              name;                 /*     0     8 */
	const char  * const *      groups;               /*     8     8 */
	size_t                     ngroups;              /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct intel_padgroup {
	unsigned int               reg_num;              /*     0     4 */
	unsigned int               base;                 /*     4     4 */
	unsigned int               size;                 /*     8     4 */
	int                        gpio_base;            /*    12     4 */
	unsigned int               padown_num;           /*    16     4 */

	/* size: 20, cachelines: 1, members: 5 */
	/* last cacheline: 20 bytes */
};
struct intel_community {
	unsigned int               barno;                /*     0     4 */
	unsigned int               padown_offset;        /*     4     4 */
	unsigned int               padcfglock_offset;    /*     8     4 */
	unsigned int               hostown_offset;       /*    12     4 */
	unsigned int               is_offset;            /*    16     4 */
	unsigned int               ie_offset;            /*    20     4 */
	unsigned int               features;             /*    24     4 */
	unsigned int               pin_base;             /*    28     4 */
	size_t                     npins;                /*    32     8 */
	unsigned int               gpp_size;             /*    40     4 */
	unsigned int               gpp_num_padown_regs;  /*    44     4 */
	const struct intel_padgroup  * gpps;             /*    48     8 */
	size_t                     ngpps;                /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	const unsigned int  *      pad_map;              /*    64     8 */
	short unsigned int         nirqs;                /*    72     2 */
	short unsigned int         acpi_space_id;        /*    74     2 */

	/* XXX 4 bytes hole, try to pack */

	void *                     regs;                 /*    80     8 */
	void *                     pad_regs;             /*    88     8 */

	/* size: 96, cachelines: 2, members: 18 */
	/* sum members: 92, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct intel_pinctrl_soc_data {
	const char  *              uid;                  /*     0     8 */
	const struct pinctrl_pin_desc  * pins;           /*     8     8 */
	size_t                     npins;                /*    16     8 */
	const struct intel_pingroup  * groups;           /*    24     8 */
	size_t                     ngroups;              /*    32     8 */
	const struct intel_function  * functions;        /*    40     8 */
	size_t                     nfunctions;           /*    48     8 */
	const struct intel_community  * communities;     /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	size_t                     ncommunities;         /*    64     8 */

	/* size: 72, cachelines: 2, members: 9 */
	/* last cacheline: 8 bytes */
};
struct intel_pinctrl_context {
	struct intel_pad_context * pads;                 /*     0     8 */
	struct intel_community_context * communities;    /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct intel_pad_context {
	u32                        conf0;                /*     0     4 */
	u32                        val;                  /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct intel_pinctrl {
	struct device *            dev;                  /*     0     8 */
	raw_spinlock_t             lock;                 /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct pinctrl_desc        pctldesc;             /*    16    88 */

	/* XXX last struct has 7 bytes of padding */

	/* --- cacheline 1 boundary (64 bytes) was 40 bytes ago --- */
	struct pinctrl_dev *       pctldev;              /*   104     8 */
	struct gpio_chip           chip;                 /*   112   568 */
	/* --- cacheline 10 boundary (640 bytes) was 40 bytes ago --- */
	struct irq_chip            irqchip;              /*   680   296 */
	/* --- cacheline 15 boundary (960 bytes) was 16 bytes ago --- */
	const struct intel_pinctrl_soc_data  * soc;      /*   976     8 */
	struct intel_community *   communities;          /*   984     8 */
	size_t                     ncommunities;         /*   992     8 */
	struct intel_pinctrl_context context;            /*  1000    16 */
	int                        irq;                  /*  1016     4 */

	/* size: 1024, cachelines: 16, members: 11 */
	/* sum members: 1016, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* paddings: 1, sum paddings: 7 */
};
struct intel_pad_context {
	u32                        padcfg0;              /*     0     4 */
	u32                        padcfg1;              /*     4     4 */
	u32                        padcfg2;              /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct intel_community_context {
	u32 *                      intmask;              /*     0     8 */
	u32 *                      hostown;              /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct gpio_pin_range {
	struct list_head           node;                 /*     0    16 */
	struct pinctrl_dev *       pctldev;              /*    16     8 */
	struct pinctrl_gpio_range  range;                /*    24    64 */

	/* size: 88, cachelines: 2, members: 3 */
	/* last cacheline: 24 bytes */
};
struct gpio_descs {
	struct gpio_array *        info;                 /*     0     8 */
	unsigned int               ndescs;               /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct gpio_desc *         desc[];               /*    16     0 */

	/* size: 16, cachelines: 1, members: 3 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct gpio_array {
	struct gpio_desc * *       desc;                 /*     0     8 */
	unsigned int               size;                 /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct gpio_chip *         chip;                 /*    16     8 */
	long unsigned int *        get_mask;             /*    24     8 */
	long unsigned int *        set_mask;             /*    32     8 */
	long unsigned int          invert_mask[];        /*    40     0 */

	/* size: 40, cachelines: 1, members: 6 */
	/* sum members: 36, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct gpiod_lookup {
	const char  *              key;                  /*     0     8 */
	u16                        chip_hwnum;           /*     8     2 */

	/* XXX 6 bytes hole, try to pack */

	const char  *              con_id;               /*    16     8 */
	unsigned int               idx;                  /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          flags;                /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* sum members: 30, holes: 2, sum holes: 10 */
	/* last cacheline: 40 bytes */
};
struct gpiod_lookup_table {
	struct list_head           list;                 /*     0    16 */
	const char  *              dev_id;               /*    16     8 */
	struct gpiod_lookup        table[];              /*    24     0 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct gpiod_hog {
	struct list_head           list;                 /*     0    16 */
	const char  *              chip_label;           /*    16     8 */
	u16                        chip_hwnum;           /*    24     2 */

	/* XXX 6 bytes hole, try to pack */

	const char  *              line_name;            /*    32     8 */
	long unsigned int          lflags;               /*    40     8 */
	int                        dflags;               /*    48     4 */

	/* size: 56, cachelines: 1, members: 6 */
	/* sum members: 46, holes: 1, sum holes: 6 */
	/* padding: 4 */
	/* last cacheline: 56 bytes */
};
struct acpi_gpio_info {
	struct acpi_device *       adev;                 /*     0     8 */
	enum gpiod_flags           flags;                /*     8     4 */
	bool                       gpioint;              /*    12     1 */

	/* XXX 3 bytes hole, try to pack */

	int                        pin_config;           /*    16     4 */
	int                        polarity;             /*    20     4 */
	int                        triggering;           /*    24     4 */
	unsigned int               quirks;               /*    28     4 */

	/* size: 32, cachelines: 1, members: 7 */
	/* sum members: 29, holes: 1, sum holes: 3 */
	/* last cacheline: 32 bytes */
};
struct trace_event_raw_gpio_direction {
	struct trace_entry         ent;                  /*     0     8 */
	unsigned int               gpio;                 /*     8     4 */
	int                        in;                   /*    12     4 */
	int                        err;                  /*    16     4 */
	char                       __data[];             /*    20     0 */

	/* size: 20, cachelines: 1, members: 5 */
	/* last cacheline: 20 bytes */
};
struct trace_event_raw_gpio_value {
	struct trace_entry         ent;                  /*     0     8 */
	unsigned int               gpio;                 /*     8     4 */
	int                        get;                  /*    12     4 */
	int                        value;                /*    16     4 */
	char                       __data[];             /*    20     0 */

	/* size: 20, cachelines: 1, members: 5 */
	/* last cacheline: 20 bytes */
};
struct trace_event_data_offsets_gpio_direction {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_gpio_value {

	/* size: 0, cachelines: 0, members: 0 */
};
struct gpio {
	unsigned int               gpio;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          flags;                /*     8     8 */
	const char  *              label;                /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct gpiochip_info {
	char                       name[32];             /*     0    32 */
	char                       label[32];            /*    32    32 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	__u32                      lines;                /*    64     4 */

	/* size: 68, cachelines: 2, members: 3 */
	/* last cacheline: 4 bytes */
};
struct gpio_v2_line_values {
	__u64                      bits __attribute__((__aligned__(8))); /*     0     8 */
	__u64                      mask __attribute__((__aligned__(8))); /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* forced alignments: 2 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct gpio_v2_line_attribute {
	__u32                      id;                   /*     0     4 */
	__u32                      padding;              /*     4     4 */
	union {
		__u64              flags __attribute__((__aligned__(8))); /*     8     8 */
		__u64              values __attribute__((__aligned__(8))); /*     8     8 */
		__u32              debounce_period_us;   /*     8     4 */
	} __attribute__((__aligned__(8)));               /*     8     8 */

	/* size: 16, cachelines: 1, members: 3 */
	/* forced alignments: 1 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct gpio_v2_line_config_attribute {
	struct gpio_v2_line_attribute attr __attribute__((__aligned__(8))); /*     0    16 */
	__u64                      mask __attribute__((__aligned__(8))); /*    16     8 */

	/* size: 24, cachelines: 1, members: 2 */
	/* forced alignments: 2 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct gpio_v2_line_config {
	__u64                      flags __attribute__((__aligned__(8))); /*     0     8 */
	__u32                      num_attrs;            /*     8     4 */
	__u32                      padding[5];           /*    12    20 */
	struct gpio_v2_line_config_attribute attrs[10] __attribute__((__aligned__(8))); /*    32   240 */

	/* size: 272, cachelines: 5, members: 4 */
	/* forced alignments: 2 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct gpio_v2_line_request {
	__u32                      offsets[64];          /*     0   256 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	char                       consumer[32];         /*   256    32 */
	struct gpio_v2_line_config config __attribute__((__aligned__(8))); /*   288   272 */
	/* --- cacheline 8 boundary (512 bytes) was 48 bytes ago --- */
	__u32                      num_lines;            /*   560     4 */
	__u32                      event_buffer_size;    /*   564     4 */
	__u32                      padding[5];           /*   568    20 */
	/* --- cacheline 9 boundary (576 bytes) was 12 bytes ago --- */
	__s32                      fd;                   /*   588     4 */

	/* size: 592, cachelines: 10, members: 7 */
	/* forced alignments: 1 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct gpio_v2_line_info {
	char                       name[32];             /*     0    32 */
	char                       consumer[32];         /*    32    32 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	__u32                      offset;               /*    64     4 */
	__u32                      num_attrs;            /*    68     4 */
	__u64                      flags __attribute__((__aligned__(8))); /*    72     8 */
	struct gpio_v2_line_attribute attrs[10] __attribute__((__aligned__(8))); /*    80   160 */
	/* --- cacheline 3 boundary (192 bytes) was 48 bytes ago --- */
	__u32                      padding[4];           /*   240    16 */

	/* size: 256, cachelines: 4, members: 7 */
	/* forced alignments: 2 */
} __attribute__((__aligned__(8)));
struct gpio_v2_line_info_changed {
	struct gpio_v2_line_info   info __attribute__((__aligned__(8))); /*     0   256 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	__u64                      timestamp_ns __attribute__((__aligned__(8))); /*   256     8 */
	__u32                      event_type;           /*   264     4 */
	__u32                      padding[5];           /*   268    20 */

	/* size: 288, cachelines: 5, members: 4 */
	/* forced alignments: 2 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct gpio_v2_line_event {
	__u64                      timestamp_ns __attribute__((__aligned__(8))); /*     0     8 */
	__u32                      id;                   /*     8     4 */
	__u32                      offset;               /*    12     4 */
	__u32                      seqno;                /*    16     4 */
	__u32                      line_seqno;           /*    20     4 */
	__u32                      padding[6];           /*    24    24 */

	/* size: 48, cachelines: 1, members: 6 */
	/* forced alignments: 1 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct gpioline_info {
	__u32                      line_offset;          /*     0     4 */
	__u32                      flags;                /*     4     4 */
	char                       name[32];             /*     8    32 */
	char                       consumer[32];         /*    40    32 */

	/* size: 72, cachelines: 2, members: 4 */
	/* last cacheline: 8 bytes */
};
struct gpioline_info_changed {
	struct gpioline_info       info;                 /*     0    72 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	__u64                      timestamp;            /*    72     8 */
	__u32                      event_type;           /*    80     4 */
	__u32                      padding[5];           /*    84    20 */

	/* size: 104, cachelines: 2, members: 4 */
	/* last cacheline: 40 bytes */
};
struct gpiohandle_request {
	__u32                      lineoffsets[64];      /*     0   256 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	__u32                      flags;                /*   256     4 */
	__u8                       default_values[64];   /*   260    64 */
	/* --- cacheline 5 boundary (320 bytes) was 4 bytes ago --- */
	char                       consumer_label[32];   /*   324    32 */
	__u32                      lines;                /*   356     4 */
	int                        fd;                   /*   360     4 */

	/* size: 364, cachelines: 6, members: 6 */
	/* last cacheline: 44 bytes */
};
struct gpiohandle_config {
	__u32                      flags;                /*     0     4 */
	__u8                       default_values[64];   /*     4    64 */
	/* --- cacheline 1 boundary (64 bytes) was 4 bytes ago --- */
	__u32                      padding[4];           /*    68    16 */

	/* size: 84, cachelines: 2, members: 3 */
	/* last cacheline: 20 bytes */
};
struct gpiohandle_data {
	__u8                       values[64];           /*     0    64 */

	/* size: 64, cachelines: 1, members: 1 */
};
struct gpioevent_request {
	__u32                      lineoffset;           /*     0     4 */
	__u32                      handleflags;          /*     4     4 */
	__u32                      eventflags;           /*     8     4 */
	char                       consumer_label[32];   /*    12    32 */
	int                        fd;                   /*    44     4 */

	/* size: 48, cachelines: 1, members: 5 */
	/* last cacheline: 48 bytes */
};
struct gpioevent_data {
	__u64                      timestamp;            /*     0     8 */
	__u32                      id;                   /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct linehandle_state {
	struct gpio_device *       gdev;                 /*     0     8 */
	const char  *              label;                /*     8     8 */
	struct gpio_desc *         descs[64];            /*    16   512 */
	/* --- cacheline 8 boundary (512 bytes) was 16 bytes ago --- */
	u32                        num_descs;            /*   528     4 */

	/* size: 536, cachelines: 9, members: 4 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct line {
	struct gpio_desc *         desc;                 /*     0     8 */
	struct linereq *           req;                  /*     8     8 */
	unsigned int               irq;                  /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        eflags;               /*    24     8 */
	u64                        timestamp_ns;         /*    32     8 */
	u32                        req_seqno;            /*    40     4 */
	u32                        line_seqno;           /*    44     4 */
	struct delayed_work        work;                 /*    48   192 */
	/* --- cacheline 3 boundary (192 bytes) was 48 bytes ago --- */
	unsigned int               sw_debounced;         /*   240     4 */
	unsigned int               level;                /*   244     4 */

	/* size: 248, cachelines: 4, members: 10 */
	/* sum members: 244, holes: 1, sum holes: 4 */
	/* last cacheline: 56 bytes */
};
struct linereq {
	struct gpio_device *       gdev;                 /*     0     8 */
	const char  *              label;                /*     8     8 */
	u32                        num_lines;            /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	wait_queue_head_t          wait;                 /*    24    24 */
	u32                        event_buffer_size;    /*    48     4 */

	/* XXX 4 bytes hole, try to pack */

	struct {
		union {
			struct __kfifo kfifo;            /*    56    24 */
			struct gpio_v2_line_event * type; /*    56     8 */
			const struct gpio_v2_line_event  * const_type; /*    56     8 */
			char *     rectype;              /*    56     8 */
			struct gpio_v2_line_event * ptr; /*    56     8 */
			const struct gpio_v2_line_event  * ptr_const; /*    56     8 */
		};                                       /*    56    24 */
		/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
		struct gpio_v2_line_event buf[] __attribute__((__aligned__(8))); /*    80     0 */
	} __attribute__((__aligned__(8))) events __attribute__((__aligned__(8)));        /*    56    24 */
	atomic_t                   seqno;                /*    80     4 */

	/* XXX 4 bytes hole, try to pack */

	struct mutex               config_mutex;         /*    88    32 */
	struct line                lines[];              /*   120     0 */

	/* size: 120, cachelines: 2, members: 9 */
	/* sum members: 108, holes: 3, sum holes: 12 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 4 */
	/* last cacheline: 56 bytes */
} __attribute__((__aligned__(8)));
struct lineevent_state {
	struct gpio_device *       gdev;                 /*     0     8 */
	const char  *              label;                /*     8     8 */
	struct gpio_desc *         desc;                 /*    16     8 */
	u32                        eflags;               /*    24     4 */
	int                        irq;                  /*    28     4 */
	wait_queue_head_t          wait;                 /*    32    24 */
	struct {
		union {
			struct __kfifo kfifo;            /*    56    24 */
			struct gpioevent_data * type;    /*    56     8 */
			const struct gpioevent_data  * const_type; /*    56     8 */
			char *     rectype;              /*    56     8 */
			struct gpioevent_data * ptr;     /*    56     8 */
			const struct gpioevent_data  * ptr_const; /*    56     8 */
		};                                       /*    56    24 */
		/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
		struct gpioevent_data buf[16];           /*    80   256 */
	} events;                                        /*    56   280 */
	/* --- cacheline 5 boundary (320 bytes) was 16 bytes ago --- */
	u64                        timestamp;            /*   336     8 */

	/* size: 344, cachelines: 6, members: 8 */
	/* last cacheline: 24 bytes */
};
struct gpio_chardev_data {
	struct gpio_device *       gdev;                 /*     0     8 */
	wait_queue_head_t          wait;                 /*     8    24 */
	struct {
		union {
			struct __kfifo kfifo;            /*    32    24 */
			struct gpio_v2_line_info_changed * type; /*    32     8 */
			const struct gpio_v2_line_info_changed  * const_type; /*    32     8 */
			char *     rectype;              /*    32     8 */
			struct gpio_v2_line_info_changed * ptr; /*    32     8 */
			const struct gpio_v2_line_info_changed  * ptr_const; /*    32     8 */
		};                                       /*    32    24 */
		struct gpio_v2_line_info_changed buf[32] __attribute__((__aligned__(8))); /*    56  9216 */
	} __attribute__((__aligned__(8))) events __attribute__((__aligned__(8)));        /*    32  9240 */
	/* --- cacheline 144 boundary (9216 bytes) was 56 bytes ago --- */
	struct notifier_block      lineinfo_changed_nb;  /*  9272    24 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 145 boundary (9280 bytes) was 16 bytes ago --- */
	long unsigned int *        watched_lines;        /*  9296     8 */
	atomic_t                   watch_abi_version;    /*  9304     4 */

	/* size: 9312, cachelines: 146, members: 6 */
	/* padding: 4 */
	/* paddings: 1, sum paddings: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct class_attribute {
	struct attribute           attr;                 /*     0    16 */

	/* XXX last struct has 6 bytes of padding */

	ssize_t                    (*show)(struct class *, struct class_attribute *, char *); /*    16     8 */
	ssize_t                    (*store)(struct class *, struct class_attribute *, const char  *, size_t); /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* paddings: 1, sum paddings: 6 */
	/* last cacheline: 32 bytes */
};
struct gpiod_data {
	struct gpio_desc *         desc;                 /*     0     8 */
	struct mutex               mutex;                /*     8    32 */
	struct kernfs_node *       value_kn;             /*    40     8 */
	int                        irq;                  /*    48     4 */
	unsigned char              irq_flags;            /*    52     1 */
	bool                       direction_can_change; /*    53     1 */

	/* size: 56, cachelines: 1, members: 6 */
	/* padding: 2 */
	/* last cacheline: 56 bytes */
};
struct acpi_connection_info {
	u8 *                       connection;           /*     0     8 */
	u16                        length;               /*     8     2 */
	u8                         access_length;        /*    10     1 */

	/* size: 16, cachelines: 1, members: 3 */
	/* padding: 5 */
	/* last cacheline: 16 bytes */
};
struct acpi_resource_irq {
	u8                         descriptor_length;    /*     0     1 */
	u8                         triggering;           /*     1     1 */
	u8                         polarity;             /*     2     1 */
	u8                         shareable;            /*     3     1 */
	u8                         wake_capable;         /*     4     1 */
	u8                         interrupt_count;      /*     5     1 */
	u8                         interrupts[1];        /*     6     1 */

	/* size: 7, cachelines: 1, members: 7 */
	/* last cacheline: 7 bytes */
};
struct acpi_resource_dma {
	u8                         type;                 /*     0     1 */
	u8                         bus_master;           /*     1     1 */
	u8                         transfer;             /*     2     1 */
	u8                         channel_count;        /*     3     1 */
	u8                         channels[1];          /*     4     1 */

	/* size: 5, cachelines: 1, members: 5 */
	/* last cacheline: 5 bytes */
};
struct acpi_resource_start_dependent {
	u8                         descriptor_length;    /*     0     1 */
	u8                         compatibility_priority; /*     1     1 */
	u8                         performance_robustness; /*     2     1 */

	/* size: 3, cachelines: 1, members: 3 */
	/* last cacheline: 3 bytes */
};
struct acpi_resource_io {
	u8                         io_decode;            /*     0     1 */
	u8                         alignment;            /*     1     1 */
	u8                         address_length;       /*     2     1 */
	u16                        minimum;              /*     3     2 */
	u16                        maximum;              /*     5     2 */

	/* size: 7, cachelines: 1, members: 5 */
	/* last cacheline: 7 bytes */
} __attribute__((__packed__));
struct acpi_resource_fixed_io {
	u16                        address;              /*     0     2 */
	u8                         address_length;       /*     2     1 */

	/* size: 3, cachelines: 1, members: 2 */
	/* last cacheline: 3 bytes */
} __attribute__((__packed__));
struct acpi_resource_fixed_dma {
	u16                        request_lines;        /*     0     2 */
	u16                        channels;             /*     2     2 */
	u8                         width;                /*     4     1 */

	/* size: 5, cachelines: 1, members: 3 */
	/* last cacheline: 5 bytes */
} __attribute__((__packed__));
struct acpi_resource_vendor {
	u16                        byte_length;          /*     0     2 */
	u8                         byte_data[1];         /*     2     1 */

	/* size: 3, cachelines: 1, members: 2 */
	/* last cacheline: 3 bytes */
} __attribute__((__packed__));
struct acpi_resource_vendor_typed {
	u16                        byte_length;          /*     0     2 */
	u8                         uuid_subtype;         /*     2     1 */
	u8                         uuid[16];             /*     3    16 */
	u8                         byte_data[1];         /*    19     1 */

	/* size: 20, cachelines: 1, members: 4 */
	/* last cacheline: 20 bytes */
};
struct acpi_resource_end_tag {
	u8                         checksum;             /*     0     1 */

	/* size: 1, cachelines: 1, members: 1 */
	/* last cacheline: 1 bytes */
};
struct acpi_resource_memory24 {
	u8                         write_protect;        /*     0     1 */
	u16                        minimum;              /*     1     2 */
	u16                        maximum;              /*     3     2 */
	u16                        alignment;            /*     5     2 */
	u16                        address_length;       /*     7     2 */

	/* size: 9, cachelines: 1, members: 5 */
	/* last cacheline: 9 bytes */
} __attribute__((__packed__));
struct acpi_resource_memory32 {
	u8                         write_protect;        /*     0     1 */
	u32                        minimum;              /*     1     4 */
	u32                        maximum;              /*     5     4 */
	u32                        alignment;            /*     9     4 */
	u32                        address_length;       /*    13     4 */

	/* size: 17, cachelines: 1, members: 5 */
	/* last cacheline: 17 bytes */
} __attribute__((__packed__));
struct acpi_resource_fixed_memory32 {
	u8                         write_protect;        /*     0     1 */
	u32                        address;              /*     1     4 */
	u32                        address_length;       /*     5     4 */

	/* size: 9, cachelines: 1, members: 3 */
	/* last cacheline: 9 bytes */
} __attribute__((__packed__));
struct acpi_memory_attribute {
	u8                         write_protect;        /*     0     1 */
	u8                         caching;              /*     1     1 */
	u8                         range_type;           /*     2     1 */
	u8                         translation;          /*     3     1 */

	/* size: 4, cachelines: 1, members: 4 */
	/* last cacheline: 4 bytes */
};
struct acpi_io_attribute {
	u8                         range_type;           /*     0     1 */
	u8                         translation;          /*     1     1 */
	u8                         translation_type;     /*     2     1 */
	u8                         reserved1;            /*     3     1 */

	/* size: 4, cachelines: 1, members: 4 */
	/* last cacheline: 4 bytes */
};
union acpi_resource_attribute {
	struct acpi_memory_attribute mem;              /*     0     4 */
	struct acpi_io_attribute   io;                 /*     0     4 */
	u8                         type_specific;      /*     0     1 */
};
struct acpi_resource_label {
	u16                        string_length;        /*     0     2 */
	char *                     string_ptr;           /*     2     8 */

	/* size: 10, cachelines: 1, members: 2 */
	/* last cacheline: 10 bytes */
} __attribute__((__packed__));
struct acpi_resource_source {
	u8                         index;                /*     0     1 */
	u16                        string_length;        /*     1     2 */
	char *                     string_ptr;           /*     3     8 */

	/* size: 11, cachelines: 1, members: 3 */
	/* last cacheline: 11 bytes */
} __attribute__((__packed__));
struct acpi_address16_attribute {
	u16                        granularity;          /*     0     2 */
	u16                        minimum;              /*     2     2 */
	u16                        maximum;              /*     4     2 */
	u16                        translation_offset;   /*     6     2 */
	u16                        address_length;       /*     8     2 */

	/* size: 10, cachelines: 1, members: 5 */
	/* last cacheline: 10 bytes */
};
struct acpi_address32_attribute {
	u32                        granularity;          /*     0     4 */
	u32                        minimum;              /*     4     4 */
	u32                        maximum;              /*     8     4 */
	u32                        translation_offset;   /*    12     4 */
	u32                        address_length;       /*    16     4 */

	/* size: 20, cachelines: 1, members: 5 */
	/* last cacheline: 20 bytes */
};
struct acpi_address64_attribute {
	u64                        granularity;          /*     0     8 */
	u64                        minimum;              /*     8     8 */
	u64                        maximum;              /*    16     8 */
	u64                        translation_offset;   /*    24     8 */
	u64                        address_length;       /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct acpi_resource_address {
	u8                         resource_type;        /*     0     1 */
	u8                         producer_consumer;    /*     1     1 */
	u8                         decode;               /*     2     1 */
	u8                         min_address_fixed;    /*     3     1 */
	u8                         max_address_fixed;    /*     4     1 */
	union acpi_resource_attribute info;              /*     5     4 */

	/* size: 9, cachelines: 1, members: 6 */
	/* last cacheline: 9 bytes */
};
struct acpi_resource_address16 {
	u8                         resource_type;        /*     0     1 */
	u8                         producer_consumer;    /*     1     1 */
	u8                         decode;               /*     2     1 */
	u8                         min_address_fixed;    /*     3     1 */
	u8                         max_address_fixed;    /*     4     1 */
	union acpi_resource_attribute info;              /*     5     4 */
	struct acpi_address16_attribute address;         /*     9    10 */
	struct acpi_resource_source resource_source;     /*    19    11 */

	/* size: 30, cachelines: 1, members: 8 */
	/* last cacheline: 30 bytes */
} __attribute__((__packed__));
struct acpi_resource_address32 {
	u8                         resource_type;        /*     0     1 */
	u8                         producer_consumer;    /*     1     1 */
	u8                         decode;               /*     2     1 */
	u8                         min_address_fixed;    /*     3     1 */
	u8                         max_address_fixed;    /*     4     1 */
	union acpi_resource_attribute info;              /*     5     4 */
	struct acpi_address32_attribute address;         /*     9    20 */
	struct acpi_resource_source resource_source;     /*    29    11 */

	/* size: 40, cachelines: 1, members: 8 */
	/* last cacheline: 40 bytes */
} __attribute__((__packed__));
struct acpi_resource_address64 {
	u8                         resource_type;        /*     0     1 */
	u8                         producer_consumer;    /*     1     1 */
	u8                         decode;               /*     2     1 */
	u8                         min_address_fixed;    /*     3     1 */
	u8                         max_address_fixed;    /*     4     1 */
	union acpi_resource_attribute info;              /*     5     4 */
	struct acpi_address64_attribute address;         /*     9    40 */
	struct acpi_resource_source resource_source;     /*    49    11 */

	/* size: 60, cachelines: 1, members: 8 */
	/* last cacheline: 60 bytes */
} __attribute__((__packed__));
struct acpi_resource_extended_address64 {
	u8                         resource_type;        /*     0     1 */
	u8                         producer_consumer;    /*     1     1 */
	u8                         decode;               /*     2     1 */
	u8                         min_address_fixed;    /*     3     1 */
	u8                         max_address_fixed;    /*     4     1 */
	union acpi_resource_attribute info;              /*     5     4 */
	u8                         revision_ID;          /*     9     1 */
	struct acpi_address64_attribute address;         /*    10    40 */
	u64                        type_specific;        /*    50     8 */

	/* size: 58, cachelines: 1, members: 9 */
	/* last cacheline: 58 bytes */
} __attribute__((__packed__));
struct acpi_resource_extended_irq {
	u8                         producer_consumer;    /*     0     1 */
	u8                         triggering;           /*     1     1 */
	u8                         polarity;             /*     2     1 */
	u8                         shareable;            /*     3     1 */
	u8                         wake_capable;         /*     4     1 */
	u8                         interrupt_count;      /*     5     1 */
	struct acpi_resource_source resource_source;     /*     6    11 */
	u32                        interrupts[1];        /*    17     4 */

	/* size: 21, cachelines: 1, members: 8 */
	/* last cacheline: 21 bytes */
} __attribute__((__packed__));
struct acpi_resource_generic_register {
	u8                         space_id;             /*     0     1 */
	u8                         bit_width;            /*     1     1 */
	u8                         bit_offset;           /*     2     1 */
	u8                         access_size;          /*     3     1 */
	u64                        address;              /*     4     8 */

	/* size: 12, cachelines: 1, members: 5 */
	/* last cacheline: 12 bytes */
} __attribute__((__packed__));
struct acpi_resource_gpio {
	u8                         revision_id;          /*     0     1 */
	u8                         connection_type;      /*     1     1 */
	u8                         producer_consumer;    /*     2     1 */
	u8                         pin_config;           /*     3     1 */
	u8                         shareable;            /*     4     1 */
	u8                         wake_capable;         /*     5     1 */
	u8                         io_restriction;       /*     6     1 */
	u8                         triggering;           /*     7     1 */
	u8                         polarity;             /*     8     1 */
	u16                        drive_strength;       /*     9     2 */
	u16                        debounce_timeout;     /*    11     2 */
	u16                        pin_table_length;     /*    13     2 */
	u16                        vendor_length;        /*    15     2 */
	struct acpi_resource_source resource_source;     /*    17    11 */
	u16 *                      pin_table;            /*    28     8 */
	u8 *                       vendor_data;          /*    36     8 */

	/* size: 44, cachelines: 1, members: 16 */
	/* last cacheline: 44 bytes */
} __attribute__((__packed__));
struct acpi_resource_common_serialbus {
	u8                         revision_id;          /*     0     1 */
	u8                         type;                 /*     1     1 */
	u8                         producer_consumer;    /*     2     1 */
	u8                         slave_mode;           /*     3     1 */
	u8                         connection_sharing;   /*     4     1 */
	u8                         type_revision_id;     /*     5     1 */
	u16                        type_data_length;     /*     6     2 */
	u16                        vendor_length;        /*     8     2 */
	struct acpi_resource_source resource_source;     /*    10    11 */
	u8 *                       vendor_data;          /*    21     8 */

	/* size: 29, cachelines: 1, members: 10 */
	/* last cacheline: 29 bytes */
} __attribute__((__packed__));
struct acpi_resource_i2c_serialbus {
	u8                         revision_id;          /*     0     1 */
	u8                         type;                 /*     1     1 */
	u8                         producer_consumer;    /*     2     1 */
	u8                         slave_mode;           /*     3     1 */
	u8                         connection_sharing;   /*     4     1 */
	u8                         type_revision_id;     /*     5     1 */
	u16                        type_data_length;     /*     6     2 */
	u16                        vendor_length;        /*     8     2 */
	struct acpi_resource_source resource_source;     /*    10    11 */
	u8 *                       vendor_data;          /*    21     8 */
	u8                         access_mode;          /*    29     1 */
	u16                        slave_address;        /*    30     2 */
	u32                        connection_speed;     /*    32     4 */

	/* size: 36, cachelines: 1, members: 13 */
	/* last cacheline: 36 bytes */
} __attribute__((__packed__));
struct acpi_resource_spi_serialbus {
	u8                         revision_id;          /*     0     1 */
	u8                         type;                 /*     1     1 */
	u8                         producer_consumer;    /*     2     1 */
	u8                         slave_mode;           /*     3     1 */
	u8                         connection_sharing;   /*     4     1 */
	u8                         type_revision_id;     /*     5     1 */
	u16                        type_data_length;     /*     6     2 */
	u16                        vendor_length;        /*     8     2 */
	struct acpi_resource_source resource_source;     /*    10    11 */
	u8 *                       vendor_data;          /*    21     8 */
	u8                         wire_mode;            /*    29     1 */
	u8                         device_polarity;      /*    30     1 */
	u8                         data_bit_length;      /*    31     1 */
	u8                         clock_phase;          /*    32     1 */
	u8                         clock_polarity;       /*    33     1 */
	u16                        device_selection;     /*    34     2 */
	u32                        connection_speed;     /*    36     4 */

	/* size: 40, cachelines: 1, members: 17 */
	/* last cacheline: 40 bytes */
} __attribute__((__packed__));
struct acpi_resource_uart_serialbus {
	u8                         revision_id;          /*     0     1 */
	u8                         type;                 /*     1     1 */
	u8                         producer_consumer;    /*     2     1 */
	u8                         slave_mode;           /*     3     1 */
	u8                         connection_sharing;   /*     4     1 */
	u8                         type_revision_id;     /*     5     1 */
	u16                        type_data_length;     /*     6     2 */
	u16                        vendor_length;        /*     8     2 */
	struct acpi_resource_source resource_source;     /*    10    11 */
	u8 *                       vendor_data;          /*    21     8 */
	u8                         endian;               /*    29     1 */
	u8                         data_bits;            /*    30     1 */
	u8                         stop_bits;            /*    31     1 */
	u8                         flow_control;         /*    32     1 */
	u8                         parity;               /*    33     1 */
	u8                         lines_enabled;        /*    34     1 */
	u16                        rx_fifo_size;         /*    35     2 */
	u16                        tx_fifo_size;         /*    37     2 */
	u32                        default_baud_rate;    /*    39     4 */

	/* size: 43, cachelines: 1, members: 19 */
	/* last cacheline: 43 bytes */
} __attribute__((__packed__));
struct acpi_resource_pin_function {
	u8                         revision_id;          /*     0     1 */
	u8                         pin_config;           /*     1     1 */
	u8                         shareable;            /*     2     1 */
	u16                        function_number;      /*     3     2 */
	u16                        pin_table_length;     /*     5     2 */
	u16                        vendor_length;        /*     7     2 */
	struct acpi_resource_source resource_source;     /*     9    11 */
	u16 *                      pin_table;            /*    20     8 */
	u8 *                       vendor_data;          /*    28     8 */

	/* size: 36, cachelines: 1, members: 9 */
	/* last cacheline: 36 bytes */
} __attribute__((__packed__));
struct acpi_resource_pin_config {
	u8                         revision_id;          /*     0     1 */
	u8                         producer_consumer;    /*     1     1 */
	u8                         shareable;            /*     2     1 */
	u8                         pin_config_type;      /*     3     1 */
	u32                        pin_config_value;     /*     4     4 */
	u16                        pin_table_length;     /*     8     2 */
	u16                        vendor_length;        /*    10     2 */
	struct acpi_resource_source resource_source;     /*    12    11 */
	u16 *                      pin_table;            /*    23     8 */
	u8 *                       vendor_data;          /*    31     8 */

	/* size: 39, cachelines: 1, members: 10 */
	/* last cacheline: 39 bytes */
} __attribute__((__packed__));
struct acpi_resource_pin_group {
	u8                         revision_id;          /*     0     1 */
	u8                         producer_consumer;    /*     1     1 */
	u16                        pin_table_length;     /*     2     2 */
	u16                        vendor_length;        /*     4     2 */
	u16 *                      pin_table;            /*     6     8 */
	struct acpi_resource_label resource_label;       /*    14    10 */
	u8 *                       vendor_data;          /*    24     8 */

	/* size: 32, cachelines: 1, members: 7 */
	/* last cacheline: 32 bytes */
} __attribute__((__packed__));
struct acpi_resource_pin_group_function {
	u8                         revision_id;          /*     0     1 */
	u8                         producer_consumer;    /*     1     1 */
	u8                         shareable;            /*     2     1 */
	u16                        function_number;      /*     3     2 */
	u16                        vendor_length;        /*     5     2 */
	struct acpi_resource_source resource_source;     /*     7    11 */
	struct acpi_resource_label resource_source_label; /*    18    10 */
	u8 *                       vendor_data;          /*    28     8 */

	/* size: 36, cachelines: 1, members: 8 */
	/* last cacheline: 36 bytes */
} __attribute__((__packed__));
struct acpi_resource_pin_group_config {
	u8                         revision_id;          /*     0     1 */
	u8                         producer_consumer;    /*     1     1 */
	u8                         shareable;            /*     2     1 */
	u8                         pin_config_type;      /*     3     1 */
	u32                        pin_config_value;     /*     4     4 */
	u16                        vendor_length;        /*     8     2 */
	struct acpi_resource_source resource_source;     /*    10    11 */
	struct acpi_resource_label resource_source_label; /*    21    10 */
	u8 *                       vendor_data;          /*    31     8 */

	/* size: 39, cachelines: 1, members: 9 */
	/* last cacheline: 39 bytes */
} __attribute__((__packed__));
union acpi_resource_data {
	struct acpi_resource_irq   irq;                /*     0     7 */
	struct acpi_resource_dma   dma;                /*     0     5 */
	struct acpi_resource_start_dependent start_dpf; /*     0     3 */
	struct acpi_resource_io    io;                 /*     0     7 */
	struct acpi_resource_fixed_io fixed_io;        /*     0     3 */
	struct acpi_resource_fixed_dma fixed_dma;      /*     0     5 */
	struct acpi_resource_vendor vendor;            /*     0     3 */
	struct acpi_resource_vendor_typed vendor_typed; /*     0    20 */
	struct acpi_resource_end_tag end_tag;          /*     0     1 */
	struct acpi_resource_memory24 memory24;        /*     0     9 */
	struct acpi_resource_memory32 memory32;        /*     0    17 */
	struct acpi_resource_fixed_memory32 fixed_memory32; /*     0     9 */
	struct acpi_resource_address16 address16;      /*     0    30 */
	struct acpi_resource_address32 address32;      /*     0    40 */
	struct acpi_resource_address64 address64;      /*     0    60 */
	struct acpi_resource_extended_address64 ext_address64; /*     0    58 */
	struct acpi_resource_extended_irq extended_irq; /*     0    21 */
	struct acpi_resource_generic_register generic_reg; /*     0    12 */
	struct acpi_resource_gpio  gpio;               /*     0    44 */
	struct acpi_resource_i2c_serialbus i2c_serial_bus; /*     0    36 */
	struct acpi_resource_spi_serialbus spi_serial_bus; /*     0    40 */
	struct acpi_resource_uart_serialbus uart_serial_bus; /*     0    43 */
	struct acpi_resource_common_serialbus common_serial_bus; /*     0    29 */
	struct acpi_resource_pin_function pin_function; /*     0    36 */
	struct acpi_resource_pin_config pin_config;    /*     0    39 */
	struct acpi_resource_pin_group pin_group;      /*     0    32 */
	struct acpi_resource_pin_group_function pin_group_function; /*     0    36 */
	struct acpi_resource_pin_group_config pin_group_config; /*     0    39 */
	struct acpi_resource_address address;          /*     0     9 */
};
struct acpi_resource {
	u32                        type;                 /*     0     4 */
	u32                        length;               /*     4     4 */
	union acpi_resource_data   data;                 /*     8    60 */

	/* size: 68, cachelines: 2, members: 3 */
	/* last cacheline: 4 bytes */
} __attribute__((__packed__));
struct acpi_gpiolib_dmi_quirk {
	bool                       no_edge_events_on_boot; /*     0     1 */

	/* XXX 7 bytes hole, try to pack */

	char *                     ignore_wake;          /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 9, holes: 1, sum holes: 7 */
	/* last cacheline: 16 bytes */
};
struct acpi_gpio_event {
	struct list_head           node;                 /*     0    16 */
	acpi_handle                handle;               /*    16     8 */
	irq_handler_t              handler;              /*    24     8 */
	unsigned int               pin;                  /*    32     4 */
	unsigned int               irq;                  /*    36     4 */
	long unsigned int          irqflags;             /*    40     8 */
	bool                       irq_is_wake;          /*    48     1 */
	bool                       irq_requested;        /*    49     1 */

	/* XXX 6 bytes hole, try to pack */

	struct gpio_desc *         desc;                 /*    56     8 */

	/* size: 64, cachelines: 1, members: 9 */
	/* sum members: 58, holes: 1, sum holes: 6 */
};
struct acpi_gpio_connection {
	struct list_head           node;                 /*     0    16 */
	unsigned int               pin;                  /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	struct gpio_desc *         desc;                 /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct acpi_gpio_chip {
	struct acpi_connection_info conn_info;           /*     0    16 */

	/* XXX last struct has 5 bytes of padding */

	struct list_head           conns;                /*    16    16 */
	struct mutex               conn_lock;            /*    32    32 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct gpio_chip *         chip;                 /*    64     8 */
	struct list_head           events;               /*    72    16 */
	struct list_head           deferred_req_irqs_list_entry; /*    88    16 */

	/* size: 104, cachelines: 2, members: 6 */
	/* paddings: 1, sum paddings: 5 */
	/* last cacheline: 40 bytes */
};
struct acpi_gpio_lookup {
	struct acpi_gpio_info      info;                 /*     0    32 */
	int                        index;                /*    32     4 */
	int                        pin_index;            /*    36     4 */
	bool                       active_low;           /*    40     1 */

	/* XXX 7 bytes hole, try to pack */

	struct gpio_desc *         desc;                 /*    48     8 */
	int                        n;                    /*    56     4 */

	/* size: 64, cachelines: 1, members: 6 */
	/* sum members: 53, holes: 1, sum holes: 7 */
	/* padding: 4 */
};
struct device_link {
	struct device *            supplier;             /*     0     8 */
	struct list_head           s_node;               /*     8    16 */
	struct device *            consumer;             /*    24     8 */
	struct list_head           c_node;               /*    32    16 */
	struct device              link_dev __attribute__((__aligned__(8))); /*    48  1000 */
	/* --- cacheline 16 boundary (1024 bytes) was 24 bytes ago --- */
	enum device_link_state     status;               /*  1048     4 */
	u32                        flags;                /*  1052     4 */
	refcount_t                 rpm_active;           /*  1056     4 */
	struct kref                kref;                 /*  1060     4 */
	struct work_struct         rm_work;              /*  1064    64 */
	/* --- cacheline 17 boundary (1088 bytes) was 40 bytes ago --- */
	bool                       supplier_preactivated; /*  1128     1 */

	/* size: 1136, cachelines: 18, members: 11 */
	/* padding: 7 */
	/* forced alignments: 1 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct pwm_args {
	u64                        period;               /*     0     8 */
	enum pwm_polarity          polarity;             /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct pwm_state {
	u64                        period;               /*     0     8 */
	u64                        duty_cycle;           /*     8     8 */
	enum pwm_polarity          polarity;             /*    16     4 */
	bool                       enabled;              /*    20     1 */

	/* size: 24, cachelines: 1, members: 4 */
	/* padding: 3 */
	/* last cacheline: 24 bytes */
};
struct pwm_device {
	const char  *              label;                /*     0     8 */
	long unsigned int          flags;                /*     8     8 */
	unsigned int               hwpwm;                /*    16     4 */
	unsigned int               pwm;                  /*    20     4 */
	struct pwm_chip *          chip;                 /*    24     8 */
	void *                     chip_data;            /*    32     8 */
	struct pwm_args            args;                 /*    40    16 */

	/* XXX last struct has 4 bytes of padding */

	struct pwm_state           state;                /*    56    24 */

	/* XXX last struct has 3 bytes of padding */

	/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
	struct pwm_state           last;                 /*    80    24 */

	/* XXX last struct has 3 bytes of padding */

	/* size: 104, cachelines: 2, members: 9 */
	/* paddings: 3, sum paddings: 10 */
	/* last cacheline: 40 bytes */
};
struct pwm_chip {
	struct device *            dev;                  /*     0     8 */
	const struct pwm_ops  *    ops;                  /*     8     8 */
	int                        base;                 /*    16     4 */
	unsigned int               npwm;                 /*    20     4 */
	struct pwm_device *        (*of_xlate)(struct pwm_chip *, const struct of_phandle_args  *); /*    24     8 */
	unsigned int               of_pwm_n_cells;       /*    32     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           list;                 /*    40    16 */
	struct pwm_device *        pwms;                 /*    56     8 */

	/* size: 64, cachelines: 1, members: 8 */
	/* sum members: 60, holes: 1, sum holes: 4 */
};
struct pwm_ops {
	int                        (*request)(struct pwm_chip *, struct pwm_device *); /*     0     8 */
	void                       (*free)(struct pwm_chip *, struct pwm_device *); /*     8     8 */
	int                        (*capture)(struct pwm_chip *, struct pwm_device *, struct pwm_capture *, long unsigned int); /*    16     8 */
	int                        (*apply)(struct pwm_chip *, struct pwm_device *, const struct pwm_state  *); /*    24     8 */
	void                       (*get_state)(struct pwm_chip *, struct pwm_device *, struct pwm_state *); /*    32     8 */
	struct module *            owner;                /*    40     8 */
	int                        (*config)(struct pwm_chip *, struct pwm_device *, int, int); /*    48     8 */
	int                        (*set_polarity)(struct pwm_chip *, struct pwm_device *, enum pwm_polarity); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*enable)(struct pwm_chip *, struct pwm_device *); /*    64     8 */
	void                       (*disable)(struct pwm_chip *, struct pwm_device *); /*    72     8 */

	/* size: 80, cachelines: 2, members: 10 */
	/* last cacheline: 16 bytes */
};
struct pwm_capture {
	unsigned int               period;               /*     0     4 */
	unsigned int               duty_cycle;           /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct pwm_lookup {
	struct list_head           list;                 /*     0    16 */
	const char  *              provider;             /*    16     8 */
	unsigned int               index;                /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	const char  *              dev_id;               /*    32     8 */
	const char  *              con_id;               /*    40     8 */
	unsigned int               period;               /*    48     4 */
	enum pwm_polarity          polarity;             /*    52     4 */
	const char  *              module;               /*    56     8 */

	/* size: 64, cachelines: 1, members: 8 */
	/* sum members: 60, holes: 1, sum holes: 4 */
};
struct trace_event_raw_pwm {
	struct trace_entry         ent;                  /*     0     8 */
	struct pwm_device *        pwm;                  /*     8     8 */
	u64                        period;               /*    16     8 */
	u64                        duty_cycle;           /*    24     8 */
	enum pwm_polarity          polarity;             /*    32     4 */
	bool                       enabled;              /*    36     1 */
	char                       __data[];             /*    37     0 */

	/* size: 40, cachelines: 1, members: 7 */
	/* padding: 3 */
	/* last cacheline: 40 bytes */
};
struct trace_event_data_offsets_pwm {

	/* size: 0, cachelines: 0, members: 0 */
};
struct pwm_export {
	struct device              child __attribute__((__aligned__(8))); /*     0  1000 */
	/* --- cacheline 15 boundary (960 bytes) was 40 bytes ago --- */
	struct pwm_device *        pwm;                  /*  1000     8 */
	struct mutex               lock;                 /*  1008    32 */
	/* --- cacheline 16 boundary (1024 bytes) was 16 bytes ago --- */
	struct pwm_state           suspend;              /*  1040    24 */

	/* XXX last struct has 3 bytes of padding */

	/* size: 1064, cachelines: 17, members: 4 */
	/* paddings: 1, sum paddings: 3 */
	/* forced alignments: 1 */
	/* last cacheline: 40 bytes */
} __attribute__((__aligned__(8)));
struct pci_sriov {
	int                        pos;                  /*     0     4 */
	int                        nres;                 /*     4     4 */
	u32                        cap;                  /*     8     4 */
	u16                        ctrl;                 /*    12     2 */
	u16                        total_VFs;            /*    14     2 */
	u16                        initial_VFs;          /*    16     2 */
	u16                        num_VFs;              /*    18     2 */
	u16                        offset;               /*    20     2 */
	u16                        stride;               /*    22     2 */
	u16                        vf_device;            /*    24     2 */

	/* XXX 2 bytes hole, try to pack */

	u32                        pgsz;                 /*    28     4 */
	u8                         link;                 /*    32     1 */
	u8                         max_VF_buses;         /*    33     1 */
	u16                        driver_max_VFs;       /*    34     2 */

	/* XXX 4 bytes hole, try to pack */

	struct pci_dev *           dev;                  /*    40     8 */
	struct pci_dev *           self;                 /*    48     8 */
	u32                        class;                /*    56     4 */
	u8                         hdr_type;             /*    60     1 */

	/* XXX 1 byte hole, try to pack */

	u16                        subsystem_vendor;     /*    62     2 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u16                        subsystem_device;     /*    64     2 */

	/* XXX 6 bytes hole, try to pack */

	resource_size_t            barsz[6];             /*    72    48 */
	bool                       drivers_autoprobe;    /*   120     1 */

	/* XXX 7 bytes hole, try to pack */

	/* --- cacheline 2 boundary (128 bytes) --- */
	u64                        kabi_reserved1;       /*   128     8 */
	u64                        kabi_reserved2;       /*   136     8 */
	u64                        kabi_reserved3;       /*   144     8 */
	u64                        kabi_reserved4;       /*   152     8 */
	u64                        kabi_reserved5;       /*   160     8 */
	u64                        kabi_reserved6;       /*   168     8 */
	u64                        kabi_reserved7;       /*   176     8 */
	u64                        kabi_reserved8;       /*   184     8 */

	/* size: 192, cachelines: 3, members: 30 */
	/* sum members: 172, holes: 5, sum holes: 20 */
};
struct rcec_ea {
	u8                         nextbusn;             /*     0     1 */
	u8                         lastbusn;             /*     1     1 */

	/* XXX 2 bytes hole, try to pack */

	u32                        bitmap;               /*     4     4 */

	/* size: 8, cachelines: 1, members: 3 */
	/* sum members: 6, holes: 1, sum holes: 2 */
	/* last cacheline: 8 bytes */
};
struct pci_bus_resource {
	struct list_head           list;                 /*     0    16 */
	struct resource *          res;                  /*    16     8 */
	unsigned int               flags;                /*    24     4 */

	/* size: 32, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct pci_bus_region {
	pci_bus_addr_t             start;                /*     0     8 */
	pci_bus_addr_t             end;                  /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct hotplug_slot {
	const struct hotplug_slot_ops  * ops;            /*     0     8 */
	struct list_head           slot_list;            /*     8    16 */
	struct pci_slot *          pci_slot;             /*    24     8 */
	struct module *            owner;                /*    32     8 */
	const char  *              mod_name;             /*    40     8 */
	u64                        kabi_reserved1;       /*    48     8 */
	u64                        kabi_reserved2;       /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u64                        kabi_reserved3;       /*    64     8 */
	u64                        kabi_reserved4;       /*    72     8 */
	u64                        kabi_reserved5;       /*    80     8 */
	u64                        kabi_reserved6;       /*    88     8 */
	u64                        kabi_reserved7;       /*    96     8 */
	u64                        kabi_reserved8;       /*   104     8 */

	/* size: 112, cachelines: 2, members: 13 */
	/* last cacheline: 48 bytes */
};
struct pci_host_bridge {
	struct device              dev __attribute__((__aligned__(8))); /*     0  1000 */
	/* --- cacheline 15 boundary (960 bytes) was 40 bytes ago --- */
	struct pci_bus *           bus;                  /*  1000     8 */
	struct pci_ops *           ops;                  /*  1008     8 */
	struct pci_ops *           child_ops;            /*  1016     8 */
	/* --- cacheline 16 boundary (1024 bytes) --- */
	void *                     sysdata;              /*  1024     8 */
	int                        busnr;                /*  1032     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           windows;              /*  1040    16 */
	struct list_head           dma_ranges;           /*  1056    16 */
	u8                         (*swizzle_irq)(struct pci_dev *, u8 *); /*  1072     8 */
	int                        (*map_irq)(const struct pci_dev  *, u8, u8); /*  1080     8 */
	/* --- cacheline 17 boundary (1088 bytes) --- */
	void                       (*release_fn)(struct pci_host_bridge *); /*  1088     8 */
	void *                     release_data;         /*  1096     8 */
	struct msi_controller *    msi;                  /*  1104     8 */
	unsigned int               ignore_reset_delay:1; /*  1112: 0  4 */
	unsigned int               no_ext_tags:1;        /*  1112: 1  4 */
	unsigned int               no_inc_mrrs:1;        /*  1112: 2  4 */
	unsigned int               no_dis_bmaster:1;     /*  1112: 3  4 */
	unsigned int               native_aer:1;         /*  1112: 4  4 */
	unsigned int               native_pcie_hotplug:1; /*  1112: 5  4 */
	unsigned int               native_shpc_hotplug:1; /*  1112: 6  4 */
	unsigned int               native_pme:1;         /*  1112: 7  4 */
	unsigned int               native_ltr:1;         /*  1112: 8  4 */
	unsigned int               native_dpc:1;         /*  1112: 9  4 */
	unsigned int               preserve_config:1;    /*  1112:10  4 */
	unsigned int               size_windows:1;       /*  1112:11  4 */

	/* XXX 20 bits hole, try to pack */
	/* XXX 4 bytes hole, try to pack */

	resource_size_t            (*align_resource)(struct pci_dev *, const struct resource  *, resource_size_t, resource_size_t, resource_size_t); /*  1120     8 */

	/* XXX 24 bytes hole, try to pack */

	/* --- cacheline 18 boundary (1152 bytes) --- */
	long unsigned int          private[] __attribute__((__aligned__(64))); /*  1152     0 */

	/* size: 1152, cachelines: 18, members: 27 */
	/* sum members: 1116, holes: 3, sum holes: 32 */
	/* sum bitfield members: 12 bits, bit holes: 1, sum bit holes: 20 bits */
	/* forced alignments: 2, forced holes: 1, sum forced holes: 24 */
} __attribute__((__aligned__(64)));
struct hotplug_slot_ops {
	int                        (*enable_slot)(struct hotplug_slot *); /*     0     8 */
	int                        (*disable_slot)(struct hotplug_slot *); /*     8     8 */
	int                        (*set_attention_status)(struct hotplug_slot *, u8); /*    16     8 */
	int                        (*hardware_test)(struct hotplug_slot *, u32); /*    24     8 */
	int                        (*get_power_status)(struct hotplug_slot *, u8 *); /*    32     8 */
	int                        (*get_attention_status)(struct hotplug_slot *, u8 *); /*    40     8 */
	int                        (*get_latch_status)(struct hotplug_slot *, u8 *); /*    48     8 */
	int                        (*get_adapter_status)(struct hotplug_slot *, u8 *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*reset_slot)(struct hotplug_slot *, int); /*    64     8 */
	u64                        kabi_reserved1;       /*    72     8 */
	u64                        kabi_reserved2;       /*    80     8 */
	u64                        kabi_reserved3;       /*    88     8 */
	u64                        kabi_reserved4;       /*    96     8 */
	u64                        kabi_reserved5;       /*   104     8 */
	u64                        kabi_reserved6;       /*   112     8 */
	u64                        kabi_reserved7;       /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	u64                        kabi_reserved8;       /*   128     8 */

	/* size: 136, cachelines: 3, members: 17 */
	/* last cacheline: 8 bytes */
};
struct pci_domain_busn_res {
	struct list_head           list;                 /*     0    16 */
	struct resource            res;                  /*    16    96 */
	/* --- cacheline 1 boundary (64 bytes) was 48 bytes ago --- */
	int                        domain_nr;            /*   112     4 */

	/* size: 120, cachelines: 2, members: 3 */
	/* padding: 4 */
	/* last cacheline: 56 bytes */
};
struct bus_attribute {
	struct attribute           attr;                 /*     0    16 */

	/* XXX last struct has 6 bytes of padding */

	ssize_t                    (*show)(struct bus_type *, char *); /*    16     8 */
	ssize_t                    (*store)(struct bus_type *, const char  *, size_t); /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* paddings: 1, sum paddings: 6 */
	/* last cacheline: 32 bytes */
};
struct pci_cap_saved_data {
	u16                        cap_nr;               /*     0     2 */
	bool                       cap_extended;         /*     2     1 */

	/* XXX 1 byte hole, try to pack */

	unsigned int               size;                 /*     4     4 */
	u32                        data[];               /*     8     0 */

	/* size: 8, cachelines: 1, members: 4 */
	/* sum members: 7, holes: 1, sum holes: 1 */
	/* last cacheline: 8 bytes */
};
struct pci_cap_saved_state {
	struct hlist_node          next;                 /*     0    16 */
	struct pci_cap_saved_data  cap;                  /*    16     8 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct pci_platform_pm_ops {
	bool                       (*bridge_d3)(struct pci_dev *); /*     0     8 */
	bool                       (*is_manageable)(struct pci_dev *); /*     8     8 */
	int                        (*set_state)(struct pci_dev *, pci_power_t); /*    16     8 */
	pci_power_t                (*get_state)(struct pci_dev *); /*    24     8 */
	void                       (*refresh_state)(struct pci_dev *); /*    32     8 */
	pci_power_t                (*choose_state)(struct pci_dev *); /*    40     8 */
	int                        (*set_wakeup)(struct pci_dev *, bool); /*    48     8 */
	bool                       (*need_resume)(struct pci_dev *); /*    56     8 */

	/* size: 64, cachelines: 1, members: 8 */
};
struct pci_pme_device {
	struct list_head           list;                 /*     0    16 */
	struct pci_dev *           dev;                  /*    16     8 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct pci_saved_state {
	u32                        config_space[16];     /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct pci_cap_saved_data  cap[];                /*    64     0 */

	/* size: 64, cachelines: 1, members: 2 */
};
struct pci_devres {
	unsigned int               enabled:1;            /*     0: 0  4 */
	unsigned int               pinned:1;             /*     0: 1  4 */
	unsigned int               orig_intx:1;          /*     0: 2  4 */
	unsigned int               restore_intx:1;       /*     0: 3  4 */
	unsigned int               mwi:1;                /*     0: 4  4 */

	/* XXX 27 bits hole, try to pack */

	u32                        region_mask;          /*     4     4 */

	/* size: 8, cachelines: 1, members: 6 */
	/* sum members: 4 */
	/* sum bitfield members: 5 bits, bit holes: 1, sum bit holes: 27 bits */
	/* last cacheline: 8 bytes */
};
struct driver_attribute {
	struct attribute           attr;                 /*     0    16 */

	/* XXX last struct has 6 bytes of padding */

	ssize_t                    (*show)(struct device_driver *, char *); /*    16     8 */
	ssize_t                    (*store)(struct device_driver *, const char  *, size_t); /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* paddings: 1, sum paddings: 6 */
	/* last cacheline: 32 bytes */
};
struct pcie_device {
	int                        irq;                  /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct pci_dev *           port;                 /*     8     8 */
	u32                        service;              /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     priv_data;            /*    24     8 */
	struct device              device __attribute__((__aligned__(8))); /*    32  1000 */

	/* size: 1032, cachelines: 17, members: 5 */
	/* sum members: 1024, holes: 2, sum holes: 8 */
	/* forced alignments: 1 */
	/* last cacheline: 8 bytes */
} __attribute__((__aligned__(8)));
struct pcie_port_service_driver {
	const char  *              name;                 /*     0     8 */
	int                        (*probe)(struct pcie_device *); /*     8     8 */
	void                       (*remove)(struct pcie_device *); /*    16     8 */
	int                        (*suspend)(struct pcie_device *); /*    24     8 */
	int                        (*resume_noirq)(struct pcie_device *); /*    32     8 */
	int                        (*resume)(struct pcie_device *); /*    40     8 */
	int                        (*runtime_suspend)(struct pcie_device *); /*    48     8 */
	int                        (*runtime_resume)(struct pcie_device *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	void                       (*error_resume)(struct pci_dev *); /*    64     8 */
	int                        port_type;            /*    72     4 */
	u32                        service;              /*    76     4 */
	struct device_driver       driver;               /*    80   192 */

	/* size: 272, cachelines: 5, members: 12 */
	/* last cacheline: 16 bytes */
};
struct pci_dynid {
	struct list_head           node;                 /*     0    16 */
	struct pci_device_id       id;                   /*    16    32 */

	/* size: 48, cachelines: 1, members: 2 */
	/* last cacheline: 48 bytes */
};
struct drv_dev_and_id {
	struct pci_driver *        drv;                  /*     0     8 */
	struct pci_dev *           dev;                  /*     8     8 */
	const struct pci_device_id  * id;                /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct pci_vpd {
	const struct pci_vpd_ops  * ops;                 /*     0     8 */
	struct bin_attribute *     attr;                 /*     8     8 */
	struct mutex               lock;                 /*    16    32 */
	unsigned int               len;                  /*    48     4 */
	u16                        flag;                 /*    52     2 */
	u8                         cap;                  /*    54     1 */

	/* Bitfield combined with previous fields */

	unsigned int               busy:1;               /*    52:24  4 */
	unsigned int               valid:1;              /*    52:25  4 */

	/* size: 56, cachelines: 1, members: 8 */
	/* bit_padding: 6 bits */
	/* last cacheline: 56 bytes */
};
struct pci_vpd_ops {
	ssize_t                    (*read)(struct pci_dev *, loff_t, size_t, void *); /*     0     8 */
	ssize_t                    (*write)(struct pci_dev *, loff_t, size_t, const void  *); /*     8     8 */
	int                        (*set_size)(struct pci_dev *, size_t); /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct pci_dev_resource {
	struct list_head           list;                 /*     0    16 */
	struct resource *          res;                  /*    16     8 */
	struct pci_dev *           dev;                  /*    24     8 */
	resource_size_t            start;                /*    32     8 */
	resource_size_t            end;                  /*    40     8 */
	resource_size_t            add_size;             /*    48     8 */
	resource_size_t            min_align;            /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	long unsigned int          flags;                /*    64     8 */

	/* size: 72, cachelines: 2, members: 8 */
	/* last cacheline: 8 bytes */
};
struct portdrv_service_data {
	struct pcie_port_service_driver * drv;           /*     0     8 */
	struct device *            dev;                  /*     8     8 */
	u32                        service;              /*    16     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct walk_rcec_data {
	struct pci_dev *           rcec;                 /*     0     8 */
	int                        (*user_callback)(struct pci_dev *, void *); /*     8     8 */
	void *                     user_data;            /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct pcie_link_state {
	struct pci_dev *           pdev;                 /*     0     8 */
	struct pci_dev *           downstream;           /*     8     8 */
	struct pcie_link_state *   root;                 /*    16     8 */
	struct pcie_link_state *   parent;               /*    24     8 */
	struct list_head           sibling;              /*    32    16 */
	u32                        aspm_support:7;       /*    48: 0  4 */
	u32                        aspm_enabled:7;       /*    48: 7  4 */
	u32                        aspm_capable:7;       /*    48:14  4 */
	u32                        aspm_default:7;       /*    48:21  4 */

	/* XXX 4 bits hole, try to pack */

	/* Force alignment to the next boundary: */
	u32                        :0;

	u32                        aspm_disable:7;       /*    52: 0  4 */
	u32                        clkpm_capable:1;      /*    52: 7  4 */
	u32                        clkpm_enabled:1;      /*    52: 8  4 */
	u32                        clkpm_default:1;      /*    52: 9  4 */
	u32                        clkpm_disable:1;      /*    52:10  4 */

	/* XXX 21 bits hole, try to pack */

	struct aspm_latency        latency_up;           /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct aspm_latency        latency_dw;           /*    64     8 */
	struct aspm_latency        acceptable[8];        /*    72    64 */

	/* size: 136, cachelines: 3, members: 17 */
	/* sum members: 128 */
	/* sum bitfield members: 39 bits, bit holes: 2, sum bit holes: 25 bits */
	/* last cacheline: 8 bytes */
};
struct aspm_latency {
	u32                        l0s;                  /*     0     4 */
	u32                        l1;                   /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct aer_stats {
	u64                        dev_cor_errs[16];     /*     0   128 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	u64                        dev_fatal_errs[27];   /*   128   216 */
	/* --- cacheline 5 boundary (320 bytes) was 24 bytes ago --- */
	u64                        dev_nonfatal_errs[27]; /*   344   216 */
	/* --- cacheline 8 boundary (512 bytes) was 48 bytes ago --- */
	u64                        dev_total_cor_errs;   /*   560     8 */
	u64                        dev_total_fatal_errs; /*   568     8 */
	/* --- cacheline 9 boundary (576 bytes) --- */
	u64                        dev_total_nonfatal_errs; /*   576     8 */
	u64                        rootport_total_cor_errs; /*   584     8 */
	u64                        rootport_total_fatal_errs; /*   592     8 */
	u64                        rootport_total_nonfatal_errs; /*   600     8 */

	/* size: 608, cachelines: 10, members: 9 */
	/* last cacheline: 32 bytes */
};
struct aer_header_log_regs {
	unsigned int               dw0;                  /*     0     4 */
	unsigned int               dw1;                  /*     4     4 */
	unsigned int               dw2;                  /*     8     4 */
	unsigned int               dw3;                  /*    12     4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct aer_capability_regs {
	u32                        header;               /*     0     4 */
	u32                        uncor_status;         /*     4     4 */
	u32                        uncor_mask;           /*     8     4 */
	u32                        uncor_severity;       /*    12     4 */
	u32                        cor_status;           /*    16     4 */
	u32                        cor_mask;             /*    20     4 */
	u32                        cap_control;          /*    24     4 */
	struct aer_header_log_regs header_log;           /*    28    16 */
	u32                        root_command;         /*    44     4 */
	u32                        root_status;          /*    48     4 */
	u16                        cor_err_source;       /*    52     2 */
	u16                        uncor_err_source;     /*    54     2 */

	/* size: 56, cachelines: 1, members: 12 */
	/* last cacheline: 56 bytes */
};
struct aer_err_info {
	struct pci_dev *           dev[5];               /*     0    40 */
	int                        error_dev_num;        /*    40     4 */
	unsigned int               id:16;                /*    44: 0  4 */
	unsigned int               severity:2;           /*    44:16  4 */
	unsigned int               __pad1:5;             /*    44:18  4 */
	unsigned int               multi_error_valid:1;  /*    44:23  4 */
	unsigned int               first_error:5;        /*    44:24  4 */
	unsigned int               __pad2:2;             /*    44:29  4 */
	unsigned int               tlp_header_valid:1;   /*    44:31  4 */
	unsigned int               status;               /*    48     4 */
	unsigned int               mask;                 /*    52     4 */
	struct aer_header_log_regs tlp;                  /*    56    16 */

	/* size: 72, cachelines: 2, members: 12 */
	/* last cacheline: 8 bytes */
};
struct aer_err_source {
	unsigned int               status;               /*     0     4 */
	unsigned int               id;                   /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct aer_rpc {
	struct pci_dev *           rpd;                  /*     0     8 */
	struct {
		union {
			struct __kfifo kfifo;            /*     8    24 */
			struct aer_err_source * type;    /*     8     8 */
			const struct aer_err_source  * const_type; /*     8     8 */
			char *     rectype;              /*     8     8 */
			struct aer_err_source * ptr;     /*     8     8 */
			const struct aer_err_source  * ptr_const; /*     8     8 */
		};                                       /*     8    24 */
		struct aer_err_source buf[128];          /*    32  1024 */
	} aer_fifo;                                      /*     8  1048 */

	/* size: 1056, cachelines: 17, members: 2 */
	/* last cacheline: 32 bytes */
};
struct aer_recover_entry {
	u8                         bus;                  /*     0     1 */
	u8                         devfn;                /*     1     1 */
	u16                        domain;               /*     2     2 */
	int                        severity;             /*     4     4 */
	struct aer_capability_regs * regs;               /*     8     8 */

	/* size: 16, cachelines: 1, members: 5 */
	/* last cacheline: 16 bytes */
};
struct pcie_pme_service_data {
	spinlock_t                 lock;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct pcie_device *       srv;                  /*     8     8 */
	struct work_struct         work;                 /*    16    64 */
	/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
	bool                       noirq;                /*    80     1 */

	/* size: 88, cachelines: 2, members: 4 */
	/* sum members: 77, holes: 1, sum holes: 4 */
	/* padding: 7 */
	/* last cacheline: 24 bytes */
};
struct pci_filp_private {
	enum pci_mmap_state        mmap_state;           /*     0     4 */
	int                        write_combine;        /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct pci_slot_attribute {
	struct attribute           attr;                 /*     0    16 */

	/* XXX last struct has 6 bytes of padding */

	ssize_t                    (*show)(struct pci_slot *, char *); /*    16     8 */
	ssize_t                    (*store)(struct pci_slot *, const char  *, size_t); /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* paddings: 1, sum paddings: 6 */
	/* last cacheline: 32 bytes */
};
struct acpi_buffer {
	acpi_size                  length;               /*     0     8 */
	void *                     pointer;              /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct acpi_bus_type {
	struct list_head           list;                 /*     0    16 */
	const char  *              name;                 /*    16     8 */
	bool                       (*match)(struct device *); /*    24     8 */
	struct acpi_device *       (*find_companion)(struct device *); /*    32     8 */
	void                       (*setup)(struct device *); /*    40     8 */
	void                       (*cleanup)(struct device *); /*    48     8 */

	/* size: 56, cachelines: 1, members: 6 */
	/* last cacheline: 56 bytes */
};
struct acpi_pci_root {
	struct acpi_device *       device;               /*     0     8 */
	struct pci_bus *           bus;                  /*     8     8 */
	u16                        segment;              /*    16     2 */

	/* XXX 6 bytes hole, try to pack */

	struct resource            secondary;            /*    24    96 */
	/* --- cacheline 1 boundary (64 bytes) was 56 bytes ago --- */
	u32                        osc_support_set;      /*   120     4 */
	u32                        osc_control_set;      /*   124     4 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	phys_addr_t                mcfg_addr;            /*   128     8 */

	/* size: 136, cachelines: 3, members: 7 */
	/* sum members: 130, holes: 1, sum holes: 6 */
	/* last cacheline: 8 bytes */
};
struct hpx_type0 {
	u32                        revision;             /*     0     4 */
	u8                         cache_line_size;      /*     4     1 */
	u8                         latency_timer;        /*     5     1 */
	u8                         enable_serr;          /*     6     1 */
	u8                         enable_perr;          /*     7     1 */

	/* size: 8, cachelines: 1, members: 5 */
	/* last cacheline: 8 bytes */
};
struct hpx_type1 {
	u32                        revision;             /*     0     4 */
	u8                         max_mem_read;         /*     4     1 */
	u8                         avg_max_split;        /*     5     1 */
	u16                        tot_max_split;        /*     6     2 */

	/* size: 8, cachelines: 1, members: 4 */
	/* last cacheline: 8 bytes */
};
struct hpx_type2 {
	u32                        revision;             /*     0     4 */
	u32                        unc_err_mask_and;     /*     4     4 */
	u32                        unc_err_mask_or;      /*     8     4 */
	u32                        unc_err_sever_and;    /*    12     4 */
	u32                        unc_err_sever_or;     /*    16     4 */
	u32                        cor_err_mask_and;     /*    20     4 */
	u32                        cor_err_mask_or;      /*    24     4 */
	u32                        adv_err_cap_and;      /*    28     4 */
	u32                        adv_err_cap_or;       /*    32     4 */
	u16                        pci_exp_devctl_and;   /*    36     2 */
	u16                        pci_exp_devctl_or;    /*    38     2 */
	u16                        pci_exp_lnkctl_and;   /*    40     2 */
	u16                        pci_exp_lnkctl_or;    /*    42     2 */
	u32                        sec_unc_err_sever_and; /*    44     4 */
	u32                        sec_unc_err_sever_or; /*    48     4 */
	u32                        sec_unc_err_mask_and; /*    52     4 */
	u32                        sec_unc_err_mask_or;  /*    56     4 */

	/* size: 60, cachelines: 1, members: 17 */
	/* last cacheline: 60 bytes */
};
struct hpx_type3 {
	u16                        device_type;          /*     0     2 */
	u16                        function_type;        /*     2     2 */
	u16                        config_space_location; /*     4     2 */
	u16                        pci_exp_cap_id;       /*     6     2 */
	u16                        pci_exp_cap_ver;      /*     8     2 */
	u16                        pci_exp_vendor_id;    /*    10     2 */
	u16                        dvsec_id;             /*    12     2 */
	u16                        dvsec_rev;            /*    14     2 */
	u16                        match_offset;         /*    16     2 */

	/* XXX 2 bytes hole, try to pack */

	u32                        match_mask_and;       /*    20     4 */
	u32                        match_value;          /*    24     4 */
	u16                        reg_offset;           /*    28     2 */

	/* XXX 2 bytes hole, try to pack */

	u32                        reg_mask_and;         /*    32     4 */
	u32                        reg_mask_or;          /*    36     4 */

	/* size: 40, cachelines: 1, members: 14 */
	/* sum members: 36, holes: 2, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct pci_fixup {
	u16                        vendor;               /*     0     2 */
	u16                        device;               /*     2     2 */
	u32                        class;                /*     4     4 */
	unsigned int               class_shift;          /*     8     4 */
	int                        hook_offset;          /*    12     4 */

	/* size: 16, cachelines: 1, members: 5 */
	/* last cacheline: 16 bytes */
};
struct nt_partition_info {
	u32                        xlink_enabled;        /*     0     4 */
	u32                        target_part_low;      /*     4     4 */
	u32                        target_part_high;     /*     8     4 */
	u32                        reserved;             /*    12     4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct ntb_info_regs {
	u8                         partition_count;      /*     0     1 */
	u8                         partition_id;         /*     1     1 */
	u16                        reserved1;            /*     2     2 */
	u64                        ep_map;               /*     4     8 */
	u16                        requester_id;         /*    12     2 */
	u16                        reserved2;            /*    14     2 */
	u32                        reserved3[4];         /*    16    16 */
	struct nt_partition_info   ntp_info[48];         /*    32   768 */

	/* size: 800, cachelines: 13, members: 8 */
	/* last cacheline: 32 bytes */
} __attribute__((__packed__));
struct ntb_ctrl_regs {
	u32                        partition_status;     /*     0     4 */
	u32                        partition_op;         /*     4     4 */
	u32                        partition_ctrl;       /*     8     4 */
	u32                        bar_setup;            /*    12     4 */
	u32                        bar_error;            /*    16     4 */
	u16                        lut_table_entries;    /*    20     2 */
	u16                        lut_table_offset;     /*    22     2 */
	u32                        lut_error;            /*    24     4 */
	u16                        req_id_table_size;    /*    28     2 */
	u16                        req_id_table_offset;  /*    30     2 */
	u32                        req_id_error;         /*    32     4 */
	u32                        reserved1[7];         /*    36    28 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct {
		u32                ctl;                  /*    64     4 */
		u32                win_size;             /*    68     4 */
		u64                xlate_addr;           /*    72     8 */
	} bar_entry[6]; /*    64    96 */
	/* --- cacheline 2 boundary (128 bytes) was 32 bytes ago --- */
	struct {
		u32                win_size;             /*   160     4 */
		u32                reserved[3];          /*   164    12 */
	} bar_ext_entry[6]; /*   160    96 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	u32                        reserved2[192];       /*   256   768 */
	/* --- cacheline 16 boundary (1024 bytes) --- */
	u32                        req_id_table[512];    /*  1024  2048 */
	/* --- cacheline 48 boundary (3072 bytes) --- */
	u32                        reserved3[256];       /*  3072  1024 */
	/* --- cacheline 64 boundary (4096 bytes) --- */
	u64                        lut_entry[512];       /*  4096  4096 */

	/* size: 8192, cachelines: 128, members: 18 */
};
struct pci_dev_reset_methods {
	u16                        vendor;               /*     0     2 */
	u16                        device;               /*     2     2 */

	/* XXX 4 bytes hole, try to pack */

	int                        (*reset)(struct pci_dev *, int); /*     8     8 */

	/* size: 16, cachelines: 1, members: 3 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct pci_dev_acs_enabled {
	u16                        vendor;               /*     0     2 */
	u16                        device;               /*     2     2 */

	/* XXX 4 bytes hole, try to pack */

	int                        (*acs_enabled)(struct pci_dev *, u16); /*     8     8 */

	/* size: 16, cachelines: 1, members: 3 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct pci_dev_acs_ops {
	u16                        vendor;               /*     0     2 */
	u16                        device;               /*     2     2 */

	/* XXX 4 bytes hole, try to pack */

	int                        (*enable_acs)(struct pci_dev *); /*     8     8 */
	int                        (*disable_acs_redir)(struct pci_dev *); /*    16     8 */

	/* size: 24, cachelines: 1, members: 4 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct controller {
	struct pcie_device *       pcie;                 /*     0     8 */
	u32                        slot_cap;             /*     8     4 */
	unsigned int               inband_presence_disabled:1; /*    12: 0  4 */

	/* XXX 15 bits hole, try to pack */
	/* Bitfield combined with next fields */

	u16                        slot_ctrl;            /*    14     2 */
	struct mutex               ctrl_lock;            /*    16    32 */
	long unsigned int          cmd_started;          /*    48     8 */
	unsigned int               cmd_busy:1;           /*    56: 0  4 */

	/* XXX 31 bits hole, try to pack */
	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	wait_queue_head_t          queue;                /*    64    24 */
	atomic_t                   pending_events;       /*    88     4 */
	unsigned int               notification_enabled:1; /*    92: 0  4 */

	/* XXX 31 bits hole, try to pack */

	unsigned int               power_fault_detected; /*    96     4 */

	/* XXX 4 bytes hole, try to pack */

	struct task_struct *       poll_thread;          /*   104     8 */
	u8                         state;                /*   112     1 */

	/* XXX 7 bytes hole, try to pack */

	struct mutex               state_lock;           /*   120    32 */
	/* --- cacheline 2 boundary (128 bytes) was 24 bytes ago --- */
	struct delayed_work        button_work;          /*   152   192 */
	/* --- cacheline 5 boundary (320 bytes) was 24 bytes ago --- */
	struct hotplug_slot        hotplug_slot;         /*   344   112 */
	/* --- cacheline 7 boundary (448 bytes) was 8 bytes ago --- */
	struct rw_semaphore        reset_lock;           /*   456    40 */
	unsigned int               depth;                /*   496     4 */
	unsigned int               ist_running;          /*   500     4 */
	int                        request_result;       /*   504     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 8 boundary (512 bytes) --- */
	wait_queue_head_t          requester;            /*   512    24 */

	/* size: 536, cachelines: 9, members: 21 */
	/* sum members: 507, holes: 4, sum holes: 19 */
	/* sum bitfield members: 3 bits, bit holes: 3, sum bit holes: 77 bits */
	/* last cacheline: 24 bytes */
};
struct slot {
	u8                         bus;                  /*     0     1 */
	u8                         device;               /*     1     1 */
	u16                        status;               /*     2     2 */
	u32                        number;               /*     4     4 */
	u8                         is_a_board;           /*     8     1 */
	u8                         state;                /*     9     1 */
	u8                         attention_save;       /*    10     1 */
	u8                         presence_save;        /*    11     1 */
	u8                         latch_save;           /*    12     1 */
	u8                         pwr_save;             /*    13     1 */

	/* XXX 2 bytes hole, try to pack */

	struct controller *        ctrl;                 /*    16     8 */
	const struct hpc_ops  *    hpc_ops;              /*    24     8 */
	struct hotplug_slot        hotplug_slot;         /*    32   112 */
	/* --- cacheline 2 boundary (128 bytes) was 16 bytes ago --- */
	struct list_head           slot_list;            /*   144    16 */
	struct delayed_work        work;                 /*   160   192 */
	/* --- cacheline 5 boundary (320 bytes) was 32 bytes ago --- */
	struct mutex               lock;                 /*   352    32 */
	/* --- cacheline 6 boundary (384 bytes) --- */
	struct workqueue_struct *  wq;                   /*   384     8 */
	u8                         hp_slot;              /*   392     1 */

	/* size: 400, cachelines: 7, members: 18 */
	/* sum members: 391, holes: 1, sum holes: 2 */
	/* padding: 7 */
	/* last cacheline: 16 bytes */
};
struct controller {
	struct mutex               crit_sect;            /*     0    32 */
	struct mutex               cmd_lock;             /*    32    32 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        num_slots;            /*    64     4 */
	int                        slot_num_inc;         /*    68     4 */
	struct pci_dev *           pci_dev;              /*    72     8 */
	struct list_head           slot_list;            /*    80    16 */
	const struct hpc_ops  *    hpc_ops;              /*    96     8 */
	wait_queue_head_t          queue;                /*   104    24 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	u8                         slot_device_offset;   /*   128     1 */

	/* XXX 3 bytes hole, try to pack */

	u32                        pcix_misc2_reg;       /*   132     4 */
	u32                        first_slot;           /*   136     4 */
	u32                        cap_offset;           /*   140     4 */
	long unsigned int          mmio_base;            /*   144     8 */
	long unsigned int          mmio_size;            /*   152     8 */
	void *                     creg;                 /*   160     8 */
	struct timer_list          poll_timer;           /*   168    72 */

	/* size: 240, cachelines: 4, members: 16 */
	/* sum members: 237, holes: 1, sum holes: 3 */
	/* last cacheline: 48 bytes */
};
struct hpc_ops {
	int                        (*power_on_slot)(struct slot *); /*     0     8 */
	int                        (*slot_enable)(struct slot *); /*     8     8 */
	int                        (*slot_disable)(struct slot *); /*    16     8 */
	int                        (*set_bus_speed_mode)(struct slot *, enum pci_bus_speed); /*    24     8 */
	int                        (*get_power_status)(struct slot *, u8 *); /*    32     8 */
	int                        (*get_attention_status)(struct slot *, u8 *); /*    40     8 */
	int                        (*set_attention_status)(struct slot *, u8); /*    48     8 */
	int                        (*get_latch_status)(struct slot *, u8 *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*get_adapter_status)(struct slot *, u8 *); /*    64     8 */
	int                        (*get_adapter_speed)(struct slot *, enum pci_bus_speed *); /*    72     8 */
	int                        (*get_mode1_ECC_cap)(struct slot *, u8 *); /*    80     8 */
	int                        (*get_prog_int)(struct slot *, u8 *); /*    88     8 */
	int                        (*query_power_fault)(struct slot *); /*    96     8 */
	void                       (*green_led_on)(struct slot *); /*   104     8 */
	void                       (*green_led_off)(struct slot *); /*   112     8 */
	void                       (*green_led_blink)(struct slot *); /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	void                       (*release_ctlr)(struct controller *); /*   128     8 */
	int                        (*check_cmd_status)(struct controller *); /*   136     8 */

	/* size: 144, cachelines: 3, members: 18 */
	/* last cacheline: 16 bytes */
};
struct event_info {
	u32                        event_type;           /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct slot *              p_slot;               /*     8     8 */
	struct work_struct         work;                 /*    16    64 */

	/* size: 80, cachelines: 2, members: 3 */
	/* sum members: 76, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct pushbutton_work_info {
	struct slot *              p_slot;               /*     0     8 */
	struct work_struct         work;                 /*     8    64 */

	/* size: 72, cachelines: 2, members: 2 */
	/* last cacheline: 8 bytes */
};
struct slot {
	struct hotplug_slot        hotplug_slot;         /*     0   112 */
	/* --- cacheline 1 boundary (64 bytes) was 48 bytes ago --- */
	struct acpiphp_slot *      acpi_slot;            /*   112     8 */
	unsigned int               sun;                  /*   120     4 */

	/* size: 128, cachelines: 2, members: 3 */
	/* padding: 4 */
};
struct acpiphp_slot {
	struct list_head           node;                 /*     0    16 */
	struct pci_bus *           bus;                  /*    16     8 */
	struct list_head           funcs;                /*    24    16 */
	struct slot *              slot;                 /*    40     8 */
	u8                         device;               /*    48     1 */

	/* XXX 3 bytes hole, try to pack */

	u32                        flags;                /*    52     4 */

	/* size: 56, cachelines: 1, members: 6 */
	/* sum members: 53, holes: 1, sum holes: 3 */
	/* last cacheline: 56 bytes */
};
struct acpiphp_attention_info {
	int                        (*set_attn)(struct hotplug_slot *, u8); /*     0     8 */
	int                        (*get_attn)(struct hotplug_slot *, u8 *); /*     8     8 */
	struct module *            owner;                /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct acpiphp_bridge {
	struct list_head           list;                 /*     0    16 */
	struct list_head           slots;                /*    16    16 */
	struct kref                ref;                  /*    32     4 */

	/* XXX 4 bytes hole, try to pack */

	struct acpiphp_context *   context;              /*    40     8 */
	int                        nr_slots;             /*    48     4 */

	/* XXX 4 bytes hole, try to pack */

	struct pci_bus *           pci_bus;              /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct pci_dev *           pci_dev;              /*    64     8 */
	bool                       is_going_away;        /*    72     1 */

	/* size: 80, cachelines: 2, members: 8 */
	/* sum members: 65, holes: 2, sum holes: 8 */
	/* padding: 7 */
	/* last cacheline: 16 bytes */
};
struct acpiphp_context {
	struct acpi_hotplug_context hp;                  /*     0    32 */
	struct acpiphp_func        func;                 /*    32    40 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	struct acpiphp_bridge *    bridge;               /*    72     8 */
	unsigned int               refcount;             /*    80     4 */

	/* size: 88, cachelines: 2, members: 4 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct acpiphp_func {
	struct acpiphp_bridge *    parent;               /*     0     8 */
	struct acpiphp_slot *      slot;                 /*     8     8 */
	struct list_head           sibling;              /*    16    16 */
	u8                         function;             /*    32     1 */

	/* XXX 3 bytes hole, try to pack */

	u32                        flags;                /*    36     4 */

	/* size: 40, cachelines: 1, members: 5 */
	/* sum members: 37, holes: 1, sum holes: 3 */
	/* last cacheline: 40 bytes */
};
struct acpiphp_root_context {
	struct acpi_hotplug_context hp;                  /*     0    32 */
	struct acpiphp_bridge *    root_bridge;          /*    32     8 */

	/* size: 40, cachelines: 1, members: 2 */
	/* last cacheline: 40 bytes */
};
struct msix_entry {
	u32                        vector;               /*     0     4 */
	u16                        entry;                /*     4     2 */

	/* size: 8, cachelines: 1, members: 2 */
	/* padding: 2 */
	/* last cacheline: 8 bytes */
};
struct dmi_device {
	struct list_head           list;                 /*     0    16 */
	int                        type;                 /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	const char  *              name;                 /*    24     8 */
	void *                     device_data;          /*    32     8 */

	/* size: 40, cachelines: 1, members: 4 */
	/* sum members: 36, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct dmi_dev_onboard {
	struct dmi_device          dev;                  /*     0    40 */
	int                        instance;             /*    40     4 */
	int                        segment;              /*    44     4 */
	int                        bus;                  /*    48     4 */
	int                        devfn;                /*    52     4 */

	/* size: 56, cachelines: 1, members: 5 */
	/* last cacheline: 56 bytes */
};
struct vmd_irq {
	struct list_head           node;                 /*     0    16 */
	struct vmd_irq_list *      irq;                  /*    16     8 */
	bool                       enabled;              /*    24     1 */

	/* XXX 3 bytes hole, try to pack */

	unsigned int               virq;                 /*    28     4 */

	/* size: 32, cachelines: 1, members: 4 */
	/* sum members: 29, holes: 1, sum holes: 3 */
	/* last cacheline: 32 bytes */
};
struct vmd_irq_list {
	struct list_head           irq_list;             /*     0    16 */
	struct srcu_struct         srcu;                 /*    16 50440 */
	/* --- cacheline 788 boundary (50432 bytes) was 24 bytes ago --- */
	unsigned int               count;                /* 50456     4 */

	/* size: 50464, cachelines: 789, members: 3 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct vmd_dev {
	struct pci_dev *           dev;                  /*     0     8 */
	spinlock_t                 cfg_lock;             /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	char *                     cfgbar;               /*    16     8 */
	int                        msix_count;           /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	struct vmd_irq_list *      irqs;                 /*    32     8 */
	struct pci_sysdata         sysdata;              /*    40    40 */
	/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
	struct resource            resources[3];         /*    80   288 */
	/* --- cacheline 5 boundary (320 bytes) was 48 bytes ago --- */
	struct irq_domain *        irq_domain;           /*   368     8 */
	struct pci_bus *           bus;                  /*   376     8 */
	/* --- cacheline 6 boundary (384 bytes) --- */
	u8                         busn_start;           /*   384     1 */

	/* size: 392, cachelines: 7, members: 10 */
	/* sum members: 377, holes: 2, sum holes: 8 */
	/* padding: 7 */
	/* last cacheline: 8 bytes */
};
struct hdmi_any_infoframe {
	enum hdmi_infoframe_type   type;                 /*     0     4 */
	unsigned char              version;              /*     4     1 */
	unsigned char              length;               /*     5     1 */

	/* size: 8, cachelines: 1, members: 3 */
	/* padding: 2 */
	/* last cacheline: 8 bytes */
};
struct hdmi_avi_infoframe {
	enum hdmi_infoframe_type   type;                 /*     0     4 */
	unsigned char              version;              /*     4     1 */
	unsigned char              length;               /*     5     1 */

	/* XXX 2 bytes hole, try to pack */

	enum hdmi_colorspace       colorspace;           /*     8     4 */
	enum hdmi_scan_mode        scan_mode;            /*    12     4 */
	enum hdmi_colorimetry      colorimetry;          /*    16     4 */
	enum hdmi_picture_aspect   picture_aspect;       /*    20     4 */
	enum hdmi_active_aspect    active_aspect;        /*    24     4 */
	bool                       itc;                  /*    28     1 */

	/* XXX 3 bytes hole, try to pack */

	enum hdmi_extended_colorimetry extended_colorimetry; /*    32     4 */
	enum hdmi_quantization_range quantization_range; /*    36     4 */
	enum hdmi_nups             nups;                 /*    40     4 */
	unsigned char              video_code;           /*    44     1 */

	/* XXX 3 bytes hole, try to pack */

	enum hdmi_ycc_quantization_range ycc_quantization_range; /*    48     4 */
	enum hdmi_content_type     content_type;         /*    52     4 */
	unsigned char              pixel_repeat;         /*    56     1 */

	/* XXX 1 byte hole, try to pack */

	short unsigned int         top_bar;              /*    58     2 */
	short unsigned int         bottom_bar;           /*    60     2 */
	short unsigned int         left_bar;             /*    62     2 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	short unsigned int         right_bar;            /*    64     2 */

	/* size: 68, cachelines: 2, members: 20 */
	/* sum members: 57, holes: 4, sum holes: 9 */
	/* padding: 2 */
	/* last cacheline: 4 bytes */
};
struct hdmi_drm_infoframe {
	enum hdmi_infoframe_type   type;                 /*     0     4 */
	unsigned char              version;              /*     4     1 */
	unsigned char              length;               /*     5     1 */

	/* XXX 2 bytes hole, try to pack */

	enum hdmi_eotf             eotf;                 /*     8     4 */
	enum hdmi_metadata_type    metadata_type;        /*    12     4 */
	struct {
		u16                x;                    /*    16     2 */
		u16                y;                    /*    18     2 */
	} display_primaries[3]; /*    16    12 */
	struct {
		u16                x;                    /*    28     2 */
		u16                y;                    /*    30     2 */
	} white_point;                                   /*    28     4 */
	u16                        max_display_mastering_luminance; /*    32     2 */
	u16                        min_display_mastering_luminance; /*    34     2 */
	u16                        max_cll;              /*    36     2 */
	u16                        max_fall;             /*    38     2 */

	/* size: 40, cachelines: 1, members: 11 */
	/* sum members: 38, holes: 1, sum holes: 2 */
	/* last cacheline: 40 bytes */
};
struct hdmi_spd_infoframe {
	enum hdmi_infoframe_type   type;                 /*     0     4 */
	unsigned char              version;              /*     4     1 */
	unsigned char              length;               /*     5     1 */
	char                       vendor[8];            /*     6     8 */
	char                       product[16];          /*    14    16 */

	/* XXX 2 bytes hole, try to pack */

	enum hdmi_spd_sdi          sdi;                  /*    32     4 */

	/* size: 36, cachelines: 1, members: 6 */
	/* sum members: 34, holes: 1, sum holes: 2 */
	/* last cacheline: 36 bytes */
};
struct hdmi_audio_infoframe {
	enum hdmi_infoframe_type   type;                 /*     0     4 */
	unsigned char              version;              /*     4     1 */
	unsigned char              length;               /*     5     1 */
	unsigned char              channels;             /*     6     1 */

	/* XXX 1 byte hole, try to pack */

	enum hdmi_audio_coding_type coding_type;         /*     8     4 */
	enum hdmi_audio_sample_size sample_size;         /*    12     4 */
	enum hdmi_audio_sample_frequency sample_frequency; /*    16     4 */
	enum hdmi_audio_coding_type_ext coding_type_ext; /*    20     4 */
	unsigned char              channel_allocation;   /*    24     1 */
	unsigned char              level_shift_value;    /*    25     1 */
	bool                       downmix_inhibit;      /*    26     1 */

	/* size: 28, cachelines: 1, members: 11 */
	/* sum members: 26, holes: 1, sum holes: 1 */
	/* padding: 1 */
	/* last cacheline: 28 bytes */
};
struct hdmi_vendor_infoframe {
	enum hdmi_infoframe_type   type;                 /*     0     4 */
	unsigned char              version;              /*     4     1 */
	unsigned char              length;               /*     5     1 */

	/* XXX 2 bytes hole, try to pack */

	unsigned int               oui;                  /*     8     4 */
	u8                         vic;                  /*    12     1 */

	/* XXX 3 bytes hole, try to pack */

	enum hdmi_3d_structure     s3d_struct;           /*    16     4 */
	unsigned int               s3d_ext_data;         /*    20     4 */

	/* size: 24, cachelines: 1, members: 7 */
	/* sum members: 19, holes: 2, sum holes: 5 */
	/* last cacheline: 24 bytes */
};
union hdmi_vendor_any_infoframe {
	struct {
		enum hdmi_infoframe_type type;         /*     0     4 */
		unsigned char      version;            /*     4     1 */
		unsigned char      length;             /*     5     1 */

		/* XXX 2 bytes hole, try to pack */

		unsigned int       oui;                /*     8     4 */
	} any;                                         /*     0    12 */
	struct hdmi_vendor_infoframe hdmi;             /*     0    24 */
};
union hdmi_infoframe {
	struct hdmi_any_infoframe  any;                /*     0     8 */
	struct hdmi_avi_infoframe  avi;                /*     0    68 */
	struct hdmi_spd_infoframe  spd;                /*     0    36 */
	union hdmi_vendor_any_infoframe vendor;        /*     0    24 */
	struct hdmi_audio_infoframe audio;             /*     0    28 */
	struct hdmi_drm_infoframe  drm;                /*     0    40 */
};
struct vc_data {
	struct tty_port            port;                 /*     0   384 */
	/* --- cacheline 6 boundary (384 bytes) --- */
	struct vc_state            state;                /*   384    20 */
	struct vc_state            saved_state;          /*   404    20 */
	short unsigned int         vc_num;               /*   424     2 */

	/* XXX 2 bytes hole, try to pack */

	unsigned int               vc_cols;              /*   428     4 */
	unsigned int               vc_rows;              /*   432     4 */
	unsigned int               vc_size_row;          /*   436     4 */
	unsigned int               vc_scan_lines;        /*   440     4 */
	unsigned int               vc_cell_height;       /*   444     4 */
	/* --- cacheline 7 boundary (448 bytes) --- */
	long unsigned int          vc_origin;            /*   448     8 */
	long unsigned int          vc_scr_end;           /*   456     8 */
	long unsigned int          vc_visible_origin;    /*   464     8 */
	unsigned int               vc_top;               /*   472     4 */
	unsigned int               vc_bottom;            /*   476     4 */
	const struct consw  *      vc_sw;                /*   480     8 */
	short unsigned int *       vc_screenbuf;         /*   488     8 */
	unsigned int               vc_screenbuf_size;    /*   496     4 */
	unsigned char              vc_mode;              /*   500     1 */
	unsigned char              vc_attr;              /*   501     1 */
	unsigned char              vc_def_color;         /*   502     1 */
	unsigned char              vc_ulcolor;           /*   503     1 */
	unsigned char              vc_itcolor;           /*   504     1 */
	unsigned char              vc_halfcolor;         /*   505     1 */

	/* XXX 2 bytes hole, try to pack */

	unsigned int               vc_cursor_type;       /*   508     4 */
	/* --- cacheline 8 boundary (512 bytes) --- */
	short unsigned int         vc_complement_mask;   /*   512     2 */
	short unsigned int         vc_s_complement_mask; /*   514     2 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          vc_pos;               /*   520     8 */
	short unsigned int         vc_hi_font_mask;      /*   528     2 */

	/* XXX 6 bytes hole, try to pack */

	struct console_font        vc_font;              /*   536    24 */
	short unsigned int         vc_video_erase_char;  /*   560     2 */

	/* XXX 2 bytes hole, try to pack */

	unsigned int               vc_state;             /*   564     4 */
	unsigned int               vc_npar;              /*   568     4 */
	unsigned int               vc_par[16];           /*   572    64 */
	/* --- cacheline 9 boundary (576 bytes) was 60 bytes ago --- */
	struct vt_mode             vt_mode;              /*   636     8 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 10 boundary (640 bytes) was 8 bytes ago --- */
	struct pid *               vt_pid;               /*   648     8 */
	int                        vt_newvt;             /*   656     4 */

	/* XXX 4 bytes hole, try to pack */

	wait_queue_head_t          paste_wait;           /*   664    24 */
	unsigned int               vc_disp_ctrl:1;       /*   688: 0  4 */
	unsigned int               vc_toggle_meta:1;     /*   688: 1  4 */
	unsigned int               vc_decscnm:1;         /*   688: 2  4 */
	unsigned int               vc_decom:1;           /*   688: 3  4 */
	unsigned int               vc_decawm:1;          /*   688: 4  4 */
	unsigned int               vc_deccm:1;           /*   688: 5  4 */
	unsigned int               vc_decim:1;           /*   688: 6  4 */
	unsigned int               vc_priv:3;            /*   688: 7  4 */
	unsigned int               vc_need_wrap:1;       /*   688:10  4 */
	unsigned int               vc_can_do_color:1;    /*   688:11  4 */
	unsigned int               vc_report_mouse:2;    /*   688:12  4 */

	/* Bitfield combined with next fields */

	unsigned char              vc_utf:1;             /*   689: 6  1 */

	/* XXX 1 bit hole, try to pack */

	unsigned char              vc_utf_count;         /*   690     1 */

	/* XXX 1 byte hole, try to pack */

	int                        vc_utf_char;          /*   692     4 */
	long unsigned int          vc_tab_stop[4];       /*   696    32 */
	/* --- cacheline 11 boundary (704 bytes) was 24 bytes ago --- */
	unsigned char              vc_palette[48];       /*   728    48 */
	/* --- cacheline 12 boundary (768 bytes) was 8 bytes ago --- */
	short unsigned int *       vc_translate;         /*   776     8 */
	unsigned int               vc_resize_user;       /*   784     4 */
	unsigned int               vc_bell_pitch;        /*   788     4 */
	unsigned int               vc_bell_duration;     /*   792     4 */
	short unsigned int         vc_cur_blink_ms;      /*   796     2 */

	/* XXX 2 bytes hole, try to pack */

	struct vc_data * *         vc_display_fg;        /*   800     8 */
	struct uni_pagedir *       vc_uni_pagedir;       /*   808     8 */
	struct uni_pagedir * *     vc_uni_pagedir_loc;   /*   816     8 */
	struct uni_screen *        vc_uni_screen;        /*   824     8 */

	/* size: 832, cachelines: 13, members: 62 */
	/* sum members: 803, holes: 9, sum holes: 27 */
	/* sum bitfield members: 15 bits, bit holes: 1, sum bit holes: 1 bits */
};
struct console_font {
	unsigned int               width;                /*     0     4 */
	unsigned int               height;               /*     4     4 */
	unsigned int               charcount;            /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	unsigned char *            data;                 /*    16     8 */

	/* size: 24, cachelines: 1, members: 4 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct vt_mode {
	char                       mode;                 /*     0     1 */
	char                       waitv;                /*     1     1 */
	short int                  relsig;               /*     2     2 */
	short int                  acqsig;               /*     4     2 */
	short int                  frsig;                /*     6     2 */

	/* size: 8, cachelines: 1, members: 5 */
	/* last cacheline: 8 bytes */
};
struct vc_state {
	unsigned int               x;                    /*     0     4 */
	unsigned int               y;                    /*     4     4 */
	unsigned char              color;                /*     8     1 */
	unsigned char              Gx_charset[2];        /*     9     2 */

	/* Bitfield combined with previous fields */

	unsigned int               charset:1;            /*     8:24  4 */

	/* XXX 7 bits hole, try to pack */

	enum vc_intensity          intensity;            /*    12     4 */
	bool                       italic;               /*    16     1 */
	bool                       underline;            /*    17     1 */
	bool                       blink;                /*    18     1 */
	bool                       reverse;              /*    19     1 */

	/* size: 20, cachelines: 1, members: 10 */
	/* sum members: 19 */
	/* sum bitfield members: 1 bits, bit holes: 1, sum bit holes: 7 bits */
	/* last cacheline: 20 bytes */
};
struct vc {
	struct vc_data *           d;                    /*     0     8 */
	struct work_struct         SAK_work;             /*     8    64 */

	/* size: 72, cachelines: 2, members: 2 */
	/* last cacheline: 8 bytes */
};
struct vgastate {
	void *                     vgabase;              /*     0     8 */
	long unsigned int          membase;              /*     8     8 */
	__u32                      memsize;              /*    16     4 */
	__u32                      flags;                /*    20     4 */
	__u32                      depth;                /*    24     4 */
	__u32                      num_attr;             /*    28     4 */
	__u32                      num_crtc;             /*    32     4 */
	__u32                      num_gfx;              /*    36     4 */
	__u32                      num_seq;              /*    40     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     vidstate;             /*    48     8 */

	/* size: 56, cachelines: 1, members: 10 */
	/* sum members: 52, holes: 1, sum holes: 4 */
	/* last cacheline: 56 bytes */
};
struct linux_logo {
	int                        type;                 /*     0     4 */
	unsigned int               width;                /*     4     4 */
	unsigned int               height;               /*     8     4 */
	unsigned int               clutsize;             /*    12     4 */
	const unsigned char  *     clut;                 /*    16     8 */
	const unsigned char  *     data;                 /*    24     8 */

	/* size: 32, cachelines: 1, members: 6 */
	/* last cacheline: 32 bytes */
};
struct fb_fix_screeninfo {
	char                       id[16];               /*     0    16 */
	long unsigned int          smem_start;           /*    16     8 */
	__u32                      smem_len;             /*    24     4 */
	__u32                      type;                 /*    28     4 */
	__u32                      type_aux;             /*    32     4 */
	__u32                      visual;               /*    36     4 */
	__u16                      xpanstep;             /*    40     2 */
	__u16                      ypanstep;             /*    42     2 */
	__u16                      ywrapstep;            /*    44     2 */

	/* XXX 2 bytes hole, try to pack */

	__u32                      line_length;          /*    48     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          mmio_start;           /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	__u32                      mmio_len;             /*    64     4 */
	__u32                      accel;                /*    68     4 */
	__u16                      capabilities;         /*    72     2 */
	__u16                      reserved[2];          /*    74     4 */

	/* size: 80, cachelines: 2, members: 15 */
	/* sum members: 72, holes: 2, sum holes: 6 */
	/* padding: 2 */
	/* last cacheline: 16 bytes */
};
struct fb_bitfield {
	__u32                      offset;               /*     0     4 */
	__u32                      length;               /*     4     4 */
	__u32                      msb_right;            /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct fb_var_screeninfo {
	__u32                      xres;                 /*     0     4 */
	__u32                      yres;                 /*     4     4 */
	__u32                      xres_virtual;         /*     8     4 */
	__u32                      yres_virtual;         /*    12     4 */
	__u32                      xoffset;              /*    16     4 */
	__u32                      yoffset;              /*    20     4 */
	__u32                      bits_per_pixel;       /*    24     4 */
	__u32                      grayscale;            /*    28     4 */
	struct fb_bitfield         red;                  /*    32    12 */
	struct fb_bitfield         green;                /*    44    12 */
	struct fb_bitfield         blue;                 /*    56    12 */
	/* --- cacheline 1 boundary (64 bytes) was 4 bytes ago --- */
	struct fb_bitfield         transp;               /*    68    12 */
	__u32                      nonstd;               /*    80     4 */
	__u32                      activate;             /*    84     4 */
	__u32                      height;               /*    88     4 */
	__u32                      width;                /*    92     4 */
	__u32                      accel_flags;          /*    96     4 */
	__u32                      pixclock;             /*   100     4 */
	__u32                      left_margin;          /*   104     4 */
	__u32                      right_margin;         /*   108     4 */
	__u32                      upper_margin;         /*   112     4 */
	__u32                      lower_margin;         /*   116     4 */
	__u32                      hsync_len;            /*   120     4 */
	__u32                      vsync_len;            /*   124     4 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	__u32                      sync;                 /*   128     4 */
	__u32                      vmode;                /*   132     4 */
	__u32                      rotate;               /*   136     4 */
	__u32                      colorspace;           /*   140     4 */
	__u32                      reserved[4];          /*   144    16 */

	/* size: 160, cachelines: 3, members: 29 */
	/* last cacheline: 32 bytes */
};
struct fb_cmap {
	__u32                      start;                /*     0     4 */
	__u32                      len;                  /*     4     4 */
	__u16 *                    red;                  /*     8     8 */
	__u16 *                    green;                /*    16     8 */
	__u16 *                    blue;                 /*    24     8 */
	__u16 *                    transp;               /*    32     8 */

	/* size: 40, cachelines: 1, members: 6 */
	/* last cacheline: 40 bytes */
};
struct fb_copyarea {
	__u32                      dx;                   /*     0     4 */
	__u32                      dy;                   /*     4     4 */
	__u32                      width;                /*     8     4 */
	__u32                      height;               /*    12     4 */
	__u32                      sx;                   /*    16     4 */
	__u32                      sy;                   /*    20     4 */

	/* size: 24, cachelines: 1, members: 6 */
	/* last cacheline: 24 bytes */
};
struct fb_fillrect {
	__u32                      dx;                   /*     0     4 */
	__u32                      dy;                   /*     4     4 */
	__u32                      width;                /*     8     4 */
	__u32                      height;               /*    12     4 */
	__u32                      color;                /*    16     4 */
	__u32                      rop;                  /*    20     4 */

	/* size: 24, cachelines: 1, members: 6 */
	/* last cacheline: 24 bytes */
};
struct fb_image {
	__u32                      dx;                   /*     0     4 */
	__u32                      dy;                   /*     4     4 */
	__u32                      width;                /*     8     4 */
	__u32                      height;               /*    12     4 */
	__u32                      fg_color;             /*    16     4 */
	__u32                      bg_color;             /*    20     4 */
	__u8                       depth;                /*    24     1 */

	/* XXX 7 bytes hole, try to pack */

	const char  *              data;                 /*    32     8 */
	struct fb_cmap             cmap;                 /*    40    40 */

	/* size: 80, cachelines: 2, members: 9 */
	/* sum members: 73, holes: 1, sum holes: 7 */
	/* last cacheline: 16 bytes */
};
struct fbcurpos {
	__u16                      x;                    /*     0     2 */
	__u16                      y;                    /*     2     2 */

	/* size: 4, cachelines: 1, members: 2 */
	/* last cacheline: 4 bytes */
};
struct fb_cursor {
	__u16                      set;                  /*     0     2 */
	__u16                      enable;               /*     2     2 */
	__u16                      rop;                  /*     4     2 */

	/* XXX 2 bytes hole, try to pack */

	const char  *              mask;                 /*     8     8 */
	struct fbcurpos            hot;                  /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	struct fb_image            image;                /*    24    80 */

	/* size: 104, cachelines: 2, members: 6 */
	/* sum members: 98, holes: 2, sum holes: 6 */
	/* last cacheline: 40 bytes */
};
struct fb_chroma {
	__u32                      redx;                 /*     0     4 */
	__u32                      greenx;               /*     4     4 */
	__u32                      bluex;                /*     8     4 */
	__u32                      whitex;               /*    12     4 */
	__u32                      redy;                 /*    16     4 */
	__u32                      greeny;               /*    20     4 */
	__u32                      bluey;                /*    24     4 */
	__u32                      whitey;               /*    28     4 */

	/* size: 32, cachelines: 1, members: 8 */
	/* last cacheline: 32 bytes */
};
struct fb_monspecs {
	struct fb_chroma           chroma;               /*     0    32 */
	struct fb_videomode *      modedb;               /*    32     8 */
	__u8                       manufacturer[4];      /*    40     4 */
	__u8                       monitor[14];          /*    44    14 */
	__u8                       serial_no[14];        /*    58    14 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	__u8                       ascii[14];            /*    72    14 */

	/* XXX 2 bytes hole, try to pack */

	__u32                      modedb_len;           /*    88     4 */
	__u32                      model;                /*    92     4 */
	__u32                      serial;               /*    96     4 */
	__u32                      year;                 /*   100     4 */
	__u32                      week;                 /*   104     4 */
	__u32                      hfmin;                /*   108     4 */
	__u32                      hfmax;                /*   112     4 */
	__u32                      dclkmin;              /*   116     4 */
	__u32                      dclkmax;              /*   120     4 */
	__u16                      input;                /*   124     2 */
	__u16                      dpms;                 /*   126     2 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	__u16                      signal;               /*   128     2 */
	__u16                      vfmin;                /*   130     2 */
	__u16                      vfmax;                /*   132     2 */
	__u16                      gamma;                /*   134     2 */
	__u16                      gtf:1;                /*   136: 0  2 */

	/* XXX 15 bits hole, try to pack */

	__u16                      misc;                 /*   138     2 */
	__u8                       version;              /*   140     1 */
	__u8                       revision;             /*   141     1 */
	__u8                       max_x;                /*   142     1 */
	__u8                       max_y;                /*   143     1 */

	/* size: 144, cachelines: 3, members: 27 */
	/* sum members: 140, holes: 1, sum holes: 2 */
	/* sum bitfield members: 1 bits, bit holes: 1, sum bit holes: 15 bits */
	/* last cacheline: 16 bytes */
};
struct fb_videomode {
	const char  *              name;                 /*     0     8 */
	u32                        refresh;              /*     8     4 */
	u32                        xres;                 /*    12     4 */
	u32                        yres;                 /*    16     4 */
	u32                        pixclock;             /*    20     4 */
	u32                        left_margin;          /*    24     4 */
	u32                        right_margin;         /*    28     4 */
	u32                        upper_margin;         /*    32     4 */
	u32                        lower_margin;         /*    36     4 */
	u32                        hsync_len;            /*    40     4 */
	u32                        vsync_len;            /*    44     4 */
	u32                        sync;                 /*    48     4 */
	u32                        vmode;                /*    52     4 */
	u32                        flag;                 /*    56     4 */

	/* size: 64, cachelines: 1, members: 14 */
	/* padding: 4 */
};
struct fb_event {
	struct fb_info *           info;                 /*     0     8 */
	void *                     data;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct fb_info {
	atomic_t                   count;                /*     0     4 */
	int                        node;                 /*     4     4 */
	int                        flags;                /*     8     4 */
	int                        fbcon_rotate_hint;    /*    12     4 */
	struct mutex               lock;                 /*    16    32 */
	struct mutex               mm_lock;              /*    48    32 */
	/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
	struct fb_var_screeninfo   var;                  /*    80   160 */
	/* --- cacheline 3 boundary (192 bytes) was 48 bytes ago --- */
	struct fb_fix_screeninfo   fix;                  /*   240    80 */

	/* XXX last struct has 2 bytes of padding */

	/* --- cacheline 5 boundary (320 bytes) --- */
	struct fb_monspecs         monspecs;             /*   320   144 */
	/* --- cacheline 7 boundary (448 bytes) was 16 bytes ago --- */
	struct work_struct         queue;                /*   464    64 */
	/* --- cacheline 8 boundary (512 bytes) was 16 bytes ago --- */
	struct fb_pixmap           pixmap;               /*   528    56 */
	/* --- cacheline 9 boundary (576 bytes) was 8 bytes ago --- */
	struct fb_pixmap           sprite;               /*   584    56 */
	/* --- cacheline 10 boundary (640 bytes) --- */
	struct fb_cmap             cmap;                 /*   640    40 */
	struct list_head           modelist;             /*   680    16 */
	struct fb_videomode *      mode;                 /*   696     8 */
	/* --- cacheline 11 boundary (704 bytes) --- */
	struct delayed_work        deferred_work;        /*   704   192 */
	/* --- cacheline 14 boundary (896 bytes) --- */
	struct fb_deferred_io *    fbdefio;              /*   896     8 */
	const struct fb_ops  *     fbops;                /*   904     8 */
	struct device *            device;               /*   912     8 */
	struct device *            dev;                  /*   920     8 */
	int                        class_flag;           /*   928     4 */

	/* XXX 4 bytes hole, try to pack */

	struct fb_tile_ops *       tileops;              /*   936     8 */
	union {
		char *             screen_base;          /*   944     8 */
		char *             screen_buffer;        /*   944     8 */
	};                                               /*   944     8 */
	long unsigned int          screen_size;          /*   952     8 */
	/* --- cacheline 15 boundary (960 bytes) --- */
	void *                     pseudo_palette;       /*   960     8 */
	u32                        state;                /*   968     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     fbcon_par;            /*   976     8 */
	void *                     par;                  /*   984     8 */
	struct apertures_struct *  apertures;            /*   992     8 */
	bool                       skip_vt_switch;       /*  1000     1 */

	/* size: 1008, cachelines: 16, members: 30 */
	/* sum members: 993, holes: 2, sum holes: 8 */
	/* padding: 7 */
	/* paddings: 1, sum paddings: 2 */
	/* last cacheline: 48 bytes */
};
struct fb_blit_caps {
	u32                        x;                    /*     0     4 */
	u32                        y;                    /*     4     4 */
	u32                        len;                  /*     8     4 */
	u32                        flags;                /*    12     4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct fb_pixmap {
	u8 *                       addr;                 /*     0     8 */
	u32                        size;                 /*     8     4 */
	u32                        offset;               /*    12     4 */
	u32                        buf_align;            /*    16     4 */
	u32                        scan_align;           /*    20     4 */
	u32                        access_align;         /*    24     4 */
	u32                        flags;                /*    28     4 */
	u32                        blit_x;               /*    32     4 */
	u32                        blit_y;               /*    36     4 */
	void                       (*writeio)(struct fb_info *, void *, void *, unsigned int); /*    40     8 */
	void                       (*readio)(struct fb_info *, void *, void *, unsigned int); /*    48     8 */

	/* size: 56, cachelines: 1, members: 11 */
	/* last cacheline: 56 bytes */
};
struct fb_deferred_io {
	long unsigned int          delay;                /*     0     8 */
	struct mutex               lock;                 /*     8    32 */
	struct list_head           pagelist;             /*    40    16 */
	void                       (*first_io)(struct fb_info *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	void                       (*deferred_io)(struct fb_info *, struct list_head *); /*    64     8 */

	/* size: 72, cachelines: 2, members: 5 */
	/* last cacheline: 8 bytes */
};
struct fb_ops {
	struct module *            owner;                /*     0     8 */
	int                        (*fb_open)(struct fb_info *, int); /*     8     8 */
	int                        (*fb_release)(struct fb_info *, int); /*    16     8 */
	ssize_t                    (*fb_read)(struct fb_info *, char *, size_t, loff_t *); /*    24     8 */
	ssize_t                    (*fb_write)(struct fb_info *, const char  *, size_t, loff_t *); /*    32     8 */
	int                        (*fb_check_var)(struct fb_var_screeninfo *, struct fb_info *); /*    40     8 */
	int                        (*fb_set_par)(struct fb_info *); /*    48     8 */
	int                        (*fb_setcolreg)(unsigned int, unsigned int, unsigned int, unsigned int, unsigned int, struct fb_info *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*fb_setcmap)(struct fb_cmap *, struct fb_info *); /*    64     8 */
	int                        (*fb_blank)(int, struct fb_info *); /*    72     8 */
	int                        (*fb_pan_display)(struct fb_var_screeninfo *, struct fb_info *); /*    80     8 */
	void                       (*fb_fillrect)(struct fb_info *, const struct fb_fillrect  *); /*    88     8 */
	void                       (*fb_copyarea)(struct fb_info *, const struct fb_copyarea  *); /*    96     8 */
	void                       (*fb_imageblit)(struct fb_info *, const struct fb_image  *); /*   104     8 */
	int                        (*fb_cursor)(struct fb_info *, struct fb_cursor *); /*   112     8 */
	int                        (*fb_sync)(struct fb_info *); /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	int                        (*fb_ioctl)(struct fb_info *, unsigned int, long unsigned int); /*   128     8 */
	int                        (*fb_compat_ioctl)(struct fb_info *, unsigned int, long unsigned int); /*   136     8 */
	int                        (*fb_mmap)(struct fb_info *, struct vm_area_struct *); /*   144     8 */
	void                       (*fb_get_caps)(struct fb_info *, struct fb_blit_caps *, struct fb_var_screeninfo *); /*   152     8 */
	void                       (*fb_destroy)(struct fb_info *); /*   160     8 */
	int                        (*fb_debug_enter)(struct fb_info *); /*   168     8 */
	int                        (*fb_debug_leave)(struct fb_info *); /*   176     8 */

	/* size: 184, cachelines: 3, members: 23 */
	/* last cacheline: 56 bytes */
};
struct fb_tilemap {
	__u32                      width;                /*     0     4 */
	__u32                      height;               /*     4     4 */
	__u32                      depth;                /*     8     4 */
	__u32                      length;               /*    12     4 */
	const __u8  *              data;                 /*    16     8 */

	/* size: 24, cachelines: 1, members: 5 */
	/* last cacheline: 24 bytes */
};
struct fb_tilerect {
	__u32                      sx;                   /*     0     4 */
	__u32                      sy;                   /*     4     4 */
	__u32                      width;                /*     8     4 */
	__u32                      height;               /*    12     4 */
	__u32                      index;                /*    16     4 */
	__u32                      fg;                   /*    20     4 */
	__u32                      bg;                   /*    24     4 */
	__u32                      rop;                  /*    28     4 */

	/* size: 32, cachelines: 1, members: 8 */
	/* last cacheline: 32 bytes */
};
struct fb_tilearea {
	__u32                      sx;                   /*     0     4 */
	__u32                      sy;                   /*     4     4 */
	__u32                      dx;                   /*     8     4 */
	__u32                      dy;                   /*    12     4 */
	__u32                      width;                /*    16     4 */
	__u32                      height;               /*    20     4 */

	/* size: 24, cachelines: 1, members: 6 */
	/* last cacheline: 24 bytes */
};
struct fb_tileblit {
	__u32                      sx;                   /*     0     4 */
	__u32                      sy;                   /*     4     4 */
	__u32                      width;                /*     8     4 */
	__u32                      height;               /*    12     4 */
	__u32                      fg;                   /*    16     4 */
	__u32                      bg;                   /*    20     4 */
	__u32                      length;               /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	__u32 *                    indices;              /*    32     8 */

	/* size: 40, cachelines: 1, members: 8 */
	/* sum members: 36, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct fb_tilecursor {
	__u32                      sx;                   /*     0     4 */
	__u32                      sy;                   /*     4     4 */
	__u32                      mode;                 /*     8     4 */
	__u32                      shape;                /*    12     4 */
	__u32                      fg;                   /*    16     4 */
	__u32                      bg;                   /*    20     4 */

	/* size: 24, cachelines: 1, members: 6 */
	/* last cacheline: 24 bytes */
};
struct fb_tile_ops {
	void                       (*fb_settile)(struct fb_info *, struct fb_tilemap *); /*     0     8 */
	void                       (*fb_tilecopy)(struct fb_info *, struct fb_tilearea *); /*     8     8 */
	void                       (*fb_tilefill)(struct fb_info *, struct fb_tilerect *); /*    16     8 */
	void                       (*fb_tileblit)(struct fb_info *, struct fb_tileblit *); /*    24     8 */
	void                       (*fb_tilecursor)(struct fb_info *, struct fb_tilecursor *); /*    32     8 */
	int                        (*fb_get_tilemax)(struct fb_info *); /*    40     8 */

	/* size: 48, cachelines: 1, members: 6 */
	/* last cacheline: 48 bytes */
};
struct aperture {
	resource_size_t            base;                 /*     0     8 */
	resource_size_t            size;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct apertures_struct {
	unsigned int               count;                /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct aperture            ranges[];             /*     8     0 */

	/* size: 8, cachelines: 1, members: 2 */
	/* sum members: 4, holes: 1, sum holes: 4 */
	/* last cacheline: 8 bytes */
};
struct backlight_ops {
	unsigned int               options;              /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	int                        (*update_status)(struct backlight_device *); /*     8     8 */
	int                        (*get_brightness)(struct backlight_device *); /*    16     8 */
	int                        (*check_fb)(struct backlight_device *, struct fb_info *); /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct backlight_device {
	struct backlight_properties props;               /*     0    28 */

	/* XXX 4 bytes hole, try to pack */

	struct mutex               update_lock;          /*    32    32 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct mutex               ops_lock;             /*    64    32 */
	const struct backlight_ops  * ops;               /*    96     8 */
	struct notifier_block      fb_notif;             /*   104    24 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 2 boundary (128 bytes) --- */
	struct list_head           entry;                /*   128    16 */
	struct device              dev __attribute__((__aligned__(8))); /*   144  1000 */
	/* --- cacheline 17 boundary (1088 bytes) was 56 bytes ago --- */
	bool                       fb_bl_on[32];         /*  1144    32 */
	/* --- cacheline 18 boundary (1152 bytes) was 24 bytes ago --- */
	int                        use_count;            /*  1176     4 */

	/* size: 1184, cachelines: 19, members: 9 */
	/* sum members: 1176, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* paddings: 1, sum paddings: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct backlight_properties {
	int                        brightness;           /*     0     4 */
	int                        max_brightness;       /*     4     4 */
	int                        power;                /*     8     4 */
	int                        fb_blank;             /*    12     4 */
	enum backlight_type        type;                 /*    16     4 */
	unsigned int               state;                /*    20     4 */
	enum backlight_scale       scale;                /*    24     4 */

	/* size: 28, cachelines: 1, members: 7 */
	/* last cacheline: 28 bytes */
};
struct fb_cmap_user {
	__u32                      start;                /*     0     4 */
	__u32                      len;                  /*     4     4 */
	__u16 *                    red;                  /*     8     8 */
	__u16 *                    green;                /*    16     8 */
	__u16 *                    blue;                 /*    24     8 */
	__u16 *                    transp;               /*    32     8 */

	/* size: 40, cachelines: 1, members: 6 */
	/* last cacheline: 40 bytes */
};
struct fb_modelist {
	struct list_head           list;                 /*     0    16 */
	struct fb_videomode        mode;                 /*    16    64 */

	/* XXX last struct has 4 bytes of padding */

	/* size: 80, cachelines: 2, members: 2 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 16 bytes */
};
struct logo_data {
	int                        depth;                /*     0     4 */
	int                        needs_directpalette;  /*     4     4 */
	int                        needs_truepalette;    /*     8     4 */
	int                        needs_cmapreset;      /*    12     4 */
	const struct linux_logo  * logo;                 /*    16     8 */

	/* size: 24, cachelines: 1, members: 5 */
	/* last cacheline: 24 bytes */
};
struct fb_fix_screeninfo32 {
	char                       id[16];               /*     0    16 */
	compat_caddr_t             smem_start;           /*    16     4 */
	u32                        smem_len;             /*    20     4 */
	u32                        type;                 /*    24     4 */
	u32                        type_aux;             /*    28     4 */
	u32                        visual;               /*    32     4 */
	u16                        xpanstep;             /*    36     2 */
	u16                        ypanstep;             /*    38     2 */
	u16                        ywrapstep;            /*    40     2 */

	/* XXX 2 bytes hole, try to pack */

	u32                        line_length;          /*    44     4 */
	compat_caddr_t             mmio_start;           /*    48     4 */
	u32                        mmio_len;             /*    52     4 */
	u32                        accel;                /*    56     4 */
	u16                        reserved[3];          /*    60     6 */

	/* size: 68, cachelines: 2, members: 14 */
	/* sum members: 64, holes: 1, sum holes: 2 */
	/* padding: 2 */
	/* last cacheline: 4 bytes */
};
struct fb_cmap32 {
	u32                        start;                /*     0     4 */
	u32                        len;                  /*     4     4 */
	compat_caddr_t             red;                  /*     8     4 */
	compat_caddr_t             green;                /*    12     4 */
	compat_caddr_t             blue;                 /*    16     4 */
	compat_caddr_t             transp;               /*    20     4 */

	/* size: 24, cachelines: 1, members: 6 */
	/* last cacheline: 24 bytes */
};
struct fb_cvt_data {
	u32                        xres;                 /*     0     4 */
	u32                        yres;                 /*     4     4 */
	u32                        refresh;              /*     8     4 */
	u32                        f_refresh;            /*    12     4 */
	u32                        pixclock;             /*    16     4 */
	u32                        hperiod;              /*    20     4 */
	u32                        hblank;               /*    24     4 */
	u32                        hfreq;                /*    28     4 */
	u32                        htotal;               /*    32     4 */
	u32                        vtotal;               /*    36     4 */
	u32                        vsync;                /*    40     4 */
	u32                        hsync;                /*    44     4 */
	u32                        h_front_porch;        /*    48     4 */
	u32                        h_back_porch;         /*    52     4 */
	u32                        v_front_porch;        /*    56     4 */
	u32                        v_back_porch;         /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u32                        h_margin;             /*    64     4 */
	u32                        v_margin;             /*    68     4 */
	u32                        interlace;            /*    72     4 */
	u32                        aspect_ratio;         /*    76     4 */
	u32                        active_pixels;        /*    80     4 */
	u32                        flags;                /*    84     4 */
	u32                        status;               /*    88     4 */

	/* size: 92, cachelines: 2, members: 23 */
	/* last cacheline: 28 bytes */
};
struct fb_con2fbmap {
	__u32                      console;              /*     0     4 */
	__u32                      framebuffer;          /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct fbcon_display {
	const u_char  *            fontdata;             /*     0     8 */
	int                        userfont;             /*     8     4 */
	u_short                    inverse;              /*    12     2 */
	short int                  yscroll;              /*    14     2 */
	int                        vrows;                /*    16     4 */
	int                        cursor_shape;         /*    20     4 */
	int                        con_rotate;           /*    24     4 */
	u32                        xres_virtual;         /*    28     4 */
	u32                        yres_virtual;         /*    32     4 */
	u32                        height;               /*    36     4 */
	u32                        width;                /*    40     4 */
	u32                        bits_per_pixel;       /*    44     4 */
	u32                        grayscale;            /*    48     4 */
	u32                        nonstd;               /*    52     4 */
	u32                        accel_flags;          /*    56     4 */
	u32                        rotate;               /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct fb_bitfield         red;                  /*    64    12 */
	struct fb_bitfield         green;                /*    76    12 */
	struct fb_bitfield         blue;                 /*    88    12 */
	struct fb_bitfield         transp;               /*   100    12 */
	const struct fb_videomode  * mode;               /*   112     8 */

	/* size: 120, cachelines: 2, members: 21 */
	/* last cacheline: 56 bytes */
};
struct fbcon_ops {
	void                       (*bmove)(struct vc_data *, struct fb_info *, int, int, int, int, int, int); /*     0     8 */
	void                       (*clear)(struct vc_data *, struct fb_info *, int, int, int, int); /*     8     8 */
	void                       (*putcs)(struct vc_data *, struct fb_info *, const short unsigned int  *, int, int, int, int, int); /*    16     8 */
	void                       (*clear_margins)(struct vc_data *, struct fb_info *, int, int); /*    24     8 */
	void                       (*cursor)(struct vc_data *, struct fb_info *, int, int, int); /*    32     8 */
	int                        (*update_start)(struct fb_info *); /*    40     8 */
	int                        (*rotate_font)(struct fb_info *, struct vc_data *); /*    48     8 */
	struct fb_var_screeninfo   var;                  /*    56   160 */
	/* --- cacheline 3 boundary (192 bytes) was 24 bytes ago --- */
	struct timer_list          cursor_timer;         /*   216    72 */
	/* --- cacheline 4 boundary (256 bytes) was 32 bytes ago --- */
	struct fb_cursor           cursor_state;         /*   288   104 */
	/* --- cacheline 6 boundary (384 bytes) was 8 bytes ago --- */
	struct fbcon_display *     p;                    /*   392     8 */
	struct fb_info *           info;                 /*   400     8 */
	int                        currcon;              /*   408     4 */
	int                        cur_blink_jiffies;    /*   412     4 */
	int                        cursor_flash;         /*   416     4 */
	int                        cursor_reset;         /*   420     4 */
	int                        blank_state;          /*   424     4 */
	int                        graphics;             /*   428     4 */
	int                        save_graphics;        /*   432     4 */
	int                        flags;                /*   436     4 */
	int                        rotate;               /*   440     4 */
	int                        cur_rotate;           /*   444     4 */
	/* --- cacheline 7 boundary (448 bytes) --- */
	char *                     cursor_data;          /*   448     8 */
	u8 *                       fontbuffer;           /*   456     8 */
	u8 *                       fontdata;             /*   464     8 */
	u8 *                       cursor_src;           /*   472     8 */
	u32                        cursor_size;          /*   480     4 */
	u32                        fd_size;              /*   484     4 */

	/* size: 488, cachelines: 8, members: 28 */
	/* last cacheline: 40 bytes */
};
struct vesafb_par {
	u32                        pseudo_palette[256];  /*     0  1024 */
	/* --- cacheline 16 boundary (1024 bytes) --- */
	int                        wc_cookie;            /*  1024     4 */

	/* XXX 4 bytes hole, try to pack */

	struct resource *          region;               /*  1032     8 */

	/* size: 1040, cachelines: 17, members: 3 */
	/* sum members: 1036, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct thermal_cooling_device {
	int                        id;                   /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	char *                     type;                 /*     8     8 */
	struct device              device __attribute__((__aligned__(8))); /*    16  1000 */
	/* --- cacheline 15 boundary (960 bytes) was 56 bytes ago --- */
	struct device_node *       np;                   /*  1016     8 */
	/* --- cacheline 16 boundary (1024 bytes) --- */
	void *                     devdata;              /*  1024     8 */
	void *                     stats;                /*  1032     8 */
	const struct thermal_cooling_device_ops  * ops;  /*  1040     8 */
	bool                       updated;              /*  1048     1 */

	/* XXX 7 bytes hole, try to pack */

	struct mutex               lock;                 /*  1056    32 */
	/* --- cacheline 17 boundary (1088 bytes) --- */
	struct list_head           thermal_instances;    /*  1088    16 */
	struct list_head           node;                 /*  1104    16 */

	/* size: 1120, cachelines: 18, members: 11 */
	/* sum members: 1109, holes: 2, sum holes: 11 */
	/* forced alignments: 1 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct idle_cpu {
	struct cpuidle_state *     state_table;          /*     0     8 */
	long unsigned int          auto_demotion_disable_flags; /*     8     8 */
	bool                       byt_auto_demotion_disable_flag; /*    16     1 */
	bool                       disable_promotion_to_c1e; /*    17     1 */
	bool                       use_acpi;             /*    18     1 */

	/* size: 24, cachelines: 1, members: 5 */
	/* padding: 5 */
	/* last cacheline: 24 bytes */
};
struct thermal_cooling_device_ops {
	int                        (*get_max_state)(struct thermal_cooling_device *, long unsigned int *); /*     0     8 */
	int                        (*get_cur_state)(struct thermal_cooling_device *, long unsigned int *); /*     8     8 */
	int                        (*set_cur_state)(struct thermal_cooling_device *, long unsigned int); /*    16     8 */
	int                        (*get_requested_power)(struct thermal_cooling_device *, u32 *); /*    24     8 */
	int                        (*state2power)(struct thermal_cooling_device *, long unsigned int, u32 *); /*    32     8 */
	int                        (*power2state)(struct thermal_cooling_device *, u32, long unsigned int *); /*    40     8 */

	/* size: 48, cachelines: 1, members: 6 */
	/* last cacheline: 48 bytes */
};
struct acpi_lpi_state {
	u32                        min_residency;        /*     0     4 */
	u32                        wake_latency;         /*     4     4 */
	u32                        flags;                /*     8     4 */
	u32                        arch_flags;           /*    12     4 */
	u32                        res_cnt_freq;         /*    16     4 */
	u32                        enable_parent_state;  /*    20     4 */
	u64                        address;              /*    24     8 */
	u8                         index;                /*    32     1 */
	u8                         entry_method;         /*    33     1 */
	char                       desc[32];             /*    34    32 */

	/* size: 72, cachelines: 2, members: 10 */
	/* padding: 6 */
	/* last cacheline: 8 bytes */
};
struct acpi_processor_power {
	int                        count;                /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	union {
		struct acpi_processor_cx states[8];      /*     8   416 */
		struct acpi_lpi_state lpi_states[8];     /*     8   576 */
	};                                               /*     8   576 */
	/* --- cacheline 9 boundary (576 bytes) was 8 bytes ago --- */
	int                        timer_broadcast_on_state; /*   584     4 */

	/* size: 592, cachelines: 10, members: 3 */
	/* sum members: 584, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct acpi_psd_package {
	u64                        num_entries;          /*     0     8 */
	u64                        revision;             /*     8     8 */
	u64                        domain;               /*    16     8 */
	u64                        coord_type;           /*    24     8 */
	u64                        num_processors;       /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct acpi_pct_register {
	u8                         descriptor;           /*     0     1 */
	u16                        length;               /*     1     2 */
	u8                         space_id;             /*     3     1 */
	u8                         bit_width;            /*     4     1 */
	u8                         bit_offset;           /*     5     1 */
	u8                         reserved;             /*     6     1 */
	u64                        address;              /*     7     8 */

	/* size: 15, cachelines: 1, members: 7 */
	/* last cacheline: 15 bytes */
} __attribute__((__packed__));
struct acpi_processor_px {
	u64                        core_frequency;       /*     0     8 */
	u64                        power;                /*     8     8 */
	u64                        transition_latency;   /*    16     8 */
	u64                        bus_master_latency;   /*    24     8 */
	u64                        control;              /*    32     8 */
	u64                        status;               /*    40     8 */

	/* size: 48, cachelines: 1, members: 6 */
	/* last cacheline: 48 bytes */
};
struct acpi_processor_performance {
	unsigned int               state;                /*     0     4 */
	unsigned int               platform_limit;       /*     4     4 */
	struct acpi_pct_register   control_register;     /*     8    15 */
	struct acpi_pct_register   status_register;      /*    23    15 */

	/* XXX 2 bytes hole, try to pack */

	unsigned int               state_count;          /*    40     4 */

	/* XXX 4 bytes hole, try to pack */

	struct acpi_processor_px * states;               /*    48     8 */
	struct acpi_psd_package    domain_info;          /*    56    40 */
	/* --- cacheline 1 boundary (64 bytes) was 32 bytes ago --- */
	cpumask_var_t              shared_cpu_map;       /*    96     8 */
	unsigned int               shared_type;          /*   104     4 */

	/* size: 112, cachelines: 2, members: 9 */
	/* sum members: 102, holes: 2, sum holes: 6 */
	/* padding: 4 */
	/* last cacheline: 48 bytes */
};
struct acpi_tsd_package {
	u64                        num_entries;          /*     0     8 */
	u64                        revision;             /*     8     8 */
	u64                        domain;               /*    16     8 */
	u64                        coord_type;           /*    24     8 */
	u64                        num_processors;       /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct acpi_processor_tx_tss {
	u64                        freqpercentage;       /*     0     8 */
	u64                        power;                /*     8     8 */
	u64                        transition_latency;   /*    16     8 */
	u64                        control;              /*    24     8 */
	u64                        status;               /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct acpi_processor_tx {
	u16                        power;                /*     0     2 */
	u16                        performance;          /*     2     2 */

	/* size: 4, cachelines: 1, members: 2 */
	/* last cacheline: 4 bytes */
};
struct acpi_processor_throttling {
	unsigned int               state;                /*     0     4 */
	unsigned int               platform_limit;       /*     4     4 */
	struct acpi_pct_register   control_register;     /*     8    15 */
	struct acpi_pct_register   status_register;      /*    23    15 */

	/* XXX 2 bytes hole, try to pack */

	unsigned int               state_count;          /*    40     4 */

	/* XXX 4 bytes hole, try to pack */

	struct acpi_processor_tx_tss * states_tss;       /*    48     8 */
	struct acpi_tsd_package    domain_info;          /*    56    40 */
	/* --- cacheline 1 boundary (64 bytes) was 32 bytes ago --- */
	cpumask_var_t              shared_cpu_map;       /*    96     8 */
	int                        (*acpi_processor_get_throttling)(struct acpi_processor *); /*   104     8 */
	int                        (*acpi_processor_set_throttling)(struct acpi_processor *, int, bool); /*   112     8 */
	u32                        address;              /*   120     4 */
	u8                         duty_offset;          /*   124     1 */
	u8                         duty_width;           /*   125     1 */
	u8                         tsd_valid_flag;       /*   126     1 */

	/* XXX 1 byte hole, try to pack */

	/* --- cacheline 2 boundary (128 bytes) --- */
	unsigned int               shared_type;          /*   128     4 */
	struct acpi_processor_tx   states[16];           /*   132    64 */

	/* size: 200, cachelines: 4, members: 16 */
	/* sum members: 189, holes: 3, sum holes: 7 */
	/* padding: 4 */
	/* last cacheline: 8 bytes */
};
struct acpi_processor {
	acpi_handle                handle;               /*     0     8 */
	u32                        acpi_id;              /*     8     4 */
	phys_cpuid_t               phys_id;              /*    12     4 */
	u32                        id;                   /*    16     4 */
	u32                        pblk;                 /*    20     4 */
	int                        performance_platform_limit; /*    24     4 */
	int                        throttling_platform_limit; /*    28     4 */
	struct acpi_processor_flags flags;               /*    32     2 */

	/* XXX 6 bytes hole, try to pack */

	struct acpi_processor_power power;               /*    40   592 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 9 boundary (576 bytes) was 56 bytes ago --- */
	struct acpi_processor_performance * performance; /*   632     8 */
	/* --- cacheline 10 boundary (640 bytes) --- */
	struct acpi_processor_throttling throttling;     /*   640   200 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 13 boundary (832 bytes) was 8 bytes ago --- */
	struct acpi_processor_limit limit;               /*   840    24 */
	struct thermal_cooling_device * cdev;            /*   864     8 */
	struct device *            dev;                  /*   872     8 */
	struct freq_qos_request    perflib_req;          /*   880    56 */
	/* --- cacheline 14 boundary (896 bytes) was 40 bytes ago --- */
	struct freq_qos_request    thermal_req;          /*   936    56 */

	/* size: 992, cachelines: 16, members: 16 */
	/* sum members: 986, holes: 1, sum holes: 6 */
	/* paddings: 2, sum paddings: 8 */
	/* last cacheline: 32 bytes */
};
struct acpi_processor_lx {
	int                        px;                   /*     0     4 */
	int                        tx;                   /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct acpi_processor_limit {
	struct acpi_processor_lx   state;                /*     0     8 */
	struct acpi_processor_lx   thermal;              /*     8     8 */
	struct acpi_processor_lx   user;                 /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct dmi_header {
	u8                         type;                 /*     0     1 */
	u8                         length;               /*     1     1 */
	u16                        handle;               /*     2     2 */

	/* size: 4, cachelines: 1, members: 3 */
	/* last cacheline: 4 bytes */
};
struct ipmi_plat_data {
	enum ipmi_plat_interface_type iftype;            /*     0     4 */
	unsigned int               type;                 /*     4     4 */
	unsigned int               space;                /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          addr;                 /*    16     8 */
	unsigned int               regspacing;           /*    24     4 */
	unsigned int               regsize;              /*    28     4 */
	unsigned int               regshift;             /*    32     4 */
	unsigned int               irq;                  /*    36     4 */
	unsigned int               slave_addr;           /*    40     4 */
	enum ipmi_addr_src         addr_source;          /*    44     4 */

	/* size: 48, cachelines: 1, members: 10 */
	/* sum members: 44, holes: 1, sum holes: 4 */
	/* last cacheline: 48 bytes */
};
struct ipmi_dmi_info {
	enum si_type               si_type;              /*     0     4 */
	unsigned int               space;                /*     4     4 */
	long unsigned int          addr;                 /*     8     8 */
	u8                         slave_addr;           /*    16     1 */

	/* XXX 7 bytes hole, try to pack */

	struct ipmi_dmi_info *     next;                 /*    24     8 */

	/* size: 32, cachelines: 1, members: 5 */
	/* sum members: 25, holes: 1, sum holes: 7 */
	/* last cacheline: 32 bytes */
};
union acpi_name_union {
	u32                        integer;            /*     0     4 */
	char                       ascii[4];           /*     0     4 */
};
struct acpi_table_desc {
	acpi_physical_address      address;              /*     0     8 */
	struct acpi_table_header * pointer;              /*     8     8 */
	u32                        length;               /*    16     4 */
	union acpi_name_union      signature;            /*    20     4 */
	acpi_owner_id              owner_id;             /*    24     2 */
	u8                         flags;                /*    26     1 */

	/* XXX 1 byte hole, try to pack */

	u16                        validation_count;     /*    28     2 */

	/* size: 32, cachelines: 1, members: 7 */
	/* sum members: 29, holes: 1, sum holes: 1 */
	/* padding: 2 */
	/* last cacheline: 32 bytes */
};
struct acpi_madt_io_sapic {
	struct acpi_subtable_header header;              /*     0     2 */
	u8                         id;                   /*     2     1 */
	u8                         reserved;             /*     3     1 */
	u32                        global_irq_base;      /*     4     4 */
	u64                        address;              /*     8     8 */

	/* size: 16, cachelines: 1, members: 5 */
	/* last cacheline: 16 bytes */
};
struct acpi_madt_interrupt_source {
	struct acpi_subtable_header header;              /*     0     2 */
	u16                        inti_flags;           /*     2     2 */
	u8                         type;                 /*     4     1 */
	u8                         id;                   /*     5     1 */
	u8                         eid;                  /*     6     1 */
	u8                         io_sapic_vector;      /*     7     1 */
	u32                        global_irq;           /*     8     4 */
	u32                        flags;                /*    12     4 */

	/* size: 16, cachelines: 1, members: 8 */
	/* last cacheline: 16 bytes */
};
struct acpi_madt_generic_interrupt {
	struct acpi_subtable_header header;              /*     0     2 */
	u16                        reserved;             /*     2     2 */
	u32                        cpu_interface_number; /*     4     4 */
	u32                        uid;                  /*     8     4 */
	u32                        flags;                /*    12     4 */
	u32                        parking_version;      /*    16     4 */
	u32                        performance_interrupt; /*    20     4 */
	u64                        parked_address;       /*    24     8 */
	u64                        base_address;         /*    32     8 */
	u64                        gicv_base_address;    /*    40     8 */
	u64                        gich_base_address;    /*    48     8 */
	u32                        vgic_interrupt;       /*    56     4 */
	u64                        gicr_base_address;    /*    60     8 */
	/* --- cacheline 1 boundary (64 bytes) was 4 bytes ago --- */
	u64                        arm_mpidr;            /*    68     8 */
	u8                         efficiency_class;     /*    76     1 */
	u8                         reserved2[1];         /*    77     1 */
	u16                        spe_interrupt;        /*    78     2 */

	/* size: 80, cachelines: 2, members: 17 */
	/* last cacheline: 16 bytes */
} __attribute__((__packed__));
struct acpi_madt_generic_distributor {
	struct acpi_subtable_header header;              /*     0     2 */
	u16                        reserved;             /*     2     2 */
	u32                        gic_id;               /*     4     4 */
	u64                        base_address;         /*     8     8 */
	u32                        global_irq_base;      /*    16     4 */
	u8                         version;              /*    20     1 */
	u8                         reserved2[3];         /*    21     3 */

	/* size: 24, cachelines: 1, members: 7 */
	/* last cacheline: 24 bytes */
};
struct acpi_madt_core_pic {
	struct acpi_subtable_header header;              /*     0     2 */
	u8                         version;              /*     2     1 */
	u32                        processor_id;         /*     3     4 */
	u32                        core_id;              /*     7     4 */
	u32                        flags;                /*    11     4 */

	/* size: 15, cachelines: 1, members: 5 */
	/* last cacheline: 15 bytes */
} __attribute__((__packed__));
struct acpi_subtable_entry {
	union acpi_subtable_headers * hdr;               /*     0     8 */
	enum acpi_subtable_type    type;                 /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct acpi_platform_list {
	char                       oem_id[7];            /*     0     7 */
	char                       oem_table_id[9];      /*     7     9 */
	u32                        oem_revision;         /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	char *                     table;                /*    24     8 */
	enum acpi_predicate        pred;                 /*    32     4 */

	/* XXX 4 bytes hole, try to pack */

	char *                     reason;               /*    40     8 */
	u32                        data;                 /*    48     4 */

	/* size: 56, cachelines: 1, members: 7 */
	/* sum members: 44, holes: 2, sum holes: 8 */
	/* padding: 4 */
	/* last cacheline: 56 bytes */
};
struct acpi_osi_entry {
	char                       string[64];           /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	bool                       enable;               /*    64     1 */

	/* size: 65, cachelines: 2, members: 2 */
	/* last cacheline: 1 bytes */
};
struct acpi_osi_config {
	u8                         default_disabling;    /*     0     1 */

	/* Bitfield combined with previous fields */

	unsigned int               linux_enable:1;       /*     0: 8  4 */
	unsigned int               linux_dmi:1;          /*     0: 9  4 */
	unsigned int               linux_cmdline:1;      /*     0:10  4 */
	unsigned int               darwin_enable:1;      /*     0:11  4 */
	unsigned int               darwin_dmi:1;         /*     0:12  4 */
	unsigned int               darwin_cmdline:1;     /*     0:13  4 */

	/* size: 4, cachelines: 1, members: 7 */
	/* bit_padding: 18 bits */
	/* last cacheline: 4 bytes */
};
struct acpi_predefined_names {
	const char  *              name;                 /*     0     8 */
	u8                         type;                 /*     8     1 */

	/* XXX 7 bytes hole, try to pack */

	char *                     val;                  /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 17, holes: 1, sum holes: 7 */
	/* last cacheline: 24 bytes */
};
struct acpi_pci_id {
	u16                        segment;              /*     0     2 */
	u16                        bus;                  /*     2     2 */
	u16                        device;               /*     4     2 */
	u16                        function;             /*     6     2 */

	/* size: 8, cachelines: 1, members: 4 */
	/* last cacheline: 8 bytes */
};
struct acpi_mem_mapping {
	acpi_physical_address      physical_address;     /*     0     8 */
	u8 *                       logical_address;      /*     8     8 */
	acpi_size                  length;               /*    16     8 */
	struct acpi_mem_mapping *  next_mm;              /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct acpi_mem_space_context {
	u32                        length;               /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	acpi_physical_address      address;              /*     8     8 */
	struct acpi_mem_mapping *  cur_mm;               /*    16     8 */
	struct acpi_mem_mapping *  first_mm;             /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct acpi_namespace_node {
	union acpi_operand_object * object;              /*     0     8 */
	u8                         descriptor_type;      /*     8     1 */
	u8                         type;                 /*     9     1 */
	u16                        flags;                /*    10     2 */
	union acpi_name_union      name;                 /*    12     4 */
	struct acpi_namespace_node * parent;             /*    16     8 */
	struct acpi_namespace_node * child;              /*    24     8 */
	struct acpi_namespace_node * peer;               /*    32     8 */
	acpi_owner_id              owner_id;             /*    40     2 */

	/* size: 48, cachelines: 1, members: 9 */
	/* padding: 6 */
	/* last cacheline: 48 bytes */
};
union acpi_operand_object {
	struct acpi_object_common  common;             /*     0    16 */
	struct acpi_object_integer integer;            /*     0    24 */
	struct acpi_object_string  string;             /*     0    32 */
	struct acpi_object_buffer  buffer;             /*     0    48 */
	struct acpi_object_package package;            /*     0    48 */
	struct acpi_object_event   event;              /*     0    24 */
	struct acpi_object_method  method;             /*     0    56 */
	struct acpi_object_mutex   mutex;              /*     0    72 */
	struct acpi_object_region  region;             /*     0    56 */
	struct acpi_object_notify_common common_notify; /*     0    40 */
	struct acpi_object_device  device;             /*     0    48 */
	struct acpi_object_power_resource power_resource; /*     0    48 */
	struct acpi_object_processor processor;        /*     0    48 */
	struct acpi_object_thermal_zone thermal_zone;  /*     0    40 */
	struct acpi_object_field_common common_field;  /*     0    48 */
	struct acpi_object_region_field field;         /*     0    72 */
	struct acpi_object_buffer_field buffer_field;  /*     0    48 */
	struct acpi_object_bank_field bank_field;      /*     0    56 */
	struct acpi_object_index_field index_field;    /*     0    56 */
	struct acpi_object_notify_handler notify;      /*     0    64 */
	struct acpi_object_addr_handler address_space; /*     0    72 */
	struct acpi_object_reference reference;        /*     0    64 */
	struct acpi_object_extra   extra;              /*     0    56 */
	struct acpi_object_data    data;               /*     0    32 */
	struct acpi_object_cache_list cache;           /*     0    24 */
	struct acpi_namespace_node node;               /*     0    48 */
};
struct acpi_walk_state {
	struct acpi_walk_state *   next;                 /*     0     8 */
	u8                         descriptor_type;      /*     8     1 */
	u8                         walk_type;            /*     9     1 */
	u16                        opcode;               /*    10     2 */
	u8                         next_op_info;         /*    12     1 */
	u8                         num_operands;         /*    13     1 */
	u8                         operand_index;        /*    14     1 */

	/* XXX 1 byte hole, try to pack */

	acpi_owner_id              owner_id;             /*    16     2 */
	u8                         last_predicate;       /*    18     1 */
	u8                         current_result;       /*    19     1 */
	u8                         return_used;          /*    20     1 */
	u8                         scope_depth;          /*    21     1 */
	u8                         pass_number;          /*    22     1 */
	u8                         namespace_override;   /*    23     1 */
	u8                         result_size;          /*    24     1 */
	u8                         result_count;         /*    25     1 */

	/* XXX 6 bytes hole, try to pack */

	u8 *                       aml;                  /*    32     8 */
	u32                        arg_types;            /*    40     4 */
	u32                        method_breakpoint;    /*    44     4 */
	u32                        user_breakpoint;      /*    48     4 */
	u32                        parse_flags;          /*    52     4 */
	struct acpi_parse_state    parser_state;         /*    56    80 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 2 boundary (128 bytes) was 8 bytes ago --- */
	u32                        prev_arg_types;       /*   136     4 */
	u32                        arg_count;            /*   140     4 */
	u16                        method_nesting_depth; /*   144     2 */
	u8                         method_is_nested;     /*   146     1 */

	/* XXX 5 bytes hole, try to pack */

	struct acpi_namespace_node arguments[7];         /*   152   336 */
	/* --- cacheline 7 boundary (448 bytes) was 40 bytes ago --- */
	struct acpi_namespace_node local_variables[8];   /*   488   384 */
	/* --- cacheline 13 boundary (832 bytes) was 40 bytes ago --- */
	union acpi_operand_object * operands[9];         /*   872    72 */
	/* --- cacheline 14 boundary (896 bytes) was 48 bytes ago --- */
	union acpi_operand_object * * params;            /*   944     8 */
	u8 *                       aml_last_while;       /*   952     8 */
	/* --- cacheline 15 boundary (960 bytes) --- */
	union acpi_operand_object * * caller_return_desc; /*   960     8 */
	union acpi_generic_state * control_state;        /*   968     8 */
	struct acpi_namespace_node * deferred_node;      /*   976     8 */
	union acpi_operand_object * implicit_return_obj; /*   984     8 */
	struct acpi_namespace_node * method_call_node;   /*   992     8 */
	union acpi_parse_object *  method_call_op;       /*  1000     8 */
	union acpi_operand_object * method_desc;         /*  1008     8 */
	struct acpi_namespace_node * method_node;        /*  1016     8 */
	/* --- cacheline 16 boundary (1024 bytes) --- */
	char *                     method_pathname;      /*  1024     8 */
	union acpi_parse_object *  op;                   /*  1032     8 */
	const struct acpi_opcode_info  * op_info;        /*  1040     8 */
	union acpi_parse_object *  origin;               /*  1048     8 */
	union acpi_operand_object * result_obj;          /*  1056     8 */
	union acpi_generic_state * results;              /*  1064     8 */
	union acpi_operand_object * return_desc;         /*  1072     8 */
	union acpi_generic_state * scope_info;           /*  1080     8 */
	/* --- cacheline 17 boundary (1088 bytes) --- */
	union acpi_parse_object *  prev_op;              /*  1088     8 */
	union acpi_parse_object *  next_op;              /*  1096     8 */
	struct acpi_thread_state * thread;               /*  1104     8 */
	acpi_parse_downwards       descending_callback;  /*  1112     8 */
	acpi_parse_upwards         ascending_callback;   /*  1120     8 */

	/* size: 1128, cachelines: 18, members: 52 */
	/* sum members: 1116, holes: 3, sum holes: 12 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 40 bytes */
};
struct acpi_gpe_handler_info {
	acpi_gpe_handler           address;              /*     0     8 */
	void *                     context;              /*     8     8 */
	struct acpi_namespace_node * method_node;        /*    16     8 */
	u8                         original_flags;       /*    24     1 */
	u8                         originally_enabled;   /*    25     1 */

	/* size: 32, cachelines: 1, members: 5 */
	/* padding: 6 */
	/* last cacheline: 32 bytes */
};
struct acpi_gpe_notify_info {
	struct acpi_namespace_node * device_node;        /*     0     8 */
	struct acpi_gpe_notify_info * next;              /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
union acpi_gpe_dispatch_info {
	struct acpi_namespace_node * method_node;      /*     0     8 */
	struct acpi_gpe_handler_info * handler;        /*     0     8 */
	struct acpi_gpe_notify_info * notify_list;     /*     0     8 */
};
struct acpi_gpe_event_info {
	union acpi_gpe_dispatch_info dispatch;           /*     0     8 */
	struct acpi_gpe_register_info * register_info;   /*     8     8 */
	u8                         flags;                /*    16     1 */
	u8                         gpe_number;           /*    17     1 */
	u8                         runtime_count;        /*    18     1 */
	u8                         disable_for_dispatch; /*    19     1 */

	/* size: 24, cachelines: 1, members: 6 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct acpi_gpe_register_info {
	struct acpi_gpe_address    status_address;       /*     0    16 */
	struct acpi_gpe_address    enable_address;       /*    16    16 */
	u16                        base_gpe_number;      /*    32     2 */
	u8                         enable_for_wake;      /*    34     1 */
	u8                         enable_for_run;       /*    35     1 */
	u8                         mask_for_run;         /*    36     1 */
	u8                         enable_mask;          /*    37     1 */

	/* size: 40, cachelines: 1, members: 7 */
	/* padding: 2 */
	/* last cacheline: 40 bytes */
};
struct acpi_gpe_address {
	u8                         space_id;             /*     0     1 */

	/* XXX 7 bytes hole, try to pack */

	u64                        address;              /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 9, holes: 1, sum holes: 7 */
	/* last cacheline: 16 bytes */
};
struct acpi_gpe_block_info {
	struct acpi_namespace_node * node;               /*     0     8 */
	struct acpi_gpe_block_info * previous;           /*     8     8 */
	struct acpi_gpe_block_info * next;               /*    16     8 */
	struct acpi_gpe_xrupt_info * xrupt_block;        /*    24     8 */
	struct acpi_gpe_register_info * register_info;   /*    32     8 */
	struct acpi_gpe_event_info * event_info;         /*    40     8 */
	u64                        address;              /*    48     8 */
	u32                        register_count;       /*    56     4 */
	u16                        gpe_count;            /*    60     2 */
	u16                        block_base_number;    /*    62     2 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u8                         space_id;             /*    64     1 */
	u8                         initialized;          /*    65     1 */

	/* size: 72, cachelines: 2, members: 12 */
	/* padding: 6 */
	/* last cacheline: 8 bytes */
};
struct acpi_gpe_xrupt_info {
	struct acpi_gpe_xrupt_info * previous;           /*     0     8 */
	struct acpi_gpe_xrupt_info * next;               /*     8     8 */
	struct acpi_gpe_block_info * gpe_block_list_head; /*    16     8 */
	u32                        interrupt_number;     /*    24     4 */

	/* size: 32, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct acpi_common_state {
	void *                     next;                 /*     0     8 */
	u8                         descriptor_type;      /*     8     1 */
	u8                         flags;                /*     9     1 */
	u16                        value;                /*    10     2 */
	u16                        state;                /*    12     2 */

	/* size: 16, cachelines: 1, members: 5 */
	/* padding: 2 */
	/* last cacheline: 16 bytes */
};
struct acpi_update_state {
	void *                     next;                 /*     0     8 */
	u8                         descriptor_type;      /*     8     1 */
	u8                         flags;                /*     9     1 */
	u16                        value;                /*    10     2 */
	u16                        state;                /*    12     2 */

	/* XXX 2 bytes hole, try to pack */

	union acpi_operand_object * object;              /*    16     8 */

	/* size: 24, cachelines: 1, members: 6 */
	/* sum members: 22, holes: 1, sum holes: 2 */
	/* last cacheline: 24 bytes */
};
struct acpi_pkg_state {
	void *                     next;                 /*     0     8 */
	u8                         descriptor_type;      /*     8     1 */
	u8                         flags;                /*     9     1 */
	u16                        value;                /*    10     2 */
	u16                        state;                /*    12     2 */

	/* XXX 2 bytes hole, try to pack */

	u32                        index;                /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	union acpi_operand_object * source_object;       /*    24     8 */
	union acpi_operand_object * dest_object;         /*    32     8 */
	struct acpi_walk_state *   walk_state;           /*    40     8 */
	void *                     this_target_obj;      /*    48     8 */
	u32                        num_packages;         /*    56     4 */

	/* size: 64, cachelines: 1, members: 11 */
	/* sum members: 54, holes: 2, sum holes: 6 */
	/* padding: 4 */
};
struct acpi_control_state {
	void *                     next;                 /*     0     8 */
	u8                         descriptor_type;      /*     8     1 */
	u8                         flags;                /*     9     1 */
	u16                        value;                /*    10     2 */
	u16                        state;                /*    12     2 */
	u16                        opcode;               /*    14     2 */
	union acpi_parse_object *  predicate_op;         /*    16     8 */
	u8 *                       aml_predicate_start;  /*    24     8 */
	u8 *                       package_end;          /*    32     8 */
	u64                        loop_timeout;         /*    40     8 */

	/* size: 48, cachelines: 1, members: 10 */
	/* last cacheline: 48 bytes */
};
union acpi_parse_object {
	struct acpi_parse_obj_common common;           /*     0    56 */
	struct acpi_parse_obj_named named;             /*     0    80 */
	struct acpi_parse_obj_asl  asl;                /*     0   200 */
};
struct acpi_scope_state {
	void *                     next;                 /*     0     8 */
	u8                         descriptor_type;      /*     8     1 */
	u8                         flags;                /*     9     1 */
	u16                        value;                /*    10     2 */
	u16                        state;                /*    12     2 */

	/* XXX 2 bytes hole, try to pack */

	struct acpi_namespace_node * node;               /*    16     8 */

	/* size: 24, cachelines: 1, members: 6 */
	/* sum members: 22, holes: 1, sum holes: 2 */
	/* last cacheline: 24 bytes */
};
struct acpi_pscope_state {
	void *                     next;                 /*     0     8 */
	u8                         descriptor_type;      /*     8     1 */
	u8                         flags;                /*     9     1 */
	u16                        value;                /*    10     2 */
	u16                        state;                /*    12     2 */

	/* XXX 2 bytes hole, try to pack */

	u32                        arg_count;            /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	union acpi_parse_object *  op;                   /*    24     8 */
	u8 *                       arg_end;              /*    32     8 */
	u8 *                       pkg_end;              /*    40     8 */
	u32                        arg_list;             /*    48     4 */

	/* size: 56, cachelines: 1, members: 10 */
	/* sum members: 46, holes: 2, sum holes: 6 */
	/* padding: 4 */
	/* last cacheline: 56 bytes */
};
struct acpi_thread_state {
	void *                     next;                 /*     0     8 */
	u8                         descriptor_type;      /*     8     1 */
	u8                         flags;                /*     9     1 */
	u16                        value;                /*    10     2 */
	u16                        state;                /*    12     2 */
	u8                         current_sync_level;   /*    14     1 */

	/* XXX 1 byte hole, try to pack */

	struct acpi_walk_state *   walk_state_list;      /*    16     8 */
	union acpi_operand_object * acquired_mutex_list; /*    24     8 */
	u64                        thread_id;            /*    32     8 */

	/* size: 40, cachelines: 1, members: 9 */
	/* sum members: 39, holes: 1, sum holes: 1 */
	/* last cacheline: 40 bytes */
};
struct acpi_result_values {
	void *                     next;                 /*     0     8 */
	u8                         descriptor_type;      /*     8     1 */
	u8                         flags;                /*     9     1 */
	u16                        value;                /*    10     2 */
	u16                        state;                /*    12     2 */

	/* XXX 2 bytes hole, try to pack */

	union acpi_operand_object * obj_desc[8];         /*    16    64 */

	/* size: 80, cachelines: 2, members: 6 */
	/* sum members: 78, holes: 1, sum holes: 2 */
	/* last cacheline: 16 bytes */
};
struct acpi_global_notify_handler {
	acpi_notify_handler        handler;              /*     0     8 */
	void *                     context;              /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct acpi_notify_info {
	void *                     next;                 /*     0     8 */
	u8                         descriptor_type;      /*     8     1 */
	u8                         flags;                /*     9     1 */
	u16                        value;                /*    10     2 */
	u16                        state;                /*    12     2 */
	u8                         handler_list_id;      /*    14     1 */

	/* XXX 1 byte hole, try to pack */

	struct acpi_namespace_node * node;               /*    16     8 */
	union acpi_operand_object * handler_list_head;   /*    24     8 */
	struct acpi_global_notify_handler * global;      /*    32     8 */

	/* size: 40, cachelines: 1, members: 9 */
	/* sum members: 39, holes: 1, sum holes: 1 */
	/* last cacheline: 40 bytes */
};
union acpi_generic_state {
	struct acpi_common_state   common;             /*     0    16 */
	struct acpi_control_state  control;            /*     0    48 */
	struct acpi_update_state   update;             /*     0    24 */
	struct acpi_scope_state    scope;              /*     0    24 */
	struct acpi_pscope_state   parse_scope;        /*     0    56 */
	struct acpi_pkg_state      pkg;                /*     0    64 */
	struct acpi_thread_state   thread;             /*     0    40 */
	struct acpi_result_values  results;            /*     0    80 */
	struct acpi_notify_info    notify;             /*     0    40 */
};
struct acpi_opcode_info {
	u32                        parse_args;           /*     0     4 */
	u32                        runtime_args;         /*     4     4 */
	u16                        flags;                /*     8     2 */
	u8                         object_type;          /*    10     1 */
	u8                         class;                /*    11     1 */
	u8                         type;                 /*    12     1 */

	/* size: 16, cachelines: 1, members: 6 */
	/* padding: 3 */
	/* last cacheline: 16 bytes */
};
union acpi_parse_value {
	u64                        integer;            /*     0     8 */
	u32                        size;               /*     0     4 */
	char *                     string;             /*     0     8 */
	u8 *                       buffer;             /*     0     8 */
	char *                     name;               /*     0     8 */
	union acpi_parse_object *  arg;                /*     0     8 */
};
struct acpi_parse_obj_common {
	union acpi_parse_object *  parent;               /*     0     8 */
	u8                         descriptor_type;      /*     8     1 */
	u8                         flags;                /*     9     1 */
	u16                        aml_opcode;           /*    10     2 */

	/* XXX 4 bytes hole, try to pack */

	u8 *                       aml;                  /*    16     8 */
	union acpi_parse_object *  next;                 /*    24     8 */
	struct acpi_namespace_node * node;               /*    32     8 */
	union acpi_parse_value     value;                /*    40     8 */
	u8                         arg_list_length;      /*    48     1 */

	/* size: 56, cachelines: 1, members: 9 */
	/* sum members: 45, holes: 1, sum holes: 4 */
	/* padding: 7 */
	/* last cacheline: 56 bytes */
};
struct acpi_parse_obj_named {
	union acpi_parse_object *  parent;               /*     0     8 */
	u8                         descriptor_type;      /*     8     1 */
	u8                         flags;                /*     9     1 */
	u16                        aml_opcode;           /*    10     2 */

	/* XXX 4 bytes hole, try to pack */

	u8 *                       aml;                  /*    16     8 */
	union acpi_parse_object *  next;                 /*    24     8 */
	struct acpi_namespace_node * node;               /*    32     8 */
	union acpi_parse_value     value;                /*    40     8 */
	u8                         arg_list_length;      /*    48     1 */

	/* XXX 7 bytes hole, try to pack */

	char *                     path;                 /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u8 *                       data;                 /*    64     8 */
	u32                        length;               /*    72     4 */
	u32                        name;                 /*    76     4 */

	/* size: 80, cachelines: 2, members: 13 */
	/* sum members: 69, holes: 2, sum holes: 11 */
	/* last cacheline: 16 bytes */
};
struct acpi_parse_obj_asl {
	union acpi_parse_object *  parent;               /*     0     8 */
	u8                         descriptor_type;      /*     8     1 */
	u8                         flags;                /*     9     1 */
	u16                        aml_opcode;           /*    10     2 */

	/* XXX 4 bytes hole, try to pack */

	u8 *                       aml;                  /*    16     8 */
	union acpi_parse_object *  next;                 /*    24     8 */
	struct acpi_namespace_node * node;               /*    32     8 */
	union acpi_parse_value     value;                /*    40     8 */
	u8                         arg_list_length;      /*    48     1 */

	/* XXX 7 bytes hole, try to pack */

	union acpi_parse_object *  child;                /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	union acpi_parse_object *  parent_method;        /*    64     8 */
	char *                     filename;             /*    72     8 */
	u8                         file_changed;         /*    80     1 */

	/* XXX 7 bytes hole, try to pack */

	char *                     parent_filename;      /*    88     8 */
	char *                     external_name;        /*    96     8 */
	char *                     namepath;             /*   104     8 */
	char                       name_seg[4];          /*   112     4 */
	u32                        extra_value;          /*   116     4 */
	u32                        column;               /*   120     4 */
	u32                        line_number;          /*   124     4 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	u32                        logical_line_number;  /*   128     4 */
	u32                        logical_byte_offset;  /*   132     4 */
	u32                        end_line;             /*   136     4 */
	u32                        end_logical_line;     /*   140     4 */
	u32                        acpi_btype;           /*   144     4 */
	u32                        aml_length;           /*   148     4 */
	u32                        aml_subtree_length;   /*   152     4 */
	u32                        final_aml_length;     /*   156     4 */
	u32                        final_aml_offset;     /*   160     4 */
	u32                        compile_flags;        /*   164     4 */
	u16                        parse_opcode;         /*   168     2 */
	u8                         aml_opcode_length;    /*   170     1 */
	u8                         aml_pkg_len_bytes;    /*   171     1 */
	u8                         extra;                /*   172     1 */
	char                       parse_op_name[20];    /*   173    20 */

	/* size: 200, cachelines: 4, members: 35 */
	/* sum members: 175, holes: 3, sum holes: 18 */
	/* padding: 7 */
	/* last cacheline: 8 bytes */
};
struct acpi_parse_state {
	u8 *                       aml_start;            /*     0     8 */
	u8 *                       aml;                  /*     8     8 */
	u8 *                       aml_end;              /*    16     8 */
	u8 *                       pkg_start;            /*    24     8 */
	u8 *                       pkg_end;              /*    32     8 */
	union acpi_parse_object *  start_op;             /*    40     8 */
	struct acpi_namespace_node * start_node;         /*    48     8 */
	union acpi_generic_state * scope;                /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	union acpi_parse_object *  start_scope;          /*    64     8 */
	u32                        aml_size;             /*    72     4 */

	/* size: 80, cachelines: 2, members: 10 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct acpi_object_common {
	union acpi_operand_object * next_object;         /*     0     8 */
	u8                         descriptor_type;      /*     8     1 */
	u8                         type;                 /*     9     1 */
	u16                        reference_count;      /*    10     2 */
	u8                         flags;                /*    12     1 */

	/* size: 16, cachelines: 1, members: 5 */
	/* padding: 3 */
	/* last cacheline: 16 bytes */
};
struct acpi_object_integer {
	union acpi_operand_object * next_object;         /*     0     8 */
	u8                         descriptor_type;      /*     8     1 */
	u8                         type;                 /*     9     1 */
	u16                        reference_count;      /*    10     2 */
	u8                         flags;                /*    12     1 */
	u8                         fill[3];              /*    13     3 */
	u64                        value;                /*    16     8 */

	/* size: 24, cachelines: 1, members: 7 */
	/* last cacheline: 24 bytes */
};
struct acpi_object_string {
	union acpi_operand_object * next_object;         /*     0     8 */
	u8                         descriptor_type;      /*     8     1 */
	u8                         type;                 /*     9     1 */
	u16                        reference_count;      /*    10     2 */
	u8                         flags;                /*    12     1 */

	/* XXX 3 bytes hole, try to pack */

	char *                     pointer;              /*    16     8 */
	u32                        length;               /*    24     4 */

	/* size: 32, cachelines: 1, members: 7 */
	/* sum members: 25, holes: 1, sum holes: 3 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct acpi_object_buffer {
	union acpi_operand_object * next_object;         /*     0     8 */
	u8                         descriptor_type;      /*     8     1 */
	u8                         type;                 /*     9     1 */
	u16                        reference_count;      /*    10     2 */
	u8                         flags;                /*    12     1 */

	/* XXX 3 bytes hole, try to pack */

	u8 *                       pointer;              /*    16     8 */
	u32                        length;               /*    24     4 */
	u32                        aml_length;           /*    28     4 */
	u8 *                       aml_start;            /*    32     8 */
	struct acpi_namespace_node * node;               /*    40     8 */

	/* size: 48, cachelines: 1, members: 10 */
	/* sum members: 45, holes: 1, sum holes: 3 */
	/* last cacheline: 48 bytes */
};
struct acpi_object_package {
	union acpi_operand_object * next_object;         /*     0     8 */
	u8                         descriptor_type;      /*     8     1 */
	u8                         type;                 /*     9     1 */
	u16                        reference_count;      /*    10     2 */
	u8                         flags;                /*    12     1 */

	/* XXX 3 bytes hole, try to pack */

	struct acpi_namespace_node * node;               /*    16     8 */
	union acpi_operand_object * * elements;          /*    24     8 */
	u8 *                       aml_start;            /*    32     8 */
	u32                        aml_length;           /*    40     4 */
	u32                        count;                /*    44     4 */

	/* size: 48, cachelines: 1, members: 10 */
	/* sum members: 45, holes: 1, sum holes: 3 */
	/* last cacheline: 48 bytes */
};
struct acpi_object_event {
	union acpi_operand_object * next_object;         /*     0     8 */
	u8                         descriptor_type;      /*     8     1 */
	u8                         type;                 /*     9     1 */
	u16                        reference_count;      /*    10     2 */
	u8                         flags;                /*    12     1 */

	/* XXX 3 bytes hole, try to pack */

	void *                     os_semaphore;         /*    16     8 */

	/* size: 24, cachelines: 1, members: 6 */
	/* sum members: 21, holes: 1, sum holes: 3 */
	/* last cacheline: 24 bytes */
};
struct acpi_object_mutex {
	union acpi_operand_object * next_object;         /*     0     8 */
	u8                         descriptor_type;      /*     8     1 */
	u8                         type;                 /*     9     1 */
	u16                        reference_count;      /*    10     2 */
	u8                         flags;                /*    12     1 */
	u8                         sync_level;           /*    13     1 */
	u16                        acquisition_depth;    /*    14     2 */
	void *                     os_mutex;             /*    16     8 */
	u64                        thread_id;            /*    24     8 */
	struct acpi_thread_state * owner_thread;         /*    32     8 */
	union acpi_operand_object * prev;                /*    40     8 */
	union acpi_operand_object * next;                /*    48     8 */
	struct acpi_namespace_node * node;               /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u8                         original_sync_level;  /*    64     1 */

	/* size: 72, cachelines: 2, members: 14 */
	/* padding: 7 */
	/* last cacheline: 8 bytes */
};
struct acpi_object_region {
	union acpi_operand_object * next_object;         /*     0     8 */
	u8                         descriptor_type;      /*     8     1 */
	u8                         type;                 /*     9     1 */
	u16                        reference_count;      /*    10     2 */
	u8                         flags;                /*    12     1 */
	u8                         space_id;             /*    13     1 */

	/* XXX 2 bytes hole, try to pack */

	struct acpi_namespace_node * node;               /*    16     8 */
	union acpi_operand_object * handler;             /*    24     8 */
	union acpi_operand_object * next;                /*    32     8 */
	acpi_physical_address      address;              /*    40     8 */
	u32                        length;               /*    48     4 */

	/* size: 56, cachelines: 1, members: 11 */
	/* sum members: 50, holes: 1, sum holes: 2 */
	/* padding: 4 */
	/* last cacheline: 56 bytes */
};
struct acpi_object_method {
	union acpi_operand_object * next_object;         /*     0     8 */
	u8                         descriptor_type;      /*     8     1 */
	u8                         type;                 /*     9     1 */
	u16                        reference_count;      /*    10     2 */
	u8                         flags;                /*    12     1 */
	u8                         info_flags;           /*    13     1 */
	u8                         param_count;          /*    14     1 */
	u8                         sync_level;           /*    15     1 */
	union acpi_operand_object * mutex;               /*    16     8 */
	union acpi_operand_object * node;                /*    24     8 */
	u8 *                       aml_start;            /*    32     8 */
	union {
		acpi_internal_method implementation;     /*    40     8 */
		union acpi_operand_object * handler;     /*    40     8 */
	} dispatch;                                      /*    40     8 */
	u32                        aml_length;           /*    48     4 */
	acpi_owner_id              owner_id;             /*    52     2 */
	u8                         thread_count;         /*    54     1 */

	/* size: 56, cachelines: 1, members: 15 */
	/* padding: 1 */
	/* last cacheline: 56 bytes */
};
struct acpi_object_notify_common {
	union acpi_operand_object * next_object;         /*     0     8 */
	u8                         descriptor_type;      /*     8     1 */
	u8                         type;                 /*     9     1 */
	u16                        reference_count;      /*    10     2 */
	u8                         flags;                /*    12     1 */

	/* XXX 3 bytes hole, try to pack */

	union acpi_operand_object * notify_list[2];      /*    16    16 */
	union acpi_operand_object * handler;             /*    32     8 */

	/* size: 40, cachelines: 1, members: 7 */
	/* sum members: 37, holes: 1, sum holes: 3 */
	/* last cacheline: 40 bytes */
};
struct acpi_object_device {
	union acpi_operand_object * next_object;         /*     0     8 */
	u8                         descriptor_type;      /*     8     1 */
	u8                         type;                 /*     9     1 */
	u16                        reference_count;      /*    10     2 */
	u8                         flags;                /*    12     1 */

	/* XXX 3 bytes hole, try to pack */

	union acpi_operand_object * notify_list[2];      /*    16    16 */
	union acpi_operand_object * handler;             /*    32     8 */
	struct acpi_gpe_block_info * gpe_block;          /*    40     8 */

	/* size: 48, cachelines: 1, members: 8 */
	/* sum members: 45, holes: 1, sum holes: 3 */
	/* last cacheline: 48 bytes */
};
struct acpi_object_power_resource {
	union acpi_operand_object * next_object;         /*     0     8 */
	u8                         descriptor_type;      /*     8     1 */
	u8                         type;                 /*     9     1 */
	u16                        reference_count;      /*    10     2 */
	u8                         flags;                /*    12     1 */

	/* XXX 3 bytes hole, try to pack */

	union acpi_operand_object * notify_list[2];      /*    16    16 */
	union acpi_operand_object * handler;             /*    32     8 */
	u32                        system_level;         /*    40     4 */
	u32                        resource_order;       /*    44     4 */

	/* size: 48, cachelines: 1, members: 9 */
	/* sum members: 45, holes: 1, sum holes: 3 */
	/* last cacheline: 48 bytes */
};
struct acpi_object_processor {
	union acpi_operand_object * next_object;         /*     0     8 */
	u8                         descriptor_type;      /*     8     1 */
	u8                         type;                 /*     9     1 */
	u16                        reference_count;      /*    10     2 */
	u8                         flags;                /*    12     1 */
	u8                         proc_id;              /*    13     1 */
	u8                         length;               /*    14     1 */

	/* XXX 1 byte hole, try to pack */

	union acpi_operand_object * notify_list[2];      /*    16    16 */
	union acpi_operand_object * handler;             /*    32     8 */
	acpi_io_address            address;              /*    40     8 */

	/* size: 48, cachelines: 1, members: 10 */
	/* sum members: 47, holes: 1, sum holes: 1 */
	/* last cacheline: 48 bytes */
};
struct acpi_object_thermal_zone {
	union acpi_operand_object * next_object;         /*     0     8 */
	u8                         descriptor_type;      /*     8     1 */
	u8                         type;                 /*     9     1 */
	u16                        reference_count;      /*    10     2 */
	u8                         flags;                /*    12     1 */

	/* XXX 3 bytes hole, try to pack */

	union acpi_operand_object * notify_list[2];      /*    16    16 */
	union acpi_operand_object * handler;             /*    32     8 */

	/* size: 40, cachelines: 1, members: 7 */
	/* sum members: 37, holes: 1, sum holes: 3 */
	/* last cacheline: 40 bytes */
};
struct acpi_object_field_common {
	union acpi_operand_object * next_object;         /*     0     8 */
	u8                         descriptor_type;      /*     8     1 */
	u8                         type;                 /*     9     1 */
	u16                        reference_count;      /*    10     2 */
	u8                         flags;                /*    12     1 */
	u8                         field_flags;          /*    13     1 */
	u8                         attribute;            /*    14     1 */
	u8                         access_byte_width;    /*    15     1 */
	struct acpi_namespace_node * node;               /*    16     8 */
	u32                        bit_length;           /*    24     4 */
	u32                        base_byte_offset;     /*    28     4 */
	u32                        value;                /*    32     4 */
	u8                         start_field_bit_offset; /*    36     1 */
	u8                         access_length;        /*    37     1 */

	/* XXX 2 bytes hole, try to pack */

	union acpi_operand_object * region_obj;          /*    40     8 */

	/* size: 48, cachelines: 1, members: 15 */
	/* sum members: 46, holes: 1, sum holes: 2 */
	/* last cacheline: 48 bytes */
};
struct acpi_object_region_field {
	union acpi_operand_object * next_object;         /*     0     8 */
	u8                         descriptor_type;      /*     8     1 */
	u8                         type;                 /*     9     1 */
	u16                        reference_count;      /*    10     2 */
	u8                         flags;                /*    12     1 */
	u8                         field_flags;          /*    13     1 */
	u8                         attribute;            /*    14     1 */
	u8                         access_byte_width;    /*    15     1 */
	struct acpi_namespace_node * node;               /*    16     8 */
	u32                        bit_length;           /*    24     4 */
	u32                        base_byte_offset;     /*    28     4 */
	u32                        value;                /*    32     4 */
	u8                         start_field_bit_offset; /*    36     1 */
	u8                         access_length;        /*    37     1 */
	u16                        resource_length;      /*    38     2 */
	union acpi_operand_object * region_obj;          /*    40     8 */
	u8 *                       resource_buffer;      /*    48     8 */
	u16                        pin_number_index;     /*    56     2 */

	/* XXX 6 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	u8 *                       internal_pcc_buffer;  /*    64     8 */

	/* size: 72, cachelines: 2, members: 19 */
	/* sum members: 66, holes: 1, sum holes: 6 */
	/* last cacheline: 8 bytes */
};
struct acpi_object_bank_field {
	union acpi_operand_object * next_object;         /*     0     8 */
	u8                         descriptor_type;      /*     8     1 */
	u8                         type;                 /*     9     1 */
	u16                        reference_count;      /*    10     2 */
	u8                         flags;                /*    12     1 */
	u8                         field_flags;          /*    13     1 */
	u8                         attribute;            /*    14     1 */
	u8                         access_byte_width;    /*    15     1 */
	struct acpi_namespace_node * node;               /*    16     8 */
	u32                        bit_length;           /*    24     4 */
	u32                        base_byte_offset;     /*    28     4 */
	u32                        value;                /*    32     4 */
	u8                         start_field_bit_offset; /*    36     1 */
	u8                         access_length;        /*    37     1 */

	/* XXX 2 bytes hole, try to pack */

	union acpi_operand_object * region_obj;          /*    40     8 */
	union acpi_operand_object * bank_obj;            /*    48     8 */

	/* size: 56, cachelines: 1, members: 16 */
	/* sum members: 54, holes: 1, sum holes: 2 */
	/* last cacheline: 56 bytes */
};
struct acpi_object_index_field {
	union acpi_operand_object * next_object;         /*     0     8 */
	u8                         descriptor_type;      /*     8     1 */
	u8                         type;                 /*     9     1 */
	u16                        reference_count;      /*    10     2 */
	u8                         flags;                /*    12     1 */
	u8                         field_flags;          /*    13     1 */
	u8                         attribute;            /*    14     1 */
	u8                         access_byte_width;    /*    15     1 */
	struct acpi_namespace_node * node;               /*    16     8 */
	u32                        bit_length;           /*    24     4 */
	u32                        base_byte_offset;     /*    28     4 */
	u32                        value;                /*    32     4 */
	u8                         start_field_bit_offset; /*    36     1 */
	u8                         access_length;        /*    37     1 */

	/* XXX 2 bytes hole, try to pack */

	union acpi_operand_object * index_obj;           /*    40     8 */
	union acpi_operand_object * data_obj;            /*    48     8 */

	/* size: 56, cachelines: 1, members: 16 */
	/* sum members: 54, holes: 1, sum holes: 2 */
	/* last cacheline: 56 bytes */
};
struct acpi_object_buffer_field {
	union acpi_operand_object * next_object;         /*     0     8 */
	u8                         descriptor_type;      /*     8     1 */
	u8                         type;                 /*     9     1 */
	u16                        reference_count;      /*    10     2 */
	u8                         flags;                /*    12     1 */
	u8                         field_flags;          /*    13     1 */
	u8                         attribute;            /*    14     1 */
	u8                         access_byte_width;    /*    15     1 */
	struct acpi_namespace_node * node;               /*    16     8 */
	u32                        bit_length;           /*    24     4 */
	u32                        base_byte_offset;     /*    28     4 */
	u32                        value;                /*    32     4 */
	u8                         start_field_bit_offset; /*    36     1 */
	u8                         access_length;        /*    37     1 */
	u8                         is_create_field;      /*    38     1 */

	/* XXX 1 byte hole, try to pack */

	union acpi_operand_object * buffer_obj;          /*    40     8 */

	/* size: 48, cachelines: 1, members: 16 */
	/* sum members: 47, holes: 1, sum holes: 1 */
	/* last cacheline: 48 bytes */
};
struct acpi_object_notify_handler {
	union acpi_operand_object * next_object;         /*     0     8 */
	u8                         descriptor_type;      /*     8     1 */
	u8                         type;                 /*     9     1 */
	u16                        reference_count;      /*    10     2 */
	u8                         flags;                /*    12     1 */

	/* XXX 3 bytes hole, try to pack */

	struct acpi_namespace_node * node;               /*    16     8 */
	u32                        handler_type;         /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	acpi_notify_handler        handler;              /*    32     8 */
	void *                     context;              /*    40     8 */
	union acpi_operand_object * next[2];             /*    48    16 */

	/* size: 64, cachelines: 1, members: 10 */
	/* sum members: 57, holes: 2, sum holes: 7 */
};
struct acpi_object_addr_handler {
	union acpi_operand_object * next_object;         /*     0     8 */
	u8                         descriptor_type;      /*     8     1 */
	u8                         type;                 /*     9     1 */
	u16                        reference_count;      /*    10     2 */
	u8                         flags;                /*    12     1 */
	u8                         space_id;             /*    13     1 */
	u8                         handler_flags;        /*    14     1 */

	/* XXX 1 byte hole, try to pack */

	acpi_adr_space_handler     handler;              /*    16     8 */
	struct acpi_namespace_node * node;               /*    24     8 */
	void *                     context;              /*    32     8 */
	void *                     context_mutex;        /*    40     8 */
	acpi_adr_space_setup       setup;                /*    48     8 */
	union acpi_operand_object * region_list;         /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	union acpi_operand_object * next;                /*    64     8 */

	/* size: 72, cachelines: 2, members: 14 */
	/* sum members: 71, holes: 1, sum holes: 1 */
	/* last cacheline: 8 bytes */
};
struct acpi_object_reference {
	union acpi_operand_object * next_object;         /*     0     8 */
	u8                         descriptor_type;      /*     8     1 */
	u8                         type;                 /*     9     1 */
	u16                        reference_count;      /*    10     2 */
	u8                         flags;                /*    12     1 */
	u8                         class;                /*    13     1 */
	u8                         target_type;          /*    14     1 */
	u8                         resolved;             /*    15     1 */
	void *                     object;               /*    16     8 */
	struct acpi_namespace_node * node;               /*    24     8 */
	union acpi_operand_object * * where;             /*    32     8 */
	u8 *                       index_pointer;        /*    40     8 */
	u8 *                       aml;                  /*    48     8 */
	u32                        value;                /*    56     4 */

	/* size: 64, cachelines: 1, members: 14 */
	/* padding: 4 */
};
struct acpi_object_extra {
	union acpi_operand_object * next_object;         /*     0     8 */
	u8                         descriptor_type;      /*     8     1 */
	u8                         type;                 /*     9     1 */
	u16                        reference_count;      /*    10     2 */
	u8                         flags;                /*    12     1 */

	/* XXX 3 bytes hole, try to pack */

	struct acpi_namespace_node * method_REG;         /*    16     8 */
	struct acpi_namespace_node * scope_node;         /*    24     8 */
	void *                     region_context;       /*    32     8 */
	u8 *                       aml_start;            /*    40     8 */
	u32                        aml_length;           /*    48     4 */

	/* size: 56, cachelines: 1, members: 10 */
	/* sum members: 49, holes: 1, sum holes: 3 */
	/* padding: 4 */
	/* last cacheline: 56 bytes */
};
struct acpi_object_data {
	union acpi_operand_object * next_object;         /*     0     8 */
	u8                         descriptor_type;      /*     8     1 */
	u8                         type;                 /*     9     1 */
	u16                        reference_count;      /*    10     2 */
	u8                         flags;                /*    12     1 */

	/* XXX 3 bytes hole, try to pack */

	acpi_object_handler        handler;              /*    16     8 */
	void *                     pointer;              /*    24     8 */

	/* size: 32, cachelines: 1, members: 7 */
	/* sum members: 29, holes: 1, sum holes: 3 */
	/* last cacheline: 32 bytes */
};
struct acpi_object_cache_list {
	union acpi_operand_object * next_object;         /*     0     8 */
	u8                         descriptor_type;      /*     8     1 */
	u8                         type;                 /*     9     1 */
	u16                        reference_count;      /*    10     2 */
	u8                         flags;                /*    12     1 */

	/* XXX 3 bytes hole, try to pack */

	union acpi_operand_object * next;                /*    16     8 */

	/* size: 24, cachelines: 1, members: 6 */
	/* sum members: 21, holes: 1, sum holes: 3 */
	/* last cacheline: 24 bytes */
};
struct acpi_os_dpc {
	acpi_osd_exec_callback     function;             /*     0     8 */
	void *                     context;              /*     8     8 */
	struct work_struct         work;                 /*    16    64 */

	/* size: 80, cachelines: 2, members: 3 */
	/* last cacheline: 16 bytes */
};
struct acpi_ioremap {
	struct list_head           list;                 /*     0    16 */
	void *                     virt;                 /*    16     8 */
	acpi_physical_address      phys;                 /*    24     8 */
	acpi_size                  size;                 /*    32     8 */
	union {
		long unsigned int  refcount;             /*    40     8 */
		struct rcu_work    rwork __attribute__((__aligned__(8))); /*    40    88 */
	} track __attribute__((__aligned__(8)));         /*    40    88 */

	/* size: 128, cachelines: 2, members: 5 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(8)));
struct acpi_hp_work {
	struct work_struct         work;                 /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct acpi_device *       adev;                 /*    64     8 */
	u32                        src;                  /*    72     4 */

	/* size: 80, cachelines: 2, members: 3 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct acpi_object_list {
	u32                        count;                /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	union acpi_object *        pointer;              /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct acpi_pld_info {
	u8                         revision;             /*     0     1 */
	u8                         ignore_color;         /*     1     1 */
	u8                         red;                  /*     2     1 */
	u8                         green;                /*     3     1 */
	u8                         blue;                 /*     4     1 */

	/* XXX 1 byte hole, try to pack */

	u16                        width;                /*     6     2 */
	u16                        height;               /*     8     2 */
	u8                         user_visible;         /*    10     1 */
	u8                         dock;                 /*    11     1 */
	u8                         lid;                  /*    12     1 */
	u8                         panel;                /*    13     1 */
	u8                         vertical_position;    /*    14     1 */
	u8                         horizontal_position;  /*    15     1 */
	u8                         shape;                /*    16     1 */
	u8                         group_orientation;    /*    17     1 */
	u8                         group_token;          /*    18     1 */
	u8                         group_position;       /*    19     1 */
	u8                         bay;                  /*    20     1 */
	u8                         ejectable;            /*    21     1 */
	u8                         ospm_eject_required;  /*    22     1 */
	u8                         cabinet_number;       /*    23     1 */
	u8                         card_cage_number;     /*    24     1 */
	u8                         reference;            /*    25     1 */
	u8                         rotation;             /*    26     1 */
	u8                         order;                /*    27     1 */
	u8                         reserved;             /*    28     1 */

	/* XXX 1 byte hole, try to pack */

	u16                        vertical_offset;      /*    30     2 */
	u16                        horizontal_offset;    /*    32     2 */

	/* size: 34, cachelines: 1, members: 28 */
	/* sum members: 32, holes: 2, sum holes: 2 */
	/* last cacheline: 34 bytes */
};
struct acpi_handle_list {
	u32                        count;                /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	acpi_handle                handles[10];          /*     8    80 */

	/* size: 88, cachelines: 2, members: 2 */
	/* sum members: 84, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct acpi_device_bus_id {
	const char  *              bus_id;               /*     0     8 */
	struct ida                 instance_ida;         /*     8    16 */
	struct list_head           node;                 /*    24    16 */

	/* size: 40, cachelines: 1, members: 3 */
	/* last cacheline: 40 bytes */
};
struct acpi_dev_match_info {
	struct acpi_device_id      hid[2];               /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	const char  *              uid;                  /*    64     8 */
	s64                        hrv;                  /*    72     8 */

	/* size: 80, cachelines: 2, members: 3 */
	/* last cacheline: 16 bytes */
};
struct nvs_region {
	__u64                      phys_start;           /*     0     8 */
	__u64                      size;                 /*     8     8 */
	struct list_head           node;                 /*    16    16 */

	/* size: 32, cachelines: 1, members: 3 */
	/* last cacheline: 32 bytes */
};
struct nvs_page {
	long unsigned int          phys_start;           /*     0     8 */
	unsigned int               size;                 /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     kaddr;                /*    16     8 */
	void *                     data;                 /*    24     8 */
	bool                       unmap;                /*    32     1 */

	/* XXX 7 bytes hole, try to pack */

	struct list_head           node;                 /*    40    16 */

	/* size: 56, cachelines: 1, members: 6 */
	/* sum members: 45, holes: 2, sum holes: 11 */
	/* last cacheline: 56 bytes */
};
struct acpi_wakeup_handler {
	struct list_head           list_node;            /*     0    16 */
	bool                       (*wakeup)(void *);    /*    16     8 */
	void *                     context;              /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* last cacheline: 32 bytes */
};
struct acpi_table_facs {
	char                       signature[4];         /*     0     4 */
	u32                        length;               /*     4     4 */
	u32                        hardware_signature;   /*     8     4 */
	u32                        firmware_waking_vector; /*    12     4 */
	u32                        global_lock;          /*    16     4 */
	u32                        flags;                /*    20     4 */
	u64                        xfirmware_waking_vector; /*    24     8 */
	u8                         version;              /*    32     1 */
	u8                         reserved[3];          /*    33     3 */
	u32                        ospm_flags;           /*    36     4 */
	u8                         reserved1[24];        /*    40    24 */

	/* size: 64, cachelines: 1, members: 11 */
};
struct lpi_device_info {
	char *                     name;                 /*     0     8 */
	int                        enabled;              /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	union acpi_object *        package;              /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct lpi_device_constraint {
	int                        uid;                  /*     0     4 */
	int                        min_dstate;           /*     4     4 */
	int                        function_states;      /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct lpi_constraints {
	acpi_handle                handle;               /*     0     8 */
	int                        min_dstate;           /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct acpi_hardware_id {
	struct list_head           list;                 /*     0    16 */
	const char  *              id;                   /*    16     8 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct acpi_data_node {
	const char  *              name;                 /*     0     8 */
	acpi_handle                handle;               /*     8     8 */
	struct fwnode_handle       fwnode;               /*    16    40 */
	struct fwnode_handle *     parent;               /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct acpi_device_data    data;                 /*    64    48 */
	struct list_head           sibling;              /*   112    16 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct kobject             kobj;                 /*   128    96 */
	/* --- cacheline 3 boundary (192 bytes) was 32 bytes ago --- */
	struct completion          kobj_done;            /*   224    32 */

	/* size: 256, cachelines: 4, members: 8 */
};
struct acpi_data_node_attr {
	struct attribute           attr;                 /*     0    16 */

	/* XXX last struct has 6 bytes of padding */

	ssize_t                    (*show)(struct acpi_data_node *, char *); /*    16     8 */
	ssize_t                    (*store)(struct acpi_data_node *, const char  *, size_t); /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* paddings: 1, sum paddings: 6 */
	/* last cacheline: 32 bytes */
};
struct pm_domain_data {
	struct list_head           list_node;            /*     0    16 */
	struct device *            dev;                  /*    16     8 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct acpi_device_physical_node {
	unsigned int               node_id;              /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           node;                 /*     8    16 */
	struct device *            dev;                  /*    24     8 */
	bool                       put_online:1;         /*    32: 0  1 */

	/* size: 40, cachelines: 1, members: 4 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* sum bitfield members: 1 bits (0 bytes) */
	/* padding: 7 */
	/* bit_padding: 7 bits */
	/* last cacheline: 40 bytes */
};
struct acpi_osc_context {
	char *                     uuid_str;             /*     0     8 */
	int                        rev;                  /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct acpi_buffer         cap;                  /*    16    16 */
	struct acpi_buffer         ret;                  /*    32    16 */

	/* size: 48, cachelines: 1, members: 4 */
	/* sum members: 44, holes: 1, sum holes: 4 */
	/* last cacheline: 48 bytes */
};
struct acpi_pnp_device_id {
	u32                        length;               /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	char *                     string;               /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct acpi_pnp_device_id_list {
	u32                        count;                /*     0     4 */
	u32                        list_size;            /*     4     4 */
	struct acpi_pnp_device_id  ids[];                /*     8     0 */

	/* size: 8, cachelines: 1, members: 3 */
	/* last cacheline: 8 bytes */
};
struct acpi_device_info {
	u32                        info_size;            /*     0     4 */
	u32                        name;                 /*     4     4 */
	acpi_object_type           type;                 /*     8     4 */
	u8                         param_count;          /*    12     1 */

	/* XXX 1 byte hole, try to pack */

	u16                        valid;                /*    14     2 */
	u8                         flags;                /*    16     1 */
	u8                         highest_dstates[4];   /*    17     4 */
	u8                         lowest_dstates[5];    /*    21     5 */

	/* XXX 6 bytes hole, try to pack */

	u64                        address;              /*    32     8 */
	struct acpi_pnp_device_id  hardware_id;          /*    40    16 */
	struct acpi_pnp_device_id  unique_id;            /*    56    16 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	struct acpi_pnp_device_id  class_code;           /*    72    16 */
	struct acpi_pnp_device_id_list compatible_id_list; /*    88     8 */

	/* size: 96, cachelines: 2, members: 13 */
	/* sum members: 89, holes: 2, sum holes: 7 */
	/* last cacheline: 32 bytes */
};
struct acpi_table_spcr {
	struct acpi_table_header   header;               /*     0    36 */
	u8                         interface_type;       /*    36     1 */
	u8                         reserved[3];          /*    37     3 */
	struct acpi_generic_address serial_port;         /*    40    12 */
	u8                         interrupt_type;       /*    52     1 */
	u8                         pc_interrupt;         /*    53     1 */
	u32                        interrupt;            /*    54     4 */
	u8                         baud_rate;            /*    58     1 */
	u8                         parity;               /*    59     1 */
	u8                         stop_bits;            /*    60     1 */
	u8                         flow_control;         /*    61     1 */
	u8                         terminal_type;        /*    62     1 */
	u8                         reserved1;            /*    63     1 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u16                        pci_device_id;        /*    64     2 */
	u16                        pci_vendor_id;        /*    66     2 */
	u8                         pci_bus;              /*    68     1 */
	u8                         pci_device;           /*    69     1 */
	u8                         pci_function;         /*    70     1 */
	u32                        pci_flags;            /*    71     4 */
	u8                         pci_segment;          /*    75     1 */
	u32                        reserved2;            /*    76     4 */

	/* size: 80, cachelines: 2, members: 21 */
	/* last cacheline: 16 bytes */
} __attribute__((__packed__));
struct acpi_table_stao {
	struct acpi_table_header   header;               /*     0    36 */
	u8                         ignore_uart;          /*    36     1 */

	/* size: 37, cachelines: 1, members: 2 */
	/* last cacheline: 37 bytes */
} __attribute__((__packed__));
struct acpi_probe_entry {
	__u8                       id[5];                /*     0     5 */
	__u8                       type;                 /*     5     1 */

	/* XXX 2 bytes hole, try to pack */

	acpi_probe_entry_validate_subtbl subtable_valid; /*     8     8 */
	union {
		acpi_tbl_table_handler probe_table;      /*    16     8 */
		acpi_tbl_entry_handler probe_subtbl;     /*    16     8 */
	};                                               /*    16     8 */
	kernel_ulong_t             driver_data;          /*    24     8 */

	/* size: 32, cachelines: 1, members: 5 */
	/* sum members: 30, holes: 1, sum holes: 2 */
	/* last cacheline: 32 bytes */
};
struct acpi_dep_data {
	struct list_head           node;                 /*     0    16 */
	acpi_handle                master;               /*    16     8 */
	acpi_handle                slave;                /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* last cacheline: 32 bytes */
};
struct acpi_table_events_work {
	struct work_struct         work;                 /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	void *                     table;                /*    64     8 */
	u32                        event;                /*    72     4 */

	/* size: 80, cachelines: 2, members: 3 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct resource_win {
	struct resource            res;                  /*     0    96 */
	/* --- cacheline 1 boundary (64 bytes) was 32 bytes ago --- */
	resource_size_t            offset;               /*    96     8 */

	/* size: 104, cachelines: 2, members: 2 */
	/* last cacheline: 40 bytes */
};
struct res_proc_context {
	struct list_head *         list;                 /*     0     8 */
	int                        (*preproc)(struct acpi_resource *, void *); /*     8     8 */
	void *                     preproc_data;         /*    16     8 */
	int                        count;                /*    24     4 */
	int                        error;                /*    28     4 */

	/* size: 32, cachelines: 1, members: 5 */
	/* last cacheline: 32 bytes */
};
struct acpi_processor_errata {
	u8                         smp;                  /*     0     1 */

	/* XXX 3 bytes hole, try to pack */

	struct {
		u8                 throttle:1;           /*     4: 0  1 */
		u8                 fdma:1;               /*     4: 1  1 */
		u8                 reserved:6;           /*     4: 2  1 */

		/* XXX 3 bytes hole, try to pack */

		u32                bmisx;                /*     8     4 */
	} piix4;                                         /*     4     8 */

	/* size: 12, cachelines: 1, members: 2 */
	/* sum members: 9, holes: 1, sum holes: 3 */
	/* last cacheline: 12 bytes */
};
struct acpi_table_ecdt {
	struct acpi_table_header   header;               /*     0    36 */
	struct acpi_generic_address control;             /*    36    12 */
	struct acpi_generic_address data;                /*    48    12 */
	u32                        uid;                  /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u8                         gpe;                  /*    64     1 */
	u8                         id[1];                /*    65     1 */

	/* size: 66, cachelines: 2, members: 6 */
	/* last cacheline: 2 bytes */
} __attribute__((__packed__));
struct acpi_ec {
	acpi_handle                handle;               /*     0     8 */
	int                        gpe;                  /*     8     4 */
	int                        irq;                  /*    12     4 */
	long unsigned int          command_addr;         /*    16     8 */
	long unsigned int          data_addr;            /*    24     8 */
	bool                       global_lock;          /*    32     1 */

	/* XXX 7 bytes hole, try to pack */

	long unsigned int          flags;                /*    40     8 */
	long unsigned int          reference_count;      /*    48     8 */
	struct mutex               mutex;                /*    56    32 */
	/* --- cacheline 1 boundary (64 bytes) was 24 bytes ago --- */
	wait_queue_head_t          wait;                 /*    88    24 */
	struct list_head           list;                 /*   112    16 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct transaction *       curr;                 /*   128     8 */
	spinlock_t                 lock;                 /*   136     4 */

	/* XXX 4 bytes hole, try to pack */

	struct work_struct         work;                 /*   144    64 */
	/* --- cacheline 3 boundary (192 bytes) was 16 bytes ago --- */
	long unsigned int          timestamp;            /*   208     8 */
	long unsigned int          nr_pending_queries;   /*   216     8 */
	unsigned int               events_in_progress;   /*   224     4 */
	unsigned int               queries_in_progress;  /*   228     4 */
	bool                       busy_polling;         /*   232     1 */

	/* XXX 3 bytes hole, try to pack */

	unsigned int               polling_guard;        /*   236     4 */

	/* size: 240, cachelines: 4, members: 20 */
	/* sum members: 226, holes: 3, sum holes: 14 */
	/* last cacheline: 48 bytes */
};
struct transaction {
	const u8  *                wdata;                /*     0     8 */
	u8 *                       rdata;                /*     8     8 */
	short unsigned int         irq_count;            /*    16     2 */
	u8                         command;              /*    18     1 */
	u8                         wi;                   /*    19     1 */
	u8                         ri;                   /*    20     1 */
	u8                         wlen;                 /*    21     1 */
	u8                         rlen;                 /*    22     1 */
	u8                         flags;                /*    23     1 */

	/* size: 24, cachelines: 1, members: 9 */
	/* last cacheline: 24 bytes */
};
struct acpi_ec_query_handler {
	struct list_head           node;                 /*     0    16 */
	acpi_ec_query_func         func;                 /*    16     8 */
	acpi_handle                handle;               /*    24     8 */
	void *                     data;                 /*    32     8 */
	u8                         query_bit;            /*    40     1 */

	/* XXX 3 bytes hole, try to pack */

	struct kref                kref;                 /*    44     4 */

	/* size: 48, cachelines: 1, members: 6 */
	/* sum members: 45, holes: 1, sum holes: 3 */
	/* last cacheline: 48 bytes */
};
struct acpi_ec_query {
	struct transaction         transaction;          /*     0    24 */
	struct work_struct         work;                 /*    24    64 */
	/* --- cacheline 1 boundary (64 bytes) was 24 bytes ago --- */
	struct acpi_ec_query_handler * handler;          /*    88     8 */
	struct acpi_ec *           ec;                   /*    96     8 */

	/* size: 104, cachelines: 2, members: 4 */
	/* last cacheline: 40 bytes */
};
struct dock_station {
	acpi_handle                handle;               /*     0     8 */
	long unsigned int          last_dock_time;       /*     8     8 */
	u32                        flags;                /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           dependent_devices;    /*    24    16 */
	struct list_head           sibling;              /*    40    16 */
	struct platform_device *   dock_device;          /*    56     8 */

	/* size: 64, cachelines: 1, members: 6 */
	/* sum members: 60, holes: 1, sum holes: 4 */
};
struct dock_dependent_device {
	struct list_head           list;                 /*     0    16 */
	struct acpi_device *       adev;                 /*    16     8 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct acpi_pci_root_info {
	struct acpi_pci_root *     root;                 /*     0     8 */
	struct acpi_device *       bridge;               /*     8     8 */
	struct acpi_pci_root_ops * ops;                  /*    16     8 */
	struct list_head           resources;            /*    24    16 */
	char                       name[16];             /*    40    16 */

	/* size: 56, cachelines: 1, members: 5 */
	/* last cacheline: 56 bytes */
};
struct acpi_pci_root_ops {
	struct pci_ops *           pci_ops;              /*     0     8 */
	int                        (*init_info)(struct acpi_pci_root_info *); /*     8     8 */
	void                       (*release_info)(struct acpi_pci_root_info *); /*    16     8 */
	int                        (*prepare_resources)(struct acpi_pci_root_info *); /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct pci_osc_bit_struct {
	u32                        bit;                  /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	char *                     desc;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct acpi_handle_node {
	struct list_head           node;                 /*     0    16 */
	acpi_handle                handle;               /*    16     8 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct acpi_pci_link_irq {
	u32                        active;               /*     0     4 */
	u8                         triggering;           /*     4     1 */
	u8                         polarity;             /*     5     1 */
	u8                         resource_type;        /*     6     1 */
	u8                         possible_count;       /*     7     1 */
	u32                        possible[16];         /*     8    64 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	u8                         initialized:1;        /*    72: 0  1 */
	u8                         reserved:7;           /*    72: 1  1 */

	/* size: 76, cachelines: 2, members: 8 */
	/* padding: 3 */
	/* last cacheline: 12 bytes */
};
struct acpi_pci_link {
	struct list_head           list;                 /*     0    16 */
	struct acpi_device *       device;               /*    16     8 */
	struct acpi_pci_link_irq   irq;                  /*    24    76 */

	/* XXX last struct has 3 bytes of padding */

	/* --- cacheline 1 boundary (64 bytes) was 36 bytes ago --- */
	int                        refcnt;               /*   100     4 */

	/* size: 104, cachelines: 2, members: 4 */
	/* paddings: 1, sum paddings: 3 */
	/* last cacheline: 40 bytes */
};
struct acpi_pci_routing_table {
	u32                        length;               /*     0     4 */
	u32                        pin;                  /*     4     4 */
	u64                        address;              /*     8     8 */
	u32                        source_index;         /*    16     4 */
	char                       source[4];            /*    20     4 */

	/* size: 24, cachelines: 1, members: 5 */
	/* last cacheline: 24 bytes */
};
struct acpi_prt_entry {
	struct acpi_pci_id         id;                   /*     0     8 */
	u8                         pin;                  /*     8     1 */

	/* XXX 7 bytes hole, try to pack */

	acpi_handle                link;                 /*    16     8 */
	u32                        index;                /*    24     4 */

	/* size: 32, cachelines: 1, members: 4 */
	/* sum members: 21, holes: 1, sum holes: 7 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct prt_quirk {
	const struct dmi_system_id  * system;            /*     0     8 */
	unsigned int               segment;              /*     8     4 */
	unsigned int               bus;                  /*    12     4 */
	unsigned int               device;               /*    16     4 */
	unsigned char              pin;                  /*    20     1 */

	/* XXX 3 bytes hole, try to pack */

	const char  *              source;               /*    24     8 */
	const char  *              actual_source;        /*    32     8 */

	/* size: 40, cachelines: 1, members: 7 */
	/* sum members: 37, holes: 1, sum holes: 3 */
	/* last cacheline: 40 bytes */
};
struct lpss_clk_data {
	const char  *              name;                 /*     0     8 */
	struct clk *               clk;                  /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct lpss_device_desc {
	unsigned int               flags;                /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	const char  *              clk_con_id;           /*     8     8 */
	unsigned int               prv_offset;           /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	size_t                     prv_size_override;    /*    24     8 */
	struct property_entry *    properties;           /*    32     8 */
	void                       (*setup)(struct lpss_private_data *); /*    40     8 */
	bool                       resume_from_noirq;    /*    48     1 */

	/* size: 56, cachelines: 1, members: 7 */
	/* sum members: 41, holes: 2, sum holes: 8 */
	/* padding: 7 */
	/* last cacheline: 56 bytes */
};
struct lpss_private_data {
	struct acpi_device *       adev;                 /*     0     8 */
	void *                     mmio_base;            /*     8     8 */
	resource_size_t            mmio_size;            /*    16     8 */
	unsigned int               fixed_clk_rate;       /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	struct clk *               clk;                  /*    32     8 */
	const struct lpss_device_desc  * dev_desc;       /*    40     8 */
	u32                        prv_reg_ctx[9];       /*    48    36 */

	/* size: 88, cachelines: 2, members: 7 */
	/* sum members: 80, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct lpss_device_links {
	const char  *              supplier_hid;         /*     0     8 */
	const char  *              supplier_uid;         /*     8     8 */
	const char  *              consumer_hid;         /*    16     8 */
	const char  *              consumer_uid;         /*    24     8 */
	u32                        flags;                /*    32     4 */

	/* XXX 4 bytes hole, try to pack */

	const struct dmi_system_id  * dep_missing_ids;   /*    40     8 */

	/* size: 48, cachelines: 1, members: 6 */
	/* sum members: 44, holes: 1, sum holes: 4 */
	/* last cacheline: 48 bytes */
};
struct hid_uid {
	const char  *              hid;                  /*     0     8 */
	const char  *              uid;                  /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct fch_clk_data {
	void *                     base;                 /*     0     8 */
	u32                        is_rv;                /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct apd_device_desc {
	unsigned int               fixed_clk_rate;       /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct property_entry *    properties;           /*     8     8 */
	int                        (*setup)(struct apd_private_data *); /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct apd_private_data {
	struct clk *               clk;                  /*     0     8 */
	struct acpi_device *       adev;                 /*     8     8 */
	const struct apd_device_desc  * dev_desc;        /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct acpi_power_dependent_device {
	struct device *            dev;                  /*     0     8 */
	struct list_head           node;                 /*     8    16 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct acpi_power_resource {
	struct acpi_device         device __attribute__((__aligned__(8))); /*     0  1696 */
	/* --- cacheline 26 boundary (1664 bytes) was 32 bytes ago --- */
	struct list_head           list_node;            /*  1696    16 */
	char *                     name;                 /*  1712     8 */
	u32                        system_level;         /*  1720     4 */
	u32                        order;                /*  1724     4 */
	/* --- cacheline 27 boundary (1728 bytes) --- */
	unsigned int               ref_count;            /*  1728     4 */
	bool                       wakeup_enabled;       /*  1732     1 */

	/* XXX 3 bytes hole, try to pack */

	struct mutex               resource_lock;        /*  1736    32 */
	struct list_head           dependents;           /*  1768    16 */

	/* size: 1784, cachelines: 28, members: 9 */
	/* sum members: 1781, holes: 1, sum holes: 3 */
	/* forced alignments: 1 */
	/* last cacheline: 56 bytes */
} __attribute__((__aligned__(8)));
struct acpi_power_resource_entry {
	struct list_head           node;                 /*     0    16 */
	struct acpi_power_resource * resource;           /*    16     8 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct acpi_bus_event {
	struct list_head           node;                 /*     0    16 */
	acpi_device_class          device_class;         /*    16    20 */
	acpi_bus_id                bus_id;               /*    36     8 */
	u32                        type;                 /*    44     4 */
	u32                        data;                 /*    48     4 */

	/* size: 56, cachelines: 1, members: 5 */
	/* padding: 4 */
	/* last cacheline: 56 bytes */
};
struct acpi_genl_event {
	acpi_device_class          device_class;         /*     0    20 */
	char                       bus_id[15];           /*    20    15 */

	/* XXX 1 byte hole, try to pack */

	u32                        type;                 /*    36     4 */
	u32                        data;                 /*    40     4 */

	/* size: 44, cachelines: 1, members: 4 */
	/* sum members: 43, holes: 1, sum holes: 1 */
	/* last cacheline: 44 bytes */
};
struct acpi_ged_device {
	struct device *            dev;                  /*     0     8 */
	struct list_head           event_list;           /*     8    16 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct acpi_ged_event {
	struct list_head           node;                 /*     0    16 */
	struct device *            dev;                  /*    16     8 */
	unsigned int               gsi;                  /*    24     4 */
	unsigned int               irq;                  /*    28     4 */
	acpi_handle                handle;               /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct acpi_table_bert {
	struct acpi_table_header   header;               /*     0    36 */
	u32                        region_length;        /*    36     4 */
	u64                        address;              /*    40     8 */

	/* size: 48, cachelines: 1, members: 3 */
	/* last cacheline: 48 bytes */
};
struct acpi_table_attr {
	struct bin_attribute       attr;                 /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	char                       name[4];              /*    64     4 */
	int                        instance;             /*    68     4 */
	char                       filename[8];          /*    72     8 */
	struct list_head           node;                 /*    80    16 */

	/* size: 96, cachelines: 2, members: 5 */
	/* last cacheline: 32 bytes */
};
struct acpi_data_attr {
	struct bin_attribute       attr;                 /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u64                        addr;                 /*    64     8 */

	/* size: 72, cachelines: 2, members: 2 */
	/* last cacheline: 8 bytes */
};
struct acpi_data_obj {
	char *                     name;                 /*     0     8 */
	int                        (*fn)(void *, struct acpi_data_attr *); /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct event_counter {
	u32                        count;                /*     0     4 */
	u32                        flags;                /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct acpi_device_properties {
	const guid_t  *            guid;                 /*     0     8 */
	const union acpi_object  * properties;           /*     8     8 */
	struct list_head           list;                 /*    16    16 */

	/* size: 32, cachelines: 1, members: 3 */
	/* last cacheline: 32 bytes */
};
struct override_status_id {
	struct acpi_device_id      hid[2];               /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct x86_cpu_id          cpu_ids[2];           /*    64    48 */
	struct dmi_system_id       dmi_ids[2];           /*   112   688 */
	/* --- cacheline 12 boundary (768 bytes) was 32 bytes ago --- */
	const char  *              uid;                  /*   800     8 */
	const char  *              path;                 /*   808     8 */
	long long unsigned int     status;               /*   816     8 */

	/* size: 824, cachelines: 13, members: 6 */
	/* last cacheline: 56 bytes */
};
struct acpi_lpat {
	int                        temp;                 /*     0     4 */
	int                        raw;                  /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct acpi_lpat_conversion_table {
	struct acpi_lpat *         lpat;                 /*     0     8 */
	int                        lpat_count;           /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct acpi_table_lpit {
	struct acpi_table_header   header;               /*     0    36 */

	/* size: 36, cachelines: 1, members: 1 */
	/* last cacheline: 36 bytes */
};
struct acpi_lpit_header {
	u32                        type;                 /*     0     4 */
	u32                        length;               /*     4     4 */
	u16                        unique_id;            /*     8     2 */
	u16                        reserved;             /*    10     2 */
	u32                        flags;                /*    12     4 */

	/* size: 16, cachelines: 1, members: 5 */
	/* last cacheline: 16 bytes */
};
struct acpi_lpit_native {
	struct acpi_lpit_header    header;               /*     0    16 */
	struct acpi_generic_address entry_trigger;       /*    16    12 */
	u32                        residency;            /*    28     4 */
	u32                        latency;              /*    32     4 */
	struct acpi_generic_address residency_counter;   /*    36    12 */
	u64                        counter_frequency;    /*    48     8 */

	/* size: 56, cachelines: 1, members: 6 */
	/* last cacheline: 56 bytes */
} __attribute__((__packed__));
struct lpit_residency_info {
	struct acpi_generic_address gaddr;               /*     0    12 */

	/* XXX 4 bytes hole, try to pack */

	u64                        frequency;            /*    16     8 */
	void *                     iomem_addr;           /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct acpi_table_wdat {
	struct acpi_table_header   header;               /*     0    36 */
	u32                        header_length;        /*    36     4 */
	u16                        pci_segment;          /*    40     2 */
	u8                         pci_bus;              /*    42     1 */
	u8                         pci_device;           /*    43     1 */
	u8                         pci_function;         /*    44     1 */
	u8                         reserved[3];          /*    45     3 */
	u32                        timer_period;         /*    48     4 */
	u32                        max_count;            /*    52     4 */
	u32                        min_count;            /*    56     4 */
	u8                         flags;                /*    60     1 */
	u8                         reserved2[3];         /*    61     3 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u32                        entries;              /*    64     4 */

	/* size: 68, cachelines: 2, members: 13 */
	/* last cacheline: 4 bytes */
};
struct acpi_wdat_entry {
	u8                         action;               /*     0     1 */
	u8                         instruction;          /*     1     1 */
	u16                        reserved;             /*     2     2 */
	struct acpi_generic_address register_region;     /*     4    12 */
	u32                        value;                /*    16     4 */
	u32                        mask;                 /*    20     4 */

	/* size: 24, cachelines: 1, members: 6 */
	/* last cacheline: 24 bytes */
} __attribute__((__packed__));
struct acpi_common_descriptor {
	void *                     common_pointer;       /*     0     8 */
	u8                         descriptor_type;      /*     8     1 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 7 */
	/* last cacheline: 16 bytes */
};
union acpi_descriptor {
	struct acpi_common_descriptor common;          /*     0    16 */
	union acpi_operand_object  object;             /*     0    72 */
	struct acpi_namespace_node node;               /*     0    48 */
	union acpi_parse_object    op;                 /*     0   200 */
};
struct acpi_create_field_info {
	struct acpi_namespace_node * region_node;        /*     0     8 */
	struct acpi_namespace_node * field_node;         /*     8     8 */
	struct acpi_namespace_node * register_node;      /*    16     8 */
	struct acpi_namespace_node * data_register_node; /*    24     8 */
	struct acpi_namespace_node * connection_node;    /*    32     8 */
	u8 *                       resource_buffer;      /*    40     8 */
	u32                        bank_value;           /*    48     4 */
	u32                        field_bit_position;   /*    52     4 */
	u32                        field_bit_length;     /*    56     4 */
	u16                        resource_length;      /*    60     2 */
	u16                        pin_number_index;     /*    62     2 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u8                         field_flags;          /*    64     1 */
	u8                         attribute;            /*    65     1 */
	u8                         field_type;           /*    66     1 */
	u8                         access_length;        /*    67     1 */

	/* size: 72, cachelines: 2, members: 15 */
	/* padding: 4 */
	/* last cacheline: 8 bytes */
};
struct acpi_init_walk_info {
	u32                        table_index;          /*     0     4 */
	u32                        object_count;         /*     4     4 */
	u32                        method_count;         /*     8     4 */
	u32                        serial_method_count;  /*    12     4 */
	u32                        non_serial_method_count; /*    16     4 */
	u32                        serialized_method_count; /*    20     4 */
	u32                        device_count;         /*    24     4 */
	u32                        op_region_count;      /*    28     4 */
	u32                        field_count;          /*    32     4 */
	u32                        buffer_count;         /*    36     4 */
	u32                        package_count;        /*    40     4 */
	u32                        op_region_init;       /*    44     4 */
	u32                        field_init;           /*    48     4 */
	u32                        buffer_init;          /*    52     4 */
	u32                        package_init;         /*    56     4 */
	acpi_owner_id              owner_id;             /*    60     2 */

	/* size: 64, cachelines: 1, members: 16 */
	/* padding: 2 */
};
struct acpi_name_info {
	char                       name[4];              /*     0     4 */
	u16                        argument_list;        /*     4     2 */
	u8                         expected_btypes;      /*     6     1 */

	/* size: 7, cachelines: 1, members: 3 */
	/* last cacheline: 7 bytes */
} __attribute__((__packed__));
struct acpi_package_info {
	u8                         type;                 /*     0     1 */
	u8                         object_type1;         /*     1     1 */
	u8                         count1;               /*     2     1 */
	u8                         object_type2;         /*     3     1 */
	u8                         count2;               /*     4     1 */
	u16                        reserved;             /*     5     2 */

	/* size: 7, cachelines: 1, members: 6 */
	/* last cacheline: 7 bytes */
} __attribute__((__packed__));
struct acpi_package_info2 {
	u8                         type;                 /*     0     1 */
	u8                         count;                /*     1     1 */
	u8                         object_type[4];       /*     2     4 */
	u8                         reserved;             /*     6     1 */

	/* size: 7, cachelines: 1, members: 4 */
	/* last cacheline: 7 bytes */
};
struct acpi_package_info3 {
	u8                         type;                 /*     0     1 */
	u8                         count;                /*     1     1 */
	u8                         object_type[2];       /*     2     2 */
	u8                         tail_object_type;     /*     4     1 */
	u16                        reserved;             /*     5     2 */

	/* size: 7, cachelines: 1, members: 5 */
	/* last cacheline: 7 bytes */
} __attribute__((__packed__));
struct acpi_package_info4 {
	u8                         type;                 /*     0     1 */
	u8                         object_type1;         /*     1     1 */
	u8                         count1;               /*     2     1 */
	u8                         sub_object_types;     /*     3     1 */
	u8                         pkg_count;            /*     4     1 */
	u16                        reserved;             /*     5     2 */

	/* size: 7, cachelines: 1, members: 6 */
	/* last cacheline: 7 bytes */
} __attribute__((__packed__));
union acpi_predefined_info {
	struct acpi_name_info      info;               /*     0     7 */
	struct acpi_package_info   ret_info;           /*     0     7 */
	struct acpi_package_info2  ret_info2;          /*     0     7 */
	struct acpi_package_info3  ret_info3;          /*     0     7 */
	struct acpi_package_info4  ret_info4;          /*     0     7 */
};
struct acpi_evaluate_info {
	struct acpi_namespace_node * prefix_node;        /*     0     8 */
	const char  *              relative_pathname;    /*     8     8 */
	union acpi_operand_object * * parameters;        /*    16     8 */
	struct acpi_namespace_node * node;               /*    24     8 */
	union acpi_operand_object * obj_desc;            /*    32     8 */
	char *                     full_pathname;        /*    40     8 */
	const union acpi_predefined_info  * predefined;  /*    48     8 */
	union acpi_operand_object * return_object;       /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	union acpi_operand_object * parent_package;      /*    64     8 */
	u32                        return_flags;         /*    72     4 */
	u32                        return_btype;         /*    76     4 */
	u16                        param_count;          /*    80     2 */
	u16                        node_flags;           /*    82     2 */
	u8                         pass_number;          /*    84     1 */
	u8                         return_object_type;   /*    85     1 */
	u8                         flags;                /*    86     1 */

	/* size: 88, cachelines: 2, members: 16 */
	/* padding: 1 */
	/* last cacheline: 24 bytes */
};
struct acpi_fixed_event_handler {
	acpi_event_handler         handler;              /*     0     8 */
	void *                     context;              /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct acpi_fixed_event_info {
	u8                         status_register_id;   /*     0     1 */
	u8                         enable_register_id;   /*     1     1 */
	u16                        status_bit_mask;      /*     2     2 */
	u16                        enable_bit_mask;      /*     4     2 */

	/* size: 6, cachelines: 1, members: 4 */
	/* last cacheline: 6 bytes */
};
struct acpi_gpe_walk_info {
	struct acpi_namespace_node * gpe_device;         /*     0     8 */
	struct acpi_gpe_block_info * gpe_block;          /*     8     8 */
	u16                        count;                /*    16     2 */
	acpi_owner_id              owner_id;             /*    18     2 */
	u8                         execute_by_owner_id;  /*    20     1 */

	/* size: 24, cachelines: 1, members: 5 */
	/* padding: 3 */
	/* last cacheline: 24 bytes */
};
struct acpi_gpe_device_info {
	u32                        index;                /*     0     4 */
	u32                        next_block_base_index; /*     4     4 */
	acpi_status                status;               /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct acpi_namespace_node * gpe_device;         /*    16     8 */

	/* size: 24, cachelines: 1, members: 4 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct acpi_reg_walk_info {
	u32                        function;             /*     0     4 */
	u32                        reg_run_count;        /*     4     4 */
	acpi_adr_space_type        space_id;             /*     8     1 */

	/* size: 12, cachelines: 1, members: 3 */
	/* padding: 3 */
	/* last cacheline: 12 bytes */
};
struct acpi_sci_handler_info {
	struct acpi_sci_handler_info * next;             /*     0     8 */
	acpi_sci_handler           address;              /*     8     8 */
	void *                     context;              /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct acpi_signal_fatal_info {
	u32                        type;                 /*     0     4 */
	u32                        code;                 /*     4     4 */
	u32                        argument;             /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct acpi_gpe_block_status_context {
	struct acpi_gpe_register_info * gpe_skip_register_info; /*     0     8 */
	u8                         gpe_skip_mask;        /*     8     1 */
	u8                         retval;               /*     9     1 */

	/* size: 16, cachelines: 1, members: 3 */
	/* padding: 6 */
	/* last cacheline: 16 bytes */
};
struct acpi_bit_register_info {
	u8                         parent_register;      /*     0     1 */
	u8                         bit_position;         /*     1     1 */
	u16                        access_bit_mask;      /*     2     2 */

	/* size: 4, cachelines: 1, members: 3 */
	/* last cacheline: 4 bytes */
};
struct acpi_port_info {
	char *                     name;                 /*     0     8 */
	u16                        start;                /*     8     2 */
	u16                        end;                  /*    10     2 */
	u8                         osi_dependency;       /*    12     1 */

	/* size: 16, cachelines: 1, members: 4 */
	/* padding: 3 */
	/* last cacheline: 16 bytes */
};
struct acpi_pci_device {
	acpi_handle                device;               /*     0     8 */
	struct acpi_pci_device *   next;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct acpi_device_walk_info {
	struct acpi_table_desc *   table_desc;           /*     0     8 */
	struct acpi_evaluate_info * evaluate_info;       /*     8     8 */
	u32                        device_count;         /*    16     4 */
	u32                        num_STA;              /*    20     4 */
	u32                        num_INI;              /*    24     4 */

	/* size: 32, cachelines: 1, members: 5 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct acpi_table_list {
	struct acpi_table_desc *   tables;               /*     0     8 */
	u32                        current_table_count;  /*     8     4 */
	u32                        max_table_count;      /*    12     4 */
	u8                         flags;                /*    16     1 */

	/* size: 24, cachelines: 1, members: 4 */
	/* padding: 7 */
	/* last cacheline: 24 bytes */
};
struct acpi_simple_repair_info {
	char                       name[4];              /*     0     4 */
	u32                        unexpected_btypes;    /*     4     4 */
	u32                        package_index;        /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	acpi_object_converter      object_converter;     /*    16     8 */

	/* size: 24, cachelines: 1, members: 4 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct acpi_repair_info {
	char                       name[4];              /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	acpi_repair_function       repair_function;      /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct acpi_namestring_info {
	const char  *              external_name;        /*     0     8 */
	const char  *              next_external_char;   /*     8     8 */
	char *                     internal_name;        /*    16     8 */
	u32                        length;               /*    24     4 */
	u32                        num_segments;         /*    28     4 */
	u32                        num_carats;           /*    32     4 */
	u8                         fully_qualified;      /*    36     1 */

	/* size: 40, cachelines: 1, members: 7 */
	/* padding: 3 */
	/* last cacheline: 40 bytes */
};
struct acpi_rw_lock {
	void *                     writer_mutex;         /*     0     8 */
	void *                     reader_mutex;         /*     8     8 */
	u32                        num_readers;          /*    16     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct acpi_get_devices_info {
	acpi_walk_callback         user_function;        /*     0     8 */
	void *                     context;              /*     8     8 */
	const char  *              hid;                  /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct aml_resource_small_header {
	u8                         descriptor_type;      /*     0     1 */

	/* size: 1, cachelines: 1, members: 1 */
	/* last cacheline: 1 bytes */
};
struct aml_resource_irq {
	u8                         descriptor_type;      /*     0     1 */
	u16                        irq_mask;             /*     1     2 */
	u8                         flags;                /*     3     1 */

	/* size: 4, cachelines: 1, members: 3 */
	/* last cacheline: 4 bytes */
} __attribute__((__packed__));
struct aml_resource_dma {
	u8                         descriptor_type;      /*     0     1 */
	u8                         dma_channel_mask;     /*     1     1 */
	u8                         flags;                /*     2     1 */

	/* size: 3, cachelines: 1, members: 3 */
	/* last cacheline: 3 bytes */
};
struct aml_resource_start_dependent {
	u8                         descriptor_type;      /*     0     1 */
	u8                         flags;                /*     1     1 */

	/* size: 2, cachelines: 1, members: 2 */
	/* last cacheline: 2 bytes */
};
struct aml_resource_end_dependent {
	u8                         descriptor_type;      /*     0     1 */

	/* size: 1, cachelines: 1, members: 1 */
	/* last cacheline: 1 bytes */
};
struct aml_resource_io {
	u8                         descriptor_type;      /*     0     1 */
	u8                         flags;                /*     1     1 */
	u16                        minimum;              /*     2     2 */
	u16                        maximum;              /*     4     2 */
	u8                         alignment;            /*     6     1 */
	u8                         address_length;       /*     7     1 */

	/* size: 8, cachelines: 1, members: 6 */
	/* last cacheline: 8 bytes */
};
struct aml_resource_fixed_io {
	u8                         descriptor_type;      /*     0     1 */
	u16                        address;              /*     1     2 */
	u8                         address_length;       /*     3     1 */

	/* size: 4, cachelines: 1, members: 3 */
	/* last cacheline: 4 bytes */
} __attribute__((__packed__));
struct aml_resource_vendor_small {
	u8                         descriptor_type;      /*     0     1 */

	/* size: 1, cachelines: 1, members: 1 */
	/* last cacheline: 1 bytes */
};
struct aml_resource_end_tag {
	u8                         descriptor_type;      /*     0     1 */
	u8                         checksum;             /*     1     1 */

	/* size: 2, cachelines: 1, members: 2 */
	/* last cacheline: 2 bytes */
};
struct aml_resource_fixed_dma {
	u8                         descriptor_type;      /*     0     1 */
	u16                        request_lines;        /*     1     2 */
	u16                        channels;             /*     3     2 */
	u8                         width;                /*     5     1 */

	/* size: 6, cachelines: 1, members: 4 */
	/* last cacheline: 6 bytes */
} __attribute__((__packed__));
struct aml_resource_large_header {
	u8                         descriptor_type;      /*     0     1 */
	u16                        resource_length;      /*     1     2 */

	/* size: 3, cachelines: 1, members: 2 */
	/* last cacheline: 3 bytes */
} __attribute__((__packed__));
struct aml_resource_memory24 {
	u8                         descriptor_type;      /*     0     1 */
	u16                        resource_length;      /*     1     2 */
	u8                         flags;                /*     3     1 */
	u16                        minimum;              /*     4     2 */
	u16                        maximum;              /*     6     2 */
	u16                        alignment;            /*     8     2 */
	u16                        address_length;       /*    10     2 */

	/* size: 12, cachelines: 1, members: 7 */
	/* last cacheline: 12 bytes */
} __attribute__((__packed__));
struct aml_resource_vendor_large {
	u8                         descriptor_type;      /*     0     1 */
	u16                        resource_length;      /*     1     2 */

	/* size: 3, cachelines: 1, members: 2 */
	/* last cacheline: 3 bytes */
} __attribute__((__packed__));
struct aml_resource_memory32 {
	u8                         descriptor_type;      /*     0     1 */
	u16                        resource_length;      /*     1     2 */
	u8                         flags;                /*     3     1 */
	u32                        minimum;              /*     4     4 */
	u32                        maximum;              /*     8     4 */
	u32                        alignment;            /*    12     4 */
	u32                        address_length;       /*    16     4 */

	/* size: 20, cachelines: 1, members: 7 */
	/* last cacheline: 20 bytes */
} __attribute__((__packed__));
struct aml_resource_fixed_memory32 {
	u8                         descriptor_type;      /*     0     1 */
	u16                        resource_length;      /*     1     2 */
	u8                         flags;                /*     3     1 */
	u32                        address;              /*     4     4 */
	u32                        address_length;       /*     8     4 */

	/* size: 12, cachelines: 1, members: 5 */
	/* last cacheline: 12 bytes */
} __attribute__((__packed__));
struct aml_resource_address {
	u8                         descriptor_type;      /*     0     1 */
	u16                        resource_length;      /*     1     2 */
	u8                         resource_type;        /*     3     1 */
	u8                         flags;                /*     4     1 */
	u8                         specific_flags;       /*     5     1 */

	/* size: 6, cachelines: 1, members: 5 */
	/* last cacheline: 6 bytes */
} __attribute__((__packed__));
struct aml_resource_extended_address64 {
	u8                         descriptor_type;      /*     0     1 */
	u16                        resource_length;      /*     1     2 */
	u8                         resource_type;        /*     3     1 */
	u8                         flags;                /*     4     1 */
	u8                         specific_flags;       /*     5     1 */
	u8                         revision_ID;          /*     6     1 */
	u8                         reserved;             /*     7     1 */
	u64                        granularity;          /*     8     8 */
	u64                        minimum;              /*    16     8 */
	u64                        maximum;              /*    24     8 */
	u64                        translation_offset;   /*    32     8 */
	u64                        address_length;       /*    40     8 */
	u64                        type_specific;        /*    48     8 */

	/* size: 56, cachelines: 1, members: 13 */
	/* last cacheline: 56 bytes */
} __attribute__((__packed__));
struct aml_resource_address64 {
	u8                         descriptor_type;      /*     0     1 */
	u16                        resource_length;      /*     1     2 */
	u8                         resource_type;        /*     3     1 */
	u8                         flags;                /*     4     1 */
	u8                         specific_flags;       /*     5     1 */
	u64                        granularity;          /*     6     8 */
	u64                        minimum;              /*    14     8 */
	u64                        maximum;              /*    22     8 */
	u64                        translation_offset;   /*    30     8 */
	u64                        address_length;       /*    38     8 */

	/* size: 46, cachelines: 1, members: 10 */
	/* last cacheline: 46 bytes */
} __attribute__((__packed__));
struct aml_resource_address32 {
	u8                         descriptor_type;      /*     0     1 */
	u16                        resource_length;      /*     1     2 */
	u8                         resource_type;        /*     3     1 */
	u8                         flags;                /*     4     1 */
	u8                         specific_flags;       /*     5     1 */
	u32                        granularity;          /*     6     4 */
	u32                        minimum;              /*    10     4 */
	u32                        maximum;              /*    14     4 */
	u32                        translation_offset;   /*    18     4 */
	u32                        address_length;       /*    22     4 */

	/* size: 26, cachelines: 1, members: 10 */
	/* last cacheline: 26 bytes */
} __attribute__((__packed__));
struct aml_resource_address16 {
	u8                         descriptor_type;      /*     0     1 */
	u16                        resource_length;      /*     1     2 */
	u8                         resource_type;        /*     3     1 */
	u8                         flags;                /*     4     1 */
	u8                         specific_flags;       /*     5     1 */
	u16                        granularity;          /*     6     2 */
	u16                        minimum;              /*     8     2 */
	u16                        maximum;              /*    10     2 */
	u16                        translation_offset;   /*    12     2 */
	u16                        address_length;       /*    14     2 */

	/* size: 16, cachelines: 1, members: 10 */
	/* last cacheline: 16 bytes */
} __attribute__((__packed__));
struct aml_resource_extended_irq {
	u8                         descriptor_type;      /*     0     1 */
	u16                        resource_length;      /*     1     2 */
	u8                         flags;                /*     3     1 */
	u8                         interrupt_count;      /*     4     1 */
	u32                        interrupts[1];        /*     5     4 */

	/* size: 9, cachelines: 1, members: 5 */
	/* last cacheline: 9 bytes */
} __attribute__((__packed__));
struct aml_resource_generic_register {
	u8                         descriptor_type;      /*     0     1 */
	u16                        resource_length;      /*     1     2 */
	u8                         address_space_id;     /*     3     1 */
	u8                         bit_width;            /*     4     1 */
	u8                         bit_offset;           /*     5     1 */
	u8                         access_size;          /*     6     1 */
	u64                        address;              /*     7     8 */

	/* size: 15, cachelines: 1, members: 7 */
	/* last cacheline: 15 bytes */
} __attribute__((__packed__));
struct aml_resource_gpio {
	u8                         descriptor_type;      /*     0     1 */
	u16                        resource_length;      /*     1     2 */
	u8                         revision_id;          /*     3     1 */
	u8                         connection_type;      /*     4     1 */
	u16                        flags;                /*     5     2 */
	u16                        int_flags;            /*     7     2 */
	u8                         pin_config;           /*     9     1 */
	u16                        drive_strength;       /*    10     2 */
	u16                        debounce_timeout;     /*    12     2 */
	u16                        pin_table_offset;     /*    14     2 */
	u8                         res_source_index;     /*    16     1 */
	u16                        res_source_offset;    /*    17     2 */
	u16                        vendor_offset;        /*    19     2 */
	u16                        vendor_length;        /*    21     2 */

	/* size: 23, cachelines: 1, members: 14 */
	/* last cacheline: 23 bytes */
} __attribute__((__packed__));
struct aml_resource_common_serialbus {
	u8                         descriptor_type;      /*     0     1 */
	u16                        resource_length;      /*     1     2 */
	u8                         revision_id;          /*     3     1 */
	u8                         res_source_index;     /*     4     1 */
	u8                         type;                 /*     5     1 */
	u8                         flags;                /*     6     1 */
	u16                        type_specific_flags;  /*     7     2 */
	u8                         type_revision_id;     /*     9     1 */
	u16                        type_data_length;     /*    10     2 */

	/* size: 12, cachelines: 1, members: 9 */
	/* last cacheline: 12 bytes */
} __attribute__((__packed__));
struct aml_resource_i2c_serialbus {
	u8                         descriptor_type;      /*     0     1 */
	u16                        resource_length;      /*     1     2 */
	u8                         revision_id;          /*     3     1 */
	u8                         res_source_index;     /*     4     1 */
	u8                         type;                 /*     5     1 */
	u8                         flags;                /*     6     1 */
	u16                        type_specific_flags;  /*     7     2 */
	u8                         type_revision_id;     /*     9     1 */
	u16                        type_data_length;     /*    10     2 */
	u32                        connection_speed;     /*    12     4 */
	u16                        slave_address;        /*    16     2 */

	/* size: 18, cachelines: 1, members: 11 */
	/* last cacheline: 18 bytes */
} __attribute__((__packed__));
struct aml_resource_spi_serialbus {
	u8                         descriptor_type;      /*     0     1 */
	u16                        resource_length;      /*     1     2 */
	u8                         revision_id;          /*     3     1 */
	u8                         res_source_index;     /*     4     1 */
	u8                         type;                 /*     5     1 */
	u8                         flags;                /*     6     1 */
	u16                        type_specific_flags;  /*     7     2 */
	u8                         type_revision_id;     /*     9     1 */
	u16                        type_data_length;     /*    10     2 */
	u32                        connection_speed;     /*    12     4 */
	u8                         data_bit_length;      /*    16     1 */
	u8                         clock_phase;          /*    17     1 */
	u8                         clock_polarity;       /*    18     1 */
	u16                        device_selection;     /*    19     2 */

	/* size: 21, cachelines: 1, members: 14 */
	/* last cacheline: 21 bytes */
} __attribute__((__packed__));
struct aml_resource_uart_serialbus {
	u8                         descriptor_type;      /*     0     1 */
	u16                        resource_length;      /*     1     2 */
	u8                         revision_id;          /*     3     1 */
	u8                         res_source_index;     /*     4     1 */
	u8                         type;                 /*     5     1 */
	u8                         flags;                /*     6     1 */
	u16                        type_specific_flags;  /*     7     2 */
	u8                         type_revision_id;     /*     9     1 */
	u16                        type_data_length;     /*    10     2 */
	u32                        default_baud_rate;    /*    12     4 */
	u16                        rx_fifo_size;         /*    16     2 */
	u16                        tx_fifo_size;         /*    18     2 */
	u8                         parity;               /*    20     1 */
	u8                         lines_enabled;        /*    21     1 */

	/* size: 22, cachelines: 1, members: 14 */
	/* last cacheline: 22 bytes */
} __attribute__((__packed__));
struct aml_resource_pin_function {
	u8                         descriptor_type;      /*     0     1 */
	u16                        resource_length;      /*     1     2 */
	u8                         revision_id;          /*     3     1 */
	u16                        flags;                /*     4     2 */
	u8                         pin_config;           /*     6     1 */
	u16                        function_number;      /*     7     2 */
	u16                        pin_table_offset;     /*     9     2 */
	u8                         res_source_index;     /*    11     1 */
	u16                        res_source_offset;    /*    12     2 */
	u16                        vendor_offset;        /*    14     2 */
	u16                        vendor_length;        /*    16     2 */

	/* size: 18, cachelines: 1, members: 11 */
	/* last cacheline: 18 bytes */
} __attribute__((__packed__));
struct aml_resource_pin_config {
	u8                         descriptor_type;      /*     0     1 */
	u16                        resource_length;      /*     1     2 */
	u8                         revision_id;          /*     3     1 */
	u16                        flags;                /*     4     2 */
	u8                         pin_config_type;      /*     6     1 */
	u32                        pin_config_value;     /*     7     4 */
	u16                        pin_table_offset;     /*    11     2 */
	u8                         res_source_index;     /*    13     1 */
	u16                        res_source_offset;    /*    14     2 */
	u16                        vendor_offset;        /*    16     2 */
	u16                        vendor_length;        /*    18     2 */

	/* size: 20, cachelines: 1, members: 11 */
	/* last cacheline: 20 bytes */
} __attribute__((__packed__));
struct aml_resource_pin_group {
	u8                         descriptor_type;      /*     0     1 */
	u16                        resource_length;      /*     1     2 */
	u8                         revision_id;          /*     3     1 */
	u16                        flags;                /*     4     2 */
	u16                        pin_table_offset;     /*     6     2 */
	u16                        label_offset;         /*     8     2 */
	u16                        vendor_offset;        /*    10     2 */
	u16                        vendor_length;        /*    12     2 */

	/* size: 14, cachelines: 1, members: 8 */
	/* last cacheline: 14 bytes */
} __attribute__((__packed__));
struct aml_resource_pin_group_function {
	u8                         descriptor_type;      /*     0     1 */
	u16                        resource_length;      /*     1     2 */
	u8                         revision_id;          /*     3     1 */
	u16                        flags;                /*     4     2 */
	u16                        function_number;      /*     6     2 */
	u8                         res_source_index;     /*     8     1 */
	u16                        res_source_offset;    /*     9     2 */
	u16                        res_source_label_offset; /*    11     2 */
	u16                        vendor_offset;        /*    13     2 */
	u16                        vendor_length;        /*    15     2 */

	/* size: 17, cachelines: 1, members: 10 */
	/* last cacheline: 17 bytes */
} __attribute__((__packed__));
struct aml_resource_pin_group_config {
	u8                         descriptor_type;      /*     0     1 */
	u16                        resource_length;      /*     1     2 */
	u8                         revision_id;          /*     3     1 */
	u16                        flags;                /*     4     2 */
	u8                         pin_config_type;      /*     6     1 */
	u32                        pin_config_value;     /*     7     4 */
	u8                         res_source_index;     /*    11     1 */
	u16                        res_source_offset;    /*    12     2 */
	u16                        res_source_label_offset; /*    14     2 */
	u16                        vendor_offset;        /*    16     2 */
	u16                        vendor_length;        /*    18     2 */

	/* size: 20, cachelines: 1, members: 11 */
	/* last cacheline: 20 bytes */
} __attribute__((__packed__));
union aml_resource {
	u8                         descriptor_type;    /*     0     1 */
	struct aml_resource_small_header small_header; /*     0     1 */
	struct aml_resource_large_header large_header; /*     0     3 */
	struct aml_resource_irq    irq;                /*     0     4 */
	struct aml_resource_dma    dma;                /*     0     3 */
	struct aml_resource_start_dependent start_dpf; /*     0     2 */
	struct aml_resource_end_dependent end_dpf;     /*     0     1 */
	struct aml_resource_io     io;                 /*     0     8 */
	struct aml_resource_fixed_io fixed_io;         /*     0     4 */
	struct aml_resource_fixed_dma fixed_dma;       /*     0     6 */
	struct aml_resource_vendor_small vendor_small; /*     0     1 */
	struct aml_resource_end_tag end_tag;           /*     0     2 */
	struct aml_resource_memory24 memory24;         /*     0    12 */
	struct aml_resource_generic_register generic_reg; /*     0    15 */
	struct aml_resource_vendor_large vendor_large; /*     0     3 */
	struct aml_resource_memory32 memory32;         /*     0    20 */
	struct aml_resource_fixed_memory32 fixed_memory32; /*     0    12 */
	struct aml_resource_address16 address16;       /*     0    16 */
	struct aml_resource_address32 address32;       /*     0    26 */
	struct aml_resource_address64 address64;       /*     0    46 */
	struct aml_resource_extended_address64 ext_address64; /*     0    56 */
	struct aml_resource_extended_irq extended_irq; /*     0     9 */
	struct aml_resource_gpio   gpio;               /*     0    23 */
	struct aml_resource_i2c_serialbus i2c_serial_bus; /*     0    18 */
	struct aml_resource_spi_serialbus spi_serial_bus; /*     0    21 */
	struct aml_resource_uart_serialbus uart_serial_bus; /*     0    22 */
	struct aml_resource_common_serialbus common_serial_bus; /*     0    12 */
	struct aml_resource_pin_function pin_function; /*     0    18 */
	struct aml_resource_pin_config pin_config;     /*     0    20 */
	struct aml_resource_pin_group pin_group;       /*     0    14 */
	struct aml_resource_pin_group_function pin_group_function; /*     0    17 */
	struct aml_resource_pin_group_config pin_group_config; /*     0    20 */
	struct aml_resource_address address;           /*     0     6 */
	u32                        dword_item;         /*     0     4 */
	u16                        word_item;          /*     0     2 */
	u8                         byte_item;          /*     0     1 */
};
struct acpi_rsconvert_info {
	u8                         opcode;               /*     0     1 */
	u8                         resource_offset;      /*     1     1 */
	u8                         aml_offset;           /*     2     1 */
	u8                         value;                /*     3     1 */

	/* size: 4, cachelines: 1, members: 4 */
	/* last cacheline: 4 bytes */
};
struct acpi_vendor_uuid {
	u8                         subtype;              /*     0     1 */
	u8                         data[16];             /*     1    16 */

	/* size: 17, cachelines: 1, members: 2 */
	/* last cacheline: 17 bytes */
};
struct acpi_vendor_walk_info {
	struct acpi_vendor_uuid *  uuid;                 /*     0     8 */
	struct acpi_buffer *       buffer;               /*     8     8 */
	acpi_status                status;               /*    16     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct acpi_fadt_info {
	const char  *              name;                 /*     0     8 */
	u16                        address64;            /*     8     2 */
	u16                        address32;            /*    10     2 */
	u16                        length;               /*    12     2 */
	u8                         default_length;       /*    14     1 */
	u8                         flags;                /*    15     1 */

	/* size: 16, cachelines: 1, members: 6 */
	/* last cacheline: 16 bytes */
};
struct acpi_fadt_pm_info {
	struct acpi_generic_address * target;            /*     0     8 */
	u16                        source;               /*     8     2 */
	u8                         register_num;         /*    10     1 */

	/* size: 16, cachelines: 1, members: 3 */
	/* padding: 5 */
	/* last cacheline: 16 bytes */
};
struct acpi_table_rsdp {
	char                       signature[8];         /*     0     8 */
	u8                         checksum;             /*     8     1 */
	char                       oem_id[6];            /*     9     6 */
	u8                         revision;             /*    15     1 */
	u32                        rsdt_physical_address; /*    16     4 */
	u32                        length;               /*    20     4 */
	u64                        xsdt_physical_address; /*    24     8 */
	u8                         extended_checksum;    /*    32     1 */
	u8                         reserved[3];          /*    33     3 */

	/* size: 36, cachelines: 1, members: 9 */
	/* last cacheline: 36 bytes */
} __attribute__((__packed__));
struct acpi_address_range {
	struct acpi_address_range * next;                /*     0     8 */
	struct acpi_namespace_node * region_node;        /*     8     8 */
	acpi_physical_address      start_address;        /*    16     8 */
	acpi_physical_address      end_address;          /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct acpi_pkg_info {
	u8 *                       free_space;           /*     0     8 */
	acpi_size                  length;               /*     8     8 */
	u32                        object_space;         /*    16     4 */
	u32                        num_packages;         /*    20     4 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct acpi_exception_info {
	char *                     name;                 /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct acpi_mutex_info {
	void *                     mutex;                /*     0     8 */
	u32                        use_count;            /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        thread_id;            /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct acpi_comment_node {
	char *                     comment;              /*     0     8 */
	struct acpi_comment_node * next;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct acpi_interface_info {
	char *                     name;                 /*     0     8 */
	struct acpi_interface_info * next;               /*     8     8 */
	u8                         flags;                /*    16     1 */
	u8                         value;                /*    17     1 */

	/* size: 24, cachelines: 1, members: 4 */
	/* padding: 6 */
	/* last cacheline: 24 bytes */
};
struct led_hw_trigger_type {
	int                        dummy;                /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct led_classdev {
	const char  *              name;                 /*     0     8 */
	enum led_brightness        brightness;           /*     8     4 */
	enum led_brightness        max_brightness;       /*    12     4 */
	int                        flags;                /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          work_flags;           /*    24     8 */
	void                       (*brightness_set)(struct led_classdev *, enum led_brightness); /*    32     8 */
	int                        (*brightness_set_blocking)(struct led_classdev *, enum led_brightness); /*    40     8 */
	enum led_brightness        (*brightness_get)(struct led_classdev *); /*    48     8 */
	int                        (*blink_set)(struct led_classdev *, long unsigned int *, long unsigned int *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*pattern_set)(struct led_classdev *, struct led_pattern *, u32, int); /*    64     8 */
	int                        (*pattern_clear)(struct led_classdev *); /*    72     8 */
	struct device *            dev;                  /*    80     8 */
	const struct attribute_group  * * groups;        /*    88     8 */
	struct list_head           node;                 /*    96    16 */
	const char  *              default_trigger;      /*   112     8 */
	long unsigned int          blink_delay_on;       /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	long unsigned int          blink_delay_off;      /*   128     8 */
	struct timer_list          blink_timer;          /*   136    72 */
	/* --- cacheline 3 boundary (192 bytes) was 16 bytes ago --- */
	int                        blink_brightness;     /*   208     4 */
	int                        new_blink_brightness; /*   212     4 */
	void                       (*flash_resume)(struct led_classdev *); /*   216     8 */
	struct work_struct         set_brightness_work;  /*   224    64 */
	/* --- cacheline 4 boundary (256 bytes) was 32 bytes ago --- */
	int                        delayed_set_value;    /*   288     4 */

	/* XXX 4 bytes hole, try to pack */

	struct rw_semaphore        trigger_lock;         /*   296    40 */
	/* --- cacheline 5 boundary (320 bytes) was 16 bytes ago --- */
	struct led_trigger *       trigger;              /*   336     8 */
	struct list_head           trig_list;            /*   344    16 */
	void *                     trigger_data;         /*   360     8 */
	bool                       activated;            /*   368     1 */

	/* XXX 7 bytes hole, try to pack */

	struct led_hw_trigger_type * trigger_type;       /*   376     8 */
	/* --- cacheline 6 boundary (384 bytes) --- */
	struct mutex               led_access;           /*   384    32 */

	/* size: 416, cachelines: 7, members: 30 */
	/* sum members: 401, holes: 3, sum holes: 15 */
	/* last cacheline: 32 bytes */
};
struct led_pattern {
	u32                        delta_t;              /*     0     4 */
	int                        brightness;           /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct led_trigger {
	const char  *              name;                 /*     0     8 */
	int                        (*activate)(struct led_classdev *); /*     8     8 */
	void                       (*deactivate)(struct led_classdev *); /*    16     8 */
	struct led_hw_trigger_type * trigger_type;       /*    24     8 */
	rwlock_t                   leddev_list_lock;     /*    32     8 */
	struct list_head           led_cdevs;            /*    40    16 */
	struct list_head           next_trig;            /*    56    16 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	const struct attribute_group  * * groups;        /*    72     8 */

	/* size: 80, cachelines: 2, members: 8 */
	/* last cacheline: 16 bytes */
};
union power_supply_propval {
	int                        intval;             /*     0     4 */
	const char  *              strval;             /*     0     8 */
};
struct power_supply_config {
	struct device_node *       of_node;              /*     0     8 */
	struct fwnode_handle *     fwnode;               /*     8     8 */
	void *                     drv_data;             /*    16     8 */
	const struct attribute_group  * * attr_grp;      /*    24     8 */
	char * *                   supplied_to;          /*    32     8 */
	size_t                     num_supplicants;      /*    40     8 */

	/* size: 48, cachelines: 1, members: 6 */
	/* last cacheline: 48 bytes */
};
struct power_supply_desc {
	const char  *              name;                 /*     0     8 */
	enum power_supply_type     type;                 /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	const enum power_supply_usb_type  * usb_types;   /*    16     8 */
	size_t                     num_usb_types;        /*    24     8 */
	const enum power_supply_property  * properties;  /*    32     8 */
	size_t                     num_properties;       /*    40     8 */
	int                        (*get_property)(struct power_supply *, enum power_supply_property, union power_supply_propval *); /*    48     8 */
	int                        (*set_property)(struct power_supply *, enum power_supply_property, const union power_supply_propval  *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*property_is_writeable)(struct power_supply *, enum power_supply_property); /*    64     8 */
	void                       (*external_power_changed)(struct power_supply *); /*    72     8 */
	void                       (*set_charged)(struct power_supply *); /*    80     8 */
	bool                       no_thermal;           /*    88     1 */

	/* XXX 3 bytes hole, try to pack */

	int                        use_for_apm;          /*    92     4 */

	/* size: 96, cachelines: 2, members: 13 */
	/* sum members: 89, holes: 2, sum holes: 7 */
	/* last cacheline: 32 bytes */
};
struct power_supply {
	const struct power_supply_desc  * desc;          /*     0     8 */
	char * *                   supplied_to;          /*     8     8 */
	size_t                     num_supplicants;      /*    16     8 */
	char * *                   supplied_from;        /*    24     8 */
	size_t                     num_supplies;         /*    32     8 */
	struct device_node *       of_node;              /*    40     8 */
	void *                     drv_data;             /*    48     8 */
	struct device              dev __attribute__((__aligned__(8))); /*    56  1000 */
	/* --- cacheline 16 boundary (1024 bytes) was 32 bytes ago --- */
	struct work_struct         changed_work;         /*  1056    64 */
	/* --- cacheline 17 boundary (1088 bytes) was 32 bytes ago --- */
	struct delayed_work        deferred_register_work; /*  1120   192 */
	/* --- cacheline 20 boundary (1280 bytes) was 32 bytes ago --- */
	spinlock_t                 changed_lock;         /*  1312     4 */
	bool                       changed;              /*  1316     1 */
	bool                       initialized;          /*  1317     1 */
	bool                       removing;             /*  1318     1 */

	/* XXX 1 byte hole, try to pack */

	atomic_t                   use_cnt;              /*  1320     4 */

	/* XXX 4 bytes hole, try to pack */

	struct thermal_zone_device * tzd;                /*  1328     8 */
	struct thermal_cooling_device * tcd;             /*  1336     8 */
	/* --- cacheline 21 boundary (1344 bytes) --- */
	struct led_trigger *       charging_full_trig;   /*  1344     8 */
	char *                     charging_full_trig_name; /*  1352     8 */
	struct led_trigger *       charging_trig;        /*  1360     8 */
	char *                     charging_trig_name;   /*  1368     8 */
	struct led_trigger *       full_trig;            /*  1376     8 */
	char *                     full_trig_name;       /*  1384     8 */
	struct led_trigger *       online_trig;          /*  1392     8 */
	char *                     online_trig_name;     /*  1400     8 */
	/* --- cacheline 22 boundary (1408 bytes) --- */
	struct led_trigger *       charging_blink_full_solid_trig; /*  1408     8 */
	char *                     charging_blink_full_solid_trig_name; /*  1416     8 */

	/* size: 1424, cachelines: 23, members: 27 */
	/* sum members: 1419, holes: 2, sum holes: 5 */
	/* forced alignments: 1 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct acpi_ac_bl {
	const char  *              hid;                  /*     0     8 */
	int                        hrv;                  /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct acpi_ac {
	struct power_supply *      charger;              /*     0     8 */
	struct power_supply_desc   charger_desc;         /*     8    96 */
	/* --- cacheline 1 boundary (64 bytes) was 40 bytes ago --- */
	struct acpi_device *       device;               /*   104     8 */
	long long unsigned int     state;                /*   112     8 */
	struct notifier_block      battery_nb;           /*   120    24 */

	/* XXX last struct has 4 bytes of padding */

	/* size: 144, cachelines: 3, members: 5 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 16 bytes */
};
struct input_id {
	__u16                      bustype;              /*     0     2 */
	__u16                      vendor;               /*     2     2 */
	__u16                      product;              /*     4     2 */
	__u16                      version;              /*     6     2 */

	/* size: 8, cachelines: 1, members: 4 */
	/* last cacheline: 8 bytes */
};
struct input_absinfo {
	__s32                      value;                /*     0     4 */
	__s32                      minimum;              /*     4     4 */
	__s32                      maximum;              /*     8     4 */
	__s32                      fuzz;                 /*    12     4 */
	__s32                      flat;                 /*    16     4 */
	__s32                      resolution;           /*    20     4 */

	/* size: 24, cachelines: 1, members: 6 */
	/* last cacheline: 24 bytes */
};
struct input_keymap_entry {
	__u8                       flags;                /*     0     1 */
	__u8                       len;                  /*     1     1 */
	__u16                      index;                /*     2     2 */
	__u32                      keycode;              /*     4     4 */
	__u8                       scancode[32];         /*     8    32 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct ff_replay {
	__u16                      length;               /*     0     2 */
	__u16                      delay;                /*     2     2 */

	/* size: 4, cachelines: 1, members: 2 */
	/* last cacheline: 4 bytes */
};
struct ff_trigger {
	__u16                      button;               /*     0     2 */
	__u16                      interval;             /*     2     2 */

	/* size: 4, cachelines: 1, members: 2 */
	/* last cacheline: 4 bytes */
};
struct ff_envelope {
	__u16                      attack_length;        /*     0     2 */
	__u16                      attack_level;         /*     2     2 */
	__u16                      fade_length;          /*     4     2 */
	__u16                      fade_level;           /*     6     2 */

	/* size: 8, cachelines: 1, members: 4 */
	/* last cacheline: 8 bytes */
};
struct ff_constant_effect {
	__s16                      level;                /*     0     2 */
	struct ff_envelope         envelope;             /*     2     8 */

	/* size: 10, cachelines: 1, members: 2 */
	/* last cacheline: 10 bytes */
};
struct ff_ramp_effect {
	__s16                      start_level;          /*     0     2 */
	__s16                      end_level;            /*     2     2 */
	struct ff_envelope         envelope;             /*     4     8 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct ff_condition_effect {
	__u16                      right_saturation;     /*     0     2 */
	__u16                      left_saturation;      /*     2     2 */
	__s16                      right_coeff;          /*     4     2 */
	__s16                      left_coeff;           /*     6     2 */
	__u16                      deadband;             /*     8     2 */
	__s16                      center;               /*    10     2 */

	/* size: 12, cachelines: 1, members: 6 */
	/* last cacheline: 12 bytes */
};
struct ff_periodic_effect {
	__u16                      waveform;             /*     0     2 */
	__u16                      period;               /*     2     2 */
	__s16                      magnitude;            /*     4     2 */
	__s16                      offset;               /*     6     2 */
	__u16                      phase;                /*     8     2 */
	struct ff_envelope         envelope;             /*    10     8 */

	/* XXX 2 bytes hole, try to pack */

	__u32                      custom_len;           /*    20     4 */
	__s16 *                    custom_data;          /*    24     8 */

	/* size: 32, cachelines: 1, members: 8 */
	/* sum members: 30, holes: 1, sum holes: 2 */
	/* last cacheline: 32 bytes */
};
struct ff_rumble_effect {
	__u16                      strong_magnitude;     /*     0     2 */
	__u16                      weak_magnitude;       /*     2     2 */

	/* size: 4, cachelines: 1, members: 2 */
	/* last cacheline: 4 bytes */
};
struct ff_effect {
	__u16                      type;                 /*     0     2 */
	__s16                      id;                   /*     2     2 */
	__u16                      direction;            /*     4     2 */
	struct ff_trigger          trigger;              /*     6     4 */
	struct ff_replay           replay;               /*    10     4 */

	/* XXX 2 bytes hole, try to pack */

	union {
		struct ff_constant_effect constant;      /*    16    10 */
		struct ff_ramp_effect ramp;              /*    16    12 */
		struct ff_periodic_effect periodic;      /*    16    32 */
		struct ff_condition_effect condition[2]; /*    16    24 */
		struct ff_rumble_effect rumble;          /*    16     4 */
	} u;                                             /*    16    32 */

	/* size: 48, cachelines: 1, members: 6 */
	/* sum members: 46, holes: 1, sum holes: 2 */
	/* last cacheline: 48 bytes */
};
struct input_device_id {
	kernel_ulong_t             flags;                /*     0     8 */
	__u16                      bustype;              /*     8     2 */
	__u16                      vendor;               /*    10     2 */
	__u16                      product;              /*    12     2 */
	__u16                      version;              /*    14     2 */
	kernel_ulong_t             evbit[1];             /*    16     8 */
	kernel_ulong_t             keybit[12];           /*    24    96 */
	/* --- cacheline 1 boundary (64 bytes) was 56 bytes ago --- */
	kernel_ulong_t             relbit[1];            /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	kernel_ulong_t             absbit[1];            /*   128     8 */
	kernel_ulong_t             mscbit[1];            /*   136     8 */
	kernel_ulong_t             ledbit[1];            /*   144     8 */
	kernel_ulong_t             sndbit[1];            /*   152     8 */
	kernel_ulong_t             ffbit[2];             /*   160    16 */
	kernel_ulong_t             swbit[1];             /*   176     8 */
	kernel_ulong_t             propbit[1];           /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	kernel_ulong_t             driver_info;          /*   192     8 */

	/* size: 200, cachelines: 4, members: 16 */
	/* last cacheline: 8 bytes */
};
struct input_value {
	__u16                      type;                 /*     0     2 */
	__u16                      code;                 /*     2     2 */
	__s32                      value;                /*     4     4 */

	/* size: 8, cachelines: 1, members: 3 */
	/* last cacheline: 8 bytes */
};
struct input_dev {
	const char  *              name;                 /*     0     8 */
	const char  *              phys;                 /*     8     8 */
	const char  *              uniq;                 /*    16     8 */
	struct input_id            id;                   /*    24     8 */
	long unsigned int          propbit[1];           /*    32     8 */
	long unsigned int          evbit[1];             /*    40     8 */
	long unsigned int          keybit[12];           /*    48    96 */
	/* --- cacheline 2 boundary (128 bytes) was 16 bytes ago --- */
	long unsigned int          relbit[1];            /*   144     8 */
	long unsigned int          absbit[1];            /*   152     8 */
	long unsigned int          mscbit[1];            /*   160     8 */
	long unsigned int          ledbit[1];            /*   168     8 */
	long unsigned int          sndbit[1];            /*   176     8 */
	long unsigned int          ffbit[2];             /*   184    16 */
	/* --- cacheline 3 boundary (192 bytes) was 8 bytes ago --- */
	long unsigned int          swbit[1];             /*   200     8 */
	unsigned int               hint_events_per_packet; /*   208     4 */
	unsigned int               keycodemax;           /*   212     4 */
	unsigned int               keycodesize;          /*   216     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     keycode;              /*   224     8 */
	int                        (*setkeycode)(struct input_dev *, const struct input_keymap_entry  *, unsigned int *); /*   232     8 */
	int                        (*getkeycode)(struct input_dev *, struct input_keymap_entry *); /*   240     8 */
	struct ff_device *         ff;                   /*   248     8 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	struct input_dev_poller *  poller;               /*   256     8 */
	unsigned int               repeat_key;           /*   264     4 */

	/* XXX 4 bytes hole, try to pack */

	struct timer_list          timer;                /*   272    72 */
	/* --- cacheline 5 boundary (320 bytes) was 24 bytes ago --- */
	int                        rep[2];               /*   344     8 */
	struct input_mt *          mt;                   /*   352     8 */
	struct input_absinfo *     absinfo;              /*   360     8 */
	long unsigned int          key[12];              /*   368    96 */
	/* --- cacheline 7 boundary (448 bytes) was 16 bytes ago --- */
	long unsigned int          led[1];               /*   464     8 */
	long unsigned int          snd[1];               /*   472     8 */
	long unsigned int          sw[1];                /*   480     8 */
	int                        (*open)(struct input_dev *); /*   488     8 */
	void                       (*close)(struct input_dev *); /*   496     8 */
	int                        (*flush)(struct input_dev *, struct file *); /*   504     8 */
	/* --- cacheline 8 boundary (512 bytes) --- */
	int                        (*event)(struct input_dev *, unsigned int, unsigned int, int); /*   512     8 */
	struct input_handle *      grab;                 /*   520     8 */
	spinlock_t                 event_lock;           /*   528     4 */

	/* XXX 4 bytes hole, try to pack */

	struct mutex               mutex;                /*   536    32 */
	unsigned int               users;                /*   568     4 */
	bool                       going_away;           /*   572     1 */

	/* XXX 3 bytes hole, try to pack */

	/* --- cacheline 9 boundary (576 bytes) --- */
	struct device              dev __attribute__((__aligned__(8))); /*   576  1000 */
	/* --- cacheline 24 boundary (1536 bytes) was 40 bytes ago --- */
	struct list_head           h_list;               /*  1576    16 */
	struct list_head           node;                 /*  1592    16 */
	/* --- cacheline 25 boundary (1600 bytes) was 8 bytes ago --- */
	unsigned int               num_vals;             /*  1608     4 */
	unsigned int               max_vals;             /*  1612     4 */
	struct input_value *       vals;                 /*  1616     8 */
	bool                       devres_managed;       /*  1624     1 */

	/* XXX 7 bytes hole, try to pack */

	ktime_t                    timestamp[3];         /*  1632    24 */
	u64                        kabi_reserved1;       /*  1656     8 */
	/* --- cacheline 26 boundary (1664 bytes) --- */
	u64                        kabi_reserved2;       /*  1664     8 */

	/* size: 1672, cachelines: 27, members: 50 */
	/* sum members: 1650, holes: 5, sum holes: 22 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 3 */
	/* last cacheline: 8 bytes */
} __attribute__((__aligned__(8)));
struct ff_device {
	int                        (*upload)(struct input_dev *, struct ff_effect *, struct ff_effect *); /*     0     8 */
	int                        (*erase)(struct input_dev *, int); /*     8     8 */
	int                        (*playback)(struct input_dev *, int, int); /*    16     8 */
	void                       (*set_gain)(struct input_dev *, u16); /*    24     8 */
	void                       (*set_autocenter)(struct input_dev *, u16); /*    32     8 */
	void                       (*destroy)(struct ff_device *); /*    40     8 */
	void *                     private;              /*    48     8 */
	long unsigned int          ffbit[2];             /*    56    16 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	struct mutex               mutex;                /*    72    32 */
	int                        max_effects;          /*   104     4 */

	/* XXX 4 bytes hole, try to pack */

	struct ff_effect *         effects;              /*   112     8 */
	struct file *              effect_owners[];      /*   120     0 */

	/* size: 120, cachelines: 2, members: 12 */
	/* sum members: 116, holes: 1, sum holes: 4 */
	/* last cacheline: 56 bytes */
};
struct input_handle {
	void *                     private;              /*     0     8 */
	int                        open;                 /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	const char  *              name;                 /*    16     8 */
	struct input_dev *         dev;                  /*    24     8 */
	struct input_handler *     handler;              /*    32     8 */
	struct list_head           d_node;               /*    40    16 */
	struct list_head           h_node;               /*    56    16 */

	/* size: 72, cachelines: 2, members: 7 */
	/* sum members: 68, holes: 1, sum holes: 4 */
	/* last cacheline: 8 bytes */
};
struct input_handler {
	void *                     private;              /*     0     8 */
	void                       (*event)(struct input_handle *, unsigned int, unsigned int, int); /*     8     8 */
	void                       (*events)(struct input_handle *, const struct input_value  *, unsigned int); /*    16     8 */
	bool                       (*filter)(struct input_handle *, unsigned int, unsigned int, int); /*    24     8 */
	bool                       (*match)(struct input_handler *, struct input_dev *); /*    32     8 */
	int                        (*connect)(struct input_handler *, struct input_dev *, const struct input_device_id  *); /*    40     8 */
	void                       (*disconnect)(struct input_handle *); /*    48     8 */
	void                       (*start)(struct input_handle *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	bool                       legacy_minors;        /*    64     1 */

	/* XXX 3 bytes hole, try to pack */

	int                        minor;                /*    68     4 */
	const char  *              name;                 /*    72     8 */
	const struct input_device_id  * id_table;        /*    80     8 */
	struct list_head           h_list;               /*    88    16 */
	struct list_head           node;                 /*   104    16 */

	/* size: 120, cachelines: 2, members: 14 */
	/* sum members: 117, holes: 1, sum holes: 3 */
	/* last cacheline: 56 bytes */
};
struct acpi_button {
	unsigned int               type;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct input_dev *         input;                /*     8     8 */
	char                       phys[32];             /*    16    32 */
	long unsigned int          pushed;               /*    48     8 */
	int                        last_state;           /*    56     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	ktime_t                    last_time;            /*    64     8 */
	bool                       suspended;            /*    72     1 */
	bool                       lid_state_initialized; /*    73     1 */

	/* size: 80, cachelines: 2, members: 8 */
	/* sum members: 66, holes: 2, sum holes: 8 */
	/* padding: 6 */
	/* last cacheline: 16 bytes */
};
struct acpi_fan_fps {
	u64                        control;              /*     0     8 */
	u64                        trip_point;           /*     8     8 */
	u64                        speed;                /*    16     8 */
	u64                        noise_level;          /*    24     8 */
	u64                        power;                /*    32     8 */
	char                       name[20];             /*    40    20 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	struct device_attribute    dev_attr;             /*    64    32 */

	/* size: 96, cachelines: 2, members: 7 */
	/* sum members: 92, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct acpi_fan_fif {
	u64                        revision;             /*     0     8 */
	u64                        fine_grain_ctrl;      /*     8     8 */
	u64                        step_size;            /*    16     8 */
	u64                        low_speed_notification; /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct acpi_fan {
	bool                       acpi4;                /*     0     1 */

	/* XXX 7 bytes hole, try to pack */

	struct acpi_fan_fif        fif;                  /*     8    32 */
	struct acpi_fan_fps *      fps;                  /*    40     8 */
	int                        fps_count;            /*    48     4 */

	/* XXX 4 bytes hole, try to pack */

	struct thermal_cooling_device * cdev;            /*    56     8 */

	/* size: 64, cachelines: 1, members: 5 */
	/* sum members: 53, holes: 2, sum holes: 11 */
};
struct acpi_pci_slot {
	struct pci_slot *          pci_slot;             /*     0     8 */
	struct list_head           list;                 /*     8    16 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct acpi_lpi_states_array {
	unsigned int               size;                 /*     0     4 */
	unsigned int               composite_states_size; /*     4     4 */
	struct acpi_lpi_state *    entries;              /*     8     8 */
	struct acpi_lpi_state *    composite_states[8];  /*    16    64 */

	/* size: 80, cachelines: 2, members: 4 */
	/* last cacheline: 16 bytes */
};
struct throttling_tstate {
	unsigned int               cpu;                  /*     0     4 */
	int                        target_state;         /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct acpi_processor_throttling_arg {
	struct acpi_processor *    pr;                   /*     0     8 */
	int                        target_state;         /*     8     4 */
	bool                       force;                /*    12     1 */

	/* size: 16, cachelines: 1, members: 3 */
	/* padding: 3 */
	/* last cacheline: 16 bytes */
};
struct container_dev {
	struct device              dev __attribute__((__aligned__(8))); /*     0  1000 */
	/* --- cacheline 15 boundary (960 bytes) was 40 bytes ago --- */
	int                        (*offline)(struct container_dev *); /*  1000     8 */

	/* size: 1008, cachelines: 16, members: 2 */
	/* forced alignments: 1 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct thermal_zone_device_ops {
	int                        (*bind)(struct thermal_zone_device *, struct thermal_cooling_device *); /*     0     8 */
	int                        (*unbind)(struct thermal_zone_device *, struct thermal_cooling_device *); /*     8     8 */
	int                        (*get_temp)(struct thermal_zone_device *, int *); /*    16     8 */
	int                        (*set_trips)(struct thermal_zone_device *, int, int); /*    24     8 */
	int                        (*change_mode)(struct thermal_zone_device *, enum thermal_device_mode); /*    32     8 */
	int                        (*get_trip_type)(struct thermal_zone_device *, int, enum thermal_trip_type *); /*    40     8 */
	int                        (*get_trip_temp)(struct thermal_zone_device *, int, int *); /*    48     8 */
	int                        (*set_trip_temp)(struct thermal_zone_device *, int, int); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*get_trip_hyst)(struct thermal_zone_device *, int, int *); /*    64     8 */
	int                        (*set_trip_hyst)(struct thermal_zone_device *, int, int); /*    72     8 */
	int                        (*get_crit_temp)(struct thermal_zone_device *, int *); /*    80     8 */
	int                        (*set_emul_temp)(struct thermal_zone_device *, int); /*    88     8 */
	int                        (*get_trend)(struct thermal_zone_device *, int, enum thermal_trend *); /*    96     8 */
	int                        (*notify)(struct thermal_zone_device *, int, enum thermal_trip_type); /*   104     8 */

	/* size: 112, cachelines: 2, members: 14 */
	/* last cacheline: 48 bytes */
};
struct thermal_zone_device {
	int                        id;                   /*     0     4 */
	char                       type[20];             /*     4    20 */
	struct device              device __attribute__((__aligned__(8))); /*    24  1000 */
	/* --- cacheline 16 boundary (1024 bytes) --- */
	struct attribute_group     trips_attribute_group; /*  1024    40 */
	struct thermal_attr *      trip_temp_attrs;      /*  1064     8 */
	struct thermal_attr *      trip_type_attrs;      /*  1072     8 */
	struct thermal_attr *      trip_hyst_attrs;      /*  1080     8 */
	/* --- cacheline 17 boundary (1088 bytes) --- */
	enum thermal_device_mode   mode;                 /*  1088     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     devdata;              /*  1096     8 */
	int                        trips;                /*  1104     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          trips_disabled;       /*  1112     8 */
	int                        passive_delay;        /*  1120     4 */
	int                        polling_delay;        /*  1124     4 */
	int                        temperature;          /*  1128     4 */
	int                        last_temperature;     /*  1132     4 */
	int                        emul_temperature;     /*  1136     4 */
	int                        passive;              /*  1140     4 */
	int                        prev_low_trip;        /*  1144     4 */
	int                        prev_high_trip;       /*  1148     4 */
	/* --- cacheline 18 boundary (1152 bytes) --- */
	unsigned int               forced_passive;       /*  1152     4 */
	atomic_t                   need_update;          /*  1156     4 */
	struct thermal_zone_device_ops * ops;            /*  1160     8 */
	struct thermal_zone_params * tzp;                /*  1168     8 */
	struct thermal_governor *  governor;             /*  1176     8 */
	void *                     governor_data;        /*  1184     8 */
	struct list_head           thermal_instances;    /*  1192    16 */
	struct ida                 ida;                  /*  1208    16 */
	/* --- cacheline 19 boundary (1216 bytes) was 8 bytes ago --- */
	struct mutex               lock;                 /*  1224    32 */
	struct list_head           node;                 /*  1256    16 */
	struct delayed_work        poll_queue;           /*  1272   192 */
	/* --- cacheline 22 boundary (1408 bytes) was 56 bytes ago --- */
	enum thermal_notify_event  notify_event;         /*  1464     4 */

	/* size: 1472, cachelines: 23, members: 31 */
	/* sum members: 1460, holes: 2, sum holes: 8 */
	/* padding: 4 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(8)));
struct thermal_zone_params {
	char                       governor_name[20];    /*     0    20 */
	bool                       no_hwmon;             /*    20     1 */

	/* XXX 3 bytes hole, try to pack */

	int                        num_tbps;             /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	struct thermal_bind_params * tbp;                /*    32     8 */
	u32                        sustainable_power;    /*    40     4 */
	s32                        k_po;                 /*    44     4 */
	s32                        k_pu;                 /*    48     4 */
	s32                        k_i;                  /*    52     4 */
	s32                        k_d;                  /*    56     4 */
	s32                        integral_cutoff;      /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        slope;                /*    64     4 */
	int                        offset;               /*    68     4 */

	/* size: 72, cachelines: 2, members: 12 */
	/* sum members: 65, holes: 2, sum holes: 7 */
	/* last cacheline: 8 bytes */
};
struct thermal_governor {
	char                       name[20];             /*     0    20 */

	/* XXX 4 bytes hole, try to pack */

	int                        (*bind_to_tz)(struct thermal_zone_device *); /*    24     8 */
	void                       (*unbind_from_tz)(struct thermal_zone_device *); /*    32     8 */
	int                        (*throttle)(struct thermal_zone_device *, int); /*    40     8 */
	struct list_head           governor_list;        /*    48    16 */

	/* size: 64, cachelines: 1, members: 5 */
	/* sum members: 60, holes: 1, sum holes: 4 */
};
struct thermal_bind_params {
	struct thermal_cooling_device * cdev;            /*     0     8 */
	int                        weight;               /*     8     4 */
	int                        trip_mask;            /*    12     4 */
	long unsigned int *        binding_limits;       /*    16     8 */
	int                        (*match)(struct thermal_zone_device *, struct thermal_cooling_device *); /*    24     8 */

	/* size: 32, cachelines: 1, members: 5 */
	/* last cacheline: 32 bytes */
};
struct acpi_thermal_state {
	u8                         critical:1;           /*     0: 0  1 */
	u8                         hot:1;                /*     0: 1  1 */
	u8                         passive:1;            /*     0: 2  1 */
	u8                         active:1;             /*     0: 3  1 */
	u8                         reserved:4;           /*     0: 4  1 */

	/* XXX 3 bytes hole, try to pack */

	int                        active_index;         /*     4     4 */

	/* size: 8, cachelines: 1, members: 6 */
	/* sum members: 4, holes: 1, sum holes: 3 */
	/* sum bitfield members: 8 bits (1 bytes) */
	/* last cacheline: 8 bytes */
};
struct acpi_thermal_state_flags {
	u8                         valid:1;              /*     0: 0  1 */
	u8                         enabled:1;            /*     0: 1  1 */
	u8                         reserved:6;           /*     0: 2  1 */

	/* size: 1, cachelines: 1, members: 3 */
	/* last cacheline: 1 bytes */
};
struct acpi_thermal_critical {
	struct acpi_thermal_state_flags flags;           /*     0     1 */

	/* XXX 7 bytes hole, try to pack */

	long unsigned int          temperature;          /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 9, holes: 1, sum holes: 7 */
	/* last cacheline: 16 bytes */
};
struct acpi_thermal_hot {
	struct acpi_thermal_state_flags flags;           /*     0     1 */

	/* XXX 7 bytes hole, try to pack */

	long unsigned int          temperature;          /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 9, holes: 1, sum holes: 7 */
	/* last cacheline: 16 bytes */
};
struct acpi_thermal_passive {
	struct acpi_thermal_state_flags flags;           /*     0     1 */

	/* XXX 7 bytes hole, try to pack */

	long unsigned int          temperature;          /*     8     8 */
	long unsigned int          tc1;                  /*    16     8 */
	long unsigned int          tc2;                  /*    24     8 */
	long unsigned int          tsp;                  /*    32     8 */
	struct acpi_handle_list    devices;              /*    40    88 */

	/* size: 128, cachelines: 2, members: 6 */
	/* sum members: 121, holes: 1, sum holes: 7 */
};
struct acpi_thermal_active {
	struct acpi_thermal_state_flags flags;           /*     0     1 */

	/* XXX 7 bytes hole, try to pack */

	long unsigned int          temperature;          /*     8     8 */
	struct acpi_handle_list    devices;              /*    16    88 */

	/* size: 104, cachelines: 2, members: 3 */
	/* sum members: 97, holes: 1, sum holes: 7 */
	/* last cacheline: 40 bytes */
};
struct acpi_thermal_trips {
	struct acpi_thermal_critical critical;           /*     0    16 */
	struct acpi_thermal_hot    hot;                  /*    16    16 */
	struct acpi_thermal_passive passive;             /*    32   128 */
	/* --- cacheline 2 boundary (128 bytes) was 32 bytes ago --- */
	struct acpi_thermal_active active[10];           /*   160  1040 */

	/* size: 1200, cachelines: 19, members: 4 */
	/* last cacheline: 48 bytes */
};
struct acpi_thermal_flags {
	u8                         cooling_mode:1;       /*     0: 0  1 */
	u8                         devices:1;            /*     0: 1  1 */
	u8                         reserved:6;           /*     0: 2  1 */

	/* size: 1, cachelines: 1, members: 3 */
	/* last cacheline: 1 bytes */
};
struct acpi_thermal {
	struct acpi_device *       device;               /*     0     8 */
	acpi_bus_id                name;                 /*     8     8 */
	long unsigned int          temperature;          /*    16     8 */
	long unsigned int          last_temperature;     /*    24     8 */
	long unsigned int          polling_frequency;    /*    32     8 */
	volatile u8                zombie;               /*    40     1 */
	struct acpi_thermal_flags  flags;                /*    41     1 */

	/* XXX 2 bytes hole, try to pack */

	struct acpi_thermal_state  state;                /*    44     8 */

	/* XXX 4 bytes hole, try to pack */

	struct acpi_thermal_trips  trips;                /*    56  1200 */
	/* --- cacheline 19 boundary (1216 bytes) was 40 bytes ago --- */
	struct acpi_handle_list    devices;              /*  1256    88 */
	/* --- cacheline 21 boundary (1344 bytes) --- */
	struct thermal_zone_device * thermal_zone;       /*  1344     8 */
	int                        kelvin_offset;        /*  1352     4 */

	/* XXX 4 bytes hole, try to pack */

	struct work_struct         thermal_check_work;   /*  1360    64 */
	/* --- cacheline 22 boundary (1408 bytes) was 16 bytes ago --- */
	struct mutex               thermal_check_lock;   /*  1424    32 */
	refcount_t                 thermal_check_count;  /*  1456     4 */

	/* size: 1464, cachelines: 23, members: 15 */
	/* sum members: 1450, holes: 3, sum holes: 10 */
	/* padding: 4 */
	/* last cacheline: 56 bytes */
};
struct acpi_table_slit {
	struct acpi_table_header   header;               /*     0    36 */
	u64                        locality_count;       /*    36     8 */
	u8                         entry[1];             /*    44     1 */

	/* size: 45, cachelines: 1, members: 3 */
	/* last cacheline: 45 bytes */
} __attribute__((__packed__));
struct acpi_table_srat {
	struct acpi_table_header   header;               /*     0    36 */
	u32                        table_revision;       /*    36     4 */
	u64                        reserved;             /*    40     8 */

	/* size: 48, cachelines: 1, members: 3 */
	/* last cacheline: 48 bytes */
};
struct acpi_srat_mem_affinity {
	struct acpi_subtable_header header;              /*     0     2 */
	u32                        proximity_domain;     /*     2     4 */
	u16                        reserved;             /*     6     2 */
	u64                        base_address;         /*     8     8 */
	u64                        length;               /*    16     8 */
	u32                        reserved1;            /*    24     4 */
	u32                        flags;                /*    28     4 */
	u64                        reserved2;            /*    32     8 */

	/* size: 40, cachelines: 1, members: 8 */
	/* last cacheline: 40 bytes */
} __attribute__((__packed__));
struct acpi_srat_gicc_affinity {
	struct acpi_subtable_header header;              /*     0     2 */
	u32                        proximity_domain;     /*     2     4 */
	u32                        acpi_processor_uid;   /*     6     4 */
	u32                        flags;                /*    10     4 */
	u32                        clock_domain;         /*    14     4 */

	/* size: 18, cachelines: 1, members: 5 */
	/* last cacheline: 18 bytes */
} __attribute__((__packed__));
struct acpi_srat_generic_affinity {
	struct acpi_subtable_header header;              /*     0     2 */
	u8                         reserved;             /*     2     1 */
	u8                         device_handle_type;   /*     3     1 */
	u32                        proximity_domain;     /*     4     4 */
	u8                         device_handle[16];    /*     8    16 */
	u32                        flags;                /*    24     4 */
	u32                        reserved1;            /*    28     4 */

	/* size: 32, cachelines: 1, members: 7 */
	/* last cacheline: 32 bytes */
};
struct acpi_hmat_proximity_domain {
	struct acpi_hmat_structure header;               /*     0     8 */
	u16                        flags;                /*     8     2 */
	u16                        reserved1;            /*    10     2 */
	u32                        processor_PD;         /*    12     4 */
	u32                        memory_PD;            /*    16     4 */
	u32                        reserved2;            /*    20     4 */
	u64                        reserved3;            /*    24     8 */
	u64                        reserved4;            /*    32     8 */

	/* size: 40, cachelines: 1, members: 8 */
	/* last cacheline: 40 bytes */
};
struct acpi_hmat_locality {
	struct acpi_hmat_structure header;               /*     0     8 */
	u8                         flags;                /*     8     1 */
	u8                         data_type;            /*     9     1 */
	u16                        reserved1;            /*    10     2 */
	u32                        number_of_initiator_Pds; /*    12     4 */
	u32                        number_of_target_Pds; /*    16     4 */
	u32                        reserved2;            /*    20     4 */
	u64                        entry_base_unit;      /*    24     8 */

	/* size: 32, cachelines: 1, members: 8 */
	/* last cacheline: 32 bytes */
};
struct acpi_hmat_cache {
	struct acpi_hmat_structure header;               /*     0     8 */
	u32                        memory_PD;            /*     8     4 */
	u32                        reserved1;            /*    12     4 */
	u64                        cache_size;           /*    16     8 */
	u32                        cache_attributes;     /*    24     4 */
	u16                        reserved2;            /*    28     2 */
	u16                        number_of_SMBIOShandles; /*    30     2 */

	/* size: 32, cachelines: 1, members: 7 */
	/* last cacheline: 32 bytes */
};
struct node_hmem_attrs {
	unsigned int               read_bandwidth;       /*     0     4 */
	unsigned int               write_bandwidth;      /*     4     4 */
	unsigned int               read_latency;         /*     8     4 */
	unsigned int               write_latency;        /*    12     4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct node_cache_attrs {
	enum cache_indexing        indexing;             /*     0     4 */
	enum cache_write_policy    write_policy;         /*     4     4 */
	u64                        size;                 /*     8     8 */
	u16                        line_size;            /*    16     2 */
	u8                         level;                /*    18     1 */

	/* size: 24, cachelines: 1, members: 5 */
	/* padding: 5 */
	/* last cacheline: 24 bytes */
};
struct memory_locality {
	struct list_head           node;                 /*     0    16 */
	struct acpi_hmat_locality * hmat_loc;            /*    16     8 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct target_cache {
	struct list_head           node;                 /*     0    16 */
	struct node_cache_attrs    cache_attrs;          /*    16    24 */

	/* XXX last struct has 5 bytes of padding */

	/* size: 40, cachelines: 1, members: 2 */
	/* paddings: 1, sum paddings: 5 */
	/* last cacheline: 40 bytes */
};
struct memory_target {
	struct list_head           node;                 /*     0    16 */
	unsigned int               memory_pxm;           /*    16     4 */
	unsigned int               processor_pxm;        /*    20     4 */
	struct resource            memregions;           /*    24    96 */
	/* --- cacheline 1 boundary (64 bytes) was 56 bytes ago --- */
	struct node_hmem_attrs     hmem_attrs[2];        /*   120    32 */
	/* --- cacheline 2 boundary (128 bytes) was 24 bytes ago --- */
	struct list_head           caches;               /*   152    16 */
	struct node_cache_attrs    cache_attrs;          /*   168    24 */

	/* XXX last struct has 5 bytes of padding */

	/* --- cacheline 3 boundary (192 bytes) --- */
	bool                       registered;           /*   192     1 */

	/* size: 200, cachelines: 4, members: 8 */
	/* padding: 7 */
	/* paddings: 1, sum paddings: 5 */
	/* last cacheline: 8 bytes */
};
struct memory_initiator {
	struct list_head           node;                 /*     0    16 */
	unsigned int               processor_pxm;        /*    16     4 */
	bool                       has_cpu;              /*    20     1 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 3 */
	/* last cacheline: 24 bytes */
};
struct acpi_memory_info {
	struct list_head           list;                 /*     0    16 */
	u64                        start_addr;           /*    16     8 */
	u64                        length;               /*    24     8 */
	short unsigned int         caching;              /*    32     2 */
	short unsigned int         write_protect;        /*    34     2 */
	unsigned int               enabled:1;            /*    36: 0  4 */

	/* size: 40, cachelines: 1, members: 6 */
	/* bit_padding: 31 bits */
	/* last cacheline: 40 bytes */
};
struct acpi_memory_device {
	struct acpi_device *       device;               /*     0     8 */
	struct list_head           res_list;             /*     8    16 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct acpi_pci_ioapic {
	acpi_handle                root_handle;          /*     0     8 */
	acpi_handle                handle;               /*     8     8 */
	u32                        gsi_base;             /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	struct resource            res;                  /*    24    96 */
	/* --- cacheline 1 boundary (64 bytes) was 56 bytes ago --- */
	struct pci_dev *           pdev;                 /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct list_head           list;                 /*   128    16 */

	/* size: 144, cachelines: 3, members: 6 */
	/* sum members: 140, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct acpi_battery_hook {
	const char  *              name;                 /*     0     8 */
	int                        (*add_battery)(struct power_supply *); /*     8     8 */
	int                        (*remove_battery)(struct power_supply *); /*    16     8 */
	struct list_head           list;                 /*    24    16 */

	/* size: 40, cachelines: 1, members: 4 */
	/* last cacheline: 40 bytes */
};
struct acpi_battery {
	struct mutex               lock;                 /*     0    32 */
	struct mutex               sysfs_lock;           /*    32    32 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct power_supply *      bat;                  /*    64     8 */
	struct power_supply_desc   bat_desc;             /*    72    96 */
	/* --- cacheline 2 boundary (128 bytes) was 40 bytes ago --- */
	struct acpi_device *       device;               /*   168     8 */
	struct notifier_block      pm_nb;                /*   176    24 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 3 boundary (192 bytes) was 8 bytes ago --- */
	struct list_head           list;                 /*   200    16 */
	long unsigned int          update_time;          /*   216     8 */
	int                        revision;             /*   224     4 */
	int                        rate_now;             /*   228     4 */
	int                        capacity_now;         /*   232     4 */
	int                        voltage_now;          /*   236     4 */
	int                        design_capacity;      /*   240     4 */
	int                        full_charge_capacity; /*   244     4 */
	int                        technology;           /*   248     4 */
	int                        design_voltage;       /*   252     4 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	int                        design_capacity_warning; /*   256     4 */
	int                        design_capacity_low;  /*   260     4 */
	int                        cycle_count;          /*   264     4 */
	int                        measurement_accuracy; /*   268     4 */
	int                        max_sampling_time;    /*   272     4 */
	int                        min_sampling_time;    /*   276     4 */
	int                        max_averaging_interval; /*   280     4 */
	int                        min_averaging_interval; /*   284     4 */
	int                        capacity_granularity_1; /*   288     4 */
	int                        capacity_granularity_2; /*   292     4 */
	int                        alarm;                /*   296     4 */
	char                       model_number[32];     /*   300    32 */
	/* --- cacheline 5 boundary (320 bytes) was 12 bytes ago --- */
	char                       serial_number[32];    /*   332    32 */
	char                       type[32];             /*   364    32 */
	/* --- cacheline 6 boundary (384 bytes) was 12 bytes ago --- */
	char                       oem_info[32];         /*   396    32 */
	int                        state;                /*   428     4 */
	int                        power_unit;           /*   432     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          flags;                /*   440     8 */

	/* size: 448, cachelines: 7, members: 34 */
	/* sum members: 444, holes: 1, sum holes: 4 */
	/* paddings: 1, sum paddings: 4 */
};
struct acpi_offsets {
	size_t                     offset;               /*     0     8 */
	u8                         mode;                 /*     8     1 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 7 */
	/* last cacheline: 16 bytes */
};
struct acpi_table_bgrt {
	struct acpi_table_header   header;               /*     0    36 */
	u16                        version;              /*    36     2 */
	u8                         status;               /*    38     1 */
	u8                         image_type;           /*    39     1 */
	u64                        image_address;        /*    40     8 */
	u32                        image_offset_x;       /*    48     4 */
	u32                        image_offset_y;       /*    52     4 */

	/* size: 56, cachelines: 1, members: 7 */
	/* last cacheline: 56 bytes */
};
struct acpi_pcct_hw_reduced {
	struct acpi_subtable_header header;              /*     0     2 */
	u32                        platform_interrupt;   /*     2     4 */
	u8                         flags;                /*     6     1 */
	u8                         reserved;             /*     7     1 */
	u64                        base_address;         /*     8     8 */
	u64                        length;               /*    16     8 */
	struct acpi_generic_address doorbell_register;   /*    24    12 */
	u64                        preserve_mask;        /*    36     8 */
	u64                        write_mask;           /*    44     8 */
	u32                        latency;              /*    52     4 */
	u32                        max_access_rate;      /*    56     4 */
	u16                        min_turnaround_time;  /*    60     2 */

	/* size: 62, cachelines: 1, members: 12 */
	/* last cacheline: 62 bytes */
} __attribute__((__packed__));
struct acpi_pcct_shared_memory {
	u32                        signature;            /*     0     4 */
	u16                        command;              /*     4     2 */
	u16                        status;               /*     6     2 */

	/* size: 8, cachelines: 1, members: 3 */
	/* last cacheline: 8 bytes */
};
struct mbox_chan_ops {
	int                        (*send_data)(struct mbox_chan *, void *); /*     0     8 */
	int                        (*flush)(struct mbox_chan *, long unsigned int); /*     8     8 */
	int                        (*startup)(struct mbox_chan *); /*    16     8 */
	void                       (*shutdown)(struct mbox_chan *); /*    24     8 */
	bool                       (*last_tx_done)(struct mbox_chan *); /*    32     8 */
	bool                       (*peek_data)(struct mbox_chan *); /*    40     8 */

	/* size: 48, cachelines: 1, members: 6 */
	/* last cacheline: 48 bytes */
};
struct mbox_chan {
	struct mbox_controller *   mbox;                 /*     0     8 */
	unsigned int               txdone_method;        /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct mbox_client *       cl;                   /*    16     8 */
	struct completion          tx_complete;          /*    24    32 */
	void *                     active_req;           /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	unsigned int               msg_count;            /*    64     4 */
	unsigned int               msg_free;             /*    68     4 */
	void *                     msg_data[20];         /*    72   160 */
	/* --- cacheline 3 boundary (192 bytes) was 40 bytes ago --- */
	spinlock_t                 lock;                 /*   232     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     con_priv;             /*   240     8 */

	/* size: 248, cachelines: 4, members: 10 */
	/* sum members: 240, holes: 2, sum holes: 8 */
	/* last cacheline: 56 bytes */
};
struct mbox_controller {
	struct device *            dev;                  /*     0     8 */
	const struct mbox_chan_ops  * ops;               /*     8     8 */
	struct mbox_chan *         chans;                /*    16     8 */
	int                        num_chans;            /*    24     4 */
	bool                       txdone_irq;           /*    28     1 */
	bool                       txdone_poll;          /*    29     1 */

	/* XXX 2 bytes hole, try to pack */

	unsigned int               txpoll_period;        /*    32     4 */

	/* XXX 4 bytes hole, try to pack */

	struct mbox_chan *         (*of_xlate)(struct mbox_controller *, const struct of_phandle_args  *); /*    40     8 */
	struct hrtimer             poll_hrt __attribute__((__aligned__(8))); /*    48    96 */
	/* --- cacheline 2 boundary (128 bytes) was 16 bytes ago --- */
	spinlock_t                 poll_hrt_lock;        /*   144     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           node;                 /*   152    16 */

	/* size: 168, cachelines: 3, members: 11 */
	/* sum members: 158, holes: 3, sum holes: 10 */
	/* forced alignments: 1 */
	/* last cacheline: 40 bytes */
} __attribute__((__aligned__(8)));
struct mbox_client {
	struct device *            dev;                  /*     0     8 */
	bool                       tx_block;             /*     8     1 */

	/* XXX 7 bytes hole, try to pack */

	long unsigned int          tx_tout;              /*    16     8 */
	bool                       knows_txdone;         /*    24     1 */

	/* XXX 7 bytes hole, try to pack */

	void                       (*rx_callback)(struct mbox_client *, void *); /*    32     8 */
	void                       (*tx_prepare)(struct mbox_client *, void *); /*    40     8 */
	void                       (*tx_done)(struct mbox_client *, void *, int); /*    48     8 */

	/* size: 56, cachelines: 1, members: 7 */
	/* sum members: 42, holes: 2, sum holes: 14 */
	/* last cacheline: 56 bytes */
};
struct cpc_register_resource {
	acpi_object_type           type;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	u64 *                      sys_mem_vaddr;        /*     8     8 */
	union {
		struct cpc_reg     reg;                  /*    16    15 */
		u64                int_value;            /*    16     8 */
	} cpc_entry;                                     /*    16    16 */

	/* size: 32, cachelines: 1, members: 3 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct cpc_desc {
	int                        num_entries;          /*     0     4 */
	int                        version;              /*     4     4 */
	int                        cpu_id;               /*     8     4 */
	int                        write_cmd_status;     /*    12     4 */
	int                        write_cmd_id;         /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	struct cpc_register_resource cpc_regs[21];       /*    24   672 */
	/* --- cacheline 10 boundary (640 bytes) was 56 bytes ago --- */
	struct acpi_psd_package    domain_info;          /*   696    40 */
	/* --- cacheline 11 boundary (704 bytes) was 32 bytes ago --- */
	struct kobject             kobj;                 /*   736    96 */

	/* size: 832, cachelines: 13, members: 8 */
	/* sum members: 828, holes: 1, sum holes: 4 */
};
struct cppc_perf_caps {
	u32                        guaranteed_perf;      /*     0     4 */
	u32                        highest_perf;         /*     4     4 */
	u32                        nominal_perf;         /*     8     4 */
	u32                        lowest_perf;          /*    12     4 */
	u32                        lowest_nonlinear_perf; /*    16     4 */
	u32                        lowest_freq;          /*    20     4 */
	u32                        nominal_freq;         /*    24     4 */

	/* size: 28, cachelines: 1, members: 7 */
	/* last cacheline: 28 bytes */
};
struct cppc_perf_ctrls {
	u32                        max_perf;             /*     0     4 */
	u32                        min_perf;             /*     4     4 */
	u32                        desired_perf;         /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct cppc_perf_fb_ctrs {
	u64                        reference;            /*     0     8 */
	u64                        delivered;            /*     8     8 */
	u64                        reference_perf;       /*    16     8 */
	u64                        wraparound_time;      /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct cppc_cpudata {
	int                        cpu;                  /*     0     4 */
	struct cppc_perf_caps      perf_caps;            /*     4    28 */
	struct cppc_perf_ctrls     perf_ctrls;           /*    32    12 */

	/* XXX 4 bytes hole, try to pack */

	struct cppc_perf_fb_ctrs   perf_fb_ctrs;         /*    48    32 */
	/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
	struct cpufreq_policy *    cur_policy;           /*    80     8 */
	unsigned int               shared_type;          /*    88     4 */

	/* XXX 4 bytes hole, try to pack */

	cpumask_var_t              shared_cpu_map;       /*    96     8 */

	/* size: 104, cachelines: 2, members: 7 */
	/* sum members: 96, holes: 2, sum holes: 8 */
	/* last cacheline: 40 bytes */
};
struct cppc_pcc_data {
	struct mbox_chan *         pcc_channel;          /*     0     8 */
	void *                     pcc_comm_addr;        /*     8     8 */
	bool                       pcc_channel_acquired; /*    16     1 */

	/* XXX 3 bytes hole, try to pack */

	unsigned int               deadline_us;          /*    20     4 */
	unsigned int               pcc_mpar;             /*    24     4 */
	unsigned int               pcc_mrtt;             /*    28     4 */
	unsigned int               pcc_nominal;          /*    32     4 */
	bool                       pending_pcc_write_cmd; /*    36     1 */
	bool                       platform_owns_pcc;    /*    37     1 */

	/* XXX 2 bytes hole, try to pack */

	unsigned int               pcc_write_cnt;        /*    40     4 */

	/* XXX 4 bytes hole, try to pack */

	struct rw_semaphore        pcc_lock;             /*    48    40 */
	/* --- cacheline 1 boundary (64 bytes) was 24 bytes ago --- */
	wait_queue_head_t          pcc_write_wait_q;     /*    88    24 */
	ktime_t                    last_cmd_cmpl_time;   /*   112     8 */
	ktime_t                    last_mpar_reset;      /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	int                        mpar_count;           /*   128     4 */
	int                        refcount;             /*   132     4 */

	/* size: 136, cachelines: 3, members: 16 */
	/* sum members: 127, holes: 3, sum holes: 9 */
	/* last cacheline: 8 bytes */
};
struct acpi_whea_header {
	u8                         action;               /*     0     1 */
	u8                         instruction;          /*     1     1 */
	u8                         flags;                /*     2     1 */
	u8                         reserved;             /*     3     1 */
	struct acpi_generic_address register_region;     /*     4    12 */
	u64                        value;                /*    16     8 */
	u64                        mask;                 /*    24     8 */

	/* size: 32, cachelines: 1, members: 7 */
	/* last cacheline: 32 bytes */
} __attribute__((__packed__));
struct apei_exec_context {
	u32                        ip;                   /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        value;                /*     8     8 */
	u64                        var1;                 /*    16     8 */
	u64                        var2;                 /*    24     8 */
	u64                        src_base;             /*    32     8 */
	u64                        dst_base;             /*    40     8 */
	struct apei_exec_ins_type * ins_table;           /*    48     8 */
	u32                        instructions;         /*    56     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	struct acpi_whea_header *  action_table;         /*    64     8 */
	u32                        entries;              /*    72     4 */

	/* size: 80, cachelines: 2, members: 10 */
	/* sum members: 68, holes: 2, sum holes: 8 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct apei_exec_ins_type {
	u32                        flags;                /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	apei_exec_ins_func_t       run;                  /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct apei_resources {
	struct list_head           iomem;                /*     0    16 */
	struct list_head           ioport;               /*    16    16 */

	/* size: 32, cachelines: 1, members: 2 */
	/* last cacheline: 32 bytes */
};
struct apei_res {
	struct list_head           list;                 /*     0    16 */
	long unsigned int          start;                /*    16     8 */
	long unsigned int          end;                  /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* last cacheline: 32 bytes */
};
struct acpi_table_hest {
	struct acpi_table_header   header;               /*     0    36 */
	u32                        error_source_count;   /*    36     4 */

	/* size: 40, cachelines: 1, members: 2 */
	/* last cacheline: 40 bytes */
};
struct acpi_hest_ia_machine_check {
	struct acpi_hest_header    header;               /*     0     4 */
	u16                        reserved1;            /*     4     2 */
	u8                         flags;                /*     6     1 */
	u8                         enabled;              /*     7     1 */
	u32                        records_to_preallocate; /*     8     4 */
	u32                        max_sections_per_record; /*    12     4 */
	u64                        global_capability_data; /*    16     8 */
	u64                        global_control_data;  /*    24     8 */
	u8                         num_hardware_banks;   /*    32     1 */
	u8                         reserved3[7];         /*    33     7 */

	/* size: 40, cachelines: 1, members: 10 */
	/* last cacheline: 40 bytes */
};
struct acpi_hest_generic {
	struct acpi_hest_header    header;               /*     0     4 */
	u16                        related_source_id;    /*     4     2 */
	u8                         reserved;             /*     6     1 */
	u8                         enabled;              /*     7     1 */
	u32                        records_to_preallocate; /*     8     4 */
	u32                        max_sections_per_record; /*    12     4 */
	u32                        max_raw_data_length;  /*    16     4 */
	struct acpi_generic_address error_status_address; /*    20    12 */
	struct acpi_hest_notify    notify;               /*    32    28 */
	u32                        error_block_length;   /*    60     4 */

	/* size: 64, cachelines: 1, members: 10 */
} __attribute__((__packed__));
struct acpi_hest_ia_deferred_check {
	struct acpi_hest_header    header;               /*     0     4 */
	u16                        reserved1;            /*     4     2 */
	u8                         flags;                /*     6     1 */
	u8                         enabled;              /*     7     1 */
	u32                        records_to_preallocate; /*     8     4 */
	u32                        max_sections_per_record; /*    12     4 */
	struct acpi_hest_notify    notify;               /*    16    28 */
	u8                         num_hardware_banks;   /*    44     1 */
	u8                         reserved2[3];         /*    45     3 */

	/* size: 48, cachelines: 1, members: 9 */
	/* last cacheline: 48 bytes */
};
struct ghes_arr {
	struct platform_device * * ghes_devs;            /*     0     8 */
	unsigned int               count;                /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct acpi_table_erst {
	struct acpi_table_header   header;               /*     0    36 */
	u32                        header_length;        /*    36     4 */
	u32                        reserved;             /*    40     4 */
	u32                        entries;              /*    44     4 */

	/* size: 48, cachelines: 1, members: 4 */
	/* last cacheline: 48 bytes */
};
struct erst_erange {
	u64                        base;                 /*     0     8 */
	u64                        size;                 /*     8     8 */
	void *                     vaddr;                /*    16     8 */
	u32                        attr;                 /*    24     4 */

	/* size: 32, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct erst_record_id_cache {
	struct mutex               lock;                 /*     0    32 */
	u64 *                      entries;              /*    32     8 */
	int                        len;                  /*    40     4 */
	int                        size;                 /*    44     4 */
	int                        refcount;             /*    48     4 */

	/* size: 56, cachelines: 1, members: 5 */
	/* padding: 4 */
	/* last cacheline: 56 bytes */
};
struct cper_pstore_record {
	struct cper_record_header  hdr;                  /*     0   128 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct cper_section_descriptor sec_hdr;          /*   128    72 */
	/* --- cacheline 3 boundary (192 bytes) was 8 bytes ago --- */
	char                       data[];               /*   200     0 */

	/* size: 200, cachelines: 4, members: 3 */
	/* last cacheline: 8 bytes */
};
struct acpi_bert_region {
	u32                        block_status;         /*     0     4 */
	u32                        raw_data_offset;      /*     4     4 */
	u32                        raw_data_length;      /*     8     4 */
	u32                        data_length;          /*    12     4 */
	u32                        error_severity;       /*    16     4 */

	/* size: 20, cachelines: 1, members: 5 */
	/* last cacheline: 20 bytes */
};
struct acpi_hest_generic_status {
	u32                        block_status;         /*     0     4 */
	u32                        raw_data_offset;      /*     4     4 */
	u32                        raw_data_length;      /*     8     4 */
	u32                        data_length;          /*    12     4 */
	u32                        error_severity;       /*    16     4 */

	/* size: 20, cachelines: 1, members: 5 */
	/* last cacheline: 20 bytes */
};
struct acpi_hest_generic_v2 {
	struct acpi_hest_header    header;               /*     0     4 */
	u16                        related_source_id;    /*     4     2 */
	u8                         reserved;             /*     6     1 */
	u8                         enabled;              /*     7     1 */
	u32                        records_to_preallocate; /*     8     4 */
	u32                        max_sections_per_record; /*    12     4 */
	u32                        max_raw_data_length;  /*    16     4 */
	struct acpi_generic_address error_status_address; /*    20    12 */
	struct acpi_hest_notify    notify;               /*    32    28 */
	u32                        error_block_length;   /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct acpi_generic_address read_ack_register;   /*    64    12 */
	u64                        read_ack_preserve;    /*    76     8 */
	u64                        read_ack_write;       /*    84     8 */

	/* size: 92, cachelines: 2, members: 13 */
	/* last cacheline: 28 bytes */
} __attribute__((__packed__));
struct acpi_hest_generic_data {
	u8                         section_type[16];     /*     0    16 */
	u32                        error_severity;       /*    16     4 */
	u16                        revision;             /*    20     2 */
	u8                         validation_bits;      /*    22     1 */
	u8                         flags;                /*    23     1 */
	u32                        error_data_length;    /*    24     4 */
	u8                         fru_id[16];           /*    28    16 */
	u8                         fru_text[20];         /*    44    20 */

	/* size: 64, cachelines: 1, members: 8 */
};
struct acpi_hest_generic_data_v300 {
	u8                         section_type[16];     /*     0    16 */
	u32                        error_severity;       /*    16     4 */
	u16                        revision;             /*    20     2 */
	u8                         validation_bits;      /*    22     1 */
	u8                         flags;                /*    23     1 */
	u32                        error_data_length;    /*    24     4 */
	u8                         fru_id[16];           /*    28    16 */
	u8                         fru_text[20];         /*    44    20 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u64                        time_stamp;           /*    64     8 */

	/* size: 72, cachelines: 2, members: 9 */
	/* last cacheline: 8 bytes */
};
struct cper_sec_proc_arm {
	u32                        validation_bits;      /*     0     4 */
	u16                        err_info_num;         /*     4     2 */
	u16                        context_info_num;     /*     6     2 */
	u32                        section_length;       /*     8     4 */
	u8                         affinity_level;       /*    12     1 */
	u8                         reserved[3];          /*    13     3 */
	u64                        mpidr;                /*    16     8 */
	u64                        midr;                 /*    24     8 */
	u32                        running_state;        /*    32     4 */
	u32                        psci_state;           /*    36     4 */

	/* size: 40, cachelines: 1, members: 10 */
	/* last cacheline: 40 bytes */
};
struct cper_arm_err_info {
	u8                         version;              /*     0     1 */
	u8                         length;               /*     1     1 */
	u16                        validation_bits;      /*     2     2 */
	u8                         type;                 /*     4     1 */
	u16                        multiple_error;       /*     5     2 */
	u8                         flags;                /*     7     1 */
	u64                        error_info;           /*     8     8 */
	u64                        virt_fault_addr;      /*    16     8 */
	u64                        physical_fault_addr;  /*    24     8 */

	/* size: 32, cachelines: 1, members: 9 */
	/* last cacheline: 32 bytes */
} __attribute__((__packed__));
struct cper_sec_pcie {
	u64                        validation_bits;      /*     0     8 */
	u32                        port_type;            /*     8     4 */
	struct {
		u8                 minor;                /*    12     1 */
		u8                 major;                /*    13     1 */
		u8                 reserved[2];          /*    14     2 */
	} version;                                       /*    12     4 */
	u16                        command;              /*    16     2 */
	u16                        status;               /*    18     2 */
	u32                        reserved;             /*    20     4 */
	struct {
		u16                vendor_id;            /*    24     2 */
		u16                device_id;            /*    26     2 */
		u8                 class_code[3];        /*    28     3 */
		u8                 function;             /*    31     1 */
		u8                 device;               /*    32     1 */
		u16                segment;              /*    33     2 */
		u8                 bus;                  /*    35     1 */
		u8                 secondary_bus;        /*    36     1 */
		u16                slot;                 /*    37     2 */
		u8                 reserved;             /*    39     1 */
	} __attribute__((__packed__)) device_id;         /*    24    16 */
	struct {
		u32                lower;                /*    40     4 */
		u32                upper;                /*    44     4 */
	} serial_number;                                 /*    40     8 */
	struct {
		u16                secondary_status;     /*    48     2 */
		u16                control;              /*    50     2 */
	} bridge;                                        /*    48     4 */
	u8                         capability[60];       /*    52    60 */
	/* --- cacheline 1 boundary (64 bytes) was 48 bytes ago --- */
	u8                         aer_info[96];         /*   112    96 */

	/* size: 208, cachelines: 4, members: 11 */
	/* last cacheline: 16 bytes */
};
struct ghes {
	union {
		struct acpi_hest_generic * generic;      /*     0     8 */
		struct acpi_hest_generic_v2 * generic_v2; /*     0     8 */
	};                                               /*     0     8 */
	struct acpi_hest_generic_status * estatus;       /*     8     8 */
	long unsigned int          flags;                /*    16     8 */
	union {
		struct list_head   list;                 /*    24    16 */
		struct timer_list  timer;                /*    24    72 */
		unsigned int       irq;                  /*    24     4 */
	};                                               /*    24    72 */

	/* size: 96, cachelines: 2, members: 4 */
	/* last cacheline: 32 bytes */
};
struct ghes_estatus_node {
	struct llist_node          llnode;               /*     0     8 */
	struct acpi_hest_generic * generic;              /*     8     8 */
	struct ghes *              ghes;                 /*    16     8 */
	int                        task_work_cpu;        /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	struct callback_head       task_work __attribute__((__aligned__(8))); /*    32    16 */

	/* size: 48, cachelines: 1, members: 5 */
	/* sum members: 44, holes: 1, sum holes: 4 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 4 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct ghes_estatus_cache {
	u32                        estatus_len;          /*     0     4 */
	atomic_t                   count;                /*     4     4 */
	struct acpi_hest_generic * generic;              /*     8     8 */
	long long unsigned int     time_in;              /*    16     8 */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*    24    16 */

	/* size: 40, cachelines: 1, members: 5 */
	/* forced alignments: 1 */
	/* last cacheline: 40 bytes */
} __attribute__((__aligned__(8)));
struct ghes_vendor_record_entry {
	struct work_struct         work;                 /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        error_severity;       /*    64     4 */
	char                       vendor_record[];      /*    68     0 */

	/* size: 72, cachelines: 2, members: 3 */
	/* padding: 4 */
	/* last cacheline: 8 bytes */
};
struct pmic_table {
	int                        address;              /*     0     4 */
	int                        reg;                  /*     4     4 */
	int                        bit;                  /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct intel_pmic_opregion_data {
	int                        (*get_power)(struct regmap *, int, int, u64 *); /*     0     8 */
	int                        (*update_power)(struct regmap *, int, int, bool); /*     8     8 */
	int                        (*get_raw_temp)(struct regmap *, int); /*    16     8 */
	int                        (*update_aux)(struct regmap *, int, int); /*    24     8 */
	int                        (*get_policy)(struct regmap *, int, int, u64 *); /*    32     8 */
	int                        (*update_policy)(struct regmap *, int, int, int); /*    40     8 */
	int                        (*exec_mipi_pmic_seq_element)(struct regmap *, u16, u32, u32, u32); /*    48     8 */
	struct pmic_table *        power_table;          /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        power_table_count;    /*    64     4 */

	/* XXX 4 bytes hole, try to pack */

	struct pmic_table *        thermal_table;        /*    72     8 */
	int                        thermal_table_count;  /*    80     4 */
	int                        pmic_i2c_address;     /*    84     4 */

	/* size: 88, cachelines: 2, members: 12 */
	/* sum members: 84, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct intel_pmic_regs_handler_ctx {
	unsigned int               val;                  /*     0     4 */
	u16                        addr;                 /*     4     2 */

	/* size: 8, cachelines: 1, members: 2 */
	/* padding: 2 */
	/* last cacheline: 8 bytes */
};
struct intel_pmic_opregion {
	struct mutex               lock;                 /*     0    32 */
	struct acpi_lpat_conversion_table * lpat_table;  /*    32     8 */
	struct regmap *            regmap;               /*    40     8 */
	struct intel_pmic_opregion_data * data;          /*    48     8 */
	struct intel_pmic_regs_handler_ctx ctx;          /*    56     8 */

	/* XXX last struct has 2 bytes of padding */

	/* size: 64, cachelines: 1, members: 5 */
	/* paddings: 1, sum paddings: 2 */
};
struct acpi_table_xsdt {
	struct acpi_table_header   header;               /*     0    36 */
	u64                        table_offset_entry[1]; /*    36     8 */

	/* size: 44, cachelines: 1, members: 2 */
	/* last cacheline: 44 bytes */
} __attribute__((__packed__));
struct sfi_table_key {
	char *                     sig;                  /*     0     8 */
	char *                     oem_id;               /*     8     8 */
	char *                     oem_table_id;         /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct sfi_table_attr {
	struct bin_attribute       attr;                 /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	char                       name[8];              /*    64     8 */

	/* size: 72, cachelines: 2, members: 2 */
	/* last cacheline: 8 bytes */
};
struct pnp_resource {
	struct list_head           list;                 /*     0    16 */
	struct resource            res;                  /*    16    96 */

	/* size: 112, cachelines: 2, members: 2 */
	/* last cacheline: 48 bytes */
};
struct pnp_port {
	resource_size_t            min;                  /*     0     8 */
	resource_size_t            max;                  /*     8     8 */
	resource_size_t            align;                /*    16     8 */
	resource_size_t            size;                 /*    24     8 */
	unsigned char              flags;                /*    32     1 */

	/* size: 40, cachelines: 1, members: 5 */
	/* padding: 7 */
	/* last cacheline: 40 bytes */
};
struct pnp_irq {
	pnp_irq_mask_t             map;                  /*     0    32 */
	unsigned char              flags;                /*    32     1 */

	/* size: 40, cachelines: 1, members: 2 */
	/* padding: 7 */
	/* last cacheline: 40 bytes */
};
struct pnp_dma {
	unsigned char              map;                  /*     0     1 */
	unsigned char              flags;                /*     1     1 */

	/* size: 2, cachelines: 1, members: 2 */
	/* last cacheline: 2 bytes */
};
struct pnp_mem {
	resource_size_t            min;                  /*     0     8 */
	resource_size_t            max;                  /*     8     8 */
	resource_size_t            align;                /*    16     8 */
	resource_size_t            size;                 /*    24     8 */
	unsigned char              flags;                /*    32     1 */

	/* size: 40, cachelines: 1, members: 5 */
	/* padding: 7 */
	/* last cacheline: 40 bytes */
};
struct pnp_option {
	struct list_head           list;                 /*     0    16 */
	unsigned int               flags;                /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          type;                 /*    24     8 */
	union {
		struct pnp_port    port;                 /*    32    40 */
		struct pnp_irq     irq;                  /*    32    40 */
		struct pnp_dma     dma;                  /*    32     2 */
		struct pnp_mem     mem;                  /*    32    40 */
	} u;                                             /*    32    40 */

	/* size: 72, cachelines: 2, members: 4 */
	/* sum members: 68, holes: 1, sum holes: 4 */
	/* last cacheline: 8 bytes */
};
struct pnp_info_buffer {
	char *                     buffer;               /*     0     8 */
	char *                     curr;                 /*     8     8 */
	long unsigned int          size;                 /*    16     8 */
	long unsigned int          len;                  /*    24     8 */
	int                        stop;                 /*    32     4 */
	int                        error;                /*    36     4 */

	/* size: 40, cachelines: 1, members: 6 */
	/* last cacheline: 40 bytes */
};
struct pnp_fixup {
	char                       id[7];                /*     0     7 */

	/* XXX 1 byte hole, try to pack */

	void                       (*quirk_function)(struct pnp_dev *); /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 15, holes: 1, sum holes: 1 */
	/* last cacheline: 16 bytes */
};
struct acpipnp_parse_option_s {
	struct pnp_dev *           dev;                  /*     0     8 */
	unsigned int               option_flags;         /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct clk_bulk_data {
	const char  *              id;                   /*     0     8 */
	struct clk *               clk;                  /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct clk_bulk_devres {
	struct clk_bulk_data *     clks;                 /*     0     8 */
	int                        num_clks;             /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct clk_lookup {
	struct list_head           node;                 /*     0    16 */
	const char  *              dev_id;               /*    16     8 */
	const char  *              con_id;               /*    24     8 */
	struct clk *               clk;                  /*    32     8 */
	struct clk_hw *            clk_hw;               /*    40     8 */

	/* size: 48, cachelines: 1, members: 5 */
	/* last cacheline: 48 bytes */
};
struct clk_hw {
	struct clk_core *          core;                 /*     0     8 */
	struct clk *               clk;                  /*     8     8 */
	const struct clk_init_data  * init;              /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct clk_rate_request {
	long unsigned int          rate;                 /*     0     8 */
	long unsigned int          min_rate;             /*     8     8 */
	long unsigned int          max_rate;             /*    16     8 */
	long unsigned int          best_parent_rate;     /*    24     8 */
	struct clk_hw *            best_parent_hw;       /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct clk_duty {
	unsigned int               num;                  /*     0     4 */
	unsigned int               den;                  /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct clk_ops {
	int                        (*prepare)(struct clk_hw *); /*     0     8 */
	void                       (*unprepare)(struct clk_hw *); /*     8     8 */
	int                        (*is_prepared)(struct clk_hw *); /*    16     8 */
	void                       (*unprepare_unused)(struct clk_hw *); /*    24     8 */
	int                        (*enable)(struct clk_hw *); /*    32     8 */
	void                       (*disable)(struct clk_hw *); /*    40     8 */
	int                        (*is_enabled)(struct clk_hw *); /*    48     8 */
	void                       (*disable_unused)(struct clk_hw *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*save_context)(struct clk_hw *); /*    64     8 */
	void                       (*restore_context)(struct clk_hw *); /*    72     8 */
	long unsigned int          (*recalc_rate)(struct clk_hw *, long unsigned int); /*    80     8 */
	long int                   (*round_rate)(struct clk_hw *, long unsigned int, long unsigned int *); /*    88     8 */
	int                        (*determine_rate)(struct clk_hw *, struct clk_rate_request *); /*    96     8 */
	int                        (*set_parent)(struct clk_hw *, u8); /*   104     8 */
	u8                         (*get_parent)(struct clk_hw *); /*   112     8 */
	int                        (*set_rate)(struct clk_hw *, long unsigned int, long unsigned int); /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	int                        (*set_rate_and_parent)(struct clk_hw *, long unsigned int, long unsigned int, u8); /*   128     8 */
	long unsigned int          (*recalc_accuracy)(struct clk_hw *, long unsigned int); /*   136     8 */
	int                        (*get_phase)(struct clk_hw *); /*   144     8 */
	int                        (*set_phase)(struct clk_hw *, int); /*   152     8 */
	int                        (*get_duty_cycle)(struct clk_hw *, struct clk_duty *); /*   160     8 */
	int                        (*set_duty_cycle)(struct clk_hw *, struct clk_duty *); /*   168     8 */
	int                        (*init)(struct clk_hw *); /*   176     8 */
	void                       (*terminate)(struct clk_hw *); /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	void                       (*debug_init)(struct clk_hw *, struct dentry *); /*   192     8 */

	/* size: 200, cachelines: 4, members: 25 */
	/* last cacheline: 8 bytes */
};
struct clk_parent_data {
	const struct clk_hw  *     hw;                   /*     0     8 */
	const char  *              fw_name;              /*     8     8 */
	const char  *              name;                 /*    16     8 */
	int                        index;                /*    24     4 */

	/* size: 32, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct clk_init_data {
	const char  *              name;                 /*     0     8 */
	const struct clk_ops  *    ops;                  /*     8     8 */
	const char  * const *      parent_names;         /*    16     8 */
	const struct clk_parent_data  * parent_data;     /*    24     8 */
	const struct clk_hw  * *   parent_hws;           /*    32     8 */
	u8                         num_parents;          /*    40     1 */

	/* XXX 7 bytes hole, try to pack */

	long unsigned int          flags;                /*    48     8 */

	/* size: 56, cachelines: 1, members: 7 */
	/* sum members: 49, holes: 1, sum holes: 7 */
	/* last cacheline: 56 bytes */
};
struct clk_lookup_alloc {
	struct clk_lookup          cl;                   /*     0    48 */
	char                       dev_id[20];           /*    48    20 */
	/* --- cacheline 1 boundary (64 bytes) was 4 bytes ago --- */
	char                       con_id[16];           /*    68    16 */

	/* size: 88, cachelines: 2, members: 3 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct clk_notifier {
	struct clk *               clk;                  /*     0     8 */
	struct srcu_notifier_head  notifier_head;        /*     8 50480 */
	/* --- cacheline 788 boundary (50432 bytes) was 56 bytes ago --- */
	struct list_head           node;                 /* 50488    16 */

	/* size: 50504, cachelines: 790, members: 3 */
	/* last cacheline: 8 bytes */
};
struct clk {
	struct clk_core *          core;                 /*     0     8 */
	struct device *            dev;                  /*     8     8 */
	const char  *              dev_id;               /*    16     8 */
	const char  *              con_id;               /*    24     8 */
	long unsigned int          min_rate;             /*    32     8 */
	long unsigned int          max_rate;             /*    40     8 */
	unsigned int               exclusive_count;      /*    48     4 */

	/* XXX 4 bytes hole, try to pack */

	struct hlist_node          clks_node;            /*    56    16 */

	/* size: 72, cachelines: 2, members: 8 */
	/* sum members: 68, holes: 1, sum holes: 4 */
	/* last cacheline: 8 bytes */
};
struct clk_notifier_data {
	struct clk *               clk;                  /*     0     8 */
	long unsigned int          old_rate;             /*     8     8 */
	long unsigned int          new_rate;             /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct clk_core {
	const char  *              name;                 /*     0     8 */
	const struct clk_ops  *    ops;                  /*     8     8 */
	struct clk_hw *            hw;                   /*    16     8 */
	struct module *            owner;                /*    24     8 */
	struct device *            dev;                  /*    32     8 */
	struct device_node *       of_node;              /*    40     8 */
	struct clk_core *          parent;               /*    48     8 */
	struct clk_parent_map *    parents;              /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u8                         num_parents;          /*    64     1 */
	u8                         new_parent_index;     /*    65     1 */

	/* XXX 6 bytes hole, try to pack */

	long unsigned int          rate;                 /*    72     8 */
	long unsigned int          req_rate;             /*    80     8 */
	long unsigned int          new_rate;             /*    88     8 */
	struct clk_core *          new_parent;           /*    96     8 */
	struct clk_core *          new_child;            /*   104     8 */
	long unsigned int          flags;                /*   112     8 */
	bool                       orphan;               /*   120     1 */
	bool                       rpm_enabled;          /*   121     1 */

	/* XXX 2 bytes hole, try to pack */

	unsigned int               enable_count;         /*   124     4 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	unsigned int               prepare_count;        /*   128     4 */
	unsigned int               protect_count;        /*   132     4 */
	long unsigned int          min_rate;             /*   136     8 */
	long unsigned int          max_rate;             /*   144     8 */
	long unsigned int          accuracy;             /*   152     8 */
	int                        phase;                /*   160     4 */
	struct clk_duty            duty;                 /*   164     8 */

	/* XXX 4 bytes hole, try to pack */

	struct hlist_head          children;             /*   176     8 */
	struct hlist_node          child_node;           /*   184    16 */
	/* --- cacheline 3 boundary (192 bytes) was 8 bytes ago --- */
	struct hlist_head          clks;                 /*   200     8 */
	unsigned int               notifier_count;       /*   208     4 */

	/* XXX 4 bytes hole, try to pack */

	struct dentry *            dentry;               /*   216     8 */
	struct hlist_node          debug_node;           /*   224    16 */
	struct kref                ref;                  /*   240     4 */

	/* size: 248, cachelines: 4, members: 33 */
	/* sum members: 228, holes: 4, sum holes: 16 */
	/* padding: 4 */
	/* last cacheline: 56 bytes */
};
struct clk_parent_map {
	const struct clk_hw  *     hw;                   /*     0     8 */
	struct clk_core *          core;                 /*     8     8 */
	const char  *              fw_name;              /*    16     8 */
	const char  *              name;                 /*    24     8 */
	int                        index;                /*    32     4 */

	/* size: 40, cachelines: 1, members: 5 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct trace_event_raw_clk {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        __data_loc_name;      /*     8     4 */
	char                       __data[];             /*    12     0 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct trace_event_raw_clk_rate {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        __data_loc_name;      /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          rate;                 /*    16     8 */
	char                       __data[];             /*    24     0 */

	/* size: 24, cachelines: 1, members: 4 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_clk_parent {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        __data_loc_name;      /*     8     4 */
	u32                        __data_loc_pname;     /*    12     4 */
	char                       __data[];             /*    16     0 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct trace_event_raw_clk_phase {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        __data_loc_name;      /*     8     4 */
	int                        phase;                /*    12     4 */
	char                       __data[];             /*    16     0 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct trace_event_raw_clk_duty_cycle {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        __data_loc_name;      /*     8     4 */
	unsigned int               num;                  /*    12     4 */
	unsigned int               den;                  /*    16     4 */
	char                       __data[];             /*    20     0 */

	/* size: 20, cachelines: 1, members: 5 */
	/* last cacheline: 20 bytes */
};
struct trace_event_data_offsets_clk {
	u32                        name;                 /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct trace_event_data_offsets_clk_rate {
	u32                        name;                 /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct trace_event_data_offsets_clk_parent {
	u32                        name;                 /*     0     4 */
	u32                        pname;                /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct trace_event_data_offsets_clk_phase {
	u32                        name;                 /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct trace_event_data_offsets_clk_duty_cycle {
	u32                        name;                 /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct clk_div_table {
	unsigned int               val;                  /*     0     4 */
	unsigned int               div;                  /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct clk_divider {
	struct clk_hw              hw;                   /*     0    24 */
	void *                     reg;                  /*    24     8 */
	u8                         shift;                /*    32     1 */
	u8                         width;                /*    33     1 */
	u8                         flags;                /*    34     1 */

	/* XXX 5 bytes hole, try to pack */

	const struct clk_div_table  * table;             /*    40     8 */
	spinlock_t *               lock;                 /*    48     8 */

	/* size: 56, cachelines: 1, members: 7 */
	/* sum members: 51, holes: 1, sum holes: 5 */
	/* last cacheline: 56 bytes */
};
struct clk_fixed_factor {
	struct clk_hw              hw;                   /*     0    24 */
	unsigned int               mult;                 /*    24     4 */
	unsigned int               div;                  /*    28     4 */

	/* size: 32, cachelines: 1, members: 3 */
	/* last cacheline: 32 bytes */
};
struct clk_fixed_rate {
	struct clk_hw              hw;                   /*     0    24 */
	long unsigned int          fixed_rate;           /*    24     8 */
	long unsigned int          fixed_accuracy;       /*    32     8 */
	long unsigned int          flags;                /*    40     8 */

	/* size: 48, cachelines: 1, members: 4 */
	/* last cacheline: 48 bytes */
};
struct clk_gate {
	struct clk_hw              hw;                   /*     0    24 */
	void *                     reg;                  /*    24     8 */
	u8                         bit_idx;              /*    32     1 */
	u8                         flags;                /*    33     1 */

	/* XXX 6 bytes hole, try to pack */

	spinlock_t *               lock;                 /*    40     8 */

	/* size: 48, cachelines: 1, members: 5 */
	/* sum members: 42, holes: 1, sum holes: 6 */
	/* last cacheline: 48 bytes */
};
struct clk_multiplier {
	struct clk_hw              hw;                   /*     0    24 */
	void *                     reg;                  /*    24     8 */
	u8                         shift;                /*    32     1 */
	u8                         width;                /*    33     1 */
	u8                         flags;                /*    34     1 */

	/* XXX 5 bytes hole, try to pack */

	spinlock_t *               lock;                 /*    40     8 */

	/* size: 48, cachelines: 1, members: 6 */
	/* sum members: 43, holes: 1, sum holes: 5 */
	/* last cacheline: 48 bytes */
};
struct clk_mux {
	struct clk_hw              hw;                   /*     0    24 */
	void *                     reg;                  /*    24     8 */
	u32 *                      table;                /*    32     8 */
	u32                        mask;                 /*    40     4 */
	u8                         shift;                /*    44     1 */
	u8                         flags;                /*    45     1 */

	/* XXX 2 bytes hole, try to pack */

	spinlock_t *               lock;                 /*    48     8 */

	/* size: 56, cachelines: 1, members: 7 */
	/* sum members: 54, holes: 1, sum holes: 2 */
	/* last cacheline: 56 bytes */
};
struct clk_composite {
	struct clk_hw              hw;                   /*     0    24 */
	struct clk_ops             ops;                  /*    24   200 */
	/* --- cacheline 3 boundary (192 bytes) was 32 bytes ago --- */
	struct clk_hw *            mux_hw;               /*   224     8 */
	struct clk_hw *            rate_hw;              /*   232     8 */
	struct clk_hw *            gate_hw;              /*   240     8 */
	const struct clk_ops  *    mux_ops;              /*   248     8 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	const struct clk_ops  *    rate_ops;             /*   256     8 */
	const struct clk_ops  *    gate_ops;             /*   264     8 */

	/* size: 272, cachelines: 5, members: 8 */
	/* last cacheline: 16 bytes */
};
struct clk_fractional_divider {
	struct clk_hw              hw;                   /*     0    24 */
	void *                     reg;                  /*    24     8 */
	u8                         mshift;               /*    32     1 */
	u8                         mwidth;               /*    33     1 */

	/* XXX 2 bytes hole, try to pack */

	u32                        mmask;                /*    36     4 */
	u8                         nshift;               /*    40     1 */
	u8                         nwidth;               /*    41     1 */

	/* XXX 2 bytes hole, try to pack */

	u32                        nmask;                /*    44     4 */
	u8                         flags;                /*    48     1 */

	/* XXX 7 bytes hole, try to pack */

	void                       (*approximation)(struct clk_hw *, long unsigned int, long unsigned int *, long unsigned int *, long unsigned int *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	spinlock_t *               lock;                 /*    64     8 */

	/* size: 72, cachelines: 2, members: 11 */
	/* sum members: 61, holes: 3, sum holes: 11 */
	/* last cacheline: 8 bytes */
};
struct clk_gpio {
	struct clk_hw              hw;                   /*     0    24 */
	struct gpio_desc *         gpiod;                /*    24     8 */

	/* size: 32, cachelines: 1, members: 2 */
	/* last cacheline: 32 bytes */
};
struct pmc_clk {
	const char  *              name;                 /*     0     8 */
	long unsigned int          freq;                 /*     8     8 */
	const char  *              parent_name;          /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct pmc_clk_data {
	void *                     base;                 /*     0     8 */
	const struct pmc_clk  *    clks;                 /*     8     8 */
	bool                       critical;             /*    16     1 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 7 */
	/* last cacheline: 24 bytes */
};
struct clk_plt_fixed {
	struct clk_hw *            clk;                  /*     0     8 */
	struct clk_lookup *        lookup;               /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct clk_plt {
	struct clk_hw              hw;                   /*     0    24 */
	void *                     reg;                  /*    24     8 */
	struct clk_lookup *        lookup;               /*    32     8 */
	spinlock_t                 lock;                 /*    40     4 */

	/* size: 48, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 48 bytes */
};
struct clk_plt_data {
	struct clk_plt_fixed * *   parents;              /*     0     8 */
	u8                         nparents;             /*     8     1 */

	/* XXX 7 bytes hole, try to pack */

	struct clk_plt *           clks[6];              /*    16    48 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct clk_lookup *        mclk_lookup;          /*    64     8 */
	struct clk_lookup *        ether_clk_lookup;     /*    72     8 */

	/* size: 80, cachelines: 2, members: 5 */
	/* sum members: 73, holes: 1, sum holes: 7 */
	/* last cacheline: 16 bytes */
};
struct data_chunk {
	size_t                     size;                 /*     0     8 */
	size_t                     icg;                  /*     8     8 */
	size_t                     dst_icg;              /*    16     8 */
	size_t                     src_icg;              /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct dma_interleaved_template {
	dma_addr_t                 src_start;            /*     0     8 */
	dma_addr_t                 dst_start;            /*     8     8 */
	enum dma_transfer_direction dir;                 /*    16     4 */
	bool                       src_inc;              /*    20     1 */
	bool                       dst_inc;              /*    21     1 */
	bool                       src_sgl;              /*    22     1 */
	bool                       dst_sgl;              /*    23     1 */
	size_t                     numf;                 /*    24     8 */
	size_t                     frame_size;           /*    32     8 */
	struct data_chunk          sgl[];                /*    40     0 */

	/* size: 40, cachelines: 1, members: 10 */
	/* last cacheline: 40 bytes */
};
struct dma_chan_percpu {
	long unsigned int          memcpy_count;         /*     0     8 */
	long unsigned int          bytes_transferred;    /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct dma_router {
	struct device *            dev;                  /*     0     8 */
	void                       (*route_free)(struct device *, void *); /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct dma_chan {
	struct dma_device *        device;               /*     0     8 */
	struct device *            slave;                /*     8     8 */
	dma_cookie_t               cookie;               /*    16     4 */
	dma_cookie_t               completed_cookie;     /*    20     4 */
	int                        chan_id;              /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	struct dma_chan_dev *      dev;                  /*    32     8 */
	const char  *              name;                 /*    40     8 */
	char *                     dbg_client_name;      /*    48     8 */
	struct list_head           device_node;          /*    56    16 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	struct dma_chan_percpu *   local;                /*    72     8 */
	int                        client_count;         /*    80     4 */
	int                        table_count;          /*    84     4 */
	struct dma_router *        router;               /*    88     8 */
	void *                     route_data;           /*    96     8 */
	void *                     private;              /*   104     8 */

	/* size: 112, cachelines: 2, members: 15 */
	/* sum members: 108, holes: 1, sum holes: 4 */
	/* last cacheline: 48 bytes */
};
struct dma_device {
	struct kref                ref;                  /*     0     4 */
	unsigned int               chancnt;              /*     4     4 */
	unsigned int               privatecnt;           /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           channels;             /*    16    16 */
	struct list_head           global_node;          /*    32    16 */
	struct dma_filter          filter;               /*    48    24 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	dma_cap_mask_t             cap_mask;             /*    72     8 */
	enum dma_desc_metadata_mode desc_metadata_modes; /*    80     4 */
	short unsigned int         max_xor;              /*    84     2 */
	short unsigned int         max_pq;               /*    86     2 */
	enum dmaengine_alignment   copy_align;           /*    88     4 */
	enum dmaengine_alignment   xor_align;            /*    92     4 */
	enum dmaengine_alignment   pq_align;             /*    96     4 */
	enum dmaengine_alignment   fill_align;           /*   100     4 */
	int                        dev_id;               /*   104     4 */

	/* XXX 4 bytes hole, try to pack */

	struct device *            dev;                  /*   112     8 */
	struct module *            owner;                /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct ida                 chan_ida;             /*   128    16 */
	struct mutex               chan_mutex;           /*   144    32 */
	u32                        src_addr_widths;      /*   176     4 */
	u32                        dst_addr_widths;      /*   180     4 */
	u32                        directions;           /*   184     4 */
	u32                        min_burst;            /*   188     4 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	u32                        max_burst;            /*   192     4 */
	u32                        max_sg_burst;         /*   196     4 */
	bool                       descriptor_reuse;     /*   200     1 */

	/* XXX 3 bytes hole, try to pack */

	enum dma_residue_granularity residue_granularity; /*   204     4 */
	int                        (*device_alloc_chan_resources)(struct dma_chan *); /*   208     8 */
	void                       (*device_free_chan_resources)(struct dma_chan *); /*   216     8 */
	struct dma_async_tx_descriptor * (*device_prep_dma_memcpy)(struct dma_chan *, dma_addr_t, dma_addr_t, size_t, long unsigned int); /*   224     8 */
	struct dma_async_tx_descriptor * (*device_prep_dma_xor)(struct dma_chan *, dma_addr_t, dma_addr_t *, unsigned int, size_t, long unsigned int); /*   232     8 */
	struct dma_async_tx_descriptor * (*device_prep_dma_xor_val)(struct dma_chan *, dma_addr_t *, unsigned int, size_t, enum sum_check_flags *, long unsigned int); /*   240     8 */
	struct dma_async_tx_descriptor * (*device_prep_dma_pq)(struct dma_chan *, dma_addr_t *, dma_addr_t *, unsigned int, const unsigned char  *, size_t, long unsigned int); /*   248     8 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	struct dma_async_tx_descriptor * (*device_prep_dma_pq_val)(struct dma_chan *, dma_addr_t *, dma_addr_t *, unsigned int, const unsigned char  *, size_t, enum sum_check_flags *, long unsigned int); /*   256     8 */
	struct dma_async_tx_descriptor * (*device_prep_dma_memset)(struct dma_chan *, dma_addr_t, int, size_t, long unsigned int); /*   264     8 */
	struct dma_async_tx_descriptor * (*device_prep_dma_memset_sg)(struct dma_chan *, struct scatterlist *, unsigned int, int, long unsigned int); /*   272     8 */
	struct dma_async_tx_descriptor * (*device_prep_dma_interrupt)(struct dma_chan *, long unsigned int); /*   280     8 */
	struct dma_async_tx_descriptor * (*device_prep_slave_sg)(struct dma_chan *, struct scatterlist *, unsigned int, enum dma_transfer_direction, long unsigned int, void *); /*   288     8 */
	struct dma_async_tx_descriptor * (*device_prep_dma_cyclic)(struct dma_chan *, dma_addr_t, size_t, size_t, enum dma_transfer_direction, long unsigned int); /*   296     8 */
	struct dma_async_tx_descriptor * (*device_prep_interleaved_dma)(struct dma_chan *, struct dma_interleaved_template *, long unsigned int); /*   304     8 */
	struct dma_async_tx_descriptor * (*device_prep_dma_imm_data)(struct dma_chan *, dma_addr_t, u64, long unsigned int); /*   312     8 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	void                       (*device_caps)(struct dma_chan *, struct dma_slave_caps *); /*   320     8 */
	int                        (*device_config)(struct dma_chan *, struct dma_slave_config *); /*   328     8 */
	int                        (*device_pause)(struct dma_chan *); /*   336     8 */
	int                        (*device_resume)(struct dma_chan *); /*   344     8 */
	int                        (*device_terminate_all)(struct dma_chan *); /*   352     8 */
	void                       (*device_synchronize)(struct dma_chan *); /*   360     8 */
	enum dma_status            (*device_tx_status)(struct dma_chan *, dma_cookie_t, struct dma_tx_state *); /*   368     8 */
	void                       (*device_issue_pending)(struct dma_chan *); /*   376     8 */
	/* --- cacheline 6 boundary (384 bytes) --- */
	void                       (*device_release)(struct dma_device *); /*   384     8 */
	void                       (*dbg_summary_show)(struct seq_file *, struct dma_device *); /*   392     8 */
	struct dentry *            dbg_dev_root;         /*   400     8 */

	/* size: 408, cachelines: 7, members: 52 */
	/* sum members: 397, holes: 3, sum holes: 11 */
	/* last cacheline: 24 bytes */
};
struct dma_chan_dev {
	struct dma_chan *          chan;                 /*     0     8 */
	struct device              device __attribute__((__aligned__(8))); /*     8  1000 */
	/* --- cacheline 15 boundary (960 bytes) was 48 bytes ago --- */
	int                        dev_id;               /*  1008     4 */

	/* size: 1016, cachelines: 16, members: 3 */
	/* padding: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 56 bytes */
} __attribute__((__aligned__(8)));
struct dma_slave_config {
	enum dma_transfer_direction direction;           /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	phys_addr_t                src_addr;             /*     8     8 */
	phys_addr_t                dst_addr;             /*    16     8 */
	enum dma_slave_buswidth    src_addr_width;       /*    24     4 */
	enum dma_slave_buswidth    dst_addr_width;       /*    28     4 */
	u32                        src_maxburst;         /*    32     4 */
	u32                        dst_maxburst;         /*    36     4 */
	u32                        src_port_window_size; /*    40     4 */
	u32                        dst_port_window_size; /*    44     4 */
	bool                       device_fc;            /*    48     1 */

	/* XXX 3 bytes hole, try to pack */

	unsigned int               slave_id;             /*    52     4 */

	/* size: 56, cachelines: 1, members: 11 */
	/* sum members: 49, holes: 2, sum holes: 7 */
	/* last cacheline: 56 bytes */
};
struct dma_slave_caps {
	u32                        src_addr_widths;      /*     0     4 */
	u32                        dst_addr_widths;      /*     4     4 */
	u32                        directions;           /*     8     4 */
	u32                        min_burst;            /*    12     4 */
	u32                        max_burst;            /*    16     4 */
	u32                        max_sg_burst;         /*    20     4 */
	bool                       cmd_pause;            /*    24     1 */
	bool                       cmd_resume;           /*    25     1 */
	bool                       cmd_terminate;        /*    26     1 */

	/* XXX 1 byte hole, try to pack */

	enum dma_residue_granularity residue_granularity; /*    28     4 */
	bool                       descriptor_reuse;     /*    32     1 */

	/* size: 36, cachelines: 1, members: 11 */
	/* sum members: 32, holes: 1, sum holes: 1 */
	/* padding: 3 */
	/* last cacheline: 36 bytes */
};
struct dmaengine_result {
	enum dmaengine_tx_result   result;               /*     0     4 */
	u32                        residue;              /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct dmaengine_unmap_data {
	u16                        map_cnt;              /*     0     2 */
	u8                         to_cnt;               /*     2     1 */
	u8                         from_cnt;             /*     3     1 */
	u8                         bidi_cnt;             /*     4     1 */

	/* XXX 3 bytes hole, try to pack */

	struct device *            dev;                  /*     8     8 */
	struct kref                kref;                 /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	size_t                     len;                  /*    24     8 */
	dma_addr_t                 addr[];               /*    32     0 */

	/* size: 32, cachelines: 1, members: 8 */
	/* sum members: 25, holes: 2, sum holes: 7 */
	/* last cacheline: 32 bytes */
};
struct dma_descriptor_metadata_ops {
	int                        (*attach)(struct dma_async_tx_descriptor *, void *, size_t); /*     0     8 */
	void *                     (*get_ptr)(struct dma_async_tx_descriptor *, size_t *, size_t *); /*     8     8 */
	int                        (*set_len)(struct dma_async_tx_descriptor *, size_t); /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct dma_async_tx_descriptor {
	dma_cookie_t               cookie;               /*     0     4 */
	enum dma_ctrl_flags        flags;                /*     4     4 */
	dma_addr_t                 phys;                 /*     8     8 */
	struct dma_chan *          chan;                 /*    16     8 */
	dma_cookie_t               (*tx_submit)(struct dma_async_tx_descriptor *); /*    24     8 */
	int                        (*desc_free)(struct dma_async_tx_descriptor *); /*    32     8 */
	dma_async_tx_callback      callback;             /*    40     8 */
	dma_async_tx_callback_result callback_result;    /*    48     8 */
	void *                     callback_param;       /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct dmaengine_unmap_data * unmap;             /*    64     8 */
	enum dma_desc_metadata_mode desc_metadata_mode;  /*    72     4 */

	/* XXX 4 bytes hole, try to pack */

	struct dma_descriptor_metadata_ops * metadata_ops; /*    80     8 */

	/* size: 88, cachelines: 2, members: 12 */
	/* sum members: 84, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct dma_tx_state {
	dma_cookie_t               last;                 /*     0     4 */
	dma_cookie_t               used;                 /*     4     4 */
	u32                        residue;              /*     8     4 */
	u32                        in_flight_bytes;      /*    12     4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct dma_slave_map {
	const char  *              devname;              /*     0     8 */
	const char  *              slave;                /*     8     8 */
	void *                     param;                /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct dma_filter {
	dma_filter_fn              fn;                   /*     0     8 */
	int                        mapcnt;               /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	const struct dma_slave_map  * map;               /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct dma_chan_tbl_ent {
	struct dma_chan *          chan;                 /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct dmaengine_unmap_pool {
	struct kmem_cache *        cache;                /*     0     8 */
	const char  *              name;                 /*     8     8 */
	mempool_t *                pool;                 /*    16     8 */
	size_t                     size;                 /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct dmaengine_desc_callback {
	dma_async_tx_callback      callback;             /*     0     8 */
	dma_async_tx_callback_result callback_result;    /*     8     8 */
	void *                     callback_param;       /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct virt_dma_desc {
	struct dma_async_tx_descriptor tx;               /*     0    88 */
	/* --- cacheline 1 boundary (64 bytes) was 24 bytes ago --- */
	struct dmaengine_result    tx_result;            /*    88     8 */
	struct list_head           node;                 /*    96    16 */

	/* size: 112, cachelines: 2, members: 3 */
	/* last cacheline: 48 bytes */
};
struct virt_dma_chan {
	struct dma_chan            chan;                 /*     0   112 */
	/* --- cacheline 1 boundary (64 bytes) was 48 bytes ago --- */
	struct tasklet_struct      task;                 /*   112    48 */
	/* --- cacheline 2 boundary (128 bytes) was 32 bytes ago --- */
	void                       (*desc_free)(struct virt_dma_desc *); /*   160     8 */
	spinlock_t                 lock;                 /*   168     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           desc_allocated;       /*   176    16 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	struct list_head           desc_submitted;       /*   192    16 */
	struct list_head           desc_issued;          /*   208    16 */
	struct list_head           desc_completed;       /*   224    16 */
	struct list_head           desc_terminated;      /*   240    16 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	struct virt_dma_desc *     cyclic;               /*   256     8 */

	/* size: 264, cachelines: 5, members: 10 */
	/* sum members: 260, holes: 1, sum holes: 4 */
	/* last cacheline: 8 bytes */
};
struct acpi_table_csrt {
	struct acpi_table_header   header;               /*     0    36 */

	/* size: 36, cachelines: 1, members: 1 */
	/* last cacheline: 36 bytes */
};
struct acpi_csrt_group {
	u32                        length;               /*     0     4 */
	u32                        vendor_id;            /*     4     4 */
	u32                        subvendor_id;         /*     8     4 */
	u16                        device_id;            /*    12     2 */
	u16                        subdevice_id;         /*    14     2 */
	u16                        revision;             /*    16     2 */
	u16                        reserved;             /*    18     2 */
	u32                        shared_info_length;   /*    20     4 */

	/* size: 24, cachelines: 1, members: 8 */
	/* last cacheline: 24 bytes */
};
struct acpi_csrt_shared_info {
	u16                        major_version;        /*     0     2 */
	u16                        minor_version;        /*     2     2 */
	u32                        mmio_base_low;        /*     4     4 */
	u32                        mmio_base_high;       /*     8     4 */
	u32                        gsi_interrupt;        /*    12     4 */
	u8                         interrupt_polarity;   /*    16     1 */
	u8                         interrupt_mode;       /*    17     1 */
	u8                         num_channels;         /*    18     1 */
	u8                         dma_address_width;    /*    19     1 */
	u16                        base_request_line;    /*    20     2 */
	u16                        num_handshake_signals; /*    22     2 */
	u32                        max_block_size;       /*    24     4 */

	/* size: 28, cachelines: 1, members: 12 */
	/* last cacheline: 28 bytes */
};
struct acpi_dma_spec {
	int                        chan_id;              /*     0     4 */
	int                        slave_id;             /*     4     4 */
	struct device *            dev;                  /*     8     8 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct acpi_dma {
	struct list_head           dma_controllers;      /*     0    16 */
	struct device *            dev;                  /*    16     8 */
	struct dma_chan *          (*acpi_dma_xlate)(struct acpi_dma_spec *, struct acpi_dma *); /*    24     8 */
	void *                     data;                 /*    32     8 */
	short unsigned int         base_request_line;    /*    40     2 */
	short unsigned int         end_request_line;     /*    42     2 */

	/* size: 48, cachelines: 1, members: 6 */
	/* padding: 4 */
	/* last cacheline: 48 bytes */
};
struct acpi_dma_filter_info {
	dma_cap_mask_t             dma_cap;              /*     0     8 */
	dma_filter_fn              filter_fn;            /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct acpi_dma_parser_data {
	struct acpi_dma_spec       dma_spec;             /*     0    16 */
	size_t                     index;                /*    16     8 */
	size_t                     n;                    /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* last cacheline: 32 bytes */
};
struct dw_dma_slave {
	struct device *            dma_dev;              /*     0     8 */
	u8                         src_id;               /*     8     1 */
	u8                         dst_id;               /*     9     1 */
	u8                         m_master;             /*    10     1 */
	u8                         p_master;             /*    11     1 */
	u8                         channels;             /*    12     1 */
	bool                       hs_polarity;          /*    13     1 */

	/* size: 16, cachelines: 1, members: 7 */
	/* padding: 2 */
	/* last cacheline: 16 bytes */
};
struct dw_dma_platform_data {
	unsigned int               nr_channels;          /*     0     4 */
	unsigned char              chan_allocation_order; /*     4     1 */
	unsigned char              chan_priority;        /*     5     1 */

	/* XXX 2 bytes hole, try to pack */

	unsigned int               block_size;           /*     8     4 */
	unsigned char              nr_masters;           /*    12     1 */
	unsigned char              data_width[4];        /*    13     4 */
	unsigned char              multi_block[8];       /*    17     8 */

	/* XXX 3 bytes hole, try to pack */

	u32                        max_burst[8];         /*    28    32 */
	unsigned char              protctl;              /*    60     1 */

	/* size: 64, cachelines: 1, members: 9 */
	/* sum members: 56, holes: 2, sum holes: 5 */
	/* padding: 3 */
};
struct dw_dma_chip {
	struct device *            dev;                  /*     0     8 */
	int                        id;                   /*     8     4 */
	int                        irq;                  /*    12     4 */
	void *                     regs;                 /*    16     8 */
	struct clk *               clk;                  /*    24     8 */
	struct dw_dma *            dw;                   /*    32     8 */
	const struct dw_dma_platform_data  * pdata;      /*    40     8 */

	/* size: 48, cachelines: 1, members: 7 */
	/* last cacheline: 48 bytes */
};
struct dw_dma {
	struct dma_device          dma;                  /*     0   408 */
	/* --- cacheline 6 boundary (384 bytes) was 24 bytes ago --- */
	char                       name[20];             /*   408    20 */

	/* XXX 4 bytes hole, try to pack */

	void *                     regs;                 /*   432     8 */
	struct dma_pool *          desc_pool;            /*   440     8 */
	/* --- cacheline 7 boundary (448 bytes) --- */
	struct tasklet_struct      tasklet;              /*   448    48 */
	struct dw_dma_chan *       chan;                 /*   496     8 */
	u8                         all_chan_mask;        /*   504     1 */
	u8                         in_use;               /*   505     1 */

	/* XXX 6 bytes hole, try to pack */

	/* --- cacheline 8 boundary (512 bytes) --- */
	void                       (*initialize_chan)(struct dw_dma_chan *); /*   512     8 */
	void                       (*suspend_chan)(struct dw_dma_chan *, bool); /*   520     8 */
	void                       (*resume_chan)(struct dw_dma_chan *, bool); /*   528     8 */
	u32                        (*prepare_ctllo)(struct dw_dma_chan *); /*   536     8 */
	void                       (*encode_maxburst)(struct dw_dma_chan *, u32 *); /*   544     8 */
	u32                        (*bytes2block)(struct dw_dma_chan *, size_t, unsigned int, size_t *); /*   552     8 */
	size_t                     (*block2bytes)(struct dw_dma_chan *, u32, u32); /*   560     8 */
	void                       (*set_device_name)(struct dw_dma *, int); /*   568     8 */
	/* --- cacheline 9 boundary (576 bytes) --- */
	void                       (*disable)(struct dw_dma *); /*   576     8 */
	void                       (*enable)(struct dw_dma *); /*   584     8 */
	struct dw_dma_platform_data * pdata;             /*   592     8 */

	/* size: 600, cachelines: 10, members: 19 */
	/* sum members: 590, holes: 2, sum holes: 10 */
	/* last cacheline: 24 bytes */
};
struct dw_dma_chan_regs {
	u32                        SAR;                  /*     0     4 */
	u32                        __pad_SAR;            /*     4     4 */
	u32                        DAR;                  /*     8     4 */
	u32                        __pad_DAR;            /*    12     4 */
	u32                        LLP;                  /*    16     4 */
	u32                        __pad_LLP;            /*    20     4 */
	u32                        CTL_LO;               /*    24     4 */
	u32                        CTL_HI;               /*    28     4 */
	u32                        SSTAT;                /*    32     4 */
	u32                        __pad_SSTAT;          /*    36     4 */
	u32                        DSTAT;                /*    40     4 */
	u32                        __pad_DSTAT;          /*    44     4 */
	u32                        SSTATAR;              /*    48     4 */
	u32                        __pad_SSTATAR;        /*    52     4 */
	u32                        DSTATAR;              /*    56     4 */
	u32                        __pad_DSTATAR;        /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u32                        CFG_LO;               /*    64     4 */
	u32                        CFG_HI;               /*    68     4 */
	u32                        SGR;                  /*    72     4 */
	u32                        __pad_SGR;            /*    76     4 */
	u32                        DSR;                  /*    80     4 */
	u32                        __pad_DSR;            /*    84     4 */

	/* size: 88, cachelines: 2, members: 22 */
	/* last cacheline: 24 bytes */
};
struct dw_dma_irq_regs {
	u32                        XFER;                 /*     0     4 */
	u32                        __pad_XFER;           /*     4     4 */
	u32                        BLOCK;                /*     8     4 */
	u32                        __pad_BLOCK;          /*    12     4 */
	u32                        SRC_TRAN;             /*    16     4 */
	u32                        __pad_SRC_TRAN;       /*    20     4 */
	u32                        DST_TRAN;             /*    24     4 */
	u32                        __pad_DST_TRAN;       /*    28     4 */
	u32                        ERROR;                /*    32     4 */
	u32                        __pad_ERROR;          /*    36     4 */

	/* size: 40, cachelines: 1, members: 10 */
	/* last cacheline: 40 bytes */
};
struct dw_dma_regs {
	struct dw_dma_chan_regs    CHAN[8];              /*     0   704 */
	/* --- cacheline 11 boundary (704 bytes) --- */
	struct dw_dma_irq_regs     RAW;                  /*   704    40 */
	struct dw_dma_irq_regs     STATUS;               /*   744    40 */
	/* --- cacheline 12 boundary (768 bytes) was 16 bytes ago --- */
	struct dw_dma_irq_regs     MASK;                 /*   784    40 */
	struct dw_dma_irq_regs     CLEAR;                /*   824    40 */
	/* --- cacheline 13 boundary (832 bytes) was 32 bytes ago --- */
	u32                        STATUS_INT;           /*   864     4 */
	u32                        __pad_STATUS_INT;     /*   868     4 */
	u32                        REQ_SRC;              /*   872     4 */
	u32                        __pad_REQ_SRC;        /*   876     4 */
	u32                        REQ_DST;              /*   880     4 */
	u32                        __pad_REQ_DST;        /*   884     4 */
	u32                        SGL_REQ_SRC;          /*   888     4 */
	u32                        __pad_SGL_REQ_SRC;    /*   892     4 */
	/* --- cacheline 14 boundary (896 bytes) --- */
	u32                        SGL_REQ_DST;          /*   896     4 */
	u32                        __pad_SGL_REQ_DST;    /*   900     4 */
	u32                        LAST_SRC;             /*   904     4 */
	u32                        __pad_LAST_SRC;       /*   908     4 */
	u32                        LAST_DST;             /*   912     4 */
	u32                        __pad_LAST_DST;       /*   916     4 */
	u32                        CFG;                  /*   920     4 */
	u32                        __pad_CFG;            /*   924     4 */
	u32                        CH_EN;                /*   928     4 */
	u32                        __pad_CH_EN;          /*   932     4 */
	u32                        ID;                   /*   936     4 */
	u32                        __pad_ID;             /*   940     4 */
	u32                        TEST;                 /*   944     4 */
	u32                        __pad_TEST;           /*   948     4 */
	u32                        CLASS_PRIORITY0;      /*   952     4 */
	u32                        __pad_CLASS_PRIORITY0; /*   956     4 */
	/* --- cacheline 15 boundary (960 bytes) --- */
	u32                        CLASS_PRIORITY1;      /*   960     4 */
	u32                        __pad_CLASS_PRIORITY1; /*   964     4 */
	u32                        __reserved;           /*   968     4 */
	u32                        DWC_PARAMS[8];        /*   972    32 */
	u32                        MULTI_BLK_TYPE;       /*  1004     4 */
	u32                        MAX_BLK_SIZE;         /*  1008     4 */
	u32                        DW_PARAMS;            /*  1012     4 */
	u32                        COMP_TYPE;            /*  1016     4 */
	u32                        COMP_VERSION;         /*  1020     4 */
	/* --- cacheline 16 boundary (1024 bytes) --- */
	u32                        FIFO_PARTITION0;      /*  1024     4 */
	u32                        __pad_FIFO_PARTITION0; /*  1028     4 */
	u32                        FIFO_PARTITION1;      /*  1032     4 */
	u32                        __pad_FIFO_PARTITION1; /*  1036     4 */
	u32                        SAI_ERR;              /*  1040     4 */
	u32                        __pad_SAI_ERR;        /*  1044     4 */
	u32                        GLOBAL_CFG;           /*  1048     4 */
	u32                        __pad_GLOBAL_CFG;     /*  1052     4 */

	/* size: 1056, cachelines: 17, members: 46 */
	/* last cacheline: 32 bytes */
};
struct dw_dma_chan {
	struct dma_chan            chan;                 /*     0   112 */
	/* --- cacheline 1 boundary (64 bytes) was 48 bytes ago --- */
	void *                     ch_regs;              /*   112     8 */
	u8                         mask;                 /*   120     1 */
	u8                         priority;             /*   121     1 */

	/* XXX 2 bytes hole, try to pack */

	enum dma_transfer_direction direction;           /*   124     4 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct list_head *         tx_node_active;       /*   128     8 */
	spinlock_t                 lock;                 /*   136     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          flags;                /*   144     8 */
	struct list_head           active_list;          /*   152    16 */
	struct list_head           queue;                /*   168    16 */
	unsigned int               descs_allocated;      /*   184     4 */
	unsigned int               block_size;           /*   188     4 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	bool                       nollp;                /*   192     1 */

	/* XXX 3 bytes hole, try to pack */

	u32                        max_burst;            /*   196     4 */
	struct dw_dma_slave        dws;                  /*   200    16 */

	/* XXX last struct has 2 bytes of padding */

	struct dma_slave_config    dma_sconfig;          /*   216    56 */

	/* size: 272, cachelines: 5, members: 16 */
	/* sum members: 263, holes: 3, sum holes: 9 */
	/* paddings: 1, sum paddings: 2 */
	/* last cacheline: 16 bytes */
};
struct dw_lli {
	__le32                     sar;                  /*     0     4 */
	__le32                     dar;                  /*     4     4 */
	__le32                     llp;                  /*     8     4 */
	__le32                     ctllo;                /*    12     4 */
	__le32                     ctlhi;                /*    16     4 */
	__le32                     sstat;                /*    20     4 */
	__le32                     dstat;                /*    24     4 */

	/* size: 28, cachelines: 1, members: 7 */
	/* last cacheline: 28 bytes */
};
struct dw_desc {
	struct dw_lli              lli;                  /*     0    28 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           desc_node;            /*    32    16 */
	struct list_head           tx_list;              /*    48    16 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct dma_async_tx_descriptor txd;              /*    64    88 */
	/* --- cacheline 2 boundary (128 bytes) was 24 bytes ago --- */
	size_t                     len;                  /*   152     8 */
	size_t                     total_len;            /*   160     8 */
	u32                        residue;              /*   168     4 */

	/* size: 176, cachelines: 3, members: 7 */
	/* sum members: 168, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 48 bytes */
};
struct dw_dma_chip_pdata {
	const struct dw_dma_platform_data  * pdata;      /*     0     8 */
	int                        (*probe)(struct dw_dma_chip *); /*     8     8 */
	int                        (*remove)(struct dw_dma_chip *); /*    16     8 */
	struct dw_dma_chip *       chip;                 /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct hsu_dma_chip {
	struct device *            dev;                  /*     0     8 */
	int                        irq;                  /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     regs;                 /*    16     8 */
	unsigned int               length;               /*    24     4 */
	unsigned int               offset;               /*    28     4 */
	struct hsu_dma *           hsu;                  /*    32     8 */

	/* size: 40, cachelines: 1, members: 6 */
	/* sum members: 36, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct hsu_dma {
	struct dma_device          dma;                  /*     0   408 */
	/* --- cacheline 6 boundary (384 bytes) was 24 bytes ago --- */
	struct hsu_dma_chan *      chan;                 /*   408     8 */
	short unsigned int         nr_channels;          /*   416     2 */

	/* size: 424, cachelines: 7, members: 3 */
	/* padding: 6 */
	/* last cacheline: 40 bytes */
};
struct hsu_dma_sg {
	dma_addr_t                 addr;                 /*     0     8 */
	unsigned int               len;                  /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct hsu_dma_desc {
	struct virt_dma_desc       vdesc;                /*     0   112 */
	/* --- cacheline 1 boundary (64 bytes) was 48 bytes ago --- */
	enum dma_transfer_direction direction;           /*   112     4 */

	/* XXX 4 bytes hole, try to pack */

	struct hsu_dma_sg *        sg;                   /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	unsigned int               nents;                /*   128     4 */

	/* XXX 4 bytes hole, try to pack */

	size_t                     length;               /*   136     8 */
	unsigned int               active;               /*   144     4 */
	enum dma_status            status;               /*   148     4 */

	/* size: 152, cachelines: 3, members: 7 */
	/* sum members: 144, holes: 2, sum holes: 8 */
	/* last cacheline: 24 bytes */
};
struct hsu_dma_chan {
	struct virt_dma_chan       vchan;                /*     0   264 */
	/* --- cacheline 4 boundary (256 bytes) was 8 bytes ago --- */
	void *                     reg;                  /*   264     8 */
	enum dma_transfer_direction direction;           /*   272     4 */

	/* XXX 4 bytes hole, try to pack */

	struct dma_slave_config    config;               /*   280    56 */
	/* --- cacheline 5 boundary (320 bytes) was 16 bytes ago --- */
	struct hsu_dma_desc *      desc;                 /*   336     8 */

	/* size: 344, cachelines: 6, members: 5 */
	/* sum members: 340, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct virtio_driver {
	struct device_driver       driver;               /*     0   192 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	const struct virtio_device_id  * id_table;       /*   192     8 */
	const unsigned int  *      feature_table;        /*   200     8 */
	unsigned int               feature_table_size;   /*   208     4 */

	/* XXX 4 bytes hole, try to pack */

	const unsigned int  *      feature_table_legacy; /*   216     8 */
	unsigned int               feature_table_size_legacy; /*   224     4 */

	/* XXX 4 bytes hole, try to pack */

	int                        (*validate)(struct virtio_device *); /*   232     8 */
	int                        (*probe)(struct virtio_device *); /*   240     8 */
	void                       (*scan)(struct virtio_device *); /*   248     8 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	void                       (*remove)(struct virtio_device *); /*   256     8 */
	void                       (*config_changed)(struct virtio_device *); /*   264     8 */
	int                        (*freeze)(struct virtio_device *); /*   272     8 */
	int                        (*restore)(struct virtio_device *); /*   280     8 */

	/* size: 288, cachelines: 5, members: 13 */
	/* sum members: 280, holes: 2, sum holes: 8 */
	/* last cacheline: 32 bytes */
};
struct vring_desc {
	__virtio64                 addr;                 /*     0     8 */
	__virtio32                 len;                  /*     8     4 */
	__virtio16                 flags;                /*    12     2 */
	__virtio16                 next;                 /*    14     2 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct vring_avail {
	__virtio16                 flags;                /*     0     2 */
	__virtio16                 idx;                  /*     2     2 */
	__virtio16                 ring[];               /*     4     0 */

	/* size: 4, cachelines: 1, members: 3 */
	/* last cacheline: 4 bytes */
};
struct vring_used_elem {
	__virtio32                 id;                   /*     0     4 */
	__virtio32                 len;                  /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct vring_used {
	__virtio16                 flags;                /*     0     2 */
	__virtio16                 idx;                  /*     2     2 */
	vring_used_elem_t          ring[] __attribute__((__aligned__(4))); /*     4     0 */

	/* size: 4, cachelines: 1, members: 3 */
	/* forced alignments: 1 */
	/* last cacheline: 4 bytes */
} __attribute__((__aligned__(4)));
struct vring {
	unsigned int               num;                  /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	vring_desc_t *             desc;                 /*     8     8 */
	vring_avail_t *            avail;                /*    16     8 */
	vring_used_t *             used;                 /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct vring_packed_desc_event {
	__le16                     off_wrap;             /*     0     2 */
	__le16                     flags;                /*     2     2 */

	/* size: 4, cachelines: 1, members: 2 */
	/* last cacheline: 4 bytes */
};
struct vring_packed_desc {
	__le64                     addr;                 /*     0     8 */
	__le32                     len;                  /*     8     4 */
	__le16                     id;                   /*    12     2 */
	__le16                     flags;                /*    14     2 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct vring_desc_state_split {
	void *                     data;                 /*     0     8 */
	struct vring_desc *        indir_desc;           /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct vring_desc_state_packed {
	void *                     data;                 /*     0     8 */
	struct vring_packed_desc * indir_desc;           /*     8     8 */
	u16                        num;                  /*    16     2 */
	u16                        next;                 /*    18     2 */
	u16                        last;                 /*    20     2 */

	/* size: 24, cachelines: 1, members: 5 */
	/* padding: 2 */
	/* last cacheline: 24 bytes */
};
struct vring_desc_extra_packed {
	dma_addr_t                 addr;                 /*     0     8 */
	u32                        len;                  /*     8     4 */
	u16                        flags;                /*    12     2 */

	/* size: 16, cachelines: 1, members: 3 */
	/* padding: 2 */
	/* last cacheline: 16 bytes */
};
struct vring_virtqueue {
	struct virtqueue           vq;                   /*     0    56 */
	bool                       packed_ring;          /*    56     1 */
	bool                       use_dma_api;          /*    57     1 */
	bool                       weak_barriers;        /*    58     1 */
	bool                       broken;               /*    59     1 */
	bool                       indirect;             /*    60     1 */
	bool                       event;                /*    61     1 */

	/* XXX 2 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	unsigned int               free_head;            /*    64     4 */
	unsigned int               num_added;            /*    68     4 */
	u16                        last_used_idx;        /*    72     2 */

	/* XXX 6 bytes hole, try to pack */

	union {
		struct {
			struct vring vring;              /*    80    32 */
			u16        avail_flags_shadow;   /*   112     2 */
			u16        avail_idx_shadow;     /*   114     2 */

			/* XXX 4 bytes hole, try to pack */

			struct vring_desc_state_split * desc_state; /*   120     8 */
			/* --- cacheline 2 boundary (128 bytes) --- */
			dma_addr_t queue_dma_addr;       /*   128     8 */
			size_t     queue_size_in_bytes;  /*   136     8 */
		} split;                                 /*    80    64 */
		struct {
			struct {
				unsigned int num;        /*    80     4 */

				/* XXX 4 bytes hole, try to pack */

				struct vring_packed_desc * desc; /*    88     8 */
				struct vring_packed_desc_event * driver; /*    96     8 */
				struct vring_packed_desc_event * device; /*   104     8 */
			} vring;                         /*    80    32 */
			bool       avail_wrap_counter;   /*   112     1 */
			bool       used_wrap_counter;    /*   113     1 */
			u16        avail_used_flags;     /*   114     2 */
			u16        next_avail_idx;       /*   116     2 */
			u16        event_flags_shadow;   /*   118     2 */
			struct vring_desc_state_packed * desc_state; /*   120     8 */
			/* --- cacheline 2 boundary (128 bytes) --- */
			struct vring_desc_extra_packed * desc_extra; /*   128     8 */
			dma_addr_t ring_dma_addr;        /*   136     8 */
			dma_addr_t driver_event_dma_addr; /*   144     8 */
			dma_addr_t device_event_dma_addr; /*   152     8 */
			size_t     ring_size_in_bytes;   /*   160     8 */
			size_t     event_size_in_bytes;  /*   168     8 */
		} packed;                                /*    80    96 */
	};                                               /*    80    96 */
	/* --- cacheline 2 boundary (128 bytes) was 48 bytes ago --- */
	bool                       (*notify)(struct virtqueue *); /*   176     8 */
	bool                       we_own_ring;          /*   184     1 */

	/* size: 192, cachelines: 3, members: 13 */
	/* sum members: 177, holes: 2, sum holes: 8 */
	/* padding: 7 */
};
struct virtio_pci_common_cfg {
	__le32                     device_feature_select; /*     0     4 */
	__le32                     device_feature;       /*     4     4 */
	__le32                     guest_feature_select; /*     8     4 */
	__le32                     guest_feature;        /*    12     4 */
	__le16                     msix_config;          /*    16     2 */
	__le16                     num_queues;           /*    18     2 */
	__u8                       device_status;        /*    20     1 */
	__u8                       config_generation;    /*    21     1 */
	__le16                     queue_select;         /*    22     2 */
	__le16                     queue_size;           /*    24     2 */
	__le16                     queue_msix_vector;    /*    26     2 */
	__le16                     queue_enable;         /*    28     2 */
	__le16                     queue_notify_off;     /*    30     2 */
	__le32                     queue_desc_lo;        /*    32     4 */
	__le32                     queue_desc_hi;        /*    36     4 */
	__le32                     queue_avail_lo;       /*    40     4 */
	__le32                     queue_avail_hi;       /*    44     4 */
	__le32                     queue_used_lo;        /*    48     4 */
	__le32                     queue_used_hi;        /*    52     4 */

	/* size: 56, cachelines: 1, members: 19 */
	/* last cacheline: 56 bytes */
};
struct virtio_pci_modern_device {
	struct pci_dev *           pci_dev;              /*     0     8 */
	struct virtio_pci_common_cfg * common;           /*     8     8 */
	void *                     device;               /*    16     8 */
	void *                     notify_base;          /*    24     8 */
	resource_size_t            notify_pa;            /*    32     8 */
	u8 *                       isr;                  /*    40     8 */
	size_t                     notify_len;           /*    48     8 */
	size_t                     device_len;           /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        notify_map_cap;       /*    64     4 */
	u32                        notify_offset_multiplier; /*    68     4 */
	int                        modern_bars;          /*    72     4 */
	struct virtio_device_id    id;                   /*    76     8 */

	/* size: 88, cachelines: 2, members: 12 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct virtio_pci_vq_info {
	struct virtqueue *         vq;                   /*     0     8 */
	struct list_head           node;                 /*     8    16 */
	unsigned int               msix_vector;          /*    24     4 */

	/* size: 32, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct virtio_pci_device {
	struct virtio_device       vdev __attribute__((__aligned__(8))); /*     0  1072 */
	/* --- cacheline 16 boundary (1024 bytes) was 48 bytes ago --- */
	struct pci_dev *           pci_dev;              /*  1072     8 */
	struct virtio_pci_modern_device mdev;            /*  1080    88 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 18 boundary (1152 bytes) was 16 bytes ago --- */
	u8 *                       isr;                  /*  1168     8 */
	void *                     ioaddr;               /*  1176     8 */
	spinlock_t                 lock;                 /*  1184     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           virtqueues;           /*  1192    16 */
	struct virtio_pci_vq_info * * vqs;               /*  1208     8 */
	/* --- cacheline 19 boundary (1216 bytes) --- */
	int                        msix_enabled;         /*  1216     4 */
	int                        intx_enabled;         /*  1220     4 */
	cpumask_var_t *            msix_affinity_masks;  /*  1224     8 */
	char *                     msix_names;           /*  1232     8 */
	unsigned int               msix_vectors;         /*  1240     4 */
	unsigned int               msix_used_vectors;    /*  1244     4 */
	bool                       per_vq_vectors;       /*  1248     1 */

	/* XXX 7 bytes hole, try to pack */

	struct virtqueue *         (*setup_vq)(struct virtio_pci_device *, struct virtio_pci_vq_info *, unsigned int, void (*)(struct virtqueue *), const char  *, bool, u16); /*  1256     8 */
	void                       (*del_vq)(struct virtio_pci_vq_info *); /*  1264     8 */
	u16                        (*config_vector)(struct virtio_pci_device *, u16); /*  1272     8 */

	/* size: 1280, cachelines: 20, members: 18 */
	/* sum members: 1269, holes: 2, sum holes: 11 */
	/* paddings: 1, sum paddings: 4 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(8)));
struct xsd_errors {
	int                        errnum;               /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	const char  *              errstring;            /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct xenbus_watch {
	struct list_head           list;                 /*     0    16 */
	const char  *              node;                 /*    16     8 */
	unsigned int               nr_pending;           /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	bool                       (*will_handle)(struct xenbus_watch *, const char  *, const char  *); /*    32     8 */
	void                       (*callback)(struct xenbus_watch *, const char  *, const char  *); /*    40     8 */

	/* size: 48, cachelines: 1, members: 5 */
	/* sum members: 44, holes: 1, sum holes: 4 */
	/* last cacheline: 48 bytes */
};
struct xenbus_transaction {
	u32                        id;                   /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct grant_entry_v1 {
	uint16_t                   flags;                /*     0     2 */
	domid_t                    domid;                /*     2     2 */
	uint32_t                   frame;                /*     4     4 */

	/* size: 8, cachelines: 1, members: 3 */
	/* last cacheline: 8 bytes */
};
struct grant_entry_header {
	uint16_t                   flags;                /*     0     2 */
	domid_t                    domid;                /*     2     2 */

	/* size: 4, cachelines: 1, members: 2 */
	/* last cacheline: 4 bytes */
};
union grant_entry_v2 {
	struct grant_entry_header  hdr;                /*     0     4 */
	struct {
		struct grant_entry_header hdr;         /*     0     4 */
		uint32_t           pad0;               /*     4     4 */
		uint64_t           frame;              /*     8     8 */
	} full_page;                                   /*     0    16 */
	struct {
		struct grant_entry_header hdr;         /*     0     4 */
		uint16_t           page_off;           /*     4     2 */
		uint16_t           length;             /*     6     2 */
		uint64_t           frame;              /*     8     8 */
	} sub_page;                                    /*     0    16 */
	struct {
		struct grant_entry_header hdr;         /*     0     4 */
		domid_t            trans_domid;        /*     4     2 */
		uint16_t           pad0;               /*     6     2 */
		grant_ref_t        gref;               /*     8     4 */
	} transitive;                                  /*     0    12 */
	uint32_t                   __spacer[4];        /*     0    16 */
};
struct gnttab_map_grant_ref {
	uint64_t                   host_addr;            /*     0     8 */
	uint32_t                   flags;                /*     8     4 */
	grant_ref_t                ref;                  /*    12     4 */
	domid_t                    dom;                  /*    16     2 */
	int16_t                    status;               /*    18     2 */
	grant_handle_t             handle;               /*    20     4 */
	uint64_t                   dev_bus_addr;         /*    24     8 */

	/* size: 32, cachelines: 1, members: 7 */
	/* last cacheline: 32 bytes */
};
struct gnttab_unmap_grant_ref {
	uint64_t                   host_addr;            /*     0     8 */
	uint64_t                   dev_bus_addr;         /*     8     8 */
	grant_handle_t             handle;               /*    16     4 */
	int16_t                    status;               /*    20     2 */

	/* size: 24, cachelines: 1, members: 4 */
	/* padding: 2 */
	/* last cacheline: 24 bytes */
};
struct gnttab_setup_table {
	domid_t                    dom;                  /*     0     2 */

	/* XXX 2 bytes hole, try to pack */

	uint32_t                   nr_frames;            /*     4     4 */
	int16_t                    status;               /*     8     2 */

	/* XXX 6 bytes hole, try to pack */

	__guest_handle_xen_pfn_t   frame_list;           /*    16     8 */

	/* size: 24, cachelines: 1, members: 4 */
	/* sum members: 16, holes: 2, sum holes: 8 */
	/* last cacheline: 24 bytes */
};
struct gnttab_copy {
	struct {
		union {
			grant_ref_t ref;                 /*     0     4 */
			xen_pfn_t  gmfn;                 /*     0     8 */
		} u;                                     /*     0     8 */
		domid_t            domid;                /*     8     2 */
		uint16_t           offset;               /*    10     2 */
	} source;                                        /*     0    16 */

	/* XXX last struct has 4 bytes of padding */

	struct {
		union {
			grant_ref_t ref;                 /*    16     4 */
			xen_pfn_t  gmfn;                 /*    16     8 */
		} u;                                     /*    16     8 */
		domid_t            domid;                /*    24     2 */
		uint16_t           offset;               /*    26     2 */
	} dest;                                          /*    16    16 */

	/* XXX last struct has 4 bytes of padding */

	uint16_t                   len;                  /*    32     2 */
	uint16_t                   flags;                /*    34     2 */
	int16_t                    status;               /*    36     2 */

	/* size: 40, cachelines: 1, members: 5 */
	/* padding: 2 */
	/* paddings: 2, sum paddings: 8 */
	/* last cacheline: 40 bytes */
};
struct gnttab_query_size {
	domid_t                    dom;                  /*     0     2 */

	/* XXX 2 bytes hole, try to pack */

	uint32_t                   nr_frames;            /*     4     4 */
	uint32_t                   max_nr_frames;        /*     8     4 */
	int16_t                    status;               /*    12     2 */

	/* size: 16, cachelines: 1, members: 4 */
	/* sum members: 12, holes: 1, sum holes: 2 */
	/* padding: 2 */
	/* last cacheline: 16 bytes */
};
struct gnttab_set_version {
	uint32_t                   version;              /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct gnttab_get_status_frames {
	uint32_t                   nr_frames;            /*     0     4 */
	domid_t                    dom;                  /*     4     2 */
	int16_t                    status;               /*     6     2 */
	__guest_handle_uint64_t    frame_list;           /*     8     8 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct gnttab_free_callback {
	struct gnttab_free_callback * next;              /*     0     8 */
	void                       (*fn)(void *);        /*     8     8 */
	void *                     arg;                  /*    16     8 */
	u16                        count;                /*    24     2 */

	/* size: 32, cachelines: 1, members: 4 */
	/* padding: 6 */
	/* last cacheline: 32 bytes */
};
struct gntab_unmap_queue_data {
	struct delayed_work        gnttab_work;          /*     0   192 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	void *                     data;                 /*   192     8 */
	gnttab_unmap_refs_done     done;                 /*   200     8 */
	struct gnttab_unmap_grant_ref * unmap_ops;       /*   208     8 */
	struct gnttab_unmap_grant_ref * kunmap_ops;      /*   216     8 */
	struct page * *            pages;                /*   224     8 */
	unsigned int               count;                /*   232     4 */
	unsigned int               age;                  /*   236     4 */

	/* size: 240, cachelines: 4, members: 8 */
	/* last cacheline: 48 bytes */
};
struct grant_frames {
	xen_pfn_t *                pfn;                  /*     0     8 */
	unsigned int               count;                /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     vaddr;                /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct gnttab_page_cache {
	spinlock_t                 lock;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           pages;                /*     8    16 */
	unsigned int               num_pages;            /*    24     4 */

	/* size: 32, cachelines: 1, members: 3 */
	/* sum members: 24, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct xen_page_foreign {
	domid_t                    domid;                /*     0     2 */

	/* XXX 2 bytes hole, try to pack */

	grant_ref_t                gref;                 /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* sum members: 6, holes: 1, sum holes: 2 */
	/* last cacheline: 8 bytes */
};
struct gnttab_ops {
	unsigned int               version;              /*     0     4 */
	unsigned int               grefs_per_grant_frame; /*     4     4 */
	int                        (*map_frames)(xen_pfn_t *, unsigned int); /*     8     8 */
	void                       (*unmap_frames)(void); /*    16     8 */
	void                       (*update_entry)(grant_ref_t, domid_t, long unsigned int, unsigned int); /*    24     8 */
	int                        (*end_foreign_access_ref)(grant_ref_t, int); /*    32     8 */
	long unsigned int          (*end_foreign_transfer_ref)(grant_ref_t); /*    40     8 */
	long unsigned int          (*read_frame)(grant_ref_t); /*    48     8 */

	/* size: 56, cachelines: 1, members: 8 */
	/* last cacheline: 56 bytes */
};
struct unmap_refs_callback_data {
	struct completion          completion;           /*     0    32 */
	int                        result;               /*    32     4 */

	/* size: 40, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct deferred_entry {
	struct list_head           list;                 /*     0    16 */
	grant_ref_t                ref;                  /*    16     4 */
	bool                       ro;                   /*    20     1 */

	/* XXX 1 byte hole, try to pack */

	uint16_t                   warn_delay;           /*    22     2 */
	struct page *              page;                 /*    24     8 */

	/* size: 32, cachelines: 1, members: 5 */
	/* sum members: 31, holes: 1, sum holes: 1 */
	/* last cacheline: 32 bytes */
};
struct xen_feature_info {
	unsigned int               submap_idx;           /*     0     4 */
	uint32_t                   submap;               /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct balloon_stats {
	long unsigned int          current_pages;        /*     0     8 */
	long unsigned int          target_pages;         /*     8     8 */
	long unsigned int          target_unpopulated;   /*    16     8 */
	long unsigned int          balloon_low;          /*    24     8 */
	long unsigned int          balloon_high;         /*    32     8 */
	long unsigned int          total_pages;          /*    40     8 */
	long unsigned int          schedule_delay;       /*    48     8 */
	long unsigned int          max_schedule_delay;   /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	long unsigned int          retry_count;          /*    64     8 */
	long unsigned int          max_retry_count;      /*    72     8 */

	/* size: 80, cachelines: 2, members: 10 */
	/* last cacheline: 16 bytes */
};
struct suspend_info {
	int                        cancelled;            /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct shutdown_handler {
	const const char           command[11];          /*     0    11 */
	bool                       flag;                 /*    11     1 */

	/* XXX 4 bytes hole, try to pack */

	void                       (*cb)(void);          /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct vcpu_runstate_info {
	int                        state;                /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	uint64_t                   state_entry_time;     /*     8     8 */
	uint64_t                   time[4];              /*    16    32 */

	/* size: 48, cachelines: 1, members: 3 */
	/* sum members: 44, holes: 1, sum holes: 4 */
	/* last cacheline: 48 bytes */
};
struct vcpu_register_runstate_memory_area {
	union {
		__guest_handle_vcpu_runstate_info h;     /*     0     8 */
		struct vcpu_runstate_info * v;           /*     0     8 */
		uint64_t           p;                    /*     0     8 */
	} addr;                                          /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct xen_memory_reservation {
	__guest_handle_xen_pfn_t   extent_start;         /*     0     8 */
	xen_ulong_t                nr_extents;           /*     8     8 */
	unsigned int               extent_order;         /*    16     4 */
	unsigned int               address_bits;         /*    20     4 */
	domid_t                    domid;                /*    24     2 */

	/* size: 32, cachelines: 1, members: 5 */
	/* padding: 6 */
	/* last cacheline: 32 bytes */
};
struct evtchn_bind_interdomain {
	domid_t                    remote_dom;           /*     0     2 */

	/* XXX 2 bytes hole, try to pack */

	evtchn_port_t              remote_port;          /*     4     4 */
	evtchn_port_t              local_port;           /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* sum members: 10, holes: 1, sum holes: 2 */
	/* last cacheline: 12 bytes */
};
struct evtchn_bind_virq {
	uint32_t                   virq;                 /*     0     4 */
	uint32_t                   vcpu;                 /*     4     4 */
	evtchn_port_t              port;                 /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct evtchn_bind_pirq {
	uint32_t                   pirq;                 /*     0     4 */
	uint32_t                   flags;                /*     4     4 */
	evtchn_port_t              port;                 /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct evtchn_bind_ipi {
	uint32_t                   vcpu;                 /*     0     4 */
	evtchn_port_t              port;                 /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct evtchn_close {
	evtchn_port_t              port;                 /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct evtchn_send {
	evtchn_port_t              port;                 /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct evtchn_status {
	domid_t                    dom;                  /*     0     2 */

	/* XXX 2 bytes hole, try to pack */

	evtchn_port_t              port;                 /*     4     4 */
	uint32_t                   status;               /*     8     4 */
	uint32_t                   vcpu;                 /*    12     4 */
	union {
		struct {
			domid_t    dom;                  /*    16     2 */
		} unbound;                               /*    16     2 */
		struct {
			domid_t    dom;                  /*    16     2 */

			/* XXX 2 bytes hole, try to pack */

			evtchn_port_t port;              /*    20     4 */
		} interdomain;                           /*    16     8 */
		uint32_t           pirq;                 /*    16     4 */
		uint32_t           virq;                 /*    16     4 */
	} u;                                             /*    16     8 */

	/* size: 24, cachelines: 1, members: 5 */
	/* sum members: 22, holes: 1, sum holes: 2 */
	/* last cacheline: 24 bytes */
};
struct evtchn_bind_vcpu {
	evtchn_port_t              port;                 /*     0     4 */
	uint32_t                   vcpu;                 /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct evtchn_set_priority {
	evtchn_port_t              port;                 /*     0     4 */
	uint32_t                   priority;             /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct sched_poll {
	__guest_handle_evtchn_port_t ports;              /*     0     8 */
	unsigned int               nr_ports;             /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	uint64_t                   timeout;              /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct physdev_eoi {
	uint32_t                   irq;                  /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct physdev_pirq_eoi_gmfn {
	xen_ulong_t                gmfn;                 /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct physdev_irq_status_query {
	uint32_t                   irq;                  /*     0     4 */
	uint32_t                   flags;                /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct physdev_irq {
	uint32_t                   irq;                  /*     0     4 */
	uint32_t                   vector;               /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct physdev_map_pirq {
	domid_t                    domid;                /*     0     2 */

	/* XXX 2 bytes hole, try to pack */

	int                        type;                 /*     4     4 */
	int                        index;                /*     8     4 */
	int                        pirq;                 /*    12     4 */
	int                        bus;                  /*    16     4 */
	int                        devfn;                /*    20     4 */
	int                        entry_nr;             /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	uint64_t                   table_base;           /*    32     8 */

	/* size: 40, cachelines: 1, members: 8 */
	/* sum members: 34, holes: 2, sum holes: 6 */
	/* last cacheline: 40 bytes */
};
struct physdev_unmap_pirq {
	domid_t                    domid;                /*     0     2 */

	/* XXX 2 bytes hole, try to pack */

	int                        pirq;                 /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* sum members: 6, holes: 1, sum holes: 2 */
	/* last cacheline: 8 bytes */
};
struct physdev_get_free_pirq {
	int                        type;                 /*     0     4 */
	uint32_t                   pirq;                 /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct evtchn_ops {
	unsigned int               (*max_channels)(void); /*     0     8 */
	unsigned int               (*nr_channels)(void); /*     8     8 */
	int                        (*setup)(evtchn_port_t); /*    16     8 */
	void                       (*remove)(evtchn_port_t, unsigned int); /*    24     8 */
	void                       (*bind_to_cpu)(evtchn_port_t, unsigned int, unsigned int); /*    32     8 */
	void                       (*clear_pending)(evtchn_port_t); /*    40     8 */
	void                       (*set_pending)(evtchn_port_t); /*    48     8 */
	bool                       (*is_pending)(evtchn_port_t); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	void                       (*mask)(evtchn_port_t); /*    64     8 */
	void                       (*unmask)(evtchn_port_t); /*    72     8 */
	void                       (*handle_events)(unsigned int, struct evtchn_loop_ctrl *); /*    80     8 */
	void                       (*resume)(void);      /*    88     8 */
	int                        (*percpu_init)(unsigned int); /*    96     8 */
	int                        (*percpu_deinit)(unsigned int); /*   104     8 */

	/* size: 112, cachelines: 2, members: 14 */
	/* last cacheline: 48 bytes */
};
struct evtchn_loop_ctrl {
	ktime_t                    timeout;              /*     0     8 */
	unsigned int               count;                /*     8     4 */
	bool                       defer_eoi;            /*    12     1 */

	/* size: 16, cachelines: 1, members: 3 */
	/* padding: 3 */
	/* last cacheline: 16 bytes */
};
struct irq_info {
	struct list_head           list;                 /*     0    16 */
	struct list_head           eoi_list;             /*    16    16 */
	short int                  refcnt;               /*    32     2 */
	short int                  spurious_cnt;         /*    34     2 */
	short int                  type;                 /*    36     2 */
	u8                         mask_reason;          /*    38     1 */
	u8                         is_active;            /*    39     1 */
	unsigned int               irq;                  /*    40     4 */
	evtchn_port_t              evtchn;               /*    44     4 */
	short unsigned int         cpu;                  /*    48     2 */
	short unsigned int         eoi_cpu;              /*    50     2 */
	unsigned int               irq_epoch;            /*    52     4 */
	u64                        eoi_time;             /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	raw_spinlock_t             lock;                 /*    64     4 */
	union {
		short unsigned int virq;                 /*    68     2 */
		enum ipi_vector    ipi;                  /*    68     4 */
		struct {
			short unsigned int pirq;         /*    68     2 */
			short unsigned int gsi;          /*    70     2 */
			unsigned char vector;            /*    72     1 */
			unsigned char flags;             /*    73     1 */
			uint16_t   domid;                /*    74     2 */
		} pirq;                                  /*    68     8 */
	} u;                                             /*    68     8 */

	/* size: 80, cachelines: 2, members: 15 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct lateeoi_work {
	struct delayed_work        delayed;              /*     0   192 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	spinlock_t                 eoi_list_lock;        /*   192     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           eoi_list;             /*   200    16 */

	/* size: 216, cachelines: 4, members: 3 */
	/* sum members: 212, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct evtchn_unmask {
	evtchn_port_t              port;                 /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct evtchn_init_control {
	uint64_t                   control_gfn;          /*     0     8 */
	uint32_t                   offset;               /*     8     4 */
	uint32_t                   vcpu;                 /*    12     4 */
	uint8_t                    link_bits;            /*    16     1 */
	uint8_t                    _pad[7];              /*    17     7 */

	/* size: 24, cachelines: 1, members: 5 */
	/* last cacheline: 24 bytes */
};
struct evtchn_expand_array {
	uint64_t                   array_gfn;            /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct evtchn_fifo_control_block {
	uint32_t                   ready;                /*     0     4 */
	uint32_t                   _rsvd;                /*     4     4 */
	event_word_t               head[16];             /*     8    64 */

	/* size: 72, cachelines: 2, members: 3 */
	/* last cacheline: 8 bytes */
};
struct evtchn_fifo_queue {
	uint32_t                   head[16];             /*     0    64 */

	/* size: 64, cachelines: 1, members: 1 */
};
struct evtchn_alloc_unbound {
	domid_t                    dom;                  /*     0     2 */
	domid_t                    remote_dom;           /*     2     2 */
	evtchn_port_t              port;                 /*     4     4 */

	/* size: 8, cachelines: 1, members: 3 */
	/* last cacheline: 8 bytes */
};
struct xenbus_device {
	const char  *              devicetype;           /*     0     8 */
	const char  *              nodename;             /*     8     8 */
	const char  *              otherend;             /*    16     8 */
	int                        otherend_id;          /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	struct xenbus_watch        otherend_watch;       /*    32    48 */
	/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
	struct device              dev __attribute__((__aligned__(8))); /*    80  1000 */
	/* --- cacheline 16 boundary (1024 bytes) was 56 bytes ago --- */
	enum xenbus_state          state;                /*  1080     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 17 boundary (1088 bytes) --- */
	struct completion          down;                 /*  1088    32 */
	struct work_struct         work;                 /*  1120    64 */
	/* --- cacheline 18 boundary (1152 bytes) was 32 bytes ago --- */
	struct semaphore           reclaim_sem;          /*  1184    24 */

	/* size: 1208, cachelines: 19, members: 10 */
	/* sum members: 1200, holes: 2, sum holes: 8 */
	/* forced alignments: 1 */
	/* last cacheline: 56 bytes */
} __attribute__((__aligned__(8)));
struct xenbus_map_node {
	struct list_head           next;                 /*     0    16 */
	union {
		struct {
			struct vm_struct * area;         /*    16     8 */
		} pv;                                    /*    16     8 */
		struct {
			struct page * pages[16];         /*    16   128 */
			/* --- cacheline 2 boundary (128 bytes) was 16 bytes ago --- */
			long unsigned int addrs[16];     /*   144   128 */
			/* --- cacheline 4 boundary (256 bytes) was 16 bytes ago --- */
			void *     addr;                 /*   272     8 */
		} hvm;                                   /*    16   264 */
	};                                               /*    16   264 */
	/* --- cacheline 4 boundary (256 bytes) was 24 bytes ago --- */
	grant_handle_t             handles[16];          /*   280    64 */
	/* --- cacheline 5 boundary (320 bytes) was 24 bytes ago --- */
	unsigned int               nr_handles;           /*   344     4 */

	/* size: 352, cachelines: 6, members: 4 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct map_ring_valloc {
	struct xenbus_map_node *   node;                 /*     0     8 */
	long unsigned int          addrs[16];            /*     8   128 */
	/* --- cacheline 2 boundary (128 bytes) was 8 bytes ago --- */
	phys_addr_t                phys_addrs[16];       /*   136   128 */
	/* --- cacheline 4 boundary (256 bytes) was 8 bytes ago --- */
	struct gnttab_map_grant_ref map[16];             /*   264   512 */
	/* --- cacheline 12 boundary (768 bytes) was 8 bytes ago --- */
	struct gnttab_unmap_grant_ref unmap[16];         /*   776   384 */
	/* --- cacheline 18 boundary (1152 bytes) was 8 bytes ago --- */
	unsigned int               idx;                  /*  1160     4 */

	/* size: 1168, cachelines: 19, members: 6 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct xenbus_ring_ops {
	int                        (*map)(struct xenbus_device *, struct map_ring_valloc *, grant_ref_t *, unsigned int, void * *); /*     0     8 */
	int                        (*unmap)(struct xenbus_device *, void *); /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct unmap_ring_hvm {
	unsigned int               idx;                  /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          addrs[16];            /*     8   128 */

	/* size: 136, cachelines: 3, members: 2 */
	/* sum members: 132, holes: 1, sum holes: 4 */
	/* last cacheline: 8 bytes */
};
struct xsd_sockmsg {
	uint32_t                   type;                 /*     0     4 */
	uint32_t                   req_id;               /*     4     4 */
	uint32_t                   tx_id;                /*     8     4 */
	uint32_t                   len;                  /*    12     4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct xenstore_domain_interface {
	char                       req[1024];            /*     0  1024 */
	/* --- cacheline 16 boundary (1024 bytes) --- */
	char                       rsp[1024];            /*  1024  1024 */
	/* --- cacheline 32 boundary (2048 bytes) --- */
	XENSTORE_RING_IDX          req_cons;             /*  2048     4 */
	XENSTORE_RING_IDX          req_prod;             /*  2052     4 */
	XENSTORE_RING_IDX          rsp_cons;             /*  2056     4 */
	XENSTORE_RING_IDX          rsp_prod;             /*  2060     4 */

	/* size: 2064, cachelines: 33, members: 6 */
	/* last cacheline: 16 bytes */
};
struct xs_watch_event {
	struct list_head           list;                 /*     0    16 */
	unsigned int               len;                  /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	struct xenbus_watch *      handle;               /*    24     8 */
	const char  *              path;                 /*    32     8 */
	const char  *              token;                /*    40     8 */
	char                       body[];               /*    48     0 */

	/* size: 48, cachelines: 1, members: 6 */
	/* sum members: 44, holes: 1, sum holes: 4 */
	/* last cacheline: 48 bytes */
};
struct xb_req_data {
	struct list_head           list;                 /*     0    16 */
	wait_queue_head_t          wq;                   /*    16    24 */
	struct xsd_sockmsg         msg;                  /*    40    16 */
	uint32_t                   caller_req_id;        /*    56     4 */
	enum xsd_sockmsg_type      type;                 /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	char *                     body;                 /*    64     8 */
	const struct kvec  *       vec;                  /*    72     8 */
	int                        num_vecs;             /*    80     4 */
	int                        err;                  /*    84     4 */
	enum xb_req_state          state;                /*    88     4 */
	bool                       user_req;             /*    92     1 */

	/* XXX 3 bytes hole, try to pack */

	void                       (*cb)(struct xb_req_data *); /*    96     8 */
	void *                     par;                  /*   104     8 */

	/* size: 112, cachelines: 2, members: 13 */
	/* sum members: 109, holes: 1, sum holes: 3 */
	/* last cacheline: 48 bytes */
};
struct xenbus_device_id {
	char                       devicetype[32];       /*     0    32 */

	/* size: 32, cachelines: 1, members: 1 */
	/* last cacheline: 32 bytes */
};
struct xenbus_driver {
	const char  *              name;                 /*     0     8 */
	const struct xenbus_device_id  * ids;            /*     8     8 */
	bool                       allow_rebind;         /*    16     1 */

	/* XXX 7 bytes hole, try to pack */

	int                        (*probe)(struct xenbus_device *, const struct xenbus_device_id  *); /*    24     8 */
	void                       (*otherend_changed)(struct xenbus_device *, enum xenbus_state); /*    32     8 */
	int                        (*remove)(struct xenbus_device *); /*    40     8 */
	int                        (*suspend)(struct xenbus_device *); /*    48     8 */
	int                        (*resume)(struct xenbus_device *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*uevent)(struct xenbus_device *, struct kobj_uevent_env *); /*    64     8 */
	struct device_driver       driver;               /*    72   192 */
	/* --- cacheline 4 boundary (256 bytes) was 8 bytes ago --- */
	int                        (*read_otherend_details)(struct xenbus_device *); /*   264     8 */
	int                        (*is_ready)(struct xenbus_device *); /*   272     8 */
	void                       (*reclaim_memory)(struct xenbus_device *); /*   280     8 */

	/* size: 288, cachelines: 5, members: 13 */
	/* sum members: 281, holes: 1, sum holes: 7 */
	/* last cacheline: 32 bytes */
};
struct xen_hvm_param {
	domid_t                    domid;                /*     0     2 */

	/* XXX 2 bytes hole, try to pack */

	uint32_t                   index;                /*     4     4 */
	uint64_t                   value;                /*     8     8 */

	/* size: 16, cachelines: 1, members: 3 */
	/* sum members: 14, holes: 1, sum holes: 2 */
	/* last cacheline: 16 bytes */
};
struct xen_bus_type {
	char *                     root;                 /*     0     8 */
	unsigned int               levels;               /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	int                        (*get_bus_id)(char *, const char  *); /*    16     8 */
	int                        (*probe)(struct xen_bus_type *, const char  *, const char  *); /*    24     8 */
	bool                       (*otherend_will_handle)(struct xenbus_watch *, const char  *, const char  *); /*    32     8 */
	void                       (*otherend_changed)(struct xenbus_watch *, const char  *, const char  *); /*    40     8 */
	struct bus_type            bus;                  /*    48   176 */

	/* XXX last struct has 7 bytes of padding */

	/* size: 224, cachelines: 4, members: 7 */
	/* sum members: 220, holes: 1, sum holes: 4 */
	/* paddings: 1, sum paddings: 7 */
	/* last cacheline: 32 bytes */
};
struct xb_find_info {
	struct xenbus_device *     dev;                  /*     0     8 */
	const char  *              nodename;             /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct xenbus_transaction_holder {
	struct list_head           list;                 /*     0    16 */
	struct xenbus_transaction  handle;               /*    16     4 */
	unsigned int               generation_id;        /*    20     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct read_buffer {
	struct list_head           list;                 /*     0    16 */
	unsigned int               cons;                 /*    16     4 */
	unsigned int               len;                  /*    20     4 */
	char                       msg[];                /*    24     0 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct xenbus_file_priv {
	struct mutex               msgbuffer_mutex;      /*     0    32 */
	struct list_head           transactions;         /*    32    16 */
	struct list_head           watches;              /*    48    16 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	unsigned int               len;                  /*    64     4 */
	union {
		struct xsd_sockmsg msg;                  /*    68    16 */
		char               buffer[4096];         /*    68  4096 */
	} u;                                             /*    68  4096 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 65 boundary (4160 bytes) was 8 bytes ago --- */
	struct mutex               reply_mutex;          /*  4168    32 */
	struct list_head           read_buffers;         /*  4200    16 */
	wait_queue_head_t          read_waitq;           /*  4216    24 */
	/* --- cacheline 66 boundary (4224 bytes) was 16 bytes ago --- */
	struct kref                kref;                 /*  4240     4 */

	/* XXX 4 bytes hole, try to pack */

	struct work_struct         wq;                   /*  4248    64 */

	/* size: 4312, cachelines: 68, members: 10 */
	/* sum members: 4304, holes: 2, sum holes: 8 */
	/* last cacheline: 24 bytes */
};
struct watch_adapter {
	struct list_head           list;                 /*     0    16 */
	struct xenbus_watch        watch;                /*    16    48 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct xenbus_file_priv *  dev_data;             /*    64     8 */
	char *                     token;                /*    72     8 */

	/* size: 80, cachelines: 2, members: 4 */
	/* last cacheline: 16 bytes */
};
struct xen_compile_info {
	char                       compiler[64];         /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	char                       compile_by[16];       /*    64    16 */
	char                       compile_domain[32];   /*    80    32 */
	char                       compile_date[32];     /*   112    32 */

	/* size: 144, cachelines: 3, members: 4 */
	/* last cacheline: 16 bytes */
};
struct xen_platform_parameters {
	xen_ulong_t                virt_start;           /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct xen_build_id {
	uint32_t                   len;                  /*     0     4 */
	unsigned char              buf[];                /*     4     0 */

	/* size: 4, cachelines: 1, members: 2 */
	/* last cacheline: 4 bytes */
};
struct hyp_sysfs_attr {
	struct attribute           attr;                 /*     0    16 */

	/* XXX last struct has 6 bytes of padding */

	ssize_t                    (*show)(struct hyp_sysfs_attr *, char *); /*    16     8 */
	ssize_t                    (*store)(struct hyp_sysfs_attr *, const char  *, size_t); /*    24     8 */
	void *                     hyp_attr_data;        /*    32     8 */

	/* size: 40, cachelines: 1, members: 4 */
	/* paddings: 1, sum paddings: 6 */
	/* last cacheline: 40 bytes */
};
struct xen_add_to_physmap_range {
	domid_t                    domid;                /*     0     2 */
	uint16_t                   space;                /*     2     2 */
	uint16_t                   size;                 /*     4     2 */
	domid_t                    foreign_domid;        /*     6     2 */
	__guest_handle_xen_ulong_t idxs;                 /*     8     8 */
	__guest_handle_xen_pfn_t   gpfns;                /*    16     8 */
	__guest_handle_int         errs;                 /*    24     8 */

	/* size: 32, cachelines: 1, members: 7 */
	/* last cacheline: 32 bytes */
};
struct xen_remove_from_physmap {
	domid_t                    domid;                /*     0     2 */

	/* XXX 6 bytes hole, try to pack */

	xen_pfn_t                  gpfn;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 10, holes: 1, sum holes: 6 */
	/* last cacheline: 16 bytes */
};
struct remap_data {
	xen_pfn_t *                fgfn;                 /*     0     8 */
	int                        nr_fgfn;              /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	pgprot_t                   prot;                 /*    16     8 */
	domid_t                    domid;                /*    24     2 */

	/* XXX 6 bytes hole, try to pack */

	struct vm_area_struct *    vma;                  /*    32     8 */
	int                        index;                /*    40     4 */

	/* XXX 4 bytes hole, try to pack */

	struct page * *            pages;                /*    48     8 */
	struct xen_remap_gfn_info * info;                /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int *                      err_ptr;              /*    64     8 */
	int                        mapped;               /*    72     4 */
	int                        h_errs[1];            /*    76     4 */
	xen_ulong_t                h_idxs[1];            /*    80     8 */
	xen_pfn_t                  h_gpfns[1];           /*    88     8 */
	int                        h_iter;               /*    96     4 */

	/* size: 104, cachelines: 2, members: 14 */
	/* sum members: 86, holes: 3, sum holes: 14 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct map_balloon_pages {
	xen_pfn_t *                pfns;                 /*     0     8 */
	unsigned int               idx;                  /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct remap_pfn {
	struct mm_struct *         mm;                   /*     0     8 */
	struct page * *            pages;                /*     8     8 */
	pgprot_t                   prot;                 /*    16     8 */
	long unsigned int          i;                    /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct serial_struct32 {
	compat_int_t               type;                 /*     0     4 */
	compat_int_t               line;                 /*     4     4 */
	compat_uint_t              port;                 /*     8     4 */
	compat_int_t               irq;                  /*    12     4 */
	compat_int_t               flags;                /*    16     4 */
	compat_int_t               xmit_fifo_size;       /*    20     4 */
	compat_int_t               custom_divisor;       /*    24     4 */
	compat_int_t               baud_base;            /*    28     4 */
	short unsigned int         close_delay;          /*    32     2 */
	char                       io_type;              /*    34     1 */
	char                       reserved_char;        /*    35     1 */
	compat_int_t               hub6;                 /*    36     4 */
	short unsigned int         closing_wait;         /*    40     2 */
	short unsigned int         closing_wait2;        /*    42     2 */
	compat_uint_t              iomem_base;           /*    44     4 */
	short unsigned int         iomem_reg_shift;      /*    48     2 */

	/* XXX 2 bytes hole, try to pack */

	unsigned int               port_high;            /*    52     4 */
	compat_int_t               reserved;             /*    56     4 */

	/* size: 60, cachelines: 1, members: 18 */
	/* sum members: 58, holes: 1, sum holes: 2 */
	/* last cacheline: 60 bytes */
};
struct n_tty_data {
	size_t                     read_head;            /*     0     8 */
	size_t                     commit_head;          /*     8     8 */
	size_t                     canon_head;           /*    16     8 */
	size_t                     echo_head;            /*    24     8 */
	size_t                     echo_commit;          /*    32     8 */
	size_t                     echo_mark;            /*    40     8 */
	long unsigned int          char_map[4];          /*    48    32 */
	/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
	long unsigned int          overrun_time;         /*    80     8 */
	int                        num_overrun;          /*    88     4 */
	bool                       no_room;              /*    92     1 */
	unsigned char              lnext:1;              /*    93: 0  1 */
	unsigned char              erasing:1;            /*    93: 1  1 */
	unsigned char              raw:1;                /*    93: 2  1 */
	unsigned char              real_raw:1;           /*    93: 3  1 */
	unsigned char              icanon:1;             /*    93: 4  1 */
	unsigned char              push:1;               /*    93: 5  1 */

	/* XXX 2 bits hole, try to pack */

	char                       read_buf[4096];       /*    94  4096 */

	/* XXX 2 bytes hole, try to pack */

	/* --- cacheline 65 boundary (4160 bytes) was 32 bytes ago --- */
	long unsigned int          read_flags[64];       /*  4192   512 */
	/* --- cacheline 73 boundary (4672 bytes) was 32 bytes ago --- */
	unsigned char              echo_buf[4096];       /*  4704  4096 */
	/* --- cacheline 137 boundary (8768 bytes) was 32 bytes ago --- */
	size_t                     read_tail;            /*  8800     8 */
	size_t                     line_start;           /*  8808     8 */
	unsigned int               column;               /*  8816     4 */
	unsigned int               canon_column;         /*  8820     4 */
	size_t                     echo_tail;            /*  8824     8 */
	/* --- cacheline 138 boundary (8832 bytes) --- */
	struct mutex               atomic_read_lock;     /*  8832    32 */
	struct mutex               output_lock;          /*  8864    32 */

	/* size: 8896, cachelines: 139, members: 26 */
	/* sum members: 8893, holes: 1, sum holes: 2 */
	/* sum bitfield members: 6 bits, bit holes: 1, sum bit holes: 2 bits */
};
struct termios {
	tcflag_t                   c_iflag;              /*     0     4 */
	tcflag_t                   c_oflag;              /*     4     4 */
	tcflag_t                   c_cflag;              /*     8     4 */
	tcflag_t                   c_lflag;              /*    12     4 */
	cc_t                       c_line;               /*    16     1 */
	cc_t                       c_cc[19];             /*    17    19 */

	/* size: 36, cachelines: 1, members: 6 */
	/* last cacheline: 36 bytes */
};
struct termios2 {
	tcflag_t                   c_iflag;              /*     0     4 */
	tcflag_t                   c_oflag;              /*     4     4 */
	tcflag_t                   c_cflag;              /*     8     4 */
	tcflag_t                   c_lflag;              /*    12     4 */
	cc_t                       c_line;               /*    16     1 */
	cc_t                       c_cc[19];             /*    17    19 */
	speed_t                    c_ispeed;             /*    36     4 */
	speed_t                    c_ospeed;             /*    40     4 */

	/* size: 44, cachelines: 1, members: 8 */
	/* last cacheline: 44 bytes */
};
struct termio {
	short unsigned int         c_iflag;              /*     0     2 */
	short unsigned int         c_oflag;              /*     2     2 */
	short unsigned int         c_cflag;              /*     4     2 */
	short unsigned int         c_lflag;              /*     6     2 */
	unsigned char              c_line;               /*     8     1 */
	unsigned char              c_cc[8];              /*     9     8 */

	/* size: 18, cachelines: 1, members: 6 */
	/* padding: 1 */
	/* last cacheline: 18 bytes */
};
struct ldsem_waiter {
	struct list_head           list;                 /*     0    16 */
	struct task_struct *       task;                 /*    16     8 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct tty_audit_buf {
	struct mutex               mutex;                /*     0    32 */
	dev_t                      dev;                  /*    32     4 */
	unsigned int               icanon:1;             /*    36: 0  4 */

	/* XXX 31 bits hole, try to pack */

	size_t                     valid;                /*    40     8 */
	unsigned char *            data;                 /*    48     8 */

	/* size: 56, cachelines: 1, members: 5 */
	/* sum members: 52 */
	/* sum bitfield members: 1 bits, bit holes: 1, sum bit holes: 31 bits */
	/* last cacheline: 56 bytes */
};
struct sysrq_state {
	struct input_handle        handle;               /*     0    72 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	struct work_struct         reinject_work;        /*    72    64 */
	/* --- cacheline 2 boundary (128 bytes) was 8 bytes ago --- */
	long unsigned int          key_down[12];         /*   136    96 */
	/* --- cacheline 3 boundary (192 bytes) was 40 bytes ago --- */
	unsigned int               alt;                  /*   232     4 */
	unsigned int               alt_use;              /*   236     4 */
	unsigned int               shift;                /*   240     4 */
	unsigned int               shift_use;            /*   244     4 */
	bool                       active;               /*   248     1 */
	bool                       need_reinject;        /*   249     1 */
	bool                       reinjecting;          /*   250     1 */
	bool                       reset_canceled;       /*   251     1 */
	bool                       reset_requested;      /*   252     1 */

	/* XXX 3 bytes hole, try to pack */

	/* --- cacheline 4 boundary (256 bytes) --- */
	long unsigned int          reset_keybit[12];     /*   256    96 */
	/* --- cacheline 5 boundary (320 bytes) was 32 bytes ago --- */
	int                        reset_seq_len;        /*   352     4 */
	int                        reset_seq_cnt;        /*   356     4 */
	int                        reset_seq_version;    /*   360     4 */

	/* XXX 4 bytes hole, try to pack */

	struct timer_list          keyreset_timer;       /*   368    72 */

	/* size: 440, cachelines: 7, members: 17 */
	/* sum members: 433, holes: 2, sum holes: 7 */
	/* last cacheline: 56 bytes */
};
struct unipair {
	short unsigned int         unicode;              /*     0     2 */
	short unsigned int         fontpos;              /*     2     2 */

	/* size: 4, cachelines: 1, members: 2 */
	/* last cacheline: 4 bytes */
};
struct unimapdesc {
	short unsigned int         entry_ct;             /*     0     2 */

	/* XXX 6 bytes hole, try to pack */

	struct unipair *           entries;              /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 10, holes: 1, sum holes: 6 */
	/* last cacheline: 16 bytes */
};
struct kbd_repeat {
	int                        delay;                /*     0     4 */
	int                        period;               /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct console_font_op {
	unsigned int               op;                   /*     0     4 */
	unsigned int               flags;                /*     4     4 */
	unsigned int               width;                /*     8     4 */
	unsigned int               height;               /*    12     4 */
	unsigned int               charcount;            /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	unsigned char *            data;                 /*    24     8 */

	/* size: 32, cachelines: 1, members: 6 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct vt_stat {
	short unsigned int         v_active;             /*     0     2 */
	short unsigned int         v_signal;             /*     2     2 */
	short unsigned int         v_state;              /*     4     2 */

	/* size: 6, cachelines: 1, members: 3 */
	/* last cacheline: 6 bytes */
};
struct vt_sizes {
	short unsigned int         v_rows;               /*     0     2 */
	short unsigned int         v_cols;               /*     2     2 */
	short unsigned int         v_scrollsize;         /*     4     2 */

	/* size: 6, cachelines: 1, members: 3 */
	/* last cacheline: 6 bytes */
};
struct vt_consize {
	short unsigned int         v_rows;               /*     0     2 */
	short unsigned int         v_cols;               /*     2     2 */
	short unsigned int         v_vlin;               /*     4     2 */
	short unsigned int         v_clin;               /*     6     2 */
	short unsigned int         v_vcol;               /*     8     2 */
	short unsigned int         v_ccol;               /*    10     2 */

	/* size: 12, cachelines: 1, members: 6 */
	/* last cacheline: 12 bytes */
};
struct vt_event {
	unsigned int               event;                /*     0     4 */
	unsigned int               oldev;                /*     4     4 */
	unsigned int               newev;                /*     8     4 */
	unsigned int               pad[4];               /*    12    16 */

	/* size: 28, cachelines: 1, members: 4 */
	/* last cacheline: 28 bytes */
};
struct vt_setactivate {
	unsigned int               console;              /*     0     4 */
	struct vt_mode             mode;                 /*     4     8 */

	/* size: 12, cachelines: 1, members: 2 */
	/* last cacheline: 12 bytes */
};
struct vt_spawn_console {
	spinlock_t                 lock;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct pid *               pid;                  /*     8     8 */
	int                        sig;                  /*    16     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 16, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct vt_event_wait {
	struct list_head           list;                 /*     0    16 */
	struct vt_event            event;                /*    16    28 */
	int                        done;                 /*    44     4 */

	/* size: 48, cachelines: 1, members: 3 */
	/* last cacheline: 48 bytes */
};
struct compat_console_font_op {
	compat_uint_t              op;                   /*     0     4 */
	compat_uint_t              flags;                /*     4     4 */
	compat_uint_t              width;                /*     8     4 */
	compat_uint_t              height;               /*    12     4 */
	compat_uint_t              charcount;            /*    16     4 */
	compat_caddr_t             data;                 /*    20     4 */

	/* size: 24, cachelines: 1, members: 6 */
	/* last cacheline: 24 bytes */
};
struct compat_unimapdesc {
	short unsigned int         entry_ct;             /*     0     2 */

	/* XXX 2 bytes hole, try to pack */

	compat_caddr_t             entries;              /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* sum members: 6, holes: 1, sum holes: 2 */
	/* last cacheline: 8 bytes */
};
struct vt_notifier_param {
	struct vc_data *           vc;                   /*     0     8 */
	unsigned int               c;                    /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct vcs_poll_data {
	struct notifier_block      notifier;             /*     0    24 */

	/* XXX last struct has 4 bytes of padding */

	unsigned int               cons_num;             /*    24     4 */
	int                        event;                /*    28     4 */
	wait_queue_head_t          waitq;                /*    32    24 */
	struct fasync_struct *     fasync;               /*    56     8 */

	/* size: 64, cachelines: 1, members: 5 */
	/* paddings: 1, sum paddings: 4 */
};
struct tiocl_selection {
	short unsigned int         xs;                   /*     0     2 */
	short unsigned int         ys;                   /*     2     2 */
	short unsigned int         xe;                   /*     4     2 */
	short unsigned int         ye;                   /*     6     2 */
	short unsigned int         sel_mode;             /*     8     2 */

	/* size: 10, cachelines: 1, members: 5 */
	/* last cacheline: 10 bytes */
};
struct vc_selection {
	struct mutex               lock;                 /*     0    32 */
	struct vc_data *           cons;                 /*    32     8 */
	char *                     buffer;               /*    40     8 */
	unsigned int               buf_len;              /*    48     4 */
	volatile int               start;                /*    52     4 */
	int                        end;                  /*    56     4 */

	/* size: 64, cachelines: 1, members: 6 */
	/* padding: 4 */
};
struct keyboard_notifier_param {
	struct vc_data *           vc;                   /*     0     8 */
	int                        down;                 /*     8     4 */
	int                        shift;                /*    12     4 */
	int                        ledstate;             /*    16     4 */
	unsigned int               value;                /*    20     4 */

	/* size: 24, cachelines: 1, members: 5 */
	/* last cacheline: 24 bytes */
};
struct kbd_struct {
	unsigned char              lockstate;            /*     0     1 */
	unsigned char              slockstate;           /*     1     1 */
	unsigned char              ledmode:1;            /*     2: 0  1 */
	unsigned char              ledflagstate:4;       /*     2: 1  1 */

	/* XXX 3 bits hole, try to pack */

	/* Force alignment to the next boundary: */
	unsigned char              :0;

	unsigned char              default_ledflagstate:4; /*     3: 0  1 */
	unsigned char              kbdmode:3;            /*     3: 4  1 */

	/* XXX 1 bit hole, try to pack */

	/* Force alignment to the next boundary: */
	unsigned char              :0;

	unsigned char              modeflags:5;          /*     4: 0  1 */

	/* size: 5, cachelines: 1, members: 7 */
	/* sum members: 2 */
	/* sum bitfield members: 17 bits, bit holes: 2, sum bit holes: 4 bits */
	/* bit_padding: 3 bits */
	/* last cacheline: 5 bytes */
};
struct kbentry {
	unsigned char              kb_table;             /*     0     1 */
	unsigned char              kb_index;             /*     1     1 */
	short unsigned int         kb_value;             /*     2     2 */

	/* size: 4, cachelines: 1, members: 3 */
	/* last cacheline: 4 bytes */
};
struct kbsentry {
	unsigned char              kb_func;              /*     0     1 */
	unsigned char              kb_string[512];       /*     1   512 */

	/* size: 513, cachelines: 9, members: 2 */
	/* last cacheline: 1 bytes */
};
struct kbdiacr {
	unsigned char              diacr;                /*     0     1 */
	unsigned char              base;                 /*     1     1 */
	unsigned char              result;               /*     2     1 */

	/* size: 3, cachelines: 1, members: 3 */
	/* last cacheline: 3 bytes */
};
struct kbdiacrs {
	unsigned int               kb_cnt;               /*     0     4 */
	struct kbdiacr             kbdiacr[256];         /*     4   768 */

	/* size: 772, cachelines: 13, members: 2 */
	/* last cacheline: 4 bytes */
};
struct kbdiacruc {
	unsigned int               diacr;                /*     0     4 */
	unsigned int               base;                 /*     4     4 */
	unsigned int               result;               /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct kbdiacrsuc {
	unsigned int               kb_cnt;               /*     0     4 */
	struct kbdiacruc           kbdiacruc[256];       /*     4  3072 */

	/* size: 3076, cachelines: 49, members: 2 */
	/* last cacheline: 4 bytes */
};
struct kbkeycode {
	unsigned int               scancode;             /*     0     4 */
	unsigned int               keycode;              /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct getset_keycode_data {
	struct input_keymap_entry  ke;                   /*     0    40 */
	int                        error;                /*    40     4 */

	/* size: 44, cachelines: 1, members: 2 */
	/* last cacheline: 44 bytes */
};
struct kbd_led_trigger {
	struct led_trigger         trigger;              /*     0    80 */
	/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
	unsigned int               mask;                 /*    80     4 */

	/* size: 88, cachelines: 2, members: 2 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct uni_pagedir {
	u16 * *                    uni_pgdir[32];        /*     0   256 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	long unsigned int          refcount;             /*   256     8 */
	long unsigned int          sum;                  /*   264     8 */
	unsigned char *            inverse_translations[4]; /*   272    32 */
	u16 *                      inverse_trans_unicode; /*   304     8 */

	/* size: 312, cachelines: 5, members: 5 */
	/* last cacheline: 56 bytes */
};
struct uni_screen {
	char32_t *                 lines[0];             /*     0     0 */

	/* size: 0, cachelines: 0, members: 1 */
};
struct con_driver {
	const struct consw  *      con;                  /*     0     8 */
	const char  *              desc;                 /*     8     8 */
	struct device *            dev;                  /*    16     8 */
	int                        node;                 /*    24     4 */
	int                        first;                /*    28     4 */
	int                        last;                 /*    32     4 */
	int                        flag;                 /*    36     4 */

	/* size: 40, cachelines: 1, members: 7 */
	/* last cacheline: 40 bytes */
};
struct rgb {
	u8                         r;                    /*     0     1 */
	u8                         g;                    /*     1     1 */
	u8                         b;                    /*     2     1 */

	/* size: 3, cachelines: 1, members: 3 */
	/* last cacheline: 3 bytes */
};
struct interval {
	uint32_t                   first;                /*     0     4 */
	uint32_t                   last;                 /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct vc_draw_region {
	long unsigned int          from;                 /*     0     8 */
	long unsigned int          to;                   /*     8     8 */
	int                        x;                    /*    16     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct hvc_struct {
	struct tty_port            port;                 /*     0   384 */
	/* --- cacheline 6 boundary (384 bytes) --- */
	spinlock_t                 lock;                 /*   384     4 */
	int                        index;                /*   388     4 */
	int                        do_wakeup;            /*   392     4 */

	/* XXX 4 bytes hole, try to pack */

	char *                     outbuf;               /*   400     8 */
	int                        outbuf_size;          /*   408     4 */
	int                        n_outbuf;             /*   412     4 */
	uint32_t                   vtermno;              /*   416     4 */

	/* XXX 4 bytes hole, try to pack */

	const struct hv_ops  *     ops;                  /*   424     8 */
	int                        irq_requested;        /*   432     4 */
	int                        data;                 /*   436     4 */
	struct winsize             ws;                   /*   440     8 */
	/* --- cacheline 7 boundary (448 bytes) --- */
	struct work_struct         tty_resize;           /*   448    64 */
	/* --- cacheline 8 boundary (512 bytes) --- */
	struct list_head           next;                 /*   512    16 */
	long unsigned int          flags;                /*   528     8 */

	/* size: 536, cachelines: 9, members: 15 */
	/* sum members: 528, holes: 2, sum holes: 8 */
	/* last cacheline: 24 bytes */
};
struct hv_ops {
	int                        (*get_chars)(uint32_t, char *, int); /*     0     8 */
	int                        (*put_chars)(uint32_t, const char  *, int); /*     8     8 */
	int                        (*flush)(uint32_t, bool); /*    16     8 */
	int                        (*notifier_add)(struct hvc_struct *, int); /*    24     8 */
	void                       (*notifier_del)(struct hvc_struct *, int); /*    32     8 */
	void                       (*notifier_hangup)(struct hvc_struct *, int); /*    40     8 */
	int                        (*tiocmget)(struct hvc_struct *); /*    48     8 */
	int                        (*tiocmset)(struct hvc_struct *, unsigned int, unsigned int); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	void                       (*dtr_rts)(struct hvc_struct *, int); /*    64     8 */

	/* size: 72, cachelines: 2, members: 9 */
	/* last cacheline: 8 bytes */
};
struct circ_buf {
	char *                     buf;                  /*     0     8 */
	int                        head;                 /*     8     4 */
	int                        tail;                 /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct serial_rs485 {
	__u32                      flags;                /*     0     4 */
	__u32                      delay_rts_before_send; /*     4     4 */
	__u32                      delay_rts_after_send; /*     8     4 */
	__u32                      padding[5];           /*    12    20 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct serial_iso7816 {
	__u32                      flags;                /*     0     4 */
	__u32                      tg;                   /*     4     4 */
	__u32                      sc_fi;                /*     8     4 */
	__u32                      sc_di;                /*    12     4 */
	__u32                      clk;                  /*    16     4 */
	__u32                      reserved[5];          /*    20    20 */

	/* size: 40, cachelines: 1, members: 6 */
	/* last cacheline: 40 bytes */
};
struct uart_ops {
	unsigned int               (*tx_empty)(struct uart_port *); /*     0     8 */
	void                       (*set_mctrl)(struct uart_port *, unsigned int); /*     8     8 */
	unsigned int               (*get_mctrl)(struct uart_port *); /*    16     8 */
	void                       (*stop_tx)(struct uart_port *); /*    24     8 */
	void                       (*start_tx)(struct uart_port *); /*    32     8 */
	void                       (*throttle)(struct uart_port *); /*    40     8 */
	void                       (*unthrottle)(struct uart_port *); /*    48     8 */
	void                       (*send_xchar)(struct uart_port *, char); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	void                       (*stop_rx)(struct uart_port *); /*    64     8 */
	void                       (*enable_ms)(struct uart_port *); /*    72     8 */
	void                       (*break_ctl)(struct uart_port *, int); /*    80     8 */
	int                        (*startup)(struct uart_port *); /*    88     8 */
	void                       (*shutdown)(struct uart_port *); /*    96     8 */
	void                       (*flush_buffer)(struct uart_port *); /*   104     8 */
	void                       (*set_termios)(struct uart_port *, struct ktermios *, struct ktermios *); /*   112     8 */
	void                       (*set_ldisc)(struct uart_port *, struct ktermios *); /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	void                       (*pm)(struct uart_port *, unsigned int, unsigned int); /*   128     8 */
	const char  *              (*type)(struct uart_port *); /*   136     8 */
	void                       (*release_port)(struct uart_port *); /*   144     8 */
	int                        (*request_port)(struct uart_port *); /*   152     8 */
	void                       (*config_port)(struct uart_port *, int); /*   160     8 */
	int                        (*verify_port)(struct uart_port *, struct serial_struct *); /*   168     8 */
	int                        (*ioctl)(struct uart_port *, unsigned int, long unsigned int); /*   176     8 */
	int                        (*poll_init)(struct uart_port *); /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	void                       (*poll_put_char)(struct uart_port *, unsigned char); /*   192     8 */
	int                        (*poll_get_char)(struct uart_port *); /*   200     8 */

	/* size: 208, cachelines: 4, members: 26 */
	/* last cacheline: 16 bytes */
};
struct uart_port {
	spinlock_t                 lock;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          iobase;               /*     8     8 */
	unsigned char *            membase;              /*    16     8 */
	unsigned int               (*serial_in)(struct uart_port *, int); /*    24     8 */
	void                       (*serial_out)(struct uart_port *, int, int); /*    32     8 */
	void                       (*set_termios)(struct uart_port *, struct ktermios *, struct ktermios *); /*    40     8 */
	void                       (*set_ldisc)(struct uart_port *, struct ktermios *); /*    48     8 */
	unsigned int               (*get_mctrl)(struct uart_port *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	void                       (*set_mctrl)(struct uart_port *, unsigned int); /*    64     8 */
	unsigned int               (*get_divisor)(struct uart_port *, unsigned int, unsigned int *); /*    72     8 */
	void                       (*set_divisor)(struct uart_port *, unsigned int, unsigned int, unsigned int); /*    80     8 */
	int                        (*startup)(struct uart_port *); /*    88     8 */
	void                       (*shutdown)(struct uart_port *); /*    96     8 */
	void                       (*throttle)(struct uart_port *); /*   104     8 */
	void                       (*unthrottle)(struct uart_port *); /*   112     8 */
	int                        (*handle_irq)(struct uart_port *); /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	void                       (*pm)(struct uart_port *, unsigned int, unsigned int); /*   128     8 */
	void                       (*handle_break)(struct uart_port *); /*   136     8 */
	int                        (*rs485_config)(struct uart_port *, struct serial_rs485 *); /*   144     8 */
	int                        (*iso7816_config)(struct uart_port *, struct serial_iso7816 *); /*   152     8 */
	unsigned int               irq;                  /*   160     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          irqflags;             /*   168     8 */
	unsigned int               uartclk;              /*   176     4 */
	unsigned int               fifosize;             /*   180     4 */
	unsigned char              x_char;               /*   184     1 */
	unsigned char              regshift;             /*   185     1 */
	unsigned char              iotype;               /*   186     1 */
	unsigned char              quirks;               /*   187     1 */
	unsigned int               read_status_mask;     /*   188     4 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	unsigned int               ignore_status_mask;   /*   192     4 */

	/* XXX 4 bytes hole, try to pack */

	struct uart_state *        state;                /*   200     8 */
	struct uart_icount         icount;               /*   208    44 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 4 boundary (256 bytes) --- */
	struct console *           cons;                 /*   256     8 */
	upf_t                      flags;                /*   264     4 */
	upstat_t                   status;               /*   268     4 */
	int                        hw_stopped;           /*   272     4 */
	unsigned int               mctrl;                /*   276     4 */
	unsigned int               timeout;              /*   280     4 */
	unsigned int               type;                 /*   284     4 */
	const struct uart_ops  *   ops;                  /*   288     8 */
	unsigned int               custom_divisor;       /*   296     4 */
	unsigned int               line;                 /*   300     4 */
	unsigned int               minor;                /*   304     4 */

	/* XXX 4 bytes hole, try to pack */

	resource_size_t            mapbase;              /*   312     8 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	resource_size_t            mapsize;              /*   320     8 */
	struct device *            dev;                  /*   328     8 */
	long unsigned int          sysrq;                /*   336     8 */
	unsigned int               sysrq_ch;             /*   344     4 */
	unsigned char              has_sysrq;            /*   348     1 */
	unsigned char              sysrq_seq;            /*   349     1 */
	unsigned char              hub6;                 /*   350     1 */
	unsigned char              suspended;            /*   351     1 */
	unsigned char              console_reinit;       /*   352     1 */

	/* XXX 7 bytes hole, try to pack */

	const char  *              name;                 /*   360     8 */
	struct attribute_group *   attr_group;           /*   368     8 */
	const struct attribute_group  * * tty_groups;    /*   376     8 */
	/* --- cacheline 6 boundary (384 bytes) --- */
	struct serial_rs485        rs485;                /*   384    32 */
	struct gpio_desc *         rs485_term_gpio;      /*   416     8 */
	struct serial_iso7816      iso7816;              /*   424    40 */
	/* --- cacheline 7 boundary (448 bytes) was 16 bytes ago --- */
	void *                     private_data;         /*   464     8 */

	/* size: 472, cachelines: 8, members: 60 */
	/* sum members: 445, holes: 6, sum holes: 27 */
	/* last cacheline: 24 bytes */
};
struct uart_icount {
	__u32                      cts;                  /*     0     4 */
	__u32                      dsr;                  /*     4     4 */
	__u32                      rng;                  /*     8     4 */
	__u32                      dcd;                  /*    12     4 */
	__u32                      rx;                   /*    16     4 */
	__u32                      tx;                   /*    20     4 */
	__u32                      frame;                /*    24     4 */
	__u32                      overrun;              /*    28     4 */
	__u32                      parity;               /*    32     4 */
	__u32                      brk;                  /*    36     4 */
	__u32                      buf_overrun;          /*    40     4 */

	/* size: 44, cachelines: 1, members: 11 */
	/* last cacheline: 44 bytes */
};
struct uart_state {
	struct tty_port            port;                 /*     0   384 */
	/* --- cacheline 6 boundary (384 bytes) --- */
	enum uart_pm_state         pm_state;             /*   384     4 */

	/* XXX 4 bytes hole, try to pack */

	struct circ_buf            xmit;                 /*   392    16 */
	atomic_t                   refcount;             /*   408     4 */

	/* XXX 4 bytes hole, try to pack */

	wait_queue_head_t          remove_wait;          /*   416    24 */
	struct uart_port *         uart_port;            /*   440     8 */

	/* size: 448, cachelines: 7, members: 6 */
	/* sum members: 440, holes: 2, sum holes: 8 */
};
struct earlycon_device {
	struct console *           con;                  /*     0     8 */
	struct uart_port           port;                 /*     8   472 */
	/* --- cacheline 7 boundary (448 bytes) was 32 bytes ago --- */
	char                       options[16];          /*   480    16 */
	unsigned int               baud;                 /*   496     4 */

	/* size: 504, cachelines: 8, members: 4 */
	/* padding: 4 */
	/* last cacheline: 56 bytes */
};
struct earlycon_id {
	char                       name[15];             /*     0    15 */
	char                       name_term;            /*    15     1 */
	char                       compatible[128];      /*    16   128 */
	/* --- cacheline 2 boundary (128 bytes) was 16 bytes ago --- */
	int                        (*setup)(struct earlycon_device *, const char  *); /*   144     8 */

	/* size: 152, cachelines: 3, members: 4 */
	/* last cacheline: 24 bytes */
};
struct xencons_interface {
	char                       in[1024];             /*     0  1024 */
	/* --- cacheline 16 boundary (1024 bytes) --- */
	char                       out[2048];            /*  1024  2048 */
	/* --- cacheline 48 boundary (3072 bytes) --- */
	XENCONS_RING_IDX           in_cons;              /*  3072     4 */
	XENCONS_RING_IDX           in_prod;              /*  3076     4 */
	XENCONS_RING_IDX           out_cons;             /*  3080     4 */
	XENCONS_RING_IDX           out_prod;             /*  3084     4 */

	/* size: 3088, cachelines: 49, members: 6 */
	/* last cacheline: 16 bytes */
};
struct xencons_info {
	struct list_head           list;                 /*     0    16 */
	struct xenbus_device *     xbdev;                /*    16     8 */
	struct xencons_interface * intf;                 /*    24     8 */
	unsigned int               evtchn;               /*    32     4 */
	XENCONS_RING_IDX           out_cons;             /*    36     4 */
	unsigned int               out_cons_same;        /*    40     4 */

	/* XXX 4 bytes hole, try to pack */

	struct hvc_struct *        hvc;                  /*    48     8 */
	int                        irq;                  /*    56     4 */
	int                        vtermno;              /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	grant_ref_t                gntref;               /*    64     4 */

	/* size: 72, cachelines: 2, members: 10 */
	/* sum members: 64, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 8 bytes */
};
struct uart_driver {
	struct module *            owner;                /*     0     8 */
	const char  *              driver_name;          /*     8     8 */
	const char  *              dev_name;             /*    16     8 */
	int                        major;                /*    24     4 */
	int                        minor;                /*    28     4 */
	int                        nr;                   /*    32     4 */

	/* XXX 4 bytes hole, try to pack */

	struct console *           cons;                 /*    40     8 */
	struct uart_state *        state;                /*    48     8 */
	struct tty_driver *        tty_driver;           /*    56     8 */

	/* size: 64, cachelines: 1, members: 9 */
	/* sum members: 60, holes: 1, sum holes: 4 */
};
struct uart_match {
	struct uart_port *         port;                 /*     0     8 */
	struct uart_driver *       driver;               /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct plat_serial8250_port {
	long unsigned int          iobase;               /*     0     8 */
	void *                     membase;              /*     8     8 */
	resource_size_t            mapbase;              /*    16     8 */
	unsigned int               irq;                  /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          irqflags;             /*    32     8 */
	unsigned int               uartclk;              /*    40     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     private_data;         /*    48     8 */
	unsigned char              regshift;             /*    56     1 */
	unsigned char              iotype;               /*    57     1 */
	unsigned char              hub6;                 /*    58     1 */
	unsigned char              has_sysrq;            /*    59     1 */
	upf_t                      flags;                /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	unsigned int               type;                 /*    64     4 */

	/* XXX 4 bytes hole, try to pack */

	unsigned int               (*serial_in)(struct uart_port *, int); /*    72     8 */
	void                       (*serial_out)(struct uart_port *, int, int); /*    80     8 */
	void                       (*set_termios)(struct uart_port *, struct ktermios *, struct ktermios *); /*    88     8 */
	void                       (*set_ldisc)(struct uart_port *, struct ktermios *); /*    96     8 */
	unsigned int               (*get_mctrl)(struct uart_port *); /*   104     8 */
	int                        (*handle_irq)(struct uart_port *); /*   112     8 */
	void                       (*pm)(struct uart_port *, unsigned int, unsigned int); /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	void                       (*handle_break)(struct uart_port *); /*   128     8 */

	/* size: 136, cachelines: 3, members: 21 */
	/* sum members: 124, holes: 3, sum holes: 12 */
	/* last cacheline: 8 bytes */
};
struct uart_8250_ops {
	int                        (*setup_irq)(struct uart_8250_port *); /*     0     8 */
	void                       (*release_irq)(struct uart_8250_port *); /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct uart_8250_port {
	struct uart_port           port;                 /*     0   472 */
	/* --- cacheline 7 boundary (448 bytes) was 24 bytes ago --- */
	struct timer_list          timer;                /*   472    72 */
	/* --- cacheline 8 boundary (512 bytes) was 32 bytes ago --- */
	struct list_head           list;                 /*   544    16 */
	u32                        capabilities;         /*   560     4 */
	short unsigned int         bugs;                 /*   564     2 */
	bool                       fifo_bug;             /*   566     1 */

	/* XXX 1 byte hole, try to pack */

	unsigned int               tx_loadsz;            /*   568     4 */
	unsigned char              acr;                  /*   572     1 */
	unsigned char              fcr;                  /*   573     1 */
	unsigned char              ier;                  /*   574     1 */
	unsigned char              lcr;                  /*   575     1 */
	/* --- cacheline 9 boundary (576 bytes) --- */
	unsigned char              mcr;                  /*   576     1 */
	unsigned char              mcr_mask;             /*   577     1 */
	unsigned char              mcr_force;            /*   578     1 */
	unsigned char              cur_iotype;           /*   579     1 */
	unsigned int               rpm_tx_active;        /*   580     4 */
	unsigned char              canary;               /*   584     1 */
	unsigned char              probe;                /*   585     1 */

	/* XXX 6 bytes hole, try to pack */

	struct mctrl_gpios *       gpios;                /*   592     8 */
	unsigned char              lsr_saved_flags;      /*   600     1 */
	unsigned char              msr_saved_flags;      /*   601     1 */

	/* XXX 6 bytes hole, try to pack */

	struct uart_8250_dma *     dma;                  /*   608     8 */
	const struct uart_8250_ops  * ops;               /*   616     8 */
	int                        (*dl_read)(struct uart_8250_port *); /*   624     8 */
	void                       (*dl_write)(struct uart_8250_port *, int); /*   632     8 */
	/* --- cacheline 10 boundary (640 bytes) --- */
	struct uart_8250_em485 *   em485;                /*   640     8 */
	void                       (*rs485_start_tx)(struct uart_8250_port *); /*   648     8 */
	void                       (*rs485_stop_tx)(struct uart_8250_port *); /*   656     8 */
	struct delayed_work        overrun_backoff;      /*   664   192 */
	/* --- cacheline 13 boundary (832 bytes) was 24 bytes ago --- */
	u32                        overrun_backoff_time_ms; /*   856     4 */

	/* size: 864, cachelines: 14, members: 30 */
	/* sum members: 847, holes: 3, sum holes: 13 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct uart_8250_em485 {
	struct hrtimer             start_tx_timer __attribute__((__aligned__(8))); /*     0    96 */
	/* --- cacheline 1 boundary (64 bytes) was 32 bytes ago --- */
	struct hrtimer             stop_tx_timer __attribute__((__aligned__(8))); /*    96    96 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	struct hrtimer *           active_timer;         /*   192     8 */
	struct uart_8250_port *    port;                 /*   200     8 */
	unsigned int               tx_stopped:1;         /*   208: 0  4 */

	/* size: 216, cachelines: 4, members: 5 */
	/* padding: 4 */
	/* bit_padding: 31 bits */
	/* forced alignments: 2 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct uart_8250_dma {
	int                        (*tx_dma)(struct uart_8250_port *); /*     0     8 */
	int                        (*rx_dma)(struct uart_8250_port *); /*     8     8 */
	dma_filter_fn              fn;                   /*    16     8 */
	void *                     rx_param;             /*    24     8 */
	void *                     tx_param;             /*    32     8 */
	struct dma_slave_config    rxconf;               /*    40    56 */
	/* --- cacheline 1 boundary (64 bytes) was 32 bytes ago --- */
	struct dma_slave_config    txconf;               /*    96    56 */
	/* --- cacheline 2 boundary (128 bytes) was 24 bytes ago --- */
	struct dma_chan *          rxchan;               /*   152     8 */
	struct dma_chan *          txchan;               /*   160     8 */
	phys_addr_t                rx_dma_addr;          /*   168     8 */
	phys_addr_t                tx_dma_addr;          /*   176     8 */
	dma_addr_t                 rx_addr;              /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	dma_addr_t                 tx_addr;              /*   192     8 */
	dma_cookie_t               rx_cookie;            /*   200     4 */
	dma_cookie_t               tx_cookie;            /*   204     4 */
	void *                     rx_buf;               /*   208     8 */
	size_t                     rx_size;              /*   216     8 */
	size_t                     tx_size;              /*   224     8 */
	unsigned char              tx_running;           /*   232     1 */
	unsigned char              tx_err;               /*   233     1 */
	unsigned char              rx_running;           /*   234     1 */

	/* size: 240, cachelines: 4, members: 21 */
	/* padding: 5 */
	/* last cacheline: 48 bytes */
};
struct old_serial_port {
	unsigned int               uart;                 /*     0     4 */
	unsigned int               baud_base;            /*     4     4 */
	unsigned int               port;                 /*     8     4 */
	unsigned int               irq;                  /*    12     4 */
	upf_t                      flags;                /*    16     4 */
	unsigned char              io_type;              /*    20     1 */

	/* XXX 3 bytes hole, try to pack */

	unsigned char *            iomem_base;           /*    24     8 */
	short unsigned int         iomem_reg_shift;      /*    32     2 */

	/* size: 40, cachelines: 1, members: 8 */
	/* sum members: 31, holes: 1, sum holes: 3 */
	/* padding: 6 */
	/* last cacheline: 40 bytes */
};
struct irq_info {
	struct hlist_node          node;                 /*     0    16 */
	int                        irq;                  /*    16     4 */
	spinlock_t                 lock;                 /*    20     4 */
	struct list_head *         head;                 /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct serial8250_config {
	const char  *              name;                 /*     0     8 */
	short unsigned int         fifo_size;            /*     8     2 */
	short unsigned int         tx_loadsz;            /*    10     2 */
	unsigned char              fcr;                  /*    12     1 */
	unsigned char              rxtrig_bytes[4];      /*    13     4 */

	/* XXX 3 bytes hole, try to pack */

	unsigned int               flags;                /*    20     4 */

	/* size: 24, cachelines: 1, members: 6 */
	/* sum members: 21, holes: 1, sum holes: 3 */
	/* last cacheline: 24 bytes */
};
struct dw8250_port_data {
	int                        line;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct uart_8250_dma       dma;                  /*     8   240 */

	/* XXX last struct has 5 bytes of padding */

	/* --- cacheline 3 boundary (192 bytes) was 56 bytes ago --- */
	u8                         dlf_size;             /*   248     1 */

	/* size: 256, cachelines: 4, members: 3 */
	/* sum members: 245, holes: 1, sum holes: 4 */
	/* padding: 7 */
	/* paddings: 1, sum paddings: 5 */
};
struct pciserial_board {
	unsigned int               flags;                /*     0     4 */
	unsigned int               num_ports;            /*     4     4 */
	unsigned int               base_baud;            /*     8     4 */
	unsigned int               uart_offset;          /*    12     4 */
	unsigned int               reg_shift;            /*    16     4 */
	unsigned int               first_offset;         /*    20     4 */

	/* size: 24, cachelines: 1, members: 6 */
	/* last cacheline: 24 bytes */
};
struct pci_serial_quirk {
	u32                        vendor;               /*     0     4 */
	u32                        device;               /*     4     4 */
	u32                        subvendor;            /*     8     4 */
	u32                        subdevice;            /*    12     4 */
	int                        (*probe)(struct pci_dev *); /*    16     8 */
	int                        (*init)(struct pci_dev *); /*    24     8 */
	int                        (*setup)(struct serial_private *, const struct pciserial_board  *, struct uart_8250_port *, int); /*    32     8 */
	void                       (*exit)(struct pci_dev *); /*    40     8 */

	/* size: 48, cachelines: 1, members: 8 */
	/* last cacheline: 48 bytes */
};
struct serial_private {
	struct pci_dev *           dev;                  /*     0     8 */
	unsigned int               nr;                   /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct pci_serial_quirk *  quirk;                /*    16     8 */
	const struct pciserial_board  * board;           /*    24     8 */
	int                        line[];               /*    32     0 */

	/* size: 32, cachelines: 1, members: 5 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct f815xxa_data {
	spinlock_t                 lock;                 /*     0     4 */
	int                        idx;                  /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct timedia_struct {
	int                        num;                  /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	const short unsigned int  * ids;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct quatech_feature {
	u16                        devid;                /*     0     2 */
	bool                       amcc;                 /*     2     1 */

	/* size: 4, cachelines: 1, members: 2 */
	/* padding: 1 */
	/* last cacheline: 4 bytes */
};
struct exar8250_platform {
	int                        (*rs485_config)(struct uart_port *, struct serial_rs485 *); /*     0     8 */
	int                        (*register_gpio)(struct pci_dev *, struct uart_8250_port *); /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct exar8250_board {
	unsigned int               num_ports;            /*     0     4 */
	unsigned int               reg_shift;            /*     4     4 */
	int                        (*setup)(struct exar8250 *, struct pci_dev *, struct uart_8250_port *, int); /*     8     8 */
	void                       (*exit)(struct pci_dev *); /*    16     8 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct exar8250 {
	unsigned int               nr;                   /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct exar8250_board *    board;                /*     8     8 */
	void *                     virt;                 /*    16     8 */
	int                        line[];               /*    24     0 */

	/* size: 24, cachelines: 1, members: 4 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct dw8250_data {
	struct dw8250_port_data    data;                 /*     0   256 */

	/* XXX last struct has 7 bytes of padding */

	/* --- cacheline 4 boundary (256 bytes) --- */
	u8                         usr_reg;              /*   256     1 */

	/* XXX 3 bytes hole, try to pack */

	int                        msr_mask_on;          /*   260     4 */
	int                        msr_mask_off;         /*   264     4 */

	/* XXX 4 bytes hole, try to pack */

	struct clk *               clk;                  /*   272     8 */
	struct clk *               pclk;                 /*   280     8 */
	struct notifier_block      clk_notifier;         /*   288    24 */

	/* XXX last struct has 4 bytes of padding */

	struct work_struct         clk_work;             /*   312    64 */
	/* --- cacheline 5 boundary (320 bytes) was 56 bytes ago --- */
	struct reset_control *     rst;                  /*   376     8 */
	/* --- cacheline 6 boundary (384 bytes) --- */
	unsigned int               skip_autocfg:1;       /*   384: 0  4 */
	unsigned int               uart_16550_compatible:1; /*   384: 1  4 */

	/* size: 392, cachelines: 7, members: 11 */
	/* sum members: 377, holes: 2, sum holes: 7 */
	/* sum bitfield members: 2 bits (0 bytes) */
	/* padding: 4 */
	/* paddings: 2, sum paddings: 11 */
	/* bit_padding: 30 bits */
	/* last cacheline: 8 bytes */
};
struct lpss8250_board {
	long unsigned int          freq;                 /*     0     8 */
	unsigned int               base_baud;            /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	int                        (*setup)(struct lpss8250 *, struct uart_port *); /*    16     8 */
	void                       (*exit)(struct lpss8250 *); /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct lpss8250 {
	struct dw8250_port_data    data;                 /*     0   256 */

	/* XXX last struct has 7 bytes of padding */

	/* --- cacheline 4 boundary (256 bytes) --- */
	struct lpss8250_board *    board;                /*   256     8 */
	struct dw_dma_chip         dma_chip;             /*   264    48 */
	struct dw_dma_slave        dma_param;            /*   312    16 */

	/* XXX last struct has 2 bytes of padding */

	/* --- cacheline 5 boundary (320 bytes) was 8 bytes ago --- */
	u8                         dma_maxburst;         /*   328     1 */

	/* size: 336, cachelines: 6, members: 5 */
	/* padding: 7 */
	/* paddings: 2, sum paddings: 9 */
	/* last cacheline: 16 bytes */
};
struct hsu_dma_slave {
	struct device *            dma_dev;              /*     0     8 */
	int                        chan_id;              /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct mid8250_board {
	unsigned int               flags;                /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          freq;                 /*     8     8 */
	unsigned int               base_baud;            /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	int                        (*setup)(struct mid8250 *, struct uart_port *); /*    24     8 */
	void                       (*exit)(struct mid8250 *); /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* sum members: 32, holes: 2, sum holes: 8 */
	/* last cacheline: 40 bytes */
};
struct mid8250 {
	int                        line;                 /*     0     4 */
	int                        dma_index;            /*     4     4 */
	struct pci_dev *           dma_dev;              /*     8     8 */
	struct uart_8250_dma       dma;                  /*    16   240 */

	/* XXX last struct has 5 bytes of padding */

	/* --- cacheline 4 boundary (256 bytes) --- */
	struct mid8250_board *     board;                /*   256     8 */
	struct hsu_dma_chip        dma_chip;             /*   264    40 */

	/* size: 304, cachelines: 5, members: 6 */
	/* paddings: 1, sum paddings: 5 */
	/* last cacheline: 48 bytes */
};
struct mctrl_gpios {
	struct uart_port *         port;                 /*     0     8 */
	struct gpio_desc *         gpio[6];              /*     8    48 */
	int                        irq[6];               /*    56    24 */
	/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
	unsigned int               mctrl_prev;           /*    80     4 */
	bool                       mctrl_on;             /*    84     1 */

	/* size: 88, cachelines: 2, members: 5 */
	/* padding: 3 */
	/* last cacheline: 24 bytes */
};
struct memdev {
	const char  *              name;                 /*     0     8 */
	umode_t                    mode;                 /*     8     2 */

	/* XXX 6 bytes hole, try to pack */

	const struct file_operations  * fops;            /*    16     8 */
	fmode_t                    fmode;                /*    24     4 */

	/* size: 32, cachelines: 1, members: 4 */
	/* sum members: 22, holes: 1, sum holes: 6 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct timer_rand_state {
	long unsigned int          last_time;            /*     0     8 */
	long int                   last_delta;           /*     8     8 */
	long int                   last_delta2;          /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct crng {
	u8                         key[32];              /*     0    32 */
	long unsigned int          generation;           /*    32     8 */
	local_lock_t               lock;                 /*    40     0 */

	/* size: 40, cachelines: 1, members: 3 */
	/* last cacheline: 40 bytes */
};
struct batch_u64 {
	u64                        entropy[12];          /*     0    96 */
	/* --- cacheline 1 boundary (64 bytes) was 32 bytes ago --- */
	local_lock_t               lock;                 /*    96     0 */
	long unsigned int          generation;           /*    96     8 */
	unsigned int               position;             /*   104     4 */

	/* size: 112, cachelines: 2, members: 4 */
	/* padding: 4 */
	/* last cacheline: 48 bytes */
};
struct batch_u32 {
	u32                        entropy[24];          /*     0    96 */
	/* --- cacheline 1 boundary (64 bytes) was 32 bytes ago --- */
	local_lock_t               lock;                 /*    96     0 */
	long unsigned int          generation;           /*    96     8 */
	unsigned int               position;             /*   104     4 */

	/* size: 112, cachelines: 2, members: 4 */
	/* padding: 4 */
	/* last cacheline: 48 bytes */
};
struct fast_pool {
	struct work_struct         mix;                  /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	long unsigned int          pool[4];              /*    64    32 */
	long unsigned int          last;                 /*    96     8 */
	unsigned int               count;                /*   104     4 */

	/* size: 112, cachelines: 2, members: 4 */
	/* padding: 4 */
	/* last cacheline: 48 bytes */
};
struct raw_config_request {
	int                        raw_minor;            /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	__u64                      block_major;          /*     8     8 */
	__u64                      block_minor;          /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct raw_device_data {
	dev_t                      binding;              /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct block_device *      bdev;                 /*     8     8 */
	int                        inuse;                /*    16     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 16, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct raw32_config_request {
	compat_int_t               raw_minor;            /*     0     4 */
	compat_u64                 block_major __attribute__((__aligned__(4))); /*     4     8 */
	compat_u64                 block_minor __attribute__((__aligned__(4))); /*    12     8 */

	/* size: 20, cachelines: 1, members: 3 */
	/* forced alignments: 2 */
	/* last cacheline: 20 bytes */
} __attribute__((__packed__));
struct hpet_info {
	long unsigned int          hi_ireqfreq;          /*     0     8 */
	long unsigned int          hi_flags;             /*     8     8 */
	short unsigned int         hi_hpet;              /*    16     2 */
	short unsigned int         hi_timer;             /*    18     2 */

	/* size: 24, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct hpet_timer {
	u64                        hpet_config;          /*     0     8 */
	union {
		u64                _hpet_hc64;           /*     8     8 */
		u32                _hpet_hc32;           /*     8     4 */
		long unsigned int  _hpet_compare;        /*     8     8 */
	} _u1;                                           /*     8     8 */
	u64                        hpet_fsb[2];          /*    16    16 */

	/* size: 32, cachelines: 1, members: 3 */
	/* last cacheline: 32 bytes */
};
struct hpet {
	u64                        hpet_cap;             /*     0     8 */
	u64                        res0;                 /*     8     8 */
	u64                        hpet_config;          /*    16     8 */
	u64                        res1;                 /*    24     8 */
	u64                        hpet_isr;             /*    32     8 */
	u64                        res2[25];             /*    40   200 */
	/* --- cacheline 3 boundary (192 bytes) was 48 bytes ago --- */
	union {
		u64                _hpet_mc64;           /*   240     8 */
		u32                _hpet_mc32;           /*   240     4 */
		long unsigned int  _hpet_mc;             /*   240     8 */
	} _u0;                                           /*   240     8 */
	u64                        res3;                 /*   248     8 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	struct hpet_timer          hpet_timers[1];       /*   256    32 */

	/* size: 288, cachelines: 5, members: 9 */
	/* last cacheline: 32 bytes */
};
struct hpet_dev {
	struct hpets *             hd_hpets;             /*     0     8 */
	struct hpet *              hd_hpet;              /*     8     8 */
	struct hpet_timer *        hd_timer;             /*    16     8 */
	long unsigned int          hd_ireqfreq;          /*    24     8 */
	long unsigned int          hd_irqdata;           /*    32     8 */
	wait_queue_head_t          hd_waitqueue;         /*    40    24 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct fasync_struct *     hd_async_queue;       /*    64     8 */
	unsigned int               hd_flags;             /*    72     4 */
	unsigned int               hd_irq;               /*    76     4 */
	unsigned int               hd_hdwirq;            /*    80     4 */
	char                       hd_name[7];           /*    84     7 */

	/* size: 96, cachelines: 2, members: 11 */
	/* padding: 5 */
	/* last cacheline: 32 bytes */
};
struct hpets {
	struct hpets *             hp_next;              /*     0     8 */
	struct hpet *              hp_hpet;              /*     8     8 */
	long unsigned int          hp_hpet_phys;         /*    16     8 */
	struct clocksource *       hp_clocksource;       /*    24     8 */
	long long unsigned int     hp_tick_freq;         /*    32     8 */
	long unsigned int          hp_delta;             /*    40     8 */
	unsigned int               hp_ntimer;            /*    48     4 */
	unsigned int               hp_which;             /*    52     4 */
	struct hpet_dev            hp_dev[];             /*    56     0 */

	/* size: 56, cachelines: 1, members: 9 */
	/* last cacheline: 56 bytes */
};
struct compat_hpet_info {
	compat_ulong_t             hi_ireqfreq;          /*     0     4 */
	compat_ulong_t             hi_flags;             /*     4     4 */
	short unsigned int         hi_hpet;              /*     8     2 */
	short unsigned int         hi_timer;             /*    10     2 */

	/* size: 12, cachelines: 1, members: 4 */
	/* last cacheline: 12 bytes */
};
struct nvram_ops {
	ssize_t                    (*get_size)(void);    /*     0     8 */
	unsigned char              (*read_byte)(int);    /*     8     8 */
	void                       (*write_byte)(unsigned char, int); /*    16     8 */
	ssize_t                    (*read)(char *, size_t, loff_t *); /*    24     8 */
	ssize_t                    (*write)(char *, size_t, loff_t *); /*    32     8 */
	long int                   (*initialize)(void);  /*    40     8 */
	long int                   (*set_checksum)(void); /*    48     8 */

	/* size: 56, cachelines: 1, members: 7 */
	/* last cacheline: 56 bytes */
};
struct virtrng_info {
	struct hwrng               hwrng;                /*     0   120 */
	/* --- cacheline 1 boundary (64 bytes) was 56 bytes ago --- */
	struct virtqueue *         vq;                   /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct completion          have_data;            /*   128    32 */
	char                       name[25];             /*   160    25 */

	/* XXX 3 bytes hole, try to pack */

	unsigned int               data_avail;           /*   188     4 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	int                        index;                /*   192     4 */
	bool                       busy;                 /*   196     1 */
	bool                       hwrng_register_done;  /*   197     1 */
	bool                       hwrng_removed;        /*   198     1 */

	/* size: 200, cachelines: 4, members: 9 */
	/* sum members: 196, holes: 1, sum holes: 3 */
	/* padding: 1 */
	/* last cacheline: 8 bytes */
};
struct file_priv {
	struct tpm_chip *          chip;                 /*     0     8 */
	struct tpm_space *         space;                /*     8     8 */
	struct mutex               buffer_mutex;         /*    16    32 */
	struct timer_list          user_read_timer;      /*    48    72 */
	/* --- cacheline 1 boundary (64 bytes) was 56 bytes ago --- */
	struct work_struct         timeout_work;         /*   120    64 */
	/* --- cacheline 2 boundary (128 bytes) was 56 bytes ago --- */
	struct work_struct         async_work;           /*   184    64 */
	/* --- cacheline 3 boundary (192 bytes) was 56 bytes ago --- */
	wait_queue_head_t          async_wait;           /*   248    24 */
	/* --- cacheline 4 boundary (256 bytes) was 16 bytes ago --- */
	ssize_t                    response_length;      /*   272     8 */
	bool                       response_read;        /*   280     1 */
	bool                       command_enqueued;     /*   281     1 */
	u8                         data_buffer[4096];    /*   282  4096 */

	/* size: 4384, cachelines: 69, members: 11 */
	/* padding: 6 */
	/* last cacheline: 32 bytes */
};
struct stclear_flags_t {
	__be16                     tag;                  /*     0     2 */
	u8                         deactivated;          /*     2     1 */
	u8                         disableForceClear;    /*     3     1 */
	u8                         physicalPresence;     /*     4     1 */
	u8                         physicalPresenceLock; /*     5     1 */
	u8                         bGlobalLock;          /*     6     1 */

	/* size: 7, cachelines: 1, members: 6 */
	/* last cacheline: 7 bytes */
} __attribute__((__packed__));
struct tpm1_version {
	u8                         major;                /*     0     1 */
	u8                         minor;                /*     1     1 */
	u8                         rev_major;            /*     2     1 */
	u8                         rev_minor;            /*     3     1 */

	/* size: 4, cachelines: 1, members: 4 */
	/* last cacheline: 4 bytes */
};
struct tpm1_version2 {
	__be16                     tag;                  /*     0     2 */
	struct tpm1_version        version;              /*     2     4 */

	/* size: 6, cachelines: 1, members: 2 */
	/* last cacheline: 6 bytes */
};
struct timeout_t {
	__be32                     a;                    /*     0     4 */
	__be32                     b;                    /*     4     4 */
	__be32                     c;                    /*     8     4 */
	__be32                     d;                    /*    12     4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct duration_t {
	__be32                     tpm_short;            /*     0     4 */
	__be32                     tpm_medium;           /*     4     4 */
	__be32                     tpm_long;             /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct permanent_flags_t {
	__be16                     tag;                  /*     0     2 */
	u8                         disable;              /*     2     1 */
	u8                         ownership;            /*     3     1 */
	u8                         deactivated;          /*     4     1 */
	u8                         readPubek;            /*     5     1 */
	u8                         disableOwnerClear;    /*     6     1 */
	u8                         allowMaintenance;     /*     7     1 */
	u8                         physicalPresenceLifetimeLock; /*     8     1 */
	u8                         physicalPresenceHWEnable; /*     9     1 */
	u8                         physicalPresenceCMDEnable; /*    10     1 */
	u8                         CEKPUsed;             /*    11     1 */
	u8                         TPMpost;              /*    12     1 */
	u8                         TPMpostLock;          /*    13     1 */
	u8                         FIPS;                 /*    14     1 */
	u8                         operator;             /*    15     1 */
	u8                         enableRevokeEK;       /*    16     1 */
	u8                         nvLocked;             /*    17     1 */
	u8                         readSRKPub;           /*    18     1 */
	u8                         tpmEstablished;       /*    19     1 */
	u8                         maintenanceDone;      /*    20     1 */
	u8                         disableFullDALogicInfo; /*    21     1 */

	/* size: 22, cachelines: 1, members: 21 */
	/* last cacheline: 22 bytes */
};
struct tpm1_get_random_out {
	__be32                     rng_data_len;         /*     0     4 */
	u8                         rng_data[128];        /*     4   128 */

	/* size: 132, cachelines: 3, members: 2 */
	/* last cacheline: 4 bytes */
};
struct tpm2_pcr_read_out {
	__be32                     update_cnt;           /*     0     4 */
	__be32                     pcr_selects_cnt;      /*     4     4 */
	__be16                     hash_alg;             /*     8     2 */
	u8                         pcr_select_size;      /*    10     1 */
	u8                         pcr_select[3];        /*    11     3 */
	__be32                     digests_cnt;          /*    14     4 */
	__be16                     digest_size;          /*    18     2 */
	u8                         digest[];             /*    20     0 */

	/* size: 20, cachelines: 1, members: 8 */
	/* last cacheline: 20 bytes */
} __attribute__((__packed__));
struct tpm2_null_auth_area {
	__be32                     handle;               /*     0     4 */
	__be16                     nonce_size;           /*     4     2 */
	u8                         attributes;           /*     6     1 */
	__be16                     auth_size;            /*     7     2 */

	/* size: 9, cachelines: 1, members: 4 */
	/* last cacheline: 9 bytes */
} __attribute__((__packed__));
struct tpm2_get_random_out {
	__be16                     size;                 /*     0     2 */
	u8                         buffer[128];          /*     2   128 */

	/* size: 130, cachelines: 3, members: 2 */
	/* last cacheline: 2 bytes */
};
struct tpm2_get_cap_out {
	u8                         more_data;            /*     0     1 */
	__be32                     subcap_id;            /*     1     4 */
	__be32                     property_cnt;         /*     5     4 */
	__be32                     property_id;          /*     9     4 */
	__be32                     value;                /*    13     4 */

	/* size: 17, cachelines: 1, members: 5 */
	/* last cacheline: 17 bytes */
} __attribute__((__packed__));
struct tpm2_pcr_selection {
	__be16                     hash_alg;             /*     0     2 */
	u8                         size_of_select;       /*     2     1 */
	u8                         pcr_select[3];        /*     3     3 */

	/* size: 6, cachelines: 1, members: 3 */
	/* last cacheline: 6 bytes */
};
struct tpmrm_priv {
	struct file_priv           priv;                 /*     0  4384 */

	/* XXX last struct has 6 bytes of padding */

	/* --- cacheline 68 boundary (4352 bytes) was 32 bytes ago --- */
	struct tpm_space           space;                /*  4384    56 */

	/* XXX last struct has 4 bytes of padding */

	/* size: 4440, cachelines: 70, members: 2 */
	/* paddings: 2, sum paddings: 10 */
	/* last cacheline: 24 bytes */
};
struct tpm2_context {
	__be64                     sequence;             /*     0     8 */
	__be32                     saved_handle;         /*     8     4 */
	__be32                     hierarchy;            /*    12     4 */
	__be16                     blob_size;            /*    16     2 */

	/* size: 18, cachelines: 1, members: 4 */
	/* last cacheline: 18 bytes */
} __attribute__((__packed__));
struct tpm2_cap_handles {
	u8                         more_data;            /*     0     1 */
	__be32                     capability;           /*     1     4 */
	__be32                     count;                /*     5     4 */
	__be32                     handles[];            /*     9     0 */

	/* size: 9, cachelines: 1, members: 4 */
	/* last cacheline: 9 bytes */
} __attribute__((__packed__));
struct tpm_readpubek_out {
	u8                         algorithm[4];         /*     0     4 */
	u8                         encscheme[2];         /*     4     2 */
	u8                         sigscheme[2];         /*     6     2 */
	__be32                     paramsize;            /*     8     4 */
	u8                         parameters[12];       /*    12    12 */
	__be32                     keysize;              /*    24     4 */
	u8                         modulus[256];         /*    28   256 */
	/* --- cacheline 4 boundary (256 bytes) was 28 bytes ago --- */
	u8                         checksum[20];         /*   284    20 */

	/* size: 304, cachelines: 5, members: 8 */
	/* last cacheline: 48 bytes */
};
struct tcpa_event {
	u32                        pcr_index;            /*     0     4 */
	u32                        event_type;           /*     4     4 */
	u8                         pcr_value[20];        /*     8    20 */
	u32                        event_size;           /*    28     4 */
	u8                         event_data[];         /*    32     0 */

	/* size: 32, cachelines: 1, members: 5 */
	/* last cacheline: 32 bytes */
};
struct tcpa_pc_event {
	u32                        event_id;             /*     0     4 */
	u32                        event_size;           /*     4     4 */
	u8                         event_data[];         /*     8     0 */

	/* size: 8, cachelines: 1, members: 3 */
	/* last cacheline: 8 bytes */
};
struct tcg_efi_specid_event_algs {
	u16                        alg_id;               /*     0     2 */
	u16                        digest_size;          /*     2     2 */

	/* size: 4, cachelines: 1, members: 2 */
	/* last cacheline: 4 bytes */
};
struct tcg_efi_specid_event_head {
	u8                         signature[16];        /*     0    16 */
	u32                        platform_class;       /*    16     4 */
	u8                         spec_version_minor;   /*    20     1 */
	u8                         spec_version_major;   /*    21     1 */
	u8                         spec_errata;          /*    22     1 */
	u8                         uintnsize;            /*    23     1 */
	u32                        num_algs;             /*    24     4 */
	struct tcg_efi_specid_event_algs digest_sizes[]; /*    28     0 */

	/* size: 28, cachelines: 1, members: 8 */
	/* last cacheline: 28 bytes */
};
struct tcg_pcr_event {
	u32                        pcr_idx;              /*     0     4 */
	u32                        event_type;           /*     4     4 */
	u8                         digest[20];           /*     8    20 */
	u32                        event_size;           /*    28     4 */
	u8                         event[];              /*    32     0 */

	/* size: 32, cachelines: 1, members: 5 */
	/* last cacheline: 32 bytes */
};
struct tcg_event_field {
	u32                        event_size;           /*     0     4 */
	u8                         event[];              /*     4     0 */

	/* size: 4, cachelines: 1, members: 2 */
	/* last cacheline: 4 bytes */
};
struct tcg_pcr_event2_head {
	u32                        pcr_idx;              /*     0     4 */
	u32                        event_type;           /*     4     4 */
	u32                        count;                /*     8     4 */
	struct tpm_digest          digests[];            /*    12     0 */

	/* size: 12, cachelines: 1, members: 4 */
	/* last cacheline: 12 bytes */
};
struct acpi_table_tpm2 {
	struct acpi_table_header   header;               /*     0    36 */
	u16                        platform_class;       /*    36     2 */
	u16                        reserved;             /*    38     2 */
	u64                        control_address;      /*    40     8 */
	u32                        start_method;         /*    48     4 */

	/* size: 52, cachelines: 1, members: 5 */
	/* last cacheline: 52 bytes */
} __attribute__((__packed__));
struct acpi_tpm2_phy {
	u8                         start_method_specific[12]; /*     0    12 */
	u32                        log_area_minimum_length; /*    12     4 */
	u64                        log_area_start_address; /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct client_hdr {
	u32                        log_max_len;          /*     0     4 */
	u64                        log_start_addr;       /*     4     8 */

	/* size: 12, cachelines: 1, members: 2 */
	/* last cacheline: 12 bytes */
} __attribute__((__packed__));
struct server_hdr {
	u16                        reserved;             /*     0     2 */
	u64                        log_max_len;          /*     2     8 */
	u64                        log_start_addr;       /*    10     8 */

	/* size: 18, cachelines: 1, members: 3 */
	/* last cacheline: 18 bytes */
} __attribute__((__packed__));
struct acpi_tcpa {
	struct acpi_table_header   hdr;                  /*     0    36 */
	u16                        platform_class;       /*    36     2 */
	union {
		struct client_hdr  client;               /*    38    12 */
		struct server_hdr  server;               /*    38    18 */
	};                                               /*    38    18 */

	/* size: 56, cachelines: 1, members: 3 */
	/* last cacheline: 56 bytes */
} __attribute__((__packed__));
struct linux_efi_tpm_eventlog {
	u32                        size;                 /*     0     4 */
	u32                        final_events_preboot_size; /*     4     4 */
	u8                         version;              /*     8     1 */
	u8                         log[];                /*     9     0 */

	/* size: 12, cachelines: 1, members: 4 */
	/* padding: 3 */
	/* last cacheline: 12 bytes */
};
struct efi_tcg2_final_events_table {
	u64                        version;              /*     0     8 */
	u64                        nr_events;            /*     8     8 */
	u8                         events[];             /*    16     0 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct tpm_tis_data {
	u16                        manufacturer_id;      /*     0     2 */

	/* XXX 2 bytes hole, try to pack */

	int                        locality;             /*     4     4 */
	int                        irq;                  /*     8     4 */
	bool                       irq_tested;           /*    12     1 */

	/* XXX 3 bytes hole, try to pack */

	long unsigned int          flags;                /*    16     8 */
	void *                     ilb_base_addr;        /*    24     8 */
	u16                        clkrun_enabled;       /*    32     2 */

	/* XXX 6 bytes hole, try to pack */

	wait_queue_head_t          int_queue;            /*    40    24 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	wait_queue_head_t          read_queue;           /*    64    24 */
	const struct tpm_tis_phy_ops  * phy_ops;         /*    88     8 */
	short unsigned int         rng_quality;          /*    96     2 */

	/* XXX 2 bytes hole, try to pack */

	unsigned int               timeout_min;          /*   100     4 */
	unsigned int               timeout_max;          /*   104     4 */

	/* size: 112, cachelines: 2, members: 13 */
	/* sum members: 95, holes: 4, sum holes: 13 */
	/* padding: 4 */
	/* last cacheline: 48 bytes */
};
struct tpm_tis_phy_ops {
	int                        (*read_bytes)(struct tpm_tis_data *, u32, u16, u8 *); /*     0     8 */
	int                        (*write_bytes)(struct tpm_tis_data *, u32, u16, const u8  *); /*     8     8 */
	int                        (*read16)(struct tpm_tis_data *, u32, u16 *); /*    16     8 */
	int                        (*read32)(struct tpm_tis_data *, u32, u32 *); /*    24     8 */
	int                        (*write32)(struct tpm_tis_data *, u32, u32); /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct tis_vendor_durations_override {
	u32                        did_vid;              /*     0     4 */
	struct tpm1_version        version;              /*     4     4 */
	long unsigned int          durations[3];         /*     8    24 */

	/* size: 32, cachelines: 1, members: 3 */
	/* last cacheline: 32 bytes */
};
struct tis_vendor_timeout_override {
	u32                        did_vid;              /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          timeout_us[4];        /*     8    32 */

	/* size: 40, cachelines: 1, members: 2 */
	/* sum members: 36, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct tpm_info {
	struct resource            res;                  /*     0    96 */
	/* --- cacheline 1 boundary (64 bytes) was 32 bytes ago --- */
	int                        irq;                  /*    96     4 */

	/* size: 104, cachelines: 2, members: 2 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct tpm_tis_tcg_phy {
	struct tpm_tis_data        priv;                 /*     0   112 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 1 boundary (64 bytes) was 48 bytes ago --- */
	void *                     iobase;               /*   112     8 */

	/* size: 120, cachelines: 2, members: 2 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 56 bytes */
};
struct crb_regs_head {
	u32                        loc_state;            /*     0     4 */
	u32                        reserved1;            /*     4     4 */
	u32                        loc_ctrl;             /*     8     4 */
	u32                        loc_sts;              /*    12     4 */
	u8                         reserved2[32];        /*    16    32 */
	u64                        intf_id;              /*    48     8 */
	u64                        ctrl_ext;             /*    56     8 */

	/* size: 64, cachelines: 1, members: 7 */
};
struct crb_regs_tail {
	u32                        ctrl_req;             /*     0     4 */
	u32                        ctrl_sts;             /*     4     4 */
	u32                        ctrl_cancel;          /*     8     4 */
	u32                        ctrl_start;           /*    12     4 */
	u32                        ctrl_int_enable;      /*    16     4 */
	u32                        ctrl_int_sts;         /*    20     4 */
	u32                        ctrl_cmd_size;        /*    24     4 */
	u32                        ctrl_cmd_pa_low;      /*    28     4 */
	u32                        ctrl_cmd_pa_high;     /*    32     4 */
	u32                        ctrl_rsp_size;        /*    36     4 */
	u64                        ctrl_rsp_pa;          /*    40     8 */

	/* size: 48, cachelines: 1, members: 11 */
	/* last cacheline: 48 bytes */
};
struct crb_priv {
	u32                        sm;                   /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	const char  *              hid;                  /*     8     8 */
	struct crb_regs_head *     regs_h;               /*    16     8 */
	struct crb_regs_tail *     regs_t;               /*    24     8 */
	u8 *                       cmd;                  /*    32     8 */
	u8 *                       rsp;                  /*    40     8 */
	u32                        cmd_size;             /*    48     4 */
	u32                        smc_func_id;          /*    52     4 */

	/* size: 56, cachelines: 1, members: 8 */
	/* sum members: 52, holes: 1, sum holes: 4 */
	/* last cacheline: 56 bytes */
};
struct tpm2_crb_smc {
	u32                        interrupt;            /*     0     4 */
	u8                         interrupt_flags;      /*     4     1 */
	u8                         op_flags;             /*     5     1 */
	u16                        reserved2;            /*     6     2 */
	u32                        smc_func_id;          /*     8     4 */

	/* size: 12, cachelines: 1, members: 5 */
	/* last cacheline: 12 bytes */
};
struct amd_iommu_pi_data {
	u32                        ga_tag;               /*     0     4 */
	u32                        prev_ga_tag;          /*     4     4 */
	u64                        base;                 /*     8     8 */
	bool                       is_guest_mode;        /*    16     1 */

	/* XXX 7 bytes hole, try to pack */

	struct vcpu_data *         vcpu_data;            /*    24     8 */
	void *                     ir_data;              /*    32     8 */

	/* size: 40, cachelines: 1, members: 6 */
	/* sum members: 33, holes: 1, sum holes: 7 */
	/* last cacheline: 40 bytes */
};
struct vcpu_data {
	u64                        pi_desc_addr;         /*     0     8 */
	u32                        vector;               /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct amd_iommu_device_info {
	int                        max_pasids;           /*     0     4 */
	u32                        flags;                /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct irq_remap_table {
	raw_spinlock_t             lock;                 /*     0     4 */
	unsigned int               min_index;            /*     4     4 */
	u32 *                      table;                /*     8     8 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct amd_iommu_fault {
	u64                        address;              /*     0     8 */
	u32                        pasid;                /*     8     4 */
	u16                        device_id;            /*    12     2 */
	u16                        tag;                  /*    14     2 */
	u16                        flags;                /*    16     2 */

	/* size: 24, cachelines: 1, members: 5 */
	/* padding: 6 */
	/* last cacheline: 24 bytes */
};
struct protection_domain {
	struct list_head           dev_list;             /*     0    16 */
	struct iommu_domain        domain;               /*    16   136 */
	/* --- cacheline 2 boundary (128 bytes) was 24 bytes ago --- */
	spinlock_t                 lock;                 /*   152     4 */
	u16                        id;                   /*   156     2 */

	/* XXX 2 bytes hole, try to pack */

	atomic64_t                 pt_root;              /*   160     8 */
	int                        glx;                  /*   168     4 */

	/* XXX 4 bytes hole, try to pack */

	u64 *                      gcr3_tbl;             /*   176     8 */
	long unsigned int          flags;                /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	unsigned int               dev_cnt;              /*   192     4 */
	unsigned int               dev_iommu[32];        /*   196   128 */

	/* size: 328, cachelines: 6, members: 10 */
	/* sum members: 318, holes: 2, sum holes: 6 */
	/* padding: 4 */
	/* last cacheline: 8 bytes */
};
struct domain_pgtable {
	int                        mode;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	u64 *                      root;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct amd_iommu {
	struct list_head           list;                 /*     0    16 */
	int                        index;                /*    16     4 */
	raw_spinlock_t             lock;                 /*    20     4 */
	struct pci_dev *           dev;                  /*    24     8 */
	struct pci_dev *           root_pdev;            /*    32     8 */
	u64                        mmio_phys;            /*    40     8 */
	u64                        mmio_phys_end;        /*    48     8 */
	u8 *                       mmio_base;            /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u32                        cap;                  /*    64     4 */
	u8                         acpi_flags;           /*    68     1 */

	/* XXX 3 bytes hole, try to pack */

	u64                        features;             /*    72     8 */
	bool                       is_iommu_v2;          /*    80     1 */

	/* XXX 1 byte hole, try to pack */

	u16                        devid;                /*    82     2 */
	u16                        cap_ptr;              /*    84     2 */
	u16                        pci_seg;              /*    86     2 */
	u64                        exclusion_start;      /*    88     8 */
	u64                        exclusion_length;     /*    96     8 */
	u8 *                       cmd_buf;              /*   104     8 */
	u32                        cmd_buf_head;         /*   112     4 */
	u32                        cmd_buf_tail;         /*   116     4 */
	u8 *                       evt_buf;              /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	u8 *                       ppr_log;              /*   128     8 */
	u8 *                       ga_log;               /*   136     8 */
	u8 *                       ga_log_tail;          /*   144     8 */
	bool                       int_enabled;          /*   152     1 */
	bool                       need_sync;            /*   153     1 */

	/* XXX 6 bytes hole, try to pack */

	struct iommu_device        iommu;                /*   160    40 */
	/* --- cacheline 3 boundary (192 bytes) was 8 bytes ago --- */
	u32                        stored_addr_lo;       /*   200     4 */
	u32                        stored_addr_hi;       /*   204     4 */
	u32                        stored_l1[6][18];     /*   208   432 */
	/* --- cacheline 10 boundary (640 bytes) --- */
	u32                        stored_l2[131];       /*   640   524 */
	/* --- cacheline 18 boundary (1152 bytes) was 12 bytes ago --- */
	u8                         max_banks;            /*  1164     1 */
	u8                         max_counters;         /*  1165     1 */

	/* XXX 2 bytes hole, try to pack */

	struct irq_domain *        ir_domain;            /*  1168     8 */
	struct irq_domain *        msi_domain;           /*  1176     8 */
	struct amd_irte_ops *      irte_ops;             /*  1184     8 */
	u32                        flags;                /*  1192     4 */

	/* XXX 4 bytes hole, try to pack */

	volatile u64  *            cmd_sem;              /*  1200     8 */
	u64                        cmd_sem_val;          /*  1208     8 */
	/* --- cacheline 19 boundary (1216 bytes) --- */
	struct irq_affinity_notify intcapxt_notify;      /*  1216    88 */

	/* size: 1304, cachelines: 21, members: 40 */
	/* sum members: 1288, holes: 5, sum holes: 16 */
	/* last cacheline: 24 bytes */
};
struct amd_irte_ops {
	void                       (*prepare)(void *, u32, u32, u8, u32, int); /*     0     8 */
	void                       (*activate)(void *, u16, u16); /*     8     8 */
	void                       (*deactivate)(void *, u16, u16); /*    16     8 */
	void                       (*set_affinity)(void *, u16, u16, u8, u32); /*    24     8 */
	void *                     (*get)(struct irq_remap_table *, int); /*    32     8 */
	void                       (*set_allocated)(struct irq_remap_table *, int); /*    40     8 */
	bool                       (*is_allocated)(struct irq_remap_table *, int); /*    48     8 */
	void                       (*clear_allocated)(struct irq_remap_table *, int); /*    56     8 */

	/* size: 64, cachelines: 1, members: 8 */
};
struct acpihid_map_entry {
	struct list_head           list;                 /*     0    16 */
	u8                         uid[256];             /*    16   256 */
	/* --- cacheline 4 boundary (256 bytes) was 16 bytes ago --- */
	u8                         hid[9];               /*   272     9 */

	/* XXX 1 byte hole, try to pack */

	u16                        devid;                /*   282     2 */
	u16                        root_devid;           /*   284     2 */
	bool                       cmd_line;             /*   286     1 */

	/* XXX 1 byte hole, try to pack */

	struct iommu_group *       group;                /*   288     8 */

	/* size: 296, cachelines: 5, members: 7 */
	/* sum members: 294, holes: 2, sum holes: 2 */
	/* last cacheline: 40 bytes */
};
struct devid_map {
	struct list_head           list;                 /*     0    16 */
	u8                         id;                   /*    16     1 */

	/* XXX 1 byte hole, try to pack */

	u16                        devid;                /*    18     2 */
	bool                       cmd_line;             /*    20     1 */

	/* size: 24, cachelines: 1, members: 4 */
	/* sum members: 20, holes: 1, sum holes: 1 */
	/* padding: 3 */
	/* last cacheline: 24 bytes */
};
struct iommu_dev_data {
	spinlock_t                 lock;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           list;                 /*     8    16 */
	struct llist_node          dev_data_list;        /*    24     8 */
	struct protection_domain * domain;               /*    32     8 */
	struct pci_dev *           pdev;                 /*    40     8 */
	u16                        devid;                /*    48     2 */
	bool                       iommu_v2;             /*    50     1 */

	/* XXX 1 byte hole, try to pack */

	struct {
		bool               enabled;              /*    52     1 */

		/* XXX 3 bytes hole, try to pack */

		int                qdep;                 /*    56     4 */
	} ats;                                           /*    52     8 */
	bool                       pri_tlp;              /*    60     1 */

	/* XXX 3 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	u32                        errata;               /*    64     4 */
	bool                       use_vapic;            /*    68     1 */
	bool                       defer_attach;         /*    69     1 */

	/* XXX 2 bytes hole, try to pack */

	struct ratelimit_state     rs;                   /*    72    40 */

	/* size: 112, cachelines: 2, members: 13 */
	/* sum members: 102, holes: 4, sum holes: 10 */
	/* last cacheline: 48 bytes */
};
struct dev_table_entry {
	u64                        data[4];              /*     0    32 */

	/* size: 32, cachelines: 1, members: 1 */
	/* last cacheline: 32 bytes */
};
struct unity_map_entry {
	struct list_head           list;                 /*     0    16 */
	u16                        devid_start;          /*    16     2 */
	u16                        devid_end;            /*    18     2 */

	/* XXX 4 bytes hole, try to pack */

	u64                        address_start;        /*    24     8 */
	u64                        address_end;          /*    32     8 */
	int                        prot;                 /*    40     4 */

	/* size: 48, cachelines: 1, members: 6 */
	/* sum members: 40, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 48 bytes */
};
union irte {
	u32                        val;                /*     0     4 */
	struct {
		u32                valid:1;            /*     0: 0  4 */
		u32                no_fault:1;         /*     0: 1  4 */
		u32                int_type:3;         /*     0: 2  4 */
		u32                rq_eoi:1;           /*     0: 5  4 */
		u32                dm:1;               /*     0: 6  4 */
		u32                rsvd_1:1;           /*     0: 7  4 */
		u32                destination:8;      /*     0: 8  4 */
		u32                vector:8;           /*     0:16  4 */
		u32                rsvd_2:8;           /*     0:24  4 */
	} fields;                                      /*     0     4 */
};
union irte_ga_lo {
	u64                        val;                /*     0     8 */
	struct {
		u64                valid:1;            /*     0: 0  8 */
		u64                no_fault:1;         /*     0: 1  8 */
		u64                int_type:3;         /*     0: 2  8 */
		u64                rq_eoi:1;           /*     0: 5  8 */
		u64                dm:1;               /*     0: 6  8 */
		u64                guest_mode:1;       /*     0: 7  8 */
		u64                destination:24;     /*     0: 8  8 */
		u64                ga_tag:32;          /*     0:32  8 */
	} fields_remap;                                /*     0     8 */
	struct {
		u64                valid:1;            /*     0: 0  8 */
		u64                no_fault:1;         /*     0: 1  8 */
		u64                ga_log_intr:1;      /*     0: 2  8 */
		u64                rsvd1:3;            /*     0: 3  8 */
		u64                is_run:1;           /*     0: 6  8 */
		u64                guest_mode:1;       /*     0: 7  8 */
		u64                destination:24;     /*     0: 8  8 */
		u64                ga_tag:32;          /*     0:32  8 */
	} fields_vapic;                                /*     0     8 */
};
union irte_ga_hi {
	u64                        val;                /*     0     8 */
	struct {
		u64                vector:8;           /*     0: 0  8 */
		u64                rsvd_1:4;           /*     0: 8  8 */
		u64                ga_root_ptr:40;     /*     0:12  8 */
		u64                rsvd_2:4;           /*     0:52  8 */
		u64                destination:8;      /*     0:56  8 */
	} fields;                                      /*     0     8 */
};
struct irte_ga {
	union irte_ga_lo           lo;                   /*     0     8 */
	union irte_ga_hi           hi;                   /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct irq_2_irte {
	u16                        devid;                /*     0     2 */
	u16                        index;                /*     2     2 */

	/* size: 4, cachelines: 1, members: 2 */
	/* last cacheline: 4 bytes */
};
struct amd_ir_data {
	u32                        cached_ga_tag;        /*     0     4 */
	struct irq_2_irte          irq_2_irte;           /*     4     4 */
	struct msi_msg             msi_entry;            /*     8    12 */

	/* XXX 4 bytes hole, try to pack */

	void *                     entry;                /*    24     8 */
	void *                     ref;                  /*    32     8 */
	struct irq_cfg *           cfg;                  /*    40     8 */
	int                        ga_vector;            /*    48     4 */
	int                        ga_root_ptr;          /*    52     4 */
	int                        ga_tag;               /*    56     4 */

	/* size: 64, cachelines: 1, members: 9 */
	/* sum members: 56, holes: 1, sum holes: 4 */
	/* padding: 4 */
};
struct irq_remap_ops {
	int                        capability;           /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	int                        (*prepare)(void);     /*     8     8 */
	int                        (*enable)(void);      /*    16     8 */
	void                       (*disable)(void);     /*    24     8 */
	int                        (*reenable)(int);     /*    32     8 */
	int                        (*enable_faulting)(void); /*    40     8 */
	struct irq_domain *        (*get_irq_domain)(struct irq_alloc_info *); /*    48     8 */

	/* size: 56, cachelines: 1, members: 7 */
	/* sum members: 52, holes: 1, sum holes: 4 */
	/* last cacheline: 56 bytes */
};
struct iommu_cmd {
	u32                        data[4];              /*     0    16 */

	/* size: 16, cachelines: 1, members: 1 */
	/* last cacheline: 16 bytes */
};
struct ivhd_header {
	u8                         type;                 /*     0     1 */
	u8                         flags;                /*     1     1 */
	u16                        length;               /*     2     2 */
	u16                        devid;                /*     4     2 */
	u16                        cap_ptr;              /*     6     2 */
	u64                        mmio_phys;            /*     8     8 */
	u16                        pci_seg;              /*    16     2 */
	u16                        info;                 /*    18     2 */
	u32                        efr_attr;             /*    20     4 */
	u64                        efr_reg;              /*    24     8 */
	u64                        res;                  /*    32     8 */

	/* size: 40, cachelines: 1, members: 11 */
	/* last cacheline: 40 bytes */
};
struct ivhd_entry {
	u8                         type;                 /*     0     1 */
	u16                        devid;                /*     1     2 */
	u8                         flags;                /*     3     1 */
	u32                        ext;                  /*     4     4 */
	u32                        hidh;                 /*     8     4 */
	u64                        cid;                  /*    12     8 */
	u8                         uidf;                 /*    20     1 */
	u8                         uidl;                 /*    21     1 */
	u8                         uid;                  /*    22     1 */

	/* size: 23, cachelines: 1, members: 9 */
	/* last cacheline: 23 bytes */
} __attribute__((__packed__));
struct ivmd_header {
	u8                         type;                 /*     0     1 */
	u8                         flags;                /*     1     1 */
	u16                        length;               /*     2     2 */
	u16                        devid;                /*     4     2 */
	u16                        aux;                  /*     6     2 */
	u64                        resv;                 /*     8     8 */
	u64                        range_start;          /*    16     8 */
	u64                        range_length;         /*    24     8 */

	/* size: 32, cachelines: 1, members: 8 */
	/* last cacheline: 32 bytes */
};
struct ivrs_quirk_entry {
	u8                         id;                   /*     0     1 */

	/* XXX 1 byte hole, try to pack */

	u16                        devid;                /*     2     2 */

	/* size: 4, cachelines: 1, members: 2 */
	/* sum members: 3, holes: 1, sum holes: 1 */
	/* last cacheline: 4 bytes */
};
struct acpi_table_dmar {
	struct acpi_table_header   header;               /*     0    36 */
	u8                         width;                /*    36     1 */
	u8                         flags;                /*    37     1 */
	u8                         reserved[10];         /*    38    10 */

	/* size: 48, cachelines: 1, members: 4 */
	/* last cacheline: 48 bytes */
};
struct acpi_dmar_header {
	u16                        type;                 /*     0     2 */
	u16                        length;               /*     2     2 */

	/* size: 4, cachelines: 1, members: 2 */
	/* last cacheline: 4 bytes */
};
struct acpi_dmar_device_scope {
	u8                         entry_type;           /*     0     1 */
	u8                         length;               /*     1     1 */
	u16                        reserved;             /*     2     2 */
	u8                         enumeration_id;       /*     4     1 */
	u8                         bus;                  /*     5     1 */

	/* size: 6, cachelines: 1, members: 5 */
	/* last cacheline: 6 bytes */
};
struct acpi_dmar_pci_path {
	u8                         device;               /*     0     1 */
	u8                         function;             /*     1     1 */

	/* size: 2, cachelines: 1, members: 2 */
	/* last cacheline: 2 bytes */
};
struct acpi_dmar_hardware_unit {
	struct acpi_dmar_header    header;               /*     0     4 */
	u8                         flags;                /*     4     1 */
	u8                         reserved;             /*     5     1 */
	u16                        segment;              /*     6     2 */
	u64                        address;              /*     8     8 */

	/* size: 16, cachelines: 1, members: 5 */
	/* last cacheline: 16 bytes */
};
struct acpi_dmar_reserved_memory {
	struct acpi_dmar_header    header;               /*     0     4 */
	u16                        reserved;             /*     4     2 */
	u16                        segment;              /*     6     2 */
	u64                        base_address;         /*     8     8 */
	u64                        end_address;          /*    16     8 */

	/* size: 24, cachelines: 1, members: 5 */
	/* last cacheline: 24 bytes */
};
struct acpi_dmar_atsr {
	struct acpi_dmar_header    header;               /*     0     4 */
	u8                         flags;                /*     4     1 */
	u8                         reserved;             /*     5     1 */
	u16                        segment;              /*     6     2 */

	/* size: 8, cachelines: 1, members: 4 */
	/* last cacheline: 8 bytes */
};
struct acpi_dmar_rhsa {
	struct acpi_dmar_header    header;               /*     0     4 */
	u32                        reserved;             /*     4     4 */
	u64                        base_address;         /*     8     8 */
	u32                        proximity_domain;     /*    16     4 */

	/* size: 20, cachelines: 1, members: 4 */
	/* last cacheline: 20 bytes */
} __attribute__((__packed__));
struct acpi_dmar_andd {
	struct acpi_dmar_header    header;               /*     0     4 */
	u8                         reserved[3];          /*     4     3 */
	u8                         device_number;        /*     7     1 */
	char                       device_name[1];       /*     8     1 */

	/* size: 9, cachelines: 1, members: 4 */
	/* last cacheline: 9 bytes */
} __attribute__((__packed__));
struct dmar_dev_scope {
	struct device *            dev;                  /*     0     8 */
	u8                         bus;                  /*     8     1 */
	u8                         devfn;                /*     9     1 */

	/* size: 16, cachelines: 1, members: 3 */
	/* padding: 6 */
	/* last cacheline: 16 bytes */
};
struct dmar_drhd_unit {
	struct list_head           list;                 /*     0    16 */
	struct acpi_dmar_header *  hdr;                  /*    16     8 */
	u64                        reg_base_addr;        /*    24     8 */
	struct dmar_dev_scope *    devices;              /*    32     8 */
	int                        devices_cnt;          /*    40     4 */
	u16                        segment;              /*    44     2 */
	u8                         ignored:1;            /*    46: 0  1 */
	u8                         include_all:1;        /*    46: 1  1 */
	u8                         gfx_dedicated:1;      /*    46: 2  1 */

	/* XXX 5 bits hole, try to pack */
	/* XXX 1 byte hole, try to pack */

	struct intel_iommu *       iommu;                /*    48     8 */

	/* size: 56, cachelines: 1, members: 10 */
	/* sum members: 54, holes: 1, sum holes: 1 */
	/* sum bitfield members: 3 bits, bit holes: 1, sum bit holes: 5 bits */
	/* last cacheline: 56 bytes */
};
struct intel_iommu {
	void *                     reg;                  /*     0     8 */
	u64                        reg_phys;             /*     8     8 */
	u64                        reg_size;             /*    16     8 */
	u64                        cap;                  /*    24     8 */
	u64                        ecap;                 /*    32     8 */
	u64                        vccap;                /*    40     8 */
	u32                        gcmd;                 /*    48     4 */
	raw_spinlock_t             register_lock;        /*    52     4 */
	int                        seq_id;               /*    56     4 */
	int                        agaw;                 /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        msagaw;               /*    64     4 */
	unsigned int               irq;                  /*    68     4 */
	unsigned int               pr_irq;               /*    72     4 */
	u16                        segment;              /*    76     2 */
	unsigned char              name[13];             /*    78    13 */

	/* XXX 5 bytes hole, try to pack */

	long unsigned int *        domain_ids;           /*    96     8 */
	struct dmar_domain * * *   domains;              /*   104     8 */
	spinlock_t                 lock;                 /*   112     4 */

	/* XXX 4 bytes hole, try to pack */

	struct root_entry *        root_entry;           /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct iommu_flush         flush;                /*   128    16 */
	struct page_req_dsc *      prq;                  /*   144     8 */
	unsigned char              prq_name[16];         /*   152    16 */
	struct completion          prq_complete;         /*   168    32 */
	/* --- cacheline 3 boundary (192 bytes) was 8 bytes ago --- */
	struct ioasid_allocator_ops pasid_allocator;     /*   200    40 */
	struct q_inval *           qi;                   /*   240     8 */
	u32 *                      iommu_state;          /*   248     8 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	struct ir_table *          ir_table;             /*   256     8 */
	struct irq_domain *        ir_domain;            /*   264     8 */
	struct irq_domain *        ir_msi_domain;        /*   272     8 */
	struct iommu_device        iommu;                /*   280    40 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	int                        node;                 /*   320     4 */
	u32                        flags;                /*   324     4 */
	struct dmar_drhd_unit *    drhd;                 /*   328     8 */

	/* size: 336, cachelines: 6, members: 33 */
	/* sum members: 327, holes: 2, sum holes: 9 */
	/* last cacheline: 16 bytes */
};
struct dmar_pci_path {
	u8                         bus;                  /*     0     1 */
	u8                         device;               /*     1     1 */
	u8                         function;             /*     2     1 */

	/* size: 3, cachelines: 1, members: 3 */
	/* last cacheline: 3 bytes */
};
struct dmar_pci_notify_info {
	struct pci_dev *           dev;                  /*     0     8 */
	long unsigned int          event;                /*     8     8 */
	int                        bus;                  /*    16     4 */
	u16                        seg;                  /*    20     2 */
	u16                        level;                /*    22     2 */
	struct dmar_pci_path       path[];               /*    24     0 */

	/* size: 24, cachelines: 1, members: 6 */
	/* last cacheline: 24 bytes */
};
struct irte {
	union {
		struct {
			__u64      present:1;            /*     0: 0  8 */
			__u64      fpd:1;                /*     0: 1  8 */
			__u64      __res0:6;             /*     0: 2  8 */
			__u64      avail:4;              /*     0: 8  8 */
			__u64      __res1:3;             /*     0:12  8 */
			__u64      pst:1;                /*     0:15  8 */
			__u64      vector:8;             /*     0:16  8 */
			__u64      __res2:40;            /*     0:24  8 */
		};                                       /*     0     8 */
		struct {
			__u64      r_present:1;          /*     0: 0  8 */
			__u64      r_fpd:1;              /*     0: 1  8 */
			__u64      dst_mode:1;           /*     0: 2  8 */
			__u64      redir_hint:1;         /*     0: 3  8 */
			__u64      trigger_mode:1;       /*     0: 4  8 */
			__u64      dlvry_mode:3;         /*     0: 5  8 */
			__u64      r_avail:4;            /*     0: 8  8 */
			__u64      r_res0:4;             /*     0:12  8 */
			__u64      r_vector:8;           /*     0:16  8 */
			__u64      r_res1:8;             /*     0:24  8 */
			__u64      dest_id:32;           /*     0:32  8 */
		};                                       /*     0     8 */
		struct {
			__u64      p_present:1;          /*     0: 0  8 */
			__u64      p_fpd:1;              /*     0: 1  8 */
			__u64      p_res0:6;             /*     0: 2  8 */
			__u64      p_avail:4;            /*     0: 8  8 */
			__u64      p_res1:2;             /*     0:12  8 */
			__u64      p_urgent:1;           /*     0:14  8 */
			__u64      p_pst:1;              /*     0:15  8 */
			__u64      p_vector:8;           /*     0:16  8 */
			__u64      p_res2:14;            /*     0:24  8 */
			__u64      pda_l:26;             /*     0:38  8 */
		};                                       /*     0     8 */
		__u64              low;                  /*     0     8 */
	};                                               /*     0     8 */
	union {
		struct {
			__u64      sid:16;               /*     8: 0  8 */
			__u64      sq:2;                 /*     8:16  8 */
			__u64      svt:2;                /*     8:18  8 */
			__u64      __res3:44;            /*     8:20  8 */
		};                                       /*     8     8 */
		struct {
			__u64      p_sid:16;             /*     8: 0  8 */
			__u64      p_sq:2;               /*     8:16  8 */
			__u64      p_svt:2;              /*     8:18  8 */
			__u64      p_res3:12;            /*     8:20  8 */
			__u64      pda_h:32;             /*     8:32  8 */
		};                                       /*     8     8 */
		__u64              high;                 /*     8     8 */
	};                                               /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct iova {
	struct rb_node             node __attribute__((__aligned__(8))); /*     0    24 */
	long unsigned int          pfn_hi;               /*    24     8 */
	long unsigned int          pfn_lo;               /*    32     8 */

	/* size: 40, cachelines: 1, members: 3 */
	/* forced alignments: 1 */
	/* last cacheline: 40 bytes */
} __attribute__((__aligned__(8)));
struct iova_rcache {
	spinlock_t                 lock;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          depot_size;           /*     8     8 */
	struct iova_magazine *     depot[32];            /*    16   256 */
	/* --- cacheline 4 boundary (256 bytes) was 16 bytes ago --- */
	struct iova_cpu_rcache *   cpu_rcaches;          /*   272     8 */

	/* size: 280, cachelines: 5, members: 4 */
	/* sum members: 276, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct iova_domain {
	spinlock_t                 iova_rbtree_lock;     /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct rb_root             rbroot;               /*     8     8 */
	struct rb_node *           cached_node;          /*    16     8 */
	struct rb_node *           cached32_node;        /*    24     8 */
	long unsigned int          granule;              /*    32     8 */
	long unsigned int          start_pfn;            /*    40     8 */
	long unsigned int          dma_32bit_pfn;        /*    48     8 */
	long unsigned int          max32_alloc_size;     /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct iova_fq *           fq;                   /*    64     8 */
	atomic64_t                 fq_flush_start_cnt;   /*    72     8 */
	atomic64_t                 fq_flush_finish_cnt;  /*    80     8 */
	struct iova                anchor __attribute__((__aligned__(8))); /*    88    40 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct iova_rcache         rcaches[6];           /*   128  1680 */
	/* --- cacheline 28 boundary (1792 bytes) was 16 bytes ago --- */
	iova_flush_cb              flush_cb;             /*  1808     8 */
	iova_entry_dtor            entry_dtor;           /*  1816     8 */
	struct timer_list          fq_timer;             /*  1824    72 */
	/* --- cacheline 29 boundary (1856 bytes) was 40 bytes ago --- */
	atomic_t                   fq_timer_on;          /*  1896     4 */

	/* XXX 4 bytes hole, try to pack */

	struct work_struct         free_iova_work;       /*  1904    64 */

	/* size: 1968, cachelines: 31, members: 18 */
	/* sum members: 1960, holes: 2, sum holes: 8 */
	/* forced alignments: 1 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct iova_fq_entry {
	long unsigned int          iova_pfn;             /*     0     8 */
	long unsigned int          pages;                /*     8     8 */
	long unsigned int          data;                 /*    16     8 */
	u64                        counter;              /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct iova_fq {
	struct iova_fq_entry       entries[256];         /*     0  8192 */
	/* --- cacheline 128 boundary (8192 bytes) --- */
	unsigned int               head;                 /*  8192     4 */
	unsigned int               tail;                 /*  8196     4 */
	spinlock_t                 lock;                 /*  8200     4 */

	/* size: 8208, cachelines: 129, members: 4 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct ioasid_allocator_ops {
	ioasid_alloc_fn_t          alloc;                /*     0     8 */
	ioasid_free_fn_t           free;                 /*     8     8 */
	struct list_head           list;                 /*    16    16 */
	void *                     pdata;                /*    32     8 */

	/* size: 40, cachelines: 1, members: 4 */
	/* last cacheline: 40 bytes */
};
struct qi_desc {
	u64                        qw0;                  /*     0     8 */
	u64                        qw1;                  /*     8     8 */
	u64                        qw2;                  /*    16     8 */
	u64                        qw3;                  /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct q_inval {
	raw_spinlock_t             q_lock;               /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     desc;                 /*     8     8 */
	int *                      desc_status;          /*    16     8 */
	int                        free_head;            /*    24     4 */
	int                        free_tail;            /*    28     4 */
	int                        free_cnt;             /*    32     4 */

	/* size: 40, cachelines: 1, members: 6 */
	/* sum members: 32, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct ir_table {
	struct irte *              base;                 /*     0     8 */
	long unsigned int *        bitmap;               /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct iommu_flush {
	void                       (*flush_context)(struct intel_iommu *, u16, u16, u8, u64); /*     0     8 */
	void                       (*flush_iotlb)(struct intel_iommu *, u16, u64, unsigned int, u64); /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct root_entry {
	u64                        lo;                   /*     0     8 */
	u64                        hi;                   /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct dmar_domain {
	int                        nid;                  /*     0     4 */
	unsigned int               iommu_refcnt[128];    /*     4   512 */
	/* --- cacheline 8 boundary (512 bytes) was 4 bytes ago --- */
	u16                        iommu_did[128];       /*   516   256 */
	/* --- cacheline 12 boundary (768 bytes) was 4 bytes ago --- */
	bool                       has_iotlb_device;     /*   772     1 */

	/* XXX 3 bytes hole, try to pack */

	struct list_head           devices;              /*   776    16 */
	struct list_head           subdevices;           /*   792    16 */
	struct iova_domain         iovad __attribute__((__aligned__(8))); /*   808  1968 */
	/* --- cacheline 43 boundary (2752 bytes) was 24 bytes ago --- */
	struct dma_pte *           pgd;                  /*  2776     8 */
	int                        gaw;                  /*  2784     4 */
	int                        agaw;                 /*  2788     4 */
	int                        flags;                /*  2792     4 */
	int                        iommu_coherency;      /*  2796     4 */
	int                        iommu_snooping;       /*  2800     4 */
	int                        iommu_count;          /*  2804     4 */
	int                        iommu_superpage;      /*  2808     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 44 boundary (2816 bytes) --- */
	u64                        max_addr;             /*  2816     8 */
	u32                        default_pasid;        /*  2824     4 */

	/* XXX 4 bytes hole, try to pack */

	struct iommu_domain        domain;               /*  2832   136 */

	/* size: 2968, cachelines: 47, members: 18 */
	/* sum members: 2957, holes: 3, sum holes: 11 */
	/* forced alignments: 1 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct dma_pte {
	u64                        val;                  /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct dmar_res_callback {
	dmar_res_handler_t         cb[5];                /*     0    40 */
	void *                     arg[5];               /*    40    40 */
	/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
	bool                       ignore_unhandled;     /*    80     1 */
	bool                       print_entry;          /*    81     1 */

	/* size: 88, cachelines: 2, members: 4 */
	/* padding: 6 */
	/* last cacheline: 24 bytes */
};
struct context_entry {
	u64                        lo;                   /*     0     8 */
	u64                        hi;                   /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct subdev_domain_info {
	struct list_head           link_phys;            /*     0    16 */
	struct list_head           link_domain;          /*    16    16 */
	struct device *            pdev;                 /*    32     8 */
	struct dmar_domain *       domain;               /*    40     8 */
	int                        users;                /*    48     4 */

	/* size: 56, cachelines: 1, members: 5 */
	/* padding: 4 */
	/* last cacheline: 56 bytes */
};
struct device_domain_info {
	struct list_head           link;                 /*     0    16 */
	struct list_head           global;               /*    16    16 */
	struct list_head           table;                /*    32    16 */
	struct list_head           subdevices;           /*    48    16 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u32                        segment;              /*    64     4 */
	u8                         bus;                  /*    68     1 */
	u8                         devfn;                /*    69     1 */
	u16                        pfsid;                /*    70     2 */
	u8                         pasid_supported:3;    /*    72: 0  1 */
	u8                         pasid_enabled:1;      /*    72: 3  1 */
	u8                         pri_supported:1;      /*    72: 4  1 */
	u8                         pri_enabled:1;        /*    72: 5  1 */
	u8                         ats_supported:1;      /*    72: 6  1 */
	u8                         ats_enabled:1;        /*    72: 7  1 */
	u8                         auxd_enabled:1;       /*    73: 0  1 */

	/* XXX 7 bits hole, try to pack */

	u8                         ats_qdep;             /*    74     1 */

	/* XXX 5 bytes hole, try to pack */

	struct device *            dev;                  /*    80     8 */
	struct intel_iommu *       iommu;                /*    88     8 */
	struct dmar_domain *       domain;               /*    96     8 */
	struct pasid_table *       pasid_table;          /*   104     8 */

	/* size: 112, cachelines: 2, members: 20 */
	/* sum members: 105, holes: 1, sum holes: 5 */
	/* sum bitfield members: 9 bits, bit holes: 1, sum bit holes: 7 bits */
	/* last cacheline: 48 bytes */
};
struct pasid_table {
	void *                     table;                /*     0     8 */
	int                        order;                /*     8     4 */
	u32                        max_pasid;            /*    12     4 */
	struct list_head           dev;                  /*    16    16 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct dmar_rmrr_unit {
	struct list_head           list;                 /*     0    16 */
	struct acpi_dmar_header *  hdr;                  /*    16     8 */
	u64                        base_address;         /*    24     8 */
	u64                        end_address;          /*    32     8 */
	struct dmar_dev_scope *    devices;              /*    40     8 */
	int                        devices_cnt;          /*    48     4 */

	/* size: 56, cachelines: 1, members: 6 */
	/* padding: 4 */
	/* last cacheline: 56 bytes */
};
struct dmar_atsr_unit {
	struct list_head           list;                 /*     0    16 */
	struct acpi_dmar_header *  hdr;                  /*    16     8 */
	struct dmar_dev_scope *    devices;              /*    24     8 */
	int                        devices_cnt;          /*    32     4 */
	u8                         include_all:1;        /*    36: 0  1 */

	/* size: 40, cachelines: 1, members: 5 */
	/* padding: 3 */
	/* bit_padding: 7 bits */
	/* last cacheline: 40 bytes */
};
struct domain_context_mapping_data {
	struct dmar_domain *       domain;               /*     0     8 */
	struct intel_iommu *       iommu;                /*     8     8 */
	struct pasid_table *       table;                /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct pasid_dir_entry {
	u64                        val;                  /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct pasid_entry {
	u64                        val[8];               /*     0    64 */

	/* size: 64, cachelines: 1, members: 1 */
};
struct pasid_table_opaque {
	struct pasid_table * *     pasid_table;          /*     0     8 */
	int                        segment;              /*     8     4 */
	int                        bus;                  /*    12     4 */
	int                        devfn;                /*    16     4 */

	/* size: 24, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_dma_map {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        __data_loc_dev_name;  /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	dma_addr_t                 dev_addr;             /*    16     8 */
	phys_addr_t                phys_addr;            /*    24     8 */
	size_t                     size;                 /*    32     8 */
	char                       __data[];             /*    40     0 */

	/* size: 40, cachelines: 1, members: 6 */
	/* sum members: 36, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct trace_event_raw_dma_unmap {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        __data_loc_dev_name;  /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	dma_addr_t                 dev_addr;             /*    16     8 */
	size_t                     size;                 /*    24     8 */
	char                       __data[];             /*    32     0 */

	/* size: 32, cachelines: 1, members: 5 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct trace_event_raw_dma_map_sg {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        __data_loc_dev_name;  /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	dma_addr_t                 dev_addr;             /*    16     8 */
	phys_addr_t                phys_addr;            /*    24     8 */
	size_t                     size;                 /*    32     8 */
	int                        index;                /*    40     4 */
	int                        total;                /*    44     4 */
	char                       __data[];             /*    48     0 */

	/* size: 48, cachelines: 1, members: 8 */
	/* sum members: 44, holes: 1, sum holes: 4 */
	/* last cacheline: 48 bytes */
};
struct trace_event_data_offsets_dma_map {
	u32                        dev_name;             /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct trace_event_data_offsets_dma_unmap {
	u32                        dev_name;             /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct trace_event_data_offsets_dma_map_sg {
	u32                        dev_name;             /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct page_req_dsc {
	union {
		struct {
			u64        type:8;               /*     0: 0  8 */
			u64        pasid_present:1;      /*     0: 8  8 */
			u64        priv_data_present:1;  /*     0: 9  8 */
			u64        rsvd:6;               /*     0:10  8 */
			u64        rid:16;               /*     0:16  8 */
			u64        pasid:20;             /*     0:32  8 */
			u64        exe_req:1;            /*     0:52  8 */
			u64        pm_req:1;             /*     0:53  8 */
			u64        rsvd2:10;             /*     0:54  8 */
		};                                       /*     0     8 */
		u64                qw_0;                 /*     0     8 */
	};                                               /*     0     8 */
	union {
		struct {
			u64        rd_req:1;             /*     8: 0  8 */
			u64        wr_req:1;             /*     8: 1  8 */
			u64        lpig:1;               /*     8: 2  8 */
			u64        prg_index:9;          /*     8: 3  8 */
			u64        addr:52;              /*     8:12  8 */
		};                                       /*     8     8 */
		u64                qw_1;                 /*     8     8 */
	};                                               /*     8     8 */
	u64                        priv_data[2];         /*    16    16 */

	/* size: 32, cachelines: 1, members: 3 */
	/* last cacheline: 32 bytes */
};
struct intel_svm_dev {
	struct list_head           list;                 /*     0    16 */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*    16    16 */
	struct device *            dev;                  /*    32     8 */
	struct intel_iommu *       iommu;                /*    40     8 */
	struct iommu_sva           sva;                  /*    48     8 */
	u32                        pasid;                /*    56     4 */
	int                        users;                /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u16                        did;                  /*    64     2 */
	u16                        dev_iotlb:1;          /*    66: 0  2 */

	/* XXX 15 bits hole, try to pack */

	u16                        sid;                  /*    68     2 */
	u16                        qdep;                 /*    70     2 */

	/* size: 72, cachelines: 2, members: 11 */
	/* sum members: 70 */
	/* sum bitfield members: 1 bits, bit holes: 1, sum bit holes: 15 bits */
	/* forced alignments: 1 */
	/* last cacheline: 8 bytes */
} __attribute__((__aligned__(8)));
struct intel_svm {
	struct mmu_notifier        notifier __attribute__((__aligned__(8))); /*     0    72 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	struct mm_struct *         mm;                   /*    72     8 */
	unsigned int               flags;                /*    80     4 */
	u32                        pasid;                /*    84     4 */
	int                        gpasid;               /*    88     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           devs;                 /*    96    16 */

	/* size: 112, cachelines: 2, members: 6 */
	/* sum members: 108, holes: 1, sum holes: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct ioapic_scope {
	struct intel_iommu *       iommu;                /*     0     8 */
	unsigned int               id;                   /*     8     4 */
	unsigned int               bus;                  /*    12     4 */
	unsigned int               devfn;                /*    16     4 */

	/* size: 24, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct hpet_scope {
	struct intel_iommu *       iommu;                /*     0     8 */
	u8                         id;                   /*     8     1 */

	/* XXX 3 bytes hole, try to pack */

	unsigned int               bus;                  /*    12     4 */
	unsigned int               devfn;                /*    16     4 */

	/* size: 24, cachelines: 1, members: 4 */
	/* sum members: 17, holes: 1, sum holes: 3 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct irq_2_iommu {
	struct intel_iommu *       iommu;                /*     0     8 */
	u16                        irte_index;           /*     8     2 */
	u16                        sub_handle;           /*    10     2 */
	u8                         irte_mask;            /*    12     1 */

	/* XXX 3 bytes hole, try to pack */

	enum irq_mode              mode;                 /*    16     4 */

	/* size: 24, cachelines: 1, members: 5 */
	/* sum members: 17, holes: 1, sum holes: 3 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct intel_ir_data {
	struct irq_2_iommu         irq_2_iommu;          /*     0    24 */

	/* XXX last struct has 4 bytes of padding */

	struct irte                irte_entry;           /*    24    16 */
	union {
		struct msi_msg     msi_entry;            /*    40    12 */
	};                                               /*    40    12 */

	/* size: 56, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 56 bytes */
};
struct set_msi_sid_data {
	struct pci_dev *           pdev;                 /*     0     8 */
	u16                        alias;                /*     8     2 */

	/* XXX 2 bytes hole, try to pack */

	int                        count;                /*    12     4 */
	int                        busmatch_count;       /*    16     4 */

	/* size: 24, cachelines: 1, members: 4 */
	/* sum members: 18, holes: 1, sum holes: 2 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct iommu_group {
	struct kobject             kobj;                 /*     0    96 */
	/* --- cacheline 1 boundary (64 bytes) was 32 bytes ago --- */
	struct kobject *           devices_kobj;         /*    96     8 */
	struct list_head           devices;              /*   104    16 */
	struct mutex               mutex;                /*   120    32 */
	/* --- cacheline 2 boundary (128 bytes) was 24 bytes ago --- */
	struct blocking_notifier_head notifier;          /*   152    48 */
	/* --- cacheline 3 boundary (192 bytes) was 8 bytes ago --- */
	void *                     iommu_data;           /*   200     8 */
	void                       (*iommu_data_release)(void *); /*   208     8 */
	char *                     name;                 /*   216     8 */
	int                        id;                   /*   224     4 */

	/* XXX 4 bytes hole, try to pack */

	struct iommu_domain *      default_domain;       /*   232     8 */
	struct iommu_domain *      domain;               /*   240     8 */
	struct list_head           entry;                /*   248    16 */

	/* size: 264, cachelines: 5, members: 12 */
	/* sum members: 260, holes: 1, sum holes: 4 */
	/* last cacheline: 8 bytes */
};
struct fsl_mc_device {
	struct device              dev __attribute__((__aligned__(8))); /*     0  1000 */
	/* --- cacheline 15 boundary (960 bytes) was 40 bytes ago --- */
	u64                        dma_mask;             /*  1000     8 */
	u16                        flags;                /*  1008     2 */

	/* XXX 2 bytes hole, try to pack */

	u32                        icid;                 /*  1012     4 */
	u16                        mc_handle;            /*  1016     2 */

	/* XXX 6 bytes hole, try to pack */

	/* --- cacheline 16 boundary (1024 bytes) --- */
	struct fsl_mc_io *         mc_io;                /*  1024     8 */
	struct fsl_mc_obj_desc     obj_desc;             /*  1032    52 */

	/* XXX last struct has 2 bytes of padding */
	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 17 boundary (1088 bytes) --- */
	struct resource *          regions;              /*  1088     8 */
	struct fsl_mc_device_irq * * irqs;               /*  1096     8 */
	struct fsl_mc_resource *   resource;             /*  1104     8 */
	struct device_link *       consumer_link;        /*  1112     8 */
	char *                     driver_override;      /*  1120     8 */

	/* size: 1128, cachelines: 18, members: 12 */
	/* sum members: 1116, holes: 3, sum holes: 12 */
	/* paddings: 1, sum paddings: 2 */
	/* forced alignments: 1 */
	/* last cacheline: 40 bytes */
} __attribute__((__aligned__(8)));
struct fsl_mc_resource {
	enum fsl_mc_pool_type      type;                 /*     0     4 */
	s32                        id;                   /*     4     4 */
	void *                     data;                 /*     8     8 */
	struct fsl_mc_resource_pool * parent_pool;       /*    16     8 */
	struct list_head           node;                 /*    24    16 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct fsl_mc_device_irq {
	struct msi_desc *          msi_desc;             /*     0     8 */
	struct fsl_mc_device *     mc_dev;               /*     8     8 */
	u8                         dev_irq_index;        /*    16     1 */

	/* XXX 7 bytes hole, try to pack */

	struct fsl_mc_resource     resource;             /*    24    40 */

	/* size: 64, cachelines: 1, members: 4 */
	/* sum members: 57, holes: 1, sum holes: 7 */
};
struct fsl_mc_obj_desc {
	char                       type[16];             /*     0    16 */
	int                        id;                   /*    16     4 */
	u16                        vendor;               /*    20     2 */
	u16                        ver_major;            /*    22     2 */
	u16                        ver_minor;            /*    24     2 */
	u8                         irq_count;            /*    26     1 */
	u8                         region_count;         /*    27     1 */
	u32                        state;                /*    28     4 */
	char                       label[16];            /*    32    16 */
	u16                        flags;                /*    48     2 */

	/* size: 52, cachelines: 1, members: 10 */
	/* padding: 2 */
	/* last cacheline: 52 bytes */
};
struct fsl_mc_io {
	struct device *            dev;                  /*     0     8 */
	u16                        flags;                /*     8     2 */

	/* XXX 2 bytes hole, try to pack */

	u32                        portal_size;          /*    12     4 */
	phys_addr_t                portal_phys_addr;     /*    16     8 */
	void *                     portal_virt_addr;     /*    24     8 */
	struct fsl_mc_device *     dpmcp_dev;            /*    32     8 */
	union {
		struct mutex       mutex;                /*    40    32 */
		raw_spinlock_t     spinlock;             /*    40     4 */
	};                                               /*    40    32 */

	/* size: 72, cachelines: 2, members: 7 */
	/* sum members: 70, holes: 1, sum holes: 2 */
	/* last cacheline: 8 bytes */
};
struct group_device {
	struct list_head           list;                 /*     0    16 */
	struct device *            dev;                  /*    16     8 */
	char *                     name;                 /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* last cacheline: 32 bytes */
};
struct iommu_group_attribute {
	struct attribute           attr;                 /*     0    16 */

	/* XXX last struct has 6 bytes of padding */

	ssize_t                    (*show)(struct iommu_group *, char *); /*    16     8 */
	ssize_t                    (*store)(struct iommu_group *, const char  *, size_t); /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* paddings: 1, sum paddings: 6 */
	/* last cacheline: 32 bytes */
};
struct group_for_pci_data {
	struct pci_dev *           pdev;                 /*     0     8 */
	struct iommu_group *       group;                /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct __group_domain_type {
	struct device *            dev;                  /*     0     8 */
	unsigned int               type;                 /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct iommu_sva_bind_group_data {
	struct mm_struct *         mm;                   /*     0     8 */
	struct iommu_sva *         handle;               /*     8     8 */
	void *                     drvdata;              /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_iommu_group_event {
	struct trace_entry         ent;                  /*     0     8 */
	int                        gid;                  /*     8     4 */
	u32                        __data_loc_device;    /*    12     4 */
	char                       __data[];             /*    16     0 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct trace_event_raw_iommu_device_event {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        __data_loc_device;    /*     8     4 */
	char                       __data[];             /*    12     0 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct trace_event_raw_map {
	struct trace_entry         ent;                  /*     0     8 */
	u64                        iova;                 /*     8     8 */
	u64                        paddr;                /*    16     8 */
	size_t                     size;                 /*    24     8 */
	char                       __data[];             /*    32     0 */

	/* size: 32, cachelines: 1, members: 5 */
	/* last cacheline: 32 bytes */
};
struct trace_event_raw_unmap {
	struct trace_entry         ent;                  /*     0     8 */
	u64                        iova;                 /*     8     8 */
	size_t                     size;                 /*    16     8 */
	size_t                     unmapped_size;        /*    24     8 */
	char                       __data[];             /*    32     0 */

	/* size: 32, cachelines: 1, members: 5 */
	/* last cacheline: 32 bytes */
};
struct trace_event_raw_switch_dirty_log {
	struct trace_entry         ent;                  /*     0     8 */
	u64                        iova;                 /*     8     8 */
	size_t                     size;                 /*    16     8 */
	bool                       enable;               /*    24     1 */
	char                       __data[];             /*    25     0 */

	/* size: 32, cachelines: 1, members: 5 */
	/* padding: 7 */
	/* last cacheline: 32 bytes */
};
struct trace_event_raw_sync_dirty_log {
	struct trace_entry         ent;                  /*     0     8 */
	u64                        iova;                 /*     8     8 */
	size_t                     size;                 /*    16     8 */
	char                       __data[];             /*    24     0 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_clear_dirty_log {
	struct trace_entry         ent;                  /*     0     8 */
	u64                        iova;                 /*     8     8 */
	size_t                     size;                 /*    16     8 */
	char                       __data[];             /*    24     0 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_iommu_error {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        __data_loc_device;    /*     8     4 */
	u32                        __data_loc_driver;    /*    12     4 */
	u64                        iova;                 /*    16     8 */
	int                        flags;                /*    24     4 */
	char                       __data[];             /*    28     0 */

	/* size: 32, cachelines: 1, members: 6 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct trace_event_raw_dev_fault {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        __data_loc_device;    /*     8     4 */
	int                        type;                 /*    12     4 */
	int                        reason;               /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        addr;                 /*    24     8 */
	u64                        fetch_addr;           /*    32     8 */
	u32                        pasid;                /*    40     4 */
	u32                        grpid;                /*    44     4 */
	u32                        flags;                /*    48     4 */
	u32                        prot;                 /*    52     4 */
	char                       __data[];             /*    56     0 */

	/* size: 56, cachelines: 1, members: 11 */
	/* sum members: 52, holes: 1, sum holes: 4 */
	/* last cacheline: 56 bytes */
};
struct trace_event_raw_dev_page_response {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        __data_loc_device;    /*     8     4 */
	int                        code;                 /*    12     4 */
	u32                        pasid;                /*    16     4 */
	u32                        grpid;                /*    20     4 */
	char                       __data[];             /*    24     0 */

	/* size: 24, cachelines: 1, members: 6 */
	/* last cacheline: 24 bytes */
};
struct trace_event_data_offsets_iommu_group_event {
	u32                        device;               /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct trace_event_data_offsets_iommu_device_event {
	u32                        device;               /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct trace_event_data_offsets_map {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_unmap {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_switch_dirty_log {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_sync_dirty_log {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_clear_dirty_log {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_iommu_error {
	u32                        device;               /*     0     4 */
	u32                        driver;               /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct trace_event_data_offsets_dev_fault {
	u32                        device;               /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct trace_event_data_offsets_dev_page_response {
	u32                        device;               /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct trace_event_raw_smmu_bond {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        __data_loc_dev;       /*     8     4 */
	int                        pasid;                /*    12     4 */
	char                       __data[];             /*    16     0 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct trace_event_raw_smmu_mm_release {
	struct trace_entry         ent;                  /*     0     8 */
	int                        pasid;                /*     8     4 */
	char                       __data[];             /*    12     0 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct trace_event_raw_smmu_mm_invalidate {
	struct trace_entry         ent;                  /*     0     8 */
	int                        pasid;                /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          start;                /*    16     8 */
	long unsigned int          end;                  /*    24     8 */
	char                       __data[];             /*    32     0 */

	/* size: 32, cachelines: 1, members: 5 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct trace_event_raw_smmu_mn {
	struct trace_entry         ent;                  /*     0     8 */
	int                        pasid;                /*     8     4 */
	char                       __data[];             /*    12     0 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct trace_event_data_offsets_smmu_bond {
	u32                        dev;                  /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct trace_event_data_offsets_smmu_mm_release {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_smmu_mm_invalidate {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_smmu_mn {

	/* size: 0, cachelines: 0, members: 0 */
};
struct iommu_dma_msi_page {
	struct list_head           list;                 /*     0    16 */
	dma_addr_t                 iova;                 /*    16     8 */
	phys_addr_t                phys;                 /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* last cacheline: 32 bytes */
};
struct iommu_dma_cookie {
	enum iommu_dma_cookie_type type;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	union {
		struct iova_domain iovad __attribute__((__aligned__(8))); /*     8  1968 */
		dma_addr_t         msi_iova;             /*     8     8 */
	} __attribute__((__aligned__(8)));               /*     8  1968 */
	/* --- cacheline 30 boundary (1920 bytes) was 56 bytes ago --- */
	struct list_head           msi_page_list;        /*  1976    16 */
	/* --- cacheline 31 boundary (1984 bytes) was 8 bytes ago --- */
	struct iommu_domain *      fq_domain;            /*  1992     8 */

	/* size: 2000, cachelines: 32, members: 4 */
	/* sum members: 1996, holes: 1, sum holes: 4 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 4 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct ioasid_set {
	int                        dummy;                /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct ioasid_data {
	ioasid_t                   id;                   /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct ioasid_set *        set;                  /*     8     8 */
	void *                     private;              /*    16     8 */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*    24    16 */

	/* size: 40, cachelines: 1, members: 4 */
	/* sum members: 36, holes: 1, sum holes: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 40 bytes */
} __attribute__((__aligned__(8)));
struct ioasid_allocator_data {
	struct ioasid_allocator_ops * ops;               /*     0     8 */
	struct list_head           list;                 /*     8    16 */
	struct list_head           slist;                /*    24    16 */
	long unsigned int          flags;                /*    40     8 */
	struct xarray              xa;                   /*    48    16 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*    64    16 */

	/* size: 80, cachelines: 2, members: 6 */
	/* forced alignments: 1 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct iova_magazine {
	long unsigned int          size;                 /*     0     8 */
	long unsigned int          pfns[128];            /*     8  1024 */

	/* size: 1032, cachelines: 17, members: 2 */
	/* last cacheline: 8 bytes */
};
struct iova_cpu_rcache {
	spinlock_t                 lock;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct iova_magazine *     loaded;               /*     8     8 */
	struct iova_magazine *     prev;                 /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct iopf_device_param {
	struct device *            dev;                  /*     0     8 */
	struct iopf_queue *        queue;                /*     8     8 */
	struct list_head           queue_list;           /*    16    16 */
	struct list_head           partial;              /*    32    16 */

	/* size: 48, cachelines: 1, members: 4 */
	/* last cacheline: 48 bytes */
};
struct iopf_queue {
	struct workqueue_struct *  wq;                   /*     0     8 */
	struct list_head           devices;              /*     8    16 */
	struct mutex               lock;                 /*    24    32 */

	/* size: 56, cachelines: 1, members: 3 */
	/* last cacheline: 56 bytes */
};
struct iopf_fault {
	struct iommu_fault         fault;                /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct list_head           list;                 /*    64    16 */

	/* size: 80, cachelines: 2, members: 2 */
	/* last cacheline: 16 bytes */
};
struct iopf_group {
	struct iopf_fault          last_fault;           /*     0    80 */
	/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
	struct list_head           faults;               /*    80    16 */
	struct work_struct         work;                 /*    96    64 */
	/* --- cacheline 2 boundary (128 bytes) was 32 bytes ago --- */
	struct device *            dev;                  /*   160     8 */

	/* size: 168, cachelines: 3, members: 4 */
	/* last cacheline: 40 bytes */
};
struct mipi_dsi_msg {
	u8                         channel;              /*     0     1 */
	u8                         type;                 /*     1     1 */
	u16                        flags;                /*     2     2 */

	/* XXX 4 bytes hole, try to pack */

	size_t                     tx_len;               /*     8     8 */
	const void  *              tx_buf;               /*    16     8 */
	size_t                     rx_len;               /*    24     8 */
	void *                     rx_buf;               /*    32     8 */

	/* size: 40, cachelines: 1, members: 7 */
	/* sum members: 36, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct mipi_dsi_packet {
	size_t                     size;                 /*     0     8 */
	u8                         header[4];            /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	size_t                     payload_length;       /*    16     8 */
	const u8  *                payload;              /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct mipi_dsi_host_ops {
	int                        (*attach)(struct mipi_dsi_host *, struct mipi_dsi_device *); /*     0     8 */
	int                        (*detach)(struct mipi_dsi_host *, struct mipi_dsi_device *); /*     8     8 */
	ssize_t                    (*transfer)(struct mipi_dsi_host *, const struct mipi_dsi_msg  *); /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct mipi_dsi_host {
	struct device *            dev;                  /*     0     8 */
	const struct mipi_dsi_host_ops  * ops;           /*     8     8 */
	struct list_head           list;                 /*    16    16 */

	/* size: 32, cachelines: 1, members: 3 */
	/* last cacheline: 32 bytes */
};
struct mipi_dsi_device {
	struct mipi_dsi_host *     host;                 /*     0     8 */
	struct device              dev __attribute__((__aligned__(8))); /*     8  1000 */
	/* --- cacheline 15 boundary (960 bytes) was 48 bytes ago --- */
	char                       name[20];             /*  1008    20 */
	/* --- cacheline 16 boundary (1024 bytes) was 4 bytes ago --- */
	unsigned int               channel;              /*  1028     4 */
	unsigned int               lanes;                /*  1032     4 */
	enum mipi_dsi_pixel_format format;               /*  1036     4 */
	long unsigned int          mode_flags;           /*  1040     8 */
	long unsigned int          hs_rate;              /*  1048     8 */
	long unsigned int          lp_rate;              /*  1056     8 */

	/* size: 1064, cachelines: 17, members: 9 */
	/* forced alignments: 1 */
	/* last cacheline: 40 bytes */
} __attribute__((__aligned__(8)));
struct mipi_dsi_device_info {
	char                       type[20];             /*     0    20 */
	u32                        channel;              /*    20     4 */
	struct device_node *       node;                 /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* last cacheline: 32 bytes */
};
struct mipi_dsi_driver {
	struct device_driver       driver;               /*     0   192 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	int                        (*probe)(struct mipi_dsi_device *); /*   192     8 */
	int                        (*remove)(struct mipi_dsi_device *); /*   200     8 */
	void                       (*shutdown)(struct mipi_dsi_device *); /*   208     8 */

	/* size: 216, cachelines: 4, members: 4 */
	/* last cacheline: 24 bytes */
};
struct drm_dsc_picture_parameter_set {
	u8                         dsc_version;          /*     0     1 */
	u8                         pps_identifier;       /*     1     1 */
	u8                         pps_reserved;         /*     2     1 */
	u8                         pps_3;                /*     3     1 */
	u8                         pps_4;                /*     4     1 */
	u8                         bits_per_pixel_low;   /*     5     1 */
	__be16                     pic_height;           /*     6     2 */
	__be16                     pic_width;            /*     8     2 */
	__be16                     slice_height;         /*    10     2 */
	__be16                     slice_width;          /*    12     2 */
	__be16                     chunk_size;           /*    14     2 */
	u8                         initial_xmit_delay_high; /*    16     1 */
	u8                         initial_xmit_delay_low; /*    17     1 */
	__be16                     initial_dec_delay;    /*    18     2 */
	u8                         pps20_reserved;       /*    20     1 */
	u8                         initial_scale_value;  /*    21     1 */
	__be16                     scale_increment_interval; /*    22     2 */
	u8                         scale_decrement_interval_high; /*    24     1 */
	u8                         scale_decrement_interval_low; /*    25     1 */
	u8                         pps26_reserved;       /*    26     1 */
	u8                         first_line_bpg_offset; /*    27     1 */
	__be16                     nfl_bpg_offset;       /*    28     2 */
	__be16                     slice_bpg_offset;     /*    30     2 */
	__be16                     initial_offset;       /*    32     2 */
	__be16                     final_offset;         /*    34     2 */
	u8                         flatness_min_qp;      /*    36     1 */
	u8                         flatness_max_qp;      /*    37     1 */
	__be16                     rc_model_size;        /*    38     2 */
	u8                         rc_edge_factor;       /*    40     1 */
	u8                         rc_quant_incr_limit0; /*    41     1 */
	u8                         rc_quant_incr_limit1; /*    42     1 */
	u8                         rc_tgt_offset;        /*    43     1 */
	u8                         rc_buf_thresh[14];    /*    44    14 */
	__be16                     rc_range_parameters[15]; /*    58    30 */
	/* --- cacheline 1 boundary (64 bytes) was 24 bytes ago --- */
	u8                         native_422_420;       /*    88     1 */
	u8                         second_line_bpg_offset; /*    89     1 */
	__be16                     nsl_bpg_offset;       /*    90     2 */
	__be16                     second_line_offset_adj; /*    92     2 */
	u32                        pps_long_94_reserved; /*    94     4 */
	u32                        pps_long_98_reserved; /*    98     4 */
	u32                        pps_long_102_reserved; /*   102     4 */
	u32                        pps_long_106_reserved; /*   106     4 */
	u32                        pps_long_110_reserved; /*   110     4 */
	u32                        pps_long_114_reserved; /*   114     4 */
	u32                        pps_long_118_reserved; /*   118     4 */
	u32                        pps_long_122_reserved; /*   122     4 */
	__be16                     pps_short_126_reserved; /*   126     2 */

	/* size: 128, cachelines: 2, members: 47 */
} __attribute__((__packed__));
struct drm_dmi_panel_orientation_data {
	int                        width;                /*     0     4 */
	int                        height;               /*     4     4 */
	const char  * const *      bios_dates;           /*     8     8 */
	int                        orientation;          /*    16     4 */

	/* size: 24, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct vga_device {
	struct list_head           list;                 /*     0    16 */
	struct pci_dev *           pdev;                 /*    16     8 */
	unsigned int               decodes;              /*    24     4 */
	unsigned int               owns;                 /*    28     4 */
	unsigned int               locks;                /*    32     4 */
	unsigned int               io_lock_cnt;          /*    36     4 */
	unsigned int               mem_lock_cnt;         /*    40     4 */
	unsigned int               io_norm_cnt;          /*    44     4 */
	unsigned int               mem_norm_cnt;         /*    48     4 */
	bool                       bridge_has_one_vga;   /*    52     1 */

	/* XXX 3 bytes hole, try to pack */

	void *                     cookie;               /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	void                       (*irq_set_state)(void *, bool); /*    64     8 */
	unsigned int               (*set_vga_decode)(void *, bool); /*    72     8 */

	/* size: 80, cachelines: 2, members: 13 */
	/* sum members: 77, holes: 1, sum holes: 3 */
	/* last cacheline: 16 bytes */
};
struct vga_arb_user_card {
	struct pci_dev *           pdev;                 /*     0     8 */
	unsigned int               mem_cnt;              /*     8     4 */
	unsigned int               io_cnt;               /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct vga_arb_private {
	struct list_head           list;                 /*     0    16 */
	struct pci_dev *           target;               /*    16     8 */
	struct vga_arb_user_card   cards[64];            /*    24  1024 */
	/* --- cacheline 16 boundary (1024 bytes) was 24 bytes ago --- */
	spinlock_t                 lock;                 /*  1048     4 */

	/* size: 1056, cachelines: 17, members: 4 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct vga_switcheroo_handler {
	int                        (*init)(void);        /*     0     8 */
	int                        (*switchto)(enum vga_switcheroo_client_id); /*     8     8 */
	int                        (*switch_ddc)(enum vga_switcheroo_client_id); /*    16     8 */
	int                        (*power_state)(enum vga_switcheroo_client_id, enum vga_switcheroo_state); /*    24     8 */
	enum vga_switcheroo_client_id (*get_client_id)(struct pci_dev *); /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct vga_switcheroo_client_ops {
	void                       (*set_gpu_state)(struct pci_dev *, enum vga_switcheroo_state); /*     0     8 */
	void                       (*reprobe)(struct pci_dev *); /*     8     8 */
	bool                       (*can_switch)(struct pci_dev *); /*    16     8 */
	void                       (*gpu_bound)(struct pci_dev *, enum vga_switcheroo_client_id); /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct vga_switcheroo_client {
	struct pci_dev *           pdev;                 /*     0     8 */
	struct fb_info *           fb_info;              /*     8     8 */
	enum vga_switcheroo_state  pwr_state;            /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	const struct vga_switcheroo_client_ops  * ops;   /*    24     8 */
	enum vga_switcheroo_client_id id;                /*    32     4 */
	bool                       active;               /*    36     1 */
	bool                       driver_power_control; /*    37     1 */

	/* XXX 2 bytes hole, try to pack */

	struct list_head           list;                 /*    40    16 */
	struct pci_dev *           vga_dev;              /*    56     8 */

	/* size: 64, cachelines: 1, members: 9 */
	/* sum members: 58, holes: 2, sum holes: 6 */
};
struct vgasr_priv {
	bool                       active;               /*     0     1 */
	bool                       delayed_switch_active; /*     1     1 */

	/* XXX 2 bytes hole, try to pack */

	enum vga_switcheroo_client_id delayed_client_id; /*     4     4 */
	struct dentry *            debugfs_root;         /*     8     8 */
	int                        registered_clients;   /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           clients;              /*    24    16 */
	const struct vga_switcheroo_handler  * handler;  /*    40     8 */
	enum vga_switcheroo_handler_flags_t handler_flags; /*    48     4 */

	/* XXX 4 bytes hole, try to pack */

	struct mutex               mux_hw_lock;          /*    56    32 */
	/* --- cacheline 1 boundary (64 bytes) was 24 bytes ago --- */
	int                        old_ddc_owner;        /*    88     4 */

	/* size: 96, cachelines: 2, members: 10 */
	/* sum members: 82, holes: 3, sum holes: 10 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct cb_id {
	__u32                      idx;                  /*     0     4 */
	__u32                      val;                  /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct cn_msg {
	struct cb_id               id;                   /*     0     8 */
	__u32                      seq;                  /*     8     4 */
	__u32                      ack;                  /*    12     4 */
	__u16                      len;                  /*    16     2 */
	__u16                      flags;                /*    18     2 */
	__u8                       data[];               /*    20     0 */

	/* size: 20, cachelines: 1, members: 6 */
	/* last cacheline: 20 bytes */
};
struct cn_queue_dev {
	atomic_t                   refcnt;               /*     0     4 */
	unsigned char              name[32];             /*     4    32 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           queue_list;           /*    40    16 */
	spinlock_t                 queue_lock;           /*    56     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	struct sock *              nls;                  /*    64     8 */

	/* size: 72, cachelines: 2, members: 5 */
	/* sum members: 64, holes: 2, sum holes: 8 */
	/* last cacheline: 8 bytes */
};
struct cn_callback_id {
	unsigned char              name[32];             /*     0    32 */
	struct cb_id               id;                   /*    32     8 */

	/* size: 40, cachelines: 1, members: 2 */
	/* last cacheline: 40 bytes */
};
struct cn_callback_entry {
	struct list_head           callback_entry;       /*     0    16 */
	refcount_t                 refcnt;               /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	struct cn_queue_dev *      pdev;                 /*    24     8 */
	struct cn_callback_id      id;                   /*    32    40 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	void                       (*callback)(struct cn_msg *, struct netlink_skb_parms *); /*    72     8 */
	u32                        seq;                  /*    80     4 */
	u32                        group;                /*    84     4 */

	/* size: 88, cachelines: 2, members: 7 */
	/* sum members: 84, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct cn_dev {
	struct cb_id               id;                   /*     0     8 */
	u32                        seq;                  /*     8     4 */
	u32                        groups;               /*    12     4 */
	struct sock *              nls;                  /*    16     8 */
	struct cn_queue_dev *      cbdev;                /*    24     8 */

	/* size: 32, cachelines: 1, members: 5 */
	/* last cacheline: 32 bytes */
};
struct fork_proc_event {
	__kernel_pid_t             parent_pid;           /*     0     4 */
	__kernel_pid_t             parent_tgid;          /*     4     4 */
	__kernel_pid_t             child_pid;            /*     8     4 */
	__kernel_pid_t             child_tgid;           /*    12     4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct exec_proc_event {
	__kernel_pid_t             process_pid;          /*     0     4 */
	__kernel_pid_t             process_tgid;         /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct id_proc_event {
	__kernel_pid_t             process_pid;          /*     0     4 */
	__kernel_pid_t             process_tgid;         /*     4     4 */
	union {
		__u32              ruid;                 /*     8     4 */
		__u32              rgid;                 /*     8     4 */
	} r;                                             /*     8     4 */
	union {
		__u32              euid;                 /*    12     4 */
		__u32              egid;                 /*    12     4 */
	} e;                                             /*    12     4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct sid_proc_event {
	__kernel_pid_t             process_pid;          /*     0     4 */
	__kernel_pid_t             process_tgid;         /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct ptrace_proc_event {
	__kernel_pid_t             process_pid;          /*     0     4 */
	__kernel_pid_t             process_tgid;         /*     4     4 */
	__kernel_pid_t             tracer_pid;           /*     8     4 */
	__kernel_pid_t             tracer_tgid;          /*    12     4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct comm_proc_event {
	__kernel_pid_t             process_pid;          /*     0     4 */
	__kernel_pid_t             process_tgid;         /*     4     4 */
	char                       comm[16];             /*     8    16 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct coredump_proc_event {
	__kernel_pid_t             process_pid;          /*     0     4 */
	__kernel_pid_t             process_tgid;         /*     4     4 */
	__kernel_pid_t             parent_pid;           /*     8     4 */
	__kernel_pid_t             parent_tgid;          /*    12     4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct exit_proc_event {
	__kernel_pid_t             process_pid;          /*     0     4 */
	__kernel_pid_t             process_tgid;         /*     4     4 */
	__u32                      exit_code;            /*     8     4 */
	__u32                      exit_signal;          /*    12     4 */
	__kernel_pid_t             parent_pid;           /*    16     4 */
	__kernel_pid_t             parent_tgid;          /*    20     4 */

	/* size: 24, cachelines: 1, members: 6 */
	/* last cacheline: 24 bytes */
};
struct proc_event {
	enum what                  what;                 /*     0     4 */
	__u32                      cpu;                  /*     4     4 */
	__u64                      timestamp_ns __attribute__((__aligned__(8))); /*     8     8 */
	union {
		struct {
			__u32      err;                  /*    16     4 */
		} ack;                                   /*    16     4 */
		struct fork_proc_event fork;             /*    16    16 */
		struct exec_proc_event exec;             /*    16     8 */
		struct id_proc_event id;                 /*    16    16 */
		struct sid_proc_event sid;               /*    16     8 */
		struct ptrace_proc_event ptrace;         /*    16    16 */
		struct comm_proc_event comm;             /*    16    24 */
		struct coredump_proc_event coredump;     /*    16    16 */
		struct exit_proc_event exit;             /*    16    24 */
	} event_data;                                    /*    16    24 */

	/* size: 40, cachelines: 1, members: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 40 bytes */
} __attribute__((__aligned__(8)));
struct local_event {
	local_lock_t               lock;                 /*     0     0 */
	__u32                      count;                /*     0     4 */

	/* size: 4, cachelines: 1, members: 2 */
	/* last cacheline: 4 bytes */
};
struct component_ops {
	int                        (*bind)(struct device *, struct device *, void *); /*     0     8 */
	void                       (*unbind)(struct device *, struct device *, void *); /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct component_master_ops {
	int                        (*bind)(struct device *); /*     0     8 */
	void                       (*unbind)(struct device *); /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct component_match_array {
	void *                     data;                 /*     0     8 */
	int                        (*compare)(struct device *, void *); /*     8     8 */
	int                        (*compare_typed)(struct device *, int, void *); /*    16     8 */
	void                       (*release)(struct device *, void *); /*    24     8 */
	struct component *         component;            /*    32     8 */
	bool                       duplicate;            /*    40     1 */

	/* size: 48, cachelines: 1, members: 6 */
	/* padding: 7 */
	/* last cacheline: 48 bytes */
};
struct component {
	struct list_head           node;                 /*     0    16 */
	struct master *            master;               /*    16     8 */
	bool                       bound;                /*    24     1 */

	/* XXX 7 bytes hole, try to pack */

	const struct component_ops  * ops;               /*    32     8 */
	int                        subcomponent;         /*    40     4 */

	/* XXX 4 bytes hole, try to pack */

	struct device *            dev;                  /*    48     8 */

	/* size: 56, cachelines: 1, members: 6 */
	/* sum members: 45, holes: 2, sum holes: 11 */
	/* last cacheline: 56 bytes */
};
struct component_match {
	size_t                     alloc;                /*     0     8 */
	size_t                     num;                  /*     8     8 */
	struct component_match_array * compare;          /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct master {
	struct list_head           node;                 /*     0    16 */
	bool                       bound;                /*    16     1 */

	/* XXX 7 bytes hole, try to pack */

	const struct component_master_ops  * ops;        /*    24     8 */
	struct device *            dev;                  /*    32     8 */
	struct component_match *   match;                /*    40     8 */
	struct dentry *            dentry;               /*    48     8 */

	/* size: 56, cachelines: 1, members: 6 */
	/* sum members: 49, holes: 1, sum holes: 7 */
	/* last cacheline: 56 bytes */
};
struct wake_irq {
	struct device *            dev;                  /*     0     8 */
	unsigned int               status;               /*     8     4 */
	int                        irq;                  /*    12     4 */
	const char  *              name;                 /*    16     8 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct subsys_private {
	struct kset                subsys;               /*     0   160 */
	/* --- cacheline 2 boundary (128 bytes) was 32 bytes ago --- */
	struct kset *              devices_kset;         /*   160     8 */
	struct list_head           interfaces;           /*   168    16 */
	struct mutex               mutex;                /*   184    32 */
	/* --- cacheline 3 boundary (192 bytes) was 24 bytes ago --- */
	struct kset *              drivers_kset;         /*   216     8 */
	struct klist               klist_devices __attribute__((__aligned__(8))); /*   224    40 */
	/* --- cacheline 4 boundary (256 bytes) was 8 bytes ago --- */
	struct klist               klist_drivers __attribute__((__aligned__(8))); /*   264    40 */
	struct blocking_notifier_head bus_notifier;      /*   304    48 */
	/* --- cacheline 5 boundary (320 bytes) was 32 bytes ago --- */
	unsigned int               drivers_autoprobe:1;  /*   352: 0  4 */

	/* XXX 31 bits hole, try to pack */
	/* XXX 4 bytes hole, try to pack */

	struct bus_type *          bus;                  /*   360     8 */
	struct kset                glue_dirs;            /*   368   160 */
	/* --- cacheline 8 boundary (512 bytes) was 16 bytes ago --- */
	struct class *             class;                /*   528     8 */

	/* size: 536, cachelines: 9, members: 12 */
	/* sum members: 528, holes: 1, sum holes: 4 */
	/* sum bitfield members: 1 bits, bit holes: 1, sum bit holes: 31 bits */
	/* forced alignments: 2 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct driver_private {
	struct kobject             kobj;                 /*     0    96 */
	/* --- cacheline 1 boundary (64 bytes) was 32 bytes ago --- */
	struct klist               klist_devices __attribute__((__aligned__(8))); /*    96    40 */
	/* --- cacheline 2 boundary (128 bytes) was 8 bytes ago --- */
	struct klist_node          knode_bus;            /*   136    32 */

	/* XXX last struct has 4 bytes of padding */

	struct module_kobject *    mkobj;                /*   168     8 */
	struct device_driver *     driver;               /*   176     8 */

	/* size: 184, cachelines: 3, members: 5 */
	/* paddings: 1, sum paddings: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 56 bytes */
} __attribute__((__aligned__(8)));
struct device_private {
	struct klist               klist_children __attribute__((__aligned__(8))); /*     0    40 */
	struct klist_node          knode_parent;         /*    40    32 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	struct klist_node          knode_driver;         /*    72    32 */

	/* XXX last struct has 4 bytes of padding */

	struct klist_node          knode_bus;            /*   104    32 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 2 boundary (128 bytes) was 8 bytes ago --- */
	struct klist_node          knode_class;          /*   136    32 */

	/* XXX last struct has 4 bytes of padding */

	struct list_head           deferred_probe;       /*   168    16 */
	struct device_driver *     async_driver;         /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	char *                     deferred_probe_reason; /*   192     8 */
	struct device *            device;               /*   200     8 */
	u8                         dead:1;               /*   208: 0  1 */

	/* size: 216, cachelines: 4, members: 10 */
	/* padding: 7 */
	/* paddings: 4, sum paddings: 16 */
	/* bit_padding: 7 bits */
	/* forced alignments: 1 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
union device_attr_group_devres {
	const struct attribute_group  * group;         /*     0     8 */
	const struct attribute_group  * * groups;      /*     0     8 */
};
struct class_dir {
	struct kobject             kobj;                 /*     0    96 */
	/* --- cacheline 1 boundary (64 bytes) was 32 bytes ago --- */
	struct class *             class;                /*    96     8 */

	/* size: 104, cachelines: 2, members: 2 */
	/* last cacheline: 40 bytes */
};
struct root_device {
	struct device              dev __attribute__((__aligned__(8))); /*     0  1000 */
	/* --- cacheline 15 boundary (960 bytes) was 40 bytes ago --- */
	struct module *            owner;                /*  1000     8 */

	/* size: 1008, cachelines: 16, members: 2 */
	/* forced alignments: 1 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct subsys_dev_iter {
	struct klist_iter          ki;                   /*     0    16 */
	const struct device_type  * type;                /*    16     8 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct device_attach_data {
	struct device *            dev;                  /*     0     8 */
	bool                       check_async;          /*     8     1 */
	bool                       want_async;           /*     9     1 */
	bool                       have_async;           /*    10     1 */

	/* size: 16, cachelines: 1, members: 4 */
	/* padding: 5 */
	/* last cacheline: 16 bytes */
};
struct class_attribute_string {
	struct class_attribute     attr;                 /*     0    32 */
	char *                     str;                  /*    32     8 */

	/* size: 40, cachelines: 1, members: 2 */
	/* last cacheline: 40 bytes */
};
struct class_compat {
	struct kobject *           kobj;                 /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct irq_affinity_devres {
	unsigned int               count;                /*     0     4 */
	unsigned int               irq[];                /*     4     0 */

	/* size: 4, cachelines: 1, members: 2 */
	/* last cacheline: 4 bytes */
};
struct platform_object {
	struct platform_device     pdev __attribute__((__aligned__(8))); /*     0  1080 */
	/* --- cacheline 16 boundary (1024 bytes) was 56 bytes ago --- */
	char                       name[];               /*  1080     0 */

	/* size: 1080, cachelines: 17, members: 2 */
	/* forced alignments: 1 */
	/* last cacheline: 56 bytes */
} __attribute__((__aligned__(8)));
struct cpu_attr {
	struct device_attribute    attr;                 /*     0    32 */
	const const struct cpumask  * map;               /*    32     8 */

	/* size: 40, cachelines: 1, members: 2 */
	/* last cacheline: 40 bytes */
};
struct probe {
	struct probe *             next;                 /*     0     8 */
	dev_t                      dev;                  /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          range;                /*    16     8 */
	struct module *            owner;                /*    24     8 */
	kobj_probe_t *             get;                  /*    32     8 */
	int                        (*lock)(dev_t, void *); /*    40     8 */
	void *                     data;                 /*    48     8 */

	/* size: 56, cachelines: 1, members: 7 */
	/* sum members: 52, holes: 1, sum holes: 4 */
	/* last cacheline: 56 bytes */
};
struct kobj_map {
	struct probe *             probes[255];          /*     0  2040 */
	/* --- cacheline 31 boundary (1984 bytes) was 56 bytes ago --- */
	struct mutex *             lock;                 /*  2040     8 */

	/* size: 2048, cachelines: 32, members: 2 */
};
struct devres_node {
	struct list_head           entry;                /*     0    16 */
	dr_release_t               release;              /*    16     8 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct devres {
	struct devres_node         node;                 /*     0    24 */
	u8                         data[] __attribute__((__aligned__(8))); /*    24     0 */

	/* size: 24, cachelines: 1, members: 2 */
	/* forced alignments: 1 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct devres_group {
	struct devres_node         node[2];              /*     0    48 */
	void *                     id;                   /*    48     8 */
	int                        color;                /*    56     4 */

	/* size: 64, cachelines: 1, members: 3 */
	/* padding: 4 */
};
struct action_devres {
	void *                     data;                 /*     0     8 */
	void                       (*action)(void *);    /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct pages_devres {
	long unsigned int          addr;                 /*     0     8 */
	unsigned int               order;                /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct attribute_container {
	struct list_head           node;                 /*     0    16 */
	struct klist               containers __attribute__((__aligned__(8))); /*    16    40 */
	struct class *             class;                /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	const struct attribute_group  * grp;             /*    64     8 */
	struct device_attribute * * attrs;               /*    72     8 */
	int                        (*match)(struct attribute_container *, struct device *); /*    80     8 */
	long unsigned int          flags;                /*    88     8 */

	/* size: 96, cachelines: 2, members: 7 */
	/* forced alignments: 1 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct internal_container {
	struct klist_node          node;                 /*     0    32 */

	/* XXX last struct has 4 bytes of padding */

	struct attribute_container * cont;               /*    32     8 */
	struct device              classdev __attribute__((__aligned__(8))); /*    40  1000 */

	/* size: 1040, cachelines: 17, members: 3 */
	/* paddings: 1, sum paddings: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct transport_class {
	struct class               class;                /*     0   168 */
	/* --- cacheline 2 boundary (128 bytes) was 40 bytes ago --- */
	int                        (*setup)(struct transport_container *, struct device *, struct device *); /*   168     8 */
	int                        (*configure)(struct transport_container *, struct device *, struct device *); /*   176     8 */
	int                        (*remove)(struct transport_container *, struct device *, struct device *); /*   184     8 */

	/* size: 192, cachelines: 3, members: 4 */
};
struct transport_container {
	struct attribute_container ac __attribute__((__aligned__(8))); /*     0    96 */
	/* --- cacheline 1 boundary (64 bytes) was 32 bytes ago --- */
	const struct attribute_group  * statistics;      /*    96     8 */

	/* size: 104, cachelines: 2, members: 2 */
	/* forced alignments: 1 */
	/* last cacheline: 40 bytes */
} __attribute__((__aligned__(8)));
struct anon_transport_class {
	struct transport_class     tclass;               /*     0   192 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	struct attribute_container container __attribute__((__aligned__(8))); /*   192    96 */

	/* size: 288, cachelines: 5, members: 2 */
	/* forced alignments: 1 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct phy_device {
	struct mdio_device         mdio __attribute__((__aligned__(8))); /*     0  1096 */
	/* --- cacheline 17 boundary (1088 bytes) was 8 bytes ago --- */
	struct phy_driver *        drv;                  /*  1096     8 */
	u32                        phy_id;               /*  1104     4 */
	struct phy_c45_device_ids  c45_ids;              /*  1108   136 */
	/* --- cacheline 19 boundary (1216 bytes) was 28 bytes ago --- */
	unsigned int               is_c45:1;             /*  1244: 0  4 */
	unsigned int               is_internal:1;        /*  1244: 1  4 */
	unsigned int               is_pseudo_fixed_link:1; /*  1244: 2  4 */
	unsigned int               is_gigabit_capable:1; /*  1244: 3  4 */
	unsigned int               has_fixups:1;         /*  1244: 4  4 */
	unsigned int               suspended:1;          /*  1244: 5  4 */
	unsigned int               suspended_by_mdio_bus:1; /*  1244: 6  4 */
	unsigned int               sysfs_links:1;        /*  1244: 7  4 */
	unsigned int               loopback_enabled:1;   /*  1244: 8  4 */
	unsigned int               downshifted_rate:1;   /*  1244: 9  4 */
	unsigned int               autoneg:1;            /*  1244:10  4 */
	unsigned int               link:1;               /*  1244:11  4 */
	unsigned int               autoneg_complete:1;   /*  1244:12  4 */
	unsigned int               interrupts:1;         /*  1244:13  4 */

	/* XXX 18 bits hole, try to pack */

	enum phy_state             state;                /*  1248     4 */
	u32                        dev_flags;            /*  1252     4 */
	phy_interface_t            interface;            /*  1256     4 */
	int                        speed;                /*  1260     4 */
	int                        duplex;               /*  1264     4 */
	int                        port;                 /*  1268     4 */
	int                        pause;                /*  1272     4 */
	int                        asym_pause;           /*  1276     4 */
	/* --- cacheline 20 boundary (1280 bytes) --- */
	u8                         master_slave_get;     /*  1280     1 */
	u8                         master_slave_set;     /*  1281     1 */
	u8                         master_slave_state;   /*  1282     1 */

	/* XXX 5 bytes hole, try to pack */

	long unsigned int          supported[2];         /*  1288    16 */
	long unsigned int          advertising[2];       /*  1304    16 */
	long unsigned int          lp_advertising[2];    /*  1320    16 */
	long unsigned int          adv_old[2];           /*  1336    16 */
	/* --- cacheline 21 boundary (1344 bytes) was 8 bytes ago --- */
	u32                        eee_broken_modes;     /*  1352     4 */

	/* XXX 4 bytes hole, try to pack */

	struct phy_led_trigger *   phy_led_triggers;     /*  1360     8 */
	unsigned int               phy_num_led_triggers; /*  1368     4 */

	/* XXX 4 bytes hole, try to pack */

	struct phy_led_trigger *   last_triggered;       /*  1376     8 */
	struct phy_led_trigger *   led_link_trigger;     /*  1384     8 */
	int                        irq;                  /*  1392     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     priv;                 /*  1400     8 */
	/* --- cacheline 22 boundary (1408 bytes) --- */
	struct phy_package_shared * shared;              /*  1408     8 */
	struct sk_buff *           skb;                  /*  1416     8 */
	void *                     ehdr;                 /*  1424     8 */
	struct nlattr *            nest;                 /*  1432     8 */
	struct delayed_work        state_queue;          /*  1440   192 */
	/* --- cacheline 25 boundary (1600 bytes) was 32 bytes ago --- */
	struct mutex               lock;                 /*  1632    32 */
	/* --- cacheline 26 boundary (1664 bytes) --- */
	bool                       sfp_bus_attached;     /*  1664     1 */

	/* XXX 7 bytes hole, try to pack */

	struct sfp_bus *           sfp_bus;              /*  1672     8 */
	struct phylink *           phylink;              /*  1680     8 */
	struct net_device *        attached_dev;         /*  1688     8 */
	struct mii_timestamper *   mii_ts;               /*  1696     8 */
	u8                         mdix;                 /*  1704     1 */
	u8                         mdix_ctrl;            /*  1705     1 */

	/* XXX 6 bytes hole, try to pack */

	void                       (*phy_link_change)(struct phy_device *, bool); /*  1712     8 */
	void                       (*adjust_link)(struct net_device *); /*  1720     8 */
	/* --- cacheline 27 boundary (1728 bytes) --- */
	const struct macsec_ops  * macsec_ops;           /*  1728     8 */

	/* size: 1736, cachelines: 28, members: 56 */
	/* sum members: 1702, holes: 6, sum holes: 30 */
	/* sum bitfield members: 14 bits, bit holes: 1, sum bit holes: 18 bits */
	/* forced alignments: 1 */
	/* last cacheline: 8 bytes */
} __attribute__((__aligned__(8)));
struct phy_tdr_config {
	u32                        first;                /*     0     4 */
	u32                        last;                 /*     4     4 */
	u32                        step;                 /*     8     4 */
	s8                         pair;                 /*    12     1 */

	/* size: 16, cachelines: 1, members: 4 */
	/* padding: 3 */
	/* last cacheline: 16 bytes */
};
struct mdio_device {
	struct device              dev __attribute__((__aligned__(8))); /*     0  1000 */
	/* --- cacheline 15 boundary (960 bytes) was 40 bytes ago --- */
	struct mii_bus *           bus;                  /*  1000     8 */
	char                       modalias[32];         /*  1008    32 */
	/* --- cacheline 16 boundary (1024 bytes) was 16 bytes ago --- */
	int                        (*bus_match)(struct device *, struct device_driver *); /*  1040     8 */
	void                       (*device_free)(struct mdio_device *); /*  1048     8 */
	void                       (*device_remove)(struct mdio_device *); /*  1056     8 */
	int                        addr;                 /*  1064     4 */
	int                        flags;                /*  1068     4 */
	struct gpio_desc *         reset_gpio;           /*  1072     8 */
	struct reset_control *     reset_ctrl;           /*  1080     8 */
	/* --- cacheline 17 boundary (1088 bytes) --- */
	unsigned int               reset_assert_delay;   /*  1088     4 */
	unsigned int               reset_deassert_delay; /*  1092     4 */

	/* size: 1096, cachelines: 18, members: 12 */
	/* forced alignments: 1 */
	/* last cacheline: 8 bytes */
} __attribute__((__aligned__(8)));
struct mii_bus {
	struct module *            owner;                /*     0     8 */
	const char  *              name;                 /*     8     8 */
	char                       id[61];               /*    16    61 */

	/* XXX 3 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
	void *                     priv;                 /*    80     8 */
	int                        (*read)(struct mii_bus *, int, int); /*    88     8 */
	int                        (*write)(struct mii_bus *, int, int, u16); /*    96     8 */
	int                        (*reset)(struct mii_bus *); /*   104     8 */
	struct mdio_bus_stats      stats[32];            /*   112  1024 */
	/* --- cacheline 17 boundary (1088 bytes) was 48 bytes ago --- */
	struct mutex               mdio_lock;            /*  1136    32 */
	/* --- cacheline 18 boundary (1152 bytes) was 16 bytes ago --- */
	struct device *            parent;               /*  1168     8 */
	enum {
		MDIOBUS_ALLOCATED    = 1,
		MDIOBUS_REGISTERED   = 2,
		MDIOBUS_UNREGISTERED = 3,
		MDIOBUS_RELEASED     = 4,
	} state;                                         /*  1176     4 */

	/* XXX 4 bytes hole, try to pack */

	struct device              dev __attribute__((__aligned__(8))); /*  1184  1000 */
	/* --- cacheline 34 boundary (2176 bytes) was 8 bytes ago --- */
	struct mdio_device *       mdio_map[32];         /*  2184   256 */
	/* --- cacheline 38 boundary (2432 bytes) was 8 bytes ago --- */
	u32                        phy_mask;             /*  2440     4 */
	u32                        phy_ignore_ta_mask;   /*  2444     4 */
	int                        irq[32];              /*  2448   128 */
	/* --- cacheline 40 boundary (2560 bytes) was 16 bytes ago --- */
	int                        reset_delay_us;       /*  2576     4 */
	int                        reset_post_delay_us;  /*  2580     4 */
	struct gpio_desc *         reset_gpiod;          /*  2584     8 */
	enum {
		MDIOBUS_NO_CAP  = 0,
		MDIOBUS_C22     = 1,
		MDIOBUS_C45     = 2,
		MDIOBUS_C22_C45 = 3,
	} probe_capabilities;                            /*  2592     4 */

	/* XXX 4 bytes hole, try to pack */

	struct mutex               shared_lock;          /*  2600    32 */
	/* --- cacheline 41 boundary (2624 bytes) was 8 bytes ago --- */
	struct phy_package_shared * shared[32];          /*  2632   256 */

	/* size: 2888, cachelines: 46, members: 22 */
	/* sum members: 2877, holes: 3, sum holes: 11 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 4 */
	/* last cacheline: 8 bytes */
} __attribute__((__aligned__(8)));
struct mdio_driver_common {
	struct device_driver       driver;               /*     0   192 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	int                        flags;                /*   192     4 */

	/* size: 200, cachelines: 4, members: 2 */
	/* padding: 4 */
	/* last cacheline: 8 bytes */
};
struct mii_timestamper {
	bool                       (*rxtstamp)(struct mii_timestamper *, struct sk_buff *, int); /*     0     8 */
	void                       (*txtstamp)(struct mii_timestamper *, struct sk_buff *, int); /*     8     8 */
	int                        (*hwtstamp)(struct mii_timestamper *, struct ifreq *); /*    16     8 */
	void                       (*link_state)(struct mii_timestamper *, struct phy_device *); /*    24     8 */
	int                        (*ts_info)(struct mii_timestamper *, struct ethtool_ts_info *); /*    32     8 */
	struct device *            device;               /*    40     8 */

	/* size: 48, cachelines: 1, members: 6 */
	/* last cacheline: 48 bytes */
};
struct mdio_bus_stats {
	u64_stats_t                transfers;            /*     0     8 */
	u64_stats_t                errors;               /*     8     8 */
	u64_stats_t                writes;               /*    16     8 */
	u64_stats_t                reads;                /*    24     8 */
	struct u64_stats_sync      syncp;                /*    32     0 */

	/* size: 32, cachelines: 1, members: 5 */
	/* last cacheline: 32 bytes */
};
struct phy_package_shared {
	int                        addr;                 /*     0     4 */
	refcount_t                 refcnt;               /*     4     4 */
	long unsigned int          flags;                /*     8     8 */
	size_t                     priv_size;            /*    16     8 */
	void *                     priv;                 /*    24     8 */

	/* size: 32, cachelines: 1, members: 5 */
	/* last cacheline: 32 bytes */
};
struct phy_c45_device_ids {
	u32                        devices_in_package;   /*     0     4 */
	u32                        mmds_present;         /*     4     4 */
	u32                        device_ids[32];       /*     8   128 */

	/* size: 136, cachelines: 3, members: 3 */
	/* last cacheline: 8 bytes */
};
struct phy_driver {
	struct mdio_driver_common  mdiodrv;              /*     0   200 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 3 boundary (192 bytes) was 8 bytes ago --- */
	u32                        phy_id;               /*   200     4 */

	/* XXX 4 bytes hole, try to pack */

	char *                     name;                 /*   208     8 */
	u32                        phy_id_mask;          /*   216     4 */

	/* XXX 4 bytes hole, try to pack */

	const const long unsigned int  * features;       /*   224     8 */
	u32                        flags;                /*   232     4 */

	/* XXX 4 bytes hole, try to pack */

	const void  *              driver_data;          /*   240     8 */
	int                        (*soft_reset)(struct phy_device *); /*   248     8 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	int                        (*config_init)(struct phy_device *); /*   256     8 */
	int                        (*probe)(struct phy_device *); /*   264     8 */
	int                        (*get_features)(struct phy_device *); /*   272     8 */
	int                        (*suspend)(struct phy_device *); /*   280     8 */
	int                        (*resume)(struct phy_device *); /*   288     8 */
	int                        (*config_aneg)(struct phy_device *); /*   296     8 */
	int                        (*aneg_done)(struct phy_device *); /*   304     8 */
	int                        (*read_status)(struct phy_device *); /*   312     8 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	int                        (*ack_interrupt)(struct phy_device *); /*   320     8 */
	int                        (*config_intr)(struct phy_device *); /*   328     8 */
	int                        (*did_interrupt)(struct phy_device *); /*   336     8 */
	irqreturn_t                (*handle_interrupt)(struct phy_device *); /*   344     8 */
	void                       (*remove)(struct phy_device *); /*   352     8 */
	int                        (*match_phy_device)(struct phy_device *); /*   360     8 */
	int                        (*set_wol)(struct phy_device *, struct ethtool_wolinfo *); /*   368     8 */
	void                       (*get_wol)(struct phy_device *, struct ethtool_wolinfo *); /*   376     8 */
	/* --- cacheline 6 boundary (384 bytes) --- */
	void                       (*link_change_notify)(struct phy_device *); /*   384     8 */
	int                        (*read_mmd)(struct phy_device *, int, u16); /*   392     8 */
	int                        (*write_mmd)(struct phy_device *, int, u16, u16); /*   400     8 */
	int                        (*read_page)(struct phy_device *); /*   408     8 */
	int                        (*write_page)(struct phy_device *, int); /*   416     8 */
	int                        (*module_info)(struct phy_device *, struct ethtool_modinfo *); /*   424     8 */
	int                        (*module_eeprom)(struct phy_device *, struct ethtool_eeprom *, u8 *); /*   432     8 */
	int                        (*cable_test_start)(struct phy_device *); /*   440     8 */
	/* --- cacheline 7 boundary (448 bytes) --- */
	int                        (*cable_test_tdr_start)(struct phy_device *, const struct phy_tdr_config  *); /*   448     8 */
	int                        (*cable_test_get_status)(struct phy_device *, bool *); /*   456     8 */
	int                        (*get_sset_count)(struct phy_device *); /*   464     8 */
	void                       (*get_strings)(struct phy_device *, u8 *); /*   472     8 */
	void                       (*get_stats)(struct phy_device *, struct ethtool_stats *, u64 *); /*   480     8 */
	int                        (*get_tunable)(struct phy_device *, struct ethtool_tunable *, void *); /*   488     8 */
	int                        (*set_tunable)(struct phy_device *, struct ethtool_tunable *, const void  *); /*   496     8 */
	int                        (*set_loopback)(struct phy_device *, bool); /*   504     8 */
	/* --- cacheline 8 boundary (512 bytes) --- */
	int                        (*get_sqi)(struct phy_device *); /*   512     8 */
	int                        (*get_sqi_max)(struct phy_device *); /*   520     8 */

	/* size: 528, cachelines: 9, members: 42 */
	/* sum members: 516, holes: 3, sum holes: 12 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 16 bytes */
};
struct software_node_ref_args {
	const struct software_node  * node;              /*     0     8 */
	unsigned int               nargs;                /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        args[8];              /*    16    64 */

	/* size: 80, cachelines: 2, members: 3 */
	/* sum members: 76, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct software_node {
	const char  *              name;                 /*     0     8 */
	const struct software_node  * parent;            /*     8     8 */
	const struct property_entry  * properties;       /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct swnode {
	int                        id;                   /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct kobject             kobj;                 /*     8    96 */
	/* --- cacheline 1 boundary (64 bytes) was 40 bytes ago --- */
	struct fwnode_handle       fwnode;               /*   104    40 */
	/* --- cacheline 2 boundary (128 bytes) was 16 bytes ago --- */
	const struct software_node  * node;              /*   144     8 */
	struct ida                 child_ids;            /*   152    16 */
	struct list_head           entry;                /*   168    16 */
	struct list_head           children;             /*   184    16 */
	/* --- cacheline 3 boundary (192 bytes) was 8 bytes ago --- */
	struct swnode *            parent;               /*   200     8 */
	unsigned int               allocated:1;          /*   208: 0  4 */

	/* size: 216, cachelines: 4, members: 9 */
	/* sum members: 204, holes: 1, sum holes: 4 */
	/* sum bitfield members: 1 bits (0 bytes) */
	/* padding: 4 */
	/* bit_padding: 31 bits */
	/* last cacheline: 24 bytes */
};
struct req {
	struct req *               next;                 /*     0     8 */
	struct completion          done;                 /*     8    32 */
	int                        err;                  /*    40     4 */

	/* XXX 4 bytes hole, try to pack */

	const char  *              name;                 /*    48     8 */
	umode_t                    mode;                 /*    56     2 */

	/* XXX 2 bytes hole, try to pack */

	kuid_t                     uid;                  /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	kgid_t                     gid;                  /*    64     4 */

	/* XXX 4 bytes hole, try to pack */

	struct device *            dev;                  /*    72     8 */

	/* size: 80, cachelines: 2, members: 8 */
	/* sum members: 70, holes: 3, sum holes: 10 */
	/* last cacheline: 16 bytes */
};
struct dev_power_governor {
	bool                       (*power_down_ok)(struct dev_pm_domain *); /*     0     8 */
	bool                       (*suspend_ok)(struct device *); /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct gpd_dev_ops {
	int                        (*start)(struct device *); /*     0     8 */
	int                        (*stop)(struct device *); /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct genpd_power_state {
	s64                        power_off_latency_ns; /*     0     8 */
	s64                        power_on_latency_ns;  /*     8     8 */
	s64                        residency_ns;         /*    16     8 */
	u64                        usage;                /*    24     8 */
	u64                        rejected;             /*    32     8 */
	struct fwnode_handle *     fwnode;               /*    40     8 */
	ktime_t                    idle_time;            /*    48     8 */
	void *                     data;                 /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u64                        kabi_reserved1;       /*    64     8 */
	u64                        kabi_reserved2;       /*    72     8 */

	/* size: 80, cachelines: 2, members: 10 */
	/* last cacheline: 16 bytes */
};
struct generic_pm_domain {
	struct device              dev __attribute__((__aligned__(8))); /*     0  1000 */
	/* --- cacheline 15 boundary (960 bytes) was 40 bytes ago --- */
	struct dev_pm_domain       domain;               /*  1000   240 */
	/* --- cacheline 19 boundary (1216 bytes) was 24 bytes ago --- */
	struct list_head           gpd_list_node;        /*  1240    16 */
	struct list_head           parent_links;         /*  1256    16 */
	struct list_head           child_links;          /*  1272    16 */
	/* --- cacheline 20 boundary (1280 bytes) was 8 bytes ago --- */
	struct list_head           dev_list;             /*  1288    16 */
	struct dev_power_governor * gov;                 /*  1304     8 */
	struct work_struct         power_off_work;       /*  1312    64 */
	/* --- cacheline 21 boundary (1344 bytes) was 32 bytes ago --- */
	struct fwnode_handle *     provider;             /*  1376     8 */
	bool                       has_provider;         /*  1384     1 */

	/* XXX 7 bytes hole, try to pack */

	const char  *              name;                 /*  1392     8 */
	atomic_t                   sd_count;             /*  1400     4 */
	enum gpd_status            status;               /*  1404     4 */
	/* --- cacheline 22 boundary (1408 bytes) --- */
	unsigned int               device_count;         /*  1408     4 */
	unsigned int               suspended_count;      /*  1412     4 */
	unsigned int               prepared_count;       /*  1416     4 */
	unsigned int               performance_state;    /*  1420     4 */
	cpumask_var_t              cpus;                 /*  1424     8 */
	int                        (*power_off)(struct generic_pm_domain *); /*  1432     8 */
	int                        (*power_on)(struct generic_pm_domain *); /*  1440     8 */
	struct raw_notifier_head   power_notifiers;      /*  1448     8 */
	struct opp_table *         opp_table;            /*  1456     8 */
	unsigned int               (*opp_to_performance_state)(struct generic_pm_domain *, struct dev_pm_opp *); /*  1464     8 */
	/* --- cacheline 23 boundary (1472 bytes) --- */
	int                        (*set_performance_state)(struct generic_pm_domain *, unsigned int); /*  1472     8 */
	struct gpd_dev_ops         dev_ops;              /*  1480    16 */
	s64                        max_off_time_ns;      /*  1496     8 */
	bool                       max_off_time_changed; /*  1504     1 */
	bool                       cached_power_down_ok; /*  1505     1 */
	bool                       cached_power_down_state_idx; /*  1506     1 */

	/* XXX 5 bytes hole, try to pack */

	int                        (*attach_dev)(struct generic_pm_domain *, struct device *); /*  1512     8 */
	void                       (*detach_dev)(struct generic_pm_domain *, struct device *); /*  1520     8 */
	unsigned int               flags;                /*  1528     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 24 boundary (1536 bytes) --- */
	struct genpd_power_state * states;               /*  1536     8 */
	void                       (*free_states)(struct genpd_power_state *, unsigned int); /*  1544     8 */
	unsigned int               state_count;          /*  1552     4 */
	unsigned int               state_idx;            /*  1556     4 */
	ktime_t                    on_time;              /*  1560     8 */
	ktime_t                    accounting_time;      /*  1568     8 */
	const struct genpd_lock_ops  * lock_ops;         /*  1576     8 */
	union {
		struct mutex       mlock;                /*  1584    32 */
		struct {
			spinlock_t slock;                /*  1584     4 */

			/* XXX 4 bytes hole, try to pack */

			long unsigned int lock_flags;    /*  1592     8 */
		};                                       /*  1584    16 */
	};                                               /*  1584    32 */
	/* --- cacheline 25 boundary (1600 bytes) was 16 bytes ago --- */
	u64                        kabi_reserved1;       /*  1616     8 */
	u64                        kabi_reserved2;       /*  1624     8 */
	u64                        kabi_reserved3;       /*  1632     8 */
	u64                        kabi_reserved4;       /*  1640     8 */

	/* size: 1648, cachelines: 26, members: 44 */
	/* sum members: 1632, holes: 3, sum holes: 16 */
	/* forced alignments: 1 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct genpd_lock_ops {
	void                       (*lock)(struct generic_pm_domain *); /*     0     8 */
	void                       (*lock_nested)(struct generic_pm_domain *, int); /*     8     8 */
	int                        (*lock_interruptible)(struct generic_pm_domain *); /*    16     8 */
	void                       (*unlock)(struct generic_pm_domain *); /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct gpd_link {
	struct generic_pm_domain * parent;               /*     0     8 */
	struct list_head           parent_node;          /*     8    16 */
	struct generic_pm_domain * child;                /*    24     8 */
	struct list_head           child_node;           /*    32    16 */
	unsigned int               performance_state;    /*    48     4 */
	unsigned int               prev_performance_state; /*    52     4 */

	/* size: 56, cachelines: 1, members: 6 */
	/* last cacheline: 56 bytes */
};
struct gpd_timing_data {
	s64                        suspend_latency_ns;   /*     0     8 */
	s64                        resume_latency_ns;    /*     8     8 */
	s64                        effective_constraint_ns; /*    16     8 */
	bool                       constraint_changed;   /*    24     1 */
	bool                       cached_suspend_ok;    /*    25     1 */

	/* size: 32, cachelines: 1, members: 5 */
	/* padding: 6 */
	/* last cacheline: 32 bytes */
};
struct generic_pm_domain_data {
	struct pm_domain_data      base;                 /*     0    24 */
	struct gpd_timing_data     td;                   /*    24    32 */

	/* XXX last struct has 6 bytes of padding */

	struct notifier_block      nb;                   /*    56    24 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
	struct notifier_block *    power_nb;             /*    80     8 */
	int                        cpu;                  /*    88     4 */
	unsigned int               performance_state;    /*    92     4 */
	void *                     data;                 /*    96     8 */

	/* size: 104, cachelines: 2, members: 7 */
	/* paddings: 2, sum paddings: 10 */
	/* last cacheline: 40 bytes */
};
struct pm_clk_notifier_block {
	struct notifier_block      nb;                   /*     0    24 */

	/* XXX last struct has 4 bytes of padding */

	struct dev_pm_domain *     pm_domain;            /*    24     8 */
	char *                     con_ids[];            /*    32     0 */

	/* size: 32, cachelines: 1, members: 3 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 32 bytes */
};
struct pm_clock_entry {
	struct list_head           node;                 /*     0    16 */
	char *                     con_id;               /*    16     8 */
	struct clk *               clk;                  /*    24     8 */
	enum pce_status            status;               /*    32     4 */

	/* size: 40, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct fw_state {
	struct completion          completion;           /*     0    32 */
	enum fw_status             status;               /*    32     4 */

	/* size: 40, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct fw_priv {
	struct kref                ref;                  /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           list;                 /*     8    16 */
	struct firmware_cache *    fwc;                  /*    24     8 */
	struct fw_state            fw_st;                /*    32    40 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	void *                     data;                 /*    72     8 */
	size_t                     size;                 /*    80     8 */
	size_t                     allocated_size;       /*    88     8 */
	size_t                     offset;               /*    96     8 */
	u32                        opt_flags;            /*   104     4 */

	/* XXX 4 bytes hole, try to pack */

	const char  *              fw_name;              /*   112     8 */

	/* size: 120, cachelines: 2, members: 10 */
	/* sum members: 112, holes: 2, sum holes: 8 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 56 bytes */
};
struct firmware_cache {
	spinlock_t                 lock;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           head;                 /*     8    16 */
	int                        state;                /*    24     4 */
	spinlock_t                 name_lock;            /*    28     4 */
	struct list_head           fw_names;             /*    32    16 */
	struct delayed_work        work;                 /*    48   192 */
	/* --- cacheline 3 boundary (192 bytes) was 48 bytes ago --- */
	struct notifier_block      pm_notify;            /*   240    24 */

	/* XXX last struct has 4 bytes of padding */

	/* size: 264, cachelines: 5, members: 7 */
	/* sum members: 260, holes: 1, sum holes: 4 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 8 bytes */
};
struct fw_cache_entry {
	struct list_head           list;                 /*     0    16 */
	const char  *              name;                 /*    16     8 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct fw_name_devm {
	long unsigned int          magic;                /*     0     8 */
	const char  *              name;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct firmware_work {
	struct work_struct         work;                 /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct module *            module;               /*    64     8 */
	const char  *              name;                 /*    72     8 */
	struct device *            device;               /*    80     8 */
	void *                     context;              /*    88     8 */
	void                       (*cont)(const struct firmware  *, void *); /*    96     8 */
	u32                        opt_flags;            /*   104     4 */

	/* size: 112, cachelines: 2, members: 7 */
	/* padding: 4 */
	/* last cacheline: 48 bytes */
};
struct node_access_nodes {
	struct device              dev __attribute__((__aligned__(8))); /*     0  1000 */
	/* --- cacheline 15 boundary (960 bytes) was 40 bytes ago --- */
	struct list_head           list_node;            /*  1000    16 */
	unsigned int               access;               /*  1016     4 */
	struct node_hmem_attrs     hmem_attrs;           /*  1020    16 */

	/* size: 1040, cachelines: 17, members: 4 */
	/* padding: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct node_cache_info {
	struct device              dev __attribute__((__aligned__(8))); /*     0  1000 */
	/* --- cacheline 15 boundary (960 bytes) was 40 bytes ago --- */
	struct list_head           node;                 /*  1000    16 */
	struct node_cache_attrs    cache_attrs;          /*  1016    24 */

	/* XXX last struct has 5 bytes of padding */

	/* size: 1040, cachelines: 17, members: 3 */
	/* paddings: 1, sum paddings: 5 */
	/* forced alignments: 1 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct node_attr {
	struct device_attribute    attr;                 /*     0    32 */
	enum node_states           state;                /*    32     4 */

	/* size: 40, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct for_each_memory_block_cb_data {
	walk_memory_blocks_func_t  func;                 /*     0     8 */
	void *                     arg;                  /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct reg_default {
	unsigned int               reg;                  /*     0     4 */
	unsigned int               def;                  /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct reg_sequence {
	unsigned int               reg;                  /*     0     4 */
	unsigned int               def;                  /*     4     4 */
	unsigned int               delay_us;             /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct regmap_range {
	unsigned int               range_min;            /*     0     4 */
	unsigned int               range_max;            /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct regmap_access_table {
	const struct regmap_range  * yes_ranges;         /*     0     8 */
	unsigned int               n_yes_ranges;         /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	const struct regmap_range  * no_ranges;          /*    16     8 */
	unsigned int               n_no_ranges;          /*    24     4 */

	/* size: 32, cachelines: 1, members: 4 */
	/* sum members: 24, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct regmap_config {
	const char  *              name;                 /*     0     8 */
	int                        reg_bits;             /*     8     4 */
	int                        reg_stride;           /*    12     4 */
	int                        pad_bits;             /*    16     4 */
	int                        val_bits;             /*    20     4 */
	bool                       (*writeable_reg)(struct device *, unsigned int); /*    24     8 */
	bool                       (*readable_reg)(struct device *, unsigned int); /*    32     8 */
	bool                       (*volatile_reg)(struct device *, unsigned int); /*    40     8 */
	bool                       (*precious_reg)(struct device *, unsigned int); /*    48     8 */
	bool                       (*writeable_noinc_reg)(struct device *, unsigned int); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	bool                       (*readable_noinc_reg)(struct device *, unsigned int); /*    64     8 */
	bool                       disable_locking;      /*    72     1 */

	/* XXX 7 bytes hole, try to pack */

	regmap_lock                lock;                 /*    80     8 */
	regmap_unlock              unlock;               /*    88     8 */
	void *                     lock_arg;             /*    96     8 */
	int                        (*reg_read)(void *, unsigned int, unsigned int *); /*   104     8 */
	int                        (*reg_write)(void *, unsigned int, unsigned int); /*   112     8 */
	bool                       fast_io;              /*   120     1 */

	/* XXX 3 bytes hole, try to pack */

	unsigned int               max_register;         /*   124     4 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	const struct regmap_access_table  * wr_table;    /*   128     8 */
	const struct regmap_access_table  * rd_table;    /*   136     8 */
	const struct regmap_access_table  * volatile_table; /*   144     8 */
	const struct regmap_access_table  * precious_table; /*   152     8 */
	const struct regmap_access_table  * wr_noinc_table; /*   160     8 */
	const struct regmap_access_table  * rd_noinc_table; /*   168     8 */
	const struct reg_default  * reg_defaults;        /*   176     8 */
	unsigned int               num_reg_defaults;     /*   184     4 */
	enum regcache_type         cache_type;           /*   188     4 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	const void  *              reg_defaults_raw;     /*   192     8 */
	unsigned int               num_reg_defaults_raw; /*   200     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          read_flag_mask;       /*   208     8 */
	long unsigned int          write_flag_mask;      /*   216     8 */
	bool                       zero_flag_mask;       /*   224     1 */
	bool                       use_single_read;      /*   225     1 */
	bool                       use_single_write;     /*   226     1 */
	bool                       can_multi_write;      /*   227     1 */
	enum regmap_endian         reg_format_endian;    /*   228     4 */
	enum regmap_endian         val_format_endian;    /*   232     4 */

	/* XXX 4 bytes hole, try to pack */

	const struct regmap_range_cfg  * ranges;         /*   240     8 */
	unsigned int               num_ranges;           /*   248     4 */
	bool                       use_hwlock;           /*   252     1 */

	/* XXX 3 bytes hole, try to pack */

	/* --- cacheline 4 boundary (256 bytes) --- */
	unsigned int               hwlock_id;            /*   256     4 */
	unsigned int               hwlock_mode;          /*   260     4 */
	bool                       can_sleep;            /*   264     1 */

	/* size: 272, cachelines: 5, members: 44 */
	/* sum members: 244, holes: 5, sum holes: 21 */
	/* padding: 7 */
	/* last cacheline: 16 bytes */
};
struct regmap_range_cfg {
	const char  *              name;                 /*     0     8 */
	unsigned int               range_min;            /*     8     4 */
	unsigned int               range_max;            /*    12     4 */
	unsigned int               selector_reg;         /*    16     4 */
	unsigned int               selector_mask;        /*    20     4 */
	int                        selector_shift;       /*    24     4 */
	unsigned int               window_start;         /*    28     4 */
	unsigned int               window_len;           /*    32     4 */

	/* size: 40, cachelines: 1, members: 8 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct regmap_async {
	struct list_head           list;                 /*     0    16 */
	struct regmap *            map;                  /*    16     8 */
	void *                     work_buf;             /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* last cacheline: 32 bytes */
};
struct regmap_bus {
	bool                       fast_io;              /*     0     1 */

	/* XXX 7 bytes hole, try to pack */

	regmap_hw_write            write;                /*     8     8 */
	regmap_hw_gather_write     gather_write;         /*    16     8 */
	regmap_hw_async_write      async_write;          /*    24     8 */
	regmap_hw_reg_write        reg_write;            /*    32     8 */
	regmap_hw_reg_update_bits  reg_update_bits;      /*    40     8 */
	regmap_hw_read             read;                 /*    48     8 */
	regmap_hw_reg_read         reg_read;             /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	regmap_hw_free_context     free_context;         /*    64     8 */
	regmap_hw_async_alloc      async_alloc;          /*    72     8 */
	u8                         read_flag_mask;       /*    80     1 */

	/* XXX 3 bytes hole, try to pack */

	enum regmap_endian         reg_format_endian_default; /*    84     4 */
	enum regmap_endian         val_format_endian_default; /*    88     4 */

	/* XXX 4 bytes hole, try to pack */

	size_t                     max_raw_read;         /*    96     8 */
	size_t                     max_raw_write;        /*   104     8 */

	/* size: 112, cachelines: 2, members: 15 */
	/* sum members: 98, holes: 3, sum holes: 14 */
	/* last cacheline: 48 bytes */
};
struct reg_field {
	unsigned int               reg;                  /*     0     4 */
	unsigned int               lsb;                  /*     4     4 */
	unsigned int               msb;                  /*     8     4 */
	unsigned int               id_size;              /*    12     4 */
	unsigned int               id_offset;            /*    16     4 */

	/* size: 20, cachelines: 1, members: 5 */
	/* last cacheline: 20 bytes */
};
struct regmap_format {
	size_t                     buf_size;             /*     0     8 */
	size_t                     reg_bytes;            /*     8     8 */
	size_t                     pad_bytes;            /*    16     8 */
	size_t                     val_bytes;            /*    24     8 */
	void                       (*format_write)(struct regmap *, unsigned int, unsigned int); /*    32     8 */
	void                       (*format_reg)(void *, unsigned int, unsigned int); /*    40     8 */
	void                       (*format_val)(void *, unsigned int, unsigned int); /*    48     8 */
	unsigned int               (*parse_val)(const void  *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	void                       (*parse_inplace)(void *); /*    64     8 */

	/* size: 72, cachelines: 2, members: 9 */
	/* last cacheline: 8 bytes */
};
struct regmap {
	union {
		struct mutex       mutex;                /*     0    32 */
		struct {
			spinlock_t spinlock;             /*     0     4 */

			/* XXX 4 bytes hole, try to pack */

			long unsigned int spinlock_flags; /*     8     8 */
		};                                       /*     0    16 */
	};                                               /*     0    32 */
	regmap_lock                lock;                 /*    32     8 */
	regmap_unlock              unlock;               /*    40     8 */
	void *                     lock_arg;             /*    48     8 */
	gfp_t                      alloc_flags;          /*    56     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	struct device *            dev;                  /*    64     8 */
	void *                     work_buf;             /*    72     8 */
	struct regmap_format       format;               /*    80    72 */
	/* --- cacheline 2 boundary (128 bytes) was 24 bytes ago --- */
	const struct regmap_bus  * bus;                  /*   152     8 */
	void *                     bus_context;          /*   160     8 */
	const char  *              name;                 /*   168     8 */
	bool                       async;                /*   176     1 */

	/* XXX 3 bytes hole, try to pack */

	spinlock_t                 async_lock;           /*   180     4 */
	wait_queue_head_t          async_waitq;          /*   184    24 */
	/* --- cacheline 3 boundary (192 bytes) was 16 bytes ago --- */
	struct list_head           async_list;           /*   208    16 */
	struct list_head           async_free;           /*   224    16 */
	int                        async_ret;            /*   240     4 */
	bool                       debugfs_disable;      /*   244     1 */

	/* XXX 3 bytes hole, try to pack */

	struct dentry *            debugfs;              /*   248     8 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	const char  *              debugfs_name;         /*   256     8 */
	unsigned int               debugfs_reg_len;      /*   264     4 */
	unsigned int               debugfs_val_len;      /*   268     4 */
	unsigned int               debugfs_tot_len;      /*   272     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           debugfs_off_cache;    /*   280    16 */
	struct mutex               cache_lock;           /*   296    32 */
	/* --- cacheline 5 boundary (320 bytes) was 8 bytes ago --- */
	unsigned int               max_register;         /*   328     4 */

	/* XXX 4 bytes hole, try to pack */

	bool                       (*writeable_reg)(struct device *, unsigned int); /*   336     8 */
	bool                       (*readable_reg)(struct device *, unsigned int); /*   344     8 */
	bool                       (*volatile_reg)(struct device *, unsigned int); /*   352     8 */
	bool                       (*precious_reg)(struct device *, unsigned int); /*   360     8 */
	bool                       (*writeable_noinc_reg)(struct device *, unsigned int); /*   368     8 */
	bool                       (*readable_noinc_reg)(struct device *, unsigned int); /*   376     8 */
	/* --- cacheline 6 boundary (384 bytes) --- */
	const struct regmap_access_table  * wr_table;    /*   384     8 */
	const struct regmap_access_table  * rd_table;    /*   392     8 */
	const struct regmap_access_table  * volatile_table; /*   400     8 */
	const struct regmap_access_table  * precious_table; /*   408     8 */
	const struct regmap_access_table  * wr_noinc_table; /*   416     8 */
	const struct regmap_access_table  * rd_noinc_table; /*   424     8 */
	int                        (*reg_read)(void *, unsigned int, unsigned int *); /*   432     8 */
	int                        (*reg_write)(void *, unsigned int, unsigned int); /*   440     8 */
	/* --- cacheline 7 boundary (448 bytes) --- */
	int                        (*reg_update_bits)(void *, unsigned int, unsigned int, unsigned int); /*   448     8 */
	bool                       defer_caching;        /*   456     1 */

	/* XXX 7 bytes hole, try to pack */

	long unsigned int          read_flag_mask;       /*   464     8 */
	long unsigned int          write_flag_mask;      /*   472     8 */
	int                        reg_shift;            /*   480     4 */
	int                        reg_stride;           /*   484     4 */
	int                        reg_stride_order;     /*   488     4 */

	/* XXX 4 bytes hole, try to pack */

	const struct regcache_ops  * cache_ops;          /*   496     8 */
	enum regcache_type         cache_type;           /*   504     4 */
	unsigned int               cache_size_raw;       /*   508     4 */
	/* --- cacheline 8 boundary (512 bytes) --- */
	unsigned int               cache_word_size;      /*   512     4 */
	unsigned int               num_reg_defaults;     /*   516     4 */
	unsigned int               num_reg_defaults_raw; /*   520     4 */
	bool                       cache_only;           /*   524     1 */
	bool                       cache_bypass;         /*   525     1 */
	bool                       cache_free;           /*   526     1 */

	/* XXX 1 byte hole, try to pack */

	struct reg_default *       reg_defaults;         /*   528     8 */
	const void  *              reg_defaults_raw;     /*   536     8 */
	void *                     cache;                /*   544     8 */
	bool                       cache_dirty;          /*   552     1 */
	bool                       no_sync_defaults;     /*   553     1 */

	/* XXX 6 bytes hole, try to pack */

	struct reg_sequence *      patch;                /*   560     8 */
	int                        patch_regs;           /*   568     4 */
	bool                       use_single_read;      /*   572     1 */
	bool                       use_single_write;     /*   573     1 */
	bool                       can_multi_write;      /*   574     1 */

	/* XXX 1 byte hole, try to pack */

	/* --- cacheline 9 boundary (576 bytes) --- */
	size_t                     max_raw_read;         /*   576     8 */
	size_t                     max_raw_write;        /*   584     8 */
	struct rb_root             range_tree;           /*   592     8 */
	void *                     selector_work_buf;    /*   600     8 */
	struct hwspinlock *        hwlock;               /*   608     8 */
	bool                       can_sleep;            /*   616     1 */

	/* size: 624, cachelines: 10, members: 72 */
	/* sum members: 580, holes: 10, sum holes: 37 */
	/* padding: 7 */
	/* last cacheline: 48 bytes */
};
struct regcache_ops {
	const char  *              name;                 /*     0     8 */
	enum regcache_type         type;                 /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	int                        (*init)(struct regmap *); /*    16     8 */
	int                        (*exit)(struct regmap *); /*    24     8 */
	void                       (*debugfs_init)(struct regmap *); /*    32     8 */
	int                        (*read)(struct regmap *, unsigned int, unsigned int *); /*    40     8 */
	int                        (*write)(struct regmap *, unsigned int, unsigned int); /*    48     8 */
	int                        (*sync)(struct regmap *, unsigned int, unsigned int); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*drop)(struct regmap *, unsigned int, unsigned int); /*    64     8 */

	/* size: 72, cachelines: 2, members: 9 */
	/* sum members: 68, holes: 1, sum holes: 4 */
	/* last cacheline: 8 bytes */
};
struct regmap_range_node {
	struct rb_node             node __attribute__((__aligned__(8))); /*     0    24 */
	const char  *              name;                 /*    24     8 */
	struct regmap *            map;                  /*    32     8 */
	unsigned int               range_min;            /*    40     4 */
	unsigned int               range_max;            /*    44     4 */
	unsigned int               selector_reg;         /*    48     4 */
	unsigned int               selector_mask;        /*    52     4 */
	int                        selector_shift;       /*    56     4 */
	unsigned int               window_start;         /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	unsigned int               window_len;           /*    64     4 */

	/* size: 72, cachelines: 2, members: 10 */
	/* padding: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 8 bytes */
} __attribute__((__aligned__(8)));
struct regmap_field {
	struct regmap *            regmap;               /*     0     8 */
	unsigned int               mask;                 /*     8     4 */
	unsigned int               shift;                /*    12     4 */
	unsigned int               reg;                  /*    16     4 */
	unsigned int               id_size;              /*    20     4 */
	unsigned int               id_offset;            /*    24     4 */

	/* size: 32, cachelines: 1, members: 6 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct trace_event_raw_regmap_reg {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        __data_loc_name;      /*     8     4 */
	unsigned int               reg;                  /*    12     4 */
	unsigned int               val;                  /*    16     4 */
	char                       __data[];             /*    20     0 */

	/* size: 20, cachelines: 1, members: 5 */
	/* last cacheline: 20 bytes */
};
struct trace_event_raw_regmap_block {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        __data_loc_name;      /*     8     4 */
	unsigned int               reg;                  /*    12     4 */
	int                        count;                /*    16     4 */
	char                       __data[];             /*    20     0 */

	/* size: 20, cachelines: 1, members: 5 */
	/* last cacheline: 20 bytes */
};
struct trace_event_raw_regcache_sync {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        __data_loc_name;      /*     8     4 */
	u32                        __data_loc_status;    /*    12     4 */
	u32                        __data_loc_type;      /*    16     4 */
	int                        type;                 /*    20     4 */
	char                       __data[];             /*    24     0 */

	/* size: 24, cachelines: 1, members: 6 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_regmap_bool {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        __data_loc_name;      /*     8     4 */
	int                        flag;                 /*    12     4 */
	char                       __data[];             /*    16     0 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct trace_event_raw_regmap_async {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        __data_loc_name;      /*     8     4 */
	char                       __data[];             /*    12     0 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct trace_event_raw_regcache_drop_region {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        __data_loc_name;      /*     8     4 */
	unsigned int               from;                 /*    12     4 */
	unsigned int               to;                   /*    16     4 */
	char                       __data[];             /*    20     0 */

	/* size: 20, cachelines: 1, members: 5 */
	/* last cacheline: 20 bytes */
};
struct trace_event_data_offsets_regmap_reg {
	u32                        name;                 /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct trace_event_data_offsets_regmap_block {
	u32                        name;                 /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct trace_event_data_offsets_regcache_sync {
	u32                        name;                 /*     0     4 */
	u32                        status;               /*     4     4 */
	u32                        type;                 /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct trace_event_data_offsets_regmap_bool {
	u32                        name;                 /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct trace_event_data_offsets_regmap_async {
	u32                        name;                 /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct trace_event_data_offsets_regcache_drop_region {
	u32                        name;                 /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct regcache_rbtree_node {
	void *                     block;                /*     0     8 */
	long int *                 cache_present;        /*     8     8 */
	unsigned int               base_reg;             /*    16     4 */
	unsigned int               blklen;               /*    20     4 */
	struct rb_node             node __attribute__((__aligned__(8))); /*    24    24 */

	/* size: 48, cachelines: 1, members: 5 */
	/* forced alignments: 1 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct regcache_rbtree_ctx {
	struct rb_root             root;                 /*     0     8 */
	struct regcache_rbtree_node * cached_rbnode;     /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct regmap_debugfs_off_cache {
	struct list_head           list;                 /*     0    16 */
	off_t                      min;                  /*    16     8 */
	off_t                      max;                  /*    24     8 */
	unsigned int               base_reg;             /*    32     4 */
	unsigned int               max_reg;              /*    36     4 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct regmap_debugfs_node {
	struct regmap *            map;                  /*     0     8 */
	struct list_head           link;                 /*     8    16 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct devcd_entry {
	struct device              devcd_dev __attribute__((__aligned__(8))); /*     0  1000 */
	/* --- cacheline 15 boundary (960 bytes) was 40 bytes ago --- */
	void *                     data;                 /*  1000     8 */
	size_t                     datalen;              /*  1008     8 */
	struct module *            owner;                /*  1016     8 */
	/* --- cacheline 16 boundary (1024 bytes) --- */
	ssize_t                    (*read)(char *, loff_t, size_t, void *, size_t); /*  1024     8 */
	void                       (*free)(void *);      /*  1032     8 */
	struct delayed_work        del_wk;               /*  1040   192 */
	/* --- cacheline 19 boundary (1216 bytes) was 16 bytes ago --- */
	struct device *            failing_dev;          /*  1232     8 */

	/* size: 1240, cachelines: 20, members: 8 */
	/* forced alignments: 1 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct platform_msi_priv_data {
	struct device *            dev;                  /*     0     8 */
	void *                     host_data;            /*     8     8 */
	msi_alloc_info_t           arg;                  /*    16    72 */
	/* --- cacheline 1 boundary (64 bytes) was 24 bytes ago --- */
	irq_write_msi_msg_t        write_msg;            /*    88     8 */
	int                        devid;                /*    96     4 */

	/* size: 104, cachelines: 2, members: 5 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct test_struct {
	char *                     get;                  /*     0     8 */
	char *                     put;                  /*     8     8 */
	void                       (*get_handler)(char *); /*    16     8 */
	int                        (*put_handler)(char *, char *); /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct test_state {
	char *                     name;                 /*     0     8 */
	struct test_struct *       tst;                  /*     8     8 */
	int                        idx;                  /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	int                        (*run_test)(int, int); /*    24     8 */
	int                        (*validate_put)(char *); /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* sum members: 36, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct mfd_cell {
	const char  *              name;                 /*     0     8 */
	int                        id;                   /*     8     4 */
	int                        level;                /*    12     4 */
	int                        (*enable)(struct platform_device *); /*    16     8 */
	int                        (*disable)(struct platform_device *); /*    24     8 */
	int                        (*suspend)(struct platform_device *); /*    32     8 */
	int                        (*resume)(struct platform_device *); /*    40     8 */
	void *                     platform_data;        /*    48     8 */
	size_t                     pdata_size;           /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	const struct property_entry  * properties;       /*    64     8 */
	const char  *              of_compatible;        /*    72     8 */
	const u64                  of_reg;               /*    80     8 */
	bool                       use_of_reg;           /*    88     1 */

	/* XXX 7 bytes hole, try to pack */

	const struct mfd_cell_acpi_match  * acpi_match;  /*    96     8 */
	int                        num_resources;        /*   104     4 */

	/* XXX 4 bytes hole, try to pack */

	const struct resource  *   resources;            /*   112     8 */
	bool                       ignore_resource_conflicts; /*   120     1 */
	bool                       pm_runtime_no_callbacks; /*   121     1 */

	/* XXX 6 bytes hole, try to pack */

	/* --- cacheline 2 boundary (128 bytes) --- */
	const char  * const *      parent_supplies;      /*   128     8 */
	int                        num_parent_supplies;  /*   136     4 */

	/* size: 144, cachelines: 3, members: 20 */
	/* sum members: 123, holes: 3, sum holes: 17 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct mfd_cell_acpi_match {
	const char  *              pnpid;                /*     0     8 */
	const long long unsigned int adr;                /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct mfd_of_node_entry {
	struct list_head           list;                 /*     0    16 */
	struct device *            dev;                  /*    16     8 */
	struct device_node *       np;                   /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* last cacheline: 32 bytes */
};
struct intel_lpss_platform_info {
	struct resource *          mem;                  /*     0     8 */
	int                        irq;                  /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          clk_rate;             /*    16     8 */
	const char  *              clk_con_id;           /*    24     8 */
	struct property_entry *    properties;           /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* sum members: 36, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct intel_lpss {
	const struct intel_lpss_platform_info  * info;   /*     0     8 */
	enum intel_lpss_dev_type   type;                 /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct clk *               clk;                  /*    16     8 */
	struct clk_lookup *        clock;                /*    24     8 */
	struct mfd_cell *          cell;                 /*    32     8 */
	struct device *            dev;                  /*    40     8 */
	void *                     priv;                 /*    48     8 */
	u32                        priv_ctx[64];         /*    56   256 */
	/* --- cacheline 4 boundary (256 bytes) was 56 bytes ago --- */
	int                        devid;                /*   312     4 */
	u32                        caps;                 /*   316     4 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	u32                        active_ltr;           /*   320     4 */
	u32                        idle_ltr;             /*   324     4 */
	struct dentry *            debugfs;              /*   328     8 */

	/* size: 336, cachelines: 6, members: 13 */
	/* sum members: 332, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct dax_operations {
	long int                   (*direct_access)(struct dax_device *, long unsigned int, long int, void * *, pfn_t *); /*     0     8 */
	bool                       (*dax_supported)(struct dax_device *, struct block_device *, int, sector_t, sector_t); /*     8     8 */
	size_t                     (*copy_from_iter)(struct dax_device *, long unsigned int, void *, size_t, struct iov_iter *); /*    16     8 */
	size_t                     (*copy_to_iter)(struct dax_device *, long unsigned int, void *, size_t, struct iov_iter *); /*    24     8 */
	int                        (*zero_page_range)(struct dax_device *, long unsigned int, size_t); /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct dax_device {
	struct hlist_node          list;                 /*     0    16 */
	struct inode               inode __attribute__((__aligned__(8))); /*    16   624 */
	/* --- cacheline 10 boundary (640 bytes) --- */
	struct cdev                cdev;                 /*   640   136 */
	/* --- cacheline 12 boundary (768 bytes) was 8 bytes ago --- */
	const char  *              host;                 /*   776     8 */
	void *                     private;              /*   784     8 */
	long unsigned int          flags;                /*   792     8 */
	const struct dax_operations  * ops;              /*   800     8 */

	/* size: 808, cachelines: 13, members: 7 */
	/* forced alignments: 1 */
	/* last cacheline: 40 bytes */
} __attribute__((__aligned__(8)));
struct dax_region {
	int                        id;                   /*     0     4 */
	int                        target_node;          /*     4     4 */
	struct kref                kref;                 /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct device *            dev;                  /*    16     8 */
	unsigned int               align;                /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	struct ida                 ida;                  /*    32    16 */
	struct resource            res;                  /*    48    96 */
	/* --- cacheline 2 boundary (128 bytes) was 16 bytes ago --- */
	struct device *            seed;                 /*   144     8 */
	struct device *            youngest;             /*   152     8 */

	/* size: 160, cachelines: 3, members: 9 */
	/* sum members: 152, holes: 2, sum holes: 8 */
	/* last cacheline: 32 bytes */
};
struct dax_mapping {
	struct device              dev __attribute__((__aligned__(8))); /*     0  1000 */
	/* --- cacheline 15 boundary (960 bytes) was 40 bytes ago --- */
	int                        range_id;             /*  1000     4 */
	int                        id;                   /*  1004     4 */

	/* size: 1008, cachelines: 16, members: 3 */
	/* forced alignments: 1 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct dev_dax_range {
	long unsigned int          pgoff;                /*     0     8 */
	struct range               range;                /*     8    16 */
	struct dax_mapping *       mapping;              /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* last cacheline: 32 bytes */
};
struct dev_dax {
	struct dax_region *        region;               /*     0     8 */
	struct dax_device *        dax_dev;              /*     8     8 */
	unsigned int               align;                /*    16     4 */
	int                        target_node;          /*    20     4 */
	int                        id;                   /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	struct ida                 ida;                  /*    32    16 */
	struct device              dev __attribute__((__aligned__(8))); /*    48  1000 */
	/* --- cacheline 16 boundary (1024 bytes) was 24 bytes ago --- */
	struct dev_pagemap *       pgmap;                /*  1048     8 */
	int                        nr_range;             /*  1056     4 */

	/* XXX 4 bytes hole, try to pack */

	struct dev_dax_range *     ranges;               /*  1064     8 */

	/* size: 1072, cachelines: 17, members: 10 */
	/* sum members: 1064, holes: 2, sum holes: 8 */
	/* forced alignments: 1 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct dev_dax_data {
	struct dax_region *        dax_region;           /*     0     8 */
	struct dev_pagemap *       pgmap;                /*     8     8 */
	enum dev_dax_subsys        subsys;               /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	resource_size_t            size;                 /*    24     8 */
	int                        id;                   /*    32     4 */

	/* size: 40, cachelines: 1, members: 5 */
	/* sum members: 32, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct dax_device_driver {
	struct device_driver       drv;                  /*     0   192 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	struct list_head           ids;                  /*   192    16 */
	int                        match_always;         /*   208     4 */

	/* XXX 4 bytes hole, try to pack */

	int                        (*probe)(struct dev_dax *); /*   216     8 */
	int                        (*remove)(struct dev_dax *); /*   224     8 */

	/* size: 232, cachelines: 4, members: 5 */
	/* sum members: 228, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct dax_id {
	struct list_head           list;                 /*     0    16 */
	char                       dev_name[30];         /*    16    30 */

	/* size: 48, cachelines: 1, members: 2 */
	/* padding: 2 */
	/* last cacheline: 48 bytes */
};
struct memregion_info {
	int                        target_node;          /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct seqcount_ww_mutex {
	seqcount_t                 seqcount;             /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct dma_fence {
	spinlock_t *               lock;                 /*     0     8 */
	const struct dma_fence_ops  * ops;               /*     8     8 */
	union {
		struct list_head   cb_list;              /*    16    16 */
		ktime_t            timestamp;            /*    16     8 */
		struct callback_head rcu __attribute__((__aligned__(8))); /*    16    16 */
	} __attribute__((__aligned__(8)));               /*    16    16 */
	u64                        context;              /*    32     8 */
	u64                        seqno;                /*    40     8 */
	long unsigned int          flags;                /*    48     8 */
	struct kref                refcount;             /*    56     4 */
	int                        error;                /*    60     4 */

	/* size: 64, cachelines: 1, members: 8 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(8)));
struct dma_fence_ops {
	bool                       use_64bit_seqno;      /*     0     1 */

	/* XXX 7 bytes hole, try to pack */

	const char  *              (*get_driver_name)(struct dma_fence *); /*     8     8 */
	const char  *              (*get_timeline_name)(struct dma_fence *); /*    16     8 */
	bool                       (*enable_signaling)(struct dma_fence *); /*    24     8 */
	bool                       (*signaled)(struct dma_fence *); /*    32     8 */
	long int                   (*wait)(struct dma_fence *, bool, long int); /*    40     8 */
	void                       (*release)(struct dma_fence *); /*    48     8 */
	void                       (*fence_value_str)(struct dma_fence *, char *, int); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	void                       (*timeline_value_str)(struct dma_fence *, char *, int); /*    64     8 */

	/* size: 72, cachelines: 2, members: 9 */
	/* sum members: 65, holes: 1, sum holes: 7 */
	/* last cacheline: 8 bytes */
};
struct dma_fence_cb {
	struct list_head           node;                 /*     0    16 */
	dma_fence_func_t           func;                 /*    16     8 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct dma_buf_ops {
	bool                       cache_sgt_mapping;    /*     0     1 */

	/* XXX 7 bytes hole, try to pack */

	int                        (*attach)(struct dma_buf *, struct dma_buf_attachment *); /*     8     8 */
	void                       (*detach)(struct dma_buf *, struct dma_buf_attachment *); /*    16     8 */
	int                        (*pin)(struct dma_buf_attachment *); /*    24     8 */
	void                       (*unpin)(struct dma_buf_attachment *); /*    32     8 */
	struct sg_table *          (*map_dma_buf)(struct dma_buf_attachment *, enum dma_data_direction); /*    40     8 */
	void                       (*unmap_dma_buf)(struct dma_buf_attachment *, struct sg_table *, enum dma_data_direction); /*    48     8 */
	void                       (*release)(struct dma_buf *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*begin_cpu_access)(struct dma_buf *, enum dma_data_direction); /*    64     8 */
	int                        (*end_cpu_access)(struct dma_buf *, enum dma_data_direction); /*    72     8 */
	int                        (*mmap)(struct dma_buf *, struct vm_area_struct *); /*    80     8 */
	void *                     (*vmap)(struct dma_buf *); /*    88     8 */
	void                       (*vunmap)(struct dma_buf *, void *); /*    96     8 */

	/* size: 104, cachelines: 2, members: 13 */
	/* sum members: 97, holes: 1, sum holes: 7 */
	/* last cacheline: 40 bytes */
};
struct dma_buf {
	size_t                     size;                 /*     0     8 */
	struct file *              file;                 /*     8     8 */
	struct list_head           attachments;          /*    16    16 */
	const struct dma_buf_ops  * ops;                 /*    32     8 */
	struct mutex               lock;                 /*    40    32 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	unsigned int               vmapping_counter;     /*    72     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     vmap_ptr;             /*    80     8 */
	const char  *              exp_name;             /*    88     8 */
	const char  *              name;                 /*    96     8 */
	spinlock_t                 name_lock;            /*   104     4 */

	/* XXX 4 bytes hole, try to pack */

	struct module *            owner;                /*   112     8 */
	struct list_head           list_node;            /*   120    16 */
	/* --- cacheline 2 boundary (128 bytes) was 8 bytes ago --- */
	void *                     priv;                 /*   136     8 */
	struct dma_resv *          resv;                 /*   144     8 */
	wait_queue_head_t          poll;                 /*   152    24 */
	struct dma_buf_poll_cb_t   cb_excl;              /*   176    40 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 3 boundary (192 bytes) was 24 bytes ago --- */
	struct dma_buf_poll_cb_t   cb_shared;            /*   216    40 */

	/* XXX last struct has 4 bytes of padding */

	/* size: 256, cachelines: 4, members: 17 */
	/* sum members: 248, holes: 2, sum holes: 8 */
	/* paddings: 2, sum paddings: 8 */
};
struct dma_buf_attachment {
	struct dma_buf *           dmabuf;               /*     0     8 */
	struct device *            dev;                  /*     8     8 */
	struct list_head           node;                 /*    16    16 */
	struct sg_table *          sgt;                  /*    32     8 */
	enum dma_data_direction    dir;                  /*    40     4 */
	bool                       peer2peer;            /*    44     1 */

	/* XXX 3 bytes hole, try to pack */

	const struct dma_buf_attach_ops  * importer_ops; /*    48     8 */
	void *                     importer_priv;        /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	void *                     priv;                 /*    64     8 */

	/* size: 72, cachelines: 2, members: 9 */
	/* sum members: 69, holes: 1, sum holes: 3 */
	/* last cacheline: 8 bytes */
};
struct dma_buf_poll_cb_t {
	struct dma_fence_cb        cb;                   /*     0    24 */
	wait_queue_head_t *        poll;                 /*    24     8 */
	__poll_t                   active;               /*    32     4 */

	/* size: 40, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct dma_resv {
	struct ww_mutex            lock;                 /*     0    40 */
	seqcount_ww_mutex_t        seq;                  /*    40     4 */

	/* XXX 4 bytes hole, try to pack */

	struct dma_fence *         fence_excl;           /*    48     8 */
	struct dma_resv_list *     fence;                /*    56     8 */

	/* size: 64, cachelines: 1, members: 4 */
	/* sum members: 60, holes: 1, sum holes: 4 */
};
struct dma_buf_attach_ops {
	bool                       allow_peer2peer;      /*     0     1 */

	/* XXX 7 bytes hole, try to pack */

	void                       (*move_notify)(struct dma_buf_attachment *); /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 9, holes: 1, sum holes: 7 */
	/* last cacheline: 16 bytes */
};
struct dma_buf_export_info {
	const char  *              exp_name;             /*     0     8 */
	struct module *            owner;                /*     8     8 */
	const struct dma_buf_ops  * ops;                 /*    16     8 */
	size_t                     size;                 /*    24     8 */
	int                        flags;                /*    32     4 */

	/* XXX 4 bytes hole, try to pack */

	struct dma_resv *          resv;                 /*    40     8 */
	void *                     priv;                 /*    48     8 */

	/* size: 56, cachelines: 1, members: 7 */
	/* sum members: 52, holes: 1, sum holes: 4 */
	/* last cacheline: 56 bytes */
};
struct dma_resv_list {
	struct callback_head       rcu __attribute__((__aligned__(8))); /*     0    16 */
	u32                        shared_count;         /*    16     4 */
	u32                        shared_max;           /*    20     4 */
	struct dma_fence *         shared[];             /*    24     0 */

	/* size: 24, cachelines: 1, members: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct dma_buf_sync {
	__u64                      flags;                /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct dma_buf_list {
	struct list_head           head;                 /*     0    16 */
	struct mutex               lock;                 /*    16    32 */

	/* size: 48, cachelines: 1, members: 2 */
	/* last cacheline: 48 bytes */
};
struct trace_event_raw_dma_fence {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        __data_loc_driver;    /*     8     4 */
	u32                        __data_loc_timeline;  /*    12     4 */
	unsigned int               context;              /*    16     4 */
	unsigned int               seqno;                /*    20     4 */
	char                       __data[];             /*    24     0 */

	/* size: 24, cachelines: 1, members: 6 */
	/* last cacheline: 24 bytes */
};
struct trace_event_data_offsets_dma_fence {
	u32                        driver;               /*     0     4 */
	u32                        timeline;             /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct default_wait_cb {
	struct dma_fence_cb        base;                 /*     0    24 */
	struct task_struct *       task;                 /*    24     8 */

	/* size: 32, cachelines: 1, members: 2 */
	/* last cacheline: 32 bytes */
};
struct dma_fence_array_cb {
	struct dma_fence_cb        cb;                   /*     0    24 */
	struct dma_fence_array *   array;                /*    24     8 */

	/* size: 32, cachelines: 1, members: 2 */
	/* last cacheline: 32 bytes */
};
struct dma_fence_array {
	struct dma_fence           base __attribute__((__aligned__(8))); /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	spinlock_t                 lock;                 /*    64     4 */
	unsigned int               num_fences;           /*    68     4 */
	atomic_t                   num_pending;          /*    72     4 */

	/* XXX 4 bytes hole, try to pack */

	struct dma_fence * *       fences;               /*    80     8 */
	struct irq_work            work;                 /*    88    24 */

	/* size: 112, cachelines: 2, members: 6 */
	/* sum members: 108, holes: 1, sum holes: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct dma_fence_chain {
	struct dma_fence           base __attribute__((__aligned__(8))); /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	spinlock_t                 lock;                 /*    64     4 */

	/* XXX 4 bytes hole, try to pack */

	struct dma_fence *         prev;                 /*    72     8 */
	u64                        prev_seqno;           /*    80     8 */
	struct dma_fence *         fence;                /*    88     8 */
	struct dma_fence_cb        cb;                   /*    96    24 */
	struct irq_work            work;                 /*   120    24 */

	/* size: 144, cachelines: 3, members: 7 */
	/* sum members: 140, holes: 1, sum holes: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct ww_class {
	atomic_long_t              stamp;                /*     0     8 */
	struct lock_class_key      acquire_key;          /*     8     0 */
	struct lock_class_key      mutex_key;            /*     8     0 */
	const char  *              acquire_name;         /*     8     8 */
	const char  *              mutex_name;           /*    16     8 */
	unsigned int               is_wait_die;          /*    24     4 */

	/* size: 32, cachelines: 1, members: 6 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct seqno_fence {
	struct dma_fence           base __attribute__((__aligned__(8))); /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	const struct dma_fence_ops  * ops;               /*    64     8 */
	struct dma_buf *           sync_buf;             /*    72     8 */
	uint32_t                   seqno_ofs;            /*    80     4 */
	enum seqno_fence_condition condition;            /*    84     4 */

	/* size: 88, cachelines: 2, members: 5 */
	/* forced alignments: 1 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct sync_file {
	struct file *              file;                 /*     0     8 */
	char                       user_name[32];        /*     8    32 */
	struct list_head           sync_file_list;       /*    40    16 */
	wait_queue_head_t          wq;                   /*    56    24 */
	/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
	long unsigned int          flags;                /*    80     8 */
	struct dma_fence *         fence;                /*    88     8 */
	struct dma_fence_cb        cb;                   /*    96    24 */

	/* size: 120, cachelines: 2, members: 7 */
	/* last cacheline: 56 bytes */
};
struct sync_merge_data {
	char                       name[32];             /*     0    32 */
	__s32                      fd2;                  /*    32     4 */
	__s32                      fence;                /*    36     4 */
	__u32                      flags;                /*    40     4 */
	__u32                      pad;                  /*    44     4 */

	/* size: 48, cachelines: 1, members: 5 */
	/* last cacheline: 48 bytes */
};
struct sync_fence_info {
	char                       obj_name[32];         /*     0    32 */
	char                       driver_name[32];      /*    32    32 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	__s32                      status;               /*    64     4 */
	__u32                      flags;                /*    68     4 */
	__u64                      timestamp_ns;         /*    72     8 */

	/* size: 80, cachelines: 2, members: 5 */
	/* last cacheline: 16 bytes */
};
struct sync_file_info {
	char                       name[32];             /*     0    32 */
	__s32                      status;               /*    32     4 */
	__u32                      flags;                /*    36     4 */
	__u32                      num_fences;           /*    40     4 */
	__u32                      pad;                  /*    44     4 */
	__u64                      sync_fence_info;      /*    48     8 */

	/* size: 56, cachelines: 1, members: 6 */
	/* last cacheline: 56 bytes */
};
struct scsi_vpd {
	struct callback_head       rcu __attribute__((__aligned__(8))); /*     0    16 */
	int                        len;                  /*    16     4 */
	unsigned char              data[];               /*    20     0 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct scsi_device {
	struct Scsi_Host *         host;                 /*     0     8 */
	struct request_queue *     request_queue;        /*     8     8 */
	struct list_head           siblings;             /*    16    16 */
	struct list_head           same_target_siblings; /*    32    16 */
	atomic_t                   device_busy;          /*    48     4 */
	atomic_t                   device_blocked;       /*    52     4 */
	atomic_t                   restarts;             /*    56     4 */
	spinlock_t                 list_lock;            /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct list_head           starved_entry;        /*    64    16 */
	short unsigned int         queue_depth;          /*    80     2 */
	short unsigned int         max_queue_depth;      /*    82     2 */
	short unsigned int         last_queue_full_depth; /*    84     2 */
	short unsigned int         last_queue_full_count; /*    86     2 */
	long unsigned int          last_queue_full_time; /*    88     8 */
	long unsigned int          queue_ramp_up_period; /*    96     8 */
	long unsigned int          last_queue_ramp_up;   /*   104     8 */
	unsigned int               id;                   /*   112     4 */
	unsigned int               channel;              /*   116     4 */
	u64                        lun;                  /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	unsigned int               manufacturer;         /*   128     4 */
	unsigned int               sector_size;          /*   132     4 */
	void *                     hostdata;             /*   136     8 */
	unsigned char              type;                 /*   144     1 */
	char                       scsi_level;           /*   145     1 */
	char                       inq_periph_qual;      /*   146     1 */

	/* XXX 5 bytes hole, try to pack */

	struct mutex               inquiry_mutex;        /*   152    32 */
	unsigned char              inquiry_len;          /*   184     1 */

	/* XXX 7 bytes hole, try to pack */

	/* --- cacheline 3 boundary (192 bytes) --- */
	unsigned char *            inquiry;              /*   192     8 */
	const char  *              vendor;               /*   200     8 */
	const char  *              model;                /*   208     8 */
	const char  *              rev;                  /*   216     8 */
	struct scsi_vpd *          vpd_pg0;              /*   224     8 */
	struct scsi_vpd *          vpd_pg83;             /*   232     8 */
	struct scsi_vpd *          vpd_pg80;             /*   240     8 */
	struct scsi_vpd *          vpd_pg89;             /*   248     8 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	unsigned char              current_tag;          /*   256     1 */

	/* XXX 7 bytes hole, try to pack */

	struct scsi_target *       sdev_target;          /*   264     8 */
	blist_flags_t              sdev_bflags;          /*   272     8 */
	unsigned int               eh_timeout;           /*   280     4 */
	unsigned int               removable:1;          /*   284: 0  4 */
	unsigned int               changed:1;            /*   284: 1  4 */
	unsigned int               busy:1;               /*   284: 2  4 */
	unsigned int               lockable:1;           /*   284: 3  4 */
	unsigned int               locked:1;             /*   284: 4  4 */
	unsigned int               borken:1;             /*   284: 5  4 */
	unsigned int               disconnect:1;         /*   284: 6  4 */
	unsigned int               soft_reset:1;         /*   284: 7  4 */
	unsigned int               sdtr:1;               /*   284: 8  4 */
	unsigned int               wdtr:1;               /*   284: 9  4 */
	unsigned int               ppr:1;                /*   284:10  4 */
	unsigned int               tagged_supported:1;   /*   284:11  4 */
	unsigned int               simple_tags:1;        /*   284:12  4 */
	unsigned int               was_reset:1;          /*   284:13  4 */
	unsigned int               expecting_cc_ua:1;    /*   284:14  4 */
	unsigned int               use_10_for_rw:1;      /*   284:15  4 */
	unsigned int               use_10_for_ms:1;      /*   284:16  4 */
	unsigned int               set_dbd_for_ms:1;     /*   284:17  4 */
	unsigned int               no_report_opcodes:1;  /*   284:18  4 */
	unsigned int               no_write_same:1;      /*   284:19  4 */
	unsigned int               use_16_for_rw:1;      /*   284:20  4 */
	unsigned int               skip_ms_page_8:1;     /*   284:21  4 */
	unsigned int               skip_ms_page_3f:1;    /*   284:22  4 */
	unsigned int               skip_vpd_pages:1;     /*   284:23  4 */
	unsigned int               try_vpd_pages:1;      /*   284:24  4 */
	unsigned int               use_192_bytes_for_3f:1; /*   284:25  4 */
	unsigned int               no_start_on_add:1;    /*   284:26  4 */
	unsigned int               allow_restart:1;      /*   284:27  4 */
	unsigned int               manage_start_stop:1;  /*   284:28  4 */
	unsigned int               start_stop_pwr_cond:1; /*   284:29  4 */
	unsigned int               no_uld_attach:1;      /*   284:30  4 */
	unsigned int               select_no_atn:1;      /*   284:31  4 */
	unsigned int               fix_capacity:1;       /*   288: 0  4 */
	unsigned int               guess_capacity:1;     /*   288: 1  4 */
	unsigned int               retry_hwerror:1;      /*   288: 2  4 */
	unsigned int               last_sector_bug:1;    /*   288: 3  4 */
	unsigned int               no_read_disc_info:1;  /*   288: 4  4 */
	unsigned int               no_read_capacity_16:1; /*   288: 5  4 */
	unsigned int               try_rc_10_first:1;    /*   288: 6  4 */
	unsigned int               security_supported:1; /*   288: 7  4 */
	unsigned int               is_visible:1;         /*   288: 8  4 */
	unsigned int               wce_default_on:1;     /*   288: 9  4 */
	unsigned int               no_dif:1;             /*   288:10  4 */
	unsigned int               broken_fua:1;         /*   288:11  4 */
	unsigned int               lun_in_cdb:1;         /*   288:12  4 */
	unsigned int               unmap_limit_for_ws:1; /*   288:13  4 */
	unsigned int               rpm_autosuspend:1;    /*   288:14  4 */
	unsigned int               queue_stopped:1;      /*   288:15  4 */

	/* Bitfield combined with next fields */

	bool                       offline_already;      /*   290     1 */

	/* XXX 1 byte hole, try to pack */

	atomic_t                   disk_events_disable_depth; /*   292     4 */
	long unsigned int          supported_events[1];  /*   296     8 */
	long unsigned int          pending_events[1];    /*   304     8 */
	struct list_head           event_list;           /*   312    16 */
	/* --- cacheline 5 boundary (320 bytes) was 8 bytes ago --- */
	struct work_struct         event_work;           /*   328    64 */
	/* --- cacheline 6 boundary (384 bytes) was 8 bytes ago --- */
	unsigned int               max_device_blocked;   /*   392     4 */
	atomic_t                   iorequest_cnt;        /*   396     4 */
	atomic_t                   iodone_cnt;           /*   400     4 */
	atomic_t                   ioerr_cnt;            /*   404     4 */
	struct device              sdev_gendev __attribute__((__aligned__(8))); /*   408  1000 */
	/* --- cacheline 22 boundary (1408 bytes) --- */
	struct device              sdev_dev __attribute__((__aligned__(8))); /*  1408  1000 */
	/* --- cacheline 37 boundary (2368 bytes) was 40 bytes ago --- */
	struct execute_work        ew;                   /*  2408    64 */
	/* --- cacheline 38 boundary (2432 bytes) was 40 bytes ago --- */
	struct work_struct         requeue_work;         /*  2472    64 */
	/* --- cacheline 39 boundary (2496 bytes) was 40 bytes ago --- */
	struct scsi_device_handler * handler;            /*  2536     8 */
	void *                     handler_data;         /*  2544     8 */
	size_t                     dma_drain_len;        /*  2552     8 */
	/* --- cacheline 40 boundary (2560 bytes) --- */
	void *                     dma_drain_buf;        /*  2560     8 */
	unsigned char              access_state;         /*  2568     1 */

	/* XXX 7 bytes hole, try to pack */

	struct mutex               state_mutex;          /*  2576    32 */
	enum scsi_device_state     sdev_state;           /*  2608     4 */

	/* XXX 4 bytes hole, try to pack */

	struct task_struct *       quiesced_by;          /*  2616     8 */
	/* --- cacheline 41 boundary (2624 bytes) --- */
	u64                        kabi_reserved1;       /*  2624     8 */
	u64                        kabi_reserved2;       /*  2632     8 */
	u64                        kabi_reserved3;       /*  2640     8 */
	u64                        kabi_reserved4;       /*  2648     8 */
	u64                        kabi_reserved5;       /*  2656     8 */
	u64                        kabi_reserved6;       /*  2664     8 */
	long unsigned int          sdev_data[];          /*  2672     0 */

	/* size: 2672, cachelines: 42, members: 116 */
	/* sum members: 2635, holes: 6, sum holes: 31 */
	/* sum bitfield members: 48 bits (6 bytes) */
	/* forced alignments: 2 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct Scsi_Host {
	struct list_head           __devices;            /*     0    16 */
	struct list_head           __targets;            /*    16    16 */
	struct list_head           starved_list;         /*    32    16 */
	spinlock_t                 default_lock;         /*    48     4 */

	/* XXX 4 bytes hole, try to pack */

	spinlock_t *               host_lock;            /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct mutex               scan_mutex;           /*    64    32 */
	struct list_head           eh_cmd_q;             /*    96    16 */
	struct task_struct *       ehandler;             /*   112     8 */
	struct completion *        eh_action;            /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	wait_queue_head_t          host_wait;            /*   128    24 */
	struct scsi_host_template * hostt;               /*   152     8 */
	struct scsi_transport_template * transportt;     /*   160     8 */
	struct blk_mq_tag_set      tag_set;              /*   168   376 */
	/* --- cacheline 8 boundary (512 bytes) was 32 bytes ago --- */
	atomic_t                   host_blocked;         /*   544     4 */
	unsigned int               host_failed;          /*   548     4 */
	unsigned int               host_eh_scheduled;    /*   552     4 */
	unsigned int               host_no;              /*   556     4 */
	int                        eh_deadline;          /*   560     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          last_reset;           /*   568     8 */
	/* --- cacheline 9 boundary (576 bytes) --- */
	unsigned int               max_channel;          /*   576     4 */
	unsigned int               max_id;               /*   580     4 */
	u64                        max_lun;              /*   584     8 */
	unsigned int               unique_id;            /*   592     4 */
	short unsigned int         max_cmd_len;          /*   596     2 */

	/* XXX 2 bytes hole, try to pack */

	int                        this_id;              /*   600     4 */
	int                        can_queue;            /*   604     4 */
	short int                  cmd_per_lun;          /*   608     2 */
	short unsigned int         sg_tablesize;         /*   610     2 */
	short unsigned int         sg_prot_tablesize;    /*   612     2 */

	/* XXX 2 bytes hole, try to pack */

	unsigned int               max_sectors;          /*   616     4 */
	unsigned int               max_segment_size;     /*   620     4 */
	long unsigned int          dma_boundary;         /*   624     8 */
	long unsigned int          virt_boundary_mask;   /*   632     8 */
	/* --- cacheline 10 boundary (640 bytes) --- */
	unsigned int               nr_hw_queues;         /*   640     4 */
	unsigned int               active_mode:2;        /*   644: 0  4 */
	unsigned int               unchecked_isa_dma:1;  /*   644: 2  4 */
	unsigned int               host_self_blocked:1;  /*   644: 3  4 */
	unsigned int               reverse_ordering:1;   /*   644: 4  4 */
	unsigned int               tmf_in_progress:1;    /*   644: 5  4 */
	unsigned int               async_scan:1;         /*   644: 6  4 */
	unsigned int               eh_noresume:1;        /*   644: 7  4 */
	unsigned int               no_write_same:1;      /*   644: 8  4 */
	unsigned int               host_tagset:1;        /*   644: 9  4 */
	unsigned int               short_inquiry:1;      /*   644:10  4 */
	unsigned int               no_scsi2_lun_in_cdb:1; /*   644:11  4 */

	/* XXX 4 bits hole, try to pack */
	/* Bitfield combined with next fields */

	char                       work_q_name[20];      /*   646    20 */

	/* XXX 6 bytes hole, try to pack */

	struct workqueue_struct *  work_q;               /*   672     8 */
	struct workqueue_struct *  tmf_work_q;           /*   680     8 */
	unsigned int               max_host_blocked;     /*   688     4 */
	unsigned int               prot_capabilities;    /*   692     4 */
	unsigned char              prot_guard_type;      /*   696     1 */

	/* XXX 7 bytes hole, try to pack */

	/* --- cacheline 11 boundary (704 bytes) --- */
	long unsigned int          base;                 /*   704     8 */
	long unsigned int          io_port;              /*   712     8 */
	unsigned char              n_io_port;            /*   720     1 */
	unsigned char              dma_channel;          /*   721     1 */

	/* XXX 2 bytes hole, try to pack */

	unsigned int               irq;                  /*   724     4 */
	enum scsi_host_state       shost_state;          /*   728     4 */

	/* XXX 4 bytes hole, try to pack */

	struct device              shost_gendev __attribute__((__aligned__(8))); /*   736  1000 */
	/* --- cacheline 27 boundary (1728 bytes) was 8 bytes ago --- */
	struct device              shost_dev __attribute__((__aligned__(8))); /*  1736  1000 */
	/* --- cacheline 42 boundary (2688 bytes) was 48 bytes ago --- */
	void *                     shost_data;           /*  2736     8 */
	struct device *            dma_dev;              /*  2744     8 */
	/* --- cacheline 43 boundary (2752 bytes) --- */
	u64                        kabi_reserved1;       /*  2752     8 */
	u64                        kabi_reserved2;       /*  2760     8 */
	u64                        kabi_reserved3;       /*  2768     8 */
	u64                        kabi_reserved4;       /*  2776     8 */
	u64                        kabi_reserved5;       /*  2784     8 */
	u64                        kabi_reserved6;       /*  2792     8 */
	long unsigned int          hostdata[] __attribute__((__aligned__(8))); /*  2800     0 */

	/* size: 2800, cachelines: 44, members: 68 */
	/* sum members: 2767, holes: 8, sum holes: 31 */
	/* sum bitfield members: 12 bits, bit holes: 1, sum bit holes: 4 bits */
	/* forced alignments: 3, forced holes: 1, sum forced holes: 4 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct scsi_target {
	struct scsi_device *       starget_sdev_user;    /*     0     8 */
	struct list_head           siblings;             /*     8    16 */
	struct list_head           devices;              /*    24    16 */
	struct device              dev __attribute__((__aligned__(8))); /*    40  1000 */
	/* --- cacheline 16 boundary (1024 bytes) was 16 bytes ago --- */
	struct kref                reap_ref;             /*  1040     4 */
	unsigned int               channel;              /*  1044     4 */
	unsigned int               id;                   /*  1048     4 */
	unsigned int               create:1;             /*  1052: 0  4 */
	unsigned int               single_lun:1;         /*  1052: 1  4 */
	unsigned int               pdt_1f_for_no_lun:1;  /*  1052: 2  4 */
	unsigned int               no_report_luns:1;     /*  1052: 3  4 */
	unsigned int               expecting_lun_change:1; /*  1052: 4  4 */

	/* XXX 27 bits hole, try to pack */

	atomic_t                   target_busy;          /*  1056     4 */
	atomic_t                   target_blocked;       /*  1060     4 */
	unsigned int               can_queue;            /*  1064     4 */
	unsigned int               max_target_blocked;   /*  1068     4 */
	char                       scsi_level;           /*  1072     1 */

	/* XXX 3 bytes hole, try to pack */

	enum scsi_target_state     state;                /*  1076     4 */
	void *                     hostdata;             /*  1080     8 */
	/* --- cacheline 17 boundary (1088 bytes) --- */
	u64                        kabi_reserved1;       /*  1088     8 */
	u64                        kabi_reserved2;       /*  1096     8 */
	u64                        kabi_reserved3;       /*  1104     8 */
	u64                        kabi_reserved4;       /*  1112     8 */
	long unsigned int          starget_data[];       /*  1120     0 */

	/* size: 1120, cachelines: 18, members: 24 */
	/* sum members: 1113, holes: 1, sum holes: 3 */
	/* sum bitfield members: 5 bits, bit holes: 1, sum bit holes: 27 bits */
	/* forced alignments: 1 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct scsi_data_buffer {
	struct sg_table            table;                /*     0    16 */
	unsigned int               length;               /*    16     4 */

	/* size: 24, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct scsi_pointer {
	char *                     ptr;                  /*     0     8 */
	int                        this_residual;        /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct scatterlist *       buffer;               /*    16     8 */
	int                        buffers_residual;     /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	dma_addr_t                 dma_handle;           /*    32     8 */
	volatile int               Status;               /*    40     4 */
	volatile int               Message;              /*    44     4 */
	volatile int               have_data_in;         /*    48     4 */
	volatile int               sent_command;         /*    52     4 */
	volatile int               phase;                /*    56     4 */

	/* size: 64, cachelines: 1, members: 10 */
	/* sum members: 52, holes: 2, sum holes: 8 */
	/* padding: 4 */
};
struct scsi_cmnd {
	struct scsi_request        req;                  /*     0    56 */
	struct scsi_device *       device;               /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct list_head           eh_entry;             /*    64    16 */
	struct delayed_work        abort_work;           /*    80   192 */
	/* --- cacheline 4 boundary (256 bytes) was 16 bytes ago --- */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*   272    16 */
	int                        eh_eflags;            /*   288     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          jiffies_at_alloc;     /*   296     8 */
	int                        retries;              /*   304     4 */
	int                        allowed;              /*   308     4 */
	unsigned char              prot_op;              /*   312     1 */
	unsigned char              prot_type;            /*   313     1 */
	unsigned char              prot_flags;           /*   314     1 */

	/* XXX 1 byte hole, try to pack */

	short unsigned int         cmd_len;              /*   316     2 */

	/* XXX 2 bytes hole, try to pack */

	/* --- cacheline 5 boundary (320 bytes) --- */
	enum dma_data_direction    sc_data_direction;    /*   320     4 */

	/* XXX 4 bytes hole, try to pack */

	unsigned char *            cmnd;                 /*   328     8 */
	struct scsi_data_buffer    sdb;                  /*   336    24 */

	/* XXX last struct has 4 bytes of padding */

	struct scsi_data_buffer *  prot_sdb;             /*   360     8 */
	unsigned int               underflow;            /*   368     4 */
	unsigned int               transfersize;         /*   372     4 */
	struct request *           request;              /*   376     8 */
	/* --- cacheline 6 boundary (384 bytes) --- */
	unsigned char *            sense_buffer;         /*   384     8 */
	void                       (*scsi_done)(struct scsi_cmnd *); /*   392     8 */
	struct scsi_pointer        SCp;                  /*   400    64 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 7 boundary (448 bytes) was 16 bytes ago --- */
	unsigned char *            host_scribble;        /*   464     8 */
	int                        result;               /*   472     4 */
	int                        flags;                /*   476     4 */
	long unsigned int          state;                /*   480     8 */
	unsigned char              tag;                  /*   488     1 */

	/* XXX 3 bytes hole, try to pack */

	unsigned int               extra_len;            /*   492     4 */
	u64                        kabi_reserved1;       /*   496     8 */
	u64                        kabi_reserved2;       /*   504     8 */
	/* --- cacheline 8 boundary (512 bytes) --- */
	u64                        kabi_reserved3;       /*   512     8 */
	u64                        kabi_reserved4;       /*   520     8 */

	/* size: 528, cachelines: 9, members: 33 */
	/* sum members: 514, holes: 5, sum holes: 14 */
	/* paddings: 2, sum paddings: 8 */
	/* forced alignments: 1 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct scsi_driver {
	struct device_driver       gendrv;               /*     0   192 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	void                       (*rescan)(struct device *); /*   192     8 */
	blk_status_t               (*init_command)(struct scsi_cmnd *); /*   200     8 */
	void                       (*uninit_command)(struct scsi_cmnd *); /*   208     8 */
	int                        (*done)(struct scsi_cmnd *); /*   216     8 */
	int                        (*eh_action)(struct scsi_cmnd *, int); /*   224     8 */
	void                       (*eh_reset)(struct scsi_cmnd *); /*   232     8 */

	/* size: 240, cachelines: 4, members: 7 */
	/* last cacheline: 48 bytes */
};
struct scsi_host_template {
	struct module *            module;               /*     0     8 */
	const char  *              name;                 /*     8     8 */
	const char  *              (*info)(struct Scsi_Host *); /*    16     8 */
	int                        (*ioctl)(struct scsi_device *, unsigned int, void *); /*    24     8 */
	int                        (*compat_ioctl)(struct scsi_device *, unsigned int, void *); /*    32     8 */
	int                        (*init_cmd_priv)(struct Scsi_Host *, struct scsi_cmnd *); /*    40     8 */
	int                        (*exit_cmd_priv)(struct Scsi_Host *, struct scsi_cmnd *); /*    48     8 */
	int                        (*queuecommand)(struct Scsi_Host *, struct scsi_cmnd *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	void                       (*commit_rqs)(struct Scsi_Host *, u16); /*    64     8 */
	int                        (*eh_abort_handler)(struct scsi_cmnd *); /*    72     8 */
	int                        (*eh_device_reset_handler)(struct scsi_cmnd *); /*    80     8 */
	int                        (*eh_target_reset_handler)(struct scsi_cmnd *); /*    88     8 */
	int                        (*eh_bus_reset_handler)(struct scsi_cmnd *); /*    96     8 */
	int                        (*eh_host_reset_handler)(struct scsi_cmnd *); /*   104     8 */
	int                        (*slave_alloc)(struct scsi_device *); /*   112     8 */
	int                        (*slave_configure)(struct scsi_device *); /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	void                       (*slave_destroy)(struct scsi_device *); /*   128     8 */
	int                        (*target_alloc)(struct scsi_target *); /*   136     8 */
	void                       (*target_destroy)(struct scsi_target *); /*   144     8 */
	int                        (*scan_finished)(struct Scsi_Host *, long unsigned int); /*   152     8 */
	void                       (*scan_start)(struct Scsi_Host *); /*   160     8 */
	int                        (*change_queue_depth)(struct scsi_device *, int); /*   168     8 */
	int                        (*map_queues)(struct Scsi_Host *); /*   176     8 */
	bool                       (*dma_need_drain)(struct request *); /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	int                        (*bios_param)(struct scsi_device *, struct block_device *, sector_t, int *); /*   192     8 */
	void                       (*unlock_native_capacity)(struct scsi_device *); /*   200     8 */
	int                        (*show_info)(struct seq_file *, struct Scsi_Host *); /*   208     8 */
	int                        (*write_info)(struct Scsi_Host *, char *, int); /*   216     8 */
	enum blk_eh_timer_return   (*eh_timed_out)(struct scsi_cmnd *); /*   224     8 */
	int                        (*host_reset)(struct Scsi_Host *, int); /*   232     8 */
	const char  *              proc_name;            /*   240     8 */
	struct proc_dir_entry *    proc_dir;             /*   248     8 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	int                        can_queue;            /*   256     4 */
	int                        this_id;              /*   260     4 */
	short unsigned int         sg_tablesize;         /*   264     2 */
	short unsigned int         sg_prot_tablesize;    /*   266     2 */
	unsigned int               max_sectors;          /*   268     4 */
	unsigned int               max_segment_size;     /*   272     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          dma_boundary;         /*   280     8 */
	long unsigned int          virt_boundary_mask;   /*   288     8 */
	short int                  cmd_per_lun;          /*   296     2 */
	unsigned char              present;              /*   298     1 */

	/* XXX 1 byte hole, try to pack */

	int                        tag_alloc_policy;     /*   300     4 */
	unsigned int               track_queue_depth:1;  /*   304: 0  4 */
	unsigned int               supported_mode:2;     /*   304: 1  4 */
	unsigned int               unchecked_isa_dma:1;  /*   304: 3  4 */
	unsigned int               emulated:1;           /*   304: 4  4 */
	unsigned int               skip_settle_delay:1;  /*   304: 5  4 */
	unsigned int               no_write_same:1;      /*   304: 6  4 */
	unsigned int               host_tagset:1;        /*   304: 7  4 */

	/* XXX 24 bits hole, try to pack */

	unsigned int               max_host_blocked;     /*   308     4 */
	struct device_attribute * * shost_attrs;         /*   312     8 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	struct device_attribute * * sdev_attrs;          /*   320     8 */
	const struct attribute_group  * * sdev_groups;   /*   328     8 */
	u64                        vendor_id;            /*   336     8 */
	unsigned int               cmd_size;             /*   344     4 */

	/* XXX 4 bytes hole, try to pack */

	struct scsi_host_cmd_pool * cmd_pool;            /*   352     8 */
	int                        rpm_autosuspend_delay; /*   360     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        kabi_reserved1;       /*   368     8 */
	u64                        kabi_reserved2;       /*   376     8 */
	/* --- cacheline 6 boundary (384 bytes) --- */
	u64                        kabi_reserved3;       /*   384     8 */
	u64                        kabi_reserved4;       /*   392     8 */

	/* size: 400, cachelines: 7, members: 62 */
	/* sum members: 383, holes: 4, sum holes: 13 */
	/* sum bitfield members: 8 bits, bit holes: 1, sum bit holes: 24 bits */
	/* last cacheline: 16 bytes */
};
struct trace_event_raw_scsi_dispatch_cmd_start {
	struct trace_entry         ent;                  /*     0     8 */
	unsigned int               host_no;              /*     8     4 */
	unsigned int               channel;              /*    12     4 */
	unsigned int               id;                   /*    16     4 */
	unsigned int               lun;                  /*    20     4 */
	unsigned int               opcode;               /*    24     4 */
	unsigned int               cmd_len;              /*    28     4 */
	unsigned int               data_sglen;           /*    32     4 */
	unsigned int               prot_sglen;           /*    36     4 */
	unsigned char              prot_op;              /*    40     1 */

	/* XXX 3 bytes hole, try to pack */

	u32                        __data_loc_cmnd;      /*    44     4 */
	char                       __data[];             /*    48     0 */

	/* size: 48, cachelines: 1, members: 12 */
	/* sum members: 45, holes: 1, sum holes: 3 */
	/* last cacheline: 48 bytes */
};
struct trace_event_raw_scsi_dispatch_cmd_error {
	struct trace_entry         ent;                  /*     0     8 */
	unsigned int               host_no;              /*     8     4 */
	unsigned int               channel;              /*    12     4 */
	unsigned int               id;                   /*    16     4 */
	unsigned int               lun;                  /*    20     4 */
	int                        rtn;                  /*    24     4 */
	unsigned int               opcode;               /*    28     4 */
	unsigned int               cmd_len;              /*    32     4 */
	unsigned int               data_sglen;           /*    36     4 */
	unsigned int               prot_sglen;           /*    40     4 */
	unsigned char              prot_op;              /*    44     1 */

	/* XXX 3 bytes hole, try to pack */

	u32                        __data_loc_cmnd;      /*    48     4 */
	char                       __data[];             /*    52     0 */

	/* size: 52, cachelines: 1, members: 13 */
	/* sum members: 49, holes: 1, sum holes: 3 */
	/* last cacheline: 52 bytes */
};
struct trace_event_raw_scsi_cmd_done_timeout_template {
	struct trace_entry         ent;                  /*     0     8 */
	unsigned int               host_no;              /*     8     4 */
	unsigned int               channel;              /*    12     4 */
	unsigned int               id;                   /*    16     4 */
	unsigned int               lun;                  /*    20     4 */
	int                        result;               /*    24     4 */
	unsigned int               opcode;               /*    28     4 */
	unsigned int               cmd_len;              /*    32     4 */
	unsigned int               data_sglen;           /*    36     4 */
	unsigned int               prot_sglen;           /*    40     4 */
	unsigned char              prot_op;              /*    44     1 */

	/* XXX 3 bytes hole, try to pack */

	u32                        __data_loc_cmnd;      /*    48     4 */
	char                       __data[];             /*    52     0 */

	/* size: 52, cachelines: 1, members: 13 */
	/* sum members: 49, holes: 1, sum holes: 3 */
	/* last cacheline: 52 bytes */
};
struct trace_event_raw_scsi_eh_wakeup {
	struct trace_entry         ent;                  /*     0     8 */
	unsigned int               host_no;              /*     8     4 */
	char                       __data[];             /*    12     0 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct trace_event_data_offsets_scsi_dispatch_cmd_start {
	u32                        cmnd;                 /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct trace_event_data_offsets_scsi_dispatch_cmd_error {
	u32                        cmnd;                 /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct trace_event_data_offsets_scsi_cmd_done_timeout_template {
	u32                        cmnd;                 /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct trace_event_data_offsets_scsi_eh_wakeup {

	/* size: 0, cachelines: 0, members: 0 */
};
struct scsi_transport_template {
	struct transport_container host_attrs __attribute__((__aligned__(8))); /*     0   104 */
	/* --- cacheline 1 boundary (64 bytes) was 40 bytes ago --- */
	struct transport_container target_attrs __attribute__((__aligned__(8))); /*   104   104 */
	/* --- cacheline 3 boundary (192 bytes) was 16 bytes ago --- */
	struct transport_container device_attrs __attribute__((__aligned__(8))); /*   208   104 */
	/* --- cacheline 4 boundary (256 bytes) was 56 bytes ago --- */
	int                        (*user_scan)(struct Scsi_Host *, uint, uint, u64); /*   312     8 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	int                        device_size;          /*   320     4 */
	int                        device_private_offset; /*   324     4 */
	int                        target_size;          /*   328     4 */
	int                        target_private_offset; /*   332     4 */
	int                        host_size;            /*   336     4 */
	unsigned int               create_work_queue:1;  /*   340: 0  4 */

	/* XXX 31 bits hole, try to pack */

	void                       (*eh_strategy_handler)(struct Scsi_Host *); /*   344     8 */

	/* size: 352, cachelines: 6, members: 11 */
	/* sum members: 348 */
	/* sum bitfield members: 1 bits, bit holes: 1, sum bit holes: 31 bits */
	/* forced alignments: 3 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct scsi_host_busy_iter_data {
	bool                       (*fn)(struct scsi_cmnd *, void *, bool); /*     0     8 */
	void *                     priv;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct scsi_idlun {
	__u32                      dev_id;               /*     0     4 */
	__u32                      host_unique_id;       /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct scsi_device_handler {
	struct list_head           list;                 /*     0    16 */
	struct module *            module;               /*    16     8 */
	const char  *              name;                 /*    24     8 */
	int                        (*check_sense)(struct scsi_device *, struct scsi_sense_hdr *); /*    32     8 */
	int                        (*attach)(struct scsi_device *); /*    40     8 */
	void                       (*detach)(struct scsi_device *); /*    48     8 */
	int                        (*activate)(struct scsi_device *, activate_complete, void *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	blk_status_t               (*prep_fn)(struct scsi_device *, struct request *); /*    64     8 */
	int                        (*set_params)(struct scsi_device *, const char  *); /*    72     8 */
	void                       (*rescan)(struct scsi_device *); /*    80     8 */

	/* size: 88, cachelines: 2, members: 10 */
	/* last cacheline: 24 bytes */
};
struct scsi_eh_save {
	int                        result;               /*     0     4 */
	unsigned int               resid_len;            /*     4     4 */
	int                        eh_eflags;            /*     8     4 */
	enum dma_data_direction    data_direction;       /*    12     4 */
	unsigned int               underflow;            /*    16     4 */
	unsigned char              cmd_len;              /*    20     1 */
	unsigned char              prot_op;              /*    21     1 */

	/* XXX 2 bytes hole, try to pack */

	unsigned char *            cmnd;                 /*    24     8 */
	struct scsi_data_buffer    sdb;                  /*    32    24 */

	/* XXX last struct has 4 bytes of padding */

	unsigned char              eh_cmnd[16];          /*    56    16 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	struct scatterlist         sense_sgl;            /*    72    32 */

	/* XXX last struct has 4 bytes of padding */

	/* size: 104, cachelines: 2, members: 11 */
	/* sum members: 102, holes: 1, sum holes: 2 */
	/* paddings: 2, sum paddings: 8 */
	/* last cacheline: 40 bytes */
};
struct scsi_mode_data {
	__u32                      length;               /*     0     4 */
	__u16                      block_descriptor_length; /*     4     2 */
	__u8                       medium_type;          /*     6     1 */
	__u8                       device_specific;      /*     7     1 */
	__u8                       header_length;        /*     8     1 */
	__u8                       longlba:1;            /*     9: 0  1 */

	/* size: 12, cachelines: 1, members: 6 */
	/* padding: 2 */
	/* bit_padding: 7 bits */
	/* last cacheline: 12 bytes */
};
struct scsi_event {
	enum scsi_device_event     evt_type;             /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           node;                 /*     8    16 */

	/* size: 24, cachelines: 1, members: 2 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct sa_name_list {
	int                        opcode;               /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	const struct value_name_pair  * arr;             /*     8     8 */
	int                        arr_sz;               /*    16     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 16, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct value_name_pair {
	int                        value;                /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	const char  *              name;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct error_info {
	short unsigned int         code12;               /*     0     2 */
	short unsigned int         size;                 /*     2     2 */

	/* size: 4, cachelines: 1, members: 2 */
	/* last cacheline: 4 bytes */
};
struct error_info2 {
	unsigned char              code1;                /*     0     1 */
	unsigned char              code2_min;            /*     1     1 */
	unsigned char              code2_max;            /*     2     1 */

	/* XXX 5 bytes hole, try to pack */

	const char  *              str;                  /*     8     8 */
	const char  *              fmt;                  /*    16     8 */

	/* size: 24, cachelines: 1, members: 5 */
	/* sum members: 19, holes: 1, sum holes: 5 */
	/* last cacheline: 24 bytes */
};
struct scsi_lun {
	__u8                       scsi_lun[8];          /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct async_scan_data {
	struct list_head           list;                 /*     0    16 */
	struct Scsi_Host *         shost;                /*    16     8 */
	struct completion          prev_finished;        /*    24    32 */

	/* size: 56, cachelines: 1, members: 3 */
	/* last cacheline: 56 bytes */
};
struct scsi_dev_info_list {
	struct list_head           dev_info_list;        /*     0    16 */
	char                       vendor[8];            /*    16     8 */
	char                       model[16];            /*    24    16 */
	blist_flags_t              flags;                /*    40     8 */
	unsigned int               compatible;           /*    48     4 */

	/* size: 56, cachelines: 1, members: 5 */
	/* padding: 4 */
	/* last cacheline: 56 bytes */
};
struct scsi_dev_info_list_table {
	struct list_head           node;                 /*     0    16 */
	struct list_head           scsi_dev_info_list;   /*    16    16 */
	const char  *              name;                 /*    32     8 */
	int                        key;                  /*    40     4 */

	/* size: 48, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 48 bytes */
};
struct double_list {
	struct list_head *         top;                  /*     0     8 */
	struct list_head *         bottom;               /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct scsi_nl_hdr {
	__u8                       version;              /*     0     1 */
	__u8                       transport;            /*     1     1 */
	__u16                      magic;                /*     2     2 */
	__u16                      msgtype;              /*     4     2 */
	__u16                      msglen;               /*     6     2 */

	/* size: 8, cachelines: 1, members: 5 */
	/* last cacheline: 8 bytes */
} __attribute__((__aligned__(8)));
struct scsi_varlen_cdb_hdr {
	__u8                       opcode;               /*     0     1 */
	__u8                       control;              /*     1     1 */
	__u8                       misc[5];              /*     2     5 */
	__u8                       additional_cdb_length; /*     7     1 */
	__be16                     service_action;       /*     8     2 */

	/* size: 10, cachelines: 1, members: 5 */
	/* last cacheline: 10 bytes */
};
struct scsi_dh_blist {
	const char  *              vendor;               /*     0     8 */
	const char  *              model;                /*     8     8 */
	const char  *              driver;               /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct rdac_mode_6_hdr {
	u8                         data_len;             /*     0     1 */
	u8                         medium_type;          /*     1     1 */
	u8                         device_params;        /*     2     1 */
	u8                         block_desc_len;       /*     3     1 */

	/* size: 4, cachelines: 1, members: 4 */
	/* last cacheline: 4 bytes */
};
struct rdac_mode_10_hdr {
	u16                        data_len;             /*     0     2 */
	u8                         medium_type;          /*     2     1 */
	u8                         device_params;        /*     3     1 */
	u16                        reserved;             /*     4     2 */
	u16                        block_desc_len;       /*     6     2 */

	/* size: 8, cachelines: 1, members: 5 */
	/* last cacheline: 8 bytes */
};
struct rdac_mode_common {
	u8                         controller_serial[16]; /*     0    16 */
	u8                         alt_controller_serial[16]; /*    16    16 */
	u8                         rdac_mode[2];         /*    32     2 */
	u8                         alt_rdac_mode[2];     /*    34     2 */
	u8                         quiescence_timeout;   /*    36     1 */
	u8                         rdac_options;         /*    37     1 */

	/* size: 38, cachelines: 1, members: 6 */
	/* last cacheline: 38 bytes */
};
struct rdac_pg_legacy {
	struct rdac_mode_6_hdr     hdr;                  /*     0     4 */
	u8                         page_code;            /*     4     1 */
	u8                         page_len;             /*     5     1 */
	struct rdac_mode_common    common;               /*     6    38 */
	u8                         lun_table[32];        /*    44    32 */
	/* --- cacheline 1 boundary (64 bytes) was 12 bytes ago --- */
	u8                         reserved2[32];        /*    76    32 */
	u8                         reserved3;            /*   108     1 */
	u8                         reserved4;            /*   109     1 */

	/* size: 110, cachelines: 2, members: 8 */
	/* last cacheline: 46 bytes */
};
struct rdac_pg_expanded {
	struct rdac_mode_10_hdr    hdr;                  /*     0     8 */
	u8                         page_code;            /*     8     1 */
	u8                         subpage_code;         /*     9     1 */
	u8                         page_len[2];          /*    10     2 */
	struct rdac_mode_common    common;               /*    12    38 */
	u8                         lun_table[256];       /*    50   256 */
	/* --- cacheline 4 boundary (256 bytes) was 50 bytes ago --- */
	u8                         reserved3;            /*   306     1 */
	u8                         reserved4;            /*   307     1 */

	/* size: 308, cachelines: 5, members: 8 */
	/* last cacheline: 52 bytes */
};
struct c9_inquiry {
	u8                         peripheral_info;      /*     0     1 */
	u8                         page_code;            /*     1     1 */
	u8                         reserved1;            /*     2     1 */
	u8                         page_len;             /*     3     1 */
	u8                         page_id[4];           /*     4     4 */
	u8                         avte_cvp;             /*     8     1 */
	u8                         path_prio;            /*     9     1 */
	u8                         reserved2[38];        /*    10    38 */

	/* size: 48, cachelines: 1, members: 8 */
	/* last cacheline: 48 bytes */
};
struct c4_inquiry {
	u8                         peripheral_info;      /*     0     1 */
	u8                         page_code;            /*     1     1 */
	u8                         reserved1;            /*     2     1 */
	u8                         page_len;             /*     3     1 */
	u8                         page_id[4];           /*     4     4 */
	u8                         subsys_id[16];        /*     8    16 */
	u8                         revision[4];          /*    24     4 */
	u8                         slot_id[2];           /*    28     2 */
	u8                         reserved[2];          /*    30     2 */

	/* size: 32, cachelines: 1, members: 9 */
	/* last cacheline: 32 bytes */
};
struct c8_inquiry {
	u8                         peripheral_info;      /*     0     1 */
	u8                         page_code;            /*     1     1 */
	u8                         reserved1;            /*     2     1 */
	u8                         page_len;             /*     3     1 */
	u8                         page_id[4];           /*     4     4 */
	u8                         reserved2[3];         /*     8     3 */
	u8                         vol_uniq_id_len;      /*    11     1 */
	u8                         vol_uniq_id[16];      /*    12    16 */
	u8                         vol_user_label_len;   /*    28     1 */
	u8                         vol_user_label[60];   /*    29    60 */
	/* --- cacheline 1 boundary (64 bytes) was 25 bytes ago --- */
	u8                         array_uniq_id_len;    /*    89     1 */
	u8                         array_unique_id[16];  /*    90    16 */
	u8                         array_user_label_len; /*   106     1 */
	u8                         array_user_label[60]; /*   107    60 */
	/* --- cacheline 2 boundary (128 bytes) was 39 bytes ago --- */
	u8                         lun[8];               /*   167     8 */

	/* size: 175, cachelines: 3, members: 15 */
	/* last cacheline: 47 bytes */
};
struct rdac_controller {
	u8                         array_id[16];         /*     0    16 */
	int                        use_ms10;             /*    16     4 */
	struct kref                kref;                 /*    20     4 */
	struct list_head           node;                 /*    24    16 */
	union {
		struct rdac_pg_legacy legacy;            /*    40   110 */
		struct rdac_pg_expanded expanded;        /*    40   308 */
	} mode_select;                                   /*    40   308 */
	/* --- cacheline 5 boundary (320 bytes) was 28 bytes ago --- */
	u8                         index;                /*   348     1 */
	u8                         array_name[31];       /*   349    31 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 6 boundary (384 bytes) --- */
	struct Scsi_Host *         host;                 /*   384     8 */
	spinlock_t                 ms_lock;              /*   392     4 */
	int                        ms_queued;            /*   396     4 */
	struct work_struct         ms_work;              /*   400    64 */
	/* --- cacheline 7 boundary (448 bytes) was 16 bytes ago --- */
	struct scsi_device *       ms_sdev;              /*   464     8 */
	struct list_head           ms_head;              /*   472    16 */
	struct list_head           dh_list;              /*   488    16 */

	/* size: 504, cachelines: 8, members: 14 */
	/* sum members: 500, holes: 1, sum holes: 4 */
	/* last cacheline: 56 bytes */
};
struct c2_inquiry {
	u8                         peripheral_info;      /*     0     1 */
	u8                         page_code;            /*     1     1 */
	u8                         reserved1;            /*     2     1 */
	u8                         page_len;             /*     3     1 */
	u8                         page_id[4];           /*     4     4 */
	u8                         sw_version[3];        /*     8     3 */
	u8                         sw_date[3];           /*    11     3 */
	u8                         features_enabled;     /*    14     1 */
	u8                         max_lun_supported;    /*    15     1 */
	u8                         partitions[239];      /*    16   239 */

	/* size: 255, cachelines: 4, members: 10 */
	/* last cacheline: 63 bytes */
};
struct rdac_dh_data {
	struct list_head           node;                 /*     0    16 */
	struct rdac_controller *   ctlr;                 /*    16     8 */
	struct scsi_device *       sdev;                 /*    24     8 */
	unsigned int               lun;                  /*    32     4 */
	unsigned char              mode;                 /*    36     1 */
	unsigned char              state;                /*    37     1 */
	char                       lun_state;            /*    38     1 */
	char                       preferred;            /*    39     1 */
	union {
		struct c2_inquiry  c2;                   /*    40   255 */
		struct c4_inquiry  c4;                   /*    40    32 */
		struct c8_inquiry  c8;                   /*    40   175 */
		struct c9_inquiry  c9;                   /*    40    48 */
	} inq;                                           /*    40   255 */

	/* size: 296, cachelines: 5, members: 9 */
	/* padding: 1 */
	/* last cacheline: 40 bytes */
};
struct rdac_queue_data {
	struct list_head           entry;                /*     0    16 */
	struct rdac_dh_data *      h;                    /*    16     8 */
	activate_complete          callback_fn;          /*    24     8 */
	void *                     callback_data;        /*    32     8 */

	/* size: 40, cachelines: 1, members: 4 */
	/* last cacheline: 40 bytes */
};
struct hp_sw_dh_data {
	int                        path_state;           /*     0     4 */
	int                        retries;              /*     4     4 */
	int                        retry_cnt;            /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct scsi_device *       sdev;                 /*    16     8 */

	/* size: 24, cachelines: 1, members: 4 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct clariion_dh_data {
	unsigned int               flags;                /*     0     4 */
	unsigned char              buffer[252];          /*     4   252 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	int                        lun_state;            /*   256     4 */
	int                        port;                 /*   260     4 */
	int                        default_sp;           /*   264     4 */
	int                        current_sp;           /*   268     4 */

	/* size: 272, cachelines: 5, members: 6 */
	/* last cacheline: 16 bytes */
};
struct alua_port_group {
	struct kref                kref;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct callback_head       rcu __attribute__((__aligned__(8))); /*     8    16 */
	struct list_head           node;                 /*    24    16 */
	struct list_head           dh_list;              /*    40    16 */
	unsigned char              device_id_str[256];   /*    56   256 */
	/* --- cacheline 4 boundary (256 bytes) was 56 bytes ago --- */
	int                        device_id_len;        /*   312     4 */
	int                        group_id;             /*   316     4 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	int                        tpgs;                 /*   320     4 */
	int                        state;                /*   324     4 */
	int                        pref;                 /*   328     4 */
	int                        valid_states;         /*   332     4 */
	unsigned int               flags;                /*   336     4 */
	unsigned char              transition_tmo;       /*   340     1 */

	/* XXX 3 bytes hole, try to pack */

	long unsigned int          expiry;               /*   344     8 */
	long unsigned int          interval;             /*   352     8 */
	struct delayed_work        rtpg_work;            /*   360   192 */
	/* --- cacheline 8 boundary (512 bytes) was 40 bytes ago --- */
	spinlock_t                 lock;                 /*   552     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           rtpg_list;            /*   560    16 */
	/* --- cacheline 9 boundary (576 bytes) --- */
	struct scsi_device *       rtpg_sdev;            /*   576     8 */

	/* size: 584, cachelines: 10, members: 19 */
	/* sum members: 573, holes: 3, sum holes: 11 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 4 */
	/* last cacheline: 8 bytes */
} __attribute__((__aligned__(8)));
struct alua_dh_data {
	struct list_head           node;                 /*     0    16 */
	struct alua_port_group *   pg;                   /*    16     8 */
	int                        group_id;             /*    24     4 */
	spinlock_t                 pg_lock;              /*    28     4 */
	struct scsi_device *       sdev;                 /*    32     8 */
	int                        init_error;           /*    40     4 */

	/* XXX 4 bytes hole, try to pack */

	struct mutex               init_mutex;           /*    48    32 */

	/* size: 80, cachelines: 2, members: 7 */
	/* sum members: 76, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct alua_queue_data {
	struct list_head           entry;                /*     0    16 */
	activate_complete          callback_fn;          /*    16     8 */
	void *                     callback_data;        /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* last cacheline: 32 bytes */
};
struct spi_device_id {
	char                       name[32];             /*     0    32 */
	kernel_ulong_t             driver_data;          /*    32     8 */

	/* size: 40, cachelines: 1, members: 2 */
	/* last cacheline: 40 bytes */
};
struct ptp_system_timestamp {
	struct timespec64          pre_ts;               /*     0    16 */
	struct timespec64          post_ts;              /*    16    16 */

	/* size: 32, cachelines: 1, members: 2 */
	/* last cacheline: 32 bytes */
};
struct spi_statistics {
	spinlock_t                 lock;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          messages;             /*     8     8 */
	long unsigned int          transfers;            /*    16     8 */
	long unsigned int          errors;               /*    24     8 */
	long unsigned int          timedout;             /*    32     8 */
	long unsigned int          spi_sync;             /*    40     8 */
	long unsigned int          spi_sync_immediate;   /*    48     8 */
	long unsigned int          spi_async;            /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	long long unsigned int     bytes;                /*    64     8 */
	long long unsigned int     bytes_rx;             /*    72     8 */
	long long unsigned int     bytes_tx;             /*    80     8 */
	long unsigned int          transfer_bytes_histo[17]; /*    88   136 */
	/* --- cacheline 3 boundary (192 bytes) was 32 bytes ago --- */
	long unsigned int          transfers_split_maxsize; /*   224     8 */

	/* size: 232, cachelines: 4, members: 13 */
	/* sum members: 228, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct spi_delay {
	u16                        value;                /*     0     2 */
	u8                         unit;                 /*     2     1 */

	/* size: 4, cachelines: 1, members: 2 */
	/* padding: 1 */
	/* last cacheline: 4 bytes */
};
struct spi_device {
	struct device              dev __attribute__((__aligned__(8))); /*     0  1000 */
	/* --- cacheline 15 boundary (960 bytes) was 40 bytes ago --- */
	struct spi_controller *    controller;           /*  1000     8 */
	struct spi_controller *    master;               /*  1008     8 */
	u32                        max_speed_hz;         /*  1016     4 */
	u8                         chip_select;          /*  1020     1 */
	u8                         bits_per_word;        /*  1021     1 */
	bool                       rt;                   /*  1022     1 */

	/* XXX 1 byte hole, try to pack */

	/* --- cacheline 16 boundary (1024 bytes) --- */
	u32                        mode;                 /*  1024     4 */
	int                        irq;                  /*  1028     4 */
	void *                     controller_state;     /*  1032     8 */
	void *                     controller_data;      /*  1040     8 */
	char                       modalias[32];         /*  1048    32 */
	const char  *              driver_override;      /*  1080     8 */
	/* --- cacheline 17 boundary (1088 bytes) --- */
	int                        cs_gpio;              /*  1088     4 */

	/* XXX 4 bytes hole, try to pack */

	struct gpio_desc *         cs_gpiod;             /*  1096     8 */
	struct spi_delay           word_delay;           /*  1104     4 */

	/* XXX last struct has 1 byte of padding */
	/* XXX 4 bytes hole, try to pack */

	struct spi_statistics      statistics;           /*  1112   232 */

	/* size: 1344, cachelines: 21, members: 17 */
	/* sum members: 1335, holes: 3, sum holes: 9 */
	/* paddings: 1, sum paddings: 1 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(8)));
struct spi_controller {
	struct device              dev __attribute__((__aligned__(8))); /*     0  1000 */
	/* --- cacheline 15 boundary (960 bytes) was 40 bytes ago --- */
	struct list_head           list;                 /*  1000    16 */
	s16                        bus_num;              /*  1016     2 */
	u16                        num_chipselect;       /*  1018     2 */
	u16                        dma_alignment;        /*  1020     2 */

	/* XXX 2 bytes hole, try to pack */

	/* --- cacheline 16 boundary (1024 bytes) --- */
	u32                        mode_bits;            /*  1024     4 */
	u32                        buswidth_override_bits; /*  1028     4 */
	u32                        bits_per_word_mask;   /*  1032     4 */
	u32                        min_speed_hz;         /*  1036     4 */
	u32                        max_speed_hz;         /*  1040     4 */
	u16                        flags;                /*  1044     2 */
	bool                       devm_allocated;       /*  1046     1 */
	bool                       slave;                /*  1047     1 */
	size_t                     (*max_transfer_size)(struct spi_device *); /*  1048     8 */
	size_t                     (*max_message_size)(struct spi_device *); /*  1056     8 */
	struct mutex               io_mutex;             /*  1064    32 */
	/* --- cacheline 17 boundary (1088 bytes) was 8 bytes ago --- */
	spinlock_t                 bus_lock_spinlock;    /*  1096     4 */

	/* XXX 4 bytes hole, try to pack */

	struct mutex               bus_lock_mutex;       /*  1104    32 */
	bool                       bus_lock_flag;        /*  1136     1 */

	/* XXX 7 bytes hole, try to pack */

	int                        (*setup)(struct spi_device *); /*  1144     8 */
	/* --- cacheline 18 boundary (1152 bytes) --- */
	int                        (*set_cs_timing)(struct spi_device *, struct spi_delay *, struct spi_delay *, struct spi_delay *); /*  1152     8 */
	int                        (*transfer)(struct spi_device *, struct spi_message *); /*  1160     8 */
	void                       (*cleanup)(struct spi_device *); /*  1168     8 */
	bool                       (*can_dma)(struct spi_controller *, struct spi_device *, struct spi_transfer *); /*  1176     8 */
	bool                       queued;               /*  1184     1 */

	/* XXX 7 bytes hole, try to pack */

	struct kthread_worker *    kworker;              /*  1192     8 */
	struct kthread_work        pump_messages;        /*  1200    40 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 19 boundary (1216 bytes) was 24 bytes ago --- */
	spinlock_t                 queue_lock;           /*  1240     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           queue;                /*  1248    16 */
	struct spi_message *       cur_msg;              /*  1264     8 */
	bool                       idling;               /*  1272     1 */
	bool                       busy;                 /*  1273     1 */
	bool                       running;              /*  1274     1 */
	bool                       rt;                   /*  1275     1 */
	bool                       auto_runtime_pm;      /*  1276     1 */
	bool                       cur_msg_prepared;     /*  1277     1 */
	bool                       cur_msg_mapped;       /*  1278     1 */
	bool                       last_cs_enable;       /*  1279     1 */
	/* --- cacheline 20 boundary (1280 bytes) --- */
	bool                       last_cs_mode_high;    /*  1280     1 */
	bool                       fallback;             /*  1281     1 */

	/* XXX 6 bytes hole, try to pack */

	struct completion          xfer_completion;      /*  1288    32 */
	size_t                     max_dma_len;          /*  1320     8 */
	int                        (*prepare_transfer_hardware)(struct spi_controller *); /*  1328     8 */
	int                        (*transfer_one_message)(struct spi_controller *, struct spi_message *); /*  1336     8 */
	/* --- cacheline 21 boundary (1344 bytes) --- */
	int                        (*unprepare_transfer_hardware)(struct spi_controller *); /*  1344     8 */
	int                        (*prepare_message)(struct spi_controller *, struct spi_message *); /*  1352     8 */
	int                        (*unprepare_message)(struct spi_controller *, struct spi_message *); /*  1360     8 */
	int                        (*slave_abort)(struct spi_controller *); /*  1368     8 */
	void                       (*set_cs)(struct spi_device *, bool); /*  1376     8 */
	int                        (*transfer_one)(struct spi_controller *, struct spi_device *, struct spi_transfer *); /*  1384     8 */
	void                       (*handle_err)(struct spi_controller *, struct spi_message *); /*  1392     8 */
	const struct spi_controller_mem_ops  * mem_ops;  /*  1400     8 */
	/* --- cacheline 22 boundary (1408 bytes) --- */
	struct spi_delay           cs_setup;             /*  1408     4 */

	/* XXX last struct has 1 byte of padding */

	struct spi_delay           cs_hold;              /*  1412     4 */

	/* XXX last struct has 1 byte of padding */

	struct spi_delay           cs_inactive;          /*  1416     4 */

	/* XXX last struct has 1 byte of padding */
	/* XXX 4 bytes hole, try to pack */

	int *                      cs_gpios;             /*  1424     8 */
	struct gpio_desc * *       cs_gpiods;            /*  1432     8 */
	bool                       use_gpio_descriptors; /*  1440     1 */
	s8                         unused_native_cs;     /*  1441     1 */
	s8                         max_native_cs;        /*  1442     1 */

	/* XXX 5 bytes hole, try to pack */

	struct spi_statistics      statistics;           /*  1448   232 */
	/* --- cacheline 26 boundary (1664 bytes) was 16 bytes ago --- */
	struct dma_chan *          dma_tx;               /*  1680     8 */
	struct dma_chan *          dma_rx;               /*  1688     8 */
	void *                     dummy_rx;             /*  1696     8 */
	void *                     dummy_tx;             /*  1704     8 */
	int                        (*fw_translate_cs)(struct spi_controller *, unsigned int); /*  1712     8 */
	bool                       ptp_sts_supported;    /*  1720     1 */

	/* XXX 7 bytes hole, try to pack */

	/* --- cacheline 27 boundary (1728 bytes) --- */
	long unsigned int          irq_flags;            /*  1728     8 */

	/* size: 1736, cachelines: 28, members: 68 */
	/* sum members: 1690, holes: 9, sum holes: 46 */
	/* paddings: 4, sum paddings: 7 */
	/* forced alignments: 1 */
	/* last cacheline: 8 bytes */
} __attribute__((__aligned__(8)));
struct spi_driver {
	const struct spi_device_id  * id_table;          /*     0     8 */
	int                        (*probe)(struct spi_device *); /*     8     8 */
	int                        (*remove)(struct spi_device *); /*    16     8 */
	void                       (*shutdown)(struct spi_device *); /*    24     8 */
	struct device_driver       driver;               /*    32   192 */

	/* size: 224, cachelines: 4, members: 5 */
	/* last cacheline: 32 bytes */
};
struct spi_message {
	struct list_head           transfers;            /*     0    16 */
	struct spi_device *        spi;                  /*    16     8 */
	unsigned int               is_dma_mapped:1;      /*    24: 0  4 */

	/* XXX 31 bits hole, try to pack */
	/* XXX 4 bytes hole, try to pack */

	void                       (*complete)(void *);  /*    32     8 */
	void *                     context;              /*    40     8 */
	unsigned int               frame_length;         /*    48     4 */
	unsigned int               actual_length;        /*    52     4 */
	int                        status;               /*    56     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	struct list_head           queue;                /*    64    16 */
	void *                     state;                /*    80     8 */
	struct list_head           resources;            /*    88    16 */

	/* size: 104, cachelines: 2, members: 11 */
	/* sum members: 92, holes: 2, sum holes: 8 */
	/* sum bitfield members: 1 bits, bit holes: 1, sum bit holes: 31 bits */
	/* last cacheline: 40 bytes */
};
struct spi_transfer {
	const void  *              tx_buf;               /*     0     8 */
	void *                     rx_buf;               /*     8     8 */
	unsigned int               len;                  /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	dma_addr_t                 tx_dma;               /*    24     8 */
	dma_addr_t                 rx_dma;               /*    32     8 */
	struct sg_table            tx_sg;                /*    40    16 */
	struct sg_table            rx_sg;                /*    56    16 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	unsigned int               cs_change:1;          /*    72: 0  4 */
	unsigned int               tx_nbits:3;           /*    72: 1  4 */
	unsigned int               rx_nbits:3;           /*    72: 4  4 */

	/* XXX 1 bit hole, try to pack */
	/* Bitfield combined with next fields */

	u8                         bits_per_word;        /*    73     1 */
	u16                        delay_usecs;          /*    74     2 */
	struct spi_delay           delay;                /*    76     4 */

	/* XXX last struct has 1 byte of padding */

	struct spi_delay           cs_change_delay;      /*    80     4 */

	/* XXX last struct has 1 byte of padding */

	struct spi_delay           word_delay;           /*    84     4 */

	/* XXX last struct has 1 byte of padding */

	u32                        speed_hz;             /*    88     4 */
	u32                        effective_speed_hz;   /*    92     4 */
	unsigned int               ptp_sts_word_pre;     /*    96     4 */
	unsigned int               ptp_sts_word_post;    /*   100     4 */
	struct ptp_system_timestamp * ptp_sts;           /*   104     8 */
	bool                       timestamped;          /*   112     1 */

	/* XXX 7 bytes hole, try to pack */

	struct list_head           transfer_list;        /*   120    16 */
	/* --- cacheline 2 boundary (128 bytes) was 8 bytes ago --- */
	u16                        error;                /*   136     2 */

	/* size: 144, cachelines: 3, members: 23 */
	/* sum members: 126, holes: 2, sum holes: 11 */
	/* sum bitfield members: 7 bits, bit holes: 1, sum bit holes: 1 bits */
	/* padding: 6 */
	/* paddings: 3, sum paddings: 3 */
	/* last cacheline: 16 bytes */
};
struct spi_controller_mem_ops {
	int                        (*adjust_op_size)(struct spi_mem *, struct spi_mem_op *); /*     0     8 */
	bool                       (*supports_op)(struct spi_mem *, const struct spi_mem_op  *); /*     8     8 */
	int                        (*exec_op)(struct spi_mem *, const struct spi_mem_op  *); /*    16     8 */
	const char  *              (*get_name)(struct spi_mem *); /*    24     8 */
	int                        (*dirmap_create)(struct spi_mem_dirmap_desc *); /*    32     8 */
	void                       (*dirmap_destroy)(struct spi_mem_dirmap_desc *); /*    40     8 */
	ssize_t                    (*dirmap_read)(struct spi_mem_dirmap_desc *, u64, size_t, void *); /*    48     8 */
	ssize_t                    (*dirmap_write)(struct spi_mem_dirmap_desc *, u64, size_t, const void  *); /*    56     8 */

	/* size: 64, cachelines: 1, members: 8 */
};
struct spi_res {
	struct list_head           entry;                /*     0    16 */
	spi_res_release_t          release;              /*    16     8 */
	long long unsigned int     data[];               /*    24     0 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct spi_replaced_transfers {
	spi_replaced_release_t     release;              /*     0     8 */
	void *                     extradata;            /*     8     8 */
	struct list_head           replaced_transfers;   /*    16    16 */
	struct list_head *         replaced_after;       /*    32     8 */
	size_t                     inserted;             /*    40     8 */
	struct spi_transfer        inserted_transfers[]; /*    48     0 */

	/* size: 48, cachelines: 1, members: 6 */
	/* last cacheline: 48 bytes */
};
struct spi_board_info {
	char                       modalias[32];         /*     0    32 */
	const void  *              platform_data;        /*    32     8 */
	const struct property_entry  * properties;       /*    40     8 */
	void *                     controller_data;      /*    48     8 */
	int                        irq;                  /*    56     4 */
	u32                        max_speed_hz;         /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u16                        bus_num;              /*    64     2 */
	u16                        chip_select;          /*    66     2 */
	u32                        mode;                 /*    68     4 */

	/* size: 72, cachelines: 2, members: 9 */
	/* last cacheline: 8 bytes */
};
struct spi_mem_op {
	struct {
		u8                 nbytes;               /*     0     1 */
		u8                 buswidth;             /*     1     1 */
		u8                 dtr:1;                /*     2: 0  1 */

		/* XXX 7 bits hole, try to pack */
		/* XXX 1 byte hole, try to pack */

		u16                opcode;               /*     4     2 */
	} cmd;                                           /*     0     6 */

	/* XXX 2 bytes hole, try to pack */

	struct {
		u8                 nbytes;               /*     8     1 */
		u8                 buswidth;             /*     9     1 */
		u8                 dtr:1;                /*    10: 0  1 */

		/* XXX 7 bits hole, try to pack */
		/* XXX 5 bytes hole, try to pack */

		u64                val;                  /*    16     8 */
	} addr;                                          /*     8    16 */
	struct {
		u8                 nbytes;               /*    24     1 */
		u8                 buswidth;             /*    25     1 */
		u8                 dtr:1;                /*    26: 0  1 */
	} dummy;                                         /*    24     3 */

	/* XXX 5 bytes hole, try to pack */

	struct {
		u8                 buswidth;             /*    32     1 */
		u8                 dtr:1;                /*    33: 0  1 */

		/* XXX 7 bits hole, try to pack */
		/* XXX 2 bytes hole, try to pack */

		enum spi_mem_data_dir dir;               /*    36     4 */
		unsigned int       nbytes;               /*    40     4 */

		/* XXX 4 bytes hole, try to pack */

		union {
			void *     in;                   /*    48     8 */
			const void  * out;               /*    48     8 */
		} buf;                                   /*    48     8 */
	} data;                                          /*    32    24 */

	/* size: 56, cachelines: 1, members: 4 */
	/* sum members: 49, holes: 2, sum holes: 7 */
	/* last cacheline: 56 bytes */
};
struct spi_mem_dirmap_info {
	struct spi_mem_op          op_tmpl;              /*     0    56 */
	u64                        offset;               /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u64                        length;               /*    64     8 */

	/* size: 72, cachelines: 2, members: 3 */
	/* last cacheline: 8 bytes */
};
struct spi_mem_dirmap_desc {
	struct spi_mem *           mem;                  /*     0     8 */
	struct spi_mem_dirmap_info info;                 /*     8    72 */
	/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
	unsigned int               nodirmap;             /*    80     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     priv;                 /*    88     8 */

	/* size: 96, cachelines: 2, members: 4 */
	/* sum members: 92, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct spi_mem {
	struct spi_device *        spi;                  /*     0     8 */
	void *                     drvpriv;              /*     8     8 */
	const char  *              name;                 /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_spi_controller {
	struct trace_entry         ent;                  /*     0     8 */
	int                        bus_num;              /*     8     4 */
	char                       __data[];             /*    12     0 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct trace_event_raw_spi_message {
	struct trace_entry         ent;                  /*     0     8 */
	int                        bus_num;              /*     8     4 */
	int                        chip_select;          /*    12     4 */
	struct spi_message *       msg;                  /*    16     8 */
	char                       __data[];             /*    24     0 */

	/* size: 24, cachelines: 1, members: 5 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_spi_message_done {
	struct trace_entry         ent;                  /*     0     8 */
	int                        bus_num;              /*     8     4 */
	int                        chip_select;          /*    12     4 */
	struct spi_message *       msg;                  /*    16     8 */
	unsigned int               frame;                /*    24     4 */
	unsigned int               actual;               /*    28     4 */
	char                       __data[];             /*    32     0 */

	/* size: 32, cachelines: 1, members: 7 */
	/* last cacheline: 32 bytes */
};
struct trace_event_raw_spi_transfer {
	struct trace_entry         ent;                  /*     0     8 */
	int                        bus_num;              /*     8     4 */
	int                        chip_select;          /*    12     4 */
	struct spi_transfer *      xfer;                 /*    16     8 */
	int                        len;                  /*    24     4 */
	u32                        __data_loc_rx_buf;    /*    28     4 */
	u32                        __data_loc_tx_buf;    /*    32     4 */
	char                       __data[];             /*    36     0 */

	/* size: 40, cachelines: 1, members: 8 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct trace_event_data_offsets_spi_controller {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_spi_message {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_spi_message_done {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_spi_transfer {
	u32                        rx_buf;               /*     0     4 */
	u32                        tx_buf;               /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct boardinfo {
	struct list_head           list;                 /*     0    16 */
	struct spi_board_info      board_info;           /*    16    72 */

	/* size: 88, cachelines: 2, members: 2 */
	/* last cacheline: 24 bytes */
};
struct acpi_spi_lookup {
	struct spi_controller *    ctlr;                 /*     0     8 */
	u32                        max_speed_hz;         /*     8     4 */
	u32                        mode;                 /*    12     4 */
	int                        irq;                  /*    16     4 */
	u8                         bits_per_word;        /*    20     1 */
	u8                         chip_select;          /*    21     1 */

	/* size: 24, cachelines: 1, members: 6 */
	/* padding: 2 */
	/* last cacheline: 24 bytes */
};
struct spi_mem_driver {
	struct spi_driver          spidrv;               /*     0   224 */
	/* --- cacheline 3 boundary (192 bytes) was 32 bytes ago --- */
	int                        (*probe)(struct spi_mem *); /*   224     8 */
	int                        (*remove)(struct spi_mem *); /*   232     8 */
	void                       (*shutdown)(struct spi_mem *); /*   240     8 */

	/* size: 248, cachelines: 4, members: 4 */
	/* last cacheline: 56 bytes */
};
struct devprobe2 {
	struct net_device *        (*probe)(int);        /*     0     8 */
	int                        status;               /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct mdio_board_info {
	const char  *              bus_id;               /*     0     8 */
	char                       modalias[32];         /*     8    32 */
	int                        mdio_addr;            /*    40     4 */

	/* XXX 4 bytes hole, try to pack */

	const void  *              platform_data;        /*    48     8 */

	/* size: 56, cachelines: 1, members: 4 */
	/* sum members: 52, holes: 1, sum holes: 4 */
	/* last cacheline: 56 bytes */
};
struct mdio_board_entry {
	struct list_head           list;                 /*     0    16 */
	struct mdio_board_info     board_info;           /*    16    56 */

	/* size: 72, cachelines: 2, members: 2 */
	/* last cacheline: 8 bytes */
};
struct mdiobus_devres {
	struct mii_bus *           mii;                  /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct mii_ioctl_data {
	__u16                      phy_id;               /*     0     2 */
	__u16                      reg_num;              /*     2     2 */
	__u16                      val_in;               /*     4     2 */
	__u16                      val_out;              /*     6     2 */

	/* size: 8, cachelines: 1, members: 4 */
	/* last cacheline: 8 bytes */
};
struct phy_led_trigger {
	struct led_trigger         trigger;              /*     0    80 */
	/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
	char                       name[76];             /*    80    76 */
	/* --- cacheline 2 boundary (128 bytes) was 28 bytes ago --- */
	unsigned int               speed;                /*   156     4 */

	/* size: 160, cachelines: 3, members: 3 */
	/* last cacheline: 32 bytes */
};
struct phy_setting {
	u32                        speed;                /*     0     4 */
	u8                         duplex;               /*     4     1 */
	u8                         bit;                  /*     5     1 */

	/* size: 8, cachelines: 1, members: 3 */
	/* padding: 2 */
	/* last cacheline: 8 bytes */
};
struct ethtool_phy_ops {
	int                        (*get_sset_count)(struct phy_device *); /*     0     8 */
	int                        (*get_strings)(struct phy_device *, u8 *); /*     8     8 */
	int                        (*get_stats)(struct phy_device *, struct ethtool_stats *, u64 *); /*    16     8 */
	int                        (*start_cable_test)(struct phy_device *, struct netlink_ext_ack *); /*    24     8 */
	int                        (*start_cable_test_tdr)(struct phy_device *, struct netlink_ext_ack *, const struct phy_tdr_config  *); /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct phy_fixup {
	struct list_head           list;                 /*     0    16 */
	char                       bus_id[64];           /*    16    64 */
	/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
	u32                        phy_uid;              /*    80     4 */
	u32                        phy_uid_mask;         /*    84     4 */
	int                        (*run)(struct phy_device *); /*    88     8 */

	/* size: 96, cachelines: 2, members: 5 */
	/* last cacheline: 32 bytes */
};
struct sfp_eeprom_base {
	u8                         phys_id;              /*     0     1 */
	u8                         phys_ext_id;          /*     1     1 */
	u8                         connector;            /*     2     1 */
	u8                         if_1x_copper_passive:1; /*     3: 0  1 */
	u8                         if_1x_copper_active:1; /*     3: 1  1 */
	u8                         if_1x_lx:1;           /*     3: 2  1 */
	u8                         if_1x_sx:1;           /*     3: 3  1 */
	u8                         e10g_base_sr:1;       /*     3: 4  1 */
	u8                         e10g_base_lr:1;       /*     3: 5  1 */
	u8                         e10g_base_lrm:1;      /*     3: 6  1 */
	u8                         e10g_base_er:1;       /*     3: 7  1 */
	u8                         sonet_oc3_short_reach:1; /*     4: 0  1 */
	u8                         sonet_oc3_smf_intermediate_reach:1; /*     4: 1  1 */
	u8                         sonet_oc3_smf_long_reach:1; /*     4: 2  1 */
	u8                         unallocated_5_3:1;    /*     4: 3  1 */
	u8                         sonet_oc12_short_reach:1; /*     4: 4  1 */
	u8                         sonet_oc12_smf_intermediate_reach:1; /*     4: 5  1 */
	u8                         sonet_oc12_smf_long_reach:1; /*     4: 6  1 */
	u8                         unallocated_5_7:1;    /*     4: 7  1 */
	u8                         sonet_oc48_short_reach:1; /*     5: 0  1 */
	u8                         sonet_oc48_intermediate_reach:1; /*     5: 1  1 */
	u8                         sonet_oc48_long_reach:1; /*     5: 2  1 */
	u8                         sonet_reach_bit2:1;   /*     5: 3  1 */
	u8                         sonet_reach_bit1:1;   /*     5: 4  1 */
	u8                         sonet_oc192_short_reach:1; /*     5: 5  1 */
	u8                         escon_smf_1310_laser:1; /*     5: 6  1 */
	u8                         escon_mmf_1310_led:1; /*     5: 7  1 */
	u8                         e1000_base_sx:1;      /*     6: 0  1 */
	u8                         e1000_base_lx:1;      /*     6: 1  1 */
	u8                         e1000_base_cx:1;      /*     6: 2  1 */
	u8                         e1000_base_t:1;       /*     6: 3  1 */
	u8                         e100_base_lx:1;       /*     6: 4  1 */
	u8                         e100_base_fx:1;       /*     6: 5  1 */
	u8                         e_base_bx10:1;        /*     6: 6  1 */
	u8                         e_base_px:1;          /*     6: 7  1 */
	u8                         fc_tech_electrical_inter_enclosure:1; /*     7: 0  1 */
	u8                         fc_tech_lc:1;         /*     7: 1  1 */
	u8                         fc_tech_sa:1;         /*     7: 2  1 */
	u8                         fc_ll_m:1;            /*     7: 3  1 */
	u8                         fc_ll_l:1;            /*     7: 4  1 */
	u8                         fc_ll_i:1;            /*     7: 5  1 */
	u8                         fc_ll_s:1;            /*     7: 6  1 */
	u8                         fc_ll_v:1;            /*     7: 7  1 */
	u8                         unallocated_8_0:1;    /*     8: 0  1 */
	u8                         unallocated_8_1:1;    /*     8: 1  1 */
	u8                         sfp_ct_passive:1;     /*     8: 2  1 */
	u8                         sfp_ct_active:1;      /*     8: 3  1 */
	u8                         fc_tech_ll:1;         /*     8: 4  1 */
	u8                         fc_tech_sl:1;         /*     8: 5  1 */
	u8                         fc_tech_sn:1;         /*     8: 6  1 */
	u8                         fc_tech_electrical_intra_enclosure:1; /*     8: 7  1 */
	u8                         fc_media_sm:1;        /*     9: 0  1 */
	u8                         unallocated_9_1:1;    /*     9: 1  1 */
	u8                         fc_media_m5:1;        /*     9: 2  1 */
	u8                         fc_media_m6:1;        /*     9: 3  1 */
	u8                         fc_media_tv:1;        /*     9: 4  1 */
	u8                         fc_media_mi:1;        /*     9: 5  1 */
	u8                         fc_media_tp:1;        /*     9: 6  1 */
	u8                         fc_media_tw:1;        /*     9: 7  1 */
	u8                         fc_speed_100:1;       /*    10: 0  1 */
	u8                         unallocated_10_1:1;   /*    10: 1  1 */
	u8                         fc_speed_200:1;       /*    10: 2  1 */
	u8                         fc_speed_3200:1;      /*    10: 3  1 */
	u8                         fc_speed_400:1;       /*    10: 4  1 */
	u8                         fc_speed_1600:1;      /*    10: 5  1 */
	u8                         fc_speed_800:1;       /*    10: 6  1 */
	u8                         fc_speed_1200:1;      /*    10: 7  1 */
	u8                         encoding;             /*    11     1 */
	u8                         br_nominal;           /*    12     1 */
	u8                         rate_id;              /*    13     1 */
	u8                         link_len[6];          /*    14     6 */
	char                       vendor_name[16];      /*    20    16 */
	u8                         extended_cc;          /*    36     1 */
	char                       vendor_oui[3];        /*    37     3 */
	char                       vendor_pn[16];        /*    40    16 */
	char                       vendor_rev[4];        /*    56     4 */
	union {
		__be16             optical_wavelength;   /*    60     2 */
		__be16             cable_compliance;     /*    60     2 */
		struct {
			u8         sff8431_app_e:1;      /*    60: 0  1 */
			u8         fc_pi_4_app_h:1;      /*    60: 1  1 */
			u8         reserved60_2:6;       /*    60: 2  1 */
			u8         reserved61:8;         /*    61: 0  1 */
		} passive;                               /*    60     2 */
		struct {
			u8         sff8431_app_e:1;      /*    60: 0  1 */
			u8         fc_pi_4_app_h:1;      /*    60: 1  1 */
			u8         sff8431_lim:1;        /*    60: 2  1 */
			u8         fc_pi_4_lim:1;        /*    60: 3  1 */
			u8         reserved60_4:4;       /*    60: 4  1 */
			u8         reserved61:8;         /*    61: 0  1 */
		} active;                                /*    60     2 */
	};                                               /*    60     2 */
	u8                         reserved62;           /*    62     1 */
	u8                         cc_base;              /*    63     1 */

	/* size: 64, cachelines: 1, members: 79 */
};
struct sfp_eeprom_ext {
	__be16                     options;              /*     0     2 */
	u8                         br_max;               /*     2     1 */
	u8                         br_min;               /*     3     1 */
	char                       vendor_sn[16];        /*     4    16 */
	char                       datecode[8];          /*    20     8 */
	u8                         diagmon;              /*    28     1 */
	u8                         enhopts;              /*    29     1 */
	u8                         sff8472_compliance;   /*    30     1 */
	u8                         cc_ext;               /*    31     1 */

	/* size: 32, cachelines: 1, members: 9 */
	/* last cacheline: 32 bytes */
};
struct sfp_eeprom_id {
	struct sfp_eeprom_base     base;                 /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct sfp_eeprom_ext      ext;                  /*    64    32 */

	/* size: 96, cachelines: 2, members: 2 */
	/* last cacheline: 32 bytes */
};
struct sfp_upstream_ops {
	void                       (*attach)(void *, struct sfp_bus *); /*     0     8 */
	void                       (*detach)(void *, struct sfp_bus *); /*     8     8 */
	int                        (*module_insert)(void *, const struct sfp_eeprom_id  *); /*    16     8 */
	void                       (*module_remove)(void *); /*    24     8 */
	int                        (*module_start)(void *); /*    32     8 */
	void                       (*module_stop)(void *); /*    40     8 */
	void                       (*link_down)(void *); /*    48     8 */
	void                       (*link_up)(void *);   /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*connect_phy)(void *, struct phy_device *); /*    64     8 */
	void                       (*disconnect_phy)(void *); /*    72     8 */

	/* size: 80, cachelines: 2, members: 10 */
	/* last cacheline: 16 bytes */
};
struct trace_event_raw_mdio_access {
	struct trace_entry         ent;                  /*     0     8 */
	char                       busid[61];            /*     8    61 */
	/* --- cacheline 1 boundary (64 bytes) was 5 bytes ago --- */
	char                       read;                 /*    69     1 */
	u8                         addr;                 /*    70     1 */

	/* XXX 1 byte hole, try to pack */

	u16                        val;                  /*    72     2 */

	/* XXX 2 bytes hole, try to pack */

	unsigned int               regnum;               /*    76     4 */
	char                       __data[];             /*    80     0 */

	/* size: 80, cachelines: 2, members: 7 */
	/* sum members: 77, holes: 2, sum holes: 3 */
	/* last cacheline: 16 bytes */
};
struct trace_event_data_offsets_mdio_access {

	/* size: 0, cachelines: 0, members: 0 */
};
struct mdio_bus_stat_attr {
	int                        addr;                 /*     0     4 */
	unsigned int               field_offset;         /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct mdio_driver {
	struct mdio_driver_common  mdiodrv;              /*     0   200 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 3 boundary (192 bytes) was 8 bytes ago --- */
	int                        (*probe)(struct mdio_device *); /*   200     8 */
	void                       (*remove)(struct mdio_device *); /*   208     8 */
	void                       (*shutdown)(struct mdio_device *); /*   216     8 */

	/* size: 224, cachelines: 4, members: 4 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 32 bytes */
};
struct fixed_phy_status {
	int                        link;                 /*     0     4 */
	int                        speed;                /*     4     4 */
	int                        duplex;               /*     8     4 */
	int                        pause;                /*    12     4 */
	int                        asym_pause;           /*    16     4 */

	/* size: 20, cachelines: 1, members: 5 */
	/* last cacheline: 20 bytes */
};
struct swmii_regs {
	u16                        bmsr;                 /*     0     2 */
	u16                        lpa;                  /*     2     2 */
	u16                        lpagb;                /*     4     2 */
	u16                        estat;                /*     6     2 */

	/* size: 8, cachelines: 1, members: 4 */
	/* last cacheline: 8 bytes */
};
struct mii_timestamping_ctrl {
	struct mii_timestamper *   (*probe_channel)(struct device *, unsigned int); /*     0     8 */
	void                       (*release_channel)(struct device *, struct mii_timestamper *); /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct mii_timestamping_desc {
	struct list_head           list;                 /*     0    16 */
	struct mii_timestamping_ctrl * ctrl;             /*    16     8 */
	struct device *            device;               /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* last cacheline: 32 bytes */
};
struct fixed_mdio_bus {
	struct mii_bus *           mii_bus;              /*     0     8 */
	struct list_head           phys;                 /*     8    16 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct fixed_phy {
	int                        addr;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct phy_device *        phydev;               /*     8     8 */
	struct fixed_phy_status    status;               /*    16    20 */
	bool                       no_carrier;           /*    36     1 */

	/* XXX 3 bytes hole, try to pack */

	int                        (*link_update)(struct net_device *, struct fixed_phy_status *); /*    40     8 */
	struct list_head           node;                 /*    48    16 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct gpio_desc *         link_gpiod;           /*    64     8 */

	/* size: 72, cachelines: 2, members: 7 */
	/* sum members: 65, holes: 2, sum holes: 7 */
	/* last cacheline: 8 bytes */
};
struct ptp_clock_time {
	__s64                      sec;                  /*     0     8 */
	__u32                      nsec;                 /*     8     4 */
	__u32                      reserved;             /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct ptp_extts_request {
	unsigned int               index;                /*     0     4 */
	unsigned int               flags;                /*     4     4 */
	unsigned int               rsv[2];               /*     8     8 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct ptp_perout_request {
	union {
		struct ptp_clock_time start;             /*     0    16 */
		struct ptp_clock_time phase;             /*     0    16 */
	};                                               /*     0    16 */
	struct ptp_clock_time      period;               /*    16    16 */
	unsigned int               index;                /*    32     4 */
	unsigned int               flags;                /*    36     4 */
	union {
		struct ptp_clock_time on;                /*    40    16 */
		unsigned int       rsv[4];               /*    40    16 */
	};                                               /*    40    16 */

	/* size: 56, cachelines: 1, members: 5 */
	/* last cacheline: 56 bytes */
};
struct ptp_pin_desc {
	char                       name[64];             /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	unsigned int               index;                /*    64     4 */
	unsigned int               func;                 /*    68     4 */
	unsigned int               chan;                 /*    72     4 */
	unsigned int               rsv[5];               /*    76    20 */

	/* size: 96, cachelines: 2, members: 5 */
	/* last cacheline: 32 bytes */
};
struct ptp_clock_request {
	enum {
		PTP_CLK_REQ_EXTTS  = 0,
		PTP_CLK_REQ_PEROUT = 1,
		PTP_CLK_REQ_PPS    = 2,
	} type;                                          /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	union {
		struct ptp_extts_request extts;          /*     8    16 */
		struct ptp_perout_request perout;        /*     8    56 */
	};                                               /*     8    56 */

	/* size: 64, cachelines: 1, members: 2 */
	/* sum members: 60, holes: 1, sum holes: 4 */
};
struct ptp_clock_info {
	struct module *            owner;                /*     0     8 */
	char                       name[16];             /*     8    16 */
	s32                        max_adj;              /*    24     4 */
	int                        n_alarm;              /*    28     4 */
	int                        n_ext_ts;             /*    32     4 */
	int                        n_per_out;            /*    36     4 */
	int                        n_pins;               /*    40     4 */
	int                        pps;                  /*    44     4 */
	struct ptp_pin_desc *      pin_config;           /*    48     8 */
	int                        (*adjfine)(struct ptp_clock_info *, long int); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*adjfreq)(struct ptp_clock_info *, s32); /*    64     8 */
	int                        (*adjphase)(struct ptp_clock_info *, s32); /*    72     8 */
	int                        (*adjtime)(struct ptp_clock_info *, s64); /*    80     8 */
	int                        (*gettime64)(struct ptp_clock_info *, struct timespec64 *); /*    88     8 */
	int                        (*gettimex64)(struct ptp_clock_info *, struct timespec64 *, struct ptp_system_timestamp *); /*    96     8 */
	int                        (*getcrosststamp)(struct ptp_clock_info *, struct system_device_crosststamp *); /*   104     8 */
	int                        (*settime64)(struct ptp_clock_info *, const struct timespec64  *); /*   112     8 */
	int                        (*enable)(struct ptp_clock_info *, struct ptp_clock_request *, int); /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	int                        (*verify)(struct ptp_clock_info *, unsigned int, enum ptp_pin_function, unsigned int); /*   128     8 */
	long int                   (*do_aux_work)(struct ptp_clock_info *); /*   136     8 */
	u64                        kabi_reserved1;       /*   144     8 */
	u64                        kabi_reserved2;       /*   152     8 */
	u64                        kabi_reserved3;       /*   160     8 */
	u64                        kabi_reserved4;       /*   168     8 */

	/* size: 176, cachelines: 3, members: 24 */
	/* last cacheline: 48 bytes */
};
struct cavium_ptp {
	struct pci_dev *           pdev;                 /*     0     8 */
	spinlock_t                 spin_lock;            /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct cyclecounter        cycle_counter;        /*    16    24 */
	struct timecounter         time_counter;         /*    40    40 */
	/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
	void *                     reg_base;             /*    80     8 */
	u32                        clock_rate;           /*    88     4 */

	/* XXX 4 bytes hole, try to pack */

	struct ptp_clock_info      ptp_info;             /*    96   176 */
	/* --- cacheline 4 boundary (256 bytes) was 16 bytes ago --- */
	struct ptp_clock *         ptp_clock;            /*   272     8 */

	/* size: 280, cachelines: 5, members: 8 */
	/* sum members: 272, holes: 2, sum holes: 8 */
	/* last cacheline: 24 bytes */
};
struct ohci {
	void *                     registers;            /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct socket_state_t {
	u_int                      flags;                /*     0     4 */
	u_int                      csc_mask;             /*     4     4 */
	u_char                     Vcc;                  /*     8     1 */
	u_char                     Vpp;                  /*     9     1 */
	u_char                     io_irq;               /*    10     1 */

	/* size: 12, cachelines: 1, members: 5 */
	/* padding: 1 */
	/* last cacheline: 12 bytes */
};
struct pccard_io_map {
	u_char                     map;                  /*     0     1 */
	u_char                     flags;                /*     1     1 */
	u_short                    speed;                /*     2     2 */

	/* XXX 4 bytes hole, try to pack */

	phys_addr_t                start;                /*     8     8 */
	phys_addr_t                stop;                 /*    16     8 */

	/* size: 24, cachelines: 1, members: 5 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct pccard_mem_map {
	u_char                     map;                  /*     0     1 */
	u_char                     flags;                /*     1     1 */
	u_short                    speed;                /*     2     2 */

	/* XXX 4 bytes hole, try to pack */

	phys_addr_t                static_start;         /*     8     8 */
	u_int                      card_start;           /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	struct resource *          res;                  /*    24     8 */

	/* size: 32, cachelines: 1, members: 6 */
	/* sum members: 24, holes: 2, sum holes: 8 */
	/* last cacheline: 32 bytes */
};
struct io_window_t {
	u_int                      InUse;                /*     0     4 */
	u_int                      Config;               /*     4     4 */
	struct resource *          res;                  /*     8     8 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct pccard_operations {
	int                        (*init)(struct pcmcia_socket *); /*     0     8 */
	int                        (*suspend)(struct pcmcia_socket *); /*     8     8 */
	int                        (*get_status)(struct pcmcia_socket *, u_int *); /*    16     8 */
	int                        (*set_socket)(struct pcmcia_socket *, socket_state_t *); /*    24     8 */
	int                        (*set_io_map)(struct pcmcia_socket *, struct pccard_io_map *); /*    32     8 */
	int                        (*set_mem_map)(struct pcmcia_socket *, struct pccard_mem_map *); /*    40     8 */

	/* size: 48, cachelines: 1, members: 6 */
	/* last cacheline: 48 bytes */
};
struct pcmcia_socket {
	struct module *            owner;                /*     0     8 */
	socket_state_t             socket;               /*     8    12 */
	u_int                      state;                /*    20     4 */
	u_int                      suspended_state;      /*    24     4 */
	u_short                    functions;            /*    28     2 */
	u_short                    lock_count;           /*    30     2 */
	pccard_mem_map             cis_mem;              /*    32    32 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	void *                     cis_virt;             /*    64     8 */
	io_window_t                io[2];                /*    72    32 */
	pccard_mem_map             win[4];               /*   104   128 */
	/* --- cacheline 3 boundary (192 bytes) was 40 bytes ago --- */
	struct list_head           cis_cache;            /*   232    16 */
	size_t                     fake_cis_len;         /*   248     8 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	u8 *                       fake_cis;             /*   256     8 */
	struct list_head           socket_list;          /*   264    16 */
	struct completion          socket_released;      /*   280    32 */
	unsigned int               sock;                 /*   312     4 */
	u_int                      features;             /*   316     4 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	u_int                      irq_mask;             /*   320     4 */
	u_int                      map_size;             /*   324     4 */
	u_int                      io_offset;            /*   328     4 */
	u_int                      pci_irq;              /*   332     4 */
	struct pci_dev *           cb_dev;               /*   336     8 */
	u8                         resource_setup_done;  /*   344     1 */

	/* XXX 7 bytes hole, try to pack */

	struct pccard_operations * ops;                  /*   352     8 */
	struct pccard_resource_ops * resource_ops;       /*   360     8 */
	void *                     resource_data;        /*   368     8 */
	void                       (*zoom_video)(struct pcmcia_socket *, int); /*   376     8 */
	/* --- cacheline 6 boundary (384 bytes) --- */
	int                        (*power_hook)(struct pcmcia_socket *, int); /*   384     8 */
	void                       (*tune_bridge)(struct pcmcia_socket *, struct pci_bus *); /*   392     8 */
	struct task_struct *       thread;               /*   400     8 */
	struct completion          thread_done;          /*   408    32 */
	unsigned int               thread_events;        /*   440     4 */
	unsigned int               sysfs_events;         /*   444     4 */
	/* --- cacheline 7 boundary (448 bytes) --- */
	struct mutex               skt_mutex;            /*   448    32 */
	struct mutex               ops_mutex;            /*   480    32 */
	/* --- cacheline 8 boundary (512 bytes) --- */
	spinlock_t                 thread_lock;          /*   512     4 */

	/* XXX 4 bytes hole, try to pack */

	struct pcmcia_callback *   callback;             /*   520     8 */
	struct device              dev __attribute__((__aligned__(8))); /*   528  1000 */
	/* --- cacheline 23 boundary (1472 bytes) was 56 bytes ago --- */
	void *                     driver_data;          /*  1528     8 */
	/* --- cacheline 24 boundary (1536 bytes) --- */
	int                        resume_status;        /*  1536     4 */

	/* size: 1544, cachelines: 25, members: 40 */
	/* sum members: 1529, holes: 2, sum holes: 11 */
	/* padding: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 8 bytes */
} __attribute__((__aligned__(8)));
struct pccard_resource_ops {
	int                        (*validate_mem)(struct pcmcia_socket *); /*     0     8 */
	int                        (*find_io)(struct pcmcia_socket *, unsigned int, unsigned int *, unsigned int, unsigned int, struct resource * *); /*     8     8 */
	struct resource *          (*find_mem)(long unsigned int, long unsigned int, long unsigned int, int, struct pcmcia_socket *); /*    16     8 */
	int                        (*init)(struct pcmcia_socket *); /*    24     8 */
	void                       (*exit)(struct pcmcia_socket *); /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct pcmcia_callback {
	struct module *            owner;                /*     0     8 */
	int                        (*add)(struct pcmcia_socket *); /*     8     8 */
	int                        (*remove)(struct pcmcia_socket *); /*    16     8 */
	void                       (*requery)(struct pcmcia_socket *); /*    24     8 */
	int                        (*validate)(struct pcmcia_socket *, unsigned int *); /*    32     8 */
	int                        (*suspend)(struct pcmcia_socket *); /*    40     8 */
	int                        (*early_resume)(struct pcmcia_socket *); /*    48     8 */
	int                        (*resume)(struct pcmcia_socket *); /*    56     8 */

	/* size: 64, cachelines: 1, members: 8 */
};
struct usb_device_id {
	__u16                      match_flags;          /*     0     2 */
	__u16                      idVendor;             /*     2     2 */
	__u16                      idProduct;            /*     4     2 */
	__u16                      bcdDevice_lo;         /*     6     2 */
	__u16                      bcdDevice_hi;         /*     8     2 */
	__u8                       bDeviceClass;         /*    10     1 */
	__u8                       bDeviceSubClass;      /*    11     1 */
	__u8                       bDeviceProtocol;      /*    12     1 */
	__u8                       bInterfaceClass;      /*    13     1 */
	__u8                       bInterfaceSubClass;   /*    14     1 */
	__u8                       bInterfaceProtocol;   /*    15     1 */
	__u8                       bInterfaceNumber;     /*    16     1 */

	/* XXX 7 bytes hole, try to pack */

	kernel_ulong_t             driver_info __attribute__((__aligned__(8))); /*    24     8 */

	/* size: 32, cachelines: 1, members: 13 */
	/* sum members: 25, holes: 1, sum holes: 7 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 7 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct usb_descriptor_header {
	__u8                       bLength;              /*     0     1 */
	__u8                       bDescriptorType;      /*     1     1 */

	/* size: 2, cachelines: 1, members: 2 */
	/* last cacheline: 2 bytes */
};
struct usb_device_descriptor {
	__u8                       bLength;              /*     0     1 */
	__u8                       bDescriptorType;      /*     1     1 */
	__le16                     bcdUSB;               /*     2     2 */
	__u8                       bDeviceClass;         /*     4     1 */
	__u8                       bDeviceSubClass;      /*     5     1 */
	__u8                       bDeviceProtocol;      /*     6     1 */
	__u8                       bMaxPacketSize0;      /*     7     1 */
	__le16                     idVendor;             /*     8     2 */
	__le16                     idProduct;            /*    10     2 */
	__le16                     bcdDevice;            /*    12     2 */
	__u8                       iManufacturer;        /*    14     1 */
	__u8                       iProduct;             /*    15     1 */
	__u8                       iSerialNumber;        /*    16     1 */
	__u8                       bNumConfigurations;   /*    17     1 */

	/* size: 18, cachelines: 1, members: 14 */
	/* last cacheline: 18 bytes */
};
struct usb_config_descriptor {
	__u8                       bLength;              /*     0     1 */
	__u8                       bDescriptorType;      /*     1     1 */
	__le16                     wTotalLength;         /*     2     2 */
	__u8                       bNumInterfaces;       /*     4     1 */
	__u8                       bConfigurationValue;  /*     5     1 */
	__u8                       iConfiguration;       /*     6     1 */
	__u8                       bmAttributes;         /*     7     1 */
	__u8                       bMaxPower;            /*     8     1 */

	/* size: 9, cachelines: 1, members: 8 */
	/* last cacheline: 9 bytes */
} __attribute__((__packed__));
struct usb_interface_descriptor {
	__u8                       bLength;              /*     0     1 */
	__u8                       bDescriptorType;      /*     1     1 */
	__u8                       bInterfaceNumber;     /*     2     1 */
	__u8                       bAlternateSetting;    /*     3     1 */
	__u8                       bNumEndpoints;        /*     4     1 */
	__u8                       bInterfaceClass;      /*     5     1 */
	__u8                       bInterfaceSubClass;   /*     6     1 */
	__u8                       bInterfaceProtocol;   /*     7     1 */
	__u8                       iInterface;           /*     8     1 */

	/* size: 9, cachelines: 1, members: 9 */
	/* last cacheline: 9 bytes */
};
struct usb_endpoint_descriptor {
	__u8                       bLength;              /*     0     1 */
	__u8                       bDescriptorType;      /*     1     1 */
	__u8                       bEndpointAddress;     /*     2     1 */
	__u8                       bmAttributes;         /*     3     1 */
	__le16                     wMaxPacketSize;       /*     4     2 */
	__u8                       bInterval;            /*     6     1 */
	__u8                       bRefresh;             /*     7     1 */
	__u8                       bSynchAddress;        /*     8     1 */

	/* size: 9, cachelines: 1, members: 8 */
	/* last cacheline: 9 bytes */
} __attribute__((__packed__));
struct usb_ssp_isoc_ep_comp_descriptor {
	__u8                       bLength;              /*     0     1 */
	__u8                       bDescriptorType;      /*     1     1 */
	__le16                     wReseved;             /*     2     2 */
	__le32                     dwBytesPerInterval;   /*     4     4 */

	/* size: 8, cachelines: 1, members: 4 */
	/* last cacheline: 8 bytes */
};
struct usb_ss_ep_comp_descriptor {
	__u8                       bLength;              /*     0     1 */
	__u8                       bDescriptorType;      /*     1     1 */
	__u8                       bMaxBurst;            /*     2     1 */
	__u8                       bmAttributes;         /*     3     1 */
	__le16                     wBytesPerInterval;    /*     4     2 */

	/* size: 6, cachelines: 1, members: 5 */
	/* last cacheline: 6 bytes */
};
struct usb_interface_assoc_descriptor {
	__u8                       bLength;              /*     0     1 */
	__u8                       bDescriptorType;      /*     1     1 */
	__u8                       bFirstInterface;      /*     2     1 */
	__u8                       bInterfaceCount;      /*     3     1 */
	__u8                       bFunctionClass;       /*     4     1 */
	__u8                       bFunctionSubClass;    /*     5     1 */
	__u8                       bFunctionProtocol;    /*     6     1 */
	__u8                       iFunction;            /*     7     1 */

	/* size: 8, cachelines: 1, members: 8 */
	/* last cacheline: 8 bytes */
};
struct usb_bos_descriptor {
	__u8                       bLength;              /*     0     1 */
	__u8                       bDescriptorType;      /*     1     1 */
	__le16                     wTotalLength;         /*     2     2 */
	__u8                       bNumDeviceCaps;       /*     4     1 */

	/* size: 5, cachelines: 1, members: 4 */
	/* last cacheline: 5 bytes */
} __attribute__((__packed__));
struct usb_ext_cap_descriptor {
	__u8                       bLength;              /*     0     1 */
	__u8                       bDescriptorType;      /*     1     1 */
	__u8                       bDevCapabilityType;   /*     2     1 */
	__le32                     bmAttributes;         /*     3     4 */

	/* size: 7, cachelines: 1, members: 4 */
	/* last cacheline: 7 bytes */
} __attribute__((__packed__));
struct usb_ss_cap_descriptor {
	__u8                       bLength;              /*     0     1 */
	__u8                       bDescriptorType;      /*     1     1 */
	__u8                       bDevCapabilityType;   /*     2     1 */
	__u8                       bmAttributes;         /*     3     1 */
	__le16                     wSpeedSupported;      /*     4     2 */
	__u8                       bFunctionalitySupport; /*     6     1 */
	__u8                       bU1devExitLat;        /*     7     1 */
	__le16                     bU2DevExitLat;        /*     8     2 */

	/* size: 10, cachelines: 1, members: 8 */
	/* last cacheline: 10 bytes */
};
struct usb_ss_container_id_descriptor {
	__u8                       bLength;              /*     0     1 */
	__u8                       bDescriptorType;      /*     1     1 */
	__u8                       bDevCapabilityType;   /*     2     1 */
	__u8                       bReserved;            /*     3     1 */
	__u8                       ContainerID[16];      /*     4    16 */

	/* size: 20, cachelines: 1, members: 5 */
	/* last cacheline: 20 bytes */
};
struct usb_ssp_cap_descriptor {
	__u8                       bLength;              /*     0     1 */
	__u8                       bDescriptorType;      /*     1     1 */
	__u8                       bDevCapabilityType;   /*     2     1 */
	__u8                       bReserved;            /*     3     1 */
	__le32                     bmAttributes;         /*     4     4 */
	__le16                     wFunctionalitySupport; /*     8     2 */
	__le16                     wReserved;            /*    10     2 */
	__le32                     bmSublinkSpeedAttr[1]; /*    12     4 */

	/* size: 16, cachelines: 1, members: 8 */
	/* last cacheline: 16 bytes */
};
struct usb_ptm_cap_descriptor {
	__u8                       bLength;              /*     0     1 */
	__u8                       bDescriptorType;      /*     1     1 */
	__u8                       bDevCapabilityType;   /*     2     1 */

	/* size: 3, cachelines: 1, members: 3 */
	/* last cacheline: 3 bytes */
};
struct usb_host_endpoint {
	struct usb_endpoint_descriptor desc;             /*     0     9 */
	struct usb_ss_ep_comp_descriptor ss_ep_comp;     /*     9     6 */
	struct usb_ssp_isoc_ep_comp_descriptor ssp_isoc_ep_comp; /*    15     8 */

	/* XXX 1 byte hole, try to pack */

	struct list_head           urb_list;             /*    24    16 */
	void *                     hcpriv;               /*    40     8 */
	struct ep_device *         ep_dev;               /*    48     8 */
	unsigned char *            extra;                /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        extralen;             /*    64     4 */
	int                        enabled;              /*    68     4 */
	int                        streams;              /*    72     4 */

	/* size: 80, cachelines: 2, members: 10 */
	/* sum members: 75, holes: 1, sum holes: 1 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct usb_host_interface {
	struct usb_interface_descriptor desc;            /*     0     9 */

	/* XXX 3 bytes hole, try to pack */

	int                        extralen;             /*    12     4 */
	unsigned char *            extra;                /*    16     8 */
	struct usb_host_endpoint * endpoint;             /*    24     8 */
	char *                     string;               /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* sum members: 37, holes: 1, sum holes: 3 */
	/* last cacheline: 40 bytes */
};
struct usb_interface {
	struct usb_host_interface * altsetting;          /*     0     8 */
	struct usb_host_interface * cur_altsetting;      /*     8     8 */
	unsigned int               num_altsetting;       /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	struct usb_interface_assoc_descriptor * intf_assoc; /*    24     8 */
	int                        minor;                /*    32     4 */
	enum usb_interface_condition condition;          /*    36     4 */
	unsigned int               sysfs_files_created:1; /*    40: 0  4 */
	unsigned int               ep_devs_created:1;    /*    40: 1  4 */
	unsigned int               unregistering:1;      /*    40: 2  4 */
	unsigned int               needs_remote_wakeup:1; /*    40: 3  4 */
	unsigned int               needs_altsetting0:1;  /*    40: 4  4 */
	unsigned int               needs_binding:1;      /*    40: 5  4 */
	unsigned int               resetting_device:1;   /*    40: 6  4 */
	unsigned int               authorized:1;         /*    40: 7  4 */

	/* XXX 24 bits hole, try to pack */
	/* XXX 4 bytes hole, try to pack */

	struct device              dev __attribute__((__aligned__(8))); /*    48  1000 */
	/* --- cacheline 16 boundary (1024 bytes) was 24 bytes ago --- */
	struct device *            usb_dev;              /*  1048     8 */
	struct work_struct         reset_ws;             /*  1056    64 */

	/* size: 1120, cachelines: 18, members: 17 */
	/* sum members: 1108, holes: 2, sum holes: 8 */
	/* sum bitfield members: 8 bits, bit holes: 1, sum bit holes: 24 bits */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 4 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct usb_interface_cache {
	unsigned int               num_altsetting;       /*     0     4 */
	struct kref                ref;                  /*     4     4 */
	struct usb_host_interface  altsetting[];         /*     8     0 */

	/* size: 8, cachelines: 1, members: 3 */
	/* last cacheline: 8 bytes */
};
struct usb_host_config {
	struct usb_config_descriptor desc;               /*     0     9 */

	/* XXX 7 bytes hole, try to pack */

	char *                     string;               /*    16     8 */
	struct usb_interface_assoc_descriptor * intf_assoc[16]; /*    24   128 */
	/* --- cacheline 2 boundary (128 bytes) was 24 bytes ago --- */
	struct usb_interface *     interface[32];        /*   152   256 */
	/* --- cacheline 6 boundary (384 bytes) was 24 bytes ago --- */
	struct usb_interface_cache * intf_cache[32];     /*   408   256 */
	/* --- cacheline 10 boundary (640 bytes) was 24 bytes ago --- */
	unsigned char *            extra;                /*   664     8 */
	int                        extralen;             /*   672     4 */

	/* size: 680, cachelines: 11, members: 7 */
	/* sum members: 669, holes: 1, sum holes: 7 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct usb_host_bos {
	struct usb_bos_descriptor * desc;                /*     0     8 */
	struct usb_ext_cap_descriptor * ext_cap;         /*     8     8 */
	struct usb_ss_cap_descriptor * ss_cap;           /*    16     8 */
	struct usb_ssp_cap_descriptor * ssp_cap;         /*    24     8 */
	struct usb_ss_container_id_descriptor * ss_id;   /*    32     8 */
	struct usb_ptm_cap_descriptor * ptm_cap;         /*    40     8 */

	/* size: 48, cachelines: 1, members: 6 */
	/* last cacheline: 48 bytes */
};
struct usb_devmap {
	long unsigned int          devicemap[2];         /*     0    16 */

	/* size: 16, cachelines: 1, members: 1 */
	/* last cacheline: 16 bytes */
};
struct usb_bus {
	struct device *            controller;           /*     0     8 */
	struct device *            sysdev;               /*     8     8 */
	int                        busnum;               /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	const char  *              bus_name;             /*    24     8 */
	u8                         uses_pio_for_control; /*    32     1 */
	u8                         otg_port;             /*    33     1 */

	/* Bitfield combined with previous fields */

	unsigned int               is_b_host:1;          /*    32:16  4 */
	unsigned int               b_hnp_enable:1;       /*    32:17  4 */
	unsigned int               no_stop_on_short:1;   /*    32:18  4 */
	unsigned int               no_sg_constraint:1;   /*    32:19  4 */

	/* XXX 12 bits hole, try to pack */

	unsigned int               sg_tablesize;         /*    36     4 */
	int                        devnum_next;          /*    40     4 */

	/* XXX 4 bytes hole, try to pack */

	struct mutex               devnum_next_mutex;    /*    48    32 */
	/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
	struct usb_devmap          devmap;               /*    80    16 */
	struct usb_device *        root_hub;             /*    96     8 */
	struct usb_bus *           hs_companion;         /*   104     8 */
	int                        bandwidth_allocated;  /*   112     4 */
	int                        bandwidth_int_reqs;   /*   116     4 */
	int                        bandwidth_isoc_reqs;  /*   120     4 */
	unsigned int               resuming_ports;       /*   124     4 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct mon_bus *           mon_bus;              /*   128     8 */
	int                        monitored;            /*   136     4 */

	/* size: 144, cachelines: 3, members: 22 */
	/* sum members: 130, holes: 2, sum holes: 8 */
	/* sum bitfield members: 4 bits, bit holes: 1, sum bit holes: 12 bits */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct usb_device {
	int                        devnum;               /*     0     4 */
	char                       devpath[16];          /*     4    16 */
	u32                        route;                /*    20     4 */
	enum usb_device_state      state;                /*    24     4 */
	enum usb_device_speed      speed;                /*    28     4 */
	unsigned int               rx_lanes;             /*    32     4 */
	unsigned int               tx_lanes;             /*    36     4 */
	struct usb_tt *            tt;                   /*    40     8 */
	int                        ttport;               /*    48     4 */
	unsigned int               toggle[2];            /*    52     8 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	struct usb_device *        parent;               /*    64     8 */
	struct usb_bus *           bus;                  /*    72     8 */
	struct usb_host_endpoint   ep0;                  /*    80    80 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 2 boundary (128 bytes) was 32 bytes ago --- */
	struct device              dev __attribute__((__aligned__(8))); /*   160  1000 */
	/* --- cacheline 18 boundary (1152 bytes) was 8 bytes ago --- */
	struct usb_device_descriptor descriptor;         /*  1160    18 */

	/* XXX 6 bytes hole, try to pack */

	struct usb_host_bos *      bos;                  /*  1184     8 */
	struct usb_host_config *   config;               /*  1192     8 */
	struct usb_host_config *   actconfig;            /*  1200     8 */
	struct usb_host_endpoint * ep_in[16];            /*  1208   128 */
	/* --- cacheline 20 boundary (1280 bytes) was 56 bytes ago --- */
	struct usb_host_endpoint * ep_out[16];           /*  1336   128 */
	/* --- cacheline 22 boundary (1408 bytes) was 56 bytes ago --- */
	char * *                   rawdescriptors;       /*  1464     8 */
	/* --- cacheline 23 boundary (1472 bytes) --- */
	short unsigned int         bus_mA;               /*  1472     2 */
	u8                         portnum;              /*  1474     1 */
	u8                         level;                /*  1475     1 */
	u8                         devaddr;              /*  1476     1 */

	/* Bitfield combined with previous fields */

	unsigned int               can_submit:1;         /*  1476: 8  4 */
	unsigned int               persist_enabled:1;    /*  1476: 9  4 */
	unsigned int               have_langid:1;        /*  1476:10  4 */
	unsigned int               authorized:1;         /*  1476:11  4 */
	unsigned int               authenticated:1;      /*  1476:12  4 */
	unsigned int               wusb:1;               /*  1476:13  4 */
	unsigned int               lpm_capable:1;        /*  1476:14  4 */
	unsigned int               usb2_hw_lpm_capable:1; /*  1476:15  4 */
	unsigned int               usb2_hw_lpm_besl_capable:1; /*  1476:16  4 */
	unsigned int               usb2_hw_lpm_enabled:1; /*  1476:17  4 */
	unsigned int               usb2_hw_lpm_allowed:1; /*  1476:18  4 */
	unsigned int               usb3_lpm_u1_enabled:1; /*  1476:19  4 */
	unsigned int               usb3_lpm_u2_enabled:1; /*  1476:20  4 */

	/* XXX 11 bits hole, try to pack */

	int                        string_langid;        /*  1480     4 */

	/* XXX 4 bytes hole, try to pack */

	char *                     product;              /*  1488     8 */
	char *                     manufacturer;         /*  1496     8 */
	char *                     serial;               /*  1504     8 */
	struct list_head           filelist;             /*  1512    16 */
	int                        maxchild;             /*  1528     4 */
	u32                        quirks;               /*  1532     4 */
	/* --- cacheline 24 boundary (1536 bytes) --- */
	atomic_t                   urbnum;               /*  1536     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          active_duration;      /*  1544     8 */
	long unsigned int          connect_time;         /*  1552     8 */
	unsigned int               do_remote_wakeup:1;   /*  1560: 0  4 */
	unsigned int               reset_resume:1;       /*  1560: 1  4 */
	unsigned int               port_is_suspended:1;  /*  1560: 2  4 */

	/* XXX 29 bits hole, try to pack */
	/* XXX 4 bytes hole, try to pack */

	struct wusb_dev *          wusb_dev;             /*  1568     8 */
	int                        slot_id;              /*  1576     4 */
	enum usb_device_removable  removable;            /*  1580     4 */
	struct usb2_lpm_parameters l1_params;            /*  1584     8 */
	struct usb3_lpm_parameters u1_params;            /*  1592    16 */
	/* --- cacheline 25 boundary (1600 bytes) was 8 bytes ago --- */
	struct usb3_lpm_parameters u2_params;            /*  1608    16 */
	unsigned int               lpm_disable_count;    /*  1624     4 */
	u16                        hub_delay;            /*  1628     2 */

	/* Bitfield combined with previous fields */

	unsigned int               use_generic_driver:1; /*  1628:16  4 */

	/* size: 1632, cachelines: 26, members: 60 */
	/* sum members: 1601, holes: 5, sum holes: 22 */
	/* sum bitfield members: 17 bits, bit holes: 2, sum bit holes: 40 bits */
	/* paddings: 1, sum paddings: 4 */
	/* bit_padding: 15 bits */
	/* forced alignments: 1 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct usb2_lpm_parameters {
	unsigned int               besl;                 /*     0     4 */
	int                        timeout;              /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct usb3_lpm_parameters {
	unsigned int               mel;                  /*     0     4 */
	unsigned int               pel;                  /*     4     4 */
	unsigned int               sel;                  /*     8     4 */
	int                        timeout;              /*    12     4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct usb_tt {
	struct usb_device *        hub;                  /*     0     8 */
	int                        multi;                /*     8     4 */
	unsigned int               think_time;           /*    12     4 */
	void *                     hcpriv;               /*    16     8 */
	spinlock_t                 lock;                 /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           clear_list;           /*    32    16 */
	struct work_struct         clear_work;           /*    48    64 */

	/* size: 112, cachelines: 2, members: 7 */
	/* sum members: 108, holes: 1, sum holes: 4 */
	/* last cacheline: 48 bytes */
};
struct usb_dynids {
	spinlock_t                 lock;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           list;                 /*     8    16 */

	/* size: 24, cachelines: 1, members: 2 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct usbdrv_wrap {
	struct device_driver       driver;               /*     0   192 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	int                        for_devices;          /*   192     4 */

	/* size: 200, cachelines: 4, members: 2 */
	/* padding: 4 */
	/* last cacheline: 8 bytes */
};
struct usb_driver {
	const char  *              name;                 /*     0     8 */
	int                        (*probe)(struct usb_interface *, const struct usb_device_id  *); /*     8     8 */
	void                       (*disconnect)(struct usb_interface *); /*    16     8 */
	int                        (*unlocked_ioctl)(struct usb_interface *, unsigned int, void *); /*    24     8 */
	int                        (*suspend)(struct usb_interface *, pm_message_t); /*    32     8 */
	int                        (*resume)(struct usb_interface *); /*    40     8 */
	int                        (*reset_resume)(struct usb_interface *); /*    48     8 */
	int                        (*pre_reset)(struct usb_interface *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*post_reset)(struct usb_interface *); /*    64     8 */
	const struct usb_device_id  * id_table;          /*    72     8 */
	const struct attribute_group  * * dev_groups;    /*    80     8 */
	struct usb_dynids          dynids;               /*    88    24 */
	struct usbdrv_wrap         drvwrap;              /*   112   200 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 4 boundary (256 bytes) was 56 bytes ago --- */
	unsigned int               no_dynamic_id:1;      /*   312: 0  4 */
	unsigned int               supports_autosuspend:1; /*   312: 1  4 */
	unsigned int               disable_hub_initiated_lpm:1; /*   312: 2  4 */
	unsigned int               soft_unbind:1;        /*   312: 3  4 */

	/* size: 320, cachelines: 5, members: 17 */
	/* padding: 4 */
	/* paddings: 1, sum paddings: 4 */
	/* bit_padding: 28 bits */
};
struct usb_device_driver {
	const char  *              name;                 /*     0     8 */
	bool                       (*match)(struct usb_device *); /*     8     8 */
	int                        (*probe)(struct usb_device *); /*    16     8 */
	void                       (*disconnect)(struct usb_device *); /*    24     8 */
	int                        (*suspend)(struct usb_device *, pm_message_t); /*    32     8 */
	int                        (*resume)(struct usb_device *, pm_message_t); /*    40     8 */
	const struct attribute_group  * * dev_groups;    /*    48     8 */
	struct usbdrv_wrap         drvwrap;              /*    56   200 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 4 boundary (256 bytes) --- */
	const struct usb_device_id  * id_table;          /*   256     8 */
	unsigned int               supports_autosuspend:1; /*   264: 0  4 */
	unsigned int               generic_subclass:1;   /*   264: 1  4 */

	/* size: 272, cachelines: 5, members: 11 */
	/* padding: 4 */
	/* paddings: 1, sum paddings: 4 */
	/* bit_padding: 30 bits */
	/* last cacheline: 16 bytes */
};
struct usb_iso_packet_descriptor {
	unsigned int               offset;               /*     0     4 */
	unsigned int               length;               /*     4     4 */
	unsigned int               actual_length;        /*     8     4 */
	int                        status;               /*    12     4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct usb_anchor {
	struct list_head           urb_list;             /*     0    16 */
	wait_queue_head_t          wait;                 /*    16    24 */
	spinlock_t                 lock;                 /*    40     4 */
	atomic_t                   suspend_wakeups;      /*    44     4 */
	unsigned int               poisoned:1;           /*    48: 0  4 */

	/* size: 56, cachelines: 1, members: 5 */
	/* padding: 4 */
	/* bit_padding: 31 bits */
	/* last cacheline: 56 bytes */
};
struct urb {
	struct kref                kref;                 /*     0     4 */
	int                        unlinked;             /*     4     4 */
	void *                     hcpriv;               /*     8     8 */
	atomic_t                   use_count;            /*    16     4 */
	atomic_t                   reject;               /*    20     4 */
	struct list_head           urb_list;             /*    24    16 */
	struct list_head           anchor_list;          /*    40    16 */
	struct usb_anchor *        anchor;               /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct usb_device *        dev;                  /*    64     8 */
	struct usb_host_endpoint * ep;                   /*    72     8 */
	unsigned int               pipe;                 /*    80     4 */
	unsigned int               stream_id;            /*    84     4 */
	int                        status;               /*    88     4 */
	unsigned int               transfer_flags;       /*    92     4 */
	void *                     transfer_buffer;      /*    96     8 */
	dma_addr_t                 transfer_dma;         /*   104     8 */
	struct scatterlist *       sg;                   /*   112     8 */
	int                        num_mapped_sgs;       /*   120     4 */
	int                        num_sgs;              /*   124     4 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	u32                        transfer_buffer_length; /*   128     4 */
	u32                        actual_length;        /*   132     4 */
	unsigned char *            setup_packet;         /*   136     8 */
	dma_addr_t                 setup_dma;            /*   144     8 */
	int                        start_frame;          /*   152     4 */
	int                        number_of_packets;    /*   156     4 */
	int                        interval;             /*   160     4 */
	int                        error_count;          /*   164     4 */
	void *                     context;              /*   168     8 */
	usb_complete_t             complete;             /*   176     8 */
	struct usb_iso_packet_descriptor iso_frame_desc[]; /*   184     0 */

	/* size: 184, cachelines: 3, members: 30 */
	/* last cacheline: 56 bytes */
};
struct giveback_urb_bh {
	bool                       running;              /*     0     1 */
	bool                       high_prio;            /*     1     1 */

	/* XXX 2 bytes hole, try to pack */

	spinlock_t                 lock;                 /*     4     4 */
	struct list_head           head;                 /*     8    16 */
	struct tasklet_struct      bh;                   /*    24    48 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	struct usb_host_endpoint * completing_ep;        /*    72     8 */

	/* size: 80, cachelines: 2, members: 6 */
	/* sum members: 78, holes: 1, sum holes: 2 */
	/* last cacheline: 16 bytes */
};
struct usb_hcd {
	struct usb_bus             self;                 /*     0   144 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 2 boundary (128 bytes) was 16 bytes ago --- */
	struct kref                kref;                 /*   144     4 */

	/* XXX 4 bytes hole, try to pack */

	const char  *              product_desc;         /*   152     8 */
	int                        speed;                /*   160     4 */
	char                       irq_descr[24];        /*   164    24 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 3 boundary (192 bytes) --- */
	struct timer_list          rh_timer;             /*   192    72 */
	/* --- cacheline 4 boundary (256 bytes) was 8 bytes ago --- */
	struct urb *               status_urb;           /*   264     8 */
	struct work_struct         wakeup_work;          /*   272    64 */
	/* --- cacheline 5 boundary (320 bytes) was 16 bytes ago --- */
	struct work_struct         died_work;            /*   336    64 */
	/* --- cacheline 6 boundary (384 bytes) was 16 bytes ago --- */
	const struct hc_driver  *  driver;               /*   400     8 */
	struct usb_phy *           usb_phy;              /*   408     8 */
	struct usb_phy_roothub *   phy_roothub;          /*   416     8 */
	long unsigned int          flags;                /*   424     8 */
	enum usb_dev_authorize_policy dev_policy;        /*   432     4 */
	unsigned int               rh_registered:1;      /*   436: 0  4 */
	unsigned int               rh_pollable:1;        /*   436: 1  4 */
	unsigned int               msix_enabled:1;       /*   436: 2  4 */
	unsigned int               msi_enabled:1;        /*   436: 3  4 */
	unsigned int               skip_phy_initialization:1; /*   436: 4  4 */
	unsigned int               uses_new_polling:1;   /*   436: 5  4 */
	unsigned int               wireless:1;           /*   436: 6  4 */
	unsigned int               has_tt:1;             /*   436: 7  4 */
	unsigned int               amd_resume_bug:1;     /*   436: 8  4 */
	unsigned int               can_do_streams:1;     /*   436: 9  4 */
	unsigned int               tpl_support:1;        /*   436:10  4 */
	unsigned int               cant_recv_wakeups:1;  /*   436:11  4 */

	/* XXX 20 bits hole, try to pack */

	unsigned int               irq;                  /*   440     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 7 boundary (448 bytes) --- */
	void *                     regs;                 /*   448     8 */
	resource_size_t            rsrc_start;           /*   456     8 */
	resource_size_t            rsrc_len;             /*   464     8 */
	unsigned int               power_budget;         /*   472     4 */

	/* XXX 4 bytes hole, try to pack */

	struct giveback_urb_bh     high_prio_bh;         /*   480    80 */
	/* --- cacheline 8 boundary (512 bytes) was 48 bytes ago --- */
	struct giveback_urb_bh     low_prio_bh;          /*   560    80 */
	/* --- cacheline 10 boundary (640 bytes) --- */
	struct mutex *             address0_mutex;       /*   640     8 */
	struct mutex *             bandwidth_mutex;      /*   648     8 */
	struct usb_hcd *           shared_hcd;           /*   656     8 */
	struct usb_hcd *           primary_hcd;          /*   664     8 */
	struct dma_pool *          pool[4];              /*   672    32 */
	/* --- cacheline 11 boundary (704 bytes) --- */
	int                        state;                /*   704     4 */

	/* XXX 4 bytes hole, try to pack */

	struct gen_pool *          localmem_pool;        /*   712     8 */
	long unsigned int          hcd_priv[] __attribute__((__aligned__(8))); /*   720     0 */

	/* size: 720, cachelines: 12, members: 41 */
	/* sum members: 696, holes: 5, sum holes: 20 */
	/* sum bitfield members: 12 bits, bit holes: 1, sum bit holes: 20 bits */
	/* paddings: 1, sum paddings: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct hc_driver {
	const char  *              description;          /*     0     8 */
	const char  *              product_desc;         /*     8     8 */
	size_t                     hcd_priv_size;        /*    16     8 */
	irqreturn_t                (*irq)(struct usb_hcd *); /*    24     8 */
	int                        flags;                /*    32     4 */

	/* XXX 4 bytes hole, try to pack */

	int                        (*reset)(struct usb_hcd *); /*    40     8 */
	int                        (*start)(struct usb_hcd *); /*    48     8 */
	int                        (*pci_suspend)(struct usb_hcd *, bool); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*pci_resume)(struct usb_hcd *, bool); /*    64     8 */
	void                       (*stop)(struct usb_hcd *); /*    72     8 */
	void                       (*shutdown)(struct usb_hcd *); /*    80     8 */
	int                        (*get_frame_number)(struct usb_hcd *); /*    88     8 */
	int                        (*urb_enqueue)(struct usb_hcd *, struct urb *, gfp_t); /*    96     8 */
	int                        (*urb_dequeue)(struct usb_hcd *, struct urb *, int); /*   104     8 */
	int                        (*map_urb_for_dma)(struct usb_hcd *, struct urb *, gfp_t); /*   112     8 */
	void                       (*unmap_urb_for_dma)(struct usb_hcd *, struct urb *); /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	void                       (*endpoint_disable)(struct usb_hcd *, struct usb_host_endpoint *); /*   128     8 */
	void                       (*endpoint_reset)(struct usb_hcd *, struct usb_host_endpoint *); /*   136     8 */
	int                        (*hub_status_data)(struct usb_hcd *, char *); /*   144     8 */
	int                        (*hub_control)(struct usb_hcd *, u16, u16, u16, char *, u16); /*   152     8 */
	int                        (*bus_suspend)(struct usb_hcd *); /*   160     8 */
	int                        (*bus_resume)(struct usb_hcd *); /*   168     8 */
	int                        (*start_port_reset)(struct usb_hcd *, unsigned int); /*   176     8 */
	long unsigned int          (*get_resuming_ports)(struct usb_hcd *); /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	void                       (*relinquish_port)(struct usb_hcd *, int); /*   192     8 */
	int                        (*port_handed_over)(struct usb_hcd *, int); /*   200     8 */
	void                       (*clear_tt_buffer_complete)(struct usb_hcd *, struct usb_host_endpoint *); /*   208     8 */
	int                        (*alloc_dev)(struct usb_hcd *, struct usb_device *); /*   216     8 */
	void                       (*free_dev)(struct usb_hcd *, struct usb_device *); /*   224     8 */
	int                        (*alloc_streams)(struct usb_hcd *, struct usb_device *, struct usb_host_endpoint * *, unsigned int, unsigned int, gfp_t); /*   232     8 */
	int                        (*free_streams)(struct usb_hcd *, struct usb_device *, struct usb_host_endpoint * *, unsigned int, gfp_t); /*   240     8 */
	int                        (*add_endpoint)(struct usb_hcd *, struct usb_device *, struct usb_host_endpoint *); /*   248     8 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	int                        (*drop_endpoint)(struct usb_hcd *, struct usb_device *, struct usb_host_endpoint *); /*   256     8 */
	int                        (*check_bandwidth)(struct usb_hcd *, struct usb_device *); /*   264     8 */
	void                       (*reset_bandwidth)(struct usb_hcd *, struct usb_device *); /*   272     8 */
	int                        (*address_device)(struct usb_hcd *, struct usb_device *); /*   280     8 */
	int                        (*enable_device)(struct usb_hcd *, struct usb_device *); /*   288     8 */
	int                        (*update_hub_device)(struct usb_hcd *, struct usb_device *, struct usb_tt *, gfp_t); /*   296     8 */
	int                        (*reset_device)(struct usb_hcd *, struct usb_device *); /*   304     8 */
	int                        (*update_device)(struct usb_hcd *, struct usb_device *); /*   312     8 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	int                        (*set_usb2_hw_lpm)(struct usb_hcd *, struct usb_device *, int); /*   320     8 */
	int                        (*enable_usb3_lpm_timeout)(struct usb_hcd *, struct usb_device *, enum usb3_link_state); /*   328     8 */
	int                        (*disable_usb3_lpm_timeout)(struct usb_hcd *, struct usb_device *, enum usb3_link_state); /*   336     8 */
	int                        (*find_raw_port_number)(struct usb_hcd *, int); /*   344     8 */
	int                        (*port_power)(struct usb_hcd *, int, bool); /*   352     8 */

	/* size: 360, cachelines: 6, members: 45 */
	/* sum members: 356, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct usb_phy {
	struct device *            dev;                  /*     0     8 */
	const char  *              label;                /*     8     8 */
	unsigned int               flags;                /*    16     4 */
	enum usb_phy_type          type;                 /*    20     4 */
	enum usb_phy_events        last_event;           /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	struct usb_otg *           otg;                  /*    32     8 */
	struct device *            io_dev;               /*    40     8 */
	struct usb_phy_io_ops *    io_ops;               /*    48     8 */
	void *                     io_priv;              /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct extcon_dev *        edev;                 /*    64     8 */
	struct extcon_dev *        id_edev;              /*    72     8 */
	struct notifier_block      vbus_nb;              /*    80    24 */

	/* XXX last struct has 4 bytes of padding */

	struct notifier_block      id_nb;                /*   104    24 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 2 boundary (128 bytes) --- */
	struct notifier_block      type_nb;              /*   128    24 */

	/* XXX last struct has 4 bytes of padding */

	enum usb_charger_type      chg_type;             /*   152     4 */
	enum usb_charger_state     chg_state;            /*   156     4 */
	struct usb_charger_current chg_cur;              /*   160    32 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	struct work_struct         chg_work;             /*   192    64 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	struct atomic_notifier_head notifier;            /*   256    16 */
	u16                        port_status;          /*   272     2 */
	u16                        port_change;          /*   274     2 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           head;                 /*   280    16 */
	int                        (*init)(struct usb_phy *); /*   296     8 */
	void                       (*shutdown)(struct usb_phy *); /*   304     8 */
	int                        (*set_vbus)(struct usb_phy *, int); /*   312     8 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	int                        (*set_power)(struct usb_phy *, unsigned int); /*   320     8 */
	int                        (*set_suspend)(struct usb_phy *, int); /*   328     8 */
	int                        (*set_wakeup)(struct usb_phy *, bool); /*   336     8 */
	int                        (*notify_connect)(struct usb_phy *, enum usb_device_speed); /*   344     8 */
	int                        (*notify_disconnect)(struct usb_phy *, enum usb_device_speed); /*   352     8 */
	enum usb_charger_type      (*charger_detect)(struct usb_phy *); /*   360     8 */

	/* size: 368, cachelines: 6, members: 31 */
	/* sum members: 360, holes: 2, sum holes: 8 */
	/* paddings: 3, sum paddings: 12 */
	/* last cacheline: 48 bytes */
};
struct usb_port_status {
	__le16                     wPortStatus;          /*     0     2 */
	__le16                     wPortChange;          /*     2     2 */
	__le32                     dwExtPortStatus;      /*     4     4 */

	/* size: 8, cachelines: 1, members: 3 */
	/* last cacheline: 8 bytes */
};
struct usb_hub_status {
	__le16                     wHubStatus;           /*     0     2 */
	__le16                     wHubChange;           /*     2     2 */

	/* size: 4, cachelines: 1, members: 2 */
	/* last cacheline: 4 bytes */
};
struct usb_hub_descriptor {
	__u8                       bDescLength;          /*     0     1 */
	__u8                       bDescriptorType;      /*     1     1 */
	__u8                       bNbrPorts;            /*     2     1 */
	__le16                     wHubCharacteristics;  /*     3     2 */
	__u8                       bPwrOn2PwrGood;       /*     5     1 */
	__u8                       bHubContrCurrent;     /*     6     1 */
	union {
		struct {
			__u8       DeviceRemovable[4];   /*     7     4 */
			__u8       PortPwrCtrlMask[4];   /*    11     4 */
		} hs;                                    /*     7     8 */
		struct {
			__u8       bHubHdrDecLat;        /*     7     1 */
			__le16     wHubDelay;            /*     8     2 */
			__le16     DeviceRemovable;      /*    10     2 */
		} __attribute__((__packed__)) ss;        /*     7     5 */
	} u;                                             /*     7     8 */

	/* size: 15, cachelines: 1, members: 7 */
	/* last cacheline: 15 bytes */
} __attribute__((__packed__));
struct phy_configure_opts_dp {
	unsigned int               link_rate;            /*     0     4 */
	unsigned int               lanes;                /*     4     4 */
	unsigned int               voltage[4];           /*     8    16 */
	unsigned int               pre[4];               /*    24    16 */
	u8                         ssc:1;                /*    40: 0  1 */
	u8                         set_rate:1;           /*    40: 1  1 */
	u8                         set_lanes:1;          /*    40: 2  1 */
	u8                         set_voltages:1;       /*    40: 3  1 */

	/* size: 44, cachelines: 1, members: 8 */
	/* padding: 3 */
	/* bit_padding: 4 bits */
	/* last cacheline: 44 bytes */
};
struct phy_configure_opts_mipi_dphy {
	unsigned int               clk_miss;             /*     0     4 */
	unsigned int               clk_post;             /*     4     4 */
	unsigned int               clk_pre;              /*     8     4 */
	unsigned int               clk_prepare;          /*    12     4 */
	unsigned int               clk_settle;           /*    16     4 */
	unsigned int               clk_term_en;          /*    20     4 */
	unsigned int               clk_trail;            /*    24     4 */
	unsigned int               clk_zero;             /*    28     4 */
	unsigned int               d_term_en;            /*    32     4 */
	unsigned int               eot;                  /*    36     4 */
	unsigned int               hs_exit;              /*    40     4 */
	unsigned int               hs_prepare;           /*    44     4 */
	unsigned int               hs_settle;            /*    48     4 */
	unsigned int               hs_skip;              /*    52     4 */
	unsigned int               hs_trail;             /*    56     4 */
	unsigned int               hs_zero;              /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	unsigned int               init;                 /*    64     4 */
	unsigned int               lpx;                  /*    68     4 */
	unsigned int               ta_get;               /*    72     4 */
	unsigned int               ta_go;                /*    76     4 */
	unsigned int               ta_sure;              /*    80     4 */
	unsigned int               wakeup;               /*    84     4 */
	long unsigned int          hs_clk_rate;          /*    88     8 */
	long unsigned int          lp_clk_rate;          /*    96     8 */
	unsigned char              lanes;                /*   104     1 */

	/* size: 112, cachelines: 2, members: 25 */
	/* padding: 7 */
	/* last cacheline: 48 bytes */
};
union phy_configure_opts {
	struct phy_configure_opts_mipi_dphy mipi_dphy; /*     0   112 */
	struct phy_configure_opts_dp dp;               /*     0    44 */
};
struct phy_ops {
	int                        (*init)(struct phy *); /*     0     8 */
	int                        (*exit)(struct phy *); /*     8     8 */
	int                        (*power_on)(struct phy *); /*    16     8 */
	int                        (*power_off)(struct phy *); /*    24     8 */
	int                        (*set_mode)(struct phy *, enum phy_mode, int); /*    32     8 */
	int                        (*configure)(struct phy *, union phy_configure_opts *); /*    40     8 */
	int                        (*validate)(struct phy *, enum phy_mode, int, union phy_configure_opts *); /*    48     8 */
	int                        (*reset)(struct phy *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*calibrate)(struct phy *); /*    64     8 */
	void                       (*release)(struct phy *); /*    72     8 */
	struct module *            owner;                /*    80     8 */

	/* size: 88, cachelines: 2, members: 11 */
	/* last cacheline: 24 bytes */
};
struct phy {
	struct device              dev __attribute__((__aligned__(8))); /*     0  1000 */
	/* --- cacheline 15 boundary (960 bytes) was 40 bytes ago --- */
	int                        id;                   /*  1000     4 */

	/* XXX 4 bytes hole, try to pack */

	const struct phy_ops  *    ops;                  /*  1008     8 */
	struct mutex               mutex;                /*  1016    32 */
	/* --- cacheline 16 boundary (1024 bytes) was 24 bytes ago --- */
	int                        init_count;           /*  1048     4 */
	int                        power_count;          /*  1052     4 */
	struct phy_attrs           attrs;                /*  1056    12 */

	/* XXX 4 bytes hole, try to pack */

	struct regulator *         pwr;                  /*  1072     8 */

	/* size: 1080, cachelines: 17, members: 8 */
	/* sum members: 1072, holes: 2, sum holes: 8 */
	/* forced alignments: 1 */
	/* last cacheline: 56 bytes */
} __attribute__((__aligned__(8)));
struct phy_attrs {
	u32                        bus_width;            /*     0     4 */
	u32                        max_link_rate;        /*     4     4 */
	enum phy_mode              mode;                 /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct usb_phy_io_ops {
	int                        (*read)(struct usb_phy *, u32); /*     0     8 */
	int                        (*write)(struct usb_phy *, u32, u32); /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct usb_charger_current {
	unsigned int               sdp_min;              /*     0     4 */
	unsigned int               sdp_max;              /*     4     4 */
	unsigned int               dcp_min;              /*     8     4 */
	unsigned int               dcp_max;              /*    12     4 */
	unsigned int               cdp_min;              /*    16     4 */
	unsigned int               cdp_max;              /*    20     4 */
	unsigned int               aca_min;              /*    24     4 */
	unsigned int               aca_max;              /*    28     4 */

	/* size: 32, cachelines: 1, members: 8 */
	/* last cacheline: 32 bytes */
};
struct usb_otg {
	u8                         default_a;            /*     0     1 */

	/* XXX 7 bytes hole, try to pack */

	struct phy *               phy;                  /*     8     8 */
	struct usb_phy *           usb_phy;              /*    16     8 */
	struct usb_bus *           host;                 /*    24     8 */
	struct usb_gadget *        gadget;               /*    32     8 */
	enum usb_otg_state         state;                /*    40     4 */

	/* XXX 4 bytes hole, try to pack */

	int                        (*set_host)(struct usb_otg *, struct usb_bus *); /*    48     8 */
	int                        (*set_peripheral)(struct usb_otg *, struct usb_gadget *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*set_vbus)(struct usb_otg *, bool); /*    64     8 */
	int                        (*start_srp)(struct usb_otg *); /*    72     8 */
	int                        (*start_hnp)(struct usb_otg *); /*    80     8 */

	/* size: 88, cachelines: 2, members: 11 */
	/* sum members: 77, holes: 2, sum holes: 11 */
	/* last cacheline: 24 bytes */
};
struct usb_hub {
	struct device *            intfdev;              /*     0     8 */
	struct usb_device *        hdev;                 /*     8     8 */
	struct kref                kref;                 /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	struct urb *               urb;                  /*    24     8 */
	u8 *                       buffer;               /*    32     8 */
	union {
		struct usb_hub_status hub;       /*     0     4 */
		struct usb_port_status port;     /*     0     8 */
	} * status; /*    40     8 */
	struct mutex               status_mutex;         /*    48    32 */
	/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
	int                        error;                /*    80     4 */
	int                        nerrors;              /*    84     4 */
	long unsigned int          event_bits[1];        /*    88     8 */
	long unsigned int          change_bits[1];       /*    96     8 */
	long unsigned int          removed_bits[1];      /*   104     8 */
	long unsigned int          wakeup_bits[1];       /*   112     8 */
	long unsigned int          power_bits[1];        /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	long unsigned int          child_usage_bits[1];  /*   128     8 */
	long unsigned int          warm_reset_bits[1];   /*   136     8 */
	struct usb_hub_descriptor * descriptor;          /*   144     8 */
	struct usb_tt              tt;                   /*   152   112 */
	/* --- cacheline 4 boundary (256 bytes) was 8 bytes ago --- */
	unsigned int               mA_per_port;          /*   264     4 */
	unsigned int               wakeup_enabled_descendants; /*   268     4 */
	unsigned int               limited_power:1;      /*   272: 0  4 */
	unsigned int               quiescing:1;          /*   272: 1  4 */
	unsigned int               disconnected:1;       /*   272: 2  4 */
	unsigned int               in_reset:1;           /*   272: 3  4 */
	unsigned int               quirk_disable_autosuspend:1; /*   272: 4  4 */
	unsigned int               quirk_check_port_auto_suspend:1; /*   272: 5  4 */
	unsigned int               has_indicators:1;     /*   272: 6  4 */

	/* XXX 1 bit hole, try to pack */
	/* Bitfield combined with next fields */

	u8                         indicator[31];        /*   273    31 */
	struct delayed_work        leds;                 /*   304   192 */
	/* --- cacheline 7 boundary (448 bytes) was 48 bytes ago --- */
	struct delayed_work        init_work;            /*   496   192 */
	/* --- cacheline 10 boundary (640 bytes) was 48 bytes ago --- */
	struct work_struct         events;               /*   688    64 */
	/* --- cacheline 11 boundary (704 bytes) was 48 bytes ago --- */
	spinlock_t                 irq_urb_lock;         /*   752     4 */

	/* XXX 4 bytes hole, try to pack */

	struct timer_list          irq_urb_retry;        /*   760    72 */
	/* --- cacheline 13 boundary (832 bytes) --- */
	struct usb_port * *        ports;                /*   832     8 */

	/* size: 840, cachelines: 14, members: 34 */
	/* sum members: 831, holes: 2, sum holes: 8 */
	/* sum bitfield members: 7 bits, bit holes: 1, sum bit holes: 1 bits */
	/* last cacheline: 8 bytes */
};
struct usb_port {
	struct usb_device *        child;                /*     0     8 */
	struct device              dev __attribute__((__aligned__(8))); /*     8  1000 */
	/* --- cacheline 15 boundary (960 bytes) was 48 bytes ago --- */
	struct usb_dev_state *     port_owner;           /*  1008     8 */
	struct usb_port *          peer;                 /*  1016     8 */
	/* --- cacheline 16 boundary (1024 bytes) --- */
	struct dev_pm_qos_request * req;                 /*  1024     8 */
	enum usb_port_connect_type connect_type;         /*  1032     4 */
	usb_port_location_t        location;             /*  1036     4 */
	struct mutex               status_lock;          /*  1040    32 */
	u32                        over_current_count;   /*  1072     4 */
	u8                         portnum;              /*  1076     1 */

	/* XXX 3 bytes hole, try to pack */

	u32                        quirks;               /*  1080     4 */
	unsigned int               is_superspeed:1;      /*  1084: 0  4 */
	unsigned int               usb3_lpm_u1_permit:1; /*  1084: 1  4 */
	unsigned int               usb3_lpm_u2_permit:1; /*  1084: 2  4 */

	/* size: 1088, cachelines: 17, members: 14 */
	/* sum members: 1081, holes: 1, sum holes: 3 */
	/* sum bitfield members: 3 bits (0 bytes) */
	/* bit_padding: 29 bits */
	/* forced alignments: 1 */
} __attribute__((__aligned__(8)));
struct find_interface_arg {
	int                        minor;                /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct device_driver *     drv;                  /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct each_dev_arg {
	void *                     data;                 /*     0     8 */
	int                        (*fn)(struct usb_device *, void *); /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct usb_qualifier_descriptor {
	__u8                       bLength;              /*     0     1 */
	__u8                       bDescriptorType;      /*     1     1 */
	__le16                     bcdUSB;               /*     2     2 */
	__u8                       bDeviceClass;         /*     4     1 */
	__u8                       bDeviceSubClass;      /*     5     1 */
	__u8                       bDeviceProtocol;      /*     6     1 */
	__u8                       bMaxPacketSize0;      /*     7     1 */
	__u8                       bNumConfigurations;   /*     8     1 */
	__u8                       bRESERVED;            /*     9     1 */

	/* size: 10, cachelines: 1, members: 9 */
	/* last cacheline: 10 bytes */
};
struct usb_set_sel_req {
	__u8                       u1_sel;               /*     0     1 */
	__u8                       u1_pel;               /*     1     1 */
	__le16                     u2_sel;               /*     2     2 */
	__le16                     u2_pel;               /*     4     2 */

	/* size: 6, cachelines: 1, members: 4 */
	/* last cacheline: 6 bytes */
};
struct usbdevfs_hub_portinfo {
	char                       nports;               /*     0     1 */
	char                       port[127];            /*     1   127 */

	/* size: 128, cachelines: 2, members: 2 */
};
struct usb_tt_clear {
	struct list_head           clear_list;           /*     0    16 */
	unsigned int               tt;                   /*    16     4 */
	u16                        devinfo;              /*    20     2 */

	/* XXX 2 bytes hole, try to pack */

	struct usb_hcd *           hcd;                  /*    24     8 */
	struct usb_host_endpoint * ep;                   /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* sum members: 38, holes: 1, sum holes: 2 */
	/* last cacheline: 40 bytes */
};
struct usb_ctrlrequest {
	__u8                       bRequestType;         /*     0     1 */
	__u8                       bRequest;             /*     1     1 */
	__le16                     wValue;               /*     2     2 */
	__le16                     wIndex;               /*     4     2 */
	__le16                     wLength;              /*     6     2 */

	/* size: 8, cachelines: 1, members: 5 */
	/* last cacheline: 8 bytes */
};
struct usb_mon_operations {
	void                       (*urb_submit)(struct usb_bus *, struct urb *); /*     0     8 */
	void                       (*urb_submit_error)(struct usb_bus *, struct urb *, int); /*     8     8 */
	void                       (*urb_complete)(struct usb_bus *, struct urb *, int); /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct usb_sg_request {
	int                        status;               /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	size_t                     bytes;                /*     8     8 */
	spinlock_t                 lock;                 /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	struct usb_device *        dev;                  /*    24     8 */
	int                        pipe;                 /*    32     4 */
	int                        entries;              /*    36     4 */
	struct urb * *             urbs;                 /*    40     8 */
	int                        count;                /*    48     4 */

	/* XXX 4 bytes hole, try to pack */

	struct completion          complete;             /*    56    32 */

	/* size: 88, cachelines: 2, members: 9 */
	/* sum members: 76, holes: 3, sum holes: 12 */
	/* last cacheline: 24 bytes */
};
struct usb_cdc_header_desc {
	__u8                       bLength;              /*     0     1 */
	__u8                       bDescriptorType;      /*     1     1 */
	__u8                       bDescriptorSubType;   /*     2     1 */
	__le16                     bcdCDC;               /*     3     2 */

	/* size: 5, cachelines: 1, members: 4 */
	/* last cacheline: 5 bytes */
} __attribute__((__packed__));
struct usb_cdc_call_mgmt_descriptor {
	__u8                       bLength;              /*     0     1 */
	__u8                       bDescriptorType;      /*     1     1 */
	__u8                       bDescriptorSubType;   /*     2     1 */
	__u8                       bmCapabilities;       /*     3     1 */
	__u8                       bDataInterface;       /*     4     1 */

	/* size: 5, cachelines: 1, members: 5 */
	/* last cacheline: 5 bytes */
};
struct usb_cdc_acm_descriptor {
	__u8                       bLength;              /*     0     1 */
	__u8                       bDescriptorType;      /*     1     1 */
	__u8                       bDescriptorSubType;   /*     2     1 */
	__u8                       bmCapabilities;       /*     3     1 */

	/* size: 4, cachelines: 1, members: 4 */
	/* last cacheline: 4 bytes */
};
struct usb_cdc_union_desc {
	__u8                       bLength;              /*     0     1 */
	__u8                       bDescriptorType;      /*     1     1 */
	__u8                       bDescriptorSubType;   /*     2     1 */
	__u8                       bMasterInterface0;    /*     3     1 */
	__u8                       bSlaveInterface0;     /*     4     1 */

	/* size: 5, cachelines: 1, members: 5 */
	/* last cacheline: 5 bytes */
};
struct usb_cdc_country_functional_desc {
	__u8                       bLength;              /*     0     1 */
	__u8                       bDescriptorType;      /*     1     1 */
	__u8                       bDescriptorSubType;   /*     2     1 */
	__u8                       iCountryCodeRelDate;  /*     3     1 */
	__le16                     wCountyCode0;         /*     4     2 */

	/* size: 6, cachelines: 1, members: 5 */
	/* last cacheline: 6 bytes */
};
struct usb_cdc_network_terminal_desc {
	__u8                       bLength;              /*     0     1 */
	__u8                       bDescriptorType;      /*     1     1 */
	__u8                       bDescriptorSubType;   /*     2     1 */
	__u8                       bEntityId;            /*     3     1 */
	__u8                       iName;                /*     4     1 */
	__u8                       bChannelIndex;        /*     5     1 */
	__u8                       bPhysicalInterface;   /*     6     1 */

	/* size: 7, cachelines: 1, members: 7 */
	/* last cacheline: 7 bytes */
};
struct usb_cdc_ether_desc {
	__u8                       bLength;              /*     0     1 */
	__u8                       bDescriptorType;      /*     1     1 */
	__u8                       bDescriptorSubType;   /*     2     1 */
	__u8                       iMACAddress;          /*     3     1 */
	__le32                     bmEthernetStatistics; /*     4     4 */
	__le16                     wMaxSegmentSize;      /*     8     2 */
	__le16                     wNumberMCFilters;     /*    10     2 */
	__u8                       bNumberPowerFilters;  /*    12     1 */

	/* size: 13, cachelines: 1, members: 8 */
	/* last cacheline: 13 bytes */
} __attribute__((__packed__));
struct usb_cdc_dmm_desc {
	__u8                       bFunctionLength;      /*     0     1 */
	__u8                       bDescriptorType;      /*     1     1 */
	__u8                       bDescriptorSubtype;   /*     2     1 */
	__u16                      bcdVersion;           /*     3     2 */
	__le16                     wMaxCommand;          /*     5     2 */

	/* size: 7, cachelines: 1, members: 5 */
	/* last cacheline: 7 bytes */
} __attribute__((__packed__));
struct usb_cdc_mdlm_desc {
	__u8                       bLength;              /*     0     1 */
	__u8                       bDescriptorType;      /*     1     1 */
	__u8                       bDescriptorSubType;   /*     2     1 */
	__le16                     bcdVersion;           /*     3     2 */
	__u8                       bGUID[16];            /*     5    16 */

	/* size: 21, cachelines: 1, members: 5 */
	/* last cacheline: 21 bytes */
} __attribute__((__packed__));
struct usb_cdc_mdlm_detail_desc {
	__u8                       bLength;              /*     0     1 */
	__u8                       bDescriptorType;      /*     1     1 */
	__u8                       bDescriptorSubType;   /*     2     1 */
	__u8                       bGuidDescriptorType;  /*     3     1 */
	__u8                       bDetailData[];        /*     4     0 */

	/* size: 4, cachelines: 1, members: 5 */
	/* last cacheline: 4 bytes */
};
struct usb_cdc_obex_desc {
	__u8                       bLength;              /*     0     1 */
	__u8                       bDescriptorType;      /*     1     1 */
	__u8                       bDescriptorSubType;   /*     2     1 */
	__le16                     bcdVersion;           /*     3     2 */

	/* size: 5, cachelines: 1, members: 4 */
	/* last cacheline: 5 bytes */
} __attribute__((__packed__));
struct usb_cdc_ncm_desc {
	__u8                       bLength;              /*     0     1 */
	__u8                       bDescriptorType;      /*     1     1 */
	__u8                       bDescriptorSubType;   /*     2     1 */
	__le16                     bcdNcmVersion;        /*     3     2 */
	__u8                       bmNetworkCapabilities; /*     5     1 */

	/* size: 6, cachelines: 1, members: 5 */
	/* last cacheline: 6 bytes */
} __attribute__((__packed__));
struct usb_cdc_mbim_desc {
	__u8                       bLength;              /*     0     1 */
	__u8                       bDescriptorType;      /*     1     1 */
	__u8                       bDescriptorSubType;   /*     2     1 */
	__le16                     bcdMBIMVersion;       /*     3     2 */
	__le16                     wMaxControlMessage;   /*     5     2 */
	__u8                       bNumberFilters;       /*     7     1 */
	__u8                       bMaxFilterSize;       /*     8     1 */
	__le16                     wMaxSegmentSize;      /*     9     2 */
	__u8                       bmNetworkCapabilities; /*    11     1 */

	/* size: 12, cachelines: 1, members: 9 */
	/* last cacheline: 12 bytes */
} __attribute__((__packed__));
struct usb_cdc_mbim_extended_desc {
	__u8                       bLength;              /*     0     1 */
	__u8                       bDescriptorType;      /*     1     1 */
	__u8                       bDescriptorSubType;   /*     2     1 */
	__le16                     bcdMBIMExtendedVersion; /*     3     2 */
	__u8                       bMaxOutstandingCommandMessages; /*     5     1 */
	__le16                     wMTU;                 /*     6     2 */

	/* size: 8, cachelines: 1, members: 6 */
	/* last cacheline: 8 bytes */
} __attribute__((__packed__));
struct usb_cdc_parsed_header {
	struct usb_cdc_union_desc * usb_cdc_union_desc;  /*     0     8 */
	struct usb_cdc_header_desc * usb_cdc_header_desc; /*     8     8 */
	struct usb_cdc_call_mgmt_descriptor * usb_cdc_call_mgmt_descriptor; /*    16     8 */
	struct usb_cdc_acm_descriptor * usb_cdc_acm_descriptor; /*    24     8 */
	struct usb_cdc_country_functional_desc * usb_cdc_country_functional_desc; /*    32     8 */
	struct usb_cdc_network_terminal_desc * usb_cdc_network_terminal_desc; /*    40     8 */
	struct usb_cdc_ether_desc * usb_cdc_ether_desc;  /*    48     8 */
	struct usb_cdc_dmm_desc *  usb_cdc_dmm_desc;     /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct usb_cdc_mdlm_desc * usb_cdc_mdlm_desc;    /*    64     8 */
	struct usb_cdc_mdlm_detail_desc * usb_cdc_mdlm_detail_desc; /*    72     8 */
	struct usb_cdc_obex_desc * usb_cdc_obex_desc;    /*    80     8 */
	struct usb_cdc_ncm_desc *  usb_cdc_ncm_desc;     /*    88     8 */
	struct usb_cdc_mbim_desc * usb_cdc_mbim_desc;    /*    96     8 */
	struct usb_cdc_mbim_extended_desc * usb_cdc_mbim_extended_desc; /*   104     8 */
	bool                       phonet_magic_present; /*   112     1 */

	/* size: 120, cachelines: 2, members: 15 */
	/* padding: 7 */
	/* last cacheline: 56 bytes */
};
struct api_context {
	struct completion          done;                 /*     0    32 */
	int                        status;               /*    32     4 */

	/* size: 40, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct set_config_request {
	struct usb_device *        udev;                 /*     0     8 */
	int                        config;               /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct work_struct         work;                 /*    16    64 */
	/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
	struct list_head           node;                 /*    80    16 */

	/* size: 96, cachelines: 2, members: 4 */
	/* sum members: 92, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct usb_dynid {
	struct list_head           node;                 /*     0    16 */
	struct usb_device_id       id __attribute__((__aligned__(8))); /*    16    32 */

	/* size: 48, cachelines: 1, members: 2 */
	/* forced alignments: 1 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct usb_dev_cap_header {
	__u8                       bLength;              /*     0     1 */
	__u8                       bDescriptorType;      /*     1     1 */
	__u8                       bDevCapabilityType;   /*     2     1 */

	/* size: 3, cachelines: 1, members: 3 */
	/* last cacheline: 3 bytes */
};
struct usb_class_driver {
	char *                     name;                 /*     0     8 */
	char *                     (*devnode)(struct device *, umode_t *); /*     8     8 */
	const struct file_operations  * fops;            /*    16     8 */
	int                        minor_base;           /*    24     4 */

	/* size: 32, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct usb_class {
	struct kref                kref;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct class *             class;                /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct ep_device {
	struct usb_endpoint_descriptor * desc;           /*     0     8 */
	struct usb_device *        udev;                 /*     8     8 */
	struct device              dev __attribute__((__aligned__(8))); /*    16  1000 */

	/* size: 1016, cachelines: 16, members: 3 */
	/* forced alignments: 1 */
	/* last cacheline: 56 bytes */
} __attribute__((__aligned__(8)));
struct usbdevfs_ctrltransfer {
	__u8                       bRequestType;         /*     0     1 */
	__u8                       bRequest;             /*     1     1 */
	__u16                      wValue;               /*     2     2 */
	__u16                      wIndex;               /*     4     2 */
	__u16                      wLength;              /*     6     2 */
	__u32                      timeout;              /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     data;                 /*    16     8 */

	/* size: 24, cachelines: 1, members: 7 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct usbdevfs_bulktransfer {
	unsigned int               ep;                   /*     0     4 */
	unsigned int               len;                  /*     4     4 */
	unsigned int               timeout;              /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     data;                 /*    16     8 */

	/* size: 24, cachelines: 1, members: 4 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct usbdevfs_setinterface {
	unsigned int               interface;            /*     0     4 */
	unsigned int               altsetting;           /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct usbdevfs_disconnectsignal {
	unsigned int               signr;                /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     context;              /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct usbdevfs_getdriver {
	unsigned int               interface;            /*     0     4 */
	char                       driver[256];          /*     4   256 */

	/* size: 260, cachelines: 5, members: 2 */
	/* last cacheline: 4 bytes */
};
struct usbdevfs_connectinfo {
	unsigned int               devnum;               /*     0     4 */
	unsigned char              slow;                 /*     4     1 */

	/* size: 8, cachelines: 1, members: 2 */
	/* padding: 3 */
	/* last cacheline: 8 bytes */
};
struct usbdevfs_conninfo_ex {
	__u32                      size;                 /*     0     4 */
	__u32                      busnum;               /*     4     4 */
	__u32                      devnum;               /*     8     4 */
	__u32                      speed;                /*    12     4 */
	__u8                       num_ports;            /*    16     1 */
	__u8                       ports[7];             /*    17     7 */

	/* size: 24, cachelines: 1, members: 6 */
	/* last cacheline: 24 bytes */
};
struct usbdevfs_iso_packet_desc {
	unsigned int               length;               /*     0     4 */
	unsigned int               actual_length;        /*     4     4 */
	unsigned int               status;               /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct usbdevfs_urb {
	unsigned char              type;                 /*     0     1 */
	unsigned char              endpoint;             /*     1     1 */

	/* XXX 2 bytes hole, try to pack */

	int                        status;               /*     4     4 */
	unsigned int               flags;                /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     buffer;               /*    16     8 */
	int                        buffer_length;        /*    24     4 */
	int                        actual_length;        /*    28     4 */
	int                        start_frame;          /*    32     4 */
	union {
		int                number_of_packets;    /*    36     4 */
		unsigned int       stream_id;            /*    36     4 */
	};                                               /*    36     4 */
	int                        error_count;          /*    40     4 */
	unsigned int               signr;                /*    44     4 */
	void *                     usercontext;          /*    48     8 */
	struct usbdevfs_iso_packet_desc iso_frame_desc[]; /*    56     0 */

	/* size: 56, cachelines: 1, members: 13 */
	/* sum members: 50, holes: 2, sum holes: 6 */
	/* last cacheline: 56 bytes */
};
struct usbdevfs_ioctl {
	int                        ifno;                 /*     0     4 */
	int                        ioctl_code;           /*     4     4 */
	void *                     data;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct usbdevfs_disconnect_claim {
	unsigned int               interface;            /*     0     4 */
	unsigned int               flags;                /*     4     4 */
	char                       driver[256];          /*     8   256 */

	/* size: 264, cachelines: 5, members: 3 */
	/* last cacheline: 8 bytes */
};
struct usbdevfs_streams {
	unsigned int               num_streams;          /*     0     4 */
	unsigned int               num_eps;              /*     4     4 */
	unsigned char              eps[];                /*     8     0 */

	/* size: 8, cachelines: 1, members: 3 */
	/* last cacheline: 8 bytes */
};
struct usbdevfs_ctrltransfer32 {
	u8                         bRequestType;         /*     0     1 */
	u8                         bRequest;             /*     1     1 */
	u16                        wValue;               /*     2     2 */
	u16                        wIndex;               /*     4     2 */
	u16                        wLength;              /*     6     2 */
	u32                        timeout;              /*     8     4 */
	compat_caddr_t             data;                 /*    12     4 */

	/* size: 16, cachelines: 1, members: 7 */
	/* last cacheline: 16 bytes */
};
struct usbdevfs_bulktransfer32 {
	compat_uint_t              ep;                   /*     0     4 */
	compat_uint_t              len;                  /*     4     4 */
	compat_uint_t              timeout;              /*     8     4 */
	compat_caddr_t             data;                 /*    12     4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct usbdevfs_disconnectsignal32 {
	compat_int_t               signr;                /*     0     4 */
	compat_caddr_t             context;              /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct usbdevfs_urb32 {
	unsigned char              type;                 /*     0     1 */
	unsigned char              endpoint;             /*     1     1 */

	/* XXX 2 bytes hole, try to pack */

	compat_int_t               status;               /*     4     4 */
	compat_uint_t              flags;                /*     8     4 */
	compat_caddr_t             buffer;               /*    12     4 */
	compat_int_t               buffer_length;        /*    16     4 */
	compat_int_t               actual_length;        /*    20     4 */
	compat_int_t               start_frame;          /*    24     4 */
	compat_int_t               number_of_packets;    /*    28     4 */
	compat_int_t               error_count;          /*    32     4 */
	compat_uint_t              signr;                /*    36     4 */
	compat_caddr_t             usercontext;          /*    40     4 */
	struct usbdevfs_iso_packet_desc iso_frame_desc[]; /*    44     0 */

	/* size: 44, cachelines: 1, members: 13 */
	/* sum members: 42, holes: 1, sum holes: 2 */
	/* last cacheline: 44 bytes */
};
struct usbdevfs_ioctl32 {
	s32                        ifno;                 /*     0     4 */
	s32                        ioctl_code;           /*     4     4 */
	compat_caddr_t             data;                 /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct usb_dev_state {
	struct list_head           list;                 /*     0    16 */
	struct usb_device *        dev;                  /*    16     8 */
	struct file *              file;                 /*    24     8 */
	spinlock_t                 lock;                 /*    32     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           async_pending;        /*    40    16 */
	struct list_head           async_completed;      /*    56    16 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	struct list_head           memory_list;          /*    72    16 */
	wait_queue_head_t          wait;                 /*    88    24 */
	wait_queue_head_t          wait_for_resume;      /*   112    24 */
	/* --- cacheline 2 boundary (128 bytes) was 8 bytes ago --- */
	unsigned int               discsignr;            /*   136     4 */

	/* XXX 4 bytes hole, try to pack */

	struct pid *               disc_pid;             /*   144     8 */
	const struct cred  *       cred;                 /*   152     8 */
	sigval_t                   disccontext;          /*   160     8 */
	long unsigned int          ifclaimed;            /*   168     8 */
	u32                        disabled_bulk_eps;    /*   176     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          interface_allowed_mask; /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	int                        not_yet_resumed;      /*   192     4 */
	bool                       suspend_allowed;      /*   196     1 */
	bool                       privileges_dropped;   /*   197     1 */

	/* size: 200, cachelines: 4, members: 19 */
	/* sum members: 186, holes: 3, sum holes: 12 */
	/* padding: 2 */
	/* last cacheline: 8 bytes */
};
struct usb_memory {
	struct list_head           memlist;              /*     0    16 */
	int                        vma_use_count;        /*    16     4 */
	int                        urb_use_count;        /*    20     4 */
	u32                        size;                 /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     mem;                  /*    32     8 */
	dma_addr_t                 dma_handle;           /*    40     8 */
	long unsigned int          vm_start;             /*    48     8 */
	struct usb_dev_state *     ps;                   /*    56     8 */

	/* size: 64, cachelines: 1, members: 8 */
	/* sum members: 60, holes: 1, sum holes: 4 */
};
struct async {
	struct list_head           asynclist;            /*     0    16 */
	struct usb_dev_state *     ps;                   /*    16     8 */
	struct pid *               pid;                  /*    24     8 */
	const struct cred  *       cred;                 /*    32     8 */
	unsigned int               signr;                /*    40     4 */
	unsigned int               ifnum;                /*    44     4 */
	void *                     userbuffer;           /*    48     8 */
	void *                     userurb;              /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	sigval_t                   userurb_sigval;       /*    64     8 */
	struct urb *               urb;                  /*    72     8 */
	struct usb_memory *        usbm;                 /*    80     8 */
	unsigned int               mem_usage;            /*    88     4 */
	int                        status;               /*    92     4 */
	u8                         bulk_addr;            /*    96     1 */
	u8                         bulk_status;          /*    97     1 */

	/* size: 104, cachelines: 2, members: 15 */
	/* padding: 6 */
	/* last cacheline: 40 bytes */
};
struct quirk_entry {
	u16                        vid;                  /*     0     2 */
	u16                        pid;                  /*     2     2 */
	u32                        flags;                /*     4     4 */

	/* size: 8, cachelines: 1, members: 3 */
	/* last cacheline: 8 bytes */
};
struct class_info {
	int                        class;                /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	char *                     class_name;           /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct usb_phy_roothub {
	struct phy *               phy;                  /*     0     8 */
	struct list_head           list;                 /*     8    16 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct mon_bus {
	struct list_head           bus_link;             /*     0    16 */
	spinlock_t                 lock;                 /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	struct usb_bus *           u_bus;                /*    24     8 */
	int                        text_inited;          /*    32     4 */
	int                        bin_inited;           /*    36     4 */
	struct dentry *            dent_s;               /*    40     8 */
	struct dentry *            dent_t;               /*    48     8 */
	struct dentry *            dent_u;               /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct device *            classdev;             /*    64     8 */
	int                        nreaders;             /*    72     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           r_list;               /*    80    16 */
	struct kref                ref;                  /*    96     4 */
	unsigned int               cnt_events;           /*   100     4 */
	unsigned int               cnt_text_lost;        /*   104     4 */

	/* size: 112, cachelines: 2, members: 14 */
	/* sum members: 100, holes: 2, sum holes: 8 */
	/* padding: 4 */
	/* last cacheline: 48 bytes */
};
struct mon_reader {
	struct list_head           r_link;               /*     0    16 */
	struct mon_bus *           m_bus;                /*    16     8 */
	void *                     r_data;               /*    24     8 */
	void                       (*rnf_submit)(void *, struct urb *); /*    32     8 */
	void                       (*rnf_error)(void *, struct urb *, int); /*    40     8 */
	void                       (*rnf_complete)(void *, struct urb *, int); /*    48     8 */

	/* size: 56, cachelines: 1, members: 6 */
	/* last cacheline: 56 bytes */
};
struct snap {
	int                        slen;                 /*     0     4 */
	char                       str[80];              /*     4    80 */

	/* size: 84, cachelines: 2, members: 2 */
	/* last cacheline: 20 bytes */
};
struct mon_iso_desc {
	int                        status;               /*     0     4 */
	unsigned int               offset;               /*     4     4 */
	unsigned int               length;               /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct mon_event_text {
	struct list_head           e_link;               /*     0    16 */
	int                        type;                 /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          id;                   /*    24     8 */
	unsigned int               tstamp;               /*    32     4 */
	int                        busnum;               /*    36     4 */
	char                       devnum;               /*    40     1 */
	char                       epnum;                /*    41     1 */
	char                       is_in;                /*    42     1 */
	char                       xfertype;             /*    43     1 */
	int                        length;               /*    44     4 */
	int                        status;               /*    48     4 */
	int                        interval;             /*    52     4 */
	int                        start_frame;          /*    56     4 */
	int                        error_count;          /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	char                       setup_flag;           /*    64     1 */
	char                       data_flag;            /*    65     1 */

	/* XXX 2 bytes hole, try to pack */

	int                        numdesc;              /*    68     4 */
	struct mon_iso_desc        isodesc[5];           /*    72    60 */
	/* --- cacheline 2 boundary (128 bytes) was 4 bytes ago --- */
	unsigned char              setup[8];             /*   132     8 */
	unsigned char              data[32];             /*   140    32 */

	/* size: 176, cachelines: 3, members: 20 */
	/* sum members: 166, holes: 2, sum holes: 6 */
	/* padding: 4 */
	/* last cacheline: 48 bytes */
};
struct mon_reader_text {
	struct kmem_cache *        e_slab;               /*     0     8 */
	int                        nevents;              /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           e_list;               /*    16    16 */
	struct mon_reader          r;                    /*    32    56 */
	/* --- cacheline 1 boundary (64 bytes) was 24 bytes ago --- */
	wait_queue_head_t          wait;                 /*    88    24 */
	int                        printf_size;          /*   112     4 */

	/* XXX 4 bytes hole, try to pack */

	size_t                     printf_offset;        /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	size_t                     printf_togo;          /*   128     8 */
	char *                     printf_buf;           /*   136     8 */
	struct mutex               printf_lock;          /*   144    32 */
	char                       slab_name[30];        /*   176    30 */

	/* size: 208, cachelines: 4, members: 11 */
	/* sum members: 198, holes: 2, sum holes: 8 */
	/* padding: 2 */
	/* last cacheline: 16 bytes */
};
struct mon_text_ptr {
	int                        cnt;                  /*     0     4 */
	int                        limit;                /*     4     4 */
	char *                     pbuf;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct iso_rec {
	int                        error_count;          /*     0     4 */
	int                        numdesc;              /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct mon_bin_hdr {
	u64                        id;                   /*     0     8 */
	unsigned char              type;                 /*     8     1 */
	unsigned char              xfer_type;            /*     9     1 */
	unsigned char              epnum;                /*    10     1 */
	unsigned char              devnum;               /*    11     1 */
	short unsigned int         busnum;               /*    12     2 */
	char                       flag_setup;           /*    14     1 */
	char                       flag_data;            /*    15     1 */
	s64                        ts_sec;               /*    16     8 */
	s32                        ts_usec;              /*    24     4 */
	int                        status;               /*    28     4 */
	unsigned int               len_urb;              /*    32     4 */
	unsigned int               len_cap;              /*    36     4 */
	union {
		unsigned char      setup[8];             /*    40     8 */
		struct iso_rec     iso;                  /*    40     8 */
	} s;                                             /*    40     8 */
	int                        interval;             /*    48     4 */
	int                        start_frame;          /*    52     4 */
	unsigned int               xfer_flags;           /*    56     4 */
	unsigned int               ndesc;                /*    60     4 */

	/* size: 64, cachelines: 1, members: 18 */
};
struct mon_bin_isodesc {
	int                        iso_status;           /*     0     4 */
	unsigned int               iso_off;              /*     4     4 */
	unsigned int               iso_len;              /*     8     4 */
	u32                        _pad;                 /*    12     4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct mon_bin_stats {
	u32                        queued;               /*     0     4 */
	u32                        dropped;              /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct mon_bin_get {
	struct mon_bin_hdr *       hdr;                  /*     0     8 */
	void *                     data;                 /*     8     8 */
	size_t                     alloc;                /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct mon_bin_mfetch {
	u32 *                      offvec;               /*     0     8 */
	u32                        nfetch;               /*     8     4 */
	u32                        nflush;               /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct mon_bin_get32 {
	u32                        hdr32;                /*     0     4 */
	u32                        data32;               /*     4     4 */
	u32                        alloc32;              /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct mon_bin_mfetch32 {
	u32                        offvec32;             /*     0     4 */
	u32                        nfetch32;             /*     4     4 */
	u32                        nflush32;             /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct mon_pgmap {
	struct page *              pg;                   /*     0     8 */
	unsigned char *            ptr;                  /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct mon_reader_bin {
	spinlock_t                 b_lock;               /*     0     4 */
	unsigned int               b_size;               /*     4     4 */
	unsigned int               b_cnt;                /*     8     4 */
	unsigned int               b_in;                 /*    12     4 */
	unsigned int               b_out;                /*    16     4 */
	unsigned int               b_read;               /*    20     4 */
	struct mon_pgmap *         b_vec;                /*    24     8 */
	wait_queue_head_t          b_wait;               /*    32    24 */
	struct mutex               fetch_lock;           /*    56    32 */
	/* --- cacheline 1 boundary (64 bytes) was 24 bytes ago --- */
	int                        mmap_active;          /*    88     4 */

	/* XXX 4 bytes hole, try to pack */

	struct mon_reader          r;                    /*    96    56 */
	/* --- cacheline 2 boundary (128 bytes) was 24 bytes ago --- */
	unsigned int               cnt_lost;             /*   152     4 */

	/* size: 160, cachelines: 3, members: 12 */
	/* sum members: 152, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct amd_chipset_type {
	enum amd_chipset_gen       gen;                  /*     0     4 */
	u8                         rev;                  /*     4     1 */

	/* size: 8, cachelines: 1, members: 2 */
	/* padding: 3 */
	/* last cacheline: 8 bytes */
};
struct amd_chipset_info {
	struct pci_dev *           nb_dev;               /*     0     8 */
	struct pci_dev *           smbus_dev;            /*     8     8 */
	int                        nb_type;              /*    16     4 */
	struct amd_chipset_type    sb_type;              /*    20     8 */

	/* XXX last struct has 3 bytes of padding */

	int                        isoc_reqs;            /*    28     4 */
	int                        probe_count;          /*    32     4 */
	bool                       need_pll_quirk;       /*    36     1 */

	/* size: 40, cachelines: 1, members: 7 */
	/* padding: 3 */
	/* paddings: 1, sum paddings: 3 */
	/* last cacheline: 40 bytes */
};
struct ehci_stats {
	long unsigned int          normal;               /*     0     8 */
	long unsigned int          error;                /*     8     8 */
	long unsigned int          iaa;                  /*    16     8 */
	long unsigned int          lost_iaa;             /*    24     8 */
	long unsigned int          complete;             /*    32     8 */
	long unsigned int          unlink;               /*    40     8 */

	/* size: 48, cachelines: 1, members: 6 */
	/* last cacheline: 48 bytes */
};
struct ehci_per_sched {
	struct usb_device *        udev;                 /*     0     8 */
	struct usb_host_endpoint * ep;                   /*     8     8 */
	struct list_head           ps_list;              /*    16    16 */
	u16                        tt_usecs;             /*    32     2 */
	u16                        cs_mask;              /*    34     2 */
	u16                        period;               /*    36     2 */
	u16                        phase;                /*    38     2 */
	u8                         bw_phase;             /*    40     1 */
	u8                         phase_uf;             /*    41     1 */
	u8                         usecs;                /*    42     1 */
	u8                         c_usecs;              /*    43     1 */
	u8                         bw_uperiod;           /*    44     1 */
	u8                         bw_period;            /*    45     1 */

	/* size: 48, cachelines: 1, members: 13 */
	/* padding: 2 */
	/* last cacheline: 48 bytes */
};
struct ehci_hcd {
	enum ehci_hrtimer_event    next_hrtimer_event;   /*     0     4 */
	unsigned int               enabled_hrtimer_events; /*     4     4 */
	ktime_t                    hr_timeouts[12];      /*     8    96 */
	/* --- cacheline 1 boundary (64 bytes) was 40 bytes ago --- */
	struct hrtimer             hrtimer __attribute__((__aligned__(8))); /*   104    96 */
	/* --- cacheline 3 boundary (192 bytes) was 8 bytes ago --- */
	int                        PSS_poll_count;       /*   200     4 */
	int                        ASS_poll_count;       /*   204     4 */
	int                        died_poll_count;      /*   208     4 */

	/* XXX 4 bytes hole, try to pack */

	struct ehci_caps *         caps;                 /*   216     8 */
	struct ehci_regs *         regs;                 /*   224     8 */
	struct ehci_dbg_port *     debug;                /*   232     8 */
	__u32                      hcs_params;           /*   240     4 */
	spinlock_t                 lock;                 /*   244     4 */
	enum ehci_rh_state         rh_state;             /*   248     4 */
	bool                       scanning:1;           /*   252: 0  1 */
	bool                       need_rescan:1;        /*   252: 1  1 */
	bool                       intr_unlinking:1;     /*   252: 2  1 */
	bool                       iaa_in_progress:1;    /*   252: 3  1 */
	bool                       async_unlinking:1;    /*   252: 4  1 */
	bool                       shutdown:1;           /*   252: 5  1 */

	/* XXX 2 bits hole, try to pack */
	/* XXX 3 bytes hole, try to pack */

	/* --- cacheline 4 boundary (256 bytes) --- */
	struct ehci_qh *           qh_scan_next;         /*   256     8 */
	struct ehci_qh *           async;                /*   264     8 */
	struct ehci_qh *           dummy;                /*   272     8 */
	struct list_head           async_unlink;         /*   280    16 */
	struct list_head           async_idle;           /*   296    16 */
	unsigned int               async_unlink_cycle;   /*   312     4 */
	unsigned int               async_count;          /*   316     4 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	__le32                     old_current;          /*   320     4 */
	__le32                     old_token;            /*   324     4 */
	unsigned int               periodic_size;        /*   328     4 */

	/* XXX 4 bytes hole, try to pack */

	__le32 *                   periodic;             /*   336     8 */
	dma_addr_t                 periodic_dma;         /*   344     8 */
	struct list_head           intr_qh_list;         /*   352    16 */
	unsigned int               i_thresh;             /*   368     4 */

	/* XXX 4 bytes hole, try to pack */

	union ehci_shadow *        pshadow;              /*   376     8 */
	/* --- cacheline 6 boundary (384 bytes) --- */
	struct list_head           intr_unlink_wait;     /*   384    16 */
	struct list_head           intr_unlink;          /*   400    16 */
	unsigned int               intr_unlink_wait_cycle; /*   416     4 */
	unsigned int               intr_unlink_cycle;    /*   420     4 */
	unsigned int               now_frame;            /*   424     4 */
	unsigned int               last_iso_frame;       /*   428     4 */
	unsigned int               intr_count;           /*   432     4 */
	unsigned int               isoc_count;           /*   436     4 */
	unsigned int               periodic_count;       /*   440     4 */
	unsigned int               uframe_periodic_max;  /*   444     4 */
	/* --- cacheline 7 boundary (448 bytes) --- */
	struct list_head           cached_itd_list;      /*   448    16 */
	struct ehci_itd *          last_itd_to_free;     /*   464     8 */
	struct list_head           cached_sitd_list;     /*   472    16 */
	struct ehci_sitd *         last_sitd_to_free;    /*   488     8 */
	long unsigned int          reset_done[15];       /*   496   120 */
	/* --- cacheline 9 boundary (576 bytes) was 40 bytes ago --- */
	long unsigned int          bus_suspended;        /*   616     8 */
	long unsigned int          companion_ports;      /*   624     8 */
	long unsigned int          owned_ports;          /*   632     8 */
	/* --- cacheline 10 boundary (640 bytes) --- */
	long unsigned int          port_c_suspend;       /*   640     8 */
	long unsigned int          suspended_ports;      /*   648     8 */
	long unsigned int          resuming_ports;       /*   656     8 */
	struct dma_pool *          qh_pool;              /*   664     8 */
	struct dma_pool *          qtd_pool;             /*   672     8 */
	struct dma_pool *          itd_pool;             /*   680     8 */
	struct dma_pool *          sitd_pool;            /*   688     8 */
	unsigned int               random_frame;         /*   696     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 11 boundary (704 bytes) --- */
	long unsigned int          next_statechange;     /*   704     8 */
	ktime_t                    last_periodic_enable; /*   712     8 */
	u32                        command;              /*   720     4 */
	unsigned int               no_selective_suspend:1; /*   724: 0  4 */
	unsigned int               has_fsl_port_bug:1;   /*   724: 1  4 */
	unsigned int               has_fsl_hs_errata:1;  /*   724: 2  4 */
	unsigned int               has_fsl_susp_errata:1; /*   724: 3  4 */
	unsigned int               big_endian_mmio:1;    /*   724: 4  4 */
	unsigned int               big_endian_desc:1;    /*   724: 5  4 */
	unsigned int               big_endian_capbase:1; /*   724: 6  4 */
	unsigned int               has_amcc_usb23:1;     /*   724: 7  4 */
	unsigned int               need_io_watchdog:1;   /*   724: 8  4 */
	unsigned int               amd_pll_fix:1;        /*   724: 9  4 */
	unsigned int               use_dummy_qh:1;       /*   724:10  4 */
	unsigned int               has_synopsys_hc_bug:1; /*   724:11  4 */
	unsigned int               frame_index_bug:1;    /*   724:12  4 */
	unsigned int               need_oc_pp_cycle:1;   /*   724:13  4 */
	unsigned int               imx28_write_fix:1;    /*   724:14  4 */
	unsigned int               is_aspeed:1;          /*   724:15  4 */
	unsigned int               zx_wakeup_clear:1;    /*   724:16  4 */

	/* XXX 15 bits hole, try to pack */

	__le32 *                   ohci_hcctrl_reg;      /*   728     8 */
	unsigned int               has_hostpc:1;         /*   736: 0  4 */
	unsigned int               has_tdi_phy_lpm:1;    /*   736: 1  4 */
	unsigned int               has_ppcd:1;           /*   736: 2  4 */

	/* XXX 5 bits hole, try to pack */
	/* Bitfield combined with next fields */

	u8                         sbrn;                 /*   737     1 */

	/* XXX 6 bytes hole, try to pack */

	struct ehci_stats          stats;                /*   744    48 */
	/* --- cacheline 12 boundary (768 bytes) was 24 bytes ago --- */
	struct dentry *            debug_dir;            /*   792     8 */
	u8                         bandwidth[64];        /*   800    64 */
	/* --- cacheline 13 boundary (832 bytes) was 32 bytes ago --- */
	u8                         tt_budget[64];        /*   864    64 */
	/* --- cacheline 14 boundary (896 bytes) was 32 bytes ago --- */
	struct list_head           tt_list;              /*   928    16 */
	long unsigned int          priv[] __attribute__((__aligned__(8))); /*   944     0 */

	/* size: 944, cachelines: 15, members: 91 */
	/* sum members: 913, holes: 6, sum holes: 25 */
	/* sum bitfield members: 26 bits, bit holes: 3, sum bit holes: 22 bits */
	/* forced alignments: 2 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct ehci_caps {
	u32                        hc_capbase;           /*     0     4 */
	u32                        hcs_params;           /*     4     4 */
	u32                        hcc_params;           /*     8     4 */
	u8                         portroute[8];         /*    12     8 */

	/* size: 20, cachelines: 1, members: 4 */
	/* last cacheline: 20 bytes */
};
struct ehci_regs {
	u32                        command;              /*     0     4 */
	u32                        status;               /*     4     4 */
	u32                        intr_enable;          /*     8     4 */
	u32                        frame_index;          /*    12     4 */
	u32                        segment;              /*    16     4 */
	u32                        frame_list;           /*    20     4 */
	u32                        async_next;           /*    24     4 */
	u32                        reserved1[2];         /*    28     8 */
	u32                        txfill_tuning;        /*    36     4 */
	u32                        reserved2[6];         /*    40    24 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u32                        configured_flag;      /*    64     4 */
	u32                        port_status[0];       /*    68     0 */
	u32                        reserved3[9];         /*    68    36 */
	u32                        usbmode;              /*   104     4 */
	u32                        reserved4[6];         /*   108    24 */
	/* --- cacheline 2 boundary (128 bytes) was 4 bytes ago --- */
	u32                        hostpc[0];            /*   132     0 */
	u32                        reserved5[17];        /*   132    68 */
	/* --- cacheline 3 boundary (192 bytes) was 8 bytes ago --- */
	u32                        usbmode_ex;           /*   200     4 */

	/* size: 204, cachelines: 4, members: 18 */
	/* last cacheline: 12 bytes */
};
struct ehci_dbg_port {
	u32                        control;              /*     0     4 */
	u32                        pids;                 /*     4     4 */
	u32                        data03;               /*     8     4 */
	u32                        data47;               /*    12     4 */
	u32                        address;              /*    16     4 */

	/* size: 20, cachelines: 1, members: 5 */
	/* last cacheline: 20 bytes */
};
struct ehci_qh {
	struct ehci_qh_hw *        hw;                   /*     0     8 */
	dma_addr_t                 qh_dma;               /*     8     8 */
	union ehci_shadow          qh_next;              /*    16     8 */
	struct list_head           qtd_list;             /*    24    16 */
	struct list_head           intr_node;            /*    40    16 */
	struct ehci_qtd *          dummy;                /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct list_head           unlink_node;          /*    64    16 */
	struct ehci_per_sched      ps;                   /*    80    48 */

	/* XXX last struct has 2 bytes of padding */

	/* --- cacheline 2 boundary (128 bytes) --- */
	unsigned int               unlink_cycle;         /*   128     4 */
	u8                         qh_state;             /*   132     1 */
	u8                         xacterrs;             /*   133     1 */
	u8                         unlink_reason;        /*   134     1 */
	u8                         gap_uf;               /*   135     1 */
	unsigned int               is_out:1;             /*   136: 0  4 */
	unsigned int               clearing_tt:1;        /*   136: 1  4 */
	unsigned int               dequeue_during_giveback:1; /*   136: 2  4 */
	unsigned int               should_be_inactive:1; /*   136: 3  4 */

	/* size: 144, cachelines: 3, members: 17 */
	/* padding: 4 */
	/* paddings: 1, sum paddings: 2 */
	/* bit_padding: 28 bits */
	/* last cacheline: 16 bytes */
};
union ehci_shadow {
	struct ehci_qh *           qh;                 /*     0     8 */
	struct ehci_itd *          itd;                /*     0     8 */
	struct ehci_sitd *         sitd;               /*     0     8 */
	struct ehci_fstn *         fstn;               /*     0     8 */
	__le32 *                   hw_next;            /*     0     8 */
	void *                     ptr;                /*     0     8 */
};
struct ehci_itd {
	__le32                     hw_next;              /*     0     4 */
	__le32                     hw_transaction[8];    /*     4    32 */
	__le32                     hw_bufp[7];           /*    36    28 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	__le32                     hw_bufp_hi[7];        /*    64    28 */

	/* XXX 4 bytes hole, try to pack */

	dma_addr_t                 itd_dma;              /*    96     8 */
	union ehci_shadow          itd_next;             /*   104     8 */
	struct urb *               urb;                  /*   112     8 */
	struct ehci_iso_stream *   stream;               /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct list_head           itd_list;             /*   128    16 */
	unsigned int               frame;                /*   144     4 */
	unsigned int               pg;                   /*   148     4 */
	unsigned int               index[8];             /*   152    32 */

	/* size: 192, cachelines: 3, members: 12 */
	/* sum members: 180, holes: 1, sum holes: 4 */
	/* padding: 8 */
} __attribute__((__aligned__(32)));
struct ehci_sitd {
	__le32                     hw_next;              /*     0     4 */
	__le32                     hw_fullspeed_ep;      /*     4     4 */
	__le32                     hw_uframe;            /*     8     4 */
	__le32                     hw_results;           /*    12     4 */
	__le32                     hw_buf[2];            /*    16     8 */
	__le32                     hw_backpointer;       /*    24     4 */
	__le32                     hw_buf_hi[2];         /*    28     8 */

	/* XXX 4 bytes hole, try to pack */

	dma_addr_t                 sitd_dma;             /*    40     8 */
	union ehci_shadow          sitd_next;            /*    48     8 */
	struct urb *               urb;                  /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct ehci_iso_stream *   stream;               /*    64     8 */
	struct list_head           sitd_list;            /*    72    16 */
	unsigned int               frame;                /*    88     4 */
	unsigned int               index;                /*    92     4 */

	/* size: 96, cachelines: 2, members: 14 */
	/* sum members: 92, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(32)));
struct ehci_qtd {
	__le32                     hw_next;              /*     0     4 */
	__le32                     hw_alt_next;          /*     4     4 */
	__le32                     hw_token;             /*     8     4 */
	__le32                     hw_buf[5];            /*    12    20 */
	__le32                     hw_buf_hi[5];         /*    32    20 */

	/* XXX 4 bytes hole, try to pack */

	dma_addr_t                 qtd_dma;              /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct list_head           qtd_list;             /*    64    16 */
	struct urb *               urb;                  /*    80     8 */
	size_t                     length;               /*    88     8 */

	/* size: 96, cachelines: 2, members: 9 */
	/* sum members: 92, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(32)));
struct ehci_fstn {
	__le32                     hw_next;              /*     0     4 */
	__le32                     hw_prev;              /*     4     4 */
	dma_addr_t                 fstn_dma;             /*     8     8 */
	union ehci_shadow          fstn_next;            /*    16     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* padding: 8 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(32)));
struct ehci_qh_hw {
	__le32                     hw_next;              /*     0     4 */
	__le32                     hw_info1;             /*     4     4 */
	__le32                     hw_info2;             /*     8     4 */
	__le32                     hw_current;           /*    12     4 */
	__le32                     hw_qtd_next;          /*    16     4 */
	__le32                     hw_alt_next;          /*    20     4 */
	__le32                     hw_token;             /*    24     4 */
	__le32                     hw_buf[5];            /*    28    20 */
	__le32                     hw_buf_hi[5];         /*    48    20 */

	/* size: 96, cachelines: 2, members: 9 */
	/* padding: 28 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(32)));
struct ehci_iso_packet {
	u64                        bufp;                 /*     0     8 */
	__le32                     transaction;          /*     8     4 */
	u8                         cross;                /*    12     1 */

	/* XXX 3 bytes hole, try to pack */

	u32                        buf1;                 /*    16     4 */

	/* size: 24, cachelines: 1, members: 4 */
	/* sum members: 17, holes: 1, sum holes: 3 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct ehci_iso_sched {
	struct list_head           td_list;              /*     0    16 */
	unsigned int               span;                 /*    16     4 */
	unsigned int               first_packet;         /*    20     4 */
	struct ehci_iso_packet     packet[];             /*    24     0 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct ehci_iso_stream {
	struct ehci_qh_hw *        hw;                   /*     0     8 */
	u8                         bEndpointAddress;     /*     8     1 */
	u8                         highspeed;            /*     9     1 */

	/* XXX 6 bytes hole, try to pack */

	struct list_head           td_list;              /*    16    16 */
	struct list_head           free_list;            /*    32    16 */
	struct ehci_per_sched      ps;                   /*    48    48 */

	/* XXX last struct has 2 bytes of padding */

	/* --- cacheline 1 boundary (64 bytes) was 32 bytes ago --- */
	unsigned int               next_uframe;          /*    96     4 */
	__le32                     splits;               /*   100     4 */
	u16                        uperiod;              /*   104     2 */
	u16                        maxp;                 /*   106     2 */
	unsigned int               bandwidth;            /*   108     4 */
	__le32                     buf0;                 /*   112     4 */
	__le32                     buf1;                 /*   116     4 */
	__le32                     buf2;                 /*   120     4 */
	__le32                     address;              /*   124     4 */

	/* size: 128, cachelines: 2, members: 15 */
	/* sum members: 122, holes: 1, sum holes: 6 */
	/* paddings: 1, sum paddings: 2 */
};
struct ehci_tt {
	u16                        bandwidth[8];         /*     0    16 */
	struct list_head           tt_list;              /*    16    16 */
	struct list_head           ps_list;              /*    32    16 */
	struct usb_tt *            usb_tt;               /*    48     8 */
	int                        tt_port;              /*    56     4 */

	/* size: 64, cachelines: 1, members: 5 */
	/* padding: 4 */
};
struct ehci_driver_overrides {
	size_t                     extra_priv_size;      /*     0     8 */
	int                        (*reset)(struct usb_hcd *); /*     8     8 */
	int                        (*port_power)(struct usb_hcd *, int, bool); /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct debug_buffer {
	ssize_t                    (*fill_func)(struct debug_buffer *); /*     0     8 */
	struct usb_bus *           bus;                  /*     8     8 */
	struct mutex               mutex;                /*    16    32 */
	size_t                     count;                /*    48     8 */
	char *                     output_buf;           /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	size_t                     alloc_size;           /*    64     8 */

	/* size: 72, cachelines: 2, members: 6 */
	/* last cacheline: 8 bytes */
};
struct ed {
	__hc32                     hwINFO;               /*     0     4 */
	__hc32                     hwTailP;              /*     4     4 */
	__hc32                     hwHeadP;              /*     8     4 */
	__hc32                     hwNextED;             /*    12     4 */
	dma_addr_t                 dma;                  /*    16     8 */
	struct td *                dummy;                /*    24     8 */
	struct ed *                ed_next;              /*    32     8 */
	struct ed *                ed_prev;              /*    40     8 */
	struct list_head           td_list;              /*    48    16 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct list_head           in_use_list;          /*    64    16 */
	u8                         state;                /*    80     1 */
	u8                         type;                 /*    81     1 */
	u8                         branch;               /*    82     1 */

	/* XXX 1 byte hole, try to pack */

	u16                        interval;             /*    84     2 */
	u16                        load;                 /*    86     2 */
	u16                        last_iso;             /*    88     2 */
	u16                        tick;                 /*    90     2 */
	unsigned int               takeback_wdh_cnt;     /*    92     4 */
	struct td *                pending_td;           /*    96     8 */

	/* size: 112, cachelines: 2, members: 19 */
	/* sum members: 103, holes: 1, sum holes: 1 */
	/* padding: 8 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(16)));
struct td {
	__hc32                     hwINFO;               /*     0     4 */
	__hc32                     hwCBP;                /*     4     4 */
	__hc32                     hwNextTD;             /*     8     4 */
	__hc32                     hwBE;                 /*    12     4 */
	__hc16                     hwPSW[2];             /*    16     4 */
	__u8                       index;                /*    20     1 */

	/* XXX 3 bytes hole, try to pack */

	struct ed *                ed;                   /*    24     8 */
	struct td *                td_hash;              /*    32     8 */
	struct td *                next_dl_td;           /*    40     8 */
	struct urb *               urb;                  /*    48     8 */
	dma_addr_t                 td_dma;               /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	dma_addr_t                 data_dma;             /*    64     8 */
	struct list_head           td_list;              /*    72    16 */

	/* size: 96, cachelines: 2, members: 13 */
	/* sum members: 85, holes: 1, sum holes: 3 */
	/* padding: 8 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(32)));
struct ohci_hcca {
	__hc32                     int_table[32];        /*     0   128 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	__hc32                     frame_no;             /*   128     4 */
	__hc32                     done_head;            /*   132     4 */
	u8                         reserved_for_hc[116]; /*   136   116 */
	/* --- cacheline 3 boundary (192 bytes) was 60 bytes ago --- */
	u8                         what[4];              /*   252     4 */

	/* size: 256, cachelines: 4, members: 5 */
} __attribute__((__aligned__(256)));
struct ohci_roothub_regs {
	__hc32                     a;                    /*     0     4 */
	__hc32                     b;                    /*     4     4 */
	__hc32                     status;               /*     8     4 */
	__hc32                     portstatus[15];       /*    12    60 */

	/* size: 72, cachelines: 2, members: 4 */
	/* last cacheline: 8 bytes */
};
struct ohci_regs {
	__hc32                     revision;             /*     0     4 */
	__hc32                     control;              /*     4     4 */
	__hc32                     cmdstatus;            /*     8     4 */
	__hc32                     intrstatus;           /*    12     4 */
	__hc32                     intrenable;           /*    16     4 */
	__hc32                     intrdisable;          /*    20     4 */
	__hc32                     hcca;                 /*    24     4 */
	__hc32                     ed_periodcurrent;     /*    28     4 */
	__hc32                     ed_controlhead;       /*    32     4 */
	__hc32                     ed_controlcurrent;    /*    36     4 */
	__hc32                     ed_bulkhead;          /*    40     4 */
	__hc32                     ed_bulkcurrent;       /*    44     4 */
	__hc32                     donehead;             /*    48     4 */
	__hc32                     fminterval;           /*    52     4 */
	__hc32                     fmremaining;          /*    56     4 */
	__hc32                     fmnumber;             /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	__hc32                     periodicstart;        /*    64     4 */
	__hc32                     lsthresh;             /*    68     4 */
	struct ohci_roothub_regs   roothub;              /*    72    72 */

	/* size: 160, cachelines: 3, members: 19 */
	/* padding: 16 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(32)));
struct urb_priv {
	struct ed *                ed;                   /*     0     8 */
	u16                        length;               /*     8     2 */
	u16                        td_cnt;               /*    10     2 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           pending;              /*    16    16 */
	struct td *                td[];                 /*    32     0 */

	/* size: 32, cachelines: 1, members: 5 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct ohci_hcd {
	spinlock_t                 lock;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct ohci_regs *         regs;                 /*     8     8 */
	struct ohci_hcca *         hcca;                 /*    16     8 */
	dma_addr_t                 hcca_dma;             /*    24     8 */
	struct ed *                ed_rm_list;           /*    32     8 */
	struct ed *                ed_bulktail;          /*    40     8 */
	struct ed *                ed_controltail;       /*    48     8 */
	struct ed *                periodic[32];         /*    56   256 */
	/* --- cacheline 4 boundary (256 bytes) was 56 bytes ago --- */
	void                       (*start_hnp)(struct ohci_hcd *); /*   312     8 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	struct dma_pool *          td_cache;             /*   320     8 */
	struct dma_pool *          ed_cache;             /*   328     8 */
	struct td *                td_hash[64];          /*   336   512 */
	/* --- cacheline 13 boundary (832 bytes) was 16 bytes ago --- */
	struct td *                dl_start;             /*   848     8 */
	struct td *                dl_end;               /*   856     8 */
	struct list_head           pending;              /*   864    16 */
	struct list_head           eds_in_use;           /*   880    16 */
	/* --- cacheline 14 boundary (896 bytes) --- */
	enum ohci_rh_state         rh_state;             /*   896     4 */
	int                        num_ports;            /*   900     4 */
	int                        load[32];             /*   904   128 */
	/* --- cacheline 16 boundary (1024 bytes) was 8 bytes ago --- */
	u32                        hc_control;           /*  1032     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          next_statechange;     /*  1040     8 */
	u32                        fminterval;           /*  1048     4 */
	unsigned int               autostop:1;           /*  1052: 0  4 */
	unsigned int               working:1;            /*  1052: 1  4 */
	unsigned int               restart_work:1;       /*  1052: 2  4 */

	/* XXX 29 bits hole, try to pack */

	long unsigned int          flags;                /*  1056     8 */
	unsigned int               prev_frame_no;        /*  1064     4 */
	unsigned int               wdh_cnt;              /*  1068     4 */
	unsigned int               prev_wdh_cnt;         /*  1072     4 */
	u32                        prev_donehead;        /*  1076     4 */
	struct timer_list          io_watchdog;          /*  1080    72 */
	/* --- cacheline 18 boundary (1152 bytes) --- */
	struct work_struct         nec_work;             /*  1152    64 */
	/* --- cacheline 19 boundary (1216 bytes) --- */
	struct dentry *            debug_dir;            /*  1216     8 */
	long unsigned int          priv[] __attribute__((__aligned__(8))); /*  1224     0 */

	/* size: 1224, cachelines: 20, members: 34 */
	/* sum members: 1212, holes: 2, sum holes: 8 */
	/* sum bitfield members: 3 bits, bit holes: 1, sum bit holes: 29 bits */
	/* forced alignments: 1 */
	/* last cacheline: 8 bytes */
} __attribute__((__aligned__(8)));
struct ohci_driver_overrides {
	const char  *              product_desc;         /*     0     8 */
	size_t                     extra_priv_size;      /*     8     8 */
	int                        (*reset)(struct usb_hcd *); /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct debug_buffer {
	ssize_t                    (*fill_func)(struct debug_buffer *); /*     0     8 */
	struct ohci_hcd *          ohci;                 /*     8     8 */
	struct mutex               mutex;                /*    16    32 */
	size_t                     count;                /*    48     8 */
	char *                     page;                 /*    56     8 */

	/* size: 64, cachelines: 1, members: 5 */
};
struct uhci_qh {
	__le32                     link;                 /*     0     4 */
	__le32                     element;              /*     4     4 */
	dma_addr_t                 dma_handle;           /*     8     8 */
	struct list_head           node;                 /*    16    16 */
	struct usb_host_endpoint * hep;                  /*    32     8 */
	struct usb_device *        udev;                 /*    40     8 */
	struct list_head           queue;                /*    48    16 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct uhci_td *           dummy_td;             /*    64     8 */
	struct uhci_td *           post_td;              /*    72     8 */
	struct usb_iso_packet_descriptor * iso_packet_desc; /*    80     8 */
	long unsigned int          advance_jiffies;      /*    88     8 */
	unsigned int               unlink_frame;         /*    96     4 */
	unsigned int               period;               /*   100     4 */
	short int                  phase;                /*   104     2 */
	short int                  load;                 /*   106     2 */
	unsigned int               iso_frame;            /*   108     4 */
	int                        state;                /*   112     4 */
	int                        type;                 /*   116     4 */
	int                        skel;                 /*   120     4 */
	unsigned int               initial_toggle:1;     /*   124: 0  4 */
	unsigned int               needs_fixup:1;        /*   124: 1  4 */
	unsigned int               is_stopped:1;         /*   124: 2  4 */
	unsigned int               wait_expired:1;       /*   124: 3  4 */
	unsigned int               bandwidth_reserved:1; /*   124: 4  4 */

	/* size: 128, cachelines: 2, members: 24 */
	/* bit_padding: 27 bits */
} __attribute__((__aligned__(16)));
struct uhci_td {
	__le32                     link;                 /*     0     4 */
	__le32                     status;               /*     4     4 */
	__le32                     token;                /*     8     4 */
	__le32                     buffer;               /*    12     4 */
	dma_addr_t                 dma_handle;           /*    16     8 */
	struct list_head           list;                 /*    24    16 */
	int                        frame;                /*    40     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           fl_list;              /*    48    16 */

	/* size: 64, cachelines: 1, members: 8 */
	/* sum members: 60, holes: 1, sum holes: 4 */
} __attribute__((__aligned__(16)));
struct uhci_hcd {
	struct dentry *            dentry;               /*     0     8 */
	long unsigned int          io_addr;              /*     8     8 */
	void *                     regs;                 /*    16     8 */
	struct dma_pool *          qh_pool;              /*    24     8 */
	struct dma_pool *          td_pool;              /*    32     8 */
	struct uhci_td *           term_td;              /*    40     8 */
	struct uhci_qh *           skelqh[11];           /*    48    88 */
	/* --- cacheline 2 boundary (128 bytes) was 8 bytes ago --- */
	struct uhci_qh *           next_qh;              /*   136     8 */
	spinlock_t                 lock;                 /*   144     4 */

	/* XXX 4 bytes hole, try to pack */

	dma_addr_t                 frame_dma_handle;     /*   152     8 */
	__le32 *                   frame;                /*   160     8 */
	void * *                   frame_cpu;            /*   168     8 */
	enum uhci_rh_state         rh_state;             /*   176     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          auto_stop_time;       /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	unsigned int               frame_number;         /*   192     4 */
	unsigned int               is_stopped;           /*   196     4 */
	unsigned int               last_iso_frame;       /*   200     4 */
	unsigned int               cur_iso_frame;        /*   204     4 */
	unsigned int               scan_in_progress:1;   /*   208: 0  4 */
	unsigned int               need_rescan:1;        /*   208: 1  4 */
	unsigned int               dead:1;               /*   208: 2  4 */
	unsigned int               RD_enable:1;          /*   208: 3  4 */
	unsigned int               is_initialized:1;     /*   208: 4  4 */
	unsigned int               fsbr_is_on:1;         /*   208: 5  4 */
	unsigned int               fsbr_is_wanted:1;     /*   208: 6  4 */
	unsigned int               fsbr_expiring:1;      /*   208: 7  4 */

	/* XXX 24 bits hole, try to pack */
	/* XXX 4 bytes hole, try to pack */

	struct timer_list          fsbr_timer;           /*   216    72 */
	/* --- cacheline 4 boundary (256 bytes) was 32 bytes ago --- */
	unsigned int               oc_low:1;             /*   288: 0  4 */
	unsigned int               wait_for_hp:1;        /*   288: 1  4 */
	unsigned int               big_endian_mmio:1;    /*   288: 2  4 */
	unsigned int               big_endian_desc:1;    /*   288: 3  4 */
	unsigned int               is_aspeed:1;          /*   288: 4  4 */

	/* XXX 27 bits hole, try to pack */
	/* XXX 4 bytes hole, try to pack */

	long unsigned int          port_c_suspend;       /*   296     8 */
	long unsigned int          resuming_ports;       /*   304     8 */
	long unsigned int          ports_timeout;        /*   312     8 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	struct list_head           idle_qh_list;         /*   320    16 */
	int                        rh_numports;          /*   336     4 */

	/* XXX 4 bytes hole, try to pack */

	wait_queue_head_t          waitqh;               /*   344    24 */
	int                        num_waiting;          /*   368     4 */
	int                        total_load;           /*   372     4 */
	short int                  load[32];             /*   376    64 */
	/* --- cacheline 6 boundary (384 bytes) was 56 bytes ago --- */
	struct clk *               clk;                  /*   440     8 */
	/* --- cacheline 7 boundary (448 bytes) --- */
	void                       (*reset_hc)(struct uhci_hcd *); /*   448     8 */
	int                        (*check_and_reset_hc)(struct uhci_hcd *); /*   456     8 */
	void                       (*configure_hc)(struct uhci_hcd *); /*   464     8 */
	int                        (*resume_detect_interrupts_are_broken)(struct uhci_hcd *); /*   472     8 */
	int                        (*global_suspend_mode_is_broken)(struct uhci_hcd *); /*   480     8 */

	/* size: 488, cachelines: 8, members: 47 */
	/* sum members: 460, holes: 5, sum holes: 20 */
	/* sum bitfield members: 13 bits, bit holes: 2, sum bit holes: 51 bits */
	/* last cacheline: 40 bytes */
};
struct urb_priv {
	struct list_head           node;                 /*     0    16 */
	struct urb *               urb;                  /*    16     8 */
	struct uhci_qh *           qh;                   /*    24     8 */
	struct list_head           td_list;              /*    32    16 */
	unsigned int               fsbr:1;               /*    48: 0  4 */

	/* size: 56, cachelines: 1, members: 5 */
	/* padding: 4 */
	/* bit_padding: 31 bits */
	/* last cacheline: 56 bytes */
};
struct uhci_debug {
	int                        size;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	char *                     data;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct xhci_cap_regs {
	__le32                     hc_capbase;           /*     0     4 */
	__le32                     hcs_params1;          /*     4     4 */
	__le32                     hcs_params2;          /*     8     4 */
	__le32                     hcs_params3;          /*    12     4 */
	__le32                     hcc_params;           /*    16     4 */
	__le32                     db_off;               /*    20     4 */
	__le32                     run_regs_off;         /*    24     4 */
	__le32                     hcc_params2;          /*    28     4 */

	/* size: 32, cachelines: 1, members: 8 */
	/* last cacheline: 32 bytes */
};
struct xhci_op_regs {
	__le32                     command;              /*     0     4 */
	__le32                     status;               /*     4     4 */
	__le32                     page_size;            /*     8     4 */
	__le32                     reserved1;            /*    12     4 */
	__le32                     reserved2;            /*    16     4 */
	__le32                     dev_notification;     /*    20     4 */
	__le64                     cmd_ring;             /*    24     8 */
	__le32                     reserved3[4];         /*    32    16 */
	__le64                     dcbaa_ptr;            /*    48     8 */
	__le32                     config_reg;           /*    56     4 */
	__le32                     reserved4[241];       /*    60   964 */
	/* --- cacheline 16 boundary (1024 bytes) --- */
	__le32                     port_status_base;     /*  1024     4 */
	__le32                     port_power_base;      /*  1028     4 */
	__le32                     port_link_base;       /*  1032     4 */
	__le32                     reserved5;            /*  1036     4 */
	__le32                     reserved6[1016];      /*  1040  4064 */

	/* size: 5104, cachelines: 80, members: 16 */
	/* last cacheline: 48 bytes */
};
struct xhci_intr_reg {
	__le32                     irq_pending;          /*     0     4 */
	__le32                     irq_control;          /*     4     4 */
	__le32                     erst_size;            /*     8     4 */
	__le32                     rsvd;                 /*    12     4 */
	__le64                     erst_base;            /*    16     8 */
	__le64                     erst_dequeue;         /*    24     8 */

	/* size: 32, cachelines: 1, members: 6 */
	/* last cacheline: 32 bytes */
};
struct xhci_run_regs {
	__le32                     microframe_index;     /*     0     4 */
	__le32                     rsvd[7];              /*     4    28 */
	struct xhci_intr_reg       ir_set[128];          /*    32  4096 */

	/* size: 4128, cachelines: 65, members: 3 */
	/* last cacheline: 32 bytes */
};
struct xhci_doorbell_array {
	__le32                     doorbell[256];        /*     0  1024 */

	/* size: 1024, cachelines: 16, members: 1 */
};
struct xhci_container_ctx {
	unsigned int               type;                 /*     0     4 */
	int                        size;                 /*     4     4 */
	u8 *                       bytes;                /*     8     8 */
	dma_addr_t                 dma;                  /*    16     8 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct xhci_slot_ctx {
	__le32                     dev_info;             /*     0     4 */
	__le32                     dev_info2;            /*     4     4 */
	__le32                     tt_info;              /*     8     4 */
	__le32                     dev_state;            /*    12     4 */
	__le32                     reserved[4];          /*    16    16 */

	/* size: 32, cachelines: 1, members: 5 */
	/* last cacheline: 32 bytes */
};
struct xhci_ep_ctx {
	__le32                     ep_info;              /*     0     4 */
	__le32                     ep_info2;             /*     4     4 */
	__le64                     deq;                  /*     8     8 */
	__le32                     tx_info;              /*    16     4 */
	__le32                     reserved[3];          /*    20    12 */

	/* size: 32, cachelines: 1, members: 5 */
	/* last cacheline: 32 bytes */
};
struct xhci_input_control_ctx {
	__le32                     drop_flags;           /*     0     4 */
	__le32                     add_flags;            /*     4     4 */
	__le32                     rsvd2[6];             /*     8    24 */

	/* size: 32, cachelines: 1, members: 3 */
	/* last cacheline: 32 bytes */
};
struct xhci_command {
	struct xhci_container_ctx * in_ctx;              /*     0     8 */
	u32                        status;               /*     8     4 */
	int                        slot_id;              /*    12     4 */
	struct completion *        completion;           /*    16     8 */
	union xhci_trb *           command_trb;          /*    24     8 */
	struct list_head           cmd_list;             /*    32    16 */

	/* size: 48, cachelines: 1, members: 6 */
	/* last cacheline: 48 bytes */
};
union xhci_trb {
	struct xhci_link_trb       link;               /*     0    16 */
	struct xhci_transfer_event trans_event;        /*     0    16 */
	struct xhci_event_cmd      event_cmd;          /*     0    16 */
	struct xhci_generic_trb    generic;            /*     0    16 */
};
struct xhci_stream_ctx {
	__le64                     stream_ring;          /*     0     8 */
	__le32                     reserved[2];          /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct xhci_stream_info {
	struct xhci_ring * *       stream_rings;         /*     0     8 */
	unsigned int               num_streams;          /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct xhci_stream_ctx *   stream_ctx_array;     /*    16     8 */
	unsigned int               num_stream_ctxs;      /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	dma_addr_t                 ctx_array_dma;        /*    32     8 */
	struct xarray              trb_address_map;      /*    40    16 */
	struct xhci_command *      free_streams_command; /*    56     8 */

	/* size: 64, cachelines: 1, members: 7 */
	/* sum members: 56, holes: 2, sum holes: 8 */
};
struct xhci_ring {
	struct xhci_segment *      first_seg;            /*     0     8 */
	struct xhci_segment *      last_seg;             /*     8     8 */
	union xhci_trb *           enqueue;              /*    16     8 */
	struct xhci_segment *      enq_seg;              /*    24     8 */
	union xhci_trb *           dequeue;              /*    32     8 */
	struct xhci_segment *      deq_seg;              /*    40     8 */
	struct list_head           td_list;              /*    48    16 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u32                        cycle_state;          /*    64     4 */
	unsigned int               err_count;            /*    68     4 */
	unsigned int               stream_id;            /*    72     4 */
	unsigned int               num_segs;             /*    76     4 */
	unsigned int               num_trbs_free;        /*    80     4 */
	unsigned int               num_trbs_free_temp;   /*    84     4 */
	unsigned int               bounce_buf_len;       /*    88     4 */
	enum xhci_ring_type        type;                 /*    92     4 */
	bool                       last_td_was_short;    /*    96     1 */

	/* XXX 7 bytes hole, try to pack */

	struct xarray *            trb_address_map;      /*   104     8 */

	/* size: 112, cachelines: 2, members: 17 */
	/* sum members: 105, holes: 1, sum holes: 7 */
	/* last cacheline: 48 bytes */
};
struct xhci_bw_info {
	unsigned int               ep_interval;          /*     0     4 */
	unsigned int               mult;                 /*     4     4 */
	unsigned int               num_packets;          /*     8     4 */
	unsigned int               max_packet_size;      /*    12     4 */
	unsigned int               max_esit_payload;     /*    16     4 */
	unsigned int               type;                 /*    20     4 */

	/* size: 24, cachelines: 1, members: 6 */
	/* last cacheline: 24 bytes */
};
struct xhci_virt_ep {
	struct xhci_ring *         ring;                 /*     0     8 */
	struct xhci_stream_info *  stream_info;          /*     8     8 */
	struct xhci_ring *         new_ring;             /*    16     8 */
	unsigned int               ep_state;             /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           cancelled_td_list;    /*    32    16 */
	struct timer_list          stop_cmd_timer;       /*    48    72 */
	/* --- cacheline 1 boundary (64 bytes) was 56 bytes ago --- */
	struct xhci_hcd *          xhci;                 /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct xhci_segment *      queued_deq_seg;       /*   128     8 */
	union xhci_trb *           queued_deq_ptr;       /*   136     8 */
	bool                       skip;                 /*   144     1 */

	/* XXX 3 bytes hole, try to pack */

	struct xhci_bw_info        bw_info;              /*   148    24 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           bw_endpoint_list;     /*   176    16 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	int                        next_frame_id;        /*   192     4 */
	bool                       use_extended_tbc;     /*   196     1 */

	/* size: 200, cachelines: 4, members: 14 */
	/* sum members: 186, holes: 3, sum holes: 11 */
	/* padding: 3 */
	/* last cacheline: 8 bytes */
};
struct xhci_hcd {
	struct usb_hcd *           main_hcd;             /*     0     8 */
	struct usb_hcd *           shared_hcd;           /*     8     8 */
	struct xhci_cap_regs *     cap_regs;             /*    16     8 */
	struct xhci_op_regs *      op_regs;              /*    24     8 */
	struct xhci_run_regs *     run_regs;             /*    32     8 */
	struct xhci_doorbell_array * dba;                /*    40     8 */
	struct xhci_intr_reg *     ir_set;               /*    48     8 */
	__u32                      hcs_params1;          /*    56     4 */
	__u32                      hcs_params2;          /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	__u32                      hcs_params3;          /*    64     4 */
	__u32                      hcc_params;           /*    68     4 */
	__u32                      hcc_params2;          /*    72     4 */
	spinlock_t                 lock;                 /*    76     4 */
	u8                         sbrn;                 /*    80     1 */

	/* XXX 1 byte hole, try to pack */

	u16                        hci_version;          /*    82     2 */
	u8                         max_slots;            /*    84     1 */
	u8                         max_interrupters;     /*    85     1 */
	u8                         max_ports;            /*    86     1 */
	u8                         isoc_threshold;       /*    87     1 */
	u32                        imod_interval;        /*    88     4 */
	int                        event_ring_max;       /*    92     4 */
	int                        page_size;            /*    96     4 */
	int                        page_shift;           /*   100     4 */
	int                        msix_count;           /*   104     4 */

	/* XXX 4 bytes hole, try to pack */

	struct clk *               clk;                  /*   112     8 */
	struct clk *               reg_clk;              /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct reset_control *     reset;                /*   128     8 */
	struct xhci_device_context_array * dcbaa;        /*   136     8 */
	struct xhci_ring *         cmd_ring;             /*   144     8 */
	unsigned int               cmd_ring_state;       /*   152     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           cmd_list;             /*   160    16 */
	unsigned int               cmd_ring_reserved_trbs; /*   176     4 */

	/* XXX 4 bytes hole, try to pack */

	struct delayed_work        cmd_timer;            /*   184   192 */
	/* --- cacheline 5 boundary (320 bytes) was 56 bytes ago --- */
	struct completion          cmd_ring_stop_completion; /*   376    32 */
	/* --- cacheline 6 boundary (384 bytes) was 24 bytes ago --- */
	struct xhci_command *      current_cmd;          /*   408     8 */
	struct xhci_ring *         event_ring;           /*   416     8 */
	struct xhci_erst           erst;                 /*   424    32 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 7 boundary (448 bytes) was 8 bytes ago --- */
	struct xhci_scratchpad *   scratchpad;           /*   456     8 */
	struct list_head           lpm_failed_devs;      /*   464    16 */
	struct mutex               mutex;                /*   480    32 */
	/* --- cacheline 8 boundary (512 bytes) --- */
	struct xhci_command *      lpm_command;          /*   512     8 */
	struct xhci_virt_device *  devs[256];            /*   520  2048 */
	/* --- cacheline 40 boundary (2560 bytes) was 8 bytes ago --- */
	struct xhci_root_port_bw_info * rh_bw;           /*  2568     8 */
	struct dma_pool *          device_pool;          /*  2576     8 */
	struct dma_pool *          segment_pool;         /*  2584     8 */
	struct dma_pool *          small_streams_pool;   /*  2592     8 */
	struct dma_pool *          medium_streams_pool;  /*  2600     8 */
	unsigned int               xhc_state;            /*  2608     4 */
	u32                        command;              /*  2612     4 */
	struct s3_save             s3;                   /*  2616    48 */
	/* --- cacheline 41 boundary (2624 bytes) was 40 bytes ago --- */
	long long unsigned int     quirks;               /*  2664     8 */
	unsigned int               num_active_eps;       /*  2672     4 */
	unsigned int               limit_active_eps;     /*  2676     4 */
	struct xhci_port *         hw_ports;             /*  2680     8 */
	/* --- cacheline 42 boundary (2688 bytes) --- */
	struct xhci_hub            usb2_rhub;            /*  2688  2312 */

	/* XXX last struct has 6 bytes of padding */

	/* --- cacheline 78 boundary (4992 bytes) was 8 bytes ago --- */
	struct xhci_hub            usb3_rhub;            /*  5000  2312 */

	/* XXX last struct has 6 bytes of padding */

	/* --- cacheline 114 boundary (7296 bytes) was 16 bytes ago --- */
	unsigned int               hw_lpm_support:1;     /*  7312: 0  4 */
	unsigned int               broken_suspend:1;     /*  7312: 1  4 */

	/* XXX 30 bits hole, try to pack */
	/* XXX 4 bytes hole, try to pack */

	u32 *                      ext_caps;             /*  7320     8 */
	unsigned int               num_ext_caps;         /*  7328     4 */

	/* XXX 4 bytes hole, try to pack */

	struct xhci_port_cap *     port_caps;            /*  7336     8 */
	unsigned int               num_port_caps;        /*  7344     4 */

	/* XXX 4 bytes hole, try to pack */

	struct timer_list          comp_mode_recovery_timer; /*  7352    72 */
	/* --- cacheline 116 boundary (7424 bytes) --- */
	u32                        port_status_u0;       /*  7424     4 */
	u16                        test_mode;            /*  7428     2 */

	/* XXX 2 bytes hole, try to pack */

	struct dentry *            debugfs_root;         /*  7432     8 */
	struct dentry *            debugfs_slots;        /*  7440     8 */
	struct list_head           regset_list;          /*  7448    16 */
	void *                     dbc;                  /*  7464     8 */
	long unsigned int          priv[] __attribute__((__aligned__(8))); /*  7472     0 */

	/* size: 7472, cachelines: 117, members: 70 */
	/* sum members: 7441, holes: 8, sum holes: 27 */
	/* sum bitfield members: 2 bits, bit holes: 1, sum bit holes: 30 bits */
	/* paddings: 3, sum paddings: 16 */
	/* forced alignments: 1 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct xhci_segment {
	union xhci_trb *           trbs;                 /*     0     8 */
	struct xhci_segment *      next;                 /*     8     8 */
	dma_addr_t                 dma;                  /*    16     8 */
	dma_addr_t                 bounce_dma;           /*    24     8 */
	void *                     bounce_buf;           /*    32     8 */
	unsigned int               bounce_offs;          /*    40     4 */
	unsigned int               bounce_len;           /*    44     4 */

	/* size: 48, cachelines: 1, members: 7 */
	/* last cacheline: 48 bytes */
};
struct xhci_interval_bw {
	unsigned int               num_packets;          /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           endpoints;            /*     8    16 */
	unsigned int               overhead[3];          /*    24    12 */

	/* size: 40, cachelines: 1, members: 3 */
	/* sum members: 32, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct xhci_interval_bw_table {
	unsigned int               interval0_esit_payload; /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct xhci_interval_bw    interval_bw[16];      /*     8   640 */
	/* --- cacheline 10 boundary (640 bytes) was 8 bytes ago --- */
	unsigned int               bw_used;              /*   648     4 */
	unsigned int               ss_bw_in;             /*   652     4 */
	unsigned int               ss_bw_out;            /*   656     4 */

	/* size: 664, cachelines: 11, members: 5 */
	/* sum members: 656, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct xhci_virt_device {
	struct usb_device *        udev;                 /*     0     8 */
	struct xhci_container_ctx * out_ctx;             /*     8     8 */
	struct xhci_container_ctx * in_ctx;              /*    16     8 */
	struct xhci_virt_ep        eps[31];              /*    24  6200 */
	/* --- cacheline 97 boundary (6208 bytes) was 16 bytes ago --- */
	u8                         fake_port;            /*  6224     1 */
	u8                         real_port;            /*  6225     1 */

	/* XXX 6 bytes hole, try to pack */

	struct xhci_interval_bw_table * bw_table;        /*  6232     8 */
	struct xhci_tt_bw_info *   tt_info;              /*  6240     8 */
	long unsigned int          flags;                /*  6248     8 */
	u16                        current_mel;          /*  6256     2 */

	/* XXX 6 bytes hole, try to pack */

	void *                     debugfs_private;      /*  6264     8 */

	/* size: 6272, cachelines: 98, members: 11 */
	/* sum members: 6260, holes: 2, sum holes: 12 */
};
struct xhci_tt_bw_info {
	struct list_head           tt_list;              /*     0    16 */
	int                        slot_id;              /*    16     4 */
	int                        ttport;               /*    20     4 */
	struct xhci_interval_bw_table bw_table;          /*    24   664 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 10 boundary (640 bytes) was 48 bytes ago --- */
	int                        active_eps;           /*   688     4 */

	/* size: 696, cachelines: 11, members: 5 */
	/* padding: 4 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 56 bytes */
};
struct xhci_root_port_bw_info {
	struct list_head           tts;                  /*     0    16 */
	unsigned int               num_active_tts;       /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	struct xhci_interval_bw_table bw_table;          /*    24   664 */

	/* XXX last struct has 4 bytes of padding */

	/* size: 688, cachelines: 11, members: 3 */
	/* sum members: 684, holes: 1, sum holes: 4 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 48 bytes */
};
struct xhci_device_context_array {
	__le64                     dev_context_ptrs[256]; /*     0  2048 */
	/* --- cacheline 32 boundary (2048 bytes) --- */
	dma_addr_t                 dma;                  /*  2048     8 */

	/* size: 2056, cachelines: 33, members: 2 */
	/* last cacheline: 8 bytes */
};
struct xhci_transfer_event {
	__le64                     buffer;               /*     0     8 */
	__le32                     transfer_len;         /*     8     4 */
	__le32                     flags;                /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct xhci_link_trb {
	__le64                     segment_ptr;          /*     0     8 */
	__le32                     intr_target;          /*     8     4 */
	__le32                     control;              /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct xhci_event_cmd {
	__le64                     cmd_trb;              /*     0     8 */
	__le32                     status;               /*     8     4 */
	__le32                     flags;                /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct xhci_generic_trb {
	__le32                     field[4];             /*     0    16 */

	/* size: 16, cachelines: 1, members: 1 */
	/* last cacheline: 16 bytes */
};
struct xhci_td {
	struct list_head           td_list;              /*     0    16 */
	struct list_head           cancelled_td_list;    /*    16    16 */
	struct urb *               urb;                  /*    32     8 */
	struct xhci_segment *      start_seg;            /*    40     8 */
	union xhci_trb *           first_trb;            /*    48     8 */
	union xhci_trb *           last_trb;             /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct xhci_segment *      bounce_seg;           /*    64     8 */
	bool                       urb_length_set;       /*    72     1 */

	/* size: 80, cachelines: 2, members: 8 */
	/* padding: 7 */
	/* last cacheline: 16 bytes */
};
struct xhci_dequeue_state {
	struct xhci_segment *      new_deq_seg;          /*     0     8 */
	union xhci_trb *           new_deq_ptr;          /*     8     8 */
	int                        new_cycle_state;      /*    16     4 */
	unsigned int               stream_id;            /*    20     4 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct xhci_erst_entry {
	__le64                     seg_addr;             /*     0     8 */
	__le32                     seg_size;             /*     8     4 */
	__le32                     rsvd;                 /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct xhci_erst {
	struct xhci_erst_entry *   entries;              /*     0     8 */
	unsigned int               num_entries;          /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	dma_addr_t                 erst_dma_addr;        /*    16     8 */
	unsigned int               erst_size;            /*    24     4 */

	/* size: 32, cachelines: 1, members: 4 */
	/* sum members: 24, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct xhci_scratchpad {
	u64 *                      sp_array;             /*     0     8 */
	dma_addr_t                 sp_dma;               /*     8     8 */
	void * *                   sp_buffers;           /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct urb_priv {
	int                        num_tds;              /*     0     4 */
	int                        num_tds_done;         /*     4     4 */
	struct xhci_td             td[];                 /*     8     0 */

	/* size: 8, cachelines: 1, members: 3 */
	/* last cacheline: 8 bytes */
};
struct s3_save {
	u32                        command;              /*     0     4 */
	u32                        dev_nt;               /*     4     4 */
	u64                        dcbaa_ptr;            /*     8     8 */
	u32                        config_reg;           /*    16     4 */
	u32                        irq_pending;          /*    20     4 */
	u32                        irq_control;          /*    24     4 */
	u32                        erst_size;            /*    28     4 */
	u64                        erst_base;            /*    32     8 */
	u64                        erst_dequeue;         /*    40     8 */

	/* size: 48, cachelines: 1, members: 9 */
	/* last cacheline: 48 bytes */
};
struct xhci_bus_state {
	long unsigned int          bus_suspended;        /*     0     8 */
	long unsigned int          next_statechange;     /*     8     8 */
	u32                        port_c_suspend;       /*    16     4 */
	u32                        suspended_ports;      /*    20     4 */
	u32                        port_remote_wakeup;   /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          resume_done[31];      /*    32   248 */
	/* --- cacheline 4 boundary (256 bytes) was 24 bytes ago --- */
	long unsigned int          resuming_ports;       /*   280     8 */
	long unsigned int          rexit_ports;          /*   288     8 */
	struct completion          rexit_done[31];       /*   296   992 */
	/* --- cacheline 20 boundary (1280 bytes) was 8 bytes ago --- */
	struct completion          u3exit_done[31];      /*  1288   992 */

	/* size: 2280, cachelines: 36, members: 10 */
	/* sum members: 2276, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct xhci_port_cap {
	u32 *                      psi;                  /*     0     8 */
	u8                         psi_count;            /*     8     1 */
	u8                         psi_uid_count;        /*     9     1 */
	u8                         maj_rev;              /*    10     1 */
	u8                         min_rev;              /*    11     1 */

	/* size: 16, cachelines: 1, members: 5 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct xhci_port {
	__le32 *                   addr;                 /*     0     8 */
	int                        hw_portnum;           /*     8     4 */
	int                        hcd_portnum;          /*    12     4 */
	struct xhci_hub *          rhub;                 /*    16     8 */
	struct xhci_port_cap *     port_cap;             /*    24     8 */

	/* size: 32, cachelines: 1, members: 5 */
	/* last cacheline: 32 bytes */
};
struct xhci_hub {
	struct xhci_port * *       ports;                /*     0     8 */
	unsigned int               num_ports;            /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct usb_hcd *           hcd;                  /*    16     8 */
	struct xhci_bus_state      bus_state;            /*    24  2280 */
	/* --- cacheline 36 boundary (2304 bytes) --- */
	u8                         maj_rev;              /*  2304     1 */
	u8                         min_rev;              /*  2305     1 */

	/* size: 2312, cachelines: 37, members: 6 */
	/* sum members: 2302, holes: 1, sum holes: 4 */
	/* padding: 6 */
	/* last cacheline: 8 bytes */
};
struct xhci_driver_overrides {
	size_t                     extra_priv_size;      /*     0     8 */
	int                        (*reset)(struct usb_hcd *); /*     8     8 */
	int                        (*start)(struct usb_hcd *); /*    16     8 */
	int                        (*check_bandwidth)(struct usb_hcd *, struct usb_device *); /*    24     8 */
	void                       (*reset_bandwidth)(struct usb_hcd *, struct usb_device *); /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct dbc_regs {
	__le32                     capability;           /*     0     4 */
	__le32                     doorbell;             /*     4     4 */
	__le32                     ersts;                /*     8     4 */
	__le32                     __reserved_0;         /*    12     4 */
	__le64                     erstba;               /*    16     8 */
	__le64                     erdp;                 /*    24     8 */
	__le32                     control;              /*    32     4 */
	__le32                     status;               /*    36     4 */
	__le32                     portsc;               /*    40     4 */
	__le32                     __reserved_1;         /*    44     4 */
	__le64                     dccp;                 /*    48     8 */
	__le32                     devinfo1;             /*    56     4 */
	__le32                     devinfo2;             /*    60     4 */

	/* size: 64, cachelines: 1, members: 13 */
};
struct dbc_str_descs {
	char                       string0[64];          /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	char                       manufacturer[64];     /*    64    64 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	char                       product[64];          /*   128    64 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	char                       serial[64];           /*   192    64 */

	/* size: 256, cachelines: 4, members: 4 */
};
struct dbc_ep {
	struct xhci_dbc *          dbc;                  /*     0     8 */
	struct list_head           list_pending;         /*     8    16 */
	struct xhci_ring *         ring;                 /*    24     8 */
	unsigned int               direction:1;          /*    32: 0  4 */

	/* size: 40, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* bit_padding: 31 bits */
	/* last cacheline: 40 bytes */
};
struct xhci_dbc {
	spinlock_t                 lock;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct device *            dev;                  /*     8     8 */
	struct xhci_hcd *          xhci;                 /*    16     8 */
	struct dbc_regs *          regs;                 /*    24     8 */
	struct xhci_ring *         ring_evt;             /*    32     8 */
	struct xhci_ring *         ring_in;              /*    40     8 */
	struct xhci_ring *         ring_out;             /*    48     8 */
	struct xhci_erst           erst;                 /*    56    32 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 1 boundary (64 bytes) was 24 bytes ago --- */
	struct xhci_container_ctx * ctx;                 /*    88     8 */
	struct dbc_str_descs *     string;               /*    96     8 */
	dma_addr_t                 string_dma;           /*   104     8 */
	size_t                     string_size;          /*   112     8 */
	enum dbc_state             state;                /*   120     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 2 boundary (128 bytes) --- */
	struct delayed_work        event_work;           /*   128   192 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	unsigned int               resume_required:1;    /*   320: 0  4 */

	/* XXX 31 bits hole, try to pack */
	/* XXX 4 bytes hole, try to pack */

	struct dbc_ep              eps[2];               /*   328    80 */
	/* --- cacheline 6 boundary (384 bytes) was 24 bytes ago --- */
	const struct dbc_driver  * driver;               /*   408     8 */
	void *                     priv;                 /*   416     8 */

	/* size: 424, cachelines: 7, members: 18 */
	/* sum members: 408, holes: 3, sum holes: 12 */
	/* sum bitfield members: 1 bits, bit holes: 1, sum bit holes: 31 bits */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 40 bytes */
};
struct dbc_driver {
	int                        (*configure)(struct xhci_dbc *); /*     0     8 */
	void                       (*disconnect)(struct xhci_dbc *); /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct dbc_request {
	void *                     buf;                  /*     0     8 */
	unsigned int               length;               /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	dma_addr_t                 dma;                  /*    16     8 */
	void                       (*complete)(struct xhci_dbc *, struct dbc_request *); /*    24     8 */
	struct list_head           list_pool;            /*    32    16 */
	int                        status;               /*    48     4 */
	unsigned int               actual;               /*    52     4 */
	struct xhci_dbc *          dbc;                  /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct list_head           list_pending;         /*    64    16 */
	dma_addr_t                 trb_dma;              /*    80     8 */
	union xhci_trb *           trb;                  /*    88     8 */
	unsigned int               direction:1;          /*    96: 0  4 */

	/* size: 104, cachelines: 2, members: 12 */
	/* sum members: 92, holes: 1, sum holes: 4 */
	/* sum bitfield members: 1 bits (0 bytes) */
	/* padding: 4 */
	/* bit_padding: 31 bits */
	/* last cacheline: 40 bytes */
};
struct trace_event_raw_xhci_log_msg {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        __data_loc_msg;       /*     8     4 */
	char                       __data[];             /*    12     0 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct trace_event_raw_xhci_log_ctx {
	struct trace_entry         ent;                  /*     0     8 */
	int                        ctx_64;               /*     8     4 */
	unsigned int               ctx_type;             /*    12     4 */
	dma_addr_t                 ctx_dma;              /*    16     8 */
	u8 *                       ctx_va;               /*    24     8 */
	unsigned int               ctx_ep_num;           /*    32     4 */
	int                        slot_id;              /*    36     4 */
	u32                        __data_loc_ctx_data;  /*    40     4 */
	char                       __data[];             /*    44     0 */

	/* size: 48, cachelines: 1, members: 9 */
	/* padding: 4 */
	/* last cacheline: 48 bytes */
};
struct trace_event_raw_xhci_log_trb {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        type;                 /*     8     4 */
	u32                        field0;               /*    12     4 */
	u32                        field1;               /*    16     4 */
	u32                        field2;               /*    20     4 */
	u32                        field3;               /*    24     4 */
	u32                        __data_loc_str;       /*    28     4 */
	char                       __data[];             /*    32     0 */

	/* size: 32, cachelines: 1, members: 8 */
	/* last cacheline: 32 bytes */
};
struct trace_event_raw_xhci_log_free_virt_dev {
	struct trace_entry         ent;                  /*     0     8 */
	void *                     vdev;                 /*     8     8 */
	long long unsigned int     out_ctx;              /*    16     8 */
	long long unsigned int     in_ctx;               /*    24     8 */
	u8                         fake_port;            /*    32     1 */
	u8                         real_port;            /*    33     1 */
	u16                        current_mel;          /*    34     2 */
	char                       __data[];             /*    36     0 */

	/* size: 40, cachelines: 1, members: 8 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct trace_event_raw_xhci_log_virt_dev {
	struct trace_entry         ent;                  /*     0     8 */
	void *                     vdev;                 /*     8     8 */
	long long unsigned int     out_ctx;              /*    16     8 */
	long long unsigned int     in_ctx;               /*    24     8 */
	int                        devnum;               /*    32     4 */
	int                        state;                /*    36     4 */
	int                        speed;                /*    40     4 */
	u8                         portnum;              /*    44     1 */
	u8                         level;                /*    45     1 */

	/* XXX 2 bytes hole, try to pack */

	int                        slot_id;              /*    48     4 */
	char                       __data[];             /*    52     0 */

	/* size: 56, cachelines: 1, members: 11 */
	/* sum members: 50, holes: 1, sum holes: 2 */
	/* padding: 4 */
	/* last cacheline: 56 bytes */
};
struct trace_event_raw_xhci_log_urb {
	struct trace_entry         ent;                  /*     0     8 */
	void *                     urb;                  /*     8     8 */
	unsigned int               pipe;                 /*    16     4 */
	unsigned int               stream;               /*    20     4 */
	int                        status;               /*    24     4 */
	unsigned int               flags;                /*    28     4 */
	int                        num_mapped_sgs;       /*    32     4 */
	int                        num_sgs;              /*    36     4 */
	int                        length;               /*    40     4 */
	int                        actual;               /*    44     4 */
	int                        epnum;                /*    48     4 */
	int                        dir_in;               /*    52     4 */
	int                        type;                 /*    56     4 */
	int                        slot_id;              /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	char                       __data[];             /*    64     0 */

	/* size: 64, cachelines: 1, members: 15 */
};
struct trace_event_raw_xhci_log_ep_ctx {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        info;                 /*     8     4 */
	u32                        info2;                /*    12     4 */
	u64                        deq;                  /*    16     8 */
	u32                        tx_info;              /*    24     4 */
	u32                        __data_loc_str;       /*    28     4 */
	char                       __data[];             /*    32     0 */

	/* size: 32, cachelines: 1, members: 7 */
	/* last cacheline: 32 bytes */
};
struct trace_event_raw_xhci_log_slot_ctx {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        info;                 /*     8     4 */
	u32                        info2;                /*    12     4 */
	u32                        tt_info;              /*    16     4 */
	u32                        state;                /*    20     4 */
	u32                        __data_loc_str;       /*    24     4 */
	char                       __data[];             /*    28     0 */

	/* size: 28, cachelines: 1, members: 7 */
	/* last cacheline: 28 bytes */
};
struct trace_event_raw_xhci_log_ctrl_ctx {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        drop;                 /*     8     4 */
	u32                        add;                  /*    12     4 */
	u32                        __data_loc_str;       /*    16     4 */
	char                       __data[];             /*    20     0 */

	/* size: 20, cachelines: 1, members: 5 */
	/* last cacheline: 20 bytes */
};
struct trace_event_raw_xhci_log_ring {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        type;                 /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     ring;                 /*    16     8 */
	dma_addr_t                 enq;                  /*    24     8 */
	dma_addr_t                 deq;                  /*    32     8 */
	dma_addr_t                 enq_seg;              /*    40     8 */
	dma_addr_t                 deq_seg;              /*    48     8 */
	unsigned int               num_segs;             /*    56     4 */
	unsigned int               stream_id;            /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	unsigned int               cycle_state;          /*    64     4 */
	unsigned int               num_trbs_free;        /*    68     4 */
	unsigned int               bounce_buf_len;       /*    72     4 */
	char                       __data[];             /*    76     0 */

	/* size: 80, cachelines: 2, members: 13 */
	/* sum members: 72, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct trace_event_raw_xhci_log_portsc {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        portnum;              /*     8     4 */
	u32                        portsc;               /*    12     4 */
	u32                        __data_loc_str;       /*    16     4 */
	char                       __data[];             /*    20     0 */

	/* size: 20, cachelines: 1, members: 5 */
	/* last cacheline: 20 bytes */
};
struct trace_event_raw_xhci_log_doorbell {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        slot;                 /*     8     4 */
	u32                        doorbell;             /*    12     4 */
	u32                        __data_loc_str;       /*    16     4 */
	char                       __data[];             /*    20     0 */

	/* size: 20, cachelines: 1, members: 5 */
	/* last cacheline: 20 bytes */
};
struct trace_event_raw_xhci_dbc_log_request {
	struct trace_entry         ent;                  /*     0     8 */
	struct dbc_request *       req;                  /*     8     8 */
	bool                       dir;                  /*    16     1 */

	/* XXX 3 bytes hole, try to pack */

	unsigned int               actual;               /*    20     4 */
	unsigned int               length;               /*    24     4 */
	int                        status;               /*    28     4 */
	char                       __data[];             /*    32     0 */

	/* size: 32, cachelines: 1, members: 7 */
	/* sum members: 29, holes: 1, sum holes: 3 */
	/* last cacheline: 32 bytes */
};
struct trace_event_data_offsets_xhci_log_msg {
	u32                        msg;                  /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct trace_event_data_offsets_xhci_log_ctx {
	u32                        ctx_data;             /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct trace_event_data_offsets_xhci_log_trb {
	u32                        str;                  /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct trace_event_data_offsets_xhci_log_free_virt_dev {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_xhci_log_virt_dev {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_xhci_log_urb {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_xhci_log_ep_ctx {
	u32                        str;                  /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct trace_event_data_offsets_xhci_log_slot_ctx {
	u32                        str;                  /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct trace_event_data_offsets_xhci_log_ctrl_ctx {
	u32                        str;                  /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct trace_event_data_offsets_xhci_log_ring {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_xhci_log_portsc {
	u32                        str;                  /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct trace_event_data_offsets_xhci_log_doorbell {
	u32                        str;                  /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct trace_event_data_offsets_xhci_dbc_log_request {

	/* size: 0, cachelines: 0, members: 0 */
};
struct usb_string_descriptor {
	__u8                       bLength;              /*     0     1 */
	__u8                       bDescriptorType;      /*     1     1 */
	__le16                     wData[1];             /*     2     2 */

	/* size: 4, cachelines: 1, members: 3 */
	/* last cacheline: 4 bytes */
};
struct dbc_info_context {
	__le64                     string0;              /*     0     8 */
	__le64                     manufacturer;         /*     8     8 */
	__le64                     product;              /*    16     8 */
	__le64                     serial;               /*    24     8 */
	__le32                     length;               /*    32     4 */
	__le32                     __reserved_0[7];      /*    36    28 */

	/* size: 64, cachelines: 1, members: 6 */
};
struct kfifo {
	union {
		struct __kfifo     kfifo;                /*     0    24 */
		unsigned char *    type;                 /*     0     8 */
		const unsigned char  * const_type;       /*     0     8 */
		char *             rectype;              /*     0     8 */
		void *             ptr;                  /*     0     8 */
		const void  *      ptr_const;            /*     0     8 */
	};                                               /*     0    24 */
	unsigned char              buf[];                /*    24     0 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct dbc_port {
	struct tty_port            port;                 /*     0   384 */
	/* --- cacheline 6 boundary (384 bytes) --- */
	spinlock_t                 port_lock;            /*   384     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           read_pool;            /*   392    16 */
	struct list_head           read_queue;           /*   408    16 */
	unsigned int               n_read;               /*   424     4 */

	/* XXX 4 bytes hole, try to pack */

	struct tasklet_struct      push;                 /*   432    48 */
	/* --- cacheline 7 boundary (448 bytes) was 32 bytes ago --- */
	struct list_head           write_pool;           /*   480    16 */
	struct kfifo               write_fifo;           /*   496    24 */
	/* --- cacheline 8 boundary (512 bytes) was 8 bytes ago --- */
	bool                       registered;           /*   520     1 */

	/* size: 528, cachelines: 9, members: 9 */
	/* sum members: 513, holes: 2, sum holes: 8 */
	/* padding: 7 */
	/* last cacheline: 16 bytes */
};
struct xhci_regset {
	char                       name[32];             /*     0    32 */
	struct debugfs_regset32    regset;               /*    32    32 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	size_t                     nregs;                /*    64     8 */
	struct list_head           list;                 /*    72    16 */

	/* size: 88, cachelines: 2, members: 4 */
	/* last cacheline: 24 bytes */
};
struct xhci_file_map {
	const char  *              name;                 /*     0     8 */
	int                        (*show)(struct seq_file *, void *); /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct xhci_ep_priv {
	char                       name[32];             /*     0    32 */
	struct dentry *            root;                 /*    32     8 */
	struct xhci_stream_info *  stream_info;          /*    40     8 */
	struct xhci_ring *         show_ring;            /*    48     8 */
	unsigned int               stream_id;            /*    56     4 */

	/* size: 64, cachelines: 1, members: 5 */
	/* padding: 4 */
};
struct xhci_slot_priv {
	char                       name[32];             /*     0    32 */
	struct dentry *            root;                 /*    32     8 */
	struct xhci_ep_priv *      eps[31];              /*    40   248 */
	/* --- cacheline 4 boundary (256 bytes) was 32 bytes ago --- */
	struct xhci_virt_device *  dev;                  /*   288     8 */

	/* size: 296, cachelines: 5, members: 4 */
	/* last cacheline: 40 bytes */
};
struct xhci_driver_data {
	u64                        quirks;               /*     0     8 */
	const char  *              firmware;             /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct usb_debug_descriptor {
	__u8                       bLength;              /*     0     1 */
	__u8                       bDescriptorType;      /*     1     1 */
	__u8                       bDebugInEndpoint;     /*     2     1 */
	__u8                       bDebugOutEndpoint;    /*     3     1 */

	/* size: 4, cachelines: 1, members: 4 */
	/* last cacheline: 4 bytes */
};
struct ehci_dev {
	u32                        bus;                  /*     0     4 */
	u32                        slot;                 /*     4     4 */
	u32                        func;                 /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct xdbc_regs {
	__le32                     capability;           /*     0     4 */
	__le32                     doorbell;             /*     4     4 */
	__le32                     ersts;                /*     8     4 */
	__le32                     __reserved_0;         /*    12     4 */
	__le64                     erstba;               /*    16     8 */
	__le64                     erdp;                 /*    24     8 */
	__le32                     control;              /*    32     4 */
	__le32                     status;               /*    36     4 */
	__le32                     portsc;               /*    40     4 */
	__le32                     __reserved_1;         /*    44     4 */
	__le64                     dccp;                 /*    48     8 */
	__le32                     devinfo1;             /*    56     4 */
	__le32                     devinfo2;             /*    60     4 */

	/* size: 64, cachelines: 1, members: 13 */
};
struct xdbc_trb {
	__le32                     field[4];             /*     0    16 */

	/* size: 16, cachelines: 1, members: 1 */
	/* last cacheline: 16 bytes */
};
struct xdbc_erst_entry {
	__le64                     seg_addr;             /*     0     8 */
	__le32                     seg_size;             /*     8     4 */
	__le32                     __reserved_0;         /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct xdbc_info_context {
	__le64                     string0;              /*     0     8 */
	__le64                     manufacturer;         /*     8     8 */
	__le64                     product;              /*    16     8 */
	__le64                     serial;               /*    24     8 */
	__le32                     length;               /*    32     4 */
	__le32                     __reserved_0[7];      /*    36    28 */

	/* size: 64, cachelines: 1, members: 6 */
};
struct xdbc_ep_context {
	__le32                     ep_info1;             /*     0     4 */
	__le32                     ep_info2;             /*     4     4 */
	__le64                     deq;                  /*     8     8 */
	__le32                     tx_info;              /*    16     4 */
	__le32                     __reserved_0[11];     /*    20    44 */

	/* size: 64, cachelines: 1, members: 5 */
};
struct xdbc_context {
	struct xdbc_info_context   info;                 /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct xdbc_ep_context     out;                  /*    64    64 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct xdbc_ep_context     in;                   /*   128    64 */

	/* size: 192, cachelines: 3, members: 3 */
};
struct xdbc_strings {
	char                       string0[64];          /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	char                       manufacturer[64];     /*    64    64 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	char                       product[64];          /*   128    64 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	char                       serial[64];           /*   192    64 */

	/* size: 256, cachelines: 4, members: 4 */
};
struct xdbc_segment {
	struct xdbc_trb *          trbs;                 /*     0     8 */
	dma_addr_t                 dma;                  /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct xdbc_ring {
	struct xdbc_segment *      segment;              /*     0     8 */
	struct xdbc_trb *          enqueue;              /*     8     8 */
	struct xdbc_trb *          dequeue;              /*    16     8 */
	u32                        cycle_state;          /*    24     4 */

	/* size: 32, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct xdbc_state {
	u16                        vendor;               /*     0     2 */
	u16                        device;               /*     2     2 */
	u32                        bus;                  /*     4     4 */
	u32                        dev;                  /*     8     4 */
	u32                        func;                 /*    12     4 */
	void *                     xhci_base;            /*    16     8 */
	u64                        xhci_start;           /*    24     8 */
	size_t                     xhci_length;          /*    32     8 */
	int                        port_number;          /*    40     4 */

	/* XXX 4 bytes hole, try to pack */

	struct xdbc_regs *         xdbc_reg;             /*    48     8 */
	dma_addr_t                 table_dma;            /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	void *                     table_base;           /*    64     8 */
	dma_addr_t                 erst_dma;             /*    72     8 */
	size_t                     erst_size;            /*    80     8 */
	void *                     erst_base;            /*    88     8 */
	struct xdbc_ring           evt_ring;             /*    96    32 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 2 boundary (128 bytes) --- */
	struct xdbc_segment        evt_seg;              /*   128    16 */
	dma_addr_t                 dbcc_dma;             /*   144     8 */
	size_t                     dbcc_size;            /*   152     8 */
	void *                     dbcc_base;            /*   160     8 */
	dma_addr_t                 string_dma;           /*   168     8 */
	size_t                     string_size;          /*   176     8 */
	void *                     string_base;          /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	struct xdbc_ring           out_ring;             /*   192    32 */

	/* XXX last struct has 4 bytes of padding */

	struct xdbc_segment        out_seg;              /*   224    16 */
	void *                     out_buf;              /*   240     8 */
	dma_addr_t                 out_dma;              /*   248     8 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	struct xdbc_ring           in_ring;              /*   256    32 */

	/* XXX last struct has 4 bytes of padding */

	struct xdbc_segment        in_seg;               /*   288    16 */
	void *                     in_buf;               /*   304     8 */
	dma_addr_t                 in_dma;               /*   312     8 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	u32                        flags;                /*   320     4 */
	raw_spinlock_t             lock;                 /*   324     4 */

	/* size: 328, cachelines: 6, members: 33 */
	/* sum members: 324, holes: 1, sum holes: 4 */
	/* paddings: 3, sum paddings: 12 */
	/* last cacheline: 8 bytes */
};
struct usb_role_switch {
	struct device              dev __attribute__((__aligned__(8))); /*     0  1000 */
	/* --- cacheline 15 boundary (960 bytes) was 40 bytes ago --- */
	struct mutex               lock;                 /*  1000    32 */
	/* --- cacheline 16 boundary (1024 bytes) was 8 bytes ago --- */
	enum usb_role              role;                 /*  1032     4 */

	/* XXX 4 bytes hole, try to pack */

	struct device *            usb2_port;            /*  1040     8 */
	struct device *            usb3_port;            /*  1048     8 */
	struct device *            udc;                  /*  1056     8 */
	usb_role_switch_set_t      set;                  /*  1064     8 */
	usb_role_switch_get_t      get;                  /*  1072     8 */
	bool                       allow_userspace_control; /*  1080     1 */

	/* size: 1088, cachelines: 17, members: 9 */
	/* sum members: 1077, holes: 1, sum holes: 4 */
	/* padding: 7 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(8)));
struct usb_role_switch_desc {
	struct fwnode_handle *     fwnode;               /*     0     8 */
	struct device *            usb2_port;            /*     8     8 */
	struct device *            usb3_port;            /*    16     8 */
	struct device *            udc;                  /*    24     8 */
	usb_role_switch_set_t      set;                  /*    32     8 */
	usb_role_switch_get_t      get;                  /*    40     8 */
	bool                       allow_userspace_control; /*    48     1 */

	/* XXX 7 bytes hole, try to pack */

	void *                     driver_data;          /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	const char  *              name;                 /*    64     8 */

	/* size: 72, cachelines: 2, members: 9 */
	/* sum members: 65, holes: 1, sum holes: 7 */
	/* last cacheline: 8 bytes */
};
struct intel_xhci_usb_data {
	struct device *            dev;                  /*     0     8 */
	struct usb_role_switch *   role_sw;              /*     8     8 */
	void *                     base;                 /*    16     8 */
	bool                       enable_sw_switch;     /*    24     1 */

	/* size: 32, cachelines: 1, members: 4 */
	/* padding: 7 */
	/* last cacheline: 32 bytes */
};
struct serio_device_id {
	__u8                       type;                 /*     0     1 */
	__u8                       extra;                /*     1     1 */
	__u8                       id;                   /*     2     1 */
	__u8                       proto;                /*     3     1 */

	/* size: 4, cachelines: 1, members: 4 */
	/* last cacheline: 4 bytes */
};
struct serio {
	void *                     port_data;            /*     0     8 */
	char                       name[32];             /*     8    32 */
	char                       phys[32];             /*    40    32 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	char                       firmware_id[128];     /*    72   128 */
	/* --- cacheline 3 boundary (192 bytes) was 8 bytes ago --- */
	bool                       manual_bind;          /*   200     1 */
	struct serio_device_id     id;                   /*   201     4 */

	/* XXX 3 bytes hole, try to pack */

	spinlock_t                 lock;                 /*   208     4 */

	/* XXX 4 bytes hole, try to pack */

	int                        (*write)(struct serio *, unsigned char); /*   216     8 */
	int                        (*open)(struct serio *); /*   224     8 */
	void                       (*close)(struct serio *); /*   232     8 */
	int                        (*start)(struct serio *); /*   240     8 */
	void                       (*stop)(struct serio *); /*   248     8 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	struct serio *             parent;               /*   256     8 */
	struct list_head           child_node;           /*   264    16 */
	struct list_head           children;             /*   280    16 */
	unsigned int               depth;                /*   296     4 */

	/* XXX 4 bytes hole, try to pack */

	struct serio_driver *      drv;                  /*   304     8 */
	struct mutex               drv_mutex;            /*   312    32 */
	/* --- cacheline 5 boundary (320 bytes) was 24 bytes ago --- */
	struct device              dev __attribute__((__aligned__(8))); /*   344  1000 */
	/* --- cacheline 21 boundary (1344 bytes) --- */
	struct list_head           node;                 /*  1344    16 */
	struct mutex *             ps2_cmd_mutex;        /*  1360     8 */

	/* size: 1368, cachelines: 22, members: 21 */
	/* sum members: 1357, holes: 3, sum holes: 11 */
	/* forced alignments: 1 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct serio_driver {
	const char  *              description;          /*     0     8 */
	const struct serio_device_id  * id_table;        /*     8     8 */
	bool                       manual_bind;          /*    16     1 */

	/* XXX 7 bytes hole, try to pack */

	void                       (*write_wakeup)(struct serio *); /*    24     8 */
	irqreturn_t                (*interrupt)(struct serio *, unsigned char, unsigned int); /*    32     8 */
	int                        (*connect)(struct serio *, struct serio_driver *); /*    40     8 */
	int                        (*reconnect)(struct serio *); /*    48     8 */
	int                        (*fast_reconnect)(struct serio *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	void                       (*disconnect)(struct serio *); /*    64     8 */
	void                       (*cleanup)(struct serio *); /*    72     8 */
	struct device_driver       driver;               /*    80   192 */

	/* size: 272, cachelines: 5, members: 11 */
	/* sum members: 265, holes: 1, sum holes: 7 */
	/* last cacheline: 16 bytes */
};
struct serio_event {
	enum serio_event_type      type;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     object;               /*     8     8 */
	struct module *            owner;                /*    16     8 */
	struct list_head           node;                 /*    24    16 */

	/* size: 40, cachelines: 1, members: 4 */
	/* sum members: 36, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct i8042_port {
	struct serio *             serio;                /*     0     8 */
	int                        irq;                  /*     8     4 */
	bool                       exists;               /*    12     1 */
	bool                       driver_bound;         /*    13     1 */
	signed char                mux;                  /*    14     1 */

	/* size: 16, cachelines: 1, members: 5 */
	/* padding: 1 */
	/* last cacheline: 16 bytes */
};
struct serport {
	struct tty_struct *        tty;                  /*     0     8 */
	wait_queue_head_t          wait;                 /*     8    24 */
	struct serio *             serio;                /*    32     8 */
	struct serio_device_id     id;                   /*    40     4 */
	spinlock_t                 lock;                 /*    44     4 */
	long unsigned int          flags;                /*    48     8 */

	/* size: 56, cachelines: 1, members: 6 */
	/* last cacheline: 56 bytes */
};
struct ps2dev {
	struct serio *             serio;                /*     0     8 */
	struct mutex               cmd_mutex;            /*     8    32 */
	wait_queue_head_t          wait;                 /*    40    24 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	long unsigned int          flags;                /*    64     8 */
	u8                         cmdbuf[8];            /*    72     8 */
	u8                         cmdcnt;               /*    80     1 */
	u8                         nak;                  /*    81     1 */

	/* size: 88, cachelines: 2, members: 7 */
	/* padding: 6 */
	/* last cacheline: 24 bytes */
};
struct input_mt {
	int                        trkid;                /*     0     4 */
	int                        num_slots;            /*     4     4 */
	int                        slot;                 /*     8     4 */
	unsigned int               flags;                /*    12     4 */
	unsigned int               frame;                /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	int *                      red;                  /*    24     8 */
	struct input_mt_slot       slots[];              /*    32     0 */

	/* size: 32, cachelines: 1, members: 7 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct input_mt_slot {
	int                        abs[14];              /*     0    56 */
	unsigned int               frame;                /*    56     4 */
	unsigned int               key;                  /*    60     4 */

	/* size: 64, cachelines: 1, members: 3 */
};
union input_seq_state {
	struct {
		short unsigned int pos;                /*     0     2 */
		bool               mutex_acquired;     /*     2     1 */
	};                                             /*     0     4 */
	void *                     p;                  /*     0     8 */
};
struct input_devres {
	struct input_dev *         input;                /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct input_event {
	__kernel_ulong_t           __sec;                /*     0     8 */
	__kernel_ulong_t           __usec;               /*     8     8 */
	__u16                      type;                 /*    16     2 */
	__u16                      code;                 /*    18     2 */
	__s32                      value;                /*    20     4 */

	/* size: 24, cachelines: 1, members: 5 */
	/* last cacheline: 24 bytes */
};
struct input_event_compat {
	compat_ulong_t             sec;                  /*     0     4 */
	compat_ulong_t             usec;                 /*     4     4 */
	__u16                      type;                 /*     8     2 */
	__u16                      code;                 /*    10     2 */
	__s32                      value;                /*    12     4 */

	/* size: 16, cachelines: 1, members: 5 */
	/* last cacheline: 16 bytes */
};
struct ff_periodic_effect_compat {
	__u16                      waveform;             /*     0     2 */
	__u16                      period;               /*     2     2 */
	__s16                      magnitude;            /*     4     2 */
	__s16                      offset;               /*     6     2 */
	__u16                      phase;                /*     8     2 */
	struct ff_envelope         envelope;             /*    10     8 */

	/* XXX 2 bytes hole, try to pack */

	__u32                      custom_len;           /*    20     4 */
	compat_uptr_t              custom_data;          /*    24     4 */

	/* size: 28, cachelines: 1, members: 8 */
	/* sum members: 26, holes: 1, sum holes: 2 */
	/* last cacheline: 28 bytes */
};
struct ff_effect_compat {
	__u16                      type;                 /*     0     2 */
	__s16                      id;                   /*     2     2 */
	__u16                      direction;            /*     4     2 */
	struct ff_trigger          trigger;              /*     6     4 */
	struct ff_replay           replay;               /*    10     4 */

	/* XXX 2 bytes hole, try to pack */

	union {
		struct ff_constant_effect constant;      /*    16    10 */
		struct ff_ramp_effect ramp;              /*    16    12 */
		struct ff_periodic_effect_compat periodic; /*    16    28 */
		struct ff_condition_effect condition[2]; /*    16    24 */
		struct ff_rumble_effect rumble;          /*    16     4 */
	} u;                                             /*    16    28 */

	/* size: 44, cachelines: 1, members: 6 */
	/* sum members: 42, holes: 1, sum holes: 2 */
	/* last cacheline: 44 bytes */
};
struct input_mt_pos {
	s16                        x;                    /*     0     2 */
	s16                        y;                    /*     2     2 */

	/* size: 4, cachelines: 1, members: 2 */
	/* last cacheline: 4 bytes */
};
struct input_dev_poller {
	void                       (*poll)(struct input_dev *); /*     0     8 */
	unsigned int               poll_interval;        /*     8     4 */
	unsigned int               poll_interval_max;    /*    12     4 */
	unsigned int               poll_interval_min;    /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	struct input_dev *         input;                /*    24     8 */
	struct delayed_work        work;                 /*    32   192 */

	/* size: 224, cachelines: 4, members: 6 */
	/* sum members: 220, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct input_led {
	struct led_classdev        cdev;                 /*     0   416 */
	/* --- cacheline 6 boundary (384 bytes) was 32 bytes ago --- */
	struct input_handle *      handle;               /*   416     8 */
	unsigned int               code;                 /*   424     4 */

	/* size: 432, cachelines: 7, members: 3 */
	/* padding: 4 */
	/* last cacheline: 48 bytes */
};
struct input_leds {
	struct input_handle        handle;               /*     0    72 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	unsigned int               num_leds;             /*    72     4 */

	/* XXX 4 bytes hole, try to pack */

	struct input_led           leds[];               /*    80     0 */

	/* size: 80, cachelines: 2, members: 3 */
	/* sum members: 76, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct mousedev_hw_data {
	int                        dx;                   /*     0     4 */
	int                        dy;                   /*     4     4 */
	int                        dz;                   /*     8     4 */
	int                        x;                    /*    12     4 */
	int                        y;                    /*    16     4 */
	int                        abs_event;            /*    20     4 */
	long unsigned int          buttons;              /*    24     8 */

	/* size: 32, cachelines: 1, members: 7 */
	/* last cacheline: 32 bytes */
};
struct mousedev {
	int                        open;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct input_handle        handle;               /*     8    72 */
	/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
	wait_queue_head_t          wait;                 /*    80    24 */
	struct list_head           client_list;          /*   104    16 */
	spinlock_t                 client_lock;          /*   120     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 2 boundary (128 bytes) --- */
	struct mutex               mutex;                /*   128    32 */
	struct device              dev __attribute__((__aligned__(8))); /*   160  1000 */
	/* --- cacheline 18 boundary (1152 bytes) was 8 bytes ago --- */
	struct cdev                cdev;                 /*  1160   136 */
	/* --- cacheline 20 boundary (1280 bytes) was 16 bytes ago --- */
	bool                       exist;                /*  1296     1 */

	/* XXX 7 bytes hole, try to pack */

	struct list_head           mixdev_node;          /*  1304    16 */
	bool                       opened_by_mixdev;     /*  1320     1 */

	/* XXX 7 bytes hole, try to pack */

	struct mousedev_hw_data    packet;               /*  1328    32 */
	/* --- cacheline 21 boundary (1344 bytes) was 16 bytes ago --- */
	unsigned int               pkt_count;            /*  1360     4 */
	int                        old_x[4];             /*  1364    16 */
	int                        old_y[4];             /*  1380    16 */
	int                        frac_dx;              /*  1396     4 */
	int                        frac_dy;              /*  1400     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 22 boundary (1408 bytes) --- */
	long unsigned int          touch;                /*  1408     8 */
	int                        (*open_device)(struct mousedev *); /*  1416     8 */
	void                       (*close_device)(struct mousedev *); /*  1424     8 */

	/* size: 1432, cachelines: 23, members: 20 */
	/* sum members: 1406, holes: 5, sum holes: 26 */
	/* forced alignments: 1 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct mousedev_motion {
	int                        dx;                   /*     0     4 */
	int                        dy;                   /*     4     4 */
	int                        dz;                   /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          buttons;              /*    16     8 */

	/* size: 24, cachelines: 1, members: 4 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct mousedev_client {
	struct fasync_struct *     fasync;               /*     0     8 */
	struct mousedev *          mousedev;             /*     8     8 */
	struct list_head           node;                 /*    16    16 */
	struct mousedev_motion     packets[16];          /*    32   384 */
	/* --- cacheline 6 boundary (384 bytes) was 32 bytes ago --- */
	unsigned int               head;                 /*   416     4 */
	unsigned int               tail;                 /*   420     4 */
	spinlock_t                 packet_lock;          /*   424     4 */
	int                        pos_x;                /*   428     4 */
	int                        pos_y;                /*   432     4 */
	u8                         ps2[6];               /*   436     6 */
	unsigned char              ready;                /*   442     1 */
	unsigned char              buffer;               /*   443     1 */
	unsigned char              bufsiz;               /*   444     1 */
	unsigned char              imexseq;              /*   445     1 */
	unsigned char              impsseq;              /*   446     1 */

	/* XXX 1 byte hole, try to pack */

	/* --- cacheline 7 boundary (448 bytes) --- */
	enum mousedev_emul         mode;                 /*   448     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          last_buttons;         /*   456     8 */

	/* size: 464, cachelines: 8, members: 17 */
	/* sum members: 459, holes: 2, sum holes: 5 */
	/* last cacheline: 16 bytes */
};
struct input_mask {
	__u32                      type;                 /*     0     4 */
	__u32                      codes_size;           /*     4     4 */
	__u64                      codes_ptr;            /*     8     8 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct evdev {
	int                        open;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct input_handle        handle;               /*     8    72 */
	/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
	struct evdev_client *      grab;                 /*    80     8 */
	struct list_head           client_list;          /*    88    16 */
	spinlock_t                 client_lock;          /*   104     4 */

	/* XXX 4 bytes hole, try to pack */

	struct mutex               mutex;                /*   112    32 */
	/* --- cacheline 2 boundary (128 bytes) was 16 bytes ago --- */
	struct device              dev __attribute__((__aligned__(8))); /*   144  1000 */
	/* --- cacheline 17 boundary (1088 bytes) was 56 bytes ago --- */
	struct cdev                cdev;                 /*  1144   136 */
	/* --- cacheline 20 boundary (1280 bytes) --- */
	bool                       exist;                /*  1280     1 */

	/* size: 1288, cachelines: 21, members: 9 */
	/* sum members: 1273, holes: 2, sum holes: 8 */
	/* padding: 7 */
	/* forced alignments: 1 */
	/* last cacheline: 8 bytes */
} __attribute__((__aligned__(8)));
struct evdev_client {
	unsigned int               head;                 /*     0     4 */
	unsigned int               tail;                 /*     4     4 */
	unsigned int               packet_head;          /*     8     4 */
	spinlock_t                 buffer_lock;          /*    12     4 */
	wait_queue_head_t          wait;                 /*    16    24 */
	struct fasync_struct *     fasync;               /*    40     8 */
	struct evdev *             evdev;                /*    48     8 */
	struct list_head           node;                 /*    56    16 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	enum input_clock_type      clk_type;             /*    72     4 */
	bool                       revoked;              /*    76     1 */

	/* XXX 3 bytes hole, try to pack */

	long unsigned int *        evmasks[32];          /*    80   256 */
	/* --- cacheline 5 boundary (320 bytes) was 16 bytes ago --- */
	unsigned int               bufsize;              /*   336     4 */

	/* XXX 4 bytes hole, try to pack */

	struct input_event         buffer[];             /*   344     0 */

	/* size: 344, cachelines: 6, members: 13 */
	/* sum members: 337, holes: 2, sum holes: 7 */
	/* last cacheline: 24 bytes */
};
struct atkbd {
	struct ps2dev              ps2dev;               /*     0    88 */

	/* XXX last struct has 6 bytes of padding */

	/* --- cacheline 1 boundary (64 bytes) was 24 bytes ago --- */
	struct input_dev *         dev;                  /*    88     8 */
	char                       name[64];             /*    96    64 */
	/* --- cacheline 2 boundary (128 bytes) was 32 bytes ago --- */
	char                       phys[32];             /*   160    32 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	short unsigned int         id;                   /*   192     2 */
	short unsigned int         keycode[512];         /*   194  1024 */

	/* XXX 6 bytes hole, try to pack */

	/* --- cacheline 19 boundary (1216 bytes) was 8 bytes ago --- */
	long unsigned int          force_release_mask[8]; /*  1224    64 */
	/* --- cacheline 20 boundary (1280 bytes) was 8 bytes ago --- */
	unsigned char              set;                  /*  1288     1 */
	bool                       translated;           /*  1289     1 */
	bool                       extra;                /*  1290     1 */
	bool                       write;                /*  1291     1 */
	bool                       softrepeat;           /*  1292     1 */
	bool                       softraw;              /*  1293     1 */
	bool                       scroll;               /*  1294     1 */
	bool                       enabled;              /*  1295     1 */
	unsigned char              emul;                 /*  1296     1 */
	bool                       resend;               /*  1297     1 */
	bool                       release;              /*  1298     1 */

	/* XXX 5 bytes hole, try to pack */

	long unsigned int          xl_bit;               /*  1304     8 */
	unsigned int               last;                 /*  1312     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          time;                 /*  1320     8 */
	long unsigned int          err_count;            /*  1328     8 */
	struct delayed_work        event_work;           /*  1336   192 */
	/* --- cacheline 23 boundary (1472 bytes) was 56 bytes ago --- */
	long unsigned int          event_jiffies;        /*  1528     8 */
	/* --- cacheline 24 boundary (1536 bytes) --- */
	long unsigned int          event_mask;           /*  1536     8 */
	struct mutex               mutex;                /*  1544    32 */
	u32                        function_row_physmap[24]; /*  1576    96 */
	/* --- cacheline 26 boundary (1664 bytes) was 8 bytes ago --- */
	int                        num_function_row_keys; /*  1672     4 */

	/* size: 1680, cachelines: 27, members: 28 */
	/* sum members: 1661, holes: 3, sum holes: 15 */
	/* padding: 4 */
	/* paddings: 1, sum paddings: 6 */
	/* last cacheline: 16 bytes */
};
struct psmouse_protocol {
	enum psmouse_type          type;                 /*     0     4 */
	bool                       maxproto;             /*     4     1 */
	bool                       ignore_parity;        /*     5     1 */
	bool                       try_passthru;         /*     6     1 */
	bool                       smbus_companion;      /*     7     1 */
	const char  *              name;                 /*     8     8 */
	const char  *              alias;                /*    16     8 */
	int                        (*detect)(struct psmouse *, bool); /*    24     8 */
	int                        (*init)(struct psmouse *); /*    32     8 */

	/* size: 40, cachelines: 1, members: 9 */
	/* last cacheline: 40 bytes */
};
struct psmouse {
	void *                     private;              /*     0     8 */
	struct input_dev *         dev;                  /*     8     8 */
	struct ps2dev              ps2dev;               /*    16    88 */

	/* XXX last struct has 6 bytes of padding */

	/* --- cacheline 1 boundary (64 bytes) was 40 bytes ago --- */
	struct delayed_work        resync_work;          /*   104   192 */
	/* --- cacheline 4 boundary (256 bytes) was 40 bytes ago --- */
	const char  *              vendor;               /*   296     8 */
	const char  *              name;                 /*   304     8 */
	const struct psmouse_protocol  * protocol;       /*   312     8 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	unsigned char              packet[8];            /*   320     8 */
	unsigned char              badbyte;              /*   328     1 */
	unsigned char              pktcnt;               /*   329     1 */
	unsigned char              pktsize;              /*   330     1 */
	unsigned char              oob_data_type;        /*   331     1 */
	unsigned char              extra_buttons;        /*   332     1 */
	bool                       acks_disable_command; /*   333     1 */

	/* XXX 2 bytes hole, try to pack */

	unsigned int               model;                /*   336     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          last;                 /*   344     8 */
	long unsigned int          out_of_sync_cnt;      /*   352     8 */
	long unsigned int          num_resyncs;          /*   360     8 */
	enum psmouse_state         state;                /*   368     4 */
	char                       devname[64];          /*   372    64 */
	/* --- cacheline 6 boundary (384 bytes) was 52 bytes ago --- */
	char                       phys[32];             /*   436    32 */
	/* --- cacheline 7 boundary (448 bytes) was 20 bytes ago --- */
	unsigned int               rate;                 /*   468     4 */
	unsigned int               resolution;           /*   472     4 */
	unsigned int               resetafter;           /*   476     4 */
	unsigned int               resync_time;          /*   480     4 */
	bool                       smartscroll;          /*   484     1 */

	/* XXX 3 bytes hole, try to pack */

	psmouse_ret_t              (*protocol_handler)(struct psmouse *); /*   488     8 */
	void                       (*set_rate)(struct psmouse *, unsigned int); /*   496     8 */
	void                       (*set_resolution)(struct psmouse *, unsigned int); /*   504     8 */
	/* --- cacheline 8 boundary (512 bytes) --- */
	void                       (*set_scale)(struct psmouse *, enum psmouse_scale); /*   512     8 */
	int                        (*reconnect)(struct psmouse *); /*   520     8 */
	int                        (*fast_reconnect)(struct psmouse *); /*   528     8 */
	void                       (*disconnect)(struct psmouse *); /*   536     8 */
	void                       (*cleanup)(struct psmouse *); /*   544     8 */
	int                        (*poll)(struct psmouse *); /*   552     8 */
	void                       (*pt_activate)(struct psmouse *); /*   560     8 */
	void                       (*pt_deactivate)(struct psmouse *); /*   568     8 */

	/* size: 576, cachelines: 9, members: 37 */
	/* sum members: 567, holes: 3, sum holes: 9 */
	/* paddings: 1, sum paddings: 6 */
};
struct psmouse_attribute {
	struct device_attribute    dattr;                /*     0    32 */
	void *                     data;                 /*    32     8 */
	ssize_t                    (*show)(struct psmouse *, void *, char *); /*    40     8 */
	ssize_t                    (*set)(struct psmouse *, void *, const char  *, size_t); /*    48     8 */
	bool                       protect;              /*    56     1 */

	/* size: 64, cachelines: 1, members: 5 */
	/* padding: 7 */
};
struct rmi_2d_axis_alignment {
	bool                       swap_axes;            /*     0     1 */
	bool                       flip_x;               /*     1     1 */
	bool                       flip_y;               /*     2     1 */

	/* XXX 1 byte hole, try to pack */

	u16                        clip_x_low;           /*     4     2 */
	u16                        clip_y_low;           /*     6     2 */
	u16                        clip_x_high;          /*     8     2 */
	u16                        clip_y_high;          /*    10     2 */
	u16                        offset_x;             /*    12     2 */
	u16                        offset_y;             /*    14     2 */
	u8                         delta_x_threshold;    /*    16     1 */
	u8                         delta_y_threshold;    /*    17     1 */

	/* size: 18, cachelines: 1, members: 11 */
	/* sum members: 17, holes: 1, sum holes: 1 */
	/* last cacheline: 18 bytes */
};
struct rmi_2d_sensor_platform_data {
	struct rmi_2d_axis_alignment axis_align;         /*     0    18 */

	/* XXX 2 bytes hole, try to pack */

	enum rmi_sensor_type       sensor_type;          /*    20     4 */
	int                        x_mm;                 /*    24     4 */
	int                        y_mm;                 /*    28     4 */
	int                        disable_report_mask;  /*    32     4 */
	u16                        rezero_wait;          /*    36     2 */
	bool                       topbuttonpad;         /*    38     1 */
	bool                       kernel_tracking;      /*    39     1 */
	int                        dmax;                 /*    40     4 */
	int                        dribble;              /*    44     4 */
	int                        palm_detect;          /*    48     4 */

	/* size: 52, cachelines: 1, members: 11 */
	/* sum members: 50, holes: 1, sum holes: 2 */
	/* last cacheline: 52 bytes */
};
struct rmi_gpio_data {
	bool                       buttonpad;            /*     0     1 */
	bool                       trackstick_buttons;   /*     1     1 */
	bool                       disable;              /*     2     1 */

	/* size: 3, cachelines: 1, members: 3 */
	/* last cacheline: 3 bytes */
};
struct rmi_f01_power_management {
	enum rmi_reg_state         nosleep;              /*     0     4 */
	u8                         wakeup_threshold;     /*     4     1 */
	u8                         doze_holdoff;         /*     5     1 */
	u8                         doze_interval;        /*     6     1 */

	/* size: 8, cachelines: 1, members: 4 */
	/* padding: 1 */
	/* last cacheline: 8 bytes */
};
struct rmi_device_platform_data_spi {
	u32                        block_delay_us;       /*     0     4 */
	u32                        split_read_block_delay_us; /*     4     4 */
	u32                        read_delay_us;        /*     8     4 */
	u32                        write_delay_us;       /*    12     4 */
	u32                        split_read_byte_delay_us; /*    16     4 */
	u32                        pre_delay_us;         /*    20     4 */
	u32                        post_delay_us;        /*    24     4 */
	u8                         bits_per_word;        /*    28     1 */

	/* XXX 1 byte hole, try to pack */

	u16                        mode;                 /*    30     2 */
	void *                     cs_assert_data;       /*    32     8 */
	int                        (*cs_assert)(const void  *, const bool ); /*    40     8 */

	/* size: 48, cachelines: 1, members: 11 */
	/* sum members: 47, holes: 1, sum holes: 1 */
	/* last cacheline: 48 bytes */
};
struct rmi_device_platform_data {
	int                        reset_delay_ms;       /*     0     4 */
	int                        irq;                  /*     4     4 */
	struct rmi_device_platform_data_spi spi_data;    /*     8    48 */
	struct rmi_2d_sensor_platform_data sensor_pdata; /*    56    52 */
	/* --- cacheline 1 boundary (64 bytes) was 44 bytes ago --- */
	struct rmi_f01_power_management power_management; /*   108     8 */

	/* XXX last struct has 1 byte of padding */

	struct rmi_gpio_data       gpio_data;            /*   116     3 */

	/* size: 120, cachelines: 2, members: 6 */
	/* padding: 1 */
	/* paddings: 1, sum paddings: 1 */
	/* last cacheline: 56 bytes */
};
struct i2c_board_info {
	char                       type[20];             /*     0    20 */
	short unsigned int         flags;                /*    20     2 */
	short unsigned int         addr;                 /*    22     2 */
	const char  *              dev_name;             /*    24     8 */
	void *                     platform_data;        /*    32     8 */
	struct device_node *       of_node;              /*    40     8 */
	struct fwnode_handle *     fwnode;               /*    48     8 */
	const struct property_entry  * properties;       /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	const struct resource  *   resources;            /*    64     8 */
	unsigned int               num_resources;        /*    72     4 */
	int                        irq;                  /*    76     4 */
	u64                        kabi_reserved1;       /*    80     8 */

	/* size: 88, cachelines: 2, members: 12 */
	/* last cacheline: 24 bytes */
};
struct synaptics_hw_state {
	int                        x;                    /*     0     4 */
	int                        y;                    /*     4     4 */
	int                        z;                    /*     8     4 */
	int                        w;                    /*    12     4 */
	unsigned int               left:1;               /*    16: 0  4 */
	unsigned int               right:1;              /*    16: 1  4 */
	unsigned int               middle:1;             /*    16: 2  4 */
	unsigned int               up:1;                 /*    16: 3  4 */
	unsigned int               down:1;               /*    16: 4  4 */

	/* XXX 3 bits hole, try to pack */
	/* Bitfield combined with next fields */

	u8                         ext_buttons;          /*    17     1 */
	s8                         scroll;               /*    18     1 */

	/* size: 20, cachelines: 1, members: 11 */
	/* sum members: 18 */
	/* sum bitfield members: 5 bits, bit holes: 1, sum bit holes: 3 bits */
	/* padding: 1 */
	/* last cacheline: 20 bytes */
};
struct synaptics_device_info {
	u32                        model_id;             /*     0     4 */
	u32                        firmware_id;          /*     4     4 */
	u32                        board_id;             /*     8     4 */
	u32                        capabilities;         /*    12     4 */
	u32                        ext_cap;              /*    16     4 */
	u32                        ext_cap_0c;           /*    20     4 */
	u32                        ext_cap_10;           /*    24     4 */
	u32                        identity;             /*    28     4 */
	u32                        x_res;                /*    32     4 */
	u32                        y_res;                /*    36     4 */
	u32                        x_max;                /*    40     4 */
	u32                        y_max;                /*    44     4 */
	u32                        x_min;                /*    48     4 */
	u32                        y_min;                /*    52     4 */

	/* size: 56, cachelines: 1, members: 14 */
	/* last cacheline: 56 bytes */
};
struct synaptics_data {
	struct synaptics_device_info info;               /*     0    56 */
	enum synaptics_pkt_type    pkt_type;             /*    56     4 */
	u8                         mode;                 /*    60     1 */

	/* XXX 3 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        scroll;               /*    64     4 */
	bool                       absolute_mode;        /*    68     1 */
	bool                       disable_gesture;      /*    69     1 */

	/* XXX 2 bytes hole, try to pack */

	struct serio *             pt_port;              /*    72     8 */
	struct synaptics_hw_state  agm;                  /*    80    20 */

	/* XXX last struct has 1 byte of padding */

	unsigned int               agm_count;            /*   100     4 */
	long unsigned int          press_start;          /*   104     8 */
	bool                       press;                /*   112     1 */
	bool                       report_press;         /*   113     1 */
	bool                       is_forcepad;          /*   114     1 */

	/* size: 120, cachelines: 2, members: 13 */
	/* sum members: 110, holes: 2, sum holes: 5 */
	/* padding: 5 */
	/* paddings: 1, sum paddings: 1 */
	/* last cacheline: 56 bytes */
};
struct min_max_quirk {
	const char  * const *      pnp_ids;              /*     0     8 */
	struct {
		u32                min;                  /*     8     4 */
		u32                max;                  /*    12     4 */
	} board_id;                                      /*     8     8 */
	u32                        x_min;                /*    16     4 */
	u32                        x_max;                /*    20     4 */
	u32                        y_min;                /*    24     4 */
	u32                        y_max;                /*    28     4 */

	/* size: 32, cachelines: 1, members: 6 */
	/* last cacheline: 32 bytes */
};
struct focaltech_finger_state {
	bool                       active;               /*     0     1 */
	bool                       valid;                /*     1     1 */

	/* XXX 2 bytes hole, try to pack */

	unsigned int               x;                    /*     4     4 */
	unsigned int               y;                    /*     8     4 */

	/* size: 12, cachelines: 1, members: 4 */
	/* sum members: 10, holes: 1, sum holes: 2 */
	/* last cacheline: 12 bytes */
};
struct focaltech_hw_state {
	struct focaltech_finger_state fingers[5];        /*     0    60 */
	unsigned int               width;                /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	bool                       pressed;              /*    64     1 */

	/* size: 68, cachelines: 2, members: 3 */
	/* padding: 3 */
	/* last cacheline: 4 bytes */
};
struct focaltech_data {
	unsigned int               x_max;                /*     0     4 */
	unsigned int               y_max;                /*     4     4 */
	struct focaltech_hw_state  state;                /*     8    68 */

	/* XXX last struct has 3 bytes of padding */

	/* size: 76, cachelines: 2, members: 3 */
	/* paddings: 1, sum paddings: 3 */
	/* last cacheline: 12 bytes */
};
struct alps_protocol_info {
	u16                        version;              /*     0     2 */
	u8                         byte0;                /*     2     1 */
	u8                         mask0;                /*     3     1 */
	unsigned int               flags;                /*     4     4 */

	/* size: 8, cachelines: 1, members: 4 */
	/* last cacheline: 8 bytes */
};
struct alps_model_info {
	u8                         signature[3];         /*     0     3 */

	/* XXX 1 byte hole, try to pack */

	struct alps_protocol_info  protocol_info;        /*     4     8 */

	/* size: 12, cachelines: 1, members: 2 */
	/* sum members: 11, holes: 1, sum holes: 1 */
	/* last cacheline: 12 bytes */
};
struct alps_nibble_commands {
	int                        command;              /*     0     4 */
	unsigned char              data;                 /*     4     1 */

	/* size: 8, cachelines: 1, members: 2 */
	/* padding: 3 */
	/* last cacheline: 8 bytes */
};
struct alps_bitmap_point {
	int                        start_bit;            /*     0     4 */
	int                        num_bits;             /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct alps_fields {
	unsigned int               x_map;                /*     0     4 */
	unsigned int               y_map;                /*     4     4 */
	unsigned int               fingers;              /*     8     4 */
	int                        pressure;             /*    12     4 */
	struct input_mt_pos        st;                   /*    16     4 */
	struct input_mt_pos        mt[4];                /*    20    16 */
	unsigned int               first_mp:1;           /*    36: 0  4 */
	unsigned int               is_mp:1;              /*    36: 1  4 */
	unsigned int               left:1;               /*    36: 2  4 */
	unsigned int               right:1;              /*    36: 3  4 */
	unsigned int               middle:1;             /*    36: 4  4 */
	unsigned int               ts_left:1;            /*    36: 5  4 */
	unsigned int               ts_right:1;           /*    36: 6  4 */
	unsigned int               ts_middle:1;          /*    36: 7  4 */

	/* size: 40, cachelines: 1, members: 14 */
	/* bit_padding: 24 bits */
	/* last cacheline: 40 bytes */
};
struct alps_data {
	struct psmouse *           psmouse;              /*     0     8 */
	struct input_dev *         dev2;                 /*     8     8 */
	struct input_dev *         dev3;                 /*    16     8 */
	char                       phys2[32];            /*    24    32 */
	char                       phys3[32];            /*    56    32 */
	/* --- cacheline 1 boundary (64 bytes) was 24 bytes ago --- */
	struct delayed_work        dev3_register_work;   /*    88   192 */
	/* --- cacheline 4 boundary (256 bytes) was 24 bytes ago --- */
	const struct alps_nibble_commands  * nibble_commands; /*   280     8 */
	int                        addr_command;         /*   288     4 */
	u16                        proto_version;        /*   292     2 */
	u8                         byte0;                /*   294     1 */
	u8                         mask0;                /*   295     1 */
	u8                         dev_id[3];            /*   296     3 */
	u8                         fw_ver[3];            /*   299     3 */

	/* XXX 2 bytes hole, try to pack */

	int                        flags;                /*   304     4 */
	int                        x_max;                /*   308     4 */
	int                        y_max;                /*   312     4 */
	int                        x_bits;               /*   316     4 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	int                        y_bits;               /*   320     4 */
	unsigned int               x_res;                /*   324     4 */
	unsigned int               y_res;                /*   328     4 */

	/* XXX 4 bytes hole, try to pack */

	int                        (*hw_init)(struct psmouse *); /*   336     8 */
	void                       (*process_packet)(struct psmouse *); /*   344     8 */
	int                        (*decode_fields)(struct alps_fields *, unsigned char *, struct psmouse *); /*   352     8 */
	void                       (*set_abs_params)(struct alps_data *, struct input_dev *); /*   360     8 */
	int                        prev_fin;             /*   368     4 */
	int                        multi_packet;         /*   372     4 */
	int                        second_touch;         /*   376     4 */
	unsigned char              multi_data[6];        /*   380     6 */

	/* XXX 2 bytes hole, try to pack */

	/* --- cacheline 6 boundary (384 bytes) was 4 bytes ago --- */
	struct alps_fields         f;                    /*   388    40 */
	u8                         quirks;               /*   428     1 */

	/* XXX 3 bytes hole, try to pack */

	struct timer_list          timer;                /*   432    72 */

	/* size: 504, cachelines: 8, members: 31 */
	/* sum members: 493, holes: 4, sum holes: 11 */
	/* last cacheline: 56 bytes */
};
struct byd_data {
	struct timer_list          timer;                /*     0    72 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	struct psmouse *           psmouse;              /*    72     8 */
	s32                        abs_x;                /*    80     4 */
	s32                        abs_y;                /*    84     4 */
	volatile long unsigned int  last_touch_time;     /*    88     8 */
	bool                       btn_left;             /*    96     1 */
	bool                       btn_right;            /*    97     1 */
	bool                       touch;                /*    98     1 */

	/* size: 104, cachelines: 2, members: 8 */
	/* padding: 5 */
	/* last cacheline: 40 bytes */
};
struct finger_pos {
	unsigned int               x;                    /*     0     4 */
	unsigned int               y;                    /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct elantech_device_info {
	unsigned char              capabilities[3];      /*     0     3 */
	unsigned char              samples[3];           /*     3     3 */
	unsigned char              debug;                /*     6     1 */
	unsigned char              hw_version;           /*     7     1 */
	unsigned char              pattern;              /*     8     1 */

	/* XXX 3 bytes hole, try to pack */

	unsigned int               fw_version;           /*    12     4 */
	unsigned int               ic_version;           /*    16     4 */
	unsigned int               product_id;           /*    20     4 */
	unsigned int               x_min;                /*    24     4 */
	unsigned int               y_min;                /*    28     4 */
	unsigned int               x_max;                /*    32     4 */
	unsigned int               y_max;                /*    36     4 */
	unsigned int               x_res;                /*    40     4 */
	unsigned int               y_res;                /*    44     4 */
	unsigned int               x_traces;             /*    48     4 */
	unsigned int               y_traces;             /*    52     4 */
	unsigned int               width;                /*    56     4 */
	unsigned int               bus;                  /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	bool                       paritycheck;          /*    64     1 */
	bool                       jumpy_cursor;         /*    65     1 */
	bool                       reports_pressure;     /*    66     1 */
	bool                       crc_enabled;          /*    67     1 */
	bool                       set_hw_resolution;    /*    68     1 */
	bool                       has_trackpoint;       /*    69     1 */
	bool                       has_middle_button;    /*    70     1 */

	/* XXX 1 byte hole, try to pack */

	int                        (*send_cmd)(struct psmouse *, unsigned char, unsigned char *); /*    72     8 */

	/* size: 80, cachelines: 2, members: 26 */
	/* sum members: 76, holes: 2, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct elantech_data {
	struct input_dev *         tp_dev;               /*     0     8 */
	char                       tp_phys[32];          /*     8    32 */
	unsigned char              reg_07;               /*    40     1 */
	unsigned char              reg_10;               /*    41     1 */
	unsigned char              reg_11;               /*    42     1 */
	unsigned char              reg_20;               /*    43     1 */
	unsigned char              reg_21;               /*    44     1 */
	unsigned char              reg_22;               /*    45     1 */
	unsigned char              reg_23;               /*    46     1 */
	unsigned char              reg_24;               /*    47     1 */
	unsigned char              reg_25;               /*    48     1 */
	unsigned char              reg_26;               /*    49     1 */

	/* XXX 2 bytes hole, try to pack */

	unsigned int               single_finger_reports; /*    52     4 */
	unsigned int               y_max;                /*    56     4 */
	unsigned int               width;                /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct finger_pos          mt[5];                /*    64    40 */
	unsigned char              parity[256];          /*   104   256 */
	/* --- cacheline 5 boundary (320 bytes) was 40 bytes ago --- */
	struct elantech_device_info info;                /*   360    80 */
	/* --- cacheline 6 boundary (384 bytes) was 56 bytes ago --- */
	void                       (*original_set_rate)(struct psmouse *, unsigned int); /*   440     8 */

	/* size: 448, cachelines: 7, members: 19 */
	/* sum members: 446, holes: 1, sum holes: 2 */
};
struct elantech_attr_data {
	size_t                     field_offset;         /*     0     8 */
	unsigned char              reg;                  /*     8     1 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 7 */
	/* last cacheline: 16 bytes */
};
struct ps2pp_info {
	u8                         model;                /*     0     1 */
	u8                         kind;                 /*     1     1 */
	u16                        features;             /*     2     2 */

	/* size: 4, cachelines: 1, members: 3 */
	/* last cacheline: 4 bytes */
};
struct lifebook_data {
	struct input_dev *         dev2;                 /*     0     8 */
	char                       phys[32];             /*     8    32 */

	/* size: 40, cachelines: 1, members: 2 */
	/* last cacheline: 40 bytes */
};
struct fsp_data {
	unsigned char              ver;                  /*     0     1 */
	unsigned char              rev;                  /*     1     1 */

	/* XXX 2 bytes hole, try to pack */

	unsigned int               buttons;              /*     4     4 */
	unsigned int               flags;                /*     8     4 */
	bool                       vscroll;              /*    12     1 */
	bool                       hscroll;              /*    13     1 */
	unsigned char              last_reg;             /*    14     1 */
	unsigned char              last_val;             /*    15     1 */
	unsigned int               last_mt_fgr;          /*    16     4 */

	/* size: 20, cachelines: 1, members: 9 */
	/* sum members: 18, holes: 1, sum holes: 2 */
	/* last cacheline: 20 bytes */
};
struct trackpoint_data {
	u8                         variant_id;           /*     0     1 */
	u8                         firmware_id;          /*     1     1 */
	u8                         sensitivity;          /*     2     1 */
	u8                         speed;                /*     3     1 */
	u8                         inertia;              /*     4     1 */
	u8                         reach;                /*     5     1 */
	u8                         draghys;              /*     6     1 */
	u8                         mindrag;              /*     7     1 */
	u8                         thresh;               /*     8     1 */
	u8                         upthresh;             /*     9     1 */
	u8                         ztime;                /*    10     1 */
	u8                         jenks;                /*    11     1 */
	u8                         drift_time;           /*    12     1 */
	bool                       press_to_select;      /*    13     1 */
	bool                       skipback;             /*    14     1 */
	bool                       ext_dev;              /*    15     1 */

	/* size: 16, cachelines: 1, members: 16 */
	/* last cacheline: 16 bytes */
};
struct trackpoint_attr_data {
	size_t                     field_offset;         /*     0     8 */
	u8                         command;              /*     8     1 */
	u8                         mask;                 /*     9     1 */
	bool                       inverted;             /*    10     1 */
	u8                         power_on_default;     /*    11     1 */

	/* size: 16, cachelines: 1, members: 5 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct cytp_contact {
	int                        x;                    /*     0     4 */
	int                        y;                    /*     4     4 */
	int                        z;                    /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct cytp_report_data {
	int                        contact_cnt;          /*     0     4 */
	struct cytp_contact        contacts[2];          /*     4    24 */
	unsigned int               left:1;               /*    28: 0  4 */
	unsigned int               right:1;              /*    28: 1  4 */
	unsigned int               middle:1;             /*    28: 2  4 */
	unsigned int               tap:1;                /*    28: 3  4 */

	/* size: 32, cachelines: 1, members: 6 */
	/* bit_padding: 28 bits */
	/* last cacheline: 32 bytes */
};
struct cytp_data {
	int                        fw_version;           /*     0     4 */
	int                        pkt_size;             /*     4     4 */
	int                        mode;                 /*     8     4 */
	int                        tp_min_pressure;      /*    12     4 */
	int                        tp_max_pressure;      /*    16     4 */
	int                        tp_width;             /*    20     4 */
	int                        tp_high;              /*    24     4 */
	int                        tp_max_abs_x;         /*    28     4 */
	int                        tp_max_abs_y;         /*    32     4 */
	int                        tp_res_x;             /*    36     4 */
	int                        tp_res_y;             /*    40     4 */
	int                        tp_metrics_supported; /*    44     4 */

	/* size: 48, cachelines: 1, members: 12 */
	/* last cacheline: 48 bytes */
};
struct vmmouse_data {
	struct input_dev *         abs_dev;              /*     0     8 */
	char                       phys[32];             /*     8    32 */
	char                       dev_name[128];        /*    40   128 */

	/* size: 168, cachelines: 3, members: 3 */
	/* last cacheline: 40 bytes */
};
struct i2c_msg {
	__u16                      addr;                 /*     0     2 */
	__u16                      flags;                /*     2     2 */
	__u16                      len;                  /*     4     2 */

	/* XXX 2 bytes hole, try to pack */

	__u8 *                     buf;                  /*     8     8 */

	/* size: 16, cachelines: 1, members: 4 */
	/* sum members: 14, holes: 1, sum holes: 2 */
	/* last cacheline: 16 bytes */
};
union i2c_smbus_data {
	__u8                       byte;               /*     0     1 */
	__u16                      word;               /*     0     2 */
	__u8                       block[34];          /*     0    34 */
};
struct i2c_client {
	short unsigned int         flags;                /*     0     2 */
	short unsigned int         addr;                 /*     2     2 */
	char                       name[20];             /*     4    20 */
	struct i2c_adapter *       adapter;              /*    24     8 */
	struct device              dev __attribute__((__aligned__(8))); /*    32  1000 */
	/* --- cacheline 16 boundary (1024 bytes) was 8 bytes ago --- */
	int                        init_irq;             /*  1032     4 */
	int                        irq;                  /*  1036     4 */
	struct list_head           detected;             /*  1040    16 */

	/* size: 1056, cachelines: 17, members: 8 */
	/* forced alignments: 1 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct i2c_adapter {
	struct module *            owner;                /*     0     8 */
	unsigned int               class;                /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	const struct i2c_algorithm  * algo;              /*    16     8 */
	void *                     algo_data;            /*    24     8 */
	const struct i2c_lock_operations  * lock_ops;    /*    32     8 */
	struct rt_mutex            bus_lock;             /*    40    32 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	struct rt_mutex            mux_lock;             /*    72    32 */
	int                        timeout;              /*   104     4 */
	int                        retries;              /*   108     4 */
	struct device              dev __attribute__((__aligned__(8))); /*   112  1000 */
	/* --- cacheline 17 boundary (1088 bytes) was 24 bytes ago --- */
	long unsigned int          locked_flags;         /*  1112     8 */
	int                        nr;                   /*  1120     4 */
	char                       name[48];             /*  1124    48 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 18 boundary (1152 bytes) was 24 bytes ago --- */
	struct completion          dev_released;         /*  1176    32 */
	struct mutex               userspace_clients_lock; /*  1208    32 */
	/* --- cacheline 19 boundary (1216 bytes) was 24 bytes ago --- */
	struct list_head           userspace_clients;    /*  1240    16 */
	struct i2c_bus_recovery_info * bus_recovery_info; /*  1256     8 */
	const struct i2c_adapter_quirks  * quirks;       /*  1264     8 */
	struct irq_domain *        host_notify_domain;   /*  1272     8 */
	/* --- cacheline 20 boundary (1280 bytes) --- */
	u64                        kabi_reserved1;       /*  1280     8 */
	u64                        kabi_reserved2;       /*  1288     8 */

	/* size: 1296, cachelines: 21, members: 21 */
	/* sum members: 1288, holes: 2, sum holes: 8 */
	/* forced alignments: 1 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct i2c_algorithm {
	int                        (*master_xfer)(struct i2c_adapter *, struct i2c_msg *, int); /*     0     8 */
	int                        (*master_xfer_atomic)(struct i2c_adapter *, struct i2c_msg *, int); /*     8     8 */
	int                        (*smbus_xfer)(struct i2c_adapter *, u16, short unsigned int, char, u8, int, union i2c_smbus_data *); /*    16     8 */
	int                        (*smbus_xfer_atomic)(struct i2c_adapter *, u16, short unsigned int, char, u8, int, union i2c_smbus_data *); /*    24     8 */
	u32                        (*functionality)(struct i2c_adapter *); /*    32     8 */
	u64                        kabi_reserved1;       /*    40     8 */
	u64                        kabi_reserved2;       /*    48     8 */

	/* size: 56, cachelines: 1, members: 7 */
	/* last cacheline: 56 bytes */
};
struct i2c_lock_operations {
	void                       (*lock_bus)(struct i2c_adapter *, unsigned int); /*     0     8 */
	int                        (*trylock_bus)(struct i2c_adapter *, unsigned int); /*     8     8 */
	void                       (*unlock_bus)(struct i2c_adapter *, unsigned int); /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct i2c_bus_recovery_info {
	int                        (*recover_bus)(struct i2c_adapter *); /*     0     8 */
	int                        (*get_scl)(struct i2c_adapter *); /*     8     8 */
	void                       (*set_scl)(struct i2c_adapter *, int); /*    16     8 */
	int                        (*get_sda)(struct i2c_adapter *); /*    24     8 */
	void                       (*set_sda)(struct i2c_adapter *, int); /*    32     8 */
	int                        (*get_bus_free)(struct i2c_adapter *); /*    40     8 */
	void                       (*prepare_recovery)(struct i2c_adapter *); /*    48     8 */
	void                       (*unprepare_recovery)(struct i2c_adapter *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct gpio_desc *         scl_gpiod;            /*    64     8 */
	struct gpio_desc *         sda_gpiod;            /*    72     8 */
	struct pinctrl *           pinctrl;              /*    80     8 */
	struct pinctrl_state *     pins_default;         /*    88     8 */
	struct pinctrl_state *     pins_gpio;            /*    96     8 */
	u64                        kabi_reserved1;       /*   104     8 */
	u64                        kabi_reserved2;       /*   112     8 */

	/* size: 120, cachelines: 2, members: 15 */
	/* last cacheline: 56 bytes */
};
struct i2c_adapter_quirks {
	u64                        flags;                /*     0     8 */
	int                        max_num_msgs;         /*     8     4 */
	u16                        max_write_len;        /*    12     2 */
	u16                        max_read_len;         /*    14     2 */
	u16                        max_comb_1st_msg_len; /*    16     2 */
	u16                        max_comb_2nd_msg_len; /*    18     2 */

	/* size: 24, cachelines: 1, members: 6 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct psmouse_smbus_dev {
	struct i2c_board_info      board;                /*     0    88 */
	/* --- cacheline 1 boundary (64 bytes) was 24 bytes ago --- */
	struct psmouse *           psmouse;              /*    88     8 */
	struct i2c_client *        client;               /*    96     8 */
	struct list_head           node;                 /*   104    16 */
	bool                       dead;                 /*   120     1 */
	bool                       need_deactivate;      /*   121     1 */

	/* size: 128, cachelines: 2, members: 6 */
	/* padding: 6 */
};
struct psmouse_smbus_removal_work {
	struct work_struct         work;                 /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct i2c_client *        client;               /*    64     8 */

	/* size: 72, cachelines: 2, members: 2 */
	/* last cacheline: 8 bytes */
};
struct touchscreen_properties {
	unsigned int               max_x;                /*     0     4 */
	unsigned int               max_y;                /*     4     4 */
	bool                       invert_x;             /*     8     1 */
	bool                       invert_y;             /*     9     1 */
	bool                       swap_x_y;             /*    10     1 */

	/* size: 12, cachelines: 1, members: 5 */
	/* padding: 1 */
	/* last cacheline: 12 bytes */
};
struct trace_event_raw_rtc_time_alarm_class {
	struct trace_entry         ent;                  /*     0     8 */
	time64_t                   secs;                 /*     8     8 */
	int                        err;                  /*    16     4 */
	char                       __data[];             /*    20     0 */

	/* size: 24, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_rtc_irq_set_freq {
	struct trace_entry         ent;                  /*     0     8 */
	int                        freq;                 /*     8     4 */
	int                        err;                  /*    12     4 */
	char                       __data[];             /*    16     0 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct trace_event_raw_rtc_irq_set_state {
	struct trace_entry         ent;                  /*     0     8 */
	int                        enabled;              /*     8     4 */
	int                        err;                  /*    12     4 */
	char                       __data[];             /*    16     0 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct trace_event_raw_rtc_alarm_irq_enable {
	struct trace_entry         ent;                  /*     0     8 */
	unsigned int               enabled;              /*     8     4 */
	int                        err;                  /*    12     4 */
	char                       __data[];             /*    16     0 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct trace_event_raw_rtc_offset_class {
	struct trace_entry         ent;                  /*     0     8 */
	long int                   offset;               /*     8     8 */
	int                        err;                  /*    16     4 */
	char                       __data[];             /*    20     0 */

	/* size: 24, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_rtc_timer_class {
	struct trace_entry         ent;                  /*     0     8 */
	struct rtc_timer *         timer;                /*     8     8 */
	ktime_t                    expires;              /*    16     8 */
	ktime_t                    period;               /*    24     8 */
	char                       __data[];             /*    32     0 */

	/* size: 32, cachelines: 1, members: 5 */
	/* last cacheline: 32 bytes */
};
struct trace_event_data_offsets_rtc_time_alarm_class {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_rtc_irq_set_freq {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_rtc_irq_set_state {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_rtc_alarm_irq_enable {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_rtc_offset_class {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_rtc_timer_class {

	/* size: 0, cachelines: 0, members: 0 */
};
struct nvmem_cell_info {
	const char  *              name;                 /*     0     8 */
	unsigned int               offset;               /*     8     4 */
	unsigned int               bytes;                /*    12     4 */
	unsigned int               bit_offset;           /*    16     4 */
	unsigned int               nbits;                /*    20     4 */

	/* size: 24, cachelines: 1, members: 5 */
	/* last cacheline: 24 bytes */
};
struct nvmem_config {
	struct device *            dev;                  /*     0     8 */
	const char  *              name;                 /*     8     8 */
	int                        id;                   /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	struct module *            owner;                /*    24     8 */
	struct gpio_desc *         wp_gpio;              /*    32     8 */
	const struct nvmem_cell_info  * cells;           /*    40     8 */
	int                        ncells;               /*    48     4 */
	enum nvmem_type            type;                 /*    52     4 */
	bool                       read_only;            /*    56     1 */
	bool                       root_only;            /*    57     1 */
	bool                       no_of_node;           /*    58     1 */

	/* XXX 5 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	nvmem_reg_read_t           reg_read;             /*    64     8 */
	nvmem_reg_write_t          reg_write;            /*    72     8 */
	int                        size;                 /*    80     4 */
	int                        word_size;            /*    84     4 */
	int                        stride;               /*    88     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     priv;                 /*    96     8 */
	bool                       compat;               /*   104     1 */

	/* XXX 7 bytes hole, try to pack */

	struct device *            base_dev;             /*   112     8 */

	/* size: 120, cachelines: 2, members: 19 */
	/* sum members: 100, holes: 4, sum holes: 20 */
	/* last cacheline: 56 bytes */
};
struct cmos_rtc_board_info {
	void                       (*wake_on)(struct device *); /*     0     8 */
	void                       (*wake_off)(struct device *); /*     8     8 */
	u32                        flags;                /*    16     4 */
	int                        address_space;        /*    20     4 */
	u8                         rtc_day_alarm;        /*    24     1 */
	u8                         rtc_mon_alarm;        /*    25     1 */
	u8                         rtc_century;          /*    26     1 */

	/* size: 32, cachelines: 1, members: 7 */
	/* padding: 5 */
	/* last cacheline: 32 bytes */
};
struct cmos_rtc {
	struct rtc_device *        rtc;                  /*     0     8 */
	struct device *            dev;                  /*     8     8 */
	int                        irq;                  /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	struct resource *          iomem;                /*    24     8 */
	time64_t                   alarm_expires;        /*    32     8 */
	void                       (*wake_on)(struct device *); /*    40     8 */
	void                       (*wake_off)(struct device *); /*    48     8 */
	u8                         enabled_wake;         /*    56     1 */
	u8                         suspend_ctrl;         /*    57     1 */
	u8                         day_alrm;             /*    58     1 */
	u8                         mon_alrm;             /*    59     1 */
	u8                         century;              /*    60     1 */

	/* XXX 3 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	struct rtc_wkalrm          saved_wkalrm;         /*    64    40 */

	/* size: 104, cachelines: 2, members: 13 */
	/* sum members: 97, holes: 2, sum holes: 7 */
	/* last cacheline: 40 bytes */
};
struct i2c_devinfo {
	struct list_head           list;                 /*     0    16 */
	int                        busnum;               /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	struct i2c_board_info      board_info;           /*    24    88 */

	/* size: 112, cachelines: 2, members: 3 */
	/* sum members: 108, holes: 1, sum holes: 4 */
	/* last cacheline: 48 bytes */
};
struct i2c_device_id {
	char                       name[20];             /*     0    20 */

	/* XXX 4 bytes hole, try to pack */

	kernel_ulong_t             driver_data;          /*    24     8 */

	/* size: 32, cachelines: 1, members: 2 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct i2c_device_identity {
	u16                        manufacturer_id;      /*     0     2 */
	u16                        part_id;              /*     2     2 */
	u8                         die_revision;         /*     4     1 */

	/* size: 6, cachelines: 1, members: 3 */
	/* padding: 1 */
	/* last cacheline: 6 bytes */
};
struct i2c_driver {
	unsigned int               class;                /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	int                        (*probe)(struct i2c_client *, const struct i2c_device_id  *); /*     8     8 */
	int                        (*remove)(struct i2c_client *); /*    16     8 */
	int                        (*probe_new)(struct i2c_client *); /*    24     8 */
	void                       (*shutdown)(struct i2c_client *); /*    32     8 */
	void                       (*alert)(struct i2c_client *, enum i2c_alert_protocol, unsigned int); /*    40     8 */
	int                        (*command)(struct i2c_client *, unsigned int, void *); /*    48     8 */
	struct device_driver       driver;               /*    56   192 */
	/* --- cacheline 3 boundary (192 bytes) was 56 bytes ago --- */
	const struct i2c_device_id  * id_table;          /*   248     8 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	int                        (*detect)(struct i2c_client *, struct i2c_board_info *); /*   256     8 */
	const short unsigned int  * address_list;        /*   264     8 */
	struct list_head           clients;              /*   272    16 */

	/* size: 288, cachelines: 5, members: 12 */
	/* sum members: 284, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct i2c_timings {
	u32                        bus_freq_hz;          /*     0     4 */
	u32                        scl_rise_ns;          /*     4     4 */
	u32                        scl_fall_ns;          /*     8     4 */
	u32                        scl_int_delay_ns;     /*    12     4 */
	u32                        sda_fall_ns;          /*    16     4 */
	u32                        sda_hold_ns;          /*    20     4 */
	u32                        digital_filter_width_ns; /*    24     4 */
	u32                        analog_filter_cutoff_freq_hz; /*    28     4 */

	/* size: 32, cachelines: 1, members: 8 */
	/* last cacheline: 32 bytes */
};
struct trace_event_raw_i2c_write {
	struct trace_entry         ent;                  /*     0     8 */
	int                        adapter_nr;           /*     8     4 */
	__u16                      msg_nr;               /*    12     2 */
	__u16                      addr;                 /*    14     2 */
	__u16                      flags;                /*    16     2 */
	__u16                      len;                  /*    18     2 */
	u32                        __data_loc_buf;       /*    20     4 */
	char                       __data[];             /*    24     0 */

	/* size: 24, cachelines: 1, members: 8 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_i2c_read {
	struct trace_entry         ent;                  /*     0     8 */
	int                        adapter_nr;           /*     8     4 */
	__u16                      msg_nr;               /*    12     2 */
	__u16                      addr;                 /*    14     2 */
	__u16                      flags;                /*    16     2 */
	__u16                      len;                  /*    18     2 */
	char                       __data[];             /*    20     0 */

	/* size: 20, cachelines: 1, members: 7 */
	/* last cacheline: 20 bytes */
};
struct trace_event_raw_i2c_reply {
	struct trace_entry         ent;                  /*     0     8 */
	int                        adapter_nr;           /*     8     4 */
	__u16                      msg_nr;               /*    12     2 */
	__u16                      addr;                 /*    14     2 */
	__u16                      flags;                /*    16     2 */
	__u16                      len;                  /*    18     2 */
	u32                        __data_loc_buf;       /*    20     4 */
	char                       __data[];             /*    24     0 */

	/* size: 24, cachelines: 1, members: 8 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_i2c_result {
	struct trace_entry         ent;                  /*     0     8 */
	int                        adapter_nr;           /*     8     4 */
	__u16                      nr_msgs;              /*    12     2 */
	__s16                      ret;                  /*    14     2 */
	char                       __data[];             /*    16     0 */

	/* size: 16, cachelines: 1, members: 5 */
	/* last cacheline: 16 bytes */
};
struct trace_event_data_offsets_i2c_write {
	u32                        buf;                  /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct trace_event_data_offsets_i2c_read {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_i2c_reply {
	u32                        buf;                  /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct trace_event_data_offsets_i2c_result {

	/* size: 0, cachelines: 0, members: 0 */
};
struct i2c_dummy_devres {
	struct i2c_client *        client;               /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct i2c_cmd_arg {
	unsigned int               cmd;                  /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     arg;                  /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct i2c_smbus_alert_setup {
	int                        irq;                  /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct trace_event_raw_smbus_write {
	struct trace_entry         ent;                  /*     0     8 */
	int                        adapter_nr;           /*     8     4 */
	__u16                      addr;                 /*    12     2 */
	__u16                      flags;                /*    14     2 */
	__u8                       command;              /*    16     1 */
	__u8                       len;                  /*    17     1 */

	/* XXX 2 bytes hole, try to pack */

	__u32                      protocol;             /*    20     4 */
	__u8                       buf[34];              /*    24    34 */
	char                       __data[];             /*    58     0 */

	/* size: 60, cachelines: 1, members: 9 */
	/* sum members: 56, holes: 1, sum holes: 2 */
	/* padding: 2 */
	/* last cacheline: 60 bytes */
};
struct trace_event_raw_smbus_read {
	struct trace_entry         ent;                  /*     0     8 */
	int                        adapter_nr;           /*     8     4 */
	__u16                      flags;                /*    12     2 */
	__u16                      addr;                 /*    14     2 */
	__u8                       command;              /*    16     1 */

	/* XXX 3 bytes hole, try to pack */

	__u32                      protocol;             /*    20     4 */
	__u8                       buf[34];              /*    24    34 */
	char                       __data[];             /*    58     0 */

	/* size: 60, cachelines: 1, members: 8 */
	/* sum members: 55, holes: 1, sum holes: 3 */
	/* padding: 2 */
	/* last cacheline: 60 bytes */
};
struct trace_event_raw_smbus_reply {
	struct trace_entry         ent;                  /*     0     8 */
	int                        adapter_nr;           /*     8     4 */
	__u16                      addr;                 /*    12     2 */
	__u16                      flags;                /*    14     2 */
	__u8                       command;              /*    16     1 */
	__u8                       len;                  /*    17     1 */

	/* XXX 2 bytes hole, try to pack */

	__u32                      protocol;             /*    20     4 */
	__u8                       buf[34];              /*    24    34 */
	char                       __data[];             /*    58     0 */

	/* size: 60, cachelines: 1, members: 9 */
	/* sum members: 56, holes: 1, sum holes: 2 */
	/* padding: 2 */
	/* last cacheline: 60 bytes */
};
struct trace_event_raw_smbus_result {
	struct trace_entry         ent;                  /*     0     8 */
	int                        adapter_nr;           /*     8     4 */
	__u16                      addr;                 /*    12     2 */
	__u16                      flags;                /*    14     2 */
	__u8                       read_write;           /*    16     1 */
	__u8                       command;              /*    17     1 */
	__s16                      res;                  /*    18     2 */
	__u32                      protocol;             /*    20     4 */
	char                       __data[];             /*    24     0 */

	/* size: 24, cachelines: 1, members: 9 */
	/* last cacheline: 24 bytes */
};
struct trace_event_data_offsets_smbus_write {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_smbus_read {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_smbus_reply {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_smbus_result {

	/* size: 0, cachelines: 0, members: 0 */
};
struct i2c_acpi_handler_data {
	struct acpi_connection_info info;                /*     0    16 */

	/* XXX last struct has 5 bytes of padding */

	struct i2c_adapter *       adapter;              /*    16     8 */

	/* size: 24, cachelines: 1, members: 2 */
	/* paddings: 1, sum paddings: 5 */
	/* last cacheline: 24 bytes */
};
struct gsb_buffer {
	u8                         status;               /*     0     1 */
	u8                         len;                  /*     1     1 */
	union {
		u16                wdata;                /*     2     2 */
		u8                 bdata;                /*     2     1 */
		u8                 data[0];              /*     2     0 */
	};                                               /*     2     2 */

	/* size: 4, cachelines: 1, members: 3 */
	/* last cacheline: 4 bytes */
};
struct i2c_acpi_lookup {
	struct i2c_board_info *    info;                 /*     0     8 */
	acpi_handle                adapter_handle;       /*     8     8 */
	acpi_handle                device_handle;        /*    16     8 */
	acpi_handle                search_handle;        /*    24     8 */
	int                        n;                    /*    32     4 */
	int                        index;                /*    36     4 */
	u32                        speed;                /*    40     4 */
	u32                        min_speed;            /*    44     4 */
	u32                        force_speed;          /*    48     4 */

	/* size: 56, cachelines: 1, members: 9 */
	/* padding: 4 */
	/* last cacheline: 56 bytes */
};
struct pps_ktime {
	__s64                      sec;                  /*     0     8 */
	__s32                      nsec;                 /*     8     4 */
	__u32                      flags;                /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct pps_ktime_compat {
	__s64                      sec;                  /*     0     8 */
	__s32                      nsec;                 /*     8     4 */
	__u32                      flags;                /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(4)));
struct pps_kinfo {
	__u32                      assert_sequence;      /*     0     4 */
	__u32                      clear_sequence;       /*     4     4 */
	struct pps_ktime           assert_tu;            /*     8    16 */
	struct pps_ktime           clear_tu;             /*    24    16 */
	int                        current_mode;         /*    40     4 */

	/* size: 48, cachelines: 1, members: 5 */
	/* padding: 4 */
	/* last cacheline: 48 bytes */
};
struct pps_kinfo_compat {
	__u32                      assert_sequence;      /*     0     4 */
	__u32                      clear_sequence;       /*     4     4 */
	struct pps_ktime_compat    assert_tu __attribute__((__aligned__(4))); /*     8    16 */
	struct pps_ktime_compat    clear_tu __attribute__((__aligned__(4))); /*    24    16 */
	int                        current_mode;         /*    40     4 */

	/* size: 44, cachelines: 1, members: 5 */
	/* forced alignments: 2 */
	/* last cacheline: 44 bytes */
} __attribute__((__packed__));
struct pps_kparams {
	int                        api_version;          /*     0     4 */
	int                        mode;                 /*     4     4 */
	struct pps_ktime           assert_off_tu;        /*     8    16 */
	struct pps_ktime           clear_off_tu;         /*    24    16 */

	/* size: 40, cachelines: 1, members: 4 */
	/* last cacheline: 40 bytes */
};
struct pps_fdata {
	struct pps_kinfo           info;                 /*     0    48 */

	/* XXX last struct has 4 bytes of padding */

	struct pps_ktime           timeout;              /*    48    16 */

	/* size: 64, cachelines: 1, members: 2 */
	/* paddings: 1, sum paddings: 4 */
};
struct pps_fdata_compat {
	struct pps_kinfo_compat    info __attribute__((__aligned__(4))); /*     0    44 */
	struct pps_ktime_compat    timeout __attribute__((__aligned__(4))); /*    44    16 */

	/* size: 60, cachelines: 1, members: 2 */
	/* forced alignments: 2 */
	/* last cacheline: 60 bytes */
} __attribute__((__packed__));
struct pps_bind_args {
	int                        tsformat;             /*     0     4 */
	int                        edge;                 /*     4     4 */
	int                        consumer;             /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct pps_source_info {
	char                       name[32];             /*     0    32 */
	char                       path[32];             /*    32    32 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        mode;                 /*    64     4 */

	/* XXX 4 bytes hole, try to pack */

	void                       (*echo)(struct pps_device *, int, void *); /*    72     8 */
	struct module *            owner;                /*    80     8 */
	struct device *            dev;                  /*    88     8 */

	/* size: 96, cachelines: 2, members: 6 */
	/* sum members: 92, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct pps_device {
	struct pps_source_info     info;                 /*     0    96 */
	/* --- cacheline 1 boundary (64 bytes) was 32 bytes ago --- */
	struct pps_kparams         params;               /*    96    40 */
	/* --- cacheline 2 boundary (128 bytes) was 8 bytes ago --- */
	__u32                      assert_sequence;      /*   136     4 */
	__u32                      clear_sequence;       /*   140     4 */
	struct pps_ktime           assert_tu;            /*   144    16 */
	struct pps_ktime           clear_tu;             /*   160    16 */
	int                        current_mode;         /*   176     4 */
	unsigned int               last_ev;              /*   180     4 */
	wait_queue_head_t          queue;                /*   184    24 */
	/* --- cacheline 3 boundary (192 bytes) was 16 bytes ago --- */
	unsigned int               id;                   /*   208     4 */

	/* XXX 4 bytes hole, try to pack */

	const void  *              lookup_cookie;        /*   216     8 */
	struct cdev                cdev;                 /*   224   136 */
	/* --- cacheline 5 boundary (320 bytes) was 40 bytes ago --- */
	struct device *            dev;                  /*   360     8 */
	struct fasync_struct *     async_queue;          /*   368     8 */
	spinlock_t                 lock;                 /*   376     4 */

	/* size: 384, cachelines: 6, members: 15 */
	/* sum members: 376, holes: 1, sum holes: 4 */
	/* padding: 4 */
};
struct pps_event_time {
	struct timespec64          ts_real;              /*     0    16 */

	/* size: 16, cachelines: 1, members: 1 */
	/* last cacheline: 16 bytes */
};
struct ptp_extts_event {
	struct ptp_clock_time      t;                    /*     0    16 */
	unsigned int               index;                /*    16     4 */
	unsigned int               flags;                /*    20     4 */
	unsigned int               rsv[2];               /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct ptp_clock_event {
	int                        type;                 /*     0     4 */
	int                        index;                /*     4     4 */
	union {
		u64                timestamp;            /*     8     8 */
		struct pps_event_time pps_times;         /*     8    16 */
	};                                               /*     8    16 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct timestamp_event_queue {
	struct ptp_extts_event     buf[128];             /*     0  4096 */
	/* --- cacheline 64 boundary (4096 bytes) --- */
	int                        head;                 /*  4096     4 */
	int                        tail;                 /*  4100     4 */
	spinlock_t                 lock;                 /*  4104     4 */

	/* size: 4112, cachelines: 65, members: 4 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct ptp_clock {
	struct posix_clock         clock;                /*     0   384 */
	/* --- cacheline 6 boundary (384 bytes) --- */
	struct device              dev __attribute__((__aligned__(8))); /*   384  1000 */
	/* --- cacheline 21 boundary (1344 bytes) was 40 bytes ago --- */
	struct ptp_clock_info *    info;                 /*  1384     8 */
	dev_t                      devid;                /*  1392     4 */
	int                        index;                /*  1396     4 */
	struct pps_device *        pps_source;           /*  1400     8 */
	/* --- cacheline 22 boundary (1408 bytes) --- */
	long int                   dialed_frequency;     /*  1408     8 */
	struct timestamp_event_queue tsevq;              /*  1416  4112 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 86 boundary (5504 bytes) was 24 bytes ago --- */
	struct mutex               tsevq_mux;            /*  5528    32 */
	struct mutex               pincfg_mux;           /*  5560    32 */
	/* --- cacheline 87 boundary (5568 bytes) was 24 bytes ago --- */
	wait_queue_head_t          tsev_wq;              /*  5592    24 */
	int                        defunct;              /*  5616     4 */

	/* XXX 4 bytes hole, try to pack */

	struct device_attribute *  pin_dev_attr;         /*  5624     8 */
	/* --- cacheline 88 boundary (5632 bytes) --- */
	struct attribute * *       pin_attr;             /*  5632     8 */
	struct attribute_group     pin_attr_group;       /*  5640    40 */
	const struct attribute_group  * pin_attr_groups[2]; /*  5680    16 */
	/* --- cacheline 89 boundary (5696 bytes) --- */
	struct kthread_worker *    kworker;              /*  5696     8 */
	struct kthread_delayed_work aux_work;            /*  5704   112 */
	/* --- cacheline 90 boundary (5760 bytes) was 56 bytes ago --- */
	u64                        kabi_reserved1;       /*  5816     8 */
	/* --- cacheline 91 boundary (5824 bytes) --- */
	u64                        kabi_reserved2;       /*  5824     8 */
	u64                        kabi_reserved3;       /*  5832     8 */
	u64                        kabi_reserved4;       /*  5840     8 */

	/* size: 5848, cachelines: 92, members: 22 */
	/* sum members: 5844, holes: 1, sum holes: 4 */
	/* paddings: 1, sum paddings: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct ptp_clock_caps {
	int                        max_adj;              /*     0     4 */
	int                        n_alarm;              /*     4     4 */
	int                        n_ext_ts;             /*     8     4 */
	int                        n_per_out;            /*    12     4 */
	int                        pps;                  /*    16     4 */
	int                        n_pins;               /*    20     4 */
	int                        cross_timestamping;   /*    24     4 */
	int                        adjust_phase;         /*    28     4 */
	int                        rsv[12];              /*    32    48 */

	/* size: 80, cachelines: 2, members: 9 */
	/* last cacheline: 16 bytes */
};
struct ptp_sys_offset {
	unsigned int               n_samples;            /*     0     4 */
	unsigned int               rsv[3];               /*     4    12 */
	struct ptp_clock_time      ts[51];               /*    16   816 */

	/* size: 832, cachelines: 13, members: 3 */
};
struct ptp_sys_offset_extended {
	unsigned int               n_samples;            /*     0     4 */
	unsigned int               rsv[3];               /*     4    12 */
	struct ptp_clock_time      ts[25][3];            /*    16  1200 */

	/* size: 1216, cachelines: 19, members: 3 */
};
struct ptp_sys_offset_precise {
	struct ptp_clock_time      device;               /*     0    16 */
	struct ptp_clock_time      sys_realtime;         /*    16    16 */
	struct ptp_clock_time      sys_monoraw;          /*    32    16 */
	unsigned int               rsv[4];               /*    48    16 */

	/* size: 64, cachelines: 1, members: 4 */
};
struct power_supply_battery_ocv_table {
	int                        ocv;                  /*     0     4 */
	int                        capacity;             /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct power_supply_resistance_temp_table {
	int                        temp;                 /*     0     4 */
	int                        resistance;           /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct power_supply_battery_info {
	int                        energy_full_design_uwh; /*     0     4 */
	int                        charge_full_design_uah; /*     4     4 */
	int                        voltage_min_design_uv; /*     8     4 */
	int                        voltage_max_design_uv; /*    12     4 */
	int                        tricklecharge_current_ua; /*    16     4 */
	int                        precharge_current_ua; /*    20     4 */
	int                        precharge_voltage_max_uv; /*    24     4 */
	int                        charge_term_current_ua; /*    28     4 */
	int                        charge_restart_voltage_uv; /*    32     4 */
	int                        overvoltage_limit_uv; /*    36     4 */
	int                        constant_charge_current_max_ua; /*    40     4 */
	int                        constant_charge_voltage_max_uv; /*    44     4 */
	int                        factory_internal_resistance_uohm; /*    48     4 */
	int                        ocv_temp[20];         /*    52    80 */
	/* --- cacheline 2 boundary (128 bytes) was 4 bytes ago --- */
	int                        temp_ambient_alert_min; /*   132     4 */
	int                        temp_ambient_alert_max; /*   136     4 */
	int                        temp_alert_min;       /*   140     4 */
	int                        temp_alert_max;       /*   144     4 */
	int                        temp_min;             /*   148     4 */
	int                        temp_max;             /*   152     4 */

	/* XXX 4 bytes hole, try to pack */

	struct power_supply_battery_ocv_table * ocv_table[20]; /*   160   160 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	int                        ocv_table_size[20];   /*   320    80 */
	/* --- cacheline 6 boundary (384 bytes) was 16 bytes ago --- */
	struct power_supply_resistance_temp_table * resist_table; /*   400     8 */
	int                        resist_table_size;    /*   408     4 */

	/* size: 416, cachelines: 7, members: 24 */
	/* sum members: 408, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct psy_am_i_supplied_data {
	struct power_supply *      psy;                  /*     0     8 */
	unsigned int               count;                /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct power_supply_attr {
	const char  *              prop_name;            /*     0     8 */
	char                       attr_name[31];        /*     8    31 */

	/* XXX 1 byte hole, try to pack */

	struct device_attribute    dev_attr;             /*    40    32 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	const char  * const *      text_values;          /*    72     8 */
	int                        text_values_len;      /*    80     4 */

	/* size: 88, cachelines: 2, members: 5 */
	/* sum members: 83, holes: 1, sum holes: 1 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct hwmon_ops {
	umode_t                    (*is_visible)(const void  *, enum hwmon_sensor_types, u32, int); /*     0     8 */
	int                        (*read)(struct device *, enum hwmon_sensor_types, u32, int, long int *); /*     8     8 */
	int                        (*read_string)(struct device *, enum hwmon_sensor_types, u32, int, const char  * *); /*    16     8 */
	int                        (*write)(struct device *, enum hwmon_sensor_types, u32, int, long int); /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct hwmon_channel_info {
	enum hwmon_sensor_types    type;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	const u32  *               config;               /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct hwmon_chip_info {
	const struct hwmon_ops  *  ops;                  /*     0     8 */
	const struct hwmon_channel_info  * * info;       /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct power_supply_hwmon {
	struct power_supply *      psy;                  /*     0     8 */
	long unsigned int *        props;                /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct hwmon_type_attr_list {
	const u32  *               attrs;                /*     0     8 */
	size_t                     n_attrs;              /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct trace_event_raw_hwmon_attr_class {
	struct trace_entry         ent;                  /*     0     8 */
	int                        index;                /*     8     4 */
	u32                        __data_loc_attr_name; /*    12     4 */
	long int                   val;                  /*    16     8 */
	char                       __data[];             /*    24     0 */

	/* size: 24, cachelines: 1, members: 5 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_hwmon_attr_show_string {
	struct trace_entry         ent;                  /*     0     8 */
	int                        index;                /*     8     4 */
	u32                        __data_loc_attr_name; /*    12     4 */
	u32                        __data_loc_label;     /*    16     4 */
	char                       __data[];             /*    20     0 */

	/* size: 20, cachelines: 1, members: 5 */
	/* last cacheline: 20 bytes */
};
struct trace_event_data_offsets_hwmon_attr_class {
	u32                        attr_name;            /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct trace_event_data_offsets_hwmon_attr_show_string {
	u32                        attr_name;            /*     0     4 */
	u32                        label;                /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct hwmon_device {
	const char  *              name;                 /*     0     8 */
	struct device              dev __attribute__((__aligned__(8))); /*     8  1000 */
	/* --- cacheline 15 boundary (960 bytes) was 48 bytes ago --- */
	const struct hwmon_chip_info  * chip;            /*  1008     8 */
	struct list_head           tzdata;               /*  1016    16 */
	/* --- cacheline 16 boundary (1024 bytes) was 8 bytes ago --- */
	struct attribute_group     group;                /*  1032    40 */
	const struct attribute_group  * * groups;        /*  1072     8 */

	/* size: 1080, cachelines: 17, members: 6 */
	/* forced alignments: 1 */
	/* last cacheline: 56 bytes */
} __attribute__((__aligned__(8)));
struct hwmon_device_attribute {
	struct device_attribute    dev_attr;             /*     0    32 */
	const struct hwmon_ops  *  ops;                  /*    32     8 */
	enum hwmon_sensor_types    type;                 /*    40     4 */
	u32                        attr;                 /*    44     4 */
	int                        index;                /*    48     4 */
	char                       name[32];             /*    52    32 */

	/* size: 88, cachelines: 2, members: 6 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct thermal_attr {
	struct device_attribute    attr;                 /*     0    32 */
	char                       name[20];             /*    32    20 */

	/* size: 56, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 56 bytes */
};
struct trace_event_raw_thermal_temperature {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        __data_loc_thermal_zone; /*     8     4 */
	int                        id;                   /*    12     4 */
	int                        temp_prev;            /*    16     4 */
	int                        temp;                 /*    20     4 */
	char                       __data[];             /*    24     0 */

	/* size: 24, cachelines: 1, members: 6 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_cdev_update {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        __data_loc_type;      /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          target;               /*    16     8 */
	char                       __data[];             /*    24     0 */

	/* size: 24, cachelines: 1, members: 4 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_thermal_zone_trip {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        __data_loc_thermal_zone; /*     8     4 */
	int                        id;                   /*    12     4 */
	int                        trip;                 /*    16     4 */
	enum thermal_trip_type     trip_type;            /*    20     4 */
	char                       __data[];             /*    24     0 */

	/* size: 24, cachelines: 1, members: 6 */
	/* last cacheline: 24 bytes */
};
struct trace_event_data_offsets_thermal_temperature {
	u32                        thermal_zone;         /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct trace_event_data_offsets_cdev_update {
	u32                        type;                 /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct trace_event_data_offsets_thermal_zone_trip {
	u32                        thermal_zone;         /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct thermal_instance {
	int                        id;                   /*     0     4 */
	char                       name[20];             /*     4    20 */
	struct thermal_zone_device * tz;                 /*    24     8 */
	struct thermal_cooling_device * cdev;            /*    32     8 */
	int                        trip;                 /*    40     4 */
	bool                       initialized;          /*    44     1 */

	/* XXX 3 bytes hole, try to pack */

	long unsigned int          upper;                /*    48     8 */
	long unsigned int          lower;                /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	long unsigned int          target;               /*    64     8 */
	char                       attr_name[20];        /*    72    20 */

	/* XXX 4 bytes hole, try to pack */

	struct device_attribute    attr;                 /*    96    32 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	char                       weight_attr_name[20]; /*   128    20 */

	/* XXX 4 bytes hole, try to pack */

	struct device_attribute    weight_attr;          /*   152    32 */
	struct list_head           tz_node;              /*   184    16 */
	/* --- cacheline 3 boundary (192 bytes) was 8 bytes ago --- */
	struct list_head           cdev_node;            /*   200    16 */
	unsigned int               weight;               /*   216     4 */

	/* size: 224, cachelines: 4, members: 16 */
	/* sum members: 209, holes: 3, sum holes: 11 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct genl_dumpit_info {
	const struct genl_family  * family;              /*     0     8 */
	struct genl_ops            op;                   /*     8    80 */
	/* --- cacheline 1 boundary (64 bytes) was 24 bytes ago --- */
	struct nlattr * *          attrs;                /*    88     8 */

	/* size: 96, cachelines: 2, members: 3 */
	/* last cacheline: 32 bytes */
};
struct thermal_genl_cpu_caps {
	int                        cpu;                  /*     0     4 */
	int                        performance;          /*     4     4 */
	int                        efficiency;           /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct param {
	struct nlattr * *          attrs;                /*     0     8 */
	struct sk_buff *           msg;                  /*     8     8 */
	const char  *              name;                 /*    16     8 */
	int                        tz_id;                /*    24     4 */
	int                        cdev_id;              /*    28     4 */
	int                        trip_id;              /*    32     4 */
	int                        trip_temp;            /*    36     4 */
	int                        trip_type;            /*    40     4 */
	int                        trip_hyst;            /*    44     4 */
	int                        temp;                 /*    48     4 */
	int                        cdev_state;           /*    52     4 */
	int                        cdev_max_state;       /*    56     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	struct thermal_genl_cpu_caps * cpu_capabilities; /*    64     8 */
	int                        cpu_capabilities_count; /*    72     4 */

	/* size: 80, cachelines: 2, members: 14 */
	/* sum members: 72, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct thermal_hwmon_device {
	char                       type[20];             /*     0    20 */

	/* XXX 4 bytes hole, try to pack */

	struct device *            device;               /*    24     8 */
	int                        count;                /*    32     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           tz_list;              /*    40    16 */
	struct list_head           node;                 /*    56    16 */

	/* size: 72, cachelines: 2, members: 5 */
	/* sum members: 64, holes: 2, sum holes: 8 */
	/* last cacheline: 8 bytes */
};
struct thermal_hwmon_attr {
	struct device_attribute    attr;                 /*     0    32 */
	char                       name[16];             /*    32    16 */

	/* size: 48, cachelines: 1, members: 2 */
	/* last cacheline: 48 bytes */
};
struct thermal_hwmon_temp {
	struct list_head           hwmon_node;           /*     0    16 */
	struct thermal_zone_device * tz;                 /*    16     8 */
	struct thermal_hwmon_attr  temp_input;           /*    24    48 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	struct thermal_hwmon_attr  temp_crit;            /*    72    48 */

	/* size: 120, cachelines: 2, members: 4 */
	/* last cacheline: 56 bytes */
};
struct _thermal_state {
	u64                        next_check;           /*     0     8 */
	u64                        last_interrupt_time;  /*     8     8 */
	struct delayed_work        therm_work;           /*    16   192 */
	/* --- cacheline 3 boundary (192 bytes) was 16 bytes ago --- */
	long unsigned int          count;                /*   208     8 */
	long unsigned int          last_count;           /*   216     8 */
	long unsigned int          max_time_ms;          /*   224     8 */
	long unsigned int          total_time_ms;        /*   232     8 */
	bool                       rate_control_active;  /*   240     1 */
	bool                       new_event;            /*   241     1 */
	u8                         level;                /*   242     1 */
	u8                         sample_index;         /*   243     1 */
	u8                         sample_count;         /*   244     1 */
	u8                         average;              /*   245     1 */
	u8                         baseline_temp;        /*   246     1 */
	u8                         temp_samples[3];      /*   247     3 */

	/* size: 256, cachelines: 4, members: 15 */
	/* padding: 6 */
};
struct thermal_state {
	struct _thermal_state      core_throttle;        /*     0   256 */

	/* XXX last struct has 6 bytes of padding */

	/* --- cacheline 4 boundary (256 bytes) --- */
	struct _thermal_state      core_power_limit;     /*   256   256 */

	/* XXX last struct has 6 bytes of padding */

	/* --- cacheline 8 boundary (512 bytes) --- */
	struct _thermal_state      package_throttle;     /*   512   256 */

	/* XXX last struct has 6 bytes of padding */

	/* --- cacheline 12 boundary (768 bytes) --- */
	struct _thermal_state      package_power_limit;  /*   768   256 */

	/* XXX last struct has 6 bytes of padding */

	/* --- cacheline 16 boundary (1024 bytes) --- */
	struct _thermal_state      core_thresh0;         /*  1024   256 */

	/* XXX last struct has 6 bytes of padding */

	/* --- cacheline 20 boundary (1280 bytes) --- */
	struct _thermal_state      core_thresh1;         /*  1280   256 */

	/* XXX last struct has 6 bytes of padding */

	/* --- cacheline 24 boundary (1536 bytes) --- */
	struct _thermal_state      pkg_thresh0;          /*  1536   256 */

	/* XXX last struct has 6 bytes of padding */

	/* --- cacheline 28 boundary (1792 bytes) --- */
	struct _thermal_state      pkg_thresh1;          /*  1792   256 */

	/* XXX last struct has 6 bytes of padding */

	/* size: 2048, cachelines: 32, members: 8 */
	/* paddings: 8, sum paddings: 48 */
};
union hfi_capabilities {
	struct {
		u8                 performance:1;      /*     0: 0  1 */
		u8                 energy_efficiency:1; /*     0: 1  1 */
		u8                 __reserved:6;       /*     0: 2  1 */
	} split;                                       /*     0     1 */
	u8                         bits;               /*     0     1 */
};
union cpuid6_edx {
	struct {
		union hfi_capabilities capabilities;   /*     0     1 */

		/* Bitfield combined with previous fields */

		u32                table_pages:4;      /*     0: 8  4 */
		u32                __reserved:4;       /*     0:12  4 */
		s32                index:16;           /*     0:16  4 */
	} split;                                       /*     0     4 */
	u32                        full;               /*     0     4 */
};
struct hfi_cpu_data {
	u8                         perf_cap;             /*     0     1 */
	u8                         ee_cap;               /*     1     1 */

	/* size: 2, cachelines: 1, members: 2 */
	/* last cacheline: 2 bytes */
};
struct hfi_instance {
	union {
		void *             local_table;          /*     0     8 */
		u64 *              timestamp;            /*     0     8 */
	};                                               /*     0     8 */
	void *                     hdr;                  /*     8     8 */
	void *                     data;                 /*    16     8 */
	cpumask_var_t              cpus;                 /*    24     8 */
	void *                     hw_table;             /*    32     8 */
	struct delayed_work        update_work;          /*    40   192 */
	/* --- cacheline 3 boundary (192 bytes) was 40 bytes ago --- */
	raw_spinlock_t             table_lock;           /*   232     4 */
	raw_spinlock_t             event_lock;           /*   236     4 */

	/* size: 240, cachelines: 4, members: 8 */
	/* last cacheline: 48 bytes */
};
struct hfi_features {
	unsigned int               nr_table_pages;       /*     0     4 */
	unsigned int               cpu_stride;           /*     4     4 */
	unsigned int               hdr_size;             /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct hfi_cpu_info {
	s16                        index;                /*     0     2 */

	/* XXX 6 bytes hole, try to pack */

	struct hfi_instance *      hfi_instance;         /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 10, holes: 1, sum holes: 6 */
	/* last cacheline: 16 bytes */
};
struct watchdog_info {
	__u32                      options;              /*     0     4 */
	__u32                      firmware_version;     /*     4     4 */
	__u8                       identity[32];         /*     8    32 */

	/* size: 40, cachelines: 1, members: 3 */
	/* last cacheline: 40 bytes */
};
struct watchdog_ops {
	struct module *            owner;                /*     0     8 */
	int                        (*start)(struct watchdog_device *); /*     8     8 */
	int                        (*stop)(struct watchdog_device *); /*    16     8 */
	int                        (*ping)(struct watchdog_device *); /*    24     8 */
	unsigned int               (*status)(struct watchdog_device *); /*    32     8 */
	int                        (*set_timeout)(struct watchdog_device *, unsigned int); /*    40     8 */
	int                        (*set_pretimeout)(struct watchdog_device *, unsigned int); /*    48     8 */
	unsigned int               (*get_timeleft)(struct watchdog_device *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*restart)(struct watchdog_device *, long unsigned int, void *); /*    64     8 */
	long int                   (*ioctl)(struct watchdog_device *, unsigned int, long unsigned int); /*    72     8 */

	/* size: 80, cachelines: 2, members: 10 */
	/* last cacheline: 16 bytes */
};
struct watchdog_device {
	int                        id;                   /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct device *            parent;               /*     8     8 */
	const struct attribute_group  * * groups;        /*    16     8 */
	const struct watchdog_info  * info;              /*    24     8 */
	const struct watchdog_ops  * ops;                /*    32     8 */
	const struct watchdog_governor  * gov;           /*    40     8 */
	unsigned int               bootstatus;           /*    48     4 */
	unsigned int               timeout;              /*    52     4 */
	unsigned int               pretimeout;           /*    56     4 */
	unsigned int               min_timeout;          /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	unsigned int               max_timeout;          /*    64     4 */
	unsigned int               min_hw_heartbeat_ms;  /*    68     4 */
	unsigned int               max_hw_heartbeat_ms;  /*    72     4 */

	/* XXX 4 bytes hole, try to pack */

	struct notifier_block      reboot_nb;            /*    80    24 */

	/* XXX last struct has 4 bytes of padding */

	struct notifier_block      restart_nb;           /*   104    24 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 2 boundary (128 bytes) --- */
	void *                     driver_data;          /*   128     8 */
	struct watchdog_core_data * wd_data;             /*   136     8 */
	long unsigned int          status;               /*   144     8 */
	struct list_head           deferred;             /*   152    16 */

	/* size: 168, cachelines: 3, members: 19 */
	/* sum members: 160, holes: 2, sum holes: 8 */
	/* paddings: 2, sum paddings: 8 */
	/* last cacheline: 40 bytes */
};
struct watchdog_governor {
	const const char           name[20];             /*     0    20 */

	/* XXX 4 bytes hole, try to pack */

	void                       (*pretimeout)(struct watchdog_device *); /*    24     8 */

	/* size: 32, cachelines: 1, members: 2 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct watchdog_core_data {
	struct device              dev __attribute__((__aligned__(8))); /*     0  1000 */
	/* --- cacheline 15 boundary (960 bytes) was 40 bytes ago --- */
	struct cdev                cdev;                 /*  1000   136 */
	/* --- cacheline 17 boundary (1088 bytes) was 48 bytes ago --- */
	struct watchdog_device *   wdd;                  /*  1136     8 */
	struct mutex               lock;                 /*  1144    32 */
	/* --- cacheline 18 boundary (1152 bytes) was 24 bytes ago --- */
	ktime_t                    last_keepalive;       /*  1176     8 */
	ktime_t                    last_hw_keepalive;    /*  1184     8 */
	ktime_t                    open_deadline;        /*  1192     8 */
	struct hrtimer             timer __attribute__((__aligned__(8))); /*  1200    96 */
	/* --- cacheline 20 boundary (1280 bytes) was 16 bytes ago --- */
	struct kthread_work        work;                 /*  1296    40 */

	/* XXX last struct has 4 bytes of padding */

	long unsigned int          status;               /*  1336     8 */

	/* size: 1344, cachelines: 21, members: 10 */
	/* paddings: 1, sum paddings: 4 */
	/* forced alignments: 2 */
} __attribute__((__aligned__(8)));
struct mdp_device_descriptor_s {
	__u32                      number;               /*     0     4 */
	__u32                      major;                /*     4     4 */
	__u32                      minor;                /*     8     4 */
	__u32                      raid_disk;            /*    12     4 */
	__u32                      state;                /*    16     4 */
	__u32                      reserved[27];         /*    20   108 */

	/* size: 128, cachelines: 2, members: 6 */
};
struct mdp_superblock_s {
	__u32                      md_magic;             /*     0     4 */
	__u32                      major_version;        /*     4     4 */
	__u32                      minor_version;        /*     8     4 */
	__u32                      patch_version;        /*    12     4 */
	__u32                      gvalid_words;         /*    16     4 */
	__u32                      set_uuid0;            /*    20     4 */
	__u32                      ctime;                /*    24     4 */
	__u32                      level;                /*    28     4 */
	__u32                      size;                 /*    32     4 */
	__u32                      nr_disks;             /*    36     4 */
	__u32                      raid_disks;           /*    40     4 */
	__u32                      md_minor;             /*    44     4 */
	__u32                      not_persistent;       /*    48     4 */
	__u32                      set_uuid1;            /*    52     4 */
	__u32                      set_uuid2;            /*    56     4 */
	__u32                      set_uuid3;            /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	__u32                      gstate_creserved[16]; /*    64    64 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	__u32                      utime;                /*   128     4 */
	__u32                      state;                /*   132     4 */
	__u32                      active_disks;         /*   136     4 */
	__u32                      working_disks;        /*   140     4 */
	__u32                      failed_disks;         /*   144     4 */
	__u32                      spare_disks;          /*   148     4 */
	__u32                      sb_csum;              /*   152     4 */
	__u32                      events_lo;            /*   156     4 */
	__u32                      events_hi;            /*   160     4 */
	__u32                      cp_events_lo;         /*   164     4 */
	__u32                      cp_events_hi;         /*   168     4 */
	__u32                      recovery_cp;          /*   172     4 */
	__u64                      reshape_position;     /*   176     8 */
	__u32                      new_level;            /*   184     4 */
	__u32                      delta_disks;          /*   188     4 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	__u32                      new_layout;           /*   192     4 */
	__u32                      new_chunk;            /*   196     4 */
	__u32                      gstate_sreserved[14]; /*   200    56 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	__u32                      layout;               /*   256     4 */
	__u32                      chunk_size;           /*   260     4 */
	__u32                      root_pv;              /*   264     4 */
	__u32                      root_block;           /*   268     4 */
	__u32                      pstate_reserved[60];  /*   272   240 */
	/* --- cacheline 8 boundary (512 bytes) --- */
	mdp_disk_t                 disks[27];            /*   512  3456 */
	/* --- cacheline 62 boundary (3968 bytes) --- */
	__u32                      reserved[0];          /*  3968     0 */
	mdp_disk_t                 this_disk;            /*  3968   128 */

	/* size: 4096, cachelines: 64, members: 43 */
};
struct mdp_superblock_1 {
	__le32                     magic;                /*     0     4 */
	__le32                     major_version;        /*     4     4 */
	__le32                     feature_map;          /*     8     4 */
	__le32                     pad0;                 /*    12     4 */
	__u8                       set_uuid[16];         /*    16    16 */
	char                       set_name[32];         /*    32    32 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	__le64                     ctime;                /*    64     8 */
	__le32                     level;                /*    72     4 */
	__le32                     layout;               /*    76     4 */
	__le64                     size;                 /*    80     8 */
	__le32                     chunksize;            /*    88     4 */
	__le32                     raid_disks;           /*    92     4 */
	union {
		__le32             bitmap_offset;        /*    96     4 */
		struct {
			__le16     offset;               /*    96     2 */
			__le16     size;                 /*    98     2 */
		} ppl;                                   /*    96     4 */
	};                                               /*    96     4 */
	__le32                     new_level;            /*   100     4 */
	__le64                     reshape_position;     /*   104     8 */
	__le32                     delta_disks;          /*   112     4 */
	__le32                     new_layout;           /*   116     4 */
	__le32                     new_chunk;            /*   120     4 */
	__le32                     new_offset;           /*   124     4 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	__le64                     data_offset;          /*   128     8 */
	__le64                     data_size;            /*   136     8 */
	__le64                     super_offset;         /*   144     8 */
	union {
		__le64             recovery_offset;      /*   152     8 */
		__le64             journal_tail;         /*   152     8 */
	};                                               /*   152     8 */
	__le32                     dev_number;           /*   160     4 */
	__le32                     cnt_corrected_read;   /*   164     4 */
	__u8                       device_uuid[16];      /*   168    16 */
	__u8                       devflags;             /*   184     1 */
	__u8                       bblog_shift;          /*   185     1 */
	__le16                     bblog_size;           /*   186     2 */
	__le32                     bblog_offset;         /*   188     4 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	__le64                     utime;                /*   192     8 */
	__le64                     events;               /*   200     8 */
	__le64                     resync_offset;        /*   208     8 */
	__le32                     sb_csum;              /*   216     4 */
	__le32                     max_dev;              /*   220     4 */
	__u8                       pad3[32];             /*   224    32 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	__le16                     dev_roles[];          /*   256     0 */

	/* size: 256, cachelines: 4, members: 37 */
};
struct mdu_version_s {
	int                        major;                /*     0     4 */
	int                        minor;                /*     4     4 */
	int                        patchlevel;           /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct mdu_array_info_s {
	int                        major_version;        /*     0     4 */
	int                        minor_version;        /*     4     4 */
	int                        patch_version;        /*     8     4 */
	unsigned int               ctime;                /*    12     4 */
	int                        level;                /*    16     4 */
	int                        size;                 /*    20     4 */
	int                        nr_disks;             /*    24     4 */
	int                        raid_disks;           /*    28     4 */
	int                        md_minor;             /*    32     4 */
	int                        not_persistent;       /*    36     4 */
	unsigned int               utime;                /*    40     4 */
	int                        state;                /*    44     4 */
	int                        active_disks;         /*    48     4 */
	int                        working_disks;        /*    52     4 */
	int                        failed_disks;         /*    56     4 */
	int                        spare_disks;          /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        layout;               /*    64     4 */
	int                        chunk_size;           /*    68     4 */

	/* size: 72, cachelines: 2, members: 18 */
	/* last cacheline: 8 bytes */
};
struct mdu_disk_info_s {
	int                        number;               /*     0     4 */
	int                        major;                /*     4     4 */
	int                        minor;                /*     8     4 */
	int                        raid_disk;            /*    12     4 */
	int                        state;                /*    16     4 */

	/* size: 20, cachelines: 1, members: 5 */
	/* last cacheline: 20 bytes */
};
struct mdu_bitmap_file_s {
	char                       pathname[4096];       /*     0  4096 */

	/* size: 4096, cachelines: 64, members: 1 */
};
struct md_cluster_operations {
	int                        (*join)(struct mddev *, int); /*     0     8 */
	int                        (*leave)(struct mddev *); /*     8     8 */
	int                        (*slot_number)(struct mddev *); /*    16     8 */
	int                        (*resync_info_update)(struct mddev *, sector_t, sector_t); /*    24     8 */
	void                       (*resync_info_get)(struct mddev *, sector_t *, sector_t *); /*    32     8 */
	int                        (*metadata_update_start)(struct mddev *); /*    40     8 */
	int                        (*metadata_update_finish)(struct mddev *); /*    48     8 */
	void                       (*metadata_update_cancel)(struct mddev *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*resync_start)(struct mddev *); /*    64     8 */
	int                        (*resync_finish)(struct mddev *); /*    72     8 */
	int                        (*area_resyncing)(struct mddev *, int, sector_t, sector_t); /*    80     8 */
	int                        (*add_new_disk)(struct mddev *, struct md_rdev *); /*    88     8 */
	void                       (*add_new_disk_cancel)(struct mddev *); /*    96     8 */
	int                        (*new_disk_ack)(struct mddev *, bool); /*   104     8 */
	int                        (*remove_disk)(struct mddev *, struct md_rdev *); /*   112     8 */
	void                       (*load_bitmaps)(struct mddev *, int); /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	int                        (*gather_bitmaps)(struct md_rdev *); /*   128     8 */
	int                        (*resize_bitmaps)(struct mddev *, sector_t, sector_t); /*   136     8 */
	int                        (*lock_all_bitmaps)(struct mddev *); /*   144     8 */
	void                       (*unlock_all_bitmaps)(struct mddev *); /*   152     8 */
	void                       (*update_size)(struct mddev *, sector_t); /*   160     8 */

	/* size: 168, cachelines: 3, members: 21 */
	/* last cacheline: 40 bytes */
};
struct mddev {
	void *                     private;              /*     0     8 */
	struct md_personality *    pers;                 /*     8     8 */
	dev_t                      unit;                 /*    16     4 */
	int                        md_minor;             /*    20     4 */
	struct list_head           disks;                /*    24    16 */
	long unsigned int          flags;                /*    40     8 */
	long unsigned int          sb_flags;             /*    48     8 */
	int                        suspended;            /*    56     4 */
	atomic_t                   active_io;            /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        ro;                   /*    64     4 */
	int                        sysfs_active;         /*    68     4 */
	struct gendisk *           gendisk;              /*    72     8 */
	struct kobject             kobj;                 /*    80    96 */
	/* --- cacheline 2 boundary (128 bytes) was 48 bytes ago --- */
	int                        hold_active;          /*   176     4 */
	int                        major_version;        /*   180     4 */
	int                        minor_version;        /*   184     4 */
	int                        patch_version;        /*   188     4 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	int                        persistent;           /*   192     4 */
	int                        external;             /*   196     4 */
	char                       metadata_type[17];    /*   200    17 */

	/* XXX 3 bytes hole, try to pack */

	int                        chunk_sectors;        /*   220     4 */
	time64_t                   ctime;                /*   224     8 */
	time64_t                   utime;                /*   232     8 */
	int                        level;                /*   240     4 */
	int                        layout;               /*   244     4 */
	char                       clevel[16];           /*   248    16 */
	/* --- cacheline 4 boundary (256 bytes) was 8 bytes ago --- */
	int                        raid_disks;           /*   264     4 */
	int                        max_disks;            /*   268     4 */
	sector_t                   dev_sectors;          /*   272     8 */
	sector_t                   array_sectors;        /*   280     8 */
	int                        external_size;        /*   288     4 */

	/* XXX 4 bytes hole, try to pack */

	__u64                      events;               /*   296     8 */
	int                        can_decrease_events;  /*   304     4 */
	char                       uuid[16];             /*   308    16 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 5 boundary (320 bytes) was 8 bytes ago --- */
	sector_t                   reshape_position;     /*   328     8 */
	int                        delta_disks;          /*   336     4 */
	int                        new_level;            /*   340     4 */
	int                        new_layout;           /*   344     4 */
	int                        new_chunk_sectors;    /*   348     4 */
	int                        reshape_backwards;    /*   352     4 */

	/* XXX 4 bytes hole, try to pack */

	struct md_thread *         thread;               /*   360     8 */
	struct md_thread *         sync_thread;          /*   368     8 */
	char *                     last_sync_action;     /*   376     8 */
	/* --- cacheline 6 boundary (384 bytes) --- */
	sector_t                   curr_resync;          /*   384     8 */
	sector_t                   curr_resync_completed; /*   392     8 */
	long unsigned int          resync_mark;          /*   400     8 */
	sector_t                   resync_mark_cnt;      /*   408     8 */
	sector_t                   curr_mark_cnt;        /*   416     8 */
	sector_t                   resync_max_sectors;   /*   424     8 */
	atomic64_t                 resync_mismatches;    /*   432     8 */
	sector_t                   suspend_lo;           /*   440     8 */
	/* --- cacheline 7 boundary (448 bytes) --- */
	sector_t                   suspend_hi;           /*   448     8 */
	int                        sync_speed_min;       /*   456     4 */
	int                        sync_speed_max;       /*   460     4 */
	int                        parallel_resync;      /*   464     4 */
	int                        ok_start_degraded;    /*   468     4 */
	long unsigned int          recovery;             /*   472     8 */
	int                        recovery_disabled;    /*   480     4 */
	int                        in_sync;              /*   484     4 */
	struct mutex               open_mutex;           /*   488    32 */
	/* --- cacheline 8 boundary (512 bytes) was 8 bytes ago --- */
	struct mutex               reconfig_mutex;       /*   520    32 */
	atomic_t                   active;               /*   552     4 */
	atomic_t                   openers;              /*   556     4 */
	int                        changed;              /*   560     4 */
	int                        degraded;             /*   564     4 */
	atomic_t                   recovery_active;      /*   568     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 9 boundary (576 bytes) --- */
	wait_queue_head_t          recovery_wait;        /*   576    24 */
	sector_t                   recovery_cp;          /*   600     8 */
	sector_t                   resync_min;           /*   608     8 */
	sector_t                   resync_max;           /*   616     8 */
	struct kernfs_node *       sysfs_state;          /*   624     8 */
	struct kernfs_node *       sysfs_action;         /*   632     8 */
	/* --- cacheline 10 boundary (640 bytes) --- */
	struct kernfs_node *       sysfs_completed;      /*   640     8 */
	struct kernfs_node *       sysfs_degraded;       /*   648     8 */
	struct kernfs_node *       sysfs_level;          /*   656     8 */
	struct work_struct         del_work;             /*   664    64 */
	/* --- cacheline 11 boundary (704 bytes) was 24 bytes ago --- */
	spinlock_t                 lock;                 /*   728     4 */

	/* XXX 4 bytes hole, try to pack */

	wait_queue_head_t          sb_wait;              /*   736    24 */
	atomic_t                   pending_writes;       /*   760     4 */
	unsigned int               safemode;             /*   764     4 */
	/* --- cacheline 12 boundary (768 bytes) --- */
	unsigned int               safemode_delay;       /*   768     4 */

	/* XXX 4 bytes hole, try to pack */

	struct timer_list          safemode_timer;       /*   776    72 */
	/* --- cacheline 13 boundary (832 bytes) was 16 bytes ago --- */
	struct percpu_ref          writes_pending;       /*   848    16 */
	int                        sync_checkers;        /*   864     4 */

	/* XXX 4 bytes hole, try to pack */

	struct request_queue *     queue;                /*   872     8 */
	struct bitmap *            bitmap;               /*   880     8 */
	struct {
		struct file *      file;                 /*   888     8 */
		/* --- cacheline 14 boundary (896 bytes) --- */
		loff_t             offset;               /*   896     8 */
		long unsigned int  space;                /*   904     8 */
		loff_t             default_offset;       /*   912     8 */
		long unsigned int  default_space;        /*   920     8 */
		struct mutex       mutex;                /*   928    32 */
		/* --- cacheline 15 boundary (960 bytes) --- */
		long unsigned int  chunksize;            /*   960     8 */
		long unsigned int  daemon_sleep;         /*   968     8 */
		long unsigned int  max_write_behind;     /*   976     8 */
		int                external;             /*   984     4 */
		int                nodes;                /*   988     4 */
		char               cluster_name[64];     /*   992    64 */
	} bitmap_info;                                   /*   888   168 */
	/* --- cacheline 16 boundary (1024 bytes) was 32 bytes ago --- */
	atomic_t                   max_corr_read_errors; /*  1056     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           all_mddevs;           /*  1064    16 */
	struct attribute_group *   to_remove;            /*  1080     8 */
	/* --- cacheline 17 boundary (1088 bytes) --- */
	struct bio_set             bio_set;              /*  1088   432 */
	/* --- cacheline 23 boundary (1472 bytes) was 48 bytes ago --- */
	struct bio_set             sync_set;             /*  1520   432 */
	/* --- cacheline 30 boundary (1920 bytes) was 32 bytes ago --- */
	struct bio_set             io_acct_set;          /*  1952   432 */
	/* --- cacheline 37 boundary (2368 bytes) was 16 bytes ago --- */
	struct bio *               flush_bio;            /*  2384     8 */
	atomic_t                   flush_pending;        /*  2392     4 */

	/* XXX 4 bytes hole, try to pack */

	ktime_t                    start_flush;          /*  2400     8 */
	ktime_t                    last_flush;           /*  2408     8 */
	struct work_struct         flush_work;           /*  2416    64 */
	/* --- cacheline 38 boundary (2432 bytes) was 48 bytes ago --- */
	struct work_struct         event_work;           /*  2480    64 */
	/* --- cacheline 39 boundary (2496 bytes) was 48 bytes ago --- */
	mempool_t *                serial_info_pool;     /*  2544     8 */
	void                       (*sync_super)(struct mddev *, struct md_rdev *); /*  2552     8 */
	/* --- cacheline 40 boundary (2560 bytes) --- */
	struct md_cluster_info *   cluster_info;         /*  2560     8 */
	unsigned int               good_device_nr;       /*  2568     4 */
	unsigned int               noio_flag;            /*  2572     4 */
	bool                       has_superblocks:1;    /*  2576: 0  1 */
	bool                       fail_last_dev:1;      /*  2576: 1  1 */
	bool                       serialize_policy:1;   /*  2576: 2  1 */

	/* size: 2584, cachelines: 41, members: 107 */
	/* sum members: 2537, holes: 10, sum holes: 39 */
	/* sum bitfield members: 3 bits (0 bytes) */
	/* padding: 7 */
	/* bit_padding: 5 bits */
	/* last cacheline: 24 bytes */
};
struct md_rdev {
	struct list_head           same_set;             /*     0    16 */
	sector_t                   sectors;              /*    16     8 */
	struct mddev *             mddev;                /*    24     8 */
	long long int              last_events;          /*    32     8 */
	struct block_device *      meta_bdev;            /*    40     8 */
	struct block_device *      bdev;                 /*    48     8 */
	struct page *              sb_page;              /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct page *              bb_page;              /*    64     8 */
	int                        sb_loaded;            /*    72     4 */

	/* XXX 4 bytes hole, try to pack */

	__u64                      sb_events;            /*    80     8 */
	sector_t                   data_offset;          /*    88     8 */
	sector_t                   new_data_offset;      /*    96     8 */
	sector_t                   sb_start;             /*   104     8 */
	int                        sb_size;              /*   112     4 */
	int                        preferred_minor;      /*   116     4 */
	struct kobject             kobj;                 /*   120    96 */
	/* --- cacheline 3 boundary (192 bytes) was 24 bytes ago --- */
	long unsigned int          flags;                /*   216     8 */
	wait_queue_head_t          blocked_wait;         /*   224    24 */
	int                        desc_nr;              /*   248     4 */
	int                        raid_disk;            /*   252     4 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	int                        new_raid_disk;        /*   256     4 */
	int                        saved_raid_disk;      /*   260     4 */
	union {
		sector_t           recovery_offset;      /*   264     8 */
		sector_t           journal_tail;         /*   264     8 */
	};                                               /*   264     8 */
	atomic_t                   nr_pending;           /*   272     4 */
	atomic_t                   read_errors;          /*   276     4 */
	time64_t                   last_read_error;      /*   280     8 */
	atomic_t                   corrected_errors;     /*   288     4 */

	/* XXX 4 bytes hole, try to pack */

	struct serial_in_rdev *    serial;               /*   296     8 */
	struct work_struct         del_work;             /*   304    64 */
	/* --- cacheline 5 boundary (320 bytes) was 48 bytes ago --- */
	struct kernfs_node *       sysfs_state;          /*   368     8 */
	struct kernfs_node *       sysfs_unack_badblocks; /*   376     8 */
	/* --- cacheline 6 boundary (384 bytes) --- */
	struct kernfs_node *       sysfs_badblocks;      /*   384     8 */
	struct badblocks           badblocks;            /*   392    64 */
	/* --- cacheline 7 boundary (448 bytes) was 8 bytes ago --- */
	struct {
		short int          offset;               /*   456     2 */

		/* XXX 2 bytes hole, try to pack */

		unsigned int       size;                 /*   460     4 */
		sector_t           sector;               /*   464     8 */
	} ppl;                                           /*   456    16 */

	/* size: 472, cachelines: 8, members: 34 */
	/* sum members: 464, holes: 2, sum holes: 8 */
	/* last cacheline: 24 bytes */
};
struct serial_in_rdev {
	struct rb_root_cached      serial_rb;            /*     0    16 */
	spinlock_t                 serial_lock;          /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	wait_queue_head_t          serial_io_wait;       /*    24    24 */

	/* size: 48, cachelines: 1, members: 3 */
	/* sum members: 44, holes: 1, sum holes: 4 */
	/* last cacheline: 48 bytes */
};
struct md_personality {
	char *                     name;                 /*     0     8 */
	int                        level;                /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           list;                 /*    16    16 */
	struct module *            owner;                /*    32     8 */
	bool                       (*make_request)(struct mddev *, struct bio *); /*    40     8 */
	int                        (*run)(struct mddev *); /*    48     8 */
	int                        (*start)(struct mddev *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	void                       (*free)(struct mddev *, void *); /*    64     8 */
	void                       (*status)(struct seq_file *, struct mddev *); /*    72     8 */
	void                       (*error_handler)(struct mddev *, struct md_rdev *); /*    80     8 */
	int                        (*hot_add_disk)(struct mddev *, struct md_rdev *); /*    88     8 */
	int                        (*hot_remove_disk)(struct mddev *, struct md_rdev *); /*    96     8 */
	int                        (*spare_active)(struct mddev *); /*   104     8 */
	sector_t                   (*sync_request)(struct mddev *, sector_t, int *); /*   112     8 */
	int                        (*resize)(struct mddev *, sector_t); /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	sector_t                   (*size)(struct mddev *, sector_t, int); /*   128     8 */
	int                        (*check_reshape)(struct mddev *); /*   136     8 */
	int                        (*start_reshape)(struct mddev *); /*   144     8 */
	void                       (*finish_reshape)(struct mddev *); /*   152     8 */
	void                       (*update_reshape_pos)(struct mddev *); /*   160     8 */
	void                       (*quiesce)(struct mddev *, int); /*   168     8 */
	void *                     (*takeover)(struct mddev *); /*   176     8 */
	int                        (*change_consistency_policy)(struct mddev *, const char  *); /*   184     8 */

	/* size: 192, cachelines: 3, members: 23 */
	/* sum members: 188, holes: 1, sum holes: 4 */
};
struct md_thread {
	void                       (*run)(struct md_thread *); /*     0     8 */
	struct mddev *             mddev;                /*     8     8 */
	wait_queue_head_t          wqueue;               /*    16    24 */
	long unsigned int          flags;                /*    40     8 */
	struct task_struct *       tsk;                  /*    48     8 */
	long unsigned int          timeout;              /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	void *                     private;              /*    64     8 */

	/* size: 72, cachelines: 2, members: 7 */
	/* last cacheline: 8 bytes */
};
struct bitmap {
	struct bitmap_counts       counts;               /*     0    48 */
	struct mddev *             mddev;                /*    48     8 */
	__u64                      events_cleared;       /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        need_sync;            /*    64     4 */

	/* XXX 4 bytes hole, try to pack */

	struct bitmap_storage      storage;              /*    72    48 */
	long unsigned int          flags;                /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	int                        allclean;             /*   128     4 */
	atomic_t                   behind_writes;        /*   132     4 */
	long unsigned int          behind_writes_used;   /*   136     8 */
	long unsigned int          daemon_lastrun;       /*   144     8 */
	long unsigned int          last_end_sync;        /*   152     8 */
	atomic_t                   pending_writes;       /*   160     4 */

	/* XXX 4 bytes hole, try to pack */

	wait_queue_head_t          write_wait;           /*   168    24 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	wait_queue_head_t          overflow_wait;        /*   192    24 */
	wait_queue_head_t          behind_wait;          /*   216    24 */
	struct kernfs_node *       sysfs_can_clear;      /*   240     8 */
	int                        cluster_slot;         /*   248     4 */

	/* size: 256, cachelines: 4, members: 17 */
	/* sum members: 244, holes: 2, sum holes: 8 */
	/* padding: 4 */
};
struct md_sysfs_entry {
	struct attribute           attr;                 /*     0    16 */

	/* XXX last struct has 6 bytes of padding */

	ssize_t                    (*show)(struct mddev *, char *); /*    16     8 */
	ssize_t                    (*store)(struct mddev *, const char  *, size_t); /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* paddings: 1, sum paddings: 6 */
	/* last cacheline: 32 bytes */
};
struct md_io_acct {
	struct bio *               orig_bio;             /*     0     8 */
	long unsigned int          start_time;           /*     8     8 */
	struct bio                 bio_clone;            /*    16   152 */

	/* size: 168, cachelines: 3, members: 3 */
	/* last cacheline: 40 bytes */
};
struct bitmap_page {
	char *                     map;                  /*     0     8 */
	unsigned int               hijacked:1;           /*     8: 0  4 */
	unsigned int               pending:1;            /*     8: 1  4 */
	unsigned int               count:30;             /*     8: 2  4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct bitmap_counts {
	spinlock_t                 lock;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct bitmap_page *       bp;                   /*     8     8 */
	long unsigned int          pages;                /*    16     8 */
	long unsigned int          missing_pages;        /*    24     8 */
	long unsigned int          chunkshift;           /*    32     8 */
	long unsigned int          chunks;               /*    40     8 */

	/* size: 48, cachelines: 1, members: 6 */
	/* sum members: 44, holes: 1, sum holes: 4 */
	/* last cacheline: 48 bytes */
};
struct bitmap_storage {
	struct file *              file;                 /*     0     8 */
	struct page *              sb_page;              /*     8     8 */
	struct page * *            filemap;              /*    16     8 */
	long unsigned int *        filemap_attr;         /*    24     8 */
	long unsigned int          file_pages;           /*    32     8 */
	long unsigned int          bytes;                /*    40     8 */

	/* size: 48, cachelines: 1, members: 6 */
	/* last cacheline: 48 bytes */
};
struct super_type {
	char *                     name;                 /*     0     8 */
	struct module *            owner;                /*     8     8 */
	int                        (*load_super)(struct md_rdev *, struct md_rdev *, int); /*    16     8 */
	int                        (*validate_super)(struct mddev *, struct md_rdev *); /*    24     8 */
	void                       (*sync_super)(struct mddev *, struct md_rdev *); /*    32     8 */
	long long unsigned int     (*rdev_size_change)(struct md_rdev *, sector_t); /*    40     8 */
	int                        (*allow_new_offset)(struct md_rdev *, long long unsigned int); /*    48     8 */

	/* size: 56, cachelines: 1, members: 7 */
	/* last cacheline: 56 bytes */
};
struct rdev_sysfs_entry {
	struct attribute           attr;                 /*     0    16 */

	/* XXX last struct has 6 bytes of padding */

	ssize_t                    (*show)(struct md_rdev *, char *); /*    16     8 */
	ssize_t                    (*store)(struct md_rdev *, const char  *, size_t); /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* paddings: 1, sum paddings: 6 */
	/* last cacheline: 32 bytes */
};
struct detected_devices_node {
	struct list_head           list;                 /*     0    16 */
	dev_t                      dev;                  /*    16     4 */

	/* size: 24, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct bitmap_super_s {
	__le32                     magic;                /*     0     4 */
	__le32                     version;              /*     4     4 */
	__u8                       uuid[16];             /*     8    16 */
	__le64                     events;               /*    24     8 */
	__le64                     events_cleared;       /*    32     8 */
	__le64                     sync_size;            /*    40     8 */
	__le32                     state;                /*    48     4 */
	__le32                     chunksize;            /*    52     4 */
	__le32                     daemon_sleep;         /*    56     4 */
	__le32                     write_behind;         /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	__le32                     sectors_reserved;     /*    64     4 */
	__le32                     nodes;                /*    68     4 */
	__u8                       cluster_name[64];     /*    72    64 */
	/* --- cacheline 2 boundary (128 bytes) was 8 bytes ago --- */
	__u8                       pad[120];             /*   136   120 */

	/* size: 256, cachelines: 4, members: 14 */
};
struct md_setup_args {
	int                        minor;                /*     0     4 */
	int                        partitioned;          /*     4     4 */
	int                        level;                /*     8     4 */
	int                        chunk;                /*    12     4 */
	char *                     device_names;         /*    16     8 */

	/* size: 24, cachelines: 1, members: 5 */
	/* last cacheline: 24 bytes */
};
struct dm_kobject_holder {
	struct kobject             kobj;                 /*     0    96 */
	/* --- cacheline 1 boundary (64 bytes) was 32 bytes ago --- */
	struct completion          completion;           /*    96    32 */

	/* size: 128, cachelines: 2, members: 2 */
};
struct edac_mc_layer {
	enum edac_mc_layer_type    type;                 /*     0     4 */
	unsigned int               size;                 /*     4     4 */
	bool                       is_virt_csrow;        /*     8     1 */

	/* size: 12, cachelines: 1, members: 3 */
	/* padding: 3 */
	/* last cacheline: 12 bytes */
};
struct dimm_info {
	struct device              dev __attribute__((__aligned__(8))); /*     0  1000 */
	/* --- cacheline 15 boundary (960 bytes) was 40 bytes ago --- */
	char                       label[32];            /*  1000    32 */
	/* --- cacheline 16 boundary (1024 bytes) was 8 bytes ago --- */
	unsigned int               location[3];          /*  1032    12 */

	/* XXX 4 bytes hole, try to pack */

	struct mem_ctl_info *      mci;                  /*  1048     8 */
	unsigned int               idx;                  /*  1056     4 */
	u32                        grain;                /*  1060     4 */
	enum dev_type              dtype;                /*  1064     4 */
	enum mem_type              mtype;                /*  1068     4 */
	enum edac_type             edac_mode;            /*  1072     4 */
	u32                        nr_pages;             /*  1076     4 */
	unsigned int               csrow;                /*  1080     4 */
	unsigned int               cschannel;            /*  1084     4 */
	/* --- cacheline 17 boundary (1088 bytes) --- */
	u16                        smbios_handle;        /*  1088     2 */

	/* XXX 2 bytes hole, try to pack */

	u32                        ce_count;             /*  1092     4 */
	u32                        ue_count;             /*  1096     4 */

	/* size: 1104, cachelines: 18, members: 15 */
	/* sum members: 1094, holes: 2, sum holes: 6 */
	/* padding: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct mem_ctl_info {
	struct device              dev __attribute__((__aligned__(8))); /*     0  1000 */
	/* --- cacheline 15 boundary (960 bytes) was 40 bytes ago --- */
	struct bus_type *          bus;                  /*  1000     8 */
	struct list_head           link;                 /*  1008    16 */
	/* --- cacheline 16 boundary (1024 bytes) --- */
	struct module *            owner;                /*  1024     8 */
	long unsigned int          mtype_cap;            /*  1032     8 */
	long unsigned int          edac_ctl_cap;         /*  1040     8 */
	long unsigned int          edac_cap;             /*  1048     8 */
	long unsigned int          scrub_cap;            /*  1056     8 */
	enum scrub_type            scrub_mode;           /*  1064     4 */

	/* XXX 4 bytes hole, try to pack */

	int                        (*set_sdram_scrub_rate)(struct mem_ctl_info *, u32); /*  1072     8 */
	int                        (*get_sdram_scrub_rate)(struct mem_ctl_info *); /*  1080     8 */
	/* --- cacheline 17 boundary (1088 bytes) --- */
	void                       (*edac_check)(struct mem_ctl_info *); /*  1088     8 */
	long unsigned int          (*ctl_page_to_phys)(struct mem_ctl_info *, long unsigned int); /*  1096     8 */
	int                        mc_idx;               /*  1104     4 */

	/* XXX 4 bytes hole, try to pack */

	struct csrow_info * *      csrows;               /*  1112     8 */
	unsigned int               nr_csrows;            /*  1120     4 */
	unsigned int               num_cschannel;        /*  1124     4 */
	unsigned int               n_layers;             /*  1128     4 */

	/* XXX 4 bytes hole, try to pack */

	struct edac_mc_layer *     layers;               /*  1136     8 */
	bool                       csbased;              /*  1144     1 */

	/* XXX 3 bytes hole, try to pack */

	unsigned int               tot_dimms;            /*  1148     4 */
	/* --- cacheline 18 boundary (1152 bytes) --- */
	struct dimm_info * *       dimms;                /*  1152     8 */
	struct device *            pdev;                 /*  1160     8 */
	const char  *              mod_name;             /*  1168     8 */
	const char  *              ctl_name;             /*  1176     8 */
	const char  *              dev_name;             /*  1184     8 */
	void *                     pvt_info;             /*  1192     8 */
	long unsigned int          start_time;           /*  1200     8 */
	u32                        ce_noinfo_count;      /*  1208     4 */
	u32                        ue_noinfo_count;      /*  1212     4 */
	/* --- cacheline 19 boundary (1216 bytes) --- */
	u32                        ue_mc;                /*  1216     4 */
	u32                        ce_mc;                /*  1220     4 */
	struct completion          complete;             /*  1224    32 */
	const struct mcidev_sysfs_attribute  * mc_driver_sysfs_attributes; /*  1256     8 */
	struct delayed_work        work;                 /*  1264   192 */
	/* --- cacheline 22 boundary (1408 bytes) was 48 bytes ago --- */
	struct edac_raw_error_desc error_desc;           /*  1456   624 */
	/* --- cacheline 32 boundary (2048 bytes) was 32 bytes ago --- */
	int                        op_state;             /*  2080     4 */

	/* XXX 4 bytes hole, try to pack */

	struct dentry *            debugfs;              /*  2088     8 */
	u8                         fake_inject_layer[3]; /*  2096     3 */
	bool                       fake_inject_ue;       /*  2099     1 */
	u16                        fake_inject_count;    /*  2100     2 */

	/* size: 2104, cachelines: 33, members: 41 */
	/* sum members: 2083, holes: 5, sum holes: 19 */
	/* padding: 2 */
	/* forced alignments: 1 */
	/* last cacheline: 56 bytes */
} __attribute__((__aligned__(8)));
struct rank_info {
	int                        chan_idx;             /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct csrow_info *        csrow;                /*     8     8 */
	struct dimm_info *         dimm;                 /*    16     8 */
	u32                        ce_count;             /*    24     4 */

	/* size: 32, cachelines: 1, members: 4 */
	/* sum members: 24, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct csrow_info {
	struct device              dev __attribute__((__aligned__(8))); /*     0  1000 */
	/* --- cacheline 15 boundary (960 bytes) was 40 bytes ago --- */
	long unsigned int          first_page;           /*  1000     8 */
	long unsigned int          last_page;            /*  1008     8 */
	long unsigned int          page_mask;            /*  1016     8 */
	/* --- cacheline 16 boundary (1024 bytes) --- */
	int                        csrow_idx;            /*  1024     4 */
	u32                        ue_count;             /*  1028     4 */
	u32                        ce_count;             /*  1032     4 */

	/* XXX 4 bytes hole, try to pack */

	struct mem_ctl_info *      mci;                  /*  1040     8 */
	u32                        nr_channels;          /*  1048     4 */

	/* XXX 4 bytes hole, try to pack */

	struct rank_info * *       channels;             /*  1056     8 */

	/* size: 1064, cachelines: 17, members: 10 */
	/* sum members: 1056, holes: 2, sum holes: 8 */
	/* forced alignments: 1 */
	/* last cacheline: 40 bytes */
} __attribute__((__aligned__(8)));
struct edac_raw_error_desc {
	char                       location[256];        /*     0   256 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	char                       label[296];           /*   256   296 */
	/* --- cacheline 8 boundary (512 bytes) was 40 bytes ago --- */
	long int                   grain;                /*   552     8 */
	u16                        error_count;          /*   560     2 */

	/* XXX 2 bytes hole, try to pack */

	enum hw_event_mc_err_type  type;                 /*   564     4 */
	int                        top_layer;            /*   568     4 */
	int                        mid_layer;            /*   572     4 */
	/* --- cacheline 9 boundary (576 bytes) --- */
	int                        low_layer;            /*   576     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          page_frame_number;    /*   584     8 */
	long unsigned int          offset_in_page;       /*   592     8 */
	long unsigned int          syndrome;             /*   600     8 */
	const char  *              msg;                  /*   608     8 */
	const char  *              other_detail;         /*   616     8 */

	/* size: 624, cachelines: 10, members: 13 */
	/* sum members: 618, holes: 2, sum holes: 6 */
	/* last cacheline: 48 bytes */
};
struct edac_device_counter {
	u32                        ue_count;             /*     0     4 */
	u32                        ce_count;             /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct edac_dev_sysfs_attribute {
	struct attribute           attr;                 /*     0    16 */

	/* XXX last struct has 6 bytes of padding */

	ssize_t                    (*show)(struct edac_device_ctl_info *, char *); /*    16     8 */
	ssize_t                    (*store)(struct edac_device_ctl_info *, const char  *, size_t); /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* paddings: 1, sum paddings: 6 */
	/* last cacheline: 32 bytes */
};
struct edac_device_ctl_info {
	struct list_head           link;                 /*     0    16 */
	struct module *            owner;                /*    16     8 */
	int                        dev_idx;              /*    24     4 */
	int                        log_ue;               /*    28     4 */
	int                        log_ce;               /*    32     4 */
	int                        panic_on_ue;          /*    36     4 */
	unsigned int               poll_msec;            /*    40     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          delay;                /*    48     8 */
	struct edac_dev_sysfs_attribute * sysfs_attributes; /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct bus_type *          edac_subsys;          /*    64     8 */
	int                        op_state;             /*    72     4 */

	/* XXX 4 bytes hole, try to pack */

	struct delayed_work        work;                 /*    80   192 */
	/* --- cacheline 4 boundary (256 bytes) was 16 bytes ago --- */
	void                       (*edac_check)(struct edac_device_ctl_info *); /*   272     8 */
	struct device *            dev;                  /*   280     8 */
	const char  *              mod_name;             /*   288     8 */
	const char  *              ctl_name;             /*   296     8 */
	const char  *              dev_name;             /*   304     8 */
	void *                     pvt_info;             /*   312     8 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	long unsigned int          start_time;           /*   320     8 */
	struct completion          removal_complete;     /*   328    32 */
	char                       name[32];             /*   360    32 */
	/* --- cacheline 6 boundary (384 bytes) was 8 bytes ago --- */
	u32                        nr_instances;         /*   392     4 */

	/* XXX 4 bytes hole, try to pack */

	struct edac_device_instance * instances;         /*   400     8 */
	struct edac_device_counter counters;             /*   408     8 */
	struct kobject             kobj;                 /*   416    96 */

	/* size: 512, cachelines: 8, members: 25 */
	/* sum members: 500, holes: 3, sum holes: 12 */
};
struct edac_dev_sysfs_block_attribute {
	struct attribute           attr;                 /*     0    16 */

	/* XXX last struct has 6 bytes of padding */

	ssize_t                    (*show)(struct kobject *, struct attribute *, char *); /*    16     8 */
	ssize_t                    (*store)(struct kobject *, struct attribute *, const char  *, size_t); /*    24     8 */
	struct edac_device_block * block;                /*    32     8 */
	unsigned int               value;                /*    40     4 */

	/* size: 48, cachelines: 1, members: 5 */
	/* padding: 4 */
	/* paddings: 1, sum paddings: 6 */
	/* last cacheline: 48 bytes */
};
struct edac_device_block {
	struct edac_device_instance * instance;          /*     0     8 */
	char                       name[32];             /*     8    32 */
	struct edac_device_counter counters;             /*    40     8 */
	int                        nr_attribs;           /*    48     4 */

	/* XXX 4 bytes hole, try to pack */

	struct edac_dev_sysfs_block_attribute * block_attributes; /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct kobject             kobj;                 /*    64    96 */

	/* size: 160, cachelines: 3, members: 6 */
	/* sum members: 156, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct edac_device_instance {
	struct edac_device_ctl_info * ctl;               /*     0     8 */
	char                       name[35];             /*     8    35 */

	/* XXX 1 byte hole, try to pack */

	struct edac_device_counter counters;             /*    44     8 */
	u32                        nr_blocks;            /*    52     4 */
	struct edac_device_block * blocks;               /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct kobject             kobj;                 /*    64    96 */

	/* size: 160, cachelines: 3, members: 6 */
	/* sum members: 159, holes: 1, sum holes: 1 */
	/* last cacheline: 32 bytes */
};
struct dev_ch_attribute {
	struct device_attribute    attr;                 /*     0    32 */
	unsigned int               channel;              /*    32     4 */

	/* size: 40, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct ctl_info_attribute {
	struct attribute           attr;                 /*     0    16 */

	/* XXX last struct has 6 bytes of padding */

	ssize_t                    (*show)(struct edac_device_ctl_info *, char *); /*    16     8 */
	ssize_t                    (*store)(struct edac_device_ctl_info *, const char  *, size_t); /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* paddings: 1, sum paddings: 6 */
	/* last cacheline: 32 bytes */
};
struct instance_attribute {
	struct attribute           attr;                 /*     0    16 */

	/* XXX last struct has 6 bytes of padding */

	ssize_t                    (*show)(struct edac_device_instance *, char *); /*    16     8 */
	ssize_t                    (*store)(struct edac_device_instance *, const char  *, size_t); /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* paddings: 1, sum paddings: 6 */
	/* last cacheline: 32 bytes */
};
struct edac_pci_counter {
	atomic_t                   pe_count;             /*     0     4 */
	atomic_t                   npe_count;            /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct edac_pci_ctl_info {
	struct list_head           link;                 /*     0    16 */
	int                        pci_idx;              /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	struct bus_type *          edac_subsys;          /*    24     8 */
	int                        op_state;             /*    32     4 */

	/* XXX 4 bytes hole, try to pack */

	struct delayed_work        work;                 /*    40   192 */
	/* --- cacheline 3 boundary (192 bytes) was 40 bytes ago --- */
	void                       (*edac_check)(struct edac_pci_ctl_info *); /*   232     8 */
	struct device *            dev;                  /*   240     8 */
	const char  *              mod_name;             /*   248     8 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	const char  *              ctl_name;             /*   256     8 */
	const char  *              dev_name;             /*   264     8 */
	void *                     pvt_info;             /*   272     8 */
	long unsigned int          start_time;           /*   280     8 */
	struct completion          complete;             /*   288    32 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	char                       name[32];             /*   320    32 */
	struct edac_pci_counter    counters;             /*   352     8 */
	struct kobject             kobj;                 /*   360    96 */

	/* size: 456, cachelines: 8, members: 16 */
	/* sum members: 448, holes: 2, sum holes: 8 */
	/* last cacheline: 8 bytes */
};
struct edac_pci_gen_data {
	int                        edac_idx;             /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct edac_pci_dev_attribute {
	struct attribute           attr;                 /*     0    16 */

	/* XXX last struct has 6 bytes of padding */

	void *                     value;                /*    16     8 */
	ssize_t                    (*show)(void *, char *); /*    24     8 */
	ssize_t                    (*store)(void *, const char  *, size_t); /*    32     8 */

	/* size: 40, cachelines: 1, members: 4 */
	/* paddings: 1, sum paddings: 6 */
	/* last cacheline: 40 bytes */
};
struct ghes_pvt {
	struct mem_ctl_info *      mci;                  /*     0     8 */
	char                       other_detail[400];    /*     8   400 */
	/* --- cacheline 6 boundary (384 bytes) was 24 bytes ago --- */
	char                       msg[80];              /*   408    80 */

	/* size: 488, cachelines: 8, members: 3 */
	/* last cacheline: 40 bytes */
};
struct ghes_hw_desc {
	int                        num_dimms;            /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct dimm_info *         dimms;                /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct memdev_dmi_entry {
	u8                         type;                 /*     0     1 */
	u8                         length;               /*     1     1 */
	u16                        handle;               /*     2     2 */
	u16                        phys_mem_array_handle; /*     4     2 */
	u16                        mem_err_info_handle;  /*     6     2 */
	u16                        total_width;          /*     8     2 */
	u16                        data_width;           /*    10     2 */
	u16                        size;                 /*    12     2 */
	u8                         form_factor;          /*    14     1 */
	u8                         device_set;           /*    15     1 */
	u8                         device_locator;       /*    16     1 */
	u8                         bank_locator;         /*    17     1 */
	u8                         memory_type;          /*    18     1 */
	u16                        type_detail;          /*    19     2 */
	u16                        speed;                /*    21     2 */
	u8                         manufacturer;         /*    23     1 */
	u8                         serial_number;        /*    24     1 */
	u8                         asset_tag;            /*    25     1 */
	u8                         part_number;          /*    26     1 */
	u8                         attributes;           /*    27     1 */
	u32                        extended_size;        /*    28     4 */
	u16                        conf_mem_clk_speed;   /*    32     2 */

	/* size: 34, cachelines: 1, members: 22 */
	/* last cacheline: 34 bytes */
} __attribute__((__packed__));
struct cpufreq_policy_data {
	struct cpufreq_cpuinfo     cpuinfo;              /*     0    12 */

	/* XXX 4 bytes hole, try to pack */

	struct cpufreq_frequency_table * freq_table;     /*    16     8 */
	unsigned int               cpu;                  /*    24     4 */
	unsigned int               min;                  /*    28     4 */
	unsigned int               max;                  /*    32     4 */

	/* size: 40, cachelines: 1, members: 5 */
	/* sum members: 32, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct freq_attr {
	struct attribute           attr;                 /*     0    16 */

	/* XXX last struct has 6 bytes of padding */

	ssize_t                    (*show)(struct cpufreq_policy *, char *); /*    16     8 */
	ssize_t                    (*store)(struct cpufreq_policy *, const char  *, size_t); /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* paddings: 1, sum paddings: 6 */
	/* last cacheline: 32 bytes */
};
struct cpufreq_driver {
	char                       name[16];             /*     0    16 */
	u16                        flags;                /*    16     2 */

	/* XXX 6 bytes hole, try to pack */

	void *                     driver_data;          /*    24     8 */
	int                        (*init)(struct cpufreq_policy *); /*    32     8 */
	int                        (*verify)(struct cpufreq_policy_data *); /*    40     8 */
	int                        (*setpolicy)(struct cpufreq_policy *); /*    48     8 */
	int                        (*target)(struct cpufreq_policy *, unsigned int, unsigned int); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*target_index)(struct cpufreq_policy *, unsigned int); /*    64     8 */
	unsigned int               (*fast_switch)(struct cpufreq_policy *, unsigned int); /*    72     8 */
	unsigned int               (*resolve_freq)(struct cpufreq_policy *, unsigned int); /*    80     8 */
	unsigned int               (*get_intermediate)(struct cpufreq_policy *, unsigned int); /*    88     8 */
	int                        (*target_intermediate)(struct cpufreq_policy *, unsigned int); /*    96     8 */
	unsigned int               (*get)(unsigned int); /*   104     8 */
	void                       (*update_limits)(unsigned int); /*   112     8 */
	int                        (*bios_limit)(int, unsigned int *); /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	int                        (*online)(struct cpufreq_policy *); /*   128     8 */
	int                        (*offline)(struct cpufreq_policy *); /*   136     8 */
	int                        (*exit)(struct cpufreq_policy *); /*   144     8 */
	void                       (*stop_cpu)(struct cpufreq_policy *); /*   152     8 */
	int                        (*suspend)(struct cpufreq_policy *); /*   160     8 */
	int                        (*resume)(struct cpufreq_policy *); /*   168     8 */
	void                       (*ready)(struct cpufreq_policy *); /*   176     8 */
	struct freq_attr * *       attr;                 /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	bool                       boost_enabled;        /*   192     1 */

	/* XXX 7 bytes hole, try to pack */

	int                        (*set_boost)(struct cpufreq_policy *, int); /*   200     8 */

	/* size: 208, cachelines: 4, members: 25 */
	/* sum members: 195, holes: 2, sum holes: 13 */
	/* last cacheline: 16 bytes */
};
struct cpufreq_stats {
	unsigned int               total_trans;          /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	long long unsigned int     last_time;            /*     8     8 */
	unsigned int               max_state;            /*    16     4 */
	unsigned int               state_num;            /*    20     4 */
	unsigned int               last_index;           /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	u64 *                      time_in_state;        /*    32     8 */
	unsigned int *             freq_table;           /*    40     8 */
	unsigned int *             trans_table;          /*    48     8 */
	unsigned int               reset_pending;        /*    56     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	long long unsigned int     reset_time;           /*    64     8 */

	/* size: 72, cachelines: 2, members: 10 */
	/* sum members: 60, holes: 3, sum holes: 12 */
	/* last cacheline: 8 bytes */
};
struct dbs_data {
	struct gov_attr_set        attr_set;             /*     0   152 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 2 boundary (128 bytes) was 24 bytes ago --- */
	void *                     tuners;               /*   152     8 */
	unsigned int               ignore_nice_load;     /*   160     4 */
	unsigned int               sampling_rate;        /*   164     4 */
	unsigned int               sampling_down_factor; /*   168     4 */
	unsigned int               up_threshold;         /*   172     4 */
	unsigned int               io_is_busy;           /*   176     4 */

	/* size: 184, cachelines: 3, members: 7 */
	/* padding: 4 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 56 bytes */
};
struct policy_dbs_info {
	struct cpufreq_policy *    policy;               /*     0     8 */
	struct mutex               update_mutex;         /*     8    32 */
	u64                        last_sample_time;     /*    40     8 */
	s64                        sample_delay_ns;      /*    48     8 */
	atomic_t                   work_count;           /*    56     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	struct irq_work            irq_work;             /*    64    24 */
	struct work_struct         work;                 /*    88    64 */
	/* --- cacheline 2 boundary (128 bytes) was 24 bytes ago --- */
	struct dbs_data *          dbs_data;             /*   152     8 */
	struct list_head           list;                 /*   160    16 */
	unsigned int               rate_mult;            /*   176     4 */
	unsigned int               idle_periods;         /*   180     4 */
	bool                       is_shared;            /*   184     1 */
	bool                       work_in_progress;     /*   185     1 */

	/* size: 192, cachelines: 3, members: 13 */
	/* sum members: 182, holes: 1, sum holes: 4 */
	/* padding: 6 */
};
struct dbs_governor {
	struct cpufreq_governor    gov;                  /*     0   104 */

	/* XXX last struct has 7 bytes of padding */

	/* --- cacheline 1 boundary (64 bytes) was 40 bytes ago --- */
	struct kobj_type           kobj_type;            /*   104    88 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	struct dbs_data *          gdbs_data;            /*   192     8 */
	unsigned int               (*gov_dbs_update)(struct cpufreq_policy *); /*   200     8 */
	struct policy_dbs_info *   (*alloc)(void);       /*   208     8 */
	void                       (*free)(struct policy_dbs_info *); /*   216     8 */
	int                        (*init)(struct dbs_data *); /*   224     8 */
	void                       (*exit)(struct dbs_data *); /*   232     8 */
	void                       (*start)(struct cpufreq_policy *); /*   240     8 */

	/* size: 248, cachelines: 4, members: 9 */
	/* paddings: 1, sum paddings: 7 */
	/* last cacheline: 56 bytes */
};
struct od_ops {
	unsigned int               (*powersave_bias_target)(struct cpufreq_policy *, unsigned int, unsigned int); /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct od_policy_dbs_info {
	struct policy_dbs_info     policy_dbs;           /*     0   192 */

	/* XXX last struct has 6 bytes of padding */

	/* --- cacheline 3 boundary (192 bytes) --- */
	unsigned int               freq_lo;              /*   192     4 */
	unsigned int               freq_lo_delay_us;     /*   196     4 */
	unsigned int               freq_hi_delay_us;     /*   200     4 */
	unsigned int               sample_type:1;        /*   204: 0  4 */

	/* size: 208, cachelines: 4, members: 5 */
	/* paddings: 1, sum paddings: 6 */
	/* bit_padding: 31 bits */
	/* last cacheline: 16 bytes */
};
struct od_dbs_tuners {
	unsigned int               powersave_bias;       /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct cs_policy_dbs_info {
	struct policy_dbs_info     policy_dbs;           /*     0   192 */

	/* XXX last struct has 6 bytes of padding */

	/* --- cacheline 3 boundary (192 bytes) --- */
	unsigned int               down_skip;            /*   192     4 */
	unsigned int               requested_freq;       /*   196     4 */

	/* size: 200, cachelines: 4, members: 3 */
	/* paddings: 1, sum paddings: 6 */
	/* last cacheline: 8 bytes */
};
struct cs_dbs_tuners {
	unsigned int               down_threshold;       /*     0     4 */
	unsigned int               freq_step;            /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct cpu_dbs_info {
	u64                        prev_cpu_idle;        /*     0     8 */
	u64                        prev_update_time;     /*     8     8 */
	u64                        prev_cpu_nice;        /*    16     8 */
	unsigned int               prev_load;            /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	struct update_util_data    update_util;          /*    32     8 */
	struct policy_dbs_info *   policy_dbs;           /*    40     8 */

	/* size: 48, cachelines: 1, members: 6 */
	/* sum members: 44, holes: 1, sum holes: 4 */
	/* last cacheline: 48 bytes */
};
struct sample {
	int32_t                    core_avg_perf;        /*     0     4 */
	int32_t                    busy_scaled;          /*     4     4 */
	u64                        aperf;                /*     8     8 */
	u64                        mperf;                /*    16     8 */
	u64                        tsc;                  /*    24     8 */
	u64                        time;                 /*    32     8 */

	/* size: 40, cachelines: 1, members: 6 */
	/* last cacheline: 40 bytes */
};
struct pstate_data {
	int                        current_pstate;       /*     0     4 */
	int                        min_pstate;           /*     4     4 */
	int                        max_pstate;           /*     8     4 */
	int                        max_pstate_physical;  /*    12     4 */
	int                        scaling;              /*    16     4 */
	int                        turbo_pstate;         /*    20     4 */
	unsigned int               max_freq;             /*    24     4 */
	unsigned int               turbo_freq;           /*    28     4 */

	/* size: 32, cachelines: 1, members: 8 */
	/* last cacheline: 32 bytes */
};
struct vid_data {
	int                        min;                  /*     0     4 */
	int                        max;                  /*     4     4 */
	int                        turbo;                /*     8     4 */
	int32_t                    ratio;                /*    12     4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct global_params {
	bool                       no_turbo;             /*     0     1 */
	bool                       turbo_disabled;       /*     1     1 */
	bool                       turbo_disabled_mf;    /*     2     1 */

	/* XXX 1 byte hole, try to pack */

	int                        max_perf_pct;         /*     4     4 */
	int                        min_perf_pct;         /*     8     4 */

	/* size: 12, cachelines: 1, members: 5 */
	/* sum members: 11, holes: 1, sum holes: 1 */
	/* last cacheline: 12 bytes */
};
struct cpudata {
	int                        cpu;                  /*     0     4 */
	unsigned int               policy;               /*     4     4 */
	struct update_util_data    update_util;          /*     8     8 */
	bool                       update_util_set;      /*    16     1 */

	/* XXX 3 bytes hole, try to pack */

	struct pstate_data         pstate;               /*    20    32 */
	struct vid_data            vid;                  /*    52    16 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	u64                        last_update;          /*    72     8 */
	u64                        last_sample_time;     /*    80     8 */
	u64                        aperf_mperf_shift;    /*    88     8 */
	u64                        prev_aperf;           /*    96     8 */
	u64                        prev_mperf;           /*   104     8 */
	u64                        prev_tsc;             /*   112     8 */
	u64                        prev_cummulative_iowait; /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct sample              sample;               /*   128    40 */
	int32_t                    min_perf_ratio;       /*   168     4 */
	int32_t                    max_perf_ratio;       /*   172     4 */
	struct acpi_processor_performance acpi_perf_data; /*   176   112 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 4 boundary (256 bytes) was 32 bytes ago --- */
	bool                       valid_pss_table;      /*   288     1 */

	/* XXX 3 bytes hole, try to pack */

	unsigned int               iowait_boost;         /*   292     4 */
	s16                        epp_powersave;        /*   296     2 */
	s16                        epp_policy;           /*   298     2 */
	s16                        epp_default;          /*   300     2 */
	s16                        epp_cached;           /*   302     2 */
	u64                        hwp_req_cached;       /*   304     8 */
	u64                        hwp_cap_cached;       /*   312     8 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	u64                        last_io_update;       /*   320     8 */
	unsigned int               sched_flags;          /*   328     4 */
	u32                        hwp_boost_min;        /*   332     4 */
	bool                       suspended;            /*   336     1 */

	/* size: 344, cachelines: 6, members: 29 */
	/* sum members: 327, holes: 3, sum holes: 10 */
	/* padding: 7 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 24 bytes */
};
struct pstate_funcs {
	int                        (*get_max)(void);     /*     0     8 */
	int                        (*get_max_physical)(void); /*     8     8 */
	int                        (*get_min)(void);     /*    16     8 */
	int                        (*get_turbo)(void);   /*    24     8 */
	int                        (*get_scaling)(void); /*    32     8 */
	int                        (*get_aperf_mperf_shift)(void); /*    40     8 */
	u64                        (*get_val)(struct cpudata *, int); /*    48     8 */
	void                       (*get_vid)(struct cpudata *); /*    56     8 */

	/* size: 64, cachelines: 1, members: 8 */
};
struct cpuidle_governor {
	char                       name[16];             /*     0    16 */
	struct list_head           governor_list;        /*    16    16 */
	unsigned int               rating;               /*    32     4 */

	/* XXX 4 bytes hole, try to pack */

	int                        (*enable)(struct cpuidle_driver *, struct cpuidle_device *); /*    40     8 */
	void                       (*disable)(struct cpuidle_driver *, struct cpuidle_device *); /*    48     8 */
	int                        (*select)(struct cpuidle_driver *, struct cpuidle_device *, bool *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	void                       (*reflect)(struct cpuidle_device *, int); /*    64     8 */

	/* size: 72, cachelines: 2, members: 7 */
	/* sum members: 68, holes: 1, sum holes: 4 */
	/* last cacheline: 8 bytes */
};
struct cpuidle_state_kobj {
	struct cpuidle_state *     state;                /*     0     8 */
	struct cpuidle_state_usage * state_usage;        /*     8     8 */
	struct completion          kobj_unregister;      /*    16    32 */
	struct kobject             kobj;                 /*    48    96 */
	/* --- cacheline 2 boundary (128 bytes) was 16 bytes ago --- */
	struct cpuidle_device *    device;               /*   144     8 */

	/* size: 152, cachelines: 3, members: 5 */
	/* last cacheline: 24 bytes */
};
struct cpuidle_device_kobj {
	struct cpuidle_device *    dev;                  /*     0     8 */
	struct completion          kobj_unregister;      /*     8    32 */
	struct kobject             kobj;                 /*    40    96 */

	/* size: 136, cachelines: 3, members: 3 */
	/* last cacheline: 8 bytes */
};
struct cpuidle_attr {
	struct attribute           attr;                 /*     0    16 */

	/* XXX last struct has 6 bytes of padding */

	ssize_t                    (*show)(struct cpuidle_device *, char *); /*    16     8 */
	ssize_t                    (*store)(struct cpuidle_device *, const char  *, size_t); /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* paddings: 1, sum paddings: 6 */
	/* last cacheline: 32 bytes */
};
struct cpuidle_state_attr {
	struct attribute           attr;                 /*     0    16 */

	/* XXX last struct has 6 bytes of padding */

	ssize_t                    (*show)(struct cpuidle_state *, struct cpuidle_state_usage *, char *); /*    16     8 */
	ssize_t                    (*store)(struct cpuidle_state *, struct cpuidle_state_usage *, const char  *, size_t); /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* paddings: 1, sum paddings: 6 */
	/* last cacheline: 32 bytes */
};
struct menu_device {
	int                        needs_update;         /*     0     4 */
	int                        tick_wakeup;          /*     4     4 */
	u64                        next_timer_ns;        /*     8     8 */
	unsigned int               bucket;               /*    16     4 */
	unsigned int               correction_factor[12]; /*    20    48 */
	/* --- cacheline 1 boundary (64 bytes) was 4 bytes ago --- */
	unsigned int               intervals[8];         /*    68    32 */
	int                        interval_ptr;         /*   100     4 */

	/* size: 104, cachelines: 2, members: 7 */
	/* last cacheline: 40 bytes */
};
struct teo_idle_state {
	unsigned int               early_hits;           /*     0     4 */
	unsigned int               hits;                 /*     4     4 */
	unsigned int               misses;               /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct teo_cpu {
	u64                        time_span_ns;         /*     0     8 */
	u64                        sleep_length_ns;      /*     8     8 */
	struct teo_idle_state      states[10];           /*    16   120 */
	/* --- cacheline 2 boundary (128 bytes) was 8 bytes ago --- */
	int                        interval_idx;         /*   136     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        intervals[8];         /*   144    64 */

	/* size: 208, cachelines: 4, members: 5 */
	/* sum members: 204, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct sdhci_pci_data {
	struct pci_dev *           pdev;                 /*     0     8 */
	int                        slotno;               /*     8     4 */
	int                        rst_n_gpio;           /*    12     4 */
	int                        cd_gpio;              /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	int                        (*setup)(struct sdhci_pci_data *); /*    24     8 */
	void                       (*cleanup)(struct sdhci_pci_data *); /*    32     8 */

	/* size: 40, cachelines: 1, members: 6 */
	/* sum members: 36, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct led_init_data {
	struct fwnode_handle *     fwnode;               /*     0     8 */
	const char  *              default_label;        /*     8     8 */
	const char  *              devicename;           /*    16     8 */
	bool                       devname_mandatory;    /*    24     1 */

	/* size: 32, cachelines: 1, members: 4 */
	/* padding: 7 */
	/* last cacheline: 32 bytes */
};
struct led_properties {
	u32                        color;                /*     0     4 */
	bool                       color_present;        /*     4     1 */

	/* XXX 3 bytes hole, try to pack */

	const char  *              function;             /*     8     8 */
	u32                        func_enum;            /*    16     4 */
	bool                       func_enum_present;    /*    20     1 */

	/* XXX 3 bytes hole, try to pack */

	const char  *              label;                /*    24     8 */

	/* size: 32, cachelines: 1, members: 6 */
	/* sum members: 26, holes: 2, sum holes: 6 */
	/* last cacheline: 32 bytes */
};
struct dmi_memdev_info {
	const char  *              device;               /*     0     8 */
	const char  *              bank;                 /*     8     8 */
	u64                        size;                 /*    16     8 */
	u16                        handle;               /*    24     2 */
	u8                         type;                 /*    26     1 */

	/* size: 32, cachelines: 1, members: 5 */
	/* padding: 5 */
	/* last cacheline: 32 bytes */
};
struct dmi_sysfs_entry {
	struct dmi_header          dh;                   /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct kobject             kobj;                 /*     8    96 */
	/* --- cacheline 1 boundary (64 bytes) was 40 bytes ago --- */
	int                        instance;             /*   104     4 */
	int                        position;             /*   108     4 */
	struct list_head           list;                 /*   112    16 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct kobject *           child;                /*   128     8 */

	/* size: 136, cachelines: 3, members: 6 */
	/* sum members: 132, holes: 1, sum holes: 4 */
	/* last cacheline: 8 bytes */
};
struct dmi_sysfs_attribute {
	struct attribute           attr;                 /*     0    16 */

	/* XXX last struct has 6 bytes of padding */

	ssize_t                    (*show)(struct dmi_sysfs_entry *, char *); /*    16     8 */

	/* size: 24, cachelines: 1, members: 2 */
	/* paddings: 1, sum paddings: 6 */
	/* last cacheline: 24 bytes */
};
struct dmi_sysfs_mapped_attribute {
	struct attribute           attr;                 /*     0    16 */

	/* XXX last struct has 6 bytes of padding */

	ssize_t                    (*show)(struct dmi_sysfs_entry *, const struct dmi_header  *, char *); /*    16     8 */

	/* size: 24, cachelines: 1, members: 2 */
	/* paddings: 1, sum paddings: 6 */
	/* last cacheline: 24 bytes */
};
struct find_dmi_data {
	struct dmi_sysfs_entry *   entry;                /*     0     8 */
	dmi_callback               callback;             /*     8     8 */
	void *                     private;              /*    16     8 */
	int                        instance_countdown;   /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	ssize_t                    ret;                  /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* sum members: 36, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct dmi_read_state {
	char *                     buf;                  /*     0     8 */
	loff_t                     pos;                  /*     8     8 */
	size_t                     count;                /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct dmi_entry_attr_show_data {
	struct attribute *         attr;                 /*     0     8 */
	char *                     buf;                  /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct dmi_system_event_log {
	struct dmi_header          header;               /*     0     4 */
	u16                        area_length;          /*     4     2 */
	u16                        header_start_offset;  /*     6     2 */
	u16                        data_start_offset;    /*     8     2 */
	u8                         access_method;        /*    10     1 */
	u8                         status;               /*    11     1 */
	u32                        change_token;         /*    12     4 */
	union {
		struct {
			u16        index_addr;           /*    16     2 */
			u16        data_addr;            /*    18     2 */
		} io;                                    /*    16     4 */
		u32                phys_addr32;          /*    16     4 */
		u16                gpnv_handle;          /*    16     2 */
		u32                access_method_address; /*    16     4 */
	};                                               /*    16     4 */
	u8                         header_format;        /*    20     1 */
	u8                         type_descriptors_supported_count; /*    21     1 */
	u8                         per_log_type_descriptor_length; /*    22     1 */
	u8                         supported_log_type_descriptos[]; /*    23     0 */

	/* size: 23, cachelines: 1, members: 12 */
	/* last cacheline: 23 bytes */
} __attribute__((__packed__));
struct dmi_device_attribute {
	struct device_attribute    dev_attr;             /*     0    32 */
	int                        field;                /*    32     4 */

	/* size: 40, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct acpi_table_ibft {
	struct acpi_table_header   header;               /*     0    36 */
	u8                         reserved[12];         /*    36    12 */

	/* size: 48, cachelines: 1, members: 2 */
	/* last cacheline: 48 bytes */
};
struct firmware_map_entry {
	u64                        start;                /*     0     8 */
	u64                        end;                  /*     8     8 */
	const char  *              type;                 /*    16     8 */
	struct list_head           list;                 /*    24    16 */
	struct kobject             kobj;                 /*    40    96 */

	/* size: 136, cachelines: 3, members: 5 */
	/* last cacheline: 8 bytes */
};
struct memmap_attribute {
	struct attribute           attr;                 /*     0    16 */

	/* XXX last struct has 6 bytes of padding */

	ssize_t                    (*show)(struct firmware_map_entry *, char *); /*    16     8 */

	/* size: 24, cachelines: 1, members: 2 */
	/* paddings: 1, sum paddings: 6 */
	/* last cacheline: 24 bytes */
};
struct fw_cfg_file {
	__be32                     size;                 /*     0     4 */
	__be16                     select;               /*     4     2 */
	__u16                      reserved;             /*     6     2 */
	char                       name[56];             /*     8    56 */

	/* size: 64, cachelines: 1, members: 4 */
};
struct fw_cfg_dma_access {
	__be32                     control;              /*     0     4 */
	__be32                     length;               /*     4     4 */
	__be64                     address;              /*     8     8 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct fw_cfg_vmcoreinfo {
	__le16                     host_format;          /*     0     2 */
	__le16                     guest_format;         /*     2     2 */
	__le32                     size;                 /*     4     4 */
	__le64                     paddr;                /*     8     8 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct fw_cfg_sysfs_entry {
	struct kobject             kobj;                 /*     0    96 */
	/* --- cacheline 1 boundary (64 bytes) was 32 bytes ago --- */
	u32                        size;                 /*    96     4 */
	u16                        select;               /*   100     2 */
	char                       name[56];             /*   102    56 */

	/* XXX 2 bytes hole, try to pack */

	/* --- cacheline 2 boundary (128 bytes) was 32 bytes ago --- */
	struct list_head           list;                 /*   160    16 */

	/* size: 176, cachelines: 3, members: 5 */
	/* sum members: 174, holes: 1, sum holes: 2 */
	/* last cacheline: 48 bytes */
};
struct fw_cfg_sysfs_attribute {
	struct attribute           attr;                 /*     0    16 */

	/* XXX last struct has 6 bytes of padding */

	ssize_t                    (*show)(struct fw_cfg_sysfs_entry *, char *); /*    16     8 */

	/* size: 24, cachelines: 1, members: 2 */
	/* paddings: 1, sum paddings: 6 */
	/* last cacheline: 24 bytes */
};
struct bmp_header {
	u16                        id;                   /*     0     2 */
	u32                        size;                 /*     2     4 */

	/* size: 6, cachelines: 1, members: 2 */
	/* last cacheline: 6 bytes */
} __attribute__((__packed__));
struct efivar_operations {
	efi_get_variable_t *       get_variable;         /*     0     8 */
	efi_get_next_variable_t *  get_next_variable;    /*     8     8 */
	efi_set_variable_t *       set_variable;         /*    16     8 */
	efi_set_variable_t *       set_variable_nonblocking; /*    24     8 */
	efi_query_variable_store_t * query_variable_store; /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct efivars {
	struct kset *              kset;                 /*     0     8 */
	struct kobject *           kobject;              /*     8     8 */
	const struct efivar_operations  * ops;           /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct linux_efi_random_seed {
	u32                        size;                 /*     0     4 */
	u8                         bits[];               /*     4     0 */

	/* size: 4, cachelines: 1, members: 2 */
	/* last cacheline: 4 bytes */
};
struct linux_efi_memreserve {
	int                        size;                 /*     0     4 */
	atomic_t                   count;                /*     4     4 */
	phys_addr_t                next;                 /*     8     8 */
	struct {
		phys_addr_t        base;                 /*    16     8 */
		phys_addr_t        size;                 /*    24     8 */
	} entry[]; /*    16     0 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct efi_generic_dev_path {
	u8                         type;                 /*     0     1 */
	u8                         sub_type;             /*     1     1 */
	u16                        length;               /*     2     2 */

	/* size: 4, cachelines: 1, members: 3 */
	/* last cacheline: 4 bytes */
};
struct variable_validate {
	efi_guid_t                 vendor __attribute__((__aligned__(4))); /*     0    16 */
	char *                     name;                 /*    16     8 */
	bool                       (*validate)(efi_char16_t *, int, u8 *, long unsigned int); /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* forced alignments: 1 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct efi_system_resource_entry_v1 {
	efi_guid_t                 fw_class __attribute__((__aligned__(4))); /*     0    16 */
	u32                        fw_type;              /*    16     4 */
	u32                        fw_version;           /*    20     4 */
	u32                        lowest_supported_fw_version; /*    24     4 */
	u32                        capsule_flags;        /*    28     4 */
	u32                        last_attempt_version; /*    32     4 */
	u32                        last_attempt_status;  /*    36     4 */

	/* size: 40, cachelines: 1, members: 7 */
	/* forced alignments: 1 */
	/* last cacheline: 40 bytes */
} __attribute__((__aligned__(4)));
struct efi_system_resource_table {
	u32                        fw_resource_count;    /*     0     4 */
	u32                        fw_resource_count_max; /*     4     4 */
	u64                        fw_resource_version;  /*     8     8 */
	u8                         entries[];            /*    16     0 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct esre_entry {
	union {
		struct efi_system_resource_entry_v1 * esre1; /*     0     8 */
	} esre;                                          /*     0     8 */
	struct kobject             kobj;                 /*     8    96 */
	/* --- cacheline 1 boundary (64 bytes) was 40 bytes ago --- */
	struct list_head           list;                 /*   104    16 */

	/* size: 120, cachelines: 2, members: 3 */
	/* last cacheline: 56 bytes */
};
struct esre_attribute {
	struct attribute           attr;                 /*     0    16 */

	/* XXX last struct has 6 bytes of padding */

	ssize_t                    (*show)(struct esre_entry *, char *); /*    16     8 */
	ssize_t                    (*store)(struct esre_entry *, const char  *, size_t); /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* paddings: 1, sum paddings: 6 */
	/* last cacheline: 32 bytes */
};
struct cper_sec_proc_generic {
	u64                        validation_bits;      /*     0     8 */
	u8                         proc_type;            /*     8     1 */
	u8                         proc_isa;             /*     9     1 */
	u8                         proc_error_type;      /*    10     1 */
	u8                         operation;            /*    11     1 */
	u8                         flags;                /*    12     1 */
	u8                         level;                /*    13     1 */
	u16                        reserved;             /*    14     2 */
	u64                        cpu_version;          /*    16     8 */
	char                       cpu_brand[128];       /*    24   128 */
	/* --- cacheline 2 boundary (128 bytes) was 24 bytes ago --- */
	u64                        proc_id;              /*   152     8 */
	u64                        target_addr;          /*   160     8 */
	u64                        requestor_id;         /*   168     8 */
	u64                        responder_id;         /*   176     8 */
	u64                        ip;                   /*   184     8 */

	/* size: 192, cachelines: 3, members: 15 */
};
struct cper_sec_proc_ia {
	u64                        validation_bits;      /*     0     8 */
	u64                        lapic_id;             /*     8     8 */
	u8                         cpuid[48];            /*    16    48 */

	/* size: 64, cachelines: 1, members: 3 */
};
struct cper_mem_err_compact {
	u64                        validation_bits;      /*     0     8 */
	u16                        node;                 /*     8     2 */
	u16                        card;                 /*    10     2 */
	u16                        module;               /*    12     2 */
	u16                        bank;                 /*    14     2 */
	u16                        device;               /*    16     2 */
	u16                        row;                  /*    18     2 */
	u16                        column;               /*    20     2 */
	u16                        bit_pos;              /*    22     2 */
	u64                        requestor_id;         /*    24     8 */
	u64                        responder_id;         /*    32     8 */
	u64                        target_id;            /*    40     8 */
	u16                        rank;                 /*    48     2 */
	u16                        mem_array_handle;     /*    50     2 */
	u16                        mem_dev_handle;       /*    52     2 */
	u8                         extended;             /*    54     1 */

	/* size: 55, cachelines: 1, members: 16 */
	/* last cacheline: 55 bytes */
} __attribute__((__packed__));
struct cper_sec_fw_err_rec_ref {
	u8                         record_type;          /*     0     1 */
	u8                         revision;             /*     1     1 */
	u8                         reserved[6];          /*     2     6 */
	u64                        record_identifier;    /*     8     8 */
	guid_t                     record_identifier_guid; /*    16    16 */

	/* size: 32, cachelines: 1, members: 5 */
	/* last cacheline: 32 bytes */
};
struct efi_runtime_map_entry {
	efi_memory_desc_t          md;                   /*     0    40 */
	struct kobject             kobj;                 /*    40    96 */

	/* size: 136, cachelines: 3, members: 2 */
	/* last cacheline: 8 bytes */
};
struct map_attribute {
	struct attribute           attr;                 /*     0    16 */

	/* XXX last struct has 6 bytes of padding */

	ssize_t                    (*show)(struct efi_runtime_map_entry *, char *); /*    16     8 */

	/* size: 24, cachelines: 1, members: 2 */
	/* paddings: 1, sum paddings: 6 */
	/* last cacheline: 24 bytes */
};
struct efi_acpi_dev_path {
	struct efi_generic_dev_path header;              /*     0     4 */
	u32                        hid;                  /*     4     4 */
	u32                        uid;                  /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct efi_pci_dev_path {
	struct efi_generic_dev_path header;              /*     0     4 */
	u8                         fn;                   /*     4     1 */
	u8                         dev;                  /*     5     1 */

	/* size: 6, cachelines: 1, members: 3 */
	/* last cacheline: 6 bytes */
};
struct efi_vendor_dev_path {
	struct efi_generic_dev_path header;              /*     0     4 */
	efi_guid_t                 vendorguid __attribute__((__aligned__(1))); /*     4    16 */
	u8                         vendordata[];         /*    20     0 */

	/* size: 20, cachelines: 1, members: 3 */
	/* forced alignments: 1 */
	/* last cacheline: 20 bytes */
} __attribute__((__packed__));
struct efi_rel_offset_dev_path {
	struct efi_generic_dev_path header;              /*     0     4 */
	u32                        reserved;             /*     4     4 */
	u64                        starting_offset;      /*     8     8 */
	u64                        ending_offset;        /*    16     8 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct efi_dev_path {
	union {
		struct efi_generic_dev_path header;      /*     0     4 */
		struct efi_acpi_dev_path acpi;           /*     0    12 */
		struct efi_pci_dev_path pci;             /*     0     6 */
		struct efi_vendor_dev_path vendor __attribute__((__aligned__(1))); /*     0    20 */
		struct efi_rel_offset_dev_path rel_offset; /*     0    24 */
	} __attribute__((__aligned__(1)));               /*     0    24 */

	/* size: 24, cachelines: 1, members: 1 */
	/* forced alignments: 1 */
	/* last cacheline: 24 bytes */
} __attribute__((__packed__));
struct acpi_hid_uid {
	struct acpi_device_id      hid[2];               /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	char                       uid[11];              /*    64    11 */

	/* size: 80, cachelines: 2, members: 2 */
	/* padding: 5 */
	/* last cacheline: 16 bytes */
};
struct dev_header {
	u32                        len;                  /*     0     4 */
	u32                        prop_count;           /*     4     4 */
	struct efi_dev_path        path[] __attribute__((__aligned__(1))); /*     8     0 */

	/* size: 8, cachelines: 1, members: 3 */
	/* forced alignments: 1 */
	/* last cacheline: 8 bytes */
} __attribute__((__aligned__(4)));
struct properties_header {
	u32                        len;                  /*     0     4 */
	u32                        version;              /*     4     4 */
	u32                        dev_count;            /*     8     4 */
	struct dev_header          dev_header[] __attribute__((__aligned__(4))); /*    12     0 */

	/* size: 12, cachelines: 1, members: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 12 bytes */
} __attribute__((__packed__));
struct efi_mokvar_sysfs_attr {
	struct bin_attribute       bin_attr;             /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct list_head           node;                 /*    64    16 */

	/* size: 80, cachelines: 2, members: 2 */
	/* last cacheline: 16 bytes */
};
struct cper_ia_err_info {
	guid_t                     err_type;             /*     0    16 */
	u64                        validation_bits;      /*    16     8 */
	u64                        check_info;           /*    24     8 */
	u64                        target_id;            /*    32     8 */
	u64                        requestor_id;         /*    40     8 */
	u64                        responder_id;         /*    48     8 */
	u64                        ip;                   /*    56     8 */

	/* size: 64, cachelines: 1, members: 7 */
};
struct cper_ia_proc_ctx {
	u16                        reg_ctx_type;         /*     0     2 */
	u16                        reg_arr_size;         /*     2     2 */
	u32                        msr_addr;             /*     4     4 */
	u64                        mm_reg_addr;          /*     8     8 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct hid_device_id {
	__u16                      bus;                  /*     0     2 */
	__u16                      group;                /*     2     2 */
	__u32                      vendor;               /*     4     4 */
	__u32                      product;              /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	kernel_ulong_t             driver_data;          /*    16     8 */

	/* size: 24, cachelines: 1, members: 5 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct hid_item {
	unsigned int               format;               /*     0     4 */
	__u8                       size;                 /*     4     1 */
	__u8                       type;                 /*     5     1 */
	__u8                       tag;                  /*     6     1 */

	/* XXX 1 byte hole, try to pack */

	union {
		__u8               u8;                   /*     8     1 */
		__s8               s8;                   /*     8     1 */
		__u16              u16;                  /*     8     2 */
		__s16              s16;                  /*     8     2 */
		__u32              u32;                  /*     8     4 */
		__s32              s32;                  /*     8     4 */
		__u8 *             longdata;             /*     8     8 */
	} data;                                          /*     8     8 */

	/* size: 16, cachelines: 1, members: 5 */
	/* sum members: 15, holes: 1, sum holes: 1 */
	/* last cacheline: 16 bytes */
};
struct hid_global {
	unsigned int               usage_page;           /*     0     4 */
	__s32                      logical_minimum;      /*     4     4 */
	__s32                      logical_maximum;      /*     8     4 */
	__s32                      physical_minimum;     /*    12     4 */
	__s32                      physical_maximum;     /*    16     4 */
	__s32                      unit_exponent;        /*    20     4 */
	unsigned int               unit;                 /*    24     4 */
	unsigned int               report_id;            /*    28     4 */
	unsigned int               report_size;          /*    32     4 */
	unsigned int               report_count;         /*    36     4 */

	/* size: 40, cachelines: 1, members: 10 */
	/* last cacheline: 40 bytes */
};
struct hid_local {
	unsigned int               usage[12288];         /*     0 49152 */
	/* --- cacheline 768 boundary (49152 bytes) --- */
	u8                         usage_size[12288];    /* 49152 12288 */
	/* --- cacheline 960 boundary (61440 bytes) --- */
	unsigned int               collection_index[12288]; /* 61440 49152 */
	/* --- cacheline 1728 boundary (110592 bytes) --- */
	unsigned int               usage_index;          /* 110592     4 */
	unsigned int               usage_minimum;        /* 110596     4 */
	unsigned int               delimiter_depth;      /* 110600     4 */
	unsigned int               delimiter_branch;     /* 110604     4 */

	/* size: 110608, cachelines: 1729, members: 7 */
	/* last cacheline: 16 bytes */
};
struct hid_collection {
	int                        parent_idx;           /*     0     4 */
	unsigned int               type;                 /*     4     4 */
	unsigned int               usage;                /*     8     4 */
	unsigned int               level;                /*    12     4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct hid_usage {
	unsigned int               hid;                  /*     0     4 */
	unsigned int               collection_index;     /*     4     4 */
	unsigned int               usage_index;          /*     8     4 */
	__s8                       resolution_multiplier; /*    12     1 */
	__s8                       wheel_factor;         /*    13     1 */
	__u16                      code;                 /*    14     2 */
	__u8                       type;                 /*    16     1 */
	__s8                       hat_min;              /*    17     1 */
	__s8                       hat_max;              /*    18     1 */
	__s8                       hat_dir;              /*    19     1 */
	__s16                      wheel_accumulated;    /*    20     2 */

	/* size: 24, cachelines: 1, members: 11 */
	/* padding: 2 */
	/* last cacheline: 24 bytes */
};
struct hid_field {
	unsigned int               physical;             /*     0     4 */
	unsigned int               logical;              /*     4     4 */
	unsigned int               application;          /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct hid_usage *         usage;                /*    16     8 */
	unsigned int               maxusage;             /*    24     4 */
	unsigned int               flags;                /*    28     4 */
	unsigned int               report_offset;        /*    32     4 */
	unsigned int               report_size;          /*    36     4 */
	unsigned int               report_count;         /*    40     4 */
	unsigned int               report_type;          /*    44     4 */
	__s32 *                    value;                /*    48     8 */
	__s32                      logical_minimum;      /*    56     4 */
	__s32                      logical_maximum;      /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	__s32                      physical_minimum;     /*    64     4 */
	__s32                      physical_maximum;     /*    68     4 */
	__s32                      unit_exponent;        /*    72     4 */
	unsigned int               unit;                 /*    76     4 */
	struct hid_report *        report;               /*    80     8 */
	unsigned int               index;                /*    88     4 */

	/* XXX 4 bytes hole, try to pack */

	struct hid_input *         hidinput;             /*    96     8 */
	__u16                      dpad;                 /*   104     2 */

	/* size: 112, cachelines: 2, members: 21 */
	/* sum members: 98, holes: 2, sum holes: 8 */
	/* padding: 6 */
	/* last cacheline: 48 bytes */
};
struct hid_report {
	struct list_head           list;                 /*     0    16 */
	struct list_head           hidinput_list;        /*    16    16 */
	unsigned int               id;                   /*    32     4 */
	unsigned int               type;                 /*    36     4 */
	unsigned int               application;          /*    40     4 */

	/* XXX 4 bytes hole, try to pack */

	struct hid_field *         field[256];           /*    48  2048 */
	/* --- cacheline 32 boundary (2048 bytes) was 48 bytes ago --- */
	unsigned int               maxfield;             /*  2096     4 */
	unsigned int               size;                 /*  2100     4 */
	struct hid_device *        device;               /*  2104     8 */

	/* size: 2112, cachelines: 33, members: 9 */
	/* sum members: 2108, holes: 1, sum holes: 4 */
};
struct hid_input {
	struct list_head           list;                 /*     0    16 */
	struct hid_report *        report;               /*    16     8 */
	struct input_dev *         input;                /*    24     8 */
	const char  *              name;                 /*    32     8 */
	bool                       registered;           /*    40     1 */

	/* XXX 7 bytes hole, try to pack */

	struct list_head           reports;              /*    48    16 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	unsigned int               application;          /*    64     4 */

	/* size: 72, cachelines: 2, members: 7 */
	/* sum members: 61, holes: 1, sum holes: 7 */
	/* padding: 4 */
	/* last cacheline: 8 bytes */
};
struct hid_device {
	__u8 *                     dev_rdesc;            /*     0     8 */
	unsigned int               dev_rsize;            /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	__u8 *                     rdesc;                /*    16     8 */
	unsigned int               rsize;                /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	struct hid_collection *    collection;           /*    32     8 */
	unsigned int               collection_size;      /*    40     4 */
	unsigned int               maxcollection;        /*    44     4 */
	unsigned int               maxapplication;       /*    48     4 */
	__u16                      bus;                  /*    52     2 */
	__u16                      group;                /*    54     2 */
	__u32                      vendor;               /*    56     4 */
	__u32                      product;              /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	__u32                      version;              /*    64     4 */
	enum hid_type              type;                 /*    68     4 */
	unsigned int               country;              /*    72     4 */

	/* XXX 4 bytes hole, try to pack */

	struct hid_report_enum     report_enum[3];       /*    80  6216 */
	/* --- cacheline 98 boundary (6272 bytes) was 24 bytes ago --- */
	struct work_struct         led_work;             /*  6296    64 */
	/* --- cacheline 99 boundary (6336 bytes) was 24 bytes ago --- */
	struct semaphore           driver_input_lock;    /*  6360    24 */
	struct device              dev __attribute__((__aligned__(8))); /*  6384  1000 */
	/* --- cacheline 115 boundary (7360 bytes) was 24 bytes ago --- */
	struct hid_driver *        driver;               /*  7384     8 */
	struct hid_ll_driver *     ll_driver;            /*  7392     8 */
	struct mutex               ll_open_lock;         /*  7400    32 */
	/* --- cacheline 116 boundary (7424 bytes) was 8 bytes ago --- */
	unsigned int               ll_open_count;        /*  7432     4 */

	/* XXX 4 bytes hole, try to pack */

	struct power_supply *      battery;              /*  7440     8 */
	__s32                      battery_capacity;     /*  7448     4 */
	__s32                      battery_min;          /*  7452     4 */
	__s32                      battery_max;          /*  7456     4 */
	__s32                      battery_report_type;  /*  7460     4 */
	__s32                      battery_report_id;    /*  7464     4 */
	enum hid_battery_status    battery_status;       /*  7468     4 */
	bool                       battery_avoid_query;  /*  7472     1 */

	/* XXX 7 bytes hole, try to pack */

	long unsigned int          status;               /*  7480     8 */
	/* --- cacheline 117 boundary (7488 bytes) --- */
	unsigned int               claimed;              /*  7488     4 */
	unsigned int               quirks;               /*  7492     4 */
	bool                       io_started;           /*  7496     1 */

	/* XXX 7 bytes hole, try to pack */

	struct list_head           inputs;               /*  7504    16 */
	void *                     hiddev;               /*  7520     8 */
	void *                     hidraw;               /*  7528     8 */
	char                       name[128];            /*  7536   128 */
	/* --- cacheline 119 boundary (7616 bytes) was 48 bytes ago --- */
	char                       phys[64];             /*  7664    64 */
	/* --- cacheline 120 boundary (7680 bytes) was 48 bytes ago --- */
	char                       uniq[64];             /*  7728    64 */
	/* --- cacheline 121 boundary (7744 bytes) was 48 bytes ago --- */
	void *                     driver_data;          /*  7792     8 */
	int                        (*ff_init)(struct hid_device *); /*  7800     8 */
	/* --- cacheline 122 boundary (7808 bytes) --- */
	int                        (*hiddev_connect)(struct hid_device *, unsigned int); /*  7808     8 */
	void                       (*hiddev_disconnect)(struct hid_device *); /*  7816     8 */
	void                       (*hiddev_hid_event)(struct hid_device *, struct hid_field *, struct hid_usage *, __s32); /*  7824     8 */
	void                       (*hiddev_report_event)(struct hid_device *, struct hid_report *); /*  7832     8 */
	short unsigned int         debug;                /*  7840     2 */

	/* XXX 6 bytes hole, try to pack */

	struct dentry *            debug_dir;            /*  7848     8 */
	struct dentry *            debug_rdesc;          /*  7856     8 */
	struct dentry *            debug_events;         /*  7864     8 */
	/* --- cacheline 123 boundary (7872 bytes) --- */
	struct list_head           debug_list;           /*  7872    16 */
	spinlock_t                 debug_list_lock;      /*  7888     4 */

	/* XXX 4 bytes hole, try to pack */

	wait_queue_head_t          debug_wait;           /*  7896    24 */

	/* size: 7920, cachelines: 124, members: 54 */
	/* sum members: 7880, holes: 8, sum holes: 40 */
	/* forced alignments: 1 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct hid_report_enum {
	unsigned int               numbered;             /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           report_list;          /*     8    16 */
	struct hid_report *        report_id_hash[256];  /*    24  2048 */

	/* size: 2072, cachelines: 33, members: 3 */
	/* sum members: 2068, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct hid_driver {
	char *                     name;                 /*     0     8 */
	const struct hid_device_id  * id_table;          /*     8     8 */
	struct list_head           dyn_list;             /*    16    16 */
	spinlock_t                 dyn_lock;             /*    32     4 */

	/* XXX 4 bytes hole, try to pack */

	bool                       (*match)(struct hid_device *, bool); /*    40     8 */
	int                        (*probe)(struct hid_device *, const struct hid_device_id  *); /*    48     8 */
	void                       (*remove)(struct hid_device *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	const struct hid_report_id  * report_table;      /*    64     8 */
	int                        (*raw_event)(struct hid_device *, struct hid_report *, u8 *, int); /*    72     8 */
	const struct hid_usage_id  * usage_table;        /*    80     8 */
	int                        (*event)(struct hid_device *, struct hid_field *, struct hid_usage *, __s32); /*    88     8 */
	void                       (*report)(struct hid_device *, struct hid_report *); /*    96     8 */
	__u8 *                     (*report_fixup)(struct hid_device *, __u8 *, unsigned int *); /*   104     8 */
	int                        (*input_mapping)(struct hid_device *, struct hid_input *, struct hid_field *, struct hid_usage *, long unsigned int * *, int *); /*   112     8 */
	int                        (*input_mapped)(struct hid_device *, struct hid_input *, struct hid_field *, struct hid_usage *, long unsigned int * *, int *); /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	int                        (*input_configured)(struct hid_device *, struct hid_input *); /*   128     8 */
	void                       (*feature_mapping)(struct hid_device *, struct hid_field *, struct hid_usage *); /*   136     8 */
	int                        (*suspend)(struct hid_device *, pm_message_t); /*   144     8 */
	int                        (*resume)(struct hid_device *); /*   152     8 */
	int                        (*reset_resume)(struct hid_device *); /*   160     8 */
	struct device_driver       driver;               /*   168   192 */

	/* size: 360, cachelines: 6, members: 21 */
	/* sum members: 356, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct hid_ll_driver {
	int                        (*start)(struct hid_device *); /*     0     8 */
	void                       (*stop)(struct hid_device *); /*     8     8 */
	int                        (*open)(struct hid_device *); /*    16     8 */
	void                       (*close)(struct hid_device *); /*    24     8 */
	int                        (*power)(struct hid_device *, int); /*    32     8 */
	int                        (*parse)(struct hid_device *); /*    40     8 */
	void                       (*request)(struct hid_device *, struct hid_report *, int); /*    48     8 */
	int                        (*wait)(struct hid_device *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*raw_request)(struct hid_device *, unsigned char, __u8 *, size_t, unsigned char, int); /*    64     8 */
	int                        (*output_report)(struct hid_device *, __u8 *, size_t); /*    72     8 */
	int                        (*idle)(struct hid_device *, int, int, int); /*    80     8 */

	/* size: 88, cachelines: 2, members: 11 */
	/* last cacheline: 24 bytes */
};
struct hid_parser {
	struct hid_global          global;               /*     0    40 */
	struct hid_global          global_stack[4];      /*    40   160 */
	/* --- cacheline 3 boundary (192 bytes) was 8 bytes ago --- */
	unsigned int               global_stack_ptr;     /*   200     4 */
	struct hid_local           local;                /*   204 110608 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 1731 boundary (110784 bytes) was 32 bytes ago --- */
	unsigned int *             collection_stack;     /* 110816     8 */
	unsigned int               collection_stack_ptr; /* 110824     4 */
	unsigned int               collection_stack_size; /* 110828     4 */
	struct hid_device *        device;               /* 110832     8 */
	unsigned int               scan_flags;           /* 110840     4 */

	/* size: 110848, cachelines: 1732, members: 9 */
	/* sum members: 110840, holes: 1, sum holes: 4 */
	/* padding: 4 */
};
struct hid_report_id {
	__u32                      report_type;          /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct hid_usage_id {
	__u32                      usage_hid;            /*     0     4 */
	__u32                      usage_type;           /*     4     4 */
	__u32                      usage_code;           /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct hiddev {
	int                        minor;                /*     0     4 */
	int                        exist;                /*     4     4 */
	int                        open;                 /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct mutex               existancelock;        /*    16    32 */
	wait_queue_head_t          wait;                 /*    48    24 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	struct hid_device *        hid;                  /*    72     8 */
	struct list_head           list;                 /*    80    16 */
	spinlock_t                 list_lock;            /*    96     4 */
	bool                       initialized;          /*   100     1 */

	/* size: 104, cachelines: 2, members: 9 */
	/* sum members: 97, holes: 1, sum holes: 4 */
	/* padding: 3 */
	/* last cacheline: 40 bytes */
};
struct hidraw {
	unsigned int               minor;                /*     0     4 */
	int                        exist;                /*     4     4 */
	int                        open;                 /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	wait_queue_head_t          wait;                 /*    16    24 */
	struct hid_device *        hid;                  /*    40     8 */
	struct device *            dev;                  /*    48     8 */
	spinlock_t                 list_lock;            /*    56     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	struct list_head           list;                 /*    64    16 */

	/* size: 80, cachelines: 2, members: 8 */
	/* sum members: 72, holes: 2, sum holes: 8 */
	/* last cacheline: 16 bytes */
};
struct hid_dynid {
	struct list_head           list;                 /*     0    16 */
	struct hid_device_id       id;                   /*    16    24 */

	/* size: 40, cachelines: 1, members: 2 */
	/* last cacheline: 40 bytes */
};
struct quirks_list_struct {
	struct hid_device_id       hid_bl_item;          /*     0    24 */
	struct list_head           node;                 /*    24    16 */

	/* size: 40, cachelines: 1, members: 2 */
	/* last cacheline: 40 bytes */
};
struct hid_debug_list {
	struct {
		union {
			struct __kfifo kfifo;            /*     0    24 */
			char *     type;                 /*     0     8 */
			const char  * const_type;        /*     0     8 */
			char *     rectype;              /*     0     8 */
			char *     ptr;                  /*     0     8 */
			const char  * ptr_const;         /*     0     8 */
		};                                       /*     0    24 */
		char               buf[];                /*    24     0 */
	} hid_debug_fifo;                                /*     0    24 */
	struct fasync_struct *     fasync;               /*    24     8 */
	struct hid_device *        hdev;                 /*    32     8 */
	struct list_head           node;                 /*    40    16 */
	struct mutex               read_mutex;           /*    56    32 */

	/* size: 88, cachelines: 2, members: 5 */
	/* last cacheline: 24 bytes */
};
struct hid_usage_entry {
	unsigned int               page;                 /*     0     4 */
	unsigned int               usage;                /*     4     4 */
	const char  *              description;          /*     8     8 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct hidraw_devinfo {
	__u32                      bustype;              /*     0     4 */
	__s16                      vendor;               /*     4     2 */
	__s16                      product;              /*     6     2 */

	/* size: 8, cachelines: 1, members: 3 */
	/* last cacheline: 8 bytes */
};
struct hidraw_report {
	__u8 *                     value;                /*     0     8 */
	int                        len;                  /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct hidraw_list {
	struct hidraw_report       buffer[64];           /*     0  1024 */
	/* --- cacheline 16 boundary (1024 bytes) --- */
	int                        head;                 /*  1024     4 */
	int                        tail;                 /*  1028     4 */
	struct fasync_struct *     fasync;               /*  1032     8 */
	struct hidraw *            hidraw;               /*  1040     8 */
	struct list_head           node;                 /*  1048    16 */
	struct mutex               read_mutex;           /*  1064    32 */

	/* size: 1096, cachelines: 18, members: 7 */
	/* last cacheline: 8 bytes */
};
struct magicmouse_sc {
	struct input_dev *         input;                /*     0     8 */
	long unsigned int          quirks;               /*     8     8 */
	int                        ntouches;             /*    16     4 */
	int                        scroll_accel;         /*    20     4 */
	long unsigned int          scroll_jiffies;       /*    24     8 */
	struct {
		short int          x;                    /*    32     2 */
		short int          y;                    /*    34     2 */
		short int          scroll_x;             /*    36     2 */
		short int          scroll_y;             /*    38     2 */
		u8                 size;                 /*    40     1 */
	} touches[16]; /*    32   160 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	int                        tracking_ids[16];     /*   192    64 */

	/* size: 256, cachelines: 4, members: 7 */
};
struct ntrig_data {
	__u16                      x;                    /*     0     2 */
	__u16                      y;                    /*     2     2 */
	__u16                      w;                    /*     4     2 */
	__u16                      h;                    /*     6     2 */
	__u16                      id;                   /*     8     2 */
	bool                       tipswitch;            /*    10     1 */
	bool                       confidence;           /*    11     1 */
	bool                       first_contact_touch;  /*    12     1 */
	bool                       reading_mt;           /*    13     1 */
	__u8                       mt_footer[4];         /*    14     4 */
	__u8                       mt_foot_count;        /*    18     1 */
	__s8                       act_state;            /*    19     1 */
	__s8                       deactivate_slack;     /*    20     1 */
	__s8                       activate_slack;       /*    21     1 */
	__u16                      min_width;            /*    22     2 */
	__u16                      min_height;           /*    24     2 */
	__u16                      activation_width;     /*    26     2 */
	__u16                      activation_height;    /*    28     2 */
	__u16                      sensor_logical_width; /*    30     2 */
	__u16                      sensor_logical_height; /*    32     2 */
	__u16                      sensor_physical_width; /*    34     2 */
	__u16                      sensor_physical_height; /*    36     2 */

	/* size: 38, cachelines: 1, members: 22 */
	/* last cacheline: 38 bytes */
};
struct hid_sensor_hub_attribute_info {
	u32                        usage_id;             /*     0     4 */
	u32                        attrib_id;            /*     4     4 */
	s32                        report_id;            /*     8     4 */
	s32                        index;                /*    12     4 */
	s32                        units;                /*    16     4 */
	s32                        unit_expo;            /*    20     4 */
	s32                        size;                 /*    24     4 */
	s32                        logical_minimum;      /*    28     4 */
	s32                        logical_maximum;      /*    32     4 */

	/* size: 36, cachelines: 1, members: 9 */
	/* last cacheline: 36 bytes */
};
struct sensor_hub_pending {
	bool                       status;               /*     0     1 */

	/* XXX 7 bytes hole, try to pack */

	struct completion          ready;                /*     8    32 */
	u32                        usage_id;             /*    40     4 */
	u32                        attr_usage_id;        /*    44     4 */
	int                        raw_size;             /*    48     4 */

	/* XXX 4 bytes hole, try to pack */

	u8 *                       raw_data;             /*    56     8 */

	/* size: 64, cachelines: 1, members: 6 */
	/* sum members: 53, holes: 2, sum holes: 11 */
};
struct hid_sensor_hub_device {
	struct hid_device *        hdev;                 /*     0     8 */
	u32                        vendor_id;            /*     8     4 */
	u32                        product_id;           /*    12     4 */
	u32                        usage;                /*    16     4 */
	int                        start_collection_index; /*    20     4 */
	int                        end_collection_index; /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	struct mutex *             mutex_ptr;            /*    32     8 */
	struct sensor_hub_pending  pending;              /*    40    64 */

	/* size: 104, cachelines: 2, members: 8 */
	/* sum members: 100, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct hid_sensor_hub_callbacks {
	struct platform_device *   pdev;                 /*     0     8 */
	int                        (*suspend)(struct hid_sensor_hub_device *, void *); /*     8     8 */
	int                        (*resume)(struct hid_sensor_hub_device *, void *); /*    16     8 */
	int                        (*capture_sample)(struct hid_sensor_hub_device *, u32, size_t, char *, void *); /*    24     8 */
	int                        (*send_event)(struct hid_sensor_hub_device *, u32, void *); /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct sensor_hub_data {
	struct mutex               mutex;                /*     0    32 */
	spinlock_t                 lock;                 /*    32     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           dyn_callback_list;    /*    40    16 */
	spinlock_t                 dyn_callback_lock;    /*    56     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	struct mfd_cell *          hid_sensor_hub_client_devs; /*    64     8 */
	int                        hid_sensor_client_cnt; /*    72     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          quirks;               /*    80     8 */
	int                        ref_cnt;              /*    88     4 */

	/* size: 96, cachelines: 2, members: 8 */
	/* sum members: 80, holes: 3, sum holes: 12 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct hid_sensor_hub_callbacks_list {
	struct list_head           list;                 /*     0    16 */
	u32                        usage_id;             /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	struct hid_sensor_hub_device * hsdev;            /*    24     8 */
	struct hid_sensor_hub_callbacks * usage_callback; /*    32     8 */
	void *                     priv;                 /*    40     8 */

	/* size: 48, cachelines: 1, members: 5 */
	/* sum members: 44, holes: 1, sum holes: 4 */
	/* last cacheline: 48 bytes */
};
struct hid_control_fifo {
	unsigned char              dir;                  /*     0     1 */

	/* XXX 7 bytes hole, try to pack */

	struct hid_report *        report;               /*     8     8 */
	char *                     raw_report;           /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 17, holes: 1, sum holes: 7 */
	/* last cacheline: 24 bytes */
};
struct hid_output_fifo {
	struct hid_report *        report;               /*     0     8 */
	char *                     raw_report;           /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct hid_class_descriptor {
	__u8                       bDescriptorType;      /*     0     1 */
	__le16                     wDescriptorLength;    /*     1     2 */

	/* size: 3, cachelines: 1, members: 2 */
	/* last cacheline: 3 bytes */
} __attribute__((__packed__));
struct hid_descriptor {
	__u8                       bLength;              /*     0     1 */
	__u8                       bDescriptorType;      /*     1     1 */
	__le16                     bcdHID;               /*     2     2 */
	__u8                       bCountryCode;         /*     4     1 */
	__u8                       bNumDescriptors;      /*     5     1 */
	struct hid_class_descriptor desc[1];             /*     6     3 */

	/* size: 9, cachelines: 1, members: 6 */
	/* last cacheline: 9 bytes */
} __attribute__((__packed__));
struct usbhid_device {
	struct hid_device *        hid;                  /*     0     8 */
	struct usb_interface *     intf;                 /*     8     8 */
	int                        ifnum;                /*    16     4 */
	unsigned int               bufsize;              /*    20     4 */
	struct urb *               urbin;                /*    24     8 */
	char *                     inbuf;                /*    32     8 */
	dma_addr_t                 inbuf_dma;            /*    40     8 */
	struct urb *               urbctrl;              /*    48     8 */
	struct usb_ctrlrequest *   cr;                   /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct hid_control_fifo    ctrl[256];            /*    64  6144 */
	/* --- cacheline 97 boundary (6208 bytes) --- */
	unsigned char              ctrlhead;             /*  6208     1 */
	unsigned char              ctrltail;             /*  6209     1 */

	/* XXX 6 bytes hole, try to pack */

	char *                     ctrlbuf;              /*  6216     8 */
	dma_addr_t                 ctrlbuf_dma;          /*  6224     8 */
	long unsigned int          last_ctrl;            /*  6232     8 */
	struct urb *               urbout;               /*  6240     8 */
	struct hid_output_fifo     out[256];             /*  6248  4096 */
	/* --- cacheline 161 boundary (10304 bytes) was 40 bytes ago --- */
	unsigned char              outhead;              /* 10344     1 */
	unsigned char              outtail;              /* 10345     1 */

	/* XXX 6 bytes hole, try to pack */

	char *                     outbuf;               /* 10352     8 */
	dma_addr_t                 outbuf_dma;           /* 10360     8 */
	/* --- cacheline 162 boundary (10368 bytes) --- */
	long unsigned int          last_out;             /* 10368     8 */
	struct mutex               mutex;                /* 10376    32 */
	spinlock_t                 lock;                 /* 10408     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          iofl;                 /* 10416     8 */
	struct timer_list          io_retry;             /* 10424    72 */
	/* --- cacheline 164 boundary (10496 bytes) --- */
	long unsigned int          stop_retry;           /* 10496     8 */
	unsigned int               retry_delay;          /* 10504     4 */

	/* XXX 4 bytes hole, try to pack */

	struct work_struct         reset_work;           /* 10512    64 */
	/* --- cacheline 165 boundary (10560 bytes) was 16 bytes ago --- */
	wait_queue_head_t          wait;                 /* 10576    24 */

	/* size: 10600, cachelines: 166, members: 30 */
	/* sum members: 10580, holes: 4, sum holes: 20 */
	/* last cacheline: 40 bytes */
};
struct hiddev_event {
	unsigned int               hid;                  /*     0     4 */
	int                        value;                /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct hiddev_devinfo {
	__u32                      bustype;              /*     0     4 */
	__u32                      busnum;               /*     4     4 */
	__u32                      devnum;               /*     8     4 */
	__u32                      ifnum;                /*    12     4 */
	__s16                      vendor;               /*    16     2 */
	__s16                      product;              /*    18     2 */
	__s16                      version;              /*    20     2 */

	/* XXX 2 bytes hole, try to pack */

	__u32                      num_applications;     /*    24     4 */

	/* size: 28, cachelines: 1, members: 8 */
	/* sum members: 26, holes: 1, sum holes: 2 */
	/* last cacheline: 28 bytes */
};
struct hiddev_collection_info {
	__u32                      index;                /*     0     4 */
	__u32                      type;                 /*     4     4 */
	__u32                      usage;                /*     8     4 */
	__u32                      level;                /*    12     4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct hiddev_report_info {
	__u32                      report_type;          /*     0     4 */
	__u32                      report_id;            /*     4     4 */
	__u32                      num_fields;           /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct hiddev_field_info {
	__u32                      report_type;          /*     0     4 */
	__u32                      report_id;            /*     4     4 */
	__u32                      field_index;          /*     8     4 */
	__u32                      maxusage;             /*    12     4 */
	__u32                      flags;                /*    16     4 */
	__u32                      physical;             /*    20     4 */
	__u32                      logical;              /*    24     4 */
	__u32                      application;          /*    28     4 */
	__s32                      logical_minimum;      /*    32     4 */
	__s32                      logical_maximum;      /*    36     4 */
	__s32                      physical_minimum;     /*    40     4 */
	__s32                      physical_maximum;     /*    44     4 */
	__u32                      unit_exponent;        /*    48     4 */
	__u32                      unit;                 /*    52     4 */

	/* size: 56, cachelines: 1, members: 14 */
	/* last cacheline: 56 bytes */
};
struct hiddev_usage_ref {
	__u32                      report_type;          /*     0     4 */
	__u32                      report_id;            /*     4     4 */
	__u32                      field_index;          /*     8     4 */
	__u32                      usage_index;          /*    12     4 */
	__u32                      usage_code;           /*    16     4 */
	__s32                      value;                /*    20     4 */

	/* size: 24, cachelines: 1, members: 6 */
	/* last cacheline: 24 bytes */
};
struct hiddev_usage_ref_multi {
	struct hiddev_usage_ref    uref;                 /*     0    24 */
	__u32                      num_values;           /*    24     4 */
	__s32                      values[1024];         /*    28  4096 */

	/* size: 4124, cachelines: 65, members: 3 */
	/* last cacheline: 28 bytes */
};
struct hiddev_list {
	struct hiddev_usage_ref    buffer[2048];         /*     0 49152 */
	/* --- cacheline 768 boundary (49152 bytes) --- */
	int                        head;                 /* 49152     4 */
	int                        tail;                 /* 49156     4 */
	unsigned int               flags;                /* 49160     4 */

	/* XXX 4 bytes hole, try to pack */

	struct fasync_struct *     fasync;               /* 49168     8 */
	struct hiddev *            hiddev;               /* 49176     8 */
	struct list_head           node;                 /* 49184    16 */
	struct mutex               thread_lock;          /* 49200    32 */

	/* size: 49232, cachelines: 770, members: 8 */
	/* sum members: 49228, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct pidff_usage {
	struct hid_field *         field;                /*     0     8 */
	s32 *                      value;                /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct pidff_device {
	struct hid_device *        hid;                  /*     0     8 */
	struct hid_report *        reports[13];          /*     8   104 */
	/* --- cacheline 1 boundary (64 bytes) was 48 bytes ago --- */
	struct pidff_usage         set_effect[7];        /*   112   112 */
	/* --- cacheline 3 boundary (192 bytes) was 32 bytes ago --- */
	struct pidff_usage         set_envelope[5];      /*   224    80 */
	/* --- cacheline 4 boundary (256 bytes) was 48 bytes ago --- */
	struct pidff_usage         set_condition[8];     /*   304   128 */
	/* --- cacheline 6 boundary (384 bytes) was 48 bytes ago --- */
	struct pidff_usage         set_periodic[5];      /*   432    80 */
	/* --- cacheline 8 boundary (512 bytes) --- */
	struct pidff_usage         set_constant[2];      /*   512    32 */
	struct pidff_usage         set_ramp[3];          /*   544    48 */
	/* --- cacheline 9 boundary (576 bytes) was 16 bytes ago --- */
	struct pidff_usage         device_gain[1];       /*   592    16 */
	struct pidff_usage         block_load[2];        /*   608    32 */
	/* --- cacheline 10 boundary (640 bytes) --- */
	struct pidff_usage         pool[3];              /*   640    48 */
	struct pidff_usage         effect_operation[2];  /*   688    32 */
	/* --- cacheline 11 boundary (704 bytes) was 16 bytes ago --- */
	struct pidff_usage         block_free[1];        /*   720    16 */
	struct hid_field *         create_new_effect_type; /*   736     8 */
	struct hid_field *         set_effect_type;      /*   744     8 */
	struct hid_field *         effect_direction;     /*   752     8 */
	struct hid_field *         device_control;       /*   760     8 */
	/* --- cacheline 12 boundary (768 bytes) --- */
	struct hid_field *         block_load_status;    /*   768     8 */
	struct hid_field *         effect_operation_status; /*   776     8 */
	int                        control_id[2];        /*   784     8 */
	int                        type_id[11];          /*   792    44 */
	/* --- cacheline 13 boundary (832 bytes) was 4 bytes ago --- */
	int                        status_id[2];         /*   836     8 */
	int                        operation_id[2];      /*   844     8 */
	int                        pid_id[64];           /*   852   256 */

	/* size: 1112, cachelines: 18, members: 24 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct pmc_bit_map {
	const char  *              name;                 /*     0     8 */
	u32                        bit_mask;             /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct pmc_reg_map {
	const struct pmc_bit_map  * d3_sts_0;            /*     0     8 */
	const struct pmc_bit_map  * d3_sts_1;            /*     8     8 */
	const struct pmc_bit_map  * func_dis;            /*    16     8 */
	const struct pmc_bit_map  * func_dis_2;          /*    24     8 */
	const struct pmc_bit_map  * pss;                 /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct pmc_data {
	const struct pmc_reg_map  * map;                 /*     0     8 */
	const struct pmc_clk  *    clks;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct pmc_dev {
	u32                        base_addr;            /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     regmap;               /*     8     8 */
	const struct pmc_reg_map  * map;                 /*    16     8 */
	struct dentry *            dbgfs_dir;            /*    24     8 */
	bool                       init;                 /*    32     1 */

	/* size: 40, cachelines: 1, members: 5 */
	/* sum members: 29, holes: 1, sum holes: 4 */
	/* padding: 7 */
	/* last cacheline: 40 bytes */
};
struct acpi_table_pcct {
	struct acpi_table_header   header;               /*     0    36 */
	u32                        flags;                /*    36     4 */
	u64                        reserved;             /*    40     8 */

	/* size: 48, cachelines: 1, members: 3 */
	/* last cacheline: 48 bytes */
};
struct acpi_pcct_subspace {
	struct acpi_subtable_header header;              /*     0     2 */
	u8                         reserved[6];          /*     2     6 */
	u64                        base_address;         /*     8     8 */
	u64                        length;               /*    16     8 */
	struct acpi_generic_address doorbell_register;   /*    24    12 */
	u64                        preserve_mask;        /*    36     8 */
	u64                        write_mask;           /*    44     8 */
	u32                        latency;              /*    52     4 */
	u32                        max_access_rate;      /*    56     4 */
	u16                        min_turnaround_time;  /*    60     2 */

	/* size: 62, cachelines: 1, members: 10 */
	/* last cacheline: 62 bytes */
} __attribute__((__packed__));
struct acpi_pcct_hw_reduced_type2 {
	struct acpi_subtable_header header;              /*     0     2 */
	u32                        platform_interrupt;   /*     2     4 */
	u8                         flags;                /*     6     1 */
	u8                         reserved;             /*     7     1 */
	u64                        base_address;         /*     8     8 */
	u64                        length;               /*    16     8 */
	struct acpi_generic_address doorbell_register;   /*    24    12 */
	u64                        preserve_mask;        /*    36     8 */
	u64                        write_mask;           /*    44     8 */
	u32                        latency;              /*    52     4 */
	u32                        max_access_rate;      /*    56     4 */
	u16                        min_turnaround_time;  /*    60     2 */
	struct acpi_generic_address platform_ack_register; /*    62    12 */
	/* --- cacheline 1 boundary (64 bytes) was 10 bytes ago --- */
	u64                        ack_preserve_mask;    /*    74     8 */
	u64                        ack_write_mask;       /*    82     8 */

	/* size: 90, cachelines: 2, members: 15 */
	/* last cacheline: 26 bytes */
} __attribute__((__packed__));
struct hwspinlock_ops {
	int                        (*trylock)(struct hwspinlock *); /*     0     8 */
	void                       (*unlock)(struct hwspinlock *); /*     8     8 */
	void                       (*relax)(struct hwspinlock *); /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct hwspinlock {
	struct hwspinlock_device * bank;                 /*     0     8 */
	spinlock_t                 lock;                 /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     priv;                 /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct hwspinlock_device {
	struct device *            dev;                  /*     0     8 */
	const struct hwspinlock_ops  * ops;              /*     8     8 */
	int                        base_id;              /*    16     4 */
	int                        num_locks;            /*    20     4 */
	struct hwspinlock          lock[];               /*    24     0 */

	/* size: 24, cachelines: 1, members: 5 */
	/* last cacheline: 24 bytes */
};
struct powercap_control_type_ops {
	int                        (*set_enable)(struct powercap_control_type *, bool); /*     0     8 */
	int                        (*get_enable)(struct powercap_control_type *, bool *); /*     8     8 */
	int                        (*release)(struct powercap_control_type *); /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct powercap_control_type {
	struct device              dev __attribute__((__aligned__(8))); /*     0  1000 */
	/* --- cacheline 15 boundary (960 bytes) was 40 bytes ago --- */
	struct idr                 idr;                  /*  1000    24 */
	/* --- cacheline 16 boundary (1024 bytes) --- */
	int                        nr_zones;             /*  1024     4 */

	/* XXX 4 bytes hole, try to pack */

	const struct powercap_control_type_ops  * ops;   /*  1032     8 */
	struct mutex               lock;                 /*  1040    32 */
	bool                       allocated;            /*  1072     1 */

	/* XXX 7 bytes hole, try to pack */

	struct list_head           node;                 /*  1080    16 */

	/* size: 1096, cachelines: 18, members: 7 */
	/* sum members: 1085, holes: 2, sum holes: 11 */
	/* forced alignments: 1 */
	/* last cacheline: 8 bytes */
} __attribute__((__aligned__(8)));
struct powercap_zone_ops {
	int                        (*get_max_energy_range_uj)(struct powercap_zone *, u64 *); /*     0     8 */
	int                        (*get_energy_uj)(struct powercap_zone *, u64 *); /*     8     8 */
	int                        (*reset_energy_uj)(struct powercap_zone *); /*    16     8 */
	int                        (*get_max_power_range_uw)(struct powercap_zone *, u64 *); /*    24     8 */
	int                        (*get_power_uw)(struct powercap_zone *, u64 *); /*    32     8 */
	int                        (*set_enable)(struct powercap_zone *, bool); /*    40     8 */
	int                        (*get_enable)(struct powercap_zone *, bool *); /*    48     8 */
	int                        (*release)(struct powercap_zone *); /*    56     8 */

	/* size: 64, cachelines: 1, members: 8 */
};
struct powercap_zone {
	int                        id;                   /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	char *                     name;                 /*     8     8 */
	void *                     control_type_inst;    /*    16     8 */
	const struct powercap_zone_ops  * ops;           /*    24     8 */
	struct device              dev __attribute__((__aligned__(8))); /*    32  1000 */
	/* --- cacheline 16 boundary (1024 bytes) was 8 bytes ago --- */
	int                        const_id_cnt;         /*  1032     4 */

	/* XXX 4 bytes hole, try to pack */

	struct idr                 idr;                  /*  1040    24 */
	struct idr *               parent_idr;           /*  1064     8 */
	void *                     private_data;         /*  1072     8 */
	struct attribute * *       zone_dev_attrs;       /*  1080     8 */
	/* --- cacheline 17 boundary (1088 bytes) --- */
	int                        zone_attr_count;      /*  1088     4 */

	/* XXX 4 bytes hole, try to pack */

	struct attribute_group     dev_zone_attr_group;  /*  1096    40 */
	const struct attribute_group  * dev_attr_groups[2]; /*  1136    16 */
	/* --- cacheline 18 boundary (1152 bytes) --- */
	bool                       allocated;            /*  1152     1 */

	/* XXX 7 bytes hole, try to pack */

	struct powercap_zone_constraint * constraints;   /*  1160     8 */

	/* size: 1168, cachelines: 19, members: 15 */
	/* sum members: 1149, holes: 4, sum holes: 19 */
	/* forced alignments: 1 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct powercap_zone_constraint {
	int                        id;                   /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct powercap_zone *     power_zone;           /*     8     8 */
	const struct powercap_zone_constraint_ops  * ops; /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct powercap_zone_constraint_ops {
	int                        (*set_power_limit_uw)(struct powercap_zone *, int, u64); /*     0     8 */
	int                        (*get_power_limit_uw)(struct powercap_zone *, int, u64 *); /*     8     8 */
	int                        (*set_time_window_us)(struct powercap_zone *, int, u64); /*    16     8 */
	int                        (*get_time_window_us)(struct powercap_zone *, int, u64 *); /*    24     8 */
	int                        (*get_max_power_uw)(struct powercap_zone *, int, u64 *); /*    32     8 */
	int                        (*get_min_power_uw)(struct powercap_zone *, int, u64 *); /*    40     8 */
	int                        (*get_max_time_window_us)(struct powercap_zone *, int, u64 *); /*    48     8 */
	int                        (*get_min_time_window_us)(struct powercap_zone *, int, u64 *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	const char  *              (*get_name)(struct powercap_zone *, int); /*    64     8 */

	/* size: 72, cachelines: 2, members: 9 */
	/* last cacheline: 8 bytes */
};
struct powercap_constraint_attr {
	struct device_attribute    power_limit_attr;     /*     0    32 */
	struct device_attribute    time_window_attr;     /*    32    32 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct device_attribute    max_power_attr;       /*    64    32 */
	struct device_attribute    min_power_attr;       /*    96    32 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct device_attribute    max_time_window_attr; /*   128    32 */
	struct device_attribute    min_time_window_attr; /*   160    32 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	struct device_attribute    name_attr;            /*   192    32 */

	/* size: 224, cachelines: 4, members: 7 */
	/* last cacheline: 32 bytes */
};
struct cper_arm_ctx_info {
	u16                        version;              /*     0     2 */
	u16                        type;                 /*     2     2 */
	u32                        size;                 /*     4     4 */

	/* size: 8, cachelines: 1, members: 3 */
	/* last cacheline: 8 bytes */
};
struct trace_event_raw_extlog_mem_event {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        err_seq;              /*     8     4 */
	u8                         etype;                /*    12     1 */
	u8                         sev;                  /*    13     1 */

	/* XXX 2 bytes hole, try to pack */

	u64                        pa;                   /*    16     8 */
	u8                         pa_mask_lsb;          /*    24     1 */
	guid_t                     fru_id;               /*    25    16 */

	/* XXX 3 bytes hole, try to pack */

	u32                        __data_loc_fru_text;  /*    44     4 */
	struct cper_mem_err_compact data;                /*    48    55 */
	/* --- cacheline 1 boundary (64 bytes) was 39 bytes ago --- */
	char                       __data[];             /*   103     0 */

	/* size: 104, cachelines: 2, members: 10 */
	/* sum members: 98, holes: 2, sum holes: 5 */
	/* padding: 1 */
	/* last cacheline: 40 bytes */
};
struct trace_event_raw_mc_event {
	struct trace_entry         ent;                  /*     0     8 */
	unsigned int               error_type;           /*     8     4 */
	u32                        __data_loc_msg;       /*    12     4 */
	u32                        __data_loc_label;     /*    16     4 */
	u16                        error_count;          /*    20     2 */
	u8                         mc_index;             /*    22     1 */
	s8                         top_layer;            /*    23     1 */
	s8                         middle_layer;         /*    24     1 */
	s8                         lower_layer;          /*    25     1 */

	/* XXX 6 bytes hole, try to pack */

	long int                   address;              /*    32     8 */
	u8                         grain_bits;           /*    40     1 */

	/* XXX 7 bytes hole, try to pack */

	long int                   syndrome;             /*    48     8 */
	u32                        __data_loc_driver_detail; /*    56     4 */
	char                       __data[];             /*    60     0 */

	/* size: 64, cachelines: 1, members: 14 */
	/* sum members: 47, holes: 2, sum holes: 13 */
	/* padding: 4 */
};
struct trace_event_raw_arm_event {
	struct trace_entry         ent;                  /*     0     8 */
	u64                        mpidr;                /*     8     8 */
	u64                        midr;                 /*    16     8 */
	u32                        running_state;        /*    24     4 */
	u32                        psci_state;           /*    28     4 */
	u8                         affinity;             /*    32     1 */

	/* XXX 3 bytes hole, try to pack */

	u32                        pei_len;              /*    36     4 */
	u32                        __data_loc_buf;       /*    40     4 */
	u32                        ctx_len;              /*    44     4 */
	u32                        __data_loc_buf1;      /*    48     4 */
	u32                        oem_len;              /*    52     4 */
	u32                        __data_loc_buf2;      /*    56     4 */
	u8                         sev;                  /*    60     1 */

	/* XXX 3 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        cpu;                  /*    64     4 */
	char                       __data[];             /*    68     0 */

	/* size: 72, cachelines: 2, members: 15 */
	/* sum members: 62, holes: 2, sum holes: 6 */
	/* padding: 4 */
	/* last cacheline: 8 bytes */
};
struct trace_event_raw_non_standard_event {
	struct trace_entry         ent;                  /*     0     8 */
	char                       sec_type[16];         /*     8    16 */
	char                       fru_id[16];           /*    24    16 */
	u32                        __data_loc_fru_text;  /*    40     4 */
	u8                         sev;                  /*    44     1 */

	/* XXX 3 bytes hole, try to pack */

	u32                        len;                  /*    48     4 */
	u32                        __data_loc_buf;       /*    52     4 */
	char                       __data[];             /*    56     0 */

	/* size: 56, cachelines: 1, members: 8 */
	/* sum members: 53, holes: 1, sum holes: 3 */
	/* last cacheline: 56 bytes */
};
struct trace_event_raw_aer_event {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        __data_loc_dev_name;  /*     8     4 */
	u32                        status;               /*    12     4 */
	u8                         severity;             /*    16     1 */
	u8                         tlp_header_valid;     /*    17     1 */

	/* XXX 2 bytes hole, try to pack */

	u32                        tlp_header[4];        /*    20    16 */
	char                       __data[];             /*    36     0 */

	/* size: 36, cachelines: 1, members: 7 */
	/* sum members: 34, holes: 1, sum holes: 2 */
	/* last cacheline: 36 bytes */
};
struct trace_event_raw_memory_failure_event {
	struct trace_entry         ent;                  /*     0     8 */
	long unsigned int          pfn;                  /*     8     8 */
	int                        type;                 /*    16     4 */
	int                        result;               /*    20     4 */
	char                       __data[];             /*    24     0 */

	/* size: 24, cachelines: 1, members: 5 */
	/* last cacheline: 24 bytes */
};
struct trace_event_data_offsets_extlog_mem_event {
	u32                        fru_text;             /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct trace_event_data_offsets_mc_event {
	u32                        msg;                  /*     0     4 */
	u32                        label;                /*     4     4 */
	u32                        driver_detail;        /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct trace_event_data_offsets_arm_event {
	u32                        buf;                  /*     0     4 */
	u32                        buf1;                 /*     4     4 */
	u32                        buf2;                 /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct trace_event_data_offsets_non_standard_event {
	u32                        fru_text;             /*     0     4 */
	u32                        buf;                  /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct trace_event_data_offsets_aer_event {
	u32                        dev_name;             /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct trace_event_data_offsets_memory_failure_event {

	/* size: 0, cachelines: 0, members: 0 */
};
struct nvmem_cell_lookup {
	const char  *              nvmem_name;           /*     0     8 */
	const char  *              cell_name;            /*     8     8 */
	const char  *              dev_id;               /*    16     8 */
	const char  *              con_id;               /*    24     8 */
	struct list_head           node;                 /*    32    16 */

	/* size: 48, cachelines: 1, members: 5 */
	/* last cacheline: 48 bytes */
};
struct nvmem_cell_table {
	const char  *              nvmem_name;           /*     0     8 */
	const struct nvmem_cell_info  * cells;           /*     8     8 */
	size_t                     ncells;               /*    16     8 */
	struct list_head           node;                 /*    24    16 */

	/* size: 40, cachelines: 1, members: 4 */
	/* last cacheline: 40 bytes */
};
struct nvmem_device {
	struct module *            owner;                /*     0     8 */
	struct device              dev __attribute__((__aligned__(8))); /*     8  1000 */
	/* --- cacheline 15 boundary (960 bytes) was 48 bytes ago --- */
	int                        stride;               /*  1008     4 */
	int                        word_size;            /*  1012     4 */
	int                        id;                   /*  1016     4 */
	struct kref                refcnt;               /*  1020     4 */
	/* --- cacheline 16 boundary (1024 bytes) --- */
	size_t                     size;                 /*  1024     8 */
	bool                       read_only;            /*  1032     1 */
	bool                       root_only;            /*  1033     1 */

	/* XXX 2 bytes hole, try to pack */

	int                        flags;                /*  1036     4 */
	enum nvmem_type            type;                 /*  1040     4 */

	/* XXX 4 bytes hole, try to pack */

	struct bin_attribute       eeprom;               /*  1048    64 */
	/* --- cacheline 17 boundary (1088 bytes) was 24 bytes ago --- */
	struct device *            base_dev;             /*  1112     8 */
	struct list_head           cells;                /*  1120    16 */
	nvmem_reg_read_t           reg_read;             /*  1136     8 */
	nvmem_reg_write_t          reg_write;            /*  1144     8 */
	/* --- cacheline 18 boundary (1152 bytes) --- */
	struct gpio_desc *         wp_gpio;              /*  1152     8 */
	void *                     priv;                 /*  1160     8 */

	/* size: 1168, cachelines: 19, members: 18 */
	/* sum members: 1162, holes: 2, sum holes: 6 */
	/* forced alignments: 1 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct nvmem_cell {
	const char  *              name;                 /*     0     8 */
	int                        offset;               /*     8     4 */
	int                        bytes;                /*    12     4 */
	int                        bit_offset;           /*    16     4 */
	int                        nbits;                /*    20     4 */
	struct device_node *       np;                   /*    24     8 */
	struct nvmem_device *      nvmem;                /*    32     8 */
	struct list_head           node;                 /*    40    16 */

	/* size: 56, cachelines: 1, members: 8 */
	/* last cacheline: 56 bytes */
};
struct net_device_devres {
	struct net_device *        ndev;                 /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct __kernel_old_timespec {
	__kernel_old_time_t        tv_sec;               /*     0     8 */
	long int                   tv_nsec;              /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct __kernel_sock_timeval {
	__s64                      tv_sec;               /*     0     8 */
	__s64                      tv_usec;              /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct mmsghdr {
	struct user_msghdr         msg_hdr;              /*     0    56 */

	/* XXX last struct has 4 bytes of padding */

	unsigned int               msg_len;              /*    56     4 */

	/* size: 64, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* paddings: 1, sum paddings: 4 */
};
struct scm_timestamping_internal {
	struct timespec64          ts[3];                /*     0    48 */

	/* size: 48, cachelines: 1, members: 1 */
	/* last cacheline: 48 bytes */
};
struct ifconf {
	int                        ifc_len;              /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	union {
		char *             ifcu_buf;             /*     8     8 */
		struct ifreq *     ifcu_req;             /*     8     8 */
	} ifc_ifcu;                                      /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct compat_ifmap {
	compat_ulong_t             mem_start;            /*     0     4 */
	compat_ulong_t             mem_end;              /*     4     4 */
	short unsigned int         base_addr;            /*     8     2 */
	unsigned char              irq;                  /*    10     1 */
	unsigned char              dma;                  /*    11     1 */
	unsigned char              port;                 /*    12     1 */

	/* size: 16, cachelines: 1, members: 6 */
	/* padding: 3 */
	/* last cacheline: 16 bytes */
};
struct compat_if_settings {
	unsigned int               type;                 /*     0     4 */
	unsigned int               size;                 /*     4     4 */
	compat_uptr_t              ifs_ifsu;             /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct compat_ifreq {
	union {
		char               ifrn_name[16];        /*     0    16 */
	} ifr_ifrn;                                      /*     0    16 */
	union {
		struct sockaddr    ifru_addr;            /*    16    16 */
		struct sockaddr    ifru_dstaddr;         /*    16    16 */
		struct sockaddr    ifru_broadaddr;       /*    16    16 */
		struct sockaddr    ifru_netmask;         /*    16    16 */
		struct sockaddr    ifru_hwaddr;          /*    16    16 */
		short int          ifru_flags;           /*    16     2 */
		compat_int_t       ifru_ivalue;          /*    16     4 */
		compat_int_t       ifru_mtu;             /*    16     4 */
		struct compat_ifmap ifru_map;            /*    16    16 */
		char               ifru_slave[16];       /*    16    16 */
		char               ifru_newname[16];     /*    16    16 */
		compat_caddr_t     ifru_data;            /*    16     4 */
		struct compat_if_settings ifru_settings; /*    16    12 */
	} ifr_ifru;                                      /*    16    16 */

	/* size: 32, cachelines: 1, members: 2 */
	/* last cacheline: 32 bytes */
};
struct compat_ifconf {
	compat_int_t               ifc_len;              /*     0     4 */
	compat_caddr_t             ifcbuf;               /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct compat_mmsghdr {
	struct compat_msghdr       msg_hdr;              /*     0    28 */
	compat_uint_t              msg_len;              /*    28     4 */

	/* size: 32, cachelines: 1, members: 2 */
	/* last cacheline: 32 bytes */
};
struct scm_ts_pktinfo {
	__u32                      if_index;             /*     0     4 */
	__u32                      pkt_length;           /*     4     4 */
	__u32                      reserved[2];          /*     8     8 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct sock_skb_cb {
	u32                        dropcount;            /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct sock_ee_data_rfc4884 {
	__u16                      len;                  /*     0     2 */
	__u8                       flags;                /*     2     1 */
	__u8                       reserved;             /*     3     1 */

	/* size: 4, cachelines: 1, members: 3 */
	/* last cacheline: 4 bytes */
};
struct sock_extended_err {
	__u32                      ee_errno;             /*     0     4 */
	__u8                       ee_origin;            /*     4     1 */
	__u8                       ee_type;              /*     5     1 */
	__u8                       ee_code;              /*     6     1 */
	__u8                       ee_pad;               /*     7     1 */
	__u32                      ee_info;              /*     8     4 */
	union {
		__u32              ee_data;              /*    12     4 */
		struct sock_ee_data_rfc4884 ee_rfc4884;  /*    12     4 */
	};                                               /*    12     4 */

	/* size: 16, cachelines: 1, members: 7 */
	/* last cacheline: 16 bytes */
};
struct sock_exterr_skb {
	union {
		struct inet_skb_parm h4;                 /*     0    24 */
		struct inet6_skb_parm h6;                /*     0    24 */
	} header;                                        /*     0    24 */
	struct sock_extended_err   ee;                   /*    24    16 */
	u16                        addr_offset;          /*    40     2 */
	__be16                     port;                 /*    42     2 */
	u8                         opt_stats:1;          /*    44: 0  1 */
	u8                         unused:7;             /*    44: 1  1 */

	/* size: 48, cachelines: 1, members: 6 */
	/* padding: 3 */
	/* last cacheline: 48 bytes */
};
struct used_address {
	struct __kernel_sockaddr_storage name;           /*     0   128 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	unsigned int               name_len;             /*   128     4 */

	/* size: 136, cachelines: 3, members: 2 */
	/* padding: 4 */
	/* last cacheline: 8 bytes */
};
struct linger {
	int                        l_onoff;              /*     0     4 */
	int                        l_linger;             /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct ucred {
	__u32                      pid;                  /*     0     4 */
	__u32                      uid;                  /*     4     4 */
	__u32                      gid;                  /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct mmpin {
	struct user_struct *       user;                 /*     0     8 */
	unsigned int               num_pg;               /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct ubuf_info {
	void                       (*callback)(struct ubuf_info *, bool); /*     0     8 */
	union {
		struct {
			long unsigned int desc;          /*     8     8 */
			void *     ctx;                  /*    16     8 */
		};                                       /*     8    16 */
		struct {
			u32        id;                   /*     8     4 */
			u16        len;                  /*    12     2 */
			u16        zerocopy:1;           /*    14: 0  2 */

			/* XXX 15 bits hole, try to pack */

			u32        bytelen;              /*    16     4 */
		};                                       /*     8    12 */
	};                                               /*     8    16 */
	refcount_t                 refcnt;               /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	struct mmpin               mmp;                  /*    32    16 */

	/* XXX last struct has 4 bytes of padding */

	/* size: 48, cachelines: 1, members: 4 */
	/* sum members: 44, holes: 1, sum holes: 4 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 48 bytes */
};
struct prot_inuse {
	int                        val[64];              /*     0   256 */

	/* size: 256, cachelines: 4, members: 1 */
};
struct gro_list {
	struct list_head           list;                 /*     0    16 */
	int                        count;                /*    16     4 */

	/* size: 24, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct napi_struct {
	struct list_head           poll_list;            /*     0    16 */
	long unsigned int          state;                /*    16     8 */
	int                        weight;               /*    24     4 */
	int                        defer_hard_irqs_count; /*    28     4 */
	long unsigned int          gro_bitmask;          /*    32     8 */
	int                        (*poll)(struct napi_struct *, int); /*    40     8 */
	int                        poll_owner;           /*    48     4 */

	/* XXX 4 bytes hole, try to pack */

	struct net_device *        dev;                  /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct gro_list            gro_hash[8];          /*    64   192 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	struct sk_buff *           skb;                  /*   256     8 */
	struct list_head           rx_list;              /*   264    16 */
	int                        rx_count;             /*   280     4 */

	/* XXX 4 bytes hole, try to pack */

	struct hrtimer             timer __attribute__((__aligned__(8))); /*   288    96 */
	/* --- cacheline 6 boundary (384 bytes) --- */
	struct list_head           dev_list;             /*   384    16 */
	struct hlist_node          napi_hash_node;       /*   400    16 */
	unsigned int               napi_id;              /*   416     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        kabi_reserved1;       /*   424     8 */
	u64                        kabi_reserved2;       /*   432     8 */
	u64                        kabi_reserved3;       /*   440     8 */
	/* --- cacheline 7 boundary (448 bytes) --- */
	u64                        kabi_reserved4;       /*   448     8 */

	/* size: 456, cachelines: 8, members: 20 */
	/* sum members: 444, holes: 3, sum holes: 12 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 4 */
	/* last cacheline: 8 bytes */
} __attribute__((__aligned__(8)));
struct sd_flow_limit {
	u64                        count;                /*     0     8 */
	unsigned int               num_buckets;          /*     8     4 */
	unsigned int               history_head;         /*    12     4 */
	u16                        history[128];         /*    16   256 */
	/* --- cacheline 4 boundary (256 bytes) was 16 bytes ago --- */
	u8                         buckets[];            /*   272     0 */

	/* size: 272, cachelines: 5, members: 5 */
	/* last cacheline: 16 bytes */
};
struct softnet_data {
	struct list_head           poll_list;            /*     0    16 */
	struct sk_buff_head        process_queue;        /*    16    24 */
	unsigned int               processed;            /*    40     4 */
	unsigned int               time_squeeze;         /*    44     4 */
	unsigned int               received_rps;         /*    48     4 */

	/* XXX 4 bytes hole, try to pack */

	struct softnet_data *      rps_ipi_list;         /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct sd_flow_limit *     flow_limit;           /*    64     8 */
	struct Qdisc *             output_queue;         /*    72     8 */
	struct Qdisc * *           output_queue_tailp;   /*    80     8 */
	struct sk_buff *           completion_queue;     /*    88     8 */
	struct sk_buff_head        xfrm_backlog;         /*    96    24 */
	struct {
		u16                recursion;            /*   120     2 */
		u8                 more;                 /*   122     1 */
	} xmit;                                          /*   120     4 */

	/* XXX last struct has 1 byte of padding */
	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 2 boundary (128 bytes) --- */
	unsigned int               input_queue_head __attribute__((__aligned__(64))); /*   128     4 */

	/* XXX 60 bytes hole, try to pack */

	/* --- cacheline 3 boundary (192 bytes) --- */
	call_single_data_t         csd __attribute__((__aligned__(64))); /*   192    32 */
	struct softnet_data *      rps_ipi_next;         /*   224     8 */
	unsigned int               cpu;                  /*   232     4 */
	unsigned int               input_queue_tail;     /*   236     4 */
	unsigned int               dropped;              /*   240     4 */

	/* XXX 4 bytes hole, try to pack */

	struct sk_buff_head        input_pkt_queue;      /*   248    24 */
	/* --- cacheline 4 boundary (256 bytes) was 16 bytes ago --- */
	struct napi_struct         backlog __attribute__((__aligned__(8))); /*   272   456 */

	/* size: 768, cachelines: 12, members: 20 */
	/* sum members: 656, holes: 4, sum holes: 72 */
	/* padding: 40 */
	/* paddings: 1, sum paddings: 1 */
	/* forced alignments: 3, forced holes: 2, sum forced holes: 64 */
} __attribute__((__aligned__(64)));
struct sock_txtime {
	__kernel_clockid_t         clockid;              /*     0     4 */
	__u32                      flags;                /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct sockcm_cookie {
	u64                        transmit_time;        /*     0     8 */
	u32                        mark;                 /*     8     4 */
	u16                        tsflags;              /*    12     2 */

	/* size: 16, cachelines: 1, members: 3 */
	/* padding: 2 */
	/* last cacheline: 16 bytes */
};
struct fastopen_queue {
	struct request_sock *      rskq_rst_head;        /*     0     8 */
	struct request_sock *      rskq_rst_tail;        /*     8     8 */
	spinlock_t                 lock;                 /*    16     4 */
	int                        qlen;                 /*    20     4 */
	int                        max_qlen;             /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	struct tcp_fastopen_context * ctx;               /*    32     8 */

	/* size: 40, cachelines: 1, members: 6 */
	/* sum members: 36, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct request_sock_queue {
	spinlock_t                 rskq_lock;            /*     0     4 */
	u8                         rskq_defer_accept;    /*     4     1 */

	/* XXX 3 bytes hole, try to pack */

	u32                        synflood_warned;      /*     8     4 */
	atomic_t                   qlen;                 /*    12     4 */
	atomic_t                   young;                /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	struct request_sock *      rskq_accept_head;     /*    24     8 */
	struct request_sock *      rskq_accept_tail;     /*    32     8 */
	struct fastopen_queue      fastopenq;            /*    40    40 */

	/* size: 80, cachelines: 2, members: 8 */
	/* sum members: 73, holes: 2, sum holes: 7 */
	/* last cacheline: 16 bytes */
};
struct inet_connection_sock_af_ops {
	int                        (*queue_xmit)(struct sock *, struct sk_buff *, struct flowi *); /*     0     8 */
	void                       (*send_check)(struct sock *, struct sk_buff *); /*     8     8 */
	int                        (*rebuild_header)(struct sock *); /*    16     8 */
	void                       (*sk_rx_dst_set)(struct sock *, const struct sk_buff  *); /*    24     8 */
	int                        (*conn_request)(struct sock *, struct sk_buff *); /*    32     8 */
	struct sock *              (*syn_recv_sock)(const struct sock  *, struct sk_buff *, struct request_sock *, struct dst_entry *, struct request_sock *, bool *); /*    40     8 */
	u16                        net_header_len;       /*    48     2 */
	u16                        net_frag_header_len;  /*    50     2 */
	u16                        sockaddr_len;         /*    52     2 */

	/* XXX 2 bytes hole, try to pack */

	int                        (*setsockopt)(struct sock *, int, int, sockptr_t, unsigned int); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*getsockopt)(struct sock *, int, int, char *, int *); /*    64     8 */
	void                       (*addr2sockaddr)(struct sock *, struct sockaddr *); /*    72     8 */
	void                       (*mtu_reduced)(struct sock *); /*    80     8 */
	u64                        kabi_reserved1;       /*    88     8 */

	/* size: 96, cachelines: 2, members: 14 */
	/* sum members: 94, holes: 1, sum holes: 2 */
	/* last cacheline: 32 bytes */
};
struct inet_connection_sock {
	struct inet_sock           icsk_inet __attribute__((__aligned__(8))); /*     0  1096 */
	/* --- cacheline 17 boundary (1088 bytes) was 8 bytes ago --- */
	struct request_sock_queue  icsk_accept_queue;    /*  1096    80 */
	/* --- cacheline 18 boundary (1152 bytes) was 24 bytes ago --- */
	struct inet_bind_bucket *  icsk_bind_hash;       /*  1176     8 */
	long unsigned int          icsk_timeout;         /*  1184     8 */
	struct timer_list          icsk_retransmit_timer; /*  1192    72 */
	/* --- cacheline 19 boundary (1216 bytes) was 48 bytes ago --- */
	struct timer_list          icsk_delack_timer;    /*  1264    72 */
	/* --- cacheline 20 boundary (1280 bytes) was 56 bytes ago --- */
	__u32                      icsk_rto;             /*  1336     4 */
	__u32                      icsk_rto_min;         /*  1340     4 */
	/* --- cacheline 21 boundary (1344 bytes) --- */
	__u32                      icsk_delack_max;      /*  1344     4 */
	__u32                      icsk_pmtu_cookie;     /*  1348     4 */
	const struct tcp_congestion_ops  * icsk_ca_ops;  /*  1352     8 */
	const struct inet_connection_sock_af_ops  * icsk_af_ops; /*  1360     8 */
	const struct tcp_ulp_ops  * icsk_ulp_ops;        /*  1368     8 */
	void *                     icsk_ulp_data;        /*  1376     8 */
	void                       (*icsk_clean_acked)(struct sock *, u32); /*  1384     8 */
	struct hlist_node          icsk_listen_portaddr_node; /*  1392    16 */
	/* --- cacheline 22 boundary (1408 bytes) --- */
	unsigned int               (*icsk_sync_mss)(struct sock *, u32); /*  1408     8 */
	__u8                       icsk_ca_state:5;      /*  1416: 0  1 */
	__u8                       icsk_ca_initialized:1; /*  1416: 5  1 */
	__u8                       icsk_ca_setsockopt:1; /*  1416: 6  1 */
	__u8                       icsk_ca_dst_locked:1; /*  1416: 7  1 */
	__u8                       icsk_retransmits;     /*  1417     1 */
	__u8                       icsk_pending;         /*  1418     1 */
	__u8                       icsk_backoff;         /*  1419     1 */
	__u8                       icsk_syn_retries;     /*  1420     1 */
	__u8                       icsk_probes_out;      /*  1421     1 */
	__u16                      icsk_ext_hdr_len;     /*  1422     2 */
	struct {
		__u8               pending;              /*  1424     1 */
		__u8               quick;                /*  1425     1 */
		__u8               pingpong;             /*  1426     1 */
		__u8               retry;                /*  1427     1 */
		__u32              ato;                  /*  1428     4 */
		long unsigned int  timeout;              /*  1432     8 */
		__u32              lrcvtime;             /*  1440     4 */
		__u16              last_seg_size;        /*  1444     2 */
		__u16              rcv_mss;              /*  1446     2 */
	} icsk_ack;                                      /*  1424    24 */
	struct {
		int                enabled;              /*  1448     4 */
		int                search_high;          /*  1452     4 */
		int                search_low;           /*  1456     4 */
		int                probe_size;           /*  1460     4 */
		u32                probe_timestamp;      /*  1464     4 */
	} icsk_mtup;                                     /*  1448    20 */
	u32                        icsk_probes_tstamp;   /*  1468     4 */
	/* --- cacheline 23 boundary (1472 bytes) --- */
	u32                        icsk_user_timeout;    /*  1472     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        kabi_reserved1;       /*  1480     8 */
	u64                        icsk_ca_priv[13];     /*  1488   104 */

	/* size: 1592, cachelines: 25, members: 33 */
	/* sum members: 1587, holes: 1, sum holes: 4 */
	/* sum bitfield members: 8 bits (1 bytes) */
	/* forced alignments: 1 */
	/* last cacheline: 56 bytes */
} __attribute__((__aligned__(8)));
struct inet_bind_bucket {
	possible_net_t             ib_net;               /*     0     8 */
	int                        l3mdev;               /*     8     4 */
	short unsigned int         port;                 /*    12     2 */
	signed char                fastreuse;            /*    14     1 */
	signed char                fastreuseport;        /*    15     1 */
	kuid_t                     fastuid;              /*    16     4 */
	struct in6_addr            fast_v6_rcv_saddr;    /*    20    16 */
	__be32                     fast_rcv_saddr;       /*    36     4 */
	short unsigned int         fast_sk_family;       /*    40     2 */
	bool                       fast_ipv6_only;       /*    42     1 */

	/* XXX 5 bytes hole, try to pack */

	struct hlist_node          node;                 /*    48    16 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct hlist_head          owners;               /*    64     8 */

	/* size: 72, cachelines: 2, members: 12 */
	/* sum members: 67, holes: 1, sum holes: 5 */
	/* last cacheline: 8 bytes */
};
struct tcp_ulp_ops {
	struct list_head           list;                 /*     0    16 */
	int                        (*init)(struct sock *); /*    16     8 */
	void                       (*update)(struct sock *, struct proto *, void (*)(struct sock *)); /*    24     8 */
	void                       (*release)(struct sock *); /*    32     8 */
	int                        (*get_info)(const struct sock  *, struct sk_buff *); /*    40     8 */
	size_t                     (*get_info_size)(const struct sock  *); /*    48     8 */
	void                       (*clone)(const struct request_sock  *, struct sock *, const gfp_t ); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	char                       name[16];             /*    64    16 */
	struct module *            owner;                /*    80     8 */

	/* size: 88, cachelines: 2, members: 9 */
	/* last cacheline: 24 bytes */
};
struct tcp_fastopen_cookie {
	__le64                     val[2];               /*     0    16 */
	s8                         len;                  /*    16     1 */
	bool                       exp;                  /*    17     1 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 6 */
	/* last cacheline: 24 bytes */
};
struct tcp_sack_block {
	u32                        start_seq;            /*     0     4 */
	u32                        end_seq;              /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct tcp_options_received {
	int                        ts_recent_stamp;      /*     0     4 */
	u32                        ts_recent;            /*     4     4 */
	u32                        rcv_tsval;            /*     8     4 */
	u32                        rcv_tsecr;            /*    12     4 */
	u16                        saw_tstamp:1;         /*    16: 0  2 */
	u16                        tstamp_ok:1;          /*    16: 1  2 */
	u16                        dsack:1;              /*    16: 2  2 */
	u16                        wscale_ok:1;          /*    16: 3  2 */
	u16                        sack_ok:3;            /*    16: 4  2 */
	u16                        smc_ok:1;             /*    16: 7  2 */
	u16                        snd_wscale:4;         /*    16: 8  2 */
	u16                        rcv_wscale:4;         /*    16:12  2 */
	u8                         saw_unknown:1;        /*    18: 0  1 */
	u8                         comp_ok:1;            /*    18: 1  1 */
	u8                         unused:6;             /*    18: 2  1 */
	u8                         num_sacks;            /*    19     1 */
	u16                        user_mss;             /*    20     2 */
	u16                        mss_clamp;            /*    22     2 */

	/* size: 24, cachelines: 1, members: 18 */
	/* last cacheline: 24 bytes */
};
struct tcp_rack {
	u64                        mstamp;               /*     0     8 */
	u32                        rtt_us;               /*     8     4 */
	u32                        end_seq;              /*    12     4 */
	u32                        last_delivered;       /*    16     4 */
	u8                         reo_wnd_steps;        /*    20     1 */
	u8                         reo_wnd_persist:5;    /*    21: 0  1 */
	u8                         dsack_seen:1;         /*    21: 5  1 */
	u8                         advanced:1;           /*    21: 6  1 */

	/* size: 24, cachelines: 1, members: 8 */
	/* padding: 2 */
	/* bit_padding: 1 bits */
	/* last cacheline: 24 bytes */
};
struct tcp_sock {
	struct inet_connection_sock inet_conn __attribute__((__aligned__(8))); /*     0  1592 */
	/* --- cacheline 24 boundary (1536 bytes) was 56 bytes ago --- */
	u16                        tcp_header_len;       /*  1592     2 */
	u16                        gso_segs;             /*  1594     2 */
	__be32                     pred_flags;           /*  1596     4 */
	/* --- cacheline 25 boundary (1600 bytes) --- */
	u64                        bytes_received;       /*  1600     8 */
	u32                        segs_in;              /*  1608     4 */
	u32                        data_segs_in;         /*  1612     4 */
	u32                        rcv_nxt;              /*  1616     4 */
	u32                        copied_seq;           /*  1620     4 */
	u32                        rcv_wup;              /*  1624     4 */
	u32                        snd_nxt;              /*  1628     4 */
	u32                        segs_out;             /*  1632     4 */
	u32                        data_segs_out;        /*  1636     4 */
	u64                        bytes_sent;           /*  1640     8 */
	u64                        bytes_acked;          /*  1648     8 */
	u32                        dsack_dups;           /*  1656     4 */
	u32                        snd_una;              /*  1660     4 */
	/* --- cacheline 26 boundary (1664 bytes) --- */
	u32                        snd_sml;              /*  1664     4 */
	u32                        rcv_tstamp;           /*  1668     4 */
	u32                        lsndtime;             /*  1672     4 */
	u32                        last_oow_ack_time;    /*  1676     4 */
	u32                        compressed_ack_rcv_nxt; /*  1680     4 */
	u32                        tsoffset;             /*  1684     4 */
	struct list_head           tsq_node;             /*  1688    16 */
	struct list_head           tsorted_sent_queue;   /*  1704    16 */
	u32                        snd_wl1;              /*  1720     4 */
	u32                        snd_wnd;              /*  1724     4 */
	/* --- cacheline 27 boundary (1728 bytes) --- */
	u32                        max_window;           /*  1728     4 */
	u32                        mss_cache;            /*  1732     4 */
	u32                        window_clamp;         /*  1736     4 */
	u32                        rcv_ssthresh;         /*  1740     4 */
	struct tcp_rack            rack;                 /*  1744    24 */

	/* XXX last struct has 2 bytes of padding */

	u16                        advmss;               /*  1768     2 */
	u8                         compressed_ack;       /*  1770     1 */
	u8                         dup_ack_counter:2;    /*  1771: 0  1 */
	u8                         tlp_retrans:1;        /*  1771: 2  1 */
	u8                         unused:5;             /*  1771: 3  1 */
	u32                        chrono_start;         /*  1772     4 */
	u32                        chrono_stat[3];       /*  1776    12 */
	u8                         chrono_type:2;        /*  1788: 0  1 */
	u8                         rate_app_limited:1;   /*  1788: 2  1 */
	u8                         fastopen_connect:1;   /*  1788: 3  1 */
	u8                         fastopen_no_cookie:1; /*  1788: 4  1 */
	u8                         is_sack_reneg:1;      /*  1788: 5  1 */
	u8                         fastopen_client_fail:2; /*  1788: 6  1 */
	u8                         nonagle:4;            /*  1789: 0  1 */
	u8                         thin_lto:1;           /*  1789: 4  1 */
	u8                         recvmsg_inq:1;        /*  1789: 5  1 */
	u8                         repair:1;             /*  1789: 6  1 */
	u8                         frto:1;               /*  1789: 7  1 */
	u8                         repair_queue;         /*  1790     1 */
	u8                         save_syn:2;           /*  1791: 0  1 */
	u8                         syn_data:1;           /*  1791: 2  1 */
	u8                         syn_fastopen:1;       /*  1791: 3  1 */
	u8                         syn_fastopen_exp:1;   /*  1791: 4  1 */
	u8                         syn_fastopen_ch:1;    /*  1791: 5  1 */
	u8                         syn_data_acked:1;     /*  1791: 6  1 */
	u8                         is_cwnd_limited:1;    /*  1791: 7  1 */
	/* --- cacheline 28 boundary (1792 bytes) --- */
	u32                        tlp_high_seq;         /*  1792     4 */
	u32                        tcp_tx_delay;         /*  1796     4 */
	u64                        tcp_wstamp_ns;        /*  1800     8 */
	u64                        tcp_clock_cache;      /*  1808     8 */
	u64                        tcp_mstamp;           /*  1816     8 */
	u32                        srtt_us;              /*  1824     4 */
	u32                        mdev_us;              /*  1828     4 */
	u32                        mdev_max_us;          /*  1832     4 */
	u32                        rttvar_us;            /*  1836     4 */
	u32                        rtt_seq;              /*  1840     4 */
	struct minmax              rtt_min;              /*  1844    24 */
	/* --- cacheline 29 boundary (1856 bytes) was 12 bytes ago --- */
	u32                        packets_out;          /*  1868     4 */
	u32                        retrans_out;          /*  1872     4 */
	u32                        max_packets_out;      /*  1876     4 */
	u32                        max_packets_seq;      /*  1880     4 */
	u16                        urg_data;             /*  1884     2 */
	u8                         ecn_flags;            /*  1886     1 */
	u8                         keepalive_probes;     /*  1887     1 */
	u32                        reordering;           /*  1888     4 */
	u32                        reord_seen;           /*  1892     4 */
	u32                        snd_up;               /*  1896     4 */
	struct tcp_options_received rx_opt;              /*  1900    24 */
	/* --- cacheline 30 boundary (1920 bytes) was 4 bytes ago --- */
	u32                        snd_ssthresh;         /*  1924     4 */
	u32                        snd_cwnd;             /*  1928     4 */
	u32                        snd_cwnd_cnt;         /*  1932     4 */
	u32                        snd_cwnd_clamp;       /*  1936     4 */
	u32                        snd_cwnd_used;        /*  1940     4 */
	u32                        snd_cwnd_stamp;       /*  1944     4 */
	u32                        prior_cwnd;           /*  1948     4 */
	u32                        prr_delivered;        /*  1952     4 */
	u32                        prr_out;              /*  1956     4 */
	u32                        delivered;            /*  1960     4 */
	u32                        delivered_ce;         /*  1964     4 */
	u32                        lost;                 /*  1968     4 */
	u32                        app_limited;          /*  1972     4 */
	u64                        first_tx_mstamp;      /*  1976     8 */
	/* --- cacheline 31 boundary (1984 bytes) --- */
	u64                        delivered_mstamp;     /*  1984     8 */
	u32                        rate_delivered;       /*  1992     4 */
	u32                        rate_interval_us;     /*  1996     4 */
	u32                        rcv_wnd;              /*  2000     4 */
	u32                        write_seq;            /*  2004     4 */
	u32                        notsent_lowat;        /*  2008     4 */
	u32                        pushed_seq;           /*  2012     4 */
	u32                        lost_out;             /*  2016     4 */
	u32                        sacked_out;           /*  2020     4 */
	struct hrtimer             pacing_timer __attribute__((__aligned__(8))); /*  2024    96 */
	/* --- cacheline 33 boundary (2112 bytes) was 8 bytes ago --- */
	struct hrtimer             compressed_ack_timer __attribute__((__aligned__(8))); /*  2120    96 */
	/* --- cacheline 34 boundary (2176 bytes) was 40 bytes ago --- */
	struct sk_buff *           lost_skb_hint;        /*  2216     8 */
	struct sk_buff *           retransmit_skb_hint;  /*  2224     8 */
	struct rb_root             out_of_order_queue;   /*  2232     8 */
	/* --- cacheline 35 boundary (2240 bytes) --- */
	struct sk_buff *           ooo_last_skb;         /*  2240     8 */
	struct tcp_sack_block      duplicate_sack[1];    /*  2248     8 */
	struct tcp_sack_block      selective_acks[4];    /*  2256    32 */
	struct tcp_sack_block      recv_sack_cache[4];   /*  2288    32 */
	/* --- cacheline 36 boundary (2304 bytes) was 16 bytes ago --- */
	struct sk_buff *           highest_sack;         /*  2320     8 */
	int                        lost_cnt_hint;        /*  2328     4 */
	u32                        prior_ssthresh;       /*  2332     4 */
	u32                        high_seq;             /*  2336     4 */
	u32                        retrans_stamp;        /*  2340     4 */
	u32                        undo_marker;          /*  2344     4 */
	int                        undo_retrans;         /*  2348     4 */
	u64                        bytes_retrans;        /*  2352     8 */
	u32                        total_retrans;        /*  2360     4 */
	u32                        urg_seq;              /*  2364     4 */
	/* --- cacheline 37 boundary (2368 bytes) --- */
	unsigned int               keepalive_time;       /*  2368     4 */
	unsigned int               keepalive_intvl;      /*  2372     4 */
	int                        linger2;              /*  2376     4 */
	u8                         bpf_sock_ops_cb_flags; /*  2380     1 */

	/* XXX 1 byte hole, try to pack */

	u16                        timeout_rehash;       /*  2382     2 */
	u32                        rcv_ooopack;          /*  2384     4 */
	u32                        rcv_rtt_last_tsecr;   /*  2388     4 */
	struct {
		u32                rtt_us;               /*  2392     4 */
		u32                seq;                  /*  2396     4 */
		u64                time;                 /*  2400     8 */
	} rcv_rtt_est;                                   /*  2392    16 */
	struct {
		u32                space;                /*  2408     4 */
		u32                seq;                  /*  2412     4 */
		u64                time;                 /*  2416     8 */
	} rcvq_space;                                    /*  2408    16 */
	struct {
		u32                probe_seq_start;      /*  2424     4 */
		u32                probe_seq_end;        /*  2428     4 */
	} mtu_probe;                                     /*  2424     8 */
	/* --- cacheline 38 boundary (2432 bytes) --- */
	u32                        mtu_info;             /*  2432     4 */

	/* XXX 4 bytes hole, try to pack */

	const struct tcp_sock_af_ops  * af_specific;     /*  2440     8 */
	struct tcp_md5sig_info *   md5sig_info;          /*  2448     8 */
	struct tcp_fastopen_request * fastopen_req;      /*  2456     8 */
	struct request_sock *      fastopen_rsk;         /*  2464     8 */
	struct saved_syn *         saved_syn;            /*  2472     8 */

	/* size: 2480, cachelines: 39, members: 138 */
	/* sum members: 2471, holes: 2, sum holes: 5 */
	/* sum bitfield members: 32 bits (4 bytes) */
	/* paddings: 1, sum paddings: 2 */
	/* forced alignments: 3 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct tcp_sock_af_ops {
	struct tcp_md5sig_key *    (*md5_lookup)(const struct sock  *, const struct sock  *); /*     0     8 */
	int                        (*calc_md5_hash)(char *, const struct tcp_md5sig_key  *, const struct sock  *, const struct sk_buff  *); /*     8     8 */
	int                        (*md5_parse)(struct sock *, int, sockptr_t, int); /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct tcp_md5sig_info {
	struct hlist_head          head;                 /*     0     8 */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*     8    16 */

	/* size: 24, cachelines: 1, members: 2 */
	/* forced alignments: 1 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct tcp_fastopen_request {
	struct tcp_fastopen_cookie cookie;               /*     0    24 */

	/* XXX last struct has 6 bytes of padding */

	struct msghdr *            data;                 /*    24     8 */
	size_t                     size;                 /*    32     8 */
	int                        copied;               /*    40     4 */

	/* XXX 4 bytes hole, try to pack */

	struct ubuf_info *         uarg;                 /*    48     8 */

	/* size: 56, cachelines: 1, members: 5 */
	/* sum members: 52, holes: 1, sum holes: 4 */
	/* paddings: 1, sum paddings: 6 */
	/* last cacheline: 56 bytes */
};
struct tcp_md5sig_key {
	struct hlist_node          node;                 /*     0    16 */
	u8                         keylen;               /*    16     1 */
	u8                         family;               /*    17     1 */
	u8                         prefixlen;            /*    18     1 */

	/* XXX 1 byte hole, try to pack */

	union tcp_md5_addr         addr;                 /*    20    16 */
	int                        l3index;              /*    36     4 */
	u8                         key[80];              /*    40    80 */
	/* --- cacheline 1 boundary (64 bytes) was 56 bytes ago --- */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*   120    16 */

	/* size: 136, cachelines: 3, members: 8 */
	/* sum members: 135, holes: 1, sum holes: 1 */
	/* forced alignments: 1 */
	/* last cacheline: 8 bytes */
} __attribute__((__aligned__(8)));
struct net_protocol {
	int                        (*early_demux)(struct sk_buff *); /*     0     8 */
	int                        (*early_demux_handler)(struct sk_buff *); /*     8     8 */
	int                        (*handler)(struct sk_buff *); /*    16     8 */
	int                        (*err_handler)(struct sk_buff *, u32); /*    24     8 */
	unsigned int               no_policy:1;          /*    32: 0  4 */
	unsigned int               netns_ok:1;           /*    32: 1  4 */
	unsigned int               icmp_strict_tag_validation:1; /*    32: 2  4 */

	/* size: 40, cachelines: 1, members: 7 */
	/* padding: 4 */
	/* bit_padding: 29 bits */
	/* last cacheline: 40 bytes */
};
struct cgroup_cls_state {
	struct cgroup_subsys_state css __attribute__((__aligned__(8))); /*     0   296 */
	/* --- cacheline 4 boundary (256 bytes) was 40 bytes ago --- */
	u32                        classid;              /*   296     4 */

	/* size: 304, cachelines: 5, members: 2 */
	/* padding: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
union tcp_md5_addr {
	struct in_addr             a4;                 /*     0     4 */
	struct in6_addr            a6;                 /*     0    16 */
};
struct inet_request_sock {
	struct request_sock        req;                  /*     0   256 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	u16                        snd_wscale:4;         /*   256: 0  2 */
	u16                        rcv_wscale:4;         /*   256: 4  2 */
	u16                        tstamp_ok:1;          /*   256: 8  2 */
	u16                        sack_ok:1;            /*   256: 9  2 */
	u16                        wscale_ok:1;          /*   256:10  2 */
	u16                        ecn_ok:1;             /*   256:11  2 */
	u16                        acked:1;              /*   256:12  2 */
	u16                        no_srccheck:1;        /*   256:13  2 */
	u16                        smc_ok:1;             /*   256:14  2 */
	u16                        comp_ok:1;            /*   256:15  2 */

	/* XXX 2 bytes hole, try to pack */

	u32                        ir_mark;              /*   260     4 */
	union {
		struct ip_options_rcu * ireq_opt;        /*   264     8 */
		struct {
			struct ipv6_txoptions * ipv6_opt; /*   264     8 */
			struct sk_buff * pktopts;        /*   272     8 */
		};                                       /*   264    16 */
	};                                               /*   264    16 */

	/* size: 280, cachelines: 5, members: 13 */
	/* sum members: 276, holes: 1, sum holes: 2 */
	/* sum bitfield members: 16 bits (2 bytes) */
	/* last cacheline: 24 bytes */
};
struct tcp_request_sock {
	struct inet_request_sock   req;                  /*     0   280 */
	/* --- cacheline 4 boundary (256 bytes) was 24 bytes ago --- */
	const struct tcp_request_sock_ops  * af_specific; /*   280     8 */
	u64                        snt_synack;           /*   288     8 */
	bool                       tfo_listener;         /*   296     1 */
	bool                       is_mptcp;             /*   297     1 */

	/* XXX 2 bytes hole, try to pack */

	u32                        txhash;               /*   300     4 */
	u32                        rcv_isn;              /*   304     4 */
	u32                        snt_isn;              /*   308     4 */
	u32                        ts_off;               /*   312     4 */
	u32                        last_oow_ack_time;    /*   316     4 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	u32                        rcv_nxt;              /*   320     4 */
	u8                         syn_tos;              /*   324     1 */

	/* size: 328, cachelines: 6, members: 12 */
	/* sum members: 323, holes: 1, sum holes: 2 */
	/* padding: 3 */
	/* last cacheline: 8 bytes */
};
struct nf_conntrack {
	atomic_t                   use;                  /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct sk_buff_fclones {
	struct sk_buff             skb1 __attribute__((__aligned__(8))); /*     0   256 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	struct sk_buff             skb2 __attribute__((__aligned__(8))); /*   256   256 */
	/* --- cacheline 8 boundary (512 bytes) --- */
	refcount_t                 fclone_ref;           /*   512     4 */

	/* size: 520, cachelines: 9, members: 3 */
	/* padding: 4 */
	/* forced alignments: 2 */
	/* last cacheline: 8 bytes */
} __attribute__((__aligned__(8)));
struct skb_seq_state {
	__u32                      lower_offset;         /*     0     4 */
	__u32                      upper_offset;         /*     4     4 */
	__u32                      frag_idx;             /*     8     4 */
	__u32                      stepped_offset;       /*    12     4 */
	struct sk_buff *           root_skb;             /*    16     8 */
	struct sk_buff *           cur_skb;              /*    24     8 */
	__u8 *                     frag_data;            /*    32     8 */

	/* size: 40, cachelines: 1, members: 7 */
	/* last cacheline: 40 bytes */
};
struct skb_checksum_ops {
	__wsum                     (*update)(const void  *, int, __wsum); /*     0     8 */
	__wsum                     (*combine)(__wsum, __wsum, int, int); /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct skb_gso_cb {
	union {
		int                mac_offset;           /*     0     4 */
		int                data_offset;          /*     0     4 */
	};                                               /*     0     4 */
	int                        encap_level;          /*     4     4 */
	__wsum                     csum;                 /*     8     4 */
	__u16                      csum_start;           /*    12     2 */

	/* size: 16, cachelines: 1, members: 4 */
	/* padding: 2 */
	/* last cacheline: 16 bytes */
};
struct napi_gro_cb {
	void *                     frag0;                /*     0     8 */
	unsigned int               frag0_len;            /*     8     4 */
	int                        data_offset;          /*    12     4 */
	u16                        flush;                /*    16     2 */
	u16                        flush_id;             /*    18     2 */
	u16                        count;                /*    20     2 */
	u16                        gro_remcsum_start;    /*    22     2 */
	long unsigned int          age;                  /*    24     8 */
	u16                        proto;                /*    32     2 */
	u8                         same_flow:1;          /*    34: 0  1 */
	u8                         encap_mark:1;         /*    34: 1  1 */
	u8                         csum_valid:1;         /*    34: 2  1 */
	u8                         csum_cnt:3;           /*    34: 3  1 */
	u8                         free:2;               /*    34: 6  1 */
	u8                         is_ipv6:1;            /*    35: 0  1 */
	u8                         is_fou:1;             /*    35: 1  1 */
	u8                         is_atomic:1;          /*    35: 2  1 */
	u8                         recursion_counter:4;  /*    35: 3  1 */
	u8                         is_flist:1;           /*    35: 7  1 */
	__wsum                     csum;                 /*    36     4 */
	struct sk_buff *           last;                 /*    40     8 */

	/* size: 48, cachelines: 1, members: 21 */
	/* last cacheline: 48 bytes */
};
struct vlan_hdr {
	__be16                     h_vlan_TCI;           /*     0     2 */
	__be16                     h_vlan_encapsulated_proto; /*     2     2 */

	/* size: 4, cachelines: 1, members: 2 */
	/* last cacheline: 4 bytes */
};
struct vlan_ethhdr {
	unsigned char              h_dest[6];            /*     0     6 */
	unsigned char              h_source[6];          /*     6     6 */
	__be16                     h_vlan_proto;         /*    12     2 */
	__be16                     h_vlan_TCI;           /*    14     2 */
	__be16                     h_vlan_encapsulated_proto; /*    16     2 */

	/* size: 18, cachelines: 1, members: 5 */
	/* last cacheline: 18 bytes */
};
struct qdisc_walker {
	int                        stop;                 /*     0     4 */
	int                        skip;                 /*     4     4 */
	int                        count;                /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	int                        (*fn)(struct Qdisc *, long unsigned int, struct qdisc_walker *); /*    16     8 */

	/* size: 24, cachelines: 1, members: 4 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct ip_auth_hdr {
	__u8                       nexthdr;              /*     0     1 */
	__u8                       hdrlen;               /*     1     1 */
	__be16                     reserved;             /*     2     2 */
	__be32                     spi;                  /*     4     4 */
	__be32                     seq_no;               /*     8     4 */
	__u8                       auth_data[];          /*    12     0 */

	/* size: 12, cachelines: 1, members: 6 */
	/* last cacheline: 12 bytes */
};
struct frag_hdr {
	__u8                       nexthdr;              /*     0     1 */
	__u8                       reserved;             /*     1     1 */
	__be16                     frag_off;             /*     2     2 */
	__be32                     identification;       /*     4     4 */

	/* size: 8, cachelines: 1, members: 4 */
	/* last cacheline: 8 bytes */
};
struct mpls_shim_hdr {
	__be32                     label_stack_entry;    /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct napi_alloc_cache {
	struct page_frag_cache     page;                 /*     0    24 */

	/* XXX last struct has 7 bytes of padding */

	unsigned int               skb_count;            /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     skb_cache[64];        /*    32   512 */

	/* size: 544, cachelines: 9, members: 3 */
	/* sum members: 540, holes: 1, sum holes: 4 */
	/* paddings: 1, sum paddings: 7 */
	/* last cacheline: 32 bytes */
};
struct scm_cookie {
	struct pid *               pid;                  /*     0     8 */
	struct scm_fp_list *       fp;                   /*     8     8 */
	struct scm_creds           creds;                /*    16    12 */
	u32                        secid;                /*    28     4 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct scm_timestamping {
	struct __kernel_old_timespec ts[3];              /*     0    48 */

	/* size: 48, cachelines: 1, members: 1 */
	/* last cacheline: 48 bytes */
};
struct scm_timestamping64 {
	struct __kernel_timespec   ts[3];                /*     0    48 */

	/* size: 48, cachelines: 1, members: 1 */
	/* last cacheline: 48 bytes */
};
struct gnet_stats_basic {
	__u64                      bytes;                /*     0     8 */
	__u32                      packets;              /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct gnet_stats_rate_est {
	__u32                      bps;                  /*     0     4 */
	__u32                      pps;                  /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct gnet_stats_rate_est64 {
	__u64                      bps;                  /*     0     8 */
	__u64                      pps;                  /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct gnet_estimator {
	signed char                interval;             /*     0     1 */
	unsigned char              ewma_log;             /*     1     1 */

	/* size: 2, cachelines: 1, members: 2 */
	/* last cacheline: 2 bytes */
};
struct net_rate_estimator {
	struct gnet_stats_basic_packed * bstats;         /*     0     8 */
	spinlock_t *               stats_lock;           /*     8     8 */
	seqcount_t *               running;              /*    16     8 */
	struct gnet_stats_basic_cpu * cpu_bstats;        /*    24     8 */
	u8                         ewma_log;             /*    32     1 */
	u8                         intvl_log;            /*    33     1 */

	/* XXX 2 bytes hole, try to pack */

	seqcount_t                 seq;                  /*    36     4 */
	u64                        last_packets;         /*    40     8 */
	u64                        last_bytes;           /*    48     8 */
	u64                        avpps;                /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u64                        avbps;                /*    64     8 */
	long unsigned int          next_jiffies;         /*    72     8 */
	struct timer_list          timer;                /*    80    72 */
	/* --- cacheline 2 boundary (128 bytes) was 24 bytes ago --- */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*   152    16 */

	/* size: 168, cachelines: 3, members: 14 */
	/* sum members: 166, holes: 1, sum holes: 2 */
	/* forced alignments: 1 */
	/* last cacheline: 40 bytes */
} __attribute__((__aligned__(8)));
struct rtgenmsg {
	unsigned char              rtgen_family;         /*     0     1 */

	/* size: 1, cachelines: 1, members: 1 */
	/* last cacheline: 1 bytes */
};
struct pcpu_gen_cookie {
	local_t                    nesting;              /*     0     8 */
	u64                        last;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(16)));
struct gen_cookie {
	struct pcpu_gen_cookie *   local;                /*     0     8 */

	/* XXX 56 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	atomic64_t                 forward_last __attribute__((__aligned__(64))); /*    64     8 */
	atomic64_t                 reverse_last;         /*    72     8 */

	/* size: 128, cachelines: 2, members: 3 */
	/* sum members: 24, holes: 1, sum holes: 56 */
	/* padding: 48 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 56 */
} __attribute__((__aligned__(64)));
struct net_fill_args {
	u32                        portid;               /*     0     4 */
	u32                        seq;                  /*     4     4 */
	int                        flags;                /*     8     4 */
	int                        cmd;                  /*    12     4 */
	int                        nsid;                 /*    16     4 */
	bool                       add_ref;              /*    20     1 */

	/* XXX 3 bytes hole, try to pack */

	int                        ref_nsid;             /*    24     4 */

	/* size: 28, cachelines: 1, members: 7 */
	/* sum members: 25, holes: 1, sum holes: 3 */
	/* last cacheline: 28 bytes */
};
struct rtnl_net_dump_cb {
	struct net *               tgt_net;              /*     0     8 */
	struct net *               ref_net;              /*     8     8 */
	struct sk_buff *           skb;                  /*    16     8 */
	struct net_fill_args       fillargs;             /*    24    28 */
	int                        idx;                  /*    52     4 */
	int                        s_idx;                /*    56     4 */

	/* size: 64, cachelines: 1, members: 6 */
	/* padding: 4 */
};
struct flow_dissector_key_control {
	u16                        thoff;                /*     0     2 */
	u16                        addr_type;            /*     2     2 */
	u32                        flags;                /*     4     4 */

	/* size: 8, cachelines: 1, members: 3 */
	/* last cacheline: 8 bytes */
};
struct flow_dissector_key_basic {
	__be16                     n_proto;              /*     0     2 */
	u8                         ip_proto;             /*     2     1 */
	u8                         padding;              /*     3     1 */

	/* size: 4, cachelines: 1, members: 3 */
	/* last cacheline: 4 bytes */
};
struct flow_dissector_key_tags {
	u32                        flow_label;           /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct flow_dissector_key_vlan {
	union {
		struct {
			u16        vlan_id:12;           /*     0: 0  2 */
			u16        vlan_dei:1;           /*     0:12  2 */
			u16        vlan_priority:3;      /*     0:13  2 */
		};                                       /*     0     2 */
		__be16             vlan_tci;             /*     0     2 */
	};                                               /*     0     2 */
	__be16                     vlan_tpid;            /*     2     2 */

	/* size: 4, cachelines: 1, members: 2 */
	/* last cacheline: 4 bytes */
};
struct flow_dissector_mpls_lse {
	u32                        mpls_ttl:8;           /*     0: 0  4 */
	u32                        mpls_bos:1;           /*     0: 8  4 */
	u32                        mpls_tc:3;            /*     0: 9  4 */
	u32                        mpls_label:20;        /*     0:12  4 */

	/* size: 4, cachelines: 1, members: 4 */
	/* last cacheline: 4 bytes */
};
struct flow_dissector_key_mpls {
	struct flow_dissector_mpls_lse ls[7];            /*     0    28 */
	u8                         used_lses;            /*    28     1 */

	/* size: 32, cachelines: 1, members: 2 */
	/* padding: 3 */
	/* last cacheline: 32 bytes */
};
struct flow_dissector_key_enc_opts {
	u8                         data[255];            /*     0   255 */
	/* --- cacheline 3 boundary (192 bytes) was 63 bytes ago --- */
	u8                         len;                  /*   255     1 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	__be16                     dst_opt_type;         /*   256     2 */

	/* size: 258, cachelines: 5, members: 3 */
	/* last cacheline: 2 bytes */
};
struct flow_dissector_key_keyid {
	__be32                     keyid;                /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct flow_dissector_key_ipv4_addrs {
	__be32                     src;                  /*     0     4 */
	__be32                     dst;                  /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct flow_dissector_key_ipv6_addrs {
	struct in6_addr            src;                  /*     0    16 */
	struct in6_addr            dst;                  /*    16    16 */

	/* size: 32, cachelines: 1, members: 2 */
	/* last cacheline: 32 bytes */
};
struct flow_dissector_key_tipc {
	__be32                     key;                  /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct flow_dissector_key_addrs {
	union {
		struct flow_dissector_key_ipv4_addrs v4addrs; /*     0     8 */
		struct flow_dissector_key_ipv6_addrs v6addrs; /*     0    32 */
		struct flow_dissector_key_tipc tipckey;  /*     0     4 */
	};                                               /*     0    32 */

	/* size: 32, cachelines: 1, members: 1 */
	/* last cacheline: 32 bytes */
};
struct flow_dissector_key_arp {
	__u32                      sip;                  /*     0     4 */
	__u32                      tip;                  /*     4     4 */
	__u8                       op;                   /*     8     1 */
	unsigned char              sha[6];               /*     9     6 */
	unsigned char              tha[6];               /*    15     6 */

	/* size: 24, cachelines: 1, members: 5 */
	/* padding: 3 */
	/* last cacheline: 24 bytes */
};
struct flow_dissector_key_ports {
	union {
		__be32             ports;                /*     0     4 */
		struct {
			__be16     src;                  /*     0     2 */
			__be16     dst;                  /*     2     2 */
		};                                       /*     0     4 */
	};                                               /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct flow_dissector_key_icmp {
	struct {
		u8                 type;                 /*     0     1 */
		u8                 code;                 /*     1     1 */
	};                                               /*     0     2 */
	u16                        id;                   /*     2     2 */

	/* size: 4, cachelines: 1, members: 2 */
	/* last cacheline: 4 bytes */
};
struct flow_dissector_key_eth_addrs {
	unsigned char              dst[6];               /*     0     6 */
	unsigned char              src[6];               /*     6     6 */

	/* size: 12, cachelines: 1, members: 2 */
	/* last cacheline: 12 bytes */
};
struct flow_dissector_key_tcp {
	__be16                     flags;                /*     0     2 */

	/* size: 2, cachelines: 1, members: 1 */
	/* last cacheline: 2 bytes */
};
struct flow_dissector_key_ip {
	__u8                       tos;                  /*     0     1 */
	__u8                       ttl;                  /*     1     1 */

	/* size: 2, cachelines: 1, members: 2 */
	/* last cacheline: 2 bytes */
};
struct flow_dissector_key_meta {
	int                        ingress_ifindex;      /*     0     4 */
	u16                        ingress_iftype;       /*     4     2 */

	/* size: 8, cachelines: 1, members: 2 */
	/* padding: 2 */
	/* last cacheline: 8 bytes */
};
struct flow_dissector_key_ct {
	u16                        ct_state;             /*     0     2 */
	u16                        ct_zone;              /*     2     2 */
	u32                        ct_mark;              /*     4     4 */
	u32                        ct_labels[4];         /*     8    16 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct flow_dissector_key_hash {
	u32                        hash;                 /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct flow_dissector_key {
	enum flow_dissector_key_id key_id;               /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	size_t                     offset;               /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct flow_dissector {
	unsigned int               used_keys;            /*     0     4 */
	short unsigned int         offset[28];           /*     4    56 */

	/* size: 60, cachelines: 1, members: 2 */
	/* last cacheline: 60 bytes */
};
struct flow_keys_basic {
	struct flow_dissector_key_control control;       /*     0     8 */
	struct flow_dissector_key_basic basic;           /*     8     4 */

	/* size: 12, cachelines: 1, members: 2 */
	/* last cacheline: 12 bytes */
};
struct flow_keys {
	struct flow_dissector_key_control control;       /*     0     8 */
	struct flow_dissector_key_basic basic __attribute__((__aligned__(8))); /*     8     4 */
	struct flow_dissector_key_tags tags;             /*    12     4 */
	struct flow_dissector_key_vlan vlan;             /*    16     4 */
	struct flow_dissector_key_vlan cvlan;            /*    20     4 */
	struct flow_dissector_key_keyid keyid;           /*    24     4 */
	struct flow_dissector_key_ports ports;           /*    28     4 */
	struct flow_dissector_key_icmp icmp;             /*    32     4 */
	struct flow_dissector_key_addrs addrs;           /*    36    32 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	u64                        kabi_reserved1;       /*    72     8 */
	u64                        kabi_reserved2;       /*    80     8 */
	u64                        kabi_reserved3;       /*    88     8 */
	u64                        kabi_reserved4;       /*    96     8 */

	/* size: 104, cachelines: 2, members: 13 */
	/* sum members: 100, holes: 1, sum holes: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 40 bytes */
} __attribute__((__aligned__(8)));
struct flow_keys_digest {
	u8                         data[16];             /*     0    16 */

	/* size: 16, cachelines: 1, members: 1 */
	/* last cacheline: 16 bytes */
};
struct xt_table {
	struct list_head           list;                 /*     0    16 */
	unsigned int               valid_hooks;          /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	struct xt_table_info *     private;              /*    24     8 */
	struct module *            me;                   /*    32     8 */
	u_int8_t                   af;                   /*    40     1 */

	/* XXX 3 bytes hole, try to pack */

	int                        priority;             /*    44     4 */
	int                        (*table_init)(struct net *); /*    48     8 */
	const const char           name[32];             /*    56    32 */

	/* size: 88, cachelines: 2, members: 8 */
	/* sum members: 81, holes: 2, sum holes: 7 */
	/* last cacheline: 24 bytes */
};
union nf_inet_addr {
	__u32                      all[4];             /*     0    16 */
	__be32                     ip;                 /*     0     4 */
	__be32                     ip6[4];             /*     0    16 */
	struct in_addr             in;                 /*     0     4 */
	struct in6_addr            in6;                /*     0    16 */
};
struct ip_ct_tcp_state {
	u_int32_t                  td_end;               /*     0     4 */
	u_int32_t                  td_maxend;            /*     4     4 */
	u_int32_t                  td_maxwin;            /*     8     4 */
	u_int32_t                  td_maxack;            /*    12     4 */
	u_int8_t                   td_scale;             /*    16     1 */
	u_int8_t                   flags;                /*    17     1 */

	/* size: 20, cachelines: 1, members: 6 */
	/* padding: 2 */
	/* last cacheline: 20 bytes */
};
struct ip_ct_tcp {
	struct ip_ct_tcp_state     seen[2];              /*     0    40 */
	u_int8_t                   state;                /*    40     1 */
	u_int8_t                   last_dir;             /*    41     1 */
	u_int8_t                   retrans;              /*    42     1 */
	u_int8_t                   last_index;           /*    43     1 */
	u_int32_t                  last_seq;             /*    44     4 */
	u_int32_t                  last_ack;             /*    48     4 */
	u_int32_t                  last_end;             /*    52     4 */
	u_int16_t                  last_win;             /*    56     2 */
	u_int8_t                   last_wscale;          /*    58     1 */
	u_int8_t                   last_flags;           /*    59     1 */

	/* size: 60, cachelines: 1, members: 11 */
	/* last cacheline: 60 bytes */
};
union nf_conntrack_man_proto {
	__be16                     all;                /*     0     2 */
	struct {
		__be16             port;               /*     0     2 */
	} tcp;                                         /*     0     2 */
	struct {
		__be16             port;               /*     0     2 */
	} udp;                                         /*     0     2 */
	struct {
		__be16             id;                 /*     0     2 */
	} icmp;                                        /*     0     2 */
	struct {
		__be16             port;               /*     0     2 */
	} dccp;                                        /*     0     2 */
	struct {
		__be16             port;               /*     0     2 */
	} sctp;                                        /*     0     2 */
	struct {
		__be16             key;                /*     0     2 */
	} gre;                                         /*     0     2 */
};
struct nf_ct_dccp {
	u_int8_t                   role[2];              /*     0     2 */
	u_int8_t                   state;                /*     2     1 */
	u_int8_t                   last_pkt;             /*     3     1 */
	u_int8_t                   last_dir;             /*     4     1 */

	/* XXX 3 bytes hole, try to pack */

	u_int64_t                  handshake_seq;        /*     8     8 */

	/* size: 16, cachelines: 1, members: 5 */
	/* sum members: 13, holes: 1, sum holes: 3 */
	/* last cacheline: 16 bytes */
};
struct ip_ct_sctp {
	enum sctp_conntrack        state;                /*     0     4 */
	__be32                     vtag[2];              /*     4     8 */
	u8                         last_dir;             /*    12     1 */
	u8                         flags;                /*    13     1 */

	/* size: 16, cachelines: 1, members: 4 */
	/* padding: 2 */
	/* last cacheline: 16 bytes */
};
struct nf_ct_event_notifier {
	int                        (*fcn)(unsigned int, struct nf_ct_event *); /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct nf_exp_event_notifier {
	int                        (*fcn)(unsigned int, struct nf_exp_event *); /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct devlink_port {
	struct list_head           list;                 /*     0    16 */
	struct list_head           param_list;           /*    16    16 */
	struct list_head           region_list;          /*    32    16 */
	struct devlink *           devlink;              /*    48     8 */
	unsigned int               index;                /*    56     4 */
	bool                       registered;           /*    60     1 */

	/* XXX 3 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	spinlock_t                 type_lock;            /*    64     4 */
	enum devlink_port_type     type;                 /*    68     4 */
	enum devlink_port_type     desired_type;         /*    72     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     type_dev;             /*    80     8 */
	struct devlink_port_attrs  attrs;                /*    88    60 */
	/* --- cacheline 2 boundary (128 bytes) was 20 bytes ago --- */
	u8                         attrs_set:1;          /*   148: 0  1 */
	u8                         switch_port:1;        /*   148: 1  1 */

	/* XXX 6 bits hole, try to pack */
	/* XXX 3 bytes hole, try to pack */

	struct delayed_work        type_warn_dw;         /*   152   192 */
	/* --- cacheline 5 boundary (320 bytes) was 24 bytes ago --- */
	struct list_head           reporter_list;        /*   344    16 */
	struct mutex               reporters_lock;       /*   360    32 */

	/* size: 392, cachelines: 7, members: 16 */
	/* sum members: 381, holes: 3, sum holes: 10 */
	/* sum bitfield members: 2 bits, bit holes: 1, sum bit holes: 6 bits */
	/* last cacheline: 8 bytes */
};
struct ip_tunnel_parm {
	char                       name[16];             /*     0    16 */
	int                        link;                 /*    16     4 */
	__be16                     i_flags;              /*    20     2 */
	__be16                     o_flags;              /*    22     2 */
	__be32                     i_key;                /*    24     4 */
	__be32                     o_key;                /*    28     4 */
	struct iphdr               iph;                  /*    32    20 */

	/* size: 52, cachelines: 1, members: 7 */
	/* last cacheline: 52 bytes */
};
struct ip_tunnel_info {
	struct ip_tunnel_key       key;                  /*     0    56 */

	/* XXX last struct has 4 bytes of padding */

	struct dst_cache           dst_cache;            /*    56    16 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	u8                         options_len;          /*    72     1 */
	u8                         mode;                 /*    73     1 */

	/* size: 80, cachelines: 2, members: 4 */
	/* padding: 6 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 16 bytes */
};
union tcp_word_hdr {
	struct tcphdr              hdr;                /*     0    20 */
	__be32                     words[5];           /*     0    20 */
};
struct devlink_dev_stats {
	u32                        reload_stats[6];      /*     0    24 */
	u32                        remote_reload_stats[6]; /*    24    24 */

	/* size: 48, cachelines: 1, members: 2 */
	/* last cacheline: 48 bytes */
};
struct devlink {
	struct list_head           list;                 /*     0    16 */
	struct list_head           port_list;            /*    16    16 */
	struct list_head           sb_list;              /*    32    16 */
	struct list_head           dpipe_table_list;     /*    48    16 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct list_head           resource_list;        /*    64    16 */
	struct list_head           param_list;           /*    80    16 */
	struct list_head           region_list;          /*    96    16 */
	struct list_head           reporter_list;        /*   112    16 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct mutex               reporters_lock;       /*   128    32 */
	struct devlink_dpipe_headers * dpipe_headers;    /*   160     8 */
	struct list_head           trap_list;            /*   168    16 */
	struct list_head           trap_group_list;      /*   184    16 */
	/* --- cacheline 3 boundary (192 bytes) was 8 bytes ago --- */
	struct list_head           trap_policer_list;    /*   200    16 */
	const struct devlink_ops  * ops;                 /*   216     8 */
	struct xarray              snapshot_ids;         /*   224    16 */
	struct devlink_dev_stats   stats;                /*   240    48 */
	/* --- cacheline 4 boundary (256 bytes) was 32 bytes ago --- */
	struct device *            dev;                  /*   288     8 */
	possible_net_t             _net;                 /*   296     8 */
	struct mutex               lock;                 /*   304    32 */
	/* --- cacheline 5 boundary (320 bytes) was 16 bytes ago --- */
	u64                        kabi_reserved1;       /*   336     8 */
	u64                        kabi_reserved2;       /*   344     8 */
	u8                         reload_failed:1;      /*   352: 0  1 */
	u8                         reload_enabled:1;     /*   352: 1  1 */
	u8                         registered:1;         /*   352: 2  1 */

	/* XXX 5 bits hole, try to pack */
	/* XXX 31 bytes hole, try to pack */

	/* --- cacheline 6 boundary (384 bytes) --- */
	char                       priv[] __attribute__((__aligned__(32))); /*   384     0 */

	/* size: 384, cachelines: 6, members: 25 */
	/* sum members: 352, holes: 1, sum holes: 31 */
	/* sum bitfield members: 3 bits, bit holes: 1, sum bit holes: 5 bits */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 31 */
} __attribute__((__aligned__(32)));
struct devlink_dpipe_headers {
	struct devlink_dpipe_header * * headers;         /*     0     8 */
	unsigned int               headers_count;        /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct devlink_ops {
	u32                        supported_flash_update_params; /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          reload_actions;       /*     8     8 */
	long unsigned int          reload_limits;        /*    16     8 */
	int                        (*reload_down)(struct devlink *, bool, enum devlink_reload_action, enum devlink_reload_limit, struct netlink_ext_ack *); /*    24     8 */
	int                        (*reload_up)(struct devlink *, enum devlink_reload_action, enum devlink_reload_limit, u32 *, struct netlink_ext_ack *); /*    32     8 */
	int                        (*port_type_set)(struct devlink_port *, enum devlink_port_type); /*    40     8 */
	int                        (*port_split)(struct devlink *, unsigned int, unsigned int, struct netlink_ext_ack *); /*    48     8 */
	int                        (*port_unsplit)(struct devlink *, unsigned int, struct netlink_ext_ack *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*sb_pool_get)(struct devlink *, unsigned int, u16, struct devlink_sb_pool_info *); /*    64     8 */
	int                        (*sb_pool_set)(struct devlink *, unsigned int, u16, u32, enum devlink_sb_threshold_type, struct netlink_ext_ack *); /*    72     8 */
	int                        (*sb_port_pool_get)(struct devlink_port *, unsigned int, u16, u32 *); /*    80     8 */
	int                        (*sb_port_pool_set)(struct devlink_port *, unsigned int, u16, u32, struct netlink_ext_ack *); /*    88     8 */
	int                        (*sb_tc_pool_bind_get)(struct devlink_port *, unsigned int, u16, enum devlink_sb_pool_type, u16 *, u32 *); /*    96     8 */
	int                        (*sb_tc_pool_bind_set)(struct devlink_port *, unsigned int, u16, enum devlink_sb_pool_type, u16, u32, struct netlink_ext_ack *); /*   104     8 */
	int                        (*sb_occ_snapshot)(struct devlink *, unsigned int); /*   112     8 */
	int                        (*sb_occ_max_clear)(struct devlink *, unsigned int); /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	int                        (*sb_occ_port_pool_get)(struct devlink_port *, unsigned int, u16, u32 *, u32 *); /*   128     8 */
	int                        (*sb_occ_tc_port_bind_get)(struct devlink_port *, unsigned int, u16, enum devlink_sb_pool_type, u32 *, u32 *); /*   136     8 */
	int                        (*eswitch_mode_get)(struct devlink *, u16 *); /*   144     8 */
	int                        (*eswitch_mode_set)(struct devlink *, u16, struct netlink_ext_ack *); /*   152     8 */
	int                        (*eswitch_inline_mode_get)(struct devlink *, u8 *); /*   160     8 */
	int                        (*eswitch_inline_mode_set)(struct devlink *, u8, struct netlink_ext_ack *); /*   168     8 */
	int                        (*eswitch_encap_mode_get)(struct devlink *, enum devlink_eswitch_encap_mode *); /*   176     8 */
	int                        (*eswitch_encap_mode_set)(struct devlink *, enum devlink_eswitch_encap_mode, struct netlink_ext_ack *); /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	int                        (*info_get)(struct devlink *, struct devlink_info_req *, struct netlink_ext_ack *); /*   192     8 */
	int                        (*flash_update)(struct devlink *, struct devlink_flash_update_params *, struct netlink_ext_ack *); /*   200     8 */
	int                        (*trap_init)(struct devlink *, const struct devlink_trap  *, void *); /*   208     8 */
	void                       (*trap_fini)(struct devlink *, const struct devlink_trap  *, void *); /*   216     8 */
	int                        (*trap_action_set)(struct devlink *, const struct devlink_trap  *, enum devlink_trap_action, struct netlink_ext_ack *); /*   224     8 */
	int                        (*trap_group_init)(struct devlink *, const struct devlink_trap_group  *); /*   232     8 */
	int                        (*trap_group_set)(struct devlink *, const struct devlink_trap_group  *, const struct devlink_trap_policer  *, struct netlink_ext_ack *); /*   240     8 */
	int                        (*trap_group_action_set)(struct devlink *, const struct devlink_trap_group  *, enum devlink_trap_action, struct netlink_ext_ack *); /*   248     8 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	int                        (*trap_policer_init)(struct devlink *, const struct devlink_trap_policer  *); /*   256     8 */
	void                       (*trap_policer_fini)(struct devlink *, const struct devlink_trap_policer  *); /*   264     8 */
	int                        (*trap_policer_set)(struct devlink *, const struct devlink_trap_policer  *, u64, u64, struct netlink_ext_ack *); /*   272     8 */
	int                        (*trap_policer_counter_get)(struct devlink *, const struct devlink_trap_policer  *, u64 *); /*   280     8 */
	int                        (*port_function_hw_addr_get)(struct devlink *, struct devlink_port *, u8 *, int *, struct netlink_ext_ack *); /*   288     8 */
	int                        (*port_function_hw_addr_set)(struct devlink *, struct devlink_port *, const u8  *, int, struct netlink_ext_ack *); /*   296     8 */

	/* size: 304, cachelines: 5, members: 38 */
	/* sum members: 300, holes: 1, sum holes: 4 */
	/* last cacheline: 48 bytes */
};
struct devlink_port_phys_attrs {
	u32                        port_number;          /*     0     4 */
	u32                        split_subport_number; /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct devlink_port_pci_pf_attrs {
	u32                        controller;           /*     0     4 */
	u16                        pf;                   /*     4     2 */
	u8                         external:1;           /*     6: 0  1 */

	/* size: 8, cachelines: 1, members: 3 */
	/* padding: 1 */
	/* bit_padding: 7 bits */
	/* last cacheline: 8 bytes */
};
struct devlink_port_pci_vf_attrs {
	u32                        controller;           /*     0     4 */
	u16                        pf;                   /*     4     2 */
	u16                        vf;                   /*     6     2 */
	u8                         external:1;           /*     8: 0  1 */

	/* size: 12, cachelines: 1, members: 4 */
	/* padding: 3 */
	/* bit_padding: 7 bits */
	/* last cacheline: 12 bytes */
};
struct devlink_port_attrs {
	u8                         split:1;              /*     0: 0  1 */
	u8                         splittable:1;         /*     0: 1  1 */

	/* XXX 6 bits hole, try to pack */
	/* XXX 3 bytes hole, try to pack */

	u32                        lanes;                /*     4     4 */
	enum devlink_port_flavour  flavour;              /*     8     4 */
	struct netdev_phys_item_id switch_id;            /*    12    33 */

	/* XXX 3 bytes hole, try to pack */

	union {
		struct devlink_port_phys_attrs phys;     /*    48     8 */
		struct devlink_port_pci_pf_attrs pci_pf; /*    48     8 */
		struct devlink_port_pci_vf_attrs pci_vf; /*    48    12 */
	};                                               /*    48    12 */

	/* size: 60, cachelines: 1, members: 6 */
	/* sum members: 53, holes: 2, sum holes: 6 */
	/* sum bitfield members: 2 bits, bit holes: 1, sum bit holes: 6 bits */
	/* last cacheline: 60 bytes */
};
struct devlink_sb_pool_info {
	enum devlink_sb_pool_type  pool_type;            /*     0     4 */
	u32                        size;                 /*     4     4 */
	enum devlink_sb_threshold_type threshold_type;   /*     8     4 */
	u32                        cell_size;            /*    12     4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct devlink_dpipe_field {
	const char  *              name;                 /*     0     8 */
	unsigned int               id;                   /*     8     4 */
	unsigned int               bitwidth;             /*    12     4 */
	enum devlink_dpipe_field_mapping_type mapping_type; /*    16     4 */

	/* size: 24, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct devlink_dpipe_header {
	const char  *              name;                 /*     0     8 */
	unsigned int               id;                   /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct devlink_dpipe_field * fields;             /*    16     8 */
	unsigned int               fields_count;         /*    24     4 */
	bool                       global;               /*    28     1 */

	/* size: 32, cachelines: 1, members: 5 */
	/* sum members: 25, holes: 1, sum holes: 4 */
	/* padding: 3 */
	/* last cacheline: 32 bytes */
};
struct devlink_flash_update_params {
	const char  *              file_name;            /*     0     8 */
	const char  *              component;            /*     8     8 */
	u32                        overwrite_mask;       /*    16     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct devlink_trap_policer {
	u32                        id;                   /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        init_rate;            /*     8     8 */
	u64                        init_burst;           /*    16     8 */
	u64                        max_rate;             /*    24     8 */
	u64                        min_rate;             /*    32     8 */
	u64                        max_burst;            /*    40     8 */
	u64                        min_burst;            /*    48     8 */

	/* size: 56, cachelines: 1, members: 7 */
	/* sum members: 52, holes: 1, sum holes: 4 */
	/* last cacheline: 56 bytes */
};
struct devlink_trap_group {
	const char  *              name;                 /*     0     8 */
	u16                        id;                   /*     8     2 */
	bool                       generic;              /*    10     1 */

	/* XXX 1 byte hole, try to pack */

	u32                        init_policer_id;      /*    12     4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* sum members: 15, holes: 1, sum holes: 1 */
	/* last cacheline: 16 bytes */
};
struct devlink_trap {
	enum devlink_trap_type     type;                 /*     0     4 */
	enum devlink_trap_action   init_action;          /*     4     4 */
	bool                       generic;              /*     8     1 */

	/* XXX 1 byte hole, try to pack */

	u16                        id;                   /*    10     2 */

	/* XXX 4 bytes hole, try to pack */

	const char  *              name;                 /*    16     8 */
	u16                        init_group_id;        /*    24     2 */

	/* XXX 2 bytes hole, try to pack */

	u32                        metadata_cap;         /*    28     4 */

	/* size: 32, cachelines: 1, members: 7 */
	/* sum members: 25, holes: 3, sum holes: 7 */
	/* last cacheline: 32 bytes */
};
struct arphdr {
	__be16                     ar_hrd;               /*     0     2 */
	__be16                     ar_pro;               /*     2     2 */
	unsigned char              ar_hln;               /*     4     1 */
	unsigned char              ar_pln;               /*     5     1 */
	__be16                     ar_op;                /*     6     2 */

	/* size: 8, cachelines: 1, members: 5 */
	/* last cacheline: 8 bytes */
};
struct fib_nh {
	struct fib_nh_common       nh_common;            /*     0    72 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	struct hlist_node          nh_hash;              /*    72    16 */
	struct fib_info *          nh_parent;            /*    88     8 */
	__u32                      nh_tclassid;          /*    96     4 */
	__be32                     nh_saddr;             /*   100     4 */
	int                        nh_saddr_genid;       /*   104     4 */

	/* size: 112, cachelines: 2, members: 6 */
	/* padding: 4 */
	/* last cacheline: 48 bytes */
};
struct fib_info {
	struct hlist_node          fib_hash;             /*     0    16 */
	struct hlist_node          fib_lhash;            /*    16    16 */
	struct list_head           nh_list;              /*    32    16 */
	struct net *               fib_net;              /*    48     8 */
	int                        fib_treeref;          /*    56     4 */
	refcount_t                 fib_clntref;          /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	unsigned int               fib_flags;            /*    64     4 */
	unsigned char              fib_dead;             /*    68     1 */
	unsigned char              fib_protocol;         /*    69     1 */
	unsigned char              fib_scope;            /*    70     1 */
	unsigned char              fib_type;             /*    71     1 */
	__be32                     fib_prefsrc;          /*    72     4 */
	u32                        fib_tb_id;            /*    76     4 */
	u32                        fib_priority;         /*    80     4 */

	/* XXX 4 bytes hole, try to pack */

	struct dst_metrics *       fib_metrics;          /*    88     8 */
	int                        fib_nhs;              /*    96     4 */
	bool                       fib_nh_is_v6;         /*   100     1 */
	bool                       nh_updated;           /*   101     1 */

	/* XXX 2 bytes hole, try to pack */

	struct nexthop *           nh;                   /*   104     8 */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*   112    16 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct fib_nh              fib_nh[];             /*   128     0 */

	/* size: 128, cachelines: 2, members: 21 */
	/* sum members: 122, holes: 2, sum holes: 6 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(8)));
struct nexthop {
	struct rb_node             rb_node __attribute__((__aligned__(8))); /*     0    24 */
	struct list_head           fi_list;              /*    24    16 */
	struct list_head           f6i_list;             /*    40    16 */
	struct list_head           fdb_list;             /*    56    16 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	struct list_head           grp_list;             /*    72    16 */
	struct net *               net;                  /*    88     8 */
	u32                        id;                   /*    96     4 */
	u8                         protocol;             /*   100     1 */
	u8                         nh_flags;             /*   101     1 */
	bool                       is_group;             /*   102     1 */

	/* XXX 1 byte hole, try to pack */

	refcount_t                 refcnt;               /*   104     4 */

	/* XXX 4 bytes hole, try to pack */

	struct callback_head       rcu __attribute__((__aligned__(8))); /*   112    16 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	union {
		struct nh_info *   nh_info;              /*   128     8 */
		struct nh_group *  nh_grp;               /*   128     8 */
	};                                               /*   128     8 */

	/* size: 136, cachelines: 3, members: 13 */
	/* sum members: 131, holes: 2, sum holes: 5 */
	/* forced alignments: 2, forced holes: 1, sum forced holes: 4 */
	/* last cacheline: 8 bytes */
} __attribute__((__aligned__(8)));
struct dst_cache {
	struct dst_cache_pcpu *    cache;                /*     0     8 */
	long unsigned int          reset_ts;             /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct nh_info {
	struct hlist_node          dev_hash;             /*     0    16 */
	struct nexthop *           nh_parent;            /*    16     8 */
	u8                         family;               /*    24     1 */
	bool                       reject_nh;            /*    25     1 */
	bool                       fdb_nh;               /*    26     1 */

	/* XXX 5 bytes hole, try to pack */

	union {
		struct fib_nh_common fib_nhc;            /*    32    72 */
		struct fib_nh      fib_nh;               /*    32   112 */
		struct fib6_nh     fib6_nh;              /*    32    96 */
	};                                               /*    32   112 */

	/* size: 144, cachelines: 3, members: 6 */
	/* sum members: 139, holes: 1, sum holes: 5 */
	/* last cacheline: 16 bytes */
};
struct nh_grp_entry {
	struct nexthop *           nh;                   /*     0     8 */
	u8                         weight;               /*     8     1 */

	/* XXX 3 bytes hole, try to pack */

	atomic_t                   upper_bound;          /*    12     4 */
	struct list_head           nh_list;              /*    16    16 */
	struct nexthop *           nh_parent;            /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* sum members: 37, holes: 1, sum holes: 3 */
	/* last cacheline: 40 bytes */
};
struct nh_group {
	struct nh_group *          spare;                /*     0     8 */
	u16                        num_nh;               /*     8     2 */
	bool                       mpath;                /*    10     1 */
	bool                       fdb_nh;               /*    11     1 */
	bool                       has_v4;               /*    12     1 */

	/* XXX 3 bytes hole, try to pack */

	struct nh_grp_entry        nh_entries[];         /*    16     0 */

	/* size: 16, cachelines: 1, members: 6 */
	/* sum members: 13, holes: 1, sum holes: 3 */
	/* last cacheline: 16 bytes */
};
struct ip_tunnel_key {
	__be64                     tun_id;               /*     0     8 */
	union {
		struct {
			__be32     src;                  /*     8     4 */
			__be32     dst;                  /*    12     4 */
		} ipv4;                                  /*     8     8 */
		struct {
			struct in6_addr src;             /*     8    16 */
			struct in6_addr dst;             /*    24    16 */
		} ipv6;                                  /*     8    32 */
	} u;                                             /*     8    32 */
	__be16                     tun_flags;            /*    40     2 */
	u8                         tos;                  /*    42     1 */
	u8                         ttl;                  /*    43     1 */
	__be32                     label;                /*    44     4 */
	__be16                     tp_src;               /*    48     2 */
	__be16                     tp_dst;               /*    50     2 */

	/* size: 56, cachelines: 1, members: 8 */
	/* padding: 4 */
	/* last cacheline: 56 bytes */
};
struct hw_port_info {
	struct net_device *        lower_dev;            /*     0     8 */
	u32                        port_id;              /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct metadata_dst {
	struct dst_entry           dst __attribute__((__aligned__(8))); /*     0   176 */
	/* --- cacheline 2 boundary (128 bytes) was 48 bytes ago --- */
	enum metadata_type         type;                 /*   176     4 */

	/* XXX 4 bytes hole, try to pack */

	union {
		struct ip_tunnel_info tun_info;          /*   184    80 */
		struct hw_port_info port_info;           /*   184    16 */
	} u;                                             /*   184    80 */

	/* size: 264, cachelines: 5, members: 3 */
	/* sum members: 260, holes: 1, sum holes: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 8 bytes */
} __attribute__((__aligned__(8)));
struct gre_base_hdr {
	__be16                     flags;                /*     0     2 */
	__be16                     protocol;             /*     2     2 */

	/* size: 4, cachelines: 1, members: 2 */
	/* last cacheline: 4 bytes */
};
struct gre_full_hdr {
	struct gre_base_hdr        fixed_header;         /*     0     4 */
	__be16                     csum;                 /*     4     2 */
	__be16                     reserved1;            /*     6     2 */
	__be32                     key;                  /*     8     4 */
	__be32                     seq;                  /*    12     4 */

	/* size: 16, cachelines: 1, members: 5 */
	/* last cacheline: 16 bytes */
};
struct pptp_gre_header {
	struct gre_base_hdr        gre_hd;               /*     0     4 */
	__be16                     payload_len;          /*     4     2 */
	__be16                     call_id;              /*     6     2 */
	__be32                     seq;                  /*     8     4 */
	__be32                     ack;                  /*    12     4 */

	/* size: 16, cachelines: 1, members: 5 */
	/* last cacheline: 16 bytes */
};
struct tipc_basic_hdr {
	__be32                     w[4];                 /*     0    16 */

	/* size: 16, cachelines: 1, members: 1 */
	/* last cacheline: 16 bytes */
};
struct icmphdr {
	__u8                       type;                 /*     0     1 */
	__u8                       code;                 /*     1     1 */
	__sum16                    checksum;             /*     2     2 */
	union {
		struct {
			__be16     id;                   /*     4     2 */
			__be16     sequence;             /*     6     2 */
		} echo;                                  /*     4     4 */
		__be32             gateway;              /*     4     4 */
		struct {
			__be16     __unused;             /*     4     2 */
			__be16     mtu;                  /*     6     2 */
		} frag;                                  /*     4     4 */
		__u8               reserved[4];          /*     4     4 */
	} un;                                            /*     4     4 */

	/* size: 8, cachelines: 1, members: 4 */
	/* last cacheline: 8 bytes */
};
struct pppoe_tag {
	__be16                     tag_type;             /*     0     2 */
	__be16                     tag_len;              /*     2     2 */
	char                       tag_data[];           /*     4     0 */

	/* size: 4, cachelines: 1, members: 3 */
	/* last cacheline: 4 bytes */
};
struct pppoe_hdr {
	__u8                       type:4;               /*     0: 0  1 */
	__u8                       ver:4;                /*     0: 4  1 */
	__u8                       code;                 /*     1     1 */
	__be16                     sid;                  /*     2     2 */
	__be16                     length;               /*     4     2 */
	struct pppoe_tag           tag[];                /*     6     0 */

	/* size: 6, cachelines: 1, members: 6 */
	/* last cacheline: 6 bytes */
};
struct mpls_label {
	__be32                     entry;                /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct batadv_unicast_packet {
	__u8                       packet_type;          /*     0     1 */
	__u8                       version;              /*     1     1 */
	__u8                       ttl;                  /*     2     1 */
	__u8                       ttvn;                 /*     3     1 */
	__u8                       dest[6];              /*     4     6 */

	/* size: 10, cachelines: 1, members: 5 */
	/* last cacheline: 10 bytes */
};
struct nf_conn {
	struct nf_conntrack        ct_general;           /*     0     4 */
	spinlock_t                 lock;                 /*     4     4 */
	u32                        timeout;              /*     8     4 */
	struct nf_conntrack_zone   zone;                 /*    12     4 */
	struct nf_conntrack_tuple_hash tuplehash[2];     /*    16   112 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	long unsigned int          status;               /*   128     8 */
	u16                        cpu;                  /*   136     2 */

	/* XXX 6 bytes hole, try to pack */

	possible_net_t             ct_net;               /*   144     8 */
	struct hlist_node          nat_bysource;         /*   152    16 */
	struct {
	} __nfct_init_offset;                            /*   168     0 */
	struct nf_conn *           master;               /*   168     8 */
	u_int32_t                  mark;                 /*   176     4 */
	u_int32_t                  secmark;              /*   180     4 */
	struct nf_ct_ext *         ext;                  /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	union nf_conntrack_proto   proto;                /*   192    64 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	u64                        kabi_reserved1;       /*   256     8 */
	u64                        kabi_reserved2;       /*   264     8 */

	/* size: 272, cachelines: 5, members: 17 */
	/* sum members: 266, holes: 1, sum holes: 6 */
	/* last cacheline: 16 bytes */
};
struct nf_conntrack_zone {
	u16                        id;                   /*     0     2 */
	u8                         flags;                /*     2     1 */
	u8                         dir;                  /*     3     1 */

	/* size: 4, cachelines: 1, members: 3 */
	/* last cacheline: 4 bytes */
};
struct nf_conntrack_tuple {
	struct nf_conntrack_man    src;                  /*     0    20 */
	struct {
		union nf_inet_addr u3;                   /*    20    16 */
		union {
			__be16     all;                  /*    36     2 */
			struct {
				__be16 port;             /*    36     2 */
			} tcp;                           /*    36     2 */
			struct {
				__be16 port;             /*    36     2 */
			} udp;                           /*    36     2 */
			struct {
				u_int8_t type;           /*    36     1 */
				u_int8_t code;           /*    37     1 */
			} icmp;                          /*    36     2 */
			struct {
				__be16 port;             /*    36     2 */
			} dccp;                          /*    36     2 */
			struct {
				__be16 port;             /*    36     2 */
			} sctp;                          /*    36     2 */
			struct {
				__be16 key;              /*    36     2 */
			} gre;                           /*    36     2 */
		} u;                                     /*    36     2 */
		u_int8_t           protonum;             /*    38     1 */
		u_int8_t           dir;                  /*    39     1 */
	} dst;                                           /*    20    20 */

	/* size: 40, cachelines: 1, members: 2 */
	/* last cacheline: 40 bytes */
};
struct nf_ct_gre {
	unsigned int               stream_timeout;       /*     0     4 */
	unsigned int               timeout;              /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct xt_table_info {
	unsigned int               size;                 /*     0     4 */
	unsigned int               number;               /*     4     4 */
	unsigned int               initial_entries;      /*     8     4 */
	unsigned int               hook_entry[5];        /*    12    20 */
	unsigned int               underflow[5];         /*    32    20 */
	unsigned int               stacksize;            /*    52     4 */
	void * * *                 jumpstack;            /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	unsigned char              entries[] __attribute__((__aligned__(8))); /*    64     0 */

	/* size: 64, cachelines: 1, members: 8 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(8)));
struct nf_conntrack_man {
	union nf_inet_addr         u3;                   /*     0    16 */
	union nf_conntrack_man_proto u;                  /*    16     2 */
	u_int16_t                  l3num;                /*    18     2 */

	/* size: 20, cachelines: 1, members: 3 */
	/* last cacheline: 20 bytes */
};
struct nf_conntrack_tuple_mask {
	struct {
		union nf_inet_addr u3;                   /*     0    16 */
		union nf_conntrack_man_proto u;          /*    16     2 */
	} src;                                           /*     0    20 */

	/* XXX last struct has 2 bytes of padding */

	/* size: 20, cachelines: 1, members: 1 */
	/* paddings: 1, sum paddings: 2 */
	/* last cacheline: 20 bytes */
};
struct nf_conntrack_tuple_hash {
	struct hlist_nulls_node    hnnode;               /*     0    16 */
	struct nf_conntrack_tuple  tuple;                /*    16    40 */

	/* size: 56, cachelines: 1, members: 2 */
	/* last cacheline: 56 bytes */
};
struct nf_ct_udp {
	long unsigned int          stream_ts;            /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
union nf_conntrack_proto {
	struct nf_ct_dccp          dccp;               /*     0    16 */
	struct ip_ct_sctp          sctp;               /*     0    16 */
	struct ip_ct_tcp           tcp;                /*     0    60 */
	struct nf_ct_udp           udp;                /*     0     8 */
	struct nf_ct_gre           gre;                /*     0     8 */
	unsigned int               tmpl_padto;         /*     0     4 */
};
struct nf_ct_ext {
	u8                         offset[9];            /*     0     9 */
	u8                         len;                  /*     9     1 */
	char                       data[];               /*    10     0 */

	/* size: 10, cachelines: 1, members: 3 */
	/* last cacheline: 10 bytes */
};
struct nf_conntrack_expect {
	struct hlist_node          lnode;                /*     0    16 */
	struct hlist_node          hnode;                /*    16    16 */
	struct nf_conntrack_tuple  tuple;                /*    32    40 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	struct nf_conntrack_tuple_mask mask;             /*    72    20 */

	/* XXX 4 bytes hole, try to pack */

	void                       (*expectfn)(struct nf_conn *, struct nf_conntrack_expect *); /*    96     8 */
	struct nf_conntrack_helper * helper;             /*   104     8 */
	struct nf_conn *           master;               /*   112     8 */
	struct timer_list          timeout;              /*   120    72 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	refcount_t                 use;                  /*   192     4 */
	unsigned int               flags;                /*   196     4 */
	unsigned int               class;                /*   200     4 */
	union nf_inet_addr         saved_addr;           /*   204    16 */
	union nf_conntrack_man_proto saved_proto;        /*   220     2 */

	/* XXX 2 bytes hole, try to pack */

	enum ip_conntrack_dir      dir;                  /*   224     4 */

	/* XXX 4 bytes hole, try to pack */

	struct callback_head       rcu __attribute__((__aligned__(8))); /*   232    16 */

	/* size: 248, cachelines: 4, members: 15 */
	/* sum members: 238, holes: 3, sum holes: 10 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 4 */
	/* last cacheline: 56 bytes */
} __attribute__((__aligned__(8)));
struct nf_ct_event {
	struct nf_conn *           ct;                   /*     0     8 */
	u32                        portid;               /*     8     4 */
	int                        report;               /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct nf_exp_event {
	struct nf_conntrack_expect * exp;                /*     0     8 */
	u32                        portid;               /*     8     4 */
	int                        report;               /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct nf_conn_labels {
	long unsigned int          bits[2];              /*     0    16 */

	/* size: 16, cachelines: 1, members: 1 */
	/* last cacheline: 16 bytes */
};
struct _flow_keys_digest_data {
	__be16                     n_proto;              /*     0     2 */
	u8                         ip_proto;             /*     2     1 */
	u8                         padding;              /*     3     1 */
	__be32                     ports;                /*     4     4 */
	__be32                     src;                  /*     8     4 */
	__be32                     dst;                  /*    12     4 */

	/* size: 16, cachelines: 1, members: 6 */
	/* last cacheline: 16 bytes */
};
struct rps_sock_flow_table {
	u32                        mask;                 /*     0     4 */

	/* XXX 60 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	u32                        ents[] __attribute__((__aligned__(64))); /*    64     0 */

	/* size: 64, cachelines: 1, members: 2 */
	/* sum members: 4, holes: 1, sum holes: 60 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 60 */
} __attribute__((__aligned__(64)));
struct ipv4_devconf {
	void *                     sysctl;               /*     0     8 */
	int                        data[32];             /*     8   128 */
	/* --- cacheline 2 boundary (128 bytes) was 8 bytes ago --- */
	long unsigned int          state[1];             /*   136     8 */

	/* size: 144, cachelines: 3, members: 3 */
	/* last cacheline: 16 bytes */
};
struct ifbond {
	__s32                      bond_mode;            /*     0     4 */
	__s32                      num_slaves;           /*     4     4 */
	__s32                      miimon;               /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct ifslave {
	__s32                      slave_id;             /*     0     4 */
	char                       slave_name[16];       /*     4    16 */
	__s8                       link;                 /*    20     1 */
	__s8                       state;                /*    21     1 */

	/* XXX 2 bytes hole, try to pack */

	__u32                      link_failure_count;   /*    24     4 */

	/* size: 28, cachelines: 1, members: 5 */
	/* sum members: 26, holes: 1, sum holes: 2 */
	/* last cacheline: 28 bytes */
};
struct netdev_boot_setup {
	char                       name[16];             /*     0    16 */
	struct ifmap               map;                  /*    16    24 */

	/* XXX last struct has 3 bytes of padding */

	/* size: 40, cachelines: 1, members: 2 */
	/* paddings: 1, sum paddings: 3 */
	/* last cacheline: 40 bytes */
};
struct bpf_xdp_link {
	struct bpf_link            link;                 /*     0    96 */
	/* --- cacheline 1 boundary (64 bytes) was 32 bytes ago --- */
	struct net_device *        dev;                  /*    96     8 */
	int                        flags;                /*   104     4 */

	/* size: 112, cachelines: 2, members: 3 */
	/* padding: 4 */
	/* last cacheline: 48 bytes */
};
struct netdev_net_notifier {
	struct list_head           list;                 /*     0    16 */
	struct notifier_block *    nb;                   /*    16     8 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct netpoll_info {
	refcount_t                 refcnt;               /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct semaphore           dev_lock;             /*     8    24 */
	struct sk_buff_head        txq;                  /*    32    24 */
	struct delayed_work        tx_work;              /*    56   192 */
	/* --- cacheline 3 boundary (192 bytes) was 56 bytes ago --- */
	struct netpoll *           netpoll;              /*   248     8 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*   256    16 */

	/* size: 272, cachelines: 5, members: 6 */
	/* sum members: 268, holes: 1, sum holes: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct udp_tunnel_info {
	short unsigned int         type;                 /*     0     2 */
	sa_family_t                sa_family;            /*     2     2 */
	__be16                     port;                 /*     4     2 */
	u8                         hw_priv;              /*     6     1 */

	/* size: 8, cachelines: 1, members: 4 */
	/* padding: 1 */
	/* last cacheline: 8 bytes */
};
struct in_device {
	struct net_device *        dev;                  /*     0     8 */
	refcount_t                 refcnt;               /*     8     4 */
	int                        dead;                 /*    12     4 */
	struct in_ifaddr *         ifa_list;             /*    16     8 */
	struct ip_mc_list *        mc_list;              /*    24     8 */
	struct ip_mc_list * *      mc_hash;              /*    32     8 */
	int                        mc_count;             /*    40     4 */
	spinlock_t                 mc_tomb_lock;         /*    44     4 */
	struct ip_mc_list *        mc_tomb;              /*    48     8 */
	long unsigned int          mr_v1_seen;           /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	long unsigned int          mr_v2_seen;           /*    64     8 */
	long unsigned int          mr_maxdelay;          /*    72     8 */
	long unsigned int          mr_qi;                /*    80     8 */
	long unsigned int          mr_qri;               /*    88     8 */
	unsigned char              mr_qrv;               /*    96     1 */
	unsigned char              mr_gq_running;        /*    97     1 */

	/* XXX 2 bytes hole, try to pack */

	u32                        mr_ifc_count;         /*   100     4 */
	struct timer_list          mr_gq_timer;          /*   104    72 */
	/* --- cacheline 2 boundary (128 bytes) was 48 bytes ago --- */
	struct timer_list          mr_ifc_timer;         /*   176    72 */
	/* --- cacheline 3 boundary (192 bytes) was 56 bytes ago --- */
	struct neigh_parms *       arp_parms;            /*   248     8 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	struct ipv4_devconf        cnf;                  /*   256   144 */
	/* --- cacheline 6 boundary (384 bytes) was 16 bytes ago --- */
	struct callback_head       callback_head __attribute__((__aligned__(8))); /*   400    16 */
	u64                        kabi_reserved1;       /*   416     8 */
	u64                        kabi_reserved2;       /*   424     8 */

	/* size: 432, cachelines: 7, members: 24 */
	/* sum members: 430, holes: 1, sum holes: 2 */
	/* forced alignments: 1 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct udp_tunnel_nic_info {
	int                        (*set_port)(struct net_device *, unsigned int, unsigned int, struct udp_tunnel_info *); /*     0     8 */
	int                        (*unset_port)(struct net_device *, unsigned int, unsigned int, struct udp_tunnel_info *); /*     8     8 */
	int                        (*sync_table)(struct net_device *, unsigned int); /*    16     8 */
	struct udp_tunnel_nic_shared * shared;           /*    24     8 */
	unsigned int               flags;                /*    32     4 */
	struct udp_tunnel_nic_table_info tables[4];      /*    36    32 */

	/* size: 72, cachelines: 2, members: 6 */
	/* padding: 4 */
	/* last cacheline: 8 bytes */
};
struct packet_type {
	__be16                     type;                 /*     0     2 */
	bool                       ignore_outgoing;      /*     2     1 */

	/* XXX 5 bytes hole, try to pack */

	struct net_device *        dev;                  /*     8     8 */
	int                        (*func)(struct sk_buff *, struct net_device *, struct packet_type *, struct net_device *); /*    16     8 */
	void                       (*list_func)(struct list_head *, struct packet_type *, struct net_device *); /*    24     8 */
	bool                       (*id_match)(struct packet_type *, struct sock *); /*    32     8 */
	void *                     af_packet_priv;       /*    40     8 */
	struct list_head           list;                 /*    48    16 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	union {
		struct net *       af_packet_net;        /*    64     8 */
		struct {
			u64        kabi_reserved1;       /*    64     8 */
		} kabi_hidden_2598;                      /*    64     8 */
		union {
		};                                       /*    64     0 */
	};                                               /*    64     8 */
	u64                        kabi_reserved2;       /*    72     8 */
	u64                        kabi_reserved3;       /*    80     8 */
	u64                        kabi_reserved4;       /*    88     8 */

	/* size: 96, cachelines: 2, members: 12 */
	/* sum members: 91, holes: 1, sum holes: 5 */
	/* last cacheline: 32 bytes */
};
struct offload_callbacks {
	struct sk_buff *           (*gso_segment)(struct sk_buff *, netdev_features_t); /*     0     8 */
	struct sk_buff *           (*gro_receive)(struct list_head *, struct sk_buff *); /*     8     8 */
	int                        (*gro_complete)(struct sk_buff *, int); /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct packet_offload {
	__be16                     type;                 /*     0     2 */
	u16                        priority;             /*     2     2 */

	/* XXX 4 bytes hole, try to pack */

	struct offload_callbacks   callbacks;            /*     8    24 */
	struct list_head           list;                 /*    32    16 */

	/* size: 48, cachelines: 1, members: 4 */
	/* sum members: 44, holes: 1, sum holes: 4 */
	/* last cacheline: 48 bytes */
};
struct netdev_notifier_info_ext {
	struct netdev_notifier_info info;                /*     0    16 */
	union {
		u32                mtu;                  /*    16     4 */
	} ext;                                           /*    16     4 */

	/* size: 24, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct netdev_notifier_change_info {
	struct netdev_notifier_info info;                /*     0    16 */
	unsigned int               flags_changed;        /*    16     4 */

	/* size: 24, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct netdev_notifier_changeupper_info {
	struct netdev_notifier_info info;                /*     0    16 */
	struct net_device *        upper_dev;            /*    16     8 */
	bool                       master;               /*    24     1 */
	bool                       linking;              /*    25     1 */

	/* XXX 6 bytes hole, try to pack */

	void *                     upper_info;           /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* sum members: 34, holes: 1, sum holes: 6 */
	/* last cacheline: 40 bytes */
};
struct netdev_notifier_changelowerstate_info {
	struct netdev_notifier_info info;                /*     0    16 */
	void *                     lower_state_info;     /*    16     8 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct netdev_notifier_pre_changeaddr_info {
	struct netdev_notifier_info info;                /*     0    16 */
	const unsigned char  *     dev_addr;             /*    16     8 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct netdev_nested_priv {
	unsigned char              flags;                /*     0     1 */

	/* XXX 7 bytes hole, try to pack */

	void *                     data;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 9, holes: 1, sum holes: 7 */
	/* last cacheline: 16 bytes */
};
struct netdev_bonding_info {
	ifslave                    slave;                /*     0    28 */
	ifbond                     master;               /*    28    12 */

	/* size: 40, cachelines: 1, members: 2 */
	/* last cacheline: 40 bytes */
};
struct netdev_notifier_bonding_info {
	struct netdev_notifier_info info;                /*     0    16 */
	struct netdev_bonding_info bonding_info;         /*    16    40 */

	/* size: 56, cachelines: 1, members: 2 */
	/* last cacheline: 56 bytes */
};
struct netpoll {
	struct net_device *        dev;                  /*     0     8 */
	char                       dev_name[16];         /*     8    16 */
	const char  *              name;                 /*    24     8 */
	union inet_addr            local_ip;             /*    32    16 */
	union inet_addr            remote_ip;            /*    48    16 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	bool                       ipv6;                 /*    64     1 */

	/* XXX 1 byte hole, try to pack */

	u16                        local_port;           /*    66     2 */
	u16                        remote_port;          /*    68     2 */
	u8                         remote_mac[6];        /*    70     6 */

	/* size: 80, cachelines: 2, members: 9 */
	/* sum members: 75, holes: 1, sum holes: 1 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct tcf_walker {
	int                        stop;                 /*     0     4 */
	int                        skip;                 /*     4     4 */
	int                        count;                /*     8     4 */
	bool                       nonempty;             /*    12     1 */

	/* XXX 3 bytes hole, try to pack */

	long unsigned int          cookie;               /*    16     8 */
	int                        (*fn)(struct tcf_proto *, void *, struct tcf_walker *); /*    24     8 */

	/* size: 32, cachelines: 1, members: 6 */
	/* sum members: 29, holes: 1, sum holes: 3 */
	/* last cacheline: 32 bytes */
};
union inet_addr {
	__u32                      all[4];             /*     0    16 */
	__be32                     ip;                 /*     0     4 */
	__be32                     ip6[4];             /*     0    16 */
	struct in_addr             in;                 /*     0     4 */
	struct in6_addr            in6;                /*     0    16 */
};
struct in_ifaddr {
	struct hlist_node          hash;                 /*     0    16 */
	struct in_ifaddr *         ifa_next;             /*    16     8 */
	struct in_device *         ifa_dev;              /*    24     8 */
	struct callback_head       callback_head __attribute__((__aligned__(8))); /*    32    16 */
	__be32                     ifa_local;            /*    48     4 */
	__be32                     ifa_address;          /*    52     4 */
	__be32                     ifa_mask;             /*    56     4 */
	__u32                      ifa_rt_priority;      /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	__be32                     ifa_broadcast;        /*    64     4 */
	unsigned char              ifa_scope;            /*    68     1 */
	unsigned char              ifa_prefixlen;        /*    69     1 */

	/* XXX 2 bytes hole, try to pack */

	__u32                      ifa_flags;            /*    72     4 */
	char                       ifa_label[16];        /*    76    16 */
	__u32                      ifa_valid_lft;        /*    92     4 */
	__u32                      ifa_preferred_lft;    /*    96     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          ifa_cstamp;           /*   104     8 */
	long unsigned int          ifa_tstamp;           /*   112     8 */

	/* size: 120, cachelines: 2, members: 17 */
	/* sum members: 114, holes: 2, sum holes: 6 */
	/* forced alignments: 1 */
	/* last cacheline: 56 bytes */
} __attribute__((__aligned__(8)));
struct udp_tunnel_nic_shared {
	struct udp_tunnel_nic *    udp_tunnel_nic_info;  /*     0     8 */
	struct list_head           devices;              /*     8    16 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct udp_tunnel_nic_table_info {
	unsigned int               n_entries;            /*     0     4 */
	unsigned int               tunnel_types;         /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct dev_kfree_skb_cb {
	enum skb_free_reason       reason;               /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct netdev_adjacent {
	struct net_device *        dev;                  /*     0     8 */
	bool                       master;               /*     8     1 */
	bool                       ignore;               /*     9     1 */
	u16                        ref_nr;               /*    10     2 */

	/* XXX 4 bytes hole, try to pack */

	void *                     private;              /*    16     8 */
	struct list_head           list;                 /*    24    16 */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*    40    16 */

	/* size: 56, cachelines: 1, members: 7 */
	/* sum members: 52, holes: 1, sum holes: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 56 bytes */
} __attribute__((__aligned__(8)));
struct netdev_hw_addr {
	struct list_head           list;                 /*     0    16 */
	unsigned char              addr[32];             /*    16    32 */
	unsigned char              type;                 /*    48     1 */
	bool                       global_use;           /*    49     1 */

	/* XXX 2 bytes hole, try to pack */

	int                        sync_cnt;             /*    52     4 */
	int                        refcount;             /*    56     4 */
	int                        synced;               /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct callback_head       callback_head __attribute__((__aligned__(8))); /*    64    16 */

	/* size: 80, cachelines: 2, members: 8 */
	/* sum members: 78, holes: 1, sum holes: 2 */
	/* forced alignments: 1 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct nda_cacheinfo {
	__u32                      ndm_confirmed;        /*     0     4 */
	__u32                      ndm_used;             /*     4     4 */
	__u32                      ndm_updated;          /*     8     4 */
	__u32                      ndm_refcnt;           /*    12     4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct ndt_stats {
	__u64                      ndts_allocs;          /*     0     8 */
	__u64                      ndts_destroys;        /*     8     8 */
	__u64                      ndts_hash_grows;      /*    16     8 */
	__u64                      ndts_res_failed;      /*    24     8 */
	__u64                      ndts_lookups;         /*    32     8 */
	__u64                      ndts_hits;            /*    40     8 */
	__u64                      ndts_rcv_probes_mcast; /*    48     8 */
	__u64                      ndts_rcv_probes_ucast; /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	__u64                      ndts_periodic_gc_runs; /*    64     8 */
	__u64                      ndts_forced_gc_runs;  /*    72     8 */
	__u64                      ndts_table_fulls;     /*    80     8 */

	/* size: 88, cachelines: 2, members: 11 */
	/* last cacheline: 24 bytes */
};
struct ndtmsg {
	__u8                       ndtm_family;          /*     0     1 */
	__u8                       ndtm_pad1;            /*     1     1 */
	__u16                      ndtm_pad2;            /*     2     2 */

	/* size: 4, cachelines: 1, members: 3 */
	/* last cacheline: 4 bytes */
};
struct ndt_config {
	__u16                      ndtc_key_len;         /*     0     2 */
	__u16                      ndtc_entry_size;      /*     2     2 */
	__u32                      ndtc_entries;         /*     4     4 */
	__u32                      ndtc_last_flush;      /*     8     4 */
	__u32                      ndtc_last_rand;       /*    12     4 */
	__u32                      ndtc_hash_rnd;        /*    16     4 */
	__u32                      ndtc_hash_mask;       /*    20     4 */
	__u32                      ndtc_hash_chain_gc;   /*    24     4 */
	__u32                      ndtc_proxy_qlen;      /*    28     4 */

	/* size: 32, cachelines: 1, members: 9 */
	/* last cacheline: 32 bytes */
};
struct neigh_seq_state {
	struct seq_net_private     p;                    /*     0     8 */
	struct neigh_table *       tbl;                  /*     8     8 */
	struct neigh_hash_table *  nht;                  /*    16     8 */
	void *                     (*neigh_sub_iter)(struct neigh_seq_state *, struct neighbour *, loff_t *); /*    24     8 */
	unsigned int               bucket;               /*    32     4 */
	unsigned int               flags;                /*    36     4 */

	/* size: 40, cachelines: 1, members: 6 */
	/* last cacheline: 40 bytes */
};
struct neighbour_cb {
	long unsigned int          sched_next;           /*     0     8 */
	unsigned int               flags;                /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct neigh_dump_filter {
	int                        master_idx;           /*     0     4 */
	int                        dev_idx;              /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct neigh_sysctl_table {
	struct ctl_table_header *  sysctl_header;        /*     0     8 */
	struct ctl_table           neigh_vars[21];       /*     8  1344 */

	/* size: 1352, cachelines: 22, members: 2 */
	/* last cacheline: 8 bytes */
};
struct netlink_dump_control {
	int                        (*start)(struct netlink_callback *); /*     0     8 */
	int                        (*dump)(struct sk_buff *, struct netlink_callback *); /*     8     8 */
	int                        (*done)(struct netlink_callback *); /*    16     8 */
	void *                     data;                 /*    24     8 */
	struct module *            module;               /*    32     8 */
	u32                        min_dump_alloc;       /*    40     4 */

	/* size: 48, cachelines: 1, members: 6 */
	/* padding: 4 */
	/* last cacheline: 48 bytes */
};
struct rtnl_link_stats {
	__u32                      rx_packets;           /*     0     4 */
	__u32                      tx_packets;           /*     4     4 */
	__u32                      rx_bytes;             /*     8     4 */
	__u32                      tx_bytes;             /*    12     4 */
	__u32                      rx_errors;            /*    16     4 */
	__u32                      tx_errors;            /*    20     4 */
	__u32                      rx_dropped;           /*    24     4 */
	__u32                      tx_dropped;           /*    28     4 */
	__u32                      multicast;            /*    32     4 */
	__u32                      collisions;           /*    36     4 */
	__u32                      rx_length_errors;     /*    40     4 */
	__u32                      rx_over_errors;       /*    44     4 */
	__u32                      rx_crc_errors;        /*    48     4 */
	__u32                      rx_frame_errors;      /*    52     4 */
	__u32                      rx_fifo_errors;       /*    56     4 */
	__u32                      rx_missed_errors;     /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	__u32                      tx_aborted_errors;    /*    64     4 */
	__u32                      tx_carrier_errors;    /*    68     4 */
	__u32                      tx_fifo_errors;       /*    72     4 */
	__u32                      tx_heartbeat_errors;  /*    76     4 */
	__u32                      tx_window_errors;     /*    80     4 */
	__u32                      rx_compressed;        /*    84     4 */
	__u32                      tx_compressed;        /*    88     4 */
	__u32                      rx_nohandler;         /*    92     4 */

	/* size: 96, cachelines: 2, members: 24 */
	/* last cacheline: 32 bytes */
};
struct rtnl_link_ifmap {
	__u64                      mem_start;            /*     0     8 */
	__u64                      mem_end;              /*     8     8 */
	__u64                      base_addr;            /*    16     8 */
	__u16                      irq;                  /*    24     2 */
	__u8                       dma;                  /*    26     1 */
	__u8                       port;                 /*    27     1 */

	/* size: 32, cachelines: 1, members: 6 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct ifla_vf_mac {
	__u32                      vf;                   /*     0     4 */
	__u8                       mac[32];              /*     4    32 */

	/* size: 36, cachelines: 1, members: 2 */
	/* last cacheline: 36 bytes */
};
struct ifla_vf_broadcast {
	__u8                       broadcast[32];        /*     0    32 */

	/* size: 32, cachelines: 1, members: 1 */
	/* last cacheline: 32 bytes */
};
struct ifla_vf_vlan {
	__u32                      vf;                   /*     0     4 */
	__u32                      vlan;                 /*     4     4 */
	__u32                      qos;                  /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct ifla_vf_vlan_info {
	__u32                      vf;                   /*     0     4 */
	__u32                      vlan;                 /*     4     4 */
	__u32                      qos;                  /*     8     4 */
	__be16                     vlan_proto;           /*    12     2 */

	/* size: 16, cachelines: 1, members: 4 */
	/* padding: 2 */
	/* last cacheline: 16 bytes */
};
struct ifla_vf_tx_rate {
	__u32                      vf;                   /*     0     4 */
	__u32                      rate;                 /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct ifla_vf_rate {
	__u32                      vf;                   /*     0     4 */
	__u32                      min_tx_rate;          /*     4     4 */
	__u32                      max_tx_rate;          /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct ifla_vf_spoofchk {
	__u32                      vf;                   /*     0     4 */
	__u32                      setting;              /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct ifla_vf_link_state {
	__u32                      vf;                   /*     0     4 */
	__u32                      link_state;           /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct ifla_vf_rss_query_en {
	__u32                      vf;                   /*     0     4 */
	__u32                      setting;              /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct ifla_vf_trust {
	__u32                      vf;                   /*     0     4 */
	__u32                      setting;              /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct if_stats_msg {
	__u8                       family;               /*     0     1 */
	__u8                       pad1;                 /*     1     1 */
	__u16                      pad2;                 /*     2     2 */
	__u32                      ifindex;              /*     4     4 */
	__u32                      filter_mask;          /*     8     4 */

	/* size: 12, cachelines: 1, members: 5 */
	/* last cacheline: 12 bytes */
};
struct rta_cacheinfo {
	__u32                      rta_clntref;          /*     0     4 */
	__u32                      rta_lastuse;          /*     4     4 */
	__s32                      rta_expires;          /*     8     4 */
	__u32                      rta_error;            /*    12     4 */
	__u32                      rta_used;             /*    16     4 */
	__u32                      rta_id;               /*    20     4 */
	__u32                      rta_ts;               /*    24     4 */
	__u32                      rta_tsage;            /*    28     4 */

	/* size: 32, cachelines: 1, members: 8 */
	/* last cacheline: 32 bytes */
};
struct ifinfomsg {
	unsigned char              ifi_family;           /*     0     1 */
	unsigned char              __ifi_pad;            /*     1     1 */
	short unsigned int         ifi_type;             /*     2     2 */
	int                        ifi_index;            /*     4     4 */
	unsigned int               ifi_flags;            /*     8     4 */
	unsigned int               ifi_change;           /*    12     4 */

	/* size: 16, cachelines: 1, members: 6 */
	/* last cacheline: 16 bytes */
};
struct rtnl_af_ops {
	struct list_head           list;                 /*     0    16 */
	int                        family;               /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	int                        (*fill_link_af)(struct sk_buff *, const struct net_device  *, u32); /*    24     8 */
	size_t                     (*get_link_af_size)(const struct net_device  *, u32); /*    32     8 */
	int                        (*validate_link_af)(const struct net_device  *, const struct nlattr  *); /*    40     8 */
	int                        (*set_link_af)(struct net_device *, const struct nlattr  *); /*    48     8 */
	int                        (*fill_stats_af)(struct sk_buff *, const struct net_device  *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	size_t                     (*get_stats_af_size)(const struct net_device  *); /*    64     8 */

	/* size: 72, cachelines: 2, members: 8 */
	/* sum members: 68, holes: 1, sum holes: 4 */
	/* last cacheline: 8 bytes */
};
struct rtnl_link {
	rtnl_doit_func             doit;                 /*     0     8 */
	rtnl_dumpit_func           dumpit;               /*     8     8 */
	struct module *            owner;                /*    16     8 */
	unsigned int               flags;                /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	struct callback_head       rcu __attribute__((__aligned__(8))); /*    32    16 */

	/* size: 48, cachelines: 1, members: 5 */
	/* sum members: 44, holes: 1, sum holes: 4 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 4 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct seg6_pernet_data {
	struct mutex               lock;                 /*     0    32 */
	struct in6_addr *          tun_src;              /*    32     8 */

	/* size: 40, cachelines: 1, members: 2 */
	/* last cacheline: 40 bytes */
};
struct bpf_tunnel_key {
	__u32                      tunnel_id;            /*     0     4 */
	union {
		__u32              remote_ipv4;          /*     4     4 */
		__u32              remote_ipv6[4];       /*     4    16 */
	};                                               /*     4    16 */
	__u8                       tunnel_tos;           /*    20     1 */
	__u8                       tunnel_ttl;           /*    21     1 */
	__u16                      tunnel_ext;           /*    22     2 */
	__u32                      tunnel_label;         /*    24     4 */

	/* size: 28, cachelines: 1, members: 6 */
	/* last cacheline: 28 bytes */
};
struct bpf_xfrm_state {
	__u32                      reqid;                /*     0     4 */
	__u32                      spi;                  /*     4     4 */
	__u16                      family;               /*     8     2 */
	__u16                      ext;                  /*    10     2 */
	union {
		__u32              remote_ipv4;          /*    12     4 */
		__u32              remote_ipv6[4];       /*    12    16 */
	};                                               /*    12    16 */

	/* size: 28, cachelines: 1, members: 5 */
	/* last cacheline: 28 bytes */
};
struct bpf_tcp_sock {
	__u32                      snd_cwnd;             /*     0     4 */
	__u32                      srtt_us;              /*     4     4 */
	__u32                      rtt_min;              /*     8     4 */
	__u32                      snd_ssthresh;         /*    12     4 */
	__u32                      rcv_nxt;              /*    16     4 */
	__u32                      snd_nxt;              /*    20     4 */
	__u32                      snd_una;              /*    24     4 */
	__u32                      mss_cache;            /*    28     4 */
	__u32                      ecn_flags;            /*    32     4 */
	__u32                      rate_delivered;       /*    36     4 */
	__u32                      rate_interval_us;     /*    40     4 */
	__u32                      packets_out;          /*    44     4 */
	__u32                      retrans_out;          /*    48     4 */
	__u32                      total_retrans;        /*    52     4 */
	__u32                      segs_in;              /*    56     4 */
	__u32                      data_segs_in;         /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	__u32                      segs_out;             /*    64     4 */
	__u32                      data_segs_out;        /*    68     4 */
	__u32                      lost_out;             /*    72     4 */
	__u32                      sacked_out;           /*    76     4 */
	__u64                      bytes_received;       /*    80     8 */
	__u64                      bytes_acked;          /*    88     8 */
	__u32                      dsack_dups;           /*    96     4 */
	__u32                      delivered;            /*   100     4 */
	__u32                      delivered_ce;         /*   104     4 */
	__u32                      icsk_retransmits;     /*   108     4 */

	/* size: 112, cachelines: 2, members: 26 */
	/* last cacheline: 48 bytes */
};
struct bpf_sock_tuple {
	union {
		struct {
			__be32     saddr;                /*     0     4 */
			__be32     daddr;                /*     4     4 */
			__be16     sport;                /*     8     2 */
			__be16     dport;                /*    10     2 */
		} ipv4;                                  /*     0    12 */
		struct {
			__be32     saddr[4];             /*     0    16 */
			__be32     daddr[4];             /*    16    16 */
			__be16     sport;                /*    32     2 */
			__be16     dport;                /*    34     2 */
		} ipv6;                                  /*     0    36 */
	};                                               /*     0    36 */

	/* size: 36, cachelines: 1, members: 1 */
	/* last cacheline: 36 bytes */
};
struct bpf_xdp_sock {
	__u32                      queue_id;             /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct bpf_fib_lookup {
	__u8                       family;               /*     0     1 */
	__u8                       l4_protocol;          /*     1     1 */
	__be16                     sport;                /*     2     2 */
	__be16                     dport;                /*     4     2 */
	__u16                      tot_len;              /*     6     2 */
	__u32                      ifindex;              /*     8     4 */
	union {
		__u8               tos;                  /*    12     1 */
		__be32             flowinfo;             /*    12     4 */
		__u32              rt_metric;            /*    12     4 */
	};                                               /*    12     4 */
	union {
		__be32             ipv4_src;             /*    16     4 */
		__u32              ipv6_src[4];          /*    16    16 */
	};                                               /*    16    16 */
	union {
		__be32             ipv4_dst;             /*    32     4 */
		__u32              ipv6_dst[4];          /*    32    16 */
	};                                               /*    32    16 */
	__be16                     h_vlan_proto;         /*    48     2 */
	__be16                     h_vlan_TCI;           /*    50     2 */
	__u8                       smac[6];              /*    52     6 */
	__u8                       dmac[6];              /*    58     6 */

	/* size: 64, cachelines: 1, members: 13 */
};
struct bpf_redir_neigh {
	__u32                      nh_family;            /*     0     4 */
	union {
		__be32             ipv4_nh;              /*     4     4 */
		__u32              ipv6_nh[4];           /*     4    16 */
	};                                               /*     4    16 */

	/* size: 20, cachelines: 1, members: 2 */
	/* last cacheline: 20 bytes */
};
struct tlsdev_ops {
	int                        (*tls_dev_add)(struct net_device *, struct sock *, enum tls_offload_ctx_dir, struct tls_crypto_info *, u32); /*     0     8 */
	void                       (*tls_dev_del)(struct net_device *, struct tls_context *, enum tls_offload_ctx_dir); /*     8     8 */
	int                        (*tls_dev_resync)(struct net_device *, struct sock *, u32, u8 *, enum tls_offload_ctx_dir); /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct nl_info {
	struct nlmsghdr *          nlh;                  /*     0     8 */
	struct net *               nl_net;               /*     8     8 */
	u32                        portid;               /*    16     4 */
	u8                         skip_notify:1;        /*    20: 0  1 */
	u8                         skip_notify_kernel:1; /*    20: 1  1 */

	/* size: 24, cachelines: 1, members: 5 */
	/* padding: 3 */
	/* bit_padding: 6 bits */
	/* last cacheline: 24 bytes */
};
struct inet_timewait_sock {
	struct sock_common         __tw_common;          /*     0   136 */
	/* --- cacheline 2 boundary (128 bytes) was 8 bytes ago --- */
	__u32                      tw_mark;              /*   136     4 */
	volatile unsigned char     tw_substate;          /*   140     1 */
	unsigned char              tw_rcv_wscale;        /*   141     1 */
	__be16                     tw_sport;             /*   142     2 */
	unsigned int               tw_kill:1;            /*   144: 0  4 */
	unsigned int               tw_transparent:1;     /*   144: 1  4 */
	unsigned int               tw_flowlabel:20;      /*   144: 2  4 */
	unsigned int               tw_pad:2;             /*   144:22  4 */
	unsigned int               tw_tos:8;             /*   144:24  4 */
	u32                        tw_txhash;            /*   148     4 */
	u32                        tw_priority;          /*   152     4 */

	/* XXX 4 bytes hole, try to pack */

	struct timer_list          tw_timer;             /*   160    72 */
	/* --- cacheline 3 boundary (192 bytes) was 40 bytes ago --- */
	struct inet_bind_bucket *  tw_tb;                /*   232     8 */

	/* size: 240, cachelines: 4, members: 14 */
	/* sum members: 232, holes: 1, sum holes: 4 */
	/* sum bitfield members: 32 bits (4 bytes) */
	/* last cacheline: 48 bytes */
};
struct tcp_timewait_sock {
	struct inet_timewait_sock  tw_sk;                /*     0   240 */
	/* --- cacheline 3 boundary (192 bytes) was 48 bytes ago --- */
	u32                        tw_rcv_wnd;           /*   240     4 */
	u32                        tw_ts_offset;         /*   244     4 */
	u32                        tw_ts_recent;         /*   248     4 */
	u32                        tw_last_oow_ack_time; /*   252     4 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	int                        tw_ts_recent_stamp;   /*   256     4 */
	u32                        tw_tx_delay;          /*   260     4 */
	struct tcp_md5sig_key *    tw_md5_key;           /*   264     8 */

	/* size: 272, cachelines: 5, members: 8 */
	/* last cacheline: 16 bytes */
};
struct udp_sock {
	struct inet_sock           inet __attribute__((__aligned__(8))); /*     0  1096 */
	/* --- cacheline 17 boundary (1088 bytes) was 8 bytes ago --- */
	int                        pending;              /*  1096     4 */
	unsigned int               corkflag;             /*  1100     4 */
	__u8                       encap_type;           /*  1104     1 */
	unsigned char              no_check6_tx:1;       /*  1105: 0  1 */
	unsigned char              no_check6_rx:1;       /*  1105: 1  1 */
	unsigned char              encap_enabled:1;      /*  1105: 2  1 */
	unsigned char              gro_enabled:1;        /*  1105: 3  1 */
	unsigned char              accept_udp_l4:1;      /*  1105: 4  1 */
	unsigned char              accept_udp_fraglist:1; /*  1105: 5  1 */

	/* XXX 2 bits hole, try to pack */

	__u16                      len;                  /*  1106     2 */
	__u16                      gso_size;             /*  1108     2 */
	__u16                      pcslen;               /*  1110     2 */
	__u16                      pcrlen;               /*  1112     2 */
	__u8                       pcflag;               /*  1114     1 */
	__u8                       unused[3];            /*  1115     3 */

	/* XXX 2 bytes hole, try to pack */

	int                        (*encap_rcv)(struct sock *, struct sk_buff *); /*  1120     8 */
	int                        (*encap_err_lookup)(struct sock *, struct sk_buff *); /*  1128     8 */
	void                       (*encap_destroy)(struct sock *); /*  1136     8 */
	struct sk_buff *           (*gro_receive)(struct sock *, struct list_head *, struct sk_buff *); /*  1144     8 */
	/* --- cacheline 18 boundary (1152 bytes) --- */
	int                        (*gro_complete)(struct sock *, struct sk_buff *, int); /*  1152     8 */

	/* XXX 56 bytes hole, try to pack */

	/* --- cacheline 19 boundary (1216 bytes) --- */
	struct sk_buff_head        reader_queue __attribute__((__aligned__(64))); /*  1216    24 */
	int                        forward_deficit;      /*  1240     4 */

	/* size: 1280, cachelines: 20, members: 23 */
	/* sum members: 1185, holes: 2, sum holes: 58 */
	/* sum bitfield members: 6 bits, bit holes: 1, sum bit holes: 2 bits */
	/* padding: 36 */
	/* forced alignments: 2, forced holes: 1, sum forced holes: 56 */
} __attribute__((__aligned__(64)));
struct udp6_sock {
	struct udp_sock            udp __attribute__((__aligned__(64))); /*     0  1280 */

	/* XXX last struct has 36 bytes of padding */

	/* --- cacheline 20 boundary (1280 bytes) --- */
	struct ipv6_pinfo          inet6;                /*  1280   152 */

	/* size: 1472, cachelines: 23, members: 2 */
	/* padding: 40 */
	/* paddings: 1, sum paddings: 36 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(64)));
struct tcp6_sock {
	struct tcp_sock            tcp __attribute__((__aligned__(8))); /*     0  2480 */
	/* --- cacheline 38 boundary (2432 bytes) was 48 bytes ago --- */
	struct ipv6_pinfo          inet6;                /*  2480   152 */

	/* size: 2632, cachelines: 42, members: 2 */
	/* forced alignments: 1 */
	/* last cacheline: 8 bytes */
} __attribute__((__aligned__(8)));
struct ipv6_stub {
	int                        (*ipv6_sock_mc_join)(struct sock *, int, const struct in6_addr  *); /*     0     8 */
	int                        (*ipv6_sock_mc_drop)(struct sock *, int, const struct in6_addr  *); /*     8     8 */
	struct dst_entry *         (*ipv6_dst_lookup_flow)(struct net *, const struct sock  *, struct flowi6 *, const struct in6_addr  *); /*    16     8 */
	int                        (*ipv6_route_input)(struct sk_buff *); /*    24     8 */
	struct fib6_table *        (*fib6_get_table)(struct net *, u32); /*    32     8 */
	int                        (*fib6_lookup)(struct net *, int, struct flowi6 *, struct fib6_result *, int); /*    40     8 */
	int                        (*fib6_table_lookup)(struct net *, struct fib6_table *, int, struct flowi6 *, struct fib6_result *, int); /*    48     8 */
	void                       (*fib6_select_path)(const struct net  *, struct fib6_result *, struct flowi6 *, int, bool, const struct sk_buff  *, int); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u32                        (*ip6_mtu_from_fib6)(const struct fib6_result  *, const struct in6_addr  *, const struct in6_addr  *); /*    64     8 */
	int                        (*fib6_nh_init)(struct net *, struct fib6_nh *, struct fib6_config *, gfp_t, struct netlink_ext_ack *); /*    72     8 */
	void                       (*fib6_nh_release)(struct fib6_nh *); /*    80     8 */
	void                       (*fib6_nh_release_dsts)(struct fib6_nh *); /*    88     8 */
	void                       (*fib6_update_sernum)(struct net *, struct fib6_info *); /*    96     8 */
	int                        (*ip6_del_rt)(struct net *, struct fib6_info *, bool); /*   104     8 */
	void                       (*fib6_rt_update)(struct net *, struct fib6_info *, struct nl_info *); /*   112     8 */
	void                       (*udpv6_encap_enable)(void); /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	void                       (*ndisc_send_na)(struct net_device *, const struct in6_addr  *, const struct in6_addr  *, bool, bool, bool, bool); /*   128     8 */
	void                       (*xfrm6_local_rxpmtu)(struct sk_buff *, u32); /*   136     8 */
	int                        (*xfrm6_udp_encap_rcv)(struct sock *, struct sk_buff *); /*   144     8 */
	int                        (*xfrm6_rcv_encap)(struct sk_buff *, int, __be32, int); /*   152     8 */
	struct neigh_table *       nd_tbl;               /*   160     8 */
	int                        (*ipv6_fragment)(struct net *, struct sock *, struct sk_buff *, int (*)(struct net *, struct sock *, struct sk_buff *)); /*   168     8 */

	/* size: 176, cachelines: 3, members: 22 */
	/* last cacheline: 48 bytes */
};
struct fib6_result {
	struct fib6_nh *           nh;                   /*     0     8 */
	struct fib6_info *         f6i;                  /*     8     8 */
	u32                        fib6_flags;           /*    16     4 */
	u8                         fib6_type;            /*    20     1 */

	/* XXX 3 bytes hole, try to pack */

	struct rt6_info *          rt6;                  /*    24     8 */

	/* size: 32, cachelines: 1, members: 5 */
	/* sum members: 29, holes: 1, sum holes: 3 */
	/* last cacheline: 32 bytes */
};
struct fib6_config {
	u32                        fc_table;             /*     0     4 */
	u32                        fc_metric;            /*     4     4 */
	int                        fc_dst_len;           /*     8     4 */
	int                        fc_src_len;           /*    12     4 */
	int                        fc_ifindex;           /*    16     4 */
	u32                        fc_flags;             /*    20     4 */
	u32                        fc_protocol;          /*    24     4 */
	u16                        fc_type;              /*    28     2 */
	u16                        fc_delete_all_nh:1;   /*    30: 0  2 */
	u16                        fc_ignore_dev_down:1; /*    30: 1  2 */
	u16                        __unused:14;          /*    30: 2  2 */
	u32                        fc_nh_id;             /*    32     4 */
	struct in6_addr            fc_dst;               /*    36    16 */
	struct in6_addr            fc_src;               /*    52    16 */
	/* --- cacheline 1 boundary (64 bytes) was 4 bytes ago --- */
	struct in6_addr            fc_prefsrc;           /*    68    16 */
	struct in6_addr            fc_gateway;           /*    84    16 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          fc_expires;           /*   104     8 */
	struct nlattr *            fc_mx;                /*   112     8 */
	int                        fc_mx_len;            /*   120     4 */
	int                        fc_mp_len;            /*   124     4 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct nlattr *            fc_mp;                /*   128     8 */
	struct nl_info             fc_nlinfo;            /*   136    24 */

	/* XXX last struct has 3 bytes of padding */

	struct nlattr *            fc_encap;             /*   160     8 */
	u16                        fc_encap_type;        /*   168     2 */
	bool                       fc_is_fdb;            /*   170     1 */

	/* XXX 5 bytes hole, try to pack */

	u64                        kabi_reserved1;       /*   176     8 */

	/* size: 184, cachelines: 3, members: 26 */
	/* sum members: 173, holes: 2, sum holes: 9 */
	/* sum bitfield members: 16 bits (2 bytes) */
	/* paddings: 1, sum paddings: 3 */
	/* last cacheline: 56 bytes */
};
struct ipv6_bpf_stub {
	int                        (*inet6_bind)(struct sock *, struct sockaddr *, int, u32); /*     0     8 */
	struct sock *              (*udp6_lib_lookup)(struct net *, const struct in6_addr  *, __be16, const struct in6_addr  *, __be16, int, int, struct udp_table *, struct sk_buff *); /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct fib_result {
	__be32                     prefix;               /*     0     4 */
	unsigned char              prefixlen;            /*     4     1 */
	unsigned char              nh_sel;               /*     5     1 */
	unsigned char              type;                 /*     6     1 */
	unsigned char              scope;                /*     7     1 */
	u32                        tclassid;             /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct fib_nh_common *     nhc;                  /*    16     8 */
	struct fib_info *          fi;                   /*    24     8 */
	struct fib_table *         table;                /*    32     8 */
	struct hlist_head *        fa_head;              /*    40     8 */

	/* size: 48, cachelines: 1, members: 10 */
	/* sum members: 44, holes: 1, sum holes: 4 */
	/* last cacheline: 48 bytes */
};
struct tcp_skb_cb {
	__u32                      seq;                  /*     0     4 */
	__u32                      end_seq;              /*     4     4 */
	union {
		__u32              tcp_tw_isn;           /*     8     4 */
		struct {
			u16        tcp_gso_segs;         /*     8     2 */
			u16        tcp_gso_size;         /*    10     2 */
		};                                       /*     8     4 */
	};                                               /*     8     4 */
	__u8                       tcp_flags;            /*    12     1 */
	__u8                       sacked;               /*    13     1 */
	__u8                       ip_dsfield;           /*    14     1 */
	__u8                       txstamp_ack:1;        /*    15: 0  1 */
	__u8                       eor:1;                /*    15: 1  1 */
	__u8                       has_rxtstamp:1;       /*    15: 2  1 */
	__u8                       unused:5;             /*    15: 3  1 */
	__u32                      ack_seq;              /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	union {
		struct {
			__u32      in_flight:30;         /*    24: 0  4 */
			__u32      is_app_limited:1;     /*    24:30  4 */
			__u32      unused:1;             /*    24:31  4 */
			__u32      delivered;            /*    28     4 */
			u64        first_tx_mstamp;      /*    32     8 */
			u64        delivered_mstamp;     /*    40     8 */
		} tx;                                    /*    24    24 */
		union {
			struct inet_skb_parm h4;         /*    24    24 */
			struct inet6_skb_parm h6;        /*    24    24 */
		} header;                                /*    24    24 */
		struct {
			__u32      flags;                /*    24     4 */

			/* XXX 4 bytes hole, try to pack */

			struct sock * sk_redir;          /*    32     8 */
			void *     data_end;             /*    40     8 */
		} bpf;                                   /*    24    24 */
	};                                               /*    24    24 */

	/* size: 48, cachelines: 1, members: 12 */
	/* sum members: 43, holes: 1, sum holes: 4 */
	/* sum bitfield members: 8 bits (1 bytes) */
	/* last cacheline: 48 bytes */
};
struct strp_stats {
	long long unsigned int     msgs;                 /*     0     8 */
	long long unsigned int     bytes;                /*     8     8 */
	unsigned int               mem_fail;             /*    16     4 */
	unsigned int               need_more_hdr;        /*    20     4 */
	unsigned int               msg_too_big;          /*    24     4 */
	unsigned int               msg_timeouts;         /*    28     4 */
	unsigned int               bad_hdr_len;          /*    32     4 */

	/* size: 40, cachelines: 1, members: 7 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct strp_callbacks {
	int                        (*parse_msg)(struct strparser *, struct sk_buff *); /*     0     8 */
	void                       (*rcv_msg)(struct strparser *, struct sk_buff *); /*     8     8 */
	int                        (*read_sock_done)(struct strparser *, int); /*    16     8 */
	void                       (*abort_parser)(struct strparser *, int); /*    24     8 */
	void                       (*lock)(struct strparser *); /*    32     8 */
	void                       (*unlock)(struct strparser *); /*    40     8 */

	/* size: 48, cachelines: 1, members: 6 */
	/* last cacheline: 48 bytes */
};
struct strparser {
	struct sock *              sk;                   /*     0     8 */
	u32                        stopped:1;            /*     8: 0  4 */
	u32                        paused:1;             /*     8: 1  4 */
	u32                        aborted:1;            /*     8: 2  4 */
	u32                        interrupted:1;        /*     8: 3  4 */
	u32                        unrecov_intr:1;       /*     8: 4  4 */

	/* XXX 27 bits hole, try to pack */
	/* XXX 4 bytes hole, try to pack */

	struct sk_buff * *         skb_nextp;            /*    16     8 */
	struct sk_buff *           skb_head;             /*    24     8 */
	unsigned int               need_bytes;           /*    32     4 */

	/* XXX 4 bytes hole, try to pack */

	struct delayed_work        msg_timer_work;       /*    40   192 */
	/* --- cacheline 3 boundary (192 bytes) was 40 bytes ago --- */
	struct work_struct         work;                 /*   232    64 */
	/* --- cacheline 4 boundary (256 bytes) was 40 bytes ago --- */
	struct strp_stats          stats;                /*   296    40 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 5 boundary (320 bytes) was 16 bytes ago --- */
	struct strp_callbacks      cb;                   /*   336    48 */

	/* size: 384, cachelines: 6, members: 13 */
	/* sum members: 372, holes: 2, sum holes: 8 */
	/* sum bitfield members: 5 bits, bit holes: 1, sum bit holes: 27 bits */
	/* paddings: 1, sum paddings: 4 */
};
struct strp_msg {
	int                        full_len;             /*     0     4 */
	int                        offset;               /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct _strp_msg {
	struct strp_msg            strp;                 /*     0     8 */
	int                        accum_len;            /*     8     4 */

	/* size: 12, cachelines: 1, members: 2 */
	/* last cacheline: 12 bytes */
};
struct sk_skb_cb {
	unsigned char              data[20];             /*     0    20 */
	struct _strp_msg           strp;                 /*    20    12 */

	/* size: 32, cachelines: 1, members: 2 */
	/* last cacheline: 32 bytes */
};
struct xdp_umem {
	void *                     addrs;                /*     0     8 */
	u64                        size;                 /*     8     8 */
	u32                        headroom;             /*    16     4 */
	u32                        chunk_size;           /*    20     4 */
	u32                        chunks;               /*    24     4 */
	u32                        npgs;                 /*    28     4 */
	struct user_struct *       user;                 /*    32     8 */
	refcount_t                 users;                /*    40     4 */
	u8                         flags;                /*    44     1 */
	bool                       zc;                   /*    45     1 */

	/* XXX 2 bytes hole, try to pack */

	struct page * *            pgs;                  /*    48     8 */
	int                        id;                   /*    56     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	struct list_head           xsk_dma_list;         /*    64    16 */
	struct work_struct         work;                 /*    80    64 */

	/* size: 144, cachelines: 3, members: 14 */
	/* sum members: 138, holes: 2, sum holes: 6 */
	/* last cacheline: 16 bytes */
};
struct xsk_map {
	struct bpf_map             map __attribute__((__aligned__(64))); /*     0   256 */

	/* XXX last struct has 8 bytes of padding */

	/* --- cacheline 4 boundary (256 bytes) --- */
	spinlock_t                 lock;                 /*   256     4 */

	/* XXX 4 bytes hole, try to pack */

	struct xdp_sock *          xsk_map[];            /*   264     0 */

	/* size: 320, cachelines: 5, members: 3 */
	/* sum members: 260, holes: 1, sum holes: 4 */
	/* padding: 56 */
	/* paddings: 1, sum paddings: 8 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(64)));
struct xdp_sock {
	struct sock                sk __attribute__((__aligned__(8))); /*     0   872 */

	/* XXX 24 bytes hole, try to pack */

	/* --- cacheline 14 boundary (896 bytes) --- */
	struct xsk_queue *         rx __attribute__((__aligned__(64))); /*   896     8 */
	struct net_device *        dev;                  /*   904     8 */
	struct xdp_umem *          umem;                 /*   912     8 */
	struct list_head           flush_node;           /*   920    16 */
	struct xsk_buff_pool *     pool;                 /*   936     8 */
	u16                        queue_id;             /*   944     2 */
	bool                       zc;                   /*   946     1 */

	/* XXX 1 byte hole, try to pack */

	enum {
		XSK_READY   = 0,
		XSK_BOUND   = 1,
		XSK_UNBOUND = 2,
	} state;                                         /*   948     4 */

	/* XXX 8 bytes hole, try to pack */

	/* --- cacheline 15 boundary (960 bytes) --- */
	struct xsk_queue *         tx __attribute__((__aligned__(64))); /*   960     8 */
	struct list_head           tx_list;              /*   968    16 */
	spinlock_t                 rx_lock;              /*   984     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        rx_dropped;           /*   992     8 */
	u64                        rx_queue_full;        /*  1000     8 */
	struct list_head           map_list;             /*  1008    16 */
	/* --- cacheline 16 boundary (1024 bytes) --- */
	spinlock_t                 map_list_lock;        /*  1024     4 */

	/* XXX 4 bytes hole, try to pack */

	struct mutex               mutex;                /*  1032    32 */
	struct xsk_queue *         fq_tmp;               /*  1064     8 */
	struct xsk_queue *         cq_tmp;               /*  1072     8 */

	/* size: 1088, cachelines: 17, members: 19 */
	/* sum members: 1039, holes: 5, sum holes: 41 */
	/* padding: 8 */
	/* forced alignments: 3, forced holes: 2, sum forced holes: 32 */
} __attribute__((__aligned__(64)));
struct tls_crypto_info {
	__u16                      version;              /*     0     2 */
	__u16                      cipher_type;          /*     2     2 */

	/* size: 4, cachelines: 1, members: 2 */
	/* last cacheline: 4 bytes */
};
struct tls12_crypto_info_aes_gcm_128 {
	struct tls_crypto_info     info;                 /*     0     4 */
	unsigned char              iv[8];                /*     4     8 */
	unsigned char              key[16];              /*    12    16 */
	unsigned char              salt[4];              /*    28     4 */
	unsigned char              rec_seq[8];           /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct tls12_crypto_info_aes_gcm_256 {
	struct tls_crypto_info     info;                 /*     0     4 */
	unsigned char              iv[8];                /*     4     8 */
	unsigned char              key[32];              /*    12    32 */
	unsigned char              salt[4];              /*    44     4 */
	unsigned char              rec_seq[8];           /*    48     8 */

	/* size: 56, cachelines: 1, members: 5 */
	/* last cacheline: 56 bytes */
};
struct tls_sw_context_rx {
	struct crypto_aead *       aead_recv;            /*     0     8 */
	struct crypto_wait         async_wait;           /*     8    40 */

	/* XXX last struct has 4 bytes of padding */

	struct strparser           strp;                 /*    48   384 */
	/* --- cacheline 6 boundary (384 bytes) was 48 bytes ago --- */
	struct sk_buff_head        rx_list;              /*   432    24 */
	/* --- cacheline 7 boundary (448 bytes) was 8 bytes ago --- */
	void                       (*saved_data_ready)(struct sock *); /*   456     8 */
	struct sk_buff *           recv_pkt;             /*   464     8 */
	u8                         control;              /*   472     1 */
	u8                         async_capable:1;      /*   473: 0  1 */
	u8                         decrypted:1;          /*   473: 1  1 */

	/* XXX 6 bits hole, try to pack */
	/* XXX 2 bytes hole, try to pack */

	atomic_t                   decrypt_pending;      /*   476     4 */
	spinlock_t                 decrypt_compl_lock;   /*   480     4 */
	bool                       async_notify;         /*   484     1 */

	/* size: 488, cachelines: 8, members: 12 */
	/* sum members: 482, holes: 1, sum holes: 2 */
	/* sum bitfield members: 2 bits, bit holes: 1, sum bit holes: 6 bits */
	/* padding: 3 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 40 bytes */
};
struct cipher_context {
	char *                     iv;                   /*     0     8 */
	char *                     rec_seq;              /*     8     8 */
	u64                        kabi_reserved1;       /*    16     8 */
	u64                        kabi_reserved2;       /*    24     8 */
	u64                        kabi_reserved3;       /*    32     8 */
	u64                        kabi_reserved4;       /*    40     8 */

	/* size: 48, cachelines: 1, members: 6 */
	/* last cacheline: 48 bytes */
};
union tls_crypto_context {
	struct tls_crypto_info     info;               /*     0     4 */
	union {
		struct tls12_crypto_info_aes_gcm_128 aes_gcm_128; /*     0    40 */
		struct tls12_crypto_info_aes_gcm_256 aes_gcm_256; /*     0    56 */
	};                                             /*     0    56 */
};
struct tls_prot_info {
	u16                        version;              /*     0     2 */
	u16                        cipher_type;          /*     2     2 */
	u16                        prepend_size;         /*     4     2 */
	u16                        tag_size;             /*     6     2 */
	u16                        overhead_size;        /*     8     2 */
	u16                        iv_size;              /*    10     2 */
	u16                        salt_size;            /*    12     2 */
	u16                        rec_seq_size;         /*    14     2 */
	u16                        aad_size;             /*    16     2 */
	u16                        tail_size;            /*    18     2 */

	/* size: 20, cachelines: 1, members: 10 */
	/* last cacheline: 20 bytes */
};
struct tls_context {
	struct tls_prot_info       prot_info;            /*     0    20 */
	u8                         tx_conf:3;            /*    20: 0  1 */
	u8                         rx_conf:3;            /*    20: 3  1 */

	/* XXX 2 bits hole, try to pack */
	/* XXX 3 bytes hole, try to pack */

	int                        (*push_pending_record)(struct sock *, int); /*    24     8 */
	void                       (*sk_write_space)(struct sock *); /*    32     8 */
	void *                     priv_ctx_tx;          /*    40     8 */
	void *                     priv_ctx_rx;          /*    48     8 */
	struct net_device *        netdev;               /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct cipher_context      tx;                   /*    64    48 */
	struct cipher_context      rx;                   /*   112    48 */
	/* --- cacheline 2 boundary (128 bytes) was 32 bytes ago --- */
	struct scatterlist *       partially_sent_record; /*   160     8 */
	u16                        partially_sent_offset; /*   168     2 */
	bool                       in_tcp_sendpages;     /*   170     1 */
	bool                       pending_open_record_frags; /*   171     1 */

	/* XXX 4 bytes hole, try to pack */

	struct mutex               tx_lock;              /*   176    32 */
	/* --- cacheline 3 boundary (192 bytes) was 16 bytes ago --- */
	long unsigned int          flags;                /*   208     8 */
	struct proto *             sk_proto;             /*   216     8 */
	struct sock *              sk;                   /*   224     8 */
	void                       (*sk_destruct)(struct sock *); /*   232     8 */
	union tls_crypto_context   crypto_send;          /*   240    56 */
	/* --- cacheline 4 boundary (256 bytes) was 40 bytes ago --- */
	union tls_crypto_context   crypto_recv;          /*   296    56 */
	/* --- cacheline 5 boundary (320 bytes) was 32 bytes ago --- */
	struct list_head           list;                 /*   352    16 */
	refcount_t                 refcount;             /*   368     4 */

	/* XXX 4 bytes hole, try to pack */

	struct callback_head       rcu __attribute__((__aligned__(8))); /*   376    16 */
	/* --- cacheline 6 boundary (384 bytes) was 8 bytes ago --- */
	u64                        kabi_reserved1;       /*   392     8 */
	u64                        kabi_reserved2;       /*   400     8 */
	u64                        kabi_reserved3;       /*   408     8 */
	u64                        kabi_reserved4;       /*   416     8 */

	/* size: 424, cachelines: 7, members: 28 */
	/* sum members: 412, holes: 3, sum holes: 11 */
	/* sum bitfield members: 6 bits, bit holes: 1, sum bit holes: 2 bits */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 4 */
	/* last cacheline: 40 bytes */
} __attribute__((__aligned__(8)));
struct bpf_scratchpad {
	union {
		__be32             diff[128];            /*     0   512 */
		u8                 buff[512];            /*     0   512 */
	};                                               /*     0   512 */

	/* size: 512, cachelines: 8, members: 1 */
};
struct sock_diag_req {
	__u8                       sdiag_family;         /*     0     1 */
	__u8                       sdiag_protocol;       /*     1     1 */

	/* size: 2, cachelines: 1, members: 2 */
	/* last cacheline: 2 bytes */
};
struct sock_diag_handler {
	__u8                       family;               /*     0     1 */

	/* XXX 7 bytes hole, try to pack */

	int                        (*dump)(struct sk_buff *, struct nlmsghdr *); /*     8     8 */
	int                        (*get_info)(struct sk_buff *, struct sock *); /*    16     8 */
	int                        (*destroy)(struct sk_buff *, struct nlmsghdr *); /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* sum members: 25, holes: 1, sum holes: 7 */
	/* last cacheline: 32 bytes */
};
struct broadcast_sk {
	struct sock *              sk;                   /*     0     8 */
	struct work_struct         work;                 /*     8    64 */

	/* size: 72, cachelines: 2, members: 2 */
	/* last cacheline: 8 bytes */
};
struct hwtstamp_config {
	int                        flags;                /*     0     4 */
	int                        tx_type;              /*     4     4 */
	int                        rx_filter;            /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct tso_t {
	int                        next_frag_idx;        /*     0     4 */
	int                        size;                 /*     4     4 */
	void *                     data;                 /*     8     8 */
	u16                        ip_id;                /*    16     2 */
	u8                         tlen;                 /*    18     1 */
	bool                       ipv6;                 /*    19     1 */
	u32                        tcp_seq;              /*    20     4 */

	/* size: 24, cachelines: 1, members: 7 */
	/* last cacheline: 24 bytes */
};
struct fib_notifier_info {
	int                        family;               /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct netlink_ext_ack *   extack;               /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct fib_notifier_net {
	struct list_head           fib_notifier_ops;     /*     0    16 */
	struct atomic_notifier_head fib_chain;           /*    16    16 */

	/* size: 32, cachelines: 1, members: 2 */
	/* last cacheline: 32 bytes */
};
struct xdp_attachment_info {
	struct bpf_prog *          prog;                 /*     0     8 */
	u32                        flags;                /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct xsk_buff_pool {
	struct device *            dev;                  /*     0     8 */
	struct net_device *        netdev;               /*     8     8 */
	struct list_head           xsk_tx_list;          /*    16    16 */
	spinlock_t                 xsk_tx_list_lock;     /*    32     4 */
	refcount_t                 users;                /*    36     4 */
	struct xdp_umem *          umem;                 /*    40     8 */
	struct work_struct         work;                 /*    48    64 */
	/* --- cacheline 1 boundary (64 bytes) was 48 bytes ago --- */
	struct list_head           free_list;            /*   112    16 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	u32                        heads_cnt;            /*   128     4 */
	u16                        queue_id;             /*   132     2 */

	/* XXX 58 bytes hole, try to pack */

	/* --- cacheline 3 boundary (192 bytes) --- */
	struct xsk_queue *         fq __attribute__((__aligned__(64))); /*   192     8 */
	struct xsk_queue *         cq;                   /*   200     8 */
	dma_addr_t *               dma_pages;            /*   208     8 */
	struct xdp_buff_xsk *      heads;                /*   216     8 */
	u64                        chunk_mask;           /*   224     8 */
	u64                        addrs_cnt;            /*   232     8 */
	u32                        free_list_cnt;        /*   240     4 */
	u32                        dma_pages_cnt;        /*   244     4 */
	u32                        free_heads_cnt;       /*   248     4 */
	u32                        headroom;             /*   252     4 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	u32                        chunk_size;           /*   256     4 */
	u32                        frame_len;            /*   260     4 */
	u8                         cached_need_wakeup;   /*   264     1 */
	bool                       uses_need_wakeup;     /*   265     1 */
	bool                       dma_need_sync;        /*   266     1 */
	bool                       unaligned;            /*   267     1 */

	/* XXX 4 bytes hole, try to pack */

	void *                     addrs;                /*   272     8 */
	spinlock_t                 cq_lock;              /*   280     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        kabi_reserved1;       /*   288     8 */
	u64                        kabi_reserved2;       /*   296     8 */
	struct xdp_buff_xsk *      free_heads[];         /*   304     0 */

	/* size: 320, cachelines: 5, members: 31 */
	/* sum members: 238, holes: 3, sum holes: 66 */
	/* padding: 16 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 58 */
} __attribute__((__aligned__(64)));
struct xdp_buff_xsk {
	struct xdp_buff            xdp;                  /*     0    56 */

	/* XXX last struct has 4 bytes of padding */

	dma_addr_t                 dma;                  /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	dma_addr_t                 frame_dma;            /*    64     8 */
	struct xsk_buff_pool *     pool;                 /*    72     8 */
	bool                       unaligned;            /*    80     1 */

	/* XXX 7 bytes hole, try to pack */

	u64                        orig_addr;            /*    88     8 */
	struct list_head           free_list_node;       /*    96    16 */

	/* size: 112, cachelines: 2, members: 7 */
	/* sum members: 105, holes: 1, sum holes: 7 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 48 bytes */
};
struct flow_match {
	struct flow_dissector *    dissector;            /*     0     8 */
	void *                     mask;                 /*     8     8 */
	void *                     key;                  /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct flow_match_meta {
	struct flow_dissector_key_meta * key;            /*     0     8 */
	struct flow_dissector_key_meta * mask;           /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct flow_match_basic {
	struct flow_dissector_key_basic * key;           /*     0     8 */
	struct flow_dissector_key_basic * mask;          /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct flow_match_control {
	struct flow_dissector_key_control * key;         /*     0     8 */
	struct flow_dissector_key_control * mask;        /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct flow_match_eth_addrs {
	struct flow_dissector_key_eth_addrs * key;       /*     0     8 */
	struct flow_dissector_key_eth_addrs * mask;      /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct flow_match_vlan {
	struct flow_dissector_key_vlan * key;            /*     0     8 */
	struct flow_dissector_key_vlan * mask;           /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct flow_match_ipv4_addrs {
	struct flow_dissector_key_ipv4_addrs * key;      /*     0     8 */
	struct flow_dissector_key_ipv4_addrs * mask;     /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct flow_match_ipv6_addrs {
	struct flow_dissector_key_ipv6_addrs * key;      /*     0     8 */
	struct flow_dissector_key_ipv6_addrs * mask;     /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct flow_match_ip {
	struct flow_dissector_key_ip * key;              /*     0     8 */
	struct flow_dissector_key_ip * mask;             /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct flow_match_ports {
	struct flow_dissector_key_ports * key;           /*     0     8 */
	struct flow_dissector_key_ports * mask;          /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct flow_match_icmp {
	struct flow_dissector_key_icmp * key;            /*     0     8 */
	struct flow_dissector_key_icmp * mask;           /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct flow_match_tcp {
	struct flow_dissector_key_tcp * key;             /*     0     8 */
	struct flow_dissector_key_tcp * mask;            /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct flow_match_mpls {
	struct flow_dissector_key_mpls * key;            /*     0     8 */
	struct flow_dissector_key_mpls * mask;           /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct flow_match_enc_keyid {
	struct flow_dissector_key_keyid * key;           /*     0     8 */
	struct flow_dissector_key_keyid * mask;          /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct flow_match_enc_opts {
	struct flow_dissector_key_enc_opts * key;        /*     0     8 */
	struct flow_dissector_key_enc_opts * mask;       /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct flow_match_ct {
	struct flow_dissector_key_ct * key;              /*     0     8 */
	struct flow_dissector_key_ct * mask;             /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct flow_action_cookie {
	u32                        cookie_len;           /*     0     4 */
	u8                         cookie[];             /*     4     0 */

	/* size: 4, cachelines: 1, members: 2 */
	/* last cacheline: 4 bytes */
};
struct flow_action_entry {
	enum flow_action_id        id;                   /*     0     4 */
	enum flow_action_hw_stats  hw_stats;             /*     4     4 */
	action_destr               destructor;           /*     8     8 */
	void *                     destructor_priv;      /*    16     8 */
	union {
		u32                chain_index;          /*    24     4 */
		struct net_device * dev;                 /*    24     8 */
		struct {
			u16        vid;                  /*    24     2 */
			__be16     proto;                /*    26     2 */
			u8         prio;                 /*    28     1 */
		} vlan;                                  /*    24     6 */
		struct {
			enum flow_action_mangle_base htype; /*    24     4 */
			u32        offset;               /*    28     4 */
			u32        mask;                 /*    32     4 */
			u32        val;                  /*    36     4 */
		} mangle;                                /*    24    16 */
		struct ip_tunnel_info * tunnel;          /*    24     8 */
		u32                csum_flags;           /*    24     4 */
		u32                mark;                 /*    24     4 */
		u16                ptype;                /*    24     2 */
		u32                priority;             /*    24     4 */
		struct {
			u32        ctx;                  /*    24     4 */
			u32        index;                /*    28     4 */
			u8         vf;                   /*    32     1 */
		} queue;                                 /*    24    12 */
		struct {
			struct psample_group * psample_group; /*    24     8 */
			u32        rate;                 /*    32     4 */
			u32        trunc_size;           /*    36     4 */
			bool       truncate;             /*    40     1 */
		} sample;                                /*    24    24 */
		struct {
			u32        index;                /*    24     4 */
			u32        burst;                /*    28     4 */
			u64        rate_bytes_ps;        /*    32     8 */
			u32        mtu;                  /*    40     4 */
		} police;                                /*    24    24 */
		struct {
			int        action;               /*    24     4 */
			u16        zone;                 /*    28     2 */

			/* XXX 2 bytes hole, try to pack */

			struct nf_flowtable * flow_table; /*    32     8 */
		} ct;                                    /*    24    16 */
		struct {
			long unsigned int cookie;        /*    24     8 */
			u32        mark;                 /*    32     4 */
			u32        labels[4];            /*    36    16 */
		} ct_metadata;                           /*    24    32 */
		struct {
			u32        label;                /*    24     4 */
			__be16     proto;                /*    28     2 */
			u8         tc;                   /*    30     1 */
			u8         bos;                  /*    31     1 */
			u8         ttl;                  /*    32     1 */
		} mpls_push;                             /*    24    12 */
		struct {
			__be16     proto;                /*    24     2 */
		} mpls_pop;                              /*    24     2 */
		struct {
			u32        label;                /*    24     4 */
			u8         tc;                   /*    28     1 */
			u8         bos;                  /*    29     1 */
			u8         ttl;                  /*    30     1 */
		} mpls_mangle;                           /*    24     8 */
		struct {
			u32        index;                /*    24     4 */
			s32        prio;                 /*    28     4 */
			u64        basetime;             /*    32     8 */
			u64        cycletime;            /*    40     8 */
			u64        cycletimeext;         /*    48     8 */
			u32        num_entries;          /*    56     4 */

			/* XXX 4 bytes hole, try to pack */

			/* --- cacheline 1 boundary (64 bytes) --- */
			struct action_gate_entry * entries; /*    64     8 */
		} gate;                                  /*    24    48 */
	};                                               /*    24    48 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	struct flow_action_cookie * cookie;              /*    72     8 */

	/* size: 80, cachelines: 2, members: 6 */
	/* last cacheline: 16 bytes */
};
struct flow_action {
	unsigned int               num_entries;          /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct flow_action_entry   entries[];            /*     8     0 */

	/* size: 8, cachelines: 1, members: 2 */
	/* sum members: 4, holes: 1, sum holes: 4 */
	/* last cacheline: 8 bytes */
};
struct flow_rule {
	struct flow_match          match;                /*     0    24 */
	struct flow_action         action;               /*    24     8 */

	/* size: 32, cachelines: 1, members: 2 */
	/* last cacheline: 32 bytes */
};
struct flow_block_offload {
	enum flow_block_command    command;              /*     0     4 */
	enum flow_block_binder_type binder_type;         /*     4     4 */
	bool                       block_shared;         /*     8     1 */
	bool                       unlocked_driver_cb;   /*     9     1 */

	/* XXX 6 bytes hole, try to pack */

	struct net *               net;                  /*    16     8 */
	struct flow_block *        block;                /*    24     8 */
	struct list_head           cb_list;              /*    32    16 */
	struct list_head *         driver_block_list;    /*    48     8 */
	struct netlink_ext_ack *   extack;               /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct Qdisc *             sch;                  /*    64     8 */
	struct list_head *         cb_list_head;         /*    72     8 */

	/* size: 80, cachelines: 2, members: 11 */
	/* sum members: 74, holes: 1, sum holes: 6 */
	/* last cacheline: 16 bytes */
};
struct flow_block_indr {
	struct list_head           list;                 /*     0    16 */
	struct net_device *        dev;                  /*    16     8 */
	struct Qdisc *             sch;                  /*    24     8 */
	enum flow_block_binder_type binder_type;         /*    32     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     data;                 /*    40     8 */
	void *                     cb_priv;              /*    48     8 */
	void                       (*cleanup)(struct flow_block_cb *); /*    56     8 */

	/* size: 64, cachelines: 1, members: 7 */
	/* sum members: 60, holes: 1, sum holes: 4 */
};
struct flow_block_cb {
	struct list_head           driver_list;          /*     0    16 */
	struct list_head           list;                 /*    16    16 */
	flow_setup_cb_t *          cb;                   /*    32     8 */
	void *                     cb_ident;             /*    40     8 */
	void *                     cb_priv;              /*    48     8 */
	void                       (*release)(void *);   /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct flow_block_indr     indr;                 /*    64    64 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	unsigned int               refcnt;               /*   128     4 */

	/* size: 136, cachelines: 3, members: 8 */
	/* padding: 4 */
	/* last cacheline: 8 bytes */
};
struct flow_indr_dev {
	struct list_head           list;                 /*     0    16 */
	flow_indr_block_bind_cb_t * cb;                  /*    16     8 */
	void *                     cb_priv;              /*    24     8 */
	refcount_t                 refcnt;               /*    32     4 */

	/* XXX 4 bytes hole, try to pack */

	struct callback_head       rcu __attribute__((__aligned__(8))); /*    40    16 */

	/* size: 56, cachelines: 1, members: 5 */
	/* sum members: 52, holes: 1, sum holes: 4 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 4 */
	/* last cacheline: 56 bytes */
} __attribute__((__aligned__(8)));
struct flow_indir_dev_info {
	void *                     data;                 /*     0     8 */
	struct net_device *        dev;                  /*     8     8 */
	struct Qdisc *             sch;                  /*    16     8 */
	enum tc_setup_type         type;                 /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	void                       (*cleanup)(struct flow_block_cb *); /*    32     8 */
	struct list_head           list;                 /*    40    16 */
	enum flow_block_command    command;              /*    56     4 */
	enum flow_block_binder_type binder_type;         /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct list_head *         cb_list;              /*    64     8 */

	/* size: 72, cachelines: 2, members: 9 */
	/* sum members: 68, holes: 1, sum holes: 4 */
	/* last cacheline: 8 bytes */
};
struct rx_queue_attribute {
	struct attribute           attr;                 /*     0    16 */

	/* XXX last struct has 6 bytes of padding */

	ssize_t                    (*show)(struct netdev_rx_queue *, char *); /*    16     8 */
	ssize_t                    (*store)(struct netdev_rx_queue *, const char  *, size_t); /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* paddings: 1, sum paddings: 6 */
	/* last cacheline: 32 bytes */
};
struct netdev_queue_attribute {
	struct attribute           attr;                 /*     0    16 */

	/* XXX last struct has 6 bytes of padding */

	ssize_t                    (*show)(struct netdev_queue *, char *); /*    16     8 */
	ssize_t                    (*store)(struct netdev_queue *, const char  *, size_t); /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* paddings: 1, sum paddings: 6 */
	/* last cacheline: 32 bytes */
};
struct sk_psock_progs {
	struct bpf_prog *          msg_parser;           /*     0     8 */
	struct bpf_prog *          skb_parser;           /*     8     8 */
	struct bpf_prog *          skb_verdict;          /*    16     8 */
	u64                        kabi_reserved1;       /*    24     8 */
	u64                        kabi_reserved2;       /*    32     8 */
	u64                        kabi_reserved3;       /*    40     8 */
	u64                        kabi_reserved4;       /*    48     8 */

	/* size: 56, cachelines: 1, members: 7 */
	/* last cacheline: 56 bytes */
};
struct sk_psock_link {
	struct list_head           list;                 /*     0    16 */
	struct bpf_map *           map;                  /*    16     8 */
	void *                     link_raw;             /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* last cacheline: 32 bytes */
};
struct sk_psock_parser {
	struct strparser           strp;                 /*     0   384 */
	/* --- cacheline 6 boundary (384 bytes) --- */
	bool                       enabled;              /*   384     1 */

	/* XXX 7 bytes hole, try to pack */

	void                       (*saved_data_ready)(struct sock *); /*   392     8 */

	/* size: 400, cachelines: 7, members: 3 */
	/* sum members: 393, holes: 1, sum holes: 7 */
	/* last cacheline: 16 bytes */
};
struct sk_psock_work_state {
	struct sk_buff *           skb;                  /*     0     8 */
	u32                        len;                  /*     8     4 */
	u32                        off;                  /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct sk_psock {
	struct sock *              sk;                   /*     0     8 */
	struct sock *              sk_redir;             /*     8     8 */
	u32                        apply_bytes;          /*    16     4 */
	u32                        cork_bytes;           /*    20     4 */
	u32                        eval;                 /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	struct sk_msg *            cork;                 /*    32     8 */
	struct sk_psock_progs      progs;                /*    40    56 */
	/* --- cacheline 1 boundary (64 bytes) was 32 bytes ago --- */
	struct sk_psock_parser     parser;               /*    96   400 */
	/* --- cacheline 7 boundary (448 bytes) was 48 bytes ago --- */
	struct sk_buff_head        ingress_skb;          /*   496    24 */
	/* --- cacheline 8 boundary (512 bytes) was 8 bytes ago --- */
	struct list_head           ingress_msg;          /*   520    16 */
	spinlock_t                 ingress_lock;         /*   536     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          state;                /*   544     8 */
	struct list_head           link;                 /*   552    16 */
	spinlock_t                 link_lock;            /*   568     4 */
	refcount_t                 refcnt;               /*   572     4 */
	/* --- cacheline 9 boundary (576 bytes) --- */
	void                       (*saved_unhash)(struct sock *); /*   576     8 */
	void                       (*saved_close)(struct sock *, long int); /*   584     8 */
	void                       (*saved_write_space)(struct sock *); /*   592     8 */
	struct proto *             sk_proto;             /*   600     8 */
	struct mutex               work_mutex;           /*   608    32 */
	/* --- cacheline 10 boundary (640 bytes) --- */
	struct sk_psock_work_state work_state;           /*   640    16 */
	struct work_struct         work;                 /*   656    64 */
	/* --- cacheline 11 boundary (704 bytes) was 16 bytes ago --- */
	union {
		struct callback_head rcu __attribute__((__aligned__(8))); /*   720    16 */
		struct work_struct gc;                   /*   720    64 */
	} __attribute__((__aligned__(8)));               /*   720    64 */
	/* --- cacheline 12 boundary (768 bytes) was 16 bytes ago --- */
	u64                        kabi_reserved1;       /*   784     8 */
	u64                        kabi_reserved2;       /*   792     8 */
	u64                        kabi_reserved3;       /*   800     8 */
	u64                        kabi_reserved4;       /*   808     8 */

	/* size: 816, cachelines: 13, members: 27 */
	/* sum members: 808, holes: 2, sum holes: 8 */
	/* forced alignments: 1 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct inet6_ifaddr {
	struct in6_addr            addr;                 /*     0    16 */
	__u32                      prefix_len;           /*    16     4 */
	__u32                      rt_priority;          /*    20     4 */
	__u32                      valid_lft;            /*    24     4 */
	__u32                      prefered_lft;         /*    28     4 */
	refcount_t                 refcnt;               /*    32     4 */
	spinlock_t                 lock;                 /*    36     4 */
	int                        state;                /*    40     4 */
	__u32                      flags;                /*    44     4 */
	__u8                       dad_probes;           /*    48     1 */
	__u8                       stable_privacy_retry; /*    49     1 */
	__u16                      scope;                /*    50     2 */

	/* XXX 4 bytes hole, try to pack */

	__u64                      dad_nonce;            /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	long unsigned int          cstamp;               /*    64     8 */
	long unsigned int          tstamp;               /*    72     8 */
	struct delayed_work        dad_work;             /*    80   192 */
	/* --- cacheline 4 boundary (256 bytes) was 16 bytes ago --- */
	struct inet6_dev *         idev;                 /*   272     8 */
	struct fib6_info *         rt;                   /*   280     8 */
	struct hlist_node          addr_lst;             /*   288    16 */
	struct list_head           if_list;              /*   304    16 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	struct list_head           if_list_aux;          /*   320    16 */
	struct list_head           tmp_list;             /*   336    16 */
	struct inet6_ifaddr *      ifpub;                /*   352     8 */
	int                        regen_count;          /*   360     4 */
	bool                       tokenized;            /*   364     1 */

	/* XXX 3 bytes hole, try to pack */

	struct callback_head       rcu __attribute__((__aligned__(8))); /*   368    16 */
	/* --- cacheline 6 boundary (384 bytes) --- */
	struct in6_addr            peer_addr;            /*   384    16 */

	/* size: 400, cachelines: 7, members: 27 */
	/* sum members: 393, holes: 2, sum holes: 7 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 3 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct fib_rule_uid_range {
	__u32                      start;                /*     0     4 */
	__u32                      end;                  /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct fib_rule_notifier_info {
	struct fib_notifier_info   info;                 /*     0    16 */
	struct fib_rule *          rule;                 /*    16     8 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_kfree_skb {
	struct trace_entry         ent;                  /*     0     8 */
	void *                     skbaddr;              /*     8     8 */
	void *                     location;             /*    16     8 */
	short unsigned int         protocol;             /*    24     2 */
	char                       __data[];             /*    26     0 */

	/* size: 32, cachelines: 1, members: 5 */
	/* padding: 6 */
	/* last cacheline: 32 bytes */
};
struct trace_event_raw_consume_skb {
	struct trace_entry         ent;                  /*     0     8 */
	void *                     skbaddr;              /*     8     8 */
	char                       __data[];             /*    16     0 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct trace_event_raw_skb_copy_datagram_iovec {
	struct trace_entry         ent;                  /*     0     8 */
	const void  *              skbaddr;              /*     8     8 */
	int                        len;                  /*    16     4 */
	char                       __data[];             /*    20     0 */

	/* size: 24, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct trace_event_data_offsets_kfree_skb {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_consume_skb {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_skb_copy_datagram_iovec {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_raw_net_dev_start_xmit {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        __data_loc_name;      /*     8     4 */
	u16                        queue_mapping;        /*    12     2 */

	/* XXX 2 bytes hole, try to pack */

	const void  *              skbaddr;              /*    16     8 */
	bool                       vlan_tagged;          /*    24     1 */

	/* XXX 1 byte hole, try to pack */

	u16                        vlan_proto;           /*    26     2 */
	u16                        vlan_tci;             /*    28     2 */
	u16                        protocol;             /*    30     2 */
	u8                         ip_summed;            /*    32     1 */

	/* XXX 3 bytes hole, try to pack */

	unsigned int               len;                  /*    36     4 */
	unsigned int               data_len;             /*    40     4 */
	int                        network_offset;       /*    44     4 */
	bool                       transport_offset_valid; /*    48     1 */

	/* XXX 3 bytes hole, try to pack */

	int                        transport_offset;     /*    52     4 */
	u8                         tx_flags;             /*    56     1 */

	/* XXX 1 byte hole, try to pack */

	u16                        gso_size;             /*    58     2 */
	u16                        gso_segs;             /*    60     2 */
	u16                        gso_type;             /*    62     2 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	char                       __data[];             /*    64     0 */

	/* size: 64, cachelines: 1, members: 19 */
	/* sum members: 54, holes: 5, sum holes: 10 */
};
struct trace_event_raw_net_dev_xmit {
	struct trace_entry         ent;                  /*     0     8 */
	void *                     skbaddr;              /*     8     8 */
	unsigned int               len;                  /*    16     4 */
	int                        rc;                   /*    20     4 */
	u32                        __data_loc_name;      /*    24     4 */
	char                       __data[];             /*    28     0 */

	/* size: 32, cachelines: 1, members: 6 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct trace_event_raw_net_dev_xmit_timeout {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        __data_loc_name;      /*     8     4 */
	u32                        __data_loc_driver;    /*    12     4 */
	int                        queue_index;          /*    16     4 */
	char                       __data[];             /*    20     0 */

	/* size: 20, cachelines: 1, members: 5 */
	/* last cacheline: 20 bytes */
};
struct trace_event_raw_net_dev_template {
	struct trace_entry         ent;                  /*     0     8 */
	void *                     skbaddr;              /*     8     8 */
	unsigned int               len;                  /*    16     4 */
	u32                        __data_loc_name;      /*    20     4 */
	char                       __data[];             /*    24     0 */

	/* size: 24, cachelines: 1, members: 5 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_net_dev_rx_verbose_template {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        __data_loc_name;      /*     8     4 */
	unsigned int               napi_id;              /*    12     4 */
	u16                        queue_mapping;        /*    16     2 */

	/* XXX 6 bytes hole, try to pack */

	const void  *              skbaddr;              /*    24     8 */
	bool                       vlan_tagged;          /*    32     1 */

	/* XXX 1 byte hole, try to pack */

	u16                        vlan_proto;           /*    34     2 */
	u16                        vlan_tci;             /*    36     2 */
	u16                        protocol;             /*    38     2 */
	u8                         ip_summed;            /*    40     1 */

	/* XXX 3 bytes hole, try to pack */

	u32                        hash;                 /*    44     4 */
	bool                       l4_hash;              /*    48     1 */

	/* XXX 3 bytes hole, try to pack */

	unsigned int               len;                  /*    52     4 */
	unsigned int               data_len;             /*    56     4 */
	unsigned int               truesize;             /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	bool                       mac_header_valid;     /*    64     1 */

	/* XXX 3 bytes hole, try to pack */

	int                        mac_header;           /*    68     4 */
	unsigned char              nr_frags;             /*    72     1 */

	/* XXX 1 byte hole, try to pack */

	u16                        gso_size;             /*    74     2 */
	u16                        gso_type;             /*    76     2 */
	char                       __data[];             /*    78     0 */

	/* size: 80, cachelines: 2, members: 21 */
	/* sum members: 61, holes: 6, sum holes: 17 */
	/* padding: 2 */
	/* last cacheline: 16 bytes */
};
struct trace_event_raw_net_dev_rx_exit_template {
	struct trace_entry         ent;                  /*     0     8 */
	int                        ret;                  /*     8     4 */
	char                       __data[];             /*    12     0 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct trace_event_data_offsets_net_dev_start_xmit {
	u32                        name;                 /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct trace_event_data_offsets_net_dev_xmit {
	u32                        name;                 /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct trace_event_data_offsets_net_dev_xmit_timeout {
	u32                        name;                 /*     0     4 */
	u32                        driver;               /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct trace_event_data_offsets_net_dev_template {
	u32                        name;                 /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct trace_event_data_offsets_net_dev_rx_verbose_template {
	u32                        name;                 /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct trace_event_data_offsets_net_dev_rx_exit_template {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_raw_napi_poll {
	struct trace_entry         ent;                  /*     0     8 */
	struct napi_struct *       napi;                 /*     8     8 */
	u32                        __data_loc_dev_name;  /*    16     4 */
	int                        work;                 /*    20     4 */
	int                        budget;               /*    24     4 */
	char                       __data[];             /*    28     0 */

	/* size: 32, cachelines: 1, members: 6 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct trace_event_data_offsets_napi_poll {
	u32                        dev_name;             /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct trace_event_raw_sock_rcvqueue_full {
	struct trace_entry         ent;                  /*     0     8 */
	int                        rmem_alloc;           /*     8     4 */
	unsigned int               truesize;             /*    12     4 */
	int                        sk_rcvbuf;            /*    16     4 */
	char                       __data[];             /*    20     0 */

	/* size: 20, cachelines: 1, members: 5 */
	/* last cacheline: 20 bytes */
};
struct trace_event_raw_sock_exceed_buf_limit {
	struct trace_entry         ent;                  /*     0     8 */
	char                       name[32];             /*     8    32 */
	long int                   sysctl_mem[3];        /*    40    24 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	long int                   allocated;            /*    64     8 */
	int                        sysctl_rmem;          /*    72     4 */
	int                        rmem_alloc;           /*    76     4 */
	int                        sysctl_wmem;          /*    80     4 */
	int                        wmem_alloc;           /*    84     4 */
	int                        wmem_queued;          /*    88     4 */
	int                        kind;                 /*    92     4 */
	char                       __data[];             /*    96     0 */

	/* size: 96, cachelines: 2, members: 11 */
	/* last cacheline: 32 bytes */
};
struct trace_event_raw_inet_sock_set_state {
	struct trace_entry         ent;                  /*     0     8 */
	const void  *              skaddr;               /*     8     8 */
	int                        oldstate;             /*    16     4 */
	int                        newstate;             /*    20     4 */
	__u16                      sport;                /*    24     2 */
	__u16                      dport;                /*    26     2 */
	__u16                      family;               /*    28     2 */
	__u16                      protocol;             /*    30     2 */
	__u8                       saddr[4];             /*    32     4 */
	__u8                       daddr[4];             /*    36     4 */
	__u8                       saddr_v6[16];         /*    40    16 */
	__u8                       daddr_v6[16];         /*    56    16 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	char                       __data[];             /*    72     0 */

	/* size: 72, cachelines: 2, members: 13 */
	/* last cacheline: 8 bytes */
};
struct trace_event_data_offsets_sock_rcvqueue_full {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_sock_exceed_buf_limit {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_inet_sock_set_state {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_raw_udp_fail_queue_rcv_skb {
	struct trace_entry         ent;                  /*     0     8 */
	int                        rc;                   /*     8     4 */
	__u16                      lport;                /*    12     2 */
	char                       __data[];             /*    14     0 */

	/* size: 16, cachelines: 1, members: 4 */
	/* padding: 2 */
	/* last cacheline: 16 bytes */
};
struct trace_event_data_offsets_udp_fail_queue_rcv_skb {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_raw_tcp_event_sk_skb {
	struct trace_entry         ent;                  /*     0     8 */
	const void  *              skbaddr;              /*     8     8 */
	const void  *              skaddr;               /*    16     8 */
	int                        state;                /*    24     4 */
	__u16                      sport;                /*    28     2 */
	__u16                      dport;                /*    30     2 */
	__u8                       saddr[4];             /*    32     4 */
	__u8                       daddr[4];             /*    36     4 */
	__u8                       saddr_v6[16];         /*    40    16 */
	__u8                       daddr_v6[16];         /*    56    16 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	char                       __data[];             /*    72     0 */

	/* size: 72, cachelines: 2, members: 11 */
	/* last cacheline: 8 bytes */
};
struct trace_event_raw_tcp_event_sk {
	struct trace_entry         ent;                  /*     0     8 */
	const void  *              skaddr;               /*     8     8 */
	__u16                      sport;                /*    16     2 */
	__u16                      dport;                /*    18     2 */
	__u8                       saddr[4];             /*    20     4 */
	__u8                       daddr[4];             /*    24     4 */
	__u8                       saddr_v6[16];         /*    28    16 */
	__u8                       daddr_v6[16];         /*    44    16 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	__u64                      sock_cookie;          /*    64     8 */
	char                       __data[];             /*    72     0 */

	/* size: 72, cachelines: 2, members: 10 */
	/* sum members: 68, holes: 1, sum holes: 4 */
	/* last cacheline: 8 bytes */
};
struct trace_event_raw_tcp_retransmit_synack {
	struct trace_entry         ent;                  /*     0     8 */
	const void  *              skaddr;               /*     8     8 */
	const void  *              req;                  /*    16     8 */
	__u16                      sport;                /*    24     2 */
	__u16                      dport;                /*    26     2 */
	__u8                       saddr[4];             /*    28     4 */
	__u8                       daddr[4];             /*    32     4 */
	__u8                       saddr_v6[16];         /*    36    16 */
	__u8                       daddr_v6[16];         /*    52    16 */
	/* --- cacheline 1 boundary (64 bytes) was 4 bytes ago --- */
	char                       __data[];             /*    68     0 */

	/* size: 72, cachelines: 2, members: 10 */
	/* padding: 4 */
	/* last cacheline: 8 bytes */
};
struct trace_event_raw_tcp_probe {
	struct trace_entry         ent;                  /*     0     8 */
	__u8                       saddr[28];            /*     8    28 */
	__u8                       daddr[28];            /*    36    28 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	__u16                      sport;                /*    64     2 */
	__u16                      dport;                /*    66     2 */
	__u32                      mark;                 /*    68     4 */
	__u16                      data_len;             /*    72     2 */

	/* XXX 2 bytes hole, try to pack */

	__u32                      snd_nxt;              /*    76     4 */
	__u32                      snd_una;              /*    80     4 */
	__u32                      snd_cwnd;             /*    84     4 */
	__u32                      ssthresh;             /*    88     4 */
	__u32                      snd_wnd;              /*    92     4 */
	__u32                      srtt;                 /*    96     4 */
	__u32                      rcv_wnd;              /*   100     4 */
	__u64                      sock_cookie;          /*   104     8 */
	char                       __data[];             /*   112     0 */

	/* size: 112, cachelines: 2, members: 16 */
	/* sum members: 110, holes: 1, sum holes: 2 */
	/* last cacheline: 48 bytes */
};
struct trace_event_data_offsets_tcp_event_sk_skb {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_tcp_event_sk {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_tcp_retransmit_synack {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_tcp_probe {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_raw_fib_table_lookup {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        tb_id;                /*     8     4 */
	int                        err;                  /*    12     4 */
	int                        oif;                  /*    16     4 */
	int                        iif;                  /*    20     4 */
	u8                         proto;                /*    24     1 */
	__u8                       tos;                  /*    25     1 */
	__u8                       scope;                /*    26     1 */
	__u8                       flags;                /*    27     1 */
	__u8                       src[4];               /*    28     4 */
	__u8                       dst[4];               /*    32     4 */
	__u8                       gw4[4];               /*    36     4 */
	__u8                       gw6[16];              /*    40    16 */
	u16                        sport;                /*    56     2 */
	u16                        dport;                /*    58     2 */
	u32                        __data_loc_name;      /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	char                       __data[];             /*    64     0 */

	/* size: 64, cachelines: 1, members: 17 */
};
struct trace_event_data_offsets_fib_table_lookup {
	u32                        name;                 /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct trace_event_raw_qdisc_dequeue {
	struct trace_entry         ent;                  /*     0     8 */
	struct Qdisc *             qdisc;                /*     8     8 */
	const struct netdev_queue  * txq;                /*    16     8 */
	int                        packets;              /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     skbaddr;              /*    32     8 */
	int                        ifindex;              /*    40     4 */
	u32                        handle;               /*    44     4 */
	u32                        parent;               /*    48     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          txq_state;            /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	char                       __data[];             /*    64     0 */

	/* size: 64, cachelines: 1, members: 10 */
	/* sum members: 56, holes: 2, sum holes: 8 */
};
struct trace_event_raw_qdisc_reset {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        __data_loc_dev;       /*     8     4 */
	u32                        __data_loc_kind;      /*    12     4 */
	u32                        parent;               /*    16     4 */
	u32                        handle;               /*    20     4 */
	char                       __data[];             /*    24     0 */

	/* size: 24, cachelines: 1, members: 6 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_qdisc_destroy {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        __data_loc_dev;       /*     8     4 */
	u32                        __data_loc_kind;      /*    12     4 */
	u32                        parent;               /*    16     4 */
	u32                        handle;               /*    20     4 */
	char                       __data[];             /*    24     0 */

	/* size: 24, cachelines: 1, members: 6 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_qdisc_create {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        __data_loc_dev;       /*     8     4 */
	u32                        __data_loc_kind;      /*    12     4 */
	u32                        parent;               /*    16     4 */
	char                       __data[];             /*    20     0 */

	/* size: 20, cachelines: 1, members: 5 */
	/* last cacheline: 20 bytes */
};
struct trace_event_data_offsets_qdisc_dequeue {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_qdisc_reset {
	u32                        dev;                  /*     0     4 */
	u32                        kind;                 /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct trace_event_data_offsets_qdisc_destroy {
	u32                        dev;                  /*     0     4 */
	u32                        kind;                 /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct trace_event_data_offsets_qdisc_create {
	u32                        dev;                  /*     0     4 */
	u32                        kind;                 /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct bridge_stp_xstats {
	__u64                      transition_blk;       /*     0     8 */
	__u64                      transition_fwd;       /*     8     8 */
	__u64                      rx_bpdu;              /*    16     8 */
	__u64                      tx_bpdu;              /*    24     8 */
	__u64                      rx_tcn;               /*    32     8 */
	__u64                      tx_tcn;               /*    40     8 */

	/* size: 48, cachelines: 1, members: 6 */
	/* last cacheline: 48 bytes */
};
struct br_mcast_stats {
	__u64                      igmp_v1queries[2];    /*     0    16 */
	__u64                      igmp_v2queries[2];    /*    16    16 */
	__u64                      igmp_v3queries[2];    /*    32    16 */
	__u64                      igmp_leaves[2];       /*    48    16 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	__u64                      igmp_v1reports[2];    /*    64    16 */
	__u64                      igmp_v2reports[2];    /*    80    16 */
	__u64                      igmp_v3reports[2];    /*    96    16 */
	__u64                      igmp_parse_errors;    /*   112     8 */
	__u64                      mld_v1queries[2];     /*   120    16 */
	/* --- cacheline 2 boundary (128 bytes) was 8 bytes ago --- */
	__u64                      mld_v2queries[2];     /*   136    16 */
	__u64                      mld_leaves[2];        /*   152    16 */
	__u64                      mld_v1reports[2];     /*   168    16 */
	__u64                      mld_v2reports[2];     /*   184    16 */
	/* --- cacheline 3 boundary (192 bytes) was 8 bytes ago --- */
	__u64                      mld_parse_errors;     /*   200     8 */
	__u64                      mcast_bytes[2];       /*   208    16 */
	__u64                      mcast_packets[2];     /*   224    16 */

	/* size: 240, cachelines: 4, members: 16 */
	/* last cacheline: 48 bytes */
};
struct br_ip {
	union {
		__be32             ip4;                  /*     0     4 */
		struct in6_addr    ip6;                  /*     0    16 */
	} src;                                           /*     0    16 */
	union {
		__be32             ip4;                  /*    16     4 */
		struct in6_addr    ip6;                  /*    16    16 */
	} dst;                                           /*    16    16 */
	__be16                     proto;                /*    32     2 */
	__u16                      vid;                  /*    34     2 */

	/* size: 36, cachelines: 1, members: 4 */
	/* last cacheline: 36 bytes */
};
struct bridge_id {
	unsigned char              prio[2];              /*     0     2 */
	unsigned char              addr[6];              /*     2     6 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct mac_addr {
	unsigned char              addr[6];              /*     0     6 */

	/* size: 6, cachelines: 1, members: 1 */
	/* last cacheline: 6 bytes */
};
struct bridge_mcast_own_query {
	struct timer_list          timer;                /*     0    72 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	u32                        startup_sent;         /*    72     4 */

	/* size: 80, cachelines: 2, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct bridge_mcast_other_query {
	struct timer_list          timer;                /*     0    72 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	long unsigned int          delay_time;           /*    72     8 */

	/* size: 80, cachelines: 2, members: 2 */
	/* last cacheline: 16 bytes */
};
struct bridge_mcast_querier {
	struct br_ip               addr;                 /*     0    36 */

	/* XXX 4 bytes hole, try to pack */

	struct net_bridge_port *   port;                 /*    40     8 */

	/* size: 48, cachelines: 1, members: 2 */
	/* sum members: 44, holes: 1, sum holes: 4 */
	/* last cacheline: 48 bytes */
};
struct net_bridge_port {
	struct net_bridge *        br;                   /*     0     8 */
	struct net_device *        dev;                  /*     8     8 */
	struct list_head           list;                 /*    16    16 */
	long unsigned int          flags;                /*    32     8 */
	struct net_bridge_vlan_group * vlgrp;            /*    40     8 */
	struct net_bridge_port *   backup_port;          /*    48     8 */
	u8                         priority;             /*    56     1 */
	u8                         state;                /*    57     1 */
	u16                        port_no;              /*    58     2 */
	unsigned char              topology_change_ack;  /*    60     1 */
	unsigned char              config_pending;       /*    61     1 */
	port_id                    port_id;              /*    62     2 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	port_id                    designated_port;      /*    64     2 */
	bridge_id                  designated_root;      /*    66     8 */
	bridge_id                  designated_bridge;    /*    74     8 */

	/* XXX 2 bytes hole, try to pack */

	u32                        path_cost;            /*    84     4 */
	u32                        designated_cost;      /*    88     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          designated_age;       /*    96     8 */
	struct timer_list          forward_delay_timer;  /*   104    72 */
	/* --- cacheline 2 boundary (128 bytes) was 48 bytes ago --- */
	struct timer_list          hold_timer;           /*   176    72 */
	/* --- cacheline 3 boundary (192 bytes) was 56 bytes ago --- */
	struct timer_list          message_age_timer;    /*   248    72 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	struct kobject             kobj;                 /*   320    96 */
	/* --- cacheline 6 boundary (384 bytes) was 32 bytes ago --- */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*   416    16 */
	struct bridge_mcast_own_query ip4_own_query;     /*   432    80 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 8 boundary (512 bytes) --- */
	struct bridge_mcast_own_query ip6_own_query;     /*   512    80 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 9 boundary (576 bytes) was 16 bytes ago --- */
	unsigned char              multicast_router;     /*   592     1 */

	/* XXX 7 bytes hole, try to pack */

	struct bridge_mcast_stats * mcast_stats;         /*   600     8 */
	struct timer_list          multicast_router_timer; /*   608    72 */
	/* --- cacheline 10 boundary (640 bytes) was 40 bytes ago --- */
	struct hlist_head          mglist;               /*   680     8 */
	struct hlist_node          rlist;                /*   688    16 */
	/* --- cacheline 11 boundary (704 bytes) --- */
	char                       sysfs_name[16];       /*   704    16 */
	struct netpoll *           np;                   /*   720     8 */
	int                        offload_fwd_mark;     /*   728     4 */
	u16                        group_fwd_mask;       /*   732     2 */
	u16                        backup_redirected_cnt; /*   734     2 */
	struct bridge_stp_xstats   stp_xstats;           /*   736    48 */

	/* size: 784, cachelines: 13, members: 36 */
	/* sum members: 771, holes: 3, sum holes: 13 */
	/* paddings: 2, sum paddings: 8 */
	/* forced alignments: 1 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct bridge_mcast_stats {
	struct br_mcast_stats      mstats;               /*     0   240 */
	/* --- cacheline 3 boundary (192 bytes) was 48 bytes ago --- */
	struct u64_stats_sync      syncp;                /*   240     0 */

	/* size: 240, cachelines: 4, members: 2 */
	/* last cacheline: 48 bytes */
};
struct net_bridge {
	spinlock_t                 lock;                 /*     0     4 */
	spinlock_t                 hash_lock;            /*     4     4 */
	struct list_head           port_list;            /*     8    16 */
	struct net_device *        dev;                  /*    24     8 */
	struct pcpu_sw_netstats *  stats;                /*    32     8 */
	long unsigned int          options;              /*    40     8 */
	__be16                     vlan_proto;           /*    48     2 */
	u16                        default_pvid;         /*    50     2 */

	/* XXX 4 bytes hole, try to pack */

	struct net_bridge_vlan_group * vlgrp;            /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct rhashtable          fdb_hash_tbl;         /*    64   168 */
	/* --- cacheline 3 boundary (192 bytes) was 40 bytes ago --- */
	union {
		struct rtable      fake_rtable __attribute__((__aligned__(8))); /*   232   240 */
		struct rt6_info    fake_rt6_info __attribute__((__aligned__(8))); /*   232   304 */
	} __attribute__((__aligned__(8)));               /*   232   304 */
	/* --- cacheline 8 boundary (512 bytes) was 24 bytes ago --- */
	u16                        group_fwd_mask;       /*   536     2 */
	u16                        group_fwd_mask_required; /*   538     2 */
	bridge_id                  designated_root;      /*   540     8 */
	bridge_id                  bridge_id;            /*   548     8 */
	unsigned char              topology_change;      /*   556     1 */
	unsigned char              topology_change_detected; /*   557     1 */
	u16                        root_port;            /*   558     2 */
	long unsigned int          max_age;              /*   560     8 */
	long unsigned int          hello_time;           /*   568     8 */
	/* --- cacheline 9 boundary (576 bytes) --- */
	long unsigned int          forward_delay;        /*   576     8 */
	long unsigned int          ageing_time;          /*   584     8 */
	long unsigned int          bridge_max_age;       /*   592     8 */
	long unsigned int          bridge_hello_time;    /*   600     8 */
	long unsigned int          bridge_forward_delay; /*   608     8 */
	long unsigned int          bridge_ageing_time;   /*   616     8 */
	u32                        root_path_cost;       /*   624     4 */
	u8                         group_addr[6];        /*   628     6 */

	/* XXX 2 bytes hole, try to pack */

	enum {
		BR_NO_STP     = 0,
		BR_KERNEL_STP = 1,
		BR_USER_STP   = 2,
	} stp_enabled;                                   /*   636     4 */
	/* --- cacheline 10 boundary (640 bytes) --- */
	u32                        hash_max;             /*   640     4 */
	u32                        multicast_last_member_count; /*   644     4 */
	u32                        multicast_startup_query_count; /*   648     4 */
	u8                         multicast_igmp_version; /*   652     1 */
	u8                         multicast_router;     /*   653     1 */
	u8                         multicast_mld_version; /*   654     1 */

	/* XXX 1 byte hole, try to pack */

	spinlock_t                 multicast_lock;       /*   656     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          multicast_last_member_interval; /*   664     8 */
	long unsigned int          multicast_membership_interval; /*   672     8 */
	long unsigned int          multicast_querier_interval; /*   680     8 */
	long unsigned int          multicast_query_interval; /*   688     8 */
	long unsigned int          multicast_query_response_interval; /*   696     8 */
	/* --- cacheline 11 boundary (704 bytes) --- */
	long unsigned int          multicast_startup_query_interval; /*   704     8 */
	struct rhashtable          mdb_hash_tbl;         /*   712   168 */
	/* --- cacheline 13 boundary (832 bytes) was 48 bytes ago --- */
	struct rhashtable          sg_port_tbl;          /*   880   168 */
	/* --- cacheline 16 boundary (1024 bytes) was 24 bytes ago --- */
	struct hlist_head          mcast_gc_list;        /*  1048     8 */
	struct hlist_head          mdb_list;             /*  1056     8 */
	struct hlist_head          router_list;          /*  1064     8 */
	struct timer_list          multicast_router_timer; /*  1072    72 */
	/* --- cacheline 17 boundary (1088 bytes) was 56 bytes ago --- */
	struct bridge_mcast_other_query ip4_other_query; /*  1144    80 */
	/* --- cacheline 19 boundary (1216 bytes) was 8 bytes ago --- */
	struct bridge_mcast_own_query ip4_own_query;     /*  1224    80 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 20 boundary (1280 bytes) was 24 bytes ago --- */
	struct bridge_mcast_querier ip4_querier;         /*  1304    48 */
	/* --- cacheline 21 boundary (1344 bytes) was 8 bytes ago --- */
	struct bridge_mcast_stats * mcast_stats;         /*  1352     8 */
	struct bridge_mcast_other_query ip6_other_query; /*  1360    80 */
	/* --- cacheline 22 boundary (1408 bytes) was 32 bytes ago --- */
	struct bridge_mcast_own_query ip6_own_query;     /*  1440    80 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 23 boundary (1472 bytes) was 48 bytes ago --- */
	struct bridge_mcast_querier ip6_querier;         /*  1520    48 */
	/* --- cacheline 24 boundary (1536 bytes) was 32 bytes ago --- */
	struct work_struct         mcast_gc_work;        /*  1568    64 */
	/* --- cacheline 25 boundary (1600 bytes) was 32 bytes ago --- */
	struct timer_list          hello_timer;          /*  1632    72 */
	/* --- cacheline 26 boundary (1664 bytes) was 40 bytes ago --- */
	struct timer_list          tcn_timer;            /*  1704    72 */
	/* --- cacheline 27 boundary (1728 bytes) was 48 bytes ago --- */
	struct timer_list          topology_change_timer; /*  1776    72 */
	/* --- cacheline 28 boundary (1792 bytes) was 56 bytes ago --- */
	struct delayed_work        gc_work;              /*  1848   192 */
	/* --- cacheline 31 boundary (1984 bytes) was 56 bytes ago --- */
	struct kobject *           ifobj;                /*  2040     8 */
	/* --- cacheline 32 boundary (2048 bytes) --- */
	u32                        auto_cnt;             /*  2048     4 */
	int                        offload_fwd_mark;     /*  2052     4 */
	struct hlist_head          fdb_list;             /*  2056     8 */

	/* size: 2064, cachelines: 33, members: 64 */
	/* sum members: 2053, holes: 4, sum holes: 11 */
	/* paddings: 2, sum paddings: 8 */
	/* forced alignments: 1 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct net_bridge_vlan_group {
	struct rhashtable          vlan_hash;            /*     0   168 */
	/* --- cacheline 2 boundary (128 bytes) was 40 bytes ago --- */
	struct rhashtable          tunnel_hash;          /*   168   168 */
	/* --- cacheline 5 boundary (320 bytes) was 16 bytes ago --- */
	struct list_head           vlan_list;            /*   336    16 */
	u16                        num_vlans;            /*   352     2 */
	u16                        pvid;                 /*   354     2 */
	u8                         pvid_state;           /*   356     1 */

	/* size: 360, cachelines: 6, members: 6 */
	/* padding: 3 */
	/* last cacheline: 40 bytes */
};
struct net_bridge_fdb_key {
	mac_addr                   addr;                 /*     0     6 */
	u16                        vlan_id;              /*     6     2 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct net_bridge_fdb_entry {
	struct rhash_head          rhnode;               /*     0     8 */
	struct net_bridge_port *   dst;                  /*     8     8 */
	struct net_bridge_fdb_key  key;                  /*    16     8 */
	struct hlist_node          fdb_node;             /*    24    16 */
	long unsigned int          flags;                /*    40     8 */

	/* XXX 16 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	long unsigned int          updated __attribute__((__aligned__(64))); /*    64     8 */
	long unsigned int          used;                 /*    72     8 */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*    80    16 */

	/* size: 128, cachelines: 2, members: 8 */
	/* sum members: 80, holes: 1, sum holes: 16 */
	/* padding: 32 */
	/* forced alignments: 2, forced holes: 1, sum forced holes: 16 */
} __attribute__((__aligned__(64)));
struct trace_event_raw_br_fdb_add {
	struct trace_entry         ent;                  /*     0     8 */
	u8                         ndm_flags;            /*     8     1 */

	/* XXX 3 bytes hole, try to pack */

	u32                        __data_loc_dev;       /*    12     4 */
	unsigned char              addr[6];              /*    16     6 */
	u16                        vid;                  /*    22     2 */
	u16                        nlh_flags;            /*    24     2 */
	char                       __data[];             /*    26     0 */

	/* size: 28, cachelines: 1, members: 7 */
	/* sum members: 23, holes: 1, sum holes: 3 */
	/* padding: 2 */
	/* last cacheline: 28 bytes */
};
struct trace_event_raw_br_fdb_external_learn_add {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        __data_loc_br_dev;    /*     8     4 */
	u32                        __data_loc_dev;       /*    12     4 */
	unsigned char              addr[6];              /*    16     6 */
	u16                        vid;                  /*    22     2 */
	char                       __data[];             /*    24     0 */

	/* size: 24, cachelines: 1, members: 6 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_fdb_delete {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        __data_loc_br_dev;    /*     8     4 */
	u32                        __data_loc_dev;       /*    12     4 */
	unsigned char              addr[6];              /*    16     6 */
	u16                        vid;                  /*    22     2 */
	char                       __data[];             /*    24     0 */

	/* size: 24, cachelines: 1, members: 6 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_br_fdb_update {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        __data_loc_br_dev;    /*     8     4 */
	u32                        __data_loc_dev;       /*    12     4 */
	unsigned char              addr[6];              /*    16     6 */
	u16                        vid;                  /*    22     2 */
	long unsigned int          flags;                /*    24     8 */
	char                       __data[];             /*    32     0 */

	/* size: 32, cachelines: 1, members: 7 */
	/* last cacheline: 32 bytes */
};
struct trace_event_data_offsets_br_fdb_add {
	u32                        dev;                  /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct trace_event_data_offsets_br_fdb_external_learn_add {
	u32                        br_dev;               /*     0     4 */
	u32                        dev;                  /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct trace_event_data_offsets_fdb_delete {
	u32                        br_dev;               /*     0     4 */
	u32                        dev;                  /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct trace_event_data_offsets_br_fdb_update {
	u32                        br_dev;               /*     0     4 */
	u32                        dev;                  /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct trace_event_raw_page_pool_release {
	struct trace_entry         ent;                  /*     0     8 */
	const struct page_pool  *  pool;                 /*     8     8 */
	s32                        inflight;             /*    16     4 */
	u32                        hold;                 /*    20     4 */
	u32                        release;              /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        cnt;                  /*    32     8 */
	char                       __data[];             /*    40     0 */

	/* size: 40, cachelines: 1, members: 7 */
	/* sum members: 36, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct trace_event_raw_page_pool_state_release {
	struct trace_entry         ent;                  /*     0     8 */
	const struct page_pool  *  pool;                 /*     8     8 */
	const struct page  *       page;                 /*    16     8 */
	u32                        release;              /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          pfn;                  /*    32     8 */
	char                       __data[];             /*    40     0 */

	/* size: 40, cachelines: 1, members: 6 */
	/* sum members: 36, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct trace_event_raw_page_pool_state_hold {
	struct trace_entry         ent;                  /*     0     8 */
	const struct page_pool  *  pool;                 /*     8     8 */
	const struct page  *       page;                 /*    16     8 */
	u32                        hold;                 /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          pfn;                  /*    32     8 */
	char                       __data[];             /*    40     0 */

	/* size: 40, cachelines: 1, members: 6 */
	/* sum members: 36, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct trace_event_raw_page_pool_update_nid {
	struct trace_entry         ent;                  /*     0     8 */
	const struct page_pool  *  pool;                 /*     8     8 */
	int                        pool_nid;             /*    16     4 */
	int                        new_nid;              /*    20     4 */
	char                       __data[];             /*    24     0 */

	/* size: 24, cachelines: 1, members: 5 */
	/* last cacheline: 24 bytes */
};
struct trace_event_data_offsets_page_pool_release {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_page_pool_state_release {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_page_pool_state_hold {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_data_offsets_page_pool_update_nid {

	/* size: 0, cachelines: 0, members: 0 */
};
struct trace_event_raw_neigh_create {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        family;               /*     8     4 */
	u32                        __data_loc_dev;       /*    12     4 */
	int                        entries;              /*    16     4 */
	u8                         created;              /*    20     1 */
	u8                         gc_exempt;            /*    21     1 */
	u8                         primary_key4[4];      /*    22     4 */
	u8                         primary_key6[16];     /*    26    16 */
	char                       __data[];             /*    42     0 */

	/* size: 44, cachelines: 1, members: 9 */
	/* padding: 2 */
	/* last cacheline: 44 bytes */
};
struct trace_event_raw_neigh_update {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        family;               /*     8     4 */
	u32                        __data_loc_dev;       /*    12     4 */
	u8                         lladdr[32];           /*    16    32 */
	u8                         lladdr_len;           /*    48     1 */
	u8                         flags;                /*    49     1 */
	u8                         nud_state;            /*    50     1 */
	u8                         type;                 /*    51     1 */
	u8                         dead;                 /*    52     1 */

	/* XXX 3 bytes hole, try to pack */

	int                        refcnt;               /*    56     4 */
	__u8                       primary_key4[4];      /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	__u8                       primary_key6[16];     /*    64    16 */
	long unsigned int          confirmed;            /*    80     8 */
	long unsigned int          updated;              /*    88     8 */
	long unsigned int          used;                 /*    96     8 */
	u8                         new_lladdr[32];       /*   104    32 */
	/* --- cacheline 2 boundary (128 bytes) was 8 bytes ago --- */
	u8                         new_state;            /*   136     1 */

	/* XXX 3 bytes hole, try to pack */

	u32                        update_flags;         /*   140     4 */
	u32                        pid;                  /*   144     4 */
	char                       __data[];             /*   148     0 */

	/* size: 152, cachelines: 3, members: 20 */
	/* sum members: 142, holes: 2, sum holes: 6 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct trace_event_raw_neigh__update {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        family;               /*     8     4 */
	u32                        __data_loc_dev;       /*    12     4 */
	u8                         lladdr[32];           /*    16    32 */
	u8                         lladdr_len;           /*    48     1 */
	u8                         flags;                /*    49     1 */
	u8                         nud_state;            /*    50     1 */
	u8                         type;                 /*    51     1 */
	u8                         dead;                 /*    52     1 */

	/* XXX 3 bytes hole, try to pack */

	int                        refcnt;               /*    56     4 */
	__u8                       primary_key4[4];      /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	__u8                       primary_key6[16];     /*    64    16 */
	long unsigned int          confirmed;            /*    80     8 */
	long unsigned int          updated;              /*    88     8 */
	long unsigned int          used;                 /*    96     8 */
	u32                        err;                  /*   104     4 */
	char                       __data[];             /*   108     0 */

	/* size: 112, cachelines: 2, members: 17 */
	/* sum members: 105, holes: 1, sum holes: 3 */
	/* padding: 4 */
	/* last cacheline: 48 bytes */
};
struct trace_event_data_offsets_neigh_create {
	u32                        dev;                  /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct trace_event_data_offsets_neigh_update {
	u32                        dev;                  /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct trace_event_data_offsets_neigh__update {
	u32                        dev;                  /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct net_dm_drop_point {
	__u8                       pc[8];                /*     0     8 */
	__u32                      count;                /*     8     4 */

	/* size: 12, cachelines: 1, members: 2 */
	/* last cacheline: 12 bytes */
};
struct net_dm_alert_msg {
	__u32                      entries;              /*     0     4 */
	struct net_dm_drop_point   points[];             /*     4     0 */

	/* size: 4, cachelines: 1, members: 2 */
	/* last cacheline: 4 bytes */
};
struct devlink_trap_metadata {
	const char  *              trap_name;            /*     0     8 */
	const char  *              trap_group_name;      /*     8     8 */
	struct net_device *        input_dev;            /*    16     8 */
	const struct flow_action_cookie  * fa_cookie;    /*    24     8 */
	enum devlink_trap_type     trap_type;            /*    32     4 */

	/* size: 40, cachelines: 1, members: 5 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct net_dm_stats {
	u64                        dropped;              /*     0     8 */
	struct u64_stats_sync      syncp;                /*     8     0 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct net_dm_hw_entry {
	char                       trap_name[40];        /*     0    40 */
	u32                        count;                /*    40     4 */

	/* size: 44, cachelines: 1, members: 2 */
	/* last cacheline: 44 bytes */
};
struct net_dm_hw_entries {
	u32                        num_entries;          /*     0     4 */
	struct net_dm_hw_entry     entries[];            /*     4     0 */

	/* size: 4, cachelines: 1, members: 2 */
	/* last cacheline: 4 bytes */
};
struct per_cpu_dm_data {
	spinlock_t                 lock;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	union {
		struct sk_buff *   skb;                  /*     8     8 */
		struct net_dm_hw_entries * hw_entries;   /*     8     8 */
	};                                               /*     8     8 */
	struct sk_buff_head        drop_queue;           /*    16    24 */
	struct work_struct         dm_alert_work;        /*    40    64 */
	/* --- cacheline 1 boundary (64 bytes) was 40 bytes ago --- */
	struct timer_list          send_timer;           /*   104    72 */
	/* --- cacheline 2 boundary (128 bytes) was 48 bytes ago --- */
	struct net_dm_stats        stats;                /*   176     8 */

	/* size: 184, cachelines: 3, members: 6 */
	/* sum members: 180, holes: 1, sum holes: 4 */
	/* last cacheline: 56 bytes */
};
struct dm_hw_stat_delta {
	struct net_device *        dev;                  /*     0     8 */
	long unsigned int          last_rx;              /*     8     8 */
	struct list_head           list;                 /*    16    16 */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*    32    16 */
	long unsigned int          last_drop_val;        /*    48     8 */

	/* size: 56, cachelines: 1, members: 5 */
	/* forced alignments: 1 */
	/* last cacheline: 56 bytes */
} __attribute__((__aligned__(8)));
struct net_dm_alert_ops {
	void                       (*kfree_skb_probe)(void *, struct sk_buff *, void *); /*     0     8 */
	void                       (*napi_poll_probe)(void *, struct napi_struct *, int, int); /*     8     8 */
	void                       (*work_item_func)(struct work_struct *); /*    16     8 */
	void                       (*hw_work_item_func)(struct work_struct *); /*    24     8 */
	void                       (*hw_trap_probe)(void *, const struct devlink  *, struct sk_buff *, const struct devlink_trap_metadata  *); /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct net_dm_skb_cb {
	union {
		struct devlink_trap_metadata * hw_metadata; /*     0     8 */
		void *             pc;                   /*     0     8 */
	};                                               /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct clock_identity {
	u8                         id[8];                /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct port_identity {
	struct clock_identity      clock_identity;       /*     0     8 */
	__be16                     port_number;          /*     8     2 */

	/* size: 10, cachelines: 1, members: 2 */
	/* last cacheline: 10 bytes */
};
struct ptp_header {
	u8                         tsmt;                 /*     0     1 */
	u8                         ver;                  /*     1     1 */
	__be16                     message_length;       /*     2     2 */
	u8                         domain_number;        /*     4     1 */
	u8                         reserved1;            /*     5     1 */
	u8                         flag_field[2];        /*     6     2 */
	__be64                     correction;           /*     8     8 */
	__be32                     reserved2;            /*    16     4 */
	struct port_identity       source_port_identity; /*    20    10 */
	__be16                     sequence_id;          /*    30     2 */
	u8                         control;              /*    32     1 */
	u8                         log_message_interval; /*    33     1 */

	/* size: 34, cachelines: 1, members: 12 */
	/* last cacheline: 34 bytes */
} __attribute__((__packed__));
struct update_classid_context {
	u32                        classid;              /*     0     4 */
	unsigned int               batch;                /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct rtnexthop {
	short unsigned int         rtnh_len;             /*     0     2 */
	unsigned char              rtnh_flags;           /*     2     1 */
	unsigned char              rtnh_hops;            /*     3     1 */
	int                        rtnh_ifindex;         /*     4     4 */

	/* size: 8, cachelines: 1, members: 4 */
	/* last cacheline: 8 bytes */
};
struct lwtunnel_encap_ops {
	int                        (*build_state)(struct net *, struct nlattr *, unsigned int, const void  *, struct lwtunnel_state * *, struct netlink_ext_ack *); /*     0     8 */
	void                       (*destroy_state)(struct lwtunnel_state *); /*     8     8 */
	int                        (*output)(struct net *, struct sock *, struct sk_buff *); /*    16     8 */
	int                        (*input)(struct sk_buff *); /*    24     8 */
	int                        (*fill_encap)(struct sk_buff *, struct lwtunnel_state *); /*    32     8 */
	int                        (*get_encap_size)(struct lwtunnel_state *); /*    40     8 */
	int                        (*cmp_encap)(struct lwtunnel_state *, struct lwtunnel_state *); /*    48     8 */
	int                        (*xmit)(struct sk_buff *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct module *            owner;                /*    64     8 */

	/* size: 72, cachelines: 2, members: 9 */
	/* last cacheline: 8 bytes */
};
struct bpf_lwt_prog {
	struct bpf_prog *          prog;                 /*     0     8 */
	char *                     name;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct bpf_lwt {
	struct bpf_lwt_prog        in;                   /*     0    16 */
	struct bpf_lwt_prog        out;                  /*    16    16 */
	struct bpf_lwt_prog        xmit;                 /*    32    16 */
	int                        family;               /*    48     4 */

	/* size: 56, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 56 bytes */
};
struct bpf_stab {
	struct bpf_map             map __attribute__((__aligned__(64))); /*     0   256 */

	/* XXX last struct has 8 bytes of padding */

	/* --- cacheline 4 boundary (256 bytes) --- */
	struct sock * *            sks;                  /*   256     8 */
	struct sk_psock_progs      progs;                /*   264    56 */
	/* --- cacheline 5 boundary (320 bytes) --- */
	raw_spinlock_t             lock;                 /*   320     4 */

	/* size: 384, cachelines: 6, members: 4 */
	/* padding: 60 */
	/* paddings: 1, sum paddings: 8 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(64)));
struct sock_map_seq_info {
	struct bpf_map *           map;                  /*     0     8 */
	struct sock *              sk;                   /*     8     8 */
	u32                        index;                /*    16     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct bpf_iter__sockmap {
	union {
		struct bpf_iter_meta * meta;             /*     0     8 */
	} __attribute__((__aligned__(8)));               /*     0     8 */
	union {
		struct bpf_map *   map;                  /*     8     8 */
	} __attribute__((__aligned__(8)));               /*     8     8 */
	union {
		void *             key;                  /*    16     8 */
	} __attribute__((__aligned__(8)));               /*    16     8 */
	union {
		struct sock *      sk;                   /*    24     8 */
	} __attribute__((__aligned__(8)));               /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* forced alignments: 4 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct bpf_shtab_elem {
	struct callback_head       rcu __attribute__((__aligned__(8))); /*     0    16 */
	u32                        hash;                 /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	struct sock *              sk;                   /*    24     8 */
	struct hlist_node          node;                 /*    32    16 */
	u8                         key[];                /*    48     0 */

	/* size: 48, cachelines: 1, members: 5 */
	/* sum members: 44, holes: 1, sum holes: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct bpf_shtab_bucket {
	struct hlist_head          head;                 /*     0     8 */
	raw_spinlock_t             lock;                 /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct bpf_shtab {
	struct bpf_map             map __attribute__((__aligned__(64))); /*     0   256 */

	/* XXX last struct has 8 bytes of padding */

	/* --- cacheline 4 boundary (256 bytes) --- */
	struct bpf_shtab_bucket *  buckets;              /*   256     8 */
	u32                        buckets_num;          /*   264     4 */
	u32                        elem_size;            /*   268     4 */
	struct sk_psock_progs      progs;                /*   272    56 */
	/* --- cacheline 5 boundary (320 bytes) was 8 bytes ago --- */
	atomic_t                   count;                /*   328     4 */

	/* size: 384, cachelines: 6, members: 6 */
	/* padding: 52 */
	/* paddings: 1, sum paddings: 8 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(64)));
struct sock_hash_seq_info {
	struct bpf_map *           map;                  /*     0     8 */
	struct bpf_shtab *         htab;                 /*     8     8 */
	u32                        bucket_id;            /*    16     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct dst_cache_pcpu {
	long unsigned int          refresh_ts;           /*     0     8 */
	struct dst_entry *         dst;                  /*     8     8 */
	u32                        cookie;               /*    16     4 */
	union {
		struct in_addr     in_saddr;             /*    20     4 */
		struct in6_addr    in6_saddr;            /*    20    16 */
	};                                               /*    20    16 */

	/* size: 40, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct devlink_dpipe_match {
	enum devlink_dpipe_match_type type;              /*     0     4 */
	unsigned int               header_index;         /*     4     4 */
	struct devlink_dpipe_header * header;            /*     8     8 */
	unsigned int               field_id;             /*    16     4 */

	/* size: 24, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct devlink_dpipe_action {
	enum devlink_dpipe_action_type type;             /*     0     4 */
	unsigned int               header_index;         /*     4     4 */
	struct devlink_dpipe_header * header;            /*     8     8 */
	unsigned int               field_id;             /*    16     4 */

	/* size: 24, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct devlink_dpipe_value {
	union {
		struct devlink_dpipe_action * action;    /*     0     8 */
		struct devlink_dpipe_match * match;      /*     0     8 */
	};                                               /*     0     8 */
	unsigned int               mapping_value;        /*     8     4 */
	bool                       mapping_valid;        /*    12     1 */

	/* XXX 3 bytes hole, try to pack */

	unsigned int               value_size;           /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     value;                /*    24     8 */
	void *                     mask;                 /*    32     8 */

	/* size: 40, cachelines: 1, members: 6 */
	/* sum members: 33, holes: 2, sum holes: 7 */
	/* last cacheline: 40 bytes */
};
struct devlink_dpipe_entry {
	u64                        index;                /*     0     8 */
	struct devlink_dpipe_value * match_values;       /*     8     8 */
	unsigned int               match_values_count;   /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	struct devlink_dpipe_value * action_values;      /*    24     8 */
	unsigned int               action_values_count;  /*    32     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        counter;              /*    40     8 */
	bool                       counter_valid;        /*    48     1 */

	/* size: 56, cachelines: 1, members: 7 */
	/* sum members: 41, holes: 2, sum holes: 8 */
	/* padding: 7 */
	/* last cacheline: 56 bytes */
};
struct devlink_dpipe_dump_ctx {
	struct genl_info *         info;                 /*     0     8 */
	enum devlink_command       cmd;                  /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct sk_buff *           skb;                  /*    16     8 */
	struct nlattr *            nest;                 /*    24     8 */
	void *                     hdr;                  /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* sum members: 36, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct devlink_dpipe_table {
	void *                     priv;                 /*     0     8 */
	struct list_head           list;                 /*     8    16 */
	const char  *              name;                 /*    24     8 */
	bool                       counters_enabled;     /*    32     1 */
	bool                       counter_control_extern; /*    33     1 */
	bool                       resource_valid;       /*    34     1 */

	/* XXX 5 bytes hole, try to pack */

	u64                        resource_id;          /*    40     8 */
	u64                        resource_units;       /*    48     8 */
	struct devlink_dpipe_table_ops * table_ops;      /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*    64    16 */

	/* size: 80, cachelines: 2, members: 10 */
	/* sum members: 75, holes: 1, sum holes: 5 */
	/* forced alignments: 1 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct devlink_dpipe_table_ops {
	int                        (*actions_dump)(void *, struct sk_buff *); /*     0     8 */
	int                        (*matches_dump)(void *, struct sk_buff *); /*     8     8 */
	int                        (*entries_dump)(void *, bool, struct devlink_dpipe_dump_ctx *); /*    16     8 */
	int                        (*counters_set_update)(void *, bool); /*    24     8 */
	u64                        (*size_get)(void *);  /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct devlink_resource_size_params {
	u64                        size_min;             /*     0     8 */
	u64                        size_max;             /*     8     8 */
	u64                        size_granularity;     /*    16     8 */
	enum devlink_resource_unit unit;                 /*    24     4 */

	/* size: 32, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct devlink_resource {
	const char  *              name;                 /*     0     8 */
	u64                        id;                   /*     8     8 */
	u64                        size;                 /*    16     8 */
	u64                        size_new;             /*    24     8 */
	bool                       size_valid;           /*    32     1 */

	/* XXX 7 bytes hole, try to pack */

	struct devlink_resource *  parent;               /*    40     8 */
	struct devlink_resource_size_params size_params; /*    48    32 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
	struct list_head           list;                 /*    80    16 */
	struct list_head           resource_list;        /*    96    16 */
	devlink_resource_occ_get_t * occ_get;            /*   112     8 */
	void *                     occ_get_priv;         /*   120     8 */

	/* size: 128, cachelines: 2, members: 11 */
	/* sum members: 121, holes: 1, sum holes: 7 */
	/* paddings: 1, sum paddings: 4 */
};
union devlink_param_value {
	u8                         vu8;                /*     0     1 */
	u16                        vu16;               /*     0     2 */
	u32                        vu32;               /*     0     4 */
	char                       vstr[32];           /*     0    32 */
	bool                       vbool;              /*     0     1 */
};
struct devlink_param_gset_ctx {
	union devlink_param_value  val;                  /*     0    32 */
	enum devlink_param_cmode   cmode;                /*    32     4 */

	/* size: 36, cachelines: 1, members: 2 */
	/* last cacheline: 36 bytes */
};
struct devlink_flash_notify {
	const char  *              status_msg;           /*     0     8 */
	const char  *              component;            /*     8     8 */
	long unsigned int          done;                 /*    16     8 */
	long unsigned int          total;                /*    24     8 */
	long unsigned int          timeout;              /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct devlink_param {
	u32                        id;                   /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	const char  *              name;                 /*     8     8 */
	bool                       generic;              /*    16     1 */

	/* XXX 3 bytes hole, try to pack */

	enum devlink_param_type    type;                 /*    20     4 */
	long unsigned int          supported_cmodes;     /*    24     8 */
	int                        (*get)(struct devlink *, u32, struct devlink_param_gset_ctx *); /*    32     8 */
	int                        (*set)(struct devlink *, u32, struct devlink_param_gset_ctx *); /*    40     8 */
	int                        (*validate)(struct devlink *, u32, union devlink_param_value, struct netlink_ext_ack *); /*    48     8 */

	/* size: 56, cachelines: 1, members: 8 */
	/* sum members: 49, holes: 2, sum holes: 7 */
	/* last cacheline: 56 bytes */
};
struct devlink_param_item {
	struct list_head           list;                 /*     0    16 */
	const struct devlink_param  * param;             /*    16     8 */
	union devlink_param_value  driverinit_value;     /*    24    32 */
	bool                       driverinit_value_valid; /*    56     1 */
	bool                       published;            /*    57     1 */

	/* size: 64, cachelines: 1, members: 5 */
	/* padding: 6 */
};
struct devlink_region_ops {
	const char  *              name;                 /*     0     8 */
	void                       (*destructor)(const void  *); /*     8     8 */
	int                        (*snapshot)(struct devlink *, const struct devlink_region_ops  *, struct netlink_ext_ack *, u8 * *); /*    16     8 */
	void *                     priv;                 /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct devlink_port_region_ops {
	const char  *              name;                 /*     0     8 */
	void                       (*destructor)(const void  *); /*     8     8 */
	int                        (*snapshot)(struct devlink_port *, const struct devlink_port_region_ops  *, struct netlink_ext_ack *, u8 * *); /*    16     8 */
	void *                     priv;                 /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct devlink_health_reporter_ops {
	char *                     name;                 /*     0     8 */
	int                        (*recover)(struct devlink_health_reporter *, void *, struct netlink_ext_ack *); /*     8     8 */
	int                        (*dump)(struct devlink_health_reporter *, struct devlink_fmsg *, void *, struct netlink_ext_ack *); /*    16     8 */
	int                        (*diagnose)(struct devlink_health_reporter *, struct devlink_fmsg *, struct netlink_ext_ack *); /*    24     8 */
	int                        (*test)(struct devlink_health_reporter *, struct netlink_ext_ack *); /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct devlink_health_reporter {
	struct list_head           list;                 /*     0    16 */
	void *                     priv;                 /*    16     8 */
	const struct devlink_health_reporter_ops  * ops; /*    24     8 */
	struct devlink *           devlink;              /*    32     8 */
	struct devlink_port *      devlink_port;         /*    40     8 */
	struct devlink_fmsg *      dump_fmsg;            /*    48     8 */
	struct mutex               dump_lock;            /*    56    32 */
	/* --- cacheline 1 boundary (64 bytes) was 24 bytes ago --- */
	u64                        graceful_period;      /*    88     8 */
	bool                       auto_recover;         /*    96     1 */
	bool                       auto_dump;            /*    97     1 */
	u8                         health_state;         /*    98     1 */

	/* XXX 5 bytes hole, try to pack */

	u64                        dump_ts;              /*   104     8 */
	u64                        dump_real_ts;         /*   112     8 */
	u64                        error_count;          /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	u64                        recovery_count;       /*   128     8 */
	u64                        last_recovery_ts;     /*   136     8 */
	refcount_t                 refcount;             /*   144     4 */

	/* size: 152, cachelines: 3, members: 17 */
	/* sum members: 143, holes: 1, sum holes: 5 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct devlink_fmsg {
	struct list_head           item_list;            /*     0    16 */
	bool                       putting_binary;       /*    16     1 */

	/* size: 24, cachelines: 1, members: 2 */
	/* padding: 7 */
	/* last cacheline: 24 bytes */
};
struct devlink_info_req {
	struct sk_buff *           msg;                  /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct trace_event_raw_devlink_hwmsg {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        __data_loc_bus_name;  /*     8     4 */
	u32                        __data_loc_dev_name;  /*    12     4 */
	u32                        __data_loc_driver_name; /*    16     4 */
	bool                       incoming;             /*    20     1 */

	/* XXX 3 bytes hole, try to pack */

	long unsigned int          type;                 /*    24     8 */
	u32                        __data_loc_buf;       /*    32     4 */

	/* XXX 4 bytes hole, try to pack */

	size_t                     len;                  /*    40     8 */
	char                       __data[];             /*    48     0 */

	/* size: 48, cachelines: 1, members: 9 */
	/* sum members: 41, holes: 2, sum holes: 7 */
	/* last cacheline: 48 bytes */
};
struct trace_event_raw_devlink_hwerr {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        __data_loc_bus_name;  /*     8     4 */
	u32                        __data_loc_dev_name;  /*    12     4 */
	u32                        __data_loc_driver_name; /*    16     4 */
	int                        err;                  /*    20     4 */
	u32                        __data_loc_msg;       /*    24     4 */
	char                       __data[];             /*    28     0 */

	/* size: 28, cachelines: 1, members: 7 */
	/* last cacheline: 28 bytes */
};
struct trace_event_raw_devlink_health_report {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        __data_loc_bus_name;  /*     8     4 */
	u32                        __data_loc_dev_name;  /*    12     4 */
	u32                        __data_loc_driver_name; /*    16     4 */
	u32                        __data_loc_reporter_name; /*    20     4 */
	u32                        __data_loc_msg;       /*    24     4 */
	char                       __data[];             /*    28     0 */

	/* size: 28, cachelines: 1, members: 7 */
	/* last cacheline: 28 bytes */
};
struct trace_event_raw_devlink_health_recover_aborted {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        __data_loc_bus_name;  /*     8     4 */
	u32                        __data_loc_dev_name;  /*    12     4 */
	u32                        __data_loc_driver_name; /*    16     4 */
	u32                        __data_loc_reporter_name; /*    20     4 */
	bool                       health_state;         /*    24     1 */

	/* XXX 7 bytes hole, try to pack */

	u64                        time_since_last_recover; /*    32     8 */
	char                       __data[];             /*    40     0 */

	/* size: 40, cachelines: 1, members: 8 */
	/* sum members: 33, holes: 1, sum holes: 7 */
	/* last cacheline: 40 bytes */
};
struct trace_event_raw_devlink_health_reporter_state_update {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        __data_loc_bus_name;  /*     8     4 */
	u32                        __data_loc_dev_name;  /*    12     4 */
	u32                        __data_loc_driver_name; /*    16     4 */
	u32                        __data_loc_reporter_name; /*    20     4 */
	u8                         new_state;            /*    24     1 */
	char                       __data[];             /*    25     0 */

	/* size: 28, cachelines: 1, members: 7 */
	/* padding: 3 */
	/* last cacheline: 28 bytes */
};
struct trace_event_raw_devlink_trap_report {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        __data_loc_bus_name;  /*     8     4 */
	u32                        __data_loc_dev_name;  /*    12     4 */
	u32                        __data_loc_driver_name; /*    16     4 */
	u32                        __data_loc_trap_name; /*    20     4 */
	u32                        __data_loc_trap_group_name; /*    24     4 */
	u32                        __data_loc_input_dev_name; /*    28     4 */
	char                       __data[];             /*    32     0 */

	/* size: 32, cachelines: 1, members: 8 */
	/* last cacheline: 32 bytes */
};
struct trace_event_data_offsets_devlink_hwmsg {
	u32                        bus_name;             /*     0     4 */
	u32                        dev_name;             /*     4     4 */
	u32                        driver_name;          /*     8     4 */
	u32                        buf;                  /*    12     4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct trace_event_data_offsets_devlink_hwerr {
	u32                        bus_name;             /*     0     4 */
	u32                        dev_name;             /*     4     4 */
	u32                        driver_name;          /*     8     4 */
	u32                        msg;                  /*    12     4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct trace_event_data_offsets_devlink_health_report {
	u32                        bus_name;             /*     0     4 */
	u32                        dev_name;             /*     4     4 */
	u32                        driver_name;          /*     8     4 */
	u32                        reporter_name;        /*    12     4 */
	u32                        msg;                  /*    16     4 */

	/* size: 20, cachelines: 1, members: 5 */
	/* last cacheline: 20 bytes */
};
struct trace_event_data_offsets_devlink_health_recover_aborted {
	u32                        bus_name;             /*     0     4 */
	u32                        dev_name;             /*     4     4 */
	u32                        driver_name;          /*     8     4 */
	u32                        reporter_name;        /*    12     4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct trace_event_data_offsets_devlink_health_reporter_state_update {
	u32                        bus_name;             /*     0     4 */
	u32                        dev_name;             /*     4     4 */
	u32                        driver_name;          /*     8     4 */
	u32                        reporter_name;        /*    12     4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct trace_event_data_offsets_devlink_trap_report {
	u32                        bus_name;             /*     0     4 */
	u32                        dev_name;             /*     4     4 */
	u32                        driver_name;          /*     8     4 */
	u32                        trap_name;            /*    12     4 */
	u32                        trap_group_name;      /*    16     4 */
	u32                        input_dev_name;       /*    20     4 */

	/* size: 24, cachelines: 1, members: 6 */
	/* last cacheline: 24 bytes */
};
struct devlink_sb {
	struct list_head           list;                 /*     0    16 */
	unsigned int               index;                /*    16     4 */
	u32                        size;                 /*    20     4 */
	u16                        ingress_pools_count;  /*    24     2 */
	u16                        egress_pools_count;   /*    26     2 */
	u16                        ingress_tc_count;     /*    28     2 */
	u16                        egress_tc_count;      /*    30     2 */

	/* size: 32, cachelines: 1, members: 7 */
	/* last cacheline: 32 bytes */
};
struct devlink_region {
	struct devlink *           devlink;              /*     0     8 */
	struct devlink_port *      port;                 /*     8     8 */
	struct list_head           list;                 /*    16    16 */
	union {
		const struct devlink_region_ops  * ops;  /*    32     8 */
		const struct devlink_port_region_ops  * port_ops; /*    32     8 */
	};                                               /*    32     8 */
	struct list_head           snapshot_list;        /*    40    16 */
	u32                        max_snapshots;        /*    56     4 */
	u32                        cur_snapshots;        /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u64                        size;                 /*    64     8 */

	/* size: 72, cachelines: 2, members: 8 */
	/* last cacheline: 8 bytes */
};
struct devlink_snapshot {
	struct list_head           list;                 /*     0    16 */
	struct devlink_region *    region;               /*    16     8 */
	u8 *                       data;                 /*    24     8 */
	u32                        id;                   /*    32     4 */

	/* size: 40, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct devlink_reload_combination {
	enum devlink_reload_action action;               /*     0     4 */
	enum devlink_reload_limit  limit;                /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct devlink_fmsg_item {
	struct list_head           list;                 /*     0    16 */
	int                        attrtype;             /*    16     4 */
	u8                         nla_type;             /*    20     1 */

	/* XXX 1 byte hole, try to pack */

	u16                        len;                  /*    22     2 */
	int                        value[];              /*    24     0 */

	/* size: 24, cachelines: 1, members: 5 */
	/* sum members: 23, holes: 1, sum holes: 1 */
	/* last cacheline: 24 bytes */
};
struct devlink_stats {
	u64                        rx_bytes;             /*     0     8 */
	u64                        rx_packets;           /*     8     8 */
	struct u64_stats_sync      syncp;                /*    16     0 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct devlink_trap_policer_item {
	const struct devlink_trap_policer  * policer;    /*     0     8 */
	u64                        rate;                 /*     8     8 */
	u64                        burst;                /*    16     8 */
	struct list_head           list;                 /*    24    16 */

	/* size: 40, cachelines: 1, members: 4 */
	/* last cacheline: 40 bytes */
};
struct devlink_trap_group_item {
	const struct devlink_trap_group  * group;        /*     0     8 */
	struct devlink_trap_policer_item * policer_item; /*     8     8 */
	struct list_head           list;                 /*    16    16 */
	struct devlink_stats *     stats;                /*    32     8 */

	/* size: 40, cachelines: 1, members: 4 */
	/* last cacheline: 40 bytes */
};
struct devlink_trap_item {
	const struct devlink_trap  * trap;               /*     0     8 */
	struct devlink_trap_group_item * group_item;     /*     8     8 */
	struct list_head           list;                 /*    16    16 */
	enum devlink_trap_action   action;               /*    32     4 */

	/* XXX 4 bytes hole, try to pack */

	struct devlink_stats *     stats;                /*    40     8 */
	void *                     priv;                 /*    48     8 */

	/* size: 56, cachelines: 1, members: 6 */
	/* sum members: 52, holes: 1, sum holes: 4 */
	/* last cacheline: 56 bytes */
};
struct gro_cells {
	struct gro_cell *          cells;                /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct gro_cell {
	struct sk_buff_head        napi_skbs;            /*     0    24 */
	struct napi_struct         napi __attribute__((__aligned__(8))); /*    24   456 */

	/* size: 480, cachelines: 8, members: 2 */
	/* forced alignments: 1 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct bpf_sk_storage_diag {
	u32                        nr_maps;              /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct bpf_map *           maps[];               /*     8     0 */

	/* size: 8, cachelines: 1, members: 2 */
	/* sum members: 4, holes: 1, sum holes: 4 */
	/* last cacheline: 8 bytes */
};
struct bpf_iter_seq_sk_storage_map_info {
	struct bpf_map *           map;                  /*     0     8 */
	unsigned int               bucket_id;            /*     8     4 */
	unsigned int               skip_elems;           /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct bpf_iter__bpf_sk_storage_map {
	union {
		struct bpf_iter_meta * meta;             /*     0     8 */
	} __attribute__((__aligned__(8)));               /*     0     8 */
	union {
		struct bpf_map *   map;                  /*     8     8 */
	} __attribute__((__aligned__(8)));               /*     8     8 */
	union {
		struct sock *      sk;                   /*    16     8 */
	} __attribute__((__aligned__(8)));               /*    16     8 */
	union {
		void *             value;                /*    24     8 */
	} __attribute__((__aligned__(8)));               /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* forced alignments: 4 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct compat_cmsghdr {
	compat_size_t              cmsg_len;             /*     0     4 */
	compat_int_t               cmsg_level;           /*     4     4 */
	compat_int_t               cmsg_type;            /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct fch_hdr {
	__u8                       daddr[6];             /*     0     6 */
	__u8                       saddr[6];             /*     6     6 */

	/* size: 12, cachelines: 1, members: 2 */
	/* last cacheline: 12 bytes */
};
struct fcllc {
	__u8                       dsap;                 /*     0     1 */
	__u8                       ssap;                 /*     1     1 */
	__u8                       llc;                  /*     2     1 */
	__u8                       protid[3];            /*     3     3 */
	__be16                     ethertype;            /*     6     2 */

	/* size: 8, cachelines: 1, members: 5 */
	/* last cacheline: 8 bytes */
};
struct tc_ratespec {
	unsigned char              cell_log;             /*     0     1 */
	__u8                       linklayer;            /*     1     1 */
	short unsigned int         overhead;             /*     2     2 */
	short int                  cell_align;           /*     4     2 */
	short unsigned int         mpu;                  /*     6     2 */
	__u32                      rate;                 /*     8     4 */

	/* size: 12, cachelines: 1, members: 6 */
	/* last cacheline: 12 bytes */
};
struct tc_prio_qopt {
	int                        bands;                /*     0     4 */
	__u8                       priomap[16];          /*     4    16 */

	/* size: 20, cachelines: 1, members: 2 */
	/* last cacheline: 20 bytes */
};
struct vlan_pcpu_stats {
	u64                        rx_packets;           /*     0     8 */
	u64                        rx_bytes;             /*     8     8 */
	u64                        rx_multicast;         /*    16     8 */
	u64                        tx_packets;           /*    24     8 */
	u64                        tx_bytes;             /*    32     8 */
	struct u64_stats_sync      syncp;                /*    40     0 */
	u32                        rx_errors;            /*    40     4 */
	u32                        tx_dropped;           /*    44     4 */

	/* size: 48, cachelines: 1, members: 8 */
	/* last cacheline: 48 bytes */
};
struct skb_array {
	struct ptr_ring            ring __attribute__((__aligned__(64))); /*     0   192 */

	/* XXX last struct has 48 bytes of padding */

	/* size: 192, cachelines: 3, members: 1 */
	/* paddings: 1, sum paddings: 48 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(64)));
struct macvlan_dev {
	struct net_device *        dev;                  /*     0     8 */
	struct list_head           list;                 /*     8    16 */
	struct hlist_node          hlist;                /*    24    16 */
	struct macvlan_port *      port;                 /*    40     8 */
	struct net_device *        lowerdev;             /*    48     8 */
	void *                     accel_priv;           /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct vlan_pcpu_stats *   pcpu_stats;           /*    64     8 */
	long unsigned int          mc_filter[4];         /*    72    32 */
	netdev_features_t          set_features;         /*   104     8 */
	enum macvlan_mode          mode;                 /*   112     4 */
	u16                        flags;                /*   116     2 */

	/* XXX 2 bytes hole, try to pack */

	unsigned int               macaddr_count;        /*   120     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 2 boundary (128 bytes) --- */
	struct netpoll *           netpoll;              /*   128     8 */

	/* size: 136, cachelines: 3, members: 13 */
	/* sum members: 130, holes: 2, sum holes: 6 */
	/* last cacheline: 8 bytes */
};
struct psched_ratecfg {
	u64                        rate_bytes_ps;        /*     0     8 */
	u32                        mult;                 /*     8     4 */
	u16                        overhead;             /*    12     2 */
	u16                        mpu;                  /*    14     2 */
	u8                         linklayer;            /*    16     1 */
	u8                         shift;                /*    17     1 */

	/* size: 24, cachelines: 1, members: 6 */
	/* padding: 6 */
	/* last cacheline: 24 bytes */
};
struct mini_Qdisc_pair {
	struct mini_Qdisc          miniq1 __attribute__((__aligned__(8))); /*     0    48 */
	struct mini_Qdisc          miniq2 __attribute__((__aligned__(8))); /*    48    48 */
	/* --- cacheline 1 boundary (64 bytes) was 32 bytes ago --- */
	struct mini_Qdisc * *      p_miniq;              /*    96     8 */

	/* size: 104, cachelines: 2, members: 3 */
	/* forced alignments: 2 */
	/* last cacheline: 40 bytes */
} __attribute__((__aligned__(8)));
struct pfifo_fast_priv {
	struct skb_array           q[3] __attribute__((__aligned__(64))); /*     0   576 */

	/* size: 576, cachelines: 9, members: 1 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(64)));
struct tc_qopt_offload_stats {
	struct gnet_stats_basic_packed * bstats;         /*     0     8 */
	struct gnet_stats_queue *  qstats;               /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct tc_mq_opt_offload_graft_params {
	long unsigned int          queue;                /*     0     8 */
	u32                        child_handle;         /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct tc_mq_qopt_offload {
	enum tc_mq_command         command;              /*     0     4 */
	u32                        handle;               /*     4     4 */
	union {
		struct tc_qopt_offload_stats stats;      /*     8    16 */
		struct tc_mq_opt_offload_graft_params graft_params; /*     8    16 */
	};                                               /*     8    16 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct mq_sched {
	struct Qdisc * *           qdiscs;               /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct qdisc_rate_table {
	struct tc_ratespec         rate;                 /*     0    12 */
	u32                        data[256];            /*    12  1024 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 16 boundary (1024 bytes) was 16 bytes ago --- */
	struct qdisc_rate_table *  next;                 /*  1040     8 */
	int                        refcnt;               /*  1048     4 */

	/* size: 1056, cachelines: 17, members: 4 */
	/* sum members: 1048, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct Qdisc_class_common {
	u32                        classid;              /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct hlist_node          hnode;                /*     8    16 */

	/* size: 24, cachelines: 1, members: 2 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct Qdisc_class_hash {
	struct hlist_head *        hash;                 /*     0     8 */
	unsigned int               hashsize;             /*     8     4 */
	unsigned int               hashmask;             /*    12     4 */
	unsigned int               hashelems;            /*    16     4 */

	/* size: 24, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct qdisc_watchdog {
	u64                        last_expires;         /*     0     8 */
	struct hrtimer             timer __attribute__((__aligned__(8))); /*     8    96 */
	/* --- cacheline 1 boundary (64 bytes) was 40 bytes ago --- */
	struct Qdisc *             qdisc;                /*   104     8 */

	/* size: 112, cachelines: 2, members: 3 */
	/* forced alignments: 1 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct tc_root_qopt_offload {
	enum tc_root_command       command;              /*     0     4 */
	u32                        handle;               /*     4     4 */
	bool                       ingress;              /*     8     1 */

	/* size: 12, cachelines: 1, members: 3 */
	/* padding: 3 */
	/* last cacheline: 12 bytes */
};
struct check_loop_arg {
	struct qdisc_walker        w;                    /*     0    24 */
	struct Qdisc *             p;                    /*    24     8 */
	int                        depth;                /*    32     4 */

	/* size: 40, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct tcf_bind_args {
	struct tcf_walker          w;                    /*     0    32 */
	long unsigned int          base;                 /*    32     8 */
	long unsigned int          cl;                   /*    40     8 */
	u32                        classid;              /*    48     4 */

	/* size: 56, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 56 bytes */
};
struct tc_bind_class_args {
	struct qdisc_walker        w;                    /*     0    24 */
	long unsigned int          new_cl;               /*    24     8 */
	u32                        portid;               /*    32     4 */
	u32                        clid;                 /*    36     4 */

	/* size: 40, cachelines: 1, members: 4 */
	/* last cacheline: 40 bytes */
};
struct qdisc_dump_args {
	struct qdisc_walker        w;                    /*     0    24 */
	struct sk_buff *           skb;                  /*    24     8 */
	struct netlink_callback *  cb;                   /*    32     8 */

	/* size: 40, cachelines: 1, members: 3 */
	/* last cacheline: 40 bytes */
};
struct tcf_t {
	__u64                      install;              /*     0     8 */
	__u64                      lastuse;              /*     8     8 */
	__u64                      expires;              /*    16     8 */
	__u64                      firstuse;             /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct psample_group {
	struct list_head           list;                 /*     0    16 */
	struct net *               net;                  /*    16     8 */
	u32                        group_num;            /*    24     4 */
	u32                        refcount;             /*    28     4 */
	u32                        seq;                  /*    32     4 */

	/* XXX 4 bytes hole, try to pack */

	struct callback_head       rcu __attribute__((__aligned__(8))); /*    40    16 */

	/* size: 56, cachelines: 1, members: 6 */
	/* sum members: 52, holes: 1, sum holes: 4 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 4 */
	/* last cacheline: 56 bytes */
} __attribute__((__aligned__(8)));
struct action_gate_entry {
	u8                         gate_state;           /*     0     1 */

	/* XXX 3 bytes hole, try to pack */

	u32                        interval;             /*     4     4 */
	s32                        ipv;                  /*     8     4 */
	s32                        maxoctets;            /*    12     4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* sum members: 13, holes: 1, sum holes: 3 */
	/* last cacheline: 16 bytes */
};
struct tcf_idrinfo {
	struct mutex               lock;                 /*     0    32 */
	struct idr                 action_idr;           /*    32    24 */
	struct net *               net;                  /*    56     8 */

	/* size: 64, cachelines: 1, members: 3 */
};
struct tc_action {
	const struct tc_action_ops  * ops;               /*     0     8 */
	__u32                      type;                 /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct tcf_idrinfo *       idrinfo;              /*    16     8 */
	u32                        tcfa_index;           /*    24     4 */
	refcount_t                 tcfa_refcnt;          /*    28     4 */
	atomic_t                   tcfa_bindcnt;         /*    32     4 */
	int                        tcfa_action;          /*    36     4 */
	struct tcf_t               tcfa_tm;              /*    40    32 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	struct gnet_stats_basic_packed tcfa_bstats;      /*    72    16 */
	struct gnet_stats_basic_packed tcfa_bstats_hw;   /*    88    16 */
	struct gnet_stats_queue    tcfa_qstats;          /*   104    20 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 2 boundary (128 bytes) --- */
	struct net_rate_estimator * tcfa_rate_est;       /*   128     8 */
	spinlock_t                 tcfa_lock;            /*   136     4 */

	/* XXX 4 bytes hole, try to pack */

	struct gnet_stats_basic_cpu * cpu_bstats;        /*   144     8 */
	struct gnet_stats_basic_cpu * cpu_bstats_hw;     /*   152     8 */
	struct gnet_stats_queue *  cpu_qstats;           /*   160     8 */
	struct tc_cookie *         act_cookie;           /*   168     8 */
	struct tcf_chain *         goto_chain;           /*   176     8 */
	u32                        tcfa_flags;           /*   184     4 */
	u8                         hw_stats;             /*   188     1 */
	u8                         used_hw_stats;        /*   189     1 */
	bool                       used_hw_stats_valid;  /*   190     1 */

	/* size: 192, cachelines: 3, members: 22 */
	/* sum members: 179, holes: 3, sum holes: 12 */
	/* padding: 1 */
};
struct tc_action_ops {
	struct list_head           head;                 /*     0    16 */
	char                       kind[16];             /*    16    16 */
	enum tca_id                id;                   /*    32     4 */

	/* XXX 4 bytes hole, try to pack */

	size_t                     size;                 /*    40     8 */
	struct module *            owner;                /*    48     8 */
	int                        (*act)(struct sk_buff *, const struct tc_action  *, struct tcf_result *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*dump)(struct sk_buff *, struct tc_action *, int, int); /*    64     8 */
	void                       (*cleanup)(struct tc_action *); /*    72     8 */
	int                        (*lookup)(struct net *, struct tc_action * *, u32); /*    80     8 */
	int                        (*init)(struct net *, struct nlattr *, struct nlattr *, struct tc_action * *, int, int, bool, struct tcf_proto *, u32, struct netlink_ext_ack *); /*    88     8 */
	int                        (*walk)(struct net *, struct sk_buff *, struct netlink_callback *, int, const struct tc_action_ops  *, struct netlink_ext_ack *); /*    96     8 */
	void                       (*stats_update)(struct tc_action *, u64, u64, u64, u64, bool); /*   104     8 */
	size_t                     (*get_fill_size)(const struct tc_action  *); /*   112     8 */
	struct net_device *        (*get_dev)(const struct tc_action  *, tc_action_priv_destructor *); /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct psample_group *     (*get_psample_group)(const struct tc_action  *, tc_action_priv_destructor *); /*   128     8 */

	/* size: 136, cachelines: 3, members: 15 */
	/* sum members: 132, holes: 1, sum holes: 4 */
	/* last cacheline: 8 bytes */
};
struct tc_cookie {
	u8 *                       data;                 /*     0     8 */
	u32                        len;                  /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct callback_head       rcu __attribute__((__aligned__(8))); /*    16    16 */

	/* size: 32, cachelines: 1, members: 3 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 4 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct tcf_block_ext_info {
	enum flow_block_binder_type binder_type;         /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	tcf_chain_head_change_t *  chain_head_change;    /*     8     8 */
	void *                     chain_head_change_priv; /*    16     8 */
	u32                        block_index;          /*    24     4 */

	/* size: 32, cachelines: 1, members: 4 */
	/* sum members: 24, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct tcf_qevent {
	struct tcf_block *         block;                /*     0     8 */
	struct tcf_block_ext_info  info;                 /*     8    32 */

	/* XXX last struct has 4 bytes of padding */

	struct tcf_proto *         filter_chain;         /*    40     8 */

	/* size: 48, cachelines: 1, members: 3 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 48 bytes */
};
struct tcf_exts {
	__u32                      type;                 /*     0     4 */
	int                        nr_actions;           /*     4     4 */
	struct tc_action * *       actions;              /*     8     8 */
	struct net *               net;                  /*    16     8 */
	int                        action;               /*    24     4 */
	int                        police;               /*    28     4 */

	/* size: 32, cachelines: 1, members: 6 */
	/* last cacheline: 32 bytes */
};
struct tc_pedit_key {
	__u32                      mask;                 /*     0     4 */
	__u32                      val;                  /*     4     4 */
	__u32                      off;                  /*     8     4 */
	__u32                      at;                   /*    12     4 */
	__u32                      offmask;              /*    16     4 */
	__u32                      shift;                /*    20     4 */

	/* size: 24, cachelines: 1, members: 6 */
	/* last cacheline: 24 bytes */
};
struct tcf_pedit_key_ex {
	enum pedit_header_type     htype;                /*     0     4 */
	enum pedit_cmd             cmd;                  /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct tcf_pedit {
	struct tc_action           common;               /*     0   192 */

	/* XXX last struct has 1 byte of padding */

	/* --- cacheline 3 boundary (192 bytes) --- */
	unsigned char              tcfp_nkeys;           /*   192     1 */
	unsigned char              tcfp_flags;           /*   193     1 */

	/* XXX 2 bytes hole, try to pack */

	u32                        tcfp_off_max_hint;    /*   196     4 */
	struct tc_pedit_key *      tcfp_keys;            /*   200     8 */
	struct tcf_pedit_key_ex *  tcfp_keys_ex;         /*   208     8 */

	/* size: 216, cachelines: 4, members: 6 */
	/* sum members: 214, holes: 1, sum holes: 2 */
	/* paddings: 1, sum paddings: 1 */
	/* last cacheline: 24 bytes */
};
struct tcf_mirred {
	struct tc_action           common;               /*     0   192 */

	/* XXX last struct has 1 byte of padding */

	/* --- cacheline 3 boundary (192 bytes) --- */
	int                        tcfm_eaction;         /*   192     4 */
	bool                       tcfm_mac_header_xmit; /*   196     1 */

	/* XXX 3 bytes hole, try to pack */

	struct net_device *        tcfm_dev;             /*   200     8 */
	struct list_head           tcfm_list;            /*   208    16 */

	/* size: 224, cachelines: 4, members: 5 */
	/* sum members: 221, holes: 1, sum holes: 3 */
	/* paddings: 1, sum paddings: 1 */
	/* last cacheline: 32 bytes */
};
struct tcf_vlan_params {
	int                        tcfv_action;          /*     0     4 */
	unsigned char              tcfv_push_dst[6];     /*     4     6 */
	unsigned char              tcfv_push_src[6];     /*    10     6 */
	u16                        tcfv_push_vid;        /*    16     2 */
	__be16                     tcfv_push_proto;      /*    18     2 */
	u8                         tcfv_push_prio;       /*    20     1 */
	bool                       tcfv_push_prio_exists; /*    21     1 */

	/* XXX 2 bytes hole, try to pack */

	struct callback_head       rcu __attribute__((__aligned__(8))); /*    24    16 */

	/* size: 40, cachelines: 1, members: 8 */
	/* sum members: 38, holes: 1, sum holes: 2 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 2 */
	/* last cacheline: 40 bytes */
} __attribute__((__aligned__(8)));
struct tcf_vlan {
	struct tc_action           common;               /*     0   192 */

	/* XXX last struct has 1 byte of padding */

	/* --- cacheline 3 boundary (192 bytes) --- */
	struct tcf_vlan_params *   vlan_p;               /*   192     8 */

	/* size: 200, cachelines: 4, members: 2 */
	/* paddings: 1, sum paddings: 1 */
	/* last cacheline: 8 bytes */
};
struct tcf_tunnel_key_params {
	struct callback_head       rcu __attribute__((__aligned__(8))); /*     0    16 */
	int                        tcft_action;          /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	struct metadata_dst *      tcft_enc_metadata;    /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct tcf_tunnel_key {
	struct tc_action           common;               /*     0   192 */

	/* XXX last struct has 1 byte of padding */

	/* --- cacheline 3 boundary (192 bytes) --- */
	struct tcf_tunnel_key_params * params;           /*   192     8 */

	/* size: 200, cachelines: 4, members: 2 */
	/* paddings: 1, sum paddings: 1 */
	/* last cacheline: 8 bytes */
};
struct tcf_csum_params {
	u32                        update_flags;         /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct callback_head       rcu __attribute__((__aligned__(8))); /*     8    16 */

	/* size: 24, cachelines: 1, members: 2 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 4 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct tcf_csum {
	struct tc_action           common;               /*     0   192 */

	/* XXX last struct has 1 byte of padding */

	/* --- cacheline 3 boundary (192 bytes) --- */
	struct tcf_csum_params *   params;               /*   192     8 */

	/* size: 200, cachelines: 4, members: 2 */
	/* paddings: 1, sum paddings: 1 */
	/* last cacheline: 8 bytes */
};
struct tcf_gact {
	struct tc_action           common;               /*     0   192 */

	/* XXX last struct has 1 byte of padding */

	/* --- cacheline 3 boundary (192 bytes) --- */
	u16                        tcfg_ptype;           /*   192     2 */
	u16                        tcfg_pval;            /*   194     2 */
	int                        tcfg_paction;         /*   196     4 */
	atomic_t                   packets;              /*   200     4 */

	/* size: 208, cachelines: 4, members: 5 */
	/* padding: 4 */
	/* paddings: 1, sum paddings: 1 */
	/* last cacheline: 16 bytes */
};
struct tcf_police_params {
	int                        tcfp_result;          /*     0     4 */
	u32                        tcfp_ewma_rate;       /*     4     4 */
	s64                        tcfp_burst;           /*     8     8 */
	u32                        tcfp_mtu;             /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	s64                        tcfp_mtu_ptoks;       /*    24     8 */
	struct psched_ratecfg      rate;                 /*    32    24 */

	/* XXX last struct has 6 bytes of padding */

	bool                       rate_present;         /*    56     1 */

	/* XXX 7 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	struct psched_ratecfg      peak;                 /*    64    24 */

	/* XXX last struct has 6 bytes of padding */

	bool                       peak_present;         /*    88     1 */

	/* XXX 7 bytes hole, try to pack */

	struct callback_head       rcu __attribute__((__aligned__(8))); /*    96    16 */

	/* size: 112, cachelines: 2, members: 10 */
	/* sum members: 94, holes: 3, sum holes: 18 */
	/* paddings: 2, sum paddings: 12 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 7 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct tcf_police {
	struct tc_action           common;               /*     0   192 */

	/* XXX last struct has 1 byte of padding */

	/* --- cacheline 3 boundary (192 bytes) --- */
	struct tcf_police_params * params;               /*   192     8 */

	/* XXX 56 bytes hole, try to pack */

	/* --- cacheline 4 boundary (256 bytes) --- */
	spinlock_t                 tcfp_lock __attribute__((__aligned__(64))); /*   256     4 */

	/* XXX 4 bytes hole, try to pack */

	s64                        tcfp_toks;            /*   264     8 */
	s64                        tcfp_ptoks;           /*   272     8 */
	s64                        tcfp_t_c;             /*   280     8 */

	/* size: 320, cachelines: 5, members: 6 */
	/* sum members: 228, holes: 2, sum holes: 60 */
	/* padding: 32 */
	/* paddings: 1, sum paddings: 1 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 56 */
} __attribute__((__aligned__(64)));
struct tcf_sample {
	struct tc_action           common;               /*     0   192 */

	/* XXX last struct has 1 byte of padding */

	/* --- cacheline 3 boundary (192 bytes) --- */
	u32                        rate;                 /*   192     4 */
	bool                       truncate;             /*   196     1 */

	/* XXX 3 bytes hole, try to pack */

	u32                        trunc_size;           /*   200     4 */

	/* XXX 4 bytes hole, try to pack */

	struct psample_group *     psample_group;        /*   208     8 */
	u32                        psample_group_num;    /*   216     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           tcfm_list;            /*   224    16 */

	/* size: 240, cachelines: 4, members: 7 */
	/* sum members: 229, holes: 3, sum holes: 11 */
	/* paddings: 1, sum paddings: 1 */
	/* last cacheline: 48 bytes */
};
struct tcf_skbedit_params {
	u32                        flags;                /*     0     4 */
	u32                        priority;             /*     4     4 */
	u32                        mark;                 /*     8     4 */
	u32                        mask;                 /*    12     4 */
	u16                        queue_mapping;        /*    16     2 */
	u16                        ptype;                /*    18     2 */

	/* XXX 4 bytes hole, try to pack */

	struct callback_head       rcu __attribute__((__aligned__(8))); /*    24    16 */

	/* size: 40, cachelines: 1, members: 7 */
	/* sum members: 36, holes: 1, sum holes: 4 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 4 */
	/* last cacheline: 40 bytes */
} __attribute__((__aligned__(8)));
struct tcf_skbedit {
	struct tc_action           common;               /*     0   192 */

	/* XXX last struct has 1 byte of padding */

	/* --- cacheline 3 boundary (192 bytes) --- */
	struct tcf_skbedit_params * params;              /*   192     8 */

	/* size: 200, cachelines: 4, members: 2 */
	/* paddings: 1, sum paddings: 1 */
	/* last cacheline: 8 bytes */
};
struct nf_nat_range2 {
	unsigned int               flags;                /*     0     4 */
	union nf_inet_addr         min_addr;             /*     4    16 */
	union nf_inet_addr         max_addr;             /*    20    16 */
	union nf_conntrack_man_proto min_proto;          /*    36     2 */
	union nf_conntrack_man_proto max_proto;          /*    38     2 */
	union nf_conntrack_man_proto base_proto;         /*    40     2 */

	/* size: 44, cachelines: 1, members: 6 */
	/* padding: 2 */
	/* last cacheline: 44 bytes */
};
struct tcf_ct_params {
	struct nf_conn *           tmpl;                 /*     0     8 */
	u16                        zone;                 /*     8     2 */

	/* XXX 2 bytes hole, try to pack */

	u32                        mark;                 /*    12     4 */
	u32                        mark_mask;            /*    16     4 */
	u32                        labels[4];            /*    20    16 */
	u32                        labels_mask[4];       /*    36    16 */
	struct nf_nat_range2       range;                /*    52    44 */

	/* XXX last struct has 2 bytes of padding */

	/* --- cacheline 1 boundary (64 bytes) was 32 bytes ago --- */
	bool                       ipv4_range;           /*    96     1 */

	/* XXX 1 byte hole, try to pack */

	u16                        ct_action;            /*    98     2 */

	/* XXX 4 bytes hole, try to pack */

	struct callback_head       rcu __attribute__((__aligned__(8))); /*   104    16 */
	struct tcf_ct_flow_table * ct_ft;                /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct nf_flowtable *      nf_ft;                /*   128     8 */

	/* size: 136, cachelines: 3, members: 12 */
	/* sum members: 129, holes: 3, sum holes: 7 */
	/* paddings: 1, sum paddings: 2 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 4 */
	/* last cacheline: 8 bytes */
} __attribute__((__aligned__(8)));
struct tcf_ct {
	struct tc_action           common;               /*     0   192 */

	/* XXX last struct has 1 byte of padding */

	/* --- cacheline 3 boundary (192 bytes) --- */
	struct tcf_ct_params *     params;               /*   192     8 */

	/* size: 200, cachelines: 4, members: 2 */
	/* paddings: 1, sum paddings: 1 */
	/* last cacheline: 8 bytes */
};
struct tcf_mpls_params {
	int                        tcfm_action;          /*     0     4 */
	u32                        tcfm_label;           /*     4     4 */
	u8                         tcfm_tc;              /*     8     1 */
	u8                         tcfm_ttl;             /*     9     1 */
	u8                         tcfm_bos;             /*    10     1 */

	/* XXX 1 byte hole, try to pack */

	__be16                     tcfm_proto;           /*    12     2 */

	/* XXX 2 bytes hole, try to pack */

	struct callback_head       rcu __attribute__((__aligned__(8))); /*    16    16 */

	/* size: 32, cachelines: 1, members: 7 */
	/* sum members: 29, holes: 2, sum holes: 3 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 2 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct tcf_mpls {
	struct tc_action           common;               /*     0   192 */

	/* XXX last struct has 1 byte of padding */

	/* --- cacheline 3 boundary (192 bytes) --- */
	struct tcf_mpls_params *   mpls_p;               /*   192     8 */

	/* size: 200, cachelines: 4, members: 2 */
	/* paddings: 1, sum paddings: 1 */
	/* last cacheline: 8 bytes */
};
struct tcfg_gate_entry {
	int                        index;                /*     0     4 */
	u8                         gate_state;           /*     4     1 */

	/* XXX 3 bytes hole, try to pack */

	u32                        interval;             /*     8     4 */
	s32                        ipv;                  /*    12     4 */
	s32                        maxoctets;            /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           list;                 /*    24    16 */

	/* size: 40, cachelines: 1, members: 6 */
	/* sum members: 33, holes: 2, sum holes: 7 */
	/* last cacheline: 40 bytes */
};
struct tcf_gate_params {
	s32                        tcfg_priority;        /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	u64                        tcfg_basetime;        /*     8     8 */
	u64                        tcfg_cycletime;       /*    16     8 */
	u64                        tcfg_cycletime_ext;   /*    24     8 */
	u32                        tcfg_flags;           /*    32     4 */
	s32                        tcfg_clockid;         /*    36     4 */
	size_t                     num_entries;          /*    40     8 */
	struct list_head           entries;              /*    48    16 */

	/* size: 64, cachelines: 1, members: 8 */
	/* sum members: 60, holes: 1, sum holes: 4 */
};
struct tcf_gate {
	struct tc_action           common;               /*     0   192 */

	/* XXX last struct has 1 byte of padding */

	/* --- cacheline 3 boundary (192 bytes) --- */
	struct tcf_gate_params     param;                /*   192    64 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	u8                         current_gate_status;  /*   256     1 */

	/* XXX 7 bytes hole, try to pack */

	ktime_t                    current_close_time;   /*   264     8 */
	u32                        current_entry_octets; /*   272     4 */
	s32                        current_max_octets;   /*   276     4 */
	struct tcfg_gate_entry *   next_entry;           /*   280     8 */
	struct hrtimer             hitimer __attribute__((__aligned__(8))); /*   288    96 */
	/* --- cacheline 6 boundary (384 bytes) --- */
	enum tk_offsets            tk_offset;            /*   384     4 */

	/* size: 392, cachelines: 7, members: 9 */
	/* sum members: 381, holes: 1, sum holes: 7 */
	/* padding: 4 */
	/* paddings: 1, sum paddings: 1 */
	/* forced alignments: 1 */
	/* last cacheline: 8 bytes */
} __attribute__((__aligned__(8)));
struct tcf_filter_chain_list_item {
	struct list_head           list;                 /*     0    16 */
	tcf_chain_head_change_t *  chain_head_change;    /*    16     8 */
	void *                     chain_head_change_priv; /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* last cacheline: 32 bytes */
};
struct tcf_net {
	spinlock_t                 idr_lock;             /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct idr                 idr;                  /*     8    24 */

	/* size: 32, cachelines: 1, members: 2 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct tcf_block_owner_item {
	struct list_head           list;                 /*     0    16 */
	struct Qdisc *             q;                    /*    16     8 */
	enum flow_block_binder_type binder_type;         /*    24     4 */

	/* size: 32, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct tcf_chain_info {
	struct tcf_proto * *       pprev;                /*     0     8 */
	struct tcf_proto *         next;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct tcf_dump_args {
	struct tcf_walker          w;                    /*     0    32 */
	struct sk_buff *           skb;                  /*    32     8 */
	struct netlink_callback *  cb;                   /*    40     8 */
	struct tcf_block *         block;                /*    48     8 */
	struct Qdisc *             q;                    /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u32                        parent;               /*    64     4 */
	bool                       terse_dump;           /*    68     1 */

	/* size: 72, cachelines: 2, members: 7 */
	/* padding: 3 */
	/* last cacheline: 8 bytes */
};
struct tcamsg {
	unsigned char              tca_family;           /*     0     1 */
	unsigned char              tca__pad1;            /*     1     1 */
	short unsigned int         tca__pad2;            /*     2     2 */

	/* size: 4, cachelines: 1, members: 3 */
	/* last cacheline: 4 bytes */
};
struct tc_action_net {
	struct tcf_idrinfo *       idrinfo;              /*     0     8 */
	const struct tc_action_ops  * ops;               /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct tc_fifo_qopt {
	__u32                      limit;                /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct tc_fifo_qopt_offload {
	enum tc_fifo_command       command;              /*     0     4 */
	u32                        handle;               /*     4     4 */
	u32                        parent;               /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	union {
		struct tc_qopt_offload_stats stats;      /*    16    16 */
	};                                               /*    16    16 */

	/* size: 32, cachelines: 1, members: 4 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct tc_fq_codel_qd_stats {
	__u32                      maxpacket;            /*     0     4 */
	__u32                      drop_overlimit;       /*     4     4 */
	__u32                      ecn_mark;             /*     8     4 */
	__u32                      new_flow_count;       /*    12     4 */
	__u32                      new_flows_len;        /*    16     4 */
	__u32                      old_flows_len;        /*    20     4 */
	__u32                      ce_mark;              /*    24     4 */
	__u32                      memory_usage;         /*    28     4 */
	__u32                      drop_overmemory;      /*    32     4 */

	/* size: 36, cachelines: 1, members: 9 */
	/* last cacheline: 36 bytes */
};
struct tc_fq_codel_cl_stats {
	__s32                      deficit;              /*     0     4 */
	__u32                      ldelay;               /*     4     4 */
	__u32                      count;                /*     8     4 */
	__u32                      lastcount;            /*    12     4 */
	__u32                      dropping;             /*    16     4 */
	__s32                      drop_next;            /*    20     4 */

	/* size: 24, cachelines: 1, members: 6 */
	/* last cacheline: 24 bytes */
};
struct tc_fq_codel_xstats {
	__u32                      type;                 /*     0     4 */
	union {
		struct tc_fq_codel_qd_stats qdisc_stats; /*     4    36 */
		struct tc_fq_codel_cl_stats class_stats; /*     4    24 */
	};                                               /*     4    36 */

	/* size: 40, cachelines: 1, members: 2 */
	/* last cacheline: 40 bytes */
};
struct codel_params {
	codel_time_t               target;               /*     0     4 */
	codel_time_t               ce_threshold;         /*     4     4 */
	codel_time_t               interval;             /*     8     4 */
	u32                        mtu;                  /*    12     4 */
	bool                       ecn;                  /*    16     1 */

	/* size: 20, cachelines: 1, members: 5 */
	/* padding: 3 */
	/* last cacheline: 20 bytes */
};
struct codel_vars {
	u32                        count;                /*     0     4 */
	u32                        lastcount;            /*     4     4 */
	bool                       dropping;             /*     8     1 */

	/* XXX 1 byte hole, try to pack */

	u16                        rec_inv_sqrt;         /*    10     2 */
	codel_time_t               first_above_time;     /*    12     4 */
	codel_time_t               drop_next;            /*    16     4 */
	codel_time_t               ldelay;               /*    20     4 */

	/* size: 24, cachelines: 1, members: 7 */
	/* sum members: 23, holes: 1, sum holes: 1 */
	/* last cacheline: 24 bytes */
};
struct codel_stats {
	u32                        maxpacket;            /*     0     4 */
	u32                        drop_count;           /*     4     4 */
	u32                        drop_len;             /*     8     4 */
	u32                        ecn_mark;             /*    12     4 */
	u32                        ce_mark;              /*    16     4 */

	/* size: 20, cachelines: 1, members: 5 */
	/* last cacheline: 20 bytes */
};
struct codel_skb_cb {
	codel_time_t               enqueue_time;         /*     0     4 */
	unsigned int               mem_usage;            /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct fq_codel_flow {
	struct sk_buff *           head;                 /*     0     8 */
	struct sk_buff *           tail;                 /*     8     8 */
	struct list_head           flowchain;            /*    16    16 */
	int                        deficit;              /*    32     4 */
	struct codel_vars          cvars;                /*    36    24 */

	/* size: 64, cachelines: 1, members: 5 */
	/* padding: 4 */
};
struct fq_codel_sched_data {
	struct tcf_proto *         filter_list;          /*     0     8 */
	struct tcf_block *         block;                /*     8     8 */
	struct fq_codel_flow *     flows;                /*    16     8 */
	u32 *                      backlogs;             /*    24     8 */
	u32                        flows_cnt;            /*    32     4 */
	u32                        quantum;              /*    36     4 */
	u32                        drop_batch_size;      /*    40     4 */
	u32                        memory_limit;         /*    44     4 */
	struct codel_params        cparams;              /*    48    20 */

	/* XXX last struct has 3 bytes of padding */

	/* --- cacheline 1 boundary (64 bytes) was 4 bytes ago --- */
	struct codel_stats         cstats;               /*    68    20 */
	u32                        memory_usage;         /*    88     4 */
	u32                        drop_overmemory;      /*    92     4 */
	u32                        drop_overlimit;       /*    96     4 */
	u32                        new_flow_count;       /*   100     4 */
	struct list_head           new_flows;            /*   104    16 */
	struct list_head           old_flows;            /*   120    16 */

	/* size: 136, cachelines: 3, members: 16 */
	/* paddings: 1, sum paddings: 3 */
	/* last cacheline: 8 bytes */
};
struct tcf_ematch_tree_hdr {
	__u16                      nmatches;             /*     0     2 */
	__u16                      progid;               /*     2     2 */

	/* size: 4, cachelines: 1, members: 2 */
	/* last cacheline: 4 bytes */
};
struct tcf_pkt_info {
	unsigned char *            ptr;                  /*     0     8 */
	int                        nexthdr;              /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct tcf_ematch {
	struct tcf_ematch_ops *    ops;                  /*     0     8 */
	long unsigned int          data;                 /*     8     8 */
	unsigned int               datalen;              /*    16     4 */
	u16                        matchid;              /*    20     2 */
	u16                        flags;                /*    22     2 */
	struct net *               net;                  /*    24     8 */

	/* size: 32, cachelines: 1, members: 6 */
	/* last cacheline: 32 bytes */
};
struct tcf_ematch_ops {
	int                        kind;                 /*     0     4 */
	int                        datalen;              /*     4     4 */
	int                        (*change)(struct net *, void *, int, struct tcf_ematch *); /*     8     8 */
	int                        (*match)(struct sk_buff *, struct tcf_ematch *, struct tcf_pkt_info *); /*    16     8 */
	void                       (*destroy)(struct tcf_ematch *); /*    24     8 */
	int                        (*dump)(struct sk_buff *, struct tcf_ematch *); /*    32     8 */
	struct module *            owner;                /*    40     8 */
	struct list_head           link;                 /*    48    16 */

	/* size: 64, cachelines: 1, members: 8 */
};
struct tcf_ematch_tree {
	struct tcf_ematch_tree_hdr hdr;                  /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct tcf_ematch *        matches;              /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct cls_cgroup_head {
	u32                        handle;               /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct tcf_exts            exts;                 /*     8    32 */
	struct tcf_ematch_tree     ematches;             /*    40    16 */
	struct tcf_proto *         tp;                   /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct rcu_work            rwork __attribute__((__aligned__(8))); /*    64    88 */

	/* size: 152, cachelines: 3, members: 5 */
	/* sum members: 148, holes: 1, sum holes: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct tcf_ematch_hdr {
	__u16                      matchid;              /*     0     2 */
	__u16                      kind;                 /*     2     2 */
	__u16                      flags;                /*     4     2 */
	__u16                      pad;                  /*     6     2 */

	/* size: 8, cachelines: 1, members: 4 */
	/* last cacheline: 8 bytes */
};
struct sockaddr_nl {
	__kernel_sa_family_t       nl_family;            /*     0     2 */
	short unsigned int         nl_pad;               /*     2     2 */
	__u32                      nl_pid;               /*     4     4 */
	__u32                      nl_groups;            /*     8     4 */

	/* size: 12, cachelines: 1, members: 4 */
	/* last cacheline: 12 bytes */
};
struct nlmsgerr {
	int                        error;                /*     0     4 */
	struct nlmsghdr            msg;                  /*     4    16 */

	/* size: 20, cachelines: 1, members: 2 */
	/* last cacheline: 20 bytes */
};
struct nl_pktinfo {
	__u32                      group;                /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct netlink_notify {
	struct net *               net;                  /*     0     8 */
	u32                        portid;               /*     8     4 */
	int                        protocol;             /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct netlink_tap {
	struct net_device *        dev;                  /*     0     8 */
	struct module *            module;               /*     8     8 */
	struct list_head           list;                 /*    16    16 */

	/* size: 32, cachelines: 1, members: 3 */
	/* last cacheline: 32 bytes */
};
struct netlink_sock {
	struct sock                sk __attribute__((__aligned__(8))); /*     0   872 */
	/* --- cacheline 13 boundary (832 bytes) was 40 bytes ago --- */
	u32                        portid;               /*   872     4 */
	u32                        dst_portid;           /*   876     4 */
	u32                        dst_group;            /*   880     4 */
	u32                        flags;                /*   884     4 */
	u32                        subscriptions;        /*   888     4 */
	u32                        ngroups;              /*   892     4 */
	/* --- cacheline 14 boundary (896 bytes) --- */
	long unsigned int *        groups;               /*   896     8 */
	long unsigned int          state;                /*   904     8 */
	size_t                     max_recvmsg_len;      /*   912     8 */
	wait_queue_head_t          wait;                 /*   920    24 */
	bool                       bound;                /*   944     1 */
	bool                       cb_running;           /*   945     1 */

	/* XXX 2 bytes hole, try to pack */

	int                        dump_done_errno;      /*   948     4 */
	struct netlink_callback    cb;                   /*   952   128 */
	/* --- cacheline 16 boundary (1024 bytes) was 56 bytes ago --- */
	struct mutex *             cb_mutex;             /*  1080     8 */
	/* --- cacheline 17 boundary (1088 bytes) --- */
	struct mutex               cb_def_mutex;         /*  1088    32 */
	void                       (*netlink_rcv)(struct sk_buff *); /*  1120     8 */
	int                        (*netlink_bind)(struct net *, int); /*  1128     8 */
	void                       (*netlink_unbind)(struct net *, int); /*  1136     8 */
	struct module *            module;               /*  1144     8 */
	/* --- cacheline 18 boundary (1152 bytes) --- */
	struct rhash_head          node;                 /*  1152     8 */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*  1160    16 */
	struct work_struct         work;                 /*  1176    64 */

	/* size: 1240, cachelines: 20, members: 24 */
	/* sum members: 1238, holes: 1, sum holes: 2 */
	/* forced alignments: 2 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct netlink_table {
	struct rhashtable          hash;                 /*     0   168 */
	/* --- cacheline 2 boundary (128 bytes) was 40 bytes ago --- */
	struct hlist_head          mc_list;              /*   168     8 */
	struct listeners *         listeners;            /*   176     8 */
	unsigned int               flags;                /*   184     4 */
	unsigned int               groups;               /*   188     4 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	struct mutex *             cb_mutex;             /*   192     8 */
	struct module *            module;               /*   200     8 */
	int                        (*bind)(struct net *, int); /*   208     8 */
	void                       (*unbind)(struct net *, int); /*   216     8 */
	bool                       (*compare)(struct net *, struct sock *); /*   224     8 */
	int                        registered;           /*   232     4 */

	/* size: 240, cachelines: 4, members: 11 */
	/* padding: 4 */
	/* last cacheline: 48 bytes */
};
struct listeners {
	struct callback_head       rcu __attribute__((__aligned__(8))); /*     0    16 */
	long unsigned int          masks[];              /*    16     0 */

	/* size: 16, cachelines: 1, members: 2 */
	/* forced alignments: 1 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct netlink_tap_net {
	struct list_head           netlink_tap_all;      /*     0    16 */
	struct mutex               netlink_tap_lock;     /*    16    32 */

	/* size: 48, cachelines: 1, members: 2 */
	/* last cacheline: 48 bytes */
};
struct netlink_compare_arg {
	possible_net_t             pnet;                 /*     0     8 */
	u32                        portid;               /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct netlink_broadcast_data {
	struct sock *              exclude_sk;           /*     0     8 */
	struct net *               net;                  /*     8     8 */
	u32                        portid;               /*    16     4 */
	u32                        group;                /*    20     4 */
	int                        failure;              /*    24     4 */
	int                        delivery_failure;     /*    28     4 */
	int                        congested;            /*    32     4 */
	int                        delivered;            /*    36     4 */
	gfp_t                      allocation;           /*    40     4 */

	/* XXX 4 bytes hole, try to pack */

	struct sk_buff *           skb;                  /*    48     8 */
	struct sk_buff *           skb2;                 /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*tx_filter)(struct sock *, struct sk_buff *, void *); /*    64     8 */
	void *                     tx_data;              /*    72     8 */

	/* size: 80, cachelines: 2, members: 13 */
	/* sum members: 76, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct netlink_set_err_data {
	struct sock *              exclude_sk;           /*     0     8 */
	u32                        portid;               /*     8     4 */
	u32                        group;                /*    12     4 */
	int                        code;                 /*    16     4 */

	/* size: 24, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct nl_seq_iter {
	struct seq_net_private     p;                    /*     0     8 */
	struct rhashtable_iter     hti;                  /*     8    64 */

	/* XXX last struct has 7 bytes of padding */

	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	int                        link;                 /*    72     4 */

	/* size: 80, cachelines: 2, members: 3 */
	/* padding: 4 */
	/* paddings: 1, sum paddings: 7 */
	/* last cacheline: 16 bytes */
};
struct bpf_iter__netlink {
	union {
		struct bpf_iter_meta * meta;             /*     0     8 */
	} __attribute__((__aligned__(8)));               /*     0     8 */
	union {
		struct netlink_sock * sk;                /*     8     8 */
	} __attribute__((__aligned__(8)));               /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* forced alignments: 2 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct genl_start_context {
	const struct genl_family  * family;              /*     0     8 */
	struct nlmsghdr *          nlh;                  /*     8     8 */
	struct netlink_ext_ack *   extack;               /*    16     8 */
	const struct genl_ops  *   ops;                  /*    24     8 */
	int                        hdrlen;               /*    32     4 */

	/* size: 40, cachelines: 1, members: 5 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct ctrl_dump_policy_ctx {
	struct netlink_policy_dump_state * state;        /*     0     8 */
	const struct genl_family  * rt;                  /*     8     8 */
	unsigned int               opidx;                /*    16     4 */
	u32                        op;                   /*    20     4 */
	u16                        fam_id;               /*    24     2 */
	u8                         policies:1;           /*    26: 0  1 */
	u8                         single_op:1;          /*    26: 1  1 */

	/* size: 32, cachelines: 1, members: 7 */
	/* padding: 5 */
	/* bit_padding: 6 bits */
	/* last cacheline: 32 bytes */
};
struct netlink_policy_dump_state {
	unsigned int               policy_idx;           /*     0     4 */
	unsigned int               attr_idx;             /*     4     4 */
	unsigned int               n_alloc;              /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct {
		const struct nla_policy  * policy;       /*    16     8 */
		unsigned int       maxtype;              /*    24     4 */
	} policies[]; /*    16     0 */

	/* size: 16, cachelines: 1, members: 4 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct trace_event_raw_bpf_test_finish {
	struct trace_entry         ent;                  /*     0     8 */
	int                        err;                  /*     8     4 */
	char                       __data[];             /*    12     0 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct trace_event_data_offsets_bpf_test_finish {

	/* size: 0, cachelines: 0, members: 0 */
};
struct bpf_test_timer {
	enum {
		NO_PREEMPT = 0,
		NO_MIGRATE = 1,
	} mode;                                          /*     0     4 */
	u32                        i;                    /*     4     4 */
	u64                        time_start;           /*     8     8 */
	u64                        time_spent;           /*    16     8 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct bpf_fentry_test_t {
	struct bpf_fentry_test_t * a;                    /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct bpf_raw_tp_test_run_info {
	struct bpf_prog *          prog;                 /*     0     8 */
	void *                     ctx;                  /*     8     8 */
	u32                        retval;               /*    16     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct ethtool_cmd {
	__u32                      cmd;                  /*     0     4 */
	__u32                      supported;            /*     4     4 */
	__u32                      advertising;          /*     8     4 */
	__u16                      speed;                /*    12     2 */
	__u8                       duplex;               /*    14     1 */
	__u8                       port;                 /*    15     1 */
	__u8                       phy_address;          /*    16     1 */
	__u8                       transceiver;          /*    17     1 */
	__u8                       autoneg;              /*    18     1 */
	__u8                       mdio_support;         /*    19     1 */
	__u32                      maxtxpkt;             /*    20     4 */
	__u32                      maxrxpkt;             /*    24     4 */
	__u16                      speed_hi;             /*    28     2 */
	__u8                       eth_tp_mdix;          /*    30     1 */
	__u8                       eth_tp_mdix_ctrl;     /*    31     1 */
	__u32                      lp_advertising;       /*    32     4 */
	__u32                      reserved[2];          /*    36     8 */

	/* size: 44, cachelines: 1, members: 17 */
	/* last cacheline: 44 bytes */
};
struct ethtool_value {
	__u32                      cmd;                  /*     0     4 */
	__u32                      data;                 /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct ethtool_gstrings {
	__u32                      cmd;                  /*     0     4 */
	__u32                      string_set;           /*     4     4 */
	__u32                      len;                  /*     8     4 */
	__u8                       data[];               /*    12     0 */

	/* size: 12, cachelines: 1, members: 4 */
	/* last cacheline: 12 bytes */
};
struct ethtool_sset_info {
	__u32                      cmd;                  /*     0     4 */
	__u32                      reserved;             /*     4     4 */
	__u64                      sset_mask;            /*     8     8 */
	__u32                      data[];               /*    16     0 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct ethtool_perm_addr {
	__u32                      cmd;                  /*     0     4 */
	__u32                      size;                 /*     4     4 */
	__u8                       data[];               /*     8     0 */

	/* size: 8, cachelines: 1, members: 3 */
	/* last cacheline: 8 bytes */
};
struct ethtool_rxfh {
	__u32                      cmd;                  /*     0     4 */
	__u32                      rss_context;          /*     4     4 */
	__u32                      indir_size;           /*     8     4 */
	__u32                      key_size;             /*    12     4 */
	__u8                       hfunc;                /*    16     1 */
	__u8                       rsvd8[3];             /*    17     3 */
	__u32                      rsvd32;               /*    20     4 */
	__u32                      rss_config[];         /*    24     0 */

	/* size: 24, cachelines: 1, members: 8 */
	/* last cacheline: 24 bytes */
};
struct ethtool_get_features_block {
	__u32                      available;            /*     0     4 */
	__u32                      requested;            /*     4     4 */
	__u32                      active;               /*     8     4 */
	__u32                      never_changed;        /*    12     4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct ethtool_gfeatures {
	__u32                      cmd;                  /*     0     4 */
	__u32                      size;                 /*     4     4 */
	struct ethtool_get_features_block features[];    /*     8     0 */

	/* size: 8, cachelines: 1, members: 3 */
	/* last cacheline: 8 bytes */
};
struct ethtool_set_features_block {
	__u32                      valid;                /*     0     4 */
	__u32                      requested;            /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct ethtool_sfeatures {
	__u32                      cmd;                  /*     0     4 */
	__u32                      size;                 /*     4     4 */
	struct ethtool_set_features_block features[];    /*     8     0 */

	/* size: 8, cachelines: 1, members: 3 */
	/* last cacheline: 8 bytes */
};
struct ethtool_per_queue_op {
	__u32                      cmd;                  /*     0     4 */
	__u32                      sub_command;          /*     4     4 */
	__u32                      queue_mask[128];      /*     8   512 */
	/* --- cacheline 8 boundary (512 bytes) was 8 bytes ago --- */
	char                       data[];               /*   520     0 */

	/* size: 520, cachelines: 9, members: 4 */
	/* last cacheline: 8 bytes */
};
struct compat_ethtool_rx_flow_spec {
	u32                        flow_type;            /*     0     4 */
	union ethtool_flow_union   h_u;                  /*     4    52 */
	struct ethtool_flow_ext    h_ext;                /*    56    20 */
	/* --- cacheline 1 boundary (64 bytes) was 12 bytes ago --- */
	union ethtool_flow_union   m_u;                  /*    76    52 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct ethtool_flow_ext    m_ext;                /*   128    20 */
	compat_u64                 ring_cookie __attribute__((__aligned__(4))); /*   148     8 */
	u32                        location;             /*   156     4 */

	/* size: 160, cachelines: 3, members: 7 */
	/* forced alignments: 1 */
	/* last cacheline: 32 bytes */
} __attribute__((__packed__));
struct compat_ethtool_rxnfc {
	u32                        cmd;                  /*     0     4 */
	u32                        flow_type;            /*     4     4 */
	compat_u64                 data __attribute__((__aligned__(4))); /*     8     8 */
	struct compat_ethtool_rx_flow_spec fs __attribute__((__aligned__(4))); /*    16   160 */
	/* --- cacheline 2 boundary (128 bytes) was 48 bytes ago --- */
	u32                        rule_cnt;             /*   176     4 */
	u32                        rule_locs[];          /*   180     0 */

	/* size: 180, cachelines: 3, members: 6 */
	/* forced alignments: 2 */
	/* last cacheline: 52 bytes */
} __attribute__((__packed__));
struct ethtool_rx_flow_rule {
	struct flow_rule *         rule;                 /*     0     8 */
	long unsigned int          priv[];               /*     8     0 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct ethtool_rx_flow_spec_input {
	const struct ethtool_rx_flow_spec  * fs;         /*     0     8 */
	u32                        rss_ctx;              /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct ethtool_link_usettings {
	struct ethtool_link_settings base;               /*     0    48 */
	struct {
		__u32              supported[3];         /*    48    12 */
		__u32              advertising[3];       /*    60    12 */
		/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
		__u32              lp_advertising[3];    /*    72    12 */
	} link_modes;                                    /*    48    36 */

	/* size: 84, cachelines: 2, members: 2 */
	/* last cacheline: 20 bytes */
};
struct ethtool_rx_flow_key {
	struct flow_dissector_key_basic basic;           /*     0     4 */
	union {
		struct flow_dissector_key_ipv4_addrs ipv4; /*     4     8 */
		struct flow_dissector_key_ipv6_addrs ipv6; /*     4    32 */
	};                                               /*     4    32 */
	struct flow_dissector_key_ports tp;              /*    36     4 */
	struct flow_dissector_key_ip ip;                 /*    40     2 */
	struct flow_dissector_key_vlan vlan;             /*    42     4 */
	struct flow_dissector_key_eth_addrs eth_addrs;   /*    46    12 */

	/* size: 64, cachelines: 1, members: 6 */
	/* padding: 6 */
} __attribute__((__aligned__(8)));
struct ethtool_rx_flow_match {
	struct flow_dissector      dissector;            /*     0    60 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	struct ethtool_rx_flow_key key __attribute__((__aligned__(8))); /*    64    64 */

	/* XXX last struct has 6 bytes of padding */

	/* --- cacheline 2 boundary (128 bytes) --- */
	struct ethtool_rx_flow_key mask __attribute__((__aligned__(8))); /*   128    64 */

	/* XXX last struct has 6 bytes of padding */

	/* size: 192, cachelines: 3, members: 3 */
	/* sum members: 188, holes: 1, sum holes: 4 */
	/* paddings: 2, sum paddings: 12 */
	/* forced alignments: 2, forced holes: 1, sum forced holes: 4 */
} __attribute__((__aligned__(8)));
struct ethnl_req_info {
	struct net_device *        dev;                  /*     0     8 */
	u32                        flags;                /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct ethnl_reply_data {
	struct net_device *        dev;                  /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct ethnl_request_ops {
	u8                         request_cmd;          /*     0     1 */
	u8                         reply_cmd;            /*     1     1 */
	u16                        hdr_attr;             /*     2     2 */
	unsigned int               req_info_size;        /*     4     4 */
	unsigned int               reply_data_size;      /*     8     4 */
	bool                       allow_nodev_do;       /*    12     1 */

	/* XXX 3 bytes hole, try to pack */

	int                        (*parse_request)(struct ethnl_req_info *, struct nlattr * *, struct netlink_ext_ack *); /*    16     8 */
	int                        (*prepare_data)(const struct ethnl_req_info  *, struct ethnl_reply_data *, struct genl_info *); /*    24     8 */
	int                        (*reply_size)(const struct ethnl_req_info  *, const struct ethnl_reply_data  *); /*    32     8 */
	int                        (*fill_reply)(struct sk_buff *, const struct ethnl_req_info  *, const struct ethnl_reply_data  *); /*    40     8 */
	void                       (*cleanup_data)(struct ethnl_reply_data *); /*    48     8 */

	/* size: 56, cachelines: 1, members: 11 */
	/* sum members: 53, holes: 1, sum holes: 3 */
	/* last cacheline: 56 bytes */
};
struct ethnl_dump_ctx {
	const struct ethnl_request_ops  * ops;           /*     0     8 */
	struct ethnl_req_info *    req_info;             /*     8     8 */
	struct ethnl_reply_data *  reply_data;           /*    16     8 */
	int                        pos_hash;             /*    24     4 */
	int                        pos_idx;              /*    28     4 */

	/* size: 32, cachelines: 1, members: 5 */
	/* last cacheline: 32 bytes */
};
struct strset_info {
	bool                       per_dev;              /*     0     1 */
	bool                       free_strings;         /*     1     1 */

	/* XXX 2 bytes hole, try to pack */

	unsigned int               count;                /*     4     4 */
	const char  *              strings;              /*     8     8 */

	/* size: 16, cachelines: 1, members: 4 */
	/* sum members: 14, holes: 1, sum holes: 2 */
	/* last cacheline: 16 bytes */
};
struct strset_req_info {
	struct ethnl_req_info      base;                 /*     0    16 */

	/* XXX last struct has 4 bytes of padding */

	u32                        req_ids;              /*    16     4 */
	bool                       counts_only;          /*    20     1 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 3 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 24 bytes */
};
struct strset_reply_data {
	struct ethnl_reply_data    base;                 /*     0     8 */
	struct strset_info         sets[16];             /*     8   256 */

	/* size: 264, cachelines: 5, members: 2 */
	/* last cacheline: 8 bytes */
};
struct linkinfo_reply_data {
	struct ethnl_reply_data    base;                 /*     0     8 */
	struct ethtool_link_ksettings ksettings;         /*     8    96 */
	/* --- cacheline 1 boundary (64 bytes) was 40 bytes ago --- */
	struct ethtool_link_settings * lsettings;        /*   104     8 */

	/* size: 112, cachelines: 2, members: 3 */
	/* last cacheline: 48 bytes */
};
struct linkmodes_reply_data {
	struct ethnl_reply_data    base;                 /*     0     8 */
	struct ethtool_link_ksettings ksettings;         /*     8    96 */
	/* --- cacheline 1 boundary (64 bytes) was 40 bytes ago --- */
	struct ethtool_link_settings * lsettings;        /*   104     8 */
	bool                       peer_empty;           /*   112     1 */

	/* size: 120, cachelines: 2, members: 4 */
	/* padding: 7 */
	/* last cacheline: 56 bytes */
};
struct link_mode_info {
	int                        speed;                /*     0     4 */
	u8                         duplex;               /*     4     1 */

	/* size: 8, cachelines: 1, members: 2 */
	/* padding: 3 */
	/* last cacheline: 8 bytes */
};
struct linkstate_reply_data {
	struct ethnl_reply_data    base;                 /*     0     8 */
	int                        link;                 /*     8     4 */
	int                        sqi;                  /*    12     4 */
	int                        sqi_max;              /*    16     4 */
	bool                       link_ext_state_provided; /*    20     1 */

	/* XXX 3 bytes hole, try to pack */

	struct ethtool_link_ext_state_info ethtool_link_ext_state_info; /*    24     8 */

	/* size: 32, cachelines: 1, members: 6 */
	/* sum members: 29, holes: 1, sum holes: 3 */
	/* last cacheline: 32 bytes */
};
struct debug_reply_data {
	struct ethnl_reply_data    base;                 /*     0     8 */
	u32                        msg_mask;             /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct wol_reply_data {
	struct ethnl_reply_data    base;                 /*     0     8 */
	struct ethtool_wolinfo     wol;                  /*     8    20 */

	/* XXX last struct has 2 bytes of padding */

	bool                       show_sopass;          /*    28     1 */

	/* size: 32, cachelines: 1, members: 3 */
	/* padding: 3 */
	/* paddings: 1, sum paddings: 2 */
	/* last cacheline: 32 bytes */
};
struct features_reply_data {
	struct ethnl_reply_data    base;                 /*     0     8 */
	u32                        hw[2];                /*     8     8 */
	u32                        wanted[2];            /*    16     8 */
	u32                        active[2];            /*    24     8 */
	u32                        nochange[2];          /*    32     8 */
	u32                        all[2];               /*    40     8 */

	/* size: 48, cachelines: 1, members: 6 */
	/* last cacheline: 48 bytes */
};
struct privflags_reply_data {
	struct ethnl_reply_data    base;                 /*     0     8 */
	const char  *              priv_flag_names;      /*     8     8 */
	unsigned int               n_priv_flags;         /*    16     4 */
	u32                        priv_flags;           /*    20     4 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct rings_reply_data {
	struct ethnl_reply_data    base;                 /*     0     8 */
	struct ethtool_ringparam   ringparam;            /*     8    36 */
	struct kernel_ethtool_ringparam kernel_ringparam; /*    44     4 */

	/* size: 48, cachelines: 1, members: 3 */
	/* last cacheline: 48 bytes */
};
struct channels_reply_data {
	struct ethnl_reply_data    base;                 /*     0     8 */
	struct ethtool_channels    channels;             /*     8    36 */

	/* size: 48, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 48 bytes */
};
struct coalesce_reply_data {
	struct ethnl_reply_data    base;                 /*     0     8 */
	struct ethtool_coalesce    coalesce;             /*     8    92 */
	/* --- cacheline 1 boundary (64 bytes) was 36 bytes ago --- */
	struct kernel_ethtool_coalesce kernel_coalesce;  /*   100     2 */

	/* XXX 2 bytes hole, try to pack */

	u32                        supported_params;     /*   104     4 */

	/* size: 112, cachelines: 2, members: 4 */
	/* sum members: 106, holes: 1, sum holes: 2 */
	/* padding: 4 */
	/* last cacheline: 48 bytes */
};
struct pause_reply_data {
	struct ethnl_reply_data    base;                 /*     0     8 */
	struct ethtool_pauseparam  pauseparam;           /*     8    16 */
	struct ethtool_pause_stats pausestat;            /*    24    16 */

	/* size: 40, cachelines: 1, members: 3 */
	/* last cacheline: 40 bytes */
};
struct eee_reply_data {
	struct ethnl_reply_data    base;                 /*     0     8 */
	struct ethtool_eee         eee;                  /*     8    40 */

	/* size: 48, cachelines: 1, members: 2 */
	/* last cacheline: 48 bytes */
};
struct tsinfo_reply_data {
	struct ethnl_reply_data    base;                 /*     0     8 */
	struct ethtool_ts_info     ts_info;              /*     8    44 */

	/* size: 56, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 56 bytes */
};
struct udp_tunnel_nic_ops {
	void                       (*get_port)(struct net_device *, unsigned int, unsigned int, struct udp_tunnel_info *); /*     0     8 */
	void                       (*set_port_priv)(struct net_device *, unsigned int, unsigned int, u8); /*     8     8 */
	void                       (*add_port)(struct net_device *, struct udp_tunnel_info *); /*    16     8 */
	void                       (*del_port)(struct net_device *, struct udp_tunnel_info *); /*    24     8 */
	void                       (*reset_ntf)(struct net_device *); /*    32     8 */
	size_t                     (*dump_size)(struct net_device *, unsigned int); /*    40     8 */
	int                        (*dump_write)(struct net_device *, unsigned int, struct sk_buff *); /*    48     8 */

	/* size: 56, cachelines: 1, members: 7 */
	/* last cacheline: 56 bytes */
};
struct ethnl_tunnel_info_dump_ctx {
	struct ethnl_req_info      req_info;             /*     0    16 */

	/* XXX last struct has 4 bytes of padding */

	int                        pos_hash;             /*    16     4 */
	int                        pos_idx;              /*    20     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 24 bytes */
};
struct nf_queue_handler {
	int                        (*outfn)(struct nf_queue_entry *, unsigned int); /*     0     8 */
	void                       (*nf_hook_drop)(struct net *); /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct nf_hook_entries_rcu_head {
	struct callback_head       head __attribute__((__aligned__(8))); /*     0    16 */
	void *                     allocation;           /*    16     8 */

	/* size: 24, cachelines: 1, members: 2 */
	/* forced alignments: 1 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct nf_nat_hook {
	int                        (*parse_nat_setup)(struct nf_conn *, enum nf_nat_manip_type, const struct nlattr  *); /*     0     8 */
	void                       (*decode_session)(struct sk_buff *, struct flowi *); /*     8     8 */
	unsigned int               (*manip_pkt)(struct sk_buff *, struct nf_conn *, enum nf_nat_manip_type, enum ip_conntrack_dir); /*    16     8 */
	u64                        kabi_reserved1;       /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct nf_ct_hook {
	int                        (*update)(struct net *, struct sk_buff *); /*     0     8 */
	void                       (*destroy)(struct nf_conntrack *); /*     8     8 */
	bool                       (*get_tuple_skb)(struct nf_conntrack_tuple *, const struct sk_buff  *); /*    16     8 */
	u64                        kabi_reserved1;       /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct nfnl_ct_hook {
	struct nf_conn *           (*get_ct)(const struct sk_buff  *, enum ip_conntrack_info *); /*     0     8 */
	size_t                     (*build_size)(const struct nf_conn  *); /*     8     8 */
	int                        (*build)(struct sk_buff *, struct nf_conn *, enum ip_conntrack_info, u_int16_t, u_int16_t); /*    16     8 */
	int                        (*parse)(const struct nlattr  *, struct nf_conn *); /*    24     8 */
	int                        (*attach_expect)(const struct nlattr  *, struct nf_conn *, u32, u32); /*    32     8 */
	void                       (*seq_adjust)(struct sk_buff *, struct nf_conn *, enum ip_conntrack_info, s32); /*    40     8 */
	u64                        kabi_reserved1;       /*    48     8 */

	/* size: 56, cachelines: 1, members: 7 */
	/* last cacheline: 56 bytes */
};
struct nf_ipv6_ops {
	void                       (*route_input)(struct sk_buff *); /*     0     8 */
	int                        (*fragment)(struct net *, struct sock *, struct sk_buff *, int (*)(struct net *, struct sock *, struct sk_buff *)); /*     8     8 */
	int                        (*reroute)(struct sk_buff *, const struct nf_queue_entry  *); /*    16     8 */
	u64                        kabi_reserved1;       /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct nf_queue_entry {
	struct list_head           list;                 /*     0    16 */
	struct sk_buff *           skb;                  /*    16     8 */
	unsigned int               id;                   /*    24     4 */
	unsigned int               hook_index;           /*    28     4 */
	struct net_device *        physin;               /*    32     8 */
	struct net_device *        physout;              /*    40     8 */
	struct nf_hook_state       state;                /*    48    48 */
	/* --- cacheline 1 boundary (64 bytes) was 32 bytes ago --- */
	u16                        size;                 /*    96     2 */

	/* size: 104, cachelines: 2, members: 8 */
	/* padding: 6 */
	/* last cacheline: 40 bytes */
};
struct nf_logger {
	char *                     name;                 /*     0     8 */
	enum nf_log_type           type;                 /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	nf_logfn *                 logfn;                /*    16     8 */
	struct module *            me;                   /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct nf_loginfo {
	u_int8_t                   type;                 /*     0     1 */

	/* XXX 3 bytes hole, try to pack */

	union {
		struct {
			u_int32_t  copy_len;             /*     4     4 */
			u_int16_t  group;                /*     8     2 */
			u_int16_t  qthreshold;           /*    10     2 */
			u_int16_t  flags;                /*    12     2 */
		} ulog;                                  /*     4    12 */
		struct {
			u_int8_t   level;                /*     4     1 */
			u_int8_t   logflags;             /*     5     1 */
		} log;                                   /*     4     2 */
	} u;                                             /*     4    12 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 13, holes: 1, sum holes: 3 */
	/* last cacheline: 16 bytes */
};
struct nf_log_buf {
	unsigned int               count;                /*     0     4 */
	char                       buf[1020];            /*     4  1020 */

	/* size: 1024, cachelines: 16, members: 2 */
};
struct nf_bridge_info {
	enum {
		BRNF_PROTO_UNCHANGED = 0,
		BRNF_PROTO_8021Q     = 1,
		BRNF_PROTO_PPPOE     = 2,
	} orig_proto:8;                                    /*     0: 0  4 */

	/* Bitfield combined with next fields */

	u8                         pkt_otherhost:1;      /*     1: 0  1 */
	u8                         in_prerouting:1;      /*     1: 1  1 */
	u8                         bridged_dnat:1;       /*     1: 2  1 */

	/* XXX 5 bits hole, try to pack */

	__u16                      frag_max_size;        /*     2     2 */

	/* XXX 4 bytes hole, try to pack */

	struct net_device *        physindev;            /*     8     8 */
	struct net_device *        physoutdev;           /*    16     8 */
	union {
		__be32             ipv4_daddr;           /*    24     4 */
		struct in6_addr    ipv6_daddr;           /*    24    16 */
		char               neigh_header[8];      /*    24     8 */
	};                                               /*    24    16 */

	/* size: 40, cachelines: 1, members: 8 */
	/* sum members: 34, holes: 1, sum holes: 4 */
	/* sum bitfield members: 11 bits, bit holes: 1, sum bit holes: 5 bits */
	/* last cacheline: 40 bytes */
};
struct ip_rt_info {
	__be32                     daddr;                /*     0     4 */
	__be32                     saddr;                /*     4     4 */
	u_int8_t                   tos;                  /*     8     1 */

	/* XXX 3 bytes hole, try to pack */

	u_int32_t                  mark;                 /*    12     4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* sum members: 13, holes: 1, sum holes: 3 */
	/* last cacheline: 16 bytes */
};
struct ip6_rt_info {
	struct in6_addr            daddr;                /*     0    16 */
	struct in6_addr            saddr;                /*    16    16 */
	u_int32_t                  mark;                 /*    32     4 */

	/* size: 36, cachelines: 1, members: 3 */
	/* last cacheline: 36 bytes */
};
struct nf_sockopt_ops {
	struct list_head           list;                 /*     0    16 */
	u_int8_t                   pf;                   /*    16     1 */

	/* XXX 3 bytes hole, try to pack */

	int                        set_optmin;           /*    20     4 */
	int                        set_optmax;           /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	int                        (*set)(struct sock *, int, sockptr_t, unsigned int); /*    32     8 */
	int                        get_optmin;           /*    40     4 */
	int                        get_optmax;           /*    44     4 */
	int                        (*get)(struct sock *, int, void *, int *); /*    48     8 */
	struct module *            owner;                /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u64                        kabi_reserved1;       /*    64     8 */

	/* size: 72, cachelines: 2, members: 10 */
	/* sum members: 65, holes: 2, sum holes: 7 */
	/* last cacheline: 8 bytes */
};
struct xt_match {
	struct list_head           list;                 /*     0    16 */
	const const char           name[29];             /*    16    29 */
	u_int8_t                   revision;             /*    45     1 */

	/* XXX 2 bytes hole, try to pack */

	bool                       (*match)(const struct sk_buff  *, struct xt_action_param *); /*    48     8 */
	int                        (*checkentry)(const struct xt_mtchk_param  *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	void                       (*destroy)(const struct xt_mtdtor_param  *); /*    64     8 */
	void                       (*compat_from_user)(void *, const void  *); /*    72     8 */
	int                        (*compat_to_user)(void *, const void  *); /*    80     8 */
	struct module *            me;                   /*    88     8 */
	const char  *              table;                /*    96     8 */
	unsigned int               matchsize;            /*   104     4 */
	unsigned int               usersize;             /*   108     4 */
	unsigned int               compatsize;           /*   112     4 */
	unsigned int               hooks;                /*   116     4 */
	short unsigned int         proto;                /*   120     2 */
	short unsigned int         family;               /*   122     2 */

	/* size: 128, cachelines: 2, members: 16 */
	/* sum members: 122, holes: 1, sum holes: 2 */
	/* padding: 4 */
};
struct xt_entry_match {
	union {
		struct {
			__u16      match_size;           /*     0     2 */
			char       name[29];             /*     2    29 */
			__u8       revision;             /*    31     1 */
		} user;                                  /*     0    32 */
		struct {
			__u16      match_size;           /*     0     2 */

			/* XXX 6 bytes hole, try to pack */

			struct xt_match * match;         /*     8     8 */
		} kernel;                                /*     0    16 */
		__u16              match_size;           /*     0     2 */
	} u;                                             /*     0    32 */
	unsigned char              data[];               /*    32     0 */

	/* size: 32, cachelines: 1, members: 2 */
	/* last cacheline: 32 bytes */
};
struct xt_target {
	struct list_head           list;                 /*     0    16 */
	const const char           name[29];             /*    16    29 */
	u_int8_t                   revision;             /*    45     1 */

	/* XXX 2 bytes hole, try to pack */

	unsigned int               (*target)(struct sk_buff *, const struct xt_action_param  *); /*    48     8 */
	int                        (*checkentry)(const struct xt_tgchk_param  *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	void                       (*destroy)(const struct xt_tgdtor_param  *); /*    64     8 */
	void                       (*compat_from_user)(void *, const void  *); /*    72     8 */
	int                        (*compat_to_user)(void *, const void  *); /*    80     8 */
	struct module *            me;                   /*    88     8 */
	const char  *              table;                /*    96     8 */
	unsigned int               targetsize;           /*   104     4 */
	unsigned int               usersize;             /*   108     4 */
	unsigned int               compatsize;           /*   112     4 */
	unsigned int               hooks;                /*   116     4 */
	short unsigned int         proto;                /*   120     2 */
	short unsigned int         family;               /*   122     2 */

	/* size: 128, cachelines: 2, members: 16 */
	/* sum members: 122, holes: 1, sum holes: 2 */
	/* padding: 4 */
};
struct xt_entry_target {
	union {
		struct {
			__u16      target_size;          /*     0     2 */
			char       name[29];             /*     2    29 */
			__u8       revision;             /*    31     1 */
		} user;                                  /*     0    32 */
		struct {
			__u16      target_size;          /*     0     2 */

			/* XXX 6 bytes hole, try to pack */

			struct xt_target * target;       /*     8     8 */
		} kernel;                                /*     0    16 */
		__u16              target_size;          /*     0     2 */
	} u;                                             /*     0    32 */
	unsigned char              data[];               /*    32     0 */

	/* size: 32, cachelines: 1, members: 2 */
	/* last cacheline: 32 bytes */
};
struct xt_standard_target {
	struct xt_entry_target     target;               /*     0    32 */
	int                        verdict;              /*    32     4 */

	/* size: 40, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct xt_error_target {
	struct xt_entry_target     target;               /*     0    32 */
	char                       errorname[30];        /*    32    30 */

	/* size: 64, cachelines: 1, members: 2 */
	/* padding: 2 */
};
struct xt_counters {
	__u64                      pcnt;                 /*     0     8 */
	__u64                      bcnt;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct xt_counters_info {
	char                       name[32];             /*     0    32 */
	unsigned int               num_counters;         /*    32     4 */

	/* XXX 4 bytes hole, try to pack */

	struct xt_counters         counters[];           /*    40     0 */

	/* size: 40, cachelines: 1, members: 3 */
	/* sum members: 36, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct xt_action_param {
	union {
		const struct xt_match  * match;          /*     0     8 */
		const struct xt_target  * target;        /*     0     8 */
	};                                               /*     0     8 */
	union {
		const void  *      matchinfo;            /*     8     8 */
		const void  *      targinfo;             /*     8     8 */
	};                                               /*     8     8 */
	const struct nf_hook_state  * state;             /*    16     8 */
	int                        fragoff;              /*    24     4 */
	unsigned int               thoff;                /*    28     4 */
	bool                       hotdrop;              /*    32     1 */

	/* size: 40, cachelines: 1, members: 6 */
	/* padding: 7 */
	/* last cacheline: 40 bytes */
};
struct xt_mtchk_param {
	struct net *               net;                  /*     0     8 */
	const char  *              table;                /*     8     8 */
	const void  *              entryinfo;            /*    16     8 */
	const struct xt_match  *   match;                /*    24     8 */
	void *                     matchinfo;            /*    32     8 */
	unsigned int               hook_mask;            /*    40     4 */
	u_int8_t                   family;               /*    44     1 */
	bool                       nft_compat;           /*    45     1 */

	/* size: 48, cachelines: 1, members: 8 */
	/* padding: 2 */
	/* last cacheline: 48 bytes */
};
struct xt_mtdtor_param {
	struct net *               net;                  /*     0     8 */
	const struct xt_match  *   match;                /*     8     8 */
	void *                     matchinfo;            /*    16     8 */
	u_int8_t                   family;               /*    24     1 */

	/* size: 32, cachelines: 1, members: 4 */
	/* padding: 7 */
	/* last cacheline: 32 bytes */
};
struct xt_tgchk_param {
	struct net *               net;                  /*     0     8 */
	const char  *              table;                /*     8     8 */
	const void  *              entryinfo;            /*    16     8 */
	const struct xt_target  *  target;               /*    24     8 */
	void *                     targinfo;             /*    32     8 */
	unsigned int               hook_mask;            /*    40     4 */
	u_int8_t                   family;               /*    44     1 */
	bool                       nft_compat;           /*    45     1 */

	/* size: 48, cachelines: 1, members: 8 */
	/* padding: 2 */
	/* last cacheline: 48 bytes */
};
struct xt_tgdtor_param {
	struct net *               net;                  /*     0     8 */
	const struct xt_target  *  target;               /*     8     8 */
	void *                     targinfo;             /*    16     8 */
	u_int8_t                   family;               /*    24     1 */

	/* size: 32, cachelines: 1, members: 4 */
	/* padding: 7 */
	/* last cacheline: 32 bytes */
};
struct xt_percpu_counter_alloc_state {
	unsigned int               off;                  /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	const char  *              mem;                  /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct compat_xt_entry_match {
	union {
		struct {
			u_int16_t  match_size;           /*     0     2 */
			char       name[29];             /*     2    29 */
			u_int8_t   revision;             /*    31     1 */
		} user;                                  /*     0    32 */
		struct {
			u_int16_t  match_size;           /*     0     2 */

			/* XXX 2 bytes hole, try to pack */

			compat_uptr_t match;             /*     4     4 */
		} kernel;                                /*     0     8 */
		u_int16_t          match_size;           /*     0     2 */
	} u;                                             /*     0    32 */
	unsigned char              data[];               /*    32     0 */

	/* size: 32, cachelines: 1, members: 2 */
	/* last cacheline: 32 bytes */
};
struct compat_xt_entry_target {
	union {
		struct {
			u_int16_t  target_size;          /*     0     2 */
			char       name[29];             /*     2    29 */
			u_int8_t   revision;             /*    31     1 */
		} user;                                  /*     0    32 */
		struct {
			u_int16_t  target_size;          /*     0     2 */

			/* XXX 2 bytes hole, try to pack */

			compat_uptr_t target;            /*     4     4 */
		} kernel;                                /*     0     8 */
		u_int16_t          target_size;          /*     0     2 */
	} u;                                             /*     0    32 */
	unsigned char              data[];               /*    32     0 */

	/* size: 32, cachelines: 1, members: 2 */
	/* last cacheline: 32 bytes */
};
struct compat_xt_counters {
	compat_u64                 pcnt __attribute__((__aligned__(4))); /*     0     8 */
	compat_u64                 bcnt __attribute__((__aligned__(4))); /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* forced alignments: 2 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(4)));
struct compat_xt_counters_info {
	char                       name[32];             /*     0    32 */
	compat_uint_t              num_counters;         /*    32     4 */
	struct compat_xt_counters  counters[] __attribute__((__aligned__(4))); /*    36     0 */

	/* size: 36, cachelines: 1, members: 3 */
	/* forced alignments: 1 */
	/* last cacheline: 36 bytes */
} __attribute__((__packed__));
struct compat_delta {
	unsigned int               offset;               /*     0     4 */
	int                        delta;                /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct xt_af {
	struct mutex               mutex;                /*     0    32 */
	struct list_head           match;                /*    32    16 */
	struct list_head           target;               /*    48    16 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct mutex               compat_mutex;         /*    64    32 */
	struct compat_delta *      compat_tab;           /*    96     8 */
	unsigned int               number;               /*   104     4 */
	unsigned int               cur;                  /*   108     4 */

	/* size: 112, cachelines: 2, members: 7 */
	/* last cacheline: 48 bytes */
};
struct compat_xt_standard_target {
	struct compat_xt_entry_target t;                 /*     0    32 */
	compat_uint_t              verdict;              /*    32     4 */

	/* size: 36, cachelines: 1, members: 2 */
	/* last cacheline: 36 bytes */
};
struct compat_xt_error_target {
	struct compat_xt_entry_target t;                 /*     0    32 */
	char                       errorname[30];        /*    32    30 */

	/* size: 64, cachelines: 1, members: 2 */
	/* padding: 2 */
};
struct nf_mttg_trav {
	struct list_head *         head;                 /*     0     8 */
	struct list_head *         curr;                 /*     8     8 */
	uint8_t                    class;                /*    16     1 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 7 */
	/* last cacheline: 24 bytes */
};
struct xt_tcp {
	__u16                      spts[2];              /*     0     4 */
	__u16                      dpts[2];              /*     4     4 */
	__u8                       option;               /*     8     1 */
	__u8                       flg_mask;             /*     9     1 */
	__u8                       flg_cmp;              /*    10     1 */
	__u8                       invflags;             /*    11     1 */

	/* size: 12, cachelines: 1, members: 6 */
	/* last cacheline: 12 bytes */
};
struct xt_udp {
	__u16                      spts[2];              /*     0     4 */
	__u16                      dpts[2];              /*     4     4 */
	__u8                       invflags;             /*     8     1 */

	/* size: 10, cachelines: 1, members: 3 */
	/* padding: 1 */
	/* last cacheline: 10 bytes */
};
struct ip_mreqn {
	struct in_addr             imr_multiaddr;        /*     0     4 */
	struct in_addr             imr_address;          /*     4     4 */
	int                        imr_ifindex;          /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct rtmsg {
	unsigned char              rtm_family;           /*     0     1 */
	unsigned char              rtm_dst_len;          /*     1     1 */
	unsigned char              rtm_src_len;          /*     2     1 */
	unsigned char              rtm_tos;              /*     3     1 */
	unsigned char              rtm_table;            /*     4     1 */
	unsigned char              rtm_protocol;         /*     5     1 */
	unsigned char              rtm_scope;            /*     6     1 */
	unsigned char              rtm_type;             /*     7     1 */
	unsigned int               rtm_flags;            /*     8     4 */

	/* size: 12, cachelines: 1, members: 9 */
	/* last cacheline: 12 bytes */
};
struct rtvia {
	__kernel_sa_family_t       rtvia_family;         /*     0     2 */
	__u8                       rtvia_addr[];         /*     2     0 */

	/* size: 2, cachelines: 1, members: 2 */
	/* last cacheline: 2 bytes */
};
struct ip_mc_list {
	struct in_device *         interface;            /*     0     8 */
	__be32                     multiaddr;            /*     8     4 */
	unsigned int               sfmode;               /*    12     4 */
	struct ip_sf_list *        sources;              /*    16     8 */
	struct ip_sf_list *        tomb;                 /*    24     8 */
	long unsigned int          sfcount[2];           /*    32    16 */
	union {
		struct ip_mc_list * next;                /*    48     8 */
		struct ip_mc_list * next_rcu;            /*    48     8 */
	};                                               /*    48     8 */
	struct ip_mc_list *        next_hash;            /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct timer_list          timer;                /*    64    72 */
	/* --- cacheline 2 boundary (128 bytes) was 8 bytes ago --- */
	int                        users;                /*   136     4 */
	refcount_t                 refcnt;               /*   140     4 */
	spinlock_t                 lock;                 /*   144     4 */
	char                       tm_running;           /*   148     1 */
	char                       reporter;             /*   149     1 */
	char                       unsolicit_count;      /*   150     1 */
	char                       loaded;               /*   151     1 */
	unsigned char              gsquery;              /*   152     1 */
	unsigned char              crcount;              /*   153     1 */

	/* XXX 6 bytes hole, try to pack */

	struct callback_head       rcu __attribute__((__aligned__(8))); /*   160    16 */

	/* size: 176, cachelines: 3, members: 19 */
	/* sum members: 170, holes: 1, sum holes: 6 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 6 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct ip_sf_socklist {
	unsigned int               sl_max;               /*     0     4 */
	unsigned int               sl_count;             /*     4     4 */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*     8    16 */
	__be32                     sl_addr[];            /*    24     0 */

	/* size: 24, cachelines: 1, members: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct ip_mc_socklist {
	struct ip_mc_socklist *    next_rcu;             /*     0     8 */
	struct ip_mreqn            multi;                /*     8    12 */
	unsigned int               sfmode;               /*    20     4 */
	struct ip_sf_socklist *    sflist;               /*    24     8 */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*    32    16 */

	/* size: 48, cachelines: 1, members: 5 */
	/* forced alignments: 1 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct ip_sf_list {
	struct ip_sf_list *        sf_next;              /*     0     8 */
	long unsigned int          sf_count[2];          /*     8    16 */
	__be32                     sf_inaddr;            /*    24     4 */
	unsigned char              sf_gsresp;            /*    28     1 */
	unsigned char              sf_oldin;             /*    29     1 */
	unsigned char              sf_crcount;           /*    30     1 */

	/* size: 32, cachelines: 1, members: 6 */
	/* padding: 1 */
	/* last cacheline: 32 bytes */
};
struct ipv4_addr_key {
	__be32                     addr;                 /*     0     4 */
	int                        vif;                  /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct inetpeer_addr {
	union {
		struct ipv4_addr_key a4;                 /*     0     8 */
		struct in6_addr    a6;                   /*     0    16 */
		u32                key[4];               /*     0    16 */
	};                                               /*     0    16 */
	__u16                      family;               /*    16     2 */

	/* size: 20, cachelines: 1, members: 2 */
	/* padding: 2 */
	/* last cacheline: 20 bytes */
};
struct inet_peer {
	struct rb_node             rb_node __attribute__((__aligned__(8))); /*     0    24 */
	struct inetpeer_addr       daddr;                /*    24    20 */

	/* XXX last struct has 2 bytes of padding */

	u32                        metrics[17];          /*    44    68 */
	/* --- cacheline 1 boundary (64 bytes) was 48 bytes ago --- */
	u32                        rate_tokens;          /*   112     4 */
	u32                        n_redirects;          /*   116     4 */
	long unsigned int          rate_last;            /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	union {
		struct {
			atomic_t   rid;                  /*   128     4 */
		};                                       /*   128     4 */
		struct callback_head rcu __attribute__((__aligned__(8))); /*   128    16 */
	} __attribute__((__aligned__(8)));               /*   128    16 */
	__u32                      dtime;                /*   144     4 */
	refcount_t                 refcnt;               /*   148     4 */

	/* size: 152, cachelines: 3, members: 9 */
	/* paddings: 1, sum paddings: 2 */
	/* forced alignments: 2 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct fib_rt_info {
	struct fib_info *          fi;                   /*     0     8 */
	u32                        tb_id;                /*     8     4 */
	__be32                     dst;                  /*    12     4 */
	int                        dst_len;              /*    16     4 */
	u8                         tos;                  /*    20     1 */
	u8                         type;                 /*    21     1 */
	u8                         offload:1;            /*    22: 0  1 */
	u8                         trap:1;               /*    22: 1  1 */
	u8                         unused:6;             /*    22: 2  1 */

	/* size: 24, cachelines: 1, members: 9 */
	/* padding: 1 */
	/* last cacheline: 24 bytes */
};
struct uncached_list {
	spinlock_t                 lock;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           head;                 /*     8    16 */

	/* size: 24, cachelines: 1, members: 2 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct ip_rt_acct {
	__u32                      o_bytes;              /*     0     4 */
	__u32                      o_packets;            /*     4     4 */
	__u32                      i_bytes;              /*     8     4 */
	__u32                      i_packets;            /*    12     4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct rt_cache_stat {
	unsigned int               in_slow_tot;          /*     0     4 */
	unsigned int               in_slow_mc;           /*     4     4 */
	unsigned int               in_no_route;          /*     8     4 */
	unsigned int               in_brd;               /*    12     4 */
	unsigned int               in_martian_dst;       /*    16     4 */
	unsigned int               in_martian_src;       /*    20     4 */
	unsigned int               out_slow_tot;         /*    24     4 */
	unsigned int               out_slow_mc;          /*    28     4 */

	/* size: 32, cachelines: 1, members: 8 */
	/* last cacheline: 32 bytes */
};
struct fib_alias {
	struct hlist_node          fa_list;              /*     0    16 */
	struct fib_info *          fa_info;              /*    16     8 */
	u8                         fa_tos;               /*    24     1 */
	u8                         fa_type;              /*    25     1 */
	u8                         fa_state;             /*    26     1 */
	u8                         fa_slen;              /*    27     1 */
	u32                        tb_id;                /*    28     4 */
	s16                        fa_default;           /*    32     2 */
	u8                         offload:1;            /*    34: 0  1 */
	u8                         trap:1;               /*    34: 1  1 */
	u8                         unused:6;             /*    34: 2  1 */

	/* XXX 5 bytes hole, try to pack */

	struct callback_head       rcu __attribute__((__aligned__(8))); /*    40    16 */

	/* size: 56, cachelines: 1, members: 12 */
	/* sum members: 50, holes: 1, sum holes: 5 */
	/* sum bitfield members: 8 bits (1 bytes) */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 5 */
	/* last cacheline: 56 bytes */
} __attribute__((__aligned__(8)));
struct fib_prop {
	int                        error;                /*     0     4 */
	u8                         scope;                /*     4     1 */

	/* size: 8, cachelines: 1, members: 2 */
	/* padding: 3 */
	/* last cacheline: 8 bytes */
};
struct net_offload {
	struct offload_callbacks   callbacks;            /*     0    24 */
	unsigned int               flags;                /*    24     4 */

	/* size: 32, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct raw_hashinfo {
	rwlock_t                   lock;                 /*     0     8 */
	struct hlist_head          ht[256];              /*     8  2048 */

	/* size: 2056, cachelines: 33, members: 2 */
	/* last cacheline: 8 bytes */
};
struct ipq {
	struct inet_frag_queue     q __attribute__((__aligned__(8))); /*     0   208 */
	/* --- cacheline 3 boundary (192 bytes) was 16 bytes ago --- */
	u8                         ecn;                  /*   208     1 */

	/* XXX 1 byte hole, try to pack */

	u16                        max_df_size;          /*   210     2 */
	int                        iif;                  /*   212     4 */
	unsigned int               rid;                  /*   216     4 */

	/* XXX 4 bytes hole, try to pack */

	struct inet_peer *         peer;                 /*   224     8 */

	/* size: 232, cachelines: 4, members: 6 */
	/* sum members: 227, holes: 2, sum holes: 5 */
	/* forced alignments: 1 */
	/* last cacheline: 40 bytes */
} __attribute__((__aligned__(8)));
struct ip_options_data {
	struct ip_options_rcu      opt __attribute__((__aligned__(8))); /*     0    32 */
	char                       data[40];             /*    32    40 */

	/* size: 72, cachelines: 2, members: 2 */
	/* forced alignments: 1 */
	/* last cacheline: 8 bytes */
} __attribute__((__aligned__(8)));
struct ipcm_cookie {
	struct sockcm_cookie       sockc;                /*     0    16 */

	/* XXX last struct has 2 bytes of padding */

	__be32                     addr;                 /*    16     4 */
	int                        oif;                  /*    20     4 */
	struct ip_options_rcu *    opt;                  /*    24     8 */
	__u8                       ttl;                  /*    32     1 */

	/* XXX 1 byte hole, try to pack */

	__s16                      tos;                  /*    34     2 */
	char                       priority;             /*    36     1 */

	/* XXX 1 byte hole, try to pack */

	__u16                      gso_size;             /*    38     2 */

	/* size: 40, cachelines: 1, members: 8 */
	/* sum members: 38, holes: 2, sum holes: 2 */
	/* paddings: 1, sum paddings: 2 */
	/* last cacheline: 40 bytes */
};
struct ip_fraglist_iter {
	struct sk_buff *           frag;                 /*     0     8 */
	struct iphdr *             iph;                  /*     8     8 */
	int                        offset;               /*    16     4 */
	unsigned int               hlen;                 /*    20     4 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct ip_frag_state {
	bool                       DF;                   /*     0     1 */

	/* XXX 3 bytes hole, try to pack */

	unsigned int               hlen;                 /*     4     4 */
	unsigned int               ll_rs;                /*     8     4 */
	unsigned int               mtu;                  /*    12     4 */
	unsigned int               left;                 /*    16     4 */
	int                        offset;               /*    20     4 */
	int                        ptr;                  /*    24     4 */
	__be16                     not_last_frag;        /*    28     2 */

	/* size: 32, cachelines: 1, members: 8 */
	/* sum members: 27, holes: 1, sum holes: 3 */
	/* padding: 2 */
	/* last cacheline: 32 bytes */
};
struct ip_reply_arg {
	struct kvec                iov[1];               /*     0    16 */
	int                        flags;                /*    16     4 */
	__wsum                     csum;                 /*    20     4 */
	int                        csumoffset;           /*    24     4 */
	int                        bound_dev_if;         /*    28     4 */
	u8                         tos;                  /*    32     1 */

	/* XXX 3 bytes hole, try to pack */

	kuid_t                     uid;                  /*    36     4 */

	/* size: 40, cachelines: 1, members: 7 */
	/* sum members: 37, holes: 1, sum holes: 3 */
	/* last cacheline: 40 bytes */
};
struct ip_mreq_source {
	__be32                     imr_multiaddr;        /*     0     4 */
	__be32                     imr_interface;        /*     4     4 */
	__be32                     imr_sourceaddr;       /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct ip_msfilter {
	__be32                     imsf_multiaddr;       /*     0     4 */
	__be32                     imsf_interface;       /*     4     4 */
	__u32                      imsf_fmode;           /*     8     4 */
	__u32                      imsf_numsrc;          /*    12     4 */
	__be32                     imsf_slist[1];        /*    16     4 */

	/* size: 20, cachelines: 1, members: 5 */
	/* last cacheline: 20 bytes */
};
struct group_req {
	__u32                      gr_interface;         /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct __kernel_sockaddr_storage gr_group;       /*     8   128 */

	/* size: 136, cachelines: 3, members: 2 */
	/* sum members: 132, holes: 1, sum holes: 4 */
	/* last cacheline: 8 bytes */
};
struct group_source_req {
	__u32                      gsr_interface;        /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct __kernel_sockaddr_storage gsr_group;      /*     8   128 */
	/* --- cacheline 2 boundary (128 bytes) was 8 bytes ago --- */
	struct __kernel_sockaddr_storage gsr_source;     /*   136   128 */

	/* size: 264, cachelines: 5, members: 3 */
	/* sum members: 260, holes: 1, sum holes: 4 */
	/* last cacheline: 8 bytes */
};
struct group_filter {
	__u32                      gf_interface;         /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct __kernel_sockaddr_storage gf_group;       /*     8   128 */
	/* --- cacheline 2 boundary (128 bytes) was 8 bytes ago --- */
	__u32                      gf_fmode;             /*   136     4 */
	__u32                      gf_numsrc;            /*   140     4 */
	struct __kernel_sockaddr_storage gf_slist[1];    /*   144   128 */

	/* size: 272, cachelines: 5, members: 5 */
	/* sum members: 268, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct in_pktinfo {
	int                        ipi_ifindex;          /*     0     4 */
	struct in_addr             ipi_spec_dst;         /*     4     4 */
	struct in_addr             ipi_addr;             /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct compat_group_req {
	__u32                      gr_interface;         /*     0     4 */
	struct __kernel_sockaddr_storage gr_group __attribute__((__aligned__(4))); /*     4   128 */

	/* size: 132, cachelines: 3, members: 2 */
	/* forced alignments: 1 */
	/* last cacheline: 4 bytes */
} __attribute__((__packed__));
struct compat_group_source_req {
	__u32                      gsr_interface;        /*     0     4 */
	struct __kernel_sockaddr_storage gsr_group __attribute__((__aligned__(4))); /*     4   128 */
	/* --- cacheline 2 boundary (128 bytes) was 4 bytes ago --- */
	struct __kernel_sockaddr_storage gsr_source __attribute__((__aligned__(4))); /*   132   128 */

	/* size: 260, cachelines: 5, members: 3 */
	/* forced alignments: 2 */
	/* last cacheline: 4 bytes */
} __attribute__((__packed__));
struct compat_group_filter {
	__u32                      gf_interface;         /*     0     4 */
	struct __kernel_sockaddr_storage gf_group __attribute__((__aligned__(4))); /*     4   128 */
	/* --- cacheline 2 boundary (128 bytes) was 4 bytes ago --- */
	__u32                      gf_fmode;             /*   132     4 */
	__u32                      gf_numsrc;            /*   136     4 */
	struct __kernel_sockaddr_storage gf_slist[1] __attribute__((__aligned__(4))); /*   140   128 */

	/* size: 268, cachelines: 5, members: 5 */
	/* forced alignments: 2 */
	/* last cacheline: 12 bytes */
} __attribute__((__packed__));
struct tcp_request_sock_ops {
	u16                        mss_clamp;            /*     0     2 */

	/* XXX 6 bytes hole, try to pack */

	struct tcp_md5sig_key *    (*req_md5_lookup)(const struct sock  *, const struct sock  *); /*     8     8 */
	int                        (*calc_md5_hash)(char *, const struct tcp_md5sig_key  *, const struct sock  *, const struct sk_buff  *); /*    16     8 */
	void                       (*init_req)(struct request_sock *, const struct sock  *, struct sk_buff *); /*    24     8 */
	__u32                      (*cookie_init_seq)(const struct sk_buff  *, __u16 *); /*    32     8 */
	struct dst_entry *         (*route_req)(const struct sock  *, struct flowi *, const struct request_sock  *); /*    40     8 */
	u32                        (*init_seq)(const struct sk_buff  *); /*    48     8 */
	u32                        (*init_ts_off)(const struct net  *, const struct sk_buff  *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*send_synack)(const struct sock  *, struct dst_entry *, struct flowi *, struct request_sock *, struct tcp_fastopen_cookie *, enum tcp_synack_type, struct sk_buff *); /*    64     8 */

	/* size: 72, cachelines: 2, members: 9 */
	/* sum members: 66, holes: 1, sum holes: 6 */
	/* last cacheline: 8 bytes */
};
struct tcpvegas_info {
	__u32                      tcpv_enabled;         /*     0     4 */
	__u32                      tcpv_rttcnt;          /*     4     4 */
	__u32                      tcpv_rtt;             /*     8     4 */
	__u32                      tcpv_minrtt;          /*    12     4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct tcp_dctcp_info {
	__u16                      dctcp_enabled;        /*     0     2 */
	__u16                      dctcp_ce_state;       /*     2     2 */
	__u32                      dctcp_alpha;          /*     4     4 */
	__u32                      dctcp_ab_ecn;         /*     8     4 */
	__u32                      dctcp_ab_tot;         /*    12     4 */

	/* size: 16, cachelines: 1, members: 5 */
	/* last cacheline: 16 bytes */
};
struct tcp_bbr_info {
	__u32                      bbr_bw_lo;            /*     0     4 */
	__u32                      bbr_bw_hi;            /*     4     4 */
	__u32                      bbr_min_rtt;          /*     8     4 */
	__u32                      bbr_pacing_gain;      /*    12     4 */
	__u32                      bbr_cwnd_gain;        /*    16     4 */

	/* size: 20, cachelines: 1, members: 5 */
	/* last cacheline: 20 bytes */
};
union tcp_cc_info {
	struct tcpvegas_info       vegas;              /*     0    16 */
	struct tcp_dctcp_info      dctcp;              /*     0    16 */
	struct tcp_bbr_info        bbr;                /*     0    20 */
};
struct tcp_repair_opt {
	__u32                      opt_code;             /*     0     4 */
	__u32                      opt_val;              /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct tcp_repair_window {
	__u32                      snd_wl1;              /*     0     4 */
	__u32                      snd_wnd;              /*     4     4 */
	__u32                      max_window;           /*     8     4 */
	__u32                      rcv_wnd;              /*    12     4 */
	__u32                      rcv_wup;              /*    16     4 */

	/* size: 20, cachelines: 1, members: 5 */
	/* last cacheline: 20 bytes */
};
struct tcp_info {
	__u8                       tcpi_state;           /*     0     1 */
	__u8                       tcpi_ca_state;        /*     1     1 */
	__u8                       tcpi_retransmits;     /*     2     1 */
	__u8                       tcpi_probes;          /*     3     1 */
	__u8                       tcpi_backoff;         /*     4     1 */
	__u8                       tcpi_options;         /*     5     1 */
	__u8                       tcpi_snd_wscale:4;    /*     6: 0  1 */
	__u8                       tcpi_rcv_wscale:4;    /*     6: 4  1 */
	__u8                       tcpi_delivery_rate_app_limited:1; /*     7: 0  1 */
	__u8                       tcpi_fastopen_client_fail:2; /*     7: 1  1 */

	/* XXX 5 bits hole, try to pack */

	__u32                      tcpi_rto;             /*     8     4 */
	__u32                      tcpi_ato;             /*    12     4 */
	__u32                      tcpi_snd_mss;         /*    16     4 */
	__u32                      tcpi_rcv_mss;         /*    20     4 */
	__u32                      tcpi_unacked;         /*    24     4 */
	__u32                      tcpi_sacked;          /*    28     4 */
	__u32                      tcpi_lost;            /*    32     4 */
	__u32                      tcpi_retrans;         /*    36     4 */
	__u32                      tcpi_fackets;         /*    40     4 */
	__u32                      tcpi_last_data_sent;  /*    44     4 */
	__u32                      tcpi_last_ack_sent;   /*    48     4 */
	__u32                      tcpi_last_data_recv;  /*    52     4 */
	__u32                      tcpi_last_ack_recv;   /*    56     4 */
	__u32                      tcpi_pmtu;            /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	__u32                      tcpi_rcv_ssthresh;    /*    64     4 */
	__u32                      tcpi_rtt;             /*    68     4 */
	__u32                      tcpi_rttvar;          /*    72     4 */
	__u32                      tcpi_snd_ssthresh;    /*    76     4 */
	__u32                      tcpi_snd_cwnd;        /*    80     4 */
	__u32                      tcpi_advmss;          /*    84     4 */
	__u32                      tcpi_reordering;      /*    88     4 */
	__u32                      tcpi_rcv_rtt;         /*    92     4 */
	__u32                      tcpi_rcv_space;       /*    96     4 */
	__u32                      tcpi_total_retrans;   /*   100     4 */
	__u64                      tcpi_pacing_rate;     /*   104     8 */
	__u64                      tcpi_max_pacing_rate; /*   112     8 */
	__u64                      tcpi_bytes_acked;     /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	__u64                      tcpi_bytes_received;  /*   128     8 */
	__u32                      tcpi_segs_out;        /*   136     4 */
	__u32                      tcpi_segs_in;         /*   140     4 */
	__u32                      tcpi_notsent_bytes;   /*   144     4 */
	__u32                      tcpi_min_rtt;         /*   148     4 */
	__u32                      tcpi_data_segs_in;    /*   152     4 */
	__u32                      tcpi_data_segs_out;   /*   156     4 */
	__u64                      tcpi_delivery_rate;   /*   160     8 */
	__u64                      tcpi_busy_time;       /*   168     8 */
	__u64                      tcpi_rwnd_limited;    /*   176     8 */
	__u64                      tcpi_sndbuf_limited;  /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	__u32                      tcpi_delivered;       /*   192     4 */
	__u32                      tcpi_delivered_ce;    /*   196     4 */
	__u64                      tcpi_bytes_sent;      /*   200     8 */
	__u64                      tcpi_bytes_retrans;   /*   208     8 */
	__u32                      tcpi_dsack_dups;      /*   216     4 */
	__u32                      tcpi_reord_seen;      /*   220     4 */
	__u32                      tcpi_rcv_ooopack;     /*   224     4 */
	__u32                      tcpi_snd_wnd;         /*   228     4 */

	/* size: 232, cachelines: 4, members: 56 */
	/* sum members: 230 */
	/* sum bitfield members: 11 bits, bit holes: 1, sum bit holes: 5 bits */
	/* last cacheline: 40 bytes */
};
struct tcp_zerocopy_receive {
	__u64                      address;              /*     0     8 */
	__u32                      length;               /*     8     4 */
	__u32                      recv_skip_hint;       /*    12     4 */
	__u32                      inq;                  /*    16     4 */
	__s32                      err;                  /*    20     4 */
	__u64                      copybuf_address;      /*    24     8 */
	__s32                      copybuf_len;          /*    32     4 */

	/* size: 40, cachelines: 1, members: 7 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct tcp_md5sig_pool {
	struct ahash_request *     md5_req;              /*     0     8 */
	void *                     scratch;              /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct tcp_splice_state {
	struct pipe_inode_info *   pipe;                 /*     0     8 */
	size_t                     len;                  /*     8     8 */
	unsigned int               flags;                /*    16     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct tcp_sack_block_wire {
	__be32                     start_seq;            /*     0     4 */
	__be32                     end_seq;              /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct static_key_false_deferred {
	struct static_key_false    key;                  /*     0    16 */
	long unsigned int          timeout;              /*    16     8 */
	struct delayed_work        work;                 /*    24   192 */

	/* size: 216, cachelines: 4, members: 3 */
	/* last cacheline: 24 bytes */
};
struct tcp_sacktag_state {
	u64                        first_sackt;          /*     0     8 */
	u64                        last_sackt;           /*     8     8 */
	u32                        reord;                /*    16     4 */
	u32                        sack_delivered;       /*    20     4 */
	int                        flag;                 /*    24     4 */
	unsigned int               mss_now;              /*    28     4 */
	struct rate_sample *       rate;                 /*    32     8 */

	/* size: 40, cachelines: 1, members: 7 */
	/* last cacheline: 40 bytes */
};
struct mptcp_out_options {

	/* size: 0, cachelines: 0, members: 0 */
};
struct tcp_out_options {
	u16                        options;              /*     0     2 */
	u16                        mss;                  /*     2     2 */
	u8                         ws;                   /*     4     1 */
	u8                         num_sack_blocks;      /*     5     1 */
	u8                         hash_size;            /*     6     1 */
	u8                         bpf_opt_len;          /*     7     1 */
	__u8 *                     hash_location;        /*     8     8 */
	__u32                      tsval;                /*    16     4 */
	__u32                      tsecr;                /*    20     4 */
	struct tcp_fastopen_cookie * fastopen_cookie;    /*    24     8 */
	struct mptcp_out_options   mptcp;                /*    32     0 */

	/* size: 32, cachelines: 1, members: 11 */
	/* last cacheline: 32 bytes */
};
struct tsq_tasklet {
	struct tasklet_struct      tasklet;              /*     0    48 */
	struct list_head           head;                 /*    48    16 */

	/* size: 64, cachelines: 1, members: 2 */
};
struct tcp_md5sig {
	struct __kernel_sockaddr_storage tcpm_addr;      /*     0   128 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	__u8                       tcpm_flags;           /*   128     1 */
	__u8                       tcpm_prefixlen;       /*   129     1 */
	__u16                      tcpm_keylen;          /*   130     2 */
	int                        tcpm_ifindex;         /*   132     4 */
	__u8                       tcpm_key[80];         /*   136    80 */

	/* size: 216, cachelines: 4, members: 6 */
	/* last cacheline: 24 bytes */
};
struct icmp_err {
	int                        errno;                /*     0     4 */
	unsigned int               fatal:1;              /*     4: 0  4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* bit_padding: 31 bits */
	/* last cacheline: 8 bytes */
};
struct tcp4_pseudohdr {
	__be32                     saddr;                /*     0     4 */
	__be32                     daddr;                /*     4     4 */
	__u8                       pad;                  /*     8     1 */
	__u8                       protocol;             /*     9     1 */
	__be16                     len;                  /*    10     2 */

	/* size: 12, cachelines: 1, members: 5 */
	/* last cacheline: 12 bytes */
};
struct tcp_seq_afinfo {
	sa_family_t                family;               /*     0     2 */

	/* size: 2, cachelines: 1, members: 1 */
	/* last cacheline: 2 bytes */
};
struct tcp_iter_state {
	struct seq_net_private     p;                    /*     0     8 */
	enum tcp_seq_states        state;                /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct sock *              syn_wait_sk;          /*    16     8 */
	struct tcp_seq_afinfo *    bpf_seq_afinfo;       /*    24     8 */
	int                        bucket;               /*    32     4 */
	int                        offset;               /*    36     4 */
	int                        sbucket;              /*    40     4 */
	int                        num;                  /*    44     4 */
	loff_t                     last_pos;             /*    48     8 */

	/* size: 56, cachelines: 1, members: 9 */
	/* sum members: 52, holes: 1, sum holes: 4 */
	/* last cacheline: 56 bytes */
};
struct bpf_iter__tcp {
	union {
		struct bpf_iter_meta * meta;             /*     0     8 */
	} __attribute__((__aligned__(8)));               /*     0     8 */
	union {
		struct sock_common * sk_common;          /*     8     8 */
	} __attribute__((__aligned__(8)));               /*     8     8 */
	uid_t                      uid __attribute__((__aligned__(8))); /*    16     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* forced alignments: 3 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct tcp_fastopen_metrics {
	u16                        mss;                  /*     0     2 */
	u16                        syn_loss:10;          /*     2: 0  2 */
	u16                        try_exp:2;            /*     2:10  2 */

	/* XXX 4 bits hole, try to pack */
	/* XXX 4 bytes hole, try to pack */

	long unsigned int          last_syn_loss;        /*     8     8 */
	struct tcp_fastopen_cookie cookie;               /*    16    24 */

	/* XXX last struct has 6 bytes of padding */

	/* size: 40, cachelines: 1, members: 5 */
	/* sum members: 34, holes: 1, sum holes: 4 */
	/* sum bitfield members: 12 bits, bit holes: 1, sum bit holes: 4 bits */
	/* paddings: 1, sum paddings: 6 */
	/* last cacheline: 40 bytes */
};
struct tcp_metrics_block {
	struct tcp_metrics_block * tcpm_next;            /*     0     8 */
	possible_net_t             tcpm_net;             /*     8     8 */
	struct inetpeer_addr       tcpm_saddr;           /*    16    20 */

	/* XXX last struct has 2 bytes of padding */

	struct inetpeer_addr       tcpm_daddr;           /*    36    20 */

	/* XXX last struct has 2 bytes of padding */

	long unsigned int          tcpm_stamp;           /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u32                        tcpm_lock;            /*    64     4 */
	u32                        tcpm_vals[5];         /*    68    20 */
	struct tcp_fastopen_metrics tcpm_fastopen;       /*    88    40 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct callback_head       callback_head __attribute__((__aligned__(8))); /*   128    16 */

	/* size: 144, cachelines: 3, members: 9 */
	/* paddings: 2, sum paddings: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct tcpm_hash_bucket {
	struct tcp_metrics_block * chain;                /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct icmp_filter {
	__u32                      data;                 /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct raw_iter_state {
	struct seq_net_private     p;                    /*     0     8 */
	int                        bucket;               /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct raw_sock {
	struct inet_sock           inet __attribute__((__aligned__(8))); /*     0  1096 */
	/* --- cacheline 17 boundary (1088 bytes) was 8 bytes ago --- */
	struct icmp_filter         filter;               /*  1096     4 */
	u32                        ipmr_table;           /*  1100     4 */

	/* size: 1104, cachelines: 18, members: 3 */
	/* forced alignments: 1 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct raw_frag_vec {
	struct msghdr *            msg;                  /*     0     8 */
	union {
		struct icmphdr     icmph;                /*     8     8 */
		char               c[1];                 /*     8     1 */
	} hdr;                                           /*     8     8 */
	int                        hlen;                 /*    16     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct ip_tunnel_encap {
	u16                        type;                 /*     0     2 */
	u16                        flags;                /*     2     2 */
	__be16                     sport;                /*     4     2 */
	__be16                     dport;                /*     6     2 */

	/* size: 8, cachelines: 1, members: 4 */
	/* last cacheline: 8 bytes */
};
struct ip_tunnel_encap_ops {
	size_t                     (*encap_hlen)(struct ip_tunnel_encap *); /*     0     8 */
	int                        (*build_header)(struct sk_buff *, struct ip_tunnel_encap *, u8 *, struct flowi4 *); /*     8     8 */
	int                        (*err_handler)(struct sk_buff *, u32); /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct udp_skb_cb {
	union {
		struct inet_skb_parm h4;                 /*     0    24 */
		struct inet6_skb_parm h6;                /*     0    24 */
	} header;                                        /*     0    24 */
	__u16                      cscov;                /*    24     2 */
	__u8                       partial_cov;          /*    26     1 */

	/* size: 28, cachelines: 1, members: 3 */
	/* padding: 1 */
	/* last cacheline: 28 bytes */
};
struct udp_dev_scratch {
	u32                        _tsize_state;         /*     0     4 */
	u16                        len;                  /*     4     2 */
	bool                       is_linear;            /*     6     1 */
	bool                       csum_unnecessary;     /*     7     1 */

	/* size: 8, cachelines: 1, members: 4 */
	/* last cacheline: 8 bytes */
};
struct udp_seq_afinfo {
	sa_family_t                family;               /*     0     2 */

	/* XXX 6 bytes hole, try to pack */

	struct udp_table *         udp_table;            /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 10, holes: 1, sum holes: 6 */
	/* last cacheline: 16 bytes */
};
struct udp_iter_state {
	struct seq_net_private     p;                    /*     0     8 */
	int                        bucket;               /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct udp_seq_afinfo *    bpf_seq_afinfo;       /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct bpf_iter__udp {
	union {
		struct bpf_iter_meta * meta;             /*     0     8 */
	} __attribute__((__aligned__(8)));               /*     0     8 */
	union {
		struct udp_sock *  udp_sk;               /*     8     8 */
	} __attribute__((__aligned__(8)));               /*     8     8 */
	uid_t                      uid __attribute__((__aligned__(8))); /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	int                        bucket __attribute__((__aligned__(8))); /*    24     4 */

	/* size: 32, cachelines: 1, members: 4 */
	/* sum members: 24, holes: 1, sum holes: 4 */
	/* padding: 4 */
	/* forced alignments: 4, forced holes: 1, sum forced holes: 4 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct inet_protosw {
	struct list_head           list;                 /*     0    16 */
	short unsigned int         type;                 /*    16     2 */
	short unsigned int         protocol;             /*    18     2 */

	/* XXX 4 bytes hole, try to pack */

	struct proto *             prot;                 /*    24     8 */
	const struct proto_ops  *  ops;                  /*    32     8 */
	unsigned char              flags;                /*    40     1 */

	/* size: 48, cachelines: 1, members: 6 */
	/* sum members: 37, holes: 1, sum holes: 4 */
	/* padding: 7 */
	/* last cacheline: 48 bytes */
};
struct arpreq {
	struct sockaddr            arp_pa;               /*     0    16 */
	struct sockaddr            arp_ha;               /*    16    16 */
	int                        arp_flags;            /*    32     4 */
	struct sockaddr            arp_netmask;          /*    36    16 */
	char                       arp_dev[16];          /*    52    16 */

	/* size: 68, cachelines: 2, members: 5 */
	/* last cacheline: 4 bytes */
};
struct icmp_ext_hdr {
	__u8                       reserved1:4;          /*     0: 0  1 */
	__u8                       version:4;            /*     0: 4  1 */
	__u8                       reserved2;            /*     1     1 */
	__sum16                    checksum;             /*     2     2 */

	/* size: 4, cachelines: 1, members: 4 */
	/* last cacheline: 4 bytes */
};
struct icmp_extobj_hdr {
	__be16                     length;               /*     0     2 */
	__u8                       class_num;            /*     2     1 */
	__u8                       class_type;           /*     3     1 */

	/* size: 4, cachelines: 1, members: 3 */
	/* last cacheline: 4 bytes */
};
struct icmp_bxm {
	struct sk_buff *           skb;                  /*     0     8 */
	int                        offset;               /*     8     4 */
	int                        data_len;             /*    12     4 */
	struct {
		struct icmphdr     icmph;                /*    16     8 */
		__be32             times[3];             /*    24    12 */
	} data;                                          /*    16    20 */
	int                        head_len;             /*    36     4 */
	struct ip_options_data     replyopts __attribute__((__aligned__(8))); /*    40    72 */

	/* size: 112, cachelines: 2, members: 6 */
	/* forced alignments: 1 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct icmp_control {
	bool                       (*handler)(struct sk_buff *); /*     0     8 */
	short int                  error;                /*     8     2 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 6 */
	/* last cacheline: 16 bytes */
};
struct ifaddrmsg {
	__u8                       ifa_family;           /*     0     1 */
	__u8                       ifa_prefixlen;        /*     1     1 */
	__u8                       ifa_flags;            /*     2     1 */
	__u8                       ifa_scope;            /*     3     1 */
	__u32                      ifa_index;            /*     4     4 */

	/* size: 8, cachelines: 1, members: 5 */
	/* last cacheline: 8 bytes */
};
struct ifa_cacheinfo {
	__u32                      ifa_prefered;         /*     0     4 */
	__u32                      ifa_valid;            /*     4     4 */
	__u32                      cstamp;               /*     8     4 */
	__u32                      tstamp;               /*    12     4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct in_validator_info {
	__be32                     ivi_addr;             /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct in_device *         ivi_dev;              /*     8     8 */
	struct netlink_ext_ack *   extack;               /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct netconfmsg {
	__u8                       ncm_family;           /*     0     1 */

	/* size: 1, cachelines: 1, members: 1 */
	/* last cacheline: 1 bytes */
};
struct inet_fill_args {
	u32                        portid;               /*     0     4 */
	u32                        seq;                  /*     4     4 */
	int                        event;                /*     8     4 */
	unsigned int               flags;                /*    12     4 */
	int                        netnsid;              /*    16     4 */
	int                        ifindex;              /*    20     4 */

	/* size: 24, cachelines: 1, members: 6 */
	/* last cacheline: 24 bytes */
};
struct devinet_sysctl_table {
	struct ctl_table_header *  sysctl_header;        /*     0     8 */
	struct ctl_table           devinet_vars[33];     /*     8  2112 */

	/* size: 2120, cachelines: 34, members: 2 */
	/* last cacheline: 8 bytes */
};
struct rtentry {
	long unsigned int          rt_pad1;              /*     0     8 */
	struct sockaddr            rt_dst;               /*     8    16 */
	struct sockaddr            rt_gateway;           /*    24    16 */
	struct sockaddr            rt_genmask;           /*    40    16 */
	short unsigned int         rt_flags;             /*    56     2 */
	short int                  rt_pad2;              /*    58     2 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	long unsigned int          rt_pad3;              /*    64     8 */
	void *                     rt_pad4;              /*    72     8 */
	short int                  rt_metric;            /*    80     2 */

	/* XXX 6 bytes hole, try to pack */

	char *                     rt_dev;               /*    88     8 */
	long unsigned int          rt_mtu;               /*    96     8 */
	long unsigned int          rt_window;            /*   104     8 */
	short unsigned int         rt_irtt;              /*   112     2 */

	/* size: 120, cachelines: 2, members: 13 */
	/* sum members: 104, holes: 2, sum holes: 10 */
	/* padding: 6 */
	/* last cacheline: 56 bytes */
};
struct pingv6_ops {
	int                        (*ipv6_recv_error)(struct sock *, struct msghdr *, int, int *); /*     0     8 */
	void                       (*ip6_datagram_recv_common_ctl)(struct sock *, struct msghdr *, struct sk_buff *); /*     8     8 */
	void                       (*ip6_datagram_recv_specific_ctl)(struct sock *, struct msghdr *, struct sk_buff *); /*    16     8 */
	int                        (*icmpv6_err_convert)(u8, u8, int *); /*    24     8 */
	void                       (*ipv6_icmp_error)(struct sock *, struct sk_buff *, int, __be16, u32, u8 *); /*    32     8 */
	int                        (*ipv6_chk_addr)(struct net *, const struct in6_addr  *, const struct net_device  *, int); /*    40     8 */

	/* size: 48, cachelines: 1, members: 6 */
	/* last cacheline: 48 bytes */
};
struct compat_rtentry {
	u32                        rt_pad1;              /*     0     4 */
	struct sockaddr            rt_dst;               /*     4    16 */
	struct sockaddr            rt_gateway;           /*    20    16 */
	struct sockaddr            rt_genmask;           /*    36    16 */
	short unsigned int         rt_flags;             /*    52     2 */
	short int                  rt_pad2;              /*    54     2 */
	u32                        rt_pad3;              /*    56     4 */
	unsigned char              rt_tos;               /*    60     1 */
	unsigned char              rt_class;             /*    61     1 */
	short int                  rt_pad4;              /*    62     2 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	short int                  rt_metric;            /*    64     2 */

	/* XXX 2 bytes hole, try to pack */

	compat_uptr_t              rt_dev;               /*    68     4 */
	u32                        rt_mtu;               /*    72     4 */
	u32                        rt_window;            /*    76     4 */
	short unsigned int         rt_irtt;              /*    80     2 */

	/* size: 84, cachelines: 2, members: 15 */
	/* sum members: 80, holes: 1, sum holes: 2 */
	/* padding: 2 */
	/* last cacheline: 20 bytes */
};
struct igmphdr {
	__u8                       type;                 /*     0     1 */
	__u8                       code;                 /*     1     1 */
	__sum16                    csum;                 /*     2     2 */
	__be32                     group;                /*     4     4 */

	/* size: 8, cachelines: 1, members: 4 */
	/* last cacheline: 8 bytes */
};
struct igmpv3_grec {
	__u8                       grec_type;            /*     0     1 */
	__u8                       grec_auxwords;        /*     1     1 */
	__be16                     grec_nsrcs;           /*     2     2 */
	__be32                     grec_mca;             /*     4     4 */
	__be32                     grec_src[];           /*     8     0 */

	/* size: 8, cachelines: 1, members: 5 */
	/* last cacheline: 8 bytes */
};
struct igmpv3_report {
	__u8                       type;                 /*     0     1 */
	__u8                       resv1;                /*     1     1 */
	__sum16                    csum;                 /*     2     2 */
	__be16                     resv2;                /*     4     2 */
	__be16                     ngrec;                /*     6     2 */
	struct igmpv3_grec         grec[];               /*     8     0 */

	/* size: 8, cachelines: 1, members: 6 */
	/* last cacheline: 8 bytes */
};
struct igmpv3_query {
	__u8                       type;                 /*     0     1 */
	__u8                       code;                 /*     1     1 */
	__sum16                    csum;                 /*     2     2 */
	__be32                     group;                /*     4     4 */
	__u8                       qrv:3;                /*     8: 0  1 */
	__u8                       suppress:1;           /*     8: 3  1 */
	__u8                       resv:4;               /*     8: 4  1 */
	__u8                       qqic;                 /*     9     1 */
	__be16                     nsrcs;                /*    10     2 */
	__be32                     srcs[];               /*    12     0 */

	/* size: 12, cachelines: 1, members: 10 */
	/* last cacheline: 12 bytes */
};
struct igmp_mc_iter_state {
	struct seq_net_private     p;                    /*     0     8 */
	struct net_device *        dev;                  /*     8     8 */
	struct in_device *         in_dev;               /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct igmp_mcf_iter_state {
	struct seq_net_private     p;                    /*     0     8 */
	struct net_device *        dev;                  /*     8     8 */
	struct in_device *         idev;                 /*    16     8 */
	struct ip_mc_list *        im;                   /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct fib_config {
	u8                         fc_dst_len;           /*     0     1 */
	u8                         fc_tos;               /*     1     1 */
	u8                         fc_protocol;          /*     2     1 */
	u8                         fc_scope;             /*     3     1 */
	u8                         fc_type;              /*     4     1 */
	u8                         fc_gw_family;         /*     5     1 */

	/* XXX 2 bytes hole, try to pack */

	u32                        fc_table;             /*     8     4 */
	__be32                     fc_dst;               /*    12     4 */
	union {
		__be32             fc_gw4;               /*    16     4 */
		struct in6_addr    fc_gw6;               /*    16    16 */
	};                                               /*    16    16 */
	int                        fc_oif;               /*    32     4 */
	u32                        fc_flags;             /*    36     4 */
	u32                        fc_priority;          /*    40     4 */
	__be32                     fc_prefsrc;           /*    44     4 */
	u32                        fc_nh_id;             /*    48     4 */

	/* XXX 4 bytes hole, try to pack */

	struct nlattr *            fc_mx;                /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct rtnexthop *         fc_mp;                /*    64     8 */
	int                        fc_mx_len;            /*    72     4 */
	int                        fc_mp_len;            /*    76     4 */
	u32                        fc_flow;              /*    80     4 */
	u32                        fc_nlflags;           /*    84     4 */
	struct nl_info             fc_nlinfo;            /*    88    24 */

	/* XXX last struct has 3 bytes of padding */

	struct nlattr *            fc_encap;             /*   112     8 */
	u16                        fc_encap_type;        /*   120     2 */

	/* size: 128, cachelines: 2, members: 23 */
	/* sum members: 116, holes: 2, sum holes: 6 */
	/* padding: 6 */
	/* paddings: 1, sum paddings: 3 */
};
struct fib_result_nl {
	__be32                     fl_addr;              /*     0     4 */
	u32                        fl_mark;              /*     4     4 */
	unsigned char              fl_tos;               /*     8     1 */
	unsigned char              fl_scope;             /*     9     1 */
	unsigned char              tb_id_in;             /*    10     1 */
	unsigned char              tb_id;                /*    11     1 */
	unsigned char              prefixlen;            /*    12     1 */
	unsigned char              nh_sel;               /*    13     1 */
	unsigned char              type;                 /*    14     1 */
	unsigned char              scope;                /*    15     1 */
	int                        err;                  /*    16     4 */

	/* size: 20, cachelines: 1, members: 11 */
	/* last cacheline: 20 bytes */
};
struct fib_dump_filter {
	u32                        table_id;             /*     0     4 */
	bool                       filter_set;           /*     4     1 */
	bool                       dump_routes;          /*     5     1 */
	bool                       dump_exceptions;      /*     6     1 */
	unsigned char              protocol;             /*     7     1 */
	unsigned char              rt_type;              /*     8     1 */

	/* XXX 3 bytes hole, try to pack */

	unsigned int               flags;                /*    12     4 */
	struct net_device *        dev;                  /*    16     8 */

	/* size: 24, cachelines: 1, members: 8 */
	/* sum members: 21, holes: 1, sum holes: 3 */
	/* last cacheline: 24 bytes */
};
struct fib_nh_notifier_info {
	struct fib_notifier_info   info;                 /*     0    16 */
	struct fib_nh *            fib_nh;               /*    16     8 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct fib_entry_notifier_info {
	struct fib_notifier_info   info;                 /*     0    16 */
	u32                        dst;                  /*    16     4 */
	int                        dst_len;              /*    20     4 */
	struct fib_info *          fi;                   /*    24     8 */
	u8                         tos;                  /*    32     1 */
	u8                         type;                 /*    33     1 */

	/* XXX 2 bytes hole, try to pack */

	u32                        tb_id;                /*    36     4 */

	/* size: 40, cachelines: 1, members: 7 */
	/* sum members: 38, holes: 1, sum holes: 2 */
	/* last cacheline: 40 bytes */
};
struct key_vector {
	t_key                      key;                  /*     0     4 */
	unsigned char              pos;                  /*     4     1 */
	unsigned char              bits;                 /*     5     1 */
	unsigned char              slen;                 /*     6     1 */

	/* XXX 1 byte hole, try to pack */

	union {
		struct hlist_head  leaf;                 /*     8     8 */
		struct key_vector * tnode[0];            /*     8     0 */
	};                                               /*     8     8 */

	/* size: 16, cachelines: 1, members: 5 */
	/* sum members: 15, holes: 1, sum holes: 1 */
	/* last cacheline: 16 bytes */
};
struct tnode {
	struct callback_head       rcu __attribute__((__aligned__(8))); /*     0    16 */
	t_key                      empty_children;       /*    16     4 */
	t_key                      full_children;        /*    20     4 */
	struct key_vector *        parent;               /*    24     8 */
	struct key_vector          kv[1];                /*    32    16 */

	/* size: 48, cachelines: 1, members: 5 */
	/* forced alignments: 1 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct trie_use_stats {
	unsigned int               gets;                 /*     0     4 */
	unsigned int               backtrack;            /*     4     4 */
	unsigned int               semantic_match_passed; /*     8     4 */
	unsigned int               semantic_match_miss;  /*    12     4 */
	unsigned int               null_node_hit;        /*    16     4 */
	unsigned int               resize_node_skipped;  /*    20     4 */

	/* size: 24, cachelines: 1, members: 6 */
	/* last cacheline: 24 bytes */
};
struct trie_stat {
	unsigned int               totdepth;             /*     0     4 */
	unsigned int               maxdepth;             /*     4     4 */
	unsigned int               tnodes;               /*     8     4 */
	unsigned int               leaves;               /*    12     4 */
	unsigned int               nullpointers;         /*    16     4 */
	unsigned int               prefixes;             /*    20     4 */
	unsigned int               nodesizes[32];        /*    24   128 */

	/* size: 152, cachelines: 3, members: 7 */
	/* last cacheline: 24 bytes */
};
struct trie {
	struct key_vector          kv[1];                /*     0    16 */
	struct trie_use_stats *    stats;                /*    16     8 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct fib_trie_iter {
	struct seq_net_private     p;                    /*     0     8 */
	struct fib_table *         tb;                   /*     8     8 */
	struct key_vector *        tnode;                /*    16     8 */
	unsigned int               index;                /*    24     4 */
	unsigned int               depth;                /*    28     4 */

	/* size: 32, cachelines: 1, members: 5 */
	/* last cacheline: 32 bytes */
};
struct fib_route_iter {
	struct seq_net_private     p;                    /*     0     8 */
	struct fib_table *         main_tb;              /*     8     8 */
	struct key_vector *        tnode;                /*    16     8 */
	loff_t                     pos;                  /*    24     8 */
	t_key                      key;                  /*    32     4 */

	/* size: 40, cachelines: 1, members: 5 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct ipfrag_skb_cb {
	union {
		struct inet_skb_parm h4;                 /*     0    24 */
		struct inet6_skb_parm h6;                /*     0    24 */
	};                                               /*     0    24 */
	struct sk_buff *           next_frag;            /*    24     8 */
	int                        frag_run_len;         /*    32     4 */

	/* size: 40, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct icmpv6_echo {
	__be16                     identifier;           /*     0     2 */
	__be16                     sequence;             /*     2     2 */

	/* size: 4, cachelines: 1, members: 2 */
	/* last cacheline: 4 bytes */
};
struct icmpv6_nd_advt {
	__u32                      reserved:5;           /*     0: 0  4 */
	__u32                      override:1;           /*     0: 5  4 */
	__u32                      solicited:1;          /*     0: 6  4 */
	__u32                      router:1;             /*     0: 7  4 */
	__u32                      reserved2:24;         /*     0: 8  4 */

	/* size: 4, cachelines: 1, members: 5 */
	/* last cacheline: 4 bytes */
};
struct icmpv6_nd_ra {
	__u8                       hop_limit;            /*     0     1 */
	__u8                       reserved:3;           /*     1: 0  1 */
	__u8                       router_pref:2;        /*     1: 3  1 */
	__u8                       home_agent:1;         /*     1: 5  1 */
	__u8                       other:1;              /*     1: 6  1 */
	__u8                       managed:1;            /*     1: 7  1 */
	__be16                     rt_lifetime;          /*     2     2 */

	/* size: 4, cachelines: 1, members: 7 */
	/* last cacheline: 4 bytes */
};
struct icmp6hdr {
	__u8                       icmp6_type;           /*     0     1 */
	__u8                       icmp6_code;           /*     1     1 */
	__sum16                    icmp6_cksum;          /*     2     2 */
	union {
		__be32             un_data32[1];         /*     4     4 */
		__be16             un_data16[2];         /*     4     4 */
		__u8               un_data8[4];          /*     4     4 */
		struct icmpv6_echo u_echo;               /*     4     4 */
		struct icmpv6_nd_advt u_nd_advt;         /*     4     4 */
		struct icmpv6_nd_ra u_nd_ra;             /*     4     4 */
	} icmp6_dataun;                                  /*     4     4 */

	/* size: 8, cachelines: 1, members: 4 */
	/* last cacheline: 8 bytes */
};
struct ping_iter_state {
	struct seq_net_private     p;                    /*     0     8 */
	int                        bucket;               /*     8     4 */
	sa_family_t                family;               /*    12     2 */

	/* size: 16, cachelines: 1, members: 3 */
	/* padding: 2 */
	/* last cacheline: 16 bytes */
};
struct pingfakehdr {
	struct icmphdr             icmph;                /*     0     8 */
	struct msghdr *            msg;                  /*     8     8 */
	sa_family_t                family;               /*    16     2 */

	/* XXX 2 bytes hole, try to pack */

	__wsum                     wcheck;               /*    20     4 */

	/* size: 24, cachelines: 1, members: 4 */
	/* sum members: 22, holes: 1, sum holes: 2 */
	/* last cacheline: 24 bytes */
};
struct ping_table {
	struct hlist_nulls_head    hash[64];             /*     0   512 */
	/* --- cacheline 8 boundary (512 bytes) --- */
	rwlock_t                   lock;                 /*   512     8 */

	/* size: 520, cachelines: 9, members: 2 */
	/* last cacheline: 8 bytes */
};
struct ip6_tnl_encap_ops {
	size_t                     (*encap_hlen)(struct ip_tunnel_encap *); /*     0     8 */
	int                        (*build_header)(struct sk_buff *, struct ip_tunnel_encap *, u8 *, struct flowi6 *); /*     8     8 */
	int                        (*err_handler)(struct sk_buff *, struct inet6_skb_parm *, u8, u8, int, __be32); /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct geneve_opt {
	__be16                     opt_class;            /*     0     2 */
	u8                         type;                 /*     2     1 */
	u8                         length:5;             /*     3: 0  1 */
	u8                         r3:1;                 /*     3: 5  1 */
	u8                         r2:1;                 /*     3: 6  1 */
	u8                         r1:1;                 /*     3: 7  1 */
	u8                         opt_data[];           /*     4     0 */

	/* size: 4, cachelines: 1, members: 7 */
	/* last cacheline: 4 bytes */
};
struct vxlan_metadata {
	u32                        gbp;                  /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct erspan_md2 {
	__be32                     timestamp;            /*     0     4 */
	__be16                     sgt;                  /*     4     2 */
	__u8                       hwid_upper:2;         /*     6: 0  1 */
	__u8                       ft:5;                 /*     6: 2  1 */
	__u8                       p:1;                  /*     6: 7  1 */
	__u8                       o:1;                  /*     7: 0  1 */
	__u8                       gra:2;                /*     7: 1  1 */
	__u8                       dir:1;                /*     7: 3  1 */
	__u8                       hwid:4;               /*     7: 4  1 */

	/* size: 8, cachelines: 1, members: 9 */
	/* last cacheline: 8 bytes */
};
struct erspan_metadata {
	int                        version;              /*     0     4 */
	union {
		__be32             index;                /*     4     4 */
		struct erspan_md2  md2;                  /*     4     8 */
	} u;                                             /*     4     8 */

	/* size: 12, cachelines: 1, members: 2 */
	/* last cacheline: 12 bytes */
};
struct nhmsg {
	unsigned char              nh_family;            /*     0     1 */
	unsigned char              nh_scope;             /*     1     1 */
	unsigned char              nh_protocol;          /*     2     1 */
	unsigned char              resvd;                /*     3     1 */
	unsigned int               nh_flags;             /*     4     4 */

	/* size: 8, cachelines: 1, members: 5 */
	/* last cacheline: 8 bytes */
};
struct nexthop_grp {
	__u32                      id;                   /*     0     4 */
	__u8                       weight;               /*     4     1 */
	__u8                       resvd1;               /*     5     1 */
	__u16                      resvd2;               /*     6     2 */

	/* size: 8, cachelines: 1, members: 4 */
	/* last cacheline: 8 bytes */
};
struct nh_config {
	u32                        nh_id;                /*     0     4 */
	u8                         nh_family;            /*     4     1 */
	u8                         nh_protocol;          /*     5     1 */
	u8                         nh_blackhole;         /*     6     1 */
	u8                         nh_fdb;               /*     7     1 */
	u32                        nh_flags;             /*     8     4 */
	int                        nh_ifindex;           /*    12     4 */
	struct net_device *        dev;                  /*    16     8 */
	union {
		__be32             ipv4;                 /*    24     4 */
		struct in6_addr    ipv6;                 /*    24    16 */
	} gw;                                            /*    24    16 */
	struct nlattr *            nh_grp;               /*    40     8 */
	u16                        nh_grp_type;          /*    48     2 */

	/* XXX 6 bytes hole, try to pack */

	struct nlattr *            nh_encap;             /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u16                        nh_encap_type;        /*    64     2 */

	/* XXX 2 bytes hole, try to pack */

	u32                        nlflags;              /*    68     4 */
	struct nl_info             nlinfo;               /*    72    24 */

	/* XXX last struct has 3 bytes of padding */

	/* size: 96, cachelines: 2, members: 15 */
	/* sum members: 88, holes: 2, sum holes: 8 */
	/* paddings: 1, sum paddings: 3 */
	/* last cacheline: 32 bytes */
};
struct inet6_protocol {
	void                       (*early_demux)(struct sk_buff *); /*     0     8 */
	void                       (*early_demux_handler)(struct sk_buff *); /*     8     8 */
	int                        (*handler)(struct sk_buff *); /*    16     8 */
	int                        (*err_handler)(struct sk_buff *, struct inet6_skb_parm *, u8, u8, int, __be32); /*    24     8 */
	unsigned int               flags;                /*    32     4 */

	/* size: 40, cachelines: 1, members: 5 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct snmp_mib {
	const char  *              name;                 /*     0     8 */
	int                        entry;                /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct fib4_rule {
	struct fib_rule            common __attribute__((__aligned__(8))); /*     0   216 */
	/* --- cacheline 3 boundary (192 bytes) was 24 bytes ago --- */
	u8                         dst_len;              /*   216     1 */
	u8                         src_len;              /*   217     1 */
	u8                         tos;                  /*   218     1 */

	/* XXX 1 byte hole, try to pack */

	__be32                     src;                  /*   220     4 */
	__be32                     srcmask;              /*   224     4 */
	__be32                     dst;                  /*   228     4 */
	__be32                     dstmask;              /*   232     4 */
	u32                        tclassid;             /*   236     4 */

	/* size: 240, cachelines: 4, members: 9 */
	/* sum members: 239, holes: 1, sum holes: 1 */
	/* forced alignments: 1 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct pimreghdr {
	__u8                       type;                 /*     0     1 */
	__u8                       reserved;             /*     1     1 */
	__be16                     csum;                 /*     2     2 */
	__be32                     flags;                /*     4     4 */

	/* size: 8, cachelines: 1, members: 4 */
	/* last cacheline: 8 bytes */
};
struct vifctl {
	vifi_t                     vifc_vifi;            /*     0     2 */
	unsigned char              vifc_flags;           /*     2     1 */
	unsigned char              vifc_threshold;       /*     3     1 */
	unsigned int               vifc_rate_limit;      /*     4     4 */
	union {
		struct in_addr     vifc_lcl_addr;        /*     8     4 */
		int                vifc_lcl_ifindex;     /*     8     4 */
	};                                               /*     8     4 */
	struct in_addr             vifc_rmt_addr;        /*    12     4 */

	/* size: 16, cachelines: 1, members: 6 */
	/* last cacheline: 16 bytes */
};
struct mfcctl {
	struct in_addr             mfcc_origin;          /*     0     4 */
	struct in_addr             mfcc_mcastgrp;        /*     4     4 */
	vifi_t                     mfcc_parent;          /*     8     2 */
	unsigned char              mfcc_ttls[32];        /*    10    32 */

	/* XXX 2 bytes hole, try to pack */

	unsigned int               mfcc_pkt_cnt;         /*    44     4 */
	unsigned int               mfcc_byte_cnt;        /*    48     4 */
	unsigned int               mfcc_wrong_if;        /*    52     4 */
	int                        mfcc_expire;          /*    56     4 */

	/* size: 60, cachelines: 1, members: 8 */
	/* sum members: 58, holes: 1, sum holes: 2 */
	/* last cacheline: 60 bytes */
};
struct sioc_sg_req {
	struct in_addr             src;                  /*     0     4 */
	struct in_addr             grp;                  /*     4     4 */
	long unsigned int          pktcnt;               /*     8     8 */
	long unsigned int          bytecnt;              /*    16     8 */
	long unsigned int          wrong_if;             /*    24     8 */

	/* size: 32, cachelines: 1, members: 5 */
	/* last cacheline: 32 bytes */
};
struct sioc_vif_req {
	vifi_t                     vifi;                 /*     0     2 */

	/* XXX 6 bytes hole, try to pack */

	long unsigned int          icount;               /*     8     8 */
	long unsigned int          ocount;               /*    16     8 */
	long unsigned int          ibytes;               /*    24     8 */
	long unsigned int          obytes;               /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* sum members: 34, holes: 1, sum holes: 6 */
	/* last cacheline: 40 bytes */
};
struct igmpmsg {
	__u32                      unused1;              /*     0     4 */
	__u32                      unused2;              /*     4     4 */
	unsigned char              im_msgtype;           /*     8     1 */
	unsigned char              im_mbz;               /*     9     1 */
	unsigned char              im_vif;               /*    10     1 */
	unsigned char              im_vif_hi;            /*    11     1 */
	struct in_addr             im_src;               /*    12     4 */
	struct in_addr             im_dst;               /*    16     4 */

	/* size: 20, cachelines: 1, members: 8 */
	/* last cacheline: 20 bytes */
};
struct vif_device {
	struct net_device *        dev;                  /*     0     8 */
	long unsigned int          bytes_in;             /*     8     8 */
	long unsigned int          bytes_out;            /*    16     8 */
	long unsigned int          pkt_in;               /*    24     8 */
	long unsigned int          pkt_out;              /*    32     8 */
	long unsigned int          rate_limit;           /*    40     8 */
	unsigned char              threshold;            /*    48     1 */

	/* XXX 1 byte hole, try to pack */

	short unsigned int         flags;                /*    50     2 */
	int                        link;                 /*    52     4 */
	struct netdev_phys_item_id dev_parent_id;        /*    56    33 */

	/* XXX 3 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) was 28 bytes ago --- */
	__be32                     local;                /*    92     4 */
	__be32                     remote;               /*    96     4 */

	/* size: 104, cachelines: 2, members: 12 */
	/* sum members: 96, holes: 2, sum holes: 4 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct vif_entry_notifier_info {
	struct fib_notifier_info   info;                 /*     0    16 */
	struct net_device *        dev;                  /*    16     8 */
	short unsigned int         vif_index;            /*    24     2 */
	short unsigned int         vif_flags;            /*    26     2 */
	u32                        tb_id;                /*    28     4 */

	/* size: 32, cachelines: 1, members: 5 */
	/* last cacheline: 32 bytes */
};
struct mr_mfc {
	struct rhlist_head         mnode;                /*     0    16 */
	short unsigned int         mfc_parent;           /*    16     2 */

	/* XXX 2 bytes hole, try to pack */

	int                        mfc_flags;            /*    20     4 */
	union {
		struct {
			long unsigned int expires;       /*    24     8 */
			struct sk_buff_head unresolved;  /*    32    24 */
		} unres;                                 /*    24    32 */
		struct {
			long unsigned int last_assert;   /*    24     8 */
			int        minvif;               /*    32     4 */
			int        maxvif;               /*    36     4 */
			long unsigned int bytes;         /*    40     8 */
			long unsigned int pkt;           /*    48     8 */
			long unsigned int wrong_if;      /*    56     8 */
			/* --- cacheline 1 boundary (64 bytes) --- */
			long unsigned int lastuse;       /*    64     8 */
			unsigned char ttls[32];          /*    72    32 */
			refcount_t refcount;             /*   104     4 */
		} res;                                   /*    24    88 */
	} mfc_un;                                        /*    24    88 */
	/* --- cacheline 1 boundary (64 bytes) was 48 bytes ago --- */
	struct list_head           list;                 /*   112    16 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*   128    16 */
	void                       (*free)(struct callback_head *); /*   144     8 */

	/* size: 152, cachelines: 3, members: 7 */
	/* sum members: 150, holes: 1, sum holes: 2 */
	/* forced alignments: 1 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct mfc_entry_notifier_info {
	struct fib_notifier_info   info;                 /*     0    16 */
	struct mr_mfc *            mfc;                  /*    16     8 */
	u32                        tb_id;                /*    24     4 */

	/* size: 32, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct mr_table_ops {
	const struct rhashtable_params  * rht_params;    /*     0     8 */
	void *                     cmparg_any;           /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct mr_table {
	struct list_head           list;                 /*     0    16 */
	possible_net_t             net;                  /*    16     8 */
	struct mr_table_ops        ops;                  /*    24    16 */
	u32                        id;                   /*    40     4 */

	/* XXX 4 bytes hole, try to pack */

	struct sock *              mroute_sk;            /*    48     8 */
	struct timer_list          ipmr_expire_timer;    /*    56    72 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	struct list_head           mfc_unres_queue;      /*   128    16 */
	struct vif_device          vif_table[32];        /*   144  3328 */
	/* --- cacheline 54 boundary (3456 bytes) was 16 bytes ago --- */
	struct rhltable            mfc_hash;             /*  3472   168 */
	/* --- cacheline 56 boundary (3584 bytes) was 56 bytes ago --- */
	struct list_head           mfc_cache_list;       /*  3640    16 */
	/* --- cacheline 57 boundary (3648 bytes) was 8 bytes ago --- */
	int                        maxvif;               /*  3656     4 */
	atomic_t                   cache_resolve_queue_len; /*  3660     4 */
	bool                       mroute_do_assert;     /*  3664     1 */
	bool                       mroute_do_pim;        /*  3665     1 */
	bool                       mroute_do_wrvifwhole; /*  3666     1 */

	/* XXX 1 byte hole, try to pack */

	int                        mroute_reg_vif_num;   /*  3668     4 */

	/* size: 3672, cachelines: 58, members: 16 */
	/* sum members: 3667, holes: 2, sum holes: 5 */
	/* last cacheline: 24 bytes */
};
struct mr_vif_iter {
	struct seq_net_private     p;                    /*     0     8 */
	struct mr_table *          mrt;                  /*     8     8 */
	int                        ct;                   /*    16     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct mr_mfc_iter {
	struct seq_net_private     p;                    /*     0     8 */
	struct mr_table *          mrt;                  /*     8     8 */
	struct list_head *         cache;                /*    16     8 */
	spinlock_t *               lock;                 /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct mfc_cache_cmp_arg {
	__be32                     mfc_mcastgrp;         /*     0     4 */
	__be32                     mfc_origin;           /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct mfc_cache {
	struct mr_mfc              _c __attribute__((__aligned__(8))); /*     0   152 */
	/* --- cacheline 2 boundary (128 bytes) was 24 bytes ago --- */
	union {
		struct {
			__be32     mfc_mcastgrp;         /*   152     4 */
			__be32     mfc_origin;           /*   156     4 */
		};                                       /*   152     8 */
		struct mfc_cache_cmp_arg cmparg;         /*   152     8 */
	};                                               /*   152     8 */

	/* size: 160, cachelines: 3, members: 2 */
	/* forced alignments: 1 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct ipmr_result {
	struct mr_table *          mrt;                  /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct compat_sioc_sg_req {
	struct in_addr             src;                  /*     0     4 */
	struct in_addr             grp;                  /*     4     4 */
	compat_ulong_t             pktcnt;               /*     8     4 */
	compat_ulong_t             bytecnt;              /*    12     4 */
	compat_ulong_t             wrong_if;             /*    16     4 */

	/* size: 20, cachelines: 1, members: 5 */
	/* last cacheline: 20 bytes */
};
struct compat_sioc_vif_req {
	vifi_t                     vifi;                 /*     0     2 */

	/* XXX 2 bytes hole, try to pack */

	compat_ulong_t             icount;               /*     4     4 */
	compat_ulong_t             ocount;               /*     8     4 */
	compat_ulong_t             ibytes;               /*    12     4 */
	compat_ulong_t             obytes;               /*    16     4 */

	/* size: 20, cachelines: 1, members: 5 */
	/* sum members: 18, holes: 1, sum holes: 2 */
	/* last cacheline: 20 bytes */
};
struct rta_mfc_stats {
	__u64                      mfcs_packets;         /*     0     8 */
	__u64                      mfcs_bytes;           /*     8     8 */
	__u64                      mfcs_wrong_if;        /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct bictcp {
	u32                        cnt;                  /*     0     4 */
	u32                        last_max_cwnd;        /*     4     4 */
	u32                        last_cwnd;            /*     8     4 */
	u32                        last_time;            /*    12     4 */
	u32                        bic_origin_point;     /*    16     4 */
	u32                        bic_K;                /*    20     4 */
	u32                        delay_min;            /*    24     4 */
	u32                        epoch_start;          /*    28     4 */
	u32                        ack_cnt;              /*    32     4 */
	u32                        tcp_cwnd;             /*    36     4 */
	u16                        unused;               /*    40     2 */
	u8                         sample_cnt;           /*    42     1 */
	u8                         found;                /*    43     1 */
	u32                        round_start;          /*    44     4 */
	u32                        end_seq;              /*    48     4 */
	u32                        last_ack;             /*    52     4 */
	u32                        curr_rtt;             /*    56     4 */

	/* size: 60, cachelines: 1, members: 17 */
	/* last cacheline: 60 bytes */
};
struct tls_rec {
	struct list_head           list;                 /*     0    16 */
	int                        tx_ready;             /*    16     4 */
	int                        tx_flags;             /*    20     4 */
	struct sk_msg              msg_plaintext;        /*    24   712 */
	/* --- cacheline 11 boundary (704 bytes) was 32 bytes ago --- */
	struct sk_msg              msg_encrypted;        /*   736   712 */
	/* --- cacheline 22 boundary (1408 bytes) was 40 bytes ago --- */
	struct scatterlist         sg_aead_in[2];        /*  1448    64 */
	/* --- cacheline 23 boundary (1472 bytes) was 40 bytes ago --- */
	struct scatterlist         sg_aead_out[2];       /*  1512    64 */
	/* --- cacheline 24 boundary (1536 bytes) was 40 bytes ago --- */
	char                       content_type;         /*  1576     1 */

	/* XXX 7 bytes hole, try to pack */

	struct scatterlist         sg_content_type;      /*  1584    32 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 25 boundary (1600 bytes) was 16 bytes ago --- */
	char                       aad_space[13];        /*  1616    13 */
	u8                         iv_data[16];          /*  1629    16 */

	/* XXX 3 bytes hole, try to pack */

	struct aead_request        aead_req __attribute__((__aligned__(8))); /*  1648    80 */
	/* --- cacheline 27 boundary (1728 bytes) --- */
	u8                         aead_req_ctx[];       /*  1728     0 */

	/* size: 1728, cachelines: 27, members: 13 */
	/* sum members: 1718, holes: 2, sum holes: 10 */
	/* paddings: 1, sum paddings: 4 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 3 */
} __attribute__((__aligned__(8)));
struct tx_work {
	struct delayed_work        work;                 /*     0   192 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	struct sock *              sk;                   /*   192     8 */

	/* size: 200, cachelines: 4, members: 2 */
	/* last cacheline: 8 bytes */
};
struct tls_sw_context_tx {
	struct crypto_aead *       aead_send;            /*     0     8 */
	struct crypto_wait         async_wait;           /*     8    40 */

	/* XXX last struct has 4 bytes of padding */

	struct tx_work             tx_work;              /*    48   200 */
	/* --- cacheline 3 boundary (192 bytes) was 56 bytes ago --- */
	struct tls_rec *           open_rec;             /*   248     8 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	struct list_head           tx_list;              /*   256    16 */
	atomic_t                   encrypt_pending;      /*   272     4 */
	spinlock_t                 encrypt_compl_lock;   /*   276     4 */
	int                        async_notify;         /*   280     4 */
	u8                         async_capable:1;      /*   284: 0  1 */

	/* XXX 7 bits hole, try to pack */
	/* XXX 3 bytes hole, try to pack */

	long unsigned int          tx_bitmask;           /*   288     8 */

	/* size: 296, cachelines: 5, members: 10 */
	/* sum members: 292, holes: 1, sum holes: 3 */
	/* sum bitfield members: 1 bits, bit holes: 1, sum bit holes: 7 bits */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 40 bytes */
};
struct netlbl_audit {
	u32                        secid;                /*     0     4 */
	kuid_t                     loginuid;             /*     4     4 */
	unsigned int               sessionid;            /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct cipso_v4_std_map_tbl {
	struct {
		u32 *              cipso;                /*     0     8 */
		u32 *              local;                /*     8     8 */
		u32                cipso_size;           /*    16     4 */
		u32                local_size;           /*    20     4 */
	} lvl;                                           /*     0    24 */
	struct {
		u32 *              cipso;                /*    24     8 */
		u32 *              local;                /*    32     8 */
		u32                cipso_size;           /*    40     4 */
		u32                local_size;           /*    44     4 */
	} cat;                                           /*    24    24 */

	/* size: 48, cachelines: 1, members: 2 */
	/* last cacheline: 48 bytes */
};
struct cipso_v4_doi {
	u32                        doi;                  /*     0     4 */
	u32                        type;                 /*     4     4 */
	union {
		struct cipso_v4_std_map_tbl * std;       /*     8     8 */
	} map;                                           /*     8     8 */
	u8                         tags[5];              /*    16     5 */

	/* XXX 3 bytes hole, try to pack */

	refcount_t                 refcount;             /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           list;                 /*    32    16 */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*    48    16 */

	/* size: 64, cachelines: 1, members: 7 */
	/* sum members: 57, holes: 2, sum holes: 7 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(8)));
struct cipso_v4_map_cache_bkt {
	spinlock_t                 lock;                 /*     0     4 */
	u32                        size;                 /*     4     4 */
	struct list_head           list;                 /*     8    16 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct cipso_v4_map_cache_entry {
	u32                        hash;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	unsigned char *            key;                  /*     8     8 */
	size_t                     key_len;              /*    16     8 */
	struct netlbl_lsm_cache *  lsm_data;             /*    24     8 */
	u32                        activity;             /*    32     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           list;                 /*    40    16 */

	/* size: 56, cachelines: 1, members: 6 */
	/* sum members: 48, holes: 2, sum holes: 8 */
	/* last cacheline: 56 bytes */
};
struct tcp_comp_context_tx {
	ZSTD_CStream *             cstream;              /*     0     8 */
	void *                     cworkspace;           /*     8     8 */
	void *                     plaintext_data;       /*    16     8 */
	void *                     compressed_data;      /*    24     8 */
	struct sk_msg              msg;                  /*    32   712 */
	/* --- cacheline 11 boundary (704 bytes) was 40 bytes ago --- */
	bool                       in_tcp_sendpages;     /*   744     1 */

	/* size: 752, cachelines: 12, members: 6 */
	/* padding: 7 */
	/* last cacheline: 48 bytes */
};
struct tcp_comp_context_rx {
	ZSTD_DStream *             dstream;              /*     0     8 */
	void *                     dworkspace;           /*     8     8 */
	void *                     plaintext_data;       /*    16     8 */
	struct strparser           strp;                 /*    24   384 */
	/* --- cacheline 6 boundary (384 bytes) was 24 bytes ago --- */
	void                       (*saved_data_ready)(struct sock *); /*   408     8 */
	struct sk_buff *           pkt;                  /*   416     8 */
	struct sk_buff *           dpkt;                 /*   424     8 */

	/* size: 432, cachelines: 7, members: 7 */
	/* last cacheline: 48 bytes */
};
struct tcp_comp_context {
	struct callback_head       rcu __attribute__((__aligned__(8))); /*     0    16 */
	struct proto *             sk_proto;             /*    16     8 */
	void                       (*sk_write_space)(struct sock *); /*    24     8 */
	struct tcp_comp_context_tx tx;                   /*    32   752 */

	/* XXX last struct has 7 bytes of padding */

	/* --- cacheline 12 boundary (768 bytes) was 16 bytes ago --- */
	struct tcp_comp_context_rx rx;                   /*   784   432 */
	/* --- cacheline 19 boundary (1216 bytes) --- */
	long unsigned int          flags;                /*  1216     8 */

	/* size: 1224, cachelines: 20, members: 6 */
	/* paddings: 1, sum paddings: 7 */
	/* forced alignments: 1 */
	/* last cacheline: 8 bytes */
} __attribute__((__aligned__(8)));
struct xfrm_policy_afinfo {
	struct dst_ops *           dst_ops;              /*     0     8 */
	struct dst_entry *         (*dst_lookup)(struct net *, int, int, const xfrm_address_t  *, const xfrm_address_t  *, u32); /*     8     8 */
	int                        (*get_saddr)(struct net *, int, xfrm_address_t *, xfrm_address_t *, u32); /*    16     8 */
	int                        (*fill_dst)(struct xfrm_dst *, struct net_device *, const struct flowi  *); /*    24     8 */
	struct dst_entry *         (*blackhole_route)(struct net *, struct dst_entry *); /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct xfrm_state_afinfo {
	u8                         family;               /*     0     1 */
	u8                         proto;                /*     1     1 */

	/* XXX 6 bytes hole, try to pack */

	const struct xfrm_type_offload  * type_offload_esp; /*     8     8 */
	const struct xfrm_type  *  type_esp;             /*    16     8 */
	const struct xfrm_type  *  type_ipip;            /*    24     8 */
	const struct xfrm_type  *  type_ipip6;           /*    32     8 */
	const struct xfrm_type  *  type_comp;            /*    40     8 */
	const struct xfrm_type  *  type_ah;              /*    48     8 */
	const struct xfrm_type  *  type_routing;         /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	const struct xfrm_type  *  type_dstopts;         /*    64     8 */
	int                        (*output)(struct net *, struct sock *, struct sk_buff *); /*    72     8 */
	int                        (*transport_finish)(struct sk_buff *, int); /*    80     8 */
	void                       (*local_error)(struct sk_buff *, u32); /*    88     8 */

	/* size: 96, cachelines: 2, members: 13 */
	/* sum members: 90, holes: 1, sum holes: 6 */
	/* last cacheline: 32 bytes */
};
struct xfrm_tunnel_skb_cb {
	union {
		struct inet_skb_parm h4;                 /*     0    24 */
		struct inet6_skb_parm h6;                /*     0    24 */
	} header;                                        /*     0    24 */
	union {
		struct ip_tunnel * ip4;                  /*    24     8 */
		struct ip6_tnl *   ip6;                  /*    24     8 */
	} tunnel;                                        /*    24     8 */

	/* size: 32, cachelines: 1, members: 2 */
	/* last cacheline: 32 bytes */
};
struct xfrm_mode_skb_cb {
	struct xfrm_tunnel_skb_cb  header;               /*     0    32 */
	__be16                     id;                   /*    32     2 */
	__be16                     frag_off;             /*    34     2 */
	u8                         ihl;                  /*    36     1 */
	u8                         tos;                  /*    37     1 */
	u8                         ttl;                  /*    38     1 */
	u8                         protocol;             /*    39     1 */
	u8                         optlen;               /*    40     1 */
	u8                         flow_lbl[3];          /*    41     3 */

	/* size: 48, cachelines: 1, members: 9 */
	/* padding: 4 */
	/* last cacheline: 48 bytes */
};
struct xfrm_spi_skb_cb {
	struct xfrm_tunnel_skb_cb  header;               /*     0    32 */
	unsigned int               daddroff;             /*    32     4 */
	unsigned int               family;               /*    36     4 */
	__be32                     seq;                  /*    40     4 */

	/* size: 48, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 48 bytes */
};
struct xfrm_input_afinfo {
	u8                         family;               /*     0     1 */
	bool                       is_ipip;              /*     1     1 */

	/* XXX 6 bytes hole, try to pack */

	int                        (*callback)(struct sk_buff *, u8, int); /*     8     8 */

	/* size: 16, cachelines: 1, members: 3 */
	/* sum members: 10, holes: 1, sum holes: 6 */
	/* last cacheline: 16 bytes */
};
struct xfrm4_protocol {
	int                        (*handler)(struct sk_buff *); /*     0     8 */
	int                        (*input_handler)(struct sk_buff *, int, __be32, int); /*     8     8 */
	int                        (*cb_handler)(struct sk_buff *, int); /*    16     8 */
	int                        (*err_handler)(struct sk_buff *, u32); /*    24     8 */
	struct xfrm4_protocol *    next;                 /*    32     8 */
	int                        priority;             /*    40     4 */

	/* size: 48, cachelines: 1, members: 6 */
	/* padding: 4 */
	/* last cacheline: 48 bytes */
};
struct xfrm_if_cb {
	struct xfrm_if *           (*decode_session)(struct sk_buff *, short unsigned int); /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct xfrm_if {
	struct xfrm_if *           next;                 /*     0     8 */
	struct net_device *        dev;                  /*     8     8 */
	struct net *               net;                  /*    16     8 */
	struct xfrm_if_parms       p;                    /*    24     8 */
	struct gro_cells           gro_cells;            /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
struct xfrm_policy_walk {
	struct xfrm_policy_walk_entry walk;              /*     0    24 */

	/* XXX last struct has 7 bytes of padding */

	u8                         type;                 /*    24     1 */

	/* XXX 3 bytes hole, try to pack */

	u32                        seq;                  /*    28     4 */

	/* size: 32, cachelines: 1, members: 3 */
	/* sum members: 29, holes: 1, sum holes: 3 */
	/* paddings: 1, sum paddings: 7 */
	/* last cacheline: 32 bytes */
};
struct xfrm_kmaddress {
	xfrm_address_t             local;                /*     0    16 */
	xfrm_address_t             remote;               /*    16    16 */
	u32                        reserved;             /*    32     4 */
	u16                        family;               /*    36     2 */

	/* size: 40, cachelines: 1, members: 4 */
	/* padding: 2 */
	/* last cacheline: 40 bytes */
};
struct xfrm_migrate {
	xfrm_address_t             old_daddr;            /*     0    16 */
	xfrm_address_t             old_saddr;            /*    16    16 */
	xfrm_address_t             new_daddr;            /*    32    16 */
	xfrm_address_t             new_saddr;            /*    48    16 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u8                         proto;                /*    64     1 */
	u8                         mode;                 /*    65     1 */
	u16                        reserved;             /*    66     2 */
	u32                        reqid;                /*    68     4 */
	u16                        old_family;           /*    72     2 */
	u16                        new_family;           /*    74     2 */

	/* size: 76, cachelines: 2, members: 10 */
	/* last cacheline: 12 bytes */
};
struct xfrm_if_parms {
	int                        link;                 /*     0     4 */
	u32                        if_id;                /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct xfrmk_spdinfo {
	u32                        incnt;                /*     0     4 */
	u32                        outcnt;               /*     4     4 */
	u32                        fwdcnt;               /*     8     4 */
	u32                        inscnt;               /*    12     4 */
	u32                        outscnt;              /*    16     4 */
	u32                        fwdscnt;              /*    20     4 */
	u32                        spdhcnt;              /*    24     4 */
	u32                        spdhmcnt;             /*    28     4 */

	/* size: 32, cachelines: 1, members: 8 */
	/* last cacheline: 32 bytes */
};
struct ip6_mh {
	__u8                       ip6mh_proto;          /*     0     1 */
	__u8                       ip6mh_hdrlen;         /*     1     1 */
	__u8                       ip6mh_type;           /*     2     1 */
	__u8                       ip6mh_reserved;       /*     3     1 */
	__u16                      ip6mh_cksum;          /*     4     2 */
	__u8                       data[];               /*     6     0 */

	/* size: 6, cachelines: 1, members: 6 */
	/* last cacheline: 6 bytes */
};
struct xfrm_flo {
	struct dst_entry *         dst_orig;             /*     0     8 */
	u8                         flags;                /*     8     1 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 7 */
	/* last cacheline: 16 bytes */
};
struct xfrm_pol_inexact_node {
	struct rb_node             node __attribute__((__aligned__(8))); /*     0    24 */
	union {
		xfrm_address_t     addr;                 /*    24    16 */
		struct callback_head rcu __attribute__((__aligned__(8))); /*    24    16 */
	} __attribute__((__aligned__(8)));               /*    24    16 */
	u8                         prefixlen;            /*    40     1 */

	/* XXX 7 bytes hole, try to pack */

	struct rb_root             root;                 /*    48     8 */
	struct hlist_head          hhead;                /*    56     8 */

	/* size: 64, cachelines: 1, members: 5 */
	/* sum members: 57, holes: 1, sum holes: 7 */
	/* forced alignments: 2 */
} __attribute__((__aligned__(8)));
struct xfrm_pol_inexact_key {
	possible_net_t             net;                  /*     0     8 */
	u32                        if_id;                /*     8     4 */
	u16                        family;               /*    12     2 */
	u8                         dir;                  /*    14     1 */
	u8                         type;                 /*    15     1 */

	/* size: 16, cachelines: 1, members: 5 */
	/* last cacheline: 16 bytes */
};
struct xfrm_pol_inexact_bin {
	struct xfrm_pol_inexact_key k;                   /*     0    16 */
	struct rhash_head          head;                 /*    16     8 */
	struct hlist_head          hhead;                /*    24     8 */
	seqcount_spinlock_t        count;                /*    32     4 */

	/* XXX 4 bytes hole, try to pack */

	struct rb_root             root_d;               /*    40     8 */
	struct rb_root             root_s;               /*    48     8 */
	struct list_head           inexact_bins;         /*    56    16 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*    72    16 */

	/* size: 88, cachelines: 2, members: 8 */
	/* sum members: 84, holes: 1, sum holes: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct xfrm_pol_inexact_candidates {
	struct hlist_head *        res[4];               /*     0    32 */

	/* size: 32, cachelines: 1, members: 1 */
	/* last cacheline: 32 bytes */
};
struct km_event {
	union {
		u32                hard;                 /*     0     4 */
		u32                proto;                /*     0     4 */
		u32                byid;                 /*     0     4 */
		u32                aevent;               /*     0     4 */
		u32                type;                 /*     0     4 */
	} data;                                          /*     0     4 */
	u32                        seq;                  /*     4     4 */
	u32                        portid;               /*     8     4 */
	u32                        event;                /*    12     4 */
	struct net *               net;                  /*    16     8 */

	/* size: 24, cachelines: 1, members: 5 */
	/* last cacheline: 24 bytes */
};
struct xfrm_mgr {
	struct list_head           list;                 /*     0    16 */
	int                        (*notify)(struct xfrm_state *, const struct km_event  *); /*    16     8 */
	int                        (*acquire)(struct xfrm_state *, struct xfrm_tmpl *, struct xfrm_policy *); /*    24     8 */
	struct xfrm_policy *       (*compile_policy)(struct sock *, int, u8 *, int, int *); /*    32     8 */
	int                        (*new_mapping)(struct xfrm_state *, xfrm_address_t *, __be16); /*    40     8 */
	int                        (*notify_policy)(struct xfrm_policy *, int, const struct km_event  *); /*    48     8 */
	int                        (*report)(struct net *, u8, struct xfrm_selector *, xfrm_address_t *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	int                        (*migrate)(const struct xfrm_selector  *, u8, u8, const struct xfrm_migrate  *, int, const struct xfrm_kmaddress  *, const struct xfrm_encap_tmpl  *); /*    64     8 */
	bool                       (*is_alive)(const struct km_event  *); /*    72     8 */

	/* size: 80, cachelines: 2, members: 9 */
	/* last cacheline: 16 bytes */
};
struct xfrmk_sadinfo {
	u32                        sadhcnt;              /*     0     4 */
	u32                        sadhmcnt;             /*     4     4 */
	u32                        sadcnt;               /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct xfrm_translator {
	int                        (*alloc_compat)(struct sk_buff *, const struct nlmsghdr  *); /*     0     8 */
	struct nlmsghdr *          (*rcv_msg_compat)(const struct nlmsghdr  *, int, const struct nla_policy  *, struct netlink_ext_ack *); /*     8     8 */
	int                        (*xlate_user_policy_sockptr)(u8 * *, int); /*    16     8 */
	struct module *            owner;                /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct ip_beet_phdr {
	__u8                       nexthdr;              /*     0     1 */
	__u8                       hdrlen;               /*     1     1 */
	__u8                       padlen;               /*     2     1 */
	__u8                       reserved;             /*     3     1 */

	/* size: 4, cachelines: 1, members: 4 */
	/* last cacheline: 4 bytes */
};
struct ip_tunnel {
	struct ip_tunnel *         next;                 /*     0     8 */
	struct hlist_node          hash_node;            /*     8    16 */
	struct net_device *        dev;                  /*    24     8 */
	struct net *               net;                  /*    32     8 */
	long unsigned int          err_time;             /*    40     8 */
	int                        err_count;            /*    48     4 */
	u32                        i_seqno;              /*    52     4 */
	atomic_t                   o_seqno;              /*    56     4 */
	int                        tun_hlen;             /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u32                        index;                /*    64     4 */
	u8                         erspan_ver;           /*    68     1 */
	u8                         dir;                  /*    69     1 */
	u16                        hwid;                 /*    70     2 */
	struct dst_cache           dst_cache;            /*    72    16 */
	struct ip_tunnel_parm      parms;                /*    88    52 */
	/* --- cacheline 2 boundary (128 bytes) was 12 bytes ago --- */
	int                        mlink;                /*   140     4 */
	int                        encap_hlen;           /*   144     4 */
	int                        hlen;                 /*   148     4 */
	struct ip_tunnel_encap     encap;                /*   152     8 */
	struct ip_tunnel_6rd_parm  ip6rd;                /*   160    24 */
	struct ip_tunnel_prl_entry * prl;                /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	unsigned int               prl_count;            /*   192     4 */
	unsigned int               ip_tnl_net_id;        /*   196     4 */
	struct gro_cells           gro_cells;            /*   200     8 */
	__u32                      fwmark;               /*   208     4 */
	bool                       collect_md;           /*   212     1 */
	bool                       ignore_df;            /*   213     1 */

	/* size: 216, cachelines: 4, members: 27 */
	/* padding: 2 */
	/* last cacheline: 24 bytes */
};
struct ip6_tnl {
	struct ip6_tnl *           next;                 /*     0     8 */
	struct net_device *        dev;                  /*     8     8 */
	struct net *               net;                  /*    16     8 */
	struct __ip6_tnl_parm      parms;                /*    24    88 */
	/* --- cacheline 1 boundary (64 bytes) was 48 bytes ago --- */
	struct flowi               fl __attribute__((__aligned__(8))); /*   112   104 */
	/* --- cacheline 3 boundary (192 bytes) was 24 bytes ago --- */
	struct dst_cache           dst_cache;            /*   216    16 */
	struct gro_cells           gro_cells;            /*   232     8 */
	int                        err_count;            /*   240     4 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          err_time;             /*   248     8 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	__u32                      i_seqno;              /*   256     4 */
	atomic_t                   o_seqno;              /*   260     4 */
	int                        hlen;                 /*   264     4 */
	int                        tun_hlen;             /*   268     4 */
	int                        encap_hlen;           /*   272     4 */
	struct ip_tunnel_encap     encap;                /*   276     8 */
	int                        mlink;                /*   284     4 */

	/* size: 288, cachelines: 5, members: 16 */
	/* sum members: 284, holes: 1, sum holes: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct xfrm_skb_cb {
	struct xfrm_tunnel_skb_cb  header;               /*     0    32 */
	union {
		struct {
			__u32      low;                  /*    32     4 */
			__u32      hi;                   /*    36     4 */
		} output;                                /*    32     8 */
		struct {
			__be32     low;                  /*    32     4 */
			__be32     hi;                   /*    36     4 */
		} input;                                 /*    32     8 */
	} seq;                                           /*    32     8 */

	/* size: 40, cachelines: 1, members: 2 */
	/* last cacheline: 40 bytes */
};
struct ip_tunnel_6rd_parm {
	struct in6_addr            prefix;               /*     0    16 */
	__be32                     relay_prefix;         /*    16     4 */
	u16                        prefixlen;            /*    20     2 */
	u16                        relay_prefixlen;      /*    22     2 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct ip_tunnel_prl_entry {
	struct ip_tunnel_prl_entry * next;               /*     0     8 */
	__be32                     addr;                 /*     8     4 */
	u16                        flags;                /*    12     2 */

	/* XXX 2 bytes hole, try to pack */

	struct callback_head       callback_head __attribute__((__aligned__(8))); /*    16    16 */

	/* size: 32, cachelines: 1, members: 4 */
	/* sum members: 30, holes: 1, sum holes: 2 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 2 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(8)));
struct __ip6_tnl_parm {
	char                       name[16];             /*     0    16 */
	int                        link;                 /*    16     4 */
	__u8                       proto;                /*    20     1 */
	__u8                       encap_limit;          /*    21     1 */
	__u8                       hop_limit;            /*    22     1 */
	bool                       collect_md;           /*    23     1 */
	__be32                     flowinfo;             /*    24     4 */
	__u32                      flags;                /*    28     4 */
	struct in6_addr            laddr;                /*    32    16 */
	struct in6_addr            raddr;                /*    48    16 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	__be16                     i_flags;              /*    64     2 */
	__be16                     o_flags;              /*    66     2 */
	__be32                     i_key;                /*    68     4 */
	__be32                     o_key;                /*    72     4 */
	__u32                      fwmark;               /*    76     4 */
	__u32                      index;                /*    80     4 */
	__u8                       erspan_ver;           /*    84     1 */
	__u8                       dir;                  /*    85     1 */
	__u16                      hwid;                 /*    86     2 */

	/* size: 88, cachelines: 2, members: 19 */
	/* last cacheline: 24 bytes */
};
struct xfrm_trans_tasklet {
	struct tasklet_struct      tasklet;              /*     0    48 */
	struct sk_buff_head        queue;                /*    48    24 */

	/* size: 72, cachelines: 2, members: 2 */
	/* last cacheline: 8 bytes */
};
struct xfrm_trans_cb {
	union {
		struct inet_skb_parm h4;                 /*     0    24 */
		struct inet6_skb_parm h6;                /*     0    24 */
	} header;                                        /*     0    24 */
	int                        (*finish)(struct net *, struct sock *, struct sk_buff *); /*    24     8 */
	struct net *               net;                  /*    32     8 */

	/* size: 40, cachelines: 1, members: 3 */
	/* last cacheline: 40 bytes */
};
struct xfrm_user_offload {
	int                        ifindex;              /*     0     4 */
	__u8                       flags;                /*     4     1 */

	/* size: 8, cachelines: 1, members: 2 */
	/* padding: 3 */
	/* last cacheline: 8 bytes */
};
struct sadb_alg {
	__u8                       sadb_alg_id;          /*     0     1 */
	__u8                       sadb_alg_ivlen;       /*     1     1 */
	__u16                      sadb_alg_minbits;     /*     2     2 */
	__u16                      sadb_alg_maxbits;     /*     4     2 */
	__u16                      sadb_alg_reserved;    /*     6     2 */

	/* size: 8, cachelines: 1, members: 5 */
	/* last cacheline: 8 bytes */
};
struct xfrm_algo_aead_info {
	char *                     geniv;                /*     0     8 */
	u16                        icv_truncbits;        /*     8     2 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 6 */
	/* last cacheline: 16 bytes */
};
struct xfrm_algo_auth_info {
	u16                        icv_truncbits;        /*     0     2 */
	u16                        icv_fullbits;         /*     2     2 */

	/* size: 4, cachelines: 1, members: 2 */
	/* last cacheline: 4 bytes */
};
struct xfrm_algo_encr_info {
	char *                     geniv;                /*     0     8 */
	u16                        blockbits;            /*     8     2 */
	u16                        defkeybits;           /*    10     2 */

	/* size: 16, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct xfrm_algo_comp_info {
	u16                        threshold;            /*     0     2 */

	/* size: 2, cachelines: 1, members: 1 */
	/* last cacheline: 2 bytes */
};
struct xfrm_algo_desc {
	char *                     name;                 /*     0     8 */
	char *                     compat;               /*     8     8 */
	u8                         available:1;          /*    16: 0  1 */
	u8                         pfkey_supported:1;    /*    16: 1  1 */

	/* XXX 6 bits hole, try to pack */
	/* XXX 7 bytes hole, try to pack */

	union {
		struct xfrm_algo_aead_info aead;         /*    24    16 */
		struct xfrm_algo_auth_info auth;         /*    24     4 */
		struct xfrm_algo_encr_info encr;         /*    24    16 */
		struct xfrm_algo_comp_info comp;         /*    24     2 */
	} uinfo;                                         /*    24    16 */
	struct sadb_alg            desc;                 /*    40     8 */

	/* size: 48, cachelines: 1, members: 6 */
	/* sum members: 40, holes: 1, sum holes: 7 */
	/* sum bitfield members: 2 bits, bit holes: 1, sum bit holes: 6 bits */
	/* last cacheline: 48 bytes */
};
struct xfrm_algo_list {
	struct xfrm_algo_desc *    algs;                 /*     0     8 */
	int                        entries;              /*     8     4 */
	u32                        type;                 /*    12     4 */
	u32                        mask;                 /*    16     4 */

	/* size: 24, cachelines: 1, members: 4 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct xfrm_aead_name {
	const char  *              name;                 /*     0     8 */
	int                        icvbits;              /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct xfrm_user_tmpl {
	struct xfrm_id             id;                   /*     0    24 */

	/* XXX last struct has 3 bytes of padding */

	__u16                      family;               /*    24     2 */

	/* XXX 2 bytes hole, try to pack */

	xfrm_address_t             saddr;                /*    28    16 */
	__u32                      reqid;                /*    44     4 */
	__u8                       mode;                 /*    48     1 */
	__u8                       share;                /*    49     1 */
	__u8                       optional;             /*    50     1 */

	/* XXX 1 byte hole, try to pack */

	__u32                      aalgos;               /*    52     4 */
	__u32                      ealgos;               /*    56     4 */
	__u32                      calgos;               /*    60     4 */

	/* size: 64, cachelines: 1, members: 10 */
	/* sum members: 61, holes: 2, sum holes: 3 */
	/* paddings: 1, sum paddings: 3 */
};
struct xfrm_userpolicy_type {
	__u8                       type;                 /*     0     1 */

	/* XXX 1 byte hole, try to pack */

	__u16                      reserved1;            /*     2     2 */
	__u8                       reserved2;            /*     4     1 */

	/* size: 6, cachelines: 1, members: 3 */
	/* sum members: 4, holes: 1, sum holes: 1 */
	/* padding: 1 */
	/* last cacheline: 6 bytes */
};
struct xfrmu_sadhinfo {
	__u32                      sadhcnt;              /*     0     4 */
	__u32                      sadhmcnt;             /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct xfrmu_spdinfo {
	__u32                      incnt;                /*     0     4 */
	__u32                      outcnt;               /*     4     4 */
	__u32                      fwdcnt;               /*     8     4 */
	__u32                      inscnt;               /*    12     4 */
	__u32                      outscnt;              /*    16     4 */
	__u32                      fwdscnt;              /*    20     4 */

	/* size: 24, cachelines: 1, members: 6 */
	/* last cacheline: 24 bytes */
};
struct xfrmu_spdhinfo {
	__u32                      spdhcnt;              /*     0     4 */
	__u32                      spdhmcnt;             /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct xfrmu_spdhthresh {
	__u8                       lbits;                /*     0     1 */
	__u8                       rbits;                /*     1     1 */

	/* size: 2, cachelines: 1, members: 2 */
	/* last cacheline: 2 bytes */
};
struct xfrm_usersa_info {
	struct xfrm_selector       sel;                  /*     0    56 */
	struct xfrm_id             id;                   /*    56    24 */

	/* XXX last struct has 3 bytes of padding */

	/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
	xfrm_address_t             saddr;                /*    80    16 */
	struct xfrm_lifetime_cfg   lft;                  /*    96    64 */
	/* --- cacheline 2 boundary (128 bytes) was 32 bytes ago --- */
	struct xfrm_lifetime_cur   curlft;               /*   160    32 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	struct xfrm_stats          stats;                /*   192    12 */
	__u32                      seq;                  /*   204     4 */
	__u32                      reqid;                /*   208     4 */
	__u16                      family;               /*   212     2 */
	__u8                       mode;                 /*   214     1 */
	__u8                       replay_window;        /*   215     1 */
	__u8                       flags;                /*   216     1 */

	/* size: 224, cachelines: 4, members: 12 */
	/* padding: 7 */
	/* paddings: 1, sum paddings: 3 */
	/* last cacheline: 32 bytes */
};
struct xfrm_usersa_id {
	xfrm_address_t             daddr;                /*     0    16 */
	__be32                     spi;                  /*    16     4 */
	__u16                      family;               /*    20     2 */
	__u8                       proto;                /*    22     1 */

	/* size: 24, cachelines: 1, members: 4 */
	/* padding: 1 */
	/* last cacheline: 24 bytes */
};
struct xfrm_aevent_id {
	struct xfrm_usersa_id      sa_id;                /*     0    24 */

	/* XXX last struct has 1 byte of padding */

	xfrm_address_t             saddr;                /*    24    16 */
	__u32                      flags;                /*    40     4 */
	__u32                      reqid;                /*    44     4 */

	/* size: 48, cachelines: 1, members: 4 */
	/* paddings: 1, sum paddings: 1 */
	/* last cacheline: 48 bytes */
};
struct xfrm_userspi_info {
	struct xfrm_usersa_info    info;                 /*     0   224 */

	/* XXX last struct has 7 bytes of padding */

	/* --- cacheline 3 boundary (192 bytes) was 32 bytes ago --- */
	__u32                      min;                  /*   224     4 */
	__u32                      max;                  /*   228     4 */

	/* size: 232, cachelines: 4, members: 3 */
	/* paddings: 1, sum paddings: 7 */
	/* last cacheline: 40 bytes */
};
struct xfrm_userpolicy_info {
	struct xfrm_selector       sel;                  /*     0    56 */
	struct xfrm_lifetime_cfg   lft;                  /*    56    64 */
	/* --- cacheline 1 boundary (64 bytes) was 56 bytes ago --- */
	struct xfrm_lifetime_cur   curlft;               /*   120    32 */
	/* --- cacheline 2 boundary (128 bytes) was 24 bytes ago --- */
	__u32                      priority;             /*   152     4 */
	__u32                      index;                /*   156     4 */
	__u8                       dir;                  /*   160     1 */
	__u8                       action;               /*   161     1 */
	__u8                       flags;                /*   162     1 */
	__u8                       share;                /*   163     1 */

	/* size: 168, cachelines: 3, members: 9 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct xfrm_userpolicy_id {
	struct xfrm_selector       sel;                  /*     0    56 */
	__u32                      index;                /*    56     4 */
	__u8                       dir;                  /*    60     1 */

	/* size: 64, cachelines: 1, members: 3 */
	/* padding: 3 */
};
struct xfrm_user_acquire {
	struct xfrm_id             id;                   /*     0    24 */

	/* XXX last struct has 3 bytes of padding */

	xfrm_address_t             saddr;                /*    24    16 */
	struct xfrm_selector       sel;                  /*    40    56 */
	/* --- cacheline 1 boundary (64 bytes) was 32 bytes ago --- */
	struct xfrm_userpolicy_info policy;              /*    96   168 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 4 boundary (256 bytes) was 8 bytes ago --- */
	__u32                      aalgos;               /*   264     4 */
	__u32                      ealgos;               /*   268     4 */
	__u32                      calgos;               /*   272     4 */
	__u32                      seq;                  /*   276     4 */

	/* size: 280, cachelines: 5, members: 8 */
	/* paddings: 2, sum paddings: 7 */
	/* last cacheline: 24 bytes */
};
struct xfrm_user_expire {
	struct xfrm_usersa_info    state;                /*     0   224 */

	/* XXX last struct has 7 bytes of padding */

	/* --- cacheline 3 boundary (192 bytes) was 32 bytes ago --- */
	__u8                       hard;                 /*   224     1 */

	/* size: 232, cachelines: 4, members: 2 */
	/* padding: 7 */
	/* paddings: 1, sum paddings: 7 */
	/* last cacheline: 40 bytes */
};
struct xfrm_user_polexpire {
	struct xfrm_userpolicy_info pol;                 /*     0   168 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 2 boundary (128 bytes) was 40 bytes ago --- */
	__u8                       hard;                 /*   168     1 */

	/* size: 176, cachelines: 3, members: 2 */
	/* padding: 7 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 48 bytes */
};
struct xfrm_usersa_flush {
	__u8                       proto;                /*     0     1 */

	/* size: 1, cachelines: 1, members: 1 */
	/* last cacheline: 1 bytes */
};
struct xfrm_user_report {
	__u8                       proto;                /*     0     1 */

	/* XXX 3 bytes hole, try to pack */

	struct xfrm_selector       sel;                  /*     4    56 */

	/* size: 60, cachelines: 1, members: 2 */
	/* sum members: 57, holes: 1, sum holes: 3 */
	/* last cacheline: 60 bytes */
};
struct xfrm_user_kmaddress {
	xfrm_address_t             local;                /*     0    16 */
	xfrm_address_t             remote;               /*    16    16 */
	__u32                      reserved;             /*    32     4 */
	__u16                      family;               /*    36     2 */

	/* size: 40, cachelines: 1, members: 4 */
	/* padding: 2 */
	/* last cacheline: 40 bytes */
};
struct xfrm_user_migrate {
	xfrm_address_t             old_daddr;            /*     0    16 */
	xfrm_address_t             old_saddr;            /*    16    16 */
	xfrm_address_t             new_daddr;            /*    32    16 */
	xfrm_address_t             new_saddr;            /*    48    16 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	__u8                       proto;                /*    64     1 */
	__u8                       mode;                 /*    65     1 */
	__u16                      reserved;             /*    66     2 */
	__u32                      reqid;                /*    68     4 */
	__u16                      old_family;           /*    72     2 */
	__u16                      new_family;           /*    74     2 */

	/* size: 76, cachelines: 2, members: 10 */
	/* last cacheline: 12 bytes */
};
struct xfrm_user_mapping {
	struct xfrm_usersa_id      id;                   /*     0    24 */

	/* XXX last struct has 1 byte of padding */

	__u32                      reqid;                /*    24     4 */
	xfrm_address_t             old_saddr;            /*    28    16 */
	xfrm_address_t             new_saddr;            /*    44    16 */
	__be16                     old_sport;            /*    60     2 */
	__be16                     new_sport;            /*    62     2 */

	/* size: 64, cachelines: 1, members: 6 */
	/* paddings: 1, sum paddings: 1 */
};
struct xfrm_dump_info {
	struct sk_buff *           in_skb;               /*     0     8 */
	struct sk_buff *           out_skb;              /*     8     8 */
	u32                        nlmsg_seq;            /*    16     4 */
	u16                        nlmsg_flags;          /*    20     2 */

	/* size: 24, cachelines: 1, members: 4 */
	/* padding: 2 */
	/* last cacheline: 24 bytes */
};
struct xfrm_link {
	int                        (*doit)(struct sk_buff *, struct nlmsghdr *, struct nlattr * *); /*     0     8 */
	int                        (*start)(struct netlink_callback *); /*     8     8 */
	int                        (*dump)(struct sk_buff *, struct netlink_callback *); /*    16     8 */
	int                        (*done)(struct netlink_callback *); /*    24     8 */
	const struct nla_policy  * nla_pol;              /*    32     8 */
	int                        nla_max;              /*    40     4 */

	/* size: 48, cachelines: 1, members: 6 */
	/* padding: 4 */
	/* last cacheline: 48 bytes */
};
struct unix_stream_read_state {
	int                        (*recv_actor)(struct sk_buff *, int, int, struct unix_stream_read_state *); /*     0     8 */
	struct socket *            socket;               /*     8     8 */
	struct msghdr *            msg;                  /*    16     8 */
	struct pipe_inode_info *   pipe;                 /*    24     8 */
	size_t                     size;                 /*    32     8 */
	int                        flags;                /*    40     4 */
	unsigned int               splice_flags;         /*    44     4 */

	/* size: 48, cachelines: 1, members: 7 */
	/* last cacheline: 48 bytes */
};
struct ipv6_params {
	__s32                      disable_ipv6;         /*     0     4 */
	__s32                      autoconf;             /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct in6_rtmsg {
	struct in6_addr            rtmsg_dst;            /*     0    16 */
	struct in6_addr            rtmsg_src;            /*    16    16 */
	struct in6_addr            rtmsg_gateway;        /*    32    16 */
	__u32                      rtmsg_type;           /*    48     4 */
	__u16                      rtmsg_dst_len;        /*    52     2 */
	__u16                      rtmsg_src_len;        /*    54     2 */
	__u32                      rtmsg_metric;         /*    56     4 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	long unsigned int          rtmsg_info;           /*    64     8 */
	__u32                      rtmsg_flags;          /*    72     4 */
	int                        rtmsg_ifindex;        /*    76     4 */

	/* size: 80, cachelines: 2, members: 10 */
	/* sum members: 76, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct compat_in6_rtmsg {
	struct in6_addr            rtmsg_dst;            /*     0    16 */
	struct in6_addr            rtmsg_src;            /*    16    16 */
	struct in6_addr            rtmsg_gateway;        /*    32    16 */
	u32                        rtmsg_type;           /*    48     4 */
	u16                        rtmsg_dst_len;        /*    52     2 */
	u16                        rtmsg_src_len;        /*    54     2 */
	u32                        rtmsg_metric;         /*    56     4 */
	u32                        rtmsg_info;           /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u32                        rtmsg_flags;          /*    64     4 */
	s32                        rtmsg_ifindex;        /*    68     4 */

	/* size: 72, cachelines: 2, members: 10 */
	/* last cacheline: 8 bytes */
};
struct ac6_iter_state {
	struct seq_net_private     p;                    /*     0     8 */
	struct net_device *        dev;                  /*     8     8 */
	struct inet6_dev *         idev;                 /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct ip6_fraglist_iter {
	struct ipv6hdr *           tmp_hdr;              /*     0     8 */
	struct sk_buff *           frag;                 /*     8     8 */
	int                        offset;               /*    16     4 */
	unsigned int               hlen;                 /*    20     4 */
	__be32                     frag_id;              /*    24     4 */
	u8                         nexthdr;              /*    28     1 */

	/* size: 32, cachelines: 1, members: 6 */
	/* padding: 3 */
	/* last cacheline: 32 bytes */
};
struct ip6_frag_state {
	u8 *                       prevhdr;              /*     0     8 */
	unsigned int               hlen;                 /*     8     4 */
	unsigned int               mtu;                  /*    12     4 */
	unsigned int               left;                 /*    16     4 */
	int                        offset;               /*    20     4 */
	int                        ptr;                  /*    24     4 */
	int                        hroom;                /*    28     4 */
	int                        troom;                /*    32     4 */
	__be32                     frag_id;              /*    36     4 */
	u8                         nexthdr;              /*    40     1 */

	/* size: 48, cachelines: 1, members: 10 */
	/* padding: 7 */
	/* last cacheline: 48 bytes */
};
struct ip6_ra_chain {
	struct ip6_ra_chain *      next;                 /*     0     8 */
	struct sock *              sk;                   /*     8     8 */
	int                        sel;                  /*    16     4 */

	/* XXX 4 bytes hole, try to pack */

	void                       (*destructor)(struct sock *); /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct ipcm6_cookie {
	struct sockcm_cookie       sockc;                /*     0    16 */

	/* XXX last struct has 2 bytes of padding */

	__s16                      hlimit;               /*    16     2 */
	__s16                      tclass;               /*    18     2 */
	__u16                      gso_size;             /*    20     2 */
	__s8                       dontfrag;             /*    22     1 */

	/* XXX 1 byte hole, try to pack */

	struct ipv6_txoptions *    opt;                  /*    24     8 */

	/* size: 32, cachelines: 1, members: 6 */
	/* sum members: 31, holes: 1, sum holes: 1 */
	/* paddings: 1, sum paddings: 2 */
	/* last cacheline: 32 bytes */
};
struct ifla_cacheinfo {
	__u32                      max_reasm_len;        /*     0     4 */
	__u32                      tstamp;               /*     4     4 */
	__u32                      reachable_time;       /*     8     4 */
	__u32                      retrans_time;         /*    12     4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct wpan_dev {
	struct wpan_phy *          wpan_phy;             /*     0     8 */
	int                        iftype;               /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           list;                 /*    16    16 */
	struct net_device *        netdev;               /*    32     8 */
	const struct wpan_dev_header_ops  * header_ops;  /*    40     8 */
	struct net_device *        lowpan_dev;           /*    48     8 */
	u32                        identifier;           /*    56     4 */
	__le16                     pan_id;               /*    60     2 */
	__le16                     short_addr;           /*    62     2 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	__le64                     extended_addr;        /*    64     8 */
	atomic_t                   bsn;                  /*    72     4 */
	atomic_t                   dsn;                  /*    76     4 */
	u8                         min_be;               /*    80     1 */
	u8                         max_be;               /*    81     1 */
	u8                         csma_retries;         /*    82     1 */
	s8                         frame_retries;        /*    83     1 */
	bool                       lbt;                  /*    84     1 */
	bool                       promiscuous_mode;     /*    85     1 */
	bool                       ackreq;               /*    86     1 */

	/* size: 88, cachelines: 2, members: 19 */
	/* sum members: 83, holes: 1, sum holes: 4 */
	/* padding: 1 */
	/* last cacheline: 24 bytes */
};
struct prefixmsg {
	unsigned char              prefix_family;        /*     0     1 */
	unsigned char              prefix_pad1;          /*     1     1 */
	short unsigned int         prefix_pad2;          /*     2     2 */
	int                        prefix_ifindex;       /*     4     4 */
	unsigned char              prefix_type;          /*     8     1 */
	unsigned char              prefix_len;           /*     9     1 */
	unsigned char              prefix_flags;         /*    10     1 */
	unsigned char              prefix_pad3;          /*    11     1 */

	/* size: 12, cachelines: 1, members: 8 */
	/* last cacheline: 12 bytes */
};
struct prefix_cacheinfo {
	__u32                      preferred_time;       /*     0     4 */
	__u32                      valid_time;           /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct in6_ifreq {
	struct in6_addr            ifr6_addr;            /*     0    16 */
	__u32                      ifr6_prefixlen;       /*    16     4 */
	int                        ifr6_ifindex;         /*    20     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct wpan_phy {
	const void  *              privid;               /*     0     8 */
	u32                        flags;                /*     8     4 */
	u8                         current_channel;      /*    12     1 */
	u8                         current_page;         /*    13     1 */

	/* XXX 2 bytes hole, try to pack */

	struct wpan_phy_supported  supported;            /*    16   184 */
	/* --- cacheline 3 boundary (192 bytes) was 8 bytes ago --- */
	s32                        transmit_power;       /*   200     4 */
	struct wpan_phy_cca        cca;                  /*   204     8 */

	/* XXX 4 bytes hole, try to pack */

	__le64                     perm_extended_addr;   /*   216     8 */
	s32                        cca_ed_level;         /*   224     4 */
	u8                         symbol_duration;      /*   228     1 */

	/* XXX 1 byte hole, try to pack */

	u16                        lifs_period;          /*   230     2 */
	u16                        sifs_period;          /*   232     2 */

	/* XXX 6 bytes hole, try to pack */

	struct device              dev __attribute__((__aligned__(8))); /*   240  1000 */
	/* --- cacheline 19 boundary (1216 bytes) was 24 bytes ago --- */
	possible_net_t             _net;                 /*  1240     8 */
	char                       priv[] __attribute__((__aligned__(32))); /*  1248     0 */

	/* size: 1248, cachelines: 20, members: 15 */
	/* sum members: 1235, holes: 4, sum holes: 13 */
	/* forced alignments: 2, forced holes: 1, sum forced holes: 6 */
	/* last cacheline: 32 bytes */
} __attribute__((__aligned__(32)));
struct wpan_phy_cca {
	enum nl802154_cca_modes    mode;                 /*     0     4 */
	enum nl802154_cca_opts     opt;                  /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct wpan_phy_supported {
	u32                        channels[32];         /*     0   128 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	u32                        cca_modes;            /*   128     4 */
	u32                        cca_opts;             /*   132     4 */
	u32                        iftypes;              /*   136     4 */
	enum nl802154_supported_bool_states lbt;         /*   140     4 */
	u8                         min_minbe;            /*   144     1 */
	u8                         max_minbe;            /*   145     1 */
	u8                         min_maxbe;            /*   146     1 */
	u8                         max_maxbe;            /*   147     1 */
	u8                         min_csma_backoffs;    /*   148     1 */
	u8                         max_csma_backoffs;    /*   149     1 */
	s8                         min_frame_retries;    /*   150     1 */
	s8                         max_frame_retries;    /*   151     1 */
	size_t                     tx_powers_size;       /*   152     8 */
	size_t                     cca_ed_levels_size;   /*   160     8 */
	const s32  *               tx_powers;            /*   168     8 */
	const s32  *               cca_ed_levels;        /*   176     8 */

	/* size: 184, cachelines: 3, members: 17 */
	/* last cacheline: 56 bytes */
};
struct ieee802154_addr {
	u8                         mode;                 /*     0     1 */

	/* XXX 1 byte hole, try to pack */

	__le16                     pan_id;               /*     2     2 */

	/* XXX 4 bytes hole, try to pack */

	union {
		__le16             short_addr;           /*     8     2 */
		__le64             extended_addr;        /*     8     8 */
	};                                               /*     8     8 */

	/* size: 16, cachelines: 1, members: 3 */
	/* sum members: 11, holes: 2, sum holes: 5 */
	/* last cacheline: 16 bytes */
};
struct wpan_dev_header_ops {
	int                        (*create)(struct sk_buff *, struct net_device *, const struct ieee802154_addr  *, const struct ieee802154_addr  *, unsigned int); /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
union fwnet_hwaddr {
	u8                         u[16];              /*     0    16 */
	struct {
		__be64             uniq_id;            /*     0     8 */
		u8                 max_rec;            /*     8     1 */
		u8                 sspd;               /*     9     1 */
		__be16             fifo_hi;            /*    10     2 */
		__be32             fifo_lo;            /*    12     4 */
	} uc;                                          /*     0    16 */
};
struct in6_validator_info {
	struct in6_addr            i6vi_addr;            /*     0    16 */
	struct inet6_dev *         i6vi_dev;             /*    16     8 */
	struct netlink_ext_ack *   extack;               /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* last cacheline: 32 bytes */
};
struct ifa6_config {
	const struct in6_addr  *   pfx;                  /*     0     8 */
	unsigned int               plen;                 /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	const struct in6_addr  *   peer_pfx;             /*    16     8 */
	u32                        rt_priority;          /*    24     4 */
	u32                        ifa_flags;            /*    28     4 */
	u32                        preferred_lft;        /*    32     4 */
	u32                        valid_lft;            /*    36     4 */
	u16                        scope;                /*    40     2 */

	/* size: 48, cachelines: 1, members: 8 */
	/* sum members: 38, holes: 1, sum holes: 4 */
	/* padding: 6 */
	/* last cacheline: 48 bytes */
};
struct ipv6_saddr_score {
	int                        rule;                 /*     0     4 */
	int                        addr_type;            /*     4     4 */
	struct inet6_ifaddr *      ifa;                  /*     8     8 */
	long unsigned int          scorebits[1];         /*    16     8 */
	int                        scopedist;            /*    24     4 */
	int                        matchlen;             /*    28     4 */

	/* size: 32, cachelines: 1, members: 6 */
	/* last cacheline: 32 bytes */
};
struct ipv6_saddr_dst {
	const struct in6_addr  *   addr;                 /*     0     8 */
	int                        ifindex;              /*     8     4 */
	int                        scope;                /*    12     4 */
	int                        label;                /*    16     4 */
	unsigned int               prefs;                /*    20     4 */

	/* size: 24, cachelines: 1, members: 5 */
	/* last cacheline: 24 bytes */
};
struct if6_iter_state {
	struct seq_net_private     p;                    /*     0     8 */
	int                        bucket;               /*     8     4 */
	int                        offset;               /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct inet6_fill_args {
	u32                        portid;               /*     0     4 */
	u32                        seq;                  /*     4     4 */
	int                        event;                /*     8     4 */
	unsigned int               flags;                /*    12     4 */
	int                        netnsid;              /*    16     4 */
	int                        ifindex;              /*    20     4 */
	enum addr_type_t           type;                 /*    24     4 */

	/* size: 28, cachelines: 1, members: 7 */
	/* last cacheline: 28 bytes */
};
struct ifaddrlblmsg {
	__u8                       ifal_family;          /*     0     1 */
	__u8                       __ifal_reserved;      /*     1     1 */
	__u8                       ifal_prefixlen;       /*     2     1 */
	__u8                       ifal_flags;           /*     3     1 */
	__u32                      ifal_index;           /*     4     4 */
	__u32                      ifal_seq;             /*     8     4 */

	/* size: 12, cachelines: 1, members: 6 */
	/* last cacheline: 12 bytes */
};
struct ip6addrlbl_entry {
	struct in6_addr            prefix;               /*     0    16 */
	int                        prefixlen;            /*    16     4 */
	int                        ifindex;              /*    20     4 */
	int                        addrtype;             /*    24     4 */
	u32                        label;                /*    28     4 */
	struct hlist_node          list;                 /*    32    16 */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*    48    16 */

	/* size: 64, cachelines: 1, members: 7 */
	/* forced alignments: 1 */
} __attribute__((__aligned__(8)));
struct ip6addrlbl_init_table {
	const struct in6_addr  *   prefix;               /*     0     8 */
	int                        prefixlen;            /*     8     4 */
	u32                        label;                /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct rd_msg {
	struct icmp6hdr            icmph;                /*     0     8 */
	struct in6_addr            target;               /*     8    16 */
	struct in6_addr            dest;                 /*    24    16 */
	__u8                       opt[];                /*    40     0 */

	/* size: 40, cachelines: 1, members: 4 */
	/* last cacheline: 40 bytes */
};
struct fib6_gc_args {
	int                        timeout;              /*     0     4 */
	int                        more;                 /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct rt6_exception {
	struct hlist_node          hlist;                /*     0    16 */
	struct rt6_info *          rt6i;                 /*    16     8 */
	long unsigned int          stamp;                /*    24     8 */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*    32    16 */

	/* size: 48, cachelines: 1, members: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct route_info {
	__u8                       type;                 /*     0     1 */
	__u8                       length;               /*     1     1 */
	__u8                       prefix_len;           /*     2     1 */
	__u8                       reserved_l:3;         /*     3: 0  1 */
	__u8                       route_pref:2;         /*     3: 3  1 */
	__u8                       reserved_h:3;         /*     3: 5  1 */
	__be32                     lifetime;             /*     4     4 */
	__u8                       prefix[];             /*     8     0 */

	/* size: 8, cachelines: 1, members: 8 */
	/* last cacheline: 8 bytes */
};
struct rt6_rtnl_dump_arg {
	struct sk_buff *           skb;                  /*     0     8 */
	struct netlink_callback *  cb;                   /*     8     8 */
	struct net *               net;                  /*    16     8 */
	struct fib_dump_filter     filter;               /*    24    24 */

	/* size: 48, cachelines: 1, members: 4 */
	/* last cacheline: 48 bytes */
};
struct netevent_redirect {
	struct dst_entry *         old;                  /*     0     8 */
	struct dst_entry *         new;                  /*     8     8 */
	struct neighbour *         neigh;                /*    16     8 */
	const void  *              daddr;                /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct trace_event_raw_fib6_table_lookup {
	struct trace_entry         ent;                  /*     0     8 */
	u32                        tb_id;                /*     8     4 */
	int                        err;                  /*    12     4 */
	int                        oif;                  /*    16     4 */
	int                        iif;                  /*    20     4 */
	__u8                       tos;                  /*    24     1 */
	__u8                       scope;                /*    25     1 */
	__u8                       flags;                /*    26     1 */
	__u8                       src[16];              /*    27    16 */
	__u8                       dst[16];              /*    43    16 */

	/* XXX 1 byte hole, try to pack */

	u16                        sport;                /*    60     2 */
	u16                        dport;                /*    62     2 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	u8                         proto;                /*    64     1 */
	u8                         rt_type;              /*    65     1 */

	/* XXX 2 bytes hole, try to pack */

	u32                        __data_loc_name;      /*    68     4 */
	__u8                       gw[16];               /*    72    16 */
	char                       __data[];             /*    88     0 */

	/* size: 88, cachelines: 2, members: 17 */
	/* sum members: 85, holes: 2, sum holes: 3 */
	/* last cacheline: 24 bytes */
};
struct trace_event_data_offsets_fib6_table_lookup {
	u32                        name;                 /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct fib6_nh_dm_arg {
	struct net *               net;                  /*     0     8 */
	const struct in6_addr  *   saddr;                /*     8     8 */
	int                        oif;                  /*    16     4 */
	int                        flags;                /*    20     4 */
	struct fib6_nh *           nh;                   /*    24     8 */

	/* size: 32, cachelines: 1, members: 5 */
	/* last cacheline: 32 bytes */
};
struct __rt6_probe_work {
	struct work_struct         work;                 /*     0    64 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct in6_addr            target;               /*    64    16 */
	struct net_device *        dev;                  /*    80     8 */

	/* size: 88, cachelines: 2, members: 3 */
	/* last cacheline: 24 bytes */
};
struct fib6_nh_frl_arg {
	u32                        flags;                /*     0     4 */
	int                        oif;                  /*     4     4 */
	int                        strict;               /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	int *                      mpri;                 /*    16     8 */
	bool *                     do_rr;                /*    24     8 */
	struct fib6_nh *           nh;                   /*    32     8 */

	/* size: 40, cachelines: 1, members: 6 */
	/* sum members: 36, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct fib6_nh_excptn_arg {
	struct rt6_info *          rt;                   /*     0     8 */
	int                        plen;                 /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct fib6_nh_match_arg {
	const struct net_device  * dev;                  /*     0     8 */
	const struct in6_addr  *   gw;                   /*     8     8 */
	struct fib6_nh *           match;                /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct fib6_nh_age_excptn_arg {
	struct fib6_gc_args *      gc_args;              /*     0     8 */
	long unsigned int          now;                  /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct fib6_nh_rd_arg {
	struct fib6_result *       res;                  /*     0     8 */
	struct flowi6 *            fl6;                  /*     8     8 */
	const struct in6_addr  *   gw;                   /*    16     8 */
	struct rt6_info * *        ret;                  /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct ip6rd_flowi {
	struct flowi6              fl6 __attribute__((__aligned__(8))); /*     0   104 */

	/* XXX last struct has 4 bytes of padding */

	/* --- cacheline 1 boundary (64 bytes) was 40 bytes ago --- */
	struct in6_addr            gateway;              /*   104    16 */

	/* size: 120, cachelines: 2, members: 2 */
	/* paddings: 1, sum paddings: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 56 bytes */
} __attribute__((__aligned__(8)));
struct fib6_nh_del_cached_rt_arg {
	struct fib6_config *       cfg;                  /*     0     8 */
	struct fib6_info *         f6i;                  /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct arg_dev_net_ip {
	struct net_device *        dev;                  /*     0     8 */
	struct net *               net;                  /*     8     8 */
	struct in6_addr *          addr;                 /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct arg_netdev_event {
	const struct net_device  * dev;                  /*     0     8 */
	union {
		unsigned char      nh_flags;             /*     8     1 */
		long unsigned int  event;                /*     8     8 */
	};                                               /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct rt6_mtu_change_arg {
	struct net_device *        dev;                  /*     0     8 */
	unsigned int               mtu;                  /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct fib6_info *         f6i;                  /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* sum members: 20, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct rt6_nh {
	struct fib6_info *         fib6_info;            /*     0     8 */
	struct fib6_config         r_cfg;                /*     8   184 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	struct list_head           next;                 /*   192    16 */

	/* size: 208, cachelines: 4, members: 3 */
	/* last cacheline: 16 bytes */
};
struct fib6_nh_exception_dump_walker {
	struct rt6_rtnl_dump_arg * dump;                 /*     0     8 */
	struct fib6_info *         rt;                   /*     8     8 */
	unsigned int               flags;                /*    16     4 */
	unsigned int               skip;                 /*    20     4 */
	unsigned int               count;                /*    24     4 */

	/* size: 32, cachelines: 1, members: 5 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct fib6_walker {
	struct list_head           lh;                   /*     0    16 */
	struct fib6_node *         root;                 /*    16     8 */
	struct fib6_node *         node;                 /*    24     8 */
	struct fib6_info *         leaf;                 /*    32     8 */
	enum fib6_walk_state       state;                /*    40     4 */
	unsigned int               skip;                 /*    44     4 */
	unsigned int               count;                /*    48     4 */
	unsigned int               skip_in_node;         /*    52     4 */
	int                        (*func)(struct fib6_walker *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	void *                     args;                 /*    64     8 */

	/* size: 72, cachelines: 2, members: 10 */
	/* last cacheline: 8 bytes */
};
struct fib6_entry_notifier_info {
	struct fib_notifier_info   info;                 /*     0    16 */
	struct fib6_info *         rt;                   /*    16     8 */
	unsigned int               nsiblings;            /*    24     4 */

	/* size: 32, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct ipv6_route_iter {
	struct seq_net_private     p;                    /*     0     8 */
	struct fib6_walker         w;                    /*     8    72 */
	/* --- cacheline 1 boundary (64 bytes) was 16 bytes ago --- */
	loff_t                     skip;                 /*    80     8 */
	struct fib6_table *        tbl;                  /*    88     8 */
	int                        sernum;               /*    96     4 */

	/* size: 104, cachelines: 2, members: 5 */
	/* padding: 4 */
	/* last cacheline: 40 bytes */
};
struct bpf_iter__ipv6_route {
	union {
		struct bpf_iter_meta * meta;             /*     0     8 */
	} __attribute__((__aligned__(8)));               /*     0     8 */
	union {
		struct fib6_info * rt;                   /*     8     8 */
	} __attribute__((__aligned__(8)));               /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* forced alignments: 2 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct fib6_cleaner {
	struct fib6_walker         w;                    /*     0    72 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	struct net *               net;                  /*    72     8 */
	int                        (*func)(struct fib6_info *, void *); /*    80     8 */
	int                        sernum;               /*    88     4 */

	/* XXX 4 bytes hole, try to pack */

	void *                     arg;                  /*    96     8 */
	bool                       skip_notify;          /*   104     1 */

	/* size: 112, cachelines: 2, members: 6 */
	/* sum members: 101, holes: 1, sum holes: 4 */
	/* padding: 7 */
	/* last cacheline: 48 bytes */
};
struct fib6_dump_arg {
	struct net *               net;                  /*     0     8 */
	struct notifier_block *    nb;                   /*     8     8 */
	struct netlink_ext_ack *   extack;               /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct fib6_nh_pcpu_arg {
	struct fib6_info *         from;                 /*     0     8 */
	const struct fib6_table  * table;                /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct lookup_args {
	int                        offset;               /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	const struct in6_addr  *   addr;                 /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct ipv6_mreq {
	struct in6_addr            ipv6mr_multiaddr;     /*     0    16 */
	int                        ipv6mr_ifindex;       /*    16     4 */

	/* size: 20, cachelines: 1, members: 2 */
	/* last cacheline: 20 bytes */
};
struct in6_flowlabel_req {
	struct in6_addr            flr_dst;              /*     0    16 */
	__be32                     flr_label;            /*    16     4 */
	__u8                       flr_action;           /*    20     1 */
	__u8                       flr_share;            /*    21     1 */
	__u16                      flr_flags;            /*    22     2 */
	__u16                      flr_expires;          /*    24     2 */
	__u16                      flr_linger;           /*    26     2 */
	__u32                      __flr_pad;            /*    28     4 */

	/* size: 32, cachelines: 1, members: 8 */
	/* last cacheline: 32 bytes */
};
struct ip6_mtuinfo {
	struct sockaddr_in6        ip6m_addr;            /*     0    28 */
	__u32                      ip6m_mtu;             /*    28     4 */

	/* size: 32, cachelines: 1, members: 2 */
	/* last cacheline: 32 bytes */
};
struct ipv6_sr_hdr {
	__u8                       nexthdr;              /*     0     1 */
	__u8                       hdrlen;               /*     1     1 */
	__u8                       type;                 /*     2     1 */
	__u8                       segments_left;        /*     3     1 */
	__u8                       first_segment;        /*     4     1 */
	__u8                       flags;                /*     5     1 */
	__u16                      tag;                  /*     6     2 */
	struct in6_addr            segments[];           /*     8     0 */

	/* size: 8, cachelines: 1, members: 8 */
	/* last cacheline: 8 bytes */
};
struct nduseroptmsg {
	unsigned char              nduseropt_family;     /*     0     1 */
	unsigned char              nduseropt_pad1;       /*     1     1 */
	short unsigned int         nduseropt_opts_len;   /*     2     2 */
	int                        nduseropt_ifindex;    /*     4     4 */
	__u8                       nduseropt_icmp_type;  /*     8     1 */
	__u8                       nduseropt_icmp_code;  /*     9     1 */
	short unsigned int         nduseropt_pad2;       /*    10     2 */
	unsigned int               nduseropt_pad3;       /*    12     4 */

	/* size: 16, cachelines: 1, members: 8 */
	/* last cacheline: 16 bytes */
};
struct nd_msg {
	struct icmp6hdr            icmph;                /*     0     8 */
	struct in6_addr            target;               /*     8    16 */
	__u8                       opt[];                /*    24     0 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct rs_msg {
	struct icmp6hdr            icmph;                /*     0     8 */
	__u8                       opt[];                /*     8     0 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct ra_msg {
	struct icmp6hdr            icmph;                /*     0     8 */
	__be32                     reachable_time;       /*     8     4 */
	__be32                     retrans_timer;        /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct icmp6_filter {
	__u32                      data[8];              /*     0    32 */

	/* size: 32, cachelines: 1, members: 1 */
	/* last cacheline: 32 bytes */
};
struct raw6_sock {
	struct inet_sock           inet __attribute__((__aligned__(8))); /*     0  1096 */
	/* --- cacheline 17 boundary (1088 bytes) was 8 bytes ago --- */
	__u32                      checksum;             /*  1096     4 */
	__u32                      offset;               /*  1100     4 */
	struct icmp6_filter        filter;               /*  1104    32 */
	__u32                      ip6mr_table;          /*  1136     4 */

	/* XXX 4 bytes hole, try to pack */

	struct ipv6_pinfo          inet6;                /*  1144   152 */

	/* size: 1296, cachelines: 21, members: 6 */
	/* sum members: 1292, holes: 1, sum holes: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct raw6_frag_vec {
	struct msghdr *            msg;                  /*     0     8 */
	int                        hlen;                 /*     8     4 */
	char                       c[4];                 /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct ipv6_destopt_hao {
	__u8                       type;                 /*     0     1 */
	__u8                       length;               /*     1     1 */
	struct in6_addr            addr;                 /*     2    16 */

	/* size: 18, cachelines: 1, members: 3 */
	/* last cacheline: 18 bytes */
} __attribute__((__packed__));
struct icmpv6_msg {
	struct sk_buff *           skb;                  /*     0     8 */
	int                        offset;               /*     8     4 */
	uint8_t                    type;                 /*    12     1 */

	/* size: 16, cachelines: 1, members: 3 */
	/* padding: 3 */
	/* last cacheline: 16 bytes */
};
struct icmp6_err {
	int                        err;                  /*     0     4 */
	int                        fatal;                /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct mld_msg {
	struct icmp6hdr            mld_hdr;              /*     0     8 */
	struct in6_addr            mld_mca;              /*     8    16 */

	/* size: 24, cachelines: 1, members: 2 */
	/* last cacheline: 24 bytes */
};
struct mld2_grec {
	__u8                       grec_type;            /*     0     1 */
	__u8                       grec_auxwords;        /*     1     1 */
	__be16                     grec_nsrcs;           /*     2     2 */
	struct in6_addr            grec_mca;             /*     4    16 */
	struct in6_addr            grec_src[];           /*    20     0 */

	/* size: 20, cachelines: 1, members: 5 */
	/* last cacheline: 20 bytes */
};
struct mld2_report {
	struct icmp6hdr            mld2r_hdr;            /*     0     8 */
	struct mld2_grec           mld2r_grec[];         /*     8     0 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct mld2_query {
	struct icmp6hdr            mld2q_hdr;            /*     0     8 */
	struct in6_addr            mld2q_mca;            /*     8    16 */
	__u8                       mld2q_qrv:3;          /*    24: 0  1 */
	__u8                       mld2q_suppress:1;     /*    24: 3  1 */
	__u8                       mld2q_resv2:4;        /*    24: 4  1 */
	__u8                       mld2q_qqic;           /*    25     1 */
	__be16                     mld2q_nsrcs;          /*    26     2 */
	struct in6_addr            mld2q_srcs[];         /*    28     0 */

	/* size: 28, cachelines: 1, members: 8 */
	/* last cacheline: 28 bytes */
};
struct igmp6_mc_iter_state {
	struct seq_net_private     p;                    /*     0     8 */
	struct net_device *        dev;                  /*     8     8 */
	struct inet6_dev *         idev;                 /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct igmp6_mcf_iter_state {
	struct seq_net_private     p;                    /*     0     8 */
	struct net_device *        dev;                  /*     8     8 */
	struct inet6_dev *         idev;                 /*    16     8 */
	struct ifmcaddr6 *         im;                   /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct frag_queue {
	struct inet_frag_queue     q __attribute__((__aligned__(8))); /*     0   208 */
	/* --- cacheline 3 boundary (192 bytes) was 16 bytes ago --- */
	int                        iif;                  /*   208     4 */
	__u16                      nhoffset;             /*   212     2 */
	u8                         ecn;                  /*   214     1 */

	/* size: 216, cachelines: 4, members: 4 */
	/* padding: 1 */
	/* forced alignments: 1 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
struct tcp6_pseudohdr {
	struct in6_addr            saddr;                /*     0    16 */
	struct in6_addr            daddr;                /*    16    16 */
	__be32                     len;                  /*    32     4 */
	__be32                     protocol;             /*    36     4 */

	/* size: 40, cachelines: 1, members: 4 */
	/* last cacheline: 40 bytes */
};
struct rt0_hdr {
	struct ipv6_rt_hdr         rt_hdr;               /*     0     4 */
	__u32                      reserved;             /*     4     4 */
	struct in6_addr            addr[];               /*     8     0 */

	/* size: 8, cachelines: 1, members: 3 */
	/* last cacheline: 8 bytes */
};
struct ipv6_rpl_sr_hdr {
	__u8                       nexthdr;              /*     0     1 */
	__u8                       hdrlen;               /*     1     1 */
	__u8                       type;                 /*     2     1 */
	__u8                       segments_left;        /*     3     1 */
	__u32                      cmpre:4;              /*     4: 0  4 */
	__u32                      cmpri:4;              /*     4: 4  4 */
	__u32                      reserved:4;           /*     4: 8  4 */
	__u32                      pad:4;                /*     4:12  4 */
	__u32                      reserved1:16;         /*     4:16  4 */
	union {
		struct in6_addr    addr[0];              /*     8     0 */
		__u8               data[0];              /*     8     0 */
	} segments;                                      /*     8     0 */

	/* size: 8, cachelines: 1, members: 10 */
	/* last cacheline: 8 bytes */
};
struct tlvtype_proc {
	int                        type;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	bool                       (*func)(struct sk_buff *, int); /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct ip6fl_iter_state {
	struct seq_net_private     p;                    /*     0     8 */
	struct pid_namespace *     pid_ns;               /*     8     8 */
	int                        bucket;               /*    16     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct sr6_tlv {
	__u8                       type;                 /*     0     1 */
	__u8                       len;                  /*     1     1 */
	__u8                       data[];               /*     2     0 */

	/* size: 2, cachelines: 1, members: 3 */
	/* last cacheline: 2 bytes */
};
struct if_set {
	if_mask                    ifs_bits[8];          /*     0    32 */

	/* size: 32, cachelines: 1, members: 1 */
	/* last cacheline: 32 bytes */
};
struct mif6ctl {
	mifi_t                     mif6c_mifi;           /*     0     2 */
	unsigned char              mif6c_flags;          /*     2     1 */
	unsigned char              vifc_threshold;       /*     3     1 */
	__u16                      mif6c_pifi;           /*     4     2 */

	/* XXX 2 bytes hole, try to pack */

	unsigned int               vifc_rate_limit;      /*     8     4 */

	/* size: 12, cachelines: 1, members: 5 */
	/* sum members: 10, holes: 1, sum holes: 2 */
	/* last cacheline: 12 bytes */
};
struct mf6cctl {
	struct sockaddr_in6        mf6cc_origin;         /*     0    28 */
	struct sockaddr_in6        mf6cc_mcastgrp;       /*    28    28 */
	mifi_t                     mf6cc_parent;         /*    56     2 */

	/* XXX 2 bytes hole, try to pack */

	struct if_set              mf6cc_ifset;          /*    60    32 */

	/* size: 92, cachelines: 2, members: 4 */
	/* sum members: 90, holes: 1, sum holes: 2 */
	/* last cacheline: 28 bytes */
};
struct sioc_sg_req6 {
	struct sockaddr_in6        src;                  /*     0    28 */
	struct sockaddr_in6        grp;                  /*    28    28 */
	long unsigned int          pktcnt;               /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	long unsigned int          bytecnt;              /*    64     8 */
	long unsigned int          wrong_if;             /*    72     8 */

	/* size: 80, cachelines: 2, members: 5 */
	/* last cacheline: 16 bytes */
};
struct sioc_mif_req6 {
	mifi_t                     mifi;                 /*     0     2 */

	/* XXX 6 bytes hole, try to pack */

	long unsigned int          icount;               /*     8     8 */
	long unsigned int          ocount;               /*    16     8 */
	long unsigned int          ibytes;               /*    24     8 */
	long unsigned int          obytes;               /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* sum members: 34, holes: 1, sum holes: 6 */
	/* last cacheline: 40 bytes */
};
struct mrt6msg {
	__u8                       im6_mbz;              /*     0     1 */
	__u8                       im6_msgtype;          /*     1     1 */
	__u16                      im6_mif;              /*     2     2 */
	__u32                      im6_pad;              /*     4     4 */
	struct in6_addr            im6_src;              /*     8    16 */
	struct in6_addr            im6_dst;              /*    24    16 */

	/* size: 40, cachelines: 1, members: 6 */
	/* last cacheline: 40 bytes */
};
struct mfc6_cache_cmp_arg {
	struct in6_addr            mf6c_mcastgrp;        /*     0    16 */
	struct in6_addr            mf6c_origin;          /*    16    16 */

	/* size: 32, cachelines: 1, members: 2 */
	/* last cacheline: 32 bytes */
};
struct mfc6_cache {
	struct mr_mfc              _c __attribute__((__aligned__(8))); /*     0   152 */
	/* --- cacheline 2 boundary (128 bytes) was 24 bytes ago --- */
	union {
		struct {
			struct in6_addr mf6c_mcastgrp;   /*   152    16 */
			struct in6_addr mf6c_origin;     /*   168    16 */
		};                                       /*   152    32 */
		struct mfc6_cache_cmp_arg cmparg;        /*   152    32 */
	};                                               /*   152    32 */

	/* size: 184, cachelines: 3, members: 2 */
	/* forced alignments: 1 */
	/* last cacheline: 56 bytes */
} __attribute__((__aligned__(8)));
struct ip6mr_result {
	struct mr_table *          mrt;                  /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct compat_sioc_sg_req6 {
	struct sockaddr_in6        src;                  /*     0    28 */
	struct sockaddr_in6        grp;                  /*    28    28 */
	compat_ulong_t             pktcnt;               /*    56     4 */
	compat_ulong_t             bytecnt;              /*    60     4 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	compat_ulong_t             wrong_if;             /*    64     4 */

	/* size: 68, cachelines: 2, members: 5 */
	/* last cacheline: 4 bytes */
};
struct compat_sioc_mif_req6 {
	mifi_t                     mifi;                 /*     0     2 */

	/* XXX 2 bytes hole, try to pack */

	compat_ulong_t             icount;               /*     4     4 */
	compat_ulong_t             ocount;               /*     8     4 */
	compat_ulong_t             ibytes;               /*    12     4 */
	compat_ulong_t             obytes;               /*    16     4 */

	/* size: 20, cachelines: 1, members: 5 */
	/* sum members: 18, holes: 1, sum holes: 2 */
	/* last cacheline: 20 bytes */
};
struct xfrm6_protocol {
	int                        (*handler)(struct sk_buff *); /*     0     8 */
	int                        (*input_handler)(struct sk_buff *, int, __be32, int); /*     8     8 */
	int                        (*cb_handler)(struct sk_buff *, int); /*    16     8 */
	int                        (*err_handler)(struct sk_buff *, struct inet6_skb_parm *, u8, u8, int, __be32); /*    24     8 */
	struct xfrm6_protocol *    next;                 /*    32     8 */
	int                        priority;             /*    40     4 */

	/* size: 48, cachelines: 1, members: 6 */
	/* padding: 4 */
	/* last cacheline: 48 bytes */
};
struct br_input_skb_cb {
	struct net_device *        brdev;                /*     0     8 */
	u16                        frag_max_size;        /*     8     2 */
	u8                         igmp;                 /*    10     1 */
	u8                         mrouters_only:1;      /*    11: 0  1 */
	u8                         proxyarp_replied:1;   /*    11: 1  1 */
	u8                         src_port_isolated:1;  /*    11: 2  1 */
	u8                         vlan_filtered:1;      /*    11: 3  1 */
	u8                         br_netfilter_broute:1; /*    11: 4  1 */

	/* XXX 3 bits hole, try to pack */

	int                        offload_fwd_mark;     /*    12     4 */

	/* size: 16, cachelines: 1, members: 9 */
	/* sum members: 15 */
	/* sum bitfield members: 5 bits, bit holes: 1, sum bit holes: 3 bits */
	/* last cacheline: 16 bytes */
};
struct fib6_rule {
	struct fib_rule            common __attribute__((__aligned__(8))); /*     0   216 */
	/* --- cacheline 3 boundary (192 bytes) was 24 bytes ago --- */
	struct rt6key              src;                  /*   216    20 */
	struct rt6key              dst;                  /*   236    20 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	u8                         tclass;               /*   256     1 */

	/* size: 264, cachelines: 5, members: 4 */
	/* padding: 7 */
	/* forced alignments: 1 */
	/* last cacheline: 8 bytes */
} __attribute__((__aligned__(8)));
struct netlbl_calipso_ops {
	int                        (*doi_add)(struct calipso_doi *, struct netlbl_audit *); /*     0     8 */
	void                       (*doi_free)(struct calipso_doi *); /*     8     8 */
	int                        (*doi_remove)(u32, struct netlbl_audit *); /*    16     8 */
	struct calipso_doi *       (*doi_getdef)(u32);   /*    24     8 */
	void                       (*doi_putdef)(struct calipso_doi *); /*    32     8 */
	int                        (*doi_walk)(u32 *, int (*)(struct calipso_doi *, void *), void *); /*    40     8 */
	int                        (*sock_getattr)(struct sock *, struct netlbl_lsm_secattr *); /*    48     8 */
	int                        (*sock_setattr)(struct sock *, const struct calipso_doi  *, const struct netlbl_lsm_secattr  *); /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	void                       (*sock_delattr)(struct sock *); /*    64     8 */
	int                        (*req_setattr)(struct request_sock *, const struct calipso_doi  *, const struct netlbl_lsm_secattr  *); /*    72     8 */
	void                       (*req_delattr)(struct request_sock *); /*    80     8 */
	int                        (*opt_getattr)(const unsigned char  *, struct netlbl_lsm_secattr *); /*    88     8 */
	unsigned char *            (*skbuff_optptr)(const struct sk_buff  *); /*    96     8 */
	int                        (*skbuff_setattr)(struct sk_buff *, const struct calipso_doi  *, const struct netlbl_lsm_secattr  *); /*   104     8 */
	int                        (*skbuff_delattr)(struct sk_buff *); /*   112     8 */
	void                       (*cache_invalidate)(void); /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	int                        (*cache_add)(const unsigned char  *, const struct netlbl_lsm_secattr  *); /*   128     8 */

	/* size: 136, cachelines: 3, members: 17 */
	/* last cacheline: 8 bytes */
};
struct calipso_doi {
	u32                        doi;                  /*     0     4 */
	u32                        type;                 /*     4     4 */
	refcount_t                 refcount;             /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           list;                 /*    16    16 */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*    32    16 */

	/* size: 48, cachelines: 1, members: 5 */
	/* sum members: 44, holes: 1, sum holes: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct calipso_map_cache_bkt {
	spinlock_t                 lock;                 /*     0     4 */
	u32                        size;                 /*     4     4 */
	struct list_head           list;                 /*     8    16 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct calipso_map_cache_entry {
	u32                        hash;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	unsigned char *            key;                  /*     8     8 */
	size_t                     key_len;              /*    16     8 */
	struct netlbl_lsm_cache *  lsm_data;             /*    24     8 */
	u32                        activity;             /*    32     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           list;                 /*    40    16 */

	/* size: 56, cachelines: 1, members: 6 */
	/* sum members: 48, holes: 2, sum holes: 8 */
	/* last cacheline: 56 bytes */
};
struct sockaddr_pkt {
	short unsigned int         spkt_family;          /*     0     2 */
	unsigned char              spkt_device[14];      /*     2    14 */
	__be16                     spkt_protocol;        /*    16     2 */

	/* size: 18, cachelines: 1, members: 3 */
	/* last cacheline: 18 bytes */
};
struct sockaddr_ll {
	short unsigned int         sll_family;           /*     0     2 */
	__be16                     sll_protocol;         /*     2     2 */
	int                        sll_ifindex;          /*     4     4 */
	short unsigned int         sll_hatype;           /*     8     2 */
	unsigned char              sll_pkttype;          /*    10     1 */
	unsigned char              sll_halen;            /*    11     1 */
	unsigned char              sll_addr[8];          /*    12     8 */

	/* size: 20, cachelines: 1, members: 7 */
	/* last cacheline: 20 bytes */
};
struct tpacket_stats {
	unsigned int               tp_packets;           /*     0     4 */
	unsigned int               tp_drops;             /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct tpacket_stats_v3 {
	unsigned int               tp_packets;           /*     0     4 */
	unsigned int               tp_drops;             /*     4     4 */
	unsigned int               tp_freeze_q_cnt;      /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct tpacket_rollover_stats {
	__u64                      tp_all __attribute__((__aligned__(8))); /*     0     8 */
	__u64                      tp_huge __attribute__((__aligned__(8))); /*     8     8 */
	__u64                      tp_failed __attribute__((__aligned__(8))); /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* forced alignments: 3 */
	/* last cacheline: 24 bytes */
} __attribute__((__aligned__(8)));
union tpacket_stats_u {
	struct tpacket_stats       stats1;             /*     0     8 */
	struct tpacket_stats_v3    stats3;             /*     0    12 */
};
struct tpacket_auxdata {
	__u32                      tp_status;            /*     0     4 */
	__u32                      tp_len;               /*     4     4 */
	__u32                      tp_snaplen;           /*     8     4 */
	__u16                      tp_mac;               /*    12     2 */
	__u16                      tp_net;               /*    14     2 */
	__u16                      tp_vlan_tci;          /*    16     2 */
	__u16                      tp_vlan_tpid;         /*    18     2 */

	/* size: 20, cachelines: 1, members: 7 */
	/* last cacheline: 20 bytes */
};
struct tpacket_hdr {
	long unsigned int          tp_status;            /*     0     8 */
	unsigned int               tp_len;               /*     8     4 */
	unsigned int               tp_snaplen;           /*    12     4 */
	short unsigned int         tp_mac;               /*    16     2 */
	short unsigned int         tp_net;               /*    18     2 */
	unsigned int               tp_sec;               /*    20     4 */
	unsigned int               tp_usec;              /*    24     4 */

	/* size: 32, cachelines: 1, members: 7 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct tpacket2_hdr {
	__u32                      tp_status;            /*     0     4 */
	__u32                      tp_len;               /*     4     4 */
	__u32                      tp_snaplen;           /*     8     4 */
	__u16                      tp_mac;               /*    12     2 */
	__u16                      tp_net;               /*    14     2 */
	__u32                      tp_sec;               /*    16     4 */
	__u32                      tp_nsec;              /*    20     4 */
	__u16                      tp_vlan_tci;          /*    24     2 */
	__u16                      tp_vlan_tpid;         /*    26     2 */
	__u8                       tp_padding[4];        /*    28     4 */

	/* size: 32, cachelines: 1, members: 10 */
	/* last cacheline: 32 bytes */
};
struct tpacket_hdr_variant1 {
	__u32                      tp_rxhash;            /*     0     4 */
	__u32                      tp_vlan_tci;          /*     4     4 */
	__u16                      tp_vlan_tpid;         /*     8     2 */
	__u16                      tp_padding;           /*    10     2 */

	/* size: 12, cachelines: 1, members: 4 */
	/* last cacheline: 12 bytes */
};
struct tpacket3_hdr {
	__u32                      tp_next_offset;       /*     0     4 */
	__u32                      tp_sec;               /*     4     4 */
	__u32                      tp_nsec;              /*     8     4 */
	__u32                      tp_snaplen;           /*    12     4 */
	__u32                      tp_len;               /*    16     4 */
	__u32                      tp_status;            /*    20     4 */
	__u16                      tp_mac;               /*    24     2 */
	__u16                      tp_net;               /*    26     2 */
	union {
		struct tpacket_hdr_variant1 hv1;         /*    28    12 */
	};                                               /*    28    12 */
	__u8                       tp_padding[8];        /*    40     8 */

	/* size: 48, cachelines: 1, members: 10 */
	/* last cacheline: 48 bytes */
};
struct tpacket_bd_ts {
	unsigned int               ts_sec;               /*     0     4 */
	union {
		unsigned int       ts_usec;              /*     4     4 */
		unsigned int       ts_nsec;              /*     4     4 */
	};                                               /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct tpacket_hdr_v1 {
	__u32                      block_status;         /*     0     4 */
	__u32                      num_pkts;             /*     4     4 */
	__u32                      offset_to_first_pkt;  /*     8     4 */
	__u32                      blk_len;              /*    12     4 */
	__u64                      seq_num __attribute__((__aligned__(8))); /*    16     8 */
	struct tpacket_bd_ts       ts_first_pkt;         /*    24     8 */
	struct tpacket_bd_ts       ts_last_pkt;          /*    32     8 */

	/* size: 40, cachelines: 1, members: 7 */
	/* forced alignments: 1 */
	/* last cacheline: 40 bytes */
} __attribute__((__aligned__(8)));
union tpacket_bd_header_u {
	struct tpacket_hdr_v1      bh1 __attribute__((__aligned__(8))); /*     0    40 */
};
struct tpacket_block_desc {
	__u32                      version;              /*     0     4 */
	__u32                      offset_to_priv;       /*     4     4 */
	union tpacket_bd_header_u  hdr __attribute__((__aligned__(8))); /*     8    40 */

	/* size: 48, cachelines: 1, members: 3 */
	/* forced alignments: 1 */
	/* last cacheline: 48 bytes */
} __attribute__((__aligned__(8)));
struct tpacket_req {
	unsigned int               tp_block_size;        /*     0     4 */
	unsigned int               tp_block_nr;          /*     4     4 */
	unsigned int               tp_frame_size;        /*     8     4 */
	unsigned int               tp_frame_nr;          /*    12     4 */

	/* size: 16, cachelines: 1, members: 4 */
	/* last cacheline: 16 bytes */
};
struct tpacket_req3 {
	unsigned int               tp_block_size;        /*     0     4 */
	unsigned int               tp_block_nr;          /*     4     4 */
	unsigned int               tp_frame_size;        /*     8     4 */
	unsigned int               tp_frame_nr;          /*    12     4 */
	unsigned int               tp_retire_blk_tov;    /*    16     4 */
	unsigned int               tp_sizeof_priv;       /*    20     4 */
	unsigned int               tp_feature_req_word;  /*    24     4 */

	/* size: 28, cachelines: 1, members: 7 */
	/* last cacheline: 28 bytes */
};
union tpacket_req_u {
	struct tpacket_req         req;                /*     0    16 */
	struct tpacket_req3        req3;               /*     0    28 */
};
struct fanout_args {
	__u16                      id;                   /*     0     2 */
	__u16                      type_flags;           /*     2     2 */
	__u32                      max_num_members;      /*     4     4 */

	/* size: 8, cachelines: 1, members: 3 */
	/* last cacheline: 8 bytes */
};
struct virtio_net_hdr {
	__u8                       flags;                /*     0     1 */
	__u8                       gso_type;             /*     1     1 */
	__virtio16                 hdr_len;              /*     2     2 */
	__virtio16                 gso_size;             /*     4     2 */
	__virtio16                 csum_start;           /*     6     2 */
	__virtio16                 csum_offset;          /*     8     2 */

	/* size: 10, cachelines: 1, members: 6 */
	/* last cacheline: 10 bytes */
};
struct packet_mclist {
	struct packet_mclist *     next;                 /*     0     8 */
	int                        ifindex;              /*     8     4 */
	int                        count;                /*    12     4 */
	short unsigned int         type;                 /*    16     2 */
	short unsigned int         alen;                 /*    18     2 */
	unsigned char              addr[32];             /*    20    32 */

	/* size: 56, cachelines: 1, members: 6 */
	/* padding: 4 */
	/* last cacheline: 56 bytes */
};
struct tpacket_kbdq_core {
	struct pgv *               pkbdq;                /*     0     8 */
	unsigned int               feature_req_word;     /*     8     4 */
	unsigned int               hdrlen;               /*    12     4 */
	unsigned char              reset_pending_on_curr_blk; /*    16     1 */
	unsigned char              delete_blk_timer;     /*    17     1 */
	short unsigned int         kactive_blk_num;      /*    18     2 */
	short unsigned int         blk_sizeof_priv;      /*    20     2 */
	short unsigned int         last_kactive_blk_num; /*    22     2 */
	char *                     pkblk_start;          /*    24     8 */
	char *                     pkblk_end;            /*    32     8 */
	int                        kblk_size;            /*    40     4 */
	unsigned int               max_frame_len;        /*    44     4 */
	unsigned int               knum_blocks;          /*    48     4 */

	/* XXX 4 bytes hole, try to pack */

	uint64_t                   knxt_seq_num;         /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	char *                     prev;                 /*    64     8 */
	char *                     nxt_offset;           /*    72     8 */
	struct sk_buff *           skb;                  /*    80     8 */
	rwlock_t                   blk_fill_in_prog_lock; /*    88     8 */
	short unsigned int         retire_blk_tov;       /*    96     2 */
	short unsigned int         version;              /*    98     2 */

	/* XXX 4 bytes hole, try to pack */

	long unsigned int          tov_in_jiffies;       /*   104     8 */
	struct timer_list          retire_blk_timer;     /*   112    72 */

	/* size: 184, cachelines: 3, members: 22 */
	/* sum members: 176, holes: 2, sum holes: 8 */
	/* last cacheline: 56 bytes */
};
struct pgv {
	char *                     buffer;               /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct packet_ring_buffer {
	struct pgv *               pg_vec;               /*     0     8 */
	unsigned int               head;                 /*     8     4 */
	unsigned int               frames_per_block;     /*    12     4 */
	unsigned int               frame_size;           /*    16     4 */
	unsigned int               frame_max;            /*    20     4 */
	unsigned int               pg_vec_order;         /*    24     4 */
	unsigned int               pg_vec_pages;         /*    28     4 */
	unsigned int               pg_vec_len;           /*    32     4 */

	/* XXX 4 bytes hole, try to pack */

	unsigned int *             pending_refcnt;       /*    40     8 */
	union {
		long unsigned int * rx_owner_map;        /*    48     8 */
		struct tpacket_kbdq_core prb_bdqc;       /*    48   184 */
	};                                               /*    48   184 */

	/* size: 232, cachelines: 4, members: 10 */
	/* sum members: 228, holes: 1, sum holes: 4 */
	/* last cacheline: 40 bytes */
};
struct packet_fanout {
	possible_net_t             net;                  /*     0     8 */
	unsigned int               num_members;          /*     8     4 */
	u32                        max_num_members;      /*    12     4 */
	u16                        id;                   /*    16     2 */
	u8                         type;                 /*    18     1 */
	u8                         flags;                /*    19     1 */

	/* XXX 4 bytes hole, try to pack */

	union {
		atomic_t           rr_cur;               /*    24     4 */
		struct bpf_prog *  bpf_prog;             /*    24     8 */
	};                                               /*    24     8 */
	struct list_head           list;                 /*    32    16 */
	spinlock_t                 lock;                 /*    48     4 */
	refcount_t                 sk_ref;               /*    52     4 */

	/* XXX 8 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	struct packet_type         prot_hook __attribute__((__aligned__(64))); /*    64    96 */
	/* --- cacheline 2 boundary (128 bytes) was 32 bytes ago --- */
	struct sock *              arr[];                /*   160     0 */

	/* size: 192, cachelines: 3, members: 12 */
	/* sum members: 148, holes: 2, sum holes: 12 */
	/* padding: 32 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 8 */
} __attribute__((__aligned__(64)));
struct packet_rollover {
	int                        sock;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	atomic_long_t              num;                  /*     8     8 */
	atomic_long_t              num_huge;             /*    16     8 */
	atomic_long_t              num_failed;           /*    24     8 */

	/* XXX 32 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	u32                        history[16] __attribute__((__aligned__(64))); /*    64    64 */

	/* size: 128, cachelines: 2, members: 5 */
	/* sum members: 92, holes: 2, sum holes: 36 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 32 */
} __attribute__((__aligned__(64)));
struct packet_sock {
	struct sock                sk __attribute__((__aligned__(8))); /*     0   872 */
	/* --- cacheline 13 boundary (832 bytes) was 40 bytes ago --- */
	struct packet_fanout *     fanout;               /*   872     8 */
	union tpacket_stats_u      stats;                /*   880    12 */

	/* XXX 4 bytes hole, try to pack */

	/* --- cacheline 14 boundary (896 bytes) --- */
	struct packet_ring_buffer  rx_ring;              /*   896   232 */
	/* --- cacheline 17 boundary (1088 bytes) was 40 bytes ago --- */
	struct packet_ring_buffer  tx_ring;              /*  1128   232 */
	/* --- cacheline 21 boundary (1344 bytes) was 16 bytes ago --- */
	int                        copy_thresh;          /*  1360     4 */
	spinlock_t                 bind_lock;            /*  1364     4 */
	struct mutex               pg_vec_lock;          /*  1368    32 */
	unsigned int               running;              /*  1400     4 */
	unsigned int               auxdata:1;            /*  1404: 0  4 */
	unsigned int               origdev:1;            /*  1404: 1  4 */
	unsigned int               has_vnet_hdr:1;       /*  1404: 2  4 */
	unsigned int               tp_loss:1;            /*  1404: 3  4 */
	unsigned int               tp_tx_has_off:1;      /*  1404: 4  4 */

	/* XXX 27 bits hole, try to pack */

	/* --- cacheline 22 boundary (1408 bytes) --- */
	int                        pressure;             /*  1408     4 */
	int                        ifindex;              /*  1412     4 */
	__be16                     num;                  /*  1416     2 */

	/* XXX 6 bytes hole, try to pack */

	struct packet_rollover *   rollover;             /*  1424     8 */
	struct packet_mclist *     mclist;               /*  1432     8 */
	atomic_t                   mapped;               /*  1440     4 */
	enum tpacket_versions      tp_version;           /*  1444     4 */
	unsigned int               tp_hdrlen;            /*  1448     4 */
	unsigned int               tp_reserve;           /*  1452     4 */
	unsigned int               tp_tstamp;            /*  1456     4 */

	/* XXX 4 bytes hole, try to pack */

	struct completion          skb_completion;       /*  1464    32 */
	/* --- cacheline 23 boundary (1472 bytes) was 24 bytes ago --- */
	struct net_device *        cached_dev;           /*  1496     8 */
	int                        (*xmit)(struct sk_buff *); /*  1504     8 */

	/* XXX 24 bytes hole, try to pack */

	/* --- cacheline 24 boundary (1536 bytes) --- */
	struct packet_type         prot_hook __attribute__((__aligned__(64))); /*  1536    96 */

	/* XXX 32 bytes hole, try to pack */

	/* --- cacheline 26 boundary (1664 bytes) --- */
	atomic_t                   tp_drops __attribute__((__aligned__(64))); /*  1664     4 */

	/* size: 1728, cachelines: 27, members: 29 */
	/* sum members: 1594, holes: 5, sum holes: 70 */
	/* sum bitfield members: 5 bits, bit holes: 1, sum bit holes: 27 bits */
	/* padding: 60 */
	/* forced alignments: 3, forced holes: 2, sum forced holes: 56 */
} __attribute__((__aligned__(64)));
struct packet_mreq_max {
	int                        mr_ifindex;           /*     0     4 */
	short unsigned int         mr_type;              /*     4     2 */
	short unsigned int         mr_alen;              /*     6     2 */
	unsigned char              mr_address[32];       /*     8    32 */

	/* size: 40, cachelines: 1, members: 4 */
	/* last cacheline: 40 bytes */
};
union tpacket_uhdr {
	struct tpacket_hdr *       h1;                 /*     0     8 */
	struct tpacket2_hdr *      h2;                 /*     0     8 */
	struct tpacket3_hdr *      h3;                 /*     0     8 */
	void *                     raw;                /*     0     8 */
};
struct packet_skb_cb {
	union {
		struct sockaddr_pkt pkt;                 /*     0    18 */
		union {
			unsigned int origlen;            /*     0     4 */
			struct sockaddr_ll ll;           /*     0    20 */
		};                                       /*     0    20 */
	} sa;                                            /*     0    20 */

	/* size: 20, cachelines: 1, members: 1 */
	/* last cacheline: 20 bytes */
};
struct vlan_info {
	struct net_device *        real_dev;             /*     0     8 */
	struct vlan_group          grp;                  /*     8   152 */
	/* --- cacheline 2 boundary (128 bytes) was 32 bytes ago --- */
	struct list_head           vid_list;             /*   160    16 */
	unsigned int               nr_vids;              /*   176     4 */

	/* XXX 4 bytes hole, try to pack */

	struct callback_head       rcu __attribute__((__aligned__(8))); /*   184    16 */

	/* size: 200, cachelines: 4, members: 5 */
	/* sum members: 196, holes: 1, sum holes: 4 */
	/* forced alignments: 1, forced holes: 1, sum forced holes: 4 */
	/* last cacheline: 8 bytes */
} __attribute__((__aligned__(8)));
struct vlan_priority_tci_mapping {
	u32                        priority;             /*     0     4 */
	u16                        vlan_qos;             /*     4     2 */

	/* XXX 2 bytes hole, try to pack */

	struct vlan_priority_tci_mapping * next;         /*     8     8 */

	/* size: 16, cachelines: 1, members: 3 */
	/* sum members: 14, holes: 1, sum holes: 2 */
	/* last cacheline: 16 bytes */
};
struct vlan_dev_priv {
	unsigned int               nr_ingress_mappings;  /*     0     4 */
	u32                        ingress_priority_map[8]; /*     4    32 */
	unsigned int               nr_egress_mappings;   /*    36     4 */
	struct vlan_priority_tci_mapping * egress_priority_map[16]; /*    40   128 */
	/* --- cacheline 2 boundary (128 bytes) was 40 bytes ago --- */
	__be16                     vlan_proto;           /*   168     2 */
	u16                        vlan_id;              /*   170     2 */
	u16                        flags;                /*   172     2 */

	/* XXX 2 bytes hole, try to pack */

	struct net_device *        real_dev;             /*   176     8 */
	unsigned char              real_dev_addr[6];     /*   184     6 */

	/* XXX 2 bytes hole, try to pack */

	/* --- cacheline 3 boundary (192 bytes) --- */
	struct proc_dir_entry *    dent;                 /*   192     8 */
	struct vlan_pcpu_stats *   vlan_pcpu_stats;      /*   200     8 */
	struct netpoll *           netpoll;              /*   208     8 */

	/* size: 216, cachelines: 4, members: 12 */
	/* sum members: 212, holes: 2, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct vlan_group {
	unsigned int               nr_vlan_devs;         /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct hlist_node          hlist;                /*     8    16 */
	struct net_device * *      vlan_devices_arrays[2][8]; /*    24   128 */

	/* size: 152, cachelines: 3, members: 3 */
	/* sum members: 148, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct vlan_vid_info {
	struct list_head           list;                 /*     0    16 */
	__be16                     proto;                /*    16     2 */
	u16                        vid;                  /*    18     2 */
	int                        refcount;             /*    20     4 */

	/* size: 24, cachelines: 1, members: 4 */
	/* last cacheline: 24 bytes */
};
struct netlbl_af4list {
	__be32                     addr;                 /*     0     4 */
	__be32                     mask;                 /*     4     4 */
	u32                        valid;                /*     8     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           list;                 /*    16    16 */

	/* size: 32, cachelines: 1, members: 4 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct netlbl_af6list {
	struct in6_addr            addr;                 /*     0    16 */
	struct in6_addr            mask;                 /*    16    16 */
	u32                        valid;                /*    32     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           list;                 /*    40    16 */

	/* size: 56, cachelines: 1, members: 4 */
	/* sum members: 52, holes: 1, sum holes: 4 */
	/* last cacheline: 56 bytes */
};
struct netlbl_domaddr_map {
	struct list_head           list4;                /*     0    16 */
	struct list_head           list6;                /*    16    16 */

	/* size: 32, cachelines: 1, members: 2 */
	/* last cacheline: 32 bytes */
};
struct netlbl_dommap_def {
	u32                        type;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	union {
		struct netlbl_domaddr_map * addrsel;     /*     8     8 */
		struct cipso_v4_doi * cipso;             /*     8     8 */
		struct calipso_doi * calipso;            /*     8     8 */
	};                                               /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct netlbl_domaddr4_map {
	struct netlbl_dommap_def   def;                  /*     0    16 */
	struct netlbl_af4list      list;                 /*    16    32 */

	/* size: 48, cachelines: 1, members: 2 */
	/* last cacheline: 48 bytes */
};
struct netlbl_domaddr6_map {
	struct netlbl_dommap_def   def;                  /*     0    16 */
	struct netlbl_af6list      list;                 /*    16    56 */

	/* size: 72, cachelines: 2, members: 2 */
	/* last cacheline: 8 bytes */
};
struct netlbl_dom_map {
	char *                     domain;               /*     0     8 */
	u16                        family;               /*     8     2 */

	/* XXX 6 bytes hole, try to pack */

	struct netlbl_dommap_def   def;                  /*    16    16 */
	u32                        valid;                /*    32     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           list;                 /*    40    16 */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*    56    16 */

	/* size: 72, cachelines: 2, members: 6 */
	/* sum members: 62, holes: 2, sum holes: 10 */
	/* forced alignments: 1 */
	/* last cacheline: 8 bytes */
} __attribute__((__aligned__(8)));
struct netlbl_domhsh_tbl {
	struct list_head *         tbl;                  /*     0     8 */
	u32                        size;                 /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct netlbl_domhsh_walk_arg {
	struct netlink_callback *  nl_cb;                /*     0     8 */
	struct sk_buff *           skb;                  /*     8     8 */
	u32                        seq;                  /*    16     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct netlbl_unlhsh_tbl {
	struct list_head *         tbl;                  /*     0     8 */
	u32                        size;                 /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct netlbl_unlhsh_addr4 {
	u32                        secid;                /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct netlbl_af4list      list;                 /*     8    32 */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*    40    16 */

	/* size: 56, cachelines: 1, members: 3 */
	/* sum members: 52, holes: 1, sum holes: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 56 bytes */
} __attribute__((__aligned__(8)));
struct netlbl_unlhsh_addr6 {
	u32                        secid;                /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct netlbl_af6list      list;                 /*     8    56 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*    64    16 */

	/* size: 80, cachelines: 2, members: 3 */
	/* sum members: 76, holes: 1, sum holes: 4 */
	/* forced alignments: 1 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct netlbl_unlhsh_iface {
	int                        ifindex;              /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           addr4_list;           /*     8    16 */
	struct list_head           addr6_list;           /*    24    16 */
	u32                        valid;                /*    40     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           list;                 /*    48    16 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct callback_head       rcu __attribute__((__aligned__(8))); /*    64    16 */

	/* size: 80, cachelines: 2, members: 6 */
	/* sum members: 72, holes: 2, sum holes: 8 */
	/* forced alignments: 1 */
	/* last cacheline: 16 bytes */
} __attribute__((__aligned__(8)));
struct netlbl_unlhsh_walk_arg {
	struct netlink_callback *  nl_cb;                /*     0     8 */
	struct sk_buff *           skb;                  /*     8     8 */
	u32                        seq;                  /*    16     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct netlbl_cipsov4_doiwalk_arg {
	struct netlink_callback *  nl_cb;                /*     0     8 */
	struct sk_buff *           skb;                  /*     8     8 */
	u32                        seq;                  /*    16     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct netlbl_domhsh_walk_arg {
	struct netlbl_audit *      audit_info;           /*     0     8 */
	u32                        doi;                  /*     8     4 */

	/* size: 16, cachelines: 1, members: 2 */
	/* padding: 4 */
	/* last cacheline: 16 bytes */
};
struct netlbl_calipso_doiwalk_arg {
	struct netlink_callback *  nl_cb;                /*     0     8 */
	struct sk_buff *           skb;                  /*     8     8 */
	u32                        seq;                  /*    16     4 */

	/* size: 24, cachelines: 1, members: 3 */
	/* padding: 4 */
	/* last cacheline: 24 bytes */
};
struct dcbmsg {
	__u8                       dcb_family;           /*     0     1 */
	__u8                       cmd;                  /*     1     1 */
	__u16                      dcb_pad;              /*     2     2 */

	/* size: 4, cachelines: 1, members: 3 */
	/* last cacheline: 4 bytes */
};
struct dcb_app_type {
	int                        ifindex;              /*     0     4 */
	struct dcb_app             app;                  /*     4     4 */
	struct list_head           list;                 /*     8    16 */
	u8                         dcbx;                 /*    24     1 */

	/* size: 32, cachelines: 1, members: 4 */
	/* padding: 7 */
	/* last cacheline: 32 bytes */
};
struct dcb_ieee_app_prio_map {
	u64                        map[8];               /*     0    64 */

	/* size: 64, cachelines: 1, members: 1 */
};
struct dcb_ieee_app_dscp_map {
	u8                         map[64];              /*     0    64 */

	/* size: 64, cachelines: 1, members: 1 */
};
struct reply_func {
	int                        type;                 /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	int                        (*cb)(struct net_device *, struct nlmsghdr *, u32, struct nlattr * *, struct sk_buff *); /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct nsh_md1_ctx {
	__be32                     context[4];           /*     0    16 */

	/* size: 16, cachelines: 1, members: 1 */
	/* last cacheline: 16 bytes */
};
struct nsh_md2_tlv {
	__be16                     md_class;             /*     0     2 */
	u8                         type;                 /*     2     1 */
	u8                         length;               /*     3     1 */
	u8                         md_value[];           /*     4     0 */

	/* size: 4, cachelines: 1, members: 4 */
	/* last cacheline: 4 bytes */
};
struct nshhdr {
	__be16                     ver_flags_ttl_len;    /*     0     2 */
	u8                         mdtype;               /*     2     1 */
	u8                         np;                   /*     3     1 */
	__be32                     path_hdr;             /*     4     4 */
	union {
		struct nsh_md1_ctx md1;                  /*     8    16 */
		struct nsh_md2_tlv md2;                  /*     8     4 */
	};                                               /*     8    16 */

	/* size: 24, cachelines: 1, members: 5 */
	/* last cacheline: 24 bytes */
};
struct switchdev_trans {
	bool                       ph_prepare;           /*     0     1 */

	/* size: 1, cachelines: 1, members: 1 */
	/* last cacheline: 1 bytes */
};
struct switchdev_attr {
	struct net_device *        orig_dev;             /*     0     8 */
	enum switchdev_attr_id     id;                   /*     8     4 */
	u32                        flags;                /*    12     4 */
	void *                     complete_priv;        /*    16     8 */
	void                       (*complete)(struct net_device *, int, void *); /*    24     8 */
	union {
		u8                 stp_state;            /*    32     1 */
		long unsigned int  brport_flags;         /*    32     8 */
		bool               mrouter;              /*    32     1 */
		clock_t            ageing_time;          /*    32     8 */
		bool               vlan_filtering;       /*    32     1 */
		bool               mc_disabled;          /*    32     1 */
	} u;                                             /*    32     8 */

	/* size: 40, cachelines: 1, members: 6 */
	/* last cacheline: 40 bytes */
};
struct switchdev_obj {
	struct net_device *        orig_dev;             /*     0     8 */
	enum switchdev_obj_id      id;                   /*     8     4 */
	u32                        flags;                /*    12     4 */
	void *                     complete_priv;        /*    16     8 */
	void                       (*complete)(struct net_device *, int, void *); /*    24     8 */
	u64                        kabi_reserved1;       /*    32     8 */
	u64                        kabi_reserved2;       /*    40     8 */
	u64                        kabi_reserved3;       /*    48     8 */
	u64                        kabi_reserved4;       /*    56     8 */

	/* size: 64, cachelines: 1, members: 9 */
};
struct switchdev_notifier_info {
	struct net_device *        dev;                  /*     0     8 */
	struct netlink_ext_ack *   extack;               /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct switchdev_notifier_port_obj_info {
	struct switchdev_notifier_info info;             /*     0    16 */
	const struct switchdev_obj  * obj;               /*    16     8 */
	struct switchdev_trans *   trans;                /*    24     8 */
	bool                       handled;              /*    32     1 */

	/* size: 40, cachelines: 1, members: 4 */
	/* padding: 7 */
	/* last cacheline: 40 bytes */
};
struct switchdev_notifier_port_attr_info {
	struct switchdev_notifier_info info;             /*     0    16 */
	const struct switchdev_attr  * attr;             /*    16     8 */
	struct switchdev_trans *   trans;                /*    24     8 */
	bool                       handled;              /*    32     1 */

	/* size: 40, cachelines: 1, members: 4 */
	/* padding: 7 */
	/* last cacheline: 40 bytes */
};
struct switchdev_deferred_item {
	struct list_head           list;                 /*     0    16 */
	struct net_device *        dev;                  /*    16     8 */
	switchdev_deferred_func_t * func;                /*    24     8 */
	long unsigned int          data[];               /*    32     0 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct l3mdev_handler {
	lookup_by_table_id_t       dev_lookup;           /*     0     8 */

	/* size: 8, cachelines: 1, members: 1 */
	/* last cacheline: 8 bytes */
};
struct sockaddr_xdp {
	__u16                      sxdp_family;          /*     0     2 */
	__u16                      sxdp_flags;           /*     2     2 */
	__u32                      sxdp_ifindex;         /*     4     4 */
	__u32                      sxdp_queue_id;        /*     8     4 */
	__u32                      sxdp_shared_umem_fd;  /*    12     4 */

	/* size: 16, cachelines: 1, members: 5 */
	/* last cacheline: 16 bytes */
};
struct xdp_ring_offset {
	__u64                      producer;             /*     0     8 */
	__u64                      consumer;             /*     8     8 */
	__u64                      desc;                 /*    16     8 */
	__u64                      flags;                /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct xdp_mmap_offsets {
	struct xdp_ring_offset     rx;                   /*     0    32 */
	struct xdp_ring_offset     tx;                   /*    32    32 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	struct xdp_ring_offset     fr;                   /*    64    32 */
	struct xdp_ring_offset     cr;                   /*    96    32 */

	/* size: 128, cachelines: 2, members: 4 */
};
struct xdp_umem_reg {
	__u64                      addr;                 /*     0     8 */
	__u64                      len;                  /*     8     8 */
	__u32                      chunk_size;           /*    16     4 */
	__u32                      headroom;             /*    20     4 */
	__u32                      flags;                /*    24     4 */

	/* size: 32, cachelines: 1, members: 5 */
	/* padding: 4 */
	/* last cacheline: 32 bytes */
};
struct xdp_statistics {
	__u64                      rx_dropped;           /*     0     8 */
	__u64                      rx_invalid_descs;     /*     8     8 */
	__u64                      tx_invalid_descs;     /*    16     8 */
	__u64                      rx_ring_full;         /*    24     8 */
	__u64                      rx_fill_ring_empty_descs; /*    32     8 */
	__u64                      tx_ring_empty_descs;  /*    40     8 */

	/* size: 48, cachelines: 1, members: 6 */
	/* last cacheline: 48 bytes */
};
struct xdp_options {
	__u32                      flags;                /*     0     4 */

	/* size: 4, cachelines: 1, members: 1 */
	/* last cacheline: 4 bytes */
};
struct xdp_desc {
	__u64                      addr;                 /*     0     8 */
	__u32                      len;                  /*     8     4 */
	__u32                      options;              /*    12     4 */

	/* size: 16, cachelines: 1, members: 3 */
	/* last cacheline: 16 bytes */
};
struct xsk_queue {
	u32                        ring_mask;            /*     0     4 */
	u32                        nentries;             /*     4     4 */
	u32                        cached_prod;          /*     8     4 */
	u32                        cached_cons;          /*    12     4 */
	struct xdp_ring *          ring;                 /*    16     8 */
	u64                        invalid_descs;        /*    24     8 */
	u64                        queue_empty_descs;    /*    32     8 */

	/* size: 40, cachelines: 1, members: 7 */
	/* last cacheline: 40 bytes */
};
struct xdp_ring_offset_v1 {
	__u64                      producer;             /*     0     8 */
	__u64                      consumer;             /*     8     8 */
	__u64                      desc;                 /*    16     8 */

	/* size: 24, cachelines: 1, members: 3 */
	/* last cacheline: 24 bytes */
};
struct xdp_mmap_offsets_v1 {
	struct xdp_ring_offset_v1  rx;                   /*     0    24 */
	struct xdp_ring_offset_v1  tx;                   /*    24    24 */
	struct xdp_ring_offset_v1  fr;                   /*    48    24 */
	/* --- cacheline 1 boundary (64 bytes) was 8 bytes ago --- */
	struct xdp_ring_offset_v1  cr;                   /*    72    24 */

	/* size: 96, cachelines: 2, members: 4 */
	/* last cacheline: 32 bytes */
};
struct xsk_map_node {
	struct list_head           node;                 /*     0    16 */
	struct xsk_map *           map;                  /*    16     8 */
	struct xdp_sock * *        map_entry;            /*    24     8 */

	/* size: 32, cachelines: 1, members: 3 */
	/* last cacheline: 32 bytes */
};
struct xdp_ring {
	u32                        producer __attribute__((__aligned__(64))); /*     0     4 */

	/* XXX 60 bytes hole, try to pack */

	/* --- cacheline 1 boundary (64 bytes) --- */
	u32                        pad __attribute__((__aligned__(64))); /*    64     4 */

	/* XXX 60 bytes hole, try to pack */

	/* --- cacheline 2 boundary (128 bytes) --- */
	u32                        consumer __attribute__((__aligned__(64))); /*   128     4 */
	u32                        flags;                /*   132     4 */

	/* size: 192, cachelines: 3, members: 4 */
	/* sum members: 16, holes: 2, sum holes: 120 */
	/* padding: 56 */
	/* forced alignments: 3, forced holes: 2, sum forced holes: 120 */
} __attribute__((__aligned__(64)));
struct xdp_rxtx_ring {
	struct xdp_ring            ptrs __attribute__((__aligned__(64))); /*     0   192 */

	/* XXX last struct has 56 bytes of padding */

	/* --- cacheline 3 boundary (192 bytes) --- */
	struct xdp_desc            desc[] __attribute__((__aligned__(64))); /*   192     0 */

	/* size: 192, cachelines: 3, members: 2 */
	/* paddings: 1, sum paddings: 56 */
	/* forced alignments: 2 */
} __attribute__((__aligned__(64)));
struct xdp_umem_ring {
	struct xdp_ring            ptrs __attribute__((__aligned__(64))); /*     0   192 */

	/* XXX last struct has 56 bytes of padding */

	/* --- cacheline 3 boundary (192 bytes) --- */
	u64                        desc[] __attribute__((__aligned__(64))); /*   192     0 */

	/* size: 192, cachelines: 3, members: 2 */
	/* paddings: 1, sum paddings: 56 */
	/* forced alignments: 2 */
} __attribute__((__aligned__(64)));
struct xsk_dma_map {
	dma_addr_t *               dma_pages;            /*     0     8 */
	struct device *            dev;                  /*     8     8 */
	struct net_device *        netdev;               /*    16     8 */
	refcount_t                 users;                /*    24     4 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           list;                 /*    32    16 */
	u32                        dma_pages_cnt;        /*    48     4 */
	bool                       dma_need_sync;        /*    52     1 */

	/* size: 56, cachelines: 1, members: 7 */
	/* sum members: 49, holes: 1, sum holes: 4 */
	/* padding: 3 */
	/* last cacheline: 56 bytes */
};
struct pcibios_fwaddrmap {
	struct list_head           list;                 /*     0    16 */
	struct pci_dev *           dev;                  /*    16     8 */
	resource_size_t            fw_addr[17];          /*    24   136 */

	/* size: 160, cachelines: 3, members: 3 */
	/* last cacheline: 32 bytes */
};
struct pci_check_idx_range {
	int                        start;                /*     0     4 */
	int                        end;                  /*     4     4 */

	/* size: 8, cachelines: 1, members: 2 */
	/* last cacheline: 8 bytes */
};
struct pci_raw_ops {
	int                        (*read)(unsigned int, unsigned int, unsigned int, int, int, u32 *); /*     0     8 */
	int                        (*write)(unsigned int, unsigned int, unsigned int, int, int, u32); /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct pci_mmcfg_region {
	struct list_head           list;                 /*     0    16 */
	struct resource            res;                  /*    16    96 */
	/* --- cacheline 1 boundary (64 bytes) was 48 bytes ago --- */
	u64                        address;              /*   112     8 */
	char *                     virt;                 /*   120     8 */
	/* --- cacheline 2 boundary (128 bytes) --- */
	u16                        segment;              /*   128     2 */
	u8                         start_bus;            /*   130     1 */
	u8                         end_bus;              /*   131     1 */
	char                       name[30];             /*   132    30 */

	/* size: 168, cachelines: 3, members: 8 */
	/* padding: 6 */
	/* last cacheline: 40 bytes */
};
struct acpi_table_mcfg {
	struct acpi_table_header   header;               /*     0    36 */
	u8                         reserved[8];          /*    36     8 */

	/* size: 44, cachelines: 1, members: 2 */
	/* last cacheline: 44 bytes */
};
struct acpi_mcfg_allocation {
	u64                        address;              /*     0     8 */
	u16                        pci_segment;          /*     8     2 */
	u8                         start_bus_number;     /*    10     1 */
	u8                         end_bus_number;       /*    11     1 */
	u32                        reserved;             /*    12     4 */

	/* size: 16, cachelines: 1, members: 5 */
	/* last cacheline: 16 bytes */
};
struct pci_mmcfg_hostbridge_probe {
	u32                        bus;                  /*     0     4 */
	u32                        devfn;                /*     4     4 */
	u32                        vendor;               /*     8     4 */
	u32                        device;               /*    12     4 */
	const char  *              (*probe)(void);       /*    16     8 */

	/* size: 24, cachelines: 1, members: 5 */
	/* last cacheline: 24 bytes */
};
struct xen_pci_frontend_ops {
	int                        (*enable_msi)(struct pci_dev *, int *); /*     0     8 */
	void                       (*disable_msi)(struct pci_dev *); /*     8     8 */
	int                        (*enable_msix)(struct pci_dev *, int *, int); /*    16     8 */
	void                       (*disable_msix)(struct pci_dev *); /*    24     8 */

	/* size: 32, cachelines: 1, members: 4 */
	/* last cacheline: 32 bytes */
};
struct xen_msi_ops {
	int                        (*setup_msi_irqs)(struct pci_dev *, int, int); /*     0     8 */
	void                       (*teardown_msi_irqs)(struct pci_dev *); /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* last cacheline: 16 bytes */
};
struct pci_root_info {
	struct acpi_pci_root_info  common;               /*     0    56 */
	struct pci_sysdata         sd;                   /*    56    40 */
	/* --- cacheline 1 boundary (64 bytes) was 32 bytes ago --- */
	bool                       mcfg_added;           /*    96     1 */
	u8                         start_bus;            /*    97     1 */
	u8                         end_bus;              /*    98     1 */

	/* size: 104, cachelines: 2, members: 5 */
	/* padding: 5 */
	/* last cacheline: 40 bytes */
};
struct irq_info {
	u8                         bus;                  /*     0     1 */
	u8                         devfn;                /*     1     1 */
	struct {
		u8                 link;                 /*     2     1 */
		u16                bitmap;               /*     3     2 */
	} __attribute__((__packed__)) irq[4]; /*     2    12 */
	u8                         slot;                 /*    14     1 */
	u8                         rfu;                  /*    15     1 */

	/* size: 16, cachelines: 1, members: 5 */
	/* last cacheline: 16 bytes */
};
struct irq_routing_table {
	u32                        signature;            /*     0     4 */
	u16                        version;              /*     4     2 */
	u16                        size;                 /*     6     2 */
	u8                         rtr_bus;              /*     8     1 */
	u8                         rtr_devfn;            /*     9     1 */
	u16                        exclusive_irqs;       /*    10     2 */
	u16                        rtr_vendor;           /*    12     2 */
	u16                        rtr_device;           /*    14     2 */
	u32                        miniport_data;        /*    16     4 */
	u8                         rfu[11];              /*    20    11 */
	u8                         checksum;             /*    31     1 */
	struct irq_info            slots[];              /*    32     0 */

	/* size: 32, cachelines: 1, members: 12 */
	/* last cacheline: 32 bytes */
};
struct irq_router {
	char *                     name;                 /*     0     8 */
	u16                        vendor;               /*     8     2 */
	u16                        device;               /*    10     2 */

	/* XXX 4 bytes hole, try to pack */

	int                        (*get)(struct pci_dev *, struct pci_dev *, int); /*    16     8 */
	int                        (*set)(struct pci_dev *, struct pci_dev *, int, int); /*    24     8 */

	/* size: 32, cachelines: 1, members: 5 */
	/* sum members: 28, holes: 1, sum holes: 4 */
	/* last cacheline: 32 bytes */
};
struct irq_router_handler {
	u16                        vendor;               /*     0     2 */

	/* XXX 6 bytes hole, try to pack */

	int                        (*probe)(struct irq_router *, struct pci_dev *, u16); /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 10, holes: 1, sum holes: 6 */
	/* last cacheline: 16 bytes */
};
struct pci_setup_rom {
	struct setup_data          data;                 /*     0    16 */
	uint16_t                   vendor;               /*    16     2 */
	uint16_t                   devid;                /*    18     2 */

	/* XXX 4 bytes hole, try to pack */

	uint64_t                   pcilen;               /*    24     8 */
	long unsigned int          segment;              /*    32     8 */
	long unsigned int          bus;                  /*    40     8 */
	long unsigned int          device;               /*    48     8 */
	long unsigned int          function;             /*    56     8 */
	/* --- cacheline 1 boundary (64 bytes) --- */
	uint8_t                    romdata[];            /*    64     0 */

	/* size: 64, cachelines: 1, members: 9 */
	/* sum members: 60, holes: 1, sum holes: 4 */
};
struct pci_root_res {
	struct list_head           list;                 /*     0    16 */
	struct resource            res;                  /*    16    96 */

	/* size: 112, cachelines: 2, members: 2 */
	/* last cacheline: 48 bytes */
};
struct pci_root_info {
	struct list_head           list;                 /*     0    16 */
	char                       name[12];             /*    16    12 */

	/* XXX 4 bytes hole, try to pack */

	struct list_head           resources;            /*    32    16 */
	struct resource            busn;                 /*    48    96 */
	/* --- cacheline 2 boundary (128 bytes) was 16 bytes ago --- */
	int                        node;                 /*   144     4 */
	int                        link;                 /*   148     4 */

	/* size: 152, cachelines: 3, members: 6 */
	/* sum members: 148, holes: 1, sum holes: 4 */
	/* last cacheline: 24 bytes */
};
struct amd_hostbridge {
	u32                        bus;                  /*     0     4 */
	u32                        slot;                 /*     4     4 */
	u32                        device;               /*     8     4 */

	/* size: 12, cachelines: 1, members: 3 */
	/* last cacheline: 12 bytes */
};
struct saved_msr {
	bool                       valid;                /*     0     1 */

	/* XXX 7 bytes hole, try to pack */

	struct msr_info            info;                 /*     8    32 */

	/* XXX last struct has 4 bytes of padding */

	/* size: 40, cachelines: 1, members: 2 */
	/* sum members: 33, holes: 1, sum holes: 7 */
	/* paddings: 1, sum paddings: 4 */
	/* last cacheline: 40 bytes */
};
struct saved_msrs {
	unsigned int               num;                  /*     0     4 */

	/* XXX 4 bytes hole, try to pack */

	struct saved_msr *         array;                /*     8     8 */

	/* size: 16, cachelines: 1, members: 2 */
	/* sum members: 12, holes: 1, sum holes: 4 */
	/* last cacheline: 16 bytes */
};
struct saved_context {
	struct pt_regs             regs;                 /*     0   168 */
	/* --- cacheline 2 boundary (128 bytes) was 40 bytes ago --- */
	u16                        ds;                   /*   168     2 */
	u16                        es;                   /*   170     2 */
	u16                        fs;                   /*   172     2 */
	u16                        gs;                   /*   174     2 */
	long unsigned int          kernelmode_gs_base;   /*   176     8 */
	long unsigned int          usermode_gs_base;     /*   184     8 */
	/* --- cacheline 3 boundary (192 bytes) --- */
	long unsigned int          fs_base;              /*   192     8 */
	long unsigned int          cr0;                  /*   200     8 */
	long unsigned int          cr2;                  /*   208     8 */
	long unsigned int          cr3;                  /*   216     8 */
	long unsigned int          cr4;                  /*   224     8 */
	u64                        misc_enable;          /*   232     8 */
	struct saved_msrs          saved_msrs;           /*   240    16 */
	/* --- cacheline 4 boundary (256 bytes) --- */
	long unsigned int          efer;                 /*   256     8 */
	u16                        gdt_pad;              /*   264     2 */
	struct desc_ptr            gdt_desc;             /*   266    10 */
	u16                        idt_pad;              /*   276     2 */
	struct desc_ptr            idt;                  /*   278    10 */
	u16                        ldt;                  /*   288     2 */
	u16                        tss;                  /*   290     2 */
	long unsigned int          tr;                   /*   292     8 */
	long unsigned int          safety;               /*   300     8 */
	long unsigned int          return_address;       /*   308     8 */
	bool                       misc_enable_saved;    /*   316     1 */

	/* size: 317, cachelines: 5, members: 25 */
	/* last cacheline: 61 bytes */
} __attribute__((__packed__));
struct restore_data_record {
	long unsigned int          jump_address;         /*     0     8 */
	long unsigned int          jump_address_phys;    /*     8     8 */
	long unsigned int          cr3;                  /*    16     8 */
	long unsigned int          magic;                /*    24     8 */
	long unsigned int          e820_checksum;        /*    32     8 */

	/* size: 40, cachelines: 1, members: 5 */
	/* last cacheline: 40 bytes */
};
